/**
* An ordinary JavaScript as obtained by "new Object()" or via 
*  ObjectLiteral syntax.
*  
*  Methods that return Objects or take Objects as parameters make use of the ability of a
*  JavaScript Object to contain an arbitrary set of named properties, without requiring
*  declaration in advance. This capability makes it possible to use a JavaScript Object much
*  like a HashMap in Java or .NET, but without the need to call get() or set() to create and
*  retrieve properties.
*  
*  For example if you created an Object using ObjectLiteral syntax
*  like so:
*  
*   var request = {
*     actionURL : "/foo.do",
*     showPrompt:false
*   };
*  
*  You could then access it's properties like so:
*  
*   var myActionURL = request.actionURL;
*   var myShowPrompt = request.showPrompt;
*  
*  .. and you could assign new values to those properties like so:
*  
*   request.actionURL = "newActionURL";
*   request.showPrompt = newShowPromptSetting;
*  
*  Note that while JavaScript allows you to get and set properties in this way on any Object,
*  SmartClient components require that if a setter or getter exists, it must be called, or no
*  action will occur. For example, if you had a ListGrid and you wanted to change the
*  ListGrid.showHeader property:
*  
*    myListGrid.setShowHeader(false); // correct
*    myListGrid.showHeader = false; // incorrect (nothing happens)
*  
*  All documented attributes have flags (eg IRW) that indicate when direct
*  property access is allowed or not.
*/
interface Object {

}
interface ObjectConstructor {

}/**
* Generic extensions to JavaScript Arrays. You can call these on any Array.
*  
*  JavaScript's native Array is retrofitted to support the List API.
*/
interface Array<T> {


/**
*  Return the item at a particular position
* 
* @param {number} position of the element to get
*/
get(pos:number): object;

/**
*  Return the number of items in this list
* 
* 
*/
getLength(): number;

/**
*  Return whether or not this array is empty
* 
* 
*/
isEmpty(): boolean;

/**
*  Return the first item in this list
* 
* 
*/
first(): any;

/**
*  Return the last item in this list
* 
* 
*/
last(): any;

/**
*  Return the position in the list of the first instance of the specified object.
*  
*  If pos is specified, starts looking after that position.
*  
*  Returns -1 if not found.
* 
* 
* @param {any} object to look for
* @param {number=} earliest index to consider
* @param {number=} last index to consider
*/
indexOf(obj:any, pos?:number, endPos?:number): number;

/**
*  Return the position in the list of the last instance of the specified object.
*  
*  If pos is specified, starts looking before that position.
*  
*  Returns -1 if not found.
* 
* 
* @param {any} object to look for
* @param {number=} last index to consider
* @param {number=} earliest index to consider
*/
lastIndexOf(obj:any, pos?:number, endPos?:number): number;

/**
*  Return if this list contains the specified object.
*  
*  If pos is specified, starts looking after that position.
* 
* 
* @param {any} item to look for
* @param {number=} optional position in the list to look after
*/
contains(obj:any, pos?:number): boolean;

/**
*  Return whether this list contains all the item in the specified list.
* 
* 
* @param {List} items to look for
*/
containsAll(list:isc.List): boolean;

/**
*  Return the list of items that are in both this list and the passed-in list(s).
* 
* 
* @param {any} lists to intersect with
*/
intersect(lists:any): isc.List;

/**
*  Return whether this list is equal to another list.
*  
*  Two lists are equal only if they have the same length and all contained items are in the same
*  order and are also equal.
* 
* 
* @param {List} list to check for equality
*/
equals(list:isc.List): boolean;

/**
*  Return the items at a list of specified positions.
* 
* 
* @param {Array<Partial<number>>} array of positions
*/
getItems(itemList:Array<Partial<number>>): Array<any>;

/**
*  Return the items between position start and end, non-inclusive at the end.
* 
* 
* @param {number} start position
* @param {number} end position
*/
getRange(start:number, end:number): Array<any>;

/**
*  Return an Array that is a shallow copy of the list, that is, containing the same items.
* 
* 
*/
duplicate(): Array<any>;

/**
*  Change the array element at a particular position.
*  
*  set() can be used to expand the length of the list.
* 
* 
* @param {number} position in the list to change
* @param {object} new value for that position
*/
set(pos:number, obj:object): object;

/**
*  Add a single item to this array at a specific position in the list, sliding other items over
*  to fit.
* 
* 
* @param {object} object to add
* @param {number} position in the list to add at
*/
addAt(obj:object, pos:number): object;

/**
*  Remove the item at the specified position, rearranging all subsequent items to fill the gap
* 
* 
* @param {number} position to remove
*/
removeAt(pos:number): any;

/**
*  Add an object to this list, at the end
* 
* 
* @param {any} object to add
*/
add(object:any): any;

/**
*  Add a list of items to this array.
*  
*  Note: you can specify that a subset range be added by passing start and end indices
* 
* 
* @param {Array<any>} list of items to add
* @param {number=} optional start index in list
* @param {number=} optional end index in list (non-inclusive)
*/
addList(list:Array<any>, listStartRow?:number, listEndRow?:number): isc.List;

/**
*  Set the length of this list.
*  
*  If the length of the list is shortened, any elements past the new length of the list are removed.
*  If the length is increased, all positions past the old length have the value
*  undefined.
* 
* 
* @param {number} new length
*/
setLength(length:number): void;

/**
*  Add list of items list to this array at item pos. All items after array[pos] will slide down to
*  fit new items.
* 
* 
* @param {Array<any>} new array of items
* @param {number} position in this list to put the new items
*/
addListAt(list:Array<any>, pos:number): Array<any>;

/**
*  Remove first instance of the passed object from this array, sliding other items around to
*  fill gaps.
* 
* 
* @param {any} item to remove
*/
remove(obj:any): boolean;

/**
*  Remove all instances of objects in the specified list from this list, sliding the remaining
*  objects around to fill gaps.
* 
* 
* @param {Array<any>} list of items to remove
*/
removeList(list:Array<any>): isc.List;

/**
*  Method called when this array changes in some way. Observe the method to react to changes in
*  this list.
*  
*  Note: dataChanged() will only fire when items are added, removed or rearranged. If a list
*  contains objects, dataChanged() will not fire if changes are made to objects within the list
*  without changing their position within the list. If an observer of dataChanged() needs to react
*  to such a change, you can manually fire dataChanged() by simply calling it.
*  
*  Note: may be called multiple times as the result of a multi-item add or remove, etc.
* 
* 
*/
dataChanged(): void;

/**
*  Return a new Array where the value of item i is the value of "property" of item i in this
*  array. If an item doesn't have that property or is null, return item will be null.
* 
* 
* @param {string} name of the property to look for
*/
getProperty(property:string): Array<any>;

/**
*  Get a map of the form { item[idField] -&gt; item[displayField] }, for all 
*  items in the list. Note that if more than one item has the same value for the
*  idField, 
*  the value for the later item in the list will clobber the value for the earlier item.
* 
* 
* @param {string} Property to use as ID (data value) in the valueMap
* @param {string} Property to use a display value in the valueMap
*/
getValueMap(idField:string, displayField:string): object;

/**
*  Calls a function for each member of an array, passing in the member, its index and
*  the array itself as arguments. Returns a new array containing the
*  resulting values.
*  
*  This behavior is part of the
*  ECMA-262 specification.
*  
*  Backwards compatibility note: Historically SmartClient provided a version of
*  array.map() which differed from the native behavior in a couple of ways:
*  
*  
* - If passed a string as the function argument, it would invoke a same-named method on
*   each member of the array. This is now deprecated in favor of 
*   calling Array.callMethod directly
*  
* - If additional arguments other than the function were passed to this
*   method, when the function was invoked for each member, these additional arguments
*   would be passed in when the function was invoked. This is also deprecated as it
*   conflicts with the default native implementation
*  
* 
* @param {Function} function to execute for each item
*/
map(method:Function): Array<any>;

/**
*  Calls a method on each member of an array and returns a new array containing the
*  resulting values.
*  
*  The method argument should be the String name of a
*  method present on each item, which will be invoked. If any item is null
*  or lacks the named method, null will be returned for that item.
*  
*  Examples:
*   // line up widgets at 20 pixels from page edge
*   [widget1, widget2].callMethod("setPageLeft", 20);
* 
*   // find furthest right widget
*   [widget1, widget2].callMethod("getPageRight").max();
*  
*  
* 
* @param {string} Name of method to execute for on item
* @param {any=} arguments to pass to the method                      invoked on each item
*/
callMethod(method:string, ...args:any[]): Array<any>;

/**
* 	Set item[property] = value for each item in this array.
* 
* @param {string} name of the property to set
* @param {any} value to set to
*/
setProperty(property:string, value:any): void;

/**
*  Delete property in each item in this array.
* 
* @param {string} name of the property to clear
*/
clearProperty(property:string): boolean;

/**
*  Return a list of each unique item in this list exactly once.
*  
*  Returns in the same order they were found in the list.
*  
*  Usage example:
*  &nbsp;&nbsp;&nbsp;&nbsp;uniqueList = myArray.getProperty("foo").getUniqueItems();
* 
* 
*/
getUniqueItems(): Array<any>;

/**
*  Find the index of the first Object where property == value in the object.
*  
*  Pass an Object instead to match multiple properties.
*  
*  Note: for string values, matches are case sensitive.
* 
* 
* @param {string | object | AdvancedCriteria} property to match, or if an Object is passed, set of                    properties and values to match
* @param {any=} value to compare against (if propertyName is a string)
*/
findIndex(propertyName:string | object | isc.AdvancedCriteria, value?:any): isc.Integer;

/**
*  Like Array.findIndex, but inspects a range from startIndex to endIndex.
*  
*  
*  For convenience, findNextIndex() may also be called with a function (called the predicate
*  function) for the propertyName parameter. In this usage pattern, the predicate
*  function is invoked for each value of the list until the predicate returns a true value.
*  The predicate function is passed three parameters: the current value, the current index, and
*  the list. The value of this when the predicate function is called is the
*  value parameter. For example:
*  var currentUserRecord = recordList.findNextIndex(0, function (record, i, recordList) {
*   if (record.username == currentUsername && !record.accountDisabled) {
*     return true;
*   }
* });
*  
* 
* 
* @param {Integer} first index to consider.
* @param {string | Function | object | AdvancedCriteria} property to match; or, if a function is passed, the predicate function to call; or, if an object is passed, set of properties and values to match.
* @param {any=} value to compare against (if propertyName is a string) or the value of this when the predicate function is invoked (if propertyName is a function)
* @param {Integer=} last index to consider (inclusive).
*/
findNextIndex(startIndex:isc.Integer, propertyName:string | Function | object | isc.AdvancedCriteria, value?:any, endIndex?:isc.Integer): isc.Integer;

/**
*  Like Array.findIndex, but returns the object itself instead of its index.
* 
* 
* @param {string | object | AdvancedCriteria} property to match, or if an Object is passed, set of                    properties and values to match
* @param {any=} value to compare against (if propertyName is a string)
*/
find(propertyName:string | object | isc.AdvancedCriteria, value?:any): object;

/**
*  Determine whether this array contains any members where the property passed in matches the value
*  passed in.
* 
* 
* @param {string} property to look for   	(object)	key:value pairs to look for
* @param {any=} value to compare against (if property is a string)
*/
containsProperty(property:string, value?:any): boolean;

/**
*  Find all objects where property == value in the object.
*  
*  Pass an Object as the propertyName argument to match multiple properties.
* 
* 
* @param {string | object | AdvancedCriteria} property to match, or if an Object is passed, set of                    properties and values to match
* @param {any=} value to compare against (if propertyName is a string)
*/
findAll(propertyName:string | object | isc.AdvancedCriteria, value?:any): Array<any>;

/**
*  Slide element at position start to position destination, moving all the other elements to cover
*  the gap.
* 
* 
* @param {number} start position
* @param {number} destination position for the value at start
*/
slide(start:number, destination:number): void;

/**
*  Slide a range of elements from start to end to position destination, moving all the other
*  elements to cover the gap.
* 
* 
* @param {number} start position
* @param {number} end position (exclusive, like substring() and slice())
* @param {number} destination position for the range
*/
slideRange(start:number, end:number, destination:number): void;

/**
*  Make an index for the items in this Array by a particular property of each item.
*  
*  Returns an Object with keys for each distinct listItem[property] value. Each key will point
*  to an array of items that share that property value. The sub-array will be in the same order
*  that they are in this list.
* 
* 
* @param {string} names of the property to index by
* @param {boolean} [false] if true, we always make an array for every index. if false, we make an Array only        when more than one item has the same value for the index property
*/
makeIndex(property:string, alwaysMakeArray:boolean): object;

/**
*  Sort a list of objects by a given property of each item.
*  
*  The optional normalizer, if passed as a function, is called for each item in the List, and
*  should return whatever value should be used for sorting, which does not have to agree with
*  the property value. By passing a normalizer function you can achieve any kind of sorting
*  you'd like, including sorting by multiple properties.
*  
*  NOTE: string sort is case INsensitive by default
* 
* 
* @param {string} name of the property to sort by
* @param {boolean} true == sort ascending, false == sort descending
* @param {Function | ValueMap=} May be specified as a function, with signature        normalize(item, propertyName, context), where item is        a pointer to the item in the array, propertyName is the        property by which the array is being sorted, and context is the        arbitrary context passed into this method. Normalizer function should return        the value normalized for sorting.        May also be specified as a ValueMap which maps property values to sortable values.
* @param {any=} Callers may pass an arbitrary context into the sort method, which             will then be made available to the normalizer function
*/
sortByProperty(property:string, up:boolean, normalizer?:Function | isc.ValueMap, context?:any): isc.List;

/**
*  Sort this Array by a list of SortSpecifiers. 
* 
* @param {Array<Partial<SortSpecifier>>} the list of SortSpecifiers to sort by
*/
setSort(sortSpecifiers:Array<Partial<isc.SortSpecifier>>): Array<any>;

/**
*  Turn sorting off for this array, indicating that the current sort
*  order should be preserved. Return true if this is supported in this List.
* 
*  Some implementations may not support this -- they should return false
*  to indicate to the caller that sort order must be maintained (eg: in
*  the case where sort order is derived from the server, etc).
* 
* 
*/
unsort(): boolean;

/**
* 
*   Returns the largest number in the array, skipping non-numeric values.
* 
*  	If the start and/or end are given, searches the specified subset of the list.
* 
* 
* @param {number=} optional start index (default is 0)
* @param {number=} optional end index (default is list.length)
*/
max(start?:number, end?:number): number;

/**
*      
*   Returns the smallest number in the array, skipping non-numeric values.
* 
*  	If the start and/or end are given, searches the specified subset of the list.
* 
* 
* @param {number=} optional start index (default is 0)
* @param {number=} optional end index (default is list.length)
*/
min(start?:number, end?:number): number;

/**
*   Returns the sum of the numbers in the array, skipping non-numeric values.
* 
*  	If the start and/or end are given, uses only the specified subset of the list.
* 
* 
* @param {number=} optional start index (default is 0)
* @param {number=} optional end index (default is list.length)
*/
sum(start?:number, end?:number): number;

/**
*  Returns true if all values between the start and end indices are true.
* 
* 
* @param {number=} optional start index (default is 0)
* @param {number=} optional end index (default is list.length)
*/
and(start?:number, end?:number): boolean;

/**
*  Returns true if at least one value between the start and end indices is true.
* 
* 
* @param {number=} optional start index (default is 0)
* @param {number=} optional end index (default is list.length)
*/
or(start?:number, end?:number): boolean;




}
interface ArrayConstructor {




/**
*  Is the object passed in a loading marker value? For use with Lists that manage remote
*  datasets, to indicate that a record has not yet been retrieved from the server. A typical
*  use case might be to check if a row has been loaded in a ListGrid - for example:
*  
*  
*  if (Array.isLoading(myList.getRecord(0))) isc.warn("Please wait for the data to load.");
*  
* 
* @param {any} data to test.
*/
isLoading(value:any): void;

/**
*  Compare two values for an ascending order sort, using locale-sensitive comparison.
* 
* @param {any} first value to compare
* @param {any} second value to compare
*/
compareAscending(a:any, b:any): number;

/**
*  Compare two values for a descending order sort, using locale-sensitive comparison.
* 
* @param {any} first value to compare
* @param {any} second value to compare
*/
compareDescending(first:any, second:any): number;




}/**
* Boolean object. Attributes, parameters, or return values declared as Boolean
*  may be null. Contrast with boolean.
*/
interface Boolean {





}
interface BooleanConstructor {







}/**
* Extra methods added to the Number object, available on all number variables. Attributes,
*  parameters, or return values declared as Number may be null.
*/
interface Number {


/**
* 
*  Return this number as a string padded out to digits length.
* 
* 
* @param {number=} [2] Number of digits to pad to. (Default is 2)
* @deprecated * Moved to a static method on NumberUtil to avoid the possibility of collision        with other libraries on the native Number object

*/
stringify(digits?:number): string;

/**
*  Returns true if the number parameter falls between the 'first' and 'second' paramters.
* 
* 
* @param {number} Number object to be evaluated
* @param {number=} Number at the lower boundary
* @param {number=} Number at the upper boundary
* @param {number=} Whether or not the numbers at either end of the boundary should be included in the comparison
*/
isBetween(number:number, first?:number, second?:number, inclusive?:number): boolean;

/**
*  Return this number as a currency-formatted string.
* 
* 
* @param {string=} Currency symbol, can be set to an empty string.                 If unset "$" will be used.
* @param {string=} Decimal separator symbol. If unset "." will be used.
* @param {boolean=} Should decimal portion be padded out to two digits? True                by default.
* @param {boolean=} Should currency symbol come at the end of the string?                   If unspecified, currency symbol will be shown at the                    beginning of the string.
* @deprecated * Moved to a static method on NumberUtil to avoid the possibility of collision        with other libraries on the native Number object

*/
toCurrencyString(currencyChar?:string, decimalChar?:string, padDecimal?:boolean, currencyCharLast?:boolean): string;




}
interface NumberConstructor {







}/**
* Generic extensions to JavaScript Strings. You can call these on any String.
*/
interface String {


/**
*  	Returns true if this string contains the specified substring.
* 
* @param {string} string to look for
*/
contains(substring:string): boolean;

/**
*  Given a number, this method will return the plural version of the string if appropriate.
*  
*  The plural value will be derived from String.pluralNounMap if present, otherwise
*  the plural value will be created by adding a lower or upper case "s"
*  character to the string.
*  
* 
*/
pluralize(): string;

/**
*  Returns true if this string starts with another string, or if the other string
*  occurs at the given position within this string.
* 
* 
* @param {string} other string to check
* @param {Integer=} optional position in this string. Defaults to 0.
*/
startsWith(substring:string, position?:isc.Integer): boolean;

/**
*  Returns true if this string ends with another string, or if the other string
*  occurs in this string beginning at position - substring.length.
* 
* 
* @param {string} other string to check
* @param {Integer=} optional position in this string. Defaults to the length of this string.
*/
endsWith(substring:string, position?:isc.Integer): boolean;




}
interface StringConstructor {


/**
*  Tests whether the given string is a valid JavaScript identifier.
* 
* 
* @param {string} the string to test.
*/
isValidID(string:string): boolean;






}
declare namespace isc {
interface Map<T> { [key: string]: T }
/**
* 
* 	Method available on the isc object to show a modal prompt to the user.
*  This method will display the message using the Dialog.Prompt singleton object.
*  Note: if this prompt is to be shown to the user during some slow JavaScript logic, we
*  advise calling this method, then using Class.delayCall or Timer.setTimeout
*  to kick off the slow logic in a separate thread. This ensures that the prompt is showing
*  before the lengthy execution begins.
*  Use "&#36;{loadingImage}" to include Canvas.loadingImageSrc.
* 
* 
* 
* @param {string} message to display
* @param {Partial<Dialog>=} additional properties for the Dialog, applied before                    the Dialog is shown
*/
function showPrompt(message:string, properties?:Partial<Dialog>):void;
/**
* 
* 	Clear the modal prompt being shown to the user.
* 
* 
*/
function clearPrompt():void;
/**
* 
* 	Method available on the isc object to show a temporary modal prompt to the user.
*  This method will display the message using the Dialog.Prompt singleton object, then hide it
*  using a fade animation effect.
*  Note: if this prompt is to be shown to the user during some slow JavaScript logic, we
*  advise calling this method, then using Class.delayCall or Timer.setTimeout
*  to kick off the slow logic in a separate thread. This ensures that the prompt is showing
*  before the lengthy execution begins.
*  
*  The prompt may be cleared before the duration has elapsed via a call to isc.clearPrompt
*  and any callback specified will still be fired even if the prompt is dismissed early.
* 
* 
* @param {string} message to display
* @param {number} how long the message should appear for in milliseconds before  fading from view.
* @param {Callback=} When the prompt is hidden, callback will be fired.
* @param {Partial<Dialog>=} additional properties for the Dialog, applied before                    the Dialog is shown
*/
function showFadingPrompt(message:string, duration:number, callback?:Callback, properties?:Partial<Dialog>):void;
/**
*  Show a modal dialog with a message, icon, and "OK" button. See Dialog.warnIcon.
*  
*  The callback will receive boolean true for an OK button click, or null if the Dialog is
*  dismissed via the close button.
* 
* 
* @param {string} message to display
* @param {Callback=} Optional Callback to fire when the user                   dismisses the dialog. This has the single parameter                    'value', indicating the value returned by the Warn                    dialog from 'okClick()' etc.
* @param {Partial<Dialog>=} additional properties for the Dialog.                   To set Dialog.buttons for                    the Dialog, set properties.buttons to                    an array of buttons     	eg:	{ buttons : [Dialog.OK, Dialog.CANCEL] }
*/
function warn(message:string, callback?:Callback, properties?:Partial<Dialog>):void;
/**
*  Show a modal dialog with a message, icon, and "OK" button. Intended for notifications which
*  are not really warnings (default icon is less severe). See Dialog.sayIcon.
*  
*  The callback will receive boolean true for an OK button click, or null if the Dialog is
*  dismissed via the close button.
* 
* 
* @param {string} message to display
* @param {Callback=} Optional Callback to fire when the user                   dismisses the dialog. This has the single parameter                    'value', indicating the value returned by the Warn                    dialog from 'okClick()' etc.
* @param {Partial<Dialog>=} additional properties for the Dialog.                   To set Dialog.buttons for                    the Dialog, set properties.buttons to an array of                    buttons     	eg:	{ buttons : [Dialog.OK, Dialog.CANCEL] }
*/
function say(message:string, callback?:Callback, properties?:Partial<Dialog>):void;
/**
*  Show a modal dialog with a message, icon, and "Yes" and "No" buttons. See Dialog.askIcon.
*  
*  The callback will receive boolean true for a Yes button click, boolean false for a No button
*  click, or null if the Dialog is dismissed via the close button.
* 
* 
* @param {string} message to display
* @param {Callback=} Callback to fire when the                   user clicks a button to dismiss the dialog.                    This has the single parameter 'value', indicating the                    value returned by the Warn dialog from 'okClick()' etc.
* @param {Partial<Dialog>=} additional properties for the Dialog.                   To set Dialog.buttons for                    the Dialog, set properties.buttons to an array                    of buttons     	eg:	{ buttons : [Dialog.OK, Dialog.CANCEL] }
*/
function ask(message:string, callback?:Callback, properties?:Partial<Dialog>):void;
/**
*  Show a modal dialog with a message, icon, and "OK" and "Cancel" buttons. See Dialog.confirmIcon.
*  
*  The callback will receive boolean true for an OK button click, or null for a Cancel click or
*  if the Dialog is dismissed via the close button.
*  
*  Note: this does not override the native window.confirm() method.
* 
* 
* @param {string} message to display
* @param {Callback=} Callback to fire when the                   user clicks a button to dismiss the dialog.                    This has the single parameter 'value', indicating the                    value returned by the Warn dialog from 'okClick()' etc.
* @param {Partial<Dialog>=} additional properties for the Dialog.                   To set Dialog.buttons for                    the Dialog, set properties.buttons to an array of                    buttons     	eg:	{ buttons : [Dialog.OK, Dialog.CANCEL] }
*/
function confirm(message:string, callback?:Callback, properties?:Partial<Dialog>):void;
/**
*  Show a modal dialog with a text entry box, asking the user to enter a value.
*  
*  As with other convenience methods that show Dialogs, such as isc.warn,
*  the dialog is shown and the function immediately returns. When the user responds, the
*  provided callback is called.
*  
*  If the user clicks OK, the value typed in is passed to the callback (including the empty
*  string ("") if nothing was entered. If the user clicks cancel, the value passed to the
*  callback is null.
*  
*  A default value for the text field can be passed via properties.defaultValue.
*  
*  Keyboard focus is automatically placed in the text entry field, and hitting the enter key is
*  the equivalent of pressing OK.
* 
* 
* @param {string} message to display
* @param {Callback=} Callback to fire when the                   user clicks a button to dismiss the dialog.                    This has the single parameter 'value', indicating the                    user entry, or null if cancel was pressed or the window                    closed
* @param {Partial<Dialog>=} additional properties for the Dialog.                   To set Dialog.buttons for                    the Dialog, set properties.buttons to an array of                    buttons     	eg:	{ buttons : [Dialog.OK, Dialog.CANCEL] }
*/
function askForValue(message:string, callback?:Callback, properties?:Partial<Dialog>):void;
/**
*  If a dialog triggered via isc.say, isc.ask,
*  isc.warn, isc.confirm or
*  isc.askForValue is currently visible, it will be dismissed. The
*  callback passed to the relevant method will never fire.
*  
*  Note this is a rarely used API with very few valid use cases. As an example, perhaps some kind of
*  periodic (non-user triggered) event would cause an entire area of the UI to be removed (such as a tab)
*  and the system wants to ensure that no modal dialogs are currently showing from that part of the UI.
*  In this case, while dismissCurrentDialog could be used to ensure the part of the UI being
*  removed didn't leave behind a modal dialog.
*  
*  To clear a modal prompt shown by isc.showPrompt, use isc.clearPrompt instead.
* 
* 
*/
function dismissCurrentDialog():void;
/**
*  Handle a complete login interaction with a typical login dialog asking for username and
*  password credentials using the LoginDialog class.
*  
*  As with other convenience methods that show Dialogs, such as isc.warn,
*  the dialog is shown and the function immediately returns. When the user responds, the
*  provided callback function is called.
*  
*  If the user clicks the "Log in" button, the credentials entered by the user are passed to
*  the provided "loginFunc" as an Object with properties "username" and "password" (NOTE: both
*  property names are all lowercase), as the variable "credentials". For example:
*  { username: "barney", password: "rUbbL3" }
*  
*  The "loginFunc" should then attempt to log in by whatever means is necessary. The second
*  parameter to the loginFunc, "dialogCallback", is a function, which must be called whether
*  login succeeds or fails with a true/false value indicating whether login succeeded.
*  
*  If the login dialog is dismissable (settable as properties.dismissable, default false) and
*  the user dismisses it, the loginFunc will be fired with null for the credentials.
*  
*  The following code shows typical usage. This code assumes you have created a global
*  function sendCredentials() that send credentials to some authentication system and fires a
*  callback function with the result:
*  
*  isc.showLoginDialog(function (credentials, dialogCallback) {
*    if (credentials == null) return; // dismissed
* 
*    // send credentials
*    sendCredentials(credentials, function (loginSucceeded) {
*      // report success or failure
*      dialogCallback(loginSucceeded);
*    })
*  })
*  
*  The login dialog has several built-in behaviors:
*  
*  
* - keyboard focus is automatically placed in the username field
*  
* - hitting enter in the username field proceeds to the password field
*  
* - hitting enter in the password field submits (fires the provided callback)
*  
*  In addition to normal properties supported by Dialog/Window, the following special
*  properties can be passed:
*  
*  
* - username: initial value for the username field
*  
* - password: initial value for the password field
*  
* - usernameTitle: title for the username field
*  
* - passwordTitle: title for the password field
*  
* - errorMessage: default error message on login failure
*  
* - loginButtonTitle: title for the login button
*  
* - dismissable: whether the dialog can be dismissed, default false
*  
* - errorStyle: CSS style for the error message, if shown
*  
*  See below for links to the default values for these properties.
* 
* 
* @param {Callback} Function to call to attempt login. Receives parameters                   "credentials" and "dialogCallback", described above
* @param {Partial<LoginDialog>=} additional properties for the Dialog
*/
function showLoginDialog(loginFunc:Callback, properties?:Partial<LoginDialog>):void;
/**
*  Same as Log.logWarn.
* 
* 
* @param {string} message to log
* @param {string=} category to log in, defaults to "Log"
*/
function logWarn(message:string, category?:string):void;
/**
*  Same as Log.logInfo.
* 
* 
* @param {string} message to log
* @param {string=} category to log in, defaults to "Log"
*/
function logInfo(message:string, category?:string):void;
/**
*  Same as Log.echo.
* 
* 
* @param {any} object to echo
*/
function echo(value:any):string;
/**
*  Same as Log.echoAll.
* 
* 
* @param {any} object to echo
*/
function echoAll(value:any):string;
/**
*  Same as Log.echoLeaf.
* 
* 
* @param {any} object to echo
*/
function echoLeaf(value:any):string;
/**
*  Logs the echoed object (using isc.echo) as a warning, prefixed with an
*  optional message.
* 
* 
* @param {any} object to echo
* @param {string} message to log
*/
function logEcho(value:any, message:string):void;
/**
*  Logs the echoed object (using isc.echoAll) as a warning, prefixed with an
*  optional message.
* 
* 
* @param {any} object to echo
* @param {string} message to log
*/
function logEchoAll(value:any, message:string):void;
/**
* 
*  Add all properties and methods from any number of objects to a destination object, 
*  overwriting properties in the destination object.
*  
*  Common uses of addProperties include creating a shallow copy of an object:
* 
*    isc.addProperties({}, someObject);
* 
*  Combining settings in order of precedence:
* 
*    isc.addProperties({}, defaults, overrides, skinOverrides);
* 
*  
*  
*  NOTES:
*  
* - Do not use addProperties to add defaults to an ISC class.
*  Use Class.addProperties, as in: 
*  MyClassName.addProperties().
*  
* - You may find it more convenient to use the instance method Class.addProperties,
*  as in: myClassInstance.addProperties(), but there's no functional
*  difference from using this method.
*  
* 
* 
* @param {object} object to add properties to
* @param {object=} objects to obtain properties from. Properties of all      arguments other than destination are applied in turn.
*/
function addProperties(destination:object, ...args:any[]):object;
/**
* 
* 	Return all keys (property names) of a given object
* 
* 
* @param {object} object to get properties from
*/
function getKeys(object:object):Array<any>;
/**
*  Return the first property name in a given Object, according to for..in iteration order.
* 
* 
* @param {object} Object to get properties from
*/
function firstKey(object:object):string;
/**
* 
* 	Return all values of a given object
* 
* 
* @param {object} object to get properties from
*/
function getValues(object:object):Array<any>;
/**
*  Given a simple javascript object, return that object sorted by keys, such that when iterating
*  through the properties of the object, they will show up in sorted order.
*  Usage example - may be used to sort a FormItem.valueMap defined
*  as an object.
* 
* @param {object} Object to sort
* @param {Function=} Comparator function to use when sorting the objects keys
*/
function sortObject(object:object, comparator?:Function):object;
/**
*  Given a simple javascript object, return that object sorted by properties, such that when 
*  iterating through the properties of the object, values will show up in sorted order.
*  Usage example - may be used to sort a FormItem.valueMap defined
*  as an object by display value.
* 
* @param {object} Object to sort
* @param {Function=} Comparator function to use when sorting the object properties
*/
function sortObjectByProperties(object:object, comparator?:Function):object;
/**
* 
*  Copy any properties that do not already have a value in destination. Null and zero values
*  are not overwritten, but 'undef' values will be.
* 
* 
* @param {object} Object to which properties will be added.
* @param {object} Object from which properties will be added.
*/
function addDefaults(destination:object, source:object):object;
/**
* 
* 	Is some property specified on the object passed in? This will return true if 
*  object[propertyName] has ever been set to any value, and not deleted.
*  May return true even if object[propertyName] === undefined if the property 
*  is present on the object and has been explicitly set to undefined.
* 
* 
* @param {object} Object to test
* @param {string} Which property is being tested for?
*/
function propertyDefined(object:object, propertyName:string):boolean;
/**
*  Shortcut for isc.ClassFactory.defineClass().
* 
* @param {string} Name for the new class.
* @param {Class=} Optional SuperClass Class object or name
*/
function defineClass(className:string, superClass?:Class):Class;
/**
*  Shortcut for isc.ClassFactory.overwriteClass().
* 
*/
function overwriteClass():void;
/**
*  Shorthand for new Date().getTime();, this returns a timeStamp - a large number
*  which is incremented by 1 every millisecond. Can be used to generate unique identifiers,
*  or perform timing tasks.
* 
* 
*/
function timeStamp():Integer;
/**
* 
* 	Method available on the isc object to open the Developer Console.
* 
* 
*/
function showConsole():void;
/**
*  Given a key and an object of key:value pairs, return the value that corresponds to
*  that key.
*  
*  If the key is not found, defaultValue will be returned if provided, otherwise the
*  key will be returned.
* 
* 
* @param {string | number} key to look for
* @param {object} object of key:value pairs
* @param {any=} default value to return if key not found
*/
function getValueForKey(key:string | number, valueMap:object, defaultValue?:any):any;
/**
*  Given a value and an object of key:value pairs, return a key that corresponds
*  to that value.
*  
*  If the key is not found, defaultKey will be returned if provided, otherwise the
*  value will be returned.
* 
* 
* @param {string | number} value to look for
* @param {object} object of key:value pairs
* @param {any=} default key to return if value not found
*/
function getKeyForValue(value:string | number, valueMap:object, defaultKey?:any):any;
/**
*  Given a key:value map, return a new map as value:key.
*  
*  If the same value appears more than once, the key will correspond to the last instance of that
*  value.
* 
* 
* @param {object} object of key:value pairs
*/
function makeReverseMap(valueMap:object):object;
/**
*  Method to set up a static Class.observe on some target object. 
*  This allows developers to perform some action every time a particular method is invoked 
*  on a target object.
*  
*  This method returns a unique observation identifier string. To cancel the observation, 
*  pass this identifier to Page.ignore.
*  
*  If multiple observations are set up for the same target object and method, the 
*  notification actions will be fired in the order in which they were registered.
*  
*  This method is available as isc.Page.observe() or just isc.observe()
*  
*  Note [potential memory leak]: If the target object is a simple JavaScript object
*  (not an instance of a SmartClient class), developers should always call 
*  Page.ignore to stop observing the object if an observation is no longer necessary.
*  
*  This ensures that if the object is subsequently allowed to go out of scope by application 
*  code, the Page level observation system will not retain a reference to it (so the browser
*  can reclaim the allocated memory).
*  While cleaning up observations that are no longer required is always good practice, this
*  memory leak concern is not an issue if the target object is an instance of a SmartClient
*  class. In that case the observation is automatically released when the 
*  target is Class.destroy.
* 
* 
* @param {object} Object to observe. This may be any JavaScript object with the specified  target method, including native arrays, and instances of SmartClient classes such as   Canvas.
* @param {string} Name of the method to observe. Every time this method is invoked  on the target object the specified action will fire   (after the default implementation completes).
* @param {Function | string} Action to take when the observed method is invoked.  If action is a string to execute, certain keywords are available for context:    
- observed is the target object being observed (on which the method was invoked).  
- returnVal is the return value from the observed method (if there is one)  
- For functions defined with explicit parameters, these will also be available as keywords within    the action string    If action is a function, the arguments for the original method will also be   passed to this action function as arguments. If developers need to access the target   object being observed from the action function they may use native javascript techniques  such as   javascript closure  to do so. The return value from the observed method is not available to the action function.
*/
function observe(object:object, methodName:string, action:Function | string):string;
/**
*  Clear an observation set up by Page.observe.
*  
*  This method is available as isc.Page.ignore() or just isc.ignore()
* 
* 
* @param {string} ID returned from Page.observe call we want to clear
*/
function ignore(observerID:string):void;
/**
*  Set the global default setting for Canvas.autoDraw.
*  
*  After calling isc.setAutoDraw(), any newly created Canvas which is not given an
*  explicit setting for Canvas.autoDraw will follow the new default setting.
*  
*  autoDraw:false is the recommended default setting for most applications since it ensures
*  that extra draws will not occur when developers inadvertently omit the autoDraw:false
*  setting on child components.
* 
* 
* @param {boolean=} whether autoDraw should be enabled or disabled. Defaults to true.
*/
function setAutoDraw(enable?:boolean):void;
/**
*  Create a deep clone of an object that can be edited without affecting the original
*  
*  All mutable types, including Objects, Arrays and Dates, are copied. All immutable types
*  (Number, String, etc) are just preserved by reference.
*  
*  Only JavaScript built-in types may be cloned. SmartClient UI widgets do not support
*  cloning but must be created explicitly via Class.create.
*  Note that you also can't duplicate a live canvas by passing into create() as an
*  argument. If you need to create multiple components with similar configuration, some common
*  patterns inclulde:
*  
* - Create a new SmartClient class with the desired default configuration, and
*    create instances of this class as needed.
*  
* - For components created by some specific instance, the AutoChild system may be used.
*    Developers can specify a standard configuration in 
*    autoChildNameDefaults and 
*    autoChildNameProperties, and use Class.createAutoChild
*    to create a number of standard auto child components.
*  
* - A less formal approach might be to have a simple getter type method which
*    created and returned a new component each time it was called, passing in a standard
*    configuration block.
*  
*  Does not handle looping references (will infinite loop).
* 
* 
* @param {object} object to clone
*/
function clone(object:object):object;
/**
*  Creates a shallow copy of the passed-in Object or Array of Objects, that is, copies all
*  properties of an Object to a new Object, so that the clone now has exactly the same property
*  values as the original Object.
*  
*  If shallowClone() is passed an immutable type such as String and Number, it is returned
*  unchanged. Dates are copied via new Date(originalDate.getTime()).
*  
*  Note that if an Array is passed, all members of the Array will be cloned. For a copy of an
*  Array that contains exactly the same members (not copies), use Array.duplicate().
*  
*  Only an Array directly passed to shallowClone() is copied. Arrays contained
*  within Arrays will not be copied.
* 
* 
* @param {object | Array<any> | object} object to be cloned
*/
function shallowClone(object:object | Array<any> | object):object | Array<Partial<object>>;
/**
*  Enables full screen reader mode. Must be called before any components are created. See
*  accessibility.
* 
* @param {boolean} new setting
*/
function setScreenReaderMode(newState:boolean):void;
/**
*  Displays a new message that's automatically dismissed after a configurable amount of time,
*  as an alternative to isc.confirm dialogs that can lower end user
*  productivity.
*  
*  This method is simply a shorthand way to call Notify.addMessage. For further study,
*  see the Notify class overview, and the class methods Notify.dismissMessage and Notify.configureMessages.
* 
* @param {HTMLString} message to be displayed
* @param {Array<Partial<NotifyAction>>=} actions (if any) for this message
* @param {NotifyType=} category of message
* @param {NotifySettings=} display and behavior settings for this message,                        overriding any stored settings for this                        NotifyType
*/
function notify(contents:HTMLString, actions?:Array<Partial<NotifyAction>>, notifyType?:NotifyType, settings?:NotifySettings):MessageID;
/**
*  Returns the (offset) left-coordinate of an icon within its containing widget.
* 
* @param {object} icon definition
*/
function getIconLeft(icon:object):number;
/**
*  Returns the (offset) top-coordinate of an icon within its containing widget.
* 
* @param {object} icon definition
*/
function getIconTop(icon:object):number;
/**
*  Returns the size / position of an icon with respect to the widget rendering out the
*  form item as an array of coordinates.
* 
* @param {object} icon definition for the icon you want to determine the                position of (defaults to first icon in this.icons).
*/
function getIconRect(icon:object):Array<any>;
/**
*  Returns the size / position of an icon on the page as an array of coordinates.
* 
* @param {object} icon definition for the icon you want to determine the                position of (defaults to first icon in this.icons).
*/
function getIconPageRect(icon:object):Array<any>;
/**
*  Sets the StatefulCanvas.radioGroup identifier for this canvas's 
*  mutually exclusive selection group.
* 
*/
function setRadioGroup():void;
/**
*  A notification Callback fired by the TabIndexManager to allow 
*  application code to react to the numeric tab-index of some registered target being
*  modified.
* 
*/
type TabIndexUpdatedCallback = object;
/**
*  A Callback fired by the TabIndexManager when application code or user
*  action attempts to synthetically shift focus to some registered target. 
*  See TabIndexManager.shiftFocus.
*  
*  A typical implementation will shift focus to some native element associated
*  with the registered target, or if this is not currently possible, return false.
* 
* 
*/
type ShiftFocusCallback = object;
/**
*  Callback scheduled by MultiWindow.setEvent. The RemoteWindow may be
*  null if the associated browser window is unloading or closing.
*  
*  Note that the event is simply an
*  OpenFin application event
*  when OpenFin is present, but may not be fully populated in fallback mode without OpenFin.
* 
*/
type MultiWindowEventCallback = object;
/**
*  Callback reporting a change to a RemoteWindow, such as it moving.
* 
*/
type RemoteWindowCallback = object;
/**
*  Callback reporting a failure in a RemoteWindow operation.
* 
*/
type RemoteWindowErrorCallback = object;
/**
*  Callback reporting the result of a boolean RemoteWindow operation.
* 
*/
type RemoteWindowBooleanCallback = object;
/**
*  Callback reporting the result of a RemoteWindow operation yielding a map.
* 
*/
type RemoteWindowMapCallback = object;
/**
*  Callback executed when a PrintCanvas.setHTML is rendered into
*  a printCanvas.
* 
* 
*/
type PrintCanvasCallback = object;
/**
*  Callback required for the property DataSourceField.getFieldValue.
* 
* 
*/
type GetFieldValueCallback = object;
/**
*  A Callback fired when DataSource.hasFile completes.
* 
* 
*/
type HasFileCallback = object;
/**
*  A Callback fired when Sound.load completes.
*  
* 
*/
type CanPlayCallback = object;
/**
*  A Callback fired when Sound.play completes.
*  
* 
*/
type PlaybackCompleteCallback = object;
/**
*  A Callback fired when DataSource.hasFileVersion completes.
* 
* 
*/
type HasFileVersionCallback = object;
/**
*  Callback fired when DataSource.getFile completes.
* 
* 
*/
type GetFileCallback = object;
/**
*  Callback fired when DataSource.getFileVersion completes.
* 
* 
*/
type GetFileVersionCallback = object;
/**
*  Generic callback interface.
* 
* 
*/
type Function = object;
/**
*  Generic callback interface.
* 
* 
*/
type ClientOnlyDataSourceCallback = object;
/**
*  A Callback to evaluate when a screen is loaded via RPCManager.loadScreen.
* 
* 
*/
type LoadScreenCallback = object;
/**
*  A Callback to evaluate when an RPCRequest completes.
*  
*  Parameters passed to this callback are:
*  
*  
* - rpcResponse: an RPCResponse encapsulating the server response to your
*    request
*  
* - data: just the "data" property from the RPCResponse, for convenience
*  
* - rpcRequest: the RPCRequest that was sent. You can use
*    RPCRequest.clientContext to track state during the server turnaround.
*  
*  For example, to take the data returned by the server and display it in a previously created
*  ListGrid with the ID "myGrid":
*  
*    isc.RPCManager.send("getData", "myGrid.setData(data)");
*  
*  Or
*  
*    isc.RPCManager.send("getData", function (rpcResponse, data, rpcRequest) { 
*                     myGrid.setData(data)
*    });
*  
* 
* 
*/
type RPCCallback = object;
/**
*  Callback to fire when a queue of requests sent via 
*  {@link com.smartgwt.client.rpc.RPCManager#sendQueue(RPCQueueCallback)} returns.
*  Note that the Array of RPCResponses passed to this callback
*  will actually be DSResponse objects for any requests that were actually 
*  DSRequests. 
*  
* 
* 
*/
type RPCQueueCallback = object;
/**
*  Callback for RPCManager.exportImage.
* 
*/
type ExportImageCallback = object;
/**
*  A Callback to evaluate after RPCManager.loadProject completes.
*  
*  If LoadProjectSettings.willHandleError is set, the callback will
*  fire even if the requested projects could not be retrieved. You can call
*  RPCManager.getLoadProjectErrorStatus or
*  RPCManager.getLoadProjectErrorMessage in this case for more information.
* 
* 
*/
type LoadProjectCallback = object;
/**
*  Callback fired with the PaletteNode obtained asynchronously.
* 
*/
type PaletteNodeCallback = object;
/**
*  Callback fired upon successful completion of Reify.getMockDS or
*  Reify.showMockDS:
*  
* - 
*  Output for all DataSource together is reported as the single string
*  parameter allDSData. When using MockDSExportSettings.format:
*  "reifyCSV", output for separate DataSources is separated by a special marker.
*  
* - 
*  Output with each DataSource as a separate string array element is also available
*  as the parameter perDSData, ordered to match the dsNames
*  parameter in Reify.getMockDS or Reify.showMockDS.
*  
*  Note that in the case of Reify.showMockDS, the callback is fired after the
*  window is closed, not when it's populated.
* 
* 
*/
type MockDSExportCallback = object;
/**
*  A Callback to evaluate when a Process has been loaded via 
*  Process.loadProcess.
* 
* 
*/
type ProcessCallback = object;
/**
*  Callback for DrawPane.getDataURL.
* 
*/
type DataURLCallback = object;
/**
*  A Callback to evaluate when form validation completes.
*  
*  The available parameters are:
* 
*/
type ValidationStatusCallback = object;
/**
*  Callback required for the property Validator.condition and
*  ValidatorDefinition.condition.
* 
*/
type ValidatorConditionCallback = object;
/**
*  Callback required for the property ValidatorDefinition.action.
* 
* 
*/
type ValidatorActionCallback = object;
/**
*  Callback for DateRangeDialog.askForRange.
* 
*/
type DateRangeCallback = object;
/**
*  A Callback called when the move completes.
* 
* 
*/
type AnimationCallback = object;
/**
*  Callback to execute after the section has been shown.
* 
*/
type ShowSectionCallback = object;
/**
*  Callback to execute after the section has been expanded.
* 
*/
type ExpandSectionCallback = object;
/**
*  Callback to execute after the section has been hidden.
* 
*/
type HideSectionCallback = object;
/**
*  Callback to execute after the section has been collapsed.
* 
*/
type CollapseSectionCallback = object;
/**
*  Callback executed when a message is sent to a channel that you have
*  Messaging.subscribe to.
* 
*/
type MessagingCallback = object;
/**
*/
interface AI extends Class {




}
/**
* Creates a new AI
* 
* @param typeCheckedProperties {Partial<AI>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {AI} The new instance
*/
class AI{
static create(typeCheckedProperties? : Partial<AI>, uncheckedProperties?: Object): AI;







}

/**
* ISC Abstraction for ActiveX controls
*/
interface ActiveXControl extends BrowserPlugin {
/**
* Sets the 'id' attribute on the object. If a name is not provided it will be
*  auto-generated. Note that in general you don't need to set this. If you have a reference to
*  your ISC ActiveX control object you can simply call
*  ActiveXControl.getPluginHandle to get a handle to the element.
* @type {string}
* @default null
*/
id?: string;

/**
* A map of key/value pairs to pass to the Active X control as parameters.
* @type {object}
* @default null
*/
params?: object;

/**
* Set this to the uuid of your Active X control - ISC will then generate the appropriate
*  classID entry for you.
* @type {string}
* @default null
*/
uuid?: string;

/**
* This sets the value of the classID property on the object. This is meant to give you
*  complete control over the generated HTML. In practice it may be more handy to set the uuid
*  property on this object and let the classID be generated from that.
* @type {string}
* @default null
*/
classID?: string;

/**
* Specifies the URL from which to load the ActiveX control.
* @type {URL}
* @default null
*/
codeBase?: URL;



/**
*  Returns the ID for this ISC ActiveX control object.
*  If the id property was specified for the object, that will be used, otherwise 
*  the ID will be auto-generated.
* 
*/
getPluginID(): string;

/**
*  Returns a handle to the element for this ISC ActiveX control object.
* 
*/
getPluginHandle(): DOMElement;



}
/**
* Creates a new ActiveXControl
* 
* @param typeCheckedProperties {Partial<ActiveXControl>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ActiveXControl} The new instance
*/
class ActiveXControl{
static create(typeCheckedProperties? : Partial<ActiveXControl>, uncheckedProperties?: Object): ActiveXControl;







}

/**
* A menu that can either show its menu options inline, or show them via a drop-down,
*  depending on available space in the surrounding Layout or ToolStrip.
*  
*  See Canvas.canAdaptWidth for background on adaptive layout.
*/
interface AdaptiveMenu extends Layout {
/**
* If there is not enough space to show the full set of items as buttons inline, how
*  should the Adaptive menu behave?
*  If showPartialInlining is true, the menu will render as many items as
*  inline buttons as can be shown in the available space, plus the menu button to 
*  access the remaining items.
*  If false, it will show just the menu button.
*  
*  If there is enough space to show the full set of items inline they will be shown
*  inline regardless of this property.
* @type {boolean}
* @default true
*/
partialInlining?: boolean;

/**
* Placement of inlined items relative to the main MenuButton. Default is to place items
*  above the menu if the parent is a Layout with Layout.orientation,
*  otherwise to the left of the menuButton (or right if the Page.isRTL.
*  
*  A setting of "center" is invalid and will cause a warning and be ignored
* @type {Alignment | VerticalAlignment}
* @default null
*/
inlinePlacement?: Alignment | VerticalAlignment;

/**
* Default setting for MenuItem.showIconOnlyInline. Individual items can set
*  showIconOnlyInline to override this setting.
* @type {boolean}
* @default false
*/
showIconOnlyInline?: boolean;

/**
* MenuItems to be show either inline or as a drop-down Menu.
*  
*  When shown inline, items are rendered as different AutoChild according to the
*  settings on the MenuItem:
*  
*  
* - normal MenuItems render as the AdaptiveMenu.inlineMenuItem, a ToolStripButton AutoChild
*  
* - MenuItems that have submenus render as the AdaptiveMenu.inlineSubmenuItem, a
*    MenuButton AutoChild
*  
* - MenuItems with MenuItem.showIconOnlyInline set render as the
*    AdaptiveMenu.inlineImgButton, a ToolStripButton AutoChild
*  
* - MenuItems where MenuItem.embeddedComponent has been specified will have the
*    embedded component displayed directly instead (no AutoChild involvement here). If the
*    control should have different appearance when inlined vs embedded in the menu, one way
*    to achieve this is to detect whether the parent is a Menu when it is drawn.
*  
* @type {Array<Partial<MenuItem>>}
* @default null
*/
items?: Array<Partial<MenuItem>>;

/**
* Instance of the normal (non-Adaptive) Menu class used to show items that do not fit inline.
* @type {Menu}
* @default null
*/
menu?: Menu;

/**
* MenuButton used as a drop-down control for showing any items of the menu that are not
*  displayed inline.
* @type {MenuButton}
* @default null
*/
menuButton?: MenuButton;

/**
* Title used for the MenuButton.
* @type {HTMLString}
* @default null
*/
menuButtonTitle?: HTMLString;

/**
* Icon used for the MenuButton. Default of null means to use the default for the
*  MenuButton class.
* @type {SCImgURL}
* @default null
*/
menuButtonIcon?: SCImgURL;

/**
* MultiAutoChild used to create inline menu items.
*  
*  The MenuItem.icon and MenuItem.title will be rendered via Button.icon and
*  Button.title respectively; other MenuItem appearance-related properties do not
*  apply.
* @type {ToolStripButton}
* @default null
*/
inlineMenuItem?: ToolStripButton;

/**
* MultiAutoChild used to create inline menu items for menu items that have a submenu.
*  
*  The MenuItem.icon and MenuItem.title will be rendered via
*  RibbonButton.icon and Button.title respectively; other MenuItem
*  appearance-related properties do not apply.
* @type {IconMenuButton}
* @default null
*/
inlineSubmenuItem?: IconMenuButton;

/**
* Whether ToolStripSeparator should be shown for inline menu items. 
*  True by default for horizontal Layout.orientation, false for vertical.
* @type {boolean}
* @default null
*/
showInlineSeparators?: boolean;

/**
* ToolStripButton to display when AdaptiveMenu.showIconOnlyInline is set for one MenuItem
* @type {ImgButton}
* @default null
*/
inlineImgButton?: ImgButton;



/**
* 
* 
* @param {Array<Partial<MenuItem>> | MenuItem} array of menuItems to replace current menuItems
*/
setItems(items:Array<Partial<MenuItem>> | MenuItem): void;

/**
* 
* 
* @param {boolean} */
setPartialInlining(partialInlining:boolean): void;



}
/**
* Creates a new AdaptiveMenu
* 
* @param typeCheckedProperties {Partial<AdaptiveMenu>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {AdaptiveMenu} The new instance
*/
class AdaptiveMenu{
static create(typeCheckedProperties? : Partial<AdaptiveMenu>, uncheckedProperties?: Object): AdaptiveMenu;







}

/**
* Adds a new screen instance to a Layout, as a new Tab in a TabSet or as a new Section in
*  a SectionStack. When the target is a TabSet or SectionStack, a static
*  AddScreenTask.title or dynamic AddScreenTask.titleFormula can be assigned
*  for the new Tab or Section.
*  
*  The new screen's Canvas.dataContext can be configured with
*  AddScreenTask.dataContextBinding evaluated
*  in the scope of this task.
*/
interface AddScreenTask extends ComponentTask {
/**
* Title of new SectionStackSection or TabSet when ComponentTask.componentId targets a SectionStack
*  or TabSet. To use a dynamic title see AddScreenTask.titleFormula.
* @type {string}
* @default null
*/
title?: string;

/**
* Formula to be used to calculate the title contents. Use AddScreenTask.title property
*  to assign a static title instead.
*  
*  Available fields for use in the formula are the current Canvas.ruleScope.
* @type {UserSummary}
* @default null
*/
titleFormula?: UserSummary;

/**
* Can the created SectionStackSection or TabSet be closed by the user?
* @type {boolean}
* @default null
*/
canClose?: boolean;

/**
* Name of screen to be added.
* @type {string}
* @default null
*/
screenName?: string;

/**
* A DataContextBinding to be applied to the created screen via
*  Canvas.setDataContext.
* @type {DataContextBinding}
* @default null
*/
dataContextBinding?: DataContextBinding;





}
/**
* Creates a new AddScreenTask
* 
* @param typeCheckedProperties {Partial<AddScreenTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {AddScreenTask} The new instance
*/
class AddScreenTask{
static create(typeCheckedProperties? : Partial<AddScreenTask>, uncheckedProperties?: Object): AddScreenTask;







}

/**
* A widget for editing a single, advanced HiliteRule for use by 
*  DataBoundComponent. Where a simple hilite provides 
*  configuration of a single criterion and either foreground or background color for 
*  application to a single field, an advanced hilite can specify more complex criteria which can 
*  both test and affect multiple fields and allow both background and foreground colors to 
*  be specified in a single rule. 
*  
*  Important Note: this class should not be used directly - it is exposed purely for
*  i18nMessages
*/
interface AdvancedHiliteEditor extends VStack {
/**
* AutoChild FilterBuilder for configuring the criteria for this Hilite.
*  
*  This component is an AutoChild and as such may be customized via 
*  advancedHiliteEditor.filterBuilderProperties.
* @type {FilterBuilder}
* @default null
*/
filterBuilder?: FilterBuilder;

/**
* The title for the Filter group.
* @type {string}
* @default "Filter"
*/
filterGroupTitle?: string;

/**
* Specifies a list of icons that can be used in hilites.
*  
*  hiliteIcons should be specified as an Array of SCImgURL.
*  When present, HiliteRule
*  will offer the user a drop down for picking one of these icons.
*  
*  If the user picks an icon, the created hiliting rule will have Hilite.icon set to 
*  the chosen icon. ListGridField.hiliteIconPosition controls where the icon will 
*  appear for that field -- the default is that it appears in front of the normal cell content.
* @type {Array<Partial<SCImgURL>>}
* @default null
*/
hiliteIcons?: Array<Partial<SCImgURL>>;

/**
* AutoChild DynamicForm for configuring the details of this Hilite.
*  
*  This component is an AutoChild and as such may be customized via 
*  advancedHiliteEditor.hiliteFormProperties.
* @type {DynamicForm}
* @default null
*/
hiliteForm?: DynamicForm;

/**
* The title for the Appearance group.
* @type {string}
* @default "Appearance"
*/
appearanceGroupTitle?: string;

/**
* The title for the Target Field(s) picker.
* @type {string}
* @default "Target Field(s)"
*/
targetFieldsItemTitle?: string;

/**
* AutoChild ImgButton that accepts this Hilite and fires the 
*  AdvancedHiliteEditor.callback.
*  
*  This component is an AutoChild and as such may be customized via 
*  advancedHiliteEditor.saveButtonProperties.
* @type {StatefulCanvas}
* @default null
*/
saveButton?: StatefulCanvas;

/**
* The title text for the AdvancedHiliteEditor.saveButton.
* @type {string}
* @default "Save"
*/
saveButtonTitle?: string;

/**
* AutoChild ImgButton that cancels this AdvancedHiliteEditor without saving 
*  any changes, firing the AdvancedHiliteEditor.callback with a null
*  parameter.
*  
*  This component is an AutoChild and as such may be customized via 
*  advancedHiliteEditor.cancelButtonProperties.
* @type {StatefulCanvas}
* @default null
*/
cancelButton?: StatefulCanvas;

/**
* The title text for the AdvancedHiliteEditor.cancelButton.
* @type {string}
* @default "Cancel"
*/
cancelButtonTitle?: string;

/**
* The message to show when the user clicks "Save" without entering any criteria.
* @type {string}
* @default "Enter at least one rule, a color or icon, and a target field, or press 'Cancel' to abandon changes."
*/
invalidHilitePrompt?: string;

/**
* The title text shown in the header bar of this editor's dialog.
* @type {string}
* @default "Advanced Highlight Editor"
*/
title?: string;

/**
* The callback to fire when the AdvancedHiliteEditor.saveButton is clicked.
* @type {Callback}
* @default null
*/
callback?: Callback;



/**
*  Save changes and fire the AdvancedHiliteEditor.callback.
* 
* 
*/
saveHilite(): void;

/**
*  Discard changes and fire the AdvancedHiliteEditor.callback with a null 
*  parameter.
* 
* 
*/
cancelEditing(): void;



}
/**
* Creates a new AdvancedHiliteEditor
* 
* @param typeCheckedProperties {Partial<AdvancedHiliteEditor>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {AdvancedHiliteEditor} The new instance
*/
class AdvancedHiliteEditor{
static create(typeCheckedProperties? : Partial<AdvancedHiliteEditor>, uncheckedProperties?: Object): AdvancedHiliteEditor;







}

/**
* Ask the user to input a value.
*/
interface AskForValueTask extends UserConfirmationTask {
/**
* Default value.
* @type {string}
* @default null
*/
defaultValue?: string;





}
/**
* Creates a new AskForValueTask
* 
* @param typeCheckedProperties {Partial<AskForValueTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {AskForValueTask} The new instance
*/
class AskForValueTask{
static create(typeCheckedProperties? : Partial<AskForValueTask>, uncheckedProperties?: Object): AskForValueTask;







}

/**
* Synonym for the Authentication class
*/
interface Auth extends Authentication {




}
/**
* Creates a new Auth
* 
* @param typeCheckedProperties {Partial<Auth>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Auth} The new instance
*/
class Auth{
static create(typeCheckedProperties? : Partial<Auth>, uncheckedProperties?: Object): Auth;







}

/**
* The Authentication or Auth class represents a convenient, standard place to keep 
*  information about the currently logged in user and their assigned user roles.
*  
*  The intended usage is that a server authentication system would require the user to log in, 
*  then provide data about the currently logged in user via Authentication.setCurrentUser 
*  and Authentication.setRoles. This data is then available in the 
*  Canvas.ruleScope so that components can use it to enable or disable or
*  hide themselves, via properties such as FormItem.readOnlyWhen.
*  
*  The format for user records is not explicitly defined or restricted by the Authentication 
*  subsystem but we recommend using the format described by Authentication.getUserSchema.
*  Having a standardized user record allows application designers to rely on a 
*  well-known set of field names at design time, and then at deployment time when a 
*  particular authentication system is chosen, the deployer can simply fill in the 
*  standardized user record from the data that the chosen authentication system returns. 
*  This also allows authentication systems to be swapped out in the future without 
*  the need to change application code.
*  
*  The DataSource returned by Authentication.getUserSchema is used solely for visual 
*  tools to help with application authoring.
*  It is not intended to be used directly to store and retrieve user data, and while we
*  recommend this format it is not a requirement that user records conform to it.
*  
*  There are no security implications to calling setRoles() or
*  other APIs on the Authentication class. The provided data affects only 
*  client-side components. All actual security enforcement must be done server-side - 
*  see the QuickStart Guide, especially the sections on Declarative Security, 
*  to understand how role-based authorization can be used on the server.
*/
interface Authentication extends Class {




}
/**
* Creates a new Authentication
* 
* @param typeCheckedProperties {Partial<Authentication>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Authentication} The new instance
*/
class Authentication{
static create(typeCheckedProperties? : Partial<Authentication>, uncheckedProperties?: Object): Authentication;
/**
* URL to open for logging the current user out.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  The dynamic variables available are the fields in the Authentication.getCurrentUser record.
* @type {string}
* @default null
*/
static logOutURL?: string;

/**
* URL to open for reseting the current user's password.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  The dynamic variables available are the fields in the Authentication.getCurrentUser record.
* @type {string}
* @default null
*/
static resetPasswordURL?: string;





/**
*  Set up the current user. This method makes the user record available in the 
*  Canvas.ruleScope as "auth.currentUser".
* 
* 
* @param {Record} Record with attributes detailing the current user
*/
static setCurrentUser(user:Record): void;

/**
*  Returns the current user specified by Authentication.setCurrentUser. 
*  
*  This method returns the user record currently available in the 
*  Canvas.ruleScope as "auth.currentUser".
* 
* 
*/
static getCurrentUser(): Record;

/**
*  Convenience method to return the "userId" attribute
*  of the Authentication.getCurrentUser if there is one.
* 
* 
*/
static getCurrentUserId(): string;

/**
*  Returns a DataSource describing the standard schema for user data.
*  
*  The schema contains the following fields:
*  
*  Field NameType
*  "userId""text"
*  "email""text"
*  "firstName""text"
*  "lastName""text"
*  "title""text"
*  "phone""phoneNumber"
*  "superUser""boolean"
*  
* 
* 
*/
static getUserSchema(): DataSource;

/**
*  Set the user roles for the current user. Roles may be retrieved via Authentication.getRoles.
*  
*  Calling setRoles() makes the specified set of user roles available in the Canvas.ruleScope 
*  as a top-level property "userRoles", so that it can be used in criteria such as 
*  Canvas.visibleWhen or FormItem.readOnlyWhen. 
*  
*  Note that if this current user has been Authentication.setSuperUser,
*  Authentication.getRoles will return the full set of available roles rather than the set of
*  roles specified here.
*  
* 
* @param {Array<Partial<string>>} set of roles which apply to the current user
*/
static setRoles(roles:Array<Partial<string>>): void;

/**
*  Returns the current set of user roles. For Authentication.setSuperUser
*  this will be the intersection of any roles specified by 
*  Authentication.setRoles and the full set of Authentication.setAvailableRoles
*  - otherwise it will be the set of roles specified by Authentication.setRoles.
*  
*  Current set of user roles are available in the Canvas.ruleScope 
*  as a top-level property "userRoles", so that it can be used in criteria such as 
*  Canvas.visibleWhen or FormItem.readOnlyWhen. 
* 
* 
*/
static getRoles(): Array<Partial<string>>;

/**
*  Is the current user assigned to the specified role?
* 
* 
* @param {string} role to check in current roles
*/
static hasRole(role:string): boolean;

/**
*  Specify the full set of available user roles.
*  
*  Note that if the current user has been marked as a 
*  Authentication.isSuperUser, Authentication.getRoles will return
*  the full set of available roles.
* 
* 
* @param {Array<Partial<string>>} full set of possible user roles.
*/
static setAvailableRoles(roles:Array<Partial<string>>): void;

/**
*  Returns the full set of available user roles specified by Authentication.setAvailableRoles.
* 
*/
static getAvailableRoles(): Array<Partial<string>>;

/**
*  Mark the current user as a super-user. This causes Authentication.getRoles to return
*  the full set of Authentication.getAvailableRoles if specified
* 
* @param {boolean} New super user status
*/
static setSuperUser(isSuperUser:boolean): void;

/**
*  Has the current user been marked as a super-user via Authentication.setSuperUser?
* 
* @param {boolean} New super user status
*/
static isSuperUser(isSuperUser:boolean): void;




}

/**
* A button that automatically sizes to the length of its title. Implemented via the 
*  StatefulCanvas.autoFit property.
* @deprecated * As of Isomorphic SmartClient version 5.5, autoFit behavior can be achieved using the Button class instead by setting the property Button.autoFit to true.

*/
interface AutoFitButton extends Button {




}
/**
* Creates a new AutoFitButton
* 
* @param typeCheckedProperties {Partial<AutoFitButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {AutoFitButton} The new instance
*/
class AutoFitButton{
static create(typeCheckedProperties? : Partial<AutoFitButton>, uncheckedProperties?: Object): AutoFitButton;







}

/**
* Class for editable multi-line text areas (uses HTML &lt;TEXTAREA&gt; object)
*  automatically expands to accommodate its content
*/
interface AutoFitTextAreaItem extends TextAreaItem {
/**
* If specified, the autoFitTextArea will not grow taller than this height
* @type {Integer}
* @default null
*/
maxHeight?: Integer;





}
/**
* Creates a new AutoFitTextAreaItem
* 
* @param typeCheckedProperties {Partial<AutoFitTextAreaItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {AutoFitTextAreaItem} The new instance
*/
class AutoFitTextAreaItem{
static create(typeCheckedProperties? : Partial<AutoFitTextAreaItem>, uncheckedProperties?: Object): AutoFitTextAreaItem;







}

/**
* Standalone class providing a general interface for integration with Automated Testing Tools
*  
*  
*  For automated testing tools we need a way to create string identifiers for DOM elements such that 
*  when a page is reloaded, we can retrieve a functionally equivalent DOM element. We call these
*  AutoTestLocator.
*  
*  This allows automated testing tools to set up or record user generated events on DOM elements
*  then play them back on page reload and have our components react correctly.
*  
*  The primary APIs for the AutoTest subsystem are AutoTest.getLocator and 
*  AutoTest.getElement. Developers should also be aware of the
*  AutoTest.installLocatorShortcut method which allows you to retrive locators via
*  a simple key combo + mouseDown event on the target element.
*  
*  Implementation considerations:
*  
*  
* - Some components react to the structure of DOM elements embedded within them - for example
*   GridRenderer cells have meaning to the grid. So in some cases we need to identify elements
*   within a component, while in others we can just return a pointer to a handle (A simple
*   canvas click handler doesn't care about what native DOM element within the handle received
*   the click).
* 
*  
* - When a DOM element is contained by a component, it is not sufficient to store the component
*   ID. Most SmartClient components are auto-generated by their parents, and rather than 
*   attempting to store a specific component identifier we should instead store the
*   "logical function" of the component.
*   For example a listGrid header button may have a different auto-generated ID across page
*   reloads due to various timing-related issues (which can change the order of of widget
*   creation), loading a new skin, or otherwise trivial changes to an application.
*   Rather than storing the header button ID therefore, we want to store this as
*   a string meaning "The header button representing field X within this list grid".
* 
*  
* - fallback strategies: In some cases a component or DOM element can be identified in 
*   several ways. For example a cell in a ListGrid can be identified by simple row and
*   column index, but also by fieldName and record primary key value. In these cases we
*   attempt to record information for multiple locator strategies and then when parsing
*   stored values we can provide APIs to govern which strategy is preferred. See the
*   LocatorStrategy documentation for more on this.
*  
*  
*  In order to address these concerns the AutoTest locator pattern is similar to an
*  XPath type structure, containing a root component identifier, followed by 
*  details of sub-components and then potentially details used to identify an element within
*  the components handle in the DOM.
*  
*  The actual implementation covers a large number of common cases, including (but not limited to)
*  the following. Note that for cases where an element is being identified from a pool of
*  possible candidates, such as the Canvas.children array, we usually will use
*  LocatorStrategy rather than simply relying on index:
*  
* - Root level components identified by explicit ID
*    
* - Any AutoChild
*    
* - Standard component parts such as scrollbars, edges, shadows, etc
*    
* - Section stack items and headers
*    
* - Window items
*    
* - ListGrid headers and cells
*    
* - TreeGrid headers and cells, including interactive open icon, checkbox icons
*    
* - DynamicForm form items, including details of elements within those items
*  
*  
*/
interface AutoTest extends Class {




}
/**
* Creates a new AutoTest
* 
* @param typeCheckedProperties {Partial<AutoTest>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {AutoTest} The new instance
*/
class AutoTest{
static create(typeCheckedProperties? : Partial<AutoTest>, uncheckedProperties?: Object): AutoTest;
/**
* Controls whether certain AutoTest APIs wait for network operations to complete
*  before returning true. When value is true, AutoTest.isElementClickable
*  will return false until all network operations have completed.
* @type {boolean}
* @default false
*/
static implicitNetworkWait?: boolean;

/**
* Sets the implicit root canvas available in scLocators starting "//testRoot[]".
*  Setting this property may enable one to use the same script to test identical
*  widget hierarchies that are rooted under different base widgets.
* @type {Canvas}
* @default null
*/
static testRoot?: Canvas;

/**
* Should HTMLStrings in fallback locator attributes be simplified to ordinary strings
*  before being written out? For example:
*  
*   title="&lt;div id=\"foo\"&gt;bar&lt;/div&gt;"
*  
*  would become
*  
*    title="bar"
*  
*  if so simplified.
* @type {boolean}
* @default true
*/
static simplifyLocatorAttributeHTML?: boolean;

/**
* When resolving locators with search segments, should hidden 
*  or undrawn canvases be skipped?
* @type {boolean}
* @default true
*/
static searchVisibleOnly?: boolean;

/**
* Default ElementWaitStyle for AutoTest.waitForElement
* @type {ElementWaitStyle}
* @default "system"
*/
static defaultElementWaitStyle?: ElementWaitStyle;





/**
*  Returns the AutoTestLocator associated with some DOM element in a SmartClient
*  application page. If coords, representing the page position, is passed in, the locator
*  may be generated with a specific trailing "target area" identifer that will map back to
*  the appropriate, potentially different, physical coordinates, even if the widget is
*  moved. The coords argument will only have an effect in cases where the mouse position
*  over the target could potentially change behavior.
* 
* @param {DOMElement} DOM element within in the page. If null the locator for the last mouse event target will be generated
* @param {boolean=} If this parameter is passed in, check whether the target element responds to native browser events directly rather than going through  the SmartClient widget/event handling model. If we detect this case, return null rather  than a live locator. This allows us to differentiate between (for example) an event on  a Canvas handle, and an event occurring directly on a simple  &lt;a href=...&gt; tag written inside a Canvas handle.
* @param {Array<any>=} X, Y page position
*/
static getLocator(DOMElement:DOMElement, checkForNativeHandling?:boolean, coords?:Array<any>): AutoTestLocator;

/**
*  Returns the QualityIndicatedLocator associated with some DOM element in a
*  SmartClient application page. If coords, representing the page position, is passed in,
*  the locator
*  may be generated with a specific trailing "target area" identifer that will map back to
*  the appropriate, potentially different, physical coordinates, even if the widget is
*  moved. The coords argument will only have an effect in cases where the mouse position
*  over the target could potentially change behavior.
* 
* @param {DOMElement} DOM element within in the page. If null the locator for the last mouse event target will be generated
* @param {boolean=} If this parameter is passed in, check whether the target element responds to native browser events directly rather than going through  the SmartClient widget/event handling model. If we detect this case, return null rather  than a live locator. This allows us to differentiate between (for example) an event on  a Canvas handle, and an event occurring directly on a simple  &lt;a href=...&gt; tag written inside a Canvas handle.
* @param {Array<any>=} X, Y page position
*/
static getLocatorWithIndicators(DOMElement:DOMElement, checkForNativeHandling?:boolean, coords?:Array<any>): QualityIndicatedLocator;

/**
*  Given an element in the DOM, returns the canvas containing this element, or null if
*  the element is not contained in any canvas handle.
* 
* @param {DOMElement} DOM element within in the page
*/
static locateCanvasFromDOMElement(element:DOMElement): Canvas;

/**
* @param {AutoTestLocator} Locator String previously returned by    AutoTest.getLocator
*/
static getElement(locator:AutoTestLocator): DOMElement;

/**
*  Given an AutoTestLocator, return the live SmartClient object it refers to, if any.
* 
* @param {AutoTestLocator} Locator String previously returned by    AutoTest.getLocator
*/
static getObject(locator:AutoTestLocator): Canvas | FormItem | SectionStackSection;

/**
*  Given an AutoTestLocator that refers to a live SmartClient object or a logical
*  subcomponent of that object, return the associated meaningful JS value, if any.
*  
*  For example:
*  
*    
* - For a locator to a ListGrid/CubeGrid cell, return the cell's value
*    
* - For a locator to a FormItem, return the FormItem's value
*    
* - For a locator to a ListGrid field header, return the checkbox/sorting state
*    
* - For a locator to a Calendar EventCanvas header or body, return the text
*  
* 
* @param {AutoTestLocator} Locator String previously returned by    AutoTest.getLocator
*/
static getValue(locator:AutoTestLocator): object;

/**
*  Returns the page-level coordinates corresponding to the supplied locator. Note: The
*  physical position might change due to app redesign, but these coordinates would still
*  reflect the same logical part of the DOM element for components where event position
*  matters.
* 
* @param {AutoTestLocator} Locator String previously returned by    AutoTest.getLocator
*/
static getPageCoords(locator:AutoTestLocator): Array<any>;

/**
*  This method allows developers to retrieve locators for elements on the page via
*  a key-combo plus mouseDown on the element in question.
*  It may be invoked from a bookmarklet stored in the browser, giving developers a
*  one-click way to retrieve locators for any SmartClient application
*  
*  When installLocatorShortcut() is invoked, it will register a Page-level mouseDown
*  handler which, if the Shift+Ctrl or Shift+Meta key-combo are being held down will
*  display the locator for the element under the mouse in a text-box and also copy it to
*  clipboard.
*  
*  As with the debugging method, developers may wish to create a 
*  bookmark in their browser to quickly enable this functionality on any SmartClient 
*  application, without any changes to the application code:
*  
*  
* - Create a new bookmark in your browser.
*  
* - Enter url "javascript:isc.AutoTest.installLocatorShortcut()".
*  
* - Label the bookmark as "Locator Shortcut"
*  
* - Consider adding this to the Bookmarks Toolbar. This allows you to enable the 
*  feature with a single click from any SmartClient application.
*  
*  
*  To uninstall the locator shortcut, call AutoTest.uninstallLocatorShortcut
* 
* 
*/
static installLocatorShortcut(): void;

/**
*  Uninstalls the locator shortcut installed by AutoTest.installLocatorShortcut
* 
*/
static uninstallLocatorShortcut(): void;

/**
*  Sets the implicit root canvas available in scLocators starting "//testRoot[]".
*  Setting this property may enable one to use the same script to test identical
*  widget hierarchies that are rooted under different base widgets.
* 
* @param {Canvas} the implicit root
*/
static setTestRoot(canvas:Canvas): void;

/**
*  Returns a locator that's optimized to have minimal length while still trying to robustly
*  represent that targeted canvas. The returned locator may simply be an ID, if the target
*  has a manually assigned ID, or it may consist of syntax starting with //:
*  that will search for a widget with the specified ID or defining property (e.g. "title").
*  A minimal locator may also contain a second, interior search denoted by //
*  that searches the hierarchy below the preceding part of the locator.
*  
*  A minimal locator should be passed to AutoTest.getObject, AutoTest.getElement, etc., to
*  retrieve the original target.
*  
*  Note that when searching by defining property, a breadth-first search of the widget
*  hierarchy will be run looking for the first widget of the appropriate framework class
*  with the specified defining property.
* 
* 
* @param {DOMElement} DOM element within in the page. If null the locator for the last mouse event target will be generated
* @param {boolean=} If this parameter is passed in, check whether the target element responds to native browser events directly rather than going through  the SmartClient widget/event handling model. If we detect this case, return null rather  than a live locator. This allows us to differentiate between (for example) an event on  a Canvas handle, and an event occurring directly on a simple  &lt;a href=...&gt; tag written inside a Canvas handle.
* @param {Array<any>=} X, Y page position
*/
static getMinimalLocator(DOMElement:DOMElement, checkForNativeHandling?:boolean, coords?:Array<any>): AutoTestLocator;

/**
*  Returns whether the canvas associated with the given DOM element
*  is in a consistent state with no pending operations. Returns null
*  if the argument is not valid or isn't associated with an element
*  representing a valid canvas.
*  Otherwise, returns true or false according as the conditions below are all satisfied:
*  
*    
* - page has finished loading
*    
* - canvas is drawn
*    
* - canvas isn't dirty
*    
* - canvas has no queued overflow operations
*    
* - canvas is not animating
*  
* 
* @param {Canvas | DOMElement | AutoTestLocator}  DOM element to test (element obtained from canvas or SmartClient locator if provided) 
*/
static isCanvasDone(element:Canvas | DOMElement | AutoTestLocator): boolean;

/**
*  Returns whether the TileLayout associated with the given DOM element
*  is in a consistent state with no pending operations. Returns null if the
*  argument is not valid or isn't associated with an element representing
*  a valid TileLayout.
*  Otherwise, returns true or false according as the conditions below are all satisfied:
*  
*    
* - page has finished loading
*    
* - the TileLayout (as a canvas) satisfies AutoTest.isCanvasDone
*    
* - the TileLayout is not currently animating any layout operations
*  
* 
* @param {Canvas | DOMElement | AutoTestLocator}  DOM element to test (element obtained from canvas or SmartClient locator if provided) 
*/
static isTileLayoutDone(element:Canvas | DOMElement | AutoTestLocator): boolean;

/**
*  Returns whether the TileGrid associated with the given DOM element
*  is in a consistent state with no pending operations. Returns null if the
*  argument is not valid or isn't associated with an element representing
*  a valid TileGrid.
*  Otherwise, returns true or false according as the conditions below are all satisfied:
*  
*    
* - page has finished loading
*    
* - the TileGrid (as a tileLayout) satisfies AutoTest.isTileLayoutDone
*    
* - the TileGrid has no pending layout animation operations queued
*  
* 
* @param {Canvas | DOMElement | AutoTestLocator}  DOM element to test (element obtained from canvas or SmartClient locator if provided) 
*/
static isTileGridDone(element:Canvas | DOMElement | AutoTestLocator): boolean;

/**
*  Returns whether the DynamicForm associated with the given DOM element
*  is in a consistent state with no pending operations. Returns null
*  if the argument is not valid or isn't associated with an element
*  representing a valid
*  DynamicForm. Otherwise, returns true or false according as the conditions
*  below are all satisfied:
*  
*    
* - page has finished loading
*    
* - form has no pending delayed "set values" or "set values focus" operations
*    
* - all contained items satisfy AutoTest.isItemDone
*  
* 
* @param {Canvas | DOMElement | AutoTestLocator}  DOM element to test (element obtained from canvas or SmartClient locator if provided) 
*/
static isFormDone(element:Canvas | DOMElement | AutoTestLocator): boolean;

/**
*  Returns whether the FormItem associated with the given DOM element
*  is in a consistent state with no pending operations. Returns null
*  if the argument is not valid or isn't associated with an element
*  representing a valid
*  FormItem. Otherwise, returns true or false according as the conditions
*  below are all satisfied:
*  
*    
* - page has finished loading
*    
* - if the container widget of the item isn't the parent DynamicForm, then the
*      container widget must satisfy AutoTest.isCanvasDone (or AutoTest.isGridDone,
*      etc., as appropriate)
*    
* - the item cannot have any fetches in progress for missing display/value field
*      values
*    
* - picklists (SelectItem or ComboBoxItem) cannot have any pending row
*      fetches
*    
* - any contained FormItems must satisfy isItemDone() themselves
*    
* - any contained Canvas must satisfy AutoTest.isCanvasDone (or 
*      AutoTest.isGridDone, etc., as appropriate)
*  
* 
* @param {FormItem | DOMElement | AutoTestLocator}  DOM element to test (element obtained from canvas or SmartClient locator if provided) 
*/
static isItemDone(element:FormItem | DOMElement | AutoTestLocator): boolean;

/**
*  Returns whether the ListGrid associated with or contained by the given
*  DOM element
*  is in a consistent state with no pending operations. Returns null if the 
*  argument is not valid or isn't associated with an element inside a valid ListGrid.
*  
*  Otherwise, returns true or false according as the conditions below are all satisfied:
*  
*    
* - page has finished loading
*    
* - no pending scroll operations
*    
* - no pending filter editor operations
*    
* - no unsaved edits to the grid records
*    
* - no asynchronous regrouping operations are in progress
*    
* - no outstanding fetch/filter operations are present for the ResultSet
*    
* - no outstanding sort operations are present that will update the ListGrid
*  
* 
* @param {Canvas | DOMElement | AutoTestLocator}  DOM element to test (element obtained from canvas or SmartClient locator if provided) 
*/
static isGridDone(element:Canvas | DOMElement | AutoTestLocator): boolean;

/**
*  Returns whether the DOM element
*  is ready to be clicked on by a Selenium test. Returns null if the 
*  argument is not valid or isn't associated with an element representing
*  a valid canvas or form item.
*  Otherwise, returns true or false according as the conditions below are all satisfied:
*  
*    
* - page has finished loading
*    
* - no network operations are outstanding (configurable, 
*      see AutoTest.implicitNetworkWait)
*    
* - canvas is visible, enabled, and not masked,
*    
* - canvas satisfies AutoTest.isCanvasDone
*    
* - if canvas is a TileGrid, it satisfies AutoTest.isTileGridDone
*    
* - if canvas is a TileLayout, it satisfies AutoTest.isTileLayoutDone
*    
* - if canvas is a ListGrid or body of a ListGrid, it satisfies AutoTest.isGridDone
*    
* - if canvas is a DynamicForm, it satisfies AutoTest.isFormDone
*  
*  Note that for a form item in a DynamicForm, the DynamicForm must satisfy the third
*  condition above, while the container widget of the element must satisfy the remaining
*  conditions.
*  
* 
* @param {Canvas | FormItem | DOMElement | AutoTestLocator}  DOM element to test (element obtained from canvas, form item, or SmartClient locator if provided) 
*/
static isElementClickable(element:Canvas | FormItem | DOMElement | AutoTestLocator): boolean;

/**
*  Given a DOM element, returns whether the associated SmartClient Canvas or FormItem is
*  ready to receive keyPress events from a Selenium test. Returns null if the locator is
*  not valid or doesn't represent a valid Canvas or FormItem. Otherwise, returns true or
*  false according as the conditions below are all satisfied:
*  
*    
* - page has finished loading
*    
* - if a TextItem, FileItem, or TextAreaItem, 
*      it has native focus,
*    
* - the element satisfies AutoTest.isElementClickable
*  
* 
* @param {Canvas | FormItem | DOMElement | AutoTestLocator} DOM element to test   (element obtained from canvas, form item, or SmartClient locator if provided)
*/
static isElementReadyForKeyPresses(element:Canvas | FormItem | DOMElement | AutoTestLocator): boolean;

/**
*  Returns a boolean to indicate whether the current application is fully loaded with no pending 
*  operations, and is ready for user interaction. This includes the following checks:
*  
*    
* - page has finished loading
*    
* - all ListGrids (as defined by isc.isA.ListGrid) satisfy AutoTest.isGridDone
*    
* - all TileGrids that are drawn satisfy AutoTest.isTileGridDone
*    
* - all TileLayouts that are drawn satisfy AutoTest.isTileLayoutDone
*    
* - all DynamicForms that are drawn satisfy AutoTest.isFormDone
*    
* - all Canvii that are drawn satisfy AutoTest.isCanvasDone
*  
*  In addition to this the options parameter allows developers to
*  check additional criteria as documented under SystemDoneConfig.
*  
*  Note: AutoTest.waitForSystemDone will poll this method repeatedly to check for all pending 
*  actions being complete. The SystemWaitConfig parameter of that method 
*  allow the user to speciofy which actions to wait for. By default AutoTest.waitForSystemDone
*  will wait for all actions to complete, including queued redraws, timers, network requests,
*  as well as page load and comopnent level isComponentDone checks.
*  
*  Depending on your application configuration, it is possible that this method
*  will not be able to determine whether it is truly in a busy state. Some examples include:
*  
*  
* - If SystemDoneConfig.includeTimers is true and the application has
*    a constantly running background timer to perform some
*    ongoing polling action or similar, this method may return false
*    even though the application is ready for interaction
*  
*  
* - Similarly if SystemDoneConfig.includeRedraws is true, and the application is
*    using messaging, or some other mechanism to periodically refresh component(s)
*    such that they are repeatedly marked as dirty, this method may return
*    false even though the application is ready for interaction
*  
*  
* - If this application makes use of some third-party library to perform
*    asynchronous actions, this method may return true even when
*    the application is waiting on a response from this third-party tool.
*  
*  
* 
* 
* @param {SystemDoneConfig | boolean=} This parameter determines what specific outstanding actions are required to be completed before the system is considered  "done" with current processing. If passed as a boolean rather than a SystemDoneConfig  object, this determines whether the system should wait for  Canvas.markForRedraw.
* @param {boolean=} whether to check for unsaved edits for ListGrids. Note the includeEdits parameter is deprecated and may be removed in the future.
*/
static isSystemDone(options?:SystemDoneConfig | boolean, includeEdits?:boolean): boolean;

/**
*  Fires the provided callback when AutoTest.isSystemDone returns true.
*  
*  By default this will wait for the following system actions to complete:
*  
*  
* - All outstanding SmartClient RPCManager
*  
* - All outstanding FileLoader
*  
* - Any Canvas.markForRedraw
*  
* - Any non-recurring Timer
*  
*  Note that this behavior may differ from the default settings for
*  AutoTest.isSystemDone
*  
*  The options parameter gives developers finer grained control
*  over what to wait for.
*  
* 
* @param {Callback} Action to fire. This will take a single parameter "done" which will be set to false if the method timed out waiting for isSystemDone() to  return true;
* @param {SystemWaitConfig=} options to configure the wait
*/
static waitForSystemDone(callback:Callback, options?:SystemWaitConfig): void;

/**
*  Waits for any in-progress actions to complete and then returns the
*  element for the specified locator.
*  
*  By default this method will wait for the AutoTest.isSystemDone 
*  before calling AutoTest.getElement with the specified locator.
*  The ElementWaitConfig.waitStyle attribute of the options 
*  parameter may be used to instead wait for the locator to resolve to a
*  AutoTest.isElementClickable.
* 
* 
* @param {AutoTestLocator} locator for the element to retrieve
* @param {Callback} callback to fire when the element has been retrieved - takes two parameters:  
- "element" (DOMElement) - the clickable element the locator resolved to (or null)    
- "done" (boolean) - this will be false if the method timed out
* @param {ElementWaitConfig=} Options to configure the wait
*/
static waitForElement(locator:AutoTestLocator, callback:Callback, options?:ElementWaitConfig): void;




}

/**
* Base class for Canvas and DrawItem.
*/
interface BaseWidget extends Class {




}
/**
* Creates a new BaseWidget
* 
* @param typeCheckedProperties {Partial<BaseWidget>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {BaseWidget} The new instance
*/
class BaseWidget{
static create(typeCheckedProperties? : Partial<BaseWidget>, uncheckedProperties?: Object): BaseWidget;







}

/**
* The BatchUploader handles the upload, validation, review and saving of a dataset
*  expressed in CSV or other upload formats.
*  
*  NOTE: BatchUploader is only available with SmartClient Power or better.
*  
*  By default, a BatchUploader consists of a single FileItem form field.
*  This form field will upload a file using the special "batchUpload" built-in DataSource.
*  The uploaded file data will be parsed and validated using the
*  BatchUploader.uploadDataSource,
*  then streamed back to the browser, along with any errors, for display in a ListGrid. 
*  
*  The BatchUploader.previewShown notification method will be fired when the uploaded records
*  are being displayed in this grid.
*  
*  The user can then correct any errors and submit the final dataset, which will be
*  added to the DataSource via a series of "add" DSRequests, all submitted as a single
*  HTTP request via RPCManager.startQueue.
*  Developers may specify a custom "add" operation to use on the target 
*  BatchUploader.uploadDataSource via the BatchUploader.uploadOperation 
*  property.
*  
*  Additional form fields can be added to the form that uploads the data file via 
*  BatchUploader.uploadFormFields. Values entered into these fields
*  are not included in the "add" DSRequests used to store the uploaded records. Instead, they
*  are stored as HttpSession attributes with the names corresponding to the 
*  names of the specified uploadFormFields (optionally with a 
*  BatchUploader.uploadFieldPrefix applied, in case
*  this is necessary to avoid name collisions in the session). This allows any custom logic
*  for the "add" operation to access these additional fields via httpSession.getAttribute().
*  If BatchUploader.uploadFormFields are not provided method 
*  httpSession.getAttribute() will not be called.
*  
*  Because all records are saved in a single HTTP request, a similar strategy of 
*  storing data as servletRequest or session attributes allows reuse of objects required to
*  perform the "add" operations (such as field values common to all added records,
*  or a SQL connection or transaction manager). 
*  
*  If you already have upload data available as a csv-formatted string or similar, the
*  BatchUploader can be used without showing an upload interface. This can be valuable 
*  if you are picking up sample data from some custom UI (a separate upload component, or
*  TextArea for copy/pasting content, for example).
*  To suppress the upload interface, set BatchUploader.showUploadForm to false and
*  invoke BatchUploader.uploadData directly. The uploadData()
*  method passes sample data to the server as values.pasteData. The server side 
*  batchUpload dataSource has logic to extract and work with this data, exactly
*  as if it had been directly uploaded as the content of a file.
*  
*  If DataSourceField.uploadFieldName is set on any of the 
*  BatchUploader.uploadDataSource's fields, the BatchUploader will use
*  that name to map the uploaded file's content.
*  
*  Note, that for BatchUploader.dataFormat header line is optional. If 
*  first non-empty line in the uploaded file has no matching field names, it is assumed that 
*  there's no header row, and all rows (including the first one) are treated as data rows.
*  
*  Imported data can be transformed during import, see DataSourceField.importStrategy for details.
*  
*  A couple of server-side techniques are interesting in conjunction with the BatchUploader.
*  One is to set the DataSource.serverConstructor property to point at your own class
*  that inherits from com.isomorphic.datasource.BasicDataSource. The most interesting
*  reason for doing this is to override the validate method and provide complete
*  custom validation - for example, checking relations to other tables.
*  
*  Another technique is to handle the initial SmartClient call in your own servlet, by setting the 
*  BatchUploader.dataURL property. You then handle the add requests
*  with a combination of your own code and SmartClient server API calls. This is a 
*  good way to add special pre- and post-processing to the normal server-side flow.
*  
*  Note: The special "batchUpload" DataSource, which should reside in the shared/ds 
*  folder of your application's webroot (see 
*  iscInstall)
*  .
*  is not part of your application's data flow, and it has nothing to do with the 
*  BatchUploader.uploadDataSource you use to actually persist the validated
*  and error-corrected data: it is simply a means to uploading the raw data in the first place.
*  Normally, you should simply ignore its presence and treat it as an internal detail of the 
*  SmartClient framework.
*  
*  However, there are circumstances in which you may wish to change it to achieve specific aims.
*  For example, you may wish to override the Java class it invokes, in order to insert your own
*  security or other validation logic into the initial upload flow. This is entirely in keeping
*  with the design, but we regard it as an out-of-the-ordinary use-case: normal usage is simply 
*  to ignore the presence of the batchUpload DataSource.
*  
*  BatchUploader is a VStack, that simply stacks members on the vertical axis without
*  trying to manage their height. If you need to control heights, you can set 
*  Layout.vPolicy to "fill"
*/
interface BatchUploader extends VStack {
/**
* Should the BatchUploader.uploadForm be displayed.
*  
*  Setting this value to false will suppress the default upload interface for the
*  BatchUploader. In this case sample data may be uploaded directly via the
*  BatchUploader.uploadData method.
* @type {boolean}
* @default true
*/
showUploadForm?: boolean;

/**
* Form used to specify file to upload, and any additional fields required.
* @type {DynamicForm}
* @default null
*/
uploadForm?: DynamicForm;

/**
* DataSource used to save uploaded records. Should have an operation of type "add".
*  
*  Be careful to note that this is the DataSource representing your data as it will be 
*  persisted to your server. It is completely different from the special "batchUpload"
*  DataSource which is used purely as a medium to upload the raw data to the server in
*  the first place.
* @type {DataSource}
* @default null
*/
uploadDataSource?: DataSource;

/**
* Optional DSRequest.operationId for the "add" operation used to add new
*  records to the BatchUploader.uploadDataSource.
* @type {string}
* @default null
*/
uploadOperation?: string;

/**
* Optional fields for the uploadForm.
* @type {Array<Partial<Partial<FormItem>>>}
* @default null
*/
uploadFormFields?: Array<Partial<Partial<FormItem>>>;

/**
* Title to display next to the FileItem field where the user enters a filename to upload
* @type {string}
* @default "Upload File"
*/
uploadFileLabel?: string;

/**
* Error message to show when the uploading process detects a file with no data.
* @type {string}
* @default "The provided file is blank. Please, provide a file with data."
*/
errorMessageFileIsBlank?: string;

/**
* Error message to show when the uploaded file has rows other than the first row that could not be parsed.
*  
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed.
*  
*  The following variables are available to be used in this message:
*  
*  
* - goodRowCount: Total rows that were parsed correctly.
*  
* - totalRows: Total rows to be parsed in the uploaded file.
*  
* - firstBadRow: First row that could not be parsed.
*  
* - badRowCount: Total rows that could not be parsed.
*  
* @type {string}
* @default "Some rows could not be parsed; the grid below shows ${goodRowCount} of ${totalRows} data rows. Row number ${firstBadRow} was the first row to fail to be parsed."
*/
errorMessageRowsNotParsed?: string;

/**
* Error message to show when the uploading process detects an unterminated quote string in the first line.
* @type {string}
* @default "Unterminated quote string - problem found in the first line at character position: ${errorOffset}."
*/
errorMessageUnterminatedQuote?: string;

/**
* Error message to show when the uploading process detects a missing delimiter or end of line after quoted value in the first line.
* @type {string}
* @default "Delimiter or end of line expected after quoted value - problem found in the first line at character position: ${errorOffset}."
*/
errorMessageDelimiterOrEndOfLine?: string;

/**
* Error message to show when the uploading process detects an invalid inputType.
* @type {string}
* @default "Invalid inputType value was set!"
*/
errorMessageInputType?: string;

/**
* Error message to show when the uploading process detects an Excel File, which is not supported.
* @type {string}
* @default "You uploaded an Excel file. Please save your data as a csv file and re-upload"
*/
errorMessageExcelFileDetected?: string;

/**
* Error message to show when the uploading process is unable to detect the delimiter.
* @type {string}
* @default "We were unable to guess the delimiter"
*/
errorMessageUndeterminedDelimiter?: string;

/**
* Message to display after data has been committed, when 
*  BatchUploader.showCommitConfirmation is true.
* @type {string}
* @default "Records added"
*/
commitConfirmationMessage?: string;

/**
* Message to display when the user clicks "Commit" but there is nothing we can commit
*  because every row in the grid has errors
* @type {string}
* @default "All records have errors; nothing to commit"
*/
allRecordsInErrorMessage?: string;

/**
* Message to display after data has been committed, when 
*  BatchUploader.showCommitConfirmation is true.
* @type {string}
* @default "Valid records added; some records remain in error"
*/
partialCommitConfirmationMessage?: string;

/**
* Message to display if at least one update was rolled back due to errors in another row.
*  See the DataSource.autoJoinTransactions for details of 
*  SmartClient's automatic transactional updates feature
* @type {string}
* @default "One or more updates were rolled-back due to errors on other rows"
*/
updatesRolledBackMessage?: string;

/**
* Whether to show the BatchUploader.commitConfirmationMessage after
*  data is successfully committed.
* @type {boolean}
* @default true
*/
showCommitConfirmation?: boolean;

/**
* Title for the BatchUploader.uploadButton.
* @type {string}
* @default "Upload"
*/
uploadButtonTitle?: string;

/**
* Button that triggers the upload.
* @type {IButton}
* @default null
*/
uploadButton?: IButton;

/**
* Grid which will show a preview of data to be uploaded, with errors flagged
* @type {ListGrid}
* @default null
*/
grid?: ListGrid;

/**
* Fields to apply to BatchUploader.grid. These will override the field definitions
*  in the BatchUploader.uploadDataSource on a field by field basis, as described under
*  DataBoundComponent.fields.
* @type {Array<Partial<ListGridField>>}
* @default null
*/
gridFields?: Array<Partial<ListGridField>>;

/**
* Button that commits changes once the user is happy with the data.
* @type {IButton}
* @default null
*/
commitButton?: IButton;

/**
* Title for the BatchUploader.commitButton.
* @type {string}
* @default "Commit"
*/
commitButtonTitle?: string;

/**
* Button that cancels the uncommitted upload.
* @type {IButton}
* @default null
*/
cancelButton?: IButton;

/**
* Title for the BatchUploader.cancelButton.
* @type {string}
* @default "Cancel"
*/
cancelButtonTitle?: string;

/**
* Button that scrolls grid to the previous error.
* @type {IButton}
* @default null
*/
previousButton?: IButton;

/**
* Title for the BatchUploader.previousButton.
* @type {string}
* @default "Previous error"
*/
previousButtonTitle?: string;

/**
* Button that scrolls grid to the next error.
* @type {IButton}
* @default null
*/
nextButton?: IButton;

/**
* Title for the BatchUploader.nextButton.
* @type {string}
* @default "Next error"
*/
nextButtonTitle?: string;

/**
* Object containing properties to send with every "add" request this batchUploader sends.
* @type {DSRequest}
* @default null
*/
requestProperties?: DSRequest;

/**
* If set, the batchUploader will copy this value to the queue of "add" requests it sends 
*  to the server to actually populate the data. You can use this facility to route the 
*  queue to your own server-side logic, for example to add pre- or post-processing.
* @type {string}
* @default null
*/
dataURL?: string;

/**
* String to prepend to the names of the additional fields specified in BatchUploader.uploadFormFields 
*  before they are stored in the HttpSession on the server. This property provides a basic
*  namespace facility, allowing you to avoid name collisions with existing session attributes.
*  
*  Example usage: if you have an additional field called "someDate" and you set 
*  uploadFieldPrefix to "myFields_", your additionalFormField will be available as an
*  HttpSession attribute called "myFields_someDate"
* @type {string}
* @default null
*/
uploadFieldPrefix?: string;

/**
* Specifies what action to take if the user attempts to commit a partially validated set
*  of data (ie, one that still contains some errors).
* @type {PartialCommitOption}
* @default "prompt"
*/
partialCommit?: PartialCommitOption;

/**
* If BatchUploader.partialCommit is set to "prompt", the text to display to the user in the 
*  confirmation dialog. By default, this text is "There are errors in your data so it 
*  cannot all be saved. If you proceed, you will lose the records with errors. Click 'OK'
*  to proceed anyway, or 'Cancel' to return to your data"
* @type {string}
* @default "See below"
*/
partialCommitPrompt?: string;

/**
* If BatchUploader.partialCommit is set to "prevent", the text to display to the user if they try 
*  to commit a dataset containing errors. By default, this text is "There are errors in your 
*  data. Please correct all errors before clicking Commit"
* @type {string}
* @default "See below"
*/
partialCommitError?: string;

/**
* If set, indicates that a warning dialog should be shown when Cancel is clicked, asking 
*  the user to confirm that this is really what they want to do. The actual warning message
*  is specified with BatchUploader.cancelConfirmMessage
* @type {boolean}
* @default true
*/
warnOnCancel?: boolean;

/**
* Confirmation message to show if the user clicks the "Cancel" button and BatchUploader.warnOnCancel
*  is true. Defaults to "You will lose any work you have done on this data. Proceed anyway?"
* @type {string}
* @default "See below"
*/
cancelConfirmMessage?: string;

/**
* Format to assume for user-provided data. Use ImportFormat "auto" for
*  auto-detection.
* @type {ImportFormat}
* @default "csv"
*/
dataFormat?: ImportFormat;

/**
* The delimiter to use when importing character-delimited files. The default is
*  comma (CSV).
* @deprecated * in favor of BatchUploader.uploadDelimiter

* @type {string}
* @default ","
*/
defaultDelimiter?: string;

/**
* The delimiter to use when importing character-delimited files. The default is
*  comma (CSV).
* @type {string}
* @default ","
*/
uploadDelimiter?: string;

/**
* The default character used to quote strings.
* @deprecated * in favor of BatchUploader.uploadQuoteString

* @type {string}
* @default "\""
*/
defaultQuoteString?: string;

/**
* The character used to quote strings. The default is double quotes.
* @type {string}
* @default "\""
*/
uploadQuoteString?: string;

/**
* On server-side, BatchUploader uses DataImport to import uploaded
*  files, specifically APIs taking java.io.Reader parameter.
*  BatchUploader.uploadEncoding setting defines the encoding, which will
*  be used to create a java.io.Reader instance to read data from uploaded
*  files. The default is "UTF-8".
* @type {string}
* @default "UTF-8"
*/
uploadEncoding?: string;

/**
* Controls if server-side API BatchUpload.parseUploadData(...) will convert imported boolean
*  fields values to actual Booleans or will leave them as Strings. The default is true.
*  
*  Default behavior would interpret boolean values by following rules:
*  
*  
* - Accept all caps permutations of "true"/"false", "yes"/"no", and "null"
*  
* - "T" == true, "F" == false
*  
* - "Y" == true, "N" == false
*  
* - "0" is false
*  
* - empty string is null
*  
* - everything else is true
*  
* @type {boolean}
* @default true
*/
autoInterpretBooleans?: boolean;

/**
* If columns were present in the imported data that were discarded because they could not be
*  matched to any DataSource fields, whether these should be displayed to the user, using the
*  BatchUploader.discardedColumnsMessage shown within the BatchUploader.uploadStatusMessages component.
* @type {boolean}
* @default true
*/
displayDiscardedColumns?: boolean;

/**
* Message displayed when columns in the imported file were discarded and
*  BatchUploader.displayDiscardedColumns is true. Within this message, ${discardedColumns} can be
*  used to show a comma separated list of the column names that were discarded (example:
*  "price, saleDate, total").
*  
*  Default message is: "The following columns in your uploaded file were ignored because they
*  did not match any of the expected column names: ${discardedColumns}"
* @type {string}
* @default "..."
*/
discardedColumnsMessage?: string;

/**
* Container for informational messages that are shown when a user attempts an upload.
*  Appears above the BatchUploader.grid.
* @type {HTMLFlow}
* @default null
*/
uploadStatusMessages?: HTMLFlow;

/**
* FileItem for selecting the file to upload.
* @type {FileItem}
* @default null
*/
uploadFileItem?: FileItem;



/**
*  This method assumes a developer has sample data in memory 
*  (for example a csv-formatted string), and invokes the dataSource upload operation directly,
*  bypassing the uploadForm.
*  
*  The sample data will be available on the server via values.pasteData. This
*  usage is expected and understood by the server side batchUpload dataSource.
*  
*  This method is most commonly used in conjunction with 
*  BatchUploader.showUploadForm.
* 
* 
* @param {string} formatted data to upload
*/
uploadData(data:string): void;

/**
*  Notification method fired when the BatchUploader.grid is populated with
*  a new set of data for the user to preview before commit.
*  
*  This notification occurs after the user has uploaded a new data file, the data
*  has been processed on the server, and the preview grid populated with the data
*  ready for committing. Developers may use this notification to examine or modify
*  the data set to be uploaded. The ListGrid.data object will be populated with
*  the array of uploaded records, and standard grid APIs such as 
*  ListGrid.getRowErrors, ListGrid.setEditValue, etc may be used 
*  to interact with this data.
*  
*  Note that developers wishing to manipulate the uploaded data can also do this
*  on the server side when user hits the commit button to submit the data to the
*  BatchUploader.uploadDataSource. This can be achieved by setting the 
*  BatchUploader.uploadOperation to call a custom 
*  DataSource.operationBindings on the target dataSource.
* 
* 
* @param {ListGrid} The BatchUploader.grid autoChild.
*/
previewShown(grid:ListGrid): void;

/**
*  Notification method fired when the BatchUploader.commitButton is clicked.
*  
*  This notification occurs before actually committing data to the server. It allows to
*  make changes to the data after user edits, but before it will be sent to server.
*  
*  Read also BatchUploader.previewShown docs for details how to change data
*  in grid and for possibility to interrupt upload process on the server as well.
* 
* 
* @param {ListGrid} The BatchUploader.grid autoChild.
*/
beforeCommit(grid:ListGrid): void;



}
/**
* Creates a new BatchUploader
* 
* @param typeCheckedProperties {Partial<BatchUploader>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {BatchUploader} The new instance
*/
class BatchUploader{
static create(typeCheckedProperties? : Partial<BatchUploader>, uncheckedProperties?: Object): BatchUploader;







}

/**
* FormItem intended for inserting blurbs of instructional HTML into DynamicForms.
*  
*  Set the defaultValue of this item to the HTML you want to embed in the form.
*/
interface BlurbItem extends FormItem {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "BlurbItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Blurb items show no title by default.
* @type {boolean}
* @default false
*/
showTitle?: boolean;

/**
* By default, texts span all remaining columns
* @type {Integer | string}
* @default "*"
*/
colSpan?: Integer | string;

/**
* Base css style for this item.
* @type {CSSStyleName}
* @default "staticTextItem"
*/
textBoxStyle?: CSSStyleName;

/**
* Should the user be able to select the text in this item?
* @type {boolean}
* @default true
*/
canSelectText?: boolean;

/**
* If true, item contents can wrap. If false, all the contents should appear on a single line.
* @type {boolean}
* @default null
*/
wrap?: boolean;

/**
* If true, text that exceeds the specified size of the form item will be clipped.
* @type {boolean}
* @default false
*/
clipValue?: boolean;





}
/**
* Creates a new BlurbItem
* 
* @param typeCheckedProperties {Partial<BlurbItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {BlurbItem} The new instance
*/
class BlurbItem{
static create(typeCheckedProperties? : Partial<BlurbItem>, uncheckedProperties?: Object): BlurbItem;







}

/**
* EditProxy that handles BlurbItem when editMode is enabled.
*/
interface BlurbItemEditProxy extends TextItemEditProxy {




}
/**
* Creates a new BlurbItemEditProxy
* 
* @param typeCheckedProperties {Partial<BlurbItemEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {BlurbItemEditProxy} The new instance
*/
class BlurbItemEditProxy{
static create(typeCheckedProperties? : Partial<BlurbItemEditProxy>, uncheckedProperties?: Object): BlurbItemEditProxy;







}

/**
* Boolean form item, implemented with customizable checkbox images
*/
interface BooleanItem extends FormItem {




}
/**
* Creates a new BooleanItem
* 
* @param typeCheckedProperties {Partial<BooleanItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {BooleanItem} The new instance
*/
class BooleanItem{
static create(typeCheckedProperties? : Partial<BooleanItem>, uncheckedProperties?: Object): BooleanItem;







}

/**
* The Browser class contains various class attributes that indicate basic properties
*  of the browser and whether certain features are enabled.
*/
interface Browser extends Class {




}
/**
* Creates a new Browser
* 
* @param typeCheckedProperties {Partial<Browser>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Browser} The new instance
*/
class Browser{
static create(typeCheckedProperties? : Partial<Browser>, uncheckedProperties?: Object): Browser;
/**
* Does the browser support both mouse and touch input?
* @type {boolean}
* @default "varies"
*/
static supportsDualInput?: boolean;

/**
* Is the application running on a touch device (e.g. iPhone, iPad, Android device, etc.)?
*  
*  SmartClient's auto-detected value for isTouch can be overridden via
*  Browser.setIsTouch.
* @type {boolean}
* @default "auto-detected based on device"
*/
static isTouch?: boolean;

/**
* Is the application running on a tablet device (e.g. iPad, Nexus 7)?
*  
*  SmartClient can correctly determine whether the device is a tablet in most cases. On any
*  uncommon device for which this variable is incorrect, you can define the isc_isTablet
*  global with the correct value, and SmartClient will use isc_isTablet for
*  Browser.isTablet instead of its own detection logic. Alternatively, you can use
*  Browser.setIsTablet to change this global variable before any components are
*  created.
*  
*  The value of this variable is only meaningful on touch devices.
* @type {boolean}
* @default "auto-detected based on device"
*/
static isTablet?: boolean;

/**
* Is the application running on a handset-sized device, with a typical screen width of around
*  3-4 inches?
*  
*  This typically implies that the application will be working with only 300-400 pixels.
* @type {boolean}
* @default "auto-detected based on device"
*/
static isHandset?: boolean;

/**
* Is the application running in a desktop browser? This is true if Browser.isTablet
*  and Browser.isHandset are both false.
* @type {boolean}
* @default "auto-detected based on device"
*/
static isDesktop?: boolean;

/**
* Whether SmartClient supports the current browser.
*  
*  Note that this flag will only be available on browsers that at least support basic
*  JavaScript.
* @type {boolean}
* @default "auto-detected based on browser"
*/
static isSupported?: boolean;

/**
* Whether the current browser supports CSS3 and whether SmartClient is configured to use
*  CSS3 features (via the setting of window.isc_css3Mode).
*  
*  If isc_css3Mode is "on" then useCSS3 is set to true. If isc_css3Mode is set to
*  "supported", "partialSupport", or is unset, then useCSS3 is set to true only if the browser
*  is a WebKit-based browser, Firefox, IE 9 in standards mode, or IE 10+. If isc_css3Mode is set
*  to "off" then useCSS3 is set to false.
* @type {boolean}
* @default "see below"
*/
static useCSS3?: boolean;

/**
* Controls how agressive components based on the GridRenderer are with respect to
*  redraws and data fetches. Modern browsers can generally handle much more frequent redraws
*  and most server configurations can handle fetching more data more frequently in order to
*  reduce the lag the end user perceives when scrolling through databound grids. Starting with
*  SmartClient 11.0/SmartGWT 6.0, this more aggressive redraw and fetch behavior us the
*  default, but can be reverted to the old behavior if desired - see below.
*  
*  This flag controls the defaults for several other properties (value on left is default for
*  high performance mode, value on right is default when this mode is disabled.
*  
*  
* - ListGrid.dataFetchDelay 1 -> 300
*  
* - ListGrid.drawAheadRatio 2.0 -> 1.3
*  
* - ListGrid.quickDrawAheadRatio 2.0 -> 1.3
*  
* - ListGrid.scrollRedrawDelay 0 -> 75
*  
* - ListGrid.scrollWheelRedrawDelay 0 -> 250
*  
* - ListGrid.touchScrollRedrawDelay 0 -> 300
*  
*  Note: since TreeGrid is a subclass of ListGrid, the above settings
*  also apply to TreeGrids.
*  
*  
* - GridRenderer.drawAheadRatio 2.0 -> 1.3
*  
* - GridRenderer.quickDrawAheadRatio 2.0 -> 1.3
*  
* - GridRenderer.scrollRedrawDelay 0 -> 75
*  
* - GridRenderer.touchScrollRedrawDelay 0 -> 300
*  
*  
*  By default, for all browsers except Android-based Chrome, this flag is set to true, but can
*  be explicitly disabled by setting isc_useHighPerformanceGridTimings=false in a
*  script block before loading SmartClient modules. Turning off high performance timings
*  effectively enables the original SmartClient/SmartGWT behavior prior to the SmartClient
*  11.0/SmartGWT 6.0 release.
* @type {boolean}
* @default "see below"
*/
static useHighPerformanceGridTimings?: boolean;

/**
* Are we in an OpenFin environment?
*  See class OpenFin for ways to call OpenFin methods from within SmartClient.
* @type {boolean}
* @default "varies"
*/
static isOpenFin?: boolean;

/**
* Are the MultiWindow APIs supported and cross-window optimizations enabled? By
*  default this is true in the MultiWindow.isMainWindow if
*  Browser.isOpenFin is loaded, false otherwise. In
*  MultiWindow.open, this property is read-only, and assumes the
*  value from the main window.
*  
*  Note: MultiWindow is currently an experimental feature and not supported
*  except by special arrangement
* @type {boolean}
* @default "varies"
*/
static isMultiWindow?: boolean;





/**
*  Setter for Browser.isTouch to allow this global variable to be changed at runtime.
*  This advanced method is provided to override SmartClient's auto-detection logic, since the
*  framework can only detect touch devices that existed at the time the platform was released.
*  Any change to Browser.isTouch must be made before any component is created;
*  it is an application error to attempt to change isTouch after
*  components have been created.
*  
*  Note that setting Browser.isTouch might affect the values of
*  Browser.isDesktop, Browser.isTablet, and/or Browser.isHandset.
* 
* 
* @param {boolean} new setting for Browser.isTablet.
*/
static setIsTouch(isTouch:boolean): void;

/**
*  Setter for Browser.isTablet to allow this global variable to be changed at runtime.
*  This advanced method is provided to override SmartClient's detection of devices, since the
*  framework can only detect devices that existed at the time the platform was released. Any
*  changes to Browser.isDesktop, Browser.isHandset, or Browser.isTablet
*  must be made before any component is created;
*  it is an application error to attempt to change isDesktop,
*  isHandset, or isTablet after components have been created.
*  
*  Note that setting Browser.isTablet might affect the values of
*  Browser.isDesktop and Browser.isHandset.
* 
* 
* @param {boolean} new setting for Browser.isTablet.
*/
static setIsTablet(isTablet:boolean): void;

/**
*  Setter for Browser.isHandset to allow this global variable to be changed at runtime.
*  This advanced method is provided to override SmartClient's detection of devices, since the
*  framework can only detect devices that existed at the time the platform was released. Any
*  changes to Browser.isDesktop, Browser.isHandset, or Browser.isTablet
*  must be made before any component is created;
*  it is an application error to attempt to change isDesktop,
*  isHandset, or isTablet after components have been created.
*  
*  Note that setting Browser.isHandset might affect the values of
*  Browser.isDesktop and Browser.isTablet.
* 
* 
* @param {boolean} new setting for Browser.isHandset.
*/
static setIsHandset(isHandset:boolean): void;

/**
*  Setter for Browser.isDesktop to allow this global variable to be changed at runtime.
*  This advanced method is provided to override SmartClient's detection of devices, since the
*  framework can only detect devices that existed at the time the platform was released. Any
*  changes to Browser.isDesktop, Browser.isHandset, or Browser.isTablet
*  must be made before any component is created;
*  it is an application error to attempt to change isDesktop,
*  isHandset, or isTablet after components have been created.
*  
*  Note that setting Browser.isDesktop might affect the values of
*  Browser.isHandset and Browser.isTablet.
* 
* 
* @param {boolean} new setting for Browser.isDesktop.
*/
static setIsDesktop(isDesktop:boolean): void;

/**
*  Sets a non-default value for Browser.isMultiWindow, such as enabling it even if
*  Browser.isOpenFin isn't present.
*  
*  Note that this method may only be called from the
*  MultiWindow.isMainWindow, and only once.
* 
* 
* @param {boolean} new setting for Browser.isMultiWindow.
*/
static setIsMultiWindow(isMultiWindow:boolean): void;




}

/**
* Container for a Browser Plugin.
*/
interface BrowserPlugin extends Canvas {




}
/**
* Creates a new BrowserPlugin
* 
* @param typeCheckedProperties {Partial<BrowserPlugin>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {BrowserPlugin} The new instance
*/
class BrowserPlugin{
static create(typeCheckedProperties? : Partial<BrowserPlugin>, uncheckedProperties?: Object): BrowserPlugin;







}

/**
* The Button widget class implements interactive, style-based button widgets.
*/
interface Button extends StatefulCanvas {
/**
* The title HTML to display in this button.
* @type {HTMLString}
* @default "Untitled Button"
*/
title?: HTMLString;

/**
* If set to true, if the StatefulCanvas.title of this button contains the
*  specified Canvas.accessKey, when the title is displayed to the user
*  it will be modified to include HTML to underline the accessKey.
*  Note that this property may cause titles that include HTML (rather than simple strings)
*  to be inappropriately modified, so should be disabled if your title string includes
*  HTML characters.
* @type {boolean}
* @default null
*/
hiliteAccessKey?: boolean;

/**
* If true and the title is clipped, then a hover containing the full title of this button
*  is enabled.
* @type {boolean}
* @default false
*/
showClippedTitleOnHover?: boolean;

/**
* Optional icon to be shown with the button title text. 
*  
*  Specify as the partial URL to an image, relative to the imgDir of this component.
*  A sprited image can be specified using the SCSpriteConfig format.
*  
*  Note that the string "blank" is a valid setting for this attribute and will always 
*  result in the system blank image, with no state suffixes applied. Typically, this 
*  might be used when an iconStyle is also specified and the iconStyle renders the icon via 
*  a stateful background-image or other CSS approach.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* Size in pixels of the icon image.
*  
*  The StatefulCanvas.iconWidth and StatefulCanvas.iconHeight
*  properties can be used to configure width and height separately.
*  
*  Note: When configuring the properties of a StatefulCanvas (or derivative)
*  AutoChild, it is best to set the iconWidth and iconHeight
*  to the same value rather than setting an iconSize. This is because certain
*  skins or customizations thereto might set the iconWidth and iconHeight,
*  making the customization of the AutoChild's iconSize ineffective.
* @type {Integer}
* @default "16"
*/
iconSize?: Integer;

/**
* Width in pixels of the icon image.
*  
*  If unset, defaults to StatefulCanvas.iconSize.
* @type {Integer}
* @default null
*/
iconWidth?: Integer;

/**
* Height in pixels of the icon image.
*  
*  If unset, defaults to StatefulCanvas.iconSize.
* @type {Integer}
* @default null
*/
iconHeight?: Integer;

/**
* Base CSS style applied to the icon image. If set, as the StatefulCanvas changes
*  StatefulCanvas.state and/or is StatefulCanvas.selected,
*  suffixes will be appended to iconStyle to form the className set on the
*  image element.
*  
*  The following table lists out the standard set of suffixes which may be appended:
*  
*  CSS Class AppliedDescription
*  iconStyleDefault CSS style
*  iconStyle+Selected
*    Applied when StatefulCanvas.selected and StatefulCanvas.showSelectedIcon
*    are true.
*  iconStyle+Focused
*    Applied when the component has keyboard focus, if
*    StatefulCanvas.showFocusedIcon is true, and
*    StatefulCanvas.showFocusedAsOver is not true.
*  iconStyle+Over
*    Applied when StatefulCanvas.showRollOverIcon is set to true and either
*    the user rolls over the component or StatefulCanvas.showFocusedAsOver is true
*    and the component has keyboard focus.
*  iconStyle+Down
*    Applied when the user presses the mouse button on the component if
*      StatefulCanvas.showDownIcon is set to true
*  iconStyle+Disabled
*    Applied when the component is Canvas.disabled
*     if StatefulCanvas.showDisabledIcon is true.
*  Combined styles
*  iconStyle+SelectedFocused
*    Combined Selected and focused styling
*  iconStyle+SelectedOver
*    Combined Selected and rollOver styling
*  iconStyle+FocusedOver
*    Combined Focused and rollOver styling
*  iconStyle+SelectedFocusedOver
*    Combined Selected, Focused and rollOver styling
*  iconStyle+SelectedDown
*    Combined Selected and mouse-down styling
*  iconStyle+FocusedDown
*    Combined Focused and mouse-down styling
*  iconStyle+SelectedFocusedDown
*    Combined Selected, Focused and mouse-down styling
*  iconStyle+SelectedDisabled
*    Combined Selected and Disabled styling
*  
*  
*  In addition, if StatefulCanvas.showRTLIcon is true, then in RTL mode, a final
*  "RTL" suffix will be appended.
* @type {CSSStyleName}
* @default null
*/
iconStyle?: CSSStyleName;

/**
* If this button is showing an icon should it appear to the left or right of the title?
*  valid options are "left" and "right".
* @type {string}
* @default "left"
*/
iconOrientation?: string;

/**
* If this button is showing an icon should it be right or left aligned?
* @type {string}
* @default null
*/
iconAlign?: string;

/**
* Pixels between icon and title text.
* @type {Integer}
* @default "6"
*/
iconSpacing?: Integer;

/**
* If using an icon for this button, whether to switch the icon image if the button becomes
*  disabled.
* @type {boolean}
* @default true
*/
showDisabledIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image on mouse rollover.
* @type {boolean}
* @default false
*/
showRollOverIcon?: boolean;

/**
* Specifies the cursor to display when the mouse pointer is over the icon image.
* @type {Cursor}
* @default null
*/
iconCursor?: Cursor;

/**
* Specifies the cursor to display when the mouse pointer is over the icon image and this
*  StatefulCanvas is Canvas.disabled.
*  
*  If not set and the mouse pointer is over the icon image, Button.iconCursor
*  will be used.
* @type {Cursor}
* @default null
*/
disabledIconCursor?: Cursor;

/**
* If using an icon for this button, whether to switch the icon image when the button
*  receives focus.
*  
*  If StatefulCanvas.showFocusedAsOver is true, the "Over" icon will be
*  displayed when the canvas has focus, otherwise a separate "Focused" icon
*  will be displayed
* @type {boolean}
* @default false
*/
showFocusedIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the mouse goes
*  down on the button.
* @type {boolean}
* @default false
*/
showDownIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the button
*  becomes selected.
* @type {boolean}
* @default false
*/
showSelectedIcon?: boolean;

/**
* If true, ignore the specified size of this widget and always size just large
*  enough to accommodate the title. If setWidth() is explicitly called on an
*  autoFit:true button, autoFit will be reset to false.
*  
*  Note that for StretchImgButton instances, autoFit will occur horizontally only, as 
*  unpredictable vertical sizing is likely to distort the media. If you do want vertical 
*  auto-fit, this can be achieved by simply setting a small height, and having 
*  overflow:"visible"
* @type {boolean}
* @default null
*/
autoFit?: boolean;

/**
* Size for this component's horizontal dimension. See Canvas.width for more
*  details.
*  
*  Note that if StatefulCanvas.autoFit is set, this property will be ignored so that the widget
*  is always sized just large enough to accommodate the title.
* @type {number | string}
* @default null
*/
width?: number | string;

/**
* Size for this component's vertical dimension. See Canvas.height for more
*  details.
*  
*  Note that if StatefulCanvas.autoFit is set on non-StretchImgButton instances, this
*  property will be ignored so that the widget is always sized just large enough to
*  accommodate the title.
* @type {number | string}
* @default null
*/
height?: number | string;

/**
* StatefulCanvases are styled by combining Button.baseStyle with State to build
*  a composite css style name. In most cases, statefulCanvas.styleName
*  will have no effect on statefulCanvas styling and should not be used.
*  
*  If the baseStyle is not explicitly specified for a class, the 
*  styleName will be used as a default baseStyle. Other than that, this
*  attribute will be ignored.
* @type {CSSStyleName}
* @default "normal"
*/
styleName?: CSSStyleName;

/**
* Base CSS style className applied to the component.
*  
*  Note that if specified, this property takes precedence over any specified
*  StatefulCanvas.styleName. If unset, the styleName will be used as a 
*  default baseStyle value.
*  
*  As the component changes StatefulCanvas.state and/or is selected, 
*  suffixes will be added to the base style. In some cases more than one suffix will 
*  be appended to reflect a combined state ("Selected" + "Disabled", for example).
*  
*  See StatefulCanvas.getStateSuffix for a description of the default set
*  of suffixes which may be applied to the baseStyle
*  
*  Rotated Titles
*  
*  The Framework doesn't have built-in support for rotating button titles in a fashion
*  similar to FacetChart.rotateLabels. However, you can manually configure
*  a button to render with a rotated title by applying custom CSS via this property.
*  
*  For example, given a button with a height of 120 and a width of 48, if you
*  copied the existing buttonXXX style declarations from skin_styles.css as new,
*  rotatedTitleButtonXXX declarations, and then added the lines:
*  
*    -ms-transform:   translate(-38px,0px) rotate(270deg);
*    -webkit-transform: translate(-38px,0px) rotate(270deg);
*    transform:     translate(-38px,0px) rotate(270deg);
*    overflow: hidden;
*    text-overflow: ellipsis;
*    width:116px;
*  in the declaration section beginning:
*  
*  .rotatedTitleButton,
*  .rotatedTitleButtonSelected,
*  .rotatedTitleButtonSelectedOver,
*  .rotatedTitleButtonSelectedDown,
*  .rotatedTitleButtonSelectedDisabled,
*  .rotatedTitleButtonOver,
*  .rotatedTitleButtonDown,
*  .rotatedTitleButtonDisabled {
*  then applying that style to the button with Canvas.overflow: "clip_h"
*  would yield a vertically-rendered title with overflow via ellipsis as expected, and also
*  wrap with Button.wrap.
*  
*  Note that:
*  
* - The explicit width applied via CSS is needed because rotated
*  elements don't inherit dimensions in their new orientation from the DOM - 
*  the transform/rotation occurs independently of layout.
*  
* - The translation transform required along the x-axis is roughly
*  (width - height) / 2, but may need slight offsetting for optimal centering.
*  
* - We've explicitly avoided describing an approach based on CSS "writing-mode", since
*  support is incomplete and bugs are present in popular browsers such as Firefox and
*  Safari that would prevent it from being used without Framework assistance.
*  
*  Note on css-margins: Developers should be aware that the css "margin" property is unreliable for
*  certain subclasses of StatefulCanvas, including Button. Developers may use 
*  the explicit Canvas.margin property to specify button margins, or for a 
*  button within a layout, consider the layout properties Layout.layoutMargin,
*  Layout.membersMargin
* @type {CSSStyleName}
* @default "button"
*/
baseStyle?: CSSStyleName;

/**
* if defined, iconOnlyBaseStyle is used as the base CSS style className,
*  instead of Button.baseStyle, if Button.canAdaptWidth is set and the 
*  Button.adaptWidthShowIconOnly.
* @type {CSSStyleName}
* @default null
*/
iconOnlyBaseStyle?: CSSStyleName;

/**
* Whether this component is selected. For some components, selection affects appearance.
* @type {boolean}
* @default false
*/
selected?: boolean;

/**
* String identifier for this canvas's mutually exclusive selection group.
* @type {string}
* @default null
*/
radioGroup?: string;

/**
* Behavior on state changes -- BUTTON, RADIO or CHECKBOX
* @type {SelectionType}
* @default "button"
*/
actionType?: SelectionType;

/**
* Current "state" of this widget. The state setting is automatically updated as the 
*  user interacts with the component (see StatefulCanvas.showRollOver, 
*  StatefulCanvas.showDown, StatefulCanvas.showDisabled).
*  
*  StatefulCanvases will have a different appearance based
*  on their current state. 
*  By default this is handled by changing the css className applied to
*  the StatefulCanvas - see StatefulCanvas.baseStyle and 
*  StatefulCanvas.getStateSuffix for a description of how this is done.
*  
*  For Img or StretchImg based subclasses of StatefulCanvas, the 
*  appearance may also be updated by changing the src of the rendered image. See
*  Img.src and StretchImgButton.src for a description of how the URL 
*  is modified to reflect the state of the widget in this case.
* @type {State}
* @default ""
*/
state?: State;

/**
* Should we visibly change state when disabled?
* @type {boolean}
* @default true
*/
showDisabled?: boolean;

/**
* Should we visibly change state when the mouse goes down in this object?
* @type {boolean}
* @default false
*/
showDown?: boolean;

/**
* Should we visibly change state when the canvas receives focus? If
*  StatefulCanvas.showFocusedAsOver is true, then "over"
*  will be used to indicate focus. Otherwise a separate "focused" state
*  will be used.
* @type {boolean}
* @default false
*/
showFocused?: boolean;

/**
* Should we visibly change state when the mouse goes over this object?
* @type {boolean}
* @default false
*/
showRollOver?: boolean;

/**
* Horizontal alignment of this component's title.
* @type {Alignment}
* @default "center"
*/
align?: Alignment;

/**
* Vertical alignment of this component's title.
* @type {VerticalAlignment}
* @default "center"
*/
valign?: VerticalAlignment;

/**
* A boolean indicating whether the button's title should word-wrap, if necessary.
* @type {boolean}
* @default false
*/
wrap?: boolean;

/**
* This flag enables Canvas.canAdaptWidth for the button.
*  
*  If enabled the button will support rendering in a 'collapsed' view if there isn't enough
*  space in a layout to render it at normal size. There are a couple of ways this can be achieved.
*  
*  
* - If Button.adaptWidthShowIconOnly is true and this button shows an icon, the title
*    will be hidden if there isn't enough space to render it, allowing it to shrink to either
*    the rendered icon width, or any specified Canvas.minWidth, whichever is larger.
*  
* - Otherwise, if the button has a specified Canvas.minWidth, and 
*    Button.autoFit is true, autoFit will be temporarily disabled, if there isn't enough
*    room, allowing the title to be clipped
*  
*  In either case the title will show on hover unless an explicit hover has been
*  specified such as by overriding Button.titleHoverHTML.
* @type {boolean}
* @default false
*/
canAdaptWidth?: boolean;

/**
* Clip the contents of the button if necessary.
* @type {Overflow}
* @default "hidden"
*/
overflow?: Overflow;

/**
* If Button.canAdaptWidth is true, and this button has a specified Button.icon, should
*  the title be hidden, allowing the button to shrink down to just show the icon when there isn't
*  enough horizontal space in a layout to show the default sized button?
* @type {boolean}
* @default true
*/
adaptWidthShowIconOnly?: boolean;



/**
*  Return the title - HTML drawn inside the component.
*  
*  Default is to simply return this.title.
* 
*/
getTitle(): HTMLString;

/**
*  Setter for the StatefulCanvas.title.
* 
* @param {HTMLString} the new title HTML.
*/
setTitle(newTitle:HTMLString): void;

/**
*  Changes the orientation of the icon relative to the text of the button.
* 
* 
* @param {string} The new orientation of the icon relative to the text of the button.
*/
setIconOrientation(orientation:string): void;

/**
*  Change the icon being shown next to the title text.
* 
* @param {SCImgURL} URL of new icon
*/
setIcon(icon:SCImgURL): void;

/**
*  Setter method for the StatefulCanvas.autoFit property. Pass in true or false to turn
*  autoFit on or off. When autoFit is set to false, canvas will be resized to
*  it's previously specified size.
* 
* @param {boolean} New autoFit setting.
*/
setAutoFit(autoFit:boolean): void;

/**
*  Sets the base CSS style. As the component changes state and/or is selected, suffixes will be
*  added to the base style.
* 
* @param {CSSStyleName} new base style
*/
setBaseStyle(style:CSSStyleName): void;

/**
*  Select this object.
* 
*/
select(): void;

/**
*  Select this object.
* 
*/
deselect(): void;

/**
*  Find out if this object is selected
* 
*/
isSelected(): boolean;

/**
*  Select this object.
* 
*/
setSelected(): void;

/**
*  Add this widget to the specified mutually exclusive selection group with the ID
*  passed in.
*  Selecting this widget will then deselect any other StatefulCanvases with the same
*  radioGroup ID.
*  StatefulCanvases can belong to only one radioGroup, so this method will remove from 
*  any other radiogroup of which this button is already a member.
* 
* @param {string} - ID of the radiogroup to which this widget should be added
*/
addToRadioGroup(groupID:string): void;

/**
*  Remove this widget from the specified mutually exclusive selection group with the ID
*  passed in.
*  No-op's if this widget is not a member of the groupID passed in.
*  If no groupID is passed in, defaults to removing from whatever radioGroup this widget
*  is a member of.
* 
* @param {string=} - optional radio group ID (to ensure the widget is removed                    from the appropriate group.
*/
removeFromRadioGroup(groupID?:string): void;

/**
*  Update the 'actionType' for this canvas (radio / checkbox / button)
*  If the canvas is currently selected, and the passed in actionType is 'button'
*  this method will deselect the canvas.
* 
*/
setActionType(): void;

/**
*  Return the 'actionType' for this canvas (radio / checkbox / button)
* 
*/
getActionType(): void;

/**
*  Sets the StatefulCanvas.state of this object, changing its appearance.
*  Note: newState cannot be
*  "Disabled"
*  
*  if StatefulCanvas.showDisabled is false.
* 
* 
* @param {State} the new state.
*/
setState(newState:State): void;

/**
*  Enable or disable this object
* 
* @param {boolean} true if this widget is to be disabled
*/
setDisabled(disabled:boolean): void;

/**
*  Return the state of this StatefulCanvas
* 
*/
getState(): State;

/**
*  This property contains the default 'action' for the Button to fire when activated.
* 
*/
action(): void;

/**
*  Is the title of this button clipped?
* 
*/
titleClipped(): boolean;

/**
*  Returns the HTML that is displayed by the default Button.titleHover
*  handler. Return null or an empty string to cancel the hover.
*  
* 
* @param {HTMLString} the HTML that would have been displayed by default
*/
titleHoverHTML(defaultHTML:HTMLString): HTMLString;

/**
*  Set whether the title of this button should be allowed to wrap if too long for the button's
*  specified width.
* 
* 
* @param {boolean} whether to wrap the title
*/
setWrap(newWrap:boolean): void;

/**
*  Sets the (horizontal) alignment of this buttons content.
* 
*/
setAlign(): void;

/**
*  Sets the vertical alignment of this buttons content.
* 
*/
setVAlign(): void;

/**
*  If this button is showing an Button.icon, a separate click handler for the
*  icon may be defined as this.iconClick.
*  Returning false will suppress the standard button click handling code.
* 
*/
iconClick(): boolean;

/**
*  Optional stringMethod to fire when the user hovers over this button and the title is
*  clipped. If Button.showClippedTitleOnHover is true, the default behavior is to
*  show a hover canvas containing the HTML returned by Button.titleHoverHTML.
*  Return false to suppress this default behavior.
* 
*/
titleHover(): boolean;



}
/**
* Creates a new Button
* 
* @param typeCheckedProperties {Partial<Button>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Button} The new instance
*/
class Button{
static create(typeCheckedProperties? : Partial<Button>, uncheckedProperties?: Object): Button;







}

/**
* FormItem for adding a Button to a form.
*/
interface ButtonItem extends CanvasItem {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "ButtonItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* By default buttonItems are sized to match their content (see ButtonItem.autoFit).
*  Specifying an explicit size for the button will disable this behavior.
* @type {number}
* @default null
*/
height?: number;

/**
* Optional baseStyle will be applied to the button.
* @type {CSSStyleName}
* @default null
*/
baseStyle?: CSSStyleName;

/**
* Optional icon image to display on the button for this item. See Button.icon.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* Buttons do not show a title by default.
* @type {boolean}
* @default false
*/
showTitle?: boolean;

/**
* These items are in a row by themselves by default
* @type {boolean}
* @default true
*/
startRow?: boolean;

/**
* These items are in a row by themselves by default
* @type {boolean}
* @default true
*/
endRow?: boolean;

/**
* This item is an autoChild generated Canvas displayed by
*  the ButtonItem and is an instance of Button by defaut, cuztomizeable 
*  via the ButtonItem.buttonConstructor attribute.
* @type {Canvas}
* @default null
*/
button?: Canvas;

/**
* Constructor class for the button.
* @type {Class}
* @default "Button"
*/
buttonConstructor?: Class;

/**
* Should the button auto fit to its title. Maps to Button.autoFit attribute.
*  Note that if an explicit width or height is specified for this item, it will be respected,
*  disabling autoFit behavior
* @type {boolean}
* @default true
*/
autoFit?: boolean;

/**
* Custom Properties to apply to our button item.
* @type {object}
* @default null
*/
buttonProperties?: object;

/**
* If this item is FormItem.getCanEdit, how should this item be displayed
*  to the user? If set, overrides the form-level DynamicForm.readOnlyDisplay default.
* @type {ReadOnlyDisplayAppearance}
* @default "disabled"
*/
readOnlyDisplay?: ReadOnlyDisplayAppearance;

/**
* Criteria to be evaluated to determine whether this item should be enabled. This
*  property is incompatible with FormItem.readOnlyWhen and any setting you provide for
*  the latter will be ignored if this property is set.
*  
*  Criteria are evaluated against the ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','form\'s current values')} as
*  well as the current Canvas.ruleScope. Criteria are re-evaluated
*  every time form values or the rule context changes, whether by end user action or by
*  programmatic calls.
*  
*  A basic criteria uses textMatchStyle:"exact". When specified in
*  componentXML this property allows
*  xmlCriteriaShorthand for defining criteria.
*  
*  Note: A ButtonItem using enableWhen must have a FormItem.name defined.
*  CanvasItem.shouldSaveValue can be set to false to prevent the field from storing
*  its value into the form's values.
* @type {AdvancedCriteria}
* @default null
*/
enableWhen?: AdvancedCriteria;

/**
* This property governs whether StatefulCanvas.showFocusedAsOver is true
*  on the automatically created Button for this item.
* @type {boolean}
* @default null
*/
showFocusedAsOver?: boolean;

/**
* The (horizontal) alignment of this button's title.
* @type {Alignment}
* @default "center"
*/
buttonTitleAlign?: Alignment;



/**
*  Called when a ButtonItem is clicked on.
* 
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the form item itself (also available as "this")
*/
click(form:DynamicForm, item:FormItem): boolean;

/**
*  Set the title.
* 
* @param {string} new title
*/
setTitle(newTitle:string): void;

/**
*  Sets the (horizontal) alignment of this button's title.
* 
* @param {Alignment} new title alignment
*/
setButtonTitleAlign(alignment:Alignment): void;

/**
*  Sets showFocusedAsOver.
* 
* @param {boolean} */
setShowFocusedAsOver(showFocusedAsOver:boolean): void;



}
/**
* Creates a new ButtonItem
* 
* @param typeCheckedProperties {Partial<ButtonItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ButtonItem} The new instance
*/
class ButtonItem{
static create(typeCheckedProperties? : Partial<ButtonItem>, uncheckedProperties?: Object): ButtonItem;







}

/**
* EditProxy that handles ButtonItem when editMode is enabled.
*/
interface ButtonItemEditProxy extends FormItemEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's title or name.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's defaultValue.
* 
* 
* @param {string} the new component defaultValue
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new ButtonItemEditProxy
* 
* @param typeCheckedProperties {Partial<ButtonItemEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ButtonItemEditProxy} The new instance
*/
class ButtonItemEditProxy{
static create(typeCheckedProperties? : Partial<ButtonItemEditProxy>, uncheckedProperties?: Object): ButtonItemEditProxy;







}

/**
* A Layout-based component for editing groups of 
*  StyleSetting. Changes are reflected in a preview canvas 
*  and can be retrieved as a block of 
*  CSSEditor.getCSSProperties or as a string of 
*  CSSEditor.getCSSText.
*  
*  Note: this feature requires
*  SmartClient Enterprise.
*/
interface CSSEditor extends VLayout {
/**
* The set of values to apply to editors.
* @type {object}
* @default null
*/
values?: object;

/**
* The set of StyleGroups to show for editing.
* @type {Array<Partial<StyleGroup>>}
* @default null
*/
groups?: Array<Partial<StyleGroup>>;



/**
*  Notification fired when the okButton is clicked.
* 
* @param {object} ordinary JS object with the final settings
*/
editComplete(settings:object): void;

/**
*  Notification fired when the cancelButton is clicked.
* 
*/
editCancelled(): void;

/**
*  Set the list of StyleGroup to display for editing. The 
*  parameter can be an array of valid StyleGroups (or objects with at least a "name" 
*  property), or a list of the names of predefined groups.
* 
* @param {Array<Partial<object>> | Array<Partial<string>>} array of StyleGroup objects or names
*/
setGroups(groups:Array<Partial<object>> | Array<Partial<string>>): void;

/**
*  Clear the current StyleGroup from the editor.
* 
*/
clearGroups(): void;

/**
*  Notification fired when values in the editor are changed. The latest values are passed
*  in the values parameter. You can use 
*  CSSEditor.getCSSText to retrieve the values as a single piece
*  of CSSText.
* 
* @param {object} ordinary JS object with properties representing the settings              following an edit
*/
valuesChanged(values:object): void;

/**
*  Returns the values being edited as a block of CSS properties.
* 
*/
getCSSProperties(): object;

/**
*  Returns the values being edited as a single string of CSSText.
* 
*/
getCSSText(): CSSText;



}
/**
* Creates a new CSSEditor
* 
* @param typeCheckedProperties {Partial<CSSEditor>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {CSSEditor} The new instance
*/
class CSSEditor{
static create(typeCheckedProperties? : Partial<CSSEditor>, uncheckedProperties?: Object): CSSEditor;







}

/**
* The Calendar component provides several different ways for a user to view and
*  edit a set of events. Note that the standard Calendar module must be loaded to make use of 
*  the Calendar class.
*  
*  CalendarEvents
*  
*  Events are represented as ordinary JavaScript Objects (see CalendarEvent). 
*  The Calendar expects to be able to read and write a basic set of properties 
*  on events: name, startDate, endDate, description, etc, which can be stored 
*  under configurable property names (see eg Calendar.startDateField).
*  
*  Much like a ListGrid manages it's ListGridRecords, the Calendar can 
*  either be passed an ordinary Array of CalendarEvents or can fetch data from a 
*  DataSource. When this is the case, if the DataSource 
*  does not contain fields with the configured property names, an attempt is made to 
*  auto-detect likely-looking fields from those that are present. To see logs indicating that 
*  this has happened, switch default logging preferences to INFO level in the Developer Console.
*  
*  If the calendar is bound to a DataSource, event changes by user action or by 
*  calling methods will be saved to the DataSource.
*  
*  Navigation
*  
*  The calendar supports a number of views by default: Calendar.dayView,
*  Calendar.weekView, Calendar.monthView and 
*  Calendar.timelineView. The user can navigate using back and forward 
*  buttons or via an attached Calendar.dateChooser.
*  
*  Event Manipulation
*  
*  Events can be created by clicking directly onto one of the views, or via the 
*  Calendar.addEventButton button. In the day, week and timeline views, the user may 
*  click and drag to create an event of a specific duration.
*  
*  Creating an event via click or click and drag pops up the
*  Calendar.eventDialog, which provides a simple form for 
*  quick event entry (for normal events, only the description is required by default - for 
*  events that are shown in a Calendar.lanes, that field is also required).
*  
*  A separate editor called the Calendar.eventEditor provides 
*  an interface for editing all possible properties of an event, including custom 
*  properties. The EventEditor is used whenever a pre-existing event is being 
*  edited, and can also be invoked
*  by the user wherever the simpler EventDialog appears.
*  
*  Events can also be programmatically Calendar.addCalendarEvent,
*  Calendar.removeEvent, or Calendar.updateCalendarEvent.
*/
interface Calendar extends Canvas, DataBoundComponent {
/**
* The date for which events are displayed in the day, week, and month tabs of 
*  the calendar. Default is today.
* @type {Date}
* @default 'Today'
*/
chosenDate?: Date;

/**
* The numeric day (0-6, Sunday-Saturday) which the calendar should consider as the first 
*  day of the week in multi-day views, and in the popup Calendar.dateChooser.
*  
*  If unset, the default is taken from the current locale.
* @type {number}
* @default null
*/
firstDayOfWeek?: number;

/**
* The base name for the CSS class applied to the grid cells of the day and week views
*  of the calendar. This style will have "Dark", "Over", "Selected", or "Disabled"
*  appended to it according to the state of the cell.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined with the
*  base style to generate stateful cell styles.
* @type {CSSStyleName}
* @default "calendar"
*/
baseStyle?: CSSStyleName;

/**
* The base name for the CSS class applied to the day headers of the month view.
*  This style will have "Dark", "Over", "Selected", or "Disabled"
*  appended to it according to the state of the cell.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined with the
*  base style to generate stateful cell styles.
* @type {CSSStyleName}
* @default "calMonthDayHeader"
*/
dayHeaderBaseStyle?: CSSStyleName;

/**
* The base name for the CSS class applied to the day body of the month view
*  of the calendar. This style will have "Dark", "Over", "Selected", or "Disabled"
*  appended to it according to the state of the cell.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined with the
*  base style to generate stateful cell styles.
* @type {CSSStyleName}
* @default "calMonthDayBody"
*/
dayBodyBaseStyle?: CSSStyleName;

/**
* The base name for the CSS class applied to the day headers of the month view.
*  This style will have "Dark", "Over", "Selected", or "Disabled"
*  appended to it according to the state of the cell.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined with the
*  base style to generate stateful cell styles.
* @type {CSSStyleName}
* @default "calMonthDayHeader"
*/
otherDayHeaderBaseStyle?: CSSStyleName;

/**
* The base name for the CSS class applied to the day body of the month view
*  of the calendar. This style will have "Dark", "Over", "Selected", or "Disabled"
*  appended to it according to the state of the cell.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined with the
*  base style to generate stateful cell styles.
* @type {CSSStyleName}
* @default "calMonthOtherDayBody"
*/
otherDayBodyBaseStyle?: CSSStyleName;

/**
* The CSS style applied to both the header and body of days from other months in the
*  Calendar.monthView, when Calendar.showOtherDays is false.
* @type {CSSStyleName}
* @default "calMonthOtherDayBlank"
*/
otherDayBlankStyle?: CSSStyleName;

/**
* In the Calendar.monthView when Calendar.showDayHeaders is true, this is the minimum
*  height applied to a day cell and its header combined.
*  
*  If showDayHeaders is false, this attribute has no effect - the minimum height 
*  of day cells is either an equal share of the available height, or the rendered height of the
*  cell's HTML content, whichever is greater. If the latter, a vertical scrollbar is shown.
* @type {Integer}
* @default "80"
*/
minimumDayHeight?: Integer;

/**
* Whether to show the Day view.
* @type {boolean}
* @default true
*/
showDayView?: boolean;

/**
* Whether to show the Week view.
* @type {boolean}
* @default true
*/
showWeekView?: boolean;

/**
* Whether to show the Month view.
* @type {boolean}
* @default true
*/
showMonthView?: boolean;

/**
* The base name for the CSS class applied to a cell that is selected via a mouse drag.
* @type {CSSStyleName}
* @default "calendarCellSelected"
*/
selectedCellStyle?: CSSStyleName;

/**
* The base name for the CSS class applied to event windows within calendars.
*  This style will have "Header", "HeaderLabel", and "Body" appended to it, according to 
*  which part of the event window is being styled. For example, to style the header, define
*  a CSS class called 'eventWindowHeader'.
* @deprecated * in favor of Calendar.eventStyleName

* @type {CSSStyleName}
* @default null
*/
eventWindowStyle?: CSSStyleName;

/**
* The base name for the CSS class applied to Calendar.eventCanvas when they're
*  rendered in calendar views.
*  This style will have "Header" and "Body" appended to it, according to 
*  which part of the event window is being styled. For example, to style the header, define
*  a CSS class called 'eventWindowHeader'.
* @type {CSSStyleName}
* @default "eventWindow"
*/
eventStyleName?: CSSStyleName;

/**
* The base name for the CSS class applied to the links rendered by Calendar.getDayBodyHTML.
*  
*  These links are rendered as plain HTML links using A elements, and the CSS style in the 
*  provided skins references the pseudo-classes :link, :visited, :active, :hover. 
*  
*  Even though it goes against the general policy of not exposing the HTML structures SC writes out
*  and not relying on them for styling, applying style to these particular selectors is acceptable,
*  as we're unlikely to use any other kind of HTML structure than a link.
* @type {CSSStyleName}
* @default "calMonthEventLink"
*/
calMonthEventLinkStyle?: CSSStyleName;

/**
* If Calendar.showWorkday is set, this is the style used for cells that are within the workday, 
*  as defined by Calendar.workdayStart and Calendar.workdayEnd, or by a date-specific range 
*  provided in Calendar.getWorkdayStart and Calendar.getWorkdayEnd implementations.
* @type {CSSStyleName}
* @default "calendarWorkday"
*/
workdayBaseStyle?: CSSStyleName;

/**
* When using Calendar.showWorkday:true, workdayStart and workdayEnd
*  specify the time of day when the workday starts and ends, specified as a
*  String acceptable to Time.parseInput.
*  
*  Both start and end time must fall on a 30 minute increment (eg 9:30, but not 9:45).
*  
*  The hours of the workday can be customized for particular dates by providing implementations
*  of Calendar.getWorkdayStart and Calendar.getWorkdayEnd.
* @type {Time}
* @default "9:00am"
*/
workdayStart?: Time;

/**
* When using Calendar.showWorkday:true, workdayStart and workdayEnd
*  specify the time of day when the workday starts and ends, specified as a
*  String acceptable to Time.parseInput.
*  
*  Both start and end time must fall on a 30 minute increment (eg 9:30, but not 9:45).
*  
*  The hours of the workday can be customized for particular dates by providing implementations
*  of Calendar.getWorkdayStart and Calendar.getWorkdayEnd.
* @type {Time}
* @default "5:00pm"
*/
workdayEnd?: Time;

/**
* When set to true, this setting enables various features related to cells that fall within 
*  the workday (as defined by Calendar.workdayStart and Calendar.workdayEnd) in vertical calendar
*  views (Calendar.dayView and Calendar.weekView). Workday cells can be 
*  Calendar.workdayBaseStyle and Calendar.sizeToWorkday,
*  and users can be prevented from scrolling the calendar beyond the 
*  Calendar.limitToWorkday.
*  
*  The hours of the workday can be customized for particular dates by providing implementations
*  of Calendar.getWorkdayStart and Calendar.getWorkdayEnd.
* @type {boolean}
* @default false
*/
showWorkday?: boolean;

/**
* When Calendar.showWorkday is true, applies the Calendar.workdayBaseStyle 
*  to cells that fall within the workday (as defined by Calendar.workdayStart and Calendar.workdayEnd),
*  in both the Calendar.dayView and Calendar.weekView.
*  
*  The hours of the workday can be customized for particular dates by providing implementations
*  of Calendar.getWorkdayStart and Calendar.getWorkdayEnd.
* @type {boolean}
* @default true
*/
styleWorkday?: boolean;

/**
* Array of days that are considered workdays when Calendar.showWorkday is true.
*  Has no effect if Calendar.dateIsWorkday is implemented.
* @type {Array<any>}
* @default "[1,2,3,4,5]"
*/
workdays?: Array<any>;

/**
* If set, and Calendar.showWorkday is true, automatically scrolls the 
*  Calendar.dayView and Calendar.weekView views to the start of the 
*  Calendar.workdayStart when the calendar is first displayed and whenever the user 
*  changes to a different day or week.
* @type {boolean}
* @default false
*/
scrollToWorkday?: boolean;

/**
* When Calendar.showWorkday is true, attempt to resize rows in the 
*  day and week views so that the Calendar.workdayStart fill the visible 
*  viewport-height, and the whole workday is visible without scrolling. If the Calendar is 
*  resized, the row-size is recalculated to keep the workday hours visible.
*  
*  Note that row-heights will not shrink below the Calendar.minRowHeight, so the entire workday 
*  may not be visible without scrolling if the workday is long or the viewport-height is 
*  insufficient.
* @type {boolean}
* @default false
*/
sizeToWorkday?: boolean;

/**
* When Calendar.showWorkday is true, this attribute prevents the user from 
*  scrolling vertical views beyond the specified workday Calendar.workdayStart and 
*  Calendar.workdayEnd hours.
* @type {boolean}
* @default false
*/
limitToWorkday?: boolean;

/**
* The number of minutes per row in Calendar.dayView and 
*  Calendar.weekView views. The default of 30 minutes shows two rows per hour.
*  Note that this value must divide into 60.
* @type {Integer}
* @default "30"
*/
minutesPerRow?: Integer;

/**
* A minute value that indicates which rows should show times in vertical views, like 
*  Calendar.dayView and Calendar.weekView. The default of 60 minutes 
*  shows titles on the first row of each hour. The value provided must be a multiple of 
*  Calendar.minutesPerRow and be no larger than 60.
* @type {Integer}
* @default "60"
*/
rowTitleFrequency?: Integer;

/**
* The name of the name field on a CalendarEvent.
* @type {string}
* @default "name"
*/
nameField?: string;

/**
* The name of the description field on a CalendarEvent.
* @type {string}
* @default "description"
*/
descriptionField?: string;

/**
* The name of the start date field on a CalendarEvent.
* @type {string}
* @default "startDate"
*/
startDateField?: string;

/**
* The name of the end date field on a CalendarEvent.
* @type {string}
* @default "endDate"
*/
endDateField?: string;

/**
* The name of the CalendarEvent.duration field on a CalendarEvent.
* @type {string}
* @default "duration"
*/
durationField?: string;

/**
* The name of the CalendarEvent.durationUnit field on a 
*  CalendarEvent.
* @type {string}
* @default "durationUnit"
*/
durationUnitField?: string;

/**
* The name of the field which will determine the Calendar.lanes in which a given 
*  event will be displayed in Timelines and in the Calendar.dayView, if 
*  Calendar.showDayLanes is true.
* @type {string}
* @default "lane"
*/
laneNameField?: string;

/**
* When set to true, hides any Calendar.lanes that doesn't have any active events
*  in the current dataset.
* @type {boolean}
* @default null
*/
hideUnusedLanes?: boolean;

/**
* When set to true, causes Calendar.lanes to be sub-divided according to their
*  set of Lane.sublanes.
* @type {boolean}
* @default null
*/
useSublanes?: boolean;

/**
* The name of the field which will determine the Lane.sublanes in which this 
*  event will be displayed, within its parent Lane, in Timelines and in the 
*  Calendar.dayView, if Calendar.showDayLanes is true.
* @type {string}
* @default "sublane"
*/
sublaneNameField?: string;

/**
* The name of the leading date field for each event. When this attribute and 
*  Calendar.trailingDateField are present in the data, a line extends out from the event showing
*  the extent of the leading and trailing dates - useful for visualizing a pipeline of events 
*  where some can be moved a certain amount without affecting others.
* @type {string}
* @default "leadingDate"
*/
leadingDateField?: string;

/**
* The name of the trailing date field for each event. When this attribute and 
*  Calendar.leadingDateField are present in the data, a line extends out from the event showing 
*  the extent of the leading and trailing dates - useful for visualizing a pipeline of events 
*  where some can be moved a certain amount without affecting others.
* @type {string}
* @default "trailingDate"
*/
trailingDateField?: string;

/**
* When set to false, hides the frozen Label-Column in vertical CalendarViews.
*  
*  Always false when Calendar.showColumnLayouts is true.
* @type {boolean}
* @default true
*/
showLabelColumn?: boolean;

/**
* The name of the field used to override Calendar.eventWindowStyle for an individual
*  CalendarEvent. See CalendarEvent.eventWindowStyle.
* @deprecated * in favor of Calendar.eventStyleNameField

* @type {string}
* @default "eventWindowStyle"
*/
eventWindowStyleField?: string;

/**
* The name of the field used to override Calendar.eventStyleName for an individual
*  CalendarEvent.styleName.
* @type {string}
* @default "styleName"
*/
eventStyleNameField?: string;

/**
* Name of the field on each CalendarEvent that determines whether it can be edited in
*  the Calendar.eventEditor. Note that an event with canEdit 
*  set to true can also have Calendar.canDragEventField or
*  Calendar.canResizeEventField set to false, 
*  which would still allow editing, but not via drag operations.
* @type {string}
* @default "canEdit"
*/
canEditField?: string;

/**
* Name of the field on each CalendarEvent that determines whether that event can be 
*  moved between lanes.
* @type {string}
* @default "canEditLane"
*/
canEditLaneField?: string;

/**
* Name of the field on each CalendarEvent that determines whether that event can be 
*  moved between individual Lane.sublanes in a Lane.
* @type {string}
* @default "canEditSublane"
*/
canEditSublaneField?: string;

/**
* Name of the field on each CalendarEvent that determines whether an event shows a
*  remove button.
* @type {string}
* @default "canRemove"
*/
canRemoveField?: string;

/**
* Name of the field on each CalendarEvent that determines whether an EventCanvas
*  can be moved or resized by dragging with the mouse. Note that 
*  Calendar.canEditEvents must be true for dragging to be allowed.
* @type {string}
* @default "canDrag"
*/
canDragEventField?: string;

/**
* Name of the field on each CalendarEvent that determines whether an event can be 
*  resized by dragging.
* @type {string}
* @default "canResize"
*/
canResizeEventField?: string;

/**
* When set to true, allows events to be managed by duration, as well as by end date. Values
*  can be set for CalendarEvent.duration and 
*  CalendarEvent.durationUnit on each event, and are then maintained,
*  instead of the end date, when alterations are made to the event via editors or dragging 
*  with the mouse.
* @type {boolean}
* @default null
*/
allowDurationEvents?: boolean;

/**
* The pixel space to leave between events and the edges of the Calendar.lanes or 
*  Lane.sublanes they appear in. Only applicable to 
*  Calendar.timelineView and to Calendar.dayView showing
*  Calendar.showDayLanes.
* @type {Integer}
* @default "0"
*/
laneEventPadding?: Integer;

/**
* The number of pixels to leave to the right of events so overlapping events can still be 
*  added using the mouse.
* @type {Integer}
* @default "10"
*/
eventDragGap?: Integer;

/**
* Augments the width of week event windows slightly to avoid duplicate adjacent borders
*  between events.
* @type {boolean}
* @default false
*/
weekEventBorderOverlap?: boolean;

/**
* Configures the levels of HeaderLevel shown above the event area, and 
*  their time units.
*  
*  Header levels are provided from the top down, so the first header level should be the largest 
*  time unit and the last one the smallest. The smallest is then used for the actual 
*  field-headers.
* @type {Array<Partial<HeaderLevel>>}
* @default null
*/
headerLevels?: Array<Partial<HeaderLevel>>;

/**
* The number of minutes that determines the positions to which events will snap when rendered,
*  and when moved or resized with the mouse.
*  
*  If unset (the default), all views will snap to each cell boundary; 30 minutes in a default
*  vertical view, or one Calendar.timelineGranularity in a default Timeline.
*  
*  If set to zero, views will snap to one of a set of known "sensible" defaults: for a default 
*  vertical, this will be 5 minutes. For timelines, the eventSnapGap is automatic depending on
*  the current Calendar.timelineGranularity. If Calendar.timelineUnitsPerColumn 
*  is greater than 1, the snapGap is set to one unit of the current granularity. So, a 
*  cell-resolution of 15 minutes would snap to every minute, assuming there are at least 15 
*  pixels per column. Otherwise, the snapGap is either 15 minutes, 1 hour, one day or one 
*  month, depending on granularity.
*  
*  If any other value is specified, it is used where possible.
*  
*  If the specified or calculated value is less than the time covered by a single pixel in the 
*  current view, then it can't be represented. In this case, it is rounded up to the lowest of 
*  a set of "sensible" time-spans that can be represented: one of 
*  [1, 5, 10, 15, 20, 30, 60, 120, 240, 360, 480, 720, 1440].
*  
*  For example - a Timeline showing "day" columns cannot support an eventSnapGap of 1 minute,
*  unless each column is at least 1440 pixels wide - if the columns were only 150px wide, then 
*  each pixel would represent around 9.6 minutes, which would result in unpleasant and unexpected 
*  time-offsets when dragging events. So, the calculated eventSnapGap will be rounded 
*  up to the nearest "sensible" time-span - in this case, 10 minutes. If the columns were only 
*  60px wide, it would be 30 minutes.
* @type {Integer}
* @default null
*/
eventSnapGap?: Integer;

/**
* Determines whether the quick event dialog is displayed when a time is clicked. If this is
*  false, the full event editor is displayed.
* @type {boolean}
* @default true
*/
showQuickEventDialog?: boolean;

/**
* The set of fields for the Calendar.eventEditor.
*  
*  The default set of fields are: 
*  
*   {name: "startHours", title: "From",   editorType: "SelectItem", type: "integer", width: 60},
*   {name: "startMinutes", showTitle: false, editorType: "SelectItem", type: "integer", width: 60},
*   {name: "startAMPM", showTitle: false, type: "select", width: 60},
*   {name: "invalidDate", type: "blurb", colSpan: 4, visible: false}
*   {name: "endHours", title: "To",    editorType: "SelectItem", type: "integer", width: 60},
*   {name: "endMinutes", showTitle: false, editorType: "SelectItem", type: "integer", width: 60},
*   {name: "endAMPM", showTitle: false, type: "select", width: 60},
*   {name: "name", title: "Name", type: "text", colSpan: 4},
*   {name: "description", title: "Description", type: "textArea", colSpan: 4, height: 50}
*  
*  See the Customized Binding example below for more information on altering default datasource 
*  fields within forms.
* @type {Array<Partial<FormItem>>}
* @default "see below"
*/
eventEditorFields?: Array<Partial<FormItem>>;

/**
* The title for the Date-field in the 
*  Calendar.showEventEditor that allows for changing the logical start-date 
*  of an event, along with its start and end times, when editing events in the 
*  Calendar.dayView and Calendar.weekView views.
* @type {HTMLString}
* @default "Date"
*/
eventEditorDateFieldTitle?: HTMLString;

/**
* The set of fields for the Calendar.eventDialog. 
*  
*  The default set of fields are: 
*  
*   {name: "name", title: "Event Name", type: nameType, width: 250 },
*   {name: "save", title: "Save Event", editorType: "SubmitItem", endRow: false},
*   {name: "details", title: "Edit Details", type: "button", startRow: false}
*  
*  See the Customized Binding example below for more information on altering default datasource 
*  fields within forms.
* @type {Array<Partial<FormItem>>}
* @default "see below"
*/
eventDialogFields?: Array<Partial<FormItem>>;

/**
* If true, users can create new events.
* @type {boolean}
* @default true
*/
canCreateEvents?: boolean;

/**
* If true, users can edit existing events.
* @type {boolean}
* @default true
*/
canEditEvents?: boolean;

/**
* If true, users can delete existing events. Defaults to Calendar.canEditEvents.
* @deprecated * in favor of Calendar.canRemoveEvents

* @type {boolean}
* @default null
*/
canDeleteEvents?: boolean;

/**
* If true, users can remove existing events. Defaults to Calendar.canEditEvents.
* @type {boolean}
* @default true
*/
canRemoveEvents?: boolean;

/**
* A boolean value controlling whether users can drag-reposition events. By default, this is 
*  false for Touch devices, where drag gestures scroll the view, and true otherwise.
*  
*  Only has an effect when Calendar.canEditEvents is true.
* @type {boolean}
* @default null
*/
canDragEvents?: boolean;

/**
* Can CalendarEvent be resized by dragging appropriate edges of the 
*  EventCanvas.verticalResize? Only has an effect when both
*  Calendar.canEditEvents and Calendar.canDragEvents
*  are true. Set this attribute to false to disallow drag-resizing.
*  
*  Always false when Calendar.showColumnLayouts is true.
* @type {boolean}
* @default true
*/
canResizeEvents?: boolean;

/**
* Determines whether the Calendar.dateChooser is displayed.
* @type {boolean}
* @default true
*/
showDateChooser?: boolean;

/**
* If true, weekend days appear in a disabled style and events cannot be created on weekends.
*  Which days are considered weekends is controlled by Calendar.weekendDays.
* @type {boolean}
* @default true
*/
disableWeekends?: boolean;

/**
* An array of integer day-numbers that should be considered to be weekend days by this
*  Calendar instance. If unset, defaults to the set of days indicated 
*  DateUtil.weekendDays.
* @type {Array<Partial<Integer>>}
* @default null
*/
weekendDays?: Array<Partial<Integer>>;

/**
* Suppresses the display of weekend days in the Calendar.weekView, 
*  Calendar.monthView and Calendar.timelineView views, and 
*  disallows the creation of events on weekends. Which days are considered weekends is 
*  controlled by Calendar.weekendDays.
* @type {boolean}
* @default true
*/
showWeekends?: boolean;

/**
* If true, the default, show a header cell for each day cell in the 
*  Calendar.monthView, with both cells having a minimum combined height of 
*  Calendar.minimumDayHeight. If false, the header cells will not be shown, and the value 
*  of Calendar.minimumDayHeight is ignored. This causes the available vertical space in month
*  views to be shared equally between day cells, such that no vertical scrollbar is required
*  unless the HTML in the cells renders them taller than will fit.
* @type {boolean}
* @default true
*/
showDayHeaders?: boolean;

/**
* If set to true, in the Calendar.monthView, days that fall in an adjacent month are
*  still shown with a header and body area, and are interactive. Otherwise days from other 
*  months are rendered in the Calendar.otherDayBlankStyle and are non-interactive.
* @type {boolean}
* @default true
*/
showOtherDays?: boolean;

/**
* When Calendar.showOtherDays is true, this attribute determines whether the month-view 
*  should change month when cells representing days in the previous or following month are 
*  clicked.
* @type {boolean}
* @default true
*/
otherDayClickNavigation?: boolean;

/**
* When Calendar.showOtherDays is true, determines whether clicking in a Month-view 
*  body-cell that represents a date outside the selected month will open the 
*  Calendar.eventEditor at the cell's location.
*  
*  Note that, when Calendar.otherDayClickNavigation is also true, the 
*  Month-view first switches to the month of the clicked date before displaying the 
*  event-editor. This causes the editor to open in a different cell than the one clicked and 
*  may be unexpected to users.
* @type {boolean}
* @default false
*/
canCreateOtherDayEvents?: boolean;

/**
* When true, shows the current Calendar.chosenDate in a selected style
*  in the Calendar.monthView. Has no effect in other views.
* @type {boolean}
* @default true
*/
selectChosenDate?: boolean;

/**
* If set to true, enables the auto-arrangement of events that share time in the calendar. The
*  default is true.
* @type {boolean}
* @default true
*/
eventAutoArrange?: boolean;

/**
* If set to true, clicking an event will bring it to the front of the zorder.
* @type {boolean}
* @default null
*/
bringEventsToFront?: boolean;

/**
* When Calendar.eventAutoArrange is true, setting eventOverlap to true causes events that 
*  share timeslots to overlap each other by a percentage of their width, specified by 
*  Calendar.eventOverlapPercent. The default is true for Calendars and false for 
*  Timeline.
* @type {boolean}
* @default true
*/
eventOverlap?: boolean;

/**
* The size of the overlap, presented as a percentage of the width of events sharing timeslots.
* @type {number}
* @default "10"
*/
eventOverlapPercent?: number;

/**
* When set to true, events that start at the same time will not overlap each other to prevent 
*  events having their close button hidden.
* @type {boolean}
* @default false
*/
eventOverlapIdenticalStartTimes?: boolean;

/**
* A boolean value controlling whether the Calendar shows tabs for available calendar views.
*  By default, this is true for handsets and false otherwise.
* @type {boolean}
* @default false
*/
minimalUI?: boolean;

/**
* A boolean value controlling whether new events of varying length can be created by dragging
*  the cursor. By default, this is false for Touch devices and true otherwise.
* @type {boolean}
* @default null
*/
canDragCreateEvents?: boolean;

/**
* TabSet for managing calendar views when multiple views are available (eg,
*  Calendar.dayView and Calendar.monthView).
* @type {TabSet}
* @default null
*/
mainView?: TabSet;

/**
* CalendarView used to display events that pertain to a given day.
* @type {CalendarView}
* @default null
*/
dayView?: CalendarView;

/**
* CalendarView used to display events that pertain to a given week.
* @type {CalendarView}
* @default null
*/
weekView?: CalendarView;

/**
* CalendarView used to display events that pertain to a given month.
* @type {CalendarView}
* @default null
*/
monthView?: CalendarView;

/**
* DateChooser used to select the date for which events will be displayed.
* @type {DateChooser}
* @default null
*/
dateChooser?: DateChooser;

/**
* When showing a Calendar.timelineView, or a Calendar.dayView when 
*  Calendar.showDayLanes is true, whether to make lane boundaries more obvious by showing
*  alternate lanes in a different color.
* @type {boolean}
* @default null
*/
alternateLaneStyles?: boolean;

/**
* When set to true, causes Calendar.timelineView to highlight the Lane under the 
*  mouse with the "Over" style.
* @type {boolean}
* @default null
*/
showLaneRollOver?: boolean;

/**
* A List of CalendarEvent objects, specifying the data to be used to populate the
*  calendar. 
*  
*  This property will typically not be explicitly specified for databound Calendars, where
*  the data is returned from the server via databound component methods such as
*  Calendar.fetchData. In this case the data objects will be set to a 
*  ResultSet rather than a simple array.
* @type {Array<Partial<CalendarEvent>>}
* @default null
*/
data?: Array<Partial<CalendarEvent>>;

/**
* The DataSource that this component should bind to for default fields and for performing
*  DSRequest.
*  
*  Can be specified as either a DataSource instance or the String ID of a DataSource.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* If true, when this component is first drawn, automatically call this.fetchData().
*  Criteria for this fetch may be picked up from Calendar.initialCriteria, and textMatchStyle may
*  be specified via ListGrid.autoFetchTextMatchStyle.
*  
*  NOTE: if autoFetchData is set, calling
*  ListGrid.fetchData before draw will cause two requests to be issued, one from the manual
*  call to fetchData() and one from the autoFetchData setting. The second request will use
*  only Calendar.initialCriteria and not any other criteria or settings from the first request.
*  Generally, turn off autoFetchData if you are going to manually call ListGrid.fetchData at any time.
* 
*  Note: If you are using saved searches - either via SavedSearchItem or ListGrid.saveDefaultSearch, autoFetchData
*  will be automatically suspended and replaced with the saved criteria/view state, if applicable.
* @type {boolean}
* @default false
*/
autoFetchData?: boolean;

/**
* If Calendar.autoFetchData is true, this attribute allows the developer to
*  specify a textMatchStyle for the initial ListGrid.fetchData call.
* @type {TextMatchStyle}
* @default null
*/
autoFetchTextMatchStyle?: TextMatchStyle;

/**
* Criteria to be used when Calendar.autoFetchData is set.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
initialCriteria?: Criteria;

/**
* Criteria that are never shown to or edited by the user and are cumulative with any criteria
*  provided via DataBoundComponent.initialCriteria and related methods.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
implicitCriteria?: Criteria;

/**
* Whether to show fields marked detail:true when a DataBoundComponent is 
*  given a DataSource but no component.fields.
*  
*  The detail property is used on DataSource fields to mark fields that 
*  shouldn't appear by default in a view that tries to show many records in a small space.
* @type {boolean}
* @default true
*/
showDetailFields?: boolean;

/**
* How to fetch and manage records retrieve from the server. See FetchMode.
*  
*  This setting only applies to the ResultSet automatically created by calling
*  ListGrid.fetchData. If a pre-existing ResultSet is passed to setData() instead, it's
*  existing setting for ResultSet.fetchMode applies.
* @type {FetchMode}
* @default "paged"
*/
dataFetchMode?: FetchMode;

/**
* If set to true, show the Calendar.timelineView.
* @type {boolean}
* @default false
*/
showTimelineView?: boolean;

/**
* CalendarView used to display events in lanes in a horizontal Timeline view.
* @type {CalendarView}
* @default null
*/
timelineView?: CalendarView;

/**
* When set to true, the default, each EventCanvas is rendered as it 
*  appears in the viewport. If set to false, all events are rendered up-front, whenever the 
*  current range changes.
*  
*  Has no effect when Calendar.showColumnLayouts is true.
* @type {boolean}
* @default true
*/
renderEventsOnDemand?: boolean;

/**
* The granularity in which the Calendar.timelineView will display events.
*  Possible values are those available in the built-in TimeUnit type.
* @type {TimeUnit}
* @default "day"
*/
timelineGranularity?: TimeUnit;

/**
* How many units of Calendar.timelineGranularity each cell represents.
* @type {Integer}
* @default "1"
*/
timelineUnitsPerColumn?: Integer;

/**
* Can Timeline events be stretched by their left and right edges?
* @deprecated * in favor of Calendar.canResizeEvents;

* @type {boolean}
* @default false
*/
canResizeTimelineEvents?: boolean;

/**
* Can events be moved between lanes? If so, the event can be dragged to a different 
*  Calendar.lanes, and the event Calendar.eventDialog and 
*  Calendar.eventEditor allow a lane to be selected with a drop-down chooser.
*  
*  In either case, the event's Calendar.laneNameField is updated automatically.
*  
*  If set to false, cross-lane dragging is disallowed and drop-down Lane-choosers are disabled
*  when editing existng events. When creating Calendar.canCreateEvents, 
*  the Lane-chooser remains enabled so an initial Lane can be selected.
*  
*  This setting can be overridden on each CalendarEvent.canEditLane.
* @type {boolean}
* @default null
*/
canEditLane?: boolean;

/**
* Can events be moved between sublanes?
*  
*  If so, the event can be dragged to a different Lane.sublanes within the same
*  parent Lane and, when it's editor is shown, an additional drop-down widget is provided 
*  allowing the sublane to be altered.
*  
*  If the sublane is locked, but the Calendar.canEditLane isn't, an update
*  to the event's Calendar.laneNameField will be allowed, assuming that the
*  new Lane has an existing sublane with the same name.
*  
*  In either case, the event's Calendar.sublaneNameField is updated 
*  automatically.
*  
*  This setting can be overridden on each CalendarEvent.canEditSublane.
* @type {boolean}
* @default null
*/
canEditSublane?: boolean;

/**
* If true, lanes can be reordered by dragging their Calendar.laneFields
*  with the mouse.
* @type {boolean}
* @default null
*/
canReorderLanes?: boolean;

/**
* The start date of the calendar Timeline. Has no effect in 
*  other views. If not specified, defaults to a timeline starting from the beginning 
*  of the current Calendar.timelineGranularity and spanning
*  Calendar.defaultTimelineColumnSpan columns of that granularity. 
*  
*  To set different start and Calendar.endDate dates after initial draw,
*  see Calendar.setTimelineRange.
*  
*  Note that the value you provide may be automatically altered if showing 
*  Calendar.headerLevels, to fit to header boundaries.
* @type {Date}
* @default null
*/
startDate?: Date;

/**
* The number of columns of the Calendar.timelineGranularity to
*  give the timeline by default if no Calendar.endDate is provided. The 
*  default is 20.
* @type {number}
* @default "20"
*/
defaultTimelineColumnSpan?: number;

/**
* When using the Next and Previous arrows to scroll a Timeline, this is the number of columns 
*  of the Calendar.timelineGranularity to scroll by. With the
*  default value of null, the Timeline will scroll by its current length.
* @type {number}
* @default null
*/
columnsPerPage?: number;

/**
* The end date of the calendar timeline view. Has no effect in other views.
*  
*  To set different Calendar.startDate and end dates after initial draw,
*  see Calendar.setTimelineRange.
*  
*  Note that the value you provide may be automatically altered if showing 
*  Calendar.headerLevels, to fit to header boundaries.
* @type {Date}
* @default null
*/
endDate?: Date;

/**
* The text to appear before the week number in the title of TimeUnit 
*  HeaderLevels when this calendar is showing a timeline.
* @type {HTMLString}
* @default "Week"
*/
weekPrefix?: HTMLString;

/**
* Indicates the type of controls to use in event-windows. Valid values are those in the 
*  DateEditingStyle type.
*  
*  If unset, the editing style will be set to the field-type on the DataSource, if there is one.
*  If there's no DataSource, it will be set to "date" if the 
*  Calendar.timelineGranularity is "day" or larger and "time" if granularity 
*  is "minute" or smaller, otherwise "datetime".
* @type {DateEditingStyle}
* @default null
*/
dateEditingStyle?: DateEditingStyle;

/**
* If true, allows the lanes in a Timeline to be grouped by providing a value for 
*  Calendar.laneGroupByField. The fields available for grouping on 
*  are those defined as Calendar.laneFields. Since these are definitions
*  for ListGridField, you can choose to ListGridField.showIf
*  the field in the timeline, but still have it available for grouping.
* @type {boolean}
* @default null
*/
canGroupLanes?: boolean;

/**
* For timelines with Calendar.canGroupLanes set to true, this is a 
*  field name or array of field names on which to group the lanes in a timeline.
* @type {string | Array<Partial<string>>}
* @default null
*/
laneGroupByField?: string | Array<Partial<string>>;

/**
* Describes the default state of lane groups in timelines when 
*  Calendar.groupLanesBy is called.
* 
*  Possible values are:
*  
*  
* - "all": open all groups
*  
* - "first": open the first group
*  
* - "none": start with all groups closed
*  
* - Array of values that should be opened
*  
* @type {GroupStartOpen | Array<any>}
* @default "first"
*/
laneGroupStartOpen?: GroupStartOpen | Array<any>;

/**
* An array of Lane definitions that represent the rows of the Calendar.timelineView, or
*  the columns of the Calendar.dayView if Calendar.showDayLanes is true.
* @type {Array<Partial<Lane>>}
* @default null
*/
lanes?: Array<Partial<Lane>>;

/**
* Field definitions for the frozen area of the Calendar.timelineView, which shows data about the
*  timeline Calendar.lanes. Each field shows one attribute of the objects provided as
*  Calendar.lanes.
*  
*  When Calendar.canGroupLanes is enabled, only fields that are specified
*  as lane fields can be used as group fields.
* @type {Array<Partial<ListGridField>>}
* @default null
*/
laneFields?: Array<Partial<ListGridField>>;

/**
* If set to true, the Calendar.dayView uses Calendar.lanes to render multiple
*  vertical "lanes" within the day, very much like a vertical Timeline.
*  
*  Day lanes are useful for showing events for various entities on the same day - agendas for 
*  various staff members, for example, or delivery schedules for a fleet of trucks.
*  
*  Each day lane is self-contained, showing in a column with a header and individual events 
*  are placed in CalendarEvent.lane, respecting padding and 
*  overlapping. If Calendar.canEditEvents is true, events can be drag-moved or drag-resized 
*  from their top and bottom edges, within the containing lane. To allow events to be dragged 
*  from one lane into another, see Calendar.canEditLane.
* @type {boolean}
* @default null
*/
showDayLanes?: boolean;

/**
* When showing Calendar.showDayLanes in the Calendar.dayView, this attribute sets 
*  the minimum width of each column or field.
* @type {Integer}
* @default null
*/
minLaneWidth?: Integer;

/**
* A set of SortSpecifier for customizing the render order of events
*  that overlap.
*  
*  In Timeline, this dictates the vertical rendering order of 
*  overlapped events in each Lane.
*  
*  In Calendar.dayView and Calendar.weekView views, it dictates the
*  horizontal rendering order of overlapped events in each column or Lane.
*  
*  By default, events that share space in a Lane or column are rendered from top to bottom,
*  or left to right according to their Calendar.startDateField - the earliest in a 
*  given lane appears top-most in that lane, or left-most in its column.
*  
*  Providing overlapSortSpecifiers allows for the events to be ordered by one or
*  more of the fields stored on the events, or in the underlying DataSource,
*  if the Calendar is databound.
* @type {Array<Partial<SortSpecifier>>}
* @default null
*/
overlapSortSpecifiers?: Array<Partial<SortSpecifier>>;

/**
* The background color for cells that represent today in all CalendarViews.
* @type {string}
* @default null
*/
todayBackgroundColor?: string;

/**
* When rendering the Calendar.eventCanvas for an event, whether to show the 
*  EventCanvas.showBody, typically containing brief details of the event - 
*  Calendar.getEventBodyHTML, 
*  Calendar.descriptionField.
*  
*  The default is true - if set to false, the event's EventCanvas.showHeader 
*  will fill the canvas.
* @type {boolean}
* @default true
*/
showEventDescriptions?: boolean;

/**
* When rendering the Calendar.eventCanvas for an event, whether to show the 
*  EventCanvas.showHeader, typically containing suitable title text - 
*  Calendar.getEventHeaderHTML, the event's Calendar.nameField.
*  
*  The default is true - if set to false, the event's EventCanvas.showBody 
*  will fill the canvas.
* @type {boolean}
* @default true
*/
showEventHeaders?: boolean;

/**
* When rendering the Calendar.eventCanvas for an event, whether to allow the 
*  content of the EventCanvas.showHeader to wrap to multiple lines.
*  
*  The default is true - if set to false, the header area is 
*  Calendar.eventHeaderHeight, unless Calendar.showEventDescriptions is
*  false, in which case the header area fills the canvas.
* @type {boolean}
* @default true
*/
eventHeaderWrap?: boolean;

/**
* When Calendar.eventHeaderWrap is false and 
*  Calendar.showEventDescriptions is true, this is the fixed 
*  height for the EventCanvas.showHeader in event canvases.
* @type {Integer}
* @default "14"
*/
eventHeaderHeight?: Integer;

/**
* If true, events will be sized to the grid, even if they start and/or end at times
*  between grid cells.
* @type {boolean}
* @default true
*/
sizeEventsToGrid?: boolean;

/**
* The title for the Calendar.dayView.
* @type {string}
* @default "Day"
*/
dayViewTitle?: string;

/**
* The title for the Calendar.weekView.
* @type {string}
* @default "Week"
*/
weekViewTitle?: string;

/**
* The title for the Calendar.monthView.
* @type {string}
* @default "Month"
*/
monthViewTitle?: string;

/**
* The title for the Calendar.timelineView.
* @type {string}
* @default "Timeline"
*/
timelineViewTitle?: string;

/**
* The title for the Calendar.nameField in the quick
*  Calendar.eventDialog and the detailed 
*  Calendar.eventEditor.
* @type {HTMLString}
* @default "Event Name"
*/
eventNameFieldTitle?: HTMLString;

/**
* The title for the Calendar.startDateField in the quick
*  Calendar.eventDialog and the detailed 
*  Calendar.eventEditor.
* @type {HTMLString}
* @default "From"
*/
eventStartDateFieldTitle?: HTMLString;

/**
* The title for the Calendar.endDateField in the quick
*  Calendar.eventDialog and the detailed 
*  Calendar.eventEditor.
* @type {HTMLString}
* @default "To"
*/
eventEndDateFieldTitle?: HTMLString;

/**
* The title for the Calendar.descriptionField field in the quick 
*  Calendar.eventDialog and the detailed 
*  Calendar.eventEditor.
* @type {HTMLString}
* @default "Description"
*/
eventDescriptionFieldTitle?: HTMLString;

/**
* The title for the Calendar.laneNameField in the quick 
*  Calendar.eventDialog and the detailed 
*  Calendar.eventEditor.
* @type {HTMLString}
* @default "Lane"
*/
eventLaneFieldTitle?: HTMLString;

/**
* The title for the Calendar.sublaneNameField in the quick 
*  Calendar.eventDialog and the detailed 
*  Calendar.eventEditor.
* @type {HTMLString}
* @default "Sublane"
*/
eventSublaneFieldTitle?: HTMLString;

/**
* The title for the Calendar.durationField in the quick
*  Calendar.eventDialog and the detailed 
*  Calendar.eventEditor.
* @type {HTMLString}
* @default "Duration"
*/
eventDurationFieldTitle?: HTMLString;

/**
* The title for the Calendar.durationUnitField in the quick
*  Calendar.eventDialog and the detailed 
*  Calendar.eventEditor.
* @type {HTMLString}
* @default "&amp;nbsp;"
*/
eventDurationUnitFieldTitle?: HTMLString;

/**
* The title for the Calendar.saveButton in the 
*  Calendar.eventDialog and the 
*  Calendar.eventEditor.
* @type {HTMLString}
* @default "Save Event"
*/
saveButtonTitle?: HTMLString;

/**
* The title for the edit button in the quick Calendar.eventDialog.
* @type {HTMLString}
* @default "Edit Details"
*/
detailsButtonTitle?: HTMLString;

/**
* The title for the Calendar.removeButton in the
*  Calendar.eventEditor.
* @type {HTMLString}
* @default "Remove Event"
*/
removeButtonTitle?: HTMLString;

/**
* The title for the Calendar.cancelButton in the
*  Calendar.eventEditor.
* @type {HTMLString}
* @default "Cancel"
*/
cancelButtonTitle?: HTMLString;

/**
* The title of the Calendar.monthButton, used for showing and hiding the
*  Calendar.monthView on Handsets.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will
*  be evaluated as JS code when the message is displayed.
*  
*  Only one dynamic variable, monthName, is available and represents the name of the month 
*  containing the currently selected date.
*  
*  The default value is a left-facing arrow followed by the Month-name of the selected date.
*  
*  When the month view is already visible, the title for the month button is set according to
*  the value of Calendar.backButtonTitle.
* @type {HTMLString}
* @default "&amp;lt; ${monthName}"
*/
monthButtonTitle?: HTMLString;

/**
* The title of the link shown in a cell of a Calendar.monthView when there
*  are too many events to be displayed at once.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  Only one dynamic variable, eventCount, is available and represents the number of events that
*  are not currently displayed and that will appear in the menu displayed when the More Events 
*  link is clicked.
*  
*  The default value is a string like "+ 3 more...".
* @type {HTMLString}
* @default "+ ${eventCount} more..."
*/
monthMoreEventsLinkTitle?: HTMLString;

/**
* The title of the Calendar.monthButton on Handsets when the 
*  Calendar.monthView is the current visible view. 
*  
*  When the month view is not the current visible view, the title for the month button is set 
*  according to the value of Calendar.monthButtonTitle.
* @type {HTMLString}
* @default "Back"
*/
backButtonTitle?: HTMLString;

/**
* The text to be displayed when a user hovers over the Calendar.previousButton
*  toolbar button.
* @type {string}
* @default "Previous"
*/
previousButtonHoverText?: string;

/**
* The text to be displayed when a user hovers over the Calendar.nextButton 
*  toolbar button
* @type {string}
* @default "Next"
*/
nextButtonHoverText?: string;

/**
* The text to be displayed when a user hovers over the Calendar.addEventButton
*  toolbar button
* @type {string}
* @default "Add an event"
*/
addEventButtonHoverText?: string;

/**
* The text to be displayed when a user hovers over the Calendar.datePickerButton
*  toolbar button
* @type {string}
* @default "Choose a date"
*/
datePickerHoverText?: string;

/**
* The message to display in the Calendar.eventEditor when the 'To' date is greater than
*  the 'From' date and a save is attempted.
* @type {string}
* @default "From must be before To"
*/
invalidDateMessage?: string;

/**
* An AutoChild of type Window that displays a quick event entry form in a 
*  popup window.
* @type {Window}
* @default null
*/
eventDialog?: Window;

/**
* An AutoChild of type Window that displays the full 
*  Calendar.eventEditor
* @type {Window}
* @default null
*/
eventEditorLayout?: Window;

/**
* An AutoChild of type DynamicForm which displays CalendarEvent. 
*  This form is created within the Calendar.eventEditorLayout
* @type {DynamicForm}
* @default null
*/
eventEditor?: DynamicForm;

/**
* An AutoChild of type HLayout which houses the 
*  Calendar.saveButton, Calendar.removeButton
*  and Calendar.cancelButton buttons in the 
*  Calendar.eventEditor.
* @type {HLayout}
* @default null
*/
eventEditorButtonLayout?: HLayout;

/**
* An AutoChild of type IButton, used to save an event from the 
*  Calendar.eventEditor.
* @type {IButton}
* @default null
*/
saveButton?: IButton;

/**
* An AutoChild of type IButton, used to permanently remove an event from the 
*  Calendar.eventEditor.
* @type {IButton}
* @default null
*/
removeButton?: IButton;

/**
* An AutoChild of type IButton, used to cancel editing of an event and close the 
*  Calendar.eventEditor.
* @type {IButton}
* @default null
*/
cancelButton?: IButton;

/**
* Set to false to hide the Calendar.addEventButton button.
* @type {boolean}
* @default null
*/
showAddEventButton?: boolean;

/**
* An ImgButton that appears in a Calendar's week/day/month views and offers an 
*  alternative way to create a new CalendarEvent.
* @type {ImgButton}
* @default null
*/
addEventButton?: ImgButton;

/**
* Set to false to hide the Calendar.datePickerButton that allows selecting a new base date for 
*  this Calendar.
* @type {boolean}
* @default null
*/
showDatePickerButton?: boolean;

/**
* An ImgButton that appears above the various views of the
*  calendar and offers alternative access to a DateChooser to pick the current day.
* @type {ImgButton}
* @default null
*/
datePickerButton?: ImgButton;

/**
* The size of the various icons displayed in the Calendar.controlsBar 
*  of this Calendar.
* @type {Integer}
* @default "16"
*/
controlBarIconSize?: Integer;

/**
* Default height of the Calendar.controlsBar shown above the main Calendar grid. When multiple 
*  views are available and tabs are visible, this value is modified to ensure that the content
*  of the controlsBar is vertically-aligned with the text in the tabs.
* @type {Integer}
* @default "34"
*/
controlsBarHeight?: Integer;

/**
* If false the controls bar at the top of the calendar will not be displayed - this means 
*  that the Calendar.controlsBar will be hidden, so the autoChildren (Calendar.previousButton, 
*  Calendar.dateLabel, Calendar.nextButton, Calendar.addEventButton, and Calendar.datePickerButton) 
*  will not be created or shown.
* @type {boolean}
* @default true
*/
showControlsBar?: boolean;

/**
* An HLayout shown above the Calendar views and displaying a set of
*  controls for interacting with the current view - namely, the Calendar.nextButton,
*  Calendar.previousButton and Calendar.addEventButton buttons, 
*  the Calendar.dateLabel and the Calendar.datePickerButton icon.
* @type {HLayout}
* @default null
*/
controlsBar?: HLayout;

/**
* Set to false to prevent the Calendar.monthButton button from displaying on Handset 
*  devices.
* @type {boolean}
* @default null
*/
showMonthButton?: boolean;

/**
* A NavigationButton that appears to the left of other navigation controls in the 
*  Calendar.controlsBar on Handset devices.
*  
*  Used to show and hide the Calendar.monthView on devices with limited space.
* @type {NavigationButton}
* @default null
*/
monthButton?: NavigationButton;

/**
* Set to false to hide the Calendar.previousButton button.
* @type {boolean}
* @default null
*/
showPreviousButton?: boolean;

/**
* An ImgButton that appears above the week/day/month views of the
*  calendar and allows the user to move the calendar backwards in time.
* @type {ImgButton}
* @default null
*/
previousButton?: ImgButton;

/**
* Set to false to hide the Calendar.nextButton button.
* @type {boolean}
* @default null
*/
showNextButton?: boolean;

/**
* An ImgButton that appears above the week/day/month views of the
*  calendar and allows the user to move the calendar forwards in time.
* @type {ImgButton}
* @default null
*/
nextButton?: ImgButton;

/**
* The AutoChild Label used to display the current date or range above the 
*  selected calendar view.
* @type {Label}
* @default null
*/
dateLabel?: Label;

/**
* The height of time-slots in the calendar.
* @type {number}
* @default "20"
*/
rowHeight?: number;

/**
* The minimum height of time-rows in vertical calendar views. Rows will not shrink below 
*  this height when Calendar.sizeToWorkday is true, meaning that a Calendar with a long workday
*  may not be able to fit all workday rows in the viewport at once, and scrolling may be
*  necessary.
*  
*  To prevent users from scrolling beyond the workday hours, see Calendar.limitToWorkday.
* @type {number}
* @default "20"
*/
minRowHeight?: number;

/**
* The name of the view that should be visible initially by default.
* @type {ViewName}
* @default null
*/
currentViewName?: ViewName;

/**
* Dictates whether times throughout the widget are formatted and edited as 24-hour values. If
*  unset, defaults to the Time.use24HourTime. If set, and no
*  Calendar.timeFormatter is installed, causes the 
*  Calendar to choose an appropriate builtin formatter.
* @type {boolean}
* @default null
*/
twentyFourHourTime?: boolean;

/**
* Date formatter for displaying events.
*  Default is to use the system-wide default short date format, configured via
*  DateUtil.setShortDisplayFormat. Specify any valid DateDisplayFormat.
* @type {DateDisplayFormat}
* @default null
*/
dateFormatter?: DateDisplayFormat;

/**
* Display format to use for the time portion of events' date information. By default, times
*  are displayed in the global format, including the influence of the global 
*  Time.use24HourTime option, which is true by default.
*  P>
*  Note that this display setting does not affect the way in which time values are edited in the 
*  Calendar.eventEditor - see Calendar.twentyFourHourTime for more
*  information.
* @type {TimeDisplayFormat}
* @default null
*/
timeFormatter?: TimeDisplayFormat;

/**
* By default, EventCanvases show their content in hovers. If you set this attribute to false,
*  hovers will only be shown if the content of the event-canvas is visually clipped.
*  
*  Note - if you have custom hover-content/handling, you should leave this property set to 
*  true.
* @type {boolean}
* @default true
*/
alwaysShowEventHovers?: boolean;

/**
* In Calendar.indicators are showing, this attribute affects where in 
*  the z-order their canvases will be rendered: either in front of, or behind normal calendar
*  events.
* @type {boolean}
* @default true
*/
showIndicatorsInFront?: boolean;

/**
* When Calendar.showViewHovers is true, dictates whether to display
*  hover prompts when the mouse rolls over the normal cells in the body of CalendarViews.
*  
*  The content of the hover is determined by a call to 
*  Calendar.getCellHoverHTML, which can be overridden to return custom results; by 
*  default, it returns the cell's date as a string.
* @type {boolean}
* @default false
*/
showCellHovers?: boolean;

/**
* When Calendar.showViewHovers is true, dictates whether to display
*  hover prompts when the mouse rolls over the Calendar.headerLevels in
*  a CalendarView.
*  
*  The content of the hover is determined by a call to 
*  Calendar.getHeaderHoverHTML, which can be overridden to return custom results;
* @type {boolean}
* @default false
*/
showHeaderHovers?: boolean;

/**
* When set to true, the default value, causes the Calendar to show customizable hovers when 
*  the mouse moves over various areas of a CalendarView.
*  
*  See Calendar.showEventHovers, 
*  Calendar.showZoneHovers,
*  Calendar.showHeaderHovers,
*  Calendar.showCellHovers,
*  Calendar.showLaneFieldHovers,
*  Calendar.showDragHovers for further configuration options.
* @type {boolean}
* @default true
*/
showViewHovers?: boolean;

/**
* When Calendar.showViewHovers is true, dictates whether to display
*  hover prompts when the mouse moves over an EventCanvas in a 
*  calendarView.
*  
*  The content of the hover is determined by a call to 
*  Calendar.getCellHoverHTML, which can be overridden to return custom results.
* @type {boolean}
* @default true
*/
showEventHovers?: boolean;

/**
* When Calendar.showViewHovers is true, dictates whether to display
*  hover prompts when the mouse moves over a Calendar.zones in a calendarView.
*  
*  When Calendar.showCellHovers is true, this attribute is ignored and
*  zone hovers are not displayed.
*  
*  The content of the hover is determined by a call to 
*  Calendar.getZoneHoverHTML, which can be overridden to return custom results.
* @type {boolean}
* @default true
*/
showZoneHovers?: boolean;

/**
* When Calendar.showViewHovers is true, dictates whether to display
*  hover prompts when the mouse moves over the cells in a 
*  Calendar.laneFields.
*  
*  The content of the hover is determined by a call to 
*  Calendar.getCellHoverHTML, which can be overridden to return custom results. Note 
*  that getCellHoverHTML() is also called when the mouse moves over cells if
*  Calendar.showCellHovers is true - when called for a laneField, no
*  "date" parameter is passed to that method.
* @type {boolean}
* @default false
*/
showLaneFieldHovers?: boolean;

/**
* When Calendar.showViewHovers is true, dictates whether to display
*  hover prompts when an event is being dragged with the mouse.
*  
*  The content of the hover is determined by a call to 
*  Calendar.getDragHoverHTML, which can be overridden to return custom results; by 
*  default, it returns the date range of the drag canvas as a string.
* @type {boolean}
* @default false
*/
showDragHovers?: boolean;

/**
* Whether Calendar.eventCanvas should show a custom widget as content, 
*  rather than the default EventCanvas.showHeader and 
*  EventCanvas.showBody HTML.
* @type {boolean}
* @default false
*/
showEventCanvasComponents?: boolean;

/**
* Multi-AutoChild component, created as a space-filling member in individual 
*  EventCanvas, when Calendar.showEventCanvasComponents is
*  true.
*  
* @type {Canvas}
* @default null
*/
eventCanvasComponent?: Canvas;

/**
* Screen to create (via RPCManager.createScreen) in lieu of calling
*  Calendar.createEventCanvasComponent.
*  
*  If this calendar has a DataBoundComponent.dataSource, the created screen is
*  provided with a Canvas.dataContext that includes the event being shown.
*  Be sure the event screen meets these Canvas.autoPopulateData
*  to utilize the dataContext.
* @type {string}
* @default null
*/
eventScreen?: string;

/**
* Context menu displayed when an EventCanvas is right-clicked, or
*  when the rollover Calendar.eventCanvasContextButton is clicked. The 
*  context button, and the menu itself, will only be displayed if 
*  Calendar.getEventCanvasMenuItems returns 
*  an array of appropriate items for the event.
* @type {Menu}
* @default null
*/
eventCanvasContextMenu?: Menu;

/**
* By default, the Calendar.eventCanvasCloseButton and the
*  Calendar.eventCanvasHResizer and 
*  Calendar.eventCanvasVResizer resizer widgets 
*  for event canvases are shown only when the mouse is over a given event. Set this attribute
*  to false to have event canvases show these widgets permanently.
* @type {boolean}
* @default true
*/
useEventCanvasRolloverControls?: boolean;

/**
* When set to true, makes individual EventCanvas selectable.
*  Events may be selected via a single click, as well as being
*  included in the page's tab order. The current selected event is shown in a special 
*  style and pressing TAB or Shift-TAB will move the selection first among the events 
*  in the same lane, and then among those in the next or previous lane.
*  
*  Pressing Enter while an editable event is selected 
*  will show either the event- Calendar.eventDialog or 
*  Calendar.eventEditor. Pressing Delete will remove the event.
*  
*  Note that when this property is false, single clicking the event canvas for an
*  editable event will bring up an editing interface for that event. 
*  When true this is no longer the case - a user can double click to bring up the editing
*  interface instead (a single click will simply select the event canvas).
* @type {boolean}
* @default null
*/
canSelectEvents?: boolean;

/**
* The selection associated with the Calendar.
* @type {Selection}
* @default null
*/
selectionManager?: Selection;

/**
* The "gripper" widget that snaps to the top of an event canvas and allows an
*  event to be dragged with the mouse.
* @type {Img}
* @default null
*/
eventCanvasGripper?: Img;

/**
* Icon used as the default eveng gripper icon.
* @type {SCImgURL}
* @default "[SKIN]/Calendar/gripper.png"
*/
eventCanvasGripperIcon?: SCImgURL;

/**
* @type {Label}
* @default null
*/
eventCanvasLabel?: Label;

/**
* HLayout that snaps to the top-right of an event canvas on rollover and contains the
*  Calendar.eventCanvasCloseButton and/or 
*  Calendar.eventCanvasContextButton buttons.
* @type {HLayout}
* @default null
*/
eventCanvasButtonLayout?: HLayout;

/**
* The close button that snaps to the top-right of an event canvas on rollover and allows an
*  event to be removed from a CalendarView.
* @type {ImgButton}
* @default null
*/
eventCanvasCloseButton?: ImgButton;

/**
* The context button that snaps to the top-right of an event canvas on rollover and shows a
*  custom Calendar.getEventCanvasMenuItems when clicked.
* @type {ImgButton}
* @default null
*/
eventCanvasContextButton?: ImgButton;

/**
* The resizer image that snaps to the bottom of event canvases in Calendar.dayView
*  and Calendar.weekView views, allowing them to be resized vertically by dragging 
*  with the mouse.
* @type {Img}
* @default null
*/
eventCanvasVResizer?: Img;

/**
* The resizer image that snaps to the left and right edges of an editable event canvas in a
*  Timeline, allowing it to be resized horizontally by dragging with the mouse.
* @type {Img}
* @default null
*/
eventCanvasHResizer?: Img;

/**
* Set to true to render any defined Calendar.zones into 
*  Calendar.timelineView.
* @type {boolean}
* @default null
*/
showZones?: boolean;

/**
* An array of CalendarEvent instances representing pre-defined periods of time to be 
*  highlighted in Calendar.timelineView. Each zone renders out a
*  ZoneCanvas, a special, non-interactive subclass of 
*  EventCanvas, which spans all lanes and draws behind any normal, interactive 
*  events in the zorder.
*  
*  The default Calendar.zoneStyleName for these components renders them 
*  semi-transparent and with a bottom-aligned title label.
* @type {Array<Partial<CalendarEvent>>}
* @default null
*/
zones?: Array<Partial<CalendarEvent>>;

/**
* CSS style to apply to the Calendar.zoneCanvas created for each
*  specified Calendar.zones.
* @type {CSSStyleName}
* @default "zoneCanvas"
*/
zoneStyleName?: CSSStyleName;

/**
* AutoChild component created for each Calendar.zones entry.
* @type {ZoneCanvas}
* @default null
*/
zoneCanvas?: ZoneCanvas;

/**
* The vertical alignment of the header-text in each Calendar.zones.
* @type {VerticalAlignment}
* @default "bottom"
*/
zoneTitleOrientation?: VerticalAlignment;

/**
* Set to true to render any defined Calendar.indicators into 
*  Calendar.timelineView.
* @type {boolean}
* @default null
*/
showIndicators?: boolean;

/**
* An array of CalendarEvent instances representing instants in time, to be 
*  highlighted in Calendar.timelineView. Each indicator renders out as 
*  an IndicatorCanvas, a special, non-interactive subclass of 
*  EventCanvas, which spans all lanes and draws behind any normal, interactive 
*  events in the zorder, but in front of any Calendar.zones. The default 
*  Calendar.indicatorStyleName for these components renders them as thin vertical
*  lines that span all lanes and have a hover but no title.
* @type {Array<Partial<CalendarEvent>>}
* @default null
*/
indicators?: Array<Partial<CalendarEvent>>;

/**
* CSS style to apply to the Calendar.indicatorCanvas created for each
*  specified Calendar.indicators.
* @type {CSSStyleName}
* @default "indicatorCanvas"
*/
indicatorStyleName?: CSSStyleName;

/**
* AutoChild component created for each Calendar.indicators entry.
* @type {IndicatorCanvas}
* @default null
*/
indicatorCanvas?: IndicatorCanvas;

/**
* To display events in Calendar.dayView, Calendar.weekView and
*  Calendar.timelineView views, the Calendar creates instances of 
*  EventCanvas for each event. Use the AutoChild system to customize 
*  these canvases.
* @type {EventCanvas}
* @default null
*/
eventCanvas?: EventCanvas;

/**
* When true, causes Calendar.columnLayout to be added to each column in 
*  vertical views. In this mode, eventCanvases are stacked in these layouts, filling width 
*  and auto-sizing vertically to content, rather than being placed, sized and overlapped 
*  according to their times.
*  
*  Because times are ignored in this mode, various behaviors are switched off automatically;
*  for example, the Calendar.showLabelColumn is hidden and event-canvases 
*  cannot be Calendar.canResizeEvents or rendered
*  Calendar.renderEventsOnDemand.
* @type {boolean}
* @default false
*/
showColumnLayouts?: boolean;

/**
* When Calendar.showColumnLayouts is true, the layouts added to each column to stack 
*  events.
* @type {VLayout}
* @default null
*/
columnLayout?: VLayout;

/**
* The title-text displayed in the popup event dialog/editor for new events.
* @type {HTMLString}
* @default "New Event"
*/
newEventWindowTitle?: HTMLString;

/**
* AutoChild Menu, shown when a user clicks the 
*  Calendar.monthMoreEventsLinkTitle link in a cell of the 
*  Calendar.monthView. Items in this menu represent additional events, 
*  not already displayed in the cell, and clicking them fires the 
*  Calendar.eventClick notification.
* @type {Menu}
* @default null
*/
monthMoreEventsMenu?: Menu;



/**
*  Scrolls Calendar Calendar.dayView or Calendar.weekView views to the
*  time represented by the time parameter. This string parameter is expected to be an 
*  arbitrary logical time value in any parsable time format - no date portion is expected, but 
*  time formats like "13:31" or "1:20am" are supported.
*  
*  Has no effect in Calendar.timelineView, where an arbitrary time-value is
*  inapplicable to any range or resolution greater than a day.
* 
* @param {string} any parsable time-string
*/
scrollToTime(time:string): void;

/**
*  Move the viewport of a CalendarView to the start of it's scrollable range.
* 
* @param {CalendarView=} the view to affect, the current view if not specified
*/
scrollToStart(view?:CalendarView): void;

/**
*  Move the viewport of a CalendarView to the end of its scrollable range.
* 
* @param {CalendarView=} the view to affect, the current view if not specified
*/
scrollToEnd(view?:CalendarView): void;

/**
*  Resets the current visible range of a calendar view so that it shows the date on which the
*  passed event occurs.
* 
* @param {CalendarEvent} the event to move the calendar view to
* @deprecated * in favor of Calendar.scrollToEvent.

*/
moveToEvent(event:CalendarEvent): void;

/**
*  Scrolls the Calendar.getCurrentViewName so the passed event is 
*  visible. If the event is outside of the view's current date-range, the default behavior is
*  to automatically reload the view with a date-range starting at the event's 
*  Calendar.startDateField and then scroll to the event vertically as 
*  necessary. Pass false as the canReload param to prevent that default behavior.
* 
* @param {CalendarEvent} the event to move the calendar view to
* @param {boolean=} set to false to prevent a view from automatically reloading                with a new range if the passed event is not in its current                scrollable range
*/
scrollToEvent(event:CalendarEvent, canReload?:boolean): void;

/**
*  Setter for updating Calendar.hideUnusedLanes after creation.
* 
* @param {boolean} whether to hide unused lanes
*/
setHideUnusedLanes(hideUnusedEvents:boolean): void;

/**
*  For Timelines, configures the levels of HeaderLevel shown above the 
*  event area, and their time units, after initialization.
* 
* @param {Array<Partial<HeaderLevel>>} the array of HeaderLevels to set
*/
setHeaderLevels(headerLevels:Array<Partial<HeaderLevel>>): void;

/**
*  Returns true if the passed date should be considered disabled. Disabled dates don't allow 
*  events to be created by clicking on them, and drag operations that would start or end on
*  such dates are also disallowed.
*  
*  The default implementation returns false only for dates that fall on a 
*  DateUtil.getWeekendDays.
* 
* @param {Date} a Date instance
* @param {CalendarView=} the view the date appears in
*/
shouldDisableDate(date:Date, view?:CalendarView): boolean;

/**
*  Indicates whether the passed date should be visible in the passed CalendarView.
*  
*  The default implementation returns true, unless the date falls on a 
*  DateUtil.getWeekendDays and Calendar.showWeekends is 
*  false.
* 
* @param {Date} a Date instance
* @param {CalendarView=} the view the date appears in
*/
shouldShowDate(date:Date, view?:CalendarView): boolean;

/**
*  Indicates whether the passed Calendar.lanes should be visible in the passed 
*  CalendarView.
*  
*  The default implementation returns true, unless the lane has no events and 
*  Calendar.hideUnusedLanes is true.
* 
* @param {Lane | string} the lane object or name
* @param {CalendarView=} the view the lane appears in
*/
shouldShowLane(lane:Lane | string, view?:CalendarView): boolean;

/**
*  Indicates whether the passed CalendarEvent should be visible in the 
*  passed CalendarView.
*  
*  The default implementation returns true - note that this method only runs for events that are 
*  known to be in the accessible range and is a mechanism for extended custom filtering.
* 
* @param {CalendarEvent} the event to check
* @param {CalendarView=} the view the event will be rendered in
*/
shouldShowEvent(event:CalendarEvent, view?:CalendarView): boolean;

/**
*  Returns the start of the working day on the passed date. By default, this method returns
*  the value of Calendar.workdayStart.
* 
* @param {Date} a Date instance
* @param {string=} the name of the relevant lane - only passed for dayView with               showDayLanes: true
*/
getWorkdayStart(date:Date, laneName?:string): string;

/**
*  Returns the end of the working day on the passed date. By default, this method returns
*  the value of Calendar.workdayEnd.
* 
* @param {Date} a Date instance
* @param {string=} the name of the relevant lane - only passed for dayView with               showDayLanes: true
*/
getWorkdayEnd(date:Date, laneName?:string): string;

/**
*  Returns the first visible date in the passed, or currently selected, calendar view.
* 
* @param {CalendarView=} the view to get the startDate for, or current view if
*/
getVisibleStartDate(view?:CalendarView): Date;

/**
*  Returns the last visible date in the passed, or currently selected, calendar view.
* 
* @param {CalendarView=} the view to get the endDate for, or current view if null
*/
getVisibleEndDate(view?:CalendarView): Date;

/**
*  Returns the start of the selected week or month depending on the current calendar view. 
*  For the month view, and for the week view when not showing weekends, this will often be a 
*  different date than that returned by Calendar.getVisibleStartDate.
* 
* @param {CalendarView=} the view to get the periodStartDate for, or current view if null
*/
getPeriodStartDate(view?:CalendarView): Date;

/**
*  Returns the end of the period selected in the passed, or current, calendar view. 
*  For the Calendar.monthView, and for the 
*  Calendar.weekView when not showing weekends, this will often be a 
*  different date than that returned by Calendar.getVisibleEndDate.
* 
* @param {CalendarView=} the view to get the periodEndDate for, or current view if null
*/
getPeriodEndDate(view?:CalendarView): Date;

/**
*  Retrieves data from the DataSource that matches the specified criteria.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
*  
*  When fetchData() is first called, if data has not already been provided via
*  ListGrid.setData, this method will create a ResultSet, which will be
*  configured based on component settings such as DataBoundComponent.fetchOperation
*  and DataBoundComponent.dataPageSize, as well as the general purpose
*  ListGrid.dataProperties. The created ResultSet will automatically send a DSRequest
*  to retrieve data from ListGrid.dataSource, and from then on will 
*  automatically manage paging through large datasets, as well as performing filtering and
*  sorting operations inside the browser when possible - see the ResultSet docs for
*  details.
*  
*  NOTE: do not use both DataBoundComponent.autoFetchData and a
*  call to fetchData() - this may result in two DSRequests to fetch data. Use
*  either DataBoundComponent.autoFetchData and Criteria or a manual call to fetchData()
*  passing criteria.
*  
*  Whether a ResultSet was automatically created or provided via ListGrid.setData, subsequent
*  calls to fetchData() will simply call ResultSet.setCriteria.
*  
*  Changes to criteria may or may not result in a DSRequest to the server due to
*  ResultSet.useClientFiltering. You can call
*  DataBoundComponent.willFetchData to determine if new criteria will result in a
*  server fetch.
*  
*  If you need to force data to be re-fetched, you can call
*  ListGrid.invalidateCache and new data will automatically be fetched
*  from the server using the current criteria and sort direction. NOTE: when using
*  invalidateCache() there is no need to also call fetchData()
*  and in fact this could produce unexpected results.
*  
*  This method takes an optional callback parameter (set to a DSCallback) to fire when
*  the fetch completes. Note that this callback will not fire if no server fetch is performed.
*  In this case the data is updated synchronously, so as soon as this method completes you
*  can interact with the new data. If necessary, you can use
*  DataBoundComponent.willFetchData to determine whether or not a server
*  fetch will occur when fetchData() is called with new criteria.
*  
*  In addition to the callback parameter for this method, developers can use 
*  ListGrid.dataArrived to be notified every time data is loaded.
*  
*  By default, this method assumes a TextMatchStyle of "exact"; that can be overridden
*  by supplying a different value in the requestProperties parameter. 
*  See DataBoundComponent.willFetchData;
*  
*  Changing the request properties
*  
*  Changes to TextMatchStyle made via requestProperties will be honored in
*  combination with the fetch criteria, possibly invalidating cache and triggering a server
*  request if needed, as documented for DataBoundComponent.willFetchData.
*  In contrast, changes to
*  DSRequest.operationId in the request properties will cause
*  the ResultSet or ResultTree to be rebuilt, always refetching from the server.
*  However, changes to other request properties after the initial fetch won't be detected, and
*  no fetch will get triggered based on that new request context.
*  
*  To pick up such changes, we recommend that you call Calendar.setData (passing an
*  empty array to ensure the data model is cleared), and then call this method to fetch again.
*  If you try to do it by calling ListGrid.invalidateCache, you may see duplicate fetches if you
*  haven't already updated the data context by calling this method with the new request
*  properties, and fail to do so before the component is Canvas.redraw.
* 
* 
* @param {Criteria=} Search criteria. If a DynamicForm is passed                     in as this argument instead of a raw criteria                      object, will be derived by calling                      DynamicForm.getValuesAsCriteria
* @param {DSCallback=} callback to invoke when a fetch is complete. Fires                     only if server contact was required
* @param {DSRequest=} additional properties to set on the DSRequest                     that will be issued
*/
fetchData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Retrieves data that matches the provided criteria and displays the matching data in this
*  component.
*  
*  This method behaves exactly like ListGrid.fetchData except that
*  DSRequest.textMatchStyle is automatically set to "substring" so that String-valued
*  fields are matched by case-insensitive substring comparison.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
* 
* @param {Criteria=} Search criteria.            If a DynamicForm is passed in as this argument            instead of a raw criteria object, will be derived by calling            DynamicForm.getValuesAsCriteria
* @param {DSCallback=} callback to invoke when a fetch is complete. Fires                     only if server contact was required; see                      ListGrid.fetchData for details
* @param {DSRequest=} for databound components only - optional              additional properties to set on the DSRequest that will be issued
*/
filterData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  When Calendar.canGroupLanes is true, this method allows the grouping
*  in Calendar.timelineViews to be altered at runtime.
* 
* @param {string | Array<Partial<string>>} one or more field names to group by
*/
groupLanesBy(groupFieldName:string | Array<Partial<string>>): void;

/**
*  Sets the Calendar.lanes in the current calendar view. Only has an effect
*  in Calendar.timelineView, and in Calendar.dayView when 
*  Calendar.showDayLanes is true.
* 
* 
* @param {Array<Partial<Lane>>} array of lanes to display
* @param {boolean=} set to false to prevent events from being refreshed
*/
setLanes(lanes:Array<Partial<Lane>>, skipRefreshEvents?:boolean): void;

/**
*  Adds a new Lane to the calendar, for display in the 
*  Calendar.timelineView, and in the 
*  Calendar.dayView if Calendar.showDayLanes is true.
* 
* 
* @param {Lane} a new Lane object to add to the calendar
*/
addLane(lane:Lane): void;

/**
*  Removes a lane from the calendar in Calendar.timelineView, or in Calendar.dayView if 
*  Calendar.showDayLanes is true.
*  
*  Accepts either a Lane or a string that represents the 
*  Lane.name of a lane.
* 
* 
* @param {Lane | string} either the actual Lane object or the name of the lane to remove
*/
removeLane(lane:Lane | string): void;

/**
*  Changes the Calendar.showDayLanes of the day view at runtime - whether to show a
*  normal day column for the Calendar.chosenDate, or the specified set of 
*  Calendar.lanes.
*  
* 
* @param {boolean} whether or not to show lanes in the day view
*/
setShowDayLanes(showDayLanes:boolean): void;

/**
*  A notification method fired when the events in the current view have been refreshed.
* 
*/
eventsRendered(): void;

/**
*  Initialize the data object with the given array. Observes methods of the data object
*  so that when the data changes, the calendar will redraw automatically.
*  
* 
* @param {Array<Partial<CalendarEvent>>} data to show in the list
*/
setData(newData:Array<Partial<CalendarEvent>>): void;

/**
*  Returns the currently selected CalendarView.
* 
*/
getSelectedView(): CalendarView;

/**
*  Returns the CalendarView with the passed ViewName.
* 
* @param {ViewName} the name of the CalendarView to return
*/
getView(viewName:ViewName): CalendarView;

/**
*  Get the name of the visible view.  Returns one of 'day', 'week', 'month' or 'timeline'.
* 
* 
*/
getCurrentViewName(): ViewName;

/**
*  Sets the currently visible view.
* 
* 
* @param {ViewName} The name of the view that should be made visible.
*/
setCurrentViewName(viewName:ViewName): ViewName;

/**
*  Returns the title text for the passed event, which is displayed in the header area of an 
*  eventCanvas rendered in a vertical or horizontal view, or as a clickable link in a cell in a 
*  Calendar.showMonthView.
*  
*  The default implementation returns the event's
*  Calendar.nameField for timelines, and that same value pre-pended with
*  the event's Calendar.startDateField for day, week and month views.
*  
* 
* @param {CalendarEvent} the event to get the description text for
* @param {CalendarView=} the view in which the event is being rendered
*/
getEventHeaderHTML(event:CalendarEvent, view?:CalendarView): HTMLString;

/**
*  Returns the description text for the passed event, for display in the body area of an event 
*  canvas. The default implementation returns the event's
*  Calendar.descriptionField.
*  
* 
* @param {CalendarEvent} the event to get the description text for
* @param {CalendarView=} the view in which the event is being rendered
*/
getEventBodyHTML(event:CalendarEvent, view?:CalendarView): HTMLString;

/**
*  Returns the CalendarEvent.startDate of the passed event.
*  
* 
* @param {CalendarEvent} the event to get the start date of
*/
getEventStartDate(event:CalendarEvent): Date;

/**
*  Returns the Calendar.endDateField of the passed event. If the event is
*  Calendar.allowDurationEvents, the result is calculated from the 
*  CalendarEvent.startDate and the specified 
*  CalendarEvent.duration and CalendarEvent.durationUnit.
*  
* 
* @param {CalendarEvent} the event to get the start date of
*/
getEventEndDate(event:CalendarEvent): Date;

/**
*  Setter for updating Calendar.showWeekends at runtime.
* 
* 
* @param {boolean} whether or not to show weekends
*/
setShowWeekends(showWeekends:boolean): void;

/**
*  For Timelines, and for Calendar.dayView with 
*  Calendar.showDayLanes set, creates a new event and adds it to a 
*  particular Lane.
* 
* 
* @param {Lane} the Lane in which to add this event
* @param {Date | object} start date of event, or CalendarEvent Object
* @param {Date=} end date of event
* @param {string=} name of event
* @param {string=} description of event
* @param {any=} new values of additional fields to be updated
* @deprecated * in favor of Calendar.addCalendarEvent

*/
addLaneEvent(laneName:Lane, startDate:Date | object, endDate?:Date, name?:string, description?:string, otherFields?:any): void;

/**
*  Create a new event in this calendar instance.
* 
* 
* @param {Date | CalendarEvent} start date of event, or CalendarEvent Object
* @param {Date=} end date of event
* @param {string=} name of event
* @param {string=} description of event
* @param {object=} new values of additional fields to be updated
* @deprecated * in favor of Calendar.addCalendarEvent

*/
addEvent(startDate:Date | CalendarEvent, endDate?:Date, name?:string, description?:string, otherFields?:object): void;

/**
*  Create a new event in this calendar.
*  
*  In all cases, the CalendarEvent passed as the first parameter must have at 
*  least a Calendar.startDateField set. If the calendar is showing 
*  Calendar.lanes, the name of the CalendarEvent.lane and, if 
*  applicable, the CalendarEvent.sublane, must also be set.
*  
*  To deal with errors during saving, see Calendar.eventSaveError.
* 
* @param {CalendarEvent} the new calendar event to add
* @param {object=} additional, custom values to be saved with the event
*/
addCalendarEvent(event:CalendarEvent, customValues?:object): void;

/**
*  Remove an event from this calendar.
* 
* 
* @param {CalendarEvent} The event object to remove from the calendar
*/
removeEvent(event:CalendarEvent): void;

/**
*  Update an event in this calendar.
* 
* 
* @param {CalendarEvent} The event object to update
* @param {Date} start date of event
* @param {Date} end date of event
* @param {string} name of event
* @param {string} description of event
* @param {object} new values of additional fields to be updated
* @deprecated * in favor of Calendar.updateCalendarEvent

*/
updateEvent(event:CalendarEvent, startDate:Date, endDate:Date, name:string, description:string, otherFields:object): void;

/**
*  Notification method fired when an attempt to save CalendarEvent
*  following edits or drag-movements results in an error from the server. May be overridden 
*  to handle specific errors and provide feedback to the user or push a server-provided record
*  to client caches with a call to DataSource.updateCaches, for example.
*  
*  The scenarioCode parameter may be used to determine 
*  CalendarSaveScenario.
*  
*  Return false from this method to cancel builtin behavior, which is as follows:
*  
*  If a save happened because an event-canvas was moved or sized with the mouse, and any kind 
*  of error occurs, the event is 
*  restored to it's pre-save position - if this method specifically returns true, the 
*  errors are also sent to errorHandling, 
*  which will show them to the user in a simple error dialog.
*  
*  If a save happened because an event was edited in the Calendar.eventDialog or 
*  Calendar.eventEditor windows and a validation error occurs, the window is re-opened for 
*  editing with the validation errors visible. If any other kind of error occurs, the window
*  will not be re-opened and, if this method specifically returns true, the error will 
*  be sent to errorHandling which will show a 
*  simple error dialog to the end user. 
*  
*  In all cases, if the server returned an updated record despite the save failing (eg, perhaps 
*  the server enforces optimistic locking and the underlying record has been changed by another 
*  user), it is assumed to be a latest-version of the record and is applied instead of the 
*  original record during those builtin UI processes. So, the Calendar.eventEditor and 
*  Calendar.eventDialog will show any changes from the latest-record. If the 
*  save was caused by dragging an event and the event's Calendar.startDateField or 
*  Calendar.endDateField dates are different in the latest-record, 
*  the EventCanvas will be positioned using the latest values, instead of returning to 
*  it's pre-save position on-screen.
*  
* 
* @param {DSResponse} DSResponse from the save-attempt
* @param {DSRequest} initiating DSRequest Properties
* @param {CalendarSaveScenario} scenario in which the save occurred
*/
eventSaveError(dsResponse:DSResponse, dsRequest:DSRequest, scenarioCode:CalendarSaveScenario): boolean;

/**
*  Save an event to this Calendar's ${isc.DocUtils.linkForRef('attr:Calendar.dataSource','dataSource) or \n ${isc.DocUtils.linkForRef('attr:Calendar.data','data array\')')}. To deal with errors while saving, see 
*  Calendar.eventSaveError
* 
* 
* @param {CalendarEvent} The event object that will be updated
* @param {CalendarEvent} The new attributes for the event
* @param {object} new values of additional fields to be updated
*/
updateCalendarEvent(event:CalendarEvent, newEvent:CalendarEvent, otherFields:object): void;

/**
*  Refreshes the passed event in the current view.
* 
* 
* @param {CalendarEvent} The event object to refresh in the current view
*/
refreshEvent(event:CalendarEvent): void;

/**
*  Update the styleName for the passed event. Refreshes the event's canvas in the current view.
* 
* 
* @param {CalendarEvent} The event object to refresh in the current view
* @param {CSSStyleName} The new CSS style to apply to the canvases showing this event
*/
setEventStyle(event:CalendarEvent, styleName:CSSStyleName): void;

/**
*  Gets the hover HTML for an event being hovered over. Override here to return custom 
*  HTML based upon the parameter event object.
* 
* 
* @param {CalendarEvent} The event being hovered
* @param {EventCanvas} the event canvas being hovered over
* @param {CalendarView} the CalendarView in which the eventCanvas lives
* @param {string=} the default HTML to show when hovering over the passed event
*/
getEventHoverHTML(event:CalendarEvent, eventCanvas:EventCanvas, view:CalendarView, defaultValue?:string): HTMLString;

/**
*  Gets the hover HTML for a Calendar.zones being hovered over. Override here to 
*  return custom HTML based upon the parameter zone object.
* 
* 
* @param {CalendarEvent} The zone being hovered over
* @param {ZoneCanvas} the zone canvas being hovered over
* @param {CalendarView} the CalendarView in which the zoneCanvas is displayed
* @param {string} the default HTML to show when hovering over the passed Zone
*/
getZoneHoverHTML(zone:CalendarEvent, zoneCanvas:ZoneCanvas, view:CalendarView, defaultValue:string): HTMLString;

/**
*  Called whenever an IndicatorCanvas is clicked in the 
*  Calendar.timelineView. There is no default implementation.
* 
* 
* @param {CalendarEvent} indicator that was clicked on
* @param {ViewName} view where the event's canvas was clicked
*/
indicatorClick(indicatorEvent:CalendarEvent, viewName:ViewName): boolean;

/**
*  Gets the hover HTML for an Calendar.indicators being hovered over. 
*  Override here to return custom HTML based upon the parameter indicator object.
* 
* 
* @param {CalendarEvent} The indicator being hovered over
* @param {IndicatorCanvas} the indicator canvas being hovered over
* @param {CalendarView} the CalendarView in which the indicatorCanvas is displayed
* @param {string} the default HTML to show when hovering over the passed Indicator
*/
getIndicatorHoverHTML(indicator:CalendarEvent, indicatorCanvas:IndicatorCanvas, view:CalendarView, defaultValue:string): HTMLString;

/**
*  Returns the hover HTML for the cell at the passed co-ordinates in the passed view. By 
*  default, the hover text is the snap date closest to the mouse, if the cell being hovered is 
*  a normal date cell - otherwise, it is the title of the Calendar.laneFields
*  being hovered over.
*  
*  Override here to return custom HTML for the passed cell.
* 
* 
* @param {CalendarView} the CalendarView the mouse is hovered over
* @param {Record} The record containing the cell being hovered
* @param {Integer} The rowNum of the cell being hovered
* @param {Integer} the colNum of the cell being hovered
* @param {Date} the snap-date at the mouse, which may be different from the result of a           call to Calendar.getCellDate
* @param {string} the default hover text for the passed values
*/
getCellHoverHTML(view:CalendarView, record:Record, rowNum:Integer, colNum:Integer, date:Date, defaultValue:string): HTMLString;

/**
*  Returns the hover HTML to show in a hover when the mouse moves over the header area.
* 
* 
* @param {CalendarView} the CalendarView the mouse is hovered over
* @param {HeaderLevel} the header level hovered over
* @param {Date} the start date of the span being hovered over
* @param {Date} the end date of the span being hovered over
* @param {string} the default text for the passed header level and date range
*/
getHeaderHoverHTML(view:CalendarView, headerLevel:HeaderLevel, startDate:Date, endDate:Date, defaultValue:string): HTMLString;

/**
*  Switches the various levels of Calendar.showViewHovers on or off at runtime.
* 
* 
* @param {boolean} whether to allow CalendarViews to show hovers
*/
setShowViewHovers(showViewHovers:boolean): void;

/**
*  Returns the HTML to show in a hover when an existing event is dragged, or when a new event
*  is being created by dragging with the mouse.
* 
* 
* @param {CalendarView} the CalendarView the mouse is hovered over
* @param {CalendarEvent} the CalendarEvent attached to the EventCanvas being dragged
* @param {string} the default text for the passed values
*/
getDragHoverHTML(view:CalendarView, event:CalendarEvent, defaultValue:string): HTMLString;

/**
*  Should a component be applied to the passed EventCanvas in the 
*  EventCanvas.calendarView in which it appears? Return false from this method to 
*  override the global value of 
*  Calendar.showEventCanvasComponents for this canvas.
* 
* @param {EventCanvas} should this eventCanvas get a component component?
*/
showEventCanvasComponent(canvas:EventCanvas): boolean;

/**
*  Called from EventCanvas.setEvent when Calendar.showEventCanvasComponents is 
*  true and an eventCanvas needs a component.  The method is expected to create and return a 
*  single component to apply as a space-filling member of the passed EventCanvas 
*  in the EventCanvas.calendarView in which it appears.
*  
*  By default, this method returns a DetailViewer showing values from the 
*  associated event, according to the fields in the Calendar's 
*  Calendar.dataSource, or the default event-fields if no dataSource is 
*  present.
*  
*  However, the component can be any Canvas widget, including a Layout 
*  containing an arrangement of other widgets. When applied as a member of an 
*  eventCanvas, the component fills its container and limits its own content to that size,
*  showing scrollbars as appropriate.
*  
*  See Calendar.updateEventCanvasComponent for details on updating components that have
*  already been created and applied.
* 
* @param {EventCanvas} the eventCanvas to get the component for
*/
createEventCanvasComponent(canvas:EventCanvas): Canvas;

/**
*  Called from EventCanvas.setEvent when Calendar.showEventCanvasComponents is 
*  true and the eventCanvas already has a 
*  Calendar.createEventCanvasComponent. This method is expected to update 
*  the passed component as necessary, based on the 
*  EventCanvas.event.
*  
*  By default, if the passed component has methods called 
*  setEvent or setData, those methods are called automatically.
* 
* @param {EventCanvas} the eventCanvas to update the component for
* @param {Canvas} the component to be updated the canvas in question
*/
updateEventCanvasComponent(canvas:EventCanvas, component:Canvas): void;

/**
*  Returns the CSSStyleName to use for the passed 
*  CalendarEvent, in the passed CalendarView. By default, returns
*  the style Calendar.eventStyleNameField, if one is specified - otherwise, 
*  in Calendar.lanes views, it returns the style specified on the 
*  Lane.eventStyleName, or the style specified on the 
*  Calendar.eventStyleName.
* 
* @param {CalendarEvent} the event to get the CSS style for
* @param {CalendarView=} the CalendarView that contains the canvas being styled
*/
getEventCanvasStyle(event:CalendarEvent, view?:CalendarView): CSSStyleName;

/**
*  If this method returns a value, it is expected to return an array of 
*  MenuItem applicable to the passed canvas and its event. If an array 
*  with valid entries is returned, the rollover
*  Calendar.eventCanvasContextButton is shown for the passed canvas.
* 
* @param {EventCanvas} the canvas to get menu items for
*/
getEventCanvasMenuItems(canvas:EventCanvas): Array<Partial<MenuItem>>;

/**
*  Selects a single event in the current view, showing it in a selected style and deselecting 
*  any other selected events.
* 
* @param {CalendarEvent} the event to select
*/
selectSingleEvent(event:CalendarEvent): boolean;

/**
*  Adds an event to the list of selected events in the current view, showing it in a selected 
*  style.
* 
* @param {CalendarEvent} the event to add to the selection
*/
selectEvent(event:CalendarEvent): boolean;

/**
*  Adds one or more events to the list of selected events in the current view, showing them in 
*  a selected style.
* 
* @param {Array<Partial<CalendarEvent>>} the events to add to the selection
*/
selectEvents(events:Array<Partial<CalendarEvent>>): boolean;

/**
*  Removes an event from the list of selected events in the current view, clearing its selected 
*  style.
* 
* @param {CalendarEvent} the event to deselect
*/
deselectEvent(event:CalendarEvent): boolean;

/**
*  Removes one or more events from the list of selected events in the current view, clearing
*  their selected styles.
* 
* @param {Array<Partial<CalendarEvent>>} the events to deselect
*/
deselectEvents(events:Array<Partial<CalendarEvent>>): boolean;

/**
*  Returns the currently selected CalendarEvent, or the first one if more 
*  than one is selected.
* 
*/
getSelectedEvent(): CalendarEvent;

/**
*  Returns the currently selected list of CalendarEvent.
* 
*/
getSelectedEvents(): Array<Partial<CalendarEvent>>;

/**
*  Returns the Calendar.eventCanvasGripperIcon to use as the gripper for
*  the passed event canvas.
* 
* @param {EventCanvas} the canvas that will show the gripper
*/
getEventCanvasGripperIcon(canvas:EventCanvas): SCImgURL;

/**
*  Sets the Calendar.zones used to highlight areas of this calendar.
* 
* 
* @param {Array<Partial<CalendarEvent>>} array of zones to display
*/
setZones(zones:Array<Partial<CalendarEvent>>): void;

/**
*  Adds a new Calendar.zones to the calendar.
* 
* 
* @param {CalendarEvent} a new zone to add to the calendar
*/
addZone(zone:CalendarEvent): void;

/**
*  Removes a Calendar.zones from the calendar.
*  
*  Accepts either a CalendarEvent or a string that represents the 
*  CalendarEvent.name of a zone.
* 
* 
* @param {CalendarEvent | string} either the actual CalendarEvent representing the zone,         or the name of the zone to remove
*/
removeZone(zone:CalendarEvent | string): void;

/**
*  Called whenever a ZoneCanvas is clicked in the 
*  Calendar.timelineView. There is no default implementation.
* 
* 
* @param {CalendarEvent} zone that was clicked on
* @param {ViewName} view where the event's canvas was clicked
*/
zoneClick(zoneEvent:CalendarEvent, viewName:ViewName): boolean;

/**
*  Returns the CSSStyleName to use for the passed 
*  Calendar.zones, in the passed CalendarView. By default, 
*  returns the style Calendar.eventStyleNameField, if one is specified,
*  or the style specified on the Calendar.zoneStyleName otherwise.
* 
* @param {CalendarEvent} the zone to get the CSS style for
* @param {CalendarView=} the CalendarView that contains the canvas being styled
*/
getZoneCanvasStyle(zone:CalendarEvent, view?:CalendarView): CSSStyleName;

/**
*  Returns the CSSStyleName to use for the passed 
*  Calendar.indicators, in the passed CalendarView. By default, 
*  returns the style Calendar.eventStyleNameField, if one is specified,
*  or the style specified on the Calendar.indicatorStyleName otherwise.
* 
* @param {CalendarEvent} the indicator to get the CSS style for
* @param {CalendarView=} the CalendarView that contains the canvas being styled
*/
getIndicatorCanvasStyle(indicator:CalendarEvent, view?:CalendarView): CSSStyleName;

/**
*  Sets the Calendar.indicators used to highlight instants in time.
* 
* @param {Array<Partial<CalendarEvent>>} array of indicators to display
*/
setIndicators(indicators:Array<Partial<CalendarEvent>>): void;

/**
*  Adds a new Calendar.indicators to the calendar.
* 
* @param {CalendarEvent} a new indicator to add to the calendar
*/
addIndicator(indicator:CalendarEvent): void;

/**
*  Removes a Calendar.indicators from the calendar.
*  
*  Accepts either a CalendarEvent or a string that represents the 
*  CalendarEvent.name of anindicator.
* 
* @param {CalendarEvent | string} either the actual CalendarEvent representing the          indicator, or the name of the indicator to remove
*/
removeIndicator(indicator:CalendarEvent | string): void;

/**
*  Set the current date for which the calendar will display events.
* 
* 
* @param {Date} the new date to set as the current date
*/
setChosenDate(newDate:Date): void;

/**
*  Should the parameter date be considered a workday? By default this method tries to find the
*  parameter date day in Calendar.workdays, and returns true if found. Override this 
*  method to provide custom logic for determining workday, perhaps returning false on holidays.
*  
*  Note that, when showing Calendar.showDayLanes in the 
*  Calendar.dayView, this method is also passed the name of the associated lane.
* 
* 
* @param {Date} date to check for being a workday
* @param {string} the name of the lane if Calendar.showDayLanes is true, null otherwise
*/
dateIsWorkday(date:Date, laneName:string): boolean;

/**
*  Move to the next day, week, or month, depending on which tab is selected.
* 
* 
*/
next(): void;

/**
*  Move to the previous day, week, month, or timeline range depending on which tab is selected.
* 
* 
*/
previous(): void;

/**
*  For views that support Calendar.lanes, returns the padding to apply to events
*  rendered in lanes in the passed or current view. By default, returns 
*  Calendar.laneEventPadding.
* 
* 
* @param {CalendarView=} the view to get the lane padding for
*/
getLanePadding(view?:CalendarView): Integer;

/**
*  For views that support Calendar.lanes, updates the title for the passed lane.
* 
* 
* @param {string} the name of the lane to change the title for
* @param {string} the new title to apply
*/
setLaneTitle(lane:string, title:string): boolean;

/**
*  For views that support Calendar.lanes, returns the array of events in the
*  current dataset that apply to the passed lane in the passed or current view.
* 
* 
* @param {Lane | string} lane object or name to get the events for
* @param {CalendarView=} the view in which the passed lane lives - uses the selected                view if unset
*/
getLaneEvents(lane:Lane | string, view?:CalendarView): Array<Partial<CalendarEvent>>;

/**
*  For views that support Calendar.lanes and allow 
*  Calendar.useSublanes, returns the array of events in the
*  current dataset that apply to the passed lane and sublane in the passed or current view.
* 
* 
* @param {Lane | string} lane object or name to get the events for
* @param {Lane | string} sublane object or name to get the events for
* @param {CalendarView=} the view in which the passed sublane lives - uses the selected                view if unset
*/
getSublaneEvents(lane:Lane | string, sublane:Lane | string, view?:CalendarView): Array<Partial<CalendarEvent>>;

/**
*  Returns the text to display between the navigation buttons above the Calendar - indicates 
*  the visible date range.
* 
* @param {string} one of "day", "week", "month" or "timeline"
* @param {Date} the start of the visible date range
* @param {Date=} the optional end of the visible date range
*/
getDateLabelText(viewName:string, startDate:Date, endDate?:Date): string;

/**
*  Cancels the current edit-session, closing the builtin event 
*  Calendar.eventDialog or Calendar.eventEditor and clearing any 
*  visible edit-selection from the Calendar.getSelectedView.
*  
* 
*/
cancelEditing(): void;

/**
*  Open the Quick Event dialog showing minimal information about an existing 
*  CalendarEvent.
*  
*  The Calendar.startDateField field on the event is used to calculate the 
*  display location for the dialog.
*  
*  If this method is called when the Event Dialog is already showing another event, and if 
*  changes have been made, a confirmation dialog is displayed and editing of the new event 
*  is cancelled unless confirmed.
*  
*  You can override this method to prevent the default action, perhaps instead showing a custom 
*  interface that performs validations or gathers custom data before making a call to 
*  Calendar.addCalendarEvent or 
*  Calendar.updateCalendarEvent when the new data is available.
*  
* 
* @param {CalendarEvent=} the event to show in the Editor
* @param {boolean=} optional boolean indicating that this is a new event, event if                an event is passed - used to pass defaults for a new event
*/
showEventDialog(event?:CalendarEvent, isNewEvent?:boolean): void;

/**
*  Open the Quick Event dialog to begin editing a new CalendarEvent.
*  
*  If passed, the event parameter is used as defaults for the new event - in addition, the 
*  event's Calendar.startDateField, and its 
*  Calendar.laneNameField, for timeline events, are used to calculate the 
*  display location for the dialog.
*  
*  If this method is called when the Event Dialog is already showing another event, and if 
*  changes have been made, a confirmation dialog is displayed and editing of the new event 
*  is cancelled unless confirmed.
*  
*  You can override this method to prevent the default action, perhaps instead showing a custom 
*  interface that performs validations or gathers custom data before making a call to
*  Calendar.addCalendarEvent or 
*  Calendar.updateCalendarEvent when the new data is available.
*  
* 
* @param {CalendarEvent=} defaults for the new event
*/
showNewEventDialog(event?:CalendarEvent): void;

/**
*  Show an Event Editor for the passed event. Event Editor's fill the Calendar and allow 
*  for editing of the built-in Event fields, like Calendar.nameField and 
*  Calendar.descriptionField, as well as any 
*  custom fields supplied via Calendar.eventEditorFields.
*  
*  If isNewEvent is true, a new event is created - in this case, if an event is passed, it
*  represents default values to apply to the new event.
*  
*  You can override this method to prevent the default action, perhaps instead showing a custom 
*  interface that performs validations or gathers custom data before making a call to 
*  Calendar.addCalendarEvent or 
*  Calendar.updateCalendarEvent when the new data is available.
*  
* 
* @param {CalendarEvent=} an existing event to show in the Editor
* @param {boolean=} optional boolean indicating that this is a new event, even if                an event is passed - used to pass defaults for a new event
*/
showEventEditor(event?:CalendarEvent, isNewEvent?:boolean): void;

/**
*  Show an Event Editor for a new event. If an CalendarEvent is passed as the 
*  parameter, it is used as defaults for the new event.
*  
* 
* @param {CalendarEvent=} defaults for the new event to show in the Editor
*/
showNewEventEditor(event?:CalendarEvent): void;

/**
*  Return CSS text for styling the cell associated with the passed date and/or rowNum & colNum,
*  which will be applied in addition to the CSS class for the cell, as overrides.
*  
*  "CSS text" means semicolon-separated style settings, suitable for inclusion in a CSS
*  stylesheet or in a STYLE attribute of an HTML element.
* 
* 
* @param {Date} the date to return CSS text for
* @param {Integer} the row number containing the date to get the CSS for
* @param {Integer} the column number containing the date to get the CSS for
* @param {CalendarView} the relevant CalendarView
*/
getDateCSSText(date:Date, rowNum:Integer, colNum:Integer, view:CalendarView): string;

/**
*  Return the CSS styleName for the associated date-cell in the passed view.
* 
* 
* @param {Date} the date to return the CSS styleName for
* @param {Integer} the row number containing the date to get the CSS styleName for
* @param {Integer} the column number containing the date to get the CSS styleName for
* @param {CalendarView} the relevant CalendarView
*/
getDateStyle(date:Date, rowNum:Integer, colNum:Integer, view:CalendarView): CSSStyleName;

/**
*  Return the HTML to be displayed in the associated date-cell in the passed view.
* 
*  Note that the Calendar.monthView has default cell HTML, controlled via
*  Calendar.getDayBodyHTML.
* 
* 
* @param {Date} the date to get the HTML for
* @param {Integer} the row number containing the date to get the HTML for
* @param {Integer} the column number containing the date to get the HTML for
* @param {CalendarView} the relevant CalendarView
*/
getDateHTML(date:Date, rowNum:Integer, colNum:Integer, view:CalendarView): HTMLString;

/**
*  When Calendar.getDateHTML returns a value, this method returns the
*  horizontal alignment for that value in its cell, in the passed view.
*  
* 
* @param {Date} the date to get the cell-alignment for
* @param {Integer} the row number containing the date to get the cell-alignment for
* @param {Integer} the column number containing the date to get the cell-alignment for
* @param {CalendarView} the relevant CalendarView
*/
getDateCellAlign(date:Date, rowNum:Integer, colNum:Integer, view:CalendarView): HTMLString;

/**
*  When Calendar.getDateHTML returns a value, this method returns the
*  vertical alignment for that value in its cell, in the passed view.
*  
* 
* @param {Date} the date to get the cell-alignment for
* @param {Integer} the row number containing the date to get the cell-alignment for
* @param {Integer} the column number containing the date to get the cell-alignment for
* @param {CalendarView} the relevant CalendarView
*/
getDateCellVAlign(date:Date, rowNum:Integer, colNum:Integer, view:CalendarView): HTMLString;

/**
*  Return the title text to display in the header-button of the ListGridField showing the 
*  passed date, in the passed view.
* 
* 
* @param {Date} the date to return the header-title for - note that the           Calendar.monthView does not pass this parameter           because a single column represents multiple dates
* @param {Integer} the week-day number of the passed date, except for the              Calendar.monthView, where no date is passed,              because the week-day number represents multiple dates.
* @param {string} the default header-title for the passed date and view
* @param {CalendarView} the relevant CalendarView
*/
getDateHeaderTitle(date:Date, dayOfWeek:Integer, defaultValue:string, view:CalendarView): string;

/**
*  Return the Date instance associated with the passed co-ordinates in the passed or selected
*  view. If the cell at the passed co-ordinates is not a date-cell, returns null. If rowNum 
*  and colNum are both unset, returns the date from the cell under the mouse.
*  
*  To determine the date at a more specific point within a cell, see Calendar.getDateFromPoint.
* 
* 
* @param {Integer=} the row number to get the date for
* @param {Integer=} the column number to get the date for
* @param {CalendarView=} the view to use - uses the selected view if not passed
*/
getCellDate(rowNum?:Integer, colNum?:Integer, view?:CalendarView): Date;

/**
*  Returns a Date instance representing the point at the passed offsets into the body of the 
*  current view.
*  
*  If snapOffsets is passed as false, returns the date representing the 
*  exact position of the passed offsets. If unset or passed as true, returns the date at the 
*  nearest eventSnapGap to the left, for Timelines, or above for Calendar.dayView 
*  and Calendar.weekView views.
*  
*  If neither x nor y offsets are passed, assumes them from the last mouse event.
*  
*  If the cell at the eventual offsets is not a date-cell, returns null.
*  
*  Note that, for the Calendar.monthView, this method is functionally equivalent to 
*  Calendar.getCellDate, which determines the date associated with a cell, without the additional 
*  offset precision offered here.
* 
* 
* @param {Integer=} the x offset into the body of the selected view - non-functional for            the Calendar.dayView. If this param and "y" are both unset,            assumes both offsets from the last mouse event.
* @param {Integer=} the y offset into the body of the selected view - non-functional for the              Calendar.timelineView. If this param and "x" are               both unset, assumes both offsets from the last mouse event.
* @param {boolean=} whether to snap the offsets to the nearest eventSnapGap - if                 unset, the default is true
* @param {CalendarView=} the view to use - or the selected view if not passed
*/
getDateFromPoint(x?:Integer, y?:Integer, snapOffsets?:boolean, view?:CalendarView): Date;

/**
*  Returns the Lane with the passed name, in the passed view
* 
* @param {string} the name of the lane to return
* @param {CalendarView=} the view to get the lane object from
*/
getLane(lane:string, view?:CalendarView): Lane;

/**
*  Returns the Lane associated with the passed event, in the passed view
* 
* @param {CalendarEvent} the event to get the lane for
* @param {CalendarView=} the view to get the lane object from
*/
getEventLane(event:CalendarEvent, view?:CalendarView): Lane;

/**
*  Returns the Lane.sublanes with the passed name, from the Lane
*  with the passed name, in the passed view.
* 
* @param {string} the name of the lane containing the sublane to return
* @param {string} the name of the sublane to return
* @param {CalendarView=} the view to get the sublane object from
*/
getSublane(lane:string, sublane:string, view?:CalendarView): Lane;

/**
*  Returns the Lane.sublanes associated with the passed event, in the passed view
* 
* @param {CalendarEvent} the event to get the sublane for
* @param {CalendarView=} the view to get the sublane object from
*/
getEventSublane(event:CalendarEvent, view?:CalendarView): Lane;

/**
*  Returns the Lane at the passed co-ordinates. To get the lane under the mouse, pass
*  null for both x and y.
* 
* @param {Integer=} the x offset into the body of the selected view
* @param {Integer=} the y offset into the body of the selected view. If this param and "x" are               both unset, assumes both offsets from the last mouse event.
* @param {CalendarView=} the view to get the lane from - selected view if not passed
*/
getLaneFromPoint(x?:Integer, y?:Integer, view?:CalendarView): Lane;

/**
*  Returns the Lane.sublanes at the passed co-ordinates. To get the sublane under
*  the mouse, pass null for both x and y.
* 
* @param {Integer=} optional x offset into the body of the selected view
* @param {Integer=} optional y offset into the body of the selected view. If this param and "x" are               both unset, assumes both offsets from the last mouse event.
* @param {CalendarView=} the view to get the sublane from - selected view if not passed
*/
getSublaneFromPoint(x?:Integer, y?:Integer, view?:CalendarView): Lane;

/**
*  Notification that fires whenever the current view changes via the 
*  Calendar.mainView.
* 
* 
* @param {ViewName} the name of the current view after the change
*/
currentViewChanged(viewName:ViewName): void;

/**
*  Return the HTML to be shown in the body of a day in the month view.
*  
*  Default is to render a series of links that call Calendar.eventClick to provide details
*  and/or an editing interface for the events.
*  
*  getDayBodyHTML() is not called for days outside of the current month if
*  Calendar.showOtherDays is false.
* 
* 
* @param {Date} JavaScript Date object representing this day
* @param {Array<Partial<CalendarEvent>>} events that fall on this day
* @param {Calendar} the calendar itself
* @param {Integer} the row number to which the parameter date belongs
* @param {Integer} the column number to which the parameter date belongs
*/
getDayBodyHTML(date:Date, events:Array<Partial<CalendarEvent>>, calendar:Calendar, rowNum:Integer, colNum:Integer): HTMLString;

/**
*  This method returns the hover HTML to be displayed when the user hovers over a cell
*  displayed in the calendar month view tab.
*  
*  Default implementation will display a list of the events occurring on the date the user is
*  hovering over. Override for custom behavior. Note that returning null will suppress the
*  hover altogether.
* 
* 
* @param {Date} Date the user is hovering over
* @param {Array<Partial<CalendarEvent>>} array of events occurring on the current date. May be empty.
*/
getMonthViewHoverHTML(date:Date, events:Array<Partial<CalendarEvent>>): HTMLString;

/**
*  Called when the body area of a day in the month view is clicked on, outside of any links
*  to a particular event.
*  
*  By default, if the user can add events, shows a dialog for adding a new event for that
*  day. Return false to cancel this action.
*  
*  Note that, when Calendar.otherDayClickNavigation is true, the calendar will first 
*  navigate the Month view to the selected month, before showing the Event Editor dialog at 
*  the proper location.
*  
*  Not called if the day falls outside the current month and Calendar.showOtherDays is false.
* 
* 
* @param {Date} JavaScript Date object representing this day
* @param {Array<Partial<CalendarEvent>>} events that fall on this day
* @param {Calendar} the calendar itself
* @param {Integer} the row number to which the parameter date belongs
* @param {Integer} the column number to which the parameter date belongs
*/
dayBodyClick(date:Date, events:Array<Partial<CalendarEvent>>, calendar:Calendar, rowNum:Integer, colNum:Integer): boolean;

/**
*  Called when the header area of a day in the month view is clicked on.
*  
*  By default, moves to the day tab and shows the clicked days events.
*  Return false to cancel this action.
*  
*  Not called if the day falls outside the current month and Calendar.showOtherDays is false.
* 
* 
* @param {Date} JavaScript Date object representing this day
* @param {Array<Partial<CalendarEvent>>} events that fall on this day
* @param {Calendar} the calendar itself
* @param {Integer} the row number to which the parameter date belongs
* @param {Integer} the column number to which the parameter date belongs
*/
dayHeaderClick(date:Date, events:Array<Partial<CalendarEvent>>, calendar:Calendar, rowNum:Integer, colNum:Integer): boolean;

/**
*  Notification fired whenever a user changes an event, whether by dragging the event or by
*  editing it in a dialog.
*  
*  In a calendar with a DataSource, eventChanged() fires after the updated event has
*  been successfully saved to the server
* 
* 
* @param {CalendarEvent} the event that changed
*/
eventChanged(event:CalendarEvent): void;

/**
*  Notification fired whenever a user removes an event.
*  
*  In a calendar with a DataSource, eventRemoved() fires after the event has
*  been successfully removed from the server
* 
* 
* @param {CalendarEvent} the event that was removed
*/
eventRemoved(event:CalendarEvent): void;

/**
*  Notification fired whenever a user adds an event.
*  
*  In a calendar with a DataSource, eventAdded() fires after the event has
*  been successfully added at the server
* 
* 
* @param {CalendarEvent} the event that was added
*/
eventAdded(event:CalendarEvent): void;

/**
*  Called whenever an event is clicked on in the day, week or month views.
*  
*  By default, a dialog appears showing details for the event, and offering the ability to
*  edit events that can be edited. Return false to cancel the default action. This is a good
*  place to, for example, show a completely customized event dialog instead of the default one.
* 
* 
* @param {CalendarEvent} event that was clicked on
* @param {ViewName} view where the event's canvas was clicked
*/
eventClick(event:CalendarEvent, viewName:ViewName): boolean;

/**
*  Called whenever the close icon of an EventCanvas is clicked in the 
*  Calendar.dayView, Calendar.weekView and Calendar.timelineView views, or when
*  the Calendar.removeButton is pressed in the Calendar.eventEditor.
*  
*  Implement this method to intercept the automatic removal of data. You can return false to 
*  prevent the default action (calling Calendar.removeEvent) and instead 
*  take action of your own. For example, returning false from this method and then showing a 
*  custom confirmation dialog - if the user cancels, do nothing, otherwise
*  make a call to Calendar.removeEvent, passing the event.
* 
* 
* @param {CalendarEvent} event that was clicked on
* @param {string} view where the event was clicked on: "day", "week" or "month"
*/
eventRemoveClick(event:CalendarEvent, viewName:string): boolean;

/**
*  Called when an event is moved via dragging by a user. Return false to disallow the move.
* 
* @param {Date} new start date and time that the event is being moved to
* @param {CalendarEvent} the event as it will be after this movement
* @param {string} the name of the lane into which the event was moved
* @deprecated * in favor of Calendar.eventRepositionStop

*/
eventMoved(newDate:Date, event:CalendarEvent, newLane:string): boolean;

/**
*  Called when an event is resized with the mouse. The passed date value is the new
*  *end* date for the event, since resizing can only be performed on the bottom edge of an event
*  in normal calendar views.
* 
* @param {Date} new end date and time that event is being resized to
* @param {CalendarEvent} the event as it will be after this resize
* @deprecated * in favor of Calendar.eventResizeStop

*/
eventResized(newDate:Date, event:CalendarEvent): boolean;

/**
*  Called when a Timeline event is moved via dragging by a user. Return false to disallow the 
*  move.
* 
* @param {CalendarEvent} the event that was moved
* @param {Date} new start date of the passed event
* @param {Date} new end date of the passed event
* @param {Lane} the Lane in which this event has been dropped
* @deprecated * in favor of Calendar.eventRepositionStop

*/
timelineEventMoved(event:CalendarEvent, startDate:Date, endDate:Date, lane:Lane): boolean;

/**
*  Called when a Timeline event is resized via dragging by a user. Return false to disallow 
*  the resize.
* 
* @param {CalendarEvent} the event that was resized
* @param {Date} new start date of the passed event, after the resize
* @param {Date} new end date of the passed event, after the resize
* @deprecated * in favor of Calendar.eventResizeStop

*/
timelineEventResized(event:CalendarEvent, startDate:Date, endDate:Date): boolean;

/**
*  Selects the calendar view in the passed tab number.
* 
* 
* @param {number} the index of the tab to select
*/
selectTab(tabnum:number): void;

/**
*  Fires whenever the user changes the current date, including picking a specific date or
*  navigating to a new week or month.
* 
*/
dateChanged(): void;

/**
*  Gets the day of the week (0-6) that the mouse is currently over.
* 
* 
*/
getActiveDay(): Integer;

/**
*  Gets a date object representing the date over which the mouse is hovering for the current
*  selected view. For month view, the time will be set to midnight of the active day. For day
*  and week views, the time will be the rounded to the closest half hour relative to the mouse
*  position.
* 
*/
getActiveTime(): Date;

/**
*  Sets the range over which the timeline will display events.
*  
*  If the end parameter is not passed, the end date of the range will default to 
*  Calendar.defaultTimelineColumnSpan columns of the current 
*  Calendar.timelineGranularity following the start date.
*  
* 
* @param {Date} start of range
* @param {Date=} end of range
*/
setTimelineRange(start:Date, end?:Date): void;

/**
*  Reset the resolution, the header levels and scrollable range, of the timeline view.
*  
*  headerLevels specifies the array of HeaderLevel to show above 
*  the timeline, and the unit and unitCount parameters dictate the 
*  scrollable range (eg, passing "week" and 6 will create a timeline with a scrollable range of 
*  six weeks, irrespective of the number of columns that requires, according to the 
*  Calendar.timelineGranularity).
*  
*  If the optional granularityPerColumn parameter is passed, each column will span
*  that number of units of the granularity, which is determined from the unit of the innermost
*  of the passed headerLevels. For example, to show a span of 12 hours with inner columns that 
*  each span 15 minutes, you could pass "hour" and "minute" -based headerLevels, unit and 
*  unitCount values of "hour" and 12 respectively, and granularityPerColumn of 15.
*  
* 
* @param {Array<Partial<HeaderLevel>>} the header levels to show in the timeline
* @param {TimeUnit} the time unit to use when calculating the range of the timeline
* @param {Integer} the count of the passed unit that the timeline should span
* @param {Integer=} how many units of the granularity (the unit of the      innermost headerLevel) should each column span? The default is 1.
*/
setResolution(headerLevels:Array<Partial<HeaderLevel>>, unit:TimeUnit, unitCount:Integer, granularityPerColumn?:Integer): void;

/**
*  Returns the length of the passed CalendarEvent in the passed 
*  TimeUnit. If unit isn't passed, returns the length of the event 
*  in milliseconds.
*  
* 
* @param {CalendarEvent} the event to get the length of
* @param {TimeUnit=} the time unit to return the length in, milliseconds if not passed
*/
getEventLength(event:CalendarEvent, unit?:TimeUnit): void;

/**
*  Notification called whenever the drop position of an event being drag-moved changes.
*  
*  The newEvent parameter represents the event as it will be after the move, 
*  including CalendarEvent.startDate and CalendarEvent.endDate dates
*  and CalendarEvent.lane and CalendarEvent.sublane where 
*  applicable.
*  
*  Return false to prevent the default action, of positioning the drag canvas to the newEvent.
*  
* 
* @param {CalendarEvent} the event that's being moved
* @param {CalendarEvent} the event as it would be if dropped now
*/
eventRepositionMove(event:CalendarEvent, newEvent:CalendarEvent): boolean;

/**
*  Notification called when an event being drag-moved is dropped.
*  
*  The newEvent parameter represents the event as it will be after the move, 
*  including CalendarEvent.startDate and CalendarEvent.endDate dates
*  and CalendarEvent.lane and CalendarEvent.sublane where 
*  applicable.
*  
*  Return false to prevent the default action, of actually 
*  Calendar.updateCalendarEvent the event.
*  
* 
* @param {CalendarEvent} the event that's about to be moved
* @param {CalendarEvent} the event as it will be, unless this method returns false
* @param {object=} additional custom values associated with the event
*/
eventRepositionStop(event:CalendarEvent, newEvent:CalendarEvent, customValues?:object): boolean;

/**
*  Notification called on each resize during an event drag-resize operation.
*  
*  The newEvent parameter represents the event as it will be after the resize.
*  
*  Return false to prevent the default action, of resizing the drag canvas to the newEvent.
*  
* 
* @param {CalendarEvent} the event that's being drag-resized
* @param {CalendarEvent} the event as it would be if dropped now
*/
eventResizeMove(event:CalendarEvent, newEvent:CalendarEvent): boolean;

/**
*  Notification called when an event drag-resize operation completes.
*  
*  The newEvent parameter represents the event as it will be after the move.
*  
*  Return false to prevent the default action, of actually 
*  Calendar.updateCalendarEvent the event.
*  
* 
* @param {CalendarEvent} the event that's about to be resized
* @param {CalendarEvent} the event as it will be, unless this method returns false
* @param {object=} additional custom values associated with the event
*/
eventResizeStop(event:CalendarEvent, newEvent:CalendarEvent, customValues?:object): boolean;

/**
*  Callback fired when the mouse is clicked in a background-cell, ie, one without an 
*  event.
* 
* 
* @param {Date} start datetime of the selected slot
* @param {Date} end datetime of the selected slot
*/
backgroundClick(startDate:Date, endDate:Date): boolean;

/**
*  Callback fired when the mouse button is depressed over a background-cell, ie, one 
*  without an event. Return false to cancel the default behavior of allowing sweep
*  selection via dragging.
* 
* 
* @param {Date} start datetime of the selected slot
*/
backgroundMouseDown(startDate:Date): boolean;

/**
*  Notification method fired when the mouse button is released over a background-cell, ie, 
*  one without an event. Return false to cancel the default behavior of showing a dialog
*  to add a new event with the passed dates.
* 
* 
* @param {Date} the datetime of the slot where the mouse button was depressed
* @param {Date} the datetime of the slot where the mouse button was released
*/
backgroundMouseUp(startDate:Date, endDate:Date): boolean;



}
/**
* Creates a new Calendar
* 
* @param typeCheckedProperties {Partial<Calendar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Calendar} The new instance
*/
class Calendar{
static create(typeCheckedProperties? : Partial<Calendar>, uncheckedProperties?: Object): Calendar;







}

/**
* CalendarView is a base class, extended by the various views available in a 
*  Calendar.
*/
interface CalendarView extends ListGrid {
/**
* The Calendar this view is in.
* @type {Calendar}
* @default null
*/
calendar?: Calendar;

/**
* The name of this view, used to identify it in the CalendarView.calendar.
* @type {string}
* @default null
*/
viewName?: string;

/**
* Should EventCanvas instances be reused when visible events change?
* @type {boolean}
* @default true
*/
useEventCanvasPool?: boolean;

/**
* If specified, overrides Calendar.eventStyleName and dictates the CSS style to 
*  use for events rendered in this view. Has no effect on events that already have a 
*  CalendarEvent.styleName.
* @type {CSSStyleName}
* @default null
*/
eventStyleName?: CSSStyleName;

/**
* CSS class applied to the CalendarView.eventDragCanvas.
* @type {CSSStyleName}
* @default "eventDragCanvas"
*/
eventDragCanvasStyleName?: CSSStyleName;

/**
* Canvas displayed while dragging or resizing an event in this view and styled
*  according to CalendarView.eventDragCanvasStyleName.
* @type {Canvas}
* @default null
*/
eventDragCanvas?: Canvas;



/**
*  Returns true if this view is the currently selected view in the parent calendar.
* 
*/
isSelectedView(): boolean;

/**
*  Returns true if this is the Calendar.timelineView, false otherwise.
* 
*/
isTimelineView(): boolean;

/**
*  Returns true if this is the Calendar.dayView, false otherwise.
* 
*/
isDayView(): boolean;

/**
*  Returns true if this is the Calendar.weekView, false otherwise.
* 
*/
isWeekView(): boolean;

/**
*  Returns true if this is the Calendar.monthView, false otherwise.
* 
*/
isMonthView(): boolean;

/**
*  Rebuild this CalendarView, including re-fetching its data as necessary. To avoid 
*  re-fetching the data, pass 'false' to this method, or call 
*  CalendarView.refreshEvents instead.
* 
* @param {boolean=} If false, prevents data from bing refreshed.
*/
rebuild(refreshData?:boolean): void;

/**
*  Clear, recalculate and redraw the events for the current range, without causing a fetch.
* 
*/
refreshEvents(): void;

/**
*  Move the viewport of this CalendarView to the start of its scrollable range.
* 
*/
scrollToStart(): void;

/**
*  Move the viewport of this CalendarView to the end of its scrollable range.
* 
*/
scrollToEnd(): void;



}
/**
* Creates a new CalendarView
* 
* @param typeCheckedProperties {Partial<CalendarView>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {CalendarView} The new instance
*/
class CalendarView{
static create(typeCheckedProperties? : Partial<CalendarView>, uncheckedProperties?: Object): CalendarView;







}

/**
* This object cannot be used; it exists for documentation purposes only as a place to put
*  documentation for callback methods, such as the callback for Callbacks.DSCallback.
*/
interface Callbacks extends Class {


/**
*  A notification Callback fired by the TabIndexManager to allow 
*  application code to react to the numeric tab-index of some registered target being
*  modified.
* 
* @param {string} the ID String passed to TabIndexManager.addTarget when          the callback was registered.
*/
TabIndexUpdatedCallback(ID:string): void;

/**
*  A Callback fired by the TabIndexManager when application code or user
*  action attempts to synthetically shift focus to some registered target. 
*  See TabIndexManager.shiftFocus.
*  
*  A typical implementation will shift focus to some native element associated
*  with the registered target, or if this is not currently possible, return false.
* 
* 
* @param {string} The ID String passed to TabIndexManager.addTarget when          the callback was registered.
*/
ShiftFocusCallback(ID:string): boolean;

/**
*  Callback scheduled by MultiWindow.setEvent. The RemoteWindow may be
*  null if the associated browser window is unloading or closing.
*  
*  Note that the event is simply an
*  OpenFin application event
*  when OpenFin is present, but may not be fully populated in fallback mode without OpenFin.
* 
* @param {RemoteWindow} window affected by event, or null if not found
* @param {MultiWindowEvent} event type as passed to MultiWindow.setEvent
* @param {object} event data see MultiWindow.setEvent() see MultiWindow.clearEvent()
*/
MultiWindowEventCallback(remoteWindow:RemoteWindow, eventType:MultiWindowEvent, event:object): void;

/**
*  Callback reporting a change to a RemoteWindow, such as it moving.
* 
* @param {RemoteWindow} window affected
*/
RemoteWindowCallback(remoteWindow:RemoteWindow): void;

/**
*  Callback reporting a failure in a RemoteWindow operation.
* 
* @param {RemoteWindow} window affected
* @param {string} error message
*/
RemoteWindowErrorCallback(remoteWindow:RemoteWindow, errorMessage:string): void;

/**
*  Callback reporting the result of a boolean RemoteWindow operation.
* 
* @param {RemoteWindow} window affected
* @param {boolean} result
*/
RemoteWindowBooleanCallback(remoteWindow:RemoteWindow, result:boolean): void;

/**
*  Callback reporting the result of a RemoteWindow operation yielding a map.
* 
* @param {RemoteWindow} window affected
* @param {Map<any>} result
*/
RemoteWindowMapCallback(remoteWindow:RemoteWindow, result:Map<any>): void;

/**
*  Callback executed when a PrintCanvas.setHTML is rendered into
*  a printCanvas.
* 
* 
* @param {PrintCanvas} The canvas being printed.
*/
PrintCanvasCallback(printPreview:PrintCanvas): void;

/**
*  Callback required for the property DataSourceField.getFieldValue.
* 
* 
* @param {object | XMLElement} record object selected from web service response data                   by OperationBinding.recordXPath
* @param {any} default value derived by the method described in          DataSourceField.valueXPath
* @param {DataSourceField} DataSourceField definition
* @param {FieldName} name of the DataSource field
*/
GetFieldValueCallback(record:object | XMLElement, value:any, field:DataSourceField, fieldName:FieldName): void;

/**
*  Callback fired when DataSource methods that send DSRequests complete (such as DataSource.fetchData).
* 
* 
* @param {DSResponse} a DSResponse instance with metadata about the returned data
* @param {any} data returned to satisfy the DataSource request. See the     dataSourceOperations topic for expected results     for each type of DataSource operation
* @param {DSRequest} the DSRequest that was sent. You can use    DSRequest.clientContext to track state during the server turnaround.
*/
DSCallback(dsResponse:DSResponse, data:any, dsRequest:DSRequest): void;

/**
*  A Callback fired when DataSource.hasFile completes.
* 
* 
* @param {DSResponse} A DSResponse instance with metadata about the                returned data.
* @param {boolean} Whether the file exists.
* @param {DSRequest} The DSRequest that was sent.
*/
HasFileCallback(dsResponse:DSResponse, data:boolean, dsRequest:DSRequest): void;

/**
*  A Callback fired when Sound.load completes.
*  
* 
*/
CanPlayCallback(): void;

/**
*  A Callback fired when Sound.play completes.
*  
* 
*/
PlaybackCompleteCallback(): void;

/**
*  A Callback fired when DataSource.hasFileVersion completes.
* 
* 
* @param {DSResponse} A DSResponse instance with metadata about the                returned data.
* @param {boolean} Whether the file version exists.
* @param {DSRequest} The DSRequest that was sent.
*/
HasFileVersionCallback(dsResponse:DSResponse, data:boolean, dsRequest:DSRequest): void;

/**
*  Callback fired when DataSource.getFile completes.
* 
* 
* @param {DSResponse} A DSResponse instance with metadata about the                returned data.
* @param {string} The file contents, or null if there was an error (such as file                not found).
* @param {DSRequest} The DSRequest that was sent.
*/
GetFileCallback(dsResponse:DSResponse, data:string, dsRequest:DSRequest): void;

/**
*  Callback fired when DataSource.getFileVersion completes.
* 
* 
* @param {DSResponse} A DSResponse instance with metadata about the                returned data.
* @param {string} The file contents, or null if there was an error (such as file                not found).
* @param {DSRequest} The DSRequest that was sent.
*/
GetFileVersionCallback(dsResponse:DSResponse, data:string, dsRequest:DSRequest): void;

/**
*  Generic callback interface.
* 
* 
*/
Function(): void;

/**
*  Generic callback interface.
* 
* 
* @param {DataSource} Client only Data Source.
*/
ClientOnlyDataSourceCallback(clientOnly:DataSource): void;

/**
*  A Callback to evaluate when a screen is loaded via RPCManager.loadScreen.
* 
* 
* @param {Canvas=} The last top-level component loaded
* @param {RPCResponse=} * @param {Map<any>=} A collection of suppressed globals.
*/
LoadScreenCallback(screen?:Canvas, rpcResponse?:RPCResponse, suppressedGlobals?:Map<any>): void;

/**
*  A Callback to evaluate when an RPCRequest completes.
*  
*  Parameters passed to this callback are:
*  
*  
* - rpcResponse: an RPCResponse encapsulating the server response to your
*    request
*  
* - data: just the "data" property from the RPCResponse, for convenience
*  
* - rpcRequest: the RPCRequest that was sent. You can use
*    RPCRequest.clientContext to track state during the server turnaround.
*  
*  For example, to take the data returned by the server and display it in a previously created
*  ListGrid with the ID "myGrid":
*  
*    isc.RPCManager.send("getData", "myGrid.setData(data)");
*  
*  Or
*  
*    isc.RPCManager.send("getData", function (rpcResponse, data, rpcRequest) { 
*                     myGrid.setData(data)
*    });
*  
* 
* 
* @param {RPCResponse} response a RPCResponse encapsulating the server response to your request
* @param {any} rawData The "data" property from the RPCResponse, for convenience. The data can also be obtained via {@link RPCResponse#getDataAsMap()}, {@link RPCResponse#getDataAsString()}, or {@link RPCResponse#getDataAsObject()}, depending on the type of data that is expected to be returned from the server.
* @param {RPCRequest} the RPCRequest that was sent.
*/
RPCCallback(response:RPCResponse, rawData:any, request:RPCRequest): void;

/**
*  Callback to fire when a queue of requests sent via 
*  {@link com.smartgwt.client.rpc.RPCManager#sendQueue(RPCQueueCallback)} returns.
*  Note that the Array of RPCResponses passed to this callback
*  will actually be DSResponse objects for any requests that were actually 
*  DSRequests. 
*  
* 
* 
* @param {Array<Partial<RPCResponse>>} array of responses returned from the sent queue of requests
*/
RPCQueueCallback(response:Array<Partial<RPCResponse>>): void;

/**
*  Callback for RPCManager.exportImage.
* 
* @param {string} image data from the server, in base64 format
*/
ExportImageCallback(imageData:string): void;

/**
*  A Callback to evaluate after RPCManager.loadProject completes.
*  
*  If LoadProjectSettings.willHandleError is set, the callback will
*  fire even if the requested projects could not be retrieved. You can call
*  RPCManager.getLoadProjectErrorStatus or
*  RPCManager.getLoadProjectErrorMessage in this case for more information.
* 
* 
* @param {Project} the first project loaded
* @param {Array<Partial<Project>>} array of all projects loaded
* @param {RPCResponse} server response
*/
LoadProjectCallback(project:Project, projects:Array<Partial<Project>>, rpcResponse:RPCResponse): void;

/**
*  Callback fired with the PaletteNode obtained asynchronously.
* 
* @param {Array<Partial<PaletteNode>>} an array of PaletteNodes
*/
PaletteNodeCallback(paletteNodes:Array<Partial<PaletteNode>>): void;

/**
*  Callback fired upon successful completion of Reify.getMockDS or
*  Reify.showMockDS:
*  
* - 
*  Output for all DataSource together is reported as the single string
*  parameter allDSData. When using MockDSExportSettings.format:
*  "reifyCSV", output for separate DataSources is separated by a special marker.
*  
* - 
*  Output with each DataSource as a separate string array element is also available
*  as the parameter perDSData, ordered to match the dsNames
*  parameter in Reify.getMockDS or Reify.showMockDS.
*  
*  Note that in the case of Reify.showMockDS, the callback is fired after the
*  window is closed, not when it's populated.
* 
* 
* @param {string} concatenated output for all DataSources
* @param {Array<Partial<string>>} same output but delivered as a per-DS array
*/
MockDSExportCallback(allDSData:string, perDSData:Array<Partial<string>>): void;

/**
*  A Callback to evaluate when a Process has been loaded via 
*  Process.loadProcess.
* 
* 
* @param {Process} the loaded Process
*/
ProcessCallback(process:Process): void;

/**
*  Callback for DrawPane.getDataURL.
* 
* @param {string} the data URL
*/
DataURLCallback(dataURL:string): void;

/**
*  A Callback to evaluate when form validation completes.
*  
*  The available parameters are:
* 
* @param {Map<any>} null if validation succeeded for all fields, or an object mapping field names to the associated errors, for those fields that failed validation.
*/
ValidationStatusCallback(errorMap:Map<any>): void;

/**
*  Callback required for the property Validator.condition and
*  ValidatorDefinition.condition.
* 
* @param {DataSourceField | ListGridField | FormItem} FormItem, DataSourceField or                      ListGridField on which this validator was                      declared. NOTE: FormItem will not                      be available during a save performed without a                      form (eg programmatic save) or if the field                      is not available in the form.
* @param {Validator} Validator declaration from eg               DataSourceField.validators.
* @param {any} value to validate
* @param {object} Field values for record being validated.
* @param {object} Object containing extra context which may be useful to the                  condition function. Contains the following properties:           component: the DynamicForm or ListGrid being validated           rowNum: the row number of the cell being validated (only if component is a ListGrid)           colNum: the column number of the cell being validated (only if component is a ListGrid)
*/
ValidatorConditionCallback(item:DataSourceField | ListGridField | FormItem, validator:Validator, value:any, record:object, additionalContext:object): boolean;

/**
*  Callback required for the property ValidatorDefinition.action.
* 
* 
* @param {boolean} The result of the validator. The value will be null if               the validator was skipped because of conditional criteria.
* @param {DataSourceField | FormItem} FormItem or DataSourceField on which this                     validator was declared. NOTE: FormItem will not                      be available during a save performed without a                      form (eg programmatic save) or if the field                      is not available in the form.
* @param {Validator} Validator declaration from eg               DataSourceField.validators.
* @param {Record} Record that was validated
* @param {DataBoundComponent} The DataBoundComponent holding the item such                    DynamicForm or ListGrid.
*/
ValidatorActionCallback(result:boolean, item:DataSourceField | FormItem, validator:Validator, record:Record, component:DataBoundComponent): void;

/**
*  Callback for DateRangeDialog.askForRange.
* 
* @param {Criterion} criterion representing the selected range
*/
DateRangeCallback(criterion:Criterion): void;

/**
*  A Callback called when the move completes.
* 
* 
* @param {boolean} true if the animation was cut short. To quit an animation                early, simply call the non-animated version of the same                API, so for example call Canvas.hide to cut short                an animation from Canvas.animateHide already in                progress.
*/
AnimationCallback(earlyFinish:boolean): void;

/**
*  Callback to execute after the section has been shown.
* 
*/
ShowSectionCallback(): void;

/**
*  Callback to execute after the section has been expanded.
* 
*/
ExpandSectionCallback(): void;

/**
*  Callback to execute after the section has been hidden.
* 
*/
HideSectionCallback(): void;

/**
*  Callback to execute after the section has been collapsed.
* 
*/
CollapseSectionCallback(): void;

/**
*  Callback executed when a message is sent to a channel that you have
*  Messaging.subscribe to.
* 
* @param {any} data contained in the message
*/
MessagingCallback(data:any): void;



}
/**
* Creates a new Callbacks
* 
* @param typeCheckedProperties {Partial<Callbacks>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Callbacks} The new instance
*/
class Callbacks{
static create(typeCheckedProperties? : Partial<Callbacks>, uncheckedProperties?: Object): Callbacks;







}

/**
* Button that cancels any changes in the form, by calling DynamicForm.cancelEditing 
*  when clicked.
*  See DynamicForm.cancelEditing for details on what happens when a form editing is 
*  cancelled.
*/
interface CancelItem extends ButtonItem {
/**
* CancelItems show a title of "Cancel" by default. May be overridden.
* @type {string}
* @default "Cancel"
*/
title?: string;





}
/**
* Creates a new CancelItem
* 
* @param typeCheckedProperties {Partial<CancelItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {CancelItem} The new instance
*/
class CancelItem{
static create(typeCheckedProperties? : Partial<CancelItem>, uncheckedProperties?: Object): CancelItem;







}

/**
* Base class for all SmartClient visual components (except FormItem).
*  
*  Canvas provides:
*  
*  
* - basic visual lifecycle support - creation and destruction of HTML via
*    Canvas.draw and Canvas.clear, visibility via
*    Canvas.show and Canvas.hide, z-layering via
*    Canvas.bringToFront and Canvas.sendToBack.
*  
* - consistent cross-browser Canvas.moveTo,
*    Canvas.resizeTo and Canvas.getScrollHeight, with
*    automatic compensation for CSSStyleName.
*  
* - clipping, scrolling and overflow management of content via Canvas.overflow
*  
* - consistent cross-browser Canvas.keyPress and Canvas.mouseDown
*    events, including mobileDevelopment to mouse events
*  
* - built-in drag and drop capabilities including Canvas.canDragReposition, 
*    Canvas.canDragResize, Canvas.canDragScroll
*    and Canvas.snapToGrid behavior.
*  
* - the ability to either contain Canvas.contents or
*    Canvas.children, including 
*    Canvas.snapTo and 
*    Canvas.percentSource for children. For more advanced layout
*    capabilities, see Layout.
*  
* - various other details like Canvas.cursor,
*    Canvas.showClickMask, Canvas.animateMove,
*    Canvas.ariaRole, and
*    Canvas.locateChildrenBy for automatedTesting.
*  
*/
interface Canvas extends BaseWidget {
/**
* Configures where the Opacity filter is used for IE6-8.
*  
*  With the default of null, opacity filters are used unless
*  Canvas.neverUseFilters has been set. When set explicitly to true,
*  opacity filters are used even if neverUseFilters is true.
*  
*  See IEFilters for background.
* @type {boolean}
* @default null
*/
useOpacityFilter?: boolean;

/**
* Global identifier for referring to a widget in JavaScript. The ID property is optional if
*  you do not need to refer to the widget from JavaScript, or can refer to it indirectly
*  (for example, by storing the reference returned by Class.create).
*  
*  An internal, unique ID will automatically be created upon instantiation for any canvas
*  where one is not provided.
*  
*  The ID property should be unique in the global scope. If window[ID]
*  is already assigned to something else a warning will be logged using the developer console,
*  and the existing reference will be replaced, calling Canvas.destroy on the
*  previous object if it is a SmartClient Class instance.
*  
*  Automatically generated IDs will be unique as long as the canvases they refer to remain
*  active - once a canvas with an automatically generated ID has been destroyed, its ID may be
*  reused for the next canvas created with no explicitly specified ID.
* @type {GlobalId}
* @default null
*/
ID?: GlobalId;

/**
* Optional name for the canvas, which can later be used to reference it. Need not be
*  globally unique, but should be unique within the Canvas.parentCanvas to get
*  defined results for Layout.getMember and Layout.getMemberNumber.
* @type {Identifier}
* @default null
*/
name?: Identifier;

/**
* 
*  If true, this canvas will draw itself immediately after it is created.
*  
*  Note that you should turn this OFF for any canvases that are provided as children
*  of other canvases, or they will draw initially, then be clear()ed and drawn again when
*  added as children, causing a large performance penalty.
*  
*  For example, the following code is incorrect and will cause extra draw()s:
*  
*  
*    isc.Layout.create({
*      members : [
*        isc.ListGrid.create()
*      ]
*    });
*  
*  It should instead be:
*  
*    isc.Layout.create({
*      members : [
*        isc.ListGrid.create({ autoDraw: false })
*      ]
*    });
*  
*  In order to avoid unwanted autoDrawing systematically, it is recommend that you call
*  isc.setAutoDraw immediately after SmartClient is loaded
*  and before any components are created, then set autoDraw:true or call
*  draw() explicitly to draw components.
*  
*  Otherwise, if the global setting for autoDraw remains true, you must set
*  autoDraw:false, as shown above, on every component in your application that
*  should not immediately draw: all Canvas children, Layout members, Window items, Tab
*  panes, etc, however deeply nested. Forgetting to set autoDraw:false will result in one
*  more clear()s - these are reported on the Results tab of the
*  debugging, and can be tracked to individual components by
*  using the "clears" log category in the Developer Console.
*  
* @type {boolean}
* @default true
*/
autoDraw?: boolean;

/**
* This Canvas's immediate parent, if any.
*  
*  Can be initialized, but any subsequent manipulation should be via
*  Canvas.addChild and Canvas.removeChild calls
*  on the parent.
* @deprecated * As of Smartclient version 9.1, deprecated in favor of Canvas.parentCanvas       and Canvas.getParentCanvas

* @type {Canvas}
* @default null
*/
parentElement?: Canvas;

/**
* This Canvas's immediate parent, if any.
*  
*  Can be initialized, but any subsequent manipulation should be via
*  Canvas.addChild and Canvas.removeChild calls
*  on the parent. The parent Canvas should be fetched using
*  Canvas.getParentCanvas.
*  
*  See containment for an overview of parent/child relationships.
* @type {Canvas}
* @default null
*/
parentCanvas?: Canvas;

/**
* The top-most Canvas (i.e., not a child of any other Canvas), if any, in this widget's
*  containment hierarchy.
* @type {Canvas}
* @default null
*/
topElement?: Canvas;

/**
* This Canvas's "master" (the Canvas to which it was added as a peer), if any.
* @deprecated * In favor or Canvas.getMasterCanvas as of SmartClient release 9.1

* @type {Canvas}
* @default null
*/
masterElement?: Canvas;

/**
* Array of all Canvii that are immediate children of this Canvas.
*  
*  Use Canvas.addChild and Canvas.removeChild to add and remove children after a Canvas
*  has been created/drawn.
*  
*  See containment for an overview of parent/child relationships.
* @type {Array<Partial<Canvas>>}
* @default null
*/
children?: Array<Partial<Canvas>>;

/**
* Array of all Canvii that are peers of this Canvas.
*  
*  Use Canvas.addPeer and Canvas.removePeer to add and remove peers after a Canvas
*  has been created/drawn.
*  
*  See containment for an overview of master/peer relationships.
* @type {Array<Partial<Canvas>>}
* @default null
*/
peers?: Array<Partial<Canvas>>;

/**
* This initialization property allows developers to create a canvas using the
*  Class.addAutoChild method, and have it added as a child to some other
*  component. This property may be set to the childName of another
*  already-created auto-child, or "none" to cause the component to be 
*  created without being added as a child to any other widget.
*  If unset, the canvas will be added as a child to the component on which
*  addAutoChild(...) was called.
*  
*  See autoChildren for an overview of the autoChild subsystem.
* @type {string}
* @default null
*/
autoParent?: string;

/**
* If specified as a pointer to an HTML element present in the DOM, this canvas will be
*  rendered inside that element on Canvas.draw. If provided as a String,
*  the String will be replaced by a DOM node as returned from
*  document.getElementById(htmlElement) on initialization.
*  
*  NOTES:
*  This feature is intended for integration with other JavaScript frameworks and
*  legacy page architectures only;
*  the native browser's reaction to DOM insertion is unspecified and unsupported. For
*  consistent cross-browser layout and positioning semantics, use Canvas parents
*  (especially Layouts) and use absolute positioning at top level.
*  
*  In some cases, the target element may need a specified height to be rendered
*  correctly. In this cases, you can expect to find a message like the following in the
*  JavaScript console:
*  "isc_DataView_0:can't resize to height: 0; clamping to 1 [enable 'sizing' log for stack trace]"
*  
*  Persistence of htmlElement: If Canvas.htmlPosition is set to "replace"
*  the htmlElement will be removed from the DOM when the canvas is drawn - therefore the
*  htmlElement attribute will be cleared at this time.
*  Otherwise if a Canvas is clear()d and then draw()n again it will
*  be rendered inside the same htmlElement.
*  If a Canvas is added as a child to Canvas parent, its htmlElement will be dropped.
*  
*  Canvas.position should typically be set to "relative" if the widget
*  is to be rendered inline within a standard page.
* @type {DOMElement | string}
* @default null
*/
htmlElement?: DOMElement | string;

/**
* If Canvas.htmlElement is specified, this attribute specifies the position where
*  the canvas should be inserted relative to the htmlElement in the DOM.
* @type {DrawPosition}
* @default "afterBegin"
*/
htmlPosition?: DrawPosition;

/**
* If Canvas.htmlElement is specified, should this canvas be drawn
*  at the same dimensions as the htmlElement?
*  See also Canvas.matchElementWidth and Canvas.matchElementHeight
*  
*  By default, if Canvas.htmlPosition is anything other than "replace",
*  setting this property will cause the canvas resize with the element if the element
*  itself subsequently resizes (for example due to page reflow).
*  
*  To disable this behavior for backwards compatibility or other reasons, use
*  Canvas.persistentMatchElement
* @type {boolean}
* @default null
*/
matchElement?: boolean;

/**
* If this canvas has a specified Canvas.htmlElement and Canvas.matchElement is set
*  to true, should the canvas perform a one time resize to fit the target element on
*  draw, or should it continue to match the target element as its size changes due to
*  page reflows?
* @type {boolean}
* @default true
*/
persistentMatchElement?: boolean;

/**
* For canvases with a specified Canvas.htmlElement where
*  Canvas.persistentMatchElement is set to true, how should the 
*  canvas match the element's width?
* @type {MatchElementSizeMode}
* @default "matchElement"
*/
matchElementWidth?: MatchElementSizeMode;

/**
* For canvases with a specified Canvas.htmlElement where
*  Canvas.persistentMatchElement is set to true, how should the 
*  canvas match the element's height?
* @type {MatchElementSizeMode}
* @default "matchElement"
*/
matchElementHeight?: MatchElementSizeMode;

/**
* Absolute or relative, corresponding to the "absolute" (with respect to parent) or
*  "relative" (with respect to document flow) values for the CSS position attribute.
*  
*  Setting position:"relative" enables SmartClient components to be embedded
*  directly into the native HTML flow of a page, causing the component to be rendered
*  within an existing DOM structure.
*  This attribute should only be set to "relative" on a top level component
*  (a component with no Canvas.getParentCanvas).
*  
*  There are 2 ways to embed relatively positioned canvases in the DOM - by default the
*  component will be written out inline when it gets Canvas.draw. For example
*  to embed a canvas in an HTML table you could use this code:
*  
*  &lt;table&gt;
*   &lt;tr&gt;
*    &lt;td&gt;
*     &lt;script&gt;
*      isc.Canvas.create({autoDraw:true, backgroundColor:"red", position:"relative"});
*     &lt;/script&gt;
*    &lt;td&gt;
*   &lt;/tr&gt;
*  &lt;/table&gt;
*  
*  Alternatively you can make use of the Canvas.htmlElement attribute.
*  
*  Relative positioning is intended as a short-term integration scenario while incrementally
*  upgrading existing applications.
*  Note that relative positioning is not used to manage layout within SmartClient components -
*  instead the Layout class would typically be used.
*  For best consistency and flexibility across browsers, all SmartClient layout managers
*  use absolute positioning.
*  
*  For canvases with a specified Canvas.htmlElement, this attribute defaults to
*  "relative". In all other cases the default value will be
*  "absolute". Note that if you plan to call Canvas.setHtmlElement after 
*  init, you will need to set this value to "relative" explicitly.
* @type {Positioning}
* @default null
*/
position?: Positioning;

/**
* Number of pixels the left side of the widget is offset to the right from its default
*  drawing context (either its parent's topleft corner, or the document flow, depending on
*  the value of the Canvas.position property).
*  
*  Can also be set as a percentage, specified as a String ending in '%', eg, "50%". In
*  this case the top coordinate is considered as a percentage of the specified width of
*  the Canvas.getParentCanvas.
* @type {number | string}
* @default "0"
*/
left?: number | string;

/**
* Number of pixels the top of the widget is offset down from its default drawing context
*  (either its parent's top-left corner, or the document flow, depending on the value of
*  the Canvas.position property).
*  
*  Can also be set as a percentage, specified as a String ending in '%', eg, "50%". In
*  this case the top coordinate is considered as a percentage of the specified height of
*  the Canvas.getParentCanvas.
* @type {number | string}
* @default "0"
*/
top?: number | string;

/**
* If set, overrides the global Canvas.defaultPageSpace.
* @type {Integer}
* @default null
*/
leavePageSpace?: Integer;

/**
* The canvas.width attribute specifies the size for a component's 
*  horizontal dimension; canvas.height specifies the size for the vertical 
*  dimension.
*  
*  May be set to an integer value (a number of pixels), a percentage value like "50%", or "*". 
*  
*  See percentSizing for details on how percentage or "*" values
*  are resolved actual size.
*  
*  If Canvas.overflow is set to "visible", the specified size acts as a
*  minimum, and the component may overflow to show all content and/or children.
*  
*  Note that developers wishing to set a default width or height for a component class
*  should set Canvas.defaultWidth or Canvas.defaultHeight 
*  instead of specifying an explicit default width or height.
*  This is important for components added to a Layout as members - it allows
*  the Layout to determine whether the canvas has an explicitly specified size that must be
*  respected, or whether it can participate in its LayoutPolicy.
* @type {number | string}
* @default null
*/
width?: number | string;

/**
* The canvas.width attribute specifies the size for a component's 
*  horizontal dimension; canvas.height specifies the size for the vertical 
*  dimension.
*  
*  May be set to an integer value (a number of pixels), a percentage value like "50%", or "*". 
*  
*  See percentSizing for details on how percentage or "*" values
*  are resolved actual size.
*  
*  If Canvas.overflow is set to "visible", the specified size acts as a
*  minimum, and the component may overflow to show all content and/or children.
*  
*  Note that developers wishing to set a default width or height for a component class
*  should set Canvas.defaultWidth or Canvas.defaultHeight 
*  instead of specifying an explicit default width or height.
*  This is important for components added to a Layout as members - it allows
*  the Layout to determine whether the canvas has an explicitly specified size that must be
*  respected, or whether it can participate in its LayoutPolicy.
* @type {number | string}
* @default null
*/
height?: number | string;

/**
* For custom components, establishes a default width for the component.
*  
*  For a component that should potentially be sized automatically by a Layout, set this
*  property rather than Canvas.width directly, because Layouts regard a width setting as
*  an explicit size that shouldn't be changed.
* @type {Integer}
* @default "100"
*/
defaultWidth?: Integer;

/**
* For custom components, establishes a default height for the component.
*  
*  For a component that should potentially be sized automatically by a Layout, set this
*  property rather than Canvas.height directly, because Layouts regard a height setting as
*  an explicit size that shouldn't be changed.
* @type {Integer}
* @default "100"
*/
defaultHeight?: Integer;

/**
* Minimum width available to this Canvas.
*  
*  The minWidth and Canvas.minHeight settings apply to:
*  
*  
* - For a canvas being managed as a member of a Layout, the minimum size the
*    layout should apply to the canvas.
*  
* - For a canvas with a width or height specified as a percent value, a minimum
*    numeric pixel value to limit how large the canvas is sized.
*  
* - determining size for a Canvas in a CanvasItem (minHeight only)
*  
* - end user Canvas.canDragResize
*  
* - minimum size when using Overflow "visible" outside of a Layout - minimum
*    size will be the greater of this setting or the minimum size needed to make all
*    content visible
*  
*  
*  Minimum sizes do not apply in various other circumstances where sizes are being
*  determined, such as ListGrid.showRecordComponents.
*  
*  See also Layout.minMemberLength as a way of establishing minimum sizes along
*  the length axis for all members of a Layout with a single setting.
* @type {number}
* @default null
*/
minWidth?: number;

/**
* Minimum height available to this Canvas. Minimum sizes do not apply to all
*  situations. See Canvas.minWidth for details.
* @type {number}
* @default null
*/
minHeight?: number;

/**
* Maximum width available to this Canvas. 
*  
*  The maxWidth and Canvas.maxHeight settings apply to:
*  
*  
* - For a canvas being managed as a member of a Layout, the maximum size the
*    layout should apply to the canvas.
*  
* - For a canvas with a width or height specified as a percent value, a maximum
*    numeric pixel value to limit how large the canvas is sized.
*  
* - determining size for a Canvas in a CanvasItem (maxHeight only)
*  
* - end user Canvas.canDragResize
*  
*  Maximum sizes do not apply in various other circumstances where sizes are being
*  determined, such as ListGrid.showRecordComponents.
* @type {number}
* @default "10000"
*/
maxWidth?: number;

/**
* Maximum height available to this Canvas. See Canvas.maxWidth for details of behavior.
* @type {number}
* @default "10000"
*/
maxHeight?: number;

/**
* Minimum width that this Canvas can be resized to by a user. Actual limit will be maximum
*  of dragMinWidth and Canvas.minWidth.
*  
*  Note that a Canvas with overflow:"visible" has an implicit minimize size based on it's
*  contents.
*  
*  Note that dragMinWidth affects only user-initiated drag resizes. To set
*  the minimum width of a Canvas embedded in a Layout, you can set +{minWidth}, or
*  Layout.minMemberLength to constrain the minimum size along the length axis of all
*  members of the Layout.
* @type {number}
* @default "10"
*/
dragMinWidth?: number;

/**
* Minimum height that this Canvas can be resized to by a user. See Canvas.dragMinWidth
*  for details of behavior.
* @type {number}
* @default "10"
*/
dragMinHeight?: number;

/**
* Maximum width that this Canvas can be resized to by a user. Actual limit will be minimum
*  of dragMaxWidth and Canvas.maxWidth.
* @type {number}
* @default "10000"
*/
dragMaxWidth?: number;

/**
* Sets maximum height that this Canvas can be resized to by a user. Actual limit will be
*  minimum of dragMaxHeight and Canvas.maxHeight.
* @type {number}
* @default "10000"
*/
dragMaxHeight?: number;

/**
* This is an advanced setting. If set to true, when a widget is
*  Canvas.hide, the widget's handle will be resized such that
*  it takes up no space, in addition to having its css visibility
*  property set to "hidden".
*  
*  In addition to preventing the size of this widget from impacting the
*  Canvas.getScrollWidth of any parent widget while hidden, this
*  setting works around a native bug observed in Internet Explorer 10, whereby
*  an &lt;IFRAME&gt; element with visibility set to hidden can
*  cause rendering problems, if the HTML loaded by the &lt;IFRAME&gt;
*  contains a &lt;frameset&gt;. In this case the browser may
*  refuse to draw other elements at the same coordinates with a lower z-index than
*  the hidden frame. Setting this property to true works around
*  this problem for cases where an &lt;IFRAME&gt; containing a
*  &lt;frameset will be rendered out, for example in an
*  HTMLFlow with contentsType set to "page".
* @type {boolean}
* @default false
*/
shrinkElementOnHide?: boolean;

/**
* If set to true, the widget's parent (if any) will automatically be shown whenever the
*    widget is shown.
* @type {boolean}
* @default false
*/
autoShowParent?: boolean;

/**
* Controls widget visibility when the widget is initialized. See Visibility type for
*  details.
*  
*  Specifying "visible" sets the CSS visiblity to "visible", forcing a child to be visible
*  even if the parent is hidden. Not supported for use with SmartClient layouts,
*  scrolling or auto-sizing but may be useful when working with third-party or legacy
*  DOM layout systems.
*  
*  Note that if Canvas.hideUsingDisplayNone is set for a hidden ancestor, setting
*  visibility will have no effect at all until that ancestor becomes visible.
* @type {Visibility}
* @default "inherit"
*/
visibility?: Visibility;

/**
* When this widget is hidden (see Canvas.visibility and Canvas.hide),
*  should display:none be applied to the Canvas.getOuterElement?
*  
*  This setting is not supported for general use, but in certain cases, it has been shown
*  that display:none is a work-around for browser bugs involving burn-through of iframes or
*  plugins, where the content of the iframe or plugin may still be visible despite the
*  containing widget being hidden.
* @type {boolean}
* @default false
*/
hideUsingDisplayNone?: boolean;

/**
* Whether native drag selection of contained text is allowed within this Canvas.
*  
*  Note that setting this property to false will not avoid text selection
*  which is initiated outside this Canvas from continuing into this Canvas, even if text
*  selection began in another Canvas.
* @type {boolean}
* @default false
*/
canSelectText?: boolean;

/**
* The CSS class applied to this widget as a whole.
* @deprecated * In favor or Canvas.styleName as of SmartClient release 5.5

* @type {CSSStyleName}
* @default "normal"
*/
className?: CSSStyleName;

/**
* The CSS class applied to this widget as a whole.
* @type {CSSStyleName}
* @default "normal"
*/
styleName?: CSSStyleName;

/**
* The CSS class to apply when printing this widget. If unset, falls back to the
*  Canvas.styleName.
* @type {CSSStyleName}
* @default null
*/
printStyleName?: CSSStyleName;

/**
* The contents of a canvas or label widget. Any HTML string is acceptable.
* @type {HTMLString}
* @default "&amp;nbsp;"
*/
contents?: HTMLString;

/**
* Dynamic contents allows the contents string to be treated as a simple but powerful
*  template. When this attribute is set to true, JavaScript expressions may be embedded 
*  within the contents string, using the format:
*  &#36;{[JavaScript to evaluate]}.
*  
*  For example, to include the current date in a templated message, 
*  canvas.contents could be set to:
*  "Today's date is &lt;b&gt;&#36;{new Date().toUSShortDate()}&lt;/b&gt;"
*  
*  Embedded expressions will be evaluated when the canvas is drawn or redrawn, and
*  the result of the evaluated expression will be displayed to the user. If the
*  expression does not evaluate to a String, the toString() representation
*  of the returned object will be displayed automatically
*  
*  Dynamic expressions are evaluated in the scope of the canvas displaying the content, 
*  so the this keyword may be used within your expression to refer to the 
*  canvas. Developers may also explicitly supply values for variables to be used within
*  the evaluation via the Canvas.dynamicContentsVars property.
*  
*  Notes:
*  
* - Calling markForRedraw() on the canvas will evaluate any embedded expressions.
*  
* - Multiple such expressions may be embedded within the contents string for a component.
*  
* - If an error occurs during evaluation, a warning is logged
*    to the debugging and the error string will be embedded in place 
*    of the expected value in the Canvas.
*  
*  
* @type {boolean}
* @default false
*/
dynamicContents?: boolean;

/**
* An optional map of name:value parameters that will be available within the scope of the
*  dynamicContents evaluation. For example - if you have e.g:
*  
*  Canvas.create({
*   dynamicContents: true,
*   dynamicContentsVars: {
*     name: "Bob"
*   },
*   contents: "hello &#36;{name}"
*  });
*  
*  The above will create a canvas with contents hello Bob. You can add, remove, and
*  change values in the dynamicContentsVars object literal, just call
*  markForRedraw() on the canvas to have the dynamicContents template re-evaluated.
*  
*  Note that this is always available inside a dynamic contents string and points to
*  the canvas instance containing the dynamic contents.
*  
*  Used only if Canvas.dynamicContents : true has been set.
* @type {ValueMap}
* @default null
*/
dynamicContentsVars?: ValueMap;

/**
* Set the CSS Margin, in pixels, for this component. Margin provides blank space outside
*  of the border.
*  
*  This property sets the same thickness of margin on every side. Differing per-side
*  margins can be set in a CSS style and applied via Canvas.styleName.
*  
*  Note that the specified size of the widget will be the size including the margin
*  thickness on each side.
* @type {number}
* @default null
*/
margin?: number;

/**
* Set the CSS padding of this component, in pixels. Padding provides space between the
*  border and the component's contents.
*  
*  This property sets the same thickness of padding on every side. Differing per-side
*  padding can be set in a CSS style and applied via Canvas.styleName.
*  
*  Note that CSS padding does not affect the placement of Canvas.children. To
*  provide a blank area around children, either use Canvas.margin or use
*  a Layout as the parent instead, and use properties such as
*  Layout.layoutMargin to create blank space.
* @type {number}
* @default null
*/
padding?: number;

/**
* Set the CSS border of this component, as a CSS string including border-width,
*  border-style, and/or color (eg "2px solid blue").
*  
*  This property applies the same border to all four sides of this component. Different
*  per-side borders can be set in a CSS style and applied via Canvas.styleName.
*  
*  If a Canvas.isGroup is being shown then border is derived from the
*  Canvas.groupBorderCSS attribute, not from the explicit border property.
* @type {string}
* @default null
*/
border?: string;

/**
* The background color for this widget. It corresponds to the CSS background-color
*  attribute. You can set this property to an RGB value (e.g. #22AAFF) or a named color
*  (e.g. red) from a list of browser supported color names.
* @type {CSSColor}
* @default null
*/
backgroundColor?: CSSColor;

/**
* URL for a background image for this widget (corresponding to the CSS "background-image"
*  attribute).
* @type {SCImgURL}
* @default null
*/
backgroundImage?: SCImgURL;

/**
* Specifies how the background image should be tiled if this widget
*  is larger than the image. It corresponds to the CSS background-repeat
*  attribute.
*  
*  The default of null means no background-repeat CSS will be
*  written out. See BackgroundRepeat type for details on other settings.
*  
*  NOTE: this setting directly sets the CSS property background-repeat but
*  does not attempt to work around various known bugs with this setting, or lack of support
*  in IE6. If you need to apply CSS-based workarounds for browser limitations with
*  this setting, it's best to do so via setting Canvas.styleName.
* @type {BackgroundRepeat}
* @default null
*/
backgroundRepeat?: BackgroundRepeat;

/**
* Specifies how the background image should be positioned on the widget.
*    It corresponds to the CSS background-position attribute. If unset,
*    no background-position attribute is specified if a background image is
*    specified.
* @type {string}
* @default null
*/
backgroundPosition?: string;

/**
* Default directory for app-specific images, relative to the Page-wide
*  Page.getAppImgDir.
* @type {URL}
* @default ""
*/
appImgDir?: URL;

/**
* Default directory for skin images (those defined by the class), relative to the
*  Page-wide Page.getSkinDir.
* @type {SCImgURL}
* @default "images/"
*/
skinImgDir?: SCImgURL;

/**
* Specifies the cursor image to display when the mouse pointer is
*    over this widget. It corresponds to the CSS cursor attribute. See Cursor type for
*    different cursors.
*    
*    See also Canvas.disabledCursor and Canvas.noDropCursor.
* @type {Cursor}
* @default "default"
*/
cursor?: Cursor;

/**
* Specifies the cursor image to display when the mouse pointer is
*    over this widget if this widget is disabled. It corresponds to the CSS cursor
*    attribute. See Cursor type for different cursors.
* @type {Cursor}
* @default "default"
*/
disabledCursor?: Cursor;

/**
* Specifies the cursor image to display when the user drags a droppable canvas
*    over this if it is not a valid drop target for the event and
*    EventHandler.showNoDropIndicator is true.
* @type {Cursor}
* @default "not-allowed"
*/
noDropCursor?: Cursor;

/**
* Renders the widget to be partly transparent. A widget's opacity property may
*    be set to any number between 0 (transparent) to 100 (opaque).
*    Null means don't specify opacity directly, 100 is fully opaque.
*    Note that heavy use of opacity may have a performance impact on some older
*    browsers.
*    
*    In older versions of Internet Explorer (Pre IE9 / HTML5), opacity is achieved
*    through proprietary filters. If
*    Canvas.neverUseFilters within this application
*    developers must set Canvas.useOpacityFilter to true for specific components
*    on which opacity support is required.
*    
*    Also note that opacity is incompatible
*    with Canvas.useBackMask.
* @type {number}
* @default null
*/
opacity?: number;

/**
* Controls what happens when the drawn size of the content of a Canvas is either
*      greater or smaller than the specified size of the Canvas. Similar to the CSS
*      property overflow, but consistent across browsers. See Overflow type for
*      details.
* @type {Overflow}
* @default "visible"
*/
overflow?: Overflow;

/**
* Indicates that this component is able to render itself at multiple possible sizes, in order
*  to fit into less space when very little space is available, or to display more information
*  or provide more immediate access to functionality when more space is available.
*  
*  Unlike simply indicating a flexible size via width:"*", setting canAdaptWidth
*  indicates that a component has 2 (or more) different ways of rendering itself with different
*  discrete sizes, but does not have the ability to use every additional available pixel.
*  
*  For example, a menu embedded in a toolbar might show as only a fixed-size icon which reveals
*  menu options when clicked, or if enough space is available, could show all of the menu
*  options directly in the toolbar. In this example, the menu should either be allocated just
*  enough space for an icon, or enough space for all options to appear inline; any other
*  amount of space being allocated is just a waste.
*  
*  When a component with canAdaptWidth is part of a Layout, Canvas.adaptWidthBy
*  will be called every time component sizes are being calculated, passing a positive or
*  negative pixel value: 
*  
*  
* - a positive pixel value indicates a surplus - all other component's minimum sizes
*    can be accommodated, including any Canvas.minWidth declared on the
*    canAdaptWidth component itself
*  
* - a negative pixel value indicates that the containing layout is going to be
*    forced to Overflow (introduce scrollbars, clip content, etc) unless some
*    component or set of components gives up at least this many pixels
*  
*  
*  If a non-zero value is returned from adaptWidthBy, this means that the
*  component is willing to shrink or expand by that many pixels. Returning 0 means that the
*  component is unable to reduce size further, or for a surplus, cannot make good use of the
*  surplus space.
*  
*  
* - A call to Canvas.adaptWidthBy may surrender as many pixels as desired
*  (as long as the widget won't drop below its minimum allowed width), no matter whether a
*  positive (surplus) or negative (overflow) pixel value is supplied, but
*  
* - A call to Canvas.adaptWidthBy may not increase its size by more than the number of
*  offered pixels - so if an overflow is present, it may not increase its size at all.
*  
*  
*  Note that when the initial width is specified as a stretch size (e.g. "*"), then after
*  Canvas.adaptWidthBy is called, the Framework will stretch (but not shrink) the
*  member like any other stretch-size Layout member, but the unadaptedWidth
*  argument will always reflect the unstretched width requested by the previous call to
*  Canvas.adaptWidthBy. This behavior may be disabled by specifying the initial
*  width as a number, or leaving it unspecified.
*  
*  Behavior is slightly different for overflow: "visible" members - in this case the
*  unadaptedWidth passed in will reflect the current visible width of the
*  member, rather than the last width requested by the previous call to
*  Canvas.adaptWidthBy or the specified width (on the first call). However, note that the
*  visible length will match your requested width unless the member is actually overflowed.
*  Stretch sizing is not supported for adaptive-width members with overflow: "visible".
*  
*  Caution: you must either determine the current size of the canvas by maintaining your own
*  internal state, or use the unadaptedWidth parameter passed to 
*  Canvas.adaptWidthBy. You must not call Canvas.getWidth or Canvas.getVisibleWidth 
*  on the canvas itself inside Canvas.adaptWidthBy as the size is in the processing of
*  being determined, but you may draw children or call Canvas.getVisibleWidth on them, as
*  we guarantee that the adaptive-width canvas is drawn before the first call to 
*  Canvas.adaptWidthBy. An example of drawing children in Canvas.adaptWidthBy to
*  compute overall width may be seen in the
*  ${isc.DocUtils.linkForExampleId('inlinedMenuMobileSample', 'Inlined Menu Mobile Sample')}.
*  
*  Note that reasonable settings for Canvas.minWidth should be applied to all other
*  flexible-sized members of a layout where a canAdaptWidth component appears,
*  because when too little space is available, a canAdaptWidth component will
*  absorb all available space until minimums for other components are hit (or the
*  canAdaptWidth component reaches its maximum size). If more than one
*  canAdaptWidth component is present, Canvas.adaptiveWidthPriority to give priority
*  to a particular component when allocating space.
*  
*  All of the above behaviors are exactly the same for height, using
*  Canvas.canAdaptHeight and Canvas.adaptHeightBy.
* @type {boolean}
* @default null
*/
canAdaptWidth?: boolean;

/**
* See Canvas.canAdaptWidth.
* @type {boolean}
* @default null
*/
canAdaptHeight?: boolean;

/**
* If multiple widgets in the same Layout have Canvas.canAdaptWidth,
*  adaptiveWidthPriority can be set to indicate which of the components should be 
*  given priority access to space.
*  
*  The widget with the highest priority setting will be offered surplus space first, and asked
*  to give up space last. Lack of a priority setting is treated as zero. Any adaptive widgets
*  with the same priority setting will be asked to give up or release space according to their
*  order in Layout.members.
* @type {Integer}
* @default null
*/
adaptiveWidthPriority?: Integer;

/**
* See Canvas.adaptiveWidthPriority.
* @type {Integer}
* @default null
*/
adaptiveHeightPriority?: Integer;

/**
* Whether to use the browser's native scrollbars or SmartClient-based scrollbars.
*  
*  SmartClient-based scrollbars are skinnable, giving you complete control over look and
*  feel. SmartClient-based scrollbars also enable some interactions not possible with
*  native scrollbars, such as ListGrid.fixedRecordHeights
*  in grids in combination with ListGrid.dataPageSize.
*  
*  Native browser scrollbars are slightly faster simply because there are less SmartClient
*  components that need to be created, drawn and updated. Each visible SmartClient-based
*  scrollbar on the screen has roughly the impact of two StretchImgButtons.
*  
*  SmartClient is always aware of the size of the scrollbar, regardless of whether native
*  or custom scrollbars are used, and regardless of what operating system and/or operating
*  system "theme" or "skin" is in use. This means SmartClient will correctly report the
*  Canvas.getViewportHeight, that is, the interior area of the
*  widget excluding space taken by scrollbars, which is key for exactly filling a component
*  with content without creating unnecessary scrolling.
*  
*  The showCustomScrollbars setting is typically overridden in load_skin.js in
*  order to change the default for all SmartClient components at once.
*  This may be achieved via the static Canvas.setShowCustomScrollbars 
*  method or via a simple addProperties block
*  , like so:
*  
*  
*    isc.Canvas.addProperties({ showCustomScrollbars:false });
*  
*  
*  
*  
*  On Browser.isTouch, custom scrollbars are disabled in favor of enabling
*  native touch scrolling if available. However, custom scrollbars and native touch
*  scrolling can be enabled for the component by setting Canvas.alwaysShowScrollbars
*  to true.
* @type {boolean}
* @default true
*/
showCustomScrollbars?: boolean;

/**
* If Canvas.showCustomScrollbars is true, should the scrollbars be drawn floating over
*  the component handle, or should the handle shrink to accommodate them?
*  
*  Floating scrollbars are typically only appropriate for scrollbars that are hidden by
*  default and get shown as the user actively scrolls the widget handle, such as in the
*  Canvas.nativeAutoHideScrollbars case. If floating scrollbars are permanently visible 
*  over the component handle, they may block some of the widget's content.
* @type {boolean}
* @default false
*/
floatingScrollbars?: boolean;

/**
* Whether this canvas should receive eventHandling from its scrollbars,
*  which are Canvas.peers. Normally, a canvas only gets bubbled events from
*  its Canvas.children.
*  
*  Note that this property only has an impact if Canvas.showCustomScrollbars is true.
* @type {boolean}
* @default true
*/
receiveScrollbarEvents?: boolean;

/**
* Should this browser always show custom scrollbars if Canvas.showCustomScrollbars is true?
*  
*  On Browser.isTouch that support native touch scrolling, if
*  Canvas.showCustomScrollbars is true and touch
*  scrolling has not been disabled by the Canvas.useTouchScrolling and/or
*  Canvas.disableTouchScrollingForDrag settings, should custom scrollbars and
*  native touch scrolling be enabled for this component? If false or unset,
*  then only native touch scrolling will be enabled. If true, then both scrolling
*  mechanisms will be enabled.
*  
*  NOTE: Because native touch scrolling (also called momentum scrolling)
*  is computationally intensive, some mobile browsers implement an optimization where the state
*  of the DOM for the element being scrolled will be frozen or partially frozen during
*  the scroll animation. This results in a delay between when the scroll position reaches
*  a certain point in the animation and when the positions of the custom scrollbar thumbs
*  are updated to reflect that scroll position.
*  
*  For non-touch devices, setting this property to true will override
*  Canvas.nativeAutoHideScrollbars, and ensure custom scrollbars are shown for the
*  component
* @type {boolean}
* @default null
*/
alwaysShowScrollbars?: boolean;

/**
* In some platform/browser configurations, scrollable regions do not show
*  visible scrollbars until the user attempts to interact with the region.
*  The interaction to show the scrollbar varies by browser/OS but may include
*  starting a trackpad scroll or simply rolling over the scrollable element.
*  
*  If nativeAutoHideScrollbars is set to true, we detect platforms that show
*  scrollbars dynamically on user interaction and for components with 
*  Canvas.showCustomScrollbars we ignore the specified 
*  Canvas.scrollbarConstructor, and instead create system-managed native scrollbars
*  via the special NativeScrollbar class, and set Canvas.floatingScrollbars to true.
*  
*  Not applicable to Browser.isTouch
*  
*  Has no impact if Canvas.alwaysShowScrollbars is true.
*  
*  If Canvas.showCustomScrollbars is false this setting will have no effect (we would
*  be showing custom scrollbars in any case)
*  
*  Also does not apply to Canvas.useTouchScrolling (where
*  scrollbars are always hidden unless Canvas.alwaysShowScrollbars is true).
* @type {boolean}
* @default false
*/
nativeAutoHideScrollbars?: boolean;

/**
* On Browser.isTouch, if this Canvas can be scrolled,
*  should touch-dragging the content area result in scrolling? Set to false
*  if touch-dragging should not cause scrolling. Note that setting to false
*  enables the use of Canvas.showCustomScrollbars on touch devices.
*  
*  useTouchScrolling can default to false if
*  Canvas.disableTouchScrollingForDrag is true
*  and various built-in drag operations are enabled that normally interfere with touch scrolling
*  (e.g. ListGrid.canDragSelect and ListGrid.canReorderRecords).
*  
*  When touch scrolling is disabled, it can be difficult to interact with parts of the
*  custom scrollbars at their default size of 16 pixels. In touch browsers, any touch 8px
*  before the thumb of a Scrollbar will be mapped to the thumb, but
*  the other parts of the scrollbar do not have a similar tolerance applied. The width of
*  the custom scrollbars can be increased by setting the Canvas.scrollbarSize to a
*  larger value, but note that when skinning, changing the
*  scrollbarSize may cause tiling of certain images and backgrounds that make
*  up the custom scrollbar. This can be fixed for a component by creating it with
*  Canvas.scrollbarConstructor set to "Scrollbar"&mdash;a basic scrollbar class that
*  does not employ spriting.
* @type {boolean}
* @default null
*/
useTouchScrolling?: boolean;

/**
* Disables Canvas.useTouchScrolling whenever a built-in drag operation has been enabled which is
*  known to be non-functional if touch scrolling is enabled. Default behavior is to leave touch
*  scrolling enabled even if it makes other enabled drag operations non-functional, since any
*  accessibility application must provide an alternative way to perform
*  drag and drop operations anyway.
*  
*  disableTouchScrollingForDrag exists so that applications can change the default
*  setting on a per-component basis (via 
*  Class.changeDefaults),
*  in order to make a system-wide or
*  per-component-type decision about whether to favor touch scrolling vs retaining the ability to
*  drag and drop via finger drags, instead of having to set useTouchScrolling on each
*  individual instance.
*  
*  See the mobileDevelopment for more background
*  information.
* @type {boolean}
* @default null
*/
disableTouchScrollingForDrag?: boolean;

/**
* How thick should we make the scrollbars for this canvas. This only applies if
*  Canvas.showCustomScrollbars is true.
*  
*  NOTE: When skinning, changing the
*  scrollbarSize may cause tiling of certain images and backgrounds that make
*  up the custom scrollbar. This can be fixed for a component by creating it with
*  Canvas.scrollbarConstructor set to "Scrollbar"&mdash;a basic scrollbar class
*  that does not employ spriting.
* @type {number}
* @default "16"
*/
scrollbarSize?: number;

/**
* The class that will be used to create custom scrollbars for this component. Set this
*  attribute to a Scrollbar subclass with e.g. a different skinImgDir, to customize scrollbar
*  appearance for this component only.
*  
*  When skinning and supported by the skin, the default
*  scrollbarConstructor is changed to a different scrollbar class which handles
*  scrollbar spriting. Spriting of the scrollbars of an individual component can therefore
*  be disabled by creating the component with scrollbarConstructor set to the
*  "Scrollbar" class. "Scrollbar" is a basic scrollbar class that does not employ spriting.
* @type {string}
* @default "Scrollbar"
*/
scrollbarConstructor?: string;

/**
* If set to true, the widget will be disabled. A widget is only considered enabled
*  if it is individually enabled and all parents above it in the containment hierarchy
*  are enabled. This allows you to enable or disable all components of a complex
*  nested widget by enabling or disabling the top-level parent only.
* @type {boolean}
* @default false
*/
disabled?: boolean;

/**
* If set to true, this widget will be enabled, if set to false, or null, this
*  widget will be disabled.
* @deprecated * As of SmartClient version 5.5 this property has been deprecated. The  Canvas.disabled property will be used to govern enabled/disabled state instead
*  if this.enabled has not been changed from its default value.

* @type {boolean}
* @default "unset"
*/
enabled?: boolean;

/**
* Criteria to be evaluated to determine whether this Canvas should be visible. Re-evaluated
*  whenever data in the Canvas.ruleScope changes.
*  
*  A basic criteria uses textMatchStyle:"exact". When specified in
*  componentXML this property allows
*  xmlCriteriaShorthand for defining criteria.
* @type {AdvancedCriteria}
* @default null
*/
visibleWhen?: AdvancedCriteria;

/**
* Criteria to be evaluated to determine whether this Canvas should be enabled. Re-evaluated
*  whenever data in the Canvas.ruleScope changes.
*  
*  A basic criteria uses textMatchStyle:"exact". When specified in
*  componentXML this property allows
*  xmlCriteriaShorthand for defining criteria.
* @type {AdvancedCriteria}
* @default null
*/
enableWhen?: AdvancedCriteria;

/**
* Should this element be redrawn in response to a resize?
*  
*  Should be set to true for components whose Canvas.getInnerHTML will not
*  automatically reflow to fit the component's new size.
* @type {boolean}
* @default null
*/
redrawOnResize?: boolean;

/**
* Can this widget be allowed to become the target of keyboard events?
*  
*  If canFocus is unset (the default), only scrollable widgets with visible scrollbars are
*  focusable, to allow for keyboard scrolling.
*  
*  A widget normally receives focus by being clicked on or tabbed to.
* @type {boolean}
* @default null
*/
canFocus?: boolean;

/**
* If specified this governs the tabIndex of the widget in the page's tab order.
*  Setting this value to -1 will ensure the canvas does not show up
*  in the page's tab order, though if Canvas.canFocus is true, the user
*  may still give it keyboard focus by clicking on the widget directly.
*  
*  By default SmartClient auto-assigns tab-indices, ensuring focusable widgets
*  are reachable by tabbing in an intuitive order based on widget hierarchy and 
*  draw order. Specifying an explicit tab index means a widget will not participate 
*  in this automatic tab position allocation and is typically not recommended except
*  for very simple cases.
*  
*  For more information on automatic tab index assignment, including recommended
*  approaches for customizing tab order assignation, see the tabOrderOverview.
*  Developers may call Canvas.clearExplicitTabIndex to clear any explicitly
*  assigned tab index, and have the widget participate in automatic tab position allocation.
*  
*  canvas.tabIndex cannot be set to greater than
*  Canvas.TAB_INDEX_FLOOR - as we reserve the values above this range for
*  auto-assigned tab-indices.
* @type {number}
* @default null
*/
tabIndex?: number;

/**
* If specified this governs the HTML accessKey for the widget.
*  
*  This should be set to a character - when a user hits the html accessKey modifier for
*  the browser, plus this character, focus will be given to the widget in question.
*  The accessKey modifier can vary by browser and platform. 
*  
*  The following list of default behavior is for reference only, developers should also
*  consult browser documentation for additional information.
*  
*  
* - Internet Explorer (all platforms): Alt + accessKey
*  
* - Mozilla Firefox (Windows, Unix): Alt+Shift + accessKey
*  
* - Mozilla Firefox (Mac): Ctrl+Opt + accessKey
*  
* - Chrome and Safari (Windows, Unix): Alt + accessKey
*  
* - Chrome and Safari (Mac): Ctrl+Opt + accessKey
*  
* @type {string}
* @default null
*/
accessKey?: string;

/**
* Context menu to show for this object, an instance of the Menu widget.
*  
*  Note: if Canvas.destroy is called on a canvas, any specified context menu is
*  not automatically destroyed as well. This is in contrast to MenuButtons which
*  automatically destroy their specified MenuButton.menu by default. The behavior
*  is intentional as context menus are commonly reused across components.
* @type {Menu}
* @default null
*/
contextMenu?: Menu;

/**
* Default class used to construct menus created by this component, including context menus.
* @type {SCClassName}
* @default "Menu"
*/
menuConstructor?: SCClassName;

/**
* Prompt displayed in hover canvas if Canvas.showHover is true.
* @type {HTMLString}
* @default null
*/
prompt?: HTMLString;

/**
* If set, native HTML5 drag and drop will be used for all drags initiated on this widget
*  (on browsers where this is supported), and native HTML5 drop events occurring over this
*  widget will be processed.
*  
*  When using native HTML5 drags, the same series of events fires as for a normal drag
*  (Canvas.dragStart, Canvas.dropMove, etc.), and the Canvas.dragType / Canvas.dropTypes
*  system works. Canvas.dragAppearance is not supported; however,
*  basic customization of the browser's tracker image is supported in certain browsers via
*  the EventHandler.setDragTrackerImage API.
*  
*  The primary difference with a native drag is that it can be cross-frame; that is, the
*  user can drag out of the current browser window and drop into a different window or tab.
*  
*  To provide information that will be available to a foreign frame, use
*  EventHandler.setNativeDragData. This API must be called when the Canvas.dragStart
*  event fires, and will not work if called at any other time.
*  
*  However, due to browser bugs and/or browser-imposed limitations, the information provided to
*  setNativeDragData cannot be accessed in the foreign frame until the actual drop
*  occurs (mouse button released). This means drop eligibility cannot be determined dynamically
*  based on the dragged data; instead, eligibility can only be determined based on the
*  Canvas.dragType / Canvas.dropTypes system. For this reason, a Canvas.dragType must
*  be set on the source of a drag.
*  
*  NOTE: Although Internet Explorer 10+ supports a subset of the 
*  HTML5 drag and drop standard,
*  native drag and drop is disabled in IE (and Microsoft Edge Legacy) because 
*  cross-window drags&mdash;the 
*  primary purpose of this API&mdash;are not possible.
* @type {boolean}
* @default null
*/
useNativeDrag?: boolean;

/**
* Indicates whether this widget can initiate custom drag-and-drop operations (other than
*  reposition or resize). Normally Canvas.canDragReposition or Canvas.canDragResize would
*  be used instead of this property.
*  
*  Note: this property may be manipulated by higher-level dragging semantics.
*  
*  If Canvas.useNativeDrag is true and this widget has been drawn, then this widget must be
*  Canvas.redraw in order for a change of the value of this attribute to take effect.
* @type {boolean}
* @default false
*/
canDrag?: boolean;

/**
* Number of pixels the cursor needs to move before the EventHandler starts a drag operation.
* @type {number}
* @default "5"
*/
dragStartDistance?: number;

/**
* If this Canvas is canAcceptDrop:true, when the user drags a droppable widget over
*    an edge of the widget, should we scroll to show the rest of the widget's content?
*    Returned from canvas.shouldDragScroll() if there are scrollbars.
* @type {boolean}
* @default true
*/
canDragScroll?: boolean;

/**
* If this widget supports drag-scrolling, This property specifies how many ms the
*    user must hover over the drag-scroll threshold before scrolling begins.
* @type {number}
* @default "100"
*/
dragScrollDelay?: number;

/**
* This indicates how the system will test for droppable targets: either by intersection
*  with the mouse or intersection with the rectangle of the dragMoveTarget.
* @type {DragIntersectStyle}
* @default "mouse"
*/
dragIntersectStyle?: DragIntersectStyle;

/**
* Indicates whether this widget can be moved by a user of your application by simply
*    dragging with the mouse.
* @type {boolean}
* @default false
*/
canDragReposition?: boolean;

/**
* Cursor to switch to if the mouse is over a widget that is drag repositionable.
* @type {Cursor}
* @default "move"
*/
dragRepositionCursor?: Cursor;

/**
* Indicates whether this widget can be resized by dragging on the edges and/or corners of
*    the widget with the mouse.
* @type {boolean}
* @default false
*/
canDragResize?: boolean;

/**
* If Canvas.canDragResize is true, this property specifies the conditions for when
*  proportional resizing is used. The default is "none"
*  
*  , which means that proportional resizing is disabled.
* @type {ProportionalResizeMode}
* @default "none"
*/
proportionalResizing?: ProportionalResizeMode;

/**
* If Canvas.proportionalResizing is set to "modifier"
*  
*  or "modifierOff"
*  
*  then proportional resizing of the widget is activated or deactivated, respectively,
*  whenever at least one key in this set of modifier keys is pressed.
*  
*  The keys allowed in this set are: "Alt", "Ctrl", and "Shift". If this set
*  of keys is empty then proportional resizing is always used if
*  proportionalResizing is "modifier"
*  and is never used if
*  proportionalResizing is "modifierOff"
*  .
* @type {Array<Partial<KeyName>>}
* @default "["Shift"]"
*/
proportionalResizeModifiers?: Array<Partial<KeyName>>;

/**
* When drag resizing is enabled via Canvas.canDragResize, restricts resizes to only
*  certain edges or corners.
*  
*  The default of null indicates the widget can be resized from any corner or edge (if
*  canDragResize is true).
*  
*  To restrict resizing to only certain corners, set resizeFrom to an Array of
*  EdgeNames.
* @type {Array<Partial<EdgeName>>}
* @default null
*/
resizeFrom?: Array<Partial<EdgeName>>;

/**
* This flag controls whether we register the component as a maskable item with the
*  EventHandler. If enabled, a backmask will be automatically created for the
*  dragMoveTarget on the fly to avoid burnthrough e.g. by plugins or frames.
*  
*  Note that this property will be defaulted to false unless the canvas contains an
*  IFrame, in which case it will be defaulted to true.
*  
*  The Canvas.dragMaskType property controls what kind of mask is used in case
*  useDragMask is enabled.
* @type {boolean}
* @default null
*/
useDragMask?: boolean;

/**
* This property controls what kind of mask is used in case Canvas.useDragMask is
*  enabled.
* @type {DragMaskType}
* @default "div"
*/
dragMaskType?: DragMaskType;

/**
* Will this Canvas fire hover events when the user hovers over it, or one of its children?
* @type {boolean}
* @default null
*/
canHover?: boolean;

/**
* If this.canHover is true, how long should the mouse be kept over this
*  widget before the hover event is fired
* @type {number}
* @default "300"
*/
hoverDelay?: number;

/**
* If this.canHover is true, should we show the global hover canvas by default
*  when the user hovers over this canvas?
* @type {boolean}
* @default true
*/
showHover?: boolean;

/**
* If Canvas.showHover is true, this property can be used to customize
*  the width of the hover canvas shown. See also Canvas.hoverAutoFitWidth and 
*  Canvas.hoverAutoFitMaxWidth.
* @type {Integer}
* @default "1"
*/
hoverWidth?: Integer;

/**
* if Canvas.showHover is true, this property will cause the specified 
*  Canvas.hoverWidth to be treated as a minimum width for the hover. If the
*  hover content string exceeds this, the hover will expand to accommodate it
*  up to Canvas.hoverAutoFitMaxWidth (without the text wrapping).
*  
*  Using this settings differs from simply disabling wrapping via 
*  Canvas.hoverWrap as the content will wrap if the
*  Canvas.hoverAutoFitMaxWidth is exceeded.
* @type {boolean}
* @default true
*/
hoverAutoFitWidth?: boolean;

/**
* Maximum auto-fit width for a hover if Canvas.hoverAutoFitWidth is 
*  enabled. May be specified as a pixel value, or a percentage of page width.
* @type {number | string}
* @default "200"
*/
hoverAutoFitMaxWidth?: number | string;

/**
* If this.showHover is true, this property can be used to customize the
*  height of the hover canvas shown.
* @type {Integer}
* @default null
*/
hoverHeight?: Integer;

/**
* If this.showHover is true, this property can be used to customize the
*  alignment of content in the hover canvas.
* @type {Alignment}
* @default null
*/
hoverAlign?: Alignment;

/**
* If this.showHover is true, this property can be used to customize the
*  vertical alignment of content in the hover canvas.
* @type {VerticalAlignment}
* @default null
*/
hoverVAlign?: VerticalAlignment;

/**
* If this.showHover is true, this property can be used to customize the
*  whether content in the hover canvas is displayed in a single line, or wraps.
*  
*  Note that if developers wish to have hovers expand horizontally to fit their
*  text without wrapping *up to some maximum*, and then wrap rather than exceeding
*  that maximum, the Canvas.hoverAutoFitWidth and Canvas.hoverAutoFitMaxWidth attributes
*  may be used instead of simply setting hoverWrap to false.
* @type {boolean}
* @default null
*/
hoverWrap?: boolean;

/**
* If this.showHover is true, this property can be used to specify the
*  css style to apply to the hover canvas.
* @type {CSSStyleName}
* @default null
*/
hoverStyle?: CSSStyleName;

/**
* If this.showHover is true, should the hover canvas be shown with opacity
*  other than 100?
* @type {number}
* @default null
*/
hoverOpacity?: number;

/**
* If this.showHover is true, should this widget's hover canvas be moved with
*  the mouse while visible?
* @type {boolean}
* @default null
*/
hoverMoveWithMouse?: boolean;

/**
* If this.showHover is true and Canvas.getHoverComponent is implemented, should
*  the hoverCanvas returned from it be automatically destroyed when it is hidden?
*  
*  The default of null indicates that the component will be automatically
*  destroyed. Set to false to prevent this.
* @type {boolean}
* @default null
*/
hoverAutoDestroy?: boolean;

/**
* Allows interaction with hovers when the cursor is positioned over them.
* @type {HoverPersistMode}
* @default null
*/
hoverPersist?: HoverPersistMode;

/**
* How far into the edge of an object do we consider the "edge" for drag resize purposes?
* @type {number}
* @default "5"
*/
edgeMarginSize?: number;

/**
* If the widget has drag resize configured on one or more of it's edges, and the
*  edgeMarginSize is large enough that the remaining space is less than
*  minNonEdgeSize, the edgeMarginSize will be reduced such that the non-edge
*  part of the widget is at least 1/3 of the total space (with two draggable edges) or half
*  of it (with one draggable edge).
* @type {Integer}
* @default "0"
*/
minNonEdgeSize?: Integer;

/**
* Cursor to use when over each edge of a Canvas that is drag resizable.
*  
*  To disable drag resize cursors, set the edgeCursorMap property to null.
* @type {object}
* @default "{...}"
*/
edgeCursorMap?: object;

/**
* Constrains drag-resizing and drag-repositioning of this canvas to either the rect of its
*  parent (if set to true) or an arbitrary rect based on its parent (if set to a
*  [Left,Top,Width,Height] rect array). In the latter mode you may use negative offsets
*  for left/top and a width/height greater than the visible or scroll width of the parent
*  to allow positioning beyond the confines of the parent.
*  
*  If this canvas has no parent, constrains dragging to within the browser window.
*  
*  Affects target and outline dragAppearance, not tracker.
*  
*  Note: keepInParentRect affects only user drag interactions, not programmatic moves.
*  
*  Example use cases:
*  keepInParentRect: true - confine to parent
*  keepInParentRect: [0, 0, 500, 500] - confine to top left 500x500 region within parent
*  keepInParentRect: [0, 0, 10000, 10000] - in combination with
*  overflow: "auto", confine to parent, but allow moving off the right and bottom of the
*  parent to force scrolling (and hence enlarge the scrollWidth of the parent).
* @type {boolean | Array<Partial<Integer>>}
* @default null
*/
keepInParentRect?: boolean | Array<Partial<Integer>>;

/**
* Visual appearance to show when the object is being dragged. May be overridden for
*  dragResize or dragReposition events via Canvas.dragResizeAppearance
*  and Canvas.dragRepositionAppearance.
* @type {DragAppearance}
* @default "outline"
*/
dragAppearance?: DragAppearance;

/**
* If Canvas.canDragResize is true, this attribute specifies the visual appearance
*  to show during drag resize. If unset Canvas.dragAppearance will be used.
* @type {DragAppearance}
* @default null
*/
dragResizeAppearance?: DragAppearance;

/**
* If Canvas.canDragReposition is true, this attribute specifies the visual appearance
*  to show during drag reposition. If unset Canvas.dragAppearance will be used.
* @type {DragAppearance}
* @default null
*/
dragRepositionAppearance?: DragAppearance;

/**
* Sets a dragType for this widget used, to be compared to
*  Canvas.dropTypes on possible drop target widgets. See
*  Canvas.dropTypes for a full explanation.
* @type {string}
* @default null
*/
dragType?: string;

/**
* When a drag and drop interaction occurs, if a Canvas.dragType is
*  configured on the source widget, it is compared to the dropTypes configured
*  on the target widget, and a drop is only allowed if the dragType is listed
*  in the target widget's dropTypes array.
*  
*  The default setting means any dragType is eligible for dropping on this
*  widget, including no dragType setting.
*  
*  See also Canvas.willAcceptDrop for dynamic determination of drop eligibility.
* @type {Array<Partial<string>> | string}
* @default "**anything**"
*/
dropTypes?: Array<Partial<string>> | string;

/**
* A different widget that should be actually dragged when dragging initiates on this
*  widget. One example of this is to have a child widget that drags its parent, as with a
*  drag box. Because the parent automatically repositions its children, setting the drag
*  target of the child to the parent and then dragging the child will result in both
*  widgets being moved.
*  
*  Valid dragTarget values are:
*  
* - null (default) [this widget is its own drag target]
*  
* - another widget, or widget ID
*  
* - "parent" drag target is this widget's Canvas.getParentCanvas
*  
* - "top" drag target is this widget's Canvas.topElement
*  
*  Note that for dragging to work as intended, the Canvas.resizeFrom setting on the
*  dragTarget must be null or a superset of the Canvas.resizeFrom on this
*  canvas.
* @type {Canvas | string}
* @default null
*/
dragTarget?: Canvas | string;

/**
* Delegates a different widget that should actually be dropped on if the cursor is over
*  this widget when the drop completes. If you're building your own "drop indicator"
*  widget, it may need this property set to the main canvas (receiving the drop) to avoid
*  itself being considered a drop target (causing the indicator to flicker).
* @type {Canvas | string}
* @default null
*/
dropTarget?: Canvas | string;

/**
* When this widget is dragged, if its dragAppearance is "target", should
*  we show a shadow behind the canvas during the drag.
* @type {boolean}
* @default null
*/
showDragShadow?: boolean;

/**
* If this widget has dragAppearance "target", this value specifies the
*  opacity to render the target while it is being dragged. A null value implies we do
*  not modify the opacity.
* @type {Integer}
* @default null
*/
dragOpacity?: Integer;

/**
* Indicates that this object can be dropped on top of other widgets. Only valid if
*  canDrag or canDragReposition is true.
* @type {boolean}
* @default false
*/
canDrop?: boolean;

/**
* Indicates that this object can receive dropped widgets (i.e. other widgets can be
*  dropped on top of it).
* @type {boolean}
* @default false
*/
canAcceptDrop?: boolean;

/**
* When explicitly set to false, disallows drop before this member in the Layout.
* @type {boolean}
* @default null
*/
canDropBefore?: boolean;

/**
* Amount of time (in milliseconds) before mouseStillDown events start to be fired
*  repeatedly for this canvas. See Canvas.mouseStillDown for details.
* @type {Integer}
* @default "400"
*/
mouseStillDownInitialDelay?: Integer;

/**
* Amount of time (in milliseconds) between repeated 'mouseStillDown' events for this
*  canvas. See Canvas.mouseStillDown for details.
* @type {Integer}
* @default "100"
*/
mouseStillDownDelay?: Integer;

/**
* Amount of time (in milliseconds) between which two clicks are considered a single click
* @type {Integer}
* @default "250"
*/
doubleClickDelay?: Integer;

/**
* If true, this canvas will receive all mouse-clicks as single Canvas.click
*  events rather than as Canvas.doubleClick events.
* @type {boolean}
* @default null
*/
noDoubleClicks?: boolean;

/**
* In earlier versions of Internet Explorer (pre IE9), a native limitation
*  exists whereby if HTML elements are
*  overlapping on the page, certain elements can appear to "burn through"
*  elements in the same position with a higher z-index.
*  Specific cases in which this have been observed include Applets,
*  &lt;IFRAME&gt; elements, and for
*  older versions of IE, native &lt;SELECT&gt; items.
*  
*  The backMask is a workaround for this issue. If useBackMask is
*  set to true, the component will render an empty &lt;IFRAME&gt; element
*  behind the canvas, which prevents this effect in all known cases.
*  
*  Has no effect in other browsers.
* @type {boolean}
* @default false
*/
useBackMask?: boolean;

/**
* If this canvas has its size specified as a percentage, this property allows the user to
*  explicitly designate another canvas upon which sizing will be based.
*  
*  If unset percentage sizing is based on
*  - the Canvas.getMasterCanvas if there is one and
*   Canvas.snapTo is set,
*  - otherwise on the amount of space available in this widget's parent canvas, if this is
*   a child of some other widget
*  - otherwise the page size.
* @type {Canvas}
* @default null
*/
percentSource?: Canvas;

/**
* Governs the model to be used when sizing canvases with percentage width or height, or
*  positioning widgets with a specified Canvas.snapTo.
*  
*  Only affects widgets with a a specified Canvas.percentSource, or
*  widgets that have Canvas.snapTo set and are peers of some
*  Canvas.getMasterCanvas.
*  
*  Determines whether the coordinates used for sizing (for percentage sized widgets) and
*  positioning (if snapTo is set) should be relative to the visible size or the
*  viewport size of the percentSource or Canvas.getMasterCanvas widget.
* @type {PercentBoxModel}
* @default "visible"
*/
percentBox?: PercentBoxModel;

/**
* Position this widget such that it is aligned with ("snapped to") an edge of its
*  Canvas.getMasterCanvas (if specified), or its
*  Canvas.getParentCanvas.
*  
*  Note that this property also impacts the sizing of this widget. If this widgets size
*  is specified as a percent value, and has no explicit
*  Canvas.percentSource, sizing will be calculated based on the size of the
*  Canvas.getMasterCanvas when snapTo is set.
*  
*  Possible values: BR, BL, TR, TL, R, L, B, T, C where B=Bottom, T=Top, L=Left, R=right
*  and C=center
*  
*  Standard snapTo behavior will attach the outer edge of the widget to the parent or master
*  element - for example setting snapTo to "B" would align the
*  bottom edge of this component with the bottom edge of the master or parent element
*  (and center this component horizontally over its master or parent element).
*  Canvas.snapEdge can be specified to change this behavior allowing the developer
*  to, for example, align the top edge of this component with the bottom edge of its
*  Canvas.getMasterCanvas.
*  
*  Canvas.snapOffsetLeft and Canvas.snapOffsetTop may also be specified to
*  offset the element from exact snapTo alignment.
* @type {string}
* @default null
*/
snapTo?: string;

/**
* If Canvas.snapTo is defined to this widget, this property can be used to
*  define which edge of this widget should be snapped to an edge of the master or parent
*  element.
*  
*  If unspecified the, default snapTo behavior is set up to align the "snapTo" edge of this
*  widget with the snapTo edge of the master or parent.
* @type {string}
* @default null
*/
snapEdge?: string;

/**
* If Canvas.snapTo is defined for this widget, this property can be used to
*  specify an offset in px or percentage for the left coordinate of this widget.
*  
*  For example if snapTo is specified as "L" and
*  snapOffsetLeft is set to 6, this widget will be rendered 6px inside the left
*  edge of its parent or master element. Alternatively if snapTo was set
*  to "R", a snapOffsetLeft value of -6 would cause the
*  component to be rendered 6px inside the right edge of its parent or Canvas.getMasterCanvas.
* @type {Integer}
* @default null
*/
snapOffsetLeft?: Integer;

/**
* If Canvas.snapTo is defined for this widget, this property can be used to
*  specify an offset in px or percentage for the top coordinate of this widget.
*  
*  For example if snapTo is specified as "T" and
*  snapOffsetTop is set to 6, this widget will be rendered 6px below the top
*  edge of its parent or master element. Alternatively if snapTo was set
*  to "B", a snapOffsetTop value of -6 would cause the
*  component to be rendered 6px inside the bottom edge of its parent or Canvas.getMasterCanvas.
* @type {Integer}
* @default null
*/
snapOffsetTop?: Integer;

/**
* Causes this canvas to snap to its parent's grid when dragging.
* @type {boolean}
* @default null
*/
snapToGrid?: boolean;

/**
* Causes this canvas to snap to its parent's grid when resizing.
*  Note that this value defaults to the Canvas's
*  Canvas.snapToGrid value if undefined.
* @type {boolean}
* @default null
*/
snapResizeToGrid?: boolean;

/**
* If true, causes this canvas's children to snap to its grid when dragging.
*  This behavior can be overridden on a per-child basis by setting the
*  Canvas.snapToGrid value on the child.
* @type {boolean}
* @default null
*/
childrenSnapToGrid?: boolean;

/**
* If true, causes this canvas's children to snap to its grid when resizing.
*  This behavior can be overridden on a per-child basis by setting the
*  Canvas.snapToGrid or
*  Canvas.snapResizeToGrid value on the child.
* @type {boolean}
* @default null
*/
childrenSnapResizeToGrid?: boolean;

/**
* The horizontal grid size to use, in pixels, when snap-to-grid is enabled.
* @type {number}
* @default "20"
*/
snapHGap?: number;

/**
* The vertical grid size to use, in pixels, when snap-to-grid is enabled.
* @type {number}
* @default "20"
*/
snapVGap?: number;

/**
* The horizontal snap direction.
*  Set this value to "before" to snap to the nearest gridpoint to the left;
*  set it to "after" to snap to the nearest gridpoint to the right; and set
*  it to "nearest" to snap to the nearest gridpoint in either direction.
* @type {string}
* @default "after"
*/
snapHDirection?: string;

/**
* The vertical snap direction.
*  Set this value to "before" to snap to the nearest gridpoint above;
*  set it to "after" to snap to the nearest gridpoint below; and set
*  it to "nearest" to snap to the nearest gridpoint in either direction.
* @type {string}
* @default "after"
*/
snapVDirection?: string;

/**
* Describes which axes to apply snap-to-grid to.
*  Valid values are "horizontal", "vertical" or "both".
* @type {string}
* @default "both"
*/
snapAxis?: string;

/**
* When this canvas is dropped onto an object supporting snap-to-grid, should it snap
*  to the grid (true, the default) or just drop wherever the mouse is (false).
* @type {boolean}
* @default true
*/
snapOnDrop?: boolean;

/**
* Should a pointer be shown? A pointer can be placed on the border to identify
*  the target of the canvas contents. The default position for the pointer is top-right.
*  The position and additional properties of the pointer is specified by
*  Canvas.pointerSettings.
*  
*  If a Canvas.pointerTarget is specified,
*  showPointer will be enabled unless it is explicitly set to false.
* @type {boolean}
* @default null
*/
showPointer?: boolean;

/**
* This property in conjunction with the PointerSettings
*  places the canvas in relation to the target canvas.
*  
*  Depending on the pointerSettings.snapTo and the
*  PointerSettings the
*  canvas will be positioned so the pointer correctly points to the target.
*  
* @type {AutoTestLocator | GlobalId}
* @default null
*/
pointerTarget?: AutoTestLocator | GlobalId;

/**
* Detail settings of the Canvas.showPointer such as where it should be located
*  along the outside of the canvas.
* @type {PointerSettings}
* @default null
*/
pointerSettings?: PointerSettings;

/**
* Should this canvas print its children absolutely positioned when generating
*  Canvas.getPrintHTML.
*  
*  By default explicitly specified absolute positioning and sizing is ignored when generating
*  print HTML. This is done intentionally: there is no way for the framework to predict how
*  explicit sizes will translate to a the printed page and if HTML for printing includes
*  the same absolute positioning and sizing as is displayed within an application it is very
*  common to encounter undesirable effects, such as seeing tables get broken over several
*  pages horizontally when there is enough room to print them on a single page of paper.
*  
*  In some cases, however, a developer may wish to have explicit sizing and positioning
*  respected within the print-view.
*  Setting this attribute to true will cause this to occur.
* @type {boolean}
* @default false
*/
printChildrenAbsolutelyPositioned?: boolean;

/**
* This boolean flag will be set to true by framework logic while generating print HTML 
*  for this widget as a result to a call to Canvas.showPrintPreview 
*  (or just Canvas.getPrintHTML). Note that this flag is set recursively as
*  parent widgets generate print HTML for their children.
*  
*  This is a read-only property and should not be modified by application code.
* @type {boolean}
* @default false
*/
isPrinting?: boolean;

/**
* Whether this canvas should be included in a printable view.
*  
*  Default is to:
*  
*  
* - omit all peers (edges generated by showEdges:true, etc)
*  
* - omit anything considered a "control", such as a button or menu (see
*  PrintProperties.omitControls)
*  
* - include everything else not marked shouldPrint:false
*  
* @type {boolean}
* @default null
*/
shouldPrint?: boolean;

/**
* Is it possible that a call to Canvas.redraw on this widget will change its 
*  size?
*  
*  Used by framework layout code when determining whether a component which has been
*  Canvas.markForRedraw needs an immediate redraw to determine
*  its drawn size.
*  
*  If unset, default behavior assumes any component with overflow set to "visible" 
*  may change size on redraw, and any component with overflow set to "hidden", "scroll",
*  or "auto" will not. This property overrides that behavior, and may be used to indicate
*  that some component with non visible overflow can change size on redraw.
*  An example use case would be a custom component with an override to explicitly 
*  resize the component as part of the redraw() flow.
* @type {boolean}
* @default null
*/
sizeMayChangeOnRedraw?: boolean;

/**
* If this property is set to true, the Canvas.destroy method
*  has been called on this canvas. This implies the canvas is no longer valid. Its ID has been
*  removed from global scope, and calling standard canvas APIs on it is likely to result in
*  errors.
* @type {boolean}
* @default null
*/
destroyed?: boolean;

/**
* This property is set to true when the Canvas.destroy method is called on a widget.
*  If this property is true, but Canvas.destroyed is not, this indicates the canvas is
*  in the process of being destroyed.
* @type {boolean}
* @default null
*/
destroying?: boolean;

/**
* The CSS border-radius for this widget. The value can be any variant of a CSS border-radius
*  value - that is, from 1 to 4 space-separated px values, where one value affects all corners
*  and 4 values affects individual corners. For example "10px" applies
*  a 10px radius to all corners, where "5px 10px 15px 20px" applies a different radius to each 
*  corner, clockwise from Top-Left: "TL TR BR BL".
* @type {string}
* @default null
*/
borderRadius?: string;

/**
* Automatically generated mask canvas displayed when Canvas.showComponentMask is called.
* @type {Canvas}
* @default null
*/
componentMask?: Canvas;

/**
* Defaults for the Canvas.componentMask autoChild.
*  Default properties include Canvas.backgroundColor being set to "black" and
*  Canvas.opacity being set to 20.
* @type {Partial<Canvas>}
* @default "{...}"
*/
componentMaskDefaults?: Partial<Canvas>;

/**
* The minimum speed in pixels per second that must be reached for momentum scrolling to kick in.
*  This setting only applies to touch-enabled devices.
* @type {Double}
* @default "200"
*/
momentumScrollMinSpeed?: Double;

/**
* Whether the Framework should correct for erroneous scrollHeight and scrollWidth values
*  reported by the browser when zoomed (via browser or OS-level zoom) by allowing 
*  Canvas.maxZoomOverflowError of overflow before enabling scrolling and displaying custom
*  scrollbars. Only relevant when Canvas.overflow is "auto".
*  
*  This property is defaulted to true in the Canvas prototype for those
*  browsers where the situation has been observed, except for Firefox, where a better solution
*  is applied that doesn't rely on Canvas.maxZoomOverflowError and never clips any content.
*  Setting this property false will disable the workaround for all browsers, including Firefox.
*  Without a workaround, scrollbars may oscillate rapidly when the browser or OS is zoomed.
* @type {boolean}
* @default "varies"
*/
correctZoomOverflow?: boolean;

/**
* When Canvas.correctZoomOverflow is true and browser or OS-level zoom is active, determines
*  how much overflow must occur before the Framework enables scrolling for this canvas.
*  
*  The larger the value, the more clipping that can occur before scrolling is enabled. So, this
*  property should never be set larger than the minimum amount needed to prevent scrollbar
*  oscillation when zoomed - which is the maximum scrollHeight or scrollWidth error.
* @type {Integer}
* @default "varies"
*/
maxZoomOverflowError?: Integer;

/**
* Should canvases with no Canvas.getParentCanvas be moved to
*  the end of the TabIndexManager tree on draw()?
*  
*  If set to false, the tab-position will not be modified on draw.
*  
*  This property is useful for cases where the tab position of a widget will be managed
*  by some explicit tabIndex management code.
* @type {boolean}
* @default true
*/
updateTabPositionOnDraw?: boolean;

/**
* Should canvases with a Canvas.getParentCanvas be added to
*  the TabIndexManager under the parent as described in Canvas.updateChildTabPositions and
*  Canvas.updateChildTabPosition?
*  
*  If set to false, the tab-position will not be modified on parent change.
*  
*  This property is useful for cases where the tab position of a widget will be managed
*  by something other than the parent canvas, for example for 
*  CanvasItem.canvas.
* @type {boolean}
* @default true
*/
updateTabPositionOnReparent?: boolean;

/**
* Should focus navigation for this canvas and its descendents be handled explicitly
*  by intercepting "Tab" key events and calling the TabIndexManager.shiftFocus
*  API?
*  
*  Setting this property to true will cause the registered TabIndexManager
*  entry for this canvas to be marked as 
*  TabIndexManager.setUseExplicitFocusNavigation,
*  and will cause standard event handling for the canvas and its descendents to 
*  intercept Tab keystrokes and explicitly call TabIndexManager.shiftFocus
*  rather than relying on native browser Tab navigation
* @type {boolean}
* @default null
*/
alwaysManageFocusNavigation?: boolean;

/**
* When set to true, shows a widget hovering at the mouse point instead of the builtin
*  hover label. Override Canvas.getHoverComponent to provide the
*  Canvas to show as the hoverComponent.
* @type {boolean}
* @default false
*/
showHoverComponents?: boolean;

/**
* Screen to create (via RPCManager.createScreen) in lieu of calling
*  Canvas.getHoverComponent.
* @type {string}
* @default null
*/
hoverScreen?: string;

/**
* For canvases Canvas.showHover, this attribute gives users a way to
*  pin the hover in place so they can interact with it (scroll it, click embedded links, etc).
*  Clicking outside the hover (on some other component) will still dismiss it.
*  
*  When enabled, the standard hover canvas will display the Hover.focusKeyHintLabel
*  to prompt the user on how to give the hover focus. This behavior may be disabled by 
*  setting Hover.showFocusKeyHint to false
*  
*  Note that setting a hoverFocusKey also sets Canvas.hoverPersist to
*  "clickPin". This makes hovers stay visible while the cursor is over them, and become pinned
*  if clicked, the same as pressing the hoverFocusKey.
* @type {KeyName}
* @default null
*/
hoverFocusKey?: KeyName;

/**
* Whether to show a snap grid for this Canvas. Note that the grid is only shown
*  when either Canvas.childrenSnapToGrid or Canvas.childrenSnapResizeToGrid
*  is true.
*  
*  Grid is based on Canvas.snapHGap and Canvas.snapVGap properties.
* @type {boolean}
* @default null
*/
showSnapGrid?: boolean;

/**
* Specifies indication style to use for snap points, either a grid of lines or an array of
*  crosses. The lines can be configured using the property Canvas.snapGridLineProperties.
* @type {SnapGridStyle}
* @default "crosses"
*/
snapGridStyle?: SnapGridStyle;

/**
* Specifies line styling to use when drawing the grid of lines for SnapGridStyle:
*  "lines".
* @type {Partial<DrawLine>}
* @default null
*/
snapGridLineProperties?: Partial<DrawLine>;

/**
* If enabled while Canvas.childrenSnapToGrid is enabled, children dragged within this Canvas
*  will also snap to positions where their edges or center lines would be aligned with the
*  edges or centers of other components, and lines will be shown to point out the possible
*  alignment (with appearance controlled by Canvas.snapAlignCenterLineStyle and
*  Canvas.snapAlignEdgeLineStyle respectively.
*  
*  By default, edge- or center-snapping is enabled for all components, but the set of eligible
*  components can be explicitly set via Canvas.snapAlignCandidates.
*  
*  See also Canvas.childrenSnapCenterAlign and Canvas.childrenSnapEdgeAlign for enabling or
*  disabling center alignment or edge alignment individually.
* @type {boolean}
* @default null
*/
childrenSnapAlign?: boolean;

/**
* Flag to disable snapping to alignment when children of this Canvas are resized
* @type {boolean}
* @default null
*/
childrenResizeSnapAlign?: boolean;

/**
* See Canvas.childrenSnapAlign. This setting enables or disables snapping on center
*  alignment only.
* @type {boolean}
* @default null
*/
childrenSnapCenterAlign?: boolean;

/**
* See Canvas.childrenSnapAlign. This setting enables or disables snapping on edge
*  alignment only.
* @type {boolean}
* @default null
*/
childrenSnapEdgeAlign?: boolean;

/**
* Flag to disable snapping to alignment when this Canvas is dragged when 
*  Canvas.childrenSnapAlign is enabled on this Canvas' parent.
*  
*  To control snapping to align for the children dragged within this Canvas, see
*  Canvas.childrenSnapAlign instead.
* @type {boolean}
* @default null
*/
snapToAlign?: boolean;

/**
* Flag to disable snapping to alignment when this Canvas is resized.
*  
*  To control snapping to align for the children resized within this Canvas, see
*  Canvas.childrenResizeSnapAlign instead.
* @type {boolean}
* @default null
*/
snapResizeToAlign?: boolean;

/**
* Flag to disable snapping to center alignment when this Canvas is dragged when 
*  Canvas.childrenSnapAlign is enabled on this Canvas' parent.
* @type {boolean}
* @default null
*/
snapToCenterAlign?: boolean;

/**
* Flag to disable snapping to edge alignment when this Canvas is dragged when 
*  Canvas.childrenSnapAlign is enabled on this Canvas' parent.
* @type {boolean}
* @default null
*/
snapToEdgeAlign?: boolean;

/**
* Flag to disable snapping to alignment against this Canvas when other Canvases
*  dragged into the same parent when Canvas.childrenSnapAlign is enabled on this
*  Canvas' parent.
* @type {boolean}
* @default null
*/
isSnapAlignCandidate?: boolean;

/**
* When Canvas.childrenSnapAlign is enabled, list of candidates to check for alignment.
*  
*  If a list of snapAlignCandidates is never provided, the default is to use all
*  children that are not explicitly excluded via Canvas.isSnapAlignCandidate, including
*  automatically adding newly added children as candidates, and ignoring children that have
*  been removed. Use Canvas.addSnapAlignCandidate and Canvas.removeSnapAlignCandidate to add
*  and remove special candidates while retaining all children as default candidates.
*  
*  Possible candidates which are not drawn or are hidden are automatically ignored.
* @type {Array<Partial<Canvas>>}
* @default null
*/
snapAlignCandidates?: Array<Partial<Canvas>>;

/**
* CSS border declaration used for the line shown to indicate snapping to a center line when
*  Canvas.childrenSnapAlign is set.
* @type {string}
* @default "1px solid blue"
*/
snapAlignCenterLineStyle?: string;

/**
* CSS border declaration used for the line shown to indicate snapping to a edge line when
*  Canvas.childrenSnapAlign is set.
* @type {string}
* @default "1px solid #555555"
*/
snapAlignEdgeLineStyle?: string;

/**
* If set, forces the main SVG image or icon in the canvas to be rendered in an image tag rather
*  than an object tag, the default. Typical use cases might be configuring the image of an
*  Img or ImgButton, or the icon of a Button.
*  
*  Rendering via object tag provides the maximum support for CSS in SVG, but may result in a
*  flicker at the browser level when changing images - either manually such as with
*  Canvas.setImage or via state change from rollover, mouseDown, etc. Using image tags to
*  inline the images breaks CSS support but may avoid flickering.
*  
*  If this property is not set, then you can also control whether an SVG image is
*  rendered in an object or image tag by setting the query param "tag" on the image URL - see
*  SCImgURL for details.
*  
*  Note that if multiple icons are potentially present in a canvas (e.g.
*  ListGrid.removeIcon in the cells of a grid body), then setting this
*  property on the widget instance may have no effect. In such case, the Canvas
*  prototype is consulted.
* @type {boolean}
* @default false
*/
useImageForSVG?: boolean;

/**
* If true, events sent to the SVG inside an object tag are forwarded to the object itself by
*  Framework handlers. If false, "pointer-events:none" CSS is set on the object so that events
*  are directly sent to the object by the browser, if supported.
*  
*  The advantage of forwarding events is that it allows hover CSS or any other event-driven
*  logic on the SVG to work as expected. If "pointer-events:none" is written out, no GUI
*  interaction will trigger events in the SVG, including hover CSS. The disadvantage is that
*  only a few critical events are forwarded, such as "mouseDown", "mouseMove", and "mouseUp" for
*  non-touch platforms, and "click" for touch platforms. Other events will be delivered to the
*  SVG, but not forwarded up to the parent document/object tag.
* @type {boolean}
* @default true
*/
forwardSVGeventsToObject?: boolean;

/**
* Whether an EdgedCanvas should be used to show image-based edges around this
*  component.
* @type {boolean}
* @default false
*/
showEdges?: boolean;

/**
* Size in pixels for corners and edges
* @type {number}
* @default "10"
*/
edgeSize?: number;

/**
* Amount the contained Canvas should be offset. Defaults to edgeSize; set to less than
*  edgeSize to allow the contained Canvas to overlap the edge and corner media.
* @type {number}
* @default null
*/
edgeOffset?: number;

/**
* Base name of images for edges. Extensions for each corner or edge piece will be added
*  to this image URL, before the extension. For example, with the default base name of
*  "edge.gif", the top-left corner image will be "edge_TL.gif".
*  
*  The full list of extensions is: "_TL", "_TR", "_BL", "_BR", "_T", "_L", "_B", "_R",
*  "_center".
* @type {SCImgURL}
* @default "[SKIN]edge.gif"
*/
edgeImage?: SCImgURL;

/**
* Array of side names ("T", "B", "L", "R") specifying which sides of the decorated
*  component should show edges. For example: 
*    customEdges : ["T", "B"]
*  
*  .. would show edges only on the top and bottom of a component.
*  
*  The default of null means edges will be shown on all sides.
* @type {Array<Partial<string>>}
* @default null
*/
customEdges?: Array<Partial<string>>;

/**
* Background color for the EdgedCanvas created to decorate this component. This can be used
*  to provide an underlying "tint" color for translucent edge media
* @type {CSSColor}
* @default null
*/
edgeBackgroundColor?: CSSColor;

/**
* Opacity of the edges. Defaults to matching this.opacity.
*  if Canvas.setOpacity is called on a Canvas where edgeOpacity is set,
*  edgeOpacity will be considered a percentage of the parent's opacity (so 50% opaque parent plus
*  edgeOpacity 50 means 25% opaque edges)
* @type {Integer}
* @default null
*/
edgeOpacity?: Integer;

/**
* Whether to show media in the center section, that is, behind the decorated Canvas.
* @type {boolean}
* @default false
*/
edgeShowCenter?: boolean;

/**
* Background color for the center section only. Can be used as a surrogate background
*  color for the decorated Canvas, if the Canvas is set to partially overlap the edges and
*  hence can't show a background color itself without occluding media.
* @type {CSSColor}
* @default null
*/
edgeCenterBackgroundColor?: CSSColor;

/**
* Whether to show a drop shadow for this Canvas. 
*  
*  Shadows may be rendered using Canvas.useCSSShadow or via images. 
*  The appearance of shadows can be customized via
*  Canvas.shadowColor (for css-based shadows) or Canvas.shadowImage 
*  (for image based shadows), Canvas.shadowDepth,
*  Canvas.shadowOffset and Canvas.shadowSoftness.
*  
*  When Canvas.useCSSShadow is false, developers should be aware that the drop shadow
*  is rendered as a Canvas.peers and is drawn outside the specified 
*  width and height of the widget meaning a widget with shadows 
*  takes up a little more space than it otherwise would. A full screen canvas with showShadow set 
*  to true as this would be likely to cause browser scrollbars to appear - developers can handle
*  this by either setting this property to false on full-screen widgets, or by setting
*  overflow to "hidden" on the &lt;body&gt; element if browser-level scrolling is
*  never intended to occur.
* @type {boolean}
* @default false
*/
showShadow?: boolean;

/**
* Depth of the shadow, or the virtual height above the page of the widget throwing the shadow.
*  
*  This is a single parameter that can be used to control both shadowSoftness and
*  shadowOffset.
* @type {number}
* @default "4"
*/
shadowDepth?: number;

/**
* Offset of the shadow. Defaults to half of shadowDepth if unset.
*  
*  Because of the blurred edges, a shadow is larger than the originating component by
*  2xsoftness. An shadowOffset of 0 means that the shadow will extend around the
*  originating component equally in all directions.
*  
*  If Canvas.useCSSShadow are being used, separate vertical and
*  horizontal offsets may be specified via Canvas.shadowHOffset and
*  Canvas.shadowVOffset.
* @type {number}
* @default null
*/
shadowOffset?: number;

/**
* Horizontal offset for the Canvas.showShadow. Takes precedence over 
*  Canvas.shadowOffset if set. Has no effect if Canvas.useCSSShadow
*  are not being used for this canvas.
* @type {number}
* @default null
*/
shadowHOffset?: number;

/**
* Vertical offset for the Canvas.showShadow. Takes precedence over 
*  Canvas.shadowOffset if set. Has no effect if Canvas.useCSSShadow
*  are not being used for this canvas.
* @type {number}
* @default null
*/
shadowVOffset?: number;

/**
* Softness, or degree of blurring, of the shadow.
*  
*  A shadow with softness:x is 2x pixels larger in each direction than the element
*  throwing the shadow, and the media for each edge should be x pixels wide/tall.
*  
*  Defaults to shadowDepth if unset.
* @type {number}
* @default null
*/
shadowSoftness?: number;

/**
* If Canvas.useCSSShadow is false, (or for browsers that do not support
*  css3), this property supplies the base name of the series of images for the sides, 
*  corners, and center of the shadow.
*  
*  The actual image names fetched for the dropShadow combine the segment name and the
*  shadowDepth setting. For example, given "ds.png" as the base name, a depth of
*  4, and the top-left segment of the shadow, we'd use "ds4_TL.png".
*  
*  The names for segments are the same as those given for controlling resizable edges; see
*  Canvas.resizeFrom. The center segment has the name "center". The center segment is
*  the only segment that doesn't include the depth in the URL, so the final image name for the
*  center given a baseName of "ds.png" would be just "ds_center.png".
* @type {SCImgURL}
* @default "[SKIN]ds.png"
*/
shadowImage?: SCImgURL;

/**
* If Canvas.showShadow is true, should we use the css box-shadow property
*  (where supported) to achieve the shadow?
*  
*  Set this property to false to switch to a media-based approach, achieved by
*  rendering the Canvas.shadowImage in an automatically generated peer.
*  This approach is also used regardless of this property value in older browsers 
*  where the css box-shadow property isn't supported.
*  
*  See also Canvas.shadowColor, Canvas.shadowDepth,
*  Canvas.shadowOffset, Canvas.shadowSoftness and Canvas.shadowSpread
* @type {boolean}
* @default true
*/
useCSSShadow?: boolean;

/**
* Color for the css-based drop shadow shown if Canvas.useCSSShadow is true 
*  and Canvas.showShadow is true.
*  
*  Has no effect if we are not using css-based shadows - in that case, use
*  Canvas.shadowImage instead.
* @type {CSSColor}
* @default "#AAAAAA"
*/
shadowColor?: CSSColor;

/**
* Explicit spread for the css-based drop shadow shown if Canvas.useCSSShadow is true 
*  and Canvas.showShadow is true. This property governs how much larger than the
*  widget the shadow will appear. A negative value (coupled with an explicit offset) will
*  result in a smaller shadow.
*  
*  Has no effect if we are not using css-based shadows - in that case, use
*  Canvas.shadowImage instead.
* @type {Integer}
* @default null
*/
shadowSpread?: Integer;

/**
* Should a grouping frame be shown around this canvas if a non-empty string has been specified
*  for Canvas.groupTitle.
* @type {boolean}
* @default true
*/
isGroup?: boolean;

/**
* Sets the style for the grouping frame around the canvas. Only necessary when showing a
*  Canvas.isGroup.
* @type {string}
* @default "2px solid black"
*/
groupBorderCSS?: string;

/**
* Sets the style for the grouping label. Only necessary when showing a Canvas.isGroup.
*  
*  Note that Canvas.groupLabelBackgroundColor overrides any background-color of this style.
* @type {CSSStyleName}
* @default "groupLabel"
*/
groupLabelStyleName?: CSSStyleName;

/**
* If set, the background color of the grouping label. Only applicable when showing a
*  Canvas.isGroup.
*  
*  This corresponds to the CSS background-color property on the grouping label. You can set this
*  property to an RGB value (e.g. #22AAFF) or a named color (e.g. red) from a list of browser
*  supported color names.
*  
* @type {CSSColor}
* @default null
*/
groupLabelBackgroundColor?: CSSColor;

/**
* Padding to apply inside the border when this canvas is showing a group
*  Canvas.isGroup.
* @type {Integer}
* @default "10"
*/
groupPadding?: Integer;

/**
* When showing this widget in a Canvas.isGroup, should any content be shifted 
*  down so that the group-label doesn't sit in front of it?
* @type {boolean}
* @default true
*/
leaveGroupLabelSpace?: boolean;

/**
* The title/label for the grouping. Only applicable when Canvas.isGroup is set
*  to true. No Canvas.isGroup or title/label will be shown unless this property
*  is a non-empty string.
* @type {HTMLString}
* @default null
*/
groupTitle?: HTMLString;

/**
* A mapping from DataSource IDs to specific Record from those DataSources, that
*  DataBoundComponent contained within this Canvas should automatically
*  bind to if a DataSource is provided but data is not provided (directly or indirectly, for example, indirectly
*  via setting ListGrid.autoFetchData.
*  
*  See Canvas.autoPopulateData for details on how this is done.
*  
*  DataSources included in the dataContext are immediately provided to
*  Canvas.ruleScope when used if no other component has done so already.
*  These records are found in rule context 'dataContext' section (ex.
*  dataContext.Customer for a Customer record in dataContext)
*  so they do not conflict with normal DataSource records.
* @type {DataContext}
* @default null
*/
dataContext?: DataContext;

/**
* A DataContext to be used if no Canvas.dataContext is provided (directly
*  or indirectly via a parent). If a DataContext is provided it completely replaces the
*  testDataContext.
*  
*  DataSources included in the testDataContext are immediately provided to
*  Canvas.ruleScope when used if no other component has done so already.
*  These records are found in rule context 'dataContext' section (ex.
*  dataContext.Customer for a Customer record in testDataContext)
*  so they do not conflict with normal DataSource records.
* @type {DataContext}
* @default null
*/
testDataContext?: DataContext;

/**
* If a Canvas.dataContext is provided, should subcomponents automatically bind to the data?
*  
*  In general, if you provide a DataSourceField.primaryKey value for a 
*  record, that specific record will be shown or edited, or have its related records loaded.
*  If you provide other field values without the primaryKey value, those values
*  will be treated as either criteria (for a grid) or as initial values (for a form).
*  
*  Components will respond to dataContext differently depending on whether they
*  typically work with just a single record (such as DynamicForm or a 
*  DetailViewer), or whether they usually work with lists of records (such as 
*  ListGrid or TileGrid, which have 
*  DataBoundComponent.dataArity set to multipe by default).
*  
*  Specifically, the following rules are used:
*  
*   
* - for a singular component (eg forms, detailViewers): 
*    
*     
* - if only the PK (primary key) value is provided, the component will fetch the
*       singular record and display or edit it
*     
* - if the PK is provided along with other values, the component will
*       assume it has a complete record, and display or edit it
*     
* - if only non-PK values are provided, the component will assume these are initial
*       values
*    
*   
* - for a multiple component (eg listGrid, tileGrid):
*    
*     
* - if only non-PK values are provided, the component will use these as criteria. 
*       For example, a grid bound to an Orders DataSource with
*       dataContext of Status:"In Process" would fetch records with that
*       status
*     
* - if the PK is provided, and the component's DataSource is related to
*       one of the DataSources in the dataContext, related records will be
*       fetched, using the PK value (similarly to if 
*       ListGrid.fetchRelatedData had been called.
*       For example, if an orderNumber value was provided, a grid bound to
*       OrderDetail would fetch line items for that orderNumber.
*     
* - if an array of multiple records is provided, that data is used as if setData()
*       had been called on the component
*    
*  
*  
*  If any of these behaviors is not desired, you can just set autoPopulateData
*  to false on the specific component that should not be auto-populated.
*  
*  Specific examples, using the Order and OrderDetail sample DataSources,
*  where OrderDetail records are associated (many-to-1) with Order records, 
*  and the PK of Order is orderNumber:
*  
*   
* - a form or DetailViewer bound to Order and a ListGrid bound to
*     OrderDetail:
*    
*     
* - if an orderNumber is provided, the form or DetailViewer would show
*       that order, and the grid would show related OrderDetail records
*     
* - if only orderNumber is provided (no additional fields), the full
*       Order record is automatically fetched for the form or DetailViewer
*    
*    
*    This fulfills a common use case of viewing or editing an Order and its
*    related OrderDetail records.
*    
*   
* - a grid bound to Order and a second grid bound to OrderDetail
*    
*     
* - if values such as orderStatus : "On Hold" were provided, the 
*       Order grid uses those as criteria
*     
* - the OrderDetail grid does nothing
*    
*    
*    This fulfills a common use case of viewing Order records that match certain 
*    criteria. The OrderDetail grid would generally be populated only by an event
*    handler installed on the Order grid, which would call
*    ListGrid.fetchRelatedData.
*    
*   
* - a grid and form both bound to Order
*    
*     
* - if values such as orderStatus : "On Hold" were provided, the
*       Order grid uses those as criteria, and the form uses those as initial
*       values
*    
*    
*    This fulfills two possible use cases:
*    
*     
* - the form is a SearchForm for searching the grid, so it should show the same
*       criteria as are applied to the grid
*     
* - the form is for editing the selected record in the grid, or adding new ones.
*       It is ready to either add a new record, or for the user to select a record to
*       show in the form, typically via logic added to the grid
*       (e.g. ListGrid.recordClick -&gt; DynamicForm.editSelectedData.
*    
*  
*  
*  By default, autoPopulateData is true for any component that is contained with a "screen"
*  RPCManager.createScreen and false for any component that is an AutoChild.
*  
*  Note that, if there is a non-DBC in your screen that wants to access fields in the expected
*  dataContext, the DynamicProperty can refer to properties that auto-populated DBCs will
*  place into the ruleContext. For example, with a DynamicForm "itemEditor" bound to
*  "supplyItem", a header above could use itemEditor.values.itemName to display the name of
*  the item. The header will then show the expected value as soon as the DynamicForm is
*  auto-populated.
*  
*  Similarly, the Canvas.drawn StringMethod fires after auto-population has occurred,
*  so any startup actions in a screen will likewise be able to utilize data from the
*  dataContext by just referring to it via ruleScope.
* @type {boolean}
* @default null
*/
autoPopulateData?: boolean;

/**
* Canvas.ID of the component that gathers the context for evaluation of criteria-based
*  rules specified by properties such as FormItem.visibleWhen, 
*  Class.dynamicProperties, and dynamicCriteria such as 
*  ListGrid.initialCriteria.
*  
*  If ruleScope is not specified, this component will search through its Canvas.parentCanvas 
*  chain until it either reaches the top or reaches a parent marked Canvas.isRuleScope. 
*  This means that typically, ruleScope does not have to be explicitly specified, since 
*  components that want to reference each other often have the same top-level parent (eg, they are 
*  part of the same screen). However, you would need to specify ruleScope in scenarios 
*  such as a modal Window that wants to reference values in the component it was launched from.
*  
*  Determination of the ruleScope happens when the component is first drawn.
*  
*  The component designated as the ruleScope manages a nested data structure called
*  the "rule context" which contains information from all DataBoundComponents that are
*  registered with the ruleScope. By specifying Criterion.fieldName as a
*  DataPath, AdvancedCriteria defined in properties such as
*  FormItem.visibleWhen can access any part of the rule context.
*  
*  By default, the rule context contains data as follows:
*  
*  
* - any DataBoundComponent that has a DataSource contributes the values of the
*   selected record or record being edited under the ID of the DataSource. For any collision
*   an editable display (such as a form or editable grid) wins over a static display (such as
*   a non-editable grid with a selection.) Hidden or cleared components have lowest priority
*   even if editable. For two editable components the first becomes the contributor.
*  
* - any ListGrid or other component that manages a selection and has been assigned an explicit
*   Canvas.ID will contribute the values of the selected record under
*   &lt;componentId&gt;.selectedRecord, the values of the grid summary record
*   under &lt;componentId&gt;.summaryRecord, and also contributes 3 flags for
*   checking for selection: anySelected, multiSelected,
*   numSelected.
*  
* - any DynamicForm or other component that edits values and has been assigned an explicit
*   Canvas.ID contributes its current values under
*   &lt;componentId&gt;.values, and contributes a flag hasChanges.
*  
* - any DynamicForm or ListGrid that has been assigned an explicit Canvas.ID
*   contributes a value &lt;componentId&gt;.focusField. When present the value
*   indicates the component has focus along with the name of the field that has focus. Its absense
*   indicates the component does not have focus at all.
*  
* - any ListGrid that has been assigned an explicit Canvas.ID contributes a flag
*   isGrouped under &lt;componentId&gt;.
*  
* - any DataSource included in a DataContext or Canvas.testDataContext that is
*   being used for this ruleScope contributes the values into the dataContext
*   section of the ruleContext (ex. dataContext.Customer) so the values do
*   not conflict with normal DataSource contributions. Note that the dataContext
*   is immutable so only the first contribution is actually saved.
*  
*  For example, given a screen where:
*  
*  
* - a ListGrid with ID "itemGrid" and DynamicForm with ID "itemForm" are both bound to the
*   supplyItem sample DataSource
*  
* - the ListGrid has a single selection, and the record selected in the ListGrid is being
*   edited in the form, and has been changed
*  
*  The default rule context available from Canvas.getRuleContext, expressed as JSON, would be:
*  
*  {
*  supplyItem : {
*    itemID : "654321",
*    itemName : "Sewing Machine",
*    price : 5.50, // note: user change
*    ..other properties..
*  },
*  itemForm.values : {
*    itemID : "654321",
*    itemName : "Sewing Machine",
*    price : 5.50, // note: user change
*    ..other properties..
*  },
*  itemForm.focusField : "itemName",
*  itemForm.hasChanges : true,
*  itemGrid.selectedRecord : {
*    itemID : "654321",
*    itemName : "Sewing Machine",
*    price : 3.50, // note: old price
*    ..other properties..
*  },
*  itemGrid.anySelected : true,
*  itemGrid.multiSelected : false,
*  itemGrid.numSelected : 1,
*  itemGrid.isGrouped : false
*  }
*  
*  In addition, an application can put custom data into the ruleScope via Canvas.provideRuleContext.
*  
*  Troubleshooting RuleScope issues
* 
*  The ruleScope system consists of two major parts: managing the rule context and using the rule context.
*  The former is handled by Canvas.provideRuleContext and Canvas.removeRuleContext. Users of rule context
*  are the criteria-based rules noted above.
*  
*  A number of Log categories are provided to make diagnosing ruleScope issues easier:
*  
*   
* - ruleContext
*   
* - whenRules
*   
* - dynamicProperties
*   
* - dynamicCriteria
*  
*  Except for configuration warnings, nothing is logged for these categories unless the log priority
*  is set to either Info or Debug. The priorities can be configured and the log messages
*  reviewed in debugging. By default none of these categories
*  are included in the Logging Preferences category choices. To add one or more of these
*  categories, select More... then click Add.... Be sure to enter the category
*  with the correct capitalization.
*  
*  At the Info log priority one or two basic log messages are generated per action.
*  The Debug priority generally includes additional information like the full criteria
*  or a message for each evaluation even when no change is made.
* 
*  Using and Interpreting log messages
* 
*  To address a specific issue on rules, select the Info priority for the target
*  category in the Developer Console then perform your application actions that are being
*  diagnosed. You may want to re-run your application to get a clean start on the changes.
* 
*  whenRules and dynamicProperties
* 
*  Both of these types of properties are triggered by a criteria that can reference the ruleContext
*  and/or the component's values (for forms). The criteria is re-evaluated on each rule context
*  change and based on the criteria being matched or not, the action is taken or property updated.
*  
*  Below are some example Info log messages:
* 
*  
*  16:53:39.032:TMR8[E0]:INFO:whenRules:Hide Canvas 'isc_Canvas_1'
*  16:53:39.034:TMR8[E1]:INFO:whenRules:Disabling Canvas 'isc_Button_3'
*  16:53:39.509:TMR9[E0]:INFO:whenRules:Hide Canvas 'isc_Canvas_2'
*  16:53:39.510:TMR9[E1]:INFO:whenRules:Hide Canvas 'isc_Button_4'
*  16:53:39.527:TMR4:INFO:whenRules:Show Canvas 'isc_Canvas_2'
*  16:53:39.527:TMR4:INFO:whenRules:Show Canvas 'isc_Button_4'
*  16:53:39.528:TMR4:INFO:whenRules:Disabling Canvas 'isc_Button_4'
*  16:53:39.529:TMR4:INFO:whenRules:Set read-only (appearance disabled) FormItem 'itemForm3.field2'
*  16:53:39.974:TMR6:INFO:whenRules:Set editable (appearance disabled) FormItem 'itemForm3.field2'
*  16:53:40.540:TMR2:INFO:whenRules:Enabling FormItemIcon 'itemForm4.field2.remove'
*  16:53:40.541:TMR2:INFO:whenRules:Hide FormItemIcon 'itemForm4.field4.edit'
*  
* 
*  These indicate various responses to visibleWhen, enableWhen, and
*  readOnlyWhen actions.
*  
*  By setting the priority to Debug you can see the creation of the rule for the
*  criteria and the evaluation of the criteria before action is taken:
*  
*  
*  16:53:41.569:TMR7:DEBUG:whenRules:itemForm6:Create whenRule: itemForm6_field2_readOnly, applyWhen {
*    "_constructor":"AdvancedCriteria", 
*    "operator":"and", 
*    "criteria":[
*      {
*        "fieldName":"layout1.date1", 
*        "operator":"greaterOrEqualField", 
*        "value":"layout1.date2"
*      }
*    ]
*  } to target {component: [DynamicForm ID:itemForm6], fieldName: "field2", formIconName: undef}
*  16:53:41.571:TMR7:DEBUG:whenRules:isc_RulesEngine_5:Applying readOnly rule 'itemForm6_field2_readOnly' for FormItem itemForm6.field2 with action result true. applyWhen criteria matched: {
*    "__normalized":true, 
*    "_constructor":"AdvancedCriteria", 
*    "operator":"and", 
*    "criteria":[
*      {
*        "__normalized":true, 
*        "fieldName":"layout1.date1", 
*        "operator":"greaterOrEqualField", 
*        "value":"layout1.date2"
*      }
*    ]
*  }
*  16:53:41.571:TMR7:DEBUG:whenRules:Set read-only (appearance disabled) FormItem 'itemForm6.field2'
*  
* 
*  dynamicProperties are similar:
* 
*  
*  16:59:13.588:XRP8:INFO:dynamicProperties:Set (populateFromDataPath) 'isc_Button_0.title' to [initial value]: Special Title
*  16:59:13.650:TMR2:WARN:Class:isc_Class_0:Attempt to evaluate formulaFunction itemForm.values.count * 2 + 10 failed. Error message:undefined is not an object (evaluating 'itemForm.values')
*  16:59:13.657:TMR9:INFO:dynamicProperties:Set (populate) 'isc_Class_0.contents' to [initial value]: 34
*  
* 
*  This example shows a failed formula indicating that the 'itemForm.values' isn't valid in the ruleContext.
*  In this case the form values haven't been provided to ruleContext yet. When they are the next log message
*  shows the result.
* 
*  dynamicCriteria
* 
*  Each time a dynamic criteria is evaluated (as triggered by a ruleContextChanged event)
*  a log entry is made:
*  
*  14:54:46.436:INP9:INFO:dynamicCriteria:isc_ListGrid_0:Evaluated dynamic criteria: filterCriteria
*  
*  This indicates that dynamicCriteria for isc_ListGrid_0 is being evaluated/re-evaluated. By changing
*  the priority to Debug the resolved criteria is also included:
* 
*  
*  14:55:16.231:INP9:DEBUG:dynamicCriteria:isc_ListGrid_0:Evaluated dynamic criteria: filterCriteria {
*    "fieldName":"orderNumber", 
*    "operator":"equals", 
*    "valuePath":"orderSearchForm.values.orderNumber", 
*    "value":10104, 
*    "_constructor":"AdvancedCriteria"
*  }
*  
* 
*  Note the valuePath of orderSearchForm.values.orderNumber is resolved to the value of
*  10104.
*  ruleContext
*  If none of the logging above provides details needed to understand what is going on, enabling the
*  ruleContext category can be enabled. At Info a single log message is reported for
*  each ruleContext change.
*  
*  For example, below are the rule context changes that occurred to trigger the above dynamicCriteria
*  messages above:
*  
*  14:54:46.432:INP9:INFO:ruleContext:isc_SearchForm_0:provideRuleContext [ruleScope: isc_DataView_0, dbcID: isc_SearchForm_0]: Order.orderNumber = 10104
*  14:54:46.433:INP9:INFO:ruleContext:isc_SearchForm_0:provideRuleContext [ruleScope: isc_DataView_0, dbcID: isc_SearchForm_0]: orderSearchForm.values.orderNumber = 10104
*  14:54:46.434:INP9:INFO:ruleContext:isc_SearchForm_0:provideRuleContext [ruleScope: isc_DataView_0, dbcID: isc_SearchForm_0]: orderSearchForm.hasChanges = true
*  
*  This indicates that isc_SearchForm_0 updated rule context with three changed values. There is no
*  indication of when a ruleContextChanged event is raised. If that is important, change
*  the log priority to Debug to see more details:
*  
*  15:16:43.291:INP9:DEBUG:ruleContext:isc_SearchForm_0:provideRuleContext [ruleScope: isc_DataView_0, dbcID: isc_SearchForm_0] START TRANSACTION
*  15:16:43.292:INP9:DEBUG:ruleContext:isc_SearchForm_0:provideRuleContext [ruleScope: isc_DataView_0, dbcID: isc_SearchForm_0]: Order.orderNumber = 10104
*  15:16:43.292:INP9:DEBUG:ruleContext:isc_SearchForm_0:provideRuleContext [ruleScope: isc_DataView_0, dbcID: isc_SearchForm_0]: orderSearchForm.values.orderNumber = 10104
*  15:16:43.293:INP9:DEBUG:ruleContext:isc_SearchForm_0:provideRuleContext [ruleScope: isc_DataView_0, dbcID: isc_SearchForm_0]: orderSearchForm.hasChanges = true
*  15:16:43.294:INP9:DEBUG:ruleContext:isc_SearchForm_0:provideRuleContext [ruleScope: isc_DataView_0, dbcID: isc_SearchForm_0] END TRANSACTION
*  
*  Here the isc_SearchForm_0 provides rule context changes as part of a 'transaction'. That is, the three
*  values are provided separately but a single ruleContextChanged event is raised at the end
*  of the 'transaction'. If only one change is part of a 'transaction' the start/end log messages are
*  suppressed. For example, the isc_ListGrid_0 updates rule context as a result of the change in the
*  dynamic criteria. The criteria is provided to rule context separately from dataLoading
*  updates:
*  
*  15:16:43.305:INP9:DEBUG:ruleContext:isc_ListGrid_0:provideRuleContext [ruleScope: isc_DataView_0, dbcID: isc_ListGrid_0]: orderDetailGrid.criteria = {fieldName: "orderNumber",
*        operator: "equals", valuePath: "orderSearchForm.values.orderNumber", value: 10104, _constructor: "AdvancedCriteria"}
*  15:16:43.317:INP9:DEBUG:ruleContext:isc_ListGrid_0:provideRuleContext [ruleScope: isc_DataView_0, dbcID: isc_ListGrid_0]: orderDetailGrid.dataLoading = true
*  15:16:44.228:TMR2:DEBUG:ruleContext:isc_ListGrid_0:provideRuleContext [ruleScope: isc_DataView_0, dbcID: isc_ListGrid_0]: orderDetailGrid.dataLoading = null
*  
* @type {string}
* @default null
*/
ruleScope?: string;

/**
* Marks this Canvas as the Canvas.ruleScope that will be discovered by any contained
*  DataBoundComponents which do not specify an explicit ruleScope.
* @type {boolean}
* @default null
*/
isRuleScope?: boolean;

/**
* ValuesManager for managing values displayed in this component.
*  If specified at initialization time, this component will be added to the valuesManager via
*  ValuesManager.addMember.
*  
*  ValuesManagers allow different fields of a single object to be displayed or edited
*  across multiple UI components. Given a single values object, a valuesManager will handle
*  determining the appropriate field values for its member components and displaying them / 
*  responding to edits if the components support this.
*  
*  Data may be derived simply from the specified fieldNames within the member components, or for
*  complex nested data structures can be specified by both component and field-level
*  DataPath.
*  
*  Note that components may be automatically bound to an existing valuesManager attached to a 
*  parent component if dataPath is specified. See Canvas.dataPath for more information.
*  Also note that if a databound component has a specified dataSource and dataPath but no specified
*  valuesManager object one will be automatically generated as part of the databinding process
* @type {ValuesManager | GlobalId}
* @default null
*/
valuesManager?: ValuesManager | GlobalId;

/**
* A dataPath may be specified on any canvas. This provides a straightforward way to display or
*  edit complex nested data.
*  
*  NOTE: the dataPath feature is intended to help certain legacy architectures, 
*  such as systems that work in terms of exchanging large messages with several different 
*  entity types in one message, and are incapable of providing separate access to each 
*  entity type.
*  See the DataPath for more information.
*  
*  For components which support displaying or editing data values, (such as DynamicForm or
*  ListGrid components), the dataPath may be set to specify how the components data is
*  accessed. In this case the dataPath essentially specifies a nested object to edit - typically
*  a path to a field value within a dataSource record. Note that a ValuesManager will be required
*  to handle connecting the dataBoundcomponent to the appropriate sub object. This may be explicitly
*  specified on the component, or a parent of the component, or automatically generated
*  if a DataSource is specified on either the component or a parent thereof.
*  
*  To provide a simple example - if a complex object existed with the following format:
*  
*  
*  { companyName:"Some Company",
*   address:{  street:"123 Main Street", city:"New York", state:"NY" }
*  }
*  
*  
*  
*  a developer could specify a DynamicForm instance with 'dataPath' set to "address" to edit
*  the nested address object:
*  
*  
*  isc.ValuesManager.create({
*    ID:'vm',
*    values: { companyName:"Some Company",
*        address:{  street:"123 Main Street", city:"New York", state:"NY" }
*    }
*  });
* 
*  isc.DynamicForm.create({
*    valuesManager:"vm",
*    dataPath:"address",
*    items:[{name:"street"}, {name:"city"}, {name:"state"}]
*  });
*  
*  
*  
*  If a component is specified with a dataPath attribute but does not have an
*  explicitly specified valuesManager, it will check its parent element chain for a specified
*  valuesManager and automatically bind to that. This simplifies binding multiple components used
*  to view or edit a nested data structure as the valuesManager needs only be defined once at a
*  reasonably high level component. Here's an example of this approach:
*  
*  
*  isc.ValuesManager.create({
*    ID:'vm',
*    values: { companyName:"Some Company",
*        address:{  street:"123 Main Street", city:"New York", state:"NY" }
*    }
*  });
* 
*  isc.Layout.create({
*    valuesManager:vm,
*    members:[
*      isc.DynamicForm.create({
*        dataPath:"/",
*        items:[{name:"companyName"}]
*      }),
*      isc.DynamicForm.create({
*        dataPath:"address",
*        items:[{name:"street"}, {name:"city"}, {name:"state"}]
*      })
*    ]
*  });
*  
*  
*  
*  Note that in this case the valuesManager is specified on a Layout, which has no 'values'
*  management behavior of its own, but contains items with a specified dataPath which do. In this
*  example you'd see 2 forms allowing editing of the nested data structure.
*  
*  dataPaths from multiple nested components may also be combined. For example:
*  
*  
*  isc.ValuesManager.create({
*    ID:'vm',
*    values: { companyName:"Some Company",
*        address:{  street:"123 Main Street", city:"New York", state:"NY" }
*        parentCompany:{
*          companyName:"Some Corporation",
*          address:{  street:"1 High Street", city:"New York", state:"NY" }
*        }
*    }
*  });
* 
*  isc.Layout.create({
*    valuesManager:vm,
*    members:[
*      isc.DynamicForm.create({
*        dataPath:"/",
*        items:[{name:"companyName"}]
*      }),
*      isc.DynamicForm.create({
*        dataPath:"address",
*        items:[{name:"street"}, {name:"city"}, {name:"state"}]
*      }),
*      isc.Layout.create({
*        dataPath:"parentCompany",
*        members:[
*          isc.DynamicForm.create({
*            dataPath:"",
*            items:[{name:"companyName", type:"staticText"}]
*          }),
*          isc.DetailViewer.create({
*            dataPath:"address",
*            fields:[{name:"street", name:"city", name:"state"}]
*          })
*        ]
*      })
*    ]
*  });
*  
*  
*  
*  In this example the detailViewer will display data from the parentCompany.address
*  object within the base record.
*  
*  Note that if a component has a specified dataSource and shows child components with a
*  specified dataPath, there is no need to explicitly declare a valuesManager at all. If a component
*  with a dataPath has a dataSource, or an ancestor with a dataSource specified, it will, a
*  valuesManager will automatically be generated on the higher level component (and be available as
*  component.valuesManager).
*  
*  Difference between "" and "/" - relative and absolute datapaths
*  In the above example, note how the form for entering the "main" company name is given a 
*  dataPath of "/", while the form for entering the parent company name is given a dataPath of
*  "". The difference here is exactly the same as you would find in a filesystem path: a 
*  dataPath starting with "/" is absolute, so "/" by itself means "root". A dataPath that 
*  does not start with "/" is relative, and the empty string indicates that dataPaths for 
*  items below this one in the hierarchy should apply the dataPath so far from the hierarchy 
*  above. 
*  
*  If that isn't clear, consider the form for entering the parent company name in the above 
*  example. The correct dataPath to the field is /parentCompany/companyName. We
*  have the "parentCompany" part of that path provided by the containing Layout, so we cannot 
*  reiterate it on the form itself. However, if we omit the dataPath property altogether, the
*  framework will not seek to apply dataPath at all. So, we specify the empty string, which
*  tells SmartClient to use dataPath and to retain the portion of the path derived so far from 
*  the containment hierarchy.
*  
*  One further clarification: relative paths are only different from absolute paths if they are
*  relative to something other than the root. So in the above example, although we specify 
*  "/" as the dataPath of the "main" company name form, we would get exactly the same behavior
*  by specifying it as "", because the correct dataPath for the companyName field is 
*  "/companyName" - ie, it is relative to the root.
* @type {DataPath}
* @default null
*/
dataPath?: DataPath;

/**
* If true, when retrieving a AutoTest.getLocator for
*  this component always return a reference directly to this component by
*  Canvas.ID, using the compact format "///canvasID", 
*  ignoring any parent elements and ignoring any configured AutoTest.testRoot.
*  This format of locator will always be resolved back to the component by ID 
*  regardless of any changes in its position in the UI structure of an application.
*  
*  This setting is appropriate for components which are expected to always be present in an application
*  with a stable Canvas.ID.
* @type {boolean}
* @default false
*/
locateByIDOnly?: boolean;

/**
* Local name for referencing this canvas from an autoTest locator string. It will be
*  used instead of index if found. This name must by unique within the parent
*  component.
*  
*  By setting a static ID on certain top-level components and then using locatorName
*  in contained components, stable locators can be created for these components without
*  the need to pervasively assign IDs.
* @type {string}
* @default null
*/
locatorName?: string;

/**
* Strategy to use when locating children in this canvas from an autoTest locator string.
* @type {LocatorStrategy}
* @default null
*/
locateChildrenBy?: LocatorStrategy;

/**
* LocatorTypeStrategy to use when finding children within this canvas.
* @type {LocatorTypeStrategy}
* @default null
*/
locateChildrenType?: LocatorTypeStrategy;

/**
* Strategy to use when locating peers of this canvas from an autoTest locator string.
* @type {LocatorStrategy}
* @default null
*/
locatePeersBy?: LocatorStrategy;

/**
* LocatorTypeStrategy to use when finding peers of this canvas.
* @type {LocatorTypeStrategy}
* @default null
*/
locatePeersType?: LocatorTypeStrategy;

/**
* When nodes are added to an EditContext, should they be masked by setting
*  EditProxy.useEditMask true if not explicitly set?
* @deprecated * As of SmartClient version 10.0, deprecated in favor of EditProxy.autoMaskChildren

* @type {boolean}
* @default null
*/
autoMaskComponents?: boolean;

/**
* An EditProxy controls the behaviors of a component when it is placed into
*  devTools.
*  
*  The editProxy AutoChild is created when a component is first placed into
*  edit mode via Canvas.setEditMode.
*  
*  editProxy properties can be supplied on a PaletteNode or
*  EditNode as PaletteNode.editProxyProperties, but must
*  be provided before the component is first placed into edit mode.
*  
*  Most editable components use a custom EditProxy. See the documentation for
*  each class' Canvas.editProxyConstructor to determine
*  the class.
* @type {EditProxy}
* @default null
*/
editProxy?: EditProxy;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "CanvasEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* The component's EditNode for a component that has been created by a
*  Palette from a PaletteNode.
* @type {EditNode}
* @default null
*/
editNode?: EditNode;

/**
* ARIA role of this component. Usually does not need to be manually set - see
*  accessibility.
* @type {string}
* @default null
*/
ariaRole?: string;

/**
* Explicit set of ARIA state mappings for this component. Any values specified in this
*  object will be combined with Canvas.getAriaStateDefaults as described in Canvas.getAriaState.
*  
*  Usually this does not need to be manually set - see accessibility.
*  
*  Usage: This attribute should be set to a mapping of aria state-names to values - for example
*  to have the "aria-haspopup" property be present with a value "true", you'd specify:
*  
*  { haspopup : true }
*  
* @type {object}
* @default null
*/
ariaState?: object;

/**
* If this canvas is being displayed in a CanvasItem, this property will be set
*  to point at the item. Otherwise this property will be null.
* @type {CanvasItem}
* @default null
*/
canvasItem?: CanvasItem;

/**
* Default total duration of animations. Can be overridden by setting animation times for
*  specific animations, or by passing a duration parameter into the appropriate
*  animate...() method.
* @type {number}
* @default "300"
*/
animateTime?: number;

/**
* Default acceleration effect to apply to all animations on this Canvas.
*  Can be overridden by setting animationAcceleration for specific animations or by passing
*  an acceleration function directly into the appropriate method.
* @type {AnimationAcceleration}
* @default "smoothEnd"
*/
animateAcceleration?: AnimationAcceleration;

/**
* Default animation effect to use if Canvas.animateShow is called without an
*  explicit effect parameter
* @type {AnimateShowEffectId | AnimateShowEffect}
* @default "wipe"
*/
animateShowEffect?: AnimateShowEffectId | AnimateShowEffect;

/**
* Default animation effect to use if Canvas.animateHide is called without an
*  explicit effect parameter
* @type {AnimateShowEffectId | AnimateShowEffect}
* @default "wipe"
*/
animateHideEffect?: AnimateShowEffectId | AnimateShowEffect;

/**
* Default time for performing an animated move. If unset, this.animateTime
*  will be used by default instead
* @type {number}
* @default null
*/
animateMoveTime?: number;

/**
* Default time for performing an animated resize. If unset, this.animateTime
*  will be used by default instead
* @type {number}
* @default null
*/
animateResizeTime?: number;

/**
* Default time for performing an animated setRect. If unset, this.animateTime
*  will be used by default instead
* @type {number}
* @default null
*/
animateRectTime?: number;

/**
* Default time for performing an animated fade. If unset, this.animateTime
*  will be used by default instead
* @type {number}
* @default null
*/
animateFadeTime?: number;

/**
* Default time for performing an animated scroll. If unset, this.animateTime
*  will be used by default instead
* @type {number}
* @default null
*/
animateScrollTime?: number;

/**
* Default time for performing an animated show. If unset, this.animateTime
*  will be used by default instead
* @type {number}
* @default null
*/
animateShowTime?: number;

/**
* Default time for performing an animated hide. If unset, this.animateTime
*  will be used by default instead
* @type {number}
* @default null
*/
animateHideTime?: number;

/**
* Default acceleration effect for performing an animated move. If unset, 
*  this.animateAcceleration will be used by default instead
* @type {AnimationAcceleration}
* @default null
*/
animateMoveAcceleration?: AnimationAcceleration;

/**
* Default acceleration function for performing an animated resize. If unset, 
*  this.animateAcceleration will be used by default instead
* @type {AnimationAcceleration}
* @default null
*/
animateResizeAcceleration?: AnimationAcceleration;

/**
* Default acceleration function for performing an animated move and resize. If unset, 
*  this.animateAcceleration will be used by default instead
* @type {AnimationAcceleration}
* @default null
*/
animateRectAcceleration?: AnimationAcceleration;

/**
* Default acceleration function for performing an animated scroll. If unset, 
*  this.animateAcceleration will be used by default instead
* @type {AnimationAcceleration}
* @default null
*/
animateScrollAcceleration?: AnimationAcceleration;

/**
* Default acceleration function for performing an animated show. If unset, 
*  this.animateAcceleration will be used by default instead
* @type {AnimationAcceleration}
* @default null
*/
animateShowAcceleration?: AnimationAcceleration;

/**
* Default acceleration function for performing an animated hide. If unset, 
*  this.animateAcceleration will be used by default instead
* @type {AnimationAcceleration}
* @default null
*/
animateHideAcceleration?: AnimationAcceleration;

/**
* When to update the Canvas.layoutChildren for a Canvas.animateResize. Updating the child layout more often may improve appearance, but risks
*  prohibitive overhead with more complicated widget hierarchies.
* @type {AnimationLayoutMode}
* @default "atEnd"
*/
animateResizeLayoutMode?: AnimationLayoutMode;

/**
* When this Canvas is included as a member in a Layout, layoutAlign controls alignment on the
*  breadth axis of the layout. Default is "left" for a VLayout, "top" for an HLayout.
* @type {Alignment | VerticalAlignment}
* @default null
*/
layoutAlign?: Alignment | VerticalAlignment;

/**
* When this Canvas is included as a member in a Layout, whether a resizeBar should be shown
*  after this member in the layout, to allow it to be resized.
*  
*  Whether a resizeBar is actually shown also depends on the 
*  Layout.defaultResizeBars attribute of the layout, and whether this
*  Canvas is the last layout member.
*  
*  By default the resize bar acts on the Canvas that it is declared on. If you want the resize
*  bar to instead act on the next member of the Layout (e.g. to collapse down or to the right),
*  set Canvas.resizeBarTarget as well.
* @type {boolean}
* @default false
*/
showResizeBar?: boolean;

/**
* When this Canvas is included as a member in a Layout, and Canvas.showResizeBar is set to
*  true so that a resizeBar is created, resizeBarTarget:"next" can be
*  set to indicate that the resizeBar should resize the next member of the layout rather than
*  this one. For resizeBars that support hiding their target member when clicked on, 
*  resizeBarTarget:"next" also means that the next member will be the one hidden.
*  
*  This is typically used to create a 3-way split pane, where left and right-hand sections can
*  be resized or hidden to allow a center section to expand.
*  
*  NOTE: as with any Layout, to ensure all available space is used, one or more members
*  must maintain a flexible size (eg 75%, or *). In a two pane Layout with a normal resize
*  bar, to fill all space after a user resizes, the member on the right should have
*  flexible size. With resizeBarTarget:"next", the member on the left should have
*  flexible size.
* @type {string}
* @default null
*/
resizeBarTarget?: string;

/**
* When this Canvas is included as a member in a Layout, extra blank space that should be left
*  after this member in a Layout.
* @type {PositiveInteger}
* @default "0"
*/
extraSpace?: PositiveInteger;

/**
* Whether this canvas should always start a new line when used as a tile in a 
*  FlowLayout. This property is not supported in a TileLayout with
*  TileLayout.layoutPolicy: "fit" or if databound (i.e. for a TileGrid).
* @type {boolean}
* @default false
*/
startLine?: boolean;

/**
* Whether this canvas should end the line it's in when used as a tile in a FlowLayout.
*  This property is not supported in a TileLayout with TileLayout.layoutPolicy:
*  "fit" or if databound (i.e.TileGrid).
* @type {boolean}
* @default false
*/
endLine?: boolean;



/**
*  Returns a summary of this canvas and its child canvii and/or fields that is useful to an AI
*  in understanding the hierarchy.
* 
* 
* @param {Array<Partial<string>>} property names to exclude from entire hierarchy
* @param {Array<Partial<string>>} property names to exclude from this canvas
*/
getUISummary(heirarchyExcluded:Array<Partial<string>>, thisCanvasExcluded:Array<Partial<string>>): object;

/**
*  Get the CSS class to apply when printing this widget. Returns the
*  Canvas.printStyleName, if specified, falling back to the 
*  Canvas.styleName otherwise.
* 
*/
getPrintStyleName(): CSSStyleName;

/**
*  See Canvas.canAdaptWidth.
* 
* @param {Integer} surplus (if positive) or overflow (if negative)
* @param {Integer} width of member currently assumed by parent layout
*/
adaptWidthBy(pixelDifference:Integer, unadaptedWidth:Integer): Integer;

/**
*  See Canvas.canAdaptWidth.
* 
* @param {Integer} surplus (if positive) or overflow (if negative)
* @param {Integer} height of member currently assumed by parent layout
*/
adaptHeightBy(pixelDifference:Integer, unadaptedHeight:Integer): Integer;

/**
* 
*  This method performs some basic initialization common to all UI components. To do custom UI
*  component initialization, you should generally override Canvas.initWidget. This
*  method does the following, in order:
*  
*  
* - Sets up a global reference to this instance as described in Canvas.ID.
*  
* - Ensures certain numeric properties have numeric values (e.g. width, height, padding,
*  margin)
*  
* - Ensures Canvas.children and Canvas.peers are Arrays.
*  
* - Calls Canvas.initWidget
*  
* - Creates Canvas.showEdges and Canvas.showShadow, if so configured.
*  
* - Calls Canvas.draw if Canvas.autoDraw is set on instance or globally.
*  
*  Unless you're in an advanced scenario where you need to inject code before the above
*  logic executes, place your initialization logic in initWidget() rather than init(). If you
*  do decided to override this method, you must call the superclass implementation like so:
*  
*   this.Super("init", arguments);
*  
* 
* 
* @param {any=} All arguments initially passed to Class.create
*/
init(...args:any[]): void;

/**
* 
*  For custom components, perform any initialization specific to your widget subclass.
*  
*  When creating a subclass of any Canvas-based component, you should generally override
*  this method rather than overriding Class.init. This is because Canvas has its own
*  Class.init override which performs some generally desirable initialization - see
*  Canvas.init for details.
*  
*  This method is called by Canvas.init when a component is create()d. When
*  overriding this method, You must call the superClass initWidget implementation, like
*  so:
*  
*   this.Super("initWidget", arguments);
*  
*  
*  In general, if you are going to call functionality supported by your superclass (eg calling
*  addTab() when your superclass is a TabSet), call Super() first. However, you can generally
*  assign properties to this before calling Super() as a way of mimicking the
*  effect of the property being passed to Class.create on normal instance
*  construction. For example, when subclassing a DynamicForm, you could set this.items to a
*  generated set of items before calling Super().
*  
*  NOTE: child creation: if you are creating a component that auto-creates certain children (eg
*  a Window which creates a Header child), typical practice is to create those children
*  immediately before drawing by overriding draw(). This postpones work until it is really
*  necessary and avoids having to update children if settings are changed between creation and
*  draw(). Alternatively, if you prefer callers to directly manipulate auto-created children,
*  it's best to create them earlier in initWidget(), in order to allow manipulation before
*  draw.
* 
* 
* @param {any=} All arguments initially passed to Class.create
*/
initWidget(...args:any[]): void;

/**
*  Retrieve a child of this Canvas by it's Canvas.getLocalId.
* 
* 
* @param {string} local ID of the Canvas
*/
getByLocalId(localId:string): Canvas;

/**
*  Retrieve the local ID of this canvas. If no local ID is assigned the normal
*  canvas ID is returned making this method a safe replacement for Canvas.getID.
*  
*  A "local ID" is name for a child widget which is unique only for this parent, and not globally
*  unique as is required for Canvas.ID. Widgets receive local IDs when loaded via
*  RPCManager.loadScreen or RPCManager.cacheScreens and
*  RPCManager.createScreen.
* 
* 
*/
getLocalId(): string;

/**
*    Returns the boolean true, if the widget has been completely drawn, and false otherwise.
* 
*/
isDrawn(): boolean;

/**
*    When a widget instance is created, it is assigned a unique global identifier that can be
*    used to access the instance by name. The getID method returns this ID for a particular
*    instance. Global IDs are essential when you need to embed a widget reference in a string,
*    usually a string that will be evaluated in the future and/or in another object, where you
*    may not have access to a variable or parameter holding the widget's reference.
* 
* 
*/
getID(): GlobalId;

/**
*  Return the inner HTML for this canvas. Called when the canvas is drawn or redrawn;
*  override to customize.
* 
* 
*/
getInnerHTML(): HTMLString;

/**
*  Setter for Canvas.snapOffsetLeft.
* 
* @param {Integer} new snapOffsetLeft value.
*/
setSnapOffsetLeft(snapOffsetLeft:Integer): void;

/**
*  Setter for Canvas.snapOffsetTop.
* 
* @param {Integer} new snapOffsetTop value.
*/
setSnapOffsetTop(snapOffsetTop:Integer): void;

/**
*    Draws the widget on the page.
* 
*/
draw(): Canvas;

/**
*  Retrieves printable HTML for this component and all printable subcomponents.
*  
*  By default any Canvas with children will simply collect the printable HTML of its children
*  by calling getPrintHTML() on each child that is considered Canvas.shouldPrint.
*  If a callback is provided, then null is always returned and the callback is fired asynchronously.
*  
*  If overriding this method for a custom component, you should either return a String
*  of printable HTML directly or return null and fire the provided callback using
*  Class.fireCallback.
*  
*  To return an empty print representation, return an empty string ("") rather than null.
*  
*  The printProperties argument, if passed, must be passed to any subcomponents on
*  which getPrintHTML() is called.
*  
*  Default implementation will set Canvas.isPrinting flag to true to indicate
*  printing is in progress, and clear this flag when the printing has completed (possibly via
*  an asynchronous callback).
*  
*  NOTE: Expecting a direct return value from the default implementation is deprecated usage.
*  This is because small changes to an application (such as adding a few more data points to a chart
*  or adding another button) or using certain browsers can make it necessary to generate the HTML
*  asynchronously. Thus, application code should not rely on the return value and always pass
*  a callback.
* 
* 
* @param {PrintProperties=} properties to configure printing behavior - may be null.
* @param {Callback=} optional callback. This is required to handle cases where HTML         generation is asynchronous - if a method generates HTML asynchronously, it should return          null, and fire the specified callback on completion of HTML generation. The          first parameter HTML should contain the generated print HTML.          The callback is only called if null is returned. Furthermore, the default          getPrintHTML() implementation always returns null and fires the callback          when a callback is provided.
*/
getPrintHTML(printProperties?:PrintProperties, callback?:Callback): HTMLString;

/**
*  Setter for the Canvas.htmlElement.
* 
* @param {DOMElement | string} New htmlElement for this canvas or Null    to clear the existing htmlElement. You may also provide a string with the elements     ID, which will have the effect of setting Canvas.htmlElement to the node     returned by document.getElementById(element). Note that you may need to set     position:relative explicitly following a call to this function.
*/
setHtmlElement(element:DOMElement | string): void;

/**
*  Setter for the Canvas.htmlPosition.
* 
* @param {DrawPosition} New htmlPosition for this canvas
*/
setHtmlPosition(element:DrawPosition): void;

/**
*  Returns whether a canvas is waiting to be redrawn. Will return true if
*  Canvas.markForRedraw has been called, but this canvas has not yet been
*  redrawn.
* 
*/
isDirty(): boolean;

/**
*  Marks the widget as "dirty" so that it will be added to a queue for redraw. Redraw of dirty
*  components is handled by a looping timer and will after a very short delay (typically less than
*  100ms). In most cases it is recommended that developers use markForRedraw()
*  instead of calling Canvas.redraw directly. Since this method queues the redraw, multiple
*  calls to markForRedraw() within a single thread of execution will only lead to a single DOM
*  manipulation which greatly improves application performance.
* 
* 
* @param {string=} ["no reason provided"] reason for performing the redraw
*/
markForRedraw(reason?:string): void;

/**
*  Redraws the widget immediately with its current property values.
* 
*  Generally, if you want a Canvas to redraw, call markForRedraw() - this will cause the Canvas to
*  be redrawn when current processing ends, so that a series of modifications made to a Canvas will
*  cause it to redraw only once.
* 
*  Only call redraw() directly if you need immediate responsiveness, for example you are redrawing
*  in response to continuous mouse motion.
* 
* 
* @param {string=} ["no reason provided"] reason for performing the redraw
*/
redraw(reason?:string): void;

/**
*  Remove all visual representation of a Canvas, including all child or member Canvases, or
*  managed top-level components such as the ListGrid drop location indicator.
*  
*  This is more expensive than hide(), because in order to become visible again, the Canvas must
*  be draw()n again. Generally, application code has no reason to call clear() unless it is
*  attempting to do advanced memory management. If you want to temporarily hide a Canvas, use
*  hide() and show(), and if you want to permanently destroy a Canvas, use Canvas.destroy.
*  
*  You would only use clear() if you were managing a very large pool of components and you
*  wanted to reclaim some of the memory used by components that had not been used in a while,
*  while still being able to just draw() them to make them active and visible again.
*  
*  Note: a clear() will happen as part of moving a Canvas to a different parent. See
*  Canvas.addChild.
* 
* 
*/
clear(): void;

/**
*  Permanently destroy a Canvas and all of it's children / members, recursively.
*  
*  Like Canvas.clear, calling destroy() removes all HTML for the component;
*  unlike clear(), a destroyed Canvas is permanently unusable: it cannot be draw()'n again,
*  cannot be referenced by its global ID, and is eligible for garbage collection (assuming
*  that application code is not holding a reference to the Canvas).
*  
*  Any attempt to call a method on a destroyed Canvas will generally result in an error. If your
*  application is forced to hold onto Canvas's that might be destroy()d without warning, you can
*  avoid errors by checking for the Canvas.destroyed property. If you override certain Canvas
*  methods, your code may be called while a Canvas is being destroy()d; in this case you can avoid
*  extra work (and possibly errors) by checking for the Canvas.destroying property.
*  
*  Note that destroy() should not be called directly in event handling code for this
*  canvas. For this reason, wherever possible we recommend using Canvas.markForDestroy
*  instead of calling this method directly.
*  
*  Also note that developers should not destroy() or markForDestroy() a component
*  while it is in the middle of an asynchronous operation. For example, if you need to submit
*  and then destroy a single-use dynamic form, the call to markForDestroy() should
*  be invoked from the callback to DynamicForm.saveData, rather than being invoked
*  synchronously after the the call to saveData().
* 
* 
*/
destroy(): void;

/**
*  Canvas.destroy this canvas on a timeout. This method should be used instead of
*  calling canvas.destroy() directly unless there's a reason a the canvas needs to be
*  destroyed synchronously. By using a timeout, this method ensures the destroy() will
*  occur after the current thread of execution completes. This allows you to easily mark canvases
*  for destruction while they're handling events, which must complete before the canvas can be
*  destroyed.
*  
*  Notes:
*  
*  
* - markForDestroy() performs some immediate cleanup and puts a component into a
*  "pending destroy" state. As far as application code is concerned, once a component has been
*  is in this state it should be considered invalid to invoke methods on the component.
*  
* - Developers should not destroy() or markForDestroy() a component
*  while it is in the middle of an asynchronous operation. For example, if you need to submit
*  and then destroy a single-use dynamic form, the call to markForDestroy() should
*  be invoked from the callback to DynamicForm.saveData, rather than being invoked
*  synchronously after the call to saveData()
*  
* 
* 
*/
markForDestroy(): void;

/**
*  Returns the DOM ID for the main element for this canvas which will have 
*  ARIA Canvas.ariaRole
*  and Canvas.ariaState applied to it. This can be useful when applying
*  Canvas.ariaState which need to refer to the DOM handle
*  of another canvas, such as 
*  aria-controls.
*  
*  Note that we do not recommend using this ID to identify the element in the DOM for integration
*  with automated testing tools. The AutoTest.getLocator and AutoTest.getElement
*  APIs should be used instead. These locators are much finer grained 
*  - they allow developers to identify sub-elements within the widget handle 
*  that map to specific logical SmartClient objects (for example FormItem Icons
*  within a DynamicForm). They are also more stable - AutoTest locators are able to reliably
*  identify components based on their position in the application hierarchy and other context, 
*  while the DOM element IDs are not guaranteed not to change across page reloads.
* 
* 
*/
getAriaHandleID(): string;

/**
*  Returns the outer DOM element of this Canvas. This method is provided for the 
*  very rare cases where a programmer needs to examine the DOM hierarchy created by
*  a drawn SmartClient component.
*  
*  Direct manipulation of the DOM elements created by SmartClient components
*  is not supported. SmartClient components should be rendered or cleared using
*  standard methods such as Canvas.draw, Canvas.clear. If direct
*  integration with existing DOM structures is required, this should be achieved via the
*  Canvas.htmlElement attribute, rather than by attempting to move the
*  component's outer element via native browser APIs.
*  The content of SmartClient components' DOM elements should also not be directly 
*  manipulated using native browser APIs - standard methods such as 
*  Canvas.setContents, Canvas.addChild, Canvas.removeChild,
*  Canvas.markForRedraw and Canvas.redraw should be used instead.
*  
*  In some cases, the element returned may match the element returned by 
*  Canvas.getContentElement, but this will not always be the case.
*  
*  If the widget is undrawn, this method will return null.
* 
* 
*/
getOuterElement(): DOMElement;

/**
*  Returns the DOM element for this Canvas which contains the Canvas.contents, or
*  for Canvas.getParentCanvas, the DOM elements for any drawn
*  children. This method is provided for the 
*  very rare cases where a programmer needs to examine the DOM hierarchy created by
*  a drawn SmartClient component.
*  
*  Direct manipulation of the DOM elements created by SmartClient components
*  is not supported. SmartClient components should be rendered or cleared using
*  standard methods such as Canvas.draw, Canvas.clear. If direct
*  integration with existing DOM structures is required, this should be achieved via the
*  Canvas.htmlElement attribute, rather than by attempting to move the
*  component's outer element via native browser APIs.
*  The content of SmartClient components' DOM elements should also not be directly 
*  manipulated using native browser APIs - standard methods such as 
*  Canvas.setContents, Canvas.addChild, Canvas.removeChild,
*  Canvas.markForRedraw and Canvas.redraw should be used instead.
*  
*  In some cases, the element returned may match the element returned by
*  Canvas.getOuterElement, but this will not always be the case.
*  
*  If the widget is undrawn, this method will return null.
* 
* 
*/
getContentElement(): DOMElement;

/**
*  Adds newChild as a child of this widget, set up a named object reference (i.e., this[name])
*  to the new widget if name argument is provided, and draw the child if this widget has been
*  drawn already.
*  
*  If newChild has a parent it will be removed from it. If it has a master, it will be detached
*  from it if the master is a child of a different parent. If newChild has peers, they'll be
*  added to this widget as children as well.
* 
* 
* @param {Canvas} new child canvas to add
* @param {string=} name to assign to child (eg: this[name] == child)
* @param {boolean=} if false, child will not automatically be drawn (only                     for advanced use)
*/
addChild(newChild:Canvas, name?:string, autoDraw?:boolean): Canvas;

/**
*  Remove a peer from this Canvas
* 
* @param {Canvas} Peer to be removed from this canvas
* @param {string=} If this peer was assigned a name when added via addPeer(), it should            be passed in here to ensure no reference is kept to the peer
*/
removePeer(peer:Canvas, name?:string): void;

/**
*  Make this Canvas no longer a peer of its master
* 
*/
depeer(): void;

/**
*  Remove this canvas from its parent if it has one.
* 
*/
deparent(): void;

/**
*  Remove a child from this parent.
* 
* @param {Canvas} Child canvas to remove from this parent.
* @param {string=} If the child canvas was assigned a name when added via addChild(), it            should be passed in here to ensure no reference is kept to the child
*/
removeChild(child:Canvas, name?:string): void;

/**
*  Adds newPeer as a peer of this widget (also making it a child of this widget's parent, if
*  any), set up a named object reference (i.e., this[name]) to the new widget if name is
*  provided, and draw the peer if this widget has been drawn already.
*  
*  The widget to be added as a peer will be removed from its old master and/or parent, if any,
*  and it will be added as a child to the parent of this canvas (if any)
* 
* 
* @param {Canvas} new peer widget to add
* @param {string=} name to assign to peer (eg: this[peer] == child)
* @param {boolean=} if false, peer will not automatically be drawn (only               for advanced use)
* @param {boolean=} if true, when draw is called on the master widget, the peer              will be drawn before the master
*/
addPeer(newPeer:Canvas, name?:string, autoDraw?:boolean, preDraw?:boolean): Canvas;

/**
*  Set the snapTo property of this canvas, and handle repositioning.
* 
* 
* @param {string} new snapTo value
*/
setSnapTo(snapTo:string): void;

/**
*  Return the snapTo value of this object
* 
* 
*/
getSnapTo(): string;

/**
*  Set the snapEdge property of this canvas, and handle repositioning.
* 
* 
* @param {string} new snapEdge value
*/
setSnapEdge(snapEdge:string): void;

/**
*  Return the snapEdge value of this object
* 
* 
*/
getSnapEdge(): string;

/**
*  Returns the parent of this canvas, if any.
*  
*  See containment for an overview of parent/child relationships.
* 
* 
*/
getParentCanvas(): Canvas;

/**
*  Returns this canvas's "master" (the canvas to which it was added as a peer), if any.
*  
*  See containment for an overview of master/peer relationships.
* 
*/
getMasterCanvas(): Canvas;

/**
*  Returns an array of object references to all ancestors of this widget in the containment
*  hierarchy, starting with the direct parent and ending with the top element.
* 
*/
getParentElements(): Array<Partial<Canvas>>;

/**
*    Returns true if element is a descendant of this widget (i.e., exists below this widget in
*    the containment hierarchy); and false otherwise.
* 
* @param {Canvas} the canvas to be tested
* @param {boolean=} If passed this method will return true if the canvas                parameter is a pointer to this widget.
*/
contains(canvas:Canvas, testSelf?:boolean): boolean;

/**
*  Returns true if the current EventHandler.getTarget is this component
*  or a descendent of this component.
*  
*  Note that, unlike Canvas.containsEvent, this method is not based on reported event coordinates
*  and there are cases where containsEvent() and containsEventTarget() will
*  return different values, including when the mouse is within the bounds of a target component, but
*  another canvas is rendered in front of it in the page's z-order.
*  
* 
*/
containsEventTarget(): boolean;

/**
*  Show a clickMask over the entire screen that intercepts mouse clicks and fires some action.
*  The mask created will be associated with this canvas - calling this method multiple times
*  will not show multiple (stacked) clickMasks if the mask associated with this canvas is
*  already up.
*  
*  The clickMask useful for modal dialogs, menus and similar uses, where any click outside of
*  some Canvas should either be suppressed (as in a modal dialog) or just cause something (like
*  dismissing a menu).
* 
* 
* @param {Callback} action to fire when the user clicks on the mask
* @param {ClickMaskMode} whether to automatically hide the clickMask on mouseDown                   and suppress the mouseDown event from reaching                    the target under the mouse
* @param {any | Array<Partial<any>>} initially unmasked targets for this clickMask. Note that if this is a "hard"  mask, unmasked children of masked parents are not supported so any non-top-level widgets  passed in will have their parents unmasked. Children of unmasked parents can never be  masked, so you need only include the top widget of a hierarchy.
*/
showClickMask(clickAction:Callback, mode:ClickMaskMode, unmaskedTargets:any | Array<Partial<any>>): string;

/**
*  Hides the click mask associated with this canvas.
* 
* @param {string=} optional ID of specific clickMask to hide. If not passed,           defaults to hiding the click mask associated with this widget only.
*/
hideClickMask(ID?:string): void;

/**
*  Determines whether a clickmask is showing
* 
* @param {string=} optional ID of specific clickMask to check. If not passed,           checks for the click mask associated with this widget only.
*/
clickMaskUp(ID?:string): boolean;

/**
*  Temporariy block all user interaction with children of this widget, with the exception of those
*  passed in in the unmaskedChildren parameter. Children will remain blocked until
*  Canvas.hideComponentMask is called.
*  
*  This method will show the Canvas.componentMask canvas to block mouse interaction with
*  children, and temporarily remove masked children from the page's tab-order.
*  
*  This behavior differs from the standard Canvas.showClickMask in that the
*  modal mask shown by Canvas.showClickMask will cover the entire screen and typically only 
*  allow "unmasking" of top level components.
*  
*  Use Canvas.hideComponentMask to hide the component level mask.
* 
* 
* @param {Array<Partial<Canvas>>=} Children passed into this parameter will continue to    be interactive while other children are blocked. They will be moved above the componentMask in    the page's z-order and remain accessible via keyboard navigation. Note that this array should    contain direct children of this widget only.
*/
showComponentMask(unmaskedChildren?:Array<Partial<Canvas>>): void;

/**
*  Hide the Canvas.showComponentMask for this widget
* 
* @param {boolean=} By default when the component-level mask is hidden   it will attempt to reset focus to whatever had focus before the mask was shown.    Pass this parameter to suppress this behavior.
*/
hideComponentMask(suppressFocusReset?:boolean): void;

/**
*  Set all four coordinates, relative to the enclosing context, at once.
*  
*  Moves the widget so that its top-left corner is at the specified top-left coordinates,
*  then resizes it to the specified width and height.
* 
* 
* @param {number | Array<any> | object=} new left coordinate, Array of                         coordinates in parameter order, or Object                          with left, top, width, height properties.                          If an Array or Object is passed, the                          remaining parameters are ignored.
* @param {number=} new top coordinate
* @param {number=} new width
* @param {number=} new height
*/
setRect(left?:number | Array<any> | object, top?:number, width?:number, height?:number): boolean;

/**
*      Return the left coordinate of this object, relative to its enclosing context, in pixels.
* 
*/
getLeft(): number;

/**
*      Set the left coordinate of this object, relative to its enclosing context, in pixels.
*      NOTE: if you're setting multiple coordinates, use setRect(), moveTo() or resizeTo()
*      instead
* 
* @param {number | string} new left coordinate
*/
setLeft(left:number | string): void;

/**
*  Return the top coordinate of this object, relative to its enclosing context, in pixels.
* 
*/
getTop(): number;

/**
*  Set the top coordinate of this object, relative to its enclosing context, in pixels.
*  
*  NOTE: if you're setting multiple coordinates, use setRect() or moveTo() instead
* 
* 
* @param {number | string} new top coordinate
*/
setTop(top:number | string): void;

/**
*  Setter for Canvas.leavePageSpace.
* 
* 
* @param {Integer} new value for leavePageSpace.
*/
setLeavePageSpace(newPageSpace:Integer): void;

/**
*  Return the width of this object, in pixels.
*  
*  See gettingCanvasSize
* 
*/
getWidth(): number;

/**
*  Resizes the widget horizontally to the specified width (moves the right side of the
*  widget). The width parameter can be expressed as a percentage of viewport size or as
*  the number of pixels.
*  
*  NOTE: if you're setting multiple coordinates, use resizeTo() or setRect() instead
* 
* 
* @param {number | string} new width
*/
setWidth(width:number | string): void;

/**
*  Return the height of this object, in pixels.
*  
*  See gettingCanvasSize
* 
*/
getHeight(): number;

/**
*  Resizes the widget vertically to the specified height (moves the bottom side of the
*  widget). The height parameter can be expressed as a percentage of viewport size or as
*  the number of pixels.
*  
*  NOTE: if you're setting multiple coordinates, use resizeTo() or setRect() instead
* 
* 
* @param {number | string} new height
*/
setHeight(height:number | string): void;

/**
*  Resizes the widget horizontally if required to satisfy the specified Canvas.minWidth.
* 
* 
* @param {number} new minimum width
*/
setMinWidth(width:number): void;

/**
*  Resizes the widget vertically if required to satisfy the specified Canvas.minHeight.
* 
* 
* @param {number} new minimum height
*/
setMinHeight(height:number): void;

/**
*  Return the right coordinate of this object as rendered, relative to its enclosing context,
*  in pixels.
* 
* 
*/
getRight(): number;

/**
*  Resizes the widget horizontally to position its right side at the specified coordinate.
*  
*  NOTE: if you're setting multiple coordinates, use setRect(), moveTo() or resizeTo()
*  instead
* 
* 
* @param {number} new right coordinate
*/
setRight(right:number): void;

/**
*  Return the bottom coordinate of this object as rendered, relative to its enclosing context,
*  in pixels.
* 
* 
*/
getBottom(): number;

/**
*  Resizes the widget vertically to position its bottom edge at the specified coordinate.
*  
*  NOTE: if you're setting multiple coordinates, use setRect(), moveTo() or resizeTo()
*  instead
* 
* 
* @param {number} new bottom coordinate
*/
setBottom(bottom:number): void;

/**
*  Returns the scrollable width of the widget's contents, including children, ignoring
*  clipping.
*  
*  See gettingCanvasSize
* 
*/
getScrollWidth(): number;

/**
*  Returns the scrollable height of the widget's contents, including children, ignoring
*  clipping.
*  
*  See gettingCanvasSize
* 
* 
*/
getScrollHeight(): number;

/**
*  Get the number of pixels this Canvas is scrolled from its left edge.
* 
*/
getScrollLeft(): number;

/**
*  Get the number of pixels this Canvas is scrolled from its top edge.
* 
*/
getScrollTop(): number;

/**
*  Set the page-relative left coordinate of this widget.
* 
* 
* @param {number} new left coordinate in pixels
*/
setPageLeft(left:number): void;

/**
*  Set the page-relative top coordinate of this widget.
* 
* 
* @param {number} new top coordinate in pixels
*/
setPageTop(top:number): void;

/**
*  Returns the page-relative left coordinate of the widget on the page, in pixels.
* 
*/
getPageLeft(): number;

/**
*  Returns the page-relative top coordinate of the widget on the page, in pixels
* 
*/
getPageTop(): number;

/**
*  Return the page-relative right coordinate of this object, in pixels.
* 
* 
*/
getPageRight(): number;

/**
*  Return the page-relative bottom coordinate of this object, in pixels.
* 
*/
getPageBottom(): number;

/**
*  Set the CSS Margin, in pixels, for this component. Margin provides blank space outside of
*  the border.
*  
*  This property sets the same thickness of margin on every side. Differing per-side
*  margins can be set in a CSS style and applied via Canvas.styleName.
*  
*  Note that the specified size of the widget will be the size including the margin
*  thickness on each side.
* 
* 
* @param {number} new margin in pixels
*/
setMargin(margin:number): void;

/**
*  Set the CSS padding of this component, in pixels. Padding provides space between the border
*  and the component's contents.
*  
*  This property sets the same thickness of padding on every side. Differing per-side
*  padding can be set in a CSS style and applied via Canvas.styleName.
*  
* 
* @param {number} new padding in pixels
*/
setPadding(newPadding:number): void;

/**
*  Return whether or not this object contains the specified global (x,y) coordinates.
*  
*  Will return false if any parent canvas does not contain the specified point, (EG:
*  you're hovering over an element's absolute location, but it is scrolled out of
*  view in a parent element)
* 
* 
* @param {Integer} GLOBAL x-coordinate
* @param {Integer} GLOBAL y-coordinate
* @param {boolean=} point lies specifically within our viewport                       (drawn area excluding margins and scrollbars if                        present)
*/
containsPoint(x:Integer, y:Integer, withinViewport?:boolean): boolean;

/**
*  Does this widget contain the specified global (x,y) coordinates, and have no other widgets
*  also at the specified position, obscuring this one? This is commonly used for (for example)
*  drag and drop interactions.
* 
* 
* @param {number} GLOBAL x-coordinate
* @param {number} GLOBAL y-coordinate
* @param {boolean=} point lies within our viewport rather than                       just our drawn area
* @param {Canvas=} If passed ignore widget(s), do not check whether                     those widgets occludes this one.
* @param {Canvas=} If passed, only check for siblings occluding the       component up as far as the specified parent widget.
*/
visibleAtPoint(x:number, y:number, withinViewport?:boolean, ignoreWidgets?:Canvas, upToParent?:Canvas): boolean;

/**
*      Returns true if the rectangles of this widget and the specified widget overlap.
* 
* @param {Canvas} other canvas to test for intersection
*/
intersects(other:Canvas): boolean;

/**
*      Returns true if the rectangle of this widget intersects with the rectangle coordinates
*      passed in, and false otherwise.
* 
* @param {number | Array<any>} left coord of rect (or rect array)
* @param {number} top coord of rect
* @param {number} width of rect
* @param {number} height of rect
*/
intersectsRect(left:number | Array<any>, top:number, width:number, height:number): boolean;

/**
*  Returns true if the rectangle of this widget encloses the rectangle of the specified widget.
* 
* @param {Canvas} other canvas to test for enclosure
*/
encloses(other:Canvas): boolean;

/**
*  Returns true if the rectangle of this widget encloses the rectangle coordinates
*  passed in, and false otherwise.
* 
* @param {number | Array<Partial<number>>} left coord of rect (or rect array)
* @param {number} top coord of rect
* @param {number} width of rect
* @param {number} height of rect
*/
enclosesRect(left:number | Array<Partial<number>>, top:number, width:number, height:number): boolean;

/**
*  Return true if the last event's mouse coordinates are within the bounds of this component.
*  
*  NOTE: Z-ordering is not considered for the purposes of this test. If the coordinate you're
*  testing is occluded by other component, but the X,Y coordinates are still within the bounds
*  that component, this method will return true.
*  
*  See the related Canvas.containsEventTarget method for checking whether a canvas 
*  contains the target canvas for the current mouse event.
* 
* 
*/
containsEvent(): boolean;

/**
*  Return the X-coordinate of the last event relative to the left edge of the content of this
*  Canvas.
* 
*  NOTE: To get a coordinate relative to the viewport of this Canvas, subtract
*  this.getScrollLeft()
* 
* 
*/
getOffsetX(): number;

/**
*  Return the Y-coordinate of the last event, relative to the top edge of the content of this
*  Canvas.
* 
*  NOTE: To get a coordinate relative to the viewport of this Canvas, subtract
*  this.getScrollTop()
* 
* 
*/
getOffsetY(): number;

/**
*  Returns the thickness of this widget's scrollbars.
*  For canvases showing custom scrollbars this is determined from this.scrollbarSize
* 
* 
*/
getScrollbarSize(): number;

/**
*  Returns the width of the viewport onto the scrollable content.
*  
*  See gettingCanvasSize
* 
* 
*/
getViewportWidth(): number;

/**
*  Returns the height of the viewport onto the scrollable content.
*  
*  See gettingCanvasSize
* 
* 
*/
getViewportHeight(): number;

/**
*  Returns the amount of space available for (an) absolutely positioned child widget(s) or
*  absolutely positioned HTML content, without introducing clipping, scrolling or overflow.
*  
*  This is the space within the viewport of the widget (including padding, but excluding
*  margins, borders or scrollbars) rendered at its specified size.
*  
*  See gettingCanvasSize
* 
* 
*/
getInnerHeight(): number;

/**
*  Returns the amount of space available for absolutely positioned child widget(s) or
*  absolutely positioned HTML content, without introducing clipping, scrolling or overflow.
*  
*  This is the space within the viewport of the widget (including padding, but excluding
*  margins, borders or scrollbars) rendered at its specified size.
*  
*  See gettingCanvasSize
* 
* 
*/
getInnerWidth(): number;

/**
*  Returns the amount of space available for interior content (or relatively positioned child
*  widget(s)) without introducing clipping, scrolling or overflow.
*  This is the space within the viewport of the widget (not including padding, and excluding
*  margins, borders or scrollbars) rendered at its specified size.
*  
*  See gettingCanvasSize
* 
* 
*/
getInnerContentHeight(): number;

/**
*  Returns the amount of space available for interior content (or relatively positioned child
*  widget(s)) without introducing clipping, scrolling or overflow.
*  This is the space within the viewport of the widget (not including padding, and excluding
*  margins, borders or scrollbars) rendered at its specified size.
*  
*  See gettingCanvasSize
* 
* 
*/
getInnerContentWidth(): number;

/**
*  Return the visible width of the Canvas.
*  
*  See gettingCanvasSize
* 
* 
*/
getVisibleWidth(): number;

/**
*  Return the visible height of the Canvas.
*  
*  See gettingCanvasSize
* 
* 
*/
getVisibleHeight(): number;

/**
*      Moves the widget deltaX pixels to the right and deltaY pixels down. Pass negative
*      numbers to move up and/or to the left.
* 
* @param {Integer} amount to move horizontally (may be negative)
* @param {Integer} amount to move vertically (may be negative)
*/
moveBy(deltaX:Integer, deltaY:Integer): boolean;

/**
*  Moves the widget so that its top-left corner is at the specified coordinates.
*  
*  This method will also accept a single parameter as an object array with left and top given
*  as properties.
* 
* 
* @param {number | object=} x-coordinate to move to in LOCAL coordinates                       or Object with left and top properties.
* @param {number=} y-coordinate to move to in LOCAL coordinates
*/
moveTo(left?:number | object, top?:number): boolean;

/**
*  Show this widget next to another widget on the page, positioned such that it will 
*  not extend beyond the browser viewport.
*  
*  Note that this method simply sets the coordinates of the widget and displays it (using
*  a Canvas.animateShow by default). It will
*  not change the Canvas.parentElement of either component.
*  
*  An example use case might be showing a menu next to a menu-button.
* 
* 
* @param {Canvas} Canvas to show next to
* @param {string=} Which side of the other canvas should we put. Options are                 "top", "bottom", "left", "right". (Defaults to "right")
* @param {boolean=} This argument controls whether this canvas can be positioned on top of the other  widget if there isn't room to put it next to the other widget extending out of the  browser viewport  If 'canOcclude' is true, simply shift this widget over the other widget, so that  it ends up onscreen. If 'canOcclude' is false, avoid extending offscreen  by positioning this widget on the other side of the other widget.
* @param {boolean=} If false do not use an animation to show the component.
*/
showNextTo(otherWidget:Canvas, side?:string, canOcclude?:boolean, skipAnimation?:boolean): void;

/**
*  Move this canvas to the specified point, or as close to the specified point as possible
*  without this widget extending beyond the edge of the browser viewport on any side.
* 
* @param {number=} Left coordinate (defaults to mouse position)
* @param {number=} Top coordinate (defaults to mouse position)
*/
placeNear(left?:number, top?:number): void;

/**
*      Resizes the widget, adding deltaX to its width and deltaY to its height (moves the right
*      and/or bottom sides of the widget).
* 
* @param {number=} amount to resize horizontally (may be negative)
* @param {number=} amount to resize vertically (may be negative)
*/
resizeBy(deltaX?:number, deltaY?:number): boolean;

/**
*  Observable method called 
*  whenever a Canvas changes size. Note that if this canvas is
*  Canvas.overflow, and is waiting for a queued redraw (see
*  Canvas.isDirty), the value for Canvas.getVisibleWidth and
*  Canvas.getVisibleHeight will be unreliable until redraw() fires.
*  In this case, if the delayed redraw does change the drawn size of the component, 
*  this notification will be fired a second time when it completes.
* 
*/
resized(): void;

/**
*  Setter method for the Canvas.percentSource attribute.
* 
* @param {Canvas=} eterNew percent source (if omitted existing                   percentSource will just be cleared).
*/
setPercentSource(sourceWidget?:Canvas): void;

/**
* 
*  layoutChildren() is where a Canvas should implement a sizing policy for it's
*  Canvas children. Since layoutChildren calls parentResized() on its children,
*  Canvas.parentResized is a good place for a child to implement a layout policy that
*  can be used within any parent.
*  
*  Recommended practice for a Canvas that manages Canvas children is to create those children
*  without any initial coordinate or size settings and do all sizing when layoutChildren() is
*  called.
*  
*  layoutChildren() is always called at least once before children are drawn, and is called
*  automatically whenever the viewport size changes (which includes both resizing and
*  introduction/removal of scrolling). layoutChildren() can also be manually invoked in any
*  other component-specific situation which changes the layout.
*  
*  NOTE: layoutChildren() may be called before draw() if a widget is resized before draw(), so
*  be sure to avoid errors such as assuming that any children you automatically create have
*  already been created.
*  
*  NOTE: auto-sizing: layoutChildren() is also called once during the initial draw(), before
*  children are drawn, with a "reason" of "initial draw". During this invocation of
*  layoutChildren() it is legal to directly draw children (call child.draw()), which is
*  otherwise never allowed. This allows a Canvas to implement an auto-sizing layout policy by
*  drawing some children before deciding on the sizes of remaining children, which is far more
*  efficient than drawing all children and resizing some of them after they are drawn.
* 
* @param {string} reason why layoutChildren() is being called, passed when framework            code invokes layoutChildren()
*/
layoutChildren(reason:string): void;

/**
*      Resizes the widget to the specified width and height (moves the right and/ or bottom
*      sides of the widget). The width and height parameters can be expressed as a percentage
*      of viewport size or as the number of pixels. See Canvas.width or Canvas.height
*      for more on canvas sizing.
* 
* @param {number=} new width for canvas.
* @param {number=} new height for canvas
*/
resizeTo(width?:number, height?:number): boolean;

/**
*  Fires when the interior size of the parent changes, including parent resize and
*  scrollbar introduction or removal.
*  
*  This method allows a child to implement a layout policy that can be used within any
*  parent, such as a Resizer component that always snaps to the parent's
*  bottom-right corner. The default implementation of this method applies a child's
*  percent sizes, if any, or implements layout based on the Canvas.snapTo property
* 
*/
parentResized(): void;

/**
*  If Canvas.dragAppearance is set to "tracker", this method will be called
*  (if defined), when the user starts to drag this widget. It is an opportunity to update the
*  drag tracker to display something relative to this canvas. Typical implementation will
*  be to call EventHandler.setDragTracker, passing in the desired custom tracker HTML
*  as a string
* 
*/
setDragTracker(): boolean;

/**
*  If this widget is showing scrollbars, and a user drags close to the edge of the viewport,
*  should we scroll the viewport in the appropriate direction?
*  Returns this.canDragScroll if there are scrollbars, else false.
* 
*/
shouldDragScroll(): void;

/**
*  Update the Canvas.overflow of a Canvas after it has been created.
* 
* @param {Overflow} New overflow value.
*/
setOverflow(newOverflow:Overflow): void;

/**
*  This method tells a component to adjust for changes in the size of its content that happen 
*  outside of SmartClient APIs. This may include:
*  
*  
* - size changes due to "replaced" HTML elements (elements that may change size after their 
*    content loads, such as &lt;img&gt; tags with no sizes). To avoid the need to call 
*    adjustForContent(), specify sizes on replaced elements wherever possible.
*  
*  
* - modification of HTML contained in a Canvas by direct manipulation of the DOM - see the 
*    domIntegration. Note that only contents 
*    supplied to a widget via Canvas.contents or via an override of 
*    Canvas.getInnerHTML should be manipulated directly. Contents automatically 
*    generated by SmartClient components (such as the basic structure of a Button) should 
*    never be manipulated: these structures are considered internal, differ by platform, 
*    and will change without notice.
*  
*  
* 
* @param {boolean} By default the adjustment will occur on a small delay for performance reasons.  Pass in this parameter to force immediate adjustment.
*/
adjustForContent(immediate:boolean): void;

/**
*  Returns the scrollTop required to scroll vertically to the end of this widget's content.
* 
*/
getScrollBottom(): Integer;

/**
*  Returns the scrollLeft required to scroll horizontally to the end of this widget's content.
* 
*/
getScrollRight(): Integer;

/**
*  Vertically scrolls the content of the widget to 0
* 
* 
*/
scrollToTop(): void;

/**
*  Vertically scrolls the content of the widget to the end of its content
* 
* 
*/
scrollToBottom(): void;

/**
*  Horizontally scrolls the content of the widget to 0
* 
* 
*/
scrollToLeft(): void;

/**
*  Horizontally scrolls the content of the widget to the end of its content
* 
* 
*/
scrollToRight(): void;

/**
*  Scroll this widget by some pixel increment in either (or both) direction(s).
*  
*  Note: If you attempt to call this API before the widget is drawn, the call will be ignored.
* 
* 
* @param {number} Number of pixels to scroll horizontally
* @param {number} Number of pixels to scroll vertically
*/
scrollBy(dX:number, dY:number): void;

/**
*  Scroll this widget by some percentage of scroll size in either (or both) direction(s).
* 
* 
* @param {number | string} Percentage to scroll horizontally. Will accept either                   a numeric percent value, or a string like "10%".
* @param {number | string} Percentage to scroll horizontally. Will accept either                   a numeric percent value, or a string like "10%".
*/
scrollByPercent(dX:number | string, dY:number | string): void;

/**
*  Scrolls the content of the widget so that the origin (top-left corner) of the content
*  is left pixels to the left and top pixels above the widget's top-left corner (but still
*  clipped by the widget's dimensions).
*  
*  This is guaranteed to be called whenever this Canvas is scrolled, whether scrolling is
*  initiated programmatically, by custom scrollbars, or a by a native scrollbar.
*  
*  Note: If you attempt to call this API before the widget is drawn, the call will be ignored.
* 
* 
* @param {Integer=} the left coordinate
* @param {Integer=} the top coordinate
*/
scrollTo(left?:Integer, top?:Integer): void;

/**
*  Scroll this widget to some position specified as a percentage of scroll size in either
*  (or both) direction(s).
* 
* 
* @param {number | string} Left Percentage position to scroll to                    Will accept either a numeric percent value, or a                     string like "10%".
* @param {number | string} Top Percentage position to scroll to                    Will accept either a numeric percent value, or a                    string like "10%".
*/
scrollToPercent(left:number | string, top:number | string): void;

/**
*  This method is the programmatic equivalent of the user pressing the "Page Up" key while 
*  this widget has the focus. It scrolls the widget's content upwards by the viewport 
*  height, if the content can be scrolled that far upwards
* 
*/
pageScrollUp(): void;

/**
*  This method is the programmatic equivalent of the user pressing the "Page Down" key while 
*  this widget has the focus. It scrolls the widget's content downwards by the viewport 
*  height, if the content can be scrolled that far downwards
* 
*/
pageScrollDown(): void;

/**
*  Sets this widget's visibility to "inherit", so that it becomes visible if all of its parents
*  are visible or it has no parents.
*  
*  If the widget has not yet been drawn (and doesn't have a parent or master), this method
*  calls the draw method as well.
* 
* 
*/
show(): void;

/**
*  Recursively show the canvas and all its parents so the canvas will be visible.
*  
*  If the widget has not yet been drawn, this method calls the draw method as well.
* 
* 
*/
showRecursively(): void;

/**
*  Returns this Canvas's "panel container". A panel container is a widget that manages a 
*  collection of panels, like a TabSet or SectionStack. If this
*  Canvas is not a child of such a panel container, this method returns null.
* 
* 
*/
getPanelContainer(): Canvas;

/**
*  Sets this Canvas's "panel container". A panel container is a widget that manages a 
*  collection of panels, like a TabSet or SectionStack. SmartClient
*  uses this method internally when child panels are added to panel container widgets; if you
*  need to create a panel container widget that does not extend one of the built-in ones (these
*  are TabSet, SectionStack and Window), your code 
*  should do the same thing.
* 
* 
* @param {Canvas} The container widget for this canvas
*/
setPanelContainer(container:Canvas): void;

/**
*  For "panel container" widgets like TabSet or SectionStack, this 
*  method reveals the child Canvas passed in by whatever means is applicable for the particular 
*  type of container. For example, when called on a TabSet, it selects the tab containing the
*  passed-in child.
*  
*  For other types of Canvas, this method simply shows the passed-in child if it is not 
*  currently visible.
*  
*  If the passed-in widget is not a child of this Canvas, this method has no effect
* 
* 
* @param {GlobalId | Canvas} the child Canvas to reveal, or its global ID
*/
revealChild(child:GlobalId | Canvas): void;

/**
*      Sets the widget's CSS visibility attribute to "hidden".
* 
*/
hide(): void;

/**
*  Returns true if the widget is visible, taking all parents into account, so that a widget
*  which is not hidden might still report itself as not visible if it is within a hidden
*  parent.
*  
*  NOTE: Undrawn widgets will report themselves as visible if they would be visible if drawn.
* 
* 
*/
isVisible(): boolean;

/**
*  set the enabled state of this object.
* 
* 
* @param {boolean} pass false to disable or anything else to enable
* @deprecated * As of Smartclient version 5.5, deprecated in favor of Canvas.setDisabled

*/
setEnabled(newState:boolean): void;

/**
*  set the disabled state of this object
* 
* @param {boolean} new disabled state of this object - pass true to disable the widget
*/
setDisabled(disabled:boolean): void;

/**
*  Enables this widget and any children / peers of this widget.
* 
*/
enable(): void;

/**
*  Disables this widget and any children and peers of this widget.
* 
*/
disable(): void;

/**
*  Is this canvas disabled? Note that the disabled state is inherited - this method will return
*  true if this widget, or any of its ancestors are marked disabled.
* 
*/
isDisabled(): boolean;

/**
*  Returns true if the widget and all widgets above it in the containment hierarchy are enabled.
*  Returns false otherwise.
* 
* @deprecated * As of SmartClient version 5.5 deprecated in favor of Canvas.isDisabled.

*/
isEnabled(): boolean;

/**
*  Change whether a widget can accept keyboard focus.
* 
* @param {boolean} whether the widget should now accept focus
*/
setCanFocus(canFocus:boolean): void;

/**
*  Returns true if this Canvas has the keyboard focus. Note that in Internet Explorer
*  focus notifications can be asynchronous (see EventHandler.synchronousFocusNotifications).
*  In this case, canvas.isFocused() method can correctly 
*  return false when, intuitively, you would expect it to return true:
*  
*    someCanvas.focus();
*    if (someCanvas.isFocused()) {
*      // In most browsers we would get here, but not in Internet Explorer with
*      // EventHandler.synchronousFocusNotifications disabled.
*    }
* 
*/
isFocused(): boolean;

/**
*  If this canvas can accept focus, give it keyboard focus. After this method, the canvas
*  will appear focused and will receive keyboard events.
* 
*/
focus(): void;

/**
*  If this canvas has keyboard focus, blur it. After this method, the canvas
*  will no longer appear focused and will stop receiving keyboard events.
* 
*/
blur(): void;

/**
*  Returns true if the keyboard focus is in this Canvas or any child of this Canvas.
* 
*/
containsFocus(): boolean;

/**
*  Set the accessKey for this canvas.
*  
*  The accessKey can be set to any alphanumeric character (symbols not supported)
*  Having set an accessKey, the canvas will be given focus when the user hits
*  Alt+[accessKey], or in Mozilla Firefox 2.0 and above, Shift+Alt+[accessKey].
* 
* 
* @param {string} Character to use as an accessKey for this widget. Case Insensitive.
*/
setAccessKey(accessKey:string): void;

/**
*  If an explicit Canvas.tabIndex was assigned to this widget, clear it.
*  This will enable automatic tab index managment behaviors via the TabIndexManager class
*  as described in the tabOrderOverview
* 
*/
clearExplicitTabIndex(): void;

/**
*  This method is executed on draw. Default implementation for top-level widgets ensures 
*  this widget is at the end of the tab-sequence.
*  
*  Has no effect if this canvas is embedded in a Canvas.getParentCanvas.
* 
*/
updateTabPositionForDraw(): void;

/**
*  Update the childrens' tab positions, slotting them under this widget in the
*  TabIndexManager, in the order defined by Canvas.getChildTabPosition.
*  This method will skip any children where Canvas.updateTabPositionOnReparent is
*  false.
*  
*  This method is called automatically on canvas draw(). It may be overridden by subclasses
*  for custom tab-order behavior.
* 
*/
updateChildTabPositions(): void;

/**
*  Ensure that a specific child is slotted correctly into the page's tab order. Default
*  implementation will, if Canvas.updateTabPositionOnReparent is true, ensure 
*  the child canvas shows up in the TabIndexManager tree
*  under the entry for this widget (the parent), in the position returned 
*  by Canvas.getChildTabPosition.
*  
*  This method is called automatically in cases where a single child's tab position may
*  need to be updated - such as if a child is added to a drawn parent.
*  
*  See also Canvas.updateChildTabPositions
* 
* 
* @param {Canvas} child to have tab position updated.
*/
updateChildTabPosition(child:Canvas): void;

/**
*  For a given child widget where Canvas.updateTabPositionOnReparent is true, 
*  return the expected tab position within this parent.
*  Default implementation will any explicit relative tab position specified by 
*  Canvas.setRelativeTabPosition, and otherwise put children in the 
*  same order as defined in the children array
*  
*  Overridden in the Layout class to return the position in the members array.
* 
* @param {Canvas} Child to get local tab position for
*/
getChildTabPosition(child:Canvas): Integer;

/**
*  Setter for the Canvas.updateTabPositionOnReparent attribute.
* 
* @param {boolean} new value for canvas.updateTabPositionOnReparent
*/
setUpdateTabPositionOnReparent(updateOnReparent:boolean): void;

/**
*  Assign a relative tab position for this canvas. The meaning of a "relative" tab position
*  varies depending on where the canvas is in the page.
*  
*  For canvases with no specified Canvas.getParentCanvas, (or where
*  Canvas.updateTabPositionOnReparent is false), this method will and move the
*  canvas to the appropriate tab-position among other top level canvases. It will also disable
*  Canvas.updateTabPositionOnDraw so if this method is called before draw, drawing
*  this canvas will not cause its tab position to change.
*  
*  For canvases embedded in a Canvas.getParentCanvas, this method will move the
*  canvas to the appropriate tab position among the other children of the parent.
*  Implementation note: This is achieved by setting an internal property to
*  indicate the new tab position which will be respected by the default 
*  Canvas.getChildTabPosition implementation, and calling 
*  Canvas.updateChildTabPositions to implement a reflow. Therefore if
*  Canvas.getChildTabPosition has been overridden, this method may have no effect.
*  
*  As with other APIs related to TabIndexManager, tab indices
*  are treated as a hierarchy by default. By setting the relative tab position of a
*  canvas which is not itself focusable but has focusable descendents, these descendents'
*  tab position will be updated.
*  
*  Note that after this method has been called, the tab position can be modified by
*  subsequent code to shift another sibling in front of this one, or reparent this canvas.
* 
* 
* @param {Integer} new relative tab position
*/
setRelativeTabPosition(position:Integer): void;

/**
*  Assign an explicit Canvas.tabIndex to this widget at runtime.
*  
*  Developers may also call Canvas.clearExplicitTabIndex to clear any explicitly
*  assigned tab index, and have the widget participate in automatic tab position allocation.
*  For more information see Canvas.tabIndex
* 
* 
* @param {number} New tabIndex for this widget. Must be less than             Canvas.TAB_INDEX_FLOOR to avoid interfering with auto-assigned              tab indices on the page.
*/
setTabIndex(tabIndex:number): void;

/**
*  Shifts focus to the next focusable element after this one. This programatically 
*  simulates the user experience of a Tab keypress, and is used automatically when 
*  a blocking Canvas.showClickMask is up to ensure focus does not
*  move to any masked elements in the UI.
*  
*  This method makes use of the TabIndexManager.shiftFocus method to request
*  focus be changed to the adjacent registered entry. By default standard focusable 
*  SmartClient UI elements, including Canvases, FormItems, FormItemIcons, etc are
*  registered with the TabIndexManager in the appropriate order, and will accept focus
*  if Canvas.canFocus, and not FormItem.disabled or
*  Canvas.showClickMask. 
*  
*  The TabIndexManager maintains a hierarchy of focusable targets - so if a parent
*  canvas contains focusable children, they will typically be nested under the parent
*  canvas in this hierarchy. If you want to shift focus to the next target outside this
*  hierarchy (IE: skip any children and put focus into the next widget on the page
*  outside this one), you can use use Canvas.focusAfterGroup.
*  
*  NOTE: Focusable elements created directly in the raw HTML bootstrap or 
*  by application code will not be passed focus by this method unless they have also been
*  explicitly registered with the TabIndexManager. See the tabOrderOverview
*  for more information.
* 
* 
*/
focusInNextTabElement(): void;

/**
*  Shifts focus to the next focusable element after this one, skipping any elements
*  nested inside the tabbing group for this canvas, such as focusable children.
*  
*  This method makes use of the TabIndexManager.shiftFocusAfterGroup method to request
*  focus be changed to the next registered entry. By default standard focusable 
*  SmartClient UI elements, including Canvases, FormItems, FormItemIcons, etc are
*  registered with the TabIndexManager in the appropriate order, and will accept focus
*  if Canvas.canFocus, and not FormItem.disabled or
*  Canvas.showClickMask. 
*  
*  This method differs from Canvas.focusInNextTabElement in that it will
*  skip any descendants of this widget in the TabIndexManager's hierarchy of potential
*  focus target. By default this means focus will be moved to the next target on the
*  page which is not a descendant of this widget.
*  
*  FormItems support a similar method: FormItem.focusAfterItem.
*  
*  NOTE: Focusable elements created directly in the raw HTML bootstrap or 
*  by application code will not be passed focus by this method unless they have also been
*  explicitly registered with the TabIndexManager. See the tabOrderOverview
*  for more information.
* 
* 
*/
focusAfterGroup(): void;

/**
*  Shifts focus to the previous focusable element before this one. This programatically 
*  simulates the user experience of a Shift+Tab keypress.
*  
*  This method makes use of the TabIndexManager.shiftFocus method to request
*  focus be changed to the adjacent registered entry. By default standard focusable 
*  SmartClient UI elements, including Canvases, FormItems, FormItemIcons, etc are
*  registered with the TabIndexManager in the appropriate order, and will accept focus
*  if Canvas.canFocus, and not FormItem.disabled or
*  Canvas.showClickMask. 
*  
*  NOTE: Focusable elements created directly in the raw HTML bootstrap or 
*  by application code will not be passed focus by this method unless they have also been
*  explicitly registered with the TabIndexManager. See the tabOrderOverview
*  for more information.
* 
* 
*/
focusInPreviousTabElement(): void;

/**
*  Shifts focus to the start or end of this canvas and its descendants.
*  
*  This method makes use of the TabIndexManager.shiftFocusWithinGroup API to
*  request focus be changed within the set of focusable targets registered under
*  this canvas.
*  
*  If the start parameter is true, if the canvas itself is focusable it will
*  be given focus, otherwise the first focusable descendant will be given focus.
*  If the start parameter is false, the last focusable descendant will be
*  given focus (or if the canvas itself is focusable but there are no focusable descendants, 
*  it will receive focus).
*  
*  A use case for this might be to programmatically shift focus to the first or last
*  button in a toolbar or similar.
* 
* @param {boolean} Should we focus at the start or the end of this widget and its  descendants.
*/
focusAtEnd(start:boolean): void;

/**
*  Get the z-Index of this canvas.
* 
* 
* @param {boolean} If passed true, for undrawn widgets, resolve "auto" to the next available zIndex.
*/
getZIndex(resolveToNumber:boolean): number;

/**
*  Puts this widget at the top of the stacking order, so it appears in front of all other
*  widgets in the same parent.
* 
*/
bringToFront(): void;

/**
*  Puts this widget at the bottom of the stacking order, so it appears behind all other
*  widgets in the same parent.
* 
*/
sendToBack(): void;

/**
*  Puts this widget just above the specified widget in the stacking order, so it appears in
*  front of the specified widget if both widgets have the same parent.
* 
* @param {Canvas} canvas to move above
*/
moveAbove(canvas:Canvas): void;

/**
*  Puts this widget just below the specified widget in the stacking order, so it appears
*  behind the specified widget if both widgets have the same parent.
* 
* @param {Canvas} canvas to move below
*/
moveBelow(canvas:Canvas): void;

/**
*    Returns the contents of a Canvas. The contents are an HTML string.
* 
*/
getContents(): HTMLString;

/**
*  Changes the contents of a widget to newContents, an HTML string.
*  
*  When Canvas.dynamicContents is set, setContents() can
*  also be called with no arguments to cause contents to be re-evaluated.
* 
* 
* @param {HTMLString=} an HTML string to be set as the contents of this widget
*/
setContents(newContents?:HTMLString): void;

/**
*  Sets the background color of this component to newColor.
* 
* @param {CSSColor} new background color, or null to remove the current background color.
*/
setBackgroundColor(newColor:CSSColor): void;

/**
*      Sets the background to an image file given by newImage. This URL should be given as a
*      string relative to the image directory for the page (./images by default).
* 
* @param {SCImgURL} new URL (local to Page image directory) for background image
*/
setBackgroundImage(newImage:SCImgURL): void;

/**
*  Set the CSS border of this component, as a CSS string including border-width,
*  border-style, and/or color (eg "2px solid blue").
*  
*  This property applies the same border to all four sides of this component. Different
*  per-side borders can be set in a CSS style and applied via Canvas.styleName.
* 
* 
* @param {string} new border to set to (eg: "2px solid black")
*/
setBorder(newBorder:string): void;

/**
*  Sets the opacity for the widget to the newOpacity value. This newOpacity
*  value must be within the range of 0 (transparent) to 100 (opaque).
*  Null means don't specify opacity directly.
*  Note that heavy use of opacity may have a performance impact on some older
*  browsers.
*  
*  In older versions of Internet Explorer (Pre IE9 / HTML5), opacity is achieved
*  through proprietary filters. If
*  Canvas.neverUseFilters within this application
*  developers must set Canvas.useOpacityFilter to true for specific components
*  on which opacity support is required.
*  
*  Also note that opacity is incompatible
*  with Canvas.useBackMask.
* 
* @param {number} new opacity level
*/
setOpacity(newOpacity:number): void;

/**
*      Sets the cursor for this widget to cursor. See the cursor property
*      for possible values.
* 
* @param {Cursor} new cursor
*/
setCursor(newCursor:Cursor): void;

/**
*  When Canvas.showHoverComponents is true, this method is called to get the component to show
*  as a hover for this Canvas. There is no default implementation of this method, so you need
*  to override it with an implementation that returns a Canvas that suits your needs.
*  
*  By default, components returned by getHoverComponent() will not be
*  automatically destroyed when the hover is hidden. To enforce this, set
*  Canvas.hoverAutoDestroy to true on the returned component.
* 
* 
*/
getHoverComponent(): Canvas | Partial<Canvas>;

/**
*  Handler fired on a delay when the user hovers the mouse over this hover-target.
*  Default implementation will fire this.hover() (if defined), and handle
*  showing the hover canvas if this.showHover is true.
* 
*/
handleHover(): void;

/**
*  If this canvas is currently showing a hover (see Canvas.handleHover), this method
*  can be called to update the HTML contents of the hover. Has no effect if this widget is 
*  showing Canvas.showHoverComponents, or if 
*  the hover canvas is not showing for this widget.
* 
* @param {string=} Option to specify new HTML for the hover. If not passed, the  result of Canvas.getHoverHTML will be used instead. Note that   if the hover HTML is empty the hover will be hidden.
*/
updateHover(hoverHTML?:string): void;

/**
*  If Canvas.showHover is true for this canvas, this notification method will be
*  fired whenever the hover shown in response to Canvas.handleHover is
*  hidden. This method may be observed or overridden.
* 
*/
hoverHidden(): void;

/**
*  If canHover is true for this widget, the hover string method will
*  be fired when the user hovers over this canvas. If this method returns false, it will
*  suppress the default behavior of showing a hover canvas if this.showHover
*  is true.
* 
*/
hover(): boolean;

/**
*  If this.showHover is true, when the user holds the mouse over this Canvas for
*  long enough to trigger a hover event, a hover canvas is shown by default. This method returns
*  the contents of that hover canvas. Default implementation returns this.prompt -
*  override for custom hover HTML. Note that returning null or an empty string will
*  suppress the hover canvas altogether.
* 
*/
getHoverHTML(): HTMLString;

/**
*  Sets the CSS class for this widget
* 
* @param {CSSStyleName} new CSS style name
*/
setStyleName(newStyle:CSSStyleName): void;

/**
*  Executed when the right mouse button is clicked. The default implementation of
*  this method auto-creates a Menu from the Canvas.contextMenu property on
*  this component and then calls Menu.showContextMenu on it to show it.
*  
*  If you want to show a standard context menu, you can simply define your Menu and set it as the
*  contextMenu property on your component - you do not need to override this method.
*  
*  If you want to do some other processing before showing a menu or do something else entirely, then
*  you should override this method. Note that the return value from this method controls whether or
*  not the native browser context menu is shown.
* 
* 
*/
showContextMenu(): boolean;

/**
* 
*  The default implementation of this method hides the contextMenu currently being shown for this
*  component (which occurs when the mouse button that toggles the context menu is released).
*  Override if you want some other behavior.
* 
* 
*/
hideContextMenu(): void;

/**
* 
*  Executed when mouse enters this widget. No default implementation.
* 
* 
*/
mouseOver(): boolean;

/**
*  Executed when the left mouse button is pressed on this widget. No default implementation.
* 
* 
*/
mouseDown(): boolean;

/**
*  Executed when the right mouse button is pressed on this widget. No default implementation.
* 
* 
*/
rightMouseDown(): boolean;

/**
* 
*  Repeating notification method for the user holding the left mouse button down over this
*  canvas.
*  
*  The mouseStillDown event is fired immediately when the mouse goes down.
*  If the user holds the mouse down, after a pause of
*  Canvas.mouseStillDownInitialDelay, it will begin to fire repeatedly every 
*  Canvas.mouseStillDownDelay milliseconds.
*  
*  This provides developers with a simple way to handle the common "repeated action" 
*  use case where a user can click a UI element to perform an action once, or click 
*  and hold to perform the action repeatedly.
*  Examples of this include standard scrollbar button behavior and buttons 
*  to increase or decrease the value in a spinner type input element.
*  
*  This event is not native to JavaScript, but is provided by the ISC system.
* 
* 
*/
mouseStillDown(): boolean;

/**
* 
*  Executed when the mouse moves within this widget. No default implementation.
* 
* 
*/
mouseMove(): boolean;

/**
* 
*  Executed when the mouse leaves this widget. No default implementation.
*  
*  Note that if the mouse goes over a child of this canvas, the mouseOut event will fire as it would
*  if the user rolled entirely off the canvas. Developers may determine whether the mouse is still over
*  a descendant of this component via Canvas.containsEventTarget.
* 
* 
*/
mouseOut(): boolean;

/**
* 
*  Executed when the left mouse is released on this widget. No default implementation.
* 
* 
*/
mouseUp(): boolean;

/**
* 
*  Executed when the left mouse is clicked (pressed and then released) on this widget. No default
*  implementation.
* 
* 
*/
click(): boolean;

/**
* 
*  Executed when the left mouse button is clicked twice in rapid succession (within
*  Canvas.doubleClickDelay by default) in this object.
* 
* 
*/
doubleClick(): boolean;

/**
* 
*  Executed when the mouse wheel is actuated.
* 
* 
*/
mouseWheel(): boolean;

/**
* 
*  Executed when dragging first starts. No default implementation. Create this handler to set
*  things up for the drag reposition.
* 
* 
*/
dragRepositionStart(): boolean;

/**
* 
*  Executed every time the mouse moves while drag-repositioning. If this method
*  does not return false, the Canvas.dragTarget (or outline if
*  Canvas.dragAppearance is set to "outline") will automatically be moved as appropriate
*  whenever the mouse moves.
* 
* 
*/
dragRepositionMove(): boolean;

/**
* 
*  Executed when the mouse button is released at the end of the drag. Your
*  widget can use this opportunity to fire custom code based upon where the
*  mouse button was released, etc.
*  
*  Returning true from this handler will cause the Canvas.dragTarget (or outline if
*  Canvas.dragAppearance is set to "outline") to be left in its current
*  location. Returning false from this handler will cause it to snap back to its
*  original location.
* 
* 
*/
dragRepositionStop(): boolean;

/**
* 
*  Executed when resize dragging first starts. No default implementation.
*  Create this handler to set things up for the drag resize.
* 
* 
*/
dragResizeStart(): boolean;

/**
* 
*  Executed every time the mouse moves while drag-resizing. If this method
*  does not return false, the Canvas.dragTarget (or outline if
*  Canvas.dragAppearance is set to "outline") will automatically be moved as appropriate
*  whenever the mouse moves.
* 
* 
*/
dragResizeMove(): boolean;

/**
* 
*  Executed when the mouse button is released at the end of the drag resize. Your
*  widget can use this opportunity to fire custom code based upon where the
*  mouse button was released, etc.
*  
*  Returning true from this handler will cause the Canvas.dragTarget (or outline if
*  Canvas.dragAppearance is set to "outline") to be left at its current size. Returning
*  false from this handler will cause it to snap back to its original location size
* 
* 
*/
dragResizeStop(): boolean;

/**
*  Executed when dragging first starts. Your widget can use this opportunity to set
*  things up for the drag, such as setting the drag tracker. Returning false from this
*  event handler will cancel the drag action entirely.
*  
*  A drag action is considered to be begin when the mouse has moved
*  Canvas.dragStartDistance with the left mouse down.
* 
* 
*/
dragStart(): boolean;

/**
*  Executed every time the mouse moves while dragging this canvas.
* 
* 
*/
dragMove(): boolean;

/**
*  Executed when the mouse button is released at the end of the drag. Your widget can
*  use this opportunity to fire code based on the last location of the drag or reset any
*  visual state that was sent.
* 
* 
*/
dragStop(): boolean;

/**
* 
*  Executed when the compatible dragged object is first moved over this drop target. Your
*  implementation can use this to show a custom visual indication that the object can be
*  dropped here.
* 
* 
*/
dropOver(): boolean;

/**
* 
*  Executed whenever the compatible dragged object is moved over this drop target. You
*  can use this to show a custom visual indication of where the drop would occur within the
*  widget, or to show the Canvas.noDropCursor to indicate that
*  this is not a valid drop target, typically if Canvas.willAcceptDrop returns false.
*  
*  For details on showing a 'no drop' cursor when the user drags over all invalid drop
*  targets, see EventHandler.showNoDropIndicator.
* 
* 
*/
dropMove(): boolean;

/**
* 
*  Executed when the dragged object is no longer over this drop target, including when the drag
*  interaction is ending with a drop on this drop target. If you have set a visual indication 
*  in dropOver or dropMove, you should reset it to its normal state in dropOut.
* 
* 
*/
dropOut(): boolean;

/**
* 
*  Executed when the mouse button is released over a compatible drop target at the end of
*  a drag sequence. Your widget should implement whatever it wants to do when receiving a
*  drop here. For example, in a file moving interface, a drop might mean that you should
*  move or copy the dragged file into the folder it was dropped on, or dropping something in
*  a trash can might mean to clear it from the screen.
* 
* 
*/
drop(): boolean;

/**
* 
*  Executed when a key is pressed on a focusable widget (Canvas.canFocus: true).
*  
*  Use EventHandler.getKey to find out the KeyName of the key that
*  was pressed, and use EventHandler.shiftKeyDown and related functions to determine
*  whether modifier keys were down.
* 
* 
*/
keyDown(): boolean;

/**
* 
*  Executed when a key is released on a focusable widget (Canvas.canFocus: true).
*  
*  Use EventHandler.getKey to find out the KeyName of the key that
*  was pressed, and use EventHandler.shiftKeyDown and related functions to determine
*  whether modifier keys were down.
* 
* 
*/
keyUp(): boolean;

/**
* 
*  Executed when a key is pressed and released on a focusable widget (Canvas.canFocus:
*  true).
*  
*  Use EventHandler.getKey to find out the KeyName of the key that
*  was pressed, and use EventHandler.shiftKeyDown and related functions to determine
*  whether modifier keys were down.
* 
* 
*/
keyPress(): boolean;

/**
* 
*  Returns true if the widget object being dragged can be dropped on this widget, and
*  false otherwise. The default implementation of this method simply compares the
*  Canvas.dragType of the dragTarget (the component being dragged from)
*  with the list of Canvas.dropTypes on this Canvas. If the Canvas.dropTypes
*  list contains the Canvas.dragType value, then this method returns true. Otherwise it
*  returns false.
*  No matter what you return, Canvas.dropOver and Canvas.dropMove
*  will still be called, and their return values will determine whether those events are bubbled
*  to parent elements.
*  However, what you return from willAcceptDrop() does determine whether
*  Canvas.drop will be called.
*  
*  
* - If you return true, then drop() will be called, and its return value
*  will determine whether the event is bubbled to parent elements
*  
* - If you return false, then drop() will not be called, and the event
*  will not be bubbled.
*  
* - If you return null, then drop() will not be called, but the event
*  will be bubbled to parent elements (giving them a chance to handle the drop).
*  
*  So, you should return false to definitively deny a drop, and return null if it could
*  make sense to allow a parent element, such as a Layout, to handle the drop. 
* 
* 
*/
willAcceptDrop(): boolean;

/**
* 
*  Override this method to provide a custom snap-to grid. Note that you do not need to do
*  this if your grid is regular (ie, grid points are every x pixels); regular grids should be
*  defined using Canvas.snapHGap and Canvas.snapVGap.
*  You should only override this method if you want to provide support for a grid of
*  irregularly-placed points
* 
* 
* @param {Integer} x-coordinate of the drag event relative to the inside of this widget
* @param {string=} "before" or "after" denoting whether the returned coordinate should  match the left or right edge of the current square. If unset Canvas.snapHDirection will  be used by default
*/
getHSnapPosition(coordinate:Integer, direction?:string): Integer;

/**
*  Override this method to provide a custom snap-to grid. Note that you do not need to do
*  this if your grid is regular (ie, grid points are every x pixels) - regular grids should be
*  defined using Canvas.snapHGap and Canvas.snapVGap.
*  You should only override this method if you want to provide support for a grid of
*  irregularly-placed points
* 
* 
* @param {Integer} y-coordinate of the drag event relative to the inside of this widget
* @param {string=} "before" or "after" denoting whether the returned coordinate should  match the top or bottom edge of the current square. If unset Canvas.snapHDirection will  be used by default
*/
getVSnapPosition(coordinate:Integer, direction?:string): Integer;

/**
*  Override this method to give programmatic control over whether or not the parameter
*  dragTarget should snap to this object's grid when dropped. Note that this only applies
*  if snap-to-grid is enabled on either dragTarget or this object. See
*  Canvas.snapToGrid and Canvas.childrenSnapToGrid.
*  
*  The default implementation simply returns true.
* 
* 
* @param {Canvas} The object about to be dropped
*/
shouldSnapOnDrop(dragTarget:Canvas): boolean;

/**
*  Set the showSnapGrid property.
* 
* @param {boolean} */
setShowSnapGrid(show:boolean): void;

/**
*  Add a candidate to Canvas.snapAlignCandidates. Duplicates are automatically avoided.
* 
* 
* @param {Canvas} */
addSnapAlignCandidate(newCandidate:Canvas): void;

/**
*  Remove a candidate from Canvas.snapAlignCandidates. If the passed widget was not actually
*  a candidate, nothing happens and no warning is logged.
* 
* 
* @param {Canvas} */
removeSnapAlignCandidate(candidate:Canvas): void;

/**
*  Return the full URL for an image to be drawn in this canvas.
*  
*  If the passed URL begins with the special prefix "[SKIN]", it will have the
*  widget.skinImgDir and Page.skinImgDir prepended. Otherwise the image is assumed to be
*  application-specific, and will have the widget.appImgDir and Page.appImgDir automatically
*  prepended.
*  
*  Note that if passed an absolute path (starting with "/" or "http://" for example), no extra
*  image directory information will be prepended to the generated URL.//
* 
* 
* @param {string} URL local to skin or application image directory
* @param {string=} optional image directory to override the default for this Canvas
*/
getImgURL(URL:string, imgDir?:string): string;

/**
*  Generates the HTML for an image unique to this Canvas.
*  
*  The full URL for the image will be formed according to the rules documented for
*  Canvas.getImgURL.
*  
*  The created image will have an identifier unique to this Canvas, and subsequent calls to
*  Canvas.getImage and
*  Canvas.setImage
*  with the name passed to this function will act on the image object produced by the HTML
*  returned from this call.
* 
* 
* @param {SCImgURL} URL local to the skin or application directory.   NOTE: instead of passing several parameters, you can pass an object as the 'src'    parameter with properties for all the various function parameters with, eg:    canvas.imgHTML( {src:"foo", width:10, height:10} );
* @param {number=} width of the image
* @param {number=} height of the image
* @param {string=} name for the image
* @param {string=} additional attributes to write in the tag
* @param {string=} image-specific image directory to override the default                 for this Canvas
*/
imgHTML(src:SCImgURL, width?:number, height?:number, name?:string, extraStuff?:string, imgDir?:string): string;

/**
*  Retrieve a native image or SVG object element by name.
*  
*  The element must have been created from HTML generated by calling
*  Canvas.imgHTML on this particular Canvas.
* 
* 
* @param {string} name of the image or SVG object to get, as originally passed to               imgHTML()
*/
getImage(identifier:string): DOMElement;

/**
*  Set the URL of an image or SVG object element by name.
*  
*  The element must have been created from HTML generated by calling
*  canvas.imgHTML() on this particular Canvas.
* 
* 
* @param {string} name of the image to change, as originally passed to               imgHTML
* @param {SCImgURL} URL for the image
* @param {string=} optional image directory, overrides the default for this Canvas
*/
setImage(identifier:string, URL:SCImgURL, imgDir?:string): void;

/**
*  Generates the HTML for a standard link (anchor) element.
* 
* 
* @param {string} URL for the link to point to
* @param {HTMLString=} HTML to display in the link element (defaults to the href)
* @param {string=} Target window for the link (defaults to opening in a new, unnamed window)
* @param {string=} optional ID for the link element to be written out
* @param {Integer=} optional tabIndex for the link
* @param {string=} optional accessKey for the link
*/
linkHTML(href:string, text?:HTMLString, target?:string, ID?:string, tabIndex?:Integer, accessKey?:string): HTMLString;

/**
*  Set the Canvas.edgeOpacity and mark the canvas for redraw
* 
* @param {Integer} new edge-opacity level
*/
setEdgeOpacity(newOpacity:Integer): void;

/**
*  Method to update Canvas.showShadow.
* 
* @param {boolean} true if the shadow should be visible false if not
*/
setShowShadow(showShadow:boolean): void;

/**
*  Setter for Canvas.groupLabelBackgroundColor.
* 
* @param {CSSColor} the new grouping label background color.
*/
setGroupLabelBackgroundColor(groupLabelBackgroundColor:CSSColor): void;

/**
*  Setter for Canvas.groupTitle.
* 
* @param {HTMLString} The new title for the grouping.
*/
setGroupTitle(newTitle:HTMLString): void;

/**
*  Set the showPointer property.
* 
* @param {boolean} */
setShowPointer(show:boolean): void;

/**
*  Provides a new DataContext to the Canvas. If the DataContext is new,
*  DataBoundComponent contained within this Canvas will be
*  automatically bound as described in Canvas.dataContext. If the DataContext replaces
*  an existing one, any contained components that were originally bound against the
*  DataContext will be re-bound.
* 
* 
* @param {DataContext} dataContext to use for automatic binding
*/
setDataContext(dataContext:DataContext): void;

/**
*  Get the current value of the rule context collected by the Canvas.ruleScope of this component
*  (which may be this component itself or whatever component is managing the
*  ruleScope for this component).
*  
*  If the databoundOnly parameter is passed as true, only data from components that
*  actually have a DataSource is included.
*  
*  Use Canvas.ruleContextChanged to get a notification of changes to the rule context.
* 
* 
* @param {boolean=} whether to include only data from components that have a                 DataSource
*/
getRuleContext(databoundOnly?:boolean): object;

/**
*  Notification that the rule context gathered by the Canvas.ruleScope has changed. 
*  
*  This notification fires only on the component designated as the Canvas.ruleScope; components
*  that are merely contributing data to the rule context do not fire
*  ruleContextChanged.
* 
* @param {object} the new rule context
*/
ruleContextChanged(data:object): void;

/**
*  Provide data to the Canvas.ruleScope component, to be made available in the rule context at the
*  specified path.
*  
*  path must be one or more valid identifiers with either dot (.) or slash (/) used
*  as a separator. 
*  
*  data can be any value, including both atomic values like a Boolean or String, or a
*  nested data structure. Pass data as null to remove data from the
*  context at the specified path. When a DataSource is passed to define the
*  provided data type (described below) a call to Canvas.removeRuleContext is the correct way
*  to remove the data along with the schema.
*  
*  dbc is the DataBoundComponent to be identified as the owner of the rule context
*  contribution. This component is used to handle any conflicts between multiple components
*  contributing to the same base path (i.e. first segment of path). For any collision an editable
*  display (such as a form or editable grid) wins over a static display (such as a non-editable
*  grid with a selection). Hidden or cleared components have lowest priority even if editable.
*  For two editable components the first becomes the owner.
*  
*  dbc can also be a DataSource to define the data type of the provided data.
*  For example, when a singular Date value is provided to the ruleContext the type is not
*  known when using the value in a ruleCriteria. To counter that, a data type can be specified
*  using a DataSource and optionally a fieldName. The path will then be associated
*  with the DataSource and fieldName so criteria processing knows to compare the value
*  correctly. When providing data as a Record or array of Records only the DataSource is needed.
*  
*  See ruleCriteria for details on matching values within criteria
*  when schema is not provided.
*  
*  Note that DataBoundComponents automatically contribute to the ruleContext as described
*  in Canvas.ruleScope.
* 
* 
* @param {string} path where data should be made available
* @param {any} data to contribute to rule context
* @param {DataBoundComponent | DataSource=} dataBoundComponent contributing to ruleContext   or DataSource defining the provided data type
* @param {string=} field name within DataSource defining the data type for a singular   value (i.e. not a Record and not an Array of Records)
*/
provideRuleContext(path:string, data:any, dbc?:DataBoundComponent | DataSource, fieldName?:string): void;

/**
*  Remove data in the rule context at the specified path along with any user-provided
*  schema.
*  
*  Normally data is removed from the ruleContext by passing null to
*  data in Canvas.provideRuleContext, however, this call will do the same but also
*  removes the schema as explicitly provided to provideRuleContext. 
*  
*  Data automatically provided to the ruleContext, as described by Canvas.ruleScope, along with
*  the associated schema is automatically removed when the contributing DataBoundComponent is
*  destroyed. Therefore there is no need to clean up those ruleContext paths manually.
* 
* 
* @param {string} path where data and schema should be removed
*/
removeRuleContext(path:string): void;

/**
*  Notification method fired when a canvas has been drawn into the page.
* 
*/
drawn(): void;

/**
*  Notification method fired when an ancestor of this component's position changes.
* 
* @param {Canvas} the ancestor that moved
* @param {Integer} horizontal difference between current and previous position
* @param {Integer} vertical difference between current and previous position
*/
parentMoved(parent:Canvas, deltaX:Integer, deltaY:Integer): void;

/**
*  Notification method fired when this component is explicitly moved.
*  Note that a component's position on the screen may also changed due to an ancestor being
*  moved. The Canvas.parentMoved method provides a notification entry point to catch
*  that case as well.
* 
* 
* @param {Integer} horizontal difference between current and previous position
* @param {Integer} vertical difference between current and previous position
*/
moved(deltaX:Integer, deltaY:Integer): void;

/**
*  Notification function fired when this widget receives or loses keyboard focus.
* 
* @param {boolean} If true this widget now has keyboard focus
*/
focusChanged(hasFocus:boolean): void;

/**
*  Notification that this component has just scrolled. Use with
*  Class.observe.
*  
*  Fires for both CSS and Scrollbar.
* 
* 
*/
scrolled(): void;

/**
*  Notification fired when this canvas becomes visible or hidden to the user.
*  Note - this method is fired when the Canvas.isVisible state of this
*  component changes. It may be fired in response an explicit call to Canvas.show
*  or Canvas.hide, or in response to a parent component
*  being shown or hidden when this widgets Canvas.visibility is set to "inherit".
*  
*  Note that a call to Canvas.show or Canvas.hide will not always fire this
*  notification. If this widget has a hidden parent, show or hide would change this
*  components Canvas.visibility property, and may update the CSS visibility attribute
*  of the drawn handle in the DOM, but would not actually hide or reveal the component to
*  the user and as such the notification would not fire.
*  
*  Note also that this notification will only be fired for components which have been
*  Canvas.draw.
* 
* @param {boolean} whether the canvas is visible to the user
*/
visibilityChanged(isVisible:boolean): void;

/**
*  Notification method fired when DataContext is bound. This can occur on the
*  initial draw or by an explicit call to Canvas.setDataContext.
*  
*  This feature allows the use of the dataContext as a general-purpose API
*  to the screen. For example, if you wanted your screen to support dynamically
*  showing or hiding parts of itself based on a button that is external to the screen,
*  you could do that by implementing this handler to show/hide that part of the screen
*  based on the current state of the dataContext.
*  
* 
*/
dataContextChanged(): void;

/**
*  Generate and show a PrintWindow containing a PrintCanvas
*  showing a printable view of this component.
* 
* 
* @param {PrintProperties=} PrintProperties object for customizing the   print HTML output
* @param {Partial<PrintWindow>=} Properties to apply to the   generated print window.
* @param {Callback=} callback to fire when the print preview canvas has   been populated with the printable HTML. This callback takes 2 parameters:   printPreview - a pointer to the generated print canvas shown in the   body of the print window.   printWindow - a pointer to the generated print window and
*/
print(printProperties?:PrintProperties, printWindowProperties?:Partial<PrintWindow>, callback?:Callback): void;

/**
*  Setter for the Canvas.valuesManager attribute. This method may be called directly at 
*  runtime to set the ValuesManager for a component; it has the same effect as calling 
*  ValuesManager.addMember, passing in this DataBoundComponent.
* 
* @param {ValuesManager} new dataPath
*/
setValuesManager(dataPath:ValuesManager): void;

/**
*  Setter for the Canvas.dataPath attribute. This method may be called directly at runtime
*  to set the dataPath on a component, and will also be re-run automatically whenever a canvas'
*  parent changes due to a call to addChild(). This method handles automatically binding
*  the component to the appropriate valuesManager if necessary.
* 
* @param {DataPath} new dataPath
*/
setDataPath(dataPath:DataPath): void;

/**
*  Returns a fully qualified DataPath for this canvas. This is calculated by combining
*  the canvas' specified DataPath with the dataPath of any parent 
*  canvases up to whichever canvas has a specified Canvas.valuesManager specified to actually
*  manage values from this component.
* 
*/
getFullDataPath(): DataPath;

/**
*  This method will set mark the target canvas as the "locator parent"
*  for this canvas, using the specified child name. After calling this method,
*  locators that reference this canvas will use the
*  childName to navigate from the specified parent to this component,
*  exactly how named AutoChild are referenced in locators.
*  
*  Note that, as with SmartClient autoChildren, the locator parent does 
*  not need to be the direct parent of this component, or even a true ancestor, 
*  in the widget hierarchy. However, you should never set the locatorParent
*  to a descendant of this widget as that would lead to infinite loops when
*  attempting to create or resolve locators.
*  
*  This method will also set locatorParent.attributeName, (or 
*  locatorParent.childName if no explicit attributeName was specified)
*  to refer to this canvas, if this is not already the case.
*  
*  If the attribute is already set to refer to some other object, this method
*  will return false without taking any
*  further action.
* 
* 
* @param {Canvas} New locator parent for this canvas
* @param {string} Name to refer from the locator parent to this  canvas in the locator
* @param {string=} Optional attribute to refer from the parent to this canvas.  If unset the childName will be used instead.
*/
setLocatorParent(locatorParent:Canvas, childName:string, attributeName?:string): boolean;

/**
*  When using the devTools framework and asking an
*  EditContext to EditContext.serializeAllEditNodes,
*  updateEditNode is called during the serialization process on each
*  EditNode.liveObject.
*  
*  You can implement updateEditNode on your liveObject and make 
*  updates to EditNode.defaults to save state "lazily" - just as serialization is
*  occurring - instead of updating editNode.defaults as the end user makes
*  changes. This can be useful if constantly calculating changes to
*  editNode.defaults would slow down interactivity.
*  
*  Note: best practice is to use EditContext.setNodeProperties and 
*  EditContext.removeNodeProperties to change properties, rather than directly
*  modifying EditNode.defaults.
* 
* 
* @param {EditContext} the EditContext
* @param {EditNode} the EditNode
*/
updateEditNode(editContext:EditContext, editNode:EditNode): void;

/**
*  Enable or disable edit mode for this component. Components in editMode must be
*  associated with an EditNode within an EditContext.
*  
*  Components with editMode enabled support certain editing interactions which
*  vary depending on the componentType and settings on the 
*  Canvas.editProxy.
*  
*  To disable edit mode just pass editingOn as false. The other parameters are
*  not needed.
*  
*  To enable edit mode on this component all three parameters are required. The
*  editNode is the edit node for this component as it exists within the
*  editContext.
*  
*  An alternative method, EditContext.enableEditing, can be used when
*  only an editContext and editNode are available.
*  
*  Placing a component into editMode causes the component's
*  Canvas.editProxy to be created.
* 
* 
* @param {boolean} true to enable editMode; false to disable
* @param {EditContext=} the EditContext
* @param {EditNode=} the EditNode
*/
setEditMode(editingOn:boolean, editContext?:EditContext, editNode?:EditNode): void;

/**
*  Set a specific Canvas.ariaState for this component.
*  
* 
* @param {string} aria state to update
* @param {string | boolean | Integer | Float} value for the aria state
*/
setAriaState(stateName:string, stateValue:string | boolean | Integer | Float): void;

/**
*  Gets the ARIA state mappings to apply to this component's
*  handle. See accessibility for more information on WAI-ARIA support in
*  SmartClient.
*  
*  The returned object consists of a mapping of aria attribute names to values to write
*  into the component HTML. For example, the following mapping:
*  
*  { haspopup:true, label:"Settings submenu"}
*  
*  Would result in ARIA html attributes like
*  aria-haspopup="true" aria-label="Settings submenu".
*  
*  The returned state mappings include Canvas.getAriaStateDefaults 
*  combined with any Canvas.ariaState.
*  
*  Note that if a property is explicitly specified it will take precedence over
*  the dynamically calculated default value. For example a Canvas.disabled
*  canvas will include aria-disabled in its markup by default, but 
*  a developer may change this behavior by explicitly setting ariaState to an object
*  with the property disabled explicitly set to false or null.
*  
*  This method will be invoked during on Canvas.draw and Canvas.redraw 
*  and the resulting state attributes will be applied to the component handle.
* 
* 
*/
getAriaState(): object;

/**
*  Retrieves dynamically calculated default ARIA state properties for this canvas. 
*  These will be combined with Canvas.ariaState
*  and applied to the widget handle as described in Canvas.getAriaState.
*  
*  For Canvas.disabled, this method returns an object
*  with disabled set to true. May be overridden to apply additional 
*  defaults in canvas subclasses.
*  
* 
*/
getAriaStateDefaults(): object;

/**
*  Animate a reposition of this canvas from its current position to the specified position
* 
* @param {Integer} new left position (or null for unchanged)
* @param {Integer} new top position (or null for unchanged)
* @param {AnimationCallback=} When the move completes this callback will be            fired. Single 'earlyFinish' parameter will be passed if the            animation was cut short, for example by a call to the non-animated            APIs Canvas.moveTo or Canvas.moveBy.
* @param {Integer=} Duration in ms of the animated move
* @param {AnimationAcceleration=} Optional acceleration effect to bias the ratios
*/
animateMove(left:Integer, top:Integer, callback?:(earlyFinish:boolean) => void, duration?:Integer, acceleration?:AnimationAcceleration): void;

/**
*  Animate a resize of this canvas from its current size to the specified size
*  
*  Note that Canvas.animateResizeLayoutMode allows you to control whether child layout is
*  rerun during every step of the animation, or just at the end, since the former may incur
*  significant overhead depending on the widget hierarchy.
* 
* 
* @param {Integer} new width (or null for unchanged)
* @param {Integer} new height (or null for unchanged)
* @param {AnimationCallback=} When the resize completes this callback will be            fired. Single 'earlyFinish' parameter will be passed if the            animation was cut short, for example by a call to the non-animated            APIs Canvas.resizeTo or Canvas.resizeBy.
* @param {Integer=} Duration in ms of the animated resize
* @param {AnimationAcceleration=} Optional acceleration effect to apply to the resize
*/
animateResize(width:Integer, height:Integer, callback?:(earlyFinish:boolean) => void, duration?:Integer, acceleration?:AnimationAcceleration): void;

/**
*  Animate a reposition / resize of this canvas from its current size and position.
* 
* @param {Integer} new left position (or null for unchanged)
* @param {Integer} new top position (or null for unchanged)
* @param {Integer} new width (or null for unchanged)
* @param {Integer} new height (or null for unchanged)
* @param {AnimationCallback=} When the setRect completes this callback will be            fired. Single 'earlyFinish' parameter will be passed if the            animation was cut short, for example by a call to the non-animated            API Canvas.setRect.
* @param {Integer=} Duration in ms of the animated setRect
* @param {AnimationAcceleration=} Optional acceleration effect to apply to the animation
*/
animateRect(left:Integer, top:Integer, width:Integer, height:Integer, callback?:(earlyFinish:boolean) => void, duration?:Integer, acceleration?:AnimationAcceleration): void;

/**
*  Animate a change in opacity from the widget's current opacity to the specified opacity.
* 
* @param {Integer} desired final opacity
* @param {AnimationCallback=} When the fade completes this callback will be            fired. Single 'earlyFinish' parameter will be passed if the            animation was cut short, for example by a call to the non-animated            API Canvas.setOpacity.
* @param {Integer=} Duration in ms of the animated fade
* @param {AnimationAcceleration=} Optional animation acceleration to bias the ratios
*/
animateFade(opacity:Integer, callback?:(earlyFinish:boolean) => void, duration?:Integer, acceleration?:AnimationAcceleration): void;

/**
*  Animate a scroll from the current scroll position to the specified position.
* 
* @param {Integer} desired final left scroll position
* @param {Integer} desired final top scroll position
* @param {AnimationCallback=} When the scroll completes this callback will be            fired. Single 'earlyFinish' parameter will be passed if the            animation was cut short, for example by a call to the non-animated            APIs Canvas.scrollTo or Canvas.scrollBy.
* @param {Integer=} Duration in ms of the animated scroll
* @param {AnimationAcceleration=} Optional acceleration to bias the animation ratios
*/
animateScroll(scrollLeft:Integer, scrollTop:Integer, callback?:(earlyFinish:boolean) => void, duration?:Integer, acceleration?:AnimationAcceleration): void;

/**
*  Show a canvas by growing it vertically to its fully drawn height over a period of time.
*  This method will not fire if the widget is already drawn and visible, or has overflow
*  other than "visible" or "hidden".
* 
* @param {AnimateShowEffectId | AnimateShowEffect=} Animation effect to use   when revealing the widget. If ommitted, default behavior can be configured via    Canvas.animateShowEffect
* @param {AnimationCallback=} When the show completes this callback will be            fired. Single 'earlyFinish' parameter will be passed if the            animation was cut short, for example by a call to the non-animated            API Canvas.show.
* @param {Integer=} Duration in ms of the animated show. If unset, duration will be  picked up from Canvas.animateShowTime
* @param {AnimationAcceleration=} Optional acceleration effect function to  bias the animation ratios. If unset, acceleration will be picked up from  Canvas.animateShowAcceleration
*/
animateShow(effect?:AnimateShowEffectId | AnimateShowEffect, callback?:(earlyFinish:boolean) => void, duration?:Integer, acceleration?:AnimationAcceleration): void;

/**
*  Hide a canvas by shrinking it vertically to zero height over a period of time.
*  This method will not fire if the widget is already drawn and visible, or has overflow
*  other than "visible" or "hidden".
* 
* @param {AnimateShowEffectId | AnimateShowEffect=} How should the content of the window be hidden during the hide? If ommitted, default behavior can be configured via  Canvas.animateHideEffect
* @param {AnimationCallback=} When the hide completes this callback will be            fired. Single 'earlyFinish' parameter will be passed if the            animation was cut short, for example by a call to the non-animated            API Canvas.hide.
* @param {Integer=} Duration in ms of the animated hide. If unset, duration will be  picked up from Canvas.animateHideTime
* @param {AnimationAcceleration=} Optional acceleration effect function to bias  the animation ratios. If unset, acceleration will be picked up from  Canvas.animateShowTime
*/
animateHide(effect?:AnimateShowEffectId | AnimateShowEffect, callback?:(earlyFinish:boolean) => void, duration?:Integer, acceleration?:AnimationAcceleration): void;

/**
*  When this Canvas is included as a member in a Layout, dynamically updates whether a 
*  resizeBar should be shown after this member in the layout, to allow it to be resized.
*  
*  Whether a resizeBar is actually shown also depends on the 
*  Layout.defaultResizeBars attribute of the layout, and whether this
*  Canvas is the last layout member.
* 
* @param {boolean} setting for this.showResizeBar
*/
setShowResizeBar(show:boolean): void;



}
/**
* Creates a new Canvas
* 
* @param typeCheckedProperties {Partial<Canvas>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Canvas} The new instance
*/
class Canvas{
static create(typeCheckedProperties? : Partial<Canvas>, uncheckedProperties?: Object): Canvas;
/**
* A declared value of the enum type s 
*  Overflow and
*  Cursor.
* @type {Constant}
* @default "auto"
*/
static AUTO?: Constant;

/**
* A declared value of the enum type 
*  Positioning.
* @type {Constant}
* @default "absolute"
*/
static ABSOLUTE?: Constant;

/**
* A declared value of the enum type 
*  Positioning.
* @type {Constant}
* @default "relative"
*/
static RELATIVE?: Constant;

/**
* A declared value of the enum type 
*  Visibility.
* @type {Constant}
* @default "inherit"
*/
static INHERIT?: Constant;

/**
* A declared value of the enum type s 
*  Overflow and
*  Visibility.
* @type {Constant}
* @default "visible"
*/
static VISIBLE?: Constant;

/**
* A declared value of the enum type s 
*  Overflow and
*  Visibility.
* @type {Constant}
* @default "hidden"
*/
static HIDDEN?: Constant;

/**
* A declared value of the enum type 
*  Overflow.
* @type {Constant}
* @default "scroll"
*/
static SCROLL?: Constant;

/**
* A declared value of the enum type 
*  Overflow.
* @type {Constant}
* @default "clip-h"
*/
static CLIP_H?: Constant;

/**
* A declared value of the enum type 
*  Overflow.
* @type {Constant}
* @default "clip-v"
*/
static CLIP_V?: Constant;

/**
* A declared value of the enum type s 
*  ImageStyle,
*  VerticalAlignment and
*  Alignment.
* @type {Constant}
* @default "center"
*/
static CENTER?: Constant;

/**
* A declared value of the enum type s 
*  Side,
*  Alignment and
*  Direction.
* @type {Constant}
* @default "left"
*/
static LEFT?: Constant;

/**
* A declared value of the enum type s 
*  Side,
*  Alignment and
*  Direction.
* @type {Constant}
* @default "right"
*/
static RIGHT?: Constant;

/**
* A declared value of the enum type s 
*  VerticalAlignment and
*  Side.
* @type {Constant}
* @default "top"
*/
static TOP?: Constant;

/**
* A declared value of the enum type s 
*  VerticalAlignment and
*  Side.
* @type {Constant}
* @default "bottom"
*/
static BOTTOM?: Constant;

/**
* A declared value of the enum type 
*  Direction.
* @type {Constant}
* @default "up"
*/
static UP?: Constant;

/**
* A declared value of the enum type 
*  Direction.
* @type {Constant}
* @default "down"
*/
static DOWN?: Constant;

/**
* A declared value of the enum type 
*  Cursor.
* @type {Constant}
* @default "default"
*/
static DEFAULT?: Constant;

/**
* A declared value of the enum type 
*  Cursor.
* @type {Constant}
* @default "wait"
*/
static WAIT?: Constant;

/**
* A declared value of the enum type 
*  Cursor.
* @type {Constant}
* @default "hand"
*/
static HAND?: Constant;

/**
* A declared value of the enum type 
*  Cursor.
* @type {Constant}
* @default "pointer"
*/
static POINTER?: Constant;

/**
* A declared value of the enum type 
*  Cursor.
* @type {Constant}
* @default "help"
*/
static HELP?: Constant;

/**
* A declared value of the enum type 
*  Cursor.
* @type {Constant}
* @default "text"
*/
static TEXT?: Constant;

/**
* A declared value of the enum type 
*  ImageStyle.
* @type {Constant}
* @default "tile"
*/
static TILE?: Constant;

/**
* A declared value of the enum type 
*  ImageStyle.
* @type {Constant}
* @default "stretch"
*/
static STRETCH?: Constant;

/**
* A declared value of the enum type 
*  ImageStyle.
* @type {Constant}
* @default "normal"
*/
static NORMAL?: Constant;

/**
* A declared value of the enum type 
*  BackgroundRepeat.
* @type {Constant}
* @default "repeat"
*/
static REPEAT?: Constant;

/**
* A declared value of the enum type 
*  BackgroundRepeat.
* @type {Constant}
* @default "no-repeat"
*/
static NO_REPEAT?: Constant;

/**
* A declared value of the enum type 
*  BackgroundRepeat.
* @type {Constant}
* @default "repeat-x"
*/
static REPEAT_X?: Constant;

/**
* A declared value of the enum type 
*  BackgroundRepeat.
* @type {Constant}
* @default "repeat-y"
*/
static REPEAT_Y?: Constant;

/**
* Specifies the lower limit for automatically assigned tab indices for focusable canvii.
* @type {Integer}
* @default "1000"
*/
static TAB_INDEX_FLOOR?: Integer;

/**
* If set, the AlphaImageLoader IE filter will never be used.  Does not remove
*  AlphaImageLoader usage in already-drawn components.
*  
*  See IEFilters for background.
* @type {boolean}
* @default null
*/
static neverUsePNGWorkaround?: boolean;

/**
* Disables automatic use of filters in IE by default. Filters will only be used if
*  Canvas.useOpacityFilter is explicitly set to true on a component.
*  
*  Does not remove filters on already drawn components, or which are applied via CSS.
*  
*  See IEFilters for background.
* @type {boolean}
* @default null
*/
static neverUseFilters?: boolean;

/**
* If enabled, uses a moderately expensive workaround to allow the use of IE filters in CSS
*  to produce gradient effects for buttons, grid rows, and other elements, without the use
*  of image backgrounds.
*  
*  See IEFilters for background.
* @type {boolean}
* @default true
*/
static allowExternalFilters?: boolean;

/**
* Image URL to be displayed while data is being loaded (if enabled for the widget
*  waiting for data). Must be square; Canvas.loadingImageSize specifies the width
*  and height.
* @type {SCImgURL}
* @default "[SKINIMG]loadingSmall.gif"
*/
static loadingImageSrc?: SCImgURL;

/**
* Specifies the width and height of Canvas.loadingImageSrc.
* @type {Integer}
* @default "16"
*/
static loadingImageSize?: Integer;

/**
* A fixed number of pixels at the top of the page in which components will not be placed.
*  This is overridable per-instance via the Canvas.leavePageSpace attribute.
*  Essentially, the effect is that all top-level components are shifted down this number of
*  pixels, and the page height is treated as this number of pixels less than the
*  real page height.
*  
*  This attribute can be useful on certain mobile devices, when components should not be
*  placed in a top portion of the screen. For example, on iOS devices in certain configurations,
*  this can be set to 20 to avoid placing any component into the status bar area. Or, if
*  using iOS 7.1's 'minimal-ui' viewport parameter, this can be set to 20 to avoid placing
*  any component into the top 20px area of the screen, which if tapped on iPhone in landscape,
*  causes Mobile Safari's address bar and tab bar to be shown.
*  
*  This setting can be changed at runtime by calling Canvas.setDefaultPageSpace.
*  
*  Note: As documented by the mobileDevelopment
*  page, when the SmartClient application is running in Mobile Safari on iPhone running iOS 7.1 or
*  later, and neither the isc_useDefaultViewport nor the isc_useMinimalUI
*  global is set to false when the framework is loaded, then the framework
*  will automatically set the defaultPageSpace to 0 in portrait orientation,
*  and to 20 in landscape orientation.
* @type {Integer}
* @default "0"
*/
static defaultPageSpace?: Integer;

/**
* A declared value of the enum type 
*  DragDataAction.
* @type {Constant}
* @default "copy"
*/
static COPY?: Constant;

/**
* A declared value of the enum type s 
*  DragDataAction and
*  Cursor.
* @type {Constant}
* @default "move"
*/
static MOVE?: Constant;





/**
*  Changes the system-wide Canvas.neverUseFilters setting.
* 
* @param {boolean} new setting
*/
static setNeverUseFilters(neverUseFilters:boolean): void;

/**
*  Changes the system-wide Canvas.allowExternalFilters setting.
* 
* @param {boolean} new setting
*/
static setAllowExternalFilters(allExternalFilters:boolean): void;

/**
*  Changes the global Canvas.defaultPageSpace.
* 
* 
* @param {Integer} the new value for defaultPageSpace.
*/
static setDefaultPageSpace(newDefaultPageSpace:Integer): void;

/**
*  Check if an event is within an "edge" of this canvas.
* 
* 
* @param {Array<Partial<EdgeName>>=} Array of legal edges. Default is all the edges that                    allow resizing (see Canvas.resizeFrom)
*/
static getEventEdge(edgeMask?:Array<Partial<EdgeName>>): EdgeName;

/**
*  Retrieve a Canvas by it's global Canvas.ID.
* 
* @param {string} global ID of the Canvas
*/
static getById(ID:string): Canvas;

/**
*  Resizes fonts as if calling Canvas.resizeFonts, but takes a final target size instead
*  of a delta representing the change from a current size. Note that all the same limitations
*  apply.
* 
* @param {Integer} the final size, expressed as a differential from the default
*/
static resizeFontsTo(targetDelta:Integer): void;

/**
*  Modify the size of fonts for some or all stylesheets defined in the page.
*  
*  This method can be used to dynamically increase or decrease font size for all of the fonts
*  in your application, or just fonts defined in your chosen SmartClient skin (the latter can
*  be achieved by passing styleSheets as "skin_styles.css" - the default name for
*  the CSS file used in each SmartClient skin).
*  
*  resizeFonts() must be called after the skin has been loaded, and before any
*  components have been created. Calling resizeFonts() at a later time is not
*  supported (you will notice that font sizes still increase, however, this approach is not
*  supported).
*  
*  Some browsers will disallow access or modification of styleSheets if they are loaded from a
*  domain that is different from the loading page. In this case resizeFonts()
*  cannot be used.
*  
*  This method has a small performance penalty which depends on the browser, number of
*  stylesheets being modified, and age of your machine. With modern browsers on modern
*  machines resizing just skin fonts, the impact is basically negligible (&lt;5ms).
*  
*  Certain controls such as icons are resized when fonts are resized (see 
*  Canvas.setAutoResizeIcons and Canvas.setAutoResizeAutoChildAttributes)
*  so you might want to set resizeRelatedControls to false where
*  you are just trying to make fonts in a dynamically loaded stylesheet match previously
*  loaded fonts, but controls such as icons should not be resized upwards again.
* 
* 
* @param {Integer} size to change fonts by. Can be negative to shrink fonts
* @param {string=} optional regular expression pattern for matching stylesheets
* @param {boolean=} resize icons and autoChild attributes? Set to false    to suppress default behavior.
*/
static resizeFonts(sizeChange:Integer, styleSheets?:string, resizeRelatedControls?:boolean): void;

/**
*  Modify the amount of padding for some CSS styles defined for the page. Only CSS styles
*  registered by Canvas.registerFontScaledPaddingStyles are modified.
*  
*  resizePadding() must be called after the skin has been loaded, and before any
*  components have been created. Calling resizePadding() at a later time is not
*  supported (you will notice that padding is modified, however, this approach is not supported).
*  
*  This method has similar browser security limitations as Canvas.resizeFonts.
*  
*  The intent is that the same font size change be passed to this method as is passed to
*  Canvas.resizeFonts, so that the targetSizeChange in the call to
*  Canvas.registerFontScaledPaddingStyles represents the right font size for the unadjusted
*  styles being registered.
* 
* @param {Integer} size change to apply to the padding of registered styles, so that               they aren't changed at all at the size change passed to                Canvas.registerFontScaledPaddingStyles, and the padding is               reduced to baseline style levels at a zero size change.
* @param {string=} optional regular expression pattern for matching stylesheets
*/
static resizePadding(fontSizeChange:Integer, styleSheets?:string): void;

/**
*  Registers one or more CSS classes to have their padding adjusted (independently on all edges)
*  according to the Canvas.resizePadding applied to the page. Each class
*  to be registered is provided along with a corresponding baseline class, and a single
*  targetSizeChange is specified for all the classes. The padding in each
*  registered class is adjusted downward towards the baseline as the padding size change
*  approaches 0 (no resizing), and upward as it increases, so that it exactly equals the
*  declared style's padding at a padding size change of targetSizeChange.
*  
*  Note that each call to this method replaces the registration of the previous call (if any),
*  and will have no effect until Canvas.resizePadding is called.
*  
*  For example:
*   isc.Canvas.registerFontScaledPaddingStyles(
*     [    "tabButtonTop",     "tabButtonBottom"], 
*     ["iconOnlyTabButtonTop", "iconOnlyTabButtonBottom"],
*     3
*   );
*  
*  In this case, the CSS style "tabButtonTop" will have its padding adjusted downward to the
*  padding from the baseline CSS style "iconOnlyTabButtonTop" style at a sizeChange
*  of 0, and be left unchanged at a sizeChange of 3.
* 
* @param {Array<Partial<CSSStyleName>>} styles whose padding should be adjusted
* @param {Array<Partial<CSSStyleName>>} corresponding baseline reference styles
* @param {Integer} sizeChange at which scaledStyles are unchanged
*/
static registerFontScaledPaddingStyles(scaledStyles:Array<Partial<CSSStyleName>>, baselineStyles:Array<Partial<CSSStyleName>>, targetSizeChange:Integer): void;

/**
*  Resizes controls as if calling Canvas.resizeControls, but takes a final target size
*  instead of a delta representing the change from a current size. Note that all the same
*  limitations apply.
* 
* @param {Integer} the final size, expressed as a differential from the default
*/
static resizeControlsTo(targetDelta:Integer): void;

/**
*  Change the basic size of UI components in the current skin by "delta" pixels. Must be
*  called after the skin has been loaded, but before any components are created.
*  
*  In general, this method changes the height of various controls, except for certain 
*  controls that appear in alternate orientations (such as resizeBars and tabs), in which case 
*  thickness properties (resizeBarThickness, tabBarThickness) are adjusted.
*  
*  The height of a text input control implies the height of most other controls:
*  
*  
* - all other FormItems (eg selects) need to be the same height or mixed controls will look
*    odd. This includes Buttons
*  
* - anything that potentially contains a FormItem needs to be as tall or slightly taller:
*    this includes grid row (inline editing), Window.headerControls, 
*    TabBar and SectionHeaders
*  
*  
*  Because of this necessary uniformity, just specifying a single pixel value is enough for the
*  framework to resize all core controls, with several caveats:
*  
*  
* - skins that make extensive use of images (eg TreeFrog) will stretch those images, which
*    may result in ugly artifacts in some combinations of operating system and browser, for
*    which no workaround is possible. For this reason, resizeControls() is
*    only officially supported in skinning, which includes any of the
*    skins in our Flat series (Tahoe, Stratus, Obsidian and Twilight) or 
*    Enterprise series (Enterprise/Blue and Graphite), or a custom skin based 
*    on one of those - including any skin generated by the Skin Editor.
*  
* - even in Enterprise-series skins, TreeGrid.showConnectors 
*    vertically stretch, becoming obviously blurry and misshapen with an increase of 4-5px.
*    To avoid this, replace the tree connector media (see TreeGrid.connectorImage).
*  
* - FormItemIcons are not resized by default, because stretched icons generally look
*    worse than non-scaled icons that are a bit smaller than the input field
*  
* - images that use skinning will not be stretched because the sizes
*    for these controls are embedded in CSS. In most cases, this is desirable; for example,
*    the downward chevron shape used for SelectItem controls doesn't stretch, and looks
*    better that way.
*  
* 
* 
* @param {Integer} number of pixels to increase or decrease from current size
*/
static resizeControls(delta:Integer): void;

/**
*  Should icons be automatically resized with Canvas.resizeControls and
*  Canvas.resizeFonts?
*  
*  If true, icon attributes registered for resize with policy "controls" will
*  be resized when resizeControls() runs, and icons registered
*  with policy "fonts" will resize when resizeFonts() runs.
*  
*  To resize icons with other policies, developers should call
*  Canvas.resizeIcons directly
* 
* 
* 
* @param {boolean} true if icons should be auto-resized
*/
static setAutoResizeIcons(autoResizeIcons:boolean): void;

/**
*  Should registered autoChild attributes be automatically resized with 
*  Canvas.resizeControls and
*  Canvas.resizeFonts?
*  
*  If true, attributes registered for resize with policy "controls" will
*  be resized when resizeControls() runs, and icons registered
*  with policy "fonts" will resize when resizeFonts() runs.
*  
*  To resize autoChild attributes with other policies, developers should call
*  Canvas.resizeAutoChildAttributes directly
* 
* 
* 
* @param {boolean} true if attributes should be auto-resized
*/
static setAutoResizeAutoChildAttributes(autoResize:boolean): void;

/**
*  Change the basic size of icons in the current skin by "delta" pixels. This method may
*  be invoked automatically from Canvas.resizeControls.
*  
*  Must be called after the skin has been loaded, but before any components are created.
* 
* 
* @param {string} Which set of icons should be resized? This should correspond to the  iconSizingPolicy argument applied when   Canvas.registerIconSizingAttributes.
* @param {Integer} number of pixels to increase or decrease from current size
*/
static resizeIcons(policy:string, delta:Integer): void;

/**
*  Register numeric or measure type properties of some class(es) as
*  "icon sizing attributes". These are
*  properties that should be adjusted when Canvas.resizeIcons is called. 
*  Typically these will consist of icon width and height attributes on some SmartClient 
*  class, such as ListGrid.checkboxFieldImageHeight and
*  ListGrid.checkboxFieldImageWidth.
*  
*  The policy parameter allows icons to be grouped together into sets so particular types of
*  UI element can be resized in a targeted manner. For example, the width and height of
*  an icon which will appear aligned with a line of text would typically want to be resized
*  at the same time as font sizes are adjusted, whereas an icon that corresponds to a 
*  block of control UI (such as a picker icon that sits outside of an associated FormItem), 
*  should be resized when that control element is resized.
*  
*  A policy can be any string. To modify the sizes of attributes registered under some 
*  policy, that same policy string should be passed to the Canvas.resizeIcons method.
*  
*  The attributes parameter consists of a JavaScript object where each key specifies 
*  the name of the class on which the attributes, with its value set to an array, indicating
*  the attribute(s) to register within that class.
*  
*  Icon sizing attributes can be specified individually or in pairs. When a single attribute
*  passed in, a call to Canvas.resizeIcons will modify that attribute by the
*  delta parameter. If a pair of attributes is passed in, this is assumed to
*  be a height,width pair. When Canvas.resizeIcons is called in this case, 
*  both attributes will be modified such that they maintain the same scale. In other words,
*  the first registered attribute (typically the height) will be adjusted by the specified
*  delta (a simple numeric adjustment). The second attribute will be adjusted by 
*  a numeric delta calculated to have the same ratio to the original width as the 
*  provided delta had to the original height. This allows icons to be resized without
*  becoming distorted.
*  
*  For example, the following code would register FormItem attributes "valueIconSize",
*  "valueIconWidth" and "valueIconHeight" for resizing as part of the "fonts" policy,
*  such that both "valueIconSize" and "valueIconHeight" would be changed by the value passed
*  to resizeIcons(..), and "valueIconWidth" would be changed such that the
*  width/height ratio was retained.
*  
*  isc.Canvas.registerIconSizingAttributes(
*   "fonts",
*   {
*    FormItem:[
*      "valueIconSize",
*      ["valueIconHeight", "valueIconWidth"]
*    ]
*   }
*  );
*  
*  
*  The Canvas.setAutoResizeIcons feature will cause icon attributes
*  registered with policy "controls" to be resized when Canvas.resizeControls
*  is run and attributes registered as "fonts" to be resized when
*  Canvas.resizeFonts is run.
* 
* 
* @param {string} Icon sizing policy for this set of attributes
* @param {object} The attribute(s) to register for resizing.
*/
static registerIconSizingAttributes(policy:string, attributes:object): void;

/**
*  Register numeric or measure type properties of on autoChild properties blocks for
*  some class(es).
*  These are
*  properties that should be adjusted when Canvas.resizeAutoChildAttributes is called. 
*  Typically these will consist of width and height attributes on some autoChild defaults block
*  for some SmartClient class, such as Window.closeButtonDefaults
*  (to modify width and height of Window.closeButton).
*  
*  As with Canvas.registerIconSizingAttributes, the policy parameter allows
*  attributes to be grouped together into sets so particular types of
*  UI element can be resized in a targeted manner. For example, the width and height of
*  an icon which will appear aligned with a line of text would typically want to be resized
*  at the same time as font sizes are adjusted, whereas an icon that corresponds to a 
*  block of control UI (such as a picker icon that sits outside of an associated FormItem), 
*  should be resized when that control element is resized.
*  
*  A policy can be any string. To modify the sizes of attributes registered under some 
*  policy, that same policy string should be passed to the Canvas.resizeAutoChildAttributes 
*  method.
*  
*  The className should be the name of the class on which the auto child properties block 
*  exists.
*  
*  The attributes parameter consists of a JavaScript object where each key specifies 
*  the name of the autoChild properties block to modify, with its value set to an array, 
*  indicating the attribute(s) to register.
*  
*  Sizing attributes can be specified individually or in pairs. When a single attribute
*  passed in, a call to Canvas.resizeAutoChildAttributes will modify that attribute by the
*  delta parameter. If a pair of attributes is passed in, this is assumed to
*  be a height,width pair. When Canvas.resizeAutoChildAttributes is called in this case, 
*  both attributes will be modified such that they maintain the same scale. In other words,
*  the first registered attribute (typically the height) will be adjusted by the specified
*  delta (a simple numeric adjustment). The second attribute will be adjusted by 
*  a numeric delta calculated to have the same ratio to the original width as the 
*  provided delta had to the original height. This allows icons to be resized without
*  becoming distorted.
*  
*  For example, the following code would register the attributes "width" and "height" properties
*  on the "closeButtonDefaults" properties block, within the Window class - thus
*  customizing the default size of the Window.closeButton autoChild.
*  It registers them as part of the "controls" policy.
*  
*  isc.Canvas.registerAutoChildSizingAttributes(
*   "fonts",
*   "Window",
*   {
*    closeButtonDefaults:[
*      ["height", "width"]
*    ]
*   }
*  );
*  
*  
*  The Canvas.setAutoResizeAutoChildAttributes
*  feature will cause registered autoChild attributes to resize automatically.
*  Those registered with policy "controls" to be resized when Canvas.resizeControls
*  is run and attributes registered as "fonts" to be resized when
*  Canvas.resizeFonts is run.
* 
* 
* @param {string} Sizing policy for this set of attributes
* @param {string} Name of the class containing the autoChild configuration block to edit
* @param {object} The attribute(s) to register for resizing.
*/
static registerAutoChildSizingAttributes(policy:string, className:string, attributes:object): void;

/**
*  Change the value of attributes registered via Canvas.registerAutoChildSizingAttributes
*  by some number of pixels. This method may
*  be invoked automatically from Canvas.resizeControls or Canvas.resizeFonts
*  
*  Must be called after the skin has been loaded, but before any components are created.
* 
* 
* @param {string} Which set of attributes should be resized? This should correspond to the  sizing policy argument applied when   Canvas.registerAutoChildSizingAttributes.
* @param {Integer} number of pixels to increase or decrease from current size
*/
static resizeAutoChildAttributes(policy:string, delta:Integer): void;

/**
*  Whether to use the browser's native scrollbars or SmartClient-based scrollbars by default
*  for all canvases.
*  
*  This method changes the default value of Canvas.showCustomScrollbars.
* 
* @param {boolean} whether to show custom (SmartClient-based) scrollbars  rather than css-scrollbars by default.
*/
static setShowCustomScrollbars(showCustomScrollbars:boolean): void;

/**
*  Returns print-formatted HTML for the specified list of components.
* 
* 
* @param {Array<Partial<Canvas>>} Components to get the print HTML for. Strings of raw HTML may also be included in this array, and will be integrated into the final HTML at the appropriate  point.
* @param {PrintProperties} properties affecting print output
* @param {Callback=} Callback to fire when the method completes. The generated print HTML will be passed in as the first parameter HTML.
* @param {HTMLString=} Optional HTML separator to render between each component's printable HTML
*/
static getPrintHTML(components:Array<Partial<Canvas>>, printProperties:PrintProperties, callback?:Callback, separator?:HTMLString): HTMLString;

/**
*  Generates the HTML for an image. Also available at the 
*  Canvas.imgHTML.
* 
* 
* @param {SCImgURL} URL local to the skin or application directory.   NOTE: instead of passing several parameters, you can pass an object as the 'src'    parameter with properties for all the various function parameters with, eg:    canvas.imgHTML( {src:"foo", width:10, height:10} );
* @param {number=} width of the image
* @param {number=} height of the image
* @param {string=} name for the image
* @param {string=} additional attributes to write in the tag
* @param {string=} image-specific image directory
*/
static imgHTML(src:SCImgURL, width?:number, height?:number, name?:string, extraStuff?:string, imgDir?:string): string;

/**
*      Given a string and a character, hilite the first occurrence of the character in the
*      string (if it occurs), preferring uppercase to lowercase.
* 
* 
* @param {string} String to return with hilited character
* @param {string} Character to hilite
* @param {string=} Prefix to apply to hilighted character - defaults to                   "&lt;span style='text-decoration:underline;'&gt;"
* @param {string=} Suffix to apply to hilited character - defaults to                     "&lt;/span&gt;"
*/
static hiliteCharacter(string:string, character:string, hilitePrefix?:string, hiliteSuffix?:string): string;

/**
*  Return the position for snapper to be placed in order to "snap to" an edge or
*  corner of target, in the same sense as Canvas.snapTo.
*  
*  Default for snapEdge is the opposite edge or corner from
*  snapTo. For example, snapTo of "T" (top) means snapEdge
*  will default to "B" (bottom), so the returned coordinates would place snapper
*  centered along the top edge of target. A snapTo of "TL" (top left)
*  means snapEdge will default to "BR" (bottom right), so the returned coordinates
*  would place the bottom right of snapper at the top left corner of
*  target.
*  
*  target can be passed as either a Canvas or 4-element Array giving the top, left,
*  width and height of the target. 
*  snapper can be passed as either a Canvas or a
*  two-element Array of the width and height of the rectangle to be placed.
* 
* 
* @param {Canvas | Array<Partial<Integer>>} canvas to snap to
* @param {string} edge against which to snap
* @param {Canvas | Array<Partial<Integer>>} canvas being snapped
* @param {string=} optional edge to snapTo. Default is the opposite edge or corner from snapTo
*/
static getSnapPosition(target:Canvas | Array<Partial<Integer>>, snapTo:string, snapper:Canvas | Array<Partial<Integer>>, snapEdge?:string): Point;

/**
*  Generate printable HTML for the designated components and trigger the native print
*  dialog, without never showing the printable HTML to the user.
* 
* 
* @param {Array<Partial<Canvas>>} components to get the print HTML for. May also include    raw HTML strings which will be folded into the generated print output
* @param {PrintProperties=} object for customizing the print HTML output
*/
static printComponents(components:Array<Partial<Canvas>>, printProperties?:PrintProperties): void;

/**
*  Creates a printCanvas containing the full printHTML for a series of components, passing
*  it as an argument to the callback (if supplied) when it fires. Note that the generated
*  preview canvas will be drawn automatically by this method. Developers may also
*  explicitly create a PrintCanvas instance and populate it with HTML derived from the
*  Canvas.getPrintHTML for finer grained control over when the print canvas is
*  drawn.
* 
* @param {Array<Partial<Canvas>>} components to get the print HTML for. May also include  raw HTML strings which will be folded into the generated print output
* @param {PrintProperties=} PrintProperties object for customizing the print HTML output
* @param {Partial<Canvas>=} properties to apply to the generated printPreview Canvas.
* @param {PrintCanvasCallback=} callback to fire when the print preview canvas has been populated  with the printable HTML. The generated canvas will be passed to the callback as a single   printPreview parameter.
* @param {string=} optional string of HTML to render between each component
*/
static getPrintPreview(components:Array<Partial<Canvas>>, printProperties?:PrintProperties, previewProperties?:Partial<Canvas>, callback?:(printPreview:PrintCanvas) => void, separator?:string): void;

/**
*  Generate and show a PrintWindow containing a PrintCanvas
*  showing a printable view of the components passed in.
* 
* 
* @param {Array<Partial<Canvas>>} components to get the print HTML for. May also include raw HTML strings which will be folded into the generated print output
* @param {PrintProperties=} PrintProperties object for customizing the   print HTML output
* @param {Partial<PrintWindow>=} Properties to apply to the   generated print window.
* @param {Callback=} callback to fire when the print preview canvas has   been populated with the printable HTML. This callback takes 2 parameters:   printPreview - a pointer to the generated print canvas shown in the   body of the print window.   printWindow - a pointer to the generated print window and
* @param {string=} Optional HTML separator to render between each component's printable           HTML
*/
static showPrintPreview(components:Array<Partial<Canvas>>, printProperties?:PrintProperties, printWindowProperties?:Partial<PrintWindow>, callback?:Callback, separator?:string): void;




}

/**
* EditProxy that handles Canvas objects when editMode is enabled.
*/
interface CanvasEditProxy extends EditProxy {




}
/**
* Creates a new CanvasEditProxy
* 
* @param typeCheckedProperties {Partial<CanvasEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {CanvasEditProxy} The new instance
*/
class CanvasEditProxy{
static create(typeCheckedProperties? : Partial<CanvasEditProxy>, uncheckedProperties?: Object): CanvasEditProxy;







}

/**
* FormItem which renders a Canvas inline in a DynamicForm instance.
*  
*  CanvasItem is CanvasItem.shouldSaveValue:false by default, meaning that
*  no value from the CanvasItem will be present in DynamicForm.getValues and no value will be
*  saved when DynamicForm.saveData is called. This is appropriate if the Canvas does
*  not participate in editing a value of the form and is embedded in the form for layout or UI
*  purposes only (e.g. ButtonItem, SectionItem). Note that some built-in CanvasItem
*  types override the shouldSaveValue default to true (e.g. MultiComboBoxItem, RichTextItem).
*  
*  If you set FormItem.shouldSaveValue:true, a 
*  CanvasItem.showValue event will be raised to provide a value that your
*  item should display. Handle this event by calling methods on the Canvas you've created
*  to cause the value to be displayed.
*  
*  The CanvasItem.showValue event will be triggered in various situations where
*  the form receives data, including a call to DynamicForm.setValues,
*  DynamicForm.editRecord, or if DynamicForm.fetchData is called and a Record
*  is returned. Bear in mind that the showValue event can be called when the form
*  and your item have not yet been drawn; in this case, store the value for later display.
*  
*  To provide a value to the form, call CanvasItem.storeValue whenever the user changes
*  the value in your Canvas. Generally, if storeValue() is called then
*  CanvasItem.shouldSaveValue should be overridden to true. Note that
*  the form will not call getValue() in order to discover your item's value, so there is
*  no purpose in overriding this method; instead, call storeValue() to proactively inform the
*  form about changes to the value. This approach is necessary in order to enable change events.
*  
*  If you cannot easily detect changes to values in your Canvas, a workaround is to call
*  storeValue right before the form saves.
*/
interface CanvasItem extends FormItem {
/**
* Height of the Canvas. Can be either a number indicating a fixed height in pixels, a
*  percentage indicating a percentage of the overall form's height, or "*" indicating take
*  whatever remaining space is available. See the formLayout overview for details.
*  
*  Height may also be explicitly specified on the CanvasItem.canvas. In this
*  any canvasItem.height will be ignored in favor of the value applied 
*  to the canvas directly. In either case, percentage values will be resolved using
*  standard formItem sizing rules as described in formLayout
* @type {Integer | string}
* @default null
*/
height?: Integer | string;

/**
* Whether this CanvasItem is intended to hold multiple values.
*  
* @type {boolean}
* @default null
*/
multiple?: boolean;

/**
* Should this item's value be saved in the form's values and hence returned from
*  DynamicForm.getValues?
*  
*  Note that by default, shouldSaveValue is false for CanvasItems,
*  meaning that no value from the CanvasItem will be present in DynamicForm.getValues
*  and no value for the CanvasItem will be saved when DynamicForm.saveData is called.
*  See the CanvasItem class overview for a discussion of values handling in
*  CanvasItems.
* @type {boolean}
* @default false
*/
shouldSaveValue?: boolean;

/**
* The canvas that will be displayed inside this item. You can pass an instance you've 
*  already created, or its global ID as a String. You can also implement 
*  CanvasItem.createCanvas to dynamically create the canvas when the FormItem
*  is initialized.
*  
*  If canvas and createCanvas() are unspecified, the 
*  canvas for this item will be auto-created using the overrideable defaults:
*  CanvasItem.canvasProperties and CanvasItem.canvasConstructor
*  
*  
*  Note that subclasses of CanvasItem may use a different AutoChild name than
*  just "canvas". For example, SliderItem uses "slider", and in that case, you need
*  to use the specific APIs provided by the subclass.
*  
*  Note that Canvas.canvasItem will be set on the canvas to point back to this
*  item.
* @type {Canvas}
* @default null
*/
canvas?: Canvas;

/**
* If CanvasItem.canvas is not specified as a canvas instance at init
*  time, a canvas will be created instead. This property denotes the class of that widget
*  (Should be set to the name of a subclass of Canvas).
* @type {string}
* @default "Canvas"
*/
canvasConstructor?: string;

/**
* Default properties for the canvas if this.canvas is not already a canvas instance.
* @type {object}
* @default "{}"
*/
canvasDefaults?: object;

/**
* Properties to apply to this canvas on creation if this.canvas is not already a canvas 
*  instance.
* @type {object}
* @default "{}"
*/
canvasProperties?: object;

/**
* Should this item's CanvasItem.canvas be automatically destroyed when the item 
*  is destroyed? Form items are destroyed automatically when a call to
*  DynamicForm.setItems removes them from their parent form, or if their
*  parent form is destroyed. This property governs whether, when this occurs, 
*  the item's canvas should also be Canvas.destroy.
*  
*  This property has no effect for canvases automatically created via the "autoChild" pattern, 
*  using CanvasItem.canvasProperties, CanvasItem.canvasDefaults etc. CanvasItems which create their
*  canvas in this way will always destroy the canvas when the item is destroyed or on an
*  explicit CanvasItem.setCanvas call, regardless of this property's value.
*  
*  Setting this property to true
*  is typically appropriate for cases where a custom CanvasItem automatically creates
*  its canvas as part of its initialization flow, and the canvas will not be re-used outside
*  the item.
*  Note that once a canvas has been destroyed it can not be re-used elsewhere within
*  an application.
* @type {boolean}
* @default false
*/
autoDestroy?: boolean;

/**
* CanvasItems support specifying overflow for the Canvas directly on the item.
* @type {Overflow}
* @default null
*/
overflow?: Overflow;

/**
* Minimum valid height for this CanvasItem in pixels. Used in calculating the row heights
*  of the containing DynamicForm if the item has a flexible CanvasItem.height.
* @type {Integer}
* @default null
*/
minHeight?: Integer;

/**
* Maximum valid height for this CanvasItem in pixels. Used in calculating the row heights
*  of the containing DynamicForm if the item has a flexible CanvasItem.height.
* @type {Integer}
* @default null
*/
maxHeight?: Integer;

/**
* Flag to disable the criteria editing overrides described in
*  CanvasItem.getCriterion
*  whereby
*  if this item contains a DynamicForm as its canvas, it will be used to edit nested
*  AdvancedCriteria automatically.
*  
*  This flag is required for cases where a canvasItem contains a DynamicForm, but the form
*  is not set up to show inner field values of nested objects, and therefore should not
*  attempt to apply nested advanced criteria directly to the form.
* @type {boolean}
* @default true
*/
editCriteriaInInnerForm?: boolean;

/**
* If FormItem.prompt is specified for this item, should the prompt be applied to the
*  CanvasItem.canvas for this item?
* @type {boolean}
* @default true
*/
applyPromptToCanvas?: boolean;

/**
* This text is shown as a tooltip prompt when the cursor hovers over this item.
*  
*  When item is FormItem.setCanEdit a different hover can be shown with
*  FormItem.readOnlyHover. Or, when item is FormItem.disabled or
*  read-only with FormItem.readOnlyDisplay a different hover can be shown
*  with FormItem.disabledHover.
*  
*  Note that when the form is Canvas.disabled this prompt will not
*  be shown.
* @type {HTMLString}
* @default null
*/
prompt?: HTMLString;



/**
*  Store (and optionally show) a value for this form item. 
*  
*  This method will fire standard FormItem.change and
*  DynamicForm.itemChanged handlers, and store the value passed in such that
*  subsequent calls to FormItem.getValue or DynamicForm.getValue will
*  return the new value for this item.
*  
*  This method is intended to provide a way for custom formItems - most commonly 
*  CanvasItem - to provide a new interface to the user, allowing them
*  to manipulate the item's value, for example in an embedded CanvasItem.canvas,
*  or a pop-up dialog launched from an FormItemIcon, etc. Developers
*  should call this method when the user interacts with this custom 
*  interface in order to store the changed value.
*  
*  CanvasItem.shouldSaveValue for CanvasItems is
*  false by default. Custom CanvasItems will need to override shouldSaveValue
*  to true if the values stored via this API should be included in the form's
*  DynamicForm.getValues and saved with the form when
*  DynamicForm.saveData is called.
*  
*  If you cannot easily detect interactions that should change the value as the 
*  user performs them, a workaround is to call
*  storeValue right before the form saves.
*  
*  Note that this method is not designed for customizing a value which is already being
*  saved by a standard user interaction. For example you should not call this method
*  from a FormItem.change. Other APIs such as 
*  FormItem.transformInput exist for this.
*  
* 
* @param {any} value to save for this item
* @param {boolean=} Should the formItem be updated to display the new value?
*/
storeValue(value:any, showValue?:boolean): void;

/**
*  This method will place an entry for the CanvasItem.canvas under this item in the 
*  TabIndexManager. This ensures the user can tab into the canvas (and its
*  descendants) in the expected position within this item's DynamicForm.
*  
*  See also Canvas.updateChildTabPositions.
*  
* 
*/
updateCanvasTabPosition(): void;

/**
*  Setter to update the CanvasItem.canvas at runtime
* 
* @param {Canvas} New canvas to display.
*/
setCanvas(canvas:Canvas): void;

/**
*  Notification method called when the FormItem.canEdit setting is modified.
*  Developers may make use of this to toggle between an editable and a read-only appearance
*  of the CanvasItem.canvas.
*  
*  The default behavior is:
*  
*  
* - If canvas is a DynamicForm, the form's DynamicForm.canEdit
*  setting is set to canEdit.
*  
* - CanvasItem.shouldDisableCanvas is called to determine if the canvas should
*  be disabled.
*  
*  
*  Standard CanvasItem-based form items may customize the default behavior.
*  For example, a MultiComboBoxItem will hide its MultiComboBoxItem.comboForm
*  if the FormItem.readOnlyDisplay is
*  "readOnly" or "static"
*  
*  and also disable the buttons when made read-only.
* 
* @param {boolean} New canEdit value
*/
canEditChanged(canEdit:boolean): boolean;

/**
*  Notification method called when the FormItem.readOnlyDisplay setting
*  is modified. Developers may make use of this to toggle between an editable and a read-only
*  appearance of the CanvasItem.canvas.
*  
*  The default behavior is: when the canvas is a DynamicForm, the form's
*  DynamicForm.readOnlyDisplay setting is set to appearance.
*  
*  Standard CanvasItem-based form items may customize the default behavior.
* 
* @param {ReadOnlyDisplayAppearance} new readOnlyDisplay value
*/
readOnlyDisplayChanged(appearance:ReadOnlyDisplayAppearance): boolean;

/**
*  Method called to determine whether the CanvasItem.canvas should be
*  Canvas.setDisabled when this CanvasItem is disabled
*  or its CanvasItem.canEditChanged.
*  By default, if the canvas is a DynamicForm, then it is disabled if
*  and only if this CanvasItem is disabled; otherwise, the canvas
*  is disabled if and only if this CanvasItem is disabled or
*  FormItem.getCanEdit.
*  
*  This method may be overridden to customize the default return value.
* 
*/
shouldDisableCanvas(): boolean;

/**
*  Does this CanvasItem have keyboard focus.
*  
*  This method will return true if this item's canvas, or any of its descendents, has
*  keyboard focus
* 
*/
isFocused(): boolean;

/**
*  Overridden to return true if CanvasItem.canvas is a dynamicForm.
*  See CanvasItem.getCriterion
*  
*  for details of editing advanced criteria using nested
*  dynamicForms.
* 
*/
hasAdvancedCriteria(): boolean;

/**
*  AdvancedCriteria objects may be edited via nested dynamicForms as described in
*  CanvasItem.getCriterion
*  
*  This method has been overridden to return true if this item's canvas is a DynamicForm,
*  where the DynamicForm.operator matches the operator of the criterion passed in
*  and dynamicForm contains items where FormItem.canEditCriterion returns true
*  for each sub-criterion in the object passed in.
* 
* @param {Criterion} criteria to test
*/
canEditCriterion(criterion:Criterion): boolean;

/**
*  The standard formItem criteria editing APIs have been overridden in the canvasItem class
*  to simplify the editing of complex AdvancedCriteria objects using nested 
*  DynamicForms.
*  
*  The following pattern is supported without need for further modification:
*  A complex Advanced criteria object may have nested sub criteria using the "and"
*  or "or" operators. For example:
*  
*  { _constructor:"AdvancedCriteria",
*   operator:"and",
*   criteria:[
*    {fieldName:"field1", value:"value1", operator:"iContains"},
*    {operator:"or", criteria:[
*      {fieldName:"innerField1", value:"value1", operator:"equals"},
*      {fieldName:"innerField2", value:"value2", operator:"iContains"}
*     ]
*    }
*   ]
*  }
*  
*  To create a form capable of editing the above criteria without providing custom overrides
*  to FormItem.getCriterion et al, you would create a form with 2 items.
*  The 'field1' criterion could be edited by a simple form item such as a TextItem.
*  The nested criteria ('innerField1' and 'innerField2') could be edited by a canvasItem
*  whose canvas property was set to a DynamicForm showing items capable of editing the 2
*  inner criteria, and whose operator was specified as "or".
*  For example:
*  
*  
*  isc.DynamicForm.create({
*    items:[
*      {name:"field1", type:"TextItem"},
*      {name:"nestedItem", shouldSaveValue:true, type:"CanvasItem",
*        canvas:isc.DynamicForm.create({
*          operator:"or",
*          items:[
*            {name:"innerField1", type:"TextItem", operator:"equals"},
*            {name:"innerField2", type:"TextItem"}
*          ]
*        })
*      }
*    ]
*  });
*  
*  
*  
*  This form would be able to edit the above advanced criteria object via
*  DynamicForm.setValuesAsCriteria. Edited values would be retrieved via
*  DynamicForm.getValuesAsCriteria.
*  
*  Note that the canvas item has shouldSaveValue set to true - this is required
*  to ensure the nested form is actually passed the values to edit. 
*  
*  The default implementation of this method checks for this.canvas being specified as a
*  dynamicForm, and in that case simply returns the result of 
*  DynamicForm.getValuesAsAdvancedCriteria on the inner form.
*  
*  Note that this functionality may be entirely bypassed by
*  setting CanvasItem.editCriteriaInInnerForm to false. This flag is useful when defining a
*  dynamicForm based canvasItem which is not intended for editing nested data -- for example
*  if a standard atomic field value is being displayed in some custom way using a
*  DynamicForm embedded in the item.
*  
* 
*/
getCriterion(): Criterion;

/**
*  Display a Criterion object in this item for editing. Overridden from 
*  FormItem.setCriterion in order to support editing nested criteria using 
*  nested dynamicForms as described in CanvasItem.getCriterion.
*  
*  Implementation checks for this.canvas being specified as a DynamicForm, and applies
*  criterion directly to the embedded form via setValuesAsCriteria()
* 
* @param {Criterion} criteria to edit
*/
setCriterion(criterion:Criterion): void;

/**
*  Set the FormItem.prompt for this item. Default implementation will also apply the
*  prompt to CanvasItem.canvas if CanvasItem.applyPromptToCanvas is true.
* 
* @param {HTMLString} new prompt for the item.
*/
setPrompt(prompt:HTMLString): void;

/**
*  This method allows dynamic creation of a CanvasItem's canvas, rather than
*  setting CanvasItem.canvas statically.
*  If specified this stringMethods will be called when the form item is 
*  initialized and should return the Canvas to display for this item.
* 
* @param {DynamicForm} the dynamicForm in which this item is contained
* @param {CanvasItem} the live form item instance
*/
createCanvas(form:DynamicForm, item:CanvasItem): Canvas;

/**
*  This method will be called whenever this FormItem's value is being set via a programmatic
*  call to e.g: DynamicForm.setValues or FormItem.setValue and may be
*  overridden by CanvasItems intended to support displaying data values to update the
*  embedded Canvas to reflect the value passed in. Note that the first parameter will be a 
*  formatted value - while the second parameter will contain the underlying data value for
*  the item.
* 
* @param {any} new display value for the item. This is the value after applying  any custom formatter or valueMap
* @param {any} underlying data value for the item
* @param {DynamicForm} the dynamicForm in which this item is contained
* @param {CanvasItem} the live form item instance
*/
showValue(displayValue:any, dataValue:any, form:DynamicForm, item:CanvasItem): void;



}
/**
* Creates a new CanvasItem
* 
* @param typeCheckedProperties {Partial<CanvasItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {CanvasItem} The new instance
*/
class CanvasItem{
static create(typeCheckedProperties? : Partial<CanvasItem>, uncheckedProperties?: Object): CanvasItem;







}

/**
* Maintains a representation of selection over a 2-dimensional grid of objects.
*  Automatically created to manage cell-selection on CubeGrid widgets.
*/
interface CellSelection extends Class {


/**
*  Return true if a particular item is selected
* 
* @param {Integer} row index of the cell to check
* @param {Integer} column index of the cell to check
*/
cellIsSelected(rowNum:Integer, colNum:Integer): boolean;

/**
*  Is anything in the list selected?
* 
*/
anySelected(): boolean;

/**
*  Returns an array of the currently selected cells. Each cell is returned as a 2 element
*  array in the form [rowNum, colNum].
* 
*/
getSelectedCells(): Array<any>;

/**
*  Returns the first record that has any cells selected.
* 
*/
getSelectedRecord(): ListGridRecord;

/**
*  Observable handler fired
*  whenever the cell selection is modified
* 
* 
*/
selectionChanged(): void;

/**
*  Select a particular cell
* 
* 
* @param {Integer} row index of cell to select
* @param {Integer} column index of cell to select
*/
selectCell(rowNum:Integer, colNum:Integer): boolean;

/**
*  Deselect a particular cell
* 
* 
* @param {Integer} row index of the cell to select
* @param {Integer} column index of the cell to select
*/
deselectCell(rowNum:Integer, colNum:Integer): boolean;

/**
*   select an array of cells
* 
* @param {Array<Partial<any>>} Array of cells to select. Each cell can be specified                    as a 2 element array [rowNum, colNum]
*/
selectCellList(list:Array<Partial<any>>): boolean;

/**
*   deselect an array of cells
* 
* 
* @param {Array<Partial<any>>} Array of cells to deselect. Each cell can be specified                    as a 2 element array [rowNum, colNum]
*/
deselectCellList(list:Array<Partial<any>>): boolean;

/**
*   select a single cell and deselect everything else
* 
* @param {Integer} row index of cell to select
* @param {Integer} column index of cell to select
*/
selectSingleCell(rowNum:Integer, colNum:Integer): boolean;



}
/**
* Creates a new CellSelection
* 
* @param typeCheckedProperties {Partial<CellSelection>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {CellSelection} The new instance
*/
class CellSelection{
static create(typeCheckedProperties? : Partial<CellSelection>, uncheckedProperties?: Object): CellSelection;







}

/**
* Checkbox form item, implemented with customizable checkbox images.
*/
interface CheckboxItem extends FormItem {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "CheckboxItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Base CSS class for this item's title cell.
*  
*  Note: This styling applies to the standard form item title cell for this item - it
*  does not apply to item's CheckboxItem.showLabel.
*  To modify the styling for that text, use CheckboxItem.textBoxStyle instead.
* @type {FormItemBaseStyle}
* @default null
*/
titleStyle?: FormItemBaseStyle;

/**
* Base CSS class for this item's title text
* @type {FormItemBaseStyle}
* @default "labelAnchor"
*/
textBoxStyle?: FormItemBaseStyle;

/**
* Width for the CheckboxItem, including both checkbox image and CheckboxItem.showLabel.
*  Note that if CheckboxItem.showLabel is false, this property will have no effect - the
*  item will render at the size required to contain the icon.
* @type {number}
* @default "150"
*/
width?: number;

/**
* CheckboxItem has special behavior for titles, see CheckboxItem.labelAsTitle.
* @type {boolean}
* @default true
*/
showTitle?: boolean;

/**
* By default a checkboxItem sets CheckboxItem.showTitle:true, and so takes up two cells with the
*  default TitleOrientation of "left" (see formLayout). However, the title cell is left blank by default, and the title specified
*  by FormItem.title is shown inside the formItem's cell instead, in an element
*  called the "label".
*  
*  To instead show the title in it's original location, set labelAsTitle:true.
*  You can also set CheckboxItem.showLabel:false to suppress the label and/or title altogether.
* @type {boolean}
* @default null
*/
labelAsTitle?: boolean;

/**
* Should we show the label text next to the checkbox item.
* @type {boolean}
* @default true
*/
showLabel?: boolean;

/**
* Should an "Over" state icon be shown when the user rolls over this checkbox
* @type {boolean}
* @default true
*/
showValueIconOver?: boolean;

/**
* Should a "Down" state icon be shown when the mouse goes down over this checkbox
* @type {boolean}
* @default true
*/
showValueIconDown?: boolean;

/**
* Should a "Disabled" state icon be shown when the item is disabled
* @type {boolean}
* @default true
*/
showValueIconDisabled?: boolean;

/**
* URL for the image to display when this checkbox is selected, or checked.
*  
*  This image is implemented using the FormItem.valueIcons,
*  and may be modified via the standard valueIcons properties such as
*  CheckboxItem.valueIconWidth
*  
*  Note that this is the base image name - if CheckboxItem.showValueIconOver et al are set, the
*  state ("Over", "Down" and "Disabled") will be
*  added to this name as the user interacts with the checkbox, as well as the image extension
*  
*  The special value "blank" means that no image will be shown.
*  
*  skinning can be used for this image, by setting this property to
*  a SCSpriteConfig formatted string. Alternatively developers can
*  omit this property and instead use CSS directly in the 
*  CheckboxItem.booleanBaseStyle property to provide a "checked" appearance.
* @type {SCImgURL}
* @default "[SKIN]/DynamicForm/checked.gif"
*/
checkedImage?: SCImgURL;

/**
* URL for the image to display when this checkbox is not selected, or unchecked.
*  
*  The special value "blank" means that no image will be shown.
*  
*  skinning can be used for this image, by setting this property to
*  a SCSpriteConfig formatted string. Alternatively developers can
*  omit this property and instead use CSS directly in the 
*  CheckboxItem.booleanBaseStyle property to provide an "unchecked" appearance.
* @type {SCImgURL}
* @default "[SKIN]/DynamicForm/unchecked.gif"
*/
uncheckedImage?: SCImgURL;

/**
* URL for the image to display when this checkbox is partially selected.
*  
*  The special value "blank" means that no image will be shown.
*  
*  skinning can be used for this image, by setting this property to
*  a SCSpriteConfig formatted string. Alternatively developers can
*  omit this property and instead use CSS directly in the 
*  CheckboxItem.booleanBaseStyle property to provide a "partially checked" appearance.
* @type {SCImgURL}
* @default "[SKIN]/DynamicForm/partialcheck.gif"
*/
partialSelectedImage?: SCImgURL;

/**
* If set, the CheckboxItem.checkedImage to use when printing.
* @type {SCImgURL}
* @default null
*/
printCheckedImage?: SCImgURL;

/**
* If set, the CheckboxItem.uncheckedImage to use when printing.
* @type {SCImgURL}
* @default null
*/
printUncheckedImage?: SCImgURL;

/**
* If set, the CheckboxItem.partialSelectedImage to use when printing.
* @type {SCImgURL}
* @default null
*/
printPartialSelectedImage?: SCImgURL;

/**
* Determines what image to display when the value for this checkbox is unset.
*  Set to true to display the CheckboxItem.unsetImage for null values, or false to
*  use the CheckboxItem.uncheckedImage for both null and explicitly unchecked values.
*  
*  If this attribute is not set, the CheckboxItem.unsetImage for null values if 
*  CheckboxItem.allowEmptyValue is true for this item, otherwise the unchecked
*  image will be used.
* @type {boolean}
* @default null
*/
showUnsetImage?: boolean;

/**
* URL for the image to display when this checkbox is unset. Note that if 
*  CheckboxItem.showUnsetImage is false or CheckboxItem.allowEmptyValue is false
*  the CheckboxItem.uncheckedImage will be used for null values rather than this
*  image.
*  
*  The special value "blank" means that no image will be shown.
*  
*  skinning can be used for this image, by setting this property to
*  a SCSpriteConfig formatted string. Alternatively developers can
*  omit this property and instead use CSS directly in the 
*  CheckboxItem.booleanBaseStyle property to provide an "unset" appearance.
* @type {SCImgURL}
* @default "[SKIN]/DynamicForm/unsetcheck.gif"
*/
unsetImage?: SCImgURL;

/**
* If set, the CheckboxItem.unsetImage to use when printing.
* @type {SCImgURL}
* @default null
*/
printUnsetImage?: SCImgURL;

/**
* The description shown in a hover when FormItem.showOldValueInHover is enabled
*  and a value represents the checked state.
* @type {HTMLString}
* @default "checked"
*/
checkedDescription?: HTMLString;

/**
* The description shown in a hover when FormItem.showOldValueInHover is enabled
*  and a value represents the unchecked state.
* @type {HTMLString}
* @default "unchecked"
*/
uncheckedDescription?: HTMLString;

/**
* The description shown in a hover when FormItem.showOldValueInHover is enabled
*  and a value represents the partial selected state.
* @type {HTMLString}
* @default "partially selected"
*/
partialSelectedDescription?: HTMLString;

/**
* The description shown in a hover when FormItem.showOldValueInHover is enabled
*  and a value represents the unset state.
* @type {HTMLString}
* @default "unset"
*/
unsetDescription?: HTMLString;

/**
* An optional base CSS style to apply to the checkbox image. If supplied, the base style is
*  suffixed with "True", "False", "Partial", or "Unset" if the checkbox is selected, unselected,
*  partially selected, or unset. The style is then suffixed with the state of the value icon
*  ("", "Over", "Down", "Disabled").
* @type {CSSStyleName}
* @default null
*/
booleanBaseStyle?: CSSStyleName;

/**
* If set, the CheckboxItem.booleanBaseStyle to use when printing.
* @type {CSSStyleName}
* @default null
*/
printBooleanBaseStyle?: CSSStyleName;

/**
* Width of the checkbox image.
* @type {number}
* @default "13"
*/
valueIconWidth?: number;

/**
* Height of the checkbox image.
* @type {number}
* @default "13"
*/
valueIconHeight?: number;

/**
* Object defining how the checkbox "checked" state will be mapped to the field value.
*  Checkboxes only support 2 states. By default a checked checkbox will have
*  value true, and an unchecked one will have value false
*  (or null if there is no default value and the value has never been set).
*  
*  A valueMap can modify this in 2 ways:
*  - If the desired checked/unchecked values can be resolved to true 
*   and false directly in JavaScript, the valueMap may be specified as
*   a 2-element array containing these values. Examples of this would include
*   &nbsp;&nbsp;[0,1]: 
*   In this case an unchecked checkbox would have value 0 and a checked box
*   would have value 1
*   &nbsp;&nbsp;[null,"Some String"]:
*   In this case an unchecked checkbox would have value null and a checked 
*   box would have value "Some String"
*  - More arbitrary data values can be resolved to checked / unchecked values via an
*   object mapping the arbitrary data values to display values of true and
*   false. For example:
*   &nbsp;&nbsp;{"A":false, "B":true}
*   In this case an unchecked checkbox would have value "A", and a checked box 
*   would have value "B"
*  
*  Note: ValueMaps in other formats will be ignored by the CheckboxItem class. To update
*  the valueMap at runtime, always use CheckboxItem.setValueMap
* @type {object | Array<any>}
* @default null
*/
valueMap?: object | Array<any>;

/**
* By default checkboxes allow the user to toggle between true and false values only.
*  Setting this property to true will allow the user to toggle between three values - 
*  true, false and unset.
* @type {boolean}
* @default false
*/
allowEmptyValue?: boolean;

/**
* Should a null value be shown as checked (i.e. true)?
* @type {boolean}
* @default false
*/
showNullAsTrue?: boolean;

/**
* Set this property to the name of another field in the same record, to have this field 
*  be shown as checked (i.e. true) if this field is null and the other field is not null. 
*  For example, you could use this to show a "US Citizen" field as true if a value is
*  entered into another field "US Social Security Number"
* @type {string}
* @default null
*/
showNullAsTrueIf?: string;

/**
* If this checkbox item is CheckboxItem.showLabel, should
*  it ignore any specified FormItem.width and instead size to fit its 
*  CheckboxItem.checkedImage?
*  
*  When set to true (the default), the checkbox item ignores any specified width,
*  ensuring that it does not impact the the containing DynamicForm's table geometry
*  unnecessarily.
* @type {boolean}
* @default true
*/
sizeToCheckboxImage?: boolean;



/**
*  Setter method to apply a valueMap to a checkbox item.
*  Note that if this method is overridden, the override must call
*  this.Super("setValueMap", arguments); to maintain functionality in this
*  class.
* 
*/
setValueMap(): void;

/**
*  Return the value tracked by this form item as a Boolean. If the value is not
*  already a boolean, or is unset and CheckboxItem.allowEmptyValue is true,
*  then null will be returned.
* 
* 
*/
getValueAsBoolean(): boolean;

/**
*  Notification method called when FormItem.showPending is enabled and
*  this checkbox item should either clear or show its pending visual state.
*  
*  The default behavior is that the FormItem.cellStyle and checkbox label
*  style are updated to include/exclude the "Pending" suffix. Returning false
*  will cancel this default behavior.
* 
* @param {DynamicForm} the managing DynamicForm instance.
* @param {FormItem} the form item itself (also available as "this").
* @param {boolean} true if the item should show its pending visual state; false otherwise.
* @param {any} the current form item value.
* @param {any} the value that would be restored by a call to DynamicForm.resetValues.
*/
pendingStatusChanged(form:DynamicForm, item:FormItem, pendingStatus:boolean, newValue:any, value:any): boolean;



}
/**
* Creates a new CheckboxItem
* 
* @param typeCheckedProperties {Partial<CheckboxItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {CheckboxItem} The new instance
*/
class CheckboxItem{
static create(typeCheckedProperties? : Partial<CheckboxItem>, uncheckedProperties?: Object): CheckboxItem;







}

/**
* EditProxy that handles CheckboxItem when editMode is enabled.
*/
interface CheckboxItemEditProxy extends FormItemEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's value as [ ] or [x].
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's value.
* 
* 
* @param {string} the new component value
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new CheckboxItemEditProxy
* 
* @param typeCheckedProperties {Partial<CheckboxItemEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {CheckboxItemEditProxy} The new instance
*/
class CheckboxItemEditProxy{
static create(typeCheckedProperties? : Partial<CheckboxItemEditProxy>, uncheckedProperties?: Object): CheckboxItemEditProxy;







}

/**
* The Class object is root of the Isomorphic SmartClient inheritance tree -- it includes
*  functionality for creating instances, adding methods and properties, getting prototypes,
*  etc.
* 
*  To add functionality to ALL classes, add them to Class.
* 
*  To create a Class, call ClassFactory.defineClass("MyClass", "MySuperClass")
*  
*  defineClass will return the created class, and make it available as
*  isc.MyClass, and as the global variable MyClass if not in
*  isc.
*  
*  You can then:
*  
*  
* - add class-level (static) properties and methods to the class:
*   MyClass.addClassProperties()	
*  	these methods and properties are accessed through the Class variable itself, eg:
*   MyClass.someStaticMethod() or MyClass.someStaticProperty
* 
*  
* - add default instance properties and methods to the class:
*   MyClass.addProperties()
*  	these methods and properties are accessed through a class instance, eg:
*   var myInstance = MyClass.create();
*   myInstance.someInstanceMethod()
* 
*  
* - create new instances of this class:
*   var myInstance = MyClass.create()
*  
*  NOTE: as a convention, all class names begin with a capital letter and all instances begin
*  with a lower case letter.
*/
interface Class {
/**
* Controls whether arguments passed to Class.create are assumed to be
*  Objects containing properties that should be added to the newly created instance. This
*  behavior is how create() works with almost all SmartClient widgets and
*  other components, allowing the convenient shorthand of setting a batch of properties via
*  an ObjectLiteral passed to create().
*  
*  The setting defaults to true if unset. To disable this behavior for a custom class,
*  such that create() works more like typical constructors found in Java and
*  other languages, use:
*  
*    isc.[i]ClassName[/i].addProperties({ addPropertiesOnCreate:false })
*  
*  
*  Note that it is not valid to disable this behavior for any subclass of Canvas
*  (Canvas relies on this property).
*  
*  Regardless of the setting for addPropertiesOnCreate, all arguments passed to
*  Class.create are still passed on to Class.init.
* @type {boolean}
* @default undefined
*/
addPropertiesOnCreate?: boolean;

/**
* For an autoChildren, a read-only reference to the component on
*  which Class.createAutoChild or Class.addAutoChild was called to create it.
*  Useful for authoring of event handlers (eg click:"this.creator.doSomething()")
* @type {Class}
* @default "varies"
*/
creator?: Class;

/**
* Specifies the component on which Class.createAutoChild should be called to
*  create autoChildren defined lazily on this component in the format
*  "autoChild:autoChildName". This property may be either specified as a live
*  component, or set to the childName of another already-created
*  AutoChild.
*  
*  If left unspecified, the Framework applies rules to determine which component to use as
*  the creator. If this component is itself an autochild, and properties or defaults for
*  the child are defined on its Class.creator but not on the component itself, then the
*  creator of this component is also used to create the new AutoChild. Otherwise, this
*  component is used.
* @type {Class | string}
* @default "varies"
*/
autoCreator?: Class | string;

/**
* Canvas.ID of the component that manages "rule context" for which
*  this class participates. A non-Canvas class can only use the ruleScope
*  for supporting Class.dynamicProperties. Unlike Canvas.ruleScope
*  ruleScope on a standalone class must be explicitly specified.
* @type {string}
* @default null
*/
ruleScope?: string;

/**
* Object mapping dynamic property names to the source - a DataPath,
*  UserSummary, UserFormula, or AdvancedCriteria.
*  This is a declarative alternative to calling Class.addDynamicProperty for each property.
*  
*  See Class.addDynamicProperty for details on using dynamic properties.
*  
*  
*  In JavaScript dynamicProperties can be declaratively initialized as follows:
*  
*  dynamicProperties: {
*    propName1: "a/b/c",
*    propName2: { formula: .. formula definition .. },
*    propName3: { textFormula: .. summary definition .. },
*    propName4: { operator: "and",
*          criteria: [{
*            fieldName: "a/b/c", operator:"greaterOrEqual", value:"value1"
*          }, {
*            fieldName: "a/b/c", operator:"lessOrEqual", value:"value2"
*          }]
*         }
*  }
*  
*  
*  
*  In ComponentXML dynamicProperties can be intialized as:
*  
*  &lt;dynamicProperties&gt;
*    &lt;property name="propName" dataPath="a/b/c"/&gt;
*    &lt;property name="propName2"&gt;
*      &lt;formula&gt;
*        &lt;UserFormula ... &gt;
*      &lt;/formula&gt;
*    &lt;/property&gt;
*    &lt;property name="propName3"&gt;
*      &lt;textFormula&gt;
*        &lt;UserSummary ... &gt;
*      &lt;/textFormula&gt;
*    &lt;/property&gt;
*    &lt;property name="propName4"&gt;
*      &lt;trueWhen&gt;
*        &lt;criteria fieldName="a/b/c" operator="iEquals" value="value1"/&gt;
*      &lt;/trueWhen&gt;
*    &lt;/property&gt;
*    &lt;property name="propName5"&gt;
*      &lt;trueWhen&gt;
*        &lt;AdvancedCriteria operator="and"&gt;
*          &lt;criteria&gt;
*            &lt;criterion fieldName="a/b/c" operator="greaterOrEqual" value="value1"/&gt;
*            &lt;criterion fieldName="a/b/c" operator="lessOrEqual"  value="value2"/&gt;
*          &lt;/criteria&gt;
*        &lt;/AdvancedCriteria&gt;
*      &lt;/trueWhen&gt;
*    &lt;/property&gt;
*  &lt;/dynamicProperties&gt;
*  
* @type {Map<any>}
* @default null
*/
dynamicProperties?: Map<any>;



/**
* 	
*  Initialize a new instance of this Class. This method is called automatically by
*  Class.create. 
*  
*  Override this method to provide initialization logic for your class. If your class is
*  a subclass of a UI component (i.e. descendant of Canvas), override
*  Canvas.initWidget instead. 
* 
* 
* @param {any=} All arguments initially passed to Class.create
*/
init(...args:any[]): void;

/**
*  Permanently destroy a class instance and any automatically created resources,
*  recursively.
* 
*/
destroy(): void;

/**
*  	Return the global identifier for this object.
* 
* 
*/
getID(): string;

/**
* 	
* 	Gets a pointer to the class object for this instance
* 
* 
*/
getClass(): Class;

/**
* 	
* 	Gets a pointer to the class object for this instance's superclass.
* 
* 
*/
getSuperClass(): Class;

/**
* 	
* 	Gets the name of this class as a string.
* 
* 
*/
getClassName(): string;

/**
* 	
*  Add properties or methods to this specific instance. 
* 	Properties with the same name as existing properties will override.
* 
* 
* @param {object=} Object containing name:value pairs to be added to this object
*/
addProperties(...args:any[]): object;

/**
* 
* 	Add properties to this instance.
* 
* 
* @param {Array<Partial<object>>} array of objects with properties to add
*/
addPropertyList(list:Array<Partial<object>>): object;

/**
*  Set a property on this object, calling the setter method if it exists.
*  
*  Whenever you set a property on an ISC component, you should call either the specific setter
*  for that property, or setProperty()/setProperties() if it doesn't have one.
*  This future-proofs your code against the later addition of required setters. 
* 
* 
* @param {string} name of the property to set
* @param {any} new value for the property
*/
setProperty(propertyName:string, newValue:any): void;

/**
*  Set multiple properties on an object, calling the appropriate setter methods if any are
*  found.
*  
*  Whenever you set a property on an ISC component, you should call either the specific setter
*  for that property, or setProperty()/setProperties() if it doesn't have one.
*  This future-proofs your code against the later addition of required setters. 
*  
*  With setProperties() in particular, some classes may be able to take shortcuts
*  and be more efficient when 2 or more related properties are set at the same time.
* 
* 
* @param {object=} objects with properties to add (think named parameters).     all the properties of each argument will be applied one     	after another so later properties will override
*/
setProperties(...args:any[]): void;

/**
* 
* 	Returns whether this object is of a particular class by class name, either as a direct
* 	instance of that class or as subclass of that class, or by implementing an interface
*  that has been mixed into the class.
* 
* 	NOTE: this only applies to ISC's class system, eg: myInstance.isA("Object") will be
* 	false.
* 
* 
* @param {string} Class name to test against
*/
isA(className:string): boolean;

/**
*  Set up a notification action to be invoked whenever some method is called on a target
*  object.
*  
*  For example, if you wanted to take an action every time some ListGrid on your page
*  had its ListGrid.selectionUpdated, instead of 
*  overriding the selectionUpdated() method on the grid, you could
*  observe it with code like this:
*  
*  myCanvas.observe(myListGrid, "selectionUpdated", "observer.gridSelectionUpdated()")
*  
*  In this example, every time selectionUpdated() fired on the grid "myListGrid", 
*  after that method completed, the specified action would be invoked. (In this case
*  the action would call a method called "gridSelectionUpdated" on the observer, "myCanvas").
*  
*  An unlimited number of observers can be set up to observe any method. The notification
*  actions will all be fired automatically in the order that the observations were set up.
*  
*  NOTES: 
*  
*  
* - The object to observe may be any JavaScript object with the specified method,
*    including simple JavaScript Objects or Arrays, or instances of SmartClient classes
*    like Canvas.
*  
* - For a given observer, observed object, and observed method combination, at most one
*    action can be registered. If you attempt to call observe() again with
*    the same combination, it will return false and the action will not be registered.
*  
* - A method could potentially trigger an observation of itself by another object, 
*    either through code within the method itself or within an observer's action.
*    In this case the observation will be set up, but the new observation action will
*    not fire as part of this thread. For subsequent calls to the method, the newly 
*    added observer will be fired.
*  
* - [Potential memory leak]: If the target object is a simple JavaScript object
*    (not an instance of a SmartClient class), developers should always call 
*    Class.ignore to stop observing the object when an observation 
*    is no longer necessary.
*    This ensures that if the observed object is subsequently allowed to go out of scope by 
*    application code, the observation system will not retain a reference
*    to it (so the browser can reclaim the allocated memory).
*    While cleaning up observations that are no longer required is always good practice, 
*    this memory leak concern is not an issue if the target object is an instance of
*    a SmartClient class. In that case the observation is automatically released when the 
*    target is Class.destroy.
*  
* 
* 
* @param {object} Object to observe. This may be any JavaScript object with the specified  target method, including native arrays, and instances of SmartClient classes such as   Canvas.
* @param {string} Name of the method to observe. Every time this method is invoked  on the target object the specified action will fire   (after the default implementation completes).
* @param {Function | string=} Optional action to take when the observed method is invoked on the target object.  If action is a string to execute, certain keywords are available for context:    
- observer is this object (the object on which the    observe(...) method was called).  
- observed is the target object being observed (on which the method was invoked).  
- returnVal is the return value from the observed method (if there is one)  
- For functions defined with explicit parameters, these will also be available as keywords within    the action string    If action is a function, this will be executed in the scope of the   observer (so this will be the object on which observe()   was invoked). The arguments for the original method will also be   passed to this action function as arguments. If developers need to access the target   object being observed from the action function they may use native javascript techniques  such as   javascript closure  to do so. The return value from the observed method is not available to the action function.  If the action parameter is omitted the default behavior will invoke the   same named method on the observer, passing in the same parameters.
*/
observe(object:object, methodName:string, action?:Function | string): boolean;

/**
*  Stop observing a method on some other object.
* 
* 
* @param {object} object to observe
* @param {string} name of the method to ignore
*/
ignore(object:object, methodName:string): boolean;

/**
*  Return true if this object is already observing a method of another object
* 
* 
* @param {object} object we may be observing
* @param {string} name of the method to observed
*/
isObserving(object:object, methodName:string): boolean;

/**
*  
*  Evaluate a string of script in the scope of this instance (so this
*  is available as a pointer to the instance).
* 
* 
* @param {string} the expression to be evaluated
* @param {object} Optional mapping of argument names to values - each key will   be available as a local variable when the script is executed.
*/
evaluate(expression:string, evalArgs:object): any;

/**
* 
* 	Method to fire a callback. Callback will be fired in the scope of the object on 
*  which this method is called.
*  Falls through to Class.fireCallback
* 
* 
* @param {Callback} Callback to fire
* @param {string=} comma separated string of variables
* @param {Array<any>=} array of arguments to pass to the method
*/
fireCallback(callback:Callback, argNames?:string, args?:Array<any>): any;

/**
*  This is a helper to delay a call to some method on this object by some specified
*  amount of time.
* 
* @param {string} name of the method to call
* @param {Array<any>=} array of arguments to pass to the method in question
* @param {number=} Number of ms to delay the call by - defaults to zero (so just pulls            execution of the method out of the current execution thread.
*/
delayCall(methodName:string, arrayArgs?:Array<any>, time?:number): string;

/**
*  Creates a component according to the "AutoChild" pattern, and adds it to this component.
*  
*  See the autoChildren to understand the general
*  purpose and usage of this method.
*  
*  addAutoChild() takes the following actions:
*  
*  
* - checks whether this.autoChildName is already populated, and returns it if so
*  
* - checks when there is a showAutoChildName with the value false, and if so
*  returns without creating a component 
*  
* - calls Class.createAutoChild to create the component
*  
* - sets this.autoChildName to the created component
*  
* - adds the component either to this component, or to some other parent, specified
*  by the "autoParent" property in the autoChild's defaults. The "autoParent" property may
*  be "none" indicating the autoChild should not be automatically added.
*  
*  
*  When adding an autoChild to a Layout subclass,
*  Layout.addMember will be called instead of the normal
*  Canvas.addChild. To prevent this behavior,
*  addAsChild:true can be set in the autoChild defaults. Similarly,
*  addAsPeer:true may be set to add an autoChild as a peer.
*  
*  Tip: because addAutoChild()
*  checks specifically for showAutoChildName:false, you do not have to add
*  showAutoChildName:true in order for an autoChild to be shown by default; leaving
*  the property undefined is sufficient.
*  
*  Note that by default the child created by this method will be destroyed when
*  Canvas.destroy is called on this instance. To disable this behavior,
*  set dontAutoDestroy to true on the auto child.
*  
* 
* @param {string} name of the autoChild
* @param {Properties} dynamic properties for the autoChild
*/
addAutoChild(childName:string, defaults:Properties): Class;

/**
*  Unconditionally creates and returns a component created according to the "AutoChild"
*  pattern.
*  
*  In addition to applying defaults and properties as described under the
*  autoChildUsage, the created autoChild:
*  
*  
* - is automatically autoDraw:false 
*  
* - has a creator property that points to this component, for easy
*  authoring of event handlers (eg click:"this.creator.doSomething()")
*  
*  
*  Unlike Class.addAutoChild, createAutoChild() does not create a
*  this.autoChildName reference to the component, check a showAutoChildName
*  flag, or automatically add the autoChild via Canvas.addChild. 
*  
*  General you use createAutoChild rather than addAutoChild when:
*  
*  
* - you are going to create several autoChildren with a common set of defaults (for
*  example the ColumnTree.column autoChild of the ColumnTree).
*  
* - children need to be created before their parents (eg, for layout/auto-sizing
*  reasons)
*  
*  
*  Note that by default the child created by this method will be destroyed when
*  Canvas.destroy is called on this instance. To disable this behavior,
*  set dontAutoDestroy to true on the auto child.
* 
* 
* @param {string} name of the autoChild
* @param {Properties} dynamic properties for the autoChild
*/
createAutoChild(childName:string, defaults:Properties): Class;

/**
* 
*  Call method on each item in argsList and return the Array of results.
* 
* 
* @param {string} Name of the method on this instance which should be called on each element of the Array
* @param {Array<any>} Array of items to call the method on
*/
map(methodName:string, items:Array<any>): Array<any>;

/**
* 
*  Call the SuperClass implementation of an instance method. For example:
*  
*   isc.defineClass("MyButton", "Button").addProperties({
*     // this override causes no change in behavior because it just 
*     // calls Super and returns whatever the superclass would return
*     getTitle : function () {
*       return this.Super("getTitle", arguments);
*     },
* 
*     // this override would add "foo" to the titles of all buttons
*     getTitle : function () {
*       // add code here to take actions before the superclass method is invoked
* 
*       var superResult = return this.Super("getTitle", arguments);
* 
*       // add code here to take action after the superclass method is invoked
* 
*       return superResult + "foo";
*     }
* 
*   })
*  
*  Note that Super() is always called with the name of the current method. You cannot call
*  the Super class implementation of another method directly.
*  
*  It is required to always pass the native 'arguments' object to Super(). Arguments
*  is a JavaScript builtin that is available within any JavaScript function - see any
*  JavaScript Reference for details.
*  
*  If you override a method in an instance, and then call Super(), the prototype
*  implementation will be called. This is similar to how anonymous classes in Java handle
*  super(). For example:
*  
*   isc.Button.create({
*     // this will set the title to indicate the runtime button class
*     initWidget : function () {
*       this.Super("initWidget", arguments);
*       this.title = "Parent Class: " + this.getSuperClass().getClassName();
*     },
*     width: 1,
*     overflow: "visible"
*   });
*  
*  See also ClassFactory.defineClass and
*  Class.addProperties for the basics of creating classes
*  and overriding methods.
* 
* 
* @param {string} name of the superclass method to call
* @param {any | Array<any>} native "arguments" object, or array of                     arguments to pass to the Super call
* @param {any=} native "arguments" object, required if an Array is                 passed for the "args" parameter in lieu of the native                  arguments object
*/
Super(methodName:string, args:any | Array<any>, nativeArgs?:any): any;

/**
*  Sets up the value of propertyName to be dynamically derived from the
*  Canvas.ruleScope, by either a simple DataPath into the ruleScope,
*  an AdvancedCriteria built against DataPath, or via a textual or
*  numeric formula using the ruleScope as available formula inputs.
*  
*  The dataPath, criteria, or formula is evaluated immediately when addDynamicProperty() is
*  called, then re-evaluated every time the ruleScope changes. An AdvancedCriteria
*  will always evaluate to boolean true or false, and a UserSummary to a string.
*  
*  It is invalid usage to use addDynamicProperty() on a property that is not
*  runtime settable. However, addDynamicProperty() will not throw an error or
*  log a warning if this is done.
*  
*  If a property is already dynamic and addDynamicProperty() is called again, the new
*  dynamic behavior replaces the old. If a property should no longer be dynamic, call
*  Class.clearDynamicProperty.
*  
*  Dynamic properties can also be declared together via Class.dynamicProperties.
*  
*  Note that you may convert a simple criteria to an AdvancedCriteria by calling
*  DataSource.convertCriteria.
* 
* 
* @param {Identifier} name of a settable property on this instance
* @param {DataPath | UserSummary | UserFormula | AdvancedCriteria} */
addDynamicProperty(propertyName:Identifier, source:DataPath | UserSummary | UserFormula | AdvancedCriteria): void;

/**
*  Clears a dynamic property previously established via Class.addDynamicProperty.
*  
*  If the property is not currently dynamic, nothing will be done (and no warning logged).
*  
*  The current value of the property will not be changed by this call.
* 
* 
* @param {Identifier} property name of the dynamic property to clear
*/
clearDynamicProperty(propertyName:Identifier): void;

/**
*  Returns true if the property is dynamic.
* 
* 
* @param {Identifier} name of a settable property on this instance
*/
hasDynamicProperty(propertyName:Identifier): boolean;

/**
*  Returns the last time the rule for the specified dynamic property fired, as a
*  Date.
* 
* 
* @param {Identifier} name of a settable property on this instance
*/
getDynamicPropertyRuleTime(propertyName:Identifier): Date;

/**
* 
*  The default toString() for instances reports that you have an instance of a class and prints
*  the instance ID if present.
* 
*/
toString(): void;

/**
*  Returns a one-line summary of the current method call, showing method name and passed
*  arguments. 
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.getCallTrace(arguments)" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
*  
*  Note the arguments object is required in most cases for this method to function. In some
*  browsers, it can be derived automatically, but developers looking to debug on multiple platforms
*  should not rely on this.
* 
* 
* @param {any=} arguments object from the call to trace. If ommitted, where supported,  arguments will be derived from the calling function, or if this is not supported, the method  will not function.
*/
getCallTrace(args?:any): void;

/**
*  Returns a "stack trace" - one line per method in the current call stack, showing the method
*  name and any parameters passed.
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.getStackTrace" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
*  
*  Platform Notes: In Mozilla Firefox, if Firebug is enabled, a stack trace will be logged
*  to the firebug console in addition to the standard stack trace string returned by
*  this method.
*  
*  In browsers other than Internet Explorer a complete stack trace may not be available - 
*  this occurs when a function is re-entrant (meaning it calls itself). In this case the
*  stack will terminate with text indicating where the recursive function call occurred.
*  
*  See debugging for further information information.
* 
* 
*/
getStackTrace(): string;

/**
*  Return a very short (generally less than 40 characters) string representation of any object,
*  suitable for viewing by a developer for debugging purposes.
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.echoLeaf" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
* 
* 
* @param {any} object to echo
*/
echoLeaf(obj:any): string;

/**
*  Return a short string representation of any object, suitable for viewing by a developer for
*  debugging purposes.
*  
*  If passed an object containing other objects, echo will not recurse into subobjects,
*  summarizing them instead via echoLeaf().
*  
*  NOTE: echo() is used to generate the output shown in the Log window when evaluating an
*  expression.
*  
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.echo()" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
*  
* 
* @param {any} object to echo
*/
echo(obj:any): string;

/**
*  Like echo(), except that if passed an Array, echoAll() will echo() every element of the
*  Array.
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.echo()" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
* 
* 
* @param {any} object to echo
*/
echoAll(obj:any): string;

/**
*  Log a message at "debug" priority
*  
*  A method named logPriority exists for each priority level, on every ISC Class and
*  instance of an ISC Class. Messages logged on a Class or instance have a default
*  category of the classname. Messages logged on an instance will also automatically
*  incorporate the instance ID. General best practice is to call logDebug() et al as
*  "this.logDebug" whenever "this" is an instance, or as "Log.logDebug" otherwise.
* 
* 
* @param {string} message to log
* @param {string=} category to log in
*/
logDebug(message:string, category?:string): void;

/**
*  Log a message at "info" priority
*  
* 
* @param {string} message to log
* @param {string=} category to log in
*/
logInfo(message:string, category?:string): void;

/**
*  Log a message at "warn" priority
*  
* 
* @param {string} message to log
* @param {string=} category to log in
*/
logWarn(message:string, category?:string): void;

/**
*  Log a message at "error" priority
*  
* 
* @param {string} message to log
* @param {string=} category to log in
*/
logError(message:string, category?:string): void;

/**
*  Log a message at "fatal" priority
*  
* 
* @param {string} message to log
* @param {string=} category to log in
*/
logFatal(message:string, category?:string): void;

/**
*  Check whether a message logged at the given priority would be visible in the log.
*  
*  As with logDebug, category is defaulted to the current className. Use this method to avoid
*  putting together expensive log messages if they will never appear in the log.
* 
* 
* @param {LogPriority} priority level
* @param {string=} category to log in
*/
logIsEnabledFor(priority:LogPriority, category?:string): void;

/**
*  Check whether a message logged at "debug" priority would be visible in the log.
*  
*  As with logDebug, category is defaulted to the current className. Use this method to avoid
*  putting together expensive log messages if they will never appear in the log.
* 
* 
* @param {string=} category to log in
*/
logIsDebugEnabled(category?:string): void;

/**
*  Check whether a message logged at "info" priority would be visible in the log.
*  
*  As with logDebug, category is defaulted to the current className. Use this method to avoid
*  putting together expensive log messages if they will never appear in the log.
* 
* 
* @param {string=} category to log in
*/
logIsInfoEnabled(category?:string): void;

/**
*  Check whether a message logged at "warn" priority would be visible in the log.
*  
*  As with logDebug, category is defaulted to the current className. Use this method to avoid
*  putting together expensive log messages if they will never appear in the log.
* 
* 
* @param {string=} category to log in
*/
logIsWarnEnabled(category?:string): void;

/**
*  Check whether a message logged at "error" priority would be visible in the log.
*  
*  As with logDebug, category is defaulted to the current className. Use this method to avoid
*  putting together expensive log messages if they will never appear in the log.
* 
* 
* @param {string=} category to log in
*/
logIsErrorEnabled(category?:string): void;

/**
*  Set the priority of messages that will be visible for some log category, when logged on
*  this Class or Instance object.
*  If called with no category, this priority will be applied to every logged message on this
*  object
*  To set the visible log priority for some category across the entire page, use
*  isc.Log.setPriority() instead.
* 
* @param {string} Category for which the log priority will be updated. If not             all logs on this canvas will be logged at the priority passed in.
* @param {LogPriority} priority level
*/
setLogPriority(category:string, priority:LogPriority): void;

/**
*  Set the default priority of logging for messages logged on this Class or Instance object.
*  All categories for which there is no explicit, instance level logging priority set will
*  log at this level on this object. 
*  To set the default visible log priority across the entire page, use
*  isc.Log.setDefaultPriority() instead.
* 
* @param {string} Category for which the log priority will be updated. If not             all logs on this canvas will be logged at the priority passed in.
* @param {LogPriority} priority level
*/
setDefaultLogPriority(category:string, priority:LogPriority): void;

/**
*  Retrieves the default priority of messages for this class or instance.
* 
*/
getDefaultLogPriority(): LogPriority;

/**
*  Clear this object's priority setting for a particular category, so that the category's 
*  effective priority returns to the specified priority for this category at the Log level
*  (or Log.defaultPriority if not set).
*  To clear the Page-level priority setting for this log category use 
*  isc.Log.clearPriority() instead.
* 
* 
* @param {string} Category name. If not specified, all logging on this object               will revert to default priority settings.
*/
clearLogPriority(category:string): void;



}
/**
* Creates a new Class
* 
* @param typeCheckedProperties {Partial<Class>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Class} The new instance
*/
class Class{
static create(typeCheckedProperties? : Partial<Class>, uncheckedProperties?: Object): Class;
/**
* Is this a core SmartClient class (part of the SmartClient framework)?
*  This attribute may be used for debugging, and by the AutoTest subsystem to 
*  differentiate between SmartClient classes (part of the smartClient framework) and
*  subclasses created by specific applications
* @type {boolean}
* @default "varies"
*/
static isFrameworkClass?: boolean;





/**
*  Notifies the SmartClient Class system that any new classes created, or changes made 
*  to existing classes should be treated as part of the framework. This ensures that
*  Class.isFrameworkClass will be set to true on any classes defined after this 
*  method call, until Class.modifyFrameworkDone is called.
*  
*  Developers may call this method before applying changes which should be considered
*  part of the core framework, rather than application code, for example in load_skin.js 
*  files. When changes are complete, Class.modifyFrameworkDone should be called.
*  Note that this is an alternative approach to calling Class.markAsFrameworkClass 
*  directly on specific classes.
* 
* 
*/
static modifyFrameworkStart(): void;

/**
*  Notifies the SmartClient Class system that the developer is done making changes
*  to the SmartClient framework (as originally indicated by a call to 
*  Class.modifyFrameworkStart).
*  
*  New classes created or changes made to existing classes after this method call
*  will be considered application code. This ensures that 
*  Class.isFrameworkClass will not be set to true on Classes defined after
*  this method call.
* 
*/
static modifyFrameworkDone(): void;

/**
*  Replaces each of the methods named in methodNames with a new implementation
*  that simply logs a warning the first time the method is called, and nothing else. This can
*  be used to mark methods of derived classes which do not support certain parent class
*  methods as unsupported.
*  
*  The messageTemplate parameter is a template for the warning message logged
*  when the unsupported method is first called. The following variables in the template
*  are substituted as follows:
*  
*  
*   Variable
*   Substitution
*  
*  
*   $class
*   The Class.getClassName.
*  
*  
*   $method
*   The name of the method.
*  
*  
*  
*  If you want the literal string of a substitution variable to appear in the warning message,
*  you can escape it by prefixing with a dollar sign. For example, to include "$class" in the
*  warning message, use "$$class" in the template.
* 
* @param {string} template for the warning message logged when first called. If null, the default template string "$class does not support the $method() method." is used.
* @param {Array<Partial<Identifier>>} the method names to mark as unsupported.
*/
static markUnsupportedMethods(messageTemplate:string, methodNames:Array<Partial<Identifier>>): void;

/**
*  Returns true if the method is supported by this class, meaning that it is not null and
*  was not replaced by Class.markUnsupportedMethods.
* 
* @param {Identifier} the name of a method to test.
*/
static isMethodSupported(methodName:Identifier): boolean;

/**
* 
* 	Gets the name of this class as a string.
* 
* 
*/
static getClassName(): string;

/**
* 	
* 	Gets a pointer to the superClass' Class object.
* 
* 
*/
static getSuperClass(): Class;

/**
* 
*  Helper method for adding method definitions to all instances of this class.
* 
*  The added methods can be called as myInstance.method().
* 
*  Functionally equivalent to Class.addProperties, which works with both properties
*  and methods.
* 
* 
* @param {object=} objects with methods to add (think named parameters).                 all the methods of each argument will be applied                  as instance-level methods.
*/
static addMethods(...args:any[]): object;

/**
* 
* 	Register a method, or set of methods, that can be provided to instances of this class as
* 	Strings (containing a JavaScript expression) and will be automatically converted into
* 	functions.
*  
*  For example:
*  
*  isc.MyClass.registerStringMethods({
*    myStringMethod: "arg1, arg2"
*  });
*  
* 
*  Note that registered stringMethods are eligible as target methods for Actions
*  declared in Component XML files. See the Declarative Actions section of the
*  componentXML.
* 
* 
* @param {object} If this is a string, name of the property to register                 If this is an object, assume passing in a set of name/value                  pairs to register
* @param {string} named arguments for the property in a comma separated string                 (not used if methodName is an object)
*/
static registerStringMethods(methodName:object, argumentString:string): void;

/**
*  Evaluate a string of script and return the result.  
*  
*  This method is a wrapper around the native javascript method eval(). It
*  papers over some native issues to ensure evaluation of script behaves consistently across
*  browsers
* 
* 
* @param {string} the expression to be evaluated
* @param {object} Optional mapping of argument names to values - each key will   be available as a local variable when the script is executed.
*/
static evaluate(expression:string, evalArgs:object): any;

/**
* 
* 	Add static (Class-level) properties and methods to this object
* 
* 	These properties can then be accessed as MyClass.property, or for functions, called as
*  MyClass.methodName()
* 
* 
* @param {object=} objects with properties to add (think named parameters).     all the properties of each argument will be applied     	as class-level properties.
*/
static addClassProperties(...args:any[]): object;

/**
*  Mark this class as a framework class (member of the SmartClient framework).
*  Sets Class.isFrameworkClass. May be used in debugging and by the
*  AutoTest subsystem
* 
*/
static markAsFrameworkClass(): void;

/**
* 
* 	Add default properties and methods to all instances of this class.
* 
* 	These properties can then be accessed as myInstance.property, 
*  and methods can be called via myInstance.methodName()
* 
* 
* @param {object=} objects with properties to add (think named parameters).     all the properties of each argument will be applied
*/
static addProperties(...args:any[]): object;

/**
* 
* 	Add default properties to all instances of this class
* 
* 
* @param {Array<Partial<object>>} array of objects with properties to add
*/
static addPropertyList(list:Array<Partial<object>>): object;

/**
*  
*  Changes a set of defaults defined as a JavaScript Object. For these kind of properties,
*  simply calling Class.addProperties would replace the original Object
*  with yours, wiping out settings required for the basic functionality of the component.
*  This method instead applies your overrides over the existing properties, without
*  destroying non-overridden properties.
*  
*  For example let's say you have a component that's defined as follows
*  
*  isc.defineClass("MyComponent");
*  isc.MyComponent.addProperties({
*    simpleProperty: "some value",
*    propertyBlock : {
*     foo: "bar",
*     zoo: "moo"
*    }
*  }
*  
*  If you wanted to override simpleProperty, you can just call Class.addProperties
*  like this:
*  
*  isc.MyComponent.addProperties({
*    simpleProperty: "my override"
*  });
*  
*  If you want to override the value of propertyBlock.moo above,
*  but you don't want to clobber the value of propertyBlock.zoo. If you use
*  the above pattern like so:
*  
*  isc.MyComponent.addProperties({
*    propertyBlock: {
*      foo: "new value",
*      zoo: "moo"
*    }
*  });
*  
*  You need to re-specify the value of propertyBlock.zoo which you didn't want
*  to override. Failing to re-specify it would destroy the value.
*  
*  Instead of re-specifying the value, you can use this method to modify the value of
*  foo - like this:
*  
*  isc.MyComponent.changeDefaults("propertyBlock", {
*    foo: "new value"
*  });
*  
*  
*  See also the AutoChild system for information about standard sets of defaults
*  that are available for customization.
* 
* 
* @param {string} name of the property to change
* @param {object} overrides for defaults
*/
static changeDefaults(defaultsName:string, newDefaults:object): void;

/**
* 	Apply a set of properties to a class object, calling the appropriate setter class methods if
* 	any are found.
* 
* 
* @param {object=} objects with properties to add (think named parameters).     all the properties of each argument will be applied one after another     	so later properties will override
*/
static setProperties(...args:any[]): void;

/**
* 
*  Returns whether this class object is the provided class or is a subclass of the provided
*  class, or implements the provided interface.
* 
* 
* @param {string} Class name to test against
*/
static isA(className:string): boolean;

/**
* 
* 	Call the SuperClass implementation of a class method.
* 
* 
* @param {string} name of the superclass method to call
* @param {any | Array<any>} native "arguments" object, or array of                     arguments to pass to the Super call
* @param {any=} native "arguments" object, required if an Array is                 passed for the "args" parameter in lieu of the native                  arguments object
*/
static Super(methodName:string, args:any | Array<any>, nativeArgs?:any): any;

/**
* 
*  Call method on each item in argsList and return the Array of results.
* 
* 
* @param {string} Name of the method on this instance which should be called on each element of the Array
* @param {Array<any>} Array of items to call the method on
*/
static map(methodName:string, items:Array<any>): Array<any>;

/**
* 
* 	Gets a named property from the instance defaults for this object.
* 
* 
* @param {string} name of the property to return
*/
static getInstanceProperty(property:string): void;

/**
* 
* 	Sets a named property from the instance defaults for this object.
* 
* 
* @param {string} name of the property to return
* @param {any} value to set to
*/
static setInstanceProperty(property:string, value:any): void;

/**
* 
*  Fire some arbitrary action specified as a Callback.
*  Returns the value returned by the action.
*  
* 
* @param {Callback} Action to fire.
* @param {string=} Comma separated string of variable names. If the callback              passed in was a string of script, any arguments passed to the               callback will be available as local variables with these names.
* @param {Array<any>=} Array of arguments to pass to the method. Note that the number              of arguments should match the number of argNames.
* @param {object=} If specified the callback will be evaluated in the scope of this             object - the this keyword will be a pointer to this              target when the callback is fired.
*/
static fireCallback(callback:Callback, argNames?:string, args?:Array<any>, target?:object): any;

/**
*  This is a helper to delay a call to a method on some target by a specified
*  amount of time. Can be used to delay a call to a static method on this class by 
*  omitting the target parameter.
* 
* @param {string} name of the method to call
* @param {Array<any>=} array of arguments to pass to the method in question
* @param {number=} Number of ms to delay the call by - defaults to zero (so just pulls            execution of the method out of the current execution thread.
* @param {object=} Target to fire the method on - if unspecified assume this is             a call to a classMethod on this Class.
*/
static delayCall(methodName:string, arrayArgs?:Array<any>, time?:number, target?:object): string;

/**
* 
*  The default toString() for a ClassObject reports that you have a ClassObject and what class
*  it is.
* 
*/
static toString(): void;

/**
*  Returns a one-line summary of the current method call, showing method name and passed
*  arguments. 
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.getCallTrace(arguments)" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
*  
*  Note the arguments object is required in most cases for this method to function. In some
*  browsers, it can be derived automatically, but developers looking to debug on multiple platforms
*  should not rely on this.
* 
* 
* @param {any=} arguments object from the call to trace. If ommitted, where supported,  arguments will be derived from the calling function, or if this is not supported, the method  will not function.
*/
static getCallTrace(args?:any): void;

/**
*  Returns a "stack trace" - one line per method in the current call stack, showing the method
*  name and any parameters passed.
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.getStackTrace" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
*  
*  Platform Notes: In Mozilla Firefox, if Firebug is enabled, a stack trace will be logged
*  to the firebug console in addition to the standard stack trace string returned by
*  this method.
*  
*  In browsers other than Internet Explorer a complete stack trace may not be available - 
*  this occurs when a function is re-entrant (meaning it calls itself). In this case the
*  stack will terminate with text indicating where the recursive function call occurred.
*  
*  See debugging for further information information.
* 
* 
*/
static getStackTrace(): string;

/**
*  Return a very short (generally less than 40 characters) string representation of any object,
*  suitable for viewing by a developer for debugging purposes.
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.echoLeaf" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
* 
* 
* @param {any} object to echo
*/
static echoLeaf(obj:any): string;

/**
*  Return a short string representation of any object, suitable for viewing by a developer for
*  debugging purposes.
*  
*  If passed an object containing other objects, echo will not recurse into subobjects,
*  summarizing them instead via echoLeaf().
*  
*  NOTE: echo() is used to generate the output shown in the Log window when evaluating an
*  expression.
*  
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.echo()" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
*  
* 
* @param {any} object to echo
*/
static echo(obj:any): string;

/**
*  Like echo(), except that if passed an Array, echoAll() will echo() every element of the
*  Array.
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.echo()" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
* 
* 
* @param {any} object to echo
*/
static echoAll(obj:any): string;

/**
*  Log a message at "debug" priority
*  
*  A method named logPriority exists for each priority level, on every ISC Class and
*  instance of an ISC Class. Messages logged on a Class or instance have a default
*  category of the classname. Messages logged on an instance will also automatically
*  incorporate the instance ID. General best practice is to call logDebug() et al as
*  "this.logDebug" whenever "this" is an instance, or as "Log.logDebug" otherwise.
* 
* 
* @param {string} message to log
* @param {string=} category to log in
*/
static logDebug(message:string, category?:string): void;

/**
*  Log a message at "info" priority
*  
* 
* @param {string} message to log
* @param {string=} category to log in
*/
static logInfo(message:string, category?:string): void;

/**
*  Log a message at "warn" priority
*  
* 
* @param {string} message to log
* @param {string=} category to log in
*/
static logWarn(message:string, category?:string): void;

/**
*  Log a message at "error" priority
*  
* 
* @param {string} message to log
* @param {string=} category to log in
*/
static logError(message:string, category?:string): void;

/**
*  Log a message at "fatal" priority
*  
* 
* @param {string} message to log
* @param {string=} category to log in
*/
static logFatal(message:string, category?:string): void;

/**
*  Check whether a message logged at the given priority would be visible in the log.
*  
*  As with logDebug, category is defaulted to the current className. Use this method to avoid
*  putting together expensive log messages if they will never appear in the log.
* 
* 
* @param {LogPriority} priority level
* @param {string=} category to log in
*/
static logIsEnabledFor(priority:LogPriority, category?:string): void;

/**
*  Check whether a message logged at "debug" priority would be visible in the log.
*  
*  As with logDebug, category is defaulted to the current className. Use this method to avoid
*  putting together expensive log messages if they will never appear in the log.
* 
* 
* @param {string=} category to log in
*/
static logIsDebugEnabled(category?:string): void;

/**
*  Check whether a message logged at "info" priority would be visible in the log.
*  
*  As with logDebug, category is defaulted to the current className. Use this method to avoid
*  putting together expensive log messages if they will never appear in the log.
* 
* 
* @param {string=} category to log in
*/
static logIsInfoEnabled(category?:string): void;

/**
*  Check whether a message logged at "warn" priority would be visible in the log.
*  
*  As with logDebug, category is defaulted to the current className. Use this method to avoid
*  putting together expensive log messages if they will never appear in the log.
* 
* 
* @param {string=} category to log in
*/
static logIsWarnEnabled(category?:string): void;

/**
*  Check whether a message logged at "error" priority would be visible in the log.
*  
*  As with logDebug, category is defaulted to the current className. Use this method to avoid
*  putting together expensive log messages if they will never appear in the log.
* 
* 
* @param {string=} category to log in
*/
static logIsErrorEnabled(category?:string): void;

/**
*  Set the priority of messages that will be visible for some log category, when logged on
*  this Class or Instance object.
*  If called with no category, this priority will be applied to every logged message on this
*  object
*  To set the visible log priority for some category across the entire page, use
*  isc.Log.setPriority() instead.
* 
* @param {string} Category for which the log priority will be updated. If not             all logs on this canvas will be logged at the priority passed in.
* @param {LogPriority} priority level
*/
static setLogPriority(category:string, priority:LogPriority): void;

/**
*  Set the default priority of logging for messages logged on this Class or Instance object.
*  All categories for which there is no explicit, instance level logging priority set will
*  log at this level on this object. 
*  To set the default visible log priority across the entire page, use
*  isc.Log.setDefaultPriority() instead.
* 
* @param {string} Category for which the log priority will be updated. If not             all logs on this canvas will be logged at the priority passed in.
* @param {LogPriority} priority level
*/
static setDefaultLogPriority(category:string, priority:LogPriority): void;

/**
*  Retrieves the default priority of messages for this class or instance.
* 
*/
static getDefaultLogPriority(): LogPriority;

/**
*  Clear this object's priority setting for a particular category, so that the category's 
*  effective priority returns to the specified priority for this category at the Log level
*  (or Log.defaultPriority if not set).
*  To clear the Page-level priority setting for this log category use 
*  isc.Log.clearPriority() instead.
* 
* 
* @param {string} Category name. If not specified, all logging on this object               will revert to default priority settings.
*/
static clearLogPriority(category:string): void;




}

/**
* Form item for selecting a color via a pop-up ColorPicker.
*/
interface ColorItem extends TextItem {
/**
* Should the pick button icon be shown for choosing colors from a ColorPicker
* @type {boolean}
* @default true
*/
showPickerIcon?: boolean;

/**
* If ColorItem.showPickerIcon is true for this item, this property governs the
*  size of the picker icon. If unset, the picker icon will be sized as a square to fit in the
*  available height for the icon.
*  
*  Note that if spriting is being used, and the image to be displayed is specified 
*  using css properties such as background-image, background-size,
*  changing this value may result in an unexpected appearance as the image will not
*  scale.
*  Scaleable spriting can be achieved using the SCSpriteConfig format.
*  See the section on spriting in the skinning for 
*  further information.
* @type {Integer}
* @default "18"
*/
pickerIconWidth?: Integer;

/**
* If ColorItem.showPickerIcon is true for this item, this property governs the
*  size of the picker icon. If unset, the picker icon will be sized as a square to fit in the
*  available height for the icon.
*  
*  Note that if spriting is being used, and the image to be displayed is specified 
*  using css properties such as background-image, background-size,
*  changing this value may result in an unexpected appearance as the image will not
*  scale.
*  Scaleable spriting can be achieved using the SCSpriteConfig format.
*  See the section on spriting in the skinning for 
*  further information.
* @type {Integer}
* @default "18"
*/
pickerIconHeight?: Integer;

/**
* If ColorItem.showPickerIcon is true for this item, this property governs the
*  FormItemIcon.src of the picker icon image to be displayed.
*  
*  skinning can be used for this image, by setting this property to
*  a SCSpriteConfig formatted string.
* @type {SCImgURL}
* @default "[SKIN]/DynamicForm/ColorPicker_icon.png"
*/
pickerIconSrc?: SCImgURL;

/**
* Prompt to show when the user hovers the mouse over the picker icon.
* @type {HTMLString}
* @default "Click to select a new color"
*/
pickerIconPrompt?: HTMLString;

/**
* The ColorPicker.defaultPickMode for the ColorPicker associated
*  with this ColorItem.
* @type {ColorPickerMode}
* @default "simple"
*/
defaultPickerMode?: ColorPickerMode;

/**
* Should "complex" mode be allowed for the ColorPicker window associated with 
*  this ColorItem?
*  If false, no "More" button is shown on the simple picker
* @type {boolean}
* @default true
*/
allowComplexMode?: boolean;

/**
* Determines whether the ColorPicker associated with this ColorItem allows the user 
*  to set transparency/opacity information whilst selecting a color. If false, no opacity
*  slider is shown and all colors are 100% opaque.
*  When an opacity value is selected, the HTML color string produced is 8 characters long
*  because the opacity setting is included.
*  You can also capture the the separate color and opacity information by overriding 
*  ColorItem.pickerColorSelected.
* @type {boolean}
* @default true
*/
supportsTransparency?: boolean;



/**
*  Store the color value selected by the user from the color picker. You will need to 
*  override this method if you wish to capture opacity information from the
*  ColorPicker.
* 
* @param {string} The selected color as a string.
* @param {Integer} The selected opacity, from 0 (transparent) to 100 (opaque),             or null if ColorItem.supportsTransparency is false or the              ColorPicker selected a color while in              ColorPickerMode.
*/
pickerColorSelected(color:string, opacity:Integer): void;



}
/**
* Creates a new ColorItem
* 
* @param typeCheckedProperties {Partial<ColorItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ColorItem} The new instance
*/
class ColorItem{
static create(typeCheckedProperties? : Partial<ColorItem>, uncheckedProperties?: Object): ColorItem;







}

/**
* The ColorPicker widget allows the user to select a color from anywhere in the 
*  color spectrum. It also supports selecting the alpha (opacity) value of the 
*  color. The picker supports a simple mode - which allows for one-click selection
*  from a standard palette of colors - and a complex mode which allow the user to
*  define any conceivable color. It is possible for the user to switch from simple
*  mode to complex by interacting with the widget. In general, the widget provides
*  very similar functionality to the color picker dialogs found in graphics packages
*  and other desktop software.
*/
interface ColorPicker extends Window {
/**
* "OK" button for the ColorPicker
* @type {IButton}
* @default null
*/
okButton?: IButton;

/**
* Should the OK button be visible. Set to false to hide the OK button.
* @type {boolean}
* @default true
*/
showOkButton?: boolean;

/**
* The class of the "OK" button. It is intended that you use either IButton or 
*    Button - other classes are unlikely to work correctly.
* @type {Class}
* @default "IButton"
*/
okButtonConstructor?: Class;

/**
* Cancel button for the ColorPicker
* @type {IButton}
* @default null
*/
cancelButton?: IButton;

/**
* Should the Cancel button be visible. Set to false to hide the Cancel button.
* @type {boolean}
* @default true
*/
showCancelButton?: boolean;

/**
* The class of the Cancel button. It is intended that you use either IButton or 
*    Button - other classes are unlikely to work correctly.
* @type {Class}
* @default "IButton"
*/
cancelButtonConstructor?: Class;

/**
* "More"/"Less" button for the ColorPicker
* @type {IButton}
* @default null
*/
modeToggleButton?: IButton;

/**
* Should the Mode Toggle button be visible. Set to false to hide the Mode Toggle button.
* @type {boolean}
* @default true
*/
showModeToggleButton?: boolean;

/**
* The class of the mode toggle button. It is intended that you use either IButton or 
*    Button - other classes are unlikely to work correctly.
* @type {Class}
* @default "IButton"
*/
modeToggleButtonConstructor?: Class;

/**
* The default color. This is the color that is selected when the picker first loads
* @type {string}
* @default "#808080"
*/
defaultColor?: string;

/**
* Width and height of the basic color boxes (they are always square, and they are
*  all the same size).
* @type {number}
* @default "20"
*/
colorButtonSize?: number;

/**
* Base CSS style applied to the basic color boxes
* @type {CSSStyleName}
* @default "ColorChooserCell"
*/
colorButtonBaseStyle?: CSSStyleName;

/**
* Array of 40 HTML color strings, used to render the basic color selection boxes.
* @type {Array<Partial<string>>}
* @default "[...]"
*/
colorArray?: Array<Partial<string>>;

/**
* Displayed width of the color swatch image. The default width is approximately
*  that used by the Windows&#174; XP color picking window
* @type {number}
* @default "170"
*/
swatchWidth?: number;

/**
* Displayed height of the color swatch image. The default height is approximately
*  that used by the Windows&#174; XP color picking window
* @type {number}
* @default "170"
*/
swatchHeight?: number;

/**
* Width of the Luminosity bar
* @type {number}
* @default "15"
*/
lumWidth?: number;

/**
* Determines whether to show the opacity slider. This allows the user to select colors
*  with an alpha element (ie, semi-transparent colors). If this attribute is set to false,
*  no opacity slider is shown, and all colors are completely opaque.
* @type {boolean}
* @default true
*/
supportsTransparency?: boolean;

/**
* The text to show underneath the selected color box, so that it can 
*  be seen through semi-transparent colors. If you do not want such text, set 
*  this value to blank. This value is irrelevant if 
*  ColorPicker.supportsTransparency is false.
* @type {HTMLString}
* @default "Lorem ipsum"
*/
opacityText?: HTMLString;

/**
* The location of the color swatch image file
* @type {SCImgURL}
* @default "[SKIN]ColorPicker/spectrum.png"
*/
swatchImageURL?: SCImgURL;

/**
* The location of the crosshair image file
* @type {SCImgURL}
* @default "[SKIN]ColorPicker/crosshair.png"
*/
crosshairImageURL?: SCImgURL;

/**
* The title for the button that switches to a less complex view.
* @type {HTMLString}
* @default "&lt;&lt; Less"
*/
lessButtonTitle?: HTMLString;

/**
* The title for the button that switches to a more complex view.
* @type {HTMLString}
* @default "More &gt;&gt;"
*/
moreButtonTitle?: HTMLString;

/**
* The label shown above the basic color blocks.
* @type {HTMLString}
* @default "Basic Colors:"
*/
basicColorLabel?: HTMLString;

/**
* The label shown next to the selected color box.
* @type {HTMLString}
* @default "Selected Color:"
*/
selectedColorLabel?: HTMLString;

/**
* The label shown next to the opacity slider. Ignored if 
*  ColorPicker.supportsTransparency is false.
* @type {HTMLString}
* @default "Opacity:"
*/
opacitySliderLabel?: HTMLString;

/**
* The initial opacity value for the component, as a percentage value between 0 and 100
* @type {number}
* @default "100"
*/
defaultOpacity?: number;

/**
* The title for the 'Red' field in the complex chooser.
* @type {HTMLString}
* @default "Red"
*/
redFieldTitle?: HTMLString;

/**
* The text to show when the mouse hovers over the 'Red' field in the complex chooser.
* @type {HTMLString}
* @default "The Red component of the selected color"
*/
redFieldPrompt?: HTMLString;

/**
* The title for the 'Green' field in the complex chooser.
* @type {HTMLString}
* @default "Green"
*/
greenFieldTitle?: HTMLString;

/**
* The text to show when the mouse hovers over the 'Green' field in the complex chooser.
* @type {HTMLString}
* @default "The Green component of the selected color"
*/
greenFieldPrompt?: HTMLString;

/**
* The title for the 'Blue' field in the complex chooser.
* @type {HTMLString}
* @default "Blue"
*/
blueFieldTitle?: HTMLString;

/**
* The text to show when the mouse hovers over the 'Blue' field in the complex chooser.
* @type {HTMLString}
* @default "The Blue component of the selected color"
*/
blueFieldPrompt?: HTMLString;

/**
* The title for the 'HTML' field in the complex chooser.
* @type {HTMLString}
* @default "HTML"
*/
htmlFieldTitle?: HTMLString;

/**
* The text to show when the mouse hovers over the 'HTML' field in the complex chooser.
* @type {HTMLString}
* @default "The selected color''s HTML coding"
*/
htmlFieldPrompt?: HTMLString;

/**
* The title for the 'Hue' field in the complex chooser.
* @type {HTMLString}
* @default "Hue"
*/
hueFieldTitle?: HTMLString;

/**
* The text to show when the mouse hovers over the 'Hue' field in the complex chooser.
* @type {HTMLString}
* @default "The Hue (base tone) of the selected color"
*/
hueFieldPrompt?: HTMLString;

/**
* The title for the 'Sat' field in the complex chooser.
* @type {HTMLString}
* @default "Sat"
*/
satFieldTitle?: HTMLString;

/**
* The text to show when the mouse hovers over the 'Saturation' field in the complex chooser.
* @type {HTMLString}
* @default "The Saturation (color purity) of the selected color"
*/
satFieldPrompt?: HTMLString;

/**
* The title for the 'Luminosity' field in the complex chooser.
* @type {HTMLString}
* @default "Lum"
*/
lumFieldTitle?: HTMLString;

/**
* The text to show when the mouse hovers over the 'Luminosity' field in the complex chooser.
* @type {HTMLString}
* @default "The Luminosity (brightness) of the selected color"
*/
lumFieldPrompt?: HTMLString;

/**
* The title for the 'OK' button.
* @type {HTMLString}
* @default "OK"
*/
okButtonTitle?: HTMLString;

/**
* The title for the 'Cancel' button.
* @type {HTMLString}
* @default "Cancel"
*/
cancelButtonTitle?: HTMLString;

/**
* If true, causes the ColorPicker to appear near where the mouse was last clicked.
*  If false, the ColorPicker is centered on first show; depending on the value of 
*  ColorPicker.autoCenterOnShow, it either reappears wherever it was last shown after hide/show(), 
*  or centered regardless of where it was last shown.
* @type {boolean}
* @default true
*/
autoPosition?: boolean;

/**
* If ColorPicker.autoPosition is false, this property controls whether to automatically center the
*  colorPicker every time it is redisplayed with the show() method.
* @type {boolean}
* @default true
*/
autoCenterOnShow?: boolean;

/**
* When this property is set to true, the ColorPicker will automatically hide when
*  a color has been selected using the swatch picker, even in "complex" mode. By default it will
*  only hide the ColorPicker in "simple" defaultPickMode.
*  
*  Set this property to false to disable the ColorPicker from automatically hiding,
*  this can be especially useful when for instance embedding this component inside another component.
* @type {boolean}
* @default null
*/
autoHide?: boolean;

/**
* The ColorPicker can operate in either a "simple" mode (where it displays just the
*  40 basic colors and allows the user to click one), or a "complex" mode (where the
*  user can specify a color from anywhere in the spectrum, with an optional alpha
*  element). The defaultPickMode attribute specifies which of these two modes is
*  in force when the picker first loads.
* @type {ColorPickerMode}
* @default "simple"
*/
defaultPickMode?: ColorPickerMode;

/**
* Should the "complex" mode be allowed for this ColorPicker?
*  If false, no "More" button is shown on the simple picker
* @type {boolean}
* @default true
*/
allowComplexMode?: boolean;



/**
*  Set the ColorPicker.supportsTransparency flag.
* 
* @param {boolean} Set to true to enable transparency/opacity
*/
setSupportsTransparency(transparencyFlag:boolean): void;

/**
*  Returns the Red element of the currently-selected color, as an integer from 0-255
* 
*/
getRed(): Integer;

/**
*  Returns the Green element of the currently-selected color, as an integer from 0-255
* 
*/
getGreen(): Integer;

/**
*  Returns the Blue element of the currently-selected color, as an integer from 0-255
* 
*/
getBlue(): Integer;

/**
*  Returns the Hue of the currently-selected color, as an integer from 0-239
* 
*/
getHue(): Integer;

/**
*  Returns the Saturation of the currently-selected color, as an integer from 0-240
* 
*/
getSaturation(): Integer;

/**
*  Returns the Luminosity (brightness) of the currently-selected color, as an 
*  integer from 0-240
* 
*/
getLuminosity(): Integer;

/**
*  Returns the currently-selected color, in HTML color representation form, as a string.
*  HTML color representation is a hash sign, followed by the red, green and blue elements
*  of the color in 2-digit hex form - for example "#F17F1D"
* 
*/
getHtmlColor(): string;

/**
*  Returns the opacity of the currently-selected color, as an integer from 0-100. If 
*  opacity is switched off, this is always 100.
* 
*/
getOpacity(): Integer;

/**
*  Sets the Red element of the selected color
* 
* @param {number} An integer between 0 and 255
*/
setRed(newValue:number): void;

/**
*  Sets the Green element of the selected color
* 
* @param {number} An integer between 0 and 255
*/
setGreen(newValue:number): void;

/**
*  Sets the Blue element of the selected color
* 
* @param {number} An integer between 0 and 255
*/
setBlue(newValue:number): void;

/**
*  Sets the Hue of the selected color
* 
* @param {number} An integer between 0 and 239
*/
setHue(newValue:number): void;

/**
*  Sets the Saturation of the selected color
* 
* @param {number} An integer between 0 and 240
*/
setSaturation(newValue:number): void;

/**
*  Sets the Luminosity (brightness) of the selected color
* 
* @param {number} An integer between 0 and 240
*/
setLuminosity(newValue:number): void;

/**
*  Changes the selected color to the one represented by the supplied HTML color 
*  string. Note that the method only accepts the parameter if it represents a 
*  valid color (otherwise it is simply ignored).
* 
* @param {string} A string in HTML color representation format (#RRGGBB)
*/
setHtmlColor(newValue:string): void;

/**
*  Sets the Opacity of the selected color. Ignored if opacity is switched off.
* 
* @param {number} An integer between 0 and 100
*/
setOpacity(newValue:number): void;

/**
*  Changes the pick mode of this ColorPicker to pickMode.
*  
*  Note: It is not allowed to set the pick mode to
*  "complex"
*  
*  if ColorPicker.allowComplexMode is false.
* 
* @param {ColorPickerMode} the new pick mode.
*/
setCurrentPickMode(pickMode:ColorPickerMode): void;

/**
*  Override this method to be kept informed when the ColorPicker changes in real-time 
*  (for example, if you need to update your own GUI accordingly). Then use the 
*  getXxxx() methods (for example, ColorPicker.getBlue or 
*  ColorPicker.getLuminosity)to obtain current state as required. 
* 
*/
colorChanged(): void;

/**
*  Override this method to be notified when the user selects a color
*  either by clicking a basic color box in simple mode, or by clicking 
*  the OK button in complex mode. It is not intended that client code 
*  call this method. The ColorPicker may automatically hide
*  itself after calling this method depending on ColorPicker.autoHide and
*  ColorPicker.defaultPickMode.
* 
* @param {string} The color selected, in HTML format.
* @param {Integer} The selected opacity, from 0 (transparent) to 100 (opaque),             or null if ColorPicker.supportsTransparency is false or the picker              selected a color while in ColorPickerMode.
*/
colorSelected(color:string, opacity:Integer): void;



}
/**
* Creates a new ColorPicker
* 
* @param typeCheckedProperties {Partial<ColorPicker>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ColorPicker} The new instance
*/
class ColorPicker{
static create(typeCheckedProperties? : Partial<ColorPicker>, uncheckedProperties?: Object): ColorPicker;




/**
*  Returns the shared global ColorPicker. 
*  Many applications will only need one ColorPicker instance; for such use 
*  cases, it is a good idea to use the shared object for performance reasons.
*  
*  The optional second parameter to this method indicates whether the shared picker
*  should retain the state (mode, color and opacity) it was in last time it was used, 
*  or revert to defaults.
*  Generally, you will want the picker to revert to default state; this gives the
*  same user experience as creating a new instance without incurring the overhead.
*  However, some use cases will benefit from the picker remembering what the user
*  did last time.
* 
* @param {object} Properties to apply to the global ColorPicker object
* @param {boolean=} Should we keep the current state?                  If false (or not provided), revert to default state
*/
static getSharedColorPicker(properties:object, keepCurrentState?:boolean): void;




}

/**
* Form item for selecting a color via a pop-up ColorPicker. This is an alias of
*  ColorItem.
*/
interface ColorPickerItem extends ColorItem {




}
/**
* Creates a new ColorPickerItem
* 
* @param typeCheckedProperties {Partial<ColorPickerItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ColorPickerItem} The new instance
*/
class ColorPickerItem{
static create(typeCheckedProperties? : Partial<ColorPickerItem>, uncheckedProperties?: Object): ColorPickerItem;







}

/**
* The SmartClient system supports hierarchical data (also referred to as tree data
*  due to its "branching" organization) with:
*  
*   
* - the Tree class, which manipulates hierarchical data sets
*   
* - the TreeGrid widget class, which extends the ListGrid class to visually
*     present tree data in an expandable/collapsible format.
*   
* - the ColumnTree widget class, which visually
*     presents tree data in a so-called 
*     "Miller Column" format.
*  
*  For information on DataBinding Trees, see treeDataBinding.
*  
*  A ColumnTree shows a single branch of the underlying Tree horizontally, from 
*  left to right. Thus, the leftmost column shows all the top-level nodes. When the user clicks
*  one of those nodes, a new column is shown immediately to the right of the top-level column,
*  showing the selected node's children. Additional columns are shown as required to present
*  lower-level children. The behavior of ColumnTree is similar to that of the Browser interface
*  in the Apple&trade; iTunes&trade; application.
*/
interface ColumnTree extends Omit<Layout, 'setDragTracker'>, Omit<DataBoundComponent, 'fields'>  {
/**
* The DataSource that this component should bind to for default fields and for performing
*  DSRequest.
*  
*  Can be specified as either a DataSource instance or the String ID of a DataSource.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* A Tree object consisting of nested TreeNodes to 
*  display in this ColumnTree. 
*  The data property will typically not be explicitly specified for 
*  databound ColumnTrees, where the data is returned from the server via databound component
*  methods such as fetchData()
* @type {Tree}
* @default null
*/
data?: Tree;

/**
* For a ColumnTree that uses a DataSource, these properties will be passed to
*  the automatically-created ResultTree. This can be used for various customizations such as
*  modifying the automatically-chosen Tree.parentIdField.
* @type {Tree}
* @default null
*/
dataProperties?: Tree;

/**
* If true, when this component is first drawn, automatically call this.fetchData().
*  Criteria for this fetch may be picked up from ColumnTree.initialCriteria, and textMatchStyle may
*  be specified via ListGrid.autoFetchTextMatchStyle.
*  
*  NOTE: if autoFetchData is set, calling
*  ListGrid.fetchData before draw will cause two requests to be issued, one from the manual
*  call to fetchData() and one from the autoFetchData setting. The second request will use
*  only ColumnTree.initialCriteria and not any other criteria or settings from the first request.
*  Generally, turn off autoFetchData if you are going to manually call ListGrid.fetchData at any time.
* 
*  Note: If you are using saved searches - either via SavedSearchItem or ListGrid.saveDefaultSearch, autoFetchData
*  will be automatically suspended and replaced with the saved criteria/view state, if applicable.
* @type {boolean}
* @default false
*/
autoFetchData?: boolean;

/**
* If ColumnTree.autoFetchData is true, this attribute allows the developer to
*  specify a textMatchStyle for the initial ListGrid.fetchData call.
* @type {TextMatchStyle}
* @default null
*/
autoFetchTextMatchStyle?: TextMatchStyle;

/**
* Criteria to be used when ColumnTree.autoFetchData is set.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
initialCriteria?: Criteria;

/**
* Criteria that are never shown to or edited by the user and are cumulative with any criteria
*  provided via DataBoundComponent.initialCriteria and related methods.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
implicitCriteria?: Criteria;

/**
* Whether to show fields marked detail:true when a DataBoundComponent is 
*  given a DataSource but no component.fields.
*  
*  The detail property is used on DataSource fields to mark fields that 
*  shouldn't appear by default in a view that tries to show many records in a small space.
* @type {boolean}
* @default true
*/
showDetailFields?: boolean;

/**
* How to fetch and manage records retrieve from the server. See FetchMode.
*  
*  This setting only applies to the ResultSet automatically created by calling
*  ListGrid.fetchData. If a pre-existing ResultSet is passed to setData() instead, it's
*  existing setting for ResultSet.fetchMode applies.
* @type {FetchMode}
* @default "paged"
*/
dataFetchMode?: FetchMode;

/**
* Enables fixed columns mode. All columns are created in advance instead of as navigation occurs.
* @type {Integer}
* @default null
*/
fixedColumns?: Integer;

/**
* With ColumnTree.fixedColumns enabled, defines the pixel or % width per column.
* @type {Array<Partial<string>>}
* @default null
*/
columnWidths?: Array<Partial<string>>;

/**
* With ColumnTree.fixedColumns enabled, defines the header title for each column.
* @type {Array<Partial<string>>}
* @default null
*/
columnTitles?: Array<Partial<string>>;

/**
* With ColumnTree.fixedColumns enabled, defines each column's ListGrid.emptyMessage.
* @type {Array<Partial<string>>}
* @default null
*/
emptyColumnMessages?: Array<Partial<string>>;

/**
* The URL of the base icon for all folder nodes in this columnTree. Note that this URL will
*  have ColumnTree.openIconSuffix or ColumnTree.closedIconSuffix appended to 
*  indicate state changes if appropriate - see documentation on ColumnTree.showOpenIcons
* @type {SCImgURL}
* @default "[SKIN]folder.png"
*/
folderIcon?: SCImgURL;

/**
* This property allows the developer to customize the icon displayed next to a node.
*  Set node[grid.customIconProperty] to the URL of the desired icon to display and
*  it will be shown instead of the standard ColumnTree.nodeIcon for this node.
*  Note that if ColumnTree.showCustomIconOpen 
*  is true for this grid, customized icons for folder nodes will be appended with the 
*  ColumnTree.openIconSuffix suffix on state change,
*  as with the standard ColumnTree.folderIcon. Also note that for
*  custom folder icons, the ColumnTree.closedIconSuffix will never be appended.
* @type {string}
* @default "icon"
*/
customIconProperty?: string;

/**
* The filename of the default icon for all leaf nodes in this grid. To specify a 
*  custom image for an individual node, set the ColumnTree.customIconProperty directly on
*  the node.
*  
*  See TreeGrid.showNodeIcons and TreeGrid.showFolderIcons for details on suppressing 
*  display of icons
* @type {SCImgURL}
* @default "[SKIN]file.png"
*/
nodeIcon?: SCImgURL;

/**
* If ColumnTree.showOpenIcons is true, this suffix will be appended to the
*  ColumnTree.folderIcon for open folders in this grid.
* @type {string}
* @default "open"
*/
openIconSuffix?: string;

/**
* This suffix will be appended to the ColumnTree.folderIcon for closed folders.
*  If ColumnTree.showOpenIcons is set to false this suffix will also be
*  appended to open folders' icons.
* @type {string}
* @default "closed"
*/
closedIconSuffix?: string;

/**
* If true, show a different icon for open folders than closed folders.
*  This is achieved by appending the ColumnTree.openIconSuffix onto the 
*  ColumnTree.folderIcon URL [for example "[SKIN]/folder.gif" might be 
*  replaced by "[SKIN]/folder_open.gif".
*  Note If this property is set to false the same icon is shown for
*  open folders as for closed folders, unless a custom folder icon was specified. This will be
*  determined by ColumnTree.folderIcon plus the ColumnTree.closedIconSuffix.
* @type {boolean}
* @default true
*/
showOpenIcons?: boolean;

/**
* Should folder nodes showing custom icons (set via the ColumnTree.customIconProperty),
*  show open state images when the folder is opened.
*  If true, the ColumnTree.openIconSuffix will be appended to the image URL
*  (so "customFolder.gif" might be replaced with 
*  "customFolder_open.gif").
*  Note that the ColumnTree.closedIconSuffix is never appended to custom folder icons.
*  Can be overridden at the node level via the default property TreeNode.showOpenIcon
*  and that property can be renamed via TreeGrid.customIconOpenProperty.
* @type {boolean}
* @default false
*/
showCustomIconOpen?: boolean;

/**
* This property allows the developer to rename the 
*  TreeNode.showOpenIcon property.
* @type {string}
* @default "showOpenIcon"
*/
customIconOpenProperty?: string;

/**
* When set to false, only displays a single column at a time, showing a slide animation 
*  when moving between columns.
* @type {boolean}
* @default null
*/
showMultipleColumns?: boolean;

/**
* For databound columnTree instances, should the entire tree of data be loaded on initial 
*  fetch, or should each column be loaded as needed. If unset the default 
*  ResultTree.loadDataOnDemand setting will be used.
* @type {boolean}
* @default null
*/
loadDataOnDemand?: boolean;

/**
* Instance of ListGrid used to display each column of the tree.
* @type {ListGrid}
* @default null
*/
column?: ListGrid;

/**
* Standard set of properties to apply to each generated ColumnTree.column
*  in this columnTree. Developers may also override ColumnTree.getColumnProperties
*  to return dynamic properties based on the node being displayed.
* @type {Partial<ListGrid>}
* @default null
*/
columnProperties?: Partial<ListGrid>;

/**
* An array of field objects, specifying the order, layout, dynamic calculation, and
*  sorting behavior of each field in each column in the columnTree object. In ColumnTrees, 
*  the fields array specifies sub-columns within each main column. 
*  Each field in the fields array is a ListGridField object.
*  
*  If ColumnTree.dataSource is also set, this value acts as a set of overrides as
*  explained in DataBoundComponent.fields.
* @type {Array<Partial<ListGridField>>}
* @default null
*/
fields?: Array<Partial<ListGridField>>;

/**
* If set, each column in the ColumnTree will show a header with the title of the selected
*  node from the column to the left.
* @type {boolean}
* @default false
*/
showHeaders?: boolean;

/**
* A title for the leftmost column if ColumnTree.showHeaders is set (the remaining columns
*  have their titles derived from the item selected in the column to the left). Ignored
*  if ColumnTree.showHeaders is not set.
*  Note: if you do not want a heading for the first column leave this attribute at its 
*  default value of " ". If you set it to null or the empty string, SmartClient will
*  fall back to displaying the field's name in the heading.
* @type {string}
* @default " "
*/
firstColumnTitle?: string;

/**
* If set, and ColumnTree.showHeaders is also set, each column's header will show 
*  a count of the number of nodes in that column
* @type {boolean}
* @default false
*/
showNodeCount?: boolean;

/**
* When using ColumnTree.showMultipleColumns, this i18n property
*  dictates the title for the ColumnTree.backButton docked to the top left 
*  which allows navigation back through the column tree.
* @type {string}
* @default "Back"
*/
backButtonTitle?: string;

/**
* When using ColumnTree.showMultipleColumns, this is the
*  "Back" button that you see hovering above the column UI and that allows backward
*  navigation.
* @type {IButton}
* @default null
*/
backButton?: IButton;

/**
* Indicates whether records can be dragged from this listGrid and dropped elsewhere.
*  
*  NOTE: If canDragRecordsOut is initially enabled or might be
*  dynamically enabled after the grid is created, it may be desirable to disable
*  Canvas.useTouchScrolling so that touch-dragging a record starts
*  a drag operation rather than a scroll, but see the discussion of 
*  ListGrid.showDragHandles. If 
*  Canvas.disableTouchScrollingForDrag is set to true, then touch
*  scrolling will be disabled automatically. However, for
*  accessibility reasons, it is recommended to leave touch
*  scrolling enabled and provide an alternative set of controls that can be used to perform
*  drag and drop of records out of the grid.
* @type {boolean}
* @default false
*/
canDragRecordsOut?: boolean;



/**
*  Only supported on ListGrid for now.
* 
* @param {Array<Partial<Hilite>>} Array of hilite objects
*/
setHilites(hilites:Array<Partial<Hilite>>): void;

/**
*  Uses a "fetch" operation on the current DataSource to retrieve data
*  that matches the provided criteria, and displays the matching data in this component as a
*  tree.
*  
*  This method will create a ResultTree to manage tree data, which will
*  subsequently be available as columnTree.data. DataSource records
*  returned by the "fetch" operation are linked into a tree structure according to
*  DataSourceField.primaryKey and
*  DataSourceField.foreignKey declarations on DataSource fields. See the
*  treeDataBinding topic for complete details.
*  
*  By default, the created ResultTree will use folder-by-folder load on demand, asking the
*  server for the children of each folder as the user opens it.
*  
*  The ResultTree created by fetchData() can be customized by setting
*  ColumnTree.dataProperties to an Object containing properties and methods to apply to
*  the created ResultTree. For example, the property that determines whether a node is a
*  folder (Tree.isFolderProperty) can be customized, or
*  level-by-level loading can be disabled via
*  ResultTree.loadDataOnDemand.
*  
*  The callback passed to fetchData will fire once, the first time that data is
*  loaded from the server. If using folder-by-folder load on demand, use the
*  ResultTree.dataArrived notification to be notified each time new nodes are loaded.
*  
*  Note that, if criteria are passed to fetchData(), they will be passed every
*  time a new "fetch" operation is sent to the server. This allows you to retrieve multiple
*  different tree structures from the same DataSource. However note that the server is expected
*  to always respond with an intact tree - returned nodes which do not have parents are dropped
*  from the dataset and not displayed.
* 
* 
* @param {Criteria=} Search criteria. If a DynamicForm is passed                     in as this argument instead of a raw criteria                      object, will be derived by calling                      DynamicForm.getValuesAsCriteria
* @param {DSCallback=} callback to invoke when a fetch is complete. Fires                     only if server contact was required
* @param {DSRequest=} additional properties to set on the DSRequest                     that will be issued
*/
fetchData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Retrieves data that matches the provided criteria and displays the matching data in this
*  component.
*  
*  This method behaves exactly like ListGrid.fetchData except that
*  DSRequest.textMatchStyle is automatically set to "substring" so that String-valued
*  fields are matched by case-insensitive substring comparison.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
* 
* @param {Criteria=} Search criteria.            If a DynamicForm is passed in as this argument            instead of a raw criteria object, will be derived by calling            DynamicForm.getValuesAsCriteria
* @param {DSCallback=} callback to invoke when a fetch is complete. Fires                     only if server contact was required; see                      ListGrid.fetchData for details
* @param {DSRequest=} for databound components only - optional              additional properties to set on the DSRequest that will be issued
*/
filterData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Get the selected record, that is, the parent of the nodes in the rightmost visible column.
*  
*  This is generally the most recently clicked node unless programmatic navigation has taken
*  place.
*  
*  If only the first column is showing, the root node is returned (which can be detected via
*  Tree.isRoot).
* 
* 
*/
getSelectedRecord(): Record;

/**
*  Navigate to the previous column.
* 
* 
*/
navigateBack(): void;

/**
*  Called when a node is selected in any column. Default behavior is to show the next level
*  of the tree in a column to the right of the current column.
*  
*  The new column will be created if it is not already showing. Any columns further to the
*  right, showing deeper levels of the tree, will be removed.
* 
* @param {ListGrid} the column where a node was selected
* @param {TreeNode} the node that was selected
*/
nodeSelected(column:ListGrid, node:TreeNode): boolean;

/**
*  Whether the indicated column should show a header. Returns this.showHeaders by default,
*  override for different behavior.
*  
* 
* @param {TreeNode} parent node for the nodes to be shown in the column
* @param {Integer} index of the column
*/
shouldShowHeader(node:TreeNode, colNum:Integer): void;

/**
*  Returns the title to show for the header of indicated column. Only called if
*  ColumnTree.shouldShowHeader returns true for this column.
*  
*  By default, returns ColumnTree.firstColumnTitle for the first column, and for subsequent
*  columns, the result of Tree.getTitle called on the
*  node passed to this function.
*  
* 
* @param {TreeNode} parent node for the nodes to be shown in the column
* @param {Integer} index of the column
*/
getColumnTitle(node:TreeNode, colNum:Integer): void;

/**
*  Retrieve a record by index. 
*  
*  If colNum is passed, returns the record found in that column at that index,
*  or null if the column doesn't exist or the index is too high.
*  
*  With no colNum parameter, a record's index is it's position counting from the
*  first record of the first column and including all records in each column. Note that both
*  index and colNum are zero-based - so the first column is column 0, not column 1.
*  
* 
* @param {Integer} index of record to return.
* @param {Integer=} optional index of the column
*/
getRecord(index:Integer, colNum?:Integer): TreeNode;

/**
* 
*  Returns the title to show for a node in the ColumnTree. The default implementation
*  returns the result of calling Tree.getTitle on the node.
*  
*  You can override this method to return a custom title for nodes in the tree.
* 
* 
* @param {TreeNode} The node for which the title is being requested.
* @param {number} The index of the node.
* @param {string} The field for which the title is being requested.
*/
getNodeTitle(node:TreeNode, recordNum:number, field:string): HTMLString;

/**
*  Returns the Tree object this ColumnTree is viewing
* 
* 
*/
getData(): void;

/**
*  Set the Tree object this ColumnTree will view
* 
* 
* @param {Tree} Tree to show
*/
setData(newData:Tree): void;

/**
*  Advanced API - get the ListGrid representing the indicated column.
* 
* @param {Integer | TreeNode} column number, or parent node of the nodes shown in the column
*/
getColumn(column:Integer | TreeNode): ListGrid;

/**
*  Additional properties to apply to the ListGrid that will show the indicated column.
*  Returns nothing by default. This method can be overridden to allow, for example, different 
*  styling, icons, or row heights per column.
* 
* @param {TreeNode} parent node for the nodes to be shown in the column
* @param {Integer} index of the column
*/
getColumnProperties(node:TreeNode, colNum:Integer): Partial<ListGrid>;

/**
* 
*  Process a drop of one or more nodes on a TreeGrid folder.
*  Note: See treeGridDrop for an overview of TreeGrid drag and drop behavior. 
*  
*  
*  This method can be overridden to provide custom drop behaviors and is a more appropriate
*  override point than the lower level Canvas.drop handler.
*  
*  
*  
*  The default behavior is to simply delegate to the ColumnTree.transferNodes method; thus, the 
*  correct way to perform a programmatic folder drop, with all the built-in behaviors described
*  below, is to call transferNodes()
*  
*  If this is a self-drop, nodes are simply reordered. An "update" operation will
*  be submitted to update the Tree.parentIdField field of the moved node(s). 
*  
*  For a drop from another widget, TreeGrid.transferDragData is called which,
*  depending on the TreeGrid.dragDataAction specified on the source
*  widget, may either remove the source nodes from the original list (dragDataAction:"move")
*  or just provide a copy to this tree (dragDataAction:"copy").
*  
*  In either case the new row(s) appear in the folder at the index
*  specified by the arguments of the same name.
*  
*  If this grid is databound, the new nodes will be added to the dataset by calling
*  DataSource.addData. Further, if the new nodes were dragged from another
*  databound component, and DataBoundComponent.addDropValues
*  is true, DataBoundComponent.getDropValues will be called for every item
*  being dropped.
*  
*  As a special case, if the sourceWidget is also databound and a
*  DataSourceField.foreignKey relationship is declared from the
*  sourceWidget's DataSource to this TreeGrid's DataSource, the interaction will
*  be treated as a "drag recategorization" use case such as files being placed in folders,
*  employees being assigned to teams, etc. "update" DSRequests will be submitted that
*  change the foreignKey field in the dropped records to point to the tree folder that was the
*  target of the drop. In this case no change will be made to the Tree data as such, only to
*  the dropped records. 
*  
*  For multi-record drops, Queuing is automatically used to combine all DSRequests into a
*  single HTTP Request (see QuickStart Guide, Server Framework chapter). This allows the
*  server to persist all changes caused by the drop in a single transaction (and this is
*  automatically done when using the built-in server DataSources with Power Edition and
*  above).
*  
*  If these default persistence behaviors are undesirable, 
*  return false to cancel them
*  , then implement your own behavior, 
*  typically by using grid.updateData() or addData() to add new records.
*  NOTE: the records you receive in this event are the actual Records from the source
*  component. Use DataSource.copyRecords to create a copy before modifying the records
*  or using them with updateData() or addData().
* 
* 
* @param {Array<Partial<TreeNode>>} List of nodes being dropped
* @param {TreeNode} The folder being dropped on
* @param {Integer} Within the folder being dropped on, the index at which the drop is          occurring. Only passed if TreeGrid.canReorderRecords is true.
* @param {Canvas} The component that is the source of the nodes (where the nodes               were dragged from)
*/
folderDrop(nodes:Array<Partial<TreeNode>>, folder:TreeNode, index:Integer, sourceWidget:Canvas): void;

/**
* 
*  Transfer a list of TreeNodes within this treeGrid or from from some other component 
*  (does not have to be a databound component) into this component. This method is only 
*  applicable to list-type components, such as ListGrid, 
*  TreeGrid or TileGrid. Please see the paragraph below for
*  special rules concerning Tree.isMultiLinkTree.
*  
*  This method implements the automatic drag-copy and drag-move behavior and calling it is
*  equivalent to completing a drag and drop of the nodes (the default 
*  ColumnTree.folderDrop implementation simply calls transferNodes())
*  
*  Note that this method is asynchronous - it may need to perform server turnarounds to prevent
*  duplicates in the target component's data. If you wish to be notified when the transfer
*  process has completed, you can either pass the optional callback to this method or implement
*  the DataBoundComponent.dropComplete method on this component.
*  
*  For a TreeGrid, see also TreeGrid.transferSelectedData.
*  
*  Multi-link trees
*  If both the target treeGrid and the sourceWidget for this transfer are 
*  multi-link treeGrids, the nodes parameter must be an array of 
*  NodeLocators rather than TreeNodes. Likewise, if the target (this) component
*  is a multi-link treeGrid, the folder parameter must be a NodeLocator.
*  
*  You can obtain a NodeLocator for a visible node occurence in a multi-link TreeGrid by 
*  calling its data model's Tree.getNodeLocator method.
* 
* 
* @param {Array<Partial<TreeNode>> | Array<Partial<NodeLocator>>} Nodes to transfer to this component
* @param {TreeNode} The target folder (eg, of a drop interaction), for context
* @param {Integer} Insert point within the target folder data for the transferred nodes
* @param {Canvas} The databound or non-databound component from which the nodes               are to be transferred.
* @param {Callback=} optional callback to be fired when the transfer process has            completed. The callback will be passed a single parameter "records",            the list of nodes actually transferred to this component (it is called            "records" because this is logic shared with ListGrid)
*/
transferNodes(nodes:Array<Partial<TreeNode>> | Array<Partial<NodeLocator>>, folder:TreeNode, index:Integer, sourceWidget:Canvas, callback?:Callback): void;

/**
* 
*  Select/deselect a Record passed in explicitly, or by index.
* 
* 
* @param {Record | number} record (or row number) to select
* @param {boolean=} new selection state (if null, defaults to true)
* @param {number=} Column number
*/
selectRecord(record:Record | number, newState?:boolean, colNum?:number): void;

/**
* 
*  Deselect a Record passed in explicitly, or by index.
*  
*  Synonym for selectRecord(record, false)
* 
* 
* @param {Record | number} record (or row number) to deselect
* @param {number=} Column number
*/
deselectRecord(record:Record | number, colNum?:number): void;

/**
* 
*  Select/deselect a list of Records passed in explicitly, or by index.
* 
* 
* @param {Array<Partial<Record>> | number} records (or row numbers) to select
* @param {boolean=} new selection state (if null, defaults to true)
* @param {number=} Column number
*/
selectRecords(records:Array<Partial<Record>> | number, newState?:boolean, colNum?:number): void;

/**
* 
*  Deselect a list of Records passed in explicitly, or by index.
*  
*  Synonym for selectRecords(records, false)
* 
* 
* @param {Array<Partial<Record>> | number} records (or row numbers) to deselect
* @param {number=} Column number
*/
deselectRecords(records:Array<Partial<Record>> | number, colNum?:number): void;

/**
*  Select all records in the supplied column (the first column if none is passed)
* 
* 
* @param {number=} Column number
*/
selectAllRecords(colNum?:number): void;

/**
*  Deselect all records in the supplied column (the first column if none is passed)
* 
* 
* @param {number=} Column number
*/
deselectAllRecords(colNum?:number): void;

/**
*  Whether at least one item is selected in the supplied column (the first column if none is passed)
* 
* 
* @param {number=} Column number
*/
anySelected(colNum?:number): boolean;



}
/**
* Creates a new ColumnTree
* 
* @param typeCheckedProperties {Partial<ColumnTree>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ColumnTree} The new instance
*/
class ColumnTree{
static create(typeCheckedProperties? : Partial<ColumnTree>, uncheckedProperties?: Object): ColumnTree;







}

/**
* The Combobox is a text input field which can show a list of options via a drop-down
*  PickList.
*  
*  The set of options will be filtered based on the current value in the text field, so only
*  options that match what has been typed so far will be displayed.
*  The set of options can be derived from a ValueMap or dynamically retrieved from a
*  dataSource. See the PickList interface for further settings.
*  
*  The two most common use cases for ComboBoxItems are:
*  
* - With ComboBoxItem.addUnknownValues set to true, the ComboBoxItem acts as a
*  freeform text entry field with the picklist providing essentially a set of suggested completions
*  similar to a URL bar in a web browser.
*    
* - With ComboBoxItem.addUnknownValues set to false, the ComboBoxItem acts similarly
*  to a SelectItem where a fixed set of options is available to the user and the text entry field
*  is essentially used to filter which of these options are visible
*  
*  Other commonly used settings to configure ComboBoxItem behavior are:
*  - ComboBoxItem.defaultToFirstOption - this will select the first option from the pickList
*  as a default value for the item - and
*  - ComboBoxItem.completeOnTab which causes the
*  current selection in the pickList (if there is one) to be chosen when the user tabs out of the
*  field, allowing a user to type a few characters and hit tab to auto-complete to the first matched
*  option. completeOnTab is automatically set to true if ComboBoxItem.addUnknownValues is 
*  false.
*  
*  ComboBoxItem does not provide built-in support for multiple selection. For a Combobox
*  that does provide such a multiple-select feature use MultiComboBoxItem.
*/
interface ComboBoxItem extends TextItem, PickList {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "SelectItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Static default value for this ComboBoxItem. To default to the first option use
*  ComboBoxItem.defaultToFirstOption instead.
* @type {any}
* @default null
*/
defaultValue?: any;

/**
* Select the first option as the default value for this ComboBoxItem. If options are derived
*  from a dataSource, the first value returned by the server will be used, otherwise the first
*  value in the valueMap. If enabled, this setting overrides ComboBoxItem.defaultValue
*  and ComboBoxItem.defaultDynamicValue.
* @type {boolean}
* @default false
*/
defaultToFirstOption?: boolean;

/**
* If showing a hint for this form item, should it be shown within the field?
*  CSS style for the hint is SelectItem.textBoxStyle with the suffix
*  "Hint" appended to it.
* @type {boolean}
* @default null
*/
showHintInField?: boolean;

/**
* Not applicable to a ComboBoxItem.
* @type {string}
* @default null
*/
mask?: string;

/**
* Not applicable to a ComboBoxItem.
* @type {boolean}
* @default null
*/
maskSaveLiterals?: boolean;

/**
* Not applicable to a ComboBoxItem.
* @type {string}
* @default " "
*/
maskPadChar?: string;

/**
* Not applicable to a ComboBoxItem.
* @type {string}
* @default "_"
*/
maskPromptChar?: string;

/**
* Not applicable to a ComboBoxItem.
* @type {boolean}
* @default null
*/
maskOverwriteMode?: boolean;

/**
* Controls where the PickList is placed. 
*  Can be specified as a PanelPlacement
*  or a specific widget that should be filled (by specifying an actual Canvas or
*  Canvas.ID).
*  
*  Default behavior is to "fillPanel" if Browser.isHandset or
*  Browser.isTablet, to better accommodate the smaller screen real estate and 
*  less precise
*  pointing ability on such devices.
*  
*  When filling the whole screen, part of the screen or a specific panel, the expanded
*  interface is created as a FormItem.picker, and 
*  incorporates a ComboBoxItem.pickerNavigationBar and 
*  ComboBoxItem.pickerExitButton that hides the expanded interface, as well 
*  as a separate ComboBoxItem.pickerSearchField.
* @type {PanelPlacement | Canvas | string}
* @default null
*/
pickListPlacement?: PanelPlacement | Canvas | string;

/**
* For PickList items with PickListItemIconPlacement set such that the pickList does
*  not render near-origin, should specified FormItem.icons be rendered inline within
*  the formItem itself, or within the ComboBoxItem.pickerNavigationBar.
*  
*  May be overridden at the icon level via FormItemIcon.iconPlacement.
*  
*  For mobile browsing with limited available screen space, icons rendered in
*  the navigation bar may be easier for the user to interact with.
* @type {PickListItemIconPlacement}
* @default "both"
*/
iconPlacement?: PickListItemIconPlacement;

/**
* The pickerSearchField is a separate TextItem created for 
*  search string entry when ComboBoxItem.pickListPlacement indicates that the search 
*  interface takes over an entire panel or the entire screen.
*  
*  The following autoChildUsage apply:
*  
*  
* - ComboBoxItem.pickerSearchFieldHint for FormItem.hint
*  
* @type {TextItem}
* @default null
*/
pickerSearchField?: TextItem;

/**
* FormItem.hint for the ComboBoxItem.pickerSearchField.
* @type {HTMLString}
* @default "Search"
*/
pickerSearchFieldHint?: HTMLString;

/**
* FormItem.hint for the ComboBoxItem.pickerSearchField when the combobox is configured to 
*  ComboBoxItem.addUnknownValues
* @type {HTMLString}
* @default "Search or enter new value"
*/
pickerSearchOrNewValueFieldHint?: HTMLString;

/**
* Form that contains the ComboBoxItem.pickerSearchField.
* @type {DynamicForm}
* @default null
*/
pickerSearchForm?: DynamicForm;

/**
* NavigationButton to dismiss the picker interface, created when 
*  ComboBoxItem.pickListPlacement indicates that the search interface takes over the
*  entire panel or screen.
*  
*  The following autoChildUsage apply:
*  
*  
* - ComboBoxItem.pickerExitButtonTitle for Button.title
*  
* @type {NavigationButton}
* @default null
*/
pickerExitButton?: NavigationButton;

/**
* The title for the ComboBoxItem.pickerExitButton.
* @type {HTMLString}
* @default "Cancel"
*/
pickerExitButtonTitle?: HTMLString;

/**
* "Accept" button for ComboBoxItem.addUnknownValues ComboBoxItems
*  showing the mobile interface.
*  
*  The pickerSaveButton is an automatically created NavigationButton autoChild
*  to dismiss the picker interface and store out the
*  value entered in the ComboBoxItem.pickerSearchField, created when 
*  ComboBoxItem.pickListPlacement indicates that the search interface takes over the
*  entire panel or screen.
*  
*  This button will only be shown when ComboBoxItem.addUnknownValues is true.
*  Note that if a user has entered a partial known value, the pickList
*  will show a filtered list of possible matches. An "Enter" keypress (or native keyboard
*  "Done" button click on a mobile browser keyboard) will select the first match from the 
*  list. The pickerSaveButton provides a way for users to explicitly use
*  the value as entered instead.
*  
*  The following autoChildUsage apply:
*  
*  
* - ComboBoxItem.pickerSaveButtonTitle for Button.title
*  
* @type {NavigationButton}
* @default null
*/
pickerSaveButton?: NavigationButton;

/**
* The title for the ComboBoxItem.pickerSaveButton.
* @type {HTMLString}
* @default "Accept"
*/
pickerSaveButtonTitle?: HTMLString;

/**
* NavigationButton to clear the picker value, created when ComboBoxItem.pickListPlacement
*  indicates that the search interface takes over the entire panel or screen.
*  
*  This button will only be shown if ComboBoxItem.addUnknownValues or ComboBoxItem.allowEmptyValue
*  is true.
*  
*  The following autoChildUsage apply:
*  
*  
* - ComboBoxItem.pickerClearButtonTitle for Button.title
*  
* @type {NavigationButton}
* @default null
*/
pickerClearButton?: NavigationButton;

/**
* The title for the ComboBoxItem.pickerClearButton.
* @type {HTMLString}
* @default "Clear"
*/
pickerClearButtonTitle?: HTMLString;

/**
* Empty message to display in the comboboxItem if PickList.hideEmptyPickList
*  is false.
* @type {string}
* @default "No items to show"
*/
emptyPickListMessage?: string;

/**
* NavigationBar created when ComboBoxItem.pickListPlacement indicates that the search
*  interface takes over the entire panel or screen.
* @type {NavigationBar}
* @default null
*/
pickerNavigationBar?: NavigationBar;

/**
* Indicates whether or not this ComboBoxItem will load its list of options 
*  DataSource.progressiveLoading. This property is copied onto the
*  underlying PickList.
* @type {boolean}
* @default true
*/
progressiveLoading?: boolean;

/**
* Should we show a special 'picker' FormItemIcon for this form item? Picker
*  icons are customizable via FormItem.pickerIconProperties. By default
*  they will be rendered inside the form item's FormItem.controlStyle
*  area. By default clicking the pickerIcon will call FormItem.showPicker.
* @type {boolean}
* @default true
*/
showPickerIcon?: boolean;

/**
* If ComboBoxItem.showPickerIcon is true for this item, this property governs the
*  size of the picker icon. If unset, the picker icon will be sized as a square to fit in the
*  available height for the icon.
*  
*  Note that if spriting is being used, and the image to be displayed is specified 
*  using css properties such as background-image, background-size,
*  changing this value may result in an unexpected appearance as the image will not
*  scale.
*  Scaleable spriting can be achieved using the SCSpriteConfig format.
*  See the section on spriting in the skinning for 
*  further information.
* @type {Integer}
* @default "15"
*/
pickerIconWidth?: Integer;

/**
* Don't specify an explicit height for the picker icon - instead have it size to match the
*  height of the combo box item.
* @type {Integer}
* @default null
*/
pickerIconHeight?: Integer;

/**
* If ComboBoxItem.showPickerIcon is true for this item, this property governs the
*  FormItemIcon.src of the picker icon image to be displayed.
*  
*  When skinning is enabled, this property will not 
*  be used to locate an image, instead, the image is drawn via CSS based on the 
*  FormItem.pickerIconStyle property.
* @type {SCImgURL}
* @default "[SKIN]/DynamicForm/ComboBoxItem_PickButton_icon.gif"
*/
pickerIconSrc?: SCImgURL;

/**
* Should the list of options be displayed whenever the user types into the 
*  combo-box textArea, or only when the user clicks on the pick button or uses the 
*  explicit Alt+Arrow Down key combination?
* @type {boolean}
* @default true
*/
showPickListOnKeypress?: boolean;

/**
* ComboBox items will submit their containing form on enter keypress 
*  if DynamicForm.saveOnEnter is true. Setting this property to
*  false will disable this behavior.
*  
*  Note that if the drop down list of options (pickList) is visible an
*  Enter keypress is used to select a value from the available set of
*  options and will not automatically cause form submission.
* @type {boolean}
* @default true
*/
saveOnEnter?: boolean;

/**
* If true, when the pickList is showing, the user can select the current value by hitting
*  the Tab key.
*  
*  Note that completeOnTab is not compatible with ComboBoxItem.formatOnBlur
* @type {boolean}
* @default null
*/
completeOnTab?: boolean;

/**
* If true, when the pickList is showing, the user can select the current value by hitting
*  the Enter key.
*  
*  If not explicitly set, completeOnEnter will default to false for items embedded
*  in a SearchForm, true otherwise.
* @type {boolean}
* @default null
*/
completeOnEnter?: boolean;

/**
* With formatOnBlur enabled, this comboBoxItem will format its value
*  according to the rules described in FormItem.mapValueToDisplay as long as the 
*  item does not have focus. Once the user puts focus into the item
*  the formatter will be removed. This provides a simple way for developers to
*  show a nicely formatted display value in a freeform text field, without the need
*  for an explicit FormItem.formatEditorValue 
*  and FormItem.parseEditorValue pair.
*  
*  Note that this attribute is not compatible with ComboBoxItem.completeOnTab
* @type {boolean}
* @default false
*/
formatOnBlur?: boolean;

/**
* A set of "special" values such as "All", "None" or "Invalid" that do not appear in the normal
*  ValueMap or in the data returned by the ComboBoxItem.optionDataSource.
*  
*  Like other uses of ValueMap, either a list of values or a mapping from stored to
*  display value can be provided.
*  
*  These values can either be shown at the top of the list of values (in the order specified), or
*  can be shown in a separate, non-scrolling region - the setting
*  SelectItem.separateSpecialValues controls this. Note that data 
*  paging can only be used if separateSpecialValues is enabled.
*  
*  If specialValues are configured, SelectItem.allowEmptyValue is ignored - an empty
*  value, if desired, must be included in the specialValues. To provide a
*  specialValue which clears the value of the field, use the special constant
*  PickList.emptyStoredValue.
*  
*  specialValues can also be used to take a value that does appear in the
*  normal data and redundantly display it at the top of the list to make it more accessible. Note
*  that in this case it is expected that the special value appears both at the top of the
*  list and in it's normal position in the list, so this works best with
*  SelectItem.separateSpecialValues mode enabled.
*  
*  Also, if an ComboBoxItem.optionDataSource is used, ComboBoxItem.specialValues that appear in the normal
*  dataset will be updated by automatic cacheSync (if 
*  the ComboBoxItem.displayField is updated). However when using a distinct ComboBoxItem.valueField and
*  ComboBoxItem.displayField, you are required to provide ComboBoxItem.specialValues as a map (there is no
*  support for FormItem.fetchMissingValues automatically fetching appropriate display values).
*  
*  Note that specialValues are not supported in conjunction with MultiComboBoxItem. Whereas with
*  SelectItem.multiple, specialValues will never be normal values 
*  that may be selected. So, specialValues should have options such as "Select All", "Select None" and others.
* @type {ValueMap}
* @default null
*/
specialValues?: ValueMap;

/**
* If true, ComboBoxItem.specialValues special values such as the empty value will be shown in a
*  separate non-scrolling area, in the ComboBoxItem.separateValuesList. Aside from making these values
*  more easily accessible, showing them in a separate list allows data paging to be used, which is
*  disabled if the separateValues are shown in the normal drop-down list along with other values.
* @type {boolean}
* @default null
*/
separateSpecialValues?: boolean;

/**
* AutoChild used to show ComboBoxItem.specialValues.
* @type {ListGrid}
* @default null
*/
separateValuesList?: ListGrid;

/**
* This property controls whether the user can enter a value that is not present in the
*  set of options for this item.
*  
*  If set to false, the value the user enters in the text box is essentially used to filter the 
*  set of options displayed in the pickList.
*  
*  In this mode, when focus is taken from the field, if the entered value does not match
*  any entries in the ValueMap or ComboBoxItem.optionDataSource, it will be discarded. Note 
*  that in this mode, ComboBoxItem.completeOnTab behavior is automatically enabled
*  so if the user enters a valid partial value such that one or more options is displayed in 
*  the pickList, and hits the Tab key, the first matching option will be chosen automatically.
*  In this mode the user may also hit the "Escape" key to discard their edits.
*  
*  Note also that when addUnknownValues is set to false, the underlying value
*  returned by FormItem.getValue will not be updated until a value is explicitly chosen. This
*  means any change or changed handlers will not fire directly in response to the user typing
*  in the field - they will fire when the user actually selects a value, or takes focus from 
*  the field.
*  
*  If this property is set to true, the user is not limited to entering values present in
*  the set of options for the item. Instead the set of options essentially become a set of
*  suggestions that may be used, or the user can enter an entirely new value.
* @type {boolean}
* @default true
*/
addUnknownValues?: boolean;

/**
* If ComboBoxItem.addUnknownValues is false, this property 
*  determines whether
*  the user can clear the comboBoxItem value, or whether they are constrained to
*  choosing one of the available options (in which case clearing the text box will
*  simply revert to the last picked value when the user leaves the field).
*  
*  See also ComboBoxItem.specialValues as a way of providing several different special values in
*  addition to an empty value, such as "Invalid". Note that setting
*  specialValues disables the use of allowEmptyValue - see
*  details of how to have an empty value while using specialValues in 
*  in ComboBoxItem.specialValues.
* @type {boolean}
* @default true
*/
allowEmptyValue?: boolean;

/**
* Optional "pending" style for this item's text box.
*  
*  If ComboBoxItem.addUnknownValues is false, when the user modifies the
*  value displayed in the combobox item text box, the underlying data value (as returned
*  from item.getValue()) is not immediately updated - instead the value is used to filter
*  the set of results displayed in the comboBoxItem pickList.
*  
*  While the comboBoxItem is in this pending state (where the result of getEnteredValue() will
*  not necessarily match the display value for whatever is returned by getValue()), the 
*  pendingTextBoxStyle may be applied to the text box for the item.
*  
*  When the element value is updated to display the actual value for the item (typically due
*  to the user selecting a value from the pickList), the standard 
*  TextItem.textBoxStyle will be reapplied.
*  
*  May be left unset in which case the standard text box style is always applied.
*  Has no effect if ComboBoxItem.addUnknownValues is true.
* @type {CSSStyleName}
* @default null
*/
pendingTextBoxStyle?: CSSStyleName;

/**
* The Class to use when creating a picker of PickList.dataSetType for 
*  a FormItem. Must be a subclass of the builtin default, 
*  PickListMenu.
* @type {SCClassName}
* @default "PickListMenu"
*/
pickListConstructor?: SCClassName;

/**
* The Class to use when creating a picker of PickList.dataSetType for 
*  a FormItem. Must be a subclass of the builtin default, 
*  PickTreeMenu.
* @type {SCClassName}
* @default "PickTreeMenu"
*/
pickTreeConstructor?: SCClassName;

/**
* Whether to show the picker as a flat list, or a collapsible tree.
*  
*  The default value, "list", will use an instance of the
*  PickList.pickListConstructor as the picker - "tree" will 
*  show an instance of PickList.pickTreeConstructor.
* @type {string}
* @default "list"
*/
dataSetType?: string;

/**
* If set, this FormItem will derive data to show in the PickList by fetching records from
*  the specified optionDataSource. The fetched data will be used as a
*  FormItem.valueMap by extracting the
*  FormItem.valueField and FormItem.displayField in
*  the loaded records, to derive one valueMap entry per record loaded from the
*  optionDataSource. Multiple fields from the fetched data may be shown in the pickList by
*  setting ComboBoxItem.pickListFields.
*  
*  The data will be retrieved via a "fetch" operation on the DataSource, passing the 
*  PickList.pickListCriteria (if set) as criteria, and passing
*  ComboBoxItem.optionFilterContext (if set) as DSRequest Properties.
*  
*  The fetch will be triggered when the pickList is first shown, or, you can set
*  SelectItem.autoFetchData to fetch when the FormItem is
*  first drawn.
*  
*  The pickList will not re-fetch its options each time it is opened, unless
*  the pickList criteria have changed. Developers who want to explicitly force a 
*  new fetch can achieve this by
*  overriding PickList.getPickListFilterCriteria
*  
*  to dynamically generate criteria that change whenever appropriate.
*  For example, to force a new fetch every time the pickList is shown,
*  the generated criteria could include a subcriterion for some arbitrary field
*  with operator notEqual and value set to a something
*  that will change each time the method runs, such as a date-stamp.
*  
*  Note that providing an initial value when
*  FormItem.fetchMissingValues is enabled, or enabling
*  SelectItem.defaultToFirstOption, can also cause a fetch to
*  be initiated immediately upon form creation. You can also call PickList.fetchData
*  at any time to manually trigger a fetch.
*  
*  Data paging is automatically enabled if the optionDataSource supports it. As the
*  pickList is scrolled by the user, requests for additional data will be automatically
*  issued.
*  
*  For a pickList attached to a ComboBoxItem, new fetches are 
*  issued as the user types, with criteria set as described under 
*  ComboBoxItem.getPickListFilterCriteria.
*  If your dataSource is not capable of filtering results by search criteria (eg, the
*  dataSource is backed by an XML flat file), you can set ComboBoxItem.filterLocally to have the
*  entire dataset loaded up front and filtering performed in the browser. This disables
*  data paging.
*  
*  Note that if a normal, static FormItem.valueMap is also specified
*  for the field (either directly in the form item or as part of the field definition in
*  the dataSource), it will be preferred to the data derived from the optionDataSource for
*  whatever mappings are present.
* @type {DataSource | string}
* @default null
*/
optionDataSource?: DataSource | string;

/**
* This property allows the developer to specify which field[s] will be displayed in the 
*  drop down list of options.
*  
*  Only applies to databound pickLists (see PickList.optionDataSource, or pickLists
*  with custom data set up via the advanced PickList.getClientPickListData method.
*  
*  If this property is unset, we display the PickList.displayField, if specified, 
*  otherwise the PickList.valueField.
*  
*  If there are multiple fields, column headers will be shown for each field, the
*  height of which can be customized via the PickList.pickListHeaderHeight attribute.
*  
*  Each field to display should be specified as a ListGridField object. Note that
*  unlike in ListGrid, dataSource fields marked as 
*  DataSourceField.hidden will be hidden by default in pickLists. To
*  override this behavior, ensure that you specify an explicit value for 
*  ListGridField.showIf.
* @type {Array<Partial<ListGridField>>}
* @default null
*/
pickListFields?: Array<Partial<ListGridField>>;

/**
* For databound pickLists (see PickList.optionDataSource), by default SmartClient
*  will cache and re-use datasets shown by different pickLists displayed by different
*  SelectItems in an LRU (least recently used) caching pattern.
*  
*  Setting this flag to false avoids this caching for situations where it is too
*  aggressive.
*  
*  Note that this does not control re-use of data within a single pickList. To
*  control when client-side filtering is used in ComboBoxItem, see
*  ComboBoxItem.useClientFiltering and ComboBoxItem.filterLocally.
* @type {boolean}
* @default true
*/
cachePickListResults?: boolean;

/**
* If this item has a specified optionDataSource and this property is
*  true, the list of fields used by this pickList will be passed to
*  the datasource as DSRequest.outputs. If the datasource supports this feature
*  the returned fields will be limited to this list. A custom datasource will need to
*  add code to implement field limiting.
*  
*  This list of used fields consists of the values of FormItem.valueField,
*  FormItem.displayField and ComboBoxItem.pickListFields.
*  
*  NOTE: When enabled, FormItem.getSelectedRecord will only include the
*  fetched fields.
* @type {boolean}
* @default null
*/
fetchDisplayedFieldsOnly?: boolean;

/**
* If this item has a specified optionDataSource, and this property is not
*  null, this will be passed to the datasource as DSRequest properties when
*  performing the filter operation on the dataSource to obtain the set of options for the
*  list. This provides, among other capabilities, a way to trigger the server to return
*  summary records.
* @type {Partial<DSRequest>}
* @default null
*/
optionFilterContext?: Partial<DSRequest>;

/**
* If this item has a specified optionDataSource, this attribute may be set
*  to specify an explicit DSRequest.operationId when performing a fetch against the
*  option dataSource to pick up display value mapping.
* @type {string}
* @default null
*/
optionOperationId?: string;

/**
* If set, this item will display a value from another field to the user instead of
*  showing the underlying data value for the FormItem.name.
*  
*  This property is used in two ways:
*  
*  The item will display the displayField value from the 
*  DynamicForm.getValues if 
*  FormItem.useLocalDisplayFieldValue is true, (or if unset and the conditions
*  outlined in the documentation for that property are met).
*  
*  If this field has an FormItem.optionDataSource, this property is used by
*  default to identify which value to use as a display value in records from this
*  related dataSource. In this usage the specified displayField must be 
*  explicitly defined in the optionDataSource to be used - see 
*  ComboBoxItem.getDisplayFieldName for more on this behavior.
*  If not using FormItem.useLocalDisplayFieldValue, the display
*  value for this item will be derived by performing a fetch against the 
*  FormItem.getOptionDataSource 
*  to find a record where the FormItem.getValueFieldName matches 
*  this item's value, and use the displayField value from that record.
*  In addition to this, PickList-based form items that provide a list of possible
*  options such as the SelectItem or ComboBoxItem will show the 
*  displayField values to the user by default, allowing them to choose
*  a new data value (see FormItem.valueField) from a list of user-friendly
*  display values.
*  
*  This essentially allows the specified optionDataSource to be used as
*  a server based valueMap.
*  
*  If FormItem.useLocalDisplayFieldValue
*  are being used and FormItem.storeDisplayValues is true, selecting a new value
*  will update both the value for this field and the associated display-field value
*  on the record being edited.
*  
*  Note: Developers may specify the FormItem.foreignDisplayField property 
*  in addition to displayField. This is useful for cases where the
*  display field name in the local dataSource differs from the display field name in
*  the optionDataSource. See the documentation for
*  DataSourceField.foreignDisplayField for more on this.
*  If a foreignDisplayField is specified, as with just displayField, if 
*  FormItem.useLocalDisplayFieldValue
*  are being used and FormItem.storeDisplayValues is true, when the user 
*  chooses a value the associated display-field value 
*  on the record being edited will be updated. In this case it would be set to the 
*  foreignDisplayField value from the related record. This means foreignDisplayField 
*  is always expected to be set to the equivalent field in the related dataSources.
*  Developers looking to display some other arbitrary field(s) from the
*  related dataSource during editing should consider using custom
*  PickList.pickListFields instead of setting a foreignDisplayField. 
*  
*  Note that if optionDataSource is set and no valid display field is
*  specified,
*  FormItem.getDisplayFieldName will return the dataSource title 
*  field by default.
*  
*  If a displayField is specified for a freeform text based item (such as a 
*  ComboBoxItem), any user-entered value will be treated as a display value.
*  In this scenario, items will derive the data value for the item from the
*  first record where the displayField value matches the user-entered value.
*  To avoid ambiguity, developers may wish to avoid this usage if display values
*  are not unique.
* @type {string}
* @default null
*/
displayField?: string;

/**
* If this form item maps data values to display values by retrieving the 
*  FormItem.displayField values from an 
*  FormItem.optionDataSource, this property 
*  denotes the the field to use as the underlying data value in records from the 
*  optionDataSource.
*  If not explicitly supplied, the valueField name will be derived as
*  described in FormItem.getValueFieldName.
* @type {string}
* @default null
*/
valueField?: string;

/**
* If filterLocally is set for this item, and this item is showing options 
*  from a dataSource, fetch the entire set of options from the server, and use these values
*  to map the item value to the appropriate display value. Also use "local"
*  type filtering on drop down list of options.
*  
*  This means data will only be fetched once from the server, and then filtered on the
*  client.
*  
*  Note - when this property is set to false, filtering will still be 
*  performed on the client if a complete set of data for some criteria has been cached
*  by a fetch, and a subsequent fetch has more restrictive criteria. To explicitly
*  disable client-side filtering set the ComboBoxItem.useClientFiltering property to false.
* @type {boolean}
* @default false
*/
filterLocally?: boolean;

/**
* If this item has a databound pickList (for example PickList.optionDataSource is
*  set), this property can be used to provide static filter criteria when retrieving the data
*  for the pickList.
* @type {Criteria}
* @default null
*/
pickListCriteria?: Criteria;

/**
* If this item is part of a databound form, and has a specified valueMap,
*  by default we show the valueMap options in the pickList for the item.
*  Setting this property to true will ensure that the options displayed in our pickList
*  are derived from the form's dataSource.
* @type {boolean}
* @default null
*/
showOptionsFromDataSource?: boolean;

/**
* If specified this properties block will be applied to the PickListMenu
*  created for this FormItem.
*  
*  Note: Not every ListGrid property is supported when assigned to a pickList.
*  Where there is a dedicated API on the form item (such as 
*  PickList.pickListCellHeight), we recommend that be used in 
*  favor of setting the equivalent property on the pickListProperties block.
*  
*  PickLists and ListGrid.showFilterEditor:
*  ComboBoxItem do not support setting showFilterEditor 
*  to true on pickListProperties. This combination of settings leads to an ambiguous user
*  experience as the two sets of filter-criteria (those from the text-box and those from the 
*  pickList filter editor) interact with each other.
*  Setting ListGrid.showFilterEditor in 
*  SelectItem.pickListProperties is a valid way to create a filterable pickList, on 
*  a SelectItem, but this setting is not supported on a SelectItem with 
*  SelectItem.multiple set to true - this combination of settings can cause a 
*  selected value to be filtered out of view at which point further selection changes will 
*  discard that value.
*  In general we recommend the ComboBoxItem class (with ComboBoxItem.addUnknownValues 
*  set as appropriate) as a better interface for filtering pickList data.
* @type {Partial<ListGrid>}
* @default null
*/
pickListProperties?: Partial<ListGrid>;

/**
* ListGrid-based AutoChild created by the system to display a list of pickable options for this item. 
*  
*  The pickList is automatically generated and displayed by the system when necessary. It may be customized
*  via properties such as ComboBoxItem.pickListConstructor, ComboBoxItem.pickTreeConstructor, ComboBoxItem.pickListProperties 
*  and more. See the PickList for more information.
*  
*  Accessing the generated pickList at runtime is an advanced usage. In most cases developers
*  should not modify this generated component directly but should instead use attributes
*  on the formItem to configure it.
* @type {ListGrid}
* @default null
*/
pickList?: ListGrid;

/**
* Specifies one or more fields by which this item should be initially sorted. It can be 
*  a ListGridField.name, or an array of field names - but note that, if
*  multiple fields are supplied, then each will be sorted in the same 
*  ListGrid.sortDirection.
*  
*  For full sorting control, set PickList.initialSort to a list of
*  custom SortSpecifier.
*  
*  This attribute can also be set to the index of a field in the fields array, but note 
*  that it will be converted to a string (field name) after initialization.
* @type {string | Array<Partial<string>> | Integer}
* @default null
*/
sortField?: string | Array<Partial<string>> | Integer;

/**
* An array of SortSpecifier objects used to set up the initial sort configuration 
*  for this pickList. If specified, this will be used instead of any 
*  PickList.sortField specified.
* @type {Array<Partial<SortSpecifier>>}
* @default null
*/
initialSort?: Array<Partial<SortSpecifier>>;

/**
* When this item is showing a PickList.dataSetType, this is 
*  the SelectItem.valueField of the record to use as the 
*  Tree.rootValue node.
* @type {string | number}
* @default null
*/
rootNodeId?: string | number;

/**
* When this item is showing a PickList.dataSetType, which 
*  nodes should be opened automatically. Options are:
*  
*  
* - "none" - no nodes are opened automatically
*  
* - "root" - opens the PickList.rootNodeId - in databound 
*        tree-pickers, this node is always hidden
*  
* - "all" - when ResultTree.loadDataOnDemand, opens the
*        PickList.rootNodeId and all of it's direct
*        descendants - otherwise, opens all loaded nodes 
*  
* @type {string}
* @default "none"
*/
autoOpenTree?: string;

/**
* When this item is showing a PickList.dataSetType, the
*  default behavior is for folder open-state to be toggled by double-clicking. Set this 
*  attribute to true to toggle folders on a single-click instead.
*  
*  Note: when set to true, users can only choose leaf-nodes, since clicking folders would
*  simply toggle them.
* @type {boolean}
* @default false
*/
singleClickFolderToggle?: boolean;

/**
* If this combo box retrieves its options from a dataSource, should options
*  be fetched from the server when the item is first written out, or should this fetch be
*  delayed until the user opens the pickList.
* @type {boolean}
* @default false
*/
autoFetchData?: boolean;

/**
* Read-only property set by the ComboBoxItem to indicate whether we should use the 
*  current typed-in value as part of the filter criteria returned by 
*  ComboBoxItem.getPickListFilterCriteria. You can check this flag in order to
*  mimic the ComboBoxItem's default behavior if you provide a custom implementation of
*  getPickListFilterCriteria().
* @type {boolean}
* @default "varies"
*/
filterWithValue?: boolean;

/**
* As the user types into this item's textBox, a comboBoxItem will show the
*  pick-list of options, and filter the set of results displayed by the current value in the
*  text box. For a databound comboBoxItem, by default the entered value is filtered against
*  the ComboBoxItem.displayField if one is specified, otherwise the
*  ComboBoxItem.valueField.
*  
*  This attribute allows the developer to explicitly change which fields to filter against,
*  causing the user-entered text to be matched against any of the specified set of fields from 
*  the ComboBoxItem.optionDataSource.
*  
*  This essentially causes ComboBoxItem.getPickListFilterCriteria to return an AdvancedCriteria
*  object representing "field1 starts with value or field2 starts with value or ...". The
*  OperatorId used is controlled by TextMatchStyle as usual, that is,
*  "startsWith" implies the operator "iStartsWith, "substring" implies "iContains" and "exact"
*  implies "iEquals".
*  
*  The most common use case for this setting would be when a comboBoxItem is showing multiple
*  ComboBoxItem.pickListFields - if the same set of fields is specified as
*  ComboBoxItem.filterFields, the user can use the text-box to filter against
*  whichever fields are visible in the pickList.
*  
*  For finer grained control over comboBoxItem filtering, the 
*  ComboBoxItem.getPickListFilterCriteria method may be overridden.
*  
* @type {Array<Partial<string>>}
* @default null
*/
filterFields?: Array<Partial<string>>;

/**
* For ComboBoxItem.optionDataSource items, this property will be passed
*  to the generated ResultSet data object for the pickList as ResultSet.useClientFiltering.
*  Setting to false will disable filtering on the client and ensure criteria are
*  always passed to the DataSource directly.
* @type {boolean}
* @default null
*/
useClientFiltering?: boolean;

/**
* Minimum length in characters before a search is performed. If too few characters
*  are entered the pick list shows ComboBoxItem.searchStringTooShortMessage.
* @type {Integer}
* @default null
*/
minimumSearchLength?: Integer;

/**
* Message to display in pick list when ComboBoxItem.minimumSearchLength characters have not been entered.
* @type {string}
* @default "Enter a longer search string to search"
*/
searchStringTooShortMessage?: string;

/**
* When applying filter criteria to pickList data, what type of matching to use.
*  
*  For a databound pickList (ComboBoxItem.optionDataSource set), textMatchStyle is
*  sent to the server as DSRequest.textMatchStyle.
*  
*  For a non-databound pickList, textMatchStyle is applied by 
*  PickList.filterClientPickListData.
* @type {TextMatchStyle}
* @default "startsWith"
*/
textMatchStyle?: TextMatchStyle;

/**
* If true, even non-matching options will be shown, with configurable 
*  ComboBoxItem.separatorRows in between. Not valid for
*  ComboBoxItem.optionDataSource.
* @type {boolean}
* @default null
*/
showAllOptions?: boolean;

/**
* Array of records to show between matching and non-matching rows in the PickList.
*  
*  Not valid for ComboBoxItem.optionDataSource.
* @type {Array<Partial<ListGridRecord>>}
* @default "[{isSeparator:true}]"
*/
separatorRows?: Array<Partial<ListGridRecord>>;

/**
* When a comboBoxItem is used to generate search criteria in a SearchForm this property
*  governs whether, if the user explicitly chose an option from the pickList, we explicitly
*  generate criteria that will search for an exact match against the chosen value.
*  
*  In order to achieve this, when this property is set to true, this item will generate
*  AdvancedCriteria in its ComboBoxItem.getCriterion method
*  .
*  
*  See ComboBoxItem.shouldGenerateExactMatchCriteria for behavior when this flag is unset.
* @type {boolean}
* @default null
*/
generateExactMatchCriteria?: boolean;

/**
* The standard FormItem.allowExpressions behavior is always disabled for 
*  ComboBoxItem.
*  
*  The interface is not compatible with the allowExpressions feature.
*  A ComboBoxItem normally starts fetching matches as you type, and that mixes 
*  very strangely with the idea of entering expressions like "a..b"
*  - you will have the ComboBox seemingly switching back and forth between treating
*  the text as a normal search string vs as a special expression on a per-keystroke
*  basis.
*  
*  We recommend a normal TextItem as the correct UI element to supply for users to
*  enter filter expressions.
* @type {boolean}
* @default null
*/
allowExpressions?: boolean;



/**
*  Expression evaluated to determine the ComboBoxItem.defaultValue when no value is 
*  provided for this item. To default to the first option use
*  ComboBoxItem.defaultToFirstOption instead.
*  
* 
* @param {FormItem} the form item itself (also available as "this")
* @param {DynamicForm} the managing DynamicForm instance
* @param {object} the current set of values for the form as a whole
*/
defaultDynamicValue(item:FormItem, form:DynamicForm, values:object): any;

/**
*  Returns the raw text value that currently appears in the text field, which can differ from 
*  FormItem.getValue in various cases - for example:
*  
*  
* - for items that constrain the value range, such as a DateItem with
*  DateItem.enforceDate:true, or a ComboBoxItem with
*  ComboBoxItem.addUnknownValues:false
*  
* - for items with a defined valueMap or edit value formatter and parser functions
*  which converts display value to data value
*  
* - while the item has focus if TextItem.changeOnKeypress is false
*  
*  Note: if the pickList is being shown in any view other than the default 
*  ComboBoxItem.pickListPlacement, as is typically the case on a mobile device,
*  this method will return the value of the ComboBoxItem.pickerSearchField.
*  
* 
*/
getEnteredValue(): string;

/**
*  Only applies to databound items (see PickList.optionDataSource).
*  Performs a fetch type operation on this item's DataSource to retrieve the set of valid
*  options for the item, based on the current PickList.pickListCriteria.
* 
* @param {DSCallback=} Callback to fire when the fetch completes. Callback will        fire with 4 parameters:  
- item a pointer to the form item  
- dsResponse the DSResponse returned by the server  
- data the raw data returned by the server  
- dsRequest the DSRequest sent to the server  
* @param {Partial<DSRequest>=} properties to apply to the       dsRequest for this fetch.
*/
fetchData(callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Returns the displayField for this item.
*  
*  Behavior varies based on the configuration of this item, as follows:
*  
* - If this item has an ComboBoxItem.optionDataSource and an explicit 
*  FormItem.foreignDisplayField is specified, this will be returned.
*  
* - Otherwise if an explicit ComboBoxItem.displayField is specified it will be returned 
*  by default. If the displayField was specified on the underlying 
*  dataSource field, and no matching field is present in the ComboBoxItem.optionDataSource for
*  the item, we avoid returning the specified displayField value and instead return the
*  title field of the option DataSource. We do this to
*  avoid confusion for the case where the displayField is intended as a display-field
*  value for showing another field value within the same record in the underlying
*  dataSource only.
*  
* - If no explicit foreignDisplay or displayField 
*  specification was found, and the FormItem.valueField for this item is hidden in the 
*  FormItem.optionDataSource, this method will return the title field for 
*  the optionDataSource.
* 
* 
*/
getDisplayFieldName(): FieldName;

/**
*  Getter method to retrieve the FormItem.valueField for this item. For
*  items with a specified FormItem.optionDataSource, this determines which
*  field in that dataSource corresponds to the value for this item.
*  
*  If unset, if a DataSourceField.foreignKey exists
*  between this field and the optionDataSource, this will be used,
*  otherwise default behavior will return the FormItem.name of this field.
* 
* 
*/
getValueFieldName(): string;

/**
*  Get the record returned from the ComboBoxItem.optionDataSource when FormItem.fetchMissingValues
*  is true, and the missing value is fetched.
*  
*  FormItem.fetchMissingValues kicks off the fetch when the form item is initialized
*  with a non null value or when setValue() is called on the item. Note that this method
*  will return null before the fetch completes, or if no record is found in the
*  optionDataSource matching the underlying value.
* 
*/
getSelectedRecord(): ListGridRecord;

/**
*  stringMethods to return filter criteria for options displayed for
*  this item. 
*  
*  See comboBoxFiltering for details on how pickList filter criteria
*  are calculated by default for a comboBoxItem.
*   
* 
*/
getPickListFilterCriteria(): Criteria;

/**
*  Returns the set of data to be displayed in this item's PickList.
*  
*  This method will be called for non-databound form items implementing the PickList
*  interface. The default implementation will derive data from the item's valueMap - 
*  can be overridden to allow a custom set of options to be displayed.
*  
*  Note that for PickLists that filter data based on user input
*  (ComboBoxItem), this method should return the data before
*  filtering. To customize the data returned after filtering, override
*  ComboBoxItem.filterClientPickListData instead.
*  
*  As an example, for a formItem with ComboBoxItem.valueField set to "valueFieldName", the
*  default implementation would take a valueMap like the following:
*  
*    valueMap: { value1: "display 1", value2: "display 2" }
*  
*  .. and returning the following set of records: 
*  
*    [
*      { valueFieldName : "value1" },
*      { valueFieldName : "value2" }
*    ]
*  
*  Due to the valueMap, these records will appear as a two row pickList displayed as
*  "display 1" and "display 2".
* 
* 
*/
getClientPickListData(): Array<Partial<ListGridRecord>>;

/**
*  Returns the data to display in the pick list.
*  
*  The default implementation applies the criteria returned by 
*  PickList.getPickListFilterCriteria to the data returned by
*  PickList.getClientPickListData. A record passes the filter if it has a
*  matching value for all fields in the criteria object. Matching is performed according
*  to TextMatchStyle.
*  
*  If PickList.showAllOptions is set, all values are shown, with matching values
*  shown below a PickList.separatorRows.
* 
* 
*/
filterClientPickListData(): Array<Partial<ListGridRecord>>;

/**
*  If this item is showing a dataBound pickList, this notification method will be fired 
*  when new data arrives from the server.
* 
* @param {Integer} index of first row returned by the server
* @param {Integer} index of last row returned by the server
* @param {ResultSet} pointer to this pickList's data
*/
dataArrived(startRow:Integer, endRow:Integer, data:ResultSet): void;

/**
*  When a comboBoxItem is used to generate search criteria in a SearchForm, 
*  if the user explicitly chose an option from the pickList, should the criterion generated
*  by ComboBoxItem.getCriterion
*  
*  enforce a search for an exact match against the chosen value?
*  
*  In order to achieve this, when this property is set to true, this item will generate
*  AdvancedCriteria in its ComboBoxItem.getCriterion
*  
*  method.
*  
*  Default implementation will return ComboBoxItem.generateExactMatchCriteria if specified, otherwise
*  true if the DataSource for this item 
*  DataSource.supportsAdvancedCriteria, false if it
*  does not.
*  
*  See comboBoxItemCriteria for a discussion of criterion generated by 
*  a ComboBoxItem.
* 
* 
*/
shouldGenerateExactMatchCriteria(): boolean;

/**
*  Will this item return advancedCriteria if DynamicForm.getValuesAsCriteria is 
*  called on this item's form?
*  
*  This method is overridden in ComboBoxItem to return true if
*  ComboBoxItem.shouldGenerateExactMatchCriteria returns true, and the user has chosen an
*  exact value from the options in the pickList. In this case we will use 
*  advancedCriteria to ensure the generated search criteria exactly matches 
*  the chosen value for this item.
*  
*  As with formItem.hasAdvancedCriteria() this will also return true if an Operator
*  was explicitly specified for this item
*  
*  See comboBoxItemCriteria for a discussion of criterion generated by 
*  a ComboBoxItem.
* 
* 
*/
hasAdvancedCriteria(): boolean;

/**
*  Returns criterion derived from the current value of this item.
*  
*  See comboBoxItemCriteria for a discussion of criterion generated by 
*  a ComboBoxItem.
* 
* 
*/
getCriterion(): Criterion;

/**
*  This method is overridden in comboBoxItem. When addUnknownValues is true, 
*  comboBoxItems allow the user to edit substring match type criteria applied to the
*  display field (if one is specified).
*  
*  The user can also edit criteria attempting to match exactly against the item's field name.
* 
* 
* @param {Criterion} sub-criterion from an AdvancedCriteria object
*/
canEditCriterion(criterion:Criterion): boolean;

/**
*  Overridden to support editing criterion against the display field or value field when
*  ComboBoxItem.addUnknownValues is true.
* 
*/
setCriterion(): void;



}
/**
* Creates a new ComboBoxItem
* 
* @param typeCheckedProperties {Partial<ComboBoxItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ComboBoxItem} The new instance
*/
class ComboBoxItem{
static create(typeCheckedProperties? : Partial<ComboBoxItem>, uncheckedProperties?: Object): ComboBoxItem;







}

/**
* Provides background communication with an HTTP server
*/
interface Comm extends Class {




}
/**
* Creates a new Comm
* 
* @param typeCheckedProperties {Partial<Comm>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Comm} The new instance
*/
class Comm{
static create(typeCheckedProperties? : Partial<Comm>, uncheckedProperties?: Object): Comm;
/**
* If true, SmartClient will use a 
*  web worker to send 
*  RPCRequests and DSRequests concurrent with the main
*  Javascript thread, if:
*  
* - The RPCRequest.transport is "xmlHttpRequest"
*  
* - The browser is Internet Explorer 10 or greater
*  
*  We do this because Internet Explorer sometimes queues the sending of data with other,
*  timer-delayed tasks on the single Javascript thread. With a busy application, this can
*  lead to an xmlHttpRequest seeming to block; the HTTP connection is made to the server, and
*  the server then goes into a wait state while the browser completes other timer-delayed 
*  tasks. If there are expensive timer-delayed tasks in the way, such as a reflow of the
*  UI, completing the send to the server can block for a significant length of time,
*  leading in turn to server turnaround times that are significantly longer than they need
*  to be.
*  
*  Internet Explorer is the only browser that behaves like this, so this option does not 
*  apply to other browsers.
* @type {boolean}
* @default false
*/
static concurrentXHRsInIE?: boolean;

/**
* The message to show the user if Comm.concurrentXHRsInIE is 
*  in force and an error occurs on the concurrent worker thread. If this attribute is
*  left at its default null value, the user is shown the error message reported by the 
*  browser, along with the URL and line number where the error occurred (this information
*  is also logged to the developer console, regardless of the message shown to the user)
* @type {string}
* @default null
*/
static concurrentXHRErrorMessage?: string;








}

/**
* Base class for tasks that target SmartClient
*  UI-specific operations.
*  
*  Note: This task is not for direct use - use one of the subclasses instead.
*/
interface ComponentTask extends ProcessElement {
/**
* ID of component targeted by this task.
* @type {GlobalId}
* @default null
*/
componentId?: GlobalId;

/**
* Base class of components that this task targets.
* @type {string | Array<Partial<string>>}
* @default null
*/
componentBaseClass?: string | Array<Partial<string>>;

/**
* Must target components of this task have a DataSource?
* @type {boolean}
* @default null
*/
componentRequiresDataSource?: boolean;



/**
*  Returns the actual component specified by ComponentTask.componentId for this task. Unless
*  skipValidation is true, the component type is validated against
*  ComponentTask.componentBaseClass and only returned if it matches. Null is returned otherwise.
* 
* 
* @param {Process} the process that is handling the workflow
* @param {boolean=} skip validation against componentBaseClass?
*/
getTargetComponent(process:Process, skipValidation?:boolean): void;



}
/**
* Creates a new ComponentTask
* 
* @param typeCheckedProperties {Partial<ComponentTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ComponentTask} The new instance
*/
class ComponentTask{
static create(typeCheckedProperties? : Partial<ComponentTask>, uncheckedProperties?: Object): ComponentTask;







}

/**
* The CubeGrid is an interactive grid component that presents very large, multi-dimensional
*  data sets (also known as data cubes) for reporting or analytic applications.
*  
*  CubeGrids are often called crosstabs, for their cross-tabular display of data dimensions in
*  stacked/nested rows and columns, or pivot tables, for their ability to "pivot" dimensions
*  between rows and columns to view a data cube from different perspectives. They are typically
*  used in the querying and reporting front-ends of data warehousing, decision support, OLAP,
*  and business intelligence systems.
*  
*  For example, CubeGrids can be connected to Pentaho Mondrian, Jasper Reports, Microsoft
*  Analysis Services and any other OLAP technology that supports the XMLA standard - the
*  Isomorphic public wiki has 
*  examples.
*  of such integration.
*  
*  NOTE: you must load the Analytics 
*  loadingOptionalModules before you can use CubeGrid.
*  
*  Multi-Dimensional Data Terminology
*  
*  The CubeGrid refers to the dimensions of a data cube as facets, to the possible values in
*  each facet as facet values, and to the values within the data cube as data values or cell
*  values. Equivalent terms that are commonly used in data warehousing or business intelligence
*  systems include:
*  facet: dimension, attribute, feature
*  facet value: dimension member, attribute value, feature value
*  cell value: data value, metric value, measure
*  
*  Visual Structure
*  
*  Like the ListGrid and TreeGrid components, the CubeGrid displays data values in a tabular
*  "body" with adjacent "headers". While the ListGrid and TreeGrid display rows of records
*  with field values, the CubeGrid displays a body of individual cell values, each associated
*  with a combination of facet values. The facet values for a cell are displayed in the column
*  headers above the cell and row headers to the left of the cell. CubeGrids can display an
*  arbitrary number of facets, by stacking multiple levels of row and/or column headers.
*  
*  Except for the innermost column facet, each facet in a CubeGrid has a facet label adjacent
*  to its row or column headers. The facet labels serve two main purposes: they display the
*  titles of the facets, and they provide drag-and-drop reordering or pivoting of facets within
*  the CubeGrid. The row facet labels also provide interactive selection, resizing, and other
*  operations on the columns of row facet values.
*  
*  The innermost column headers provide special behaviors and controls for manipulating the
*  columns of data in a CubeGrid. End users may select, resize, reorder, minimize, maximize, or
*  auto-fit the columns of data via mouse interactions with these headers. Customizable
*  indicators and controls may be included at the top of each innermost column header.
*  
*  If a CubeGrid is not large enough to display all of its cell values, horizontal and/or
*  vertical scrollbars will appear below and to the right of the body. The body of the CubeGrid
*  may be scrolled on either axis. The headers are "frozen" from scrolling on one axis - row
*  headers only scroll vertically, while column headers only scroll horizontally - so the facet
*  values for the visible cells are always displayed.
*  
*  Data Loading
*  
*  Data can be provided to the Cube via CubeGrid.data as an Array of
*  CellRecord, each representing the data for one cell.
*  
*  For large datasets, CubeGrid.dataSource with one field per
*  facetId, and the CubeGrid will load data on demand to fill the visible area, including
*  lazily loading data for expanding/collapsing tree facets and when facetValues are made
*  visible programmatically or via menus.
*  
*  Picking Facets
*  
*  By "facet" we mean an aspect of the data which is orthogonal to other aspects of the data,
*  that is, combining values from any two "facets" should make sense.
*  
*  For example, in sales data, two facets might be "quarter" and "region" - it makes sense to
*  combine any quarter and region, although for some combinations, there may not be data
*  available.
*  
*  An example of two aspects that would not be independent facets are "state" and "city"
*  - it's senseless to combine arbitrary states with arbitrary cities - most combinations are
*  invalid. Consider instead a Facet.isTree that combines "city" and "state"
*  values. 
*  
*  Note that if "city" and "state" are represented as facets, they may look correct if they are
*  both on the same axis of the grid and
*  CubeGrid.hideEmptyFacetValues is used to trim nonsense
*  combinations, but if the data is CubeGrid.canMoveFacets such that "state" and
*  "city" are on opposing axes, there will be a roughly diagonal "stripe" of data for
*  combinations of "state" and "city" that make sense, and all other space will be blank. This
*  is a strong indication that two facets should be represented as a single tree facet instead.
*/
interface CubeGrid extends Omit<ListGrid, 'exportClientData' | 'getEditValues' | 'setEditValues' | 'clearEditValue' | 'discardAllEdits' | 'saveAllEdits' | 'getCellRecord'>  {
/**
* CSS class for the CubeGrid as a whole
* @type {CSSStyleName}
* @default "normal"
*/
styleName?: CSSStyleName;

/**
* CSS class for the CubeGrid body
* @type {CSSStyleName}
* @default "cubeGridBody"
*/
bodyStyleName?: CSSStyleName;

/**
* GridRenderer.baseStyle for this listGrid.
*  If this property is unset, base style may be derived from ListGrid.normalBaseStyle
*  or ListGrid.tallBaseStyle as described in
*  ListGrid.getBaseStyle.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined
*  with the base style to generate stateful cell styles.
* @type {CSSStyleName}
* @default "cubeCell"
*/
baseStyle?: CSSStyleName;

/**
* Whether alternating rows should be drawn in alternating styles, in order to create a "ledger"
*  effect for easier reading. If enabled, the cell style for alternate rows will have "Dark"
*  appended to it.
* @type {boolean}
* @default true
*/
alternateRecordStyles?: boolean;

/**
* Default directory for skin images (those defined by the class), relative to the
*  Page-wide Page.getSkinDir.
* @type {SCImgURL}
* @default "images/CubeGrid/"
*/
skinImgDir?: SCImgURL;

/**
* Button.baseStyle for the buttons in the innermost column header for 
*  this cubeGrid.
* @type {CSSStyleName}
* @default "innerHeader"
*/
innerHeaderBaseStyle?: CSSStyleName;

/**
* Button.baseStyle for the buttons in this grid's column headers.
*  
*  Exception: The innermost column
*  header will always be styled using CubeGrid.innerHeaderBaseStyle.
* @type {CSSStyleName}
* @default "colHeader"
*/
colHeaderBaseStyle?: CSSStyleName;

/**
* Button.baseStyle for the facet-label buttons above this grid's 
*  column headers.
* @type {CSSStyleName}
* @default "colHeaderLabel"
*/
colHeaderLabelBaseStyle?: CSSStyleName;

/**
* Button.baseStyle for the buttons in this grid's row headers.
* @type {CSSStyleName}
* @default "rowHeader"
*/
rowHeaderBaseStyle?: CSSStyleName;

/**
* Button.baseStyle for the facet-label buttons above the grid's 
*  row headers.
* @type {CSSStyleName}
* @default "rowHeaderLabel"
*/
rowHeaderLabelBaseStyle?: CSSStyleName;

/**
* This property is not supported for CubeGrid.
* @type {boolean}
* @default null
*/
rotateHeaderTitles?: boolean;

/**
* If specified and this.showHover is true, this is the default width to apply
*  to hover tips shown for facetValues. If unset, the hover canvas will be sized to 
*  this.hoverWidth if specified instead.
* @type {number | string}
* @default null
*/
facetValueHoverWidth?: number | string;

/**
* If specified and this.showHover is true, this is the default width to apply
*  to hover tips shown for facetLabels. If unset, the hover canvas will be sized to 
*  this.hoverWidth if specified instead.
* @type {number | string}
* @default null
*/
facetLabelHoverWidth?: number | string;

/**
* If specified and this.showHover is true, this is the default height to apply
*  to hover tips shown for facetValues. If unset, the hover canvas will be sized to 
*  this.hoverHeight if specified instead.
* @type {number | string}
* @default null
*/
facetValueHoverHeight?: number | string;

/**
* If specified and this.showHover is true, this is the default height to apply
*  to hover tips shown for facetLabels. If unset, the hover canvas will be sized to 
*  this.hoverHeight if specified instead.
* @type {number | string}
* @default null
*/
facetLabelHoverHeight?: number | string;

/**
* Allows the developer to override the horizontal text alignment of hover tips shown for
*  facet values. If unspecified the hover canvas content alignment will be set by
*  this.hoverAlign if specified.
* @type {Alignment}
* @default null
*/
facetValueHoverAlign?: Alignment;

/**
* Allows the developer to override the horizontal text alignment of hover tips shown for
*  facetLabels. If unspecified the hover canvas content alignment will be set by
*  this.hoverAlign if specified.
* @type {Alignment}
* @default null
*/
facetLabelHoverAlign?: Alignment;

/**
* Allows the developer to override the vertical text alignment of hover tips shown for
*  facet values. If unspecified the hover canvas content alignment will be set by
*  this.hoverVAlign if specified.
* @type {VerticalAlignment}
* @default null
*/
facetValueHoverVAlign?: VerticalAlignment;

/**
* Allows the developer to override the vertical text alignment of hover tips shown for
*  facetLabels. If unspecified the hover canvas content alignment will be set by
*  this.hoverVAlign if specified.
* @type {VerticalAlignment}
* @default null
*/
facetLabelHoverVAlign?: VerticalAlignment;

/**
* Allows the developer to override the css class applied to hover tips shown for
*  facet values. If unspecified, and this.hoverStyle is not null, that 
*  css class will be applied to facet value hovers instead.
* @type {CSSStyleName}
* @default null
*/
facetValueHoverStyle?: CSSStyleName;

/**
* Allows the developer to override the css class applied to hover tips shown for
*  facet labels. If unspecified, and this.hoverStyle is not null, that 
*  css class will be applied to facet label hovers instead.
* @type {CSSStyleName}
* @default null
*/
facetLabelHoverStyle?: CSSStyleName;

/**
* A FacetValueMap describing the set of facet values that should be regarded as
*  "fixed" in this cubeGrid. These are used as fixed criteria for load on demand, and also
*  allow using a dataset with more facets in it than are currently shown in the grid.
* @type {FacetValueMap}
* @default null
*/
fixedFacetValues?: FacetValueMap;

/**
* Hilites to be applied to the data for this component. See hiliting.
* @type {Array<Partial<Hilite>>}
* @default null
*/
hilites?: Array<Partial<Hilite>>;

/**
* DataSource to use to fetch CubeGrid data.
*  
*  The DataSource should have a field named after each facetId. The CubeGrid will submit
*  requests for data as DataSource "fetch" operations that request 
*  CellRecord only for currently visible area
*  (including ListGrid.drawAheadRatio). The Criteria passed
*  in each fetch operation will be a FacetValueMap that corresponds to a rectangular
*  swath of cells the CubeGrid needs data for, for example:
*    { region:"west", product:["chair", "table"], timePeriod:"Q1 2004" }
*  
*  Note that in the criteria above, the Array value for the "product" facet indicates the
*  CubeGrid needs cellRecords for both the "chair" and "table" facetValues.
*  
*  CubeGrid.fixedFacetValues, if any, are always sent in each "fetch" operation.
*  
*  Scrolling, expanding/collapsing tree facets, pivoting, and changing the currently
*  visible facets may all trigger data requests.
*  
*  The CubeGrid will generally attempt to fetch a minimal set of data to fill the viewport,
*  sending multiple fetch operations in a batch (via RPCManager.startQueue)
*  which minimally describe newly revealed areas without specifying each individual cell.
*  The CubeGrid will automatically handle being provided more data than was asked
*  for, so server-side fetch-ahead policies can be implemented without any client-side
*  customization.
*  
*  Note that the DataSource.serverType shipped with the SmartClient
*  SDK is capable of responding to the CubeGrid's data requests without writing any custom
*  server code.
* @type {DataSource}
* @default null
*/
dataSource?: DataSource;

/**
* If true, when this component is first drawn, automatically call this.fetchData().
*  Criteria for this fetch may be picked up from ListGrid.initialCriteria, and textMatchStyle may
*  be specified via ListGrid.autoFetchTextMatchStyle.
*  
*  NOTE: if autoFetchData is set, calling
*  ListGrid.fetchData before draw will cause two requests to be issued, one from the manual
*  call to fetchData() and one from the autoFetchData setting. The second request will use
*  only ListGrid.initialCriteria and not any other criteria or settings from the first request.
*  Generally, turn off autoFetchData if you are going to manually call ListGrid.fetchData at any time.
* 
*  Note: If you are using saved searches - either via SavedSearchItem or ListGrid.saveDefaultSearch, autoFetchData
*  will be automatically suspended and replaced with the saved criteria/view state, if applicable.
* @type {boolean}
* @default true
*/
autoFetchData?: boolean;

/**
* If CubeGrid.autoFetchData is true, this attribute allows the developer to
*  specify a textMatchStyle for the initial ListGrid.fetchData call.
* @type {TextMatchStyle}
* @default "exact"
*/
autoFetchTextMatchStyle?: TextMatchStyle;

/**
* Facet definitions for this CubeGrid. Facets, also called "dimensions", are orthogonal
*  aspects of the data model.
*  
*  For example, you can look at profit by the facets "plant and product" or by "product and
*  plant" and it's the same number, because the facets - plant and product - are the same.
*  What would change the profit numbers would be to remove a facet, called "summarizing",
*  or add a new facet, called "drilling down". For example, showing profit by plant and
*  product, you could "drill down" by adding the region facet, which would divide profit
*  among each region. Or you could remove the "plant" facet, showing total profit for each
*  "product", summed across all plants.
*  
*  This property need not be set and will automatically be constructed during widget
*  initialization if data is provided up front and CubeGrid.rowFacets and 
*  CubeGrid.columnFacets have been set. If CubeGrid.facets is not set and
*  there is no initial data but a DataSource is present, drawing the grid will
*  automatically issue a fetch to allow CubeGrid.facets to be resolved.
* @type {Array<Partial<Facet>>}
* @default null
*/
facets?: Array<Partial<Facet>>;

/**
* If true, show facet label context menus with some built-in operations.
*  Otherwise, use generic context menu handling.
* @type {boolean}
* @default false
*/
showFacetContextMenus?: boolean;

/**
* If true, show facet value context menus with some built-in operations.
*  Otherwise, use generic context menu handling.
*  Use this in place of ListGrid.showHeaderContextMenu and
*  ListGrid.showHeaderMenuButton for CubeGrids.
* @type {boolean}
* @default true
*/
showFacetValueContextMenus?: boolean;

/**
* Title for the Field-visibility submenu item.
* @type {string}
* @default "Values"
*/
fieldVisibilitySubmenuTitle?: string;

/**
* Determines whether cell values in the body can be selected.
* @type {boolean}
* @default true
*/
canSelectValues?: boolean;

/**
* Determines whether row or column facetValue headers can be selected.
* @type {boolean}
* @default true
*/
canSelectHeaders?: boolean;

/**
* If true, when multiple facets appear on one side in a nested headers presentation, the
*  selection state of parent/child headers are automatically kept in sync.
* @type {boolean}
* @default true
*/
autoSelectHeaders?: boolean;

/**
* Whether to select cells in the body when row or column headers are selected.
* @type {AutoSelectionModel}
* @default "both"
*/
autoSelectValues?: AutoSelectionModel;

/**
* If true, clicking on the existing selection causes it to be entirely deselected.
* @type {boolean}
* @default false
*/
simpleDeselect?: boolean;

/**
* The list of Facet.id for facets that will appear on top of the body.
* @type {Array<Partial<string>>}
* @default null
*/
columnFacets?: Array<Partial<string>>;

/**
* The list of Facet.id for facets that will appear to the left of the body.
* @type {Array<Partial<string>>}
* @default null
*/
rowFacets?: Array<Partial<string>>;

/**
* This causes the headers for any combination of facetValues for which there are no 
*  cellRecords to be suppressed.
*  
*  To use this feature, either:
*  
*  
* - all must be provided via ListGrid.setData before the CubeGrid is first drawn, OR
*  
* - all data must be returned by the first DataSource fetch, OR
*  
* - CubeGrid.hideEmptyAxis must be set to either "row" or "column" so that empty values
*  are only automatically hidden for one axis
*  
*  This last point is required because there is no way to determine whether a row is empty
*  unless data for all columns of the row has been loaded (and vice-versa). For this
*  reason if you set hideEmptyFacetValues but do not set hideEmptyAxis, the default
*  behavior of DataSource is automatically disabled and
*  only CubeGrid.fixedFacetValues will be sent as criteria.
* @type {boolean}
* @default null
*/
hideEmptyFacetValues?: boolean;

/**
* With CubeGrid.hideEmptyFacetValues, controls on which axis hiding of empty values is applied,
*  "row" (only empty rows are hidden), "column" (only empty columns are hidden) or both
*  (the default).
* @type {Axis}
* @default null
*/
hideEmptyAxis?: Axis;

/**
* An array of "cellRecords", each of which represents data for one cell of the body area.
* @type {Array<Partial<CellRecord>>}
* @default null
*/
data?: Array<Partial<CellRecord>>;

/**
* Name of the property in a cell record that holds the cell value.
* @type {Identifier}
* @default "_value"
*/
valueProperty?: Identifier;

/**
* Name of the property in a cell record that holds it's unique ID. Note cell record IDs are
*  optional.
* @type {Identifier}
* @default "ID"
*/
cellIdProperty?: Identifier;

/**
* Whether cells can be edited in this grid. Can be overridden on a per-facetValue basis.
* @type {boolean}
* @default false
*/
canEdit?: boolean;

/**
* CubeGrids only support editing by cell.
* @type {boolean}
* @default true
*/
editByCell?: boolean;

/**
* CubeGrids only support editing by cell.
* @type {boolean}
* @default true
*/
saveByCell?: boolean;

/**
* For Browser.isTouch, canDragSelect defaults to false
*  so that touch scrolling can be used to navigate scrollable CubeGrids. In all other
*  browsers it defaults to true.
*  
*  NOTE: If canDragSelect is enabled, it may be desirable to disable
*  Canvas.useTouchScrolling so that touch-dragging cells of the CubeGrid
*  selects them rather than starting a scroll. If Canvas.disableTouchScrollingForDrag
*  is set to true, then touch scrolling will be disabled automatically.
*  However, for accessibility reasons, it is recommended to
*  leave touch scrolling enabled and provide an alternative set of controls that can be
*  used to perform drag-selection.
* @type {boolean}
* @default null
*/
canDragSelect?: boolean;

/**
* If true, body columns can be reordered via the innermost column headers.
* @type {boolean}
* @default null
*/
canReorderColumns?: boolean;

/**
* If true, body columns can be resized via the innermost column headers.
* @type {boolean}
* @default null
*/
canResizeColumns?: boolean;

/**
* If true, hierarchical facets will show expand/collapse controls to allow the user to
*  expand and collapse the tree of facetValues for that facet.
* @type {boolean}
* @default false
*/
canCollapseFacets?: boolean;

/**
* If true, when multiple facets are shown on a side, all facetValues in the second level
*  of headers or higher will show controls to "minimize" the values of the next facet.
*  Minimizing means showing only one, or very few, of the next facet's values.
*  
*  Set FacetValue.isMinimizeValue to indicate which facetValues should be
*  shown when a facet is minimized.
* @type {boolean}
* @default false
*/
canMinimizeFacets?: boolean;

/**
* Turns on indenting of any Facet.isTree column facets. This can be
*  overridden at the facet level via Facet.indentVTree. Setting this property also
*  ensures that the header is sized tall enough to accommodate the fully expanded facet. 
*  
*  The amount of indenting per level can be set with CubeGrid.vTreeFacetIndent, and the
*  direction of the indenting specified with CubeGrid.vTreeFacetIndentDirection.
*  
*  Note that if you specify an explicit height for such a fscet, such as by setting
*  Facet.height or Facet.labelHeight, then the greater of that or the space
*  required to accommodate the fully expanded facet will determine the actual height used.
* @type {boolean}
* @default false
*/
indentVTreeFacets?: boolean;

/**
* Determines how many pixels to move for each level when Facet.isTree
*  column facets are being CubeGrid.indentVTreeFacets.
* @type {Integer}
* @default "8"
*/
vTreeFacetIndent?: Integer;

/**
* Determines layout of facet value titles in each column facet being 
*  CubeGrid.indentVTreeFacets.
* @type {FacetIndentDirection}
* @default "descending"
*/
vTreeFacetIndentDirection?: FacetIndentDirection;

/**
* FacetValueMap of facet values representing a set of facetValues by which the
*  cubeGrid data is sorted.
* @type {FacetValueMap}
* @default null
*/
sortedFacetValues?: FacetValueMap;

/**
* Direction of sorting if sortedFacet or sortedFacetValues is specified.
* @type {SortDirection}
* @default "ascending"
*/
sortDirection?: SortDirection;

/**
* If true, sort controls will be shown on FacetHeaders.
*  
*  When clicked, sort controls call CubeGrid.sortByFacetId.
* @type {boolean}
* @default null
*/
canSortFacets?: boolean;

/**
* If true, sort controls will be shown on facet values.
*  
*  When clicked, sort controls call CubeGrid.sortByFacetValues.
* @type {boolean}
* @default null
*/
canSortData?: boolean;

/**
* If true, allow columns in the grid body to be minimized (reduced to the width of the
*  minimize control) by clicking on a minimize control in the innermost column headers.
* @type {boolean}
* @default null
*/
canMinimizeColumns?: boolean;

/**
* Whether row and column facets can be rearranged by the user, by dragging and dropping the
*  facet labels.
* @type {boolean}
* @default false
*/
canMoveFacets?: boolean;

/**
* If enabled row headers for this cubeGrid will be rendered using a GridRenderer
*  component. This improves performance for very large cubeGrids.
*  
*  Note that this attribute must be set for hierarchical row facets to be indented properly.
* @type {boolean}
* @default false
*/
rowHeaderGridMode?: boolean;

/**
* Automatically size row headers to fit wrapped text.
* @type {boolean}
* @default false
*/
autoSizeHeaders?: boolean;

/**
* Default alignment for facet labels.
* @type {Alignment}
* @default "center"
*/
facetTitleAlign?: Alignment;

/**
* Default alignment for facet values (in headers).
* @type {Alignment}
* @default "center"
*/
facetValueAlign?: Alignment;

/**
* Default align for cell values (in body).
* @type {Alignment}
* @default "center"
*/
cellAlign?: Alignment;

/**
* Whether to allow text wrapping on facet titles.
* @type {boolean}
* @default false
*/
wrapFacetTitles?: boolean;

/**
* Whether to allow text wrapping on facet value titles.
*  
*  Note that this property is incompatible with CubeGrid.indentVTreeFacets column
*  facets.
* @type {boolean}
* @default false
*/
wrapFacetValueTitles?: boolean;

/**
* Whether to pad titles so they aren't flush with header borders.
* @type {boolean}
* @default true
*/
padTitles?: boolean;

/**
* Default width of inner column headers.
* @type {number}
* @default "100"
*/
defaultFacetWidth?: number;

/**
* facetValueId of the default rollupValue for each facet. Can be overridden per facet via
*  facet.rollupValue.
* @type {string}
* @default "sum"
*/
rollupValue?: string;

/**
* In a CubeGrid that displays values of different types (eg "Revenue" and "Income"), the
*  different types of values on display are enumerated as the facet values of the "metric
*  facet". 
*  
*  The metric facet is treated identically to any other facet by the CubeGrid: it can be
*  represented as row or column headers, can be innermost or have other facets under it,
*  can be moved around, etc. However when a metric facet is used, CubeGrid.metricFacetId
*  must be set to allow the CubeGrid to generate meaningful descriptions of values shown in
*  cells for use in hovers and other situations; see CubeGrid.valueTitle for a full
*  explanation.
* @type {string}
* @default "metric"
*/
metricFacetId?: string;

/**
* A label for the data values shown in cells, such as "Sales in Thousands", typically used
*  when the CubeGrid must generate a description for a cell value or set of cell values.
*  
*  For example, in a CubeGrid showing "Revenue" by region and product, a cell with a
*  CellRecord like:
*  
*  {product:"chairs", region:"northwest", _value:"$5k"}
*  
*  Should be described as "Revenue for Chairs for Northwest Region", not "Chairs for
*  Revenue for Northwest Region".
*  
*  For CubeGrids that show multiple types of values at once (eg both "Revenue" and
*  "Income") see CubeGrid.metricFacetId.
* @type {string}
* @default null
*/
valueTitle?: string;

/**
* Minimum width for the body of this cubeGrid.
* @type {number}
* @default null
*/
bodyMinWidth?: number;

/**
* Minimum height for the body of this cubeGrid.
* @type {number}
* @default null
*/
bodyMinHeight?: number;

/**
* This property is not supported for CubeGrid.
*  
*  Consider setting explicit widths via FacetValue.width or CubeGrid.defaultFacetWidth.
* @type {boolean}
* @default null
*/
autoFitFieldWidths?: boolean;

/**
* If set to true, context menu items will be included on the cells and headers providing the
*  user with an option to create a chart of the cubeGrid's data set.
*  See CubeGrid.makeChart for more information.
* @type {boolean}
* @default false
*/
enableCharting?: boolean;

/**
* Name of the SmartClient Class to be used when creating charts. Must support the
*  Chart interface.
* @type {string}
* @default "FacetChart"
*/
chartConstructor?: string;

/**
* Default type of chart to plot.
* @type {ChartType}
* @default "Column"
*/
chartType?: ChartType;

/**
* If CubeGrid.makeChart is called with a chart specification that will show more than
*  chartConfirmThreshold data elements, the user will be presented with a
*  isc.confirm.
*  
*  Set to 0 to disable this confirmation.
* @type {number}
* @default "2000"
*/
chartConfirmThreshold?: number;

/**
* Title for the selection highlight menu item.
* @type {string}
* @default "Highlight Selection"
*/
highlightSelectionTitle?: string;

/**
* Title for the cell highlight menu item.
* @type {string}
* @default "Highlight Cell"
*/
highlightCellTitle?: string;

/**
* Title for the highlight menu item.
* @type {string}
* @default "Highlight"
*/
highlightTitle?: string;

/**
* Title for the menu item that clears highlights.
* @type {string}
* @default "None"
*/
noHighlightsTitle?: string;

/**
* Title for the show highlights menu item.
* @type {string}
* @default "Show Highlights"
*/
showHighlightsTitle?: string;

/**
* Title for the show all highlights menu item.
* @type {string}
* @default "Show all"
*/
showAllHighlightsTitle?: string;

/**
* Title for the hide all highlights menu item.
* @type {string}
* @default "Hide all"
*/
hideAllHighlightsTitle?: string;

/**
* Title for the show hover tips menu item.
* @type {string}
* @default "Show Hover Tips"
*/
showHoverTipsTitle?: string;

/**
* Title for the Chart menu item.
* @type {string}
* @default "Chart"
*/
chartItemTitle?: string;

/**
* Title for the Chart dialog.
* @type {string}
* @default "Chart"
*/
chartDialogTitle?: string;

/**
* Title for the auto-fit column menu item.
* @type {string}
* @default "AutoFit Column"
*/
autoFitColumnTitle?: string;

/**
* Title for the maximize-column menu item.
* @type {string}
* @default "Maximize Column"
*/
maximizeColumnTitle?: string;

/**
* Title for the minimize-column menu item.
* @type {string}
* @default "Minimize Column"
*/
minimizeColumnTitle?: string;

/**
* Title for the Rename menu item.
* @type {string}
* @default "Rename..."
*/
renameFacetValueTitle?: string;

/**
* Message displayed when renaming a facet value.
* @type {string}
* @default "Enter the new name for this facet value:"
*/
renameFacetValueMessage?: string;

/**
* Title for the resizeHandle control.
* @type {string}
* @default "Move"
*/
controlReorderHandleTitle?: string;

/**
* Title for the minimize control.
* @type {string}
* @default "Minimize"
*/
controlMinimizeTitle?: string;

/**
* Title for the maximize control.
* @type {string}
* @default "Maximize"
*/
controlMaximizeTitle?: string;

/**
* Title for the sort-up control.
* @type {string}
* @default "Sort Up"
*/
controlSortUpTitle?: string;

/**
* Title for the sort-down control.
* @type {string}
* @default "Sort Down"
*/
controlSortDownTitle?: string;

/**
* Title for the close control.
* @type {string}
* @default "Close"
*/
controlCloseTitle?: string;

/**
* Title for the chart-type control.
* @type {string}
* @default "Chart Type"
*/
chartTypeTitle?: string;

/**
* Title for the stacked chart item.
* @type {string}
* @default "Stacked"
*/
chartStackedTitle?: string;

/**
* Default separator string used by CubeGrid.exportClientData to separate column
*  and row facet value titles.
* @type {string}
* @default " - "
*/
exportFacetSeparatorString?: string;

/**
* Array of MenuItem to replace the default menu. 
*  Call CubeGrid.getDefaultFacetValueContextItems to get a default set of
*  items to start with.
* @type {Array<Partial<MenuItem>>}
* @default null
*/
facetValueContextItems?: Array<Partial<MenuItem>>;

/**
* Sets the background color for the row and column headers of the cube,
*  if not otherwise set by a more specific property.
*  (see CubeGrid.exportRowFacetBGColor and
*  CubeGrid.exportColumnFacetBGColor).
*  See also exportBGColor.
* @type {CSSColor}
* @default null
*/
exportFacetBGColor?: CSSColor;

/**
* Sets the background color for the row headers of the cube.
*  See also exportBGColor.
* @type {CSSColor}
* @default null
*/
exportRowFacetBGColor?: CSSColor;

/**
* Sets the background color for the column headers of the cube.
*  See also exportBGColor.
* @type {CSSColor}
* @default null
*/
exportColumnFacetBGColor?: CSSColor;

/**
* Sets the text color for the row and column headers of the cube, 
*  if not otherwise set by a more specific property.
*  (see CubeGrid.exportRowFacetTextColor and
*  CubeGrid.exportColumnFacetTextColor).
* @type {CSSColor}
* @default null
*/
exportFacetTextColor?: CSSColor;

/**
* Sets the text color for the row headers of the cube.
* @type {CSSColor}
* @default null
*/
exportRowFacetTextColor?: CSSColor;

/**
* Sets the text color for the column headers of the cube.
* @type {CSSColor}
* @default null
*/
exportColumnFacetTextColor?: CSSColor;

/**
* FormatString for numeric or date formatting. See DataSourceField.format.
* @type {FormatString}
* @default null
*/
valueFormat?: FormatString;

/**
* FormatString used during exports for numeric or date formatting. See
*  DataSourceField.exportFormat.
* @type {FormatString}
* @default null
*/
valueExportFormat?: FormatString;



/**
*  Returns the row number of the most recent mouse event.
* 
* @param {Integer=} optional y-coordinate to obtain row number, in lieu of the y            coordinate of the last mouse event
*/
getEventRow(y?:Integer): Integer;

/**
*  Returns the column number of the most recent mouse event.
* 
* @param {Integer=} optional x-coordinate to obtain column number for, in lieu of the x            coordinate of the last mouse event
*/
getEventColumn(x?:Integer): Integer;

/**
*  Return the CSS class for a cell. By default this method has the following implementation:
*  - return any custom style for the record (GridRenderer.recordCustomStyleProperty) 
*   if defined.
*  - create a style name based on the result of GridRenderer.getBaseStyle and the 
*   state of the record using the rules described in cellStyleSuffixes.
*  
*  Cell Styles are customizable by:
*  
*  
* - attaching a custom style to a record by setting 
*   record[this.recordCustomStyleProperty] to some valid CSS style name. 
*  
* - modifying the base style returned by getBaseStyle() [see that method for further 
*    documentation on this]
*  
* - overriding this function
*  
*  
*  In addition to this, GridRenderer.getCellCSSText may be overriden to
*  provide custom cssText to apply on top of the styling attributes derived from the named
*  style.
*  
* 
* 
* @param {ListGridRecord} record object for this row and column
* @param {number} number of the row
* @param {number} number of the column
*/
getCellStyle(record:ListGridRecord, rowNum:number, colNum:number): CSSStyleName;

/**
*  Return the base stylename for this cell. Default implementation just returns this.baseStyle.
*  See ListGrid.getCellStyle for a general discussion of how to style cells.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getBaseStyle(record:ListGridRecord, rowNum:number, colNum:number): CSSStyleName;

/**
*  Note: This is a ListGrid feature which is inapplicable on this class.
* 
* @param {ListGridViewState} Object describing the desired view state for the grid
*/
setViewState(viewState:ListGridViewState): void;

/**
*  Note: This is a ListGrid feature which is inapplicable on this class.
* 
*/
getViewState(): ListGridViewState;

/**
*  Set the temporary edit values for some cell in the cubeGrid.
*  Note that only the CubeGrid.valueProperty of the object passed in
*  will be displayed in the cell.
* 
* @param {Array<Partial<number>>} 2 element array of the form          [rowNum,colNum] indicating the record being edited
* @param {object} New values for the record
*/
setEditValues(cell:Array<Partial<number>>, values:object): void;

/**
* 
*  Set the edit value for some cell in the cube grid.
*  Note that cubeGrids display one record per cell - the value passed in should be the 
*  desired edit value for the CubeGrid.valueProperty of the record.
* 
* 
* @param {number} Row number
* @param {number} Column number
* @param {any} New value for the record
*/
setEditValue(rowNum:number, colNum:number, value:any): void;

/**
* 
*  Returns the current temporary locally stored edit value for a cell being edited.
*  Note this is the CubeGrid.valueProperty that will be saved for the
*  cell in question.
* 
* 
* @param {number} index of the row for which the editValue should be returned
* @param {number} index of the column for which value should be returned
*/
getEditValue(rowNum:number, colNum:number): any;

/**
*  Only supported on ListGrid for now.
* 
* @param {Array<Partial<Hilite>>} Array of hilite objects
*/
setHilites(hilites:Array<Partial<Hilite>>): void;

/**
*  Returns the combination of unsaved edits (if any) and original values (if any) for a given
*  cell being edited.
*  
*  The returned value is never null, and can be freely modified.
* 
* 
* @param {number | object} rowNum of the record being edited, or an Object                   containing values for all the record's primary keys
* @param {number=} colNum of the record being edited. Only required if the records             rowNum is passed in as the first parameter
*/
getEditedRecord(valuesID:number | object, colNum?:number): object;

/**
*  Returns the current value of a cell. If the cell has an outstanding edit value, this will
*  be returned, otherwise the underlying value of the record will be returned.
* 
* 
* @param {number | object} rowNum of the record being edited, or an Object                   containing values for all the record's primary keys
* @param {number} colNum of the cell. Only required if the first parameter is               a rowNum
*/
getEditedCell(record:number | object, colNum:number): any;

/**
*  Returns the current set of unsaved edits for a given row being edited.
* 
* 
* @param {number | object} rowNum of the record being edited, or an Object                   containing values for all the record's primary keys
* @param {number} colNum of the record being edited. Only required if valuesID             is passed in as a rowNum.
*/
getEditValues(valuesID:number | object, colNum:number): object;

/**
* 
*  Clear a field value being tracked as an unsaved user edit for some cell.
*  The saved record value will be displayed in the the appropriate cell instead.
* 
* 
* @param {number | object} Row number, primary keys object for the record,                       or editValues object
* @param {number=} Column number for the cell in question. Only required if the               first parameter is a row number.
*/
clearEditValue(editValuesID:number | object, colNum?:number): void;

/**
* 
*  Validates and saves edits for some cell. If rowNum and colNum are not passed in, the current
*  edit cell will be saved.
*  
*  The 'callback' parameter provides a notification when the save attempt completes.
*  Cases under which the callback will fire are:
*  
* - Save completed successfully
*  
* - No changes to the edited cell, so save not required
*  
* - Validation failure occurred on the client or on the server
*  Note that if no rowNum/colNum were passed in and the editor is not showing for the cell,
*  the callback will NOT fire - in this case, the method is a no-op.
*  
*  Other, standard callbacks such as ListGrid.editComplete,
*  ListGrid.editFailed and ListGrid.cellChanged will
*  fire normally.
*  
*  Note this method does not hide the inline editors if they are showing - to explicitly save 
*  and end editing, use the method 'endEditing()'
* 
* 
* @param {EditCompletionEvent=} Event used to complete cell editing.     Optional, and defaults to "programmatic". Can be used by the      callback method to perform custom actions such as navigation when the      save completes.
* @param {Callback=} Callback to fire on completion of the saving process.                  If no edits were made or client-side validation fails                   the callback will be fired synchronously at the end of this                   method.                   Takes the following parameters:                   - rowNum (Number) edited row number                   - colNum (Number) edited column number                   - editCompletionEvent (EditCompletionEvent) event                    passed in (defaults to "programmatic")                   - success (boolean) false if the save was unable to                    complete due to a validation failure or server-side                    error.
* @param {number=} Which row should be saved. If unspecified the current edit row              is saved by default. Note that if there is no current edit cell               this method will no op.
* @param {number=} Which row should be saved. If unspecified the current edit column              is saved by default. Note that if there is no current edit cell               this method will no op.
*/
saveEdits(editCompletionEvent?:EditCompletionEvent, callback?:Callback, rowNum?:number, colNum?:number): void;

/**
*  Method to determine which records currently have pending (unsubmitted) edits.
*  Returns an array of 2 element arrays indicating the [rowNum,colNum] of the
*  cells in question.
* 
*/
getAllEditCells(): Array<any>;

/**
* 
* 
* @param {Array<Partial<any>>=} allows you to specify which cell(s) to drop          edits for. Each record should be identified as an array containing          [rowNum,colNum]
* @param {boolean=} By default this method will hide the editor if               it is currently showing for any row in the grid. Passing in                this parameter will leave the editor visible (and just reset                the edit values underneath the editor).
*/
discardAllEdits(records?:Array<Partial<any>>, dontHideEditor?:boolean): void;

/**
*  If this cubeGrid can be edited, this method will return true if the record passed in has been
*  edited, but the edits have not yet been saved to the CubeGrid's data object.
*  
*  Note that if this grid is bound to a ListGrid.dataSource, and 
*  an asynchronous save has been submitted, this method will compare the local edit values 
*  against the submitted values by default, returning false
*  (no changes), if they match. This is useful for detecting whether the user is actively
*  editing values and hasn't yet committed them.
*  
*  The ignorePendingValues parameter may be used by
*  developers who want to ignore this case and simply compare edit values
*  against the record in the local data set.
* 
* 
* @param {number} row index of record to check for changes
* @param {number} column index of the record to check for changes
* @param {boolean=} If true, this method will compare the current edit values against the underlying records in the dataset, not taking pending edit values into account
*/
recordHasChanges(rowNum:number, colNum:number, ignorePendingValues?:boolean): boolean;

/**
*  Determines whether any cells in this cubeGrid have been edited but not yet saved to the
*  underlying data set. Note that for asynchronous saves, this method will return false
*  if the current edit values match any submitted (but not yet saved) values. This behavior
*  can be turned off with the ignorePendingValues parameter.
* 
* @param {boolean=} If true, this method will compare the current edit values against the underlying records in the dataset, not taking pending edit values into account
*/
hasChanges(ignorePendingValues?:boolean): boolean;

/**
*  Save a number of outstanding edits for this CubeGrid. If no cells are specified, all 
*  outstanding edits will be saved
* 
* @param {Array<Partial<any>>=} optionally specify which cells to save. Each cell should be specified as a 2 element array in the format [rowNum,colNum].
* @param {Callback=} If specified this callback will be fired on a successful save of the specified rows. Note that if there are no pending edits to be saved this callback will  not fire - you can check for this condition using CubeGrid.hasChanges or  CubeGrid.recordHasChanges.
*/
saveAllEdits(cells?:Array<Partial<any>>, saveCallback?:Callback): boolean;

/**
*  Get a facet definition by facetId. Constant time.
* 
* 
* @param {string} the id of the facet to retrieve
*/
getFacet(facetId:string): Facet;

/**
*  Get a facet value definition by facetId and facetValueId. Constant time.
* 
* 
* @param {string} the id of the facet to retrieve
* @param {string} the id of the facet value to retrieve
*/
getFacetValue(facetId:string, facetValueId:string): FacetValue;

/**
*  Get the facetValue definition for the facetValue to show when this facet is "rolled up" under
*  another facet, during a breakout.
* 
*  A facet is not required to have a rollup value, and if it does not have one, then rollups will
*  simply be blank rows. The facetValueId of the rollup value can be declared as
*  cubeGrid.rollupValue or facet.rollupValue. 
* 
* 
*/
getRollupValue(): string;

/**
*  Given a cell coordinate within this CubeGrid return a FacetValueMap
*  indicating the facet values for the cell.
* 
* @param {number} row index of the cell
* @param {number} column index of the cell
*/
getCellFacetValues(rowNum:number, colNum:number): FacetValueMap;

/**
*  Return a FacetValueMap of the facet values for the row field at the specified level
*  containing the requested row number. Note that outer row fields may span several grid rows.
* 
* 
* @param {Integer} 0-based index into the grid rows (and inner row header fields)
* @param {Integer} target header level; 0 represents the outer row header
*/
getRowHeaderFacetValues(rowNum:Integer, level:Integer): FacetValueMap;

/**
*  Return a FacetValueMap of the facet values for the column field at the specified
*  level containing the requested column number. Note that outer column fields may span several
*  grid columns.
* 
* 
* @param {Integer} 0-based index into the grid columns (and inner column header fields)
* @param {Integer} target header level; 0 represents the outer column header
*/
getColumnHeaderFacetValues(colNum:Integer, level:Integer): FacetValueMap;

/**
*  Get the index of the first column in the grid that matches the specified FacetValueMap.
*  
*  The facetValues passed in should contain values for at least one column facet. It may
*  contain properties other than column facets, which will be ignored. If values are 
*  sparse (values not specified for every column facet), the first column matching the specified
*  facet values will be returned.
* 
* 
* @param {FacetValueMap} facet values to find
*/
getFacetValuesColumn(facetValues:FacetValueMap): Integer;

/**
*  Get the index of the first row in the grid that matches the specified FacetValueMap.
*  
*  The facetValues passed in should contain values for at least one row facet. It may contain
*  properties other than row facets, which will be ignored. If values are 
*  sparse (values not specified for every row facet), the first row matching the specified
*  facet values will be returned.
* 
* 
* @param {FacetValueMap} facet values to find
*/
getFacetValuesRow(facetValues:FacetValueMap): Integer;

/**
*  Return a FacetValueMap indicating the facet values for a specific 
*  row in the grid.
* 
* @param {number} index of the row
*/
getRowFacetValues(rowNum:number): FacetValueMap;

/**
*  Return a FacetValueMap indicating the facet values for a specific 
*  column in the grid.
* 
* @param {number} index of the column
*/
getColumnFacetValues(colNum:number): FacetValueMap;

/**
*  Return the pointer to a particular record by record and column number.
* 
* @param {number} row index of record to return.
* @param {number} column index of record to return.
*/
getCellRecord(rowNum:number, colNum:number): ListGridRecord;

/**
*  Given a record in this grid, this method returns the rowNum on which the record is displayed.
* 
* @param {CellRecord} record to find coordinates for
*/
getCellRow(cellRecord:CellRecord): Integer;

/**
*  Given a record in this grid, this method returns the colNum in which the record is displayed.
* 
* @param {CellRecord} record to find coordinates for
*/
getCellColumn(cellRecord:CellRecord): Integer;

/**
*  Given a record in this grid, this method returns the coordinates of the cell in which the 
*  record is displayed as a 2 element array.
* 
* @param {CellRecord} record to find coordinates for
*/
getCellCoordinates(record:CellRecord): Array<any>;

/**
*  Notification method fired when new data arrives from the server to be displayed in this
*  CubeGrid. For example in response to the user openng a collapsed facet, or as a result of
*  an initial fetch request for all data from a CubeGrid where CubeGrid.facets is not
*  set and there is no initial data. Only applies to databound CubeGrids.
* 
*/
dataArrived(): void;

/**
*  This method is not currently supported for this grid-type. See 
*  ListGrid.loadAllRecords for more information.
* 
* 
* @param {Integer=} optional maximum record count - if passed, no fetch takes place                if maxRecords is below the known length of the data
* @param {DSCallback=} callback to fire if a fetch is issued - if all data was                 already loaded, the callback is fired with no parameters
*/
loadAllRecords(maxRecords?:Integer, callback?:DSCallback): boolean;

/**
*  Method handler fired when the user clicks on a facet label.
* 
* 
*/
facetLabelClick(): void;

/**
*  Get the HTML for the facet label hover. Default implementation returns null.
*  methodType callback
* 
* @param {string} ID for the facet
*/
facetLabelHoverHTML(facetId:string): void;

/**
*  Get the HTML for the facetValue button hover. Default implementation returns null.
*  
* 
* @param {FacetValueMap} facetValues object for the button
*/
facetValueHoverHTML(facetValues:FacetValueMap): void;

/**
*  Toggles the open state of the specified field. No-ops if it's not showing.
* 
* @param {FacetValueMap} field specified as a facetValueMap
*/
toggleFieldOpenState(facetValueMap:FacetValueMap): boolean;

/**
*  Return whether the specified CubeGrid field is open, taking into account both
*  Facet.canCollapse and Facet.canMinimize.
*  
*  Note that if you don't already have a FacetValueMap to the field in question, you can
*  get one by calling CubeGrid.getRowHeaderFacetValues or CubeGrid.getColumnHeaderFacetValues,
*  
*  You can also construct a FacetValueMap on your own by using the Facet.ids from
*  CubeGrid.rowFacets or CubeGrid.columnFacets together with the FacetValue.ids of the
*  Facet.values for the row or column that you want to query. Given a Facet.id,
*  you can use CubeGrid.getFacet to obtain the correponding Facet.
* 
* 
* @param {FacetValueMap} field specified as a facetValueMap
*/
isFieldOpen(facetValueMap:FacetValueMap): boolean;

/**
*  Expands the specified field. No-ops if it's not showing, or if it's already expanded.
* 
* @param {FacetValueMap} field specified as a facetValueMap
*/
expandField(facetValueMap:FacetValueMap): boolean;

/**
*  Collapses the specified field. No-ops if it's not showing, or it it's already collapsed.
* 
* @param {FacetValueMap} field specified as a facetValueMap
*/
collapseField(facetValueMap:FacetValueMap): boolean;

/**
*  auto-size the header facet horizontally
* 
* @param {string} ID of facet to resize.
*/
autoSizeFacet(facetId:string): void;

/**
* 
*  Handler fired when facet is closed
* 
*    methodType handler
* 
* @param {string} ID of facet that was closed
*/
closeFacet(facetId:string): void;

/**
* 
*  Handler fired when column is closed
* 
*    methodType handler
* 
* @param {FacetValue} FacetValues for the appropriate col.
*/
closeColumn(headerFacetValues:FacetValue): void;

/**
*  Hides the specified field if it is currently visible. No-ops if it's already hidden.
* 
* @param {FacetValueMap} field specified as a facetValueMap
*/
hideFacetValues(facetValueMap:FacetValueMap): boolean;

/**
*  Shows the specified field if it was previsouly hidden. No-ops if it's already showing.
* 
* @param {FacetValueMap} field specified as a facetValueMap
*/
showFacetValues(facetValueMap:FacetValueMap): boolean;

/**
*  Handler/Notification function for facetValue selection change (no default implementation).
* 
* 
* @param {FacetValueMap} facetValues with new selection state
* @param {boolean} new selection state
*/
facetValueSelectionChanged(facetValues:FacetValueMap, newState:boolean): void;

/**
*  Returns an array of the selected cell records.
* 
*  methodType getter
* 
*/
getSelectedCells(): Array<Partial<CellRecord>>;

/**
*  Returns an array of the IDs of all selected cell records.
* 
*  methodType getter
* 
*/
getSelectedCellIds(): Array<Partial<string>>;

/**
*  Determine whether any cells are selected in this cubeGrid.
* 
*  methodType tester
* 
*/
anyCellSelected(): boolean;

/**
*  Determine whether the cell passed in is selected in this cubeGrid.
* 
*  methodType tester
* 
* @param {CellRecord} cell to test
*/
cellIsSelected(cell:CellRecord): boolean;

/**
*  Select / deselect a single cell - accepts cell ID or CellRecord.
* 
* 
* @param {CellRecord | GlobalId} cell to select / deselect
* @param {boolean=} new selection state (if null, defaults to true)
*/
selectCell(cell:CellRecord | GlobalId, newState?:boolean): void;

/**
*  Deselect a single cell - accepts cell ID or cell record.
* 
* 
* @param {CellRecord | GlobalId} cell to deselect
*/
deselectCell(cell:CellRecord | GlobalId): void;

/**
*  Select/deselect cells that match a FacetValueMap.
*  Also supports an explicit list of CellRecords or cell IDs.
* 
* 
* @param {Array<Partial<CellRecord>> | FacetValueMap | Array<Partial<GlobalId>>} cells to select
* @param {boolean=} new selection state (if null, defaults to true)
*/
selectCells(cellList:Array<Partial<CellRecord>> | FacetValueMap | Array<Partial<GlobalId>>, newState?:boolean): void;

/**
*  Deselect cells that match a FacetValueMap.
*  Also supports an explicit list of CellRecords or cell IDs.
* 
* 
* @param {Array<Partial<CellRecord>> | FacetValueMap | Array<Partial<GlobalId>>} cells to deselect
*/
deselectCells(cellList:Array<Partial<CellRecord>> | FacetValueMap | Array<Partial<GlobalId>>): void;

/**
*  Select all cells.
* 
* 
*/
selectAllCells(): void;

/**
*  Deselect all cells.
* 
* 
*/
deselectAllCells(): void;

/**
*  Returns an array of facetValues objects indicating the headers that are selected in the
*  headerBar for this facet. If facetId is not passed, returns selection for all
*  facets.
* 
* 
* @param {string=} id for facet for which we are getting selected facetValues
*/
getSelectedFacetValues(facetId?:string): Array<Partial<FacetValueMap>>;

/**
*  Return whether the header indicated by the set of facetValues is selected.
* 
* 
* @param {FacetValueMap} facetValues to test
*/
facetValuesSelected(facetValues:FacetValueMap): boolean;

/**
*  Return whether any facet value for this facet is selected in headers. If no facetId passed,
*  return whether any facet has a selection.
* 
*  methodType tester
* 
* @param {string} Id for facet to test
*/
facetHasSelection(facetId:string): boolean;

/**
*  Return the list of facets that have any selection in their headers.
* 
*  methodType getter
* 
*/
getFacetsHavingSelection(): Array<Partial<string>>;

/**
*  Select/deselect the header showing a given set of facet values.
* 
* 
* @param {FacetValueMap} facetValues to select the header of
* @param {boolean=} new selection state - if null defaults to true
*/
selectFacetValues(facetValues:FacetValueMap, newState?:boolean): void;

/**
*  Deselect the header showing a given set of facet values.
* 
*  methodType action
* 
* @param {FacetValueMap} facetValues to deselect
*/
deselectFacetValues(facetValues:FacetValueMap): void;

/**
*  Select/deselect the header for a given facet value.
* 
*  methodType action
* 
* @param {string} ID of facet
* @param {string} ID of facetValue to select
* @param {boolean=} new selection state - if null defaults to true
*/
selectFacetValue(facetId:string, facetValueId:string, newState?:boolean): void;

/**
*  Deselect the header for a given facet value.
* 
*  methodType action
* 
* @param {string} ID of facet
* @param {string} ID of facetValue to select
*/
deselectFacetValue(facetId:string, facetValueId:string): void;

/**
*  Select/deselect all headers in a headerBar (specified by facetId) or
*  all headerBars (if no facetId).
* 
*  methodType action
* 
* @param {string=} ID of facet - if null, selects all headerbars' headers
* @param {boolean=} new selection state - if null defaults to true
*/
selectAllFacetValues(facetId?:string, newState?:boolean): void;

/**
*  Deselect all headers in a headerBar (specified by facetId) or
*  all headerBars (if no facetId).
* 
*  methodType action
* 
* @param {string=} ID of facet - if null, selects all headerbars' headers
*/
deselectAllFacetValues(facetId?:string): void;

/**
*  Deselect all cells and facetValues.
* 
*    methodType action
* 
*/
deselectAll(): void;

/**
*  Resizes all columns for the provided facetValueId, which must be a facetValueId from the
*  innermost column facet.
* 
* 
* @param {Identifier} facetValueId of columns to be resized
* @param {number} column's new width
*/
resizeFacetValue(facetValueId:Identifier, newWidth:number): void;

/**
*    Apply a hilite to a specific cell.
*    Note: can be called either as 
*      hiliteCell(cellObject, hiliteID) or 
*      hiliteCell(row, column, hiliteID)
* 
*    methodType action
* 
* @param {any | any} cell to hilite / row of cell to hilite
* @param {string | any} hilite ID / column of cell to hilite
* @param {string=} optional third parameter - hilite ID.
*/
hiliteCell(cellObj:any | any, hiliteID:string | any, alternateHiliteID?:string): boolean;

/**
*    Apply a hilite to an array of cells.
* 
*    methodType action
* 
* @param {Array<Partial<any>>} cells to hilite
* @param {string} ID of hilite to apply to cells
*/
hiliteCellList(cellObjList:Array<Partial<any>>, hiliteID:string): boolean;

/**
*    Apply a hilite to all cells corresponding to a facetValue.
* 
*    methodType action
* 
* @param {string} facet ID
* @param {string} facet value ID
* @param {string} hilite ID
*/
hiliteFacetValue(facetID:string, facetValueID:string, hiliteID:string): boolean;

/**
*  Set the title of a facet (appears in facet label).
* 
* 
* @param {Identifier} facet to update
* @param {string} title for the facet
*/
setFacetTitle(facetId:Identifier, newTitle:string): void;

/**
*  Set the align of a facet title (appears in facet label).
* 
* 
* @param {Identifier} facet to update
* @param {Alignment} new alignment for facet title
*/
setFacetTitleAlign(facetId:Identifier, align:Alignment): void;

/**
*  Set the title for a facet value.
* 
* 
* @param {Identifier} facet to update
* @param {Identifier} facetValue to update
* @param {string} title for the facet
*/
setFacetValueTitle(facetId:Identifier, facetValueId:Identifier, newTitle:string): void;

/**
*  Set the align for the title for a facet value.
* 
* 
* @param {Identifier} facet to update
* @param {Identifier} facetValue to update
* @param {Alignment} new alignment for facet value title
*/
setFacetValueTitleAlign(facetId:Identifier, facetValueId:Identifier, align:Alignment): void;

/**
*  Get the current visual order and width for the facet values of a facet or facetValueGroup as
*  an object of the form:
*  
*  [ {id:facetValueId, width:currentWidth }, ... ]
*  
* 
* 
* @param {string | string} Which facet do we want details for?
*/
getFacetValueLayout(id:string | string): Array<any>;

/**
*  Get the current widths of the row facets, as an object of the form:
*  
*  [ {facetId:facetId, width:currentWidth }, ... ]
*  
* 
* 
*/
getRowFacetLayout(): Array<any>;

/**
*  Get the current heights of the column facets, as an object of the form:
*  
*  [ {facetId:facetId, height:currentHeight}, ... ]
*  
* 
* 
*/
getColumnFacetLayout(): Array<any>;

/**
*  Setter for the CubeGrid.enableCharting attribute.
* 
* @param {boolean} */
setEnableCharting(enableCharting:boolean): void;

/**
*  Chart the portion of the dataset indicated by fixedFacetValues, for all 
*  values of the variableFacets. Note that the current CubeGrid.fixedFacetValues of
*  the CubeGrid will be implicitly added to whatever you pass as fixedFacetValues,
*  since the idea is that this chart should be taken from data available in the CubeGrid.
*  
*  One, two or more variableFacets may be passed. Two variable facets for a column chart will
*  result in Chart.stacked or clustering. Three facets or more may be
*  supported by some CubeGrid.chartType or
*  CubeGrid.chartConstructor.
*  
* 
* @param {FacetValueMap} set of facet values to hold constant. Pass null to                     chart the entire dataset.
* @param {Array<Partial<string>>} set of facets to be charted
* @param {Partial<Chart>} properties to pass through to the created            Chart
*/
makeChart(fixedFacetValues:FacetValueMap, variableFacets:Array<Partial<string>>, chartProperties:Partial<Chart>): Chart;

/**
*  Returns a default set of items, which can be updated/modified, and then assigned to
*  CubeGrid.facetValueContextItems to be used in the context menu of the appropriate 
*  header button.
* 
* 
* @param {FacetValueMap} FacetValueMap for the appropriate header button
*/
getDefaultFacetValueContextItems(facetValues:FacetValueMap): Array<Partial<MenuItem>>;

/**
*  Add a row facet to the view at index "index". Handles the facet already being in the view
*  (does a pivot).
* 
*  The facet being added should currently have a fixed facet value (unless it's already part of the
*  view), which will be removed from cubeGrid.fixedFacetValues.
* 
*  methodType action
* 
* @param {Identifier} facetId to add. Definition must have been provided at init time.
* @param {Integer=} index to add the facet at. 0 = outermost (default innermost)
*/
addRowFacet(facetId:Identifier, index?:Integer): void;

/**
*  Add a column facet to the view at index "index". Handles the facet already being in the view
*  (does a pivot).
* 
*  The facet being added should currently have a fixed facet value (unless it's already part of the
*  view), which will be removed from cubeGrid.fixedFacetValues.
* 
*  methodType action
* 
* @param {Identifier} facetId to add. Definition must have been provided at init time.
* @param {Integer=} index to add the facet at. 0 = outermost (default innermost)
*/
addColumnFacet(facetId:Identifier, index?:Integer): void;

/**
* 
*  Add a facet to the view, into the row or column facets (intoRows true or false), at index
*  "index". Handles the facet already being in the view (does a pivot).
*  
*  The facet being added should currently have a fixed facet value (unless it's already part of the
*  view), which will be removed from cubeGrid.fixedFacetValues.
* 
* 
* @param {Identifier} facetId to add. Definition must have been provided at init time.
* @param {boolean=} [true] whether to add facet as a row facet
* @param {Integer=} index to add the facet at. 0 = outermost (default innermost)
*/
addFacet(facetId:Identifier, intoRows?:boolean, index?:Integer): void;

/**
* 
*  Remove a facet from the current view, using a fixed value from that facet. For example, remove
*  the "months" facet from the view, collapsing to just January, or total for all months.
* 
* 
* @param {Identifier} facetId to remove
* @param {Identifier=} New fixed value for the facet, to be added to             CubeGrid.fixedFacetValues. Default is the rollup value for the facet.
*/
removeFacet(facetId:Identifier, fixedFacetValueId?:Identifier): void;

/**
* 
*  Modify fixedFacetValues for this cubeGrid.
* 
* 
* @param {Identifier} facetId
* @param {Identifier} New fixed value for the facet, to be added to             CubeGrid.fixedFacetValues. Default is the rollup value for the facet.
*/
setFixedFacetValue(facetId:Identifier, fixedFacetValueId:Identifier): void;

/**
*  Note that CubeGrid does not support a WYSIWYG print view by default(also used when
*  RPCManager.exportContent). Instead we recommend 
*  DataSource.exportClientData.
* 
* 
* @param {PrintProperties=} properties to configure printing behavior - may be null.
* @param {Callback=} optional callback. This is required to handle cases where HTML         generation is asynchronous - if a method generates HTML asynchronously, it should return          null, and fire the specified callback on completion of HTML generation. The          first parameter HTML should contain the generated print HTML.          The callback is only called if null is returned. Furthermore, the default          getPrintHTML() implementation always returns null and fires the callback          when a callback is provided.
*/
getPrintHTML(printProperties?:PrintProperties, callback?:Callback): HTMLString;

/**
*  Exports this component's data with client-side formatters applied, so is suitable for direct
*  display to users. This feature requires the SmartClient server.
*  
*  The export format will combine the column facet value titles, generating a single row of
*  column headers at the top with titles such as "All Years - Budget" if Time and Scenario were
*  column facets. The row facet value titles for separate facets won't be combined, so that
*  each row facet will have a separate column, with the facet titles at the top in the 
*  "column header" row, and the row facet value titles below their corresponding facet title.
*  Data values each get their own row and column position.
* 
* 
* @param {object} contains configuration settings for the export, including:  
- facetSeparatorString (String) - if specified, the separator to use in favor   of CubeGrid.exportFacetSeparatorString when combining titles from multiple   facet values. 
* @param {Partial<DSRequest>=} Request properties for the export.
* @param {RPCCallback=} Optional callback. If you specify DSRequest.exportToClient: false in the request  properties, this callback will fire after export completes. Otherwise the callback will  fire right before the download request is made to the server.
*/
exportClientData(settings:object, requestProperties?:Partial<DSRequest>, callback?:(response:RPCResponse, rawData:any, request:RPCRequest) => void): void;

/**
*  StringMethod handler fired when mouseover occurs over a facetValues header button
*  
* 
* @param {FacetValueMap} FacetValueMap for the appropriate header button
*/
facetValueOver(facetValues:FacetValueMap): void;

/**
*  StringMethod handler fired when mouseout occurs for a facetValues header button
* 
* @param {FacetValueMap} FacetValueMap for the appropriate header button
*/
facetValueOut(facetValues:FacetValueMap): void;

/**
*  StringMethod handler fired when mouse hovers over a facetValue button in a header.
* 
* @param {FacetValueMap} FacetValueMap for the appropriate header button
*/
facetValueHover(facetValues:FacetValueMap): void;

/**
*  StringMethod handler fired when context click occurs over a facetValue button in a header.
* 
* 
* @param {FacetValueMap} FacetValueMap describing the appropriate header button
*/
facetValueContextClick(facetValues:FacetValueMap): void;

/**
*  StringMethod handler fired when the user right clicks on a facet label.
* 
* 
* @param {string} ID of facet
*/
facetContextClick(facetId:string): void;

/**
*  
*  Notification fired when a facet or facetValueGroup is reordered
* 
* 
* @param {string} facetValueGroupId or facetId
*/
facetValueReordered(groupId:string): void;

/**
*    Notification fired when a new facet is added.
*  
* 
* @param {string} facetId that was added
*/
facetAdded(facetId:string): void;

/**
*  Notification fired when a facet is removed.
*  
* 
* @param {string} facetId that was removed
*/
facetRemoved(facetId:string): void;

/**
*  Notification fired when a facet is moved.
*  
* 
* @param {string} facetId which moved
*/
facetMoved(facetId:string): void;

/**
*  Notification fired when a fixed facet value is set for some facet.
* 
* @param {string} facetId
* @param {string} new fixed facet value
*/
fixedFacetValueChanged(facetId:string, facetValueId:string): void;

/**
*  Called when a sort control is clicked on a FacetValueHeader. Does nothing by default.
*  
* 
* @param {FacetValueMap} facetValues to sort
* @param {boolean} true for ascending
*/
sortByFacetValues(facetValues:FacetValueMap, sortDirection:boolean): void;

/**
*  Called when a sort control is clicked on a FacetHeader. Does nothing by default.
*  
* 
* @param {string} ID of facet to sort
* @param {boolean} true for ascending
*/
sortByFacetId(facetId:string, sortDirection:boolean): void;

/**
*  StringMethod handler fired when mouseover occurs over a facet label
* 
* 
* 
* @param {string} ID of the appropriate facet
*/
facetLabelOver(facetId:string): void;

/**
*  StringMethod handler fired when mouseout occurs over a facet label
* 
* 
* @param {string} ID of the appropriate facet
*/
facetLabelOut(facetId:string): void;

/**
*  StringMethod handler fired from hover over a facet label
* 
* 
* @param {string} ID of the appropriate facet
*/
facetLabelHover(facetId:string): void;

/**
* 
*    Handler/Notification function for cell selection change
*    May update header button styles.
* 
*    methodType handler
* 
* @param {Array<Partial<CellRecord>>} array of cells with new selection state
*/
cellSelectionChanged(cellList:Array<Partial<CellRecord>>): boolean;



}
/**
* Creates a new CubeGrid
* 
* @param typeCheckedProperties {Partial<CubeGrid>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {CubeGrid} The new instance
*/
class CubeGrid{
static create(typeCheckedProperties? : Partial<CubeGrid>, uncheckedProperties?: Object): CubeGrid;
/**
* A declared value of the enum type 
*  FacetIndentDirection.
* @type {Constant}
* @default "descending"
*/
static DESCENDING?: Constant;

/**
* A declared value of the enum type 
*  FacetIndentDirection.
* @type {Constant}
* @default "ascending"
*/
static ASCENDING?: Constant;








}

/**
* Static singleton class with APIs for dmiOverview of
*  server side methods when running the SmartClient java server.
*/
interface DMI extends Class {




}
/**
* Creates a new DMI
* 
* @param typeCheckedProperties {Partial<DMI>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DMI} The new instance
*/
class DMI{
static create(typeCheckedProperties? : Partial<DMI>, uncheckedProperties?: Object): DMI;




/**
* 
*  Calls a server-side DMI method. At a minimum, you need to specify the appID 
*  (applicationDeclaration), ServerObject.className
*  or ServerObject.ID and methodName to call.
*  Arguments and callback are optional. There are two ways to invoke this method:
*  
*  DMI.call(appID, className, methodName, 
*      arg1, arg2 ...argN, callback);
*  
*  or:
*  
*  DMI.call({
*    appID: appID,
*    className: className,
*    methodName: methodName,
*    arguments: [arg1, arg2, ...argN], //optional
*    callback: callback, //optional
*    requestParams: requestProps // optional
*  });
*  
*  If you use the first signature, you must either specify a callback or if you don't want a
*  callback, pass a null as the last argument. The second signature allows you to
*  specify requestParams that are applied to the RPCRequest generated by this DMI call.
*  This allows you to override some defaults - for example to suppress the "Contacting Server"
*  prompt, change it's text; change the timeout or set any other property settable on
*  RPCRequest.
*  
*  Arguments to be passed to the server method may be specified.
*  If present, each argument will be serialized into JSON, 
*  sent to the server and translated to an equivalent Java object to be passed into the
*  method. The translation follows the same rules as for data passed to the server as 
*  part of a standard RPCRequest.data. 
*  For example if a server side method has the signature
*  
*  someMethod(String title, Map overrides);
*  
*  a DMI call to that method would could pass in a standard String and JavaScript object -
*  for example:
*  
*  isc.DMI.call({
*    appID: "someApp",
*    className: "com.smartclient.demo.MyClass",
*    methodName: "someMethod",
*    arguments: ["Title String", {field1:"Value 1", field2:"Value 2"}]
*  });
*  
*  
*  Note that you can use the loadDMIStubsTag tag to bind all methods of
*  ServerObjects defined in a given .app.xml file and call methods on them
*  directly.
* 
* 
* @param {string | object} the appID (.app.xml file to look in) or comprehensive request             object as documented above.
* @param {string} ServerObject.className or ServerObject.ID
* @param {string} the name of the method to call on the serverObject
* @param {any} The next N-1 params specify arguments to the server-side method.
* @param {RPCCallback} The callback of the response. If you do not want a callback, you             must specify a null value for this parameter when              using the first signature (documented above).
*/
static call(appID:string | object, className:string, methodName:string, args:any, callback:(response:RPCResponse, rawData:any, request:RPCRequest) => void): RPCRequest;

/**
* 
*  Returns a URL to a server-side DMI method. At a minimum, you need to specify the appID 
*  (applicationDeclaration), ServerObject.className
*  or ServerObject.ID and methodName to call.
*  Arguments are optional. There are two ways to invoke this method:
*  
*  DMI.getURL(appID, className, methodName, 
*      arg1, arg2 ...argN);
*  
*  or:
*  
*  DMI.getURL({
*    appID: appID,
*    className: className,
*    methodName: methodName,
*    arguments: [arg1, arg2, ...argN], //optional
*    requestParams: requestProps // optional
*  });
*  
*  The second signature allows you to specify requestParams that are applied to the
*  RPCRequest generated by this DMI call.
*  
*  Note that because the entirety of the request is encoded in the URL, there is an
*  inherent limitation on the amount of data that you can send viat he criteria argument to
*  the server. The actual length depends on your server configuration and other factors
*  such as the size of cookies (if any) being sent to the server and other HTTP headers in
*  use. Conservatively, assume that you have about 2 kilobytes to work with.
* 
* 
* @param {string | object} the appID (.app.xml file to look in) or comprehensive                     request object as documented above.
* @param {string} ServerObject.className or ServerObject.ID
* @param {string} the name of the method to call on the serverObject
* @param {any=} The next N-1 params specify arguments to the server-side method.
*/
static getURL(appID:string | object, className:string, methodName:string, args?:any): string;




}

/**
* Provides a tree view of any DOM-compliant structure, such as an XML or HTML document.
*/
interface DOMGrid extends TreeGrid {
/**
* Root element (or document) to view in the tree.
* @type {DOMElement}
* @default null
*/
rootElement?: DOMElement;



/**
*  Set the root element (or document) to view in the tree.
* 
* @param {DOMElement} new root element
*/
setRootElement(rootElement:DOMElement): void;



}
/**
* Creates a new DOMGrid
* 
* @param typeCheckedProperties {Partial<DOMGrid>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DOMGrid} The new instance
*/
class DOMGrid{
static create(typeCheckedProperties? : Partial<DOMGrid>, uncheckedProperties?: Object): DOMGrid;







}

/**
* A ServiceTask configured to perform an add.
*/
interface DSAddTask extends ServiceTask {




}
/**
* Creates a new DSAddTask
* 
* @param typeCheckedProperties {Partial<DSAddTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DSAddTask} The new instance
*/
class DSAddTask{
static create(typeCheckedProperties? : Partial<DSAddTask>, uncheckedProperties?: Object): DSAddTask;







}

/**
* A ServiceTask configured to perform a fetch.
*/
interface DSFetchTask extends ServiceTask {




}
/**
* Creates a new DSFetchTask
* 
* @param typeCheckedProperties {Partial<DSFetchTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DSFetchTask} The new instance
*/
class DSFetchTask{
static create(typeCheckedProperties? : Partial<DSFetchTask>, uncheckedProperties?: Object): DSFetchTask;







}

/**
* A ServiceTask configured to perform a remove.
*/
interface DSRemoveTask extends ServiceTask {




}
/**
* Creates a new DSRemoveTask
* 
* @param typeCheckedProperties {Partial<DSRemoveTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DSRemoveTask} The new instance
*/
class DSRemoveTask{
static create(typeCheckedProperties? : Partial<DSRemoveTask>, uncheckedProperties?: Object): DSRemoveTask;







}

/**
* Response sent by the server in response to a DSRequest. Contains
*  all the properties available on the basic RPCResponse, in addition to the
*  properties listed here.
*/
interface DSResponse extends RPCResponse {
/**
* The DSRequest.clientContext object as set on the DSRequest.
* @type {object}
* @default null
*/
clientContext?: object;

/**
* The DataSource of this DSResponse.
* @type {string}
* @default null
*/
dataSource?: string;

/**
* The operation type of the request corresponding to this DSResponse.
* @type {DSOperationType}
* @default null
*/
operationType?: DSOperationType;

/**
* The operation ID of the request corresponding to this DSResponse.
* @type {string}
* @default null
*/
operationId?: string;

/**
* For "fetch" operations, this is the array of Records fetched. For "update", "add", and
*  "remove" operations, this is typically an array containing a single Record representing
*  the record that was updated, added, or removed.
* @type {Array<Partial<Record>> | any}
* @default null
*/
data?: Array<Partial<Record>> | any;

/**
* Same meaning as RPCResponse.status, except DSResponses have additional error codes,
*  such as RPCResponse.STATUS_VALIDATION_ERROR.
* @type {Integer}
* @default "see below"
*/
status?: Integer;

/**
* An extra property of each DSResponse to a queued request that indicates whether the queue
*  as a whole succeeded. A queueStatus of RPCResponse.STATUS_SUCCESS, or 0, indicates
*  that the queue succeeded whereas a queueStatus of RPCResponse.STATUS_FAILURE, or -1,
*  indicates that the queue failed.
* 
*  For example, if two "update" requests are sent in a queue and the first succeeded, but
*  the second failed validation, then both DSResponses' queueStatus would be -1, but the
*  DSResponse.status of the first would be RPCResponse.STATUS_SUCCESS and
*  the status of the second would be an error code such as RPCResponse.STATUS_VALIDATION_ERROR.
* @type {Integer}
* @default "see below"
*/
queueStatus?: Integer;

/**
* Server-side validation errors for an attempted "update" or "add" operation, as a 
*  JS Object where each property name is a field name from the record and each property
*  value contains error information.
*  
*  To extract just the simple error strings for each field we recommend passing this
*  object to DataSource.getSimpleErrors
*  
*  The Java API DSResponse.addError(fieldName, errorMessage) is used to send server-side
*  errors to the client. See the Java Server Reference for details.
* @type {object}
* @default null
*/
errors?: object;

/**
* Starting row of returned server results, when using paged result fetching
*  
*  Note that startRow and endRow are zero-based, inclusive at the beginning and exclusive at
*  the end (like substring), so startRow: 0, endRow: 2 is a response containing two records.
* @type {number}
* @default null
*/
startRow?: number;

/**
* End row of returned server results, when using paged result fetching
*  
*  Note that startRow and endRow are zero-based, inclusive at the beginning and exclusive at
*  the end (like substring), so startRow: 0, endRow: 2 is a response containing two records.
* @type {number}
* @default null
*/
endRow?: number;

/**
* Total number of rows available from the server that match the current filter criteria,
*  when using paged result fetching.
* @type {number}
* @default null
*/
totalRows?: number;

/**
* This attribute may be set to indicate that
*  DataSource.progressiveLoading, for a paged
*  data fetch, and as such the DSResponse.totalRows is not
*  guaranteed to be accurate. Client side code, including the 
*  ResultSet.lengthIsProgressive method may make use of this attribute.
*  
*  The SmartClient server will automatically set this property whenever progressive
*  loading is enabled. Developers may also set this property themselves for custom
*  progressive loading implementations.
* @type {boolean}
* @default null
*/
progressiveLoading?: boolean;

/**
* This attribute may be set for responses where
*  DataSource.progressiveLoading is active to indicate
*  the estimated true total row count for the data set. In progressive loading mode,
*  DSResponse.totalRows can be thought of as an indication of the 
*  last row the user is allowed to request (EG by scrolling through a databound
*  ListGrid), whereas estimatedTotalRows can convey the server's knowledge
*  of the number of matching rows to the client.
*  
*  This value may be validly set to a String in the following format:
*  
* - "500+": There are at least 500 records
*    
* - "-500": There are fewer than 500 records
*    
* - "450-500": There are between 450 and 500 records
*    
* - "~500": There are approximately 500 records
*    
* - "500": There are exactly than 500 records
*  
*  If you are using a SmartClient server side dataSource that supports the 
*  DataSource.progressiveLoadingThreshold feature, this property will be populated
*  automatically to an exact value if a row count query was performed and the result exceeded the 
*  progressiveLoadingThreshold causing progressiveLoading to be enabled.
*  
*  Developers may also write custom dataSource logic to populate this attribute
*  if desired
* @type {string}
* @default null
*/
estimatedTotalRows?: string;

/**
* Optional flag that can be set by the server to force ResultSets to drop any caches of
*  records from the DataSource that was the target of the operation.
* @type {boolean}
* @default null
*/
invalidateCache?: boolean;

/**
* Timestamp (millisecond value) to indicate when this dsResponse was cached in 
*  Offline. Not applicable if the response has never been 
*  stored offline.
* @type {number}
* @default null
*/
offlineTimestamp?: number;

/**
* If set, indicates that this response came from the offline cache, not the server. This 
*  flag is the only reliable way for application code to determine the source of a response.
* @type {boolean}
* @default null
*/
fromOfflineCache?: boolean;

/**
* HTTP headers returned by the server as a map from header name to header value.
*  
*  Headers are available only when the default RPCTransport "xmlHttpRequest" is in use,
*  and browsers may limit access to headers for cross-domain requests or in other
*  security-sensitive scenarios.
* @type {object}
* @default null
*/
httpHeaders?: object;





}
/**
* Creates a new DSResponse
* 
* @param typeCheckedProperties {Partial<DSResponse>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DSResponse} The new instance
*/
class DSResponse{
static create(typeCheckedProperties? : Partial<DSResponse>, uncheckedProperties?: Object): DSResponse;







}

/**
* A ServiceTask configured to perform a update.
*/
interface DSUpdateTask extends ServiceTask {




}
/**
* Creates a new DSUpdateTask
* 
* @param typeCheckedProperties {Partial<DSUpdateTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DSUpdateTask} The new instance
*/
class DSUpdateTask{
static create(typeCheckedProperties? : Partial<DSUpdateTask>, uncheckedProperties?: Object): DSUpdateTask;







}

/**
* TextItem subclass for managing a DataPath
*/
interface DataPathItem extends TextItem {




}
/**
* Creates a new DataPathItem
* 
* @param typeCheckedProperties {Partial<DataPathItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DataPathItem} The new instance
*/
class DataPathItem{
static create(typeCheckedProperties? : Partial<DataPathItem>, uncheckedProperties?: Object): DataPathItem;







}

/**
* A DataSource is data-provider-independent description of a set of objects
*  that will be loaded, edited and saved within the user interface of your application.
*  
*  Each DataSource consists of a list of DataSource.fields that make up a
*  DataSource record, along with DataSourceField.type, 
*  DataSourceField.validators, 
*  DataSourceField.foreignKey to other DataSources, and other
*  metadata.
*  
*  The abstract object description provided by a DataSource is easily mapped to a variety of
*  backend object models and storage schemes. The following table shows analogous terminology
*  across systems.
*  
*  
*   Isomorphic SmartClient
*   Relational Database
*   Enterprise Java Beans (EJB)
*   Entity/Relationship Modeling
*   OO/UML
*   XML Schema/WSDL
*   LDAP
*  
*   DataSource
*   Table
*   EJB class
*   Entity
*   Class
*   Element Schema (ComplexType)
*   Objectclass
*  
*   Record
*   Row
*   EJB instance
*   Entity instance
*   Class instance/Object
*   Element instance (ComplexType)
*   Entry
*  
*   Field
*   Column
*   Property
*   Attribute
*   Property/Attribute
*   Attribute or Element (SimpleType)
*   Attribute
*  
*  
*  DataSources can be dataSourceDeclaration in either JavaScript or XML
*  format, and can also be metadataImport from existing metadata formats,
*  including XML Schema.
*  
*  Data Binding is the process by which
*  DataBoundComponent can automatically configure
*  themselves for viewing, editing and saving data described by DataSources. DataBinding is
*  covered in the ${isc.DocUtils.linkForDocNode('QuickStartGuide', 'QuickStart Guide')}, Chapter 6, Data
*  Binding.
*  
*  clientServerIntegration is the process by which a DataSource
*  can be connected to server systems such as SQL DataBases, Java Object models, WSDL web
*  services and other data providers. Data Integration comes in two variants: client-side and
*  server-side. serverDataIntegration uses the
*  SmartClient Java-based server to connect to data represented by Java Objects or
*  JDBC-accessible databases. clientDataIntegration
*  connects SmartClient DataSources to XML, JSON or other formats accessible via HTTP.
*  
*  DataSources have a concept of dataSourceOperations ("fetch",
*  "add", "update" and "remove") that can be performed on the set of objects represented by a
*  DataSource. Once a DataSource has been integrated with your data store, databinding-capable 
*  UI components can leverage the 4 core DataSource operations to provide many complete user
*  interactions without the need to configure how each individual component loads and saves
*  data.
*  
*  These interactions include ListGrid, TreeGrid, 
*  DetailViewer, DynamicForm-based 
*  DynamicForm.editRecord and DynamicForm.saveData,
*  grid-based ListGrid.canEdit and ListGrid.saveByCell,
*  and custom interactions provided by ${isc.DocUtils.linkForExampleId('patternReuse')} custom
*  databinding-capable components.
*/
interface DataSource extends Class {
/**
* Unique identifier for this DataSource. Required for all DataSources.
* 
*  DataSources will make themselves available as JavaScript globals under
*  the same name as their ID only if DataSource.addGlobalId is set.
* @type {string}
* @default null
*/
ID?: string;

/**
* An optional description of the DataSource's content. Not automatically exposed on any
*  component, but useful for developer documentation, and as such is included on any
*  openapiSupport generated by the framework. Markdown is
*  a commonly used syntax, but you may also embed HTML content in a CDATA tag.
* @type {string}
* @default null
*/
description?: string;

/**
* Whether to make this DataSource available as a global variable for convenience.
* @type {boolean}
* @default true
*/
addGlobalId?: boolean;

/**
* Indicates the format to be used for HTTP requests and responses when
*  fulfilling DSRequests (eg, when DataSource.fetchData is called).
* @type {DSDataFormat}
* @default "iscServer"
*/
dataFormat?: DSDataFormat;

/**
* Should HTTP responses to requests by this dataSource be formatted using the
*  strict JSON subset of the javascript language?
*  If set to true, responses returned by the server should match the format described
*  here.
*  
*  Only applies to dataSources which send requests to a server and have
*  DataSource.dataFormat set to "json" or "iscServer".
*  
*  Note: using strict JSON avoids a known issue in Internet Explorer 9 where
*  datasource transactions can leak memory due to a browser behavior where the native
*  eval() method fails to clean up references when the objects
*  go out of scope. See RPCManager.allowIE9Leak for more on this.
* @type {boolean}
* @default null
*/
useStrictJSON?: boolean;

/**
* Controls the format in which inputs are sent to the dataURL when fulfilling DSRequests.
*  May be overridden for individual request types using
*  OperationBinding.dataProtocol.
* @type {DSProtocol}
* @default null
*/
dataProtocol?: DSProtocol;

/**
* Like OperationBinding.useHttpProxy, but serves as a default for this DataSource
*  that may be overridden by individual operationBindings.
* @type {boolean}
* @default null
*/
useHttpProxy?: boolean;

/**
* Applies only to dataFormat: "json" and DataSource.dataTransport:"scriptInclude". Specifies
*  the name of the query parameter that tells your JSON service what function to call as
*  part of the response.
* @type {string}
* @default "callback"
*/
callbackParam?: string;

/**
* Additional properties to pass through to the DSRequests made by this
*  DataSource. This must be set before any DSRequests are issued and before
*  any component is bound to the DataSource.
*  
*  These properties are applied before DataSource.transformRequest is called.
* @type {Partial<DSRequest>}
* @default null
*/
requestProperties?: Partial<DSRequest>;

/**
* Transport to use for all operations on this DataSource. Defaults to
*  RPCManager.defaultTransport. This would typically only be set to
*  enable "scriptInclude" transport for contacting DataSource.dataFormat web
*  services hosted on servers other than the origin server.
*  
*  When using the "scriptInclude" transport, be sure to set DataSource.callbackParam or
*  OperationBinding.callbackParam to match the name of the query parameter name expected
*  by your JSON service provider.
* @type {RPCTransport}
* @default "xmlHttpRequest"
*/
dataTransport?: RPCTransport;

/**
* If set, the DataSource will ensure that it never uses a cached HTTP response, even if the
*  server marks the response as cacheable.
*  
*  Note that this does not disable caching at higher levels in the framework, for example, the
*  caching performed by ResultSet.
* @type {boolean}
* @default true
*/
preventHTTPCaching?: boolean;

/**
* For a DataSource stored in .xml format on the SmartClient server, indicates what server-side
*  connector to use to execute requests, that is, what happens if you call
*  dsRequest.execute() in server code.
* @type {DSServerType}
* @default "generic"
*/
serverType?: DSServerType;

/**
* For DataSources using the sqlDataSource for
*  persistence, what database table name to use. The default is to use the DataSource ID
*  as the table name.
* @type {string}
* @default null
*/
tableName?: string;

/**
* This property only applies to the built-in SQL DataSource provided in Pro and better 
*  editions of SmartClient
*  
*  Defines the name of the schema we use to qualify the DataSource.tableName in generated
*  SQL. If you do not provide this property, table names will not be qualified in generated 
*  SQL, and thus the default schema will be used. Support for multiple schemas (or schemata)
*  varies quite significantly across the supported databases, as does the meaning of the 
*  phrase "default schema". In addition, some databases allow you to override the default 
*  schema in the JDBC connection URL, which is a preferable approach if all your tables are 
*  in the same (non-default) schema.
*  
*  The following table provides information by product:
*  
*  
*  Product Notes
*  DB2
*  Arbitrarily named schemas are supported. The default schema is named after the connecting
*  user, though this can be overridden by specifying the "currentSchema" property on the JDBC
*  connection URL
*  DB2 for iSeries
*  Arbitrarily named schemas are supported. "Schema" is synonymous with "library". The 
*  default schema depends on the setting of the "naming" connection property. When this is 
*  set to "sql", behavior is similar to other DB2 editions: the default schema is named after
*  the connecting user, unless overridden by specifying a library name in the JDBC connection 
*  URL. When "naming" is set to "system", the schema of an unqualified table is resolved 
*  using a traditional search of the library list; the library list can be provided in the 
*  "libraries" property
*  Firebird
*  Firebird does not support the concept of schema at all - all "schema objects" like tables 
*  and indexes belong directly to the database. In addition, Firebird actively rejects 
*  qualified table names in queries as syntax errors; therefore, you should not set
*  the schema property for a DataSource that will be backed by a Firebird database
*  
*  HSQLDB
*  Arbitrarily named schemas are supported. The default schema is auto-created when the
*  database is created; by default it is called "PUBLIC", but can be renamed. It is not 
*  possible to set the default schema in the JDBC connection URL
*  Informix
*  Informix databases can be flagged as "ANSI mode" at creation time. ANSI-mode databases 
*  behave similarly to DB2 for schema support: arbitrarily named schemas are supported, and
*  the default schema is the one named after the connected user. Non-ANSI databases have no
*  real schema support at all. It is not possible to set the default schema in the JDBC 
*  connection URL with either type of database
*  Microsoft SQL Server
*  Prior to SQL Server 2005, schema support is similar to Oracle: "schema" is synonymous with 
*  "owner". As of SQL Server 2005, schema is supported as a separate concept, and a user's 
*  default schema can be configured (though it still defaults to a schema with the same name
*  as the user). It is not possible to set the default schema in the JDBC connection URL
*  MySQL
*  MySQL does not have a separate concept of "schema"; it treats the terms "schema" and 
*  "database" interchangeably. In fact MySQL databases actually behave more like schemas, in 
*  that a connection to database X can refer to a table in database Y simply by qualifying 
*  the name in the query. Also, because schema and database are the same concept in MySQL, 
*  overriding the "default schema" is done implicitly when you specify which database to 
*  connect to in your JDBC connection URL
*  Oracle
*  Arbitrarily named schemas are not supported; in Oracle, "schema" is synonymous with "user",
*  so each valid user in the database is associated implicitly with a schema of the same name,
*  and there are no other schemas possible. It is possible to refer to tables in another
*  user's schema (assuming you have the privileges to do so) by simply qualifying the table name.
*  The default schema is always implied by the connecting user and cannot be overridden.
*  Postgres
*  Arbitrarily named schemas are supported.
*  Rather than the concept of a "default schema", Postgres supports the idea of a search path 
*  of schemas, whereby unqualified table references cause a search of the list of schemas in 
*  order, and the first schema in the path is the "current" one for creation purposes. 
*  Unfortunately, there is no way to specify this search path on the JDBC connection URL, so
*  the default schema comes from the user definition, ultimately defaulting to the default 
*  "public" schema
*  
* @type {string}
* @default null
*/
schema?: string;

/**
* For SQL DataSources, tells the framework whether to surround the associated 
*  DataSource.tableName with quotation marks whenever it appears in
*  generated queries. This is only required if you have to connect to a table with a name
*  that is in breach of your database product's naming conventions. For example, some 
*  products (eg, Oracle) internally convert all unquoted references to upper case, so if
*  you create a table called myTest, the database actually calls it 
*  MYTEST unless you quoted the name in the create command, like this:
*  
*  &nbsp;&nbsp;CREATE TABLE "myTest"
*  
*  If you do quote the name like this, or if you have to connect to a legacy table
*  that has been named in this way, then you must set this property to tell the SQL engine 
*  that it must quote any references to this table name (this requirement depends on the 
*  database in use - as noted below, some are not affected by this problem). If you do 
*  not, you will see exceptions along the lines of "Table or view 'myTest' does not exist".
*  
*  Note, other database products (eg, Postgres) convert unquoted names to lower case, which
*  leads to the same issues. Still others (eg, SQL Server) are not case sensitive and are 
*  not affected by this issue.
*  
*  Generally, we recommend that you avoid using this property unless you have a specific 
*  reason to do so. It is preferable to avoid the issue altogether by simply not quoting 
*  table names at creation time, if you are able to do so.
* @type {boolean}
* @default null
*/
quoteTableName?: boolean;

/**
* For a sqlDataSource that is referred by
*  DataSourceField.otherFKs, this property defines the table alias name to
*  use in generated SQL. If omitted DataSource.ID will be used to construct the alias.
*  
*  Aliasing is necessary when the same table appears more than once in a query. In addition
*  to use cases described in DataSourceField.relatedTableAlias, this may happen when
*  includeFrom field using DataSourceField.otherFKs
*  would be included multiple times in a related DataSource.
*  
*  See the "Automatically generated table aliases" section of the customQuerying
*  for the complete set of general rules how aliases are generated. Also, see
*  DataSourceField.otherFKs for more details and usage samples.
* @type {string}
* @default null
*/
relatedTableAlias?: string;

/**
* For DataSources using the sqlDataSource for
*  persistence, which database configuration to use. Database configurations can be
*  created using the adminConsole. If unset, the default
*  database configuration is used (which is also settable using the "Databases" tab).
* @type {string}
* @default null
*/
dbName?: string;

/**
* For DataSources of DataSource.serverType "hibernate", the name of a Spring 
*  bean to query to obtain Hibernate Configuration for this particular DataSource. Note
*  that this is intended for DataSource-specific configuration overrides for unusual 
*  circumstances, such as a DataSource whose physical data store is a completely 
*  different database to that used by other DataSources. See the 
*  hibernateIntegration article for more 
*  information
* @type {string}
* @default null
*/
configBean?: string;

/**
* If set, tells the SQL engine to quote column names in all generated DML and DDL 
*  statements for this dataSource. This will ensure that queries generated against
*  tables that do not follow the database product's natural column-naming conventions 
*  will still work.
*  
*  In general we recommend that you allow the database to use its natural naming scheme
*  when creating tables (put more simply, just do not quote column names in the 
*  CREATE TABLE statement); if you do this, you will not need to worry about 
*  quoting column names when querying. However, if you are dealing with pre-existing 
*  tables, or do not have control over the database naming conventions used, this property
*  may become necessary. This property may also be necessary if you are using field/column
*  names that clash with reserved words in the underlying database (these vary by database,
*  but a field called "date" or "timestamp" would have problems with most database products)
*  
*  Note: Only applicable to dataSources of DataSource.serverType "sql".
* @type {boolean}
* @default null
*/
quoteColumnNames?: boolean;

/**
* Allows you to specify SQL query execution time threshold in milliseconds, which if exceeded 
*  query is identified as "slow" and may be logged under specific logging category.
*  
*  This setting applies to all SQL queries, unless more specific thresholds are set using 
*  DataSource.logSlowFetch, DataSource.logSlowAdd, 
*  DataSource.logSlowUpdate, DataSource.logSlowRemove, 
*  DataSource.logSlowCustom or even more specific affecting just the 
*  operationBinding it is configured at: attr:operationBinding.logSlowSQL.
*  
*  If none of the thresholds above are set, global sql.log.queriesSlowerThan 
*  sqlSettings will be used.
*  
*  For the details on how to setup the logging part see the 
*  serverLogging
* @type {Integer}
* @default null
*/
logSlowSQL?: Integer;

/**
* Allows you to specify DSOperationType SQL query execution 
*  time threshold in milliseconds, which if exceeded query is identified as "slow" and 
*  may be logged under specific logging category.
*  
*  See DataSource.logSlowSQL for more details.
* @type {Integer}
* @default null
*/
logSlowFetch?: Integer;

/**
* Allows you to specify DSOperationType SQL query execution 
*  time threshold in milliseconds, which if exceeded query is identified as "slow" and 
*  may be logged under specific logging category.
*  
*  See DataSource.logSlowSQL for more details.
* @type {Integer}
* @default null
*/
logSlowAdd?: Integer;

/**
* Allows you to specify DSOperationType SQL query execution 
*  time threshold in milliseconds, which if exceeded query is identified as "slow" and 
*  may be logged under specific logging category.
*  
*  See DataSource.logSlowSQL for more details.
* @type {Integer}
* @default null
*/
logSlowUpdate?: Integer;

/**
* Allows you to specify DSOperationType SQL query execution 
*  time threshold in milliseconds, which if exceeded query is identified as "slow" and 
*  may be logged under specific logging category.
*  
*  See DataSource.logSlowSQL for more details.
* @type {Integer}
* @default null
*/
logSlowRemove?: Integer;

/**
* Allows you to specify DSOperationType SQL query execution 
*  time threshold in milliseconds, which if exceeded query is identified as "slow" and 
*  may be logged under specific logging category.
*  
*  See DataSource.logSlowSQL for more details.
* @type {Integer}
* @default null
*/
logSlowCustom?: Integer;

/**
* Allows you to specify an arbitrary prefix string to apply to all json format responses 
*  sent from the server to this application.
*  
*  The inclusion of such a prefix ensures your code is not directly executable outside of
*  your application, as a preventative measure against
*  javascript hijacking.
*  
*  Only applies to responses formatted as json objects. Does not apply to responses returned
*  via scriptInclude type transport.
*  Note: If the prefix / suffix served by your backend is not a constant, you can use 
*  OperationBinding.dataFormat instead and explicitly parse
*  the prefix out as part of DataSource.transformResponse.
* @type {string}
* @default null
*/
jsonPrefix?: string;

/**
* Allows you to specify an arbitrary suffix string to apply to all json format responses 
*  sent from the server to this application.
*  
*  The inclusion of such a suffix ensures your code is not directly executable outside of
*  your application, as a preventative measure against
*  javascript hijacking.
*  
*  Only applies to responses formatted as json objects. Does not apply to responses returned
*  via scriptInclude type transport.
* @type {string}
* @default null
*/
jsonSuffix?: string;

/**
* For DataSources of DataSource.serverType "sql" only, indicates whether
*  we should automatically add a sort field for ResultSet.fetchMode on 
*  this DataSource. If left unset, this property defaults to the one of the global values 
*  described in the DataSource.defaultSortField documentation. 
*  Also note, this property can be overridden 
*  OperationBinding.forceSort.
*  
*  Note, the ability to set this property per-DataSource is only provided to allow for
*  complete configurability in unusual cases. See the defaultSortField docs
*  for details of why use of this property should be considered a red flag.
* @type {string}
* @default null
*/
forceSort?: string;

/**
* For DataSources of DataSource.serverType "sql" only, the name of a field to 
*  include in the ORDER BY clause as a means of enforcing a stable sort 
*  order, for ResultSet.fetchMode only. This property has no
*  effect for non-SQL dataSources, or for non-paged fetches.
*  
*  Generally speaking, databases make no guarantee about the order of rows in a
*  resultset unless an ORDER BY clause was provided. This is not usually a
*  problem if you actually don't care about the row order, but there is a catch if you 
*  are using paged fetching: because the pages are fetched using completely different 
*  queries, the database is at liberty to use different orderings from one fetch to another,
*  and in some cases, with some databases, that is what actually happens. This leads to 
*  broken paging behavior, with some records duplicated and others omitted.
*  
*  Note that it is unusual for a database to change strategies between queries like this;
*  generally speaking, rows are ordered in some kind of natural ordering in the absence of
*  an explicit order - typically insertion order, or by primary key value. However, it 
*  does happen, and is more likely with some database products than others.
*  
*  This property only has an effect if forceSort is in effect for the fetch 
*  operation. See below for DataSource- and operation-level options, but ordinarily this
*  is arranged by setting the forceSort property for the current database 
*  configuration in your server.properties file:
*   # Given this database definition
*   sql.MyDB.database.type: mysql
* 
*   # Either of these settings will enable automatic sorting
*   sql.MyDB.forceSort: true
*   # Or
*   sql.mysql.forceSort: true
*  
*  Note, the defaultSortField should ideally provide a unique ordering: so
*  for an employee table, payroll number would be preferable to employee name. A 
*  non-unique ordering will usually be sufficient to ensure stablity of ordering from one
*  query to the next, because it will usually ensure that the database is forced to use the
*  same index in each case. However, the database may still choose to order rows 
*  differently within the confines of the non-unique ordering, so only a unique ordering 
*  is guaranteed to ensure stability.
*  
*  Fields of FieldType creatorTimestamp are also good 
*  candidates for this purpose - assuming you have a suitable index in place, and assuming
*  sorting by temporal values does not introduce performance problems with your database of
*  choice - as they are often unique or near-to-unique, and they reflect the insertion 
*  order, which is the "natural ordering" in some (not all) databases.
*  
*  Note that this automatic sorting does not interfere with the ordinary sorting that your
*  application may do: it is applied in addition to any application sort order. So
*  if your application imposes no ListGrid.setSort, the resultset will 
*  be sorted by the defaultSortField; if your application requests a sort 
*  order of, eg, "state" then "city", the resultset will be ordered by "state", then 
*  "city", and then the defaultSortField
*  
*  If forceSort is enabled and you do not provide a 
*  defaultSortField for a given database, SmartClient will instead use the
*  DataSourceField.primaryKey field(s). If the dataSource does not 
*  define a primaryKey, we will just use the dataSource's first defined 
*  field. Recommendation: Unless you have a reason to explicitly declare a 
*  defaultSortField, we recommend that you leave it undefined for any 
*  DataSource that declares a primaryKey (and we recommend that all
*  dataSources declare a primaryKey). The primaryKey field is
*  usually the ideal candidate for this purpose, because it is unique and almost certainly
*  indexed.
*  
*  Note that forceSort is enabled by default for PostgreSQL, because this 
*  product is known to be less likely to retain a stable sort order between two similar,
*  unordered queries.
*  DataSource- and OperationBinding-level forceSort flag
*  As well as the above-mentioned global settings, it is possible to configure automatic
*  sorting behavior for paged fetches at both the individual
*  DataSource.forceSort and 
*  OperationBinding.forceSort. However, this facility is 
*  only provided to allow complete configurability in unusual cases. Generally speaking, 
*  if a database requires ordering for correct behavior with paged fetches, it 
*  always requires ordering for correct behavior with paged fetches; you can't 
*  ordinarily pick and choose which tables or which individual fetches need to be ordered.
*  
*  That said, there may be real world cases where a database that normally requires 
*  ordering for correct behavior, nevertheless has individual cases where that is not 
*  required - maybe on tables that have only a single index, or in cases where there are
*  no joins involved, or maybe in other circumstances related to how that specific database
*  product works internally. Or again, there may be unusual individual cases where a 
*  database that ordinarily works fine for paged fetches without requiring ordering, needs
*  to apply an ordering.
*  
*  We provide the DataSource- and operation-level forceSort flags to allow 
*  you to work around or take advantage of these database-specific quirks. However, use of
*  them should be considered a red flag because they might cause problems if things change
*  in the future (new database release, change in the underlying query, addition of a 
*  foreignKey relation at the database level, etc).
* @type {string}
* @default null
*/
defaultSortField?: string;

/**
* The list of fields that compose records from this DataSource.
*  
*  Each DataSource field can have type, user-visible title, validators, and other metadata
*  attached.
*  
*  
*  After a DataSource has been Class.create, access the list of fields via
*  DataSource.getFieldNames and access individual fields via
*  DataSource.getField.
*  
* @type {Array<Partial<DataSourceField>>}
* @default null
*/
fields?: Array<Partial<DataSourceField>>;

/**
* ID of another DataSource this DataSource inherits its DataSource.fields from.
*  
*  Local fields (fields defined in this DataSource) are added to inherited fields 
*  to form the full set of fields. Fields with the same name are merged in the same way
*  that DataBoundComponent.fields are merged with
*  DataSource fields.
*  
*  The default order of the combined fields is new local fields first (including any fields
*  present in the parent DataSource which the local DataSource re-declares), then parent
*  fields. You can set DataSource.useParentFieldOrder to instead use the parent's field
*  order, with new local fields appearing last. You can set DataSource.showLocalFieldsOnly to
*  have all non-local fields hidden.
*  
*  Note that only fields are inherited - other properties such as dataURL and
*  dataFormat are not. You can use ordinary inheritance, that is, creating a subclass of
*  DataSource, in order to share properties such as dataURL across a series of DataSources
*  that also inherit fields from each other via inheritsFrom.
*  
*  This feature can be used for:
*  
*  
* - creating a customized view (eg, only certain fields shown) which will be used by
*  multiple DataBoundComponent.
*  
* - adding presentation-specific attributes to metadata that has been automatically
*  derived from XMLTools.loadXMLSchema or other metadata formats
*  
* - modeling object subclassing and extension in server-side code and storage systems
*  
* - modeling relational database joins, and the equivalents in other systems
*  
* - creating hooks for others to customize your application in a maintainable way. For
*  example, if you have a dataSource "employee", you can create a dataSource
*  "customizedEmployee" which inherits from "employee" but does not initially define any
*  fields, and bind all DataBoundComponent to
*  "customizedEmployee". Customizations of fields (including appearance changes, field
*  order, new fields, hiding of fields, and custom validation rules) can be added to
*  "customizedEmployee", so that they are kept separately from the original field data and
*  have the best possible chance of working with future versions of the "employee"
*  dataSource.
*  
* @type {string}
* @default null
*/
inheritsFrom?: string;

/**
* For audit DataSources, this required property specifies the ID of the
*  DataSource.audit DataSource. Automatically populated for
*  DataSource.generateAuditDS audit DataSources.
* @type {string}
* @default "varies"
*/
auditedDataSourceID?: string;

/**
* For an DataSource.audit DataSource, controls whether the Framework will
*  attempt to auto-generate the audit DataSource. Note that this property is independent of
*  DataSource.auditDataSourceID so that, by default, even when the audit DataSource is given
*  a non-default ID, the Framework will still attempt to auto-generate it.
* @type {boolean}
* @default true
*/
generateAuditDS?: boolean;

/**
* The list of extra manually managed fields that will be added to the DataSource.fields of the
*  DataSource.audit.
*  
*  This feature enables the storage of additional information in the Audit DataSource alongside the
*  standard audit data. In order to do that the audited DataSource needs to declare DataSource.auditDSConstructor
*  referring custom DataSource.serverConstructor, so that all requests to add data to the Audit DataSource
*  could be intercepted allowing to make changes to the new records (obtained using DSRequest.getValues()
*  server-side API). In this particular use case values for the addedAuditFields need to be provided.
*  
*  Example of an audited DataSource (schematically):
*  
*  &lt;DataSource audit="true" auditDSConstructor="package.AuditDS"&gt;
*  &lt;fields&gt;....&lt;/fields&gt;
*  &lt;addedAuditFields&gt;
*   &lt;addedAuditField name="altitude" type="float" /&gt;
*   &lt;addedAuditField name="longitude" type="float" /&gt;
*   &lt;addedAuditField name="logCorrelationId" type="text" /&gt;
*  &lt;/addedAuditFields&gt;
*  &lt;/DataSource&gt;
*  
*  An example implementation of AuditDS could be as follows:
*  
*  public class AuditDS extends SQLDataSource {
*    public DSResponse executeAdd(DSRequest req) throws Exception {
*      // populate additional fields
*      Map values = req.getValues();
*      values.put("altitude", 54.685);
*      values.put("longitude", 25.286);
*      values.put("logCorrelationId", "foobar");
*      // execute "add" request
*      return super.executeAdd(req);
*    }
*  }
*  
* @type {Array<Partial<DataSourceField>>}
* @default null
*/
addedAuditFields?: Array<Partial<DataSourceField>>;

/**
* For dataSources of DataSource.serverType "sql" and "hibernate", specifies the inheritance 
*  mode to use. This property has no effect for any other type of DataSource.
* @type {DSInheritanceMode}
* @default "full"
*/
inheritanceMode?: DSInheritanceMode;

/**
* For fields of DataSourceField.type "sequence" in a dataSource of 
*  DataSource.serverType "sql", indicates the 
*  SequenceMode to use. This property has no effect for fields or dataSources of 
*  other types.
*  
*  You can set a default sequenceMode for all DataSources of a given database type by setting
*  property "sql.{database_type}.default.sequence.mode" in server.properties. 
*  You set a global default sequenceMode that applies to all database types by setting property
*  "sql.default.sequence.mode". For example:
*   sql.mysql.default.sequence.mode: jdbcDriver
*  
*  
* @type {SequenceMode}
* @default "native"
*/
sequenceMode?: SequenceMode;

/**
* For a DataSource with DataSource.serverType, this flag indicates 
*  whether any fields of DataSourceField.type "sequence" should be backed by a 
*  native database sequence (if the flag is true) or an auto-increment/identity column (if
*  it is false). It is only applicable in cases where the database in use supports both 
*  approaches, and SmartClient supports both strategies with that particular database.
*  For most databases, even those that natively support either approach, SmartClient uses 
*  one or the other, and this cannot be configured.
*  
*  Right now, the only supported database is Microsoft SQL Server. If you specify this flag 
*  on a non-SQL DataSource or if any database other than SQL Server is in use, the flag is
*  simply ignored.
*  
*  If not set, this flag defaults to the server.properties setting 
*  sql.{dbName}.use.sequences, which in turn defaults to false.
* @type {boolean}
* @default null
*/
useSequences?: boolean;

/**
* Like DataBoundComponent.useFlatFields, but applies to all DataBound components
*  that bind to this DataSource.
* @type {boolean}
* @default null
*/
useFlatFields?: boolean;

/**
* For a DataSource that inherits DataSource.fields from another DataSource 
*  (via DataSource.inheritsFrom), indicates that only the fields listed in this DataSource
*  should be shown. All other inherited parent fields will be marked "hidden:true".
* @type {boolean}
* @default null
*/
showLocalFieldsOnly?: boolean;

/**
* For a DataSource that inherits DataSource.fields from another DataSource 
*  (via DataSource.inheritsFrom), indicates that the parent's field order should be used
*  instead of the order of the fields as declared in this DataSource. New fields, if any,
*  are placed at the end.
* @type {boolean}
* @default null
*/
useParentFieldOrder?: boolean;

/**
* Indicates that for server responses, for any data being interpreted as DataSource records, 
*  only data that
*  corresponds to declared fields should be retained; any extra fields should be discarded.
*  
*  For DataSource.dataFormat data, this means extra properties in selected
*  objects are dropped.
*  
*  By default, for DMI DSResponses, DSResponse.data is filtered on the server to just
*  the set of fields defined on the DataSource (see the overview in dmiOverview).
*  
*  This type of filtering can also be enabled for non-DMI DSResponses. By default it is enabled
*  for Hibernate and JPA datasources to avoid unintentional lazy loading too much of a data model.
*  For the rest of datasources this is disabled by default.
*  
*  Explicitly setting this property to false disables (or to true enables)
*  this filtering for this DataSource only. This setting
*  overrides the configuration in server_properties. This setting can
*  be overridden by ServerObject.dropExtraFields.
* @type {boolean}
* @default null
*/
dropExtraFields?: boolean;

/**
* Analogous to DataSource.dropExtraFields, for data sent to the server. Setting this
*  attribute to false ensures that for any records in the data object, only fields that
*  correspond to declared dataSource fields will be present on the dsRequest data object passed
*  to DataSource.transformRequest and ultimately sent to the server.
* @type {boolean}
* @default true
*/
sendExtraFields?: boolean;

/**
* This property allows you to specify that your DataSource's schema (field definitions) should
*  be automatically derived from some kind of metadata. This causes SmartClient to create 
*  a special super-DataSource, which is used purely as a source of default schema for this 
*  DataSource; this is arranged by causing the autoDerived DataSource to automatically 
*  DataSource.inheritsFrom the special super-DataSource. This allows you to 
*  auto-derive schema from existing metadata, whilst still being able to override any or all
*  of it as required. By default additional derived field definitions are placed at the end,
*  but that can be changed by DataSource.useParentFieldOrder flag. Also, derived field
*  definitions may be hidden using DataSource.showLocalFieldsOnly.
*  This property has a different implementation depending upon the 
*  DataSource.serverType of the DataSource:
*  
* - For a DataSource with serverType: "sql", automatically derive the dataSource's schema 
*  from the Spring bean or Java class specified in DataSource.schemaBean. If 
*  schemaBean is not specified, derive the schema from the columns in the SQL 
*  table specified in DataSource.tableName. More information on SQL DataSources 
*  is sqlDataSource 
*  
* - For a DataSource with serverType: "hibernate", automatically derive the dataSource's
*  schema from the Spring bean, Hibernate mapping or Java class named in the 
*  DataSource.schemaBean property. If no such thing exists, derive the schema from
*  the Hibernate mapping or Java class specified in the DataSource.beanClassName
*  property (this allows you to specify schema and mapping separately, in the unusual 
*  circumstance that you have a need to do so). Note that the "mappings" referred to here
*  can mean either .hbm.xml files or annotated classes; both are supported.
*  If neither of these is successful, derive the schema from the underlying SQL table 
*  specified in DataSource.tableName. More information on Hibernate DataSources is
*  hibernateIntegration 
*  
* - For a DataSource with serverType: "jpa1" or "jpa", automatically derive the 
*  dataSource's schema from the annotated JPA class named in the DataSource.schemaBean 
*  property. If the schemaBean property is not defined, derive the schema from the 
*  annotated JPA class named in the DataSource.beanClassName property (as with 
*  Hibernate, this allows you to specify schema and mapping separately if you need to do 
*  so). JPA DataSource generation relies on annotations (the orm.xml mapping file is not 
*  supported). More information on JPA DataSources is jpaIntegration 
*  
* - For other DataSource types, attempt to find a Spring bean with the name specified in
*  the DataSource.schemaBean property. If no such bean is found (or Spring is not 
*  present), attempt to instantiate an object whose fully-qualified class name is the value
*  in the schemaBean property. If one of these approaches succeeds, we derive
*  the schema from the discovered object (by treating it as a Java Bean and assuming that 
*  each one of its getters corresponds to a like-named field in the DataSource). More 
*  information on custom DataSource implementations is writeCustomDataSource.
*  
*  Note that when dataSource schema is derived by introspecting the Java class or Spring bean the
*  derived fields are sorted alphabetically, but with DataSourceField.primaryKey
*  fields at the top.
*  Field types
*  The following table shows how SQL types are derived into 
*  FieldType when we use an SQL table as a source of metadata
*  for a SQL or Hibernate DataSource:
*  
*  SQL typeDataSourceField.type
*  CHAR, VARCHAR, LONGVARCHAR, TEXT, CLOBtext
*  BIT, TINYINT, SMALLINT, INTEGER, BIGINT, DECIMAL*, NUMBER**integer
*  REAL, FLOAT, DOUBLE, DECIMAL*, NUMBER**float
*  DATEdate
*  TIMEtime
*  TIMESTAMPdatetime
*  BLOB, BINARY, VARBINARY, LONGVARBINARYbinary
*  
*  *For DECIMAL types, we inspect the "DECIMAL_DIGITS" attribute of the JDBC 
*  metadata and designate the field type "integer" if that attribute is 0, or "float" if
*  it is some other value.
*  **NUMBER is an Oracle-only type that appears in the JDBC metadata as DECIMAL
*  and is handled exactly the same way as DECIMAL
*  
*  The following table shows how Java types are derived into DataSource types when we use
*  an unannotated Java class (Spring bean, Hibernate mapping or POJO) as a source of 
*  metadata for a SQL, Hibernate or custom DataSource:
*  
*  Java typeDataSourceField.type
*  boolean, Booleanboolean
*  char, Character, String, Readertext
*  byte, short, int, long, Byte, Short, Integer, Long, BigIntegerinteger
*  float, double, Float, Double, BigDecimalfloat
*  Enumenum (see discussion below)
*  InputStreambinary
*  java.sql.Date, java.util.Date, java.util.Calendardate
*  java.sql.Timetime
*  java.sql.Timestampdatetime
*  
*  
*  Finally, this table shows how Java types are derived into DataSource types when we use an 
*  annotated class as a source of metadata. Note annotated classes are necessary for JPA
*  DataSources, but you can choose to use them for other types of DataSource as well. 
*  For Hibernate DataSources, this is very worthwhile because Hibernate will also make use
*  of the annotations as config, meaning you don't need to specify .hbm.xml 
*  files. For SQL and custom DataSources, there is no benefit at the persistence level, 
*  but it may still be worthwhile because the use of an annotated class gives us better 
*  metadata and allows us to generate a better, closer-fitting autoDerive DataSource than 
*  we can from examination of SQL schema or plain Java Beans:
*  
*  Java typeDataSourceField.type
*  boolean, Booleanboolean
*  char, Character, String, Readertext
*  byte, short, int, long, Byte, Short, Integer, Long, BigIntegerinteger
*  float, double, Float, Double, BigDecimalfloat
*  InputStreambinary
*  java.util.Date (with Temporal set to DATE), java.sql.Datedate
*  java.util.Date (with Temporal set to TIME), java.sql.Timetime
*  java.util.Date (with Temporal set to TIMESTAMP), java.util.Calendar, java.sql.Timestampdatetime
*  Enum (with Enumerated set to STRING)enum (see discussion below)
*  Enum (with Enumerated set to ORDINAL)intEnum (see discussion below)
*  Field with Lob annotationbinary
*  Field with GeneratedValue annotationsequence, if the field is an integer type (see discussion below)
*  
*  
*  Primary keys, sequences and identity columns
*  We attempt to derive information about primary keys from the metadata we have.
*  
*  If the metadata source is an SQL table:
*  
*  
* - If the table does not have a native primary key constraint, no attempt is made to 
*    identify primary key fields. Otherwise:
*  
* - The column or columns that make up the table's native primary key constraint are 
*    identified using the JDBC DatabaseMetaData.getPrimaryKeys() API
*  
* - Each DataSource field that corresponds to one of these native primary key columns 
*    is marked primaryKey: true
*  
* - For each of these columns, the metadata returned by 
*    DatabaseMetaData.getColumns() is inspected. If the metadata includes
*    IS_AUTOINCREMENT=YES, we mark the corresponding field as 
*    type="sequence". This information should be reliably provided by 
*    databases that implement "auto-increment" or "identity" column types, such as MySQL
*    or Microsoft SQL Server
*  
* - If the previous step does not identify a column as a sequence, we inspect the 
*    ResultSetMetaData obtained by running a dummy query on the table. If 
*    the isAutoIncrement() API returns true for that column, we mark the 
*    corresponding field as type="sequence"
*  
* - If the previous steps have not identified the column as a sequence, we check the 
*    TYPE_NAME in the column metadata. If it is "serial", this means the 
*    column is a PostgreSQL "serial" or "serial8" type column. Postgres does not 
*    transparently implement auto-increment columns, but it does provide this serial 
*    type, which causes the column to be implicitly bound to an underlying sequence. So
*    this type causes us to mark the field type="sequence", and we also set
*    DataSourceField.implicitSequence true
*  
* - If the previous steps have not identified the column as a sequence, we check the 
*    COLUMN_DEF in the column metadata. If this contains the token "$$ISEQ"
*    and ends with "NEXTVAL", this means the column is an Oracle "GENERATED AS IDENTITY" 
*    column. This type of column was introduced in Oracle 12c and is conceptually 
*    exactly the same thing as the Postgres "serial" column described above. We treat 
*    it the same way: mark it type="sequence" and 
*    implicitSequence="true"
*  
* - If the previous steps have not identified the column as a sequence, then you may
*    be using a pure sequence database, such as an Oracle version earlier than 12c, or 
*    you may be using a database where both sequences and identity columns are available
*    (Oracle, Postgres, DB2), and a sequence is being used either by design or because 
*    the table was created before the database product supported identity columns. In 
*    this case, we cannot determine if the column should be a sequence or not. However,
*    in many applications, the fact that the column is an integer and is a primary key
*    would imply that it is also a sequence. Therefore, if the column is an integer and
*    the server.properties flag auto.derive.integer.pk.always.sequence
*    is true, we mark the field as type="sequence"
*  
* - If, after all this, SmartClient ends up incorrectly marking a primary key field as 
*    a sequence (or vice versa), you can always override it on a per-field basis by 
*    simply redeclaring the field with the correct type in your .ds.xml 
*    file:
*  &lt;DataSource serverType="sql" tableName="myTable" autoDeriveSchema="true"&gt;
*   &lt;fields&gt;
*    &lt;!-- This field was incorrectly marked as a sequence --&gt;
*    &lt;field name="notASeq" type="integer" /&gt;
*    &lt;!-- This field was incorrectly marked as an integer when it should be a sequence --&gt;
*    &lt;field name="isASeq" type="sequence" /&gt;
*   &lt;/fields&gt;
*  &lt;/DataSource&gt;
*  
*   
*  If the metadata source is Hibernate mappings described in a .hbm.xml file:
*  
*  
* - The first field we encounter that is described in the mapping with an &lt;id&gt; tag
*    is marked as a primaryKey
*  
* - If that field is marked as being generated, we set its type to "sequence"
*  
*   
*  If the metadata source is an annotated object (whether JPA, Hibernate or just an 
*  annotated POJO):
*  
*  
* - Any field with an @Id annotation is is marked as a primaryKey (this 
*    differs from the Hibernate .hbm.xml file case because that is specific 
*    to Hibernate, which does support composite keys, but not by specifying multiple 
*    &lt;id&gt; tags. Annotations are supported, via annotated POJOs, for any kind of 
*    persistence strategy, so multiple @Id fields are perfectly valid)
*  
* - Any field with a @GeneratedValue annotation is either marked as 
*    type="sequence" (if it is an integer type) or as 
*    DataSourceField.autoGenerated="true" (for other 
*    field types)
*  
*  Finally, if the metadata is a plain, unannotated Java object, no attempt is made to
*  derive primary key fields.
*  enums and valueMaps
*  When we come across Java Enum properties in plain or annotated classes, 
*  as well as setting the field type as noted in the above tables, we also generate a 
*  valueMap for the field, based on the Enum members. 
*  
*  For cases where we generate a field of SmartClient type "enum" (see the above tables), 
*  the keys of the valueMap are the result of calling name() on each member
*  of the underlying Java Enum (in other words, its value exactly as declared in its 
*  enum declaration). For cases where we generate a field of SmartClient type "intEnum",
*  the keys of the valueMap are strings representing the ordinal number of each member 
*  in the Java Enum - "0", "1", etc. Note that this behavior will be overriden by 
*  DataSource.enumTranslateStrategy if both are set.
*  
*  In both of these case, the display values generated for the valueMap are the result 
*  of calling toString() on each Enum member. If that gives the same 
*  value as calling name(), the value is passed through 
*  DataTools.deriveTitleFromName(), which applies the same processing rules
*  as DataSource.getAutoTitle to derive a more user-friendly display value.
*  Further notes
*  schemaBean implies autoDeriveSchema, because it has no other 
*  purpose than to name the bean to use for auto-derived schema. Thus, if you specify
*  schemaBean you do not need to specify autoDeriveSchema as well
*  (though it does no harm to do so). However, tableName and 
*  beanClassName can be validly specified without implying 
*  autoDeriveSchema, so in those cases you must explicitly specify 
*  autoDeriveSchema.
*  
*  The underlying super-DataSource is cached in server memory, so that it does 
*  not have to be derived and created each time you need it. However, the cache manager 
*  will automatically refresh the cached copy if it detects that the deriving DataSource 
*  has changed. Thus, if you change the metadata your DataSource is deriving (if, for 
*  example, you add a column to a table), all you need to do is touch the 
*  .ds.xml file (ie, update its last changed timestamp - you don't actually
*  have to change it) and the cached copy will be refreshed next time it is needed.
*  
*  When autoDeriveSchema is set, SQLDataSource will automatically discover foreignKeys and
*  deliver table and column name information to the client in hashed form so that two
*  DataSources that are linked by native SQL foreign keys will automatically discover each
*  other if loaded into the same application, and set 
*  DataSourceField.foreignKey automatically. Because the table and column 
*  names are delivered as cryptohashes, there is no information leakage, but regardless,
*  the feature can be disabled via setting datasource.autoLinkFKs to false in
*  server_properties. This hashed linkage information is
*  delivered to the client in properties DataSource.tableCode and
*  DataSourceField.fkTableCode/DataSourceField.fkColumnCode
* @type {boolean}
* @default null
*/
autoDeriveSchema?: boolean;

/**
* For DataSources that specify DataSource.autoDeriveSchema, this property indicates the name
*  of the Spring bean, Hibernate mapping or fully-qualified Java class to use as parent 
*  schema.
* @type {string}
* @default null
*/
schemaBean?: string;

/**
* For JPA and Hibernate DataSources this property indicates, that data source has composite primary key and
*  specifies fully-qualified Java class:
*  
* - with @EmbeddedId you have to specify class name of declared id
*  
* - with @IdClass you have to specify class specified in annotation declaration
* @type {string}
* @default null
*/
idClassName?: string;

/**
* When using the patternOperators OperatorId, 
*  character that matches any single character.
*  
*  Pass multiple strings to provide multiple alternative wildcards.
* @type {string | Array<Partial<string>>}
* @default "["?","%"]"
*/
patternSingleWildcard?: string | Array<Partial<string>>;

/**
* When using the patternOperators OperatorId, 
*  character that matches a series of one or more characters.
*  
*  Pass multiple strings to provide multiple alternative wildcards.
* @type {string | Array<Partial<string>>}
* @default "*"
*/
patternMultiWildcard?: string | Array<Partial<string>>;

/**
* When using the patternOperators OperatorId, 
*  character that escapes the DataSource.patternSingleWildcard or DataSource.patternMultiWildcard if placed before 
*  it, so that it is treated as a literal character.
* @type {string}
* @default "\"
*/
patternEscapeChar?: string;

/**
* OperatorId like "matchesPattern" use patterns like
*  "foo*txt" to match text values. The patterns are similar to the patterns you use to
*  match names of files in a command-line interface, or to the pattern allowed for the SQL
*  "LIKE" operator.
*  
*  translatePatternOperators controls whether these pattern operators should
*  be translated to a nested series of "startsWith"/"endsWidth"/"contains" operators before
*  being sent to the server. This allows a server that only implements simple operators
*  like "startsWith" to support pattern operators such as "matchesPattern" and
*  "containsPattern", but with caveats:
*   
*     
* - single-character wildcards are not supported
*     
* - multiple wildcards are not truly order-dependent, for example *1*2*3* will
*        match 1,2,3 as interior characters in any order. 
*     
* - may be less efficient than a direct server-side implementation that is able to
*        translate the pattern directly to the underlying storage engine.
*   
*  
*  Note that since "containsPattern" is essentially equivalent to "matchesPattern" but with
*  "*" wildcards at the beginning and end of every pattern, the second limitation (pattern
*  not really order dependent) may be fairly obvious to users when using this feature. For
*  example, "m*t" will match "we meet" and "we teem".
*  
*  The following are examples of how patterns are translated to simpler operators. Note
*  that the case sensitive version of the operator is referred to below, but of course
*  "iMatchesPattern" and "iContainsPattern" will be translated to case-insensitive versions
*  of these operators, such as "iStartsWith".
*  
*  *foo (endsWith)
*  foo* (startsWith)
*  *foo* (contains)
*  *foo*bar (contains and endsWith)
*  foo*bar* (startsWith and contains)
*  foo*bar (startsWith and endsWith)
*  *foo*bar* (multiple contains)
*  
*  Also supported: one startsWith, multiple contains, one endsWith.
* @type {boolean}
* @default false
*/
translatePatternOperators?: boolean;

/**
* Sets the XML namespace prefixes available for XPaths on a DataSource-wide basied. See
*  OperationBinding.xmlNamespaces for details.
* @type {object}
* @default "See below"
*/
xmlNamespaces?: object;

/**
* For an XML DataSource, URN of the WebService to use to invoke operations. This URN
*  comes from the "targetNamespace" attribute of the &lt;wsdl:definitions&gt; element in a
*  WSDL (Web Service Description Language) document, and serves as the unique identifier of
*  the service.
*  
*  Having loaded a WebService using XMLTools.loadWSDL, setting
*  serviceNamespace combined with specifying 
*  OperationBinding
*  that set OperationBinding.wsOperation will cause a DataSource to invoke web
*  service operations to fulfill DataSource requests (DSRequest).
*  
*  Setting serviceNamespace also defaults
*  DataSource.dataURL to the service's location,
*  DataSource.dataFormat to "xml" and
*  OperationBinding.dataProtocol to "soap".
* @type {string}
* @default null
*/
serviceNamespace?: string;

/**
* For a DataSource derived from WSDL or XML schema, the XML namespace this schema belongs
*  to. This is a read-only attribute automatically present on DataSources returned from
*  SchemaSet.getSchema and WebService.getSchema.
* @type {string}
* @default null
*/
schemaNamespace?: string;

/**
* See OperationBinding.recordXPath. recordXPath can be specified
*  directly on the DataSource for a simple read-only DataSource only capable of "fetch"
*  operations, or on clientOnly DataSources using $link{groupDef:testData}.
* @type {XPathExpression}
* @default null
*/
recordXPath?: XPathExpression;

/**
* Default URL to contact to fulfill all DSRequests. Can also be set on a
*  per-operationType basis via OperationBinding.dataURL.
*  
*  NOTE: Best practice is to use the same dataURL for all DataSources which
*  fulfill DSRequests via the server-side RPCManager API. Otherwise, cross-DataSource
*  RPCManager.startQueue will not be possible.
* @type {URL}
* @default null
*/
dataURL?: URL;

/**
* Tag name to use when serializing to XML. If unspecified, the dataSource.ID
*  will be used.
* @type {string}
* @default null
*/
tagName?: string;

/**
* Whether we store server responses for this DataSource into 
*  Offline, and then use those stored responses
*  at a later time if we are offline (ie, the application cannot connect to the server). 
*  Note that by default we do NOT use offline storage for a dataSource.
* @type {boolean}
* @default null
*/
useOfflineStorage?: boolean;

/**
* A clientOnly DataSource simulates the behavior of a remote data store by manipulating a
*  static dataset in memory as DSRequest are executed on it. Any changes
*  are lost when the user reloads the page or navigates away.
*  
*  A clientOnly DataSource will return responses asynchronously, just as a DataSource
*  accessing remote data does. This allows a clientOnly DataSource to be used as a
*  temporary placeholder while a real DataSource is being implemented - if a clientOnly
*  DataSource is replaced by a DataSource that accesses a remote data store, UI code for
*  components that used the clientOnly DataSource will not need be changed.
*  
*  A clientOnly DataSource can also be used as a shared cache of modifiable data across
*  multiple UI components when immediate saving is not desirable. In this case, several
*  components may interact with a clientOnly DataSource and get the benefit of
*  ResultSet behaviors such as automatic cache sync and in-browser data filtering
*  and sorting. When it's finally time to save, DataSource.cacheData can be
*  inspected for changes and data can be saved to the original DataSource via
*  DataSource.addData, DataSource.updateData and DataSource.removeData, possibly in a
*  RPCManager.startQueue. Note that
*  DataSource.getClientOnlyDataSource lets you easily obtain a
*  clientOnly DataSource representing a subset of the data available from a
*  normal DataSource. 
*  
*  See also DataSource.cacheAllData - a cacheAllData behaves like a write-through
*  cache, performing fetch and filter operations locally while still performing remote save
*  operations immediately.
*  
*  ClientOnly DataSources can be populated programmatically via DataSource.cacheData - see
*  clientOnlyDataSources for other ways to populate a
*  client-only DataSource with data.
* @type {boolean}
* @default false
*/
clientOnly?: boolean;

/**
* If set, causes this DataSource to use a read-only "mock" or "test" dataset, if specified, or if no test
*  data is available, then to load data normally but then operate similarly to a DataSource.clientOnly
*  DataSource, never writing changes back to the server.
*  
*  mockMode has no effect on MockDataSource or a DataSource.clientOnly DataSource. 
*  
*  For other DataSources, a one-time fetch will be performed to retrieve sample data, similar to a
*  DataSource.cacheAllData DataSource, except that changes will never be saved back to the server. Only a
*  subset of data will be retrieved, based on DataSource.mockDataRows. DataSource.mockDataCriteria can optionally
*  be set to retrieve specific data.
*  
*  Alternatively, mock data can be provided with DataSource.cacheData.
*  
*  DataSources can be loaded in mockMode via passing settings to DataSource.load, or
*  if loaded with a screen or project, by passing settings to RPCManager.loadScreen or
*  the server-side Project.load() API.
* @type {boolean}
* @default null
*/
mockMode?: boolean;

/**
* When DataSource.mockMode is enabled, number of rows of data to retrieve via an initial "fetch" DSRequest, for
*  use as sample data. Set to null to retrieve all available rows.
* @type {Integer}
* @default "75"
*/
mockDataRows?: Integer;

/**
* When DataSource.mockMode is enabled, criteria to use in an initial "fetch" DSRequest to retrieve sample data.
* @type {Criteria}
* @default null
*/
mockDataCriteria?: Criteria;

/**
* Decides under what conditions the ResultSet cache should be dropped when the
*  ResultSet.criteria changes.
* @type {CriteriaPolicy}
* @default "dropOnShortening"
*/
criteriaPolicy?: CriteriaPolicy;

/**
* The default textMatchStyle to use for DSRequests that do not explicitly state 
*  a DSRequest.textMatchStyle.  Note, however, that DSRequests 
*  issued by ListGrids and other DataBoundComponent will 
*  generally have a setting for textMatchStyle on the component itself (see
*  ListGrid.autoFetchTextMatchStyle, for example).
* @type {TextMatchStyle}
* @default "exact"
*/
defaultTextMatchStyle?: TextMatchStyle;

/**
* For fields on this dataSource that specify 
*  DataSourceField.ignoreTextMatchStyle true, the prevailing
*  textMatchStyle is ignored and SmartClient matches exact values. This property dictates
*  whether that match is case-sensitive like the "exactCase" textMatchStyle, or 
*  case-insensitive like the "exact" textMatchStyle (the default). Please see the 
*  TextMatchStyle for a discussion of the nuances
*  of case-sensitive matching.
* @type {boolean}
* @default false
*/
ignoreTextMatchStyleCaseSensitive?: boolean;

/**
* Class for ResultTrees used by this datasource. If null, defaults to using
*  ResultTree.
*  
*  This can be set to a custom subclass of ResultTree that, for example, hangs on to extra
*  information necessary for integration with web services.
* @type {string}
* @default null
*/
resultTreeClass?: string;

/**
* Class for ResultSets used by this datasource. If null, defaults to using
*  ResultSet.
*  
*  This can be set to a custom subclass of ResultSet that, for example, hangs onto to extra
*  information necessary for integration with web services.
* @type {string}
* @default null
*/
resultSetClass?: string;

/**
* Whether to attempt validation on the client at all for this DataSource. If unset (the
*  default), client-side validation is enabled.
*  
*  Disabling client-side validation entirely is a good way to test server-side validation.
* @type {boolean}
* @default null
*/
useLocalValidators?: boolean;

/**
* The required message when a field that has been marked as
*  DataSourceField.required is not filled in by the user.
*  
*  Note that DataSourceField.requiredMessage wins over this setting if both are set.
* @type {HTMLString}
* @default null
*/
requiredMessage?: HTMLString;

/**
* Whether RPCRequests sent by this DataSource should enable
*  RPCRequest.showPrompt in order to block user interactions until the
*  request completes. 
*  
*  DataSource requests default to blocking UI interaction because, very often, if the user
*  continues to interact with an application that is waiting for a server response, some
*  kind of invalid or ambiguous situation can arise.
*  
*  Examples include pressing a "Save" button a second time before the first save completes,
*  making further edits while edits are still being saved, or trying to initiate editing on
*  a grid that hasn't loaded data.
*  
*  Defaulting to blocking the UI prevents these bad interactions, or alternatively, avoids
*  the developer having to write repetitive code to block invalid interactions on every
*  screen.
*  
*  If an operation should ever be non-blocking, methods that initiate DataSource requests
*  (such as DataSource.fetchData) will generally have a
*  requestProperties argument allowing showPrompt to be set to
*  false for a specific request.
* @type {boolean}
* @default true
*/
showPrompt?: boolean;

/**
* Sets the strategy this DataSource uses to translate Java enumerated types (objects of type
*  enum) to and from Javascript.
*  
*  This property is only applicable if you are using the SmartClient server
* @type {EnumTranslateStrategy}
* @default null
*/
enumTranslateStrategy?: EnumTranslateStrategy;

/**
* The name of the property this DataSource uses for constant name when translating
*  Java enumerated types to and from Javascript, if the EnumTranslateStrategy is set
*  to "bean". Defaults to "_constant" if not set.
*  
*  This property is only applicable if you are using the SmartClient server
* @type {string}
* @default null
*/
enumConstantProperty?: string;

/**
* The name of the property this DataSource uses for ordinal number when translating
*  Java enumerated types to and from Javascript, if the EnumTranslateStrategy is set
*  to "bean". Defaults to "_ordinal" if not set.
*  
*  This property is only applicable if you are using the SmartClient server
* @type {string}
* @default null
*/
enumOrdinalProperty?: string;

/**
* If set, titles are automatically derived from DataSourceField.name for any 
*  field that does not have a DataSourceField.title and is not marked
*  DataSourceField.hidden:true, by calling the method DataSource.getAutoTitle.
* @type {boolean}
* @default true
*/
autoDeriveTitles?: boolean;

/**
* When true, indicates that this DataSource supports multi-level sorting.
* @type {boolean}
* @default true
*/
canMultiSort?: boolean;

/**
* When true, indicates that any updates for this DataSource include only those fields 
*  that have actually changed (and primaryKey fields); when false (the default), all 
*  field values are included in updates, whether they have changed or not
* @type {boolean}
* @default false
*/
sparseUpdates?: boolean;

/**
* When true, indicates that fields in this DataSource will never be positively updated
*  to the null value; they may arrive at null values by being omitted, but we will 
*  not send actual null values in update requests. When false (the default), null is 
*  not treated in any special way.
*  
*  Setting this value causes null-assigned fields to be replaced with the field's
*  DataSourceField.nullReplacementValue, if one is declared.
*  If no nullReplacementValue is declared for the field, the null assignment
*  is replaced with the DataSource's DataSource.nullStringValue, 
*  DataSource.nullIntegerValue, DataSource.nullFloatValue 
*  or DataSource.nullDateValue, depending on the field type.
*  
*  For "add" operations, setting DataSource.omitNullDefaultsOnAdd causes
*  null-valued fields to be removed from the request entirely, rather than replaced with
*  default values as described above.
* @type {boolean}
* @default false
*/
noNullUpdates?: boolean;

/**
* When true, and DataSource.noNullUpdates is also true, indicates that "add" 
*  requests on this DataSource will have null-valued fields removed from the request 
*  entirely before it is sent to the server, as opposed to the default behavior of 
*  replacing such null values with defaults.
* @type {boolean}
* @default false
*/
omitNullDefaultsOnAdd?: boolean;

/**
* If DataSource.noNullUpdates is set, the value to use for any text field that has a null
*  value assigned on an update operation, and does not specify an explicit
*  DataSourceField.nullReplacementValue.
* @type {string}
* @default ""
*/
nullStringValue?: string;

/**
* If DataSource.noNullUpdates is set, the value to use for any integer field that has a null
*  value assigned on an update operation, and does not specify an explicit
*  DataSourceField.nullReplacementValue.
* @type {Integer}
* @default "0"
*/
nullIntegerValue?: Integer;

/**
* If DataSource.noNullUpdates is set, the value to use for any float field that has a null
*  value assigned on an update operation, and does not specify an explicit
*  DataSourceField.nullReplacementValue.
* @type {Float}
* @default "0.0"
*/
nullFloatValue?: Float;

/**
* If DataSource.noNullUpdates is set, the value to use for any boolean field that has a null
*  value assigned on an update operation, and does not specify an explicit
*  DataSourceField.nullReplacementValue.
* @type {boolean}
* @default false
*/
nullBooleanValue?: boolean;

/**
* If DataSource.noNullUpdates is set, the value to use for any date or time field that has a
*  null value assigned on an update operation, and does not specify an explicit
*  DataSourceField.nullReplacementValue.
*  
*  Unlike strings and numbers, there is no "natural" choice for a null replacement value 
*  for dates. The default value we have chosen is midnight on January 1st 1970, simply 
*  because this is "the epoch" - the value that is returned by calling "new Date(0)"
* @type {Date}
* @default "See below"
*/
nullDateValue?: Date;

/**
* Whether a user must be authenticated in order to access this DataSource. This establishes a
*  default for the DataSource as a whole; individual DataSource.operationBindings within the
*  DataSource may still override this setting by explicitly setting
*  OperationBinding.requiresAuthentication.
*  
*  Whether the user is authenticated is determined by calling
*  httpServletRequest.getRemoteUser(), hence works with both simple J2EE security
*  (realms and form-based authentication) and JAAS (Java Authentication & Authorization
*  Service).
*  
*  If you wish to use an authentication scheme that does not make use of the servlet API's 
*  standards, SmartClient Server also implements the setAuthenticated method
*  on RPCManager. You can use this API to tell SmartClient that all the 
*  requests in the queue currently being processed are associated with an authenticated
*  user; in this case, SmartClient will not attempt to authenticate the user via 
*  httpServletRequest.getRemoteUser()
*  
*  You can set the default value for this property via setting "authentication.defaultRequired"
*  in server_properties. This allows you to, for example,
*  cause all DataSources to require
*  authentication for all operations by default.
*  
*  Note that setting this property does not automatically cause an authentication mechanism to
*  appear - you still need to separately configure an authentication system. Likewise, setting
*  requiresAuthentication="false" does not automatically allow users to bypass your authentication
*  mechanism - you need to set up a URL that will accept DSRequests and process them similar to
*  the default "IDACall" servlet, and which is not protected by the authentication system. See
*  servletDetails for details on the IDACall servlet.
* @type {boolean}
* @default null
*/
requiresAuthentication?: boolean;

/**
* Similar to OperationBinding.requiresRole, but controls access to the DataSource as a
*  whole.
* @type {string}
* @default null
*/
requiresRole?: string;

/**
* Indicates that the specified VelocityExpression must evaluate to true for a user 
*  to access this DataSource.
*  
*  See also OperationBinding.requires.
* @type {VelocityExpression}
* @default null
*/
requires?: VelocityExpression;

/**
* Indicates that declarative security rules are waived for rows that were created by the 
*  current user. Practically, this means that when a security check fails, instead of a 
*  security exception being thrown, we alter the criteria to ensure that the request can 
*  only return or affect rows that were created by the current authenticated user. This 
*  allows you to create security regimes whereby users can see and edit data they created, 
*  but have access to other users' data forbidden or limited.
*  
*  In order for this to work, we require two things:
*  
* - The DataSource must specify a field of type "creator" - this field type is described 
*    on FieldType
*  
* - The authentication regime in use must include the idea of a "current user". The 
*    authentication provided by the Servlet API is an example of such a regime.
*  
*  This setting can be overridden at operationBinding and field level, allowing extremely
*  fine-grained control.
* @type {boolean}
* @default null
*/
creatorOverrides?: boolean;

/**
* Requires that the currently authenticated user match the contents of
*  this field, for client-initiated requests (i.e., where
*  DSRequest.isClientRequest() returns true on the server).
*  
*  Note, the behaviors described below can be affected by the dataSource properties
*  DataSource.ownerIdNullRole and 
*  DataSource.ownerIdNullAccess, so please read the documentation 
*  for those two properties in conjunction with this article.
* 
*  When a new row is added by a client-initiated DSRequest, the
*  ownerIdField will be automatically populated with the currently
*  authenticated user (clobbering any value supplied by the client).
*  Client-initiated attempts to update the ownerIdField will also be
*  prevented.
* 
*  If you wish to set the ownerIdField to a different value via an "add"
*  or "update" operation, you can do so in server-side DMI code (possibly
*  consulting DSRequest.getClientSuppliedValues() to get the
*  value that was clobbered).
* 
*  For client-initiated "fetch", "update" or "remove" operations, the
*  server will modify client-supplied criteria so that only rows whose
*  ownerIdField matches the currently authenticated user can be read,
*  updated or deleted. For built-in DataSource types (SQL, Hibernate and 
*  JPA), the additional criteria required to match the ownerIdField 
*  field will ignore the prevailing
*  DSRequest.textMatchStyle and always use exact 
*  equality. If you have a custom or generic DataSource implementation, 
*  you will want to do the same thing, to avoid false positives on 
*  partial matches (eg, a user with name "a" gets records where the owner
*  is any user with an "a" in the name). You can determine when this is 
*  necessary by looking for a DSRequest attribute called 
*  "restrictedOwnerIdField". For example, code similar to the following:
* 	String restrictedField = (String)dsRequest.getAttribute("restrictedOwnerIdField");
* 	if (field.getName() != null && field.getName().equals(restrictedField)) {
*  // Use exact matching for this field
* 	} else {
*  OK to use the textMatchStyle
* 	}
*  
*  Also note, for server-initiated requests, this automatic criteria-narrowing is not 
*  applied; if your application requires server-initiated "fetch", "update" and "remove" 
*  requests to be limited to the currently-authenticated user, your code must add the 
*  necessary criteria to the request.
* 
*  The ownerIdField setting can be overridden at the
*  OperationBinding.ownerIdField level.
* 
*  If ownerIdField is specified,
*  DataSource.requiresAuthentication will
*  default to true. If requiresAuthentication is
*  explicitly set to false, then unauthenticated users will be
*  able to see all records. To avoid this, you can use
*  DataSource.guestUserId to specify a default user to
*  apply when no one has authenticated.
* @type {string}
* @default null
*/
ownerIdField?: string;

/**
* Value to use for the DataSource.ownerIdField if no one 
*  has authenticated.
* 
*  This setting can be overridden at the operationBinding level.
* @type {string}
* @default null
*/
guestUserId?: string;

/**
* If DataSource.ownerIdField is in force, specifies a role that will
*  allow the ownerIdField to take a null value. Any user that has that role 
*  is allowed to create client-initiated "add" and "update" operations that specify a null
*  value for the ownerIdField, and the system will persist the null value 
*  rather than forcing in the currently authenticated user's user id as it normally would.
*  If any client-initiated "add" or "update" request specifies any non-null value 
*  for the ownerIdField, the normal behavior of the system will reassert and 
*  the current user's user id will be forced into the ownerIdField. This 
*  allows authorised users (ie, those with the necessary role) to choose between saving 
*  public or private records, just by sending a null or non-null value for the 
*  ownerIdField.
*  
*  This property has no effect if ownerIdField is not specified.
*  
*  This property can be used in conjunction with DataSource.ownerIdNullAccess to 
*  create the concept of shared, or public, records - see that property's documentation 
*  for an example.
* @type {string}
* @default null
*/
ownerIdNullRole?: string;

/**
* If DataSource.ownerIdField is in force, specifies the access that 
*  is allowed to records with a null ownerIdField. This property has no 
*  effect if ownerIdField is not specified.
*  
*  This property can be used in conjunction with DataSource.ownerIdNullRole to 
*  create the concept of shared, or public, records. For example, if you set 
*  ownerIdNullRole to "administrator", any users with the "administrator" 
*  role will be allowed to write records with a null ownerIdField. If you 
*  also set ownerIdNullAccess to "view", all those records with a null owner
*  will be viewable by all, in addition to their own records. We use this functionality 
*  with the SavedSearches, to enable precisely this: 
*  users can save their own searches, which are private to them, but administrators can 
*  also save searches with a null ownerIdField, which become standard, shared
*  searches that appear to all users, in addition to their own private searches.
* @type {NullAccessType}
* @default null
*/
ownerIdNullAccess?: NullAccessType;

/**
* Setting a DataSource to be serverOnly="true" ensures that it will not be visible
*  to the client. Any request through IDACall to this DataSource will return a failure response.
*  Only requests which have been initiated on the server-side will be executed against this DataSource.
* @type {string}
* @default null
*/
serverOnly?: string;

/**
* If true, causes all operations on this DataSource to automatically start or join a 
*  transaction associated with the current HttpServletRequest. This means that multiple
*  operations sent to the server in a RPCManager.startQueue will be
*  committed in a single transaction. 
*  
*  Note that this includes fetch operations - setting this property to true has the same
*  effect as a transaction policy of ALL for just this DataSource's operations - see the 
*  server-side RPCManager.setTransactionPolicy() for details of the different 
*  TransactionPolicy settings.
*  
*  If this property is explicitly false, this causes all operations on this DataSource 
*  to be committed individually - the same as a transaction policy of NONE, just for this
*  DataSource's operations.
*  
*  In either case, you can override the setting for individual operations - see 
*  OperationBinding.autoJoinTransactions.
*  
*  If this property if null or not defined, we fall back to the default setting for this
*  type of DataSource. These are defined in server_properties
*  as follows:
*  
*  
* - Hibernate: hibernate.autoJoinTransactions
*  
* - JPA/JPA2: jpa.autoJoinTransactions
*  
* - SQL: There is one setting per configured database connection (DataSource.dbName).
*  For example, the setting for the default MySQL connection is 
*  sql.Mysql.autoJoinTransactions
*  
*  If the setting is not defined at the DataSource-type level, we use the system global 
*  default, which is defined in server.properties as 
*  autoJoinTransactions.
*  
*  At the dbName and global system levels, you can set the autoJoinTransactions attribute 
*  to a valid Transaction Policy, rather than a simple true or false (although these 
*  values work too - true is the same as ALL, false is the same as NONE). For valid 
*  TransactionPolicy values and their meanings, see the server-side Javadoc for 
*  RPCManager.setTransactionPolicy()
*  
*  Note that the configuration settings discussed here can be overridden for a particular 
*  queue of requests by setting the server-side RPCManager's transaction policy. Look in
*  the server-side Javadoc for RPCManager.getTransactionPolicy().
*  
*  Transactions can also be initiated manually, separate from the
*  RPCManager/HttpServletRequest lifecycle, useful for both multi-threaded web
*  applications, and standalone applications that don't use a servlet container - see
*  standaloneDataSourceUsage.
*  
*  NOTE: Setting this property to true does not cause a transactional persistence 
*  mechanism to automatically appear - you have to ensure that your DataSource supports 
*  transactions. The SmartClient built-in SQL, Hibernate and JPA DataSources support transactions, 
*  but note that they do so at the provider level. This means that you can combine 
*  updates to, say, an Oracle database and a MySQL database in the same queue, but they
*  will be committed in two transactions - one per database. The SmartClient 
*  server will commit or rollback these two transactions as if they were one, so a 
*  failure in some Oracle update would cause all the updates to both databases to be 
*  rolled back. However, this is not a true atomic transaction; it is possible for one
*  transaction to be committed whilst the other is not - in the case of hardware failure,
*  for example.
*  
*  NOTE: Not all the supported SQL databases are supported for transactions. Databases 
*  supported in this release are:
*  
*  
* - DB2
*  
* - HSQLDB
*  
* - Firebird
*  
* - Informix
*  
* - Microsoft SQL Server
*  
* - MySQL (you must use InnoDB tables; the default MyISAM storage engine does not
*  support transactions)
*  
* - MariaDB
*  
* - Oracle
*  
* - PostgreSQL
*  
* @type {boolean}
* @default null
*/
autoJoinTransactions?: boolean;

/**
* This flag is part of the Automatic Transactions feature; it is only applicable in 
*  Power Edition and above.
*  
*  If true, causes all transactional operations on this DataSource to use the current 
*  Spring-managed transaction, if one exists. If there is no current Spring transaction 
*  to use at the time of execution, a server-side Exception is thrown. Note, a 
*  "transactional operation" is one that would have joined the SmartClient shared 
*  transaction in the absence of Spring integration - see 
*  DataSource.autoJoinTransactions.
*  
*  This feature is primarily intended for situations where you have 
*  dmiOverview that make use of both Spring DAO operations and 
*  SmartClient DSRequest operations, and you would like all of them to share the same 
*  transaction. An example of the primary intended use case:
*   &#x0040;Transactional(isolation=Isolation.READ_COMMITTED, propagation=Propagation.REQUIRED)
*   public class WorldService {
* 
*    public DSResponse doSomeStuff(DSRequest dsReq, HttpServletRequest servletReq) 
*    throws Exception 
*    {
*     ApplicationContext ac = (ApplicationContext)servletReq.getSession().getServletContext().getAttribute("applicationContext");
*     WorldDao dao = (WorldDao)ac.getBean("worldDao");
*     dao.insert(req.getValues());
*     DSRequest other = new DSRequest("MyOtherDataSource", "add");
*     // Set up the 'other' dsRequest with critiera, values, etc
*     // ...
* 
*     // This dsRequest execution will use the same transaction that the DAO operation
*     // above used; if it fails, the DAO operation will be rolled back
*     other.execute();
* 
*     return new DSResponse();
*    }
*   }
*  Note: if you want to rollback an integrated Spring-managed transaction, you can use 
*  any of the normal Spring methods for transaction rollback:
*  
* - Programmatically mark the transaction for rollback with the 
*  setRollbackOnly() API
*  
* - Throw a RuntimeException, or
*  
* - Throw an ordinary checked Exception. but configure Spring to rollback
*    on that Exception. This can be done in the @Transactional annotation:
*    &#x0040;Transactional(isolation=Isolation.READ_COMMITTED, propagation=Propagation.REQUIRED, rollbackFor=MyRollbackException.class)
*  
*  Spring's exception-handling model is different from SmartClient's, so care must be
*  taken to get the correct error processing. If a transactional DSRequest fails, 
*  SmartClient code will throw an ordinary checked Exception; but Spring will
*  ignore that Exception. So you can either:
*  
* - Wrap every DSRequest.execute() in a try/catch block. Catch 
*  Exception and throw a RuntimeException instead
*  
* - Just use the "rollbackFor" annotation to make your transactional method rollback 
*  for all instances of Exception
*  
*  
*  Note: Spring transaction integration is conceptually different from SmartClient's 
*  DataSource.autoJoinTransactions, because SmartClient
*  transactions apply to a queue of DSRequests, whereas Spring transactions are scoped to 
*  a single method invocation. If you want to make a whole SmartClient queue share a 
*  single Spring-managed transaction, you can wrap the processing of an entire queue in a 
*  call to a transactional Spring method. See the Using Spring Transactions with 
*  SmartClient DMI section at the bottom of the 
*  springIntegration for more details.
*  
*  You can set useSpringTransaction as the default setting for all dataSources 
*  for a given database provider by adding the property 
*  {dbName}.useSpringTransaction to your server.properties file.
*  For example, Mysql.useSpringTransaction: true or 
*  hibernate.useSpringTransaction: true. You can set it as the default for 
*  all providers with a server.properties setting like this: 
*  useSpringTransaction: true. When useSpringTransaction is 
*  the default, you can switch it off for a specific dataSource by explicitly setting the
*  flag to false for that DataSource.
*  
*  Finally, this setting can be overridden at the operationBinding level - see 
*  OperationBinding.useSpringTransaction
*  Configuration
*  When using Spring transactions, SmartClient needs a way to lookup the JNDI connection 
*  being used by Spring, and this needs to be configured. First, register a bean like 
*  this in your applicationContext.xml file:
*   &lt;bean id="dataSource" class="org.springframework.jndi.JndiObjectFactoryBean"&gt;
*     &lt;!-- Set this to the JNDI name Spring is using --&gt;
*     &lt;property name="jndiName" value="isomorphic/jdbc/defaultDatabase"/&gt;
*   &lt;/bean&gt;
*  
*  and then add a line like this to your server.properties:
*   # Set this property to match the "id" of the JndiObjectFactoryBean registered in Spring
*   sql.spring.jdbcDataSourceBean: dataSource
*  
* @type {boolean}
* @default null
*/
useSpringTransaction?: boolean;

/**
* The paging strategy to use for this DataSource. If this property is not set, the 
*  default paging strategy, specified with the server_properties
*  setting sql.defaultPaging, is used. 
*  
*  This setting can be overridden with the OperationBinding.sqlPaging property.
*  
*  NOTE: Operations that involve a OperationBinding.customSQL 
*  clause ignore this property, because customSQL operations usually need to be treated
*  as special cases. For these operations, the paging strategy comes from the 
*  server_properties setting
*  sql.defaultCustomSQLPaging or 
*  sql.defaultCustomSQLProgressivePaging, depending on whether or not 
*  DataSource.progressiveLoading is in force. Note that these
*  can always be overridden by a sqlPaging setting on the OperationBinding.
* @type {SQLPagingStrategy}
* @default null
*/
sqlPaging?: SQLPagingStrategy;

/**
* If explicitly set true or left null, causes the server to use a "hint" in the SQL we 
*  generate for paged queries. If explicitly set false, forces off the use of hints. This
*  property can be overridden per operationBinding - see 
*  OperationBinding.sqlUsePagingHint.
*  
*  Note this property is only applicable to DataSource.serverType DataSources, 
*  only when a DataSource.sqlPaging of "sqlLimit" is in force,
*  and it only has an effect for those specific database products where we employ a native
*  hint in the generated SQL in an attempt to improve performance.
* @type {boolean}
* @default null
*/
sqlUsePagingHint?: boolean;

/**
* This property is only applicable to DataSource.serverType DataSources, and 
*  only for OperationBinding that express a 
*  OperationBinding.customSQL clause. In these circumstances, we 
*  generally switch off paging because we are unable to generate a "row count" query that
*  tells the framework the size of the complete, unpaged resultset.
*  
*  The useSubselectForRowCount flag causes the framework to derive a rowcount
*  query by simply wrapping the entire customSQL clause in a subselect, like so:
*  &nbsp;&nbsp;&nbsp;&nbsp;SELECT COUNT(*) FROM ({customSQL clause here})
*  
*  However, this is not guaranteed to give good results. Because the customSQL clause can
*  contain anything that you can write in SQL, running it inside a subselect in order to 
*  count the rows might not work, might have unintended side-effects (if, for example, it 
*  is a stored procedure call that performs updates as part of its function), or it might 
*  just be a bad idea - for example, if the customSQL query is slow-running, you'll make 
*  it twice as slow with this flag, simply because you'll be running it twice. We 
*  recommend using this flag with care.
*  
*  NOTE: This setting can be overridden per-operation - see 
*  OperationBinding.useSubselectForRowCount. You can also set a global default for
*  this setting, so you don't have to specify it in every dataSource - define 
*  useSubselectForRowCount as true in your 
*  server_properties file.
* @type {boolean}
* @default null
*/
useSubselectForRowCount?: boolean;

/**
* Only applicable to the built-in SQL DataSource
*  
*  tableCode and the related properties DataSourceField.columnCode,
*  DataSourceField.fkTableCode and DataSourceField.fkColumnCode are read-only
*  attributes that are secure and unique cryptographic hashes of table and column names
*  used by this DataSource.
*  
*  These properties are used automatically by client-side framework code to 
*  link dataSources together by DataSourceField.foreignKey when a
*  foreignKey is not explicitly declared, but is found in the SQL schema via
*  the DataSource.autoDeriveSchema feature.
*  
*  A secure hash is used rather than the actual SQL table or column name for security
*  reasons - sending the actual SQL table or column name to the client could aid in
*  attempted SQL injection attacks.
*  
*  This feature can be disabled system-wide via setting datasource.autoLinkFKs
*  to false in server_properties.
* @type {string}
* @default null
*/
tableCode?: string;

/**
* If true, causes SmartClient Server to use the "progressive loading" pattern for 
*  fetches on this dataSource, as described in the Paging and total dataset length
*  section of the ResultSet. Essentially, this means that
*  we avoid issuing a row count query and instead advertise total rows as being slightly 
*  more than the number of rows we have already read (see DataSource.endGap). This 
*  allows users to load more of
*  a dataset by scrolling past the end of the currently-loaded rows, but it prevents them
*  from scrolling directly to the end of the dataset.
*  
*  Generally, progressive loading is appropriate when you have to deal with very large 
*  datasets. Note that by default, a dataSource will switch into progressive loading mode 
*  automatically when it detects that it is dealing with a dataset beyond a certain size - 
*  see DataSource.progressiveLoadingThreshold.
*  
*  This setting can be overridden for individual fetch operations with the 
*  OperationBinding.progressiveLoading property, and also at the level of the
*  individual DSRequest.progressiveLoading. You can also specify 
*  progressiveLoading on 
*  DataBoundComponent.progressiveLoading and certain types of 
*  FormItem - SelectItem.progressiveLoading and 
*  ComboBoxItem.progressiveLoading.
*  
*  Currently, this property only applies to users of the built-in SQLDataSource, but you 
*  could use it in custom DataSource implementations to trigger the server behavior 
*  described in the ResultSet documentation linked to above.
* @type {boolean}
* @default null
*/
progressiveLoading?: boolean;

/**
* Indicates the dataset size that will cause SmartClient Server to automatically
*  switch into DataSource.progressiveLoading for this DataSource.
*  To prevent automatic switching to progressive loading, set this property to -1. This
*  may also be prevented on a per-request basis by setting DSRequest.progressiveLoading
*  to false.
* @type {Integer}
* @default "200000"
*/
progressiveLoadingThreshold?: Integer;

/**
* If we are DataSource.progressiveLoading, indicates the number of 
*  extra records SmartClient Server will read beyond the end record requested by the client,
*  in order to establish if there are more records to view. This property has no effect 
*  if we are not progressive-loading.
*  
*  This property can be tweaked in conjunction with DataSource.endGap to change behavior
*  at the end of a dataset. For example, with the default values of lookAhead: 1
*  and endGap: 20, we can end up with the viewport shrinking if we get a case
*  where there really was only one more record (because the client was initially told there
*  were 20 more). This is not a problem per se, but it may be surprising to the user.
*  You could prevent this happening (at the cost of larger reads) by setting 
*  lookAhead to be endGap+1.
* @type {Integer}
* @default "1"
*/
lookAhead?: Integer;

/**
* For DataSource.serverType DataSources, sets the
*  default DataSourceField.sqlStorageStrategy to use for boolean fields
*  where no sqlStorageStrategy has been declared on the field.
*  
*  Can also be set system-wide via the server_properties setting
*  sql.defaultBooleanStorageStrategy, or for a particular database configuration by
*  setting sql.dbName.defaultBooleanStorageStrategy (see
*  adminConsole for more information on SQL configuration).
*  
*  Note that when this property is unset, the default DataSourceField.sqlStorageStrategy
*  strategy is effectively "string".
* @type {string}
* @default null
*/
defaultBooleanStorageStrategy?: string;

/**
* If we are DataSource.progressiveLoading, indicates the number of 
*  extra records SmartClient Server will advertise as being available, if it detects that 
*  there are more records to view (see DataSource.lookAhead). This property has no 
*  effect if we are not progressive-loading.
* @type {Integer}
* @default "20"
*/
endGap?: Integer;

/**
* Applies to sqlDataSource only.
*  
*  If this attribute is set to a non-negative value, it acts as a hard safety limit for 
*  client-initiated DSRequests for "all rows". If the server encounters more rows 
*  in a response than this safety limit, it will abort immediately with an Exception.
*  
*  This attribute is not meant to be a regular application facility. As mentioned 
*  above, it is a safety mechanism, intended primarily to prevent bugs in development from
*  causing long delays or server Out Of Memory crashes by unintentionally issuing requests
*  that fetch huge numbers of rows (eg, by failing to specify filter criteria).
*  
*  Note the following:
*  
* - This limit only has an effect when "all rows" are requested - ie, when the request
*  does not specify an DSRequest.endRow, or specifies endRow:-1.
*  If you specify a non-negative endRow, it will be honored even if that 
*  means we need to return more than clientRequestMaxRows records
*  
* - If you need to handle very large datasets in a manageable way, consider using 
*  progressiveLoading to stream the data progressively
*  
* - Note that this attribute applies to client-initiated requests only. If you want to
*  provide a hard safety limit for all fetches, including requests initiated on the server,
*  set a DataSource.requestMaxRows instead. If both properties are specified, 
*  clientRequestMaxRows wins for client-initiated requests, so it is possible
*  to configure different limits for client- and server-initiated requests
*  
*  
*  To set a default clientRequestMaxRows that will apply to all dataSources
*  that do not specify the attribute, add the following to your server.properties
*  file:
*    # Fail with an error if we try to fetch more than 10000 rows in a client request
*    sql.clientRequestMaxRows: 10000
*  
* @type {Integer}
* @default "-1"
*/
clientRequestMaxRows?: Integer;

/**
* Applies to sqlDataSource only.
*  
*  The same as DataSource.clientRequestMaxRows, but applies 
*  to all requests, not just client-initiated ones. See the documentation for 
*  clientRequestMaxRows for details. To reiterate the warning given for 
*  that property, this is a safety limit that results in an Exception being thrown: it
*  is not intended to be used as a regular application facility
*  
*  To set a default requestMaxRows that will apply to all dataSources that do
*  not specify the attribute, add the following to your server.properties
*  file:
*    # Fail with an error if we try to fetch more than 50000 rows in any request
*    sql.clientRequestMaxRows: 50000
*  
* @type {Integer}
* @default "-1"
*/
requestMaxRows?: Integer;

/**
* Set this property to true to have a DataSource fetch all of its data client-side on the 
*  first fetch request. However, unlike a DataSource.clientOnly DataSource, this DataSource 
*  will still save changes normally, sending remote requests.
*  
*  You can manually set this attribute after initialization by calling 
*  DataSource.setCacheAllData; setting DataSource.autoCacheAllData:true causes a DataSource to 
*  automatically switch to cacheAllData:true when a fetch results in the
*  entire dataset being brought client-side.
*  
*  To cause automatic cache updates, you can set DataSource.cacheMaxAge to a number of seconds
*  and once data has been client-side for that length of time, the next fetch causes the
*  cache to be dropped and a new cache retrieved.
*  
*  Note that multiple DataSource.operationBindings of type "fetch" which return distinct
*  results will not work with cacheAllData: only one cache is created and is
*  used for all fetch operations, regardless of whether DSRequest.operationId has
*  been set. However, "fetch" operationBindings used as a
*  OperationBinding.cacheSyncOperation will work normally, so long as they return
*  all data fields that are returned by the default "fetch" operation, so that the cache
*  can be updated.
*  
*  To specify which operationId to use for fetching all data, use
*  DataSource.cacheAllOperationId.
*  
*  To use the cache only for requests that have the cacheAllOperationId,
*  allowing any other operationId (or absence of an operationId) to contact the server as
*  normal, set DataSource.cacheAcrossOperationIds.
* @type {boolean}
* @default null
*/
cacheAllData?: boolean;

/**
* DSRequest.operationId to use for fetching data in case DataSource.cacheAllData is
*  true. By default a standard fetch operation is used (with no operationId
*  specified).
* @type {string}
* @default null
*/
cacheAllOperationId?: string;

/**
* When DataSource.cacheAllData mode is enabled and a
*  DataSource.cacheAllOperationId has been set, this flag affects whether cached
*  results are used for all "fetch" requests regardless of their
*  DSRequest.operationId, or are used only for "fetch" requests that use the
*  cacheAllOperationId, allowing other requests to go to server normally.
*  
*  Default of true means that the cacheAllOperationId will be
*  used to fetch all rows, but the resulting cache will be used for all "fetch" operations
*  regardless of the operationId specified on the request.
*  
*  Switching to "false" effectively creates caching just for one specific
*  operationId - the cacheAllOperationId - while allowing all
*  other requests to go to the server normally.
* @type {boolean}
* @default true
*/
cacheAcrossOperationIds?: boolean;

/**
* The maximum time, in seconds, to maintain the client-side cache. If a fetch occurs after
*  the cacheMaxAge has expired, the current cache will be dropped and another complete
*  cache fetched.
* @type {number}
* @default "60"
*/
cacheMaxAge?: number;

/**
* For a DataSource.cacheAllData or client-only DataSource, a set of records to use as a dataset, 
*  specified as an Array of JavaScript Objects representing records.
*  
*  See clientOnlyDataSources for ways to populate a
*  client-only DataSource with cache data.
*  
*  Additionally, when a DataSource is loaded in DataSource.mockMode, cacheData,
*  if provided, is used as the mock data.
*  
*  For any other DataSource, cacheData is dropped when loaded.
* @type {Array<Partial<Record>>}
* @default null
*/
cacheData?: Array<Partial<Record>>;

/**
* For a client-only DataSource, a set of records to use as a dataset, specified as an
*  Array of JavaScript Objects. 
*  
*  See clientOnlyDataSources for ways to populate a
*  client-only DataSource with test data.
* @deprecated * In favor of DataSource.cacheData.

* @type {Array<Partial<Record>>}
* @default null
*/
testData?: Array<Partial<Record>>;

/**
* Set this attribute if you need to send the dsRequest.parentNode to the server-side.
* @type {boolean}
* @default false
*/
sendParentNode?: boolean;

/**
* When a DataSource is not DataSource.cacheAllData:true and a fetch results in the
*  entire dataset being retrieved, this attribute being set to true causes the DataSource
*  to automatically switch to cacheAllData:true and prevent further server-trips for fetch 
*  requests.
*  
*  DataSource.cacheAllData is automatically enabled in either of these conditions:
*  
*   
* - 
*    The request has no criteria and no startRow/endRow request properties. The latter can
*    be accomplished by disabling paging with a DataBoundComponent.dataFetchMode
*    setting of "basic" or "local" or by an explicit fetchData request with those request properties
*    excluded.
*  
*  
* - 
*    The request has no criteria but has startRow/endRow specified and the response received has
*    all data available (startRow:0 and endRow:totalRows).
*  
*  
* @type {boolean}
* @default false
*/
autoCacheAllData?: boolean;

/**
* When set, causes a DataSource.clientOnly or DataSource.cacheAllData DataSource to 
*  create a second DataSource to perform it's one-time fetch. By default, this attribute
*  will be considered true when clientOnly is true, cacheAllData is false or unset and
*  a dataURL or testFileName is specified on the DataSource.
* @type {boolean}
* @default null
*/
useTestDataFetch?: boolean;

/**
* Whether to convert relative date values to concrete date values before sending to the 
*  server. Default value is true, which means that the server does not need to understand 
*  how to filter using relative dates - it receives all date values as absolute dates.
*  
*  If the server would receive relative date values from the client, by default they would
*  be unchanged in DMI and automatically converted during the request execution. This may
*  be changed via server.properties setting datasources.autoConvertRelativeDates
*  which can be set to the following values:
*  
*  postDMI - the default value described above
*  preDMI - relative date values will be converted to absolute date values
*  right away, so they will be already converted in DMI
*  disabled - relative date values will not be automatically converted, so it
*  must be done completely manually or by calling the
*  DSRequest.convertRelativeDates() server-side API.
*  
*  Normally there is no need to convert relative dates on the server, this is done by default on
*  the client before the request is sent to the server. The primary purpose for converting relative
*  dates on the server is when there is a need to store and use relative dates at a later point such
*  as in an automated job without any involvement from the client. See more details in the
*  javadoc for DataSource.convertRelativeDates(Criterion) server-side API.
* @type {boolean}
* @default true
*/
autoConvertRelativeDates?: boolean;

/**
* Causes Tree.discoverTree to be called on dsResponse.data in order to automatically
*  discover tree structures in the response data.
*  
*  If autoDiscoverTree is set, discoverTree() is called after the default dsResponse.data
*  has been derived (OperationBinding.recordXPath and
*  DataSourceField.valueXPath have been applied) and after
*  DataSource.transformResponse has been called.
*  
*  If a DataSourceField is declared with
*  DataSourceField.childrenProperty, discoverTree() will be invoked with
*  DiscoverTreeSettings.newChildrenProperty set to the name of
*  the field marked as the childrenField. Similarly, if the DataSource has a
*  DataSource.titleField it will be used as the
*  DiscoverTreeSettings.nameProperty.
* @type {boolean}
* @default false
*/
autoDiscoverTree?: boolean;

/**
* Settings to use when discoverTree() is automatcially called because
*  DataSource.autoDiscoverTree is set to true for this DataSource
* @type {DiscoverTreeSettings}
* @default null
*/
discoverTreeSettings?: DiscoverTreeSettings;

/**
* For DataSources using the sqlDataSource for
*  persistence, whether to use ANSI-style joins (ie, joins implemented with JOIN directives
*  in the table clause, as opposed to additional join expressions in the where clause).
*  The default value of null has the same meaning as setting this flag to false.
*  
*  Note, outer joins (see DataSourceField.joinType) only work with certain
*  database products if you choose not to use ANSI joins. Other than that, the join 
*  strategies are equivalent.
*  
*  If you wish to switch on ANSI-style joins for every DataSource, without the need to 
*  manually set this property on all of them, set
*  server_properties flag 
*  sql.useAnsiJoins to true.
* @type {boolean}
* @default null
*/
useAnsiJoins?: boolean;

/**
* For this dataSource, should the millisecond portion of time and datetime values be 
*  trimmed off before before being sent from client to server or vice versa. By default, 
*  millisecond information is preserved (ie, it is not trimmed). You only need to consider
*  trimming millisecond values if their presence causes a problem - for example, a custom
*  server that is not expecting that extra information and so fails parsing.
*  
*  Note that there is no inherent support for millisecond precision in SmartClient widgets;
*  if you need millisecond-precise visibility and editability of values in your client, 
*  you must write custom formatters and editors (or sponsor the addition of such things to
*  the framework). Server-side, millisecond-precise values are delivered to and obtained
*  from DataSources, so DataSource implementations that are capable of persisting and 
*  reading millisecond values should work transparently. Of the built-in DataSource types,
*  the JPA and Hibernate DataSources will transparently handle millisecond-precise values
*  as long as the underlying database supports millisecond precision, and the underlying 
*  column is of an appropriate type. The SQLDataSource provides accuracy to the nearest
*  second by default; you can switch on millisecond precision per-field with the 
*  DataSourceField.storeMilliseconds attribute.
* @type {boolean}
* @default null
*/
trimMilliseconds?: boolean;

/**
* If set to "false", transformation of values for
*  DataSourceField.multiple fields, normally controlled by
*  DataSourceField.multipleStorage, is instead disabled for this entire DataSource.
* @type {boolean}
* @default null
*/
transformMultipleFields?: boolean;

/**
* Enables saving of a log of changes to this DataSource in a second DataSource with the same
*  fields, called the "audit DataSource". NOTE: this feature applies to Enterprise
*  Edition only; for more information on edition-specific features, see
*  http://smartclient.com/product.
*  
*  When auditing is enabled, every time a DSRequest modifies this DataSource, a Record is added
*  to the audit DataSource that shows the record as it existed after the change was made (or
*  for a "remove", the values of the record at the time of deletion). In addition, the audit
*  DataSource has the following additional metadata fields:
*  
*  
* - DataSource.auditTypeFieldName: type of the change ("update", "add" or "remove")
*  
* - DataSource.auditUserFieldName: username of the user that made the change. The username is
*    determined in the same way that the 
*    OperationBinding.requiresRole subsystem determines the 
*    current user.
*  
* - DataSource.auditTimeStampFieldName: a field of type "datetime" recording
*    the timestamp of the change
*  
* - DataSource.auditRevisionFieldName: a field of type "sequence" recording a
*    simple increasing integer value
*  
* - DataSource.auditChangedFieldsFieldName: a
*    DataSourceField.multiple field of type "string". For "update"
*    operations, records which fields have changed; otherwise, null
*  
*  
*  If any of the field names above collide with field names of the DataSource being audited,
*  an integer suffix will also be added, starting with 2 (for example, "audit_modifier2", then
*  "audit_modifier3", etc).
*  
*  To omit a data field from the automatically generated audit DataSource, just set
*  DataSourceField.audit to false. Audit can be disabled for a given DSRequest via
*  the server-side API DSRequest.setSkipAudit(), or for a specific operation 
*  via the OperationBinding.skipAudit setting.
*  
*  When viewing DataSources in the dataSourcesTab, a button will
*  be shown at the top of a DataSource's section if it has an audit DataSource to let you
*  conveniently open another section to view it.
*  
*  Note: The DataSource audit feature works only with single row operations; operations with 
*  OperationBinding.allowMultiUpdate enabled are not supported.
*  
*  Auto-generated Audit DataSources
*  
*  The audit DataSource is normally automatically generated, and unless otherwise specified
*  with DataSource.auditDataSourceID, the ID of the audit DataSource will be
*  audit_[OriginalDSID]. 
*  
*  By default, the automatically generated audit DataSource will be of the same type as the
*  DataSource being audited, however, if the DataSource being audited is not already a
*  SQLDataSource, we recommend using DataSource.auditDSConstructor to use
*  a SQLDataSource as the audit DataSource. This is because a SQLDataSource used an audit
*  DataSource will automatically generate a SQL table for storing audit data the first time
*  changes are made. JPA would require manual creation of a Java Bean, and Hibernate requires 
*  hbm2ddl.auto=update to be set,
*  which is widely considered unsafe for production use.
*  
*  Note that the automatically generated audit DataSource may have additional fields that can be populated when new
*  audit data is added. See DataSource.addedAuditFields for more details.
*  
*  Automatically created audit DataSources can be loaded and queried just like other
*  DataSources, using the DataSourceLoader, and using the server-side API
*  DataSource.getAuditDataSource(). However, you must load the DataSource
*  being audited before loading its automatically created audit DataSource.
*  
*  Note, that automatic SQL tables creation can be disabled. See 
*  DataSource.autoCreateAuditTable for details.
*  
*  Connecting custom "Users" DataSource to auto-generated Audit DataSources
*  
*  Audit DataSources utilize authenticated username (see the docs for RPCManager.getUserId() server-side
*  API) to populate the "audit_modifier" field. To enable quick access to additional user information, you can associate
*  them with a custom "Users" DataSource.
*  
*  This association can be configured via the framework setting "userDataSource.foreignKey" in the "server.properties"
*  file, which establishes a relationship between authenticated user and your custom "Users" DataSource. Declaring this
*  informs the system that the authenticated username is a valid ID for records within a specific "Users" DataSource.
*  Use the regular DataSourceField.foreignKey format, where, in example below, "UserDS" represents the name of the
*  related "Users" DataSource, and "userName" serves as the primary key of that DataSource:
*  
*  userDataSource.foreignKey: UserDS.userName
*  
*  This configuration enables you to access additional user information seamlessly through built-in framework features
*  like DSRequest.additionalOutputs and ListGridField.includeFrom when reviewing audit data. For instance,
*  if "UserDS" declares fields such as "firstName" and "lastName," you can retrieve them using additional listGridFields:
*  
*  {name: "userFirstName", includeFrom: "UserDS.firstName"},
*  {name: "userLastName", includeFrom: "UserDS.lastName"}
*  
*  Alternatively, if you're fetching data manually, you can utilize additionalOutputs as follows:
*  
*  additionalOutputs: "userFirstName!UserDS.firstName,userLastName!UserDS.lastName"
*  
*  Manually created Audit DataSources
*  
*  The audit DataSource can also be manually created. In this case, you can 
*  can either follow the naming conventions described above for the ID of the audit DataSource
*  and the names of metadata fields, or use the linked properties to assign custom names. If
*  you omit any data fields from the tracked DataSource in your audit DataSource, those fields
*  will be ignored for auditing purposes, exactly as though DataSourceField.audit had
*  been set to false for an automatically-generated audit DataSource.
*  
*  Also, note that in case of manually defined audit DataSource, if this DataSource
*  is defined so it inherits the audited DataSource, all the audited DataSource's fields
*  will be inherited, this including the primary keys. Since for the audit
*  DataSource the primary key should be the revision field, in order to prevent the
*  audit DataSource having two primary keys, the inherited DataSource's primary key 
*  will have to be declared in audit DataSource, but with the primaryKey attribute 
*  omitted (as well not being of type "sequence") in the audit DataSource.
* @type {boolean}
* @default false
*/
audit?: boolean;

/**
* For DataSources with DataSource.audit, optionally specifies the ID of the audit
*  DataSource. If this property is not specified, the ID of the audit DataSource will
*  be audit_[OriginalDSID] 
* @type {string}
* @default null
*/
auditDataSourceID?: string;

/**
* For DataSources with DataSource.audit, specifies the field name used to store
*  the user who performed the operation. If empty string is specified as the field name, the
*  audit DataSource will not store this field.
* @type {string}
* @default "audit_modifier"
*/
auditUserFieldName?: string;

/**
* For DataSources with DataSource.audit, specifies the field name used to store
*  the timestamp when the operation was performed (in a field of type "datetime"). If empty
*  string is specified as the field name, the audit DataSource will not store this field.
* @type {string}
* @default "audit_changeTime"
*/
auditTimeStampFieldName?: string;

/**
* For DataSources with DataSource.audit, specifies the field name used to store
*  the revision number for the change (in a field of type "sequence"). If empty
*  string is specified as the field name, the audit DataSource will not store this field.
* @type {string}
* @default "audit_revision"
*/
auditRevisionFieldName?: string;

/**
* For DataSources with DataSource.audit, specifies the field name used to store
*  the names of the fields which were updated. If empty string is specified as the field name,
*  the audit DataSource will not store this field.
*  
*  Note that this field will only be set for DSOperationType operations.
* @type {string}
* @default "audit_changedFields"
*/
auditChangedFieldsFieldName?: string;

/**
* For DataSources with DataSource.audit, specifies the length of the field used 
*  to store the names of the fields which were updated. See also 
*  DataSource.auditChangedFieldsFieldName
*  
*  To set the changedFields field length for all DataSources that do not override the default,
*  set audit.auditChangedFieldsFieldLength in your server.properties
*  file. For example 
*     audit.auditChangedFieldsFieldLength: 512
*  
* @type {Integer}
* @default "255"
*/
auditChangedFieldsFieldLength?: Integer;

/**
* For DataSources with DataSource.audit, specifies the field name used to store
*  the DSOperationType (in a field of type "text"). If empty
*  string is specified as the field name, the audit DataSource will not store this field.
* @type {string}
* @default "audit_operationType"
*/
auditTypeFieldName?: string;

/**
* For DataSources with DataSource.audit, optionally specifies the
*  DataSource.serverConstructor for the automatically generated audit DataSource. The
*  default is to use the same serverConstructor as the DataSource where
*  audit="true" was declared.
*  
*  This property is primarily intended to allow the use of SQLDataSource
*  (DataSource.serverType) as an audit DataSource for a DataSource that
*  might be of another type. For example, you might have a DataSource that implements all CRUD
*  operations via Java logic in dmiOverview methods, and so doesn't provide generic
*  storage; by using SQLDataSource as the type of your audit DataSource, you don't need to
*  implement your own scheme for storing and querying audit data, and the necessary audit
*  tables will be automatically generated in the database.
*  
*  Similarly, even if you do use a reusable DataSource type such as the built-in JPADataSource,
*  using SQLDataSource for audit DataSources means there's no need to write a JPA bean just to
*  achieve storage of an audit trail.
*  
*  To simplify this intended usage, the string "sql" is allowed for
*  auditDSConstructor as a means of specifying that the built-in SQLDataSource class
*  should be used. For any other type, use the fully qualified Java classname, as is normal
*  for serverConstructor.
* @type {string}
* @default null
*/
auditDSConstructor?: string;

/**
* Setting autoCreateAuditTable to true indicates that audit DataSource 
*  will automatically create SQL table when DataSource.audit is enabled.
*  
*  Note, that autoCreateAuditTable attribute takes effect only if framework setting 
*  audit.autoCreateTables in server.properties is set to false,
*  which by default is set to true.
* @type {boolean}
* @default true
*/
autoCreateAuditTable?: boolean;

/**
* Only applicable to binaryFields on DataSource.audit
*  DataSources.
*  
*  When determining if a binary field has changed for auditing purposes, should we compare the
*  metadata values (ie, the associated _filename and _filesize 
*  fields) or the actual binary content? If you set this flag to false, this will cause 
*  SmartClient to fetch the existing content of any binary field from the database before any
*  update, and then compare it byte by byte to the new content. You should consider if this 
*  will have performance implications for your application, particularly if you store large 
*  binary values.
*  
*  Note that value comparison of any kind is only performed if the field's 
*  DataSourceField.audit setting is "change", but also note that this is the default 
*  setting for binary fields
* @type {boolean}
* @default true
*/
compareMetadataForAuditChangeStatus?: boolean;

/**
* User-visible name for this DataSource.
*  
*  For example, for the supplyItem DataSource, "Supply Item".
*  
*  If is unset, getAutoTitle() method will be used with dataSource.ID.
*  value in order to derive a default value for the title.
*  
*  For example "employee" ID will be derived to "Employee", "team_member" ID will be 
*  derived to "Team Member".
* @type {string}
* @default "dataSource.ID"
*/
title?: string;

/**
* User-visible plural name for this DataSource.
*  
*  For example, for the supplyItem DataSource, "Supply Items".
*  
*  Defaults to dataSource.title + "s".
* @type {string}
* @default "dataSource.ID"
*/
pluralTitle?: string;

/**
* Best field to use for a user-visible title for an individual record from this dataSource.
*  
*  For example, for a DataSource of employees, a "full name" field would probably most clearly
*  label an employee record.
*  
*  If not explicitly set, the titleField is determined by looking for fields named "name",
*  "dataSourceIdName", "title", "dataSourceIdTitle", "label",
*  "dataSourceIdLabel", "id" and "dataSourceIdId". For example, for a
*  DataSource with ID "customer", a field called customerName would be found if there
*  were no "name" field. Search is case insensitive, and an underscore is allowed after
*  dataSourceId (so that, for example, "CUSTOMER_NAME" would also be found and preferred).
*  
*  For purposes of this search, any trailing numerals in the DataSource ID are discarded,
*  so a DataSource with ID "office2" will search for title fields as if the ID were just
*  "office".
*  
*  If no field is found that matches any of the names above, then the first field is
*  designated as the titleField.
* @type {string}
* @default "see below"
*/
titleField?: string;

/**
* Designates a field of FieldType:"image" as the field to use when rendering a
*  record as an image, for example, in a TileGrid.
*  
*  For example, for a DataSource of employees, a "photo" field of type "image" should be
*  designated as the iconField.
*  
*  If not explicitly set, iconField looks for fields named "picture", "thumbnail", "icon",
*  "image" and "img", in that order, and will use any of these fields as the iconField if it
*  exists and has type "image". 
*  
*  To avoid any field being used as the iconField, set iconField to null.
* @type {string}
* @default "see below"
*/
iconField?: string;

/**
* Name of the field that has the second most pertinent piece of textual information in the
*  record, for use when a DataBoundComponent needs to show a short summary of a record.
*  
*  For example, for a DataSource of employees, a "job title" field would probably be the second
*  most pertinent text field aside from the employee's "full name".
*  
*  Unlike DataSource.titleField, infoField is not automatically determined in the absence of an
*  explicit setting.
* @type {string}
* @default null
*/
infoField?: string;

/**
* Name of the field that has the most pertinent numeric, date, or enum value, for use when a
*  DataBoundComponent needs to show a short summary of a record.
*  
*  For example, for a DataSource of employees, good choices might be the "salary" field, "hire
*  date" or "status" (active, vacation, on leave, etc).
*  
*  Unlike DataSource.titleField, dataField is not automatically determined in the absence of an
*  explicit setting.
* @type {string}
* @default null
*/
dataField?: string;

/**
* Name of the field that has a long description of the record, or has the primary text data
*  value for a record that represents an email message, SMS, log or similar.
*  
*  For example, for a DataSource representing employees, a field containing the employee's
*  "bio" might be a good choice, or for an email message, the message body.
*  
*  If descriptionField is unset, it defaults to any field named "description" or "desc" in the
*  record, or the first long text field (greater than 255 characters) in the record, or null if
*  no such field exists.
* @type {string}
* @default null
*/
descriptionField?: string;

/**
* fieldName for a field in the dataSource expected to contain an explicit array of child nodes.
*  Enables loading a databound tree as a hierarchical data structure, rather than a flat list of
*  nodes linked by foreignKey.
*  Note this is an alternative to setting DataSourceField.childrenProperty directly on
*  the childrenField object.
*  By default the children field will be assumed to be DataSourceField.multiple,
*  for XML databinding. This implies that child data should be delivered in the format:
*  
*    &lt;childrenFieldName&gt;
*      &lt;item name="firstChild" ...&gt;
*      &lt;item name="secondChild" ...&gt;
*    &lt;/childrenFieldName&gt;
*  
*  However data may also be delivered as a direct list of childrenFieldName 
*  elements:
*  
*    &lt;childrenFieldName name="firstChild" ...&gt;
*    &lt;childrenFieldName name="secondChild" ...&gt;
*  
*  If you want to return your data in this format, you will need to explicitly set 
*  multiple to false in the appropriate dataSource field definition.
* @type {string}
* @default null
*/
childrenField?: string;

/**
* This property allows you to write and use custom DataSource subclasses on the server, by 
*  specifying either 
*  
* - the fully-qualified name of the DataSource subclass that should be instantiated
*  server-side for this dataSource, or
*  
* - the token "spring:" followed by a valid Spring bean ID, if you wish to instantiate 
*  your custom dataSource object using Spring dependency injection. For example, 
*  "spring:MyDataSourceBean". See also serverInit for special
*  concerns with framework initialization when using Spring. It is also particularly important
*  that you read the discussion of caching and thread-safety linked to below, as there are 
*  special considerations in this area when using Spring.
*  
* - the token "cdi:" followed by a valid CDI bean name, if you wish to instantiate 
*  your custom dataSource object using CDI dependency injection. For example, 
*  "cdi:MyDataSourceBean".
*  
*  
*  One reason you might wish to do this would be to override the validate() method to provide 
*  some arbitrary custom validation (such as complex database lookups, validation embedded in 
*  legacy applications, etc). It is also possible - though obviously a more substantial task -
*  to override the execute() method in your custom DataSource. This is one way of creating 
*  a completely customized DataSource implementation.
*  
*  Note: If you use this property, you are responsible for making sure that it refers to
*  a valid server-side class that extends com.isomorphic.datasource.BasicDataSource,
*  or to a Spring bean of the same description.
*  If your implementation relies on methods or state only present in certain specialized 
*  subclasses of DataSource (for example, you want the normal behavior and features of a 
*  HibernateDataSource, but with a specialized validate() method), then you should extend the 
*  subclass rather than the base class.
*  
*  NOTE: Please take note of the points made in 
*  serverDataSourceImplementation of caching and thread-safety
*  issues in server-side DataSources.
* @type {string}
* @default null
*/
serverConstructor?: string;

/**
* For dataSources of DataSource.serverType "sql", determines whether we qualify column
*  names with table names in any SQL we generate. This property can be overridden on specific
*  operationBindings.
* @type {boolean}
* @default true
*/
qualifyColumnNames?: boolean;

/**
* If true, indicates that the SmartClient Server should automatically apply a 
*  ValidatorType of "hasRelatedRecord" to every field 
*  on this dataSource that has a DataSourceField.foreignKey defined.
* @type {boolean}
* @default null
*/
validateRelatedRecords?: boolean;

/**
* This property has different meanings depending on the DataSource.serverType:
*  
*  For SQL DataSources (DataSources with serverType "sql")
*  If set, results from the database will be used to create one instance of the indicated Java
*  bean per database row. Otherwise a Map is used to represent each row retrieved from SQL.
*  
*  With this feature active, a DSResponse from this DataSource will contain a Collection of
*  instances of the indicated beanClassName, available via DSResponse.getData().
*  This creates a couple of possibilities:
*  
*  
*  Add business logic for derived properties, such as computed formulas
*  For example, declare a DataSourceField named "revenueProjection". By default this
*  field will call getRevenueProjection() on your bean to retrieve the value to send to the
*  client. Your implementation of getRevenueProjection() could apply some kind of formula to
*  other values loaded from the database.
*  Call business logic on retrieved beans via DMI
*  By adding a dmiOverview method that calls DSRequest.execute() to retrieve a DSResponse,
*  you have an opportunity to call business logic methods on the beans representing each
*  row affected by the DSRequest. For example, notify a related BPEL process of changes to
*  certain fields.
*  
*  
*  By using beanClassName on a specific OperationBinding, you can:
*  
*  
* - Use a bean to represent your data only when it matters; for example, avoid the overhead
*    of using a bean for "fetch" operations, but do use a bean for "update" operations so
*    that you can execute relevant business logic after the update completes.
*  
* - Skip the use of beans for complex reporting queries that produce results unrelated to
*    your persistent object model. Set beanClassName to blank ("") on a specific
*    operationBinding to override DataSource.beanClassName for that specific operation.
*  
* - For SQL joins that produce additional data fields, use a special, operation-specific
*    bean that represents a join of multiple entities and contains business logic specific
*    to that joined dataset
*  
*  
*  Note that beanClassName affects what numeric field types will be used for 
*  inbound DSRequest data. For fields with numeric types, the DSRequest.data 
*  in DSRequests will automatically be converted to the type of the target field, before the 
*  request is received in a DMI. For details, see dsRequestBeanTypes.
*  
*  Note that dmiOverview also has a built-in facility for populating a bean 
*  with the inbound DSRequest.data - just declare the bean as a method argument.
*  
*  For generic DataSources (DataSources with serverType "generic")
*  reify sets this property when it creates a generic 
*  DataSource using the Javabean Wizard. It has no built-in server-side effects.
*  
*  For Hibernate DataSources (DataSources with serverType "hibernate")
*  The name of the Java bean or POJO class that is mapped in Hibernate. This will typically 
*  be the fully-qualified class name - eg com.foo.MyClass - but it may be the 
*  simple class name - just MyClass - or it may be some other value. It all 
*  depends on how your classes have been mapped in Hibernate.
*  
*  The declared Java bean will affect how its properties will be mapped to built-in numeric 
*  types, see hibernateIntegration for details.
*  
*  Note: If you are intending to use Hibernate as a data-access layer only, 
*  you do not need to create Hibernate mappings or Java objects: SmartClient will generate
*  everything it needs on the fly.
*  
*  For JPA DataSources (DataSources with serverType "jpa" or "jpa1")
*  The fully qualified class name of the JPA annotated entity.
*  
*  NOTE for Hibernate and JPA users: When you use JPA, or use Hibernate as a full ORM
*  system (ie, not just allowing SmartClient Server to drive Hibernate as a data access layer),
*  the beans returned on the server-side are live. This means that if you make any 
*  changes to them, the ORM system will persist those changes. This is true even if the beans
*  were created as part of a fetch operation.
*  
*  This causes a problem in the common case where you want to use a DMI or custom DataSource 
*  implementation to apply some post-processing to the beans fetched from the persistent 
*  store. If you change the values in the beans directly, those changes will be persisted.
*  
*  If you want to alter the data returned from a JPA or Hibernate persistent store as part of
*  a fetch request just so you can alter what gets sent to the client, you can use the 
*  server-side DSResponse's getRecords() method. This will return
*  your bean data in "record" format - ie, as a List of Maps. You can alter these records 
*  without affecting your persistent store, and then call setData() on the 
*  DSResponse), passing the altered list of records. See the server-side Javadocs
*  for DSResponse for details of these two methods.
* @type {string}
* @default null
*/
beanClassName?: string;

/**
* Criteria that are implicitly applied to all fetches made through this DataSource. These
*  criteria are never shown to or edited by the user and are cumulative with any other 
*  criteria provided on the DSRequest.
*  
*  For example, a DataSource might *always* implicitly limit its fetch results to records 
*  owned by the current user's department. Components and ResultSets requesting data 
*  from the DataSource can then apply further implicitCriteria of their own, separately 
*  from their regular, user-editable criteria.
*  
*  For instance, a grid bound to this dataSource might be further limited to 
*  implicitly show only the subset of records created by the current user. See 
*  DataBoundComponent.implicitCriteria and ResultSet.implicitCriteria for 
*  more on these localized options.
*  
*  Note that, while implicitCriteria can be declared in a server DataSource 
*  file using componentXML, it is an entirely client-side 
*  feature, added by client-side components. So it does not affect server-side requests, 
*  and will not be added to client-side requests that don't come from a SmartClient UI 
*  (eg RestHandler).
* @type {Criteria}
* @default null
*/
implicitCriteria?: Criteria;

/**
* Sets what level of JSON validation will apply for this DataSource.
*  
*  Note that the point of "partial" validation mode is that if the JSON ihat's delivered is
*  correct, we'll still need to validate to get "date" and such in the correct time, but
*  shouldn't need to for the rest.
* @type {SkipJSONValidation}
* @default "none"
*/
skipJSONValidation?: SkipJSONValidation;

/**
* Namespaces definitions to add to the root element of outbound XML messages sent to a web
*  service, as a mapping from namespace prefix to namespace URI.
*  
*  The default value is:
*  
*   globalNamespaces : {
*    xsi: "http://www.w3.org/2001/XMLSchema-instance",
*    xsd: "http://www.w3.org/2001/XMLSchema"
*   },
*  
*  This default value allows the use of the xsi:type and xsi:nil attributes without further
*  declarations.
*  
*  Note that some web services will only accept specific revisions of the XML Schema URI.
*  If xsi-namespaced attributes seem to be ignored by an older webservice, try the URI
*  "http://www.w3.org/1999/XMLSchema-instance" instead.
* @type {object}
* @default "..."
*/
globalNamespaces?: object;

/**
* Very advanced: for servers that do not support paging, and must return large numbers of XML
*  records in one HTTP response, SmartClient breaks up the processing of the response
*  in order to avoid the "script running slowly" dialog appearing for an end user.
*  
*  If you have a relatively small number of records with a great deal of properties or
*  subobjects on each record, and you have not set DataSource.dropExtraFields to eliminate unused
*  data, and you see the "script running slowly" dialog, you may need to set this number
*  lower.
* @type {Integer}
* @default "150"
*/
resultBatchSize?: Integer;

/**
* The native field name used by this DataSource on the server to represent the
*  fileName for fileSource operations. Any
*  extensions to the fileName to indicate type or format (e.g. ".ds.xml") are stored in the
*  DataSource.fileTypeField and DataSource.fileFormatField, if specified
*  for this DataSource.
* 
*  If not specified for a DataSource, the fileNameField will be inferred
*  on the server as follows:
* 
*  
*  -->
*  
* - If there is a field named "fileName", "name", or "title",
*    then that field is used.
*  
* - Otherwise, if there is a single primary key, and it has the type "text",
*    then that field is used.
*  
* - Otherwise, an error is logged
*  
* @type {string}
* @default null
*/
fileNameField?: string;

/**
* The native field name used by this DataSource on the server to represent the
*  fileType for fileSource. 
* 
*  If the fileTypeField is not configured, then a field named "fileType"
*  will be used, if it exists. Otherwise, the DataSource will not track
*  fileTypes -- this may be acceptable if, for instance, you use a separate
*  DataSource for each fileType.
* 
*  The fileType is specified according to the extension that would have been used in the
*  filesystem -- for instance, the fileType for employees.ds.xml would be "ds".
* @type {string}
* @default null
*/
fileTypeField?: string;

/**
* The native field name used by this DataSource on the server to represent
*  the fileFormat for fileSource. 
* 
*  If the fileFormatField is not configured, then a field named
*  "fileFormat" will be used, if it exists. Otherwise, the DataSource will not
*  track fileFormats -- this may be acceptable if, for instance, the
*  fileFormat is always the same.
* 
*  The fileFormat is specified according to the extension that would have been used in the
*  filesystem -- for instance, the fileFormat for employees.ds.xml would be "xml".
* @type {string}
* @default null
*/
fileFormatField?: string;

/**
* The native field name used by this DataSource on the server to represent
*  the fileContents for fileSource.
* 
*  If the fileContentsField is not configured, then a field named
*  "fileContents" or "contents" will be used, if it exists. If not found, the longest text field which is not the
*  DataSource.fileNameField, DataSource.fileTypeField or
*  DataSource.fileFormatField will be used.
* 
*  Note that the only method which will actually return the fileContents
*  is DataSource.getFile -- the other fileSource
*  methods omit the fileContents for the sake of efficiency.
* @type {string}
* @default null
*/
fileContentsField?: string;

/**
* The native field name used by this DataSource on the server to represent
*  fileLastModified for fileSource.
* 
*  If the fileLastModifiedField is not configured, then a field named
*  "fileLastModified" will be used, if it exists. 
*  Otherwise, the server will look for a field with a "modifierTimestamp" type.
*  If that is not found, the DataSource will not keep track of the last
*  modified date.
* @type {string}
* @default null
*/
fileLastModifiedField?: string;

/**
* The native field name used by this DataSource on the server to represent
*  fileVersion for fileSource.
*  
*  Automatic file versioning is configured by the presence of this property: if you want
*  automatic versioning for a FileSource DataSource, it is sufficient to simply define a 
*  fileVersionField. When automatic versioning is on:
*  
* - Calls to DataSource.saveFile will save a new version of the file, retaining previous 
*  versions up to the maximum configured by DataSource.maxFileVersions; when that
*  maximum is reached, the oldest version is overwritten
*  
* - The DataSource.getFile API always returns the most recent version
*  
* - The DataSource.listFiles API only includes the most recent version of any file
*  
* - You can view and retrieve earlier versions of a file with the 
*  DataSource.listFileVersions and DataSource.getFileVersion APIs. Note that retrieving a
*  previous version of a file and then calling saveFile() goes through the
*  normal process of saving a new version
*  
*  
*  The fileVersion field is expected to be of type "datetime", and automatic 
*  versioning will not work otherwise. Note, to minimize the possibility of version 
*  timestamp collisions, we recommend that fileVersion fields specify 
*  DataSourceField.storeMilliseconds: true.
*  
*  If the fileVersionField is not configured, no automatic file versioning will be done.
* @type {string}
* @default null
*/
fileVersionField?: string;

/**
* If DataSource.fileVersionField is enabled for a FileSource
*  DataSource, this property configures the maximum number of versions to retain.
* @type {Integer}
* @default "20"
*/
maxFileVersions?: Integer;

/**
* For DataSources with type DSServerType, looks
*  up the locations to use as DataSource.projectFileLocations from the project's configuration (i.e.
*  project.properties, server_properties
*  etc.).
* 
*  For instance, to look up the value of project.datasources and use it
*  for projectFileLocations, use "datasources" as the 
*  projectFileKey.
* 
*  If you specify both projectFileKey and 
*  projectFileLocations, then both with be used, with the
*  projectFileLocations applied last.
* @type {string}
* @default null
*/
projectFileKey?: string;

/**
* For DataSources with type DSServerType,
*  specifies locations for the project files. In XML, each location is
*  expressed with a &lt;location&gt; tag, e.g.:
* 
*  
*    &lt;projectFileLocations&gt;
*      &lt;location&gt;[WEBROOT]/shared/ds&lt;/location&gt;
*      &lt;location&gt;ds://datasources&lt;/location&gt;
*    &lt;/projectFileLocations&gt;
*  
* 
*  Directories should be specified as absolute paths on the server. If you
*  want to construct a webroot-relative path, then prefix the path with
*  [WEBROOT] (unlike in
*  server_properties, where you would use
*  $webRoot as the prefix).
* 
*  To specify another DataSource to be used via 
*  fileSource, use ds://dsName
*  (where "dsName" is the name of the other DataSource).
* 
*  A projectFile DataSource uses the standard
*  fileSource field names: fileName,
*  fileType, fileFormat,
*  fileContents, fileSize and
*  fileLastModified. When defining a projectFile
*  DataSource, you can use DataSource.inheritsFrom with
*  a value of "ProjectFile" to inherit definitions for these fields -- e.g.:
* 
*  
*    &lt;DataSource ID="MyDataSources" type="projectFile" inheritsFrom="ProjectFile"&gt;
*      &lt;projectFileLocations&gt;
*        &lt;location&gt;[WEBROOT]/shared/ds&lt;/location&gt;
*        &lt;location&gt;ds://datasources&lt;/location&gt;
*      &lt;/projectFileLocations&gt;
*    &lt;/DataSource&gt; 
*  
* 
*  For directory locations, the
*  fileName is relative to the directory specified. Note that
*  the fileName does not include any extension for type or
*  format. For instance, for "employees.ds.xml", the fileName
*  would be "employees", the fileType would be "ds" and the
*  fileFormat would be "xml".
* 
*  A projectFile DataSource executes the various 
*  fileSource in the following manner.
*  The general rule is that fileName, fileType,
*  and fileFormat are treated as primary keys. If files with the
*  same combination of those attributes exist in more than one of the
*  configured locations, the locations are considered in reverse
*  order, with priority given to the location listed last. When modifying
*  an existing file, the last location which contains the file will be
*  used. When creating a new file, the file will be created in the last
*  configured location.
* 
*  
*  DataSource.listFiles 
*  Returns a combined list of files from
*    all configured locations. Note that listFiles does not 
*    recurse into subdirectories. If the same combination of 
*    fileName / fileType / fileFormat exists in more than
*    one configured location, then the data for fileSize and
*    fileLastModified will be taken from the last configured
*    location which contains the file.
* 
*  DataSource.hasFile
*  Indicates whether the file exists in any of the configured locations.
* 
*  DataSource.getFile
*  Returns file data by searching the locations in reverse order.
* 
*  DataSource.saveFile
*  If the file exists, it will be saved in the last location in which
*    it exists. If it is a new file, it will be saved in the last
*    configured location.
* 
*  DataSource.renameFile
*  The file will be renamed in the last location in which it exists.
*    Note that if the file exists in more than one location, the rename
*    will not affect other locations. Thus, a subsequent listFiles
*    operation will return the file from the other location (as well
*    as the renamed file).
* 
*  DataSource.removeFile
*  The file will be removed from the last location in which it exists.
*    Note that if the file exists in more than one location, the removal
*    will not affect other locations. Thus, a subsequent listFiles
*    operation will return the file from the other location.
* 
*  
* 
*  For convenience, a projectFile DataSource also responds to the standard
*  DataSource operations, in the following manner:
* 
*  
*  add
*  Executes a saveFile operation, either adding the file or updating
*    an existing file.
* 
*  fetch
*  Executes a listFiles operation. Note that the results will not
*    include the fileContents. In order to obtain the
*    fileContents, you must use a
*    DataSource.getFile.
* 
*  update
*  Executes a renameFile operation. Note that this will not update
*    the fileContents -- for that, you need to use "add", or
*    a DataSource.saveFile. 
* 
*  remove
*  Executes a removeFile operation.
* 
*  
* 
*  If you specify both DataSource.projectFileKey and 
*  projectFileLocations, then both with be used, with the
*  projectFileLocations applied last.
* @type {Array<Partial<string>>}
* @default null
*/
projectFileLocations?: Array<Partial<string>>;

/**
* Used by Reify to identify DataSources that are provided as samples. Reify will not
*  automatically alter sample DataSource relations when removing DataSources from a
*  project.
* @type {boolean}
* @default null
*/
isSampleDS?: boolean;

/**
* Before we start editing values in DataBoundComponents bound to this 
*  dataSource, should we perform a deep clone of the underlying values (a "deep clone" is 
*  one created by traversing the original values object recursively, cloning the contents 
*  of nested objects and arrays). If this flag is explicitly set to false, we perform a 
*  shallow clone of the underlying values before edit (a "shallow clone" is a copy created 
*  by simply copying the top-level attributes of an object). Note, this setting only 
*  affects deep-cloning of attributes that are bound to a field; for other, non-field 
*  values, see DataSource.deepCloneNonFieldValuesOnEdit.
*  
*  If this flag is not explicitly set, it defaults to the value of the same-named static 
*  property, DataSource.deepCloneOnEdit. This flag can also be overridden 
*  per-component and per-field - see DataBoundComponent.deepCloneOnEdit and 
*  DataSourceField.deepCloneOnEdit.
*  
*  Note, this flag only has an effect if you are editing a values object that contains nested 
*  objects or arrays, using Canvas.dataPaths. When editing "flat" data - the
*  mainstream case - there is no difference between a deep clone and a shallow clone.
* @type {boolean}
* @default null
*/
deepCloneOnEdit?: boolean;

/**
* When editing values in DataBoundComponents bound to this dataSource, should we
*  perform a deep clone of values that are not associated with a field (ie, attributes on
*  the record that do not map to a component field either directly by name, or by 
*  FormItem.dataPath). If this flag is not explicitly set, it defaults to
*  the value of the same-named static property, 
*  DataSource.deepCloneNonFieldValuesOnEdit. This flag can also be 
*  overridden per-component - see DataBoundComponent.deepCloneNonFieldValuesOnEdit.
*  
*  Note, a "deep clone" is one created by traversing the original values object 
*  recursively, cloning the contents of nested objects and arrays; a "shallow clone" is a
*  copy created by simply copying the top-level attributes of an object; if you have 
*  nested objects that are copied like this, the "copies" are actual references to the 
*  original objects.
*  
*  Like the other deepCloneOnEdit settings, this flag only has an effect if you are 
*  editing a values object that contains nested objects or arrays.
* @type {boolean}
* @default null
*/
deepCloneNonFieldValuesOnEdit?: boolean;

/**
* Default scriptlet to be executed on the server for each operation. If OperationBinding.script is
*  specified, it will be executed for the operation binding in question instead of running this scriptlet.
*  
*  Scriptlets are used similarly to DMIs configured via DataSource.serverObject or
*  OperationBinding.serverObject -
*  they can add business logic by modifying the DSRequest before it's executed, modifying the
*  default DSResponse, or taking other, unrelated actions. 
*  
*  For example:
*  
*   &lt;DataSource&gt;
*     &lt;script language="groovy"&gt;
*      ... Groovy code ...
*     &lt;/script&gt;
*    ... other DataSource properties
*   &lt;/DataSource&gt;
*  
*  
*  Scriptlets can be written in any language supported by the "JSR 223" standard, including Java
*  itself. See the dmiOverview for rules on how to return data,
*  add additional imports, and other settings.
*  
*  The following variables are available for DMI scriptlets:
*  
*  
* - requestContext: RequestContext (from com.isomorphic.servlet)
*  
* - dataSource: the current DataSource (same as DSRequest.getDataSource())
*  
* - dsRequest: the current DSRequest
*  
* - criteria: shortcut to DSRequest.getCriteria() (a Map)
*  
* - values: shortcut to DSRequest.getValues() (a Map)
*  
* - oldValues: shortcut to DSRequest.getOldValues() (a Map)
*  
* - sqlConnection: SQLDataSource only: the current SQLConnection object. If
*  using DataSource.autoJoinTransactions are enabled, this
*  SQLConnection is in the context of the current transaction.
*  
* - rpcManager: the current RPCManager
*  
* - applicationContext: the Spring ApplicationContext (when applicable)
*  
* - beanFactory: the Spring BeanFactory (when applicable)
*  
*  
*  Scriptlets also have access to a set of contextual variables related to the Servlets
*  API, as follows:
*  
*  
* - servletRequest: the current ServletRequest
*  
* - session: the current HttpSession
*  
* - servletResponse: the current ServletResponse (advanced use only)
*  
* - servletContext: the current ServletContext(advanced use only)
*  
*  As with DMI in general, be aware that if you write scriptlets that depend upon these variables,
*  you preclude your DataSource from being used in the widest possible variety of
*  circumstances. For example, adding a scriptlet that relies on the HttpSession
*  prevents your DataSource from being used in a command-line process.
*  
*  Note that if a dataSource configuration has both a &lt;script&gt; block and
*  a specified OperationBinding.serverObject for some operation, the
*  script block will be executed, and the serverObject ignored.
* @type {string}
* @default null
*/
script?: string;

/**
* Applicable to server-side REST DataSources only
*  
*  A scriptlet to be executed on the server after data has been fetched from the REST service,
*  but before it is processed through templating. 
*  The intention is that this scriptlet transforms the response data in some way, before that
*  transformed data is passed through templating and further downstream transformation steps 
*  such as record transformation. If your use case 
*  does not involve templating, there is no difference between putting your transformation 
*  logic in this script, or putting it in a transformResponseScript - they are merely
*  pre-templating and post-templating transformation opportunities, so if no templating is
*  involved, they are conceptually the same thing. Accordingly, exactly the same 
*  variables are available to the transformRawResponseScript as to the 
*  attr:DataSource.transformResponseScript - see that property's documentation for details.
*  
*  Note, if you prefer a Java solution rather than placing scripts in your .ds.xml 
*  files, you can instead extend the Java RESTDataSource class and override its 
*  transformRawResponse() method. If you both override the Java method and 
*  provide a transformRawResponseScript, the Java method runs first and any 
*  transformations it makes will be visible to the script. See 
*  DataSource.serverConstructor for details of how to use a custom 
*  class to implement a DataSource server-side.
* @type {string}
* @default null
*/
transformRawResponseScript?: string;

/**
* For Direct Method Invocation (DMI) binding, declares the ServerObject to use as the default
*  target for all DataSource.operationBindings. Specifying this
*  attribute in an XML DataSource stored on the server enables DMI for this DataSource.
*  
*  Note that if a dataSource configuration has both a OperationBinding.script block and
*  a specified serverObject for some operation, the
*  script block will be executed, and the serverObject ignored.
* @type {ServerObject}
* @default null
*/
serverObject?: ServerObject;

/**
* Optional array of OperationBindings, which provide instructions to the DataSource about how each
*  DSOperation is to be performed.
*  
*  When using the SmartClient Server, OperationBindings are specified in your DataSource
*  descriptor (.ds.xml file) and control server-side behavior such as what Java object to route
*  DSRequest to (OperationBinding.serverObject) or customizations to SQL, JQL and HQL queries
*  (OperationBinding.customSQL, OperationBinding.customJQL and OperationBinding.customHQL).
*  See the ${isc.DocUtils.linkForExampleId('javaDataIntegration', 'Java Integration samples')}.
*  
*  For DataSources bound to WSDL-described web services using
*  DataSource.serviceNamespace, OperationBindings are used to bind each DataSource
*  OperationBinding.operationType to an
*  OperationBinding.wsOperation of a WSDL-described 
*  WebService, so that a DataSource can both fetch and save data to a web
*  service.
*  
*  For example, this code accomplishes part of the binding to the 
*  SalesForce partner web services
*  
*  
*  isc.DataSource.create({
*   serviceNamespace : "urn:partner.soap.sforce.com",
*   operationBindings : [
*     { operationType:"fetch", wsOperation:"query", recordName: "sObject" },
*     { operationType:"update", wsOperation:"update", recordName: "SaveResult" },
*     { operationType:"add", wsOperation:"create", recordName: "SaveResult" },
*     { operationType:"remove", wsOperation:"delete", recordName: "DeleteResult" }
*   ],
*   ...
*  }); 
*  
*  
*  
*  NOTE: additional code is required to handle authentication and other details, see the
*  complete code in smartclientSDK/examples/databinding/SalesForce.
*  
*  For DataSources that contact non-WSDL-described XML or JSON services, OperationBindings can
*  be used to separately configure the URL, HTTP method, input and output processing for each
*  operationType. This makes it possible to fetch JSON data from one URL for the "fetch"
*  operationType and save to a web service for the "update" operationType, while appearing as a
*  single integrated DataSource to a DataBoundComponent such as an
*  ListGrid.canEdit.
*  
*  If no operationBinding is defined for a given DataSource operation, all of the properties
*  which are valid on the operationBinding are checked for on the DataSource itself. 
*  
*  This also means that for a read-only DataSource, that is, a DataSource only capable of fetch
*  operations, operationBindings need not be specified, and instead all operationBinding
*  properties can be set on the DataSource itself. In the ${isc.DocUtils.linkForExampleId('rssFeed', 'RSS Feed')}
*  sample, you can see an example of using OperationBinding properties directly on the 
*  DataSource in order to read an RSS feed.
* @type {Array<Partial<OperationBinding>>}
* @default null
*/
operationBindings?: Array<Partial<OperationBinding>>;

/**
* If a DSRequest arrives from the client that requests
*  serverSummaries, should it be allowed?
*  
*  Note this setting only affects dsRequests that come from the browser (or
*  another client). This setting has no effect on server summaries declared in .ds.xml files or
*  summaries configured in DSRequests created programmatically on the server side, which are
*  always allowed.
*  
*  Default value of null means this DataSource will use the system-wide default, which is set via
*  datasources.allowClientRequestedSummaries in
*  server_properties, and defaults to allowing client-requested
*  summaries.
*  
*  If client-requested summarization is allowed, but the server-side &lt;operationBinding&gt;
*  provides specific summarization settings, the client-requested summarization is ignored.
* @type {boolean}
* @default null
*/
allowClientRequestedSummaries?: boolean;

/**
* Provides a default value for OperationBinding.recordName.
* @type {string}
* @default null
*/
recordName?: string;

/**
* Controls when primary keys are required for "update" and "remove" server operations, when allowMultiUpdate 
*  has not been explicitly configured on either the OperationBinding.allowMultiUpdate or via the server-side API DSRequest.setAllowMultiUpdate().
*  
*  Default value of null means this DataSource will use the system-wide default, which is set via
*  datasources.defaultMultiUpdatePolicy in
*  server_properties, and defaults to not allowing multi updates for 
*  requests associated with an RPCManager, see MultiUpdatePolicy for details.
* @type {MultiUpdatePolicy}
* @default null
*/
defaultMultiUpdatePolicy?: MultiUpdatePolicy;

/**
* Indicates that declarativeSecurity should be enforced on the client. This
*  property only applies to clientOnlyDataSources. If
*  property is unset for a clientOnly DataSource it will be set true automatically.
*  Set this property to false to explicitly disable this feature for a 
*  clientOnly DataSource.
*  
*  Note that certain security features are not supported on the client like
*  Velocity rules.
* @type {boolean}
* @default null
*/
enforceSecurityOnClient?: boolean;

/**
* If the criteria applied to a fetch type operation contain fields that are not present
*  in the dataSource, should they be ignored when performing filtering on the client.
*  This property is useful for cases where you custom server logic makes use of criteria 
*  values to determine what set of records to return to the client, but the data
*  does not actually have record values for these fields and as such the client-side filtering
*  logic should ignore them.
* @type {boolean}
* @default true
*/
dropUnknownCriteria?: boolean;

/**
* If set to true, both client and server-side advanced filtering used by SmartClient will follow
*  SQL99 behavior for dealing with NULL values, which is often counter-intuitive to users.
*  Specifically, when a field has NULL value, all of the following expressions are false:
*  
*   field == "someValue" (normally false)
*   field != "someValue" (normally true)
*   not (field == "someValue")  (normally true)
*   not (field != "someValue")  (normally false)
*  
*  This property can be overridden per-query by specifying strictSQLFiltering
*  directly as a property on the AdvancedCriteria.
*  
*  NOTE: On the server side, this property is only applicable if you are using the 
*  SQL DataSource; the other built-in types (Hibernate and JPA/JPA2) do not offer this mode.
* @type {boolean}
* @default false
*/
strictSQLFiltering?: boolean;

/**
* By default, all DataSources are assumed to be capable of handling 
*  AdvancedCriteria on fetch or filter type operations. This property may be
*  set to false to indicate that this dataSource does not support 
*  advancedCriteria. See DataSource.supportsAdvancedCriteria for further information on this.
*  
*  NOTE: If you specify this property in a DataSource descriptor 
*  (.ds.xml file), it is enforced on the server. This means that if you run 
*  a request containing AdvancedCriteria against a DataSource that advertises itself as 
*  allowAdvancedCriteria:false, it will be rejected.
* @type {boolean}
* @default null
*/
allowAdvancedCriteria?: boolean;



/**
*  Call this method to switch cacheAllData on or off after initialization. Passing a 
*  shouldCache value of false clears any existing client-side cache,
*  cancels any outstanding requests for a full cache and issues any other pending requests
*  normally.
*  
* 
* @param {boolean} New value for DataSource.cacheAllData
*/
setCacheAllData(shouldCache:boolean): void;

/**
*  Call this method to set the data in the client-side cache after initialization.
* 
* @param {Array<Partial<Record>>} Array of records to apply as the client-side cache
*/
setCacheData(data:Array<Partial<Record>>): void;

/**
*  Call this method to set the data in the client-side test-data after initialization. 
* 
* @param {Array<Partial<Record>>} Array of records to apply as the client-side test-data
* @deprecated * In favor of DataSource.setCacheData.

*/
setTestData(data:Array<Partial<Record>>): void;

/**
*  Drop the current dataSource cache. This has two effects:
*  
*  
* - For DataSources DataSource.cacheAllData or DataSource.clientOnly, discard
*    the current client-side cache data.
*  
* - If notify is passed, cause all ResultSet associated with this
*    dataSource to drop their caches. This occurs
*    regardless of the dataSource type - and can be thought of as equivalent to 
*    processing a response with DSResponse.invalidateCache set.
*  
* 
* @param {boolean=} Should data objects associated with this dataSource have their  cache invalidated?
*/
invalidateCache(notify?:boolean): void;

/**
*  Switch into clientOnly mode, taking the cache from the cacheAllData ResultSet if it 
*  exists.
* 
*/
setClientOnly(): void;

/**
*  When DataSource.cacheAllData is true, has all the data been retrieved to the client?
* 
*/
hasAllData(): boolean;

/**
*  Takes all relative date values found anywhere within a Criteria / AdvancedCriteria object
*  and converts them to concrete date values, returning the new criteria object.
* 
* @param {Criteria} criteria to convert
* @param {string=} optional timezone offset. Defaults to the current timezone
* @param {Integer=} first day of the week (zero is Sunday). Defaults to                DateChooser.firstDayOfWeek
* @param {Date=} base value for relative conversion - defaults to now
*/
convertRelativeDates(criteria:Criteria, timezoneOffset?:string, firstDayOfWeek?:Integer, baseDate?:Date): Criteria;

/**
*  Notification method fired when a DataSource operation such as an 
*  DataSource.addData, DataSource.removeData or 
*  DataSource.updateData modifies the underlying data for a DataSource.
*  
*  This method is used by ResultSets to keep the user-visible data up to date as
*  changes are made.
*  
*  
*  Note: rather than overriding this method, we recommend using 
*  Class.observe to be notified when it is fired.
*  
*  
* 
* @param {DSResponse} response from the operation that modified the underlying  data set
* @param {DSRequest} request that initiated the data change
*/
dataChanged(dsResponse:DSResponse, dsRequest:DSRequest): void;

/**
*  Causes any components using this DataSource to be notified of changes that have been made to
*  the remote dataset accessed via this DataSource, as though the provided DSResponse had just
*  successfully completed. This will cause cache managers such as ResultSet or 
*  ResultTree to automatically update their caches, and components using such cache
*  managers to visually update to show modified data.
*  
*  This API should be used when you have found out about changes made by other users or by
*  automatic processes. For example, using the SmartClient Messaging system to receive
*  real-time updates via HTTP streaming, you may get updates that should affect a ListGrid
*  which is using a ResultSet to view a portion of a large dataset.
*  
*  See the concurrentEdits overview for more on handling concurrent edits in
*  SmartClient DataSources.
*  
*  The provided DSResponse should have
*  DSResponse.operationType "update", "add" or "remove" - there is no
*  way for a "fetch" response to meaningfully update arbitrary caches. However, if you
*  have a list of updated records (possibly retrieved via DataSource.fetchData) you can still call
*  updateCaches()with DSResponses of type "update". Typically DataSource operations
*  that manipulate data operate on a single record at a time, but if you explicitly set the
*  response.data attribute to an array of records, framework code will handle this
*  as it would multiple updates.
*  
*  Example usage: if you had a ListGrid bound to the supplyItem sample DataSource, 
*  and that ListGrid was showing a Record with itemId 23, and you wanted to update 
*  the unitCost field to a new value, you would use the following code:
*  
*   
*   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// updatedRecord is the record we want to update
*   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;var record = supplyItemDS.copyRecord(updatedRecord);
*   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;record.unitCost = 500;
*   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;var dsResponse = {
*   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;data: [record],
*   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;operationType: "update"
*   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;};
*   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;supplyItemDS.updateCaches(dsResponse);
*   
*  
*  
*  
*  To cause all components that have cache managers to drop their caches, provide a
*  DSResponse with DSResponse.invalidateCache set.
*  
*  As an alternative to calling updateCaches() directly, if updates to other
*  DataSources occur as a result of server-side logic, you can use the server-side API
*  DSResponse.addRelatedUpdate(DSResponse) (Pro Edition and above), which ultimately calls
*  updateCaches() for you - see that method's documentation for details.
*  
*  NOTE:: if updateCaches is called for a
*  DataSource.clientOnly DataSource, it will update
*  DataSource.cacheData synchronously in addition to notifying all cache managers as
*  normal.
*  
*  If a DataSource has DataSource.cacheAllData set and a full cache has been obtained, calling
*  updateCaches will automatically update the cache.
*  
*  Note that the DSResponse provided to this method will not go through
*  DataSource.transformResponse or other processing that would normally occur for a
*  DSResponse resulting from a DSRequest sent by the application in this page.
* 
* 
* @param {DSResponse} the provided DSResponse must minimally have                 dataSource, operationType, and data set
* @param {DSRequest=} optional dsRequest. If not specified, a DSRequest will be                automatically created based on the DataSource and operationType of                 the DSResponse
*/
updateCaches(dsResponse:DSResponse, dsRequest?:DSRequest): void;

/**
*  For a DataSource that describes a DOM structure, the list of legal child elements that can
*  be contained by the element described by this DataSource.
*  
*  For a DataSource described by XML schema, this is the list of legal subelements of
*  complexType (elements of simpleType become DataSourceFields with atomic type).
*  
*  Note that currently, if an XML schema file contains ordering constraints, DataSources
*  derived from XML Schema do not capture these constraints.
* 
* 
*/
getLegalChildTags(): void;

/**
*  Copies all DataSource field values of a Record (including a TreeNode) to a new 
*  Record, omitting component-specific metadata such as selected state from grids, 
*  or parent folders for TreeNodes.
* 
* @param {Record} The record to be copied.
*/
copyRecord(record:Record): Record;

/**
*  Copies all DataSource field values of an (Array) of Records (including a TreeNode) 
*  to a new array of Records, omitting component-specific metadata such as selected state 
*  from grids, or parent folders for TreeNodes. This method calls 
*  DataSource.copyRecord for each item in the array.
* 
* @param {Array<Partial<Record>>} The array of Record objects to be copied.
*/
copyRecords(records:Array<Partial<Record>>): Array<Partial<Record>>;

/**
*  Creates a shallow copy of the given DSRequest. The request's DSRequest.data,
*  if any, is shallow copied in the cloned request.
*  
*  The DSRequest.callback property of the given request is not copied into
*  the cloned request.
* 
* @param {DSRequest} the DSRequest to clone.
*/
cloneDSRequest(dsRequest:DSRequest): DSRequest;

/**
*  Creates a shallow copy of the given DSResponse. All properties that would affect
*  the processing of the response are copied into the resulting DSResponse so that the cloned
*  response could substitute for the original response. The response's DSResponse.data,
*  if any, is shallow copied in the cloned response.
* 
* @param {DSResponse} the DSResponse to clone.
*/
cloneDSResponse(dsResponse:DSResponse): DSResponse;

/**
*  For a dataSource using clientDataIntegration,
*  return the data that should be sent to the DataSource.dataURL.
*  
*  By default, HTTP requests sent to non-SmartClient servers do not include DSRequest
*  metadata such as DSRequest.startRow, DSRequest.endRow,
*  and DSRequest.oldValues. Only the core
*  dataSourceOperations is sent, such as the criteria
*  passed to ListGrid.fetchData or the updated values submitted by
*  DynamicForm.saveData. 
*  
*  transformRequest() allows you to transform dsRequest metadata into a
*  format understood by your server and include it in the HTTP request, so that you can
*  integrate DataSource features such as data paging with servers that support such
*  features.
*  
*  How the data is actually sent to the URL is controlled by 
*  OperationBinding.dataProtocol. If using the "getParams" or
*  "postParams" protocol, data is expected to be a JavaScript Object where each property
*  will become a GET or POST'd parameter. If using dataProtocol:"soap" or "postXML", data
*  will be serialized as an XML message by DataSource.xmlSerialize.
*  
*  As an example, if you have a dataURL that can return paged data given URL parameters
*  "start" and "end", you could implement transformRequest like so:
*  
*  
*   isc.DataSource.create({
*    ... 
*    transformRequest : function (dsRequest) {
*      if (dsRequest.operationType == "fetch") {
*        var params = {
*         start : dsRequest.startRow,
*         end : dsRequest.endRow
*        };
*        // combine paging parameters with criteria
*        return isc.addProperties({}, dsRequest.data, params);
*      }
*    }
*   });
*  
*  
*  
*  Other reasons to implement transformRequest():
*  
*  
* - transform a Criteria object into the custom query language of a web
*    service
*  
* - add a session id to requests that require authentication
*  
* - detect colliding updates by sending both updated values and the values the user
*    originally retrieved before editing began (available as DSRequest.oldValues)
*  
*  Special case: If the dataProtocol is "clientCustom"
*  the SmartClient system will not attempt to send data to the server in any way. Instead 
*  transformRequest should be implemented such that it accesses or updates the underlying
*  data-set and calls DataSource.processResponse when the operation is complete. This
*  setting allows straightforward integration with non SmartClient comm mechanisms that 
*  directly send requests to the server (such as GWT-RPC), or handle data manipulation without
*  sending HTTP at all (such as Google Gears).
*  Developers should not generate a response for a request and call 
*  processResponse() synchronously from within transformRequest
*  as this can lead to unpredictable results. 
*  If you're integrating with a synchronous data provider or able to process requests
*  synchronously from data in browser memory, we'd recommend using a
*  DataSource.clientOnly with a custom
*  DataSource.getClientOnlyResponse implementation instead.
*  
*  A transformRequest override may also be used to set the DSRequest.dataProtocol
*  to clientCustom at runtime, giving developers a way to intercept normal handling for
*  some particular request, and provide entirely custom handling written on the client.
*  
*  Note: The RestDataSource class overrides transformRequest() to handle xml-serializing
*  the request (including meta data) into a standard format.
* 
* 
* @param {DSRequest} the DSRequest being processed
*/
transformRequest(dsRequest:DSRequest): any;

/**
*  Helper method to retrieve the updated data from a successful dataSource update or add 
*  operation.
* 
* @param {DSRequest} Update request object passed to the server. Note that this request may have been modified by DataSource.transformRequest
* @param {DSResponse} Response returned by the server
* @param {boolean} If DSResponse.data is empty, should data be derived from the submitted request.
*/
getUpdatedData(dsRequest:DSRequest, dsResponse:DSResponse, useDataFromRequest:boolean): any | Array<Partial<any>>;

/**
*  Returns the appropriate OperationBinding.dataProtocol for a DSRequest
* 
* @param {DSRequest} DataSource Request object
*/
getDataProtocol(dsRequest:DSRequest): DSProtocol;

/**
*  Process a dsResponse for a request initiated by a DataSource with
*  OperationBinding.dataProtocol.
*  requestId parameter should be dsRequest.requestId as found on the dsRequest
*  passed to DataSource.transformRequest.
*  
*  You must provide a response for both error and non-error cases. For an error case, a
*  sufficient response is:
*  
*  { status : -1 }
*  
* 
* @param {string} requestId attribute from the associated dataSource request object
* @param {Partial<DSResponse>} Configuration for the dsResponse
*/
processResponse(requestId:string, dsResponse:Partial<DSResponse>): void;

/**
*  Get the XML to be posted to the dataURL based on the passed DSRequest.
*  
*  This API is intended to be overridden in order to integrate with web services that
*  expect XML messages rather than simple HTTP parameters, but lack a
*  WSDL description. For WSDL-described web services, having loaded the service
*  description, SmartClient knows the correct XML message structure, so customization is
*  best done by modifying the JavaScript data that is used to form the message.
* 
* 
* @param {DSRequest} the request to encode as an XML message.
*/
getXMLRequestBody(dsRequest:DSRequest): string;

/**
*  Serialize a JavaScript object as XML.
*  
*  The JavaScript Object passed to DataSource.xmlSerialize becomes an XML
*  element named after the DataSource.tagName (or DataSource.ID if
*  tagName is unset). Each property of the object becomes a subElement. For example,
*  using a DataSource to serialize like this:
*  
*    var inputObject = {
*     startRow : 5,
*     endRow : 50,
*     data : [
*       { field1 : "value1", field2: new Date() },
*       { field1 : "value3", field2: null }
*     ]
*    };
*    var myDS = isc.DataSource.create({ tagName:"DSRequest" });
*    myDS.xmlSerialize(inputObject);
*  
*  .. produces the following XML:
*  
*    &lt;DSRequest&gt;
*      &lt;startRow&gt;5&lt;/startRow&gt;
*      &lt;endRow&gt;50&lt;/endRow&gt;
*      &lt;data&gt;
*        &lt;field1&gt;value1&lt;/field1&gt;
*        &lt;field2&gt;2005-10-14T18:01:16&lt;/field2&gt;
*      &lt;/data&gt;
*      &lt;data&gt;
*        &lt;field1&gt;value3&lt;/field1&gt;
*        &lt;field2&gt;&lt;/field2&gt;
*      &lt;/data&gt;
*    &lt;/DSRequest&gt;
*  
*  
*  Various properties on the DataSource and DataSourceField can affect how serialization is
*  performed, see for example DataSource.tagName,
*  DataSource.schemaNamespace, 
*  DataSourceField.xmlAttribute, 
*  DataSourceField.multiple and DataSourceField.childTagName.
*  By setting the DataSourceField.type to the ID of another
*  DataSource which has further XML serialization settings, you can control serialization of
*  nested structures.
*  
*  If you are working with a WSDL-described web service, XML serialization is performed
*  automatically by APIs like WebService.callOperation - you only need to
*  know about serialization in order to understand how to put together JavaScript data that
*  will fill in an XML message properly, and for simple messages, setting
*  DSRequest.useFlatFields makes that unnecessary as well.  
*  
*  Note: when trying to send data to a web service, it is best to avoid putting
*  together any XML yourself, instead modify the JavaScript data being fed to SmartClient's
*  SOAP engine. This is because the WSDL and SOAP rules for correctly namespacing and
*  encoding Web Service messages are very complex and are subject to change with new
*  versions of the web service you are contacting, whereas the data itself is easy to
*  manipulate and less likely to change.
*  
*  To troubleshoot message formation, you can set the log category "xmlSerialize" to 
*  INFO or DEBUG level in order to see diagnostics about XML
*  message formation, and you can use the RPC tab in the Developer Console to see the
*  actual messages being passed to web services.
* 
* 
* @param {any} data to be serialized
* @param {SerializationContext} options for the serialization engine
*/
xmlSerialize(data:any, flags:SerializationContext): string;

/**
*  Transform a list of XML elements to DataSource records.
*  
*  recordsFromXML() will create a List of DataSource records in the form of
*  JavaScript objects. The value for each field is extracted from the XML according to the
*  rules described under DataSourceField.valueXPath. 
*  
*  Derived JavaScript values for each field will be the appropriate JavaScript type, eg,
*  for a field of "date" type, the JS value will be a JavaScript Date object.
*  
*  Note that if the set of nodes exceeds DataSource.resultBatchSize, this method
*  will break the XML processing logic into multiple threads and therefore complete
*  asynchronously. In this case the return value will be null, and the callback parameter
*  must be used to process the extracted records.
*  
* 
* @param {Array<Partial<XMLElement>>} XML elements to transform, eg, the result of a                    call to XMLTools.selectNodes
* @param {Callback} Callback to fire when the transform completes. Takes a single  parameter records - the array of DataSource records derived from the   XML elements.
*/
recordsFromXML(elements:Array<Partial<XMLElement>>, callback:Callback): List;

/**
*  Modify the DSResponse object derived from the response returned from the 
*  DataSource.dataURL.
*  
*  This is an override point that makes it possible to use DataSource features such as
*  paging with web services that support such features, by allowing you to fill in metadata
*  fields in the DSResponse object (such as DSResponse.startRow) based on
*  service-specific metadata fields contained in the service's response.
*  
*  The DSResponse passed to this method already has DSResponse.data, which is
*  derived differently depending on the DataSource.dataFormat setting:
*  
*  
* - dataFormat:"xml" : either the
*  OperationBinding.recordXPath or 
*  OperationBinding.recordName is used to select the XML elements
*  that represent DataSource records. The selected XML elements are passed to
*  DataSource.recordsFromXML, which transforms the XML elements to typed
*  JavaScript data using the DataSource as a schema.
*  
* - dataFormat:"json" : the
*  OperationBinding.recordXPath, if specified, is used to select
*  records from the returned JSON data via XMLTools.selectObjects. 
*  DataSourceField.valueXPath is used to derive correctly typed field values.
*  
* - dataFormat:"custom" : dsResponse.data is the raw response
*  in String form. It must be parsed into an Array of Objects for subsequent processing to
*  work.
*  
*  
*  In addition to dsResponse.data, DSResponse.status is defaulted
*  to 0 (indicating no error), and DSResponse.startRow is assumed to be zero,
*  with DSResponse.endRow and DSResponse.totalRows
*  both set to dsResponse.data.length - 1, that is, the returned data is
*  assumed to be all records that matched the filter criteria.
*  
*  Examples of using this API include:
*  
*  
* - setting DSResponse.startRow, 
*    DSResponse.endRow and DSResponse.totalRows
*    to allow paging for a service that supports it. For example, if an XML service
*    returns a "resultRow" tag that contained the row number of the first row of the
*    returned results:
*   
*    dsResponse.startRow = isc.XMLTools.selectNumber(xmlData, "//resultRow");
*   
*   
*  
* - setting DSResponse.status to recognized ISC error values based on
*    service-specific errors, in order to trigger standard ISC error handling. For
*    example, setting dsResponse.status to
*    RPCResponse.STATUS_VALIDATION_ERROR and filling in
*    DSResponse.errors in order to cause validation errors to be shown in
*    forms and grids.
*  
* - for services that either do not return cache update data, or return partial data,
*    using DSRequest.oldValues to create cache update data (whether this is
*    appropriate is application-specific), or setting
*    DSResponse.invalidateCache.
*  
*  
*  NOTE: this method is NOT an appropriate time to call
*  methods on visual components such as grids, initiate new DSRequests or RPCRequests, or
*  in general do anything other than fill in fields on the DSResponse based on data that is
*  already available. Any actions that need to be taken as a result of the web
*  service response should be implemented exactly as for a DataSource where
*  transformResponse() has not been overridden, that is, use the callback
*  passed to high-level methods such as 
*  dataBoundComponentMethods, and do error
*  handling via either DataSource.handleError or by setting
*  RPCRequest.willHandleError.
* 
* 
* @param {DSResponse} default DSResponse derived from the response data
* @param {DSRequest} DSRequest object that initiated this request
* @param {XMLDocument | JSON} XML document or JSON objects returned by the web                  service
*/
transformResponse(dsResponse:DSResponse, dsRequest:DSRequest, data:XMLDocument | JSON): DSResponse;

/**
*  Fetch a single record from the DataSource by DataSourceField.primaryKey.
*  This simply calls DataSource.fetchData after creating Criteria that contain the primary
*  key field and value.
*  
*  If you call this method on a DataSource with a composite primary key - ie, one with 
*  multiple primaryKey fields - this method returns the first record where the first 
*  defined primary field matches the supplied pkValue; this may or may not be meaningful, 
*  depending on your use case. Generally, for DataSources with composite keys, it makes
*  more sense to use fetchData() directly, rather than this convenience 
*  method.
* 
* 
* @param {any} value for the field marked           DataSourceField.primaryKey:true in this DataSource            (or the first field so marked if there is more than one)
* @param {DSCallback=} callback to invoke on completion
* @param {Partial<DSRequest>=} additional properties to set on the                          DSRequest that will be issued
*/
fetchRecord(pkValue:any, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Perform a "fetch" DataSource operation against this DataSource, sending search criteria
*  and retrieving matching records.
*  
*  NOTE: do not attempt to override this method to create a custom DataSource. For
*  a server-side custom DataSource, use the DataSource.serverConstructor attribute,
*  and the ${isc.DocUtils.linkForExampleId('customDataSource', 'Custom DataSource samples')}. For a
*  client-side custom DataSource, see DataSource.dataProtocol.
*  
*  In contrast to ListGrid.fetchData, which creates a ResultSet to manage
*  the returned data, calling dataSource.fetchData() provides the returned
*  data in the callback as a 
*  simple JavaScript Array of JavaScript Objects.
*   Calling
*  dataSource.fetchData() does not automatically update any visual components or
*  caches: code in the callback passed to fetchData() decides what to do with
*  the returned data. 
*  
*  For example, given a ListGrid "myGrid" and a DataSource "employees", the following code
*  would populate "myGrid" with data fetched from the DataSource:
*  
*   isc.DataSource.get("employees").fetchData(null, "myGrid.setData(data)");
*   
*  
*  Unlike calling myGrid.fetchData(), which creates a ResultSet, the
*  data provided to the grid is "disconnected" data, unmanaged by SmartClient's databinding
*  facilities and safe to directly modify. This is useful when, for example, a ListGrid is
*  being used as a more sophisticated version of HTML's multi-select component.
*  
*  Disconnected datasets may be used to populate various visual components. For example,
*  while an individual FormItem can be configured to fetch
*  FormItem.valueMap options from a DataSource via the
*  FormItem.optionDataSource property, the following code shows
*  storing a dataset to derive valueMaps from later:
*  
*   
*   isc.DataSource.get("countries").fetchData(null, "window.countries = data");
* 
*   ... later, a form is created dynamically ...
* 
*   function showForm() {
*     isc.DynamicForm.create({
*       items : [
*        { name:"country", title:"Pick Country",
*         valueMap: window.countries.getValueMap("countryId", "countryName")
*        },
*     ...
*   
*   
*  
*  
*  You can also create a ResultSet from the data retrieved from fetchData(),
*  like so:
*  
*   
*   isc.DataSource.get("countries").fetchData(null,
*     function (dsResponse, data) {
*       isc.ResultSet.create({
*         dataSource:"countries",
*         allRows:data
*       })
*     }
*   )
*   
*   
*  
*  
*  This gives you a dataset that supports client-side filtering (via
*  ResultSet.setCriteria), can provide 
*  ResultSet.getValueMap, will 
*  ResultSet.disableCacheSync to the DataSource made via
*  other components, and can be re-used with multiple visual components.
*  
*  See also DataSource.getClientOnlyDataSource and DataSource.cacheAllData for
*  similar capabilities for dealing with smaller datasets entirely within the browser, or working
*  with modifiable caches representing subsets of the data available from a DataSource.
*  
*  See also the server-side com.isomorphic.js.JSTranslater class in the
*  ${isc.DocUtils.linkForDocNode('JavaServerReference', 'Java Server Reference')}
*  for other, similar approaches
*  involving dumping data into the page during initial page load. Note: care should
*  be taken when using this approach. Large datasets degrade the basic performance of some
*  browsers, so use PickList.optionDataSource and similar
*  facilities to manage datasets that may become very large.
*  
*  Data-Driven Visual Component Creation
*  
*  
*  DataSource.fetchData() can also be used to create SmartClient components in
*  a data-driven way. Many properties on SmartClient visual components are configured via
*  an Array of Objects - the same data format that dataSource.fetchData()
*  returns. These include ListGrid.fields, TabSet.tabs,
*  DynamicForm.items, Facet.values and even DataSource.fields.
*  
*  For example, if you had a DataSource "myFormFields" whose fields included the
*  basic properties of FormItem (name, title, type, etc), this example
*  code would create a form based on stored field definitions, loaded from the
*  "myFormFields" DataSource on the fly:
*  
*   isc.DataSource.get("myFormFields").fetchData(null, 
*     "isc.DynamicForm.create({ items:data })"
*   )
*  
*  
*  
*  This capability to dynamically create visual components from dynamically fetched data
*  provides a foundation for creating interfaces that can be customized by end users.
*  See also the server-side API com.isomorphic.datasource.DataSource.addDynamicDSGenerator() for
*  dynamically creating DataSources supporting all server-side DataSource features, and
*  DataSource.inheritsFrom for sharing field definitions across multiple DataSources.
* 
* 
* @param {Criteria=} search criteria
* @param {DSCallback=} callback to invoke on completion
* @param {Partial<DSRequest>=} additional properties to set on                            the DSRequest that will be issued
*/
fetchData(criteria?:Criteria, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Perform a "fetch" DataSource operation against this DataSource, sending search criteria
*  and retrieving matching records.
*  
*  This is identical to DataSource.fetchData except that
*  DSRequest.textMatchStyle is set to "substring" to cause case insensitive
*  substring matching (if the server respects this setting).
* 
* 
* @param {Criteria=} search criteria
* @param {DSCallback=} callback to invoke on completion
* @param {Partial<DSRequest>=} additional properties to set on                            the DSRequest that will be issued
*/
filterData(criteria?:Criteria, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Exports arbitrary client-side data, with client-side formatters applied, so is suitable 
*  for direct display to users. This method can be used to export data formatted outside
*  of any kind of visual component.
*  
*  If you do not specify an OperationBinding.operationId in the 
*  requestProperties you pass to this method, it behaves exactly the same as
*  the DataSource.exportClientData of the same name.
*  If you do specify an operationId, the framework expects your DataSource to
*  configure an OperationBinding of OperationBinding.operationType
*  "clientExport" with the same operationId. The framework will then send the
*  exportClientData request via the ordinary DSRequest mechanism,
*  which allows you to use normal framework features in the client data export. 
*  
*  For example, you could add a dmiOverview to your
*  operationBinding, which would allow you to write server-side code that
*  intervenes in the export process - for instance, by calling the
*  getExportObject() API to do something special with the export document,
*  like saving it to a database table or sending it to an email list.
*  
*  When you use the specific operationId version of this API, both the 
*  SmartClient Server and server-side DataSources are required.
*  
*  To export unformatted data, see DataSource.exportData which does
*  not include client-side formatters, but requires both the SmartClient server and the 
*  presence of server-side DataSources.
*  
*  Note that field DataSourceField.displayFormat is honored for 
*  "date" and "datetime" fields when exporting direct to Excel; see the displayFormat
*  docs for details.
* 
* 
* @param {Array<Partial<Record>>} Array of Records to export
* @param {Partial<DSRequest>=} Request properties for the export
* @param {DSCallback=} Optional callback. Note that this is only applicable if you also specify DSRequest.exportToClient: false in the  request properties
*/
exportClientData(data:Array<Partial<Record>>, requestProperties?:Partial<DSRequest>, callback?:DSCallback): void;

/**
*  Perform a "fetch" DataSource operation against this DataSource, sending search criteria,
*  retrieving matching records and exporting the results. See 
*  OperationBinding.exportResults or DSRequest.exportResults and for more
*  information.
* 
* 
* @param {Criteria=} search criteria
* @param {Partial<DSRequest>=} additional properties to set on                            the DSRequest that will be issued
* @param {DSCallback=} callback to invoke on completion. Note that this parameter                 only applies where DSRequest.exportToClient is                  explicitly set to false, because file downloads do not                  provide ordinary SmartClient callbacks
*/
exportData(criteria?:Criteria, requestProperties?:Partial<DSRequest>, callback?:DSCallback): void;

/**
*  Produces a clientOnly "copy" of a particular subset of data from a normal 
*  DataSource, via calling fetchData() to fetch matching rows, and constructing 
*  a clientOnly DataSource that DataSource.inheritsFrom the original DataSource.
*  
*  This clientOnly "copy" can be useful in situations where you want to allow 
*  a series of local changes without immediately committing to the server. 
*  See also ListGrid.autoSaveEdits for more fine-grained tracking of 
*  edits (eg, special styling for uncommitted changes).
*  
*  The new DataSource is returned via the "callback" argument. If DataSource.cacheAllData is
*  enabled and DataSource.hasAllData returns true, the new DataSource is synchronously
*  returned as the result of the method. In this case, if a callback was passed, it also
*  is executed synchronously.
* 
* 
* @param {Criteria} The criteria for the clientOnly DS
* @param {ClientOnlyDataSourceCallback} The callback to fire passing the clientOnly DS
* @param {Partial<DSRequest>=} optional properties to pass through to the DSRequest
* @param {Partial<DataSource>=} optional properties to pass                            through to the clientOnly DS
*/
getClientOnlyDataSource(criteria:Criteria, callback:(clientOnly:DataSource) => void, requestProperties?:Partial<DSRequest>, dataSourceProperties?:Partial<DataSource>): void;

/**
*  Perform an "add" DataSource operation against this DataSource, to create a new DataSource
*  record.
*  
*  NOTE: do not use repeated calls to addData() to provide the initial
*  dataset for a DataSource.clientOnly DataSource, instead, provide
*  initial data via DataSource.cacheData. Using addData() for subsequent,
*  incremental updates from sources like user editing is fine.
* 
* 
* @param {Record} new record
* @param {DSCallback=} callback to invoke on completion
* @param {Partial<DSRequest>=} additional properties to set on                            the DSRequest that will be issued
*/
addData(newRecord:Record, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Perform an "update" DataSource operation against this DataSource, to update values in an
*  existing DataSource record.
* 
* 
* @param {Partial<Record>} updated record
* @param {DSCallback=} callback to invoke on completion
* @param {Partial<DSRequest>=} additional properties to set on                            the DSRequest that will be issued
*/
updateData(updatedRecord:Partial<Record>, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Perform a "remove" DataSource operation against this DataSource, to delete an existing
*  DataSource record.
* 
* 
* @param {Record | any} primary key values of record to delete, (or complete record)
* @param {DSCallback=} callback to invoke on completion
* @param {Partial<DSRequest>=} additional properties to set on                            the DSRequest that will be issued
*/
removeData(data:Record | any, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Contacts the server to run server-side validation on a DSRequest and either returns
*  DSResponse.errors validation errors or a DSResponse.status code of 0.
*  
*  A "validate" dsRequest is effectively always RPCRequest.willHandleError:true.
*  It is a normal condition for a "validate" DSResponse to have validation errors and
*  the response will never go to system-wide handling for unexpected errors
*  (RPCManager.handleError).
* 
* 
* @param {Record} record values to validate
* @param {DSCallback=} callback to invoke on completion
* @param {Partial<DSRequest>=} additional properties to set on                          the DSRequest that will be issued
*/
validateData(values:Record, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Invoke an operation declared with OperationBinding.operationType "custom".
*  
*  This is a rarely used API. If the operation you are performing can be thought of
*  as one of the standard "CRUD" DSOperationType, declare it
*  with a CRUD operationType. For example, if your operation updates a record, declare it
*  with operationType "update" and invoke it via DataSource.updateData - this will cause
*  ResultSet to work correctly.
*  
*  In particular:
*  
*  
* - do not use this API just because you need to add additional server-side logic to a
*  CRUD operation (DMI allows this)
*  
* - do not use this API to implement variants of core CRUD operations
*  (DSRequest.operationId is the correct way to do this)
*  
* - do not use this API just because an operation affects more than one record. Most
*  kinds of multi-record operations should use RPCManager.startQueue. 
*  However, a custom operation is appropriate for genuine "batch" updates, as 
*  opposed to just a number of ordinary updates by primaryKey - see 
*  OperationBinding.allowMultiUpdate
*  
* - do not use this API just because you are calling a stored procedure in SQL - if the
*  stored procedure performs some kind of CRUD operation on the records of this DataSource,
*  use a standard CRUD operationType
*  
*  
*  Instead, the specific purpose of this API is to bypass all checks and side effects that
*  normally occur for CRUD operations, for example, that a "fetch" requires valid Criteria
*  or that an "update" or "remove" operation contains a valid primary key, or that an "add"
*  operation returns the newly added record. performCustomOperation allows
*  you to pass an arbitrary Record to the server, act on it with custom code, and return
*  arbitray results or even no results.
*  
*  The "data" parameter becomes DSRequest.data. With the SmartClient
*  Server Framework, the data is accessible server-side via DSRequest.getValues() and in
*  velocitySupport (such as &lt;customSQL&gt;) as $values.
*  
*  Note that with SQLDataSource, performCustomOperation must be used if you
*  plan to have a &lt;customSQL&gt; tag in your operationBinding that will execute SQL
*  operations other than SELECT, UPDATE, INSERT, DELETE (such as creating a new table).
*  By declaring OperationBinding.operationType "custom" in your .ds.xml file, all
*  checks related to normal CRUD operations will be skipped and your &lt;customSQL&gt;
*  can do arbitrary things.
* 
* 
* @param {string} the operation ID
* @param {Partial<Record>=} data to pass to the server.
* @param {DSCallback=} callback to invoke on completion
* @param {Partial<DSRequest>=} additional properties to set on                            the DSRequest that will be issued
*/
performCustomOperation(operationId:string, data?:Partial<Record>, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Download a file stored in a field of type:"binary" in a DataSource record.
*  
*  This will trigger the browser's "Save As" dialog and allow the user to save the file
*  associated with some record.
*  
*  Note that if this method is called for a record with no associated file, the
*  download URL may not be functional. By default when dataSources encounter a 
*  FieldType, an additional field,
*  &lt;fieldName&gt;_filename, is generated to
*  store the filename for the binary field value. If this field is present in the
*  data source but has no value for this record, developers can assume they're working
*  with a record with no stored file. If this field is not present in some custom
*  dataSource configuration, or the record is not loaded on the client, an additional
*  server transaction may be required to determine whether the record has an associated
*  file before calling this method to download a file.
*  
*  See the overview of binaryFields for more details.
* 
* 
* @param {Record} Record to download. Only required to have a value for the            primary key field.
* @param {FieldName=} Optional name of the binary field containing the file.                  If not provided, the first binary field is used.
* @param {Partial<DSRequest>=} Additional properties to set on the                           DSRequest that will be issued.
*/
downloadFile(data:Record, fieldName?:FieldName, requestProperties?:Partial<DSRequest>): void;

/**
*  Display a file stored in a field of type:"binary" in a new browser window.
*  
*  This will open a new browser window to show the file. Depending on the file type,
*  the user's installed plugins and applications, and the user's browser settings, this may
*  cause the file to be actually displayed in the new browser window, or may prompt the
*  user to either launch an external application to view the file or save the file to disk.
*  
*  Note that if this method is called for a record with no associated file, the
*  target window's new URL may not be functional. By default when dataSources encounter a 
*  FieldType, an additional field,
*  &lt;fieldName&gt;_filename, is generated to
*  store the filename for the binary field value. If this field is present in the
*  data source but has no value for this record, developers can assume they're working
*  with a record with no stored file. If this field is not present in some custom
*  dataSource configuration, or the record is not loaded on the client, an additional
*  server transaction may be required to determine whether the record has an associated
*  file before calling this method to view a file.
*  
*  See the overview of binaryFields for details.
* 
* 
* @param {Record} Record to download. Only required to have a value for the            primary key field.
* @param {FieldName=} Optional name of the binary field containing the file.                  If not provided, the first binary field is used.
* @param {Partial<DSRequest>=} Additional properties to set on the                           DSRequest that will be issued.
*/
viewFile(data:Record, fieldName?:FieldName, requestProperties?:Partial<DSRequest>): void;

/**
*  Returns a direct URL to access a file stored in a field of type:"binary".
*  
*  This URL can be used as the "src" attribute of an Img widget or &lt;img&gt; tag (if the file is
*  an image), or can be used in an ordinary HTML link (&lt;a&gt; tag) to download the file.
*  However, for the latter use case, see also DataSource.downloadFile and DataSource.viewFile.
*  
*  The URL returned is not to a static file on disk, rather, the returned URL essentially
*  encodes a DSRequest as URL parameters, in a format understood by the IDACall servlet
*  that comes with the Server Framework. 
*  
*  Hence, this URL will dynamically retrieve whatever file is currently stored in the
*  binary field via executing a normal DSRequest server side. The request will run through
*  normal security checks, so if your application requires authentication, the user must
*  have a valid session and be authorized to access the binary field.
*  
*  Note that if this method is called for a record with no associated file, the
*  returned URL may not be functional. By default when dataSources encounter a 
*  FieldType, an additional field,
*  &lt;fieldName&gt;_filename, is generated to
*  store the filename for the binary field value. If this field is present in the
*  data source but has no value for this record, developers can assume they're working
*  with a record with no stored file. If this field is not present in some custom
*  dataSource configuration, or the record is not loaded on the client, an additional
*  server transaction may be required to determine whether the record has an associated
*  file before calling this method to retrieve a download URL.
* 
* 
* @param {Record | any} Record containing at least the primary key field,                  or just the value of the primary key field.
* @param {FieldName=} Optional name of the binary field containing the file. If                  not provided, the first binary field is used.
* @param {Partial<DSRequest>=} Additional properties to set on the                           DSRequest that will be issued.
*/
getFileURL(data:Record | any, fieldName?:FieldName, requestProperties?:Partial<DSRequest>): string;

/**
*  Returns a URL to DataSource fetch operation. This API is intended to return
*  media such as images or videos to the browser. 
*  
*  Note that because the entirety of the request is encoded in the URL, there is an
*  inherent limitation on the amount of data that you can send viat he criteria argument to
*  the server. The actual length depends on your server configuration and other factors
*  such as the size of cookies (if any) being sent to the server and other HTTP headers in
*  use. Conservatively, assume that you have about 2 kilobytes to work with.
* 
* 
* @param {Criteria} Criteria to be sent to server.
* @param {Partial<DSRequest>=} additional properties to set on                            the DSRequest that will be issued
*/
getFetchDataURL(criteria:Criteria, requestProperties?:Partial<DSRequest>): string;

/**
* 
*  Gets the contents of a file stored in this DataSource.
* 
* 
* @param {FileSpec | string} Either a FileSpec, or a String which         will be parsed to determine the fileName, fileType and fileFormat.         For instance, "employees.ds.xml" would be parsed as         {fileName: "employees", fileType: "ds", fileFormat: "xml"}.         If fileType and fileFormat are not provided, will return         the first file with the specified fileName.
* @param {GetFileCallback} Callbacks.GetFileCallback executed with the results. The         data parameter is either a String with the         contents of the file, or null to indicate error (such as         file not found).          You can examine DSResponse.status         and DSResponse.data         for additional information about any error.
*/
getFile(fileSpec:FileSpec | string, callback:(dsResponse:DSResponse, data:string, dsRequest:DSRequest) => void): void;

/**
* 
*  Indicates whether a file exists in this DataSource. 
* 
* 
* @param {FileSpec | string} Either a FileSpec, or a String which         will be parsed to determine the fileName, fileType and fileFormat.         For instance, "employees.ds.xml" would be parsed as         {fileName: "employees", fileType: "ds", fileFormat: "xml"}.         If fileType or fileFormat are not provided, will indicate whether         any file with the provided fileName exists.
* @param {HasFileCallback} Callbacks.HasFileCallback executed with the results.         The data parameter is a boolean indicating         whether the file is present.         You can examine DSResponse.status         and DSResponse.data         for additional information about any error.
*/
hasFile(fileSpec:FileSpec | string, callback:(dsResponse:DSResponse, data:boolean, dsRequest:DSRequest) => void): void;

/**
* 
*  Get a list of files from the DataSource. Note, if 
*  DataSource.fileVersionField is switched on for the dataSource,
*  the resulting list contains only the most recent version of each file.
* 
* 
* @param {Criteria} Criteria to apply. References to fileName,         fileType and fileFormat fields will         be translated to the native field names configured for         this DataSource.
* @param {DSCallback} Callback executed with the results.         The data parameter is either an array of records,         or null to indicate an error.          The records will have the DataSource.fileNameField,         DataSource.fileTypeField,         DataSource.fileFormatField,         DataSource.fileLastModifiedField, and          DataSource.fileVersionField fields populated, but          not the DataSource.fileContentsField field. (You          can use DataSource.getFile to get the fileContents).         You can examine DSResponse.status         and DSResponse.data         for additional information about any error.
*/
listFiles(criteria:Criteria, callback:DSCallback): void;

/**
* 
*  Save a file to the DataSource.
* 
* 
* @param {FileSpec | string} Either a FileSpec, or a String which         will be parsed to determine the fileName, fileType and fileFormat.         For instance, "employees.ds.xml" would be parsed as         {fileName: "employees", fileType: "ds", fileFormat: "xml"}.         Depending on the configuration of the DataSource, the fileType         and fileFormat may be optional.
* @param {string} The contents of the file
* @param {DSCallback=} Callback executed with the results.         The data parameter is either a record represening         the new file, or null to indicate an error. The record will have its         fileName, fileType and fileFormat         field populated, but not the fileContents         field.         You can examine DSResponse.status         and DSResponse.data         for additional information about any error.
*/
saveFile(fileSpec:FileSpec | string, contents:string, callback?:DSCallback): void;

/**
* 
*  Rename a file stored in this DataSource. Note, if 
*  DataSource.fileVersionField is switched on for the dataSource,
*  all versions of the file are renamed.
* 
* 
* @param {FileSpec | string} Either a FileSpec, or a String which         will be parsed to determine the fileName, fileType and fileFormat         of the file to rename.         For instance, "employees.ds.xml" would be parsed as         {fileName: "employees", fileType: "ds", fileFormat: "xml"}.         Depending on the configuration of the DataSource, the fileType and fileFormat         may be optional.
* @param {FileSpec | string} Either a FileSpec, or a String which         will be parsed to determine the fileName, fileType and fileFormat         to rename the file to.         For instance, "employees.ds.xml" would be parsed as         {fileName: "employees", fileType: "ds", fileFormat: "xml"}.         If the fileType or fileFormat are not provided, then they will not         be changed.
* @param {DSCallback=} Callback executed with the results.         The data parameter is either an array of         records represening the renamed file(s), or null to         indicate an error.         The records will have their fileName fields and         fileType fields populated, but not the         fileContents field.         You can examine DSResponse.status         and DSResponse.data         for additional information about any error.
*/
renameFile(oldFileSpec:FileSpec | string, newFileSpec:FileSpec | string, callback?:DSCallback): void;

/**
* 
*  Remove a file stored in this DataSource. Note, if 
*  DataSource.fileVersionField is switched on for the dataSource,
*  all versions of the file are removed (to remove an individual file version, use the 
*  DataSource.removeFileVersion API).
* 
* 
* @param {FileSpec | string} Either a FileSpec, or a String which         will be parsed to determine the fileName, fileType and fileFormat.         For instance, "employees.ds.xml" would be parsed as         {fileName: "employees", fileType: "ds", fileFormat: "xml"}.         Depending the configuration of the DataSource, the fileType         and fileFormat may be optional.
* @param {DSCallback=} Callback executed with the results.         The data parameter is either an array of         records represening the removed file(s), or null to         indicate an error.         The records will have their fileName fields and         fileType fields populated, but not the         fileContents field.         You can examine DSResponse.status         and DSResponse.data         for additional information about any error.
*/
removeFile(fileSpec:FileSpec | string, callback?:DSCallback): void;

/**
* 
*  Get the list of a given file's versions from the dataSource, sorted in version order 
*  (most recent version first). If the dataSource does not specify a 
*  DataSource.fileVersionField, this API will return an error
* 
* 
* @param {FileSpec | string} Either a FileSpec, or a String which         will be parsed to determine the fileName, fileType and fileFormat.         For instance, "employees.ds.xml" would be parsed as         {fileName: "employees", fileType: "ds", fileFormat: "xml"}.         If fileType and fileFormat are not provided, will return         the first file with the specified fileName.
* @param {DSCallback} Callback executed with the results.         The data parameter is either an array of records,         or null to indicate an error.          The records will have the DataSource.fileNameField,         DataSource.fileTypeField,         DataSource.fileFormatField,         DataSource.fileLastModifiedField and          DataSource.fileVersionField fields populated, but          not the DataSource.fileContentsField field. (You         can use DataSource.getFileVersion to get the          fileContents).         You can examine DSResponse.status         and DSResponse.data         for additional information about any error.
*/
listFileVersions(fileSpec:FileSpec | string, callback:DSCallback): void;

/**
* 
*  Gets the contents of a particular file version stored in this DataSource.
* 
* 
* @param {FileSpec | string} Either a FileSpec, or a String which         will be parsed to determine the fileName, fileType and fileFormat.         For instance, "employees.ds.xml" would be parsed as         {fileName: "employees", fileType: "ds", fileFormat: "xml"}.         If fileType and fileFormat are not provided, will return         the first file with the specified fileName.
* @param {Date} A version timestamp. This must exactly match the version         timestamp recorded in the DataSource. You can obtain the list of          versions for a given file with the DataSource.listFileVersions API.
* @param {GetFileVersionCallback} Callbacks.GetFileVersionCallback executed with the results. The         data parameter is either a String with the         contents of the file, or null to indicate error (such as         file not found).          You can examine DSResponse.status         and DSResponse.data         for additional information about any error.
*/
getFileVersion(fileSpec:FileSpec | string, version:Date, callback:(dsResponse:DSResponse, data:string, dsRequest:DSRequest) => void): void;

/**
* 
*  Indicates whether a particular file version exists in this DataSource. 
* 
* 
* @param {FileSpec | string} Either a FileSpec, or a String which         will be parsed to determine the fileName, fileType and fileFormat.         For instance, "employees.ds.xml" would be parsed as         {fileName: "employees", fileType: "ds", fileFormat: "xml"}.         If fileType or fileFormat are not provided, will indicate whether         any file with the provided fileName exists.
* @param {Date} A version timestamp. This must exactly match the version         timestamp recorded in the DataSource for hasFileVersion to         return true. Note, you can obtain the list of versions for a given          file with the DataSource.listFileVersions API.
* @param {HasFileCallback} Callbacks.HasFileVersionCallback         executed with the results. The data parameter is a boolean         indicating whether the file version is present.         You can examine DSResponse.status         and DSResponse.data         for additional information about any error.
*/
hasFileVersion(fileSpec:FileSpec | string, version:Date, callback:(dsResponse:DSResponse, data:boolean, dsRequest:DSRequest) => void): void;

/**
* 
*  Remove a particular file version stored in this DataSource. Any other versions of the
*  file are left untouched.
* 
* 
* @param {FileSpec | string} Either a FileSpec, or a String which         will be parsed to determine the fileName, fileType and fileFormat.         For instance, "employees.ds.xml" would be parsed as         {fileName: "employees", fileType: "ds", fileFormat: "xml"}.         If fileType and fileFormat are not provided, will return         the first file with the specified fileName.
* @param {Date} A version timestamp. This must exactly match the version         timestamp recorded in the DataSource. You can obtain the list of          versions for a given file with the DataSource.listFileVersions API.
* @param {DSCallback=} Callback executed with the results.         The data parameter is either a record representing the         removed file version, or null to indicate an error.         The record will have its DataSource.fileNameField,         DataSource.fileTypeField,         DataSource.fileFormatField,         DataSource.fileLastModifiedField, and          DataSource.fileVersionField fields populated, but          not the DataSource.fileContentsField field.         You can examine DSResponse.status         and DSResponse.data         for additional information about any error.
*/
removeFileVersion(fileSpec:FileSpec | string, version:Date, callback?:DSCallback): void;

/**
*  Executes the given DSRequest on this DataSource.
*  
*  This method is typically used by a DataSource whose DataSource.dataProtocol
*  is set to "clientCustom".
*  Execution of a DSRequest can be delayed, either after a timeout or until some condition
*  is met, by saving the DSRequest object passed to the DataSource.transformRequest
*  override and calling execute() on the DSRequest at a later time.
* 
* @param {DSRequest} the DSRequest to execute.
*/
execute(dsRequest:DSRequest): void;

/**
*  Override point to allow application code to suppress use of a particular offline
*  response. For example, application code may wish to examine the response's 
*  DSResponse.offlineTimestamp to make a decision about whether
*  the response is too stale to be useful.
*  
*  This is an application override point only; there is no default implementation.
* 
* 
* @param {DSRequest} The dsRequest object
* @param {DSResponse} The corresponding dsResponse object returned from                 offline cache
*/
useOfflineResponse(dsRequest:DSRequest, dsResponse:DSResponse): boolean;

/**
*  If you define this method on a DataSource, it will be called whenever the server returns
*  a DSResponse with a status other than RPCResponse.STATUS_SUCCESS. You can use
*  this hook to do DataSource-specific error handling. 
*  Unless you return false from this method, 
*  
*  RPCManager.handleError will be called by SmartClient right after this method completes.
*  
* 
* @param {DSResponse} the DSResponse or DSResponse object returned from the server
* @param {DSRequest} the DSRequest or DSRequest that was sent to the server
*/
handleError(response:DSResponse, request:DSRequest): boolean;

/**
*  Return the field definition object.
* 
* @param {FieldName} Name of the field to retrieve
*/
getField(fieldName:FieldName): DataSourceField;

/**
*  Return the field definition object corresponding to the supplied dataPath
* 
* @param {string} dataPath of the field to retrieve
*/
getFieldForDataPath(dataPath:string): DataSourceField;

/**
*  Given a fieldName and a dataValue, apply any DataSourceField.valueMap for the
*  field and return the display value for the field
* 
* @param {FieldName} name of the field to retrieve a value for
* @param {any} data value for the field
*/
getDisplayValue(fieldName:FieldName, value:any): any;

/**
*  Retrieves the list of fields declared on this DataSource.
* 
* 
* @param {boolean} If true, returns only those fields that are                 not marked as hidden
*/
getFieldNames(excludeHidden:boolean): Array<Partial<FieldName>>;

/**
*  Returns the path having the shortest distance between this and the given targetDS,
*  as determined by DataSource.getShortestPathToRelation.
* 
* @param {string | DataSource} The DataSource at the relationship's other end.
*/
getDefaultPathToRelation(targetDS:string | DataSource): RelationPath;

/**
*  Returns the path having the shortest distance between this and the given targetDS. In
*  case of a tie, only the first as determined by depth-first search is returned.
* 
* @param {string | DataSource} The DataSource at the relationship's other end.
*/
getShortestPathToRelation(targetDS:string | DataSource): RelationPath;

/**
*  Returns all known paths between this and the given targetDS.
* 
* @param {string | DataSource} The DataSource at the relationship's other end.
*/
getAllPathsToRelation(targetDS:string | DataSource): RelationPath;

/**
*  Returns this DataSource's DataSourceField.primaryKey fields as a map
*  of fieldName to field.
* 
*/
getPrimaryKeyFields(): Record;

/**
*  Returns a list of the names of this DataSource's 
*  DataSourceField.primaryKey fields.
* 
*/
getPrimaryKeyFieldNames(): Array<Partial<string>>;

/**
*  Returns a pointer to the primaryKey field for this DataSource. If this dataSource has
*  a composite primary key (ie, multiple primaryKey fields), returns just the first 
*  primaryKey field.
* 
*/
getPrimaryKeyField(): DataSourceField;

/**
*  Returns the primary key fieldName for this DataSource.  If this dataSource has
*  a composite primary key (ie, multiple primaryKey fields), returns just the first 
*  primaryKey field name.
* 
*/
getPrimaryKeyFieldName(): string;

/**
*  Return a reasonable user-visible title given a fieldName. Called when
*  DataSource.autoDeriveTitles is true and by default, calls the class method
*  DataSource.getAutoTitle. Override to provide a different policy
*  for auto-deriving titles for a particular DataSource or subclass of DataSource.
* 
* 
* @param {string} identifier for which a title is desired.
*/
getAutoTitle(identifier:string): string;

/**
*  Formats the supplied value using the SimpleType derived from the field definition.
*  
*  Note that if DataSourceField.format is defined for a date, time or numeric-based
*  value, or DataSourceField.dateFormatter or DataSourceField.timeFormatter is
*  defined for a date or time-based value, that format is given priority and used to format
*  the value rather than the SimpleType.
* 
* 
* @param {DataSourceField | FieldName} name of the field to use to format value
* @param {any} raw value to be formatted
*/
formatFieldValue(field:DataSourceField | FieldName, value:any): string;

/**
*  Returns the complete set of data cached by this dataSource. Note that this may 
*  have been supplied via DataSource.cacheData, or may have been fetched
*  from the server for dataSources with DataSource.cacheAllData set to true.
* 
*/
getCacheData(): Array<Partial<Record>>;

/**
*  Return a "spoofed" response for a DataSource.clientOnly or DataSource.cacheAllData DataSource.
*  
*  The default implementation will use DataSource.cacheData to provide an appropriate
*  response, by using DataSource.applyFilter for a "fetch" request, and
*  by modifying the cacheData for other requests.
*  
*  Override this method to provide simulations of other server-side behavior, such as
*  modifying other records, or to implement synchronous client-side data providers
*  (such as Google Gears). For asynchronous third-party data providers, such as
*  GWT-RPC, HTML5 sockets, or bridges to plug-in based protocols (Java, Flash,
*  Silverlight..), use DSProtocol instead. 
*  
*  Overriding this method is also a means of detecting that a normal DataSource (not
*  clientOnly) would be contacting the server.
* 
* 
* @param {DSRequest} DataSource request to respond to
* @param {Array<Partial<Record>>} for cacheAllData DataSources, the data from the local cache
*/
getClientOnlyResponse(request:DSRequest, serverData:Array<Partial<Record>>): DSResponse;

/**
*  Returns records in the passed Array that match the provided filter
*  Criteria. Handles simple or AdvancedCriteria criteria.
*  
*  By default:
*  
*  
* - any criteria that do not correspond to a DataSource field are ignored
*  
* - for simple criteria, any null or empty string criteria are ignored and
*    all other criteria are passed to DataSource.fieldMatchesFilter
*  
* - for advanced criteria, each criterion is evaluated via DataSource.evaluateCriterion
*  
*  This method is called by ResultSet.applyFilter to provide filtering when a
*  ResultSet has a complete cache and filtering can be performed client-side. You may want
*  to override this method in order to mimic the filtering behavior that your server performs.
* 
* 
* @param {Array<Partial<Record>>} the list of rows
* @param {Criteria} the filter criteria
* @param {Partial<DSRequest>=} optional dataSource request properties
* @param {Integer=} starting index - earlier rows are excluded
* @param {Integer=} ending index (exclusive) - this row and beyond are excluded
*/
applyFilter(data:Array<Partial<Record>>, criteria:Criteria, requestProperties?:Partial<DSRequest>, startPos?:Integer, endPos?:Integer): Array<any>;

/**
*  Compares a criteria value to a field value and returns whether they match, as follows:
*  
*  
* - any non-String filter value is directly compared (==) to the field value
*  
* - any String filter value is compared according to
*    DSRequest.textMatchStyle in the passed requestProperties,
*    regardless of the actual field type
*  
* - if the filter value is an Array, the comparison is a logical OR. If textMatchStyle
*    is "exact", it matches if fieldValue (or any of it's entries, if it's also an array) 
*    is contained in the filterValue Array. If textMatchStyle if substring, it matches 
*    if any of the entries in filterValue appear as a case-insensitive substring of any
*    of the entries in fieldValue.
*  
* - Dates are compared as logical dates if either the field value or the filter value is a logical date.
*    Only if none of them is a logical date they will be compared as standard Dates
*  
* 
* @param {any} field value to be compared
* @param {any} filter value to be compared
* @param {Partial<DSRequest>=} optional dataSource request properties
*/
fieldMatchesFilter(fieldValue:any, filterValue:any, requestProperties?:Partial<DSRequest>): boolean;

/**
*  Does this dataSource support the specified "textMatchStyle" when performing a filter
*  operation against a text field.
* 
* @param {TextMatchStyle} textMatchStyle to check. If passed a null value,   assume an exact match is being requested.
*/
supportsTextMatchStyle(textMatchStyle:TextMatchStyle): void;

/**
*  Given two sets of criteria, determine whether they are equivalent, the new criteria is
*  guaranteed more restrictive, or the new criteria is not guaranteed more restrictive,
*  returning 0, 1 or -1 respectively.
*  
*  Comparisons between AdvancedCriteria are made via recursively calling
*  Operator.compareCriteria for all criteria involved.
*  
*  For simple Criteria, by default (CriteriaPolicy:"dropOnShortening"), returns:
*  
*  
* - -1 if the new criteria has fewer properties than the old criteria (indicating that it
*    isn't more restrictive)
*  
* - -1 if the value for any property in the old criteria is an array and 1) the value for
*    the same property in the new criteria isn't an array, or 2) is an array but
*    of different length, or 3) the arrays do not contain the exact same set of objects
*    (order can be different)
*  
* - -1 if the value for any given property in the old criteria is not an array, and the
*    the value for the same property property in the new criteria is different
*  
* - -1 if both values for a given property are strings and the new criteria value doesn't
*    contain the old criteria value
*  
* - 1 if none of the above are true and, for at least one of the properties, the
*    respective criteria values are both strings, and the old criteria value is a substring
*    of, and is shorter than, the new criteria value
*  
* - 0 otherwise (indicating the sets of criteria are equivalent)
*  
*  
*  For (CriteriaPolicy:"dropOnChange"), returns:
*  
*  
* - -1 if the two sets of criteria have a different number of properties
*  
* - -1 if the value for any property in the old criteria is an array and 1) the value for
*    the same property in the new criteria isn't an array, or 2) is an array but
*    of different length, or 3) the arrays do not contain the exact same set of objects
*    (order can be different)
*  
* - -1 if the value for any given property in the old criteria is not an array, and the
*    the value for the same property in the new criteria is different
*  
* - 0 otherwise (indicating the sets of criteria are equivalent)
*  
*  This method is called by ResultSet.compareCriteria to determine whether a change
*  in criteria should cause the cache to be invalidated. You may want
*  to override this method in order to mimic the filtering behavior that your server performs.
* 
* 
* @param {Criteria} new filter criteria
* @param {Criteria} previous filter criteria
* @param {Partial<DSRequest>=} dataSource request properties
* @param {string=} overrides CriteriaPolicy
*/
compareCriteria(newCriteria:Criteria, oldCriteria:Criteria, requestProperties?:Partial<DSRequest>, policy?:string): number;

/**
*  Returns the depth-first match of a criterion matching the given fieldName.
* 
* 
* @param {Criteria} the criteria to search
* @param {FieldName} the fieldName to find criteria for
*/
getFieldCriterion(criterion:Criteria, fieldName:FieldName): Criteria;

/**
*  Convenience method to compare two Date objects appropriately, depending on whether the
*  passed-in fieldName refers to a field of FieldType "datetime" or 
*  "date". In the former case, the dates are compared using DateUtil.compareDates;
*  in the latter case, or if the supplied fieldName is null or unknown to this DataSource,
*  the dates are compared using DateUtil.compareLogicalDates.
* 
* @param {Date} First date in comparison
* @param {Date} Second date in comparison
* @param {FieldName} The name of the field for which the comparison is being run
*/
compareDates(date1:Date, date2:Date, fieldName:FieldName): number;

/**
*  Convenience method to test if two records are equal. Testing is done only for the
*  fields defined in the DataSource, anything else is ignored.
* 
* 
* @param {any} record to be compared against.
* @param {any} record to be compared.
*/
recordsAreEqual(record1:any, record2:any): boolean;

/**
*  Converts criteria expressed in SmartClient's simple criteria format to an 
*  AdvancedCriteria object. This instance method differs from the class method
*  DataSource.convertCriteria in that it makes use of the dataSource 
*  as schema to help in the conversion. For example, this method is able to honor 
*  DataSourceField.ignoreTextMatchStyle and use the dataSource's 
*  DataSource.defaultTextMatchStyle rather than assuming "substring"
* 
* 
* @param {Criteria} simple criteria
* @param {TextMatchStyle=} default style of matching text. Defaults to                     the dataSource's defaultTextMatchStyle
*/
convertDataSourceCriteria(criteria:Criteria, textMatchStyle?:TextMatchStyle): AdvancedCriteria;

/**
*  Do fetch and filter operations on this dataSource support being passed 
*  AdvancedCriteria?
*  
*  For a DataSource to support being passed AdvancedCriteria, it must be 
*  DataSource.clientOnly or DataSource.cacheAllData, or have 
*  server side logic which can process AdvancedCriteria objects passed from the client.
*  
*  AdvancedCriteria are supported on the server for standard 
*  sqlDataSource, hibernateIntegration and 
*  jpaIntegration DataSources in SmartClient Enterprise or Power editions
*  (not supported in SmartClient Pro).
*  
*  The framework assumes that custom dataSources support AdvancedCriteria; if you have a 
*  a custom DataSOurce implementation that does not support AdvancedCriteria, you can set
*  the DataSource.allowAdvancedCriteria property to false.
*  
* 
*/
supportsAdvancedCriteria(): boolean;

/**
*  Add a new search operator, only to this DataSource. 
*  
*  If an existing Operator is passed, restricts the set of FieldTypes to which that
*  operator can be applied in this DataSource.
*  
*  See also DataSource.addSearchOperator for adding operators to all
*  DataSources.
* 
* 
* @param {Operator} definition of the operator to add
* @param {Array<Partial<FieldType>>=} types to which this operator applies
*/
addSearchOperator(operator:Operator, types?:Array<Partial<FieldType>>): void;

/**
*  Get the Operator definition for an OperatorId.
* 
* 
* @param {OperatorId} the id of the operator
*/
getSearchOperator(operatorId:OperatorId): Operator;

/**
*  Get the list of OperatorIds available on this DataSource for the given FieldType.
*  
*  If DataSource.setTypeOperators has been called for this DataSource and FieldType, returns that
*  list, otherwise, returns the set of valid operators for the FieldType as specified by 
*  SimpleType.validOperators, otherwise, the system-wide set of valid operators for the
*  type as registered via DataSource.addSearchOperator.
*  
* 
* @param {FieldType | string=} Defaults to "text" if not passed.
*/
getTypeOperators(typeName?:FieldType | string): Array<Partial<OperatorId>>;

/**
*  Set the list of OperatorIds valid for a given FieldType.
*  
* 
* @param {FieldType | string} * @param {Array<Partial<OperatorId>>} available Operators
*/
setTypeOperators(typeName:FieldType | string, operators:Array<Partial<OperatorId>>): void;

/**
*  Returns true if the operator list for the passed type has been customized via a call to 
*  DataSource.setTypeOperators.
*  
* 
* @param {string | FieldType} */
hasCustomTypeOperators(typeName:string | FieldType): void;

/**
*  Get the list of OperatorIds available for this field.
*  
*  By default, if DataSourceField.validOperators is set,
*  returns that list, otherwise returns the result of DataSource.getTypeOperators.
* 
* @param {string | DataSourceField} Field (or field name) to obtain operators for
*/
getFieldOperators(field:string | DataSourceField): Array<Partial<OperatorId>>;

/**
*  Get the default OperatorId for this field.
*  
*  By default, if DataSourceField.defaultOperator is set,
*  returns that value, otherwise returns the data-type default from 
*  SimpleType.defaultOperator.
* 
* @param {string | DataSourceField} Field (or field name) to obtain the default                     operator for
*/
getFieldDefaultOperator(field:string | DataSourceField): Array<Partial<OperatorId>>;

/**
*  Get the list of Operators available for this field, as a ValueMap from
*  OperatorId to the Operator.title specified for the Operator, or
*  the corresponding property in Operators if Operator.titleProperty is set.
*  
*  This valueMap is suitable for use in a UI for building queries, similar to the
*  FilterBuilder, and optionally omits operators marked Operator.hidden:true.
*  
*  It is also possible to have this function return only operators of a given 
*  OperatorValueType, or everything except operators of that type. This is useful, 
*  for example, if you want to return all the logical operators (like "and"), or everything 
*  except the logical operators.
* 
* 
* @param {string | DataSourceField} Field (or field name) to obtain operator map for.
* @param {boolean=} whether to include Operators marked hidden:true
* @param {OperatorValueType=} If passed, returns only operators of this OperatorValueType
* @param {boolean=} If set, reverses the meaning of the valueType     parameter, so operators of that OperatorValueType are the only ones omitted
*/
getFieldOperatorMap(field:string | DataSourceField, includeHidden?:boolean, valueType?:OperatorValueType, omitValueType?:boolean): ValueMap;

/**
*  Get the list of Operators available for this FieldType, as a ValueMap from
*  OperatorId to the Operator.title specified for the Operator, or
*  the corresponding property in Operators if Operator.titleProperty is set.
*  
*  This valueMap is suitable for use in a UI for building queries, similar to the
*  FilterBuilder, and optionally omits operators marked Operator.hidden : true.
*  
*  It is also possible to have this function return only operators of a given 
*  OperatorValueType, or everything except operators of that type. This is useful, 
*  for example, if you want to return all the logical operators (like "and"), or everything 
*  except the logical operators.
* 
* 
* @param {FieldType=} Type to obtain operator map for. Defaults to "text" if not passed.
* @param {boolean=} whether to include Operators marked hidden:true
* @param {OperatorValueType=} If passed, returns only operators of this OperatorValueType
* @param {boolean=} If set, reverses the meaning of the valueType     parameter, so operators of that OperatorValueType are the only ones omitted
*/
getTypeOperatorMap(type?:FieldType, includeHidden?:boolean, valueType?:OperatorValueType, omitValueType?:boolean): ValueMap;

/**
*  Converts a list of Records to simple text formats with a Record per line and values separated
*  by a configurable separator, including both tab-separated-values and comma-separated-values
*  (aka CSV).
*  
*  In addition to the settings parameter for this method,
*  DataSourceField.exportForceText can be set. 
*  
*  If two or more different text exports are needed for the same DataSource creating a conflict
*  for any DataSourceField setting, DataSource.inheritsFrom can be used to create a
*  child DataSource where these settings can be changed without recapitulating all field
*  definitions.
* 
* 
* @param {Array<Partial<Record>>} records to convert
* @param {Partial<TextExportSettings>=} settings for the export
*/
recordsAsText(records:Array<Partial<Record>>, settings?:Partial<TextExportSettings>): string;

/**
*  Derive a list of Records from Microsoft Excel-compatible tab-separated-values format, using
*  the current DataSource field order, or an explicitly specified list of fields.
*  
*  If a specified field does not exist in the DataSource, it's assumed the values for that
*  field should end up as Strings.
*  
* 
* @param {string} records as CSV/TSV (separator can be specified)
* @param {Partial<TextImportSettings>=} optional settings for the import
*/
recordsFromText(text:string, settings?:Partial<TextImportSettings>): Array<Partial<Record>>;

/**
*  Evaluate the given criterion with respect to the passed record.
*  
*  Typically called by the Operator.condition function of a custom
*  Operator to evaluate Criterion.criteria.
* 
* 
* @param {Record} record to evaluate
* @param {Criterion} criterion to use
*/
evaluateCriterion(record:Record, criterion:Criterion): boolean;

/**
*  Split a criteria apart based on fields.
*  
*  This method will take a simple or AdvancedCriteria 
*  criteria object and extract the subcriteria that apply to 
*  the specified array of fields. If passed an AdvancedCriteria, the
*  criteria should be DataSource.isFlatCriteria and the
*  outer operator must be "and".
*  
*  A new criteria object is returned with any criteria applicable
*  to the specified fields. The passed
*  criteria is then modified to remove these fields resulting in
*  two distinct criteria.
*  
*  To avoid modifying an original criteria, use DataSource.copyCriteria to
*  make a copy to be passed in.
*  
*  By default the field-specific criteria returned will be in simple criteria format
*  even if the criteria passed in was Advanced. Developers may suppress this conversion
*  by passing in the preserveAdvanced parameter. Note that 
*  not every OperatorId can
*  be converted to a simple format. This method will only to convert field level
*  criterion with operators that correspond to one of the available
*  TextMatchStyle options - namely 
*  "equals", "iEquals"
*  "iContains" or "startsWith". 
*  
*  This method may return null if it was unable to split the specified criteria
*  by the specified fields.
* 
* 
* @param {Criteria} criteria to be split. May be modified if criteria is              extracted.
* @param {Array<Partial<string>>} list of fields to extract from criteria
* @param {boolean=} If passed an AdvancedCriteria, should the  returned field-specific split criteria object also be an AdvancedCriteria.
*/
splitCriteria(criteria:Criteria, fields:Array<Partial<string>>, preserveAdvanced?:boolean): Criteria;



}
/**
* Creates a new DataSource
* 
* @param typeCheckedProperties {Partial<DataSource>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DataSource} The new instance
*/
class DataSource{
static create(typeCheckedProperties? : Partial<DataSource>, uncheckedProperties?: Object): DataSource;
/**
* The URL where the DataSourceLoader servlet has been installed. Defaults to the
*  Page.setIsomorphicDir plus "/DataSourceLoader". Change by calling
*  calling DataSource.setLoaderURL
* @type {URL}
* @default "[ISOMORPHIC]/DataSourceLoader"
*/
static loaderURL?: URL;

/**
* Default value if DataSourceField.stringInBrowser is not defined. See
*  stringInBrowser docs for details.
*  Note this setting should be used only if you are *not* using the SmartClient Server.
* @type {boolean}
* @default null
*/
static defaultStringInBrowser?: boolean;

/**
* Specifies how time field values should be serialized when being sent to the server for
*  dataSources with dataFormat "xml" or "json".
*  If false the time field will be serialized as a logical time object in
*  UTC, using the standard truncated XML Schema format: "HH:MM:SS".
*  If true the time field will be serialized as a complete dateTime object
*  matching the value of the underlying JavaScript date object representing this time on the
*  client.
* @type {boolean}
* @default false
*/
static serializeTimeAsDatetime?: boolean;

/**
* A message returned by a DataSource when it is returning an empty dataset for a fetch
*  because the browser is currently offline and there is no suitable cached offline response.
* @type {string}
* @default "This data not available while offline"
*/
static offlineMessage?: string;

/**
* A message returned by a DataSource when an uploaded file's size exceeded
*  DataSourceField.maxFileSize.
*  
*  If this is not set, then Validator.maxFileSizeExceeded value will be used.
* @type {HTMLString}
* @default "Size of '${uploadedFileName}' (${uploadedFileSize}) exceeded maximum allowed file size of ${maxFileSize}"
*/
static maxFileSizeExceededMessage?: HTMLString;

/**
* A message returned by a DataSource when an uploaded file was empty, but the
*  field is DataSourceField.required.
* @type {HTMLString}
* @default "'${uploadedFileName}' was empty, but empty files are not allowed."
*/
static requiredFileMessage?: HTMLString;

/**
* A message returned by a DataSource when an operation requires criteria, but none was provided.
* @type {HTMLString}
* @default "Operation requires criteria for the following field(s): [${requiredCriterion}]"
*/
static requiredCriterionMessage?: HTMLString;

/**
* Provides the global default setting for deep or shallow cloning of objects prior to
*  edit. See DataSource.deepCloneOnEdit for details of what this means.
*  
*  The default setting of null is the same as false; with this default in place, 
*  deepCloneOnEdit must be set explicitly set at dataSource, component or 
*  field level if you require deep cloning.
* @type {boolean}
* @default null
*/
static deepCloneOnEdit?: boolean;

/**
* Provides the global default setting for deep or shallow cloning of non-field attributes
*  of value objects prior to edit. See 
*  DataSource.deepCloneNonFieldValuesOnEdit for details of what this means.
*  
*  The default setting of null is the same as false; with this default in place, 
*  deepCloneNonFieldValuesOnEdit must be set explicitly set at dataSource or
*  component level if you require deep cloning.
* @type {boolean}
* @default null
*/
static deepCloneNonFieldValuesOnEdit?: boolean;





/**
*  A utility that checks for discrepancies between any two DataSources, typically used to
*  warn about issues commonly found during the Reify design / development cycle, and logs
*  its findings to the console. Similar to the server-side
*  ReifyDataSourceValidator in scope, but with no support for
*  server-only attributes (e.g., Declarative Security).
*  
*  INFO level messages are logged when any of the following conditions are discovered:
*  
*   
* - A field defined in "live" is not also present in mock
*   
* - A field defined in "live" has a different title than mock (fields using i18n titles are not checked) 
*  
*  WARN level messages are logged when any of the following conditions are discovered:
*  
*   
* - A field defined in mock is not also present in "live"
*   
* - A field defined in mock has a different type than in live (and the live type is a not a sub-type of the mock type)
*   
* - Fields in mock have a different order than in "live" (after ignoring any fields that mock lacks)
*  
*  
* 
* 
* @param {DataSource} DataSource to compare using 'live' rules
* @param {DataSource} DataSource to compare using 'mock' rules
*/
static verifyDataSourcePair(live:DataSource, mock:DataSource): Array<Partial<object>>;

/**
*  Lookup a DataSource by ID.
* 
* 
* @param {GlobalId} DataSource ID
*/
static getDataSource(ID:GlobalId): DataSource;

/**
*  Synonym of DataSource.getDataSource: Lookup a DataSource by ID.
* 
* 
* @param {GlobalId} DataSource ID
*/
static get(ID:GlobalId): DataSource;

/**
*  Helper method to return the value of the supplied field from within the supplied record,
*  looking up the value from the supplied dataPath. This method is only of real
*  use when you are dealing with complex nested data via Canvas.dataPath; 
*  it is obviously a trivial matter to obtain a field value from a flat record directly.
*  
*  If the dataPath is null, this method will follow any ListGridField.dataPath 
*  specified on the component field instead. In either case, it will also extract 
*  SimpleType.getAtomicValue from custom SimpleType fields
*  where this is required.
* 
* 
* @param {DataSourceField | ListGridField | DetailViewerField | FormItem} Field definition from a dataSource or dataBoundComponent.
* @param {Record} The valueset in which to look up the field value
* @param {string} Optionally, a string expressing the dataPath to use for value lookup.              If null, the dataPath from the field will be used
* @param {Canvas} Optionally, a component to prvide extra context for the dataPath               search. This is typically required if the dataPath traverses a list
* @param {string} An optional reason for the get request, to be passed into                  any SimpleType.getAtomicValue method - see that                  API for details
*/
static getFieldValue(field:DataSourceField | ListGridField | DetailViewerField | FormItem, record:Record, dataPath:string, component:Canvas, reason:string): any;

/**
*  Helper method to save the argument value inside the argument values record, storing the 
*  value at the supplied dataPath, or at the field's declared dataPath if the argument 
*  dataPath is null. This method is only of real use when you are dealing with complex 
*  nested data via Canvas.dataPath; it is obviously a trivial matter 
*  to store a field value in a flat record directly.
*  
*  This method will call the SimpleType.updateAtomicValue
*  method of custom SimpleType fields where this is required.
* 
* 
* @param {DataSourceField | ListGridField | DetailViewerField | FormItem} Field definition from a dataSource or dataBoundComponent.
* @param {string} The dataPath to store the value at. If null, the dataPath will             be picked up from the field
* @param {any} The value to save
* @param {Record} The valueset in which to save the field value
* @param {string} An optional reason for the save request, to be passed into                  any SimpleType.updateAtomicValue method - see that                  API for details
*/
static saveValueViaDataPath(field:DataSourceField | ListGridField | DetailViewerField | FormItem, dataPath:string, value:any, values:Record, reason:string): void;

/**
*  Helper method to save the argument value inside the argument values record, storing the 
*  value at the supplied dataPath, or at the field's declared dataPath if the argument 
*  dataPath is null. This method is only of real use when you are dealing with complex 
*  nested data via Canvas.dataPath; it is obviously a trivial matter 
*  to store a field value in a flat record directly.
*  
*  This method will call the SimpleType.updateAtomicValue
*  method of custom SimpleType fields where this is required.
* 
* 
* @param {DataSourceField | ListGridField | DetailViewerField | FormItem} Field definition from a dataSource or dataBoundComponent.
* @param {string} The dataPath at which to clear the value. If null, the dataPath will             be picked up from the field
* @param {Record} The valueset from which to clear the field value
*/
static clearValueAtDataPath(field:DataSourceField | ListGridField | DetailViewerField | FormItem, dataPath:string, values:Record): void;

/**
* 
*  Converts a file path to a FileSpec.
* 
* 
* @param {string} The path to convert, e.g. "employees.ds.xml"
*/
static makeFileSpec(path:string): FileSpec;

/**
*  Load a DataSource or an array of DataSources using the DataSourceLoader servlet. When
*  a callback is specified, this is fired after the DataSources are loaded. The callback
*  is passed a single parameter, the dsID list passed into the method. If
*  no loading occurs because the requested DataSource(s) are already loaded, a warning is 
*  logged and the callback is fired immediately.
*  
*  To force reloading of DataSources that have already been loaded,
*  pass true for the forceReload parameter. Note that if a DataSource has been
*  created locally with the specified ID, even if this is a MockDataSource, the
*  forceReload parameter will be required to force the "real" dataSource
*  to be loaded.
* 
* 
* @param {string | Array<Partial<string>>} DataSource ID or Array of DataSource IDs
* @param {Function} Callback to fire after DataSource loading completes
* @param {boolean | DSLoadSettings=} Forcibly reload a dataSource if it's already loaded
*/
static load(dsID:string | Array<Partial<string>>, callback:Function, forceReload?:boolean | DSLoadSettings): void;

/**
*  Variation of DataSource.load that will also automatically load any DataSources that
*  the requested DataSources inherit from (via DataSource.inheritsFrom).
*  
*  If the parent DataSource is already loaded, calling loadWithParents will not
*  automatically reload them unless the forceReload parameter is passed.
*  
* 
* @param {string | Array<Partial<string>>} DataSource ID or Array of DataSource IDs
* @param {Function} Callback to fire after DataSource loading completes
* @param {boolean | DSLoadSettings=} Forcibly reload a dataSource if it's already loaded
*/
static loadWithParents(dsID:string | Array<Partial<string>>, callback:Function, forceReload?:boolean | DSLoadSettings): void;

/**
*  Given an array of SortSpecifiers, return a simple list of Strings in the format
*  expected by DSRequest.sortBy.
* 
* 
* @param {Array<Partial<SortSpecifier>>} The list of specifiers to return in  sortBy format
*/
static getSortBy(sortSpecifiers:Array<Partial<SortSpecifier>>): Array<Partial<string>>;

/**
*  Return an array of SortSpecifiers, given an array of Strings in the format
*  expected by DSRequest.sortBy.
* 
* 
* @param {Array<Partial<string>>} A list of sortBy strings in the format expected by DSRequest.sortBy
*/
static getSortSpecifiers(sortBy:Array<Partial<string>>): Array<Partial<SortSpecifier>>;

/**
*  Create a copy of a criteria.
* 
* 
* @param {Criteria} criteria to copy
*/
static copyCriteria(criteria:Criteria): Criteria;

/**
*  Returns true if a given AdvancedCriteria is "flat." That is, the criteria consists of either:
*  
*  
* - a top-level "and" or "or" Criterion, where none of the 
*    Criterion.criteria use LogicalOperator,
*    hence have no subcriteria of their own
*  
* - a single Criterion that is not a logical operator, hence has no subcriteria
*  
* 
* 
* @param {AdvancedCriteria} the AdvancedCriteria to check for flatness
*/
static isFlatCriteria(criteria:AdvancedCriteria): boolean;

/**
*  Returns new criteria that has at most one top-level LogicalOperator ("and" or "or").
*  This new criteria will be considered "flat" by DataSource.isFlatCriteria.
*  
*  Not all AdvancedCriteria can be flattened and remain logically equivalent. When
*  criteria will be logically modified by flattening, all criteria that appear anywhere in
*  the AdvancedCriteria structure will appear under a single top-level operator, which
*  will be the same top-level operator as the passed AdvancedCriteria.
*  
*  For example, given criteria like this (in the JSON representation of AdvancedCriteria):
*  
*    { operator: "and", criteria: [
*      { fieldName: "continent", operator: "equals", value: "Europe"},
*      { operator: "or", criteria: [
*       { fieldName: "countryName", operator: "iEndsWith", value: "land"},
*       { fieldName: "population", operator: "lessThan", value: 3000000}
*      ]}
*     ]
*    }
*  
*  The returned criteria would be:
*  
*    { operator: "and", criteria: [
*      { fieldName: "continent", operator: "equals", value: "Europe"},
*      { fieldName: "countryName", operator: "iEndsWith", value: "land"},
*      { fieldName: "population", operator: "lessThan", value: 3000000}
*     ]}
*  
*  This returned criteria is not logically equivalent to the passed criteria - the
*  "iEndsWith" and "lessThan" criteria that were formerly nested under a logical "or"
*  operator must now both be satisfied instead of either being satisfied.
*  You can use DataSource.canFlattenCriteria to detect whether an AdvancedCriteria is going
*  to be changed by flattenCriteria().
*  
*  Because the returned criteria may not be logically equivalent,
*  flattenCriteria should not be used as a means of simplifying criteria to
*  make server implementation easier or anything of the kind. The primary purpose of
*  returning logically different criteria is to enable an end user to switch from an
*  interface for editing nested criteria to an interface that can't handle nested
*  criteria and convert the criteria while preserving as much as possible.
* 
* 
* @param {AdvancedCriteria} the AdvancedCriteria to flatten
*/
static flattenCriteria(criteria:AdvancedCriteria): AdvancedCriteria;

/**
*  Returns true if calling DataSource.flattenCriteria on the passed criteria would produce
*  logically equivalent criteria.
* 
* 
* @param {AdvancedCriteria} the AdvancedCriteria to check for flatness
*/
static canFlattenCriteria(criteria:AdvancedCriteria): boolean;

/**
*  Returns a human-readable string describing the clauses in this advanced criteria or criterion.
*  
* 
* @param {AdvancedCriteria | Criterion} Criteria to convert to a readable string
* @param {DataSource} DataSource to provide definitions of operators
*/
static getAdvancedCriteriaDescription(criteria:AdvancedCriteria | Criterion, dataSource:DataSource): string;

/**
*  Returns the DataSource.setLoaderURL
* 
* 
*/
static getLoaderURL(): string;

/**
*  Sets the URL where the DataSourceLoader servlet has been installed; this is used by the 
*  DataSource.load method. Note, one reason you may wish to modify the loader URL
*  is to include a Cross-Site Request Forgery (CSRF) token, as described 
*  RPCManager.actionURL
* 
* 
* @param {string} The new loaderURL
*/
static setLoaderURL(url:string): void;

/**
*  Add a new search operator to all DataSources. 
*  
*  See also DataSource.addSearchOperator for adding operators to specific
*  DataSources only.
* 
* 
* @param {Operator} definition of the operator to add
*/
static addSearchOperator(operator:Operator): void;

/**
*  Set the list of valid OperatorIds for a given FieldType.
*  
* 
* @param {string | FieldType} * @param {Array<Partial<OperatorId>>} available Operators
*/
static setTypeOperators(typeName:string | FieldType, operators:Array<Partial<OperatorId>>): void;

/**
*  Returns true if the operator list for the passed type has been customized via a call to 
*  DataSource.setTypeOperators.
*  
* 
* @param {string | FieldType} * @param {DataSource} */
static hasCustomTypeOperators(typeName:string | FieldType, ds:DataSource): void;

/**
*  Utility method to derive a reasonable user-visible title from an identifier.
*  
*  The following approach is taken:
*  
*  
* - any underscores (_) or dollar signs ($) become spaces, except that there will never
*    be either a leading or trailing space.
*  
* - if the fieldName is either entirely uppercase or lowercase, all words separated by
*    spaces are given a leading capital letter. Example USER_NAME or user_name -> "User
*    Name".
*  
* - if there is any use of mixed case, camelCaps convention is assumed, and the field name
*    is split into separate words based on 1) everywhere an uppercase letter appears after a
*    lowercase letter 2) everywhere a series of uppercase letters ends. Letter case will
*    not be modified, with the exception that the first word or any word after an
*    underscore will have its first letter capitalized. Examples: useHTTPProxy -> "Use
*    HTTP Proxy", audit_userName -> "Audit User Name"
*  
* 
* 
* @param {string} identifier for which a title is desired.
*/
static getAutoTitle(identifier:string): string;

/**
*  Converts criteria expressed in SmartClient's simple criteria format to an AdvancedCriteria
*  object. 
* 
* 
* @param {Criteria} simple criteria
* @param {TextMatchStyle=} default style of matching text. Defaults to                     "substring"
*/
static convertCriteria(criteria:Criteria, textMatchStyle?:TextMatchStyle): AdvancedCriteria;

/**
*  Combines two criteria (either simple criteria objects or AdvancedCriteria) using the 
*  "outerOperator". Note that the combined criteria object will be an AdvancedCriteria
*  unless: 
*  
* - both input criteria objects are simple, and
*  
* - the "outerOperator" is "and", and
*  
* - there is no collision of key names on the two criteria
*  
* 
* @param {Criteria} first criteria object
* @param {Criteria} second criteria object
* @param {CriteriaCombineOperator=} operator to use to combine the criteria.                      Defaults to "and"
* @param {TextMatchStyle=} style of matching text, if it is necessary to                     convert a simple criteria object to an                      AdvancedCriteria. Defaults to "substring"
*/
static combineCriteria(criteria1:Criteria, criteria2:Criteria, outerOperator?:CriteriaCombineOperator, textMatchStyle?:TextMatchStyle): Criteria;

/**
*  Applies a RecordSummaryFunction to a record and returns the result.
* 
* @param {SummaryFunction} Summary Function or identifier for  registered recordSummaryFunction to execute. If passed in as an explicit function  record, fields and summaryField parameters will be passed through to the function.
* @param {any} Record to retrieve a summary for
* @param {Array<Partial<DataSourceField>>} Set of fields to include in the summary
* @param {DataSourceField} field in which this summary will be displayed.
*/
static applyRecordSummaryFunction(summaryFunction:SummaryFunction, record:any, fields:Array<Partial<DataSourceField>>, summaryField:DataSourceField): any;

/**
*  Register a new standard RecordSummaryFunction. This will then be available
*  by calling SimpleType.applySummaryFunction and passing in just the new method name.
* 
* @param {string} identifier for the new summary function
* @param {Function | StringMethod} new summary function implementation. This method should take 3 parameters: record (the record for which the  summary is being generated), fields (an array of fields to include in the  generated summary) and summaryField (a pointer to the field in which the  summary will be displayed [may be null].
*/
static registerRecordSummaryFunction(methodName:string, summaryFunction:Function | StringMethod): void;

/**
*  Exports arbitrary client-side data, with client-side formatters applied, so is suitable 
*  for direct display to users. This method can be used to export data formatted outside
*  of any kind of visual component.
*  
*  Requires the SmartClient server, but does not rely on any server-side DataSources. If 
*  you need to intervene in the export process server-side - for example, if you need to
*  do something not directly supported with the exported object, such as attach it to an
*  email - use the DataSource.exportClientData with an 
*  appropriate OperationBinding, as described in the method documentation.
*  
*  To export unformatted data, see DataSource.exportData which does
*  not include client-side formatters, but requires both the SmartClient server and the 
*  presence of server-side DataSources.
*  
*  Note that field DataSourceField.displayFormat is honored for 
*  "date" and "datetime" fields when exporting direct to Excel; see the displayFormat
*  docs for details.
* 
* 
* @param {Array<Partial<Record>>} Array of Records to export
* @param {Partial<DSRequest>=} Request properties for the export
* @param {DSCallback=} Optional callback. Note that this is only applicable if you also specify DSRequest.exportToClient: false in the  request properties
*/
static exportClientData(data:Array<Partial<Record>>, requestProperties?:Partial<DSRequest>, callback?:DSCallback): void;

/**
*  Getter method for extracting server-side validation errors for an attempted
*  "update" or "add" operation, as a JS Object where each property name is a 
*  field name from the record and each property value is an array of error message strings.
*  For example:
*    {
*      userId : ["A user with this userId already exists"],
*      orderId : ["Must be a numeric value", "No Order with ID '6A18294' exists"]
*    }
*  
*  The Java API DSResponse.addError(fieldName, errorMessage) is used to send server-side errors
*  to the client. See the Java Server Reference for details.
* 
* 
* @param {DSResponse} response from which you want to extract the errors
*/
static getSimpleErrors(dsResponse:DSResponse): object;




}

/**
* A DataView coordinates the asynchronous loading of WSDL WebService and XML Schema
*  definitions in applications created by Reify.
*  
*  For applications that do not use WSDL Web Services and were not created by Reify,
*  DataView is equivalent to it's superclass VLayout.
*/
interface DataView extends VLayout {
/**
* Minimum size, in pixels, below which flexible-sized members should never be shrunk, even
*  if this requires the Layout to overflow. Note that this property only applies along
*  the length axis of the Layout, and has no affect on breadth.
*  
*  Does not apply to members given a fixed size in pixels - such members will never be
*  shrunk below their specified size in general.
* @type {Integer}
* @default "18"
*/
minMemberLength?: Integer;



/**
* 
*  Executed when the dataView has loaded all dependencies (such as DataSources or WebServices).
*  No default implementation.
* 
* 
*/
dataViewLoaded(): void;



}
/**
* Creates a new DataView
* 
* @param typeCheckedProperties {Partial<DataView>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DataView} The new instance
*/
class DataView{
static create(typeCheckedProperties? : Partial<DataView>, uncheckedProperties?: Object): DataView;







}

/**
* A component that connects to a database (and, depending on the RDBMS in use, optionally a 
*  particular catalog and/or schema) and displays the currently-defined tables, optionally 
*  filtered in a variety of ways. It also allows you to view the column details
*  of a selected table, and optionally retrieves and displays the data currently in that table.
*  
*  The DatabaseBrowser can also create a SmartClient DataSource from any existing SQL table.
*/
interface DatabaseBrowser extends Window {
/**
* The type of database server this DatabaseBrowser should connect to. Valid values are 
*  "sql" (SmartClient's own built-in support for SQL databases) or "hibernate"
* @type {string}
* @default "sql"
*/
serverType?: string;

/**
* The name of the database configuration to use. Database configurations are set up 
*  through the "Databases" tab of the Developer Console. If not set, defaults to the 
*  current default database configuration.
*  
*  Note that this property only applies where DatabaseBrowser.serverType is "sql"
* @type {string}
* @default null
*/
dbName?: string;

/**
* The name of the database catalog to use, if the RDBMS in use recognizes the concept of
*  multiple catalogs. Leave this property unset to include all catalogs.
*  
*  Note that this property only applies where DatabaseBrowser.serverType is "sql"
* @type {string}
* @default null
*/
catalog?: string;

/**
* The name of the database schema to use. Leave this property unset to include all 
*  schemata. Note that this property is not applicable to the MySQL database, which
*  does not recognize the concept of multiple schemata within a database.
*  
*  Note that this property only applies where DatabaseBrowser.serverType is "sql"
* @type {string}
* @default null
*/
schema?: string;

/**
* Instance of TreeGrid used to display the database schema (tables and columns)
* @type {ListGrid}
* @default null
*/
schemaTree?: ListGrid;

/**
* Instance of DynamicForm used to select a database from the list for display in the 
*  DatabaseBrowser.schemaTree.
* @type {ListGrid}
* @default null
*/
databaseList?: ListGrid;

/**
* Instance of ListGrid used to display the actual data in the table selected in the 
*  DatabaseBrowser.schemaTree.
* @type {ListGrid}
* @default null
*/
dataGrid?: ListGrid;

/**
* Instance of Button used to cancel this dialog.
* @type {Button}
* @default null
*/
cancelButton?: Button;

/**
* Instance of Button used to continue once a table has been selected.
* @type {Button}
* @default null
*/
selectButton?: Button;

/**
* A title to show in the header button of the DatabaseBrowser's schema tree. If not set,
*  defaults to the name of the connected database
* @type {string}
* @default null
*/
title?: string;

/**
* If set, specifies a substring which must exist in a table name for it to be included in
*  this DatabaseBrowser. If this property is set to a List of strings, table names are
*  included if they match any one of the strings. The comparison is case-insensitive.
*  
*  For example, includeSubstring: ["E", "qry"] would match all the following 
*  table names: "table", "QryTbl", "QRY", "ORDERS"
* @type {string | Array<Partial<string>>}
* @default null
*/
includeSubstring?: string | Array<Partial<string>>;

/**
* If set, specifies a substring which must NOT exist in a table name for it to be included in
*  this DatabaseBrowser. If this property is set to a List of strings, table names are
*  excluded if they match any one of the strings. The comparison is case-insensitive.
*  
*  For example, excludeSubstring: ["E", "qry"] would exclude all the following 
*  table names: "table", "QryTbl", "QRY", "ORDERS"
*  
*  Note that if you specify both include and exclude criteria and they conflict (ie,
*  according to the criteria you set, a table should be both included and excluded), 
*  exclude wins.
* @type {string | Array<Partial<string>>}
* @default null
*/
excludeSubstring?: string | Array<Partial<string>>;



/**
*  Returns the DataSource most recently auto-derived by this DatabaseBrowser. This will
*  correspond to the currently-selected table.
* 
*/
getGeneratedDataSource(): void;

/**
*  
* 
*/
getResults(): void;



}
/**
* Creates a new DatabaseBrowser
* 
* @param typeCheckedProperties {Partial<DatabaseBrowser>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DatabaseBrowser} The new instance
*/
class DatabaseBrowser{
static create(typeCheckedProperties? : Partial<DatabaseBrowser>, uncheckedProperties?: Object): DatabaseBrowser;







}

/**
* Simple interactive calendar interface used to pick a date.
*  Used by the DateItem class.
*/
interface DateChooser extends VLayout {
/**
* An AutoChild HLayout, rendered above the DateGrid,
*  and showing a number of widgets for navigating the DateChooser. These include buttons 
*  for moving to the previous DateChooser.previousYearButton or 
*  DateChooser.previousMonthButton, the next 
*  DateChooser.nextYearButton or DateChooser.nextMonthButton, 
*  and for selecting a specific DateChooser.yearChooserButton, 
*  DateChooser.monthChooserButton or 
*  DateChooser.weekChooserButton.
* @type {HLayout}
* @default null
*/
navigationLayout?: HLayout;

/**
* When editing a "date" value, with no time portion, clicking on a date-cell selects the 
*  date and closes the DateChooser. When a DateChooser.showTimeItem 
*  is required, however, the DateChooser.applyButton must be clicked
*  to close the chooser, by default.
*  
*  Set this attribute to true to have the DateChooser close when a user clicks in a 
*  date-cell, even if the DateChooser.timeItem is showing.
* @type {boolean}
* @default null
*/
closeOnDateClick?: boolean;

/**
* When set to true, show a button that allows the calendar to be navigated by fiscal year.
* @type {boolean}
* @default false
*/
showFiscalYearChooser?: boolean;

/**
* A button shown in the DateChooser.navigationLayout which, 
*  when clicked, shows a picker for selecting a specific fiscal year.
* @type {IButton}
* @default null
*/
fiscalYearChooserButton?: IButton;

/**
* When set to true, show a button that allows the calendar to be navigated by week or
*  fiscal week, depending on the value of DateChooser.showFiscalYearChooser.
* @type {boolean}
* @default false
*/
showWeekChooser?: boolean;

/**
* A button shown in the DateChooser.navigationLayout which shows
*  a picker for selecting a specific week of the year. When DateChooser.showFiscalYearChooser
*  is true, the week number represents a fiscal week number, one offset from the start of 
*  the fiscal year. Otherwise, it represents a week number offset from the start of the
*  calendar year.
* @type {IButton}
* @default null
*/
weekChooserButton?: IButton;

/**
* A button shown in the DateChooser.navigationLayout that shifts
*  the calendar view backward by a year.
* @type {IButton}
* @default null
*/
previousYearButton?: IButton;

/**
* StatefulCanvas.ariaLabel for the DateChooser.previousYearButton.
* @type {string}
* @default "Previous year"
*/
previousYearButtonAriaLabel?: string;

/**
* A button shown in the DateChooser.navigationLayout that shifts
*  the calendar view backward by a month.
* @type {IButton}
* @default null
*/
previousMonthButton?: IButton;

/**
* StatefulCanvas.ariaLabel for the DateChooser.previousMonthButton.
* @type {string}
* @default "Previous month"
*/
previousMonthButtonAriaLabel?: string;

/**
* A button shown in the DateChooser.navigationLayout that shows 
*  a picker for selecting a specific month.
* @type {IButton}
* @default null
*/
monthChooserButton?: IButton;

/**
* A button shown in the DateChooser.navigationLayout that shows 
*  a picker for selecting a specific calendar year.
* @type {IButton}
* @default null
*/
yearChooserButton?: IButton;

/**
* A button shown in the DateChooser.navigationLayout that shifts
*  the calendar view forward by a month.
* @type {IButton}
* @default null
*/
nextMonthButton?: IButton;

/**
* StatefulCanvas.ariaLabel for the DateChooser.nextMonthButton.
* @type {string}
* @default "Next month"
*/
nextMonthButtonAriaLabel?: string;

/**
* A button shown in the DateChooser.navigationLayout that shifts
*  the calendar view forward by a year.
* @type {IButton}
* @default null
*/
nextYearButton?: IButton;

/**
* StatefulCanvas.ariaLabel for the DateChooser.nextYearButton.
* @type {string}
* @default "Next year"
*/
nextYearButtonAriaLabel?: string;

/**
* An AutoChild HLayout, rendered below the DateGrid,
*  and showing the DateChooser.todayButton,
*  DateChooser.cancelButton and, when working with "datetime" values, 
*  DateChooser.applyButton buttons.
* @type {HLayout}
* @default null
*/
buttonLayout?: HLayout;

/**
* A ListGrid subclass, responsible for rendering the calendar view.
* @type {DateGrid}
* @default null
*/
dateGrid?: DateGrid;

/**
* A button shown below the DateGrid which, when clicked, 
*  navigates the calendar to today.
* @type {IButton}
* @default null
*/
todayButton?: IButton;

/**
* A button shown below the DateGrid which, when clicked, 
*  closes the DateChooser without selecting a value.
* @type {IButton}
* @default null
*/
cancelButton?: IButton;

/**
* When a DateChooser is configured for DateChooser.timeItem,
*  clicking on a date cell in the DateChooser.dateGrid will not automatically
*  dismiss the DateChooser canvas. In this case, use the Apply button to
*  accept the selected date and time and dismiss the chooser.
* @type {IButton}
* @default null
*/
applyButton?: IButton;

/**
* Height of the header area (containing the navigation buttons) in pixels.
* @deprecated * in favor of DateChooser.navigationLayoutHeight

* @type {Integer}
* @default "20"
*/
headerHeight?: Integer;

/**
* Height of the DateChooser.navigationLayout, containing the 
*  various buttons for navigating the DateChooser.dateGrid.
* @type {Integer}
* @default "20"
*/
navigationLayoutHeight?: Integer;

/**
* Overridden directory where images for this widget (such as the month and year button icons)
*  may be found.
* @type {SCImgURL}
* @default "images/common/"
*/
skinImgDir?: SCImgURL;

/**
* Icon for the previous year button
* @type {URL}
* @default "[SKIN]doubleArrow_left.gif"
*/
prevYearIcon?: URL;

/**
* Icon for the previous year button if Page.isRTL is true.
*  If not set, and the page is in RTL mode, the DateChooser.nextYearIcon will be
*  used in place of the DateChooser.prevYearIcon and vice versa.
* @type {URL}
* @default null
*/
prevYearIconRTL?: URL;

/**
* Width of the icon for the previous year button
* @type {Integer}
* @default "14"
*/
prevYearIconWidth?: Integer;

/**
* Height of the icon for the previous year button
* @type {Integer}
* @default "7"
*/
prevYearIconHeight?: Integer;

/**
* Icon for the previous month button
* @type {URL}
* @default "[SKIN]arrow_left.gif"
*/
prevMonthIcon?: URL;

/**
* Icon for the previous month button if Page.isRTL is true.
*  If not set, and the page is in RTL mode, the DateChooser.nextMonthIcon will be
*  used in place of the DateChooser.prevMonthIcon and vice versa.
* @type {URL}
* @default null
*/
prevMonthIconRTL?: URL;

/**
* Width of the icon for the previous month button
* @type {Integer}
* @default "7"
*/
prevMonthIconWidth?: Integer;

/**
* Height of the icon for the previous month button
* @type {Integer}
* @default "7"
*/
prevMonthIconHeight?: Integer;

/**
* Icon for the next year button
* @type {URL}
* @default "[SKIN]doubleArrow_right.gif"
*/
nextYearIcon?: URL;

/**
* Icon for the next year button if Page.isRTL is true.
*  If not set, and the page is in RTL mode, the DateChooser.nextYearIcon will be
*  used in place of the DateChooser.prevYearIcon and vice versa.
* @type {URL}
* @default null
*/
nextYearIconRTL?: URL;

/**
* Width of the icon for the next year button
* @type {Integer}
* @default "14"
*/
nextYearIconWidth?: Integer;

/**
* Height of the icon for the next year button
* @type {Integer}
* @default "7"
*/
nextYearIconHeight?: Integer;

/**
* Icon for the next month button
* @type {URL}
* @default "[SKIN]arrow_right.gif"
*/
nextMonthIcon?: URL;

/**
* Icon for the next month button
* @type {URL}
* @default null
*/
nextMonthIconRTL?: URL;

/**
* Width of the icon for the next month button if Page.isRTL is true.
*  If not set, and the page is in RTL mode, the DateChooser.nextMonthIcon will be
*  used in place of the DateChooser.prevMonthIcon and vice versa.
* @type {Integer}
* @default "7"
*/
nextMonthIconWidth?: Integer;

/**
* Height of the icon for the next month button
* @type {Integer}
* @default "7"
*/
nextMonthIconHeight?: Integer;

/**
* If this property is set to true the previous and next year buttons will render out the 
*  previous and next month button icons twice rather than using the
*  DateChooser.prevYearIcon and DateChooser.nextYearIcon.
*  
*  Set to true by default as not all skins contain media for the year icons.
* @type {boolean}
* @default true
*/
showDoubleYearIcon?: boolean;

/**
* Style for the pop-up year menu.
* @type {CSSStyleName}
* @default "dateChooserYearMenu"
*/
yearMenuStyle?: CSSStyleName;

/**
* Earliest year that may be selected. If this chooser was opened by a 
*  DateItem, the default is inherited from DateItem.startDate.
*  Otherwise, the default is 10 years before today.
*  
*  When opened from a RelativeDateItem, this property and 
*  DateChooser.endYear are defaulted to null, and the year-picker shows
*  years surrounding the current year, according to 
*  DateChooser.startYearRange and
*  DateChooser.endYearRange.
* @type {Integer}
* @default "2010"
*/
startYear?: Integer;

/**
* Latest year that may be selected. If this chooser was opened by a 
*  DateItem, the default is inherited from DateItem.endDate.
*  Otherwise, the default is 5 years after today.
*  
*  When opened from a RelativeDateItem, this property and 
*  DateChooser.startYear are defaulted to null, and the year-picker shows
*  years surrounding the current year, according to 
*  DateChooser.startYearRange and
*  DateChooser.endYearRange.
* @type {Integer}
* @default "2025"
*/
endYear?: Integer;

/**
* When DateChooser.startYear is unset, this is the years before today 
*  that will be available for selection in the year menu.
* @type {Integer}
* @default "30"
*/
startYearRange?: Integer;

/**
* When DateChooser.endYear is unset, this is the years after today that 
*  will be available for selection in the year menu.
* @type {Integer}
* @default "10"
*/
endYearRange?: Integer;

/**
* Style for the pop-up year menu.
* @type {CSSStyleName}
* @default "dateChooserMonthMenu"
*/
monthMenuStyle?: CSSStyleName;

/**
* Style for the pop-up week menu.
* @type {CSSStyleName}
* @default "dateChooserWeekMenu"
*/
weekMenuStyle?: CSSStyleName;

/**
* Array of members to show in the DateChooser.buttonLayout.
*  
*  The default value of buttonLayoutControls is an Array of Strings listing 
*  the standard buttons in their default order:
*  
*   buttonLayoutControls : ["todayButton", "cancelButton", "applyButton"]
*  
*  You can override buttonLayoutControls to change the order of the standard 
*  buttons. You can also omit standard buttons this way, although it's more efficient to
*  use the related "show" property if available (eg DateChooser.showTodayButton). 
*  
*  By embedding a Canvas directly in this list you can add arbitrary additional controls to
*  the buttonLayout. 
*  
*  Note that having added controls to buttonLayoutControls, you can still call APIs directly on
*  those controls to change their appearance, and you can also show() and hide() them if
*  they should not be shown in some circumstances.
*  
*  Tip: custom controls need to set layoutAlign:"center" to appear vertically centered.
* @type {Array<Partial<string>>}
* @default "(see below)"
*/
buttonLayoutControls?: Array<Partial<string>>;

/**
* Determines whether the "Today" button will be displayed, allowing the user to select 
*  the current date.
* @type {boolean}
* @default true
*/
showTodayButton?: boolean;

/**
* Determines whether the "Cancel" button will be displayed.
* @type {boolean}
* @default false
*/
showCancelButton?: boolean;

/**
* Determines whether the DateChooser.applyButton will be displayed.
* @type {boolean}
* @default null
*/
showApplyButton?: boolean;

/**
* Title for "Today" button.
* @type {string}
* @default "Today"
*/
todayButtonTitle?: string;

/**
* Title for the cancellation button.
* @type {string}
* @default "Cancel"
*/
cancelButtonTitle?: string;

/**
* Title for the DateChooser.applyButton button.
* @type {string}
* @default "Apply"
*/
applyButtonTitle?: string;

/**
* If set specifies a fixed height for the Today and Cancel buttons.
* @type {Integer}
* @default null
*/
todayButtonHeight?: Integer;

/**
* Whether it should be valid to pick a weekend day. If set to true, weekend days appear
*  in disabled style and cannot be picked. 
*  
*  Which days are considered weekends is controlled by DateChooser.weekendDays if
*  set or by DateUtil.weekendDays otherwise.
* @type {boolean}
* @default false
*/
disableWeekends?: boolean;

/**
* Whether weekend days should be shown. Which days are considered weekends is controlled 
*  by DateChooser.weekendDays if set or by DateUtil.weekendDays otherwise.
* @type {boolean}
* @default true
*/
showWeekends?: boolean;

/**
* Whether weekend days should be styled differently from weekdays. If false, suppresses
*  the custom DateChooser.baseWeekendStyle and DateChooser.weekendHeaderStyle,
*  instead using the DateChooser.baseWeekdayStyle and 
*  DateChooser.weekendHeaderStyle.
* @type {boolean}
* @default true
*/
styleWeekends?: boolean;

/**
* An array of integer day-numbers that should be considered to be weekend days by this
*  DateChooser instance. If unset, defaults to the set of days indicated 
*  DateUtil.weekendDays.
* @type {Array<Partial<Integer>>}
* @default null
*/
weekendDays?: Array<Partial<Integer>>;

/**
* Day of the week to show in the first column. 0=Sunday, 1=Monday, ..., 6=Saturday. The
*  default value for this attribute is picked up from the current locale and can also be 
*  altered system-wide with the DateUtil.setFirstDayOfWeek.
* @type {Integer}
* @default "0"
*/
firstDayOfWeek?: Integer;

/**
* Base CSS style applied to this picker's buttons. Will have "Over", "Selected" and "Down"
*  suffix appended as the user interacts with buttons.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined with the
*  base style to generate stateful cell styles in Grids.
* @type {CSSStyleName}
* @default "dateChooserButton"
*/
baseButtonStyle?: CSSStyleName;

/**
* Base CSS style applied to weekdays. Will have "Over", "Selected" and "Down"
*  suffix appended as the user interacts with buttons. Defaults to DateChooser.baseButtonStyle.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined with the
*  base style to generate stateful cell styles in Grids.
* @type {CSSStyleName}
* @default "dateChooserWeekday"
*/
baseWeekdayStyle?: CSSStyleName;

/**
* Base CSS style applied to weekends. Will have "Over", "Selected" and "Down"
*  suffix appended as the user interacts with buttons. Defaults to DateChooser.baseWeekdayStyle.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined with the
*  base style to generate stateful cell styles in Grids.
* @type {CSSStyleName}
* @default "dateChooserWeekend"
*/
baseWeekendStyle?: CSSStyleName;

/**
* Base CSS style applied to cells in the DateChooser.showFiscalYearChooser.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined with the
*  base style to generate stateful cell styles in Grids.
* @type {CSSStyleName}
* @default "dateChooserFiscalYearCell"
*/
baseFiscalYearStyle?: CSSStyleName;

/**
* Base CSS style applied to the header of the 
*  DateChooser.showFiscalYearChooser in the 
*  DateChooser.dateGrid.
* @type {CSSStyleName}
* @default null
*/
fiscalYearHeaderStyle?: CSSStyleName;

/**
* Base CSS style applied to cells in the DateChooser.showWeekChooser.
* @type {CSSStyleName}
* @default "dateChooserWeekCell"
*/
baseWeekStyle?: CSSStyleName;

/**
* Base CSS style applied to the header of the 
*  DateChooser.showWeekChooser in the
*  DateChooser.dateGrid.
* @type {CSSStyleName}
* @default null
*/
weekHeaderStyle?: CSSStyleName;

/**
* An array of Date instances that should be disabled if they appear in the calendar view.
* @type {Array<Partial<Date>>}
* @default null
*/
disabledDates?: Array<Partial<Date>>;

/**
* Base CSS style applied to weekday dates which have been DateChooser.disabledDates.
* @type {CSSStyleName}
* @default "dateChooserDisabledWeekday"
*/
disabledWeekdayStyle?: CSSStyleName;

/**
* Base CSS style applied to weekend dates which have been DateChooser.disabledDates.
* @type {CSSStyleName}
* @default "dateChooserDisabledWeekend"
*/
disabledWeekendStyle?: CSSStyleName;

/**
* CSS style applied to the Fiscal Year and Week columns for the currently selected week 
*  (the one being displayed in the DateChooser.showWeekChooser).
* @type {CSSStyleName}
* @default "dateChooserSelectedWeek"
*/
selectedWeekStyle?: CSSStyleName;

/**
* Whether alternating weeks should be drawn in alternating styles. If enabled, the cell style
*  for alternate rows will have DateChooser.alternateStyleSuffix appended to it.
* @type {boolean}
* @default null
*/
alternateWeekStyles?: boolean;

/**
* The text appended to the style name when using DateChooser.alternateWeekStyles.
* @type {string}
* @default "Dark"
*/
alternateStyleSuffix?: string;

/**
* CSS style applied to the day-of-week headers. By default this applies to all days of the 
*  week. To apply a separate style to weekend headers, set 
*  DateChooser.weekendHeaderStyle
* @type {CSSStyleName}
* @default "dateChooserButtonDisabled"
*/
headerStyle?: CSSStyleName;

/**
* Optional CSS style applied to the day-of-week headers for weekend days. If unset 
*  DateChooser.headerStyle will be applied to both weekdays and weekend days.
* @type {string}
* @default null
*/
weekendHeaderStyle?: string;

/**
* CSS style to apply to navigation buttons and date display at the top of the
*  component. If null, the CSS style specified in DateChooser.baseButtonStyle is used.
* @type {CSSStyleName}
* @default null
*/
baseNavButtonStyle?: CSSStyleName;

/**
* Constructor for navigation buttons at the top of the component.
* @type {SCClassName}
* @default "IButton"
*/
navButtonConstructor?: SCClassName;

/**
* CSS style to apply to the buttons at the bottom of the DateChooser ("Today" and
*  "Cancel"). If null, the CSS style specified in DateChooser.baseButtonStyle is used.
* @type {CSSStyleName}
* @default null
*/
baseBottomButtonStyle?: CSSStyleName;

/**
* When showing the DateChooser.showFiscalYearChooser, should firstDayOfWeek
*  be defaulted to the same day as the fiscal start date? If true and a fiscal year 
*  starts on a Tuesday, the calendar will display Tuesday to Monday from left to right.
* @type {boolean}
* @default true
*/
useFirstDayOfFiscalWeek?: boolean;

/**
* Should this dateChooser be dismissed if the user presses the Escape key?
* @type {boolean}
* @default false
*/
closeOnEscapeKeypress?: boolean;

/**
* TimeItem for editing the time portion of dates. Visible by default for fields 
*  of type "datetime" and can be controlled by setting DateChooser.showTimeItem.
* @type {TimeItem}
* @default null
*/
timeItem?: TimeItem;

/**
* Custom properties to apply to the DateChooser.timeItem used 
*  for editing the time portion of the date.
* @type {Partial<TimeItem>}
* @default null
*/
timeItemProperties?: Partial<TimeItem>;

/**
* Whether to show the DateChooser.timeItem for editing the time portion
*  of the date. When unset, the time field is shown automatically if the field type is
*  "datetime". Note that the item's DateChooser.showSecondItem is 
*  not shown by default.
* @type {boolean}
* @default null
*/
showTimeItem?: boolean;

/**
* Title for the DateChooser.timeItem.
* @type {string}
* @default "Time"
*/
timeItemTitle?: string;

/**
* When showing the DateChooser.showTimeItem, whether the 
*  TimeItem should be set to use 24-hour time. The default is true.
* @type {boolean}
* @default true
*/
use24HourTime?: boolean;

/**
* Title for the DateChooser.showFiscalYearChooser field in the date grid.
* @type {string}
* @default "Year"
*/
fiscalYearFieldTitle?: string;

/**
* Title for the DateChooser.showWeekChooser field in the date grid.
* @type {string}
* @default "Wk"
*/
weekFieldTitle?: string;

/**
* When showing the DateChooser.timeItem, whether to show the "second" 
*  picker. When unset, the second field is not shown.
* @type {boolean}
* @default null
*/
showSecondItem?: boolean;

/**
* How long (how many characters) should be day names be. May be 1, 2 or 3 characters.
* @type {number}
* @default "2"
*/
dayNameLength?: number;



/**
*  Set the picker to show the given date.
*  
* 
* @param {Date} new value
*/
setData(date:Date): void;

/**
*  Get the current value of the picker.
*  
*  See DateChooser.dataChanged for how to respond to the user picking a date.
* 
* 
*/
getData(): Date;

/**
*  Returns the FiscalCalendar object that will be used by this DateChooser.
* 
* 
*/
getFiscalCalendar(): FiscalCalendar;

/**
*  Sets the FiscalCalendar object that will be used by this DateChooser. If unset,
*  the DateUtil.getFiscalCalendar is used.
* 
* 
* @param {FiscalCalendar=} the fiscal calendar for this chooser
*/
setFiscalCalendar(fiscalCalendar?:FiscalCalendar): void;

/**
*  Override this method to alter the year representations that are shown in the DateChooser's
*  "Select a year" dropdown. The default implementation returns the full four-digit 
*  Gregorian year (ie, the same value that is passed in)
* 
* 
* @param {Integer} The Gregorian year number to derive a display value for
*/
getYearTitle(year:Integer): string;

/**
*  Override this method to alter the year representation shown in the DateChooser's header.
*  The default implementation returns the full four-digit Gregorian year (ie, the same 
*  value that is passed in)
* 
* 
* @param {Integer} The Gregorian year number to derive a display value for
*/
getHeaderYearTitle(year:Integer): string;

/**
*  Method to override or observe in order to be notified when a user picks a date value.
*  
*  
*  Has no default behavior (so no need to call Super).
*  
*  Use DateChooser.getData to get the current date value.
*  
* 
*/
dataChanged(): void;

/**
*  Fired when the user clicks the cancel button in this date chooser. Default implementation
*  clears the date chooser.
* 
*/
cancelClick(): void;

/**
*  Fired when the user clicks the Today button. Default implementation will select the current
*  date in the date chooser.
* 
*/
todayClick(): void;



}
/**
* Creates a new DateChooser
* 
* @param typeCheckedProperties {Partial<DateChooser>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DateChooser} The new instance
*/
class DateChooser{
static create(typeCheckedProperties? : Partial<DateChooser>, uncheckedProperties?: Object): DateChooser;







}

/**
* A ListGrid subclass that manages calendar views.
*/
interface DateGrid extends ListGrid {




}
/**
* Creates a new DateGrid
* 
* @param typeCheckedProperties {Partial<DateGrid>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DateGrid} The new instance
*/
class DateGrid{
static create(typeCheckedProperties? : Partial<DateGrid>, uncheckedProperties?: Object): DateGrid;







}

/**
* A FormItem for editing DateUtil.createLogicalDate values, where 
*  times are ignored.
*  
*  The item renders with one of two appearances, depending on the value of DateItem.useTextField 
*  - when set to true, dates are edited directly DateItem.textField, and formatted 
*  according to your locale and settings such as DateItem.dateFormatter.
*  
*  When set to false, the default appearance, the item uses separate selectors for 
*  DateItem.yearSelector, DateItem.monthSelector and 
*  DateItem.daySelector values. To control which selectors are visible and in what 
*  order, use DateItem.selectorFormat. In this mode, the selectable dates may be 
*  limited by the item's DateItem.startDate and DateItem.endDate 
*  dates. See those two settings for more information.
*  
*  In either mode, a DateChooser is provided assuming that the 
*  pickerIcon is visible. 
*  
*  As noted, this item is for editing DateUtil.createLogicalDate.
*  To edit DateUtil.createLogicalTime, see TimeItem,
*  and to edit DateUtil.createDatetime, see DateTimeItem.
*  For RelativeDateString, see RelativeDateItem.
*  
*  For detailed information on working with dates, times and datetimes, see the 
*  dateFormatAndStorage.
*/
interface DateItem extends FormItem {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "DateItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Text field to hold the entire date in "type in" format, if 
*  DateItem.useTextField is true.
* @type {TextItem}
* @default null
*/
textField?: TextItem;

/**
* Custom properties to apply to this dateItem's generated DateItem.textField.
*  Only applies if DateItem.useTextField is true.
* @type {Partial<TextItem>}
* @default null
*/
textFieldProperties?: Partial<TextItem>;

/**
* If DateItem.useTextField is true and browserInputType is set to
*  "date", then a native HTML5 date input
*  is used in place of a text input.
*  
*  The use of a native HTML5 date input causes certain features to be disabled. Input masks,
*  the picker icon, and a custom DateItem.dateFormatter are not supported.
*  DateItem.showHintInField are currently supported, but future browser
*  changes might force this support to be removed. Therefore, it is safest to not
*  use in-field hints (set showHintInField to false) in conjunction with a native HTML5 date
*  input.
*  
*  NOTE: For optimal appearance this feature requires specific CSS which may
*  not be present in certain legacy skins.
* @type {string}
* @default null
*/
browserInputType?: string;

/**
* If this item is showing a FormItem.hint, should the hint text be allowed to
*  wrap? Setting this property to false will render the hint on a single line
*  without wrapping, expanding the width required to render the item if necessary.
*  
*  If unset this property will be picked up from the DynamicForm.wrapHintText
*  setting.
*  
*  This setting does not apply to hints that are TextItem.showHintInField.
* @type {boolean}
* @default false
*/
wrapHintText?: boolean;

/**
* If DateItem.useTextField is true and a FormItem.hint is
*  set, should the hint be shown within the field?
*  
*  Note that when using a native HTML5 date input (see DateItem.browserInputType),
*  in-field hints are currently supported, but future browser changes might not allow
*  in-field hints to be supported. Therefore, it is safest to not use in-field
*  hints in conjunction with a native HTML5 date input.
*  
*  To change this attribute after being drawn, it is necessary to call FormItem.redraw
*  or redraw the form.
* @type {boolean}
* @default null
*/
showHintInField?: boolean;

/**
* If DateItem.showHintInField and if supported by the browser, should the HTML5
*  placeholder attribute
*  be used to display the hint within the field? If set to false, then use of
*  the placeholder attribute is disabled and an alternative technique to display
*  the hint in-field is used instead.
*  
*  The HTML5 placeholder attribute is supported in the following major browsers:
*  
*  
* - Chrome 4+
*  
* - Firefox 4+
*  
* - Internet Explorer 10+
*  
* - Safari 5+
*  
* - Opera 11.50+
*  
* - Android 2.1+ WebView (used by the stock Browser app and when
*    phonegapIntegration)
*  
* - Mobile Safari for iOS 3.2+
*  
*  
*  In browsers other than the above, in-field hints are implemented via a different technique.
*  
*  Note that placeholder behavior is known to differ in Internet Explorer and certain old
*  versions of the above browsers due to a recent change in the HTML5 specification regarding
*  the placeholder attribute. Under the old rules, the placeholder is cleared
*  when the element is focused. In the latest HTML5 spec as published by WHATWG, the placeholder
*  is still displayed when the element is focused as long as the value is an empty string.
*  
*  Styling the placeholder
*  While there isn't a standard way to style the placeholder text, Chrome, Firefox,
*  Internet Explorer, and Safari provide vendor-prefixed pseudo-classes and/or pseudo-elements
*  that can be used in CSS selectors:
*  
*  
*   Browser
*   Pseudo-class or pseudo-element
*  
*  
*   Chrome, Safari
*   ::-webkit-input-placeholder
*  
*  
*   Firefox 4 - 18
*   :-moz-placeholder
*  
*  
*   Firefox 19+
*   ::-moz-placeholder
*  
*  
*   Internet Explorer
*   :-ms-input-placeholder
*  
*  
*  
*  Note that unlike other browsers, Firefox 19+ applies opacity:0.4 to the placeholder text.
*  See Bug 556145 - Placeholder text default style should use opacity instead of GrayText
*  
*  Because browsers are required to ignore the entire rule if a selector is invalid,
*  separate rules are needed for each browser. For example:
*  ::-webkit-input-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
* :-moz-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
* ::-moz-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
* :-ms-input-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
*  
*  If using Sass, it may be useful to utilize Sass'
*  parent selector feature.
*  For example:
*  .myCustomItem,
* .myCustomItemRTL,
* .myCustomItemDisabled,
* .myCustomItemDisabledRTL,
* .myCustomItemError,
* .myCustomItemErrorRTL,
* .myCustomItemFocused,
* .myCustomItemFocusedRTL,
* .myCustomItemHint,
* .myCustomItemHintRTL,
* .myCustomItemDisabledHint,
* .myCustomItemDisabledHintRTL {
*   // ...
* 
*   &amp;::-webkit-input-placeholder {
*     color: blue;
*     opacity: 1;
*   }
*   &amp;:-moz-placeholder {
*     color: blue;
*     opacity: 1;
*   }
*   &amp;::-moz-placeholder {
*     color: blue;
*     opacity: 1;
*   }
*   &amp;:-ms-input-placeholder {
*     color: blue;
*     opacity: 1;
*   }
* &#125;
*  
*  If using Compass, the
*  input-placeholder mixin
*  that was added in version 1.0 can further simplify the code to style the placeholder text
*  For example:
*  .myCustomItem,
* .myCustomItemRTL,
* .myCustomItemDisabled,
* .myCustomItemDisabledRTL,
* .myCustomItemError,
* .myCustomItemErrorRTL,
* .myCustomItemFocused,
* .myCustomItemFocusedRTL,
* .myCustomItemHint,
* .myCustomItemHintRTL,
* .myCustomItemDisabledHint,
* .myCustomItemDisabledHintRTL {
*   // ...
* 
*   &#64;include input-placeholder {
*     color: blue;
*     opacity: 1;
*   }
* &#125;
*  Accessibility concerns
*  The HTML5 specification notes that a placeholder should not be used as a replacement
*  for a title. The placeholder is intended to be a short hint that assists the user
*  who is entering a value into the empty field. The placeholder can be mistaken by some
*  users for a pre-filled value, particularly in Internet Explorer because the same color
*  is used, and the placeholder text color may provide insufficient contrast, particularly
*  in Firefox 19+ because of the default 0.4 opacity. Furthermore, not having a title reduces
*  the hit area available for setting focus on the item.
* @type {boolean}
* @default true
*/
usePlaceholderForHint?: boolean;

/**
* When DateItem.useTextField is false, whether titles should be shown
*  for for child-items in this DateItem. By default, showItemTitles is false.
* @type {boolean}
* @default false
*/
showItemTitles?: boolean;

/**
* When DateItem.useTextField is false, the default orientation of 
*  titles for the DateItem.daySelector, DateItem.monthSelector 
*  and DateItem.yearSelector selectors.
*  TitleOrientation lists valid options.
*  
*  Note that titles on the left or right take up a cell in tabular
*  formLayout, but titles on top do not.
* @type {TitleOrientation}
* @default "top"
*/
itemTitleOrientation?: TitleOrientation;

/**
* When DateItem.useTextField is false, the default alignment of 
*  titles for the DateItem.daySelector, DateItem.monthSelector 
*  and DateItem.yearSelector selectors, within their cells.
* @type {Alignment}
* @default "center"
*/
itemTitleAlign?: Alignment;

/**
* SelectItem for picking a day of the month.
*  
*  To control which selectors are visible and in what order, use DateItem.selectorFormat.
* @type {SelectItem}
* @default null
*/
daySelector?: SelectItem;

/**
* Custom properties to apply to this dateItem's generated DateItem.daySelector.
* @type {Partial<SelectItem>}
* @default null
*/
daySelectorProperties?: Partial<SelectItem>;

/**
* SelectItem for picking a month.
*  
*  To control which selectors are visible and in what order, use DateItem.selectorFormat.
* @type {SelectItem}
* @default null
*/
monthSelector?: SelectItem;

/**
* Custom properties to apply to this dateItem's generated DateItem.monthSelector.
* @type {Partial<SelectItem>}
* @default null
*/
monthSelectorProperties?: Partial<SelectItem>;

/**
* SelectItem for picking a year.
*  
*  To control which selectors are visible and in what order, use DateItem.selectorFormat.
* @type {SelectItem}
* @default null
*/
yearSelector?: SelectItem;

/**
* Custom properties to apply to this dateItem's generated DateItem.yearSelector.
* @type {Partial<SelectItem>}
* @default null
*/
yearSelectorProperties?: Partial<SelectItem>;

/**
* When set to true (the default), use a single shared date-picker across all widgets that
*  use one. When false, create a new picker using the autoChild system. See 
*  DateItem.pickerDefaults and 
*  DateItem.pickerProperties for details on setting up an unshared
*  picker.
* @type {boolean}
* @default true
*/
useSharedPicker?: boolean;

/**
* SmartClient class for the FormItem.picker autoChild displayed to allow the user
*  to directly select dates.
* @type {string}
* @default "DateChooser"
*/
pickerConstructor?: string;

/**
* Defaults for the DateChooser created by this form item. The picker for a particular
*  item may be further customized via DateItem.pickerProperties.
*  
*  By default the following DateChooser properties are set:
*  
* - border:"1px solid black"
*    
* - showTodayButton:true
*    
* - showCancelButton:true
*    
* - autoHide:true
*    
* - closeOnEscapeKeypress:true
*  
*  These may be modified or overridden by the loaded skin. Note that as with any
*  defaults block, modifications should be made using the the 
*  Class.changeDefaults to apply changes on top of existing settings.
* @type {Partial<DateChooser>}
* @default "{...}"
*/
pickerDefaults?: Partial<DateChooser>;

/**
* Properties for the DateChooser created by this form item. If specified these will
*  be applied on top of the DateItem.pickerDefaults
* @type {Partial<DateChooser>}
* @default null
*/
pickerProperties?: Partial<DateChooser>;

/**
* When set to true, the item uses a DateItem.textField for 
*  working with the item's value.
*  
*  When false or unset, the default, the item's value is represented by separate 
*  DateItem.daySelector, DateItem.monthSelector, 
*  and/or DateItem.yearSelector selectors. In this mode, null values are not
*  supported, and a default value of Today will be enforced if no 
*  FormItem.defaultValue is specified. This means that a DateItem 
*  with useTextField set to false is effectively a 
*  FormItem.required field.
*  
*  If you want to change the appearance of a DateItem, you will need to configure some
*  autoChildren such as DateItem.textField via DateItem.textFieldProperties,
*  or in useTextField:false mode, the DateItem.daySelector and other selectors, 
*  configured via daySelectorProperties et al.
* @type {boolean}
* @default null
*/
useTextField?: boolean;

/**
* When set to true, the default, and when DateItem.useTextField is 
*  set to false, such that the item displays multiple separate pickers, 
*  useTextField is automatically switched
*  on when the item is rendering in a non-interactive way, such as when printing, or when 
*  FormItem.canEdit is false and the read-only display-mode is 
*  FormItem.readOnlyDisplay.
* @type {boolean}
* @default true
*/
autoUseTextField?: boolean;

/**
* If DateItem.useTextField is true this property governs the alignment
*  of text within the text field. Defaults to "right" by default or
*  "left" if the page is in Page.isRTL.
*  
*  This attribute does not have an effect if a native HTML5 date input is being used.
*  See DateItem.browserInputType.
* @type {Alignment}
* @default "varies"
*/
textAlign?: Alignment;

/**
* If DateItem.useTextField is not false this property determines if
*  an input mask should be used. The format of the mask is determined by the 
*  DateItem.inputFormat or DateItem.dateFormatter (in that order).
*  
*  This attribute does not have an effect if a native HTML5 date input is being used.
*  See DateItem.browserInputType.
* @type {boolean}
* @default null
*/
useMask?: boolean;

/**
* If DateItem.useTextField and DateItem.useMask are both true
*  this value is the separator between date components. If unset 
*  DateUtil.getDefaultDateSeparator will be used.
* @type {string}
* @default null
*/
maskDateSeparator?: string;

/**
* Can this field be set to a non-date value [other than null]?
*  
*  When set to true, FormItem.setValue will return false without setting the item value
*  and log a warning if passed something other than a valid date value.
*  If the dateItem is showing a DateItem.useTextField,
*  and a user enters a text value which cannot be parsed into a valid date, the item will
*  automatically redraw and display the DateItem.invalidDateStringMessage (though at this
*  point calling FormItem.getValue will return the string entered by the user).
*  
*  When set to false, a user may enter a value that is not a valid date (for example, "Not
*  applicable") and the value will not immediately be flagged as an error. However note
*  that for the value to actually pass validation you would need to declare the field as
*  not of "date" type, for example:
*  
*    {name:"startDate", type:"dateOrOther", editorType:"DateItem", useTextField:true },
*  
*  The type "dateOrOther" could be declared as a SimpleType, with validators that
*  will accept either a valid date or certain special Strings (like "Not Available").
*  
*  Only applies to dateItems where DateItem.useTextField is true. Non-Date values
*  are never supported in items where useTextField is false.
*  
*  This attribute does not have an effect if a native HTML5 date input is being used.
*  See DateItem.browserInputType.
* @type {boolean}
* @default false
*/
enforceDate?: boolean;

/**
* Dictates whether values applied to this item via setValue() or
*  DynamicForm.values will be accepted if they fall outside the range 
*  specified by the item's DateItem.startDate and 
*  DateItem.endDate dates.
*  
*  When set to false, values outside the valid range will be accepted, which may result in
*  additional entries being added to the various pickers, when 
*  DateItem.useTextField is false.
*  
*  When set to true, FormItem.setValue will return false for values that fall 
*  outside the range, the value will be rejected and the item defaulted to the start of 
*  its defined range. When this happens, 
*  FormItem.change will not fire, the item will not show the 
*  FormItem.showPending, and 
*  DynamicForm.valuesHaveChanged will return false, even 
*  though calling DynamicForm.saveData will result in a changed record, 
*  if the FormItem.form is DynamicForm.dataSource 
*  and the current record came from the dataSource. 
*  
*  This attribute does not have an effect if a native HTML5 date input is being used.
*  See DateItem.browserInputType.
* @type {boolean}
* @default false
*/
enforceValueRange?: boolean;

/**
* Validation error message to display if the user enters an invalid date
* @type {string}
* @default "Invalid date"
*/
invalidDateStringMessage?: string;

/**
* Prompt to show when the user hovers the mouse over the picker icon for this DateItem. May
*  be overridden for localization of your application.
* @type {HTMLString}
* @default "Show Date Chooser"
*/
pickerIconPrompt?: HTMLString;

/**
* Minimum date the selectors will allow the user to pick. The default value is January 
*  1st, 10 years before the current year.
*  
*  NOTE: by design, setting startDate and endDate will not
*  always prevent the user from picking invalid values. In particular:
*  
*  
* - the set of available days will only be restricted if the start and end dates fall
*  within the same month
*  
* - the set of available months will only be restricted if the start and end dates fall
*  within the same year
*  
*  
*  This is by design as it allows the user to set the day, month and year in
*  whatever order is convenient, rather than forcing them to pick in a specific order.
*  
*  For actual enforcement of a date being in correct range before data is submitted, a
*  Validator of type "dateRange" should always be declared.
* @type {Date}
* @default "See below"
*/
startDate?: Date;

/**
* Maximum date the selectors will allow the user to pick. The default value is December 
*  31st, 5 years after the current year.
*  
*  See DateItem.startDate for details on how this restriction works.
* @type {Date}
* @default "See below"
*/
endDate?: Date;

/**
* Only used if we're showing the date in a text field. When parsing a date, if the year
*  is specified with 1 or 2 digits and is less than the centuryThreshold, then the year will
*  be assumed to be 20xx; otherwise it will be interpreted according to default browser
*  behavior, which will consider it to be 19xx.
*  
*  By default, the centuryThreshold is calculated as the current year + 25.
*  
*  If you need to allow 1 and 2 digit years, set this attribute to 
*  null to have the control retain your year-value as entered.
* @type {number}
* @default "25"
*/
centuryThreshold?: number;

/**
* When showing the DateChooser and the field is of type "datetime", whether
*  the DateChooser.showTimeItem should be set to use 24-hour time. The 
*  default is true.
*  
*  Has no effect if DateItem.showPickerTimeItem is explicitly set to false.
* @type {boolean}
* @default true
*/
use24HourTime?: boolean;

/**
* If this field is of type "datetime", when showing the
*  DateChooser, should the DateChooser.showTimeItem be
*  displayed?
*  
*  Has no effect for fields of type "date".
*  
*  This attribute does not have an effect if a native HTML5 date input is being used.
*  See DateItem.browserInputType.
* @type {boolean}
* @default true
*/
showPickerTimeItem?: boolean;

/**
* A set of properties to apply to the TimeItem displayed in the picker when
*  DateItem.showPickerTimeItem is true.
*  
*  Has no effect for fields of type "date".
* @type {Partial<TimeItem>}
* @default null
*/
pickerTimeItemProperties?: Partial<TimeItem>;

/**
* If DateItem.useTextField is true this property can be used to 
*  customize the format in which dates are displayed for this item.
*  Should be set to a standard DateDisplayFormat.
*  
*  As with any formItem rendering out a date value, if no explicit dateFormatter is
*  supplied, dateFormatter will be derived from DynamicForm.dateFormatter or
*  DynamicForm.datetimeFormatter, depending on the specified FormItem.type for
*  this field, if set, otherwise from the standard default 
*  DateUtil.setShortDisplayFormat or DateUtil.setShortDatetimeDisplayFormat.
*  
*  NOTE: For entirely custom formats, developers may apply a custom
*  
*  DateItem.formatEditorValue method. To ensure the
*  DateItem is able to parse user-entered date strings back into Dates, for most cases
*  developers can specify an explicit DateItem.inputFormat, or if necessary a
*  custom DateItem.parseEditorValue.
*  
*  
*  
*  This attribute does not have an effect if a native HTML5 date or datetime input is being used.
*  See DateItem.browserInputType.
* @type {DateDisplayFormat}
* @default null
*/
dateFormatter?: DateDisplayFormat;

/**
* If DateItem.useTextField is true this property can be used to 
*  customize the format in which dates are displayed.
*  Should be set to a standard DateDisplayFormat or
*  a function which will return a formatted date string.
*  
*  If unset, the standard shortDate format as set up via 
*  DateUtil.setShortDisplayFormat will be used.
*  
*  NOTE: you may need to update the DateItem.inputFormat to ensure the
*  DateItem is able to parse user-entered date strings back into Dates
*  
*  This attribute does not have an effect if a native HTML5 date input is being used.
*  See DateItem.browserInputType.
* @deprecated * Use DateItem.dateFormatter instead.

* @type {DateDisplayFormat}
* @default null
*/
displayFormat?: DateDisplayFormat;

/**
* If DateItem.useTextField is true this property can be used to specify
*  the input format for date strings. 
*  If unset, the input format will be determined based on the specified
*  DateItem.dateFormatter if possible (see DateItem.getInputFormat), otherwise
*  picked up from the Date class (see DateUtil.setInputFormat).
*  
*  Should be set to a standard DateInputFormat
*  
*  Note that the DateInputFormat property is sufficient to parse date or datetime
*  strings specified in most standard date formats. However should an entirely custom
*  parsing function be required developers can 
*  implement a custom DateItem.parseEditorValue method.
*  
*  
*  This attribute does not have an effect if a native HTML5 date input is being used.
*  See DateItem.browserInputType.
* @type {DateInputFormat}
* @default null
*/
inputFormat?: DateInputFormat;

/**
* If showing date selectors rather than the date text field (so when 
*  this.useTextField is false), this property allows customization of the 
*  order of the day, month and year selector fields. If unset, these fields will match the
*  specified DateItem.inputFormat for this item, but note that the attribute default will
*  likely be set by i18n to a valid DateItemSelectorFormat.
*  
*  Note: selectors may be omitted entirely by setting selectorFormat to (for example) 
*  "MD". In this case the value for the omitted selector will match the
*  FormItem.defaultValue specified for the item. For example,
*  if the selector format is "MD" (month and day only), the year comes from the Date
*  specified as the defaultValue.
* @type {DateItemSelectorFormat}
* @default "varies"
*/
selectorFormat?: DateItemSelectorFormat;

/**
* Default date to show in the date chooser. If this items value is currently unset,
*  this property may be specified to set a default date to highlight in the dateChooser 
*  for this item. If unset, the date chooser will highlight the current date by default.
*  Note that this has no effect if the item as a whole currently has a value - in that
*  case the date chooser will always highlight the current value for the item.
* @type {Date}
* @default null
*/
defaultChooserDate?: Date;

/**
* When set to true, show a button that allows the calendar to be navigated by fiscal year.
* @type {boolean}
* @default false
*/
showChooserFiscalYearPicker?: boolean;

/**
* When set to true, show a button that allows the calendar to be navigated by week or
*  fiscal week, depending on the value of DateItem.showChooserFiscalYearPicker.
* @type {boolean}
* @default false
*/
showChooserWeekPicker?: boolean;



/**
*  Notification method called when FormItem.showPending is enabled and
*  this date item should either clear or show its pending visual state.
*  
*  The default behavior is that the FormItem.titleStyle and
*  FormItem.cellStyle are updated to include/exclude the "Pending" suffix.
*  In addition, when displayed in the pending state:
*  
*  
* - If DateItem.useTextField is true, then the "Pending"
*    suffix will be appended to the FormItem.textBoxStyle applied to the
*    DateItem.textField; otherwise
*  
* - (useTextField is false) the color of the
*    DateItem.daySelector, DateItem.monthSelector
*    and/or DateItem.yearSelector will change when the day, month, or year
*    is different, respectively.
*  
*  Returning false will cancel this default behavior.
* 
* @param {DynamicForm} the managing DynamicForm instance.
* @param {FormItem} the form item itself (also available as "this").
* @param {boolean} true if the item should show its pending visual state; false otherwise.
* @param {any} the current form item value.
* @param {any} the value that would be restored by a call to DynamicForm.resetValues.
*/
pendingStatusChanged(form:DynamicForm, item:FormItem, pendingStatus:boolean, newValue:any, value:any): boolean;

/**
*  Returns the raw text value typed into this items text field if DateItem.useTextField 
*  is true (otherwise returns the result of this.getValue()).
* 
*/
getEnteredValue(): string;

/**
*  Setter for DateItem.startDate. 
*  Note: A DateUtil.createLogicalDate is expected. 
* 
* @param {Date | string} the new startDate.
*/
setStartDate(startDate:Date | string): void;

/**
*  Setter for DateItem.endDate.
*  Note: A DateUtil.createLogicalDate is expected. 
* 
* @param {Date | string} the new endDate.
*/
setEndDate(endDate:Date | string): void;

/**
*  If DateItem.useTextField is true, falls through to standard
*  TextItem.setSelectionRange implementation on this items freeform text entry field.
*  Otherwise has no effect.
* 
* @param {Integer} character index for start of new selection
* @param {Integer} character index for end of new selection
*/
setSelectionRange(start:Integer, end:Integer): void;

/**
*  If DateItem.useTextField is true, falls through to standard
*  TextItem.getSelectionRange implementation on this items freeform text entry field.
*  Otherwise has no effect.
* 
*/
getSelectionRange(): Array<any>;

/**
*  If DateItem.useTextField is true, falls through to standard
*  TextItem.selectValue implementation on this items freeform text entry field.
*  Otherwise has no effect.
* 
*/
selectValue(): void;

/**
*  If DateItem.useTextField is true, falls through to standard
*  TextItem.deselectValue implementation on this items freeform text entry field.
*  Otherwise has no effect.
* 
* @param {boolean=} If this parameter is passed, new cursor insertion position will be  moved to the start, rather than the end of this item's value.
*/
deselectValue(start?:boolean): void;

/**
*  Convert a text value entered in this item's text field to a final data value 
*  for storage.
*  
*  If DateItem.useTextField is true, entirely custom date formatting and
*  parsing logic may be applied via overrides to DateItem.parseEditorValue and
*  DateItem.formatEditorValue. These methods apply to this FormItem only - 
*  system-wide Date and Datetime formatting and parsing may also be customized via
*  the APIs on the Date class. See dateFormatAndStorage for more
*  on this.
*  
*  Note: custom parsing for this item may also be achieved by modifying the
*  DateItem.inputFormat. This mechanism provides support many common date formats
*  without the need for an entirely custom parser function.
* 
* 
* @param {string} value as entered by the user
* @param {DynamicForm} pointer to the dynamicForm containing this item
* @param {FormItem} pointer to this item
*/
parseEditorValue(value:string, form:DynamicForm, item:FormItem): any;

/**
*  Convert this item's data value to a text value for display in this item's
*  text field.
*  
*  If DateItem.useTextField is true, entirely custom date formatting and
*  parsing logic may be applied via overrides to DateItem.parseEditorValue and
*  DateItem.formatEditorValue. These methods apply to this FormItem only - 
*  system-wide Date and Datetime formatting and parsing may also be customized via
*  the APIs on the Date class. See dateFormatAndStorage for more
*  on this.
*  
*  Note: custom formatting for this item may also be achieved via the
*  DateItem.dateFormatter which allows you to directly specify various standard
*  date display formats.
* 
* @param {any} Underlying data value to format. May be null.
* @param {ListGridRecord} The record currently being edited by this form.   Essentially the form's current values object.
* @param {DynamicForm} pointer to the DynamicForm
* @param {FormItem} pointer to the FormItem
*/
formatEditorValue(value:any, record:ListGridRecord, form:DynamicForm, item:FormItem): string;

/**
*  If DateItem.useTextField is true this method returns a
*  standard DateInputFormat, determining how values entered
*  by the user are to be converted to Javascript Date objects.
*  
*  If an explicit DateItem.inputFormat has been specified it will be returned,
*  otherwise, if a custom DateItem.dateFormatter or FormItem.format are
*  specified, the input format will be automatically derived from that property.
*  
*  Otherwise, the global DateUtil.setInputFormat is used.
*  
*  Note that the inputFormat will ignore any separator characters and padding of values.
*  However if necessary entirely custom date formatting and parsing may be achieved via the
*  DateItem.formatEditorValue and
*  DateItem.parseEditorValue methods.
*  
*  
* 
*/
getInputFormat(): DateInputFormat;

/**
*  Returns the FiscalCalendar object that will be used by this item's DateChooser.
* 
* 
*/
getFiscalCalendar(): FiscalCalendar;

/**
*  Sets the FiscalCalendar object that will be used by this item's DateChooser. If 
*  unset, the DateUtil.getFiscalCalendar is used.
* 
* 
* @param {FiscalCalendar=} the fiscal calendar for this chooser, if set, or the global      one otherwise
*/
setFiscalCalendar(fiscalCalendar?:FiscalCalendar): void;

/**
*  Returns the default date to display in the date chooser if this form items value is
*  currently unset.
*  
*  Default implementation returns DateItem.defaultChooserDate
* 
*/
getDefaultChooserDate(): Date;



}
/**
* Creates a new DateItem
* 
* @param typeCheckedProperties {Partial<DateItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DateItem} The new instance
*/
class DateItem{
static create(typeCheckedProperties? : Partial<DateItem>, uncheckedProperties?: Object): DateItem;
/**
* A declared value of the enum type 
*  DateItemSelectorFormat.
* @type {Constant}
* @default "DMY"
*/
static DAY_MONTH_YEAR?: Constant;

/**
* A declared value of the enum type 
*  DateItemSelectorFormat.
* @type {Constant}
* @default "MDY"
*/
static MONTH_DAY_YEAR?: Constant;

/**
* A declared value of the enum type 
*  DateItemSelectorFormat.
* @type {Constant}
* @default "YMD"
*/
static YEAR_MONTH_DAY?: Constant;

/**
* A declared value of the enum type 
*  DateItemSelectorFormat.
* @type {Constant}
* @default "DM"
*/
static DAY_MONTH?: Constant;

/**
* A declared value of the enum type 
*  DateItemSelectorFormat.
* @type {Constant}
* @default "MD"
*/
static MONTH_DAY?: Constant;

/**
* A declared value of the enum type 
*  DateItemSelectorFormat.
* @type {Constant}
* @default "YM"
*/
static YEAR_MONTH?: Constant;

/**
* A declared value of the enum type 
*  DateItemSelectorFormat.
* @type {Constant}
* @default "MY"
*/
static MONTH_YEAR?: Constant;








}

/**
* EditProxy that handles DateItem when editMode is enabled.
*/
interface DateItemEditProxy extends FormItemEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's value using Date.toShortDate.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's defaultValue.
* 
* 
* @param {string} the new component default value
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new DateItemEditProxy
* 
* @param typeCheckedProperties {Partial<DateItemEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DateItemEditProxy} The new instance
*/
class DateItemEditProxy{
static create(typeCheckedProperties? : Partial<DateItemEditProxy>, uncheckedProperties?: Object): DateItemEditProxy;







}

/**
* Simple modal dialog for collecting a date range from the end user.
*/
interface DateRangeDialog extends Window {
/**
* The title to display in the header-bar of this Dialog.
* @type {string}
* @default "Select Date Range"
*/
headerTitle?: string;

/**
* @type {DateRangeItem}
* @default null
*/
rangeItem?: DateRangeItem;

/**
* The title for the "Clear" button on this dialog.
* @type {string}
* @default "Clear"
*/
clearButtonTitle?: string;

/**
* Button used for clearing the dialog's values. Note that, since this is an AutoChild,
*  it can be configured using clearButtonDefaults and clearButtonProperties.
* @type {IButton}
* @default null
*/
clearButton?: IButton;

/**
* The title for the "OK" button on this dialog.
* @type {string}
* @default "OK"
*/
okButtonTitle?: string;

/**
* Button used for accepting the values entered into the dialog. Note that, since this is an 
*  AutoChild, it can be configured using okButtonDefaults and okButtonProperties.
* @type {IButton}
* @default null
*/
okButton?: IButton;

/**
* The title for the "Cancel" button on this dialog.
* @type {string}
* @default "Cancel"
*/
cancelButtonTitle?: string;

/**
* Button used for cancelling the dialog. Note that, since this is an AutoChild, it can
*  be configured using cancelButtonDefaults and cancelButtonProperties.
* @type {IButton}
* @default null
*/
cancelButton?: IButton;





}
/**
* Creates a new DateRangeDialog
* 
* @param typeCheckedProperties {Partial<DateRangeDialog>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DateRangeDialog} The new instance
*/
class DateRangeDialog{
static create(typeCheckedProperties? : Partial<DateRangeDialog>, uncheckedProperties?: Object): DateRangeDialog;




/**
*  Helper method to launch a DateRangeDialog to have a date range input by the user.
* 
* @param {boolean} whether to allow relative date entry via                  RelativeDateItems, default true
* @param {Partial<DateRangeItem>} properties for the DateRangeItem
* @param {Partial<DateRangeDialog>} properties for the Window
* @param {DateRangeCallback} method to fire once user has input values, with a single                   parameter "criterion" of type Criterion
*/
static askForRange(allowRelativeDates:boolean, rangeItemProperties:Partial<DateRangeItem>, windowProperties:Partial<DateRangeDialog>, callback:(criterion:Criterion) => void): void;




}

/**
* Allows a user to select an absolute or relative range of dates via two
*  RelativeDateItems (if DateRangeItem.allowRelativeDates is true) or two
*  DateItem.
*  
*  The item's FormItem.type is expected to be one of "date" or "datetime" and
*  dictates whether the dates in the range include a time portion. If unset and the item's 
*  form is databound, the data type is detected from the associated 
*  DataSourceField. If there is no such field, or the form is not 
*  databound, the default data type value is "date".
*  
*  DateRangeItem is just a convenience relative to using two RelativeDateItem or
*  DateItem controls in a form, then using FormItem.operator and
*  FormItem.criteriaField to cause them to produce a date range. If you need more
*  control over layout, validation, event handling or any other aspect of appearance or
*  behavior, stop using DateRangeItem and use two DateItem/RelativeDateItem controls directly
*  instead.
*/
interface DateRangeItem extends CanvasItem {
/**
* Controls the placement of the DateRangeItem.toField and DateRangeItem.fromField in the 
*  DateRangeItem.dateRangeForm.
*  
*  Note that we don't recommend "horizontal" placement for mobile, and we also don't recommend 
*  it for DateRangeItem.allowRelativeDates mode, since RelativeDateItem changes width 
*  drastically during editing, which causes a lot of unpleasant side-to-side shifting of 
*  controls.
* @type {DateFieldLayout}
* @default "vertical"
*/
fieldLayout?: DateFieldLayout;

/**
* Allow dateRangeItems' values to show up in the form's values array, or if 
*  DynamicForm.getValuesAsCriteria is called, for the criterion to be included
*  in the returned AdvancedCriteria object
* @type {boolean}
* @default true
*/
shouldSaveValue?: boolean;

/**
* The title for the DateRangeItem.fromField part of the range.
* @type {string}
* @default "From"
*/
fromTitle?: string;

/**
* The title for the DateRangeItem.toField part of the range.
* @type {string}
* @default "To"
*/
toTitle?: string;

/**
* The title orientation for the to / from sub-items. If unset this will be derived from
*  FormItem.titleOrientation or 
*  DynamicForm.titleOrientation.
* @type {TitleOrientation}
* @default null
*/
innerTitleOrientation?: TitleOrientation;

/**
* Whether to allow the user to specify relative dates (via RelativeDateItems) or whether
*  dates are absolute (via DateItems).
* @type {boolean}
* @default false
*/
allowRelativeDates?: boolean;

/**
* The field for the "from" date - a RelativeDateItem or DateItem according to
*  DateRangeItem.allowRelativeDates.
* @type {FormItem}
* @default null
*/
fromField?: FormItem;

/**
* The field for the "to" date - a RelativeDateItem or DateItem according to
*  DateRangeItem.allowRelativeDates.
* @type {FormItem}
* @default null
*/
toField?: FormItem;

/**
* Initial value for the "from" date.
* @type {Date | RelativeDateString | TimeUnit}
* @default "today"
*/
fromDate?: Date | RelativeDateString | TimeUnit;

/**
* Initial value for the "to" date.
* @type {Date | RelativeDateString | TimeUnit}
* @default "today"
*/
toDate?: Date | RelativeDateString | TimeUnit;

/**
* Format for direct user input of date values.
*  
*  If unset, the input format will be determined based on the specified
*  DateDisplayFormat if possible, otherwise picked up from the Date class (see
*  DateUtil.setInputFormat).
* @deprecated * This property is supported but the standard FormItem.dateFormatter and DateRangeItem.inputFormat 
*  may now be used to specify date formatting parsing behavior for dateRangeItems

* @type {DateInputFormat}
* @default null
*/
dateInputFormat?: DateInputFormat;

/**
* For fields of type "date", if this is an editable field such as a
*  TextItem, this property 
*  allows you to specify the DateItem.inputFormat applied to the item.
* @type {DateInputFormat}
* @default null
*/
inputFormat?: DateInputFormat;

/**
* Format for displaying dates in to the user. 
*  Defaults to the system-wide default established by DateUtil.setNormalDisplayFormat.
* @deprecated * This property is supported but the standard FormItem.dateFormatter and DateRangeItem.inputFormat 
*  may now be used to specify date formatting parsing behavior for dateRangeItems

* @type {DateDisplayFormat}
* @default null
*/
dateDisplayFormat?: DateDisplayFormat;

/**
* Not applicable to a DateRangeItem.
* @type {UserFormula}
* @default null
*/
formula?: UserFormula;

/**
* Not applicable to a DateRangeItem.
* @type {UserSummary}
* @default null
*/
textFormula?: UserSummary;

/**
* If this attribute is set to true when FormItem.getCriterion is
*  called, the item will validate the "to" and "from" fields and
*  return null if either field fails validation.
*  See DateRangeItem.validateRange
* @type {boolean}
* @default false
*/
validateCriteria?: boolean;

/**
* DynamicForm AutoChild automatically created by the dateRangeItem and applied
*  to the item as CanvasItem.canvas.
*  This DynamicForm contains the "from" and "to" fields the user will interact with to actually
*  select a date-range. Note that as a standard autoChild, developers may customize this form 
*  by modifying dateRangeProperties.
* @type {DynamicForm}
* @default null
*/
dateRangeForm?: DynamicForm;

/**
* The FormItem class to create when DateRangeItem.allowRelativeDates is true.
* @type {string}
* @default "RelativeDateItem"
*/
relativeItemConstructor?: string;

/**
* The FormItem class to create when DateRangeItem.allowRelativeDates is false,
*  but the DateRangeItem does not have type "datetime".
* @type {string}
* @default "DateItem"
*/
absoluteItemConstructor?: string;

/**
* The FormItem class to create when DateRangeItem.allowRelativeDates is false,
*  and the DateRangeItem's type is "datetime".
* @type {string}
* @default "DateTimeItem"
*/
absoluteDateTimeItemConstructor?: string;

/**
* Error message to display if the user enters a date range where the "To" field value
*  is earlier than the "From" field value.
* @type {string}
* @default "'To' field value cannot be earlier than 'From' field value."
*/
invalidRangeErrorMessage?: string;



/**
*  Sets the DateRangeItem.fromDate for this DateRangeItem.
* 
* @param {Date | RelativeDateString | TimeUnit} the date from which this item should start it's range
*/
setFromDate(fromDate:Date | RelativeDateString | TimeUnit): void;

/**
*  Sets the DateRangeItem.toDate for this DateRangeItem.
* 
* @param {Date | RelativeDateString | TimeUnit} the date at which this item should end it's range
*/
setToDate(fromDate:Date | RelativeDateString | TimeUnit): void;

/**
*  Overridden to return true: dateRangeItems always generate AdvancedCriteria.
* 
*/
hasAdvancedCriteria(): boolean;

/**
*  Returns the Criterion entered in the date field.
*  
*  A Criterion with an "and" OperatorId will be
*  returned with both a "greaterOrEqual" and "lessOrEqual" sub-criteria. If either date is 
*  omitted, only the "greaterOrEqual" (from date) or "lessOrEqual" (to date) Criterion is 
*  included.
* 
* 
*/
getCriterion(): Criterion;

/**
*  Validate both "to" and "from" date-fields.
* 
*/
validateRange(): boolean;

/**
*  Returns true if the specified criterion contains:
*  
* - A single "lessOrEqual" or "greaterOrEqual" criterion on this field
*    
* - An "and" type criterion containing a "lessOrEqual" and a "greaterOrEqual" criterion
*      on this field
*    
* - A single "equals" criterion. Internally, this will be converted into a range
*      by constructing an "and" type criterion containing both a "lessOrEqual" and 
*      a "greaterOrEqual" criterion on this field. Note that subsequent calls to 
*      DateRangeItem.getCriterion will return this more complex 
*      criterion.
*  
* 
* @param {Criterion} criterion to test
*/
canEditCriterion(criterion:Criterion): boolean;

/**
*  Applies the specified criterion to this item for editing. Applies any specified 
*  "greaterOrEqual" operator criterion or sub-criterion to our DateRangeItem.fromField and any
*  specified "lessOrEqual" operator criterion or sub-criterion to our DateRangeItem.toField.
*  
*  Note that a single "equals" criterion can also be passed. See 
*  DateRangeItem.canEditCriterion for more detail.
* 
* @param {Criterion} criterion to edit
*/
setCriterion(criterion:Criterion): void;

/**
*  Sets the value for this dateRangeItem. The value parameter is a 
*  DateRange object that optionally includes both start and end values. If
*  passed null, both start- and end-range values are cleared.
* 
* @param {DateRange} the new value for this item
*/
setValue(value:DateRange): void;

/**
*  Retrieves the current value of this dateRangeItem. The return value is a 
*  DateRange object that excludes start and end values if they aren't
*  set.
* 
*/
getValue(): DateRange;



}
/**
* Creates a new DateRangeItem
* 
* @param typeCheckedProperties {Partial<DateRangeItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DateRangeItem} The new instance
*/
class DateRangeItem{
static create(typeCheckedProperties? : Partial<DateRangeItem>, uncheckedProperties?: Object): DateRangeItem;







}

/**
* A simple DateItem for editing 
*  DateUtil.createDatetime values, where date and time elements are
*  relevant.
*  
*  The item edits datetimes directly as text-values, formatted 
*  according to your locale and settings such as DateItem.dateFormatter.
*  
*  To edit DateUtil.createLogicalDate, see DateItem,
*  and to edit DateUtil.createLogicalTime, see TimeItem.
*  For RelativeDateString, see RelativeDateItem.
*  
*  For detailed information on working with dates, times and datetimes, see the 
*  dateFormatAndStorage.
*/
interface DateTimeItem extends DateItem {
/**
* This property defaults to true in DateTimeItems and cannot be altered, since editing is 
*  via DateTimeItem.displayFormat text-entry only.
* @type {boolean}
* @default true
*/
useTextField?: boolean;

/**
* If DateTimeItem.useTextField is true and browserInputType is set to
*  "datetime", then a native HTML5 local datetime input
*  is used in place of a text input.
*  
*  The use of a native HTML5 datetime input causes certain features to be disabled. Input masks,
*  the picker icon, and a custom DynamicForm.datetimeFormatter are not supported.
*  In-field hints are currently supported in Chrome/Chromium/Opera 15 and iOS 5.0+, but future browser
*  changes might force this support to be removed. Therefore, it is safest to not
*  use in-field hints (set showHintInField to false) in conjunction with a native HTML5 datetime
*  input. In-field hints are not supported in Opera 12 when using a native HTML5 datetime
*  input. If in-field hints are not supported in the browser, then showHintInField has no
*  effect and any hint will be shown to the side of the input.
*  
*  NOTES:
*  
*   
* - This feature requires specific CSS changes. Currently these changes have been made
*     to the Enterprise, EnterpriseBlue, and Graphite skins only.
*   
* - In Chrome/Chromium/Opera 15 and Opera 12, native datetime inputs need to be made
*     wider in order to fit the full datetime value within the native control. However,
*     on iOS 5.0+, the normal width is fine. Be sure to test the layout of the form in
*     all browsers that you wish to support.
*  
* @type {string}
* @default null
*/
browserInputType?: string;

/**
* This property can be used to customize the format in which datetimes are displayed.
*  Should be set to a standard DateDisplayFormat or
*  a function which will return a formatted date time string.
*  
*  If unset, the standard shortDateTime format as set up in 
*  DateUtil.setShortDatetimeDisplayFormat will be used.
*  
*  NOTE: you may need to update the DateTimeItem.inputFormat
*  to ensure the DateItem is able to parse user-entered date strings back into Dates
* @type {DateDisplayFormat}
* @default null
*/
displayFormat?: DateDisplayFormat;

/**
* If DateItem.useTextField is true this property can be used to specify
*  the input format for date strings. 
*  If unset, the input format will be determined based on the specified
*  DateItem.dateFormatter if possible (see DateItem.getInputFormat), otherwise
*  picked up from the Date class (see DateUtil.setInputFormat).
*  
*  Should be set to a standard DateInputFormat
*  
*  Note that the DateInputFormat property is sufficient to parse date or datetime
*  strings specified in most standard date formats. However should an entirely custom
*  parsing function be required developers can 
*  implement a custom DateItem.parseEditorValue method.
*  
*  
*  This attribute does not have an effect if a native HTML5 date input is being used.
*  See DateItem.browserInputType.
* @type {DateInputFormat}
* @default null
*/
inputFormat?: DateInputFormat;





}
/**
* Creates a new DateTimeItem
* 
* @param typeCheckedProperties {Partial<DateTimeItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DateTimeItem} The new instance
*/
class DateTimeItem{
static create(typeCheckedProperties? : Partial<DateTimeItem>, uncheckedProperties?: Object): DateTimeItem;







}

/**
* Static singleton class containing APIs for interacting with Dates.
*/
interface DateUtil extends Class {




}
/**
* Creates a new DateUtil
* 
* @param typeCheckedProperties {Partial<DateUtil>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DateUtil} The new instance
*/
class DateUtil{
static create(typeCheckedProperties? : Partial<DateUtil>, uncheckedProperties?: Object): DateUtil;
/**
* This property may be set to an array of names of days of the week. 
*  For example:
*  
*  ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"]
*  
*  The appropriate day name will then be returned from Date.getShortDayName, and may
*  be used whenever SmartClient components display day-names (for example in the
*  DateItem).
*  Note: For US based applications the first item in the array should be the name for Sunday,
*  then Monday, Tuesday, etc. For browsers with different locales this may vary.
*  To determine the first day for some locale, you can run the following code:
*  
*   alert(new Date(2000, 0, 2).getDay());
*  
*  You should see an alert with a number between zero and 6. This represents the numerical
*  'day' value for Sunday for your browser's locale, since Jan 2nd 2000 was a Sunday.
*  Therefore if this code alerted the number 6, Sunday should appear last in your list
*  of day-names, and Monday first.
* @type {Array<any>}
* @default null
*/
static shortDayNames?: Array<any>;

/**
* This property may be set to an array of names of days of the week. 
*  For example:
*  
*  ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"]
*  
*  The appropriate day name will then be returned from Date.getDayName and 
*  DateUtil.getDayNames, and may
*  be used whenever SmartClient components display day-names (for example in the
*  DateItem).
* @type {Array<any>}
* @default null
*/
static dayNames?: Array<any>;

/**
* This property may be set to an array of shortened month-names.
*  For example:
*  
*  ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
*  
*  The appropriate month name will then be returned from Date.getShortMonthName,
*  and may be used whenever SmartClient components display month-names (for example in the
*  DateItem).
* @type {Array<any>}
* @default null
*/
static shortMonthNames?: Array<any>;

/**
* This property may be set to an array of names of months.
*  For example:
*  
*  ["January", "February", "March", "April", "May", "June", "July", 
*  "August", "September", "October", "November", "December"]
*  
*  The appropriate month name will then be returned from Date.getMonthName,
*  and may be used whenever SmartClient components display month-names (for example in the
*  DateItem).
* @type {Array<any>}
* @default null
*/
static monthNames?: Array<any>;

/**
* Days that are considered "weekend" days.  Values should be the integers returned by the
*  JavaScript built-in Date.getDay(), eg, 0 is Sunday and 6 is Saturday. Override to
*  accommodate different workweeks such as Saudi Arabia (Saturday -> Wednesday) or Israel
*  (Sunday -> Thursday).
* @type {Array<Partial<Integer>>}
* @default "[0, 6]"
*/
static weekendDays?: Array<Partial<Integer>>;





/**
*  Return the parameter date formatted according to the parameter FormatString. 
*  This method is used to implement the DataSourceField.format
*  functionality, but it can also be used to format arbitrary dates programmatically.
* 
* @param {Date} The date to format
* @param {FormatString} The format to apply to this date
*/
static format(date:Date, format:FormatString): string;

/**
*  Create a new Date to represent a logical date value (rather than a specific datetime value),
*  typically for display in a DataSourceField.type. The generated
*  Date value will have year, month and date set to the specified values
*  (in browser native local time).
*  
*  For simplicity, you can pass a Date-instance in the first parameter to create a logical
*  version of it. Passing null is the same as passing 
*  new Date(). When the first parameter is a Date instance, the 
*  month and date parameters may still be passed to 
*  fine-tune the result - for example, createLogicalDate(null, null, 1) will 
*  return a Date instance representing the first day of the current month.
*  
*  See dateFormatAndStorage for more information on date, time and datetime 
*  values in SmartClient.
* 
* 
* @param {Integer | Date} full year or a Date instance - if passed a Date, its parts are      mapped to year, month and date parameters. Passing null is the same as passing      new Date()
* @param {Integer} month (zero based, so 0 is January) - defaults to today's month or,      if the year parameter is a Date instance, to the month from that      instance
* @param {Integer} date within the month - defaults to today's date or, if the      year parameter is a Date instance, to the date from that instance
*/
static createLogicalDate(year:Integer | Date, month:Integer, date:Integer): Date;

/**
*  Returns the full year from the passed datetime, as it will be displayed to the user. 
*  This might not be the same value as that returned by getFullYear() if a 
*  Time.setDefaultDisplayTimezone
*  has been applied. Only necessary for datetimes - for logical dates and times, this
*  method returns the same value as getFullYear().
* 
* @param {Date} datetime instance to work with
*/
static getDisplayYear(datetime:Date): Integer;

/**
*  Returns the month number from the passed datetime, as it will be displayed to the user. 
*  This might not be the same value as that returned by getMonth() if a 
*  Time.setDefaultDisplayTimezone 
*  has been applied. Only necessary for datetimes - for logical dates and times, this
*  method returns the same value as getMonth().
* 
* @param {Date} datetime instance to work with
*/
static getDisplayMonth(datetime:Date): Integer;

/**
*  Returns the day of month from the passed datetime, as it will be displayed to the user. 
*  This might not be the same value as that returned by getDate() if a 
*  Time.setDefaultDisplayTimezone 
*  has been applied. Only necessary for datetimes - for logical dates and times, this
*  method returns the same value as getDate().
* 
* @param {Date} datetime instance to work with
*/
static getDisplayDay(datetime:Date): Integer;

/**
*  Returns the hours value from the passed datetime, as it will be displayed to the user. 
*  This might not be the same value as that returned by getHours() if a 
*  Time.setDefaultDisplayTimezone 
*  has been applied. Only necessary for datetimes - for logical dates and times, this
*  method returns the same value as getHours().
* 
* @param {Date} datetime instance to work with
*/
static getDisplayHours(datetime:Date): Integer;

/**
*  Returns the minutes value from the passed datetime, as it will be displayed to the user. 
*  This might not be the same value as that returned by getMinutes() if a 
*  Time.setDefaultDisplayTimezone 
*  has been applied. Only necessary for datetimes - for logical dates and times, this
*  method returns the same value as getMinutes().
* 
* @param {Date} datetime instance to work with
*/
static getDisplayMinutes(datetime:Date): Integer;

/**
*  Create a new Date object to represent a logical time value (rather than a specific datetime
*  value), typically for display in a DataSourceField.type. The generated
*  Date value will have year, month and date set to the epoch date (Jan 1 1970), and time
*  elements set to the supplied hour, minute and second (in browser native local time).
*  
*  For simplicity, you can pass a Date-instance in the first parameter to create a logical
*  time from that Date instance. Passing null is the same as passing 
*  new Date(). When the first parameter is a Date instance, the 
*  minutes and seconds parameters may still be passed to 
*  fine-tune the result - for example, createLogicalTime(null, 0, 0) will 
*  return a Date instance with time-values representing the start of the current hour.
*  
*  See dateFormatAndStorage for more information on date, time and datetime 
*  values in SmartClient.
* 
* 
* @param {Integer | Date} integer hour (0-23) or a Date instance - if passed a Date     instance, its time-elements are mapped to hour, minute and second parameters.       Passing null is the same as passing new Date()
* @param {Integer} minute (0-59) - defaults to zero or, if the hour     parameter is a Date instance, the minutes from that Date instance
* @param {Integer} second (0-59) - defaults to zero or, if the hour     parameter is a Date instance, the seconds from that Date
*/
static createLogicalTime(hour:Integer | Date, minute:Integer, second:Integer): Date;

/**
*  Create a new Date object in the current locale time, typically for display in a 
*  DataSourceField.type.
*  
*  For simplicity, you can pass a Date-instance in the first parameter to create a new
*  date from that Date instance. Passing null is the same as passing 
*  new Date(). When the first parameter is a Date instance, the various 
*  datetime-element parameters - month, hour, etc - may still be passed to 
*  fine-tune the result.
*  
*  See dateFormatAndStorage for more information on date, time and datetime 
*  values in SmartClient.
* 
* 
* @param {Integer | Date=} full year or a Date instance - if passed a Date, its parts are      mapped to all the other parameters. Passing null is the same as passing      new Date()
* @param {Integer=} month (zero based, so 0 is January) - defaults to today's month or,      if the year parameter is a Date instance, to the month from that      instance
* @param {Integer=} date within the month - defaults to today's date or, if the      year parameter is a Date instance, to the date from that instance
* @param {Integer=} integer hour (0-23) - defaults to zero or, if the year     parameter is a Date instance, the hours from that Date instance
* @param {Integer=} minute (0-59) - defaults to zero or, if the year     parameter is a Date instance, the minutes from that Date instance
* @param {Integer=} second (0-59) - defaults to zero or, if the year     parameter is a Date instance, the seconds from that Date
*/
static createDatetime(year?:Integer | Date, month?:Integer, date?:Integer, hour?:Integer, minute?:Integer, second?:Integer): Date;

/**
*  Get a logical date - a value appropriate for a DataSourceField of type "date" - from a
*  datetime value (a value from a DataSourceField of type "datetime").
*  
*  This method correctly takes into account the current
*  Time.setDefaultDisplayTimezone, specifically, the returned Date
*  will reflect the day, month and year that appears when the datetime is rendered
*  by a SmartClient component rather than the date values that would be returned by
*  Date.getDay() et al (which can differ, since getDay() uses the browser's local timezone).
*  
*  For further background on date, time and datetime types, storage and transmission, see
*  dateFormatAndStorage.
* 
* 
* @param {Date} a Date instance representing a datetime value
*/
static getLogicalDateOnly(date:Date): Date;

/**
*  Get a logical time - a value appropriate for a DataSourceField of type "time" - from a
*  datetime value (a value from a DataSourceField of type "datetime").
*  
*  This method correctly takes into account the current
*  Time.setDefaultDisplayTimezone, specifically, the returned Date will
*  reflect the hour, minute and second that appears when the datetime is rendered by a SmartClient
*  component rather than the time values that would be returned by Date.getHours() et al (which
*  can differ, since getHours() uses the browser's local timezone).
*  
*  For further background on date, time and datetime types, storage and transmission, see
*  dateFormatAndStorage.
* 
* 
* @param {Date} a Date instance representing a datetime value
*/
static getLogicalTimeOnly(date:Date): Date;

/**
*  Combine a logical date (a value appropriate for a DataSourceField of type "date") with a
*  logical time (a value appropriate for a DataSourceField of type "time") into a datetime
*  value (a value appropriate for a DataSourceField of type "datetime")
*  
*  This method correctly takes into account the current
*  Time.setDefaultDisplayTimezone, specifically, the returned datetime
*  value will show the same date and time as the passed date and time objects when rendered by
*  a SmartClient component that has been configured with a field of type "datetime".
*  
*  For further background on date, time and datetime types, storage and transmission, see
*  dateFormatAndStorage.
* 
* 
* @param {Date} a Date instance representing logical date value
* @param {Date} a Date instance representing logical time value
*/
static combineLogicalDateAndTime(date:Date, time:Date): Date;

/**
*  Compare two dates; returns 0 if equal, -1 if the first date is greater (later), or 1 if
*  the second date is greater. If either value is not a Date object, it is treated as the
*  epoch (midnight on Jan 1 1970) for comparison purposes.
* 
* @param {Date} first date to compare
* @param {Date} second date to compare
*/
static compareDates(date1:Date, date2:Date): Integer;

/**
*  Compare two dates, normalizing out the time elements so that only the date elements are
*  considered; returns 0 if equal, -1 if the first date is greater (later), or 1 if
*  the second date is greater.
* 
* @param {Date} first date to compare
* @param {Date} second date to compare
*/
static compareLogicalDates(date1:Date, date2:Date): Integer;

/**
*  Sets up the default system-wide input format for strings being parsed into dates via
*  DateUtil.parseInput(). This will effect how SmartClient components showing
*  editable date or datetime fields parse user-entered values into live Date objects.
*  
*  The input format can be specified as a DateInputFormat - a 3 character string like
*  "MDY" indicating the order of the Month, Day and Year components of date strings.
*  
*  As an example - an input format of "MDY" would parse "01/02/1999" to Jan 2nd 1999
*  This standard parsing logic will also handle date-time strings such as "01/02/1999 08:45", or
*  "01/02/1999 16:21:05".
*  
*  Notes:
*  
*  
* - If the inputFormat is not explicitly set,the system automatically determines
*    the standard input format will be based on the specified 
*    DateUtil.setShortDisplayFormat wherever possible.
*    For example if the short display format has been set to "toEuropeanShortDate" the input
*    format will default to "DMY".
*  
* - The default date parsing functionality built into SmartClient will handle dates presented
*    with any separator string, and can handle 1 or 2 digit day and month values, months formatted
*    as classMethod:DateUtil.getMonthNames or classMethod:DateUtil.getShortMonthNames, and 2 or 4
*    digit year values. This means that in many cases custom date display formats can be parsed
*    back to Date values without the need for a custom parser function. However if more
*    sophisticated parsing logic is required, a function may be passed into this method. In
*    this case the parser function should be able to handle parsing date and datetime values
*    formatted via
*    Date.toShortDate and Date.toShortDateTime.
*    
*  
* - Date parsing and formatting logic may be overridden at the component level by setting
*    properties directly on the component or field in question.
*  
* 
* @param {DateInputFormat | Function} Default format for strings to be parsed into Dates.  If this method is passed a function, it is expected to take a single parameter (the formatted date string), and return the appropriate Javascript Date object (or null if appropriate). 
*/
static setInputFormat(format:DateInputFormat | Function): void;

/**
*  Retrieves the default format for strings being parsed into dates via
*  DateUtil.parseInput()
* 
*/
static getInputFormat(): string;

/**
*  Parse a date passed in as a string, returning the appropriate date object.
* 
* @param {string} date value as a string
* @param {DateInputFormat=} Format of the date string being passed.                   If not passed, the default date input format as set up                    via setInputFormat() will be used.
* @param {Integer=} For date formats that support a 2 digit                 year, if parsed year is 2 digits and less than this                  number, assume year to be 20xx rather than 19xx
* @param {boolean=} If the string passed in was not a valid date, in some cases we can convert to a      valid date (for example incrementing the year if the month is greater than 12).      This optional parameter will suppress such conversions - anything that doesn't      parse directly to a valid date will simply return null.
*/
static parseInput(dateString:string, format?:DateInputFormat, centuryThreshold?:Integer, suppressConversion?:boolean): Date;

/**
*  Set the default formatter for date objects to the method name passed in. After calling this
*  method, subsequent calls to Date.toNormalDate
*  will return a string formatted
*  according to this format specification. Note: this will be the standard long date format used
*  by SmartClient components.
*  
*  The format parameter may be a FormatString, a DateDisplayFormat 
*  string, or a function. If passed a function, this function will be executed in the scope of
*  the Date and should return the formatted string.
*  
*  Initial default normalDisplayFormat is "toLocaleString"
* 
* @param {FormatString | DateDisplayFormat | Function} new formatter
*/
static setNormalDisplayFormat(format:FormatString | DateDisplayFormat | Function): void;

/**
*  Set the default normal format for datetime values. After calling this method, subsequent
*  calls to Date.toNormalDatetime
*  will return a string formatted according to
*  this format specification. Note that this will be the standard datetime format used by
*  SmartClient components.
*  
*  The format parameter may be a FormatString, a DateDisplayFormat 
*  string, or a function. If passed a function, this function will be executed in the scope of
*  the Date and should return the formatted string.
* 
* 
* @param {FormatString | DateDisplayFormat | Function} new formatter
*/
static setNormalDatetimeDisplayFormat(format:FormatString | DateDisplayFormat | Function): void;

/**
*  Set the default short format for dates. After calling this method, subsequent calls to
*  Date.toShortDate
*  will return a string formatted
*  according to this format specification. Note that this will be the standard short date
*  format used by SmartClient components.
*  
*  The format parameter may be a FormatString, a DateDisplayFormat 
*  string, or a function. If passed a function, this function will be executed in the scope of
*  the Date and should return the formatted string.
*  
*  Initial default shortDateFormat is "toUSShortDate". This property
*  is commonly modified for localization of applications. See
*  http://en.wikipedia.org/wiki/Date_format_by_country
*  for a useful overview of standard date formats per country.
* 
* 
* @param {FormatString | DateDisplayFormat | Function} new formatter
*/
static setShortDisplayFormat(format:FormatString | DateDisplayFormat | Function): void;

/**
*  Sets a new default separator that will be used when formatting dates. By default, this
*  is a forward slash character: "/"
* 
* @param {string} separator to use in dates
*/
static setDefaultDateSeparator(separator:string): void;

/**
*  gets the default date separator string
* 
*/
static getDefaultDateSeparator(): string;

/**
*  Set the default short format for datetime values. After calling this method, subsequent
*  calls to Date.toShortDateTime
*  will return a string formatted 
*  according to this format specification. Note that this will be the standard datetime format
*  used by SmartClient components.
*  
*  The format parameter may be a FormatString, a DateDisplayFormat 
*  string, or a function. If passed a function, this function will be executed in the scope of
*  the Date and should return the formatted string.
*  
*  Initial default format is "toUSShortDatetime". See
*  http://en.wikipedia.org/wiki/Date_format_by_country
*  for a useful overview of standard date formats per country.
* 
* 
* @param {FormatString | DateDisplayFormat | Function} new formatter
*/
static setShortDatetimeDisplayFormat(format:FormatString | DateDisplayFormat | Function): void;

/**
*  Sets the global fiscal calendar, which is used for all calls to
*  getFiscalYear() / getFiscalWeek() if those methods aren't passed a fiscalCalander.
* 
* 
* @param {FiscalCalendar} the object representing the start month and date of      the fiscal year in the current locale
*/
static setFiscalCalendar(fiscalCalendar:FiscalCalendar): void;

/**
*  Returns the global FiscalCalendar representing the start month and
*  date of the fiscal year in the current locale.
* 
*/
static getFiscalCalendar(): FiscalCalendar;

/**
*  Returns the start date of the fiscal year for the passed date.
* 
* 
* @param {Date | number} the date, or the year-number, to get the fiscal year for
* @param {FiscalCalendar=} the object representing the starts of one or more               fiscal years
*/
static getFiscalStartDate(date:Date | number, fiscalCalendar?:FiscalCalendar): Date;

/**
*  Sets the global attribute that dictates whether the DateChooser shelled
*  from DateItem show a UI for working with Fiscal Years.
* 
* 
* @param {boolean} whether to show Fiscal Year pickers in DateChoosers by default
*/
static setShowChooserFiscalYearPickers(showChooserFiscalYearPickers:boolean): void;

/**
*  Sets the global attribute that dictates whether the DateChooser shelled
*  from DateItem show a UI for working with Weeks.
* 
* 
* @param {boolean} whether to show Fiscal Week pickers in DateChoosers by default
*/
static setShowChooserWeekPickers(showChooserWeekPickers:boolean): void;

/**
*  Sets the global attribute that dictates which day should be treated as the first day of the
*  week in calendars and date calculations. The parameter is expected to be an integer value 
*  between 0 (Sunday) and 6 (Saturday).
*  
*  The default value is picked up from the current locale.
* 
* 
* @param {Integer} the number of the day to use as the first day of the week
*/
static setFirstDayOfWeek(firstDayOfWeek:Integer): void;

/**
*  Returns the global attribute that dictates which day should be treated as the first day of 
*  the week in calendars and date calculations. The parameter is expected to be an integer 
*  value between 0 (Sunday) and 6 (Saturday).
*  
*  The default value is picked up from the current locale.
* 
* 
*/
static getFirstDayOfWeek(): Integer;

/**
*  Returns the FiscalYear object for the fiscal year in which the passed date exists.
* 
* 
* @param {Date | Integer} the date to get the fiscal year for
* @param {FiscalCalendar=} the object representing the start of the fiscal period
*/
static getFiscalYear(date:Date | Integer, fiscalCalendar?:FiscalCalendar): FiscalYear;

/**
*  Returns a date's week-number, according to the fiscal calendar
* 
* 
* @param {Date} the date to get the fiscal year for
* @param {FiscalCalendar=} the object representing the starts of fiscal years
*/
static getFiscalWeek(date:Date, fiscalCalendar?:FiscalCalendar): Integer;

/**
*  Return a logicalDate representing the current day in the 
*  Time.setDefaultDisplayTimezone.
* 
*/
static today(): void;

/**
*  Return an array of the short names of each day, suitable for use in a selection list, etc.
*  Day names are picked up from a DateUtil.shortDayNames list specified in each locale.
* 
* @param {Integer} maximum length of each day string - default is no maximum (full strings)
*/
static getShortDayNames(length:Integer): Array<Partial<string>>;

/**
*  Return an array of the full names of each day, suitable for use in a selection list, etc.
*  Day names are picked up from DateUtil.dayNames, which defaults to an array of 
*  English-language strings and these are updated to localized values by loading a locale.
*  
*  If DateUtil.dayNames is purposely cleared, this method will fall back to 
*  deriving day-names from the native browser date string. Note, if we have to use this native
*  backup behavior, the day names may vary by browser as well as locale - for example, they 
*  may be in an abbreviated form, similar to the result of calling 
*  DateUtil.getShortDayNames. 
* 
*/
static getDayNames(): Array<Partial<string>>;

/**
*  Sets the days that are considered DateUtil.weekendDays. The parameter 
*  should be array of the integers returned by the JavaScript built-in Date.getDay(), eg, 0 is 
*  Sunday and 6 is Saturday. Override to accommodate different workweeks such as Saudi Arabia 
*  (Saturday -> Wednesday) or Israel (Sunday -> Thursday).
* 
* 
* @param {Array<Partial<Integer>>} the array of day-numbers to assign as weekend days
*/
static setWeekendDays(weekendDays:Array<Partial<Integer>>): void;

/**
*  Return an array of days that are considered "weekend" days. Values will be the integers
*  returned by the JavaScript built-in Date.getDay(), eg, 0 is Sunday and 6 is Saturday.
*  Override DateUtil.weekendDays to accommodate different workweeks such as Saudi Arabia
*  (Saturday -> Wednesday) or Israel (Sunday -> Thursday).
* 
*/
static getWeekendDays(): Array<Partial<Integer>>;

/**
*  Converts a RelativeDateShortcut to a RelativeDateString.
* 
* @param {RelativeDateShortcut} shortcut string to convert
* @param {RelativeDateRangePosition=} Are we interested in the start or end of the specified relative date? This applies to shortcuts which do not specify a specific  moment (such as $today) - it does not apply to shortcuts which  already specify a specific moment such as $startOfToday. If unspecified  rangePosition is always assumed to be "start"
*/
static mapRelativeDateShortcut(relativeDate:RelativeDateShortcut, rangePosition?:RelativeDateRangePosition): RelativeDateString;

/**
*  Returns a new Date instance, representing the baseDate adjusted by 
*  the relative amount of the RelativeDateString.
* 
* @param {Date} Date instance to apply a relative amount to - defaults to new Date()
* @param {RelativeDateString} the relative amount to apply to the              baseDate
*/
static adjustDate(baseDate:Date, relativeDateString:RelativeDateString): Date;

/**
*  Converts a RelativeDate, RelativeDateShortcut,
*  
*  or RelativeDateString to a concrete Date.
* 
* @param {RelativeDate | RelativeDateShortcut | RelativeDateString} the relative  date to convert
* @param {Date=} base value for conversion. Defaults to the current date/time.
* @param {RelativeDateRangePosition=} optional date-range position. Only has an effect  if the date passed in is a RelativeDateShortcut where the range position   is not implicit, such as "$yesterday"
* @param {boolean=} should the generated date be marked as a "logical" date? A  logical date object is a Date value where the time component is ignored for formatting and  serialization purposes - such as the date displayed within a component field of  specified type "date". See dateFormatAndStorage for more on logical dates vs  datetime type values.
*/
static getAbsoluteDate(relativeDate:RelativeDate | RelativeDateShortcut | RelativeDateString, baseDate?:Date, rangePosition?:RelativeDateRangePosition, isLogicalDate?:boolean): Date;

/**
*  Returns the start of some period, like day, week or month, relative to a passed Date 
*  instance.
* 
* @param {Date} the base date to find the period start from
* @param {string} the period to return the start of, one of mn/h/d/w/m/y
* @param {boolean=} process and return a logicalDate with no time element
* @param {Integer=} which day should be considered the firstDayOfWeek -         overrides the default provided by the locale
*/
static getStartOf(date:Date, period:string, logicalDate?:boolean, firstDayOfWeek?:Integer): Date;

/**
*  Returns the end of some period, like day, week or month, relative to a passed Date 
*  instance.
* 
* @param {Date} the base date to find the period end from
* @param {string} the period to return the end of, one of mn/h/d/w/m/y
* @param {boolean=} process and return a logicalDate with no time element
* @param {Integer=} which day should be considered the firstDayOfWeek -         overrides the default provided by the locale
*/
static getEndOf(date:Date, period:string, logicalDate?:boolean, firstDayOfWeek?:Integer): Date;




}

/**
* Chooses a next element in a Process by evaluating a series of criteria against the
*  Process.state and choosing the element associated with the criteria that matched, or
*  a MultiDecisionTask.defaultElement if none of the criteria match.
* @deprecated * In favor of MultiDecisionTask as of SmartClient release 13.0

*/
interface DecisionGateway extends MultiDecisionTask {
/**
* A Map from ProcessElement.ID to Criteria that will cause this ProcessElement to
*  be chosen as the next element if the criteria matches.
*  
*  If no criteria is matched the next element is MultiDecisionTask.defaultElement or the workflow
*  is finished.
*  
*  Data values in this criteria prefixed with "$" will be treated as dynamic expressions
*  as detailed in taskInputExpression. Specifically, this means that for 
*  simple criteria, any property value that is a String and is prefixed with "$" will be
*  assumed to be an expression, and for AdvancedCriteria, the same treatment will be
*  applied to Criterion.value.
*  
*  Note that dynamic expressions starting with "$input" are not applicable
*  for an decisionGateway but "$inputRecord" can be used for direct reference to Process.state. 
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Process.ruleScope.
* @deprecated * In favor of MultiDecisionTask.decisionList as of SmartClient release 12.1

* @type {Map<Criteria>}
* @default null
*/
criteriaMap?: Map<Criteria>;





}
/**
* Creates a new DecisionGateway
* 
* @param typeCheckedProperties {Partial<DecisionGateway>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DecisionGateway} The new instance
*/
class DecisionGateway{
static create(typeCheckedProperties? : Partial<DecisionGateway>, uncheckedProperties?: Object): DecisionGateway;







}

/**
* Chooses one or another next process element based on AdvancedCriteria applied to
*  Process.state.
*  
*  If the AdvancedCriteria evaluate to true, the DecisionTask.nextElement is
*  chosen, otherwise the DecisionTask.failureElement.
*/
interface DecisionTask extends ProcessElement {
/**
* Simple or AdvancedCriteria to be applied against the Process.state.
*  
*  Data values in this criteria prefixed with "$" will be treated as dynamic expressions
*  as detailed in taskInputExpression. Specifically, this means that for 
*  simple criteria, any property value that is a String and is prefixed with "$" will be
*  assumed to be an expression, and for AdvancedCriteria, the same treatment will be
*  applied to Criterion.value.
*  
*  Note that dynamic expressions starting with "$input" are not applicable
*  for an DecisionTask but "$inputRecord" can be used for direct reference to Process.state. 
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Process.ruleScope.
* @type {Criteria}
* @default null
*/
criteria?: Criteria;

/**
* Next Process.sequences or Process.elements to execute
*  if the criteria match the process state. 
*  
*  nextElement does not need to be specified if this element is part of a
*  Process.sequences and has a next element in the sequence.
*  
*  Note that if there is both a sequence and a normal element
*  with the same name in the current Process, the sequence will
*  be used.
* @type {string}
* @default null
*/
nextElement?: string;

/**
* ID of the next sequence or element to proceed to if the criteria do not match.
* @type {string}
* @default null
*/
failureElement?: string;





}
/**
* Creates a new DecisionTask
* 
* @param typeCheckedProperties {Partial<DecisionTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DecisionTask} The new instance
*/
class DecisionTask{
static create(typeCheckedProperties? : Partial<DecisionTask>, uncheckedProperties?: Object): DecisionTask;







}

/**
* A simple container that implements the policy that at most one of its contained components
*  is visible at any given time.
*  
*  The set of mutually exclusive components is specified by Deck.panes, and whichever
*  component is visible fills the space of the Deck automatically.
*  
*  To switch to a new pane, call Deck.setCurrentPane, or simply call
*  Canvas.show on the pane directly - the Deck will notice that you
*  have shown a different pane and hide other panes automatically.
*  
*  Canvas.children may also be used; any components that are specified as children are
*  unmanaged by the Deck and so can place themselves arbitrarily.
*  
*  Deck achieves its mutually-exclusive display behavior by using the superclass
*  Layout.members property, which means that properties such as Layout.layoutMargin
*  and Layout.vPolicy do apply to deck. However, trying to manipulate
*  deck.members with APIs such as Layout.addMember is not supported and
*  will have undefined results.
*/
interface Deck extends Layout {
/**
* Set of mutually exclusive panes displayed in this Deck.
*  
*  If Deck.currentPane is not set, when the Deck is first drawn, the
*  first pane in this array becomes the currentPane.
* @type {Array<Partial<Canvas>>}
* @default null
*/
panes?: Array<Partial<Canvas>>;

/**
* The pane currently shown in this Deck. All other panes are hidden.
* @type {Canvas}
* @default null
*/
currentPane?: Canvas;



/**
*  Change the CurrentPane.
*  
*  If the passed pane is not contained in this Deck, logs a warning and does
*  nothing.
* 
* @param {Canvas | GlobalId} the pane to show, as either a Canvas or the Canvas.ID
*/
setCurrentPane(pane:Canvas | GlobalId): void;

/**
*  Hides the current pane, without showing any other pane.
* 
*/
hideCurrentPane(): void;

/**
*  Add a pane to this deck. If the specified pane is already present in the deck, it
*  will be moved to the specified position.
* 
* @param {Canvas} pane to add
* @param {Integer=} position for the new pane in the Deck.panes array.  If no index is specified, the pane will be added to the end of the panes array.
*/
addPane(pane:Canvas, index?:Integer): void;

/**
*  Remove a pane from this deck.
* 
* @param {Canvas} pane to remove
*/
removePane(pane:Canvas): void;



}
/**
* Creates a new Deck
* 
* @param typeCheckedProperties {Partial<Deck>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Deck} The new instance
*/
class Deck{
static create(typeCheckedProperties? : Partial<Deck>, uncheckedProperties?: Object): Deck;







}

/**
* Displays one or more records "horizontally" with one property per line.
*/
interface DetailViewer extends Omit<Canvas, 'setDragTracker'>, Omit<DataBoundComponent, 'fields'>  {
/**
* Criteria to be used when DetailViewer.autoFetchData is set.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
initialCriteria?: Criteria;

/**
* Criteria that are never shown to or edited by the user and are cumulative with any criteria
*  provided via DataBoundComponent.initialCriteria and related methods.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
implicitCriteria?: Criteria;

/**
* If true, when this component is first drawn, automatically call this.fetchData().
*  Criteria for this fetch may be picked up from DetailViewer.initialCriteria, and textMatchStyle may
*  be specified via ListGrid.autoFetchTextMatchStyle.
*  
*  NOTE: if autoFetchData is set, calling
*  ListGrid.fetchData before draw will cause two requests to be issued, one from the manual
*  call to fetchData() and one from the autoFetchData setting. The second request will use
*  only DetailViewer.initialCriteria and not any other criteria or settings from the first request.
*  Generally, turn off autoFetchData if you are going to manually call ListGrid.fetchData at any time.
* 
*  Note: If you are using saved searches - either via SavedSearchItem or ListGrid.saveDefaultSearch, autoFetchData
*  will be automatically suspended and replaced with the saved criteria/view state, if applicable.
* @type {boolean}
* @default false
*/
autoFetchData?: boolean;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "DetailViewerEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* A single record object or an array of them, specifying data. Note that DetailViewers do
*  not observe changes to the data array 
*  (in other words they will not automatically
*  re-draw when the data provided via this property is altered).
* @type {Array<Partial<DetailViewerRecord>> | Array<Partial<Record>> | any}
* @default null
*/
data?: Array<Partial<DetailViewerRecord>> | Array<Partial<Record>> | any;

/**
* DetailViewers do not yet support paging, and will fetch and render all available
*  records.
* @type {FetchMode}
* @default "basic"
*/
dataFetchMode?: FetchMode;

/**
* A DetailViewer is a DataBoundComponent.dataArity:either component by
*  default which means data population from Canvas.dataContext will be treated like
*  a DynamicForm (i.e. single). However, by explicitly marking an instance of DetailViewer
*  as dataArity:multiple, dataContext population will be treated
*  similarly to a ListGrid.
* @type {string}
* @default "either"
*/
dataArity?: string;

/**
* The DataSource that this component should bind to for default fields and for performing
*  DSRequest.
*  
*  Can be specified as either a DataSource instance or the String ID of a DataSource.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* Height of rows rendered by the DetailViewer. Acts as a minimum - the DetailViewer never
*  clips values. This attribute can be set as null.
* @type {Integer}
* @default "19"
*/
rowHeight?: Integer;

/**
* An array of field objects, specifying the order and type of fields to display in this
*  DetailViewer. In DetailViewers, the fields specify rows.
* @type {Array<Partial<DetailViewerField>>}
* @default null
*/
fields?: Array<Partial<DetailViewerField>>;

/**
* Specifies a list of icons that can be used in DataBoundComponent.editHilites.
*  
*  hiliteIcons should be specified as an Array of SCImgURL.
*  When present, the hilite editing interface shown when DataBoundComponent.editHilites is called 
*  will offer the user a drop down for picking one of these icons when defining either a 
*  simple or advanced hilite rule.
*  
*  If the user picks an icon, the created hiliting rule will have Hilite.icon set to 
*  the chosen icon. DataBoundComponent.hiliteIconPosition controls where the icon will 
*  appear for that field -- the default is that it appears in front of the normal cell content.
*  This can also be overridden at the field level.
* @type {Array<Partial<string>>}
* @default "["[SKINIMG]/Dialog/notify.png", "[SKINIMG]/Dialog/warn.png", "[SKINIMG]/actions/approve.png"]"
*/
hiliteIcons?: Array<Partial<string>>;

/**
* When DetailViewer.hiliteIcons are present, where the hilite icon will be placed 
*  relative to the field value. See HiliteIconPosition. Can be overridden at the
*  field level.
* @type {HiliteIconPosition}
* @default "before"
*/
hiliteIconPosition?: HiliteIconPosition;

/**
* Default width and height of DetailViewer.hiliteIcons for this component.
*  Can be overridden at the component level via explicit 
*  DetailViewer.hiliteIconWidth and
*  DetailViewer.hiliteIconHeight, or at the field level via 
*  ListGridField.hiliteIconSize,
*  ListGridField.hiliteIconWidth and 
*  ListGridField.hiliteIconHeight
* @type {number}
* @default "12"
*/
hiliteIconSize?: number;

/**
* Width for hilite icons for this component.
*  Overrides DetailViewer.hiliteIconSize.
*  Can be overridden at the field level.
* @type {number}
* @default null
*/
hiliteIconWidth?: number;

/**
* Height for hilite icons for this listGrid.
*  Overrides DetailViewer.hiliteIconSize.
*  Can be overridden at the field level
* @type {number}
* @default null
*/
hiliteIconHeight?: number;

/**
* How much padding should there be on the left of DetailViewer.hiliteIcons by default?
*  Can be overridden at the field level
* @type {number}
* @default "2"
*/
hiliteIconLeftPadding?: number;

/**
* How much padding should there be on the right of DetailViewer.hiliteIcons by default?
*  Can be overridden at the field level
* @type {number}
* @default "2"
*/
hiliteIconRightPadding?: number;

/**
* Name of the field in the DetailViewerRecord which specifies the data property for that record.
* @type {string}
* @default "name"
*/
fieldIdProperty?: string;

/**
* The number of records to display in a block. A block is a horizontal row on a page
*      containing one or more records, as specified by the value of recordsPerBlock. The
*      height of a block is equal to the height of a single record. The default setting of
*      1 causes each record to appear by itself in a vertical row. Setting recordsPerBlock
*      to 2 would cause records to appear side by side in groups of two.
*      Use a value of "*" to indicate all records.
* @type {number}
* @default "1"
*/
recordsPerBlock?: number;

/**
* A string (HTML acceptable) that will be written to a page to separate blocks.
* @type {HTMLString}
* @default "&lt;br&gt;&lt;br&gt;"
*/
blockSeparator?: HTMLString;

/**
* Whether to show the field when the value is null
* @type {boolean}
* @default true
*/
showEmptyField?: boolean;

/**
* Text to show for an empty cell
* @type {HTMLString}
* @default "&amp;nbsp;"
*/
emptyCellValue?: HTMLString;

/**
* text to put before a label
* @type {HTMLString}
* @default ""
*/
labelPrefix?: HTMLString;

/**
* text to put after a label
* @type {HTMLString}
* @default ":"
*/
labelSuffix?: HTMLString;

/**
* Horizontal alignment of value-labels in this viewer. If unspecified, defaults to 
*  "left" when in RTL mode and "right" otherwise.
* @type {Alignment}
* @default null
*/
labelAlign?: Alignment;

/**
* Horizontal alignment of values in this viewer. If unspecified, defaults to 
*  "right" when in RTL mode and "left" otherwise.
* @type {Alignment}
* @default null
*/
valueAlign?: Alignment;

/**
* Should the label be allowed to wrap, or be fixed to one line no matter how long
* @type {boolean}
* @default false
*/
wrapLabel?: boolean;

/**
* Whether values should be allowed to wrap by default, or should be shown on one line
*  regardless of length.
* @type {boolean}
* @default true
*/
wrapValues?: boolean;

/**
* CSS style for the component as a whole.
* @type {CSSStyleName}
* @default "detailViewer"
*/
styleName?: CSSStyleName;

/**
* CSS style for each block (one record's worth of data).
* @type {CSSStyleName}
* @default "detailBlock"
*/
blockStyle?: CSSStyleName;

/**
* CSS style for a normal detail label
* @type {CSSStyleName}
* @default "detailLabel"
*/
labelStyle?: CSSStyleName;

/**
* CSS style for a normal value
* @type {CSSStyleName}
* @default "detail"
*/
cellStyle?: CSSStyleName;

/**
* CSS style for a header
* @type {CSSStyleName}
* @default "detailHeader"
*/
headerStyle?: CSSStyleName;

/**
* Optional CSS style for a cell in printable HTML for this component. If unset
*  DetailViewer.cellStyle will be used for printing as well as normal presentation.
* @type {CSSStyleName}
* @default null
*/
printCellStyle?: CSSStyleName;

/**
* Optional CSS style for a label cell in printable HTML for this component. If unset
*  DetailViewer.labelStyle will be used for printing as well as normal presentation.
* @type {CSSStyleName}
* @default null
*/
printLabelStyle?: CSSStyleName;

/**
* Optional CSS style for a header in printable HTML for this component. If unset
*  DetailViewer.headerStyle will be used for printing as well as normal presentation.
* @type {CSSStyleName}
* @default null
*/
printHeaderStyle?: CSSStyleName;

/**
* CSS style for a separator
* @type {CSSStyleName}
* @default "detail"
*/
separatorStyle?: CSSStyleName;

/**
* How should Date type values be displayed in this DetailViewer by default?
*  
*  This property specifies the default DateDisplayFormat to apply to Date values
*  displayed in this grid for all fields except those of DetailViewerField.type
*  (See also DetailViewer.timeFormatter).
*  If DetailViewer.datetimeFormatter is specified, that will be applied by default
*  to fields of type "datetime".
*  
*  Note that if DetailViewerField.dateFormatter or DetailViewerField.timeFormatter are
*  specified those properties will take precedence over the component level settings.
*  
*  If unset, date values will be formatted according to the system wide 
*  DateUtil.setNormalDisplayFormat.
* @type {DateDisplayFormat}
* @default null
*/
dateFormatter?: DateDisplayFormat;

/**
* Display format to use for fields specified as type 'datetime'. Default is to use the
*  system-wide default long ("normal") date time format, configured via
*  DateUtil.setNormalDatetimeDisplayFormat. Specify any
*  valid DateDisplayFormat to change the display format for datetimes used by this 
*  viewer.
*  
*  May be specified as a function. If specified as a function, this function will
*  be executed in the scope of the Date
*  and should return the formatted string.
*  
*  
*  May also be specified at the field level via
*  DetailViewerField.dateFormatter
* @type {DateDisplayFormat}
* @default null
*/
datetimeFormatter?: DateDisplayFormat;

/**
* Display format to use for fields specified as type 'time'. May also be specified at 
*  the field level via DetailViewerField.timeFormatter.
*  If unset, time fields will be formatted based on the system wide 
*  Time.setNormalDisplayFormat
* @type {TimeDisplayFormat}
* @default null
*/
timeFormatter?: TimeDisplayFormat;

/**
* Property name on a record that will hold the link text for that record.
*  
*  This property is configurable to avoid possible collision with data values in the
*  record.
*  
*  Use DetailViewerField.linkTextProperty if you have more than one link field and
*  the fields' records do not use the same property to store the linkText.
* @type {string}
* @default "linkText"
*/
linkTextProperty?: string;

/**
* Show DetailViewer.emptyMessage when there is no data to display?
* @type {boolean}
* @default true
*/
showEmptyMessage?: boolean;

/**
* The string to display in the body of a detailViewer with no records.
* @type {HTMLString}
* @default "No items to display."
*/
emptyMessage?: HTMLString;

/**
* CSS style to display this message in
* @type {CSSStyleName}
* @default "normal"
*/
emptyMessageStyle?: CSSStyleName;

/**
* The string to display in the body of a detailViewer which is loading records.
*  Use "&#36;{loadingImage}" to include Canvas.loadingImageSrc.
* @type {HTMLString}
* @default "&amp;nbsp;${loadingImage}"
*/
loadingMessage?: HTMLString;

/**
* CSS style to use for the DetailViewer.loadingMessage.
* @type {CSSStyleName}
* @default "normal"
*/
loadingMessageStyle?: CSSStyleName;

/**
* Whether to show fields marked detail:true when a DataBoundComponent is 
*  given a DataSource but no component.fields.
*  
*  The detail property is used on DataSource fields to mark fields that 
*  shouldn't appear by default in a view that tries to show many records in a small space.
* @type {boolean}
* @default true
*/
showDetailFields?: boolean;

/**
* Instance of FieldPickerWindow used if DetailViewer.canPickFields is set.
* @type {FieldPickerWindow}
* @default null
*/
fieldPickerWindow?: FieldPickerWindow;

/**
* If set, right-clicking on the DetailViewer will show a context menu that offers to bring up a
*  FieldPicker for configuring which fields are displayed and their order.
* @type {boolean}
* @default false
*/
canPickFields?: boolean;

/**
* The title for the Configure Fields menu item.
* @type {string}
* @default "Configure Fields..."
*/
configureFieldsText?: string;

/**
* Names of properties on DetailViewerField for which the FieldPicker should
*  show an editing interface, for convenience.
*  
*  For example, specify ["decimalPad", "decimalPrecision"] to allow end users to modify
*  DetailViewerField.decimalPad and DetailViewerField.decimalPrecision respectively.
* @type {Array<Partial<string>>}
* @default null
*/
fieldPickerFieldProperties?: Array<Partial<string>>;



/**
*  Sends the current filter criteria and sort direction to the server, then exports data in the
*  requested DSRequest.exportAs.
*  
*  A variety of DSRequest settings, such as 
*  DSRequest.exportAs and DSRequest.exportFilename, affect the 
*  exporting process: see DSRequest.exportResults for further detail.
*  
*  Note that data exported via this method skips client-side fields defined only in
*  the component, excludes any client-side formatting and
*  relies on both the SmartClient server and server-side DataSources. To export client-data 
*  including client-only fields and with client-side formatting applied, 
*  see ListGrid.exportClientData, which still requires the
*  SmartClient server but does not rely on server-side DataSource definitions (.ds.xml files).
*  
*  For more information on exporting data, see DataSource.exportData.
* 
* 
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
* @param {DSCallback=} callback to invoke on completion. Note that this parameter                 only applies where DSRequest.exportToClient is                  explicitly set to false, because file downloads do not                  provide ordinary SmartClient callbacks
*/
exportData(requestProperties?:DSRequest, callback?:DSCallback): void;

/**
*  Exports this component's data with client-side formatters applied, so is suitable for direct
*  display to users. See ListGrid.exportClientData for details of the general 
*  requirements and restrictions when exporting client data.
*  
*  The following notes apply when exporting client data from DetailViewers:
*  
* - Data is exported in "grid" format, with each record shown in a single row and each 
*    column representing a single field. This is quite different from the way DetailViewers
*    display records in the browser
*  
*  
*  If your detailViewer has custom formatters, formatted values will be exported by default, with
*  HTML normalized to text where possible. Since some levels of HTML normalizing aren't 
*  possible, this may result in missing or incorrect export values. In this case, you have 
*  two possible approaches:
*  
* - Set DetailViewerField.exportRawValues on the field. This will export
*    the raw underlying value of the field; your formatter will not be called
*  
* - Have your formatter call DetailViewer.isExportingClientData
*    and perform whatever alternative formatting you require if that method returns true
*  
* 
* @param {Partial<DSRequest>=} Request properties for the export. Note that specifying DSRequest.exportData on the request properties  allows the developer to pass in an explicit data set to export.
* @param {RPCCallback=} Optional callback. If you specify DSRequest.exportToClient: false in the request  properties, this callback will fire after export completes. Otherwise the callback will  fire right before the download request is made to the server.
*/
exportClientData(requestProperties?:Partial<DSRequest>, callback?:(response:RPCResponse, rawData:any, request:RPCRequest) => void): void;

/**
*  Returns true if this component is currently 
*  DetailViewer.exportClientData. This method can be called from
*  custom cell formatters if you need to return a different formatted value for an export
*  than for a live detailViewer
* 
*/
isExportingClientData(): boolean;

/**
* 
*  Displays the currently selected record(s) of the selectionComponent widget (typically a
*  listGrid) in this component.
*  
*  For a DynamicForm the first record of the selection is shown after the form is placed
*  into DynamicForm.canEdit.
*  A subsequent call to DynamicForm.editRecord or similar will return the
*  form to editability.
*  
*  Note that since field-level canEdit:true settings override the form-level
*  canEdit setting the automatic change to read-only may not change every field.
* 
* 
* @param {ListGrid | TileGrid | GlobalId} the ListGrid or TileGrid or ID of a ListGrid/TileGrid whose currently   selected record(s) is/are to be viewed
*/
viewSelectedData(selectionComponent:ListGrid | TileGrid | GlobalId): void;

/**
*  Only supported on ListGrid for now.
* 
* @param {Array<Partial<Hilite>>} Array of hilite objects
*/
setHilites(hilites:Array<Partial<Hilite>>): void;

/**
*  Sets the data displayed by this detail viewer.
* 
* 
* @param {Array<Partial<DetailViewerRecord>> | Array<Partial<Record>> | any} new data to be displayed
*/
setData(newData:Array<Partial<DetailViewerRecord>> | Array<Partial<Record>> | any): void;

/**
*  Based on the relationship between the DataSource this component is bound to and the
*  DataSource specified as the "schema" argument, call fetchData() to retrieve records in this
*  data set that are related to the passed-in record.
*  
*  Relationships between DataSources are declared via DataSourceField.foreignKey.
*  
*  For example, given two related DataSources "orders" and "orderItems", where we want to fetch
*  the "orderItems" that belong to a given "order". "orderItems" should declare a field that
*  is a DataSourceField.foreignKey to the "orders" table (for example, it
*  might be named "orderId" with foreignKey="orders.id"). Then, to load the records related to
*  a given "order", call fetchRelatedData() on the component bound to "orderItems", pass the
*  "orders" DataSource as the "schema" and pass a record from the "orders" DataSource as the
*  "record" argument.
*  
*  Note: When you expect a large number of records to be returned it is not recommended to
*  display these in the DetailViewer as it doesn't have the same level of support for large
*  datasets as the ListGrid.
* 
* 
* @param {ListGridRecord} DataSource record
* @param {Canvas | DataSource | GlobalId} schema of the DataSource record, or              DataBoundComponent already bound to that schema
* @param {DSCallback=} callback to invoke on completion
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
*/
fetchRelatedData(record:ListGridRecord, schema:Canvas | DataSource | GlobalId, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Check whether a field is currently visible
* 
* 
* @param {string | DetailViewerField} field to be checked
*/
fieldIsVisible(field:string | DetailViewerField): boolean;

/**
*  Return CSS text for styling this cell, which will be applied in addition to the CSS class
*  for the cell, as overrides.
*  
*  "CSS text" means semicolon-separated style settings, suitable for inclusion in a CSS
*  stylesheet or in a STYLE attribute of an HTML element.
* 
* 
* @param {any} actual value of this cell
* @param {DetailViewerField} field object for this cell
* @param {Record} record object for this cell
* @param {DetailViewer} the viewer instance to which this cell belongs
*/
getCellCSSText(value:any, field:DetailViewerField, record:Record, viewer:DetailViewer): CSSText;

/**
*  Return the CSS class for a cell. Default implementation calls
*  DetailViewerField.getCellStyle on the field if defined, otherwise
*  returns DetailViewer.cellStyle
* 
* @param {any} actual value of this cell
* @param {DetailViewerField} field object for this cell
* @param {Record} record object for this cell
* @param {DetailViewer} the viewer instance to which this cell belongs
*/
getCellStyle(value:any, field:DetailViewerField, record:Record, viewer:DetailViewer): CSSStyleName;

/**
*  Optional method to format the value to display for cells in this DetailViewer.
*  Note that if DetailViewerField.formatCellValue is specified this method will not
*  be called for cells within that field.
* 
* @param {string} the raw value of the cell (may be formatted by  DetailViewerField.formatCellValue
* @param {DetailViewerRecord} the record being displayed
* @param {DetailViewerField} the field being displayed
*/
formatCellValue(value:string, record:DetailViewerRecord, field:DetailViewerField): void;

/**
*  Get the index of the provided record.
*  
*  Override in subclasses to provide more specific behavior, for instance, when data holds a
*  large number of records
* 
* 
* @param {Record} the record whose index is to be retrieved
*/
getRecordIndex(record:Record): number;

/**
*  Return the message to show if the component has no data. Default implementation returns a 
*  centered DetailViewer.emptyMessage or "&amp;nbsp;" if showEmptyMessage is false. If
*  the component has no data because the browser is offline, we instead display the 
*  DataBoundComponent.offlineMessage or "&amp;nbsp;" if showOfflineMessage is false
* 
*/
emptyMessageHTML(): string;

/**
*  Returns a snapshot of the current view state of this DetailViewer.
*  This includes the field state of the DetailViewer, returned as a
*  DetailViewerViewState object.
*  This object can be passed to DetailViewer.setViewState to reset this detail 
*  viewer's view state to the current state (assuming the same data / fields are present in 
*  the detail viewer).
* 
*/
getViewState(): DetailViewerViewState;

/**
*  Reset this detail viewer's view state to match the DetailViewerViewState object passed in.
*  Used to restore previous state retrieved from the detail viewer by a call to 
*  DetailViewer.getViewState.
* 
* 
* @param {DetailViewerViewState} Object describing the desired view state for the detail viewer
*/
setViewState(viewState:DetailViewerViewState): void;



}
/**
* Creates a new DetailViewer
* 
* @param typeCheckedProperties {Partial<DetailViewer>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DetailViewer} The new instance
*/
class DetailViewer{
static create(typeCheckedProperties? : Partial<DetailViewer>, uncheckedProperties?: Object): DetailViewer;







}

/**
* EditProxy that handles DetailViewer components when editMode is enabled.
*/
interface DetailViewerEditProxy extends CanvasEditProxy {
/**
* If EditProxy.inlineEditEvent for this viewer edits the
*  DetailViewer.data, character that should be used as a separator between
*  values, or between pairs of field name vs values if the user is entering such
*  a ValueMap using the DetailViewerEditProxy.dataDisplaySeparatorChar.
*  
*  If EditProxy.inlineEditMultiline is enabled, newlines will be used as value
*  separators and the dataSeparatorChar
*  
*  The DetailViewerEditProxy.dataEscapeChar can be used to enter the separator
*  char as part of a field name or value.
* @type {string}
* @default ","
*/
dataSeparatorChar?: string;

/**
* If EditProxy.inlineEditEvent for this viewer edits the
*  DetailViewer.data, character that should be used as a separator for
*  entering ValueMap-style entries that map from a field name to a value.
*  
*  With the default of ":", the following input:
*  
*    1:Fixed, 2:Won't Fix, 3:Resolved
*  
*  Would be assumed to be a mapping like this (expressed in JSON):
*  
*   {
*    "1" : "Fixed",
*    "2" : "Won't Fix",
*    "3" : "Resolved"
*   }
*  
*  
*  Any entry without a separator char has an implied value of null.
*  For example, for this input:
*  
*     Fixed:Reported Fixed, WontFix:Won't Fix, Resolved
*  
*  The resulting data would be:
*  
*   {
*    "Fixed" : "Reported Fixed",
*    "WontFix" : "Won't Fix",
*    "Resolved" : null
*   }
*  
*  
*  The DetailViewerEditProxy.dataEscapeChar can be used to enter literal colon characters.
*  
*  Set dataDisplaySeparatorChar to null to prevent entry of values
*  - user input will always be treated as just a list of legal field names.
* @type {string}
* @default ":"
*/
dataDisplaySeparatorChar?: string;

/**
* If EditProxy.inlineEditEvent for this viewer edits the
*  DetailViewer.data, character that can be used to enter literal separator
*  chars (such as the DetailViewerEditProxy.dataSeparatorChar) or literal
*  leading or trailing whitespace.
*  
*  Repeat this character twice to enter it literally. For example, with the default
*  of "\", inputting "\\" would result in a literal backslash in the value.
* @type {string}
* @default "\"
*/
dataEscapeChar?: string;



/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's data one-field-per-line as specified in
*  DetailViewerEditProxy.dataDisplaySeparatorChar.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's data and fields.
* 
* 
* @param {string} the new component data
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new DetailViewerEditProxy
* 
* @param typeCheckedProperties {Partial<DetailViewerEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DetailViewerEditProxy} The new instance
*/
class DetailViewerEditProxy{
static create(typeCheckedProperties? : Partial<DetailViewerEditProxy>, uncheckedProperties?: Object): DetailViewerEditProxy;







}

/**
* Dialogs are a specialized version of Window used for small windows that contain just
*  a text message or a text mesage with some standard buttons.
*  
*  Many typical modal dialogs such as alerts and confirmations are built into the system with
*  convenience APIs - see isc.say, isc.warn 
*  and isc.askForValue
*  .
*  
*  Dialogs can be modal or non-modal according to Window.isModal.
*  
*  NOTE: If you are building a dialog that will involve more than just buttons and a message,
*  consider starting from the Window class instead, where arbitrary components can be
*  added to the body area via Window.addItem.
*  
*  This is an example of creating a custom dialog:
*  
*  
*  isc.Dialog.create({
*    message : "Please choose whether to proceed",
*    icon:"[SKIN]ask.png",
*    buttons : [
*      isc.Button.create({ title:"OK" }),
*      isc.Button.create({ title:"Cancel" })
*    ],
*    buttonClick : function (button, index) {
*      this.hide();
*    }
*  });
*  
*  
*  
*/
interface Dialog extends Window {
/**
* @type {Integer}
* @default "360"
*/
defaultWidth?: Integer;

/**
* Style of the Dialog background
* @type {CSSStyleName}
* @default "dialogBackground"
*/
styleName?: CSSStyleName;

/**
* Message to show in this dialog.
*  
*  If a message is set the primary purpose of the dialog will be assumed to be to show a message with
*  buttons - auto-sizing to the message text will be enabled, and, if Dialog.icon has also
*  been set, the Dialog.messageLabel and Dialog.messageIcon
*  AutoChildren will be created and placed together in the Dialog.messageStack
*  AutoChild, with the toolbar underneath as usual. If any of these behaviors are inconvenient or
*  you want more precise control over a message and some custom widgets, start from the superclass
*  Window instead, and add controls via Window.addItem.
*  
*  The message string may contain "${loadingImage}", if so, the standard loading spinner will appear at
*  that location in the text (see Canvas.loadingImageSrc).
*  
*  The message will be styled with the Dialog.messageStyle.
* @type {HTMLString}
* @default null
*/
message?: HTMLString;

/**
* Style to apply to the message text shown in the center of the dialog
* @type {CSSStyleName}
* @default "normal"
*/
messageStyle?: CSSStyleName;

/**
* AutoChild that shows Dialog.message.
* @type {Label}
* @default null
*/
messageLabel?: Label;

/**
* AutoChild that shows Dialog.icon.
* @type {Img}
* @default null
*/
messageIcon?: Img;

/**
* AutoChild that combines Dialog.message and Dialog.icon.
* @type {Layout}
* @default null
*/
messageStack?: Layout;

/**
* Icon to show in this dialog - see Dialog.message.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* Size of the icon to show in this dialog.
* @type {Integer}
* @default "32"
*/
iconSize?: Integer;

/**
* Whether to show a toolbar of buttons at the bottom of the Dialog.
*  Default of null will cause the value to be resolved automatically to true or
*  false when the Dialog is first drawn according as Dialog.toolbarButtons
*  contains buttons or not.
* @type {boolean}
* @default null
*/
showToolbar?: boolean;

/**
* If a toolbar is showing, automatically place keyboard focus in the first button.
*  
*  An alternative button can be specified by Dialog.autoFocusButton.
* @type {boolean}
* @default true
*/
autoFocus?: boolean;

/**
* If a toolbar is showing and Dialog.autoFocus is enabled, which button should
*  receive initial focus.
* @type {Canvas | string | Integer}
* @default true
*/
autoFocusButton?: Canvas | string | Integer;

/**
* AutoChild of type Toolbar used to create the Dialog.toolbarButtons.
* @type {Toolbar}
* @default null
*/
toolbar?: Toolbar;

/**
* Array of Buttons to show in the Dialog.showToolbar, if shown.
*  
*  The set of buttons to use is typically set by calling one of the shortcuts such as
*  isc.say or isc.confirm
*  .
*  A custom set of
*  buttons can be passed to these shortcuts methods via the "properties" argument, or to a
*  directly created Dialog.
*  
*  In both cases, a mixture of DialogButtons, custom buttons,
*  and other components (such as a LayoutSpacer) can be passed. 
*  
*  Built-in buttons can be referred to as isc.Dialog.OK, for example:
*  
*  isc.Dialog.create({
*   buttons:[
*     isc.Dialog.OK,
*     isc.Dialog.CANCEL,
*     isc.LayoutSpacer.create({width:50}),
*     { title:"Not now", click:"doSomething()" }
*   ]
*  })
*  
*  Built-in buttons will call standard methods on the Dialog itself, such as
*  Dialog.cancelClick, as explained in the
*  DialogButtons.
*  
*  
* @type {Array<Partial<Button>> | Array<Partial<Partial<Button>>>}
* @default null
*/
buttons?: Array<Partial<Button>> | Array<Partial<Partial<Button>>>;

/**
* This is a synonym for Dialog.buttons
* @type {Array<Partial<Button>> | Array<Partial<Partial<Button>>>}
* @default null
*/
toolbarButtons?: Array<Partial<Button>> | Array<Partial<Partial<Button>>>;

/**
* Icon to show in the isc.ask
*  dialog.
* @type {SCImgURL}
* @default "[SKIN]ask.png"
*/
askIcon?: SCImgURL;

/**
* Icon to show in the isc.say
*  dialog.
* @type {SCImgURL}
* @default "[SKIN]say.png"
*/
sayIcon?: SCImgURL;

/**
* Icon to show in the isc.warn
*  dialog.
* @type {SCImgURL}
* @default "[SKIN]warn.png"
*/
warnIcon?: SCImgURL;

/**
* Icon to show in the isc.confirm
*  dialog.
* @type {SCImgURL}
* @default "[SKIN]confirm.png"
*/
confirmIcon?: SCImgURL;



/**
*  Set the buttons for the toolbar displayed in this dialog.
*  Synonym for Dialog.setToolbarButtons
* 
* @param {Array<Partial<Button>> | Array<Partial<Partial<Button>>>} [null] buttons for the toolbar
*/
setButtons(newButtons:Array<Partial<Button>> | Array<Partial<Partial<Button>>>): void;

/**
*  Set the Dialog.toolbarButtons for this dialog.
*  Synonym for Dialog.setButtons.
* 
* @param {Array<Partial<Button>> | Array<Partial<Partial<Button>>>} [null] buttons for the toolbar
*/
setToolbarButtons(newButtons:Array<Partial<Button>> | Array<Partial<Partial<Button>>>): void;

/**
*  Method to update the message on this Dialog.
*  
* 
* @param {HTMLString} new message to show
*/
setMessage(newMessage:HTMLString): void;

/**
*  Method to save this Dialog's data. Called from okClick(),
*  applyClick().
*  No default implementation - override to perform some action if required.
* 
* 
*/
saveData(): void;

/**
*  Handles a click on the close button of this window. The default implementation
*  calls Window.close and returns false to prevent bubbling of the click event.
*  
*  Override this method if you want
*  other actions to be taken.
*  
*  Custom implementations may call close() to trigger the default behavior.
* 
*/
closeClick(): boolean;

/**
*  Handle a click on the 'cancel' button of this Dialog.
*  Default implementation is to return null and hide the Dialog.
*  Override to do something else.
* 
*/
cancelClick(): void;

/**
*  Handle a click on the 'ok' button of this Dialog.
*  Default implementation is to call saveData(), hide the Dialog, then return
*  true.
*  Override to do something else.
* 
*/
okClick(): void;

/**
*  Handle a click on the 'apply' button of this Dialog.
*  Default implementation is to call saveData(), but NOT close the Dialog.
* 
*/
applyClick(): void;

/**
*  Handle a click on the 'yes' button of this Dialog.
*  Default implementation is to return true.
*  Override to do something else
* 
*/
yesClick(): void;

/**
*  Handle a click on the 'no' button of this Dialog.
*  Default implementation is to return false.
*  Override to do something else.
* 
*/
noClick(): void;

/**
*  Handle a click on the 'done' button of this Dialog.
*  Default implementation is to hide the dialog then return true.
*  Override to do something else.
* 
*/
doneClick(): void;

/**
*  Fires when any button in this Dialog's toolbar is clicked. Default implementation does nothing.
* 
* 
* @param {StatefulCanvas} button that was clicked
* @param {Integer} index of the button that was clicked
*/
buttonClick(button:StatefulCanvas, index:Integer): void;



}
/**
* Creates a new Dialog
* 
* @param typeCheckedProperties {Partial<Dialog>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Dialog} The new instance
*/
class Dialog{
static create(typeCheckedProperties? : Partial<Dialog>, uncheckedProperties?: Object): Dialog;
/**
* Title for the "OK" button.
* @type {HTMLString}
* @default "OK"
*/
static OK_BUTTON_TITLE?: HTMLString;

/**
* Title for the "Apply" button.
* @type {HTMLString}
* @default "Apply"
*/
static APPLY_BUTTON_TITLE?: HTMLString;

/**
* Title for the "Yes" button.
* @type {HTMLString}
* @default "Yes"
*/
static YES_BUTTON_TITLE?: HTMLString;

/**
* Title for the "No" button.
* @type {HTMLString}
* @default "No"
*/
static NO_BUTTON_TITLE?: HTMLString;

/**
* Title for the "Cancel" button.
* @type {HTMLString}
* @default "Cancel"
*/
static CANCEL_BUTTON_TITLE?: HTMLString;

/**
* Title for the "Done" button.
* @type {HTMLString}
* @default "Done"
*/
static DONE_BUTTON_TITLE?: HTMLString;

/**
* Default title for the dialog displayed in response to the isc.confirm method.
*  Note that a custom title can be specified as the title attribute of the
*  properties parameter passed to that method.
* @type {HTMLString}
* @default "Confirm"
*/
static CONFIRM_TITLE?: HTMLString;

/**
* Default title for the dialog displayed in response to the isc.say method.
*  Note that a custom title can be specified as the title attribute of the
*  properties parameter passed to that method.
* @type {HTMLString}
* @default "Note"
*/
static SAY_TITLE?: HTMLString;

/**
* Default title for the dialog displayed in response to the isc.warn method.
*  Note that a custom title can be specified as the title attribute of the
*  properties parameter passed to that method.
* @type {HTMLString}
* @default "Warning"
*/
static WARN_TITLE?: HTMLString;

/**
* Default title for the dialog displayed in response to the isc.ask method.
*  Note that a custom title can be specified as the title attribute of the
*  properties parameter passed to that method.
* @type {HTMLString}
* @default "Question"
*/
static ASK_TITLE?: HTMLString;

/**
* Default title for the dialog displayed by isc.askForValue.
*  A custom title can alternatively be specified as the title attribute of the
*  properties parameter passed to that method.
* @type {HTMLString}
* @default "Please enter a value"
*/
static ASK_FOR_VALUE_TITLE?: HTMLString;

/**
* The "Prompt" object on the dialog class is a singleton Dialog instance.
*  The Prompt is used to show text to the user in a modal fashion - it will expand to show
*  all the text that you put into it.
*  By default this Dialog has no end-user controls and is expected to be programmatically
*  dismissed.
*  Common use-case: During server-interactions, the Prompt will be used to display a suitable
*  wait message, and suppress user input.
* 
*  Notes:
*  Because this is a singleton object, properties set on the Prompt directly will persist each
*  time it is shown.
*  Developers should use the showPrompt() and clearPrompt() methods
*  to show and hide the prompt rather than manipulating the prompt directly.
* @type {Partial<Dialog>}
* @default "dialog instance properties"
*/
static Prompt?: Partial<Dialog>;

/**
* A singleton Dialog instance that will show text to the user and provide buttons for their
*  response. The Dialog will expand to show all the text that you put into it. This dialog
*  is shown in response to calls to isc.say,
*  isc.warn, isc.ask and
*  isc.confirm.
*  
*  This can be used in cases where a developer would alternatively make use of the native
*  JavaScript alert() and confirm() methods. The main differences
*  between those methods and using the Warn object are:
*  - The Warn object can be customized by modifying which buttons are visible, the style
*   applied to it, etc.
*  - The isc.ask() and isc.warn() methods are asynchronous - rather
*   than returning a value indicating the user's response, a callback method will be fired
*   when the user interacts with the dialog.
*  
*  Notes:
*  Because this is a singleton object, properties set on the Warn object directly will persist
*  each time it is shown.
*  Developers should use the warn() or ask() methods to show and
*  hide this object rather than manipulating the Dialog directly.
* @type {Partial<Dialog>}
* @default "dialog instance properties"
*/
static Warn?: Partial<Dialog>;

/**
* A singleton Dialog instance that will be shown in response to a isc.askForValue call.
* 
*  Notes:
*  Because this is a singleton object, properties set on the Ask object directly will persist
*  each time it is shown.
*  Developers should use the askForValue() method to show this object rather than
*  manipulating the Dialog directly.
* @type {Partial<Dialog>}
* @default "dialog instance properties"
*/
static Ask?: Partial<Dialog>;








}

/**
* TextForm item for managing a text field that displays a decimal value.
*/
interface DoubleItem extends FloatItem {


/**
*  Return the value tracked by this form item as a Double. If the value cannot
*  be parsed to a valid double, null will be returned.
* 
* 
*/
getValueAsDouble(): Double;



}
/**
* Creates a new DoubleItem
* 
* @param typeCheckedProperties {Partial<DoubleItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DoubleItem} The new instance
*/
class DoubleItem{
static create(typeCheckedProperties? : Partial<DoubleItem>, uncheckedProperties?: Object): DoubleItem;







}

/**
* DrawItem subclass to render multi-segment, orthogonal-routing paths.
*/
interface DrawBlockConnector extends DrawCurve {




}
/**
* Creates a new DrawBlockConnector
* 
* @param typeCheckedProperties {Partial<DrawBlockConnector>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawBlockConnector} The new instance
*/
class DrawBlockConnector{
static create(typeCheckedProperties? : Partial<DrawBlockConnector>, uncheckedProperties?: Object): DrawBlockConnector;







}

/**
* DrawItem that renders cubic bezier curves.
*/
interface DrawCurve extends DrawItem {
/**
* Array of control knobs to display for this item. Each KnobType specified in this
*  will turn on UI element(s) allowing the user to manipulate this DrawCurve. To update the
*  set of knobs at runtime use DrawItem.showKnobs and DrawItem.hideKnobs.
*  
*  DrawCurve supports the
*  "startPoint", "endPoint", "controlPoint1", and "controlPoint2"
*  
*  knob types.
* @type {Array<Partial<KnobType>>}
* @default null
*/
knobs?: Array<Partial<KnobType>>;

/**
* If the DrawItem.titleLabel is showing, should the DrawItem.titleLabelBackground
*  be created and placed behind the titleLabel?
*  
*  This defaults to true for DrawSectors and shapes that are not commonly filled
*  (e.g. DrawLines).
* @type {boolean}
* @default true
*/
showTitleLabelBackground?: boolean;

/**
* Start point of the curve
* @type {Point}
* @default "[0,0]"
*/
startPoint?: Point;

/**
* End point of the curve
* @type {Point}
* @default "[100,100]"
*/
endPoint?: Point;

/**
* First cubic bezier control point.
* @type {Point}
* @default "[100,0]"
*/
controlPoint1?: Point;

/**
* Second cubic bezier control point.
* @type {Point}
* @default "[0,100]"
*/
controlPoint2?: Point;

/**
* Style of drawing the endpoints of a line.
*  
*  Note that for dashed and dotted lines, the lineCap style affects each dash or dot.
* @type {LineCap}
* @default "butt"
*/
lineCap?: LineCap;

/**
* If this item is showing "controlPoint1"
*  
*  DrawItem.knobs, this attribute specifies the AutoChild for the
*  DrawKnob for control point 1 of current drawCurve.
* @type {DrawKnob}
* @default null
*/
c1Knob?: DrawKnob;

/**
* If this item is showing "controlPoint2"
*  
*  DrawItem.knobs, this attribute specifies the AutoChild for the
*  DrawKnob for control point 2 of current drawCurve.
* @type {DrawKnob}
* @default null
*/
c2Knob?: DrawKnob;



/**
*  Get the center point of the rectangle from the curve's DrawCurve.startPoint
*  to the DrawCurve.endPoint.
* 
*/
getCenter(): Point;

/**
*  Update the startPoint
* 
* 
* @param {Coordinate} left coordinate for start point, in pixels
* @param {Coordinate} top coordinate for start point, in pixels
*/
setStartPoint(left:Coordinate, top:Coordinate): void;

/**
*  Update the endPoint
* 
* 
* @param {Coordinate} left coordinate for end point, in pixels
* @param {Coordinate} top coordinate for end point, in pixels
*/
setEndPoint(left:Coordinate, top:Coordinate): void;

/**
*  Updates the first cubic B&#233;zier control point.
* 
* 
* @param {Coordinate} left coordinate for control point, in pixels
* @param {Coordinate} top coordinate for control point, in pixels
*/
setControlPoint1(left:Coordinate, top:Coordinate): void;

/**
*  Updates the second cubic B&#233;zier control point.
* 
* 
* @param {Coordinate} left coordinate for control point, in pixels
* @param {Coordinate} top coordinate for control point, in pixels
*/
setControlPoint2(left:Coordinate, top:Coordinate): void;

/**
*  Returns the smallest box containing the entire curve.
* 
*/
getBoundingBox(): Array<Partial<Double>>;

/**
*  Move the start point, end point, and control points of the curve such that the
*  DrawCurve.startPoint ends up at the specified coordinates and the shape of the curve is
*  unchanged.
* 
* 
* @param {Integer} new x coordinate in pixels
* @param {Integer} new y coordinate in pixels
*/
moveStartPointTo(x:Integer, y:Integer): void;

/**
*  Increment start, end and control points of this curve
* 
* 
* @param {Distance} new x coordinate in pixels
* @param {Distance} new y coordinate in pixels
*/
moveBy(x:Distance, y:Distance): void;



}
/**
* Creates a new DrawCurve
* 
* @param typeCheckedProperties {Partial<DrawCurve>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawCurve} The new instance
*/
class DrawCurve{
static create(typeCheckedProperties? : Partial<DrawCurve>, uncheckedProperties?: Object): DrawCurve;







}

/**
* DrawItem subclass to render diamond shapes
*/
interface DrawDiamond extends DrawItem {
/**
* Left coordinate of the diamond. This is the X coordinate of the western point of the
*  diamond.
* @type {Coordinate}
* @default "0"
*/
left?: Coordinate;

/**
* Top coordinate of the diamond. This is the Y coordinate of the northern point of the
*  diamond.
* @type {Coordinate}
* @default "0"
*/
top?: Coordinate;

/**
* Width of the diamond. Must be non-negative.
* @type {Distance}
* @default "100"
*/
width?: Distance;

/**
* Height of the diamond. Must be non-negative.
* @type {Distance}
* @default "100"
*/
height?: Distance;

/**
* The mode in which the DrawItem.titleLabel (if shown) is rotated with
*  this draw item.
* @type {TitleRotationMode}
* @default "withItemAlwaysUp"
*/
titleRotationMode?: TitleRotationMode;



/**
*  Move and resize the drawDiamond to match the specified coordinates and size.
* 
* @param {Integer} new left coordinate
* @param {Integer} new top coordinate
* @param {Integer} new width
* @param {Integer} new height
*/
setRect(left:Integer, top:Integer, width:Integer, height:Integer): void;



}
/**
* Creates a new DrawDiamond
* 
* @param typeCheckedProperties {Partial<DrawDiamond>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawDiamond} The new instance
*/
class DrawDiamond{
static create(typeCheckedProperties? : Partial<DrawDiamond>, uncheckedProperties?: Object): DrawDiamond;







}

/**
* DrawItem subclass to manage a group of other DrawItem instances.
*  
*  A DrawGroup has no local visual representation other than that of its drawItems. Adding items
*  to a drawGroup allows for central event handling, and allows them to be manipulated
*  (drawn, scaled, etc) together.
*  
*  DrawItems are added to a DrawGroup by creating the DrawItems with DrawItem.drawGroup
*  set to the DrawGroup, or by creating a DrawGroup with DrawGroup.drawItems.
*  
*  DrawGroups handle events by having an explicitly specified group rectangle 
*  (see DrawGroup.getGroupRect). This rectangle has no visual representation within the draw pane
*  (is not visible) but any user-interactions within the specified coordinates will trigger
*  group level events.
*  
*  DrawGroups may contain other DrawGroups.
*/
interface DrawGroup extends DrawItem {
/**
* DrawGroup only supports the
*  "move"
*  
*  knob type.
* @type {Array<Partial<KnobType>>}
* @default null
*/
knobs?: Array<Partial<KnobType>>;

/**
* Should dragging the group (when DrawItem.canDrag is true) or dragging the
*  move knob also move the items within this DrawGroup?
* @type {boolean}
* @default true
*/
moveItemsWithGroup?: boolean;

/**
* When should this drawGroup receive event notifications?
*  If set to true, the developer can specify an explicit 
*  DrawGroup.getGroupRect. Whenever the user interacts with this
*  rectangle, the drawGroup will be notified and the appropriate event handlers will be
*  fired. Note that rectangle need not contain all DrawItems within the group, and 
*  is manually managed by the developer.
*  If set to false, the DrawGroup.getGroupRect 
*  coordinates are unused - instead
*  as a user interacts with specific drawItems within this group, the appropriate event handler
*  would be fired on the item, then the event would "bubble" to the
*  drawGroup, firing the appropriate event handler at the group level as well.
* @type {boolean}
* @default false
*/
useGroupRect?: boolean;

/**
* Left coordinate of the DrawGroup.getGroupRect in pixels relative to the
*  DrawPane (the "drawing coordinate system").
* @type {Coordinate}
* @default "0"
*/
left?: Coordinate;

/**
* Top coordinate of the DrawGroup.getGroupRect in pixels relative to the
*  DrawPane (the "drawing coordinate system").
* @type {Coordinate}
* @default "0"
*/
top?: Coordinate;

/**
* Width of the DrawGroup.getGroupRect in pixels relative to the
*  DrawPane (the "drawing coordinate system").
* @type {Distance}
* @default "1"
*/
width?: Distance;

/**
* Height of the DrawGroup.getGroupRect in pixels relative to the
*  DrawPane (the "drawing coordinate system").
* @type {Distance}
* @default "1"
*/
height?: Distance;

/**
* Initial list of DrawItems for this DrawGroup.
*  
*  DrawItems can be added to a DrawGroup after initialization by setting
*  DrawItem.drawGroup.
* @type {Array<Partial<DrawItem>>}
* @default null
*/
drawItems?: Array<Partial<DrawItem>>;

/**
* If the "move"
*  
*  DrawItem.knobs is shown for this group and DrawGroup.useGroupRect
*  is true, should the DrawGroup.groupRectOutline be shown?
*  
*  Set to false to disable showing the groupRectOutline.
* @type {boolean}
* @default null
*/
showGroupRectOutline?: boolean;

/**
* If this group is showing a "move"
*  
*  DrawItem.knobs, the groupRectOutline is a DrawRect AutoChild
*  that identifies the group's group rect (see DrawGroup.useGroupRect).
*  
*  useGroupRect must be true and the "move"
*  control knob must be
*  showing for the groupRectOutline AutoChild to be created and shown.
* @type {DrawRect}
* @default null
*/
groupRectOutline?: DrawRect;



/**
*  Erases all DrawItems in the DrawGroup.
* 
* 
*/
erase(): void;

/**
*  Sets the left coordinate of this DrawGroup's DrawGroup.getGroupRect.
*  Note that setting the left coordinate will not move the items in this DrawGroup.
* 
* 
* @param {Coordinate} new left coordinate
*/
setLeft(left:Coordinate): void;

/**
*  Sets the top coordinate of this DrawGroup's DrawGroup.getGroupRect.
*  Note that setting the top coordinate will not move the items in this DrawGroup.
* 
* 
* @param {Coordinate} new top coordinate in pixels
*/
setTop(top:Coordinate): void;

/**
*  Sets the width of this DrawGroup's DrawGroup.getGroupRect.
*  Note that setting the width will not move or resize the items in this DrawGroup.
* 
* 
* @param {Distance} new width for the group rectangle
*/
setWidth(width:Distance): void;

/**
*  Sets the height of this DrawGroup's DrawGroup.getGroupRect.
*  Note that setting the height will not move or resize the items in this DrawGroup.
* 
* 
* @param {Distance} new height for the group rectangle
*/
setHeight(height:Distance): void;

/**
*  This method will return an array of integers (left, top, width, height) defining the area
*  of the "group rectangle" for the group. If DrawGroup.useGroupRect is true, this is
*  the area of the DrawPane where user interactions will fire event notifications on this DrawGroup.
*  
*  This is a convienence method to get the current coordinates of the
*  DrawGroup.useGroupRect. Developers must use
*  DrawGroup.setLeft, DrawGroup.setTop, DrawGroup.setWidth or
*  DrawGroup.setHeight to set each coordinate directly.
* 
*/
getGroupRect(): Array<Partial<Integer>>;

/**
*  Sets both the left and top coordinates of this DrawGroup's DrawGroup.getGroupRect.
*  Note that this does not move or resize the items in this DrawGroup.
* 
* 
* @param {Integer} new left coordinate in pixels
* @param {Integer} new top coordinate in pixels
*/
moveTo(left:Integer, top:Integer): void;

/**
*  Updates the DrawGroup's left coordinate by dX and the top coordinate
*  by dY. Note that this does not move or resize the items in this DrawGroup.
* 
* 
* @param {Distance} change to left coordinate in pixels
* @param {Distance} change to top coordinate in pixels
*/
moveBy(dX:Distance, dY:Distance): void;

/**
*  Rotate each item in the group to the specified number of degrees.
* 
* 
* @param {Float} */
rotateTo(degrees:Float): void;

/**
*  Rotate each item in the group by the specified number of degrees.
* 
* 
* @param {Float} */
rotateBy(degrees:Float): void;

/**
*  Scale all drawItem[] shapes by the x, y multipliers
* 
* @param {Float} scale in the x direction
* @param {Float} scale in the y direction
*/
scaleBy(x:Float, y:Float): void;

/**
*  Get the center point of the DrawGroup.getGroupRect.
* 
*/
getCenter(): Point;

/**
*  Returns the left, top, (left + width), and (top + height) values
* 
*/
getBoundingBox(): Array<Partial<Double>>;

/**
*  Scale the each item in the drawGroup by the x, y multipliers
* 
* @param {Float} scale in the x direction
* @param {Float} scale in the y direction
*/
scaleTo(x:Float, y:Float): void;

/**
*  Notification fired when the user starts to drag this DrawGroup. Will only fire if DrawItem.canDrag
*  is true for this group.
*  
*  Note that if DrawGroup.useGroupRect is true, this notification will be triggered
*  by the user interacting with the specified DrawGroup.getGroupRect for the group.
*  If DrawGroup.useGroupRect is false, the notification will bubble up from interactions
*  with individual items within the group.
*  
*  Default drag behavior will be to reposition all items in the group (and update the group
*  rectangle).
* 
*/
dragStart(): boolean;

/**
*  Notification fired for every mouseMove event triggered while the user is dragging this
*  DrawGroup. Will only fire if DrawItem.canDrag is true for this group.
*  
*  Note that if DrawGroup.useGroupRect is true, this notification will be triggered
*  by the user interacting with the specified DrawGroup.getGroupRect for the group.
*  If DrawGroup.useGroupRect is false, the notification will bubble up from interactions
*  with individual items within the group.
*  
*  Default drag behavior will be to reposition all items in the group (and update the group rectangle).
* 
*/
dragMove(): boolean;

/**
*  Notification fired when the user stops dragging this DrawGroup. Will only fire if DrawItem.canDrag
*  is true for this group.
*  
*  Note that if DrawGroup.useGroupRect is true, this notification will be triggered
*  by the user interacting with the specified DrawGroup.getGroupRect for the group.
*  If DrawGroup.useGroupRect is false, the notification will bubble up from interactions
*  with individual items within the group.
* 
*/
dragStop(): boolean;

/**
*  Notification fired when the user presses the left mouse button on this DrawGroup.
*  
*  Note that if DrawGroup.useGroupRect is true, this notification will be triggered
*  by the user interacting with the specified DrawGroup.getGroupRect for the group.
*  If DrawGroup.useGroupRect is false, the notification will bubble up from interactions
*  with individual items within the group.
* 
*/
mouseDown(): boolean;

/**
*  Notification fired when the user releases the left mouse button on this DrawGroup.
*  
*  Note that if DrawGroup.useGroupRect is true, this notification will be triggered
*  by the user interacting with the specified DrawGroup.getGroupRect for the group.
*  If DrawGroup.useGroupRect is false, the notification will bubble up from interactions
*  with individual items within the group.
* 
*/
mouseUp(): boolean;

/**
*  Notification fired when the user clicks on this DrawGroup.
*  
*  Note that if DrawGroup.useGroupRect is true, this notification will be triggered
*  by the user interacting with the specified DrawGroup.getGroupRect for the group.
*  If DrawGroup.useGroupRect is false, the notification will bubble up from interactions
*  with individual items within the group.
* 
*/
click(): boolean;

/**
*  Notification fired when the mouse enters this DrawGroup.
*  
*  Note that if DrawGroup.useGroupRect is true, this notification will be triggered
*  by the user interacting with the specified DrawGroup.getGroupRect for the group.
*  If DrawGroup.useGroupRect is false, the notification will bubble up from interactions
*  with individual items within the group.
* 
*/
mouseOver(): boolean;

/**
*  Notification fired when the user moves the mouse over this DrawGroup.
*  
*  Note that if DrawGroup.useGroupRect is true, this notification will be triggered
*  by the user interacting with the specified DrawGroup.getGroupRect for the group.
*  If DrawGroup.useGroupRect is false, the notification will bubble up from interactions
*  with individual items within the group.
* 
*/
mouseMove(): boolean;

/**
*  Notification fired when the mouse leaves this DrawGroup.
*  
*  Note that if DrawGroup.useGroupRect is true, this notification will be triggered
*  by the user interacting with the specified DrawGroup.getGroupRect for the group.
*  If DrawGroup.useGroupRect is false, the notification will bubble up from interactions
*  with individual items within the group.
* 
*/
mouseOut(): boolean;



}
/**
* Creates a new DrawGroup
* 
* @param typeCheckedProperties {Partial<DrawGroup>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawGroup} The new instance
*/
class DrawGroup{
static create(typeCheckedProperties? : Partial<DrawGroup>, uncheckedProperties?: Object): DrawGroup;







}

/**
* DrawItem subclass to render embedded images.
*/
interface DrawImage extends DrawItem {
/**
* Left coordinate in pixels relative to the DrawPane.
* @type {Coordinate}
* @default "0"
*/
left?: Coordinate;

/**
* Top coordinate in pixels relative to the DrawPane.
* @type {Coordinate}
* @default "0"
*/
top?: Coordinate;

/**
* Width in pixels relative to the DrawPane.
* @type {Distance}
* @default "16"
*/
width?: Distance;

/**
* Height in pixels relative to the DrawPane.
* @type {Distance}
* @default "16"
*/
height?: Distance;

/**
* Title (tooltip hover text) for this image.
* @type {string}
* @default null
*/
title?: string;

/**
* URL to the image file.
* @type {URL}
* @default "blank.png"
*/
src?: URL;

/**
* Configures whether a Matrix filter is used to render this DrawImage in Internet
*  Explorer 6-8.
*  
*  With the default of null, matrix filters may be used unless
*  Canvas.neverUseFilters has been set. When set explicitly to
*  true, matrix filters may be used even if neverUseFilters is
*  true.
*  
*  This setting exists due to platform limitations in Internet Explorer where it is
*  impossible to implement shearing transforms on an image without a matrix filter. Shear
*  can arise by:
*  
*  
* - specifying a nonzero DrawItem.xShearFactor or
*  DrawItem.yShearFactor,
*  
* - specifying a nonuniform DrawItem.scale (where the scale values along
*  the x- and y-dimensions are not equal) and a nonzero DrawItem.rotation,
*  or
*  
* - using "resize"
*  
*  DrawItem.knobs on a rotated DrawImage.
*  
*  When prohibited from using a matrix filter, DrawImage will ignore the shearing
*  components of its local transform. If any of the above conditions are met then the
*  DrawImage might not be drawn correctly. Setting useMatrixFilter to
*  true avoids this possibility but it also suffers from a range of
*  side-effects mentioned IEFilters.
* @type {boolean}
* @default null
*/
useMatrixFilter?: boolean;



/**
*  Returns the top, left, top+width, left+height
* 
* 
*/
getBoundingBox(): Array<Partial<Double>>;

/**
*  Get the center point of the image.
* 
*/
getCenter(): Point;

/**
*  Change the URL of the image displayed.
* 
* @param {URL} new URL
*/
setSrc(src:URL): void;

/**
*  Set the left coordinate of the drawImage.
* 
* @param {Coordinate} new left coordinate
*/
setLeft(left:Coordinate): void;

/**
*  Set the top coordinate of the drawImage.
* 
* @param {Coordinate} new top coordinate
*/
setTop(top:Coordinate): void;

/**
*  Set the width of the drawImage.
* 
* @param {Distance} new width
*/
setWidth(width:Distance): void;

/**
*  Set the height of the drawImage.
* 
* @param {Distance} new height
*/
setHeight(height:Distance): void;

/**
*  Updates the drawImage to match the specified coordinates and size in 
*  DrawPane.
* 
* @param {Integer} new left coordinate
* @param {Integer} new top coordinate
* @param {Integer} new width
* @param {Integer} new height
*/
setRect(left:Integer, top:Integer, width:Integer, height:Integer): void;

/**
*  Move the drawImage by the specified delta
* 
* @param {Distance} number of pixels to move horizontally
* @param {Distance} number of pixels to move vertically
*/
moveBy(dX:Distance, dY:Distance): void;

/**
*  Move the drawImage to the specified position
* 
* @param {Integer} new left coordinate
* @param {Integer} new top coordinate
*/
moveTo(left:Integer, top:Integer): void;



}
/**
* Creates a new DrawImage
* 
* @param typeCheckedProperties {Partial<DrawImage>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawImage} The new instance
*/
class DrawImage{
static create(typeCheckedProperties? : Partial<DrawImage>, uncheckedProperties?: Object): DrawImage;







}

/**
* Base class for graphical elements drawn in a DrawPane. All properties and methods
*  documented here are available on all DrawItems unless otherwise specified.
*  
*  Each DrawItem has its own local transform that maps its 
*  DrawItem to the drawing coordinate system that is shared by
*  all DrawItems in the same DrawPane (explained DrawPane). The local
*  transform is a combination of rotation, scaling, and other affine transformations. The
*  DrawItem is first DrawItem.translate, then DrawItem.scale,
*  then DrawItem.xShearFactor in the direction of the x-axis, then
*  DrawItem.yShearFactor in the directiton of the y-axis, and then finally
*  DrawItem.rotation.
*  
*  Note that DrawItems as such should never be created, only concrete subclasses such as
*  DrawGroup and DrawLine.
*  
*  See DrawPane for the different approaches to create DrawItems.
*/
interface DrawItem extends BaseWidget {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "DrawItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Relative stacking order of this draw item with respect to other items in the same DrawPane
*  or DrawGroup.
*  
*  null means that the zIndex has not been resolved. Upon adding this draw item to a
*  DrawPane or DrawGroup, this item's zIndex will be resolved to
*  the next higher auto-assigned zIndex. Note that this may still be less than another item's
*  zIndex if DrawItem.bringToFront was called on that item.
*  
*  If two items within the same DrawPane or DrawGroup have the
*  same zIndex, then they are stacked in the order in which they were added to the
*  DrawPane or DrawGroup.
*  
*  When the DrawPane's DrawPane.drawingType is
*  "bitmap",
*  
*  zIndex, DrawItem.bringToFront, and DrawItem.sendToBack are not supported
*  for DrawLabels on iOS due to platform limitations.
* @type {Integer}
* @default null
*/
zIndex?: Integer;

/**
* If set, specifies the cursor to display when the mouse pointer is over this DrawItem.
* @type {Cursor}
* @default null
*/
cursor?: Cursor;

/**
* Will this DrawItem fire hover events when the user hovers over it?
* @type {boolean}
* @default null
*/
canHover?: boolean;

/**
* If DrawItem.canHover is true, should we show the global hover canvas by default
*  when the user hovers over this DrawItem?
* @type {boolean}
* @default true
*/
showHover?: boolean;

/**
* If this.canHover is true, how long should the mouse be kept over this
*  widget before the hover event is fired
* @type {number}
* @default "300"
*/
hoverDelay?: number;

/**
* Default DrawItem.getHoverHTML that is displayed in the global hover canvas if
*  the user hovers over this DrawItem and DrawItem.showHover is true.
* @type {HTMLString}
* @default null
*/
prompt?: HTMLString;

/**
* DrawPane this drawItem should draw in.
*  
*  If this item has a DrawGroup, the drawGroup's drawPane is automatically used.
* @type {DrawPane}
* @default null
*/
drawPane?: DrawPane;

/**
* DrawGroup this drawItem is a member of.
* @type {DrawGroup}
* @default null
*/
drawGroup?: DrawGroup;

/**
* Pixel width for lines.
* @type {Integer}
* @default "3"
*/
lineWidth?: Integer;

/**
* Line color
* @type {CSSColor}
* @default "#808080"
*/
lineColor?: CSSColor;

/**
* Opacity for lines, as a number between 0 (transparent) and 1 (opaque).
* @type {Float}
* @default "1.0"
*/
lineOpacity?: Float;

/**
* Pattern for lines, eg "solid" or "dash".
*  
*  Note that support in old browsers, such as Internet Explorer versions before IE11, is
*  limited for DrawPane.drawingType "bitmap" to items with straight
*  edges - DrawLines, DrawPaths, and DrawRects with no 
*  DrawRect.rounding.
* @type {LinePattern}
* @default "solid"
*/
linePattern?: LinePattern;

/**
* Style of drawing the endpoints of a line.
*  
*  Note that for dashed and dotted lines, the lineCap style affects each dash or dot.
* @type {LineCap}
* @default "round"
*/
lineCap?: LineCap;

/**
* Fill color to use for shapes. The default of 'null' is transparent.
* @type {CSSColor}
* @default null
*/
fillColor?: CSSColor;

/**
* Fill gradient to use for shapes. If a string it uses the gradient identifier parameter provided in 
*  DrawPane.addGradient. Otherwise it expects one of SimpleGradient,
*  LinearGradient or RadialGradient.
* @type {Gradient | string}
* @default null
*/
fillGradient?: Gradient | string;

/**
* Opacity of the fillColor, as a number between 0 (transparent) and 1 (opaque).
* @type {Float}
* @default "1.0"
*/
fillOpacity?: Float;

/**
* Shadow used for all DrawItem subtypes.
* @type {Shadow}
* @default null
*/
shadow?: Shadow;

/**
* Rotation in degrees about the DrawItem.getCenter.
*  The positive direction is clockwise.
* @type {Float}
* @default "0.0"
*/
rotation?: Float;

/**
* The slope of an x-shearing transformation applied to this DrawItem. Each point in the
*  shape is moved along the x-axis a distance that is proportional to the initial
*  y-coordinate of the point.
* @type {Float}
* @default "0.0"
*/
xShearFactor?: Float;

/**
* The slope of a y-shearing transformation applied to this DrawItem. Each point in the
*  shape is moved along the y-axis a distance that is proportional to the initial
*  x-coordinate of the point.
* @type {Float}
* @default "0.0"
*/
yShearFactor?: Float;

/**
* Array holds 2 values representing scaling along x and y dimensions.
* @type {Array<Partial<Float>>}
* @default null
*/
scale?: Array<Partial<Float>>;

/**
* Array holds two values representing translation along the x and y dimensions.
* @type {Array<Partial<Float>>}
* @default null
*/
translate?: Array<Partial<Float>>;

/**
* Style of arrow head to draw at the beginning of the line or path.
* @type {ArrowStyle}
* @default null
*/
startArrow?: ArrowStyle;

/**
* Style of arrow head to draw at the end of the line or path.
* @type {ArrowStyle}
* @default null
*/
endArrow?: ArrowStyle;

/**
* Array of control knobs to display for this item. Each KnobType specified in this array
*  will turn on UI element(s) allowing the user to manipulate this drawItem. To update the
*  set of knobs at runtime use DrawItem.showKnobs and DrawItem.hideKnobs.
*  
*  NOTE: Unless otherwise documented, DrawItem types only support
*  "resize" and "move"
*  
*  knobs.
* @type {Array<Partial<KnobType>>}
* @default null
*/
knobs?: Array<Partial<KnobType>>;

/**
* A string to show at the DrawItem.getCenter of this DrawItem.
*  
*  When set to a non-null value (including an empty string), the DrawItem.titleLabel
*  DrawLabel AutoChild will be created automatically and positioned at the center of
*  this DrawItem.
* @type {string}
* @default null
*/
title?: string;

/**
* When a non-null DrawItem.title is set, this AutoChild is created
*  automatically and positioned at the DrawItem.getCenter of this DrawItem
*  . The titleLabel moves with this DrawItem and, depending
*  on TitleRotationMode, can rotate with this DrawItem
*  as well.
*  
*  The following autoChildUsage applies:
*  DrawItem.title for DrawLabel.contents.
*  
*  Related to the titleLabel is the DrawItem.titleLabelBackground which allows a border to be placed around the titleLabel
*  and/or a background added. By default, shapes that are commonly filled such as
*  DrawTriangles, with the exception of DrawSectors, do not show the 
*  titleLabelBackground (see DrawItem.showTitleLabelBackground).
* @type {DrawLabel}
* @default null
*/
titleLabel?: DrawLabel;

/**
* Whether the DrawItem.titleLabel should be scaled to the maximum possible size that fits
*  inside the bounds of this item. Currently only DrawRects and DrawPolygons
*  with 90 degree angles are supported.
*  
*  Note that DrawItem.titleAutoFit isn't supported for rotated, sheared, or scaled
*  DrawItems, and that therefore the value of TitleRotationMode, which relates
*  to rotation of the item, is ignored when this property is set. However, we do support
*  having the label automatically rotate to run vertically if there's more space - see
*  DrawItem.titleAutoFitRotationMode.
* @type {boolean}
* @default false
*/
titleAutoFit?: boolean;

/**
* Whether to rotate the DrawItem.titleLabel 90 degrees clockwise while trying to maximize its
*  size in accordance with DrawItem.titleAutoFit. If automatic rotation is specified, the
*  default, the label will be rotated if and only if it allows the label to become larger.
* @type {TitleAutoFitRotationMode}
* @default "auto"
*/
titleAutoFitRotationMode?: TitleAutoFitRotationMode;

/**
* Specifies margin between label and item edges when using DrawItem.titleAutoFit.
* @type {number}
* @default "2"
*/
titleAutoFitMargin?: number;

/**
* The mode in which the DrawItem.titleLabel (if shown) is rotated with
*  this draw item.
* @type {TitleRotationMode}
* @default "neverRotate"
*/
titleRotationMode?: TitleRotationMode;

/**
* If the DrawItem.titleLabel is showing, should the DrawItem.titleLabelBackground
*  be created and placed behind the titleLabel?
*  
*  This defaults to true for DrawSectors and shapes that are not commonly filled
*  (e.g. DrawLines).
* @type {boolean}
* @default false
*/
showTitleLabelBackground?: boolean;

/**
* When the DrawItem.titleLabel is showing and DrawItem.showTitleLabelBackground
*  is true, this DrawRect AutoChild is created and placed behind the
*  titleLabel.
* @type {DrawRect}
* @default null
*/
titleLabelBackground?: DrawRect;

/**
* If the DrawItem.titleLabelBackground is visible, how much padding
*  should be left between the bounds of the DrawItem.titleLabel and the edges of
*  the titleLabelBackground?
* @type {Integer}
* @default "2"
*/
titleLabelPadding?: Integer;

/**
* Constrains drag-resizing and drag-repositioning of this draw item to either the current
*  visible area of the DrawPane or an arbitrary bounding box (if set to
*  an array of the form [left, top, left + width, top + height]). When using
*  a bounding box-type argument the left/top values can be negative, or the width/height
*  values can be greater than the dimensions of the viewable area, to allow positioning
*  or resizing the draw item beyond the confines of the draw pane.
*  
*  Note: keepInParentRect affects only user drag interactions, not programmatic moves or
*  resizes.
* @type {boolean | Array<Partial<Float>>}
* @default null
*/
keepInParentRect?: boolean | Array<Partial<Float>>;

/**
* If this DrawItem is showing "resize"
*  
*  DrawItem.knobs, should resizing the shape solely update the local
*  transform (for example, the DrawItem's DrawItem.scale or
*  DrawItem.translate)?.
*  
*  The default is false, which means that the DrawItem is allowed to modify
*  its shape properties in order to fit within a given width and height. Some examples:
*  
*  
* - A DrawOval might decrease its DrawOval.radius
*  when resized to a smaller size.
*  
* - A DrawPath might change its DrawPath.points
*  to lengthen all line segments in the path by some proportion so that it fits into a
*  larger size.
*  
*  This approach allows a DrawItem to maintain the same
*  DrawItem.lineWidth even as it is being resized.
*  
*  If this property is set to true then all visual aspects of the DrawItem,
*  including the line width and the fill, will be magnified or reduced during resizes
*  as if the DrawItem were placed under a lens.
* @type {boolean}
* @default false
*/
resizeViaLocalTransformOnly?: boolean;

/**
* This property specifies the conditions for when proportional resizing is used.
*  
*  By default the DrawItem is forced to only resize proportionally while any modifier key
*  specified in DrawItem.proportionalResizeModifiers is pressed. For example, the
*  DrawItem will change its width and height by the same percentage as long as the "Shift"
*  key is held down.
*  
*  Note that this property only has an effect if the DrawItem is showing
*  "resize"
*  
*  DrawItem.knobs.
* @type {ProportionalResizeMode}
* @default "modifier"
*/
proportionalResizing?: ProportionalResizeMode;

/**
* If DrawItem.proportionalResizing is set to "modifier"
*  
*  or "modifierOff"
*  
*  then proportional resizing of the DrawItem is activated or deactivated, respectively,
*  whenever at least one key in this set of modifier keys is pressed.
*  
*  The keys allowed in this set are: "Alt", "Ctrl", and "Shift". If this set
*  of keys is empty then proportional resizing is always used if
*  proportionalResizing is "modifier"
*  and is never used if
*  proportionalResizing is "modifierOff"
*  .
* @type {Array<Partial<KeyName>>}
* @default "["Shift"]"
*/
proportionalResizeModifiers?: Array<Partial<KeyName>>;

/**
* An opaque object specifying the local transformation that should be applied to this
*  DrawItem, obtained through a call to DrawItem.getShapeData.
*  Note: if this property is specified, you should avoid also specifying a
*  DrawItem.translate, DrawItem.scale, DrawItem.xShearFactor, DrawItem.yShearFactor, or DrawItem.rotation.
* @type {object}
* @default null
*/
shapeData?: object;

/**
* Number of pixels the cursor needs to move before the EventHandler starts a drag operation.
* @type {number}
* @default "5"
*/
dragStartDistance?: number;

/**
* Is this DrawItem draggable? If true, then the DrawItem can be drag-repositioned by the user.
* @type {boolean}
* @default false
*/
canDrag?: boolean;

/**
* Context menu to show for this object, an instance of the Menu widget.
*  
*  Note: if Canvas.destroy is called on a canvas, any specified context menu is
*  not automatically destroyed as well. This is in contrast to MenuButtons which
*  automatically destroy their specified MenuButton.menu by default. The behavior
*  is intentional as context menus are commonly reused across components.
* @type {Menu}
* @default null
*/
contextMenu?: Menu;

/**
* Should events inside this DrawItem be attributed to it regardless of which pixels are
*  actually set, if no fill is specified? If set for DrawItems that aren't closed, will
*  capture events occurring in the region that would filled if a fill were specified.
*  This property is true by default for closed shapes, and false for paths, lines, etc.
*  
*  Note: this property must be true if you're writing to an HTML5 &lt;canvas&gt; element
*  directly in your code (only applies to DrawingType "bitmap"
*  ).
* @type {boolean}
* @default "varies"
*/
eventOpaque?: boolean;

/**
* If this item is showing a
*  "move"
*  
*  DrawItem.knobs, this attribute specifies where the knob should appear
*  with respect to the draw item.
*  
*  The resize and move knobs show at the same position by default. However, when
*  both knobs are shown the move knob is offset slightly to allow access to both.
*  This position can be adjusted manually with DrawItem.moveKnobOffset.
* @type {MoveKnobPoint}
* @default "TL"
*/
moveKnobPoint?: MoveKnobPoint;

/**
* If this item is showing a "move" DrawItem.knobs, this attribute
*  allows you to specify an offset in pixels from the DrawItem.moveKnobPoint for the
*  move knob. Offset should be specified as a 2-element array of [left offset, top offset].
*  
*  This offset overrides the built-in offset used when showing both resize and move knobs.
* @type {Array<Partial<Integer>>}
* @default null
*/
moveKnobOffset?: Array<Partial<Integer>>;

/**
* If this item is showing "move"
*  
*  DrawItem.knobs, this attribute specifies the AutoChild for the
*  DrawKnob that allows a user to move the DrawItem with help of a knob located at
*  DrawItem.moveKnobPoint. Default move knob shape is green circle.
* @type {DrawKnob}
* @default null
*/
moveKnob?: DrawKnob;

/**
* If this item is showing "rotate"
*  
*  DrawItem.knobs, this attribute specifies the AutoChild for the
*  DrawKnob that allows a user to rotate the DrawItem with help of a knob located
*  above. Default rotate knob shape is green circle.
* @type {DrawKnob}
* @default null
*/
rotateKnob?: DrawKnob;

/**
* If this item is showing "resize"
*  
*  DrawItem.knobs, this attribute
*  specifies the points with respect to the draw item where resize knobs should appear.
* @type {Array<Partial<ResizeKnobPoint>>}
* @default "["TL","TR","BL","BR","T","R","B","L"]"
*/
resizeKnobPoints?: Array<Partial<ResizeKnobPoint>>;

/**
* If this item is showing "resize"
*  
*  DrawItem.knobs, this attribute specifies the MultiAutoChild for the
*  DrawKnob that allows a user to resize the DrawItem with help of knobs located at
*  corners of a bounding rectangle of current DrawItem. The default shape is a light teal
*  circle.
* @type {DrawKnob}
* @default null
*/
cornerResizeKnob?: DrawKnob;

/**
* If this item is showing "resize"
*  
*  DrawItem.knobs, this attribute specifies the MultiAutoChild for the
*  DrawKnob that allows a user to resize the DrawItem with help of knobs located at
*  centers of edges of a bounding rectangle of current DrawItem. The default shape is a light
*  teal square.
* @type {DrawKnob}
* @default null
*/
sideResizeKnob?: DrawKnob;

/**
* If this item is showing "resize"
*  
*  DrawItem.knobs, this attribute specifies the AutoChild for the
*  DrawRect that draws a rectangle frame which connects all resize knobs of current
*  DrawItem.
* @type {DrawRect}
* @default null
*/
resizeOutline?: DrawRect;

/**
* If this item is showing "resize"
*  
*  DrawItem.knobs will the resize outline be shown or not.
* @type {boolean}
* @default true
*/
showResizeOutline?: boolean;

/**
* If true, when a DrawItem is DrawItem.moveTo or DrawItem.resizeTo, the transform
*  is applied by manipulating the shape coordinates, if possible, rather than by introducing
*  scaling, shearing, rotation, or translation. This is only supported currently for 
*  DrawRect, DrawOval, DrawDiamond, DrawImage, and DrawLabel,
*  and only if no shearing is already present. Further, it's only possible to keep the
*  transform simple if both axes are scaled by the same amount during the resize (or end up at
*  the same scale if the DrawItem is already scaled unevenly), unless the rotation angle is a
*  multiple of 90 degrees.
*  
*  For DrawPolygon and other shapes not based on a box (top/left/width/height), we can't
*  safely just modify coordinates to effect a resize as we can do for DrawRect (and 
*  similar), so resizing will normally introduce or modify the transform, potentially 
*  introducing scaling or shearing, rather than modifying coordinates. For such 
*  DrawItems, we avoid trying to manipulate the coordinates, in part, because there's a
*  danger that the floating point error may accumulate over time and warp the shape.
* @type {boolean}
* @default true
*/
useSimpleTransform?: boolean;

/**
* If this item is showing "startPoint"
*  
*  DrawItem.knobs, this attribute specifies the AutoChild for the
*  DrawKnob for start point of current drawItem.
* @type {DrawKnob}
* @default null
*/
startKnob?: DrawKnob;

/**
* If this item is showing "endPoint"
*  
*  DrawItem.knobs, this attribute specifies the AutoChild for the
*  DrawKnob for end point of current drawItem.
* @type {DrawKnob}
* @default null
*/
endKnob?: DrawKnob;

/**
* Flag indicating a drawItem has been destroyed, similar to Canvas.destroyed.
* @type {boolean}
* @default null
*/
destroyed?: boolean;

/**
* Flag indicating a drawItem is mid-destruction, similar to Canvas.destroying.
* @type {boolean}
* @default null
*/
destroying?: boolean;



/**
*  Sets a property on this DrawItem, calling the appropriate setter method if one is found and
*  is Class.isMethodSupported.
* 
* @param {string} name of the property to set
* @param {any} new value for the property
*/
setPropertyValue(propertyName:string, newValue:any): void;

/**
*  Returns the DrawItem.zIndex of this draw item.
* 
* @param {boolean=} if true and this item's zIndex is null, resolve to the next higher zIndex.
*/
getZIndex(resolveToNumber?:boolean): Integer;

/**
*  Setter for DrawItem.zIndex.
* 
* @param {Integer} new zIndex. If null and this draw item is already in a DrawPane or DrawGroup, then this item's zIndex will be set to the next higher auto-assigned zIndex.  Note that when setting draw items' zIndexes via this advanced API, the application should take over management of all draw items' zIndexes, and DrawItem.bringToFront / DrawItem.sendToBack should not be used, as those APIs assume automatic management of zIndexes.
*/
setZIndex(newZIndex:Integer): void;

/**
*  Places this draw item at the top of the stacking order so that it appears in front of other
*  draw items in the same DrawPane or DrawGroup.
*  
*  When the DrawPane's DrawPane.drawingType is
*  "bitmap",
*  
*  DrawItem.zIndex, bringToFront(), and DrawItem.sendToBack are not supported
*  for DrawLabels on iOS due to platform limitations.
* 
*/
bringToFront(): void;

/**
*  Places this draw item at the bottom of the stacking order so that it appears behind other
*  draw items in the same DrawPane or DrawGroup.
*  
*  When the DrawPane's DrawPane.drawingType is
*  "bitmap",
*  
*  DrawItem.zIndex, DrawItem.bringToFront, and sendToBack() are not supported
*  for DrawLabels on iOS due to platform limitations.
* 
*/
sendToBack(): void;

/**
* 
* 
* @param {Cursor} new cursor.
*/
setCursor(newCursor:Cursor): void;

/**
*  Returns the center point of this draw item in local coordinates. Generally this is the
*  center of the DrawItem.getBoundingBox, but some item types may use
*  a different point. For example, DrawTriangle uses the
*  incenter
*  of the triangle.
* 
*/
getCenter(): Point;

/**
*  Calculates the bounding box of the shape in the DrawPane.
*  
*  Note that the bounding box of the shape when transformed into the global coordinate system
*  is available from the method DrawItem.getResizeBoundingBox.
* 
* 
*/
getBoundingBox(): Array<Partial<Double>>;

/**
*  Calculates the bounding box of the shape reflected by the
*  DrawItem.showResizeOutline shown when dragging the
*  DrawItem.resizeKnobPoints. This method is similar to DrawItem.getBoundingBox
*  except that the coordinates returned by this method are in the global coordinate system
*  (described DrawPane) rather than the local coordinate system.
* 
*/
getResizeBoundingBox(): Array<any>;

/**
*  Returns true if the given point in the drawing coordinate system, when converted to
*  coordinates in this DrawItem's local coordinate system, is within the
*  DrawItem.getBoundingBox of this DrawItem's shape.
*  
*  This method can be used to quickly check whether the given point is definitely not within
*  the DrawItem shape. To check whether the point is within the DrawItem shape, use the slower
*  but exact DrawItem.isPointInPath method.
* 
* @param {Integer} X coordinate of the point in the drawing coordinate system.
* @param {Integer} Y coordinate of the point in the drawing coordinate system.
*/
isInBounds(x:Integer, y:Integer): boolean;

/**
*  Returns an opaque JavaScript object representing the current local transformation applied to
*  the DrawItem's local coordinates, as defined by DrawItem.translate, DrawItem.scale, 
*  DrawItem.xShearFactor, DrawItem.yShearFactor, and DrawItem.rotation. The object may be serialized
*  and deserialized as JSON, and passed into the constructor block as DrawItem.shapeData to
*  restore the local transformation.
*  Note: this doesn't include any sepatate configuration, such as for a DrawRect
*  the current values of DrawRect.left, DrawRect.top,
*  DrawRect.width, or DrawRect.height.
* 
* 
*/
getShapeData(): object;

/**
*  Returns true if the given point in the drawing coordinate system is within this DrawItem's shape,
*  taking into account local transforms.
* 
* @param {Integer} X coordinate of the test point.
* @param {Integer} Y coordinate of the test point.
*/
isPointInPath(x:Integer, y:Integer): boolean;

/**
*  Computes the angle in degrees from the positive X axis to the difference vector
*  v2 - v1 between the two given vectors.
* 
* @param {Double} X coordinate of v1
* @param {Double} Y coordinate of v1
* @param {Double} X coordinate of v2
* @param {Double} Y coordinate of v2
*/
computeAngle(px1:Double, py1:Double, px2:Double, py2:Double): Double;

/**
*  Returns the page-relative left coordinate of the widget on the page, in pixels.
* 
*/
getPageLeft(): number;

/**
*  Returns the page-relative top coordinate of the widget on the page, in pixels
* 
*/
getPageTop(): number;

/**
*  Setter for DrawItem.canDrag.
* 
* 
* @param {boolean} new value for this.canDrag.
*/
setCanDrag(canDrag:boolean): void;

/**
*  Notification fired when the user starts to drag this DrawItem. Will only fire if DrawItem.canDrag
*  is true.
*  
*  Note that if this item is part of a DrawGroup, then the group's DrawGroup.useGroupRect
*  setting affects whether this item receives the notification. If useGroupRect is true, then
*  this item will not receive the notification. Otherwise, the item receives the
*  notification and notification bubbles up to the group.
* 
*/
dragStart(): boolean;

/**
*  Notification fired for every mouseMove event triggered while the user is dragging this
*  DrawItem. Will only fire if DrawItem.canDrag is true.
*  
*  Note that if this item is part of a DrawGroup, then the group's DrawGroup.useGroupRect
*  setting affects whether this item receives the notification. If useGroupRect is true, then
*  this item will not receive the notification. Otherwise, the item receives the
*  notification and notification bubbles up to the group.
* 
*/
dragMove(): boolean;

/**
*  Notification fired when the user stops dragging this DrawItem. Will only fire if DrawItem.canDrag
*  is true.
*  
*  Note that if this item is part of a DrawGroup, then the group's DrawGroup.useGroupRect
*  setting affects whether this item receives the notification. If useGroupRect is true, then
*  this item will not receive the notification. Otherwise, the item receives the
*  notification and notification bubbles up to the group.
* 
*/
dragStop(): boolean;

/**
*  Notification fired when the user presses the left mouse button on this DrawItem.
*  
*  Note that if this item is part of a DrawGroup, then the group's DrawGroup.useGroupRect
*  setting affects whether this item receives the notification. If useGroupRect is true, then
*  this item will not receive the notification. Otherwise, the item receives the
*  notification and notification bubbles up to the group.
* 
*/
mouseDown(): boolean;

/**
*  Notification fired when the user releases the left mouse button on this DrawItem.
*  
*  Note that if this item is part of a DrawGroup, then the group's DrawGroup.useGroupRect
*  setting affects whether this item receives the notification. If useGroupRect is true, then
*  this item will not receive the notification. Otherwise, the item receives the
*  notification and notification bubbles up to the group.
* 
*/
mouseUp(): boolean;

/**
*  Notification fired when the user clicks on this DrawItem.
*  
*  Note that if this item is part of a DrawGroup, then the group's DrawGroup.useGroupRect
*  setting affects whether this item receives the notification. If useGroupRect is true, then
*  this item will not receive the notification. Otherwise, the item receives the
*  notification and notification bubbles up to the group.
* 
*/
click(): boolean;

/**
*  Notification fired when the mouse enters this DrawItem.
*  
*  Note that if this item is part of a DrawGroup, then the group's DrawGroup.useGroupRect
*  setting affects whether this item receives the notification. If useGroupRect is true, then
*  this item will not receive the notification. Otherwise, the item receives the
*  notification and notification bubbles up to the group.
* 
*/
mouseOver(): boolean;

/**
*  Notification fired when the user moves the mouse over this DrawItem.
*  
*  Note that if this item is part of a DrawGroup, then the group's DrawGroup.useGroupRect
*  setting affects whether this item receives the notification. If useGroupRect is true, then
*  this item will not receive the notification. Otherwise, the item receives the
*  notification and notification bubbles up to the group.
* 
*/
mouseMove(): boolean;

/**
*  Notification fired when the mouse leaves this DrawItem.
*  
*  Note that if this item is part of a DrawGroup, then the group's DrawGroup.useGroupRect
*  setting affects whether this item receives the notification. If useGroupRect is true, then
*  this item will not receive the notification. Otherwise, the item receives the
*  notification and notification bubbles up to the group.
* 
*/
mouseOut(): boolean;

/**
*  Executed when the right mouse button is clicked. The default implementation of
*  this method auto-creates a Menu from the Canvas.contextMenu property on
*  this component and then calls Menu.showContextMenu on it to show it.
*  
*  If you want to show a standard context menu, you can simply define your Menu and set it as the
*  contextMenu property on your component - you do not need to override this method.
*  
*  If you want to do some other processing before showing a menu or do something else entirely, then
*  you should override this method. Note that the return value from this method controls whether or
*  not the native browser context menu is shown.
* 
* 
*/
showContextMenu(): boolean;

/**
*  If DrawItem.canHover is true for this DrawItem, the hover() string method will
*  be fired when the user hovers over this DrawItem. If this method returns false, it will
*  suppress the default behavior of showing a hover canvas if DrawItem.showHover
*  is true.
* 
*/
hover(): boolean;

/**
*  If DrawItem.showHover is true, when the user holds the mouse over this DrawItem for
*  long enough to trigger a hover event, a hover canvas is shown by default. This method returns
*  the contents of that hover canvas. Default implementation returns DrawItem.prompt -
*  override for custom hover HTML. Note that returning null or an empty string will
*  suppress the hover canvas altogether.
* 
*/
getHoverHTML(): HTMLString;

/**
*  Draws this item into its current DrawItem.drawPane.
*  
*  NOTE: For performance reasons, the DrawPane may draw this item on a
*  delay to allow multiple items to be added and drawn at one time. The DrawPane.refreshNow
*  API will force this item to be drawn immediately.
* 
*/
draw(): void;

/**
*  Called when we start drawing for this DrawItem to the DrawItem.drawPane's underlying
*  HTML5 &lt;canvas&gt; element. Only called if the DrawingType is
*  "bitmap".
*  
*  There is no default implementation of this method.
* 
*/
drawStart(): void;

/**
*  Called when we finish drawing for this DrawItem to the DrawItem.drawPane's underlying
*  HTML5 &lt;canvas&gt; element. Only called if the DrawingType is
*  "bitmap".
*  
*  There is no default implementation of this method.
* 
*/
drawEnd(): void;

/**
*  Notification method fired when this component is explicitly moved.
*  Note that a component's position on the screen may also be changed due to an ancestor being
*  moved. The Canvas.parentMoved method provides a notification entry point to catch
*  that case as well.
* 
* 
* @param {Integer} horizontal difference between current and previous position
* @param {Integer} vertical difference between current and previous position
*/
moved(deltaX:Integer, deltaY:Integer): void;

/**
*  Observable method called whenever 
*  a DrawItem changes size.
* 
*/
resized(): void;

/**
*  Shows a set of control knobs for this drawItem. Updates DrawItem.knobs to include the
*  specified knobType, and if necessary draws out the appropriate control knobs.
* 
* @param {KnobType | Array<Partial<KnobType>>} knobs to show
*/
showKnobs(knobType:KnobType | Array<Partial<KnobType>>): void;

/**
*  Shows all supported control knobs for this drawItem. Updates DrawItem.knobs to include the
*  supported knobTypes and if necessary draws out the appropriate control knobs.
* 
*/
showAllKnobs(): void;

/**
*  Hides a set of control knobs for this drawItem. Updates DrawItem.knobs to remove the
*  specified knobType, and clears any drawn knobs for this knobType. 
* 
* @param {KnobType | Array<Partial<KnobType>>} knobs to hide
*/
hideKnobs(knobType:KnobType | Array<Partial<KnobType>>): void;

/**
*  Hides all control knobs for this drawItem. Updates DrawItem.knobs to remove
*  all knobTypes and clears any drawn knobs. 
* 
*/
hideAllKnobs(): void;

/**
*  Setter for DrawItem.moveKnobOffset.
* 
* @param {Array<Partial<Integer>>=} the new move knob offset. This is a 2-element array of [left offset, top offset]. If null, then [0,0]  is assumed.
*/
setMoveKnobOffset(newMoveKnobOffset?:Array<Partial<Integer>>): void;

/**
*  Setter for the DrawItem.title of this DrawItem.
* 
* @param {string=} new title.
*/
setTitle(newTitle?:string): void;

/**
*  If DrawItem.canDrag is true and the DrawItem.knobs include "resize" knobs,
*  then this notification method will be fired when the user drag-resizes the draw item.
* 
* @param {string} provides which knob of the DrawItem.resizeKnobPoints was dragged
* @param {Integer} new x-coordinate of the knob
* @param {Integer} new y-coordinate of the knob
* @param {Integer} horizontal distance moved
* @param {Integer} vertical distance moved
*/
dragResizeMove(position:string, x:Integer, y:Integer, dX:Integer, dY:Integer): void;

/**
*  Setter for DrawItem.drawPane.
* 
* @param {DrawPane} new value for this.drawPane.
*/
setDrawPane(drawPane:DrawPane): void;

/**
*  Erase this drawItem's visual representation and remove it from its DrawGroup (if any) and
*  DrawPane.
*  
*  To re-draw the item within the DrawPane, call DrawItem.draw again, or use
*  DrawPane.addDrawItem to move to another DrawGroup.
*  
* 
*/
erase(): void;

/**
*  Permanently destroys this DrawItem, similar to Canvas.destroy.
* 
* 
*/
destroy(): void;

/**
*  Generates a string containing the SVG source of this DrawItem.
* 
*  NOTE: The generated SVG source assumes that the default namespace is http://www.w3.org/2000/svg
*  and that namespace prefix xlink refers to namespace name http://www.w3.org/1999/xlink.
* 
* 
*/
getSvgString(): void;

/**
*  Make this drawItem visible.
* 
*/
show(): void;

/**
*  Hide this drawItem.
* 
*/
hide(): void;

/**
*  Update lineWidth for this drawItem.
* 
* @param {Integer} new pixel lineWidth
*/
setLineWidth(width:Integer): void;

/**
*  Update lineColor for this drawItem.
* 
* @param {CSSColor} new line color. Pass null for transparent.
*/
setLineColor(color:CSSColor): void;

/**
*  Update lineOpacity for this drawItem.
* 
* @param {Float} new opacity, as a number between 0 (transparent) and 1 (opaque).
*/
setLineOpacity(opacity:Float): void;

/**
*  Update linePattern for this drawItem.
* 
* @param {LinePattern} new linePattern to use
*/
setLinePattern(pattern:LinePattern): void;

/**
*  Update lineCap for this drawItem.
* 
* @param {LineCap} new lineCap to use
*/
setLineCap(cap:LineCap): void;

/**
*  Update shadow for this drawItem.
* 
* @param {Shadow} new shadow
*/
setShadow(shadow:Shadow): void;

/**
*  Does this DrawItem Class.isMethodSupported DrawItem.setStartArrow?
*  For example, this is false for DrawRect and DrawOval, and true for DrawLine.
* 
*/
supportsStartArrow(): boolean;

/**
*  Set the arrowhead at the beginning of this path.
*  
*  NOTE: Not all DrawItem classes support arrowheads. You can use DrawItem.supportsStartArrow
*  to dynamically check whether a DrawItem instance supports this method.
* 
* @param {ArrowStyle} style of arrow to use
*/
setStartArrow(arrowStyle:ArrowStyle): void;

/**
*  Does this DrawItem Class.isMethodSupported DrawItem.setEndArrow?
*  For example, this is false for DrawRect and DrawOval, and true for DrawLine.
* 
*/
supportsEndArrow(): boolean;

/**
*  Set the arrowhead at the end of this path.
*  
*  NOTE: Not all DrawItem classes support arrowheads. You can use DrawItem.supportsEndArrow
*  to dynamically check whether a DrawItem instance supports this method.
* 
* @param {ArrowStyle} style of arrow to use
*/
setEndArrow(arrowStyle:ArrowStyle): void;

/**
*  Move the shape by the specified deltas for the left and top coordinate.
* 
* 
* @param {Integer} change to left coordinate in pixels
* @param {Integer} change to top coordinate in pixels
*/
moveBy(dX:Integer, dY:Integer): void;

/**
*  Move the DrawItem to the specified coordinates in the global coordinate system. The
*  specified coordinates will become the top-left point of the
*  DrawItem.getResizeBoundingBox.
* 
* @param {Integer} new left coordinate in pixels
* @param {Integer} new top coordinate in pixels
*/
moveTo(left:Integer, top:Integer): void;

/**
*  Change the center point for this DrawItem.
* 
* @param {Integer} X coordinate of the center point (in the global coordinate system).
* @param {Integer} Y coordinate of the center point (in the global coordinate system).
*/
setCenterPoint(left:Integer, top:Integer): void;

/**
*  Resize the shape by the specified deltas.
* 
* @param {Integer} number of pixels to resize by horizontally
* @param {Integer} number of pixels to resize by vertically
*/
resizeBy(dX:Integer, dY:Integer): void;

/**
*  Resize to the specified size
* 
* @param {Integer} new width
* @param {Integer} new height
*/
resizeTo(width:Integer, height:Integer): void;

/**
*  Rotate the shape by the relative rotation in degrees
* 
* @param {Float} number of degrees to rotate from current orientation.
*/
rotateBy(degrees:Float): void;

/**
*  Rotate the shape by the absolute rotation in degrees
* 
* @param {Float} number of degrees to rotate
*/
rotateTo(degrees:Float): void;

/**
*  Scale the shape by the x, y multipliers
* 
* @param {Float} scale in the x direction
* @param {Float} scale in the y direction
*/
scaleBy(x:Float, y:Float): void;

/**
*  Scale the shape by the x, y multipliers
* 
* @param {Float} scale in the x direction
* @param {Float} scale in the y direction
*/
scaleTo(x:Float, y:Float): void;

/**
*  Update fillColor for this drawItem.
* 
* @param {CSSColor} new fillColor to use. Pass null for transparent.
*/
setFillColor(color:CSSColor): void;

/**
*  Update fillGradient for this drawItem.
* 
* @param {Gradient} new gradient to use. Pass null for transparent.
*/
setFillGradient(gradient:Gradient): void;

/**
*  Update fillOpacity for this drawItem.
* 
* @param {Float} new opacity, as a number between 0 (transparent) and 1 (opaque).
*/
setFillOpacity(opacity:Float): void;



}
/**
* Creates a new DrawItem
* 
* @param typeCheckedProperties {Partial<DrawItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawItem} The new instance
*/
class DrawItem{
static create(typeCheckedProperties? : Partial<DrawItem>, uncheckedProperties?: Object): DrawItem;




/**
*  Computes the angle in degrees from the positive X axis to the difference vector
*  v2 - v1 between the two given vectors.
* 
* @param {Double} X coordinate of v1
* @param {Double} Y coordinate of v1
* @param {Double} X coordinate of v2
* @param {Double} Y coordinate of v2
*/
static computeAngle(px1:Double, py1:Double, px2:Double, py2:Double): Double;




}

/**
* EditProxy that handles DrawItem except for
*  DrawLabel when editMode is enabled.
*/
interface DrawItemEditProxy extends EditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's title.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's title.
* 
* 
* @param {string} the new component title
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new DrawItemEditProxy
* 
* @param typeCheckedProperties {Partial<DrawItemEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawItemEditProxy} The new instance
*/
class DrawItemEditProxy{
static create(typeCheckedProperties? : Partial<DrawItemEditProxy>, uncheckedProperties?: Object): DrawItemEditProxy;







}

/**
* Canvas that renders a DrawItem into a DrawPane and provides interactivity for
*  that drawItem, including drag and drop.
*  
*  A DrawKnob can either be initialized with a DrawKnob.knobShape or created via
*  the AutoChild pattern.
*  
*  DrawKnobs are used by the DrawItem.knobs subsystem.
*/
interface DrawKnob extends Canvas {
/**
* The DrawItem instance rendered into this DrawKnob's drawPane
* @type {DrawItem}
* @default null
*/
knobShape?: DrawItem;

/**
* Default properties for this component's DrawKnob.knobShape. Has the
*  following properties by default:
*  
*  radius : 5,
*  lineWidth:2,
*  fillColor:"#FF0000",
*  fillOpacity:0.5,
*  
*  As with any auto-child defaults block, use Class.changeDefaults
*  to modify this object.
* @type {Partial<DrawItem>}
* @default "{...}"
*/
knobShapeDefaults?: Partial<DrawItem>;

/**
* DrawPane into which this DrawKnob's DrawKnob.knobShape will be rendered.
* @type {DrawPane}
* @default null
*/
drawPane?: DrawPane;

/**
* X-Coordinate for this DrawKnob. DrawKnob will initially be drawn centered over this
*  coordinate
* @type {Integer}
* @default null
*/
x?: Integer;

/**
* Y-Coordinate for this DrawKnob. DrawKnob will initially be drawn centered over this
*  coordinate
* @type {Integer}
* @default null
*/
y?: Integer;



/**
*  Sets the center point of the drawKnob. If the optional viewboxCoords
*  argument is passed, coordinates are expected to be adjusted for drawPane pan
*  and zoom. Otherwise coordinates are expected to be absolute pixel coordinates within
*  the drawPane.
* 
* @param {Integer} new x coordinate for this drawKnob
* @param {Integer} new y coordinate for this drawKnob
* @param {boolean=} If true, the x and y values are expected to be in the viewbox coordinate system (described DrawPane) - already adjusted for any zoom or pan applied to the drawPane.
*/
setCenterPoint(x:Integer, y:Integer, viewboxCoords?:boolean): void;

/**
*  Method called in response to the user dragging this DrawKnob. 
*  May be observed or overridden
*  to allow drawItems to react to user drag interactions on this knob.
*  
*  Note that the default implementation does nothing. When working with draw knobs directly this
*  is typically where you would both update the shape being controlled by the draw knob, and
*  ensure the drawKnob gets repositioned. You may also need to update the drawKnob
*  position in response to the drawItem being repositioned, resized, etc.
*  
* 
* @param {Integer} new x-coordinate of the drawKnob
* @param {Integer} new y-coordinate of the drawKnob
* @param {Integer} horizontal distance moved
* @param {Integer} vertical distance moved
* @param {string} either "start", "move", or "stop", to indicate the current phase of dragging of the DrawKnob for which the points need to be updated
*/
updatePoints(x:Integer, y:Integer, dX:Integer, dY:Integer, state:string): void;



}
/**
* Creates a new DrawKnob
* 
* @param typeCheckedProperties {Partial<DrawKnob>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawKnob} The new instance
*/
class DrawKnob{
static create(typeCheckedProperties? : Partial<DrawKnob>, uncheckedProperties?: Object): DrawKnob;







}

/**
* DrawItem subclass to render a single-line text label.
*/
interface DrawLabel extends DrawItem {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "DrawLabelEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* DrawLabel only supports the
*  "move"
*  
*  knob type.
* @type {Array<Partial<KnobType>>}
* @default null
*/
knobs?: Array<Partial<KnobType>>;

/**
* This is the content that will exist as the label.
* @type {string}
* @default null
*/
contents?: string;

/**
* For DrawPane.drawingType DrawPanes, whether to escape the specified
*  DrawLabel.contents of this label so that any markup syntax is rendered "as is," without being
*  interpreted as SVG. This setting should not be customized when working with other 
*  DrawPane.drawingTypes, as the DrawLabel.contents are always escaped in such
*  case.
*  
*  In SVG, a DrawLabel's DrawLabel.contents are rendered inside a &lt;text&gt;
*  tag, so any SVG that's legal inside that tag can be set as the DrawLabel.contents when 
*  DrawLabel.escapeContents is false. See 
*  Mozilla SVG Developer Reference
*  for more information about what exactly is supported.
*  
*  Note that the Framework will not be able to determine the width or height of a 
*  DrawLabel when this property is false, so the Framework will consider both dimensions
*  to be zero, and centering will not work (e.g. for DrawItem.titleLabel autochildren).
*  For top-level DrawLabels, you may be able to get the DOM to center your content by
*  setting DrawLabel.alignment as "center" - the DOM will then interpret the
*  DrawLabel.left, DrawLabel.top coordinates of the label as its centerpoint even though our Framework
*  doesn't know the label's actual size.
*  
*  For a DrawItem.titleLabel, the DrawLabel.alignment setting
*  is ignored, as the Framework always positions it using "start" alignment, but SVG code such
*  as the following demonstrates that centering is possible (via the "style" setting):
*  &lt;tspan text-decoration='underline' font-size='20px' 
*     style='dominant-baseline:central; text-anchor:middle;'&gt;MyLabel
*  &lt;/tspan&gt;
*  
* @type {boolean}
* @default true
*/
escapeContents?: boolean;

/**
* Sets the amount from the left of its positioning that the element should be placed.
* @type {Coordinate}
* @default "0"
*/
left?: Coordinate;

/**
* Sets the amount from the top of its positioning that the element should be placed.
* @type {Coordinate}
* @default "0"
*/
top?: Coordinate;

/**
* Sets the text alignment from the x position. Similar to HTML5 context.textAlign
*  with alignment values such as "start", "center", and "end".
*  
*  Note that this setting is ignored for DrawItem.titleLabel autochildren, which are
*  always considered to have "start" alignment to make handling of
*  DrawItem.titleRotationMode simpler.
* @type {LabelAlignment}
* @default "start"
*/
alignment?: LabelAlignment;

/**
* Font family name, similar to the CSS font-family attribute.
* @type {string}
* @default "Tahoma"
*/
fontFamily?: string;

/**
* Font size in pixels, similar to the CSS font-size attribute.
* @type {Integer}
* @default "18"
*/
fontSize?: Integer;

/**
* Font weight, similar to the CSS font-weight attribute, eg "normal", "bold".
* @type {string}
* @default "bold"
*/
fontWeight?: string;

/**
* Font style, similar to the CSS font-style attribute, eg "normal", "italic".
* @type {string}
* @default "normal"
*/
fontStyle?: string;

/**
* For DrawPane.drawingType "svg" only, the CSS class applied to this label.
*  Similar to Canvas.styleName. The font properties DrawLabel.fontSize, DrawLabel.fontWeight,
*  DrawLabel.fontStyle, and DrawLabel.fontFamily, unless set to null, take priority over any CSS
*  settings. This property can be used in combination with DrawLabel.escapeContents if needed. but
*  note that in SVG, the precedence of CSS and inline styling applied to an element works
*  differently that it does in HTML. See
*  Mozilla SVG Developer Reference
*  
*  Note that only font sizes defined in pixels are supported through this property.
* @type {CSSStyleName}
* @default "normal"
*/
styleName?: CSSStyleName;

/**
* The text color of the label.
* @type {CSSColor}
* @default "#808080"
*/
lineColor?: CSSColor;

/**
* Rotation in degrees about the DrawLabel.top DrawLabel.left corner. The positive
*  direction is clockwise.
* @type {Float}
* @default "0.0"
*/
rotation?: Float;



/**
*  Sets the DrawLabel.escapeContents property for this DrawLabel.
* 
* @param {boolean} whether to escape DrawLabel.contents
*/
setEscapeContents(escapeContents:boolean): void;

/**
*  Sets this DrawLabel's DrawLabel.styleName.
* 
* @param {CSSStyleName} the new styleName
*/
setStyleName(styleName:CSSStyleName): void;

/**
*  Get the center point of the label.
* 
*/
getCenter(): Point;

/**
*  Returns the left, top, left + textWidth, top + textHeight
* 
*/
getBoundingBox(): Array<Partial<Double>>;

/**
*  Sets the text color of the label.
* 
* @param {CSSColor} new text color.
*/
setLineColor(color:CSSColor): void;

/**
*  Sets this DrawLabel's DrawLabel.contents.
* 
* @param {string} the new contents.
*/
setContents(contents:string): void;

/**
*  Sets this DrawLabel's DrawLabel.fontSize.
* 
* @param {Integer} the new font size in pixels.
*/
setFontSize(size:Integer): void;



}
/**
* Creates a new DrawLabel
* 
* @param typeCheckedProperties {Partial<DrawLabel>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawLabel} The new instance
*/
class DrawLabel{
static create(typeCheckedProperties? : Partial<DrawLabel>, uncheckedProperties?: Object): DrawLabel;
/**
* This setting exists due to platform limitations in some versions of Internet Explorer
*  where the browser does not recognize the five generic font families
*  (cursive, fantasy, monospace,
*  sans-serif, and serif) and instead uses a non-configurable,
*  default font. This setting specifies a specific font to replace the
*  cursive keyword should a DrawLabel list it in its
*  DrawLabel.fontFamily.
* @type {string}
* @default "Zapf-Chancery"
*/
static defaultCursiveFont?: string;

/**
* This setting exists due to platform limitations in some versions of Internet Explorer
*  where the browser does not recognize the five generic font families
*  (cursive, fantasy, monospace,
*  sans-serif, and serif) and instead uses a non-configurable,
*  default font. This setting specifies a specific font to replace the
*  fantasy keyword should a DrawLabel list it in its
*  DrawLabel.fontFamily.
* @type {string}
* @default "Western"
*/
static defaultFantasyFont?: string;

/**
* This setting exists due to platform limitations in some versions of Internet Explorer
*  where the browser does not recognize the five generic font families
*  (cursive, fantasy, monospace,
*  sans-serif, and serif) and instead uses a non-configurable,
*  default font. This setting specifies a specific font to replace the
*  monospace keyword should a DrawLabel list it in its
*  DrawLabel.fontFamily.
* @type {string}
* @default "Courier"
*/
static defaultMonospaceFont?: string;

/**
* This setting exists due to platform limitations in some versions of Internet Explorer
*  where the browser does not recognize the five generic font families
*  (cursive, fantasy, monospace,
*  sans-serif, and serif) and instead uses a non-configurable,
*  default font. This setting specifies a specific font to replace the
*  sans-serif keyword should a DrawLabel list it in its
*  DrawLabel.fontFamily.
* @type {string}
* @default "Helvetica"
*/
static defaultSansSerifFont?: string;

/**
* This setting exists due to platform limitations in some versions of Internet Explorer
*  where the browser does not recognize the five generic font families
*  (cursive, fantasy, monospace,
*  sans-serif, and serif) and instead uses a non-configurable,
*  default font. This setting specifies a specific font to replace the
*  serif keyword should a DrawLabel list it in its
*  DrawLabel.fontFamily.
* @type {string}
* @default "Times"
*/
static defaultSerifFont?: string;

/**
* A declared value of the enum type 
*  LabelAlignment.
* @type {Constant}
* @default "start"
*/
static START?: Constant;

/**
* A declared value of the enum type 
*  LabelAlignment.
* @type {Constant}
* @default "end"
*/
static END?: Constant;

/**
* A declared value of the enum type 
*  LabelAlignment.
* @type {Constant}
* @default "center"
*/
static CENTER?: Constant;

/**
* A declared value of the enum type 
*  LabelAlignment.
* @type {Constant}
* @default "left"
*/
static LEFT?: Constant;

/**
* A declared value of the enum type 
*  LabelAlignment.
* @type {Constant}
* @default "right"
*/
static RIGHT?: Constant;








}

/**
* EditProxy that handles DrawLabel when editMode is enabled.
*/
interface DrawLabelEditProxy extends DrawItemEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's contents.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's contents.
* 
* 
* @param {string} the new component contents
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new DrawLabelEditProxy
* 
* @param typeCheckedProperties {Partial<DrawLabelEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawLabelEditProxy} The new instance
*/
class DrawLabelEditProxy{
static create(typeCheckedProperties? : Partial<DrawLabelEditProxy>, uncheckedProperties?: Object): DrawLabelEditProxy;







}

/**
* DrawItem subclass to render line segments.
*/
interface DrawLine extends DrawItem {
/**
* Array of control knobs to display for this item. Each KnobType specified in this
*  will turn on UI element(s) allowing the user to manipulate this DrawLine. To update the
*  set of knobs at runtime use DrawItem.showKnobs and DrawItem.hideKnobs.
*  
*  DrawLine supports the
*  "startPoint", "endPoint", and "move"
*  
*  knob types.
* @type {Array<Partial<KnobType>>}
* @default null
*/
knobs?: Array<Partial<KnobType>>;

/**
* The mode in which the DrawItem.titleLabel (if shown) is rotated with
*  this draw item.
* @type {TitleRotationMode}
* @default "withLineAlwaysUp"
*/
titleRotationMode?: TitleRotationMode;

/**
* If the DrawItem.titleLabel is showing, should the DrawItem.titleLabelBackground
*  be created and placed behind the titleLabel?
*  
*  This defaults to true for DrawSectors and shapes that are not commonly filled
*  (e.g. DrawLines).
* @type {boolean}
* @default true
*/
showTitleLabelBackground?: boolean;

/**
* Start point of the line
* @type {Point}
* @default "[0,0]"
*/
startPoint?: Point;

/**
* End point of the line
* @type {Point}
* @default "[100,100]"
*/
endPoint?: Point;

/**
* Starting left coordinate of the line. Overrides left coordinate of DrawLine.startPoint if
*  both are set.
* @type {Coordinate}
* @default "0"
*/
startLeft?: Coordinate;

/**
* Starting top coordinate of the line. Overrides top coordinate of DrawLine.startPoint if
*  both are set.
* @type {Coordinate}
* @default "0"
*/
startTop?: Coordinate;

/**
* Ending left coordinate of the line. Overrides left coordinate of DrawLine.endPoint if
*  both are set.
* @type {Coordinate}
* @default "100"
*/
endLeft?: Coordinate;

/**
* Ending top coordinate of the line. Overrides top coordinate of DrawLine.endPoint if
*  both are set.
* @type {Coordinate}
* @default "100"
*/
endTop?: Coordinate;



/**
*  Update the startPoint
* 
* 
* @param {Coordinate} left coordinate for start point, in pixels
* @param {Coordinate} top coordinate for start point, in pixels
*/
setStartPoint(left:Coordinate, top:Coordinate): void;

/**
*  Update the endPoint
* 
* 
* @param {Coordinate} left coordinate for end point, in pixels
* @param {Coordinate} top coordinate for end point, in pixels
*/
setEndPoint(left:Coordinate, top:Coordinate): void;

/**
*  Returns a bounding box for the DrawLine, taking into account the
*  DrawItem.lineWidth.
* 
* 
*/
getBoundingBox(): Array<Partial<Double>>;

/**
*  Get the midpoint of the line.
* 
*/
getCenter(): Point;

/**
*  Returns true if the given point in the drawing coordinate system is within this DrawItem's shape,
*  taking into account local transforms.
* 
* @param {Integer} X coordinate of the test point.
* @param {Integer} Y coordinate of the test point.
*/
isPointInPath(x:Integer, y:Integer): boolean;

/**
*  Move both the start and end points of the line by a relative amount.
* 
* 
* @param {Distance} change to left coordinate in pixels
* @param {Distance} change to top coordinate in pixels
*/
moveBy(left:Distance, top:Distance): void;

/**
*  Move both the start and end points of the line such that the DrawLine.startPoint ends up at
*  the specified coordinate and the line length and angle are unchanged.
* 
* 
* @param {Integer} new left coordinate in pixels
* @param {Integer} new top coordinate in pixels
*/
moveStartPointTo(left:Integer, top:Integer): void;



}
/**
* Creates a new DrawLine
* 
* @param typeCheckedProperties {Partial<DrawLine>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawLine} The new instance
*/
class DrawLine{
static create(typeCheckedProperties? : Partial<DrawLine>, uncheckedProperties?: Object): DrawLine;







}

/**
* DrawItem subclass to render a connector between two points. If the points are aligned
*  on one axis, this connector will draw as a straight line. If the points are offset on
*  both axes and there is enough space, the connector will by default draw short horizontal 
*  segments from the start and end points, and a diagonal segment connecting the ends of these
*  'tail' segments. Connector style and orientation defaults may be changed through
*  configuration.
*/
interface DrawLinePath extends DrawItem {
/**
* The ConnectorStyle controlling the presentation and behavior of this line's 
*  tail segments.
* @type {ConnectorStyle}
* @default "diagonal"
*/
connectorStyle?: ConnectorStyle;

/**
* The ConnectorOrientation controlling the orientation and behavior of this line's 
*  tail segments.
* @type {ConnectorOrientation}
* @default "auto"
*/
connectorOrientation?: ConnectorOrientation;

/**
* Array of control knobs to display for this item. Each KnobType specified in this
*  will turn on UI element(s) allowing the user to manipulate this DrawLinePath. To update the
*  set of knobs at runtime use DrawItem.showKnobs and DrawItem.hideKnobs.
*  
*  DrawLinePath supports the
*  "startPoint", "endPoint", "controlPoint1", and "controlPoint2"
*  
*  knob types.
* @type {Array<Partial<KnobType>>}
* @default null
*/
knobs?: Array<Partial<KnobType>>;

/**
* The mode in which the DrawItem.titleLabel (if shown) is rotated with
*  this draw item.
* @type {TitleRotationMode}
* @default "withLineAlwaysUp"
*/
titleRotationMode?: TitleRotationMode;

/**
* If the DrawItem.titleLabel is showing, should the DrawItem.titleLabelBackground
*  be created and placed behind the titleLabel?
*  
*  This defaults to true for DrawSectors and shapes that are not commonly filled
*  (e.g. DrawLines).
* @type {boolean}
* @default true
*/
showTitleLabelBackground?: boolean;

/**
* Start point of the line
* @type {Point}
* @default "[0,0]"
*/
startPoint?: Point;

/**
* End point of the line
* @type {Point}
* @default "[100,100]"
*/
endPoint?: Point;

/**
* The point at which the leading tail segment joins the connecting center segment.
* @type {Point}
* @default null
*/
controlPoint1?: Point;

/**
* The point at which the trailing tail segment joins the connecting center segment.
*  Has no effect on lines with right angle ConnectorStyles.
* @type {Point}
* @default null
*/
controlPoint2?: Point;

/**
* Length of the horizontal/vertical "tail segments" between the start and end points of 
*  this DrawLinePath and the connecting center segment.
* @type {Distance}
* @default "30"
*/
tailSize?: Distance;

/**
* Starting left coordinate of the line. Overrides left coordinate of DrawLinePath.startPoint if
*  both are set.
* @type {Coordinate}
* @default "0 , IRW"
*/
startLeft?: Coordinate;

/**
* Starting top coordinate of the line. Overrides top coordinate of DrawLinePath.startPoint if
*  both are set.
* @type {Coordinate}
* @default "0 , IRW"
*/
startTop?: Coordinate;

/**
* Ending left coordinate of the line. Overrides left coordinate of DrawLinePath.endPoint if
*  both are set.
* @type {Coordinate}
* @default "0 , IRW"
*/
endLeft?: Coordinate;

/**
* Ending top coordinate of the line. Overrides top coordinate of DrawLinePath.endPoint if
*  both are set.
* @type {Coordinate}
* @default "0 , IRW"
*/
endTop?: Coordinate;

/**
* Style of arrow head to draw at the end of the line or path.
* @type {ArrowStyle}
* @default ""open", IRW"
*/
endArrow?: ArrowStyle;



/**
*  Get the center point of the line path.
* 
*/
getCenter(): Point;

/**
*  Update the startPoint
* 
* 
* @param {Coordinate} left coordinate for start point, in pixels
* @param {Coordinate} top coordinate for start point, in pixels
*/
setStartPoint(left:Coordinate, top:Coordinate): void;

/**
*  Update the endPoint
* 
* 
* @param {Coordinate} left coordinate for end point, in pixels
* @param {Coordinate} top coordinate for end point, in pixels
*/
setEndPoint(left:Coordinate, top:Coordinate): void;

/**
*  Sets the coordinates of the controlPoint1 knob and by extension the coordinates of this
*  DrawLinePath's leading tail segment.
* 
* @param {Coordinate} left coordinate for start point, in pixels
* @param {Coordinate} top coordinate for start point, in pixels
*/
setControlPoint1(left:Coordinate, top:Coordinate): void;

/**
*  Sets the coordinates of the controlPoint2 knob and by extension the coordinates of this
*  DrawLinePath's trailing tail segment.
* 
* @param {Coordinate} left coordinate for start point, in pixels
* @param {Coordinate} top coordinate for start point, in pixels
*/
setControlPoint2(left:Coordinate, top:Coordinate): void;

/**
*  Returns the startPoint, endPoint
* 
*/
getBoundingBox(): Array<Partial<Double>>;

/**
*  Move both the start and end points of the line by a relative amount.
* 
* 
* @param {Distance} change to left coordinate in pixels
* @param {Distance} change to top coordinate in pixels
*/
moveBy(left:Distance, top:Distance): void;

/**
*  Moves the line path such that the DrawLinePath.startPoint ends up at the specified point.
* 
* 
* @param {Integer} new left coordinate in pixels
* @param {Integer} new top coordinate in pixels
*/
moveStartPointTo(left:Integer, top:Integer): void;



}
/**
* Creates a new DrawLinePath
* 
* @param typeCheckedProperties {Partial<DrawLinePath>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawLinePath} The new instance
*/
class DrawLinePath{
static create(typeCheckedProperties? : Partial<DrawLinePath>, uncheckedProperties?: Object): DrawLinePath;







}

/**
* DrawItem subclass to render oval shapes, including circles.
*/
interface DrawOval extends DrawItem {
/**
* Left coordinate in pixels relative to the DrawPane.
* @type {Coordinate}
* @default "0"
*/
left?: Coordinate;

/**
* Top coordinate in pixels relative to the DrawPane.
* @type {Coordinate}
* @default "0"
*/
top?: Coordinate;

/**
* Width in pixels relative to the DrawPane.
* @type {Distance}
* @default "100"
*/
width?: Distance;

/**
* Height in pixels relative to the DrawPane.
* @type {Distance}
* @default "100"
*/
height?: Distance;

/**
* Center point of the oval. If unset, derived from left/top/width/height.
* @type {Point}
* @default null
*/
centerPoint?: Point;

/**
* Radius of the oval. Since this is used to initialize the DrawOval.getRadiusX
*  and DrawOval.getRadiusY radii, then the oval is a circle.
*  
*  If unset, the horizontal and vertical radii are set to half the DrawOval.width
*  and DrawOval.height.
* @type {Distance}
* @default null
*/
radius?: Distance;

/**
* The mode in which the DrawItem.titleLabel (if shown) is rotated with
*  this draw item.
* @type {TitleRotationMode}
* @default "withItemAlwaysUp"
*/
titleRotationMode?: TitleRotationMode;



/**
*  Returns the top, left, top+height, left+width 
* 
*/
getBoundingBox(): Array<Partial<Double>>;

/**
*  Change the center point for this oval.
* 
* @param {Coordinate} left coordinate (in the global coordinate system)
* @param {Coordinate} top coordinate (in the global coordinate system)
*/
setCenterPoint(left:Coordinate, top:Coordinate): void;

/**
*  Move the drawOval by the specified delta
* 
* @param {Distance} number of pixels to move horizontally
* @param {Distance} number of pixels to move vertically
*/
moveBy(dX:Distance, dY:Distance): void;

/**
*  Move the drawOval to the specified left/top position. You may also call
*  DrawOval.setCenterPoint to reposition the oval around a new center position.
* 
* @param {Integer} new left coordinate
* @param {Integer} new top coordinate
*/
moveTo(left:Integer, top:Integer): void;

/**
*  Set the left coordinate of the drawOval
* 
* @param {Coordinate} new left coordinate
*/
setLeft(left:Coordinate): void;

/**
*  Set the top coordinate of the drawOval
* 
* @param {Coordinate} new top coordinate
*/
setTop(top:Coordinate): void;

/**
*  Resize by the specified delta. Note that the resize will occur from the current top/left 
*  coordinates, meaning the center positon of the oval may change. You may also use
*  DrawOval.setRadii to change the radius in either direction without modifying the
*  centerpoint.
* 
* @param {Distance} number of pixels to resize by horizontally
* @param {Distance} number of pixels to resize by vertically
*/
resizeBy(dX:Distance, dY:Distance): void;

/**
*  Resize to the specified size. Note that the resize will occur from the current top/left 
*  coordinates, meaning the center positon of the oval may change. You may also use
*  DrawOval.setRadii to change the radius in either direction without modifying the
*  centerpoint.
* 
* @param {Integer} new width
* @param {Integer} new height
*/
resizeTo(width:Integer, height:Integer): void;

/**
*  Set the width of the drawOval
* 
* @param {Distance} new width
*/
setWidth(width:Distance): void;

/**
*  Set the height of the drawOval
* 
* @param {Distance} new height
*/
setHeight(height:Distance): void;

/**
*  Move and resize the drawOval to match the specified coordinates and size.
* 
* @param {Integer} new left coordinate
* @param {Integer} new top coordinate
* @param {Integer} new width
* @param {Integer} new height
*/
setRect(left:Integer, top:Integer, width:Integer, height:Integer): void;

/**
*  Returns the horizontal radius of the DrawOval.
* 
*/
getRadiusX(): Distance;

/**
*  Returns the vertical radius of the DrawOval.
* 
*/
getRadiusY(): Distance;

/**
*  Resize the drawOval by setting its horizontal and vertical radius, and retaining its current
*  center point.
* 
* @param {Distance} new horizontal radius
* @param {Distance} new vertical radius
*/
setRadii(rx:Distance, ry:Distance): void;

/**
*  Resize the drawOval by setting its radius, and retaining its current center point.
*  Equivalent to setRadii(radius, radius).
* 
* @param {Distance} new radius. This will be applied on both axes, meaning calling this method will always result in the DrawOval being a circle.
*/
setRadius(radius:Distance): void;

/**
*  Resize and reposition the drawOval by setting its radius, and centerPoint.
* 
* @param {Coordinate} new horizontal center point coordinate
* @param {Coordinate} new vertical center point coordinate
* @param {Distance} new horizontal radius
* @param {Distance} new vertical radius
*/
setOval(cx:Coordinate, cy:Coordinate, rx:Distance, ry:Distance): void;

/**
*  Get the center of the oval.
* 
*/
getCenter(): Point;



}
/**
* Creates a new DrawOval
* 
* @param typeCheckedProperties {Partial<DrawOval>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawOval} The new instance
*/
class DrawOval{
static create(typeCheckedProperties? : Partial<DrawOval>, uncheckedProperties?: Object): DrawOval;







}

/**
* A DrawPane is a container for drawing bitmap and vector graphics using browser's built-in
*  freeform drawing capabilities. These include the HTML5 &lt;canvas&gt; tag and
*  SVG (Scalable Vector Graphics) where available, and the VML (Vector Markup
*  Language) for legacy browsers (Internet Explorer 8 and earlier).
*  
*  To draw in a DrawPane you create DrawLines, DrawOvals,
*  DrawPaths and other DrawItem-based components, and place them in the
*  DrawPane via DrawPane.drawItems or add them incrementally via
*  DrawPane.addDrawItem.
*  
*  DrawItems support a variety of common features, such as 
*  Gradient, DrawItem.startArrow, events such as
*  DrawItem.click and built-in DrawItem.knobs for end user
*  resizing and manipulation of shapes.
*  
*  Common shapes such as DrawRect, DrawOval and
*  DrawTriangle have dedicated DrawItem subclasses. For other shapes,
*  consider:
*  
*  
* - DrawPath - a multi-segment line with straight segments, defined by a series
*    of DrawPath.points
*  
* - DrawPolygon - a closed shape with straight sides, defined by a series of
*    DrawPolygon.points
*  
* - DrawShape - a multi-segment line or closed shape whose sides can be defined by a
*    series of commands, including curved arcs
*  
* 
*  
* 
*  Note on Coordinate Systems
*  There are three different coordinate systems involved when a DrawItem is drawn onto a DrawPane:
*  
*  
* - The "local coordinate system" for a DrawItem refers to the Cartesian coordinate system
*    in which dimensional and positional values are interpreted. For example, when a
*    DrawRect is configured with left:20, top:30, width:200, and height:100, the
*    DrawRect represents a rectangle from (20, 30) to (220, 130) in its local coordinate
*    system. For this same DrawRect, DrawRect.top is going to be 30 even if
*    the shape is scaled by 3x, such that the (transformed) top coordinate in the drawing
*    coordinate system actually lies outside the visible region of the DrawPane. Similarly,
*    no matter what rotation is applied, DrawRect.top will continue to be 30.
*    
*    Use DrawItem.getBoundingBox to obtain the bounding box of the item in local
*    coordinates. Subclass properties also typically provide data in the local coordinate
*    system, such as DrawRect.left, DrawRect.top, DrawRect.width,
*    DrawRect.height, DrawPath.points, and DrawTriangle.points.
*    
*    There is a local coordinate system for each DrawItem.
*  
* - The "drawing coordinate system" refers to the Cartesian coordinate system shared by
*    all DrawItems after their local transforms, such as DrawItem.scale or
*    DrawItem.rotation, have been applied.
*    
*    Since DrawGroups pass through applied transforms to the underlying items,
*    DrawGroup properties such as DrawGroup.left, DrawGroup.top,
*    DrawGroup.width, and DrawGroup.height, represent coordinates in the drawing
*    coordinate system, as does therefore DrawGroup.getBoundingBox. The APIs
*    DrawPane.getDrawingPoint, DrawPane.getDrawingX, and
*    DrawPane.getDrawingY, also return drawing coordinates.
*    
*    For DrawItems with no local transforms, the drawing coordinate system is identical to
*    the local coordinate system.
*  
* - The "global coordinate system" refers to the drawing coordinate system with global
*    DrawPane transforms DrawPane.translate, DrawPane.zoomLevel and
*    DrawPane.rotation applied.
*    
*    Use DrawItem.getResizeBoundingBox to obtain the bounding box of a
*    DrawItem in global coordinates. The APIs DrawItem.getPageLeft and 
*    DrawItem.getPageTop reflect global coordinates rounded to the nearest pixel and
*    offset by the page-relative coordinates of the DrawPane's top left corner.
*    (See for example Canvas.getPageLeft and Canvas.getPageTop.)
*    
*    With the default global transforms, the global coordinate system is identical to the
*    drawing coordinate system.
*  
*  
*  The view port of the DrawPane is the rectangle in the global coordinate system from (0, 0)
*  that is as wide as the DrawPane's Canvas.getInnerContentWidth
*  and as high as the DrawPane's Canvas.getInnerContentHeight.
*  Note: In the case of a FacetChart showing a FacetChart.canZoom,
*  the view port height is decreased by the height of the zoom chart.
*  
*  One other coordinate system in use by a DrawPane when DrawPane.canDragScroll
*  is enabled is the "viewbox coordinate system". The viewbox coordinate system is the drawing
*  coordinate system with the DrawPane.translate and DrawPane.zoomLevel transforms
*  applied.
*/
interface DrawPane extends Canvas {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "DrawPaneEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Which type of drawing back-end should be used by this DrawPane? A default drawing
*  back-end is automatically selected based on the browser.
*  
*  The drawingType can only be set to a drawing back-end type that is supported
*  by the browser. It is provided for cases where the browser supports more than one drawing
*  back-end. See the DrawingType documentation for the supported drawing back-ends
*  and the list of browsers that support each type of drawing back-end.
* @type {DrawingType}
* @default "based on browser"
*/
drawingType?: DrawingType;

/**
* When DrawPane.canDragScroll is enabled, this is the width of the area
*  in viewbox coordinates that can be accessed through drag-scrolling.
* @type {Integer}
* @default "1000"
*/
drawingWidth?: Integer;

/**
* When DrawPane.canDragScroll is enabled, this is the height of the area
*  in viewbox coordinates that can be accessed through drag-scrolling.
* @type {Integer}
* @default "1000"
*/
drawingHeight?: Integer;

/**
* Can the user drag-scroll the DrawPane?
* @type {boolean}
* @default false
*/
canDragScroll?: boolean;

/**
* Rotation in degrees for the DrawPane as a whole about the center of the
*  DrawPane. The positive direction corresponds to clockwise rotation (for example,
*  45 is rotation clockwise by 45 degrees and -10 is rotation counterclockwise by 10 degrees).
* @type {Float}
* @default "0"
*/
rotation?: Float;

/**
* Zoom for the DrawPane as a whole, where 1 is normal size.
* @type {Float}
* @default "1"
*/
zoomLevel?: Float;

/**
* Global translation. This array has two numbers. The first number is the X translation amount
*  in pixels and the second number is the Y translation amount in pixels.
* @type {Array<Partial<Integer>>}
* @default null
*/
translate?: Array<Partial<Integer>>;

/**
* Array of gradients that can be referenced by DrawItems placed on this DrawPane.
*  Each gradient must have an ID assigned to be used for reference.
* @type {Array<Partial<Gradient>>}
* @default null
*/
gradients?: Array<Partial<Gradient>>;

/**
* Array of DrawItems to initially display in this DrawPane.
* @type {Array<Partial<DrawItem>>}
* @default null
*/
drawItems?: Array<Partial<DrawItem>>;



/**
*  Returns the X coordinate in the DrawPane of the last event.
*  Note: If you need both the X and Y coordinates in the drawing coordinate system of the last event,
*  it is more efficient to call DrawPane.getDrawingPoint instead.
*  
*  See the documentation of DrawPane.getDrawingPoint for a clarifying
*  example.
* 
*/
getDrawingX(): Integer;

/**
*  Returns the Y coordinate in the DrawPane of the last event.
*  Note: If you need both the X and Y coordinates in the drawing coordinate system of the last event,
*  it is more efficient to call DrawPane.getDrawingPoint instead.
*  
*  See the documentation of DrawPane.getDrawingPoint for a clarifying
*  example.
* 
*/
getDrawingY(): Integer;

/**
*  Returns the point of the last event in the DrawPane.
*  
*  To give a concrete example, suppose that this DrawPane has DrawPane.zoomLevel
*  2 and drag-panning is not enabled (just to simplify this example). If Canvas.getOffsetX
*  and Canvas.getOffsetY is (0, 0) (i.e. the mouse pointer is located at
*  the top left point of this DrawPane), then getDrawingPoint() would return
*  (0, 0). If getOffsetX/Y() is (20, 40), then getDrawingPoint() would return (10, 20) because
*  when the 2&times; zoom level is applied, (10, 20) is translated to (20, 40) on the screen. You
*  could, for example, create a new DrawLine with DrawLine.startPoint
*  (10, 20) and when this line is drawn on screen, the position of the line's start point would
*  be at offset (20, 40) on screen.
* 
*/
getDrawingPoint(): Point;

/**
*  Call DrawItem.erase on all DrawItems currently associated with the DrawPane. 
*  
*  The DrawItems will continue to exist, and you can call draw() on them to make them appear again, or
*  DrawItem.destroy to get rid of them permanetly. Use DrawPane.destroyItems to permanently
*  get rid of all DrawItems.
* 
* 
*/
erase(): void;

/**
*  Permanently DrawItem.destroy all DrawItems currently associated with this DrawPane,
*  leaving the DrawPane itself intact
* 
*/
destroyItems(): void;

/**
*  Adds a draw item to this DrawPane. If already added to a DrawPane,
*  the draw item is removed from its current DrawPane and added to this DrawPane.
*  
*  NOTE: For performance reasons, this DrawPane may draw the new item on a
*  delay to allow multiple items to be added and drawn at one time. The DrawPane.refreshNow
*  API will force the item to be drawn immediately.
* 
* @param {DrawItem} item to add.
* @param {boolean} If explicitly set to false, and this drawPane is drawn, don't draw  the newly added item
*/
addDrawItem(item:DrawItem, autoDraw:boolean): void;

/**
*  Get a "data:" URL encoding the current contents of the DrawPane.
*  
*  The returned "data:" URLs can be used anywhere a URL to an image is valid, for example,
*  Img.src.
*  
*  This method will directly return the data URL on modern browsers when using &lt;canvas&gt;-style
*  rendering (the default), and if there are no DrawImages in this DrawPane
*  that load cross-domain images.
*  
*  On legacy browers (any version of IE in "quirks" mode, all versions of IE prior to 9.0),
*  or if there is a DrawImage that loads a cross-domain image, data
*  URL generation requires a server trip and requires the SmartClient Server to be installed with
*  the same set of javaModuleDependencies as are required for PDF
*  export of charts in legacy IE. The method will return null and a callback must be passed,
*  which fires when the data URL has been retrieved from the server.
*  
*  If the callback is passed but no server trip is required, the callback is fired immediately.
*  
*  For obtaining PNG or other image data for use in server-side processing (such as attaching to
*  automated emails or saving to a database), see also the server-side APIs in
*  com.isomorphic.contentexport.ImageExport.
*  
*  Note: It is recommended to pass a callback instead of relying on the method returning the
*  data URL directly. This is because the callback will always be called with the generated
*  data URL, whereas work-arounds for browser bugs may require asynchronous generation of the
*  data URL, meaning that a data URL might not be returned immediately in certain browsers for
*  certain DrawPane contents.
* 
* 
* @param {DataURLCallback=} callback to fire when the data URL is available. The callback is called regardless of whether the data URL is also returned.
* @param {DataURLFormat=} the format of the data URL. If not specified, then "all"  is assumed.
*/
getDataURL(callback?:(dataURL:string) => void, format?:DataURLFormat): string;

/**
*  Converts this DrawPane to the source of an &lt;svg&gt; element equivalent to the
*  current drawing.
*  
*  In Pro edition and above, the returned string can be used with
*  RPCManager.exportImage to download an image, or with server-side APIs in
*  com.isomorphic.contentexport.ImageExport to obtain various kinds of images for further
*  server-side processing.
* 
* 
*/
getSvgString(): string;

/**
*  Retrieves printable HTML for this component and all printable subcomponents.
*  
*  By default any Canvas with children will simply collect the printable HTML of its
*  children by calling getPrintHTML() on each child that is considered
*  Canvas.shouldPrint.
*  
*  If overriding this method for a custom component, you should either return a String of
*  printable HTML string directly or return null, and fire the callback (if provided)
*  using Class.fireCallback.
*  
*  To return an empty print representation, return an empty string ("") rather than null.
*  
*  The printProperties argument, if passed, must be passed to any subcomponents on
*  which getPrintHTML() is called.
*  
*  Notes on printing
*  
*  To print a DrawPane for export on IE8 and earlier, it is important to pass
*  PrintProperties with PrintProperties.printForExport:true:
*  
*  var exportHTML = drawPane.getPrintHTML({ printForExport:true });
*  
* 
* @param {PrintProperties=} properties to configure printing behavior - may be null.
* @param {Callback=} optional callback. This is required to handle cases where HTML         generation is asynchronous - if a method generates HTML asynchronously, it should return          null, and fire the specified callback on completion of HTML generation. The          first parameter HTML should contain the generated print HTML.          The callback is only called if null is returned. Furthermore, the default          getPrintHTML() implementation always returns null and fires the callback          when a callback is provided.
*/
getPrintHTML(printProperties?:PrintProperties, callback?:Callback): HTMLString;

/**
*  Immediately draws or redraws any items of this DrawPane that are scheduled to be
*  drawn or redrawn after a delay.
*  
*  For performance reasons, this DrawPane may delay refreshing its display to
*  allow for multiple draw item updates to be drawn at the same time. If this is occurring,
*  refreshNow() will immediately refresh the display instead of refreshing the display in a timer.
* 
*/
refreshNow(): void;

/**
*  Returns the DrawPane's underlying HTML5 &lt;canvas&gt; element. Will only return a valid
*  element if the DrawingType is "bitmap".
*  
*  To create a DrawItem drawn by custom HTML5 &lt;canvas&gt; drawing code, you should:
*  
* - Subclass the DrawRect class, setting DrawItem.lineOpacity to 0, and
*  DrawItem.eventOpaque to true.
*  
* - Define your HTML5 &lt;canvas&gt; drawing routine as DrawItem.drawStart or
*  DrawItem.drawEnd.
*  
* - Limit your drawing to the DrawItem's 
*  DrawItem.getResizeBoundingBox.
*  
* 
*/
getBitmap(): DOMElement;

/**
*  Called when we start drawing to the underlying HTML5 &lt;canvas&gt; element of a DrawPane,
*  right after the element is cleared. Only called if the DrawingType is
*  "bitmap".
*  
*  There is no default implementation of this method.
* 
*/
drawStart(): void;

/**
*  Called after we finish drawing to the underlying HTML5 &lt;canvas&gt; element of a DrawPane,
*  after the last DrawItem has been drawn. Only called if the DrawingType is
*  "bitmap".
*  
*  There is no default implementation of this method.
* 
*/
drawEnd(): void;

/**
*  Sets the zoom on this DrawPane to the specified magnification, maintaining the
*  current viewport position.
* 
* 
* @param {Float} Desired zoom level as a float where 1.0 is equivalent to 100% magnification. Must be greater than 0.
*/
setZoomLevel(zoomLevel:Float): void;

/**
*  Synonym of DrawPane.setZoomLevel.
* 
* @param {Float} Desired zoom level as a float where 1.0 is equivalent to 100% magnification. Must be greater than 0.
*/
zoom(zoomLevel:Float): void;

/**
*  Sets the DrawPane.rotation of the DrawPane.
* 
* 
* @param {Float} the new rotation in degrees. The positive direction corresponds to clockwise rotation.
*/
setRotation(degrees:Float): void;

/**
*  Synonym of DrawPane.setRotation.
* 
* @param {Float} the new rotation in degrees. The positive direction corresponds to clockwise rotation.
*/
rotate(degrees:Float): void;

/**
*  Add a new gradient to the drawPane shared gradient list (DrawPane.gradients). If the
*  gradient does not have an ID a new one will be assigned.
* 
* 
* @param {Gradient} gradient to add
*/
addGradient(gradient:Gradient): Identifier;

/**
*  Returns gradient for gradientID.
* 
* 
* @param {Identifier} ID of gradient to retrieve
*/
getGradient(gradientID:Identifier): Gradient;

/**
*  Removes gradient for gradientID.
* 
* 
* @param {Identifier} ID of gradient to remove
*/
removeGradient(gradientID:Identifier): void;

/**
*  Creates a simple linear gradient which can be used by any DrawItem of this DrawPane.
*  Any DrawItem's DrawItem.fillGradient can reference the gradient by the
*  given ID.
* 
* 
* @param {Identifier} the ID of the simple linear gradient
* @param {SimpleGradient} the simple linear gradient
* @deprecated * in favor of DrawPane.addGradient

*/
createSimpleGradient(id:Identifier, simple:SimpleGradient): Identifier;

/**
*  Creates a linear gradient which can be used by any DrawItem of this DrawPane.
*  Any DrawItem's DrawItem.fillGradient can reference the gradient by the
*  given ID.
* 
* 
* @param {Identifier} the ID of the linear gradient
* @param {LinearGradient} the linear gradient
* @deprecated * in favor of DrawPane.addGradient

*/
createLinearGradient(id:Identifier, linearGradient:LinearGradient): Identifier;

/**
*  Creates a radial gradient which can be used by any DrawItem of this DrawPane.
*  Any DrawItem's DrawItem.fillGradient can reference the gradient by the
*  given ID.
* 
* 
* @param {Identifier} the ID of the radial gradient
* @param {RadialGradient} the radial gradient
* @deprecated * in favor of DrawPane.addGradient

*/
createRadialGradient(id:Identifier, radialGradient:RadialGradient): Identifier;



}
/**
* Creates a new DrawPane
* 
* @param typeCheckedProperties {Partial<DrawPane>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawPane} The new instance
*/
class DrawPane{
static create(typeCheckedProperties? : Partial<DrawPane>, uncheckedProperties?: Object): DrawPane;




/**
*  Computes the top-, left-, bottom-, and right-most coordinates in a list
*  of points, then translates and scales all points to fit the entire shape
*  into the given width and height.
*  
*  
*  The example call below scales a set of points into a 100x100 thumbnail:
* 
*  
*   var scaledPoints = DrawPane.scaleAndCenter(100, 100, 50, 50,
*       [[500, 50], [525, 50], [550, 75], [575, 75],
*        [600, 75], [600, 125], [575, 125], [550, 125],
*        [525, 150], [500, 150]]);
*  
*  
* 
* @param {Integer} width of target space
* @param {Integer} height of target space
* @param {Integer} center point x
* @param {Integer} center point y
* @param {Array<Partial<Point>>} list of points to scale and translate
*/
static scaleAndCenter(width:Integer, height:Integer, xc:Integer, yc:Integer, points:Array<Partial<Point>>): void;

/**
*  Computes a cubic Be&#769;zier curve polynomial:
*  
*  B(t) = (1 - t)3P1 + 3(1 - t)2tCP1 + 3(1 - t)t2CP2 + t3P2
*  
* 
* @param {Double} starting point coordinate
* @param {Double} first control point coordinate
* @param {Double} second control point coordinate
* @param {Double} end point coordinate
* @param {Double} the value of the parameter of the curve, between 0 and 1
*/
static bezier(p1:Double, cp1:Double, cp2:Double, p2:Double, t:Double): Double;

/**
*  Computes the minimum and maximum value of the cubic Be&#769;zier curve polynomial
*  defined in DrawPane.bezier,
*  for 0 &le; t &le; 1.
* 
* @param {Double} starting point coordinate
* @param {Double} first control point coordinate
* @param {Double} second control point coordinate
* @param {Double} end point coordinate
*/
static bezierExtrema(p1:Double, cp1:Double, cp2:Double, p2:Double): Array<Partial<Double>>;

/**
*  Calculate the bounding box of the cubic Be&#769;zier curve with endpoints
*  p1 and p2 and control points cp1 and
*  cp2.
* 
* @param {Point} start point of the curve
* @param {Point} first cubic Be&#769;zier control point
* @param {Point} second cubic Be&#769;zier control point
* @param {Point} end point of the curve
*/
static getBezierBoundingBox(p1:Point, cp1:Point, cp2:Point, p2:Point): Array<Partial<Double>>;

/**
*  Computes the top-, left-, bottom-, and right-most coordinates containing the
*  Be&#769;zier curve defined by startPoint, controlPoint1,
*  controlPoint2, and endPoint, then translates and scales these
*  four points to fit the entire curve into the given width and height.
* 
* @param {Integer} width of target space
* @param {Integer} height of target space
* @param {Integer} center point x
* @param {Integer} center point y
* @param {Point} start point of the curve
* @param {Point} end point of the curve
* @param {Point} first cubic Be&#769;zier control point
* @param {Point} second cubic Be&#769;zier control point
*/
static scaleAndCenterBezier(width:Integer, height:Integer, xc:Integer, yc:Integer, startPoint:Point, endPoint:Point, controlPoint1:Point, controlPoint2:Point): void;

/**
*  Calls DrawPane.getPolygonPoints with angles spread evenly over the full 360 degrees.
* 
* @param {Integer} the number of vertices the polygon
* @param {Integer} width of target space
* @param {Integer} height of target space
* @param {Integer} center point x
* @param {Integer} center point y
* @param {Double} the angle (in radians) with respect to the center point of the first vertex of the polygon
*/
static getRegularPolygonPoints(n:Integer, width:Integer, height:Integer, xc:Integer, yc:Integer, startAngle:Double): Array<Partial<Point>>;

/**
*  Computes an array of Points for a polygon that has an equal distance from its center to
*  any of its vertices and that fits in the given width and height.
* 
* @param {Integer} width of target space
* @param {Integer} height of target space
* @param {Integer} center point x
* @param {Integer} center point y
* @param {Array<Partial<Double>>} the complete list of angles (in radians) with respect to the center point at which the polygon must have vertices
*/
static getPolygonPoints(width:Integer, height:Integer, xc:Integer, yc:Integer, angles:Array<Partial<Double>>): Array<Partial<Point>>;




}

/**
* EditProxy that handles DrawPane when editMode is enabled.
*/
interface DrawPaneEditProxy extends CanvasEditProxy {




}
/**
* Creates a new DrawPaneEditProxy
* 
* @param typeCheckedProperties {Partial<DrawPaneEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawPaneEditProxy} The new instance
*/
class DrawPaneEditProxy{
static create(typeCheckedProperties? : Partial<DrawPaneEditProxy>, uncheckedProperties?: Object): DrawPaneEditProxy;







}

/**
* Draws a multi-segment line.
*/
interface DrawPath extends DrawItem {
/**
* DrawPath only supports the
*  "move"
*  
*  knob type.
* @type {Array<Partial<KnobType>>}
* @default null
*/
knobs?: Array<Partial<KnobType>>;

/**
* If the DrawItem.titleLabel is showing, should the DrawItem.titleLabelBackground
*  be created and placed behind the titleLabel?
*  
*  This defaults to true for DrawSectors and shapes that are not commonly filled
*  (e.g. DrawLines).
* @type {boolean}
* @default true
*/
showTitleLabelBackground?: boolean;

/**
* Array of Points for the line, specified in the DrawPane.
* @type {Array<Partial<Point>>}
* @default "[[0,0], [100,100]]"
*/
points?: Array<Partial<Point>>;



/**
*  Returns the min, max points
* 
*/
getBoundingBox(): Array<Partial<Double>>;

/**
*  Get the mean center of the path.
* 
*/
getCenter(): Point;

/**
*  Move all points in the path such that the first point ends up at the specified coordinates
*  and the line lengths and angles are unchanged.
* 
* 
* @param {Integer} new left coordinate in pixels
* @param {Integer} new top coordinate in pixels
*/
moveFirstPointTo(left:Integer, top:Integer): void;

/**
*  Move the points by dX,dY
* 
* 
* @param {Distance} delta x coordinate in pixels
* @param {Distance} delta y coordinate in pixels
*/
moveBy(dX:Distance, dY:Distance): void;

/**
*  Resize to the specified size
* 
* @param {Integer} new width
* @param {Integer} new height
*/
resizeTo(width:Integer, height:Integer): void;

/**
*  Resize by the specified delta
* 
* @param {Distance} number of pixels to resize by horizontally
* @param {Distance} number of pixels to resize by vertically
*/
resizeBy(dX:Distance, dY:Distance): void;



}
/**
* Creates a new DrawPath
* 
* @param typeCheckedProperties {Partial<DrawPath>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawPath} The new instance
*/
class DrawPath{
static create(typeCheckedProperties? : Partial<DrawPath>, uncheckedProperties?: Object): DrawPath;







}

/**
* DrawItem subclass to render polygons
*/
interface DrawPolygon extends DrawPath {
/**
* DrawPolygon only supports the
*  "move"
*  
*  knob type.
* @type {Array<Partial<KnobType>>}
* @default null
*/
knobs?: Array<Partial<KnobType>>;

/**
* If the DrawItem.titleLabel is showing, should the DrawItem.titleLabelBackground
*  be created and placed behind the titleLabel?
*  
*  This defaults to true for DrawSectors and shapes that are not commonly filled
*  (e.g. DrawLines).
* @type {boolean}
* @default false
*/
showTitleLabelBackground?: boolean;

/**
* Array of points of the polygon, specified in the DrawPane.
* @type {Array<Partial<Point>>}
* @default "[[0,0], [50,50], [100,0]]"
*/
points?: Array<Partial<Point>>;

/**
* Style of drawing the corners of the polygon.
* @type {LineCap}
* @default "butt"
*/
lineCap?: LineCap;

/**
* The mode in which the DrawItem.titleLabel (if shown) is rotated with
*  this draw item.
* @type {TitleRotationMode}
* @default "withItemAlwaysUp"
*/
titleRotationMode?: TitleRotationMode;





}
/**
* Creates a new DrawPolygon
* 
* @param typeCheckedProperties {Partial<DrawPolygon>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawPolygon} The new instance
*/
class DrawPolygon{
static create(typeCheckedProperties? : Partial<DrawPolygon>, uncheckedProperties?: Object): DrawPolygon;







}

/**
* DrawItem subclass to render rectangle shapes, optionally with rounded corners.
*/
interface DrawRect extends DrawItem {
/**
* Left coordinate in pixels relative to the DrawPane.
* @type {Coordinate}
* @default "0"
*/
left?: Coordinate;

/**
* Top coordinate in pixels relative to the DrawPane.
* @type {Coordinate}
* @default "0"
*/
top?: Coordinate;

/**
* Width in pixels relative to the DrawPane.
* @type {Distance}
* @default "100"
*/
width?: Distance;

/**
* Height in pixels relative to the DrawPane.
* @type {Distance}
* @default "100"
*/
height?: Distance;

/**
* Rounding of corners, from 0 (square corners) to 1.0 (shorter edge is a semicircle).
* @type {Float}
* @default "0"
*/
rounding?: Float;

/**
* Style of drawing the endpoints of a line.
*  
*  Note that for dashed and dotted lines, the lineCap style affects each dash or dot.
* @type {LineCap}
* @default "butt"
*/
lineCap?: LineCap;

/**
* The mode in which the DrawItem.titleLabel (if shown) is rotated with
*  this draw item.
* @type {TitleRotationMode}
* @default "withItemAlwaysUp"
*/
titleRotationMode?: TitleRotationMode;



/**
*  Move the drawRect such that it is centered over the specified coordinates.
* 
* @param {Coordinate} left coordinate for new center position
* @param {Coordinate} top coordinate for new center postiion
*/
setCenter(left:Coordinate, top:Coordinate): void;

/**
*  Get the center point of the rectangle.
* 
*/
getCenter(): Point;

/**
*  Returns the top, left, top+height, left+width 
* 
*/
getBoundingBox(): Array<Partial<Double>>;

/**
*  Returns true if the given point in the drawing coordinate system is within this DrawItem's shape,
*  taking into account local transforms.
* 
* @param {Integer} X coordinate of the test point.
* @param {Integer} Y coordinate of the test point.
*/
isPointInPath(x:Integer, y:Integer): boolean;

/**
*  Move the drawRect by the specified delta
* 
* @param {Distance} number of pixels to move horizontally
* @param {Distance} number of pixels to move vertically
*/
moveBy(dX:Distance, dY:Distance): void;

/**
*  Move the drawRect to the specified position
* 
* @param {Integer} new left coordinate
* @param {Integer} new top coordinate
*/
moveTo(left:Integer, top:Integer): void;

/**
*  Set the left coordinate of the drawRect
* 
* @param {Coordinate} new left coordinate
*/
setLeft(left:Coordinate): void;

/**
*  Set the top coordinate of the drawRect
* 
* @param {Coordinate} new top coordinate
*/
setTop(top:Coordinate): void;

/**
*  Resize to the specified size
* 
* @param {Integer} new width
* @param {Integer} new height
*/
resizeTo(width:Integer, height:Integer): void;

/**
*  Resize by the specified delta
* 
* @param {Distance} number of pixels to resize by horizontally
* @param {Distance} number of pixels to resize by vertically
*/
resizeBy(dX:Distance, dY:Distance): void;

/**
*  Set the width of the drawRect
* 
* @param {Distance} new width
*/
setWidth(width:Distance): void;

/**
*  Set the height of the drawRect
* 
* @param {Distance} new height
*/
setHeight(height:Distance): void;

/**
*  Move and resize the drawRect to match the specified coordinates and size.
* 
* @param {Integer} new left coordinate
* @param {Integer} new top coordinate
* @param {Integer} new width
* @param {Integer} new height
*/
setRect(left:Integer, top:Integer, width:Integer, height:Integer): void;

/**
*  Setter method for DrawRect.rounding
* 
* @param {Float} new rounding value. Should be between zero (a rectangle) and 1 (shorter  edge is a semicircle)
*/
setRounding(rounding:Float): void;



}
/**
* Creates a new DrawRect
* 
* @param typeCheckedProperties {Partial<DrawRect>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawRect} The new instance
*/
class DrawRect{
static create(typeCheckedProperties? : Partial<DrawRect>, uncheckedProperties?: Object): DrawRect;







}

/**
* DrawItem subclass to render Pie Slices.
*/
interface DrawSector extends DrawItem {
/**
* DrawSector only supports the
*  "move"
*  
*  knob type.
* @type {Array<Partial<KnobType>>}
* @default null
*/
knobs?: Array<Partial<KnobType>>;

/**
* If the DrawItem.titleLabel is showing, should the DrawItem.titleLabelBackground
*  be created and placed behind the titleLabel?
*  
*  This defaults to true for DrawSectors and shapes that are not commonly filled
*  (e.g. DrawLines).
* @type {boolean}
* @default true
*/
showTitleLabelBackground?: boolean;

/**
* Rotation in degrees about the DrawSector.centerPoint of the DrawSector.
*  The positive direction is clockwise.
* @type {Float}
* @default "0.0"
*/
rotation?: Float;

/**
* Center point of the sector
* @type {Point}
* @default "[0,0]"
*/
centerPoint?: Point;

/**
* Start angle of the sector in degrees. Default of 0.0 will create a sector that starts
*  with a line from the DrawSector.centerPoint and extends horizontally to the right for the 
*  indicated DrawSector.radius, then sweeps clockwise toward the DrawSector.endAngle.
*  
*  Note that the startAngle may validly be a greater numeric value than the endAngle.
*  The sector will always be drawn clockwise from startAngle to endAngle, so 
*  a sector with startAngle of 350 and endAngle of 10 would draw a 20-degree
*  segment sticking out to the right of the centerPoint.
*  
*  Drawing a full circle: A developer may have a drawSector transcribe a full circle by using
*  values of 360 or greater as the end point. For example a drawSector with startAngle set
*  to 350 and endAngle set to 710 would transcribe a full circle which starts and ends on the
*  same line (10 degrees above the horizontal, to the right of the centerPoint).
* @type {Float}
* @default "0.0"
*/
startAngle?: Float;

/**
* End angle of the sector in degrees. See DrawSector.startAngle for further
*  details.
* @type {Float}
* @default "20.0"
*/
endAngle?: Float;

/**
* Radius of the sector.
* @type {Distance}
* @default "100"
*/
radius?: Distance;



/**
*  Returns the centerPoint endPoint
* 
*/
getBoundingBox(): Array<Partial<Double>>;

/**
*  Returns the sector's DrawSector.centerPoint.
* 
*/
getCenter(): Point;

/**
*  Calculates the coordinates of the midpoint of this DrawSector's circular arc. The formula
*  for this point is:
*  
*  var averageAngle = (startAngle + endAngle) / 2; // in degrees
* [centerX + radius * cosdeg(averageAngle), centerY + radius * sindeg(averageAngle)]
*  
* 
*/
getArcMidpoint(): Point;

/**
*  Change the center point for this sector.
* 
* @param {Coordinate} X coordinate of the center point (in the global coordinate system).
* @param {Coordinate} Y coordinate of the center point (in the global coordinate system.
*/
setCenterPoint(left:Coordinate, top:Coordinate): void;

/**
*  Move the DrawSector by the specified amounts.
* 
* @param {Distance} number of pixels to move by horizontally
* @param {Distance} number of pixels to move by vertically
*/
moveBy(x:Distance, y:Distance): void;



}
/**
* Creates a new DrawSector
* 
* @param typeCheckedProperties {Partial<DrawSector>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawSector} The new instance
*/
class DrawSector{
static create(typeCheckedProperties? : Partial<DrawSector>, uncheckedProperties?: Object): DrawSector;




/**
*  Calculates the midpoint coordinates of the circular arc of the sector defined by the
*  given centerPoint, startAngle, endAngle, and radius. The formula for this point is:
*  
*  var averageAngle = (startAngle + endAngle) / 2; // in degrees
* [centerX + radius * cosdeg(averageAngle), centerY + radius * sindeg(averageAngle)]
*  
* 
* @param {Double} X coordinate of the center point of the sector.
* @param {Double} Y coordinate of the center point of the sector.
* @param {Double} start angle of the sector in degrees.
* @param {Double} end angle of the sector in degrees.
* @param {Double} radius of the sector.
*/
static getArcMidpoint(centerX:Double, centerY:Double, startAngle:Double, endAngle:Double, radius:Double): Point;




}

/**
* DrawItem to render a shape defined by executing the series of drawing commands in the
*  DrawShape.commands array.
*/
interface DrawShape extends DrawItem {
/**
* The mode in which the DrawItem.titleLabel (if shown) is rotated with
*  this draw item.
* @type {TitleRotationMode}
* @default "withItemAlwaysUp"
*/
titleRotationMode?: TitleRotationMode;

/**
* The drawing commands that will be executed to render the shape.
* @type {Array<Partial<DrawShapeCommand>>}
* @default null
*/
commands?: Array<Partial<DrawShapeCommand>>;



/**
*  Sets the DrawShape.commands that define this shape.
* 
* @param {Array<Partial<DrawShapeCommand>>} the new commands.
*/
setCommands(commands:Array<Partial<DrawShapeCommand>>): void;

/**
*  Move the drawShape by the specified delta
* 
* @param {Distance} number of pixels to move horizontally
* @param {Distance} number of pixels to move vertically
*/
moveBy(dX:Distance, dY:Distance): void;

/**
*  Resize by the specified delta
* 
* @param {Distance} number of pixels to resize by horizontally
* @param {Distance} number of pixels to resize by vertically
*/
resizeBy(dX:Distance, dY:Distance): void;



}
/**
* Creates a new DrawShape
* 
* @param typeCheckedProperties {Partial<DrawShape>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawShape} The new instance
*/
class DrawShape{
static create(typeCheckedProperties? : Partial<DrawShape>, uncheckedProperties?: Object): DrawShape;







}

/**
* DrawItem subclass to render triangles
*/
interface DrawTriangle extends DrawPolygon {
/**
* Array of points of the triangle. specified in the
*  DrawPane.
* @type {Array<Partial<Point>>}
* @default "[[0,0], [50,50], [100,0]]"
*/
points?: Array<Partial<Point>>;



/**
*  Returns the incenter
*  of the triangle in DrawPane.
* 
*/
getCenter(): Point;

/**
*  Resize by the specified delta
* 
* @param {Distance} number of pixels to resize by horizontally
* @param {Distance} number of pixels to resize by vertically
*/
resizeBy(dX:Distance, dY:Distance): void;



}
/**
* Creates a new DrawTriangle
* 
* @param typeCheckedProperties {Partial<DrawTriangle>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DrawTriangle} The new instance
*/
class DrawTriangle{
static create(typeCheckedProperties? : Partial<DrawTriangle>, uncheckedProperties?: Object): DrawTriangle;







}

/**
* The DynamicForm manages a collection of FormItems which represent user input controls. The
*  DynamicForm provides formLayout, value management, validation and
*  databinding for the controls it manages.
*  
*  
*  
*  To create a DynamicForm, set DynamicForm.fields to an Array of Objects describing the
*  FormItems you want to use. For example:
*  
*   isc.DynamicForm.create({
*     fields:[
*       {name:"userName", type:"text"}, // creates a TextItem
*       {name:"usState", type:"select"} // creates a SelectItem
*     ]
*   })
*  
*  
*  The item name is an identifier for the item that must be unique just within
*  this form. It is used:
*  
*  
* - as the name under which the item's value is stored in the form (the form's
*    current values are accessible as DynamicForm.getValues
*  
* - when retrieving the FormItem's current value (via
*    DynamicForm.getValue) 
*  
* - to retrieve the item itself via DynamicForm.getItem
*  
*  FormItems can also be created by binding the form to a DataSource via
*  setDataSource(). In this case, FormItems are
*  chosen based on the data type of the field - see FormItemType. You can override
*  the automatically chosen FormItem via DataSourceField.editorType. 
*  
*  FormItem lifecycle is managed by the DynamicForm itself. FormItem instances are created
*  and destroyed automatically when new fields are added to the form. 
*  
*  When using DataSource binding, you can also add additional FormItems not specified in the
*  DataSource, or override any properties on the automatically generated FormItems, without
*  having to re-declare any information that comes from the DataSource. See the QuickStart
*  Guide chapter on Data Binding for an overview.
*  
*  All FormItems share a common set of properties for controlling formLayout. Other properties common to all FormItems are documented on the FormItem
*  class, and properties specific to particular FormItems are documented on the respective
*  FormItems. 
*  
*  NOTE: For very simple forms consisting of exactly one item, you still use a DynamicForm.
*  See the "fontSelector" form in the ${isc.DocUtils.linkForExampleId('toolstrip', 'Toolstrip example')}.
*/
interface DynamicForm extends Omit<Canvas, 'setDragTracker'>  {
/**
* If true, when this component is first drawn, automatically call this.fetchData().
*  Criteria for this fetch may be picked up from DynamicForm.initialCriteria, and textMatchStyle may
*  be specified via ListGrid.autoFetchTextMatchStyle.
*  
*  NOTE: if autoFetchData is set, calling
*  ListGrid.fetchData before draw will cause two requests to be issued, one from the manual
*  call to fetchData() and one from the autoFetchData setting. The second request will use
*  only DynamicForm.initialCriteria and not any other criteria or settings from the first request.
*  Generally, turn off autoFetchData if you are going to manually call ListGrid.fetchData at any time.
* 
*  Note: If you are using saved searches - either via SavedSearchItem or ListGrid.saveDefaultSearch, autoFetchData
*  will be automatically suspended and replaced with the saved criteria/view state, if applicable.
* @type {boolean}
* @default false
*/
autoFetchData?: boolean;

/**
* If DynamicForm.autoFetchData is true, this attribute allows the developer to
*  specify a textMatchStyle for the initial ListGrid.fetchData call.
* @type {TextMatchStyle}
* @default null
*/
autoFetchTextMatchStyle?: TextMatchStyle;

/**
* Criteria to be used when DynamicForm.autoFetchData is set.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
initialCriteria?: Criteria;

/**
* Criteria that are never shown to or edited by the user and are cumulative with any criteria
*  provided via DataBoundComponent.initialCriteria and related methods.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
implicitCriteria?: Criteria;

/**
* Associated userTask if current dynamic form used along with workflow. 
*  See UserTask for more details.
* @type {UserTask}
* @default null
*/
userTask?: UserTask;

/**
* When calling DynamicForm.saveData on a form or valuesManager, by default if the server
*  returns an error code, any callback passed into saveData() will not be fired.
*  If the error code returned by the server indicates a validation error, it will be
*  displayed to the user by updating the form items to show the error messages, and firing
*  any specified hiddenValidationErrors handler, otherwise the standard RPCManager
*  error handling logic would be invoked.
*  
*  Developers who want to handle errors themselves can override this default by specifying
*  RPCRequest.willHandleError on the DSRequest. In this case the callback passed in 
*  will be fired even if the server returns an error status code.
*  
*  If suppressValidationErrorCallback is set to true, if a save attempt returns
*  a validation error code, the user-specified callback will not be fired
*  even if willHandleError:true was specified on the dsRequest 
*  - though for other error codes, the callback would be fired if willHandleError is 
*  specified on the request.
*  Note that this is the historical behavior for
*  SmartClient builds 8.0 and earlier
*  
* @type {boolean}
* @default false
*/
suppressValidationErrorCallback?: boolean;

/**
* Default DSOperationType to be performed when DynamicForm.saveData is called.
*  This property is automatically set on a call to DynamicForm.editRecord or
*  DynamicForm.editNewRecord, or may be set directly via 
*  DynamicForm.setSaveOperationType.
*  
*  If saveOperationType is unset, the form will heuristically determine
*  whether an "add" or "update" operation is intended based on whether the primaryKey field
*  is present and editable.
* @type {DSOperationType}
* @default null
*/
saveOperationType?: DSOperationType;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "FormEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Synonym for DynamicForm.fields
* @type {Array<Partial<Partial<FormItem>>>}
* @default null
*/
items?: Array<Partial<Partial<FormItem>>>;

/**
* An array of field objects, specifying the order, layout, and types of each field in the
*  DynamicForm.
*  
*  When both dynamicForm.fields and dynamicForm.dataSource are
*  set, dynamicForm.fields acts as a set of overrides as explained in
*  DataBoundComponent.fields.
*  
*  See formLayout for information about how flags specified on
*  the FormItems control how the form is laid out.
* @type {Array<Partial<Partial<FormItem>>>}
* @default null
*/
fields?: Array<Partial<Partial<FormItem>>>;

/**
* An Object containing the initial values of the form as properties, where each
*  propertyName is the name of a Items in the form, and each property
*  value is the value held by that form item.
*  
*  The form's values may contain values that are not managed by any FormItem, and these
*  values will be preserved and available when the form values are subsequently retrieved
*  via DynamicForm.getValues.
*  
*  Providing values on initialization is equivalent to calling DynamicForm.setValues.
*  
*  As the user manipulates form items to change values, change events fire
*  FormItem.change and 
*  DynamicForm.itemChange.
*  
*  Note that form values are logical values, for example, the value of a DateItem is
*  a JavaScript Date object, not a String, even if the user enters the date via a text
*  input. Likewise the value of a TextItem or CheckboxItem that started out
*  null remains null until the user changes it; the value will not be automatically
*  converted to the null string ("") or false respectively, as happens with native HTML
*  elements.
* @type {object}
* @default null
*/
values?: object;

/**
* Layout style to use with this form. 
*  
*  The default of "table" uses a tabular layout similar to HTML tables, but with much more
*  powerful control over sizing, item visibility and reflow, overflow handling, etc.
*  
*  itemLayout:"absolute" allows absolute positioning of every form item. This
*  provides maximum flexibility in placement, with the following limitations:
*  
* - titles, which normally take up an adjacent cell, are not shown. Use
*    StaticTextItems to show titles
*  
* - no automatic reflow when showing or hiding items. FormItem.setLeft
*    and FormItem.setTop can be used for manual reflow.
*  
* - only pixel and percent sizes are allowed, no "*". Percent widths mean percentage
*    of the overall form size rather than the column size
*  
* - with different font styling or internationalized titles, items may overlap that did
*    not overlap in the skin used at design time
*  
* @type {string}
* @default "table"
*/
itemLayout?: string;

/**
* Switches the entire form to render in a style that is typical for form on a mobile
*  device with one FormItem per row.
*  
*  The DynamicForm.linearOnMobile attribute allows linear mode to be enabled on
*  mobile interfaces automatically.
*  
*  While linearMode is active, the form's table layout is determined using the
*  following logic:
*  
* - If TitleOrientation is not explicitly specified, it will be defaulted to 
*    top, causing each item's title to appear above its element.
*    Note: in linear mode, each item will have the same title orientation.
*    The item-level FormItem.titleOrientation attribute will be ignored.
*  
* - For those items with a FormItem.hint that support
*    TextItem.showHintInField,
*    showHintInField will be defaulted true if unset.
*  
* - For items showing validation errors inline, by default
*    DynamicForm.showErrorText will be displayed, and
*    errors will DynamicForm.errorOrientation.
*  
* - The DynamicForm.numCols property for the form is ignored. The form will render as
*    a single column of items - or two columns if titleOrientation is
*    specified as "left" or "right" by default.
*    To override this, a developer may specify an explicit DynamicForm.linearNumCols
*    allowing multiple items to potentially be placed next to each other.
*  
* - FormItem.width is ignored and all items receive "*" width unless
*    FormItem.linearWidth is specified.
*  
*  Note that if column widths have been specified via DynamicForm.colWidths, but the number
*  of columns doesn't match the number of columns being rendered in linear mode, it will be
*  ignored.
*  
*  If DynamicForm.linearAutoSpanItems is true, each item in linear mode will span the full
*  set of columns by default. With this setting enabled, linearNumCols can 
*  be understood as a way to allow some items that are known to not require the full width
*  of the UI to appear next to each other in an otherwise single-column form.
*  
*  When linearAutoSpanItems is enabled, the following properties are available
*  for further customizing the placement and sizing of items in a linear form:
*  
*  
* - FormItem.colSpan is ignored and defaulted to "*", unless 
*    FormItem.linearColSpan is specified.
*  
* - FormItem.startRow and FormItem.endRow are ignored and
*    all items will default to startRow:false and endRow:true.
*    This may be overridden per item via FormItem.linearStartRow and
*    FormItem.linearEndRow.
*  
*  For the best appearance, try to get your form to horizontally fill the screen, or almost
*  all of it, on handset-sized devices. Any kind of fixed width on mobile is probably not
*  going to work as well. One way to achieve this is by using a SplitPane.
* @type {boolean}
* @default null
*/
linearMode?: boolean;

/**
* When a form is rendered in DynamicForm.linearMode, should each item appear on its own
*  row, spanning the full set of rendered columns by default?
* @type {boolean}
* @default true
*/
linearAutoSpanItems?: boolean;

/**
* Switches the entire form to render in a style that is typical for form on a
*  Browser.isHandset with one FormItem per row by
*  automatically defaulting DynamicForm.linearMode to true on handsets.
*  
*  Note: This property should not be changed framework-wide via
*  Class.addProperties as the framework itself assumes and relies upon normal
*  behavior for forms. If you want most of your forms to use DynamicForm.linearMode on mobile,
*  create a subclass where linearOnMobile defaults to true, and pervasively
*  use that subclass.
* @type {boolean}
* @default false
*/
linearOnMobile?: boolean;

/**
* The number of columns of titles and items in this form's layout grid. A title and
*  corresponding item each have their own column, so to display two form elements per
*  row (each having a title and item), you would set this property to 4.
* @type {number}
* @default "2"
*/
numCols?: number;

/**
* Specifies the DynamicForm.numCols when the form is being rendered in
*  DynamicForm.linearMode, overriding any automatically derived value in that mode.
* @type {number}
* @default null
*/
linearNumCols?: number;

/**
* If true, we ensure that column widths are at least as large as you specify them. This
*  means that if any single column overflows (due to, eg, a long unbreakable title),
*  the form as a whole overflows.
*  
*  If false, columns will have their specified sizes as long as no column overflows. If
*  any column overflows, space will be taken from any other columns that aren't filling the
*  available room, until there is no more free space, in which case the form as a whole
*  overflows.
* @type {boolean}
* @default false
*/
fixedColWidths?: boolean;

/**
* An array of widths for the columns of items in this form's layout grid. 
*  
*  If specified, these widths should sum to the total width of the form (form.width).
*  If not specified, we assume every other column will contain form item titles, and so
*  should have form.titleWidth, and all other columns should share the
*  remaining space.
*  
*  Acceptable values for each element in the array are:
*  
*  
* - A number (e.g. 100) representing the number of pixel widths to allocate to a
*    column.
*  
* - A percent (e.g. 20%) representing the percentage of the total form.width to
*    allocate to a column.
*  
* - "*" (all) to allocate remaining width (form.width minus all specified column
*    widths). Multiple columns can use "*", in which case remaining width is divided
*    between all columns marked "*".
*  
*  
*  Note that if title columns are left at the default DynamicForm.titleWidth or assigned a fixed
*  width, while the others are configured to use the remaining horizontal space (i.e. with a
*  percent or "*" as described above), then care must be taken if you have long titles with
*  no spaces or DynamicForm.wrapItemTitles is false.
*  
*  Depending on the title font and exact column width applied, the title may overflow its
*  assigned column, causing the form itself to overflow. If the form's parent has
*  Canvas.overflow: "auto" and the form has width: "100%" or its parent is
*  a Layout with Layout.hPolicy: "fill", this could cause a horizontal
*  scrollbar to appear in a situation where it doesn't seem necessary.
*  
*  If the parent's height is just right so that the space taken by the unwanted horizontal
*  scrollbar introduces a vertical scrollbar, this may even lead to oscillating scrollbars
*  on the parent. To avoid, you must address the original problem of the title overflowing
*  its assigned column, by widening it, using a smaller font, or allowing wrapping to occur.
* @type {Array<any>}
* @default null
*/
colWidths?: Array<any>;

/**
* Minimum width of a form column.
* @type {number}
* @default "20"
*/
minColWidth?: number;

/**
* The amount of empty space, in pixels, surrounding each form item within its cell in
*  the layout grid.
* @type {number}
* @default "2"
*/
cellPadding?: number;

/**
* Width of border for the table that form is drawn in. This is primarily used for debugging
*  form layout.
* @type {number}
* @default "0"
*/
cellBorder?: number;

/**
* If the form has sections, [implemented as SectionItems], this attribute controls
*  whether multiple sections can be expanded at once.
* @type {VisibilityMode}
* @default "multiple"
*/
sectionVisibilityMode?: VisibilityMode;

/**
* For databound forms, whether to show fields marked as detail fields.
* @type {boolean}
* @default true
*/
showDetailFields?: boolean;

/**
* When creating form items for fields with text type data, if the specified length of the
*  field exceeds this threshold we will create form item of type 
*  this.longTextEditorType (a TextAreaItem by default), rather than a simple
*  text item. Overridden by explicitly specifying editorType for the field.
* @type {number}
* @default "255"
*/
longTextEditorThreshold?: number;

/**
* Name of the Form Item class to use for text fields which exceed the 
*  longTextEditorThreshold for this form.
* @type {string}
* @default "textArea"
*/
longTextEditorType?: string;

/**
* Default DateDisplayFormat for Date type values displayed in this form.
*  
*  If some field's value is set to a native Date object, how should it be displayed to the
*  user? If specified this is the default display format to use, and will apply to all fields
*  except those specified as FormItem.type 
*  (See DynamicForm.timeFormatter).
*  
*  May be overridden at the component level for fields of type datetime via 
*  DynamicForm.datetimeFormatter.
*  
*  Note that if specified, FormItem.dateFormatter and FormItem.timeFormatter
*  take precedence over the format specified at the component level.
*  
*  If no explicit formatter is specified at the field or component level, dates will be 
*  formatted according to the system-wide
*  DateUtil.setShortDisplayFormat or 
*  DateUtil.setShortDatetimeDisplayFormat depending
*  on the specified field type.
* @type {DateDisplayFormat}
* @default null
*/
dateFormatter?: DateDisplayFormat;

/**
* Default TimeDisplayFormat for FormItem.type field values displayed
*  in this form.
*  
*  Note that if specified, FormItem.dateFormatter and FormItem.timeFormatter
*  take precedence over the format specified at the component level.
*  
*  If no explicit formatter is specified at the field or component level, time values will be 
*  formatted according to the system-wide
*  Time.setNormalDisplayFormat.
*  specified field type.
* @type {TimeDisplayFormat}
* @default null
*/
timeFormatter?: TimeDisplayFormat;

/**
* Default DateDisplayFormat for Date type values displayed in this form in fields
*  of type datetime.
*  
*  For datetime fields, this attribute will be used instead of DynamicForm.dateFormatter
*  when formatting Date values.
*  
*  Note that if specified, FormItem.dateFormatter and FormItem.timeFormatter
*  take precedence over the format specified at the component level.
*  
*  If no explicit formatter is specified at the field or component level, datetime field
*  values will be formatted according to the system-wide
*  DateUtil.setShortDatetimeDisplayFormat.
* @type {DateDisplayFormat}
* @default null
*/
datetimeFormatter?: DateDisplayFormat;

/**
* If set at init time, this dynamicForm will be created as a member form of the
*  specified valuesManager. To update the form's valuesManager after init, use the
*  DynamicForm.setValuesManager, or the 
*  ValuesManager.addMember /
*  ValuesManager.removeMember APIs on 
*  ValuesManager.
* @type {ValuesManager | GlobalId}
* @default null
*/
valuesManager?: ValuesManager | GlobalId;

/**
* Default orientation for titles for items in this form. TitleOrientation
*  lists valid options.
*  
*  Note that titles on the left or right take up a cell in tabular
*  formLayout, but titles on top do not.
* @type {TitleOrientation}
* @default "left"
*/
titleOrientation?: TitleOrientation;

/**
* The string pre-pended to the title of every item in this form. See also +{requiredTitlePrefix} for
*  fields that are required.
* @type {HTMLString}
* @default ""
*/
titlePrefix?: HTMLString;

/**
* The string pre-pended to the title of an item in this form if its
*  titleOrientation property is set to "right".
* @type {HTMLString}
* @default ":&amp;nbsp;"
*/
rightTitlePrefix?: HTMLString;

/**
* The string appended to the title of every item in this form. See also +{requiredTitleSuffix} for
*  fields that are required.
* @type {HTMLString}
* @default "&amp;nbsp;:"
*/
titleSuffix?: HTMLString;

/**
* The string appended to the title of an item in this form if its titleOrientation
*  property is set to "right".
* @type {HTMLString}
* @default ""
*/
rightTitleSuffix?: HTMLString;

/**
* The width in pixels allocated to the title of every item in this form. If you
*      don't specify explicit DynamicForm.colWidths, you can set this
*      value to the string "*" to divide the usable space evenly between titles and
*      fields.
* @type {number | "*"}
* @default "100"
*/
titleWidth?: number | "*";

/**
* Should the titles for form items be clipped if they are too large for the available 
*  space?
*  
*  Can be overridden for individual items via FormItem.clipTitle.
* @type {boolean}
* @default false
*/
clipItemTitles?: boolean;

/**
* Whether titles for form items should wrap. If not specified, titles will wrap by
*  default. Can be overridden for individual items via FormItem.wrapTitle
* @type {boolean}
* @default null
*/
wrapItemTitles?: boolean;

/**
* If true, field errors are written into the form next to the item(s) where the errors
*  occurred. Errors may appear as text or just an icon (via DynamicForm.showErrorText:false).
*  
*  If false, errors are written at the top of the form.
*  
*  To do some other kind of error display, override DynamicForm.showErrors.
* @type {boolean}
* @default true
*/
showInlineErrors?: boolean;

/**
* DynamicForm.showErrorIcons, 
*  DynamicForm.showErrorText, DynamicForm.errorOrientation,
*  and DynamicForm.showErrorStyle control how validation errors are
*  displayed when they are displayed inline in the form (next to the form item where there
*  is a validation error). To instead display all errors at the top of the form, set
*  DynamicForm.showInlineErrors:false.
*  
*  showErrorIcons, showErrorText, errorOrientation
*  and showErrorStyle
*  are all boolean properties, and can be set on a DynamicForm to control the behavior
*  form-wide, or set on individual FormItems. 
*  
*  The HTML displayed next to a form item with errors is generated by 
*  FormItem.getErrorHTML.
*  The default implementation of that method respects showErrorIcons and
*  showErrorText as follows:
*  
*  showErrorIcons, or showErrorIcon at the FormItem level controls
*  whether an error icon should appear next to fields which have validation errors. The icon's
*  appearance is governed by FormItem.errorIconSrc, FormItem.errorIconWidth and
*  FormItem.errorIconHeight
*  
*  showErrorText determines whether the text of the validation error should be
*  displayed next to fields which have validation errors. The attribute
*  DynamicForm.showTitlesWithErrorMessages may be set to prefix error messages with the 
*  form item's title + ":" (may be desired if the item has 
*  FormItem.showTitle set to false).
*  If showErrorText is unset, the error text will be shown if
*  DynamicForm.linearMode is true (or DynamicForm.linearOnMobile is true for
*  mobile devices), otherwise it will not be shown.
*  
*  In addition to this:
*  
*  DynamicForm.errorOrientation controls where the error HTML should appear relative 
*  to form items. Therefore the combination of DynamicForm.showErrorText:false and
*  DynamicForm.errorOrientation:"left" creates a compact validation error display
*  consisting of just an icon, to the left of the item with the error message
*  available via a hover (similar appearance to ListGrid validation error display).
*  If errorOrientation is unset, the error orientation will default to "top" 
*  if DynamicForm.linearMode is enabled (or DynamicForm.linearOnMobile is true
*  for mobile devices) and error text is not showing, "left" otherwise.
*  
*  showErrorStyle determines whether fields with validation
*  errors should have special styling applied to them. Error styling is achieved by
*  applying suffixes to existing styling applied to various parts of the form item.
*  See FormItemBaseStyle for more on this.
* @type {boolean}
* @default true
*/
showErrorIcons?: boolean;

/**
* DynamicForm.showErrorIcons, 
*  DynamicForm.showErrorText, DynamicForm.errorOrientation,
*  and DynamicForm.showErrorStyle control how validation errors are
*  displayed when they are displayed inline in the form (next to the form item where there
*  is a validation error). To instead display all errors at the top of the form, set
*  DynamicForm.showInlineErrors:false.
*  
*  showErrorIcons, showErrorText, errorOrientation
*  and showErrorStyle
*  are all boolean properties, and can be set on a DynamicForm to control the behavior
*  form-wide, or set on individual FormItems. 
*  
*  The HTML displayed next to a form item with errors is generated by 
*  FormItem.getErrorHTML.
*  The default implementation of that method respects showErrorIcons and
*  showErrorText as follows:
*  
*  showErrorIcons, or showErrorIcon at the FormItem level controls
*  whether an error icon should appear next to fields which have validation errors. The icon's
*  appearance is governed by FormItem.errorIconSrc, FormItem.errorIconWidth and
*  FormItem.errorIconHeight
*  
*  showErrorText determines whether the text of the validation error should be
*  displayed next to fields which have validation errors. The attribute
*  DynamicForm.showTitlesWithErrorMessages may be set to prefix error messages with the 
*  form item's title + ":" (may be desired if the item has 
*  FormItem.showTitle set to false).
*  If showErrorText is unset, the error text will be shown if
*  DynamicForm.linearMode is true (or DynamicForm.linearOnMobile is true for
*  mobile devices), otherwise it will not be shown.
*  
*  In addition to this:
*  
*  DynamicForm.errorOrientation controls where the error HTML should appear relative 
*  to form items. Therefore the combination of DynamicForm.showErrorText:false and
*  DynamicForm.errorOrientation:"left" creates a compact validation error display
*  consisting of just an icon, to the left of the item with the error message
*  available via a hover (similar appearance to ListGrid validation error display).
*  If errorOrientation is unset, the error orientation will default to "top" 
*  if DynamicForm.linearMode is enabled (or DynamicForm.linearOnMobile is true
*  for mobile devices) and error text is not showing, "left" otherwise.
*  
*  showErrorStyle determines whether fields with validation
*  errors should have special styling applied to them. Error styling is achieved by
*  applying suffixes to existing styling applied to various parts of the form item.
*  See FormItemBaseStyle for more on this.
* @type {boolean}
* @default false
*/
showErrorText?: boolean;

/**
* DynamicForm.showErrorIcons, 
*  DynamicForm.showErrorText, DynamicForm.errorOrientation,
*  and DynamicForm.showErrorStyle control how validation errors are
*  displayed when they are displayed inline in the form (next to the form item where there
*  is a validation error). To instead display all errors at the top of the form, set
*  DynamicForm.showInlineErrors:false.
*  
*  showErrorIcons, showErrorText, errorOrientation
*  and showErrorStyle
*  are all boolean properties, and can be set on a DynamicForm to control the behavior
*  form-wide, or set on individual FormItems. 
*  
*  The HTML displayed next to a form item with errors is generated by 
*  FormItem.getErrorHTML.
*  The default implementation of that method respects showErrorIcons and
*  showErrorText as follows:
*  
*  showErrorIcons, or showErrorIcon at the FormItem level controls
*  whether an error icon should appear next to fields which have validation errors. The icon's
*  appearance is governed by FormItem.errorIconSrc, FormItem.errorIconWidth and
*  FormItem.errorIconHeight
*  
*  showErrorText determines whether the text of the validation error should be
*  displayed next to fields which have validation errors. The attribute
*  DynamicForm.showTitlesWithErrorMessages may be set to prefix error messages with the 
*  form item's title + ":" (may be desired if the item has 
*  FormItem.showTitle set to false).
*  If showErrorText is unset, the error text will be shown if
*  DynamicForm.linearMode is true (or DynamicForm.linearOnMobile is true for
*  mobile devices), otherwise it will not be shown.
*  
*  In addition to this:
*  
*  DynamicForm.errorOrientation controls where the error HTML should appear relative 
*  to form items. Therefore the combination of DynamicForm.showErrorText:false and
*  DynamicForm.errorOrientation:"left" creates a compact validation error display
*  consisting of just an icon, to the left of the item with the error message
*  available via a hover (similar appearance to ListGrid validation error display).
*  If errorOrientation is unset, the error orientation will default to "top" 
*  if DynamicForm.linearMode is enabled (or DynamicForm.linearOnMobile is true
*  for mobile devices) and error text is not showing, "left" otherwise.
*  
*  showErrorStyle determines whether fields with validation
*  errors should have special styling applied to them. Error styling is achieved by
*  applying suffixes to existing styling applied to various parts of the form item.
*  See FormItemBaseStyle for more on this.
* @type {boolean}
* @default true
*/
showErrorStyle?: boolean;

/**
* DynamicForm.showErrorIcons, 
*  DynamicForm.showErrorText, DynamicForm.errorOrientation,
*  and DynamicForm.showErrorStyle control how validation errors are
*  displayed when they are displayed inline in the form (next to the form item where there
*  is a validation error). To instead display all errors at the top of the form, set
*  DynamicForm.showInlineErrors:false.
*  
*  showErrorIcons, showErrorText, errorOrientation
*  and showErrorStyle
*  are all boolean properties, and can be set on a DynamicForm to control the behavior
*  form-wide, or set on individual FormItems. 
*  
*  The HTML displayed next to a form item with errors is generated by 
*  FormItem.getErrorHTML.
*  The default implementation of that method respects showErrorIcons and
*  showErrorText as follows:
*  
*  showErrorIcons, or showErrorIcon at the FormItem level controls
*  whether an error icon should appear next to fields which have validation errors. The icon's
*  appearance is governed by FormItem.errorIconSrc, FormItem.errorIconWidth and
*  FormItem.errorIconHeight
*  
*  showErrorText determines whether the text of the validation error should be
*  displayed next to fields which have validation errors. The attribute
*  DynamicForm.showTitlesWithErrorMessages may be set to prefix error messages with the 
*  form item's title + ":" (may be desired if the item has 
*  FormItem.showTitle set to false).
*  If showErrorText is unset, the error text will be shown if
*  DynamicForm.linearMode is true (or DynamicForm.linearOnMobile is true for
*  mobile devices), otherwise it will not be shown.
*  
*  In addition to this:
*  
*  DynamicForm.errorOrientation controls where the error HTML should appear relative 
*  to form items. Therefore the combination of DynamicForm.showErrorText:false and
*  DynamicForm.errorOrientation:"left" creates a compact validation error display
*  consisting of just an icon, to the left of the item with the error message
*  available via a hover (similar appearance to ListGrid validation error display).
*  If errorOrientation is unset, the error orientation will default to "top" 
*  if DynamicForm.linearMode is enabled (or DynamicForm.linearOnMobile is true
*  for mobile devices) and error text is not showing, "left" otherwise.
*  
*  showErrorStyle determines whether fields with validation
*  errors should have special styling applied to them. Error styling is achieved by
*  applying suffixes to existing styling applied to various parts of the form item.
*  See FormItemBaseStyle for more on this.
* @type {string}
* @default "left"
*/
errorOrientation?: string;

/**
* If DynamicForm.showInlineErrors is false we show all errors for the form item in 
*  a single item rendered at the top of the form.
*  This attribute contains a properties block for this item.
* @type {object}
* @default null
*/
errorItemProperties?: object;

/**
* If DynamicForm.showInlineErrors is false we show all errors for the form item in 
*  a single item rendered at the top of the form.
*  This attribute specifies the cellStyle to apply to this item.
* @type {string}
* @default "formCellError"
*/
errorItemCellStyle?: string;

/**
* If DynamicForm.showInlineErrors is false, all errors for the items
*  in the form are rendered as a single item at the top of the form. This attribute specifies
*  an introductory message rendered out before the individual error messages.
* @type {HTMLString}
* @default "The following errors were found:"
*/
errorsPreamble?: HTMLString;

/**
* Indicates whether on validation failure, the error message displayed to the
*      user should be pre-pended with the title for the item.
* @type {boolean}
* @default false
*/
showTitlesWithErrorMessages?: boolean;

/**
* Indicates whether the titles of required items in this form should use the special
*  prefix and suffix specified by the DynamicForm.requiredTitlePrefix and 
*  DynamicForm.requiredTitleSuffix attributes, instead of the standard prefix and suffix.
* @type {boolean}
* @default true
*/
hiliteRequiredFields?: boolean;

/**
* The string pre-pended to the title of every required item in this form if
*  DynamicForm.hiliteRequiredFields is true.
* @type {HTMLString}
* @default "&lt;b&gt;"
*/
requiredTitlePrefix?: HTMLString;

/**
* The string pre-pended to the title of every required item in this form if
*  DynamicForm.hiliteRequiredFields is true and the TitleOrientation property is set to "right".
* @type {HTMLString}
* @default "&lt;b&gt;:&amp;nbsp;"
*/
requiredRightTitlePrefix?: HTMLString;

/**
* The string appended to the title of every required item in this form if
*  DynamicForm.hiliteRequiredFields is true.
* @type {HTMLString}
* @default "&amp;nbsp;:&lt;/b&gt;"
*/
requiredTitleSuffix?: HTMLString;

/**
* The string appended to the title of every required item in this form if
*  DynamicForm.hiliteRequiredFields is true and the TitleOrientation property is set to "right".
* @type {HTMLString}
* @default "&lt;/b&gt;"
*/
requiredRightTitleSuffix?: HTMLString;

/**
* The required message for required field errors.
* @type {HTMLString}
* @default null
*/
requiredMessage?: HTMLString;

/**
* If set to false, the form will be marked read-only. A widget on the form
*  is editable if either (1) beginning with the widget and continuing up the containment
*  hierarchy, including the form, the first widget to have a non-null canEdit
*  attribute has canEdit:true, or (2) neither the widget nor any parent has a non-null
*  canEdit attribute. This setting allows you to enable or disable the default
*  editability of the form's items at one time.
*  
*  This setting differs from the enabled/disabled state in that most form items will
*  allow copying of the contents while read-only but do not while disabled.
*  
*  Note that a form is considered editable if canEdit is null (default) or
*  true.
* 
*  
* @type {boolean}
* @default null
*/
canEdit?: boolean;

/**
* If DynamicForm.canEdit is set to false, how should the items in this
*  form be displayed to the user?
*  
*  Can be overridden via FormItem.readOnlyDisplay on individual form items.
* @type {ReadOnlyDisplayAppearance}
* @default "readOnly"
*/
readOnlyDisplay?: ReadOnlyDisplayAppearance;

/**
* Default FormItem.readOnlyTextBoxStyle setting for items in this form.
* @type {FormItemBaseStyle}
* @default "staticTextItem"
*/
readOnlyTextBoxStyle?: FormItemBaseStyle;

/**
* For fields of FormItemType, if the field is non editable, and
*  being displayed with FormItem.readOnlyDisplay, should
*  the value (URL) be displayed as text, or should an image be rendered?
*  
*  May be overridden for individual items via FormItem.showImageAsURL.
* @type {boolean}
* @default false
*/
showImageAsURL?: boolean;

/**
* Default FormItem.clipStaticValue setting for items in this form. When unset, this
*  is equivalent to false.
* @type {boolean}
* @default null
*/
clipStaticValue?: boolean;

/**
* Default FormItem.showDeletions setting for items in this form.
* @type {boolean}
* @default null
*/
showDeletions?: boolean;

/**
* Should items within this form that are showing a FormItem.hint have the hint text
*  wrap? May be overridden at the item level via FormItem.wrapHintText. If
*  wrapHintText is unset on both the form and item, then the default behavior
*  is not wrapping the hint.
*  
*  This setting does not apply to hints that are TextItem.showHintInField.
* @type {boolean}
* @default true
*/
wrapHintText?: boolean;

/**
* Minimum horizontal space made available for FormItem.hint text.
*  Typically this reflects how much space the hint text takes up before it wraps.
*  May be overridden at the item level via FormItem.minHintWidth.
*  
*  This setting does not apply to hints that are TextItem.showHintInField.
* @type {Integer}
* @default "80"
*/
minHintWidth?: Integer;

/**
* If the user rolls over an item, how long a delay before we fire any hover action / show
*  a hover for that item?
* @type {number}
* @default "500"
*/
itemHoverDelay?: number;

/**
* A default width for hovers shown for items
* @type {number | string}
* @default null
*/
itemHoverWidth?: number | string;

/**
* A default height for hovers shown for items
* @type {number | string}
* @default null
*/
itemHoverHeight?: number | string;

/**
* Text alignment for hovers shown for items
* @type {Alignment}
* @default null
*/
itemHoverAlign?: Alignment;

/**
* Vertical text alignment for hovers shown for items
* @type {number | string}
* @default null
*/
itemHoverVAlign?: number | string;

/**
* CSS Style for hovers shown for items
* @type {CSSStyleName}
* @default "formHover"
*/
itemHoverStyle?: CSSStyleName;

/**
* Opacity for hovers shown for items
* @type {number}
* @default null
*/
itemHoverOpacity?: number;

/**
* Default setting for the form items' FormItem.showOldValueInHover setting.
* @type {boolean}
* @default null
*/
showOldValueInHover?: boolean;

/**
* Default template HTML string when an item does not set its own FormItem.originalValueMessage.
*  
*  Variables in the template are substituted as follows:
*  
*  
*   Variable
*   Substitution
*  
*  
*   $value
*   The item's old value as stored in the
*     object
*     returned by DynamicForm.getOldValues.
*  
*  
*   $newValue
*   The item's new value.
*  
*  
*  
*  For $value and $newValue, any formatters or stored/display value
*  mapping will be applied.
* @type {HTMLString}
* @default "Original value: $value"
*/
originalValueMessage?: HTMLString;

/**
* A property list of itemName:errorMessage pairs, specifying the set of error messages
*      displayed with the corresponding form elements. Each errorMessage may be either a
*      single string or an array of strings, for example:
*      {fieldName:errors, fieldName:errors}
*      where each errors object will be either an error message string or an array
*      of error message strings.
* @type {object}
* @default null
*/
errors?: object;

/**
* If true, form fields will be validated when each item's "change" handler is fired
*  as well as when the entire form is submitted or validated.
*  
*  Note that this property can also be set at the item level or on each validator
*  to enable finer granularity validation in response to user interaction.
*  If true at the form or field level, validators not explicitly set with
*  validateOnChange:false will be fired on change - displaying errors and
*  rejecting the change on validation failure.
* @type {boolean}
* @default false
*/
validateOnChange?: boolean;

/**
* If validateOnChange is true, and validation fails for an item on change, with no suggested
*  value, should we revert to the previous value, or continue to display the bad value entered
*  by the user. May be set at the item or form level.
* @type {boolean}
* @default null
*/
rejectInvalidValueOnChange?: boolean;

/**
* The error message for a failed validator that does not specify its own errorMessage.
* @type {HTMLString}
* @default "Invalid value"
*/
unknownErrorMessage?: HTMLString;

/**
* A message to display to the user if server-side validation fails with an error but the
*  server did not provide an error message
* @type {string}
* @default "Error during validation; no error details were provided"
*/
noErrorDetailsMessage?: string;

/**
* If true, form items will be validated when each item's "editorExit" handler is fired
*  as well as when the entire form is submitted or validated.
*  
*  Note that this property can also be set at the item level to enable finer granularity
*  validation in response to user interaction - if true at either level, validation
*  will occur on editorExit.
* @type {boolean}
* @default false
*/
validateOnExit?: boolean;

/**
* When true, indicates that changes to items in this form will be automatically saved on a 
*  DynamicForm.implicitSaveDelay, as well as when the entire form is
*  submitted. Unless DynamicForm.implicitSaveOnBlur is set 
*  to false, changes will also be automatically saved on editorExit for each item. This
*  attribute can also be set directly on FormItems.
* @type {boolean}
* @default false
*/
implicitSave?: boolean;

/**
* If true, form item values will be automatically saved when each item's "editorExit" 
*  handler is fired as well as on a delay and when the entire form is submitted. This
*  attribute can also be set directly on FormItems.
* @type {boolean}
* @default false
*/
implicitSaveOnBlur?: boolean;

/**
* When DynamicForm.implicitSave is true, this attribute dictates the 
*  millisecond delay after which form items are automatically saved during editing.
* @type {number}
* @default "2000"
*/
implicitSaveDelay?: number;

/**
* Indicates that if validation fails, the user should not be allowed to exit
*  the field - focus will be forced back into the field until the error is corrected.
*  
*  Enabling this property also implies FormItem.validateOnExit is automatically
*  enabled. If there are server-based validators on this item, setting this property
*  also implies that FormItem.synchronousValidation is forced on.
* @type {boolean}
* @default null
*/
stopOnError?: boolean;

/**
* If enabled, whenever validation is triggered and a request to the server is required,
*  user interactivity will be blocked until the request returns. Can be set for the entire
*  form or individual FormItems.
*  
*  If false, the form will try to avoid blocking user interaction until it is strictly
*  required. That is until the user attempts to use a FormItem whose state could be
*  affected by a server request that has not yet returned.
* @type {boolean}
* @default false
*/
synchronousValidation?: boolean;

/**
* If true, when the form is drawn, focus will automatically be put into the first focusable
*  element in the form.
*  Note that to put focus in a different item you can explicitly call 
*  dynamicForm.focusInItem(itemName)
* @type {boolean}
* @default false
*/
autoFocus?: boolean;

/**
* If true, when DynamicForm.validate fails focus will automatically
*  be put into the first focusable field which failed validation.
* @type {boolean}
* @default true
*/
autoFocusOnError?: boolean;

/**
* If this property is set to true, whenever a text-based field in this form 
*  (TextItem, TextAreaItem) is given focus programmatically 
*  (see DynamicForm.focusInItem), all text within the item will be selected.
*  
*  Note that this flag affects only programmatic focus. It's the normal behavior of text
*  fields to select all text if the user navigates into them via keyboard, or if the user
*  navigates via mouse, to place the text insertion point at the mouse click, and
*  SmartClient preserves these behaviors. selectOnFocus is only needed for
*  cases like a form within a pop-up dialog that should have the first field selected.
*  
*  If you also want the value to be selected when the user clicks on the field, set 
*  DynamicForm.selectOnClick instead.
*  
*  If selectOnFocus is false, the selection is not modified on focus - any
*  previous selection within the item will be maintained.
*  
*  May be overridden at the form item level via FormItem.selectOnFocus.
* @type {boolean}
* @default false
*/
selectOnFocus?: boolean;

/**
* If this property is set to true, whenever a text-based field in this form 
*  (TextItem, TextAreaItem) is given focus - whether
*  programmatically (see DynamicForm.focusInItem), or via a mouse click, all text
*  within the item will be selected.
*  
*  If you only want the value to be selected when on programmatic focus or keyboard
*  navigation (this is the native browser behavior), set 
*  DynamicForm.selectOnFocus instead.
*  
*  May be overridden at the form item level via FormItem.selectOnClick.
* @type {boolean}
* @default false
*/
selectOnClick?: boolean;

/**
* DynamicForms are considered to have focus if any of their form items have focus.
*  Note that setting dynamicForm.canFocus to false will have no effect on
*  whether form items within the form may receive focus. This property will only govern
*  whether the form may receive focus if the form contains no focusable items.
* @type {boolean}
* @default true
*/
canFocus?: boolean;

/**
* Should this form allow browser auto-completion of its items' values?   
*  Applies only to items based on native HTML form elements (TextItem,
*  PasswordItem, etc), and will only have a user-visible impact for browsers
*  where native autoComplete behavior is actually supported and enabled via user settings.
*  
*  This property may be explicitly specified per item via FormItem.autoComplete.
*  
*  Note that even with this value set to "none", native browser 
*  auto-completion may occur for log in forms (forms containing username and 
*  PasswordItem fields). This behavior varies by browser, and is
*  a result of an 
*  intentional change by some browser developers
*  to disregard the HTML setting autocomplete=off for password items or
*  log-in forms.
* @type {AutoComplete}
* @default "none"
*/
autoComplete?: AutoComplete;

/**
* If this browser has a 'spellCheck' feature for text-based form item elements, should
*  it be used for items in this form? Can be overridden at the item level via 
*  FormItem.browserSpellCheck
*  
*  Notes:
*  - this property only applies to text based items such as TextItem and TextAreaItem.
*  - this property is not supported on all browsers.
* @type {boolean}
* @default true
*/
browserSpellCheck?: boolean;

/**
* validationURL can be set to do server-side validation against a different URL from where
*  the form will ultimately save, as part of an incremental upgrade strategy for
*  Struts-like applications. 
*  
*  If set, calling DynamicForm.submit causes an RPC to be sent to this URL to
*  perform server-side validation of the form values. If the validation fails, the
*  validation errors returned by the server are rendered in the form. If the validation
*  succeeds, the form is submitted to the URL specified by DynamicForm.action.
*  
*  The form values are available on the server as request parameters (just like a normal form
*  submit) and also as the values of a DSRequest sent as an RPC alongside the normal
*  submit.
*  
*  The expected response to this request is a DSResponse sent via the RPC mechanism. If
*  validation is successful, an empty response with the STATUS_SUCCESS status code is
*  sufficient. If there are validation errors, the DSResponse should have the status set to 
*  STATUS_VALIDATION_ERROR and the errors should be set on the response via the
*  addError()/setErrorReport() API on DSResponse. See the javadoc for DSResponse for
*  details.
* @type {URL}
* @default null
*/
validationURL?: URL;

/**
* If set to true, client-side validators will not run on the form when validate() is
*  called. Server-side validators (if any) will still run on attempted save.
* @type {boolean}
* @default null
*/
disableValidation?: boolean;

/**
* The URL to which the form will submit its values.
*  
*  NOTE: this is used only in the very rare case that a form is used to submit data
*  directly to a URL. Normal server contact is through RPCManager.
*  See DynamicForm.canSubmit for more on this.
* @type {URL}
* @default "#"
*/
action?: URL;

/**
* The name of a window or frame that will receive the results returned by the form's
*  action. The default null indicates to use the current frame.
*  
*  NOTE: this is used only in the very rare case that a form is used to submit data
*  directly to a URL. Normal server contact is through
*  dataBoundComponentMethods.
* @type {string}
* @default null
*/
target?: string;

/**
* The mechanism by which form data is sent to the action URL. See FormMethod type
*  for details.
*  
*  NOTE: this is used only in the very rare case that a form is used to submit data
*  directly to a URL. Normal server contact is through 
*  dataBoundComponentMethods.
* @type {FormMethod}
* @default "POST"
*/
method?: FormMethod;

/**
* encoding for the form, use MULTIPART_ENCODING for file upload forms
* @type {Encoding}
* @default "normal"
*/
encoding?: Encoding;

/**
* Governs whether this form will be used to perform a standard HTML form submission.
*  Note that if true, DynamicForm.submit will perform a native HTML submission
*  to the specified DynamicForm.action URL.
*  Wherever possible we strongly recommend using the 
*  dataBoundComponentMethods to send data to
*  the server as they provide a far more sophisticated interface, with built in 
*  options for server validation, required fields, etc.
* @type {boolean}
* @default false
*/
canSubmit?: boolean;

/**
* If true, when the user hits the Enter key while focused in a text-item in
*  this form, we automatically submit the form to the server using the 
*  DynamicForm.submit method.
* @type {boolean}
* @default false
*/
saveOnEnter?: boolean;

/**
* Keyboard shortcut that causes the value of the currently focused form item to be reverted
*  to whatever value would be shown if DynamicForm.resetValues were called.
* @type {KeyIdentifier}
* @default null
*/
revertValueKey?: KeyIdentifier;

/**
* When FormItem.operator has been set for any FormItem in this form, what
*  logical operator should be applied across the Criterion produced by the form
*  items? Only applicable to forms that have a DataBoundComponent.dataSource.
* @type {OperatorId}
* @default "and"
*/
operator?: OperatorId;

/**
* If set, this DynamicForm will set both 
*  DataBoundComponent.showComplexFields and 
*  showComplexFieldsRecursively on any nested component used for showing/editing
*  a complex field. Thus any of this form's items that handle complex fields will themselves
*  also show complex fields. This allows for handling of field structures of any complexity.
*  
*  If set, this value automatically sets DataBoundComponent.showComplexFields
*  as well.
* @type {boolean}
* @default null
*/
showComplexFieldsRecursively?: boolean;

/**
* FormItem class to use for any singular (ie, non-list) complex fields 
*  on this DynamicForm.
* @type {string}
* @default "NestedEditorItem"
*/
nestedEditorType?: string;

/**
* FormItem class to use for any list-type complex fields on this DynamicForm.
*  List-type fields are denoted by marking them multiple: true in the 
*  DataSource.
* @type {string}
* @default "NestedListEditorItem"
*/
nestedListEditorType?: string;

/**
* This property applies to all of the items that a form has, and works according to 
*  FormItem.showPending. 
*  Also, in a form with showPending:true, an individual FormItem can set 
*  showPending:false and vice versa.
* @type {boolean}
* @default null
*/
showPending?: boolean;

/**
* How to fetch and manage records retrieve from the server. See FetchMode.
*  
*  This setting only applies to the ResultSet automatically created by calling
*  ListGrid.fetchData. If a pre-existing ResultSet is passed to setData() instead, it's
*  existing setting for ResultSet.fetchMode applies.
* @type {FetchMode}
* @default "paged"
*/
dataFetchMode?: FetchMode;

/**
* The DataSource that this component should bind to for default fields and for performing
*  DSRequest.
*  
*  Can be specified as either a DataSource instance or the String ID of a DataSource.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* Default OperatorId to use for fields in a form that produces
*  AdvancedCriteria. Default is "iContains" unless DynamicForm.allowExpressions is
*  enabled for the form as a whole, in which case the default is
*  DataSource.translatePatternOperators.
*  
*  Does not apply to special fields where exact match is obviously the right default
*  setting, such as fields of type:"enum", or fields with a
*  FormItem.valueMap or FormItem.optionDataSource.
*  
*  defaultSearchOperator also has no effect in a form that does not produce
*  AdvancedCriteria - see DynamicForm.getValuesAsCriteria for
*  settings that cause a form to produce AdvancedCriteria.
* @type {OperatorId}
* @default null
*/
defaultSearchOperator?: OperatorId;

/**
* Should users be able to tab into the FormItem.icons and 
*  FormItem.showPickerIcon for items within this form by default?
*  
*  May be overridden at the item level by FormItem.canTabToIcons.
*  
*  Developers may also suppress tabbing to individual icons by 
*  setting FormItemIcon.tabIndex to -1.
* @type {boolean}
* @default true
*/
canTabToIcons?: boolean;

/**
* A DynamicForm is a DataBoundComponent.dataArity:single component.
* @type {string}
* @default "single"
*/
dataArity?: string;

/**
* For editable fields with a specified FormItem.displayField and 
*  FormItem.optionDataSource, if the user selects a new value (typically from
*  PickList based item such as a SelectItem), should the selected displayValue be updated
*  on the record being edited in addition to the value for the actual item.
*  Note that this only applies for fields using 
*  FormItem.useLocalDisplayFieldValue - typically
*  DataSourceField.foreignKey where the display field is
*  DataSourceField.includeFrom another dataSource.
*  
*  Default value is true. This is typically desirable for editing records
*  with a displayField-mapped field, as it ensures the edited record will be be updated
*  to contain the correct display value as well as the correct data value. As such, the
*  expected display value is available on the record for display (for example in a ListGrid
*  cell).
*  
*  It may not be desirable for an interface specifically intended for 
*  DynamicForm.getValuesAsCriteria - in this case, results ought
*  to be limited by an item's actual selected value, not by whatever text is displayed to 
*  the user.
*  
*  See DataSourceField.displayField for more details.
*  
*  Note: the modified display field value will be passed to the server along with the
*  modified foreignKey field value if a 
*  DynamicForm.saveData is performed. This occurs
*  even if the displayField is
*  DataSourceField.includeFrom and therefore
*  read-only. In this case the server will simply ignore the modified display field value.
*  This is as expected - a subsequent fetch for the same record would recalculate the
*  displayField value on the server using the updated foreignKey field value (and return
*  the same display value previously displayed to the user).
*  
*  This attribute can also be set for FormItem.storeDisplayValues.
* @type {boolean}
* @default true
*/
storeDisplayValues?: boolean;

/**
* If true, the headers for any SectionStackSection.items will be included in the page's tab
*  order for accessibility. May also be set at the item level via SectionItem.canTabToHeader
*  
*  If unset, section headers will be focusable if isc.setScreenReaderMode
*  
*  has been called.
*  See accessibility.
* @type {boolean}
* @default null
*/
canTabToSectionHeaders?: boolean;

/**
* Default alignment for item titles. If unset default alignment will be derived from
*  Page.isRTL as described in DynamicForm.getTitleAlign
* @type {Alignment}
* @default null
*/
titleAlign?: Alignment;

/**
* For dynamicForms containing a FileItem for uploading files,
*  should the browser verify that the file is accessible before submitting
*  the uploaded file to the server?
*  
*  In some cases the browser may not be able to access the selected file.
*  This can occur when the file has been modified in the file system after selection in the
*  browser, or if the current user doesn't have permission to view the file.
*  
*  By default, before submitting the file to the server
*  the browser will verify that it can access the file's contents and display 
*  the DynamicForm.fileAccessFailedWarning if file access fails.
*  Note that accessing the file's contents is an asynchronous process, so
*  form submission is not performed synchronously.
*  
*  This means that if application code calls DynamicForm.saveData on a form containing a
*  fileItem and then synchronously Canvas.clear it from the DOM, the upload will
*  never be kicked off.
*  Setting checkFileAccessOnSubmit to false will suppress the (asynchronous) check, 
*  and can be used to bypass this limitation, but this is not recommended except as a short term
*  backwards-compatibility workaround. Instead we'd recommend using the DynamicForm.saveData
*  to clear the form when the upload has completed. This also gives the user an opportunity to correct
*  validation errors and re-submit the form if necessary.
* @type {boolean}
* @default true
*/
checkFileAccessOnSubmit?: boolean;

/**
* Warning to display to the user if an attempt to DynamicForm.submitForm a
*  form is unable to submit to the server. The most common cause for this failure is that the user
*  has typed an invalid file-path into an upload type field.
* @deprecated * see DynamicForm.formSubmitFailed

* @type {string}
* @default "Form was unable to be submitted. The most likely cause for this is an invalid value in an upload field."
*/
formSubmitFailedWarning?: string;

/**
* Warning to display to the user if a selected file in an UploadItem cannot be accessed.
*  This will be displayed on form submission when the browser is unable to access the
*  selected file for upload.
*  
*  Typically this indicates a browser-level security restriction - for example the file has been edited
*  on the disk, or had its permissions changed after the user selected it, but before they 
*  attempted to submit the form.
*  
*  When this occurs the selected file will be cleared from the upload item and this message will be
*  displayed to the user in a isc.warn.
* @type {string}
* @default "Unable to access the selected file(s) for upload. Please re-select the file and try again."
*/
fileAccessFailedWarning?: string;

/**
* If this component is bound to a dataSource, this attribute may be specified to customize
*  what fields from the dataSource may be edited by default. For example the SearchForm
*  class has this attribute set to "canFilter" which allows search forms to edit
*  dataSource fields marked as canEdit:false (but not those marked as
*  canFilter:false).
*  
*  Note that if canEdit is explicitly specified on a field in 
*  the DataBoundComponent.fields array, that property will be respected in preference to 
*  the canEditAttribute value. (See FormItem.canEdit, ListGridField.canEdit).
*  Also note that individual dataBoundComponents may have additional logic around whether a field
*  can be edited - for example ListGrid.canEditCell may be overridden.
* @type {Identifier}
* @default "canEdit"
*/
canEditFieldAttribute?: Identifier;

/**
* For a form that produces filter criteria
*  (see DynamicForm.getValuesAsCriteria), allows the user to
*  enter simple expressions in any field in this form that takes text input.
*  
*  Also note that enabling allowExpressions for an entire form changes the
*  DynamicForm.defaultSearchOperator to
*  DataSource.translatePatternOperators,
*  so that simple search expressions similar to SQL "LIKE" patterns can be entered in most
*  fields.
*  
*  See FormItem.allowExpressions for details.
* @type {boolean}
* @default null
*/
allowExpressions?: boolean;

/**
* Default TextItem.suppressBrowserClearIcon value for TextItems within this 
*  form.
* @type {boolean}
* @default false
*/
suppressBrowserClearIcons?: boolean;



/**
*  Notification fired when an asynchronous validation completes.
* 
* @param {boolean} true if validation succeeded, false if it failed
* @param {object} Map of errors by fieldName. Will be null if validation succeeded.
*/
handleAsyncValidationReply(success:boolean, errors:object): void;

/**
*  Is this component waiting for an asynchronous validation to complete?
*  This method will return true after DynamicForm.validate is called on a component with
*  server-side validators for some field(s), until the server responds.
*  
*  Note that the notification method DynamicForm.handleAsyncValidationReply will be fired
*  when validation completes.
* 
*/
isPendingAsyncValidation(): boolean;

/**
* 
*  Prepare to edit a new record by clearing the current set of values (or replacing them 
*  with initialValues if specified).
*  
*  This method will also call DynamicForm.setSaveOperationType to ensure
*  subsequent calls to saveData() will use an add rather than
*  an update operation.
* 
* 
* @param {Record | object=} initial set of values for the editor as a map of field names to their corresponding   values
*/
editNewRecord(initialValues?:Record | object): void;

/**
* 
*  Edit an existing record. Updates this editors values to match the values of the record 
*  passed in, via DynamicForm.setValues.
*  
*  This method will also call DynamicForm.setSaveOperationType to ensure 
*  subsequent calls to saveData() will use an update rather than
*  an add operation.
* 
* 
* @param {Record} the record to be edited as a map of field names to their corresponding values
*/
editRecord(record:Record): void;

/**
* 
*  Edit the record selected in the specified selection component (typically a
*  ListGrid).
*  
*  Updates the values of this editor to match the selected record's values.
*  
*  If this form has a dataSource, then saving via DynamicForm.saveData will use the 
*  "update" operation type.
* 
* 
* @param {ListGrid | TileGrid | GlobalId} the ListGrid or TileGrid or ID of a ListGrid/TileGrid whose currently   record(s) is/are to be edited
*/
editSelectedData(selectionComponent:ListGrid | TileGrid | GlobalId): void;

/**
* 
*  validateData() can be used to check for errors in server-side validators 
*  without showing such errors to the user. Errors, if any, can be discovered by looking at
*  the DSResponse object returned in the callback.
*  
*  validateData() will first call DynamicForm.validate to check for client-side
*  errors, and will return false without contacting the server if errors are
*  present. In this case, any errors detected client-side will be displayed; to avoid this
*  and purely perform silent, server-side validation, you can use 
*  DataSource.validateData with the form's DynamicForm.getValues.
*  DynamicForm.valuesAreValid can be used in lieu of a call to DynamicForm.validate if silent
*  checking of client-side errors is also desired.
* 
* 
* @param {DSCallback=} callback to invoke on completion
* @param {Partial<DSRequest>=} additional properties to set on                          the DSRequest that will be issued
*/
validateData(callback?:DSCallback, requestProperties?:Partial<DSRequest>): boolean;

/**
* 
*  Resets values to the state it was the last time setValues() or
*  rememberValues() was called. If neither of those methods has been called,
*  values will be set back to their initial values at init time.
* 
* 
*/
reset(): void;

/**
* 
*  This method exists for clean integration with existing server frameworks that have a
*  'cancel' feature which typically clears session state associated with the form. When
*  this method is called, an RPC is sent to the server. You must pass the appropriate
*  params property in the request properties as required by your server framework.
*  
*  For example:
*  
*  var requestProperties = {
*    params: {CANCEL: "cancel"}
*  };
*  
* 
*  Note that no other form data is sent. By default the current top-level page is replaced
*  with the reply. If you wish to ignore the server reply instead, include the following
*  request properties:
*  
*  var requestProperties = { 
*    // other props ...
*    ignoreTimeout: true,
*    target: null
*  };
*  
* 
* @param {DSRequest} additional properties to set on the RPCRequest                    that will be issued
* @deprecated * As of Smartclient version 13.0, due to the removal of Apache Struts.

*/
cancel(requestProperties:DSRequest): void;

/**
*  If the form or valuesManager has associated userTask workflow task than notify it about
*  cancelling the changes. 
* 
*/
cancelEditing(): void;

/**
*  Finish editing and store edited values in Process.state. 
* 
*/
completeEditing(): void;

/**
*  submit() is automatically called when a SubmitItem included in the
*  form is clicked, or, if DynamicForm.saveOnEnter is set, when the
*  "Enter" key is pressed in a text input. Submit can also be manually called.
*  
*  If this form is part of a ValuesManager, this method will simply fall through to
*  the submit method on the valuesManager. If not, and
*  DynamicForm.submitValues exists, it will be called, and
*  no further action will be taken.
*  
*  Otherwise, default behavior varies based on DynamicForm.canSubmit: if
*  canSubmit is false, DynamicForm.saveData will be called to
*  handle saving via SmartClient databinding. 
*  
*  If canSubmit is true, the form will be submitted like an ordinary HTML
*  form via DynamicForm.submitForm.
*  
*  The parameters to submit() apply only if submit() will be
*  calling DynamicForm.saveData. If you override submit(), you can safely
*  ignore the parameters as SmartClient framework code does not pass them.
*  
* 
* @param {DSCallback=} callback to invoke on completion.                     [Ignored if this.canSubmit is true]
* @param {DSRequest=} additional properties to set on the DSRequest                     that will be issued                      [Ignored if this.canSubmit is true]
*/
submit(callback?:DSCallback, requestProperties?:DSRequest): void;

/**
* 
*  Validate and then save the form's current values to the DataSource this form is
*  bound to.
*  
*  If client-side validators are defined, they are executed first, and if any errors are
*  found the save is aborted and the form will show the errors.
*  
*  If client-side validation passes, a DSRequest will be sent, exactly as though
*  DataSource.addData or DataSource.updateData had been called with 
*  ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','the form\'s values')} as data. The
*  DSRequest.operationType will be either "update" or "add", depending on the
*  current DynamicForm.saveOperationType.
*  
*  On either a client-side or server-side validation failure, validation errors will be
*  displayed in the form. Visible items within a DynamicForm will be redrawn to display
*  errors. Validation failure occurring on hidden items, or DataSource fields with no 
*  associated form items may be handled via DynamicForm.handleHiddenValidationErrors
*  or ValuesManager.handleHiddenValidationErrors.
*  
*  In the case of a validation error, the callback will not be called by default
*  since the form has already handled the failed save by displaying the validation errors
*  to the user. If you need to do something additional in this case, you can set
*  RPCRequest.willHandleError via the requestProperties parameter to
*  force your callback to be invoked. However, first consider:
*  
*  
* - if you are trying to customize display of validation errors, there are several
*  DynamicForm.showErrorIcons and DynamicForm.showErrors may be a better
*  place to put customizations.
*  
* - for unrecoverable general errors (eg server is down),
*  RPCManager.handleError in invoked, so consider placing
*  customizations there unless an unrecoverable error should be handled specially by this
*  specific form.
*  
*  Note: If a form is to be cleared after saving data, we recommend clearing the
*  form from the callback rather than calling saveData() and then synchronously clearing
*  the form. This gives users a chance to view and respond to any validation errors
*  returned by the server. It is also required to ensure forms containing an
*  FileItem, have a chance to upload the file to the server.
* 
* 
* @param {DSCallback=} callback to invoke on completion
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
*/
saveData(callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Returns true if DynamicForm.saveOperationType is currently "add". See
*  DynamicForm.saveOperationType.
* 
* 
*/
isNewRecord(): boolean;

/**
*  Setter for the default DSOperationType when DynamicForm.saveData is called.
*  Note that this property can also be set by calling DynamicForm.editRecord or 
*  DynamicForm.editNewRecord
* 
* 
* @param {DSOperationType} Operation type to use as a default. Valid values are "add" or "update".
*/
setSaveOperationType(operationType:DSOperationType): void;

/**
*  Returns the DSOperationType to be performed when DynamicForm.saveData
*  or ValuesManager.saveData is called.
*  Valid options are "add" or "update".
*  
*  If a DSRequest configuration object is passed in containing an explicit operationType
*  this will be returned. Otherwise DynamicForm.saveOperationType 
*  will be returned if set. Note that
*  saveOperationType is automatically set via calls to data binding methods such as
*  DynamicForm.editNewRecord, or it may be 
*  DynamicForm.setSaveOperationType.
*  
*  If no explicit saveOperationType is present, the system will use the following 
*  heuristic to determine the save operationType: 
*  
*  
* - If the form has no value for the DataSource.getPrimaryKeyField
*    this method will return "add". The assumption is that this is a new record, and the
*    field will be populated when the record is created, 
*    (as with a "sequence" type field).
*  
* - If, ${isc.DocUtils.linkForRef('method:DynamicForm.setValues','when the form\'s values were populated')}, 
*    the form had value for the DataSource.getPrimaryKeyField
*    but it has subsequently be changed, this method will return "add". In this case
*    the value has been changed, either by the user or programmatically so a different
*    (new) record is assumed. This is determined by looking at the
*    DynamicForm.getOldValues for the form.
*  
* - If the DataSource.getPrimaryKeyField is editable and
*    a value is now present for the primary key field, but was not present in the
*    DynamicForm.getOldValues for the form, this method will return
*    "add". In this case either no initial values were provided, or a 'sparse' 
*    set of values for a new record (with no primary key) were provided to the form
*    and the user has subsequently explicitly entered a new primaryKey field value.
*  
* - Otherwise this method will return "update". Either the primaryKey field is non
*    editable, or the user has not changed it from its initial value.
*  
* 
* 
* @param {Partial<DSRequest>=} Optional DSRequest config block for the save operation
*/
getSaveOperationType(requestProperties?:Partial<DSRequest>): DSOperationType;

/**
*  Retrieve data that matches the provided criteria, and edit the first record returned
*  
* 
* @param {Criteria=} search criteria
* @param {DSCallback=} callback to invoke on completion
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
*/
fetchData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Retrieve data that matches the provided criteria, and edit the first record returned.
*  Differs from DynamicForm.fetchData in that a case insensitive substring match
*  will be performed against the criteria to retrieve the data.
*  
* 
* @param {Criteria=} search criteria
* @param {DSCallback=} callback to invoke on completion
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
*/
filterData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Binds this dynamicForm to a DynamicForm.valuesManager at runtime.
* 
* @param {ValuesManager | GlobalId} the ValuesManager that controls this                         form's values
*/
setValuesManager(valuesManager:ValuesManager | GlobalId): void;

/**
* 
*  Displays the currently selected record(s) of the selectionComponent widget (typically a
*  listGrid) in this component.
*  
*  For a DynamicForm the first record of the selection is shown after the form is placed
*  into DynamicForm.canEdit.
*  A subsequent call to DynamicForm.editRecord or similar will return the
*  form to editability.
*  
*  Note that since field-level canEdit:true settings override the form-level
*  canEdit setting the automatic change to read-only may not change every field.
* 
* 
* @param {ListGrid | TileGrid | GlobalId} the ListGrid or TileGrid or ID of a ListGrid/TileGrid whose currently   selected record(s) is/are to be viewed
*/
viewSelectedData(selectionComponent:ListGrid | TileGrid | GlobalId): void;

/**
* 
*  Returns the form item type (Class Name) to be created for some field.
*  By default field.editorType will be used if present - otherwise backs off to
*  deriving the appropriate form item type from the data type of the field (see
*  FormItemType for details).
* 
* 
* @param {object} field definition for which we are deriving form item type.
* @param {object=} Current set of values being edited by this form. May be null.
*/
getEditorType(field:object, values?:object): string;

/**
*  Only supported on ListGrid for now.
* 
* @param {Array<Partial<Hilite>>} Array of hilite objects
*/
setHilites(hilites:Array<Partial<Hilite>>): void;

/**
*  Synonym for DynamicForm.setFields
* 
* 
* @param {Array<Partial<Partial<FormItem>>>} list of new items to show in the form
*/
setItems(itemList:Array<Partial<Partial<FormItem>>>): void;

/**
*  Set the DynamicForm.fields for this DynamicForm. Takes an array of item
*  definitions, which will be converted to FormItems and displayed in the form.
*  
*  
*  Note: Do not attempt to create FormItem instances directly. This method should be
*  passed the raw properties for each item only.
*  
*  
*  Objects passed to setFields() may not be reused in other forms and may not be
*  used in subsequent calls to setFields() with the same form, new objects must be
*  created instead.
*  
*  To create a form where some items are conditionally present, rather than repeated calls to
*  setFields() or setItems(), you should generally use
*  FormItem.hide and FormItem.show and/or FormItem.showIf rather than
*  calling setItems() or setFields(). setItems() and
*  setFields() are appropriate for dynamically generated forms where there are
*  few if any items that are the same each time the form is used.
* 
* 
* @param {Array<Partial<Partial<FormItem>>>} list of new items to show in the form
*/
setFields(itemList:Array<Partial<Partial<FormItem>>>): void;

/**
*  Method to retrieve the DynamicForm.fields for this DynamicForm. 
* 
* 
*/
getFields(): Array<Partial<FormItem>>;

/**
*  Method to retrieve the DynamicForm.fields for this DynamicForm. 
* 
* 
*/
getItems(): Array<Partial<FormItem>>;

/**
*  This method is called automatically by the DynamicForm when the set of items changes
*  and ensures that items show up in the correct tab order positions.
*  
*  Makes use of DynamicForm.sortItemsIntoTabOrder to order the items and ensures
*  the items are ordered in the TabIndexManager correctly.
* 
*/
assignItemsTabPositions(): void;

/**
*  Helper method to take our specified items and sort them into their desired
*  tab sequence
*  
*  Default behavior will respect explicitly specified tab index as a local tab
*  index, otherwise just use specified order within the items array
* 
*/
sortItemsIntoTabOrder(): Array<Partial<FormItem>>;

/**
*  Modify this form's TitleOrientation at runtime
* 
* @param {TitleOrientation} new default item titleOrientation
*/
setTitleOrientation(orientation:TitleOrientation): void;

/**
*  Replaces the current values of the entire form with the values passed in.
*  
*  Note: when working with a form that is saving to a DataSource, you would typically call
*  either DynamicForm.editRecord for an existing record, or DynamicForm.editNewRecord for a new
*  record. In addition to setting the current values of the form, these APIs establish the
*  DSRequest.operationType used to save ("update" vs "add").
*  
*  Values should be provided as an Object containing the new values as properties, where each
*  propertyName is the name of a Items in the form, and each property value is
*  the value to apply to that form item via FormItem.setValue. 
*  
*  Values with no corresponding form item may also be passed, will be tracked by the form
*  and returned by subsequent calls to DynamicForm.getValues.
*  
*  Any FormItem for which a value is not provided will revert to its
*  FormItem.defaultValue. To cause all FormItems to revert to default
*  values, pass null.
*  
*  This method also calls DynamicForm.rememberValues so that a subsequent later call to
*  DynamicForm.resetValues will revert to the passed values.
* 
* 
* @param {object=} values for the form, or null to reset all items to default values
*/
setValues(newData?:object): void;

/**
*  	Make a snapshot of the current set of values, so we can reset to them later.
*  	Creates a new object, then adds all non-method properties of values
*  	to the new object. Use resetValues() to revert to these values.
*      Note that this method is automatically called when the values for this form are
*      set programmatically via a call to DynamicForm.setValues.
* 
* 
*/
rememberValues(): object;

/**
* 
*  Same as DynamicForm.reset.
* 
* 
*/
resetValues(): void;

/**
*  Reset to default form values and clear errors
* 
*/
clearValues(): void;

/**
*  Compares the current set of values with the values stored by the call to the
*  DynamicForm.rememberValues method. rememberValues() runs when the
*  form is initialized and on every call to DynamicForm.setValues.
*  Returns true if the values have changed, and false otherwise.
* 
*/
valuesHaveChanged(): boolean;

/**
*  Returns the set of values last stored by DynamicForm.rememberValues.
*  Note that rememberValues() is called automatically by
*  DynamicForm.setValues, and on form initialization, so this typically contains
*  all values as they were before the user edited them.
* 
* 
*/
getOldValues(): object;

/**
*  Returns all values within this DynamicForm that have changed since 
*  DynamicForm.rememberValues last ran. Note that DynamicForm.rememberValues
*  runs on dynamicForm initialization, and with every call to DynamicForm.setValues
*  so this will typically contain all values the user has explicitly edited since then.
* 
*/
getChangedValues(): object;

/**
*  An Object containing the values of the form as properties, where each propertyName is
*  the name of a Items in the form, and each property value is the value
*  held by that form item.
*  
*  Note that modifying the returned object is not a supported way of adding or modifying values.
*  Instead use DynamicForm.setValue or DynamicForm.setValues.
* 
*/
getValues(): object;

/**
*  Based on the relationship between the DataSource this component is bound to and the
*  DataSource specified as the "schema" argument, call fetchData() to retrieve records in this
*  data set that are related to the passed-in record.
*  
*  Relationships between DataSources are declared via DataSourceField.foreignKey.
*  
*  For example, given two related DataSources "orders" and "orderItems", where we want to fetch
*  the "orderItems" that belong to a given "order". "orderItems" should declare a field that
*  is a DataSourceField.foreignKey to the "orders" table (for example, it
*  might be named "orderId" with foreignKey="orders.id"). Then, to load the records related to
*  a given "order", call fetchRelatedData() on the component bound to "orderItems", pass the
*  "orders" DataSource as the "schema" and pass a record from the "orders" DataSource as the
*  "record" argument.
* 
* 
* @param {ListGridRecord} DataSource record
* @param {Canvas | DataSource | GlobalId} schema of the DataSource record, or              DataBoundComponent already bound to that schema
* @param {DSCallback=} callback to invoke on completion
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
*/
fetchRelatedData(record:ListGridRecord, schema:Canvas | DataSource | GlobalId, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Return search criteria based on the current set of values within this form.
*  
*  The returned search criteria will be a simple Criteria object, except for
*  in the following cases, in which case an AdvancedCriteria object will be returned:
*  
*  
* - The advanced parameter may be passed to explicitly request a 
*  AdvancedCriteria object be returned
*  
* - If DynamicForm.setValuesAsCriteria was called with an AdvancedCriteria
*    object, this method will return advanced criteria.
*  
* - If DynamicForm.operator is set to "or" rather than 
*    "and" the generated criteria will always be advanced.
*  
* - If any item within this form returns true for FormItem.hasAdvancedCriteria,
*    which can be caused by setting FormItem.operator, and is always true for
*    items such as DateRangeItem
*  
* - If FormItem.allowExpressions is enabled
*  
*  The criteria returned will be picked up from the current values for this form. For simple
*  criteria, each form item simply maps its value to it's fieldName. See
*  FormItem.getCriterion
*  
*  for details on how form items generate advanced criteria.
*  Note that any values or criteria specified via DynamicForm.setValues or
*  DynamicForm.setValuesAsCriteria which do not correspond to an item within the form will be
*  combined with the live item values when criteria are generated.
*  
*  The returned criteria object can be used to filter data via methods such as
*  ListGrid.fetchData, DataSource.fetchData, or, for more advanced usage,
*  ResultSet.setCriteria.
*  
*  Note that any form field which the user has left blank is omitted as criteria, that is,
*  a blank field is assumed to mean "allow any value for this field" and not "this field must
*  be blank". Examples of empty values include a blank text field or SelectItem with an empty
*  selection.
* 
* 
* @param {boolean} if true, return an AdvancedCriteria object even if the  form item values could be represented in a simple Criterion object.
* @param {TextMatchStyle=} This parameter may be passed to indicate whether  the criteria are to be applied to a substring match (filter) or exact match (fetch).  When advanced criteria are returned this parameter will cause the appropriate  operator to be generated for individual fields' criterion clauses.
*/
getValuesAsCriteria(advanced:boolean, textMatchStyle?:TextMatchStyle): Criteria | AdvancedCriteria;

/**
*  This method will display the specified criteria in this form for editing. The criteria
*  parameter may be a simple Criterion object or an AdvancedCriteria object.
*  
*  For simple criteria, the specified fieldName will be used to apply criteria to form items,
*  as with a standard setValues() call.
*  
*  For AdvancedCriteria, behavior is as follows:
*  
*  
* - If the top level operator doesn't match the DynamicForm.operator for
*  this form, the entire criteria will be nested in an outer advanced criteria object with
*  the appropriate operator.
*  
* - Each criterion within AdvancedCriteria will be applied to a form item if
*  FormItem.shouldSaveValue is true for the item and 
*  FormItem.canEditCriterion returns true for the criterion in question. By default
*  this method checks for a match with both the fieldName and operator
*  of the criterion. The criterion is actually passed to the item for editing via
*  FormItem.setCriterion
*  .
*  Note that these methods may be overridden for custom
*  handling. Also note that the default CanvasItem.setCriterion implementation
*   handles editing nested criteria via embedded dynamicForms.
*  
* - Criteria which don't map to any form item will be stored directly on the form and
*  recombined with the edited values from each item when DynamicForm.getValuesAsCriteria is
*  called.
*  
* 
* @param {Criterion} criteria to edit.
*/
setValuesAsCriteria(criteria:Criterion): void;

/**
*  Return an AdvancedCriteria object based on the current set of values within this form.
*  
*  Similar to DynamicForm.getValuesAsCriteria, except the returned criteria object
*  is guaranteed to be an AdvancedCriteria object, even if none of the form's fields has a
*  specified FormItem.operator
* 
* 
* @param {TextMatchStyle=} If specified the text match style will be used to  generate the appropriate operator for per field criteria.
*/
getValuesAsAdvancedCriteria(textMatchStyle?:TextMatchStyle): AdvancedCriteria;

/**
*  Retrieve a FormItem in this form by it's FormItem.name,
*  FormItem.dataPath, or index within
*  the DynamicForm.items array.
*  
*  FormItems that also have a FormItem.ID may be accessed directly as a global variable
*  window[itemID] or just itemID
* 
* 
* @param {string | Integer} name of the item you're looking for
*/
getItem(itemName:string | Integer): FormItem;

/**
*  Synonym for dynamicForm.getItem()
* 
* 
* @param {FieldName} name of the item you're looking for
*/
getField(itemName:FieldName): FormItem;

/**
*  Returns the value stored in the form for some field.
* 
* @param {string} name of the field for which you're retrieving a value. Nested     values may be retrieved by passing in a DataPath
*/
getValue(fieldName:string): any;

/**
*   Sets the value for some field
* 
* @param {string} Name of the field being updated. A DataPath may              be passed to set nested values
* @param {string} New value.
*/
setValue(fieldName:string, value:string): void;

/**
*  Clears the value for some field via a call to FormItem.clearValue if appropriate.
*  If there is no item associated with the field name, the field will just be cleared within
*  our stored set of values.
* 
* @param {string} Name of the field being cleared. A DataPath may be used for clearing details of nested data structures.
*/
clearValue(fieldName:string): void;

/**
*  Show a form item via FormItem.show
* 
* @param {string} Name of the item to show
*/
showItem(itemName:string): void;

/**
*  Hide a form item via FormItem.hide
* 
* @param {string} Name of the item to show
*/
hideItem(itemName:string): void;

/**
*  Set the valueMap for a specified item
* 
* @param {string} itemName of the item upon which the valueMap should be set.
* @param {ValueMap} new valueMap for the field in question.
*/
setValueMap(itemName:string, valueMap:ValueMap): void;

/**
*  Returns any errors that are currently visible to the user for this form, without performing
*  validation.
* 
* 
*/
getErrors(): object;

/**
*  Returns any errors that are currently visible to the user for the specified field in this
*  form, without performing validation.
* 
* 
* @param {string} fieldName to check for errors
*/
getFieldErrors(fieldName:string): string | Array<Partial<string>>;

/**
*  Setter for validation errors on this form. Errors passed in should be a Javascript object
*  of the format
*  {fieldName1:errors, fieldName2:errors}
*  Where the errors value may be either a string (single error message) or an
*  array of strings (if multiple errors should be applied to the field in question).
* 
* @param {object} list of errors as an object with the field names as keys
* @param {boolean} If true redraw form to display errors now. Otherwise errors can be displayed by calling    DynamicForm.showErrors    Note: When the errors are shown,    DynamicForm.handleHiddenValidationErrors will    be fired for errors on hidden fields, or with no associated formItem.
*/
setErrors(errors:object, showErrors:boolean): void;

/**
*      Sets error message(s) for the specified itemName to the error string or array of
*      strings. You must call form.markForRedraw() to display the new error message(s).
*      Note: you can call this multiple times for an individual itemName
*   which will result in an array of errors being remembered.
* 
* 
* @param {string} name of the item to set
* @param {string | Array<any>} error message string or array of strings
* @deprecated * This method has been deprecated as of SmartClient release 5.7. Use DynamicForm.addFieldErrors or DynamicForm.setFieldErrors instead

*/
setError(itemName:string, errorMessage:string | Array<any>): void;

/**
*  Adds field validation error[s] to the specified field. Errors passed in will be added
*  to any existing errors on the field caused by validation or a previous call to this method.
*  
*  The errors parameter may be passed in as a string (a single error message), or an array of
*  strings.
*  The showErrors parameter allows the errors to be displayed immediately. Alternatively, call
*  DynamicForm.showFieldErrors to display the errors for this field.
* 
* @param {string} field to apply the new errors to
* @param {string | Array<Partial<string>>} errors to apply to the field in question
* @param {boolean} If true this method will fall through to DynamicForm.showFieldErrors to update the display
*/
addFieldErrors(fieldName:string, errors:string | Array<Partial<string>>, show:boolean): void;

/**
*  Set field validation error[s] for some field.
*  The errors parameter may be passed in as a string (a single error message), or an array of
*  strings.
*  The showErrors parameter allows the errors to be displayed immediately. Alternatively, an
*  explicit call to DynamicForm.showFieldErrors will display the errors for this field.
* 
* @param {string} field to apply the new errors to
* @param {string | Array<Partial<string>>} errors to apply to the field in question
* @param {boolean} If true this method will fall through to DynamicForm.showFieldErrors to update the display
*/
setFieldErrors(fieldName:string, errors:string | Array<Partial<string>>, show:boolean): void;

/**
*  Clear any validation errors on the field passed in.
* 
* @param {string} field to clear errors from
* @param {boolean} If true this method will fall through to DynamicForm.showFieldErrors to update the display
*/
clearFieldErrors(fieldName:string, show:boolean): void;

/**
* 	Clears all errors for this DynamicForm.
* 
* @param {boolean} If true, redraw the form to clear any visible error messages.
*/
clearErrors(show:boolean): void;

/**
*  Returns whether there are currently any errors visible to the user for this form, without
*  performing validation.
*  
*  Note that validation errors are set up automatically by validation (see DynamicForm.validate),
*  or may be explicitly set via DynamicForm.setErrors or 
*  DynamicForm.setFieldErrors.
* 
*/
hasErrors(): boolean;

/**
*  Returns whether there are currently any errors visible to the user for the specified field in
*  this form, without performing any validation.
*  
*  Note that validation errors are set up automatically by validation (see DynamicForm.validate),
*  or may be explicitly set via DynamicForm.setErrors or
*  DynamicForm.setFieldErrors.
* 
* @param {string} field to test for validation errors
*/
hasFieldErrors(fieldName:string): boolean;

/**
*  If DynamicForm.showInlineErrors is false, the form will render all errors in a list at
*  the top of the form. This method returns the HTML for this list of errors.
* 
* @param {object} Map of field names to error messages. Each field may contain a single            error message (string) or an array of errors
*/
getErrorsHTML(errors:object): HTMLString;

/**
*  If DynamicForm.showInlineErrors is true, this method is called for each item in the form
*  and returns the error HTML to be written out next to the item.
*  Default implementation falls through to FormItem.getErrorHTML on the item in question.
* 
* @param {FormItem} Form item for which the HTML should be retrieved
* @param {string | Array<any>} Error message to display for the item, or array of error message               strings.
*/
getItemErrorHTML(item:FormItem, error:string | Array<any>): void;

/**
*  Return the orientation of the title for a specific item or the default title orientation if
*  no item is passed.
*  
* 
* @param {FormItem=} item to check
*/
getTitleOrientation(item?:FormItem): TitleOrientation;

/**
*  Get the alignment for the title for some item. Default implementation is as follows:
*  
* - If FormItem.titleAlign is specified, it will be respected
*    
* - If not, and DynamicForm.titleAlign is set, it will be
*      respected
*    
* - Otherwise titles will be aligned according to Page.isRTL;
*      for DynamicForm.titleOrientation "top", this
*      method returns "left" if text direction is LTR, and
*      "right" if not; for horizontal orientations, this method returns 
*      "right" if text direction is LTR, or "left" if text 
*      direction is RTL.
*  
* 
* @param {FormItem} item for which we're getting title alignment
*/
getTitleAlign(item:FormItem): Alignment;

/**
*  Is the title for the given form item clipped? The form item must have title clipping enabled.
* 
* 
* @param {FormItem} the form item.
*/
titleClipped(item:FormItem): boolean;

/**
*  Submits the form to the URL defined by DynamicForm.action, 
*  identically to how a plain HTML &lt;form&gt; element would submit data,
*  as either an HTTP GET or POST as specified by DynamicForm.method.
*  
*  Notes:
*  
*  
* - this is used only in the very rare case that a form is used to submit data
*  directly to a URL. Normal server contact is through 
*  dataBoundComponentMethods.
*  
* - For this method to reliably include values for every field in the grid, 
*    DynamicForm.canSubmit must be set to true
*  
* - To submit values for fields that do not have an editor, use HiddenItem 
*  with a FormItem.defaultValue set. This is analogous to &lt;input type="hidden"&gt;
*  in HTML forms.
*  
* 
*/
submitForm(): void;

/**
*  Method called when an attempt to DynamicForm.submitForm a
*  form is unable to submit to the server. Default behavior is to display the
*  DynamicForm.formSubmitFailedWarning in a warning dialog.
*  The most common cause for this failure is that the user
*  has typed an invalid file-path into an upload type field.
*  
*  Note: This is very unlikely to occur with modern versions of IE, which don't allow the
*  path of a file to be edited by hand (only selected via file navigation). It was last seen
*  in IE6-7 under Windows XP.
*  
*  Rather than throwing an exception on the client during submit(),
*  normally all failures in native form submission are handled by the server. For further
*  information, see upload.
* 
* @deprecated * only known to be called in IE6-7, not supported by SmartClient 12+

*/
formSubmitFailed(): void;

/**
*  Sets the DynamicForm.action for this form.
* 
* @param {URL} New action URL
*/
setAction(action:URL): void;

/**
*  Sets the DynamicForm.target for this form.
* 
* @param {string} New submission target
*/
setTarget(target:string): void;

/**
*  Sets the DynamicForm.method for this form.
* 
* @param {FormMethod} html form submission method (get or post)
*/
setMethod(method:FormMethod): void;

/**
*  Validates the form without submitting it, and redraws the form to display error messages
*  if there are any validation errors. Returns true if validation succeeds, or false if
*  validation fails.
*  
*  For databound forms, any DataSource field validators will be run even if there is no
*  associated item in the form. Validators will also be run on hidden form items. In both
*  these cases, validation failure can be handled via 
*  DynamicForm.handleHiddenValidationErrors.
*  
*  If this form has any fields which require server-side validation (see 
*  Validator.serverCondition) this will also be initialized. Such validation will
*  occur asynchronously. Developers can use DynamicForm.isPendingAsyncValidation and
*  DynamicForm.handleAsyncValidationReply to detect and respond to asynchronous
*  validation.
*  
*  Note that for silent validation, DynamicForm.valuesAreValid (client-side) and 
*  DynamicForm.checkForValidationErrors (client and server-side) can be used instead.
* 
* 
* @param {boolean=} Should validators be processed for non-visible fields     such as dataSource fields with no associated item or fields with visibility set to     "hidden"?
*/
validate(validateHiddenFields?:boolean): boolean;

/**
*  Method to determine whether the current form values would pass validation.
*  This method operates client-side, without contacting the server, running validators on the
*  form's values and returning a value indicating whether validation was successful. 
*  
*  Unlike DynamicForm.validate this method will not store the errors on the DynamicForm
*  or display them to the user.
*  
*  Note that DynamicForm.checkForValidationErrors allows for checking for server-side errors, and
*  finding out what those errors are via a callback.
* 
* 
* @param {boolean} Should validators be processed for non-visible fields     such as dataSource fields with no associated item or fields with visibility set to     "hidden"?
* @param {boolean=} If unset, this method returns a simple boolean value indicating success or failure of validation. If this parameter is passed, this method will return an object mapping each field name to the errors(s) encountered on validation failure, or null if validation was successful.
*/
valuesAreValid(validateHiddenFields:boolean, returnErrors?:boolean): boolean | Map<any>;

/**
*  Performs silent validation of the current form values, like DynamicForm.valuesAreValid. In
*  contrast to DynamicForm.valuesAreValid, this method allows checking for server-side errors, and
*  finding out what the errors are. 
*  
*  The callback must be passed unless server-side validation is being skipped, and If passed,
*  it always fires, errors or not, firing synchronously if server validation is skipped.
* 
* 
* @param {ValidationStatusCallback} callback to invoke after validation is complete
* @param {boolean=} should validators be processed for non-visible fields     such as dataSource fields with no associated item or fields with visibility set to     "hidden"
* @param {boolean=} whether to skip doing server-side validation
*/
checkForValidationErrors(callback:(errorMap:Map<any>) => void, validateHiddenFields?:boolean, skipServerValidation?:boolean): Map<any>;

/**
*  Call DynamicForm.validate to check for validation errors. If no errors are found,
*  return the current values for this form, otherwise return null.
* 
*/
getValidatedValues(): object;

/**
*  If this form has any outstanding validation errors, show them now.
*  This method is called when the set of errors is changed by DynamicForm.setErrors or
*  DynamicForm.validate.
*  Default implementation will redraw the form to display error messages and call
*  DynamicForm.handleHiddenValidationErrors to
*  display errors with no visible field.
*  Note that this method may be overridden to perform custom display of validation errors. 
* 
*/
showErrors(): void;

/**
*  If this form has any outstanding validation errors for the field passed in, show them now.
*  Called when field errors are set directly via DynamicForm.setFieldErrors / 
*  DynamicForm.addFieldErrors / DynamicForm.clearFieldErrors.
*  Default implementation simply falls through to DynamicForm.showErrors.
* 
* @param {string} field to show errors for
*/
showFieldErrors(fieldName:string): void;

/**
*  Returns true if this DynamicForm has the keyboard focus.
*  
*  Unlike standard canvases, for a DynamicForm this method will return true when keyboard
*  focus is currently on one of the form's DynamicForm.items.
*  
*  Note that in some cases the items of a form may be written directly into a different
*  FormItem.containerWidget. In this case the dynamicForm containing the
*  items may not have been drawn on the page itself, but this method can still return true
*  if one of the items has focus.
* 
*/
isFocused(): boolean;

/**
*  Return the current focus item for this form. 
*  
*  This is the item which either currently has focus, or if focus is not
*  currently within this form, would be given focus on a call to
*  DynamicForm.focus. May return null if this form has never had focus,
*  in which case a call to form.focus() would put focus into the 
*  first focusable item within the the form.
*  
*  Note that if focus is currently in a sub-item of a DateItem or RadioGroupItem,
*  this method will return the parent item, not the sub-item.
* 
*/
getFocusItem(): FormItem;

/**
*  Give keyboard focus to this form. If this form has had focus before, focus will be
*  passed to the item which last had focus (see DynamicForm.getFocusItem) - 
*  otherwise focus will be passed to the first focusable item in the form.
*  
*  To put focus in a specific item, use DynamicForm.focusInItem instead.
* 
*/
focus(): void;

/**
*  Move the keyboard focus into a particular item.
* 
* @param {number | string | FormItem} Item (or reference to) item to focus in.
*/
focusInItem(itemName:number | string | FormItem): void;

/**
*  If the current mouse event occurred over an item in this dynamicForm, returns that item.
* 
*/
getEventItem(): FormItem;

/**
*  If the current mouse event occurred over an item, or the title of an item in this
*  dynamicForm, return details about where the event occurred.
* 
*/
getEventItemInfo(): FormItemEventInfo;

/**
*  Retrieves the HTML to display in a hover canvas when the user holds the mouse pointer over
*  some item. Return null to suppress the hover canvas altogether.
*  Default implementation returns the prompt for the item if defined.
*  Can be overridden via item.itemHoverHTML()
* 
* 
* @param {FormItem} Item the user is hovering over.
*/
itemHoverHTML(item:FormItem): void;

/**
*  Retrieves the HTML to display in a hover canvas when the user holds the mouse pointer over
*  some item's title. Return null to suppress the hover canvas altogether.
*  Default implementation returns the prompt for the item if defined. If no prompt is defined
*  and the item title is clipped, the item title will be shown in a hover by default.
*  Can be overridden by FormItem.titleHoverHTML.
* 
* 
* @param {FormItem} Item the user is hovering over.
*/
titleHoverHTML(item:FormItem): HTMLString;

/**
*  Retrieves the HTML to display in a hover canvas when the user holds the mousepointer over
*  some item's value. Return null to suppress the hover canvas altogether.
*  Can be overridden by FormItem.valueHoverHTML.
* 
* 
* @param {FormItem} Item the user is hovering over.
*/
valueHoverHTML(item:FormItem): void;

/**
*  Is this form editable or read-only? Setting the form to non-editable causes all
*  form items to render as read-only unless a form item is specifically marked as editable
*  (the item's FormItem.canEdit attribute is true).
* 
* 
* @param {boolean} Can this form be edited?
*/
setCanEdit(canEdit:boolean): void;

/**
*  Can the field be edited? This method looks at DynamicForm.canEdit for the grid as well as the
*  FormItem.canEdit value, to determine whether editing is actually allowed.
*  For a detailed discussion, see the documentation at DynamicForm.canEdit.
* 
* 
* @param {FormItem | number | string} field object or identifier
*/
fieldIsEditable(field:FormItem | number | string): boolean;

/**
*  Setter for the DynamicForm.readOnlyDisplay attribute.
* 
* @param {ReadOnlyDisplayAppearance} New read-only display appearance.
*/
setReadOnlyDisplay(appearance:ReadOnlyDisplayAppearance): void;

/**
*  Handler fired when the entire set of values is replaced, as by a call to 
*  DynamicForm.setValues, DynamicForm.resetValues or DynamicForm.editRecord.
*  
*  Note that it is invalid to call such methods from this handler because doing so would 
*  result in an infinite loop.
* 
* 
*/
valuesChanged(): void;

/**
*  Handler fired when there is a changed() event fired on a FormItem within this form.
*  
*  Fires after the change() handler on the FormItem itself, and only if the item did not
*  cancel the change event and chooses to allow it to propagate to the form as a whole. 
* 
* 
* @param {FormItem} the FormItem where the change event occurred
* @param {any} new value for the FormItem
*/
itemChanged(item:FormItem, newValue:any): void;

/**
*  Handler fired when there is a change() event fired on a FormItem within this form.
*  
*  Fires after the change() handler on the FormItem itself, and only if the item did not
*  cancel the change event and chooses to allow it to propagate to the form as a whole. 
* 
* 
* @param {FormItem} the FormItem where the change event occurred
* @param {any} new value for the FormItem
* @param {any} value the FormItem had previous to this change() event
*/
itemChange(item:FormItem, newValue:any, oldValue:any): boolean;

/**
*  Handler fired when a FormItem within this form receives a keypress event.
*  
*  Fires after the keyPress handler on the FormItem itself, and only if the item did not
*  cancel the event and chooses to allow it to propagate to the form as a whole. 
*  
* 
* @param {FormItem} the FormItem where the change event occurred
* @param {KeyName} name of the key that was pressed (EG: "A", "Space")
* @param {number} numeric character value of the pressed key.
*/
itemKeyPress(item:FormItem, keyName:KeyName, characterValue:number): boolean;

/**
*  Called when the mouse is right-clicked in some formItem. If the implementation
*  returns false, default browser behavior is cancelled.
*  
*  Note that it can be bad practice to cancel this method if the mouse is over the data 
*  element of an item, because doing so would replace the builtin browser-default menus 
*  that users may expect. You can use DynamicForm.getEventItemInfo to return an 
*  FormItemEventInfo that can be used to determine which part of the 
*  item is under the mouse.
* 
* 
* @param {FormItem} the form item showing its context menu
*/
showItemContextMenu(item:FormItem): boolean;

/**
*  Triggered when a SubmitItem is included in the form is submitted and gets pressed.
*  
* 
* @param {object} the form values
* @param {DynamicForm} the form being submitted
*/
submitValues(values:object, form:DynamicForm): void;

/**
*  Method to display validation error messages for fields that are not currently visible 
*  in this form.
*  This will be called when validation fails for
*  - a hidden field in this form
*  - if this form is databound, a datasource field with specified validators, for which we
*   have no specified form item.
*  Implement this to provide custom validation error handling for these fields.
*  By default hidden validation errors will be logged as warnings in the developerConsole.
*  Return false from this method to suppress that behavior.
* 
* @param {object} The set of errors returned - this is an object of the form           &nbsp;&nbsp;{fieldName:errors}            Where the 'errors' object is either a single string or an array            of strings containing the error messages for the field.
*/
handleHiddenValidationErrors(errors:object): boolean;



}
/**
* Creates a new DynamicForm
* 
* @param typeCheckedProperties {Partial<DynamicForm>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {DynamicForm} The new instance
*/
class DynamicForm{
static create(typeCheckedProperties? : Partial<DynamicForm>, uncheckedProperties?: Object): DynamicForm;
/**
* A declared value of the enum type 
*  FormMethod.
* @type {Constant}
* @default "GET"
*/
static GET?: Constant;

/**
* A declared value of the enum type 
*  FormMethod.
* @type {Constant}
* @default "POST"
*/
static POST?: Constant;

/**
* A declared value of the enum type 
*  Encoding.
* @type {Constant}
* @default "normal"
*/
static NORMAL?: Constant;

/**
* A declared value of the enum type 
*  Encoding.
* @type {Constant}
* @default "multipart"
*/
static MULTIPART?: Constant;








}

/**
* EdgedCanvas acts as a decorative, image-based frame around another single Canvas.
*/
interface EdgedCanvas extends Canvas {
/**
* Standard skin directory for edge images (sides and corners).
* @type {SCImgURL}
* @default "images/edges/"
*/
skinImgDir?: SCImgURL;

/**
* Base name of images for edges. Extensions for each corner or edge piece will be added
*  to this image URL, before the extension. For example, with the default base name of
*  "edge.gif", the top-left corner image will be "edge_TL.gif".
*  
*  The full list of extensions is: "_TL", "_TR", "_BL", "_BR", "_T", "_L", "_B", "_R",
*  "_center".
* @type {SCImgURL}
* @default "[SKIN]/rounded/frame/FFFFFF/6.png"
*/
edgeImage?: SCImgURL;

/**
* CSS color (WITHOUT "#") for the edges. If specified, will be used as part of image
*  names. Example: "edge_88FF88_TL.gif".
* @type {CSSColor}
* @default null
*/
edgeColor?: CSSColor;

/**
* Array of side names ("T", "B", "L", "R") specifying which sides of the decorated
*  component should show edges. For example: 
*    customEdges : ["T", "B"]
*  
*  .. would show edges only on the top and bottom of a component.
*  
*  The default of null means edges will be shown on all sides.
* @type {Array<Partial<string>>}
* @default null
*/
customEdges?: Array<Partial<string>>;

/**
* Whether to show media in the center section, that is, behind the decorated Canvas.
* @type {boolean}
* @default false
*/
showCenter?: boolean;

/**
* Size in pixels for corners and edges
* @type {Integer}
* @default "6"
*/
edgeSize?: Integer;

/**
* Height in pixels for left corners and edges. Defaults to edgeSize when unset.
* @type {Integer}
* @default null
*/
edgeLeft?: Integer;

/**
* Height in pixels for right corners and edges. Defaults to edgeSize when unset.
* @type {Integer}
* @default null
*/
edgeRight?: Integer;

/**
* Height in pixels for top corners and edges. Defaults to edgeSize when unset.
* @type {Integer}
* @default null
*/
edgeTop?: Integer;

/**
* Height in pixels for bottom corners and edges. Defaults to edgeSize when unset.
* @type {Integer}
* @default null
*/
edgeBottom?: Integer;

/**
* Amount the contained Canvas should be offset. Defaults to edgeSize; set to less than
*  edgeSize to allow the contained Canvas to overlap the edge and corner media.
* @type {Integer}
* @default null
*/
edgeOffset?: Integer;

/**
* Amount the contained Canvas should be offset from the top. Defaults to the size for 
*  the top edge. Set smaller to allow the contained Canvas to overlap the edge and 
*  corner media.
* @type {Integer}
* @default null
*/
edgeOffsetTop?: Integer;

/**
* Amount the contained Canvas should be offset from the bottom. Defaults to the size for
*  the bottom edge. Set smaller to allow the contained Canvas to overlap the edge and
*  corner media.
* @type {Integer}
* @default null
*/
edgeOffsetBottom?: Integer;

/**
* Amount the contained Canvas should be offset from the left. Defaults to the size for
*  the left edge. Set smaller to allow the contained Canvas to overlap the edge and
*  corner media.
* @type {Integer}
* @default null
*/
edgeOffsetLeft?: Integer;

/**
* Amount the contained Canvas should be offset from the right. Defaults to the size for
*  the right edge. Set smaller to allow the contained Canvas to overlap the edge and
*  corner media.
* @type {Integer}
* @default null
*/
edgeOffsetRight?: Integer;

/**
* Background color for the center section only. Can be used as a surrogate background
*  color for the decorated Canvas, if the Canvas is set to partially overlap the edges and
*  hence can't show a background color itself without occluding media.
* @type {CSSColor}
* @default null
*/
centerBackgroundColor?: CSSColor;

/**
* Optional property specifying the CSS ClassName to apply to the various parts of this edged canvas
*  (top, bottom, corners, sides and center). To
*  apply separate styles for each part, use EdgedCanvas.addEdgeStyleSuffix.
* @type {CSSStyleName}
* @default null
*/
edgeStyleName?: CSSStyleName;

/**
* If specified, the EdgedCanvas.edgeStyleName will be treated as a base style name and
*  appended with following suffixes to support separate styling per cell:
*  
*  _TL (top left cell)
*  _T (top center cell)
*  _TR (top right cell)
*  _L (middle left cell)
*  _C (center cell)
*  _R (middle right cell)
*  _BL (bottom left cell)
*  _B (bottom center cell)
*  _BR (bottom right cell)
* @type {boolean}
* @default false
*/
addEdgeStyleSuffix?: boolean;





}
/**
* Creates a new EdgedCanvas
* 
* @param typeCheckedProperties {Partial<EdgedCanvas>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {EdgedCanvas} The new instance
*/
class EdgedCanvas{
static create(typeCheckedProperties? : Partial<EdgedCanvas>, uncheckedProperties?: Object): EdgedCanvas;







}

/**
* An EditContext provides an editing environment for a set of components.
*  
*  An EditContext is typically populated by adding a series of EditNode created via a
*  Palette, either via drag and drop creation, or when loading from a saved version,
*  via EditContext.addFromPaletteNode or 
*  EditContext.addPaletteNodesFromXML.
*  
*  An EditContext then provides interfaces for further editing of the components represented
*  by EditNodes.
*  
*  Developers may explicitly define an edit context and initialize it with a
*  EditContext.rootComponent - the root of the user interface being created.
*  The EditContext itself is not visible to the user, but the root component's 
*  EditNode.liveObject may be.
*  As child editNodes are added to the rootComponent node or its descendants, liveObjects
*  in the user will update to reflect these changes. The live objects for the
*  edit nodes will be nested using the appropriate parent-child relationships, for 
*  the types of node in question. For example Canvases will be added as
*  Layout.members of layouts and FormItems will be added as 
*  DynamicForm.fields of DynamicForms.
*  
*  To enable drag and drop creation of widgets from a Palette, a developer can
*  use Canvas.setEditMode to enable editing behaviors on the live object of the
*  desired drop target (typically the root component).
*  To enable editNode creation via double-click on a Palette, developers can set
*  the Palette.defaultEditContext.
*  
*  Developers can also make use of EditPane or EditTree classes which provide
*  a visual interface for managing an EditContext.
*/
interface EditContext extends Class {
/**
* Root of data to edit. Must contain the "type" property, with the name of a
*  valid DataSource or nothing will be able to be dropped on this
*  EditContext. A "liveObject" property representing the rootComponent is also
*  suggested. Otherwise, a live object will be created from the palette node.
*  
*  Can be retrieved at any time. Use EditContext.getRootEditNode to retrieve the
*  EditNode created from the rootComponent.
* @type {PaletteNode}
* @default null
*/
rootComponent?: PaletteNode;

/**
* Palette to use when an EditNode is being created directly by this EditContext,
*  instead of being created due to a user interaction with a palette (eg dragging from
*  a TreePalette, or clicking on MenuPalette).
*  
*  If no defaultPalette is provided, the EditContext uses an automatically created
*  HiddenPalette.
* @type {Palette}
* @default null
*/
defaultPalette?: Palette;

/**
* Additional Palette to consult for metadata when
*  deserializing EditNode. Note that the
*  EditContext.defaultPalette is always consulted and need not be
*  provided again here.
* @type {Array<Partial<Palette>>}
* @default null
*/
extraPalettes?: Array<Partial<Palette>>;

/**
* When enabled, changes to a EditNode.liveObject's position
*  and size will be persisted to their EditNode by default.
*  This applies to both programmatic calls and user interaction (drag reposition
*  or drag resize).
*  
*  This feature can be disabled by either setting this property or
*  EditProxy.persistCoordinates to false. This
*  property affects all nodes within the EditContext whereas the latter
*  property affects children of a single node. 
*  
*  In some use-cases, like Reify, coordinates should not be
*  persisted except when a component explicitly enables this feature.
*  By setting this property to null no component will
*  persist coordinates of children unless
*  EditProxy.persistCoordinates is explicitly set to
*  true.
* @type {boolean}
* @default true
*/
persistCoordinates?: boolean;

/**
* Dropping a component near the edge of another component allows the component to be
*  dropped through an ancestor component. To suppress this action set
*  allowDropThrough to false.
* @type {boolean}
* @default null
*/
allowDropThrough?: boolean;

/**
* Controls whether components can be dropped into other components which support child
*  components. 
*  
*  When enabled, during a drop interaction in which a PaletteNode or EditNode
*  is the drop data, the componentSchema of the current
*  candidate drop target is inspected to see whether that parent allows children of the
*  type being dropped. If it does, the drop will result in a call to EditContext.addNode for
*  a paletteNode or for an existing EditNode in the same tree.
*  
*  Specific components can disable nested drops by explicitly setting EditProxy.allowNestedDrops
*  to false.
*  
*  This mode is enabled by default unless explicitly disabled by setting this property to
*  false.
* @type {boolean}
* @default null
*/
allowNestedDrops?: boolean;

/**
* New nodes added to the editContext are automatically placed
*  into edit mode if the new node's parent is in edit mode. To
*  suppress this action set autoEditNewNodes to false.
* @type {boolean}
* @default null
*/
autoEditNewNodes?: boolean;

/**
* The default parent EditNode to be used when a new
*  EditNode is added to the EditContext without a specified parent. This
*  commonly occurs when a paletteNode is double-clicked in a palette.
*  
*  If not specified, the root editNode (see EditContext.getRootEditNode) is used.
*  
*  Note: this property is automatically cleared if node is removed from the
*  editTree such as on calls to EditContext.destroyAll or EditContext.removeNode.
* @type {EditNode}
* @default null
*/
defaultParent?: EditNode;

/**
* If set, auto-enables EditProxy.useCopyPasteShortcuts on the EditProxy for the
*  EditContext.getRootEditNode. This works whether there is currently a root editNode
*  or one is added later.
* @type {boolean}
* @default null
*/
useCopyPasteShortcuts?: boolean;

/**
* Properties to apply to all EditProxy.editMasks created for components 
*  in edit mode. This mask can be modified when the node is selected by
*  EditContext.selectedBorder, EditContext.selectedTintColor and
*  EditContext.selectedTintOpacity depending on the EditContext.selectedAppearance
*  setting.
* @type {object}
* @default null
*/
editMaskProperties?: object;

/**
* Defines selection behavior when in edit mode. Only two styles are supported:
*  "single" and "multiple". Multiple enables hoop selection.
* @type {SelectionStyle}
* @default "multiple"
*/
selectionType?: SelectionStyle;

/**
* Should editNodes added to this EditContext be selectable? When true,
*  each EditProxy.canSelectChildren property is enabled unless explicitly set to
*  false. This allows an individual component to override this setting.
* @type {boolean}
* @default null
*/
canSelectEditNodes?: boolean;

/**
* Appearance that is applied to selected component.
*  
*  This value is applied as a default to EditProxy.selectedAppearance.
* @type {SelectedAppearance}
* @default null
*/
selectedAppearance?: SelectedAppearance;

/**
* Set the CSS border to be applied to the selection outline of the selected components.
*  This property is used when EditProxy.selectedAppearance is outlineMask
*  or outlineEdges.
*  
*  This value is applied as a default to EditProxy.selectedBorder.
* @type {string}
* @default "1px dashed #44ff44"
*/
selectedBorder?: string;

/**
* The background color for the selection outline label. The
*  default is defined on SelectionOutline.
*  
*  This value is applied as a default to EditProxy.selectedLabelBackgroundColor.
*  
*  NOTE: A selected component label is only supported when EditProxy.selectedAppearance
*  is "outlineEdges".
* @type {string}
* @default null
*/
selectedLabelBackgroundColor?: string;

/**
* Mask color applied to EditProxy.editMask of selected component when
*  EditProxy.selectedAppearance is "tintMask".
*  
*  This value is applied as a default to EditProxy.selectedTintColor.
* @type {CSSColor}
* @default "#cccccc"
*/
selectedTintColor?: CSSColor;

/**
* Opacity applied to EditProxy.editMask of selected component when
*  EditProxy.selectedAppearance is "tintMask".
*  
*  This value is applied as a default to EditProxy.selectedTintOpacity.
* @type {number}
* @default "25"
*/
selectedTintOpacity?: number;

/**
* Should the selection outline show a label for selected components? A component may
*  also be highlighted with the selection outline and label to indicate the target of
*  a drop. To suppress showing a label at any time set this property to false.
*  
*  To suppress labels during selection but still show them when targeted for a drop,
*  see EditContext.showSelectedLabelOnSelect.
*  
*  NOTE: A selected component label is only supported when EditProxy.selectedAppearance
*  is "outlineEdges".
* @type {boolean}
* @default null
*/
showSelectedLabel?: boolean;

/**
* Should the selection outline show a label when the component is selected? This property
*  is similar to EditContext.showSelectedLabel. Whereas
*  EditContext.showSelectedLabel controls whether a label is shown at
*  any time, this property allows normal selection to suppress the label but still show a label
*  during the drop process on the target. Leave EditContext.showSelectedLabel
*  unset and set this property to false.
*  
*  NOTE: A selected component label is only supported when EditProxy.selectedAppearance
*  is "outlineEdges".
* @type {boolean}
* @default null
*/
showSelectedLabelOnSelect?: boolean;

/**
* Should a group selection outline covering the outermost bounding boxes of all selected
*  components be shown in this container?
*  
*  Treated as true if not set and hoop selection is enabled (see
*  EditProxy.canSelectChildren and
*  EditContext.selectionType.
* @type {boolean}
* @default null
*/
canGroupSelect?: boolean;

/**
* Should the group selection box shown when EditContext.canGroupSelect
*  is true allow dragging the group as a whole?
*  
*  Treated as true if not set and EditContext.canGroupSelect
*  is true.
* @type {boolean}
* @default null
*/
canDragGroup?: boolean;

/**
* Should the group selection box shown when EditContext.canGroupSelect
*  is true be hidden during drag?
*  
*  Treated as true if not explicitly set to false.
* @type {boolean}
* @default null
*/
hideGroupBorderOnDrag?: boolean;

/**
* Defines the mode of inclusion for components encountered during hoop selection which
*  is enabled when EditContext.selectionType is multiple.
*  encloses mode causes selection of components that are completely
*  enclosed by the hoop. intersects mode selects components that come
*  into contact with the hoop.
* @type {HoopSelectionStyle}
* @default "encloses"
*/
hoopSelectionMode?: HoopSelectionStyle;

/**
* Properties to apply to EditProxy.hoopSelector.
* @type {object}
* @default null
*/
hoopSelectorProperties?: object;

/**
* Whether inline editing should be enabled for any components that are added and are placed into
*  editMode. Enabling this will turn on inline edit for any EditProxy where
*  EditProxy.supportsInlineEdit is true.
* @type {boolean}
* @default null
*/
enableInlineEdit?: boolean;



/**
* Palette to use when an EditNode is being created directly by this EditContext,
*  instead of being created due to a user interaction with a palette (eg dragging from
*  a TreePalette, or clicking on MenuPalette).
*  
*  If no defaultPalette is provided, the EditContext uses an automatically created
*  HiddenPalette.
*/
getDefaultPalette(): Palette;

/**
* Palette to use when an EditNode is being created directly by this EditContext,
*  instead of being created due to a user interaction with a palette (eg dragging from
*  a TreePalette, or clicking on MenuPalette).
*  
*  If no defaultPalette is provided, the EditContext uses an automatically created
*  HiddenPalette.
* @param {Palette} the default Palette
*/
setDefaultPalette(palette:Palette): void;

/**
*  Add a new EditNode to the EditContext, under the specified parent. If the parentNode
*  is not provided it will be determined from EditContext.defaultParent.
*  
*  The EditContext will interrogate the parent and new nodes to determine what field 
*  within the parent allows a child of this type, and to find a method to add the newNode's 
*  liveObject to the parentNode's liveObject. The new relationship will then be stored
*  in the tree of EditNodes.
*  
*  For example, when a Tab is dropped on a TabSet, the field TabSet.tabs is discovered as
*  the correct target field via naming conventions, and the method TabSet.addTab() is likewise 
*  discovered as the correct method to add a Tab to a TabSet.
* 
* 
* @param {EditNode} new node to be added
* @param {EditNode=} parent to add the new node under.
* @param {Integer=} index within the parent's children array
* @param {string=} the property of the liveParent to which the new node should                 be added, if not auto-discoverable from the schema
* @param {boolean=} whether to skip adding the liveObject to the liveParent                      (default false)
* @param {boolean=} whether to replace existing single field node                       if newNode liveObject is the same (default false)
*/
addNode(newNode:EditNode, parentNode?:EditNode, index?:Integer, parentProperty?:string, skipParentComponentAdd?:boolean, forceSingularFieldReplace?:boolean): EditNode;

/**
*  Returns the root EditNode of the EditContext typically created from EditContext.rootComponent.
* 
* 
*/
getRootEditNode(): EditNode;

/**
*  Notification fired when a different PaletteNode is substituted for one
*  being dropped into a container.
* 
* 
* @param {PaletteNode} node that was originally dropped
* @param {PaletteNode} node that was substituted
* @param {EditNode} parent node of the drop
*/
substitutedNode(origNode:PaletteNode, newNode:PaletteNode, parentNode:EditNode): void;

/**
*  Notification fired when an EditNode is converted to a different type
*  when moved from one container to another.
* 
* 
* @param {EditNode} node that was being moved
* @param {EditNode} node that was placed into new container
* @param {EditNode} parent node of the drop
*/
convertedNode(origNode:EditNode, newNode:EditNode, parentNode:EditNode): void;

/**
*  Notification fired when an EditNode has been added to the EditContext
* 
* 
* @param {EditNode} node that was added
* @param {EditNode} parent node of the node that was added
* @param {EditNode} root node of the edit context
*/
nodeAdded(newNode:EditNode, parentNode:EditNode, rootNode:EditNode): void;

/**
*  Notification fired when an EditNode has been removed from the EditContext
* 
* 
* @param {EditNode} node that was removed
* @param {EditNode} parent node of the node that was removed
* @param {EditNode} root node of the edit context
*/
nodeRemoved(removedNode:EditNode, parentNode:EditNode, rootNode:EditNode): void;

/**
*  Notification fired when an EditNode has been moved to a new position in the
*  component tree.
* 
* 
* @param {EditNode} node that was removed
* @param {EditNode} parent node of the node that was removed
* @param {EditNode} node that was added
* @param {EditNode} parent node of the node that was added
* @param {EditNode} root node of the edit context
*/
nodeMoved(oldNode:EditNode, oldParentNode:EditNode, newNode:EditNode, newParentNode:EditNode, rootNode:EditNode): void;

/**
*  Creates a new EditNode from a PaletteNode, using the
*  EditContext.defaultPalette. If you have an array of possibly inter-related
*  PaletteNodes, then you should use
*  EditContext.addFromPaletteNodes on the array instead,
*  in order to preserve the relationships.
* 
* 
* @param {PaletteNode} the palette node to use to create the new node
* @param {EditNode=} optional the parent node if the new node should appear                under a specific parent
*/
addFromPaletteNode(paletteNode:PaletteNode, parentNode?:EditNode): EditNode;

/**
*  Creates and returns an EditNode using the EditContext.defaultPalette. Does not add the newly
*  created EditNode to an EditContext.
*  
* 
* @param {PaletteNode} the palette node to use to create the new node
*/
makeEditNode(paletteNode:PaletteNode): EditNode;

/**
*  Gets the tree of editNodes being edited by this editContext. Standard tree
*  traversal methods can then be used to locate desired editNodes for interaction.
*  
*  Note: the returned tree is read-only and must only be modified by calling
*  methods on EditContext like EditContext.addNode or EditContext.setNodeProperties.
* 
* 
*/
getEditNodeTree(): Tree;

/**
*  Returns EditNode as an array that match the
*  specified type or types. By default the types are matched against the
*  EditNode.type or the general type of the component. By setting strict
*  to true the match is made against the editNode type exactly.
*  
*  For example, searching for "Canvas" nodes will return nodes for any component that
*  derives from Canvas unless strict is set. In the strict case, the search
*  will only return nodes for explict Canvas nodes.
* 
* 
* @param {Array<Partial<string>> | string} type or types of nodes to find
* @param {boolean=} true to match the EditNode.type exactly
*/
getEditNodesByType(types:Array<Partial<string>> | string, strict?:boolean): Array<Partial<EditNode>>;

/**
*  Does the EditNode have a DataSource assigned?
* 
* 
* @param {EditNode} editNode to check for a DataSource
*/
editNodeHasDataSource(editNode:EditNode): boolean;

/**
*  Does the EditNode have at least one field assigned?
*  
*  Note that if this method is called for a component editNode that could have child
*  components rather than fields, it will return true if there are any
*  child nodes other than a DataSource.
* 
* 
* @param {EditNode} editNode to check for fields
*/
editNodeHasFields(editNode:EditNode): boolean;

/**
*  Removes all EditNode from the EditContext, but does not destroy 
*  the EditNode.liveObject.
* 
*/
removeAll(): void;

/**
*  Removes all EditNode from the EditContext, and calls
*  Canvas.destroy on the
*  EditNode.liveObject.
* 
*/
destroyAll(): void;

/**
*  Removes EditNode from the EditContext. The editNode
*  liveObject is not destroyed.
* 
* @param {EditNode} node to be removed
*/
removeNode(editNode:EditNode): void;

/**
*  Serialize the tree of EditNode to an XML representation
*  of PaletteNode. The result can be supplied to 
*  EditContext.addPaletteNodesFromXML to recreate
*  the EditNodes.
* 
* 
* @param {SerializationSettings=} Additional serialization settings
*/
serializeAllEditNodes(settings?:SerializationSettings): string;

/**
*  Encode the tree of EditNode to a JSON representation
*  of PaletteNode. The result can be supplied to 
*  EditContext.addPaletteNodesFromJSON to recreate
*  the EditNodes.
* 
* 
* @param {SerializationSettings=} Additional serialization settings
*/
serializeAllEditNodesAsJSON(settings?:SerializationSettings): string;

/**
*  Serialize the provided EditNode to an XML
*  representation of PaletteNode. Note that the
*  EditNodes must have been added to this EditContext. The result can be
*  supplied to EditContext.addPaletteNodesFromXML to
*  recreate the EditNodes.
* 
* 
* @param {Array<Partial<EditNode>>} EditNodes to be serialized
* @param {SerializationSettings=} Additional serialization settings
*/
serializeEditNodes(nodes:Array<Partial<EditNode>>, settings?:SerializationSettings): string;

/**
*  Serialize the provided EditNode to a JSON
*  representation of PaletteNode. Note that the
*  EditNodes must have been added to this EditContext. The result can be
*  supplied to EditContext.addPaletteNodesFromJSON to
*  recreate the EditNodes.
* 
* 
* @param {Array<Partial<EditNode>>} EditNodes to be serialized
* @param {SerializationSettings=} Additional serialization settings
*/
serializeEditNodesAsJSON(nodes:Array<Partial<EditNode>>, settings?:SerializationSettings): string;

/**
*  Recreate EditNode from an XML representation of 
*  PaletteNode (possibly created by calling
*  EditContext.serializeAllEditNodes or EditContext.serializeEditNodes.
*  
*  By default, components that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
* 
* 
* @param {string} XML string
* @param {EditNode=} parent node (defaults to the root)
* @param {Array<Partial<string>>=} widgets to allow to take their global IDs
* @param {Function=} Callback to fire after nodes have been added
*/
addPaletteNodesFromXML(xmlString:string, parentNode?:EditNode, globals?:Array<Partial<string>>, callback?:Function): void;

/**
*  Recreate EditNode from a JSON representation of 
*  PaletteNode (possibly created by calling
*  EditContext.serializeAllEditNodesAsJSON or EditContext.serializeEditNodesAsJSON.
*  
*  By default, components that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
* 
* 
* @param {string} JSON string representing an array of PaletteNodes
* @param {EditNode=} parent to add to (defaults to the root)
* @param {Array<Partial<string>>=} widgets to allow to take their global IDs
* @param {Function=} Callback to fire after nodes have been added
*/
addPaletteNodesFromJSON(jsonString:string, parentNode?:EditNode, globals?:Array<Partial<string>>, callback?:Function): void;

/**
*  Obtain PaletteNode from an XML representation,
*  but do not add them to the EditContext.
* 
* 
* @param {string} XML string
* @param {PaletteNodeCallback} Callback used to return the PaletteNodes
*/
getPaletteNodesFromXML(xmlString:string, callback:(paletteNodes:Array<Partial<PaletteNode>>) => void): void;

/**
*  Add PaletteNode from a JavaScript source representation.
*  
*  By default, components that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
* 
* 
* @param {string} JavaScript code to eval.
* @param {EditNode=} parent node (defaults to the root)
* @param {Array<Partial<string>>=} widgets to allow to take their global IDs
*/
addPaletteNodesFromJS(jsCode:string, parentNode?:EditNode, globals?:Array<Partial<string>>): void;

/**
*  Obtain PaletteNode from a JavaScript source representation.
*  
*  By default, components that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
* 
* 
* @param {string} JavaScript code to eval.
* @param {PaletteNodeCallback} Callback used to return the PaletteNodes
* @param {Array<Partial<string>>=} widgets to allow to take their global IDs
*/
getPaletteNodesFromJS(jsCode:string, callback:(paletteNodes:Array<Partial<PaletteNode>>) => void, globals?:Array<Partial<string>>): void;

/**
*  Add the supplied PaletteNode to the parentNode, preserving internal
*  references from one supplied PaletteNode to another. This method should
*  be used with an array of possibly inter-related PaletteNodes (for
*  instance, those produced as a result of serialization via
*  EditContext.serializeAllEditNodes, rather than
*  calling EditContext.addFromPaletteNode on each
*  individual PaletteNode.
* 
* 
* @param {Array<Partial<PaletteNode>>} array of PaletteNodes
* @param {EditNode=} parent to add to (defaults to the root)
*/
addFromPaletteNodes(paletteNodes:Array<Partial<PaletteNode>>, parentNode?:EditNode): Array<Partial<EditNode>>;

/**
*  Returns true if editNode is in edit mode.
* 
* 
* @param {EditNode} the EditNode
*/
isNodeEditingOn(editNode:EditNode): boolean;

/**
*  Enable edit mode for an EditNode. This is a shortcut for calling
*  Canvas.setEditMode.
* 
* 
* @param {EditNode} the EditNode on which to enable editing
*/
enableEditing(editNode:EditNode): void;

/**
*  Returns the specified property from the editNode's serializable "defaults".
* 
* @param {EditNode} the editNode to query
* @param {string} the property name to query
*/
getNodeProperty(editNode:EditNode, name:string): void;

/**
*  Update an editNode's serializable "defaults" with the supplied properties. If you
*  wish to remove a property from the defaults (rather than setting it to null), then
*  use EditContext.removeNodeProperties instead.
* 
* @param {EditNode} the editNode to update
* @param {Partial<Canvas>} the properties to apply
* @param {boolean=} whether to skip updating the                     EditNode.liveObject,                     e.g. if you have already updated the liveObject
*/
setNodeProperties(editNode:EditNode, properties:Partial<Canvas>, skipLiveObjectUpdate?:boolean): void;

/**
*  Removes the specified properties from an editNode's serializable "defaults".
*  Note that the EditNode.liveObject is not updated by this method. 
*  To set a property to null (rather than removing it), use
*  EditContext.setNodeProperties instead.
* 
* @param {EditNode} the editNode to update
* @param {Array<Partial<string>>} an array of property names to remove
*/
removeNodeProperties(editNode:EditNode, properties:Array<Partial<string>>): void;

/**
*  Update an editNode's EditProxy properties. If editProxy has not yet
*  been created, editProxyProperties is updated or created instead.
* 
* 
* @param {EditNode} the editNode to update
* @param {Partial<EditProxy>} the properties to apply
*/
setEditProxyProperties(editNode:EditNode, properties:Partial<EditProxy>): void;

/**
*  Creates a PaletteNode from an EditNode in this context's
*  EditContext.getEditNodeTree.
*  
*  This essentially creates a new PaletteNode with the EditNode.defaults from the
*  passed editNode. The returned paletteNode could then be used with
*  EditContext.addFromPaletteNode to effectively create a copy of the original editNode -
*  specifically a new editNode with a new EditNode.liveObject created from the same
*  defaults.
*  
*  However note that makePaletteNode() does not copy descendant nodes - use
*  EditContext.makePaletteNodeTree for that.
*  
*  May return null if the passed editNode cannot validly by transformed into a paletteNode, for
*  example if EditNode.canDuplicate was set false.
* 
* 
* @param {EditNode} the editNode to use to make a paletteNode
*/
makePaletteNode(editNode:EditNode): PaletteNode;

/**
*  Creates a Tree of PaletteNode from an EditNode in this
*  context's EditContext.getEditNodeTree, by using EditContext.makePaletteNode on the
*  passed EditNode and its descendents within the
*  EditContext.getEditNodeTree.
*  
*  The root node of the returned Tree will be a PaletteNode derived from the passed
*  EditNode.
* 
* 
* @param {EditNode} root editNode to make Tree of PaletteNodes from
*/
makePaletteNodeTree(editNode:EditNode): Tree;

/**
*  Copies the passed editNode or editNodes to an internal "clipboard" space, for later application
*  via EditContext.pasteEditNodes.
* 
* @param {EditNode | Array<Partial<EditNode>>} */
copyEditNodes(editNode:EditNode | Array<Partial<EditNode>>): void;

/**
*  "Pastes" editNodes previously captured via EditContext.copyEditNodes.
*  
*  New editNodes will be added as root-level nodes of the EditContext.getEditNodeTree
*  unless a targetEditNode is passed.
* 
* @param {EditNode=} */
pasteEditNodes(targetEditNode?:EditNode): void;

/**
*  Executed when the left mouse is clicked (pressed and then released) on any selectable
*  component with EditProxy.editMask enabled.
*  implementation.
* 
* 
* @param {EditNode} the editNode clicked
* @param {object} the object clicked
*/
editMaskClicked(editNode:EditNode, liveObject:object): void;

/**
*  Overridable method to provide a custom selection outline label. This method
*  is called when a label is to be shown with an outline.
*  
*  The default implementation returns the same description shown in the edit tree.
* 
* 
* @param {object} the Canvas or FormItem component to label
*/
getSelectedLabelText(component:object): HTMLString;

/**
*  Returns all selected EditNodes as an Array.
* 
* 
*/
getSelectedEditNodes(): Array<Partial<EditNode>>;

/**
*  Returns selected EditNode or first selected EditNode if multiple
*  nodes are selected.
* 
* 
*/
getSelectedEditNode(): EditNode;

/**
*  Returns true if the editNode is selected. 
* 
* 
*/
isEditNodeSelected(): boolean;

/**
*  Select an EditNode.
* 
* 
* @param {EditNode} editNode to select
*/
selectEditNode(editNode:EditNode): void;

/**
*  Select a single EditNode and deselect everything else.
* 
* 
* @param {EditNode} editNode to select
*/
selectSingleEditNode(editNode:EditNode): void;

/**
*  Select all EditNodes.
* 
* 
*/
selectAllEditNodes(): void;

/**
*  Deselect a list of EditNodes.
* 
* 
* @param {Array<Partial<EditNode>>} editNodes to deselect
*/
deselectEditNodes(editNodes:Array<Partial<EditNode>>): void;

/**
*  Deselect all EditNodes.
* 
* 
*/
deselectAllEditNodes(): void;

/**
*  Called when editMode selection changes. Note this method fires exactly once for any given
*  change.
*  
*  This event is fired once after selection/deselection has completed. The result is
*  one event per mouse-down event. For a drag selection there will be one event fired
*  when the range is completed.
* 
* 
* @param {EditNode} first selected node, if any
* @param {Array<Partial<EditNode>>} List of nodes that are now selected
*/
selectedEditNodesUpdated(editNode:EditNode, editNodeList:Array<Partial<EditNode>>): void;

/**
*  Fires whenever editNode.defaults are modified by setNodeProperties() and/or editProxy
*  features
* 
* @param {EditNode} currently editing node
* @param {EditContext} current context
* @param {Array<Partial<string>>} properties that were modified
*/
editNodeUpdated(editNode:EditNode, editContext:EditContext, modifiedProperties:Array<Partial<string>>): void;

/**
*  Notification method fired when an inline title or value editor is shown or closed
*  for a component in the designer pane.
* 
* @param {FormItem} the field within the inline editor when showing.             null if the editor is closed.
* @param {string} the type of editor showing: "title" or "value"
*/
inlineEditorShowing(field:FormItem, type:string): void;



}
/**
* Creates a new EditContext
* 
* @param typeCheckedProperties {Partial<EditContext>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {EditContext} The new instance
*/
class EditContext{
static create(typeCheckedProperties? : Partial<EditContext>, uncheckedProperties?: Object): EditContext;
/**
* The number of pixels to offset a pasted node from the node being copied
* @type {Integer}
* @default "5"
*/
static editNodePasteOffset?: Integer;








}

/**
* A container that allows drag and drop instantiation of visual components from a
*  Palette, and direct manipulation of the position and size of those components.
*  
*  Any drag onto an EditPane from a Palette will add an EditNode created from the dragged
*  PaletteNode.
*  
*  EditPane automatically creates an EditContext and provides several APIs and
*  settings that are passthroughs to the underlying EditContext for convenience.
*/
interface EditPane extends Canvas {
/**
* An EditContext is automatically created to manage EditMode behavior. The public
*  EditContext APIs exposed by the EditPane are passed through to this object.
*  
*  Additional editContext properties can be supplied as
*  EditPane.editContextProperties.
* @type {EditContext}
* @default null
*/
editContext?: EditContext;

/**
* Properties to be applied to the EditPane.editContext when created.
* @type {Partial<EditContext>}
* @default null
*/
editContextProperties?: Partial<EditContext>;

/**
* New nodes added to the editContext are automatically placed
*  into edit mode if the new node's parent is in edit mode. To
*  suppress this action set autoEditNewNodes to false.
* @type {boolean}
* @default null
*/
autoEditNewNodes?: boolean;

/**
* Root of data to edit. Must contain the "type" property, with the name of a
*  valid DataSource or nothing will be able to be dropped on this
*  EditContext. A "liveObject" property representing the rootComponent is also
*  suggested. Otherwise, a live object will be created from the palette node.
*  
*  Can be retrieved at any time. Use EditPane.getRootEditNode to retrieve the
*  EditNode created from the rootComponent.
* @type {PaletteNode}
* @default null
*/
rootComponent?: PaletteNode;

/**
* Palette to use when an EditNode is being created directly by this EditContext,
*  instead of being created due to a user interaction with a palette (eg dragging from
*  a TreePalette, or clicking on MenuPalette).
*  
*  If no defaultPalette is provided, the EditContext uses an automatically created
*  HiddenPalette.
* @type {Palette}
* @default null
*/
defaultPalette?: Palette;

/**
* Additional Palette to consult for metadata when
*  deserializing EditNode. Note that the
*  EditPane.defaultPalette is always consulted and need not be
*  provided again here.
* @type {Array<Partial<Palette>>}
* @default null
*/
extraPalettes?: Array<Partial<Palette>>;

/**
* When enabled, changes to a EditNode.liveObject's position
*  and size will be persisted to their EditNode by default.
*  This applies to both programmatic calls and user interaction (drag reposition
*  or drag resize).
*  
*  This feature can be disabled by either setting this property or
*  EditProxy.persistCoordinates to false. This
*  property affects all nodes within the EditContext whereas the latter
*  property affects children of a single node. 
*  
*  In some use-cases, like Reify, coordinates should not be
*  persisted except when a component explicitly enables this feature.
*  By setting this property to null no component will
*  persist coordinates of children unless
*  EditProxy.persistCoordinates is explicitly set to
*  true.
* @type {boolean}
* @default true
*/
persistCoordinates?: boolean;

/**
* Controls whether components can be dropped into other components which support child
*  components. 
*  
*  When enabled, during a drop interaction in which a PaletteNode or EditNode
*  is the drop data, the componentSchema of the current
*  candidate drop target is inspected to see whether that parent allows children of the
*  type being dropped. If it does, the drop will result in a call to EditPane.addNode for
*  a paletteNode or for an existing EditNode in the same tree.
*  
*  Specific components can disable nested drops by explicitly setting EditProxy.allowNestedDrops
*  to false.
*  
*  This mode is enabled by default unless explicitly disabled by setting this property to
*  false.
* @type {boolean}
* @default null
*/
allowNestedDrops?: boolean;

/**
* Should the selection outline show a label for selected components? A component may
*  also be highlighted with the selection outline and label to indicate the target of
*  a drop. To suppress showing a label at any time set this property to false.
*  
*  To suppress labels during selection but still show them when targeted for a drop,
*  see EditContext.showSelectedLabelOnSelect.
*  
*  NOTE: A selected component label is only supported when EditProxy.selectedAppearance
*  is "outlineEdges".
* @type {boolean}
* @default null
*/
showSelectedLabel?: boolean;

/**
* Set the CSS border to be applied to the selection outline of the selected components.
*  This property is used when EditProxy.selectedAppearance is outlineMask
*  or outlineEdges.
*  
*  This value is applied as a default to EditProxy.selectedBorder.
* @type {string}
* @default null
*/
selectedBorder?: string;

/**
* The background color for the selection outline label. The
*  default is defined on SelectionOutline.
*  
*  This value is applied as a default to EditProxy.selectedLabelBackgroundColor.
*  
*  NOTE: A selected component label is only supported when EditProxy.selectedAppearance
*  is "outlineEdges".
* @type {string}
* @default null
*/
selectedLabelBackgroundColor?: string;

/**
* Should a group selection outline covering the outermost bounding boxes of all selected
*  components be shown in this container?
*  
*  Treated as true if not set and hoop selection is enabled (see
*  EditProxy.canSelectChildren and
*  EditContext.selectionType.
* @type {boolean}
* @default null
*/
canGroupSelect?: boolean;

/**
* Should the group selection box shown when EditContext.canGroupSelect
*  is true allow dragging the group as a whole?
*  
*  Treated as true if not set and EditContext.canGroupSelect
*  is true.
* @type {boolean}
* @default null
*/
canDragGroup?: boolean;

/**
* Should the group selection box shown when EditContext.canGroupSelect
*  is true be hidden during drag?
*  
*  Treated as true if not explicitly set to false.
* @type {boolean}
* @default null
*/
hideGroupBorderOnDrag?: boolean;

/**
* If set, auto-enables EditProxy.useCopyPasteShortcuts on the EditProxy for the
*  EditPane.getRootEditNode. This works whether there is currently a root editNode
*  or one is added later.
* @type {boolean}
* @default null
*/
useCopyPasteShortcuts?: boolean;



/**
*  Returns the EditContext instance managed by the EditPane.
* 
*/
getEditContext(): EditContext;

/**
*  Returns an Array of PaletteNodes representing all current EditNodes in this
*  pane, suitable for saving and restoring via passing each paletteNode to EditContext.addNode.
* 
*/
getSaveData(): Array<Partial<PaletteNode>>;

/**
*  Returns the root EditNode of the EditContext typically created from EditPane.rootComponent.
* 
* 
*/
getRootEditNode(): EditNode;

/**
*  Creates and returns an EditNode using the EditPane.defaultPalette. Does not add the newly
*  created EditNode to an EditContext.
*  
* 
* @param {PaletteNode} the palette node to use to create the new node
*/
makeEditNode(paletteNode:PaletteNode): EditNode;

/**
*  Gets the tree of editNodes being edited by this editContext. Standard tree
*  traversal methods can then be used to locate desired editNodes for interaction.
*  
*  Note: the returned tree is read-only and must only be modified by calling
*  methods on EditContext like EditContext.addNode or EditContext.setNodeProperties.
* 
* 
*/
getEditNodeTree(): Tree;

/**
*  Add a new EditNode to the EditContext, under the specified parent. If the parentNode
*  is not provided it will be determined from EditContext.defaultParent.
*  
*  The EditContext will interrogate the parent and new nodes to determine what field 
*  within the parent allows a child of this type, and to find a method to add the newNode's 
*  liveObject to the parentNode's liveObject. The new relationship will then be stored
*  in the tree of EditNodes.
*  
*  For example, when a Tab is dropped on a TabSet, the field TabSet.tabs is discovered as
*  the correct target field via naming conventions, and the method TabSet.addTab() is likewise 
*  discovered as the correct method to add a Tab to a TabSet.
* 
* 
* @param {EditNode} new node to be added
* @param {EditNode=} parent to add the new node under.
* @param {Integer=} index within the parent's children array
* @param {string=} the property of the liveParent to which the new node should                 be added, if not auto-discoverable from the schema
* @param {boolean=} whether to skip adding the liveObject to the liveParent                      (default false)
* @param {boolean=} whether to replace existing single field node                       if newNode liveObject is the same (default false)
*/
addNode(newNode:EditNode, parentNode?:EditNode, index?:Integer, parentProperty?:string, skipParentComponentAdd?:boolean, forceSingularFieldReplace?:boolean): EditNode;

/**
*  Creates a new EditNode from a PaletteNode, using the
*  EditPane.defaultPalette. If you have an array of possibly inter-related
*  PaletteNodes, then you should use
*  EditPane.addFromPaletteNodes on the array instead,
*  in order to preserve the relationships.
* 
* 
* @param {PaletteNode} the palette node to use to create the new node
* @param {EditNode=} optional the parent node if the new node should appear                under a specific parent
*/
addFromPaletteNode(paletteNode:PaletteNode, parentNode?:EditNode): EditNode;

/**
*  Add the supplied PaletteNode to the parentNode, preserving internal
*  references from one supplied PaletteNode to another. This method should
*  be used with an array of possibly inter-related PaletteNodes (for
*  instance, those produced as a result of serialization via
*  EditPane.serializeAllEditNodes, rather than
*  calling EditPane.addFromPaletteNode on each
*  individual PaletteNode.
* 
* 
* @param {Array<Partial<PaletteNode>>} array of PaletteNodes
* @param {EditNode=} parent to add to (defaults to the root)
*/
addFromPaletteNodes(paletteNodes:Array<Partial<PaletteNode>>, parentNode?:EditNode): Array<Partial<EditNode>>;

/**
*  Removes EditNode from the EditContext. The editNode
*  liveObject is not destroyed.
* 
* @param {EditNode} node to be removed
*/
removeNode(editNode:EditNode): void;

/**
*  Moves an EditNode from one child index to another in the EditContext under the specified parent.
*  
*  No changes are made to the live objects.
* 
* 
* @param {EditNode} parent to reorder child nodes
* @param {Integer} index within the parent's children array to be moved
* @param {Integer} index within the parent's children array at which to place moved node
*/
reorderNode(parentNode:EditNode, index:Integer, moveToIndex:Integer): void;

/**
*  Removes all EditNode from the EditContext, but does not destroy 
*  the EditNode.liveObject.
* 
*/
removeAll(): void;

/**
*  Removes all EditNode from the EditContext, and calls
*  Canvas.destroy on the
*  EditNode.liveObject.
* 
*/
destroyAll(): void;

/**
*  Returns true if editNode is in edit mode.
* 
* 
* @param {EditNode} the EditNode
*/
isNodeEditingOn(editNode:EditNode): boolean;

/**
*  Enable edit mode for an EditNode. This is a shortcut for calling
*  Canvas.setEditMode.
* 
* 
* @param {EditNode} the EditNode on which to enable editing
*/
enableEditing(editNode:EditNode): void;

/**
*  Returns the specified property from the editNode's serializable "defaults".
* 
* @param {EditNode} the editNode to query
* @param {string} the property name to query
*/
getNodeProperty(editNode:EditNode, name:string): void;

/**
*  Update an editNode's serializable "defaults" with the supplied properties. If you
*  wish to remove a property from the defaults (rather than setting it to null), then
*  use EditPane.removeNodeProperties instead.
* 
* @param {EditNode} the editNode to update
* @param {Partial<Canvas>} the properties to apply
* @param {boolean=} whether to skip updating the                     EditNode.liveObject,                     e.g. if you have already updated the liveObject
*/
setNodeProperties(editNode:EditNode, properties:Partial<Canvas>, skipLiveObjectUpdate?:boolean): void;

/**
*  Removes the specified properties from an editNode's serializable "defaults".
*  Note that the EditNode.liveObject is not updated by this method. 
*  To set a property to null (rather than removing it), use
*  EditPane.setNodeProperties instead.
* 
* @param {EditNode} the editNode to update
* @param {Array<Partial<string>>} an array of property names to remove
*/
removeNodeProperties(editNode:EditNode, properties:Array<Partial<string>>): void;

/**
* Palette to use when an EditNode is being created directly by this EditContext,
*  instead of being created due to a user interaction with a palette (eg dragging from
*  a TreePalette, or clicking on MenuPalette).
*  
*  If no defaultPalette is provided, the EditContext uses an automatically created
*  HiddenPalette.
*/
getDefaultPalette(): Palette;

/**
* Palette to use when an EditNode is being created directly by this EditContext,
*  instead of being created due to a user interaction with a palette (eg dragging from
*  a TreePalette, or clicking on MenuPalette).
*  
*  If no defaultPalette is provided, the EditContext uses an automatically created
*  HiddenPalette.
* @param {Palette} the default Palette
*/
setDefaultPalette(palette:Palette): void;

/**
*  Creates a PaletteNode from an EditNode in this context's
*  EditPane.getEditNodeTree.
*  
*  This essentially creates a new PaletteNode with the EditNode.defaults from the
*  passed editNode. The returned paletteNode could then be used with
*  EditContext.addFromPaletteNode to effectively create a copy of the original editNode -
*  specifically a new editNode with a new EditNode.liveObject created from the same
*  defaults.
*  
*  However note that makePaletteNode() does not copy descendant nodes - use
*  EditPane.makePaletteNodeTree for that.
*  
*  May return null if the passed editNode cannot validly by transformed into a paletteNode, for
*  example if EditNode.canDuplicate was set false.
* 
* 
* @param {EditNode} the editNode to use to make a paletteNode
*/
makePaletteNode(editNode:EditNode): PaletteNode;

/**
*  Creates a Tree of PaletteNode from an EditNode in this
*  context's EditPane.getEditNodeTree, by using EditPane.makePaletteNode on the
*  passed EditNode and its descendents within the
*  EditContext.getEditNodeTree.
*  
*  The root node of the returned Tree will be a PaletteNode derived from the passed
*  EditNode.
* 
* 
* @param {EditNode} root editNode to make Tree of PaletteNodes from
*/
makePaletteNodeTree(editNode:EditNode): Tree;

/**
*  Copies the passed editNode or editNodes to an internal "clipboard" space, for later application
*  via EditPane.pasteEditNodes.
* 
* @param {EditNode | Array<Partial<EditNode>>} */
copyEditNodes(editNode:EditNode | Array<Partial<EditNode>>): void;

/**
*  "Pastes" editNodes previously captured via EditPane.copyEditNodes.
*  
*  New editNodes will be added as root-level nodes of the EditPane.getEditNodeTree
*  unless a targetEditNode is passed.
* 
* @param {EditNode=} */
pasteEditNodes(targetEditNode?:EditNode): void;

/**
*  Recreate EditNode from an XML representation of 
*  PaletteNode (possibly created by calling
*  EditPane.serializeAllEditNodes or EditPane.serializeEditNodes.
*  
*  By default, components that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
* 
* 
* @param {string} XML string
* @param {EditNode=} parent node (defaults to the root)
* @param {Array<Partial<string>>=} widgets to allow to take their global IDs
* @param {Function=} Callback to fire after nodes have been added
*/
addPaletteNodesFromXML(xmlString:string, parentNode?:EditNode, globals?:Array<Partial<string>>, callback?:Function): void;

/**
*  Recreate EditNode from a JSON representation of 
*  PaletteNode (possibly created by calling
*  EditPane.serializeAllEditNodesAsJSON or EditPane.serializeEditNodesAsJSON.
*  
*  By default, components that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
* 
* 
* @param {string} JSON string representing an array of PaletteNodes
* @param {EditNode=} parent to add to (defaults to the root)
* @param {Array<Partial<string>>=} widgets to allow to take their global IDs
* @param {Function=} Callback to fire after nodes have been added
*/
addPaletteNodesFromJSON(jsonString:string, parentNode?:EditNode, globals?:Array<Partial<string>>, callback?:Function): void;

/**
*  Obtain PaletteNode from an XML representation,
*  but do not add them to the EditContext.
* 
* 
* @param {string} XML string
* @param {PaletteNodeCallback} Callback used to return the PaletteNodes
*/
getPaletteNodesFromXML(xmlString:string, callback:(paletteNodes:Array<Partial<PaletteNode>>) => void): void;

/**
*  Obtain PaletteNode from a JavaScript source representation.
*  
*  By default, components that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
* 
* 
* @param {string} JavaScript code to eval.
* @param {PaletteNodeCallback} Callback used to return the PaletteNodes
* @param {Array<Partial<string>>=} widgets to allow to take their global IDs
*/
getPaletteNodesFromJS(jsCode:string, callback:(paletteNodes:Array<Partial<PaletteNode>>) => void, globals?:Array<Partial<string>>): void;

/**
*  Add PaletteNode from a JavaScript source representation.
*  
*  By default, components that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
* 
* 
* @param {string} JavaScript code to eval.
* @param {EditNode=} parent node (defaults to the root)
* @param {Array<Partial<string>>=} widgets to allow to take their global IDs
*/
addPaletteNodesFromJS(jsCode:string, parentNode?:EditNode, globals?:Array<Partial<string>>): void;

/**
*  Serialize the tree of EditNode to an XML representation
*  of PaletteNode. The result can be supplied to 
*  EditPane.addPaletteNodesFromXML to recreate
*  the EditNodes.
* 
* 
* @param {SerializationSettings=} Additional serialization settings
*/
serializeAllEditNodes(settings?:SerializationSettings): string;

/**
*  Encode the tree of EditNode to a JSON representation
*  of PaletteNode. The result can be supplied to 
*  EditPane.addPaletteNodesFromJSON to recreate
*  the EditNodes.
* 
* 
* @param {SerializationSettings=} Additional serialization settings
*/
serializeAllEditNodesAsJSON(settings?:SerializationSettings): string;

/**
*  Serialize the provided EditNode to an XML
*  representation of PaletteNode. Note that the
*  EditNodes must have been added to this EditContext. The result can be
*  supplied to EditPane.addPaletteNodesFromXML to
*  recreate the EditNodes.
* 
* 
* @param {Array<Partial<EditNode>>} EditNodes to be serialized
* @param {SerializationSettings=} Additional serialization settings
*/
serializeEditNodes(nodes:Array<Partial<EditNode>>, settings?:SerializationSettings): string;

/**
*  Serialize the provided EditNode to a JSON
*  representation of PaletteNode. Note that the
*  EditNodes must have been added to this EditContext. The result can be
*  supplied to EditPane.addPaletteNodesFromJSON to
*  recreate the EditNodes.
* 
* 
* @param {Array<Partial<EditNode>>} EditNodes to be serialized
* @param {SerializationSettings=} Additional serialization settings
*/
serializeEditNodesAsJSON(nodes:Array<Partial<EditNode>>, settings?:SerializationSettings): string;



}
/**
* Creates a new EditPane
* 
* @param typeCheckedProperties {Partial<EditPane>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {EditPane} The new instance
*/
class EditPane{
static create(typeCheckedProperties? : Partial<EditPane>, uncheckedProperties?: Object): EditPane;







}

/**
* An EditProxy is attached to an editable component when editMode is enabled. This proxy
*  has methods and properties which affect the component during editing.
*/
interface EditProxy extends Class {
/**
* When true an EditProxy.editMask will be auto-generated and
*  placed over the component to allow selection, positioning and resizing.
*  
*  If this property is not set it will enabled when added to an EditContext if its
*  parent component has an editProxy and EditProxy.autoMaskChildren is true.
* @type {boolean}
* @default null
*/
useEditMask?: boolean;

/**
* When child nodes are added to an EditContext, should they be masked by setting
*  EditProxy.useEditMask true if not explicitly set?
* @type {boolean}
* @default null
*/
autoMaskChildren?: boolean;

/**
* Whether to allow selection of the children of this EditNode. The appearance and
*  behavior of selected components is controlled by SelectedAppearance, or centrally
*  across an EditContext via EditContext.selectedAppearance.
*  
*  Individual children can be marked non-selectable via setting EditProxy.canSelect
*  to false.
*  
*  Use the EditContext to access and manipulate the currently selected set of
*  EditNodes, via APIs such as EditContext.getSelectedEditNode,
*  EditContext.selectSingleEditNode and
*  EditContext.selectedEditNodesUpdated.
* @type {boolean}
* @default null
*/
canSelectChildren?: boolean;

/**
* If not null the Canvas.childrenSnapToGrid on the component represented by this
*  EditProxy is set to this value only while in edit mode. This allows snapToGrid functionality
*  to be enforced during edit mode but not when live.
* @type {boolean}
* @default null
*/
childrenSnapToGrid?: boolean;

/**
* If not null the Canvas.childrenSnapResizeToGrid on the component represented by this
*  EditProxy is set to this value only while in edit mode. This allows snapResizeToGrid functionality
*  to be enforced during edit mode but not when live.
* @type {boolean}
* @default null
*/
childrenSnapResizeToGrid?: boolean;

/**
* Can this component be selected? Selection is allowed unless this
*  property is set to false.
* @type {boolean}
* @default null
*/
canSelect?: boolean;

/**
* This property acts as a component-specific override for the EditContext.allowNestedDrops
*  property. Unless explicitly set to false, the EditContext.allowNestedDrops controls whether
*  a drop can be made into this component.
* @type {boolean}
* @default null
*/
allowNestedDrops?: boolean;

/**
* Whether to enable keyboard shortcuts to EditContext.copyEditNodes and
*  EditContext.pasteEditNodes editNodes.
*  
*  Enabled by default if EditProxy.canSelectChildren is also enabled.
*  
*  For pasting, if EditContext.allowNestedDrops is enabled, only one editNode is selected and
*  it is a valid container for the contents of the clipboard, editNodes will be pasted as new
*  children of the selected container. Otherwise, they will be pasted at the root level of the
*  EditContext.getEditNodeTree.
*  
*  useCopyPasteShortcuts may only be set on the root editNode
*  within any one EditContext.getEditNodeTree.
* @type {boolean}
* @default null
*/
useCopyPasteShortcuts?: boolean;

/**
* An editMask is created for any component placed into editMode with
*  EditProxy.useEditMask:true.
*  
*  Common customization properties can be provided by EditContext.editMaskProperties.
* @type {Canvas}
* @default null
*/
editMask?: Canvas;

/**
* Appearance that is applied to selected component. Default value is determined from
*  EditContext.selectedAppearance.
*  
*  When value is null the appearance is determined by:
*  
*  
* - If multiple selection is enabled, "tintMask" is used
*  
* - Otherwise, "outlineMask" is used 
*  
* @type {SelectedAppearance}
* @default null
*/
selectedAppearance?: SelectedAppearance;

/**
* Set the CSS border to be applied to the selection outline of the selected components.
*  Default value is determined from EditContext.selectedBorder.
*  This property is used when EditProxy.selectedAppearance is outlineMask
*  or outlineEdges.
* @type {string}
* @default null
*/
selectedBorder?: string;

/**
* The background color for the selection outline label. The
*  default is defined on SelectionOutline or EditContext.selectedLabelBackgroundColor.
*  
*  NOTE: A selected component label is only supported when
*  EditProxy.selectedAppearance is "outlineEdges".
* @type {string}
* @default null
*/
selectedLabelBackgroundColor?: string;

/**
* Mask color applied to EditProxy.editMask of selected component when
*  EditProxy.selectedAppearance is "tintMask".
*  Default value is determined from EditContext.selectedTintColor.
* @type {CSSColor}
* @default null
*/
selectedTintColor?: CSSColor;

/**
* Opacity applied to EditProxy.editMask of selected component when
*  EditProxy.selectedAppearance is "tintMask".
* @type {number}
* @default null
*/
selectedTintOpacity?: number;

/**
* Should drag handles or thumb be shown when this component is selected?
*  These are shown unless this property is set to false.
* @type {boolean}
* @default null
*/
showDragHandle?: boolean;

/**
* Whether this EditProxy has an inline edit behavior, which allows an end user to
*  configure a component by editing a simple text representation of its configuration. 
*  
*  For example, when inline edit is enabled, a SelectItem allows
*  SelectItemEditProxy.getInlineEditText as a comma-separated
*  string, and a ListGrid's columns and data can be edited as several lines of
*  comma-separated headings and data values.
*  
*  See EditProxy.inlineEditEvent for more details and configuration options.
* @type {boolean}
* @default true
*/
supportsInlineEdit?: boolean;

/**
* Should the inline editor be shown when new component is first dropped?
* @type {boolean}
* @default null
*/
inlineEditOnDrop?: boolean;

/**
* Event that triggers inline editing, showing the EditProxy.inlineEditForm, which consists of a single
*  text input (single or multi-line according to EditProxy.inlineEditMultiline) shown in the
*  EditProxy.inlineEditForm AutoChild.
*  
*  The initial value in the form comes from EditProxy.getInlineEditText and is applied via
*  EditProxy.setInlineEditText.
*  
*  Many EditProxy subclasses have built-in modes for inline editing.
* @type {InlineEditEvent}
* @default null
*/
inlineEditEvent?: InlineEditEvent;

/**
* See EditProxy.inlineEditEvent.
* @type {DynamicForm}
* @default null
*/
inlineEditForm?: DynamicForm;

/**
* Label AutoChild used to display EditProxy.inlineEditInstructions below the text entry
*  area if provided. Defaults to the same styling as the system Hover.
* @type {Label}
* @default null
*/
inlineEditInstructionLabel?: Label;

/**
* Instructions that appear below the text entry area if inline editing is enabled. See
*  EditProxy.inlineEditEvent and EditProxy.inlineEditInstructionLabel.
* @type {HTMLString}
* @default null
*/
inlineEditInstructions?: HTMLString;

/**
* Whether inline editing should be single or multi-line.
*  
*  Single-line input appears at the control's top-left corner, multiline covers the control.
* @type {boolean}
* @default false
*/
inlineEditMultiline?: boolean;

/**
* Should component be brought to front when selected? Applies when EditProxy.useEditMask:true.
* @type {boolean}
* @default null
*/
bringToFrontOnSelect?: boolean;

/**
* Changes to all child EditNode.liveObject's position
*  and size can be persisted to their EditNode based on this
*  attribute setting and EditContext.persistCoordinates. This
*  applies to both programmatic calls and user interaction (drag reposition
*  or drag resize).
*  
*  The default value of null allows EditContext.persistCoordinates
*  to control all coordinate persistence. An explicit value of false
*  overrides the EditContext setting so that no children of the component save coordinates.
*  
*  All coordinate persisting can be disabled with EditContext.persistCoordinates.
*  Additionally, all control of persistence can be deferred to each EditProxy by setting
*  EditContext.persistCoordinates to null.
* @type {boolean}
* @default null
*/
persistCoordinates?: boolean;

/**
* Hoop selector canvas used for selecting multiple components.
*  
*  Common customization properties can be provided by EditContext.hoopSelectorProperties.
* @type {Canvas}
* @default null
*/
hoopSelector?: Canvas;



/**
*  Setter for EditProxy.canSelectChildren.
* 
* @param {boolean} the new canSelectChildren
*/
setCanSelectChildren(canSelect:boolean): void;

/**
*  This method applies the EditProxy.selectedAppearance to the selected component
*  or resets it to the non-selected appearance. Override this method to create a custom
*  appearance.
* 
* 
* @param {boolean} true to show component as selected, false otherwise
*/
showSelectedAppearance(show:boolean): void;

/**
*  Manual means of triggering inline editing. See InlineEditEvent.
* 
* 
* @param {string=} optional String to append to current value as editing starts
*/
startInlineEditing(appendChar?:string): void;

/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  For a canvas with isGroup enabled, the groupTitle
*  is returned. Otherwise the contents is returned.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  For a canvas with isGroup enabled, the groupTitle
*  is updated. Otherwise the contents is updated.
* 
* 
* @param {string} the new component state
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new EditProxy
* 
* @param typeCheckedProperties {Partial<EditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {EditProxy} The new instance
*/
class EditProxy{
static create(typeCheckedProperties? : Partial<EditProxy>, uncheckedProperties?: Object): EditProxy;







}

/**
* Window that simply contains a SavedSearchEditor as the AutoChild savedSearchEditor.
*  
*  Automatically used by SavedSearchItem and ListGrid.canSaveSearches; cannot be used directly
*  and is documented only for skinning and internationalization purposes.
*/
interface EditSearchWindow extends Window {
/**
* @type {string}
* @default "Save View Configuration"
*/
title?: string;

/**
* Has no effect unless ListGrid.showBackgroundComponents is true.
*  
*  SavedSearchEditor embedded in the EditSearchWindow.
* @type {SavedSearchEditor}
* @default null
*/
savedSearchEditor?: SavedSearchEditor;





}
/**
* Creates a new EditSearchWindow
* 
* @param typeCheckedProperties {Partial<EditSearchWindow>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {EditSearchWindow} The new instance
*/
class EditSearchWindow{
static create(typeCheckedProperties? : Partial<EditSearchWindow>, uncheckedProperties?: Object): EditSearchWindow;







}

/**
* A TreeGrid that allows drag and drop creation and manipulation of a tree of 
*  objects described by DataSources.
*  
*  Nodes can be added via drag and drop from a Palette or may be programmatically 
*  added via EditContext.addNode. Nodes may be dragged within the tree to reparent 
*  them.
*  
*  Eligibility to be dropped on any given node is determined by inspecting the
*  DataSource of the parent node. Drop is allowed only if the parent schema has
*  a field which accepts the type of the dropped node.
*  
*  On successful drop, the newly created component will be added to the parent node under the
*  detected field. Array fields, declared by setting
*  dataSourceField.multiple:true, are supported. 
*  
*  An EditTree is initialized by setting EditTree.rootComponent or
*  EditTree.editContext. EditTree.data (the Tree instance) should never be directly
*  set or looked at.
*  
*  EditTree automatically creates an EditContext and provides several APIs and
*  settings that are passthroughs to the underlying EditContext for convenience.
*/
interface EditTree extends TreeGrid {
/**
* Option to show filter editor is disabled for editTree
* @type {boolean}
* @default false
*/
canShowFilterEditor?: boolean;

/**
* Option to save searches is disabled for editTree
* @type {boolean}
* @default false
*/
canSaveSearches?: boolean;

/**
* The EditContext managed by this EditTree. If not set an instance
*  will be automatically created.
* @type {EditContext}
* @default null
*/
editContext?: EditContext;

/**
* New nodes added to the editContext are automatically placed
*  into edit mode if the new node's parent is in edit mode. To
*  suppress this action set autoEditNewNodes to false.
* @type {boolean}
* @default null
*/
autoEditNewNodes?: boolean;

/**
* Root of data to edit. Must contain the "type" property, with the name of a
*  valid DataSource or nothing will be able to be dropped on this
*  EditContext. A "liveObject" property representing the rootComponent is also
*  suggested. Otherwise, a live object will be created from the palette node.
*  
*  Can be retrieved at any time. Use EditTree.getRootEditNode to retrieve the
*  EditNode created from the rootComponent.
* @type {PaletteNode}
* @default null
*/
rootComponent?: PaletteNode;

/**
* Palette to use when an EditNode is being created directly by this EditContext,
*  instead of being created due to a user interaction with a palette (eg dragging from
*  a TreePalette, or clicking on MenuPalette).
*  
*  If no defaultPalette is provided, the EditContext uses an automatically created
*  HiddenPalette.
* @type {Palette}
* @default null
*/
defaultPalette?: Palette;

/**
* Additional Palette to consult for metadata when
*  deserializing EditNode. Note that the
*  EditTree.defaultPalette is always consulted and need not be
*  provided again here.
* @type {Array<Partial<Palette>>}
* @default null
*/
extraPalettes?: Array<Partial<Palette>>;

/**
* When enabled, changes to a EditNode.liveObject's position
*  and size will be persisted to their EditNode by default.
*  This applies to both programmatic calls and user interaction (drag reposition
*  or drag resize).
*  
*  This feature can be disabled by either setting this property or
*  EditProxy.persistCoordinates to false. This
*  property affects all nodes within the EditContext whereas the latter
*  property affects children of a single node. 
*  
*  In some use-cases, like Reify, coordinates should not be
*  persisted except when a component explicitly enables this feature.
*  By setting this property to null no component will
*  persist coordinates of children unless
*  EditProxy.persistCoordinates is explicitly set to
*  true.
* @type {boolean}
* @default true
*/
persistCoordinates?: boolean;

/**
* Controls whether components can be dropped into other components which support child
*  components. 
*  
*  When enabled, during a drop interaction in which a PaletteNode or EditNode
*  is the drop data, the componentSchema of the current
*  candidate drop target is inspected to see whether that parent allows children of the
*  type being dropped. If it does, the drop will result in a call to EditTree.addNode for
*  a paletteNode or for an existing EditNode in the same tree.
*  
*  Specific components can disable nested drops by explicitly setting EditProxy.allowNestedDrops
*  to false.
*  
*  This mode is enabled by default unless explicitly disabled by setting this property to
*  false.
* @type {boolean}
* @default null
*/
allowNestedDrops?: boolean;

/**
* Should the selection outline show a label for selected components? A component may
*  also be highlighted with the selection outline and label to indicate the target of
*  a drop. To suppress showing a label at any time set this property to false.
*  
*  To suppress labels during selection but still show them when targeted for a drop,
*  see EditContext.showSelectedLabelOnSelect.
*  
*  NOTE: A selected component label is only supported when EditProxy.selectedAppearance
*  is "outlineEdges".
* @type {boolean}
* @default null
*/
showSelectedLabel?: boolean;

/**
* Set the CSS border to be applied to the selection outline of the selected components.
*  This property is used when EditProxy.selectedAppearance is outlineMask
*  or outlineEdges.
*  
*  This value is applied as a default to EditProxy.selectedBorder.
* @type {string}
* @default null
*/
selectedBorder?: string;

/**
* The background color for the selection outline label. The
*  default is defined on SelectionOutline.
*  
*  This value is applied as a default to EditProxy.selectedLabelBackgroundColor.
*  
*  NOTE: A selected component label is only supported when EditProxy.selectedAppearance
*  is "outlineEdges".
* @type {string}
* @default null
*/
selectedLabelBackgroundColor?: string;

/**
* Should a group selection outline covering the outermost bounding boxes of all selected
*  components be shown in this container?
*  
*  Treated as true if not set and hoop selection is enabled (see
*  EditProxy.canSelectChildren and
*  EditContext.selectionType.
* @type {boolean}
* @default null
*/
canGroupSelect?: boolean;

/**
* Should the group selection box shown when EditContext.canGroupSelect
*  is true allow dragging the group as a whole?
*  
*  Treated as true if not set and EditContext.canGroupSelect
*  is true.
* @type {boolean}
* @default null
*/
canDragGroup?: boolean;

/**
* Should the group selection box shown when EditContext.canGroupSelect
*  is true be hidden during drag?
*  
*  Treated as true if not explicitly set to false.
* @type {boolean}
* @default null
*/
hideGroupBorderOnDrag?: boolean;

/**
* If set, auto-enables EditProxy.useCopyPasteShortcuts on the EditProxy for the
*  EditTree.getRootEditNode. This works whether there is currently a root editNode
*  or one is added later.
* @type {boolean}
* @default null
*/
useCopyPasteShortcuts?: boolean;



/**
*  Returns the EditContext instance managed by the EditTree.
* 
*/
getEditContext(): EditContext;

/**
*  Returns the root EditNode of the EditContext typically created from EditTree.rootComponent.
* 
* 
*/
getRootEditNode(): EditNode;

/**
*  Creates and returns an EditNode using the EditTree.defaultPalette. Does not add the newly
*  created EditNode to an EditContext.
*  
* 
* @param {PaletteNode} the palette node to use to create the new node
*/
makeEditNode(paletteNode:PaletteNode): EditNode;

/**
*  Gets the tree of editNodes being edited by this editContext. Standard tree
*  traversal methods can then be used to locate desired editNodes for interaction.
*  
*  Note: the returned tree is read-only and must only be modified by calling
*  methods on EditContext like EditContext.addNode or EditContext.setNodeProperties.
* 
* 
*/
getEditNodeTree(): Tree;

/**
*  Add a new EditNode to the EditContext, under the specified parent. If the parentNode
*  is not provided it will be determined from EditContext.defaultParent.
*  
*  The EditContext will interrogate the parent and new nodes to determine what field 
*  within the parent allows a child of this type, and to find a method to add the newNode's 
*  liveObject to the parentNode's liveObject. The new relationship will then be stored
*  in the tree of EditNodes.
*  
*  For example, when a Tab is dropped on a TabSet, the field TabSet.tabs is discovered as
*  the correct target field via naming conventions, and the method TabSet.addTab() is likewise 
*  discovered as the correct method to add a Tab to a TabSet.
* 
* 
* @param {EditNode} new node to be added
* @param {EditNode=} parent to add the new node under.
* @param {Integer=} index within the parent's children array
* @param {string=} the property of the liveParent to which the new node should                 be added, if not auto-discoverable from the schema
* @param {boolean=} whether to skip adding the liveObject to the liveParent                      (default false)
* @param {boolean=} whether to replace existing single field node                       if newNode liveObject is the same (default false)
*/
addNode(newNode:EditNode, parentNode?:EditNode, index?:Integer, parentProperty?:string, skipParentComponentAdd?:boolean, forceSingularFieldReplace?:boolean): EditNode;

/**
*  Creates a new EditNode from a PaletteNode, using the
*  EditTree.defaultPalette. If you have an array of possibly inter-related
*  PaletteNodes, then you should use
*  EditTree.addFromPaletteNodes on the array instead,
*  in order to preserve the relationships.
* 
* 
* @param {PaletteNode} the palette node to use to create the new node
* @param {EditNode=} optional the parent node if the new node should appear                under a specific parent
*/
addFromPaletteNode(paletteNode:PaletteNode, parentNode?:EditNode): EditNode;

/**
*  Add the supplied PaletteNode to the parentNode, preserving internal
*  references from one supplied PaletteNode to another. This method should
*  be used with an array of possibly inter-related PaletteNodes (for
*  instance, those produced as a result of serialization via
*  EditTree.serializeAllEditNodes, rather than
*  calling EditTree.addFromPaletteNode on each
*  individual PaletteNode.
* 
* 
* @param {Array<Partial<PaletteNode>>} array of PaletteNodes
* @param {EditNode=} parent to add to (defaults to the root)
*/
addFromPaletteNodes(paletteNodes:Array<Partial<PaletteNode>>, parentNode?:EditNode): Array<Partial<EditNode>>;

/**
*  Removes EditNode from the EditContext. The editNode
*  liveObject is not destroyed.
* 
* @param {EditNode} node to be removed
*/
removeNode(editNode:EditNode): void;

/**
*  Moves an EditNode from one child index to another in the EditContext under the specified parent.
*  
*  No changes are made to the live objects.
* 
* 
* @param {EditNode} parent to reorder child nodes
* @param {Integer} index within the parent's children array to be moved
* @param {Integer} index within the parent's children array at which to place moved node
*/
reorderNode(parentNode:EditNode, index:Integer, moveToIndex:Integer): void;

/**
*  Removes all EditNode from the EditContext, but does not destroy 
*  the EditNode.liveObject.
* 
*/
removeAll(): void;

/**
*  Removes all EditNode from the EditContext, and calls
*  Canvas.destroy on the
*  EditNode.liveObject.
* 
*/
destroyAll(): void;

/**
*  Returns true if editNode is in edit mode.
* 
* 
* @param {EditNode} the EditNode
*/
isNodeEditingOn(editNode:EditNode): boolean;

/**
*  Enable edit mode for an EditNode. This is a shortcut for calling
*  Canvas.setEditMode.
* 
* 
* @param {EditNode} the EditNode on which to enable editing
*/
enableEditing(editNode:EditNode): void;

/**
*  Returns the specified property from the editNode's serializable "defaults".
* 
* @param {EditNode} the editNode to query
* @param {string} the property name to query
*/
getNodeProperty(editNode:EditNode, name:string): void;

/**
*  Update an editNode's serializable "defaults" with the supplied properties. If you
*  wish to remove a property from the defaults (rather than setting it to null), then
*  use EditTree.removeNodeProperties instead.
* 
* @param {EditNode} the editNode to update
* @param {Partial<Canvas>} the properties to apply
* @param {boolean=} whether to skip updating the                     EditNode.liveObject,                     e.g. if you have already updated the liveObject
*/
setNodeProperties(editNode:EditNode, properties:Partial<Canvas>, skipLiveObjectUpdate?:boolean): void;

/**
*  Removes the specified properties from an editNode's serializable "defaults".
*  Note that the EditNode.liveObject is not updated by this method. 
*  To set a property to null (rather than removing it), use
*  EditTree.setNodeProperties instead.
* 
* @param {EditNode} the editNode to update
* @param {Array<Partial<string>>} an array of property names to remove
*/
removeNodeProperties(editNode:EditNode, properties:Array<Partial<string>>): void;

/**
* Palette to use when an EditNode is being created directly by this EditContext,
*  instead of being created due to a user interaction with a palette (eg dragging from
*  a TreePalette, or clicking on MenuPalette).
*  
*  If no defaultPalette is provided, the EditContext uses an automatically created
*  HiddenPalette.
*/
getDefaultPalette(): Palette;

/**
* Palette to use when an EditNode is being created directly by this EditContext,
*  instead of being created due to a user interaction with a palette (eg dragging from
*  a TreePalette, or clicking on MenuPalette).
*  
*  If no defaultPalette is provided, the EditContext uses an automatically created
*  HiddenPalette.
* @param {Palette} the default Palette
*/
setDefaultPalette(palette:Palette): void;

/**
*  Creates a PaletteNode from an EditNode in this context's
*  EditTree.getEditNodeTree.
*  
*  This essentially creates a new PaletteNode with the EditNode.defaults from the
*  passed editNode. The returned paletteNode could then be used with
*  EditContext.addFromPaletteNode to effectively create a copy of the original editNode -
*  specifically a new editNode with a new EditNode.liveObject created from the same
*  defaults.
*  
*  However note that makePaletteNode() does not copy descendant nodes - use
*  EditTree.makePaletteNodeTree for that.
*  
*  May return null if the passed editNode cannot validly by transformed into a paletteNode, for
*  example if EditNode.canDuplicate was set false.
* 
* 
* @param {EditNode} the editNode to use to make a paletteNode
*/
makePaletteNode(editNode:EditNode): PaletteNode;

/**
*  Creates a Tree of PaletteNode from an EditNode in this
*  context's EditTree.getEditNodeTree, by using EditTree.makePaletteNode on the
*  passed EditNode and its descendents within the
*  EditContext.getEditNodeTree.
*  
*  The root node of the returned Tree will be a PaletteNode derived from the passed
*  EditNode.
* 
* 
* @param {EditNode} root editNode to make Tree of PaletteNodes from
*/
makePaletteNodeTree(editNode:EditNode): Tree;

/**
*  Copies the passed editNode or editNodes to an internal "clipboard" space, for later application
*  via EditTree.pasteEditNodes.
* 
* @param {EditNode | Array<Partial<EditNode>>} */
copyEditNodes(editNode:EditNode | Array<Partial<EditNode>>): void;

/**
*  "Pastes" editNodes previously captured via EditTree.copyEditNodes.
*  
*  New editNodes will be added as root-level nodes of the EditTree.getEditNodeTree
*  unless a targetEditNode is passed.
* 
* @param {EditNode=} */
pasteEditNodes(targetEditNode?:EditNode): void;

/**
*  Recreate EditNode from an XML representation of 
*  PaletteNode (possibly created by calling
*  EditTree.serializeAllEditNodes or EditTree.serializeEditNodes.
*  
*  By default, components that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
* 
* 
* @param {string} XML string
* @param {EditNode=} parent node (defaults to the root)
* @param {Array<Partial<string>>=} widgets to allow to take their global IDs
* @param {Function=} Callback to fire after nodes have been added
*/
addPaletteNodesFromXML(xmlString:string, parentNode?:EditNode, globals?:Array<Partial<string>>, callback?:Function): void;

/**
*  Recreate EditNode from a JSON representation of 
*  PaletteNode (possibly created by calling
*  EditTree.serializeAllEditNodesAsJSON or EditTree.serializeEditNodesAsJSON.
*  
*  By default, components that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
* 
* 
* @param {string} JSON string representing an array of PaletteNodes
* @param {EditNode=} parent to add to (defaults to the root)
* @param {Array<Partial<string>>=} widgets to allow to take their global IDs
* @param {Function=} Callback to fire after nodes have been added
*/
addPaletteNodesFromJSON(jsonString:string, parentNode?:EditNode, globals?:Array<Partial<string>>, callback?:Function): void;

/**
*  Obtain PaletteNode from an XML representation,
*  but do not add them to the EditContext.
* 
* 
* @param {string} XML string
* @param {PaletteNodeCallback} Callback used to return the PaletteNodes
*/
getPaletteNodesFromXML(xmlString:string, callback:(paletteNodes:Array<Partial<PaletteNode>>) => void): void;

/**
*  Obtain PaletteNode from a JavaScript source representation.
*  
*  By default, components that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
* 
* 
* @param {string} JavaScript code to eval.
* @param {PaletteNodeCallback} Callback used to return the PaletteNodes
* @param {Array<Partial<string>>=} widgets to allow to take their global IDs
*/
getPaletteNodesFromJS(jsCode:string, callback:(paletteNodes:Array<Partial<PaletteNode>>) => void, globals?:Array<Partial<string>>): void;

/**
*  Add PaletteNode from a JavaScript source representation.
*  
*  By default, components that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
* 
* 
* @param {string} JavaScript code to eval.
* @param {EditNode=} parent node (defaults to the root)
* @param {Array<Partial<string>>=} widgets to allow to take their global IDs
*/
addPaletteNodesFromJS(jsCode:string, parentNode?:EditNode, globals?:Array<Partial<string>>): void;

/**
*  Serialize the tree of EditNode to an XML representation
*  of PaletteNode. The result can be supplied to 
*  EditTree.addPaletteNodesFromXML to recreate
*  the EditNodes.
* 
* 
* @param {SerializationSettings=} Additional serialization settings
*/
serializeAllEditNodes(settings?:SerializationSettings): string;

/**
*  Encode the tree of EditNode to a JSON representation
*  of PaletteNode. The result can be supplied to 
*  EditTree.addPaletteNodesFromJSON to recreate
*  the EditNodes.
* 
* 
* @param {SerializationSettings=} Additional serialization settings
*/
serializeAllEditNodesAsJSON(settings?:SerializationSettings): string;

/**
*  Serialize the provided EditNode to an XML
*  representation of PaletteNode. Note that the
*  EditNodes must have been added to this EditContext. The result can be
*  supplied to EditTree.addPaletteNodesFromXML to
*  recreate the EditNodes.
* 
* 
* @param {Array<Partial<EditNode>>} EditNodes to be serialized
* @param {SerializationSettings=} Additional serialization settings
*/
serializeEditNodes(nodes:Array<Partial<EditNode>>, settings?:SerializationSettings): string;

/**
*  Serialize the provided EditNode to a JSON
*  representation of PaletteNode. Note that the
*  EditNodes must have been added to this EditContext. The result can be
*  supplied to EditTree.addPaletteNodesFromJSON to
*  recreate the EditNodes.
* 
* 
* @param {Array<Partial<EditNode>>} EditNodes to be serialized
* @param {SerializationSettings=} Additional serialization settings
*/
serializeEditNodesAsJSON(nodes:Array<Partial<EditNode>>, settings?:SerializationSettings): string;



}
/**
* Creates a new EditTree
* 
* @param typeCheckedProperties {Partial<EditTree>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {EditTree} The new instance
*/
class EditTree{
static create(typeCheckedProperties? : Partial<EditTree>, uncheckedProperties?: Object): EditTree;







}

/**
* Task that ends a workflow. This task is not necessary to end a workflow - having
*  a task execute with no ProcessElement.nextElement is sufficient to end
*  the workflow.
*  
*  This task is primarily used in the workflow editor to render a "no-op" task or as
*  an explicit visual marker for the end of workflow.
*/
interface EndProcessTask extends ProcessElement {




}
/**
* Creates a new EndProcessTask
* 
* @param typeCheckedProperties {Partial<EndProcessTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {EndProcessTask} The new instance
*/
class EndProcessTask{
static create(typeCheckedProperties? : Partial<EndProcessTask>, uncheckedProperties?: Object): EndProcessTask;







}

/**
* The EventCanvas component is a lightweight VLayout subclass for 
*  displaying a CalendarEvent in a CalendarView.
*  
*  Each instance can be CalendarEvent.styleName, and can render a single area,
*  or separate EventCanvas.showHeader and EventCanvas.showBody
*  areas, for the look of a Window.
*  
*  The component's close and context buttons, and any necessary resizers, are
*  shown on EventCanvas.showRolloverControls.
*/
interface EventCanvas extends VLayout {
/**
* Readonly property indicating whether this is a special ZoneCanvas subclass.
* @type {boolean}
* @default false
*/
isZoneCanvas?: boolean;

/**
* Readonly property dictating whether this is a special IndicatorCanvas 
*  subclass.
* @type {boolean}
* @default false
*/
isIndicatorCanvas?: boolean;

/**
* When set to true, escapes the HTML displayed in this EventCanvas and it's hover.
* @type {boolean}
* @default null
*/
escapeHTML?: boolean;

/**
* Renders a header DIV above the main body of the event, an area of limited
*  height, styled to stand out from the main EventCanvas.showBody of the 
*  event, and typically showing a CalendarEvent.name or title - like a Window.
*  This header area can be styled via EventCanvas.headerStyle and the HTML it shows 
*  is retrieved from a call to EventCanvas.getHeaderHTML. 
*  The default is taken from Calendar.showEventHeaders.
* @type {boolean}
* @default null
*/
showHeader?: boolean;

/**
* Renders a body DIV that fills the main area of the canvas, or all of it if no 
*  EventCanvas.showHeader is shown. This area typically displays an 
*  CalendarEvent.description. This area can be styled via 
*  EventCanvas.bodyStyle and the HTML it shows is retrieved
*  from a call to EventCanvas.getBodyHTML. The default is taken
*  from Calendar.showEventDescriptions.
* @type {boolean}
* @default null
*/
showBody?: boolean;

/**
* Indicates the orientation of the event in its containing view. Affects drag and resize
*  orientation and which edges of the canvas are available for resizing.
* @type {boolean}
* @default null
*/
verticalResize?: boolean;

/**
* The CSS class for this EventCanvas. Defaults to the style on 
*  CalendarEvent.styleName, if specified, or on the
*  Calendar.eventStyleName otherwise.
*  
*  Also see EventCanvas.headerStyle and EventCanvas.bodyStyle.
* @type {CSSStyleName}
* @default null
*/
styleName?: CSSStyleName;

/**
* The CalendarEvent associated with this EventCanvas.
* @type {CalendarEvent}
* @default null
*/
event?: CalendarEvent;

/**
* The Calendar in which this EventCanvas is being rendered.
* @type {Calendar}
* @default null
*/
calendar?: Calendar;

/**
* The CalendarView in which this EventCanvas is being rendered.
* @type {CalendarView}
* @default null
*/
calendarView?: CalendarView;

/**
* When set to true, the EventCanvas.getHeaderHTML for the
*  associated event is not rendered inside the eventCanvas itself.
*  
*  Instead, it is rendered in it's own EventCanvas.label and shown 
*  as a peer of this eventCanvas, immediately outside of it.
* @type {boolean}
* @default null
*/
showLabel?: boolean;

/**
* When EventCanvas.showLabel is true, this autoChild is 
*  used to display the EventCanvas.getHeaderHTML, adjacent to
*  this eventCanvas.
* @type {Label}
* @default null
*/
label?: Label;

/**
* When set to true, shows the EventCanvas.gripper component, which snaps,
*  centered, to the top edge of the eventCanvas and can be used to move it with the mouse.
* @type {boolean}
* @default null
*/
showGripper?: boolean;

/**
* The source for the icon displayed as the "gripper" that snaps to the top of an event 
*  canvas and allows an event to be dragged with the mouse.
* @type {SCImgURL}
* @default null
*/
gripperIcon?: SCImgURL;

/**
* When EventCanvas.showGripper is true, this is the component that will
*  be rendered adjacent to the canvas and allow the canvas to be moved with the mouse.
* @type {Img}
* @default null
*/
gripper?: Img;

/**
* Whether the EventCanvas.showHeader should autosize vertically to 
*  display all contents. If true, the header will wrap to multiple lines. If false, the
*  header will be sized according to the specified EventCanvas.headerHeight,
*  or to the full height of the canvas is EventCanvas.showBody is false.
* @type {boolean}
* @default null
*/
headerWrap?: boolean;

/**
* The height for the EventCanvas.showHeader, when 
*  EventCanvas.headerWrap is false and 
*  EventCanvas.showBody is true. If showBody is false, the
*  header area fills the canvas.
* @type {Integer}
* @default null
*/
headerHeight?: Integer;

/**
* CSS class for the EventCanvas.showHeader of the EventCanvas.
*  If unset, defaults to the EventCanvas.styleName with the suffix
*  "Header".
* @type {CSSStyleName}
* @default null
*/
headerStyle?: CSSStyleName;

/**
* CSS class for the EventCanvas.showBody of the EventCanvas.
*  If unset, defaults to the EventCanvas.styleName with the suffix
*  "Body".
* @type {CSSStyleName}
* @default null
*/
bodyStyle?: CSSStyleName;

/**
* When set to true, shows a Calendar.eventCanvasContextButton in the 
*  top corner of an EventCanvas, beside the 
*  Calendar.eventCanvasContextButton. When clicked, shows a 
*  Calendar.getEventCanvasMenuItems containing items applicable to 
*  this canvas.
* @type {boolean}
* @default false
*/
showContextButton?: boolean;

/**
* When set to the default value of true, this attribute causes a set of components to be
*  shown when the mouse rolls over this EventCanvas. These components include the
*  Calendar.eventCanvasCloseButton and 
*  Calendar.eventCanvasContextButton buttons, the latter's 
*  Calendar.eventCanvasContextMenu and the images used for 
*  drag-resizing.
*  
*  Using rollover controls is more efficient that showing static buttons in each 
*  eventCanvas, so this is the default behavior. See 
*  Calendar.useEventCanvasRolloverControls for the alternative.
* @type {boolean}
* @default true
*/
showRolloverControls?: boolean;



/**
*  Assigns a new CalendarEvent to this EventCanvas, including updates to 
*  drag, style and EventCanvas.showRolloverControls properties.
* 
* 
* @param {CalendarEvent} the new event to apply to this EventCanvas
* @param {CSSStyleName=} optional CSS class to apply to this EventCanvas
* @param {CSSStyleName=} optional separate CSS class to apply to the                   EventCanvas.showHeader.
* @param {CSSStyleName=} optional separate CSS class to apply to the                   EventCanvas.showBody.
*/
setEvent(event:CalendarEvent, styleName?:CSSStyleName, headerStyle?:CSSStyleName, bodyStyle?:CSSStyleName): void;

/**
*  Returns the HTML to show in the header of this EventCanvas. The default implementation 
*  returns the Calendar.nameField of the current 
*  EventCanvas.event.
* 
* 
*/
getHeaderHTML(): HTMLString;

/**
*  Return the HTML to show in the body of this EventCanvas. The default implementation 
*  calls Calendar.getEventBodyHTML, which returns the value of the 
*  Calendar.descriptionField for the current 
*  CalendarEvent.
* 
* 
*/
getBodyHTML(): HTMLString;



}
/**
* Creates a new EventCanvas
* 
* @param typeCheckedProperties {Partial<EventCanvas>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {EventCanvas} The new instance
*/
class EventCanvas{
static create(typeCheckedProperties? : Partial<EventCanvas>, uncheckedProperties?: Object): EventCanvas;







}

/**
* 
*  The ISC system provides a predictable cross-browser event-handling mechanism for ISC
*  widgets. Events can be handled both at the page level (i.e., globally), and at the level of
*  individual widgets.
*  
*  With the exception of a few page-specific events ('load', 'unload', 'idle' and 'resize'),
*  events are processed in the following sequence:
*  
*  1. The event is sent to any global (page-level) event handlers. These handlers can cancel
*  further propagation of the event by returning false. You can register to listen for any of the
*  events linked in the seeAlso section (below) by calling Page.setEvent
*  method.
*  
*  2. If the event occurred on a form element or a link, it is passed on to the browser so that
*  the element will perform its default action. No widget receives the event.
*  
*  3. If the event occurred on an enabled widget (but not on a form element or link inside
*  the widget), it is sent to that widget's event handler, if any. This handler can cancel
*  further propagation of the event by returning false. An "enabled" widget is any widget that
*  defines an event handler for one of the supported events. Interceptable events are defined in
*  the "widgetEvents" section of Canvas.
*  
*  4. The event is "bubbled" up to the widget's parent in the containment hierarchy, if any.
*  Again, the parent's handler for the event can cancel further propagation by returning
*  false. This step is repeated, with the event "bubbling" up through the containment
*  hierarchy, until a top-level widget is reached or the event is explicitly canceled.
*  In brief, the ISC event model offers the best features of browser event models:
*  
*  
* - Page-first event handling allows you to reliably process or cancel any event before it
*    affects the objects on the page.
*  
* - Event "bubbling" ensures that parent widgets receive events sent to their children,
*    and allows you to create generalized parent-level handlers rather than duplicating
*    code in each child.
*  
*  Note: Canceling propagation of an event may cancel its side effects as well, including the
*  generation of other (synthetic) events. For example, if a global mouseDown handler returns
*  false, drag-and-drop events will not be generated. Specific effects are discussed in the
*  descriptions of the various events in the following sections.
*  
*  SmartClient libraries will not interfere with native event handling when events occur
*  outside of a target widget. You can therefore have HTML that is not ISC-based on the same
*  page as widget objects that will react to native events as you would expect.
*  
*  You can use isc.Event as an alias for isc.EventHandler.
*  
*  
*/
interface EventHandler extends Class {




}
/**
* Creates a new EventHandler
* 
* @param typeCheckedProperties {Partial<EventHandler>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {EventHandler} The new instance
*/
class EventHandler{
static create(typeCheckedProperties? : Partial<EventHandler>, uncheckedProperties?: Object): EventHandler;
/**
* amount of time between idle messages (msec)
* @type {Integer}
* @default "10"
*/
static IDLE_DELAY?: Integer;

/**
* Return this constant from a child event to stop the event propagating to its parent,
*  without suppressing any native browser handling associated with the event.
*  Developers should not need to modify this value - it should be treated as read-only
*  in most circumstances.
* @type {string}
* @default "***STOP***"
*/
static STOP_BUBBLING?: string;

/**
* Constant containing the full set of edges a component may be resized from.
*  When a component is marked as canDragResize, this will be the default set of edges
*  from which it may be resized.
* @type {Array<Partial<string>>}
* @default "["T","L","B","R","TL","TR","BL","BR"]"
*/
static ALL_EDGES?: Array<Partial<string>>;

/**
* If set to true, when the user drags a Canvas.canDrop canvas
*  over any component with Canvas.canAcceptDrop set to false or where 
*  Canvas.willAcceptDrop returns false, the Canvas.noDropCursor
*  will be shown automatically to indicate this is not a valid drop point.
*  
*  This property can be modified at runtime, meaning a developer could choose to show the
*  no drop indicator for specific drag/drop interactions by changing the value from
*  a dragStart handler or similar.
*  
*  Note that when this property is false, developers may still use the
*  Canvas.dropMove handler for potential drop targets and use Canvas.setCursor
*  to explicitly indicate invalid drop areas within a widget. This is the approach used
*  by default for TreeGrid drag/drop interactions, for example.
* @type {boolean}
* @default false
*/
static showNoDropIndicator?: boolean;

/**
* Advanced property governing whether focus and blur notifications throughout the
*  SmartClient system should be fired synchronously in Internet Explorer and 
*  Microsoft Edge, as they are in other browsers.
*  
*  Internet Explorer differs from other supported browsers in that the native 
*  onfocus and onblur browser events are fired asynchronously.
*  In all other browsers these handlers are fired synchronously.
*  Historically, SmartClient focus change event notifications such as 
*  Canvas.focusChanged and FormItem.focus / FormItem.blur were
*  fired from these native event handlers, meaning that they would also be asynchronous in
*  Internet Explorer and synchronous in all other browsers.
*  Internet Explorer does provide developers with a separate focus-change notification 
*  which fires synchronously in the form of the focusin and focusout
*  events (documented here).
*  When synchronousFocusNotifications is set to true the SmartClient system will leverage
*  these events to provide synchronous notifications in Internet Explorer.
*  
*  For example, consider a Canvas with a 'focusChanged' handler, as follows:
*  
*  
*  // ... Canvas definition
*  isc.Canvas.create({
*    ID:"testCanvas", backgroundColor:"lightblue",
*    contents:"testCanvas",
*    canFocus:true,
*    focusChanged:function (hasFocus) {
*      this.logWarn('focusChanged:' + hasFocus);
*    },
*    autoDraw:true
*  });
*  
*  
*  
*  ...along with the following code to put focus into that canvas:
*  
*  
*  // ... Code to execute in the flow of the application
*  isc.logWarn("Before calling focus");
*  testCanvas.focus();
*  isc.logWarn("After calling focus");
*  
*  
*  
* 
*  If synchronousFocusNotifications is false, the focus changed notification will
*  be fired asynchronously in Internet Explorer, meaning the order of events logged
*  in the developer console would be:
*  
*  WARN:Log:Before calling focus
*  WARN:Log:After calling focus
*  WARN:Canvas:testCanvas:focusChanged:true
*  
*  In all other browsers, the focus changed notification is synchronous:
*  
*  WARN:Log:Before calling focus
*  WARN:Canvas:testCanvas:focusChanged:true
*  WARN:Log:After calling focus
*  
*  Setting synchronousFocusNotifications to true makes event notifications synchronous
*  in Internet Explorer as well.
*  
*  As of SmartClient version 11.1 (SmartGWT version 6.1), this property is true
*  by default. For backwards compatibility purposes, it may be explicitly set
*  to false to reinstate the previous asynchronous focus notification behavior 
*  in Internet Explorer, should application code depend on this behavior.
* @type {boolean}
* @default true
*/
static synchronousFocusNotifications?: boolean;





/**
*  Returns the natively reported target (or source) DOM element for the current mouse event.
*  NOTE: SmartClient cannot guarantee that the same element will
*  be reported in all browser/platform configurations for all event types.
*  If you wish to make use of this value, we recommend testing your use case 
*  in all target browser configurations.
* 
* 
*/
static getNativeMouseTarget(): DOMElement;

/**
*  Method to return the Canvas.canFocus canvas with 
*  current keyboard focus.
* 
* 
*/
static getFocusCanvas(): Canvas;

/**
*  This API may be called to set the native HTML5 drag tracker image. The x and
*  y parameters may be specified to affect the placement of the drag tracker image
*  relative to the mouse cursor. The size of the drag tracker image is the intrinsic size of the
*  image. Browsers may apply certain visual effects (such as a slight transparency) to this image.
*  
*  Can only be called during the Canvas.dragStart event (or methods called during the
*  handling of that event).
*  
*  NOTES:
*  
*  
* - Not supported in Opera 12.x or Safari.
*  
* - For best results, this image should be preloaded. Otherwise, the image might not appear
*  for the first drag using this image.
*  
* - This API does not work in Chrome or Firefox on Windows 7 if the "Use visual styles on windows and buttons"
*  setting is turned off.
*  
* 
* @param {SCImgURL} image source
* @param {Integer=} offset-x from the mouse cursor
* @param {Integer=} offset-y from the mouse cursor
*/
static setDragTrackerImage(src:SCImgURL, x?:Integer, y?:Integer): void;

/**
*  For a cross-frame drag, retrieves the data made available when the drag was initiated in the
*  foreign frame via EventHandler.setNativeDragData.
*  
*  Can only be called during the Canvas.drop event (or methods called during the handling
*  of that event, such as ListGrid.recordDrop); will return null if called at any other
*  time, or if called during a non-HTML5 drag and drop.
* 
*/
static getNativeDragData(): object;

/**
*  Sets the data available in a cross-frame HTML5 drag (see Canvas.useNativeDrag).
*  
*  Data provided to this method must be valid for serialization to JSON via the
*  JSONEncoder, or can simply be a String.
*  
*  Can only be called during the Canvas.dragStart event (or methods called during the
*  handling of that event).
*  
*  Do not pass in sensitive data (e.g. passwords, auth/session tokens, credit card numbers, SSNs, etc.).
* 
* @param {object | string} data to make available to foreign frames
* @param {string=} text data to set. This is the text that users may see if the drag is dropped into an external application such as Notepad or a non-Smart&#67;lient/Smart&nbsp;GWT web application.
*/
static setNativeDragData(data:object | string, strData?:string): void;

/**
*  Applies to Canvas.mouseWheel events only.
*  Returns a numeric value indicating how far the mouse wheel was rotated. This value will be
*  positive if the user scrolled the mousewheel forward or up, or negative if scrolled in the
*  other direction. For a standard wheel-mouse, an increment of 1 relates to the smallest
*  possible rotation of the mouse wheel. For other scrolling devices, such as scroll 
*  gestures on a track pad, wheel delta may be reported in finer grained increments 
*  (causing this method to return a fractional value).
*  
*  Note that behavior for trackpad scroll-gestures may differ by browser, but where 
*  separate vertical and horizontal scroll information is available, this method
*  refers to a vertical scroll gesture.
*  
*  Developers should also be aware
*  that some browsers and operating systems allow the user to configure the sensitivity
*  of the mouse wheel or trackpad, which may change this value.
* 
* 
* @deprecated * in favor of EventHandler.getWheelDeltaY

*/
static getWheelDelta(): Float;

/**
*  Horizontal scroll delta reported by a Canvas.mouseWheel event
*  (such as a horizontal swipe on a track-pad).
*  
*  Returns a numeric value indicating how far the mouse wheel was rotated / the magnitude
*  of the scroll gesture. This value will be
*  positive if the user scrolled the mousewheel to the right, negative if scrolled in the
*  other direction. 
*  
* 
*/
static getWheelDeltaX(): Float;

/**
*  Applies to Canvas.mouseWheel events only.
*  Returns a numeric value indicating how far the mouse wheel was rotated. This value will be
*  positive if the user scrolled the mousewheel forward or up, or negative if scrolled in the
*  other direction. For a standard wheel-mouse, an increment of 1 relates to the smallest
*  possible rotation of the mouse wheel. For other scrolling devices, such as scroll 
*  gestures on a track pad, wheel delta may be reported in finer grained increments 
*  (causing this method to return a fractional value).
*  
*  Note that behavior for trackpad scroll-gestures may differ by browser, but where 
*  separate vertical and horizontal scroll information is available, this method
*  refers to a vertical scroll gesture.
*  
*  Developers should also be aware
*  that some browsers and operating systems allow the user to configure the sensitivity
*  of the mouse wheel or trackpad, which may change this value.
* 
* 
*/
static getWheelDeltaY(): Float;

/**
*  Sets the initial coordinate offset of the last event, typically a mouseDown or touchStart,
*  from the drag target. For example, when grabbing and dragging a Scrollbar thumb with
*  the mouse, you'd expect positive coordinates that reflect your position relative to the top,
*  left corner of the thumb. If a drag tracker will be used, call EventHandler.setDragTracker
*  instead, which takes optional arguments offsetX and offsetY that
*  act similarly to those passed to this method.
*  
*  Your canvas can call this method to set the initial drag offset to whatever you want like so:
*  
*   dragStart : function () {
*     isc.EventHandler.setDragOffset(5, 20);
*   }
* 
* 
* @param {Integer} initial x-offset for the drag
* @param {Integer} initial y-offset for the drag
*/
static setDragOffset(offsetX:Integer, offsetY:Integer): void;

/**
*  Set the HTML for the drag tracker that follows the mouse during a drag and drop interaction.
*  
*  Your canvas can use this routine to set the drag tracker to whatever HTML you want like so:
*  
*   dragStart : function () {
*     isc.EventHandler.setDragTracker('Your contents here');
*   }
*  
* 
* 
* @param {string} HTML for the tracker
* @param {Integer=} new width for the tracker. Default value: 10
* @param {Integer=} new height for the tracker. Default value: 10
* @param {Integer=} x-offset for the tracker
* @param {Integer=} y-offset for the tracker
* @param {Partial<Canvas>=} properties to configure the dragTracker
*/
static setDragTracker(html:string, newWidth?:Integer, newHeight?:Integer, offsetX?:Integer, offsetY?:Integer, properties?:Partial<Canvas>): void;

/**
*  During a drag with Canvas.dragAppearance of either "target" or
*  "outline", returns the page-relative coordinates of whatever element is being dragged.
*  
*  Calling this method allows you to write drag and drop logic that works identically even if
*  dragAppearance is subsequently changed.
* 
* 
*/
static getDragRect(): any;

/**
*  Return the canvas that is the target of the mouse event.
*  Returns null if no canvas found.
* 
* 
*/
static getTarget(): Canvas;

/**
* 
*  Returns the current dragTarget. This is the component on which the drag and drop
*  interaction was initiated. This only returns something meaningful during a drag and drop
*  interaction.
* 
* 
*/
static getDragTarget(): Canvas;

/**
*  	Return the page-relative X (horizontal) coordinate of an event.
* 
* 
*/
static getX(): Integer;

/**
*  	Return the page-relative Y (vertical) coordinate of an event.
* 
* 
*/
static getY(): Integer;

/**
*  	Returns true if the left mouse button is being pressed.
* 
* 
*/
static leftButtonDown(): boolean;

/**
*  	Returns true if the right mouse button is being pressed.
* 
* 
*/
static rightButtonDown(): boolean;

/**
*  Returns true if the middle mouse button is being pressed.
*  
*  Checking whether the middle mouse button is pressed can be used to implement power user
*  shortcuts; however, note that many pointing devices do not have a middle button. Thus, the
*  application should not require the user to press a middle button in order
*  to perform some action.
* 
*/
static middleButtonDown(): boolean;

/**
*  Returns the numeric characterValue reported by the browser.
*  
*  Only available on keyPress events, and only for character (or ascii control) keys.
*  
*  See the keyboardEvents for related APIs and more information
*  on keyboard event handling in SmartClient.
* 
* 
*/
static getKeyEventCharacterValue(): Integer;

/**
*  Returns the character for a keypress event, derived from the EventHandler.getKeyEventCharacterValue.
*  
*  Only available on keyPress events, and only for character (or ascii control) keys.
*  
*  See the keyboardEvents for related APIs and more information
*  on keyboard event handling in SmartClient.
* 
* 
*/
static getKeyEventCharacter(): string;

/**
*  Return the name of the key for the current event.
*  Note that this is only set reliably for keyboard events.
*  
*  This method returns a calculated key name for the current key. See KeyName
*  for a list of all possible key names.
*  
*  Note: this differs from the natively reported 
*  event.key 
*  value for the event. To retrieve that value, use EventHandler.getKeyEventKey
*  rather than this method.
*  
*  See the keyboardEvents for related APIs and more information
*  on keyboard event handling in SmartClient.
*  
* 
*/
static getKey(): KeyName;

/**
*  Return the natively reported
*  event.key value
*  for the current event.
*  
*  Note that this differs from EventHandler.getKey in a couple of important ways:
*  
*  
* - The actual key values returned for specific keys
*    differ in a number of ways. For example alpha characters keys are natively reported as either
*    upper or lower case depending on what would actually by typed, 
*    (whereas EventHandler.getKey is always uppercase for alpha keys) and
*    the reported name for various 'named' keys (such as the arrow keys differ).
*    
*    The full set of native key names is available
*    here,
*    and the SmartClient key names is available KeyName.
*  
* - EventHandler.getKeyEventKey() simply provides access to the event.key
*    value. As such the reported values are generated by the browser itself and SmartClient
*    has no control over whether they may vary by browser, etc.
*  
*  
*  See the keyboardEvents for related APIs and more information
*  on keyboard event handling in SmartClient.
* 
* 
*/
static getKeyEventKey(): string;

/**
*  This method is a synonym for EventHandler.getKeyEventKey
* 
* 
*/
static getReportedKey(): string;

/**
*  Return the natively reported
*  event.code value
*  for the current event (keyboard) event.
*  
*  Note that EventHandler.getReportedKey() simply provides access to the event.code
*  value reported by the browser. SmartClient has no control over whether these values are accurate
*  or vary by browser.
*  
*  See the keyboardEvents for related APIs and more information
*  on keyboard event handling in SmartClient.
* 
* 
*/
static getKeyEventCode(): string;

/**
*  	Return true if the shift key is being held down.
*  	Note that this is only set reliably for keyboard events.
* 
* 
*/
static shiftKeyDown(): boolean;

/**
*  	Return true if the control key is being held down.
*  	Note that this is only set reliably for keyboard events.
* 
* 
*/
static ctrlKeyDown(): boolean;

/**
*  	Return true if the alt (option) key is being held down.
*  	Note that this is only set reliably for keyboard events.
* 
* 
*/
static altKeyDown(): boolean;

/**
*  Return whether this Canvas is masked by a clickMask (see Canvas.showClickMask).
* 
* 
* @param {Canvas} widget to check.
*/
static targetIsMasked(target:Canvas): boolean;




}

/**
* Extends an arbitrary DataSource with the ability to queue requests made on it and
*  dispatch the queued requests on demand. To use, create a FacadeDataSource instance with
*  the DataSource.inheritsFrom property set to the DataSource that you wish
*  to extend.
*  
*  This advanced class is intended to be used for testing data-bound components. This should
*  not be used in production code.
*  
*  See also the overview of the dsFacade.
*/
interface FacadeDataSource extends DataSource {
/**
* Should requests be queued?
*  
*  When DS requests are made on the FacadeDataSource, a new, derived DS request on the underlying
*  DataSource.inheritsFrom DataSource is created. If queueRequests is true,
*  then the derived DS request is added to the FacadeDataSource.queuedRequests
*  array. If false, then the derived DS request is DataSource.execute
*  immediately on the inherited DataSource.
* @type {boolean}
* @default false
*/
queueRequests?: boolean;

/**
* An array of derived DS requests that are queued to be DataSource.execute on the
*  underlying DataSource.inheritsFrom DataSource.
*  
*  When a DS request is made on this FacadeDataSource, if FacadeDataSource.queueRequests
*  is true, then a new DS request is created based on the given DS request and added to this
*  queue.
*  
*  To clear the queue, set FacadeDataSource.queueRequests to false
*  or call FacadeDataSource.clearQueue.
* @type {Array<Partial<DSRequest>>}
* @default null
*/
queuedRequests?: Array<Partial<DSRequest>>;



/**
*  Setter for FacadeDataSource.queueRequests.
* 
* @param {boolean} */
setQueueRequests(queueRequests:boolean): void;

/**
*  Shorthand to clear the FacadeDataSource.queuedRequests without
*  changing the value of FacadeDataSource.queueRequests.
* 
*/
clearQueue(): void;



}
/**
* Creates a new FacadeDataSource
* 
* @param typeCheckedProperties {Partial<FacadeDataSource>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FacadeDataSource} The new instance
*/
class FacadeDataSource{
static create(typeCheckedProperties? : Partial<FacadeDataSource>, uncheckedProperties?: Object): FacadeDataSource;







}

/**
* Facet definition object made use of by the CubeGrid and
*  FacetChart classes.
*/
interface Facet extends Class {
/**
* id of this facet. Any string or number.
* @type {string | number}
* @default null
*/
id?: string | number;

/**
* User-visible title of this facet. Shown on the facet label in the CubeGrid.
* @type {string}
* @default null
*/
title?: string;

/**
* Array of facetValue definitions.
* @type {Array<Partial<FacetValue>>}
* @default null
*/
values?: Array<Partial<FacetValue>>;

/**
* When applied to a Chart, does the chart's data contain multiple values per record
*  for this facet. See Chart.data for a full overview of inlinedValues
*  behavior.
* @type {boolean}
* @default null
*/
inlinedValues?: boolean;

/**
* facetValueId of the rollup facetValue for this facet.
*  
*  Note: This property is specific to CubeGrid. See
*  CubeGrid.rollupValue.
* @type {string}
* @default "cubeGrid.rollupValue"
*/
rollupValue?: string;

/**
* Marks this facet as a hierarchical facet.
*  
*  If set, Facet.values will be linked as for a
*  Tree.modelType Tree, using
*  FacetValue.id and
*  FacetValue.parentId.
*  Expand/collapse controls will be shown allowing navigation of the facet's values.
*  
*  The CubeGrid's CubeGrid.dataSource system automatically avoids
*  fetching data for facetValues that are not currently visible due to the expand/collapse
*  state of a tree facet.
*  
*  Initial open/close state can be controlled via
*  Facet.collapsed and FacetValue.collapsed.
*  
*  Note: This property is specific to CubeGrid.
* @type {boolean}
* @default false
*/
isTree?: boolean;

/**
* Indicates internal hierarchy should be displayed in reverse of normal tree order (so
*  that parents follow children).
*  
*  Note: This property is specific to CubeGrid.
* @type {boolean}
* @default false
*/
showParentsLast?: boolean;

/**
* Should folders in the facet tree be sorted separately from leaves or should the facet
*  values retain their data order regardless of whether a value is a leaf or folder in the
*  tree?
* @type {boolean}
* @default false
*/
separateFolders?: boolean;

/**
* If Facet.separateFolders is true, should folders be displayed above or below leaves?
*  When set to true folders will appear above leaves unless the data order has
*  been reversed by setting Facet.showParentsLast to true.
* @type {boolean}
* @default true
*/
sortFoldersBeforeLeaves?: boolean;

/**
* For tree facets, default collapse state for parent nodes.
*  
*  Note: This property is specific to CubeGrid.
* @type {boolean}
* @default false
*/
collapsed?: boolean;

/**
* Default CubeGrid.canMinimizeFacets for parent nodes.
*  
*  Note: This property is specific to CubeGrid.
* @type {boolean}
* @default false
*/
minimized?: boolean;

/**
* For tree facets, whether expand/collapse controls should be shown.
*  
*  Note: This property is specific to CubeGrid (see, for
*  example, CubeGrid.canCollapseFacets).
* @type {boolean}
* @default "facet.isTree"
*/
canCollapse?: boolean;

/**
* If facet minimizing is enabled, whether this facet should show controls to minimize the
*  next facet. Generally a tree facet should not also allow minimizing the next facet -
*  the interaction of the two types of collapsing can be confusing.
*  
*  Note: This property is specific to CubeGrid. See
*  CubeGrid.canMinimizeFacets.
* @type {boolean}
* @default "cubeGrid.canMinimizeFacets"
*/
canMinimize?: boolean;

/**
* Integer number of pixels. For row facets, width of headers.
*  
*  Note: This property is specific to CubeGrid.
* @type {number}
* @default "cubeGrid.defaultFacetWidth"
*/
width?: number;

/**
* Integer number of pixels. For column facets, specifies the height of header.
*  Has no effect on row facets.
*  
*  If this property conflicts with a Facet.labelHeight, the greater of the two
*  properties will be used for determining the height of the affected row.
*  
*  Note: This property is specific to CubeGrid.
* @type {number}
* @default null
*/
height?: number;

/**
* Integer number of pixels. For column facets other than the innermost, specifies
*  the height of the header. For row facets, specifies the height of the row
*  containing that row facet's label (which is the same row containing the 
*  innermost column facet if one or more column facets are present).
*  
*  If this property conflicts with a Facet.height, the greater of the two
*  properties will be used for determining the height of the affected row.
*  
*  Note: This property is specific to CubeGrid.
* @type {number}
* @default null
*/
labelHeight?: number;

/**
* Default alignment for facet label title, and cells for this facet.
*  Can be overridden at the facetValue level, or by setting titleAlign or cellAlign
*  on the facet.
*  
*  Note: This property is specific to CubeGrid (see, for
*  example, CubeGrid.facetValueAlign).
* @type {Alignment}
* @default null
*/
align?: Alignment;

/**
* Alignment of facet label title.
*  
*  Note: This property is specific to CubeGrid.
* @type {Alignment}
* @default "cubeGrid.facetTitleAlign"
*/
titleAlign?: Alignment;

/**
* Default alignment of cells (in the body) for this facet.
*  
*  Note: This property is specific to CubeGrid.
* @type {Alignment}
* @default "cubeGrid.cellAlign"
*/
cellAlign?: Alignment;

/**
* CSS line style to apply as a border before this facet, eg "1px dashed blue"
*  
*  Note: This property is specific to CubeGrid.
* @type {string}
* @default null
*/
borderBefore?: string;

/**
* CSS line style to apply as a border after this facet, eg "1px dashed blue"
*  
*  Note: This property is specific to CubeGrid.
* @type {string}
* @default null
*/
borderAfter?: string;

/**
* Controls indenting for a Facet.isTree column facet if set non-null,
*  overrriding the value of CubeGrid.indentVTreeFacets.
* @type {boolean}
* @default null
*/
indentVTree?: boolean;

/**
* Selection boundary determining what facets / facetValues can be selected together by
*  drag selection / shift+click selection.
*  
*  Note: This property is specific to CubeGrid.
* @type {SelectionBoundary}
* @default null
*/
selectionBoundary?: SelectionBoundary;

/**
* Value for facet summary.
*  
*  Note: This property is specific to CubeGrid.
* @type {string}
* @default "cubeGrid.summaryValue"
*/
summaryValue?: string;

/**
* Title for facet summary.
*  
*  Note: This property is specific to CubeGrid.
* @type {string}
* @default "cubeGrid.summaryTitle"
*/
summaryTitle?: string;

/**
* If true, treat all values in this facet as a facetValueGroup - causes
*  synched header reorder and resize.
*  
*  Note: This property is specific to CubeGrid and is only
*  supported when all of a facet's values are used.
* @type {boolean}
* @default true
*/
synchColumnLayout?: boolean;

/**
* For FacetChart only, this property specifies the value axis
*  label when a FacetChart is in FacetChart.proportional
*  and has this facet as its FacetChart.getLegendFacet. If the
*  proportionalTitle is not specified then
*  FacetChart.proportionalAxisLabel is used as the default title.
* @type {string}
* @default null
*/
proportionalTitle?: string;





}
/**
* Creates a new Facet
* 
* @param typeCheckedProperties {Partial<Facet>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Facet} The new instance
*/
class Facet{
static create(typeCheckedProperties? : Partial<Facet>, uncheckedProperties?: Object): Facet;







}

/**
* HTML5-based charting engine, implementing all Chart.chartType of the
*  Chart interface.
*  
*  Can be used directly, or specified as ListGrid.chartConstructor or
*  CubeGrid.chartConstructor.
*  
*  NOTE: you must load the standard Drawing and 
*  loadingOptionalModules Charts modules before you can use FacetChart.  Also, 
*  the Charts Module is available in Pro Edition or better, please see 
*  smartclient.com/product for licensing 
*  information.
*  
*  To create a FacetChart, set FacetChart.facets to an Array of Facet objects describing the
*  chart dimensions and FacetChart.valueProperty to value field name. For example:
*  
*  
*  isc.FacetChart.create({
*    facets: [{
*      id: "season",  // the key used for this facet in the data above
*      title: "Season" // the user-visible title you want in the chart
*    }],
*    valueProperty: "temp", // the property in our data that is the numerical value to chart
*    data: [
*      {season: "Spring", temp: 79},
*      {season: "Summer", temp: 102},
*      {season: "Autumn", temp: 81},
*      {season: "Winter", temp: 59}
*    ],
*    title: "Average temperature in Las Vegas"
*  });
*  
*  
*  
*  
*  A DataSource may be provided instead of inline FacetChart.data to use the
*  chart as a DataBoundComponent. In this case, FacetChart.facetFields may be provided instead of FacetChart.facets, to specify which
*  DataSource fields to use as the facets. If neither is set, the framework will attempt to
*  auto-derive the FacetChart.facetFields. The
*  FacetChart.valueProperty will also be auto-derived for databound charts
*  if it hasn't been set in the chart instance.
*  
*  The following SDK examples demonstrate charts with a single facet:
*  
*  
* - ${isc.DocUtils.linkForExampleId('logScalingChart', 'Log Scaling')} example,
*  
* - ${isc.DocUtils.linkForExampleId('dataPointsChart', 'Interactive Data Points')} example, and
*  
* - ${isc.DocUtils.linkForExampleId('addingElements', 'Adding Element')} example.
*  
*  See the following SDK examples for examples of charts with multiple facets:
*  
*  
* - ${isc.DocUtils.linkForExampleId('simpleChart', 'Simple Chart')} example,
*  
* - ${isc.DocUtils.linkForExampleId('multiSeriesChart', 'Multi-Series Chart')} example, and
*  
* - ${isc.DocUtils.linkForExampleId('dynamicDataCharting', 'Dynamic Data')} example.
*  
*  
*  the Inlined Facet
*  
*  Having an "inlined facet" is another method to provide data to the chart. In this case each CellRecord
*  contains multiple data values; one facet definition is considered "inlined", meaning that
*  the facetValueIds from this facet appear as properties in each Record, and each such
*  property holds one data value. In this case the singular valueProperty is ignored.
*  For example:
*  
*  
*  isc.FacetChart.create({
*    facets: [
*      {
*        inlinedValues: true,
*        values: [
*          {id: "spring", title: "Spring"},
*          {id: "summer", title: "Summer"},
*          {id: "autumn", title: "Autumn"},
*          {id: "winter", title: "Winter"}
*        ]
*      }
*    ],
*    data: [
*      {spring: 79, summer: 102, autumn: 81, winter: 59}
*    ],
*    title: "Average temperature in Las Vegas"
*  });
*  
*  Example with two facets:
*  
*  isc.FacetChart.create({
*    facets: [
*      {
*        inlinedValues: true,
*        values: [
*          {id: "spring", title: "Spring"},
*          {id: "summer", title: "Summer"},
*          {id: "autumn", title: "Autumn"},
*          {id: "winter", title: "Winter"}
*        ]
*      },
*      {id: "city"}
*    ],
*    data: [
*      {city: "Las Vegas", spring: 79, summer: 102, autumn: 81, winter: 59},
*      {city: "New York", spring: 60, summer: 83, autumn: 66, winter: 40}
*    ],
*    stacked: false,
*    title: "Average temperatures"
*  });
*  
*  
*  
*  
*  Dual axis or multi-axis charts
*  
*  FacetChart supports drawing multiple vertical axes. This is commonly used to show values
*  with different units (for example: sales in dollars, total units shipped) and/or very different
*  ranges (for example: gross revenue, profit) on the same chart. Each set of values, referred
*  to as a "metric", gets its own axis and gradation marks.
*  
*  To use multiple axes, you add an additional facet called the "metric facet" that specifies
*  each axis to be plotted as a facetValueId. The metric facet is an inlined facet, so as with
*  inlined facets in general, each CellRecord has a value for each facetValueId of the metric
*  facet. You then set FacetChart.extraAxisMetrics to the list of
*  metrics that should be plotted as additional axes.
*  
*  For example, if you were plotting revenue and profit for each month of the year, you would
*  have one facet named "metric" with facetValueIds "revenue" and "profit" and a second facet
*  "month". Each CellRecord would have the revenue and profit for one month, stored under the
*  properties "revenue" and "profit". Setting extraAxisMetrics to ["profit"]
*  would cause profit to be plotted as the second axis. See the
*  ${isc.DocUtils.linkForExampleId('dualAxisChartMA', 'Dual Axis')} SDK sample for an example.
*  
*  You can have multiple extra axes and the additional axes and gradation tics will be drawn at
*  increasing distances from the chart. By default, the first metric is drawn as a column chart
*  and subsequent metrics are drawn as lines; you can override this via
*  FacetChart.extraAxisSettings. See the
*  ${isc.DocUtils.linkForExampleId('threePlusChartMA', '3+ Axes')} SDK sample for an example of multiple extra axes.
*  
*  Multi-axis, multi-facet charts are also allowed. Extending the previous example, you might
*  add a new facet "company", for a total of 3 facets. Each CellRecord would have "revenue"
*  and "profit" for one combination of "company" and "month". The default appearance in this
*  case would show revenue as clustered columns (one cluster per month, one column per company)
*  and would show profit as multiple lines (one per company). See the
*  ${isc.DocUtils.linkForExampleId('multiSeriesChartMA', 'Multi-Series')} SDK sample for an example of a
*  multi-axis, multi-facet chart.
*  
*  Mixed plots
*  In some cases you want to show some data series as one shape and other data series as
*  another shape but use the same axis. This is commonly used when one series is of a
*  fundamentally different kind than the other series (for example, a projection or average)
*  but still has the same scale.
*  
*  To achieve a mixed plot like this, define it as a multi-axis chart as explained above, but
*  set MetricSettings.showAxis false to avoid a second axis appearing, and set
*  MetricSettings.matchGradations to cause the same gradations to be used for both
*  plots.
*  
*  See the ${isc.DocUtils.linkForExampleId('mixedPlotsChart', 'Mixed Plots')} SDK example.
*  
*  Histogram Charts
*  
*  A "histogram" chart is similar to a FacetChart.stacked "column" chart, showing
*  multiple facet values vertically for each position along the x-axis /
*  FacetChart.getDataLabelFacet, but instead of each vertical facet
*  value being defined only by a length, a "histogram" chart defines a segment for each,
*  represented by both a start point (the FacetChart.valueProperty) and
*  an end point (the FacetChart.endValueMetric).
*  
*  Segments may overlap, with the last segment drawn receiving the highest z-ordering. To
*  override this default behavior, values may be provided using an additional metric -
*  FacetChart.zIndexMetric - whose value must be a non-negative integer no greater than
*  FacetChart.maxDataZIndex.
*  
*  Scatter Charts
*  
*  Scatter charts differ from other chart types in that both axes represent continuous numeric
*  data rather than a discrete set of facet values (like months of the year). For this reason
*  Scatter charts use the same concept of a "metric" facet as is used by Dual-Axis charts,
*  where the metric facet is expected to have exactly two metrics: the
*  FacetChart.xAxisMetric and FacetChart.yAxisMetric.
*  
*  Unlike all other chart types, a scatter plot may be specified with only the metric facet.
*  However one additional facet can be defined, which allows multiple sets of x,y points to be
*  drawn in different colors, analogous to the different colors of a multi-series line chart.
*  
*  See the ${isc.DocUtils.linkForExampleId('scatterPlotCharting', 'Scatter Plot')} SDK example.
*  
*  Date values on the X axis
*  
*  FacetChart also supports scatter charts where the x-axis represents date- or time-valued
*  data and the y-axis represents numeric data, as normal. To enable this mode all records in
*  the data must have values for the facetValueId of the
*  FacetChart.xAxisMetric that are true Date objects, not Strings or
*  nulls. For these charts, vertical lines are drawn to represent a sequence of
*  significant datetime values on the x-axis, such as the first day of the month or week. The
*  mechanism used to select these Dates and format them into the x-axis labels is the same
*  mechanism used by charts with FacetChart.labelCollapseMode set to
*  "time".
*  
*  Bubble Charts
*  
*  A "bubble" chart is a type of scatter chart where the size of each rendered data
*  point represents an additional metric value, allowing 3 continuous data values to be
*  visualized together. When using chartType:"Bubble", the additional metric 
*  is configured via FacetChart.pointSizeMetric. 
*  Points will be sized between the FacetChart.minDataPointSize and
*  FacetChart.maxDataPointSize, optionally with 
*  FacetChart.logScalePointSize. A legend will be included showing 
*  how point size represents data values, and a multi-facet Bubble chart can optionally use a 
*  different shape for each facetValue via 
*  FacetChart.useMultiplePointShapes.
*  
*  Variable-size points can also be used with other, non-scatter chart types (such as "Line"
*  or "Radar") when FacetChart.showDataPoints is enabled, by setting 
*  pointSizeMetric to the FacetValue.id of a Facet.values
*  of the metric facet. In this case, a legend for point sizes is not shown by default, but can
*  be enabled via FacetChart.showPointSizeLegend.
*  
*  Whenever drawing variable size data points, by default, the largest data points are drawn 
*  first so that smaller data points are less likely to be completely occluded by larger data 
*  points, but this can be disabled by setting 
*  FacetChart.autoSortBubblePoints to false. Visual
*  appearance of data points can be further customized by setting the 
*  FacetChart.bubbleProperties.
*  
*  See the ${isc.DocUtils.linkForExampleId('bubbleChart', 'Bubble Chart')} SDK example.
*  
*  Color Scale Charts
*  
*  FacetChart supports rendering an additional metric value as the color of each data
*  point. This feature requires that FacetChart.showDataPoints be
*  enabled and is configured via FacetChart.colorScaleMetric. Instead
*  of data points being drawn using a separate color for each facetValue of the
*  legend facet, the data points will be drawn using a color interpolated between the
*  FacetChart.scaleStartColor and
*  FacetChart.scaleEndColor, optionally with
*  FacetChart.logScalePointColor. A legend is included by default
*  via FacetChart.showColorScaleLegend that shows how the data
*  values are mapped to a color via a gradient over the range of colors used in the chart.
*  Visual appearance of data points in color scale charts can be further customized by setting
*  the FacetChart.bubbleProperties, just as with bubble charts.
*  
*  Note that when color is being used to show values of the colorScaleMetric then
*  color cannot be used to distinguish between different facetValues. Therefore
*  color scale charts cannot have a (non-metric) legend facet.
*  
*  See the ${isc.DocUtils.linkForExampleId('colorScaleChart', 'Color Scale Chart')} SDK example.
*  
*  Notes on printing
*  
*  FacetCharts support printing on all supported desktop browsers. When using Pro Edition or 
*  better with the SmartClient Server Framework installed, charts can also be exported to PDF 
*  via RPCManager.exportContent or to images via RPCManager.exportImage.
*/
interface FacetChart extends DrawPane, Chart, DataBoundComponent {
/**
* Default styleName for the chart.
* @type {CSSStyleName}
* @default "scChart"
*/
styleName?: CSSStyleName;

/**
* Property in each record that holds a data value. For databound charts, if
*  valueProperty isn't set in the chart instance, it will be auto-derived from
*  the DataSource fields. The first numeric-typed DataSource field will be assumed
*  to be the valueProperty.
*  
*  Not used if there is an inline facet, see Chart.data.
* @type {string}
* @default "_value"
*/
valueProperty?: string;

/**
* Dataset for this chart.
*  
*  Two basic formats are supported:
*  
*  
* - "Standard model": data is an array of CellRecords where each record
*  contains one data value. Each record also contains a property named after each facetId
*  whose value is a facetValueId from that facet.
*  
*  For example, with a facet with id "regions" and facetValues "west", "north" and "east", and
*  with FacetChart.valueProperty with it's default value "_value", the data property
*  could be:
*  
*   isc.Chart.create({
*     facets:[{ id:"regions" }],
*     data : [
*      {regions:"west", _value:4},
*      {regions:"north", _value:2},
*      {regions:"east", _value:5}
*     ]
*   })
*  If there were a second facet with id "product" and facetValues "cars" and "trucks", a Chart
*  with a complete set of values would be:
*  
*   isc.Chart.create({
*     facets:[{ id:"regions" }, { id:"product" }],
*     data : [
*      {product:"cars", regions:"west", _value:4},
*      {product:"cars", regions:"north", _value:2},
*      {product:"cars", regions:"east", _value:5},
*      {product:"trucks", regions:"west", _value:1},
*      {product:"trucks", regions:"north", _value:9},
*      {product:"trucks", regions:"east", _value:3}
*     ]
*   })
*  This 2 facet (or "2 dimensional") dataset, if rendered as a bar chart, would use stacked or
*  clustered bars and a legend.
*  
* 
*  
* - "Inlined facet": data is a single CellRecord or Array of CellRecords where
*  each record contains multiple data values. In this case, one facet definition is considered
*  "inlined", meaning that the facetValueIds from this facet appear as properties in each
*  record, and each such property holds one data value. For example, a complete chart
*  definition whose dataset is equivalent to the previous example would be:
*  
*   isc.Chart.create({
*     facets: [{ 
*      inlinedValues:true,
*      values : [ { id:"west" }, { id:"north" }, { id : "east" } ]
*     }],
*     data : { west:4, north:5, east:2 }
*   })
*  Note that the property "inlinedValues" must be set on the facet definition, and the set of
*  facetValueIds must be specified outside of the data array (with the "standard model",
*  facetValueIds can be automatically derived from the data). 
*  
*  A two facet chart with an inlined facet can be defined as follows:
*  
*   isc.Chart.create({
*     facets: [
*      { 
*       inlinedValues:true,
*       values : [ { id:"west" }, { id:"north" }, { id : "east" } ]
*      }, 
*      { id:"product" }
*     ],
*     data : [
*       { product:"cars", west:4, north:5, east:2 },
*       { product:"trucks", west:1, north:9, east:3 }
*     ]
*   })
*  
*  Comparing between the formats, the "standard model" format treats all facets identically,
*  which can be of use when integrating with server technology that likewise treats all facets
*  identically. The "inlined facet" format is a more compact data representation and allows
*  easier conversion from data displayed in a ListGrid.
* @type {Array<Partial<CellRecord>> | CellRecord}
* @default null
*/
data?: Array<Partial<CellRecord>> | CellRecord;

/**
* The DataSource that this component should bind to for default fields and for performing
*  DSRequest.
*  
*  Can be specified as either a DataSource instance or the String ID of a DataSource.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* An Array of facets, exactly analogous to CubeGrid.facets, except that:
*  
*  
* - the "inlinedValues" property can be set on a facet to change data representation
*  as described under Chart.data.
*  
* - for a non-inlined facet, Charts support auto-derivation of facetValues from the
*  data.
*  
*  
*  In all chart types except "Bubble" and "Scatter", the chart displays a value for each
*  discrete value of one facet (i.e. single-facet charts) or it displays a value for each
*  combination of discrete values of two facets (multi-facet charts). The two discrete
*  facets are the FacetChart.getDataLabelFacet and the
*  FacetChart.getLegendFacet. They are named based on where the
*  Facet.values of the facet appear in the chart. The facet whose values
*  are rendered as labels along the data axis or in the main chart area is the data label
*  facet, and the facet whose values are rendered in the legend is the legend facet.
*  
*  For single-facet charts, most chart types have a data label facet as the first facet
*  but no legend facet. Single-facet Pie charts have a legend facet as the first facet
*  but no data label facet. Bubble and Scatter plots may have a legend facet as the
*  second facet, after the metric facet.
*  
*  In all multi-facet charts, the data label facet is always first and the legend facet
*  is second. In most chart types the data label facet and the legend facet may be
*  swapped on the fly by the user clicking on the "Swap Facets" item of the context menu.
*  
*  For databound charts, FacetChart.facetFields may be specified instead of this property. If
*  both are provided, FacetChart.facetFields is definitive but Facet properties will be
*  picked up from FacetChart.facets also present in the FacetChart.facetFields.
* @type {Array<Partial<Facet>> | Facet}
* @default null
*/
facets?: Array<Partial<Facet>> | Facet;

/**
* Specifies the "id" of the default metric facet value. The default metric is used
*  with FacetChart.lowErrorMetric and FacetChart.highErrorMetric when showing error bars.
* @type {string}
* @default "metric"
*/
metricFacetId?: string;

/**
* See ChartType for a list of known types - Column, Bar, Line, Pie, Doughnut,
*  Area, Radar, and Histogram charts are supported.
* @type {ChartType}
* @default "Column"
*/
chartType?: ChartType;

/**
* Whether to use stacking for charts where this makes sense (column, area, pie, line and
*  radar charts). If stacked is not set and two facets are supplied, clustering is assumed.
*  If null (the default), line charts will be unstacked, and others will be stacked.
* @type {boolean}
* @default null
*/
stacked?: boolean;

/**
* Whether shapes are filled, for example, whether a multi-series line chart appears as a
*  stack of filled regions as opposed to just multiple lines.
*  
*  If unset, fills will be automatically used when there are multiple facets and stacking
*  is active (so Line and Radar charts will show stacked regions). 
*  
*  You can explicitly set filled:false to create multi-facet Line or Radar charts where
*  translucent regions overlap, or filled:true to fill in a single-facet Line or Radar
*  chart.
* @type {boolean}
* @default null
*/
filled?: boolean;

/**
* For multi-facet charts, render data values as a proportion of the sum of all data
*  values that have the same label.
*  
*  Gradation labels will be switched to show percentage instead of absolute values.
*  
*  This setting is valid only for Column, Bar, Area and Radar chart types and only in
*  FacetChart.stacked mode. Stacked columns will be as tall as the chart rect and
*  stacked bars will be as wide as the chart rect. Area and Radar charts will be
*  completely filled except for facet values where all values are 0.
* @type {boolean}
* @default null
*/
proportional?: boolean;

/**
* Default title for the value axis label when the chart is in
*  FacetChart.proportional. This title will be used unless the
*  FacetChart.getLegendFacet defines a
*  Facet.proportionalTitle.
* @type {string}
* @default "Percent"
*/
proportionalAxisLabel?: string;

/**
* Other ChartType that the end user will be allowed to switch to, using
*  the built-in context menu.
*  
*  The actual list of ChartTypes displayed in the context menu may be a subset of
*  allowedChartTypes, since the FacetChart will automatically disallow certain
*  modes that are clearly invalid, for example, not allowing switching to Pie mode if
*  either FacetChart.canZoom is enabled, or if the chart is FacetChart.extraAxisMetrics.
* @type {Array<Partial<ChartType>>}
* @default null
*/
allowedChartTypes?: Array<Partial<ChartType>>;

/**
* Title for the chart as a whole.
* @type {string}
* @default null
*/
title?: string;

/**
* Whether to show a title.
* @type {boolean}
* @default true
*/
showTitle?: boolean;

/**
* Horizontal alignment of the chart's FacetChart.title.
* @type {TitleAlign}
* @default "center"
*/
titleAlign?: TitleAlign;

/**
* if aligning the title left or right, the amount of space before (for left aligned)
*  or after (for right aligned) to pad the title from the border edge
* @type {Integer}
* @default "0"
*/
titlePadding?: Integer;

/**
* Properties for title label.
* @type {Partial<DrawLabel>}
* @default null
*/
titleProperties?: Partial<DrawLabel>;

/**
* The height of the bordered rect around the title - defaults to 0 (assuming no border)
* @type {Integer}
* @default "0"
*/
titleRectHeight?: Integer;

/**
* Properties for title background (if being drawn).
* @type {Partial<DrawLabel>}
* @default null
*/
titleBackgroundProperties?: Partial<DrawLabel>;

/**
* Properties for bottom boundary of the title area, when there is already an outer
*  container around the whole chart. see FacetChart.drawTitleBoundary
* @type {Partial<DrawLine>}
* @default null
*/
titleBoundaryProperties?: Partial<DrawLine>;

/**
* should a background color be set behind the Title. Use
*  FacetChart.titleBackgroundProperties to set these values if this is true.
* @type {boolean}
* @default null
*/
drawTitleBackground?: boolean;

/**
* Whether a boundary should be drawn below the title area for circumstances where the 
*  chart area already has an outer border. If the chart has no outer border, then the 
*  FacetChart.titleBackgroundProperties settings should be used instead.
* @type {boolean}
* @default null
*/
drawTitleBoundary?: boolean;

/**
* A label for the data values, such as "Sales in Thousands", typically used as the label for
*  the value axis.
* @type {string}
* @default null
*/
valueTitle?: string;

/**
* Whether to show the FacetChart.valueTitle (or, in the case of
*  FacetChart.proportional, the
*  FacetChart.proportionalAxisLabel) as a label on the value axis.
*  
*  Automatically disabled for non-rectangular charts (eg Pie, Radar).
* @type {boolean}
* @default null
*/
showValueAxisLabel?: boolean;

/**
* Properties for labels of value axis.
* @type {Partial<DrawLabel>}
* @default null
*/
valueAxisLabelProperties?: Partial<DrawLabel>;

/**
* Whether to show a label for the data axis as a whole (the data axis is where labels for
*  each data point appear). If true, Facet.title for the data label facet will be
*  shown as the label.
*  
*  Automatically disabled for non-rectangular charts (eg Pie, Radar).
* @type {boolean}
* @default null
*/
showDataAxisLabel?: boolean;

/**
* Properties for labels of data axis.
* @type {Partial<DrawLabel>}
* @default null
*/
dataAxisLabelProperties?: Partial<DrawLabel>;

/**
* Ideal number of pixels to leave between each gradation on the primary axis, which is
*  typically the y (vertical) axis except for Bar charts.
*  
*  The chart will detect the range of values being displayed and available pixels on the
*  vertical axis, and generate gradations that are spaced approximately
*  pixelsPerGradations apart. Note that the Framework will attempt to
*  approach the specified target gap from above - the chart will never be drawn with
*  gradations spaced closer than pixelsPerGradation.
* @type {Integer}
* @default "28"
*/
pixelsPerGradation?: Integer;

/**
* Ideal number of pixels to leave between each gradation on the x (horizontal axis), for
*  Scatter plots only.
*  
*  Defaults to the value of FacetChart.pixelsPerGradation if unset.
* @type {Integer}
* @default null
*/
otherAxisPixelsPerGradation?: Integer;

/**
* Candidate gradation gaps to evaluate when trying to determine what gradations should be
*  displayed on the primary axis, which is typically the y (vertical) axis except for Bar
*  charts.
*  
*  Candidates are expressed as a series of numbers between 1 and 10, representing boundaries
*  within a given order of magnitude (power of 10).
*  
*  For example, the setting [1, 2.5, 5] means that, for a chart showing values that are only
*  between 0 and 1, gradations of 0.1, 0.25 and 0.5 would be evaluated to see which is a
*  closer fit given the FacetChart.pixelsPerGradation setting and the chart's height. The same
*  setting, with a chart showing values from 0 to 1,000,000 would imply that gradation gaps
*  of 100,000, 250,000 and 500,000 would be evaluated.
* @type {Array<Partial<Float>>}
* @default "[1, 2, 5]"
*/
gradationGaps?: Array<Partial<Float>>;

/**
* Like FacetChart.gradationGaps, except allows control of gradations for the X
*  (horizontal) axis, for Scatter charts only.
*  
*  See also FacetChart.otherAxisGradationTimes for control of gradations when the X
*  axis is time-valued.
*  
*  Defaults to the value of FacetChart.pixelsPerGradation if unset.
* @type {Array<Partial<Float>>}
* @default null
*/
otherAxisGradationGaps?: Array<Partial<Float>>;

/**
* For charts that have a date/time-valued X-axis, gradations can instead be specified as
*  Strings, consisting of a number and trailing letter code, where the letter code indicates
*  the unit of time. Valid time units are "ms" (millisecond), "s" (second), "mn" (minute),
*  "h" (hour), "d" (day), "w" (week), "m" (month), "q" (quarter, 3-months), "y" (year).
*  
*  When time units are used, there is no way to scale the same unit to a much larger or
*  smaller range of time (as there is with numeric gradations). For example, a setting of
*  "30mn" meaning "30 minutes" does not mean that 30 hours is a natural choice for chart
*  with a longer timeline (days should obviously be chosen instead). Therefore, when
*  specifying time gradations, candidate gradations must be provided for the entire possible
*  displayed range. If insufficient gradations are specified, this can result in unreadable
*  charts; for example, if the largest available gradation is "15mn" and the chart is
*  showing a full week's data in around 500px, there will be more than one gradation per
*  pixel, and labels will be drawn on top of each other.
*  
*  To prevent this, be sure to specify enough gradations to cover the all time ranges your
*  chart may need to display. However, if gradations are not specified for granularities
*  under 1 second or over 1 year, further gradations will be chosen based on using
*  FacetChart.otherAxisGradationGaps to choose fractions of seconds or multiples of years.
*  
*  The default setting is effectively:
*  
*  ["1s", "15s", "30s", "1mn", "5mn", "15mn", "30mn", "1h", "1d", "1w", "1m", "1q", "1y"]
*  
* @type {Array<Partial<string>>}
* @default null
*/
otherAxisGradationTimes?: Array<Partial<string>>;

/**
* If all data values would be spread across less than FacetChart.minDataSpreadPercent of the
*  axis, the start values of axes will be automatically adjusted to make better use of
*  space.
*  
*  For example, if a column chart has all data values between 500,000 and 500,100, if the
*  axis starts at 0, differences in column heights will be visually indistinguishable.
*  In this case, since all data values appear in well under 30% of the axis length, the
*  default minDataSpreadPercent setting would cause the axis to start at a
*  value that would make the column heights obviously different (for example, starting the
*  axis as 500,000).
*  
*  Setting an explicit FacetChart.axisStartValue or FacetChart.axisEndValue,
*  disables this behavior, as does setting
*  minDataSpreadPercent to 0.
*  
*  For multi-axis charts, use MetricSettings.minDataSpreadPercent for per-axis
*  settings.
*  
*  For Bubble and Scatter charts, minDataSpreadPercent affects only the y-axis
*  of the chart. The property FacetChart.minXDataSpreadPercent must be used to enable the
*  corresponding feature for the x-axis.
* @type {Integer}
* @default "30"
*/
minDataSpreadPercent?: Integer;

/**
* Start value for the primary axis of the chart.
*  
*  If set to an explicit value, this will be respected.
*  If unset, the axis start value will default to 0, or to a value that makes good
*  use of vertical space based on FacetChart.minDataSpreadPercent.
*  
*  For multi-axis charts, Bubble charts, and Scatter charts, the
*  facetChart.axisStartValue affects only the first axis of the chart.
*  Start values for other axes of multi-axis charts can be set on a per-axis basis via
*  MetricSettings.axisStartValue. For Scatter charts, the FacetChart.xAxisStartValue
*  property must be used to set the start value of the x-axis.
*  
*  Note that if this chart's data includes points that fall below this value, they
*  are ommitted and effectively treated as null values. For charts showing a data line,
*  developers may wish to set FacetChart.discontinuousLines to true in this case.
* @type {Float}
* @default null
*/
axisStartValue?: Float;

/**
* End value for the primary axis of the chart.
*  
*  If set to an explicit value, this will be respected.
*  If unset, the axis end value will default to a value large enough to the
*  largest data point, rounded up to the nearest (next) gradation.
*  
*  For multi-axis charts, Bubble charts, and Scatter charts, the
*  facetChart.axisEndValue affects only the first axis of the chart.
*  End values for other axes of multi-axis charts can be set on a per-axis basis via
*  MetricSettings.xAxisEndValue. For Scatter charts, the FacetChart.xAxisEndValue
*  property must be used to set the end value of the x-axis.
*  
*  Note that if this chart's data includes points that fall above this value, they
*  are ommitted and effectively treated as null values. For charts showing a data line,
*  developers may wish to set FacetChart.discontinuousLines to true in this case.
* @type {Float}
* @default null
*/
axisEndValue?: Float;

/**
* For Bubble and Scatter charts only, the start value for the x-axis.
*  
*  Defaults to 0, or to a value that makes good use of horizontal space based on
*  FacetChart.minXDataSpreadPercent.
*  
*  If the x-axis metric is date-valued, this value should be a date (typically
*  applies to Scatter charts only).
* @type {Float | Date}
* @default null
*/
xAxisStartValue?: Float | Date;

/**
* For Bubble and Scatter charts only, the end value for the x-axis.
*  
*  If set to an explicit value, this will be respected.
*  If unset, the axis end value will default to a value large enough to show the
*  largest data point.
*  
*  If the x-axis metric is date-valued, this value should be a date (typically
*  applies to Scatter charts only).
* @type {Float | Date}
* @default null
*/
xAxisEndValue?: Float | Date;

/**
* For scatter charts only, if all data points would be spread across less than
*  FacetChart.minXDataSpreadPercent of the x-axis, the start value of x-axis will be automatically
*  adjusted to make better use of space.
*  
*  Setting an explicit FacetChart.xAxisStartValue disables this behavior, as does setting
*  minXDataSpreadPercent to 0.
* @type {Integer}
* @default "30"
*/
minXDataSpreadPercent?: Integer;

/**
* Properties for gradation labels
* @type {Partial<DrawLabel>}
* @default null
*/
gradationLabelProperties?: Partial<DrawLabel>;

/**
* Properties for gradation lines
* @type {Partial<DrawLine>}
* @default null
*/
gradationLineProperties?: Partial<DrawLine>;

/**
* Properties for the gradation line drawn for zero (slightly thicker by default).
* @type {Partial<DrawLine>}
* @default null
*/
gradationZeroLineProperties?: Partial<DrawLine>;

/**
* Whether to show alternating color bands in the background of chart. See
*  FacetChart.backgroundBandProperties.
* @type {boolean}
* @default true
*/
bandedBackground?: boolean;

/**
* Properties for background band
* @type {Partial<DrawRect>}
* @default null
*/
backgroundBandProperties?: Partial<DrawRect>;

/**
* Whether to show gradation labels in radar charts.
* @type {boolean}
* @default true
*/
showRadarGradationLabels?: boolean;

/**
* For scatter charts only, the "id" of the metric facet value to use for the x-axis.
*  
*  The default x-axis metric is the second value of the metric facet.
* @type {string}
* @default null
*/
xAxisMetric?: string;

/**
* For scatter charts only, the "id" of the metric facet value to use for the y-axis.
*  
*  The default y-axis metric is the first value of the metric facet.
* @type {string}
* @default null
*/
yAxisMetric?: string;

/**
* Whether to draw lines between adjacent data points in "Scatter" plots. See also
*  DataLineType for enabling smoothing.
* @type {boolean}
* @default false
*/
showScatterLines?: boolean;

/**
* How to draw lines between adjacent data points in Line and Scatter charts.
*  See DataLineType.
*  
*  Does not apply to boundary lines for shapes in Area or Radar plots.
* @type {DataLineType}
* @default null
*/
dataLineType?: DataLineType;

/**
* The "id" of the metric facet value that assigns a probability to each combination
*  of facets and their values. Each probability must be a non-negative number.
*  These probabilities are used by all methods of FacetChart that calculate statistical
*  values (e.g. FacetChart.getMean and FacetChart.getStdDev). The default value of
*  this property is null which causes the FacetChart to assign probabilities to the
*  data records according to a uniform probability distribution.
*  
*  Note that the FacetChart handles cases where the sum total of all probabilities
*  in the FacetChart.data is not exactly one by scaling the assigned probabilities.
* @type {string}
* @default null
*/
probabilityMetric?: string;

/**
* If set, the FacetChart.showExpectedValueLine,
*  FacetChart.showStandardDeviationLines,
*  FacetChart.bandedStandardDeviations,
*  and FacetChart.showRegressionLine are drawn on top of the data rather
*  than underneath.
* @type {boolean}
* @default false
*/
showStatisticsOverData?: boolean;

/**
* Display a line at the FacetChart.getMean.
*  
*  Note that this expected value is computed using all of the data points, pooled across all
*  facets. The computation relies only on the values of the main value axis metric and the
*  FacetChart.probabilityMetric.
*  
*  See http://en.wikipedia.org/wiki/Expected_value.
* @type {boolean}
* @default false
*/
showExpectedValueLine?: boolean;

/**
* Properties for the FacetChart.showExpectedValueLine.
*  
*  Note that for rectangular charts the properties are for a DrawLine,
*  and for radar charts the properties are for a DrawOval.
* @type {Partial<DrawItem>}
* @default null
*/
expectedValueLineProperties?: Partial<DrawItem>;

/**
* Display multiple FacetChart.getStdDev away from the mean as lines.
*  The exact deviations to display can be customized with FacetChart.standardDeviations.
*  
*  Note that these standard deviations are computed using all of the data points, pooled
*  across all facets. The computation relies only on the values of the main value axis
*  metric and the FacetChart.probabilityMetric.
*  
*  See http://en.wikipedia.org/wiki/Standard_deviation.
* @type {boolean}
* @default false
*/
showStandardDeviationLines?: boolean;

/**
* Properties for the FacetChart.showStandardDeviationLines.
*  
*  Note that for rectangular charts the properties are for a DrawLine,
*  and for radar charts the properties are for a DrawOval.
* @type {Partial<DrawItem>}
* @default null
*/
standardDeviationLineProperties?: Partial<DrawItem>;

/**
* Whether to display both the positive and negative of the
*  FacetChart.standardDeviations.
* @type {boolean}
* @default true
*/
useSymmetricStandardDeviations?: boolean;

/**
* When FacetChart.showStandardDeviationLines is set, the number of standard deviation lines drawn
*  and their respective standard deviation away from the mean are specified by this property.
*  The default is to display lines corresponding to the mean plus or minus one standard
*  deviation.
*  
*  Note that having zero in this list of standard deviations is identical to drawing
*  a line at the mean.
*  
*  For example assume that chart1 and chart2 both plot data with mean 1 and standard deviation 0.1.
*  chart1 will draw a blue line at the value 1 and two red lines at the values 0.7 and 1.2.
*  chart2 will draw three red lines at values 0.9, 1.0, and 1.1.
*  
*  
*  
*  isc.FacetChart.create({
*    ID: "chart1",
*    standardDeviations: [-3, 2],
*    showExpectedValueLine: true,
*    showStandardDeviationLines: true,
*    expectedValueLineProperties: { lineColor: "blue" },
*    standardDeviationLineProperties: { lineColor: "red" },
*    // ...
*  });
* 
*  isc.FacetChart.create({
*    ID: "chart2",
*    standardDeviations: [-1, 0, 1],
*    showExpectedValueLine: false,
*    showStandardDeviationLines: true,
*    expectedValueLineProperties: { lineColor: "blue" },
*    standardDeviationLineProperties: { lineColor: "red" },
*    // ...
*  });
*  
*  
*  
* @type {Array<Partial<Float>>}
* @default "[-1, 1]"
*/
standardDeviations?: Array<Partial<Float>>;

/**
* Whether to show color bands between the FacetChart.standardDeviations
*  lines.
*  
*  Standard deviation bands are not available for pie or radar charts.
* @type {boolean}
* @default false
*/
bandedStandardDeviations?: boolean;

/**
* An Array of DrawRect properties to specify the bands between the FacetChart.showStandardDeviationLines.
*  The length of the Array must be one less than the length of the FacetChart.standardDeviations Array.
*  
*  Having no band between certain standard deviations from the mean can be specified by
*  having a null element at the corresponding index of this Array.
*  
*  Note that if FacetChart.useSymmetricStandardDeviations is set then for each standard
*  deviation band that is drawn a corresponding band will also be drawn on the opposite side of the mean line.
* @type {Array<Partial<Partial<DrawItem>>>}
* @default null
*/
standardDeviationBandProperties?: Array<Partial<Partial<DrawItem>>>;

/**
* For scatter plots only, whether to display a regression curve that best fits the data of
*  the two metric facet values.
*  
*  The type of regression curve used depends on the RegressionLineType property,
*  which can be:
*  
*  
* - "line" &ndash; to draw a linear regression curve, or
*  
* - "polynomial" &ndash; to draw a polynomial regression curve (of degree
*  FacetChart.regressionPolynomialDegree).
* - 
*  
*  
*  Note that the regression is computed using all of the data points and it does not depend
*  on the values of any non-metric facets. For example, adding a legend facet will not
*  change the regression curve.
*  
*  See http://en.wikipedia.org/wiki/Simple_linear_regression.
*  See http://en.wikipedia.org/wiki/Polynomial_regression.
* @type {boolean}
* @default false
*/
showRegressionLine?: boolean;

/**
* Properties for the FacetChart.showRegressionLine.
* @type {Partial<DrawLine>}
* @default null
*/
regressionLineProperties?: Partial<DrawLine>;

/**
* Regression algorithm used for the FacetChart.showRegressionLine.
* @type {RegressionLineType}
* @default "line"
*/
regressionLineType?: RegressionLineType;

/**
* For scatter plots only, specify the degree of polynomial to use for any polynomial regression
*  that is calculated.
* @type {number}
* @default "3"
*/
regressionPolynomialDegree?: number;

/**
* For Line, Area, Radar, Scatter or Bubble charts, whether to show data points for each
*  individual data value.
*  
*  If shown, the FacetChart.pointClick and FacetChart.getPointHoverHTML APIs can be used to create
*  interactivity.
* @type {boolean}
* @default false
*/
showDataPoints?: boolean;

/**
* Size in pixels for data points drawn for line, area, radar and other chart types.
* @type {Integer}
* @default "5"
*/
dataPointSize?: Integer;

/**
* Common properties to apply for all data points (see FacetChart.showDataPoints).
* @type {Partial<DrawItem>}
* @default null
*/
dataPointProperties?: Partial<DrawItem>;

/**
* For charts where FacetChart.showDataPoints is enabled, this property specifies an array of
*  geometric shapes to draw for the data points of each series.
* @type {Array<Partial<PointShape>>}
* @default "["Oval", "Square", "Diamond", "Triangle"]"
*/
pointShapes?: Array<Partial<PointShape>>;

/**
* For charts where FacetChart.showDataPoints is enabled, this property specifies an additional
*  metric (i.e. an "id" of a metric facet value) that determines the size of the data
*  points drawn. For example, when a circle is drawn to represent a data point then
*  the size of the data point is the diameter of the circle, in pixels.
*  
*  The size is calculated by linearly scaling the value of the
*  pointSizeMetric of the point between the FacetChart.minDataPointSize and
*  FacetChart.maxDataPointSize. The data point that has the lowest value for the
*  pointSizeMetric will be drawn as a shape minDataPointSize
*  pixels in size, and the data point that has the highest value for the
*  pointSizeMetric will be drawn as a shape maxDataPointSize
*  pixels in size.
*  
*  Using a log-scale to calulate the size of the data points is achieved by enabling
*  FacetChart.logScalePointSize.
*  
*  If the ChartType is "Bubble" then the default
*  pointSizeMetric is "pointSize".
*  
*  Note that setting pointSizeMetric to non-null implicitly
*  enables FacetChart.showDataPoints.
* @type {string}
* @default null
*/
pointSizeMetric?: string;

/**
* The minimum allowed data point size when controlled by FacetChart.pointSizeMetric.
* @type {Float}
* @default "3"
*/
minDataPointSize?: Float;

/**
* The maximum allowed data point size when controlled by FacetChart.pointSizeMetric.
* @type {Float}
* @default "14"
*/
maxDataPointSize?: Float;

/**
* Whether to use logarithmic scaling for the FacetChart.pointSizeMetric.
*  Defaults to the value of FacetChart.logScale.
* @type {boolean}
* @default false
*/
logScalePointSize?: boolean;

/**
* When FacetChart.logScalePointSize is true, base value for logarithmic point size metric values.
* @type {Integer}
* @default "10"
*/
pointSizeLogBase?: Integer;

/**
* Properties for the shapes displayed around the data points (for example, in a bubble
*  chart).
*  
*  When either the FacetChart.pointSizeMetric or the
*  FacetChart.colorScaleMetric is active the default
*  bubbleProperties displays each data points with a linear gradient.
* @type {Partial<DrawItem>}
* @default null
*/
bubbleProperties?: Partial<DrawItem>;

/**
* Setting this flag to false prevents the chart from drawing fill gradients
*  into the bubbles of each data point. This flag is required to be set for IE8 and
*  earlier in order to draw bubble charts displaying high volumes of data.
* @type {boolean}
* @default "!(isc.Browser.isIE &amp;&amp; isc.Browser.version &lt;= 8)"
*/
allowBubbleGradients?: boolean;

/**
* Whether the chart should use multiple shapes to show data points. If set to
*  true then the chart is allowed to use all
*  FacetChart.pointShapes: circles, squares, diamonds, and triangles. If
*  set to false then just the first supported shape (circles, for example)
*  will be used. The default is false for bubble charts and
*  FacetChart.colorScaleMetric and true for all other chart
*  types.
* @type {boolean}
* @default null
*/
useMultiplePointShapes?: boolean;

/**
* Whether to show an additional legend to the right of the chart to indicate
*  FacetChart.pointSizeMetric. The default is true for bubble charts and
*  false for all other chart types.
* @type {boolean}
* @default null
*/
showPointSizeLegend?: boolean;

/**
* When a FacetChart.showPointSizeLegend is shown, this property controls
*  the number of gradations of the FacetChart.pointSizeMetric that the chart
*  tries to display.
*  
*  Note that if FacetChart.usePointSizeLogGradations is set then the number of gradations is
*  not given by this property but rather by the entries of FacetChart.pointSizeLogGradations.
* @type {Integer}
* @default "5"
*/
pointSizeGradations?: Integer;

/**
* Whether to use classic logarithmic gradations, where each order of magnitude is shown
*  as a gradation as well as a few intervening values, for the
*  FacetChart.pointSizeMetric values displayed in the
*  FacetChart.showPointSizeLegend. Gradations also begin and end on an
*  order of magnitude. For example, 1, 2, 4, 6, 8, 10, 20, 40, 60, 80, 100.
*  
*  Default gradations can be overridden via FacetChart.pointSizeLogBase and
*  FacetChart.pointSizeLogGradations.
* @type {boolean}
* @default false
*/
usePointSizeLogGradations?: boolean;

/**
* When FacetChart.usePointSizeLogGradations is set, this property specifies the
*  FacetChart.pointSizeMetric value gradations to show in the
*  FacetChart.showPointSizeLegend in between powers, expressed as a series
*  of integer or float values between 1 and FacetChart.pointSizeLogBase.
* @type {Array<Partial<Float>>}
* @default "[1, 5]"
*/
pointSizeLogGradations?: Array<Partial<Float>>;

/**
* Whether to draw multiple bubble legends horizontally stacked to the right of the chart,
*  one per shape type.
*  
*  Note that this setting has no effect if FacetChart.useMultiplePointShapes is disabled.
* @type {boolean}
* @default false
*/
showBubbleLegendPerShape?: boolean;

/**
* Whether to draw data points in order of descending FacetChart.pointSizeMetric
*  so that small values are less likely to be completely occluded by larger values. Set
*  this to false to draw the data points in the same order that they appear
*  in the data.
* @type {boolean}
* @default true
*/
autoSortBubblePoints?: boolean;

/**
* For charts where FacetChart.showDataPoints is enabled, this property specifies an additional
*  metric (i.e. an "id" of a metric facet value) that causes the data points to be
*  colored from FacetChart.scaleStartColor to FacetChart.scaleEndColor based on a linear scale
*  over the values of this metric. Log-scaling for color scale is also supported with
*  FacetChart.logScalePointColor.
* @type {string}
* @default null
*/
colorScaleMetric?: string;

/**
* Whether to use logarithmic scaling for the FacetChart.colorScaleMetric of the
*  data points. Defaults to the value of FacetChart.logScale.
* @type {boolean}
* @default false
*/
logScalePointColor?: boolean;

/**
* When FacetChart.logScalePointColor is true, this property specifies the base
*  value for logarithmic FacetChart.colorScaleMetric values.
* @type {Integer}
* @default "10"
*/
pointColorLogBase?: Integer;

/**
* Whether to show an additional legend underneath the chart to indicate color values.
*  The default is true if a valid FacetChart.colorScaleMetric is
*  specified.
* @type {boolean}
* @default null
*/
showColorScaleLegend?: boolean;

/**
* The starting color of the color scale when the data points are colored according to a
*  FacetChart.colorScaleMetric. If neither this property nor the
*  FacetChart.scaleEndColor is set then the whole color range is used by default.
*  
*  Note that using CSS color shortcuts (e.g. "lightblue") is not allowed for this property.
* @type {CSSColor}
* @default null
*/
scaleStartColor?: CSSColor;

/**
* The ending color of the color scale when the data points are colored according to a
*  FacetChart.colorScaleMetric. If neither this property nor the
*  FacetChart.scaleStartColor is set then the whole color range is used by
*  default.
*  
*  Note that using CSS color shortcuts (e.g. "lightblue") is not allowed for this property.
* @type {CSSColor}
* @default null
*/
scaleEndColor?: CSSColor;

/**
* Whether to use logarithmic scaling for values.
*  
*  Logarithmic scale charts show an equivalent percentage increase as equivalent distance
*  on the chart. That is, 10 and 100 are the same distance apart as 100 and 1000 (each
*  being a 10 times or 1000% increase).
* @type {boolean}
* @default false
*/
logScale?: boolean;

/**
* Whether to use classic logarithmic gradations, where each order of magnitude is shown as
*  a gradation as well as a few intervening lines.  Gradations also begin and end on an order
*  of magnitude. For example, 1,2,4,6,8,10,20,40,60,80,100.
*  
*  Default gradations can be overridden via FacetChart.logBase and FacetChart.logGradations.
* @type {boolean}
* @default false
*/
useLogGradations?: boolean;

/**
* When FacetChart.useLogGradations, base value for logarithmic gradation lines. Gradation
*  lines will be shown at every power of this value plus intervening values specified by
*  FacetChart.logGradations.
* @type {Integer}
* @default "10"
*/
logBase?: Integer;

/**
* When FacetChart.useLogGradations is set, gradation lines to show in between powers,
*  expressed as a series of integer or float values between 1 and FacetChart.logBase.
*  
*  Some common possibilities (for base 10):
*  
*   [ 1 ] // show only orders of magnitude (0.1, 1, 10, 100, etc)
*   [ 1, 5 ] // show only orders of magnitude plus halfway mark
*   [ 1, 2, 4, 8 ] // show powers of 2 between orders
*   [ 1, 2.5, 5, 7.5 ] // show quarters
*  
*  Or base 2:
*  
*   [ 1 ]
*   [ 1, 1.5 ]
*  
* @type {Array<Partial<Float>>}
* @default "[ 1,2,4,6,8 ]"
*/
logGradations?: Array<Partial<Float>>;

/**
* Whether to show a rectangular shape around the area of the chart where data is plotted.
* @type {boolean}
* @default false
*/
showChartRect?: boolean;

/**
* Properties for chart rect. By default, DrawRect.rounding of the chart
*  rect. causes the gradation lines to be automatically inset from the edge so that they
*  do not run right along the curve. Set FacetChart.padChartRectByCornerRadius to
*  false to change this default.
* @type {Partial<DrawRect>}
* @default null
*/
chartRectProperties?: Partial<DrawRect>;

/**
* If FacetChart.showChartRect is enabled and if FacetChart.chartRectProperties specifies a
*  nonzero DrawRect.rounding, whether the padding around the inside of the
*  chart rect. should include at least the radius of the rounded corner.
* @type {boolean}
* @default true
*/
padChartRectByCornerRadius?: boolean;

/**
* Margin around the main chart rect: between title and chart, between chart and axis
*  labels, and chart rect and right edge of chart.
* @type {Integer}
* @default "5"
*/
chartRectMargin?: Integer;

/**
* For rectangular charts (bar, column, line), margin around the inside of the
*  main chart area, so that data elements are not flush to edge.
* @type {Integer}
* @default "10"
*/
dataMargin?: Integer;

/**
* This property controls whether to rotate the labels on the X-axis. If rotateLabels is
*  "always" then all of the data labels will be rotated by 90 degrees. If rotateLabels is
*  "auto" (the default) then the labels will only be rotated if it is required in order for
*  the labels to be legible and non-overlapping. If rotateLabels is "never" then the
*  labels will not be rotated.
*  
*  Note that automatic rotation is incompatible with setting
*  
*  FacetChart.getMinClusterSize, so that "auto" will be treated as "never"
*  
*  if that method has been specified on a column, bar, or histogram chart.
* @type {LabelRotationMode}
* @default "auto"
*/
rotateLabels?: LabelRotationMode;

/**
* This property controls whether to rotate the labels on the
*  FacetChart.getDataLabelFacet of radar or FacetChart.stacked pie charts so that
*  each label is parallel to its radial gradation (these are the labels that appear around
*  the perimeter). For now, "auto" means the same thing as "always" - but this may change
*  in the future if heuristics are added to determine when the affected labels are likely to
*  overlap and not be legible. If rotateLabels is "never" then the labels will not be
*  rotated.
*  
* @type {LabelRotationMode}
* @default "auto"
*/
radarRotateLabels?: LabelRotationMode;

/**
* Whether to automatically rotate labels if needed in order to make them legible and
*  non-overlapping.
* @deprecated * As of SmartClient 9.0 this property is replaced by the property FacetChart.rotateLabels. Setting rotateLabels to "auto" is equivalent to setting
*  autoRotateLabels to true. Setting rotateLabels to "never" is equivalent
*  to setting autoRotateLabels to false.

* @type {boolean}
* @default true
*/
autoRotateLabels?: boolean;

/**
* This property controls whether to rotate the labels shown for data-values in 
*  ChartType. "auto" will rotate all data-values if any of them 
*  are wider than their columns. In all cases, whether rotated or not, data-values are 
*  hidden and instead shown in hovers if any of them exceed their bar's width.
* @type {LabelRotationMode}
* @default "auto"
*/
rotateDataValues?: LabelRotationMode;

/**
* What to do when there are too many data points to be able to show labels for every data point
*  at the current chart size - see LabelCollapseMode.
*  
*  Each of the possible strategies is re-applied when the user resizes the chart as a whole, so if
*  labels are omitted the user can make them visible via resize or zoom.
*  
*  If the labelCollapseMode is "numeric" then vertical lines will be drawn at gradation
*  values automatically chosen by the chart.
*  
*  If the labelCollapseMode is "time" then vertical lines are drawn to represent a
*  sequence of significant datetime values on the x-axis, such as the first day of the
*  month or week. The chart automatically chooses the sequence of Dates such that the
*  spacing between them expresses the smallest granularity of time possible while still
*  allowing the axis labels to make good use of the space. If, for example, the Date
*  values in the data span a few years in time then the chart may select January 1 of the
*  same year of the earliest data point and every January 1 thereafter (in range of the
*  data) as the sequence of Dates and label each Date by the four-digit year. If the time
*  span of the data values is on the order of minutes then the chart may select multiples
*  of 15 minutes as the seqeunce of Dates. FacetChart currently supports the following
*  granularities of time: years, quarters, months, weeks, days, hours, half-hours,
*  quarter-hours, 5 minutes, minutes, 30 seconds, and 15 seconds.
*  
*  The format of the Date labels is fixed by FacetChart. In particular,
*  FacetChart.formatAxisValue
*  will not be called on values for the x-axis. However, FacetChart uses 
*  DateUtil.shortMonthNames
*  
*  for the time granularities of quarters, months, and weeks, uses the
*  Time.setShortDisplayFormat
*  
*  to format labels for time granularities from minutes to hours, and uses the
*  Time.setNormalDisplayFormat
*  
*  to format labels for the time granularities of 15 seconds and 30 seconds. The label
*  format can be customized by changing these three formatters. Also note that for the time
*  granularity of weeks the sequence of Dates will be the first day of each week, as
*  specified by DateUtil.setFirstDayOfWeek.
*  Note that if the labelCollapseMode is "time" or "numeric" then the FacetChart.data must
*  be initially sorted with the FacetChart.getDataLabelFacet's values in
*  ascending order.
* @type {LabelCollapseMode}
* @default null
*/
labelCollapseMode?: LabelCollapseMode;

/**
* Minimum gap between labels on the X axis before some labels are omitted or larger time
*  granularity is shown (eg show days instead of hours) based on the
*  FacetChart.labelCollapseMode.
*  
*  Default is based on label orientation. If labels are vertical, the minimum gap is the height
*  of half a line of text. If horizontal it's the width of 4 "X" letters.
* @type {Integer}
* @default null
*/
minLabelGap?: Integer;

/**
* Properties for data label
* @type {Partial<DrawLabel>}
* @default null
*/
dataLabelProperties?: Partial<DrawLabel>;

/**
* If set to false, data labels for values are entirely omitted.
*  
*  This property would generally only be set to false if several small charts
*  are shown together and the data labels are drawn elsewhere on the screen (above an
*  entire stack of charts, for instance) or are otherwise implicit.
* @type {boolean}
* @default true
*/
showDataLabels?: boolean;

/**
* Default precision used when formatting float numbers for axis labels
* @type {Integer}
* @default "2"
*/
decimalPrecision?: Integer;

/**
* Enables "zooming" on the X axis, specifically, only a portion of the overall dataset is shown
*  in the main chart, and a FacetChart.zoomChart appears with slider controls
*  allowing a range to be selected for display in the main chart.
*  
*  A FacetChart.labelCollapseMode is automatically enabled if
*  unset and is based on the type of the first non-null data value.
* @type {boolean}
* @default null
*/
canZoom?: boolean;

/**
* Mini-chart created to allow zooming when FacetChart.canZoom is enabled.
*  
*  This chart automatically has certain visual tweaks applied, including FacetChart.showInlineLabels,
*  FacetChart.zoomMutePercent and FacetChart.zoomLogScale. It can be
*  further configured via FacetChart.zoomChartProperties.
*  
*  The selected range from this chart defaults to being shown with distinct styling as well (if
*  FacetChart.zoomShowSelection is set), which can be controlled via
*  FacetChart.zoomSelectionChartProperties.
* @type {FacetChart}
* @default null
*/
zoomChart?: FacetChart;

/**
* Properties to further configure the FacetChart.zoomChart.
* @type {Partial<FacetChart>}
* @default null
*/
zoomChartProperties?: Partial<FacetChart>;

/**
* Mini-chart created when FacetChart.canZoom is enabled. This chart represents the currently
*  selected range of data shown in the main chart.
* @type {FacetChart}
* @default null
*/
zoomSelectionChart?: FacetChart;

/**
* Properties to further configure the FacetChart.zoomSelectionChart.
* @type {Partial<FacetChart>}
* @default null
*/
zoomSelectionChartProperties?: Partial<FacetChart>;

/**
* Whether the selected range should be shown in a different style, which can be configured via
*  FacetChart.zoomSelectionChartProperties. This has performance consequences and makes the rendering
*  of the mini-chart slightly slower.
* @type {boolean}
* @default true
*/
zoomShowSelection?: boolean;

/**
* Slider controls shown on the mini-chart which is created when FacetChart.canZoom is enabled.
* @type {RangeSlider}
* @default null
*/
zoomChartSlider?: RangeSlider;

/**
* Height of the FacetChart.zoomChart. The zoomChart is always as wide as the main chart.
* @type {Float}
* @default "100"
*/
zoomChartHeight?: Float;

/**
* By default when FacetChart.canZoom is enabled, the FacetChart.zoomChart uses logarithmic scaling so
*  that spikes in the data don't result in a zoomed chart that is mostly a flat line.
*  
*  Logarithmic scaling is automatically disabled if the dataset spans zero (eg, has negative and
*  positive values) as this can't be shown in a logarithmic scale.
*  
*  Set zoomLogScale to explicitly enable or disable logarithmic scaling.
* @type {boolean}
* @default null
*/
zoomLogScale?: boolean;

/**
* Should be set to a number between -100 and 100. If set, all colors in the chart are "muted" by
*  this percentage by shifting them toward white (or for negative numbers, toward black).
* @type {Float}
* @default null
*/
colorMutePercent?: Float;

/**
* FacetChart.colorMutePercent to use for the FacetChart.zoomChart.
* @type {Float}
* @default "-35"
*/
zoomMutePercent?: Float;

/**
* For a FacetChart.canZoom, determines what portion of the overall dataset should be
*  initially shown in the main chart.
*  
*  Default is to show the end of the dataset if the X axis shows time and includes today's date,
*  otherwise to show the start of the dataset.
*  
*  Set this property to override this default, or use FacetChart.zoomStartValue and
*  FacetChart.zoomEndValue to start with a particular range.
* @type {ZoomStartPosition}
* @default null
*/
zoomStartPosition?: ZoomStartPosition;

/**
* For a FacetChart.canZoom, start value of the data range shown in the main
*  chart. If FacetChart.zoomEndValue is not also set on initialization (or if
*  FacetChart.setZoomEndValue is called after initialization) then the
*  range shown will be from zoomStartValue to the end of the dataset.
*  
*  The value provided should be a value in the range of the facet for the X axis, for example, for
*  a time-based axis, a Date instance, for a numeric axis, a Number, for an axis that just has
*  text labels (such as city names), a String.
* @type {any}
* @default null
*/
zoomStartValue?: any;

/**
* For a FacetChart.canZoom, end value of the data range shown in the main chart.
*  If FacetChart.zoomStartValue is not also set on intialization (or if
*  FacetChart.setZoomStartValue is called after initialization) then
*  the range shown will be from the beginning of the dataset up to
*  zoomEndValue.
*  
*  The value provided should be a value in the range of the facet for the X axis, for example, for
*  a time-based axis, a Date instance, for a numeric axis, a Number, for an axis that just has
*  text labels (such as city names), a String.
* @type {any}
* @default null
*/
zoomEndValue?: any;

/**
* Should the FacetChart.canZoom be printed with this FacetChart?
*  If true, then the SVG string returned by DrawPane.getSvgString will
*  include the zoom chart's SVG as well.
* @type {boolean}
* @default true
*/
printZoomChart?: boolean;

/**
* Causes labels for the X axis to be shown above the axis and to the right of the gradation line
*  they label, making for a vertically more compact chart at the risk of gradation labels being
*  partially obscured by data values. Also causes the last label to be skipped (nowhere to place
*  it).
* @type {boolean}
* @default false
*/
showInlineLabels?: boolean;

/**
* If set, gradation lines are drawn on top of data rather than underneath.
* @type {boolean}
* @default false
*/
showGradationsOverData?: boolean;

/**
* The legend is automatically shown for charts that need it (generally, multi-series
*  charts) but can be forced off by setting showLegend to false.
* @type {boolean}
* @default null
*/
showLegend?: boolean;

/**
* Horizontal alignment of the chart's FacetChart.showLegend.
* @type {LegendAlign}
* @default "center"
*/
legendAlign?: LegendAlign;

/**
* Space between the legend and the chart rect or axis labels (whatever the legend is
*  adjacent to.
* @type {Integer}
* @default "10"
*/
legendMargin?: Integer;

/**
* Padding around the legend as a whole.
* @type {Integer}
* @default "5"
*/
legendPadding?: Integer;

/**
* If drawing a border around the legend, the height of the drawn Rectangle.
* @type {Integer}
* @default "5"
*/
legendRectHeight?: Integer;

/**
* Size of individual color swatches in legend.
* @type {Integer}
* @default "16"
*/
legendSwatchSize?: Integer;

/**
* Properties for the swatches of color shown in the legend.
* @type {Partial<DrawRect>}
* @default null
*/
legendSwatchProperties?: Partial<DrawRect>;

/**
* Padding between color swatch and its label.
* @type {Integer}
* @default "5"
*/
legendTextPadding?: Integer;

/**
* Padding between each swatch and label pair.
* @type {Integer}
* @default "5"
*/
legendItemPadding?: Integer;

/**
* Properties for rectangle around the legend as a whole.
* @type {Partial<DrawRect>}
* @default null
*/
legendRectProperties?: Partial<DrawRect>;

/**
* Properties for top boundary of the legend are, when there is already an outer
*  container around the whole chart. see FacetChart.drawLegendBoundary
* @type {Partial<DrawLine>}
* @default null
*/
legendBoundaryProperties?: Partial<DrawLine>;

/**
* Properties for labels shown next to legend color swatches.
* @type {Partial<DrawLabel>}
* @default null
*/
legendLabelProperties?: Partial<DrawLabel>;

/**
* Whether a boundary should be drawn above the Legend area for circumstances where the 
*  chart area already has an outer border. If the chart has no outer border, then the 
*  FacetChart.legendRectProperties settings should be used instead.
* @type {boolean}
* @default null
*/
drawLegendBoundary?: boolean;

/**
* Whether to automatically show shadows for various charts.
* @type {boolean}
* @default true
*/
showShadows?: boolean;

/**
* Properties for shadows.
* @type {Partial<DrawOval>}
* @default null
*/
shadowProperties?: Partial<DrawOval>;

/**
* Distance between bars. May be reduced if bars would be smaller than
*  FacetChart.minBarThickness.
* @type {Integer}
* @default "4"
*/
barMargin?: Integer;

/**
* If bars would be smaller than this size, margins are reduced until bars overlap.
* @type {Integer}
* @default "4"
*/
minBarThickness?: Integer;

/**
* Bars will not be drawn over this thickness, instead, margins will be increased.
* @type {Integer}
* @default "150"
*/
maxBarThickness?: Integer;

/**
* For clustered charts, ratio between margins between individual bars and margins between
*  clusters.
* @type {Float}
* @default "4"
*/
clusterMarginRatio?: Float;

/**
* Properties for bar
* @type {Partial<DrawRect>}
* @default null
*/
barProperties?: Partial<DrawRect>;

/**
* Properties for lines that show data (as opposed to gradations or borders around the data
*  area).
* @type {Partial<DrawLine>}
* @default null
*/
dataLineProperties?: Partial<DrawLine>;

/**
* Properties for lines that outline a data shape (in filled charts such as area or radar
*  charts).
* @type {Partial<DrawItem>}
* @default null
*/
dataOutlineProperties?: Partial<DrawItem>;

/**
* Properties for data shapes (filled areas in area or radar charts).
* @type {Partial<DrawPath>}
* @default null
*/
dataShapeProperties?: Partial<DrawPath>;

/**
* Properties for a "value line" - a line shows where a particular discrete value is
*  placed, eg, vertical lines connecting points of a line chart to the X axis, or radial
*  lines in a Radar chart.
* @type {Partial<DrawLine>}
* @default null
*/
valueLineProperties?: Partial<DrawLine>;

/**
* Whether to treat non-numeric values in the dataset as indicating a break in the data
*  line. If set to false then null values are ignored. Defaults to
*  true for FacetChart.filled charts and to false
*  for line charts.
* @type {boolean}
* @default null
*/
discontinuousLines?: boolean;

/**
* Properties for radar background
* @type {Partial<DrawOval>}
* @default null
*/
radarBackgroundProperties?: Partial<DrawOval>;

/**
* Distance in pixels that radial labels are offset from the outside of the circle.
*  Default can vary depending upon ChartType and FacetChart.radarRotateLabels.
* @type {Integer}
* @default null
*/
radialLabelOffset?: Integer;

/**
* Default angle in degrees where pie charts start drawing sectors to represent data values.
*  Default of 0 places the first value starting from the "east" position. 
*  Use 270 or -90 for north.
* @type {Integer}
* @default "0"
*/
pieStartAngle?: Integer;

/**
* Properties for pie slices
* @type {Partial<DrawSector>}
* @default null
*/
pieSliceProperties?: Partial<DrawSector>;

/**
* Properties for the border around a pie chart.
* @type {Partial<DrawOval>}
* @default null
*/
pieBorderProperties?: Partial<DrawOval>;

/**
* Properties for pie ring border
* @type {Partial<DrawOval>}
* @default null
*/
pieRingBorderProperties?: Partial<DrawOval>;

/**
* Whether to show a "doughnut hole" in the middle of pie charts. Defaults to whether
*  chartType is set to "Doughnut" (shown) vs "Pie" (not shown) but can be forced on or off
*  via showDoughnut.
* @type {boolean}
* @default null
*/
showDoughnut?: boolean;

/**
* If showing a doughnut hole (see FacetChart.showDoughnut), ratio of the size of the doughnut
*  hole to the size of the overall pie chart, as a number between 0 to 1.
* @type {Float}
* @default "0.2"
*/
doughnutRatio?: Float;

/**
* Properties for doughnut hole
* @type {Partial<DrawOval>}
* @default null
*/
doughnutHoleProperties?: Partial<DrawOval>;

/**
* Properties for pie label line
* @type {Partial<DrawLine>}
* @default null
*/
pieLabelLineProperties?: Partial<DrawLine>;

/**
* Angle where first label is placed in a Pie chart in stacked mode, in degrees.
* @type {Integer}
* @default "20"
*/
pieLabelAngleStart?: Integer;

/**
* How far label lines stick out of the pie radius in a Pie chart in stacked mode.
* @type {Integer}
* @default "7"
*/
pieLabelLineExtent?: Integer;

/**
* Specifies the attribute in the metric facet that will define the end point of segments in
*  a histogram chart. The start point is set via the FacetChart.valueProperty.
* @type {string}
* @default null
*/
endValueMetric?: string;

/**
* Specifies the attribute in the metric facet that will define the z-ordering of the
*  segments in a histogram chart. If the z-ordering isn't specified, it will be assigned
*  based on data order, with the last data point ordered above the first. Relative
*  z-ordering is only important between segments within the same data label facet, since
*  segments that differ in their data label facet value should never overlap,
*  
*  Note that zIndex values should be integers between 0 and FacetChart.maxDataZIndex, inclusive,
*  and don't directly map to the DrawItem.zIndex values of the underlying
*  DrawRects. This allows the Framework to use automatic z-ordering in the chart
*  logic without any additional sorting or overhead that would otherwise be required.
* @type {string}
* @default null
*/
zIndexMetric?: string;

/**
* Maximum allowed zIndex that can be specified through FacetChart.zIndexMetric in a histogram
*  chart. Any zIndex values exceeding this property will be internally clipped so as to not
*  exceed it. While this property can be increased, note that very large values may hit
*  limitations related to the browser's implementation of the current 
*  DrawPane.drawingType.
* @type {Integer}
* @default "10000"
*/
maxDataZIndex?: Integer;

/**
* lowErrorMetric and FacetChart.highErrorMetric can be used to cause error
*  bars to appear above and below the main data point.
*  
*  lowErrorMetric and highErrorMetric provide the name of an
*  additional attributes that appears in each Record holding the low error value and high
*  error value respectively.
*  
*  Error bars are supported for single-axis charts only.
* @type {string}
* @default null
*/
lowErrorMetric?: string;

/**
* See FacetChart.lowErrorMetric.
* @type {string}
* @default null
*/
highErrorMetric?: string;

/**
* Width of the horizontal line of the "T"-shape portion of the error bar).
* @type {Integer}
* @default "6"
*/
errorBarWidth?: Integer;

/**
* Properties of the lines used to draw error bars (short, horizontal lines at the low
*  and high metric values, and a vertical connecting line).
*  
*  Note that the DrawItem.lineColor property has no effect as the color
*  of the error bars is derived from the color of the data line.
* @type {Partial<DrawLine>}
* @default null
*/
errorLineProperties?: Partial<DrawLine>;

/**
* This property helps specify the color of the error bars and its value must be a
*  number between -100 and 100. Error bars have the same color as the data line, but the
*  colors are actually "muted" by this percentage by shifting them toward white (or for
*  negative numbers, toward black). The default is to darken the data colors by 60%
*  to get the error bar colors.
* @type {Float}
* @default "-60"
*/
errorBarColorMutePercent?: Float;

/**
* Shows the value of the nearest data value in a floating label whenever the mouse moves
*  within the main chart area. The visual element representing the data
*  value will also be emphasized by brightening or highlighting it (appearance differs by
*  chart type).
*  
*  Calculates nearest value based on FacetChart.getNearestDrawnValue.
*  
*  The data value will be formatted using FacetChart.formatDataValue.
*  The label's appearance is controlled by FacetChart.hoverLabelProperties.
* @deprecated * in favor of FacetChart.showDataValuesMode, a compound setting that supports showing data-values in the chart and in hovers in various combinations. The
*  equivalent to showValueOnHover:true is ShowDataValuesMode.inHoverOnly.

* @type {boolean}
* @default null
*/
showValueOnHover?: boolean;

/**
* Properties for text in a floating label that represents the data value shown whenever
*  the mouse moves withing the main chart area when FacetChart.showValueOnHover is enabled.
* @type {Partial<DrawLabel>}
* @default null
*/
hoverLabelProperties?: Partial<DrawLabel>;

/**
* An extra amount of padding to show around the FacetChart.hoverLabelProperties
*  when FacetChart.showValueOnHover is enabled.
* @type {Integer}
* @default "4"
*/
hoverLabelPadding?: Integer;

/**
* Properties for rectangle that draws behind of a floating hover label that represents
*  the data value. See FacetChart.showValueOnHover for more details.
* @type {Partial<DrawRect>}
* @default null
*/
hoverRectProperties?: Partial<DrawRect>;

/**
* Determines the percentage by which to brighten a data shape (filled area in area, bar, pie,
*  or radar chart) when showing the associated data label due to
*  FacetChart.showDataValuesMode. Legal values are between 0 and 100. inclusive.
*  
*  The property default varies based on the currently loaded skin.
* @type {Integer}
* @default "varies"
*/
brightenPercent?: Integer;

/**
* Should data values be shown as text labels near the shape representing the value, for example,
*  above columns of a column chart, or adjacent to points in a line chart?
*  
*  If set to false, then data values will not be shown.
*  
*  If set to true, data values will be shown unless the data density is high enough that
*  labels will potentially overlap, in which case, data values will not be shown and
*  hovers will be shown instead, in the same way as FacetChart.showValueOnHover shows hovers.
* @deprecated * in favor of FacetChart.showDataValuesMode, a compound setting that supports showing data-values in the chart and in hovers in various combinations. The
*  equivalent to showDataValues:true is ShowDataValuesMode.inChartOnly 
*  or ShowDataValuesMode.inChartOrHover if showValueOnHover was also
*  set to true.

* @type {boolean}
* @default false
*/
showDataValues?: boolean;

/**
* Strategy for determining whether and when to show data-values - either in the chart,
*  near the shape representing a value (above columns of a column chart for example, or 
*  adjacent to points in a line chart), in hovers, or some combination of both, including 
*  automatic rotation where supported.
*  
*  Depending on the chart type, there are different options for showing data-values - eg, 
*  FacetChart.stacked cannot show values inline in the chart-body; 
*  column-charts can, and they can rotate titles if they're wider than their columns; 
*  pie charts can show some data-values in the chart but not others; all the types can 
*  show values in hovers. 
*  
*  If set to never, then data-values will never be shown; inChartOnly 
*  allows data-values in the chart-body, where supported and where they will fit, but 
*  suppresses them in hovers and inHoverOnly always shows all data-values in hovers.
*  
*  If set to auto, first try to show values in the chart, where the chart-type 
*  supports it, and where they'll fit. If they don't all fit, show the ones that 
*  do, including rotating them if necessary, if the chart-type allows it, and 
*  then switch on hovers as well, as needed. This mode is particularly useful in situations 
*  where the chart-type can be changed by the user.
* @type {ShowDataValuesMode}
* @default "never"
*/
showDataValuesMode?: ShowDataValuesMode;

/**
* Maximum distance from the *outer radius* of the nearest bubble when hover will be shown.
* @type {number}
* @default "50"
*/
bubbleHoverMaxDistance?: number;

/**
* Margin between FacetChart.extraAxisMetrics.
* @type {number}
* @default "10"
*/
valueAxisMargin?: number;

/**
* When set, ticks are shown for the Y (vertical) axis if it's a value axis.
*  
*  Normally, ticks are not shown for the primary axis, since 
*  FacetChart.getGradations show value demarcations. Gradation 
*  lines are always show for FacetChart.extraAxisMetrics in multi-axis charts
*  (since there are no gradation lines for the additional axes).
*  
*  FacetChart.showXTicks can be used to control whether ticks are shown for the horizontal axis,
*  for certain chart types. See also FacetChart.majorTickGradations for control of which ticks
*  are shown as major vs minor ticks.
* @type {boolean}
* @default false
*/
showYTicks?: boolean;

/**
* When set, ticks are shown for the X (horizontal) axis for Scatter plots or Bar charts.
* @type {boolean}
* @default false
*/
showXTicks?: boolean;

/**
* Length of the tick marks used when either FacetChart.showXTicks or 
*  FacetChart.showYTicks is enabled, or when FacetChart.extraAxisMetrics
*  are in use.
*  
*  If minor tick marks are also shown, their length is controlled by FacetChart.minorTickLength.
* @type {Integer}
* @default "5"
*/
tickLength?: Integer;

/**
* Length of minor ticks marks shown along axis, if FacetChart.showMinorTicks
*  are enabled.
* @type {Integer}
* @default "2"
*/
minorTickLength?: Integer;

/**
* @deprecated * use FacetChart.tickLength instead

* @type {number}
* @default null
*/
gradationTickMarkLength?: number;

/**
* Margin between the tick marks and the labels of the
*  FacetChart.extraAxisMetrics.
* @type {number}
* @default "7"
*/
tickMarkToValueAxisMargin?: number;

/**
* If FacetChart.showYTicks are being shown, controls whether a distinction is made
*  between major and minor tick marks.
*  
*  If minor ticks are used, by default, major ticks are used for powers of 10 and minor
*  ticks are used for other gradations. See FacetChart.majorTickGradations for control over
*  which ticks are rendered as major vs minor ticks.
* @type {boolean}
* @default true
*/
showMinorTicks?: boolean;

/**
* List of tick marks that should be drawn as major ticks, expressed as a series of numbers
*  between 1 and 10, representing boundaries within a given order of magnitude (power of
*  10). These numbers must be multiples of FacetChart.gradationGaps, or no ticks will end up as
*  minor ticks.
*  
*  The default setting of [1] means that major ticks are used for powers of 10 only. A
*  setting of [1,5] would mean that major ticks are also used at half-orders of magnitude,
*  such as 0.5 or 50. For example, if used with a FacetChart.gradationGaps setting of [1,2.5]
*  for a chart showing values between 0 and 1, this would result in major ticks at 0, 1 and
*  0.5, and minor ticks at 0.25 and 0.75.
*  
*  See also FacetChart.majorTickTimeIntervals for controlling major vs minor ticks for the
*  X-axis of time/date-valued Scatter plots.
* @type {Array<Partial<Float>>}
* @default "[1]"
*/
majorTickGradations?: Array<Partial<Float>>;

/**
* When ticks are being FacetChart.showXTicks for a Scatter plot where the
*  X axis uses time/date values, controls the intervals which are shown as major ticks.
*  
*  The intervals are specified as Strings, in the same way as
*  FacetChart.otherAxisGradationTimes.
*  
*  For any given interval, the first major tick is shown for the next greatest time unit.
*  For example, for interval such as "2h" (2 hours), the first major tick starts on the day
*  boundary (whether that day boundary is visible in the chart or not).
*  
*  By default, all ticks are shown as major ticks.
* @type {Array<Partial<string>>}
* @default null
*/
majorTickTimeIntervals?: Array<Partial<string>>;

/**
* Whether the positions of value axes can be changed. The default is true
*  for charts with three or more vertical, value axes.
* @type {boolean}
* @default null
*/
canMoveAxes?: boolean;

/**
* Defines the set of metrics that will be plotted as additional vertical axes. See the
*  main FacetChart docs for an overview of how multi-axis charts are used.
*  
*  Each metric corresponds to different value property
*  of the data records and superimposes its drawn data onto the chart rectangle.
*  The value properties are called metrics, and they can be either the FacetChart.valueProperty
*  or the "id" of a FacetValue of the inlined Facet (which is then called the
*  metric facet). Each value axis has its own gradations that are shown as tick marks
*  along the length of the axis. This property, extraAxisMetrics, specifies the metrics
*  to use for additional value axes to the main value axis.
*  
*  The additional value axis may have their own gradations, chart type, log scale, data colors
*  and gradients, and other chart properties. These properties are specified with
*  the FacetChart.extraAxisSettings property.
*  
*  Value axes, including the main value axis, are labelled in the legend along with
*  representations of the charted data. The labels are taken from the FacetValue.title of
*  each metric's FacetValue (or the FacetChart.valueTitle if the metric is the FacetChart.valueProperty).
*  
*  The order of the metrics determines the position of the corresponding axes on
*  the chart as well as the z-ordering of the corresponding data lines.
*  The first and second extra value axes are placed to the right of the chart rectangle,
*  and any remaining extra value axes are placed to the left of the main value axis
*  (and therefore to the left of the chart rectangle).
* @type {Array<Partial<string>>}
* @default "[]"
*/
extraAxisMetrics?: Array<Partial<string>>;

/**
* For charts will multiple vertical axes, optionally provides settings for how each
*  FacetChart.extraAxisMetrics is plotted. See the main FacetChart
*  docs for an overview of how multi-axis charts are used.
*  
*  The chart of each metric's values may be of any rectangular chart type that uses a
*  vertical value axis ("Column", "Area", or "Line" - "Histogram" is not supported).
*  Because the charts will be superimposed over the same drawing area, there can only
*  be one "Column" chart and one "Area" chart. The column chart is placed on the bottom
*  followed by the area chart, and then the line charts are drawn on top in the
*  order of their metric in the FacetChart.extraAxisMetrics array.
*  If the MetricSettings.chartTypes
*  are left unspecified then by default the first metric will be drawn as
*  columns and the remaining will be drawn as lines.
* @type {Array<Partial<MetricSettings>>}
* @default "[]"
*/
extraAxisSettings?: Array<Partial<MetricSettings>>;

/**
* When set to true, introduces scrollbars when this widget is smaller than the specified 
*  chart-content minimum FacetChart.minContentWidth or 
*  FacetChart.minContentHeight. These minimum sizes limit all chart-content,
*  including data and labels, titles and legends.
*  
*  See FacetChart.autoScrollData for a means to introduce scrolling
*  according to the data being displayed.
* @type {boolean}
* @default false
*/
autoScrollContent?: boolean;

/**
* When FacetChart.autoScrollContent is true, limits the minimum 
*  width of the chart-content, including data, labels, titles and legends. If this widget is
*  sized smaller than this width, scrollbars are introduced to reach the hidden content. See
*  FacetChart.minContentHeight to affect the minimum vertical 
*  content-size.
* @type {Integer}
* @default "150"
*/
minContentWidth?: Integer;

/**
* When FacetChart.autoScrollContent is true, limits the minimum 
*  height of the chart-content, including data, labels, title and legends. If this widget is
*  sized smaller than this height, scrollbars are introduced to reach the hidden content. See
*  FacetChart.minContentWidth to affect the minimum horizontal 
*  content-size.
* @type {Integer}
* @default "150"
*/
minContentHeight?: Integer;

/**
* Minimum width for this chart instance.
* @type {Integer}
* @default "1"
*/
minChartWidth?: Integer;

/**
* Minimum height for this chart instance.
* @type {Integer}
* @default "1"
*/
minChartHeight?: Integer;

/**
* For some FacetChart.chartType, should the chart body be automatically 
*  expanded and scrollbars introduced according to data?
*  
*  When true for a column, histogram, line, or area chart that has FacetChart.getDataLabelFacet, the chart expands horizontally, showing a scroll 
*  bar, if that's needed to make room for the facet value labels or, for column and histogram 
*  charts, to make space for the FacetChart.getMinClusterSize or the margins between them.
*  
*  When true for a Bar chart, expansion and scrollbar are vertical, and also make space for the 
*  FacetChart.getMinClusterSize or the margins between them.
*  
*  Note that this feature is incompatible with the following properties:
*  
* - LabelCollapseMode (other than the default of "none")
*  
* - FacetChart.rotateLabels (in "auto" mode)
*  
* - DrawPane.canDragScroll
*  
* - FacetChart.canZoom
*  If FacetChart.rotateLabels is set to "auto" it will be treated as "never" if
*  autoScrollData has been set. If any of the other properties have non-default
*  values, a warning will be logged and autoScrollData will be disabled. The
*  factors used to drive expansion can be limited by setting AutoScrollDataApproach.
*  
*  You can also enforce a minimum size for the chart-content, and scrollbars will be introduced
*  if this widget shrinks below that size. See 
*  FacetChart.autoScrollContent, along with
*  FacetChart.minContentWidth and
*  FacetChart.minContentHeight.
* @type {boolean}
* @default false
*/
autoScrollData?: boolean;

/**
* If set, overrides the default behavior of FacetChart.autoScrollData, potentially limiting what
*  factors drive the automatic expansion of the chart. (The "both" setting is no
*  different than the default of null.)
*  
*  When labels are on the x-axis, and if you're sizing bars very tightly to labels by 
*  defining FacetChart.getMinClusterSize, you may not want label-driven expansion, as the 
*  default separation assigned between them is very generous, and is based on the widest 
*  labels. (You may also set FacetChart.minLabelGap to gain more control over the separation.)
* @type {AutoScrollDataApproach}
* @default null
*/
autoScrollDataApproach?: AutoScrollDataApproach;

/**
* Whether to place the FacetChart.showTitle with respect to the full, 
*  scrollable width of the chart when FacetChart.autoScrollData is active. The default of false 
*  means that the title will be placed in the visible, non-overflowed region of the chart, for 
*  greater visibility.
*  
*  Note that alignment of the title itself is governed by 
*  FacetChart.titleAlign.
* @type {boolean}
* @default false
*/
centerTitle?: boolean;

/**
* Whether to place the FacetChart.showLegend with respect to the full, 
*  scrollable width of the chart when FacetChart.autoScrollData is active. The default of false 
*  means that the legend will be placed in the visible, non-overflowed region of the chart, for 
*  greater visibility.
*  
*  Note that alignment of the legend itself is governed by 
*  FacetChart.legendAlign.
*  
*  Note that this setting has no impact on axis labeling, which always occurs with respect to
*  the full, expanded width of the chart.
* @type {boolean}
* @default false
*/
centerLegend?: boolean;

/**
* Specifies what DataSource fields to use as the chart FacetChart.facets for a databound
*  chart. If FacetChart.facets is also explicitly set, FacetChart.facetFields is definitive but
*  Facet properties will be picked up from FacetChart.facets also present in the
*  FacetChart.facetFields.
*  
*  If neither this property nor FacetChart.facets is set, a databound chart will attempt to
*  auto-derive FacetChart.facetFields from the DataSource fields. The first two text or
*  text-derived fields in the DataSource will be assumed to be the FacetChart.facetFields.
* @type {Array<Partial<FieldName>> | FieldName}
* @default null
*/
facetFields?: Array<Partial<FieldName>> | FieldName;

/**
* If FacetChart.autoFetchData is true, this attribute allows the developer to
*  specify a textMatchStyle for the initial ListGrid.fetchData call.
* @type {TextMatchStyle}
* @default "substring"
*/
autoFetchTextMatchStyle?: TextMatchStyle;

/**
* FacetCharts do not yet support paging, and will fetch all records that meet the criteria.
* @type {FetchMode}
* @default "basic"
*/
dataFetchMode?: FetchMode;

/**
* This DataBoundComponent property is not applicable to charts.
* @type {boolean}
* @default false
*/
showDetailFields?: boolean;

/**
* Padding from edge of Y the Axis Label.
* @type {Integer}
* @default "5"
*/
gradationLabelPadding?: Integer;

/**
* Padding between each swatch and label pair.
* @type {Integer}
* @default "5"
*/
yAxisLabelPadding?: Integer;

/**
* Horizontal alignment of y-axis labels, shown to the left of the chart.
* @type {Alignment}
* @default "right"
*/
yAxisLabelAlign?: Alignment;

/**
* Horizontal alignment of labels shown in extra y-axes, shown to the right of the chart.
* @type {Alignment}
* @default "left"
*/
extraAxisLabelAlign?: Alignment;

/**
* Whether to call FacetChart.formatAxisValue or FacetChart.formatFacetValueId on a facet value id when
*  the id is a string. Can be set false to allow the formatting function(s) to be written
*  without having to handle the string case.
* @type {boolean}
* @default true
*/
formatStringFacetValueIds?: boolean;

/**
* Causes the chart to use the colors specified in FacetChart.dataColors but specify chart-specific 
*  gradients based on the primary data color per chart type.
* @type {boolean}
* @default true
*/
useAutoGradients?: boolean;

/**
* Setting to define whether the border around the bar chart area should be the same color
*  as the main chart area.
* @type {boolean}
* @default null
*/
matchBarChartDataLineColor?: boolean;

/**
* When hovering over a bar chart block, should the whole bar chart area be brightened
*  Default behaviour is to just brighten the line color around the bar chart area
* @type {boolean}
* @default null
*/
brightenAllOnHover?: boolean;

/**
* An array of colors to use for a series of visual elements representing data (eg columns,
*  bars, pie slices), any of which may be adjacent to any other.
*  
*  Colors must be in the format of a leading hash (#) plus 6 hexadecimal digits, for example, 
*  "#FFFFFF" is white, "#FF0000" is pure red.
* @type {Array<Partial<CSSColor>>}
* @default "see below"
*/
dataColors?: Array<Partial<CSSColor>>;

/**
* A dictionary of gradients to use for a series of visual elements representing data (eg columns,
*  bars, pie slices), any of which may be adjacent to any other.
* @type {any}
* @default "see below"
*/
dataGradients?: any;

/**
* Criteria to be used when FacetChart.autoFetchData is set.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
initialCriteria?: Criteria;

/**
* Criteria that are never shown to or edited by the user and are cumulative with any criteria
*  provided via DataBoundComponent.initialCriteria and related methods.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
implicitCriteria?: Criteria;

/**
* If true, when this component is first drawn, automatically call this.fetchData().
*  Criteria for this fetch may be picked up from FacetChart.initialCriteria, and textMatchStyle may
*  be specified via ListGrid.autoFetchTextMatchStyle.
*  
*  NOTE: if autoFetchData is set, calling
*  ListGrid.fetchData before draw will cause two requests to be issued, one from the manual
*  call to fetchData() and one from the autoFetchData setting. The second request will use
*  only FacetChart.initialCriteria and not any other criteria or settings from the first request.
*  Generally, turn off autoFetchData if you are going to manually call ListGrid.fetchData at any time.
* 
*  Note: If you are using saved searches - either via SavedSearchItem or ListGrid.saveDefaultSearch, autoFetchData
*  will be automatically suspended and replaced with the saved criteria/view state, if applicable.
* @type {boolean}
* @default false
*/
autoFetchData?: boolean;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "FacetChartEditProxy"
*/
editProxyConstructor?: SCClassName;



/**
*  Provides a new data set to the chart after it has been created or drawn.
*  The FacetChart will redraw to show the new data automatically.
* 
* 
* @param {Array<Partial<CellRecord>>} data to show in the chart
*/
setData(newData:Array<Partial<CellRecord>>): void;

/**
*  Get a facet definition by facetId.
* 
* @param {string} the id of the facet to retrieve
*/
getFacet(facetId:string): Facet;

/**
*  Get facet value definition by facetId and facetValueId.
* 
* @param {string} the id of the facet to retrieve
* @param {string} the id of the facet value to retrieve
*/
getFacetValue(facetId:string, facetValueId:string): FacetValue;

/**
*  Setter for FacetChart.proportional.
* 
* @param {boolean} Whether the chart should now use proportional mode.
*/
setProportional(proportional:boolean): void;

/**
*  Setter for FacetChart.pixelsPerGradation.
* 
* 
* @param {Integer} new FacetChart.pixelsPerGradation value
*/
setPixelsPerGradation(pixelsPerGradation:Integer): void;

/**
*  Setter for FacetChart.otherAxisPixelsPerGradation.
* 
* 
* @param {Integer} new FacetChart.otherAxisPixelsPerGradation value
*/
setOtherAxisPixelsPerGradation(pixelsPerGradation:Integer): void;

/**
*  Setter for FacetChart.gradationGaps.
* 
* 
* @param {Array<Partial<Float>>} new FacetChart.gradationGaps value
*/
setGradationGaps(gradationGaps:Array<Partial<Float>>): void;

/**
*  Setter for FacetChart.otherAxisGradationGaps.
* 
* 
* @param {Array<Partial<Float>>} new FacetChart.otherAxisGradationGaps value
*/
setOtherAxisGradationGaps(gradationGaps:Array<Partial<Float>>): void;

/**
*  Setter for FacetChart.otherAxisGradationTimes.
* 
* 
* @param {Array<Partial<string>>} new FacetChart.otherAxisGradationTimes value
*/
setOtherAxisGradationTimes(otherAxisGradationTimes:Array<Partial<string>>): void;

/**
*  Method to change the current ChartType.
*  Will redraw the chart if drawn. 
* 
* 
* @param {boolean} whether to draw lines between adjacent data points in "Scatter" plots
*/
setShowScatterLines(showScatterLines:boolean): void;

/**
*  Method to change the current ChartType.
*  Will redraw the chart if drawn. 
* 
* 
* @param {DataLineType} ow to draw lines between adjacent data points in Line and Scatter charts
*/
setDataLineType(dataLineType:DataLineType): void;

/**
*  Setter for FacetChart.useMultiplePointShapes.
* 
* @param {boolean} Whether the chart should now use multiple shapes to show data points.
*/
setUseMultiplePointShapes(useMultiplePointShapes:boolean): void;

/**
*  Setter for FacetChart.scaleStartColor.
* 
* @param {CSSColor} The new start color for the color scale.
*/
setScaleStartColor(scaleStartColor:CSSColor): void;

/**
*  Setter for FacetChart.scaleEndColor.
* 
* @param {CSSColor} The new end color for the color scale.
*/
setScaleEndColor(scaleEndColor:CSSColor): void;

/**
*  Fires when the end user changes the zoom position in a FacetChart.canZoom.
* 
* @param {any} starting value for the data range shown in the main chart
* @param {any} ending value for the data range shown in the main chart
*/
zoomChanged(startValue:any, endValue:any): void;

/**
*  Fires when a data value is clicked, and provides information about the data value that
*  was clicked as a DrawnValue object.
*  
*  Specifically, this fires for clicks on pie slices, bars or columns, areas, lines or
*  points (in a Bubble or Scatter plot).
*  
*  If there are multiple data values at the clicked position, you can use
*  FacetChart.getNearestDrawnValues to discover the full list of values at the current
*  coordinate (pass in Canvas.getOffsetX for the coordinates).
*  
*  If you want to create behaviors for clicking or moving near shapes without
*  requiring a direct hit, implement a standard Canvas.click handler on the
*  FacetChart as a whole and use FacetChart.getNearestDrawnValue to discover the nearest
*  data values.
* 
* @param {DrawnValue} information about the value that was clicked
*/
valueClick(drawnValue:DrawnValue): void;

/**
*  Fires when the user clicks on the legend area of the chart.
*  
*  If the user specifically clicks on a color swatch or it's label, the FacetValue
*  clicked on will be provided.
*  
*  If the chart is a FacetChart.extraAxisMetrics, the
*  FacetValue for the clicked-on metric will also be provided.
* 
* @param {FacetValue} facetValue that was clicked, or null if click was in    empty space
* @param {FacetValue} for a multi-axis chart, facetValue representing    the clicked metric. Null if click is in empty space or chart is not multi-axis
*/
legendClick(facetValue:FacetValue, metricFacetValue:FacetValue): void;

/**
*  Fires when the user clicks on a data label, that is, a text label showing values from
*  the first facet. For example, the labels underneath the X-axis of a column chart,
*  labelling each column.
* 
* 
* @param {FacetValue} facetValue that was clicked, or null if click was in    empty space
*/
dataLabelClick(facetValue:FacetValue): void;

/**
*  Returns the minimum cluster size (for clustered charts), or minimum bar thickness (for 
*  histogram or stacked charts) for the specified FacetChart.getDataLabelFacet value. Only applicable to a column, bar, or histogram chart. No default
*  implementation. Both this minimum and FacetChart.minBarThickness are used together to
*  determine the effective minimum of the cluster or bar stack.
*  
*  Per-facet-value minimum cluster sizes aren't supported for 
*  MetricSettings charts, in which multiple chart types are overlaid onto
*  the same chart.
*  
*  
*  Note that this method is simply an override point, since it has no default
*  implementation.
*  
* 
* 
* @param {number} data label axis index of cluster or bar stack
* @param {string | number | Date} data label facet value id                       of cluster or bar stack
*/
getMinClusterSize(index:number, facetValueId:string | number | Date): Integer;

/**
*  Defines the format of the label for a segment in a histogram chart. By default, it
*  simply returns a label of the form "Y1 to Y2" describing the start and end values,
*  applying FacetChart.formatDataValue to format the values.                
*  
*  Note that this method has no impact on the facet value labels appearing on the
*  horizontal axis of the histogram chart.
* 
* 
* @param {any} raw start value of the segment
* @param {any} raw end value of the segment
*/
formatSegmentLabel(startValue:any, endValue:any): string;

/**
*  Method to change the current FacetChart.zIndexMetric - see property for more details. Will
*  redraw the chart if drawn.
* 
* 
* @param {string} name of zIndex metric
*/
setZIndexMetric(zIndexMetric:string): void;

/**
*  Fires when the mouse hovers over a data label, that is, a text label showing values from
*  the first facet. For example, the labels underneath the X-axis of a column chart,
*  labelling each column.
* 
* 
* @param {FacetValue} facetValue that was hovered
*/
dataLabelHover(facetValue:FacetValue): void;

/**
*  Called when the mouse hovers over a data label, that is, a text label showing values from
*  the first facet. For example, the labels underneath the X-axis of a column chart,
*  labelling each column.
* 
* 
* @param {FacetValue} facetValue that was hovered
*/
getDataLabelHoverHTML(facetValue:FacetValue): HTMLString;

/**
*  Fires when the mouse hovers over a color swatch or its label in the legend area of the
*  chart.
*  
*  The FacetValue that the user is hovering over is provided.
*  If the chart is a FacetChart.extraAxisMetrics, the
*  FacetValue for the hovered-over metric will also be provided.
* 
* 
* @param {FacetValue} facetValue that the mouse is over
* @param {FacetValue} for a multi-axis chart, facetValue representing    the hovered-over metric. Null if chart is not multi-axis
*/
legendHover(facetValue:FacetValue, metricFacetValue:FacetValue): void;

/**
*  Called when the mouse hovers over a color swatch or its label in the legend area of the
*  chart.
*  
*  The FacetValue that the user is hovering over is provided.
*  If the chart is a FacetChart.extraAxisMetrics, the
*  FacetValue for the hovered-over metric will also be provided.
* 
* 
* @param {FacetValue} facetValue that the mouse is over
* @param {FacetValue} for a multi-axis chart, facetValue representing    the hovered-over metric. Null if chart is not multi-axis
*/
getLegendHoverHTML(facetValue:FacetValue, metricFacetValue:FacetValue): HTMLString;

/**
*  Setter for FacetChart.showYTicks.
* 
* 
* @param {boolean} new FacetChart.showYTicks value
*/
setShowYTicks(showTicks:boolean): void;

/**
*  Setter for FacetChart.showXTicks.
* 
* 
* @param {boolean} new FacetChart.showXTicks value
*/
setShowXTicks(showTicks:boolean): void;

/**
*  Setter for FacetChart.tickLength.
* 
* 
* @param {Integer} new FacetChart.tickLength value
*/
setTickLength(tickLength:Integer): void;

/**
*  Setter for FacetChart.minorTickLength.
* 
* 
* @param {Integer} new FacetChart.minorTickLength value
*/
setMinorTickLength(minorTickLength:Integer): void;

/**
*  Setter for FacetChart.showMinorTicks.
* 
* 
* @param {boolean} new FacetChart.showMinorTicks value
*/
setShowMinorTicks(showMinorTicks:boolean): void;

/**
*  Setter for FacetChart.majorTickGradations.
* 
* 
* @param {Array<Partial<Float>>} new FacetChart.majorTickGradations value
*/
setMajorTickGradations(majorTickGradations:Array<Partial<Float>>): void;

/**
*  Setter for FacetChart.majorTickTimeIntervals.
* 
* 
* @param {Array<Partial<string>>} new FacetChart.majorTickTimeIntervals value
*/
setMajorTickTimeIntervals(majorTickTimeIntervals:Array<Partial<string>>): void;

/**
*  Retrieves printable HTML for this component and all printable subcomponents.
*  
*  By default any Canvas with children will simply collect the printable HTML of its
*  children by calling getPrintHTML() on each child that is considered
*  Canvas.shouldPrint.
*  
*  If overriding this method for a custom component, you should either return a String of
*  printable HTML string directly or return null, and fire the callback (if provided)
*  using Class.fireCallback.
*  
*  To return an empty print representation, return an empty string ("") rather than null.
*  
*  The printProperties argument, if passed, must be passed to any subcomponents on
*  which getPrintHTML() is called.
*  
*  Notes on printing
*  
*  To print a FacetChart for export on IE8 and earlier, it is important to pass
*  PrintProperties with PrintProperties.printForExport:true:
*  
*  var exportHTML = chart.getPrintHTML({ printForExport:true });
*  
* 
* @param {PrintProperties=} properties to configure printing behavior - may be null.
* @param {Callback=} optional callback. This is required to handle cases where HTML         generation is asynchronous - if a method generates HTML asynchronously, it should return          null, and fire the specified callback on completion of HTML generation. The          first parameter HTML should contain the generated print HTML.          The callback is only called if null is returned. Furthermore, the default          getPrintHTML() implementation always returns null and fires the callback          when a callback is provided.
*/
getPrintHTML(printProperties?:PrintProperties, callback?:Callback): HTMLString;

/**
*  Sets FacetChart.autoScrollContent and updates the chart.
* 
* @param {boolean} whether the chart should automatically show scrollbars   when it's size is smaller than the minimum content    FacetChart.minContentWidth or FacetChart.minContentHeight.
*/
setAutoScrollContent(autoScrollContent:boolean): void;

/**
*  Returns the FacetChart.minContentWidth for this facet chart when FacetChart.autoScrollContent is enabled.
* 
* 
*/
getMinContentWidth(): Integer;

/**
*  Returns the FacetChart.minContentHeight for this facet chart when FacetChart.autoScrollContent is enabled.
* 
* 
*/
getMinContentHeight(): Integer;

/**
*  Get the top coordinate of the central chart area, where data elements appear. 
*  
*  This is only allowed to be called when FacetChart.chartDrawn fires.
* 
* 
*/
getChartTop(): Float;

/**
*  Get the height the central chart area, where data elements appear.
*  
*  This is only allowed to be called when FacetChart.chartDrawn fires.
* 
* 
* @param {boolean} if false then cached value will be returned, otherwise will be recalculated.
*/
getChartHeight(recalc:boolean): Float;

/**
*  Get the left margin of the central chart area, where data elements appear.
*  
*  This is only allowed to be called when FacetChart.chartDrawn fires.
* 
* 
*/
getChartLeft(): Float;

/**
*  Get the width of the central chart area, where data elements appear.
*  
*  This is only allowed to be called when FacetChart.chartDrawn fires.
* 
* 
* @param {boolean} if false then cached value will be returned, otherwise will be recalculated.
*/
getChartWidth(recalc:boolean): Float;

/**
*  Returns the FacetChart.minChartWidth for the chart body.
* 
*/
getMinChartWidth(): void;

/**
*  Returns the FacetChart.minChartHeight for the chart body.
* 
*/
getMinChartHeight(): void;

/**
*  Sets FacetChart.autoScrollData and updates the chart.
* 
* @param {boolean} whether chart should automatically expand                  and show scrollbars to accommodate content.
*/
setAutoScrollData(autoScrollData:boolean): void;

/**
*  Sets AutoScrollDataApproach and updates the chart.
* 
* @param {AutoScrollDataApproach} what should drive horizontal expansion                            of the chart?
*/
setAutoScrollDataApproach(autoScrollDataApproach:AutoScrollDataApproach): void;

/**
*  Method to change FacetChart.stacked. Use null to
*  apply a default value for the current ChartType.
* 
* 
* @param {boolean} new value
*/
setStacked(stacked:boolean): void;

/**
*  Method to change FacetChart.filled.
*  Use null to apply a default value for the current ChartType.
* 
* 
* @param {boolean} new value
*/
setFilled(filled:boolean): void;

/**
*  Method to change the current ChartType.
*  Will redraw the chart if drawn. 
*  Will use default settings for the new chart type for FacetChart.stacked
*  and FacetChart.filled if those values are null.
*  
*  Note that for FacetChart.extraAxisMetrics charts this method changes the
*  chartType for the main value axis only.
* 
* 
* @param {ChartType} new chart type
*/
setChartType(chartType:ChartType): void;

/**
*  Called when most elements of the chart other than data data have been drawn, including
*  gradations and legend.
*  
*  This notification will be fired each time the chart is redrawn (due to resize, data change,
*  etc). If you want to draw additional information on the chart using DrawPane
*  (FacetChart's superclass) and various DrawItems, you should do in response to this
*  notification. Due to auto-sizing, APIs that are typically used to position custom
*  DrawItems (such as FacetChart.getChartLeft) may return bad values if called at other times.
*  
*  Additional DrawItems added in this method will appear underneath data elements such as bars
*  or columns. See FacetChart.chartDrawn for placing DrawItems on top of data elements.
* 
* 
*/
chartBackgroundDrawn(): void;

/**
*  Called when all elements of the chart (data lines / shapes, gradations, legend, labels etc)
*  have completed drawing. 
*  
*  See FacetChart.chartBackgroundDrawn for usage information.
* 
* 
*/
chartDrawn(): void;

/**
*  Return an array of the gradation values used in the current chart. Pass these values
*  to FacetChart.getXCoord / FacetChart.getYCoord (depending on the orientation of the chart)
*  to discover the coordinates where gradations are drawn.
*  
*  This is only allowed to be called when FacetChart.chartDrawn fires.
* 
* 
*/
getGradations(): Array<Partial<Float>>;

/**
*  Returns the Y coordinate where the passed data value either was or would be drawn.
*  For example, this would be the Y coordinate that a line would pass through on
*  a line chart, or the top of a column on a column chart.
*  
*  This is only allowed to be called after FacetChart.chartDrawn fires.
*  
*  If the FacetChart.chartType is
*  "Area", "Bubble", "Column", "Histogram", "Line", or "Scatter"
*  
*  then the value argument should be a number. For
*  "Bar"
*  charts this method expects a FacetValueMap that uniquely identifies the data cell
*  whose Y-axis coordinate is to be retrieved.
* 
* 
* @param {Float | FacetValueMap} the value to be drawn.
*/
getYCoord(value:Float | FacetValueMap): Float;

/**
*  Returns the X coordinate where the passed data value either was or would be drawn.
*  For example, this would be the X coordinate where a bar would end in a bar
*  chart.
*  
*  This is only allowed to be called after FacetChart.chartDrawn fires.
*  
*  If the FacetChart.chartType is
*  "Bar", "Bubble", or "Scatter"
*  
*  then the value argument should be a number. For other rectangular charts,
*  this method expects a FacetValueMap that uniquely identifies the data cell whose
*  X-axis coordinate is to be retrieved.
*  
*  Note that when FacetChart.canZoom is enabled, this API is valid only for
*  data values between FacetChart.zoomStartValue and
*  FacetChart.zoomEndValue.
* 
* 
* @param {Float | FacetValueMap} the value to be drawn.
*/
getXCoord(value:Float | FacetValueMap): Float;

/**
*  Returns the centerpoint for radar charts and pie charts.
*  
*  Note that unstacked pie charts draw multiple pies, each with their own centers.
*  
*  This is only allowed to be called when FacetChart.chartDrawn fires.
* 
* 
*/
getChartCenter(): Point;

/**
*  Returns the radius for radar charts and pie charts. For stacked pie charts this is radius of
*  the outermost pie.
*  
*  Note that unstacked pie charts draw multiple pies, each with their own radii.
*  
*  This is only allowed to be called when FacetChart.chartDrawn fires.
* 
* 
*/
getChartRadius(): Float;

/**
*  Return the text string to display along the axes for FacetChart.gradationLabelProperties or facet value labels given the raw metric value or facet value id.
*  
*  Note that this formatter will be called for the gradation labels on an axis, including those
*  generated for LabelCollapseMode "numeric". It also will be called for each facet
*  value along the linear axes of a chart (vertical or horizontal) when LabelCollapseMode
*  is not active, unless a FacetValue.title has been specified (in 
*  Facet.values) or the raw value is a string and FacetChart.formatStringFacetValueIds is
*  false.
*  
*  Note that the rendering of values in hovers or via FacetChart.showDataValues is handled
*  by FacetChart.formatDataValue.
* 
* @param {any} raw value of the metric or facet value id
* @param {boolean} whether the raw value is for the horizontal/x-axis (true) or for the vertical/y-axis (false)
*/
formatAxisValue(value:any, forHorizontalAxis:boolean): string;

/**
*  Return the text string to display for facet value labels that appear in chart legends or as
*  labels for FacetChart.chartTypes that have circumference or non-axis labels, such as
*  for example "Pie" or "Radar" charts.
*  
* 
* @param {any} raw value of the metric
* @param {Facet} facet containing the value
*/
formatFacetValueId(value:any, facet:Facet): string;

/**
*  Return the text string to display in labels 
*  FacetChart.showDataValuesMode given the raw value 
*  for the metric displayed on the value axis.
*  
*  This method may also be passed the FormattingContext for which the 
*  value is being formatted, the record associated with the value and the id of its 
*  FacetChart.facets.
*  
*  Note that the rendering of values for gradation labels is handled by
*  FacetChart.formatAxisValue.
* 
* @param {any} raw value of the metric
* @param {FormattingContext=} context in which the value is being displayed
* @param {Record=} cell record for the data value
* @param {string=} facet ID for the data value
*/
formatDataValue(value:any, context?:FormattingContext, record?:Record, facetId?:string): string;

/**
*  Count the number of data points.
* 
* @param {FacetValueMap=} a set of facetValues describing a slice of the data
*/
getNumDataPoints(criteria?:FacetValueMap): Integer;

/**
*  Calculate the mean, or expected value, of the data over a single metric.
*  See http://en.wikipedia.org/wiki/Expected_value.
*  
*  The first argument, criteria, determines which metric is used to calculate the mean.
*  The criteria may be a String that is the "id" of some FacetValue of
*  the metric facet, or a FacetValueMap that contains an entry for the metric facet,
*  or null to use the FacetChart.valueProperty. A FacetValueMap criteria may also be used to restrict
*  the calculation to a slice of the data.
* 
* @param {string | FacetValueMap} the "id" of a metric facet value, or a set of mappings describing the data over which to calculate, or null
*/
getMean(criteria:string | FacetValueMap): Float;

/**
*  Calculate the median of the data over a single metric.
*  See http://en.wikipedia.org/wiki/Median.
*  
*  The first argument, criteria, determines which metric is used to calculate the median.
*  The criteria may be a String that is the "id" of some FacetValue of
*  the metric facet, or a FacetValueMap that contains an entry for the metric facet,
*  or null to use the FacetChart.valueProperty. A FacetValueMap criteria may also be used to restrict
*  the calculation to a slice of the data.
* 
* @param {string | FacetValueMap} the "id" of a metric facet value, or a set of mappings describing the data over which to calculate, or null
*/
getMedian(criteria:string | FacetValueMap): Float;

/**
*  Calculate a percentile of the data over a single metric.
*  See http://en.wikipedia.org/wiki/Percentile.
*  
*  The first argument, criteria, determines which metric is used to calculate a percentile.
*  The criteria may be a String that is the "id" of some FacetValue of
*  the metric facet, or a FacetValueMap that contains an entry for the metric facet,
*  or null to use the FacetChart.valueProperty. A FacetValueMap criteria may also be used to restrict
*  the calculation to a slice of the data.
*  
*  The second argument is the percentile to calculate and it must be a number from 0 to 100.
* 
* @param {string | FacetValueMap} the "id" of a metric facet value, or a set of mappings describing the data over which to calculate, or null
* @param {Float} the percentile to calculate
*/
getPercentile(criteria:string | FacetValueMap, percentile:Float): Float;

/**
*  Calculate the range of the data from a single metric.
*  
*  The first argument, criteria, determines which metric is used to calculate the range.
*  The criteria may be a String that is the "id" of some FacetValue of
*  the metric facet, or a FacetValueMap that contains an entry for the metric facet,
*  or null to use the FacetChart.valueProperty. A FacetValueMap criteria may also be used to restrict
*  the calculation to a slice of the data.
* 
* @param {string | FacetValueMap} the "id" of a metric facet value, or a set of mappings describing the data over which to calculate, or null
*/
getRange(criteria:string | FacetValueMap): Float;

/**
*  Calculate the standard deviation of the data from a single metric.
*  See http://en.wikipedia.org/wiki/Standard_deviation.
*  
*  The first argument, criteria, determines which metric is used to calculate the standard deviation.
*  The criteria may be a String that is the "id" of some FacetValue of
*  the metric facet, or a FacetValueMap that contains an entry for the metric facet,
*  or null to use the FacetChart.valueProperty. A FacetValueMap criteria may also be used to restrict
*  the calculation to a slice of the data.
* 
* @param {string | FacetValueMap} the "id" of a metric facet value, or a set of mappings describing the data over which to calculate, or null
* @param {boolean} false to calculate a sample standard deviation, true to calculate a population standard deviation
*/
getStdDev(criteria:string | FacetValueMap, population:boolean): Float;

/**
*  Calculate the variance of the data from a single metric.
*  See http://en.wikipedia.org/wiki/Variance.
*  
*  The first argument, criteria, determines which metric is used to calculate the variance.
*  The criteria may be a String that is the "id" of some FacetValue of
*  the metric facet, or a FacetValueMap that contains an entry for the metric facet,
*  or null to use the FacetChart.valueProperty. A FacetValueMap criteria may also be used to restrict
*  the calculation to a slice of the data.
* 
* @param {string | FacetValueMap} the "id" of a metric facet value, or a set of mappings describing the data over which to calculate, or null
* @param {boolean} false to calculate a sample variance, true to calculate a population variance
*/
getVariance(criteria:string | FacetValueMap, population:boolean): Float;

/**
*  Calculate the maximum of the data from a single metric.
*  
*  The first argument, criteria, determines which metric is used to calculate the maximum.
*  The criteria may be a String that is the "id" of some FacetValue of
*  the metric facet, or a FacetValueMap that contains an entry for the metric facet,
*  or null to use the FacetChart.valueProperty. A FacetValueMap criteria may also be used to restrict
*  the calculation to a slice of the data.
* 
* @param {string | FacetValueMap} the "id" of a metric facet value, or a set of mappings describing the data over which to calculate, or null
*/
getMax(criteria:string | FacetValueMap): Float;

/**
*  Calculate the minimum of the data from a single metric.
*  
*  The first argument, criteria, determines which metric is used to calculate the minimum.
*  The criteria may be a String that is the "id" of some FacetValue of
*  the metric facet, or a FacetValueMap that contains an entry for the metric facet,
*  or null to use the FacetChart.valueProperty. A FacetValueMap criteria may also be used to restrict
*  the calculation to a slice of the data.
* 
* @param {string | FacetValueMap} the "id" of a metric facet value, or a set of mappings describing the data over which to calculate, or null
*/
getMin(criteria:string | FacetValueMap): Float;

/**
*  Setter for RegressionLineType.
* 
* @param {RegressionLineType} New value for this.regressionLineType
*/
setRegressionLineType(regressionLineType:RegressionLineType): void;

/**
*  Setter for FacetChart.showRegressionLine.
* 
* @param {boolean} New value for this.showRegressionLine.
*/
setShowRegressionLine(showRegressionLine:boolean): void;

/**
*  Setter for FacetChart.regressionPolynomialDegree.
* 
* @param {number} New value for this.regressionPolynomialDegree.
*/
setRegressionPolynomialDegree(regressionPolynomialDegree:number): void;

/**
*  For scatter plots only, get a Function from the specified independent variable X to the
*  specified dependent variable Y that defines the line that best fits the data.
*  See http://en.wikipedia.org/wiki/Simple_linear_regression.
* 
* @param {string=} ID of an inlined facet value to use as the independent variable. Defaults to the FacetChart.xAxisMetric.
* @param {string=} ID of an inlined facet value to use as the dependent variable. Defaults to the FacetChart.yAxisMetric.
*/
getSimpleLinearRegressionFunction(xMetric?:string, yMetric?:string): void;

/**
*  For scatter plots only, get a Function from the specified independent variable X to the
*  specified dependent variable Y that defines the polynomial that best fits the data.
*  See http://en.wikipedia.org/wiki/Polynomial_regression.
* 
* @param {Integer=} the degree of the polynomial. Defaults to FacetChart.regressionPolynomialDegree.
* @param {string=} ID of an inlined facet value to use as the independent variable. Defaults to the FacetChart.xAxisMetric.
* @param {string=} ID of an inlined facet value to use as the dependent variable. Defaults to the FacetChart.yAxisMetric.
*/
getPolynomialRegressionFunction(degree?:Integer, xMetric?:string, yMetric?:string): void;

/**
*  When FacetChart.showDataPoints is true, fires when a point is clicked on.
* 
* 
* @param {Float} the value at the point
* @param {Record} the record at the point
* @param {string} the ID of the metric at the point
*/
pointClick(value:Float, record:Record, metricId:string): void;

/**
*  When FacetChart.showDataPoints is true and the mouse hovers over a point, this method is called
*  and may return HTML to show in a hover.
* 
* 
* @param {Float} the value at the point
* @param {Record} the record at the point
* @param {string} the ID of the metric at the point
*/
getPointHoverHTML(value:Float, record:Record, metricId:string): HTMLString;

/**
*  Returns the Facet in the list of FacetChart.facets whose
*  Facet.values are rendered in the chart's legend.
*  
*  Most single-facet charts do not have a legend facet. The exceptions are that single-facet
*  Pie/Doughnut charts have a legend facet as the first facet and Bubble and Scatter charts
*  may optionally have a legend facet as the second facet, after the metric facet.
*  
*  In all multi-facet charts, the legend facet is the second facet.
*  
*  Note that the user may swap the legend facet and the
*  FacetChart.getDataLabelFacet in most chart types using the context menu.
* 
*/
getLegendFacet(): Facet;

/**
*  Returns the Facet in the list of FacetChart.facets whose
*  Facet.values are rendered as labels along the data axis of the chart or in
*  the main chart area.
*  
*  Most single-facet charts and all multi-facet charts have the data label facet as their
*  first facet. The exceptions are that single-facet Pie/Doughnut charts and Bubble and
*  Scatter charts do not have data label facets.
*  
*  Note that the user may swap the data label facet and the
*  FacetChart.getLegendFacet in most chart types using the context menu.
* 
*/
getDataLabelFacet(): Facet;

/**
*  Setter for FacetChart.dataColors.
* 
* @param {Array<Partial<CSSColor>>} New set of data colors
*/
setDataColors(dataColors:Array<Partial<CSSColor>>): void;

/**
*  Get a color from the FacetChart.dataColors Array.
*  
*  
*  Override to provide a dynamic color generation scheme.
*  
*  In most cases, the method is passed the associated data Record, which can
*  be used to retrieve the value being processed.
*  
*  Must return a color in the format of a leading hash (#) plus 6 hexadecimal digits 
*  as specified for FacetChart.dataColors.
*  
* 
* 
* @param {number} index of the legend facet value to be colored
* @param {string | number | Date} id of the legend facet value to be colored
* @param {string} purpose for the requested color - such as "legend", "line",             "area", "points", etc.
* @param {Record=} record associated with this call
*/
getDataColor(index:number, facetValueId:string | number | Date, purpose:string, record?:Record): CSSColor;

/**
*  Specifies the color to use for data lines in the chart. No default implementation. If not
*  defined or null is returned, the Framework will default to value of FacetChart.getDataColor.
*  
*  
*  Note that this method is simply an override point, since it has no default implementation -
*  must return a color in the format of of a leading hash (#) plus 6 hexadecimal digits as
*  specified for FacetChart.dataColors.
*  
* 
* 
* @param {number} index of the legend facet value to be colored
* @param {string | number | Date} id of the legend facet value to be colored
* @param {string} purpose for the requested color - such as "legend", "line",             "area", "points", etc.
*/
getDataLineColor(index:number, facetValueId:string | number | Date, purpose:string): CSSColor;

/**
*  Specifies the width to use for data lines in the chart. No default implementation. If not
*  defined or null is returned, the line width will be set by the appropriate chart properties,
*  such as FacetChart.dataLineProperties, FacetChart.barProperties, or FacetChart.bubbleProperties.
*  
*  
*  Note that this method is simply an override point, since it has no default implementation.
*  
* 
* 
* @param {number} index of the legend facet value to target
* @param {string | number | Date} id of the legend facet value to target
* @param {string} purpose for the requested width - such as "legend", "line",             "area", "points", etc.
*/
getDataLineWidth(index:number, facetValueId:string | number | Date, purpose:string): Integer;

/**
*  Get a gradient from the FacetChart.dataGradients Array.
*  
*  Override to provide a dynamic gradient generation scheme.
* 
* 
* @param {number} index of the legend facet value to be colored
* @param {string | number | Date} id of the legend facet value to be colored
* @param {string} purpose for the requested gradient - such as "legend", "line",             "area", "points", etc.
*/
getDataGradient(index:number, facetValueId:string | number | Date, purpose:string): string;

/**
*  Returns a DrawnValue object for the data value that is shown nearest to the passed
*  coordinates only if it's under the given coordinates, or under the current mouse event
*  coordinates if no coordinates are passed. This method is similar to FacetChart.getNearestDrawnValue,
*  but the DrawnValue is only returned if it's under the coordinates.
*  
*  See FacetChart.drawnValueContainsPoint for the criteria that determine whether a DrawnValue is
*  under (contains) the coordinates.
* 
* @param {Integer=} X coordinate. If this parameter is specified, then y is a required parameter.
* @param {Integer=} Y coordinate
* @param {string=} metric over which to determine the drawn value
*/
getDrawnValueAtPoint(x?:Integer, y?:Integer, metric?:string): DrawnValue;

/**
*  Returns an array of DrawnValue objects for the data values of each metric that are shown
*  nearest to the passed coordinates, but only if they're under the given coordinates, or under
*  the current mouse event coordinates if no coordinates are passed. This method is similar to
*  FacetChart.getNearestDrawnValues, but DrawnValues are only included in the returned array if
*  they're under the coordinates.
*  
*  See FacetChart.drawnValueContainsPoint for the criteria that determine whether a DrawnValue is under
*  (contains) the coordinates.
* 
* @param {Integer=} X coordinate. If this parameter is specified, then y is a required parameter.
* @param {Integer=} Y coordinate
*/
getDrawnValuesAtPoint(x?:Integer, y?:Integer): Array<Partial<DrawnValue>>;

/**
*  Returns whether a given DrawnValue contains a point. The point's X and Y coordinates may
*  be passed into this method, or, if unspecified, the coordinates used are the current mouse event
*  coordinates.
*  
*  For Area, Bubble, Line, Radar, and Scatter charts, a DrawnValue is considered to contain a
*  point if the Euclidean distance from the DrawnValue's center (DrawnValue.x,
*  DrawnValue.y) to the point is less than
*  FacetChart.dataPointSize. For Pie charts, the DrawnValue is
*  considered to contain a point if the point is within the pie slice. Similarly, for Doughnut
*  charts, the DrawnValue is considered to contain a point if the point is within the pie
*  slice and not in the doughnut hole. For Bar and Column charts, the DrawnValue is considered
*  to contain a point if the point is within the bar or column, respectively. Note that for
*  stacked Bar and Column charts, the point must also be in the stacked portion as opposed to
*  anywhere within the bar or column.
* 
* 
* @param {DrawnValue} the DrawnValue to check. The DrawnValue must be a valid DrawnValue from this chart.
* @param {Integer=} X coordinate of the point. If this parameter is specified, then y is a required parameter.
* @param {Integer=} Y coordinate of the point
*/
drawnValueContainsPoint(drawnValue:DrawnValue, x?:Integer, y?:Integer): boolean;

/**
*  Returns rendering information for the data value that is shown nearest to the passed
*  coordinates, as a DrawnValue object.
*  
*  Passed X and Y coordinates should be relative to the FacetChart. If neither an X or Y
*  coordinate is passed, both X and Y will use the current Canvas.getOffsetX and
*  Canvas.getOffsetY.
*  
*  If called before FacetChart.chartDrawn, logs a warning and returns null. For a chart with
*  multiple vertical axes, returns the nearest point from the first metric only (see
*  FacetChart). For scatter charts, returns a DrawnValue where the
*  DrawnValue.value is from the FacetChart.yAxisMetric.
*  
*  To get the nearest DrawnValue only if it contains the given coordinates, you can either use
*  the FacetChart.getDrawnValueAtPoint API or call FacetChart.drawnValueContainsPoint on the return
*  value.
*  
*  Behavior for different chart types is as follows:
*  
*  Bar / Column
*  
*  Returns the centerpoint of the end of the nearest bar or column by considering the Y
*  coordinate (bar) or X coordinate (column) only.
*  
*  Line / Area
*  
*  Returns the nearest point based on which data label is nearest to the passed X coordinate.
*  For multi-series charts, if Y coordinate is not passed the data point returned is from the
*  series that has the highest value at the data label.
*  
*  Radar
*  
*  Returns the data point nearest the passed coordinates by straight line distance.
*  Passing only one coordinate will cause a warning to be logged and null to be returned;
*  passing neither coordinate is allowed (getOffsetX/Y will be used).
*  
*  Pie
*  
*  Returns the data point for the segment that would be hit if a line were drawn from the
*  passed coordinates to the center of the pie.
*  
*  If there are multiple stacked pies, uses the pie that contains the passed coordinates,
*  otherwise the outermost pie.
*  
*  If there are multiple non-stacked pies, uses the pie that is nearest the passed coordinates
*  by straight-line distance to the center of the pie.
*  
*  Passing only one coordinate will cause a warning to be logged and null to be returned;
*  passing neither coordinate is allowed (getOffsetX/Y will be used).
*  
*  If the chart is a FacetChart.extraAxisMetrics then this method takes an
*  optional parameter, metric, which causes this method to return a DrawnValue
*  from the specified metric. If a metric is not passed then the first metric of the metric facet
*  will be used (or just the FacetChart.valueProperty if there is no metric facet).
* 
* 
* @param {Integer=} X coordinate. If this parameter is specified, then y is a required parameter.
* @param {Integer=} Y coordinate
* @param {string=} metric over which to determine the drawn value
*/
getNearestDrawnValue(x?:Integer, y?:Integer, metric?:string): DrawnValue;

/**
*  Returns an array of DrawnValue objects containing rendering information for the data
*  values having each metric that are shown nearest to the passed coordinates.
*  
*  Passed X and Y coordinates should be relative to the FacetChart. If neither an X or Y
*  coordinate is passed, both X and Y will use the current Canvas.getOffsetX and
*  Canvas.getOffsetY.
*  
*  The behavior for different chart types is the same as FacetChart.getNearestDrawnValue. This
*  method also logs a warning and returns null if called before FacetChart.chartDrawn.
*  
*  To get the nearest DrawnValues only if they contain the given coordinates, you can either use
*  the FacetChart.getDrawnValuesAtPoint API or check whether each DrawnValue in the returned array
*  contains the point by calling FacetChart.drawnValueContainsPoint.
* 
* 
* @param {Integer=} X coordinate. If this parameter is specified, then y is a required parameter.
* @param {Integer=} Y coordinate
*/
getNearestDrawnValues(x?:Integer, y?:Integer): Array<Partial<DrawnValue>>;

/**
*  Returns rendering information for the data value specified by the passed facet values.
*  
*  If called before FacetChart.chartDrawn, logs a warning and returns null.
* 
* 
* @param {FacetValueMap} facet values of desired data value
*/
getDrawnValue(facetValues:FacetValueMap): DrawnValue;

/**
*  Returns rendering information for the data values specified by the passed facet values.
*  
*  If called before FacetChart.chartDrawn, logs a warning and returns null.
* 
* @param {FacetValueMap=} facet values of desired DrawnValues. If no FacetValueMap is provided, then all DrawnValues are returned.
*/
getDrawnValues(facetValues?:FacetValueMap): Array<Partial<DrawnValue>>;

/**
*  For a FacetChart.canZoom, simultaneously sets the
*  FacetChart.setZoomStartValue and
*  FacetChart.setZoomEndValue.
* 
* @param {any} starting value for the data range shown in the main chart
* @param {any} ending value for the data range shown in the main chart
*/
zoomTo(startValue:any, endValue:any): void;

/**
*  Setter for FacetChart.zoomStartValue.
*  
*  Note that the FacetChart.zoomStartValue and FacetChart.zoomEndValue may be set simultaneously using
*  FacetChart.zoomTo.
* 
* @param {any} New start value for the data range shown in the main chart
*/
setZoomStartValue(zoomStartValue:any): void;

/**
*  Setter for FacetChart.zoomEndValue.
*  
*  Note that the FacetChart.zoomStartValue and FacetChart.zoomEndValue may be set simultaneously using
*  FacetChart.zoomTo.
* 
* @param {any} New end value for the data range shown in the main chart
*/
setZoomEndValue(zoomEndValue:any): void;

/**
*  Retrieves data from the DataSource that matches the specified criteria.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
*  
*  When fetchData() is first called, if data has not already been provided via
*  ListGrid.setData, this method will create a ResultSet, which will be
*  configured based on component settings such as DataBoundComponent.fetchOperation
*  and DataBoundComponent.dataPageSize, as well as the general purpose
*  ListGrid.dataProperties. The created ResultSet will automatically send a DSRequest
*  to retrieve data from ListGrid.dataSource, and from then on will 
*  automatically manage paging through large datasets, as well as performing filtering and
*  sorting operations inside the browser when possible - see the ResultSet docs for
*  details.
*  
*  NOTE: do not use both DataBoundComponent.autoFetchData and a
*  call to fetchData() - this may result in two DSRequests to fetch data. Use
*  either DataBoundComponent.autoFetchData and Criteria or a manual call to fetchData()
*  passing criteria.
*  
*  Whether a ResultSet was automatically created or provided via ListGrid.setData, subsequent
*  calls to fetchData() will simply call ResultSet.setCriteria.
*  
*  Changes to criteria may or may not result in a DSRequest to the server due to
*  ResultSet.useClientFiltering. You can call
*  DataBoundComponent.willFetchData to determine if new criteria will result in a
*  server fetch.
*  
*  If you need to force data to be re-fetched, you can call
*  ListGrid.invalidateCache and new data will automatically be fetched
*  from the server using the current criteria and sort direction. NOTE: when using
*  invalidateCache() there is no need to also call fetchData()
*  and in fact this could produce unexpected results.
*  
*  This method takes an optional callback parameter (set to a DSCallback) to fire when
*  the fetch completes. Note that this callback will not fire if no server fetch is performed.
*  In this case the data is updated synchronously, so as soon as this method completes you
*  can interact with the new data. If necessary, you can use
*  DataBoundComponent.willFetchData to determine whether or not a server
*  fetch will occur when fetchData() is called with new criteria.
*  
*  In addition to the callback parameter for this method, developers can use 
*  ListGrid.dataArrived to be notified every time data is loaded.
*  
*  By default, this method assumes a TextMatchStyle of "exact"; that can be overridden
*  by supplying a different value in the requestProperties parameter. 
*  See DataBoundComponent.willFetchData;
*  
*  Changing the request properties
*  
*  Changes to TextMatchStyle made via requestProperties will be honored in
*  combination with the fetch criteria, possibly invalidating cache and triggering a server
*  request if needed, as documented for DataBoundComponent.willFetchData.
*  In contrast, changes to
*  DSRequest.operationId in the request properties will cause
*  the ResultSet or ResultTree to be rebuilt, always refetching from the server.
*  However, changes to other request properties after the initial fetch won't be detected, and
*  no fetch will get triggered based on that new request context.
*  
*  To pick up such changes, we recommend that you call FacetChart.setData (passing an
*  empty array to ensure the data model is cleared), and then call this method to fetch again.
*  If you try to do it by calling ListGrid.invalidateCache, you may see duplicate fetches if you
*  haven't already updated the data context by calling this method with the new request
*  properties, and fail to do so before the component is Canvas.redraw.
* 
* 
* @param {Criteria=} Search criteria. If a DynamicForm is passed                     in as this argument instead of a raw criteria                      object, will be derived by calling                      DynamicForm.getValuesAsCriteria
* @param {DSCallback=} callback to invoke when a fetch is complete. Fires                     only if server contact was required
* @param {DSRequest=} additional properties to set on the DSRequest                     that will be issued
*/
fetchData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Based on the relationship between the DataSource this component is bound to and the
*  DataSource specified as the "schema" argument, call fetchData() to retrieve records in this
*  grid that are related to the passed-in record.
*  
*  Relationships between DataSources are declared via DataSourceField.foreignKey.
*  
*  For example, given two related DataSources "orders" and "orderItems", where we want to fetch
*  the "orderItems" that belong to a given "order". "orderItems" should declare a field that
*  is a DataSourceField.foreignKey to the "orders" table (for example, it
*  might be named "orderId" with foreignKey="orders.id"). Then, to load the records related to
*  a given "order", call fetchRelatedData() on the component bound to "orderItems", pass the
*  "orders" DataSource as the "schema" and pass a record from the "orders" DataSource as the
*  "record" argument.
*  
*  Note that multiple foreign keys into the schema are supported by this method.
* 
* 
* @param {ListGridRecord} DataSource record
* @param {Canvas | DataSource | GlobalId} schema of the DataSource record, or              DataBoundComponent already bound to that schema
* @param {DSCallback=} callback to invoke on completion
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
*/
fetchRelatedData(record:ListGridRecord, schema:Canvas | DataSource | GlobalId, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Invalidate the current data cache for this databound component via a call to
*  the dataset's invalidateCache() method, for example,
*  ResultSet.invalidateCache.
*  
*  NOTE: there is no need to call invalidateCache() when a save operation
*  is performed on a DataSource. Automatic cache synchronization features will automatically
*  update caches - see ResultSet for details. If automatic cache synchronization isn't
*  working, troubleshoot the problem using the steps suggested 
*  in the FAQ rather
*  than just calling invalidateCache(). Calling invalidateCache() unnecessarily
*  causes extra server load and added code complexity.
*  
*  Calling invalidateCache() will automatically cause a new fetch to 
*  be performed with the current set of criteria if data had been previously fetched and the
*  component is currently drawn with data visible - there is no need to manually call
*  fetchData() after invalidateCache() and this could result in duplicate fetches. 
*  
*  While data is being re-loaded after a call to invalidateCache(), the widget is
*  in a state similar to initial data load - it doesn't know the total length of the dataset
*  and any APIs that act on records or row indices will necessarily fail and should not be
*  called. To detect that the widget is in this state, call ResultSet.lengthIsKnown.
*  
*  invalidateCache() only has an effect if this components dataset is a data
*  manager class that manages a cache (eg ResultSet or ResultTree). If data was provided as a
*  simple Array or List, invalidateCache() does nothing.
*  
* 
*/
invalidateCache(): void;



}
/**
* Creates a new FacetChart
* 
* @param typeCheckedProperties {Partial<FacetChart>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FacetChart} The new instance
*/
class FacetChart{
static create(typeCheckedProperties? : Partial<FacetChart>, uncheckedProperties?: Object): FacetChart;
/**
* All ChartType that are supported by this class.
*  Should be defined by concrete implementations of the charting interface.
* @type {Array<Partial<ChartType>>}
* @default "["Area", "Bar", "Bubble", "Column", "Doughnut", "Histogram", "Line", "Pie", "Radar", "Scatter"]"
*/
static allChartTypes?: Array<Partial<ChartType>>;

/**
* Title for the "Proportional" option of the context menu for charts that
*  support FacetChart.proportional.
* @type {HTMLString}
* @default "Proportional"
*/
static proportionalContextMenuItemTitle?: HTMLString;

/**
* Title for the "Regression Lines" option of the context menu for scatter plots.
* @type {HTMLString}
* @default "Regression Lines"
*/
static regressionLinesContextMenuItemTitle?: HTMLString;

/**
* Title for the "None" option of the "Regression Lines" option
*  in the context menu for scatter plots.
* @type {HTMLString}
* @default "None"
*/
static hideRegressionLinesContextMenuItemTitle?: HTMLString;

/**
* Title for the "Straight Line" option of the "Regression Lines"
*  option in the context menu for scatter plots.
* @type {HTMLString}
* @default "Straight Line"
*/
static linearRegressionLinesContextMenuItemTitle?: HTMLString;

/**
* Title for the "Polynomial Curve" option of the "Regression Lines"
*  option in the context menu for scatter plots.
* @type {HTMLString}
* @default "Polynomial Curve"
*/
static polynomialRegressionLinesContextMenuItemTitle?: HTMLString;

/**
* Title for the "Polynomial Degree..." option of the "Regression Lines"
*  option in the context menu for scatter plots.
* @type {HTMLString}
* @default "Polynomial Degree..."
*/
static polynomialDegreeRegressionLinesContextMenuItemTitle?: HTMLString;

/**
* Message of the prompt created by the "Polynomial Degree..." option in the context menu
*  for scatter plots to ask the user for a polynomial degree to use for polynomial regression.
* @type {string}
* @default "Enter a degree for the polynomial regression curve (must be a positive integer):"
*/
static polynomialDegreePrompt?: string;

/**
* Warning message issued when an invalid polynomial degree is entered into the prompt dialog created
*  by the "Polynomial Degree..." option in the context menu for scatter plots.
* @type {string}
* @default "The regression polynomial degree must be a positive integer."
*/
static invalidPolynomialDegreeMessage?: string;

/**
* Title for the "Chart Type" submenu in the context menu for 
*  non-scatter/bubble charts.
* @type {HTMLString}
* @default "Chart Type"
*/
static chartTypeContextMenuItemTitle?: HTMLString;

/**
* Title for the "Fill" submenu in the context menu for 
*  non-scatter/bubble charts.
* @type {HTMLString}
* @default "Fill"
*/
static fillContextMenuItemTitle?: HTMLString;

/**
* Title for the "Filled" item in the 
*  FacetChart.fillContextMenuItemTitle submenu of the context menu for 
*  non-scatter/bubble charts.
* @type {HTMLString}
* @default "Filled"
*/
static fillFilledContextMenuItemTitle?: HTMLString;

/**
* Title for the "Unfilled" item in the 
*  FacetChart.fillContextMenuItemTitle submenu of the context menu for 
*  non-scatter/bubble charts.
* @type {HTMLString}
* @default "Unfilled"
*/
static fillUnfilledContextMenuItemTitle?: HTMLString;

/**
* Title for the "Auto" item in the 
*  FacetChart.fillContextMenuItemTitle submenu of the context menu for 
*  non-scatter/bubble charts.
* @type {HTMLString}
* @default "Auto"
*/
static fillAutoContextMenuItemTitle?: HTMLString;

/**
* Title for the "Stack" submenu in the context menu for 
*  non-scatter/bubble charts.
* @type {HTMLString}
* @default "Stack"
*/
static stackContextMenuItemTitle?: HTMLString;

/**
* Title for the "Stacked" item in the 
*  FacetChart.stackContextMenuItemTitle submenu of the context menu for 
*  non-scatter/bubble charts.
* @type {HTMLString}
* @default "Stacked"
*/
static stackStackedContextMenuItemTitle?: HTMLString;

/**
* Title for the "Unstacked" item in the 
*  FacetChart.stackContextMenuItemTitle submenu of the context menu for 
*  non-scatter/bubble charts.
* @type {HTMLString}
* @default "Unstacked"
*/
static stackUnstackedContextMenuItemTitle?: HTMLString;

/**
* Title for the "Auto" item in the 
*  FacetChart.stackContextMenuItemTitle submenu of the context menu for 
*  non-scatter/bubble charts.
* @type {HTMLString}
* @default "Auto"
*/
static stackAutoContextMenuItemTitle?: HTMLString;

/**
* Title for the "Swap Facets" item in the context menu for 
*  non-scatter/bubble charts.
* @type {HTMLString}
* @default "Swap Facets"
*/
static swapFacetsContextMenuItemTitle?: HTMLString;

/**
* Title for the "Area" item in the "Chart Type" submenu in the
*  context menu.
* @type {HTMLString}
* @default "Area"
*/
static chartTypeAreaTitle?: HTMLString;

/**
* Title for the "Column" item in the "Chart Type" submenu in the
*  context menu.
* @type {HTMLString}
* @default "Column"
*/
static chartTypeColumnTitle?: HTMLString;

/**
* Title for the "Bar" item in the "Chart Type" submenu in the
*  context menu.
* @type {HTMLString}
* @default "Bar"
*/
static chartTypeBarTitle?: HTMLString;

/**
* Title for the "Line" item in the "Chart Type" submenu in the
*  context menu.
* @type {HTMLString}
* @default "Line"
*/
static chartTypeLineTitle?: HTMLString;

/**
* Title for the "Radar" item in the "Chart Type" submenu in the
*  context menu.
* @type {HTMLString}
* @default "Radar"
*/
static chartTypeRadarTitle?: HTMLString;

/**
* Title for the "Pie" item in the "Chart Type" submenu in the
*  context menu.
* @type {HTMLString}
* @default "Pie"
*/
static chartTypePieTitle?: HTMLString;

/**
* Title for the "Doughnut" item in the "Chart Type" submenu in the
*  context menu.
* @type {HTMLString}
* @default "Doughnut"
*/
static chartTypeDoughnutTitle?: HTMLString;

/**
* Title for the "Scatter" item in the "Chart Type" submenu in the
*  context menu.
* @type {HTMLString}
* @default "Scatter"
*/
static chartTypeScatterTitle?: HTMLString;

/**
* Title for the "Bubble" item in the "Chart Type" submenu in the
*  context menu.
* @type {HTMLString}
* @default "Bubble"
*/
static chartTypeBubbleTitle?: HTMLString;

/**
* Title for the "Histogram" item in the "Chart Type" submenu in the
*  context menu.
* @type {HTMLString}
* @default "Histogram"
*/
static chartTypeHistogramTitle?: HTMLString;








}

/**
* EditProxy that handles FacetChart when editMode is enabled.
*/
interface FacetChartEditProxy extends EditProxy {
/**
* If InlineEditEvent for this chart edits the
*  FacetChart.data, character that should be used as a separator between
*  values, or between pairs of label vs values.
*  
*  The FacetChartEditProxy.dataEscapeChar can be used to enter the separator
*  char as part of a field name or value.
* @type {string}
* @default ","
*/
dataSeparatorChar?: string;

/**
* If InlineEditEvent for this chart edits the
*  FacetChart.data, character that should be used as a separator for
*  entering label vs value entries.
*  
*  With the default of ":", the following input defines four values with titles:
*  
*    North:10, South:20, East:30, West:40
*  
*  
*  The FacetChartEditProxy.dataEscapeChar can be used to enter literal colon characters.
* @type {string}
* @default ":"
*/
dataDisplaySeparatorChar?: string;

/**
* If InlineEditEvent for this chart edits the
*  FacetChart.data, character that can be used to enter literal separator
*  chars (such as the FacetChartEditProxy.dataSeparatorChar).
*  
*  Repeat this character twice to enter it literally. For example, with the default
*  of "\", inputting "\\" would result in a literal backslash in the value.
* @type {string}
* @default "\"
*/
dataEscapeChar?: string;



/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's facets and data.
*  
*  Lines starting with "--" or "==" are considered titles. A single title
*  is used as the chart title. Titles are matched to the next series of
*  data. If titles are provided for each series, a legend will be shown.
*  
*  Series data can be entered as a list of values separated by commas
*  (see FacetChartEditProxy.dataSeparatorChar) or as a valueMap-style
*  list of label:value pairs. The first data series defines the
*  number of chart values and the titles, if provided.
* 
* 
* @param {string} the new component data
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new FacetChartEditProxy
* 
* @param typeCheckedProperties {Partial<FacetChartEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FacetChartEditProxy} The new instance
*/
class FacetChartEditProxy{
static create(typeCheckedProperties? : Partial<FacetChartEditProxy>, uncheckedProperties?: Object): FacetChartEditProxy;







}

/**
* Fetch data related to a record in another component.
*/
interface FetchRelatedDataTask extends ComponentTask {
/**
* Component to pull record for locating related data.
* @type {GlobalId}
* @default null
*/
recordSourceComponent?: GlobalId;

/**
* The DataSource used with FetchRelatedDataTask.recordSourceComponent to pull related data.
*  If not specified, FetchRelatedDataTask.recordSourceComponent will be used to obtain the schema.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;





}
/**
* Creates a new FetchRelatedDataTask
* 
* @param typeCheckedProperties {Partial<FetchRelatedDataTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FetchRelatedDataTask} The new instance
*/
class FetchRelatedDataTask{
static create(typeCheckedProperties? : Partial<FetchRelatedDataTask>, uncheckedProperties?: Object): FetchRelatedDataTask;







}

/**
* FieldPicker provides a configuration dialog that displays, side-by-side, the available and
*  currently-displayed fields of a DataBoundComponent. It allows for easy customization 
*  of the order in which the fields of a DataBoundComponent are displayed, and of which
*  are visible. If so configured, it also allows for convenient launching of the HiliteEditor,
*  FormulaBuilder, and SummaryBuilder. A FieldPicker instance runs in its own window, 
*  a FieldPickerWindow
*/
interface FieldPicker extends VLayout {
/**
* The component whose fields should be edited.
*  
*  Note that if DataBoundComponent.useAllDataSourceFields is set on the component, it
*  will be cleared when the FieldPicker applies the requested ordering since that setting
*  imposes a fixed ordering on the fields.
* @type {Canvas}
* @default null
*/
dataBoundComponent?: Canvas;

/**
* An optional DataSource that is used to create a disposable
*  FieldPicker.dataBoundComponent if none is provided.
*  Has no effect if a FieldPicker.dataBoundComponent is specified.
* @type {DataSource}
* @default null
*/
dataSource?: DataSource;

/**
* If a sampleRecord is provided, the FieldPicker will show a second column in the
*  Current Fields dialog showing the cell value that will appear for that field given the
*  provided sample record.
*  
*  A value of "first" means the first record. If the underlying
*  FieldPicker.dataBoundComponent is a TreeGrid, you can specify "firstOpenLeaf"
*  to use the first open leaf as the sampleRecord (this is often desirable in trees where the
*  first record may be a folder that's used for organizational purposes only and hence would
*  have no actual data for columns other than the tree column).
* @type {Record | DefaultSampleRecord}
* @default "first"
*/
sampleRecord?: Record | DefaultSampleRecord;

/**
* Shows a "Highlights..." button that shows an interface for editing hilites in the attached
*  DataBoundComponent.
* @type {boolean}
* @default true
*/
showHilitesButton?: boolean;

/**
* When set to false, hides the right-most set of buttons, used for re-ordering fields in the
*  Visible Fields list.
* @type {boolean}
* @default true
*/
showFieldOrderButtons?: boolean;

/**
* @type {string}
* @default "Highlights..."
*/
hilitesText?: string;

/**
* @type {string}
* @default "Available Fields"
*/
availableFieldsTitle?: string;

/**
* @type {string}
* @default "Visible Fields"
*/
currentFieldsTitle?: string;

/**
* The title displayed for the Add Custom Fields Button
* @type {string}
* @default "Add Custom Fields"
*/
addCustomFieldsButtonTitle?: string;

/**
* The title displayed for the title property of the available fields
* @type {string}
* @default "Name"
*/
availableTitleTitle?: string;

/**
* The title displayed for the title property of the current fields
* @type {string}
* @default "Field Title"
*/
currentTitleTitle?: string;

/**
* The title displayed for the sample value property of the current fields
* @type {string}
* @default "Sample Value"
*/
sampleValueTitle?: string;

/**
* Whether the current fields' filter row allows the sample value column to be filtered.
* @type {boolean}
* @default false
*/
canFilterSampleValue?: boolean;

/**
* @type {string}
* @default "Must save pending changes to proceed. OK?"
*/
confirmText?: string;

/**
* @type {string}
* @default "You are about to remove the field. Are you sure?"
*/
removeText?: string;

/**
* @type {HTMLString}
* @default "Drag and drop or use arrows to move fields.  Drag reorder to change field order."
*/
instructions?: HTMLString;

/**
* A Label displaying the text assigned as the FieldPicker's
*  FieldPicker.instructions. Shown across the top of the widget.
* @type {Label}
* @default null
*/
instructionLabel?: Label;

/**
* The title shown on the Save and Exit button
* @type {string}
* @default "Apply"
*/
saveAndExitButtonTitle?: string;

/**
* The title shown on the Cancel button
* @type {string}
* @default "Cancel"
*/
cancelButtonTitle?: string;

/**
* The title shown on the 'Visible Fields' grid's context menu item, whose click handler puts the 
* selected item back in the 'Available Fields' collection.
* @type {string}
* @default "Remove"
*/
removeItemTitle?: string;

/**
* The hint shown when editing a field with no title defined.
* @type {string}
* @default "[No title specified]"
*/
emptyTitleHint?: string;

/**
* Provides a set of controls to appear as 
*  SectionHeader.controls above the available fields grid.
* @type {Array<Partial<Canvas>>}
* @default null
*/
availableFieldsHeaderControls?: Array<Partial<Canvas>>;

/**
* A ListGrid showing the list of available fields.
* @type {ListGrid}
* @default null
*/
availableFieldsGrid?: ListGrid;

/**
* A HLayout used to show the 
*  FieldPicker.saveAndExitButton and FieldPicker.cancelChangesButton 
*  buttons.
* @type {HLayout}
* @default null
*/
buttonLayout?: HLayout;

/**
* An AutoChild IButton that saves the current field-set and exits the 
*  Field Picker.
* @type {IButton}
* @default null
*/
saveAndExitButton?: IButton;

/**
* An AutoChild IButton that saves the current field-set and exits the 
*  Field Picker.
* @type {IButton}
* @default null
*/
cancelChangesButton?: IButton;

/**
* A ListGrid showing the list of currently selected fields.
* @type {ListGrid}
* @default null
*/
currentFieldsGrid?: ListGrid;



/**
*  Callback invoked when picker changes are committed, if a disposable 
*  DataBoundComponent is present.
* 
* @param {Array<Partial<ListGridField>>} committed fields from disposable component
* @param {Array<Partial<Hilite>>} Array of hilite objects
*/
callback(fields:Array<Partial<ListGridField>>, hilites:Array<Partial<Hilite>>): void;

/**
*  Provides a new set of available fields.
* 
* @param {Array<Partial<DataSourceField>>} */
setAvailableFields(newFields:Array<Partial<DataSourceField>>): void;



}
/**
* Creates a new FieldPicker
* 
* @param typeCheckedProperties {Partial<FieldPicker>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FieldPicker} The new instance
*/
class FieldPicker{
static create(typeCheckedProperties? : Partial<FieldPicker>, uncheckedProperties?: Object): FieldPicker;







}

/**
* Class containing properties that configure the display of
*  and interaction with the fields showing in a FieldPicker.
*/
interface FieldPickerField extends Class {




}
/**
* Creates a new FieldPickerField
* 
* @param typeCheckedProperties {Partial<FieldPickerField>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FieldPickerField} The new instance
*/
class FieldPickerField{
static create(typeCheckedProperties? : Partial<FieldPickerField>, uncheckedProperties?: Object): FieldPickerField;
/**
* The title displayed for the frozen property
* @type {string}
* @default "Is Frozen"
*/
static frozenTitle?: string;

/**
* The title displayed for the precision property
* @type {string}
* @default "Precision"
*/
static precisionTitle?: string;

/**
* The title displayed for the decimalPrecision property
* @type {string}
* @default "Decimal Precision"
*/
static decimalPrecisionTitle?: string;

/**
* The title displayed for the decimalPad property
* @type {string}
* @default "Decimal Pad"
*/
static decimalPadTitle?: string;

/**
* The title displayed for the align property
* @type {string}
* @default "Header Text Alignment"
*/
static alignTitle?: string;

/**
* The title displayed for the cellAlign property
* @type {string}
* @default "Data Alignment"
*/
static cellAlignTitle?: string;








}

/**
* A dialog for picking fields to display from among the available fields.
*  
*  This is typically useful in scenarios where there are many more fields than can reasonably
*  fit on screen. The application can start off displaying a few of the fields by default (such
*  as the most commonly-needed fields), and show a FieldPickerWindow to allow the user to
*  customize which fields to display as well as the order in which to display them.
*/
interface FieldPickerWindow extends Window {
/**
* @type {string}
* @default "Field Picker"
*/
title?: string;

/**
* By default, a FieldPickerWindow will close automatically if the mouse is clicked outside
*  of it. To have the window shown with true modality, set this attribute to false.
* @type {boolean}
* @default true
*/
autoDismiss?: boolean;

/**
* A FieldPicker for altering the working field-set in a 
*  DataBoundComponent.
* @type {FieldPicker}
* @default null
*/
fieldPicker?: FieldPicker;





}
/**
* Creates a new FieldPickerWindow
* 
* @param typeCheckedProperties {Partial<FieldPickerWindow>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FieldPickerWindow} The new instance
*/
class FieldPickerWindow{
static create(typeCheckedProperties? : Partial<FieldPickerWindow>, uncheckedProperties?: Object): FieldPickerWindow;







}

/**
* Binary data interface for use in DynamicForms. Allows users to select a single file for
*  upload. In read-only mode (canEdit:false), can display the contents of "imageFile" fields.
* 
*  
*  Editable mode
*  
*  The FileItem.editForm will be automatically generated and displayed as 
*  CanvasItem.canvas, allowing the user to select file(s) to upload.
*  
*  See the upload for information on using this control.
* 
*  
*  Read-only mode
*  
*  For fields of type "blob" the raw data value will be displayed in the 
*  generated FileItem.displayForm.
*  
*  For other fields, the FileItem.displayCanvas will be displayed.
*  
*  For "imageFile" fields with FileItem.showFileInline
*  set to true, the image-file will be streamed and displayed inline within the displayCanvas.
*  
*  Otherwise, the displayCanvas will render out FileItem.viewIconSrc and 
*  FileItem.downloadIconSrc icons and the fileName.
*/
interface FileItem extends CanvasItem {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "FileItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Should this item's value be saved in the form's values and hence returned from
*  DynamicForm.getValues?
*  
*  shouldSaveValue:false is used to mark formItems which do not correspond to
*  the underlying data model and should not save a value into the form's
*  DynamicForm.values. Example includes visual separators, password re-type fields,
*  or checkboxes used to show/hide other form items.
*  
*  A shouldSaveValue:false item should be given a value either via
*  FormItem.defaultValue or by calling
*  DynamicForm.setValue or 
*  FormItem.setValue. Providing a value via
*  DynamicForm.values or DynamicForm.setValues 
*  will automatically switch the item to shouldSaveValue:true.
*  
*  Note that 
*  
* - if an item is shouldSaveValue true, but has no name, a warning is logged, and 
*    shouldSaveValue will be set to false.
*  
* @type {boolean}
* @default true
*/
shouldSaveValue?: boolean;

/**
* When true, allow the file-selection dialog shelled by the browser to select multiple 
*  files.
*  
*  Support is not full-cycle at the server - that is, there are server APIs for retrieving
*  each file that was uploaded, but no built-in support for storing multiple files against
*  a single DataSource field. However, you can write custom server DMI code to do
*  something with the files - for instance, you could create multiple new DataSource 
*  records for each file via a server DMI like this below:
* 
*  
*   String fileNameStr = (String)dsRequest.getValues().get("image_filename").toString();
* 
*   String[] fileNames = fileNameStr.split(", ");
*   List files = dsRequest.getUploadedFiles();
* 
*   for (int i = 0; i 
* @type {boolean}
* @default true
*/
multiple?: boolean;

/**
* A comma-separated list of valid MIME types, used as a filter for the file picker window.
*  
*  Note that this property makes use of the HTML accept attribute, and so
*  relies on the browser to perform the desired filtering. For further study, see:
*  
* - HTML &lt;input&gt; accept Attribute
*  
* - The Input (Form Input) element
*  
* - File input 'accept' attribute - is it useful?
*  
* @type {string}
* @default null
*/
accept?: string;

/**
* Indicates whether to stream the image and display it
*  inline or to display the View and Download icons.
* @type {boolean}
* @default null
*/
showFileInline?: boolean;

/**
* This attribute enables camera capture functionality for mobile devices, accepting the 
*  following values:
*  
*  
* - Set it to "user" to capture using the front-facing camera.
*  
* - Set it to "environment" to capture using the rear-facing camera.
*  
*  
*  Please note that in the latest versions of Android and iOS, utilizing this attribute 
*  will consistently load the rear camera. This behavior is due to the direct camera software's 
*  ability to switch between the two cameras seamlessly.
*  
*  When working with the capture functionality of iPhones and Android devices, it's important 
*  to consider the supported DataSourceField.mimeTypes for audio, video, and image files that 
*  can be used with the fileItem.accept attribute. Here's a list of commonly supported mime types 
*  for capturing on these devices:
*  
*  Supported Image Mime Types:
*  
*  
* - image/jpeg: JPEG image format (.jpg, .jpeg)
*  
* - image/png: Portable Network Graphics format (.png)
*  
*  Supported Audio Mime Types:
*  
*  
* - audio/3gpp: 3GPP format, commonly used for audio capture.
*  
* - audio/mp4: MP4 format, widely supported for audio capture.
*  
*  Supported Video Mime Types:
*  
*  
* - video/3gpp: 3GPP format, commonly used for video capture.
*  
* - video/mp4: MP4 format, widely supported for video capture.
*  
*  Behavior Based on 'accept' Attribute
*  
*  The behavior of using the capture attribute depends on the value used in the accept 
*  attribute. For example:
*  
*  
* - accept="image/*" will load the camera ready to take pictures.
*  
* - accept="audio/*" will load the default audio recorder, not the camera.
*  
* - accept="video/*" will load the camera in video mode, ready to capture videos.
*  
*  Please note that the SmartClient framework does not control the native device behavior 
*  beyond these attribute settings.
*  
*  Lastly, keep in mind that these settings have no effect on desktop browsers; they apply 
*  exclusively to mobile devices.
*  
*  This information is "circa 2023" and may not apply to all devices.
*  
* @type {string}
* @default null
*/
capture?: string;

/**
* The DynamicForm created automatically when FormItem.canEdit
*  is true. Displays a single FileItem.uploadItem for manipulating a file.
*  
*  This component is an AutoChild and as such may be customized via 
*  fileItem.editFormDefaults and
*  fileItem.editFormProperties.
*  
* @type {DynamicForm}
* @default null
*/
editForm?: DynamicForm;

/**
* The UploadItem created automatically and displayed in the 
*  FileItem.editForm when FormItem.canEdit is true.
*  
*  This component is an AutoChild and as such may be customized via 
*  fileItem.uploadItemDefaults and
*  fileItem.uploadItemProperties.
*  
* @type {UploadItem}
* @default null
*/
uploadItem?: UploadItem;

/**
* The DynamicForm created automatically when FormItem.canEdit
*  is false and the field is of type "blob". Displays a single 
*  FileItem.displayItem for viewing the content of a blob file.
*  
*  This component is an AutoChild and as such may be customized via 
*  fileItem.displayFormDefaults and
*  fileItem.displayFormProperties.
* @type {DynamicForm}
* @default null
*/
displayForm?: DynamicForm;

/**
* The StaticTextItem created automatically and displayed in the 
*  FileItem.displayForm when FormItem.canEdit is false and the field type is
*  "blob".
*  
*  This component is an AutoChild and as such may be customized via 
*  fileItem.displayItemDefaults and
*  fileItem.displayItemProperties. 
*  
* @type {StaticTextItem}
* @default null
*/
displayItem?: StaticTextItem;

/**
* The Canvas created automatically when FormItem.canEdit
*  is false and the field is of any type other than "blob".
*  
*  If the field is of type "imageFile", and FileItem.showFileInline 
*  is true, the contents of the canvas are set to HTML that streams the image file for 
*  display. Otherwise, the item renders icons that allow the file to be 
*  FileItem.viewIconSrc or FileItem.downloadIconSrc.
*  
*  This component is an AutoChild and as such may be customized via 
*  fileItem.displayCanvasDefaults and 
*  fileItem.displayCanvasProperties.
* @type {Canvas}
* @default null
*/
displayCanvas?: Canvas;

/**
* Returns the URL for an Icon that will allow the file to be viewed.
* @type {SCImgURL}
* @default "[SKIN]actions/view.png"
*/
viewIconSrc?: SCImgURL;

/**
* Returns the URL for an Icon that will allow the file to be downloaded
* @type {SCImgURL}
* @default "[SKIN]actions/download.png"
*/
downloadIconSrc?: SCImgURL;



/**
*  Updates the FileItem.multiple setting at runtime, propagating it to the Browser's
*  file dialog. Causes a redraw.
*  
* 
* @param {boolean} the HTML of the view link
*/
setMultiple(multiple:boolean): void;



}
/**
* Creates a new FileItem
* 
* @param typeCheckedProperties {Partial<FileItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FileItem} The new instance
*/
class FileItem{
static create(typeCheckedProperties? : Partial<FileItem>, uncheckedProperties?: Object): FileItem;







}

/**
* EditProxy that handles FileItem when editMode is enabled.
*/
interface FileItemEditProxy extends FormItemEditProxy {




}
/**
* Creates a new FileItemEditProxy
* 
* @param typeCheckedProperties {Partial<FileItemEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FileItemEditProxy} The new instance
*/
class FileItemEditProxy{
static create(typeCheckedProperties? : Partial<FileItemEditProxy>, uncheckedProperties?: Object): FileItemEditProxy;







}

/**
* A form that allows the user to input advanced search criteria, including operators on
*  field values such as "less than", and sub-clauses using "AND" and "OR" operators.
*  
*  A FilterBuilder produces an AdvancedCriteria object, which the DataSource
*  subsystem can use to filter datasets, including the ability to perform such filtering within
*  the browser for datasets that are completely loaded.
*  
*  The operators available for each field can be customized at the DataSource level via
*  DataSourceField.validOperators, DataSource.setTypeOperators and related
*  APIs.
*/
interface FilterBuilder extends Layout {
/**
* The message to display next to fieldNames that do not exist in the available dataSource.
* @type {string}
* @default "[missing field definition]"
*/
missingFieldPrompt?: string;

/**
* If specified, the FilterBuilder will dynamically fetch DataSourceField definitions from 
*  this DataSource rather than using FilterBuilder.dataSource. The FieldPicker 
*  will default to being a ComboBoxItem rather than a SelectItem so that the user 
*  will have type-ahead auto-completion.
*  
*  The records returned from the fieldDataSource must have properties 
*  corresponding to a DataSourceField definition, at a minimum, 
*  DataSourceField.name and DataSourceField.type. Any property 
*  legal on a DataSourceField is legal on the returned records, including 
*  DataSourceField.valueMap.
*  
*  Even when a fieldDataSource is specified, FilterBuilder.dataSource may
*  still be specified in order to control the list of 
*  DataSource.setTypeOperators for each field.
* @type {DataSource}
* @default null
*/
fieldDataSource?: DataSource;

/**
* Should the FieldPicker items be sorted alphabetically in the drop down list.
* @type {boolean}
* @default true
*/
sortFields?: boolean;

/**
* AutoChild for the FormItem that allows a user to pick a DataSource field when 
*  creating filter clauses.
*  
*  This will be a SelectItem by default, or a ComboBoxItem if
*  FilterBuilder.fieldDataSource has been specified.
* @type {PickList}
* @default null
*/
fieldPicker?: PickList;

/**
* The title for the FilterBuilder.fieldPicker select-item.
* @type {string}
* @default "Field Name"
*/
fieldPickerTitle?: string;

/**
* Properties to combine with the FieldPicker autoChild FormItem.
* @type {Partial<FormItem>}
* @default null
*/
fieldPickerProperties?: Partial<FormItem>;

/**
* AutoChild for the FormItem that allows a user to select the operator
*  when creating filter clauses. Each clause will create an operatorPicker automatically.
*  To customize this item, use FilterBuilder.operatorPickerProperties
* @type {SelectItem}
* @default null
*/
operatorPicker?: SelectItem;

/**
* Properties to combine with the FilterBuilder.operatorPicker autoChild FormItem.
* @type {Partial<FormItem>}
* @default null
*/
operatorPickerProperties?: Partial<FormItem>;

/**
* The title for the operator-picker select-item.
* @type {string}
* @default "Operator"
*/
operatorPickerTitle?: string;

/**
* Width for the field picker formItem displayed in clauses within this FilterBuilder.
* @type {Integer | string}
* @default "*"
*/
fieldPickerWidth?: Integer | string;

/**
* Width for the operator picker formItem displayed in clauses within this FilterBuilder.
* @type {Integer | string}
* @default "150"
*/
operatorPickerWidth?: Integer | string;

/**
* Width for the value-chooser formItem displayed in clauses within this FilterBuilder.
*  Note that depending on the selected operator type, this item may not be displayed, or
*  may have different characteristics. See FilterBuilder.getValueFieldProperties for information
*  on customizing the value item.
* @type {Integer | string}
* @default "150"
*/
valueItemWidth?: Integer | string;

/**
* DataSource this filter should use for field definitions and available Operators.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* Initial criteria.
*  
*  When initialized with criteria, appropriate clauses for editing the provided criteria will
*  be automatically generated.
*  
*  Note that empty or partial criteria are allowed, for example, criteria that specify
*  Criterion.fieldName only will generate an expression with the operator not chosen
*  yet, and a Criterion with a logical operator ("and" or "or") but not
*  Criterion.criteria defined will generate an empty subclause.
* @type {AdvancedCriteria}
* @default null
*/
criteria?: AdvancedCriteria;

/**
* If true, when the user hits the Enter key while focused in a text-item in this 
*  FilterBuilder, we automatically invoke the user-supplied FilterBuilder.search method.
* @type {boolean}
* @default null
*/
saveOnEnter?: boolean;

/**
* By default only non-hidden fields are shown for selection. To include hidden fields for
*  selection set this property to true.
* @type {boolean}
* @default null
*/
showHiddenFields?: boolean;

/**
* If true (the default), show field titles in the drop-down box used to select a field for querying.
*  If false, show actual field names instead.
* @type {boolean}
* @default true
*/
showFieldTitles?: boolean;

/**
* If true (the default), validates each entered value when it changes, to make sure it is a 
*  a valid value of its type (valid string, number, and so on). No other validation is 
*  carried out. If you switch this property off, it is still possible to validate the 
*  FilterBuilder by calling FilterBuilder.validate from your own code.
* @type {boolean}
* @default true
*/
validateOnChange?: boolean;

/**
* List of explicit fields for user field selection. If not specified, the list of fields is
*  derived from the FilterBuilder.dataSource.
*  
*  Note: this property is not a security feature as it only controls the UI. To consistently
*  limit searchability for certain fields use DataSourceField.canFilter.
* @type {Array<Partial<string>>}
* @default null
*/
allowedFields?: Array<Partial<string>>;

/**
* If set, a button will be shown for each clause allowing it to be removed.
* @type {boolean}
* @default true
*/
showRemoveButton?: boolean;

/**
* The hover prompt text for the remove button.
* @type {string}
* @default "Remove"
*/
removeButtonPrompt?: string;

/**
* The hover prompt text for the remove button in the last remaining clause, when 
*  FilterBuilder.allowEmpty is false.
* @type {string}
* @default "The last clause cannot be removed"
*/
lastClausePrompt?: string;

/**
* The removal ImgButton that appears before each clause if
*  FilterBuilder.showRemoveButton is set.
* @type {ImgButton}
* @default null
*/
removeButton?: ImgButton;

/**
* If set, a button will be shown underneath all current clauses allowing a new clause to be
*  added.
* @type {boolean}
* @default true
*/
showAddButton?: boolean;

/**
* The hover prompt text for the add button.
* @type {string}
* @default "Add"
*/
addButtonPrompt?: string;

/**
* An ImgButton that allows new clauses to be added if FilterBuilder.showAddButton
*  is set.
* @type {ImgButton}
* @default null
*/
addButton?: ImgButton;

/**
* If set to false, the last clause cannot be removed.
* @type {boolean}
* @default false
*/
allowEmpty?: boolean;

/**
* If set to false and showing clause FilterBuilder.showRemoveButton
*  and FilterBuilder.allowEmpty the remove clause button
*  will be hidden.
* @type {boolean}
* @default null
*/
showLastRemoveButton?: boolean;

/**
* Dictates whether values entered by a user should be retained in the value fields when a 
*  different field is selected. Default value is true.
*  
*  Note that, when switching between fields that have an optionDataSource or valueMap, this
*  property is ignored and the values are never retained.
* @type {boolean}
* @default true
*/
retainValuesAcrossFields?: boolean;

/**
* Logical operators to allow for TopOperatorAppearances of "radio" and "bracket".
*  Note that this list may be further limited according to the 
*  DataSource.getTypeOperatorMap returned by the 
*  DataSource.
* @type {Array<Partial<OperatorId>>}
* @default "["and", "or", "not"]"
*/
topOperatorOptions?: Array<Partial<OperatorId>>;

/**
* Default logical operator for all top-level clauses in the FilterBuilder.
*  
*  May be able to be changed by the user via the UI, according to TopOperatorAppearance.
* @type {LogicalOperator}
* @default "and"
*/
topOperator?: LogicalOperator;

/**
* Logical operators to allow if we have a TopOperatorAppearance of "radio".
* @deprecated * in favor of FilterBuilder.topOperatorOptions.

* @type {Array<Partial<OperatorId>>}
* @default "["and", "or", "not"]"
*/
radioOptions?: Array<Partial<OperatorId>>;

/**
* How to display and edit the FilterBuilder.topOperator for this FilterBuilder.
*  
*  See TopOperatorAppearance for a list of options.
* @type {TopOperatorAppearance}
* @default "bracket"
*/
topOperatorAppearance?: TopOperatorAppearance;

/**
* HLayout of radioOperationForm and optional modeSwitcher.
* @type {HLayout}
* @default null
*/
radioOperatorLayout?: HLayout;

/**
* With TopOperatorAppearance:"radio", form that appears above the stack of clauses
*  and allows picking the LogicalOperator for the overall FilterBuilder.
*  
*  By default, consists of a simple RadioGroupItem.
* @type {DynamicForm}
* @default null
*/
radioOperatorForm?: DynamicForm;

/**
* Label to change between simple and advanced mode. When clicked the filter mode
*  is switched to the other mode. This label is only shown if
*  FilterBuilder.showModeSwitcher is true.
*  
*  Shows either FilterBuilder.modeSwitcherSimpleMessage or
*  FilterBuilder.modeSwitcherAdvancedMessage
*  depending on the current state of the filter.
* @type {Label}
* @default null
*/
modeSwitcher?: Label;

/**
* The title for the Operator RadioGroupItem displayed in the FilterBuilder.radioOperatorForm.
* @type {string}
* @default "Overall Operator"
*/
radioOperatorTitle?: string;

/**
* If true, causes a CheckboxItem to appear to the left of each clause in "inline" 
*  TopOperatorAppearance. This checkbox allows the user to select 
*  individual clauses so that, for example, clauses can be removed from the filterBuilder 
*  by application code. This property is ignored for appearances other than "inline".
* @type {boolean}
* @default false
*/
showSelectionCheckbox?: boolean;

/**
* With TopOperatorAppearance "bracket" and "inline", a form that appears to the left
*  of the stack of clauses and allows picking the LogicalOperator for the overall 
*  FilterBuilder (or for that specific FilterClause, in the case of "inline")
*  
*  By default, consists of a CheckboxItem if FilterBuilder.showSelectionCheckbox is true, and a 
*  simple SelectItem containing the available logical operators.
*  
*  If this FilterBuilder shows nested sub-clauses, the same defaults will be applied to the
*  top-operator item for each sub-clause.
* @type {DynamicForm}
* @default null
*/
topOperatorForm?: DynamicForm;

/**
* Automatically generated SelectItem autoChild shown in the FilterBuilder.topOperatorForm.
*  Developers may customize this item using the standard autoChild pattern (by
*  modifying topOperatorItemDefaults and 
*  topOperatorItemProperties).
*  
*  If this FilterBuilder shows nested sub-clauses, the same defaults will be applied to the
*  top-operator item for each sub-clause.
* @type {SelectItem}
* @default null
*/
topOperatorItem?: SelectItem;

/**
* Width for the FilterBuilder.topOperatorItem autoChild.
* @type {number | string}
* @default "*"
*/
topOperatorItemWidth?: number | string;

/**
* The title for the left-aligned Operator selectItem in the FilterBuilder.topOperatorForm.
* @type {string}
* @default "Clause Operator"
*/
topOperatorTitle?: string;

/**
* Default operator for subclauses added via the FilterBuilder.subClauseButton.
* @type {LogicalOperator}
* @default "or"
*/
defaultSubClauseOperator?: LogicalOperator;

/**
* Title for the "Match All" (and) operator 
*  when using TopOperatorAppearance:"radio".
* @type {string}
* @default "Match All"
*/
matchAllTitle?: string;

/**
* Title for the "Match None" (not) operator
*  when using TopOperatorAppearance:"radio".
* @type {string}
* @default "Match None"
*/
matchNoneTitle?: string;

/**
* Title for the "Match Any" (or) operator
*  when using TopOperatorAppearance:"radio".
* @type {string}
* @default "Match Any"
*/
matchAnyTitle?: string;

/**
* Title for the "And" operator (only applicable to the "inline" appearance)
* @type {string}
* @default "and"
*/
inlineAndTitle?: string;

/**
* Title for the "Or" operator (only applicable to the "inline" appearance)
* @type {string}
* @default "or"
*/
inlineOrTitle?: string;

/**
* Title for the "And Not" operator (only applicable to the "inline" appearance)
* @type {string}
* @default "and not"
*/
inlineAndNotTitle?: string;

/**
* Title for the "Advanced.." mode switcher label (only applicable to the "radio" appearance).
* @type {string}
* @default "Advanced.."
*/
modeSwitcherAdvancedMessage?: string;

/**
* Title for the "Simple Mode.." mode switcher label (only applicable to the "bracket" appearance).
* @type {string}
* @default "Simple Mode.."
*/
modeSwitcherSimpleMessage?: string;

/**
* Message displayed when switching to "radio" mode if the criteria will be logically changed.
* @type {string}
* @default "Criteria will be modified to fit in simpler editing interface"
*/
modeSwitcherFlattenWarningMessage?: string;

/**
* When enabled allows FilterBuilder in topOperatorAppearance:"radio" or 
*  topOperatorAppearance:"bracket" mode to be switch to the other view by the user.
*  "radio" mode is considered simple where "bracket" mode is advanced mode.
*  
*  Note that when switching from "bracket" to "radio" mode any entered criteria will be
*  flattened by calling DataSource.flattenCriteria. If the criteria cannot be
*  flattened without losing symantics (see DataSource.canFlattenCriteria) the user is
*  prompted to confirm.
*  
*  If showModeSwitcher is set and topOperatorAppearance is unset:
*  
*  
* - when first drawn, the filterBuilder will choose which mode to use based on the
*    provided FilterBuilder.criteria if any: advanced mode ("bracket") will be used if
*    AdvancedCriteria are provided which cannot be flattened without loss of data (see
*    DataSource.canFlattenCriteria), otherwise simple mode ("radio") will be used. 
*  
* - for any calls to FilterBuilder.setCriteria after draw, the FilterBuilder will
*    switch to advanced mode if the criteria cannot be shown in simple mode without losing
*    information, but will never automatically switch to simple mode, but an explicit call
*    FilterBuilder.setTopOperatorAppearance can be used to do so.
*  
* @type {boolean}
* @default null
*/
showModeSwitcher?: boolean;

/**
* VStack of all clauses that are part of this FilterBuilder
* @type {VStack}
* @default null
*/
clauseStack?: VStack;

/**
* For operators that check that a value is within a range, text to show between the start and
*  end input fields for specifying the limits of the range.
* @type {string}
* @default "and"
*/
rangeSeparator?: string;

/**
* Whether to show a button that allows the user to add subclauses. Defaults to false if 
*  the TopOperatorAppearance is "radio" or "inline", true in all other cases.
* @type {boolean}
* @default "See Description"
*/
showSubClauseButton?: boolean;

/**
* The title of the subClauseButton
* @type {string}
* @default "+()"
*/
subClauseButtonTitle?: string;

/**
* The hover prompt text for the subClauseButton.
* @type {string}
* @default "Add Subclause"
*/
subClauseButtonPrompt?: string;

/**
* Button allowing the user to add subclauses grouped by a LogicalOperator.
* @type {IButton}
* @default null
*/
subClauseButton?: IButton;

/**
* Widget used as a "bracket" to hint to the user that a subclause groups several
*  field-by-field filter criteria under one logical operator.
*  
*  By default, a simple CSS-style Canvas with borders on three sides. A vertical StretchImg
*  could provide a more elaborate appearance.
* @type {Canvas}
* @default null
*/
bracket?: Canvas;



/**
*  Remove a clause this FilterBuilder is currently showing.
* 
* @param {FilterClause} clause as retrieved from filterBuilder.clauses
*/
removeClause(clause:FilterClause): void;

/**
*  Programmatically change the FilterBuilder.topOperator for this FilterBuilder.
* 
* @param {LogicalOperator} new top-level operator
*/
setTopOperator(operator:LogicalOperator): void;

/**
*  Modify TopOperatorAppearance at runtime.
*  
*  Note that when changing from "bracket" to "radio" mode the criteria
*  will be flattened by calling DataSource.flattenCriteria which could
*  result in a logical change to the criteria.
* 
* 
* @param {TopOperatorAppearance} new topOperatorAppearance
*/
setTopOperatorAppearance(appearance:TopOperatorAppearance): void;

/**
*  Add a new FilterClause to this FilterBuilder.
*  
*  This API is intended for the rare use case of adding a highly customized FilterClause
*  component that does not include the standard field/operator/value picking interface, instead
*  providing a custom interface and returning a criterion via FilterClause.getCriterion.
*  
*  If you just want to programmatically add a new FilterClause showing a specific Criterion use
*  FilterBuilder.addCriterion.
*  
*  If you want to use the standard field/operator/value interface but provide a custom control
*  for editing the value, see DataSource.addSearchOperator and Operator.editorType.
*  
* 
* @param {FilterClause} A FilterClause instance
*/
addClause(filterClause:FilterClause): void;

/**
*  Returns an array of child FilterBuilders, representing the list of complex 
*  clauses, or an empty array if there aren't any.
*  
* 
*/
getChildFilters(): Array<Partial<FilterBuilder>>;

/**
*  Returns a human-readable string describing the clauses in this filterBuilder.
*  
* 
*/
getFilterDescription(): string;

/**
*  Validate the clauses of this FilterBuilder.
* 
*/
validate(): boolean;

/**
*  Get the list of OperatorId that are valid for the passed field. By 
*  default, all operators returned by DataSource.getFieldOperators are used.
*  
*  Called automatically by the default implementation of the same method on each
*  FilterClause.getFieldOperators, whenever its fieldName is changed.
*  
* 
* @param {string} the name of the field for which to return the set of available operators
*/
getFieldOperators(fieldName:string): Array<Partial<OperatorId>>;

/**
*  
*  Override to return properties for the FormItem(s) used for the "value" field displayed within
*  clauses within this filterBuilder.
*  
*  Note that the Operator.valueType impacts when this method is called. For operators with
*  valueType "fieldType" or "custom", a single value field is displayed.
*  For operators with valueType "valueRange" two value-field items are displayed
*  (one for the start and one for the end position). The valueItemType parameter may
*  be used to determine which form item is being generated.
*  
* 
* @param {FieldType} type of the DataSource field for this filter row
* @param {string} name of the DataSource field for this filter row
* @param {OperatorId} OperatorId for the chosen operator
* @param {ValueItemType} What valueItem is being generated.
*/
getValueFieldProperties(type:FieldType, fieldName:string, operatorId:OperatorId, itemType:ValueItemType): Partial<FormItem>;

/**
*  Get the criteria entered by the user.
*  
* 
* @param {boolean=} By default if a user has selected a field and operator  type, but has failed to enter a value for the field it will be skipped. This optional parameter  allows you to retrieve all criteria, including those with an empty value attribute.
*/
getCriteria(includeEmptyValues?:boolean): AdvancedCriteria;

/**
*  Set new criteria for editing. 
*  
*  An interface for editing the provided criteria will be generated identically to what happens
*  when initialized with Criteria.
*  
*  Any existing criteria entered by the user will be discarded. 
*  
* 
* @param {AdvancedCriteria} new criteria. Pass null or {} to effectively reset the                  filterBuilder to it's initial state when no criteria are                   specified
*/
setCriteria(criteria:AdvancedCriteria): void;

/**
*  Clear all current criteria.
* 
*/
clearCriteria(): void;

/**
*  Add a new criterion, including recursively adding sub-criteria for a criterion that
*  contains other criteria.
*  
* 
* @param {Criterion} new criterion to be added
*/
addCriterion(criterion:Criterion): void;

/**
*  Returns the type of editor to use for the field.
*  
*  Default behavior is to use the Operator.editorType for a custom operator, otherwise, 
*  use RelativeDateItem for before/after/between operators on date fields, otherwise, 
*  use the same editor as would be chosen by a SearchForm.
* 
* 
* @param {DataSourceField} DataSourceField definition
* @param {OperatorId} OperatorId for the chosen operator
*/
getEditorType(field:DataSourceField, operatorId:OperatorId): SCClassName;

/**
*  Returns the list of this FilterBuilder's FilterClauses that are currently selected. A 
*  clause is "selected" if the user has checked the checkbox next to it; therefore, this 
*  method always returns an empty list unless the 
*  FilterBuilder.showSelectionCheckbox property is set. This method is only 
*  applicable where TopOperatorAppearance is "inline" (because that is the only 
*  appearance that supports showSelectionCheckbox)
* 
* 
*/
getSelectedClauses(): Array<Partial<FilterClause>>;

/**
*  A StringMethod that is automatically invoked if FilterBuilder.saveOnEnter is set 
*  and the user presses Enter whilst in a text-item in any clause or subclause.
* 
* 
* @param {AdvancedCriteria} The criteria represented by the filterBuilder
*/
search(criteria:AdvancedCriteria): void;

/**
*  Handler fired when there is a change() event fired on any FormItem within the 
*  filterBuilder. 
* 
* 
*/
filterChanged(): void;



}
/**
* Creates a new FilterBuilder
* 
* @param typeCheckedProperties {Partial<FilterBuilder>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FilterBuilder} The new instance
*/
class FilterBuilder{
static create(typeCheckedProperties? : Partial<FilterBuilder>, uncheckedProperties?: Object): FilterBuilder;




/**
*  Returns a human-readable string describing the clauses in this advanced criteria or criterion.
*  
* 
* @param {AdvancedCriteria | Criterion} Criteria to convert to a readable string
* @param {DataSource} DataSource to provide definitions of operators
*/
static getFilterDescription(criteria:AdvancedCriteria | Criterion, dataSource:DataSource): string;




}

/**
* A horizontal, Layout-based widget that allows a user to input a single criterion based on 
*  one field and one operator.
*  
*  Note that FilterClauses must be used in conjunction with a FilterBuilder. By default
*  the FilterBuilder will auto-generate its clauses based on specified criteria, but for advanced
*  usage a FilterClause may be instantiated directly and passed to a filterBuilder via 
*  FilterBuilder.addClause.
*/
interface FilterClause extends Layout {
/**
* Initial criterion for this FilterClause.
*  
*  When initialized with a criterion, the clause will be automatically set up for editing
*  the supplied criterion.
*  
*  Note that an empty or partial criterion is allowed, for example, it may specify
*  Criterion.fieldName only and will generate an expression with the operator not chosen.
* @type {Criteria}
* @default null
*/
criterion?: Criteria;

/**
* If true (the default), show field titles in the drop-down box used to select a field for querying.
*  If false, show actual field names instead.
* @type {boolean}
* @default true
*/
showFieldTitles?: boolean;

/**
* If true (the default), validates the entered value when it changes, to make sure it is a 
*  a valid value of its type (valid string, number, and so on). No other validation is 
*  carried out. If you switch this property off, it is still possible to validate the 
*  FilterClause by calling FilterClause.validate from your own code.
* @type {boolean}
* @default true
*/
validateOnChange?: boolean;

/**
* AutoChild for the FormItem that allows a user to pick a DataSource field when 
*  creating filter clauses.
*  
*  This will be a SelectItem by default, or a ComboBoxItem if
*  FilterBuilder.fieldDataSource has been specified.
* @type {PickList}
* @default null
*/
fieldPicker?: PickList;

/**
* The title for the FieldPicker select-item.
* @type {string}
* @default "Field Name"
*/
fieldPickerTitle?: string;

/**
* Properties to combine with the FieldPicker autoChild FormItem.
* @type {Partial<FormItem>}
* @default null
*/
fieldPickerProperties?: Partial<FormItem>;

/**
* AutoChild for the FormItem that allows a user to select the operator
*  when creating filter clauses. Each clause will create an operatorPicker automatically.
*  To customize this item, use FilterClause.operatorPickerProperties
* @type {SelectItem}
* @default null
*/
operatorPicker?: SelectItem;

/**
* Properties to combine with the FilterClause.operatorPicker autoChild FormItem.
* @type {Partial<FormItem>}
* @default null
*/
operatorPickerProperties?: Partial<FormItem>;

/**
* The title for the operator-picker select-item.
* @type {string}
* @default "Operator"
*/
operatorPickerTitle?: string;

/**
* The title for the value-item.
* @type {string}
* @default "Value"
*/
valueItemTitle?: string;

/**
* A hint to show in the value-item when using an operator that takes an array of values.
* @type {string}
* @default "Comma-separated values"
*/
valueSetHint?: string;

/**
* A hint to show in the value-item when using an operator that takes user-entered values.
* @type {string}
* @default "Enter a value"
*/
valueItemTextHint?: string;

/**
* A hint to show in the value-item when using an operator that allows users to select values
*  from a list.
* @type {string}
* @default "Select a value"
*/
valueItemListHint?: string;

/**
* A hint to show in the value-item when using an operator that allows users to select 
*  field-names from a list.
* @type {string}
* @default "Select a field"
*/
valueItemFieldHint?: string;

/**
* AutoChild containing the UI for the filter-properties in this FilterClause.
* @type {SearchForm}
* @default null
*/
clause?: SearchForm;

/**
* If set, show a button for this clause allowing it to be removed.
* @type {boolean}
* @default true
*/
showRemoveButton?: boolean;

/**
* The hover prompt text for the remove button.
* @type {string}
* @default "Remove"
*/
removeButtonPrompt?: string;

/**
* The clause removal ImgButton that appears before this clause if
*  FilterClause.showRemoveButton is set.
* @type {ImgButton}
* @default null
*/
removeButton?: ImgButton;



/**
*  Returns the FilterBuilder containing this clause, or null if
*  this filterClause is not embedded in a filterBuilder.
* 
*/
getFilterBuilder(): void;

/**
*  
*  Override to return properties for the FormItem(s) used for the "value" field displayed in this
*  filterClause.
*  
*  Default implementation simply calls FilterBuilder.getValueFieldProperties on the
*  filterBuilder in which this clause is displayed.
*  
*  Note that the Operator.valueType impacts when this method is called. For operators with
*  valueType "fieldType" or "custom", a single value field is displayed.
*  For operators with valueType "valueRange" two value-field items are displayed
*  (one for the start and one for the end position). The valueItemType parameter may
*  be used to determine which form item is being generated.
*  
* 
* @param {FieldType} type of the DataSource field for this filter row
* @param {string} name of the DataSource field for this filter row
* @param {OperatorId} OperatorId for the chosen operator
* @param {ValueItemType} What valueItem is being generated.
*/
getValueFieldProperties(type:FieldType, fieldName:string, operatorId:OperatorId, itemType:ValueItemType): Partial<FormItem>;

/**
*  Remove this clause by destroy()ing it.
*  
* 
*/
remove(): void;

/**
*  Return the criterion specified by this FilterClause.
*  
* 
*/
getCriterion(): Criteria;

/**
*  Validate this clause.
* 
*/
validate(): boolean;

/**
*  Get the list of OperatorId that are valid for this field. By default, 
*  calls through to the same method on FilterBuilder.getFieldOperators,
*  which defaults to all operators returned by DataSource.getFieldOperators.
*  
*  Called whenever the fieldName is changed.
*  
* 
* @param {string} the name of the field for which to return the set of available      operators
*/
getFieldOperators(fieldName:string): Array<Partial<OperatorId>>;



}
/**
* Creates a new FilterClause
* 
* @param typeCheckedProperties {Partial<FilterClause>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FilterClause} The new instance
*/
class FilterClause{
static create(typeCheckedProperties? : Partial<FilterClause>, uncheckedProperties?: Object): FilterClause;







}

/**
* A subclass of ResultSet designed to provide a synchronously filterable List
*  interface for an array of data.
*  
*  Developers should set FilteredList.allRows to the full set of data objects, and use
*  ResultSet.criteria to the apply criteria to the data set. Standard List APIs 
*  such as List.get, List.getLength, List.getRange, etc will then 
*  allow access to a filtered subset of this data.
*  
*  The FilteredList.dataSource attribute may be used to specify the format of records
*  to be stored within this list, but this is not required. If no DataSource is
*  explicitly specified, filteredList will automatically generate
*  its own DataSource with DataSource.dropUnknownCriteria set to false.
*/
interface FilteredList extends ResultSet {
/**
* Complete set of records for this filteredList. Sorting and filtering of this
*  list will occur synchronously on the client.
* @type {Array<Partial<Record>>}
* @default "[]"
*/
allRows?: Array<Partial<Record>>;

/**
* Optional dataSource to specifying field names and types for records within this
*  List. Note that since a full data set should be provided to the list via
*  FilteredList.allRows, this filteredList will not issue fetch requests against
*  this DataSource.
*  
*  If no DataSource was explicitly specified, filteredList will automatically generate
*  its own DataSource with DataSource.dropUnknownCriteria set to false.
* @type {DataSource}
* @default null
*/
dataSource?: DataSource;

/**
* When true, allows the ResultSet to be modified by list APIs List.addAt,
*  List.set, and List.removeAt. Only applies to 
*  ResultSet.fetchMode:"local" ResultSets, since in all other cases, such
*  modifications would break the consistency of server and client row numbering needed for
*  data paging, and also create some issues with automatic cache synchronization. See the
*  "Modifying ResultSets" subtopic in the ResultSet for the
*  alternative approach of updating the DataSource.
*  
*  One known case where modification can be useful is when an array has been passed to
*  ListGrid.setData for a ListGrid with ListGrid.filterLocalData:true. If
*  the data is filtered using the ListGrid.showFilterEditor, then a new
*  local ResultSet will be created as ListGrid.data to reflect the filtering.
* @type {boolean}
* @default true
*/
modifiable?: boolean;



/**
*  Updates FilteredList.allRows at run time.
* 
* 
* @param {Array<Partial<Record>>} New set of unfiltered cache data
*/
setAllRows(allRows:Array<Partial<Record>>): void;

/**
*  Invoking invalidateCache() will have no effect on a filteredList. To drop the
*  allRows cache of data, consider passing an empty array to 
*  FilteredList.setAllRows.
*  
* 
*/
invalidateCache(): void;



}
/**
* Creates a new FilteredList
* 
* @param typeCheckedProperties {Partial<FilteredList>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FilteredList} The new instance
*/
class FilteredList{
static create(typeCheckedProperties? : Partial<FilteredList>, uncheckedProperties?: Object): FilteredList;







}

/**
* This class is a synonym for LayoutSpacer that can be used to make intent clearer.
*  It is used by some development tools for that purpose.
*/
interface FixedSpacer extends LayoutSpacer {




}
/**
* Creates a new FixedSpacer
* 
* @param typeCheckedProperties {Partial<FixedSpacer>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FixedSpacer} The new instance
*/
class FixedSpacer{
static create(typeCheckedProperties? : Partial<FixedSpacer>, uncheckedProperties?: Object): FixedSpacer;







}

/**
* ISC abstraction for Flashlets.
*/
interface Flashlet extends BrowserPlugin {
/**
* Sets the 'name' attribute on the flashlet object. If a name is not provided it will be
*  auto-generated. Note that in general you don't need to set this. If you have a handle to
*  your ISC Flashlet object you can simply call Flashlet.getPluginHandle to get a
*  handle to the element.
* @type {string}
* @default null
*/
name?: string;

/**
* A map of key/value pairs to pass to the flashlet as parameters. Note that these will be set
*  on the outer &lt;object&gt; element as well as the inner &lt;embed&gt; element.
* @type {object}
* @default null
*/
params?: object;

/**
* This attribute specifies the clsid of the outer &lt;object&gt; tag.
*  
*  The default classID is: "clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"
*  
* @type {string}
* @default "see below"
*/
classID?: string;

/**
* This attribute specifies the minimum version of the flash player required to show this
*  flashlet.
*  
*  The default codeBase is: "http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=5,0,0,0"
*  
* @type {URL}
* @default "see below"
*/
codeBase?: URL;

/**
* This attribute specifies the page the user should go to to get the plugin required to view
*  this flashlet.
*  
*  The default pluginsPage is: "http://www.macromedia.com/shockwave/download/index.cgi?P1_Prod_Version=ShockwaveFlash"
*  
* @type {URL}
* @default "see below"
*/
pluginsPage?: URL;

/**
* Location from which to load the Flashlet.
* @type {URL}
* @default null
*/
src?: URL;



/**
*  
*  Sets the source file for the flash component 
* 
* 
*/
setSrc(): void;

/**
*  
*  Returns a handle to the flashlet DOM element (valid only after the component has been drawn). 
* 
* 
*/
getPluginHandle(): DOMElement;



}
/**
* Creates a new Flashlet
* 
* @param typeCheckedProperties {Partial<Flashlet>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Flashlet} The new instance
*/
class Flashlet{
static create(typeCheckedProperties? : Partial<Flashlet>, uncheckedProperties?: Object): Flashlet;




/**
*  Is Shockwave Flash installed on this browser?
* 
*/
static flashAvailable(): boolean;

/**
*  Which version of Flash is installed on this browser?
* 
*/
static getFlashVersion(): number;




}

/**
* This class is a synonym for LayoutSpacer that can be used to make intent clearer.
*  It is used by some development tools for that purpose.
*/
interface FlexSpacer extends LayoutSpacer {




}
/**
* Creates a new FlexSpacer
* 
* @param typeCheckedProperties {Partial<FlexSpacer>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FlexSpacer} The new instance
*/
class FlexSpacer{
static create(typeCheckedProperties? : Partial<FlexSpacer>, uncheckedProperties?: Object): FlexSpacer;







}

/**
* 
*  A TextItem for managing a text field that displays a floating point value. FloatItem is the
*  default FormItem if the FormItem.type is "float".
*  
*  FloatItem displays its value according to the FormItem.decimalPrecision and
*  FormItem.decimalPad properties of the FormItem. While the value is being edited,
*  the item will display the value with its original precision and without extra zero-padding.
*  
*/
interface FloatItem extends TextItem {




}
/**
* Creates a new FloatItem
* 
* @param typeCheckedProperties {Partial<FloatItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FloatItem} The new instance
*/
class FloatItem{
static create(typeCheckedProperties? : Partial<FloatItem>, uncheckedProperties?: Object): FloatItem;







}

/**
* Arranges a set of Canvas components into rows, flowing into available space so that
*  different numbers of components may appear in each row.
*  
*  FlowLayout is essentially just a subclass of TileLayout where the
*  default TileLayout.layoutPolicy is "flow" instead of "fit".
*/
interface FlowLayout extends TileLayout {




}
/**
* Creates a new FlowLayout
* 
* @param typeCheckedProperties {Partial<FlowLayout>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FlowLayout} The new instance
*/
class FlowLayout{
static create(typeCheckedProperties? : Partial<FlowLayout>, uncheckedProperties?: Object): FlowLayout;







}

/**
* Force-loads the custom fonts declared in the ${isc.DocUtils.linkForRef('group:skinning','current skin\'s CSS file')}
*  and updates any potentially-affected, already-drawn canvii when loading completes. Without
*  FontLoader, auto-sized canvii that use custom fonts may end up with the wrong size if a page
*  is loaded when its custom fonts are not available in the browser cache.
*  
*  To disable FontLoader, set &nbsp;window.isc_loadCustomFonts = false&nbsp; before
*  SmartClient is loaded.
*  
*  If you want to avoid the Framework redrawing canvii after the fonts load, you may want to
*  code your app to delay drawing anything until the skin fonts are loaded. You can check
*  FontLoader.isLoadingComplete to see whether loading is done, and if it's not, you
*  can set a "fontsLoaded" Page.setEvent
*  to run code
*  when it's done or
*  a "fontLoadingFailed" Page.setEvent
*  to
*  run code if there's an error.
*  
*  Advanced Usage
*  FontLoader will use the new 
*  CSS Font Loading API
*  in browsers in which it's available and has proven reliable. Otherwise, we fall back to
*  canvas measurement techniques to detect loading. To force fallback and avoid the API, set
*  &nbsp;window.isc_useCSSFontAPI = false&nbsp;, or to force the API to be used
*  (where it exists but may be unreliable) set &nbsp;window.isc_useCSSFontAPI = 
*  true&nbsp, before SmartClient is loaded.
*  
*  If you're loading additional style sheets, beyond the skin, that declare custom fonts with
*  &#064;font-face, you can request that the FontLoader force-load them as well by
*  specifying them in window.isc_additionalFonts as an array of the font family
*  names matching those from the &#064;font-face declarations.
*  
*  Note that currently, if you have more than one font with the same font family name in your
*  CSS, you'll need to use the CSS Font Loading API approach if you want them all loaded by the
*  FontLoader. Under the measurement approach, the FontLoader is only able to load the default
*  font for each font family, since it has no knowledge beyond the specified family names.
*/
interface FontLoader extends Class {




}
/**
* Creates a new FontLoader
* 
* @param typeCheckedProperties {Partial<FontLoader>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FontLoader} The new instance
*/
class FontLoader{
static create(typeCheckedProperties? : Partial<FontLoader>, uncheckedProperties?: Object): FontLoader;




/**
*  Whether all requested custom fonts have been loaded. If this method returns true, you
*  know that the FontLoader won't need to redraw any canvii drawn afterwards, and
*  that a
*  "fontsLoaded" or "fontLoadingFailed" Page.setEvent
*  won't fire (it may have previously fired).
*  
* 
* 
*/
static isLoadingComplete(): boolean;




}

/**
* Clear form values and errors.
*/
interface FormClearValuesTask extends ComponentTask {




}
/**
* Creates a new FormClearValuesTask
* 
* @param typeCheckedProperties {Partial<FormClearValuesTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormClearValuesTask} The new instance
*/
class FormClearValuesTask{
static create(typeCheckedProperties? : Partial<FormClearValuesTask>, uncheckedProperties?: Object): FormClearValuesTask;







}

/**
* Disable or enable a form field.
*/
interface FormDisableFieldTask extends ComponentTask {
/**
* Field to show/hide.
* @type {FieldName}
* @default null
*/
targetField?: FieldName;

/**
* Should the target form item be disabled?
* @type {boolean}
* @default null
*/
disable?: boolean;





}
/**
* Creates a new FormDisableFieldTask
* 
* @param typeCheckedProperties {Partial<FormDisableFieldTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormDisableFieldTask} The new instance
*/
class FormDisableFieldTask{
static create(typeCheckedProperties? : Partial<FormDisableFieldTask>, uncheckedProperties?: Object): FormDisableFieldTask;







}

/**
* Start editing a new record.
*/
interface FormEditNewRecordTask extends ComponentTask {
/**
* Initial values for new edit record.
*  
*  Data values prefixed with "$" will be treated as a taskInputExpression
*  excluding "$input" and "$inputRecord" references.
* @type {Record}
* @default null
*/
initialValues?: Record;





}
/**
* Creates a new FormEditNewRecordTask
* 
* @param typeCheckedProperties {Partial<FormEditNewRecordTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormEditNewRecordTask} The new instance
*/
class FormEditNewRecordTask{
static create(typeCheckedProperties? : Partial<FormEditNewRecordTask>, uncheckedProperties?: Object): FormEditNewRecordTask;







}

/**
* EditProxy that handles DynamicForm when editMode is enabled.
*/
interface FormEditProxy extends CanvasEditProxy {
/**
* Controls which parts of a FormItem respond to a click
*  and result in selecting the component.
* @type {SelectItemsMode}
* @default "item"
*/
selectItemsMode?: SelectItemsMode;





}
/**
* Creates a new FormEditProxy
* 
* @param typeCheckedProperties {Partial<FormEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormEditProxy} The new instance
*/
class FormEditProxy{
static create(typeCheckedProperties? : Partial<FormEditProxy>, uncheckedProperties?: Object): FormEditProxy;







}

/**
* Edit a record currently showing in some other component. The source
*  record is obtained as follows:
*  
*  
* - for a ListGrid: the first selected record or, if none is selected, the first record
*  
* - for a DynamicForm: the form values
*  
* - for a DetailViewer: the first record
*  
*/
interface FormEditRecordTask extends FormEditNewRecordTask {
/**
* Component to pull record for editing.
* @type {GlobalId}
* @default null
*/
recordSourceComponent?: GlobalId;





}
/**
* Creates a new FormEditRecordTask
* 
* @param typeCheckedProperties {Partial<FormEditRecordTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormEditRecordTask} The new instance
*/
class FormEditRecordTask{
static create(typeCheckedProperties? : Partial<FormEditRecordTask>, uncheckedProperties?: Object): FormEditRecordTask;







}

/**
* Edit a record currently selected in some other component. If nothing is selected
*  a new record is edited.
*/
interface FormEditSelectedTask extends ComponentTask {
/**
* Component to pull record for editing.
* @type {GlobalId}
* @default null
*/
selectionComponentId?: GlobalId;





}
/**
* Creates a new FormEditSelectedTask
* 
* @param typeCheckedProperties {Partial<FormEditSelectedTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormEditSelectedTask} The new instance
*/
class FormEditSelectedTask{
static create(typeCheckedProperties? : Partial<FormEditSelectedTask>, uncheckedProperties?: Object): FormEditSelectedTask;







}

/**
* A UI component that can participate in a DynamicForm, allowing editing or display of one of
*  the DynamicForm.values.
*  
*  FormItems are never created via the Class.create method,
*  instead, an Array of plain Object are passed as
*  DynamicForm.items when the form is created.
* 
*  
*  
*  See the DynamicForm documentation for details and sample code.
*/
interface FormItem extends Class {
/**
* When AutoTest.getElement is used to parse locator strings generated by
*  AutoTest.getLocator for this form item, should the item be identified?
*  If the locator has a specified FormItem.name, it is considered to definitely
*  locate the item and no fallback approach will be used.
*  
*  Otherwise, the following options are available:
*  
*  
* - "title" use the title as an identifier within this form
*  
* - "value" use the value of the item to identify it (often used
*  for items with a static defaultValue such as HeaderItems
*  
* - "index" use the index within the form's items array.
*  
*  If unset, and the locator has no specified name, default behavior is to
*  identify by title (if available), then value (if available), otherwise by index.
* @type {string}
* @default null
*/
locateItemBy?: string;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "FormItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* ARIA role of this formItem. Usually does not need to be manually set - see
*  accessibility.
* @type {string}
* @default null
*/
ariaRole?: string;

/**
* ARIA state mappings for this formItem. Usually this does not need to be manually
*  set - see accessibility.
*  
*  This attribute should be set to a mapping of aria state-names to values - for example
*  to have the "aria-multiline" property be present with a value "true", you'd specify:
*  
*  { multiline : true }
*  
* @type {object}
* @default null
*/
ariaState?: object;

/**
* The DynamicForm picks a field renderer based on the type of the field (and sometimes other
*  attributes of the field).
* @type {FormItemType}
* @default "text"
*/
type?: FormItemType;

/**
* Name of the FormItem to use for editing, eg "TextItem" or "SelectItem".
*  
*  The type of FormItem to use for editing is normally derived automatically from
*  FormItem.type, which is the data type of the field, by the rules
*  explained FormItemType.
* @type {any}
* @default null
*/
editorType?: any;

/**
* Name for this form field. Must be unique within the form as well as a valid JavaScript
*  identifier - see FieldName for details and how to check for validity.
*  
*  The FormItem's name determines the name of the property it edits within the form.
*  
*  Note that an item must have a valid name or FormItem.dataPath in order
*  for its value to be validated and/or saved.
* @type {FieldName}
* @default null
*/
name?: FieldName;

/**
* dataPath for this item. Allows the user to edit details nested data structures in a
*  flat set of form fields
*  
*  NOTE: the dataPath feature is intended to help certain legacy architectures, 
*  such as systems that work in terms of exchanging large messages with several different 
*  entity types in one message, and are incapable of providing separate access to each 
*  entity type.
*  See the DataPath for more information.
*  
*  Note that an item must have a valid dataPath or FormItem.name in order
*  for its value to be validated and/or saved.
* @type {DataPath}
* @default null
*/
dataPath?: DataPath;

/**
* User visible title for this form item.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* Value used when no value is provided for this item. Note that whenever this item's value
*  is cleared programmatically (for example via item.clearValue() or 
*  item.setValue(null)), it will be
*  reverted to the defaultValue. 
*  
*  Developers should use the
*  DynamicForm.values object if their intention is to provide an initial value for a
*  field in a form rather than a value to use in place of null.
*  
*  Developers looking to provide a 'hint' or placeholder value for an empty item may wish to use
*  FormItem.hint (possibly in conjunction with TextItem.showHintInField), or FormItem.prompt.
*  
*  Note: Some items provide a user interface allowing the user to explicitly clear them - for
*  example a standard TextItem. If such an item has a defaultValue specified, and the user explicitly
*  clears that value, the value of the item will be (correctly) reported as null, and will remain
*  null over form item redraw()s. However any programmatic call to set the value to null 
*  (including, but not limited to item.clearValue(), item.setValue(null),
*  dynamicForm.setValues(...) with a null value for this field, etc) will
*  reset the item value to its default.
* @type {any}
* @default null
*/
defaultValue?: any;

/**
* Value for this form item.
*  This value may be set directly on the form item initialization
*  block but is not updated on live items and should not be directly accessed.
*  Once a form item has been created by the dynamicForm use FormItem.setValue and
*  FormItem.getValue directly.
* @type {any}
* @default null
*/
value?: any;

/**
* Global identifier for referring to the formItem in JavaScript. The ID property is
*  optional if you do not need to refer to the widget from JavaScript, or can refer to it
*  indirectly (for example, via form.getItem("itemName")).
*  
*  An internal, unique ID will automatically be created upon instantiation for any formItem
*  where one is not provided.
* @type {GlobalId}
* @default null
*/
ID?: GlobalId;

/**
* Text to display when this form item has a null or undefined value.
*  
*  If the formItem has a databound pickList, and its FormItem.displayField or
*  FormItem.valueField (if the former isn't set) has an undefined
*  ListGridField.emptyCellValue setting, that field's
*  emptyCellValue will automatically be set to the emptyDisplayValue.
* @type {string}
* @default ""
*/
emptyDisplayValue?: string;

/**
* Should this form item be hidden? Setting this property to true on 
*  an item configuration will have the same effect as having a FormItem.showIf
*  implementation which returns false.
*  
*  Note this differs slightly from DataSourceField.hidden. That property 
*  will cause the field in question to be omitted entirely from databound
*  components by default. A dataSourceField with hidden set to
*  true can still be displayed in a DynamicForm either by being
*  explicitly included in the specified DynamicForm.items, or 
*  by having DataBoundComponent.showHiddenFields set to true.
*  In this case, this property will not be inherited onto the FormItem instance,
*  meaning the item will be visible in the form even though the hidden
*  property was set to true on the dataSourceField configuration object.
* @type {boolean}
* @default null
*/
hidden?: boolean;

/**
* In a form, valueMaps are used for FormItem types that allow the user to pick from a
*  limited set of values, such as a SelectItem. The valueMap can be either an Array of
*  legal values or an Object where each property maps a stored value to a user-displayable
*  value.
*  
*  To set the initial selection for a form item with a valueMap, use
*  FormItem.defaultValue.
*  
*  See also DataSourceField.valueMap.
* @type {Array<any> | object}
* @default null
*/
valueMap?: Array<any> | object;

/**
* If set, this FormItem will map stored values to display values as though a
*  ValueMap were specified, by fetching records from the 
*  specified optionDataSource and extracting the
*  FormItem.valueField and 
*  FormItem.displayField in loaded records, to derive one
*  valueMap entry per record loaded from the optionDataSource.
*  
*  With the default setting of FormItem.fetchMissingValues, fetches will be initiated against
*  the optionDataSource any time the FormItem has a non-null value and no corresponding
*  display value is available. This includes when the form is first initialized, as well
*  as any subsequent calls to FormItem.setValue, such as may happen when
*  DynamicForm.editRecord is called. Retrieved values are automatically cached by
*  the FormItem.
*  
*  Note that if a normal, static FormItem.valueMap is also specified for
*  the field (either directly in the form item or as part of the field definition in the
*  dataSource), it will be preferred to the data derived from the optionDataSource for
*  whatever mappings are present.
*  
*  In a databound form, if FormItem.displayField is specified for a FormItem and 
*  optionDataSource is unset, optionDataSource will default to
*  the form's current DataSource
* @type {DataSource | string}
* @default null
*/
optionDataSource?: DataSource | string;

/**
* If this item has a specified optionDataSource, and this property is
*  not null, this will be passed to the datasource as RPCRequest properties when
*  performing the fetch operation on the dataSource to obtain a data-value to display-value
*  mapping
* @type {Partial<RPCRequest>}
* @default null
*/
optionFilterContext?: Partial<RPCRequest>;

/**
* If this item has a specified optionDataSource, and this property may be used
*  to specify criteria to pass to the datasource when
*  performing the fetch operation on the dataSource to obtain a data-value to display-value
*  mapping.
*  
*  The criteria generated for this fetch will consist of the specified optionCriteria, 
*  combined with criteria required to identify the current item value.
*  For example, if a developer's use case was DataSource of user records, with 
*  FormItem.valueField set to "userID"" and FormItem.displayField
*  set to "userName", the generated criteria to retrieve the display value
*  for the item would look for an exact match between the current item value and the
*  userID field in the dataSource. The optionCriteria would allow additional
*  restrictions on this fetch (searching for records matching some other 
*  "region" field, say).
*  The sub-criterion containing the current valueField value will always look for an
*  exact match (rather than any kind of substring match), so if FormItem.optionTextMatchStyle
*  is set to something other than "exact", developers may expect to see AdvancedCriteria
*  passed to the server
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope. Criteria are re-evaluated when
*  the Canvas.getRuleContext changes.
* @type {Criteria}
* @default null
*/
optionCriteria?: Criteria;

/**
* If this item has a specified optionDataSource, this property determines
*  the textMatchStyle to use when interpretating any FormItem.optionCriteria during the
*  fetch to map valueField values to displayField values.
* @type {TextMatchStyle}
* @default "exact"
*/
optionTextMatchStyle?: TextMatchStyle;

/**
* If this item has a specified optionDataSource, this attribute may be set
*  to specify an explicit DSRequest.operationId when performing a fetch against the
*  option dataSource to pick up display value mapping.
* @type {string}
* @default null
*/
optionOperationId?: string;

/**
* If this form item maps data values to display values by retrieving the 
*  FormItem.displayField values from an 
*  FormItem.optionDataSource, this property 
*  denotes the the field to use as the underlying data value in records from the 
*  optionDataSource.
*  If not explicitly supplied, the valueField name will be derived as
*  described in FormItem.getValueFieldName.
* @type {string}
* @default null
*/
valueField?: string;

/**
* If set, this item will display a value from another field to the user instead of
*  showing the underlying data value for the FormItem.name.
*  
*  This property is used in two ways:
*  
*  The item will display the displayField value from the 
*  DynamicForm.getValues if 
*  FormItem.useLocalDisplayFieldValue is true, (or if unset and the conditions
*  outlined in the documentation for that property are met).
*  
*  If this field has an FormItem.optionDataSource, this property is used by
*  default to identify which value to use as a display value in records from this
*  related dataSource. In this usage the specified displayField must be 
*  explicitly defined in the optionDataSource to be used - see 
*  FormItem.getDisplayFieldName for more on this behavior.
*  If not using FormItem.useLocalDisplayFieldValue, the display
*  value for this item will be derived by performing a fetch against the 
*  FormItem.getOptionDataSource 
*  to find a record where the FormItem.getValueFieldName matches 
*  this item's value, and use the displayField value from that record.
*  In addition to this, PickList-based form items that provide a list of possible
*  options such as the SelectItem or ComboBoxItem will show the 
*  displayField values to the user by default, allowing them to choose
*  a new data value (see FormItem.valueField) from a list of user-friendly
*  display values.
*  
*  This essentially allows the specified optionDataSource to be used as
*  a server based valueMap.
*  
*  If FormItem.useLocalDisplayFieldValue
*  are being used and FormItem.storeDisplayValues is true, selecting a new value
*  will update both the value for this field and the associated display-field value
*  on the record being edited.
*  
*  Note: Developers may specify the FormItem.foreignDisplayField property 
*  in addition to displayField. This is useful for cases where the
*  display field name in the local dataSource differs from the display field name in
*  the optionDataSource. See the documentation for
*  DataSourceField.foreignDisplayField for more on this.
*  If a foreignDisplayField is specified, as with just displayField, if 
*  FormItem.useLocalDisplayFieldValue
*  are being used and FormItem.storeDisplayValues is true, when the user 
*  chooses a value the associated display-field value 
*  on the record being edited will be updated. In this case it would be set to the 
*  foreignDisplayField value from the related record. This means foreignDisplayField 
*  is always expected to be set to the equivalent field in the related dataSources.
*  Developers looking to display some other arbitrary field(s) from the
*  related dataSource during editing should consider using custom
*  PickList.pickListFields instead of setting a foreignDisplayField. 
*  
*  Note that if optionDataSource is set and no valid display field is
*  specified,
*  FormItem.getDisplayFieldName will return the dataSource title 
*  field by default.
*  
*  If a displayField is specified for a freeform text based item (such as a 
*  ComboBoxItem), any user-entered value will be treated as a display value.
*  In this scenario, items will derive the data value for the item from the
*  first record where the displayField value matches the user-entered value.
*  To avoid ambiguity, developers may wish to avoid this usage if display values
*  are not unique.
* @type {string}
* @default null
*/
displayField?: string;

/**
* If FormItem.displayField is specified for a field, should the
*  display value for the field be picked up from the 
*  DynamicForm.getValues?
*  
*  This behavior is typically valuable for dataBound components where the
*  displayField is specified at the DataSourceField level. See
*  DataSourceField.displayField for more on this.
*  
*  Note that for DataSources backed by the
*  serverDataIntegration, fields with a specified
*  DataSourceField.foreignKey and DataSourceField.displayField will
*  automatically have this property set to true if not explicitly set to false
*  in the dataSource configuration.
*  
*  Otherwise, if not explicitly set, local display value will be used unless:
*  
*  
* - This item has an explicitly specified optionDataSource, rather than
*    deriving its optionDataSource from a specified 
*    DataSourceField.foreignKey specification
*  
* - The FormItem.name differs from the 
*    FormItem.getValueFieldName for the item
*  
* @type {boolean}
* @default null
*/
useLocalDisplayFieldValue?: boolean;

/**
* For items with an FormItem.optionDataSource, this property specifies an explicit
*  display field for records within the option dataSource. Typically this property
*  will be set in conjunction with FormItem.displayField in the case where
*  the name of the displayField within the record being edited differs from the 
*  displayField in the optionDataSource.
*  
*  See DataSourceField.foreignDisplayField for additional details.
* @type {string}
* @default null
*/
foreignDisplayField?: string;

/**
* If this item is displaying multiple values, this property will be the
*  string that separates those values for display purposes.
* @type {string}
* @default ', '
*/
multipleValueSeparator?: string;

/**
* If this form item has a specified FormItem.optionDataSource, should the
*  item ever perform a fetch against this dataSource to retrieve the related record.
*  
*  The fetch occurs if the item value is non null on initial draw of the form
*  or whenever setValue() is called. Once the fetch completes, the returned record 
*  is available via the FormItem.getSelectedRecord api.
*  
*  By default, a fetch will only occur if FormItem.displayField is specified, and
*  the item does not have an explicit FormItem.valueMap containing the
*  data value as a key.
*  However you can also set FormItem.alwaysFetchMissingValues to have a fetch occur
*  even if no displayField is specified. This ensures 
*  FormItem.getSelectedRecord will return a record if possible - useful for
*  custom formatter functions, etc.
*  
*  Note - for efficiency we cache the associated record once a fetch has been performed, meaning
*  if the value changes, then reverts to a previously seen value, we do not kick
*  off an additional fetch to pick up the display value for the previously seen data value.
*  If necessary this cache may be explicitly invalidated via a call to 
*  FormItem.invalidateDisplayValueCache
* @type {boolean}
* @default true
*/
fetchMissingValues?: boolean;

/**
* If this form item has a specified FormItem.optionDataSource and 
*  FormItem.fetchMissingValues is true, when the item value changes, a fetch will be
*  performed against the optionDataSource to retrieve the related record 
*  if FormItem.displayField is specified and the new item value is not present in any
*  valueMap explicitly specified on the item.
*  
*  Setting this property to true means that a fetch will occur against the optionDataSource 
*  to retrieve the related record even if FormItem.displayField is unset, or the
*  item has a valueMap which explicitly contains this field's value.
*  
*  An example of a use case where this might be set would be if FormItem.formatValue
*  or FormItem.formatEditorValue were written to display properties from the
*  FormItem.getSelectedRecord.
*  
*  Note - for efficiency we cache the associated record once a fetch has been performed, meaning
*  if the value changes, then reverts to a previously seen value, we do not kick
*  off an additional fetch even if this property is true. If necessary this cache may be
*  explicitly invalidated via a call to FormItem.invalidateDisplayValueCache
* @type {boolean}
* @default false
*/
alwaysFetchMissingValues?: boolean;

/**
* Value shown in field when FormItem.fetchMissingValues is active and a
*  fetch is pending. The field is read-only while a fetch is pending.
*  
*  Set to null to show actual value until display value is loaded.
* @type {string}
* @default "Loading..."
*/
loadingDisplayValue?: string;

/**
* If this form item is mapping data values to a display value by fetching records from a 
*  dataSource (see FormItem.optionDataSource, FormItem.displayField 
*  and FormItem.fetchMissingValues), setting this property to true ensures that when
*  the form item value is set, entire data-set from the dataSource is loaded at once and 
*  used as a valueMap, rather than just loading the display value for the current value.
*  This avoids the need to perform fetches each time setValue() is called with a new value.
*  
*  See also PickList.filterLocally for behavior on form items such as SelectItems
*  that show pick-lists.
* @type {boolean}
* @default null
*/
filterLocally?: boolean;

/**
* Display format to use for date type values within this formItem.
*  
*  Note that Fields of type "date", "datetime" or 
*  "time" will be edited using a DateItem or TimeItem by 
*  default, but this can be overridden - for canEdit:false fields, a
*  StaticTextItem is used by default, and the developer can always specify 
*  a custom FormItem.editorType as well as FormItem.type.
*  
*  The FormItem.timeFormatter may also be used to format underlying Date values as
*  times (ommitting the date part entirely). If both dateFormatter and
*  timeFormatter are specified on an item, for
*  fields specified as FormItem.type the
*  timeFormatter will be used, otherwise the dateFormatter
*  
*  If item.dateFormatter and item.timeFormatter is unspecified,
*  date display format may be defined at the component level via
*  DynamicForm.dateFormatter, or for fields of type "datetime"
*  DynamicForm.datetimeFormatter. Otherwise the
*  default is to use the system-wide default short date format, configured via
*  DateUtil.setShortDisplayFormat. Specify any valid DateDisplayFormat
*  to change the format used by this item.
*  
*  Note that if this is a freeform editable field, such a TextItem, with type
*  specified as "date" or "datetime" the system will automatically
*  attempt to parse user entered values back to a Date value, assuming the entered string
*  matches the date format for the field. Developers may further customize this via an
*  explicit FormItem.inputFormat or via entirely custom
*  
*  FormItem.formatEditorValue and FormItem.parseEditorValue methods.
*  
*  
* @type {DateDisplayFormat}
* @default null
*/
dateFormatter?: DateDisplayFormat;

/**
* Time-format to apply to date type values within this formItem. If specified, any
*  dates displayed in this item will be formatted as times using the appropriate format.
*  This is most commonly only applied to fields specified as type "time" though
*  if no explicit FormItem.dateFormatter is specified it will be respected for other 
*  fields as well.
*  
*  If unspecified, a timeFormatter may be defined 
*  DynamicForm.timeFormatter and will be respected by fields
*  of type "time".
* @type {TimeDisplayFormat}
* @default null
*/
timeFormatter?: TimeDisplayFormat;

/**
* Fields of type "date" or "time" will be edited using
*  a DateItem or TimeItem by default.
*  
*  However this can be overridden - for canEdit:false fields, a
*  StaticTextItem is used by default, and the developer can always specify 
*  a custom FormItem.editorType as well as FormItem.type.
*  
*  For fields of type "date", set this property to a valid
*  DateDisplayFormat to specify how the date should be formatted.
*  
*  For fields of type "time", set this property to a valid 
*  TimeDisplayFormat to specify how the time should be formatted.
*  
*  Note that if FormItem.dateFormatter or FormItem.timeFormatter are specified
*  they will take precedence over this setting.
*  
*  If this field is of type "date" and is editable, the 
*  FormItem.inputFormat may be used to specify how user-edited date strings will
*  be parsed.
* @deprecated * in favor of FormItem.format, FormItem.dateFormatter and FormItem.timeFormatter

* @type {any}
* @default null
*/
displayFormat?: any;

/**
* For fields of type "date", if this is an editable field such as a
*  TextItem, this property 
*  allows you to specify the DateItem.inputFormat applied to the item.
* @type {DateInputFormat}
* @default null
*/
inputFormat?: DateInputFormat;

/**
* Applies only to fields of type "float" and affects how many significant digits are shown.
*  
*  For example, with decimalPrecision 3, if the field value is 343.672677, 343.673 is shown.
*  
*  If the value is 125.2, 125.2 is shown - decimalPrecision will not cause extra zeros to be
*  added. Use DataSourceField.decimalPad for this.
*  
*  A number is always shown with its original precision when edited.
* @type {number}
* @default null
*/
decimalPrecision?: number;

/**
* Applies only to fields of type "float" and enforces a minimum number of digits shown after
*  the decimal point.
*  
*  For example, a field value of 343.1, 343.104 and 343.09872677 would all be shown as 343.10
*  if decimalPad is 2.
*  
*  The original unpadded value is always shown when the value is edited.
* @type {number}
* @default null
*/
decimalPad?: number;

/**
* FormatString for numeric or date formatting. See DataSourceField.format.
* @type {FormatString}
* @default null
*/
format?: FormatString;

/**
* FormatString used during exports for numeric or date formatting. See
*  DataSourceField.exportFormat.
* @type {FormatString}
* @default null
*/
exportFormat?: FormatString;

/**
* A mapping of logical form item values to SCImgURLs or the special value "blank",
*  which means that no image will be displayed.
*  If specified, when the form item is set to the value in question, an icon will be 
*  displayed with the appropriate source URL.
* @type {object}
* @default null
*/
valueIcons?: object;

/**
* This property allows the developer to specify an icon to display when this item has
*  no value. It is configured in the same way as any other valueIcon 
*  (see FormItem.valueIcons)
* @type {string}
* @default null
*/
emptyValueIcon?: string;

/**
* If FormItem.valueIcons is set, this property may be set to show the valueIcon
*  only and prevent the standard form item element or text from displaying
* @type {boolean}
* @default null
*/
showValueIconOnly?: boolean;

/**
* If FormItem.valueIcons is set, this property may be set to prevent the value
*  icons from showing up next to the form items value
* @type {boolean}
* @default null
*/
suppressValueIcon?: boolean;

/**
* If FormItem.valueIcons is specified, use this property to specify a width for
*  the value icon written out.
* @type {number}
* @default null
*/
valueIconWidth?: number;

/**
* If FormItem.valueIcons is specified, use this property to specify a height for the
*  value icon written out.
* @type {number}
* @default null
*/
valueIconHeight?: number;

/**
* If FormItem.valueIcons is specified, this property may be used to specify both
*  the width and height of the icon written out.
*  Note that FormItem.valueIconWidth and FormItem.valueIconHeight take
*  precedence over this value, if specified.
* @type {number}
* @default "16"
*/
valueIconSize?: number;

/**
* If we're showing a value icon, this attribute governs the amount of space between the 
*  icon and the start edge of the form item cell.
*  
*  NOTE: In RTL mode, the valueIconLeftPadding is applied to the right of
*  the value icon.
* @type {number}
* @default "0"
*/
valueIconLeftPadding?: number;

/**
* If we're showing a value icon, this attribute governs the amount of space between the 
*  icon and the value text.
*  
*  NOTE: In RTL mode, the valueIconRightPadding is applied to the left of
*  the value icon.
* @type {number}
* @default "3"
*/
valueIconRightPadding?: number;

/**
* Prefix to apply to the beginning of any FormItem.valueIcons when determining the
*  URL for the image.
*  Will not be applied if the valueIcon URL is absolute.
* @type {string}
* @default null
*/
imageURLPrefix?: string;

/**
* Suffix to apply to the end of any FormItem.valueIcons when determining the URL for
*  the image. A common usage would be to specify a suffix of ".gif" in which
*  case the valueIcons property would map values to the names of images without
*  the ".gif" extension.
* @type {string}
* @default null
*/
imageURLSuffix?: string;

/**
* A Read-Only pointer to this formItem's DynamicForm widget.
* @type {DynamicForm}
* @default null
*/
form?: DynamicForm;

/**
* A Read-Only pointer to the SmartClient canvas that holds this form item. In most cases this
*  will be the FormItem.form containing the item but in some cases
*  editable components handle writing out form items directly. An example of this
*  is editing - when a listGrid shows per-field editors, the
*  containerWidget for each item will be the listGrid body.
*  
*  Note that even if the containerWidget is not a DynamicForm, a DynamicForm
*  will still exist for the item (available as FormItem.form), allowing access
*  to standard APIs such as DynamicForm.getValues
* @type {Canvas}
* @default null
*/
containerWidget?: Canvas;

/**
* Should we show a special 'picker' FormItemIcon for this form item? Picker
*  icons are customizable via FormItem.pickerIconProperties. By default
*  they will be rendered inside the form item's FormItem.controlStyle
*  area. By default clicking the pickerIcon will call FormItem.showPicker.
* @type {boolean}
* @default null
*/
showPickerIcon?: boolean;

/**
* If FormItem.showPickerIcon is true for this item, should the picker icon show
*  a focused image when the form item has focus?
* @type {boolean}
* @default false
*/
showFocusedPickerIcon?: boolean;

/**
* Block of default properties to apply to the pickerIcon for this widget.
*  Intended for class-level customization: To modify this value we recommend using
*  Class.changeDefaults rather than directly assigning a value to the property.
* @type {Partial<FormItemIcon>}
* @default "..."
*/
pickerIconDefaults?: Partial<FormItemIcon>;

/**
* If FormItem.showPickerIcon is true for this item, this block of properties will
*  be applied to the pickerIcon. Allows for advanced customization of this icon.
* @type {Partial<FormItemIcon>}
* @default null
*/
pickerIconProperties?: Partial<FormItemIcon>;

/**
* If FormItem.showPickerIcon is true, this attribute specifies the
*  FormItemIcon.name applied to the picker icon
* @type {Identifier}
* @default "picker"
*/
pickerIconName?: Identifier;

/**
* If FormItem.showPickerIcon is true for this item, this property governs the
*  FormItemIcon.src of the picker icon image to be displayed.
*  
*  skinning can be used for this image, by setting this property to
*  a SCSpriteConfig formatted string.
* @type {SCImgURL}
* @default ""
*/
pickerIconSrc?: SCImgURL;

/**
* If FormItem.showPickerIcon is true for this item, this property governs the
*  size of the picker icon. If unset, the picker icon will be sized as a square to fit in the
*  available height for the icon.
*  
*  Note that if spriting is being used, and the image to be displayed is specified 
*  using css properties such as background-image, background-size,
*  changing this value may result in an unexpected appearance as the image will not
*  scale.
*  Scaleable spriting can be achieved using the SCSpriteConfig format.
*  See the section on spriting in the skinning for 
*  further information.
* @type {Integer}
* @default null
*/
pickerIconWidth?: Integer;

/**
* If FormItem.showPickerIcon is true for this item, this property governs the
*  size of the picker icon. If unset, the picker icon will be sized as a square to fit in the
*  available height for the icon.
*  
*  Note that if spriting is being used, and the image to be displayed is specified 
*  using css properties such as background-image, background-size,
*  changing this value may result in an unexpected appearance as the image will not
*  scale.
*  Scaleable spriting can be achieved using the SCSpriteConfig format.
*  See the section on spriting in the skinning for 
*  further information.
* @type {Integer}
* @default null
*/
pickerIconHeight?: Integer;

/**
* Prompt to show when the user hovers the mouse over the picker icon.
* @type {HTMLString}
* @default null
*/
pickerIconPrompt?: HTMLString;

/**
* The component that will be displayed when FormItem.showPicker is called due to a click
*  on the FormItem.showPickerIcon.
*  
*  Can be specified directly as a Canvas, or created automatically via the
*  AutoChild pattern. The default autoChild configuration for the picker is 
*  a Canvas with backgroundColor set and no other modifications.
*  
*  Note that the picker is not automatically destroyed with the FormItem that uses it, in
*  order to allow recycling of picker components. To destroy a single-use picker, override
*  Canvas.destroy.
* @type {Canvas}
* @default null
*/
picker?: Canvas;

/**
* Class name of the picker to be created.
* @type {SCClassName}
* @default null
*/
pickerConstructor?: SCClassName;

/**
* Default properties for the picker.
* @type {Partial<Canvas>}
* @default "{}"
*/
pickerProperties?: Partial<Canvas>;

/**
* Validators for this form item. 
*  
*  Note: these validators will only be run on the client; to
*  do real client-server validation, validators must be specified via
*  DataSourceField.validators.
* @type {Array<Partial<Validator>>}
* @default null
*/
validators?: Array<Partial<Validator>>;

/**
* Whether a non-empty value is required for this field to pass validation.
*  
*  If the user does not fill in the required field, the error message to be shown will
*  be taken from these properties in the following order: FormItem.requiredMessage,
*  DynamicForm.requiredMessage, DataSource.requiredMessage, 
*  Validator.requiredField.
*  
*  Note: if specified on a FormItem, required is only enforced on the
*  client. required should generally be specified on a
*  DataSourceField.
* @type {boolean}
* @default null
*/
required?: boolean;

/**
* The required message for required field errors.
* @type {HTMLString}
* @default null
*/
requiredMessage?: HTMLString;

/**
* Criteria to be evaluated to determine whether this FormItem should be FormItem.required.
*  
*  Criteria are evaluated against the ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','form\'s current values')} as well as 
*  the current Canvas.ruleScope. Criteria are re-evaluated every time
*  form values or the rule context changes, whether by end user action or by programmatic calls.
*  
*  A basic criteria uses textMatchStyle:"exact". When specified in
*  componentXML this property allows
*  xmlCriteriaShorthand for defining criteria.
*  
*  Note: A FormItem using requiredWhen must have a FormItem.name defined.
* @type {Criteria}
* @default null
*/
requiredWhen?: Criteria;

/**
* Whether this item is currently visible.
*  
*  visible can only be set on creation. After creation, use
*  FormItem.show and FormItem.hide to manipulate visibility.
* @type {boolean}
* @default true
*/
visible?: boolean;

/**
* Criteria to be evaluated to determine whether this FormItem should be visible.
*  
*  Criteria are evaluated against the ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','form\'s current values')} as well as 
*  the current Canvas.ruleScope. Criteria are re-evaluated every time
*  form values or the rule context changes, whether by end user action or by programmatic calls.
*  
*  If both FormItem.showIf and visibleWhen are specified, visibleWhen is
*  ignored.
*  
*  A basic criteria uses textMatchStyle:"exact". When specified in
*  componentXML this property allows
*  xmlCriteriaShorthand for defining criteria.
*  
*  Note: A FormItem using visibleWhen must have a FormItem.name defined. FormItem.shouldSaveValue can
*  be set to false to prevent the field from storing its value
*  into the form's values.
* @type {AdvancedCriteria}
* @default null
*/
visibleWhen?: AdvancedCriteria;

/**
* Is this form item editable (canEdit:true) or read-only (canEdit:false)? Setting the
*  form item to non-editable causes it to render as read-only. Can be updated at runtime via
*  the FormItem.setCanEdit method.
*  
*  Read-only appearance may be specified via FormItem.readOnlyDisplay.
*  The default setting for this value ("readOnly") differs from
*  the disabled state in that the form item is not rendered with disabled styling and
*  most form items will allow copying of the contents while read-only but do not while
*  disabled.
*  
*  Note that for forms bound to a DataSource, if this property is not explicitly
*  set at the item level, its default value will match the
*  DynamicForm.canEditFieldAttribute on the associated dataSource field.
*  
*  Developers should also be aware that the FormItem.readOnlyDisplay attribute is
*  unrelated to the DataSourceField.readOnlyEditorType attribute. When a
*  DynamicForm is first bound to a dataSource, for
*  DataSourceField.canEdit DataSourceFields,
*  DataSourceField.readOnlyEditorType will determine what FormItemType
*  should be created for the field. Once created, a FormItem's type can not be changed.
*  Setting FormItem.canEdit at runtime will simply change the appearance
*  of the item to allow or disallow editing of the item.
* 
*  
* @type {boolean}
* @default null
*/
canEdit?: boolean;

/**
* If this item is FormItem.getCanEdit, how should this item be displayed
*  to the user? If set, overrides the form-level DynamicForm.readOnlyDisplay default.
* @type {ReadOnlyDisplayAppearance}
* @default null
*/
readOnlyDisplay?: ReadOnlyDisplayAppearance;

/**
* Base text box style to apply when this item is FormItem.getCanEdit and
*  is using FormItem.readOnlyDisplay
*  "static".
*  
*  If set, overrides the form-level DynamicForm.readOnlyTextBoxStyle default.
* @type {FormItemBaseStyle}
* @default null
*/
readOnlyTextBoxStyle?: FormItemBaseStyle;

/**
* Modified FormItem.controlStyle to apply when this item is FormItem.getCanEdit and
*  is using FormItem.readOnlyDisplay
*  "static".
*  
* @type {FormItemBaseStyle}
* @default null
*/
readOnlyControlStyle?: FormItemBaseStyle;

/**
* If this item is FormItem.getCanEdit and is using
*  FormItem.readOnlyDisplay
*  "static",
*  
*  should the item value be clipped if it overflows the specified size of the item?
*  If set, overrides the form-level DynamicForm.clipStaticValue default.
* @type {boolean}
* @default null
*/
clipStaticValue?: boolean;

/**
* Criteria to be evaluated to determine whether this FormItem should be made
*  FormItem.setCanEdit. Appearance when read-only is determined by
*  FormItem.readOnlyDisplay.
*  
*  Criteria are evaluated against the ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','form\'s current values')} as well as 
*  the current Canvas.ruleScope. Criteria are re-evaluated every time
*  form values or the rule context changes, whether by end user action or by programmatic calls.
*  
*  A basic criteria uses textMatchStyle:"exact". When specified in
*  componentXML this property allows
*  xmlCriteriaShorthand for defining criteria.
*  
*  Note: A FormItem using readOnlyWhen must have a FormItem.name defined. FormItem.shouldSaveValue can
*  be set to false to prevent the field from storing its value
*  into the form's values.
* @type {AdvancedCriteria}
* @default null
*/
readOnlyWhen?: AdvancedCriteria;

/**
* Whether this item is disabled. Can be updated at runtime via the setDisabled()
*  method. Note that if the widget containing this formItem is disabled, the formItem will
*  behave in a disabled manner regardless of the setting of the item.disabled property.
*  
*  Note that not all items can be disabled, and not all browsers show an obvious disabled style
*  for native form elements.
* @type {boolean}
* @default false
*/
disabled?: boolean;

/**
* If FormItem.canEdit is set to false, should FormItem.icons be disabled
*  by default?
*  
*  This may also be specified at the icon level. See FormItemIcon.disableOnReadOnly.
* @type {boolean}
* @default true
*/
disableIconsOnReadOnly?: boolean;

/**
* Is this form item focusable? Setting this property to true on an otherwise
*  non-focusable element such as a StaticTextItem will cause the item to be
*  included in the page's tab order and respond to keyboard events.
* @type {boolean}
* @default null
*/
canFocus?: boolean;

/**
* If specified this governs the HTML accessKey for the item.
*  
*  This should be set to a character - when a user hits the html accessKey modifier for
*  the browser, plus this character, focus will be given to the item.
*  The accessKey modifier can vary by browser and platform. 
*  
*  The following list of default behavior is for reference only, developers should also
*  consult browser documentation for additional information.
*  
*  
* - Internet Explorer (all platforms): Alt + accessKey
*  
* - Mozilla Firefox (Windows, Unix): Alt+Shift + accessKey
*  
* - Mozilla Firefox (Mac): Ctrl+Opt + accessKey
*  
* - Chrome and Safari (Windows, Unix): Alt + accessKey
*  
* - Chrome and Safari (Mac): Ctrl+Opt + accessKey
*  
* @type {string}
* @default null
*/
accessKey?: string;

/**
* TabIndex for the form item within the form, which controls the order in which controls
*  are visited when the user hits the tab or shift-tab keys to navigate between items. 
*  
*  tabIndex is automatically assigned as the order that items appear in the
*  DynamicForm.items list.
*  
*  To specify the tabindex of an item within the page as a whole (not just this form), use
*  FormItem.globalTabIndex instead.
* @type {Integer}
* @default null
*/
tabIndex?: Integer;

/**
* TabIndex for the form item within the page. Takes precedence over any local tab index
*  specified as FormItem.tabIndex.
*  
*  Use of this API is extremely advanced and essentially implies taking over
*  management of tab index assignment for all components on the page.
* @type {Integer}
* @default null
*/
globalTabIndex?: Integer;

/**
* Allows the DynamicForm.selectOnFocus behavior to be configured on a
*  per-FormItem basis. Normally all items in a form default to the value of
*  DynamicForm.selectOnFocus.
* @type {boolean}
* @default null
*/
selectOnFocus?: boolean;

/**
* Allows the DynamicForm.selectOnClick behavior to be configured on a
*  per-FormItem basis. Normally all items in a form default to the value of
*  DynamicForm.selectOnClick.
* @type {boolean}
* @default null
*/
selectOnClick?: boolean;

/**
* Should this form item fire its FormItem.change handler (and store its
*  value in the form) on every keypress? Set to false to suppress the 'change'
*  handler firing (and the value stored) on every keypress.
*  
*  Note: If false, the value returned by FormItem.getValue
*  will not reflect the value displayed in the form item element as long as focus is in
*  the form item element.
* @type {boolean}
* @default true
*/
changeOnKeypress?: boolean;

/**
* Should we show a title cell for this formItem?
*  
*  Note: the default value of this attribute is overridden by some subclasses.
* @type {boolean}
* @default true
*/
showTitle?: boolean;

/**
* On which side of this item should the title be placed. TitleOrientation
*  lists valid options.
*  
*  Note that titles on the left or right take up a cell in tabular
*  formLayout, but titles on top do not.
* @type {TitleOrientation}
* @default "left"
*/
titleOrientation?: TitleOrientation;

/**
* Alignment of this item's title in its cell.
*  
*  If null, dynamically set according to text direction.
* @type {Alignment}
* @default null
*/
titleAlign?: Alignment;

/**
* Vertical alignment of this item's title in its cell. Only applies when
*  FormItem.titleOrientation is "left" or "right".
* @type {VerticalAlignment}
* @default "center"
*/
titleVAlign?: VerticalAlignment;

/**
* If the title for this form item is showing, and is too large for the available space
*  should the title be clipped?
*  
*  Null by default - if set to true or false, overrides DynamicForm.clipItemTitles.
* @type {boolean}
* @default null
*/
clipTitle?: boolean;

/**
* If specified determines whether this items title should wrap.
*  Overrides DynamicForm.wrapItemTitles at the DynamicForm level.
* @type {boolean}
* @default null
*/
wrapTitle?: boolean;

/**
* If true, this item will cause the entire form to be redrawn
*  when the item's "elementChanged" event is done firing
* @type {boolean}
* @default false
*/
redrawOnChange?: boolean;

/**
* If true, form items will be validated when each item's "change" handler is fired
*  as well as when the entire form is submitted or validated.
*  
*  Note that this property can also be set at the form level or on each validator;
*  If true at the form or field level, validators not explicitly set with
*  validateOnChange:false will be fired on change - displaying errors and
*  rejecting the change on validation failure.
* @type {boolean}
* @default false
*/
validateOnChange?: boolean;

/**
* If true, form items will be validated when each item's "editorExit" handler is fired
*  as well as when the entire form is submitted or validated.
*  
*  Note that this property can also be set at the form level.
*  If true at either level the validator will be fired on editorExit.
* @type {boolean}
* @default false
*/
validateOnExit?: boolean;

/**
* Indicates that if validation fails, the user should not be allowed to exit
*  the field - focus will be forced back into the field until the error is corrected.
*  
*  This property defaults to DynamicForm.stopOnError if unset.
*  
*  Enabling this property also implies FormItem.validateOnExit is automatically
*  enabled. If there are server-based validators on this item, setting this property
*  also implies that FormItem.synchronousValidation is forced on.
* @type {boolean}
* @default null
*/
stopOnError?: boolean;

/**
* If validateOnChange is true, and validation fails for this item on change, with no suggested
*  value, should we revert to the previous value, or continue to display the bad value entered
*  by the user. May be set at the item or form level.
* @type {boolean}
* @default false
*/
rejectInvalidValueOnChange?: boolean;

/**
* If enabled, whenever validation is triggered and a request to the server is required,
*  user interactivity will be blocked until the request returns. Can be set for the entire
*  form or individual FormItems.
*  
*  If false, the form will try to avoid blocking user interaction until it is strictly
*  required. That is until the user attempts to use a FormItem whose state could be
*  affected by a server request that has not yet returned.
* @type {boolean}
* @default null
*/
synchronousValidation?: boolean;

/**
* Width of the FormItem. Can be either a number indicating a fixed width in pixels, or
*  "*" indicating the FormItem fills the space allocated to it's column (or columns, for a
*  FormItem.colSpan item). You may also use "100%" as a synonym for "*", but
*  other percentages are not supported.
*  
*  Note that for "absolute" item layout rather than the default "table" layout, the rules
*  for specifying the width are slightly different. All percent sizes are allowed, but not
*  "*". See DynamicForm.itemLayout for further details.
*  
*  
*  See the formLayout overview for details.
* @type {Integer | string}
* @default "*"
*/
width?: Integer | string;

/**
* Specifies a width for an item in DynamicForm.linearMode, overriding
*  the default width of "*" in that mode.
* @type {Integer | string}
* @default null
*/
linearWidth?: Integer | string;

/**
* Height of the FormItem. Can be either a number indicating a fixed height in pixels, a
*  percentage indicating a percentage of the overall form's height, or "*" indicating take
*  whatever remaining space is available. See the formLayout overview for details.
*  
*  
*  For form items having a FormItem.showPickerIcon (e.g. SelectItem,
*  ComboBoxItem) and SpinnerItems, if there is no explicit 
*  FormItem.pickerIconHeight, the pickerIcon will be sized to match the available
*  space based on the specified item height.
*  Note that if spriting is being used, and the image to be displayed in these icons
*  is specified 
*  using css properties such as background-image, background-size,
*  changing this value may result in an unexpected appearance as the image will not
*  scale.
*  Scaleable spriting can be achieved using the SCSpriteConfig format.
*  See the section on spriting in the skinning for 
*  further information.
*  Alternatively, the FormItem.pickerIconStyle could be changed to a
*  custom CSS style name, and in the case of SpinnerItems, 
*  the FormItemIcon.baseStyle and
*  FormItemIcon.src of the SpinnerItem.increaseIcon
*  and SpinnerItem.decreaseIcon AutoChildren could be customized.
*  
*  Note that when FormItem is rendered as read-only with readOnlyDisplay as "static"
*  the property FormItem.staticHeight is used instead.
* @type {Integer | string}
* @default "20"
*/
height?: Integer | string;

/**
* Height of the FormItem when canEdit is false and
*  readOnlyDisplay is "static". The normal FormItem.height is used
*  if this property is not set.
* @type {Integer}
* @default null
*/
staticHeight?: Integer;

/**
* If specified, this property will govern the height of the cell in which this form
*  item is rendered.
*  Will not apply when the containing DynamicForm sets itemLayout:"absolute".
* @type {number}
* @default null
*/
cellHeight?: number;

/**
* Number of columns that this item's title spans. 
*  
*  This setting only applies for items that are showing a title and whose
*  TitleOrientation is either "left" or "right".
* @type {number}
* @default "1"
*/
titleColSpan?: number;

/**
* Number of columns that this item spans. 
*  
*  The colSpan setting does not include the title shown for items with
*  FormItem.showTitle:true, so the effective colSpan is one higher than this
*  setting for items that are showing a title and whose TitleOrientation is either
*  "left" or "right".
* @type {Integer | string}
* @default "1"
*/
colSpan?: Integer | string;

/**
* Specifies a column span for an item in DynamicForm.linearMode,
*  overriding the default value of "*" in that mode.
* @type {Integer | string}
* @default null
*/
linearColSpan?: Integer | string;

/**
* Number of rows that this item spans
* @type {number}
* @default "1"
*/
rowSpan?: number;

/**
* Whether this item should always start a new row in the form layout.
* @type {boolean}
* @default false
*/
startRow?: boolean;

/**
* Whether this item should end the row it's in in the form layout
* @type {boolean}
* @default false
*/
endRow?: boolean;

/**
* Specifies FormItem.startRow for an item in DynamicForm.linearMode,
*  overriding the default of false in that mode.
* @type {Integer | string}
* @default null
*/
linearStartRow?: Integer | string;

/**
* Specifies FormItem.endRow for an item in DynamicForm.linearMode,
*  overriding the default of true in that mode.
* @type {Integer | string}
* @default null
*/
linearEndRow?: Integer | string;

/**
* Alignment of this item in its cell. Note that the alignment of text / content within this
*  item is controlled separately via FormItem.textAlign (typically textAlign
*  applies to items showing a "textBox", such as a TextItem or SelectItem,
*  as well as text-only form item types such as StaticTextItem and HeaderItem).
*  If FormItem.applyAlignToText is true, then the textAlign
*  setting, if unset, will default to the align setting if set.
* @type {Alignment}
* @default null
*/
align?: Alignment;

/**
* Vertical alignment of this item within its cell. This property governs the position
*  of the item's text box within the cell (not the content within the text box).
*  If FormItem.shouldApplyHeightToTextBox is true, for this to have a visible effect,
*  the cell height must exceed the specified height of the item, either due to
*  an explicit FormItem.cellHeight specification, or due to the row being expanded
*  by another taller item.
*  
*  Has no effect if DynamicForm.itemLayout is set to "absolute" for the
*  form.
* @type {VerticalAlignment}
* @default "center"
*/
vAlign?: VerticalAlignment;

/**
* Alignment of the text / content within this form item. Note that FormItem.align may
*  be used to control alignment of the entire form item within its cell. textAlign
*  does not apply to all form item types; typically it applies only to items showing a "textBox",
*  such as a TextItem or SelectItem, as well as text-only form item types
*  such as StaticTextItem and HeaderItem.
*  
*  If FormItem.applyAlignToText is true, then textAlign
*  will default to the align setting if set. Otherwise, if this item has
*  FormItem.icons, then textAlign will default to
*  "left"
*  
*  ("right"
*  
*  in Page.isRTL).
* @type {Alignment}
* @default null
*/
textAlign?: Alignment;

/**
* If the FormItem.textAlign is unset, should the FormItem.align
*  setting, if set, be used for this item's textAlign?
*  
*  applyAlignToText defaults to false for most form item types. It defaults
*  to true for StaticTextItem and HeaderItem, which are text-based form item
*  types that do not have a natural distinction between the item and its cell.
* @type {boolean}
* @default false
*/
applyAlignToText?: boolean;

/**
* Left coordinate of this item in pixels. Applies only when the containing DynamicForm
*  sets itemLayout:"absolute".
* @type {Integer}
* @default "0"
*/
left?: Integer;

/**
* Top coordinate of this item in pixels. Applies only when the containing DynamicForm
*  sets itemLayout:"absolute".
* @type {Integer}
* @default "0"
*/
top?: Integer;

/**
* If this item is FormItem.getCanEdit and is using
*  FormItem.readOnlyDisplay
*  "static",
*  
*  should the item value wrap?
* @type {boolean}
* @default null
*/
wrapStaticValue?: boolean;

/**
* Set of autocompletion keywords to be used with the native "autocomplete" attribute, 
*  in accordance with the 
*  HTML5 Autofill specification.
*  
*  When autoCompleteKeywords are provided, the FormItem.autoComplete setting is ignored.
* @type {Array<Partial<string>>}
* @default null
*/
autoCompleteKeywords?: Array<Partial<string>>;

/**
* Should this item allow browser auto-completion of its value? 
*  Applies only to items based on native HTML form elements (TextItem,
*  PasswordItem, etc), and will only have a user-visible impact for browsers
*  where native autoComplete behavior is actually supported and enabled via user settings.
*  
*  Alternatively, FormItem.autoCompleteKeywords can be specified, in which case 
*  this setting is ignored. If autoCompleteKeywords are not provided, and 
*  autoComplete is not set on this FormItem, the value of 
*  DynamicForm.autoComplete is used.
*  
*  Note that even with this value set to "none", native browser 
*  auto-completion may occur for log in forms (forms containing username and 
*  PasswordItem fields). This behavior varies by browser, and is
*  a result of an 
*  intentional change by some browser developers
*  to disregard the HTML setting autocomplete=off for password items or
*  log-in forms.
*  
*  In some browsers any form redraw (including a redraw from a call to 
*  DynamicForm.setValues) will re-populate the form with the natively 
*  remembered login credentials. This can make it very difficult to control the
*  values displayed to the user, as a call to 'setValues()' may appear to be ignored.
*  While behavior varies by browser we have specifically
*  observed this behavior in Safari. Moreover in this browser, if the user 
*  asks the browser to remember login credentials for a URL, any form with a password 
*  item and a text item may be auto-filled with the remembered login credentials, 
*  even if the form's configuration and field names differ from those on the
*  login form.
*  
*  If an application has both an initial log in form, and a separate form within
*  the application which makes contains a Password item (a use case might be an
*  interface for a user with manager privileges for modifying other users' passwords),
*  this will cause the second form to autofill with unexpected values.
*  
*  Should this arise, developers can avoid this by making the initial log in 
*  interface into a separate log in page from the main application page.
*  This is often good practice in any case for reasons outlined in the 
*  "Authentication" section of the Quick Start guide.
* @type {AutoComplete}
* @default null
*/
autoComplete?: AutoComplete;

/**
* If this browser supports spell-checking of text editing elements, do we want this
*  enabled for this item? If unset the property will be inherited from the containing form.
*  
*  Notes:
*  - this property only applies to text based items such as TextItem and TextAreaItem.
*  - this property is not supported on all browsers.
* @type {boolean}
* @default null
*/
browserSpellCheck?: boolean;

/**
* Form item input type - governs which keyboard should be displayed for
*  mobile devices (supported on iPhone / iPad)
* @type {string}
* @default null
*/
browserInputType?: string;

/**
* An array of descriptor objects for icons to display in a line after this form item.
*  These icons are clickable images, often used to display some kind of helper for 
*  populating a form item.
* @type {Array<Partial<Partial<FormItemIcon>>>}
* @default null
*/
icons?: Array<Partial<Partial<FormItemIcon>>>;

/**
* Default icon image source.   
*  Specify as the partial URL to an image, relative to the imgDir of this component.
*  To specify image source for a specific icon use the icon.src property.
*  If this item is drawn in the disabled state, the url will be modified by adding 
*  "_Disabled" to get a disabled state image for the icon.
*  If icon.showOver is true, this url will be modified by adding "_Over" to get
*  an over state image for the icon.
*  
*  skinning can be used for this image, by setting this property to
*  a SCSpriteConfig formatted string.
* @type {SCImgURL}
* @default "[SKIN]/DynamicForm/default_formItem_icon.gif"
*/
defaultIconSrc?: SCImgURL;

/**
* If we're showing icons, should we change their image source to the appropriate over
*  source when the user rolls over (or puts focus onto) them? Can be overridden on a per
*  icon basis by the formItemIcon showOver property.
* @type {boolean}
* @default null
*/
showOverIcons?: boolean;

/**
* If we're showing icons, should we change their image source to the appropriate
*  focused source when this item has focus? Can be overridden on a per
*  icon basis by the formItemIcon showFocused property.
* @type {boolean}
* @default null
*/
showFocusedIcons?: boolean;

/**
* Horizontal space (in px) to leave between form item icons. The space appears either on
*  the left or right of each icon. May be overridden at the icon level via FormItemIcon.hspace.
*  Must be non-negative.
* @type {Integer}
* @default "3"
*/
iconHSpace?: Integer;

/**
* How should icons be aligned vertically for this form item.
* @type {VerticalAlignment}
* @default "bottom"
*/
iconVAlign?: VerticalAlignment;

/**
* Default width for form item icons. May be overridden at the icon level by FormItemIcon.width.
* @type {Integer}
* @default "20"
*/
iconWidth?: Integer;

/**
* Default height for form item icons. May be overridden at the icon level by FormItemIcon.height.
* @type {Integer}
* @default "20"
*/
iconHeight?: Integer;

/**
* This text is shown as a tooltip prompt when the cursor hovers over this item.
*  
*  When item is FormItem.setCanEdit a different hover can be shown with
*  FormItem.readOnlyHover. Or, when item is FormItem.disabled or
*  read-only with FormItem.readOnlyDisplay a different hover can be shown
*  with FormItem.disabledHover.
*  
*  Note that when the form is Canvas.disabled this prompt will not
*  be shown.
* @type {HTMLString}
* @default null
*/
prompt?: HTMLString;

/**
* This text is shown as a tooltip prompt when the cursor hovers over this item
*  and the item is FormItem.setCanEdit.
*  
*  Note that when the form is Canvas.disabled this prompt will not
*  be shown.
* @type {HTMLString}
* @default null
*/
readOnlyHover?: HTMLString;

/**
* This text is shown as a tooltip prompt when the cursor hovers over this item
*  and the item is FormItem.disabled or
*  FormItem.setCanEdit with FormItem.readOnlyDisplay.
*  
*  You can also override FormItem.itemHoverHTML on the item to show
*  a custom hover, whether or not the item is disabled. 
*  
*  Note that when the form itself is Canvas.disabled, no prompt will
*  be shown.
* @type {HTMLString}
* @default null
*/
disabledHover?: HTMLString;

/**
* Default prompt (and tooltip-text) for icons.
* @type {HTMLString}
* @default ""
*/
iconPrompt?: HTMLString;

/**
* Set to false to suppress writing out any FormItem.icons for this item.
* @type {boolean}
* @default true
*/
showIcons?: boolean;

/**
* Show the FormItem.icons when the item gets focus, and hide them when it loses focus.
*  Can be overridden at the icon level by FormItemIcon.showOnFocus.
*  
*  Note that icons marked as disabled will not be shown on focus even if this flag is
*  true by default. This may be overridden by FormItem.showDisabledIconsOnFocus.
* @type {boolean}
* @default null
*/
showIconsOnFocus?: boolean;

/**
* If FormItem.showIconsOnFocus is true, should icons marked as disabled be
*  shown on focus?
*  
*  Default setting is false - it is not commonly desirable to 
*  present a user with a disabled icon on focus.
*  
*  Can be overridden at the icon level by FormItemIcon.showDisabledOnFocus
* @type {boolean}
* @default false
*/
showDisabledIconsOnFocus?: boolean;

/**
* Show the picker icon when the item gets focus, and hide it when it loses focus. Can be
*  overridden at the icon level by FormItemIcon.showOnFocus.
*  
*  Note that a pickerIcon marked as disabled will not be shown on focus even if this flag is
*  true by default. This may be overridden by FormItem.showDisabledIconsOnFocus.
* @type {boolean}
* @default null
*/
showPickerIconOnFocus?: boolean;

/**
* If FormItem.showPickerIconOnFocus is true, should the picker icon be
*  shown on focus if it is disabled (as in a read-only item, for example?)
*  
*  Default setting is false - it is not commonly desirable to 
*  present a user with a disabled icon on focus.
*  
*  Can be overridden at the icon level by FormItemIcon.showDisabledOnFocus
* @type {boolean}
* @default false
*/
showDisabledPickerIconOnFocus?: boolean;

/**
* Should this item's FormItem.icons and 
*  FormItem.showPickerIcon be included in 
*  the page's tab order by default? If not explicitly set, this property 
*  will be derived from DynamicForm.canTabToIcons.
*  
*  Developers may also suppress tabbing to individual icons by 
*  setting FormItemIcon.tabIndex to -1.
*  
*  Note that if this form item has tabIndex -1, neither the form item nor the icons
*  will be included in the page's tab order.
* @type {boolean}
* @default null
*/
canTabToIcons?: boolean;

/**
* Height of the error icon, if we're showing icons when validation errors occur.
* @type {number}
* @default "16"
*/
errorIconHeight?: number;

/**
* Height of the error icon, if we're showing icons when validation errors occur.
* @type {number}
* @default "16"
*/
errorIconWidth?: number;

/**
* URL of the image to show as an error icon, if we're showing icons when validation
*  errors occur.
* @type {SCImgURL}
* @default "[SKIN]/DynamicForm/validation_error_icon.png"
*/
errorIconSrc?: SCImgURL;

/**
* DynamicForm.showErrorIcons, 
*  DynamicForm.showErrorText, DynamicForm.errorOrientation,
*  and DynamicForm.showErrorStyle control how validation errors are
*  displayed when they are displayed inline in the form (next to the form item where there
*  is a validation error). To instead display all errors at the top of the form, set
*  DynamicForm.showInlineErrors:false.
*  
*  showErrorIcons, showErrorText, errorOrientation
*  and showErrorStyle
*  are all boolean properties, and can be set on a DynamicForm to control the behavior
*  form-wide, or set on individual FormItems. 
*  
*  The HTML displayed next to a form item with errors is generated by 
*  FormItem.getErrorHTML.
*  The default implementation of that method respects showErrorIcons and
*  showErrorText as follows:
*  
*  showErrorIcons, or showErrorIcon at the FormItem level controls
*  whether an error icon should appear next to fields which have validation errors. The icon's
*  appearance is governed by FormItem.errorIconSrc, FormItem.errorIconWidth and
*  FormItem.errorIconHeight
*  
*  showErrorText determines whether the text of the validation error should be
*  displayed next to fields which have validation errors. The attribute
*  DynamicForm.showTitlesWithErrorMessages may be set to prefix error messages with the 
*  form item's title + ":" (may be desired if the item has 
*  FormItem.showTitle set to false).
*  If showErrorText is unset, the error text will be shown if
*  DynamicForm.linearMode is true (or DynamicForm.linearOnMobile is true for
*  mobile devices), otherwise it will not be shown.
*  
*  In addition to this:
*  
*  DynamicForm.errorOrientation controls where the error HTML should appear relative 
*  to form items. Therefore the combination of FormItem.showErrorText:false and
*  FormItem.errorOrientation:"left" creates a compact validation error display
*  consisting of just an icon, to the left of the item with the error message
*  available via a hover (similar appearance to ListGrid validation error display).
*  If errorOrientation is unset, the error orientation will default to "top" 
*  if DynamicForm.linearMode is enabled (or DynamicForm.linearOnMobile is true
*  for mobile devices) and error text is not showing, "left" otherwise.
*  
*  showErrorStyle determines whether fields with validation
*  errors should have special styling applied to them. Error styling is achieved by
*  applying suffixes to existing styling applied to various parts of the form item.
*  See FormItemBaseStyle for more on this.
* @type {boolean}
* @default null
*/
showErrorIcon?: boolean;

/**
* DynamicForm.showErrorIcons, 
*  DynamicForm.showErrorText, DynamicForm.errorOrientation,
*  and DynamicForm.showErrorStyle control how validation errors are
*  displayed when they are displayed inline in the form (next to the form item where there
*  is a validation error). To instead display all errors at the top of the form, set
*  DynamicForm.showInlineErrors:false.
*  
*  showErrorIcons, showErrorText, errorOrientation
*  and showErrorStyle
*  are all boolean properties, and can be set on a DynamicForm to control the behavior
*  form-wide, or set on individual FormItems. 
*  
*  The HTML displayed next to a form item with errors is generated by 
*  FormItem.getErrorHTML.
*  The default implementation of that method respects showErrorIcons and
*  showErrorText as follows:
*  
*  showErrorIcons, or showErrorIcon at the FormItem level controls
*  whether an error icon should appear next to fields which have validation errors. The icon's
*  appearance is governed by FormItem.errorIconSrc, FormItem.errorIconWidth and
*  FormItem.errorIconHeight
*  
*  showErrorText determines whether the text of the validation error should be
*  displayed next to fields which have validation errors. The attribute
*  DynamicForm.showTitlesWithErrorMessages may be set to prefix error messages with the 
*  form item's title + ":" (may be desired if the item has 
*  FormItem.showTitle set to false).
*  If showErrorText is unset, the error text will be shown if
*  DynamicForm.linearMode is true (or DynamicForm.linearOnMobile is true for
*  mobile devices), otherwise it will not be shown.
*  
*  In addition to this:
*  
*  DynamicForm.errorOrientation controls where the error HTML should appear relative 
*  to form items. Therefore the combination of FormItem.showErrorText:false and
*  FormItem.errorOrientation:"left" creates a compact validation error display
*  consisting of just an icon, to the left of the item with the error message
*  available via a hover (similar appearance to ListGrid validation error display).
*  If errorOrientation is unset, the error orientation will default to "top" 
*  if DynamicForm.linearMode is enabled (or DynamicForm.linearOnMobile is true
*  for mobile devices) and error text is not showing, "left" otherwise.
*  
*  showErrorStyle determines whether fields with validation
*  errors should have special styling applied to them. Error styling is achieved by
*  applying suffixes to existing styling applied to various parts of the form item.
*  See FormItemBaseStyle for more on this.
* @type {boolean}
* @default null
*/
showErrorText?: boolean;

/**
* DynamicForm.showErrorIcons, 
*  DynamicForm.showErrorText, DynamicForm.errorOrientation,
*  and DynamicForm.showErrorStyle control how validation errors are
*  displayed when they are displayed inline in the form (next to the form item where there
*  is a validation error). To instead display all errors at the top of the form, set
*  DynamicForm.showInlineErrors:false.
*  
*  showErrorIcons, showErrorText, errorOrientation
*  and showErrorStyle
*  are all boolean properties, and can be set on a DynamicForm to control the behavior
*  form-wide, or set on individual FormItems. 
*  
*  The HTML displayed next to a form item with errors is generated by 
*  FormItem.getErrorHTML.
*  The default implementation of that method respects showErrorIcons and
*  showErrorText as follows:
*  
*  showErrorIcons, or showErrorIcon at the FormItem level controls
*  whether an error icon should appear next to fields which have validation errors. The icon's
*  appearance is governed by FormItem.errorIconSrc, FormItem.errorIconWidth and
*  FormItem.errorIconHeight
*  
*  showErrorText determines whether the text of the validation error should be
*  displayed next to fields which have validation errors. The attribute
*  DynamicForm.showTitlesWithErrorMessages may be set to prefix error messages with the 
*  form item's title + ":" (may be desired if the item has 
*  FormItem.showTitle set to false).
*  If showErrorText is unset, the error text will be shown if
*  DynamicForm.linearMode is true (or DynamicForm.linearOnMobile is true for
*  mobile devices), otherwise it will not be shown.
*  
*  In addition to this:
*  
*  DynamicForm.errorOrientation controls where the error HTML should appear relative 
*  to form items. Therefore the combination of FormItem.showErrorText:false and
*  FormItem.errorOrientation:"left" creates a compact validation error display
*  consisting of just an icon, to the left of the item with the error message
*  available via a hover (similar appearance to ListGrid validation error display).
*  If errorOrientation is unset, the error orientation will default to "top" 
*  if DynamicForm.linearMode is enabled (or DynamicForm.linearOnMobile is true
*  for mobile devices) and error text is not showing, "left" otherwise.
*  
*  showErrorStyle determines whether fields with validation
*  errors should have special styling applied to them. Error styling is achieved by
*  applying suffixes to existing styling applied to various parts of the form item.
*  See FormItemBaseStyle for more on this.
* @type {boolean}
* @default null
*/
showErrorStyle?: boolean;

/**
* If DynamicForm.showInlineErrors is true, where should the error icon and text appear
*  relative to the form item itself. Valid options are "top", 
*  "bottom", "left" or "right".
*  If unset the orientation will be derived from DynamicForm.errorOrientation.
* @type {string}
* @default null
*/
errorOrientation?: string;

/**
* Specifies "hint" string to show next to the form item to indicate something to the user.
*  This string generally appears to the right of the form item.
* @type {HTMLString}
* @default null
*/
hint?: HTMLString;

/**
* If a hint is defined for this form item, should it be shown?
* @type {boolean}
* @default true
*/
showHint?: boolean;

/**
* If this item is showing a FormItem.hint, should the hint text be allowed to
*  wrap? Setting this property to false will render the hint on a single line
*  without wrapping, expanding the width required to render the item if necessary.
*  
*  If unset this property will be picked up from the DynamicForm.wrapHintText
*  setting.
*  
*  This setting does not apply to hints that are TextItem.showHintInField.
* @type {boolean}
* @default null
*/
wrapHintText?: boolean;

/**
* If this item is showing a FormItem.hint, this setting specifies how much
*  horizontal space is made available for rendering the hint text by default.
*  Typically this reflects how much space the hint text takes up before it wraps.
*  
*  Note that the presence of a hint may cause a form item to expand horizontally past its
*  specified FormItem.width.
*  This property value acts as a minimum - if the hint text can not wrap within this width
*  (either due to FormItem.wrapHintText being set to false, or
*  due to it containing long, un-wrappable content), it will further expand to take
*  up the space it needs.
*  
*  If unset this property will be picked up from the DynamicForm.minHintWidth
*  setting.
*  
*  This setting does not apply to hints that are TextItem.showHintInField.
* @type {Integer}
* @default null
*/
minHintWidth?: Integer;

/**
* When this item receives focus, should it be re-styled to indicate it has focus?
*  
*  See formItemStyling for more details on formItem styling.
* @type {boolean}
* @default false
*/
showFocused?: boolean;

/**
* When the user rolls over this item, should it be re-styled to indicate it has focus?
*  
*  When enabled, the "Over" styling is applied to the text box,
*  control table (if present), and pickerIcon (if present), and any icons
*  where FormItemIcon.showOver is true and FormItemIcon.showOverWhen is
*  set to "textBox".
*  These behaviors can be disabled piecemeal via FormItem.updateTextBoxOnOver, 
*  FormItem.updateControlOnOver and FormItem.updatePickerIconOnOver properties.
*  
*  Developers may also show rollover styling for other icons (see 
*  FormItem.showOverIcons and FormItemIcon.showOverWhen).
*  
*  See formItemStyling for more details on formItem styling.
* @type {boolean}
* @default false
*/
showOver?: boolean;

/**
* If FormItem.showOver is true, setting this property to false will explicitly
*  disable showing the "Over" state for the TextBox element of this item.
* @type {boolean}
* @default null
*/
updateTextBoxOnOver?: boolean;

/**
* If FormItem.showOver is true, setting this property to false will explicitly
*  disable showing the "Over" state for the control table element of
*  this item (if present).
* @type {boolean}
* @default null
*/
updateControlOnOver?: boolean;

/**
* If FormItem.showOver is true, setting this property to false will explicitly
*  disable showing the "Over" state for the PickerIcon of this item (if present)
* @type {boolean}
* @default null
*/
updatePickerIconOnOver?: boolean;

/**
* When this item is disabled, should it be re-styled to indicate its disabled state?
*  
*  See formItemStyling for more details on formItem styling.
* @type {boolean}
* @default true
*/
showDisabled?: boolean;

/**
* When this item is in RTL mode, should its style name include an "RTL" suffix?
* @type {boolean}
* @default false
*/
showRTL?: boolean;

/**
* When true, causes the "Pending" optional suffix to be added if the item's
*  current value differs from the value that would be restored by a call to DynamicForm.resetValues.
*  
*  FormItem.shouldSaveValue must be true for this setting
*  to have an effect.
*  
*  Styling of the value is updated only after the FormItem.change event is processed,
*  so depending on the value of FormItem.changeOnKeypress, styling may
*  be updated immediately on keystroke or only when the user leaves the field.
*  
*  Default styling is provided for the Enterprise, EnterpriseBlue, and Graphite skins only.
*  showPending should not be enabled for an item when using a skin without
*  default styling unless the default FormItem.pendingStatusChanged behavior is
*  canceled and a custom pending visual state is implemented by the item.
*  
*  NOTE: Whether an item is shown as pending is not reflected to screen
*  readers. Therefore, it is not advisable to design a UI where it is necessary for the user
*  to know whether an item is shown as pending in order to work with the form.
* @type {boolean}
* @default null
*/
showPending?: boolean;

/**
* For items that support SelectItem.multiple, this causes distinct CSS styling
*  to be applied to values that the user has unselected.
*  
*  Only allowed when FormItem.showPending is true. Defaults
*  to the form-level DynamicForm.showDeletions setting if set; otherwise, to the
*  value of showPending.
*  
*  Only supported for MultiComboBoxItem and for SelectItem when
*  SelectItem.multiple is set. The specific default behaviors are:
*  
*  
* - For MultiComboBoxItem, buttons corresponding to deleted values
*    (also called "deselected buttons") will be disabled and have their Button.baseStyle
*    set to MultiComboBoxItem.deselectedButtonStyle.
*  
* - For a multiple SelectItem, FormItem.valueDeselectedCSSText is
*    applied to any deleted value in the text box. In addition, "Deselected" is appended
*    to the cells' ListGrid.baseStyle for cells in the pickList menu corresponding
*    to deleted values.
*  
*  
*  NOTE: When a value is shown as deleted, this is not reflected to screen
*  readers, and screen readers are instructed to ignore the deleted value. Therefore, it is
*  not advisable to design a UI where it is necessary for the user to know whether a value
*  is shown as deleted in order to work with the form.
* @type {boolean}
* @default null
*/
showDeletions?: boolean;

/**
* CSS style applied to the form item as a whole, including the text element, any icons, and
*  any hint text for the item. Applied to the cell containing the form item.
*  
*  See formItemStyling for an overview of formItem styling, and 
*  the CompoundFormItem_skinning discussion for special
*  skinning considerations.
* @type {FormItemBaseStyle}
* @default "formCell"
*/
cellStyle?: FormItemBaseStyle;

/**
* CSS class for the "hint" string. For items that support
*  TextItem.showHintInField, this only applies when showHintInField is false.
* @type {CSSStyleName}
* @default "formHint"
*/
hintStyle?: CSSStyleName;

/**
* By default, FormItem.canEdit fields use the same style name as editable 
*  fields for in-field hints, unless they are FormItem.isDisabled or 
*  configured to use a disabled ReadOnlyDisplayAppearance. This is described 
*  under TextItem.showHintInField
*  
*  If useDisabledHintStyleForReadOnly is set, the "HintDisabled" style will be 
*  used for read-only fields regardless of their ReadOnlyDisplayAppearance. This
*  allows you to use a different in-field hint style for read-only fields without having to 
*  use a general disabled appearance for those fields
* @type {boolean}
* @default null
*/
useDisabledHintStyleForReadOnly?: boolean;

/**
* Base CSS class name for a regular form-item's title. Note that this is a FormItemBaseStyle 
*  so will pick up stateful suffixes on focus, disabled state change etc. by default.
*  
*  When the FormItem.title is shown FormItem.titleOrientation 
*  the item, the title element is given the 
*  verticalTitleStyle if it's set, as it is in some
*  skins - otherwise, it will fall back to titleStyle.
*  
*  Note the appearance of the title is also affected by
*  DynamicForm.titlePrefix/DynamicForm.titleSuffix and 
*  DynamicForm.requiredTitlePrefix/DynamicForm.requiredTitleSuffix.
* @type {FormItemBaseStyle}
* @default "formTitle"
*/
titleStyle?: FormItemBaseStyle;

/**
* Base CSS stylename for a form item's title when generating print HTML for the item.
*  If unset FormItem.titleStyle will be used instead.
* @type {FormItemBaseStyle}
* @default null
*/
printTitleStyle?: FormItemBaseStyle;

/**
* Base CSS class name for a form item's text box element.
*  
*  See formItemStyling for an overview of formItem styling, and 
*  the CompoundFormItem_skinning discussion for special
*  skinning considerations.
*  
*  If the textBoxStyle is changed at runtime, FormItem.updateState
*  must be called to update the visual state of this item.
* @type {FormItemBaseStyle}
* @default null
*/
textBoxStyle?: FormItemBaseStyle;

/**
* Base CSS class name for a form item's text box element when getting printable HTML for the
*  form. If unset FormItem.textBoxStyle will be used instead. Note that focused styling
*  will never be displayed while printing, though error and disabled styling will.
*  
*  By default this style will be used for printHTML for the item even if the item is
*  FormItem.canEdit with FormItem.readOnlyDisplay.
*  To override this behavior, developers may also specify a custom print style for this 
*  state via the 
*  FormItem.printReadOnlyTextBoxStyle.
* @type {FormItemBaseStyle}
* @default null
*/
printTextBoxStyle?: FormItemBaseStyle;

/**
* CSS class name to apply to the print view of an item's text box if the item
*  is FormItem.canEdit, with FormItem.readOnlyDisplay.
*  
*  If specified this will take precedence over FormItem.printTextBoxStyle for
*  static readonly items.
* @type {FormItemBaseStyle}
* @default null
*/
printReadOnlyTextBoxStyle?: FormItemBaseStyle;

/**
* Base CSS class name for a form item's picker icon cell. If unset, inherits from
*  this item's FormItem.controlStyle.
* @type {FormItemBaseStyle}
* @default null
*/
pickerIconStyle?: FormItemBaseStyle;

/**
* Base CSS class name for a form item's "control box". This is an HTML element
*  which contains the text box and picker icon for the item.
*  
*  See FormItem.alwaysShowControlBox for details on when the control box
*  is written out.
*  
*  See formItemStyling for an overview of formItem styling, and 
*  the CompoundFormItem_skinning discussion for special
*  skinning considerations.
* @type {FormItemBaseStyle}
* @default null
*/
controlStyle?: FormItemBaseStyle;

/**
* Custom CSS text to be applied to cells with pending edits that have not yet been
*  submitted.
* @type {CSSText}
* @default "color:#0066CC;"
*/
editPendingCSSText?: CSSText;

/**
* Custom CSS text to be applied to values that have been deleted, when
*  FormItem.showDeletions is enabled.
* @type {CSSText}
* @default "color:#A8A8A8;text-decoration:line-through;"
*/
valueDeselectedCSSText?: CSSText;

/**
* If set to true, when an item has errors and is focused, an "ErrorFocused" suffix 
*  will appear on the stylename.
* @type {boolean}
* @default false
*/
showFocusedErrorState?: boolean;

/**
* CSS class for a form item's cell in the form layout
* @deprecated * As of SmartClient version 5.5, deprecated in favor of FormItem.cellStyle

* @type {CSSStyleName}
* @default null
*/
cellClassName?: CSSStyleName;

/**
* CSS class for a form item's cell when a validation error is showing.
* @deprecated * As of SmartClient version 5.5 deprecated in favor of FormItem.cellStyle

* @type {CSSStyleName}
* @default null
*/
errorCellClassName?: CSSStyleName;

/**
* CSS class for the form item's title.
* @deprecated * As of SmartClient Version 5.5, use FormItem.titleStyle instead

* @type {CSSStyleName}
* @default null
*/
titleClassName?: CSSStyleName;

/**
* CSS class for a form item's title when a validation error is showing.
* @deprecated * As of SmartClient Version 5.5, use FormItem.titleStyle instead

* @type {CSSStyleName}
* @default null
*/
titleErrorClassName?: CSSStyleName;

/**
* CSS class for the "hint" string.
* @deprecated * As of SmartClient version 5.5, deprecated in favor of FormItem.hintStyle

* @type {CSSStyleName}
* @default null
*/
hintClassName?: CSSStyleName;

/**
* If specified, this is the number of milliseconds to wait between the user rolling over 
*  this form item, and triggering any hover action for it.
*  If not specified this.form.itemHoverDelay will be used instead.
* @type {number}
* @default null
*/
hoverDelay?: number;

/**
* Option to specify a width for any hover shown for this item.
* @type {number | string}
* @default null
*/
hoverWidth?: number | string;

/**
* Option to specify a height for any hover shown for this item.
* @type {number | string}
* @default null
*/
hoverHeight?: number | string;

/**
* Text alignment for text displayed in this item's hover canvas, if shown.
* @type {Alignment}
* @default null
*/
hoverAlign?: Alignment;

/**
* Vertical text alignment for text displayed in this item's hover canvas, if shown.
* @type {VerticalAlignment}
* @default null
*/
hoverVAlign?: VerticalAlignment;

/**
* Explicit CSS Style for any hover shown for this item.
* @type {CSSStyleName}
* @default null
*/
hoverStyle?: CSSStyleName;

/**
* Opacity for any hover shown for this item
* @type {number}
* @default null
*/
hoverOpacity?: number;

/**
* If true and the title is clipped, then a hover containing the full title of this item
*  is enabled.
*  
*  The FormItem.titleHover method is called before the
*  hover is displayed, allowing the hover to be canceled if desired. The HTML shown in the
*  hover can be customized by overriding FormItem.titleHoverHTML.
*  
* @type {boolean}
* @default true
*/
showClippedTitleOnHover?: boolean;

/**
* If true and the value is clipped, then a hover containing the full value of this item
*  is enabled.
*  
*  The FormItem.valueHover method is called before the
*  hover is displayed, allowing the hover to be canceled if desired. The HTML shown in the
*  hover can be customized by overriding FormItem.valueHoverHTML.
*  
* @type {boolean}
* @default true
*/
showClippedValueOnHover?: boolean;

/**
* Causes the original value to be shown to the end user when the user hovers over the
*  FormItem as such (when the FormItem.itemHover event would fire).
*  
*  When FormItem.showOldValueInHover and the form's DynamicForm.showOldValueInHover
*  are both unset, defaults to the value of FormItem.showPending.
*  
*  The message shown is controlled by FormItem.originalValueMessage, unless the item is 
*  FormItem.disabled and FormItem.disabledHover is 
*  set - in this case, the hover shows the disabledHover HTML.
* @type {boolean}
* @default null
*/
showOldValueInHover?: boolean;

/**
* Message shown when FormItem.showOldValueInHover is enabled and
*  the value has been modified.
*  
*  If unset, defaults to the form's DynamicForm.originalValueMessage. Otherwise,
*  overrides the form-default originalValueMessage for this item.
* @type {HTMLString}
* @default null
*/
originalValueMessage?: HTMLString;

/**
* Text shown as the value in the FormItem.originalValueMessage when 
*  FormItem.showOldValueInHover is enabled, and when 
*  the value has been modified but was originally unset.
* @type {HTMLString}
* @default "None"
*/
nullOriginalValueText?: HTMLString;

/**
* OperatorId to be used when DynamicForm.getValuesAsCriteria is called.
*  
*  item.operator can be used to create a form that offers search functions such
*  as numeric range filtering, without the more advanced user interface of the
*  FilterBuilder. For example, two SpinnerItems could be created with
*  formItem.operator set to "greaterThan" and "lessThan" respectively to
*  enable filtering by a numeric range.
*  
*  When item.operator is set for any FormItem in a form,
*  form.getValuesAsCriteria() will return an AdvancedCriteria object
*  instead of a normal Criteria object. Each FormItem will produce one
*  Criterion affecting the DataSource field specified by FormItem.criteriaField.
*  The criteria produced by the FormItems will be grouped under the logical operator
*  provided by DynamicForm.operator.
*  
*  If operator is set for some fields but not others, the default operator is
*  "equals" for fields with a valueMap or an optionDataSource, and for fields of type "enum"
*  (or of a type that inherits from "enum"). The default operator for all other fields is
*  controlled by DynamicForm.defaultSearchOperator. 
*  
*  Note: formItem.operator is only supported for a form that has a 
*  DataBoundComponent.dataSource. In a form with no DataSource, 
*  setting formItem.operator will have no effect.
* @type {OperatorId}
* @default null
*/
operator?: OperatorId;

/**
* When using FormItem.operator, the name of the DataSource field for the
*  Criterion this FormItem generates. If not specified, defaults to
*  FormItem.name.
*  
*  Generally, because criteriaField defaults to item.name, you don't
*  need to specify it. However, if more than one FormItem specifies criteria for the same
*  DataSource field, they will need unique values for FormItem.name but should set
*  FormItem.criteriaField to the name of DataSource field they both target.
*  
*  For example, if two DateItems are used to provide a min and max date for a single field called
*  "joinDate", set FormItem.criteriaField to "joinDate" on both fields but give the fields
*  distinct names (eg "minDate" and "maxDate") and use those names for any programmatic access,
*  such as DynamicForm.setValue.
* @type {FieldName}
* @default null
*/
criteriaField?: FieldName;

/**
* Formula to be used to calculate the numeric value of this FormItem. For a field of type
*  "text" (or subtypes) FormItem.textFormula is used instead.
*  
*  Available fields for use in the formula are the current Canvas.ruleScope.
*  The formula is re-evaluated every time the rule context changes.
*  
*  Values calculated by the formula will always replace the current value of a non-editable
*  field. For an editable field, the current value will be replaced if the end user has not
*  changed the value since the last time it was computed by the formula, or if the value of the
*  field is invalid according to declared FormItem.validators.
*  
*  Note: A FormItem using a formula must have a FormItem.name defined. FormItem.shouldSaveValue can
*  be set to false to prevent the formula field from storing the calculated value
*  into the form's values.
* @type {UserFormula}
* @default null
*/
formula?: UserFormula;

/**
* Formula to be used to calculate the text value of this FormItem. For a numeric field
*  FormItem.formula is used instead.
*  
*  Available fields for use in the formula are the current Canvas.ruleScope.
*  The formula is re-evaluated every time the rule context changes.
*  
*  See FormItem.formula for details on available fields for the formula and when the
*  formula is calculated.
*  
*  Note: A FormItem using a textFormula must have a FormItem.name defined. FormItem.shouldSaveValue can
*  be set to false to prevent the formula field from storing the calculated value
*  into the form's values.
* @type {UserSummary}
* @default null
*/
textFormula?: UserSummary;

/**
* Set this to true to allow the parent form to save it's data when 'Enter' is pressed on 
*  this formItem and DynamicForm.saveOnEnter is true on the parent form.
* @type {boolean}
* @default null
*/
saveOnEnter?: boolean;

/**
* If true, indicates that this FormItem is capable of editing "opaque" values, ie, 
*  objects that are more complex than simple primitive types like numbers, strings and
*  dates. Ordinarily, you use the SimpleType to 
*  convert these opaque values into "atomic" values that can be edited by the built-in 
*  editors like TextItem. However, sometimes you to create a custom editor
*  that knows how to edit a particular opaque type in a domain-specific way - for example,
*  a composite custom FormItem that allows the user to edit both a number and a currency 
*  code, both of which are needed to make a proper monetary amount (for that particular 
*  application).
* 
*  When this value is set, the FormItem will manage the opaque value directly, rather
*  than it being filtered through calls to 
*  SimpleType.getAtomicValue and 
*  SimpleType.updateAtomicValue. Note, if you set this flag on
*  a FormItem that does not have the ability to edit an opaque value (which is something
*  that must be custom-coded) then you will get garbage in your editor, if not an outright
*  crash.
* @type {boolean}
* @default null
*/
canEditOpaqueValues?: boolean;

/**
* If true, multiple values may be selected.
*  
*  Only certain FormItems that support both singular and multiple values actually use
*  this setting, such as SelectItem. Other FormItems such as
*  MultiComboBoxItem are always effectively multiple:true, even if
*  multiple:true is not set.
* @type {boolean}
* @default false
*/
multiple?: boolean;

/**
* The destroyed attribute will be set to true if this item has been destroyed()
*  Note that FormItem lifecycle is managed by the DynamicForm itself. 
*  FormItem instances are created and destroyed automatically when new fields are 
*  added to the form.
* @type {boolean}
* @default null
*/
destroyed?: boolean;

/**
* When DynamicForm.showInlineErrors and FormItem.showErrorText are both true and
*  FormItem.errorOrientation is "left" or "right", errorMessageWidth is the amount to reduce
*  the width of the editor to accommodate the error message and icon.
* @type {Integer}
* @default "80"
*/
errorMessageWidth?: Integer;

/**
* If FormItem.height is specified, should it be applied to the
*  item's text box element?
*  
*  If unset, behavior is determined as described in FormItem.shouldApplyHeightToTextBox
* @type {boolean}
* @default null
*/
applyHeightToTextBox?: boolean;

/**
* A formItem showing a FormItem.showPickerIcon will always
*  write out a "control box" around the text box and picker icon. This is an HTML
*  element styled using the specified FormItem.controlStyle.
*  
*  This attribute controls whether the control box should be written out even
*  if the picker icon is not being shown. If unset, default behavior will write out
*  a control table if FormItem.showPickerIcon is true and the icon is
*  not suppressed via FormItemIcon.showIf. This means the control table
*  can be written out with no visible picker if FormItem.showPickerIconOnFocus
*  is true and the item does not have focus.
*  
*  This attribute is useful for developers who wish to rely on styling specified
*  via the FormItem.controlStyle even while the picker icon is not visible.
*  
*  See the formItemStyling for details of the
*  control table and other styling options.
* @type {boolean}
* @default null
*/
alwaysShowControlBox?: boolean;

/**
* If specified, this overrides the DynamicForm.storeDisplayValues property
*  for this field.
* @type {boolean}
* @default null
*/
storeDisplayValues?: boolean;

/**
* For fields of FormItemType, if the field is non editable, and
*  being displayed with FormItem.readOnlyDisplay, should
*  the value (URL) be displayed as text, or should an image be rendered?
*  
*  If unset, DynamicForm.showImageAsURL will be consulted instead.
* @type {boolean}
* @default null
*/
showImageAsURL?: boolean;

/**
* When true, indicates that changes to this item will cause an automatic save on a 
*  DynamicForm.implicitSaveDelay, as well as when the entire form is
*  submitted. If implicitSaveOnBlur is set to true on either this 
*  FormItem.implicitSaveOnBlur or it's 
*  DynamicForm.implicitSaveOnBlur, changes will also be automatically saved 
*  immediately on editorExit.
* @type {boolean}
* @default false
*/
implicitSave?: boolean;

/**
* If set to true, this item's value will be saved immediately when its 
*  "editorExit" handler is fired. This attribute works separately from 
*  FormItem.implicitSave, which causes saves during editing, after a 
*  DynamicForm.implicitSaveDelay,
*  and when the entire form is submitted.
* @type {boolean}
* @default false
*/
implicitSaveOnBlur?: boolean;

/**
* Should this item's value be saved in the form's values and hence returned from
*  DynamicForm.getValues?
*  
*  shouldSaveValue:false is used to mark formItems which do not correspond to
*  the underlying data model and should not save a value into the form's
*  DynamicForm.values. Example includes visual separators, password re-type fields,
*  or checkboxes used to show/hide other form items.
*  
*  A shouldSaveValue:false item should be given a value either via
*  FormItem.defaultValue or by calling
*  DynamicForm.setValue or 
*  FormItem.setValue. Providing a value via
*  DynamicForm.values or DynamicForm.setValues 
*  will automatically switch the item to shouldSaveValue:true.
*  
*  Note that 
*  
* - if an item is shouldSaveValue true, but has no name, a warning is logged, and 
*    shouldSaveValue will be set to false.
*  
* @type {boolean}
* @default true
*/
shouldSaveValue?: boolean;

/**
* Should this form item always produce an AdvancedCriteria sub criterion object?
*  When set to true, causes FormItem.hasAdvancedCriteria to 
*  return true. Can also be set at the ListGrid.useAdvancedCriteria level.
* @type {boolean}
* @default null
*/
useAdvancedCriteria?: boolean;

/**
* This attribute gives users a way to pin this item's hover in place so they can interact 
*  with it (scroll it, click embedded links, etc).
*  
*  Overrides the Canvas.hoverFocusKey on the parent form.
* @type {KeyName}
* @default null
*/
hoverFocusKey?: KeyName;

/**
* Allows interaction with hovers when the cursor is positioned over them.
*  
*  Overrides the Canvas.hoverPersist on the parent form.
* @type {HoverPersistMode}
* @default null
*/
hoverPersist?: HoverPersistMode;

/**
* For items showing a text value, should the user be able to select the text in this item? 
*  
*  For FormItem.canEdit items, see FormItem.readOnlyCanSelectText
* @type {boolean}
* @default true
*/
canSelectText?: boolean;

/**
* For items showing a text value with FormItem.canEdit set to false, 
*  should the user be able to select the text in the item?
*  
*  Default behavior allows selection if FormItem.readOnlyDisplay is 
*  "static" or "readOnly" [but not "disabled"].
*  Developers may add or remove ReadOnlyDisplayAppearance values to change this behavior.
*  
*  Note that this does not apply to FormItem.disabled, 
*  where text selection is
*  never enabled
* @type {Array<Partial<ReadOnlyDisplayAppearance>>}
* @default "["static", "readOnly"]"
*/
readOnlyCanSelectText?: Array<Partial<ReadOnlyDisplayAppearance>>;

/**
* Does the current formItem support native cut and paste events?
*  
*  This attribute only applies to freeform text entry fields such as TextItem and
*  TextAreaItem, and only if FormItem.changeOnKeypress is true. 
*  If true, developers can detect the user editing the value 
*  via cut or paste interactions (triggered from keyboard shortcuts or the native 
*  browser menu options) using the FormItem.isCutEvent and
*  FormItem.isPasteEvent methods. This allows custom cut/paste handling
*  to be added to the various change notification flow methods including 
*  FormItem.change, and
*  FormItem.transformInput.
* @type {boolean}
* @default false
*/
supportsCutPasteEvents?: boolean;

/**
* For a form that produces filter criteria
*  (see DynamicForm.getValuesAsCriteria), allows the user
*  to type in simple expressions to cause filtering with that operator. For
*  example, entering "&gt;5" means values greater than 5, and "&gt;0 and &lt;5" means values between
*  0 and 5.
*  
*  The following table lists character sequences that can be entered as a prefix to a value, 
*  and the corresponding OperatorId that will be used. 
*  
*  
*  PrefixOperator
*  &lt;lessThan
*  &gt;greaterThan
*  &lt;=lessThanOrEqual
*  &gt;=greaterThanOrEqual
*  someValue...someValuebetweenInclusive
*  !notEqual
*  ^startsWith
*  |endsWith
*  !^notStartsWith plus logical not
*  !@notEndsWith plus logical not
*  ~contains
*  !~notContains
*  $isBlank
*  !$notBlank
*  #isNull
*  !#isNotNull
*  ==exact match (for fields where 'contains' is the default)
*  
*  
*  Two further special notations are allowed:
*  
*  
* - /regex/ means the value is taken as a regular expression and applied via the
*  "regexp" operator
*  
* - =.fieldName means the value should match the value of another field. Either the
*  user-visible title of the field (field.title) or the field's name (field.name) may be used.
*  
*  
*  In all cases, if an operator is disallowed for the field (via
*  DataSourceField.validOperators at either the dataSource or field
*  level), the operator character is ignored (treated as part of a literal value).
*  
*  By default, the case-insensitive version of the operator is used (eg, startsWith will
*  actually use "iStartsWith"). To avoid this, explicitly set item.operator (the default
*  operator) to any case sensitive operator (eg "equals" or "contains") and case sensitive
*  operators will be used for user-entered expressions.
*  
*  Compound expressions (including "and" and "or") are allowed only for numeric or date/time
*  types.
*  
*  Note that if the user does not type a prefix or use other special notation as described
*  above, the operator specified via FormItem.operator is used, or if
*  formItem.operator is unspecified, a default operator chosen as described
*  under FormItem.operator. 
*  
*  Also note that whatever you enter will be used literally, including any whitespace
*  characters. For example if you input '== China ' then ' China ' will be the value.
*  
*  The allowExpression behavior can be enabled for every field in a form via
*  DynamicForm.allowExpressions.
*  
*  Finally, note that, like FormItem.operator, enabling
*  allowExpressions:true causes
*  DynamicForm.getValuesAsCriteria) to return
*  AdvancedCriteria.
* @type {boolean}
* @default null
*/
allowExpressions?: boolean;

/**
* Array of valid filtering operators (eg "greaterThan") that are legal for this FormItem.
*  
*  Applies only to form/formItem when FormItem.allowExpressions is true, allowing the
*  user to input expressions.
* @type {Array<Partial<OperatorId>>}
* @default null
*/
validOperators?: Array<Partial<OperatorId>>;

/**
* The default search-operator for this item when it or its form allow 
*  FormItem.allowExpressions.
* @type {OperatorId}
* @default null
*/
defaultOperator?: OperatorId;



/**
*  Setter for FormItem.readOnlyDisplay.
*  
*  Note that calling this method for a ButtonItem with ButtonItem.enableWhen
*  set is an error, since FormItem.readOnlyDisplay is then considered to always be "disabled".
* 
* @param {ReadOnlyDisplayAppearance} new readOnlyDisplay value.
*/
setReadOnlyDisplay(appearance:ReadOnlyDisplayAppearance): void;

/**
*  Notification method fires when the user clicks a FormItem.valueIcons for
*  this item.
* 
* @param {DynamicForm} the form containing this item
* @param {FormItem} the FormItem containing the valueIcon
* @param {any} the current value of the item.
*/
valueIconClick(form:DynamicForm, item:FormItem, value:any): boolean;

/**
*  Returns true if this item's FormItem.containerWidget is a 
*  GridRenderer or GridRenderer subclass
* 
* 
*/
isInGrid(): boolean;

/**
*  If this item is being used to edit cells in a ListGrid (see FormItem.isInGrid), 
*  this method returns the grid in question.
* 
* 
*/
getListGrid(): ListGrid;

/**
*  If this formItem is part of a ListGrid's 
*  ListGrid.canEdit, returns the number of the row currently being
*  edited. If the formItem is not part of a ListGrid inline edit for any reason, this 
*  method returns null. Reasons for a formItem not being part of an inline edit include
*  
* - The item is part of an ordinary DynamicForm, not an inline edit form
*  
* - There is no row in the grid currently being edited
*  
* - A row is being edited, but this formItem is not currently visible and is being 
*  excluded because of horizontal incremental rendering (where SmartClient avoids drawing
*  grid columns that would not be visible without scrolling)
*  
* 
* 
*/
getGridRowNum(): Integer;

/**
*  If this formItem is part of a ListGrid's 
*  ListGrid.canEdit, returns the number of the grid column this 
*  formItem is responsible for editing, but only if a row is currently being
*  edited. If the formItem is not part of a ListGrid inline edit for any reason, this 
*  method returns null. Reasons for a formItem not being part of an inline edit include
*  
* - The item is part of an ordinary DynamicForm, not an inline edit form
*  
* - There is no row in the grid currently being edited
*  
* - A row is being edited, but this formItem is not currently visible and is being 
*  excluded because of horizontal incremental rendering (where SmartClient avoids drawing
*  grid columns that would not be visible without scrolling)
*  
* 
* 
*/
getGridColNum(): Integer;

/**
*  Returns the vertical-alignment for this item within its cell. By default, when
*  FormItem.titleOrientation is "top", this method will 
*  return "top", so that items of varying height are top-aligned, beneath their titles.
* 
*/
getVAlign(): VerticalAlignment;

/**
*  	Return the name for the this formItem.
* 
*/
getFieldName(): string;

/**
*  Return the dataPath for the this formItem.
* 
*/
getDataPath(): DataPath;

/**
*  Return the fully-qualified dataPath for the this formItem (ie, the dataPath expressed 
*  in absolute terms from the root of the hierarchy, rather than relative to the item's 
*  parent form). Note that the item's name is substituted into the full dataPath if the
*  item does not specify an explicit dataPath. For example, if we have a field called 
*  name that specifies no dataPath, on a form that specifies a dataPath of 
*  /order/items, this method will return /order/items/name
* 
*/
getFullDataPath(): DataPath;

/**
*  Returns true if 'Enter' key presses in this formItem should allow a saveOnEnter: true
*  parent form to save it's data. The default implementation returns the value of
*  FormItem.saveOnEnter or false if that property is unset.
* 
*/
shouldSaveOnEnter(): boolean;

/**
*  Returns the title HTML for this item.
* 
*/
getTitle(): HTMLString;

/**
*  Return true if the form item is currently visible. Note that like the similar
*  Canvas.isVisible, it indicates visibility settings only and so
*  will return true for an item that is not drawn.
*  
* 
*/
isVisible(): boolean;

/**
*  Return the coordinates of this object as a 4 element array.
* 
*/
getRect(): Array<any>;

/**
*  Return the page-level coordinates of this object as a 4 element array.
* 
*/
getPageRect(): Array<any>;

/**
*  If FormItem.height is specified, should it be applied to the
*  item's text box element? If this method returns false, the
*  text box will not have an explicit height applied, though the containing cell
*  will be sized to accommodate any specified height.
*  
*  This is used in cases where the text box does not have distinctive styling
*  (for example in standard StaticTextItems). As the textBox has no explicit
*  height, it fits the content. Since the text box is not visually distinct to 
*  the user, this makes FormItem.vAlign behave as expected with the 
*  text value of the item being vertically aligned within the cell.
*  
*  Default implementation will return FormItem.applyHeightToTextBox if explicitly set
*  otherwise false if FormItem.readOnlyDisplay is set to 
*  "static" and the item is FormItem.getCanEdit, otherwise
*  true.
* 
*/
shouldApplyHeightToTextBox(): boolean;

/**
*  Returns the specified FormItem.height of this formItem in pixels. 
*  For heights specified as a percentage value or "*", the 
*  pixel height may not be available prior to the item being drawn. In cases where
*  the height has not yet been resolved to a pixel value, this method will return
*  -1.
* 
*/
getPixelHeight(): Integer;

/**
* 	Output the drawn height for this item in pixels.
*  Note: this is only reliable after this item has been written out into the DOM.
* 
*/
getVisibleHeight(): Integer;

/**
* 	Takes an icon definition object, and returns the height for that icon in px.
* 
* @param {object} icon definition object for this item.
*/
getIconHeight(icon:object): Integer;

/**
* 	Output the width for this element. Note this returns the specified width for the 
*  element, which may be "*" or a percentage value. Use 'getVisibleWidth()' to get the
*  drawn width in pixels.
* 
*/
getWidth(): Integer | string;

/**
*  Returns the specified FormItem.width of this formItem in pixels. 
*  For widths specified as a percentage value or "*", the 
*  pixel width may not be available prior to the item being drawn. In cases where
*  the width has not yet been resolved to a pixel value, this method will return
*  -1.
* 
*/
getPixelWidth(): Integer;

/**
* 	Output the drawn width for this item in pixels. This method is only reliable after
*  the item has been drawn into the page.
* 
*/
getVisibleWidth(): Integer;

/**
* 	Returns the visible width of this item's title in px. If that is not applicable (for 
*  example, the form item has no title) or cannot be determined (for example, the form 
*  is not drawn), returns 0.
* 
* @param {boolean} If true, returns the visible width of the title text                 only; if false (the default) returns the width of the                  title cell
*/
getVisibleTitleWidth(labelOnly:boolean): number;

/**
* 	Takes an icon definition object, and returns the width for that icon in px.
* 
* @param {object} icon definition object for this item.
*/
getIconWidth(icon:object): Integer;

/**
*  For a form with DynamicForm.itemLayout:"absolute" only, set the left
*  coordinate of this form item.
*  
*  Causes the form to redraw.
* 
*/
setLeft(): void;

/**
*  For a form with DynamicForm.itemLayout:"absolute" only, set the top
*  coordinate of this form item.
*  
*  Causes the form to redraw.
* 
*/
setTop(): void;

/**
*  Update the visual state of a FormItem to reflect any changes in state or any changes in
*  style settings (e.g. FormItem.textBoxStyle).
*  
*  Calls to updateState() normally occur automatically as a consequence of
*  focus changes, items becoming disabled, etc. This method is advanced and intended only
*  for use in workarounds.
* 
* 
*/
updateState(): void;

/**
*  Except when printing and FormItem.getPrintValueIconStyle
*  is implemented, this method is called to obtain the base CSS style to use on the
*  FormItem.getValueIcon for the item's current value.
*  
*  If not null, the base style is suffixed with the state of the value icon
*  ("", "Over", "Down", "Disabled").
* 
* 
* @param {any} value of this item.
*/
getValueIconStyle(value:any): CSSStyleName;

/**
*  If implemented, this method is called when printing to obtain
*  the base CSS style to use on the FormItem.getPrintValueIcon
*  for the item's current value.
*  
*  If not null, the base style is suffixed with the state of the value icon
*  ("", "Over", "Down", "Disabled").
*  
*  NOTE: It is not recommended to apply CSS background-image styling to the
*  value icon style. This is because browsers typically default to not printing background
*  images.
* 
* 
* @param {any} value of this item.
*/
getPrintValueIconStyle(value:any): CSSStyleName;

/**
*  Optional method to retrieve a custom state suffix to append to the style name that is 
*  applied to some element of a formItem - see FormItemBaseStyle for more 
*  information on how state-based FormItem style names are derived. 
*  
*  If this method exists on a formItem, the framework will call it, passing in the state
*  suffix it has derived. Your getCustomState() implementation can make use 
*  of this derived state or ignore it. For example, if you wanted two different types of
*  focus styling depending on some factor unrelated to focus, you would probably make use
*  of the incoming "Focused" state and return something like "Focused1" or "Focused2". On 
*  the other hand, if you want your custom state to just override whatever the system 
*  derived, you would ignore the incoming state. Finally, if you do not wish to provide a
*  custom style for this formItem element at this time - for example, you are only 
*  interested in providing a custom "textBox" style and this call is for a "cell" element 
*  type - your getCustomStyle() method should just return the state it was
*  passed.
*  
*  This method is an advanced API, and you should only provide an implementation of it if 
*  you have specialized styling requirements. If you do implement it, note that it will 
*  be called very frequently, from rendering code: if your custom logic does significant 
*  processing, it could introduce a system-wide performance problem.
* 
* 
* @param {FormItemElementType} The element type to return a custom state for
* @param {string} The state suffix the system derived
*/
getCustomState(elementType:FormItemElementType, derivedState:string): string;

/**
*  Setter for FormItem.cellStyle.
* 
* 
* @param {FormItemBaseStyle} the new cellStyle value.
*/
setCellStyle(newCellStyle:FormItemBaseStyle): void;

/**
*  Return the validation errors in the form associated with this item, if any.
*  Errors will be returned as either a string (single error message) or an array of strings
*  (multiple error messages).
* 
*/
getErrors(): string | Array<Partial<string>>;

/**
*  Output the HTML for an error message in a form element. Default behavior respects
*  FormItem.showErrorIcon and FormItem.showErrorText as described in the
*  documentation for those attributes.
* 
* @param {string | Array<Partial<string>>} error message string or array of error messages
*/
getErrorHTML(error:string | Array<Partial<string>>): HTMLString;

/**
*  Returns true if this item has been written out into the DOM.
* 
* 
*/
isDrawn(): boolean;

/**
*  Returns the desired tab-position of some icon with respect to other focusable
*  sub-elements for this formItem.
*  
*  Default implementation returns the index of the icon in the icons array, 
*  (plus one if a pickerIcon is showing) meaning users can tab through icons in order.
*  Has no effect for non-focusable icons.
* 
*/
getIconTabPosition(): Integer;

/**
*  Returns the desired tab-position of the picker icon with respect to other focusable
*  sub-elements for this formItem.
*  
*  Default implementation returns zero, making the picker the first focusable element
*  after the items text box.
* 
*/
getPickerIconTabPosition(): Integer;

/**
*  Adds a FormItemIcon to this item. If the optional index parameter is not passed,
*  the icon is added to the end of the existing FormItem.icons.
*  
*  If the passed icon already exists in the FormItem.icons, by 
*  FormItemIcon.name, the original icon is moved to the new index and no new 
*  icon is added.
* 
* @param {FormItemIcon} the icon to add
* @param {Integer=} the index to add the icon at - if omitted, add the icon to the end      of the current icon list
*/
addIcon(icon:FormItemIcon, index?:Integer): FormItemIcon;

/**
*  Given an icon's FormItemIcon.name, remove it from this item.
* 
* @param {Identifier} the name of the icon to remove
*/
removeIcon(icon:Identifier): boolean;

/**
*  Set an icon as enabled or disabled at runtime. 
* 
* @param {Identifier} FormItemIcon.name of the icon to be disabled/enabled.
* @param {boolean} true if icon should be disabled
*/
setIconDisabled(icon:Identifier, disabled:boolean): void;

/**
*  This method will enable some icon in this item's FormItem.icons array, if it is 
*  currently disabled.
* 
* 
* @param {Identifier} FormItemIcon.name of the icon to be enabled.
*/
enableIcon(icon:Identifier): void;

/**
*  This method will disable some icon in this item's FormItem.icons array, if it is 
*  currently enabled.
* 
* 
* @param {Identifier} FormItemIcon.name of the icon to be disabled.
*/
disableIcon(icon:Identifier): void;

/**
*  This method will show some icon in this item's FormItem.icons array, if it is not
*  already visible. Note that once this method has been called, any previously specified
*  FormItemIcon.showIf will be discarded.
*  
*  Note that if the form item's showIcons property is set to false, no icons will be displayed
*  for the item. In this case this method will not cause the icon to be displayed.
* 
* 
* @param {Identifier} FormItemIcon.name of the icon to be shown.
*/
showIcon(icon:Identifier): void;

/**
*  This method will hide some icon in this item's FormItem.icons array, if it is 
*  currently visible. Note that once this method has been called, any previously specified
*  FormItemIcon.showIf will be discarded.
* 
* 
* @param {Identifier} FormItemIcon.name of the icon to be hidden.
*/
hideIcon(icon:Identifier): void;

/**
*  Sets FormItemIcon.showOnFocus for the supplied icon, and causes that icon's
*  visibility to be updated and the item redrawn as appropriate.
* 
* 
* @param {Identifier} FormItemIcon.name of the icon to update
* @param {boolean} new value of FormItemIcon.showOnFocus
*/
setIconShowOnFocus(icon:Identifier, showOnFocus:boolean): void;

/**
*  Sets FormItem.showIconsOnFocus and causes the visibility of all FormItem.icons to be updated
*  and the item redrawn as appropriate.
* 
* 
* @param {boolean} new value of FormItem.showIconsOnFocus
*/
setShowIconsOnFocus(showIconsOnFocus:boolean): void;

/**
*  Sets FormItem.showPickerIconOnFocus and causes the visibility of the picker icon to be
*  updated and the item redrawn as appropriate.
* 
* 
* @param {boolean} new value of FormItem.showPickerIconOnFocus
*/
setShowPickerIconOnFocus(showPickerIconOnFocus:boolean): void;

/**
*  Given a FormItemIcon.name, returns the FormItemIcon object.
* 
* @param {Identifier} specified FormItemIcon.name
*/
getIcon(name:Identifier): FormItemIcon;

/**
*  Method to show a picker for this item. By default this method is called if the user
*  clicks on a FormItem.showPickerIcon. May also be called programmatically.
*  
*  Default implementation lazily creates and shows the FormItem.picker.
*  
*  Developers wishing to show a custom picker widget can either implement a 
*  FormItem.pickerIconClick handler with an entirely custom implementation (and bypass
*  the call to showPicker() altogether), 
*  override this method, 
*  or use the AutoChild to customize the automatically generated
*  FormItem.picker.
* 
* 
*/
showPicker(): void;

/**
*  Redraw this form item.
*  
*  Depending on the item and the FormItem.containerWidget it's embedded in, this may redraw
*  this particular item or require a redraw of all items in the form.
*  
*  Do not call this method unless the documentation directs you to do so. Calling
*  redraw() unnecessarily has significant performance consequences.
* 
* 
* @param {string=} optional reason for performing the redraw, which may appear in             diagnostic logs if enabled
*/
redraw(reason?:string): void;

/**
*  Show this form item.
*  
*  This will cause the form to redraw. If this item had an item.showIf expression, it will
*  be destroyed.
* 
*/
show(): void;

/**
*  Hide this form item.
*  
*  This will cause the form to redraw. If this item had an item.showIf expression, it will
*  be destroyed.
* 
*/
hide(): void;

/**
*  Returns this item's value with any valueMap applied to it - the value as currently
*  displayed to the user.
* 
* @param {any=} optional stored value to be mapped to a display value. Default is to        use the form's current value
*/
getDisplayValue(value?:any): any;

/**
*  Given a value for this FormItem, return the value to be displayed.
*  
*  This method is called by the framework to derive a display value for a given data value
*  in a FormItem. Your own code can call this method if you need to programmatically obtain
*  the display value (for example, to display in a hover prompt or error message). However,
*  it is not intended as an override point, and you should not treat 
*  it as one.
*  
*  There are several supported ways to apply custom formatting to your form values:
*  
* - If you want to apply a consistent custom format to every instance of a given
*  SimpleType, specify a SimpleType.format on the SimpleType.  
*  This is the most general approach. Note, this is a static formatter: it will only 
*  affect the format of values the user can interact with if TextItem.formatOnBlur
*  is set
*  
* - If you want to apply a consistent custom format to a 
*  DataSourceField, the best approach is 
*  DataSourceField.format. This overrides SimpleType-level formatting and,
*  again, is static formatting
*  
* - For a FormItem that is not DataSource-described, or for special formats that 
*  should only be used on a particular form, FormItem.format can also be 
*  declared for individual FormItems. This overrides DataSource-level formatting
*  
* - For temporal values, you can declare FormItem.dateFormatter 
*  and FormItem.timeFormatter at both FormItem and 
*  DynamicForm levels. Generally, however, we recommend the generic 
*  declarative FormItem.format as the simpler approach
*  
* - If you want to apply a specialized format that cannot be expressed declaratively,
*  use FormItem.formatValue
*  
*  for static-valued items like StaticTextItem or SelectItem,
*  and FormItem.formatEditorValue
*  
*  for other types of FormItem
*  
*  Deriving the display value
*  The process of deriving a display value from a data value involves the following steps:
*  
* - If the item declares a FormItem.valueMap, the display value is 
*  derived by looking up the value in the valueMap
*  
* - If the item does not have a valueMap - or the value was not found in the item's 
*  valueMap - and the item declares a FormItem.displayField
*  and an FormItem.optionDataSource, the display value is derived 
*  by looking up the "displayField" corresponding to the value in the optionDataSource's
*  local cache
*  
* - Formatting is now applied to the derived display value. Note, it is perfectly normal 
*  at this point for no display value has to be derived - this will be the case for any 
*  field with no valueMap and no optionDataSource. In this case, 
*  the passed-in value is treated as the display value for all further purposes.
*    
* - If the FormItem involves static display value(s), like StaticTextItem
*      or SelectItem
*      
* - If the FormItem
*      has a FormItem.formatValue method, it is called
*      
*      
* - Otherwise, if the formItem declares a FormItem.format, the formst
*        is applied in line with the rules of FormatString
*      
* - Otherwise, if the FormItem is of a SimpleType that declares a
*        SimpleType.format, the format is applied
*    
*    
*    
* - Otherwise, if the FormItem 
*      has a FormItem.formatEditorValue method, 
*            it is called
*      
*    
*    
* - Otherwise, if the FormItem is of a SimpleType that
*      declares an SimpleType.editFormatter,
*      
*      the edit formatter is called
*    
* - Otherwise, if the value is a Date:
*      
* - If the formItem declares a FormItem.timeFormatter and no 
*        FormItem.dateFormatter, the timeFormatter is called
*      
* - Otherwise, if the formItem is of a SimpleType that 
*        SimpleType.inheritsFrom "time", the value is formatted
*        using the 
*        Time.shortDisplayFormat
*        
*      
* - Otherwise, the date or datetime is formatted using the rules described 
*        for FormItem.dateFormatter.
*    
*    
*    
* - Otherwise, if the FormItem involves static display value(s) and is of a 
*      SimpleType that declares a 
*      SimpleType.normalDisplayFormatter, this is used
*    
* - Otherwise, if the value is not null and is of a "simple" type (ie, it is not an
*      object or an array), a display value is derived by calling the Javascript 
*      toLocaleString() method, if the value has one, or the 
*      toString() method if it does not
*    
* - Otherwise, if the value is null or a zero-length string, the display value is 
*      set to the formItem's FormItem.emptyDisplayValue
*    
* - Otherwise, the "display value" is the simple, unformatted data value that was 
*      passed in
*  
*  
*  
*  Treatment of arrays
*  Ordinarily, arrays are treated like any other value. This means you can, for example, 
*  create a FormItem.formatEditorValue implementation
*  
*  that is capable of formatting an array-valued field in some way that makes sense for the 
*  particular application domain.
*  
*  However, for items that are marked to handle DataSourceField.multiple 
*  values, array values are treated differently. In this case, the display value is built 
*  up by calling mapValueToDisplay() recursively for each array entry, and 
*  concatenating these partial display values together using the 
*  FormItem.multipleValueSeparator.
*  
* 
* @param {any} value to be mapped to a display value
*/
mapValueToDisplay(value:any): string;

/**
*  Allows customization of how the FormItem's stored value is formatted for display.
*  If you are considering using this method, you should first consider using 
*  FormItem.format, which provides for simple and flexible declarative 
*  formatting of dates, times and numbers, without the need to write formatting code.
*  
*  By default, this formatter will only be applied to static displays such
*  as StaticTextItem or SelectItem, and does not apply to values 
*  displayed in a freely editable text entry field 
*  (such as a TextItem or TextAreaItem).
*  
*  To define formatting logic for editable text, developers may:
*  
*  
* - set TextItem.formatOnBlur to true, which causes the static formatter
*  to be applied while the item does not have focus, and then be cleared when the user
*  moves focus to the text field
*  
* - use FormItem.formatEditorValue and supply a
*  corresponding FormItem.parseEditorValue that can convert a formatted and subsequently
*  user-edited value back to a stored value.
*  
* 
* @param {any} Underlying data value to format. May be null.
* @param {ListGridRecord} The record currently being edited by this form.   Essentially the form's current values object.
* @param {DynamicForm} pointer to the DynamicForm
* @param {FormItem} pointer to the FormItem
*/
formatValue(value:any, record:ListGridRecord, form:DynamicForm, item:FormItem): string;

/**
*  Allows customization of how the FormItem's stored value is formatted for display
*  in an editable text entry area, such as a TextItem TextAreaItem. For
*  display values which will not be directly editable by the user, use
*  FormItem.formatValue instead.
*  
*  When customizing how values are displayed during editing, it is almost always necessary
*  to provide a FormItem.parseEditorValue as well, in order to convert a formatted
*  and subsequently user-edited value back to a stored value.
* 
* 
* @param {any} Underlying data value to format. May be null.
* @param {ListGridRecord} The record currently being edited by this form.   Essentially the form's current values object.
* @param {DynamicForm} pointer to the DynamicForm
* @param {FormItem} pointer to the FormItem
*/
formatEditorValue(value:any, record:ListGridRecord, form:DynamicForm, item:FormItem): string;

/**
*  Allows customization of how a used-entered text value is converted to the FormItem's
*  logical stored value (the value available from FormItem.getValue). 
*  
*  This method only applies to form items which show an editable text entry area, such at
*  the TextItem or TextAreaItem.
*  
*  See also FormItem.formatEditorValue
* 
* 
* @param {string} value as entered by the user
* @param {DynamicForm} pointer to the dynamicForm containing this item
* @param {FormItem} pointer to this item
*/
parseEditorValue(value:string, form:DynamicForm, item:FormItem): any;

/**
*  Given a display value for this FormItem, return the underlying data value. This is 
*  done by reverse value-mapping, and/or parsing.
*  
*  This method is called by the framework to derive an underlying data value for a given 
*  display value (ie, the value the user sees and interacts with) in a FormItem. Your own 
*  code can call this method if you need to programmatically obtain the underlying data 
*  value for a given display value. However,
*  it is not intended as an override point, and you should not treat 
*  it as one.
*  
*  If you have a field that requires the stored value to be different from the displayed 
*  value, and the requirement cannot be satisfied with a FormItem.valueMap
*  for some reason, you can add custom parsing logic by 
*  implementing FormItem.parseEditorValue
*  
*  
*  This method is also not intended as a place where you can validate, sanitize, 
*  transform or canonicalize user input
*  
* - To ensure you get well-formed input values, use TextItem.mask or 
*  the FormItem.change
*  
*  
*  
* - To transform or canonicalize input values, use a ValidatorType
*  with "transformTo". See the link to "mask validator" for more details and an example of this
*  
* - To transform or canonicalize input character-by-character as the user types, use 
*  FormItem.transformInput
*  
*  
*  Deriving the data value
*  The process of deriving an underlying data display value from a display value involves 
*  the following steps:
*  
* - If the formItem 
*    declares a FormItem.parseEditorValue
*    method,
*    
*    it is called
*  
* - Otherwise, if the formItem is of a SimpleType that 
*    declares a SimpleType.parseInput method, it
*    
*    is called
*  
* - If the formItem is of a SimpleType that 
*    SimpleType.inheritsFrom "date", "time" or "datetime", it will 
*    be parsed as a date, time or datetime. Note, this parsing step is applied on top of 
*    custom SimpleType- and FormItem-level parsing
*  
* - If the formItem declares a FormItem.valueMap, a value is derived by 
*  looking up the display value (including the effects of any parsing we may have done so far)
*  in the valueMap
*  
*  Note: Unlike the corollary method FormItem.mapValueToDisplay,
*  there is no special built-in handling of DataSourceField.multiple:true 
*  fields. If you want an array to be parsed out of some user input, you must write the
*  parser method to do so.
* 
* @param {string} display value
*/
mapDisplayToValue(value:string): any;

/**
*  Set the valueMap for this item.
* 
* @param {Array<any> | object} new valueMap
*/
setValueMap(valueMap:Array<any> | object): void;

/**
*  Method to set the FormItem.optionDataSource at runtime
* 
* @param {DataSource} new optionDatasource
*/
setOptionDataSource(dataSource:DataSource): void;

/**
*  Sets the FormItem.valueIcons for this item.
* 
* @param {object} mapping of logical values for this item to icon src SCImgURLs or the special value "blank".
*/
setValueIcons(map:object): void;

/**
*  Getter method to retrieve the FormItem.valueField for this item. For
*  items with a specified FormItem.optionDataSource, this determines which
*  field in that dataSource corresponds to the value for this item.
*  
*  If unset, if a DataSourceField.foreignKey exists
*  between this field and the optionDataSource, this will be used,
*  otherwise default behavior will return the FormItem.name of this field.
* 
* 
*/
getValueFieldName(): string;

/**
*  Returns the displayField for this item.
*  
*  Behavior varies based on the configuration of this item, as follows:
*  
* - If this item has an FormItem.optionDataSource and an explicit 
*  FormItem.foreignDisplayField is specified, this will be returned.
*  
* - Otherwise if an explicit FormItem.displayField is specified it will be returned 
*  by default. If the displayField was specified on the underlying 
*  dataSource field, and no matching field is present in the FormItem.optionDataSource for
*  the item, we avoid returning the specified displayField value and instead return the
*  title field of the option DataSource. We do this to
*  avoid confusion for the case where the displayField is intended as a display-field
*  value for showing another field value within the same record in the underlying
*  dataSource only.
*  
* - If no explicit foreignDisplay or displayField 
*  specification was found, and the FormItem.valueField for this item is hidden in the 
*  FormItem.optionDataSource, this method will return the title field for 
*  the optionDataSource.
* 
* 
*/
getDisplayFieldName(): FieldName;

/**
*  Returns the FormItem.optionDataSource for this item. 
*  
*  Always uses item.optionDataSource if specified. Otherwise, if
*  DataSourceField.foreignKey was specified, uses the target DataSource. Otherwise,
*  uses the DataSource of this item's form (if one is configured). 
* 
* 
*/
getOptionDataSource(): DataSource;

/**
*  Set the value of the form item to the value passed in
*  
*  NOTE: for valueMap'd items, newValue should be data value not displayed value
* 
* @param {any} value to set the element to
*/
setValue(newValue:any): void;

/**
*  If this field has a specified FormItem.optionDataSource, should we perform a fetch against
*  that dataSource to find the record that matches this field's value?
*  
*  If the value is non-null, this method is called when the item is first rendered 
*  or whenever the value is changed via a call to FormItem.setValue. If it returns 
*  true, a fetch will be dispatched against the optionDataSource to get the record
*  matching the value
*  
*  When the fetch completes, if a record was found that matches the
*  data value (and the form item value has not subsequently changed again), 
*  the item will be re-rendered to reflect any changes to the display value,
*  and the record matching the value
*  will be available via FormItem.getSelectedRecord.
*  
*  Default behavior will return false if FormItem.fetchMissingValues is 
*  set to false. Otherwise it will return true if FormItem.alwaysFetchMissingValues is
*  set to true, or if a FormItem.displayField is specified for this item and the item 
*  value is not already present in the item's valueMap.
* 
* 
* @param {any} The new data value of the item.
*/
shouldFetchMissingValue(newValue:any): boolean;

/**
*  If this item has a specified FormItem.displayField, the value displayed to the
*  user for this item may be derived from another field.
*  
*  The display field can be either another field value in the same record or a field that
*  must be retrieved from a related FormItem.optionDataSource if
*  FormItem.fetchMissingValues is true. In this latter case, we perform a fetch against
*  the option dataSource when the item value changes in order to determine the
*  display value to show (and we make the associated record available via
*  FormItem.getSelectedRecord).
*  
*  We cache this data on the form item, so if the item value changes to a new value, then reverts
*  to a previously-seen value, the display value and selected record are already available 
*  without the need for an additional fetch. The cached values will also be kept in synch with
*  the dataSource data assuming it is modified via standard add, update or delete operations.
*  
*  This method explicitly invalidates this cache of optionDataSource data, and if the item value
*  is non null and fetchMissingValues is still true, re-fetches the data.
*  
* 
*/
invalidateDisplayValueCache(): void;

/**
*  Get the record returned from the FormItem.optionDataSource when FormItem.fetchMissingValues
*  is true, and the missing value is fetched.
*  
*  FormItem.fetchMissingValues kicks off the fetch when the form item is initialized
*  with a non null value or when setValue() is called on the item. Note that this method
*  will return null before the fetch completes, or if no record is found in the
*  optionDataSource matching the underlying value.
* 
*/
getSelectedRecord(): ListGridRecord;

/**
*  Notification method fired for FormItem.optionDataSource
*  with FormItem.fetchMissingValues enabled when the
*  FormItem.getSelectedRecord is updated as a result of
*  the value changing or a fetch for a new record completing.
*  Note that a formItem with an optionDataSource may avoid fetching an associated
*  record altogether in some cases. See FormItem.fetchMissingValues and
*  FormItem.alwaysFetchMissingValues. Developers should also be aware that
*  if PickList.fetchDisplayedFieldsOnly is set (or some custom 
*  FormItem.optionOperationId has been specified), the
*  data returned from the server may include only a subset of dataSource fields rather than
*  complete records.
* 
* @param {ListGridRecord} new selected record. May be null if the item has been set to a value with no associated record.
*/
selectedRecordChanged(record:ListGridRecord): void;

/**
*  Clear the value for this form item.
*  
*  Note that if a default value is specified, value will be set to that default value,
*  otherwise value will be cleared, (and removed from the containing form's values).
* 
*/
clearValue(): void;

/**
*  Sets the FormItem.prompt for this item.
* 
* @param {HTMLString} new prompt for the item.
*/
setPrompt(newPrompt:HTMLString): void;

/**
*  Sets the FormItem.hint for this item.
* 
* @param {HTMLString} new hint for the item.
*/
setHint(newHint:HTMLString): void;

/**
*  Set the hintStyle for this item
* 
* @param {CSSStyleName} new style for hint text
*/
setHintStyle(hintStyle:CSSStyleName): void;

/**
*  Store (and optionally show) a value for this form item. 
*  
*  This method will fire standard FormItem.change and
*  DynamicForm.itemChanged handlers, and store the value passed in such that
*  subsequent calls to FormItem.getValue or DynamicForm.getValue will
*  return the new value for this item.
*  
*  This method is intended to provide a way for custom formItems - most commonly 
*  CanvasItem - to provide a new interface to the user, allowing them
*  to manipulate the item's value, for example in an embedded CanvasItem.canvas,
*  or a pop-up dialog launched from an FormItemIcon, etc. Developers
*  should call this method when the user interacts with this custom 
*  interface in order to store the changed value.
*  
*  CanvasItem.shouldSaveValue for CanvasItems is
*  false by default. Custom CanvasItems will need to override shouldSaveValue
*  to true if the values stored via this API should be included in the form's
*  DynamicForm.getValues and saved with the form when
*  DynamicForm.saveData is called.
*  
*  If you cannot easily detect interactions that should change the value as the 
*  user performs them, a workaround is to call
*  storeValue right before the form saves.
*  
*  Note that this method is not designed for customizing a value which is already being
*  saved by a standard user interaction. For example you should not call this method
*  from a FormItem.change. Other APIs such as 
*  FormItem.transformInput exist for this.
*  
* 
* @param {any} value to save for this item
* @param {boolean=} Should the formItem be updated to display the new value?
*/
storeValue(value:any, showValue?:boolean): void;

/**
*  Return the value tracked by this form item.
*  
*  Note that for FormItems that have a ValueMap or where a
*  FormItem.formatValue has been defined, getValue() returns the
*  underlying value of the FormItem, not the displayed value.
* 
* 
*/
getValue(): any;

/**
*  Return the value tracked by this form item as an Integer. If the value cannot
*  be parsed to an int that matches the original value, null will be returned.
* 
* 
*/
getValueAsInteger(): Integer;

/**
*  Manually sets this FormItem to the result of FormItem.formula or FormItem.textFormula.
*  Formulas are normally automatically recomputed and the result automatically applied to the FormItem
*  according to the rules described under FormItem.formula. applyFormula() exists
*  to cover any rare cases where these rules are not correct.
*  
*  Calling applyFormula() has no effect if no FormItem.formula or 
*  FormItem.textFormula is configured for this FormItem.
* 
* 
*/
applyFormula(): void;

/**
*  Is this form item editable or read-only?
*  
*  This setting differs from the enabled/disabled state in that most form items will
*  allow copying of the contents while read-only but do not while disabled.
*  
*  Important note: this method is not intended as an override point to
*  make an item conditionally read-only. It is not called at the appropriate
*  times to serve as a dynamic control over editability. Developers may instead
*  use FormItem.readOnlyWhen to dynamically control editability
*  of items.
* 
* 
*/
getCanEdit(): boolean;

/**
*  Is this form item editable (canEdit:true) or read-only (canEdit:false)?
*  Setting the form item to non-editable causes it to render as read-only,
*  using the appearance specified via FormItem.readOnlyDisplay.
*  
*  The default appearance for canEdit:false items
*  (readOnlyDisplay:"readOnly") differs from the disabled state in that
*  the form item is not rendered with disabled styling and
*  most form items will allow copying of the contents while read-only but do not while
*  disabled.
* 
* 
* @param {boolean} Can this form item be edited?
*/
setCanEdit(canEdit:boolean): void;

/**
*  Does this form item produce an AdvancedCriteria sub criterion object?
*  If this method returns true, DynamicForm.getValuesAsCriteria on the
*  form containing this item will always return an AdvancedCriteria object, calling
*  FormItem.getCriterion
*  
*  on each item to retrieve the individual criteria.
*  
*  Default implementation will return true if FormItem.operator is
*  explicitly specified.
* 
* 
*/
hasAdvancedCriteria(): boolean;

/**
*  When a dynamic form is editing an advanced criteria
*  object via DynamicForm.setValuesAsCriteria, this method is used to determine
*  which sub-criteria apply to which form item(s).
*  
*  This method will be called on each item, and passed the sub-criterion of the
*  AdvancedCriteria object. It should return true if the item can edit the criterion,
*  otherwise false. If it returns true, setValuesAsCriteria() will call 
*  FormItem.setCriterion to actually apply the criterion to the form item, and
*  DynamicForm.getValuesAsCriteria can subsequently retrieve the edited criterion
*  by calling FormItem.getCriterion.
*  
*  Default implementation will return true if the criterion fieldName and
*  operator match the fieldName and operator (or default operator) for
*  this item.
* 
* 
* @param {Criterion} sub-criterion from an AdvancedCriteria object
*/
canEditCriterion(criterion:Criterion): boolean;

/**
*  Override this method if you need to provide a specialized criterion from this formItem
*  when creating an AdvancedCriteria via DynamicForm.getValuesAsCriteria.
*  
*  This API is provided to allow you to specify a more complex criterion than the 
*  "field-operator-value" criterions that are built-in. Note that the built-in behavior is
*  generally quite flexible and powerful enough for most requirements. An example of a case
*  where you might want to override this method is if you wanted to implement a date range 
*  selection (ie, date &gt; x AND date &lt; y) on a form that was combining its other criteria 
*  fields with an "or" operator.
*  
*  Note that this method is part of the criteria editing subsystem: if overridden, it
*  is likely that you will want to also override FormItem.hasAdvancedCriteria to
*  ensure this method is called by the form, and to support editing of existing advanced
*  criteria you may also need to override FormItem.canEditCriterion and 
*  FormItem.setCriterion.
*  
*  The default implementation will return a criterion including the form item value, fieldName
*  and specified FormItem.operator, or a default operator derived from the
*  form item data type if no explicit operator is specified.
* 
* 
* @param {TextMatchStyle=} If passed assume the textMatchStyle  will be used when performing a fetch operation with these criteria. This may impact  the criterion's operator property.
*/
getCriterion(textMatchStyle?:TextMatchStyle): Criterion;

/**
*  Update this form item to reflect a criterion object from within an AdvancedCriteria.
*  Called by DynamicForm.setValuesAsCriteria when FormItem.canEditCriterion
*  returns true for this item.
*  
*  Default implementation simply calls FormItem.setValue with the value
*  of the criterion passed in
* 
* @param {Criterion} criterion to edit
*/
setCriterion(criterion:Criterion): void;

/**
*  Is the value clipped?
*  
*  The form item must have value clipping enabled. If a form item type supports the
*  clipValue attribute, then clipValue must be true. TextItems and derivatives
*  (e.g. SpinnerItem) automatically clip their values.
* 
* 
*/
valueClipped(): boolean;

/**
*  	Clear all error messages for this item
* 
*/
clearErrors(): void;

/**
*  Set the error message(s) for this item
* 
* @param {string | Array<Partial<string>>} error message(s)
*/
setErrors(errors:string | Array<Partial<string>>): void;

/**
*  Return whether this item currently has any validation errors as
*   a result of a previous validation pass.
* 
*/
hasErrors(): boolean;

/**
*  Validate this item.
*  
* 
*/
validate(): boolean;

/**
*  Setter to mark this formItem as FormItem.required, or not required at runtime.
*  Note that an alternative approach to updating the required flag directly
*  would be to simply use a ValidatorType type validator.
*  
*  Note that this method will not re-validate this item by default or clear any 
*  existing validation errors. If desired, this may be achieved by calling
*  FormItem.validate or DynamicForm.clearErrors.
* 
* @param {boolean} new FormItem.required value.
*/
setRequired(required:boolean): void;

/**
*  For text-based items, this method returns the index of the start of the current
*  selection if the item currently has the focus (if no text is selected, this equates to
*  the current position of the text editing cursor). See TextItem.getSelectionRange
*  for details of what is returned if the item does not have the focus (note, it is 
*  important to read this documentation, because the behavior when the item does not have
*  the focus varies by browser)
* 
*/
getCursorPosition(): Integer;

/**
*  This method is fired when the user rolls off this item (or the title for this item) and
*  will clear any hover canvas shown by the item.
* 
*/
stopHover(): void;

/**
*  Optional stringMethod to fire when the user hovers over this item.
*  Return false to suppress default behavior of showing a hover canvas containing the
*  HTML returned by formItem.itemHoverHTML() / 
*  form.itemHoverHTML().
* 
* 
* @param {FormItem} Pointer to this item
* @param {DynamicForm} This items form
*/
itemHover(item:FormItem, form:DynamicForm): void;

/**
*  Optional stringMethod to fire when the user hovers over this item's title.
*  Return false to suppress default behavior of showing a hover canvas containing the
*  HTML returned by formItem.titleHoverHTML() / 
*  form.titleHoverHTML().
* 
* 
* @param {FormItem} Pointer to this item
* @param {DynamicForm} This items form
*/
titleHover(item:FormItem, form:DynamicForm): void;

/**
*  Optional stringMethod to fire when the user hovers over this item's value.
*  Return false to suppress default behavior of showing a hover canvas containing the
*  HTML returned by FormItem.valueHoverHTML / DynamicForm.valueHoverHTML.
* 
* 
* @param {FormItem} Pointer to this item
* @param {DynamicForm} This items form
*/
valueHover(item:FormItem, form:DynamicForm): void;

/**
*  If defined, this method should return the HTML to display in a hover canvas when the 
*  user holds the mousepointer over this item's title. Return null to suppress the hover 
*  canvas altogether.
*  
*  If not defined, DynamicForm.titleHoverHTML will be evaluated to determine
*  hover content instead.
* 
* 
* @param {FormItem} Pointer to this item
* @param {DynamicForm} This items form
*/
titleHoverHTML(item:FormItem, form:DynamicForm): HTMLString;

/**
*  If defined, this method should return the HTML to display in a hover canvas when the
*  user holds the mousepointer over this item's value. Return null to suppress the hover
*  canvas altogether.
*  
*  If not defined, DynamicForm.valueHoverHTML will be evaluated to determine
*  hover content instead.
* 
* 
* @param {FormItem} Pointer to this item
* @param {DynamicForm} This items form
*/
valueHoverHTML(item:FormItem, form:DynamicForm): HTMLString;

/**
*  If defined, this method should return the HTML to display in a hover canvas when the 
*  user holds the mousepointer over this item. Return null to suppress the hover 
*  canvas altogether.
*  
*  If not defined, dynamicForm.itemHoverHTML() will be evaluated to 
*  determine hover content instead.
* 
* 
* @param {FormItem} Pointer to this item
* @param {DynamicForm} This items form
*/
itemHoverHTML(item:FormItem, form:DynamicForm): HTMLString;

/**
*  Setter for FormItem.tabIndex.
* 
* @param {Integer} new tabIndex for the item
*/
setTabIndex(tabIndex:Integer): void;

/**
*  Set this item to be enabled or disabled at runtime. 
* 
* @param {boolean} true if this item should be disabled
*/
setDisabled(disabled:boolean): void;

/**
*  Setter method for FormItem.showDisabled
* 
* @param {boolean} new showDisabled setting
*/
setShowDisabled(showDisabled:boolean): void;

/**
*  Is this item disabled?
* 
*/
isDisabled(): boolean;

/**
*  Set this item to be enabled at runtime.
* 
*/
enable(): void;

/**
*  Set this item to be disabled at runtime.
* 
*/
disable(): void;

/**
*  Returns true for items that can accept keyboard focus such as data items 
*  (TextItem, TextAreaItem, etc), 
*  CanvasItem with a focusable canvas, or items where FormItem.canFocus
*  was explicitly set to true.
*  
* 
*/
getCanFocus(): boolean;

/**
*  	Move the keyboard focus into this item's focusable element
* 
*/
focusInItem(): void;

/**
*  	Takes focus from this form item's focusable element.
* 
*/
blurItem(): void;

/**
*  Returns true if this formItem has the keyboard focus. Note that in Internet Explorer
*  focus notifications can be asynchronous (see EventHandler.synchronousFocusNotifications).
*  In this case, this method can correctly 
*  return false when, intuitively, you would expect it to return true:
*  
*    someItem.focusInItem();
*    if (someItem.isFocused()) {
*      // In most browsers we would get here, but not in Internet Explorer with
*      // EventHandler.synchronousFocusNotifications disabled
*    }
*  
* 
* 
*/
isFocused(): boolean;

/**
*  Shifts focus to the next focusable element after this item, skipping any elements
*  nested inside the tabbing group for this item, such as sub-elements, nested canvases
*  in a CanvasItem, or icons.
*  
*  This method makes use of the TabIndexManager.shiftFocusAfterGroup method to request
*  focus be changed to the next registered entry. By default standard focusable 
*  SmartClient UI elements, including Canvases, FormItems, FormItemIcons, etc are
*  registered with the TabIndexManager in the appropriate order, and will accept focus
*  if Canvas.canFocus, and not FormItem.disabled or
*  Canvas.showClickMask. 
*  
*  Canvases support a similar method: Canvas.focusAfterGroup.
*  
*  NOTE: Focusable elements created directly in the raw HTML bootstrap or 
*  by application code will not be passed focus by this method unless they have also been
*  explicitly registered with the TabIndexManager. See the tabOrderOverview
*  for more information.
* 
* @param {boolean} direction to shift focus - true to move forward, false to move     backwards (as with a shift+tab interaction).
*/
focusAfterItem(forward:boolean): void;

/**
*  Is the user performing a native "cut" event to modify the value of a freeform text
*  field? This method may be invoked during change notification flow methods including 
*  FormItem.change, FormItem.changed and
*  FormItem.transformInput. See FormItem.supportsCutPasteEvents.
* 
*/
isCutEvent(): boolean;

/**
*  Is the user performing a native "paste" event to modify the value of a freeform text
*  field? This method may be invoked during change notification flow methods including 
*  FormItem.change, FormItem.changed and
*  FormItem.transformInput. See FormItem.supportsCutPasteEvents.
* 
*/
isPasteEvent(): boolean;

/**
*  Should some keypress event on this item be prevented from bubbling (such that
*  the containing form and ancestors do not receive the event).
*  
*  This method is called after standard item keypress handlers when the user presses
*  a key while focused in this item. Returning true will suppress bubbling of the event
*  to the containing form. This is useful to avoid having the form react to key
*  events which "have meaning" to the focused item.
*  
*  Default implementation varies by item type. In short character keys 
*  are suppressed for all editable fields, as are keys which will modify the 
*  current state of the field
*  ("Arrow" keys for moving around free form text editors, etc).
*  
*  
*  Developers may override this method to allow the form to react to certain keypresses,
*  for example allowing scrolling of the form when the user presses the arrow keys, 
*  but they should be aware that doing so could lead to confusing user experience
*  if the keypress will also move the position of the caret within a text box (say).
*  
*  Note that when this method returns true, no Canvas.keyPress event
*  will fire on the form for the key pressed. However developers will still receive the
*  separate DynamicForm.itemKeyPress event.
* 
* @param {KeyName} name of the key pressed
* @param {number} If this was a character key, this is the numeric value    for the character
*/
shouldStopKeyPressBubbling(keyName:KeyName, characterValue:number): boolean;

/**
*  Returns the left coordinate of this form item in pixels. Note that this method
*  is only reliable after the item has been drawn.
* 
*/
getLeft(): Integer;

/**
*  Returns the top coordinate of the form item in pixels. Note that this method is only 
*  reliable after the item has been drawn out.
* 
*/
getTop(): Integer;

/**
*  Returns the drawn page-left coordinate of this form item in pixels.
* 
*/
getPageLeft(): Integer;

/**
*  Returns the drawn page-top coordinate of this form item in pixels.
* 
*/
getPageTop(): Integer;

/**
*  Return the value tracked by this form item as a Float. If the value cannot
*  be parsed to a valid float, null will be returned.
* 
* 
*/
getValueAsFloat(): Float;

/**
*  Expression that's evaluated to see if an item should be dynamically hidden.
*  
*  showIf() is evaluated whenever the form draws or redraws.
*  
*  Note that explicit calls to FormItem.show or FormItem.hide will
*  will wipe out the showIf expression.
*  
*  Alternatively, you can use Criteria to declare when a FormItem is
*  visible via FormItem.visibleWhen.
* 
* 
* @param {FormItem} the form item itself (also available as "this")
* @param {any} current value of the form item
* @param {DynamicForm} the managing DynamicForm instance
* @param {object} the current set of values for the form as a whole
*/
showIf(item:FormItem, value:any, form:DynamicForm, values:object): boolean;

/**
*  Expression evaluated to determine the FormItem.defaultValue when no value is 
*  provided for this item.
*  
*  formItem.defaultDynamicValue may be specified as a method or
*  StringMethod to evaluate. It should return a calculated
*  default value for the item.
*  
*  If you don't need dynamic evaluation, you can just use item.defaultValue.
* 
* 
* @param {FormItem} the form item itself (also available as "this")
* @param {DynamicForm} the managing DynamicForm instance
* @param {object} the current set of values for the form as a whole
*/
defaultDynamicValue(item:FormItem, form:DynamicForm, values:object): any;

/**
*  Called when this FormItem receives focus.
*  
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the form item itself (also available as "this")
*/
focus(form:DynamicForm, item:FormItem): void;

/**
*  Called when this FormItem loses focus.
*  
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the form item itself (also available as "this")
*/
blur(form:DynamicForm, item:FormItem): void;

/**
*  Notification method fired when the user enters this formItem.
*  Differs from FormItem.focus in that while focus and blur
*  may fire multiple as the user navigates sub elements of an item (such as interacting
*  with a pick list), editorEnter will typically fire once when the user 
*  starts to edit this item as a whole, and once when the user moves onto a different
*  item or component
* 
* @param {DynamicForm} form containing this item
* @param {FormItem} form item recieving focus
* @param {any} current item value.
*/
editorEnter(form:DynamicForm, item:FormItem, value:any): void;

/**
*  Notification method fired when the user leaves this formItem.
*  Differs from FormItem.blur in that while focus and blur
*  may fire multiple as the user navigates sub elements of an item (such as interacting
*  with a pick list), editorEnter will typically fire once when the user 
*  starts to edit this item as a whole, and editorExit fires once when the 
*  user moves onto a different item or component
* 
* @param {DynamicForm} form managing this form item
* @param {FormItem} pointer to the form item being managed
* @param {any} current value of the form item
*/
editorExit(form:DynamicForm, item:FormItem, value:any): void;

/**
*  Called when this FormItem is clicked on.
*  
*  Note: click() is available on StaticTextItem, BlurbItems, ButtonItem, and
*  derivatives. Other form items (such as HiddenItem) do not support click().
* 
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the form item itself (also available as "this")
*/
click(form:DynamicForm, item:FormItem): boolean;

/**
*  Called when this FormItem is double-clicked.
* 
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the form item itself (also available as "this")
*/
doubleClick(form:DynamicForm, item:FormItem): boolean;

/**
*  Called when the mouse is right-clicked anywhere in this formItem. If the implementation
*  returns false, default browser behavior is cancelled.
*  
*  Note that it can be bad practice to cancel this method if the mouse is over the data 
*  element for the item, because doing so would replace the builtin browser-default menus 
*  that users may expect. You can use DynamicForm.getEventItemInfo to return an 
*  FormItemEventInfo that can be used to determine which part of the 
*  item is under the mouse.
* 
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the form item itself (also available as "this")
*/
showContextMenu(form:DynamicForm, item:FormItem): boolean;

/**
*  Notification method called when the FormItem.showPickerIcon is clicked.
*  This method will fire after the FormItemIcon.click handler for the
*  FormItem.pickerIconProperties. If the event is not cancelled, 
*  the standard FormItem.iconClick will also fire.
*  
*  The default implementation will call FormItem.showPicker.
* 
* 
* @param {DynamicForm} the DynamicForm containing the picker icon's item.
* @param {FormItem} the FormItem containing the picker icon.
* @param {FormItemIcon} the picker icon.
*/
pickerIconClick(form:DynamicForm, item:FormItem, pickerIcon:FormItemIcon): boolean;

/**
*  Notification method called when the user clicks on a form item icon.
*  
*  The icon's FormItemIcon.click method if any is called first. Then, if the clicked
*  icon is the FormItem.showPickerIcon, the FormItem.pickerIconClick method is
*  called. Then, this method is called.
*  
*  This event may be cancelled by returning false to 
*  suppress the FormItem.click handler from also firing in response to the
*  user interaction.
*  
* 
* @param {DynamicForm} a pointer to this item's form
* @param {FormItem} a pointer to this form item
* @param {FormItemIcon} a pointer to the icon that received the click event.
*/
iconClick(form:DynamicForm, item:FormItem, icon:FormItemIcon): boolean;

/**
*    StringMethod.
*    Default action to fire when an icon has keyboard focus and the user types a key.
*    May be overridden by setting keyPress on the form item icon directly.
* 
* @param {KeyName} name of the key pressed
* @param {string} character produced by the keypress
* @param {DynamicForm} a pointer to this item's form
* @param {FormItem} a pointer to this form item
* @param {FormItemIcon} a pointer to the icon that received the click event.
*/
iconKeyPress(keyName:KeyName, character:string, form:DynamicForm, item:FormItem, icon:FormItemIcon): void;

/**
*  Called when a FormItem's value is about to change as the result of user interaction. This
*  method fires after the user performed an action that would change the value of this field,
*  but before the element itself is changed. 
*  
*  Returning false cancels the change. Note that if what you want to do is
*  transform the user's input, for example, automatically change separator
*  characters to a standard separator character, you should implement
*  FormItem.transformInput rather than using a combination of
*  change() and setValue() to accomplish the same thing. Returning false from
*  change is intended for rejecting input entirely, such as typing invalid
*  characters.
*  
*  Note that if you ask the form for the current value in this handler, you will get the old
*  value because the change has not yet been committed. The new value is available as a
*  parameter to this method.
*  
*  Change/Changed notifications vs "...When" rules: the change and
*  changed events only fire when an end user modifies a field value. 
*  If you are trying to dynamically control the visibility or enabled state of other 
*  components in response to these events, consider instead using properties 
*  such as Canvas.visibleWhen, FormItem.readOnlyWhen,
*  Canvas.enableWhen on the target component. (Similar properties are 
*  available on FormItem, Canvas, MenuItem and other components).
* 
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the form item itself (also available as "this")
* @param {any} The new value of the form item
* @param {any} The previous value of the form item
*/
change(form:DynamicForm, item:FormItem, value:any, oldValue:any): boolean;

/**
*  Called when a FormItem's value has been changed as the result of user interaction. This
*  method fires after the newly specified value has been stored.
*  
*  Change/Changed notifications vs "...When" rules: the change and
*  changed events only fire when an end user modifies a field value. 
*  If you are trying to dynamically control the visibility or enabled state of other 
*  components in response to these events, consider instead using properties 
*  such as Canvas.visibleWhen, FormItem.readOnlyWhen,
*  Canvas.enableWhen on the target component. (Similar properties are 
*  available on FormItem, Canvas, MenuItem and other components).
* 
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the form item itself (also available as "this")
* @param {any} The current value (after the change).
*/
changed(form:DynamicForm, item:FormItem, value:any): void;

/**
*  Notification method called when FormItem.showPending is enabled and this
*  form item should either clear or show its "Pending" visual state.
*  
*  The default behavior is that the FormItem.titleStyle and
*  FormItem.cellStyle are updated to include/exclude the "Pending" suffix.
*  Standard form item types may implement additional default behavior (see any item-specific
*  pendingStatusChanged() documentation). Returning false will cancel the default
*  behavior.
*  
*  The pendingStatusChanged() notification method is typically used by CanvasItem-derived
*  form items, where a custom or supplemental pending visual state is desired.
* 
* @param {DynamicForm} the managing DynamicForm instance.
* @param {FormItem} the form item itself (also available as "this").
* @param {boolean} true if the item should show its pending visual state; false otherwise.
* @param {any} the current form item value.
* @param {any} the value that would be restored by a call to DynamicForm.resetValues.
*/
pendingStatusChanged(form:DynamicForm, item:FormItem, pendingStatus:boolean, newValue:any, value:any): boolean;

/**
*  Called when a FormItem's value is about to change as the result of user interaction. This
*  method fires after the user performed an action that would change the value of this field,
*  and allows the developer to modify / reformat the value before it gets validated / saved.
*  Fires before FormItem.change.
*  
*  Return the reformatted value.
* 
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the form item itself (also available as "this")
* @param {any} The new value of the form item
* @param {any} The previous (current) value of the form item
*/
transformInput(form:DynamicForm, item:FormItem, value:any, oldValue:any): any;

/**
*  Notification method fired when the user clicks the title for this item
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the form item whose title was clicked
*/
titleClick(form:DynamicForm, item:FormItem): boolean;

/**
*  Notification method fired when the user double-clicks the title for this item
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the form item whose title was double-clicked
*/
titleDoubleClick(form:DynamicForm, item:FormItem): boolean;

/**
*  StringMethod fired when the user presses a key while focused in this form item.
* 
* 
* @param {FormItem} Item over which the keypress occurred
* @param {DynamicForm} Pointer to the item's form
* @param {KeyName} Name of the key pressed (Example: "A", "Enter")
* @param {number} If this was a character key, this is the numeric value    for the character
*/
keyPress(item:FormItem, form:DynamicForm, keyName:KeyName, characterValue:number): boolean;

/**
*  StringMethod fired in response to a keydown while focused in this form item.
* 
* 
* @param {FormItem} Item over which the keydown occurred
* @param {DynamicForm} Pointer to the item's form
* @param {KeyName} Name of the key pressed (Example: "A", "Enter")
*/
keyDown(item:FormItem, form:DynamicForm, keyName:KeyName): boolean;

/**
*  StringMethod fired in response to a keyup while focused in this form item.
* 
* 
* @param {FormItem} Item over which the keyup occurred
* @param {DynamicForm} Pointer to the item's form
* @param {KeyName} Name of the key pressed (Example: "A", "Enter")
*/
keyUp(item:FormItem, form:DynamicForm, keyName:KeyName): boolean;

/**
*  Except when printing and FormItem.getPrintValueIcon
*  is implemented, implementing this stringMethod allows the developer to specify the image
*  source for an icon to be displayed for the current form item value.
*  
*  The special value "blank" means that no image will be displayed. This is typically used
*  in conjunction with FormItem.getValueIconStyle to implement spriting of the
*  value icon. Note that when spriting the value icon, it is recommended to implement
*  getPrintValueIcon() and FormItem.getPrintValueIconStyle
*  when printing.
*  
*  Takes precedence over FormItem.valueIcons
*  
*  The returned SCImgURL, if not null or "blank", will be suffixed with
*  FormItem.imageURLSuffix.
* 
* 
* @param {any} value of the item.
*/
getValueIcon(value:any): SCImgURL;

/**
*  If implemented, this stringMethod is called when printing to
*  obtain the image source for an icon to be displayed for the current form item value.
*  The special value "blank" means that no image will be displayed.
*  
*  Implementing getPrintValueIcon() may be useful in order to swap out the
*  value icon with a more printer-friendly image (perhaps a grayscale-only image). Another
*  use is to avoid spriting when printing. Value icon spriting may be problematic when
*  printing because browsers typically default to not printing background images.
*  
*  Takes precedence over FormItem.valueIcons and FormItem.getValueIcon
*  when printing.
*  
*  The returned SCImgURL, if not null or "blank", will be suffixed with
*  FormItem.imageURLSuffix.
* 
* 
* @param {any} value of the item.
*/
getPrintValueIcon(value:any): SCImgURL;



}
/**
* Creates a new FormItem
* 
* @param typeCheckedProperties {Partial<FormItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormItem} The new instance
*/
class FormItem{
static create(typeCheckedProperties? : Partial<FormItem>, uncheckedProperties?: Object): FormItem;
/**
* Default FormItemIcon.hspace value for pickers
*  created by FormItem.getPickerIcon.
* @type {Integer}
* @default "0"
*/
static defaultPickerIconSpace?: Integer;

/**
* Default FormItemIcon.height value for pickers
*  created by FormItem.getPickerIcon.
* @type {Integer}
* @default "22"
*/
static defaultPickerIconHeight?: Integer;





/**
*  Returns a FormItemIcon for a standard picker with skin-specific
*  settings.
* 
* @param {PickerIconName} Name of picker icon
* @param {FormItemIcon=} Properties to apply to new picker icon
*/
static getPickerIcon(pickerName:PickerIconName, properties?:FormItemIcon): FormItemIcon;




}

/**
* EditProxy that handles FormItems when editMode is enabled.
*/
interface FormItemEditProxy extends EditProxy {
/**
* If InlineEditEvent for this FormItem edits the
*  FormItem.valueMap, character that should be used as a separator between
*  values, or between pairs of stored vs display values if the user is entering such
*  a ValueMap using the FormItemEditProxy.valueMapDisplaySeparatorChar.
*  
*  If EditProxy.inlineEditMultiline is enabled, newlines will be used as value
*  separators and the valueMapSeparatorChar
*  
*  The FormItemEditProxy.valueMapEscapeChar can be used to enter the separator
*  char as part of a valueMap value.
* @type {string}
* @default ","
*/
valueMapSeparatorChar?: string;

/**
* If InlineEditEvent for this FormItem edits the
*  FormItem.valueMap, character that should be used as a separator for
*  entering ValueMaps that map from a stored value to a user-displayed value.
*  
*  With the default of ":", the following input:
*  
*    1:Fixed, 2:Won't Fix, 3:Resolved
*  
*  Would be assumed to be a mapping like this (expressed in JSON):
*  
*   {
*    "1" : "Fixed",
*    "2" : "Won't Fix",
*    "3" : "Resolved"
*   }
*  
*  
*  If the input has inconsistent use of the separator char, the input will be assumed
*  to be stored-to-displayed mapping if the separator char is present in a majority
*  of values, and any values that lack a separator will use the same value for
*  storage and display. For example, for this input:
*  
*     Fixed:Reported Fixed, WontFix:Won't Fix, Resolved
*  
*  The resulting valueMap would be:
*  
*   {
*    "Fixed" : "Reported Fixed",
*    "WontFix" : "Won't Fix",
*    "Resolved" : "Resolved"
*   }
*  
*  
*  The FormItemEditProxy.valueMapEscapeChar can be used to enter literal colon characters.
*  
*  Set valueMapDisplaySeparatorChar to null to prevent entry of stored
*  vs displayed values - user input will always be treated as just a list of legal
*  values.
* @type {string}
* @default ":"
*/
valueMapDisplaySeparatorChar?: string;

/**
* If InlineEditEvent for this FormItem edits the
*  FormItem.valueMap, character that can be used to mark the default selected
*  option. Can appear before or after a value, for example, with this input:
*  
*    Fixed,Won't Fix,Resolved*
*  
*  "Resolved" would be the default selected value.
*  
*  If multiple values are marked selected for a SelectItem,
*  SelectItem.multiple will automatically be set.
*  
*  The FormItemEditProxy.valueMapEscapeChar can be used to allow the
*  valueMapSelectedChar to appear at the beginning or end of a 
*  valueMap value.
* @type {string}
* @default "*"
*/
valueMapSelectedChar?: string;

/**
* If InlineEditEvent for this FormItem edits the
*  FormItem.valueMap, character that can be used to enter literal separator
*  chars (such as the FormItemEditProxy.valueMapSeparatorChar) or literal
*  leading or trailing whitespace.
*  
*  Repeat this character twice to enter it literally. For example, with the default
*  of "\", inputting "\\" would result in a literal backslash in the value.
* @type {string}
* @default "\"
*/
valueMapEscapeChar?: string;





}
/**
* Creates a new FormItemEditProxy
* 
* @param typeCheckedProperties {Partial<FormItemEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormItemEditProxy} The new instance
*/
class FormItemEditProxy{
static create(typeCheckedProperties? : Partial<FormItemEditProxy>, uncheckedProperties?: Object): FormItemEditProxy;







}

/**
* Revert unsaved changes in a form.
*/
interface FormResetValuesTask extends ComponentTask {




}
/**
* Creates a new FormResetValuesTask
* 
* @param typeCheckedProperties {Partial<FormResetValuesTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormResetValuesTask} The new instance
*/
class FormResetValuesTask{
static create(typeCheckedProperties? : Partial<FormResetValuesTask>, uncheckedProperties?: Object): FormResetValuesTask;







}

/**
* Saves changes made in a form (validates first).
*/
interface FormSaveDataTask extends ComponentTask {
/**
* Should a notification message (defined as notifyMessage) be shown after save completes
*  successfully?
* @type {boolean}
* @default null
*/
showNotification?: boolean;

/**
* The message to be shown when save completes if FormSaveDataTask.showNotification is set. If
*  no message is configured a default message is used based on the whether the form
*  is bound to a DataSource or not: FormSaveDataTask.boundNotifyMessage or FormSaveDataTask.unboundNotifyMessage.
* @type {string}
* @default null
*/
notifyMessage?: string;

/**
* The default message to be shown when a target form is bound to a DataSource.
* @type {string}
* @default "${dsTitle} saved successfully"
*/
boundNotifyMessage?: string;

/**
* The default message to be shown when a target form is not bound to a DataSource.
* @type {string}
* @default "Saved successfully"
*/
unboundNotifyMessage?: string;

/**
* Where to show the message, specified as an edge ("T", "B", "R", "L") similar to
*  Canvas.snapTo, or "C" for center. The message will be shown at the center of the
*  edge specified (or the very center for "C").
* @type {string}
* @default "T"
*/
notifyPosition?: string;

/**
* NotifyType for FormSaveDataTask.notifyMessage.
* @type {NotifyType}
* @default "message"
*/
notifyType?: NotifyType;

/**
* The form will be DynamicForm.clearValues after a successful save unless
*  this property is set to false.
* @type {boolean}
* @default true
*/
clearAfterSave?: boolean;

/**
* ID of the next sequence or element to proceed to if a failure condition arises
*  from operation.
* @type {string}
* @default null
*/
failureElement?: string;

/**
* Additional properties to set on the DSRequest that will be issued
*  to perform save.
*  
*  Note that willHandleError will always be set true.
* @type {DSRequest}
* @default null
*/
requestProperties?: DSRequest;

/**
* Does this processElement pass through output from the last executed task
*  (i.e. transient state)?
*  
*  See taskInputExpression for details on the
*  transient state outputs.
*  
*  Note that this property does not affect the task at all but is an indicator
*  to the user and to the workflow editor of the behavior of the task as coded
*  (See Process.passThruTaskOutput).
* @type {boolean}
* @default false
*/
passThruOutput?: boolean;





}
/**
* Creates a new FormSaveDataTask
* 
* @param typeCheckedProperties {Partial<FormSaveDataTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormSaveDataTask} The new instance
*/
class FormSaveDataTask{
static create(typeCheckedProperties? : Partial<FormSaveDataTask>, uncheckedProperties?: Object): FormSaveDataTask;







}

/**
* Put a value in just one field of a form.
*/
interface FormSetFieldValueTask extends ComponentTask {
/**
* Field to assign new value.
* @type {FieldName}
* @default null
*/
targetField?: FieldName;

/**
* Value to assign to FormSetFieldValueTask.targetField.
* @type {any}
* @default null
*/
value?: any;





}
/**
* Creates a new FormSetFieldValueTask
* 
* @param typeCheckedProperties {Partial<FormSetFieldValueTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormSetFieldValueTask} The new instance
*/
class FormSetFieldValueTask{
static create(typeCheckedProperties? : Partial<FormSetFieldValueTask>, uncheckedProperties?: Object): FormSetFieldValueTask;







}

/**
* Set form values.
*/
interface FormSetValuesTask extends ComponentTask {
/**
* Values to be set on the form.
*  
*  Data values prefixed with "$" will be treated as a
*  taskInputExpression. Use FormSetValuesTask.fixedValues for any values that start with
*  "$" but should be treated as a literal.
* @type {Record}
* @default null
*/
values?: Record;

/**
* Values to be combined with the data from the ServiceTask.values if specified,
*  via simple copying of fields, with explicitly specified ServiceTask.values
*  overriding fixedValues.
* @type {Record}
* @default null
*/
fixedValues?: Record;





}
/**
* Creates a new FormSetValuesTask
* 
* @param typeCheckedProperties {Partial<FormSetValuesTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormSetValuesTask} The new instance
*/
class FormSetValuesTask{
static create(typeCheckedProperties? : Partial<FormSetValuesTask>, uncheckedProperties?: Object): FormSetValuesTask;







}

/**
* Validate a form and show errors to user.
*/
interface FormValidateValuesTask extends ComponentTask {
/**
* Does this processElement pass through output from the last executed task
*  (i.e. transient state)?
*  
*  See taskInputExpression for details on the
*  transient state outputs.
*  
*  Note that this property does not affect the task at all but is an indicator
*  to the user and to the workflow editor of the behavior of the task as coded
*  (See Process.passThruTaskOutput).
* @type {boolean}
* @default false
*/
passThruOutput?: boolean;





}
/**
* Creates a new FormValidateValuesTask
* 
* @param typeCheckedProperties {Partial<FormValidateValuesTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormValidateValuesTask} The new instance
*/
class FormValidateValuesTask{
static create(typeCheckedProperties? : Partial<FormValidateValuesTask>, uncheckedProperties?: Object): FormValidateValuesTask;







}

/**
* Shows an interface allowing a user to enter simple formulas by typing them into a text
*  field.
*  
*  Available values for the formula are determined by the DataSource fields, and are given
*  simple single-letter aliases (such as "A", "B", ...) similar to column names in Excel.
*  The set of available values is shown in the FormulaBuilder.fieldKey as a simple
*  mapping between the DataSourceField.title and it's short name.
*  
*  If FormulaBuilder.targetRuleScope is specified the formula will use full field
*  path names instead of single-letter aliases and the resulting formula will not include
*  the formulaVars property.
*  
*  By default, available math functions are shown in a hover from the
*  FormulaBuilder.helpIcon that appears after the formula field.
*/
interface FormulaBuilder extends VLayout {
/**
* DataSource providing the available fields for the formulaBuilder.
*  
*  By default the formulaBuilder will include only fields of numeric type or derived
*  from a numeric type. Set FormulaBuilder.fields to override this.
* @type {DataSource | string}
* @default null
*/
dataSource?: DataSource | string;

/**
* Canvas.ruleScope providing the list of available fields for the formulaBuilder.
*  
*  By default the formulaBuilder will include only fields of numeric type or derived
*  from a numeric type. Set FormulaBuilder.fields to override this.
*  
* @type {string | Canvas}
* @default null
*/
targetRuleScope?: string | Canvas;

/**
* Set this to override the underlying set of available fields.
* @type {Array<Partial<string>>}
* @default null
*/
fields?: Array<Partial<string>>;

/**
* The Field object representing the field being created or edited.
* @type {string}
* @default null
*/
field?: string;

/**
* Are we editing an existing field?
* @type {boolean}
* @default false
*/
editMode?: boolean;

/**
* TextItem that users type into when entering a formula.
* @type {TextItem}
* @default null
*/
formulaField?: TextItem;

/**
* TextItem that allows users to set the title for this field.
* @type {TextItem}
* @default null
*/
titleField?: TextItem;

/**
* Whether to show the help icon that appears after the FormulaBuilder.formulaField.
* @type {boolean}
* @default true
*/
showHelpIcon?: boolean;

/**
* Icon that appears after the FormulaBuilder.formulaField, showing help on hover.
* @type {FormItemIcon}
* @default null
*/
helpIcon?: FormItemIcon;

/**
* Text label for the checkbox that allows the user to automatically hide the
*  fields used in the formula.
* @type {string}
* @default "Auto hide fields used in formula"
*/
autoHideCheckBoxLabel?: string;

/**
* Whether to show a checkbox offering the user the ability to automatically
*  hide any fields involved in the formula.
* @type {boolean}
* @default true
*/
showAutoHideCheckBox?: boolean;

/**
* CheckBox that, when selected, hides columns in the component that are used in this formula.
* @type {TextItem}
* @default null
*/
autoHideCheckBox?: TextItem;

/**
* Indicates whether to use "formula" or some other keyword in various captions and text
* @type {string}
* @default "Formula"
*/
builderTypeText?: string;

/**
* The list of math functions available in this FormulaBuilder, as an array of 
*  MathFunction names.
*  
*  The following function list is supported in FormulaBuilders by default: min(), max(), 
*  round(), ceil(), floor(), abs(), pow(), sin(), cos(), tan(), ln() and log().
* @type {Array<Partial<string>>}
* @default null
*/
mathFunctions?: Array<Partial<string>>;

/**
* Was the builder operation cancelled? Set to true when the user cancels with the cancel
*  button or the dialog's close-button.
* @type {boolean}
* @default false
*/
cancelled?: boolean;

/**
* The prefix to apply to the variable that is inserted in response to a record click in the
*  grid that shows the list of available fields.
* @type {string}
* @default null
*/
valuePrefix?: string;

/**
* The suffix to apply to the variable that is inserted in response to a record click in the
*  grid that shows the list of available fields.
* @type {string}
* @default null
*/
valueSuffix?: string;

/**
* The prompt to show when mouse hovers over a field supplied by 
*  a nearby component through FormulaBuilder.targetRuleScope.
*  
*  The prompt is a dynamic string and is formatted like Label.dynamicContents
*  where the following variables are available:
*  
*  
* - fieldName
*  
* - componentName
*  
* @type {string}
* @default "This value comes from the '${fieldName}' field in the nearby component [${componentName}]"
*/
nearbyComponentFieldPrompt?: string;

/**
* The prompt to show when mouse hovers over a field supplied by 
*  the current component through FormulaBuilder.targetRuleScope.
*  
*  The prompt is a dynamic string and is formatted like Label.dynamicContents
*  where the following variables are available:
*  
*  
* - fieldName
*  
* - componentName
*  
* @type {string}
* @default "This value comes from the '${fieldName}' field in the current component [${componentName}]"
*/
currentComponentFieldPrompt?: string;

/**
* The prompt to show when mouse hovers over a field supplied by 
*  a DataSource through FormulaBuilder.targetRuleScope.
*  
*  The prompt is a dynamic string and is formatted like Label.dynamicContents
*  where the following variables are available:
*  
*  
* - fieldName
*  
* - dataSource
*  
* @type {string}
* @default "This value comes from the '${fieldName}' field in any nearby component that uses DataSource '${dataSource}'. Values that are edited by end users are preferred."
*/
dataSourceFieldPrompt?: string;

/**
* ListGrid displaying the list of available fields and their corresponding formula keys.
* @type {ListGrid}
* @default null
*/
fieldKey?: ListGrid;

/**
* The text to display as a preamble to the instruction text that appears in the 
*  FormulaBuilder.instructions.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  Only one dynamic variable, builderType, is available and represents the type of this
*  builder - either Formula or Summary.
*  
*  Default value returns 
*  
*  The following fields are available for use in this [Formula/Summary]
*  
* @type {HTMLString}
* @default "The following fields are available for use in this ${builderType}.  Click any field to have it added to the ${builderType}."
*/
instructionsTextStart?: HTMLString;

/**
* Label displaying the instruction text above the fieldKey grid.
* @type {Label}
* @default null
*/
instructions?: Label;

/**
* Label used for displaying messages related to the validity of the current formula.
* @type {Label}
* @default null
*/
messageLabel?: Label;

/**
* Button to Cancel this FormulaBuilder. The formula is not tested, formulaBuilder.cancelled
*  is set to true and formulaBuilder.fireOnClose is fired.
* @type {Button}
* @default null
*/
cancelButton?: Button;

/**
* Button to Test the formula by generating it's function and executing it
* @type {Button}
* @default null
*/
testButton?: Button;

/**
* Button to Save the formula, by generating it's function, testing it and firing 
*  formulaBuilder.fireOnClose, and then start editing another, new one.
* @type {Button}
* @default null
*/
saveAddAnotherButton?: Button;

/**
* Button to Save the formula, by generating it's function, testing it and firing 
*  formulaBuilder.fireOnClose
* @type {Button}
* @default null
*/
saveButton?: Button;

/**
* When FormulaBuilder.testFunction reports an invalid formula, 
*  this attribute provides the error-text to display in the
*  FormulaBuilder.messageLabel.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  The dynamic variables in this case, builderType and errorText, represent the type of this 
*  builder, either Formula or Summary, and the description of the error, respectively.
*  
*  The default output is:
*  
*  Invalid [Formula/Summary]: + the description of the error detected 
*  
* @type {HTMLString}
* @default "Invalid ${builderType}: ${errorText}"
*/
invalidBuilderPrompt?: HTMLString;

/**
* If an invalid builder prompt is displayed, but no explicit error message was returned when
*  attempting to evaluate the formula, this string will be used as a default.
* @type {string}
* @default "[No Explicit Error]"
*/
defaultErrorText?: string;

/**
* When FormulaBuilder.testFunction reports an empty formula, 
*  this attribute provides the error-text to display in the
*  FormulaBuilder.messageLabel.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  Only one dynamic variable, builderType, is available and represents the type of this 
*  builder, either Formula or Summary.
*  
*  The default output is:
*  
*  Invalid blank [Formula/Summary] 
*  
* @type {HTMLString}
* @default "Invalid blank ${builderType}"
*/
invalidBlankPrompt?: HTMLString;

/**
* When FormulaBuilder.testFunction reports a valid formula and no other 
*  errors, this attribute provides the error-text to display in the
*  FormulaBuilder.messageLabel.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  Only one dynamic variable, builderType, is available and represents the type of this 
*  builder, either Formula or Summary.
*  
*  The default output is:
*  
*  Valid [Formula/Summary] 
*  
* @type {HTMLString}
* @default "Valid ${builderType}"
*/
validBuilderPrompt?: HTMLString;

/**
* The title for the window that opens when the FormulaBuilder.helpIcon
*  is clicked.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will
*  be evaluated as JS code when the message is displayed.
*  
*  Only one dynamic variable, builderType, is available and represents the type of this 
*  builder, either Formula or Summary.
*  
*  The default output is:
*  
*  [Formula/Summary] Help
*  
* @type {HTMLString}
* @default "${builderType} Help"
*/
helpWindowTitle?: HTMLString;

/**
* The text to display next to the "Title" field.
* @type {string}
* @default "Title"
*/
titleFieldTitle?: string;

/**
* The default value for new Formula and Summary fields.
* @type {string}
* @default "New Field"
*/
defaultNewFieldTitle?: string;

/**
* The default title for the "Key" column in FormulaBuilder.fields.
* @type {string}
* @default "Key"
*/
keyColumnTitle?: string;

/**
* The default title for the "Source Field" column in in FormulaBuilder.fields.
* @type {string}
* @default "Source Field"
*/
sourceFieldColumnTitle?: string;

/**
* The default title for the "Cancel" button.
* @type {string}
* @default "Cancel"
*/
cancelButtonTitle?: string;

/**
* The default title for the "Save & Add Another" button.
* @type {string}
* @default "Save /&amp; Add Another"
*/
saveAddAnotherButtonTitle?: string;

/**
* The default title for the "Save" button.
* @type {string}
* @default "Save"
*/
saveButtonTitle?: string;

/**
* The text to display in the dialog that opens when there are unsaved changes and the user 
*  cancels the builder.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  Only one dynamic variable, builderType, is available and represents the type of this 
*  builder, either Formula or Summary.
*  
*  The default output is:
*  
*  Save changes to this [Formula/Summary]?
*  
* @type {HTMLString}
* @default "Save changes to this ${builderType}?"
*/
saveConfirmationPrompt?: HTMLString;

/**
* When FormulaBuilder.testFunction reports an attempt to generate a 
*  function from an invalid formula, this is the text to display in the 
*  FormulaBuilder.messageLabel.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  Only one dynamic variable, builderType, is available and represents the type of this 
*  builder, either Formula or Summary.
*  
*  The default output is:
*  
*  The generated function is invalid - Check your [Formula/Summary] and retry.
*  
* @type {HTMLString}
* @default "The generated function is invalid - Check your ${builderType} and retry."
*/
invalidGeneratedFunctionPrompt?: HTMLString;

/**
* The default title for the "Sample" panel, which displays a sample result for the formula.
* @type {string}
* @default "Sample:"
*/
sampleHeaderTitle?: string;

/**
* The default title for the "Test" button.
* @type {string}
* @default "Test"
*/
testButtonTitle?: string;

/**
* Should the user be showed a warning when the entered Title value already exists?
* @type {string}
* @default null
*/
warnDuplicateTitles?: string;

/**
* The message to display when warnDuplicateTitles is true 
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed.
* @type {HTMLString}
* @default "Another field already has the title '${fieldTitle}'.  Continue anyway?"
*/
warnDuplicateTitlesMessage?: HTMLString;

/**
* When set to true, automatically tests the formula by calling 
*  FormulaBuilder.testFunction whenever typing into the 
*  FormulaBuilder.formulaField pauses.
*  
*  The default is true.
* @type {boolean}
* @default true
*/
autoTest?: boolean;

/**
* When FormulaBuilder.autoTest is true, this property indicates the delay in 
*  milliseconds between a user pausing and FormulaBuilder.testFunction
*  being called.
*  
*  The default is 200 milliseconds.
* @type {Integer}
* @default "200"
*/
autoTestDelay?: Integer;

/**
* Record to use when testing the formula dynamically (if FormulaBuilder.autoTest is enabled) or when
*  showing samples of formula output.
*  
*  If not specified, the selected record in the component that launched the FormulaBuilder will
*  be used, or if there's no selection, the first visible row, or with no component, a dummy
*  data row derived automatically from the provided DataSource.
* @type {Record}
* @default null
*/
testRecord?: Record;

/**
* This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed.
*  
*  Default value returns 
*  
*  For Record: + the value of the rows title-field 
*  Output: + the result of he generated function
*  
*  
* @type {HTMLString}
* @default "&lt;nobr&gt;For record: ${title}&lt;/nobr&gt;&lt;br&gt;&lt;nobr&gt;Output: ${output}&lt;/nobr&gt;"
*/
samplePrompt?: HTMLString;



/**
*  Call to set the formula-string in this FormulaBuilder.
*  
*  Note that calling setFormula() will update the UI, generate the formula's function and 
*  test it automatically.
* 
* 
* @param {string} The new formula-string for this builder
*/
setFormula(newValue:string): void;

/**
*  Call to retrieve the text the FormulaBuilder would show by default for help, or override to
*  provide alternate help text.
* 
* 
*/
getHelpText(): string;

/**
*  Returns the entire property-set for the updated field, including title and formula-related
*  properties
*  
* 
*/
getUpdatedFieldObject(): string;

/**
*  Test the formula by generating it's function and trying to run it.
* 
*/
testFunction(): string;

/**
*  Gets the FormulaBuilder.testRecord for this formula.
* 
*/
getTestRecord(): Record;

/**
*  Call to finish working, test the formula and call 
*  FormulaBuilder.fireOnClose. If the formula saves ok, don't close
*  the builder but instead reset it, ready to add a new formula-field.
* 
* 
*/
saveAddAnother(): void;

/**
*  Call to finish working, test the formula and call 
*  FormulaBuilder.fireOnClose. Called automatically
*  when the Save button is clicked.
* 
* 
*/
save(): void;

/**
*  Override to execute a callback function when the Formula is Cancelled or Saved.
*  
* 
*/
fireOnClose(): void;

/**
*  Evaluates and returns the dynamic FormulaBuilder.samplePrompt string which is 
*  displayed beneath the formulaField and updated when typing pauses.
*  
* 
* @param {TestFunctionResult} The return value from a call to testFunction().
*/
getSamplePrompt(result:TestFunctionResult): HTMLString;



}
/**
* Creates a new FormulaBuilder
* 
* @param typeCheckedProperties {Partial<FormulaBuilder>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FormulaBuilder} The new instance
*/
class FormulaBuilder{
static create(typeCheckedProperties? : Partial<FormulaBuilder>, uncheckedProperties?: Object): FormulaBuilder;







}

/**
* Component that wraps the FusionCharts charting engine.
*  
*  Provides multiple-series (area, bar, column, line, radar) and single-series (doughnut, pie)
*  chart types. These charts all depict a single continuous dimension (eg revenue), broken down
*  by one or two discrete dimensions (eg product, region).
*  
*  NOTE: you must load the PluginBridges and Charts
*  loadingOptionalModules before you can use FusionChart.
*/
interface FusionChart extends Flashlet, Chart {
/**
* Base URL where FusionCharts are installed. SmartClient expects to find the FusionCharts
*  ".swf" files under this URL.
*  
*  The default value indicates that SmartClient will look for a FusionCharts directory
*  parallel to the "isomorphic/" directory.
*  
*  Note that the URL formation logic automatically compensates for various inconsistencies and
*  typos in the default names for the chart .swf files, as well as differences between
*  FusionCharts 2.3 and 3.0.  The chart .swf files should be left exactly as found in the
*  FusionCharts distribution. If you have renamed the charts for other purposes, you
*  can set FusionChart.chartURL to the full path to any given chart type.
* @type {URL}
* @default "[ISOMORPHIC]/../FusionCharts/"
*/
chartsBaseURL?: URL;

/**
* Full URL to the chart. Needed only if you have renamed the charts such that automatic
*  URL formation won't work.
* @type {URL}
* @default null
*/
chartURL?: URL;

/**
* Version of FusionCharts to assume. If version is "2.3", different names are used for
*  the .swf files for each chart (to match the default names in FusionCharts 2.3), and a
*  lower version of Flash is required (6.0).
* @type {string}
* @default "3.0"
*/
fusionVersion?: string;

/**
* Properties to passthrough to the 'graph' element of the XML generated for FusionCharts.
*  
*  Some automatic conversions are done:
*  
*  
* - boolean values become '0' and '1' as FusionCharts expects 
*  
* - color values with a leading '#' have the '#' stripped to match FusionCharts 
*  
* @type {object}
* @default null
*/
chartProperties?: object;

/**
* An array of colors to use for a series of visual elements representing data (eg columns,
*  bars, pie slices), any of which may be adjacent to any other.
*  
*  Colors are expressed as hexadecimal RRGGBB Strings with no leading '#' character.
* @type {Array<Partial<CSSColor>>}
* @default "see below"
*/
dataColors?: Array<Partial<CSSColor>>;



/**
*  Get a color from the FusionChart.dataColors Array, or white ("FFFFFF") if the index is beyond the
*  end of the Array. 
*  
*  Override to provide a dynamic color generation scheme.
* 
* 
* @param {number} index of the visual element to be colored
*/
getDataColor(index:number): string;



}
/**
* Creates a new FusionChart
* 
* @param typeCheckedProperties {Partial<FusionChart>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {FusionChart} The new instance
*/
class FusionChart{
static create(typeCheckedProperties? : Partial<FusionChart>, uncheckedProperties?: Object): FusionChart;
/**
* All ChartType that are supported by this class.
*  Should be defined by concrete implementations of the charting interface.
* @type {Array<Partial<ChartType>>}
* @default "["Area", "Bar", "Column", "Pie", "Doughnut", "Line", "Radar"]"
*/
static allChartTypes?: Array<Partial<ChartType>>;








}

/**
* The Gauge widget class implements a graphical speedometer-style gauge for displaying
*  a measurement by means of a needle on a dial. The dial is divided into sectors, each having
*  its own color and value.
*  
*  NOTE: you must load the standard Drawing module before you can use Gauge.
*/
interface Gauge extends DrawPane {
/**
* Default height of the Gauge.pivotPoint if no specific pivotPoint is specified.
*  
*  Can be specified as a numeric pixel value, or a String percentage value.
* @type {number | string}
* @default "70%"
*/
pivotPointHeight?: number | string;

/**
* AutoChild representing the shape drawn at the Gauge.pivotPoint (where all sectors of the
*  gauge meet).
* @type {DrawItem}
* @default null
*/
pivotShape?: DrawItem;

/**
* AutoChild representing the needle shape that points to the gauge's current value. Default is
*  to use a DrawTriangle.
* @type {DrawItem}
* @default null
*/
needle?: DrawItem;

/**
* MultiAutoChild representing the labels used to different data points on the gauge.
* @type {DrawLabel}
* @default null
*/
valueLabel?: DrawLabel;

/**
* MultiAutoChild representing the tick marks drawn along the circumference of the gauge.
*  Default is to use DrawLine.
* @type {DrawItem}
* @default null
*/
tickLine?: DrawItem;

/**
* MultiAutoChild representing the sectors drawn to show different segments of the gauge.
* @type {DrawSector}
* @default null
*/
sectorShape?: DrawSector;

/**
* Radius in pixels of the dial.
* @type {Float}
* @default "150"
*/
dialRadius?: Float;

/**
* Font size of sector labels. Must be at least 3.
* @type {Integer}
* @default "11"
*/
fontSize?: Integer;

/**
* Pixel width for gauge sector borders.
* @type {Integer}
* @default "1"
*/
borderWidth?: Integer;

/**
* Color for gauge sector borders.
* @type {CSSColor}
* @default "#333333"
*/
borderColor?: CSSColor;

/**
* Array of preset fill colors used by the default implementation of Gauge.getDefaultFillColor
*  to initialize the fill color of new sectors.
* 
*  The default array of colors is:
*  
*   
*    #AFFFFF
*    #008080
*    #AAAFFF
*    #FF0000
*    #FFCC99
*    #800080
*   
*  
* @type {Array<Partial<CSSColor>>}
* @default "[ "#AFFFFF", "#008080", "#AAAFFF", "#FF0000", "#FFCC99", "#800080" ]"
*/
sectorColors?: Array<Partial<CSSColor>>;

/**
* The minimum dial value.
* @type {Float}
* @default "0"
*/
minValue?: Float;

/**
* The maximum dial value.
* @type {Float}
* @default "100"
*/
maxValue?: Float;

/**
* The current value on the dial.
* @type {Float}
* @default "0"
*/
value?: Float;

/**
* The number of major tick lines.
* @type {Integer}
* @default "0"
*/
numMajorTicks?: Integer;

/**
* The number of minor tick lines.
* @type {Integer}
* @default "0"
*/
numMinorTicks?: Integer;

/**
* The label prefix.
* @type {string}
* @default ""
*/
labelPrefix?: string;

/**
* The label suffix.
* @type {string}
* @default "%"
*/
labelSuffix?: string;

/**
* Whether the sectors are drawn clockwise, and increasing the value causes the
*  needle to move clockwise.
* @type {boolean}
* @default true
*/
drawnClockwise?: boolean;

/**
* The GaugeSectors contained in this Gauge. 
* 
*  If this this property is not specified, the gauge will
*  be created with a default sector filling the gauge.
* @type {Array<Partial<GaugeSector>>}
* @default null
*/
sectors?: Array<Partial<GaugeSector>>;

/**
* The pivot point of the needle.
* @type {Point}
* @default null
*/
pivotPoint?: Point;



/**
*  Sets the minimum and maximum dial values, rescaling all sectors and the dial value.
* 
* 
* @param {Float} the new minimum dial value
* @param {Float} the new maximum dial value
*/
setValueRange(minValue:Float, maxValue:Float): void;

/**
*  Sets the minimum dial value, rescaling all sectors and the dial value.
*  
*  See Gauge.setValueRange to set both minValue and maxValue together.
* 
* @param {Float} the new minimum dial value. Must be at least 1 less than the maximum dial value. If minValue is not at least 1 less than the maximum value, then it is set to maxValue - 1.
*/
setMinValue(minValue:Float): void;

/**
*  All DrawItems currently associated with this Gauge are destroyed and 
*  new DrawItems are created instead.
* 
* 
* @param {Float} Radius in pixels of the dial
*/
setDialRadius(dialRadius:Float): void;

/**
*  All DrawItems currently associated with this Gauge are destroyed and 
*  new DrawItems are created instead.
*  
*  The pivot point is set by default by choosing 1/2 of width and 70% of height of the Gauge. See
*  Gauge.pivotPointHeight
* 
* 
* @param {Point} The pivot point of the needle
*/
setPivotPoint(point:Point): void;

/**
*  Sets the maximum dial value, rescaling all sectors and the dial value.
*  
*  See Gauge.setValueRange to set both minValue and maxValue together.
* 
* @param {Float} the new maximum dial value. Must be at least 1 greater than the minimum dial value. If maxValue is not at least 1 greater than the minimum value, then it is set to 1 + minValue.
*/
setMaxValue(maxValue:Float): void;

/**
*  Sets the value on the dial that the needle is displaying.
* 
* 
* @param {Float} the new dial value. Must be between Gauge.minValue and Gauge.maxValue.
*/
setValue(value:Float): void;

/**
*  Sets the number of major tick lines.
* 
*  NOTE: To divide the dial into n regions, you will need n + 1 ticks.
*  For example, if the minimum value is 0 and the maximum value is 100, then to place major
*  tick lines at 0, 10, 20, 30, ..., 90, 100, you need 11 (10 + 1) major ticks.
* 
* 
* @param {Integer} the number of major tick lines to draw. Must be either 0 or an integer greater than or equal to 2.
*/
setNumMajorTicks(numMajorTicks:Integer): void;

/**
*  Sets the number of minor tick lines.
* 
*  NOTE: To divide the dial into n regions, you will need n + 1 ticks.
*  For example, if the minimum value is 0 and the maximum value is 100, then to place minor
*  tick lines at 0, 1, 2, 3, 4, 5, ..., 99, 100, you need 101 (100 + 1) minor ticks.
* 
* 
* @param {Integer} the number of minor tick lines to draw. Must be either 0 or an integer greater than or equal to 2.
*/
setNumMinorTicks(numMinorTicks:Integer): void;

/**
*  Sets the Gauge.labelPrefix property and re-creates all sector labels.
* 
* 
* @param {string} the new label prefix.
*/
setLabelPrefix(labelPrefix:string): void;

/**
*  Sets the Gauge.labelSuffix property and re-creates all sector labels.
* 
* 
* @param {string} the new label suffix.
*/
setLabelSuffix(labelSuffix:string): void;

/**
*  Sets the Gauge.drawnClockwise property and redraws the gauge.
* 
* 
* @param {boolean} whether the sectors are drawn clockwise.
*/
setDrawnClockwise(drawnClockwise:boolean): void;

/**
*  Formats a value as a string to be used as the contents of a DrawLabel. The default
*  implementation prepends Gauge.labelPrefix and appends
*  Gauge.labelSuffix to value.
*  
*  NOTE: If a subclass overrides this, then whenever it changes the way that values are
*  formatted, it must call Gauge.reformatLabelContents.
* 
* 
* @param {Float} the value to format.
*/
formatLabelContents(value:Float): string;

/**
*  Resets the contents of all labels. This involves calling Gauge.formatLabelContents
*  to get the label contents for each corresponding value and repositioning the label.
* 
* 
*/
reformatLabelContents(): void;

/**
*  Gets the number of sectors.
* 
* 
*/
getNumSectors(): Integer;

/**
*  Gets the value of the sector at sectorIndex.
* 
* 
* @param {Integer} index of the target sector.
*/
getSectorValue(sectorIndex:Integer): Float;

/**
*  Gets the default fill color for the sector at index sectorIndex. The default
*  implementation cycles through Gauge.sectorColors
*  using modular arithmetic.
* 
* 
* @param {Integer} index of the target sector.
*/
getDefaultFillColor(sectorIndex:Integer): CSSColor;

/**
*  Gets the fill color of the sector at index sectorIndex.
* 
* 
* @param {Integer} index of the target sector.
*/
getSectorFillColor(sectorIndex:Integer): CSSColor;

/**
*  Sets the fill color of the sector at sectorIndex.
* 
* 
* @param {Integer} index of the target sector.
* @param {CSSColor} the new fill color.
*/
setSectorFillColor(sectorIndex:Integer, fillColor:CSSColor): void;

/**
*  Gets the label contents of the label for the sector at sectorIndex.
* 
* 
* @param {Integer} index of the target sector.
*/
getSectorLabelContents(sectorIndex:Integer): string;

/**
*  Sets the sectors for this gauge.
* 
* @param {Array<Partial<GaugeSector>>} the sectors to show on the gauge.
*/
setSectors(sectors:Array<Partial<GaugeSector>>): void;

/**
*  Adds a new sector.
* 
* 
* @param {GaugeSector | Double} the new GaugeSector or the new sector's value. This is formatted with Gauge.formatLabelContents to get its label.
*/
addSector(newSector:GaugeSector | Double): Integer;

/**
*  Removes the sector at sectorIndex.
*  
*  NOTE: There must always be one sector and it is not possible to remove the sole remaining
*  sector. Calling this method to attempt to remove the sole remaining sector is a no-op.
* 
* 
* @param {Integer} the index of the sector to remove.
*/
removeSector(sectorIndex:Integer): void;



}
/**
* Creates a new Gauge
* 
* @param typeCheckedProperties {Partial<Gauge>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Gauge} The new instance
*/
class Gauge{
static create(typeCheckedProperties? : Partial<Gauge>, uncheckedProperties?: Object): Gauge;







}

/**
* Gets the value properties from a component and makes them available within the workflow
*  as the taskInputExpression. Using
*  ProcessElement.bindOutput, these values can also be placed into
*  the Process.state.
*  
*  For a canvas the ComponentTask.componentId
*  specifies everything necessary to identify the target.
*  For a form control more information is needed. The
*  ComponentTask.componentId identifies the container
*  (i.e. DynamicForm) and the individual field is specified as
*  GetPropertiesTask.targetFieldName.
*  
*  The next task might be a save operation on a DataSource, or SetPropertiesTask
*  to copy settings from one component to another.
*  
*  GetPropertiesTask is an advanced and rarely used task.
*  If you need a component property to be dynamic, you can configure that property as a
*  Class.dynamicProperties without the need for a
*  Workflow. Dynamic Properties created this way automatically
*  update as your users make changes.
*/
interface GetPropertiesTask extends ComponentTask {
/**
* Properties to be retrieved from ComponentTask.componentId.
* @type {Array<Partial<string>>}
* @default null
*/
properties?: Array<Partial<string>>;

/**
* If ComponentTask.componentId targets a DynamicForm, this property
*  specifies the name of the target field.
* @type {FieldName}
* @default null
*/
targetFieldName?: FieldName;





}
/**
* Creates a new GetPropertiesTask
* 
* @param typeCheckedProperties {Partial<GetPropertiesTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GetPropertiesTask} The new instance
*/
class GetPropertiesTask{
static create(typeCheckedProperties? : Partial<GetPropertiesTask>, uncheckedProperties?: Object): GetPropertiesTask;







}

/**
* A widget for visually editing CSS gradients.
*  
*  Note: this feature requires
*  SmartClient Enterprise.
*/
interface GradientEditor extends VLayout {
/**
* The angle by which drag-rotating a linear-gradient in the preview should snap. The 
*  default of 1 means a linear-gradient can be positioned to any exact degree of a circle.
* @type {Integer}
* @default "1"
*/
dragSnapAngle?: Integer;

/**
* Whether to show the GradientEditor.previewCanvas, used to display 
*  the gradient.
* @type {boolean}
* @default true
*/
showPreviewCanvas?: boolean;

/**
* An AutoChild of type Canvas, used to display the gradient according to 
*  the defined color-stops. The canvas supports drag and click mouse interactions to update
*  a gradient's rotation and origin.
* @type {Canvas}
* @default null
*/
previewCanvas?: Canvas;

/**
* The CSS gradient-statement to work with.
* @type {string}
* @default null
*/
gradient?: string;

/**
* A DynamicForm that shows the gradient-string produced by this editor.
* @type {DynamicForm}
* @default null
*/
outputForm?: DynamicForm;



/**
*  Returns the current state of the widget as a CSS gradient
* 
*/
getGradient(): string;

/**
*  Parses the passed gradient and applies it to the UI for editing.
* 
* @param {string} CSS gradient string
*/
setGradient(gradient:string): void;

/**
*  Notification fired whenever a user updates the gradient.
* 
* @param {string} the current GradientEditor.gradient
*/
gradientChanged(gradient:string): void;



}
/**
* Creates a new GradientEditor
* 
* @param typeCheckedProperties {Partial<GradientEditor>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GradientEditor} The new instance
*/
class GradientEditor{
static create(typeCheckedProperties? : Partial<GradientEditor>, uncheckedProperties?: Object): GradientEditor;







}

/**
* EditProxy that handles ListGrid and TreeGrid components when editMode is enabled.
*/
interface GridEditProxy extends LayoutEditProxy {
/**
* Should changes to grid field order be persisted?
*  
*  Note that changes are saved directly into the ListGridFields not via fieldState or viewState settings.
*  EditNodes will also be introduced for fields as needed if they do not already exist.
*  
*  Only valid with SelectedAppearance settings that allow direct interactivity
*  (such as "outlineEdges").
* @type {boolean}
* @default true
*/
saveFieldOrder?: boolean;

/**
* Should changes to grid field visibility be persisted?
*  
*  Note that changes are saved directly into the ListGridFields not via fieldState or viewState settings.
*  EditNodes will also be introduced for fields as needed if they do not already exist.
*  
*  Only valid with SelectedAppearance settings that allow direct interactivity
*  (such as "outlineEdges").
* @type {boolean}
* @default true
*/
saveFieldVisibility?: boolean;

/**
* Should changes to which fields are ListGridField.frozen be persisted?
*  
*  Note that changes are saved directly into the ListGridFields not via fieldState or viewState settings.
*  EditNodes will also be introduced for fields as needed if they do not already exist.
*  
*  Only valid with SelectedAppearance settings that allow direct interactivity
*  (such as "outlineEdges").
* @type {boolean}
* @default true
*/
saveFieldFrozenState?: boolean;

/**
* Should changes to which fields are sorted be persisted?
*  
*  Only valid with SelectedAppearance settings that allow direct interactivity
*  (such as "outlineEdges").
* @type {boolean}
* @default true
*/
saveSort?: boolean;

/**
* Should changes to grid grouping (including both grouping and ungrouping the grid) be
*  persisted?
*  
*  Only valid with SelectedAppearance settings that allow direct interactivity
*  (such as "outlineEdges").
* @type {boolean}
* @default true
*/
saveGroupBy?: boolean;

/**
* Should changes to filter criteria by end user editing of criteria in the
*  ListGrid.showFilterEditor by persisted?
*  
*  Only valid with SelectedAppearance settings that allow direct interactivity
*  (such as "outlineEdges").
* @type {boolean}
* @default true
*/
saveFilterCriteria?: boolean;

/**
* Can highlights be edited from header context menu?
*  Overrides ListGrid.canEditHilites when in edit mode.
* @type {boolean}
* @default true
*/
canEditHilites?: boolean;

/**
* Can new formula fields be created from header context menu?
*  Overrides ListGrid.canAddFormulaFields when in edit mode.
* @type {boolean}
* @default true
*/
canAddFormulaFields?: boolean;

/**
* Can new summary fields be created from header context menu?
*  Overrides ListGrid.canAddSummaryFields when in edit mode.
* @type {boolean}
* @default true
*/
canAddSummaryFields?: boolean;

/**
* Can records be grouped from header context menu?
*  Overrides ListGrid.canGroupBy when in edit mode.
* @type {boolean}
* @default true
*/
canGroupBy?: boolean;

/**
* Indicates whether fields in this listGrid can be reordered by dragging and
*  dropping header fields. 
*  Overrides ListGrid.canReorderFields when in edit mode.
* @type {boolean}
* @default true
*/
canReorderFields?: boolean;

/**
* Indicates whether fields in this listGrid can be resized by dragging header
*  fields.
*  Overrides ListGrid.canResizeFields when in edit mode.
* @type {boolean}
* @default true
*/
canResizeFields?: boolean;

/**
* Controls whether highlights created while in edit mode are editable by end users at
*  runtime (when the grid is no longer in edit mode). See Hilite.canEdit.
* @type {boolean}
* @default true
*/
generateEditableHilites?: boolean;

/**
* Controls whether formula fields created while in edit mode are editable by end users at
*  runtime (when the grid is no longer in edit mode). See
*  ListGridField.canEditFormula.
* @type {boolean}
* @default true
*/
generateEditableFormulas?: boolean;

/**
* Controls whether summary fields created while in edit mode are editable by end users at
*  runtime (when the grid is no longer in edit mode). See
*  ListGridField.canEditSummary.
* @type {boolean}
* @default true
*/
generateEditableSummaries?: boolean;



/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the grid's wiki-style data - see MockDataSource.mockData for a
*  description of this format.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the grid's data and field configuration.
* 
* 
* @param {string} the new grid configuration
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new GridEditProxy
* 
* @param typeCheckedProperties {Partial<GridEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GridEditProxy} The new instance
*/
class GridEditProxy{
static create(typeCheckedProperties? : Partial<GridEditProxy>, uncheckedProperties?: Object): GridEditProxy;







}

/**
* Edit an existing record in a grid or start editing a new one. A new record is added
*  unless GridEditRecordTask.criteria is specified. Alternatively, if
*  GridEditRecordTask.editFirstRecord is specified, the first
*  record is edited. 
*  
*  If criteria is provided and the criteria matches more than one record, the first matched
*  record is edited. Additionally, if the record to be edited is not visible, the record will
*  be scrolled into view.
*  
*  Note that the record to be matched must already be loaded in the grid - no fetch will be
*  performed.
*/
interface GridEditRecordTask extends ComponentTask {
/**
* Initial values for a new edit record.
*  
*  Data values prefixed with "$" will be treated as a taskInputExpression
*  excluding "$input" and "$inputRecord" references.
* @type {Record}
* @default null
*/
initialValues?: Record;

/**
* When neither GridEditRecordTask.initialValues nor Criteria are provided should the
*  first record in the grid be edited? If not set, a new record is added.
* @type {boolean}
* @default null
*/
editFirstRecord?: boolean;

/**
* Criteria (including AdvancedCriteria) used to locate the record to be edited.
*  If criteria matches more than one record, the first record is edited.
*  
*  Data values in this criteria prefixed with "$" will be treated as dynamic expressions
*  which can access the inputs to this task as $input - see
*  taskInputExpression. Specifically, this means that for simple criteria,
*  any property value that is a String and is prefixed with "$" will be assumed to be an
*  expression, and for AdvancedCriteria, the same treatment will be applied to
*  Criterion.value.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Process.ruleScope.
* @type {Criteria}
* @default null
*/
criteria?: Criteria;





}
/**
* Creates a new GridEditRecordTask
* 
* @param typeCheckedProperties {Partial<GridEditRecordTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GridEditRecordTask} The new instance
*/
class GridEditRecordTask{
static create(typeCheckedProperties? : Partial<GridEditRecordTask>, uncheckedProperties?: Object): GridEditRecordTask;







}

/**
* Export data currently shown in a grid keeping all grid-specific formatting.
*/
interface GridExportClientDataTask extends ComponentTask {
/**
* Additional properties to set on the DSRequest that will be issued
*  to perform client-side export.
* @type {DSRequest}
* @default null
*/
requestProperties?: DSRequest;





}
/**
* Creates a new GridExportClientDataTask
* 
* @param typeCheckedProperties {Partial<GridExportClientDataTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GridExportClientDataTask} The new instance
*/
class GridExportClientDataTask{
static create(typeCheckedProperties? : Partial<GridExportClientDataTask>, uncheckedProperties?: Object): GridExportClientDataTask;







}

/**
* Export data currently shown in a grid.
*/
interface GridExportDataTask extends ComponentTask {
/**
* Additional properties to set on the DSRequest that will be issued
*  to perform server-side export.
* @type {DSRequest}
* @default null
*/
requestProperties?: DSRequest;





}
/**
* Creates a new GridExportDataTask
* 
* @param typeCheckedProperties {Partial<GridExportDataTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GridExportDataTask} The new instance
*/
class GridExportDataTask{
static create(typeCheckedProperties? : Partial<GridExportDataTask>, uncheckedProperties?: Object): GridExportDataTask;







}

/**
* Fetch data matching specified criteria into grid.
*/
interface GridFetchDataTask extends ComponentTask {
/**
* Criteria to use for fetch. If criteria is null or empty, the filter
*  criteria of the grid will be cleared.
* @type {Criteria}
* @default null
*/
criteria?: Criteria;

/**
* Should criteria be applied to the grid as
*  ListGrid.implicitCriteria? When criteria is applied this way,
*  it is not shown to the user and cannot be changed by the user.
* @type {boolean}
* @default null
*/
applyToImplicitCriteria?: boolean;

/**
* Additional properties to set on the DSRequest that will be issued
*  to perform the fetch or clear criteria.
* @type {DSRequest}
* @default null
*/
requestProperties?: DSRequest;





}
/**
* Creates a new GridFetchDataTask
* 
* @param typeCheckedProperties {Partial<GridFetchDataTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GridFetchDataTask} The new instance
*/
class GridFetchDataTask{
static create(typeCheckedProperties? : Partial<GridFetchDataTask>, uncheckedProperties?: Object): GridFetchDataTask;







}

/**
* Remove data that is selected in a grid.
*/
interface GridRemoveSelectedDataTask extends ComponentTask {
/**
* Can multiple records be removed? If not and multiple records are selected,
*  the GridRemoveSelectedDataTask.refuseMultiRemoveMessage is shown as a warning and the workflow is
*  terminated.
*  
*  Default is true unless set to false.
* @type {boolean}
* @default null
*/
allowMultiRecordRemove?: boolean;

/**
* Message to be shown when multiple records are selected for removal but 
*  GridRemoveSelectedDataTask.allowMultiRecordRemove is not set.
* @type {string}
* @default "Only one record may be removed at a time"
*/
refuseMultiRemoveMessage?: string;

/**
* Should we show a confirmation message before removal and allow the user to cancel the
*  operation? If user cancels the removal the workflow is terminated.
*  
*  Message to be shown is defined in the properties: GridRemoveSelectedDataTask.confirmationMessage,
*  GridRemoveSelectedDataTask.unboundConfirmationMessage, GridRemoveSelectedDataTask.multipleConfirmationMessage and
*  GridRemoveSelectedDataTask.unboundMultipleConfirmationMessage. The appropriate message is chosen
*  based on the grid configuration and the number of records being removed.
* @type {boolean}
* @default null
*/
showConfirmation?: boolean;

/**
* Message shown to user to confirm deleting a single record on a bound grid.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  Only three dynamic variables, dsTitle; dsPluralTitle and recordTitle, are available and
*  represent the bound DataSource.title and DataSource.pluralTitle and
*  the value of the DataSource.titleField from the first selected record.
* @type {string}
* @default "Delete ${dsTitle} ${recordTitle}"
*/
confirmationMessage?: string;

/**
* Message shwon to user to confirm deleting a single record on an unbound grid.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  Only one dynamic variable, recordTitle, is available and
*  represents the value of the DataSource.titleField from the first selected record.
* @type {string}
* @default "Delete record?"
*/
unboundConfirmationMessage?: string;

/**
* Message shown to user to confirm deleting a group of records on a bound grid.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  Only two dynamic variables, dsTitle and dsPluralTitle, are available and
*  represent the bound DataSource.title and DataSource.pluralTitle .
* @type {string}
* @default "Delete multiple ${dsPluralTitle}?"
*/
multipleConfirmationMessage?: string;

/**
* Message shown to user to confirm deleting a group of records on an unbound grid.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  Only one dynamic variable, recordTitle, is available and
*  represents the value of the DataSource.titleField from the first selected record.
* @type {string}
* @default "Delete multiple records?"
*/
unboundMultipleConfirmationMessage?: string;

/**
* ID of the next sequence or element to proceed to if a failure condition arises
*  from operation.
* @type {string}
* @default null
*/
failureElement?: string;

/**
* Additional properties to set on the ListGrid.removeSelectedData
*  call to perform removal.
*  
*  Note that willHandleError will always be set true.
* @type {DSRequest}
* @default null
*/
requestProperties?: DSRequest;





}
/**
* Creates a new GridRemoveSelectedDataTask
* 
* @param typeCheckedProperties {Partial<GridRemoveSelectedDataTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GridRemoveSelectedDataTask} The new instance
*/
class GridRemoveSelectedDataTask{
static create(typeCheckedProperties? : Partial<GridRemoveSelectedDataTask>, uncheckedProperties?: Object): GridRemoveSelectedDataTask;







}

/**
* A flexible, high-speed table that offers consistent cross-platform sizing, clipping, and events.
*/
interface GridRenderer extends Canvas {
/**
* Total number of rows in the grid.
* 
*  NOTE: in order to create a valid grid, you must either provide a totalRows value or implement
*  getTotalRows()
* @type {number}
* @default "0"
*/
totalRows?: number;

/**
* Whether all rows should be drawn all at once, or only rows visible in the viewport.
*  
*  Drawing all rows causes longer initial rendering time, but allows smoother vertical scrolling.
*  With a very large number of rows, showAllRows will become too slow.
*  
*  See also GridRenderer.drawAheadRatio and GridRenderer.drawAllMaxCells.
* @type {boolean}
* @default false
*/
showAllRows?: boolean;

/**
* When incremental rendering is switched on and there are variable record heights, the virtual
*  scrolling mechanism manages the differences in scroll height calculations due to the
*  unknown sizes of unrendered rows to make the scrollbar and viewport appear correctly.
*  
*  virtualScrolling is switched on automatically when fixedRowHeights is false but you should 
*  switch it on any time there are variable record heights.
* @type {boolean}
* @default null
*/
virtualScrolling?: boolean;

/**
* Whether all columns should be drawn all at once, or only columns visible in the viewport.
*  
*  Drawing all columns causes longer initial rendering time, but allows smoother horizontal
*  scrolling. With a very large number of columns, showAllColumns will become too slow.
* @type {boolean}
* @default false
*/
showAllColumns?: boolean;

/**
* If drawing all rows would cause less than drawAllMaxCells cells to be rendered,
*  the full dataset will instead be drawn even if ListGrid.showAllRecords
*  is false and the viewport size and GridRenderer.drawAheadRatio setting would normally have caused
*  incremental rendering to be used.
*  
*  The drawAllMaxCells setting prevents incremental rendering from being used in
*  situations where it's really unnecessary, such as a 40 row, 5 column dataset (only 200
*  cells) which happens to be in a grid with a viewport showing only 20 or so rows.
*  Incremental rendering causes a brief "flash" during scrolling as the visible portion of the
*  dataset is redrawn, and a better scrolling experience can be obtained in this situation by
*  drawing the entire dataset up front, which in this example would have negligible effect on
*  initial draw time.
*  
*  drawAllMaxCells:0 disables this features. You may want to disable this feature
*  if performance is an issue and:
*  
*  
* - you are very frequently redraw a grid
*  
* - you do a lot of computation when rendering each cell (eg formulas)
*  
* - you are showing many grids on one screen and the user won't scroll most of them
*  
* @type {Integer}
* @default "250"
*/
drawAllMaxCells?: Integer;

/**
* If true, if the user clicks on the scroll buttons at the end of the track or clicks once on
*  the scroll track, there will be an instant redraw of the grid content so that the user
*  doesn't see any blank space. For drag scrolling or other types of scrolling, the
*  GridRenderer.scrollRedrawDelay applies.
* @type {boolean}
* @default true
*/
instantScrollTrackRedraw?: boolean;

/**
* While drag scrolling in an incrementally rendered grid, time in milliseconds to wait
*  before redrawing, after the last mouse movement by the user. This delay may be 
*  separately customized for mouse-wheel scrolling via GridRenderer.scrollWheelRedrawDelay.
*  
*  See also
*  GridRenderer.instantScrollTrackRedraw for cases where this delay is skipped.
*  
*  NOTE: In Browser.isTouch,
*  GridRenderer.touchScrollRedrawDelay is used instead.
* @type {Integer}
* @default "0"
*/
scrollRedrawDelay?: Integer;

/**
* Like GridRenderer.scrollRedrawDelay, but applies when the component is being
*  drag-scrolled (via a scrollbar). This value is typically set higher than GridRenderer.scrollRedrawDelay
*  to avoid too many concurrent fetches to the server for ResultSet-backed components
*  since it's quite easy to induce such a case with a scrollbar and a grid bound to a large databaset.
* @type {Integer}
* @default "75"
*/
dragScrollRedrawDelay?: Integer;

/**
* In Browser.isTouch, the time in milliseconds to wait after a scroll
*  before redrawing. In non-touch browsers, the GridRenderer.scrollRedrawDelay
*  or GridRenderer.scrollWheelRedrawDelay is used instead.
* @type {Integer}
* @default "0"
*/
touchScrollRedrawDelay?: Integer;

/**
* While scrolling an incrementally rendered grid, using the mouseWheel, time in 
*  milliseconds to wait before redrawing, after the last mouseWheel movement by the user. 
*  If not specified GridRenderer.scrollRedrawDelay will be used as a default for both
*  drag scrolling and mouseWheel scrolling.
*  
*  See also
*  GridRenderer.instantScrollTrackRedraw for cases where this delay is skipped.
* @type {Integer}
* @default null
*/
scrollWheelRedrawDelay?: Integer;

/**
* How far should we render rows ahead of the currently visible area? This is expressed as a
*  ratio from viewport size to rendered area size.
*  
*  Tweaking drawAheadRatio allows you to make tradeoffs between continuous scrolling speed vs
*  initial render time and render time when scrolling by large amounts.
*  
*  NOTE: Only applies when showAllRows is false.
* @type {Float}
* @default "2.0"
*/
drawAheadRatio?: Float;

/**
* Alternative to GridRenderer.drawAheadRatio, to be used when the user
*  is rapidly changing the grids viewport (for example drag scrolling through the grid).
*  If unspecified GridRenderer.drawAheadRatio will be used in all cases
* @type {Float}
* @default "2.0"
*/
quickDrawAheadRatio?: Float;

/**
* The default height of each row in pixels.
* @type {number}
* @default "20"
*/
cellHeight?: number;

/**
* Should we vertically clip cell contents, or allow rows to expand vertically to show all
*  contents?
*  
*  If we allow rows to expand, the row height as derived from
*  GridRenderer.getRowHeight or the
*  default GridRenderer.cellHeight is treated as a minimum.
* @type {boolean}
* @default true
*/
fixedRowHeights?: boolean;

/**
* Should we horizontally clip cell contents, or allow columns to expand horizontally to
*  show all contents?
* 
*  If we allow columns to expand, the column width is treated as a minimum.
* @type {boolean}
* @default true
*/
fixedColumnWidths?: boolean;

/**
* If true, make columns only wide enough to fit content, ignoring any widths specified.
*  Overrides fixedFieldWidths.
* @type {boolean}
* @default false
*/
autoFit?: boolean;

/**
* Should content within cells be allowed to wrap?
* @type {boolean}
* @default false
*/
wrapCells?: boolean;

/**
* The amount of empty space, in pixels, surrounding each value in its cell.
* @type {number}
* @default "2"
*/
cellPadding?: number;

/**
* If true, rightMouseDown events will fire 'selectOnRightMouseDown()' for the appropriate cells.
* @type {boolean}
* @default true
*/
canSelectOnRightMouse?: boolean;

/**
* If true, cellHover and rowHover events will fire and then a hover will be shown (if not
*  canceled) when the user leaves the mouse over a row / cell unless the corresponding field has
*  ListGridField.showHover set to false. If unset or null, the hover will be
*  shown if the corresponding field has showHover:true. If false, then hovers are disabled.
*  
*  Note that standard hovers override GridRenderer.showClippedValuesOnHover. Thus,
*  to enable clipped value hovers, canHover must be unset or null and the corresponding field
*  must have showHover unset or null as well.
* @type {boolean}
* @default null
*/
canHover?: boolean;

/**
* If true, and canHover is also true, when the user hovers over a cell, hover text will pop up
*  next to the mouse. The contents of the hover is determined by GridRenderer.cellHoverHTML.
* @type {boolean}
* @default null
*/
showHover?: boolean;

/**
* If true and a cell's value is clipped, then a hover containing the full cell value is
*  enabled.
*  
*  Note that standard cell hovers override clipped value hovers. Thus, to enable clipped value
*  hovers, GridRenderer.canHover must be unset or null and the corresponding field must have
*  ListGridField.showHover unset or null as well.
* @type {boolean}
* @default null
*/
showClippedValuesOnHover?: boolean;

/**
* The base name for the CSS class applied to cells. This style will have "Dark", 
*  "Over", "Selected", or "Disabled" appended to it according to the state of the cell.
* @type {CSSStyleName}
* @default "cell"
*/
baseStyle?: CSSStyleName;

/**
* Whether alternating rows (or blocks of rows, depending
*  on GridRenderer.alternateRowFrequency) should be drawn in alternating styles,
*  in order to create a "ledger" effect for easier reading.
*  
*  If enabled, the cell style for alternate rows will have the
*  GridRenderer.alternateRowSuffix appended to it.
*  See also GridRenderer.alternateColumnStyles.
* @type {boolean}
* @default false
*/
alternateRowStyles?: boolean;

/**
* Suffix to append to GridRenderer.alternateRowStyles.
*  Note that if GridRenderer.alternateColumnStyles is enabled, cells which fall
*  into both an alternate row and column will have both suffixes appended - for
*  example "cellDarkAltCol".
* @type {string}
* @default "Dark"
*/
alternateRowSuffix?: string;

/**
* The number of consecutive rows to draw in the same style before alternating, when
*  GridRenderer.alternateRowStyles is true.
* @type {number}
* @default "1"
*/
alternateRowFrequency?: number;

/**
* Whether alternating columns (or blocks of columns, depending
*  on GridRenderer.alternateColumnFrequency) should be drawn in alternating styles,
*  in order to create a vertical "ledger" effect for easier reading.
*  
*  If enabled, the cell style for alternate rows will have the
*  GridRenderer.alternateColumnSuffix appended to it.
*  See also GridRenderer.alternateRowStyles.
* @type {boolean}
* @default false
*/
alternateColumnStyles?: boolean;

/**
* Suffix to append to GridRenderer.alternateColumnStyles.
*  Note that if GridRenderer.alternateRowStyles is enabled, cells which fall
*  into both an alternate row and column will have both suffixes appended - for
*  example "cellDarkAltCol".
* @type {string}
* @default "AltCol"
*/
alternateColumnSuffix?: string;

/**
* The number of consecutive columns to draw in the same style before alternating, when
*  GridRenderer.alternateColumnStyles is true.
* @type {number}
* @default "1"
*/
alternateColumnFrequency?: number;

/**
* Value to show in empty cells (when getCellValue returns null).
* @type {string}
* @default "&amp;nbsp;"
*/
emptyCellValue?: string;

/**
* Indicates whether the text of the emptyMessage property should be displayed if no data is
*  available.
* @type {boolean}
* @default true
*/
showEmptyMessage?: boolean;

/**
* The string to display in the body of a listGrid with an empty data array, if
*  showEmptyMessage is true.
* @type {string}
* @default "No data to display"
*/
emptyMessage?: string;

/**
* CSS styleName for the table as a whole if we're showing the empty message
* @type {CSSStyleName}
* @default null
*/
emptyMessageTableStyle?: CSSStyleName;

/**
* The CSS style name applied to the emptyMessage string if displayed.
* @type {CSSStyleName}
* @default null
*/
emptyMessageStyle?: CSSStyleName;

/**
* Indicates whether the text of the offlineMessage property should be displayed if no data
*  is available because we are offline and there is no suitable cached response
* @type {boolean}
* @default true
*/
showOfflineMessage?: boolean;

/**
* The CSS style name applied to the offlineMessage string if displayed.
* @type {CSSStyleName}
* @default null
*/
offlineMessageStyle?: CSSStyleName;

/**
* The string to display in the body of a listGrid with an empty data array, if
*  showOfflineMessage is true and the data array is empty because we are offline and there
*  is no suitable cached response
* @type {string}
* @default "No data available while offline"
*/
offlineMessage?: string;

/**
* Note: This property only has an effect in Internet Explorer
*  
*  Advanced property to improve performance for dynamic styling of gridRenderer cells in
*  Internet Explorer, at the expense of slightly slower initial drawing, and some 
*  limitations on supported styling options.
*  
*  fastCellUpdates speeds up the dynamic styling system used by rollovers,
*  selections, and custom styling that calls GridRenderer.refreshCellStyle, at
*  the cost of slightly slower draw() and redraw() times.
*  
*  Notes:
*  
*  
* - When this property is set, ListGrid cells may be styled using the 
*    ListGrid.tallBaseStyle. See ListGrid.getBaseStyle for 
*    more information.
*  
* - If any cell styles specify a a background image URL, the URL will be resolved relative
*    to the page location rather than the location of the CSS stylesheet. This means cell
*    styles with a background URL should either supply a fully qualified path, or the
*    background image media should be made available at a second location for IE.
*  
* - fastCellUpdates will not work if the styles involved are in an external stylesheet loaded
*    from a remote host. Either the stylesheet containing cell styles needs to be loaded
*    from the same host as the main page, or the cell styles need to be inlined in the html 
*    of the bootstrap page.
*  
* - fastCellUpdates will not work if the css styles for cells are defined in
*    a .css file loaded via @import. Instead the .css
*    file should be loaded via a &lt;link ...&gt; tag.
*  
* @type {boolean}
* @default true
*/
fastCellUpdates?: boolean;

/**
* Should drag-and-drop operations snap the dragged object into line with the nearest cell?
* @type {boolean}
* @default false
*/
snapToCells?: boolean;

/**
* If true, snap-to-cell drops will snap the dropped object inside the selected cell's border.
*  If false, snap-to-cell drops will snap the dropped object to the edge of the selected cell, 
*  regardless of borders
* @type {boolean}
* @default false
*/
snapInsideBorder?: boolean;

/**
* Denotes the name of a property that can be set on records to display a custom style.
*  For example if this property is set to "customStyle", setting 
*  record.customStyle to a css styleName will cause the record in question to
*  render out with that styling applied to it. Note that this will be a static 
*  style - it will not be modified as the state of the record (selected / over etc) changes.
* @type {"CustomStyle"}
* @default "string"
*/
recordCustomStyleProperty?: "CustomStyle";

/**
* Should the "Selected" style be applied to selected records?
* @type {boolean}
* @default true
*/
showSelectedStyle?: boolean;

/**
* Should this Grid write an ID (obtained via GridRenderer.getRowElementId)
*  into each row element?
*  
*  Note that for integration with automated testing tools, we recommend using 
*  AutoTest.getLocator and AutoTest.getElement to identify 
*  specific elements within the DOM wherever possible. These are more robust than simple
*  cell locators as they can identify the DOM element representing a specific data elements
*  using primary key and other approaches rather than relying on the generated DOM structure.
*  
*  Note that if isc.setScreenReaderMode is enabled, row elements will
*  be written out with a generated ID even if this property is false.
*  The default generated ListGrid.ariaRole for listGrids makes
*  use of DOM IDs to identify rows and cells in various cases.
* @type {boolean}
* @default false
*/
writeOutRowElementId?: boolean;

/**
* Should this Grid write an ID (obtained via GridRenderer.getCellElementId)
*  into each cell element?
*  
*  Note that for integration with automated testing tools, we recommend using 
*  AutoTest.getLocator and AutoTest.getElement to identify 
*  specific elements within the DOM wherever possible. These are more robust than simple
*  cell locators as they can identify the DOM element representing a specific data elements
*  using primary key and other approaches rather than relying on the generated DOM structure.
*  
*  Note that if isc.setScreenReaderMode is enabled, cell elements will
*  be written out with a generated ID even if this property is false.
*  The default generated ListGrid.ariaRole for listGrids makes
*  use of DOM IDs to identify rows and cells in various cases.
* @type {boolean}
* @default false
*/
writeOutCellElementId?: boolean;

/**
* If GridRenderer.showHover is true, should cell hover HTML be displayed on disabled cells?
* @type {boolean}
* @default false
*/
showHoverOnDisabledCells?: boolean;



/**
*  Given a pointer to an element in the DOM, this method will check whether this
*  element is contained within a cell of the gridRenderer, and if so return a
*  2 element array denoting the [rowNum,colNum] of the element
*  in question.
* 
* @param {DOMElement} DOM element to test
*/
getCellFromDomElement(element:DOMElement): Array<any>;

/**
*  Setter for GridRenderer.fastCellUpdates. Has no effect in browsers other than
*  Internet Explorer.
* 
* @param {boolean} whether to enable fastCellUpdates.
*/
setFastCellUpdates(fastCellUpdates:boolean): void;

/**
*  Is the value in a given cell clipped?
* 
* @param {number} row number of the cell
* @param {number} column number of the cell
*/
cellValueIsClipped(rowNum:number, colNum:number): boolean;

/**
*  Return the total number of rows in the grid.
* 
*  NOTE: in order to create a valid grid, you must either provide a totalRows value or implement
*  getTotalRows()
* 
* 
*/
getTotalRows(): number;

/**
*  Sets the width of a single column.
* 
* 
* @param {number} the number of the column to resize
* @param {number} the new width
*/
setColumnWidth(colNum:number, newWidth:number): void;

/**
* 
*  Sets the width of all columns in the grid.
* 
* 
* @param {Array<any>} array of new widths - one for each column.
*/
setColumnWidths(newWidths:Array<any>): void;

/**
*  When using GridRenderer.getRowSpan, returns the row number where a row-spanning cell
*  starts.
*  
*  For example, if row 2 col 0 spans 3 cells, getCellStartRow() for row 2 col 0,
*  row 3 col 0, row 4 col 0 will all return 2, because that's the row when spanning starts.
* 
* 
* @param {Integer} row number of cell for which the start row should be returned
* @param {Integer} column number of cell for which the start row should be returned
*/
getCellStartRow(rowNum:Integer, colNum:Integer): Integer;

/**
*  When using GridRenderer.getRowSpan, returns the number of cells spanned by the cell
*  at the given coordinates.
*  
*  If the passed coordinates are in the middle of a series of spanned cells, the row span of
*  the spanning cell is returned. For example, if row 2 col 0 spans 3 cells, calls to
*  getCellRowSpan() for row 2 col 0, row 3 col 0, row 4 col 0 will all return 3.
*  
*  This method returns row span information for the current rendered cells. In contrast, if
*  the grid is about to be redrawn, a call to getRowSpan() may return row span
*  values for how the grid is about to be drawn. Also, user-provided getRowSpan() functions
*  are not required to operate properly when called outside of the grid rendering loop.
*  
*  Note: This method is a utility method for developers - it is not called
*  directly by the grid rendering path and therefore is not intended for override. To
*  set up custom row-spanning behavior, override GridRenderer.getRowSpan instead.
* 
* 
* @param {Integer} row number of cell to return the row span for
* @param {Integer} column number of cell to return the row span for
*/
getCellRowSpan(rowNum:Integer, colNum:Integer): Integer;

/**
*  Return the CSS class for a cell. By default this method has the following implementation:
*  - return any custom style for the record (GridRenderer.recordCustomStyleProperty) 
*   if defined.
*  - create a style name based on the result of GridRenderer.getBaseStyle and the 
*   state of the record using the rules described in cellStyleSuffixes.
*  
*  Cell Styles are customizable by:
*  
*  
* - attaching a custom style to a record by setting 
*   record[this.recordCustomStyleProperty] to some valid CSS style name. 
*  
* - modifying the base style returned by getBaseStyle() [see that method for further 
*    documentation on this]
*  
* - overriding this function
*  
*  
*  In addition to this, GridRenderer.getCellCSSText may be overriden to
*  provide custom cssText to apply on top of the styling attributes derived from the named
*  style.
*  
* 
* 
* @param {ListGridRecord} record object for this row and column
* @param {number} number of the row
* @param {number} number of the column
*/
getCellStyle(record:ListGridRecord, rowNum:number, colNum:number): CSSStyleName;

/**
*  Whether this cell should be considered enabled. Affects whether events will fire for the
*  cell, and the default styling behavior in getCellStyle. 
* 
* 
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellIsEnabled(rowNum:number, colNum:number): boolean;

/**
*  Get the DOM ID that should be used for a row element if 
*  isc.setScreenReaderMode is enabled 
*  or GridRenderer.writeOutRowElementId is true.
*  
*  Note that for integration with automated testing tools, we recommend using 
*  AutoTest.getLocator and AutoTest.getElement to identify 
*  specific elements within the DOM wherever possible. These are more robust than simple
*  cell locators as they can identify the DOM element representing a specific data elements
*  using primary key and other approaches rather than relying on the generated DOM structure.
*  
*  When using incremental rendering, the rowNum param represents the
*  rowNum in virtual coordinates, and the physicalRowNum param represents the
*  index that the row will ultimately have in table.rows.
* 
* 
* @param {number} virtual row number
* @param {number} physical row number
*/
getRowElementId(rowNum:number, physicalRowNum:number): string;

/**
*  Get the DOM ID that should be used for a cell element if 
*  isc.setScreenReaderMode is enabled 
*  or GridRenderer.writeOutRowElementId is true.
*  
*  Note that for integration with automated testing tools, we recommend using 
*  AutoTest.getLocator and AutoTest.getElement to identify 
*  specific elements within the DOM wherever possible. These are more robust than simple
*  cell locators as they can identify the DOM element representing a specific data elements
*  using primary key and other approaches rather than relying on the generated DOM structure.
*  
*  When using incremental rendering, the rowNum and colNum params
*  represents virtual coordinates, and the physicalRowNum param represents the
*  index that the row/cell will ultimately have in table.rows or row.cells.
* 
* 
* @param {number} virtual row number
* @param {number} physical row number
* @param {number} virtual col number
* @param {number} physical col number
*/
getCellElementId(rowNum:number, physicalRowNum:number, colNum:number, physicalColNum:number): string;

/**
*  Refresh the styling of an individual cell without redrawing the grid.
*  
*  The cell's CSS class and CSS text will be refreshed, to the current values returned by
*  getCellStyle() and getCellCSSText() respectively.
*  
*  The cell's contents (as returned by getCellValue()) will not be refreshed. To
*  refresh both styling and contents, call refreshCell() instead.
* 
* 
* @param {number} row number of cell to refresh
* @param {number} column number of cell to refresh
*/
refreshCellStyle(rowNum:number, colNum:number): void;

/**
*  Refresh an individual cell without redrawing the grid.
*  
*  The cell's value, CSS class, and CSS text will be refreshed, to the current values returned
*  by getCellValue(), getCellStyle() and getCellCSSText() respectively.
* 
* 
* @param {number} row number of cell to refresh
* @param {number} column number of cell to refresh
*/
refreshCell(rowNum:number, colNum:number): void;

/**
*  Refresh an entire row of cells without redrawing the grid.
*  
*  The cells' values, CSS classes, and CSS text will be refreshed, to the current values
*  returned by getCellValue(), getCellStyle() and getCellCSSText() respectively.
* 
* 
* @param {number} row number of cell to refresh
*/
refreshRow(rowNum:number): void;

/**
*  Returns the page offsets and size of the cell at the passed row and column. If auto-sizing 
*  is enabled, sizes are not definitive until the grid has finished drawing, so calling this 
*  method before drawing completes will return the configured column sizes.
* 
* @param {number} row index of the cell
* @param {number} column index of the cell
*/
getCellPageRect(rowNum:number, colNum:number): Array<Partial<Integer>>;

/**
*  Return the left coordinate (in local coordinate space) of a particular column.
* 
* @param {Integer} number of the column
*/
getColumnLeft(colNum:Integer): Integer;

/**
*  Return the left coordinate for a given column number as a GLOBAL coordinate
* 
* @param {Integer} number of the column
*/
getColumnPageLeft(colNum:Integer): Integer;

/**
*  Return the width of a particular column.
* 
* @param {number} number of the column.
*/
getColumnWidth(colNum:number): number;

/**
*  Returns the top coordinate for a given row number, relative to the top of body content. Use
*  GridRenderer.getRowPageTop for a page-relative coordinate.
*  
*  This method is reliable only for rows that are currently drawn, which is generally only rows
*  that are visible in the viewport. If row heights vary (see fixedRowHeights),
*  coordinates for rows that are not currently shown are rough approximations.
* 
* 
* @param {Integer} */
getRowTop(rowNum:Integer): Integer;

/**
*  Returns the Y-coordinate for a given row number as a page-relative coordinate. See
*  GridRenderer.getRowTop.
* 
* 
* @param {Integer} */
getRowPageTop(rowNum:Integer): Integer;

/**
*  Returns the row number of the most recent mouse event.
* 
* @param {Integer=} optional y-coordinate to obtain row number, in lieu of the y            coordinate of the last mouse event
*/
getEventRow(y?:Integer): Integer;

/**
*  Returns the column number of the most recent mouse event.
* 
* @param {Integer=} optional x-coordinate to obtain column number for, in lieu of the x            coordinate of the last mouse event
*/
getEventColumn(x?:Integer): Integer;

/**
*  	Returns the nearest row to the event coordinates
* 
*/
getNearestRowToEvent(): void;

/**
*  	Returns the nearest column to the event coordinates
* 
*/
getNearestColToEvent(): void;

/**
*  Get the rows that are currently visible in the viewport, as an array of 
*  [firstRowNum, lastRowNum]. 
*  
*  If the grid contains no records, will return [-1,-1]. Will also return [-1,-1] if called at
*  an invalid time (for example, data is in the process of being fetched - see
*  ResultSet.lengthIsKnown).
*  
* 
*/
getVisibleRows(): Array<Partial<Integer>>;

/**
*  Get the rows that are currently drawn (exist in the DOM), as an array of [firstRowNum,
*  lastRowNum]. 
*  
*  The drawn rows differ from the GridRenderer.getVisibleRows because of
*  GridRenderer.drawAheadRatio. The drawn rows are the appropriate range to consider if
*  you need to, eg, using GridRenderer.refreshCell to update all the cells in a column.
*  
*  If the grid is undrawn or the GridRenderer.emptyMessage is currently shown, returns
*  [null,null];
* 
* 
*/
getDrawnRows(): Array<any>;

/**
*  Return the record that holds the value for this cell.
*  
*  Implementing getCellRecord is optional: the actual HTML placed into each
*  grid cell comes from getCellValue, and a valid grid can be created without any
*  notion of "records" at all.
*  
*  If you do implement getCellRecord, the value you return is passed to you as the
*  "record" parameter in other methods.
* 
* 
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getCellRecord(rowNum:number, colNum:number): object;

/**
*  Return the HTML to display in this cell. Implementing this is required to get a non-empty
*  grid.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getCellValue(record:ListGridRecord, rowNum:number, colNum:number): string;

/**
*  Given a record displayed in this grid, find and return the rowNum in which the record
*  appears.
*  
*  As with GridRenderer.getCellRecord implementing this method is optional as a valid
*  grid may be created without any notion of records.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
*/
findRowNum(record:ListGridRecord): number;

/**
*  Given a record displayed in this grid, find and return the colNum in which the record
*  appears.
*  
*  As with GridRenderer.getCellRecord implementing this method is optional as a valid
*  grid may be created without any notion of records, or records may not be displayed in a
*  single column (as with the ListGrid class where each record is 
*  displayed in an entire row. 
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
*/
findColNum(record:ListGridRecord): number;

/**
*  Return the base stylename for this cell. Default implementation just returns this.baseStyle.
*  See ListGrid.getCellStyle for a general discussion of how to style cells.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getBaseStyle(record:ListGridRecord, rowNum:number, colNum:number): CSSStyleName;

/**
*  Return CSS text for styling this cell, which will be applied in addition to the CSS class
*  for the cell, as overrides.
*  
*  "CSS text" means semicolon-separated style settings, suitable for inclusion in a CSS
*  stylesheet or in a STYLE attribute of an HTML element.
*  
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getCellCSSText(record:ListGridRecord, rowNum:number, colNum:number): string;

/**
*  Return the height this row should be. Default is this.cellHeight. If
*  GridRenderer.fixedRowHeights is false, the row may be rendered taller than this
*  specified size.
*  
*  If records will be variable height,
*  you should switch on GridRenderer.virtualScrolling.
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number
*/
getRowHeight(record:ListGridRecord, rowNum:number): number;

/**
*  Return how many rows this cell should span. Default is 1. 
*  
*  NOTE: if using horizontal incremental rendering, getRowSpan() may be called for
*  a rowNum in the middle of a spanning cell, and should return the remaining span from
*  that rowNum onward.
*  
*  NOTE: if a cell spans multiple rows, getCellRecord/Style/etc will be called with the topmost
*  row coordinates only.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getRowSpan(record:ListGridRecord, rowNum:number, colNum:number): number;

/**
*  Called when the mouse pointer leaves a cell
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellOut(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when the mouse pointer enters a cell
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellOver(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when the mouse pointer leaves a row
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowOut(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when the mouse pointer enters a row
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowOver(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a cell receives a contextclick event.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellContextClick(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a row receives a contextclick event.
* 
* @param {ListGridRecord} cell record as returned by getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowContextClick(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a cell receives a mousedown event.
*  
* 
* @param {ListGridRecord} cell record as returned by getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellMouseDown(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a row receives a mousedown event.
*  
* 
* @param {ListGridRecord} record object returned from 'getCellRecord()'
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowMouseDown(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a cell receives a mouseup event.
* 
* 
* @param {ListGridRecord} Record object (retrieved from getCellRecord(rowNum, colNum))
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellMouseUp(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a row receives a mouseup event.
* 
* 
* @param {ListGridRecord} Record object returned from getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowMouseUp(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a cell receives a click event.
* 
* 
* @param {ListGridRecord} Record object returned from getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellClick(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a cell receives a double click event.
* 
* 
* @param {ListGridRecord} Record object returned from getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellDoubleClick(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a row receives a click event.
* 
* 
* @param {ListGridRecord} Record object returned from getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowClick(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a row receives a double click event.
* 
* 
* @param {ListGridRecord} Record object returned from getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowDoubleClick(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when the mouse hovers over a cell if this.canHover is true. 
*  Returning false will suppress the hover text from being shown if this.showHover is true.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellHover(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Optional stringMethod to fire when the user hovers over a cell and the value is clipped.
*  If this.showClippedValuesOnHover is true, the default behavior is to show a hover canvas
*  containing the HTML returned by cellValueHoverHTML(). Return false to suppress this default
*  behavior.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellValueHover(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when the mouse hovers over a row if this.canHover is true. 
*  Returning false will suppress the hover text from being shown if this.showHover is true.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowHover(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  StringMethod to dynamically assemble an HTML string to show in a hover window over the
*  appropriate cell/record when this.canHover and this.showHover are both true.
*  Called when the mouse hovers over a cell.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellHoverHTML(record:ListGridRecord, rowNum:number, colNum:number): HTMLString;

/**
*  Returns the HTML that is displayed by the default cellValueHover handler. Return null or
*  an empty string to cancel the hover.
*  
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
* @param {HTMLString} the HTML that would have been displayed by default
*/
cellValueHoverHTML(record:ListGridRecord, rowNum:number, colNum:number, defaultHTML:HTMLString): HTMLString;

/**
*  StringMethod to dynamically create a Canvas-based component to show as a hover window 
*  over the appropriate cell/record when this.canHover and this.showHover are both true and
*  when an override of getCellHoverComponent() is present.
*  Called when the mouse hovers over a cell.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getCellHoverComponent(record:ListGridRecord, rowNum:number, colNum:number): Canvas;

/**
*  Called when (row-based) selection changes within this grid. Note this method fires for
*  each record for which selection is modified - so when a user clicks inside a grid this
*  method will typically fire twice (once for the old record being deselected, and once for
*  the new record being selected).
*  
*  NOTE: For updating other components based on selections or triggering selection-oriented
*  events within an application, see the
*  DataBoundComponent.selectionUpdated event
*  which is likely more suitable. Calls to Selection.getSelection
*  from within this event may not return a valid set of selected records if the event has
*  been triggered by a call to Selection.selectAll or 
*  Selection.deselectAll - in this case use the 
*  DataBoundComponent.selectionUpdated event instead.
* 
* 
* @param {ListGridRecord} record for which selection changed
* @param {boolean} New selection state (true for selected, false for unselected)
*/
selectionChanged(record:ListGridRecord, state:boolean): void;

/**
*  Called when (cell-based) selection changes within this grid.
* 
* 
* @param {Array<any>} Array of cells whos selected state was modified.
*/
cellSelectionChanged(cellList:Array<any>): boolean;



}
/**
* Creates a new GridRenderer
* 
* @param typeCheckedProperties {Partial<GridRenderer>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GridRenderer} The new instance
*/
class GridRenderer{
static create(typeCheckedProperties? : Partial<GridRenderer>, uncheckedProperties?: Object): GridRenderer;







}

/**
* Save all changes in a grid which has auto-saving disabled. Before the save, any pending
*  edit rows are saved to the grid.
*/
interface GridSaveAllEditsTask extends ComponentTask {




}
/**
* Creates a new GridSaveAllEditsTask
* 
* @param typeCheckedProperties {Partial<GridSaveAllEditsTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GridSaveAllEditsTask} The new instance
*/
class GridSaveAllEditsTask{
static create(typeCheckedProperties? : Partial<GridSaveAllEditsTask>, uncheckedProperties?: Object): GridSaveAllEditsTask;







}

/**
* Select or deselect one or more records as specified by criteria. Target records will also
*  be scrolled into view or, for a tree, parent folders will be opened to reveal the node.
*  
*  Task Output:
*  
*   
* - On a select, the set of newly selected records is the output, even if other records
*     are also selected.
*   
* - On a deselect, the entire set of de-selected records is the output.
*  
*/
interface GridSelectRecordsTask extends ComponentTask {
/**
* Criteria defining the records that should be selected or deselected. All records are
*  selected or deselected if not specified.
*  
*  To target a single record just specify criteria for its primary key.
* @type {Criteria}
* @default null
*/
criteria?: Criteria;

/**
* Set to false to clear selection.
* @type {boolean}
* @default null
*/
select?: boolean;

/**
* Should multiple records matching Criteria be affected? If set to
*  false only the first matching record is affected.
* @type {boolean}
* @default null
*/
selectMultiple?: boolean;

/**
* For grids that allow multiple selection, should any existing selection be retained?
*  Only applies when selecting records.
* @type {boolean}
* @default null
*/
keepExistingSelection?: boolean;

/**
* Set to false to prevent the first affected record from being scrolled
*  into view.
* @type {boolean}
* @default null
*/
scrollIntoView?: boolean;





}
/**
* Creates a new GridSelectRecordsTask
* 
* @param typeCheckedProperties {Partial<GridSelectRecordsTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GridSelectRecordsTask} The new instance
*/
class GridSelectRecordsTask{
static create(typeCheckedProperties? : Partial<GridSelectRecordsTask>, uncheckedProperties?: Object): GridSelectRecordsTask;







}

/**
* Sets the edit value of a given field. The targeted row
*  is determined in the following order:
*  
*  
* - the current edit row
*  
* - the first selected row
*  
* - the first row in the grid
*  
* - a new edit row
*  
*/
interface GridSetEditValueTask extends ComponentTask {
/**
* Target field in current edit row to be updated
* @type {number | string}
* @default null
*/
targetField?: number | string;

/**
* Value to assign to GridSetEditValueTask.targetField.
* @type {any}
* @default null
*/
value?: any;





}
/**
* Creates a new GridSetEditValueTask
* 
* @param typeCheckedProperties {Partial<GridSetEditValueTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GridSetEditValueTask} The new instance
*/
class GridSetEditValueTask{
static create(typeCheckedProperties? : Partial<GridSetEditValueTask>, uncheckedProperties?: Object): GridSetEditValueTask;







}

/**
* Transfer selected records from one grid to another.
*/
interface GridTransferDataTask extends ComponentTask {
/**
* Source component from which the record(s) will be transferred.
* @type {GlobalId}
* @default null
*/
sourceComponent?: GlobalId;





}
/**
* Creates a new GridTransferDataTask
* 
* @param typeCheckedProperties {Partial<GridTransferDataTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GridTransferDataTask} The new instance
*/
class GridTransferDataTask{
static create(typeCheckedProperties? : Partial<GridTransferDataTask>, uncheckedProperties?: Object): GridTransferDataTask;







}

/**
* View a record currently selected in some other component. Ignored if nothing is selected.
*/
interface GridViewSelectedDataTask extends ComponentTask {
/**
* Component to pull record for viewing.
* @type {GlobalId}
* @default null
*/
selectionComponentId?: GlobalId;





}
/**
* Creates a new GridViewSelectedDataTask
* 
* @param typeCheckedProperties {Partial<GridViewSelectedDataTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GridViewSelectedDataTask} The new instance
*/
class GridViewSelectedDataTask{
static create(typeCheckedProperties? : Partial<GridViewSelectedDataTask>, uncheckedProperties?: Object): GridViewSelectedDataTask;







}

/**
* Grouping titles that will be displayed when data is grouped
*  in a ListGrid.
*/
interface GroupingMessages extends Class {




}
/**
* Creates a new GroupingMessages
* 
* @param typeCheckedProperties {Partial<GroupingMessages>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {GroupingMessages} The new instance
*/
class GroupingMessages{
static create(typeCheckedProperties? : Partial<GroupingMessages>, uncheckedProperties?: Object): GroupingMessages;
/**
* When a ListGrid is grouped by a date field in 'Upcoming' mode, 
*  this is the group title for all records in which the grouped date field occurs before
*  the current date.
* @type {string}
* @default "Before"
*/
static upcomingBeforeTitle?: string;

/**
* When a ListGrid is grouped by a date field in 'Upcoming' mode, 
*  this is the group title for all records in which the grouped date field occurs today,
*  relative to the current date.
* @type {string}
* @default "Today"
*/
static upcomingTodayTitle?: string;

/**
* When a ListGrid is grouped by a date field in 'Upcoming' mode, 
*  this is the group title for all records in which the grouped date field occurs tomorrow,
*  relative to the current date.
* @type {string}
* @default "Tomorrow"
*/
static upcomingTomorrowTitle?: string;

/**
* When a ListGrid is grouped by a date field in 'Upcoming' mode, 
*  this is the group title for all records in which the grouped date field occurs this week,
*  relative to the current date.
* @type {string}
* @default "This Week"
*/
static upcomingThisWeekTitle?: string;

/**
* When a ListGrid is grouped by a date field in 'Upcoming' mode, 
*  this is the group title for all records in which the grouped date field occurs next week, 
*  relative to the current date.
* @type {string}
* @default "Next Week"
*/
static upcomingNextWeekTitle?: string;

/**
* When a ListGrid is grouped by a date field in 'Upcoming' mode, 
*  this is the group title for all records in which the grouped date field occurs this
*  month, relative to the current date.
* @type {string}
* @default "This Month"
*/
static upcomingThisMonthTitle?: string;

/**
* When a ListGrid is grouped by a date field in 'Upcoming' mode, 
*  this is the group title for all records in which the grouped date field occurs next 
*  month, relative to the current date.
* @type {string}
* @default "Next Month"
*/
static upcomingNextMonthTitle?: string;

/**
* When a ListGrid is grouped by a date field in 'Upcoming' mode, 
*  this is the group title for all records in which the grouped date field occurs in the
*  same year as the current date.
* @type {string}
* @default "This Year"
*/
static upcomingThisYearTitle?: string;

/**
* When a ListGrid is grouped by a date field in 'Upcoming' mode, 
*  this is the group title for all records in which the grouped date field occurs next 
*  year, relative to the current date.
* @type {string}
* @default "Next Year"
*/
static upcomingNextYearTitle?: string;

/**
* When a ListGrid is grouped by a date field in 'Upcoming' mode, 
*  this is the group title for all records in which the grouped date field occurs later 
*  than the end of next year, relative to the current date.
* @type {string}
* @default "Later"
*/
static upcomingLaterTitle?: string;

/**
* Title to use for the menu option which groups a date field by day of week, across all 
*  weeks and years. For example, all values that are on any Tuesday are grouped together.
* @type {string}
* @default "by Day"
*/
static byDayTitle?: string;

/**
* Title to use for the menu option which groups a date field by week number, across all 
*  years. For example, all values that are in Week 30 of any year are grouped together.
* @type {string}
* @default "by Week"
*/
static byWeekTitle?: string;

/**
* Title to use for the menu option which groups a date field by month number, across all 
*  years. For example, all values that are in December of any year are grouped together.
* @type {string}
* @default "by Month"
*/
static byMonthTitle?: string;

/**
* Title to use for the menu option which groups a date field by quarter, across all 
*  years. For example, all values that are in Q4 of any year are grouped together.
* @type {string}
* @default "by Quarter"
*/
static byQuarterTitle?: string;

/**
* Title to use for the menu option which groups a date field by year.
* @type {string}
* @default "by Year"
*/
static byYearTitle?: string;

/**
* Title to use for the menu option which groups a date field by day of month, across all 
*  months and years. For example, all values that are on day 25 of any month in any year 
*  are grouped together.
* @type {string}
* @default "by Day of Month"
*/
static byDayOfMonthTitle?: string;

/**
* Title to use for the menu option which groups a date field by upcoming dates.
* @type {string}
* @default "by Upcoming"
*/
static byUpcomingTitle?: string;

/**
* Title to use for the menu option which groups a date field by specific dates. All 
*  values that are within the 24 hours of a specific date in a given year are 
*  grouped together.
* @type {string}
* @default "by Date"
*/
static byDateTitle?: string;

/**
* Title to use for the menu option which groups a date field by week number and year. All
*  values that are in the same week in a given year are grouped together.
* @type {string}
* @default "by Week and Year"
*/
static byWeekAndYearTitle?: string;

/**
* Title to use for the menu option which groups a date field by month number and year. 
*  All values that are in the same month in a given year are grouped together.
* @type {string}
* @default "by Month and Year"
*/
static byMonthAndYearTitle?: string;

/**
* Title to use for the menu option which groups a date field by quarter and year. All 
*  values that are in the same quarter of a given year are grouped together.
* @type {string}
* @default "by Quarter and Year"
*/
static byQuarterAndYearTitle?: string;

/**
* Title to use for the menu option which groups a date field by specific day of week. All 
*  values that are in the same week and day of a given year are grouped together.
* @type {string}
* @default "by Day of specific Week"
*/
static byDayOfWeekAndYearTitle?: string;

/**
* Title to use for the menu option which groups a date field by specific day of month. All 
*  values that are in the same day and month of a given year are grouped together.
* @type {string}
* @default "by Day of specific Month"
*/
static byDayOfMonthAndYearTitle?: string;

/**
* Title to use for the menu option which groups a time field by hours.
* @type {string}
* @default "by Hours"
*/
static byHoursTitle?: string;

/**
* Title to use for the menu option which groups a time field by minutes.
* @type {string}
* @default "by Minutes"
*/
static byMinutesTitle?: string;

/**
* Title to use for the menu option which groups a time field by seconds.
* @type {string}
* @default "by Seconds"
*/
static bySecondsTitle?: string;

/**
* Title to use for the menu option which groups a time field by milliseconds.
* @type {string}
* @default "by Milliseconds"
*/
static byMillisecondsTitle?: string;

/**
* Title to use for the week number grouping mode
* @type {string}
* @default "Week #"
*/
static weekNumberTitle?: string;

/**
* Suffix to append to the timezoneMinutes grouping mode
* @type {string}
* @default "minutes"
*/
static timezoneMinutesSuffix?: string;

/**
* Suffix to append to the timezoneSeconds grouping mode
* @type {string}
* @default "seconds"
*/
static timezoneSecondsSuffix?: string;








}

/**
* A subclass of Layout that applies a sizing policy along the horizontal axis, interpreting
*  percent and "*" sizes as proportions of the width of the layout. HLayouts will set any members
*  that do not have explicit heights to match the layout.
*/
interface HLayout extends Layout {




}
/**
* Creates a new HLayout
* 
* @param typeCheckedProperties {Partial<HLayout>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HLayout} The new instance
*/
class HLayout{
static create(typeCheckedProperties? : Partial<HLayout>, uncheckedProperties?: Object): HLayout;







}

/**
* A subclass of Layout that simply stacks members on the horizontal axis without trying to
*  manage their width. On the vertical axis, any members that do not have explicit heights will
*  be sized to match the height of the stack.
*/
interface HStack extends Layout {




}
/**
* Creates a new HStack
* 
* @param typeCheckedProperties {Partial<HStack>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HStack} The new instance
*/
class HStack{
static create(typeCheckedProperties? : Partial<HStack>, uncheckedProperties?: Object): HStack;







}

/**
* Use the HTMLFlow component to display HTML content that should expand to its natural size
*  without scrolling. 
*  
*  HTML content can be loaded and reloaded from a URL via the property
*  contentsURL. This method of loading is for simple HTML content
*  only; SmartClient components should be loaded via the ViewLoader class.
*  
*  NOTE: Since the size of an HTMLFlow component is determined by its HTML contents, this
*  component will draw at varying sizes if given content of varying size. When using HTMLFlow
*  components within a Layout, consider what will happen if the HTMLFlow renders at various
*  sizes. An HTMLFlow which can expand should be placed in a container where other components
*  can render smaller, where the container is allowed to scroll, or where there is padding to
*  expand into.
*  
*  HTMLFlow is a DataBoundComponent but only supports one method at this time,
*  HTMLFlow.fetchRelatedData.
*/
interface HTMLFlow extends Canvas, DataBoundComponent {
/**
* The contents of a canvas or label widget. Any HTML string is acceptable.
* @type {HTMLString}
* @default "&amp;nbsp;"
*/
contents?: HTMLString;

/**
* Dynamic contents allows the contents string to be treated as a simple but powerful
*  template. When this attribute is set to true, JavaScript expressions may be embedded 
*  within the contents string, using the format:
*  &#36;{[JavaScript to evaluate]}.
*  
*  For example, to include the current date in a templated message, 
*  canvas.contents could be set to:
*  "Today's date is &lt;b&gt;&#36;{new Date().toUSShortDate()}&lt;/b&gt;"
*  
*  Embedded expressions will be evaluated when the canvas is drawn or redrawn, and
*  the result of the evaluated expression will be displayed to the user. If the
*  expression does not evaluate to a String, the toString() representation
*  of the returned object will be displayed automatically
*  
*  Dynamic expressions are evaluated in the scope of the canvas displaying the content, 
*  so the this keyword may be used within your expression to refer to the 
*  canvas. Developers may also explicitly supply values for variables to be used within
*  the evaluation via the Canvas.dynamicContentsVars property.
*  
*  Notes:
*  
* - Calling markForRedraw() on the canvas will evaluate any embedded expressions.
*  
* - Multiple such expressions may be embedded within the contents string for a component.
*  
* - If an error occurs during evaluation, a warning is logged
*    to the debugging and the error string will be embedded in place 
*    of the expected value in the Canvas.
*  
*  
* @type {boolean}
* @default false
*/
dynamicContents?: boolean;

/**
* URL to load content from.
*  
*  If specified, this component will load HTML content from the specified URL when it is
*  first drawn.
*  
*  This feature relies on the XMLHttpRequest object which can be disabled by end-users in some
*  supported browsers. See platformDependencies for more information.
* @type {URL}
* @default null
*/
contentsURL?: URL;

/**
* HTML to show while content is being fetched, active only if the contentsURL
*  property has been set.
*  Use "&#36;{loadingImage}" to include Canvas.loadingImageSrc.
*  
*  The loading message will show both during the initial load of content, and during reload if
*  the contents are reloaded or the contentsURL changed. For a first-time only loading
*  message, initialize the contents property instead.
*  Note: the loadingMessage is never displayed when loading complete web pages 
*  rather than HTML fragments (see HTMLFlow.contentsType).
* @type {HTMLString}
* @default "&amp;nbsp;${loadingImage}"
*/
loadingMessage?: HTMLString;

/**
* Parameters to be sent to the contentsURL when fetching content.
* @type {object}
* @default null
*/
contentsURLParams?: object;

/**
* Selects the HTTP method that will be used when fetching content. Valid values are "POST"
*  and "GET".
* @type {SendMethod}
* @default "GET"
*/
httpMethod?: SendMethod;

/**
* The default setting of null or 'fragment' indicates that HTML loaded from
*  HTMLFlow.contentsURL is assumed to be an HTML fragment rather than a complete page. Set to
*  "page" to load HTML as a standalone page, via an IFRAME. 
*  
*  contentsType:"page" should only be used for controlled HTML content, and only
*  when such content cannot be delivered as an HTML fragment instead (the default). To
*  dynamically load SmartClient components, use ViewLoader, never this mechanism
*  (click noFrames for why).
*  
*  Loading HTML content as a fragment is less resource intensive and avoids visual artifacts
*  such as translucent media becoming opaque or disappearing when placed over an IFRAME. 
*  
*  Loading third-party, uncontrolled content could lead to the surrounding page disappearing if
*  a user clicks on an HTML link with target=_top.
*  
*  With contentsType:"page", HTMLFlow.loadingMessage is not supported, and only
*  "GET" is supported for HTMLFlow.httpMethod.
*  
*  Note that a native bug has been observed in Internet Explorer version 10 whereby
*  if an HTMLFlow with contentsType set to "page"
*  loads a page containing an HTML &lt;frameset&gt;, when the HTMLFlow
*  is Canvas.hide, it can interfere with the rendering of other elements on
*  the page. Setting Canvas.shrinkElementOnHide to true will work
*  around this behavior.
* @type {ContentsType}
* @default null
*/
contentsType?: ContentsType;

/**
* By default an HTMLFlow will explicitly prevent browser caching.
*  
*  Set to true to allow browser caching if the browser would normally do so, in other
*  words, if the HTTP headers returned with the response indicate that the response can be
*  cached.
* @type {boolean}
* @default false
*/
allowCaching?: boolean;

/**
* If evalScriptBlocks is true, HTMLFlow will pre-process the loaded HTML in order to
*  mimic how the HTML would execute if it were loaded as an independent page or loaded via an
*  IFRAME. 
*  
*  This feature is intended to assist with migrating existing applications to SmartClient.
*  
*  evalScriptBlocks is enabled by default when loading remote content (via
*  HTMLFlow.contentsURL) and disabled by default for content supplied via HTMLFlow.setContents.
*  
*  Note that, if evalScriptBlocks is false, &lt;SCRIPT&gt; blocks will still be detected and disabled
*  to avoid the inconsistent results across different browsers.
*  
*  Only applies when contentsType is not "page".
* @type {boolean}
* @default null
*/
evalScriptBlocks?: boolean;

/**
* If true, SmartClient components created while executing the loaded HTML are captured
*  for rendering inside the HTMLFlow.
*  
*  Only applies when contentsType is not "page".
* @type {boolean}
* @default true
*/
captureSCComponents?: boolean;

/**
* Text selection for copy and paste is enabled by default in the HTMLFlow class.
*  Note that this setting has no impact if ContentsType is set to
*  "page".
*  
*  In this case contents is loaded from a target URL via an IFRAME element, and text selection
*  behavior will be dictated by the loaded HTML.
* @type {boolean}
* @default true
*/
canSelectText?: boolean;

/**
* When this HTMLFlow is focused, causes Ctrl-A / Command-A keypresses to select just
*  the content, as opposed to all content on the screen becoming selected. This HTMLFlow
*  must be Canvas.canFocus in order for this setting to have an effect.
*  
*  Not valid with HTMLFlow.contentsType
*  "page".
*  
* @type {boolean}
* @default null
*/
selectContentOnSelectAll?: boolean;

/**
* When using HTMLFlow.contentsType
*  "page", 
*  
*  sets the &lt;iframe&gt; sandbox attribute to the provided value.
*  
*  Use the value "*ALL*" to cause the "sandbox" attribute to be output with no value
*  (which causes full sandboxing).
*  
*  See any HTML reference for other legal values of the "sandbox" attribute, which allow
*  you to remove individual restrictions on the loaded content.
*  
*  Note that SmartClient simply applies the provided value to the generated &lt;iframe&gt;
*  element and cannot fix bugs or differences in sandbox behavior across different browsers.
* @type {string}
* @default null
*/
iframeSandbox?: string;

/**
* If this component is passed a URL with a protocol that differs from the current page,
*  should it be automatically changed to match the page?
* @type {boolean}
* @default true
*/
autoChangeProtocol?: boolean;



/**
*  Change the URL this component loads content from. Triggers a fetch for content from the new
*  URL.
*  
*  Can also be called with no arguments to reload content from the existing HTMLFlow.contentsURL.
*  
*  This feature relies on the XMLHttpRequest object which can be disabled by end-users in some
*  supported browsers. See platformDependencies for more information.
* 
* 
* @param {URL=} URL to retrieve contents from
* @param {object=} Parameters to send to the contentsURL. Merged with              component.contentsURLParams if both are set.
*/
setContentsURL(url?:URL, params?:object): void;

/**
*  Returns true if this htmlFlow is currently loading content from the server.
*  Note: Does not apply to htmlFlows with HTMLFlow.contentsType set to 
*  "page"
* 
* 
*/
loadingContent(): boolean;

/**
*  Changes the contents of a widget to newContents, an HTML string.
* 
* 
* @param {HTMLString=} an HTML string to be set as the contents of this widget
*/
setContents(newContents?:HTMLString): void;

/**
*  Based on the relationship between the DataSource this component is bound to and the
*  DataSource specified as the "schema" argument, call fetchData() to retrieve records in this
*  data set that are related to the passed-in record.
*  
*  Relationships between DataSources are declared via DataSourceField.foreignKey.
*  
*  For example, given two related DataSources "orders" and "orderItems", where we want to fetch
*  the "orderItems" that belong to a given "order". "orderItems" should declare a field that
*  is a DataSourceField.foreignKey to the "orders" table (for example, it
*  might be named "orderId" with foreignKey="orders.id"). Then, to load the records related to
*  a given "order", call fetchRelatedData() on the component bound to "orderItems", pass the
*  "orders" DataSource as the "schema" and pass a record from the "orders" DataSource as the
*  "record" argument.
*  
*  Note: When you expect a large number of records to be returned it is not recommended to
*  display these in the DetailViewer as it doesn't have the same level of support for large
*  datasets as the ListGrid.
* 
* 
* @param {ListGridRecord} DataSource record
* @param {Canvas | DataSource | GlobalId} schema of the DataSource record, or              DataBoundComponent already bound to that schema
* @param {DSCallback=} callback to invoke on completion
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
*/
fetchRelatedData(record:ListGridRecord, schema:Canvas | DataSource | GlobalId, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
* 
*  This method is called when a transport error occurs. Typically, this is the result of the
*  server returning an HTTP error code such as 404 - document not found. You can inspect the
*  RPCResponse object for the reasons for the error and take appropriate action. Typical
*  properties to look at are rpcResponse.status, and rpcResponse.httpResponseCode.
*  
*  This method is called from the response processing pipeline. If you want to provide your
*  own HTML response that should be rendered into this component as the result of the error,
*  you can do so by setting rpcResponse.data to your HTML string. Returning false from this
*  method suppresses any further response handling. The default implementation of this method
*  causes an error message to be logged to the Developer Console and sets the HTML to the error
*  string.
* 
* 
* @param {RPCRequest} The RPCRequest that was made to the server
* @param {RPCResponse} The RPCResponse that was received
*/
handleError(rpcRequest:RPCRequest, rpcResponse:RPCResponse): boolean;

/**
*  Override to modify the loaded HTML before it is rendered.
* 
* 
* @param {HTMLString} the html as loaded from the server return (HTML) html to be rendered
*/
transformHTML(html:HTMLString): void;

/**
*  StringMethod fired when content is completely loaded in this htmlFlow. Has no default 
*  implementation. May be observed or overridden as a notification type method
*  to fire custom
*  logic when loading completes.
*  
*  Notes:
*  
* - A call to Canvas.setContents 
*  will cause this notification to be fired when
*  the contents have been set. If HTMLFlow.evalScriptBlocks is true, and the HTML passed
*  into setContents() contains any &lt;script src=... &gt;
*  tags, this callback will be fired asynchronously once the scripts have been loaded
*  from the server and executed, as well as having the widget content updated
*  
* - When using HTMLFlow.contentsURL, this does not apply to htmlFlows with
*  HTMLFlow.contentsType set to "page"
* 
* 
*/
contentLoaded(): void;



}
/**
* Creates a new HTMLFlow
* 
* @param typeCheckedProperties {Partial<HTMLFlow>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HTMLFlow} The new instance
*/
class HTMLFlow{
static create(typeCheckedProperties? : Partial<HTMLFlow>, uncheckedProperties?: Object): HTMLFlow;







}

/**
* Use the HTMLPane component to display HTML content in a pane of specified size. If the HTML
*  content is larger than the size of the pane, the pane will provide scrollbars for viewing
*  clipped content.
*  
*  You can set the size of an HTMLPane directly via the width and height properties, or
*  indirectly by placing the HTMLPane in a container component (Layout, Window, 
*  SectionStack, etc) that manages the sizes of its members.
*/
interface HTMLPane extends HTMLFlow {




}
/**
* Creates a new HTMLPane
* 
* @param typeCheckedProperties {Partial<HTMLPane>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HTMLPane} The new instance
*/
class HTMLPane{
static create(typeCheckedProperties? : Partial<HTMLPane>, uncheckedProperties?: Object): HTMLPane;







}

/**
* This class is a synonym for Canvas that can be used to make intent clearer.
*  It is used by some development tools for that purpose.
*  
*  Another synonym is AbsoluteContainer.
*/
interface HandPlacedContainer extends Canvas {




}
/**
* Creates a new HandPlacedContainer
* 
* @param typeCheckedProperties {Partial<HandPlacedContainer>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HandPlacedContainer} The new instance
*/
class HandPlacedContainer{
static create(typeCheckedProperties? : Partial<HandPlacedContainer>, uncheckedProperties?: Object): HandPlacedContainer;







}

/**
* EditProxy that handles Canvas objects when editMode is enabled.
*/
interface HandPlacedContainerEditProxy extends EditProxy {




}
/**
* Creates a new HandPlacedContainerEditProxy
* 
* @param typeCheckedProperties {Partial<HandPlacedContainerEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HandPlacedContainerEditProxy} The new instance
*/
class HandPlacedContainerEditProxy{
static create(typeCheckedProperties? : Partial<HandPlacedContainerEditProxy>, uncheckedProperties?: Object): HandPlacedContainerEditProxy;







}

/**
* This class is a DynamicForm with the default DynamicForm.itemLayout
*  property set to "absolute".
*  
*  Another synonym is AbsoluteForm.
*/
interface HandPlacedForm extends DynamicForm {




}
/**
* Creates a new HandPlacedForm
* 
* @param typeCheckedProperties {Partial<HandPlacedForm>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HandPlacedForm} The new instance
*/
class HandPlacedForm{
static create(typeCheckedProperties? : Partial<HandPlacedForm>, uncheckedProperties?: Object): HandPlacedForm;







}

/**
* Component for showing a header with an optional Header.title. Styling matches
*  a HeaderItem.
*  
*  This header is a special type of ToolStrip, so any ToolStrip controls can
*  be added to the Header. Consider also the SectionStack and
*  SectionHeader if you want expand/collapse behavior or multiple sections.
*/
interface Header extends ToolStrip {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "HeaderEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Title to show for the header. If configured, a Label is automatically created
*  as the first member using the style headerItem to match the styling of
*  the header itself and a HeaderItem.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* Label autoChild for Header.title display.
*  
*  This can be customized via the standard AutoChild pattern.
* @type {Label}
* @default null
*/
titleLabel?: Label;



/**
*  Setter for the Header.title.
* 
* @param {HTMLString} the new title HTML.
*/
setTitle(newTitle:HTMLString): void;



}
/**
* Creates a new Header
* 
* @param typeCheckedProperties {Partial<Header>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Header} The new instance
*/
class Header{
static create(typeCheckedProperties? : Partial<Header>, uncheckedProperties?: Object): Header;







}

/**
* HeaderEditProxy that handles Header objects when editMode is enabled.
*/
interface HeaderEditProxy extends LayoutEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's title.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's title.
* 
* 
* @param {string} the new component title
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new HeaderEditProxy
* 
* @param typeCheckedProperties {Partial<HeaderEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HeaderEditProxy} The new instance
*/
class HeaderEditProxy{
static create(typeCheckedProperties? : Partial<HeaderEditProxy>, uncheckedProperties?: Object): HeaderEditProxy;







}

/**
* FormItem for showing a header within a DynamicForm.
*  
*  Set the defaultValue of this item to the HTML you want to embed in the form.
*/
interface HeaderItem extends FormItem {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "TextItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Header text
* @type {string}
* @default "Header"
*/
defaultValue?: string;

/**
* Don't show a separate title cell for headers
* @type {boolean}
* @default false
*/
showTitle?: boolean;

/**
* Base CSS class for this item
* @type {FormItemBaseStyle}
* @default "headerItem"
*/
textBoxStyle?: FormItemBaseStyle;

/**
* Should the user be able to select the text in this item?
* @type {boolean}
* @default true
*/
canSelectText?: boolean;

/**
* by default, headers span all remaining columns
* @type {Integer | string}
* @default "*"
*/
colSpan?: Integer | string;

/**
* these items are in a row by themselves by default
* @type {boolean}
* @default true
*/
startRow?: boolean;

/**
* these items are in a row by themselves by default
* @type {boolean}
* @default true
*/
endRow?: boolean;

/**
* Alignment of this HeaderItem in its cell.
*  
*  Note: Because a HeaderItem fills its cell by default, if the
*  HeaderItem.applyAlignToText setting is changed to false, then the
*  FormItem.textAlign setting (rather than the align setting)
*  of this HeaderItem should be used to control the alignment of the header text.
* @type {Alignment}
* @default null
*/
align?: Alignment;

/**
* If the FormItem.textAlign is unset, should the HeaderItem.align
*  setting, if set, be used for this HeaderItem's textAlign?
* @type {boolean}
* @default true
*/
applyAlignToText?: boolean;





}
/**
* Creates a new HeaderItem
* 
* @param typeCheckedProperties {Partial<HeaderItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HeaderItem} The new instance
*/
class HeaderItem{
static create(typeCheckedProperties? : Partial<HeaderItem>, uncheckedProperties?: Object): HeaderItem;







}

/**
* A component that connects to a Hibernate configuration and displays the currently-mapped 
*  entities. It also allows you to view the bean properties of a selected entity, and optionally 
*  retrieves and displays the data currently in the corresponding database table.
*  
*  The HibernateBrowser can also create a SmartClient DataSource from any existing Hibernate
*  mapping.
*/
interface HibernateBrowser extends Window {
/**
* Instance of TreeGrid used to display the Hibernate mappings (beans and properties)
* @type {TreeGrid}
* @default null
*/
mappingTree?: TreeGrid;

/**
* Instance of ListGrid used to display the actual data in the database table associated with
*  the mapping selected in the HibernateBrowser.mappingTree.
* @type {ListGrid}
* @default null
*/
dataGrid?: ListGrid;

/**
* A title to show in the header button of the HibernateBrowser's mapping tree.
* @type {string}
* @default "Hibernate Mappings"
*/
title?: string;

/**
* Instance of Button used to cancel this dialog.
* @type {Button}
* @default null
*/
cancelButton?: Button;

/**
* Instance of Button used to continue once a table has been selected.
* @type {Button}
* @default null
*/
selectButton?: Button;

/**
* If set, specifies a substring which must exist in an entity name for it to be included in
*  this HibernateBrowser. If this property is set to a List of strings, entity names are
*  included if they match any one of the strings. The comparison is case-insensitive.
*  
*  For example, includeSubstring: ["Order", "inv"] would match all the following 
*  entity names: "ORDERS", "Inventory", "client_invoicing"
* @type {string | Array<Partial<string>>}
* @default null
*/
includeSubstring?: string | Array<Partial<string>>;

/**
* If set, specifies a substring which must NOT exist in an entity name for it to be included in
*  this HibernateBrowser. If this property is set to a List of strings, entity names are
*  excluded if they match any one of the strings. The comparison is case-insensitive.
*  
*  For example, excludeSubstring: ["Order", "inv"] would exclude all the following 
*  entity names: "ORDERS", "Inventory", "client_invoicing"
*  
*  Note that if you specify both include and exclude criteria and they conflict (ie,
*  according to the criteria you set, an entity should be both included and excluded), 
*  exclude wins.
* @type {string | Array<Partial<string>>}
* @default null
*/
excludeSubstring?: string | Array<Partial<string>>;



/**
*  Returns the DataSource most recently auto-derived by this HibernateBrowser. This will
*  correspond to the currently-selected class mapping.
* 
*/
getGeneratedDataSource(): void;

/**
*  
* 
*/
getResults(): void;



}
/**
* Creates a new HibernateBrowser
* 
* @param typeCheckedProperties {Partial<HibernateBrowser>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HibernateBrowser} The new instance
*/
class HibernateBrowser{
static create(typeCheckedProperties? : Partial<HibernateBrowser>, uncheckedProperties?: Object): HibernateBrowser;







}

/**
* HiddenItems track a value but have no visible appearance and do not take up space in the form
*  layout.
*  
*  When using SmartClient databinding it is usually not necessary to use a HiddenItem, since
*  the DynamicForm will track values for which no actual form control exists, and will submit
*  these 'extra' values when DynamicForm.saveData is called. HiddenItems only apply
*  to forms that are submitted like ordinary HTML forms, via the
*  DynamicForm.submitForm method.
*/
interface HiddenItem extends FormItem {
/**
* we never show a separate title cell for hidden fields
* @type {boolean}
* @default false
*/
showTitle?: boolean;

/**
* hidden fields don't take up any columns
* @type {Integer | string}
* @default "0"
*/
colSpan?: Integer | string;

/**
* hidden fields don't take up any rows
* @type {number}
* @default "0"
*/
rowSpan?: number;

/**
* If this form item has a specified FormItem.optionDataSource, should the
*  item ever perform a fetch against this dataSource to retrieve the related record.
*  
*  This is disabled by default for hiddenItems as there is typically no need to
*  perform a fetch and retrieve a display-field value to show the user for a 
*  hidden item. This does mean that if a developer needs access to the related record
*  for a hidden-item's value, they will need to enable both this setting and
*  FormItem.alwaysFetchMissingValues.
* @type {boolean}
* @default true
*/
fetchMissingValues?: boolean;

/**
* If this form item has a specified FormItem.optionDataSource and 
*  FormItem.fetchMissingValues is true, when the item value changes, a fetch will be
*  performed against the optionDataSource to retrieve the related record 
*  if FormItem.displayField is specified and the new item value is not present in any
*  valueMap explicitly specified on the item.
*  
*  Setting this property to true means that a fetch will occur against the optionDataSource 
*  to retrieve the related record even if FormItem.displayField is unset, or the
*  item has a valueMap which explicitly contains this field's value.
*  
*  An example of a use case where this might be set would be if FormItem.formatValue
*  or FormItem.formatEditorValue were written to display properties from the
*  FormItem.getSelectedRecord.
*  
*  Note - for efficiency we cache the associated record once a fetch has been performed, meaning
*  if the value changes, then reverts to a previously seen value, we do not kick
*  off an additional fetch even if this property is true. If necessary this cache may be
*  explicitly invalidated via a call to FormItem.invalidateDisplayValueCache
*  
*  Note: For hiddenItem HiddenItem.fetchMissingValues is defaulted
*  to false so developers wishing to get access to the record related to
*  the current hiddenItem value would need to explicitly set both that property, and this
*  one to true.
* @type {boolean}
* @default false
*/
alwaysFetchMissingValues?: boolean;





}
/**
* Creates a new HiddenItem
* 
* @param typeCheckedProperties {Partial<HiddenItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HiddenItem} The new instance
*/
class HiddenItem{
static create(typeCheckedProperties? : Partial<HiddenItem>, uncheckedProperties?: Object): HiddenItem;







}

/**
* A Palette with no visible representation that handles programmatic creation of components.
*/
interface HiddenPalette extends Class, Palette {
/**
* A list of PaletteNode for component creation.
* @type {Array<Partial<PaletteNode>>}
* @default null
*/
data?: Array<Partial<PaletteNode>>;





}
/**
* Creates a new HiddenPalette
* 
* @param typeCheckedProperties {Partial<HiddenPalette>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HiddenPalette} The new instance
*/
class HiddenPalette{
static create(typeCheckedProperties? : Partial<HiddenPalette>, uncheckedProperties?: Object): HiddenPalette;







}

/**
* A widget for defining and editing a set of HiliteRule for use by 
*  DataBoundComponent. Presents a list of available fields 
*  and allows editing of simple hilites directly and more complex hilites via 
*  AdvancedHiliteEditors. 
*  
*  Important Note: this class should not be used directly - it is exposed purely for
*  i18nMessages
*/
interface HiliteEditor extends VLayout {
/**
* AutoChild ListGrid showing the list of fields to create hilites for.
*  
*  This component is an AutoChild and as such may be customized via 
*  hiliteEditor.fieldListProperties.
* @type {ListGrid}
* @default null
*/
fieldList?: ListGrid;

/**
* The title for the 'Available Fields' column in the 
*  HiliteEditor.fieldList.
* @type {string}
* @default "Available Fields"
*/
availableFieldsColumnTitle?: string;

/**
* Specifies a list of icons that can be used in Hilite.
*  
*  hiliteIcons should be specified as an Array of SCImgURL.
*  When present, HiliteRules
*  will offer the user a drop down for picking one of these icons.
*  
*  If the user picks an icon, the created hiliting rule will have Hilite.icon set to 
*  the chosen icon. ListGridField.hiliteIconPosition controls where the icon will 
*  appear for that field -- the default is that it appears in front of the normal cell content.
* @type {Array<Partial<SCImgURL>>}
* @default null
*/
hiliteIcons?: Array<Partial<SCImgURL>>;

/**
* AutoChild HiliteRule used to create new simple hilites.
*  
*  This component is an AutoChild and as such may be customized via 
*  hiliteEditor.hiliteRuleProperties.
* @type {HiliteRule}
* @default null
*/
hiliteRule?: HiliteRule;

/**
* AutoChild IButton that opens an AdvancedHiliteEditor to create a new
*  advanced rule.
*  
*  This component is an AutoChild and as such may be customized via 
*  hiliteEditor.addAdvancedRuleButtonProperties.
* @type {StatefulCanvas}
* @default null
*/
addAdvancedRuleButton?: StatefulCanvas;

/**
* The title text for the HiliteEditor.addAdvancedRuleButton
*  button.
* @type {string}
* @default "Add Advanced Rule"
*/
addAdvancedRuleButtonTitle?: string;

/**
* AutoChild ImgButton that saves the hilites in this editor and fires the 
*  HiliteEditor.callback.
*  
*  This component is an AutoChild and as such may be customized via 
*  hiliteEditor.saveButtonProperties.
* @type {StatefulCanvas}
* @default null
*/
saveButton?: StatefulCanvas;

/**
* The title text for the HiliteEditor.saveButton.
* @type {string}
* @default "Save"
*/
saveButtonTitle?: string;

/**
* AutoChild ImgButton that cancels this editor without saving 
*  any changes, firing the HiliteEditor.callback with a null
*  parameter.
*  
*  This component is an AutoChild and as such may be customized via 
*  hiliteEditor.cancelButtonProperties.
* @type {StatefulCanvas}
* @default null
*/
cancelButton?: StatefulCanvas;

/**
* The title text for the HiliteEditor.cancelButton.
* @type {string}
* @default "Cancel"
*/
cancelButtonTitle?: string;

/**
* The callback to fire when HiliteEditor.saveHilites is called.
* @type {Callback}
* @default null
*/
callback?: Callback;



/**
*  Removes the passed HiliteRule from this editor.
* 
* 
* @param {HiliteRule} the hiliteRule to remove
*/
removeRule(hiliteRule:HiliteRule): void;

/**
*  Clear all Hilites from the editor.
* 
* 
*/
clearHilites(): void;

/**
*  Initialize this editor with a set of Hilites.
* 
* 
* @param {Array<Partial<Hilite>>} the array of hilite objects to apply
*/
setHilites(hilites:Array<Partial<Hilite>>): void;

/**
*  Save the set of Hilites and fire the HiliteEditor.callback.
* 
* 
* @param {Callback} the function to call when saving is complete
*/
saveHilites(callback:Callback): void;



}
/**
* Creates a new HiliteEditor
* 
* @param typeCheckedProperties {Partial<HiliteEditor>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HiliteEditor} The new instance
*/
class HiliteEditor{
static create(typeCheckedProperties? : Partial<HiliteEditor>, uncheckedProperties?: Object): HiliteEditor;







}

/**
* A widget for editing the criteria of a single DataBoundComponent hilite. 
*  The default implementation presents a series of FormItem
*  for selecting the various elements of a simple criterion and a foreground or background
*  color. To specify more complex criteria, specify both foreground and background colors or
*  to apply the hilite to multiple fields, you can create an 
*  AdvancedHiliteEditor.
*  
*  Important Note: this class should not be used directly - it is exposed purely for
*  i18nMessages
*/
interface HiliteRule extends HLayout {
/**
* AutoChild FilterClause displaying the FormItem used to 
*  specify the criteria for this HiliteRule.
*  
*  This component is an AutoChild and as such may be customized via 
*  hiliteRule.clauseProperties.
* @type {FilterClause}
* @default null
*/
clause?: FilterClause;

/**
* AutoChild DynamicForm displaying the FormItem used to 
*  specify the hiliting properties of this rule.
*  
*  This component is an AutoChild and as such may be customized via 
*  hiliteRule.hiliteFormProperties.
* @type {DynamicForm}
* @default null
*/
hiliteForm?: DynamicForm;

/**
* The title for the Color picker field.
* @type {string}
* @default "Color"
*/
colorFieldTitle?: string;

/**
* AutoChild Label displaying the human-readable description of an advanced
*  hilite-rule.
*  
*  This component is an AutoChild and as such may be customized via 
*  hiliteRule.advancedClauseLabelProperties.
* @type {Label}
* @default null
*/
advancedClauseLabel?: Label;

/**
* AutoChild ImgButton displayed by an advanced hilite-rule and used to open
*  it for editing in an AdvancedHiliteEditor.
*  
*  This component is an AutoChild and as such may be customized via 
*  hiliteRule.advancedClauseEditButtonProperties.
* @type {ImgButton}
* @default null
*/
advancedClauseEditButton?: ImgButton;

/**
* If true, show a HiliteRule.removeButton for this HiliteRule, allowing it 
*  to be removed.
* @type {boolean}
* @default true
*/
showRemoveButton?: boolean;

/**
* The hover prompt text for the HiliteRule.removeButton.
* @type {string}
* @default "Remove"
*/
removeButtonPrompt?: string;

/**
* The Hilite removal ImgButton that appears before this Hilite if HiliteRule.showRemoveButton 
*  is set.
*  
*  This component is an AutoChild and as such may be customized via 
*  hiliteRule.removeButtonProperties.
* @type {ImgButton}
* @default null
*/
removeButton?: ImgButton;

/**
* The FormItem.title of the 'Text' color picker.
* @type {string}
* @default "Text"
*/
foregroundColorTitle?: string;

/**
* The FormItem.title of the 'Background' color picker.
* @type {string}
* @default "Background"
*/
backgroundColorTitle?: string;

/**
* The FormItem.title of the 'Icon' picker.
* @type {string}
* @default "Icon"
*/
iconFieldTitle?: string;



/**
*  Remove this HiliteRule. Default implementation calls markForDestroy(). 
* 
* 
*/
remove(): void;

/**
*  Return the hilite definition being edited, including criteria and hilite properties.
* 
* 
*/
getHilite(): Hilite;



}
/**
* Creates a new HiliteRule
* 
* @param typeCheckedProperties {Partial<HiliteRule>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {HiliteRule} The new instance
*/
class HiliteRule{
static create(typeCheckedProperties? : Partial<HiliteRule>, uncheckedProperties?: Object): HiliteRule;







}

/**
* The Hover class handles showing a simple SmartClient canvas containing arbitrary HTML, or
*  triggering some other action in response to a user holding the mouse-pointer (or hovering)
*  over a specific widget.
*/
interface Hover extends Class {




}
/**
* Creates a new Hover
* 
* @param typeCheckedProperties {Partial<Hover>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Hover} The new instance
*/
class Hover{
static create(typeCheckedProperties? : Partial<Hover>, uncheckedProperties?: Object): Hover;
/**
* When the Hover canvas is shown by default, should it move as the user moves the
*  mouse pointer?
*  May be overridden by including a moveWithMouse attribute on the 
*  properties block passed to Hover.show
* @type {boolean}
* @default false
*/
static moveWithMouse?: boolean;

/**
* When positioning the hover canvas, this will be the default left offset from the 
*  mousepointer, if no explicit position was passed to the Hover.show method
* @type {number}
* @default "15"
*/
static leftOffset?: number;

/**
* When positioning the hover canvas, this will be the default top offset from the 
*  mousepointer, if no explicit position was passed to the Hover.show method
* @type {number}
* @default "15"
*/
static topOffset?: number;

/**
* When positioning the hover canvas, this will be the minimum offset from page edge.
*  The hover is always positioned not to show outside the page but this offset keeps the
*  canvas from possibly being positioned adjoining the page edge.
* @type {number}
* @default "5"
*/
static edgeOffset?: number;

/**
* Defaults to apply to the Hover canvas shown when the user hovers over some widget.
*  By default this property is set to this object:
*     { defaultWidth:100, 
*      defaultHeight:1,
*      baseStyle:"canvasHover",
*      align:"left",
*      valign:"top",
*      opacity:null
*     }
*  
*  Note that these properties can be overridden by individual widgets showing hovers, by
*  modifying Canvas.hoverWidth, Canvas.hoverHeight, 
*  Canvas.hoverStyle, Canvas.hoverAlign, Canvas.hoverVAlign, 
*  Canvas.hoverOpacity, and Canvas.hoverWrap.
* @type {object}
* @default "{...}"
*/
static hoverCanvasDefaults?: object;

/**
* If Canvas.hoverFocusKey is set, should we show a 
*  Hover.focusKeyHintLabel within the standard
*  hover canvas, prompting the user which focus key to press?
* @type {boolean}
* @default true
*/
static showFocusKeyHint?: boolean;

/**
* Dynamic string message to display within the Hover.focusKeyHintLabel. 
*  
*  The specified hover focus key name is available via the variable ${focusKey}.
* @type {string}
* @default 'Pin Hover: Press &lt;i&gt;${focusKey}&lt;/i&gt;'
*/
static focusKeyHintMessage?: string;

/**
* Width for the Hover.focusKeyHintLabel
* @type {Integer}
* @default "130"
*/
static focusKeyHintLabelWidth?: Integer;

/**
* Height for the Hover.focusKeyHintLabel
* @type {Integer}
* @default "30"
*/
static focusKeyHintLabelHeight?: Integer;

/**
* The Hover.focusKeyHintLabel appears floated over the bottom-right edge of the
*  hover canvas. This property determines how far to offset the label from the
*  edge.
* @type {Integer}
* @default "10"
*/
static focusKeyHintLabelOffset?: Integer;

/**
* The focusKeyHintLabel is automatically generated when showing the standard hover
*  for canvases with a specified Canvas.hoverFocusKey,
*  if Hover.showFocusKeyHint is true.
*  
*  Its content is determined via Hover.focusKeyHintMessage and its sizing and placement
*  may be configured via Hover.focusKeyHintLabelWidth, Hover.focusKeyHintLabelHeight
*  and Hover.focusKeyHintLabelOffset. Other configuration can be applied via the standard
*  autoChildUsage.
* @type {Label}
* @default null
*/
static focusKeyHintLabel?: Label;

/**
* Constructor for the Hover.focusKeyHintLabel
* @type {Class}
* @default "Label"
*/
static focusKeyHintLabelConstructor?: Class;

/**
* Allows interaction with hovers when the cursor is positioned over them.
*  
*  It is recommended to use Canvas.hoverPersist on canvas-instances 
*  for most cases of controlling hover persistence. Changing the system-wide default hover 
*  setting could interfere with features of built-in components, but is allowed for rare 
*  cases like manually showing a hover via Hover.show - you should just restore the 
*  default mode after the hover is hidden.
* @type {HoverPersistMode}
* @default "none"
*/
static persistent?: HoverPersistMode;





/**
*  Displays a standard Hover canvas containing the specified HTML content.
*  This method may also be called to modify the content of the hover if it is already showing.
*  Call Hover.hide to hide the canvas again.
*  A common use case for calling this method is to asynchronously fetch detail data from the
*  server about some component, and display it in the Hover canvas when the data is returned.
*  Note that in this case you will typically need to verify that the user is still hovering 
*  over the component in question before calling Hover.show() - if the user has moved the mouse 
*  off the component, the information will not apply to whatever is now under the mouse. 
*  Suggested approaches for handling this are to either use a Canvas.mouseOut handler
*  to track when the user moves off the component, or checking EventHandler.getTarget
*  as part of the asynchronous callback
*  
*  The default Hover canvas position will be based on the mouse pointer position, adjusted by
*  Hover.leftOffset and Hover.topOffset. If this position would render the
*  Hover canvas partially clipped, it will be automatically modified to ensure the Hover 
*  is entirely visible.
* 
* @param {HTMLString | Canvas} contents for the hover
* @param {Partial<Label>} object containing attributes for managing the hover canvas'  appearance. Valid properties include:  
- left, top, width, height  
- baseStyle  
- opacity  
- wrap   
- moveWithMouse [overrides Hover.moveWithMouse]  
- autoFitWidth: If true, any specified width will be treated as a minimum and the    hover canvas will expand horizontally to fit the content string (without wrapping)    up to the specified autoFitMaxWidth. This setting differs from    simply setting Label.wrap for the hover in that wrapping of    text will occur if the autoFitMaxWidth is exceeded.  
- autoFitMaxWidth: Maximum width to expand to without wrapping (if autoFitWidth is true).  
*/
static show(contents:HTMLString | Canvas, properties:Partial<Label>): void;

/**
*  Hide hoverCanvas shown via Hover.show
* 
*/
static hide(): void;




}

/**
* The IButton widget class is a class that implements the same APIs as the 
*  Button class. Depending on the current skin, IButtons may be
*  on the StretchImgButton component, which renders via images, or may be based on the
*  Button component, which renders via CSS styles.
*/
interface IButton extends Button {




}
/**
* Creates a new IButton
* 
* @param typeCheckedProperties {Partial<IButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {IButton} The new instance
*/
class IButton{
static create(typeCheckedProperties? : Partial<IButton>, uncheckedProperties?: Object): IButton;







}

/**
* IMenuButton based version of the MenuButton class.
*/
interface IMenuButton extends MenuButton {
/**
* Default title for the button.
* @type {string}
* @default "Show Menu"
*/
title?: string;

/**
* Default height of the button.
* @type {number | string}
* @default "22"
*/
height?: number | string;

/**
* Show menu button image (up / down arrowhead) for this menu button.
* @type {boolean}
* @default true
*/
showMenuButtonImage?: boolean;

/**
* Image for menu button indicating that the button expands a menu. This image is shown
*  for menus expanding down from the button.  Menu direction is controlled by
*  MenuButton.showMenuBelow.
* @type {SCImgURL}
* @default "[SKIN]menu_button.gif"
*/
menuButtonImage?: SCImgURL;

/**
* Image for menu button indicating that the button expands a menu. This image is shown
*  for menus expanding up from the button.  Menu direction is controlled by
*  MenuButton.showMenuBelow.
* @type {SCImgURL}
* @default "[SKIN]menu_button_up.gif"
*/
menuButtonImageUp?: SCImgURL;

/**
* If this MenuButton has a specified Canvas.accessKey, underline it
*  in the title of the button by default
* @type {boolean}
* @default true
*/
hiliteAccessKey?: boolean;

/**
* The menu drops down below the menu button.
*  Set to false if the menu should appear above the menu button.
* @type {boolean}
* @default true
*/
showMenuBelow?: boolean;

/**
* The horizontal alignment of this button's menu, in relation to the button. When unset,
*  default behavior is to align the right edges of button and menu if the page is in RTL 
*  mode, and the left edges otherwise.
* @type {Alignment}
* @default null
*/
menuAlign?: Alignment;

/**
* The menu to show.
*  
*  For a menu button with no menu (menu: null) the up/down arrow image can
*  be suppressed by setting
*  MenuButton.showMenuButtonImage: false.
* @type {Menu}
* @default null
*/
menu?: Menu;

/**
* Allows you to specify an animation effect to apply to the menu when it is being shown.
*  Valid options are "none" (no animation), "fade", "slide" and "wipe".
*  If unspecified falls through to menu.showAnimationEffect
* @type {string}
* @default null
*/
menuAnimationEffect?: string;

/**
* If this menuButton is Canvas.destroy, should it also destroy
*  its MenuButton.menu?
* @type {boolean}
* @default true
*/
autoDestroyMenu?: boolean;



/**
*  Setter for the 'showMenuButtonImage' property - shows/hides the menu button image
*  at runtime.
* 
* 
* @param {boolean} Should the image be shown
*/
setShowMenuButtonImage(show:boolean): void;

/**
*  Setter for the 'showMenuButtonBelow' property - determines whether the menu will be
*  shown above or below the MenuButton.
* 
* @param {boolean} True if the menu should be shown below the MenuButton.
*/
setShowMenuBelow(below:boolean): void;



}
/**
* Creates a new IMenuButton
* 
* @param typeCheckedProperties {Partial<IMenuButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {IMenuButton} The new instance
*/
class IMenuButton{
static create(typeCheckedProperties? : Partial<IMenuButton>, uncheckedProperties?: Object): IMenuButton;







}

/**
* Subclass of PickTreeItem which shows an IMenuButton rather than a
*  simple MenuButton as it's main button.
*/
interface IPickTreeItem extends PickTreeItem {
/**
* This item is an AutoChild generated Canvas displayed by
*  the IPickTreeItem and is an instance of ITreeMenuButton by default.
* @type {Canvas}
* @default null
*/
button?: Canvas;





}
/**
* Creates a new IPickTreeItem
* 
* @param typeCheckedProperties {Partial<IPickTreeItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {IPickTreeItem} The new instance
*/
class IPickTreeItem{
static create(typeCheckedProperties? : Partial<IPickTreeItem>, uncheckedProperties?: Object): IPickTreeItem;







}

/**
* Button used to display a hierarchical Menu group for representing / selecting tree data.
*  This is derived from the MenuButton and is StretchImgButton based.
*  
*  Important Note: this class should not be used directly - it is exposed purely for
*  i18nMessages
*/
interface ITreeMenuButton extends TreeMenuButton {




}
/**
* Creates a new ITreeMenuButton
* 
* @param typeCheckedProperties {Partial<ITreeMenuButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ITreeMenuButton} The new instance
*/
class ITreeMenuButton{
static create(typeCheckedProperties? : Partial<ITreeMenuButton>, uncheckedProperties?: Object): ITreeMenuButton;







}

/**
* A simple subclass of RibbonButton.
* @deprecated * this is an old synonym for RibbonButton, scheduled for removal in the next release

*/
interface IconButton extends RibbonButton {




}
/**
* Creates a new IconButton
* 
* @param typeCheckedProperties {Partial<IconButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {IconButton} The new instance
*/
class IconButton{
static create(typeCheckedProperties? : Partial<IconButton>, uncheckedProperties?: Object): IconButton;







}

/**
* A specialized subclass of ImgButton designed to show an icon that launches a
*  Canvas.contextMenu when clicked. The icon is specified as the
*  IconImgButton.src property.
*/
interface IconImgButton extends ImgButton {
/**
* If true, this widget will fire Canvas.showContextMenu to
*  show the Canvas.contextMenu if one is defined, rather than
*  Canvas.click, when the left mouse is clicked.
*  
*  Note that this property has a different interpretation in IconButton as
*  RibbonButton.showMenuOnClick.
* @type {boolean}
* @default true
*/
showMenuOnClick?: boolean;

/**
* The base filename or stateful image configuration for the image. 
*  Note that as the StatefulCanvas.state 
*  of the component changes, the image displayed will be updated as described in
*  statefulImages.
* @type {SCImgURL | SCStatefulImgConfig}
* @default "{_base:"[SKIN]/actions/edit.png"}"
*/
src?: SCImgURL | SCStatefulImgConfig;





}
/**
* Creates a new IconImgButton
* 
* @param typeCheckedProperties {Partial<IconImgButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {IconImgButton} The new instance
*/
class IconImgButton{
static create(typeCheckedProperties? : Partial<IconImgButton>, uncheckedProperties?: Object): IconImgButton;







}

/**
* A simple subclass of RibbonMenuButton.
* @deprecated * this is an old synonym for RibbonMenuButton, scheduled for removal in the next release

*/
interface IconMenuButton extends IconButton {




}
/**
* Creates a new IconMenuButton
* 
* @param typeCheckedProperties {Partial<IconMenuButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {IconMenuButton} The new instance
*/
class IconMenuButton{
static create(typeCheckedProperties? : Partial<IconMenuButton>, uncheckedProperties?: Object): IconMenuButton;







}

/**
* The Img widget class implements a simple widget that displays a single image.
*/
interface Img extends StatefulCanvas {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "ImgEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* The value of this attribute is specified as the value of the 'name' attribute in the
*  resulting HTML.
*  
*  Note: this attribute is ignored if the imageType is set to "tile"
* @type {string}
* @default "main"
*/
name?: string;

/**
* Should we visibly change state when the mouse goes over this object?
*  
*  This will impact the StatefulCanvas.baseStyle of the component on
*  roll over. It may also impact the Img.src - see
*  also Img.showImageRollOver.
* @type {boolean}
* @default false
*/
showRollOver?: boolean;

/**
* Should the image be updated on rollOver as described in statefulImages?
*  
*  If not explicitly set, behavior is as follows:
*  If Img.src is specified as a string, Img.showRollOver will be used to 
*  determine whether to show a roll-over image.
*  If Img.src is specified as a SCStatefulImgConfig, the appropriate 
*  SCStatefulImgConfig.Over state image will be displayed if defined.
* @type {boolean}
* @default null
*/
showImageRollOver?: boolean;

/**
* Should we visibly change state when the canvas receives focus? If
*  StatefulCanvas.showFocusedAsOver is true, then "over"
*  will be used to indicate focus. Otherwise a separate "focused" state
*  will be used.
*  
*  This will impact the StatefulCanvas.baseStyle of the component on
*  focus. It may also impact the Img.src - see
*  also Img.showImageFocused.
* @type {boolean}
* @default false
*/
showFocused?: boolean;

/**
* Should the image be updated on focus as described in statefulImages?
*  
*  If not explicitly set, behavior is as follows:
*  If Img.src is specified as a string, Img.showFocused will be used to determine
*  whether to show a focused image.
*  If Img.src is specified as a SCStatefulImgConfig, the appropriate 
*  SCStatefulImgConfig.Over state image will be displayed if defined.
*  
*  Note that if Img.src is defined as a string, the "Over" media may be used 
*  to indicate a focused state. See Img.showFocusedAsOver and Img.showImageFocusedAsOver.
*  This is not the case for components with Img.src defined as a SCStatefulImgConfig
*  configuration.
* @type {boolean}
* @default null
*/
showImageFocused?: boolean;

/**
* If StatefulCanvas.showFocused is true for this widget, should the
*  "over" state be used to indicate the widget as focused. If set to false,
*  a separate "focused" state will be used.
*  
*  This property effects the css styling for the focused state.
*  If Img.src is specified as a string it will also cause the "Over" media to be
*  displayed to indicate focus, unless explicitly overridden by 
*  Img.showImageFocusedAsOver. Note that this has no impact on the
*  image to be displayed if Img.src is specified as a SCStatefulImgConfig.
* @type {boolean}
* @default true
*/
showFocusedAsOver?: boolean;

/**
* If Img.src is defined as a string, and this component is configured to
*  Img.showImageFocused, this property will cause the 
*  "over" state image to be used to indicate focused state.
*  (If unset, Img.showFocusedAsOver will be consulted instead).
*  
*  Note that this has no impact on the
*  image to be displayed if Img.src is specified as a SCStatefulImgConfig.
* @type {boolean}
* @default null
*/
showImageFocusedAsOver?: boolean;

/**
* Should we visibly change state when the mouse goes down in this object?
*  This will impact the StatefulCanvas.baseStyle of the component on
*  mouse down. It may also impact the Img.src - see
*  also Img.showImageDown.
* @type {boolean}
* @default false
*/
showDown?: boolean;

/**
* Should the image be updated on mouse down as described in statefulImages?
*  
*  If not explicitly set, behavior is as follows:
*  If Img.src is specified as a string, Img.showDown will be used to 
*  determine whether to show a mouse down image.
*  If Img.src is specified as a SCStatefulImgConfig, the appropriate 
*  SCStatefulImgConfig.Down state image will be displayed if defined.
* @type {boolean}
* @default null
*/
showImageDown?: boolean;

/**
* Should we visibly change state when disabled?
*  
*  This will impact the StatefulCanvas.baseStyle of the component 
*  when disabled. It may also impact the Img.src - see
*  also Img.showImageDisabled.
* @type {boolean}
* @default true
*/
showDisabled?: boolean;

/**
* Should the image be updated when disabled as described in statefulImages?
*  
*  If not explicitly set, behavior is as follows:
*  If Img.src is specified as a string, Img.showDisabled will be used to 
*  determine whether to show a disabled image.
*  If Img.src is specified as a SCStatefulImgConfig, the appropriate 
*  SCStatefulImgConfig.Disabled state image will be displayed if defined.
* @type {boolean}
* @default null
*/
showImageDisabled?: boolean;

/**
* The base filename or stateful image configuration for the image. 
*  Note that as the StatefulCanvas.state 
*  of the component changes, the image displayed will be updated as described in
*  statefulImages.
* @type {SCImgURL | SCStatefulImgConfig}
* @default "[SKINIMG]blank.gif"
*/
src?: SCImgURL | SCStatefulImgConfig;

/**
* If specified this property will be included as the alt text for the image HMTL
*  element. This is useful for improving application accessibility.
*  
*  altText and hover prompt / tooltip behavior: Note that some
*  browsers, including Internet Explorer 9, show a native hover tooltip containing the 
*  img tag's alt attribute. Developers should not rely on this behavior to show
*  the user a hover prompt - instead the Img.prompt attribute should be used.
*  To set alt text and ensure a hover prompt shows up in all browsers, developers may
*  set Img.prompt and altText to the same value. If both 
*  these attributes are set, the standard SmartClient prompt behavior will show a hover
*  prompt in most browsers, but will be suppressed for browsers where a native tooltip 
*  is shown for altText. Note that setting altText and prompt to
*  different values is not recommended - the prompt value will be ignored in favor of the
*  altText in this case.
* @type {string}
* @default null
*/
altText?: string;

/**
* Prompt displayed in hover canvas if Canvas.showHover is true.
* @type {HTMLString}
* @default null
*/
prompt?: HTMLString;

/**
* Setting this attribute configures an image map for this image. The value is expected as a
*  sequence of &lg;AREA&gt tags - e.g:
*  
*  Img.create({ 
*    src: "myChart.gif",
*    activeAreaHTML:
*      "&lt;AREA shape='rect' coords='10,50,30,200' title='30' href='javascript:alert(\"30 units\")'&gt;" +
*      "&lt;AREA shape='rect' coords='50,90,80,200' title='22' href='javascript:alert(\"22 units\")'&gt;"
*  });
*  
*  Implementation notes:
*  
*  
* - Quotes in the activeAreaHTML must be escaped or alternated appropriately.
*  
* - Image maps do not stretch to fit scaled images. You must ensure that the dimensions of
*  your Img component match the anticipated width and height of your image map (which will typically
*  match the native dimensions of your image). 
*  
* - To change the image map of an existing Img component, first set yourImg.activeAreaHTML,
*  then call yourImg.markForRedraw(). Calls to yourImg.setSrc() will not automatically update the
*  image map. 
*  
* - activeAreaHTML is not supported on tiled Img components (imageType:"tile"). 
*  
* - Native browser support for image map focus/blur, keyboard events, and certain AREA tag
*  attributes (eg NOHREF, DEFAULT...) varies by platform. If your image map HTML uses attributes
*  beyond the basics (shape, coords, href, title), you should test on all supported browsers to
*  ensure that it functions as expected.
*  
* @type {string}
* @default null
*/
activeAreaHTML?: string;

/**
* Indicates whether the image should be tiled/cropped, stretched, or centered when the
*      size of this widget does not match the size of the image. 
*      CENTER shows the image in it's natural size, but can't do so while the 
*      transparency fix is active for IE. The transparency fix can be manually disabled
*      by setting Img.usePNGFix to false.
*      See ImageStyle for further details.
* @type {ImageStyle}
* @default "stretch"
*/
imageType?: ImageStyle;

/**
* Explicit size for the image, for Img.imageType settings that would normally use the
*  image's natural size (applies to Img.imageType "center" and "normal" only).
* @type {Integer}
* @default null
*/
imageHeight?: Integer;

/**
* Explicit size for the image, for Img.imageType settings that would normally use the
*  image's natural size (applies to Img.imageType "center" and "normal" only).
* @type {Integer}
* @default null
*/
imageWidth?: Integer;

/**
* Convenience for setting height and width to the same value, at init time only
* @type {number}
* @default null
*/
size?: number;

/**
* Determines whether any specified StatefulCanvas.getTitle will be 
*  displayed for this component.
*  Applies to Image-based components only, where the title will be rendered out in a label
*  floating over the component
* @type {boolean}
* @default false
*/
showTitle?: boolean;

/**
* If false, never apply the png fix needed in Internet Explorer to make png transparency
*  work correctly.
* @type {boolean}
* @default true
*/
usePNGFix?: boolean;



/**
*  Change the style of image rendering.
* 
* 
* @param {ImageStyle} new style of image rendering
*/
setImageType(imageType:ImageStyle): void;

/**
*  Changes the URL of this image and redraws it.
*  
*  Does nothing if the src has not changed - if src has not changed but other
*  state has changed such that the image needs updating, call Img.resetSrc instead.
* 
* 
* @param {SCImgURL} new URL for the image
*/
setSrc(URL:SCImgURL): void;

/**
*  Refresh the image being shown. Call this when the Img.src attribute has not changed, but
*  other state that affects the image URL (such as being selected) has changed.
* 
* 
*/
resetSrc(): void;

/**
*  If this.showHover is true, when the user holds the mouse over this Canvas for
*  long enough to trigger a hover event, a hover canvas is shown by default. This method returns
*  the contents of that hover canvas.
*  
*  Overridden from Canvas: 
*  If Img.prompt is specified, and Img.altText is unset, default implementation is unchanged -
*  the prompt text will be displayed in the hover.
*  If Img.altText and Img.prompt are set this method will return null to suppress
*  the standard hover behavior in browsers where the alt attribute on an img tag causes
*  a native tooltip to appear, such as Internet Explorer.
*  On other browsers the altText value will be returned.
* 
* 
*/
getHoverHTML(): string;



}
/**
* Creates a new Img
* 
* @param typeCheckedProperties {Partial<Img>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Img} The new instance
*/
class Img{
static create(typeCheckedProperties? : Partial<Img>, uncheckedProperties?: Object): Img;







}

/**
* A Img that behaves like a button, going through up/down/over state transitions in response to
*  user events. Supports an optional title, and will auto-size to accommodate the title text if
*  overflow is set to "visible".
*  
*  Example uses are Window minimize/close buttons.
*/
interface ImgButton extends Img {
/**
* The title HTML to display in this button.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* If set to true, if the StatefulCanvas.title of this button contains the
*  specified Canvas.accessKey, when the title is displayed to the user
*  it will be modified to include HTML to underline the accessKey.
*  Note that this property may cause titles that include HTML (rather than simple strings)
*  to be inappropriately modified, so should be disabled if your title string includes
*  HTML characters.
* @type {boolean}
* @default null
*/
hiliteAccessKey?: boolean;

/**
* If true and the title is clipped, then a hover containing the full title of this button
*  is enabled.
* @type {boolean}
* @default false
*/
showClippedTitleOnHover?: boolean;

/**
* Optional icon to be shown with the button title text. 
*  
*  Specify as the partial URL to an image, relative to the imgDir of this component.
*  A sprited image can be specified using the SCSpriteConfig format.
*  
*  Note that the string "blank" is a valid setting for this attribute and will always 
*  result in the system blank image, with no state suffixes applied. Typically, this 
*  might be used when an iconStyle is also specified and the iconStyle renders the icon via 
*  a stateful background-image or other CSS approach.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* Size in pixels of the icon image.
*  
*  The StatefulCanvas.iconWidth and StatefulCanvas.iconHeight
*  properties can be used to configure width and height separately.
*  
*  Note: When configuring the properties of a StatefulCanvas (or derivative)
*  AutoChild, it is best to set the iconWidth and iconHeight
*  to the same value rather than setting an iconSize. This is because certain
*  skins or customizations thereto might set the iconWidth and iconHeight,
*  making the customization of the AutoChild's iconSize ineffective.
* @type {Integer}
* @default "16"
*/
iconSize?: Integer;

/**
* Width in pixels of the icon image.
*  
*  If unset, defaults to StatefulCanvas.iconSize.
* @type {Integer}
* @default null
*/
iconWidth?: Integer;

/**
* Height in pixels of the icon image.
*  
*  If unset, defaults to StatefulCanvas.iconSize.
* @type {Integer}
* @default null
*/
iconHeight?: Integer;

/**
* If this button is showing an icon should it appear to the left or right of the title?
*  valid options are "left" and "right".
* @type {string}
* @default "left"
*/
iconOrientation?: string;

/**
* If this button is showing an icon should it be right or left aligned?
* @type {string}
* @default null
*/
iconAlign?: string;

/**
* Pixels between icon and title text.
* @type {Integer}
* @default "6"
*/
iconSpacing?: Integer;

/**
* If using an icon for this button, whether to switch the icon image if the button becomes
*  disabled.
* @type {boolean}
* @default true
*/
showDisabledIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image on mouse rollover.
* @type {boolean}
* @default false
*/
showRollOverIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the button
*  receives focus.
*  
*  If StatefulCanvas.showFocusedAsOver is true, the "Over" icon will be
*  displayed when the canvas has focus, otherwise a separate "Focused" icon
*  will be displayed
* @type {boolean}
* @default false
*/
showFocusedIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the mouse goes
*  down on the button.
* @type {boolean}
* @default false
*/
showDownIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the button
*  becomes selected.
* @type {boolean}
* @default false
*/
showSelectedIcon?: boolean;

/**
* If true, ignore the specified size of this widget and always size just large
*  enough to accommodate the title. If setWidth() is explicitly called on an
*  autoFit:true button, autoFit will be reset to false.
*  
*  Note that for StretchImgButton instances, autoFit will occur horizontally only, as 
*  unpredictable vertical sizing is likely to distort the media. If you do want vertical 
*  auto-fit, this can be achieved by simply setting a small height, and having 
*  overflow:"visible"
* @type {boolean}
* @default null
*/
autoFit?: boolean;

/**
* Size for this component's horizontal dimension. See Canvas.width for more
*  details.
*  
*  Note that if StatefulCanvas.autoFit is set, this property will be ignored so that the widget
*  is always sized just large enough to accommodate the title.
* @type {number | string}
* @default null
*/
width?: number | string;

/**
* Size for this component's vertical dimension. See Canvas.height for more
*  details.
*  
*  Note that if StatefulCanvas.autoFit is set on non-StretchImgButton instances, this
*  property will be ignored so that the widget is always sized just large enough to
*  accommodate the title.
* @type {number | string}
* @default null
*/
height?: number | string;

/**
* Base CSS style className applied to the component.
*  
*  Note that if specified, this property takes precedence over any specified
*  StatefulCanvas.styleName. If unset, the styleName will be used as a 
*  default baseStyle value.
*  
*  As the component changes StatefulCanvas.state and/or is selected, 
*  suffixes will be added to the base style. In some cases more than one suffix will 
*  be appended to reflect a combined state ("Selected" + "Disabled", for example).
*  
*  See StatefulCanvas.getStateSuffix for a description of the default set
*  of suffixes which may be applied to the baseStyle
*  
*  Rotated Titles
*  
*  The Framework doesn't have built-in support for rotating button titles in a fashion
*  similar to FacetChart.rotateLabels. However, you can manually configure
*  a button to render with a rotated title by applying custom CSS via this property.
*  
*  For example, given a button with a height of 120 and a width of 48, if you
*  copied the existing buttonXXX style declarations from skin_styles.css as new,
*  rotatedTitleButtonXXX declarations, and then added the lines:
*  
*    -ms-transform:   translate(-38px,0px) rotate(270deg);
*    -webkit-transform: translate(-38px,0px) rotate(270deg);
*    transform:     translate(-38px,0px) rotate(270deg);
*    overflow: hidden;
*    text-overflow: ellipsis;
*    width:116px;
*  in the declaration section beginning:
*  
*  .rotatedTitleButton,
*  .rotatedTitleButtonSelected,
*  .rotatedTitleButtonSelectedOver,
*  .rotatedTitleButtonSelectedDown,
*  .rotatedTitleButtonSelectedDisabled,
*  .rotatedTitleButtonOver,
*  .rotatedTitleButtonDown,
*  .rotatedTitleButtonDisabled {
*  then applying that style to the button with Canvas.overflow: "clip_h"
*  would yield a vertically-rendered title with overflow via ellipsis as expected, and also
*  wrap with Button.wrap.
*  
*  Note that:
*  
* - The explicit width applied via CSS is needed because rotated
*  elements don't inherit dimensions in their new orientation from the DOM - 
*  the transform/rotation occurs independently of layout.
*  
* - The translation transform required along the x-axis is roughly
*  (width - height) / 2, but may need slight offsetting for optimal centering.
*  
* - We've explicitly avoided describing an approach based on CSS "writing-mode", since
*  support is incomplete and bugs are present in popular browsers such as Firefox and
*  Safari that would prevent it from being used without Framework assistance.
*  
*  Note on css-margins: Developers should be aware that the css "margin" property is unreliable for
*  certain subclasses of StatefulCanvas, including Button. Developers may use 
*  the explicit Canvas.margin property to specify button margins, or for a 
*  button within a layout, consider the layout properties Layout.layoutMargin,
*  Layout.membersMargin
* @type {CSSStyleName}
* @default "imgButton"
*/
baseStyle?: CSSStyleName;

/**
* Whether this component is selected. For some components, selection affects appearance.
* @type {boolean}
* @default false
*/
selected?: boolean;

/**
* String identifier for this canvas's mutually exclusive selection group.
* @type {string}
* @default null
*/
radioGroup?: string;

/**
* Behavior on state changes -- BUTTON, RADIO or CHECKBOX
* @type {SelectionType}
* @default "button"
*/
actionType?: SelectionType;

/**
* Current "state" of this widget. The state setting is automatically updated as the 
*  user interacts with the component (see StatefulCanvas.showRollOver, 
*  StatefulCanvas.showDown, StatefulCanvas.showDisabled).
*  
*  StatefulCanvases will have a different appearance based
*  on their current state. 
*  By default this is handled by changing the css className applied to
*  the StatefulCanvas - see StatefulCanvas.baseStyle and 
*  StatefulCanvas.getStateSuffix for a description of how this is done.
*  
*  For Img or StretchImg based subclasses of StatefulCanvas, the 
*  appearance may also be updated by changing the src of the rendered image. See
*  Img.src and StretchImgButton.src for a description of how the URL 
*  is modified to reflect the state of the widget in this case.
* @type {State}
* @default ""
*/
state?: State;

/**
* Should we visibly change state when disabled?
* @type {boolean}
* @default true
*/
showDisabled?: boolean;

/**
* Should we visibly change state when the mouse goes down in this object?
* @type {boolean}
* @default false
*/
showDown?: boolean;

/**
* Should we visibly change state when the canvas receives focus? Note that by default the
*  over state is used to indicate focus.
* @deprecated * as of SmartClient version 6.1 in favor of StatefulCanvas.showFocused

* @type {boolean}
* @default false
*/
showFocus?: boolean;

/**
* Should we visibly change state when the canvas receives focus? If
*  StatefulCanvas.showFocusedAsOver is true, then "over"
*  will be used to indicate focus. Otherwise a separate "focused" state
*  will be used.
* @type {boolean}
* @default false
*/
showFocused?: boolean;

/**
* Should we visibly change state when the mouse goes over this object?
* @type {boolean}
* @default false
*/
showRollOver?: boolean;

/**
* Horizontal alignment of this component's title.
* @type {Alignment}
* @default "center"
*/
align?: Alignment;

/**
* Vertical alignment of this component's title.
* @type {VerticalAlignment}
* @default "center"
*/
valign?: VerticalAlignment;

/**
* Determines whether any specified StatefulCanvas.getTitle will be 
*  displayed for this component.
*  Applies to Image-based components only, where the title will be rendered out in a label
*  floating over the component
* @type {boolean}
* @default false
*/
showTitle?: boolean;

/**
* Horizontal padding to be applied to this widget's label. If this value is null, 
*  the label will be given a horizontal padding of zero.
*  
*  The specified amount of padding is applied to the left and right edges of the button, so
*  the total amount of padding is 2x the specified value.
* @type {number}
* @default null
*/
labelHPad?: number;

/**
* Vertical padding to be applied to this widget's label. If this value is null, 
*  the label will be given a vertial padding of zero.
*  
*  The specified amount of padding is applied to the top and bottom edges of the button, so
*  the total amount of padding is 2x the specified value.
* @type {number}
* @default null
*/
labelVPad?: number;

/**
* The base filename or stateful image configuration for the image. 
*  Note that as the StatefulCanvas.state 
*  of the component changes, the image displayed will be updated as described in
*  statefulImages.
* @type {SCImgURL | SCStatefulImgConfig}
* @default "[SKIN]/ImgButton/button.png"
*/
src?: SCImgURL | SCStatefulImgConfig;



/**
*  Return the title - HTML drawn inside the component.
*  
*  Default is to simply return this.title.
* 
*/
getTitle(): HTMLString;

/**
*  Setter for the StatefulCanvas.title.
* 
* @param {HTMLString} the new title HTML.
*/
setTitle(newTitle:HTMLString): void;

/**
*  Changes the orientation of the icon relative to the text of the button.
* 
* 
* @param {string} The new orientation of the icon relative to the text of the button.
*/
setIconOrientation(orientation:string): void;

/**
*  Change the icon being shown next to the title text.
* 
* @param {SCImgURL} URL of new icon
*/
setIcon(icon:SCImgURL): void;

/**
*  Setter method for the StatefulCanvas.autoFit property. Pass in true or false to turn
*  autoFit on or off. When autoFit is set to false, canvas will be resized to
*  it's previously specified size.
* 
* @param {boolean} New autoFit setting.
*/
setAutoFit(autoFit:boolean): void;

/**
*  Sets the base CSS style. As the component changes state and/or is selected, suffixes will be
*  added to the base style.
* 
* @param {CSSStyleName} new base style
*/
setBaseStyle(style:CSSStyleName): void;

/**
*  Select this object.
* 
*/
select(): void;

/**
*  Select this object.
* 
*/
deselect(): void;

/**
*  Find out if this object is selected
* 
*/
isSelected(): boolean;

/**
*  Select this object.
* 
*/
setSelected(): void;

/**
*  Add this widget to the specified mutually exclusive selection group with the ID
*  passed in.
*  Selecting this widget will then deselect any other StatefulCanvases with the same
*  radioGroup ID.
*  StatefulCanvases can belong to only one radioGroup, so this method will remove from 
*  any other radiogroup of which this button is already a member.
* 
* @param {string} - ID of the radiogroup to which this widget should be added
*/
addToRadioGroup(groupID:string): void;

/**
*  Remove this widget from the specified mutually exclusive selection group with the ID
*  passed in.
*  No-op's if this widget is not a member of the groupID passed in.
*  If no groupID is passed in, defaults to removing from whatever radioGroup this widget
*  is a member of.
* 
* @param {string=} - optional radio group ID (to ensure the widget is removed                    from the appropriate group.
*/
removeFromRadioGroup(groupID?:string): void;

/**
*  Update the 'actionType' for this canvas (radio / checkbox / button)
*  If the canvas is currently selected, and the passed in actionType is 'button'
*  this method will deselect the canvas.
* 
*/
setActionType(): void;

/**
*  Return the 'actionType' for this canvas (radio / checkbox / button)
* 
*/
getActionType(): void;

/**
*  Sets the StatefulCanvas.state of this object, changing its appearance.
*  Note: newState cannot be
*  "Disabled"
*  
*  if StatefulCanvas.showDisabled is false.
* 
* 
* @param {State} the new state.
*/
setState(newState:State): void;

/**
*  Enable or disable this object
* 
* @param {boolean} true if this widget is to be disabled
*/
setDisabled(disabled:boolean): void;

/**
*  Return the state of this StatefulCanvas
* 
*/
getState(): State;

/**
*  This property contains the default 'action' for the Button to fire when activated.
* 
*/
action(): void;

/**
*  Is the title of this button clipped?
* 
*/
titleClipped(): boolean;

/**
*  Returns the HTML that is displayed by the default ImgButton.titleHover
*  handler. Return null or an empty string to cancel the hover.
*  
* 
* @param {HTMLString} the HTML that would have been displayed by default
*/
titleHoverHTML(defaultHTML:HTMLString): HTMLString;

/**
*  Optional stringMethod to fire when the user hovers over this button and the title is
*  clipped. If ImgButton.showClippedTitleOnHover is true, the default behavior is to
*  show a hover canvas containing the HTML returned by ImgButton.titleHoverHTML.
*  Return false to suppress this default behavior.
* 
*/
titleHover(): boolean;



}
/**
* Creates a new ImgButton
* 
* @param typeCheckedProperties {Partial<ImgButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ImgButton} The new instance
*/
class ImgButton{
static create(typeCheckedProperties? : Partial<ImgButton>, uncheckedProperties?: Object): ImgButton;







}

/**
* EditProxy that handles Img objects when editMode is enabled.
*/
interface ImgEditProxy extends StatefulCanvasEditProxy {




}
/**
* Creates a new ImgEditProxy
* 
* @param typeCheckedProperties {Partial<ImgEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ImgEditProxy} The new instance
*/
class ImgEditProxy{
static create(typeCheckedProperties? : Partial<ImgEditProxy>, uncheckedProperties?: Object): ImgEditProxy;







}

/**
* SectionHeader class based on an HLayout with StretchImg background.
*/
interface ImgSectionHeader extends HLayout {
/**
* Custom controls to be shown on top of this section header.
*  
*  These controls are shown in the ImgSectionHeader.controlsLayout.
*  
*  Note that this is an init-time property. If you need to dynamically change what 
*  controls are displayed to the user, we would recommend embedding the
*  controls in a Layout or similar container. 
*  This will allow you to show/hide or add/remove members at runtime
*  by manipulating the existing control(s) set up at init time.
* @type {Array<Partial<Canvas>>}
* @default null
*/
controls?: Array<Partial<Canvas>>;

/**
* A Layout containing specified ImgSectionHeader.controls if any. Sets
*  Layout.membersMargin:5, Layout.defaultLayoutAlign:"center", and
*  RTL-sensitive Layout.align (right by default).
* @type {Layout}
* @default null
*/
controlsLayout?: Layout;

/**
* Title to show for the section
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* If the title for this section header is too large for the available space, should the title be
*  clipped?
*  
*  This feature is supported only in browsers that support the CSS UI text-overflow
*  property (IE6+, Firefox 7+, Safari, Chrome, Opera 9+).
* @type {boolean}
* @default true
*/
clipTitle?: boolean;

/**
* If true and the title is clipped, then a hover containing the full title of this section header
*  is enabled.
* @type {boolean}
* @default true
*/
showClippedTitleOnHover?: boolean;

/**
* Optional icon to be shown with the button title text. 
*  
*  Specify as the partial URL to an image, relative to the imgDir of this component.
*  A sprited image can be specified using the SCSpriteConfig format.
*  
*  Note that the string "blank" is a valid setting for this attribute and will always 
*  result in the system blank image, with no state suffixes applied. Typically, this 
*  might be used when an iconStyle is also specified and the iconStyle renders the icon via 
*  a stateful background-image or other CSS approach.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* If this button is showing an icon should it be right or left aligned?
* @type {string}
* @default null
*/
iconAlign?: string;

/**
* Size in pixels of the icon image.
*  
*  The StatefulCanvas.iconWidth and StatefulCanvas.iconHeight
*  properties can be used to configure width and height separately.
*  
*  Note: When configuring the properties of a StatefulCanvas (or derivative)
*  AutoChild, it is best to set the iconWidth and iconHeight
*  to the same value rather than setting an iconSize. This is because certain
*  skins or customizations thereto might set the iconWidth and iconHeight,
*  making the customization of the AutoChild's iconSize ineffective.
* @type {Integer}
* @default "16"
*/
iconSize?: Integer;

/**
* Height in pixels of the icon image.
*  
*  If unset, defaults to StatefulCanvas.iconSize.
* @type {Integer}
* @default null
*/
iconHeight?: Integer;

/**
* Width in pixels of the icon image.
*  
*  If unset, defaults to StatefulCanvas.iconSize.
* @type {Integer}
* @default null
*/
iconWidth?: Integer;

/**
* If this button is showing an icon should it appear to the left or right of the title?
*  valid options are "left" and "right".
* @type {string}
* @default "left"
*/
iconOrientation?: string;

/**
* Prompt displayed in hover canvas if Canvas.showHover is true.
* @type {HTMLString}
* @default null
*/
prompt?: HTMLString;

/**
* By default doubleClicks are disabled for SectionHeaders. All mouse click
*  events will be handled as single clicks. Set this property to false
*  to enable standard double-click handling.
* @type {boolean}
* @default true
*/
noDoubleClicks?: boolean;

/**
* Background of the section header, based on a StretchImg.
* @type {StretchImg}
* @default null
*/
background?: StretchImg;



/**
*  For a SectionHeader embedded in a SectionStack, this method will return 
*  a pointer to the SectionStack in which this section header is
*  embedded.
* 
*/
getSectionStack(): SectionStack;

/**
*  Change the icon being shown for the header.
* 
* @param {SCImgURL} URL of new icon
*/
setIcon(icon:SCImgURL): void;

/**
*  If this header is showing an icon should it appear to the left or right of the title?
*  Valid options are "left" and "right".
* 
* @param {string} the new orientation
*/
setIconOrientation(orientation:string): void;

/**
*  Sets the horizontal alignment of the title.
* 
* @param {string} the new alignment
*/
setAlign(align:string): void;

/**
*  Sets the text shown as a tooltip for the header.
* 
* @param {HTMLString} the new tooltip
*/
setPrompt(prompt:HTMLString): void;

/**
*  Is the title of this section header clipped by ImgSectionHeader.controls
*  or the edge of the header?
* 
*/
titleClipped(): boolean;

/**
*  Returns the HTML that is displayed by the default ImgSectionHeader.titleHover
*  handler. Return null or an empty string to cancel the hover.
*  
* 
* @param {HTMLString} the HTML that would have been displayed by default
*/
titleHoverHTML(defaultHTML:HTMLString): HTMLString;

/**
*  Optional stringMethod to fire when the user hovers over this section header and the title is
*  clipped. If ImgSectionHeader.showClippedTitleOnHover is true, the default behavior is to
*  show a hover canvas containing the HTML returned by ImgSectionHeader.titleHoverHTML.
*  Return false to suppress this default behavior.
* 
*/
titleHover(): boolean;



}
/**
* Creates a new ImgSectionHeader
* 
* @param typeCheckedProperties {Partial<ImgSectionHeader>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ImgSectionHeader} The new instance
*/
class ImgSectionHeader{
static create(typeCheckedProperties? : Partial<ImgSectionHeader>, uncheckedProperties?: Object): ImgSectionHeader;







}

/**
* Resize bar for use in Layout.resizeBarClass, based on the
*  Img class. As with the Splitbar class, 
*  widgets of this class can be displayed as a resize-bar for widgets 
*  in Layouts where showResizeBar is set to true. Provides a different appearance from
*  the Splitbar class.
*  
*  To specify the resizeBar class for some layout, use the Layout.resizeBarClass
*  property.
*/
interface ImgSplitbar extends Img {
/**
* When a Splitbar is created by a layout, the target property
*  of the Splitbar will be a pointer to the member for which it is acting as a resizeBar.
*  The Splitbar will be positioned next to its target, and will resize it on drag completion.
*  
*  See Layout.resizeBarClass, Canvas.showResizeBar and 
*  Canvas.resizeBarTarget for details on configuring the resize bars shown in Layouts.
* @type {Canvas}
* @default null
*/
target?: Canvas;

/**
* Is this split bar vertically orientated?
*  When a Splitbar is created by a layout to be the resizeBar for some
*  member of the layout, the vertical property will be set to true
*  if the layout is horizontal, meaning this resizeBar will be taller than it is wide, 
*  and will allow horizontal resizing of the member.
* @type {boolean}
* @default null
*/
vertical?: boolean;

/**
* canDrag set to true to allow dragging of the split bar. Dragging the
*  Splitbar will resize it's Splitbar.target
* @type {boolean}
* @default true
*/
canDrag?: boolean;

/**
* If this property is true, a click on the Splitbar will collapse its 
*  Splitbar.target, hiding it and shifting the Splitbar and other members
*  of the layout across to fill the newly available space. If the target is already hidden
*  a click will expand it again (showing it at its normal size).
*  
*  Note that on touch devices, to enable collapsing/uncollapsing the target
*  in response to a tap, Splitbar.canCollapseOnTap must be set to
*  true.
* @type {boolean}
* @default true
*/
canCollapse?: boolean;

/**
* Default directory for skin images (those defined by the class), relative to the
*  Page-wide Page.getSkinDir.
* @type {SCImgURL}
* @default "images/SplitBar/"
*/
skinImgDir?: SCImgURL;

/**
* The base filename or stateful image configuration for the image. 
*  Note that as the StatefulCanvas.state 
*  of the component changes, the image displayed will be updated as described in
*  statefulImages.
* @type {string}
* @default null
*/
src?: string;

/**
* Default src to display when ImgSplitbar.vertical is false, 
*  and ImgSplitbar.src is unset.
* @type {string}
* @default "[SKIN]hgrip.png"
*/
hSrc?: string;

/**
* Default src to display when ImgSplitbar.vertical is true, 
*  and ImgSplitbar.src is unset.
* @type {SCImgURL}
* @default "[SKIN]vgrip.png"
*/
vSrc?: SCImgURL;





}
/**
* Creates a new ImgSplitbar
* 
* @param typeCheckedProperties {Partial<ImgSplitbar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ImgSplitbar} The new instance
*/
class ImgSplitbar{
static create(typeCheckedProperties? : Partial<ImgSplitbar>, uncheckedProperties?: Object): ImgSplitbar;







}

/**
* Specialized StretchImgButton used by TabSet/TabBar for tabs
*/
interface ImgTab extends StretchImgButton {
/**
* How big are the end pieces by default
* @type {number}
* @default "2"
*/
capSize?: number;

/**
* Base path for the images.  Note that when used within a TabSet, the
*  TabSet.tabBarPosition is appended as an additional path segment, yielding
*  "images/Tab/top/" et al.
* @type {URL}
* @default "images/Tab/"
*/
skinImgDir?: URL;

/**
* Base path for images shown within this ImgTab's label. This will be used for
*  icons (such as the close icon) by default.
* @type {URL}
* @default "images/"
*/
labelSkinImgDir?: URL;

/**
* @type {CSSStyleName}
* @default "tab"
*/
baseStyle?: CSSStyleName;

/**
* Like StretchImgButton.titleStyle, can set to provide a separate style for the
*  title text.
*  
*  If set and the ImgTab is StretchImgButton.vertical, a "v" will be
*  automatically prepended to the style name (hence "tabTitle" -> "vtabTitle").
* @type {CSSStyleName}
* @default null
*/
titleStyle?: CSSStyleName;

/**
* Base URL for tab images
* @type {SCImgURL}
* @default "[SKIN]tab.gif"
*/
src?: SCImgURL;

/**
* Should we visibly change state when the mouse goes over this tab
* @type {boolean}
* @default false
*/
showRollOver?: boolean;

/**
* Should we visibly change state when the tab receives keyboard focus?
* @deprecated * as of SmartClient 6.1 in favor of ImgTab.showFocused

* @type {boolean}
* @default true
*/
showFocus?: boolean;

/**
* Should we visibly change state when the tab receives keyboard focus?
* @type {boolean}
* @default true
*/
showFocused?: boolean;

/**
* Alignment of title text
* @type {Alignment}
* @default "center"
*/
align?: Alignment;





}
/**
* Creates a new ImgTab
* 
* @param typeCheckedProperties {Partial<ImgTab>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ImgTab} The new instance
*/
class ImgTab{
static create(typeCheckedProperties? : Partial<ImgTab>, uncheckedProperties?: Object): ImgTab;







}

/**
* A subclass of EventCanvas, used to render 
*  Calendar.indicators at important points in 
*  CalendarView.
*  
*  An IndicatorCanvas is a non-interactive, semi-transparent canvas that highlights a portion of a 
*  calendar view, by rendering across all lanes and behind normal Calendar.data.
*  
*  By default, the canvas shows no label but does show a hover.
*  
*  Default styling is specified at the Calendar.indicatorStyleName 
*  and can be overridden for CalendarEvent.styleName.
*/
interface IndicatorCanvas extends EventCanvas {




}
/**
* Creates a new IndicatorCanvas
* 
* @param typeCheckedProperties {Partial<IndicatorCanvas>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {IndicatorCanvas} The new instance
*/
class IndicatorCanvas{
static create(typeCheckedProperties? : Partial<IndicatorCanvas>, uncheckedProperties?: Object): IndicatorCanvas;







}

/**
* This class is a synonym for Window that can be used to make intent clearer.
*  It is used by some development tools for that purpose.
*/
interface InlineWindow extends Window {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "WindowEditProxy"
*/
editProxyConstructor?: SCClassName;





}
/**
* Creates a new InlineWindow
* 
* @param typeCheckedProperties {Partial<InlineWindow>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {InlineWindow} The new instance
*/
class InlineWindow{
static create(typeCheckedProperties? : Partial<InlineWindow>, uncheckedProperties?: Object): InlineWindow;







}

/**
* FormItem intended for inputting integer numbers.
*/
interface IntegerItem extends TextItem {




}
/**
* Creates a new IntegerItem
* 
* @param typeCheckedProperties {Partial<IntegerItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {IntegerItem} The new instance
*/
class IntegerItem{
static create(typeCheckedProperties? : Partial<IntegerItem>, uncheckedProperties?: Object): IntegerItem;







}

/**
* Utilities for working with JSON data.
*/
interface JSON extends Class {




}
/**
* Creates a new JSON
* 
* @param typeCheckedProperties {Partial<JSON>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {JSON} The new instance
*/
class JSON{
static create(typeCheckedProperties? : Partial<JSON>, uncheckedProperties?: Object): JSON;




/**
*  Serialize an object as a JSON string by creating a JSONEncoder and calling
*  JSONEncoder.encode.
*  
*  Note that using the String produced by this API with JSON.decode will not
*  successfully preserve dates. Use JSONEncoder.dateFormat "dateConstructor" or
*  "logicalDateConstructor" to have dates round-trip properly.
* 
* 
* @param {any} object to serialize
* @param {Partial<JSONEncoder>=} optional settings for encoding
*/
static encode(object:any, settings?:Partial<JSONEncoder>): string;

/**
*  De-serialize an object from JSON. Currently, this is simply a JavaScript eval() and should
*  be used for trusted data only.
* 
* 
* @param {string} JSON data to be de-serialized
*/
static decode(jsonString:string): object;




}

/**
* Class for encoding objects as JSON strings.
*/
interface JSONEncoder extends Class {
/**
* Controls the output of the JSONEncoder when instances of SmartClient classes (eg a ListGrid)
*  are included in the data to be serialized. See JSONInstanceSerializationMode.
*  
*  Note that the JSONEncoder does not support a format that will recreate the instance if
*  passed to decode() or eval().
* @type {JSONInstanceSerializationMode}
* @default "long"
*/
serializeInstances?: JSONInstanceSerializationMode;

/**
* If true, don't show SmartClient internal properties when encoding and object.
* @type {boolean}
* @default false
*/
skipInternalProperties?: boolean;

/**
* If objects that cannot be serialized to JSON are encountered during serialization, show a
*  placeholder rather than just omitting them. 
*  
*  The resulting String will not be valid JSON and so cannot be decoded/eval()'d
* @type {boolean}
* @default false
*/
showDebugOutput?: boolean;

/**
* Format for encoding JavaScript Date values in JSON. See JSONDateFormat for
*  valid options, or override JSONEncoder.encodeDate to do something custom.
* @type {JSONDateFormat}
* @default "xmlSchema"
*/
dateFormat?: JSONDateFormat;

/**
* By default, obscure non-printable characters such as DC3 (Device Control 3, U+0013 hexadecimal)
*  will be escaped according to JSON standards. ECMA-404 / The JSON Data Interchange Format
*  requires the quotation mark (U+0022), reverse solidus (U+005C), and control characters (U+0000
*  through U+001F) to be escaped.
*  
*  These characters are very rarely used in JSON data in web applications. If you know that
*  your application does not use such characters in JSON data, there can be a performance
*  advantage to setting escapeNonPrintable to false in order to disable the
*  logic for escaping these characters. This is a detectable difference only when dealing
*  with very large JSON structures on older browsers that do not provide native support (for
*  example, Internet Explorer 8).
* @type {boolean}
* @default true
*/
escapeNonPrintable?: boolean;

/**
* Whether all property names should be quoted, or only those property names that are not valid
*  identifiers or are JavaScript reserved words (such as "true").
*  
*  Encoding only where required produces slightly shorter, more readable output which is still
*  compatible with JavaScript's eval():
*  
*  {
*    someProp : "someValue",
*    "true" : "otherValue",
*    otherProp : "otherValue"
*  }
*  
*  .. but is not understood by many server-side JSON parser implementations.
* @type {boolean}
* @default true
*/
strictQuoting?: boolean;

/**
* What the JSONEncoder should do if it encounters a circular reference.
* @type {JSONCircularReferenceMode}
* @default "path"
*/
circularReferenceMode?: JSONCircularReferenceMode;

/**
* The string marker used to represent circular references. See JSONEncoder.circularReferenceMode.
* @type {string}
* @default "$$BACKREF$$"
*/
circularReferenceMarker?: string;

/**
* Whether to add indentation to the returned JSON string. This makes the returned JSON much
*  easier to read but adds size. Note that when delivering JSON responses compressed, the size
*  difference between prettyPrinted JSON and normal JSON is negligible.
* @type {boolean}
* @default true
*/
prettyPrint?: boolean;



/**
*  Serialize an object as a JSON string.
*  
*  Automatically handles circular references - see JSONEncoder.circularReferenceMode.
*  
*  
*  Note that using the String produced by this API with JSON.decode will not
*  successfully preserve dates. Use JSONEncoder.dateFormat "dateConstructor" or
*  "logicalDateConstructor" to have dates round-trip properly.
* 
* 
* @param {any} object to serialize
*/
encode(object:any): string;

/**
*  Encode a JavaScript Date value.
*  
*  By default, follows the JSONEncoder.dateFormat setting. Override to do
*  custom encoding.
*  
* 
* @param {Date} JavaScript date object to be serialized
*/
encodeDate(theDate:Date): string;



}
/**
* Creates a new JSONEncoder
* 
* @param typeCheckedProperties {Partial<JSONEncoder>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {JSONEncoder} The new instance
*/
class JSONEncoder{
static create(typeCheckedProperties? : Partial<JSONEncoder>, uncheckedProperties?: Object): JSONEncoder;







}

/**
* Regular expression-based source code colorizer for JS source.
*  
*  NOTE: This class exists only for use with the SmartClient Feature Explorer and 
*  SmartGWT Showcases and cannot be used in any other environment.
*/
interface JSSyntaxHiliter extends SyntaxHiliter {




}
/**
* Creates a new JSSyntaxHiliter
* 
* @param typeCheckedProperties {Partial<JSSyntaxHiliter>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {JSSyntaxHiliter} The new instance
*/
class JSSyntaxHiliter{
static create(typeCheckedProperties? : Partial<JSSyntaxHiliter>, uncheckedProperties?: Object): JSSyntaxHiliter;







}

/**
* Labels display a small amount of Label.align Label.contents
*  with optional Label.icon and Label.autoFit.
*  
*  For a general-purpose container for HTML content, use HTMLFlow or HTMLPane
*  instead.
*/
interface Label extends Button {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "LabelEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* The contents of a canvas or label widget. Any HTML string is acceptable.
* @type {HTMLString}
* @default "&amp;nbsp;"
*/
contents?: HTMLString;

/**
* Dynamic contents allows the contents string to be treated as a simple but powerful
*  template. When this attribute is set to true, JavaScript expressions may be embedded 
*  within the contents string, using the format:
*  &#36;{[JavaScript to evaluate]}.
*  
*  For example, to include the current date in a templated message, 
*  canvas.contents could be set to:
*  "Today's date is &lt;b&gt;&#36;{new Date().toUSShortDate()}&lt;/b&gt;"
*  
*  Embedded expressions will be evaluated when the canvas is drawn or redrawn, and
*  the result of the evaluated expression will be displayed to the user. If the
*  expression does not evaluate to a String, the toString() representation
*  of the returned object will be displayed automatically
*  
*  Dynamic expressions are evaluated in the scope of the canvas displaying the content, 
*  so the this keyword may be used within your expression to refer to the 
*  canvas. Developers may also explicitly supply values for variables to be used within
*  the evaluation via the Canvas.dynamicContentsVars property.
*  
*  Notes:
*  
* - Calling markForRedraw() on the canvas will evaluate any embedded expressions.
*  
* - Multiple such expressions may be embedded within the contents string for a component.
*  
* - If an error occurs during evaluation, a warning is logged
*    to the debugging and the error string will be embedded in place 
*    of the expected value in the Canvas.
*  
*  
* @type {boolean}
* @default false
*/
dynamicContents?: boolean;

/**
* Horizontal alignment of label text. See Alignment type for details.
* @type {Alignment}
* @default "left"
*/
align?: Alignment;

/**
* Vertical alignment of label text. See VerticalAlignment type for details.
* @type {VerticalAlignment}
* @default "center"
*/
valign?: VerticalAlignment;

/**
* If false, the label text will not be wrapped to the next line.
* @type {boolean}
* @default true
*/
wrap?: boolean;

/**
* If true, ignore the specified size of this widget and always size just large
*  enough to accommodate the title. If setWidth() is explicitly called on an
*  autoFit:true button, autoFit will be reset to false.
*  
*  Note that for StretchImgButton instances, autoFit will occur horizontally only, as 
*  unpredictable vertical sizing is likely to distort the media. If you do want vertical 
*  auto-fit, this can be achieved by simply setting a small height, and having 
*  overflow:"visible"
* @type {boolean}
* @default null
*/
autoFit?: boolean;

/**
* Size for this component's horizontal dimension. See Canvas.width for more
*  details.
*  
*  Note that if StatefulCanvas.autoFit is set, this property will be ignored so that the widget
*  is always sized just large enough to accommodate the title.
* @type {number | string}
* @default null
*/
width?: number | string;

/**
* Size for this component's vertical dimension. See Canvas.height for more
*  details.
*  
*  Note that if StatefulCanvas.autoFit is set on non-StretchImgButton instances, this
*  property will be ignored so that the widget is always sized just large enough to
*  accommodate the title.
* @type {number | string}
* @default null
*/
height?: number | string;

/**
* Optional icon to be shown with the button title text. 
*  
*  Specify as the partial URL to an image, relative to the imgDir of this component.
*  A sprited image can be specified using the SCSpriteConfig format.
*  
*  Note that the string "blank" is a valid setting for this attribute and will always 
*  result in the system blank image, with no state suffixes applied. Typically, this 
*  might be used when an iconStyle is also specified and the iconStyle renders the icon via 
*  a stateful background-image or other CSS approach.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* Size in pixels of the icon image.
*  
*  The StatefulCanvas.iconWidth and StatefulCanvas.iconHeight
*  properties can be used to configure width and height separately.
*  
*  Note: When configuring the properties of a StatefulCanvas (or derivative)
*  AutoChild, it is best to set the iconWidth and iconHeight
*  to the same value rather than setting an iconSize. This is because certain
*  skins or customizations thereto might set the iconWidth and iconHeight,
*  making the customization of the AutoChild's iconSize ineffective.
* @type {Integer}
* @default "16"
*/
iconSize?: Integer;

/**
* Width in pixels of the icon image.
*  
*  If unset, defaults to StatefulCanvas.iconSize.
* @type {Integer}
* @default null
*/
iconWidth?: Integer;

/**
* Height in pixels of the icon image.
*  
*  If unset, defaults to StatefulCanvas.iconSize.
* @type {Integer}
* @default null
*/
iconHeight?: Integer;

/**
* If this button is showing an icon should it appear to the left or right of the title?
*  valid options are "left" and "right".
* @type {string}
* @default "left"
*/
iconOrientation?: string;

/**
* If this button is showing an icon should it be right or left aligned?
* @type {string}
* @default null
*/
iconAlign?: string;

/**
* Pixels between icon and title text.
* @type {Integer}
* @default "6"
*/
iconSpacing?: Integer;

/**
* If using an icon for this button, whether to switch the icon image if the button becomes
*  disabled.
* @type {boolean}
* @default true
*/
showDisabledIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image on mouse rollover.
* @type {boolean}
* @default false
*/
showRollOverIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the button
*  receives focus.
*  
*  If StatefulCanvas.showFocusedAsOver is true, the "Over" icon will be
*  displayed when the canvas has focus, otherwise a separate "Focused" icon
*  will be displayed
* @type {boolean}
* @default false
*/
showFocusedIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the mouse goes
*  down on the button.
* @type {boolean}
* @default false
*/
showDownIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the button
*  becomes selected.
* @type {boolean}
* @default false
*/
showSelectedIcon?: boolean;

/**
* Set the CSS class for this widget. For a Label, this is equivalent to
*  setting Button.baseStyle.
* @type {CSSStyleName}
* @default "normal"
*/
styleName?: CSSStyleName;



/**
*  Changes the orientation of the icon relative to the text of the button.
* 
* 
* @param {string} The new orientation of the icon relative to the text of the button.
*/
setIconOrientation(orientation:string): void;

/**
*  Change the icon being shown next to the title text.
* 
* @param {SCImgURL} URL of new icon
*/
setIcon(icon:SCImgURL): void;

/**
*  Dynamically change the CSS class for this widget. For a Label, this is equivalent to
*  StatefulCanvas.setBaseStyle.
* 
* 
* @param {CSSStyleName} new CSS style name
*/
setStyleName(newStyle:CSSStyleName): void;

/**
*  Changes the contents of a widget to newContents, an HTML string.
*  
*  When Canvas.dynamicContents is set, setContents() can
*  also be called with no arguments to cause contents to be re-evaluated.
* 
* 
* @param {HTMLString=} an HTML string to be set as the contents of this widget
*/
setContents(newContents?:HTMLString): void;



}
/**
* Creates a new Label
* 
* @param typeCheckedProperties {Partial<Label>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Label} The new instance
*/
class Label{
static create(typeCheckedProperties? : Partial<Label>, uncheckedProperties?: Object): Label;







}

/**
* EditProxy that handles Label objects when editMode is enabled.
*/
interface LabelEditProxy extends StatefulCanvasEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's contents.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's contents.
* 
* 
* @param {string} the new component contents
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new LabelEditProxy
* 
* @param typeCheckedProperties {Partial<LabelEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {LabelEditProxy} The new instance
*/
class LabelEditProxy{
static create(typeCheckedProperties? : Partial<LabelEditProxy>, uncheckedProperties?: Object): LabelEditProxy;







}

/**
* Arranges a set of "member" Canvases in horizontal or vertical stacks, applying a layout
*  policy to determine member heights and widths.
*  
*  A Layout manages a set of "member" Canvases provided as Layout.members. Layouts
*  can have both "members", whose position and size are managed by the Layout, and normal
*  Canvas children, which manage their own position and size.
*  
*  Rather than using the Layout class directly, use the HLayout, VLayout,
*  HStack and VStack classes, which are subclasses of Layout preconfigured for
*  horizontal or vertical stacking, with the "fill" (VLayout) or "none" (VStack)
*  LayoutPolicy already set.
*  
*  Layouts and Stacks may be nested to create arbitrarily complex layouts.
*  
*  Since Layouts can be either horizontally or vertically oriented, throughout the
*  documentation of Layout and it's subclasses, the term "length" refers to the axis of
*  stacking, and the term "breadth" refers to the other axis. Hence, "length" means height in
*  the context of a VLayout or VStack, but means width in the context of an HLayout or HStack.
*  
*  To show a resizer bar after (to the right or bottom of) a layout member, set
*  Canvas.showResizeBar to
*  true on that member component (not on the HLayout or VLayout). Resizer bars override
*  Layout.membersMargin spacing.
*  
*  Like other Canvas subclasses, Layout and Stack components may have % width and height
*  values. To create a dynamically-resizing layout that occupies the entire page (or entire
*  parent component), set width and height to "100%".
*/
interface Layout extends Canvas {
/**
* Part of the automatedTesting system, strategy to use when generated
*  locators for members from within this Layout's members array.
* @type {LocatorStrategy}
* @default null
*/
locateMembersBy?: LocatorStrategy;

/**
* LocatorTypeStrategy to use when finding members within this layout.
* @type {LocatorTypeStrategy}
* @default null
*/
locateMembersType?: LocatorTypeStrategy;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "LayoutEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* An array of canvases that will be contained within this layout. You can set the
*  following properties on these canvases (in addition to the standard component
*  properties):
*  
*  
* - Canvas.layoutAlign -- specifies the member's alignment along the
*    breadth axis; valid values are "top", "center" and "bottom" for a horizontal layout
*    and "left", "center" and "right" for a vertical layout (see
*    Layout.defaultLayoutAlign for default implementation.)
*  
* - Canvas.showResizeBar -- set to true to show a resize bar
*    (default is false)
*  
*  Height and width settings found on members are interpreted by the Layout according to
*  the Layout.vPolicy.
*  
*  Note that it is valid to have null slots in the provided members Array,
*  and the Layout will ignore those slots. This can be useful to keep code compact, for
*  example, when constructing the members Array, you might use an expression
*  that either returns a component or null depending on whether the component should be
*  present. If the expression returns null, the null slot will be ignored by the Layout.
* @type {Array<Partial<Canvas>>}
* @default null
*/
members?: Array<Partial<Canvas>>;

/**
* A Layout may overflow if it has one or more members with a fixed width or height, or that
*  themselves overflow. For details on member sizing see LayoutPolicy.
*  
*  Note that for overflow: "auto", "scroll", or "visible", members exceeding the Layout's
*  specified breadth but falling short of its overflow breadth will keep the alignment set
*  via Layout.defaultLayoutAlign or Canvas.layoutAlign.
* @type {Overflow}
* @default "visible"
*/
overflow?: Overflow;

/**
* Orientation of this layout.
* @deprecated * in favor of Layout.vertical, which, if specified takes precedence over this setting

* @type {Orientation}
* @default "horizontal"
*/
orientation?: Orientation;

/**
* Should this layout appear with members stacked vertically or horizontally. Defaults to 
*  false if unspecified.
* @type {boolean}
* @default null
*/
vertical?: boolean;

/**
* Sizing policy applied to members on vertical axis
* @type {LayoutPolicy}
* @default "fill"
*/
vPolicy?: LayoutPolicy;

/**
* Sizing policy applied to members on horizontal axis
* @type {LayoutPolicy}
* @default "fill"
*/
hPolicy?: LayoutPolicy;

/**
* See Layout.minMemberLength.
* @deprecated * use the more intuitively named Layout.minMemberLength

* @type {Integer}
* @default "1"
*/
minMemberSize?: Integer;

/**
* Minimum size, in pixels, below which flexible-sized members should never be shrunk, even
*  if this requires the Layout to overflow. Note that this property only applies along
*  the length axis of the Layout, and has no affect on breadth.
*  
*  Does not apply to members given a fixed size in pixels - such members will never be
*  shrunk below their specified size in general.
* @type {Integer}
* @default "1"
*/
minMemberLength?: Integer;

/**
* Set this property to cause the layout to assign the breadths of other members as if the
*  available breadth is actually wide enough to accommodate the
*  minBreadthMember (even though the Layout might not actually be that
*  wide, and may overflow its assigned size along the breadth axis due to the breadth of the
*  minBreadthMember.
*  
*  Without this property set, members of a layout aren't ever expanded in breadth (by the
*  layout) to fit an overflow of the layout along the breadth axis. Setting this property
*  will make sure all members (other than the one specified) get expanded to fill the full
*  visual breadth of the layout (assuming they are configured to use 100% layout breadth).
* @type {string | Integer | Canvas}
* @default null
*/
minBreadthMember?: string | Integer | Canvas;

/**
* Whether the layout policy is continuously enforced as new members are added or removed
*  and as members are resized.
*  
*  This setting implies that any member that resizes larger, or any added member, will take
*  space from other members in order to allow the overall layout to stay the same size.
* @type {boolean}
* @default true
*/
enforcePolicy?: boolean;

/**
* Specifies the default alignment for layout members on the breadth axis (horizontal axis
*  for a VLayout, vertical axis for an HLayout). Can be overridden on a per-member basis
*  by setting Canvas.layoutAlign.
*  
*  If unset, default member layout alignment will be "top" for a horizontal layout, and
*  "left" for a vertical layout, or "right" if in Page.isRTL mode.
*  
*  When attempting to center components be sure that you have set a specific size on the
*  component(s) involved. If components fill all available space in the layout, centering
*  looks the same as not centering.
*  
*  Similarly, if a component has no visible boundary (like a border), it can appear similar
*  to when it's not centered if the component is larger than you expect - use the Watch tab
*  in the Developer Console to see the component's extents visually.
* @type {Alignment | VerticalAlignment}
* @default null
*/
defaultLayoutAlign?: Alignment | VerticalAlignment;

/**
* Alignment of all members in this Layout on the length axis (vertical for a VLayout,
*  horizontal for an HLayout). Defaults to "top" for vertical Layouts, and "left" for
*  horizontal Layouts.
*  
*  Horizontal layouts should only be set to Alignment, and vertical layouts to
*  VerticalAlignment, otherwise they will be considered invalid values, and assigning an
*  invalid value here will log a warning to the Developer Console.
*  
*  For alignment on the breadth axis, see Layout.defaultLayoutAlign and
*  Canvas.layoutAlign.
*  
*  When attempting to center components be sure that you have set a specific size on the
*  component(s) involved. If components fill all available space in the layout, centering
*  looks the same as not centering.
*  
*  Similarly, if a component has no visible boundary (like a border), it can appear similar
*  to when it's not centered if the component is larger than you expect - use the Watch tab
*  in the Developer Console to see the component's extents visually.
* @type {Alignment | VerticalAlignment}
* @default null
*/
align?: Alignment | VerticalAlignment;

/**
* Reverse the order of stacking for this Layout, so that the last member is shown first.
*  
*  Requires a manual call to reflow() if changed on the fly.
*  
*  In RTL mode, for horizontal Layouts the value of this flag will be flipped during
*  initialization.
* @type {boolean}
* @default false
*/
reverseOrder?: boolean;

/**
* If this widget has padding specified (as Canvas.padding or in the
*  CSS style applied to this layout), should it show up as space outside the members,
*  similar to layoutMargin?
*  
*  If this setting is false, padding will not affect member positioning (as CSS padding
*  normally does not affect absolutely positioned children). Leaving this setting true
*  allows a designer to more effectively control layout purely from CSS.
*  
*  Note that Layout.layoutMargin if specified, takes precedence over this value.
* @type {boolean}
* @default true
*/
paddingAsLayoutMargin?: boolean;

/**
* Space outside of all members. This attribute, along with Layout.layoutLeftMargin
*  and related properties do not have a true setter method.
*  
*  It may be assigned directly at runtime. After setting the property,
*  Layout.setLayoutMargin may be called with no arguments to reflow the layout.
*  
* @type {Integer}
* @default null
*/
layoutMargin?: Integer;

/**
* Space outside of all members, on the left-hand side. Defaults to Layout.layoutMargin.
*  
*  Requires a manual call to setLayoutMargin() if changed on the fly.
* @type {Integer}
* @default null
*/
layoutLeftMargin?: Integer;

/**
* Space outside of all members, on the right-hand side. Defaults to Layout.layoutMargin.
*  
*  Requires a manual call to setLayoutMargin() if changed on the fly.
* @type {Integer}
* @default null
*/
layoutRightMargin?: Integer;

/**
* Space outside of all members, on the top side. Defaults to Layout.layoutMargin.
*  
*  Requires a manual call to setLayoutMargin() if changed on the fly.
* @type {Integer}
* @default null
*/
layoutTopMargin?: Integer;

/**
* Space outside of all members, on the bottom side. Defaults to Layout.layoutMargin.
*  
*  Requires a manual call to setLayoutMargin() if changed on the fly.
* @type {Integer}
* @default null
*/
layoutBottomMargin?: Integer;

/**
* Space between each member of the layout.
*  
*  Requires a manual call to reflow() if changed on the fly.
* @type {Integer}
* @default "0"
*/
membersMargin?: Integer;

/**
* Whether to leave a gap for a vertical scrollbar even when one is not actually present.
*  
*  This setting avoids the layout resizing all members when the vertical scrollbar is
*  introduced or removed, which can avoid unnecessary screen shifting and improve
*  performance.
* @type {boolean}
* @default false
*/
leaveScrollbarGap?: boolean;

/**
* Number of pixels by which each member should overlap the preceding member, used for
*  creating an "stack of cards" appearance for the members of a Layout.
*  
*  memberOverlap can be used in conjunction with Layout.stackZIndex to create
*  a particular visual stacking order.
*  
*  Note that overlap of individual members can be accomplished with a negative setting for
*  Canvas.extraSpace.
* @type {PositiveInteger}
* @default "0"
*/
memberOverlap?: PositiveInteger;

/**
* Equivalent to Layout.layoutLeftMargin for a horizontal layout, or Layout.layoutTopMargin 
*  for a vertical layout.
*  
*  If both layoutStartMargin and the more specific properties (top/left margin) 
*  are both set, the more specific properties win.
* @type {Integer}
* @default null
*/
layoutStartMargin?: Integer;

/**
* Equivalent to Layout.layoutRightMargin for a horizontal layout, or Layout.layoutBottomMargin 
*  for a vertical layout.
*  
*  If both layoutEndMargin and the more specific properties (right/bottom margin) 
*  are both set, the more specific properties win.
* @type {Integer}
* @default null
*/
layoutEndMargin?: Integer;

/**
* Policy for whether resize bars are shown on members by default. Note that this setting
*  changes the effect of Canvas.showResizeBar for members of this layout.
* @type {LayoutResizeBarPolicy}
* @default "marked"
*/
defaultResizeBars?: LayoutResizeBarPolicy;

/**
* A MultiAutoChild created to resize members of this Layout.
*  
*  A resize bar will be created for any member of this Layout that has
*  Canvas.showResizeBar set to true. Resize bars will be
*  instances of the class specified by Layout.resizeBarClass by default, and will
*  automatically be sized to the member's breadth, and to the thickness specified by
*  Layout.resizeBarSize.
*  
*  To customize the appearance or behavior of resizeBars within some layout a custom
*  resize bar class can be created by subclassing Splitbar or ImgSplitbar and
*  setting Layout.resizeBarClass or resizeBarConstructor to this custom class.
*  
*  Alternatively, resizeBarProperties may be specified. See autoChildUsage
*  for more information.
*  
*  
*  
*  The built-in Splitbar class supports drag resizing of its target member,
*  and clicking on the bar with a mouse to collapse/uncollapse the target member.
* @type {Splitbar}
* @default "see below"
*/
resizeBar?: Splitbar;

/**
* Default class to use for creating Layout.resizeBar. This may be
*  overridden by resizeBarConstructor.
*  
*  Classes that are valid by default are Splitbar, ImgSplitbar, and
*  Snapbar.
* @type {string}
* @default "Splitbar"
*/
resizeBarClass?: string;

/**
* Thickness of the resizeBar in pixels.
* @type {Integer}
* @default "7"
*/
resizeBarSize?: Integer;

/**
* If true when members are added / removed, they should be animated as they are shown
*  or hidden in position
* @type {boolean}
* @default null
*/
animateMembers?: boolean;

/**
* If specified this is the duration of show/hide animations when members are being shown
*  or hidden due to being added / removed from this layout.
* @type {number}
* @default null
*/
animateMemberTime?: number;

/**
* Layouts provide a default implementation of a drag and drop interaction. If you set
*  Canvas.canAcceptDrop:true and canDropComponents:true
*  on a Layout, when a droppable Canvas (Canvas.canDrop is dragged over
*  the layout, it will show a dropLine (a simple insertion line) at the drop location. 
*  
*  When the drop occurs, the dragTarget (obtained using
*  EventHandler.getDragTarget) is added as a member of this layout at the location
*  shown by the dropLine (calculated by Layout.getDropPosition). This default
*  behavior allows either members or external components that have
*  Canvas.canDragReposition (or Canvas.canDrag) and Canvas.canDrop set
*  to true to be added to or reordered within the Layout.
*  
*  You can control the thickness of the dropLine via Layout.dropLineThickness and
*  you can customize the style using css styling in the skin file (look for .layoutDropLine
*  in skin_styles.css for your skin). 
*  
*  If you want to dynamically create a component to be added to the Layout in response to a
*  drop event you can do so as follows: 
*  
*  
*  isc.VLayout.create({
*   ...various layout properties...
*   canDropComponents: true,
*   drop : function () {
*    // create the new component 
*    var newMember = isc.Canvas.create(); 
*    // add to the layout at the current drop position 
*    // (the dropLine will be showing here)
*    this.addMember(newMember, this.getDropPosition()); 
*    // hide the dropLine that was automatically shown 
*    // by builtin SmartClient methods
*    this.hideDropLine();
*   }
*  });
*  
*  
*  
*  If you want to completely suppress the builtin drag and drop logic, but still receive drag
*  and drop events for your own custom implementation, set Canvas.canAcceptDrop to
*  true and canDropComponents to false on your Layout.
* @type {boolean}
* @default true
*/
canDropComponents?: boolean;

/**
* Thickness, in pixels of the dropLine shown during drag and drop when
*  Layout.canDropComponents is set to true. See the discussion in
*  Layout for more info.
* @type {number}
* @default "2"
*/
dropLineThickness?: number;

/**
* Controls whether to show a drop-indicator during a drag and drop operation. Set to 
*  false if you either don't want to show drop-lines, or plan to create your own.
* @type {boolean}
* @default null
*/
showDropLines?: boolean;

/**
* If set to true, when a member is dragged out of layout, a visible placeholder canvas 
*  will be displayed in place of the dragged widget for the duration of the drag and drop
*  interaction.
* @type {boolean}
* @default null
*/
showDragPlaceHolder?: boolean;

/**
* If Layout.showDragPlaceHolder is true, this 
*  properties object can be used to customize the appearance of the placeholder displayed
*  when the user drags a widget out of this layout.
* @type {Partial<Canvas>}
* @default null
*/
placeHolderProperties?: Partial<Canvas>;

/**
* For use in conjunction with Layout.memberOverlap, controls the z-stacking order of
*  members.
*  
*  If set to "lastOnTop", members stack from the first member at bottom to the last member
*  at top. If set to "firstOnTop", members stack from the last member at bottom to the
*  first member at top.
* @type {string}
* @default null
*/
stackZIndex?: string;

/**
* If set, a Layout with breadthPolicy:"fill" will specially interpret a percentage breadth on
*  a member as a percentage of available space excluding the Layout.layoutMargin. If false,
*  percentages work exactly as for a non-member, with layoutMargins, if any, ignored.
* @type {boolean}
* @default true
*/
managePercentBreadth?: boolean;

/**
* If Layout.showDragPlaceHolder is true, this 
*  defaults object determines the default appearance of the placeholder displayed
*  when the user drags a widget out of this layout.
*  Default value for this property sets the placeholder Canvas.styleName to
*  "layoutPlaceHolder"
*  To modify this object, use Class.changeDefaults
* @type {Partial<Canvas>}
* @default null
*/
placeHolderDefaults?: Partial<Canvas>;

/**
* Line showed to mark the drop position when components are being dragged onto this Layout.
*  A simple Canvas typically styled via CSS. The default dropLine.styleName is
*  "layoutDropLine".
* @type {Canvas}
* @default null
*/
dropLine?: Canvas;



/**
*  Override point for changing the offset on the breadth axis for members, that is, the offset
*  relative to the left edge for a vertical layout, or the offset relative to the top edge for
*  a horizontal layout.
*  
*  The method is passed the default offset that would be used for the member if
*  getMemberOffset() were not implemented. This default offset already takes into account
*  Layout.layoutMargin, as well as the Layout.defaultLayoutAlign,
*  which is also passed to getMemberOffset().
*  
*  This method is an override point only; it does not exist by default and cannot be called.
* 
* 
* @param {Canvas} Component to be positioned
* @param {number} Value of the currently calculated member offset. If this   value is returned unchanged the layout will have its default behavior
* @param {string} alignment of the enclosing layout, on the breadth axis
*/
getMemberOffset(member:Canvas, defaultOffset:number, alignment:string): void;

/**
*  Changes the Layout.align for this Layout.
*  
*  Horizontal layouts should only be changed to Alignment, and vertical layouts to
*  VerticalAlignment, otherwise they will be considered invalid values, and assigning an
*  invalid value here will log a warning to the Developer Console.
*  
*  For alignment on the breadth axis, see Layout.defaultLayoutAlign and
*  Canvas.layoutAlign.
* 
* 
* @param {Alignment | VerticalAlignment} */
setAlign(align:Alignment | VerticalAlignment): void;

/**
*  Method to force a reflow of the layout after directly assigning a value to any of the
*  layout*Margin properties. Takes no arguments.
* 
* 
* @param {Integer=} optional new setting for layout.layoutMargin. Regardless of whether a new             layout margin is passed, the layout reflows according to the current settings              for layoutStartMargin et al
*/
setLayoutMargin(newMargin?:Integer): void;

/**
*  Reveals the child or member Canvas passed in by showing it if it is currently hidden
* 
* 
* @param {GlobalId | Canvas} the child Canvas to reveal, or its global ID
*/
revealChild(child:GlobalId | Canvas): void;

/**
*  Return the breadth for a member of this layout which either didn't specify a breadth or
*  specified a percent breadth with Layout.managePercentBreadth:true.
*  
*  Called only for Layouts which have a LayoutPolicy for the breadth
*  axis of "fill", since Layouts with a breadth policy of "none" leave all member breadths alone.
* 
* 
* @param {Canvas} Component to be sized
* @param {number} Value of the currently calculated member breadth. This   may be returned verbatim or manipulated in this method.
*/
getMemberDefaultBreadth(member:Canvas, defaultBreadth:number): void;

/**
*  
* 
*/
getMemberSizes(): Array<any>;

/**
*  Returns whether there is a pending reflow of the members of the layout.
*  
*  Modifying the set of members, resizing members or changing layout settings will cause a
*  recalculation of member sizes to be scheduled. The recalculation is delayed
*  so that it is not performed redundantly if multiple changes are made in a row. 
*  
*  To force immediate recalculation of new member sizes and resizing of members, call
*  Layout.reflowNow.
*  
* 
*/
layoutIsDirty(): boolean;

/**
*  Layout members according to current settings.
*  
*  Members will reflow automatically when the layout is resized, members resize, the list of
*  members changes or members change visibility. It is only necessary to manually call
*  reflow() after changing settings on the layout, for example,
*  layout.reverseOrder.
* 
* 
* @param {string=} reason reflow() had to be called (appear in logs if enabled)
*/
reflow(reason?:string): void;

/**
*  Layout members according to current settings, immediately.
*  
*  Generally, when changes occur that require a layout to reflow (such as members being shown
*  or hidden), the Layout will reflow only after a delay, so that multiple changes cause only
*  one reflow. To remove this delay for cases where it is not helpful, reflowNow() can be
*  called.
* 
*/
reflowNow(): void;

/**
*  Given a numerical index or a member Canvas.name or member Canvas.ID,
*  return a pointer to the appropriate member. If passed a member Canvas, just returns it.
*  
*  Note that if more than one member has the same name, passing in a
*  name has an undefined result.
* 
* 
* @param {string | Integer | Canvas} identifier for the required member
*/
getMember(memberID:string | Integer | Canvas): Canvas;

/**
*  Given a member Canvas or member Canvas.ID or Canvas.name, return the
*  index of that member within this layout's members array. If passed a number, just returns it.
*  
*  Note that if more than one member has the same name, passing in a
*  name has an undefined result.
* 
* 
* @param {string | Canvas | Integer} identifier for the required member
*/
getMemberNumber(memberID:string | Canvas | Integer): Integer;

/**
*  Returns true if the layout includes the specified canvas.
* 
* @param {Canvas} the canvas to check for
*/
hasMember(canvas:Canvas): boolean;

/**
*  Get the Array of members.
*  
*  
*  NOTE: the returned array should not be modified directly. Use Layout.addMember /
*  Layout.removeMember to add or remove members from the Layout. Call
*  List.duplicate on the returned Array if you need a copy of the members
*  array for some other purpose.
*  
* 
*/
getMembers(): Array<Partial<Canvas>>;

/**
*  Convenience method to return the number of members this Layout has
* 
*/
getMembersLength(): Integer;

/**
*  Add a canvas to the layout, optionally at a specific position.
*  
*  Depending on the layout policy, adding a new member may cause existing members to
*  resize.
*  
*  When adding a member to a drawn Layout, the layout will not immediately reflow, that is, the
*  member will not immediately draw and existing members will not immediately resize. This is 
*  to allow multiple new members to be added and multiple manual resizes to take place without
*  requiring layout members to redraw and resize multiple times. 
*  
*  To force an immediate reflow in order to, for example, find out what size a newly added
*  member has been assigned, call Layout.reflowNow.
* 
* 
* @param {Canvas} the canvas object to be added to the layout
* @param {Integer=} the position in the layout to place newMember (starts with 0);              if omitted, it will be added at the last position
*/
addMember(newMember:Canvas, position?:Integer): void;

/**
*  Add one or more canvases to the layout, optionally at a specific position. See
*  Layout.addMember for details.
* 
* 
* @param {Array<Partial<Canvas>> | Canvas} array of canvases to be added or single Canvas
* @param {number=} position to add newMembers; if omitted newMembers will be added              at the last position
*/
addMembers(newMembers:Array<Partial<Canvas>> | Canvas, position?:number): void;

/**
*  Removes the specified member from the layout. If it has a resize bar, the bar will be
*  destroyed.
* 
* 
* @param {Canvas} the canvas to be removed from the layout
*/
removeMember(member:Canvas): void;

/**
* 
*  Removes the specified members from the layout. If any of the removed members have resize
*  bars, the bars will be destroyed.
* 
* 
* @param {Array<Partial<Canvas>> | Canvas} array of members to be removed, or single member
*/
removeMembers(members:Array<Partial<Canvas>> | Canvas): void;

/**
*  Display a new set of members in this layout. Equivalent to calling removeMembers() then
*  addMembers(). Note that the new members may include members already present, in which case
*  they will be reordered / integrated with any other new members passed into this method.
* 
* @param {Array<Partial<Canvas>>} */
setMembers(members:Array<Partial<Canvas>>): void;

/**
*  Show the specified member, firing the specified callback when the show is complete.
*  
*  Members can always be directly shown via member.show(), but if
*  Layout.animateMembers is enabled, animation will only occur if showMember() is
*  called to show the member.
* 
* 
* @param {Canvas} Member to show
* @param {Function=} action to fire when the member has been shown
*/
showMember(member:Canvas, callback?:Function): void;

/**
*  Hide the specified member, firing the specified callback when the hide is complete.
*  
*  Members can always be directly hidden via member.hide(), but if
*  Layout.animateMembers is enabled, animation will only occur if hideMember() is
*  called to hide the member.
* 
* 
* @param {Canvas} Member to hide
* @param {Function=} callback to fire when the member is hidden.
*/
hideMember(member:Canvas, callback?:Function): void;

/**
*  Hide all other members and make the single parameter member visible.
* 
* 
* @param {Canvas} member to show
*/
setVisibleMember(member:Canvas): void;

/**
*  Shift a member of the layout to a new position
*    
* 
* @param {number} current position of the member to move to a new position
* @param {number} new position to move the member to
*/
reorderMember(memberNum:number, newPosition:number): void;

/**
*  Move a range of members to a new position
*    
* 
* @param {number} beginning of range of members to move
* @param {number} end of range of members to move, non-inclusive
* @param {number} new position to move the members to
*/
reorderMembers(start:number, end:number, newPosition:number): void;

/**
*  Replaces an existing member of the layout with a different widget. The new member will be
*  assigned the width and height of the existing member (including sizes configured via end
*  user resize), so no reflow will occur unless the new component has visible overflow and it
*  differs from that of the widget it replaced.
* 
* 
* @param {Canvas} an existing member of the layout to be replaced
* @param {Canvas} a different widget that should replace oldMember
*/
replaceMember(oldMember:Canvas, newMember:Canvas): void;

/**
*  Fires once at initialization if the layout has any initial members, and then fires whenever
*  members are added, removed or reordered.
*  
* 
*/
membersChanged(): void;

/**
*  Layouts ensure children are ordered
*  in the tab-sequence with members being reachable first (in member order), then any
*  non-member children.
*  
*  As with Canvas.getChildTabPosition if Canvas.setRelativeTabPosition
*  was called explicitly called for some child, it will be respected over member order.
* 
* 
* @param {Canvas} The child for which the tab position should be returned
*/
getChildTabPosition(child:Canvas): Integer;

/**
*  When Layout.canDropComponents is true, this method will be called when a component is
*  dropped onto the layout to determine what component to add as a new layout member.
*  
*  By default, the actual component being dragged (isc.EventHandler.getDragTarget()) will be
*  added to the layout. For a different behavior, such as wrapping dropped components in
*  Windows, or creating components on the fly from dropped data, override this method. 
*  
*  You can also return null to cancel the drop.
* 
* 
* @param {Canvas} current drag target
* @param {Integer} index of the drop in the list of current members
*/
getDropComponent(dragTarget:Canvas, dropPosition:Integer): Canvas;

/**
*  Layouts have built-in handling of component drag and drop. See the discussion in
*  Layout on how it works. If you override this builtin implementation of drop() and
*  you're using the built-in dropLine via Layout.canDropComponents:true, be sure to call
*  Layout.hideDropLine to hide the dropLine after doing your custom drop() handling.
* 
* 
*/
drop(): boolean;

/**
* 
*  Get the position a new member would be dropped. This drop position switches in the
*  middle of each member, and both edges (before beginning, after end) are legal drop positions
*  
*  Use this method to obtain the drop position for e.g. a custom drop handler.
* 
* 
*/
getDropPosition(): Integer;

/**
*  Calling this method hides the dropLine shown during a drag and drop interaction with a
*  Layout that has Layout.canDropComponents set to true. This method is only useful for
*  custom implementations of Layout.drop as the default implementation calls this
*  method automatically.
* 
* 
*/
hideDropLine(): void;



}
/**
* Creates a new Layout
* 
* @param typeCheckedProperties {Partial<Layout>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Layout} The new instance
*/
class Layout{
static create(typeCheckedProperties? : Partial<Layout>, uncheckedProperties?: Object): Layout;
/**
* A declared value of the enum type 
*  Orientation.
* @type {Constant}
* @default "vertical"
*/
static VERTICAL?: Constant;

/**
* A declared value of the enum type 
*  Orientation.
* @type {Constant}
* @default "horizontal"
*/
static HORIZONTAL?: Constant;

/**
* A declared value of the enum type 
*  LayoutPolicy.
* @type {Constant}
* @default "none"
*/
static NONE?: Constant;

/**
* A declared value of the enum type 
*  LayoutPolicy.
* @type {Constant}
* @default "fill"
*/
static FILL?: Constant;








}

/**
* EditProxy that handles Layout objects when editMode is enabled.
*/
interface LayoutEditProxy extends CanvasEditProxy {




}
/**
* Creates a new LayoutEditProxy
* 
* @param typeCheckedProperties {Partial<LayoutEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {LayoutEditProxy} The new instance
*/
class LayoutEditProxy{
static create(typeCheckedProperties? : Partial<LayoutEditProxy>, uncheckedProperties?: Object): LayoutEditProxy;







}

/**
* This class exists principally to make it easier to create resize bars when using visual
*  design tools, since a LayoutResizeBar can be dropped into a Layout like any
*  other Canvas. The recommended way to create a resize bar is to simply set 
*  Canvas.showResizeBar on the member that you want to be able to resize or collapse,
*  creating it as an Layout.resizeBar.
*  
*  Note that this class extends whatever class is specified as the prototype default for
*  Layout.resizeBarClass in the current skin. So for some skins it may actually extend
*  Snapbar rather than Splitbar.
*/
interface LayoutResizeBar {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "LayoutResizeBarEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* If this property is true, a click on the Splitbar will collapse its 
*  Splitbar.target, hiding it and shifting the Splitbar and other members
*  of the layout across to fill the newly available space. If the target is already hidden
*  a click will expand it again (showing it at its normal size).
*  
*  Note that on touch devices, to enable collapsing/uncollapsing the target
*  in response to a tap, Splitbar.canCollapseOnTap must be set to
*  true.
* @type {boolean}
* @default true
*/
canCollapse?: boolean;

/**
* Whether this LayoutResizeBar Layout.members should
*  resize the member before or after it in the layout. If LayoutResizeBar.canCollapse is true, this
*  property also determines which member is hidden when the LayoutResizeBar is
*  clicked.
*  
*  Compare this property with the corresponding setting Canvas.resizeBarTarget,
*  meaningful when an autochild resize bar is created with Canvas.showResizeBar.
*  Note that if such an autochild would be shown right before an LayoutResizeBar
*  member, due to Canvas.showResizeBar or 
*  Layout.defaultResizeBars, it will be disabled in favor of the
*  LayoutResizeBar, though the Canvas.resizeBarTarget
*  setting will be applied if this property's prototype default hasn't been overridden in
*  the resize bar instance.
* @type {ResizeDirection}
* @default "before"
*/
resizeDirection?: ResizeDirection;



/**
*  Setter for ResizeDirection.
* 
* @param {ResizeDirection} the new direction to target
*/
setResizeDirection(direction:ResizeDirection): void;



}
/**
* Creates a new LayoutResizeBar
* 
* @param typeCheckedProperties {Partial<LayoutResizeBar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {LayoutResizeBar} The new instance
*/
class LayoutResizeBar{
static create(typeCheckedProperties? : Partial<LayoutResizeBar>, uncheckedProperties?: Object): LayoutResizeBar;







}

/**
* EditProxy that handles LayoutResizeBar
*  objects when editMode is enabled.
*/
interface LayoutResizeBarEditProxy extends EditProxy {




}
/**
* Creates a new LayoutResizeBarEditProxy
* 
* @param typeCheckedProperties {Partial<LayoutResizeBarEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {LayoutResizeBarEditProxy} The new instance
*/
class LayoutResizeBarEditProxy{
static create(typeCheckedProperties? : Partial<LayoutResizeBarEditProxy>, uncheckedProperties?: Object): LayoutResizeBarEditProxy;







}

/**
* Add a spacer to a Layout that takes up space just like a normal member, without actually
*  drawing anything. A LayoutSpacer is semantically equivalent to using an empty canvas,
*  but higher performance for this particular use case.
*/
interface LayoutSpacer extends Canvas {




}
/**
* Creates a new LayoutSpacer
* 
* @param typeCheckedProperties {Partial<LayoutSpacer>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {LayoutSpacer} The new instance
*/
class LayoutSpacer{
static create(typeCheckedProperties? : Partial<LayoutSpacer>, uncheckedProperties?: Object): LayoutSpacer;







}

/**
* A form item that displays a URL. In the default read-only mode (FormItem.canEdit
*  is false) the URL is shown as a link; in editable mode the URL is shown in a textbox.
*  
*  The link to open is specified as the item value with FormItem.setValue or 
*  FormItem.defaultValue. The link title defaults to the URL unless
*  LinkItem.linkTitle is specified.
*  
*  Additionally, a custom action can be triggered when the link is clicked: see
*  LinkItem.target for details.
*/
interface LinkItem extends TextItem {
/**
* If FormItem.canEdit is set to false, should FormItem.icons be disabled
*  by default?
*  
*  This may also be specified at the icon level. See FormItemIcon.disableOnReadOnly.
* @type {boolean}
* @default false
*/
disableIconsOnReadOnly?: boolean;

/**
* How should icons be aligned vertically for this form item.
* @type {VerticalAlignment}
* @default "center"
*/
iconVAlign?: VerticalAlignment;

/**
* If FormItem.canEdit is set to false, how should this
*  LinkItem be displayed to the user?
*  
*  Link items are, by default, canEdit:false. Note that the link remains active regardless
*  of the readOnlyDisplay setting.
* @type {ReadOnlyDisplayAppearance}
* @default null
*/
readOnlyDisplay?: ReadOnlyDisplayAppearance;

/**
* By default, clicking a link rendered by this item opens it in a new browser window. You 
*  can alter this behavior by setting this property. The value of this property will be 
*  passed as the value to the target attribute of the anchor tag used to render 
*  the link.
*  
*  If you set linkItem.target to "javascript", the default behaviour is to catch and consume
*  mouse-clicks that would result in the link being followed. Instead, the
*  FormItem.click event is fired.
* @type {string}
* @default "_blank"
*/
target?: string;

/**
* Optional title HTML to display for this item's link. If unset, the LinkItem's
*  value (the URL) will be used for the link's title.
* @type {HTMLString}
* @default null
*/
linkTitle?: HTMLString;



/**
*  Setter for LinkItem.linkTitle.
* 
* @param {HTMLString} new linkTitle HTML.
*/
setLinkTitle(title:HTMLString): void;



}
/**
* Creates a new LinkItem
* 
* @param typeCheckedProperties {Partial<LinkItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {LinkItem} The new instance
*/
class LinkItem{
static create(typeCheckedProperties? : Partial<LinkItem>, uncheckedProperties?: Object): LinkItem;







}

/**
* A ListGrid is a DataBoundComponent that displays a list of objects in a grid, where
*  each row represents one object and each cell in the row represents one property.
*/
interface ListGrid extends VLayout, Omit<DataBoundComponent, 'fields'>  {
/**
* The CSS style name applied to the DataBoundComponent.offlineMessage if displayed.
* @type {CSSStyleName}
* @default "offlineMessage"
*/
offlineMessageStyle?: CSSStyleName;

/**
* If true, when this component is first drawn, automatically call this.fetchData().
*  Criteria for this fetch may be picked up from ListGrid.initialCriteria, and textMatchStyle may
*  be specified via ListGrid.autoFetchTextMatchStyle.
*  
*  NOTE: if autoFetchData is set, calling
*  ListGrid.fetchData before draw will cause two requests to be issued, one from the manual
*  call to fetchData() and one from the autoFetchData setting. The second request will use
*  only ListGrid.initialCriteria and not any other criteria or settings from the first request.
*  Generally, turn off autoFetchData if you are going to manually call ListGrid.fetchData at any time.
* 
*  Note: If you are using saved searches - either via SavedSearchItem or ListGrid.saveDefaultSearch, autoFetchData
*  will be automatically suspended and replaced with the saved criteria/view state, if applicable.
* @type {boolean}
* @default false
*/
autoFetchData?: boolean;

/**
* Criteria to be used when ListGrid.autoFetchData is set.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
initialCriteria?: Criteria;

/**
* Criteria that are never shown to or edited by the user and are cumulative with any criteria
*  provided via DataBoundComponent.initialCriteria and related methods.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use
*  them, see the gridFiltering.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
*  
*  To see implicitCriteria in action, take a look at 
*  this example
*  of using implicitCriteria to have an external form filter the grid in parallel with the 
*  ListGrid.showFilterEditor.
* @type {Criteria}
* @default null
*/
implicitCriteria?: Criteria;

/**
* Default background color to use when exporting data to Excel/OpenOffice format using
*  ListGrid.exportData or 
*  ListGrid.exportClientData.
*  
*  If unset (the default), cells that are not provided a background color by more specific APIs
*  will be the default background color used by the spreadsheet program where they are viewed.
*  
*  See exportBGColor for an overview.
* @type {CSSColor}
* @default null
*/
exportDefaultBGColor?: CSSColor;

/**
* When exporting data to Excel/OpenOffice format using ListGrid.exportData or
*  ListGrid.exportClientData, background color to use 
*  for even-numbered rows, to create a "banded" or "ledger" effect. Odd-numbered rows will 
*  use the ListGrid.exportDefaultBGColor.
*  
*  See exportBGColor for an overview.
* @type {CSSColor}
* @default null
*/
exportAlternateRowBGColor?: CSSColor;

/**
* When exporting data to Excel/OpenOffice format using ListGrid.exportData or
*  ListGrid.exportClientData, causes the ListGrid.headerHeight and
*  HeaderSpan.height to be applied to the corresponding cells in the
*  spreadsheet.
* @type {boolean}
* @default false
*/
exportHeaderHeights?: boolean;

/**
* When exporting data to Excel/OpenOffice format using ListGrid.exportData or
*  ListGrid.exportClientData, whether widths of fields should be replicated
*  in the resulting spreadsheet.
*  
*  Because Excel's unit of measurement for field widths is based on the default system font,
*  there is no exact way to translate field widths in pixels to Excel column widths. The
*  ListGrid.exportWidthScale property can be set to adjust scaling; it's default value errs on
*  the side of making Excel's columns slightly wider than the ListGrid field's actual width to
*  avoid clipping.
*  
*  Note that you can switch off width export for individual fields with the 
*  ListGridField.exportFieldWidth flag.
* @type {boolean}
* @default false
*/
exportFieldWidths?: boolean;

/**
* Scaling factor to translate from ListGrid field widths in pixels to Excel/OpenOffice units
*  for field width, which are 1/256th of the width of the widest digit character in the default
*  font for the spreadsheet. See ListGrid.exportFieldWidths for where this is used.
* @type {Float}
* @default "0.12"
*/
exportWidthScale?: Float;

/**
* When exporting data to Excel/OpenOffice format using ListGrid.exportData or
*  ListGrid.exportClientData, whether titles in the
*  ListGrid.header and ListGrid.headerSpans should be
*  allowed to wrap.
*  
*  Excel will wrap at the column boundary automatically; for explicit control over wrapping,
*  insert "&lt;br&gt;" tags into your titles.
*  
*  See also ListGrid.exportFieldWidths for replicating the widths of fields in the exported
*  spreadsheet.
* @type {boolean}
* @default false
*/
exportWrapHeaderTitles?: boolean;

/**
* When exporting data to Excel/OpenOffice format using ListGrid.exportData or
*  ListGrid.exportClientData, whether field
*  ListGridField.align and 
*  ListGridField.cellAlign should be replicated in the resulting 
*  spreadsheet. 
*  
*  If this attribute is not set, cells will be assigned a default alignment by the spreadsheet,
*  which is typically right-aligned for numeric and date values, and left-aligned for 
*  everything else (including dates and numbers that have been exported as strings, as would
*  be the case, for example, if DSRequest.exportDatesAsFormattedString is set)
* @type {boolean}
* @default false
*/
exportFieldAlignments?: boolean;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "GridEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* ARIA role for this ListGrid if isc.setScreenReaderMode is
*  enabled.
*  
*  The WAI-Aria standards
*  contain a number of roles and related attributes that could apply to data presented in
*  a ListGrid or its subclasses. In order to make screenreader support as straightforward
*  as possible we have built-in support for writing out appropriate aria roles and 
*  attributes on the listGrid and its component elements for a couple of standard modes, as
*  well as providing override points allowing developers to explicitly specify the 
*  properties that get written out.
*  
*  The two "standard" ariaRoles supported for ListGrids are 
*  "grid" and
*  "list".
*  
*  When ariaRole is set to "list" we write out the following 
*  standard properties by default:
*  
*  
* - rows have role set to "listitem"
*  
* - ListGrid.getRowAriaState will return aria properties for 
*    setsize, posinset, 
*    selected (for selected rows) and 
*    expanded (for ListGrid.canExpandRecords rows)
*  
* - Additionally, if ListGrid.screenReaderWriteRowLabelledBy is true, 
*    rows will write out an aria-labelldby that will cause ScreenReaders
*    to read the column header and cell / row separators in addition to the cell
*    content for the row
*  
*  
*  When ariaRole is set to "grid" we write out the following 
*  standard properties by default:
*  
*  
* - aria-rowcount and aria-colcount will be specified on
*    the listGrid itself
*  
* - The ListGrid.header will have role row 
*    and aria-rowindex set to 1
*  
* - Column header buttons will have role columnheader, and 
*   aria-colindex set to the appropriate value for the column.
*   Additionally aria-sort will be specified to reflect the current sort-state
*   for the field, and if the header menu is enabled, aria-haspopup will be
*   true
*  
* - Rows within the grid body will have role row
*  
* - ListGrid.getRowAriaState will return aria properties for 
*    rowindex, selected (for selected rows) and 
*    expanded (for ListGrid.canExpandRecords rows)
*  
* - Cells within rows will have role gridcell
*  
*  Developers may configure different ARIA HTML roles and attributes by modifying 
*  this attribute (listGrid.ariaRole) and implementing custom
*  handling for the following APIs:
*  
*  
*   ListGrid
*   listGrid.ariaRole, Canvas.ariaState, 
*     ListGrid.getAriaState.
*  
*   header / header buttons
*   ListGrid.headerAriaRole, 
*    ListGrid.headerButtonAriaRole, ListGridField.headerButtonAriaRole
*    ListGrid.headerButtonAriaState, ListGridField.headerButtonAriaState
*  
*   rows
*   ListGrid.rowRole, ListGrid.recordRowRoleProperty,
*   ListGrid.rowAriaState, ListGrid.recordRowAriaStateProperty
*   , ListGrid.getRowRole, ListGrid.getRowAriaState. 
*   To update row state
*   at runtime, developers may redraw the grid or its body.
*  
*   cells
*   ListGrid.cellRole, ListGrid.recordCellRoleProperty
*   , ListGrid.getCellRole, ListGrid.getCellAriaState. 
*   To update cell state
*   at runtime, developers may redraw the grid or its body.
*  
*  
* @type {string}
* @default "list"
*/
ariaRole?: string;

/**
* Returns the default WAI ARIA role for cells within this listGrid.
*  See ListGrid.getCellRole
* @type {string}
* @default null
*/
cellRole?: string;

/**
* If this property is set on a record it will designate the WAI ARIA role for cells
*  within this records row
* @type {string}
* @default "cellRole"
*/
recordCellRoleProperty?: string;

/**
* Returns the default WAI ARIA role for rows within this listGrid.
*  See ListGrid.getRowRole
* @type {string}
* @default null
*/
rowRole?: string;

/**
* If this property is set on a record it will designate the WAI ARIA role for this
*  record's row.
* @type {string}
* @default "rowRole"
*/
recordRowRoleProperty?: string;

/**
* Returns a mapping of default WAI ARIA attributes for rows within this listGrid.
*  See ListGrid.getRowAriaState
* @type {object}
* @default null
*/
rowAriaState?: object;

/**
* If this property is set on a record it will designate a mapping of WAI ARIA attribute
*  names and values for this record's row.
* @type {string}
* @default "rowAriaState"
*/
recordRowAriaStateProperty?: string;

/**
* Canvas.ariaRole for this listGrid's ListGrid.showHeader.
*  See ListGrid.getHeaderAriaRole
* @type {string}
* @default null
*/
headerAriaRole?: string;

/**
* Default Canvas.ariaRole for ListGrid.showHeader.
*  See ListGrid.getHeaderButtonAriaRole.
* @type {string}
* @default null
*/
headerButtonAriaRole?: string;

/**
* Default Canvas.ariaState for ListGrid.showHeader.
*  See ListGrid.getHeaderButtonAriaState.
* @type {object}
* @default null
*/
headerButtonAriaState?: object;

/**
* If ListGrid.showHover is true, should cell hover HTML be displayed on disabled cells?
* @type {boolean}
* @default false
*/
showHoverOnDisabledCells?: boolean;

/**
* Default CSS class for the ListGrid as a whole.
* @type {CSSStyleName}
* @default "listGrid"
*/
styleName?: CSSStyleName;

/**
* A list of ListGridRecord objects, specifying the data to be used to populate the
*  ListGrid. In ListGrids, the data array specifies rows.
*  
*  When using a DataSource, rather than directly providing data, you will
*  typically call ListGrid.fetchData instead, which will automatically establish
*  data as a ResultSet (see the ListGrid.fetchData docs for details).
*  
*  If you call fetchData, any previously supplied data is
*  discarded. Also, it is not necessary to call setData() after calling
*  ListGrid.fetchData.
*  
*  When calling setData(), direct changes
*  to the list using Framework APIs such as List.add or 
*  List.remove will be automatically observed
*  and the
*  ListGrid will redraw in response. However, direct changes to individual Records will not
*  be automatically observed
*  and require calls to ListGrid.refreshCell or 
*  ListGrid.refreshRow to cause the ListGrid to visually update. Calling methods such as
*  ListGrid.updateData, ListGrid.removeData or ListGrid.addData always causes
*  automatic visual refresh.
*  
*  Note that direct manipulation of the data object without using the List APIs (for
*  example by directly assigning a new Record object to some index or calling non-Framework
*  APIs such as pop(), shift(), etc.) will not be reflected in the grid automatically, but 
*  developers can call List.dataChanged directly to notify the grid of changes.
*  
* @type {Array<Partial<ListGridRecord>>}
* @default null
*/
data?: Array<Partial<ListGridRecord>>;

/**
* Property name on a record that will be checked to determine whether a record is enabled.
*  
*  Setting this property on a record will effect the visual style and interactivity of
*  the record. If set to false the record (row in a ListGrid or
*  TreeGrid) will not highlight when the mouse moves over it, nor will it respond to
*  mouse clicks.
* @type {string}
* @default "enabled"
*/
recordEnabledProperty?: string;

/**
* Property name on a record that will be checked to determine whether a record can be
*  expanded.
* @type {string}
* @default "canExpand"
*/
canExpandRecordProperty?: string;

/**
* If set to false on a record and ListGrid.canRemoveRecords is
*  true, removal of that record is disallowed in the UI. The icon in the remove field
*  is not shown.
* @type {string}
* @default "_canRemove"
*/
recordCanRemoveProperty?: string;

/**
* If true, when folders are opened / closed children will be animated into view.
*  
*  For a ListGrid, this property applies when ListGrid.canGroupBy is enabled.
* @type {boolean}
* @default true
*/
animateFolders?: boolean;

/**
* If ListGrid.animateFolders is true for this grid, this number can be set to designate
*  the maximum number of rows to animate at a time when opening / closing a folder.
*  
*  For a ListGrid, this property applies when ListGrid.canGroupBy is enabled.
* @type {Integer}
* @default null
*/
animateFolderMaxRows?: Integer;

/**
* When animating folder opening / closing, if TreeGrid.animateFolderSpeed is not
*  set, this property designates the duration of the animation in ms.
*  
*  For a ListGrid, this property applies when ListGrid.canGroupBy is enabled.
* @type {number}
* @default "100"
*/
animateFolderTime?: number;

/**
* When animating folder opening / closing, this property designates the speed of the
*  animation in pixels shown (or hidden) per second. Takes precedence over the
*  TreeGrid.animateFolderTime property, which allows the developer to specify a
*  duration for the animation rather than a speed.
*  
*  For a ListGrid, this property applies when ListGrid.canGroupBy is enabled.
* @type {number}
* @default "3000"
*/
animateFolderSpeed?: number;

/**
* When animating folder opening / closing, this property can be set to apply an
*  animated acceleration effect. This allows the animation speed to be "weighted", for
*  example expanding or collapsing at a faster rate toward the beginning of the animation
*  than at the end.
*  
*  For a ListGrid, this property applies when ListGrid.canGroupBy is enabled.
* @type {AnimationAcceleration}
* @default null
*/
animateFolderEffect?: AnimationAcceleration;

/**
* An array of field objects, specifying the order, layout, formatting, and
*  sorting behavior of each field in the listGrid object. In ListGrids, the fields
*  array specifies columns. Each field in the fields array is a ListGridField object.
*  Any listGrid that will display data should have at least one visible field.
*  
*  If ListGrid.dataSource is also set, this value acts as a set of overrides as
*  explained in DataBoundComponent.fields.
* @type {Array<Partial<ListGridField>>}
* @default null
*/
fields?: Array<Partial<ListGridField>>;

/**
* An array of listGrid field configuration objects. When a listGrid is initialized, if this
*  property is set and there is no value for the fields attribute, this.fields will
*  be defaulted to a generated array of field objects duplicated from this array.
*  
*  This property is useful for cases where a standard set of fields will be displayed
*  in multiple listGrids - for example a subclass of ListGrid intended to display a particular
*  type of data:
*  In this example we would not assign a single ListGrid.fields array directly to the
*  class via addProperties() as every generated instance of this class would
*  then point to the same fields array object. This would cause unexpected behavior such as
*  changes to the field order in one grid affecting other grids on the page.
*  Instead we could use addProperties() on our new subclass to set
*  defaultFields to a standard array of fields to display. Each generated instance
*  of the subclass would then show up with default fields duplicated from this array.
* @type {Array<Partial<Partial<ListGridField>>>}
* @default null
*/
defaultFields?: Array<Partial<Partial<ListGridField>>>;

/**
* The DataSource that this component should bind to for default fields and for performing
*  DSRequest.
*  
*  Can be specified as either a DataSource instance or the String ID of a DataSource.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* A ListGrid is a DataBoundComponent.dataArity:multiple component.
* @type {string}
* @default "multiple"
*/
dataArity?: string;

/**
* If true, for fields where ListGridField.optionDataSource is specified,
*  a valueMap will be automatically created by making a DataSource.fetchData call
*  against the specified dataSource and extracting a valueMap from the returned records
*  based on the displayField and valueField.
*  
*  If set to false, valueMaps will not be automatically fetched. In this case, setting
*  field.optionDataSource is effectively a shortcut for setting optionDataSource on
*  the editor via ListGridField.editorProperties.
*  
*  Can also be disabled on a per-field basis with ListGridField.autoFetchDisplayMap.
* @type {boolean}
* @default true
*/
autoFetchDisplayMap?: boolean;

/**
* If a field has ListGridField.displayField specified and has no
*  ListGridField.optionDataSource, this field will display the value from the
*  displayField of each record by default (for more on this behavior
*  see ListGridField.optionDataSource).
*  
*  If such a field is editable, changing the edit value for the field on some record,
*  without updating the edit value for the associated display field on the same record
*  would mean the user would continue to see the unchanged display field value.
*  Developers can resolve this situation by programmatically setting an edit value for
*  the display field as well as the data field, or avoid it by specifying an optionDataSource
*  and ensuring ListGrid.autoFetchDisplayMap is true, or setting an explicit valueMap
*  for the field.
*  
*  By default, when the edit value on a field with a specified displayField and
*  no optionDataSource is set, we log a warning to notify the developer. This warning may
*  be disabled by setting warnOnUnmappedValueFieldChange to false.
*  
*  Note: There are actually a couple of cases in which the system will automatically
*  derive a new display-field value and apply it to the record:
*  
* - If the edit value was changed by a user actually editing the record
*  (rather than a programmatic call to setEditValue()), and the edit-item had
*  a valueMap or optionDataSource set, we automatically pick up the display value from
*  that item and store it as an edit-value for the displayField of the record
*    
* - If the listGrid has a loaded record in its data set whose valueField value matches
*  the edit value for the valueField, we automatically apply the displayField value from that
*  record as an edit value for the displayField on the newly edited record.
*  In either case, the display value for the record is updated automatically
*  (and the warning would not be logged).
* @type {boolean}
* @default true
*/
warnOnUnmappedValueFieldChange?: boolean;

/**
* For grids with a specified ListGrid.dataSource, this property can be set to
*  true to cause the grid directly update its local data set instead of
*  performing an operation against it's configured DataSource.
*  
*  When using this mode, data must be provided to the grid via ListGrid.setData,
*  and must be provided as 
*  a simple Array of Records
*  . Setting saveLocally is invalid if
*  either ListGrid.fetchData is called or if a ResultSet is provided as the data
*  model.
*  
*  saveLocally mode includes changes made via 
*  ListGrid.canEdit, record removal via ListGrid.canRemoveRecords, as
*  well as programmatic calls to ListGrid.updateData,
*  ListGrid.addData and ListGrid.removeData. This
*  also causes saves to be performed synchronously (unlike normal DataSource operations).
*  
*  Note that using this mode also disables the automatic cache synchronization provided by
*  the DataSource system - changes made to this grid are saved only to this grid's data
*  set.
*  
*  See also ListGrid.filterLocalData to allow filtering, such as filtering performed
*  by the ListGrid.filterEditor, to also work only with the local data set.
*  
*  If saveLocally is unset, and ListGrid.filterLocalData is true, the saveLocally behavior is
*  enabled by default
* @type {boolean}
* @default null
*/
saveLocally?: boolean;

/**
* For editable grids with a specified ListGrid.dataSource, where
*  ListGrid.saveLocally is false, this attribute may be used to specify standard
*  DSRequest properties to apply to all save operations performed by this grid (whether
*  triggered by user interaction, or explicit saveEdits or saveAllEdits call).
*  
*  An example usage would be to customize the prompt displayed while saving is in progress
*  if ListGrid.waitForSave is true.
*  
*  Note that for more advanced customization of save operations,
*  DataBoundComponent.addOperation and DataBoundComponent.updateOperation
*  are available to developers, allowing specification of an explicit OperationBinding
*  for the add / update operation performed on save.
* @type {Partial<DSRequest>}
* @default null
*/
saveRequestProperties?: Partial<DSRequest>;

/**
* If ListGrid.saveLocally is specified, but this grid is bound to a DataSource which
*  includes remote field validators, by default edits will be saved synchronously and
*  these validators will not be executed.
*  Set this property to true to ensure these remote validators are called when
*  saving edits in saveLocally mode. Note that since these remote validators need to run on
*  the server, saving with this property set is asynchronous, even though the data that
*  ultimately gets updated is already present on the client.
* @type {boolean}
* @default null
*/
useRemoteValidators?: boolean;

/**
* If true, the set of fields given by the "default binding" (see 
*  DataBoundComponent.fields) is used, with any fields specified in
*  component.fields acting as overrides that can suppress or modify the
*  display of individual fields, without having to list the entire set of fields that
*  should be shown.
*  
*  If component.fields contains fields that are not found in the DataSource,
*  they will be shown after the most recently referred to DataSource field. If the new
*  fields appear first, they will be shown first.
*  
*  ${isc.DocUtils.linkForExampleId('validationFieldBinding', 'This example')} shows a mixture of component
*  fields and DataSource fields, and how they interact for validation.
*  
*  This setting may be cleared if a FieldPicker is used to edit the component's field
*  order.
* @type {boolean}
* @default null
*/
useAllDataSourceFields?: boolean;

/**
* Whether to include fields marked detail:true from this component's
*  DataSource.
*  
*  When this property is true, the ListGrid will include all
*  detail fields unless fields have been specifically declared using the
*  ListGrid.fields array.
*  
*  Any field which has been included directly in the fields array will be
*  included regardless of the fields detail attribute.
*  
*  Detail fields included will initially be hidden but the user may show these fields via
*  the default header context menu (ListGrid.showHeaderContextMenu).
*  
*  The field's visibility can also be overridden programatically using the standard
*  ListGrid.showField, ListGrid.hideField and ListGridField.showIf
*  APIs, for example, set showIf:"true" to show a detail field initially.
*  
*  Setting this property to false will completely exclude all detail fields from the list
*  grid's fields array, such that they cannot be shown by the user or programmatically.
* @type {boolean}
* @default true
*/
showDetailFields?: boolean;

/**
* Should ellipses be displayed when cell content is clipped? May be overridden at the field 
*  level via ListGridField.showEllipsisWhenClipped
* @type {boolean}
* @default true
*/
showEllipsisWhenClipped?: boolean;

/**
* Best field to use for a user-visible title for an individual record from this grid.
*  If ListGrid.dataSource is non null, this property may be specified on the
*  dataSource instead.
*  
*  If not explicitly set, titleField looks for fields named "title", "name", and "id"
*  in that order. If a field exists with one of those names, it becomes the titleField.
*  If not, then the first field is designated as the titleField.
* @type {string}
* @default "see below"
*/
titleField?: string;

/**
* For databound ListGrids, this attribute can be used to customize the ResultSet
*  object created for this grid when data is fetched
* @type {ResultSet}
* @default null
*/
dataProperties?: ResultSet;

/**
* Saves the name of the default search to localStorage, under the key "sc_defaultSearch:" + minimal
*  locator.
*  
*  See SavedSearches "Default Searches" for why this is stored separately from the search itself
*  if there is no saved default search, but there is a SavedSearches.defaultField, the first
*  user-specfic search marked default will be used, otherwise the first admn search marked default
*  
*  If ListGrid.autoFetchData is enabled, the autoFetch will be postponed as needed until a default search can be
*  looked up and applied. If anything fails in the default search lookup, an autoFetch will still be
*  performed (without any saved search information)
* @type {boolean}
* @default true
*/
saveDefaultSearch?: boolean;

/**
* When enabled (the default), causes a "Saved views >" submenu to appear in the header
*  context menu, last, allowing the user to create new saved searches, select previously
*  saved searches, and edit or copy existing searches.
*  
*  Note that disabling this feature does not mean that saved searches are disallowed - you
*  can stil implement a separate UI via a SavedSearchItem that targets this component.
*  But, when disabled, the grid-integrated menu described above will not be shown.
*  
*  This feature uses the global settings found in SavedSearches. Some aspects can
*  be overridden for this component. See ListGrid.savedSearchDS, ListGrid.savedSearchAdminRole.
*  
*  To avoid leaking local storage, this setting will be disregarded, disabling the feature,
*  unless the grid specifies DataBoundComponent.savedSearchId, or an
*  explicit Canvas.getLocalId is present.
*  
*  Note: this feature requires
*  SmartClient Pro or better.
* @type {boolean}
* @default true
*/
canSaveSearches?: boolean;

/**
* Set to "criteria" if you want only criteria to be stored for ListGrids and TreeGrids instead of the full viewState of the component.
* @type {SavedSearchStoredState}
* @default null
*/
savedSearchStoredState?: SavedSearchStoredState;

/**
* Text to show in menu listing saved searches when there are no saved searches.
* @type {HTMLString}
* @default "[None]"
*/
noSavedSearchesText?: HTMLString;

/**
* Text to show for the saved searches submenu.
* @type {HTMLString}
* @default "Saved views"
*/
savedSearchText?: HTMLString;

/**
* Text to show for saving the current view as a new "saved search".
* @type {HTMLString}
* @default "Save View..."
*/
newSearchText?: HTMLString;

/**
* Properties for the separator record between locally saved and admin searches.
* @type {ListGridRecord}
* @default "{isSeparator:true}"
*/
savedSearchAdminSeparator?: ListGridRecord;

/**
* Override of SavedSearches.defaultDataSource for this component.
* @type {string}
* @default null
*/
savedSearchDS?: string;

/**
* Override of SavedSearches.adminRole for this component.
* @type {string}
* @default null
*/
savedSearchAdminRole?: string;

/**
* When grouped, a copy of the original ungrouped data.
* @type {object}
* @default null
*/
originalData?: object;

/**
* The data tree that results from a call to ListGrid.groupBy.
*  This will be a ResultTree if ListGrid.dataSource is
*  present, otherwise it will be a Tree.
* @type {Tree}
* @default null
*/
groupTree?: Tree;

/**
* Describes the default state of ListGrid groups when groupBy is called.
* 
*  Possible values are:
*  
*  
* - "all": open all groups
*  
* - "first": open the first group
*  
* - "none": start with all groups closed
*  
* - Array of group values that should be opened
*  
* @type {GroupStartOpen | Array<any>}
* @default "first"
*/
groupStartOpen?: GroupStartOpen | Array<any>;

/**
* Can a group be collapsed/expanded? When true a collapse/expand icon is shown
*  (ListGrid.groupIcon) and the user can collapse or expand the group by
*  clicking either the row as a whole or the opener icon (see ListGrid.collapseGroupOnRowClick);
* 
*  When false the group icon is not shown and clicking on the row does
*  not change group state. Additionally GroupStartOpen is
*  initialized to "all".
* @type {boolean}
* @default true
*/
canCollapseGroup?: boolean;

/**
* If ListGrid.canCollapseGroup is true, will a click anywhere on the group row
*  toggle the group's expanded state? If false, the user must click the
*  ListGrid.groupIcon directly to toggle the group.
* @type {boolean}
* @default true
*/
collapseGroupOnRowClick?: boolean;

/**
* When a list grid is ListGrid.groupBy, each group shows
*  under an auto generated header node. By default the title of the group will be
*  shown, with a hanging indent in this node, and will span all columns in the grid.
*  Setting this property causes the titles of auto-generated group nodes to appear as
*  though they were values of the designated field instead of spanning all columns
*  and record values in the designated groupTitleField will appear indented under
*  the group title in a manner similar to how a TreeGrid shows a Tree.
*  
*  Note if ListGrid.showGroupSummaryInHeader is true, the header nodes will not show
*  a single spanning title value by default - instead they will show the summary values for
*  each field. In this case, if groupTitleField is unset, a
*  ListGrid.showGroupTitleColumn can be automatically generated to
*  show the title for each group.
* @type {string}
* @default null
*/
groupTitleField?: string;

/**
* If this grid is ListGrid.groupBy and ListGrid.showGroupSummaryInHeader
*  is true, instead of group header nodes showing up with a single cell value spanning the full
*  set of columns, summaries for each field will show up in the appropriate columns of the
*  header node.
*  
*  In this case there are 2 options for where the group title will show up. Developers may
*  specify an existing field to put the title values into via ListGrid.groupTitleField.
*  If no groupTitleField is specified, this property may be set to true
*  which causes a groupTitleColumn to be automatically generated.
*  Each group header will show the group title in this column (records within the group will
*  not show a value for this column). The column appears in the leftmost position within the
*  grid (unless ListGrid.showRowNumbers is true, in which case this column shows up
*  in the second-leftmost position), and by default will auto-fit to its data.
*  
*  To customize this field, developers may modify
*  ListGrid.groupTitleColumnProperties
*  or
*  ListGrid.groupTitleColumnDefaults at the class level.
* @type {boolean}
* @default true
*/
showGroupTitleColumn?: boolean;

/**
* Custom properties for the automatically generated groupTitleColumn.
*  
*  See ListGrid.showGroupTitleColumn for an overview of the groupTitleColumn.
* @type {Partial<ListGridField>}
* @default null
*/
groupTitleColumnProperties?: Partial<ListGridField>;

/**
* Default properties for the automatically generated groupTitleColumn.
*  Default object includes properties to enable autoFitWidth to group title values.
*  
*  To modify the behavior or appearance of this column, developers may set
*  ListGrid.groupTitleColumnProperties at the instance level, or override this
*  object at the class level. If overriding this object, we recommend using
*  Class.changeDefaults rather than replacing this object entirely.
*  
*  See ListGrid.showGroupTitleColumn for an overview of the groupTitleColumn.
* @type {Partial<ListGridField>}
* @default "object"
*/
groupTitleColumnDefaults?: Partial<ListGridField>;

/**
* If this grid is ListGrid.groupBy, and ListGrid.showGroupSummary
*  is true, setting this property causes field summary values for each group to be displayed
*  directly in the group header node, rather than showing up at the bottom of each
*  expanded group.
*  
*  Note that this means the group header node will be showing multiple field values
*  rather than the default display of a single cell spanning all columns containing the
*  group title. Developers may specify an explicit ListGrid.groupTitleField, or
*  rely on the automatically generated ListGrid.showGroupTitleColumn
*  to have group titles be visible as well as the summary values.
*  
*  Also note that multi-line group summaries are not supported when showing
*  the group summary in the group header. If multiple
*  ListGridField.summaryFunction are defined for some field
*  only the first will be displayed when this property is set to true.
* @type {boolean}
* @default false
*/
showGroupSummaryInHeader?: boolean;

/**
* Should group summaries be visible when the group is collapsed?
*  
*  This property only applies to ListGrid.groupBy grids showing
*  ListGrid.showGroupSummary. When set to true, the
*  group summary row(s) for each group will show up under the group header nodes when
*  the group is collapsed, or at then end of the grouped set of data if the group
*  is expanded.
*  
*  This property has no effect if ListGrid.showGroupSummaryInHeader is true.
* @type {boolean}
* @default false
*/
showCollapsedGroupSummary?: boolean;

/**
* The CSS style that ListGrid.groupBy rows will have.
*  
*  Note that this is not a ListGrid.getBaseStyle, so, if this
*  property is set, group nodes will not show stateful styling
*  (different styles for ListGrid.showRollOver,
*  ListGrid.alternateRecordStyles, etc). To enable stateful styling for
*  groupNodes, set this property to null and specify a
*  ListGrid.groupNodeBaseStyle
* @type {string}
* @default "groupNode"
*/
groupNodeStyle?: string;

/**
* ListGrid.getBaseStyle for ListGrid.groupBy rows.
*  
*  Note that this property has no effect if ListGrid.groupNodeStyle is
*  non null.
* @type {string}
* @default null
*/
groupNodeBaseStyle?: string;

/**
* The URL of the base icon for the group icons in this listGrid. Default value may 
*  be overridden by the skinning.
* @type {SCImgURL}
* @default "[SKINIMG]/TreeGrid/opener.gif"
*/
groupIcon?: SCImgURL;

/**
* Default width and height of group icons for this ListGrid.
* @type {number}
* @default "16"
*/
groupIconSize?: number;

/**
* Default number of pixels by which to indent subgroups relative to parent group.
* @type {number}
* @default "20"
*/
groupIndentSize?: number;

/**
* Default number of pixels by which to indent all groups.
* @type {number}
* @default "10"
*/
groupLeadingIndent?: number;

/**
* If false, grouping via context menu will be disabled.
* @type {boolean}
* @default true
*/
canGroupBy?: boolean;

/**
* Maximum number of records to which a groupBy can be applied. If there are more records,
*  grouping will not be available via the default header context menu, and calls to
*  ListGrid.groupBy will be ignored.
*  
*  The maximum exists because ListGrid grouping is performed in-browser, hence requires loading of
*  all records that match the current filter criteria before records can be grouped. The default
*  maximum represents a number of records which are safe to load in legacy browsers such as Internet
*  Explorer 8 (modern browsers can handle far more), and is also a good upper limit from the
*  perspective of loading data from a database.
*  
*  Going beyond this limit can cause "script running slowly" errors from legacy browsers (as well as
*  high database load). To build an interface for grouping that handles arbitrary data volume, use
*  a TreeGrid with TreeGrid.loadDataOnDemand with server-side grouping code.
* @type {Integer}
* @default "1000"
*/
groupByMaxRecords?: Integer;

/**
* Prompt to indicate that grouping is disabled as the
*  data set size exceeds ListGrid.groupByMaxRecords.
*  
*  This prompt will be shown as a hover for the disabled ListGrid.canGroupBy.
*  
*  See also ListGrid.groupByMaxRecordsExceededMessage.
* @type {string}
* @default "Grouping is not supported for datasets of this size"
*/
disabledGroupByPrompt?: string;

/**
* Warning shown to the user when a grouping attempt failed as the data set length exceeds
*  ListGrid.groupByMaxRecords.
*  
*  If defined, this prompt will be shown to the user in a isc.warn
*  when the user attempts to group a data set that exceeds the
*  ListGrid.groupByMaxRecords threshold.
*  
*  This can occur if an already-grouped grid's filter criteria are modified such 
*  that a new set of records is loaded from the DataSource which exceeds the
*  ListGrid.groupByMaxRecords threshold.
*  
*  It can also occur when a user attempts to group a grid with a partially loaded
*  data set where the true size of the data set is not known due to progressiveLoading.
*  In this case, the grouping logic will attempt to retrieve all the 
*  records in the data set and may get back a new total row count from the DataSource
*  which exceeds ListGrid.groupByMaxRecords.
*  
*  In either case the warning will be displayed to the user and the 
*  ListGrid.canGroupBy will be disabled.
*  
*  See also ListGrid.disabledGroupByPrompt.
* @type {string}
* @default "Grouping has been disabled. This data set is too large to apply grouping efficiently."
*/
groupByMaxRecordsExceededMessage?: string;

/**
* When grouping is requested with this number of records or more, an asynchronous approach is
*  used to avoid the browser showing a "script is running slowly.." message prompting the
*  user to stop execution of JavaScript.
*  
*  Note that ListGrid.groupByMaxRecords must be set at least as high as ListGrid.groupByAsyncThreshold
*  or asynchronous grouping will never be used.
*  
*  During async grouping, interactivity is blocked and the ListGrid.asynchGroupingPrompt is shown
*  to the user, then hidden when grouping completes; ListGrid.groupByComplete then fires.
*  
*  Note that this async processing covers grouping only - it does not cover whole grid or
*  per-group summaries, client-side sort or filter, or other operations that may cause the browser
*  to show the "script is running slowly" prompt when working with very large sets of records in a
*  grid.
*  
*  At this time, there is no generally effective way to avoid this warning dialog appearing with very
*  large datasets in Microsoft's Internet Explorer (IE). IE's severely flawed detection algorithm for
*  runaway scripts has been shown to interrupt computations after only 0.2 seconds elapsed time
*  even if the computation would have finished in 0.3 seconds. Optimizations that reduce
*  execution time can sometimes trigger the "script running slowly" dialog sooner. Since not
*  every operation can reasonably be made asynchronous, the current recommendation is to avoid
*  working with overly large datasets until the affected versions of IE are obsoleted.
* @type {Integer}
* @default "50"
*/
groupByAsyncThreshold?: Integer;

/**
* If set to false, do not show the ListGrid.asynchGroupingPrompt dialog during
*  ListGrid.groupByAsyncThreshold.
* @type {boolean}
* @default null
*/
showAsynchGroupingPrompt?: boolean;

/**
* The prompt to display while interactivity is blocked during ListGrid.groupByAsyncThreshold.
* @type {HTMLString}
* @default "${loadingImage}&amp;nbsp;Grouping data..."
*/
asynchGroupingPrompt?: HTMLString;

/**
* True if this listGrid is grouped, false otherwise
* @type {boolean}
* @default false
*/
isGrouped?: boolean;

/**
* Default alias to use for groups with no value
* @type {string}
* @default '-none-'
*/
nullGroupTitle?: string;

/**
* List of fields to group grid records. If only a single field is used, that field
*  may be specified as a string. After initialization, use ListGrid.groupBy
*  to update the grouping field list, instead of modifying groupByField directly.
* @type {string | Array<Partial<string>>}
* @default "see below"
*/
groupByField?: string | Array<Partial<string>>;

/**
* Default width and height of value icons for this ListGrid.
*  Can be overridden at the listGrid level via explicit ListGrid.valueIconWidth and
*  ListGrid.valueIconHeight, or at the field level via ListGridField.valueIconSize,
*  ListGridField.valueIconWidth and {ListGridField.valueIconHeight}
* @type {number}
* @default "16"
*/
valueIconSize?: number;

/**
* Width for value icons for this listGrid.
*  Overrides ListGrid.valueIconSize.
*  Can be overridden at the field level
* @type {number}
* @default null
*/
valueIconWidth?: number;

/**
* Height for value icons for this listGrid.
*  Overrides ListGrid.valueIconSize.
*  Can be overridden at the field level
* @type {number}
* @default null
*/
valueIconHeight?: number;

/**
* How much padding should there be on the left of valueIcons by default
*  Can be overridden at the field level
* @type {number}
* @default "2"
*/
valueIconLeftPadding?: number;

/**
* How much padding should there be on the right of valueIcons by default
* @type {number}
* @default "2"
*/
valueIconRightPadding?: number;

/**
* Specifies a list of icons that can be used in DataBoundComponent.editHilites.
*  
*  hiliteIcons should be specified as an Array of SCImgURL.
*  When present, the hilite editing interface shown when DataBoundComponent.editHilites is called 
*  will offer the user a drop down for picking one of these icons when defining either a 
*  simple or advanced hilite rule.
*  
*  If the user picks an icon, the created hiliting rule will have Hilite.icon set to 
*  the chosen icon. DataBoundComponent.hiliteIconPosition controls where the icon will 
*  appear for that field -- the default is that it appears in front of the normal cell content.
*  This can also be overridden at the field level.
* @type {Array<Partial<string>>}
* @default "["[SKINIMG]/Dialog/notify.png", "[SKINIMG]/Dialog/warn.png", "[SKINIMG]/actions/approve.png"]"
*/
hiliteIcons?: Array<Partial<string>>;

/**
* When ListGrid.hiliteIcons are present, where the hilite icon will be placed 
*  relative to the field value. See HiliteIconPosition. Can be overridden at the
*  field level.
* @type {HiliteIconPosition}
* @default "before"
*/
hiliteIconPosition?: HiliteIconPosition;

/**
* Default width and height of ListGrid.hiliteIcons for this component.
*  Can be overridden at the component level via explicit 
*  ListGrid.hiliteIconWidth and
*  ListGrid.hiliteIconHeight, or at the field level via 
*  ListGridField.hiliteIconSize,
*  ListGridField.hiliteIconWidth and 
*  ListGridField.hiliteIconHeight
* @type {number}
* @default "12"
*/
hiliteIconSize?: number;

/**
* Width for hilite icons for this component.
*  Overrides ListGrid.hiliteIconSize.
*  Can be overridden at the field level.
* @type {number}
* @default null
*/
hiliteIconWidth?: number;

/**
* Height for hilite icons for this listGrid.
*  Overrides ListGrid.hiliteIconSize.
*  Can be overridden at the field level
* @type {number}
* @default null
*/
hiliteIconHeight?: number;

/**
* How much padding should there be on the left of ListGrid.hiliteIcons by default?
*  Can be overridden at the field level
* @type {number}
* @default "2"
*/
hiliteIconLeftPadding?: number;

/**
* How much padding should there be on the right of ListGrid.hiliteIcons by default?
*  Can be overridden at the field level
* @type {number}
* @default "2"
*/
hiliteIconRightPadding?: number;

/**
* Default size of thumbnails shown for fieldTypes image and imageFile. Overrideable on a
*  per-field basis via ListGridField.imageSize or
*  ListGridField.imageWidth/ListGridField.imageHeight
* @type {number}
* @default "16"
*/
imageSize?: number;

/**
* Header spans are a second level of headers that appear above the normal ListGrid headers,
*  spanning one or more listGrid fields in a manner similar to a column-spanning cell in an
*  HTML table.
*  
*  A header span can be created by simply naming the fields the header should span. The
*  example below creates a headerSpan that spans the first two fields of the ListGrid.
*  
*  
*   isc.ListGrid.create({
*     headerHeight:40,
*     fields : [
*       { name:"field1" },
*       { name:"field2" },
*       { name:"field3" }
*     ],
*     headerSpans : [
*       {
*         fields: ["field1", "field2"],
*         title: "Field 1 and 2"
*       }
*     ]
*   });
*  
*  
*  
*  Header spans can be nested, allowing fields to be grouped by multiple levels of
*  granularity. See HeaderSpan.spans for further information on nesting spans.
*  
*  Header spans will automatically react to resizing of the headers they span, and will be
*  hidden automatically when all of the spanned fields are hidden.
*  
*  Header spans appear in the ListGrid.header area of the ListGrid, sharing space
*  with the existing headers, so it's typical to set ListGrid.headerHeight to
*  approximately double its normal height when using headerSpans, or if using nested header
*  spans, the default header height multiplied by the number of levels of header spans to be
*  shown.
*  
*  See HeaderSpan for many properties that allow the control of the appearance of
*  headerSpans.
*  
*  Note that headerSpans are created via the AutoChild pattern, hence
*  you can change the SmartClient component being used, or any of its properties.
*  
*  
*  Neither headerSpans themselves nor the fields within them may be drag reordered, but other
*  unspanned headers may be.
*  
*  A span can only span adjacent fields - if a span is defined and the spanned fields don't
*  sit next to each other in the specified fields array, the fields array will be automatically
*  reordered to match the order specified in the span's HeaderSpan.fields array.
*  
*  Note that headerSpans primarily provide a visual cue for grouping multiple headers
*  together. If you have an OLAP, data "cube" or multi-dimensional data model, the
*  CubeGrid component is the right choice.
* @type {Array<Partial<HeaderSpan>>}
* @default null
*/
headerSpans?: Array<Partial<HeaderSpan>>;

/**
* Default height for a ListGrid.headerSpans with no height specified.
*  
*  If headerSpanHeight is not specified (the default), headerSpans will be 1/2
*  of ListGrid.headerHeight.
* @type {Integer}
* @default null
*/
headerSpanHeight?: Integer;

/**
* Default alignment for ListGrid.headerSpans with no
*  HeaderSpan.valign specified.
* @type {string}
* @default "center"
*/
headerSpanVAlign?: string;

/**
* When ListGrid.headerSpans are in use, this property sets the default
*  vertical alignment for fields which do not have a headerSpan.
* @type {string}
* @default null
*/
unspannedHeaderVAlign?: string;

/**
* SCClassName to use for headerSpans. Typically a Button or
*  StretchImgButton subclass.
*  
*  If unset, headerSpans will be created using the ListGrid.headerButtonConstructor.
* @type {SCClassName}
* @default null
*/
headerSpanConstructor?: SCClassName;

/**
* ListGrid.headerSpans are created via the AutoChild pattern, hence
*  headerSpanConstructor, headerSpanDefaults and
*  headerSpanProperties are valid.
* @type {StatefulCanvas}
* @default null
*/
headerSpan?: StatefulCanvas;

/**
* When ListGrid.headerSpans are in use, whether to show a hierarchical column picker
*  that includes both headerSpans and normal headers, with normal headers indented under
*  headerSpans similarly to how a TreeGrid displays a Tree.
*  
*  If showTreeColumnPicker is false, no column picker will be shown on the
*  headerSpan itself, and the column picker for a clicked on a normal field header will include
*  only normal fields.
* @type {boolean}
* @default true
*/
showTreeColumnPicker?: boolean;

/**
* Button.baseStyle to apply to the field header buttons for
*  this ListGrid when showing header spans.
*  Note that, depending on the ListGrid.headerButtonConstructor of the header
*  buttons, you may also need to set ListGrid.headerTitleStyle.
* @type {CSSStyleName}
* @default null
*/
spannedHeaderBaseStyle?: CSSStyleName;

/**
* If a page is rendered in Page.isRTL, should
*  cell alignments specified ListGridField.cellAlign be reversed (so
*  an align:"right" field will have content aligned on the left and
*  vice versa)?
*  
*  This is true by default to match user expectation that text flows from
*  start-to end and is aligned with the start of text flow (left in LTR mode,
*  right in RTL mode) by default. May be set to false to have the specified
*  alignments be taken literally in RTL mode.
* @type {boolean}
* @default true
*/
reverseRTLAlign?: boolean;

/**
* If this property is true, any mouse click outside of the open cell editors
*    will end editing mode, hiding the cell editors and saving any changes to those
*    cell values.
* @type {boolean}
* @default null
*/
modalEditing?: boolean;

/**
* If ListGrid.showFilterEditor is true, when creating a SelectItem 
*  for editing criteria for a field with a ValueMap, should the SelectItem
*  default to SelectItem.multiple?
*  
*  This overrides SearchForm.useMultiSelectForValueMaps on the filterEditor's
*  edit form.
* @type {boolean}
* @default true
*/
useMultiSelectForFilterValueMaps?: boolean;

/**
* If we're showing the filterEditor (ListGrid.showFilterEditor is true), 
*  and ListGrid.filterByCell or ListGrid.filterOnKeypress are enabled, this
*  property is the delay in milliseconds between the user changing the filter and the
*  filter request being sent to the server. If multiple changes are made to the filter
*  within this fetch delay, only the most recent will actually cause a re-filter
* @type {number}
* @default "300"
*/
fetchDelay?: number;

/**
* If we're showing the filterEditor (ListGrid.showFilterEditor is true), this
*  property determines the delay in kicking off the filter request if the current filter
*  values are submitted by clicking the filter button or hitting return. By default, this
*  property is set to zero so that a filter request is immediately sent.
* @type {number}
* @default "0"
*/
explicitFetchDelay?: number;

/**
* Since ListGrid.body is configured with overflow: auto by default, no overflow
*  is expected for the ListGrid itself so by default it has overflow: hidden.
* @type {Overflow}
* @default "hidden"
*/
overflow?: Overflow;

/**
* Sets the Canvas.minHeight for the entire list (smaller than this
*  doesn't tend to work very well). If not set, this value will be defaulted when
*  Canvas.draw is called to something reasonable based on whether we're showing the
*  ListGrid.showFilterEditor, ListGrid.showHeader, 
*  ListGrid.showGridSummary, and/or the ListGrid.showEmptyMessage.
*  Any top or bottom CSS padding specified by ListGrid.emptyMessageStyle will be taken into
*  account, increasing minHeight so that the empty message can be shown without
*  overflow.
*  
*  Note: Minimum sizes do not apply to all situations.
*  See Canvas.minWidth for details.
* @type {number}
* @default "varies"
*/
minHeight?: number;

/**
* Prefix prepended to the name of a ListGridField.recordSummaryFunction
*  ListGridField.type field when accessing its value as record metadata.
*  The Framework may write out this value to make rendering the cell values or calculating
*  a ListGrid.showGridSummary or ListGrid.showGroupSummary
*  more efficient.
* @type {string}
* @default "_"
*/
recordSummaryAttributePrefix?: string;

/**
* Whether all records should be drawn all at once, or only records visible in the
*  viewport.
*  
*  Drawing all records causes longer initial rendering time, but allows smoother vertical
*  scrolling. With a very large number of records, showAllRecords will become too slow.
*  
*  This setting is incompatible with ListGrid.dataFetchMode: "paged" as it requires all
*  records matching the criteria to be fetched from the server at once.
* @type {boolean}
* @default false
*/
showAllRecords?: boolean;

/**
* Whether all columns should be drawn all at once, or only columns visible in the viewport.
*  
*  Drawing all columns causes longer initial rendering time, but allows smoother horizontal
*  scrolling. With a very large number of columns, showAllColumns will become too slow.
* @type {boolean}
* @default false
*/
showAllColumns?: boolean;

/**
* If drawing all rows would cause less than drawAllMaxCells cells to be rendered,
*  the full dataset will instead be drawn even if ListGrid.showAllRecords
*  is false and the viewport size and ListGrid.drawAheadRatio setting would normally have caused
*  incremental rendering to be used.
*  
*  The drawAllMaxCells setting prevents incremental rendering from being used in
*  situations where it's really unnecessary, such as a 40 row, 5 column dataset (only 200
*  cells) which happens to be in a grid with a viewport showing only 20 or so rows.
*  Incremental rendering causes a brief "flash" during scrolling as the visible portion of the
*  dataset is redrawn, and a better scrolling experience can be obtained in this situation by
*  drawing the entire dataset up front, which in this example would have negligible effect on
*  initial draw time.
*  
*  drawAllMaxCells:0 disables this features. You may want to disable this feature
*  if performance is an issue and:
*  
*  
* - you are very frequently redraw a grid
*  
* - you do a lot of computation when rendering each cell (eg formulas)
*  
* - you are showing many grids on one screen and the user won't scroll most of them
*  
* @type {Integer}
* @default "250"
*/
drawAllMaxCells?: Integer;

/**
* How far should we render records ahead of the currently visible area? This is expressed as
*  a ratio from viewport size to rendered area size.
*  
*  Tweaking drawAheadRatio allows you to make tradeoffs between continuous scrolling speed vs
*  initial render time and render time when scrolling by large amounts.
*  
*  NOTE: Only applies when showAllRecords is false.
* @type {Float}
* @default "2.0"
*/
drawAheadRatio?: Float;

/**
* Alternative to ListGrid.drawAheadRatio, to be used when the user
*  is rapidly changing the grids viewport (for example drag scrolling through the grid).
*  If unspecified ListGrid.drawAheadRatio will be used in all cases
* @type {Float}
* @default "2.0"
*/
quickDrawAheadRatio?: Float;

/**
* If true, if the user clicks on the scroll buttons at the end of the track or clicks once on
*  the scroll track, there will be an instant redraw of the grid content so that the user
*  doesn't see any blank space. For drag scrolling or other types of scrolling, the
*  ListGrid.scrollRedrawDelay applies.
* @type {boolean}
* @default true
*/
instantScrollTrackRedraw?: boolean;

/**
* While drag scrolling in an incrementally rendered grid, time in milliseconds to wait
*  before redrawing, after the last mouse movement by the user. This delay may be 
*  separately customized for mouse-wheel scrolling via ListGrid.scrollWheelRedrawDelay.
*  
*  See also
*  GridRenderer.instantScrollTrackRedraw for cases where this delay is skipped.
*  
*  NOTE: In Browser.isTouch,
*  GridRenderer.touchScrollRedrawDelay is used instead.
* @type {Integer}
* @default "0"
*/
scrollRedrawDelay?: Integer;

/**
* Like ListGrid.scrollRedrawDelay, but applies when the component is being
*  drag-scrolled (via a scrollbar). This value is typically set higher than ListGrid.scrollRedrawDelay
*  to avoid too many concurrent fetches to the server for ResultSet-backed components
*  since it's quite easy to induce such a case with a scrollbar and a grid bound to a large databaset.
* @type {Integer}
* @default "75"
*/
dragScrollRedrawDelay?: Integer;

/**
* While scrolling an incrementally rendered grid, using the mouseWheel, time in 
*  milliseconds to wait before redrawing, after the last mouseWheel movement by the user. 
*  If not specified ListGrid.scrollRedrawDelay will be used as a default for both
*  drag scrolling and mouseWheel scrolling.
*  
*  Note that if specified, this value will typically be larger than 
*  ListGrid.scrollRedrawDelay. From experimentation, the default setting of 
*  250 is typically enough time for a user to rapidly scroll through a
*  grid (rotating the scroll wheel with repeated flicks), without redrawing between
*  rotations, but this will differ depending on how long the redraw takes. A larger
*  delay may be warranted for grids with large numbers of columns, heavy custom 
*  formatting, etc.
*  
*  See also
*  GridRenderer.instantScrollTrackRedraw for cases where this delay is skipped.
* @type {Integer}
* @default "0"
*/
scrollWheelRedrawDelay?: Integer;

/**
* While scrolling an incrementally rendered grid, using the inertial scrolling, time in 
*  milliseconds to wait before redrawing, after the last touchScroll by the user. 
*  If not specified ListGrid.scrollRedrawDelay will be used as a default for both
*  drag scrolling and touch scrolling.
*  
*  Note that if specified, this value will typically be larger than 
*  ListGrid.scrollRedrawDelay.
*  
*  See also
*  GridRenderer.instantScrollTrackRedraw for cases where this delay is skipped.
* @type {Integer}
* @default "0"
*/
touchScrollRedrawDelay?: Integer;

/**
* When incremental rendering is switched on and there are variable record heights, the virtual
*  scrolling mechanism manages the differences in scroll height calculations due to the
*  unknown sizes of un-rendered rows to make the scrollbar and viewport appear correctly.
*  
*  When the virtualScrolling system is active, the last scroll position
*  scrolls the last record to the top of the viewport, leaving blank space underneath.
*  This is a necessary and unavoidable consequence of mapping the position of the scrollbar
*  thumb to an unknown amount of remaining space without being able to know the total
*  scrollable area in advance (since record heights vary).
*  
*  virtualScrolling is switched on automatically when ListGrid.fixedRecordHeights is false and
*  also when
*  using the ListGrid.showRecordComponents, as
*  recordComponents expand the rows that contain them. This flag should be manually enabled
*  when calling ListGrid.addEmbeddedComponent if embedded components can
*  cause record sizes to expand beyond specified cellHeight.
*  
*  virtualScrolling is also automatically enabled when ListGrid.canExpandRecords is true
*  to handle the fact that expanded rows may render at variable heights.
* @type {boolean}
* @default null
*/
virtualScrolling?: boolean;

/**
* When using ListGrid.dataFetchMode, how many records to fetch at a time. If set to
*  a positive integer, dataPageSize will override the default 
*  ResultSet.resultSize for ResultSets automatically created when you call
*  ListGrid.fetchData (and similarly for the ResultTree.resultSize of
*  ResultTrees). Leaving dataPageSize at its default means to just use the default page
*  size of the data container.
*  
*  Note that regardless of the dataPageSize setting, a component will always fetch
*  all of data that it needs to draw. Settings such as
*  ListGrid.showAllRecords,
*  ListGrid.drawAllMaxCells and
*  ListGrid.drawAheadRatio can cause more rows than the configured
*  dataPageSize to be fetched.
* @type {Integer}
* @default null
*/
dataPageSize?: Integer;

/**
* How to fetch and manage records retrieve from the server. See FetchMode.
*  
*  This setting only applies to the ResultSet automatically created by calling
*  ListGrid.fetchData. If a pre-existing ResultSet is passed to setData() instead, it's
*  existing setting for ResultSet.fetchMode applies.
* @type {FetchMode}
* @default "paged"
*/
dataFetchMode?: FetchMode;

/**
* Delay in milliseconds before fetching data.
*  
*  Note: the floor value for this attribute is 1. If you set this value to zero, it will
*  be defaulted to 1 for you instead.
* @type {Integer}
* @default "1"
*/
dataFetchDelay?: Integer;

/**
* GridRenderer used to render the dataset.
*  
*  Note that this is a multi-instance component when there are frozen fields because in
*  addition to the primary body AutoChild, a "frozen body" AutoChild is created to render
*  the frozen portion of the dataset.
* @type {GridRenderer}
* @default null
*/
body?: GridRenderer;

/**
* Overflow setting for the "body", that is, the area of the grid where data values are
*  rendered.
*  
*  This is a very advanced setting which is typically only changed by subclasses
*  of ListGrid which never show a header. To achieve auto-fitting, instead use properties
*  such as ListGrid.autoFitData, ListGrid.autoFitFieldWidths and
*  ListGrid.fixedRecordHeights.
* @type {Overflow}
* @default "auto"
*/
bodyOverflow?: Overflow;

/**
* Background color applied to the ListGrid body (that is, the area of the grid where
*  data values are rendered).
*  Note that this will typically not be visible to the user unless there are few enough
*  rows that there is visible space in the body below the last row. To style data cells,
*  override ListGrid.baseStyle instead.
* @type {string}
* @default "white"
*/
bodyBackgroundColor?: string;

/**
* CSS style used for the body of this grid. If applying a background-color to the body
*  via a CSS style applied using this property, be sure to set
*  ListGrid.bodyBackgroundColor to null.
* @type {CSSStyleName}
* @default null
*/
bodyStyleName?: CSSStyleName;

/**
* The value to display for cells whose value is null or the empty string after applying
*  ListGrid.formatCellValue and valueMap (if any).
*  
*  This is the grid-wide attribute. You may also set the emptyCellValue on a per-field basis.
* @type {HTMLString}
* @default "&amp;nbsp;"
*/
emptyCellValue?: HTMLString;

/**
* Minimum height for ListGrid cells, settable by the skin, based on the size of the 
*  checkbox media used for boolean fields plus minimal surrounding padding. 
*  minimumCellHeight is used by Canvas.resizeControls to 
*  avoid shrinking ListGrid rows so much that correct display is impossible. 
*  Do not set minimumCellHeight on a per-instance basis - it's only for use in custom skins.
* @type {number}
* @default "20"
*/
minimumCellHeight?: number;

/**
* Default height for each row in pixels. See ListGrid.fixedRecordHeights and
*  ListGrid.enforceVClipping for information on how rows are sized when 
*  cell content height exceeds this specified value.
* @type {number}
* @default "20"
*/
cellHeight?: number;

/**
* If ListGrid.baseStyle is unset, base style will be derived from
*  ListGrid.normalBaseStyle if this grid has fixed row heights and
*  the specified ListGrid.cellHeight matches this value. Otherwise
*  ListGrid.tallBaseStyle will be used.
* @type {number}
* @default "20"
*/
normalCellHeight?: number;

/**
* Should we vertically clip cell contents, or allow rows to expand vertically to show all
*  contents?
*  
*  If we allow rows to expand, the row height as derived from
*  GridRenderer.getRowHeight or the
*  default ListGrid.cellHeight is treated as a minimum.
*  
*  Setting fixedRecordHeights to false enables the ListGrid.virtualScrolling
*  system.
*  
*  NOTE:
*  
* - Setting fixedRecordHeights to false for CubeGrid is not supported, though a
*  similar option for the row headers is available as CubeGrid.autoSizeHeaders.
*  
* - By default, for performance reasons, clipping is not enforced for
*  some kinds of content (such as images) on all browsers. Set
*  ListGrid.enforceVClipping to enforce clipping for
*  all types of content on all browsers.
*  
* @type {boolean}
* @default true
*/
fixedRecordHeights?: boolean;

/**
* For performance reasons, even when ListGrid.fixedRecordHeights is set, vertical clipping
*  is not enforced by default for some kinds of content (such as images) on all browsers.
*  Set ListGrid.enforceVClipping to enforce clipping for all types of
*  content on all browsers.
*  
*  This additional setting is likely to be phased out as browsers improve.
* @type {boolean}
* @default false
*/
enforceVClipping?: boolean;

/**
* Should we horizontally clip cell contents, or allow columns to expand horizontally to
*  show all contents?
*  
*  If we allow columns to expand, the column width is treated as a minimum.
*  
*  NOTE: the header does not automatically respond to expanded field widths.
*  If your grid is showing a header we'd recommend developers consider
*  setting ListGrid.autoFitFieldWidths rather than using this attribute.
* @type {boolean}
* @default true
*/
fixedFieldWidths?: boolean;

/**
* Should this ListGrid automatically expand to accommodate the size of records and fields?
*  
*  Valid settings are
*  
* - "vertical": expand vertically to accommodate records.
*    
* - "horizontal": expand horizontally to accommodate fields.
*    
* - "both": expand horizontally and vertically to accommodate content.
*  
*  How far the ListGrid will expand may be limited via the following properties:
*  ListGrid.autoFitMaxHeight, ListGrid.autoFitMaxRecords,
*  ListGrid.autoFitMaxWidth, ListGrid.autoFitMaxColumns.
*  
*  Note that this property causes the grid as a whole to expand to fit records or fields.
*  To have the fields or records themselves expand to fit cell contents, see
*  ListGrid.autoFitFieldWidths and ListGrid.fixedRecordHeights.
* @type {Autofit}
* @default null
*/
autoFitData?: Autofit;

/**
* If ListGrid.autoFitData is set to "vertical" or "both" this
*  property provides an upper limit on how far the ListGrid will expand vertically to accommodate
*  its content. If content exceeds this height, scrollbars will be introduced as usual.
*  In addition to this property, ListGrid.autoFitMaxRecords allows you to limit vertical
*  expansion based on the number of rows to be rendered.
* @type {Integer}
* @default null
*/
autoFitMaxHeight?: Integer;

/**
* If ListGrid.autoFitData is set to "vertical" or "both" this
*  property provides the maximum number of records for which the ListGrid will expand. If more
*  records are present, scrolling will be introduced to reach them as normal.
*  If unset, by default the ListGrid will expand to accommodate as many records as are present.
* @type {Integer}
* @default "50"
*/
autoFitMaxRecords?: Integer;

/**
* If ListGrid.autoFitData is set to "vertical" or "both",
*  setting this property will cause the ListGrid body to size large enough to accommodate
*  the actual data and also leave this many extra rows' worth of blank space below the last
*  record. If a maximum size is specified via ListGrid.autoFitMaxHeight or
*  ListGrid.autoFitMaxRecords, it will still be respected. Once the data set
*  is large enough to fill or exceed that space, this property no longer has an effect.
* @type {Integer}
* @default null
*/
autoFitExtraRecords?: Integer;

/**
* If ListGrid.autoFitData is set to "horizontal" or "both"
*  this property provides an upper limit on how far the ListGrid will expand horizontally to
*  accommodate its content. Value may be specified as a numeric pixel value or
*  a percentage value.
*  
*  If content exceeds this width, scrollbars will be introduced as usual.
*  In addition to this property, ListGrid.autoFitMaxColumns allows you to limit
*  horizontal expansion based on the number of columns to be rendered.
* @type {Integer | string}
* @default null
*/
autoFitMaxWidth?: Integer | string;

/**
* If ListGrid.autoFitData is set to "horizontal" or "both"
*  this property provides the maximum number of columns for which the ListGrid will expand.
*  If more columns are present, scrolling will be introduced to reach them as normal.
*  If unset the ListGrid will expand to accommodate as many columns as are defined for the
*  grid.
* @type {Integer}
* @default "50"
*/
autoFitMaxColumns?: Integer;

/**
* Can the user perform one-time autofit for specific columns in this grid?
*  
*  If set to true, the default header menu will include options to auto fit
*  ListGrid.autoFitAllText such that they fit their
*  content or titles as specified via ListGridField.autoFitWidthApproach.
*  Autofitting of individual fields via a
*  ListGrid.autoFitFieldText, or the
*  ListGrid.headerAutoFitEvent will also be enabled when this
*  property is set unless ListGridField.canAutoFitWidth is explicitly set to false
*  
*  Note that the ability to perform one-time autofitting of fields via this 
*  subsystem is separate from the programmatic autofit behavior enabled 
*  via ListGrid.autoFitFieldWidths.
*  
*  This subsystem is requires canResizeFields be enabled and will be disabled if
*  that property is set to false
* @type {boolean}
* @default true
*/
canAutoFitFields?: boolean;

/**
* Event on a ListGrid header that triggers auto fitting to data and/or title.
*  
*  Note that if sorting is enabled for the field and the headerAutoFitEvent is "click", both
*  sorting and autofit occur on a click.
*  
*  Only has an impact when ListGrid.canAutoFitFields or 
*  ListGridField.canAutoFitWidth is set to true.
* @type {AutoFitEvent}
* @default "doubleClick"
*/
headerAutoFitEvent?: AutoFitEvent;

/**
* Should ListGrid fields autofit their widths to titles or content?
*  This property may be overridden on a per-field basis via ListGridField.autoFitWidth.
*  Developers may wish to consider disabling autoFit for fields known to have 
*  exceptionally long content as this can lead to large horizontal scrollbars and unwieldy UI.
*  
*  The ListGrid.autoFitWidthApproach controls whether fitting is to values, titles
*  or both. This property may also be overridden on a per field basis.
*  
*  If ListGridField.width is also set on the field, it will be taken as a minimum width.
*  ListGrid.minFieldWidth will also be respected.
*  
*  By default, the entire available width of the grid will still be used, by allocating any "extra"
*  space to specific columns - see ListGrid.autoFitFieldsFillViewport for details on
*  controlling this behavior.
*  
*  When this feature is enabled, autofitting is active on an ongoing basis.
*  Autofitting will be performed:
*  
*  
* - whenever the dataset is completely changed or rows are added or removed
*  
* - whenever a field which is autofitting is changed
*  
* - on a manual call to ListGrid.autoFitField or
*     ListGrid.autoFitFields
*  
*  Auto-fitting behavior continues until the user resizes the field manually, at which
*  point it stops. The user can also perform a one-time auto-fit of fields via
*  the header context menu if ListGrid.canAutoFitFields is enabled.
*  
*  When autofitting to column values, ListGrid.getDefaultFieldWidth will be
*  called to determine the space required for a field's values. This method
*  uses values from the rendered set of rows to calculate the required column width, which means the
*  field width may still be smaller than values from non-rendered rows. See
*  ListGrid.showAllRecords and ListGrid.drawAheadRatio) to control incremental
*  rendering of rows.
*  
*  Note that for icon type fields, the ListGrid.autoFitIconFields
*  property setting may turn on auto-fit-width behavior for specific fields by default,
*  even if autoFitFieldWidths is false for the grid as a whole.
*  
*  Using this feature has a performance penalty roughly comparable to always rendering
*  one additional field per field where autofitting is enabled. Specifically, enabling it
*  for all fields would be comparable to both doubling the number of fields
*  and disabling ListGrid.showAllColumns.
*  In a grid where only half the fields are normally visible and hence only half are
*  normally rendered, this would be roughly 4 times slower overall.
*  
*  This performance penalty is a result of ListGrid.getDefaultFieldWidth having to
*  render out the data set offscreen and measure the rendered content - it does not apply
*  for cases where this method can return a simple fixed values (as with icon fields).
*  
*  Which fields are currently autofitting is saved as part of the
*  ListGrid.getViewState of the ListGrid.
*  
*  Interaction with wrapping: If ListGrid.wrapCells is 
*  enabled, autoFit behavior based on ListGrid.autoFitWidthApproach
*  will render fields wide enough to contain the unwrapped cell values.
*  If ListGridField.wrap is enabled, when fitting to 
*  a title, a field will render wide enough to accommodate the wrapped title without
*  clipping (so wide enough for the natural wrap-point / longest word or unwrappable string).
* @type {boolean}
* @default null
*/
autoFitFieldWidths?: boolean;

/**
* If this listGrid has specified ListGrid.headerSpans, setting this
*  attribute to true will cause spans to expand to accommodate long titles if necessary.
* @type {boolean}
* @default false
*/
autoSizeHeaderSpans?: boolean;

/**
* If ListGrid.autoFitFieldWidths is enabled and the calculated field sizes
*  are wide enough that horizontal scrolling would be introduced, this attribute may be
*  set to an array of fieldNames, causing those fields to be clipped rather than
*  forcing horizontal scrollbars to appear.
*  
*  Note: If any ListGridField.frozen are included in this list they
*  will not be clipped.
* @type {Array<Partial<string>>}
* @default null
*/
autoFitClipFields?: Array<Partial<string>>;

/**
* If ListGrid.autoFitFieldWidths is enabled, and extra space is available after
*  autofitting all fields, should the grid automatically expand one field to fill the extra
*  space.
*  
*  When enabled, the field to expand may be specified via ListGrid.autoFitExpandField.
*  
*  Note this logic will not expand a ListGridField.frozen.
* @type {boolean}
* @default true
*/
autoFitFieldsFillViewport?: boolean;

/**
* The field to expand if ListGrid.autoFitFieldWidths and
*  ListGrid.autoFitFieldsFillViewport are enabled and
*  auto-fitting will not fill all available horizontal space.
*  
*  If unset, will default to the text field with the longest
*  DataSourceField.length if length is set, otherwise, the first text
*  field with no width specified.
*  
*  Note that expanding ListGridField.frozen is not supported.
* @type {string}
* @default null
*/
autoFitExpandField?: string;

/**
* When a user requests column autofitting via the
*  ListGrid.getHeaderContextMenuItems or via a
*  ListGrid.headerAutoFitEvent, what autofit approach is used.
* @type {AutoFitWidthApproach}
* @default "value"
*/
autoFitWidthApproach?: AutoFitWidthApproach;

/**
* SmartClient listGrids have special logic to automatically size fields that
*  are displayed as an icon - that is fields with
*  ListGridFieldType, fields displaying only
*  ListGridField.showValueIconOnly, and boolean fields (which
*  are rendered as a checkmark type icon by default.
*  
*  This attribute controls this behavior - governing whether icon fields should
*  be sized to fit their content (icon), title, or whether to disable this
*  behavior. Setting this value to "title" or "iconWidth"
*  will cause ListGridField.autoFitWidth to be enabled by default for all
*  icon fields with the ListGridField.autoFitWidthApproach set to
*  "value" or "both" as appropriate. Note that the
*  width required for the icons is calculated by ListGrid.getDefaultFieldWidth
*  which performs a simple calculation based on the specified icon width for these
*  types of fields.
*  
*  This setting governs default behavior for icon fields - for specific fields within
*  a grid, this default behavior can be overridden by setting an explicit
*  ListGridField.width or
*  explicitly enabling ListGridField.autoFitWidth and setting
*  ListGridField.autoFitWidthApproach on the field in question.
* @type {AutoFitIconFieldType}
* @default "title"
*/
autoFitIconFields?: AutoFitIconFieldType;

/**
* Should listGrids automatically size date fields to fit their values or titles?
*  If set to "value", fields of type date will be rendered at the
*  size specified by ListGrid.defaultDateFieldWidth, 
*  (or ListGrid.defaultEditableDateFieldWidth for editable fields). This static
*  value is appropriate for dates rendered with the standard short-date formatter.
*  If set to "title" or "both", the drawn width of the title
*  will be taken into account when sizing the column.
*  
*  This is achieved by enabling ListGridField.autoFitWidth
*  on date fields when this property is set to anything other than "none",
*  setting the ListGridField.autoFitWidthApproach to the value specified here
*  and having logic in ListGrid.getDefaultFieldWidth pick up the
*  ListGrid.defaultDateFieldWidth or ListGrid.defaultEditableDateFieldWidth
*  if appropriate.
* @type {AutoFitWidthApproach}
* @default "value"
*/
autoFitDateFields?: AutoFitWidthApproach;

/**
* Default width for date type fields. See ListGrid.autoFitDateFields for
*  details on how this property is used.
* @type {Integer}
* @default "varies"
*/
defaultDateFieldWidth?: Integer;

/**
* Default width for editable date type fields. See ListGrid.autoFitDateFields for
*  details on how this property is used.
* @type {Integer}
* @default "varies"
*/
defaultEditableDateFieldWidth?: Integer;

/**
* Default width for datetime type fields. See ListGrid.autoFitDateFields for
*  details on how this property is used.
* @type {Integer}
* @default "varies"
*/
defaultDateTimeFieldWidth?: Integer;

/**
* Default width for editable datetime type fields. See ListGrid.autoFitDateFields
*  for details on how this property is used.
* @type {Integer}
* @default "varies"
*/
defaultEditableDateTimeFieldWidth?: Integer;

/**
* Should listGrids automatically size time fields to fit their values or titles?
*  If set to "value", fields of type time will be rendered at the
*  size specified by ListGrid.defaultTimeFieldWidth. This static
*  value is appropriate for dates rendered with the standard time formatter.
*  If set to "title" or "both", the drawn width of the title
*  will be taken into account when sizing the column.
*  
*  This is achieved by enabling ListGridField.autoFitWidth
*  on date fields when this property is set to anything other than "none",
*  setting the ListGridField.autoFitWidthApproach to the value specified here
*  and having logic in ListGrid.getDefaultFieldWidth pick up the
*  ListGrid.defaultTimeFieldWidth if appropriate.
* @type {AutoFitWidthApproach}
* @default "value"
*/
autoFitTimeFields?: AutoFitWidthApproach;

/**
* Default width for time type fields. See ListGrid.autoFitDateFields for
*  details on how this property is used.
* @type {Integer}
* @default "varies"
*/
defaultTimeFieldWidth?: Integer;

/**
* Whether to skip line breaks for all fields by default when
*  ListGridField.escapeHTML. This property can be overridden at the
*  field level by ListGridField.skipLineBreaks.
* @type {boolean}
* @default null
*/
skipLineBreaks?: boolean;

/**
* Whether to leave a gap for the vertical scrollbar, even when it's not present.
*  
*  Note that if leaveScrollbarGap is false and vertical scrolling is introduced, fields
*  will be resized to fit the smaller body area if possible, in order to avoid horizontal
*  scrolling also being required.
* @type {boolean}
* @default true
*/
leaveScrollbarGap?: boolean;

/**
* Should content within cells be allowed to wrap?
*  
*  Even if content is allowed to wrap, if ListGrid.fixedRecordHeights is set, the content
*  will be clipped off at the cell boundary. Either set a larger, fixed ListGrid.cellHeight
*  to reveal more content, or set ListGrid.fixedRecordHeights to false to allow auto-sizing.
* @type {boolean}
* @default false
*/
wrapCells?: boolean;

/**
* Should cells be written out with css that will preserve whitespace?
*  
*  If true, depending on the value of ListGrid.wrapCells, the css generated
*  for cells will use the
*  white-space
*  property values of pre or pre-wrap.
*  This avoids collapsing sequences of whitespace without requiring
*  special &amp;nbsp; characters.
* @type {boolean}
* @default false
*/
preserveWhitespace?: boolean;

/**
* If true and a cell's value is clipped, then a hover containing the full cell value is
*  enabled.
*  
*  Note that standard cell hovers override clipped value hovers. Thus, to enable clipped value
*  hovers, ListGrid.canHover must be unset or null and the corresponding field must have
*  ListGridField.showHover unset or null as well.
* @type {boolean}
* @default null
*/
showClippedValuesOnHover?: boolean;

/**
* The amount of empty space, in pixels, surrounding each value in its cell.
* @type {number}
* @default "2"
*/
cellPadding?: number;

/**
* How should Date type values be displayed in this ListGrid by default?
*  
*  This property specifies the default DateDisplayFormat to apply to Date values
*  displayed in this grid for all fields except those of ListGridField.type
*  (See also ListGrid.timeFormatter).
*  If ListGrid.datetimeFormatter is specified, that will be applied by default
*  to fields of type "datetime".
*  
*  Note that if ListGridField.dateFormatter or ListGridField.timeFormatter are
*  specified those properties will take precedence over the component level settings.
*  
*  If unset, date values will be formatted according to the system wide
*  DateUtil.setShortDisplayFormat or
*  DateUtil.setShortDatetimeDisplayFormat for
*  datetime type fields.
*  
*  If this field is editable the dateFormatter will also be passed to the editor created
*  to edit this field as DateItem.dateFormatter.
*  In this case you may also need to set ListGrid.dateInputFormat.
* @type {DateDisplayFormat}
* @default null
*/
dateFormatter?: DateDisplayFormat;

/**
* Display format to use for fields specified as type 'datetime'. Default is to use the
*  system-wide default date time format, configured via
*  DateUtil.setShortDatetimeDisplayFormat. Specify any
*  valid DateDisplayFormat to change the display format for datetimes used by this grid.
*  
*  May be specified as a function. If specified as a function, this function will be executed in the scope of the Date
*  and should return the formatted string.
*  
*  
*  May also be specified at the field level via
*  ListGridField.dateFormatter
*  
*  If this field is editable the dateFormatter will also be passed to the editor created
*  to edit this field as DateItem.dateFormatter.
*  In this case you may also need to set ListGrid.dateInputFormat.
* @type {DateDisplayFormat}
* @default null
*/
datetimeFormatter?: DateDisplayFormat;

/**
* If this is an editable listGrid, this property will specify the
*  DateItem.inputFormat applied to editors for fields of type
*  "date". May be overridden per field via ListGridField.inputFormat.
* @type {DateInputFormat}
* @default null
*/
dateInputFormat?: DateInputFormat;

/**
* Display format to use for fields specified as type 'time'. May also be specified at
*  the field level via ListGridField.timeFormatter.
*  If unset, time fields will be formatted based on the system wide
*  Time.shortDisplayFormat.
*  If this field is editable, the timeFormatter will also be passed to the editor
*  created to edit any time type fields as TimeItem.timeFormatter
* @type {TimeDisplayFormat}
* @default "toShortPaddedTime"
*/
timeFormatter?: TimeDisplayFormat;

/**
* Property name on a record that will hold the link text for that record.
*  
*  This property is configurable to avoid possible collision with data values in the
*  record.
*  
*  Use ListGridField.linkTextProperty if you have more than one link field and
* @type {string}
* @default "linkText"
*/
linkTextProperty?: string;

/**
* Note: This property only has an effect in Internet Explorer
*  
*  Advanced property to improve performance for dynamic styling of gridRenderer cells in
*  Internet Explorer, at the expense of slightly slower initial drawing, and some 
*  limitations on supported styling options.
*  
*  fastCellUpdates speeds up the dynamic styling system used by rollovers,
*  selections, and custom styling that calls GridRenderer.refreshCellStyle, at
*  the cost of slightly slower draw() and redraw() times.
*  
*  Notes:
*  
*  
* - When this property is set, ListGrid cells may be styled using the 
*    ListGrid.tallBaseStyle. See ListGrid.getBaseStyle for 
*    more information.
*  
* - If any cell styles specify a a background image URL, the URL will be resolved relative
*    to the page location rather than the location of the CSS stylesheet. This means cell
*    styles with a background URL should either supply a fully qualified path, or the
*    background image media should be made available at a second location for IE.
*  
* - fastCellUpdates will not work if the styles involved are in an external stylesheet loaded
*    from a remote host. Either the stylesheet containing cell styles needs to be loaded
*    from the same host as the main page, or the cell styles need to be inlined in the html 
*    of the bootstrap page.
*  
* - fastCellUpdates will not work if the css styles for cells are defined in
*    a .css file loaded via @import. Instead the .css
*    file should be loaded via a &lt;link ...&gt; tag.
*  
* @type {boolean}
* @default true
*/
fastCellUpdates?: boolean;

/**
* GridRenderer.baseStyle for this listGrid.
*  If this property is unset, base style may be derived from ListGrid.normalBaseStyle
*  or ListGrid.tallBaseStyle as described in
*  ListGrid.getBaseStyle.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined
*  with the base style to generate stateful cell styles.
* @type {CSSStyleName}
* @default null
*/
baseStyle?: CSSStyleName;

/**
* "Normal" baseStyle for this listGrid. Only applies if ListGrid.baseStyle is
*  set to null.
*  
*  If baseStyle is unset, this
*  property will be used as a base cell style if the grid is showing fixed height rows, and
*  the specified cellHeight matches ListGrid.normalCellHeight (and in Internet Explorer,
*  ListGrid.fastCellUpdates is false). Otherwise ListGrid.tallBaseStyle will
*  be used.
*  
*  Having separate styles defined for fixed vs. variable height rows allows the developer
*  to specify css which is designed to render at a specific height (typically using
*  background images, which won't scale), without breaking support for styling rows
*  of variable height.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined
*  with the base style to generate stateful cell styles.
* @type {CSSStyleName}
* @default "cell"
*/
normalBaseStyle?: CSSStyleName;

/**
* "Tall" baseStyle for this listGrid. Only applies if ListGrid.baseStyle is
*  set to null.
*  
*  If baseStyle is unset, this
*  property will be used as a base cell style unless the grid is showing fixed height
*  rows with a specified cellHeight that matches ListGrid.normalCellHeight, in
*  which case ListGrid.normalBaseStyle will be used. Note that in Internet Explorer
*  if ListGrid.fastCellUpdates is true, tallBaseStyle will also be
*  used even if the cellHeight matches the specified normalCellHeight for the
*  grid.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined
*  with the base style to generate stateful cell styles.
* @type {CSSStyleName}
* @default "cell"
*/
tallBaseStyle?: CSSStyleName;

/**
* A base name for the CSS class applied to cells when editing has failed.
*  If this listGrid is editable, this style will be applied to any edited cells for which
*  validation failed.
*  As with the default 'baseStyle' property, this style will have "Dark", "Over", "Selected",
*  or "Disabled" appended to it according to the state of the cell.
*  If null, cells for which editing has failed will be rendered using the normal base style
*  classNames, but with custom CSSText applied as derived from this.editFailedCSSText
* @type {CSSStyleName}
* @default null
*/
editFailedBaseStyle?: CSSStyleName;

/**
* Custom CSS text to be applied to cells when editing has failed.
*  If this listGrid is editable, this css text will be applied to any edited cells for which
*  validation failed, on top of the base style for the cell.
*  For further customization of styling for cells that failed editing validation, use
*  this.editFailedBaseStyle instead.
* @type {string}
* @default "color:red;border:1px solid red;"
*/
editFailedCSSText?: string;

/**
* A base name for the CSS class applied to cells containing pending (unsaved) edits
*  As with the default 'baseStyle' property, this style will have "Dark", "Over", "Selected",
*  or "Disabled" appended to it according to the state of the cell.
*  
*  If this property is null (the default setting), cells with pending edits will pick up
*  custom css text to be applied on top of the normal base style from
*  this.editPendingCSSText.
* @type {CSSStyleName}
* @default null
*/
editPendingBaseStyle?: CSSStyleName;

/**
* Custom CSS text to be applied to cells with pending edits that have not yet been
*  submitted.
*  For further customization of styling for cells with pending edits use
*  this.editPendingBaseStyle instead.
* @type {string}
* @default "color:#0066CC;"
*/
editPendingCSSText?: string;

/**
* Denotes the name of a property that can be set on records to display a custom style.
*  For example if this property is set to "customStyle", setting 
*  record.customStyle to a css styleName will cause the record in question to
*  render out with that styling applied to it. Note that this will be a static 
*  style - it will not be modified as the state of the record (selected / over etc) changes.
* @type {string}
* @default "customStyle"
*/
recordCustomStyleProperty?: string;

/**
* This attribute allows custom base styles to be displayed on a per-record basis.
*  To specify a custom base-style for some record set
*  record[listGrid.recordBaseStyleProperty] to the desired base style name -
*  for example if recordBaseStyleProperty is "_baseStyle", set
*  record._baseStyle to the custom base style name.
* @type {string}
* @default "_baseStyle"
*/
recordBaseStyleProperty?: string;

/**
* If this listGrid contains any frozen fields, this property can be used to apply a custom
*  baseStyle to all cells in those frozen fields. If unset, the standard base style will be
*  used for both frozen and unfrozen cells.
* @type {string}
* @default null
*/
frozenBaseStyle?: string;

/**
* If this list grid is showing any ListGridField.frozen fields, and a horizontal
*  scrollbar is visible at the bottom of the liquid columns, should an equivalent scrollbar gap
*  be left visible below the frozen columns?
*  Note that if set to true any backgroundColor or border applied to the ListGrid
*  will show up below the bottom row of the frozen column(s).
* @type {boolean}
* @default false
*/
shrinkForFreeze?: boolean;

/**
* Whether alternating rows (or blocks of rows, depending
*  on GridRenderer.alternateRowFrequency) should be drawn in alternating styles,
*  in order to create a "ledger" effect for easier reading.
*  
*  If enabled, the cell style for alternate rows will have the
*  GridRenderer.alternateRowSuffix appended to it.
*  See also GridRenderer.alternateColumnStyles.
* @type {boolean}
* @default false
*/
alternateRecordStyles?: boolean;

/**
* Suffix to append to GridRenderer.alternateRowStyles.
*  Note that if GridRenderer.alternateColumnStyles is enabled, cells which fall
*  into both an alternate row and column will have both suffixes appended - for
*  example "cellDarkAltCol".
* @type {string}
* @default "Dark"
*/
alternateRecordSuffix?: string;

/**
* The number of consecutive rows to draw in the same style before alternating, when
*  GridRenderer.alternateRowStyles is true.
* @type {number}
* @default "1"
*/
alternateRecordFrequency?: number;

/**
* Whether alternating columns (or blocks of columns, depending
*  on GridRenderer.alternateColumnFrequency) should be drawn in alternating styles,
*  in order to create a vertical "ledger" effect for easier reading.
*  
*  If enabled, the cell style for alternate rows will have the
*  GridRenderer.alternateColumnSuffix appended to it.
*  See also GridRenderer.alternateRowStyles.
* @type {boolean}
* @default false
*/
alternateFieldStyles?: boolean;

/**
* Suffix to append to GridRenderer.alternateColumnStyles.
*  Note that if GridRenderer.alternateRowStyles is enabled, cells which fall
*  into both an alternate row and column will have both suffixes appended - for
*  example "cellDarkAltCol".
* @type {string}
* @default "AltCol"
*/
alternateFieldSuffix?: string;

/**
* The number of consecutive columns to draw in the same style before alternating, when
*  GridRenderer.alternateColumnStyles is true.
* @type {number}
* @default "1"
*/
alternateFieldFrequency?: number;

/**
* Optional css style to apply to the body if ListGrid.alternateRecordStyles is true
*  for this grid. If unset ListGrid.bodyStyleName will be used to style the body
*  regardless of the ListGrid.alternateRecordStyles setting.
* @type {CSSStyleName}
* @default null
*/
alternateBodyStyleName?: CSSStyleName;

/**
* When assembling a value for a ListGrid.canAddSummaryFields, if a referenced
*  field is hilited, should the hilite HTML be included in the summary field value?
*  
*  To control hilites showing in group summaries, see ListGrid.showHilitesInGroupSummary.
* @type {boolean}
* @default true
*/
includeHilitesInSummaryFields?: boolean;

/**
* Determines whether hiliting for any field in this grid is shown in a group summary.
*  This setting affects all fields of the grid.
*  
*  To suppress hilites for a specific field see ListGridField.showHilitesInGroupSummary.
*  
*  Hiliting in summary fields (columns) can be enabled by setting
*  ListGrid.includeHilitesInSummaryFields to true.
* @type {boolean}
* @default true
*/
showHilitesInGroupSummary?: boolean;

/**
* If set, end users can create advanced hiliting rules that will use the
*  Hilite.replacementValue feature to cause values in hilited cells
*  to be replaced with a user-entered value. For example, a user could create a hilite rule
*  that replaces numeric values ranging from 0.5 to 1.0 with the text "LOW".
*  
*  Specifically, when the "Add Advanced Rule" button is pressed and
*  hiliteCanReplaceValue is true, the user will see a text entry field titled
*  "Replace value with" (ListGrid.hiliteReplaceValueFieldTitle) and if they enter a value, that
*  value will appear in the grid cell in lieu of the cell's original value.
* @type {boolean}
* @default null
*/
hiliteCanReplaceValue?: boolean;

/**
* Title used for the text box shown when ListGrid.hiliteCanReplaceValue is set.
* @type {string}
* @default "Replace value with"
*/
hiliteReplaceValueFieldTitle?: string;

/**
* If set to true, custom HTML applied as part of hiliting will be applied after
*  ListGrid.formatCellValue for each cell. If false, hilite
*  HTML will be applied before formatting.
*  
*  This applies to the following hilite properties:
*  
*  
* - Hilite.replacementValue
*  
* - Hilite.htmlBefore
*  
* - Hilite.htmlAfter
*  
* - Hilite.htmlValue
*  
*  
*  May be overridden per field via ListGridField.hiliteHTMLAfterFormat
* @type {boolean}
* @default true
*/
hiliteHTMLAfterFormat?: boolean;

/**
* Should the "Selected" style be applied to selected records?
* @type {boolean}
* @default true
*/
showSelectedStyle?: boolean;

/**
* If true, when the user navigates to a cell using arrow keys and hits space,
*  the cell will respond to a click event.
* @type {boolean}
* @default true
*/
generateClickOnSpace?: boolean;

/**
* If true, when the user navigates to a cell using arrow keys and hits Enter,
*  the cell will respond to a click event.
* @type {boolean}
* @default false
*/
generateClickOnEnter?: boolean;

/**
* If true, when the user navigates to a cell using arrow keys and hits Space,
*  the cell will respond to a double click event.
* @type {boolean}
* @default false
*/
generateDoubleClickOnSpace?: boolean;

/**
* If true, when the user navigates to a cell using arrow keys and hits Enter,
*  the cell will respond to a double click event.
* @type {boolean}
* @default true
*/
generateDoubleClickOnEnter?: boolean;

/**
* Action to perform when the listGrid has keyboard focus (but not editing focus) and a user
*  presses the arrow keys to navigate around the grid.
*  
*  If ListGrid.canSelectCells is true, navigation occurs by cell - the user can move
*  to a new cell in any direction.
*  If ListGrid.canSelectCells is false, navigation typically occurs by row - the user can
*  move up or down throw the rows in the grid.
*  
*  For actions that fire events (click or doubleClick), both cell and record level events are
*  fired (for example for arrowKeyAction "activate", ListGrid.cellDoubleClick
*  and ListGrid.recordDoubleClick are fired for the new position.
*  Note that if ListGrid.canSelectCells is false, the events will be fired as if a click or
*  double click had occurred on the first cell where
*  ListGridField.ignoreKeyboardClicks is not true.
*  
*  Possible actions are:
*  
*  
* - "select" : select the next row or cell in the grid and call 
*    click handlers.
*  
* - "selectOnly" : select the next row or cell in the grid without firing 
*    click handlers.
*  
* - "focus" : move focus to the next row or cell in the grid without 
*    changing the selection or calling click handlers.
*  
* - "activate" : select and activate the next row or cell in the list (calls
*  recordDoubleClick handler)
*  
* - "none" : no action
*  
* - null : if ListGrid.selectionAppearance is "checkbox", behaves as if set
*  to "focus"; otherwise, behaves as if set to "select"
*  
*  
*  Note: If this grid is editable, behavior while editing is governed by the result of 
*  ListGrid.getArrowKeyEditAction.
*  
*  See also ListGrid.generateClickOnEnter, ListGrid.generateClickOnSpace,
*  ListGrid.generateDoubleClickOnEnter and ListGrid.generateDoubleClickOnSpace
* @type {string}
* @default null
*/
arrowKeyAction?: string;

/**
* When the grid body gets keyboard focus, should we highlight the current focus row,
*  using the rollover cell style?
*  
*  This property may be explicitly set to control this behavior independently of
*  ListGrid.showRollOver.
*  Otherwise (if this property is null), we will show the roll-over styling for the
*  keyboard focus row if ListGrid.showRollOver is true.
* @type {boolean}
* @default null
*/
hiliteRowOnFocus?: boolean;

/**
* When enabled, ListGrid.createRecordComponent will be called when saved rows are being
*  rendered, and any returned component will be displayed embedded within the row or cell.
*  
*  recordComponents are not created for newly added rows which have not yet been saved. 
*  See the unsavedRecords for more
*  information.
*  
*  Depending on the ListGrid.showRecordComponentsByCell setting,
*  createRecordComponent() will be called either once per row, or once for
*  every cell.
*  
*  Depending on ListGrid.recordComponentPosition, components can either be placed underneath
*  normal record or cell content ("expand" setting) or placed so that they overlap normal
*  cell content ("within" setting). For the "within" setting, the default is to fill the
*  row or cell, but the component can specify percent size or even use
*  Canvas.snapTo to place itself within the row or cell.
*  
*  The "expand" setting is incompatible with ListGrid.canFreezeFields
*  unless all recordComponents are the same height and they are present
*  in every row, in which case the fixed height of all recordComponents can be
*  set via ListGrid.recordComponentHeight to re-enable frozen fields.
*  
*  Using recordComponents potentially means creating one component for every
*  visible grid row or cell and so can impact performance. Before using this subsystem:
*  
*  
* - consider using ListGridField.valueIcons (possibly with a specified 
*    ListGridField.valueIconClick handler) for icons based on field values
*    which may be displayed alone in the cell or alongside standard content
*    (see ListGridField.showValueIconOnly);
*  
* - for clickable icons representing actions that can be taken on a record, also 
*    consider using ListGridFieldType, or 
*    multiple such fields
*  
* - for controls that only need to appear on rollover, consider
*    ListGrid.showRollOverCanvas
*  
* - if you are trying to customize the editor for a field, you can provide a custom
*    control via ListGridField.editorType, and FormItem.icons are a common
*    way to add clickable buttons. You can also
*    ListGrid.getEditorType. These options are
*    usually better that using recordComponents as custom editors, since
*    you won't have to manage issues like making the recordComponent appear
*    only when editing, having changes affect editing,
*    triggering saves and handling validation errors, etc.
*  
*  
*  See RecordComponentPoolingMode for an overview of how best to optimize use of
*  recordComponents for different data sets.
*  
*  Regardless of the pooling mode, you can explicitly refresh record components via
*  ListGrid.invalidateRecordComponents and
*  ListGrid.refreshRecordComponent.
*  
*  Interaction with ListGrid.autoFitFieldWidths: per-cell record
*  components are not taken into account when determining the size for column auto fit.
*  The default ListGrid.getDefaultFieldWidth implementation looks at cell content
*  only. We typically recommend that, for fields showing record-components, 
*  ListGridField.autoFitWidth and ListGridField.canAutoFitWidth be disabled, or if 
*  the record components are of a predictable size, a ListGridField.defaultWidth
*  be specified.
*  This is particularly pertinent where ListGrid.recordComponentPosition
*  is set to "within", in which case cells' content is often empty or completely covered
*  by record-components.
* @type {boolean}
* @default null
*/
showRecordComponents?: boolean;

/**
* if ListGrid.showRecordComponents is true, how should the component appear within
*  the cell. Valid options are
*  
* - "within": the component will be rendered inside the record / cell.
*  Canvas.snapTo may be set to specify where the component should render within
*  the row or cell, and Canvas.snapOffsetTop / Canvas.snapOffsetLeft may
*  be set to indent recordComponents within their parent cells.
*  Note that if unset, the component will show up at the top/left edge
*  for components embedded within an entire row, or for per-cell components, cell
*  align and valign will be respected. Note also that, when rendering components "within"
*  cells, specified component heights will be respected and will change the height of the
*  row. However, if you want components to completely fill a cell at it's default height,
*  set height: "100%" or rows will render at the default height of the component. 
*  
* - "expand": the component will be written into the cell below the
*  normal cell content, causing the cell to expand vertically to accommodate it.
*  
* - null: If this attribute is unset, we will default to showing
*  recordComponents with position "within" if
*  ListGrid.showRecordComponentsByCell is true, otherwise using "expand"
*  logic.
*  
* @type {EmbeddedPosition}
* @default null
*/
recordComponentPosition?: EmbeddedPosition;

/**
* If true, shows ListGrid.showRecordComponents in cells, rather
*  than just in records.
* @type {boolean}
* @default null
*/
showRecordComponentsByCell?: boolean;

/**
* The method of RecordComponentPoolingMode to employ for
*  ListGrid.showRecordComponents.
*  
*  The default mode is "viewport", which means that recordComponents are destroyed as soon
*  their record is no longer being rendered (scrolled out of the viewport, eliminated by
*  search criteria, etc).
*  
*  For a large or dynamic data set where the components shown on different rows are
*  similar, switch to "recycle" mode, which pools recordComponents by detaching them from
*  records that are not visible and re-using them in other records. In this mode, you
*  should implement ListGrid.updateRecordComponent to apply any changes to make reused
*  components applicable to the new record they appear in, if necessary. For example, if
*  you have several controls in your recordComponents, and not all of the
*  controls apply to every record, your updateRecordComponent() implementation
*  could simply hide or disable inapplicable controls, and this would be much faster than
*  creating a whole new set of controls every time a given record is scrolled into view.
*  
*  If you are using ListGrid.showRecordComponentsByCell, and you
*  have components of different types in different columns and still want to take
*  advantage of component recycling, you can set ListGrid.poolComponentsPerColumn to
*  ensure that components intended for one column are not recycled for use in another
*  column that should have a different component.
*  
*  Note that, if different records have distinctly different components embedded
*  in them, or multiple columns in each record embed different components, you should
*  leave the recordComponentPoolingMode at "viewport" if your dataset is very large or
*  use "data" otherwise.
* @type {RecordComponentPoolingMode}
* @default "viewport"
*/
recordComponentPoolingMode?: RecordComponentPoolingMode;

/**
* Should recycled ListGrid.showRecordComponents, be pooled
*  per column or per record. Only applies if ListGrid.showRecordComponentsByCell is true.
*  
*  When ListGrid.recordComponentPoolingMode is "recycle" and you have components of
*  different types in different columns, set this property to true to ensure that
*  components intended for one column are not recycled for use in another column that
*  should have a different component.
*  
*  If no components applicable to a particular column are available in the pool, the system
*  calls ListGrid.createRecordComponent.
* @type {boolean}
* @default true
*/
poolComponentsPerColumn?: boolean;

/**
* Should we show different styling for the cell the mouse is over?
*  
*  If true, the cell style will have the suffix "Over" appended.
*  
*  Can be overridden on a per-record basis via ListGridRecord.showRollOver.
* @type {boolean}
* @default true
*/
showRollOver?: boolean;

/**
* Name of the property that can be set on a per-record basis to disabled rollover for an
*  individual record when ListGrid.showRollOver is true.
* @type {string}
* @default "showRollOver"
*/
recordShowRollOverProperty?: string;

/**
* Are rollovers cell-level or row-level?
* @type {boolean}
* @default null
*/
useCellRollOvers?: boolean;

/**
* When enabled, when the mouse moves over a row or cell (depending on
*  ListGrid.useCellRollOvers), an arbitrary Canvas can be shown layered on top of the row or
*  cell (the ListGrid.rollOverCanvas), layered underneath the row or cell (the
*  ListGrid.rollUnderCanvas), or both. 
*  
*  This can be used to dynamically show controls or informational displays only on
*  rollover. For example, controls to delete a row might appear only on rollover so they
*  do not clutter the static display, or a "rollUnder" Canvas could be used to display
*  additional information that can appear behind normal cell values (like displaying
*  percent complete via as a bar of color that appears behind text values).
*  
*  Canvas.snapTo can be used to place the
*  rollOver/rollUnderCanvas. With useCellRollOvers, positioning is relative
*  to the cell, for row-level rollOver, position is relative to the portion of the
*  row that is scrolled into view (this implies a row-level rollOver/UnderCanvas can never
*  be placed horizontally scrolled out of view, but this is possible for a cell-level
*  rollOver). 
*  
*  snapTo positioning makes it easy to do something like place a button at the
*  right edge of the grid, next to the scrollbar: just set snapTo:"R" on the
*  rollOverCanvas.
*  
*  The rollOver/rollUnder Canvas can be a single static component (the same for all
*  cells/rows) configured via the AutoChild system, or can instead be provided
*  dynamically by implementing ListGrid.getRollOverCanvas and/or ListGrid.getRollUnderCanvas.
*  
*  The rollOver/rollUnder canvas will be automatically added to the grid's
*  ListGrid.body as an 
*  ListGrid.addEmbeddedComponent.
*  For grids with ListGridField.frozen, the behavior is as follows:
*  
* - If ListGrid.useCellRollOvers is false (the default), embedded components
*   will be added to both the body and the frozen body
*  
* - Otherwise the component will be added to whichever body contains the cell the
*   user is currently over
*  The rollOver/rollUnder canvas added to the frozen body will be created by calling
*  the ListGrid.getFrozenRollOverCanvas or ListGrid.getFrozenRollUnderCanvas methods.
*  The default implementation for these methods matches their equivalents for non-frozen
*  rollOver / rollUnder canvases - it will use the autoChild subsystem to create a
*  canvas from the ListGrid.rollOverCanvas autoChild configuration.
*  
*  showRollOverCanvas has no effect if ListGrid.showRollOver is false.
*  
*  See also ListGrid.showSelectedRollOverCanvas.
* @type {boolean}
* @default null
*/
showRollOverCanvas?: boolean;

/**
* AutoChild created and embedded in the grid if ListGrid.showRollOver
*  is true and ListGrid.showRollOverCanvas is
*  true or for selected records, if
*  ListGrid.showSelectedRollOverCanvas 
*  is true. This component will be created and displayed above the current rollOver
*  row or cell.
*  
*  Note that if this grid has frozen fields, the AutoChild subsystem will use the 
*  rollOverCanvas configuration settings to create the ListGrid.frozenRollOverCanvas
*  (displayed in the frozen listGrid body).
*  
*  The rollOverCanvas has the following read-only attributes set:
*  - this.grid - a pointer to the grid
*  - this.record - a pointer to the current roll over record object in the grid
* @type {Canvas}
* @default null
*/
rollOverCanvas?: Canvas;

/**
* Automatically generated canvas embedded in the grid's frozen body if 
*  ListGrid.showRollOver
*  is true and ListGrid.showRollOverCanvas is
*  true or for selected records, if
*  ListGrid.showSelectedRollOverCanvas 
*  is true. This component will be created and displayed above the current rollOver
*  row or cell in the frozen body.
*  
*  The frozenRollOverCanvas will be created using the AutoChild subsystem, and
*  will derive its configuration from the ListGrid.rollOverCanvas
*  autoChild properties ("rollOverCanvasProperties", et al).
*  
*  The frozenRollOverCanvas has the following read-only attributes set:
*  - this.grid - a pointer to the grid
*  - this.record - a pointer to the current roll over record object in the grid
* @type {Canvas}
* @default null
*/
frozenRollOverCanvas?: Canvas;

/**
* If roll overs are enabled, should the ListGrid.rollUnderCanvas
*  be displayed?
*  
*  Use of the showRollUnderCanvas is enabled if ListGrid.showRollOver
*  is true, and either ListGrid.showRollOverCanvas
*  is true and showRollUnderCanvas is unset, or showRollUnderCanvas
*  is explicitly set to true.
*  
*  See also ListGrid.showSelectedRollUnderCanvas.
* @type {boolean}
* @default null
*/
showRollUnderCanvas?: boolean;

/**
* AutoChild created and embedded in the grid if ListGrid.showRollOver
*  is true, and either ListGrid.showRollOverCanvas
*  is true and ListGrid.showRollUnderCanvas is
*  unset, or showRollUnderCanvas is explicitly set to true.
*  This component will be created and displayed behind the current rollOver row or cell in the
*  page's z-order, meaning that it will only be visible if the cell styling is transparent.
*  
*  Note that if this grid has frozen fields, the AutoChild subsystem will use the 
*  rollUnderCanvas configuration settings to create the ListGrid.frozenRollUnderCanvas
*  (displayed in the frozen listGrid body).
*  
*  The rollUnderCanvas has the following read-only attributes set:
*  - this.grid - a pointer to the grid
*  - this.record - a pointer to the current roll over record object in the grid
* @type {Canvas}
* @default null
*/
rollUnderCanvas?: Canvas;

/**
* Automatically generated canvas embedded in the grid's frozen body as a 
*  ListGrid.rollUnderCanvas.
*  This component will be created and displayed above the current rollOver
*  row or cell in the frozen body.
*  
*  The frozenRollUnderCanvas will be created using the AutoChild subsystem, and
*  will derive its configuration from the ListGrid.rollUnderCanvas
*  autoChild properties ("rollUnderCanvasProperties", et al).
*  
*  The frozenRollUnderCanvas has the following read-only attributes set:
*  - this.grid - a pointer to the grid
*  - this.record - a pointer to the current roll over record object in the grid
* @type {Canvas}
* @default null
*/
frozenRollUnderCanvas?: Canvas;

/**
* This setting causes the ListGrid.rollOverCanvas to be
*  displayed when the user rolls over selected records in the grid (but not when
*  rolling over other records). This can be useful to display a "Selected Over"
*  appearance which can't be easily achieved via standard cell styling.
* @type {boolean}
* @default false
*/
showSelectedRollOverCanvas?: boolean;

/**
* This setting causes the ListGrid.rollUnderCanvas to be
*  displayed when the user rolls over selected records in the grid (but not when
*  rolling over other records). This can be useful to display a "Selected Over"
*  appearance which can't be easily achieved via standard cell styling.
*  
*  As with ListGrid.showRollUnderCanvas, if this property is unset, but
*  the related ListGrid.showSelectedRollOverCanvas property is true, both the
*  the roll under and roll under canvases will be displayed as the user rolls 
*  over selected records.
* @type {boolean}
* @default false
*/
showSelectedRollUnderCanvas?: boolean;

/**
* This setting causes the ListGrid.rollOverCanvas to be
*  sized to cover the normal row and the expansion layout. Otherwise the
*  rollOverCanvas is only shown for the un-expanded part of the row.
* @type {boolean}
* @default null
*/
showRollOverInExpansion?: boolean;

/**
* If ListGrid.showRollOver or ListGrid.hiliteRowOnFocus is true
*  the current keyboard focus row for navigation via arrow keys, etc, will
*  be hilighted with "Over" styling. This is particularly 
*  valuable to indicate which row has keyboard focus where there are multiple 
*  selected rows, or where the user is navigating without changing selection 
*  (see ListGrid.arrowKeyAction).
*  However, note that as the user interacts with the rows using the mouse, the rollover 
*  styling will be updated to reflect the mouse position if ListGrid.showRollOver is true.
*  
*  When the user rolls the mouse off the grid, the default behavior is to re-style
*  the current focus row with "Over" styling if the grid has keyboard
*  focus. That way a user has a clear visual indication of where navigation 
*  would start. This may be disabled by setting preserveFocusStylingOnMouseOut
*  to false.
* @type {boolean}
* @default true
*/
preserveFocusStylingOnMouseOut?: boolean;

/**
* If the ListGrid.rollOverCanvas is enabled, setting this property
*  to true ensures that when the rollOverCanvas is displayed it
*  is animated into view via Canvas.animateShow. Note that the animation effect
*  may be customized via Canvas.animateShowEffect, Canvas.animateShowTime and
*  Canvas.animateShowAcceleration set in rollOverCanvasProperties.
* @type {boolean}
* @default false
*/
animateRollOver?: boolean;

/**
* If the ListGrid.rollUnderCanvas is enabled, setting this property
*  to true ensures that when the rollUnderCanvas is displayed it
*  is animated into view via Canvas.animateShow. Note that the animation effect
*  may be customized via Canvas.animateShowEffect, Canvas.animateShowTime and
*  Canvas.animateShowAcceleration set in rollUnderCanvasProperties.
* @type {boolean}
* @default false
*/
animateRollUnder?: boolean;

/**
* If true this grid will create and show per-row backgroundComponents
*  as detailed ListGrid.backgroundComponent.
* @type {boolean}
* @default false
*/
showBackgroundComponents?: boolean;

/**
* Has no effect unless ListGrid.showBackgroundComponents is true.
*  
*  Canvas created and embedded in the body behind a given record.  When
*  ListGridRecord.backgroundComponent is set, this autoChild canvas
*  will be constructed (if listGridRecord.backgroundComponent is not already a Canvas) and
*  its properties combined with those of listGridRecord.backgroundComponent and then
*  displayed behind a specific record in the page's z-order, meaning
*  it will only be visible if the cell styling is transparent.
* @type {Canvas}
* @default null
*/
backgroundComponent?: Canvas;

/**
* If true, cellHover and rowHover events will fire and then a hover will be shown (if not
*  canceled) when the user leaves the mouse over a row / cell unless the corresponding field has
*  ListGridField.showHover set to false. If unset or null, the hover will be
*  shown if the corresponding field has showHover:true. If false, then hovers are disabled.
*  
*  Note that standard hovers override ListGrid.showClippedValuesOnHover. Thus,
*  to enable clipped value hovers, canHover must be unset or null and the corresponding field
*  must have showHover unset or null as well.
* @type {boolean}
* @default null
*/
canHover?: boolean;

/**
* If true, and ListGrid.canHover is also true, shows popup hover text 
*  next to the mouse when the user hovers over a cell. The content of the hover is 
*  determined by ListGrid.cellHoverHTML.
*  
*  This is the default setting for the grid and can be overridden on a 
*  ListGridField.showHover basis.
* @type {boolean}
* @default true
*/
showHover?: boolean;

/**
* If true and a header button's title is clipped, then a hover containing the full field
*  title is enabled.
* @type {boolean}
* @default true
*/
showClippedHeaderTitlesOnHover?: boolean;

/**
* Style to apply to hovers shown over this grid.
* @type {CSSStyleName}
* @default "gridHover"
*/
hoverStyle?: CSSStyleName;

/**
* The selection associated with the ListGrid.
* @deprecated * in favor of ListGrid.selectionManager

* @type {Selection | CellSelection}
* @default null
*/
selection?: Selection | CellSelection;

/**
* The selection associated with the ListGrid.
* @type {Selection | CellSelection | MultiLinkSelection}
* @default null
*/
selectionManager?: Selection | CellSelection | MultiLinkSelection;

/**
* How selection of rows should be presented to the user.
*  
*  For selectionAppearance:"checkbox" with multiple selection
*  allowed, you would typically use ListGrid.selectionType:"simple" (the default). Because
*  selectionType and selectionAppearance are unrelated,
*  the combination of selectionAppearance:"checkbox" and
*  selectionType:"multiple" results in a grid where multiple selection can
*  only be achieved via shift-click or ctrl-click.
*  
*  If using "checkbox" for a ListGrid, see also
*  ListGrid.checkboxField for customization APIs.
*  
*  If using "checkbox" for a TreeGrid, an extra icon,
*  TreeGrid.getExtraIcon is not supported. Additionally only
*  ListGrid.selectionType:"simple" and "single" are supported.
*  You can also toggle the display of a disabled checkbox on a treeGrid, displayed
*  when the node can't be selected, via TreeGrid.showDisabledSelectionCheckbox.
*  
*  Note that the default behavior when you enable checkbox selection is to continue to show
*  the selected style. This can be changed by setting ListGrid.showSelectedStyle to false.
* @type {SelectionAppearance}
* @default "rowStyle"
*/
selectionAppearance?: SelectionAppearance;

/**
* Controls whether a checkbox for selecting all records appears in the header with
*  ListGrid.selectionAppearance set to "checkbox"
* @type {boolean}
* @default null
*/
canSelectAll?: boolean;

/**
* Should partial selection of all records be shown in header with a special icon?
*  The partial icon will show in the header when ListGrid.canSelectAll is
*  enabled and at least one record is selected but all records are not selected.
*  To only show all selected and none selected states, set this attribute to false.
* @type {boolean}
* @default null
*/
showHeaderPartialSelection?: boolean;

/**
* Controls whether a checkbox for selecting ListGrid.groupBy appears
*  in the group node if SelectionAppearance is set to "checkbox"
* @type {boolean}
* @default false
*/
canSelectGroups?: boolean;

/**
* Should partially selected parents (in a Tree data set) be shown with special icon?
*  This has an impact in grouped grids where ListGrid.canSelectGroups is true. The
*  partial icon will show up for the group header node when a group is partially
*  selected.
* @type {boolean}
* @default false
*/
showPartialSelection?: boolean;

/**
* Should partially selected checkbox be deselected or selected on click? This setting
*  affects ListGrid.canSelectAll, ListGrid.canSelectGroups and folder checkbox selection in a Tree data set.
*  
*  By default clicking a partially selected checkbox selects it.
* @type {boolean}
* @default false
*/
deselectOnPartialCheckboxClick?: boolean;

/**
* Defines a listGrid's clickable-selection behavior.
*  
*  The default selection appearance is governed by ListGrid.selectionAppearance: if
*  selectionAppearance is "checkbox", this will be "simple", otherwise, this will be
*  "multiple".
* @type {SelectionStyle}
* @default null
*/
selectionType?: SelectionStyle;

/**
* If specified, the selection object for this list will use this property to mark records
*  as selected. In other words, if this attribute were set to "isSelected"
*  any records in the listGrid data where "isSelected" is true
*  will show up as selected in the grid. Similarly if records are selected within the grid
*  after the grid has been created, this property will be set to true on the selected
*  records.
* @type {string}
* @default null
*/
selectionProperty?: string;

/**
* If true, when an update operation occurs on a selected record in a
*  ListGrid.dataSource listGrid, ensure the updated record is
*  re-selected when the operation completes.
*  The ListGrid.reselectOnUpdateNotifications attributes governs whether
*  ListGrid.selectionUpdated and ListGrid.selectionChanged will fire
*  when this occurs.
* @type {boolean}
* @default true
*/
reselectOnUpdate?: boolean;

/**
* if ListGrid.reselectOnUpdate is true, this property governs what
*  selection changed notifications should be triggered when a selected record
*  is edited then automatically reselected when the edited data is merged into
*  the data set.
* @type {SelectionNotificationType}
* @default "selectionChanged"
*/
reselectOnUpdateNotifications?: SelectionNotificationType;

/**
* If set to false on a record, selection of that record is disallowed.
* @type {string}
* @default "canSelect"
*/
recordCanSelectProperty?: string;

/**
* Enables cell-level selection behavior as well as
*  ListGrid.useCellRollOvers.
*  
*  To query and manipulate cell-level selections, use ListGrid.getCellSelection to retrieve
*  the CellSelection.
*  
*  Note that the ListGrid has a data model of one Record per row, unlike the
*  CubeGrid which supports one CellRecord per cell. For this reason
*  record-oriented APIs that act on the selection will act on entire Records that have
*  any selected cells (examples include drag and drop and transferSelectedData()).
*  
*  More generally, canSelectCells is primarily intended to enable developers
*  to build Excel-like interactions on local datasets, by using ListGrid.setData plus
*  ListGrid.saveLocally:true rather than record-oriented DataSources and data binding.
*  You can also use canSelectCells in conjunction with SelectionAppearance 
*  set to "checkbox" to complete this experience.
*  
*  The following keyboard selection behaviors are enabled with this property in
*  addition to standard single-selection Arrow Key navigation:
*  
*  SHIFT +    [Arrow Key]: begin or continue incremental selection
*  
*  SHIFT + CTRL + [Arrow Key]: incremental selection to the end of row or column
*  
*  CTRL + A: select all cells (enabled only with ListGrid.canSelectAll)
*  
*  Incremental selection allows selection of rows and columns of cells via keyboard
*  or mouse provided the shift key is down. Behavior is designed to match Excel.
*  Thus, if a previous selection has begun, cells will be selected from that origin.
*  
*  Users may also navigate through cells using the Tab and Shift+Tab
*  keypresses if ListGrid.navigateOnTab is true. When a user tabs to the
*  end of the row, the ListGrid.rowEndEditAction is used to determine whether
*  to shift selection to the next row, return to the beginning of the same row, or
*  simply move on through the page's tab order.
* @type {boolean}
* @default false
*/
canSelectCells?: boolean;

/**
* If isc.setScreenReaderMode, and
*  ListGrid.canSelectCells is true should the user be able to navigate 
*  the grid cell-by-cell, highlighting and focusing on individual cells within the
*  selected row via left and right arrow keypresses?
*  
*  If ListGrid.canSelectCells is true, this property will have no effect as
*  all navigation is by cell.
* @type {boolean}
* @default false
*/
screenReaderNavigateByCell?: boolean;

/**
* If ListGrid.canSelectCells is true, this property allows the user
*  to navigate through the cells of a grid using Tab and Shift+Tab keypresses.
*  When a user tabs to the
*  end of the row, the ListGrid.rowEndEditAction is used to determine whether
*  to shift selection to the next row, return to the beginning of the same row, or
*  simply move on through the page's tab order.
*  
*  Note - if this property is not explicitly set, navigateOnTab behavior will be 
*  enabled for grids unless isc.setScreenReaderMode in 
*  which case it will be disabled. 
*  Developers should be aware that setting navigateOnTab explicitly to true
*  enabled the behavior even in screenReader mode. This may have an impact on the 
*  accessibility of an application - screen reader mode users navigating the
*  application via the keyboard would have to tab through every single data cell
*  in the grid grid before being able to tab to the next component.
* @type {boolean}
* @default null
*/
navigateOnTab?: boolean;

/**
* For ListGrids with ListGrid.canSelectCells, enabling this
*  property will cause the listGrid to intercept standard browser copy/paste shortcut
*  keys and perform the following behavior.
*  
*  
* - ctrl+c: retrieve selected cell data 
*    via a call to ListGrid.getSelectedCellData, and temporarily store it 
*    in memory in a "clipboard" variable.
*  
* - ctrl+v: apply any previously copied data stored in the "clipboard" variable 
*    into the current grid selection via ListGrid.applyCellData.
*  
* - ctrl+d: copy cell values from top row of selected cells down to all rows
*  
* - ctrl+r: copy cell values from left column of selected cells right to all columns
*  
*  Note: setting this property to true will disable standard copy and paste behavior
*  to the native Browser or OS-level clipboard. To copy data to and
*  from applications outside of the browser, use the technique shown in the
*  ${isc.DocUtils.linkForExampleId('gridToExcel', 'Grid to Excel')} and
*  ${isc.DocUtils.linkForExampleId('excelToGrid', 'Excel to Grid')} samples.
*  
*  If this property is unset, default behavior will enable these shortcuts if
*  ListGrid.canSelectCells is true, and ListGrid.canDragSelectText and
*  ListGrid.selectCellTextOnClick are both false, so as to minimize the chances
*  of interfering with native copy and paste of cell content.
* @type {boolean}
* @default null
*/
useCopyPasteShortcuts?: boolean;

/**
* Name of a CSS Style to use as the ListGrid.baseStyle for a cell that
*  is currently a selection origin for shifted incremental cell selection.
*  Only has an effect if ListGrid.canSelectCells is true.
*  
* @type {CSSStyleName}
* @default null
*/
originBaseStyle?: CSSStyleName;

/**
* If this property is true, users can drag the mouse to select several rows or cells.
*  This is mutually exclusive with rearranging rows or cells by dragging.
*  
*  NOTE: If canDragSelect is initially enabled or might be
*  dynamically enabled after the grid is created, it may be desirable to disable
*  Canvas.useTouchScrolling so that touch-dragging records/cells
*  selects them rather than starting a scroll. If Canvas.disableTouchScrollingForDrag
*  is set to true, then touch scrolling will be disabled automatically.
*  However, for accessibility reasons, it is recommended to
*  leave touch scrolling enabled and provide an alternative set of controls that can be
*  used to perform drag-selection.
* @type {boolean}
* @default false
*/
canDragSelect?: boolean;

/**
* If this property is true, users can drag the mouse to select text within grid rows,
*  ready to be cliped to clipboard.
*  This is mutually exclusive with
*  ListGrid.canReorderRecords, and with
*  ListGrid.canDragSelect.
*  
*  To enable selecting cell text on click, see ListGrid.selectCellTextOnClick.
* @type {boolean}
* @default false
*/
canDragSelectText?: boolean;

/**
* If set to false, dropping over an empty part of the grid body is disallowed and the
*  no-drop indicator is displayed.
* @type {boolean}
* @default true
*/
canDropInEmptyArea?: boolean;

/**
* If ListGrid.selectionType is set to
*  "single",
*  
*  setting this property to true means selection will be displayed to the
*  user with the ListGrid.selectionCanvas and/or
*  ListGrid.selectionUnderCanvas rather than with CSS styling.
*  
*  If showSelectionCanvas is set to true, then the
*  selectionUnderCanvas will automatically be enabled unless
*  ListGrid.showSelectionUnderCanvas is set to false.
*  
*  NOTE: It is recommended to use the selectionUnderCanvas rather than the
*  selectionCanvas if possible because the selectionCanvas is
*  stacked on top of the selected record and this may interfere with event handling in rare
*  cases. If no interactive components are shown in the selectionCanvas and it
*  simply provides custom styling, then the selectionUnderCanvas should be used
*  instead.
*  
*  With ListGrid.canFreezeFields, the selectionCanvas
*  is displayed only over the non-frozen fields of the selected row.
* @type {boolean}
* @default null
*/
showSelectionCanvas?: boolean;

/**
* AutoChild created and embedded in the grid if ListGrid.showSelectionCanvas
*  is true and the ListGrid.selectionType is
*  "single".
*  
*  This component will be created and displayed above the selected record whenever the
*  selection changes.
*  
*  NOTE: It is recommended to use the ListGrid.selectionUnderCanvas
*  rather than the selectionCanvas if possible because the selectionCanvas
*  is stacked on top of the selected record and this may interfere with event handling in rare
*  cases. If no interactive components are shown in the selectionCanvas and it
*  simply provides custom styling, then the selectionUnderCanvas should be used
*  instead.
*  
*  The selectionCanvas has the following read-only attributes set:
*  - this.grid - a pointer to the grid
*  - this.record - a pointer to the currently selected record in the grid
* @type {Canvas}
* @default null
*/
selectionCanvas?: Canvas;

/**
* If ListGrid.selectionType is set to
*  "single",
*  
*  and either ListGrid.showSelectionCanvas is true and
*  showSelectionUnderCanvas is unset, or showSelectionUnderCanvas
*  is explicitly set to true, then selection will be displayed to the
*  user with the ListGrid.selectionCanvas and/or
*  ListGrid.selectionUnderCanvas rather than with CSS styling.
*  Setting showSelectionUnderCanvas to false will disable
*  the use of the selectionUnderCanvas.
*  
*  With ListGrid.canFreezeFields, the selectionUnderCanvas
*  is displayed only behind the non-frozen fields of the selected row.
* @type {boolean}
* @default null
*/
showSelectionUnderCanvas?: boolean;

/**
* AutoChild created and embedded in the grid if ListGrid.showSelectionCanvas
*  is true and ListGrid.showSelectionUnderCanvas
*  is unset, or showSelectionUnderCanvas is explicitly set to true,
*  and the ListGrid.selectionType is
*  "single".
*  
*  This component will be created and displayed behind the selected record whenever the
*  selection changes.
*  
*  The selectionUnderCanvas has the following read-only attributes set:
*  - this.grid - a pointer to the grid
*  - this.record - a pointer to the currently selected record object in the grid
* @type {Canvas}
* @default null
*/
selectionUnderCanvas?: Canvas;

/**
* If the ListGrid.selectionCanvas is enabled, setting
*  this property to true ensures that when the selectionCanvas
*  is displayed it is animated into view via Canvas.animateShow. Note that the
*  animation effect may be customized via Canvas.animateShowEffect,
*  Canvas.animateShowTime and Canvas.animateShowAcceleration set in
*  selectionCanvasProperties.
* @type {boolean}
* @default false
*/
animateSelection?: boolean;

/**
* If the ListGrid.selectionUnderCanvas is enabled, setting
*  this property to true ensures that when the selectionUnderCanvas
*  is displayed it is animated into view via Canvas.animateShow. Note that the
*  animation effect may be customized via Canvas.animateShowEffect,
*  Canvas.animateShowTime and Canvas.animateShowAcceleration set in
*  selectionUnderCanvasProperties.
* @type {boolean}
* @default false
*/
animateSelectionUnder?: boolean;

/**
* Returns the specially generated checkbox field used when SelectionAppearance is
*  "checkbox". Created via the AutoChild pattern so that
*  checkboxFieldDefaults and checkboxFieldProperties are available
*  for skinning purposes. Note that ListGridField.shouldPrint is false
*  for the checkboxField by default - if you want this column to show up in the grid's print
*  view, use checkboxFieldProperties to set this property to true.
*  
*  This field will render an icon to indicate the selected state of each row, which, when
*  clicked will toggle the selection state. The icon src may be configured using
*  ListGrid.checkboxFieldTrueImage and ListGrid.checkboxFieldFalseImage, as
*  well as ListGrid.checkboxFieldImageWidth and ListGrid.checkboxFieldImageHeight.
*  
*  The checkboxField can be detected by calling ListGrid.isCheckboxField on any
*  ListGridField object.
* @type {ListGridField}
* @default null
*/
checkboxField?: ListGridField;

/**
* If ListGrid.selectionAppearance is set to "checkbox" this property
*  determines the image to display in the checkbox field for a selected row.
*  If unset, the ListGrid.booleanTrueImage will be used. Note that the special
*  value "blank" means that no image will be shown.
* @type {SCImgURL}
* @default null
*/
checkboxFieldTrueImage?: SCImgURL;

/**
* If ListGrid.selectionAppearance is set to "checkbox" this property
*  determines the image to display in the checkbox field for an unselected row.
*  If unset, the ListGrid.booleanFalseImage will be used. Note that the special
*  value "blank" means that no image will be shown.
* @type {SCImgURL}
* @default null
*/
checkboxFieldFalseImage?: SCImgURL;

/**
* If ListGrid.selectionAppearance is set to "checkbox" this property
*  determines the image to display in the checkbox field for a partially selected row.
*  If unset, the ListGrid.booleanPartialImage will be used. Note that the special
*  value "blank" means that no image will be shown.
* @type {SCImgURL}
* @default null
*/
checkboxFieldPartialImage?: SCImgURL;

/**
* If set, the ListGrid.checkboxFieldTrueImage to use when printing.
* @type {SCImgURL}
* @default null
*/
printCheckboxFieldTrueImage?: SCImgURL;

/**
* If set, the ListGrid.checkboxFieldFalseImage to use when printing.
* @type {SCImgURL}
* @default null
*/
printCheckboxFieldFalseImage?: SCImgURL;

/**
* If set, the ListGrid.checkboxFieldPartialImage to use when printing.
* @type {SCImgURL}
* @default null
*/
printCheckboxFieldPartialImage?: SCImgURL;

/**
* If ListGrid.selectionAppearance is set to "checkbox" this property
*  may be set to govern the width of the checkbox image displayed to indicate whether a row is
*  selected. If unset, the checkboxField image will be sized to match the
*  ListGrid.booleanImageWidth for this grid.
* @type {Integer}
* @default null
*/
checkboxFieldImageWidth?: Integer;

/**
* If ListGrid.selectionAppearance is set to "checkbox" this property
*  may be set to govern the height of the checkbox image displayed to indicate whether a row is
*  selected. If unset, the checkboxField image will be sized to match the
*  ListGrid.booleanImageHeight for this grid.
* @type {Integer}
* @default null
*/
checkboxFieldImageHeight?: Integer;

/**
* Indicates whether the text of the emptyMessage property should be displayed if no data is
*  available.
* @type {boolean}
* @default true
*/
showEmptyMessage?: boolean;

/**
* The string to display in the body of a listGrid with an empty data array, if
*  showEmptyMessage is true.
* @type {HTMLString}
* @default "No items to show."
*/
emptyMessage?: HTMLString;

/**
* The CSS style name applied to the ListGrid.emptyMessage if displayed.
* @type {CSSStyleName}
* @default "emptyMessage"
*/
emptyMessageStyle?: CSSStyleName;

/**
* The prompt to show when the mouse hovers over the Filter button in the FilterEditor.
* @type {string}
* @default "Filter"
*/
filterButtonPrompt?: string;

/**
* The string to display in the body of a listGrid while data is being loaded.
*  Use "&#36;{loadingImage}" to include Canvas.loadingImageSrc.
* @type {HTMLString}
* @default "${loadingImage}&amp;nbsp;Loading data..."
*/
loadingDataMessage?: HTMLString;

/**
* The CSS style name applied to the loadingDataMessage string if displayed.
* @type {CSSStyleName}
* @default "loadingDataMessage"
*/
loadingDataMessageStyle?: CSSStyleName;

/**
* If you have a databound listGrid and you scroll out of the currently loaded dataset, by
*  default you will see blank rows until the server returns the data for those rows. The
*  loadingMessage attribute allows you to specify arbitrary html that will be shown in each
*  such "blank" record while the data for that record is loading.
* @type {string}
* @default "&amp;nbsp;"
*/
loadingMessage?: string;

/**
* If record[this.singleCellValueProperty] is set for some record, the
*  record will be displayed as a single cell spanning every column in the grid, with
*  contents set to the value of record[this.singleCellValueProperty].
* @type {string}
* @default "singleCellValue"
*/
singleCellValueProperty?: string;

/**
* If record[this.isSeparatorProperty] is set for some record, the
*  record will be displayed as a simple separator row.
* @type {string}
* @default "isSeparator"
*/
isSeparatorProperty?: string;

/**
* Should this listGrid display a filter row. If true, this ListGrid
*  will be drawn with a single editable row, (separate from the body) with a filter button.
*  
*  Values entered into this row are used as filter criteria to filter this List's data.
*  The ListGrid.filterByCell and ListGrid.filterOnKeypress attributes allow
*  developers to configure whether filtering occurs automatically on change or requires
*  an enter-keypress or filter button click.
*  ListGrid.autoFetchTextMatchStyle determines
*  the textMatchStyle for the request passed to ListGrid.fetchData.
*  
*  The default FormItem.operator for an item in the filterEditor can
*  be set via ListGridField.filterOperator.  When field.filterOperator 
*  has been set calls to retrieve the criteria from the grid return AdvancedCriteria. 
*  See also ListGrid.allowFilterOperators for a UI that allows end users to change the search
*  operator on the fly
*  
*  Note that if ListGrid.filterData or ListGrid.fetchData is called
*  directly while the filter editor is showing, the filter editor values will be updated to
*  reflect the new set of criteria. If you wish to retain the user entered filter criteria
*  and modify a subset of field values programmatically, this can be achieved by
*  copying the existing set of criteria and adding other changes - something
*  like this:
*  
*  
*   var newCriteria = myListGrid.getFilterEditorCriteria();
*   isc.DataSource.combineCriteria(newCriteria, {
*    field1:"new value1",
*    field2:"new value2"
*   });
*   myListGrid.setCriteria(newCriteria);
*  
*  
*  
*  In this example code we're using ListGrid.getFilterEditorCriteria rather than
*  ListGrid.getCriteria - this ensures that if the user has typed a new value into
*  the filter editor, but not yet clicked the filter button, we pick up the value the user
*  entered. This sample code uses DataSource.combineCriteria to combine the
*  existing filterEditorCriteria with some new custom criteria. This technique is
*  applicable to both simple and advanced criteria.
*  
*  If you call filterData() and pass in criteria for dataSource
*  fields that are not present in the ListGrid, these criteria will continue to be applied
*  along with the user-visible criteria.
*  
*  filterEditor and advanced criteria: If a developer calls filterData()
*  on a ListGrid and passes in AdvancedCriteria, expected behavior of the filter
*  editor becomes ambiguous, since AdvancedCriteria has far more complex filter
*  expression support than the ordinary filterEditor can represent.
*  
*  Default behavior for AdvancedCriteria will combine the AdvancedCriteria with the values
*  in the filter editor as follows:
*  
*  
* - If the top level criteria has operator of type "and":
*  Each field in the top level
*  criteria array for which a 'canFilter' true field is shown in the listGrid will show up
*  if the specified operator matches the default filter behavior
*  (based on the ListGrid.autoFetchTextMatchStyle).
*  If the user enters values in the filter editor, these will be combined with the
*  existing AdvancedCriteria by either replacing or adding field level criteria at the top
*  level.
*  
* - If the top level criteria is a single field-criteria:
*  If the field shows up in the listGrid and is canFilter:true, it will be displayed to
*  the user (if the operator matches the default filter behavior for the field).
*  If the user enters new filter criteria in the filterEditor, they will be combined with
*  this existing criterion via a top level "and" operator, or if the user modifies the
*  field for which the criterion already existed, it will be replaced.
*  
* - Otherwise, if there are multiple top level criteria combined with an "or" operator,
*  these will not be shown in the filter editor. Any filter parameters the user enters will
*  be added to the existing criteria via an additional top level "and" operator, meaning
*  the user will essentially filter a subset of the existing criteria
*  
* @type {boolean}
* @default false
*/
showFilterEditor?: boolean;

/**
* Should a menu item allowing the user to show or hide the 
*  ListGrid.showFilterEditor be displayed in 
*  the ListGrid.headerContextMenu?
*  
*  Note that if this ListGrid is not ListGrid.dataSource 
*  it can not be filtered. In this case the context menu
*  option to show the filterEditor will not be displayed even if 
*  canShowFilterEditor is true.
* @type {boolean}
* @default true
*/
canShowFilterEditor?: boolean;

/**
* When ListGrid.canShowFilterEditor is true, this is the title
*  for the filterEditor show/hide menu-item, in the 
*  ListGrid.headerContextMenu, when the filterEditor is hidden.
*  
*  ListGrid.hideFilterEditorTitle affects the same menu-item
*  when the filterEditor is visible.
* @type {string}
* @default "Show Filter Row"
*/
showFilterEditorTitle?: string;

/**
* When ListGrid.canShowFilterEditor is true, this is the title
*  for the filterEditor show/hide menu-item, in the 
*  ListGrid.headerContextMenu, when the filterEditor is visible.
*  
*  ListGrid.showFilterEditorTitle affects the same menu-item
*  when the filterEditor is hidden.
* @type {string}
* @default "Hide Filter Row"
*/
hideFilterEditorTitle?: string;

/**
* If ListGrid.showFilterEditor is set to true, the filterEditor
*  is automatically created as an AutoChild.
*  
*  The filterEditor autoChild is a RecordEditor - essentially it is a specialized listGrid
*  in edit mode for editing a single set of values to be used as criteria. Once created, 
*  developers may access it and use standard listGrid APIs to interact with it.
*  For example, given a listGrid myListGrid,
*  live edit items could be accessed via
*  
*  
*  myListGrid.filterEditor.getEditFormItem(someFieldName);
*  
*  
*  
*  
*  Developers may configure the AutoChild using ListGrid.filterEditorProperties.
* @type {RecordEditor}
* @default null
*/
filterEditor?: RecordEditor;

/**
* Properties to apply to the automatically generated ListGrid.filterEditor
*  if ListGrid.showFilterEditor is true.
* @type {Partial<RecordEditor>}
* @default null
*/
filterEditorProperties?: Partial<RecordEditor>;

/**
* If ListGrid.showFilterEditor is true, this attribute may be used to customize the
*  filter button shown to the right of the filterEditor row.
* @type {Partial<Button>}
* @default null
*/
filterButtonProperties?: Partial<Button>;

/**
* Height for the filterEditor, if shown.
* @type {number}
* @default "22"
*/
filterEditorHeight?: number;

/**
* When set to false, this attribute prevents user-criteria in the 
*  ListGrid.filterEditor from being cleared when the user hides it.
* @type {boolean}
* @default true
*/
clearCriteriaOnFilterEditorHide?: boolean;

/**
* When this grid is initially filtered via ListGrid.autoFetchData, or filtered by the user
*  via the ListGrid.showFilterEditor, this attribute can be used to set the
*  textMatchStyle on the dsRequest passed to fetchData().
*  
*  To use a mixture of textMatchStyles, set an appropriate
*  FormItem.operator on a field's
*  ListGridField.filterEditorProperties.
* @type {TextMatchStyle}
* @default "substring"
*/
autoFetchTextMatchStyle?: TextMatchStyle;

/**
* Causes a menu item titled ListGrid.filterUsingText to appear in the
*  ListGrid.showHeaderContextMenu that allows the end user to pick an advanced
*  OperatorId to use for this field.
*  
*  Once an operator has been chosen, the active operator is indicated by an
*  ListGrid.operatorIcon placed within the field (you can alternatively cause the
*  icon to ListGrid.alwaysShowOperatorIcon). The operatorIcon 
*  shows the same textual representation of the search operator as is used by the
*  FormItem.allowExpressions feature. Clicking on the icon provides a second way to 
*  modify the search operator. 
*  
*  This feature is enabled by default if DataSource.supportsAdvancedCriteria is true, 
*  for all fields where it is normally possible to filter by typing in a search string. This 
*  excludes field types such as "date" or "boolean" which show specialized filter controls. 
*  Use ListGridField.allowFilterOperators to disable this interface for individual 
*  fields, or set DataSourceField.canFilter to false to disallow filtering entirely for 
*  a field.
*  
*  Note that this feature is similar to ListGrid.allowFilterExpressions, which allows 
*  the end users to directly type in characters such as "&gt;" to control filtering.
*  allowFilterOperators is easier to use and more discoverable than
*  allowFilterExpressions, and also avoids the drawback where special characters 
*  like "&gt;" cannot be used in filter values. However, allowFilterExpressions 
*  allows users to make use of certain operators that allowFilterOperators does 
*  not support, such as using the "betweenInclusive" operator by typing "5...10".
*  
*  When both allowfilterExpressions and allowFilterOperators are set,
*  filter expressions entered in to the edit-area are parsed and the operator automatically 
*  applied to the ListGrid.operatorIcon.
*  
*  If ListGrid.allowFilterWindow is enabled another option,
*  ListGrid.advancedFilteringText, is added to the "Filter using" menu.
* @type {boolean}
* @default null
*/
allowFilterOperators?: boolean;

/**
* When ListGrid.allowFilterOperators is enabled, whether to show the ListGrid.operatorIcon for all
*  filterable fields, or only for fields where the user has explicitly chosen a search operator
*  different from the default operator for the field.
*  
*  The default operator for a field is determined by ListGrid.autoFetchTextMatchStyle or by
*  setting ListGridField.filterOperator for a specific field.
* @type {boolean}
* @default null
*/
alwaysShowOperatorIcon?: boolean;

/**
* Inline icon shown inside ListGrid.showFilterEditor fields when
*  ListGrid.allowFilterOperators is enabled.
* @type {FormItemIcon}
* @default null
*/
operatorIcon?: FormItemIcon;

/**
* Text for the menu item shown in the ListGrid.showHeaderContextMenu when
*  ListGrid.allowFilterOperators is enabled.
* @type {string}
* @default "Filter using"
*/
filterUsingText?: string;

/**
* Text to show after the name of the default filterOperator in the 
*  ListGrid.showHeaderContextMenu when ListGrid.allowFilterOperators is enabled.
* @type {string}
* @default "(default)"
*/
defaultFilterOperatorSuffix?: string;

/**
* Can the user edit cells in this listGrid? Can be set for the listGrid, and overridden for
*    individual fields.
*    If 'canEdit' is false at the listGrid level, fields can never be edited - in this case
*    the canEdit property on individual fields will be ignored.
*    If 'canEdit' is set to true at the listGrid level, setting the 'canEdit' property to
*    false at the field level will prevent the field from being edited inline unless a
*    custom override of ListGrid.canEditCell allows it.
*    If 'canEdit' is not set at the listGrid level, setting 'canEdit' to true at the field
*    level enables the field to be edited inline.
*    
*    For more information on editing, see the editing.
* @type {boolean}
* @default null
*/
canEdit?: boolean;

/**
* Property name on a record that should be checked to determine whether the record may be
*  edited.
*  
*  This property is configurable to avoid possible collision with data values in record.
*  With the default setting of "_canEdit", a record can be set non-editable by ensuring
*  record._canEdit == false.
*  
*  For controlling editability for the entire grid or for a field, set grid.canEdit or
*  field.canEdit.
* @type {string}
* @default "_canEdit"
*/
recordEditProperty?: string;

/**
* When this attribute is set, editors will be appear to be present in every row of the 
*  grid, allowing the user to immediately start editing any cell, rather than showing 
*  up in a single record at a time.
*  This attribute is only valid when ListGrid.editByCell is false.
*  
*  This setting has some limitations and is typically only used for simple grids with 
*  a limited set of fields and standard editors.
*  
*  
* - Not all formItem types are supported. Default editors for standard data types 
*  (text, boolean, date, datetime, integer and float) are all supported, but custom
*   editorType, including CanvasItem based editors are not. Fields with an unsupported
*   editor type will show static values for all rows other than the current edit row, though
*   users can start editing these with a single click 
*  
* - alwaysShowEditors:true grids do not support showing different
*   editor types for the same field in different rows
*  
* - In some cases there may be visual differences between the editor displayed in the
*   edit row and the editor displayed in other rows.
*  
* - From a design perspective, this mode presents a very "busy-looking" UI,
*    which can made it harder to read the actual data. Functionally having 
*    ListGrid.editEvent set to "click" provides the same single-click to edit
*    any cell user experience without the busy UI.
*  
* - In some cases there may be a performance penalty for writing out so many controls
*    (editors for every cell of the grid).
*  
*  Note that in addition to alwaysShowEditors, listGrid support single-click editing
*  via ListGrid.editEvent, and, for boolean fields, 
*  ListGridField.canToggle
* @type {boolean}
* @default null
*/
alwaysShowEditors?: boolean;

/**
* Determines whether when the user edits a cell in this listGrid the entire row becomes
*  editable, or just the cell that received the edit event.
*  
*  No effect if this.canEdit is false or null.
* @type {boolean}
* @default null
*/
editByCell?: boolean;

/**
* Whether edits should be saved whenever the user moves between cells in the current edit
*  row.
*  
*  If unset, defaults to ListGrid.editByCell.
*  
*  To avoid automatic saving entirely, set ListGrid.autoSaveEdits:false.
* @type {boolean}
* @default null
*/
saveByCell?: boolean;

/**
* Whether client-side validation checks should be performed when the user moves between
*  cells in the current edit row. If unset, defaults to ListGrid.editByCell.
*  
*  Note that validation always occurs when a row is to be saved, so setting
*  ListGrid.saveByCell:true forces validation on cell transitions. To completely disable
*  automatic validation, set ListGrid.neverValidate:true.
* @type {boolean}
* @default null
*/
validateByCell?: boolean;

/**
* If true, validation will be performed on each edited cell when each editor's
*  "change" handler is fired.
* @type {boolean}
* @default null
*/
validateOnChange?: boolean;

/**
* If true, validation will not occur as a result of cell editing for this grid.
* @type {boolean}
* @default null
*/
neverValidate?: boolean;

/**
* If set, provide UI for the user to remove records from the grid as an additional field
*  showing the ListGrid.removeIcon, which, when clicked, will call
*  ListGrid.removeRecordClick which removes the row from the data set (or if
*  ListGrid.deferRemoval is true changes the ListGrid.markRecordRemoved status
*  for the record). Individual records can be marked to prevent removal - see
*  ListGrid.recordCanRemoveProperty.
*  
*  To add a confirmation dialog before a record is removed, set
*  ListGrid.warnOnRemoval.
*  
*  If deferring removal, the record will appear marked with the ListGrid.removedCSSText
*  until the removal is committed via a call to ListGrid.saveEdits.
*  Otherwise, the record will disappear from view. If ListGrid.animateRemoveRecord
*  is true, the removed record will appear to shrink out of view when it is removed.
*  
*  By default the field will display the ListGrid.removeIcon next to each record, and
*  will be rendered as the rightmost column. Two mechanisms exist to further modify this field:
*  
*  
* - To change the position of the remove-field, include an explicitly specified field with
*    the attribute ListGridField.isRemoveField set. This will then
*    be used as the remove field instead of adding a field to the beginning of the set of
*    columns.
*  
* - Additional direct configuration of the remove field may be achieved by modifying
*    ListGrid.removeFieldProperties.
*  
*  If ListGrid.deferRemoval is true, when a record is marked as removed, the the icon will
*  change to display the ListGrid.unremoveIcon for this row. Clicking on this icon
*  will call ListGrid.unmarkRecordRemoved to mark the record as no longer pending
*  deletion.
* @type {boolean}
* @default false
*/
canRemoveRecords?: boolean;

/**
* If ListGrid.canRemoveRecords is true, when the user clicks the remove icon
*  for some record, should we show a warning message
*  (defined as ListGrid.warnOnRemovalMessage) and allow the user to cancel removal?
* @type {boolean}
* @default false
*/
warnOnRemoval?: boolean;

/**
* Warning message to show the user on a click on the 'remove' icon
*  if ListGrid.canRemoveRecords is true and
*  ListGrid.warnOnRemoval is true.
* @type {string}
* @default "Are you sure you want to delete this record?"
*/
warnOnRemovalMessage?: string;

/**
* Custom CSS text to be applied to records that have been
*  ListGrid.markRecordRemoved.
*  
*  This CSS text will be applied on top of standard disabled styling for the cell.
* @type {string}
* @default "text-decoration:line-through;"
*/
removedCSSText?: string;

/**
* When enabled, the field shown by ListGrid.canRemoveRecords causes records to be
*  marked for future removal via ListGrid.markRecordRemoved instead of immediately being
*  removed.
*  
*  When a record has been marked for removal, an icon in the
*  canRemoveRecords field allowing it to be unmarked will be displayed.
*  
*  If not explicitly specified by this property, removal of records will be deferred if
*  ListGrid.autoSaveEdits is false for the grid.
* @type {boolean}
* @default null
*/
deferRemoval?: boolean;

/**
* When ListGrid.canRemoveRecords is enabled, default icon to show in
*  the auto-generated field that allows removing records.
* @type {SCImgURL}
* @default "[SKIN]/actions/remove.png"
*/
removeIcon?: SCImgURL;

/**
* When ListGrid.canRemoveRecords is enabled, this icon will be shown in the
*  auto generated field fro removing records if the record has been marked as removed via
*  ListGrid.markRecordRemoved. At this point, clicking on the icon will
*  unmark the record as removed.
* @type {SCImgURL}
* @default "[SKIN]/actions/undo.png"
*/
unremoveIcon?: SCImgURL;

/**
* Default width and height of ListGrid.removeIcon for this ListGrid.
* @type {number}
* @default "16"
*/
removeIconSize?: number;

/**
* When ListGrid.canRemoveRecords is enabled, should records be animated out of view
*  when they are removed by the user?
* @type {boolean}
* @default true
*/
animateRemoveRecord?: boolean;

/**
* When animating record removal
*  ListGrid.animateRemoveRecord, if
*  ListGrid.animateRemoveSpeed is not
*  set, this property designates the duration of the animation in ms.
* @type {number}
* @default "100"
*/
animateRemoveTime?: number;

/**
* When ListGrid.animateRemoveRecord, this property
*  designates the speed of the animation in pixels per second. Takes precedence over the
*  ListGrid.animateRemoveTime property, which allows the developer to specify a
*  duration for the animation rather than a speed.
* @type {number}
* @default "200"
*/
animateRemoveSpeed?: number;

/**
* The title to use for the ListGrid.removeFieldDefaults.
*  
*  By default this title is not displayed in the remove column header button as the
*  ListGrid.removeFieldDefaults sets ListGridField.showTitle to false.
* @type {string}
* @default "&amp;nbsp;"
*/
removeFieldTitle?: string;

/**
* Default configuration properties for the "remove field"
*  displayed when ListGrid.canRemoveRecords is enabled.
*  Class.changeDefaults should be used when modifying this object.
*  
*  The default configuration includes a ListGridField.recordClick handler which
*  calls ListGrid.removeData to actually perform the data removal.
* @type {Partial<ListGridField>}
* @default "{...}"
*/
removeFieldDefaults?: Partial<ListGridField>;

/**
* Configuration properties for the "remove field" displayed when
*  ListGrid.canRemoveRecords is enabled.
*  These configuration settings will be overlaid
*  on top of the ListGrid.removeFieldDefaults.
* @type {Partial<ListGridField>}
* @default null
*/
removeFieldProperties?: Partial<ListGridField>;

/**
* If we're showing the ListGrid.showFilterEditor, should this list
*  be filtered every time the user changes edit values for particular cells rather than
*  waiting for an Enter keypress or a click on the filterEditor submit button.
*  
*  Note that by default fields in the filter editor will be set to 
*  FormItem.changeOnKeypress, so the grid will not filter as
*  the user types in text-based items.
*  To enable filtering as the user types in text fields, we recommend the 
*  ListGrid.filterOnKeypress attribute. Also note that filterOnKeypress:true
*  implies filtering will occur on change to edit values for cells, even if filterByCell
*  is not set to true.
* @type {boolean}
* @default true
*/
filterByCell?: boolean;

/**
* If we're showing the ListGrid.showFilterEditor, should this list
*  be filtered for every keypress within the filter editor fields?
*  
*  This is equivalent to setting ListGrid.filterByCell to true and explicitly setting
*  FormItem.changeOnKeypress to true for each text-based field within the
*  ListGridField.filterEditorProperties.
*  
* @type {boolean}
* @default false
*/
filterOnKeypress?: boolean;

/**
* If this is an editable listGrid, this property determines whether the user will be
*  able to dismiss the edit form, or navigate to another cell while the save is in
*  process (before the asynchronous server response returns).
* @type {boolean}
* @default false
*/
waitForSave?: boolean;

/**
* If this is an editable listGrid, this property determines how failure to save due to
*  validation errors should be displayed to the user.
*  
*  If this property is true, when validation errors occur the errors will be displayed
*  to the user in an alert, and focus will be returned to the first cell to fail validation.
*  
*  If false, the cells that failed validation will be silently styled with the
*  editFailedBaseStyle.
*  
*  Note: stopOnErrors being set to true implies that 'waitForSave' is also true.
*  We will not dismiss the editor until save has completed if stopOnErrors is true.
* @type {boolean}
* @default false
*/
stopOnErrors?: boolean;

/**
* If this ListGrid is editable, should edits be saved out when the user finishes editing
*  a row (or a cell if ListGrid.saveByCell is true).
*  
*  The default of true indicates that edits will be
*  ListGrid.saveByCell as the
*  user navigates through the grid and/or ${isc.DocUtils.linkForRef('type:EnterKeyEditAction','hits \'Enter\'')} to end
*  editing. See the editing overview for details.
*  
*  Setting autoSaveEdits false creates a "mass update" / "mass delete"
*  interaction where edits will be retained for all edited cells (across rows if
*  appropriate) until ListGrid.saveEdits is called to save a particular row, or
*  ListGrid.saveAllEdits is called to save all changes in a batch.
*  
*  Note: when ListGrid.groupByField is enabled, or when
*  working with hierarchical data in a TreeGrid, users have the option to hide
*  records from view by collapsing the parent folder or group. This, in conjunction with
*  autoSaveEdits being set to false can lead to a case where
*  a user is unable to save edits due to validation errors on hidden rows. Therefore we
*  recommend developers consider having validators in place such that errors are caught
*  and displayed to the user on change or editor exit rather than being caught only when
*  saving is attempted. If it's not possible for all validation to be performed immediately 
*  on row exit, we recommend that a different UI design be used that does not involve
*  autoSaveEdits being set to false.
* @type {boolean}
* @default true
*/
autoSaveEdits?: boolean;

/**
* If this grid is editable, and an edit has caused validation failure for some cell,
*  should we show an icon to indicate validation failure?
* @type {boolean}
* @default true
*/
showErrorIcons?: boolean;

/**
* Height of the error icon, if we're showing icons when validation errors occur.
* @type {Integer}
* @default "16"
*/
errorIconHeight?: Integer;

/**
* Height of the error icon, if we're showing icons when validation errors occur.
* @type {Integer}
* @default "16"
*/
errorIconWidth?: Integer;

/**
* Src of the image to show as an error icon, if we're showing icons when validation
*    errors occur.
* @type {SCImgURL}
* @default "[SKIN]/ListGrid/validation_error_icon.png"
*/
errorIconSrc?: SCImgURL;

/**
* If this is an editable listGrid, when the user attempts to cancel an edit, should we
*  display a confirmation prompt before discarding the edited values for the record?
* @type {boolean}
* @default false
*/
confirmCancelEditing?: boolean;

/**
* If this is an editable listGrid, and this.confirmCancelEditing is true
*  this property is used as the message to display in the confirmation dismissal prompt.
* @type {string}
* @default "Cancelling this edit will discard unsaved changes for this record. Continue?"
*/
cancelEditingConfirmationMessage?: string;

/**
* For editable listGrids, outstanding unsaved edits when the user performs a new filter
*  or sort will be discarded. This flag determines whether we should display a confirmation
*  dialog with options to save or discard the edits, or cancel the action in this case.
* @type {boolean}
* @default true
*/
confirmDiscardEdits?: boolean;

/**
* For editable listGrids, outstanding unsaved edits when the user performs a new filter
*  or sort will be discarded by default. This flag determines whether we should save such
*  edits automatically in this case. See also ListGrid.confirmDiscardEdits, which 
*  allows the user to choose whether to save or discard the unsaved edits.
* @type {boolean}
* @default false
*/
autoConfirmSaveEdits?: boolean;

/**
* If this.confirmDiscardEdits is true, this property can be used to customize the
*  error message string displayed to the user in a dialog with options to
*  cancel the action, or save or discard pending edits in response to sort/filter actions
*  that would otherwise drop unsaved edit values.
* @type {string}
* @default "This action will discard unsaved changes for this list."
*/
confirmDiscardEditsMessage?: string;

/**
* If ListGrid.confirmDiscardEdits is true this is the title for the save button
*  appearing in the lost edits confirmation dialog. Override this for localization if necessary.
* @type {string}
* @default "Save"
*/
discardEditsSaveButtonTitle?: string;

/**
* If the user is editing a record in this listGrid, and attempts to navigate to a field
*  beyond the end of the row, via tab (or shift-tab off the first editable field), this
*  property determines what action to take:
*  
* - "next": start editing the next (or previous) record in the list
*  
* - "same": put focus back into the first editable field of the same record.
*  
* - "done": hide the editor
*  
* - "stop": leave focus in the cell being edited
*  
* - "none": no action
*  
* @type {RowEndEditAction}
* @default null
*/
rowEndEditAction?: RowEndEditAction;

/**
* If the user is editing the last record in this listGrid, and attempts to navigate
*  beyond the last row either by tabbing off the last editable field, or using the down
*  arrow key, this property determines what action to take:
*  
* - "next": start editing a new record at the end of the list.
*  
* - "done": hide the editor
*  
* - "stop": leave focus in the cell being edited
*  
* - "none": no action
*  
*  
*  See the editing and also the
*  unsavedRecords for context about how newly
*  added records behave.
* @type {RowEndEditAction}
* @default null
*/
listEndEditAction?: RowEndEditAction;

/**
* If this is an editable ListGrid, setting this property to true causes an extra 
*  row with the ListGrid.newRecordRowMessage to be displayed below the last record.
*  
*  Clicking this row will start editing a new record at the end of the data set, as if
*  ListGrid.startEditingNew had been called.
*  
*  Note that for ListGrid.dataSource, the new record row will
*  be suppressed if the grid has not ListGrid.fetchData, unless
*  ListGrid.saveLocally has been set.
* @type {boolean}
* @default null
*/
showNewRecordRow?: boolean;

/**
* If this listGrid is showing the 'newRecordRow' (used for adding new rows to the end
*  of the data), this property determines what message should be displayed in this row.
* @type {string}
* @default "-- Add New Row --"
*/
newRecordRowMessage?: string;

/**
* What to do when a user hits enter while editing a cell:
*  
*  
* - "nextCell": start editing the next editable cell in this record (or the first
*    editable cell in the next record if focus is in the last editable cell in the row)
*  
* - "nextRow": start editing the same field in the next row (skipping any rows where
*    that would be a non-editable cell.
*  
* - "nextRowStart": start editing the first editable cell in the next row.
*  
* - "done": hide the editor (editing is complete)
*  
*  Note that if this.autoSaveEdits is true, this may cause a save of the current edit values
* @type {EnterKeyEditAction}
* @default "done"
*/
enterKeyEditAction?: EnterKeyEditAction;

/**
* What to do when a user hits escape while editing a cell:
*  
* - "cancel": close the editor and discard the current set of edit values
*  
* - "done": just close the editor (the edit is complete, but the edited values are retained).
*  
*  Note that if ListGrid.autoSaveEdits is true, this may cause a save of the current edit values
* @type {EscapeKeyEditAction}
* @default "cancel"
*/
escapeKeyEditAction?: EscapeKeyEditAction;

/**
* What to do when a user hits arrow key while editing a field?
*  If not explicitly specified ListGrid.getArrowKeyEditAction 
*  will return an appropriate action depending on the field type.
* @type {ArrowKeyEditAction}
* @default null
*/
arrowKeyEditAction?: ArrowKeyEditAction;

/**
* Event that will trigger inline editing, see ListGridEditEvent for options.
*  
*  Note this setting has no effect unless ListGrid.canEdit has been set to enable editing.
*  
*  See also ListGrid.editOnFocus and ListGrid.startEditing.
* @type {ListGridEditEvent}
* @default "doubleClick"
*/
editEvent?: ListGridEditEvent;

/**
* Should we start editing when this widget receives focus (if this ListGrid supports
*  editing)?
*  
*  Note that this property being set to true will cause editing to occur on a single
*  click, even if ListGrid.editEvent is "doubleClick", because single clicking
*  the grid will place keyboard focus there automatically.
*  
*  If this property is set together with ListGrid.listEndEditAction being set to "next", 
*  users can create a new edit row in an empty grid by simply tabbing into the grid.
* @type {boolean}
* @default null
*/
editOnFocus?: boolean;

/**
* Should we start editing when the widget has focus and the user presses the "f2" key
*  (if this ListGrid supports editing)?
*  
*  Note that if ListGrid.editEvent is set to "click" or
*  "doubleClick", the Space or Enter key may
*  also be used to start editing, depending on the value for ListGrid.generateClickOnSpace,
*  ListGrid.generateDoubleClickOnSpace, ListGrid.generateClickOnEnter and
*  ListGrid.generateDoubleClickOnEnter.
*  
*  If ListGrid.canEdit is false, or ListGrid.editEvent is set to "none" this
*  property has no effect.
* @type {boolean}
* @default true
*/
editOnF2Keypress?: boolean;

/**
* When the user starts editing a row, should the row also be selected?
*  
*  See ListGrid.editSelectionType for how edit-selection behaves.
* @type {boolean}
* @default true
*/
selectOnEdit?: boolean;

/**
* If ListGrid.selectOnEdit is true, what should be the edit-selection behavior
*  be?
*  
*  Default setting of "single" will cause the edit row to be automatically
*  selected and any other selection in the grid to be dropped.
*  If set to "multiple", selection will be additive (as a record goes 
*  into edit mode, it is selected in addition to any pre-existant selection).
*  
*  If set to null behavior is as follows:
*  
* - For grids with ListGrid.selectionType set to
*  "simple" edit rows will be selected additively - this is the same
*  behavior as if the editSelectionType was "multiple"
*  
* - Otherwise edit rows will be selected singly - this is the same behavior as
*  if the editSelectionType was "single"
*  
* @type {SelectionStyle}
* @default "single"
*/
editSelectionType?: SelectionStyle;

/**
* In a ListGrid that has a DataSource and has filter criteria that include values for
*  fields declared as FieldType in the DataSource, by default a newly
*  edited row will use those filter criteria as initial values.
*  
*  For example, if a ListGrid is showing all Accounts that have status:"Active" and a new row
*  is created, the new row will default to status:"Active" unless this flag is set to false.
* @type {boolean}
* @default true
*/
enumCriteriaAsInitialValues?: boolean;

/**
* Whether to do inline autoComplete in text fields during inline editing
*  Overridden by ListGridField.autoComplete if specified.
*  If unset picks up the default from the appropriate editor class (subclass of FormItem).
* @type {AutoComplete}
* @default null
*/
autoComplete?: AutoComplete;

/**
* When the length of the field specified by DataSourceField.length exceeds this
*  value, the ListGrid shows an edit field of type ListGrid.longTextEditorType
*  rather than the standard text field when the field enters inline edit mode.
* @type {Integer}
* @default "255"
*/
longTextEditorThreshold?: Integer;

/**
* When the length of the field specified by DataSourceField.length exceeds
*  this.longTextEditorThreshold show an edit field of this type
*  rather than the standard text field when the field enters inline edit mode.
* @type {string}
* @default "PopUpTextAreaItem"
*/
longTextEditorType?: string;

/**
* A Toolbar used to manager the headers shown for each column of the grid.
* @type {Layout}
* @default null
*/
header?: Layout;

/**
* The context menu displayed for column headers.
* @type {Canvas}
* @default null
*/
headerContextMenu?: Canvas;

/**
* The menu displayed when a cell is right clicked on.
* @type {Layout}
* @default null
*/
cellContextMenu?: Layout;

/**
* The menu displayed when a cell is right clicked on.
* @type {Layout}
* @default null
*/
spanContextMenu?: Layout;

/**
* Should the header be included in the tab-order for the page? If not explicitly specified,
*  the header will be included in the tab order for the page if 
*  isc.setScreenReaderMode 
*  
*  is called.
* @type {boolean}
* @default null
*/
canTabToHeader?: boolean;

/**
* Should the ListGrid.sorterConstructor be included in the 
*  tab-order for the page?
* @type {boolean}
* @default false
*/
canTabToSorter?: boolean;

/**
* The height of this listGrid's header, in pixels.
* @type {number}
* @default "22"
*/
headerHeight?: number;

/**
* Minimum size, in pixels, for ListGrid headers.
* @type {Integer}
* @default "15"
*/
minFieldWidth?: Integer;

/**
* Should we show the header for this ListGrid?
* @type {boolean}
* @default true
*/
showHeader?: boolean;

/**
* Set the CSS style used for the header as a whole.
* @type {CSSStyleName}
* @default null
*/
headerBarStyle?: CSSStyleName;

/**
* BackgroundColor for the header toolbar. Typically this is set to match the color
*  of the header buttons.
* @type {CSSColor}
* @default "#CCCCCC"
*/
headerBackgroundColor?: CSSColor;

/**
* Should the header show a drop-shadow?
*  Shadow will be applied to the header, or for a grid with frozen columns, the
*  header layout. 
*  
*  Header shadow will only be displayed if Canvas.useCSSShadow are
*  being used.
* @type {boolean}
* @default false
*/
showHeaderShadow?: boolean;

/**
* If ListGrid.showHeaderShadow is true, the Canvas.shadowVOffset for
*  the header shadow
* @type {number}
* @default "1"
*/
headerShadowVOffset?: number;

/**
* If ListGrid.showHeaderShadow is true, the Canvas.shadowHOffset for
*  the header shadow
* @type {number}
* @default "0"
*/
headerShadowHOffset?: number;

/**
* If ListGrid.showHeaderShadow is true, the Canvas.shadowSoftness for
*  the header shadow
* @type {number}
* @default "1"
*/
headerShadowSoftness?: number;

/**
* If ListGrid.showHeaderShadow is true, the Canvas.shadowColor for
*  the header shadow.
* @type {CSSColor}
* @default null
*/
headerShadowColor?: CSSColor;

/**
* Widget class for this ListGrid's header buttons. If unset, constructor will be
*  picked up directly from the standard Toolbar button constructor.
* @type {Class}
* @default null
*/
headerButtonConstructor?: Class;

/**
* Button.baseStyle to apply to the buttons in the header, and the sorter, for
*  this ListGrid.
*  Note that, depending on the ListGrid.headerButtonConstructor of the header
*  buttons, you may also need to set ListGrid.headerTitleStyle.
* @type {CSSStyleName}
* @default null
*/
headerBaseStyle?: CSSStyleName;

/**
* StretchImgButton.titleStyle to apply to the buttons in the header, and the sorter,
*  for this ListGrid.
*  Note that this will typically only have an effect if
*  ListGrid.headerButtonConstructor is set to StretchImgButton or a subclass
*  thereof.
* @type {CSSStyleName}
* @default null
*/
headerTitleStyle?: CSSStyleName;

/**
* If this listGrid contains any frozen fields, this property can be used to apply a custom
*  headerBaseStyle to the frozen set of fields. If unset, the standard headerBaseStyle
*  will be used for both frozen and unfrozen cells.
* @type {CSSStyleName}
* @default null
*/
frozenHeaderBaseStyle?: CSSStyleName;

/**
* If this listGrid contains any frozen fields, this property can be used to apply a custom
*  headerTitleStyle to the frozen set of fields. If unset, the standard headerTitleStyle
*  will be used for both frozen and unfrozen cells.
* @type {CSSStyleName}
* @default null
*/
frozenHeaderTitleStyle?: CSSStyleName;

/**
* Defaults to apply to all header buttons. To modify this object,
*  use Class.changeDefaults
*  rather than replacing with an entirely new object.
* @type {Partial<Button>}
* @default "{...}"
*/
headerButtonDefaults?: Partial<Button>;

/**
* Properties to apply to all header buttons.
*  Overrides defaults applied via ListGrid.headerButtonDefaults.
* @type {Partial<Button>}
* @default null
*/
headerButtonProperties?: Partial<Button>;

/**
* Whether the ListGrid should manage the clipping of titles of header buttons, showing
*  ellipses if the title is clipped, and potentially showing the full title
*  on ListGrid.showClippedHeaderTitlesOnHover.
*  
*  In some cases this may be preferable to the button component's default clipping behavior
*  because if a ListGrid.showSortArrow or sort numeral is displayed for
*  a header, then the button's default clipping behavior may clip the sort arrow/numeral
*  whereas ListGrid-managed title clipping utilizes special HTML which keeps the sort
*  arrow/numeral visible.
*  
*  This feature is automatically enabled if supported by the browser. The only supported
*  use of this attribute is to disable the feature by setting clipHeaderTitles
*  to false.
*  
*  Note that this feature is incompatible with ListGridField.wrap, and will
*  automatically be disabled for wrapping fields.
* @type {boolean}
* @default "varies"
*/
clipHeaderTitles?: boolean;

/**
* If ListGridField.wrap is not explicitly set, should fields wrap? If autofitting,
*  see the docs on that property for the details of how the minimum width for a field is
*  determined.
* @type {boolean}
* @default null
*/
wrapHeaderTitles?: boolean;

/**
* Whether to rotate the field titles so they're rendered vertically from bottom to top.
*  Can be overridden for individual fields by setting ListGridField.rotateTitle.
*  
*  Note that you can manually set the header height and field widths as you please when
*  using this feature, but it's not compatible with ListGrid.autoFitHeaderHeights or
*  autofitting of field widths in any AutoFitWidthApproach other than "value".
*  
*  You can use ListGrid.headerTitleVAlign or ListGridField.valign to control vertical
*  positioning of the titles, and ListGridField.align to control the horizontal.
*  You may also choose between ListGrid.clipHeaderTitles or 
*  ListGrid.wrapHeaderTitles, and set ListGrid.showHeaderMenuButton as you please
*  (which reserves space in each header button for the header menu button).
*  
*  Note that this feature is incompatible with clipping via ListGrid.clipHeaderTitles:false,
*  and may not work with older browsers, particular IE versions before IE10. The
*  "TreeFrog" and "Basic" skins are not supported for this feature.
* @type {boolean}
* @default null
*/
rotateHeaderTitles?: boolean;

/**
* Specifies vertical alignment in the column headers: "top", "center", or "bottom". Can 
*  be overridden for individual fields by setting ListGridField.valign.
*  
*  When using ListGrid.rotateHeaderTitles, this attribute defaults to "bottom"
*  if it remains unset.
* @type {VerticalAlignment}
* @default null
*/
headerTitleVAlign?: VerticalAlignment;

/**
* If HeaderSpan.wrap is not explicitly set, should fields wrap? If autofitting,
*  see the docs on that property for the details of how the minimum width for a field is
*  determined.
* @type {boolean}
* @default null
*/
wrapHeaderSpanTitles?: boolean;

/**
* Widget class for the corner sort button, if showing. This button displays the current
*  sort direction of the primary sort field (either the only sorted field or the first in a
*  ListGrid.canMultiSort grid) and reverses the direction of that field
*  when clicked. For consistent appearance, this
*  is usually set to match ListGrid.headerButtonConstructor
* @type {Class}
* @default "Button"
*/
sorterConstructor?: Class;

/**
* The title for the corner sort button. The title will only
*  Class.changeDefaults rather than replacing with an
*  entirely new object.
* @type {string}
* @default "corner menu"
*/
sorterButtonTitle?: string;

/**
* Defaults to apply to the corner sort button. To modify this object, use
*  Class.changeDefaults rather than replacing with an
*  entirely new object.
* @type {object}
* @default "{...}"
*/
sorterDefaults?: object;

/**
* Properties to apply to the sorter button. Overrides defaults applied via
*  ListGrid.sorterDefaults.
* @type {Partial<Button>}
* @default null
*/
sorterProperties?: Partial<Button>;

/**
* If set, whenever grouping is performed by an end user or by a programmatic call to 
*  ListGrid.groupBy, data is implicitly sorted by all of the grouped columns, in the order 
*  they were passed to groupBy. Any user-configured sorting is applied after sorting by 
*  grouped columns.
*  
*  Sorting by grouped fields will be in ascending or descending order according to whether 
*  the grid is currently sorted (by any field) in ascending or descending order, defaulting 
*  to ascending if the grid is not sorted. Implicit sorting by group can be forced to be 
*  always ascending or always descending by setting ListGrid.groupSortDirection.
*  
*  The sorting is "implicit" in the sense that the sorting is not shown in the ListGrid 
*  headers, and not shown in the MultiSortDialog if enabled. An end user cannot 
*  currently remove the implicit sorting themselves (except by removing the grouping), 
*  though it is possible to override it by providing an explicit sort on the group's column.
*  Clicking on the grouped field's header reveals the usual sort indicators with all
*  the same semantics.
*  
*  The correct way to remove implicit sorting programmatically is to call 
*  ListGrid.setSortByGroupFirst.
*  
*  Programmatic calls to ListGrid.getSort will not include the implicit sort in 
*  the list of return sort specifiers, and calls to ListGrid.setSort will implicitly add 
*  the sorting by grouped columns before the specified sort.
*  
*  Note that directly calling ResultSet.getSort() will include the implicit sort 
*  information.
* @type {boolean}
* @default null
*/
sortByGroupFirst?: boolean;

/**
* When ListGrid.sortByGroupFirst is active, the sorting direction applied for implicit 
*  sorting by the field(s) used for grouping. Default of null means that sort direction 
*  is based on the current direction of user-configured sort, or is "ascending" if the 
*  user has not sorted the data.
* @type {SortDirection}
* @default null
*/
groupSortDirection?: SortDirection;

/**
* For any fields of FieldType, should sorting be performed
*  against the fileName of the value for the field? For SmartClient server backed
*  dataSources, this is applied to the record automatically as described in the
*  binaryFields overview.
*  
*  If set to false, binary fields will be sorted against the record value for the
*  field in question. Client-side sorting does not support this, so developers who
*  actually want to support a sort against the binary itself would typically set
*  ResultSet.useClientSorting to false on the ListGrid.dataProperties block for
*  this grid.
*  
*  Note that this setting will have no effect if DataSourceField.sortByField is
*  specified
* @type {boolean}
* @default true
*/
sortBinaryByFileName?: boolean;

/**
* Enables or disables interactive sorting behavior for this listGrid. Does not
*  affect sorting by direct calls to the ListGrid.sort or
*  ListGrid.setSort methods.
* @type {boolean}
* @default true
*/
canSort?: boolean;

/**
* If true, show the field-header for the sorted field (or the first field in a
*  ListGrid.canMultiSort grid) in the selected state.
* @type {boolean}
* @default true
*/
selectHeaderOnSort?: boolean;

/**
* Specifies the number of the field by which to sort this listGrid. Column numbers
*  start at 0 for the left-most column.
* @deprecated * as of version 7.0 in favor of ListGrid.sortField

* @type {number}
* @default null
*/
sortFieldNum?: number;

/**
* Specifies the field by which this grid should be initially sorted. Can be set to either a
*  ListGridField.name or the index of the field in the fields Array.
*  
*  ListGrids also support being initialized with multiple-field sort via
*  ListGrid.initialSort. If initialSort is specified, it will be used in preference
*  to this property.
*  
*  To sort the grid after it has been initialized, use ListGrid.sort or
*  ListGrid.setSort. Details about the current sort of a live grid can be
*  retrieved by calling ListGrid.getSortField or ListGrid.getSort
* @type {string | Integer}
* @default null
*/
sortField?: string | Integer;

/**
* Sorting direction of this ListGrid. If specified when the ListGrid is initialized,
*  this property will be the default sorting direction for the ListGrid.sortField.
*  May be overridden by specifying ListGridField.sortDirection.
*  
*  After initialization, this property will be updated on ListGrid.sort or
*  ListGrid.setSort to reflect the current sort direction of the grid. When
*  this grid is sorted by multiple fields, the grid's sortDirection reflects the
*  sort direction of the primary sort field.
* @type {SortDirection}
* @default "ascending"
*/
sortDirection?: SortDirection;

/**
* Indicates whether a sorting arrow should appear for the listGrid, and its
*  location. See SortArrow for details.
*  
*  Clicking the sort arrow reverses the direction of sorting for the current sort
*  column (if any), or sorts the listGrid by its first sortable column. The arrow
*  image on the button indicates the current direction of sorting.
*  If undefined, the sort arrow will show up in the sorted field, and the
*  corner sort button will be displayed if a vertical scrollbar is being displayed
* @type {SortArrow}
* @default null
*/
showSortArrow?: SortArrow;

/**
* When ListGrid.leaveHeaderMenuButtonSpace is true, configures the amount of space beyond the
*  ListGrid.headerMenuButtonWidth on the right side of a ListGrid header button (left for
*  Page.isRTL) to reserve for the sort arrow if sorting is active for
*  that field and the arrow will be shown. May be increased for more separation between
*  the sort arrow and the title text, at the expense of a reduced space for the title text.
*  
*  This value may need to be customized in your skin or if ListGrid.sortAscendingImage or 
*  ListGrid.sortDescendingImage are changed.
* @type {Integer}
* @default "7"
*/
sortArrowMenuButtonSpaceOffset?: Integer;

/**
* When ListGrid.leaveHeaderMenuButtonSpace is true, configures the amount of space beyond the
*  ListGrid.headerMenuButtonWidth on the right side of a ListGrid header button (left for
*  Page.isRTL) to reserve for the sort numeral if 
*  ListGrid.canMultiSort is active for that field and the numeral will be shown.
*  May be increased for more separation between the title text and the sort arrow when 
*  multi-sorting. 
*  
*  Note that larger values may required if 10 or more fields are sorted at once, as the
*  numeral will occupy more space. This value may need to be customized in your skin or if
*  ListGrid.sortAscendingImage or ListGrid.sortDescendingImage are changed.
* @type {Integer}
* @default "9"
*/
sortNumeralMenuButtonSpaceOffset?: Integer;

/**
* Indicates whether the field picker item and submenu should be present in the header
*  context menu. This menu allows the user to hide visible fields and show hidden fields.
*  
*  By default only fields explicitly included in the ListGrid.fields array will
*  be available in this menu, unless ListGrid.canPickOmittedFields is set to true for
*  a databound grid.
*  
*  A specific field can be omitted from the column picker via ListGridField.canHide.
* @type {boolean}
* @default true
*/
canPickFields?: boolean;

/**
* If this grid has a specified ListGrid.dataSource, and
*  ListGrid.useAllDataSourceFields is false, setting this property to true
*  will cause all dataSource fields not included in the specified set of fields to
*  show up in the ListGrid.canPickFields.
*  
*  Has no effect if ListGrid.fields is unset (as in this case all dataSource fields
*  will be displayed by default), or if ListGrid.canPickFields is false.
* @type {boolean}
* @default false
*/
canPickOmittedFields?: boolean;

/**
* Whether an interface should be shown to allow user is allowed to dynamically "freeze" or
*  "unfreeze" columns with respect to horizontally scrolling. If unset, this property defaults
*  to true unless:
*  
* - ListGrid.fixedRecordHeights is false
*  
* - ListGrid.bodyOverflow is "visible"
*  
* - ListGrid.autoFitData is set to "horizontal" or
*  "both"
*  
* - Any field has overflow set to "visible"
*  
*  Note that the canFreezeFields setting enables or disables the user
*  interface for freezing and unfreezing fields only. Fields can be programmatically
*  frozen via setting ListGridField.frozen to true when the grid is
*  created, or dynamically frozen and unfrozen via ListGrid.freezeField and
*  ListGrid.unfreezeField.
*  
*  Developers should also be aware that if the cell content for some field exceeds
*  the specified ListGrid.cellHeight, and ListGrid.enforceVClipping is not set to true, 
*  this can cause misalignment between rows in frozen and unfrozen columns. See the
*  frozenFields for more on this.
* @type {boolean}
* @default null
*/
canFreezeFields?: boolean;

/**
* Maximum width available for any frozenFields shown 
*  in this grid. May be specified as a percentage or numeric pixel value.
*  
*  If the frozen fields' combined width exceeds this value, a
*  horizontal scrollbar will be shown, allowing the frozen fields to be horizontally
*  scrolled (independently from the unfrozen fields).
* @type {string | Integer}
* @default null
*/
frozenFieldsMaxWidth?: string | Integer;

/**
* If the result of a formula evaluation is invalid (specifically, if isNaN(result)==true),
*  badFormulaResultValue is displayed instead. The default value is ".".
* @type {string}
* @default "."
*/
badFormulaResultValue?: string;

/**
* If a summary format string contains an invalid field reference, replace the reference
*  with the missingSummaryFieldValue. The default value is "-".
* @type {string}
* @default "-"
*/
missingSummaryFieldValue?: string;

/**
* Adds an item to the header context menu allowing users to launch a dialog to define a new
*  field based on values present in other fields, using the FormulaBuilder.
*  
*  User-added formula fields can be persisted via ListGrid.getFieldState and 
*  ListGrid.setFieldState.
* @type {boolean}
* @default false
*/
canAddFormulaFields?: boolean;

/**
* Adds an item to the header context menu allowing users to launch a dialog to define a new
*  text field that can contain both user-defined text and the formatted values present in other 
*  fields, using the SummaryBuilder.
*  
*  User-added summary fields can be persisted via ListGrid.getFieldState and 
*  ListGrid.setFieldState.
* @type {boolean}
* @default false
*/
canAddSummaryFields?: boolean;

/**
* Adds an item to the header context menu allowing users to launch a dialog to define
*  grid hilites using the HiliteEditor.
*  
*  User-added hilites can be persisted via DataBoundComponent.getHiliteState and 
*  DataBoundComponent.setHiliteState.
* @type {boolean}
* @default false
*/
canEditHilites?: boolean;

/**
* Whether to show a context menu with standard items for all context clicks on rows in the
*  body.
* @type {boolean}
* @default false
*/
showCellContextMenus?: boolean;

/**
* Whether to show a context menu on the header with standard items for showing and hiding
*  fields. Not supported for CubeGrid.
* @type {boolean}
* @default true
*/
showHeaderContextMenu?: boolean;

/**
* Whether to show a context menu on the header span with standard items for showing and hiding
*  fields. Not supported for CubeGrid.
* @type {boolean}
* @default true
*/
showHeaderSpanContextMenu?: boolean;

/**
* If set to true and ListGrid.showHeaderContextMenu is true, the
*  ListGrid.headerMenuButton will be displayed when the user rolls
*  over the header buttons in this grid. Not supported for CubeGrid.
* @type {boolean}
* @default true
*/
showHeaderMenuButton?: boolean;

/**
* If ListGrid.showHeaderMenuButton is true, when auto-fitting fields to
*  the title width via ListGrid.autoFitFieldWidths or ListGridField.autoFitWidth,
*  should the button be sized such that there is enough space for the header menu button to
*  show without covering the field title?
*  
*  May be explicitly specified at the ListGridField.leaveHeaderMenuButtonSpace
*  or at the ListGrid.leaveHeaderMenuButtonSpace. If not explicitly
*  specified space will be left for fields with
*  ListGridField.align set to "left" or "right", but not for
*  fields with align set to "center".
* @type {boolean}
* @default null
*/
leaveHeaderMenuButtonSpace?: boolean;

/**
* Constructor for the ListGrid.headerMenuButton. If unset a standard "Button" will
*  be rendered out. Note that this property may be overridden by different skins.
* @type {string}
* @default null
*/
headerMenuButtonConstructor?: string;

/**
* If ListGrid.showHeaderMenuButton is true, when the user rolls over the header buttons in this
*  grid the headerMenuButton will be shown over the header button in question. When clicked
*  this button will display the standard header context menu (see
*  ListGrid.displayHeaderContextMenu).
*  
*  headerMenuButton exist to customize the appearance of the
*  headerMenuButton. Also see the AutoChild documentation for information on how
*  to make free-form modifications to autoChild widgets
* @type {StatefulCanvas}
* @default null
*/
headerMenuButton?: StatefulCanvas;

/**
* If ListGrid.showHeaderMenuButton is true, this property governs the icon shown on the
*  auto-generated headerMenuButton
* @type {URL}
* @default "[SKIN]/ListGrid/headerMenuButton_icon.gif"
*/
headerMenuButtonIcon?: URL;

/**
* If ListGrid.showHeaderMenuButton is true, this property governs the width of the icon
*  shown on the auto-generated headerMenuButton
* @type {number}
* @default "7"
*/
headerMenuButtonIconWidth?: number;

/**
* If ListGrid.showHeaderMenuButton is true, this property governs the height of the icon
*  shown on the auto-generated headerMenuButton
* @type {number}
* @default "7"
*/
headerMenuButtonIconHeight?: number;

/**
* If ListGrid.showHeaderMenuButton is true, this property governs the width of the
*  auto-generated headerMenuButton
* @type {number}
* @default "16"
*/
headerMenuButtonWidth?: number;

/**
* If ListGrid.showHeaderMenuButton is true, this property governs the height of the
*  auto-generated headerMenuButton
* @type {number | string}
* @default "100%"
*/
headerMenuButtonHeight?: number | string;

/**
* If ListGrid.showHeaderMenuButton is true, this property governs the width of the
*  auto-generated headerMenuButton over a 
*  ListGridField.rotateTitle header button.
* @type {number}
* @default "16"
*/
rotatedHeaderMenuButtonWidth?: number;

/**
* If ListGrid.showHeaderMenuButton is true, this property governs the height of the
*  auto-generated headerMenuButton over a
*  ListGridField.rotateTitle header button.
* @type {number | string}
* @default "100%"
*/
rotatedHeaderMenuButtonHeight?: number | string;

/**
* Indicates whether records can be dragged from this listGrid and dropped elsewhere.
*  
*  NOTE: If canDragRecordsOut is initially enabled or might be
*  dynamically enabled after the grid is created, it may be desirable to disable
*  Canvas.useTouchScrolling so that touch-dragging a record starts
*  a drag operation rather than a scroll, but see the discussion of 
*  ListGrid.showDragHandles. If 
*  Canvas.disableTouchScrollingForDrag is set to true, then touch
*  scrolling will be disabled automatically. However, for
*  accessibility reasons, it is recommended to leave touch
*  scrolling enabled and provide an alternative set of controls that can be used to perform
*  drag and drop of records out of the grid.
* @type {boolean}
* @default false
*/
canDragRecordsOut?: boolean;

/**
* Indicates whether records can be dropped into this listGrid.
* @type {boolean}
* @default false
*/
canAcceptDroppedRecords?: boolean;

/**
* Indicates whether records can be reordered by dragging within this ListGrid.
*  
*  NOTE: If canReorderRecords is initially enabled or might be
*  ListGrid.setCanReorderRecords after the grid is created,
*  it may be desirable to disable Canvas.useTouchScrolling
*  so that touch-dragging a record starts a reorder operation rather than a scroll,
*  but see the discussion of ListGrid.showDragHandles. If 
*  Canvas.disableTouchScrollingForDrag is set to true, then touch
*  scrolling will be disabled automatically. However, for accessibility
*  reasons, it is recommended to leave touch scrolling enabled and provide an alternative
*  set of controls that can be used to perform drag-reordering of records.
* @type {boolean}
* @default false
*/
canReorderRecords?: boolean;

/**
* Indicates whether fields in this listGrid can be reordered by dragging and dropping
*  header fields. If true, can be overridden at the field level via 
*  ListGridField.canReorder.
* @type {boolean}
* @default true
*/
canReorderFields?: boolean;

/**
* Indicates whether fields in this listGrid can be resized by dragging header
*  fields.
* @type {boolean}
* @default true
*/
canResizeFields?: boolean;

/**
* When records are being dragged from within a ListGrid, what sort of drag-tracker
*  should be displayed?
*  Note that if multiple records are being dragged the displayed tracker will be
*  based on the first selected record.
* @type {DragTrackerMode}
* @default "icon"
*/
dragTrackerMode?: DragTrackerMode;

/**
* If true, the grid contents are redrawn in real time as fields are resized.
*  This can be slow with a large grid and/or on some platforms. By default, this is enabled
*  in modern desktop browsers. This is automatically switched off in mobile browsers.
* @type {boolean}
* @default "see below"
*/
resizeFieldsInRealTime?: boolean;

/**
* Indicates what to do with data dragged into another DataBoundComponent. See
*  DragDataAction type for details.
* @type {DragDataAction}
* @default "move"
*/
dragDataAction?: DragDataAction;

/**
* This is the pixel-amount by which child components are offset within the grid-body, by 
*  default from the left, or from the right when Page.isRTL is in effect. For
*  ListGrid.canExpandRecords, this attribute is overridden by 
*  ListGrid.expansionIndent.
*  
*  This setting overrides the ListGrid.embeddedComponentMargin for
*  embedded-components, on the appropriate side.
* @type {Integer}
* @default "25"
*/
embeddedComponentIndent?: Integer;

/**
* This is the space to apply as margin around child-components embedded in this grid. 
*  This value is overridden on one side for 
*  ListGrid.canExpandRecords by 
*  ListGrid.expansionIndent and in other scenarios
*  by ListGrid.embeddedComponentIndent.
* @type {Integer}
* @default "0"
*/
embeddedComponentMargin?: Integer;

/**
* When ListGrid.canExpandRecords is true, this is the 
*  pixel-amount by which child components are offset within the grid-body, by default
*  from the left, or from the right when Page.isRTL is in effect. If unset,
*  assumes the width of the ListGrid.expansionField, so that child
*  components line up with the following field, according to RTL.
*  
*  This setting overrides the ListGrid.embeddedComponentIndent for
*  embedded-components, on the appropriate side.
* @type {Integer}
* @default null
*/
expansionIndent?: Integer;

/**
* Where do 'skin' images (those provided with the class) live?
* @type {SCImgURL}
* @default "images/ListGrid/"
*/
skinImgDir?: SCImgURL;

/**
* Image to show when sorting ascending. See ImgProperties for format.
* @type {ImgProperties}
* @default "{...}"
*/
sortAscendingImage?: ImgProperties;

/**
* Image to show when sorting descending. See ImgProperties for format.
* @type {ImgProperties}
* @default "{...}"
*/
sortDescendingImage?: ImgProperties;

/**
* Default image to use for the dragTracker when things are dragged within or out of this
*  list. See ImgProperties for format.
* @type {ImgProperties}
* @default "{...}"
*/
trackerImage?: ImgProperties;

/**
* An optional CSS style to apply to the checkbox image. If supplied, and the checkbox is
*  enabled, the base style is suffixed with "True", "False", or "Partial" if the checkbox
*  is selected, unselected, or partially selected; if the checkbox is disabled, the suffix
*  is "TrueDisabled", "FalseDisabled", or "PartialDisabled".
*  
*  NOTE: This attribute is not supported by TreeGrid.
* @type {CSSStyleName}
* @default null
*/
booleanBaseStyle?: CSSStyleName;

/**
* If set, the ListGrid.booleanBaseStyle to use when printing.
* @type {CSSStyleName}
* @default null
*/
printBooleanBaseStyle?: CSSStyleName;

/**
* Image to display for a true value in a boolean field. The special value "blank" means
*  that no image will be shown.
*  
*  To turn this off explicitly set ListGridField.suppressValueIcon to true.
*  
*  If this, ListGrid.booleanFalseImage and ListGrid.booleanPartialImage
*  are unset, this will be set to the default CheckboxItem.checkedImage.
*  
*  skinning can be used for this image, by setting this property to
*  a SCSpriteConfig formatted string. Alternatively developers can
*  omit this property and instead use CSS directly in the 
*  ListGrid.booleanBaseStyle property to provide a "boolean true" appearance.
* @type {SCImgURL}
* @default null
*/
booleanTrueImage?: SCImgURL;

/**
* Image to display for a false value in a boolean field. Default null value
*  or the special value "blank" means no image will be displayed.
*  
*  To turn this off explicitly set ListGridField.suppressValueIcon to true
*  
*  If this, ListGrid.booleanTrueImage and ListGrid.booleanPartialImage
*  are unset, this will be set to the default CheckboxItem.uncheckedImage.
*  
*  
*  skinning can be used for this image, by setting this property to
*  a SCSpriteConfig formatted string. Alternatively developers can
*  omit this property and instead use CSS directly in the 
*  ListGrid.booleanBaseStyle property to provide a "boolean false" appearance.
* @type {SCImgURL}
* @default null
*/
booleanFalseImage?: SCImgURL;

/**
* Image to display for a partially true value in a boolean field (typically selection).
*  The special value "blank" means that no image will be shown.
*  
*  To turn this off explicitly set ListGridField.suppressValueIcon to true.
*  
*  If this, ListGrid.booleanTrueImage and ListGrid.booleanFalseImage
*  are unset, this will be set to the default CheckboxItem.partialSelectedImage.
*  
*  skinning can be used for this image, by setting this property to
*  a SCSpriteConfig formatted string. Alternatively developers can
*  omit this property and instead use CSS directly in the 
*  ListGrid.booleanBaseStyle property to provide a "boolean true" appearance.
* @type {SCImgURL}
* @default null
*/
booleanPartialImage?: SCImgURL;

/**
* If set, the ListGrid.booleanTrueImage to use when printing.
*  
*  If this, ListGrid.printBooleanFalseImage and ListGrid.printBooleanPartialImage
*  are unset, this will be set to the default CheckboxItem.printCheckedImage.
* @type {SCImgURL}
* @default null
*/
printBooleanTrueImage?: SCImgURL;

/**
* If set, the ListGrid.booleanFalseImage to use when printing.
*  
*  If this, ListGrid.printBooleanTrueImage and ListGrid.printBooleanPartialImage
*  are unset, this will be set to the default CheckboxItem.printUncheckedImage.
* @type {SCImgURL}
* @default null
*/
printBooleanFalseImage?: SCImgURL;

/**
* If set, the ListGrid.booleanPartialImage to use when printing.
*  
*  If this, ListGrid.printBooleanTrueImage and ListGrid.printBooleanFalseImage
*  are unset, this will be set to the default CheckboxItem.printPartialSelectedImage.
* @type {SCImgURL}
* @default null
*/
printBooleanPartialImage?: SCImgURL;

/**
* Width for the ListGrid.booleanTrueImage, ListGrid.booleanFalseImage
*  and ListGrid.booleanPartialImage.
*  Note: If ListGrid.booleanTrueImage is unset, the CheckboxItem.checkedImage
*  will be used to indicate a true value in a boolean field. In this case this property is
*  ignored in favor of CheckboxItem.valueIconWidth.
* @type {number}
* @default "16"
*/
booleanImageWidth?: number;

/**
* Height for the ListGrid.booleanTrueImage, ListGrid.booleanFalseImage
*  and ListGrid.booleanPartialImage.
*  Note: If ListGrid.booleanTrueImage is unset, the CheckboxItem.checkedImage
*  will be used to indicate a true value in a boolean field. In this case this property is
*  ignored in favor of CheckboxItem.valueIconHeight.
* @type {number}
* @default "16"
*/
booleanImageHeight?: number;

/**
* When set to true, shows the ListGrid.dragHandleField on initial draw.
* @type {boolean}
* @default null
*/
showInitialDragHandles?: boolean;

/**
* The title to use for the ListGrid.dragHandleField.
*  
*  By default this title is not displayed in the drag column header button as the
*  autochild defaults for the field set ListGridField.showTitle to
*  false.
* @type {string}
* @default "&amp;nbsp;"
*/
dragHandleFieldTitle?: string;

/**
* Default icon to show in the ListGrid.dragHandleField..
* @type {SCImgURL}
* @default "[SKIN]/actions/drag.png"
*/
dragHandleIcon?: SCImgURL;

/**
* Default width and height of ListGrid.dragHandleIcon for this ListGrid.
* @type {number}
* @default "16"
*/
dragHandleIconSize?: number;

/**
* An automatically generated field that can be dragged to drag the current selection
*  (where otherwise the grid itself might be scrolled). Visibility is controlled by 
*  ListGrid.showInitialDragHandles, ListGrid.showDragHandles, and ListGrid.hideDragHandles.
* @type {ListGridField}
* @default null
*/
dragHandleField?: ListGridField;

/**
* When set to true, shows an additional field at the beginning of the field-list
*  (respecting RTL) that displays the current rowNum for each record.
* @type {boolean}
* @default null
*/
showRowNumbers?: boolean;

/**
* The CSS Style name for the ListGrid.rowNumberField.
* @type {CSSStyleName}
* @default "specialCol"
*/
rowNumberStyle?: CSSStyleName;

/**
* If ListGrid.showRowNumbers is true, should we apply the ListGrid.rowNumberStyle to
*  the ListGrid.rowNumberField
* @type {boolean}
* @default true
*/
applyRowNumberStyle?: boolean;

/**
* The number to start the row-count from - default value is 1.
* @type {number}
* @default "1"
*/
rowNumberStart?: number;

/**
* An automatically generated field that displays the current row number when
*  ListGrid.showRowNumbers is true.
* @type {ListGridField}
* @default null
*/
rowNumberField?: ListGridField;

/**
* Dictates whether the data in this grid should be exported raw by
*  ListGrid.exportClientData. If set to true,
*  data will not be processed by field-formatters during exports.
*  Decreases the time taken for large exports. This property can also be set at the
*  ListGridField.exportRawValues.
* @type {boolean}
* @default null
*/
exportRawValues?: boolean;

/**
* Dictates whether numeric values should be exported as raw numbers instead of
*  formatted values when using ListGrid.exportClientData.
*  
*  This property is only consulted if exportRawValues is not set to
*  true at the ListGrid.exportRawValues or
*  ListGridField.exportRawValues level. That property causes all values,
*  including numeric values, to be exported unformatted.
*  
*  This is useful for cases where an explicit ListGrid formatter function simply displays the number
*  as a formatted string for the user (for example "1,234"). Exporting that formatted
*  string rather than the underlying numeric value causes spreadsheet applications such as
*  Excel to lose some functionality.
*  
*  If this property is not explicitly set, numeric values will be exported as raw
*  numbers for DSRequest.exportAs only.
*  
*  May be overridden at the field level via ListGridField.exportRawNumbers.
* @type {boolean}
* @default null
*/
exportRawNumbers?: boolean;

/**
* The method of RecordComponentPoolingMode to employ for
*  ListGrid.canExpandRecords.
*  
*  The default mode is "destroy", which means that automatically created expansionComponents
*  are destroyed when rows are collapsed.
* @type {ExpansionComponentPoolingMode}
* @default "destroy"
*/
expansionComponentPoolingMode?: ExpansionComponentPoolingMode;

/**
* When set to true, shows an additional field at the beginning of the field-list
*  (respecting RTL) to allow users to expand and collapse individual records.
*  See ListGrid.expandRecord and ListGrid.expansionMode for details
*  on record expansion.
*  
*  ListGrid.virtualScrolling is automatically enabled when canExpandRecords is
*  set to true.
*  
*  Note that expanded records are not currently supported in conjunction
*  with ListGridField.frozen.
* @type {boolean}
* @default false
*/
canExpandRecords?: boolean;

/**
* When ListGrid.canExpandRecords is true, this property indicates whether multiple
*  records can be expanded simultaneously. If set to false, expanding a record will
*  automatically collapse any record which is already expanded. The default value is
*  true.
* @type {boolean}
* @default true
*/
canExpandMultipleRecords?: boolean;

/**
* When ListGrid.canExpandRecords and ListGrid.canExpandMultipleRecords are
*  both true, this property dictates the number of
*  records which can be expanded simultaneously. If the expanded record count hits the
*  value of this property, further attempts to expand records will result in a popup
*  warning (see ListGrid.maxExpandedRecordsPrompt) and expansion will be cancelled.
*  
*  The default value is null, meaning there is no limit on the number of expanded records.
* @type {Integer}
* @default null
*/
maxExpandedRecords?: Integer;

/**
* This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed. Note that the local variable count will be
*  available and set to this.maxExpandedRecords. The string will be executed in the scope of the
*  ListGrid so this may also be used to determine other information about this grid.
*  
*  Default value returns 
*  
*  This grid is limited to [ListGrid.maxExpandedRecords] simultaneously
*  expanded records. Please collapse some expanded records and retry.
*  
* @type {HTMLString}
* @default "This grid is limited to ${count} simultaneously expanded records.  Please collapse some expanded records and retry."
*/
maxExpandedRecordsPrompt?: HTMLString;

/**
* The field whose contents to show in the expanded portion of a record when
*  ListGrid.canExpandRecords is true and
*  ExpansionMode is detailField.
* @type {string}
* @default null
*/
detailField?: string;

/**
* The ExpansionMode for records in this grid.
*  
*  If ListGrid.canExpandRecords is true but expansionMode
*  is not set, it defaults to "detailRelated" if ListGrid.detailDS is set, or to 
*  "details" otherwise.
* @type {ExpansionMode}
* @default null
*/
expansionMode?: ExpansionMode;

/**
* The field providing the facility to expand and collapse rows.
* @type {ListGridField}
* @default null
*/
expansionField?: ListGridField;

/**
* If ListGrid.canExpandRecords is set to true, this property
*  determines the image to display in the expansion field for expanded rows.
*  If unset, the ListGrid.booleanTrueImage will be used.
* @type {SCImgURL}
* @default null
*/
expansionFieldTrueImage?: SCImgURL;

/**
* If ListGrid.canExpandRecords is set to true, this property
*  determines the image to display in the expansion field for collapsed rows.
*  If unset, the ListGrid.booleanFalseImage will be used.
* @type {SCImgURL}
* @default null
*/
expansionFieldFalseImage?: SCImgURL;

/**
* If this grid is in RTL mode, should an "_rtl" suffix be added to the ListGrid.expansionFieldTrueImage
*  and ListGrid.expansionFieldFalseImage image URLs? This should only
*  be enabled if RTL media for the true and false expansion field images are available.
*  
*  If both this property and ListGrid.expansionFieldImageShowSelected are true, and
*  the grid is in RTL mode, both suffixes will be applied to selected rows'
*  expansion field images (combined as "selected_rtl").
* @type {boolean}
* @default false
*/
expansionFieldImageShowRTL?: boolean;

/**
* Should a "_selected" suffix be added to the 
*  ListGrid.expansionFieldTrueImage
*  and ListGrid.expansionFieldFalseImage image URLs for
*  selected rows?
*  
*  This allows developers to provide separate expansion field media for selected
*  rows, in case the selected row style does not contrast well with the standard
*  expansion field image media.
*  
*  If both this property and ListGrid.expansionFieldImageShowRTL are true, and
*  the grid is in RTL mode, both suffixes will be applied to selected rows'
*  expansion field image (combined as "selected_rtl")
* @type {boolean}
* @default false
*/
expansionFieldImageShowSelected?: boolean;

/**
* If ListGrid.canExpandRecords is set to true, this property
*  may be set to govern the width of the expansion image displayed to indicate whether a row
*  is expanded. If unset, the expansionField image will be sized to match the
*  ListGrid.booleanImageWidth for this grid.
* @type {Integer}
* @default null
*/
expansionFieldImageWidth?: Integer;

/**
* If ListGrid.canExpandRecords is set to true, this property
*  may be set to govern the height of the expansion image displayed to indicate whether a
*  row is expanded. If unset, the expansionField image will be sized to match the
*  ListGrid.booleanImageHeight for this grid.
* @type {Integer}
* @default null
*/
expansionFieldImageHeight?: Integer;

/**
* Automatically generated HTMLFlow for displaying the contents of
*  ListGrid.detailField in a record's expanded section when
*  ExpansionMode is detailField.
*  
*  This component is an AutoChild and as such may be customized via
*  listGrid.expansionDetailFieldProperties and
*  listGrid.expansionDetailFieldDefaults.
*  
*  Note, however, that this is a multi-instance component (potentially one per record),
*  so it is created using Class.createAutoChild not
*  Class.addAutoChild, and no default single instance is created by
*  name on the grid.
* @type {HTMLFlow}
* @default null
*/
expansionDetailField?: HTMLFlow;

/**
* Automatically generated DetailViewer for displaying the details of a record
*  in its expanded section when
*  ExpansionMode is details. Note that
*  only those fields
*  which do not already appear in the grid are displayed in the expanded section.
*  
*  This component is an AutoChild and as such may be customized via
*  listGrid.expansionDetailsProperties and
*  listGrid.expansionDetailsDefaults.
*  
*  Note, however, that this is a multi-instance component (potentially one per record),
*  so it is created using Class.createAutoChild not
*  Class.addAutoChild, and no default single instance is created by
*  name on the grid.
* @type {DetailViewer}
* @default null
*/
expansionDetails?: DetailViewer;

/**
* Automatically generated ListGrid for displaying data related to a record
*  in its expanded section when
*  ExpansionMode is related.
*  The DataSource containing the related data is provided by
*  ListGrid.getRelatedDataSource which, by default,
*  returns the DataSource referred to in ListGridRecord.detailDS.
*  
*  This component is an AutoChild and as such may be customized via
*  listGrid.expansionRelatedProperties and
*  listGrid.expansionRelatedDefaults.
*  
*  Note, however, that this is a multi-instance component (potentially one per record),
*  so it is created using Class.createAutoChild not
*  Class.addAutoChild, and no default single instance is created by
*  name on the grid.
* @type {ListGrid}
* @default null
*/
expansionRelated?: ListGrid;

/**
* Automatically generated DynamicForm for editing the details of a record
*  in its expanded section when
*  ExpansionMode is editor. Note that only
*  those fields which do not already appear in the grid will appear in the expanded section.
*  
*  According to the value of ListGrid.showExpansionEditorSaveButton, a save button is shown
*  beneath the editor. You can save the values in the editor by clicking this button
*  
*  This component is an AutoChild and as such may be customized via
*  listGrid.expansionEditorProperties and
*  listGrid.expansionEditorDefaults.
*  
*  Note, however, that this is a multi-instance component (potentially one per record),
*  so it is created using Class.createAutoChild not
*  Class.addAutoChild, and no default single instance is created by
*  name on the grid.
* @type {DynamicForm}
* @default null
*/
expansionEditor?: DynamicForm;

/**
* When ListGrid.canExpandRecords is true and
*  ListGrid.expansionMode is editor, whether a dialog should be
*  displayed when an expanded row is collapsed while it's nested editor has changed values.
* @type {boolean}
* @default null
*/
expansionEditorShowSaveDialog?: boolean;

/**
* When ListGrid.canExpandRecords is true and
*  ListGrid.expansionMode is editor, the prompt to display
*  in a dialog when an expanded row is collapsed while it's nested editor has changed values.
* @type {string}
* @default "You have unsaved changes - do you want to save them now?"
*/
expansionEditorSaveDialogPrompt?: string;

/**
* When ExpansionMode is editor, should the row be collapsed following a
*  save initiated by the expansion-component's ListGrid.expansionEditorSaveButton.
* @type {boolean}
* @default true
*/
expansionEditorCollapseOnSave?: boolean;

/**
* When ExpansionMode is editor, should a Save button be shown below the
*  the expanded editor?
*  
*  Note that if an expanded-row containing an editor is collapsed while changes are
*  outstanding, changes will be either be automatically updated to the grid, or will first
*  show a confirmation dialog, according to the value of
*  ListGrid.expansionEditorShowSaveDialog.
* @type {boolean}
* @default true
*/
showExpansionEditorSaveButton?: boolean;

/**
* Automatically generated IButton for saving the values in the expanded
*  portion of a ListGrid row.
*  
*  This component is an AutoChild and as such may be customized via
*  listGrid.expansionEditorSaveButtonProperties and
*  listGrid.expansionEditorSaveButtonDefaults.
*  
*  Note, however, that this is a multi-instance component (potentially one per record),
*  so it is created using Class.createAutoChild not
*  Class.addAutoChild, and no default single instance is created by
*  name on the grid.
* @type {IButton}
* @default null
*/
expansionEditorSaveButton?: IButton;

/**
* The title for the ListGrid.expansionEditorSaveButton.
* @type {string}
* @default "Save"
*/
expansionEditorSaveButtonTitle?: string;

/**
* Automatically generated HLayout appearing in a record's expanded section
*  when ExpansionMode is detailRelated.
*  This component contains two other autoChild components,
*  a DetailViewer for viewing fields from the record which are not already
*  present in the grid and a separate embedded ListGrid for displaying other
*  data related to this record via record.detailDS. See ListGrid.expansionDetails
*  and ListGrid.expansionRelated for more information.
*  
*  This component is an AutoChild and as such may be customized via
*  listGrid.expansionDetailRelatedProperties and
*  listGrid.expansionDetailRelatedDefaults.
*  
*  Note, however, that this is a multi-instance component (potentially one per record),
*  so it is created using Class.createAutoChild not
*  Class.addAutoChild, and no default single instance is created by
*  name on the grid.
* @type {HLayout}
* @default null
*/
expansionDetailRelated?: HLayout;

/**
* Automatically generated VLayout which fills a record's expanded section
*  and contains other builtin ExpansionMode. You can also
*  override ListGrid.getExpansionComponent to provide
*  components of your own specification.
*  
*  This component is an AutoChild and as such may be customized via
*  listGrid.expansionLayoutProperties and
*  listGrid.expansionLayoutDefaults.
*  
*  Note, however, that this is a multi-instance component (potentially one per record),
*  so it is created using Class.createAutoChild not
*  Class.addAutoChild, and no default single instance is created by
*  name on the grid.
* @type {VLayout}
* @default null
*/
expansionLayout?: VLayout;

/**
* Screen to create (via RPCManager.createScreen) in lieu of calling
*  ListGrid.getExpansionComponent.
*  
*  If this grid has a ListGrid.dataSource, the created screen is
*  provided with a Canvas.dataContext that includes the record being expanded.
*  Be sure the expansion screen meets these Canvas.autoPopulateData
*  to utilize the dataContext.
* @type {string}
* @default null
*/
expansionScreen?: string;

/**
* If ListGrid.canExpandRecords is true and ExpansionMode
*  is "related", this property specifies the dataSource for the
*  related records grid to be shown embedded in expanded records.
*  
*  This property may also be specified on a per-record basis - see
*  ListGrid.recordDetailDSProperty
* @type {string}
* @default null
*/
detailDS?: string;

/**
* The name of the ListGridRecord property that specifies the DataSource to use when
*  ExpansionMode is "related". The default is
*  ListGridRecord.detailDS. Note that you can set the ListGrid.detailDS at the
*  grid level instead if the same dataSource is to be used for all records.
* @type {string}
* @default "detailDS"
*/
recordDetailDSProperty?: string;

/**
* For ExpansionMode that show another grid or tree, what the
*  child's expansionMode should be.
*  Default value null means no further expansion.
* @type {ExpansionMode}
* @default null
*/
childExpansionMode?: ExpansionMode;

/**
* For ExpansionMode that show another grid or tree, is that
*  component editable?
*  The default value for this property is false.
* @type {boolean}
* @default null
*/
expansionCanEdit?: boolean;

/**
* Causes filtering to be performed against the local data set, even when a
*  ListGrid.dataSource is provided.
*  
*  When using this mode, data must be provided to the grid via ListGrid.setData,
*  and must be provided as 
*  a simple Array of Records
*  . 
*  
*  Note that a ListGrid.dataSource must be provided for filtering to occur
*  even when filtering locally.
*  
*  If this property is set to true, the supplied data is applied as the 
*  ResultSet.allRows of a ResultSet, which is then 
*  filtered according to the specified criteria, and the results displayed. If false, a 
*  normal databound fetch will occur, retrieving records that match the specified criteria 
*  from this component's ListGrid.dataSource.
*  
*  filterLocalData includes both calls to ListGrid.fetchData and
*  ListGrid.filterData as well as automatic filtering when the
*  ListGrid.filterEditor is enabled.
*  
*  If this property is not explicitly set, default behavior will filter against the
*  dataSource unless the grid has a specified DataPath, in which case
*  filtering will occur locally.
*  
*  See also ListGrid.saveLocally to cause saves to ignore the DataSource and affect
*  the local data set only.
* @type {boolean}
* @default null
*/
filterLocalData?: boolean;

/**
* If ListGrid.canAcceptDroppedRecords is true for this listGrid, this property governs
*  whether the user can drop between, or over records within the grid.
*  This controls what RecordDropPosition is passed to the ListGrid.recordDrop
*  event handler.
* @type {RecordDropAppearance}
* @default "between"
*/
recordDropAppearance?: RecordDropAppearance;

/**
* If a user is editing a ListGrid.canEdit listGrid, and they hide
*  a field while the editor is showing, should we discard any edits in the edit row for
*  the field being hidden?
*  
*  Default behavior is to discard the edits - set this flag to false to preserve edits
* @type {boolean}
* @default true
*/
discardEditsOnHideField?: boolean;

/**
* Initial ListGridFieldState for the grid.
*  
*  ViewState can be used to initialize all view properties of the grid.
*  When doing so, fieldState is not needed because viewState
*  includes it as well. If both are provided, fieldState has priority for
*  field state.
*  
*  
*  To retrieve current state call ListGrid.getFieldState.
*  
* @type {ListGridFieldState}
* @default null
*/
fieldState?: ListGridFieldState;

/**
* How much horizontal space should the ListGrid.getCheckboxField leave
*  around the checkbox icon when ListGrid.selectionAppearance is set to
*  "checkbox"?
*  
*  The automatically generated checkbox field will be sized to the width of the
*  checkbox icon (specified via ListGrid.checkboxFieldImageWidth or
*  ListGrid.booleanImageWidth) plus this value.
* @type {Integer}
* @default "15"
*/
checkboxFieldHSpace?: Integer;

/**
* Returns a snapshot of the current selection within this listGrid as
*  a ListGridSelectedState object.
*  This object can be passed to ListGrid.setSelectedState to reset this grid's selection
*  the current state (assuming the same data is present in the grid).
* @type {ListGridSelectedState}
* @default null
*/
selectedState?: ListGridSelectedState;

/**
* Initial sort state for the grid.
*  
*  ListGrid.viewState can be used to initialize all view properties of the grid.
*  When doing so, sortState is not needed because viewState
*  includes it as well. If both are provided, sortState has priority for
*  sort state.
*  
*  
*  To retrieve current state call ListGrid.getSortState.
*  
* @type {ListGridSortState}
* @default null
*/
sortState?: ListGridSortState;

/**
* Should the current ListGrid.getFilterEditorCriteria be included 
*  along with other details when saving this grid's ListGrid.viewState?
*  
*  If true, the current criteria will be saved when calling ListGrid.getViewState and will
*  be applied to the grid via a filter when calling ListGrid.setViewState.
*  
*  Notes
*  
*  
* - If this grid has never actually fetched data, criteria will not be included
*  
* - If this flag is set to false, ListGrid.setViewState will not
*    modify the grid's filter even if criteria were included when viewState was stored.
*  
* @type {boolean}
* @default true
*/
saveCriteriaInViewState?: boolean;

/**
* Initial group state for the grid.
*  
*  ListGrid.viewState can be used to initialize all view properties of the grid.
*  When doing so, groupState is not needed because viewState
*  includes it as well. If both are provided, groupState has priority for
*  group state.
*  
*  
*  To retrieve current state call ListGrid.getGroupState.
*  
* @type {ListGridGroupState}
* @default null
*/
groupState?: ListGridGroupState;

/**
* Initial view state may be provided for the listGrid at init time.
*  To set view state at runtime, use ListGrid.setViewState.
*  See ListGridViewState for details of what is included in the viewState for
*  a ListGrid by default.
*  
*  View state is a composite object containing various more granular states such as 
*  FieldState, ListGrid.saveCriteriaInViewState, etc.
*  As such it is not necessary to specificify fieldState, etc. in addition to 
*  viewState at init time, but if both are provided the specific states 
*  (fieldState, etc) will have priority.
*  
*  
*  To retrieve current state call ListGrid.getViewState.
*  
*  
*  Note: For SmartClient Pro users, 
*  the SavedSearches provides a sophisticated, out of the box
*  set of capabilities and user-interface components for users to store and apply multiple
*  named views or "saved searches". See ListGrid.canSaveSearches for how to enable menu options
*  to save searches in grids. Note that each saved search includes
*  the ListGrid.savedSearchStoredState for the grid by default.
* @type {ListGridViewState}
* @default null
*/
viewState?: ListGridViewState;

/**
* Setting this property to a non-null value will enable automatic saving of 
*  ListGrid.getViewState to offline storage. This saved view state will then
*  be restored automatically when the user visits the page again.
*  
*  Note: SmartClient Pro users, 
*  may also be interested in the ListGrid.canSaveSearches feature. This uses the
*  SavedSearches to allow users to explicitly store and apply
*  multiple named views or "saved searches". Each saved search includes
*  the ListGrid.savedSearchStoredState for the grid by default.
*  
*  autoPersistViewState may be set to a list of 
*  ListGridViewState ListGridViewStatePart that
*  should be automatically persisted into offline storage when changed.
*  
*  This feature saves the derived state whenever the grid's view state
*  changes due to user interaction (see ListGrid.viewStateChanged), and
*  restores the saved state from offline storage when the grid is drawn.
*  
*  The state is saved to offline storage using the grid's AutoTestLocator
*  as the key. See Locator details below.
*  
*  Note that autoPersistViewState should only be set on specific listGrid instances,
*  and never as a default value for the class by changing the ListGrid defaults. Enabling this
*  feature as a default would be an invalid usage as it would apply to listgrids
*  (and subclasses of ListGrid) created and re-used internally by framework features as well
*  as those explicitly created by application code.
*  
*  The current saved value can be retrieved or cleared by calling
*  ListGrid.getSavedViewState or ListGrid.clearSavedViewState respectively.
*  
*  Locator details
*  
*  The grid must have a stable locator so that previous state can be
*  retrieved during initial draw and saved back into the same place. 
*  If the grid has an explicit Canvas.ID the locator will always be stable. Setting an
*  explicit ID on a known parent of the grid can also lead to a stable ID as described in
*  the usingSelenium.
*  
*  For purposes of this feature the top-level parent of the grid must have an explicit ID.
*  
*  Additional details on locators and their use can be found in AutoTest and
*  LocatorStrategy.
* @type {Array<Partial<ListGridViewStatePart>>}
* @default null
*/
autoPersistViewState?: Array<Partial<ListGridViewStatePart>>;

/**
* Width to size non-visible fields (which may be specified with
*  DataBoundComponent.exportFields or DSRequest.exportFields) during
*  ListGrid.exportData or ListGrid.exportClientData, if they have no defined numeric
*  ListGridField.width.
* @type {Integer}
* @default "100"
*/
exportHiddenFieldWidth?: Integer;

/**
* Array of components that make up this grid. This array controls which standard and/or custom
*  parts will be displayed within this ListGrid.
*  
*  ListGrid is a subclass of VLayout and consists of a number of member components. The
*  standard set of members are automatically generated by the grid, and include (for example)
*  the header (a Toolbar of buttons for each field) and the body
*  (a GridRenderer displaying the actual data contained in the grid).
*  The default value of gridComponents is an Array of ListGridComponents listing
*  the standard components in their default order:
*  
*  
*   gridComponents : ["filterEditor", "header",
*            "body", "summaryRow"]
*  
*  
*  
* 
*  You can override gridComponents to change the order of standard components.
*  You can also omit standard components this way, although it more efficient to
*  use the related "show" property if available (eg ListGrid.showFilterEditor). Note that
*  this array must contain an entry for the "body" - listGrids with no body showing
*  are unsupported.
*  Advanced note: The live components generated for each of these standard
*  ListGridComponent types may differ across different listGrids. For example if this
*  grid has any ListGridField.frozen, the "body" entry will actually
*  be created as an HLayout containing two GridRenderers (one for frozen fields,
*  and one for unfrozen fields). This is really an implementation detail - the "body" entry
*  in the gridComponents array simply specifies where the UI for the body should render within
*  the ListGrid layout.
*  
*  By embedding a Canvas directly in this list you can add arbitrary additional components to the
*  listGrid as members, and have them be displayed alongside the standard automatically generated
*  parts of the ListGrid.
*  
*  Note that having added controls to gridComponents, you can still call APIs directly on
*  those controls to change their appearance, and you can also show() and hide() them if
*  they should not be shown in some circumstances.
*  
*  Tip: custom controls need to set layoutAlign:"center" to appear vertically centered.
*  
*  See this example
*  of subclassing ListGrid and using gridComponents to add a tool bar with standard functions that 
*  you want throughout your application.
* @type {Array<Partial<ListGridComponent>> | Array<Partial<any>>}
* @default "(see below)"
*/
gridComponents?: Array<Partial<ListGridComponent>> | Array<Partial<any>>;

/**
* If this property is set to true, header buttons for either ListGrid.fields or
*  ListGrid.headerSpans will automatically expand to accommodate their
*  titles vertically. This means if you have a "tall" title - typically a long string where
*  ListGridField.wrap is set to true such that you end up with several lines of text -
*  the button will render large enough to accommodate it. If necessary this will cause the
*  header for the grid as a whole to expand beyond the specified ListGrid.headerHeight.
*  
*  Note that you need not set HeaderSpan.height or ListGrid.headerSpanHeight if you set
*  this property, but if you do, they will be used as minimum values.
* @type {boolean}
* @default null
*/
autoFitHeaderHeights?: boolean;

/**
* When using ListGrid.autoFitFieldWidths, padding in pixels left on each side of fields that
*  show images.
* @type {Integer}
* @default "2"
*/
iconPadding?: Integer;

/**
* If ListGridField.userFormula is set for some field, and this grid is showing
*  ListGrid.showGroupSummary or a
*  ListGrid.showGridSummary, this property determines what field value
*  should be present in those summary rows. Should the field apply the user-formula to the
*  calculated summary row, or should it apply a standard grid or group summary to the
*  user-formula values displayed in the grid?
*  
*  May be overridden at the field level via ListGridField.applyAfterSummary
* @type {boolean}
* @default false
*/
applyFormulaAfterSummary?: boolean;

/**
* Default cursor to display when the user rolls over icons within cells of an
*  ListGridField.type field.
*  
*  May be overridden by ListGridField.iconCursor.
*  
*  Note: Unlike the field-level ListGridField.iconCursor property, listGrid.iconCursor
*  has no effect on the cursor displayed for ListGridField.valueIcons. 
*  See ListGrid.getValueIconCursor for more details.
* @type {Cursor}
* @default "pointer"
*/
iconCursor?: Cursor;

/**
* Should cells in this grid be allowed to span multiple rows?
*  If set to true, the ListGrid.getRowSpan method will be
*  called for every cell when rendering out the listGrid to determine how many
*  rows the cell should span.
*  
*  See ListGrid.getRowSpan for more details
* @type {boolean}
* @default false
*/
allowRowSpanning?: boolean;

/**
* Enables various styling behaviors that potentially make sense when ListGrid.getRowSpan has
*  been overridden to introduce spanning cells, and spanning is largest on the left and
*  smaller as cells go to the right. Specifically:
*  
*  
* - computes ListGrid.alternateRecordStyles based on the span of the
*    cell in the left-most column
*  
* - enables ListGrid.canSelectCells, including
*    ListGrid.useCellRollOvers styling
*  
* - enables row-span-sensitive cell selection. See also RowSpanSelectionMode
*    for available behaviors
*  
*  
*  Because this setting enables ListGrid.canSelectCells, it is incompatible with any APIs
*  that expect a record-oriented data model.
*  
*  Because this setting only makes sense when row spanning decreases from the first column
*  to the last, it has unspecified behavior with ListGrid.canReorderFields.
* @type {boolean}
* @default null
*/
useRowSpanStyling?: boolean;

/**
* Chooses the selection mode when ListGrid.useRowSpanStyling is enabled. See
*  RowSpanSelectionMode.
* @type {RowSpanSelectionMode}
* @default "forward"
*/
rowSpanSelectionMode?: RowSpanSelectionMode;

/**
* If ListGrid.allowRowSpanning is enabled, this property may be used to specify
*  editing behavior for cells that span multiple rows.
* @type {RowSpanEditMode}
* @default "first"
*/
rowSpanEditMode?: RowSpanEditMode;

/**
* If ListGrid.showRecordComponents is true, this attribute may be used to
*  specify a standard height for record components.
*  If specified every row in the grid will be sized tall enough to accommodate a recordComponent
*  of this size.
*  
*  Note that if this property is unset, the grid will not be able to know row heights in
*  advance, and ListGridField.frozen are not currently supported in
*  this case. If you are putting a recordComponent in every row, and they all have a consistent
*  height, set recordComponentHeight and you will then be able to use frozen fields
*  and avoid the whitespace side-effect of virtual scrolling by setting 
*  ListGrid.virtualScrolling:false.
*  
*  Similarly, if your recordComponents are never tall enough that they will expand the row
*  beyond the ListGrid.cellHeight, set ListGrid.virtualScrolling:false to avoid the whitespace
*  side-effect of ListGrid.virtualScrolling and to allow 
*  ListGridField.frozen to be used. In this mode, you can have
*  recordComponents on some rows but not others, and recordComponents of different heights, so
*  long as no recordComponent ever causes a row to grow beyond ListGrid.cellHeight (which would
*  happen if the recordComponents height + 2*ListGrid.cellPadding is larger than 
*  ListGrid.cellHeight).
* @type {Integer}
* @default null
*/
recordComponentHeight?: Integer;

/**
* Whether cell contents should wrap during printing. Equivalent to Autofit, but
*  specific to printed output.
* @type {boolean}
* @default true
*/
printAutoFit?: boolean;

/**
* Whether cell contents should wrap during printing. Equivalent to ListGrid.wrapCells, but
*  specific to printed output.
* @type {boolean}
* @default true
*/
printWrapCells?: boolean;

/**
* Style for header cells in printed output. Defaults to ListGrid.headerBaseStyle if null.
* @type {CSSStyleName}
* @default "printHeader"
*/
printHeaderStyle?: CSSStyleName;

/**
* Style for non-header cells in printed output. Defaults to ListGrid.baseStyle if null.
* @type {CSSStyleName}
* @default null
*/
printBaseStyle?: CSSStyleName;

/**
* Advanced property - when generating printHTML for a large ListGrid, rows are printed in
*  batches in order to avoid triggering a native "script is running slowly" browser dialog.
*  
*  For grids with exceptional numbers of columns or complex formatting logic, this number
*  might need to be adjusted downward.
* @type {Integer}
* @default "100"
*/
printMaxRows?: Integer;

/**
* When set to true and canHover is also true, shows a widget hovering at the mouse point.
*  
*  A number of builtin modes are provided - see ListGrid.hoverMode. Note, if a 
*  hoverMode is set but showHoverComponents is left null, it will
*  default to true.
*  
*  Also supported at the ListGridField.showHoverComponents.
* @type {boolean}
* @default null
*/
showHoverComponents?: boolean;

/**
* When ListGrid.showHoverComponents is true, the builtin mode
*  to use when automatically creating a hover component for rows in this grid.
*  
*  A number of builtin modes are provided - see HoverMode. You can also override
*  ListGrid.getCellHoverComponent to provide a custom hover
*  widget - in that case, this attribute is ignored.
*  
*  If showHoverComponents is true but hoverMode is not set, it 
*  defaults to "detailRelated" if ListGrid.detailDS is set, or to "details" otherwise.
*  If showHoverComponents is not set (ie, is null) and hoverMode
*  is set, showHoverComponents defaults to true.
* @type {HoverMode}
* @default null
*/
hoverMode?: HoverMode;

/**
* Screen to create (via RPCManager.createScreen) in lieu of calling
*  Canvas.getHoverComponent or ListGrid.getCellHoverComponent.
*  
*  If this grid has a DataBoundComponent.dataSource, the created screen is
*  provided with a Canvas.dataContext that includes the record being shown at the row.
* @type {string}
* @default null
*/
hoverScreen?: string;

/**
* When set to false, clicking a record's ListGrid.expansionField will
*  not add the record to the current selection.
* @type {boolean}
* @default true
*/
selectOnExpandRecord?: boolean;

/**
* If this property is set to true, clicking on a cell will natively select the 
*  cell's content, ready to be copied to the browser clipboard.
*  
*  For control of this behavior at the field level, 
*  ListGridField.selectCellTextOnClick may be used. These properties interact
*  as follows:
*  
*  listGrid.selectCellTextOnClick value
*  listGridField.selectCellTextOnClick value
*  Behavior
*  
*  true
*    unset or true
*      Cell contents will be natively selected on click.
*  
*    false
*      Cell contents will not be natively selected on click.
*  
*  
*  unset
*    true
*      Cell contents will be natively selected on click.
*  
*    unset or false
*      Cell contents will not be natively selected on click.
*  
*  false
*    true, false or unset
*      Cell contents will not be natively selected on click.
*  
*  
*  This is related to the ListGrid.canDragSelectText attribute which enables
*  native text selection of grid content by standard browser interactions (drag
*  selecting or double-click selecting).
*  
*  Note that developers may also be interested in the related formItem properties
*  FormItem.selectOnClick and FormItem.selectOnFocus.
* @type {boolean}
* @default null
*/
selectCellTextOnClick?: boolean;

/**
* When scrollToCell is called, this is used as defaults if xPosition weren't explicitly passed
*  into the method.
* @type {Alignment}
* @default "center"
*/
scrollToCellXPosition?: Alignment;

/**
* When scrollToCell is called, this is used as defaults if yPosition weren't explicitly passed
*  into the method.
* @type {VerticalAlignment}
* @default "center"
*/
scrollToCellYPosition?: VerticalAlignment;

/**
* If the listGrid is empty, should the user be able to put focus into the grid body by tabbing
*  to it?
*  
*  Note that if ListGrid.editOnFocus is true for this grid and ListGrid.listEndEditAction is set to next,
*  having this property set to true will allow users to automatically create a new edit row by
*  simply tabbing into the grid.
* @type {boolean}
* @default true
*/
canFocusInEmptyGrid?: boolean;

/**
* Should this ListGrid show a summary row beneath the last record of the grid. This summary
*  row will contain per-field summary information. See ListGridField.showGridSummary and
*  ListGrid.getGridSummaryFunction for details on how the summary value to be displayed
*  for each column will be calculated.
*  
*  Note that the ListGrid.summaryRow will be created to actually
*  display the summary row.
* @type {boolean}
* @default false
*/
showGridSummary?: boolean;

/**
* Value to display to the user if showing summary values (through ListGrid.showGridSummary,
*  ListGrid.showGroupSummary or ListGridFieldType), and
*  the summary function returns "null" (implying it was unable to calculate a
*  valid summary value). This property will only be used in the default formatting behavior. If
*  an explicit formatter has been specified - via ListGrid.formatCellValue or
*  ListGridField.formatGridSummary, for example - this property has
*  no effect.
* @type {string}
* @default "&amp;nbsp;"
*/
invalidSummaryValue?: string;

/**
* Property name on a record that will be checked to determine whether a record should
*  be included when calculating totals for the ListGrid.showGridSummary.
* @type {string}
* @default "includeInSummary"
*/
includeInSummaryProperty?: string;

/**
* If ListGrid.showGridSummary is true, this attribute will be set to true on the
*  record object representing the grid summary row.
* @type {string}
* @default "isGridSummary"
*/
gridSummaryRecordProperty?: string;

/**
* If ListGrid.showGroupSummary is true, this attribute will be set to true on each
*  record object representing a group-level summary row.
* @type {string}
* @default "isGroupSummary"
*/
groupSummaryRecordProperty?: string;

/**
* If ListGrid.showGridSummary is true, by default summary values are calculated on the
*  client based on the current data-set for the grid (see ListGrid.getGridSummary and
*  ListGrid.getGridSummaryFunction).
*  
*  In some cases however it may make sense to calculate summary values on the server and retrieve
*  them via a dataSource fetch. If set, this property specifies a dataSource to fetch against for
*  the summary row. The dataSource should return a single record with summary data for each
*  field for which summary data should be shown. Note that specifying this property completely
*  bypasses the standard client-side grid summary calculation logic.
*  
*  The fetch may be further customized via ListGrid.summaryRowCriteria and
*  ListGrid.summaryRowFetchRequestProperties
* @type {DataSource}
* @default null
*/
summaryRowDataSource?: DataSource;

/**
* If ListGrid.showGridSummary is true, and a ListGrid.summaryRowDataSource is
*  specified this property may be used to specify fetch criteria to apply when retrieving
*  summary data to show in the summary row. If unset, and any filter criteria have been
*  specified for the grid, they will be used.
*  
*  If this property is set, the DSRequest.textMatchStyle will default to
*  "exact". Otherwise ListGrid.autoFetchTextMatchStyle will be used. This can be
*  overridden via ListGrid.summaryRowFetchRequestProperties.
* @type {Criteria}
* @default null
*/
summaryRowCriteria?: Criteria;

/**
* Whether to allow selection of the summary row, for example by clicking on the record. The
*  default is to disallow it.
*  
*  If this property is set, further customization of selection can be made by applying the
*  appropriate selection-related properties to the autochild via
*  ListGrid.summaryRow. The request to fetch the summary row(s)
*  can be customized via ListGrid.summaryRowFetchRequestProperties.
* @type {boolean}
* @default false
*/
canSelectSummaryRows?: boolean;

/**
* If ListGrid.showGridSummary is true, and a ListGrid.summaryRowDataSource is specified
*  this property may be used to customize the fetch request used when retrieving summary data
*  to show in the summary row. An example use case might be specifying a
*  DSRequest.operationId to perform a custom fetch operation which retrieved only summary
*  values based on criteria.
*  
*  By default, only the first ListGrid.summaryRowCriteria match is returned if the
*  criteria matches multiple records.
* @type {Partial<DSRequest>}
* @default null
*/
summaryRowFetchRequestProperties?: Partial<DSRequest>;

/**
* If showing any record summary fields (IE: fields of ListGridFieldType),
*  this attribute specifies a custom base style to apply to cells in the summary field
* @type {CSSStyleName}
* @default "recordSummaryCell"
*/
recordSummaryBaseStyle?: CSSStyleName;

/**
* Automatically generated ListGrid for displaying grid summary information (see
*  ListGrid.showGridSummary).
*  
*  This component is an AutoChild and as such may be customized 
*  via
*  listGrid.summaryRowProperties and listGrid.summaryRowDefaults.
*  
*  
* @type {ListGrid}
* @default null
*/
summaryRow?: ListGrid;

/**
* Default height for the ListGrid.summaryRow. Note that this
*  height is a minimum - the summary row has ListGrid.autoFitData set to "vertical" so
*  if multiple rows are visible in the grid summary, the summaryRow component
*  will expand to accommodate them.
* @type {Integer}
* @default "20"
*/
summaryRowHeight?: Integer;

/**
* ListGrid.baseStyle for the ListGrid.summaryRow
* @type {CSSStyleName}
* @default "gridSummaryCell"
*/
summaryRowStyle?: CSSStyleName;

/**
* If true, causes the ListGrid.summaryRow component to be hidden if it has
*  no data after summaries have been recalculated
* @type {boolean}
* @default null
*/
hideEmptySummaryRow?: boolean;

/**
* Causes this grid to filter its data on the client where possible, eliminating trips to 
*  the server when criteria becomes more restrictive, since the filter must apply to data
*  which is already on the client.
*  
*  This attribute is a shortcut to setting useClientFiltering on the grid's 
*  ListGrid.dataProperties and overrides any value set there.
*  
*  While the default value is null, the effective default is true, since 
*  useClientFiltering is true on ResultSet.useClientFiltering.
*  
*  Note that using client-filtering can avoid up to 90% of the most expensive database 
*  requests, so it's critical in most cases to leave it switched on. See the sample 
*  here.
*  
*  In cases where client-filtering alone is not sufficient, note the following:
*  
*  
* - You can easily prevent client-side filtering for a given field by setting
*    DataSourceField.filterOn, if you can't replicate the 
*    filtering on the client.
*  
* - an override of ResultSet.compareCriteria can be used to handle any other, 
*    more complicated differences in client- vs server-side filtering that may arise for a 
*    single grid 
*  
* - a custom search operator can be used if you want to create custom 
*    filtering behavior which applies to a variety of different grids & DataSources
*  
* @type {boolean}
* @default null
*/
useClientFiltering?: boolean;

/**
* Should the ListGrid.showFilterEditor in this grid always produce 
*  AdvancedCriteria?
* @type {boolean}
* @default null
*/
useAdvancedCriteria?: boolean;

/**
* Default OperatorId to use for text-based fields in this grid's 
*  ListGrid.filterEditor, when producing AdvancedCriteria. 
*  When ListGrid.allowFilterExpressions or
*  ListGrid.allowFilterOperators are enabled for the grid,
*  the default is DataSource.translatePatternOperators. Otherwise,
*  the default is "iContains".
*  
*  Does not apply to special fields where exact match is obviously the right default
*  setting, such as fields of type:"enum", or fields with a
*  FormItem.valueMap or FormItem.optionDataSource.
* @type {OperatorId}
* @default null
*/
defaultFilterOperator?: OperatorId;

/**
* For use with ListGrid.showFilterEditor:true, allows simple search expressions to be entered
*  into filter fields, as though DynamicForm.allowExpressions were true.
*  
*  This attribute can also be set at the ListGridField.allowFilterExpressions.
* @type {boolean}
* @default null
*/
allowFilterExpressions?: boolean;

/**
* If this component is bound to a dataSource, this attribute may be specified to customize
*  what fields from the dataSource may be edited by default. For example the SearchForm
*  class has this attribute set to "canFilter" which allows search forms to edit
*  dataSource fields marked as canEdit:false (but not those marked as
*  canFilter:false).
*  
*  Note that if canEdit is explicitly specified on a field in 
*  the DataBoundComponent.fields array, that property will be respected in preference to 
*  the canEditAttribute value. (See FormItem.canEdit, ListGridField.canEdit).
*  Also note that individual dataBoundComponents may have additional logic around whether a field
*  can be edited - for example ListGrid.canEditCell may be overridden.
* @type {string}
* @default "canEdit"
*/
canEditFieldAttribute?: string;

/**
* Should the ListGrid.showGridSummary or ListGrid.showGroupSummary be
*  updated during editing of grid records? This can be set false to improve performance when a
*  large number of ListGridField or DataSourceField are present for the grid.
*  
*  Note that summaries will always be updated when the edits are saved, so to avoid
*  recalculation overhead when a row or cell edit is completed, you must also set
*  ListGrid.autoSaveEdits: false. Summaries will then be updated upon your manual save, such as
*  ListGrid.saveAllEdits.
* @type {boolean}
* @default true
*/
updateSummariesDuringEditing?: boolean;

/**
* CSS Style to apply to the drag tracker when dragging occurs on this component.
* @type {CSSStyleName}
* @default "gridDragTracker"
*/
dragTrackerStyle?: CSSStyleName;

/**
* Special cell-separator that may be inserted between cell values when screen-readers
*  are reading the content of a row. Only applies when screen-reader mode is enabled, and
*  for ListGrids with ListGrid.ariaRole.
*  
*  When isc.setScreenReaderMode is enabled,
*  for ListGrids with ListGrid.ariaRole set to "list", each row
*  in the grid will be rendered as HTML with role "listItem".
*  In this mode, the screenReaderCellSeparator property, (along with
*  ListGrid.screenReaderIncludeFieldTitles and ListGrid.screenReaderRowSeparator), 
*  gives developers a way to customize how screen readers
*  read the row's text value in a way that may be helpful for grids with multiple columns.
*  Instead of just picking up the value of each cell in the row strung together, 
*  readers will instead pick up the field title, the cell value and then the cellSeparator for
*  each cell, and then the rowSeparator to mark the end of the row. Most screenreaders will
*  also read the row index and total row count - something like "3 of 20".
*  
*  
*  Note that screen readers vary widely on which punctuation symbols are read aloud, and
*  sometimes it depends on the context of the punctuation. However, the widely-used JAWS, NVDA,
*  and VoiceOver screen readers all read the forward slash '/' as "slash".
*  See Why Donâ€™t Screen Readers Always Read Whatâ€™s on the Screen? Part 1: Punctuation and Typographic Symbols
*  for a table of findings on which punctuation symbols are read aloud by JAWS, NVDA,
*  and VoiceOver.
*  
*  Implementation notes: the generated row HTML makes use of the 
*  aria-labelledby
*  property to achieve this - pointing the
*  screenReader to the title button for the column, the cell content, and a special hidden
*  element containing the cellSeparator.
*  Note that this aria-labelledby setting is applied in addition to other aria state specified
*  by ListGrid.getRowAriaState.
*  
*  To entirely disable this feature, developers may set 
*  ListGrid.screenReaderWriteRowLabelledBy to false. In this case other
*  row aria attributes will still be picked up from ListGrid.getRowAriaState, including
*  the setsize and posinset attributes that tell the screen reader where
*  they currently are in the list.
*  
*  See ListGrid.ariaRole for more information on the ARIA attributes generated
*  by ListGrids.
* @type {HTMLString}
* @default "/"
*/
screenReaderCellSeparator?: HTMLString;

/**
* Special row-separator that may be inserted at the end of the row value when screen-readers
*  are reading the content of a row. Only applies when screen-reader mode is enabled, and
*  for ListGrids with ListGrid.ariaRole.
*  
*  See the documentation for ListGrid.screenReaderCellSeparator for details on how
*  the header values, cell separators and row separators are picked up by screenreaders
*  in this mode.
*  
*  For more information on the ARIA attributes generated
*  by ListGrids, see ListGrid.ariaRole.
* @type {HTMLString}
* @default ","
*/
screenReaderRowSeparator?: HTMLString;

/**
* Should column titles be read along with cell values when screen-readers
*  are reading the content of a row? Only applies when screen-reader mode is enabled, and
*  for ListGrids with ListGrid.ariaRole.
*  
*  Has no effect if ListGrid.showHeader is false.
*  
*  See the documentation for ListGrid.screenReaderCellSeparator for implementation 
*  details on how the header values, cell separators and row separators are picked up by screenreaders
*  in this mode.
*  
*  For more information on the ARIA attributes generated
*  by ListGrids, see ListGrid.ariaRole.
* @type {boolean}
* @default true
*/
screenReaderIncludeFieldTitles?: boolean;

/**
* When isc.setScreenReaderMode is enabled, for grids with
*  ListGrid.ariaRole set to "list", should a custom
*  aria-labelledby
*  attribute be written out in addition to any other 
*  ListGrid.getRowAriaState 
*  to ensure the ListGrid.screenReaderIncludeFieldTitles,
*  ListGrid.screenReaderCellSeparator and ListGrid.screenReaderRowSeparator are
*  read out along with cell content when reading rows?
*  
*  Setting this property to false will disable the writing out of this labelled-by
*  attribute. See the documentation for ListGrid.screenReaderCellSeparator for more on
*  how the header values, cell separators and row separators are picked up by screenreaders
*  in this mode.
*  
*  For more information on the ARIA attributes generated
*  by ListGrids, see ListGrid.ariaRole.
* @type {boolean}
* @default true
*/
screenReaderWriteRowLabelledBy?: boolean;

/**
* If this grid has specified ListGrid.headerSpans, should field titles be
*  prefixed with the titles of the headerSpans in which they are contained when
*  using the FormulaBuilder or SummaryBuilder.
* @type {boolean}
* @default true
*/
showHeaderSpanTitlesInFormulaBuilder?: boolean;

/**
* If this grid has specified ListGrid.headerSpans, and
*  ListGrid.showHeaderSpanTitlesInFormulaBuilder is true,
*  this string will be inserted between the headerSpan title(s) and the field title in the
*  field chooser grid in the FormulaBuilder and SummaryBuilder.
* @type {string}
* @default " - "
*/
formulaBuilderSpanTitleSeparator?: string;

/**
* If this grid has specified ListGrid.headerSpans, should field titles be
*  prefixed with the titles of the headerSpans in which they are contained when
*  using the MultiSortDialog.
* @type {boolean}
* @default true
*/
showHeaderSpanTitlesInSortEditor?: boolean;

/**
* If this grid has specified ListGrid.headerSpans, and
*  ListGrid.showHeaderSpanTitlesInSortEditor is true, this
*  string will be inserted between the headerSpan title(s) and the field title in the field
*  chooser grid on the MultiSortDialog
* @type {string}
* @default " - "
*/
sortEditorSpanTitleSeparator?: string;

/**
* If this grid has specified ListGrid.headerSpans, should field titles be
*  prefixed with the titles of the headerSpans in which they are contained when
*  using the DataBoundComponent.editHilites.
* @type {boolean}
* @default true
*/
showHeaderSpanTitlesInHiliteEditor?: boolean;

/**
* If this grid has specified ListGrid.headerSpans, and
*  ListGrid.showHeaderSpanTitlesInHiliteEditor is true, this string will be inserted
*  between the headerSpan title and the field title in the hiliteEditor field chooser
*  grid.
* @type {string}
* @default " - "
*/
hiliteEditorSpanTitleSeparator?: string;

/**
* This property may be set to customize the css style for the
*  hover shown on ListGrid.headerHover.
* @type {CSSStyleName}
* @default null
*/
headerHoverStyle?: CSSStyleName;

/**
* This property may be set to customize the alignment for the
*  hover shown on ListGrid.headerHover.
* @type {Alignment}
* @default null
*/
headerHoverAlign?: Alignment;

/**
* This property may be set to customize the vertical alignment for the
*  hover shown on ListGrid.headerHover.
* @type {VerticalAlignment}
* @default null
*/
headerHoverVAlign?: VerticalAlignment;

/**
* This property may be set to customize the opacity for the
*  hover shown on ListGrid.headerHover.
* @type {Integer}
* @default null
*/
headerHoverOpacity?: Integer;

/**
* Optional default height for the
*  hover shown on ListGrid.headerHover.
* @type {Integer}
* @default null
*/
headerHoverHeight?: Integer;

/**
* Optional default width for the
*  hover shown on ListGrid.headerHover.
* @type {Integer}
* @default null
*/
headerHoverWidth?: Integer;

/**
* This property may be set to customize the wrap attribute for the
*  hover shown on ListGrid.headerHover.
* @type {boolean}
* @default null
*/
headerHoverWrap?: boolean;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid, this
*  attribute will be shown as the menu item title to sort a field in ascending order.
* @type {string}
* @default "Sort Ascending"
*/
sortFieldAscendingText?: string;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid, this
*  attribute will be shown as the menu item title to sort a field in descending order.
* @type {string}
* @default "Sort Descending"
*/
sortFieldDescendingText?: string;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid, this
*  attribute will be shown as the menu item title to clear an existing sort on this field.
* @type {string}
* @default "Clear Sort"
*/
clearSortFieldText?: string;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid, this
*  attribute will be shown as the menu item title to clear any existing sort on all fields. This
*  menu-item is displayed only in the context menu for the sorter button.
* @type {string}
* @default "Clear All Sorting"
*/
clearAllSortingText?: string;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid,
*  and a ListGrid.showFilterEditor is visible, this
*  attribute will be shown as the menu item title to clear any existing filter. This
*  menu-item is displayed only in the context menu for the sorter button.
* @type {string}
* @default "Clear Filter"
*/
clearFilterText?: string;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid,
*  and a ListGrid.showFilterEditor is visible and
*  ListGrid.allowFilterWindow is enabled, this
*  attribute will be shown as the menu item title to configure advanced filtering. This
*  menu-item is displayed in the context menu for the sorter button and in the 
*  ListGrid.allowFilterOperators operators menu.
* @type {string}
* @default "Advanced filtering..."
*/
advancedFilteringText?: string;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid,
*  and multi-sorting is enabled, this attribute is used as the title for a menu item that opens
*  a MultiSortDialog to configure the sort-specification for this grid. This
*  menu-item is displayed only in the context menu for the sorter button.
* @type {string}
* @default "Configure Sort..."
*/
configureSortText?: string;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid,
*  and multi-grouping is enabled, this attribute is used as the title for a menu item that opens
*  a MultiGroupDialog to configure the grouping for this grid.
* @type {string}
* @default "Configure Grouping..."
*/
configureGroupingText?: string;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid,
*  and user-driven auto fit of fields is enabled via ListGridField.canAutoFitWidth or 
*  ListGrid.canAutoFitFields, this attribute will be shown as the menu
*  item title for an item to perform a one-time autoFit of the field to its title or content
*  via a call to ListGrid.autoFitField.
* @type {string}
* @default "Auto Fit"
*/
autoFitFieldText?: string;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid,
*  and ListGrid.canAutoFitFields is true, this attribute will be shown as the menu
*  item title for an item to perform a one-time autoFit of all visible fields via
*  the ListGrid.autoFitField method.
* @type {string}
* @default "Auto Fit All Columns"
*/
autoFitAllText?: string;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid, and
*  ListGrid.canPickFields is true, this
*  attribute will be shown as the title for the menu item which contains a submenu with items
*  allowing the user to show and hide fields in the grid.
* @type {string}
* @default "Columns"
*/
fieldVisibilitySubmenuTitle?: string;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid and
*  ListGrid.canFreezeFields is true, this string will be shown as the
*  title for the menu item to freeze a currently unfrozen field.
*  
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with title available as a variable containing
*  the field title.
*  
*  Default value returns "Freeze " + the field's summary title.
* @type {HTMLString}
* @default "Freeze ${title}"
*/
freezeFieldText?: HTMLString;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid and
*  ListGrid.canFreezeFields is true, this string will be shown as the
*  title for the menu item to unfreeze a currently frozen field.
*  
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with title available as a variable containing
*  the field title.
*  
*  Default value returns "Unfreeze " + the field's summary title.
* @type {HTMLString}
* @default "Unfreeze ${title}"
*/
unfreezeFieldText?: HTMLString;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid and
*  ListGrid.canGroupBy is true, this string will be shown as the
*  title for the menu item to toggle the group by setting for a field.
*  
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with title available as a variable containing
*  the field title.
*  
*  Default value returns "Group by " + the field's summary title.
* @type {HTMLString}
* @default "Group by ${title}"
*/
groupByText?: HTMLString;

/**
* If we're showing a ListGrid.showHeaderContextMenu for this grid, and
*  ListGrid.isGrouped is true, this
*  attribute will be shown as the title for the menu item to ungroup the grid.
* @type {string}
* @default "Ungroup"
*/
ungroupText?: string;

/**
* Adds the ability for a user to define additional criteria above and beyond those expressed
*  in the ListGrid.showFilterEditor via a FilterBuilder which appears
*  in a modal Window over the grid and can be accessed by various menus within the grid or
*  triggered by external controls.
*  
*  Causes a menu item titled ListGrid.advancedFilteringText to appear in the
*  ListGrid.filterUsingText menu show in the ListGrid.showHeaderContextMenu
*  that allows the end user to configure an advanced filter on the grid that can supplement the
*  ListGrid.showFilterEditor. Note that the menu option will show even
*  if ListGrid.allowFilterOperators is disabled.
*  
*  To use this feature, the grid must be configured with a DataSource. In fact, this
*  feature is enabled by default if the grid has a DataSource and both
*  DataSource.supportsAdvancedCriteria and ListGrid.allowFilterOperators are true.
*  This default can be disabled by setting allowFilterWindow to
*  false. 
*  
*  This example
*  shows the allowFilterWindow setting in use.
*  
*  Note: this feature requires
*  SmartClient Pro or better.
* @type {boolean}
* @default null
*/
allowFilterWindow?: boolean;

/**
* Instance of Window used to ListGrid.showFilterWindow the
*  ListGrid.filterWindowFilter.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
*  
*  By default the FilterBuilder shows with the top operator selection as a
*  radio group and allows switching between simple and advanced modes. These defaults can
*  be changed using autoChildUsage features such as setting
*  ListGrid.filterWindowFilter properties on a grid instance or globally
*  by changing the ListGrid defaults.
*  
*  
*  For example, in JavaScript, to always use advanced mode on a single grid:
*  
*  var grid = isc.ListGrid.create({
*    ...
*    filterWindowFilterProperties: {
*      topOperatorAppearance: "bracket",
*      showModeSwitcher: false
*    }
*  });
*  
*  or to always use advanced mode:
*  
*  isc.ListGrid.changeDefaults("filterWindowFilter", {
*    topOperatorAppearance: "bracket",
*    showModeSwitcher: false
*  });
*  
*  
*  
* @type {Window}
* @default null
*/
filterWindow?: Window;

/**
* Instance of FilterBuilder shown in ListGrid.filterWindow by ListGrid.showFilterWindow.
*  See ListGrid.filterWindow for more information on the filter defaults and changing them.
* @type {FilterBuilder}
* @default null
*/
filterWindowFilter?: FilterBuilder;

/**
* The instruction text to display at the top of the ListGrid.filterWindow.
* @type {HTMLString}
* @default "Enter criteria below. These criteria are &lt;i&gt;in addition to&lt;/i&gt; any criteria entered in the filter immediately above column headers."
*/
filterWindowInstructions?: HTMLString;

/**
* Advanced filtering criteria, either Criteria or
*  AdvancedCriteria, that is combined with the 
*  ListGrid.getFilterEditorCriteria during filtering.
*  
*  This criteria is normally configured via ListGrid.showFilterWindow
*  shown because of the ListGrid.allowFilterWindow option but can be assigned directly as well.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
* @type {Criteria}
* @default null
*/
filterWindowCriteria?: Criteria;

/**
* Should an indicator be shown to indicate that ListGrid.filterWindowCriteria is configured?
*  The indicator is a small triangle shown in the top-right of the grid header or filter.
* @type {boolean}
* @default true
*/
showFilterWindowCriteriaIndicator?: boolean;

/**
* Instance of Canvas used to show visual indicator that ListGrid.filterWindowCriteria
*  is configured. ListGrid.showFilterWindowCriteriaIndicator must be enabled to show indicator.
* @type {Canvas}
* @default null
*/
filterWindowCriteriaIndicator?: Canvas;

/**
* The color of the ListGrid.showFilterWindowCriteriaIndicator.
* @type {string}
* @default "#0066cc"
*/
criteriaIndicatorColor?: string;

/**
* The color of the ListGrid.showFilterWindowCriteriaIndicator
*  when shown on the ListGrid.sorterConstructor or the last
*  ListGrid.headerButtonConstructor in the grid header. The default is to use
*  ListGrid.criteriaIndicatorColor.
* @type {string}
* @default null
*/
criteriaIndicatorHeaderColor?: string;

/**
* When declared, the specified form is automatically used as a search form for this grid, 
*  with criteria that are additive with any criteria shown in the FilterEditor or 
*  ListGrid.allowFilterWindow.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
*  
*  This is similar to the effect of adding a ListGrid.filterEditorSubmit override that pulls 
*  in criteria from the external form, and having the external form call 
*  ListGrid.filterByEditor instead of ListGrid.fetchData, as shown in the 
*  +exampleLink{additiveFilter} example.
*  
*  In particular. the grid will automatically filter when the searchForm.search() or 
*  dynamicForm.submit() event fires on the form (happens if a SubmitItem is present and is 
*  pressed), and will automatically trigger filtering if enter is pressed in the form, as 
*  though searchOnEnter / saveOnEnter had been set.
*  
*  If the FilterEditor is enabled and listGrid.filterOnKeypress is set, the grid will 
*  automatically watch for searchForm.criteriaChanged(), and filter whenever that method 
*  fires. For the purposes of this behavior, the FilterEditor is considered to be enabled 
*  even if it is not currently visible but ListGrid.canShowFilterEditor is true (as 
*  otherwise filtering behavior in the form would change when the FilterEditor appears).
*  
*  The criteria from the specified searchForm will only be shown and edited in
*  the external form; they will neither be shown nor editable in the FilterEditor, even if 
*  the FilterEditor also allows criteria on the same fields. That is, they are treated 
*  much like ListGrid.implicitCriteria.
*  
*  Note: when using listGrid.searchForm don't add your own logic to call 
*  fetchData() or the criteria you specify will appear in the FilterEditor and be editable 
*  there, potentially creating user confusion, especially if some of the criteria in your 
*  form cannot be displayed and edited in the FilterEditor.
* @type {DynamicForm | ValuesManager}
* @default null
*/
searchForm?: DynamicForm | ValuesManager;

/**
* When set to false, no hover is shown for the filter editor fields. Otherwise, a hover shows
*  the current field's criteria description along with the ListGrid.filterWindowCriteria
*  description if configured.
*  
*  Hovers can also be disabled for individual fields using
*  ListGridField.showFilterEditorHovers.
*  
*  The descriptive text for criteria is formatted by DataSource.getAdvancedCriteriaDescription.
* @type {boolean}
* @default null
*/
showFilterEditorHovers?: boolean;

/**
* The field criteria prefix to show in filter editor field hover before the descriptive
*  version of the field's criteria, if any. The descriptive text is formatted by
*  DataSource.getAdvancedCriteriaDescription.
* @type {HTMLString}
* @default "Field criteria:"
*/
fieldCriteriaText?: HTMLString;

/**
* The additional criteria prefix to show in filter editor field hover, the filter action
*  button or the sorter button before the descriptive version of the +{filterWindowCriteria},
*  if any. The descriptive text is formatted by DataSource.getAdvancedCriteriaDescription.
* @type {HTMLString}
* @default "Grid additional criteria:"
*/
gridAdditionalCriteriaText?: HTMLString;

/**
* When true, indicates that this ListGrid supports sorting on multiple fields. Note that even
*  when set to true, multi-field sorting may not be available if the grid is databound and the
*  ${isc.DocUtils.linkForRef('attr:DataSource.canMultiSort','DataSource doesn\'t support multi-sort')}, or if sorting for a
*  field is ListGridField.canSortClientOnly but not all data is available.
* @type {boolean}
* @default true
*/
canMultiSort?: boolean;

/**
* Class level defaults to apply to the MultiSortDialog which gets automatically
*  generated when DataBoundComponent.askForSort is called.
*  
*  See also ListGrid.showHeaderSpanTitlesInSortEditor and ListGrid.sortEditorSpanTitleSeparator
* @type {Partial<MultiSortDialog>}
* @default null
*/
multiSortDialogDefaults?: Partial<MultiSortDialog>;

/**
* Properties to apply to the MultiSortDialog which gets automatically generated
*  when DataBoundComponent.askForSort is called.
*  
*  See also ListGrid.showHeaderSpanTitlesInSortEditor and ListGrid.sortEditorSpanTitleSeparator
* @type {Partial<MultiSortDialog>}
* @default null
*/
multiSortDialogProperties?: Partial<MultiSortDialog>;

/**
* When true, indicates that this ListGrid supports grouping on multiple fields.
* @type {boolean}
* @default false
*/
canMultiGroup?: boolean;

/**
* An array of SortSpecifier objects used to set up the initial sort configuration for
*  this grid. If specified, this will be used instead of any ListGrid.sortField specified.
* @type {Array<Partial<SortSpecifier>>}
* @default null
*/
initialSort?: Array<Partial<SortSpecifier>>;

/**
* When multiple fields are sorted, the Style to apply to the numeral that appears after the
*  sort-arrows in the header-buttons of sorted fields.
* @type {CSSStyleName}
* @default "sortNumeral"
*/
sortNumeralStyle?: CSSStyleName;

/**
* When multiple fields are sorted, set this to false to hide the sort-numeral
*  displayed by default after the sort-arrows in the header-buttons of sorted fields.
* @type {boolean}
* @default null
*/
showSortNumerals?: boolean;

/**
* Screen to create (via RPCManager.createScreen) in lieu of calling
*  ListGrid.getRecordComponent.
*  
*  If this grid has a ListGrid.dataSource, the created screen is
*  provided with a Canvas.dataContext that includes the record being shown at the row.
*  Be sure the record screen meets these Canvas.autoPopulateData
*  to utilize the dataContext.
* @type {string}
* @default null
*/
recordScreen?: string;

/**
* Name of the SmartClient Class to be used when creating charts. Must support the
*  Chart interface.
* @type {string}
* @default "FacetChart"
*/
chartConstructor?: string;

/**
* Default type of chart to plot.
* @type {ChartType}
* @default "Column"
*/
chartType?: ChartType;

/**
* If this is ListGrid.groupBy and has 
*  frozenFields, should the group title show in the
*  frozen or unfrozen body?
*  
*  Setting this property to false will cause the group title to show in the 
*  unfrozen body in this case, meaning it will appear to the right of the frozen fields,
*  and scroll horizontally as the user scrolls the unfrozen fields. 
*  This can be useful for grids where there isn't
*  enough available space to show the group title text in the frozen body.
*  
*  Note that if ListGrid.groupTitleField is explicitly set, or 
*  ListGrid.showGroupSummaryInHeader is true, this property has no
*  effect. In this case rather than the group title showing in a single cell spanning
*  multiple other fields, it will be rendered into a specific column.
* @type {boolean}
* @default true
*/
showGroupTitleInFrozenBody?: boolean;

/**
* If this listGrid supports ListGrid.canGroupBy, setting this property will cause
*  the grid to render an extra row at the end of each group when grouped, containing summary
*  information for the fields.
*  Summary information will be calculated by the ListGridField.getGroupSummary method
*  if specified, otherwise via the specified ListGridField.summaryFunction.
* @type {boolean}
* @default false
*/
showGroupSummary?: boolean;

/**
* If this grid is ListGrid.groupByField, and
*  ListGrid.showGroupSummary is true, this attribute may be set to an array of
*  groupBy field names for which group summaries should appear.
*  
*  This is particularly useful for listGrids grouped by more than one field as it allows developers
*  to display the group summary for a particular nested group without showing a summary for every
*  level of the tree.
* @type {Array<Partial<string>>}
* @default null
*/
groupByFieldSummaries?: Array<Partial<string>>;

/**
* ListGridRecord.customStyle for the group-level summary row displayed when
*  ListGrid.showGroupSummary is true.
* @type {CSSStyleName}
* @default "gridSummaryCell"
*/
groupSummaryStyle?: CSSStyleName;

/**
* Class-level defaults to apply to the MultiGroupDialog which gets automatically 
*  generated when ListGrid.configureGrouping is called.
* @type {Partial<MultiGroupDialog>}
* @default null
*/
multiGroupDialogDefaults?: Partial<MultiGroupDialog>;

/**
* Properties to apply to the MultiGroupDialog which gets automatically 
*  generated when ListGrid.configureGrouping is called.
* @type {Partial<MultiGroupDialog>}
* @default null
*/
multiGroupDialogProperties?: Partial<MultiGroupDialog>;

/**
* If set to true, an advanced field picker based on the FieldPicker will be shown instead
*  of the column picker submenu if there are more fields in the grid than
*  ListGrid.advancedFieldPickerThreshold.
*  
*  When there are large numbers of available fields, the FieldPicker-based interface is more
*  usable for both defining visible fields and defining field order.
* @type {boolean}
* @default false
*/
useAdvancedFieldPicker?: boolean;

/**
* If set to true, the ListGrid.useAdvancedFieldPicker provides
*  an interface allowing users to modify fields' titles.
*  
*  Note that when enabled, the ListGrid.getFieldState for this component
*  will include field titles by default (see DataBoundComponent.shouldIncludeTitleInFieldState).
* @type {boolean}
* @default false
*/
canEditTitles?: boolean;

/**
* Instance of FieldPickerWindow used if ListGrid.useAdvancedFieldPicker is set.
* @type {FieldPickerWindow}
* @default null
*/
fieldPickerWindow?: FieldPickerWindow;

/**
* Names of properties on ListGridField for which the FieldPicker should
*  show an editing interface, for convenience.
*  
*  For example, specify ["frozen", "decimalPrecision"] to allow end users to modify
*  ListGridField.frozen and ListGridField.decimalPrecision respectively.
* @type {Array<Partial<string>>}
* @default null
*/
fieldPickerFieldProperties?: Array<Partial<string>>;

/**
* When ListGrid.useAdvancedFieldPicker is set, total number of available fields that must be
*  present in the grid before the advanced field picker interface is used instead of the normal
*  columns submenu.
*  
*  Set to 0 to have the advanced picker always used (when useAdvancedFieldPicker is true).
* @type {Integer}
* @default "25"
*/
advancedFieldPickerThreshold?: Integer;

/**
* When set to false, sample values of the FieldPicker are never shown. This property applies to the entire FieldPicker.
* @type {boolean}
* @default true
*/
fieldPickerShowSampleValues?: boolean;

/**
* When AutoTest.getElement is used to parse locator strings generated by
*  AutoTest.getLocator for a cell in this grid, how should the row be identified?
*  
*  Note that getLocator() will actually store all available information about the row in
*  the generated string -- this attribute effects how a stored string will be parsed only.
*  
*  Valid options area:
*  
*  
* - "primaryKey" Only applies to databound grids: If the cell in question has
*  a primary key cell value, use it to identify cells in autoTest locator strings.
*  
* - "titleField" If the cell in question has a value for the
*  ListGrid.getTitleField, use it to identify cells in autoTest
*  locator strings
*  
* - "targetCellValue" Identify rows by storing the cell value for the target
*  row / field in autoTest locator strings
*  
* - "index"The rowNum will be used to identify the row.
*  
*  If unset, default behavior is to identify by primary key (if available), otherwise by
*  titleField (if available), otherwise by cell value (if available), and lastly by index.
* @type {string}
* @default null
*/
locateRowsBy?: string;

/**
* When AutoTest.getElement is used to parse locator strings generated by
*  AutoTest.getLocator for a cell in this grid, how should the column be identified?
*  
*  Note that getLocator() will actually store all available information about the column in
*  the generated string -- this attribute effects how a stored string will be parsed only.
*  
*  Valid options area:
*  
*  
* - "fieldName" Attempt to identify by fieldName.
*  
* - "index" Attempt to identify by colNum (index in the fields array).
*  
*  If unset, default behavior is to identify by fieldName (if available), otherwise by index.
* @type {string}
* @default null
*/
locateColumnsBy?: string;



/**
*  Retrieves data from the DataSource that matches the specified criteria.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
*  
*  When fetchData() is first called, if data has not already been provided via
*  ListGrid.setData, this method will create a ResultSet, which will be
*  configured based on component settings such as DataBoundComponent.fetchOperation
*  and DataBoundComponent.dataPageSize, as well as the general purpose
*  ListGrid.dataProperties. The created ResultSet will automatically send a DSRequest
*  to retrieve data from ListGrid.dataSource, and from then on will 
*  automatically manage paging through large datasets, as well as performing filtering and
*  sorting operations inside the browser when possible - see the ResultSet docs for
*  details.
*  
*  NOTE: do not use both DataBoundComponent.autoFetchData and a
*  call to fetchData() - this may result in two DSRequests to fetch data. Use
*  either DataBoundComponent.autoFetchData and Criteria or a manual call to fetchData()
*  passing criteria.
*  
*  Whether a ResultSet was automatically created or provided via ListGrid.setData, subsequent
*  calls to fetchData() will simply call ResultSet.setCriteria.
*  
*  Changes to criteria may or may not result in a DSRequest to the server due to
*  ResultSet.useClientFiltering. You can call
*  DataBoundComponent.willFetchData to determine if new criteria will result in a
*  server fetch.
*  
*  If you need to force data to be re-fetched, you can call
*  ListGrid.invalidateCache and new data will automatically be fetched
*  from the server using the current criteria and sort direction. NOTE: when using
*  invalidateCache() there is no need to also call fetchData()
*  and in fact this could produce unexpected results.
*  
*  This method takes an optional callback parameter (set to a DSCallback) to fire when
*  the fetch completes. Note that this callback will not fire if no server fetch is performed.
*  In this case the data is updated synchronously, so as soon as this method completes you
*  can interact with the new data. If necessary, you can use
*  DataBoundComponent.willFetchData to determine whether or not a server
*  fetch will occur when fetchData() is called with new criteria.
*  
*  In addition to the callback parameter for this method, developers can use 
*  ListGrid.dataArrived to be notified every time data is loaded.
*  
*  By default, this method assumes a TextMatchStyle of "exact"; that can be overridden
*  by supplying a different value in the requestProperties parameter. 
*  See DataBoundComponent.willFetchData;
*  
*  Changing the request properties
*  
*  Changes to TextMatchStyle made via requestProperties will be honored in
*  combination with the fetch criteria, possibly invalidating cache and triggering a server
*  request if needed, as documented for DataBoundComponent.willFetchData.
*  In contrast, changes to
*  DSRequest.operationId in the request properties will cause
*  the ResultSet or ResultTree to be rebuilt, always refetching from the server.
*  However, changes to other request properties after the initial fetch won't be detected, and
*  no fetch will get triggered based on that new request context.
*  
*  To pick up such changes, we recommend that you call ListGrid.setData (passing an
*  empty array to ensure the data model is cleared), and then call this method to fetch again.
*  If you try to do it by calling ListGrid.invalidateCache, you may see duplicate fetches if you
*  haven't already updated the data context by calling this method with the new request
*  properties, and fail to do so before the component is Canvas.redraw.
* 
* 
* @param {Criteria=} Search criteria. If a DynamicForm is passed                     in as this argument instead of a raw criteria                      object, will be derived by calling                      DynamicForm.getValuesAsCriteria
* @param {DSCallback=} callback to invoke when a fetch is complete. Fires                     only if server contact was required
* @param {DSRequest=} additional properties to set on the DSRequest                     that will be issued
*/
fetchData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Sends the current filter criteria and sort direction to the server, then exports data in the
*  requested DSRequest.exportAs.
*  
*  A variety of DSRequest settings, such as 
*  DSRequest.exportAs and DSRequest.exportFilename, affect the 
*  exporting process: see DSRequest.exportResults for further detail.
*  
*  Note that data exported via this method skips client-side fields defined only in
*  the component, excludes any client-side formatting and
*  relies on both the SmartClient server and server-side DataSources. To export client-data 
*  including client-only fields and with client-side formatting applied, 
*  see ListGrid.exportClientData, which still requires the
*  SmartClient server but does not rely on server-side DataSource definitions (.ds.xml files).
*  
*  For more information on exporting data, see DataSource.exportData.
* 
* 
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
* @param {DSCallback=} callback to invoke on completion. Note that this parameter                 only applies where DSRequest.exportToClient is                  explicitly set to false, because file downloads do not                  provide ordinary SmartClient callbacks
*/
exportData(requestProperties?:DSRequest, callback?:DSCallback): void;

/**
*  Retrieves data that matches the provided criteria and displays the matching data in this
*  component.
*  
*  This method behaves exactly like ListGrid.fetchData except that
*  DSRequest.textMatchStyle is automatically set to "substring" so that String-valued
*  fields are matched by case-insensitive substring comparison.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
* 
* @param {Criteria=} Search criteria.            If a DynamicForm is passed in as this argument            instead of a raw criteria object, will be derived by calling            DynamicForm.getValuesAsCriteria
* @param {DSCallback=} callback to invoke when a fetch is complete. Fires                     only if server contact was required; see                      ListGrid.fetchData for details
* @param {DSRequest=} for databound components only - optional              additional properties to set on the DSRequest that will be issued
*/
filterData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Based on the relationship between the DataSource this component is bound to and the
*  DataSource specified as the "schema" argument, call fetchData() to retrieve records in this
*  grid that are related to the passed-in record.
*  
*  Relationships between DataSources are declared via DataSourceField.foreignKey.
*  
*  For example, given two related DataSources "orders" and "orderItems", where we want to fetch
*  the "orderItems" that belong to a given "order". "orderItems" should declare a field that
*  is a DataSourceField.foreignKey to the "orders" table (for example, it
*  might be named "orderId" with foreignKey="orders.id"). Then, to load the records related to
*  a given "order", call fetchRelatedData() on the component bound to "orderItems", pass the
*  "orders" DataSource as the "schema" and pass a record from the "orders" DataSource as the
*  "record" argument.
*  
*  Note that multiple foreign keys into the schema are supported by this method.
* 
* 
* @param {ListGridRecord} DataSource record
* @param {Canvas | DataSource | GlobalId} schema of the DataSource record, or              DataBoundComponent already bound to that schema
* @param {DSCallback=} callback to invoke on completion
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
*/
fetchRelatedData(record:ListGridRecord, schema:Canvas | DataSource | GlobalId, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Clear the current criteria used to filter data. This method clears filter-values from 
*  fields in the ListGrid.filterEditor but will not change their current 
*  ListGridField.filterOperator or clear associated
*  ListGrid.allowFilterOperators. See ListGrid.clearAllCriteria
*  for a means of doing that.
* 
* 
* @param {DSCallback=} callback to invoke on completion
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
*/
clearCriteria(callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Retrieves a copy of the explicit criteria currently applied to this component, including
*  criteria provided by a grid's ListGrid.filterWindow or ListGrid.searchForm, 
*  but not criteria applied DataBoundComponent.implicitCriteria. 
*  May return null.
*  
*  Note: if ListGrid.showFilterEditor is true, the criteria returned by this method may not
*  match the values currently displayed in the filter editor, since the user may have entered
*  values which have not yet been applied to the data. ListGrid.getFilterEditorCriteria
*  may be used to retrieve the current criteria displayed in the filterEditor.
* 
*/
getCriteria(): Criteria;

/**
*  Sets this component's filter criteria.
*  Default implementation calls this.data.setCriteria().
*  
*  Note: if ListGrid.showFilterEditor is true, the ListGrid.setFilterEditorCriteria
*  method may be used to update the values displayed in the filter editor without effecting the
*  data object.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
* 
* @param {Criteria | AdvancedCriteria} new criteria to show
*/
setCriteria(criteria:Criteria | AdvancedCriteria): void;

/**
*  Invalidate the current data cache for this databound component via a call to
*  the dataset's invalidateCache() method, for example,
*  ResultSet.invalidateCache.
*  
*  NOTE: there is no need to call invalidateCache() when a save operation
*  is performed on a DataSource. Automatic cache synchronization features will automatically
*  update caches - see ResultSet for details. If automatic cache synchronization isn't
*  working, troubleshoot the problem using the steps suggested 
*  in the FAQ rather
*  than just calling invalidateCache(). Calling invalidateCache() unnecessarily
*  causes extra server load and added code complexity.
*  
*  Calling invalidateCache() will automatically cause a new fetch to 
*  be performed with the current set of criteria if data had been previously fetched and the
*  component is currently drawn with data visible - there is no need to manually call
*  fetchData() after invalidateCache() and this could result in duplicate fetches. 
*  
*  While data is being re-loaded after a call to invalidateCache(), the widget is
*  in a state similar to initial data load - it doesn't know the total length of the dataset
*  and any APIs that act on records or row indices will necessarily fail and should not be
*  called. To detect that the widget is in this state, call ResultSet.lengthIsKnown.
*  
*  invalidateCache() only has an effect if this components dataset is a data
*  manager class that manages a cache (eg ResultSet or ResultTree). If data was provided as a
*  simple Array or List, invalidateCache() does nothing.
*  
* 
*/
invalidateCache(): void;

/**
*  Unlike ListGrid.invalidateCache this will perform an asynchronous
*  (background) refresh of this component's data and then call the provided callback method on
*  completion. A grid needs to have a DataSource associated with it to use this method.
*  
*  If refreshData() is called while the grid is waiting for a response from
*  ListGrid.fetchData the refreshData() call will be aborted. This is
*  because the fetch has higher priority.
*  
*  If ListGrid.fetchData is called while the grid is waiting for a response from
*  refreshData() and the fetchData() call has altered the criteria
*  or sort specifiers, the refreshData() call will be aborted.
*  
*  If data is being edited or has been edited without being saved when
*  refreshData() is called, the data will be retained so you can save it after the
*  refresh is complete. If you however want to throw away your edited but unsaved data when
*  calling refreshData() you first need to call ListGrid.discardAllEdits
*  which will discard any unsaved edited data.
*  
*  Note that for a TreeGrid with TreeGrid.loadDataOnDemand: true, all currently opened
*  parent nodes will be re-fetched, except for TreeGrid.dataFetchMode TreeGrids,
*  for which only opened parent nodes that are visible or contain visible children
*  are re-fetched. We do this in a single queued batch of fetches to maximize efficiency.
*  
*  Except for changes to the dataset length, ListGrid.dataChanged is
*  not fired after refreshData(), as the Framework is not in a position to know
*  for sure if data has actually changed (which would require traversing the entire dataset to
*  determine) and whether criteria, sort or other specifiers of the dataset also have not
*  changed. Applications that need to take action on refreshData() should use the
*  callback to do so.
* 
* 
* @param {DSCallback=} callback method to run once the refresh completes.
*/
refreshData(callback?:DSCallback): void;

/**
*  Compares the specified criteria with the current criteria applied to this component's
*  data object and determines whether the new criteria could be satisfied from the currently
*  cached set of data, or if a new filter/fetch operation will be required.
*  
*  This is equivalent to calling this.data.willFetchData(...).
*  Always returns true if this component is not showing a set of data from the dataSource.
*  
*  Note that to predict correctly the decision that will be made by filter/fetch, you'll need to
*  pass the same TextMatchStyle that will be used by the future filter/fetch. Fetching
*  manually (e.g. ListGrid.fetchData) will by default use "exact" while filtering
*  (e.g. ListGrid.filterData) will by default use "substring". If the component
*  is configured for autofetch (i.e. ListGrid.autoFetchData: true), that will
*  use ListGrid.autoFetchTextMatchStyle, which defaults to "substring". If nothing/null
*  is passed for the style, this method assumes you want the style from the last filter/fetch.
*  
*  To determine what TextMatchStyle is being used, check the RPC Tab of the
*  debugging and check the relevant DSRequest. 
*  
* 
* @param {Criteria} new criteria to test.
* @param {TextMatchStyle=} New text match style. If not passed assumes    textMatchStyle will not be modified.
*/
willFetchData(newCriteria:Criteria, textMatchStyle?:TextMatchStyle): boolean;

/**
*  Perform a DataSource "add" operation to add new records to this component's DataSource.
* 
* 
* @param {Record} new record
* @param {DSCallback=} method to call on operation completion
* @param {Partial<DSRequest>=} additional properties to set on the DSRequest                     that will be issued
*/
addData(newRecord:Record, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Perform a DataSource "update" operation to update existing records in this component's
*  DataSource.
* 
* 
* @param {Record} updated record
* @param {DSCallback=} method to call on operation completion
* @param {Partial<DSRequest>=} additional properties to set on the DSRequest                     that will be issued
*/
updateData(updatedRecord:Record, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Remove the currently selected records from this component.
*  If this is a databound grid, the records will be removed directly from the DataSource.
*  The grid will automatically be updated if the record deletion succeeds.
*  
*  If no records are selected, no action is taken and neither callback will be called.
*  
*  For a grid with no DataSource or where saveLocally is true, the data removal
*  is performed on the client and both callbacks will fire with no arguments.
* 
* 
* @param {DSCallback=} callback to fire when each record has been removed
* @param {DSRequest=} additional properties to set on the DSRequest                     that will be issued
* @param {RPCQueueCallback=} callback to fire after all selected data has                      been removed
*/
removeSelectedData(callback?:DSCallback, requestProperties?:DSRequest, queueCallback?:(response:Array<Partial<RPCResponse>>) => void): void;

/**
*  Returns the first selected record in this grid.
*  
*  This method is appropriate if the ListGrid.selectionType is
*  "single",
*  
*  or if you only care about the first selected record in a multiple-record selection. To access
*  all selected records, use ListGrid.getSelection instead.
*  
*  NOTE: If a record is returned, it should be treated as read-only and not
*  modified.
* 
*/
getSelectedRecord(): ListGridRecord;

/**
*  Returns all selected records in this grid.
*  
*  NOTE: Records in the returned array should be treated as read-only and not
*  modified.
* 
* @param {boolean=} When true, partially selected records will not be returned.                  Otherwise, both fully and partially selected records are                  returned.
*/
getSelectedRecords(excludePartialSelections?:boolean): Array<Partial<ListGridRecord>>;

/**
*  Returns true if the record is selected.
*  
* 
* @param {ListGridRecord} record to check
*/
isSelected(record:ListGridRecord): boolean;

/**
*  When using tree-oriented selection modes like TreeGrid.cascadeSelection, returns true
*  if the record is considered partially selected because only some of it's children are
*  selected.
*  
* 
* @param {ListGridRecord} record to check
*/
isPartiallySelected(record:ListGridRecord): boolean;

/**
*  When exporting data to Excel/OpenOffice format using ListGrid.exportData or
*  ListGrid.exportClientData, background color to use 
*  for the cell at the given rowNum and colNum.
*  
*  See exportBGColor for an overview.
* 
* 
* @param {Integer} row number of cell
* @param {Integer} column number of cell
* @param {Record} the record object behind the row being exported
*/
getExportBGColor(rowNum:Integer, colNum:Integer, record:Record): CSSColor;

/**
*  When exporting data to Excel/OpenOffice format using ListGrid.exportData or
*  ListGrid.exportClientData, background color to use 
*  for the given rowNum.
*  
*  See exportBGColor for an overview.
* 
* 
* @param {Integer} row number
* @param {Record} the record object behind the row being exported
*/
getExportRowBGColor(rowNum:Integer, record:Record): CSSColor;

/**
*  When exporting data to Excel/OpenOffice format using ListGrid.exportData or
*  ListGrid.exportClientData, background color to use 
*  for the given colNum.
*  
*  See exportBGColor for an overview.
* 
* 
* @param {Integer} column number
*/
getExportColumnBGColor(colNum:Integer): CSSColor;

/**
*  Exports this component's data with client-side formatters applied, so is suitable for direct 
*  display to users, using the specified DSRequest.exportAs.
*  
*  A variety of DSRequest settings, such as DSRequest.exportAs and 
*  DSRequest.exportFilename, affect the exporting process: 
*  see DSRequest.exportResults for further detail.
*  
*  If this component is DataBoundComponent.setDataSource and not all records 
*  that match the current ListGrid.getCriteria have 
*  ResultSet.allMatchingRowsCached, you can call
*  ListGrid.loadAllRecords - this method accepts a callback which is fired when all
*  necessary data has arrived. When that callback fires, a call to 
*  exportClientData will have access to the full dataset for the filter.
*  
*  This feature requires the SmartClient server.
*  
*  If your ListGrid has custom formatters, formatted values will be exported by default, with 
*  HTML normalized to text where possible. Since some levels of HTML normalizing aren't possible, 
*  this may result in missing or incorrect export values. In this case, you have three options:
*  
*  
* - Set ListGridField.exportRawValues on the field. This will export
*    the raw underlying value of the field; your formatter will not be called
*  
* - Have your formatter call ListGrid.isExportingClientData
*    and perform whatever alternative formatting you require if that method returns true
*  
* - Set ListGridField.exportRawNumbers on the field. This will export
*    the raw underlying number of the field; your formatter will not be called
*  
*  Note that during export, the ListGridField.escapeHTML setting on a field determines
*  how escaped and unescaped HTML values are handled. In particular, if
*  escapeHTML is not set for a field, a value like "&lt;FOO&gt;" will be
*  exported as the empty string, and you'd need the escaped value "&amp;lt;FOO&amp;gt;" to end
*  up exporting "&lt;FOO&gt;".
*  
*  Ordinarily, calls to this method go through the static classMethod 
*  DataSource.exportClientData. In this case, no server-side DataSources
*  are required. However, if this component is 
*  DataBoundComponent.setDataSource and you specify a valid 
*  DSRequest.operationId in the properties passed to this method, the call
*  will go through the instance method DataSource.exportClientData instead. As the 
*  documentation for that method explains, this allows you more control on the server side. 
*  This approach requires both the SmartClient server and server-side DataSource definitions.
*  
*  To export data from this component's dataSource, 
*  see DataBoundComponent.exportData, which does not include client-side 
*  formatters, but does include formatters declared in the .ds.xml file.
*  exportData() relies on both the SmartClient server and server-side DataSources.
* 
* @param {Partial<DSRequest>=} Request properties for the export. Note that specifying DSRequest.exportData on the request properties  allows the developer to pass in an explicit data set to export.
* @param {RPCCallback=} Optional callback. If you specify DSRequest.exportToClient: false in the request  properties, this callback will fire after export completes. Otherwise the callback will  fire right before the download request is made to the server.
*/
exportClientData(requestProperties?:Partial<DSRequest>, callback?:(response:RPCResponse, rawData:any, request:RPCRequest) => void): void;

/**
*  Returns true if this component is currently 
*  ListGrid.exportClientData. This method can be called from
*  custom cell formatters if you need to return a different formatted value for an export
*  than for a live ListGrid
* 
*/
isExportingClientData(): boolean;

/**
*  Dynamically retrieves the aria properties to write out for this listGrid in
*  isc.setScreenReaderMode.
*  
*  If ListGrid.ariaRole is set to "grid" this will return an object
*  with rowcount and colcount attributes populated to indicate
*  this size of the grid and its data.
*  
*  If a static Canvas.ariaState has been specified, the default implementation
*  will apply these dynamically derived properties in addition to any properties specified
*  on the static object.
*  
*  Note that Canvas.redraw the grid will re-evaluate this method and
*  apply the result to the handle.
*  
* 
*/
getAriaState(): object;

/**
*  Retrieves dynamically calculated default Canvas.ariaState 
*  properties for this listGrid. These will be combined with explicitly specified aria
*  state as described in ListGrid.getAriaState.
*  
*  Overridden by ListGrid to pick up aria-rowcount and aria-colcount.
*  
* 
*/
getAriaStateDefaults(): object;

/**
*  Returns the WAI ARIA role for cells within this listGrid.
*  
*  If the record has a value for the ListGrid.recordCellRoleProperty, this
*  will be respected.
*  Otherwise if ListGrid.cellRole is specified, it will be used.
*  
*  If neither property is set, the default implementation will return "gridcell" 
*  if this listGrid has ListGrid.ariaRole, otherwise null, meaning
*  no role will be written out for cells.
* 
* 
* @param {Integer} row index of the cell
* @param {Integer} column index of the cell
* @param {ListGridRecord} Record for the cell in question
*/
getCellRole(rowNum:Integer, colNum:Integer, record:ListGridRecord): string;

/**
*  Returns a map of Canvas.ariaState to be
*  written into cells within this grid. Default implementation return null, meaning
*  no per-cell aria state is written out
* 
* 
* @param {Integer} row index of the cell
* @param {Integer} column index of the cell
* @param {ListGridRecord} record for the cell in question
* @param {string} ARIA role for the cell as returned by ListGrid.getCellRole
*/
getCellAriaState(rowNum:Integer, colNum:Integer, record:ListGridRecord, role:string): object;

/**
*  Returns the WAI ARIA role for rows within this listGrid.
*  
*  If the record has a value for the ListGrid.recordRowRoleProperty, this
*  will be respected.
*  Otherwise if ListGrid.rowRole is specified, it will be used.
*  
*  If the property is not explicitly set, default implementation will 
*  return "separator" for separator rows,
*  "listitem" for data rows if this listGrid has 
*  ListGrid.ariaRole, or "row" for data rows if this 
*  listGrid has ariaRole:"grid".
* 
* 
* @param {Integer} row index
* @param {ListGridRecord} Record for the row in question
*/
getRowRole(rowNum:Integer, record:ListGridRecord): string;

/**
*  Returns a map of Canvas.ariaState to be
*  written into rows within this grid.
*  
*  
*  Default implementation returns an object with combined properties from any
*  specified ListGrid.rowAriaState, overlaid with
*  any ListGrid.recordRowAriaStateProperty properties specified on the record itself,
*  plus the following attributes:
*  
*  
* - setsize - total row count for the grid if ListGrid.ariaRole is set to 
*  "list"
*  
* - posinset - index of the row if ListGrid.ariaRole is set to "list"
*  
* - rowindex - index of the row if ListGrid.ariaRole is set to "grid"
*  
* - selected - true if the record is ListGrid.getSelection
*  
* - expanded - true for ListGrid.canExpandRecords 
*  or Tree.isOpen
*  
* 
* 
* @param {Integer} row index
* @param {ListGridRecord} record for the row in question
* @param {string} ARIA role for the cell as returned by ListGrid.getRowRole
*/
getRowAriaState(rowNum:Integer, record:ListGridRecord, role:string): object;

/**
*  Returns the Canvas.ariaRole for this listGrid's ListGrid.showHeader.
*  
*  If ListGrid.headerAriaRole is explicitly provided, it will be used.
*  Otherwise default implementation returns "row" if ListGrid.ariaRole is set to 
*  "grid"
* 
* 
*/
getHeaderAriaRole(): string;

/**
*  Returns the Canvas.ariaRole for this listGrid's ListGrid.showHeader.
*  
*  If ListGridField.headerButtonAriaRole or ListGrid.headerButtonAriaRole is set, it will be used.
*  Othewise, the default implementation returns "columnheader" if ListGrid.ariaRole is set to 
*  "grid", "button" otherwise.
* 
* 
* @param {ListGridField} header button field object
*/
getHeaderButtonAriaRole(field:ListGridField): string;

/**
*  Returns a map of Canvas.ariaState to be
*  written into header buttons for this grid.
*  
*  Default implementation returns an object with combined properties from any
*  specified ListGrid.headerButtonAriaState, 
*  overlaid with any ListGridField.headerButtonAriaState properties specified on the 
*  field itself, plus the following attributes:
*  
*  
* - haspopup - true if the button should show the header context menu
*  
* - colindex - index of the column if ListGrid.ariaRole is "grid"
*  
* - sort - "ascending", "descending" or "none" depending on the sort-state of the field
*  
* 
*  Also, if an explicit property is set in ariaState, it will be respected and will take precedence over 
*  the calculated property.
*  
* 
* @param {Button} The live ListGrid column header button. Note that the "name" attribute of the button will be set to the field name  for the column.
*/
getHeaderButtonAriaState(button:Button): object;

/**
*  Marks the widget as "dirty" so that it will be added to a queue for redraw. Redraw of dirty
*  components is handled by a looping timer and will after a very short delay (typically less than
*  100ms). In most cases it is recommended that developers use markForRedraw()
*  instead of calling Canvas.redraw directly. Since this method queues the redraw, multiple
*  calls to markForRedraw() within a single thread of execution will only lead to a single DOM
*  manipulation which greatly improves application performance.
* 
* 
* @param {string=} ["no reason provided"] reason for performing the redraw
*/
markForRedraw(reason?:string): void;

/**
*  Get the rows that are currently visible in the viewport, as an array of 
*  [firstRowNum, lastRowNum]. 
*  
*  If the grid contains no records, will return [-1,-1]. Will also return [-1,-1] if called at
*  an invalid time (for example, data is in the process of being fetched - see
*  ResultSet.lengthIsKnown).
*  
* 
*/
getVisibleRows(): Array<Partial<Integer>>;

/**
*  Get the rows that are currently drawn (exist in the DOM), as an array of [firstRowNum,
*  lastRowNum]. 
*  
*  The drawn rows differ from the ListGrid.getVisibleRows because of
*  ListGrid.drawAheadRatio. The drawn rows are the appropriate range to consider if
*  you need to, eg, using ListGrid.refreshCell to update all the cells in a column.
*  
*  If the grid is undrawn or the ListGrid.emptyMessage is currently shown, returns
*  [null,null];
* 
* 
*/
getDrawnRows(): Array<any>;

/**
*  Return the CSS class for a cell. By default this method has the following implementation:
*  - return any custom style for the record (GridRenderer.recordCustomStyleProperty) 
*   if defined.
*  - create a style name based on the result of GridRenderer.getBaseStyle and the 
*   state of the record using the rules described in cellStyleSuffixes.
*  
*  Cell Styles are customizable by:
*  
*  
* - attaching a custom style to a record by setting 
*   record[this.recordCustomStyleProperty] to some valid CSS style name. 
*  
* - modifying the base style returned by getBaseStyle() [see that method for further 
*    documentation on this]
*  
* - overriding this function
*  
*  
*  In addition to this, GridRenderer.getCellCSSText may be overriden to
*  provide custom cssText to apply on top of the styling attributes derived from the named
*  style.
*  
* 
* 
* @param {ListGridRecord} record object for this row and column
* @param {number} number of the row
* @param {number} number of the column
*/
getCellStyle(record:ListGridRecord, rowNum:number, colNum:number): CSSStyleName;

/**
*  Refresh the styling of an individual cell without redrawing the grid.
*  
*  The cell's CSS class and CSS text will be refreshed, to the current values returned by
*  getCellStyle() and getCellCSSText() respectively.
*  
*  The cell's contents (as returned by getCellValue()) will not be refreshed. To
*  refresh both styling and contents, call refreshCell() instead.
* 
* 
* @param {number} row number of cell to refresh
* @param {number} column number of cell to refresh
*/
refreshCellStyle(rowNum:number, colNum:number): void;

/**
*  Called when the mouse pointer enters a cell
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellOver(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when the mouse pointer enters a row
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowOver(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when the mouse pointer leaves a cell
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellOut(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when the mouse pointer leaves a row
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowOut(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when the mouse hovers over a cell if this.canHover is true. 
*  Returning false will suppress the hover text from being shown if this.showHover is true.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellHover(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Optional stringMethod to fire when the user hovers over a cell and the value is clipped.
*  If this.showClippedValuesOnHover is true, the default behavior is to show a hover canvas
*  containing the HTML returned by cellValueHoverHTML(). Return false to suppress this default
*  behavior.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellValueHover(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when the mouse hovers over a row if this.canHover is true. 
*  Returning false will suppress the hover text from being shown if this.showHover is true.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowHover(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  StringMethod to dynamically assemble an HTML string to show in a hover window over the
*  appropriate cell/record when this.canHover and this.showHover are both true.
*  Called when the mouse hovers over a cell.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellHoverHTML(record:ListGridRecord, rowNum:number, colNum:number): HTMLString;

/**
*  Returns the HTML that is displayed by the default cellValueHover handler. Return null or
*  an empty string to cancel the hover.
*  
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
* @param {HTMLString} the HTML that would have been displayed by default
*/
cellValueHoverHTML(record:ListGridRecord, rowNum:number, colNum:number, defaultHTML:HTMLString): HTMLString;

/**
*  Called when a cell receives a contextclick event.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellContextClick(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a row receives a contextclick event.
* 
* @param {ListGridRecord} cell record as returned by getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowContextClick(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a cell receives a mousedown event.
*  
* 
* @param {ListGridRecord} cell record as returned by getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellMouseDown(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a row receives a mousedown event.
*  
* 
* @param {ListGridRecord} record object returned from 'getCellRecord()'
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowMouseDown(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a cell receives a mouseup event.
* 
* 
* @param {ListGridRecord} Record object (retrieved from getCellRecord(rowNum, colNum))
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellMouseUp(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a row receives a mouseup event.
* 
* 
* @param {ListGridRecord} Record object returned from getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
rowMouseUp(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a cell receives a click event.
*  
*  Note that returning false from this method will not prevent any
*  specified ListGrid.rowClick handler from firing.
* 
* 
* @param {ListGridRecord} Record object returned from getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellClick(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Called when a cell receives a double click event.
* 
* 
* @param {ListGridRecord} Record object returned from getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellDoubleClick(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Returns the top coordinate for a given row number, relative to the top of body content. Use
*  ListGrid.getRowPageTop for a page-relative coordinate.
*  
*  This method is reliable only for rows that are currently drawn, which is generally only rows
*  that are visible in the viewport. If row heights vary (see fixedRowHeights),
*  coordinates for rows that are not currently shown are rough approximations.
* 
* 
* @param {Integer} */
getRowTop(rowNum:Integer): Integer;

/**
*  Returns the Y-coordinate for a given row number as a page-relative coordinate. See
*  ListGrid.getRowTop.
* 
* 
* @param {Integer} */
getRowPageTop(rowNum:Integer): Integer;

/**
*  The DataSource that this component should bind to for default fields and 
*  for performing DSRequest.
* 
* 
*/
getDataSource(): DataSource;

/**
*  Returns the ListGrid.autoFitMaxWidth. Note that this method always returns
*  an integer value - autoFitMaxWidth specified as a percentage will be resolved
*  to a pixel value before being returned.
* 
* 
*/
getAutoFitMaxWidth(): Integer;

/**
*  Setter for GridRenderer.fastCellUpdates. Has no effect in browsers other than
*  Internet Explorer.
* 
* @param {boolean} whether to enable fastCellUpdates.
*/
setFastCellUpdates(fastCellUpdates:boolean): void;

/**
*  When assembling a value for a ListGrid.canAddSummaryFields, if a referenced
*  field is hilited, should the hilite HTML be included in the summary field value?
*  
*  Example use case: Consider a grid containing a numeric field, and a summary field
*  which contains some string value, plus the contents of the numeric field.
*  If a hilite is defined for the grid which turns the numeric field text red when
*  the value is negative, this property will govern whether the number will also be
*  rendered in red within the summary field cells. Any other text in the summary field
*  cells would not be effected by this hilite.
*  
*  Default implementation returns DataBoundComponent.includeHilitesInSummaryFields.
*  
*  To control hilites showing in group summaries, see ListGrid.showHilitesInGroupSummary.
* 
* 
* @param {string} name of the summary field
* @param {string} name of the field referenced by this summary
*/
shouldIncludeHiliteInSummaryField(summaryFieldName:string, usedFieldName:string): boolean;

/**
*  Marks a record deleted such that a later call to ListGrid.saveEdits or ListGrid.saveAllEdits
*  will cause a "remove" DSRequest to be submitted.
*  
*  A removed record is disabled and non-editable, and uses ListGrid.removedCSSText for its CSS
*  style, which by default will show strikethrough text.
*  
*  Contrast this method with removeSelectedData(), which immediately submits a DSRequest to
*  remove the selected records from the dataset.
*  
*  Records that have been marked for removal using this method may be 'unmarked' via a call to
*  ListGrid.unmarkRecordRemoved, or by discarding edit values (ListGrid.discardEdits).
* 
* 
* @param {number} row number for the record to mark
*/
markRecordRemoved(rowNum:number): void;

/**
*  Marks an array of records deleted such that a later call to ListGrid.saveEdits or 
*  ListGrid.saveAllEdits will cause a "remove" DSRequest to be submitted.
*  
*  This method is similar to ListGrid.markRecordRemoved but should be more efficient
*  in avoiding unneeded duplicate refreshes due to the multiple records getting marked.
* 
* 
* @param {Array<Partial<ListGridRecord>> | number} records or indices to mark removed
*/
markRecordsRemoved(records:Array<Partial<ListGridRecord>> | number): void;

/**
*  Returns true if the specified record is marked as removed via a call to
*  ListGrid.markRecordRemoved
* 
* @param {Integer} index of row to verify
*/
recordMarkedAsRemoved(rowNum:Integer): boolean;

/**
*  Reverses a previous call to ListGrid.markRecordRemoved.
*  
*  Note that a record that is marked for removal and then un-marked retains any uncommitted
*  edits from before it was marked for removal. These can be discarded with
*  ListGrid.discardEdits.
* 
* @param {Integer} index of record to clear the 'removed'
*/
unmarkRecordRemoved(rowNum:Integer): void;

/**
*  Marks the currently selected records as removed, as though ListGrid.markRecordRemoved had
*  been called.
* 
* 
*/
markSelectionRemoved(): void;

/**
*  How should "Up" and "Down" arrow keypresses be handled when the user is editing
*  an item in the grid.
*  
*  Returning "none" will cause the grid to take no action and allow default up/down
*  arrow key behavior within the editor to proceed. Returning "editNext" will create
*  an appropriate EditCompletionEvent ("arrow_up" or
*  "arrow_down" and cause the grid to start editing the previous or next row).
*  
*  Default behavior varies by item type. For items where up and down arrows have
*  significant functionality to the editor this method returns "none", allowing
*  that standard behavior to proceed. This includes:
*  - Multi line editors (such as TextAreaItems)
*  - SelectItems
*  - SpinnerItems
*  For other items, the default return value will be "edit_next"
*  
*  To override these defaults, developers may specify an explicit arrowKeyEditAction
*  at the ListGrid.arrowKeyEditAction, or 
*  ListGridField.arrowKeyEditAction level.
* 
* @param {FormItem} Edit item receiving the up or down arrow keypress event
* @param {KeyName} Key pressed (one of "Arrow_Up" or "Arrow_Down")
*/
getArrowKeyEditAction(item:FormItem, keyName:KeyName): ArrowKeyEditAction;

/**
*  When ListGrid.sortByGroupFirst is active, the sorting 
*  SortSpecifier.normalizer applied for implicit sorting by the field(s)
*  used for grouping.
*  
*  No default implementation.
* 
* 
* @param {ListGridRecord} record to normalize
* @param {FieldName} name of the field on which sorting occurred
* @param {ListGrid} the grid is passed to allow property and method access
*/
groupSortNormalizer(record:ListGridRecord, fieldName:FieldName, context:ListGrid): any;

/**
*  Get the computed value of a ListGrid.canAddFormulaFields.
* 
* @param {ListGridField} field that has a formula
* @param {Record} record to use to compute formula value
*/
getFormulaFieldValue(field:ListGridField, record:Record): Double | string;

/**
*  Get the computed value of a ListGrid.canAddSummaryFields.
* 
* @param {ListGridField} field that has a summary format
* @param {Record} record to use to compute formula value
*/
getSummaryFieldValue(field:ListGridField, record:Record): string;

/**
*  Shows an additional field near the beginning of the field list (after any
*  ListGrid.showRowNumbers field) that can be dragged to drag the current
*  selection. This feature is useful in Browser.isTouch where
*  both touch scrolling and dragging are needed on the same grid, and allows scrolling to
*  be triggered on the other fields so that both operations are available. Targeted touch
*  environments include both mobile devices, and Windows hardware that supports
*  Browser.supportsDualInput such as Microsoft Surface.
*  
*  Note that the ListGrid.dragHandleField will never be shown unless 
*  ListGrid.canReorderRecords or ListGrid.canDragRecordsOut are true.
*  
*  In IE11 or Microsoft Edge, dragging a record in a grid may not be possible using a touch
*  device without enabling drag handles, or disabling native touch scrolling by setting
*  &nbsp;window.isc_useNativeTouchScrolling = false&nbsp; before SmartClient
*  is loaded.
*  
*  Background
*  
*  One alternative to adding a drag handle field would be to use long touch to start a drag
*  (with normal touch triggering scrolling). However, this is unsupportable in IE11 or Edge
*  on Microsoft Surface (with native scrolling) because native scrolling cannot be canceled
*  on the fly using Event.preventDefault(), but instead must be disabled by applying the
*  appropriate CSS at rendering time. (Such limitations are not present elsewhere, such as
*  on Android or IPhone browsers.)
*  
*  For more details, some links are provided below. Note that while IE10 is mentioned in
*  some of the links, the reasoning is still relevant now for IE11 and Edge as the
*  limitations remain:
*  
*  
* - Cross-browser support of touchMove
*  
* - preventDefault() doesn't work in IE11 on MS Surface
*  
* - preventDefault() doesn't work in Edge on MS Surface
*  
* - preventDefault() failure reported to Microsoft against IE10
*  
* 
*/
showDragHandles(): void;

/**
*  Hides the ListGrid.dragHandleField, if currently shown.
* 
*/
hideDragHandles(): void;

/**
*  Returns the true row index for a grouped record excluding group and
*  summary records. Records in closed groups are included in number.
*  
*  Function is not applicable for non-grouped grids and will return -1
*  if called.
* 
* 
* @param {ListGridRecord} record to number
*/
getGroupedRecordIndex(record:ListGridRecord): Integer;

/**
*  Identifies whether the passed-in field is the specially generated
*  ListGrid.rowNumberField used when ListGrid.showRowNumbers is
*  true. Use this method in your custom event handlers to avoid inappropriately
*  performing actions when the rowNumberField is clicked on.
* 
* 
* @param {ListGridField} field to test
*/
isRowNumberField(field:ListGridField): boolean;

/**
*  Returns the specially generated expansion field used when ListGrid.canExpandRecords is
*  true.
*  
*  Called during ListGrid.setFields, this method can be overridden to add advanced dynamic
*  defaults to the expansion field (call Super, modify the default field returned by Super,
*  return the modified field). Normal customization can be handled by just setting
*  AutoChild properties, as mentioned under the docs for ListGrid.expansionField.
* 
* 
*/
getExpansionField(): ListGridField;

/**
*  Identifies whether the passed-in field is the specially generated
*  ListGrid.expansionField used when ListGrid.canExpandRecords is
*  true. Use this method in your custom event handlers to avoid inappropriately
*  performing actions when the expansionField is clicked on.
* 
* 
* @param {ListGridField} field to test
*/
isExpansionField(field:ListGridField): boolean;

/**
*  Indicates whether a given record or rowNum can be expanded. The default implementation
*  checks the value of ListGrid.canExpandRecords and
*  record[ListGrid.canExpandRecordProperty].
*  
*  Override this method for more specific control over individual record expansion.
*  
*  Note: Rows with no underlying record in the data array - for example newly
*  added edit rows that have not yet been saved - cannot be expanded.
* 
* 
* @param {ListGridRecord} record to work with
* @param {number} rowNum of the record to work with
*/
canExpandRecord(record:ListGridRecord, rowNum:number): boolean;

/**
*  Setter for ListGrid.canExpandRecords
* 
* @param {boolean} new value for listGrid.canExpandRecords.
*/
setCanExpandRecords(canExpand:boolean): void;

/**
*  Expands a given ListGridRecord by creating a subcomponent and inserting it
*  in to the record's grid-row. A number of built-in ExpansionMode
*  are supported by the default implementation of
*  ListGrid.getExpansionComponent and you can override
*  that method to provide your own expansion behavior.
*  
*  Once a record has been expanded, the currently visible expansion component may be
*  retrieved via ListGrid.getCurrentExpansionComponent.
* 
* 
* @param {ListGridRecord} record to expand
*/
expandRecord(record:ListGridRecord): void;

/**
*  Expands the passed list of ListGridRecord by creating a subcomponent for
*  each record and inserting them it in to the record's grid-row. Calls
*  ListGrid.expandRecord for each passed record, but only marks the
*  grid for redraw once, after all expansions are complete.
*  
* 
* @param {Array<Partial<ListGridRecord>>} records to expand
*/
expandRecords(records:Array<Partial<ListGridRecord>>): void;

/**
*  Collapses a given ListGridRecord which has been previously expanded using
*  ListGrid.expandRecord.
*  
*  Depending on the ListGrid.expansionComponentPoolingMode,
*  this method may automatically destroy expansionComponents. By default, components
*  created automatically by the ListGrid will be auto-destroyed. This
*  behavior can be changed by setting a different pooling mode.
*  
*  Note that components created via an override to ListGrid.getExpansionComponent
*  will not be auto-destroyed - developers should override
*  collapseRecord to take care of clean-up for such components.
* 
* 
* @param {ListGridRecord} record to collapse
*/
collapseRecord(record:ListGridRecord): void;

/**
*  Collapses the passed list of expanded ListGridRecord. Calls
*  ListGrid.collapseRecord for each passed record, but only marks
*  the grid for redraw once, after all records have been collapsed.
*  
* 
* @param {Array<Partial<ListGridRecord>>} records to collapse
*/
collapseRecords(records:Array<Partial<ListGridRecord>>): void;

/**
*  Whether a given ListGridRecord is expanded or collapsed.
* 
* 
* @param {ListGridRecord} record in question
*/
isExpanded(record:ListGridRecord): boolean;

/**
*  Returns the list of ListGridRecord from this ListGrid that are
*  ListGrid.expandRecord
* 
* 
*/
getExpandedRecords(): Array<Partial<ListGridRecord>>;

/**
*  Returns the expansion component derived from ListGrid.getExpansionComponent
*  currently visible in some record, or null if the specified record is not showing
*  an expansion component.
* 
* 
* @param {Integer | ListGridRecord} rowNum or record to get the expansionComponent for
*/
getCurrentExpansionComponent(record:Integer | ListGridRecord): Canvas;

/**
*  When ListGrid.canExpandRecords is true, gets the embedded-component to show as a given
*  record's expansionComponent. This component is then housed in
*  ListGrid.expansionLayout and embedded into a record's row.
*  
*  By default, this method returns one of a set of built-in components, according to the
*  value of ExpansionMode. You can override this method
*  to return any component you wish to provide as an expansionComponent.
*  
*  As long as the record is expanded, this component may be retrieved via a call to
*  ListGrid.getCurrentExpansionComponent.
*  
*  When an expanded record is collapsed, the component is disassociated from the record and
*  may or may not be automatically destroyed. By default, built-in components will be
*  destroyed on unembed according to the ListGrid.expansionComponentPoolingMode being used. Custom expansion components, created via an override of
*  getExpansionComponents(), will not be auto-destroyed - developers should
*  override ListGrid.collapseRecord to take care of clean-up for such components.
* 
* 
* @param {ListGridRecord} record to get the expansionComponent for
*/
getExpansionComponent(record:ListGridRecord): Canvas | Partial<Canvas>;

/**
*  Returns the DataSource containing data related to the passed record. Used
*  when ListGrid.canExpandRecords is true and ExpansionMode is "related". The
*  default implementation returns the DataSource specified in
*  ListGridRecord.detailDS if set, otherwise ListGrid.detailDS.
* 
* 
* @param {ListGridRecord} The record to get the Related dataSource for.
*/
getRelatedDataSource(record:ListGridRecord): DataSource;

/**
*  Setter for ListGrid.fixedRecordHeights
* 
* @param {boolean} New fixedRecordHeights value
*/
setFixedRecordHeights(fixedRecordHeights:boolean): void;

/**
*  Setter for ListGrid.wrapCells
* 
* @param {boolean} New wrapCells value
*/
setWrapCells(wrapCells:boolean): void;

/**
*  Setter for the ListGrid.canReorderRecords attribute.
* 
* @param {boolean} new value for this.canReorderRecords
*/
setCanReorderRecords(canReorderRecords:boolean): void;

/**
*  Provides a new data set to the ListGrid after the grid has been created or drawn.
*  The ListGrid will redraw to show the new data automatically.
*  
*  Note that passing null will not clear ListGrid.data, but will regroup it and reapply the
*  current sort, highlighting, and summaries to the grid. Size will be recalculated for fields
*  marked as ListGridField.autoFitWidth:true and a ListGrid.selectionManager will be created if none exists. To clear the grid instead, pass [].
* 
* 
* @param {Array<Partial<ListGridRecord>>} data to show in the list
*/
setData(newData:Array<Partial<ListGridRecord>>): void;

/**
*  Changes selectionType on the fly.
* 
* @param {SelectionStyle} New selection style.
*/
setSelectionType(selectionType:SelectionStyle): void;

/**
*  Changes selectionAppearance on the fly.
* 
* @param {SelectionAppearance} new selection appearance
*/
setSelectionAppearance(selectionAppearance:SelectionAppearance): void;

/**
*  Update the ListGrid.bodyOverflow for this listGrid.
* 
* @param {Overflow} new overflow setting for the body
*/
setBodyOverflow(overflow:Overflow): void;

/**
*  Update the ListGrid.bodyStyleName for this listGrid.
* 
* @param {CSSStyleName} new body style name
*/
setBodyStyleName(styleName:CSSStyleName): void;

/**
*  Update the ListGrid.alternateBodyStyleName for this listGrid.
* 
* @param {CSSStyleName} new body style name when showing alternateRecordStyles
*/
setAlternateBodyStyleName(styleName:CSSStyleName): void;

/**
*  Setter for ListGrid.alternateRecordStyles
* 
* @param {boolean} New value for this.alternateRecordStyles
*/
setAlternateRecordStyles(alternateStyles:boolean): void;

/**
*  Setter for ListGrid.autoFitData.
* 
* @param {Autofit} One of "vertical", "horizontal" or "both". To disable auto fit behavior, pass in null.
*/
setAutoFitData(autoFitData:Autofit): void;

/**
*  Setter for ListGrid.autoFitExtraRecords.
* 
* @param {Integer} Number of extra rows beyond the data-size we'll expand to accommodate if ListGrid.autoFitData is enabled vertically.
*/
setAutoFitExtraRecords(extraRecords:Integer): void;

/**
*  Setter for ListGrid.autoFitMaxRecords.
* 
* @param {Integer} Maximum number of rows we'll expand to accommodate if ListGrid.autoFitData is enabled vertically.
*/
setAutoFitMaxRecords(maxRecords:Integer): void;

/**
*  Setter for ListGrid.autoFitMaxHeight.
* 
* @param {Integer} Maximum height in px we'll expand to accommodate if ListGrid.autoFitData is enabled vertically.
*/
setAutoFitMaxHeight(height:Integer): void;

/**
*  Setter for ListGrid.autoFitMaxColumns.
* 
* @param {Integer} Maximum number of fields we'll expand to accommodate if ListGrid.autoFitData is enabled horizontally.
*/
setAutoFitMaxColumns(maxColumns:Integer): void;

/**
*  Setter for ListGrid.autoFitMaxWidth.
* 
* @param {Integer | string} Width we'll expand to accommodate if ListGrid.autoFitData is enabled horizontally.
*/
setAutoFitMaxWidth(width:Integer | string): void;

/**
*  Programmatically cause a field to auto-fit horizontally to it's contents or title.
*  
*  Does not establish permanent auto-fitting - use ListGrid.setAutoFitWidth or
*  ListGrid.setAutoFitFieldWidths to do so.
*  
*  Note that unlike the ongoing autoFit set up by ListGrid.autoFitFieldWidths or
*  ListGridField.autoFitWidth, any specified ListGridField.width will not be
*  taken as a minimum width - the field may shrink below the current specified width when
*  this method is run. However, ListGridField.minWidth will be respected.
*  
*  As with ListGrid.autoFitFieldWidths, the auto-fit sizing is determined via the
*  ListGrid.autoFitWidthApproach.
* 
* 
* @param {string} */
autoFitField(fieldName:string): Integer;

/**
*  Perform a one-time horizontal auto-fit of the fields passed. Fields will be sized
*  to match their contents or title (as specified in ListGrid.autoFitWidthApproach)
*  Does not establish permanent auto-fitting - use ListGrid.setAutoFitWidth to do so.
*  
*  Note that unlike the ongoing autoFit set up by ListGrid.autoFitFieldWidths or
*  ListGridField.autoFitWidth, any specified ListGridField.width will not be
*  taken as a minimum width - the field(s) may shrink below the current specified width when
*  this method is run. However, ListGridField.minWidth will be respected.
*   
* 
* @param {Array<Partial<ListGridField>>=} Array of fields to auto fit. If this parameter is not passed, autoFitting will occur on all visible fields.
*/
autoFitFields(fields?:Array<Partial<ListGridField>>): void;

/**
*  Setter for ListGridField.autoFitWidth. Enables or disables dynamic autoFitWidth behavior
*  on the specified field. Note if the field is currently autoFitWidth:true, and this method is
*  disabling autoFit, the field will not be resized by default - if you wish to resize to an
*  explicit width, use ListGrid.resizeField.
* 
* 
* @param {string} field to auto-fit
* @param {boolean} Should autoFitWidth be enabled or disabled?
*/
setAutoFitWidth(fieldName:string, autoFit:boolean): void;

/**
*  Setter for ListGrid.autoFitFieldWidths. Modifies the default auto-fit-width behavior
*  for fields in this grid. Note that this may be overridden at the field level via
*  ListGridField.autoFitWidth.
* 
* @param {boolean} New value for autoFitFieldWidths
* @param {boolean=} If autoFitFieldWidths was true, and is being set to false, should fields be resized to their originally specified size?  Pass in this parameter to suppress this behavior.
*/
setAutoFitFieldWidths(autoFit:boolean, dontResetWidths?:boolean): void;

/**
*  Setter for the ListGrid.autoFitWidthApproach.
* 
* @param {AutoFitWidthApproach} new AutoFitWidth approach
*/
setAutoFitWidthApproach(approach:AutoFitWidthApproach): void;

/**
*  Dynamically set properties for a particular field. This method will update the fields
*  header-button without having to explicitly reset the fields in the grid. 
*  
*  NOTE: Where explicit setters exist for field properties (such as
*  ListGrid.resizeField, ListGrid.setFieldTitle, ListGrid.setFieldIcon, etc.) these should be
*  used instead.
* 
* 
* @param {number | string} name of the field, or index.
* @param {Partial<ListGridField>} properties to apply to the header
*/
setFieldProperties(fieldNum:number | string, properties:Partial<ListGridField>): void;

/**
*  Updates ListGridField.minWidth for the specified field and redraws the associated
*  column if required.
* 
* 
* @param {Integer | string} name of the field, or index.
* @param {number} */
setFieldMinWidth(fieldNum:Integer | string, width:number): void;

/**
*  Updates ListGridField.maxWidth for the specified field and redraws the associated
*  column if required.
* 
* 
* @param {Integer | string} name of the field, or index.
* @param {number} */
setFieldMaxWidth(fieldNum:Integer | string, width:number): void;

/**
*  Updates ListGrid.minFieldWidth and redraws any columns as needed.
* 
* 
* @param {Integer} */
setMinFieldWidth(width:Integer): void;

/**
*  Change the title of a field after the grid is created.
* 
* 
* @param {Integer | string} name of the field, or index.
* @param {string} new title
*/
setFieldTitle(fieldNum:Integer | string, title:string): void;

/**
*  Update the ListGridField.headerBaseStyle for a field within the grid at runtime.
* 
* @param {string} name of the field.
* @param {CSSStyleName} new baseStyle for the field header
*/
setFieldHeaderBaseStyle(name:string, newStyle:CSSStyleName): void;

/**
*  Update the ListGridField.headerTitleStyle for a field within the grid at runtime.
* 
* @param {string} name of the field.
* @param {CSSStyleName} new titleTyle for the field header
*/
setFieldHeaderTitleStyle(name:string, newStyle:CSSStyleName): void;

/**
*  Change the ListGridField.icon for a field after the grid is created
* 
* @param {string} field to update
* @param {SCImgURL} icon for the field
*/
setFieldIcon(fieldName:string, icon:SCImgURL): void;

/**
*  Change the ListGridField.cellIcon for a field after the grid is created
* 
* @param {string} field to update
* @param {SCImgURL} new cellIcon for the field
*/
setFieldCellIcon(fieldName:string, cellIcon:SCImgURL): void;

/**
*  Force an array of fields to be shown. This method does not add new fields to the grid,
*  it simply changes field visibility. If a field.showIf expression exists, it will be
*  destroyed.
*  
*  Note: for showing multiple fields it is more efficient to call this method than to call
*  ListGrid.showField repeatedly.
* 
* 
* @param {Array<Partial<string>> | Array<Partial<ListGridField>>} Fields to show.
* @param {boolean=} If passed, don't resize non-explicitly sized columns                    to fill the available space.
*/
showFields(field:Array<Partial<string>> | Array<Partial<ListGridField>>, suppressRelayout?:boolean): void;

/**
*  Force a field to be shown. This method does not add new fields to the grid,
*  it simply changes field visibility. If a field.showIf expression exists, it will be
*  destroyed.
*  
*  Note: for showing multiple fields it is more efficient to call ListGrid.showFields than
*  to call this method repeatedly.
* 
* 
* @param {string | ListGridField} field to show
* @param {boolean=} If passed, don't resize non-explicitly sized columns                    to fill the available space.
*/
showField(field:string | ListGridField, suppressRelayout?:boolean): void;

/**
*  Updates the ListGridField.hideOnPhone attribute at runtime.
* 
* @param {ListGridField | string} field or field name to update
* @param {boolean} new setting for hideOnPhone property
*/
setHideOnPhone(field:ListGridField | string, hideOnPhone:boolean): void;

/**
*  Updates the ListGridField.hideOnTablet attribute at runtime.
* 
* @param {ListGridField | string} field or field name to update
* @param {boolean} new setting for hideOnTablet property
*/
setHideOnTablet(field:ListGridField | string, hideOnTablet:boolean): void;

/**
*  Force an array of fields to be hidden.
*  
*  NOTE: If a field.showIf expression exists, it will be destroyed.
*  
*  When hiding multiple fields, this method should be called rather than
*  calling ListGrid.hideField repeatedly for each field to hide.
* 
* 
* @param {Array<Partial<string>> | Array<Partial<ListGridField>>} fields to hide
* @param {boolean=} if passed, don't relayout non-explicit sized fields                   to fit the available space
*/
hideFields(fields:Array<Partial<string>> | Array<Partial<ListGridField>>, suppressRelayout?:boolean): void;

/**
*  Force a field to be hidden.
* 
*  NOTE: If a field.showIf expression exists, it will be destroyed.
*  
*  Note also that if multiple fields are to be hidden it is more efficient to call
*  ListGrid.hideFields passing in the array of fields to hide rather than to call this method
*  repeatedly. In particular, this will ensure ListGrid.recalculateSummaries is only run once.
* 
* 
* @param {string | ListGridField} field to hide
* @param {boolean=} if passed, don't relayout non-explicit sized fields                   to fit the available space
*/
hideField(field:string | ListGridField, suppressRelayout?:boolean): void;

/**
*  Check whether a field is currently visible
* 
* 
* @param {string | ListGridField} field to be checked
*/
fieldIsVisible(field:string | ListGridField): boolean;

/**
*  Method to return the fieldName which represents the "title" for records in this
*  Component.
*  If this.titleField is explicitly specified it will always be used.
*  Otherwise, default implementation will check DataSource.titleField for databound
*  compounds.
*  For non databound components returns the first defined field name of "title",
*  "name", or "id" where the field is visible. If we don't find any
*  field-names that match these titles, the first field in the component will be used instead.
* 
*/
getTitleField(): string;

/**
*  Sets the fields array and/or field widths to newFields and sizes, respectively.
*  
*  If newFields is specified, it is assumed that the new fields may have nothing in common with
*  the old fields, and the component is substantially rebuilt. Furthermore, it's invalid to
*  modify any of the existing ListGridFields after they've been passed to this function.
*  Consider the following methods for more efficient, more incremental changes:
*  ListGrid.resizeField, ListGrid.reorderField, ListGrid.showField, ListGrid.hideField, or
*  ListGrid.setFieldProperties.
*  
*  Two specific values of newFields have explicit meanings:
*  
*  
* - null - a newFields value of null indicates there are no field overrides.
*       All current fields are removed and, if the grid is bound to a 
*       DataSource, the "default binding" is used. (see 
*       DataBoundComponent.fields).
*  
* - empty array ([]) - providing an empty array for the
*       newFields indicates that no fields are desired even if a dataSource is
*       provided.
*  
* 
* 
* @param {Array<Partial<ListGridField>>=} array of fields to draw
*/
setFields(newFields?:Array<Partial<ListGridField>>): void;

/**
*  If the filter editor (ListGrid.showFilterEditor) is visible for this grid,
*  this method will explicitly put focus into the specified field in the filter editor.
* 
* @param {string=} Name of the field to put focus into. If unspecified focus will go               to the first field in the editor
*/
focusInFilterEditor(fieldName?:string): void;

/**
*  If the filter editor (ListGrid.showFilterEditor) is visible for this grid,
*  this method will perform a filter based on the current values in the editor.
* 
*/
filterByEditor(): void;

/**
*  Returns a snapshot of the current presentation of this listGrid's fields as
*  a ListGridFieldState object.
*  
*  This object can later be passed to ListGrid.setFieldState to reset this grid's
*  fields to the current state.
*  
*  Note that the information stored includes the current width and visibility of each of this
*  grid's fields, as well as any ListGrid.canAddFormulaFields or
*  ListGrid.canAddSummaryFields added by the user.
*  
*  The optional sparse parameter governs whether the returned field state
*  should omit state information for hidden fields. If this parameter is not passed
*  explicitly, field state will be sparse if DataBoundComponent.sparseFieldState
*  is true.
*  When applying sparse field state to a component via ListGrid.setFieldState, any 
*  explicitly defined fields on the component that were not captured in the stored
*  state object will be hidden.
* 
* 
* @param {boolean=} If true, field state will be ommitted for hidden fields.
*/
getFieldState(sparse?:boolean): ListGridFieldState;

/**
*  Sets some presentation properties (visibility, width, userFormula and userSummary) of the
*  listGrid fields based on the ListGridFieldState object passed in.
*  Used to restore previous state retrieved from the grid by a call to ListGrid.getFieldState.
*  
*  The optional isSparse parameter may be passed to indicate
*  whether the fieldState object is "sparse" - whether it includes explicit
*  state information for hidden fields. In this case any fields defined on the component
*  not explicitly included in the fieldState object will be hidden.
*  If isSparse is not explicitly passed as a parameter, 
*  sparseness will be assumed if DataBoundComponent.sparseFieldState
*  is true.
* 
* 
* @param {ListGridFieldState} state to apply to the listGrid's fields.
* @param {boolean=} If true, the fieldState passed in is assumed to be "sparse".  Any fields defined on this component without explicit field state values will be  hidden.
*/
setFieldState(fieldState:ListGridFieldState, isSparse?:boolean): void;

/**
*  Returns the specially generated checkbox field used when SelectionAppearance is
*  "checkbox".
*  
*  Called during ListGrid.setFields, this method can be overridden to add advanced dynamic
*  defaults to the checkbox field (call Super, modify the default field returned by Super,
*  return the modified field). Normal customization can be handled by just setting
*  AutoChild properties, as mentioned under the docs for ListGrid.checkboxField.
* 
* 
*/
getCheckboxField(): ListGridField;

/**
*  Identifies whether the passed-in field is the specially generated
*  ListGrid.checkboxField used when SelectionAppearance is
*  "checkbox". Use this method in your custom event handlers to avoid inappropriately
*  performing actions when the checkboxField is clicked on.
* 
* 
* @param {ListGridField} field to test
*/
isCheckboxField(field:ListGridField): boolean;

/**
*  For components bound to a ListGrid.dataSource with records identified by 
*  unique DataSourceField.primaryKey, this method returns a snapshot of 
*  the component's current selection, as a ListGridSelectedState object.
*  This object can be passed to ListGrid.setSelectedState to reset this grid's current
*  selection state (assuming the same data is present in the grid).
*  Selected state is not supported if the component has no dataSource, or the dataSource has no
*  primaryKey fields.
* 
*/
getSelectedState(): ListGridSelectedState;

/**
*  When ListGrid.canSelectCells is active, returns the CellSelection object that tracks
*  and manages the current selection. Returns null if ListGrid.canSelectCells is false.
* 
*/
getCellSelection(): CellSelection;

/**
*  Reset this grid's selection to match the ListGridSelectedState object passed in.
*  Used to restore previous state retrieved from the grid by a call to
*  ListGrid.getSelectedState.
* 
* 
* @param {ListGridSelectedState} Object describing the desired selection state of                       the grid
*/
setSelectedState(selectedState:ListGridSelectedState): void;

/**
*  Returns a snapshot of the current sort state within this listGrid as
*  a ListGridSortState object.
*  This object can be passed to ListGrid.setSortState to reset this grid's sort to
*  the current state (assuming the same fields are present in the grid).
* 
*/
getSortState(): ListGridSortState;

/**
*  Reset this grid's sort state (sort field and direction or list of
*  SortSpecifiers) to match the
*  ListGridSortState object passed in.
*  Used to restore previous state retrieved from the grid by a call to
*  ListGrid.getSortState.
* 
* 
* @param {ListGridSortState} Object describing the desired sort state for the grid.
*/
setSortState(sortState:ListGridSortState): void;

/**
*  Returns a snapshot of the current view state of this ListGrid.
*  This includes the field, sort, hilite, group, and selected state of the grid, its criteria
*  and, when canShowFilterEditor is true, whether the filter-editor is visible, returned as a 
*  string representation of a ListGridViewState object.
*  This string can be stored over page-reloads (for example, in Offline.put or
*  as field value in a record stored to a DataSource) and then reapplied to the grid
*  via ListGrid.setViewState later to reset the grid to 
*  to the current state (assuming the same data / fields are present).
*  
*  To detect when view state changes, developers may use the ListGrid.viewStateChanged.
*  
*  Note that the ListGrid.autoPersistViewState feature will automatically persist the view state
*  to Offline storage and reapply when the grid is reloaded, without the need to invoke this
*  method explicitly.
* 
* 
*/
getViewState(): ListGridViewState;

/**
*  Returns a snapshot of the current user-provided criteria for this ListGrid.
*  
*  This object can be passed to ListGrid.setUserCriteriaState to reset this grid's 
*  user-criteria to the current state.
* 
*/
getUserCriteriaState(): ListGridUserCriteriaState;

/**
*  Reset this grid's user-criteria to match the ListGridUserCriteriaState object 
*  passed in.
*  
*  Used to restore previous state retrieved from the grid by a call to
*  ListGrid.getUserCriteriaState.
* 
* @param {ListGridUserCriteriaState} Object describing the desired criteria
*/
setUserCriteriaState(userCriteriaState:ListGridUserCriteriaState): void;

/**
*  Returns a snapshot of the current grouping state of this ListGrid.
*  This object can be passed to ListGrid.setGroupState to reset this grid's grouping
*  to the current state (assuming the same data / fields are present in the grid).
* 
*/
getGroupState(): ListGridGroupState;

/**
*  Reset this grid's grouping to match the ListGridGroupState object passed in.
*  Used to restore previous state retrieved from the grid by a call to
*  ListGrid.getGroupState.
* 
* 
* @param {ListGridGroupState} Object describing the desired grouping state of                       the grid
*/
setGroupState(groupState:ListGridGroupState): void;

/**
*  Reset this grid's view state to match the ListGridViewState object passed in.
*  Used to restore previous state retrieved from the grid by a call to
*  ListGrid.getViewState.
* 
* 
* @param {ListGridViewState} Object describing the desired view state for the grid
*/
setViewState(viewState:ListGridViewState): void;

/**
*  Clear this grid's auto-saved ListGridViewState as described in
*  ListGrid.autoPersistViewState.
* 
* 
*/
clearSavedViewState(): void;

/**
*  Returns the ListGridViewState for this ListGrid as last saved by the
*  ListGrid.autoPersistViewState setting.
* 
* 
*/
getSavedViewState(): ListGridViewState;

/**
*  Bind to a new DataSource.
*  
*  Like passing the "dataSource" property on creation, binding to a DataSource means that the
*  component will use the DataSource to provide default data for its fields.
*  
*  When binding to a new DataSource, if the component has any existing "fields" or has a dataset,
*  these will be discarded by default, since it is assumed the new DataSource may represent a
*  completely unrelated set of objects. If the old "fields" are still relevant, pass them to
*  setDataSource().
*  
* 
* @param {GlobalId | DataSource} DataSource to bind to
* @param {Array<Partial<DataSourceField>>=} optional array of fields to use
*/
setDataSource(dataSource:GlobalId | DataSource, fields?:Array<Partial<DataSourceField>>): void;

/**
*  Re-evaluates ListGridField.showIf for each field, dynamically showing and
*  hiding the appropriate set of fields
* 
*/
refreshFields(): void;

/**
*  Returns an array of widths of the visible fields in this ListGrid, in px. This
*  method is implemented by calling ListGrid.getFieldWidth for each field.
*  If field widths cannot be determined, the returned array will contain nulls.
* 
*/
getCurrentFieldWidths(): Array<Partial<Integer>>;

/**
*  Returns a numeric value for the width of some field within this ListGrid.
* 
* @param {Integer | FieldName} Index or name of the field for which the width is to be                  determined.
*/
getFieldWidth(fieldNum:Integer | FieldName): Integer;

/**
*  Setter for the ListGrid.frozenFieldsMaxWidth attribute
* 
* @param {string | Integer} new maximum width for frozen fields
*/
setFrozenFieldsMaxWidth(width:string | Integer): void;

/**
*  Method to calculate and return the default width of a field. This method is called
*  to calculate the size of each field's content as part of the
*  ListGrid.autoFitFieldWidths behavior. Note that this method
*  returns a size for content, so will not be consulted if
*  ListGridField.autoFitWidthApproach is set 
*  to "title".
*  
*  If ListGridField.defaultWidth is specified, this will be returned.
*  
*  Otherwise, the default implementation varies by ListGridFieldType.
*  For fields of type "icon", or fields which show only a
*  ListGridField.valueIcons as a value, and for boolean fields which
*  show a checkbox value, the width will be calculated based on the icon size and
*  ListGrid.iconPadding.
*  For other fields the ListGrid.getFieldContentWidth method will be used to calculate
*  a width based on the rendered width of content. Note that for
*  "image" type fields, this method will rely on the ListGridField.imageWidth
*  being specified.
*  
*  Note that this width is the default width of "content" - it does not take into account
*  the rendered size of the field title.
* 
* 
* @param {ListGridField} Field for which the size should be determined
*/
getDefaultFieldWidth(field:ListGridField): Integer;

/**
*  Returns the pixel width of the content of a visible field in this grid.
* 
* @param {ListGridField} field to test
*/
getFieldContentWidth(field:ListGridField): Integer;

/**
*  Get the current grouping of this listGrid as an array of fieldNames.
*  
*  This method returns an array containing the names of the field(s) by which this
*  grid is grouped (either from ListGrid.groupByField having been explicitly set or
*  from a call to ListGrid.groupBy). If this grid is not currently grouped, this method
*  will return null.
* 
*/
getGroupByFields(): Array<Partial<string>>;

/**
*  Opens the node represented by the "record" parameter, if it is a folder and is not already
*  open. This method only applies to ListGrid.groupBy ListGrids.
* 
* @param {Record} node to open
*/
openGroup(record:Record): boolean;

/**
*  Closes the node represented by the "record" parameter, if it is a folder and is not already
*  closed. This method only applies to ListGrid.groupBy ListGrids.
* 
* @param {Record} node to close
*/
closeGroup(record:Record): boolean;

/**
*  Redraw just the ListGrid.header 
* 
* 
* @param {boolean} If true, redraw the grid header with a direct inline call     to its redraw() method. Otherwise,      Canvas.markForRedraw
*/
redrawHeader(rightNow:boolean): void;

/**
*  Return the base styleName for this cell. Has the following implementation by default:
*  
*  
* - If ListGrid.editFailedBaseStyle is defined, and the
*    cell is displaying a validation error return this value.
*  
* - If ListGrid.editPendingBaseStyle is defined, and
*    the cell is displaying an edit value that has not yet been saved (see
*    ListGrid.autoSaveEdits) return this value.
*  
* - Otherwise return ListGrid.recordBaseStyleProperty,
*    if defined, otherwise ListGridField.baseStyle.
*  
*  If no custom style is found for the cell as described above, the default baseStyle will be
*  returned. If ListGrid.baseStyle is specified this will be used. Otherwise for
*  grids showing fixed height rows which match ListGrid.normalCellHeight
*  ListGrid.normalBaseStyle will be used. For grids with variable, or modified
*  cell heights, ListGrid.tallBaseStyle will be used.
*  
*  Note also that
*  enabling ListGrid.fastCellUpdates will cause the tallBaseStyle to be
*  used rather than ListGrid.normalBaseStyle.
*  
*  As noted under ListGrid.enforceVClipping, cell content which renders taller than the
*  available space within a cell may cause rows to expand even if ListGrid.fixedRecordHeights 
*  is true. This can lead to misaligned rows when frozen columns are used.
*  Developers should be aware that changing cell styling such that there is increased
*  borders or padding will reduce the available space for content within the specified cell height,
*  making this scenario more common. To fix this, specify a larger cellHeight, or set
*  enforceVClipping to true.
*  
* 
* 
* @param {ListGridRecord} Record associated with this cell. May be null                 for a new edit row at the end of this grid's data set.
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getBaseStyle(record:ListGridRecord, rowNum:number, colNum:number): CSSStyleName;

/**
*  Return CSS text for styling this cell, which will be applied in addition to the CSS class
*  for the cell, as overrides.
*  
*  "CSS text" means semicolon-separated style settings, suitable for inclusion in a CSS
*  stylesheet or in a STYLE attribute of an HTML element.
*  
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getCellCSSText(record:ListGridRecord, rowNum:number, colNum:number): string;

/**
*      Obtains the display value for a specific cell according to the given input
*      parameters.
*      To format the value displayed in the cell, make use of the
*      ListGrid.formatCellValue methods rather than 
*      overriding this method directly.
* 
* @param {object} the current record object
* @param {number} number of the record in the current set of displayed                   record (e.g. 0 for the first displayed record)
* @param {number} number of the field in the listGrid.fields array
*/
getCellValue(record:object, recordNum:number, fieldNum:number): string;

/**
*  Is the value in a given cell clipped?
* 
* @param {number} row number of the cell
* @param {number} column number of the cell
*/
cellValueIsClipped(rowNum:number, colNum:number): boolean;

/**
*  Returns the cursor to display when the mouse pointer is
*  over an ListGridField.icon in an "icon" type field.
*  
*  Default behavior will display the ListGridField.iconCursor if specified, otherwise
*  the component level ListGrid.iconCursor.
* 
* 
* @param {ListGridField} field displaying the icon
*/
getIconCursor(field:ListGridField): Cursor;

/**
*  Returns the cursor to display when the mouse pointer is
*  over a ListGridField.valueIcons in a a cell.
*  
*  Default behavior will display the ListGridField.iconCursor if specified, otherwise
*  the "pointer" cursor if a ListGridField.valueIconClick hander is present.
*  (If no valueIconClick handler is defined this method will return null and the cursor will
*  be unchanged when the user rolls over the value icon image).
* 
* 
* @param {ListGridField} field displaying the valueIcon
* @param {ListGridRecord} record being rolled over
* @param {any} value of this cell
*/
getValueIconCursor(field:ListGridField, record:ListGridRecord, value:any): Cursor;

/**
*  Return the horizontal alignment for cell contents. Default implementation returns
*  ListGridField.cellAlign if specified, otherwise ListGridField.align.
* 
* 
* @param {ListGridRecord} this cell's record
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getCellAlign(record:ListGridRecord, rowNum:number, colNum:number): Alignment;

/**
*  Return the vertical alignment for cell contents.
*  Expected values are: 'top', 'center', or 'bottom'
* 
* 
* @param {ListGridRecord} this cell's record
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getCellVAlign(record:ListGridRecord, rowNum:number, colNum:number): VerticalAlignment;

/**
*  Return how many rows this cell should span. Default is 1.
*  
*  This method will only be called if ListGrid.allowRowSpanning is set to
*  true
*  
*  When using row spanning, consider setting ListGrid.useRowSpanStyling to enable
*  row-span-sensitive styling behaviors.
*  
*  Note that the standard implementation assumes that the number of rows spanned by cells
*  decreases or stays the same, starting with the first (leftmost) column in the grid and
*  moving rightwards.
*  
*  When using row spanning:
*  
*  
* - APIs that allow modifying the contents of cells (such as ListGrid.getCellStyle or
*  ListGridField.formatCellValue) will be called only once per row-spanning cell
*  
* - when using ListGrid.canSelectCells, only the
*  spanning cell is considered selected, and not any of the cells spanned through. For
*  example, if the cell at row 2 column 0 spans 2 cells, CellSelection
*  will be true for 2,0 but false for 3,0.
*  
* - if using incremental rendering (either horizontal or vertical),
*  getRowSpan() may be called for a rowNum in the middle of a spanning
*  cell, and should return the remaining span from that rowNum onward.
*  
* - cell-level events such as ListGrid.recordClick will report the logical
*  rowNum for spanned cells. In other words if a cell spans two rows, a different
*  rowNum parameter will be passed to the recordClick handler depending on whether the user
*  clicks at the top of the spanning cell or the bottom. Developers can normalize
*  this to the starting cell via the ListGrid.getCellStartRow API.
*  
* - for cells that span multiple records, editing behavior may be controlled by the
*  RowSpanEditMode attribute.
*  
* - rowSpanning can be used in conjunction with
*  ListGrid.showRecordComponents. If record
*  component are enabled on a grid with row-spanning cells the behavior is as follows:
*   
*   
* - Having ListGrid.recordComponentPosition set to "expand" is not currently supported
*     for grids that render out spanning cells.
*   
* - The method to retrieve / create record components will not be run for cells that
*     are "spanned". In other words if the first row in a grid spans 2 rows for some
*     field, the second logical row is "spanned" for that field - that cell doesn't
*     render any content and won't attempt to create a recordComponent.
*   
* - If ListGrid.showRecordComponentsByCell is false, the method to create record
*     components will be called for every non-spanned cell in the first column
*     of the grid.
*   
* - Percentage sizing of record components spanning multiple cells will be
*     calculated relative to the set of spanned cells.
*   
*  
*  
*  More generally, the ListGrid has a data model of one Record per row, and spanning cells
*  doesn't fit well with this model, meaning that many ListGrid features are incompatible
*  with rowSpanning.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getRowSpan(record:ListGridRecord, rowNum:number, colNum:number): number;

/**
*  When using ListGrid.getRowSpan, returns the number of cells spanned by the cell
*  at the given coordinates.
*  
*  If the passed coordinates are in the middle of a series of spanned cells, the row span of
*  the spanning cell is returned. For example, if row 2 col 0 spans 3 cells, calls to
*  getCellRowSpan() for row 2 col 0, row 3 col 0, row 4 col 0 will all return 3.
*  
*  This method returns row span information for the current rendered cells. In contrast, if
*  the grid is about to be redrawn, a call to getRowSpan() may return row span
*  values for how the grid is about to be drawn. Also, user-provided getRowSpan() functions
*  are not required to operate properly when called outside of the grid rendering loop.
*  
*  Note: This method is a utility method for developers - it is not called
*  directly by the grid rendering path and therefore is not intended for override. To
*  set up custom row-spanning behavior, override ListGrid.getRowSpan instead.
* 
* 
* @param {Integer} row number of cell to return the row span for
* @param {Integer} column number of cell to return the row span for
*/
getCellRowSpan(rowNum:Integer, colNum:Integer): Integer;

/**
*  When using ListGrid.getRowSpan, returns the row number where a row-spanning cell
*  starts.
*  
*  For example, if row 2 col 0 spans 3 cells, getCellStartRow() for row 2 col 0,
*  row 3 col 0, row 4 col 0 will all return 2, because that's the row when spanning starts.
* 
* 
* @param {Integer} row number of cell for which the start row should be returned
* @param {Integer} column number of cell for which the start row should be returned
*/
getCellStartRow(rowNum:Integer, colNum:Integer): Integer;

/**
*  Returns the appropriate valueIcon for a cell based on the field and the data value for the
*  cell. Default implementation returns null if ListGridField.suppressValueIcon is true
*  otherwise looks at ListGridField.valueIcons.
* 
* @param {ListGridField} field associated with the cell
* @param {any} data value for the cell's record in this field.
* @param {ListGridRecord} record associated with this cell
*/
getValueIcon(field:ListGridField, value:any, record:ListGridRecord): SCImgURL;

/**
*  Get the value for some cell with default formatters applied.
*  
*  This method is useful for cases where a developer wishes to
*  conditionally customize a cell's formatting, but needs to see what
*  the default formatted value would be.
*  
*  For example - a developer might wish to apply a custom
*  ListGridField.formatCellValue to some
*  link type field, and be able to return the default
*  active link HTML in some cases. In this case a formatter could
*  check for the conditions in which custom formatting should be applied
*  and run appropriate custom logic to generate a value for display - otherwise
*  return the result of this method to leave the standard formatted-value intact.
* 
* 
* @param {Record} the cell's record object
* @param {Integer} rowNum for the cell
* @param {Integer} colNum for the cell
*/
getDefaultFormattedValue(record:Record, rowNum:Integer, colNum:Integer): string;

/**
*  Get a field value for some record with default field formatters applied.
*  
*  This method differs from ListGrid.getDefaultFormattedValue in a couple of ways.
*  Firstly, this method does not rely on the rowNum and colNum parameters to find the
*  record and field in the grid.
*  Secondly, unlike ListGrid.getDefaultFormattedValue this method will call
*  any explicit cell formatter specified on the field passed in (though it will not
*  call a ListGrid.formatCellValue if one exists).
*  
*  This is useful for cases where a developer wishes to
*  display a formatted value for some record and field combination which does not
*  necessarily map to a cell displayed in the ListGrid.
*  
*  If rowNum and colNum parameters are passed through to
*  the field level cell formatter if one exists. If not explicitly provided these are
*  defaulted to -1.
* 
* 
* @param {Record} the record object
* @param {ListGridField} the field object
* @param {Integer=} rowNum (passed to any field level cell formatter)
* @param {Integer=} colNum (passed to any field level cell formatter)
*/
getDefaultFormattedFieldValue(record:Record, field:ListGridField, rowNum?:Integer, colNum?:Integer): string;

/**
*  Setter for the ListGrid.showRecordComponents attribute
* 
* @param {boolean} new value for this.showRecordComponents
*/
setShowRecordComponents(showRecordComponents:boolean): void;

/**
*  Returns the extents of the rows and columns currently visible in this grid's viewport.
*  
*  Note: if there are any ListGridField.frozen, they are not included
*  in the draw area range returned by this method. Frozen fields are assumed to never be
*  scrolled out of view. The column coordinates returned by this method will only include
*  unfrozen columns.
* 
* 
*/
getDrawArea(): Array<Partial<Integer>>;

/**
*  If ListGrid.showRecordComponents is true, by default any created record components
*  are destroyed once they are no longer in use (for example, if the ListGrid as a whole is
*  destroyed). This method may be used to suppress this behavior for some component. Typical
*  usage might call this method as part of ListGrid.createRecordComponent to suppress
*  this behavior.
* 
* @param {Canvas} component in question.
* @param {boolean} If true, the component will not be destroyed automatically when  the grid is destroyed
*/
setDontAutoDestroyComponent(component:Canvas, dontAutoDestroy:boolean): void;

/**
*  Invalidates the currently visible set of ListGrid.showRecordComponents
*  and gets fresh ones for the visible rows in the grid according to the
*  ListGrid.recordComponentPoolingMode
*  
*  See also ListGrid.refreshRecordComponent which allows you to refresh a specific
*  recordComponent
* 
* 
*/
invalidateRecordComponents(): void;

/**
*  Discards any ListGrid.showRecordComponents currently assigned to the
*  specified record (or cell) and gets a fresh one, according to the
*  ListGrid.recordComponentPoolingMode
*  
*  See also ListGrid.invalidateRecordComponents which allows you to refresh all record
*  components that are currently visible in the grid.
* 
* 
* @param {Integer} Row to refresh
* @param {Integer=} Column to refresh. This parameter should be passed   if ListGrid.showRecordComponentsByCell is true.
*/
refreshRecordComponent(rowNum:Integer, colNum?:Integer): void;

/**
*  Setter for the ListGrid.recordComponentHeight
* 
* @param {Integer} recordComponent height
*/
setRecordComponentHeight(height:Integer): void;

/**
* 
*  Event handler for when rows in the body are clicked upon. The default implementation handles
*  firing ListGrid.startEditing if appropriate, and fires
*  ListGridField.recordClick and/or ListGrid.recordClick if set. Developers
*  should typically implement recordClick rather than overriding this method.
*  
*  Note that this method fires in addition to any specified ListGrid.cellClick
*  handler (even if that method cancels the event as a whole by returning false).
* 
* 
* @param {ListGridRecord} record object returned from getCellRecord()
* @param {Integer} index of the row where the click occurred
* @param {Integer} index of the col where the click occurred
* @param {boolean=} indicates whether this was a synthesized record                       click in response to a keyboard event
*/
rowClick(record:ListGridRecord, recordNum:Integer, fieldNum:Integer, keyboardGenerated?:boolean): boolean;

/**
*  Event handler for when a body record is double-clicked.
*  
*  Default implementation fires 'editCell' if appropriate, and handles firing
*  'recordDoubleClick' stringMethod if defined at the field or LG level (That method has a
*  different signature from this one)
* 
* 
* @param {ListGridRecord} record object returned from getCellRecord()
* @param {number} index of the row where the click occurred
* @param {number} index of the col where the click occurred
* @param {boolean=} indicates whether this was a synthesized record                       doubleclick in response to a keyboard event
*/
rowDoubleClick(record:ListGridRecord, recordNum:number, fieldNum:number, keyboardGenerated?:boolean): boolean;

/**
*  If ListGrid.showCellContextMenus is true this method returns
*  the menu items to be displayed in the default cell context menu.
*  
*  This method is called at various times, so this method should not instantiate any classes, 
*  because they'll be re-created on each call, resulting in a leak - your implementation should 
*  return an array of menuItem config-blocks only, so you shouldn't instantiate actual Menu 
*  instances to apply as the MenuItem.submenu of items - instead, set submenu 
*  to a simple array of menuItems. If your use-case necessitates that class instances are 
*  created, because specific submenus have a different Menu class, for example, you should keep 
*  a reference to them and either, if their content is dynamic, destroy and recreate them with
*  the new items, or just return the existing instances otherwise. 
*  
*  The default set of menu items includes items for built-in ListGrid features, like showing or
*  hiding an inline edit form, or removing records.
* 
* 
* @param {Integer} The record the user clicked in
* @param {Integer} Index of the record the user clicked in
* @param {Integer} Index of the column the user clicked in
*/
getCellContextMenuItems(record:Integer, rowNum:Integer, colNum:Integer): Array<Partial<MenuItem>>;

/**
*  When ListGrid.showHoverComponents is set, this method is called to get the component to show
*  as a hover for the current cell.
*  
*  By default, this method returns one of a set of builtin components, according to the
*  value of HoverMode. You can override this method
*  to return any component you wish to provide as a hoverComponent, or invoke the superclass
*  method to have the default hover component generated, then further customize it.
*  
*  By default, components returned by getCellHoverComponent() will be
*  automatically destroyed when the hover is hidden. To prevent this, set
*  Canvas.hoverAutoDestroy to false on the returned component.
*  
*  If you return a component that fetches data or loads content dynamically:
*  
*  
* - set rpcRequest.promptStyle to "cursor" or set rpcRequest.showPrompt to false on any 
*    network requests, or the default masking that blocks the screen during network requests 
*    will dismiss the hover
*  
* - as covered above, your component may have been automatically destroyed by the time your 
*    content has been loaded. Check Canvas.destroyed before taking action in an 
*    asynchronous callback
*  
*  
* - if your component grows in size after data is loaded, and it would then be rendered
*    partially off-screen, it will be automatically re-positioned to keep it on-screen. 
*    However this will not automatically happen in cases where you provide HTML content that
*    changes size after initial render, in which case a call to 
*    Canvas.adjustForContent will be required. See that API for details.
*  
*  
* 
* @param {Record} record to get the hoverComponent for
* @param {Integer} row number for the cell
* @param {Integer} column number of the cell
*/
getCellHoverComponent(record:Record, rowNum:Integer, colNum:Integer): Canvas | Partial<Canvas>;

/**
*  Select/deselect a Record passed in explicitly, or by index.
*  
*  Note that this method selects records unconditionally, allowing multiple selected
*  records, even when ListGrid.selectionType is "single". To enforce mutually-exclusive
*  record-selection, use ListGrid.selectSingleRecord.
* 
* @param {Record | number} record (or row number) to select
* @param {boolean=} new selection state (if null, defaults to true)
*/
selectRecord(record:Record | number, newState?:boolean): void;

/**
* 
*  Deselect a Record passed in explicitly, or by index.
*  
*  Synonym for selectRecord(record, false)
* 
* 
* @param {Record | number} record (or row number) to deselect
*/
deselectRecord(record:Record | number): void;

/**
* 
*  Select/deselect a list of Records passed in explicitly, or by index.
*  
*  Note that developers may wish to use ListGrid.selectRange to select a single
*  contiguous range.
* 
* 
* @param {Array<Partial<Record>> | number} records (or row numbers) to select
* @param {boolean=} new selection state (if null, defaults to true)
*/
selectRecords(records:Array<Partial<Record>> | number, newState?:boolean): void;

/**
* 
*  Deselect a list of Records passed in explicitly, or by index.
*  
*  Synonym for selectRecords(records, false)
*  
*  Note that developers may wish to use ListGrid.deselectRange to select a single
*  contiguous range.
* 
* 
* @param {Array<Partial<Record>> | number} records (or row numbers) to deselect
*/
deselectRecords(records:Array<Partial<Record>> | number): void;

/**
*  Select all records.
*  
*  Note that this method will select records even if ListGrid.canSelectRecord returns
*  false for the record in question. See also ListGrid.userSelectAllRecords
* 
* 
* @param {boolean=} For TreeGrids, or listGrids showing  hierarchical ListGrid.isGrouped, if true is   passed for this parameter, only visible nodes will be selected.   Nodes embedded in a closed parent folder  (and thus hidden from the user) will not be selected.
*/
selectAllRecords(visibleNodesOnly?:boolean): void;

/**
* 
*  Deselect all records
* 
* 
*/
deselectAllRecords(): void;

/**
*  Select a contiguous range of records by index
* 
* @param {Integer} start of selection range
* @param {Integer} end of selection range (non-inclusive)
* @param {boolean=} new selection state (if null, defaults to true)
*/
selectRange(startRow:Integer, endRow:Integer, newState?:boolean): void;

/**
*  Deselect a contiguous range of records by index.
*  
*  This is a synonym for selectRange(startRow, endRow, false);
* 
* @param {Integer} start of selection range
* @param {Integer} end of selection range (non-inclusive)
*/
deselectRange(startRow:Integer, endRow:Integer): void;

/**
*  Selects every user-selectable record in the grid. Unlike ListGrid.selectAllRecords,
*  if a record is ListGrid.canSelectRecord, this method will not
*  attempt to select it.
* 
*/
userSelectAllRecords(): void;

/**
*  Whether at least one item is selected
* 
*/
anySelected(): boolean;

/**
*  Select a single Record passed in explicitly, or by index, and deselect everything else.
*  When programmatic selection of records is a requirement and 
*  ListGrid.selectionType is "single", use this method rather than 
*  DataBoundComponent.selectRecord to 
*  enforce mutually-exclusive record-selection.
* 
* 
* @param {Record | number} record (or row number) to select
*/
selectSingleRecord(record:Record | number): void;

/**
*  If ListGrid.canSelectCells is set to true then, whenever an end-user or
*  programmatic cell-selection is attempted, this method is called for 
*  each cell in the selection. If it returns false, the cell will not be selected.
* 
* 
* @param {Integer} rowNum being selected
* @param {Integer} colNum being selected
*/
canSelectCell(rowNum:Integer, colNum:Integer): boolean;

/**
*  If ListGrid.selectionType is not set to "none",
*  This method will be called for each record the user attempts to select. If it returns false, the
*  record will not be selected.
*  
*  The default implementation will return true for any records where
*  ListGrid.recordCanSelectProperty is not explicitly set to false, and false
*  if this method was called by a click on the ListGrid.expansionField
*  and ListGrid.selectOnExpandRecord is set to false.
*  
*  
*  Note this method will not be called at all if ListGrid.canSelectCells is true.
* 
* 
* @param {ListGridRecord} record being selected
*/
canSelectRecord(record:ListGridRecord): boolean;

/**
*  Handle a keyPress event on the ListGrid as a whole.
*  
*  Note that the majority of keyboard handling for a ListGrid is performed by
*  ListGrid.bodyKeyPress and most overrides are better performed there.
* 
* 
*/
keyPress(): boolean;

/**
*  Handle a keyPress event on the body.
*  
*  Default implementation handles navigating between records with arrow keys, and activating
*  records with space and enter.
* 
* 
*/
bodyKeyPress(): boolean;

/**
*  Puts keyboard focus into the specified cell, showing a highlighted (roll-over style)
*  appearance, and ensuring that arrow-key navigation will start from the specified cell.
*  
*  Only applies where ListGrid.canSelectCells is true.
* 
* @param {Integer} Index of target row
* @param {Integer} Index of target col
*/
focusInCell(row:Integer, col:Integer): void;

/**
*  Puts keyboard focus into the specified row, showing a highlighted (roll-over style)
*  appearance, and ensuring that arrow-key navigation will start from the specified row.
*  
*  Only applies where ListGrid.canSelectCells is false.
* 
* @param {Integer} Index of target row
*/
focusInRow(row:Integer): void;

/**
*  Get the row that currently has keyboard focus. Arrow key navigation moves relative to this
*  row.
* 
* 
*/
getFocusRow(): Integer;

/**
*  Scroll the grid to specified column such that the row appears near the center of the
*  viewport.
*  
*  See ListGrid.scrollToCell for a full description of how
*  this method interacts with incremental loading and rendering of data.
* 
* @param {number} Index of the column to scroll into view
* @param {Alignment=} Horizontal position of scrolled column (optional)
*/
scrollToColumn(colNum:number, xPosition?:Alignment): void;

/**
*  Scroll the grid to specified row such that the row appears near the center of the
*  viewport, loading data if necessary.
*  
*  See ListGrid.scrollToCell for a full description of how
*  this method interacts with incremental loading and rendering of data.
* 
* 
* @param {number} Row index of the cell to scroll into view
* @param {VerticalAlignment=} Vertical position of scrolled row (optional)
*/
scrollToRow(rowNum:number, yPosition?:VerticalAlignment): void;

/**
*  Will scroll the listGrid body such that the specified cell is visible close to the
*  center of the viewport.
*  
*  This method has no effect if the cell is already visible in the viewport.
*  
*  When scrolling vertically, this will cause data to be automatically loaded
*  if ListGrid.dataFetchMode and you scroll into an area of
*  the data that isn't loaded. Only rows around the target row will be
*  loaded, not all intervening rows. See also ResultSet.
*  
*  Scrolling into an undrawn area will cause the body area of the grid to
*  redraw, but this won't happen synchronously unless you explicitly call
*  redraw(). Scrolling into an area of the data that is not yet loaded
*  will never synchronously draw new rows, even if you call redraw() -
*  wait for ListGrid.dataArrived to be notified when new rows have been
*  loaded.
*  
*  Calling this method with a row index larger than the
*  current dataset will clamp to the end of the dataset (similarly horizontal
*  scrolling will clamp to the last column).
*  
*  If a call to this method is made while data is still loading, such
*  that the last row of the dataset is not yet known the grid will attempt to compensate
*  by scrolling the record into view when data arrives, if it is valid.
*  For better control over scrolling, developers should consider
*  calling scrollToRow() or scrollToCell
*  from ListGrid.dataArrived if data is still loading.
*  
*  With mixed-height rows it will only reliably work if virtualScrolling
*  is enabled.
* 
* 
* @param {Integer} Row index of the cell to scroll into view
* @param {Integer} Column index of the cell to scroll into view
* @param {Alignment=} Horizontal position of scrolled cell (optional)
* @param {VerticalAlignment=} Vertical position of scrolled cell (optional)
*/
scrollToCell(rowNum:Integer, colNum:Integer, xPosition?:Alignment, yPosition?:VerticalAlignment): void;

/**
*  Executed when the listGrid receives a 'click' event on an enabled, non-separator
*  record. The default implementation does nothing -- override to perform some action
*  when any record or field is clicked.
*  A record event handler can be specified either as
*  a function to execute, or as a string of script to evaluate. If the handler is defined
*  as a string of script, all the parameters below will be available as variables for use
*  in the script.
*  To do something specific if a particular field is clicked, add a recordClick
*  method or string of script to that field (same parameters) when you're setting up
*  the list.
*  Notes:
*  
* - This will not be called if the click is below the last item of the list.
*  
* - This method is called from the default implementation of
*  ListGrid.rowClick, so if that method is overridden
*  this method may not be fired.
* 
* 
* @param {ListGrid} the listGrid that contains the click event
* @param {ListGridRecord} the record that was clicked on
* @param {number} number of the record clicked on in the current set of                 displayed records (starts with 0)
* @param {ListGridField} the field that was clicked on (field definition)
* @param {number} number of the field clicked on in the listGrid.fields                 array
* @param {any} value of the cell (after valueMap, etc. applied)
* @param {any} raw value of the cell (before valueMap, etc applied)
* @param {ListGridRecord} the clicked record with any unsaved                  edit values overlaid (see listGrid.getEditedRecord()).
*/
recordClick(viewer:ListGrid, record:ListGridRecord, recordNum:number, field:ListGridField, fieldNum:number, value:any, rawValue:any, editedRecord:ListGridRecord): boolean;

/**
*  Executed when the listGrid receives a 'doubleClick' event on an enabled, non-separator
*  record. The default implementation does nothing -- override to perform
*  some action when any record or field is double clicked.
*  A record event handler can be specified either as a function to execute, or as a string
*  of script to evaluate. If the handler is defined as a string of script, all the
*  parameters below will be available as variables for use in the script.
*  To do something specific if a particular field is double clicked, add a
*  recordDoubleClick method or string of script to that field (same parameters) when you're
*  setting up the list.
*  Notes:
*  
* - This will not be called if the click is below the last item of the list.
*  
* - This method is called from the default implementation of ListGrid.rowDoubleClick,
*  so if that method is overridden this method may not be fired.
* 
* @param {ListGrid} the listGrid that contains the doubleclick event
* @param {ListGridRecord} the record that was double-clicked
* @param {number} number of the record clicked on in the current set of                 displayed records (starts with 0)
* @param {ListGridField} the field that was clicked on (field definition)
* @param {number} number of the field clicked on in the listGrid.fields                 array
* @param {object} value of the cell (after valueMap, etc. applied)
* @param {object} raw value of the cell (before valueMap, etc applied)
* @param {ListGridRecord} the clicked record with any unsaved                  edit values overlaid (see listGrid.getEditedRecord()).
*/
recordDoubleClick(viewer:ListGrid, record:ListGridRecord, recordNum:number, field:ListGridField, fieldNum:number, value:object, rawValue:object, editedRecord:ListGridRecord): boolean;

/**
*  Setter for the ListGrid.showGridSummary attribute
* 
* @param {boolean} new value for this.showGridSummary
*/
setShowGridSummary(showGridSummary:boolean): void;

/**
*  Recalculates values for fields with
*  ListGridField.recordSummaryFunction or 
*  ListGridField.userFormula defined and for values
*  displayed in the ListGrid.showGridSummary and
*  ListGrid.showGroupSummary.
* 
* @param {Array<Partial<Record>>=} Optional array of records to recalculate summaries for,         or null for all records
* @param {Array<Partial<ListGridField>>=} Optional array of fields to recalculate summaries         for, or null for all fields  Note that the records should be from ListGrid.data; thus, if the grid is grouped,  the records should be from the grouped data rather than ListGrid.originalData.
*/
recalculateSummaries(records?:Array<Partial<Record>>, fields?:Array<Partial<ListGridField>>): void;

/**
*  Refresh the ListGrid.showGridSummary, by either re-calculating from
*  already-loaded data or doing a new fetch from the ListGrid.summaryRowDataSource.
*  
*  Note unlike ListGrid.recalculateSummaries, this method will not force a refresh of
*  field-level summaries (see ListGridField.recordSummaryFunction) or group level
*  summaries (see ListGrid.showGroupSummary).
* 
*/
recalculateGridSummary(): void;

/**
*  Determines the SummaryFunction to use when calculating per-field summary values
*  describing multiple records in this grid.
*  Used to determine the summary function to use for both ListGrid.showGridSummary and
*  ListGrid.showGroupSummary.
*  
*  Default implementation picks up ListGridField.summaryFunction if explicitly specified,
*  otherwise checks for a default summary function based on field type (see
*  SimpleType.setDefaultSummaryFunction). Note that a default summary function will
*  not be supplied if the field represents a DataSourceField.primaryKey or
*  DataSourceField.foreignKey, since it would likely not be meaningful.
* 
* @param {ListGridField} field to check for summary function
*/
getGridSummaryFunction(field:ListGridField): SummaryFunction;

/**
*  When ListGrid.showGridSummary is true this method is called for each field
*  which will show a grid summary value (as described in ListGridField.showGridSummary)
*  to get the summary value to display below the relevant column.
*  
*  The default implementation is as follows:
*  
* - If this is a databound grid and not all data is loaded, returns null for every field
*    
* - Otherwise if ListGridField.getGridSummary is defined, calls that method passing
*      in the current data set for the grid
*    
* - If ListGridField.getGridSummary is undefined, makes use of the
*      ListGrid.getGridSummaryFunction for the field to
*      calculate the summary based on the current data set
*  
*  This method may return an array of values. This implies that the grid summary should show
*  multiple rows. Note that if a field has more than one summaryFunction specified, this method
*  will pick up values from each summary function and return them in an array, meaning
*  these summaries will show up on multiple rows in the grid.
*  
*  This method may be overridden to completely customize the summary value displayed for
*  columns in this grid. An example use case would be when summary information is available on
*  the client and does not need to be calculated directly from the data.
*  
*  If you update this method after the grid has been drawn so that new summaries will be
*  generated from the same data, the changes won't be reflected in any Canvas.redraw
*  or other interaction until the next ListGrid.dataChanged, unless you call
*  ListGrid.recalculateGridSummary.
*  
*  Note: this method will not be called if ListGrid.summaryRowDataSource is
*  specified.
* 
* 
* @param {ListGridField} field for which the summary value should be returned
*/
getGridSummary(field:ListGridField): any;

/**
*  This method returns the data to be displayed in the 
*  ListGrid.summaryRow when ListGrid.showGridSummary is true.
*  
*  By default this will call ListGrid.getGridSummary for each field and generate an array of records
*  containing the resulting values.
*  
*  This method may be overridden for custom grid-summary display, and 
*  may return multiple records if more than one summary row is desired.
* 
* 
*/
getGridSummaryData(): Array<Partial<ListGridRecord>>;

/**
*  If this grid is ListGrid.groupByField, and ListGrid.showGroupSummary is true,
*  this method will be called for each group to return the
*  group summary data to display
*  at the end of the group.
*  
*  By default this will call ListGridField.getGroupSummary if defined for each field and
*  generate an array of records containing the resulting values. If no explicit per-field
*  getGroupSummary method is present, this method will fall back to calling the appropriate
*  ListGridField.summaryFunction.
*  
*  This method may be overridden for custom group-summary display, and
*  may return multiple records if more than one summary row per group is desired.
* 
* 
* @param {Array<Partial<Record>>} the records in the group, for which the summary values                 are being calculated
* @param {Record} object with specified groupValue and groupName for this group
* @param {boolean=} if set to false and the node has existing summary data,                 returns the stored summary data, rather than recalculating
*/
getGroupSummaryData(records:Array<Partial<Record>>, groupNode:Record, recalculate?:boolean): Array<Partial<ListGridRecord>>;

/**
*  Provides access to the summary (see ListGridFieldType fields)
*  value of the record for other fields when called from inside the body of
*  ListGridField.getRecordSummary (since they're not available directly off
*  the record). The behavior is unspecified if not called from inside the
*  ListGridField.getRecordSummary method.
*  
* 
* @param {ListGridRecord} record for which a summary is being generated
* @param {ListGridField | Integer | GlobalId} field, or its number or id
*/
getRecordSummary(record:ListGridRecord, field:ListGridField | Integer | GlobalId): any;

/**
*  Returns whether the supplied record is a group or grid summary record. Useful in conjunction
*  with ListGrid.getGroupMembers for determining which records are group summary records.
* 
* @param {ListGridRecord} Record object such as from ListGrid.getGroupMembers
*/
isSummaryRecord(record:ListGridRecord): boolean;

/**
*  Updates the user formula of the supplied field. This method is preferred over setting the
*  the field property directly since it allows any component dependencies to be updated. If
*  the formula is not passed or undefined, it is assumed it has already been updated and only
*  the dependency propagation logic will run. Known component dependencies are:
*  
* - the cached record values of the formula for this field
*  
* - the common formula variable =&gt; field name map maintained by the component for
*  calls to the FormulaBuilder
* 
* @param {ListGridField | string} field owning the userFormula
* @param {UserFormula=} optional formula to install
*/
setUserFormula(field:ListGridField | string, userFormula?:UserFormula): void;

/**
*  Updates the user summary of the supplied field. This method is preferred over setting the
*  field property directly since it allows any component dependencies to be updated. If the
*  summary is not passed or undefined, it is assumed it has already been updated and only the
*  dependency propagation logic will run.
*  Known component dependencies are:
*  
* - the cached record values of the summary for this field
* 
* @param {ListGridField | string} field owning the userSummary
* @param {UserSummary=} optional summary to install
*/
setUserSummary(field:ListGridField | string, userSummary?:UserSummary): void;

/**
*  Updates the user formula text of the supplied field. This method is preferred over setting
*  the text property directly since it allows any component dependencies to be updated. If the
*  formula text is not passed or undefined, it is assumed it has already been updated and only
*  the dependency propagation logic will run. Known component dependencies are:
*  
* - the cached record values of the formula for this field
* 
* @param {ListGridField | string} field owning the userFormula
* @param {string=} optional formula text to install
*/
setUserFormulaText(field:ListGridField | string, text?:string): void;

/**
*  Updates the user summary text of the supplied field. This method is preferred over setting
*  the text property directly since it allows any component dependencies to be updated. If the
*  summary text is not passed or undefined, it is assumed it has already been updated and only
*  the dependency propagation logic will run. Known component dependencies are:
*  
* - the cached record values of the summary for this field
* 
* @param {ListGridField | string} field owning the userSummary
* @param {string=} optional summary text to install
*/
setUserSummaryText(field:ListGridField | string, text?:string): void;

/**
*  Setter for the ListGrid.showFilterEditor property. Allows the filter editor to be
*  shown or hidden at runtime.
*  
*  By default, hiding the filterEditor will also clear any user-criteria it 
*  specified and refetch the data. To prevent this behavior, you can set 
*  ListGrid.clearCriteriaOnFilterEditorHide to false.
* 
* @param {boolean} true if the filter editor should be shown, false if it should be hidden
*/
setShowFilterEditor(value:boolean): void;

/**
* 
*  If we're showing the filter (query-by-example) row for this ListGrid, this method is
*  used to determine the valueMap to display in the filter row for this field.
*  Default implementation will return the field.filterEditorValueMap if specified, or
*  field.valueMap.
* 
* 
* @param {ListGridField} field definition field for which we need a valueMap
*/
getFilterEditorValueMap(field:ListGridField): ValueMap;

/**
*  If we're showing the filter (query-by-example) row for this ListGrid, this method is
*  used to determine the type of form item to display in the filter edit row for this field.
*  Default implementation will return the field.filterEditorType if specified, or
*  the result of DynamicForm.getEditorType otherwise.
* 
* 
* @param {ListGridField} field definition to get the editorType for
*/
getFilterEditorType(field:ListGridField): string;

/**
*  If ListGrid.showFilterEditor is true, this method will update the criteria shown
*  in the filterEditor without performing a filter.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
* 
* @param {Criteria | AdvancedCriteria} New criteria to show
*/
setFilterEditorCriteria(criteria:Criteria | AdvancedCriteria): void;

/**
*  If ListGrid.showFilterEditor is true, this method will return the criteria currently
*  displayed in the filterEditor. Note that these values may differ from the
*  criteria returned by ListGrid.getCriteria if the filter editor values have been modified
*  without performing an actual filter.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
* 
* @param {boolean=} By default this method will include criteria applied to  fields, including criteria that are not actually visible/editable in the filterEditor for the  grid. Pass in this parameter to get only values for visible fields returned.
*/
getFilterEditorCriteria(omitHiddenFields?:boolean): Criteria | AdvancedCriteria;

/**
*  Extracts and returns the criteria for the passed field from the 
*  ListGrid.showFilterEditor. The result can be an 
*  AdvancedCriteria, if the field in question produces more than one restriction, such
*  as separate greaterThan and lessThan criteria for a range.
* 
* @param {string} name of the field to get the criteria for
*/
getFilterEditorCriterion(fieldName:string): Criterion;

/**
*  Can this cell be edited?
*  
*  The default implementation of canEditCell() respects the various property
*  settings affecting editability:
*  
* - ListGridField.canEdit can be set to disable editing
*      for a field
*    
* - If the grid is bound to a dataSource, the ListGrid.canEditFieldAttribute value on the
*      dataSource field may enable / disable editing
*    
* - a record with the ListGrid.recordEditProperty set to
*      false is not editable
*    
* - disabled records are not editable
*  
*  You can override this method to control editability on a cell-by-cell basis. For example,
*  if you had a grid that allows editing of "orders", and you had a field "shipDate" that is
*  normally editable, but should not be editable if the order is already "complete", you might
*  implement canEditCell() as follows:
*  
*  
*   isc.ListGrid.create({
*     ...
*     canEditCell : function (rowNum, colNum) {
*       var record = this.getRecord(rowNum),
*         fieldName = this.getFieldName(colNum);
*       if (fieldName == "shipDate" &&
*         record.orderStatus == "complete")
*       {
*         return false;
*       }
*       // use default rules for all other fields
*       return this.Super("canEditCell", arguments);
*     }
*   });
*  
*    
*  
*  Notes on providing custom implementations:
*  
*  
* - In order to allow complete control over editing, canEditCell() is called
*  very frequently. If you see delays on row to row navigation, check that your implementation
*  is efficient
*  
* - If you change the editability of a cell on the fly, for example, during
*  ListGrid.editorExit on another cell, call refreshCell() to show or hide the editor
*  
* - If this ListGrid allows new records to be created, canEditCell() may be
*  called when there is no record available, in which case getRecord() will return null. The
*  values input so far by the user are available via ListGrid.getEditValues.
*  
*  
*  
*  For more information on editing, see the editing.
* 
* 
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
canEditCell(rowNum:number, colNum:number): boolean;

/**
*  Can the field be edited? This method looks at ListGrid.canEdit for the grid as well as the
*  ListGridField.canEdit value, to determine whether editing is allowed. This method's
*  return value is not authoritative for editibility since ListGrid.canEditCell could return
*  a more specific value.
*  
*  For a detailed discussion, see the documentation at ListGrid.canEdit.
* 
* 
* @param {ListGridField | number | FieldName} field object, number, or name
*/
fieldIsEditable(field:ListGridField | number | FieldName): boolean;

/**
*  Updates the ListGrid.canRemoveRecords property for this listGrid at runtime.
* 
* @param {boolean} new canRemoveRecords value
*/
setCanRemoveRecords(canRemove:boolean): void;

/**
*  Start inline editing at the provided coordinates.
*  
*  Invoked when a cell is editable and the editEvent occurs on that cell. Can
*  also be invoked explicitly.
*  
*  If this method is called while editing is already in progress, the value from the current
*  editCell will either be stored locally as a temporary edit value, or saved via 'saveEdits()'
*  depending on this.saveByCell, and the position of the new edit cell.
*  Will update the UI to show the editor for the new cell, and put focus in it unless
*  explicitly suppressed by the optional suppressFocus parameter.
* 
* 
* @param {Integer=} Row number of the cell to edit. Defaults to first                 editable row
* @param {Integer=} Column number of the cell to edit. Defaults to first                 editable column
* @param {boolean=} If passed this parameter suppresses the default                 behavior of focusing in the edit form item when                  the editor is shown.
*/
startEditing(rowNum?:Integer, colNum?:Integer, suppressFocus?:boolean): boolean;

/**
*  Start inline editing at a record identified by criteria. If the criteria matches more than
*  one record, the first matched record is edited. Additionally, if the record to be edited is 
*  not visible, the record will be scrolled into view.
*  
*  Note that the record to be matched must already be loaded in the grid - no fetch will be
*  performed.
* 
* 
* @param {Criteria=} Criteria identifying the existing row to edit
*/
editExistingRecord(criteria?:Criteria): void;

/**
*  Notification that the user is no longer hovering over some cell. Hides the current hover canvas
*  if one is showing.
* 
*/
stopHover(): void;

/**
*  Method to retrieve a live edit form item for an ListGrid.canEdit ListGrid.
*  This is the automatically generated editor displayed in a cell while editing the grid.
*  
*  Note that this is an advanced method and developers should be aware of the following issues:
*  
* - Edit form items are created automatically as necessary and their lifecycle is
*  managed by the grid. Items may be created or destroyed as a result of fields being
*  shown and hidden, horizontal scrolling with ListGrid.showAllColumns,
*  row or cell transitions, configuration changes, and other circumstances.
*  This method will return null when an item is not present.
*  Developers should therefore be aware that:
*  
* - There may not be a live item for a field while a cell is not currently editable and 
*   visible to the user.
*  
* - The item returned from a call to getEditItem() can become invalid
*   as the user interacts with a grid. Even if the user is currently editing the field in 
*   question, this method may return a different live item if invoked later in the
*   application flow. Therefore application code should avoid holding onto references 
*   to the items returned by this method, and instead invoke the method directly
*   to dynamically retrieve the current edit item as required.
*  
* - If you change configuration of an edit item created by the grid, your changes will
*   be lost if the item is destroyed and re-created
*  
*  
*  
* - EditFormItems can be thought of as autoChildren. They should be configured via
*  ListGridField.editorProperties (or ListGridField.editorProperties for the
*  ListGrid.showFilterEditor edit items), rather than by setting properties
*  on the live items directly.
*  
* - The items' values are managed by the ListGrid through the edit-values subsystem.
*  If you want to change an edit value for a field, call ListGrid.setEditValue and the
*  grid will handle updating the value in the live item if necessary. You should not need to call
*  setValue(); directly on the item and doing so will not always update the
*  edit value for the grid.
*  
*  In general - bear in mind that this is an advanced usage and if there is an equivalent API
*  available on the ListGrid it is always preferable to use that.
* 
* @param {string | Integer} fieldName or colNum to get the edit item for.
*/
getEditFormItem(field:string | Integer): FormItem;

/**
* 
*  Returns the valueMap to display for a field when it is displayed in the editor while
*  editing some record.
*  Called when a user starts to edit a field, or whenever the field valueMap is updated via
*  a call to ListGrid.setValueMap or ListGrid.setEditorValueMap.
*  Default implementation will return the field.editorValueMap if specified, otherwise
*  field.valueMap - can be overridden to provide a
*  different specific valueMap for some field based on the record/field data.
* 
* 
* @param {ListGridField} field definition field for which we need a valueMap
* @param {object} Field values for record being edited. Note that this will include               the current edit values for fields that have not yet been saved.                May be null, if editing a new record.
*/
getEditorValueMap(field:ListGridField, values:object): ValueMap;

/**
*  Returns the valueIcons for a field when it is displayed in the editor while editing some
*  record. Default implementation will return ListGridField.editorValueIcons if specified
*  otherwise ListGridField.valueIcons
* 
* @param {object} field definition
* @param {object} current edit values for the record
*/
getEditorValueIcons(field:object, values:object): object;

/**
* 
*  Set a valueMap to display for this field while editing.
*  This method sets the ListGridField.editorValueMap property -
*  note that if ListGrid.getEditorValueMap has been overridden it may not make use
*  of this property.
* 
* @param {object | number | FieldName} field object, number, or name
* @param {object} ValueMap to apply to the field
*/
setEditorValueMap(fieldID:object | number | FieldName, map:object): void;

/**
* 
*  Returns the form item type (Class Name) to display for a field when it is displayed in the
*  editor while editing some record.
*  Default implementation will return field.editorType if specified.
*  If not specified, the default form item for the appropriate data type will be displayed
*  - can be overridden to provide a different specific form item type for some field based on
*  the record/field data.
* 
* 
* @param {ListGridField} field definition field for which we need a valueMap
* @param {object} current edit values for the record (may be null, if editing a               new record)
*/
getEditorType(field:ListGridField, values:object): string;

/**
*  Get the default properties for editor form items displayed while editing some field.
*  Overriding this method allows developers to dynamically customize the form item displayed in
*  an editable grid, based on the cell being edited.
*  
*  Note: you should set FormItem.editorType in the returned properties to
*  control the type of form item that is used.
*  
* 
* @param {ListGridField} field whose properties are needed
*/
getEditorProperties(field:ListGridField): object;

/**
*  Refresh an individual cell without redrawing the grid.
*  
*  The cell's value, CSS class, and CSS text will be refreshed, to the current values returned
*  by getCellValue(), getCellStyle() and getCellCSSText() respectively.
* 
* 
* @param {number} row number of cell to refresh
* @param {number} column number of cell to refresh
*/
refreshCell(rowNum:number, colNum:number): void;

/**
*  Refresh an entire row of cells without redrawing the grid.
*  
*  The cells' values, CSS classes, and CSS text will be refreshed, to the current values
*  returned by getCellValue(), getCellStyle() and getCellCSSText() respectively.
* 
* 
* @param {number} row number of cell to refresh
*/
refreshRow(rowNum:number): void;

/**
* 
*  Start editing a new row, after the last pre-existing record in the current set of data.
*  
*  This new row will be saved via the "add" dataSourceOperations.
*  
*  See the editing and also the
*  unsavedRecords for context about how unsaved
*  records behave.
*  
*  You can optionally pass newValues which are the initial values for the newly
*  added record. See also ListGridField.defaultValue as a means of setting default
*  values every time the user begins editing a new record, for instance, by pressing downArrow
*  on the last normal record in the grid when ListGrid.listEndEditAction is "next".
*  
*  If editing is already underway elsewhere in the grid, startEditingNew() behaves just like
*  ListGrid.startEditing.
* 
* 
* @param {Map<any> | Record=} Optional initial set of properties for the new record
* @param {boolean=} Whether to suppress the default behavior of moving focus                  to the newly shown editor.
*/
startEditingNew(newValues?:Map<any> | Record, suppressFocus?:boolean): void;

/**
*  Returns an array of every rowNum for which we have pending (unsubmitted) edits.
*  This will return records that have been marked as removed (see ListGrid.markRecordRemoved
*  as well as records with unsaved changes to field values.
* 
*/
getAllEditRows(): Array<Partial<Integer>>;

/**
*  Returns the current set of unsaved edits for a given row being edited.
* 
* 
* @param {number | object} rowNum of the record being edited, or an Object                   containing values for all the record's primary keys
*/
getEditValues(valuesID:number | object): object;

/**
*  Returns the combination of unsaved edits (if any) and original values (if any) for a given
*  row being edited.
*  
*  The returned value is never null, and can be freely modified.
* 
* 
* @param {number | object} rowNum of the record being edited, or an Object                   containing values for all the record's primary keys
*/
getEditedRecord(valuesID:number | object): object;

/**
*  Returns the current value of a cell. If the cell has an outstanding edit value, this will
*  be returned, otherwise the underlying value of the record will be returned.
* 
* 
* @param {number | object} rowNum of the record being edited, or an Object                   containing values for all the record's primary keys
* @param {number | string} colNum or fieldName of the cell
*/
getEditedCell(record:number | object, field:number | string): any;

/**
* 
*  This method sets up a set of editValues for some row / cell. It differs from
*  'setEditValue()' in that:
*  &nbsp;- it takes values for multiple fields
*  &nbsp;- it clears out any previous edit values for the record
* 
* 
* @param {number} Row number for the record being edited
* @param {object} New values for the row
*/
setEditValues(rowNum:number, values:object): void;

/**
* 
*  Modifies a field value being tracked as an unsaved user edit.
*  Use for suggested or reformatted values for edits that remain unsaved.
* 
* 
* @param {number} Row number (or edit values ID)
* @param {number | string} Column number of cell, or name of field                      having editValue updated
* @param {any} New value for the appropriate field.
*/
setEditValue(rowNum:number, colNum:number | string, value:any): void;

/**
* 
*  Returns the current temporary locally stored edit value for some field within a record
*  being edited.
* 
* 
* @param {number} index of the row for which the editValue should be returned
* @param {number | string} index of the field, or fieldName, for which value should be               returned
*/
getEditValue(rowNum:number, colNum:number | string): any;

/**
* 
*  Clear a field value being tracked as an unsaved user edit.
*  The saved record value will be displayed in the the appropriate cell instead.
*  Will also discard any validation errors for the specified field / row.
*  
*  Note that for edits to an existing record, clearing all edit values will drop the edit session
*  for the row altogether (it will no longer be returned by 
*  ListGrid.getAllEditRows). This is the not the case for unsaved
*  ListGrid.startEditingNew - for these rows edit values will be retained
*  even if they are empty. To explicitly discard a new edit row, use
*  ListGrid.discardEdits instead.
*  
* 
* @param {number | object} Row number, primary keys object for the record,                       or editValues object
* @param {number | string} Column number, or Name of field for which                   the value is to be cleared
*/
clearEditValue(editValuesID:number | object, colNum:number | string): void;

/**
*  Returns the index of the row being edited or null
*  if there is no current edit row.
* 
* 
*/
getEditRow(): Integer;

/**
*  Returns the index of the column being edited or null
*  if there is no current edit column.
* 
* 
*/
getEditCol(): Integer;

/**
* 
*  Returns the field object associated with cell currently being edited
* 
* 
*/
getEditField(): object;

/**
* 
*  Cancel the current edit without saving.
* 
* 
*/
cancelEditing(): void;

/**
* 
*  Complete the current edit by storing the value and hiding the inline editor. Note that
*  if ListGrid.autoSaveEdits is true, the value will be saved to the server.
* 
* 
*/
endEditing(): void;

/**
* 
*  Method to find the next editable cell given a starting row/col, and a direction,
*  either iterating through fields within each row, or checking the same field in each row.
*  
*  Note, this is potentially an expensive method. For example, consider a listGrid where the 
*  user can add rows but not edit any existing rows; in this case, canEditCell()
*  would inspect and reject every row in the dataSet before returning true for the last row.
*  Consider this before making use of this method on grids with large dataSets
* 
* @param {number} Index of starting row
* @param {number} Index of starting column
* @param {number} +1 if searching forward for the next edit cell, -1 if                 searching backwards
* @param {boolean} true if we should check every field in each row -                     false if we should check the same field in each                      row.
* @param {boolean=} Should we check whether the cell passed in is                     editable? Default behavior WILL check that cell -                      pass false to suppress checking that cell.
*/
findNextEditCell(rowNum:number, colNum:number, direction:number, stepThroughFields:boolean, checkStartingCell?:boolean): Array<any>;

/**
* 
*  Cancel outstanding edits, discarding edit values, and hiding editors for the record[s] passed
*  in if appropriate.
*  
*  If no rows are passed in, all outstanding edit values will be dropped. This will not
*  automatically end editing; call ListGrid.endEditing before calling discardAllEdits()
*  if you also want to end editing.
*  
*  Note that this also clears the ListGrid.markRecordRemoved state of any records
*  that have been marked as removed.
* 
* 
* @param {Array<Partial<number>>=} allows you to specify which row(s) to drop edits for
* @param {boolean=} By default this method will hide the editor if               it is currently showing for any row in the grid. Passing in                this parameter will leave the editor visible (and just reset                the edit values underneath the editor).
*/
discardAllEdits(rows?:Array<Partial<number>>, dontHideEditor?:boolean): void;

/**
* 
*  Cancel outstanding edits for the specified rows, discarding edit values, and hiding editors
*  if appropriate.
*  
*  Note that if this method is called on a new edit row (created via
*  ListGrid.startEditingNew for example), which has not yet been saved, this method
*  will remove the row entirely.
*  
*  Also note that this method will clear the ListGrid.markRecordRemoved state
*  of records that have been marked as removed.
* 
* 
* @param {Integer} Row to cancel
* @param {Integer} Column to cancel. Note that this parameter is ignored in ListGrids but            may be required in subclasses of ListGrid where each cell represents             one record in the data set (EG CubeGrid)
* @param {boolean=} By default this method will hide the editor if               it is currently showing for the row in question. Passing in                this parameter will leave the editor visible (and just reset                the edit values underneath the editor).
*/
discardEdits(rowNum:Integer, colNum:Integer, dontHideEditor?:boolean): void;

/**
* 
*  Validates and saves edits within the row currently being edited (or another row with unsaved
*  edits, if indicated).
*  
*  This method can be called to manually trigger saves if the default mechanisms of
*  ListGrid.saveByCell or row by row saving are not suitable.
*  
*  The 'callback' parameter provides a notification when the save attempt completes, which is
*  likely to be asynchronous for databound grids. Cases under which the callback will fire are:
*  
* - Save completed successfully
*  
* - No changes to the edited row, so save not required
*  
* - Validation failure occurred on the client or on the server
*  Note that if this method was unable to determine the row to be saved, the callback will NOT
*  fire - in this case, the method is a no-op.
*  
*  Other, standard callbacks such as ListGrid.editComplete, ListGrid.editFailed and
*  ListGrid.cellChanged will fire normally.
*  
*  Note this method does not hide the inline editors if they are showing - to explicitly save
*  and end editing, use the method 'endEditing()'
*  
*  If this method is called for a row which has been marked for deletion
*  (see ListGrid.markRecordRemoved) it will cause the record to be removed from the
*  data-set.
*  
*  For more information on editing, see the editing.
* 
* 
* @param {EditCompletionEvent=} Event used to complete cell editing.     Optional, and defaults to "programmatic". Can be used by the      callback method to perform custom actions such as navigation when the      save completes.
* @param {Callback=} Callback to fire on completion of the saving process.                  If no edits were made or client-side validation fails                   the callback will be fired synchronously at the end of this                   method.                   Takes the following parameters:                   - rowNum (Number) edited row number                   - colNum (Number) edited column number                   - editCompletionEvent (EditCompletionEvent) event                    passed in (defaults to "programmatic")                   - success (boolean) false if the save was unable to                    complete due to a validation failure or server-side                    error.
* @param {number=} Which row should be saved. If unspecified the current edit row              is saved by default. Note that if there is no current edit row               this method will no op.
*/
saveEdits(editCompletionEvent?:EditCompletionEvent, callback?:Callback, rowNum?:number): void;

/**
*  If this listGrid can be edited, this method will return true if the row passed in has been
*  edited, but the edits have not yet been saved to the ListGrid's data object.
*  
*  Note this method will not return true if a record has been marked as
*  ListGrid.markRecordRemoved, but has no other changes. Developers can use
*  ListGrid.recordMarkedAsRemoved to check for this case.
*  
*  Note that if this grid is bound to a ListGrid.dataSource, and 
*  an asynchronous save has been submitted, this method will compare the local edit values 
*  against the submitted values by default, returning false
*  (no changes), if they match. This is useful for detecting whether the user is actively
*  editing values and hasn't yet committed them.
*  
*  The ignorePendingValues parameter may be used by
*  developers who want to ignore this case and simply compare edit values
*  against the record in the local data set.
*  
* 
* @param {Integer} index of row to check for changes
* @param {boolean=} If true, this method will compare the current edit values against the underlying record in the dataset, not taking pending edit values into account
*/
rowHasChanges(rowNum:Integer, ignorePendingValues?:boolean): boolean;

/**
*  Whether the grid as a whole has any unsaved edits, in any row. Note that this method will return
*  true if any rows are ListGrid.markRecordRemoved in addition to any rows that have
*  unsaved edits.
*  
*  Note that if this grid is bound to a ListGrid.dataSource, and 
*  an asynchronous save has been submitted, this method will compare the local edit values 
*  against the submitted values by default, returning false
*  (no changes), if they match. This is useful for detecting whether the user is actively
*  editing values and hasn't yet committed them.
*  
*  The ignorePendingValues parameter may be used by
*  developers who want to ignore this case and simply compare edit values
*  against the record in the local data set.
*  
* 
* @param {boolean=} If true, this method will compare the current edit values against the underlying records in the dataset, not taking pending edit values into account
*/
hasChanges(ignorePendingValues?:boolean): boolean;

/**
*  If this listGrid can be edited, this method will return true if the cell passed in has been
*  edited, but the edits have not yet been saved to the ListGrid's data object.
* 
* @param {Integer} index of row to check for changes
* @param {Integer} index of the col to check for changes
*/
cellHasChanges(rowNum:Integer, colNum:Integer): boolean;

/**
*  Save a number of outstanding edits for this ListGrid. If no rows are specified, all
*  outstanding edits will be saved.
*  
*  Note that if ListGrid.saveRequestProperties is specified and the grid is performing
*  a databound save, these properties will be applied to each generated DSRequest.
* 
* 
* @param {Array<Partial<number>>=} optionally specify which rows to save
* @param {Callback=} If specified this callback will be fired on a successful save of the specified rows. Note that if there are no pending edits to be saved this callback will  not fire - you can check for this condition using ListGrid.hasChanges or  ListGrid.rowHasChanges. Use ListGrid.editFailed to find out about failures  encountered during saving (on a per-row basis).
*/
saveAllEdits(rows?:Array<Partial<number>>, saveCallback?:Callback): boolean;

/**
*  Validate the current set of edit values for the row in question.
*  
*  Called when the user moves to a new edit row, or when an edited record is to be saved if
*  client side validation is enabled for this grid.
*  
*  This method may also be called directly to perform row level validation at any time.
* 
* 
* @param {Integer} index of row to be validated.
*/
validateRow(rowNum:Integer): boolean;

/**
*  Validate the current edit value for the cell in question. Called when the user moves to a
*  new edit cell if ListGrid.validateByCell is true.
*  This method may also be called directly to perform cell level validation at any time.
* 
* @param {number} index of row to be validated.
* @param {string | number} field name (or column index) of field to be validated
*/
validateCell(rowNum:number, fieldName:string | number): boolean;

/**
*  Returns the message to display when a user attempts to save a required field with an
*  empty value. By default returns Validator.requiredField.
* 
* @param {object} definition of the field being edited
* @param {object} record object being edited
*/
getRequiredFieldMessage(field:object, record:object): string;

/**
*  Does this grid currently have errors associated with editValues for any row in the grid.
* 
*/
hasErrors(): boolean;

/**
*  Does the specified row have unresolved errors?
* 
* @param {Integer} rowNum to check for errors
*/
rowHasErrors(rowNum:Integer): boolean;

/**
*  Given a rowNum and a colNum or fieldName, determine whether we currently have stored
*  validation errors for the record/field in question.
* 
* @param {number} index of row to check for validation errors
* @param {string | number} name of field, or index of column to check for validation                   errors
*/
cellHasErrors(rowNum:number, fieldID:string | number): boolean;

/**
*  Returns any currently stored validation errors for this row in the following format:
*  &nbsp;&nbsp;{fieldName:[array of error messages], ...}
* 
* @param {number} index of row to check for validation errors.
*/
getRowErrors(rowNum:number): object;

/**
*  Returns the current set of errors for this cell.
* 
* @param {number} index of row to check for validation errors.
* @param {string | number} field to check for validation errors - can be fieldName                     or index of the column.
*/
getCellErrors(rowNum:number, fieldName:string | number): Array<Partial<string>>;

/**
*  Set a validation error for some cell.
* 
* @param {number} row index of cell to add validation error for
* @param {string | number} col index or field name of cell to add validation error for
* @param {string | Array<Partial<string>>} validation error/errors for the cell.
*/
setFieldError(rowNum:number, fieldName:string | number, errorMessage:string | Array<Partial<string>>): void;

/**
*  Set the validation errors for some row (replacing any pre-existent validation errors)
*  
*  Note that in the case of a ListGrid.groupByField, or a 
*  TreeGrid, some records may be hidden form view (part of a collapsed group or
*  parent folder). In this case there is no meaningful row number associated with a record.
*  This method cannot be called on such rows - developers should make the row visible first.
*  This is by design - users should always be able to see errors.
*   
* 
* @param {number} row to add validation error for
* @param {any} validation errors for the row in the format             {fieldName:errorMessage, ...} or              {fieldName:[errorMessage1, errorMessage2], ...}
*/
setRowErrors(rowNum:number, errors:any): void;

/**
*  Clears any validation errors for some cell.
* 
* @param {number} row index of cell to add validation error for
* @param {number | string} col index or field name of cell to add validation error for
*/
clearFieldError(rowNum:number, fieldName:number | string): void;

/**
*  Clear any stored validation errors for some row
* 
* @param {number} index of row to clear validation error for
*/
clearRowErrors(rowNum:number): void;

/**
*  Remove a record from this ListGrid.
*  
*  If this grid is bound to a DataSource, it will perform a DataSource "remove" operation to
*  remove records from this component's DataSource.
*  
*  Otherwise the data will be removed from the grid's ListGrid.data object.
*  
*  To make changes to the local data object even when a DataSource is present, use
*  ListGrid.saveLocally.
* 
* 
* @param {Record} listGrid record, or primary key values of record to delete.
* @param {DSCallback=} method to call on operation completion.  Note that if this is method does not trigger a dataSource remove operation, the callback  will still be fired when the data has been removed, but the dsResponse parameter  will be null.
* @param {Partial<DSRequest>=} additional properties to set on any DSRequest                     that will be issued
*/
removeData(data:Record, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Method fired when the user clicks the "remove" icon if ListGrid.canRemoveRecords is true.
*  Default behavior will remove the record from the data set, or if we're
*  ListGrid.deferRemoval mark record as removed [or for records already
*  marked as removed, clear this removed marker].
*  
*  If ListGrid.warnOnRemoval is set, this method will also show a warning dialog to users
*  allowing them to cancel the removal.
*  
*  This method may be called directly to cause a record to be removed or marked for removal
*  as if the user had hit the "remove" icon.
*  
*  
*  May be overridden to perform custom logic on remove click.
* 
* @param {Integer} Row on which the icon was clicked
*/
removeRecordClick(rowNum:Integer): void;

/**
*  Sets the custom tracker HTML to display next to the mouse when the user initiates a drag
*  operation on this component. Default implementation will examine ListGrid.dragTrackerMode
*  and set the custom drag tracker to display the appropriate HTML based on the selected record.
*  
*  To display custom drag tracker HTML, this method may be overridden - call 
*  EventHandler.setDragTracker to actually update the drag tracker HTML.
* 
*/
setDragTracker(): boolean;

/**
*  Return an icon to display as a drag tracker when the user drags some record.
*  Default implementation:
*  If ListGridField.valueIcons is specified for the title field of this grid
*  (see ListGrid.getTitleField), the appropriate value icon will be displayed.
*  If no appropriate valueIcon can be found, the icon will be derived from
*  ListGrid.trackerImage.
*  If multiple records are selected, only the first record is examined for valueIcons.
*  
*  Note: Only called if ListGrid.dragTrackerMode is set to "icon".
* 
* @param {Array<Partial<ListGridRecord>>} Records being dragged
*/
getDragTrackerIcon(records:Array<Partial<ListGridRecord>>): string;

/**
*  Return "title" HTML to display as a drag tracker when the user drags some record.
*  Default implementation will display the cell value for the title field (see
*  ListGrid.getTitleField) for the record(s) being dragged (including any
*  icons / custom formatting / styling, etc).
*  
*  Note: Only called if ListGrid.dragTrackerMode is set to "title".
* 
* @param {ListGridRecord} First selected record being dragged
* @param {number} row index of first record being dragged
*/
getDragTrackerTitle(record:ListGridRecord, rowNum:number): string;

/**
* 
*  This method overrides Canvas.willAcceptDrop and works as follows:
*  
*  
* - If Canvas.willAcceptDrop (the superclass definition) returns false, this
*    method always returns false. This allows Canvas.dragType and
*    Canvas.dropTypes to be used to configure eligibility for drop. By default,
*    a ListGrid has no dropTypes configured and so this check will not prevent a drop.
*  
* - If this is a self-drop, that is, the user is dragging a record within this list, this is
*    an attempted drag-reorder. If ListGrid.canReorderRecords is
*    false, this method returns false.
*  
* - If the EventHandler.getDragTarget is another widget, if
*    ListGrid.canAcceptDroppedRecords is false this method returns false.
*  
* - If a call to ListGrid.getDragData on the dragTarget fails to return
*    an record object or an array of records, this method returns null.
*  
* - If the drop target record is disabled or has ListGridRecord.canAcceptDrop
*    set to false, return false.
*  
*  Note that this method may be called repeatedly during a drag-drop interaction to update the
*  UI and notify the user as to when they may validly drop data.
* 
* 
*/
willAcceptDrop(): boolean;

/**
*  Simulates a drag / drop type transfer of the selected records in some other component to this
*  component, without requiring any user interaction. This method acts on the dropped records 
*  exactly as if they had been dropped in an actual drag / drop interaction, including any 
*  special databound behavior invoked by calling
*  DataBoundComponent.getDropValues for each dropped record.
*  
*  To transfer all data in, for example, a ListGrid, call 
*  ListGrid.selectAllRecords first.
*  
*  Note that drag/drop type transfers of records between components are asynchronous operations:
*  SmartClient may need to perform server turnarounds to establish whether dropped records 
*  already exist in the target component. Therefore, it is possible to issue a call to 
*  transferSelectedData() and/or the ListGrid.drop method of a databound 
*  component whilst a transfer is still active. When this happens, SmartClient adds the 
*  second and subsequent transfer requests to a queue and runs them one after the other. If 
*  you want to be notified when a transfer process has actually completed, either provide a 
*  callback to this method or implement DataBoundComponent.dropComplete.
*  
*  See the dragging documentation for an overview of list grid drag/drop data
*  transfer.
*  
* 
* @param {DataBoundComponent} source component from which the records will be transferred
* @param {Integer=} target index (drop position) of the rows within this grid.
* @param {Callback=} optional callback to be fired when the transfer process has            completed. The callback will be passed a single parameter "records",            the list of records actually transferred to this component.
*/
transferSelectedData(source:DataBoundComponent, index?:Integer, callback?:Callback): void;

/**
*  Handle a drop event. Default implementation supports moving data within this grid or 
*  transferring data into the grid from some other component.
*  
*  Developers wishing to implement custom listGrid record drag and drop behavior should typically
*  use the ListGrid.recordDrop method rather than overriding this method directly.
* 
* 
*/
drop(): boolean;

/**
*  Process a drop of one or more records on a ListGrid record.
*  
*  This method can be overridden to provide custom drop behaviors, and is a more appropriate
*  override point than the lower level Canvas.drop handler.
*  
*  If this is a self-drop, records are simply reordered.
*  
*  For a drop from another widget, ListGrid.transferDragData is called, which
*  depending on the ListGrid.dragDataAction specified on the source
*  widget, may either remove the source records from the original list
*  (dragDataAction:"move") or just provide a copy to this list
*  (dragDataAction:"copy").
*  
*  If this grid is databound, the new records will be added to the dataset by calling
*  DataSource.addData. Further, if the new records were dragged from another
*  databound component, and DataBoundComponent.addDropValues
*  is true, DataBoundComponent.getDropValues will be called for every item
*  being dropped.
*  
*  For multi-record drops, Queuing is automatically used to combine all DSRequests into a
*  single HTTP Request (see QuickStart Guide, Server Framework chapter). This allows the
*  server to persist all changes caused by the drop in a single transaction (and this is
*  automatically done when using the built-in server DataSources with Power Edition and
*  above).
*  
*  Note that reordering records has no effect on a databound grid.
*  
*  The newly dropped data is then selected automatically.
*  
*  If these default persistence behaviors are undesirable, return false to cancel them, then
*  and implement your own behavior, typically by using grid.updateData() or addData() to add
*  new records.
*  NOTE: the records you receive in this event are the actual Records from the source
*  component. Use DataSource.copyRecords to create a copy before modifying the records
*  or using them with updateData() or addData().
*  
*  NOTE: for a drop beyond the last visible record of a ListGrid, targetRecord
*  will be null and the index will be one higher than the last record. This
*  includes a drop into an empty ListGrid, where index will be 0.
* 
* 
* @param {Array<Partial<ListGridRecord>>} records being dropped
* @param {ListGridRecord} record being dropped on. May be null
* @param {Integer} index of record being dropped on
* @param {Canvas} widget where dragging began
*/
recordDrop(dropRecords:Array<Partial<ListGridRecord>>, targetRecord:ListGridRecord, index:Integer, sourceWidget:Canvas): void;

/**
* 
*  During a drag-and-drop interaction, this method is called to transfer a set of records that
*  were dropped onto some other component. This method is called after the set of records has
*  been copied to the other component. Whether or not this component's data is modified is 
*  determined by the value of DataBoundComponent.dragDataAction.
*  
*  With a dragDataAction of "move", a databound component will issue "remove"
*  dsRequests against its DataSource to actually remove the data, via
*  DataSource.removeData.
* 
* 
*/
transferDragData(): Array<any>;

/**
* 
*  During a drag-and-drop interaction, this method returns the set of records being dragged out
*  of the component. In the default implementation, this is the list of currently selected
*  records.
*  
*  This method is consulted by ListGrid.willAcceptDrop.
*  
*  NOTE: If this component is a Tree.isMultiLinkTree TreeGrid, 
*  this method returns a list of NodeLocators rather than a list of records. Each 
*  nodeLocator contains a pointer to the associated record in its node
*  property.
* 
* 
* @param {DataBoundComponent} source component from which the records will be transferred
*/
getDragData(source:DataBoundComponent): Array<Partial<Record>>;

/**
*  Return the pointer to a particular record by record number.
*  Synonym for ListGrid.getCellRecord.
* 
* 
* @param {number} row index of record to return.
*/
getRecord(recordNum:number): ListGridRecord;

/**
*  Return the pointer to a particular record by record number.
*  Notes:
*  - If this is a databound grid, and the record for some row has not yet been loaded,
*   returns the ResultSet.getLoadingMarker, and a fetch will be
*   initialized to retrieve the record from the server.
*  - If this is a new row in an editable ListGrid, and has not yet been saved, this method will
*   return null.
* 
* @param {number} row index of record to return.
*/
getCellRecord(recordNum:number): ListGridRecord;

/**
*  Return the total number of rows in the grid.
*  
*  Note that, when creating new rows via inline editing, this can be more than the total number
*  of rows in the dataset (that is, grid.data.getLength())
* 
* 
*/
getTotalRows(): Integer;

/**
*  Get the array of all currently visible fields for this ListGrid.
*  
*  This list fields is only valid once the ListGrid has been Canvas.draw or once
*  ListGrid.setFields has been called explicitly. If called earlier, only the list of directly
*  specified fields will be returned (the Array passed to create()).
*  
*  This Array should be treated as read-only. To modify the set of visible fields, use
*  ListGrid.showField, ListGrid.hideField and related APIs. To update properties of individual
*  fields, use ListGrid.setFieldProperties or more specific APIs such as
*  ListGrid.setFieldTitle.
*  
*  To get the Array of all fields, including fields that are not currently visible or were
*  specified implicitly, use ListGrid.getAllFields.
* 
* 
*/
getFields(): Array<Partial<ListGridField>>;

/**
*  Get the complete array of fields for this ListGrid, including fields that are not currently
*  visible or were specified implicitly via ListGrid.dataSource.
*  
*  This list of fields is only valid once the ListGrid has been Canvas.draw or once
*  ListGrid.setFields has been called explicitly. If called earlier, only the list of directly
*  specified fields will be returned (the Array passed to create()).
*  
*  This Array should be treated as read-only. To modify the set of visible fields, use
*  ListGrid.showField, ListGrid.hideField and related APIs. To update properties of individual
*  fields, use ListGrid.setFieldProperties or more specific APIs such as
*  ListGrid.setFieldTitle.
* 
* 
*/
getAllFields(): Array<Partial<ListGridField>>;

/**
*  Given a column number or field id, return the field name of a field.
* 
* @param {number | GlobalId} number or id of the field.
*/
getFieldName(colNum:number | GlobalId): string;

/**
*  Given a field name, return the appropriate field definition. Unlike 
*  ListGrid.getField, this method will return the field definition even if
*  it's not visible in the grid.
* 
* 
* @param {FieldName} name of the field to retrieve
*/
getFieldByName(fieldName:FieldName): ListGridField;

/**
*  Given a column number or field name, return the field definition of a field which is visible
*  in the grid. To retrieve the definition of any field, including hidden ones, use
*  ListGrid.getFieldByName.
*  
*  When using DataBoundComponent.fields, the field definition may be
*  a mix of information derived from ListGrid.fields and ListGrid.dataSource.
* 
* 
* @param {Integer | FieldName} number or name of the field
*/
getField(colNum:Integer | FieldName): ListGridField;

/**
*    Given a field or field id, return it's index in the fields array
* 
* @param {string | number} field number or field.name
*/
getFieldNum(fieldID:string | number): Integer;

/**
*  Return the title of a field, specified by name or index.
* 
* @param {string | number} name or index of the field
*/
getFieldTitle(fieldId:string | number): string;

/**
*  Return the summary title of particular field. This is the title of
*  the field to be used in the show / hide fields context menu. Default implementation will
*  use ListGridField.getSummaryTitle or ListGridField.summaryTitle if specified,
*  otherwise ListGridField.title.
* 
* @param {ListGridField} field for which we're returning the title
*/
getSummaryTitle(field:ListGridField): string;

/**
*  Set the ListGridField.valueMap for a field.
*  See also the ListGrid.setEditorValueMap
*  and ListGrid.getEditorValueMap methods which allow further
*  customization of the valueMap displayed while the field is in edit mode.
* 
* @param {string | Integer} Name or index of field to update
* @param {object} ValueMap for the passed field
*/
setValueMap(fieldID:string | Integer, map:object): void;

/**
*  Given a field with a specified ListGridField.valueMap or ListGridField.displayField
*  this method will return the display value for any underlying data value.
* 
* @param {string | number | ListGridField} Field or field identifier with valueMap
* @param {any} Data value for this field
* @param {Record} The record containing the data values to be inspected
*/
getDisplayValue(fieldID:string | number | ListGridField, valueFieldValue:any, record:Record): string;

/**
*  Get the index of the provided record.
*  
*  This is essentially the same as calling listGrid.data.indexOf(record), except that
*  the currently visible range of records is checked first. This is important for
*  responsiveness in functions that respond to user actions when the user is working near the
*  end of a very large dataset (eg 500k records).
* 
* 
* @param {ListGridRecord} the record whose index is to be retrieved
*/
getRecordIndex(record:ListGridRecord): Integer;

/**
*  Synonym of ListGrid.getRecordIndex.
* 
* 
* @param {ListGridRecord} the record whose index is to be retrieved
*/
getRowNum(record:ListGridRecord): Integer;

/**
*  Returns the row number of the most recent mouse event.
* 
* @param {Integer=} optional y-coordinate to obtain row number, in lieu of the y            coordinate of the last mouse event
*/
getEventRow(y?:Integer): Integer;

/**
*  Returns the column number of the most recent mouse event.
* 
* @param {Integer=} optional x-coordinate to obtain column number for, in lieu of the x            coordinate of the last mouse event
*/
getEventColumn(x?:Integer): Integer;

/**
*  Returns the RecordDropPosition for some record drop operation.
*  This value is passed to the ListGrid.recordDrop event notification method.
*  
*  Default implementation determines the position to return based on
*  the specified ListGrid.recordDropAppearance for the grid and the
*  EventHandler.getY.
* 
* 
*/
getRecordDropPosition(): RecordDropPosition;

/**
*  Return the drop-index for a given row and reorderPosition.
*  
*  When there are no rows in the grid, getDropIndex() returns zero.
*  
*  If parameter recordNum is not passed, the current event row is used, see
*  ListGrid.getEventRow.
*  
*  Parameter ReorderPosition indicates where the drop-item should appear
*  in relation to the row at index recordNum. If no reorderPosition is provided, it is
*  calculated based on the physical position of the mouse in the drop-target row when the drop
*  occurs; if the mouse is in the top half of a row, the drop-index is before
*  recordNum. Otherwise, the drop-index is after recordNum.
* 
* 
* @param {number=} Index of the record to drop onto
* @param {ReorderPosition=} Where to drop in relation to recordNum
*/
getDropIndex(recordNum?:number, reorderPosition?:ReorderPosition): number;

/**
*  Called when (cell-based) selection changes within this grid.
* 
* 
* @param {Array<any>} Array of cells whos selected state was modified.
*/
cellSelectionChanged(cellList:Array<any>): boolean;

/**
*  Called when (row-based) selection changes within this grid. Note this method fires for
*  each record for which selection is modified - so when a user clicks inside a grid this
*  method will typically fire twice (once for the old record being deselected, and once for
*  the new record being selected).
*  
*  NOTE: For updating other components based on selections or triggering selection-oriented
*  events within an application, see the
*  DataBoundComponent.selectionUpdated event
*  which is likely more suitable. Calls to ListGrid.getSelection
*  from within this event may not return a valid set of selected records if the event has
*  been triggered by a call to DataBoundComponent.selectAllRecords
*  or DataBoundComponent.deselectAllRecords - in this case
*  use the DataBoundComponent.selectionUpdated event instead.
* 
* 
* @param {ListGridRecord} record for which selection changed
* @param {boolean} New selection state (true for selected, false for unselected)
*/
selectionChanged(record:ListGridRecord, state:boolean): void;

/**
*  Called when the selection changes. Note that this method fires exactly once for any given
*  change to the selection unlike the ListGrid.selectionChanged event.
*  
*  This event is fired once after selection/deselection has completed. The result is
*  one event per mouse-down event. For a drag selection there will be two events fired:
*  one when the first record is selected and one when the range is completed.
*  
*  This event is also fired when selection is updated by a direct call to one of the
*  DataBoundComponent select/deselect methods. Calls on the Selection
*  object do not trigger this event.
* 
* 
* @param {object} first selected record in the selection, if any, which may or may not be the first record in sort order if the DataBoundComponent is sorted. This parameter is typically used when only one record can be selected at a time.
* @param {Array<Partial<object>>} List of records that are now selected
*/
selectionUpdated(record:object, recordList:Array<Partial<object>>): void;

/**
*  Setter method for ListGrid.canFreezeFields
* 
* @param {boolean} New value for listGrid.canFreezeFields
*/
setCanFreezeFields(canFreeze:boolean): void;

/**
*  Return the left offset (in local coordinate space) of a particular column.
* 
* @param {Integer} number of the column
*/
getColumnLeft(colNum:Integer): Integer;

/**
*  Return the left coordinate for a given column number as a GLOBAL coordinate
* 
* @param {Integer} number of the column
*/
getColumnPageLeft(colNum:Integer): Integer;

/**
*  Return the width of a particular column.
* 
* @param {number} number of the column
*/
getColumnWidth(colNum:number): Integer;

/**
*  Returns the page offsets and size of the cell at the passed row and column. If auto-sizing
*  is enabled, sizes are not definitive until the grid has finished drawing, so calling this
*  method before drawing completes will return the configured column sizes.
* 
* @param {number} row index of the cell
* @param {number} column index of the cell
*/
getCellPageRect(rowNum:number, colNum:number): Array<Partial<Integer>>;

/**
*  Freeze the indicated field, so that it remains in place and visible when horizontal
*  scrolling occurs.
* 
* 
* @param {ListGridField | Integer | string | Array<any>} field or fields to freeze. fields may be specified as ListGridField objects, field names or colNum.
*/
freezeField(field:ListGridField | Integer | string | Array<any>): void;

/**
*  Unfreeze a frozen field, so that it will now scroll along with other fields when horizontal
*  scrolling occurs.
* 
* 
* @param {ListGridField | Integer | string | Array<any>} field or fields to unfreeze. fields may be specified as ListGridField objects, field names or colNum.
*/
unfreezeField(field:ListGridField | Integer | string | Array<any>): void;

/**
*  Freeze or unfreeze the indicated field according to whether it is currently frozen.
*  
*  Called when the ListGrid freezes or unfreezes fields by user action.
* 
* 
* @param {ListGridField | Integer | string | Array<any>} field or fields to freeze. fields may be specified as ListGridField objects, field names or colNum.
*/
toggleFrozen(field:ListGridField | Integer | string | Array<any>): void;

/**
*  Returns the primary ListGrid.body, which, when there are frozen fields, is the
*  GridRenderer used to render the non-frozen portion of the dataset; otherwise, the
*  primary body (the only body) is the GridRenderer used to render the entire dataset.
* 
*/
getBody(): GridRenderer;

/**
*  Get the drawn height of a row.
* 
* 
* @param {number} */
getDrawnRowHeight(rowNum:number): number;

/**
*  Return the height this row should be. Default is this.cellHeight. If
*  ListGrid.fixedRecordHeights is false, the row may be rendered taller than this
*  specified size.
*  
*  If records will be variable height,
*  you should switch on ListGrid.virtualScrolling.
*  
*  Note if ListGrid.allowRowSpanning is enabled,
*  this method should return the height of a single row (with rowSpan set to 1).
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord
* @param {number} row number
*/
getRowHeight(record:ListGridRecord, rowNum:number): number;

/**
*  This method is called to retrieve the ListGrid.rollOverCanvas when
*  the user moves over a new row or cell if ListGrid.showRollOverCanvas is true,
*  or when the user moves over the selected record if ListGrid.showSelectedRollOverCanvas 
*  is true.
*  
*  The default implementation uses the AutoChild subystem to create the ListGrid.rollOverCanvas
*  auto child. It may be overridden for custom behavior.
*  
*  Note that for efficiency this should not typically create a new Canvas every time that it is
*  called. Instead usually a single rollOver canvas should be created and updated to reflect
*  the current rollOver row if necessary.
*  
*  Return null to avoid showing a rollOverCanvas for this row.
*  
*  See also ListGrid.getFrozenRollOverCanvas.
*  
* 
* 
* @param {Integer} index of the current rollOver row.
* @param {Integer} index of the current rollOver column. This parameter will be null unless ListGrid.useCellRollOvers is true for the grid.
*/
getRollOverCanvas(rowNum:Integer, colNum:Integer): Canvas;

/**
*  For grids with frozen columns, this method is called to retrieve the 
*  ListGrid.frozenRollOverCanvas when
*  the user moves over a new row or cell if ListGrid.showRollOverCanvas is true,
*  or when the user moves over the selected record if ListGrid.showSelectedRollOverCanvas 
*  is true.
*  
*  The default implementation uses the AutoChild subystem to create the ListGrid.frozenRollOverCanvas
*  based on the rollOverCanvas auto child settings. 
*  It may be overridden for custom behavior.
*  
*  Note that for efficiency this should not typically create a new Canvas every time that it is
*  called. Instead usually a single rollOver canvas should be created and updated to reflect
*  the current rollOver row if necessary.
*  
*  Return null to avoid showing a rollOverCanvas for this row.
*  
*  See also ListGrid.getRollOverCanvas.
*  
* 
* 
* @param {Integer} index of the current rollOver row.
* @param {Integer} index of the current rollOver column. This parameter will be null unless ListGrid.useCellRollOvers is true for the grid.
*/
getFrozenRollOverCanvas(rowNum:Integer, colNum:Integer): Canvas;

/**
*  This method is called to retrieve the ListGrid.rollUnderCanvas when
*  the user moves over a new row or cell if 
*  ListGrid.showRollUnderCanvas or showing a
*  ListGrid.showSelectedRollUnderCanvas.
*  
*  The default implementation uses the AutoChild subystem to create the ListGrid.rollUnderCanvas
*  auto child. It may be overridden for custom behavior.
*  
*  Note that for efficiency this should not typically create a new Canvas every time that it is
*  called. Instead usually a single rollOver canvas should be created and updated to reflect
*  the current rollOver row if necessary.
*  
*  Return null to avoid showing a rollUnderCanvas for this row.
*  
*  See also ListGrid.getFrozenRollUnderCanvas.
*  
* 
* 
* @param {Integer} index of the current rollOver row.
* @param {Integer} index of the current rollOver column. This parameter will be null unless ListGrid.useCellRollOvers is true for the grid.
*/
getRollUnderCanvas(rowNum:Integer, colNum:Integer): Canvas;

/**
*  For grids with frozen columns, this method is called to retrieve the 
*  ListGrid.frozenRollUnderCanvas when
*  ListGrid.showRollUnderCanvas or showing a
*  ListGrid.showSelectedRollUnderCanvas.
*  
*  The default implementation uses the AutoChild subystem to create the ListGrid.rollUnderCanvas
*  auto child. It may be overridden for custom behavior.
*  
*  Note that for efficiency this should not typically create a new Canvas every time that it is
*  called. Instead usually a single rollOver canvas should be created and updated to reflect
*  the current rollOver row if necessary.
*  
*  Return null to avoid showing a rollUnderCanvas for frozen fields for this row.
*  
*  See also ListGrid.getRollUnderCanvas.
*  
* 
* 
* @param {Integer} index of the current rollOver row.
* @param {Integer} index of the current rollOver column. This parameter will be null unless ListGrid.useCellRollOvers is true for the grid.
*/
getFrozenRollUnderCanvas(rowNum:Integer, colNum:Integer): Canvas;

/**
*  Method to update properties on a field's header button at runtime.
*  This property allows customization of any settable properties on the ListGridField's
*  header button after it has been generated. 
* 
* @param {string} Field to update
* @param {Partial<Canvas>} new properties to apply to the header button
*/
setFieldButtonProperties(name:string, properties:Partial<Canvas>): void;

/**
*  Method to update properties on a headerSpan's header button at runtime.
*  This property allows customization of any settable properties on the HeaderSpan's
*  header button after it has been generated.
* 
* @param {string} HeaderSpan.name of span to update
* @param {Partial<Canvas>} new properties to apply to the header button
*/
setHeaderSpanButtonProperties(name:string, properties:Partial<Canvas>): void;

/**
*  Modify the height of a listGrid. To hide the header set height to zero.
* 
* @param {number} new height for the header
*/
setHeaderHeight(height:number): void;

/**
*  Show or hide the ListGrid header.
* 
* @param {boolean} true to show the header, false to hide it.
*/
setShowHeader(show:boolean): void;

/**
*  Is the field title for the specified field clipped?
* 
* 
* @param {number} field number for the header button title to test
*/
headerTitleClipped(fieldNum:number): boolean;

/**
*  Returns the HTML that is displayed by the default ListGrid.headerHover
*  handler. Return null or an empty string to cancel the hover.
*  
* 
* 
* @param {number} field number for the header that was hovered
* @param {HTMLString} the HTML that would have been displayed by default
*/
headerHoverHTML(fieldNum:number, defaultHTML:HTMLString): HTMLString;

/**
*  Handle a click in the list header.
*  
*  By default, calls ListGrid.sort to sort by the field that was clicked, if
*  ListGrid.canSort, and calls ListGrid.autoFitField if
*  ListGrid.canAutoFitFields is true and ListGrid.headerAutoFitEvent is set to "click".
* 
* 
* @param {number} field number for the header that was clicked
*/
headerClick(fieldNum:number): void;

/**
*  Handle a double click in the list header.
*  
*  By default, calls ListGrid.autoFitField if ListGrid.canAutoFitFields is true and
*  ListGrid.headerAutoFitEvent is "doubleClick".
* 
* 
* @param {number} field number for the header that was clicked
*/
headerDoubleClick(fieldNum:number): void;

/**
*  Reorder a particular field
* 
* 
* @param {number} Number of the field to reorder
* @param {number} New position for that field
*/
reorderField(fieldNum:number, moveToPosition:number): void;

/**
*  Reorder a set of adjacent fields, from start to end exclusive at the end, by distance
*  moveDelta.
* 
*  NOTE: start and end coordinates are in terms of the currently visible fields, not the full set of
*  fields.
* 
* 
* @param {number} Start of the range of fields to move, inclusive
* @param {number} End of the range of fields to move, non-inclusive
* @param {number} Distance to move by
*/
reorderFields(start:number, end:number, moveDelta:number): void;

/**
*  Setter method for updating ListGrid.canResizeFields at runtime.
* 
* @param {boolean} new value for this.canResizeFields
*/
setCanResizeFields(canResize:boolean): void;

/**
*  Resize a particular field to a new width. Note that this method
*  will also set ListGridField.autoFitWidth to false if it was previously true.
* 
* 
* @param {number} Number of the field to resize
* @param {number} New width of the field
*/
resizeField(fieldNum:number, newWidth:number): void;

/**
*  Notification method fired when the user right-clicks on the corner
*  ListGrid.sorterConstructor. Return false to suppress the default
*  behavior of showing the sorter's context menu.
* 
* 
*/
sorterContextClick(): boolean;

/**
*  If ListGrid.showHeaderContextMenu is true this method is fired
*  when the user right-clicks on the header for this grid.
*  Default implementation will display a menu with entries derived from
*  ListGrid.getHeaderContextMenuItems for the appropriate column.
* 
* @param {Canvas} which button in the header received the right-click event            (may be the sorter button or header menu button)
* @param {Array<any>=} Optional 2-element array specifying position at which the menu should              be shown. If this is not passed in the menu will be shown at the              mouseEvent position (default context menu behavior).
*/
displayHeaderContextMenu(target:Canvas, position?:Array<any>): void;

/**
*  If we're showing a ListGrid.showHeaderContextMenu for this grid and
*  ListGrid.canFreezeFields is true, this string will be shown as the
*  title for the menu item to toggle whether a field is frozen or unfrozen.
*  
*  Default implementation evaluates and returns ListGrid.freezeFieldText or
*  ListGrid.unfreezeFieldText depending on whether the field is currently frozen.
* 
* @param {ListGridField} field to get the menu item title for
*/
getToggleFreezeText(field:ListGridField): HTMLString;

/**
*  If we're showing a ListGrid.showHeaderContextMenu for this grid and
*  ListGrid.canGroupBy is true, this string will be shown as the
*  title for the menu item to toggle the group by setting for a field.
*  Default implementation evaluates and returns the dynamic ListGrid.groupByText
*  string.
* 
* @param {ListGridField} field to get the menu item title for
*/
getGroupByText(field:ListGridField): HTMLString;

/**
*  Return the menus items that should be shown in a menu triggered from a 
*  ListGrid.headerSpans. The default implementation returns the parent 
*  element's context menu, unless ListGrid.showHeaderSpanContextMenu is 
*  true, in which case it returns standard items for showing / hiding fields and 
*  freezing / unfreezing header spans. Note that no column picker will be shown unless 
*  ListGrid.showTreeColumnPicker is true.
* 
* 
* @param {HeaderSpan} the component representing the headerSpan. This component will              have all the properties specified via ListGrid.headerSpans.
*/
getHeaderSpanContextMenuItems(headerSpan:HeaderSpan): Array<Partial<MenuItem>>;

/**
*  Returns the current search-operator applied to criteria for a given field in this grid's 
*  ListGrid.filterEditor. Typically, this will be the operator most 
*  recently selected by the user, or applied by a call to ListGrid.setCriteria or
*  similar..
*  
*  If no operator has been applied by the user, the result is the default provided 
*  ListGridField.filterOperator, or null, indicating a default 
*  operator according to data-type.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use
*  them, see the gridFiltering.
* 
* @param {string} name of the field to get the search operator for
*/
getFieldSearchOperator(fieldName:string): void;

/**
*  Clears the current search operator from a field in the grid's 
*  ListGrid.filterEditor. This will reset the field to its 
*  ListGridField.filterOperator and, unless 
*  ListGrid.alwaysShowOperatorIcon is true, hide the field's operatorIcon.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use
*  them, see the gridFiltering.
*  
*  If ListGrid.filterOnKeypress is true, a fetch may be issued when the operator
*  is cleared - see ListGrid.setFieldSearchOperator for details. To prevent this fetch,
*  pass the suppressFilter parameter.
*  
*  To retrieve a field's current search operator, use ListGrid.getFieldSearchOperator. 
*  To programmatically modify a field's current search operator, use 
*  ListGrid.setFieldSearchOperator.
*  
*  This method has no effect if no specific operator has been set on the field, either by the 
*  user or as a result of other criteria applied by ListGrid.setCriteria or similar.
* 
* @param {string} name of the field to clear the search operator from
* @param {boolean} prevent this call from causing a filter as a result of        operator change - clearing, eg, "isNull" criteria will cause a refilter
*/
clearFieldSearchOperator(fieldName:string, suppressFilter:boolean): void;

/**
*  Applies a new OperatorId to a field in the grid's 
*  ListGrid.filterEditor, and updates it's operatorIcon.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use
*  them, see the gridFiltering.
*  
*  Depending on the field's current filter-value and operator, calls to this method may result 
*  in the current filter-value being cleared, and/or a fetch being issued if 
*  ListGrid.filterByCell or
*  ListGrid.filterOnKeypress are set.
*  
*  In general, if the field has a current filter-value, it will be cleared if
*  
*  
* - the new operator does not require a value (operator-type 
*    OperatorValueType - OperatorId, 
*    for example)
*  
* - the current and new operators have different 
*    OperatorValueType and ListGrid.allowFilterExpressions
*    is false
*  
* - the item has an FormItem.optionDataSource or valueMap which no
*    longer includes the current value
*  
*  
*  If filterOnKeypress is true, a fetch will be issued if 
*  
* - the value was just cleared, in one of the ways listed above
*  
* - the operator changes while there is a current value
*  
* - the new operator does not require a value and the old one did
*  
*  
*  If passed null, the field's operator will be cleared, reverting it to its 
*  ListGridField.filterOperator and, if 
*  ListGrid.alwaysShowOperatorIcon is false, the operatorIcon is hidden. See 
*  ListGrid.clearFieldSearchOperator for an alternative way to clear a field's operator. 
*  
*  To retrieve a field's current search operator, use ListGrid.getFieldSearchOperator. 
*  
* 
* @param {string} name of the field to apply a new search operator to
* @param {OperatorId} id of the search operator to apply
*/
setFieldSearchOperator(fieldName:string, operator:OperatorId): void;

/**
*  If ListGrid.showHeaderContextMenu is true this method returns
*  the menu items to be displayed in the default header context menu.
*  
*  This method is called at various times, including during redraws, as the mouse moves over a
*  ListGrid header button and each time the menu is actually displayed - this allows for 
*  dynamic content depending on the current state of the grid and its fields.
*  
*  Consequently, this method should not instantiate any classes, because they'll be re-created
*  on each call, resulting in a leak - your implementation should return an array of menuItem 
*  config-blocks only, so you shouldn't instantiate actual Menu instances to apply as the 
*  MenuItem.submenu of items - instead, set submenu to a simple array of
*  menuItems. If your use-case necessitates that class instances are created, because specific
*  submenus have a different Menu class, for example, you should keep a reference to them and 
*  either, if their content is dynamic, destroy and recreate them with the new items, or just
*  return the existing instances otherwise. 
*  
*  The default set of menu items includes items for built-in ListGrid features like showing 
*  and hiding fields, freezing fields or grouping by them, and other functions.
* 
* 
* @param {Integer=} Index of the field the user clicked in the  ListGrid.fields array. Note: if the user right-clicked the sorter button   this parameter will be null.
*/
getHeaderContextMenuItems(fieldNum?:Integer): Array<Partial<MenuItem>>;

/**
*  This method, the equivalent of the builtin 
*  ListGrid.clearFilterText, clears all user-visible criteria 
*  applied to this grid, including values and filter-operators in the
*  ListGrid.showFilterEditor and criteria in the 
*  ListGrid.allowFilterWindow, and issues a re-filter.
* 
*/
clearAllCriteria(): void;

/**
*  Clears criteria applied to this grid via the 
*  ListGrid.allowFilterWindow and issues a re-filter.
*  
*  To clear all user-editable criteria, see ListGrid.clearAllCriteria.
* 
*/
clearFilterWindowCriteria(): void;

/**
*  Shows the dialog for ListGrid.filterWindowCriteria allowing end-users to edit
*  the advanced filter. This method can be called directly but it is also used to show the
*  dialog when ListGrid.allowFilterWindow is enabled and the user chooses the
*  ListGrid.advancedFilteringText menu option.
*  
*  Note: this feature requires
*  SmartClient Pro or better.
* 
* 
*/
showFilterWindow(): void;

/**
*  Setter for ListGrid.filterWindowCriteria.
* 
* @param {Criteria} criteria for advanced filtering
*/
setFilterWindowCriteria(criteria:Criteria): void;

/**
*  Returns the current ListGrid.filterWindowCriteria.
* 
* 
*/
getFilterWindowCriteria(): Criteria;

/**
*  Notification method fired when the user clicks on the corner
*  ListGrid.sorterConstructor. Return false to suppress the sort.
* 
* 
*/
sorterClick(): boolean;

/**
*  Turn sorting off, typically because data has changed and is no longer sorted.
*  
*  Calling unsort() disables visual indication of which columns are sorted,
*  and calls unsort() on the underlying dataset if supported.
*  
*  Note that a grid viewing a paged dataset may not be able to support ResultSet.unsort
*  because the sort order is what establishes the row numbering that allows data to be
*  fetched in batches. In this case the dataset will be explicitly sorted to an empty 
*  array which may cause the cache to be invalidated and a new fetch performed.
*  
*  unsort() is automatically called in response to edits or changes to the
*  data set that would cause records to be reordered.
* 
* 
*/
unsort(): void;

/**
*  If a list has become unsorted due to data modification or a call to
*  ListGrid.unsort, this method will resort the list by the previous
*  ListGrid.setSort array, if there is one, or by the previous
*  sort-field and -direction.
* 
* 
*/
resort(): void;

/**
*  Sort this grid's data, with the option to explicitly specify a single field to
*  sort by and sort direction.
*  
*  If sortField is not provided and listGrid.sortField is undefined, the data will be sorted
*  by the first sortable column according to ListGridField.sortDirection if specified,
*  or ListGrid.sortDirection.
*  
*  ListGrids also support multiple-field sorting. See ListGrid.setSort for details.
*  
*  Note that for editable grids, sorting is performed by underlying data values, not
*  for unsaved ListGrid.getEditValues. When the user saves
*  edits in a sorted grid, the grid will automatically be ListGrid.unsort. See
*  editing for further details.
* 
* 
* @param {string | number=} the field name or column number to sort by
* @param {SortDirection=} the direction to sort in
*/
sort(sortField?:string | number, sortDirection?:SortDirection): boolean;

/**
*  Returns the current sort field for this grid. Note that if ListGrid.setSort has been
*  used to sort by multiple fields, you can call ListGrid.getSort to retrieve details about
*  the complete sort applied to the grid.
* 
* 
*/
getSortField(): string;

/**
*  Show a dialog to configure the sorting of multiple fields on this component. Calls through
*  to MultiSortDialog.askForSort, passing this component as the fieldSource and the
*  current DataBoundComponent.getSort if there is one.
*  
*  The generated multiSortDialog can be customized via ListGrid.multiSortDialogDefaults, 
*  ListGrid.multiSortDialogProperties.
* 
* 
*/
askForSort(): void;

/**
*  Toggles the sort-direction of the field with the passed name and resorts the grid.
* 
* @param {string} The name of a field, visible, hidden or existing only in the  dataSource
*/
toggleSort(fieldName:string): void;

/**
*  Adds another SortSpecifier to this grid's sort configuration and resorts.
* 
* @param {SortSpecifier} A SortSpecifier object indicating an additional field  and direction to sort by
*/
addSort(sortSpecifier:SortSpecifier): void;

/**
*  Returns the current SortSpecifier for this ListGrid. Will return null if this grid
*  has never been sorted (and has no specified ListGrid.initialSort or ListGrid.sortField).
*  
*  Note that if sorting was applied via ListGrid.sort [rather than ListGrid.setSort] the
*  sortSpecifiers returned will have been created based on the specified sort field / direction
*  passed into ListGrid.sort.
* 
* 
*/
getSort(): Array<Partial<SortSpecifier>>;

/**
*  Returns the SortSpecifier for the passed fieldName, or null if the field is not
*  sorted.
* 
* @param {string} The name of a field, visible, hidden or existing only in the dataSource
*/
getSortSpecifier(fieldName:string): SortSpecifier;

/**
*  Returns the number of fields involved in this grid's current sort configuration.
* 
*/
getSortFieldCount(): number;

/**
*  Returns true if the passed fieldName is in the current sort-specification.
* 
* @param {string} The name of a field, visible, hidden or existing only in the dataSource
*/
isSortField(fieldName:string): boolean;

/**
*  When multiple fields are sorted, this method returns the HTML for the sort-numeral that
*  appears after the sort-arrows in the header-buttons of sorted fields. If you don't want
*  sort-numerals in the header-buttons, you can override this
*  method to return null or an empty string, or set ListGrid.showSortNumerals to false.
*  
*  Note that the sortIndex passed in is zero based. The default implementation of this method
*  returns an HTML element with the ListGrid.sortNumeralStyle applied to it, containing the
*  specified sortIndex incremented by 1 (therefore showing the more user-friendly 1-based
*  numbering system).
* 
* 
* @param {string} The name of a sort-field to get the ListGrid.sortNumeralStyle HTML for.
* @param {Integer} The sort index for the field.
*/
getSortNumeralHTML(fieldName:string, sortIndex:Integer): HTMLString;

/**
*  This method clears any existing sort on this grid by calling ListGrid.setSort with
*  a null parameter. The internal list of SortSpecifiers is removed and the grid is
*  unsorted.
* 
*/
clearSort(): void;

/**
*/
getSortByGroupFirst(): void;

/**
* @param {boolean} */
setSortByGroupFirst(sortByGroupFirst:boolean): void;

/**
*  Optional notification fired when either user or framework code calls 
*  ListGrid.setSort. This notification fires before the default behavior; 
*  return false from the handler
*  to cancel the default behavior. Note,
*  the notification is fired before the default functionality, but after prechecks 
*  have completed; your method will only be called if the default behavior would have been
*  called. For example, if there are pending edits and the user does not confirm that these 
*  should be saved, normal sorting would not have gone ahead, so equally your handler will 
*  not be called.
*  
*  The default setSort() method does two things to reflect the set of 
*  SortSpecifiers passed to it:
*  
* - Change the grid UI (show directional arrows, numerals to indicate sort priority, etc)
*  
* - Actually sort the grid data
*  
*  If your reason for implementing a custom setSortHandler() is to inhibit or 
*  replace one of those behaviors, you should cancel the default behavior and directly invoke
*  just that part of it you require. The following implementation will replicate the default
*  behavior:
*   setSortHandler : function(sortSpecifiers) {
*     this.displaySort(sortSpecifiers);
*     this.applySortToData(sortSpecifiers);
*     return false; // Prevent the framework from running its own default impl
*   }
*  
* 
* 
* @param {Array<Partial<SortSpecifier>>} Array of SortSpecifier objects
*/
setSortHandler(sortSpecifiers:Array<Partial<SortSpecifier>>): boolean;

/**
*  Sort the grid on one or more fields.
*  
*  Pass in an array of SortSpecifiers to have the grid's data sorted by the fields in
*  each SortSpecifier.property and in the directions specified. The
*  grid can be sorted by any combination of fields, including fields specified in the fields
*  array, whether visible or hidden, and DataSource.fields, if there is one.
*  
*  If multiple fields are sorted, those that are visible show a directional icon and a small
*  ListGrid.sortNumeralStyle
*  indicating that field's index in the sort configuration.
*  
*  See ListGrid.addSort and ListGrid.toggleSort APIs
*  for information on making changes to the current sort configuration.
*  
*  Note that for editable grids, sorting is performed by underlying data values, not
*  for unsaved ListGrid.getEditValues.
* 
* 
* @param {Array<Partial<SortSpecifier>>} Array of SortSpecifier objects
*/
setSort(sortSpecifiers:Array<Partial<SortSpecifier>>): void;

/**
*  Modify the grid UI to reflect the parameter sortSpecifiers. For a single sortSpecifier, 
*  this consists of marking the field with a directional arrow in its header button (if it 
*  visible).
*  
*  If multiple fields are sorted, those that are visible show a directional icon and a small
*  ListGrid.sortNumeralStyle indicating that field's index in the sort 
*  configuration.
*  
*  See ListGrid.addSort and ListGrid.toggleSort APIs
*  for information on making changes to the current sort configuration.
*  
*  NOTE: This method is primarily used by ListGrid.setSort; it is not intended 
*  to be called by user code, unless you are implementing a custom 
*  ListGrid.setSortHandler). For the normal use case, calling this 
*  method directly will fail to execute vital pre-steps. If you are not implementing a custom
*  handler as described above, do not call this method directly - call setSort() 
*  instead.
* 
* 
* @param {Array<Partial<SortSpecifier>>} Array of SortSpecifier objects
*/
displaySort(sortSpecifiers:Array<Partial<SortSpecifier>>): void;

/**
*  Sort the grid's data to reflect the parameter sortSpecifiers.
*  
*  NOTE: This method is primarily used by ListGrid.setSort; it is not intended 
*  to be called by user code, unless you are implementing a custom 
*  ListGrid.setSortHandler). For the normal use case, calling this 
*  method directly will fail to execute vital pre-steps. If you are not implementing a custom
*  handler as described above, do not call this method directly - call setSort() 
*  instead.
* 
* 
* @param {Array<Partial<SortSpecifier>>} Array of SortSpecifier objects
*/
applySortToData(sortSpecifiers:Array<Partial<SortSpecifier>>): void;

/**
*  Attaches the component to the provided record. If position is specified as
*  "within" Canvas.snapTo and Canvas.snapOffsetLeft,
*  Canvas.snapOffsetTop may be set to specify where the component
*  will render within the cell or record. If unset, for components embedded within a record
*  we will default to embedding at the top/left coordinate, and for components embedded within
*  a cell, we will respect the align / valign properties for the cell in question. Any
*  percentage sizing will be interpreted as percentage of row size.
*  
*  Otherwise it will appear to be embedded within the record, underneath the field values.
*  
*  Embedded components become children of the grid and will stay attached to a record through
*  scrolling, sorting and other operations that cause records to shift position.
*  
*  If position is set to "expand", embedded components may offer a
*  resize interface, eg, by setting ListGridField.canDragResize:true,
*  and the grid will react accordingly, growing or shrinking the record to match the embedded
*  component's new extents.
*  
*  Embedded components can be explicitly removed with ListGrid.removeEmbeddedComponent.
*  
*  If a record is removed from the dataset or is replaced in the dataset, for example, it is
*  eliminated through filtering (removes record) or is successfully edited in a databound grid
*  (replaces record), the component is cleared but not logically removed from the grid. It is the
*  responsibility of code that sets up the embedded component to remove it if the record is removed
*  from the dataSet.
*  
*  When embedding components will result in variable height records,
*  you should switch on ListGrid.virtualScrolling.
* 
* 
* @param {Canvas} component to embed
* @param {ListGridRecord} record to attach the component to
* @param {Integer=} rowNum of the record to attach the component to
* @param {Integer=} colNum in which to embed the component
* @param {EmbeddedPosition=} positioning with respect to the record or cell (Defaults to "expand").
*/
addEmbeddedComponent(component:Canvas, record:ListGridRecord, rowNum?:Integer, colNum?:Integer, position?:EmbeddedPosition): void;

/**
*  Removes an embedded component previously associated with the provided record. If a Canvas
*  is passed as the record parameter, it is assumed to be a component and the
*  record is detected automatically from it. If
*  destroyOnUnEmbed is true for the component, it will also be
*  destroyed.
* 
* 
* @param {ListGridRecord | Canvas} record that the component was previously attached   to or the component itself
* @param {Integer | Canvas=} component to unembed, or the colNum in which it appears
*/
removeEmbeddedComponent(record:ListGridRecord | Canvas, component?:Integer | Canvas): void;

/**
*  Retrieve the ListGrid.showRecordComponents currently being shown at the given
*  coordinates.
*  
*  recordComponents are dynamically assigned to row/cell coordinates and, depending
*  on the ListGrid.recordComponentPoolingMode, any kind of redraw of the containing
*  ListGrid (due to sort change, scrolling, editing etc) may cause a
*  recordComponent to be assigned to another row, Canvas.clear or
*  permanently Canvas.destroy.
*  
*  Hence you should always call getRecordComponent() right before taking action on
*  the recordComponent - don't cache the component associated with row/cell
*  coordinate. Similarly, it's invalid to call getRecordComponent() during a redraw
*  (for example, from ListGridField.formatCellValue code).
*  
*  It's always invalid to try to use a recordComponent outside of a ListGrid (by eg
*  adding it to some other layout).
*  
*  If ListGrid.showRecordComponentsByCell is true and the colNum parameter is not passed, the call
*  will return the first component in the passed rowNum.
*  
*  Returns null if there is no component at the specified coordinates.
* 
* 
* @param {Integer} row number to get record component for
* @param {Integer=} optional column number to get the record component for
*/
getRecordComponent(rowNum:Integer, colNum?:Integer): Canvas;

/**
*  Chart the data in this listGrid as a multi-series chart.
*  
*  Each row provides a series of data. Each series of data is labeled by a value from one
*  column, called the labelField.
*  
*  For example, cell values are sales figures, and fields are "Product", "August",
*  "September", "October". In this case each row gives a series: sales figures for each of 3
*  months. The labelField in this case is the "Product" field, meaning each row
*  represents sales figures for each of 3 months for a particular product. This dataset
*  can be charted via any multi-series chart: stacked or clustered bar or column chart, line
*  chart with multiple lines, or area chart (stacked lines).
*  
*  By default, all visible fields other than the label field are assumed to be labels for
*  series values, but an explicit list of fields can be provided as dataFields.
*  
*  By default, all data is charted if all data is loaded, otherwise, data visible in the
*  viewport is charted. An explicit set of rows can be provided via dataRows.
* 
* 
* @param {string} name of the field
* @param {Array<Partial<string>>=} optional list of fields to use as labels. By   default, all fields are used.
* @param {Array<Partial<ListGridRecord>>=} set of records to chart. Can be obtained by eg   ResultSet.getRange.
* @param {Partial<FacetChart>=} properties to pass to the created chart
* @param {boolean=} if true, use the labelField as the "first" set of labels,   for example, as the bar labels in a stacked bar chart, whereas the second set of labels    would appear as the legend.
*/
chartData(labelField:string, dataFields?:Array<Partial<string>>, dataRows?:Array<Partial<ListGridRecord>>, chartProperties?:Partial<FacetChart>, labelFieldFirst?:boolean): FacetChart;

/**
*  Chart a single row of data, with each cell value labeled by the column header.
* 
* 
* @param {number} row to chart
* @param {Array<Partial<string>>=} optional list of fields to use as labels. By   default, all fields are used.
* @param {Partial<Chart>=} properties to pass to the created chart
*/
chartRow(rowNum:number, dataFields?:Array<Partial<string>>, chartProperties?:Partial<Chart>): Chart;

/**
*  Chart a single column of data, with each cell value labeled by a value from another
*  column.
* 
* 
* @param {string} name of the ListGridField to use as a data
* @param {string} name of the ListGridField to use as labels for data
* @param {Partial<Chart>=} properties to pass to the created chart
*/
chartColumn(dataColumn:string, labelColumn:string, chartProperties?:Partial<Chart>): Chart;

/**
*  Loads all records that match this grid's current filter-criteria, optionally firing a 
*  callback when the data arrives.
*  
*  If the length of the data is ResultSet.lengthIsKnown, or is greater than 
*  the passed maxRecords, this call returns false. No fetch 
*  is issued and the callback, if passed, is not fired.
*  
*  If all data is ResultSet.allMatchingRowsCached, no fetch is issued 
*  and this call returns true. The callback, if passed, will be fired, but its 
*  parameters will be null, since there was no fetch to provide the values from.
*  
*  In all other cases, this call returns true and a fetch is issued for all necessary records. 
*  When the data arrives, the callback is fired.
* 
* 
* @param {Integer=} optional maximum record count - if passed, no fetch takes place                if maxRecords is below the known length of the data
* @param {DSCallback=} callback to fire if a fetch is issued - if all data was                 already loaded, the callback is fired with no parameters
*/
loadAllRecords(maxRecords?:Integer, callback?:DSCallback): boolean;

/**
*  Programmatically regroup the grid according to the current grouping configuration.
* 
* 
*/
regroup(): void;

/**
*  For a ListGrid.groupByField grid, returns all the direct children of the supplied node
*  in the ListGrid.groupTree if recordsOnly false. Otherwise, if
*  recordsOnly is true, returns instead a list of all descendants under the
*  supplied node that are actual records from the grid's original data - i.e. that are not other
*  group nodes (for multi-grouping) or summary records.
*  
*  Note that null may be returned if the grid is not currently grouped or the supplied node is
*  not a valid GroupNode.
* 
* 
* @param {GroupNode} node from ListGrid.groupTree
* @param {boolean} true to return all descendants that are actual                 records from the grid's original data, or false                  to return all immediate children of the supplied group node
*/
getGroupMembers(node:GroupNode, recordsOnly:boolean): Array<Partial<ListGridRecord>>;

/**
*  Display the current set of records grouped by their values for the given field or
*  fields. With no arguments, disables all grouping.
*  
*  Grouping transforms the current dataset into a Tree on the fly, then provides a familiar tree
*  interface for exploring the grouped data. Note that for performance reasons grouping
*  is only available for dataSets with less than ListGrid.groupByMaxRecords entries.
*  
*  Grouping works automatically with any dataset, providing simple default grouping based on
*  each field's declared type - see SimpleType.groupingModes for more information. 
*  Additionally, you can use ListGridField.getGroupValue 
*  to control how records are grouped, and 
*  ListGridField.getGroupTitle
*  
*  to control how groups are titled. You can affect multiple fields of the same data-type via the 
*  same-named APIs on an appropriate custom SimpleType.getGroupValue.
*  
*  Grouping can be performed programmatically via this API, or you can set
*  ListGrid.canGroupBy to enable menus that allow the user to perform
*  grouping. To group a grid automatically, instantiate the grid with a
*  ListGrid.groupByField setting. To take action when an end use requests grouping,
*  see ListGrid.handleGroupBy.
*  
*  While grouped, the automatically created Tree is available as
*  ListGrid.groupTree and the original dataset is available as
*  ListGrid.originalData.
*  
*  Data Requirements
*  
*  Before grouping can be performed, all records that match current
*  ListGrid.fetchData must be loaded. If ListGrid.dataFetchMode is
*  in use, not all matching records are cached, and the 
*  ResultSet.getLength is less than
*  ListGrid.groupByMaxRecords, the grid will automatically request all unloaded records from the
*  server, then perform grouping once they arrive.
*  
*  If the total number of rows available from the server exceeds ListGrid.groupByMaxRecords,
*  calling groupBy will have no effect, and menu items for grouping will appear
*  disabled.
*  
*  Grouping Notification
*  
*  Grouping is often an asynchronous operation, both because of automatic loading of remaining
*  rows, and because asynchronous processing is required to work around bugs in some browsers
*  related to misdetection of "hung" scripts (see ListGrid.groupByAsyncThreshold). Define a
*  ListGrid.handleGroupBy method to be called when grouping is about to start (potentially
*  canceling it), and define a ListGrid.groupByComplete method to be notified when it's done.
* 
* 
* @param {Array<Partial<string>>=} name of fields to group by
*/
groupBy(...args:any[]): void;

/**
*  Returns the HTML code necessary to render a group node, including icon, title, and padding.
*  The amount of the padding is at least ListGrid.groupLeadingIndent pixels, and
*  an additional ListGrid.groupIndentSize pixels for each increasing level of the
*  node.
*  
*  The result of this method will be displayed to the user for the 
*  appropriate row, either in a single cell which spans multiple columns, or in 
*  the ListGrid.groupTitleField. For the case where group titles are displayed 
*  in a cell spanning multiple columns, if this grid has frozen fields, this method 
*  may be run for both the frozen and unfrozen body. This method will return the html
*  described above for the frozen body, and an empty string for the unfrozen body 
*  (or vice versa depending on ListGrid.showGroupTitleInFrozenBody). This ensures
*  the groupNodeHTML is not displayed twice.
* 
* 
* @param {object} Specified group node
* @param {GridRenderer=} The body in which the returned value will be displayed. This parameter allows the default implementation to return an empty string if appropriate  for the case where there is both a frozen and unfrozen body. Note that if this parameter  may be empty. If not passed, the full group node HTML will be returned.
*/
getGroupNodeHTML(node:object, gridBody?:GridRenderer): HTMLString;

/**
*  If this grid ListGrid.isGrouped, this method will return the
*  current selection. Unlike the standard ListGrid.getSelection,
*  this method will return ListGrid.isGroupNode in addition to standard
*  ListGridRecords within the grid's data.
* 
* @param {boolean=} By default a group header node is considered  selected if any members of the group are selected. If this flag is passed in, only  header nodes where all members of the group are selected will be included  in the returned results.
* @param {boolean=} If this parameter is passed as true,  this method will return just the group header nodes from the group tree. If omitted  or false, both header nodes and data records will be returned.
*/
getGroupTreeSelection(excludePartialSelection?:boolean, groupNodesOnly?:boolean): Array<Partial<ListGridRecord>>;

/**
*  Returns all selected records in this grid. If this grid is
*  ListGrid.isGrouped, group header nodes will not be included in the 
*  returned array. Developers can make use of ListGrid.getGroupTreeSelection to 
*  get the selection including the selected group header nodes.
*  
*  NOTE: Records in the returned array should be treated as read-only and not
*  modified.
* 
* @param {boolean=} When true, partially selected records will not be returned.                  Otherwise, both fully and partially selected records are                  returned.
*/
getSelection(excludePartialSelections?:boolean): Array<Partial<ListGridRecord>>;

/**
*  If this listGrid is ListGrid.groupBy, is the record
*  passed in a group header node.
* 
* @param {ListGridRecord} record to test
*/
isGroupNode(record:ListGridRecord): boolean;

/**
*  Setter for ListGrid.showGroupSummaryInHeader
* 
* @param {boolean} new showGroupSummaryInHeader state
*/
setShowGroupSummaryInHeader(showGroupSummaryInHeader:boolean): void;

/**
*  Setter for ListGrid.showCollapsedGroupSummary
* 
* @param {boolean} new showCollapsedGroupSummary value
*/
setShowCollapsedGroupSummary(showCollapsedGroupSummary:boolean): void;

/**
*  Removes the grouping from the listGrid, restoring its original data
* 
* 
*/
ungroup(): void;

/**
*  Update the headerSpans configuration on the grid dynamically.
* 
* 
* @param {Array<Partial<HeaderSpan>>} same configuration block as that passed to                     ListGrid.headerSpans.
*/
setHeaderSpans(headerSpans:Array<Partial<HeaderSpan>>): void;

/**
*  Update the title of a ListGrid.headerSpans dynamically.
* 
* 
* @param {string} name of the headerSpan, as specified via HeaderSpan.name.
* @param {string} new title for the headerSpan
*/
setHeaderSpanTitle(name:string, newTitle:string): void;

/**
*  Update the headerTitle of a ListGrid.headerSpans dynamically.
* 
* 
* @param {string} name of the headerSpan, as specified via HeaderSpan.name.
* @param {string} new headerTitle for the headerSpan
*/
setHeaderSpanHeaderTitle(name:string, newTitle:string): void;

/**
*  Update the HeaderSpan.headerBaseStyle for a span within the grid at runtime.
* 
* @param {string} name of the headerSpan, as specified via HeaderSpan.name.
* @param {CSSStyleName} new baseStyle for the headerSpan
*/
setHeaderSpanBaseStyle(name:string, newStyle:CSSStyleName): void;

/**
*  Update the HeaderSpan.headerTitleStyle for a span within the grid at runtime.
* 
* @param {string} name of the headerSpan, as specified via HeaderSpan.name.
* @param {CSSStyleName} new titleStyle for the headerSpan
*/
setHeaderSpanTitleStyle(name:string, newTitle:CSSStyleName): void;

/**
*  Setter for the ListGrid.showGroupSummary attribute
* 
* @param {boolean} new value for this.showGroupSummary
*/
setShowGroupSummary(showGroupSummary:boolean): void;

/**
*  Setter for the ListGrid.groupByFieldSummaries attribute
* 
* @param {Array<Partial<string>>} new value for this.groupByFieldSummaries
*/
setGroupByFieldSummaries(groupByFieldSummaries:Array<Partial<string>>): void;

/**
*  Open a MultiGroupDialog to configure the fields used for grouping.
* 
* 
*/
configureGrouping(): void;

/**
*  Returns the original, ungrouped data in the grid. If the grid is ungrouped,
*  returns ListGrid.data.
* 
*/
getOriginalData(): object;

/**
*  Returns the selected cells as a series of Records where each field value is stored under
*  it's offset from the top-left of the selection. For example, a 2x2 cell selection starting
*  from the first column would return two Records, each with two values stored under the names
*  "0" and "1".
*  
*  If ListGrid.canSelectCells is not enabled, this API always returns null.
* 
* 
*/
getSelectedCellData(): any;

/**
*  Applies a set of Records containing coordinate-based data as returned by
*  ListGrid.getSelectedCellData and applies the data at the current selection.
*  
*  For consistency with Excel, given a record in the cellData, after the data value with the
*  most negative column index is found, the rest of the values in the record are applied
*  contiguously to the right of it, using the positional data for ordering only.
*  
*  Will only modify cells in the grid which are editable, and changes will be applied
*  as editValues, exactly as though the user had typed the values in (see
*  editing).
*  
*  See also ListGrid.applyRecordData.
* 
* 
* @param {any} list of Records as described above
*/
applyCellData(cellData:any): void;

/**
*  Applies a list of Records as changes to the current selection.
*  
*  Values found in each of the passed records will be applied to the same-named fields in the
*  Records starting from the top-left of the current selection, in order.
*  
*  Will only modify cells in the grid which are editable, and changes will be applied
*  as editValues, exactly as though the user had typed the values in (see
*  editing).
*  
*  See also ListGrid.applyCellData.
* 
* 
* @param {any} list of Records as described above
*/
applyRecordData(recordData:any): void;

/**
*  Fires after user edits have been successfully saved to the server, only for cells where the
*  value was actually modified.
*  
*  If you want immediate notification of a changes before changes has been saved to the
*  server, implement ListGridField.change or
*  ListGridField.changed instead.
*  
*  You can alternatively use ListGridField.cellChanged to get notification only of
*  saved changes for a specific field. If both a listGridField and the containing listGrid
*  have a handler for this event, only the handler defined on the field is called.
* 
* 
* @param {ListGridRecord} record for the cell being changed
* @param {any} new value for the cell
* @param {any} old value for the cell
* @param {number} row number for the cell
* @param {number} column number of the cell
* @param {ListGrid} grid where cell was changed. Also available as "this"
*/
cellChanged(record:ListGridRecord, newValue:any, oldValue:any, rowNum:number, colNum:number, grid:ListGrid): void;

/**
*  Callback fired when inline edits have been successfully saved.
*  
*  No default implementation.
* 
* 
* @param {number} current index of the row that was saved
* @param {number} index of the column that was saved, if applicable
* @param {object | Record} new values that were saved
* @param {Record} the complete original values from before the save occurred
* @param {EditCompletionEvent} Event that led to the save
* @param {DSResponse=} for DataSource saves, DSResponse object returned
*/
editComplete(rowNum:number, colNum:number, newValues:object | Record, oldValues:Record, editCompletionEvent:EditCompletionEvent, dsResponse?:DSResponse): void;

/**
*  Called when an attempt to save inline edits fails, due to a validation error or other
*  server error.
*  
*  The default implementation of editFailed does nothing for normal validation errors,
*  which are displayed before editFailed() is called. For any other errors, the default
*  implementation will call RPCManager.handleError, which by default
*  will result in a warning dialog.
* 
* 
* @param {number} current index of the row we attempted to save
* @param {number} index of the column where the edit failed, if applicable
* @param {object | Record} new values that we attempted to save
* @param {Record} the complete original values from before the save occurred
* @param {EditCompletionEvent} Edit completion event that led to the save                         attempt
* @param {DSResponse=} DSResponse, for saves through a DataSource
*/
editFailed(rowNum:number, colNum:number, newValues:object | Record, oldValues:Record, editCompletionEvent:EditCompletionEvent, dsResponse?:DSResponse): void;

/**
*  Callback fired when the user starts editing a new cell.
*  
*  This callback is typically used to establish dynamic default values via
*  ListGrid.setEditValue or ListGrid.setEditValues.
*  
*  Can also be overridden on a per-field basis via
*  ListGridField.editorEnter.
* 
* 
* @param {ListGridRecord} record for the cell being edited. Will be null                for a new, unsaved record.
* @param {any} value for the cell being edited
* @param {Integer} row number for the cell
* @param {Integer} column number of the cell
*/
editorEnter(record:ListGridRecord, value:any, rowNum:Integer, colNum:Integer): void;

/**
*  Callback fired when the user starts editing a new row.
* 
* 
* @param {ListGridField} record for the cell being edited
* @param {object} edit values for the current row
* @param {number} row number for the cell
*/
rowEditorEnter(record:ListGridField, editValues:object, rowNum:number): void;

/**
*  Callback fired when the user attempts to navigate away from the current edit cell,
*  or complete the current edit.
*  
*  Return false from this method to cancel the default behavior (Saving / cancelling the
*  current edit / moving to the next edit cell).
*  
*  This callback is typically used to dynamically update values or value maps for related
*  fields (via ListGrid.setEditValue and ListGrid.setEditorValueMap respectively, or to
*  implement custom navigation (via ListGrid.startEditing.
*  
*  Can be overridden at the field level as field.editorExit.
* 
* 
* @param {EditCompletionEvent} How was the edit completion fired?
* @param {ListGridRecord} record for the cell being edited
* @param {any} new edit value for the cell being edited. Note that if the  user has not made any changes this will be undefined
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
editorExit(editCompletionEvent:EditCompletionEvent, record:ListGridRecord, newValue:any, rowNum:number, colNum:number): boolean;

/**
*  Callback fired when the user attempts to navigate away from the current edit row,
*  or complete the current edit.
*  
*  Return false from this method to cancel the default behavior (Saving / cancelling the
*  current edit / moving to the next edit cell).
* 
* 
* @param {EditCompletionEvent} How was the edit completion fired?
* @param {ListGridRecord} record for the cell being edited
* @param {object} new values for the record [Note that fields that have                 not been edited will not be included in this object]
* @param {number} row number for the row being left
*/
rowEditorExit(editCompletionEvent:EditCompletionEvent, record:ListGridRecord, newValues:object, rowNum:number): boolean;

/**
*  Formatter to apply to values displayed within cells.
*  
*  The value passed to this method is either the field value found in the cell record
*  or, if there are unsaved edits, the current user-entered value for the cell.
*  NOTE: unsaved user edits may contain nulls, bad values or values of the wrong
*  type, so formatters used for editable data should be bulletproof. For example, if you
*  have a function "myNumberFormatter" that should only be passed actual Numbers, you might
*  define formatCellValue like so:
*  
*    isc.isA.Number(parseInt(value)) ?
*       myNumberFormatter(parseInt(value)) : value
*  
*  Note that this formatter will not be applied to the value displayed within editors for
*  cells - use formatEditorValue to achieve this.
*  
*  If formatCellValue is defined at the field level for some cell being edited,
*  the field level method will be used to format the edit value and this method will not
*  be called for that cell.
* 
* 
* @param {any} raw value for the cell being
* @param {ListGridRecord} Record object for the cell. Note: If this is a new row that has not been saved, in an  editable grid, it has no associated record object. In this case the edit values will  be passed in as this parameter.
* @param {number} row number for the cell
* @param {number} column number for the cell.
*/
formatCellValue(value:any, record:ListGridRecord, rowNum:number, colNum:number): HTMLString;

/**
*  Formatter for inactive content.
*  
*  If present, this method will be invoked instead of ListGrid.formatCellValue in cases 
*  where the grid is rendering non-interactive content outside. Examples of cases
*  where this can happen include:
*  
* - dragTracker HTML for a row when ListGrid.dragTrackerMode is set to
*   "record"
*  
* - measurement HTML used for sizing columns during autoFit
*  
* - measurement HTML used for sizing rows when ListGrid.fixedRecordHeights
*    is false and the grid has both frozen and unfrozen fields
*  
*  May also be overridden at the ListGridField.formatInactiveCellValue.
*  
*  This is useful for cases where it would not be appropriate to render the 
*  standard formatted cell value outside of the body of the grid. An example might be
*  if the formatted value contains a DOM element with a specified ID - an approach
*  sometimes used for integrating third party components into SmartClient listGrid
*  cells. In this case developers will wish to avoid having the framework render an
*  element with the same ID outside of the grid, and should instead return HTML that
*  would render at the same size, with an appropriate appearance.
* 
* 
* @param {any} raw value for the cell being
* @param {ListGridRecord} Record object for the cell. Note: If this is a new row that has not been saved, in an  editable grid, it has no associated record object. In this case the edit values will  be passed in as this parameter.
* @param {number} row number for the cell
* @param {number} column number for the cell.
*/
formatInactiveCellValue(value:any, record:ListGridRecord, rowNum:number, colNum:number): HTMLString;

/**
*  Formatter to apply to values displayed within editors while a cell is being edited.
*  The value passed to this method is the raw value for the cell.
*  If formatEditorValue is defined at the field level for some cell being edited,
*  the field level method will be used to format the edit value and this method will not
*  be called for that cell.
*  To convert the formatted value displayed within an editor back to a raw value, the
*  parseEditorValue method is used.
* 
* 
* @param {any} raw value for the cell being edited
* @param {ListGridRecord} Record object for the cell. Note: If this is a new row that has not been saved,  it has no associated record object. In this case the edit values will  be passed in as this parameter.
* @param {number} row number for the cell
* @param {number} column number for the cell.
*/
formatEditorValue(value:any, record:ListGridRecord, rowNum:number, colNum:number): any;

/**
*  Method used to convert the value displayed in an editor for some cell being edited into
*  a raw value for saving.
*  If parseEditorValue is defined at the field level for some cell being edited,
*  the field level method will be used to parse the edit value and this method will not
*  be called for that cell.
* 
* 
* @param {any} value displayed in the editor for the cell
* @param {ListGridRecord} record object for the row being edited. May be null if this             is a new row being added to the end of the list.
* @param {number} row number for the cell
* @param {number} column number for the cell.
*/
parseEditorValue(value:any, record:ListGridRecord, rowNum:number, colNum:number): any;

/**
*  Notification method executed when the ListGrid.setSort change
*  for this grid.
* 
* @param {Array<Partial<SortSpecifier>>} new sort specifiers - may be empty indicating  the grid was unsorted
*/
sortChanged(sortSpecifiers:Array<Partial<SortSpecifier>>): void;

/**
*  Notification method executed when columns are resized or reordered, or fields are
*  shown or hidden, frozen or unfrozen. Has no default implementation.
* 
* 
*/
fieldStateChanged(): void;

/**
*  Notification method executed whenever the groupState of this grid changes.
*  Group state is accessible via ListGrid.getGroupState, and contains group state
*  information.
* 
* 
*/
groupStateChanged(): void;

/**
*  Notification method executed whenever the viewState of this grid changes.
*  View state is accessible via ListGrid.getViewState, and contains field state
*  information, sort information, selection information, hiliting information and
*  grouping information.
* 
* 
*/
viewStateChanged(): void;

/**
*  Notification method fired when new data arrives from the server to be displayed in this
*  ListGrid, (for example in response to the user scrolling a new set of rows into view).
*  Only applies to databound listGrids where the ListGrid.data attribute is a
*  ResultSet. This ResultSet may have been created manually and applied to the grid via
*  a call to ListGrid.setData or may have been created and automatically assigned if
*  ListGrid.fetchData was used to populate the grid. This method is fired directly in
*  response to ResultSet.dataArrived firing on the data object.
*  
*  Note that dataArrived(), unlike ListGrid.dataChanged, only fires in limited
*  circumstances - when data for a ResultSet arrives from the server due to a fetch
*  or cache invalidation, or as a result of filtering. If you want to catch all data
*  changes, you should instead react to ListGrid.dataChanged.
* 
* 
* @param {Integer} starting index of the newly loaded set of rows
* @param {Integer} ending index of the newly loaded set of rows (non inclusive).
*/
dataArrived(startRow:Integer, endRow:Integer): void;

/**
*  Notification method fired when the ListGrid's data changes, for any reason.
*  If overridden (rather than Class.observe), you must Class.Super to ensure proper Framework behavior.
*  
*  Examples of why data changed might be:
*  
* - a call to ListGrid.addData, ListGrid.updateData, or ListGrid.removeData
*  
* - DataSource updates from the server for ResultSet data
*  (triggered by record editing, etc.)
*  
* - fetches arriving back from the server for ResultSet data
*  
* - changes to array data if made through APIs such as Array.set,
*  Array.add, etc.
*  
* - cache invalidation
*  
* - filtering
*  
*  Calling ListGrid.setData doesn't call this notification directly, but it may
*  fire if one of the above listed events is triggered (e.g. a server fetch for 
*  ResultSet data).
*  
*  Note that the operationType parameter is optional and will be passed and
*  contain the operation (e.g. "update") if this notification was triggered by a fetch,
*  an ListGrid.addData, ListGrid.updateData, or ListGrid.removeData, or a DataSource
*  update for ResultSet data (the first three reasons listed above) but otherwise
*  will be undefined.
* 
* 
* @param {string=} optionally passed operation causing the change
*/
dataChanged(operationType?:string): void;

/**
*  Handle a hover over a button in the header.
* 
* 
* @param {number} field number for the header that was hovered
*/
headerHover(fieldNum:number): void;

/**
*  Notification method that fires when the drawArea changes due to scrolling. Receives
*  the previous drawArea co-ordinates as parameters. Call ListGrid.getDrawArea to
*  get the new drawArea co-ordinates.
*  
*  Note that if this grid is showing any ListGridField.frozen, they
*  will not be included in the oldStartCol, oldEndCol range
*  reported by this method. Frozen fields are assumed never to be scrolled out of view.
* 
* 
* @param {number} the startRow from before the drawArea changed
* @param {number} the endRow from before the drawArea changed
* @param {number} the startCol from before the drawArea changed
* @param {number} the endCol from before the drawArea changed
*/
drawAreaChanged(oldStartRow:number, oldEndRow:number, oldStartCol:number, oldEndCol:number): void;

/**
*  When ListGrid.showRecordComponents is true, return false from this method
*  to prevent the recordComponent system from processing the passed record or cell.
*  
*  If ListGrid.showRecordComponentsByCell is true, 
*  it is important to implement this method - especially if the grid has many fields - 
*  otherwise, the internal automatic detection will process every visible cell, checking,
*  for example, whether a component already exists for that cell, before eventually running 
*  ListGrid.createRecordComponent to check if one is required. Implementing this
*  method avoids all that processing for cells that aren't expected to have components.
*  
*  The second parameter is only applicable if ListGrid.showRecordComponentsByCell is true.
* 
* 
* @param {ListGridRecord} record being processed
* @param {Integer=} column index of the cell in which the record component  may be shown. Will be null unless showRecordComponentsByCell is true.
*/
showRecordComponent(record:ListGridRecord, colNum?:Integer): boolean;

/**
*  When ListGrid.showRecordComponents is true, this method is called to create
*  per-row or per-cell embedded components to display in the grid.
*  
*  The colNum parameter is applicable only when
*  ListGrid.showRecordComponentsByCell is true.
*  
*  If this row should not have a recordComponent, return null.
*  
*  This method should create and return a new component for the record passed in every
*  time it is called and never return the same Canvas instance twice. To re-use components
*  with different rows, set RecordComponentPoolingMode to "recycle". In this mode,
*  in addition to implementing this method, developers should also implement
*  ListGrid.updateRecordComponent which allows already created components to be
*  altered for re-use in new records. See the ListGrid.showRecordComponents overview
*  for more information.
* 
* 
* @param {ListGridRecord} record to create a component for
* @param {Integer} cell to which the component applies
*/
createRecordComponent(record:ListGridRecord, colNum:Integer): Canvas;

/**
*  When ListGrid.showRecordComponents is true, this method is called to update
*  components created by ListGrid.createRecordComponent when they are to be
*  applied to a different record in the grid. See the
*  ListGrid.showRecordComponents
*  for more information on recordComponents.
*  
*  The colNum parameter is applicable only when
*  ListGrid.showRecordComponentsByCell is true. Note that if
*  ListGrid.poolComponentsPerColumn is set to false, the component may have been generated
*  by a ListGrid.createRecordComponent call applied to a different field.
*  
*  Return null to avoid re-adding the component to the row or cell.
* 
* 
* @param {ListGridRecord} record to which the passed component applies
* @param {Integer} cell to which the passed component applies
* @param {Canvas} the component to update
* @param {boolean} was the passed component previously embedded in a   different record?
*/
updateRecordComponent(record:ListGridRecord, colNum:Integer, component:Canvas, recordChanged:boolean): Canvas;

/**
*  Optional notification fired when the user performs a filter using the
*  ListGrid.showFilterEditor. Useful for applying additional criteria not 
*  available in the filterEditor. Note that it is often easiest to do this with the 
*  SearchForm attribute, which requires no code.
*  
*  May fire as criteria values are being edited if 
*  ListGrid.filterByCell or ListGrid.filterOnKeypress is true, otherwise will fire when the
*  user clicks the filter button or presses the Enter key while focus is in the Filter Editor.
*  
*  
*  Return false
*  to cancel the default behavior - you must cancel the default behavior if your
*  code is going to call ListGrid.filterData, ListGrid.setCriteria or any other API that
*  affects the criteria applied to the grid.
*  
*  The criteria parameter contains the current criteria applied to the
*  grid including edits the user has just made using the Filter Editor and those applied
*  with the ListGrid.showFilterWindow. A call to
*  ListGrid.getFilterEditorCriteria does not include the ListGrid.getFilterWindowCriteria.
*  
*  If you wish to access the criteria applied to the grid without picking
*  up any edits to the Filter Editor, use ListGrid.getCriteria instead.
*  
*  Developers may wish to perform a filter using the Filter Editor values from code
*  running outside the standard filterEditorSubmit flow. For example, if you wanted a
*  confirmation dialog to be shown before filtering was performed, you would cancel the
*  default behavior as described above, but then need to replicate the default behavior
*  once the user confirms that they want to proceed. To replicate the default behavior, 
*  just call:
*  
*  grid.filterData(grid.getFilterEditorCriteria());
*  
*  or, to ensure the specified ListGrid.autoFetchTextMatchStyle is picked up
*  
*  
*  grid.filterData(grid.getFilterEditorCriteria(), 
*      null, {textMatchStyle:grid.autoFetchTextMatchStyle});
*  
*  
*  
* 
* 
* @param {Criteria} criteria derived from the filter editor values
*/
filterEditorSubmit(criteria:Criteria): boolean;

/**
*  Callback fired when the user attempts to group or ungroup the listGrid, or when
*  ListGrid.groupBy is called programmatically.
*  Return false to cancel grouping.
*  
*  
*  This notification is fired before the ListGrid.groupTree is updated to
*  reflect the grouping. See also ListGrid.groupByComplete.
*  
*  Note that this method is not called when the data is regrouped, either
*  ListGrid.regroup, or in response to new data arriving from the server,
*  and such regrouping can't be canceled - instead use callback ListGrid.handleRegroup.
* 
* 
* @param {Array<Partial<string>>} the list of ListGrid field-names by which the grid is         about to be grouped. If the grid is being ungrouped, this param will be         an empty array
* @param {Array<Partial<GroupSpecifier>>} list of GroupSpecifier objects detailing         grouping specifics for each grouped field
*/
handleGroupBy(fields:Array<Partial<string>>, specifiers:Array<Partial<GroupSpecifier>>): boolean;

/**
*  Callback fired when the listGrid is grouped or ungrouped.
*  
*  Unlike ListGrid.handleGroupBy, this notification will fire when grouping
*  is complete, and the ListGrid.data object has been updated.
*  On successful grouping the fields argument will list the new 
*  grouping and the ListGrid.groupTree will have been updated to reflect the
*  grouped data.
*  
*  Note that the fields argument may be an empty array if the data
*  is not grouped. This implies that a user or developer explicitly ungrouped the
*  grid, or that a groupBy attempt failed due to the data length exceeding 
*  ListGrid.groupByMaxRecords.
*  
*  By design, this method is not called when the data is regrouped, either 
*  ListGrid.regroup, or in response to new data arriving from the server.
*  You can use the callback ListGrid.groupTreeChanged to be notified in that situation.
*  
*  If you monitor only this method and call ListGrid.groupBy before data is fetched, the
*  notification that you'll receive will be for grouping the initial (perhaps empty) data
*  set only. To have this method actually trigger when grouping of the fetched data is
*  done, you should avoid calling ListGrid.groupBy before the initial fetch, and instead do
*  it in the the ListGrid.fetchData.
* 
* 
* @param {Array<Partial<string>>} ListGrid field names by which the grid is now grouped. If the grid is currently not grouped, this parameter will be an  empty array.
*/
groupByComplete(fields:Array<Partial<string>>): void;

/**
*  Callback fired when a regroup operation is begun, either from a direct call to
*  ListGrid.regroup, or because of ListGrid.dataChanged when
*  the grid is already ListGrid.groupBy.
*  
*  After this call, the Framework should eventually call ListGrid.groupTreeChanged.
* 
*/
handleRegroup(): void;

/**
*  Callback fired when a ListGrid.groupByField operation completes, whether it
*  started as a direct call to ListGrid.groupBy or ListGrid.regroup or data changing,
*  including incremental changes.
*  
*  If changeType is "groupBy", ListGrid.handleGroupBy should have been
*  called at the beginning of the operation, and ListGrid.groupByComplete will be called
*  immediately before this method. If changeType is "regroup", then 
*  ListGrid.handleRegroup should have been called at the beginning of the operation.
* 
* 
* @param {GroupTreeChangeType} did the change originate from ListGrid.groupBy,                     ListGrid.regroup. or an incremental data change?
* @param {boolean} did the operation complete successfully?
*/
groupTreeChanged(changeType:GroupTreeChangeType, success:boolean): void;

/**
*  Notification method executed whenever the end user uses the HiliteEditor to change
*  the set of hilites applied to this grid. This method will not be called after
*  a purely programmatic change to the hilites made with a call to
*  DataBoundComponent.setHilites. The array of currently applied hilite objects is
*  accessible via DataBoundComponent.getHilites.
* 
*/
hilitesChanged(): void;

/**
*  Optional stringMethod to fire when the user hovers over the error icon of a cell with 
*  validation errors. The default behavior is to show a hover canvas containing the
*  validation error message text. Return false to suppress this default behavior.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellErrorIconHover(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Optional stringMethod to fire when the mouse moves over the error icon of a cell with 
*  validation errors.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellErrorIconOver(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Optional stringMethod to fire when the mouse moves off the error icon of a cell with 
*  validation errors.
* 
* 
* @param {ListGridRecord} cell record as returned by getCellRecord()
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
cellErrorIconOut(record:ListGridRecord, rowNum:number, colNum:number): boolean;

/**
*  Callback fired when the end-user changes criteria. This occurs via the +{FilterEditor}
*  or +{showFilterWindow,advanced filtering} interface. It does not fire when a change is
*  made via ListGrid.setCriteria, ListGrid.fetchData, ListGrid.setFilterWindowCriteria or
*  other APIs are called to change the criteria.
* 
* 
*/
criteriaChanged(): void;

/**
*  Notification fired when a user either creates a new formula field or edits an existing formula field.
* 
* @param {ListGridField} the formula field
* @param {UserFormula} the new or updated formula definition
*/
formulaUpdated(field:ListGridField, formula:UserFormula): void;

/**
*  Notification fired when a user either creates a new summary field or edits an existing
*  summary field.
* 
* @param {ListGridField} the summary field
* @param {UserSummary} the new or updated summary definition
*/
summaryUpdated(field:ListGridField, summary:UserSummary): void;



}
/**
* Creates a new ListGrid
* 
* @param typeCheckedProperties {Partial<ListGrid>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ListGrid} The new instance
*/
class ListGrid{
static create(typeCheckedProperties? : Partial<ListGrid>, uncheckedProperties?: Object): ListGrid;
/**
* A declared value of the enum types 
*  RecordDropPosition and
*  ReorderPosition.
* @type {Constant}
* @default "before"
*/
static BEFORE?: Constant;

/**
* A declared value of the enum types 
*  RecordDropPosition and
*  ReorderPosition.
* @type {Constant}
* @default "after"
*/
static AFTER?: Constant;

/**
* A declared value of the enum types 
*  RecordDropPosition,
*  ReorderPosition and
*  RecordDropAppearance.
* @type {Constant}
* @default "over"
*/
static OVER?: Constant;

/**
* A declared value of the enum type 
*  RecordDropAppearance.
* @type {Constant}
* @default "between"
*/
static BETWEEN?: Constant;

/**
* A declared value of the enum type 
*  RecordDropAppearance.
* @type {Constant}
* @default "both"
*/
static BOTH?: Constant;

/**
* A declared value of the enum type 
*  RecordDropAppearance.
* @type {Constant}
* @default "body"
*/
static BODY?: Constant;

/**
* A declared value of the enum type 
*  RecordDropPosition.
* @type {Constant}
* @default "none"
*/
static NONE?: Constant;

/**
* A declared value of the enum type 
*  EditCompletionEvent.
* @type {Constant}
* @default "click_outside"
*/
static CLICK_OUTSIDE?: Constant;

/**
* A declared value of the enum type 
*  EditCompletionEvent.
* @type {Constant}
* @default "click"
*/
static CLICK?: Constant;

/**
* A declared value of the enum type 
*  EditCompletionEvent.
* @type {Constant}
* @default "doubleClick"
*/
static DOUBLE_CLICK?: Constant;

/**
* A declared value of the enum type 
*  EditCompletionEvent.
* @type {Constant}
* @default "enter"
*/
static ENTER_KEYPRESS?: Constant;

/**
* A declared value of the enum type 
*  EditCompletionEvent.
* @type {Constant}
* @default "escape"
*/
static ESCAPE_KEYPRESS?: Constant;

/**
* A declared value of the enum type 
*  EditCompletionEvent.
* @type {Constant}
* @default "arrow_up"
*/
static UP_ARROW_KEYPRESS?: Constant;

/**
* A declared value of the enum type 
*  EditCompletionEvent.
* @type {Constant}
* @default "arrow_down"
*/
static DOWN_ARROW_KEYPRESS?: Constant;

/**
* A declared value of the enum type 
*  EditCompletionEvent.
* @type {Constant}
* @default "tab"
*/
static TAB_KEYPRESS?: Constant;

/**
* A declared value of the enum type 
*  EditCompletionEvent.
* @type {Constant}
* @default "shift_tab"
*/
static SHIFT_TAB_KEYPRESS?: Constant;

/**
* A declared value of the enum type 
*  EditCompletionEvent.
* @type {Constant}
* @default "field_change"
*/
static EDIT_FIELD_CHANGE?: Constant;

/**
* A declared value of the enum type 
*  EditCompletionEvent.
* @type {Constant}
* @default "programmatic"
*/
static PROGRAMMATIC?: Constant;

/**
* A declared value of the enum type 
*  GroupTreeChangeType
* @type {Constant}
* @default "groupBy"
*/
static GROUP_BY?: Constant;

/**
* A declared value of the enum type 
*  GroupTreeChangeType
* @type {Constant}
* @default "regroup"
*/
static REGROUP?: Constant;

/**
* A declared value of the enum type 
*  GroupTreeChangeType
* @type {Constant}
* @default "incremental"
*/
static INCREMENTAL?: Constant;








}

/**
* A ListGrid that implements the Palette behavior, so it can be used as the source for 
*  drag and drop instantiation of components when combined with an EditContext as 
*  the drop target.
*  
*  Each ListGridRecord can be a PaletteNode.
*/
interface ListPalette extends ListGrid, Palette {
/**
* Option to show filter editor is disabled for listPalettes
* @type {boolean}
* @default false
*/
canShowFilterEditor?: boolean;

/**
* Option to save searches is disabled for listPalettes
* @type {boolean}
* @default false
*/
canSaveSearches?: boolean;





}
/**
* Creates a new ListPalette
* 
* @param typeCheckedProperties {Partial<ListPalette>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ListPalette} The new instance
*/
class ListPalette{
static create(typeCheckedProperties? : Partial<ListPalette>, uncheckedProperties?: Object): ListPalette;







}

/**
* Dialog shown for editing properties of HTML lists in a RichTextEditor. Contains a
*  ListPropertiesPane.
*  
*  Cannot be directly used; shown in documentation only for skinning purposes.
*/
interface ListPropertiesDialog extends Window {
/**
* The title of this ListPropertiesDialog.
* @type {string}
* @default "List Properties"
*/
title?: string;

/**
* The ListPropertiesPane contained by this ListPropertiesDialog.
* @type {ListPropertiesPane}
* @default null
*/
listPropertiesPane?: ListPropertiesPane;

/**
* The title of the ListPropertiesDialog.applyButton.
* @type {string}
* @default "Apply"
*/
applyButtonTitle?: string;

/**
* The Apply button. When clicked, the ListPropertiesDialog.applyClick
*  event is fired.
*  
*  ListPropertiesDialog.applyButtonTitle is a autoChildUsage
*  for the button's Button.title.
* @type {IButton}
* @default null
*/
applyButton?: IButton;

/**
* The title of the ListPropertiesDialog.cancelButton.
* @type {string}
* @default "Cancel"
*/
cancelButtonTitle?: string;

/**
* The Cancel button. When clicked, the ListPropertiesDialog.cancelClick
*  event is fired.
*  
*  ListPropertiesDialog.cancelButtonTitle is a autoChildUsage
*  for the button's Button.title.
* @type {IButton}
* @default null
*/
cancelButton?: IButton;



/**
*  Notification method fired when the ListPropertiesDialog.applyButton
*  is clicked.
* 
* @param {ListProperties} the list properties to apply
*/
applyClick(listProperties:ListProperties): void;

/**
*  Notification method fired when the ListPropertiesDialog.cancelButton
*  is clicked.
* 
*/
cancelClick(): void;



}
/**
* Creates a new ListPropertiesDialog
* 
* @param typeCheckedProperties {Partial<ListPropertiesDialog>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ListPropertiesDialog} The new instance
*/
class ListPropertiesDialog{
static create(typeCheckedProperties? : Partial<ListPropertiesDialog>, uncheckedProperties?: Object): ListPropertiesDialog;







}

/**
* Pane containing controls for editing the style of HTML lists in a RichTextEditor.
*  
*  Cannot be directly used; shown in documentation only for skinning purposes.
*/
interface ListPropertiesPane extends Layout {
/**
* The properties corresponding to the currently-selected list configuration.
* @type {ListProperties}
* @default null
*/
listProperties?: ListProperties;

/**
* Shows available bullet options as a series of tiles.
* @type {TileLayout}
* @default null
*/
sampleTileLayout?: TileLayout;

/**
* Tile used to demonstrate each bullet style.
* @type {Canvas}
* @default null
*/
sampleTile?: Canvas;

/**
* Form used to show the ListPropertiesPane.startNumberField for
*  configuring the starting value of a list.
* @type {DynamicForm}
* @default null
*/
startNumberForm?: DynamicForm;

/**
* The FormItem.title of the ListPropertiesPane.startNumberField.
* @type {string}
* @default "Start at"
*/
startNumberFieldTitle?: string;

/**
* SpinnerItem used to modify the starting value of the list.
*  
*  ListPropertiesPane.startNumberFieldTitle is a
*  autoChildUsage for the field's FormItem.title.
* @type {SpinnerItem}
* @default null
*/
startNumberField?: SpinnerItem;



/**
*  Notification method fired when the pane's ListPropertiesPane.listProperties changes.
* 
* @param {ListProperties} the new list configuration properties
*/
listPropertiesChanged(listProperties:ListProperties): void;



}
/**
* Creates a new ListPropertiesPane
* 
* @param typeCheckedProperties {Partial<ListPropertiesPane>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ListPropertiesPane} The new instance
*/
class ListPropertiesPane{
static create(typeCheckedProperties? : Partial<ListPropertiesPane>, uncheckedProperties?: Object): ListPropertiesPane;







}

/**
* A logging system similar to the Java log4j package: messages are logged with a "category" and
*  "priority", and developers can dynamically set which log messages are being displayed.
*  
*  5 log priorities are available, with the following general meaning:
*  
*  
* - "debug": diagnostic info which is only likely to be understood by a developer with
*  source access, or would occur too frequently for normal usage
*  
* - "info": reports of significant events in the normal operation of the subsystem
*  
* - "warn": some kind of problem is likely to occur, an API appears is apparently being
*  misused or will yield a partial or very slow result
*  
* - "error": a definite error has occurred which may be recoverable
*  
* - "fatal": total failure with no possibility of recovery
*  
*  
*  Log categories do not need to be declared in advance - you can simply make up a category name and
*  start logging to it, and control whether that category's messages will be displayed via
*  setPriority().
*  
*  NOTE: to open the Developer Console in any page that loads ISC, type
*  javascript:isc.Log.show() in the URL bar - this URL is bookmarkable. 
*  
*  The Developer Console should always be open while developing any ISC-enabled application,
*  because ISC logs many important errors and warnings to the Developer Console.
*  
*  NOTE: if you have the Microsoft JavaScript Debugger installed, ISC will be unable to log stack
*  traces on JS errors until you go to Tools->Internet Options->Advanced Tab and check "Disable
*  script debugging". The ability to see stack traces in the Developer Console is generally much
*  more useful for debugging ISC-based applications than the generic Javascript Debugging
*  facilities.
*/
interface Log extends Class {




}
/**
* Creates a new Log
* 
* @param typeCheckedProperties {Partial<Log>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Log} The new instance
*/
class Log{
static create(typeCheckedProperties? : Partial<Log>, uncheckedProperties?: Object): Log;
/**
* A declared value of the enum type 
*  LogPriority.
* @type {Integer}
* @default "1"
*/
static FATAL?: Integer;

/**
* A declared value of the enum type 
*  LogPriority.
* @type {Integer}
* @default "2"
*/
static ERROR?: Integer;

/**
* A declared value of the enum type 
*  LogPriority.
* @type {Integer}
* @default "3"
*/
static WARN?: Integer;

/**
* A declared value of the enum type 
*  LogPriority.
* @type {Integer}
* @default "4"
*/
static INFO?: Integer;

/**
* A declared value of the enum type 
*  LogPriority.
* @type {Integer}
* @default "5"
*/
static DEBUG?: Integer;

/**
* Any logs below this priority will be suppressed, unless a more specific setting exists for
*  the category.
* @type {LogPriority}
* @default 3
*/
static defaultPriority?: LogPriority;

/**
* At this priority and above, a stack trace will be included automatically along with the log
*  message itself.
* @type {LogPriority}
* @default 2
*/
static stackTracePriority?: LogPriority;

/**
* Maximum number of logged messages to retain in memory.
*  
*  Note that if the Developer Console is open, it will accumulate an unbounded number of
*  messages in the "Log Messages" area. messageCount only affects the number
*  of messages held in memory in the main application's browser window or tab.
* @type {Integer}
* @default "1000"
*/
static messageCount?: Integer;





/**
*  Returns a one-line summary of the current method call, showing method name and passed
*  arguments. 
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.getCallTrace(arguments)" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
*  
*  Note the arguments object is required in most cases for this method to function. In some
*  browsers, it can be derived automatically, but developers looking to debug on multiple platforms
*  should not rely on this.
* 
* 
* @param {any=} arguments object from the call to trace. If ommitted, where supported,  arguments will be derived from the calling function, or if this is not supported, the method  will not function.
*/
static getCallTrace(args?:any): void;

/**
*  Returns a "stack trace" - one line per method in the current call stack, showing the method
*  name and any parameters passed.
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.getStackTrace" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
*  
*  Platform Notes: In Mozilla Firefox, if Firebug is enabled, a stack trace will be logged
*  to the firebug console in addition to the standard stack trace string returned by
*  this method.
*  
*  In browsers other than Internet Explorer a complete stack trace may not be available - 
*  this occurs when a function is re-entrant (meaning it calls itself). In this case the
*  stack will terminate with text indicating where the recursive function call occurred.
*  
*  See debugging for further information information.
* 
* 
*/
static getStackTrace(): string;

/**
*  Return a very short (generally less than 40 characters) string representation of any object,
*  suitable for viewing by a developer for debugging purposes.
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.echoLeaf" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
* 
* 
* @param {any} object to echo
*/
static echoLeaf(obj:any): string;

/**
*  Return a short string representation of any object, suitable for viewing by a developer for
*  debugging purposes.
*  
*  If passed an object containing other objects, echo will not recurse into subobjects,
*  summarizing them instead via echoLeaf().
*  
*  NOTE: echo() is used to generate the output shown in the Log window when evaluating an
*  expression.
*  
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.echo()" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
*  
* 
* @param {any} object to echo
*/
static echo(obj:any): string;

/**
*  Like echo(), except that if passed an Array, echoAll() will echo() every element of the
*  Array.
*  This function is available as a static on every ISC Class and as an instance
*  method on every instance of an ISC Class.
*  General best practice is to call the method as "this.echo()" whenever "this" is an
*  instance, or call the static classMethod on the Log class otherwise. 
* 
* 
* @param {any} object to echo
*/
static echoAll(obj:any): string;

/**
*  A common usage of Class.logDebug is to call the method directly on
*  the Log class.
* 
* @param {string} message to log
* @param {string=} category to log in
*/
static logDebug(message:string, category?:string): void;

/**
*  A common usage of Class.logInfo is to call the method directly on
*  the Log class.
* 
* @param {string} message to log
* @param {string=} category to log in
*/
static logInfo(message:string, category?:string): void;

/**
*  A common usage of Class.logWarn is to call the method directly on
*  the Log class.
* 
* @param {string} message to log
* @param {string=} category to log in
*/
static logWarn(message:string, category?:string): void;

/**
*  A common usage of Class.logError is to call the method directly on
*  the Log class.
* 
* @param {string} message to log
* @param {string=} category to log in
*/
static logError(message:string, category?:string): void;

/**
*  A common usage of Class.logFatal is to call the method directly on
*  the Log class.
* 
* @param {string} message to log
* @param {string=} category to log in
*/
static logFatal(message:string, category?:string): void;

/**
*  A common usage of Class.setLogPriority is to call the method
*  directly on the Log class.
* 
* @param {string} Category for which the log priority will be updated. If not             all logs on this canvas will be logged at the priority passed in.
* @param {LogPriority} priority level
*/
static setLogPriority(category:string, priority:LogPriority): void;

/**
*  A common usage of Class.setDefaultLogPriority is to call the
*  method directly on the Log class.
* 
* @param {string} Category for which the log priority will be updated. If not             all logs on this canvas will be logged at the priority passed in.
* @param {LogPriority} priority level
*/
static setDefaultLogPriority(category:string, priority:LogPriority): void;

/**
*  A common usage of Class.getDefaultLogPriority is to call the
*  method directly on the Log class.
* 
*/
static getDefaultLogPriority(): LogPriority;

/**
*  Apply a batch a batch of priority settings, as a object mapping category names to priority
*  levels.
* 
* 
* @param {object} priority settings for multiple categories
*/
static applyLogPriorities(settings:object): void;

/**
*  Get all priority settings as an object mapping category names to priority levels.
* 
* 
* @param {Class | object=} Optional param to get priorities specific to                      some ISC class or instance.
* @param {boolean=} If this method is retrieving the priorities specific                  to logging for some class or instance, this parameter                  can be used to view only the overrides to the default                  log priorities on this object.
*/
static getLogPriorities(object?:Class | object, overridesOnly?:boolean): object;

/**
*  Return the priority setting for a particular category.
*  
*  If there is no priority setting specific to this category, null will be
*  returned, NOT Log.defaultPriority.
* 
* 
* @param {string} category name
* @param {Class | object=} Optional class or instance to check for                        specific log priority overrides
*/
static getPriority(category:string, object?:Class | object): LogPriority;

/**
*  Set the priority of messages that will be visible for this log category.
*  
*  After calling setPriority, any messages logged to the given category whose priority is
*  below the specified priority will not appear in the Log.
* 
* 
* @param {string} category name
* @param {LogPriority} priority level to set
* @param {Class | object=} Optional ISC class or instance - if passed the priority will be set for logging    occurring on the class or instance only.
*/
static setPriority(category:string, priority:LogPriority, object?:Class | object): void;

/**
*  Set the default priority of messages that will be visible.
* 
* 
* @param {LogPriority} priority level to set
* @param {Class | object=} Optional ISC class or instance - if passed the default priority will be set for logging    occurring on the class or instance only.
*/
static setDefaultPriority(priority:LogPriority, object?:Class | object): void;

/**
*  Retrieves the default priority of messages that will be visible.
* 
* 
* @param {Class | object=} Optional ISC class or instance - if passed the returns the default priority for    the class or instance only.
*/
static getDefaultPriority(object?:Class | object): LogPriority;

/**
*  Clear the priority setting for a particular category, so that the category's effective
*  priority returns to Log.defaultPriority
*  If the optional second parameter is passed, the specific priority setting for the 
*  category on that object will be cleared, so logs in that category on that object will
*  be logged at the global priority level for the category.
* 
* 
* @param {string} category name
* @param {Class | object=} Optional instance or class object - if passed                    clear logging priority for the appropriate category                     on that object.
*/
static clearPriority(category:string, object?:Class | object): void;

/**
*  Would a message logged to the given category at the given priority appear in the Log?
*  
*  NOTE: if there is no specific priority setting for a given category, the
*  Log.defaultPriority is used.
* 
* 
* @param {string} category name
* @param {LogPriority} priority level to check
*/
static isEnabledFor(category:string, priority:LogPriority): void;

/**
*  Return an Array of the most recently logged messages as an Array of Strings. Up to
*  Log.messageCount messages may be returned.
* 
*/
static getMessages(): Array<Partial<string>>;

/**
*  Open the Developer Console.
*  
*  The Developer Console should always be open while developing any ISC-enabled
*  application, because ISC logs many important errors and warnings to the Developer Console.
*  
*  In Internet Explorer, the Developer Console is able to log a stack trace for every JS error,
*  including errors that occur in non-ISC code.
*  
*  NOTE: if you have the Microsoft JavaScript Debugger installed, ISC will be unable to log
*  stack traces on JS errors until you go to Tools->Internet Options->Advanced Tab and check
*  "Disable script debugging". The ability to see stack traces in the Developer Console is
*  generally much more useful for debugging ISC-based applications than the generic Javascript
*  Debugging facilities.
*  
* 
*/
static show(): void;

/**
*  Clear all currently displayed Log messages
* 
*/
static clear(): void;

/**
*  Causes a stack trace to be logged any time a message containing the provided pattern is
*  logged. This can help figure out the origin of warnings or other mysterious logs in a large
*  complex application.
*  
*  The passed messagePattern is interpreted as a JavaScript regular expression.
*  
*  Note: log messages that do not appear in the Developer Console because of
*  Log.setLogPriority will never trigger a stack trace.
*  
* 
* @param {string} * @param {string=} */
static traceLogMessage(messagePattern:string, prefix?:string): void;

/**
* 
*  Observe a method on an object, logging a stack trace whenever the method is called.
*  
*  Call a second time with identical arguments to disable tracing.
* 
* 
* @param {object} object to observe
* @param {string} name of the method to observe
*/
static traceMethod(object:object, methodName:string): void;

/**
*  
*  Observe a method on an object, logging execution time whenever the method is called.
*  
*  Call a second time with identical arguments to disable tracing.
* 
* 
* @param {object} object to observe
* @param {string} name of the method to observe
*/
static timeMethod(object:object, methodName:string): void;




}

/**
* Logs out the current user by opening the Authentication.logOutURL in another tab or window.
*/
interface LogOutTask extends ProcessElement {




}
/**
* Creates a new LogOutTask
* 
* @param typeCheckedProperties {Partial<LogOutTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {LogOutTask} The new instance
*/
class LogOutTask{
static create(typeCheckedProperties? : Partial<LogOutTask>, uncheckedProperties?: Object): LogOutTask;







}

/**
* Handle a complete login interaction with a typical login dialog asking for username and
*  password credentials. Use this
*  class to quickly present a traditional username/password authentication mechanism in a
*  SmartClient window.
*  
*  To adapt this class to your requirements, first implement LoginDialog.loginFunc to submit
*  the username and password to the authentication mechanism of your choice, calling
*  dialogCallback once the authentication process completes.
*/
interface LoginDialog extends Window {
/**
* Whether the user should be able to dismiss the login dialog without entering
*  credentials. Set to true if logging in is optional. When set, a close button will be
*  present, and hitting escape will also dismiss the dialog.
*  
*  If the Dialog is dismissed, LoginDialog.loginFunc is called with null arguments.
*  
*  Note that this attribute overrides the dismissOnEscape and showCloseButton attributes.
* @type {boolean}
* @default false
*/
dismissable?: boolean;

/**
* Do not set LoginDialog.dismissOnEscape; it is controlled by the
*  LoginDialog.dismissable
*  property.
* @type {boolean}
* @default null
*/
dismissOnEscape?: boolean;

/**
* Do not set LoginDialog.showCloseButton; it is controlled by the
*  LoginDialog.dismissable
*  property.
* @type {boolean}
* @default true
*/
showCloseButton?: boolean;

/**
* If true, the login form will allow blank passwords to be submitted. Otherwise
*  the form fails to be validated until the user enters at least one character into
*  the password field.
* @type {boolean}
* @default false
*/
allowBlankPassword?: boolean;

/**
* If true, display a LinkItem (LoginDialog.lostPasswordItem)
*  meant for the user to click if the account's
*  credentials are forgotten. The text of the link is controlled by
*  LoginDialog.lostPasswordItemTitle. If clicked, the link will fire
*  LoginDialog.lostPassword.
* @type {boolean}
* @default false
*/
showLostPasswordLink?: boolean;

/**
* If true, display a LinkItem (LoginDialog.registrationItem)
*  meant for the user to click if the user wishes to register a new account.
*  The text of the link is controlled by
*  LoginDialog.registrationItemTitle. If clicked, the link will fire
*  LoginDialog.register.
* @type {boolean}
* @default false
*/
showRegistrationLink?: boolean;

/**
* Specifies the title of the dialog box.
* @type {string}
* @default "Please log in"
*/
title?: string;

/**
* Specifies the CSS style of the error text shown for a login failure.
* @type {string}
* @default "formCellError"
*/
errorStyle?: string;

/**
* Specifies the title of the "usernameItem" field of the LoginDialog.loginForm.
* @type {string}
* @default "Username"
*/
usernameItemTitle?: string;

/**
* Specifies the title of the "passwordItem" field of the LoginDialog.loginForm.
* @type {string}
* @default "Password"
*/
passwordItemTitle?: string;

/**
* Specifies the contents of the login submission button of the LoginDialog.loginForm.
* @type {string}
* @default "Log in"
*/
loginButtonTitle?: string;

/**
* Specifies the contents of the password request button (if configured) on
*  the LoginDialog.loginForm.
* @type {string}
* @default undefined
*/
lostPasswordItemTitle?: string;

/**
* Specifies the contents of the registration link (if configured) on
*  the LoginDialog.loginForm.
* @type {string}
* @default undefined
*/
registrationItemTitle?: string;

/**
* Specifies the default error message displayed on the login form when
*  authentication fails.
* @type {string}
* @default "Invalid username or password"
*/
errorMessage?: string;

/**
* Specifies the dialog contents. By default, the dialog only contains
*  LoginDialog.loginForm. If desired, additional widgets may be placed before/after
*  the loginForm. To specify these widgets as autoChildren, use the syntax
*  "autoChild:childName" autoChildren.
* @type {Array<Partial<Canvas>> | Canvas | string}
* @default "["autoChild:loginForm"]"
*/
items?: Array<Partial<Canvas>> | Canvas | string;

/**
* Customizes the fields present in the dialog, or specifies new fields to be
*  present, in the same manner as with DynamicForm.fields.
* @type {Array<Partial<Partial<FormItem>>>}
* @default null
*/
formFields?: Array<Partial<Partial<FormItem>>>;

/**
* Field item containing login error message (if required) in LoginDialog.loginForm.
* @type {BlurbItem}
* @default null
*/
loginFailureItem?: BlurbItem;

/**
* Username field item in LoginDialog.loginForm.
* @type {TextItem}
* @default null
*/
usernameItem?: TextItem;

/**
* LinkItem to page requesting forgotten password in LoginDialog.loginForm.
*  To handle user clicks on this link, implement LoginDialog.lostPassword.
*  To handle a user click as a physical link to another page, set
*  FormItem.defaultValue via loginDialog.lostPasswordItemProperties:
*  
*  lostPasswordItemProperties: {
*    defaultValue: "register.html"
*  },
*  
* @type {LinkItem}
* @default null
*/
lostPasswordItem?: LinkItem;

/**
* LinkItem to page requesting new user registration in LoginDialog.loginForm.
*  To handle user clicks on this link, implement LoginDialog.register.
*  To handle a user click as a physical link to another page, set
*  FormItem.defaultValue via loginDialog.registrationItemProperties:
*  
*  registrationItemProperties: {
*    defaultValue: "register.html"
*  },
*  
* @type {LinkItem}
* @default null
*/
registrationItem?: LinkItem;

/**
* Password field item in LoginDialog.loginForm.
* @type {PasswordItem}
* @default null
*/
passwordItem?: PasswordItem;

/**
* Login submission button in LoginDialog.loginForm.
* @type {ButtonItem}
* @default null
*/
loginButton?: ButtonItem;

/**
* Form used to request login credentials from the user.
* @type {DynamicForm}
* @default null
*/
loginForm?: DynamicForm;



/**
*  Called if the user clicks on the LoginDialog.registrationItem
*  on the login form. Implement this method to allow the user to register for a
*  new account.
* 
* @param {object} Current values of form fields
* @param {DynamicForm} Form on which the link was clicked
*/
register(values:object, form:DynamicForm): void;

/**
*  Called if the user clicks on the LoginDialog.lostPasswordItem link
*  on the login form. Implement this method to allow the user to request the password
*  be resent or reset.
* 
* @param {object} Current values of form fields
* @param {DynamicForm} Form on which the link was clicked
*/
lostPassword(values:object, form:DynamicForm): void;

/**
*  User-supplied callback function to process login transactions.
*  If the user clicks the "Log in" button, the credentials entered by the user are passed to
*  loginFunc as an Object with properties "username" and "password" (NOTE: both
*  property names are all lowercase), as the variable "credentials". For example:
*  { username: "barney", password: "rUbbL3" }
*  
*  This function should then attempt to log in by whatever means is necessary. The second
*  parameter to the loginFunc, "dialogCallback", is a function, which must be called whether
*  login succeeds or fails with a true/false value indicating whether login succeeded.
*  
*  If the login dialog is dismissable (settable as properties.dismissable, default false) and
*  the user dismisses it, loginFunc will be fired with null for the credentials.
*  
*  The following code shows typical usage. This code assumes you have created a global
*  function sendCredentials() that send credentials to some authentication system and fires a
*  callback function with the result:
*  
*  ...
*  loginFunc : function (credentials, dialogCallback) {
*    if (credentials == null) return; // dismissed
* 
*    // send credentials
*    sendCredentials(credentials, function (loginSucceeded) {
*      // report success or failure
*      dialogCallback(loginSucceeded);
*    })
*  })
*  ...
*  
* 
* @param {object} Login credentials supplied by the user
* @param {Function} Function that must be called once the login transaction completes
*/
loginFunc(credentials:object, dialogCallback:Function): void;



}
/**
* Creates a new LoginDialog
* 
* @param typeCheckedProperties {Partial<LoginDialog>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {LoginDialog} The new instance
*/
class LoginDialog{
static create(typeCheckedProperties? : Partial<LoginDialog>, uncheckedProperties?: Object): LoginDialog;
/**
* Default title for the dialog displayed by isc.showLoginDialog.
*  A custom title can alternatively be specified as the title attribute of the
*  properties parameter passed to that method.
* @type {HTMLString}
* @default "Please log in"
*/
static LOGIN_TITLE?: HTMLString;

/**
* Default title for the LoginDialog.usernameItem field in the
*  dialog displayed by isc.showLoginDialog.
* @type {HTMLString}
* @default "Username"
*/
static USERNAME_TITLE?: HTMLString;

/**
* Default title for the LoginDialog.passwordItem field in the
*  dialog displayed by isc.showLoginDialog.
* @type {HTMLString}
* @default "Password"
*/
static PASSWORD_TITLE?: HTMLString;

/**
* Default title for login button in the dialog displayed by
*  isc.showLoginDialog.
* @type {HTMLString}
* @default "Log in"
*/
static LOGIN_BUTTON_TITLE?: HTMLString;

/**
* Default error message displayed on failed login in the dialog shown by
*  isc.showLoginDialog.
* @type {HTMLString}
* @default "Invalid username or password"
*/
static LOGIN_ERROR_MESSAGE?: HTMLString;








}

/**
* A Mail object describes an email that will be sent automatically by the 
*  SmartClient Server as part of a normal DataSource operation. Mail definitions
*  are part of an OperationBinding, and they can only be specified in a 
*  DataSource's XML definition. Despite the declarative nature of mail definitions, templating
*  support via Velocity makes it very flexible; it is very easy to include values from records
*  just fetched from permanent storage by this operation, for example. See 
*  velocitySupport for an overview of SmartClient Server's pervasive support for 
*  templated declarative operations using Velocity. 
*  
*  The default context for the message is the dsResponse.data; that is, $foo in the
*  message refers to the field "foo" in the DataSource response (the equivalent of
*  calling dsResponse.getFieldValue("foo") in Java). You also have access to the normal
*  Velocity variables described in the overview linked to above.
*  
*  If multiple records appear in the dsResponse, one message will be sent per record
*  in the response unless "multiple" is set to false. This allows a set of
*  recipients to be selected by a "fetch" operationBinding and messages sent to each.
*  This also means that if no data is selected by a fetch or affected by an update,
*  no mail is sent.
*  
*  If you have the transactionChaining feature (available 
*  in Power and better versions), all of the variables normally accessible in other Velocity 
*  contexts are available in the message template as well, for example: 
*  $responseData.last('order').orderId
*  
*  You can provide the body of the email directly in the Mail.messageTemplate
*  tag, or you can provide a filename in the Mail.templateFile attribute,
*  which tells SmartClient to use the contents of that file as the body of the email. If you 
*  provide neither of these, your email will have no body; if you provide both, SmartClient will
*  ignore the file and just use the content directly provided.
*  
*  Mail server configuration in server.properties
*  
*  Static mail server to use for sending emails can be configured in the
*  server_properties file. The following values can be provided:
*  mail.system.mail.smtp.host: The name of the SMTP server to use; defaults 
*  to "localhost"
*  mail.system.mail.smtp.port: What port is the MTA listening on; defaults to 25
*  mail.system.mail.smtp.auth: Whether this server requires authentication; 
*  defaults to false
*  mail.system.mail.smtp.user: SMTP user, if authentication is on; no default
*  mail.system.mail.smtp.password: Password, if authentication is on; no default
*  
*  Note that we will also pass any other properties that start mail.system through
*  to the underlying Javamail Session object, so you can use Javamail features
*  that aren't explicitly exposed by SmartClient's mail support. For example, many cloud-based
*  SMTP providers require that you issue a STARTTLS command before authenticating; you can 
*  achieve this by adding the following line to server_properties:
*  mail.system.mail.smtp.starttls.enable: true
*  
*  Mail server configuration in OperationBinding
*  
*  Dynamic mail server can be configured in DataSource.operationBindings under 
*  OperationBinding.mail tag. This allows to alter mail server settings for different
*  operations and depending on data using velocitySupport. Same values
*  can be provided as in server.properties omitting "mail.system.mail.smtp" prefix,
*  i.e. "host", "port", "auth", "user" and "password".
*  
*  Just like in server.properties any other properties can be provided and will be
*  passed through to the underlying Javamail Session object. Note that property
*  names must omit "mail.smtp" prefix, for example:
*  &lt;starttls.enable&gt;true&lt;/starttls.enable&gt;
*/
interface Mail extends Class {
/**
* Qualified name of the file containing the message template, relative to webroot. Like all
*  the other VelocityExpression properties of Mail, you can use Velocity
*  substitution variables in this property. The content itself (ie, the contents of the file
*  named in this property) can also contain Velocity substitutions, of course.
*  
*  You must specify either this property or Mail.messageTemplate, but
*  clearly it makes no sense to specify them both.
* @type {VelocityExpression}
* @default null
*/
templateFile?: VelocityExpression;

/**
* The text that will form the body of the mail message. Like all other String 
*  properties of Mail, you can use Velocity substitution variables in this property.
*  
*  You must specify either this property or Mail.templateFile, but clearly
*  it makes no sense to specify them both.
* @type {VelocityExpression}
* @default null
*/
messageTemplate?: VelocityExpression;

/**
* Comma-separated list of recipients. Like all other String 
*  properties of Mail, you can use Velocity substitution variables in this property.
*  
*  If you specify a "To:" line in the body of your email, recipient addresses will be parsed 
*  and this property will be ignored.
* @type {VelocityExpression}
* @default null
*/
to?: VelocityExpression;

/**
* Comma-separated list of "CC" recipients. Like all other String 
*  properties of Mail, you can use Velocity substitution variables in this property.
*  
*  If you specify a "Cc:" line in the body of your email, recipient addresses will be parsed 
*  and this property will be ignored.
* @type {VelocityExpression}
* @default null
*/
cc?: VelocityExpression;

/**
* Comma-separated list of "BCC" recipients. Like all other String 
*  properties of Mail, you can use Velocity substitution variables in this property.
*  
*  If you specify a "Bcc:" line in the body of your email, recipient addresses will be parsed 
*  and this property will be ignored.
* @type {VelocityExpression}
* @default null
*/
bcc?: VelocityExpression;

/**
* The email address the message will appear to come from. Like all other String 
*  properties of Mail, you can use Velocity substitution variables in this property.
*  
*  If you specify a "From:" line in the body of your email, the from address will be parsed 
*  and this property will be ignored.
*  
*  Ordinarily, this property (or the parsed equivalent in the email body) is mandatory for all
*  emails. However, if you specify the property mail.system.mail.smtp.from in 
*  server.properties, it will become the default "from" address, and this property
*  becomes optional.
* @type {VelocityExpression}
* @default null
*/
from?: VelocityExpression;

/**
* An email address to indicate as the reply-to address on the message. Like all other 
*  String properties of Mail, you can use Velocity substitution 
*  variables in this property.
* @type {VelocityExpression}
* @default null
*/
replyTo?: VelocityExpression;

/**
* The string to appear in the subject line of the message. Like all other String 
*  properties of Mail, you can use Velocity substitution variables in this property.
*  
*  If you specify a "Subject:" line in the body of your email, the subject will be parsed 
*  and this property will be ignored.
* @type {VelocityExpression}
* @default null
*/
subject?: VelocityExpression;

/**
* By default, multiple mail messages are sent if the dsResponse contains multiple records. 
*  Set this property to false to prevent this behavior.
* @type {boolean}
* @default null
*/
multiple?: boolean;

/**
* Allows you to specify an alternative source of substitution values for the templating 
*  process (by default, the data record(s) returned by the DataSource operation are used).
*  If set, this VelocityExpression should evaluate to a Java Map
*  object, or a Java List containing only Maps.
*  
*  For example, assuming you had suitable context stored in a Map that is
*  held in the current session as attribute "foo", the following would cause the mail 
*  templating system to look in there for values when running substitution:
*  
*    messageData="$session.foo"
*  
* @type {VelocityExpression}
* @default null
*/
messageData?: VelocityExpression;

/**
* Used to override the content-type header applied to the email. Primarily used to send 
*  HTML emails rather than plain text ones (use "text/html" to do this).
* @type {string}
* @default null
*/
contentType?: string;

/**
* Used to override the character encoding we apply to the email body. If you do not set 
*  this property, it is derived from the server_properties setting 
*  mail.system.default.encoding, which is set to "UTF-8" by default. This
*  setting should be appropriate for most use cases.
* @type {string}
* @default null
*/
encoding?: string;

/**
* Host of the SMTP server that will send the email. Like all other String
*  properties of Mail, you can use Velocity substitution variables in this property.
* @type {VelocityExpression}
* @default null
*/
host?: VelocityExpression;

/**
* Port of the SMTP server that will send the email. Like all other String
*  properties of Mail, you can use Velocity substitution variables in this property.
* @type {VelocityExpression}
* @default null
*/
port?: VelocityExpression;

/**
* Defines whether authentication is required by SMTP server that will send the email. Like
*  all other String properties of Mail, you can use Velocity substitution
*  variables in this property.
* @type {VelocityExpression}
* @default null
*/
auth?: VelocityExpression;

/**
* User if authentication is required by SMTP server that will send the email. Like
*  all other String properties of Mail, you can use Velocity substitution
*  variables in this property.
* @type {VelocityExpression}
* @default null
*/
user?: VelocityExpression;

/**
* Password if authentication is required by SMTP server that will send the email. Like
*  all other String properties of Mail, you can use Velocity substitution
*  variables in this property.
* @type {VelocityExpression}
* @default null
*/
password?: VelocityExpression;





}
/**
* Creates a new Mail
* 
* @param typeCheckedProperties {Partial<Mail>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Mail} The new instance
*/
class Mail{
static create(typeCheckedProperties? : Partial<Mail>, uncheckedProperties?: Object): Mail;







}

/**
* The definition of a function for use in the FormulaBuilder. A function consists of 
*  a name (what the user actually types to use the function), a description (shown in help) and 
*  an actual JavaScript function that executes the calculation.
*  
*  The built-in functions cover all static functionality on the JavaScript Math object:
*  
*  
* - max(val1,val2): Maximum of two values
*  
* - min(val1,val2): Minimum of two values
*  
* - round(value,decimalDigits): Round a value up or down, optionally providing 
*    decimalDigits as the maximum number of decimal places to round to. For fixed 
*    or precision rounding, use toFixed() and toPrecision() respectively.
*  
*  
* - ceil(value): Round a value up
*  
* - floor(value): Round a value down
*  
* - abs(value): Absolute value
*  
* - pow(value1,value2): value1 to the power of value2
*  
* - sqrt(value): Square root of a value
*  
* - dateAdd(value,interval,amount): Excel&trade;-compatible dataAdd function: adds 
*    quantities of a time interval to a date value. Also supports being passed interval 
*    names, like "hour" or "week".
*  
*  
* - toPrecision(value,precision): Format a number to a length of precision digits, 
*    rounding or adding a decimal point and zero-padding as necessary. Note that the 
*    values 123, 12.3 and 1.23 have an equal precision of 3. Returns a formatted string 
*    and should be used as the outermost function call in a formula. For rounding, use 
*    round().
*  
*  
* - toFixed(value,digits): Round or zero-pad a number to digits decimal places. 
*    Returns a formatted string and should be used as the outermost function call in a 
*    formula. To round values or restrict precision, use round() and 
*    toPrecision() respectively.
*  
*  
* - sin(value): Sine of a value
*  
* - cos(value): Cosine of a value
*  
* - tan(value): Tangent of a value
*  
* - ln(value): natural logarithm of a value
*  
* - log(base,value): logarithm of a value with the specified base
*  
* - asin(value): Arcsine of a value
*  
* - acos(value): Arccosine of a value
*  
* - atan(value): Arctangent of a value (-PI/2 to PI/2 radians)
*  
* - atan2(value1,value2): Angle theta of a point (-PI to PI radians)
*  
* - exp(value): The value of Evalue
*  
* - random(): Random number between 0 and 1
*  
*/
interface MathFunction extends Class {
/**
* Name of the function (what the user actually types). For example, a name of "min" would
*  indicate that the user types "min(someValue)" to use this function.
*  
*  Mixed-case names may be used. As a convenience, a few aliases are registered by 
*  MathFunction.registerFunction (see that method for details).
* @type {Identifier}
* @default null
*/
name?: Identifier;

/**
* A short description of this function
* @type {string}
* @default null
*/
description?: string;

/**
* Javascript method to perform the calculation associated with this function
* @type {Function}
* @default null
*/
jsFunction?: Function;

/**
* Indicates the sort-order of this MathFunction in an index returned from static method
*  MathFunction.getDefaultFunctionIndex. A lower value (&gt;= 0) will cause a function
*  to appear before a MathFunction with a higher value of the property. The default
*  of -1 means to exclude the MathFunction from the index entirely.
* @type {Integer}
* @default "-1"
*/
defaultSortPosition?: Integer;





}
/**
* Creates a new MathFunction
* 
* @param typeCheckedProperties {Partial<MathFunction>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MathFunction} The new instance
*/
class MathFunction{
static create(typeCheckedProperties? : Partial<MathFunction>, uncheckedProperties?: Object): MathFunction;




/**
*  Registers a new math function for use with FormulaFields. Mixed-case names are allowed,
*  and as a convenience, the following aliases are also available:
*  
* - name in all lowercase 
*  
* - name in all uppercase
*  
* - name with first letter uppercase, and the rest unchanged
* 
*  Note: The aliases are shallow copies of each other, so be aware that if MathFunction.jsFunction
*  depends on instance state, objects accessed by instance properties will be shared by all
*  copies.
* 
* 
* @param {MathFunction} */
static registerFunction(newFunction:MathFunction): void;

/**
*  Returns an index of all registered functions by name
*  
* 
*/
static getRegisteredFunctionIndex(): Integer;

/**
*  Returns an index of all default registered functions by name, ordered by 
*  MathFunction.defaultSortPosition. (Also includes those user-registered
*  functions with non-default (&gt;= 0) values for that property.)
*  
* 
*/
static getDefaultFunctionIndex(): Integer;




}

/**
* The Menu widget class implements interactive menu widgets, with optional icons, submenus,
*  and shortcut keys.
*  
*  A Menu is initialized with a set of MenuItems specified as Menu.items, each
*  of which represents one row in the menu's display and specifies the action to take when that
*  menu item is selected.
*  
*  Each MenuItem can have a MenuItem.title,
*  MenuItem.icon, MenuItem.keys, optional
*  MenuItem.submenu and various other settings. Alternatively, a MenuItem
*  can contain an arbitrary widget via MenuItem.embeddedComponent.
*  
*  To create a context menu for a component, provide a Menu instance for the
*  Canvas.contextMenu property. Note that some components like ListGrid have
*  more specific properties because they have distinct regions or because they have a default
*  set of context menu actions available (for example: ListGrid.headerContextMenu and
*  related APIs).
*  
*  If you want a button that pops up a menu when clicked, or a bar of such buttons, see the
*  MenuButton and MenuBar classes.
*  
*  To create a pop-up panel interface that looks nothing like a Menu (but still
*  dismisses automatically on an outside click), use Canvas.showClickMask to arrange
*  for automatic dismissal, and the Canvas.showNextTo utility method to place the
*  component near whatever triggered it, while automatically staying on-screen.
*/
interface Menu extends Omit<ListGrid, 'fields' | 'dataProperties'>  {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "MenuEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Option to show filter editor is disabled for menus
* @type {boolean}
* @default false
*/
canShowFilterEditor?: boolean;

/**
* Option to save searches is disabled for menus
* @type {boolean}
* @default false
*/
canSaveSearches?: boolean;

/**
* Custom properties for the automatically generated icon column.
*  
*  See Menu.showIcons for an overview of the icon column.
* @type {Partial<ListGridField>}
* @default null
*/
iconFieldProperties?: Partial<ListGridField>;

/**
* Default properties for the automatically generated icon column.
*  Default object includes properties to set width and to show icon for this column.
*  
*  To modify the behavior or appearance of this column, developers may set
*  Menu.iconFieldProperties at the instance level, or override this
*  object at the class level. If overriding this object, we recommend using
*  Class.changeDefaults rather than replacing this object entirely.
*  
*  See Menu.showIcons for an overview of the icon column.
* @type {Partial<ListGridField>}
* @default "object"
*/
iconFieldDefaults?: Partial<ListGridField>;

/**
* Custom properties for the automatically generated title column.
* @type {Partial<ListGridField>}
* @default null
*/
titleFieldProperties?: Partial<ListGridField>;

/**
* Default properties for the automatically generated title column.
*  Default object includes properties to set width and to show title for this column.
*  
*  To modify the behavior or appearance of this column, developers may set
*  Menu.titleFieldProperties at the instance level, or override this
*  object at the class level. If overriding this object, we recommend using
*  Class.changeDefaults rather than replacing this object entirely.
* @type {Partial<ListGridField>}
* @default "object"
*/
titleFieldDefaults?: Partial<ListGridField>;

/**
* Custom properties for the automatically generated key column.
*  
*  See Menu.showKeys for an overview of the key column.
* @type {Partial<ListGridField>}
* @default null
*/
keyFieldProperties?: Partial<ListGridField>;

/**
* Default properties for the automatically generated key column.
*  Default object includes properties to set width and to show key for this column.
*  
*  To modify the behavior or appearance of this column, developers may set
*  Menu.keyFieldProperties at the instance level, or override this
*  object at the class level. If overriding this object, we recommend using
*  Class.changeDefaults rather than replacing this object entirely.
*  
*  See Menu.showKeys for an overview of the key column.
* @type {Partial<ListGridField>}
* @default "object"
*/
keyFieldDefaults?: Partial<ListGridField>;

/**
* Custom properties for the automatically generated submenu column.
*  
*  See Menu.showSubmenus for an overview of the submenu column.
* @type {Partial<ListGridField>}
* @default null
*/
submenuFieldProperties?: Partial<ListGridField>;

/**
* Default properties for the automatically generated submenu column.
*  Default object includes properties to set width, align and to show submenu icon for this
*  column.
*  
*  To modify the behavior or appearance of this column, developers may set
*  Menu.submenuFieldProperties at the instance level, or override this
*  object at the class level. If overriding this object, we recommend using
*  Class.changeDefaults rather than replacing this object entirely.
*  
*  See Menu.showSubmenus for an overview of the submenu column.
* @type {Partial<ListGridField>}
* @default "object"
*/
submenuFieldDefaults?: Partial<ListGridField>;

/**
* An array of menuItem objects, specifying the menu items this menu should show.
*  
*  Data may also be set to a Tree in which case a hierarchy of menus and
*  submenus will automatically be generated to match the tree structure. See also
*  Menu.dataSource for dynamically fetching menuItems and submenus from a
*  hierarchical DataSource.
*  
*  Note that items that are marked as MenuItem.hidden will be automatically
*  filtered out of the data dispayed to the user. To retrieve the full set of items at 
*  runtime, including hidden items, use Menu.getAllItems.
* @type {Array<Partial<MenuItem>> | Array<Partial<Record>> | Tree | any}
* @default null
*/
data?: Array<Partial<MenuItem>> | Array<Partial<Record>> | Tree | any;

/**
* Synonym for Menu.data
* @type {Array<Partial<MenuItem>>}
* @default null
*/
items?: Array<Partial<MenuItem>>;

/**
* Optional DataSource to fetch menuItems and submenus from, instead of using Menu.items.
*  
*  Data is tree-based in menus, so the provided DataSource should be set up for hierarchical
*  fetching - see the treeDataBinding.
*  
*  Note that, although Menu is a subclass of ListGrid, some APIs, like 
*  ListGrid.setCriteria and ListGrid.autoFetchData
*  are not supported in menus. If a dataSource is supplied, it is automatically fetched 
*  against as required, without the need for autoFetchData. To apply criteria to the 
*  fetches made in this way, see Menu.initialCriteria.
*  Moreover, fetchData() is also an example of a ListGrid API that doesn't apply to menu, and, as 
*  was done for setCriteria() and other APIs like setCriteria().
* @type {DataSource}
* @default null
*/
dataSource?: DataSource;

/**
* Criteria to be used when fetching items for this Menu. Note that 
*  ListGrid.setCriteria is not supported in Menus.
* @type {Criteria}
* @default null
*/
initialCriteria?: Criteria;

/**
* This DataBoundComponent attribute is non-functional in Menus, where fetches are always
*  automatic.
* @type {boolean}
* @default false
*/
autoFetchData?: boolean;

/**
* For a Menu that uses a DataSource, these properties will be passed to
*  the automatically-created ResultTree. This can be used for various customizations such as
*  modifying the automatically-chosen Tree.parentIdField.
* @type {Partial<Tree>}
* @default null
*/
dataProperties?: Partial<Tree>;

/**
* Optional target canvas for this menu. Available as a parameter to dynamic menuItem configuration
*  methods such as MenuItem.checkIf.
*  
*  Whenever a Menu is shown as a contextMenu by a widget due to Canvas.contextMenu
*  being set, menu.target is automatically set to the widget that showed the
*  contextMenu.
*  
*  If this item has any MenuItem.submenu the target will be propagated down
*  to these child menus.
* @type {Canvas}
* @default null
*/
target?: Canvas;

/**
* Array of columns to show for this menu.
*  Standard menu fields may be included by specifying MenuFieldID
*  directly. Additional custom fields may be specified as ListGridField objects.
*  If this property is unset, default behavior will show the
*  MenuFieldID, with the exception of any that have
*  been suppressed via Menu.showIcons, Menu.showKeys and Menu.showSubmenus
* @type {Array<Partial<MenuFieldID>> | Array<Partial<ListGridField>>}
* @default null
*/
fields?: Array<Partial<MenuFieldID>> | Array<Partial<ListGridField>>;

/**
* CSS style used for the body of this menu when there is no icon field. When there is an
*  icon field, then Menu.iconBodyStyleName, if set, will override this setting.
*  
*  If applying a background-color to the body via a CSS style applied using this property,
*  be sure to set ListGrid.bodyBackgroundColor to null.
* @type {CSSStyleName}
* @default "normal"
*/
bodyStyleName?: CSSStyleName;

/**
* If set, the CSS style used for the body of this menu when there is an icon field.
*  In RTL mode, the iconBodyStyleName is suffixed with "RTL", which allows skins
*  to apply different styles in LTR and RTL modes.
*  
*  Note: Any skin which uses iconBodyStyleName should add "RTL" styles as well,
*  even if identical to LTR styles. Otherwise, menus may lose their styling in RTL mode.
* @type {CSSStyleName}
* @default null
*/
iconBodyStyleName?: CSSStyleName;

/**
* The default menu width.
* @type {Integer}
* @default "150"
*/
defaultWidth?: Integer;

/**
* The height of each item in the menu, in pixels.
* @type {number}
* @default "20"
*/
cellHeight?: number;

/**
* Menus will not draw on initialization, until they're explicitly show()n
* @type {boolean}
* @default false
*/
autoDraw?: boolean;

/**
* CSS style for a normal cell
* @type {CSSStyleName}
* @default "menu"
*/
baseStyle?: CSSStyleName;

/**
* Explicitly disable alternateRecordStyles at the menu level by default so setting
*  to true for all ListGrids will not impact menus' appearance.
* @type {boolean}
* @default false
*/
alternateRecordStyles?: boolean;

/**
* A boolean indicating whether this menu should use shortcut keys. Set useKeys to
*  false in a menu's initialization block to explicitly disable shortcut keys.
* @type {boolean}
* @default true
*/
useKeys?: boolean;

/**
* A boolean, indicating whether the shortcut key column should be displayed. If
*  showKeys is not set, the menu will show the key column only if one of its items
*  specifies a keys property. If showKeys is false, the keys will not be displayed,
*  but will still function.
* @type {boolean}
* @default true
*/
showKeys?: boolean;

/**
* A boolean, indicating whether the checkmark/custom icon column should be displayed.
* @type {boolean}
* @default true
*/
showIcons?: boolean;

/**
* A boolean, indicating whether the submenu indicator column should be displayed. If
*  showSubmenus is not set, the menu will show the indicator column only if one of its
*  items specifies a submenu property. If showSubmenus is false, the submenu arrows
*  will not be displayed, but submenus will still appear on rollover.
* @type {boolean}
* @default true
*/
showSubmenus?: boolean;

/**
* Should submenus show up on our left or right. Can validly be set to "left"
*  or "right". If unset, submenus show up on the right by default in
*  Left-to-right text mode, or on the left in Right-to-left text mode (see Page.isRTL).
* @type {string}
* @default null
*/
submenuDirection?: string;

/**
* Message to show when a menu is shown with no items.
* @type {HTMLString}
* @default "[Empty menu]"
*/
emptyMessage?: HTMLString;

/**
* The default width applied to custom icons in this menu. This is used whenever
*      item.iconWidth is not specified.
* @type {number}
* @default "16"
*/
iconWidth?: number;

/**
* The default height applied to custom icons in this menu. This is used whenever
*      item.iconHeight is not specified.
* @type {number}
* @default "16"
*/
iconHeight?: number;

/**
* When this menu is shown how should it animate into view? By default the menu will just
*  show at the specified size/position. Options for animated show effects are "fade"
*  to fade from transparent to visible, "slide" to slide the menu into view,
*  or "wipe" to have the menu grow into view, revealing its content as it
*  grows. Can be overridden by passing the 'animationEffect' parameter to 'menu.show()'
* @type {string}
* @default null
*/
showAnimationEffect?: string;

/**
* Default image to use for the submenu indicator. Valid src, width and height must be
*  specified. See ImgProperties for format.
*  If Menu.submenuDirection is set to "left", the image src will have
*  the suffix "_left" appended to it.
* @type {Partial<Img>}
* @default "{...}"
*/
submenuImage?: Partial<Img>;

/**
* Default image to use for the submenu indicator when item is disabled. Valid src, width and
*  height must be specified. See ImgProperties for format.
*  If Menu.submenuDirection is set to "left", the image src will have
*  the suffix "_left" appended to it.
* @type {Partial<Img>}
* @default "{...}"
*/
submenuDisabledImage?: Partial<Img>;

/**
* Default image to display for checkmarked items. See ImgProperties for format.
*  Valid src, width and height must be specified.
* @type {Partial<Img>}
* @default "{...}"
*/
checkmarkImage?: Partial<Img>;

/**
* Default image to display for disabled checkmarked items. See ImgProperties
*  for format. Valid src, width and height must be specified.
* @type {Partial<Img>}
* @default "{...}"
*/
checkmarkDisabledImage?: Partial<Img>;

/**
* If true, clicking or pressing Enter on a menu item that has a submenu will
*  select that item (with standard behavior of hiding the menus, calling click
*  handlers, etc) instead of showing the submenu.
* @type {boolean}
* @default null
*/
canSelectParentItems?: boolean;

/**
* When false, when a menu item is chosen (via mouse click or keyboard), the menu is not
*  automatically hidden, staying in place for further interactivity
* @type {boolean}
* @default true
*/
autoDismiss?: boolean;

/**
* When true any generated submenus will inherit Menu.autoDismiss
*  from this menu.
* @type {boolean}
* @default true
*/
cascadeAutoDismiss?: boolean;

/**
* When false, when a user clicks outside the menu, or hits the Escape key, this menu
*  will not be automatically hidden, staying in place for further interactivity.
* @type {boolean}
* @default true
*/
autoDismissOnBlur?: boolean;

/**
* When using a Tree or hierarchical DataSource as the menu's data, optional subclass of
*  Menu that should be used when generating submenus.
* @type {SCClassName}
* @default null
*/
submenuConstructor?: SCClassName;

/**
* When using a Tree or hierarchical DataSource as the menu's data, submenus are
*  automatically generated from child nodes. fetchSubmenus
*  can be set to false to disable this for the whole menu, or can be set false on a
*  per-item basis via MenuItem.fetchSubmenus.
* @type {boolean}
* @default true
*/
fetchSubmenus?: boolean;

/**
* Where should the menu be placed on the screen?
*  
*  Default is to use PanelPlacement "fillScreen" if Browser.isHandset. In
*  any non-handset device, placement is unset, so the menu defaults to normal
*  placement (near the originating MenuButton, or the mouse for a context menu, or
*  according to left/top/width/height for a manually created Menu).
*  
*  When using any placement setting that fills a portion of the screen or a 
*  panel, submenus are displayed by sliding them into place on top of the currently active
*  menu, and a NavigationBar is used to manage navigation to the
*  main menu (and provide dismissal, via a Menu.cancelButtonTitle.
* @type {PanelPlacement}
* @default null
*/
placement?: PanelPlacement;

/**
* Navigation bar shown when Menu.placement setting indicates that the menu should
*  be shown filling a portion of the screen or a panel.
* @type {NavigationBar}
* @default null
*/
navigationBar?: NavigationBar;

/**
* Title for the "Done" button shown when the NavigationBar is present.
* @type {HTMLString}
* @default "Done"
*/
cancelButtonTitle?: HTMLString;

/**
* When the Menu.placement setting indicates that the menu should be shown filling
*  a portion of the screen or a panel, navStack is a container element
*  created to hold the NavigationBar and any submenus that are shown by the menu.
* @type {Canvas}
* @default null
*/
navStack?: Canvas;

/**
* showEdges dynamically defaults to false when the Menu.placement setting
*  indicates the Menu will be filling a portion of the screen or a panel.
* @type {boolean}
* @default null
*/
showEdges?: boolean;

/**
* Whether to show a drop shadow for this Canvas. 
*  
*  Developers should be aware that the drop shadow
*  is drawn outside the specified width and height of the widget meaning a widget with shadows 
*  takes up a little more space than it otherwise would. A full screen canvas with showShadow set 
*  to true as this would be likely to cause browser scrollbars to appear - developers can handle
*  this by either setting this property to false on full-screen widgets, or by setting
*  overflow to "hidden" on the &lt;body&gt; element browser-level scrolling is never intended to occur.
*  
*  showShadow dynamically defaults to false when the Menu.placement setting
*  indicates the Menu will be filling a portion of the screen or a panel.
* @type {boolean}
* @default null
*/
showShadow?: boolean;

/**
* If set, alternative body style for the menu used when there is no icon field and the
*  Menu.placement settings indicate the menu will be filling a portion of the screen or
*  a panel. Generally this alternative style should not have rounded or excessively large
*  edges. If unset, then Menu.bodyStyleName is used instead.
*  
*  When there is an icon field, Menu.iconFillSpaceStyleName, if set, overrides this setting.
* @type {CSSStyleName}
* @default null
*/
fillSpaceStyleName?: CSSStyleName;

/**
* If set, alternative body style for the menu used when there is an icon field and the
*  Menu.placement settings indicate the menu will be filling a portion of the screen or
*  a panel. Generally this alternative style should not have rounded or excessively large
*  edges. In RTL mode, the iconFillSpaceStyleName is suffixed with "RTL", which
*  allows skins to apply different styles in LTR and RTL modes. If unset, then
*  Menu.iconBodyStyleName is used instead.
*  
*  Note: Like iconBodyStyleName, any skin which uses iconFillSpaceStyleName
*  should add "RTL" styles as well, even if identical to LTR styles. Otherwise, menus may
*  lose their styling in RTL mode.
* @type {CSSStyleName}
* @default null
*/
iconFillSpaceStyleName?: CSSStyleName;

/**
* Does this hide menu items marked as MenuItem.hidden by filtering
*  them out of the data set?
*  
*  In order to support hiding items marked as hidden, if any items are hidden or marked with
*  MenuItem.visibleWhen, Menus convert the specified 
*  Menu.data to a FilteredList with criteria set to
*  filter out items marked with the Menu.itemHiddenProperty.
*  
*  Note that this means for filterHiddenItems:true menus, developers wishing to
*  interact with the menu data set must use List APIs such as 
*  List.getLength and List.get,
*  rather than accessing simple array attributes such as menu.getData().length
*  or menu.getData()[index]
*  
*  To get the full specified data set as an array, including hidden items, use Menu.getAllItems.
* @type {boolean}
* @default true
*/
filterHiddenItems?: boolean;

/**
* Items with this property set to true will be hidden within the menu.
* @type {string}
* @default "hidden"
*/
itemHiddenProperty?: string;

/**
* For a menu that has a MenuButton generated for it automatically (for example when
*  included in a MenuBar, the width that the MenuButton should have. If unset, the
*  MenuButton will be as wide as menu.width.
* @type {Integer}
* @default null
*/
menuButtonWidth?: Integer;



/**
*  This DataBoundComponent method is not supported - use
*  Menu.initialCriteria to apply criteria to the fetches made by 
*  menus.
* 
* @param {Criteria | AdvancedCriteria} new criteria to show
*/
setCriteria(criteria:Criteria | AdvancedCriteria): void;

/**
*  This DataBoundComponent method does not apply to Menu.
* 
* 
* @param {Criteria=} Search criteria. If a DynamicForm is passed                     in as this argument instead of a raw criteria                      object, will be derived by calling                      DynamicForm.getValuesAsCriteria
* @param {DSCallback=} callback to invoke when a fetch is complete. Fires                     only if server contact was required
* @param {DSRequest=} additional properties to set on the DSRequest                     that will be issued
*/
fetchData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Show or hide the checkmark/custom icon column at runtime.
* 
* 
* @param {boolean} whether the icon column should be displayed
*/
setShowIcons(showIcons:boolean): void;

/**
*  Show or hide the submenu indicator column at runtime.
* 
* 
* @param {boolean} whether the submenu indicator column should be displayed
*/
setShowSubmenus(showSubmenus:boolean): void;

/**
*  Executed when a menu item with no click handler is clicked by the user. This
*  itemClick handler must be specified as a function. It is passed an item parameter that
*  is a reference to the clicked menu item.
* 
* 
* @param {object} pointer to the item in question
* @param {number=} Index of the column clicked by the user. May be null if             this menu item was activated in response to a keyboard event.
*/
itemClick(item:object, colNum?:number): boolean;

/**
*  Show this menu as a context menu, that is, immediately adjacent to the current mouse position.
* 
* 
*/
showContextMenu(): boolean;

/**
*  Change the set of items to display in this menu. Note that if 
*  Menu.filterHiddenItems is true and any items are MenuItem.hidden, 
*  data supplied as an Array will be converted to a FilteredList in order to filter out 
*  hidden items. 
*  To get the full specified data set as an array, including hidden items, use Menu.getAllItems.
*  
* 
* @param {Array<Partial<MenuItem>> | Array<Partial<Record>> | Tree | any} new items for this menu
*/
setData(items:Array<Partial<MenuItem>> | Array<Partial<Record>> | Tree | any): void;

/**
*  Synonym for Menu.setData.
* 
* @param {Array<Partial<MenuItem>>} new items for this menu
*/
setItems(items:Array<Partial<MenuItem>>): void;

/**
*  Retrieves the full set of items for this menu, including MenuItem.hidden
* 
*/
getAllItems(): Array<Partial<MenuItem>>;

/**
*  Hide the context menu - alias for hide()
* 
*/
hideContextMenu(): void;

/**
*  Get a particular MenuItem by index. Note that by default the index passed in is interpreted as
*  the position within currently visible set of items for the menu, not including hidden items.
*  
*  To look up an item based on its position in the full set of Menu.getAllItems,
*  including those that are not visible, pass in the logicalIndex parameter.
*  
*  If passed a MenuItem, returns it.
* 
* 
* @param {Integer} index of the MenuItem
* @param {boolean=} Interpret the index passed in with regards to the full set of specified items.
*/
getItem(item:Integer, logicalIndex?:boolean): MenuItem;

/**
*  Set arbitrary properties for a particular menu item.
* 
* 
* @param {MenuItem | Integer} MenuItem or visible index of the MenuItem
* @param {Partial<MenuItem>} properties to apply to the item
*/
setItemProperties(item:MenuItem | Integer, properties:Partial<MenuItem>): void;

/**
*  Given a MenuItem, return its index in the Menu.getAllItems,
*  including MenuItem.hidden.
*  
*  To retrieve the item's position in the currently visible set of items, use 
*  Menu.getVisibleItemNum.
* 
* 
* @param {MenuItem} Menu Item to find
*/
getItemNum(item:MenuItem): Integer;

/**
*  Given a MenuItem, return its index in the currently visible set of items.
*  
*  To get the index of the item in the Menu.getAllItems, including
*  MenuItem.hidden, use Menu.getItemNum instead
* 
* 
* @param {MenuItem} Menu Item to find
*/
getVisibleItemNum(item:MenuItem): Integer;

/**
*  Synonym for menu.getData()
* 
* 
*/
getItems(): Array<Partial<MenuItem>>;

/**
*  Get the visible MenuItems in the current menu.
*  
*  Note that if Menu.filterHiddenItems is true, and any items are marked as 
*  MenuItem.hidden or have MenuItem.visibleWhen, 
*  this method will return a 
*  FilteredList with criteria set to exclude any hidden items
*  
*  To retrieve the full set of items, including hidden items, use Menu.getAllItems.
*  
* 
*/
getData(): Array<Partial<MenuItem>>;

/**
*  Return the CSS class for a cell. By default this method has the following implementation:
*  - return any custom style for the record (GridRenderer.recordCustomStyleProperty) 
*   if defined.
*  - create a style name based on the result of GridRenderer.getBaseStyle and the 
*   state of the record using the rules described in cellStyleSuffixes.
*  
*  Cell Styles are customizable by:
*  
*  
* - attaching a custom style to a record by setting 
*   record[this.recordCustomStyleProperty] to some valid CSS style name. 
*  
* - modifying the base style returned by getBaseStyle() [see that method for further 
*    documentation on this]
*  
* - overriding this function
*  
*  
*  In addition to this, GridRenderer.getCellCSSText may be overriden to
*  provide custom cssText to apply on top of the styling attributes derived from the named
*  style.
*  
* 
* 
* @param {ListGridRecord} record object for this row and column
* @param {number} number of the row
* @param {number} number of the column
*/
getCellStyle(record:ListGridRecord, rowNum:number, colNum:number): CSSStyleName;

/**
*  Show the submenu for the specified item, if it has one.
*  
*  Normally triggered automatically by user interaction.
* 
* 
* @param {MenuItem | number} the item in question, or its index
*/
showSubmenu(item:MenuItem | number): void;

/**
*  Get the submenu for a particular menu item.
*  
*  Override to provide dynamic generation of submenus.
* 
* 
* @param {MenuItem | Integer} MenuItem or visible index of the MenuItem
*/
getSubmenu(item:MenuItem | Integer): Menu;

/**
*  Enables or disables the menu item according to the value of newState, and redraws
*  the menu if necessary. Returns true if there's a change in the enabled state.
* 
* 
* @param {MenuItem | Integer} MenuItem or visible index of the MenuItem
* @param {boolean=} true to enable the menu item, false to disable it. If not               passed, true is assumed
*/
setItemEnabled(item:MenuItem | Integer, newState?:boolean): boolean;

/**
*  Checks or unchecks the menu item according to the value of newState, and redraws
*  the menu if necessary. Returns true if there's a change in the checked state.
* 
* 
* @param {MenuItem | Integer} MenuItem or visible index of the MenuItem
* @param {boolean=} true to check the menu item, false to uncheck it. If not               passed, true is assumed
*/
setItemChecked(item:MenuItem | Integer, newState?:boolean): boolean;

/**
*  Hides or shows the menu item according to the value of newState, and redraws
*  the menu if necessary. Returns true if there's a change in the hidden state.
* 
* 
* @param {MenuItem | number} MenuItem in question, or its index. Note that if an index is passed it will be interpreted as the index in the full specified  set of items rather than the visible-index of the item.
* @param {boolean=} true to hide the menu item, false to show it.
*/
setItemHidden(item:MenuItem | number, newState?:boolean): boolean;

/**
*  Sets the title of a particular menu item to the string specified by newTitle and
*  redraws the menu if necessary.
* 
* 
* @param {MenuItem | Integer} MenuItem or visible index of the MenuItem
* @param {string} new title
*/
setItemTitle(item:MenuItem | Integer, newTitle:string): boolean;

/**
*  Sets the icon and disabled icon (if specified) for a particular menu item and redraws
*  the menu if necessary. Returns true if the icon changed.
* 
* 
* @param {MenuItem | Integer} MenuItem or visible index of the MenuItem
* @param {string} new icon URL
* @param {string=} new icon URL for disabled image
*/
setItemIcon(item:MenuItem | Integer, newIcon:string, newDisabledIcon?:string): boolean;



}
/**
* Creates a new Menu
* 
* @param typeCheckedProperties {Partial<Menu>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Menu} The new instance
*/
class Menu{
static create(typeCheckedProperties? : Partial<Menu>, uncheckedProperties?: Object): Menu;




/**
*  Hide all menus that are currently open. This method is useful to hide the current set of
*  menus including submenus, and dismiss the menu's clickMask.
* 
* 
*/
static hideAllMenus(): void;




}

/**
* A MenuBar is a bar of buttons used to show a set of menus.
*/
interface MenuBar extends Toolbar {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "MenuEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* An array of menu object initializers or instantiated menu objects. Buttons for each
*  menu item will automatically be created. See the Menu Widget Class for fundamental
*  menu properties and other properties. Titles for the buttons are derived from the 
*  title property of each menu.
* @type {Array<Partial<Menu>>}
* @default null
*/
menus?: Array<Partial<Menu>>;

/**
* By default exclude menubars from the page's tab order. To include a menubar in the page's
*  tab order, set tabIndex to an explicit tab index, or null for automatically 
*  assigned tabIndex
* @type {number}
* @default "-1"
*/
tabIndex?: number;



/**
*  Invalid to call on Menubar, use MenuBar.setMenus instead.
* 
* @param {Array<Partial<Partial<Button>>>=} invalid; do not call
*/
setButtons(newButtons?:Array<Partial<Partial<Button>>>): void;

/**
*  Dynamically reset the set of menus displayed by this menu bar. 
* 
* @param {Array<Partial<Menu>>} array of new menus for this menubar
*/
setMenus(menus:Array<Partial<Menu>>): void;

/**
*  Dynamically update the menuBar to include additional menus. Will update the visible set
*  of buttons as appropriate
* 
* @param {Array<any>} Array of new menus to add
* @param {number} desired starting position of the new menus in the existing menus  array
*/
addMenus(newMenus:Array<any>, position:number): void;

/**
*  Dynamically remove menus from the menuBar. Will update the visible set of buttons as 
*  appropriate.
* 
* @param {Array<any>} Array of menus to remove (will accept actual Menu components,            or numbers representing the index of the menus in the current menus array)
*/
removeMenus(menus:Array<any>): void;

/**
*  Shows (opens) a menu.
* 
* @param {Menu | Integer} menu to show (may be specified as a menu object, or index of                  the menu from MenuBar.menus).
*/
showMenu(menu:Menu | Integer): void;



}
/**
* Creates a new MenuBar
* 
* @param typeCheckedProperties {Partial<MenuBar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MenuBar} The new instance
*/
class MenuBar{
static create(typeCheckedProperties? : Partial<MenuBar>, uncheckedProperties?: Object): MenuBar;







}

/**
* Simple subclass of button associated with a menu widget (gets shown below the button).
*/
interface MenuButton extends Button {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "MenuEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Default title for the button.
* @type {string}
* @default "Show Menu"
*/
title?: string;

/**
* Default height of the button.
* @type {number | string}
* @default "22"
*/
height?: number | string;

/**
* This property corresponds to the inherited Button.icon property,
*  which is used to display the menuButtonImage, so anything you attempt 
*  to set there would be clobbered by the internal usage. 
*  
*  You could add an icon via the MenuButton.title property, 
*  by using Canvas.imgHTML to generate an appropriate 
*  &lt;img&gt; tag and pre-pending it to your title.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* Show menu button image (up / down arrowhead) for this menu button.
* @type {boolean}
* @default true
*/
showMenuButtonImage?: boolean;

/**
* Image for menu button indicating that the button expands a menu. This image is shown
*  for menus expanding down from the button.  Menu direction is controlled by
*  MenuButton.showMenuBelow.
* @type {SCImgURL}
* @default "[SKIN]menu_button.gif"
*/
menuButtonImage?: SCImgURL;

/**
* Image for menu button indicating that the button expands a menu. This image is shown
*  for menus expanding up from the button.  Menu direction is controlled by
*  MenuButton.showMenuBelow.
* @type {SCImgURL}
* @default "[SKIN]menu_button_up.gif"
*/
menuButtonImageUp?: SCImgURL;

/**
* If this MenuButton has a specified Canvas.accessKey, underline it
*  in the title of the button by default
* @type {boolean}
* @default true
*/
hiliteAccessKey?: boolean;

/**
* The menu drops down below the menu button.
*  Set to false if the menu should appear above the menu button.
* @type {boolean}
* @default true
*/
showMenuBelow?: boolean;

/**
* The horizontal alignment of this button's menu, in relation to the button. When unset,
*  default behavior is to align the right edges of button and menu if the page is in RTL 
*  mode, and the left edges otherwise.
* @type {Alignment}
* @default null
*/
menuAlign?: Alignment;

/**
* The menu to show.
*  
*  For a menu button with no menu (menu: null) the up/down arrow image can
*  be suppressed by setting
*  MenuButton.showMenuButtonImage: false.
* @type {Menu}
* @default null
*/
menu?: Menu;

/**
* Allows you to specify an animation effect to apply to the menu when it is being shown.
*  Valid options are "none" (no animation), "fade", "slide" and "wipe".
*  If unspecified falls through to menu.showAnimationEffect
* @type {string}
* @default null
*/
menuAnimationEffect?: string;

/**
* If this menuButton is Canvas.destroy, should it also destroy
*  its MenuButton.menu?
* @type {boolean}
* @default true
*/
autoDestroyMenu?: boolean;

/**
* Should the menu be shown automatically when the mouse moves over the button?
*  
*  When enabled, menus used with this MenuButton should not be used with any
*  other component.
* @type {boolean}
* @default false
*/
showMenuOnRollOver?: boolean;

/**
* When MenuButton.showMenuOnRollOver is true, this is the delay 
*  in milliseconds before the menu is automatically hidden following mouseOut.
* @type {number}
* @default "250"
*/
rollOverMenuHideDelay?: number;



/**
*  Setter for the 'showMenuButtonImage' property - shows/hides the menu button image
*  at runtime.
* 
* 
* @param {boolean} Should the image be shown
*/
setShowMenuButtonImage(show:boolean): void;

/**
*  Setter for the 'showMenuButtonBelow' property - determines whether the menu will be
*  shown above or below the MenuButton.
* 
* @param {boolean} True if the menu should be shown below the MenuButton.
*/
setShowMenuBelow(below:boolean): void;

/**
*  Programmatically forces this MenuButton to show it's menu.
* 
*/
showMenu(): void;



}
/**
* Creates a new MenuButton
* 
* @param typeCheckedProperties {Partial<MenuButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MenuButton} The new instance
*/
class MenuButton{
static create(typeCheckedProperties? : Partial<MenuButton>, uncheckedProperties?: Object): MenuButton;







}

/**
* EditProxy that handles MenuButton and MenuBar objects when editMode is enabled.
*/
interface MenuEditProxy extends CanvasEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's menu definition in wiki-style.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's menu.
*  
*  Lines starting with "--", "==" or "title:" are considered titles for the
*  MenuButtons. The menuItem definitions follow the title to define the menu
*  contents.
*  
*  Each menuItem title is entered on its own line. A keyTitle can follow the title
*  separated by a comma. A leading "x" or "o" marks the menuItem as checked.
*  MenuItems can be marked as disabled with a leading or trailing dash (-).
*  A sub-menu is indicated with a trailing &gt;. Any line consisting entirely of
*  one or more dashes (-) or equals (=) indicates a separator line. 
* 
* 
* @param {string} the new component menu
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new MenuEditProxy
* 
* @param typeCheckedProperties {Partial<MenuEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MenuEditProxy} The new instance
*/
class MenuEditProxy{
static create(typeCheckedProperties? : Partial<MenuEditProxy>, uncheckedProperties?: Object): MenuEditProxy;







}

/**
* A Menu that implements the Palette behavior, so it can be used as the source for 
*  drag and drop instantiation of components when combined with an EditContext as 
*  the drop target.
*  
*  Each MenuItem can be a PaletteNode.
*/
interface MenuPalette extends Menu, Palette {




}
/**
* Creates a new MenuPalette
* 
* @param typeCheckedProperties {Partial<MenuPalette>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MenuPalette} The new instance
*/
class MenuPalette{
static create(typeCheckedProperties? : Partial<MenuPalette>, uncheckedProperties?: Object): MenuPalette;







}

/**
* The Real-Time Messaging module creates a channel for messages to be sent from the server to
*  the client (a.k.a. "server push") in real-time (as opposed to periodically polling the
*  server for updates).
*  
*  See messaging for information.
*/
interface Messaging extends Class {




}
/**
* Creates a new Messaging
* 
* @param typeCheckedProperties {Partial<Messaging>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Messaging} The new instance
*/
class Messaging{
static create(typeCheckedProperties? : Partial<Messaging>, uncheckedProperties?: Object): Messaging;
/**
* URL where the MessagingServlet has been installed. See the server-side JavaDoc for
*  details on the MessagingServlet.
* @type {URL}
* @default "[ISOMORPHIC]/messaging"
*/
static messagingURL?: URL;

/**
* URL where the WebSocket endpoint is registered. On the server, the URI of the endpoint
*  is controled by the server.properties configuration setting
*  messaging.websocket.URI and defaults to /isomorphic/websocket
*  
*  This URL must ultimately start with ws:// or wss://, but you may specify this value as
*  starting with http:// or https:// and it will be automatically converted to ws(s)://
* @type {URL}
* @default "[ISOMORPHIC]/websocket"
*/
static websocketURL?: URL;

/**
* Whether to attempt to use the WebSocket protocol for Realtime Messaging. Enabled by
*  default. When enabled, the webSocket protocol is
*  preferred and tried first, but a failure to initially connect using webSockets causes
*  the client to fall back to http push.
*  
*  See messaging for more detail on messaging protocols.
*  
*  Note that once a successful connection using the webSocket protocol is established, the
*  protocol is marked as available and will be used until the client browser page is
*  reloaded (at which point it will again be preferred, but backed off if the initial
*  connection fails).
* @type {boolean}
* @default true
*/
static useWebSocket?: boolean;

/**
* Whether or not to fall back to Comet if Messaging.useWebSocket communication
*  fails, and has never succeeded since page load. Once WebSocket communication succeeds,
*  we never fall back.
*  
*  If we're not falling back to Comet, we'll reattempt connecting via WebSockets if we time
*  out trying to open a WebSocket connection, or an open WebSocket reports an error.
* @type {boolean}
* @default true
*/
static fallBackToComet?: boolean;

/**
* Number of milliseconds to wait when initiating a server connection before dropping the
*  attempt and trying again.
* @type {Integer}
* @default "10000"
*/
static websocketConnectTimeout?: Integer;

/**
* Number of milliseconds to wait when initiating a server connection before dropping the
*  attempt and trying again.
* @type {Integer}
* @default "4000"
*/
static connectTimeout?: Integer;





/**
*  Send data to one or more channels, by sending the data to the server, where it is expected to
*  be delivered to any other browser subscribed to the same channel.
*  
*  
*  Data can be any nested structure of primitive types (String, Date, Number) or compound data
*  structure consisting of Arrays and Objects containing primitive types.
*  
*  
*  The optional callback fires when the message has been successfully sent, or if an error
*  occurs.
* 
* 
* @param {string | Array<Partial<string>>} * @param {any} * @param {RPCCallback} */
static send(channels:string | Array<Partial<string>>, data:any, callback:(response:RPCResponse, rawData:any, request:RPCRequest) => void): void;

/**
*  Returns an array of identifiers for all currently subscribed channels on this client
* 
* 
*/
static getSubscribedChannels(): Array<Partial<string>>;

/**
*  Subscribes the client to the messaging channel identified by channel.
*  
*  When the server or another connected browser sends a message on this channel, the callback 
*  will be invoked with a single 'data' parameter containing the data that was just sent to the
*  channel.
*  
*  Calling subscribe() again for a channel you are already subscribed to will result in the new
*  callback replacing the old, and will cause the server connection to be re-established.
* 
* 
* @param {string} * @param {MessagingCallback} callback fired whenever data is sent to this channel
* @param {Callback} callback fired when the subscription is established
* @param {string=} JMS selector used with Queues to filter the messages that arrive to the channel (optional).
*/
static subscribe(channel:string, callback:(data:any) => void, subscribeCallback:Callback, selector?:string): void;

/**
*  Unsubscribes the client from the messaging channel identified by channel.
*  
*  This will cause a reconnect to the server - for this reason
*  we defer the actual reconnect to allow for multiple unsubscribe() calls in sequence.
* 
* 
* @param {string} */
static unsubscribe(channel:string): void;

/**
*  Returns whether the Messaging system currently has a connection to the server.
*  
*  This is not perfectly authoritative since the server may crash at any given instant. This
*  method will return true if we have opened a connection to the server and the connection has
*  been successfully acknowledged by the server.
* 
* 
*/
static connected(): boolean;

/**
*  Sever the connection to the server and stop receiving messages.
* 
*/
static disconnect(): void;

/**
*  Called when the messaging connection allowing the server to send messages to the client is
*  established - whether that's the result of an initial connect() or a re-establishment after
*  it is severed.
* 
*/
static connectionUp(): void;

/**
*  Called when the messaging connection allowing the server to send messages to the client is
*  disconnected. This can occur either when you explicitly disconnect the connection or if a
*  keepalive packet from the server does not arrive on time. This latter is defined as the
*  messaging.keepaliveInterval plus the messaging.keepaliveReestablishDelay.
*  With default settings, a maximum of 4 seconds will elapse between the connection actually
*  going down and you receiving this callback.
*  
* 
*/
static connectionDown(): void;




}

/**
* Provides a compact interface for editing a date range, by providing a formatted, read-only
*  display of the current selected date range with an icon to launch a DateRangeDialog 
*  to edit the range.
*/
interface MiniDateRangeItem extends StaticTextItem {
/**
* Base CSS class name for a form item's text box element.
*  
*  See formItemStyling for an overview of formItem styling, and 
*  the CompoundFormItem_skinning discussion for special
*  skinning considerations.
*  
*  If the textBoxStyle is changed at runtime, FormItem.updateState
*  must be called to update the visual state of this item.
* @type {FormItemBaseStyle}
* @default "textItem"
*/
textBoxStyle?: FormItemBaseStyle;

/**
* Allow miniDateRangeItems' values to show up in the form's values array, or if 
*  DynamicForm.getValuesAsCriteria is called, for the criterion to be included
*  in the returned AdvancedCriteria object
* @type {boolean}
* @default true
*/
shouldSaveValue?: boolean;

/**
* Pop-up DateRangeDialog for entering a date range.
* @type {DateRangeDialog}
* @default null
*/
rangeDialog?: DateRangeDialog;

/**
* MiniDateRangeItems are marked as canFocus:true, and set up with properties such that focus
*  will always go to the icon to launch the dateRange dialog. Set canFocus to false to 
*  suppress this behavior.
* @type {boolean}
* @default true
*/
canFocus?: boolean;

/**
* MiniDateRangeItems rely on their icon being able to receive focus for normal user
*  interaction as they have no other focusable element. canTabToIcons is
*  overridden to achieve this even if the property has been set to false at
*  the DynamicForm.canTabToIcons.
* @type {boolean}
* @default true
*/
canTabToIcons?: boolean;

/**
* The text to prepend to the formatted date when only a MiniDateRangeItem.fromDate is supplied.
* @type {string}
* @default "Since"
*/
fromDateOnlyPrefix?: string;

/**
* The text to prepend to the formatted date when only a MiniDateRangeItem.toDate is supplied.
* @type {string}
* @default "Before"
*/
toDateOnlyPrefix?: string;

/**
* The prompt to show when the mouse is hovered over the MiniDateRangeItem.pickerIcon.
* @type {HTMLString}
* @default "Show Date Chooser"
*/
pickerIconPrompt?: HTMLString;

/**
* Icon that launches a DateChooser for choosing an absolute date.
* @type {Partial<FormItemIcon>}
* @default null
*/
pickerIcon?: Partial<FormItemIcon>;

/**
* Whether the DateRangeDialog opened when the 
*  MiniDateRangeItem.pickerIcon is clicked should display 
*  RelativeDateItems or DateItems.
* @type {boolean}
* @default true
*/
allowRelativeDates?: boolean;

/**
* Format for displaying dates to the user. 
*  
*  If this attribute is unset, the display value is formatted intelligently according to the
*  dates involved. For example, if both dates appear in the same month, the value will be 
*  formatted as 
*  Month date1 - date2, Year 
*  and, if in different months of the same year, 
*  Month1 date1 - Month2 date2, Year.
*  If either date-value is unset, the display-value is formatted according to 
*  MiniDateRangeItem.fromDateOnlyPrefix and 
*  MiniDateRangeItem.toDateOnlyPrefix.
* @type {DateDisplayFormat}
* @default null
*/
dateDisplayFormat?: DateDisplayFormat;

/**
* Initial value for the "from" date.
* @type {Date | RelativeDateString | TimeUnit}
* @default "today"
*/
fromDate?: Date | RelativeDateString | TimeUnit;

/**
* Initial value for the "to" date.
* @type {Date | RelativeDateString | TimeUnit}
* @default "today"
*/
toDate?: Date | RelativeDateString | TimeUnit;

/**
* If this attribute is set to true, the pop up date range dialog will automatically
*  validate the user-entered "to" and "from" values on 
*  "OK"-click, and refuse to dismiss if these items contain invalid values.
* @type {boolean}
* @default true
*/
autoValidate?: boolean;



/**
*  Setter for MiniDateRangeItem.autoValidate
* 
* @param {boolean} New auto-validate setting.
*/
setAutoValidate(autoValidate:boolean): void;

/**
*  Overridden to return true: dateRangeItems always generate AdvancedCriteria.
* 
*/
hasAdvancedCriteria(): boolean;

/**
*  Returns the Criterion entered in the fields shown in the 
*  MiniDateRangeItem.rangeDialog.
*  
*  If both dates are entered, a Criterion with an "and" OperatorId 
*  and both "greaterOrEqual" and "lessOrEqual" sub-criteria will be returned. If either
*  date is omitted, only the "greaterOrEqual" (from date) or "lessOrEqual" (to date) 
*  Criterion is returned.
* 
* 
*/
getCriterion(): Criterion;

/**
*  Applies the specified criterion to this item for editing. Applies any specified 
*  "greaterOrEqual" operator criterion or sub-criterion to our DateRangeItem.fromField and any
*  specified "lessOrEqual" operator criterion or sub-criterion to our DateRangeItem.toField.
*  
*  Note that a single "equals" criterion can also be passed. See 
*  DateRangeItem.canEditCriterion for more detail.
* 
* @param {Criterion} criterion to edit
*/
setCriterion(criterion:Criterion): void;

/**
*  Returns true if the specified criterion contains:
*  
* - A single "lessOrEqual" or "greaterOrEqual" criterion on this field
*    
* - An "and" type criterion containing a "lessOrEqual" and a "greaterOrEqual" criterion
*      on this field
*    
* - A single "equals" criterion. Internally, this will be converted into a range
*      by constructing an "and" type criterion containing both a "lessOrEqual" and 
*      a "greaterOrEqual" criterion on this field. Note that subsequent calls to 
*      DateRangeItem.getCriterion will return this more complex 
*      criterion.
*  
* 
* @param {Criterion} criterion to test
*/
canEditCriterion(criterion:Criterion): boolean;

/**
*  Sets the value for this miniDateRangeItem. The value parameter is a 
*  DateRange object that optionally includes both start and end values.
* 
* @param {DateRange} the new value for this item
*/
setValue(value:DateRange): void;

/**
*  Retrieves the current value of this dateRangeItem. The return value is a 
*  DateRange object that excludes start and end values if they aren't
*  set.
* 
*/
getValue(): DateRange;



}
/**
* Creates a new MiniDateRangeItem
* 
* @param typeCheckedProperties {Partial<MiniDateRangeItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MiniDateRangeItem} The new instance
*/
class MiniDateRangeItem{
static create(typeCheckedProperties? : Partial<MiniDateRangeItem>, uncheckedProperties?: Object): MiniDateRangeItem;







}

/**
* Compact control for up/down navigation that roughly looks like an up arrowhead next to a
*  down arrowhead.
*/
interface MiniNavControl extends StretchImgButton {
/**
* @type {SCImgURL}
* @default "images/NavigationBar"
*/
skinImgDir?: SCImgURL;

/**
* Image used for the up arrowhead.
* @type {SCImgURL}
* @default "[SKIN]/up.png"
*/
upButtonSrc?: SCImgURL;

/**
* Image used for the down arrowhead.
* @type {SCImgURL}
* @default "[SKIN]/down.png"
*/
downButtonSrc?: SCImgURL;



/**
*  Notification method fired when the up button is clicked.
* 
* 
*/
upClick(): void;

/**
*  Notification method fired when the down button is clicked.
* 
* 
*/
downClick(): void;



}
/**
* Creates a new MiniNavControl
* 
* @param typeCheckedProperties {Partial<MiniNavControl>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MiniNavControl} The new instance
*/
class MiniNavControl{
static create(typeCheckedProperties? : Partial<MiniNavControl>, uncheckedProperties?: Object): MiniNavControl;







}

/**
* MinimalScrollbar is an alternative scrollbar interface to the standard Scrollbar.
*  It has a number of differences:
*  
*  
* - MinimalScrollbar has no end buttons - it consists of just a thumb floating over a track
*  
* - The appearance is achieved entirely via CSS - see MinimalScrollbar.trackBaseStyle and
*    MinimalScrollbar.thumbBaseStyle
*  
* - By default MinimalScrollbars automatically become visible when the user is actually 
*    scrolling a target, and hide themselves when the interaction is complete. 
*    See MinimalScrollbar.autoShow.
*    In effect this means that as a user drag-scrolls a component, the scrollbars
*    will show up as a temporary indicator of the current drag position.
*    Note: we recommend always Canvas.floatingScrollbars in
*    conjunction with autoShow:true
*  
* - MinimalScrollbars have an MinimalScrollbar.interactive mode in which they appear more substantial.
*    By default this mode is turned on as
*    MinimalScrollbar.setInteractiveOnMouseOver
*  
* - The MinimalScrollbar.contrastSuffix feature allows minimalScrollbars to show an appropriate
*    appearance to contrast against either light or dark colored backgrounds.
*    By default canvases handle switching to the appropriate contrast setting based on the
*    components rendered background-color.
*  
*  To enable MinimalScrollbars we recommend the following component-level settings:
*  
*  
* - Canvas.scrollbarConstructor
*  
* - Canvas.showCustomScrollbars
*  
* - Canvas.nativeAutoHideScrollbars
*  
* - Canvas.floatingScrollbars
*  
*/
interface MinimalScrollbar extends Canvas {
/**
* Suffix to apply to the MinimalScrollbar.thumbBaseStyle
*  and MinimalScrollbar.trackBaseStyle to show scrollbars with an appropriate
*  contrast against a light or dark background
* @type {MinimalScrollbarContrastSuffix}
* @default "Dark"
*/
contrastSuffix?: MinimalScrollbarContrastSuffix;

/**
* Base style to apply to the minimal scrollbar track.
*  The actual Canvas.styleName applied to the track will be generated
*  by applying the specified MinimalScrollbar.contrastSuffix and, if MinimalScrollbar.interactive is true
*  the additional sufix "Interactive"
*  
*  For example, a minimalScrollbar with trackBaseStyle set to "minimalScrollTrack",
*  and MinimalScrollbar.contrastSuffix set to "Light" would pick up the cssStyle
*  "minimalScrollTrackLight" by default, or 
*  "minimalScrollTrackLightInteractive" when MinimalScrollbar.interactive was set
*  to true
* @type {string}
* @default "minimalScrollTrack"
*/
trackBaseStyle?: string;

/**
* Base style to apply to the minimal scrollbar thumb.
*  The actual Canvas.styleName applied to the thumb will be generated
*  by applying the specified MinimalScrollbar.contrastSuffix and, if MinimalScrollbar.interactive is true
*  the additional sufix "Interactive"
*  
*  For example, a minimalScrollbar with thumbBaseStyle set to "minimalScrollThumb",
*  and MinimalScrollbar.contrastSuffix set to "Light" would pick up the cssStyle
*  "minimalScrollThumbLight" by default, or 
*  "minimalScrollThumbLightInteractive" when MinimalScrollbar.interactive was set
*  to true
* @type {string}
* @default "minimalScrollThumb"
*/
thumbBaseStyle?: string;

/**
* The minimum pixel size of the draggable thumb regardless of how large the scrolling
*  region becomes.
* @type {number}
* @default "12"
*/
thumbMinSize?: number;

/**
* Inset of the thumb relative to the track. An inset of N pixels means the thumb is 2N
*  pixels smaller in breadth than the track.
* @type {number}
* @default "2"
*/
thumbInset?: number;

/**
* Is this scrollbar currently in "interactive" mode? If true, the scrollbar will be shown
*  with a more prominent appearance, making it clear the user can interact with it by
*  clicking the track or dragging the thumb.
*  
*  If MinimalScrollbar.setInteractiveOnMouseOver is true, this property will automatically
*  be updated on mouseOver.
* @type {boolean}
* @default false
*/
interactive?: boolean;

/**
* Default thickness for this scrollbar.
*  
*  If the scrollbar is in MinimalScrollbar.interactive, breadth
*  will be set to MinimalScrollbar.interactiveScrollbarSize instead.
* @type {number}
* @default "12"
*/
scrollbarSize?: number;

/**
* Thickness for this scrollbar in MinimalScrollbar.interactive.
* @type {number}
* @default "19"
*/
interactiveScrollbarSize?: number;

/**
* Always set to MinimalScrollbar.interactive mode on roll over.
* @type {boolean}
* @default true
*/
setInteractiveOnMouseOver?: boolean;

/**
* Should this scrollbar automatically show itself when the scroll position / size
*  changes and hide itself after a MinimalScrollbar.autoHideDelay when left untouched?
* @type {boolean}
* @default true
*/
autoShow?: boolean;

/**
* If MinimalScrollbar.autoShow is true, the scrollbar will auto hide after
*  the user has stopped interacting with it for this many milliseconds.
* @type {number}
* @default "1000"
*/
autoHideDelay?: number;



/**
*  Set MinimalScrollbar.interactive at runtime. For MinimalScrollbar.setInteractiveOnMouseOver
*  scrollbars this method is invoked automatically at the appropriate times
* 
*/
setInteractive(): void;



}
/**
* Creates a new MinimalScrollbar
* 
* @param typeCheckedProperties {Partial<MinimalScrollbar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MinimalScrollbar} The new instance
*/
class MinimalScrollbar{
static create(typeCheckedProperties? : Partial<MinimalScrollbar>, uncheckedProperties?: Object): MinimalScrollbar;







}

/**
* A special kind of DataSource.clientOnly that can be configured
*  with MockDataSource.mockData - a simple text format for table or tree
*  data.
*  
*  MockDataSources are produced by the Reify Mockup Importer when starting from mockup formats
*  that use the mock data format. The docs for the 
*  balsamiqImport explain various steps for converting a
*  MockDataSource to a real DataSource.
*  
*  MockDataSource is primarily intended as a temporary form of DataSource used
*  during the process of converting a mockup into a real application. Generally, if creating a
*  client-only DataSource in JavaScript ,
*  there is no reason to use the mock data format, as the mock data is not especially readable
*  when written as a String literal. The mock data format can be a slightly more
*  compact and readable as compared to declaring DataSource.cacheData in XML.
*  
*  Note: If a MockDataSource has DataSource.addGlobalId
*  set to true, it will be made available in global scope.
*  
*  Unlike other DataSources, if a MockDataSource is created with an ID that is already in use
*  by another DataSource, the existing DataSource will not be destroy()'d 
*  and the new MockDataSource will not be available by ID.
*  Similarly, if a MockDataSource exists and a new DataSource is created with the same ID
*  the MockDataSource will be destroy()'d automatically without logging a
*  warning to the developer console.
*  This means if application code changes to replace a MockDataSource with a "real" 
*  dataSource it will function as expected, without warnings, even if the MockDataSource 
*  creation code was not removed, regardless of the order in which
*  the MockDataSource and "real" dataSource are created.
*/
interface MockDataSource extends DataSource {
/**
* Data intended for a ListGrid or TreeGrid, expressed in a simple text
*  format popularized by mockup tools such as balsamiq and now
*  commonly supported in a variety of mockup tools.
*  
*  Balsamiq publishes documentation of the grid format 
*  here,
*  with a simple example of using tree-specific formatting
*  here.
*  
*  MockData can also be provided as XML, CSV or JSON text by setting MockDataFormat to
*  the correct format.
*  
*  An alternative format of data consisting of an array of Record can
*  also be provided. In this case the records are converted to "grid" MockDataType.
* @type {string | Array<Partial<Record>>}
* @default "md"
*/
mockData?: string | Array<Partial<Record>>;

/**
* Format of data provided in MockDataSource.mockData. See MockDataFormat.
* @type {MockDataFormat}
* @default "mock"
*/
mockDataFormat?: MockDataFormat;

/**
* When MockDataSource.mockDataFormat is "mock", whether MockDataSource.mockData 
*  is in the "grid" or "tree" format. See MockDataType.
*  
*  If not specified, the type will be detected from the data.
* @type {MockDataType}
* @default null
*/
mockDataType?: MockDataType;





}
/**
* Creates a new MockDataSource
* 
* @param typeCheckedProperties {Partial<MockDataSource>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MockDataSource} The new instance
*/
class MockDataSource{
static create(typeCheckedProperties? : Partial<MockDataSource>, uncheckedProperties?: Object): MockDataSource;







}

/**
* MockupElements are produced by the balsamiqImport as
*  placeholders for Balsamiq controls that cannot be meaningfully translated to SmartClient
*  controls (such as the big red X markup control).
*  
*  MockupElement is just an instance of Img that uses .png files stored in the
*  tools/mockups folder. 
*  
*  MockupElement is not intended to be included in any final applications.
*/
interface MockupElement extends Img {




}
/**
* Creates a new MockupElement
* 
* @param typeCheckedProperties {Partial<MockupElement>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MockupElement} The new instance
*/
class MockupElement{
static create(typeCheckedProperties? : Partial<MockupElement>, uncheckedProperties?: Object): MockupElement;







}

/**
* A simple subclass of Window whose default configuration is appropriate for
*  a standalone, modal window. This includes appropriate default settings for 
*  Window.isModal and Window.autoCenter, and sizing information, as well
*  as having the window be initially ModalWindow.visibility
*  
*  This class is used by some development tools to simplify the creation of a modal window.
*/
interface ModalWindow extends Window {
/**
* If true, this Window widget will automatically be centered on the page when shown.
*  If false, it will show up in the last position it was placed (either programmatically,
*  or by user interaction).
*  
*  Note: If an auto-centering Window is either programmatically moved or dragged 
*  by an end user, auto-centering behavior is automatically turned off. To manually center 
*  a Window, you can use Window.centerInPage. Auto-centering will also be disabled if
*  you pass an explicit Canvas.left or Canvas.top value at Class.create.
* @type {boolean}
* @default true
*/
autoCenter?: boolean;

/**
* If true, when shown this Window will intercept and block events to all other existing
*  components on the page.
*  
*  Use Window.showModalMask to darken all other elements on the screen when a modal dialog
*  is showing.
*  
*  Chained modal windows - that is, modal windows that launch other modal windows - are
*  allowed. You can accomplish this by simply creating a second modal Window while a modal
*  Window is showing.
*  
*  Note only top-level Windows (Windows without parents) can be modal.
* @type {boolean}
* @default true
*/
isModal?: boolean;

/**
* Controls widget visibility when the widget is initialized. See Visibility type for
*  details.
*  
*  Specifying "visible" sets the CSS visiblity to "visible", forcing a child to be visible
*  even if the parent is hidden. Not supported for use with SmartClient layouts,
*  scrolling or auto-sizing but may be useful when working with third-party or legacy
*  DOM layout systems.
*  
*  Note that if Canvas.hideUsingDisplayNone is set for a hidden ancestor, setting
*  visibility will have no effect at all until that ancestor becomes visible.
* @type {Visibility}
* @default "hidden"
*/
visibility?: Visibility;

/**
* The canvas.width attribute specifies the size for a component's 
*  horizontal dimension; canvas.height specifies the size for the vertical 
*  dimension.
*  
*  May be set to an integer value (a number of pixels), a percentage value like "50%", or "*". 
*  
*  See percentSizing for details on how percentage or "*" values
*  are resolved actual size.
*  
*  If Canvas.overflow is set to "visible", the specified size acts as a
*  minimum, and the component may overflow to show all content and/or children.
*  
*  Note that developers wishing to set a default width or height for a component class
*  should set Canvas.defaultWidth or Canvas.defaultHeight 
*  instead of specifying an explicit default width or height.
*  This is important for components added to a Layout as members - it allows
*  the Layout to determine whether the canvas has an explicitly specified size that must be
*  respected, or whether it can participate in its LayoutPolicy.
* @type {number | string}
* @default "75%"
*/
width?: number | string;

/**
* Maximum width available to this Canvas. 
*  
*  The maxWidth and ModalWindow.maxHeight settings apply to:
*  
*  
* - For a canvas being managed as a member of a Layout, the maximum size the
*    layout should apply to the canvas.
*  
* - For a canvas with a width or height specified as a percent value, a maximum
*    numeric pixel value to limit how large the canvas is sized.
*  
* - determining size for a Canvas in a CanvasItem (maxHeight only)
*  
* - end user Window.canDragResize
*  
*  Maximum sizes do not apply in various other circumstances where sizes are being
*  determined, such as ListGrid.showRecordComponents.
* @type {number}
* @default "900"
*/
maxWidth?: number;

/**
* The canvas.width attribute specifies the size for a component's 
*  horizontal dimension; canvas.height specifies the size for the vertical 
*  dimension.
*  
*  May be set to an integer value (a number of pixels), a percentage value like "50%", or "*". 
*  
*  See percentSizing for details on how percentage or "*" values
*  are resolved actual size.
*  
*  If Canvas.overflow is set to "visible", the specified size acts as a
*  minimum, and the component may overflow to show all content and/or children.
*  
*  Note that developers wishing to set a default width or height for a component class
*  should set Canvas.defaultWidth or Canvas.defaultHeight 
*  instead of specifying an explicit default width or height.
*  This is important for components added to a Layout as members - it allows
*  the Layout to determine whether the canvas has an explicitly specified size that must be
*  respected, or whether it can participate in its LayoutPolicy.
* @type {number | string}
* @default "65%"
*/
height?: number | string;

/**
* Maximum height available to this Canvas. See ModalWindow.maxWidth for details of behavior.
* @type {number}
* @default "750"
*/
maxHeight?: number;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "WindowEditProxy"
*/
editProxyConstructor?: SCClassName;





}
/**
* Creates a new ModalWindow
* 
* @param typeCheckedProperties {Partial<ModalWindow>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ModalWindow} The new instance
*/
class ModalWindow{
static create(typeCheckedProperties? : Partial<ModalWindow>, uncheckedProperties?: Object): ModalWindow;







}

/**
* A MultiComboBoxItem is a combo box that allows the user to select multiple options. Each
*  selected option is represented as a button that can be clicked to deselect the option.
* 
*  The relative layout of the buttons to the combo box is specified with the
*  MultiComboBoxItem.layoutStyle attribute. The buttons will be kept in the order that
*  they were added, with the most recently added button being adjacent to the combo box.
* 
*  MultiComboBoxItem uses the AutoChild pattern to construct the
*  MultiComboBoxItem.comboBox and the MultiComboBoxItem.button
*  so that they can be easily customized. For example, you can customize the criteria used to
*  fetch by
*  overriding ComboBoxItem.getPickListFilterCriteria via
*  MultiComboBoxItem.comboBox.
*  
*/
interface MultiComboBoxItem extends CanvasItem {
/**
* Should this item's value be saved in the form's values and hence returned from
*  DynamicForm.getValues?
*  
*  shouldSaveValue:false is used to mark formItems which do not correspond to
*  the underlying data model and should not save a value into the form's
*  DynamicForm.values. Example includes visual separators, password re-type fields,
*  or checkboxes used to show/hide other form items.
*  
*  A shouldSaveValue:false item should be given a value either via
*  FormItem.defaultValue or by calling
*  DynamicForm.setValue or 
*  FormItem.setValue. Providing a value via
*  DynamicForm.values or DynamicForm.setValues 
*  will automatically switch the item to shouldSaveValue:true.
*  
*  Note that 
*  
* - if an item is shouldSaveValue true, but has no name, a warning is logged, and 
*    shouldSaveValue will be set to false.
*  
* @type {boolean}
* @default true
*/
shouldSaveValue?: boolean;

/**
* The Class to use when creating a picker of PickList.dataSetType for 
*  a FormItem. Must be a subclass of the builtin default, 
*  PickListMenu.
* @type {SCClassName}
* @default "PickListMenu"
*/
pickListConstructor?: SCClassName;

/**
* The Class to use when creating a picker of PickList.dataSetType for 
*  a FormItem. Must be a subclass of the builtin default, 
*  PickTreeMenu.
* @type {SCClassName}
* @default "PickTreeMenu"
*/
pickTreeConstructor?: SCClassName;

/**
* Whether to show the picker as a flat list, or a collapsible tree.
*  
*  The default value, "list", will use an instance of the
*  PickList.pickListConstructor as the picker - "tree" will 
*  show an instance of PickList.pickTreeConstructor.
* @type {string}
* @default "list"
*/
dataSetType?: string;

/**
* When this item is showing a PickList.dataSetType, this is 
*  the SelectItem.valueField of the record to use as the 
*  Tree.rootValue node.
* @type {string | number}
* @default null
*/
rootNodeId?: string | number;

/**
* When this item is showing a PickList.dataSetType, which 
*  nodes should be opened automatically. Options are:
*  
*  
* - "none" - no nodes are opened automatically
*  
* - "root" - opens the PickList.rootNodeId - in databound 
*        tree-pickers, this node is always hidden
*  
* - "all" - when ResultTree.loadDataOnDemand, opens the
*        PickList.rootNodeId and all of it's direct
*        descendants - otherwise, opens all loaded nodes 
*  
* @type {string}
* @default "none"
*/
autoOpenTree?: string;

/**
* The optionDataSource of the combo box.
* @type {DataSource | string}
* @default null
*/
optionDataSource?: DataSource | string;

/**
* Similar to ComboBoxItem.addUnknownValues, controls whether additional values can be
*  added to the ComboBox or whether the user must choose from the available values in the picklist
*  only.
*  
*  If this setting is changed after the MultiComboBoxItem has been created, the current value of
*  the item is reset to null and all buttons for non-default values (values not in the FormItem.defaultValue
*  array) are removed.
* @type {boolean}
* @default false
*/
addUnknownValues?: boolean;

/**
* If true, hitting tab always exits the field, and will also add a value to the list of
*  selected values if there is match (and depending on the setting for
*  MultiComboBoxItem.addUnknownValues).
*  
*  If false, if the user has typed in a value and hits tab, focus remains in the field.
*  If there is a match or if MultiComboBoxItem.addUnknownValues is true, a value will be added.
*  Otherwise, the input cursor remains at the end of the entered value.
* @type {boolean}
* @default true
*/
alwaysExitOnTab?: boolean;

/**
* Should the MultiComboBoxItem fetch data from the MultiComboBoxItem.optionDataSource
*  immediately or wait until the user first opens the pickList.
* @type {boolean}
* @default false
*/
autoFetchData?: boolean;

/**
* The valueMap of the combo box.
* @type {Array<any> | object}
* @default null
*/
valueMap?: Array<any> | object;

/**
* An AutoChild attribute to create the combo box in a MultiComboBoxItem.
* @type {ComboBoxItem}
* @default null
*/
comboBox?: ComboBoxItem;

/**
* The DynamicForm holding the MultiComboBoxItem.comboBox.
* @type {DynamicForm}
* @default null
*/
comboForm?: DynamicForm;

/**
* If this item has a specified optionDataSource, this attribute may be set
*  to specify an explicit DSRequest.operationId when performing a fetch against the
*  option dataSource to pick up display value mapping.
* @type {string}
* @default null
*/
optionOperationId?: string;

/**
* The displayField of the combo box.
* @type {string}
* @default null
*/
displayField?: string;

/**
* The valueField of the combo box.
* @type {string}
* @default null
*/
valueField?: string;

/**
* The layout used to arrange the MultiComboBoxItem.comboForm and the buttons
*  representing the values of the MultiComboBoxItem. Note that the constructor cannot be changed
*  (setting a valueLayoutConstructor has no effect) because the exact layout class used depends
*  on the current MultiComboBoxItem.layoutStyle.
* @type {Layout}
* @default null
*/
valueLayout?: Layout;

/**
* An AutoChild attribute used to create the buttons in the MultiComboBoxItem.
* @type {IButton}
* @default null
*/
button?: IButton;

/**
* When FormItem.showPending is true, the Button.baseStyle
*  used on MultiComboBoxItem.button that are in the "Pending" visual state.
*  
*  If unset, then the baseStyle of pending buttons is not changed.
* @type {CSSStyleName}
* @default "buttonPending"
*/
pendingButtonStyle?: CSSStyleName;

/**
* When FormItem.showDeletions is true, the Button.baseStyle
*  used on MultiComboBoxItem.button for values that have been deleted (also called "deselected
*  buttons").
*  
*  If unset, then the baseStyle of deselected buttons is not changed.
*  
*  NOTE: Deselected buttons are also disabled, so styling should be provided
*  for the deselectedButtonStyle&nbsp;+&nbsp;"Disabled" style name.
* @type {CSSStyleName}
* @default "buttonDeselected"
*/
deselectedButtonStyle?: CSSStyleName;

/**
* Specifies the layout style of the combo box and the buttons in the MultiComboBoxItem.
*  Available values are "flow" (the default), "horizontal", "horizontalReverse",
*  "vertical", and "verticalReverse".
* 
*  
*  
* - "flow":&nbsp; The buttons appear to the left of the combo box. When there is no
*  more room, the combo box and/or buttons flow onto a new line. The buttons autoFit
*  by default.
*  
* - "horizontal":&nbsp; The combo box appears on right and buttons are horizontally
*  stacked directly left of it. The buttons must autofit.
*  
* - "horizontalReverse":&nbsp; Like "horizontal" but the combo box appears on the left.
*  The buttons must autofit.
*  
* - "vertical":&nbsp; The combo box appears on top and buttons are stacked beneath it.
*  Buttons do not autofit by default.
*  
* - "verticalReverse":&nbsp; Like "vertical" but the combo box appears at bottom.
*  The buttons do not autofit by default.
*  
* @type {MultiComboBoxLayoutStyle}
* @default "flow"
*/
layoutStyle?: MultiComboBoxLayoutStyle;

/**
* Specifies whether to autofit the buttons in the MultiComboBoxItem. The
*  default value is true if MultiComboBoxItem.layoutStyle is "flow", but false for a
*  layoutStyle of "vertical" or "verticalReverse". If the layoutStyle
*  is "horizontal" or "horizontalReverse" then the buttons will autofit regardless
*  of the setting of this property.
* @type {boolean}
* @default null
*/
autoFitButtons?: boolean;

/**
* Specifies the size of the combo box field.
*  
*  Note that this attribute only has an effect in "flow", "horizontal", and
*  "horizontalReverse" MultiComboBoxItem.layoutStyle. In the other modes, the combo box
*  is as wide as the overall MultiComboBoxItem.
* @type {number}
* @default "130"
*/
comboBoxWidth?: number;

/**
* Specifies whether to arrange the buttons of the MultiComboBoxItem in the order that they were
*  selected (the default), or to sort the buttons by MultiComboBoxItem.displayField.
* @type {boolean}
* @default true
*/
useInsertionOrder?: boolean;



/**
*  This method will be called whenever this FormItem's value is being set via a programmatic
*  call to e.g: DynamicForm.setValues or FormItem.setValue and may be
*  overridden by CanvasItems intended to support displaying data values to update the
*  embedded Canvas to reflect the value passed in.
*  
*  The value of a MultiComboBoxItem to the form is an array of valueField values corresponding
*  to the selected combo box options.
* 
* @param {any} new display value for the item. This is the value after applying  any custom formatter or valueMap
* @param {any} underlying data value for the item
* @param {DynamicForm} the dynamicForm in which this item is contained
* @param {CanvasItem} the live form item instance
*/
showValue(displayValue:any, dataValue:any, form:DynamicForm, item:CanvasItem): void;

/**
* @param {MultiComboBoxLayoutStyle} the new layout style
*/
setLayoutStyle(layoutStyle:MultiComboBoxLayoutStyle): void;

/**
*  Sets the MultiComboBoxItem.autoFitButtons property.
* 
* @param {boolean} whether to autofit the buttons
*/
setAutoFitButtons(autoFitButtons:boolean): void;

/**
*  Notification method called when FormItem.showPending is enabled and this
*  MultiComboBoxItem should either clear or show its pending visual state.
*  
*  The default behavior is that the FormItem.titleStyle and
*  FormItem.cellStyle are updated to include/exclude the "Pending" suffix.
*  In addition, when displayed in the pending state and a
*  MultiComboBoxItem.pendingButtonStyle is set, then:
*  
*  
* - If MultiComboBoxItem.useInsertionOrder is false, buttons for
*    any new values will have their Button.baseStyle set to
*    pendingButtonStyle; otherwise
*  
* - (useInsertionOrder is true) buttons for values will have their
*    Button.baseStyle set to pendingButtonStyle if either the
*    value is new or it is in a different place within the value array.
*  
*  Returning false will cancel this default behavior.
* 
* @param {DynamicForm} the managing DynamicForm instance.
* @param {FormItem} the form item itself (also available as "this").
* @param {boolean} true if the item should show its pending visual state; false otherwise.
* @param {any} the current form item value.
* @param {any} the value that would be restored by a call to DynamicForm.resetValues.
*/
pendingStatusChanged(form:DynamicForm, item:FormItem, pendingStatus:boolean, newValue:any, value:any): boolean;



}
/**
* Creates a new MultiComboBoxItem
* 
* @param typeCheckedProperties {Partial<MultiComboBoxItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MultiComboBoxItem} The new instance
*/
class MultiComboBoxItem{
static create(typeCheckedProperties? : Partial<MultiComboBoxItem>, uncheckedProperties?: Object): MultiComboBoxItem;
/**
* The default hint string.
* @type {string}
* @default "Enter values"
*/
static defaultHint?: string;








}

/**
* Chooses a next element in a Process by evaluating a series of criteria against the
*  Process.state and choosing the element associated with the criteria that matched, or
*  a MultiDecisionTask.defaultElement if none of the criteria match.
*/
interface MultiDecisionTask extends ProcessElement {
/**
* List of DecisionBranch to be processed to find the first with matching
*  criteria. The specified DecisionBranch.targetTask is then used to identify the the next
*  element.
*  
*  If no criteria is matched the next element is MultiDecisionTask.defaultElement or the workflow
*  is finished.
*  
*  When providing a MultiDecisionTask in XML, the decisionList is expressed as:
*  
*    &lt;MultiDecisionTask ID="continentDecision" description="Which continent?" defaultElement="summary"&gt;
*      &lt;decisionList&gt;
*        &lt;decisionBranch targetTask="europeVATTask"&gt;
*          &lt;criteria fieldName="order.continent" operator="equals" value="Europe" /&gt;
*        &lt;/decisionBranch&gt;
*        ...
*      &lt;/decisionList&gt;
*    &lt;MultiDecisionTask&gt;
*  
* @type {Array<Partial<DecisionBranch>>}
* @default null
*/
decisionList?: Array<Partial<DecisionBranch>>;

/**
* Next element to pick if no criteria match. If this decision is part of a
*  Process.sequences and has a next element in the sequence, the
*  defaultElement is assumed to be the next element and does not need to be
*  specified.
* @type {string}
* @default null
*/
defaultElement?: string;

/**
* Not applicable to a MultiDecisionTask.
* @type {string}
* @default null
*/
nextElement?: string;





}
/**
* Creates a new MultiDecisionTask
* 
* @param typeCheckedProperties {Partial<MultiDecisionTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MultiDecisionTask} The new instance
*/
class MultiDecisionTask{
static create(typeCheckedProperties? : Partial<MultiDecisionTask>, uncheckedProperties?: Object): MultiDecisionTask;







}

/**
* The MultiFileItem provides an interface for a user to save one or more files that are
*  related to a DataSource record, where each file is represented by a record in a
*  related DataSource.
*  
*  Use MultiFileItem when a record may have one or more files associated with it (such as
*  attachments to an email message) where no information about the files needs to be stored other
*  than the files themselves. If you have several fields associated with each file (such as an
*  uploaded document with a version, comments and processes associated with it), consider
*  instead an ordinary DataSource with on field of type "binary", and using the FileItem
*  for upload.
*  
*  See the upload overview for more information on upload.
*  
*  DataSource Setup
*  
*  In a relationship sometimes called a "master-detail" relationship, the MultiFileItem stores
*  files in a "detail" DataSource which are related to a "master" DataSource record being
*  edited by the form which contains the MultiFileItem.
*  
*  To use a MultiFileItem:
*  
*  
* - declare a "detail" DataSource to store the related files. At a minimum, this
*  DataSource must have:
*  
*  
* - a DataSourceField.primaryKey field
*  
* - a field declaring a DataSourceField.foreignKey relationship to the
*  primaryKey of the "master" DataSource
*  
* - a field of type "binary"
*  
*  
* - DataBoundComponent.dataSource a DynamicForm to the "master" DataSource 
*  
* - in the DynamicForm bound to the "master" DataSource, declare a field with
*  FormItem.editorType:"MultiFileItem" and a dataSource
*  property set to the ID of the "detail" DataSource
*  
*  An example "detail" DataSource for storing files is shown below. This "detail" DataSource
*  assumes a "master" DataSource with DataSource.ID "masterRecord" and with a primaryKey
*  field "id".
*  
*  
*   &lt;DataSource ID="uploadedFiles" serverType="sql"&gt;
*    &lt;fields&gt;
*     &lt;field name="fileId" type="sequence" primaryKey="true" hidden="true"/&gt;
*     &lt;field name="masterRecordId" type="number" foreignKey="masterRecord.id" hidden="true"/&gt;
*     &lt;field name="file" type="binary" title="File"/&gt;
*    &lt;/fields&gt;
*   &lt;/DataSource&gt;
*  
*  
*  
*  Aside from a single "binary" field, the "detail" DataSource should generally have only
*  hidden fields, as shown above. Additional internal fields (such as a "lastUpdated" field)
*  may be added, but will not be editable via MultiFileItem. 
*  
*  Display
*  
*  The MultiFileItem appears as a list of files related to the current record. An optional
*  button, the MultiFileItem.removeButton allows removing files. A
*  second optional button, the MultiFileItem.editButton, launches a
*  picker for uploading further files.
*  
*  Saving
*  
*  In all cases, uploading a new file is an "add" DSRequest against the
*  MultiFileItem.dataSource.
*  
*  The MultiFileItem has two modes, according to whether the "master" record is being newly created
*  via an "add" operation or whether the master record is pre-existing ("update" operation).
*  
*  If the master record is pre-existing, each file added by the user is uploaded as soon as the
*  user exits the picker launched from the edit button, and the list of files shown in the main
*  form reflects the actual list of stored files. 
*  
*  If the master record is being newly created, files are not actually uploaded until
*  after the master record is confirmed saved, and the list of fields shown in the main
*  form reflects files which will be uploaded after the master record is saved.
*  
*  In both cases, if there are multiple files to upload, they are uploaded one at a time, as a
*  series of separate "add" DSRequests against the MultiFileItem.dataSource.
*  
*  Also in both cases, deletion of any file is immediate. In the case of a pre-existing master
*  record, all files shown actually exist as DataSource records, and deletion is performed as a
*  "remove" DSRequest against the MultiFileItem.dataSource.
*/
interface MultiFileItem extends CanvasItem {
/**
* DataSource where files are stored. 
*  
*  This DataSource is expected to have a field of type "binary" as well as a primaryKey and
*  foreignKey declaration to some other DataSource; see the MultiFileItem for an
*  overview.
*  
*  This DataSource need only be capable of "fetch", "add" and "remove" - "update" is unused.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* Button for removing files. Supports the properties of a FormItemIcon.
* @type {StatefulCanvas}
* @default null
*/
removeButton?: StatefulCanvas;

/**
* Button for launching a picker to add new files for upload. Supports the properties of a
*  FormItemIcon.
* @type {StatefulCanvas}
* @default null
*/
editButton?: StatefulCanvas;

/**
* MultiFileItems use a MultiFilePicker instance as their picker.
*  The generated picker autoChild may be customized via the standard 
*  AutoChild pattern.
* @type {SCClassName}
* @default "MultiFilePicker"
*/
pickerConstructor?: SCClassName;

/**
* Empty message to display when there are no files listed.
* @type {string}
* @default "Click icon to add..."
*/
emptyMessage?: string;

/**
* The FormItemIcon.prompt of the MultiFileItem.editButton.
* @type {string}
* @default "Add files"
*/
editButtonPrompt?: string;

/**
* The FormItemIcon.prompt of the MultiFileItem.removeButton.
* @type {string}
* @default "Remove selected files"
*/
removeButtonPrompt?: string;

/**
* The initial title of the upload button in the picker lauched by the
*  MultiFileItem.editButton that is used before the form is saved.
* @type {string}
* @default "OK"
*/
pickerUploadButtonInitialTitle?: string;

/**
* The title of the upload button in the picker lauched by the
*  MultiFileItem.editButton that is used after the form is saved.
* @type {string}
* @default "Save"
*/
pickerUploadButtonTitle?: string;

/**
* The title of the cancel button in the picker lauched by the
*  MultiFileItem.editButton.
* @type {string}
* @default "Cancel"
*/
pickerCancelButtonTitle?: string;

/**
* The contents of the "Add another" file button in the picker launched by the
*  MultiFileItem.editButton.
* @type {HTMLString}
* @default "&lt;u&gt;Add&amp;nbsp;another&lt;/u&gt;"
*/
pickerAddAnotherFileButtonTitle?: HTMLString;

/**
* Specifies the label of the progress meter in the picker lauched by the
*  MultiFileItem.editButton. This property is a dynamic string, similar to the
*  Canvas.dynamicContents feature, with the variables fileName and
*  formattedFileSize.
* @type {HTMLString}
* @default "Saving ${fileName} ${formattedFileSize}"
*/
pickerUploadProgressLabel?: HTMLString;





}
/**
* Creates a new MultiFileItem
* 
* @param typeCheckedProperties {Partial<MultiFileItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MultiFileItem} The new instance
*/
class MultiFileItem{
static create(typeCheckedProperties? : Partial<MultiFileItem>, uncheckedProperties?: Object): MultiFileItem;







}

/**
* The MultiFilePicker is a pop-up picker used by the MultiFileItem to allow the user to
*  enter several files for upload.
*/
interface MultiFilePicker extends VStack {
/**
* If set true, the picker will be displayed inside a Window.
* @type {boolean}
* @default false
*/
showInWindow?: boolean;

/**
* Minimum number of upload fields to show. This many fields will show up initially.
* @type {Integer}
* @default "1"
*/
minUploadFields?: Integer;

/**
* The maximum number of upload fields to show. If not specified, user can add as many
*  upload fields as he wishes.
* @type {Integer}
* @default null
*/
maxUploadFields?: Integer;

/**
* Title for the Window implemented as a container for this picker in some skins.
* @type {string}
* @default "Add files"
*/
title?: string;





}
/**
* Creates a new MultiFilePicker
* 
* @param typeCheckedProperties {Partial<MultiFilePicker>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MultiFilePicker} The new instance
*/
class MultiFilePicker{
static create(typeCheckedProperties? : Partial<MultiFilePicker>, uncheckedProperties?: Object): MultiFilePicker;







}

/**
* A dialog that allows the user to set up complex grouping arrangements by defining a group of
*  GroupSpecifiers.
*  
*  Each GroupSpecifier applies to a single property and grouping - so, for instance, in 
*  a grid with two columns, Nationhood and Country, you could group first 
*  by Nationhood with its selected groupingMode and then by Country with its selected groupingMode.
*  
*  Important Note: this class should not be used directly - it is exposed purely for
*  i18nMessages
*/
interface MultiGroupDialog extends Window {
/**
* The title-text to appear in this Dialog's Header-bar.
* @type {string}
* @default "Group"
*/
title?: string;

/**
* The title-text to appear on the addLevelButton. 
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiGroupPanel contained in this dialog. You only need to
*  consider the properties on the MultiGroupPanel for i18n.
* @type {string}
* @default "Add Level"
*/
addLevelButtonTitle?: string;

/**
* The title-text to appear on the deleteLevelButton
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiGroupPanel contained in this dialog. You only need to
*  consider the properties on the MultiGroupPanel for i18n.
* @type {string}
* @default "Delete Level"
*/
deleteLevelButtonTitle?: string;

/**
* The title-text to appear on the copyLevelButton
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiGroupPanel contained in this dialog. You only need to
*  consider the properties on the MultiGroupPanel for i18n.
* @type {string}
* @default "Copy Level"
*/
copyLevelButtonTitle?: string;

/**
* The hover-prompt for the Level Up button.
* @type {string}
* @default "Move Level Up"
*/
levelUpButtonTitle?: string;

/**
* The hover-prompt for the Level Down button.
* @type {string}
* @default "Move Level Down"
*/
levelDownButtonTitle?: string;

/**
* This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed.
*  
*  Default value returns 
*  
*  Columns may only be used once: [some field's title] is used multiple times
*  
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiGroupPanel contained in this dialog. You only need to
*  consider the properties on the MultiGroupPanel for i18n.
* @type {HTMLString}
* @default "Columns may only be used once: '${title}' is used multiple times."
*/
invalidListPrompt?: HTMLString;

/**
* The title-text to appear in the header of the "property" field.
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiGroupPanel contained in this dialog. You only need to
*  consider the properties on the MultiGroupPanel for i18n.
* @type {string}
* @default "Column"
*/
propertyFieldTitle?: string;

/**
* The title-text to appear in the header of the "property" field.
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiGroupPanel contained in this dialog. You only need to
*  consider the properties on the MultiGroupPanel for i18n.
* @type {string}
* @default "Grouping"
*/
groupingFieldTitle?: string;

/**
* The initial group configuration to show in the 
*  MultiGroupPanel.optionsGrid.
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiGroupPanel contained in this dialog.
* @type {Array<any>}
* @default null
*/
initialGrouping?: Array<any>;

/**
* The maximum number of levels of grouping that can be applied. Since each group-property or
*  field-name can be used only once in a given multi-group operation, if no maxLevels value
*  or a value larger than the total number of available properties is specified, it will 
*  default to the total number of available properties.
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiGroupPanel contained in this dialog.
* @type {number}
* @default null
*/
maxLevels?: number;

/**
* The title-text to appear on the applyButton
* @type {string}
* @default "Apply"
*/
applyButtonTitle?: string;

/**
* The title-text to appear on the cancelButton
* @type {string}
* @default "Cancel"
*/
cancelButtonTitle?: string;

/**
* Automatically generated IButton providing the mechanism for accepting
*  the current group configuration. Fires the passed callback with a single parameter,
*  groupLevels, representing the current group configuration as an array of 
*  GroupSpecifiers.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.applyButtonProperties and 
*  multiGroupPanel.applyButtonDefaults.
* @type {IButton}
* @default null
*/
applyButton?: IButton;

/**
* Automatically generated IButton providing the mechanism for closing this
*  Dialog without accepting the current group configuration. The passed callback is fired 
*  with a single null parameter, indicating that the operation was cancelled.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.cancelButtonProperties and 
*  multiGroupPanel.cancelButtonDefaults.
* @type {IButton}
* @default null
*/
cancelButton?: IButton;

/**
* Automatically generated IButton providing a mechanism for adding new levels
*  to the group configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.addLevelButtonProperties and 
*  multiGroupPanel.addLevelButtonDefaults.
* @type {IButton}
* @default null
*/
addLevelButton?: IButton;

/**
* Automatically generated IButton providing a mechanism for deleting levels
*  from the group configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.deleteLevelButtonProperties and 
*  multiGroupPanel.deleteLevelButtonDefaults.
* @type {IButton}
* @default null
*/
deleteLevelButton?: IButton;

/**
* Automatically generated IButton providing a mechanism for duplicating levels
*  in the group configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.copyLevelButtonProperties and 
*  multiGroupPanel.copyLevelButtonDefaults.
* @type {IButton}
* @default null
*/
copyLevelButton?: IButton;

/**
* Automatically generated ImgButton providing a mechanism for moving existing
*  group-levels up in the group configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.levelUpButtonProperties and 
*  multiGroupPanel.levelUpButtonDefaults.
* @type {ImgButton}
* @default null
*/
levelUpButton?: ImgButton;

/**
* Automatically generated ImgButton providing a mechanism for moving existing
*  group-levels down in the group configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.levelDownButtonProperties and 
*  multiGroupPanel.levelDownButtonDefaults.
* @type {ImgButton}
* @default null
*/
levelDownButton?: ImgButton;

/**
* The list of fields which the user can choose to group by.
* @type {Array<Partial<DataSourceField>>}
* @default null
*/
fields?: Array<Partial<DataSourceField>>;

/**
* Automatically generated ListGrid allowing the user to configure a set of 
*  GroupSpecifiers.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.optionsGridProperties and 
*  multiGroupPanel.optionsGridDefaults.
* @type {ListGrid}
* @default null
*/
optionsGrid?: ListGrid;



/**
*  Return the number of levels of grouping that have been configured.
* 
* 
*/
getNumLevels(): number;

/**
*  Returns all configured grouping levels, as an array of GroupSpecifiers.
* 
* 
*/
getGroup(): Array<Partial<GroupSpecifier>>;

/**
*  Validate that no two GroupSpecifiers group on the same 
*  GroupSpecifier.property.
* 
* 
*/
validate(): boolean;



}
/**
* Creates a new MultiGroupDialog
* 
* @param typeCheckedProperties {Partial<MultiGroupDialog>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MultiGroupDialog} The new instance
*/
class MultiGroupDialog{
static create(typeCheckedProperties? : Partial<MultiGroupDialog>, uncheckedProperties?: Object): MultiGroupDialog;




/**
*  Launches a MultiGroupDialog and obtains a group-definition from the user.
* 
* 
* @param {Array<Partial<string>> | DataSource | DataBoundComponent} A source for Fields                which the user can choose to group by
* @param {Array<any>} The initial group definition.
* @param {Callback} Called when the user defines and accepts one or more GroupSpecifiers. Single parameter groupLevels is an Array of  GroupSpecifier or null if the user cancelled the dialog.
* @param {Partial<MultiGroupDialog>=} Configuration to apply to the generated dialog
*/
static askForGrouping(fieldSource:Array<Partial<string>> | DataSource | DataBoundComponent, initialGrouping:Array<any>, callback:Callback, properties?:Partial<MultiGroupDialog>): void;




}

/**
* A widget that allows the user to set up complex grouping arrangements by defining a group of
*  GroupSpecifiers.
*  
*  Each GroupSpecifier applies to a single property and grouping - so, for instance, in 
*  a grid with two columns, Nationhood and Country, you could group 
*  first by Nationhood with its selected groupingMode and then by 
*  Country with its selected groupingMode.
*  
*  Important Note: this class should not be used directly - it is exposed purely for
*  i18nMessages
*/
interface MultiGroupPanel extends Layout {
/**
* The list of fields which the user can choose to group by.
* @type {Array<Partial<DataSourceField>>}
* @default null
*/
fields?: Array<Partial<DataSourceField>>;

/**
* The title-text to appear on the addLevelButton
* @type {string}
* @default "Add Level"
*/
addLevelButtonTitle?: string;

/**
* The title-text to appear on the deleteLevelButton
* @type {string}
* @default "Delete Level"
*/
deleteLevelButtonTitle?: string;

/**
* The title-text to appear on the copyLevelButton
* @type {string}
* @default "Copy Level"
*/
copyLevelButtonTitle?: string;

/**
* This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed.
*  
*  Default value returns 
*  
*  Columns may only be used once: [some field's title] is used multiple times
*  
* @type {HTMLString}
* @default "Columns may only be used once: '${title}' is used multiple times."
*/
invalidListPrompt?: HTMLString;

/**
* The title-text to appear in the header of the "property" field.
* @type {string}
* @default "Column"
*/
propertyFieldTitle?: string;

/**
* The title-text to appear in the header of the "grouping" field.
* @type {string}
* @default "Grouping"
*/
groupingFieldTitle?: string;

/**
* The title-text to appear in the first column for the first group-level.
* @type {string}
* @default "Group by"
*/
firstGroupLevelTitle?: string;

/**
* The title-text to appear in the first column for all group-levels other than the first.
* @type {string}
* @default "Then by"
*/
otherGroupLevelTitle?: string;

/**
* Automatically generated IButton providing a mechanism for adding new levels
*  to the group configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.addLevelButtonProperties and 
*  multiGroupPanel.addLevelButtonDefaults.
* @type {IButton}
* @default null
*/
addLevelButton?: IButton;

/**
* Automatically generated IButton providing a mechanism for deleting levels
*  from the group configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.deleteLevelButtonProperties and 
*  multiGroupPanel.deleteLevelButtonDefaults.
* @type {IButton}
* @default null
*/
deleteLevelButton?: IButton;

/**
* Automatically generated IButton providing a mechanism for duplicating levels
*  in the group configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.copyLevelButtonProperties and 
*  multiGroupPanel.copyLevelButtonDefaults.
* @type {IButton}
* @default null
*/
copyLevelButton?: IButton;

/**
* The hover-prompt for the Level Up button.
* @type {string}
* @default "Move Level Up"
*/
levelUpButtonTitle?: string;

/**
* Automatically generated ImgButton providing a mechanism for moving existing
*  group-levels up in the group configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.levelUpButtonProperties and 
*  multiGroupPanel.levelUpButtonDefaults.
* @type {ImgButton}
* @default null
*/
levelUpButton?: ImgButton;

/**
* The hover-prompt for the Level Down button.
* @type {string}
* @default "Move Level Down"
*/
levelDownButtonTitle?: string;

/**
* Automatically generated ImgButton providing a mechanism for moving existing
*  group-levels down in the group configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.levelDownButtonProperties and 
*  multiGroupPanel.levelDownButtonDefaults.
* @type {ImgButton}
* @default null
*/
levelDownButton?: ImgButton;

/**
* Automatically generated ListGrid allowing the user to configure a set of 
*  GroupSpecifiers.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiGroupPanel.optionsGridProperties and 
*  multiGroupPanel.optionsGridDefaults.
* @type {ListGrid}
* @default null
*/
optionsGrid?: ListGrid;

/**
* The initial group configuration to show in the 
*  MultiGroupPanel.optionsGrid.
* @type {Array<any>}
* @default null
*/
initialGrouping?: Array<any>;

/**
* The maximum number of levels of grouping that can be applied. Since each group-property or
*  field-name can be used only once in a given multi-group operation, if no maxLevels value
*  or a value larger than the total number of available properties is specified, it will 
*  default to the total number of available properties.
* @type {number}
* @default null
*/
maxLevels?: number;



/**
*  Return the number of levels of grouping that have been configured.
* 
* 
*/
getNumLevels(): number;

/**
*  Returns all configured grouping levels, as an array of GroupSpecifiers.
* 
* 
*/
getGroup(): Array<Partial<GroupSpecifier>>;

/**
*  Validate that no two GroupSpecifiers group on the same 
*  GroupSpecifier.property.
* 
* 
*/
validate(): boolean;

/**
*  Fired whenever the group configuration changes. The single parameter is an array of
*  GroupSpecifiers that represent the list of group-levels as they appear after 
*  whatever change has occurred.
* 
* 
* @param {Array<Partial<GroupSpecifier>>} The current group configuration, after any changes
*/
groupChanged(groupLevels:Array<Partial<GroupSpecifier>>): void;



}
/**
* Creates a new MultiGroupPanel
* 
* @param typeCheckedProperties {Partial<MultiGroupPanel>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MultiGroupPanel} The new instance
*/
class MultiGroupPanel{
static create(typeCheckedProperties? : Partial<MultiGroupPanel>, uncheckedProperties?: Object): MultiGroupPanel;







}

/**
* Maintains a 'selected' subset of a Tree.multiLinkTree. Multi-link
*  trees cannot use the Selection because they allow 
*  multiple occurences of the same node; therefore, selection caching must be done in a way 
*  that allows duplicate nodes to be unambiguously identified. Because the base 
*  Selection class caches pointers to the selected nodes or records directly, it 
*  is fundamentally unable to do this. MultiLinkSelection, by contrast, caches 
*  NodeLocator objects.
*  
*  Includes methods for programmatically selecting node occurences and checking which node
*  occurences are selected, and also for selecting node occurences as a result of mouse events,
*  including drag selection support.
*  The selection object is used automatically to handle selection APIs on TreeGrid
*  instances where the data model is multi-linked (see ResultTree.linkDataSource and 
*  Tree.linkData for further details).
*  
*  Note that selection and deselection are skipped for nodes that aren't enabled, or that are
*  marked as non-selectable; the relevant properties are ListGrid.recordEnabledProperty 
*  and ListGrid.recordCanSelectProperty. The recommended approach to affect disabled 
*  objects via the Selection APIs is to temporarily enable them beforehand.
*/
interface MultiLinkSelection extends Class {


/**
*  Select a particular node occurence. Note if you do not pass a NodeLocator, 
*  the recordNum parameter is required.
* 
* @param {TreeNode | NodeLocator} node to select, or a NodeLocator that                        identifies it
* @param {Integer=} Optional index into the underlying Tree's                         openList (which will be the same as the record                         number in a TreeGrid)
*/
select(node:TreeNode | NodeLocator, recordNum?:Integer): boolean;

/**
*  Deselect a particular node occurence. Note if you do not pass a NodeLocator, 
*  the recordNum parameter is required.
* 
* @param {TreeNode | NodeLocator} node to deselect, or a NodeLocator that                        identifies it
* @param {Integer=} Optional index into the underlying Tree's                         openList (which will be the same as the record                         number in a TreeGrid)
*/
deselect(node:TreeNode | NodeLocator, recordNum?:Integer): boolean;

/**
*  Select a single node occurence and deselect everything else. Note if you do not pass a 
*  NodeLocator, the recordNum parameter is required.
* 
* @param {TreeNode | NodeLocator} node to select, or a NodeLocator that                        identifies it
* @param {Integer=} Optional index into the underlying Tree's                         openList (which will be the same as the record                         number in a TreeGrid)
*/
selectSingle(node:TreeNode | NodeLocator, recordNum?:Integer): boolean;

/**
*  Returns the selected nodes in this grid as a list of NodeLocators.
* 
* 
*/
getSelection(): Array<Partial<NodeLocator>>;

/**
*  Returns the selected nodes in this grid as a direct array of records. Contrast this with 
*  MultiLinkSelection.getSelection, which returns a list of NodeLocators.
*  Note, because this is MultiLinkSelection, this method may return an array 
*  containing the same node multiple times, with no way of discerning which particular 
*  occurence(s) are selected. If you need an unambiguous list of selected node occurences, use
*  getSelection().
* 
* 
*/
getSelectedRecords(): Array<Partial<TreeNode>>;



}
/**
* Creates a new MultiLinkSelection
* 
* @param typeCheckedProperties {Partial<MultiLinkSelection>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MultiLinkSelection} The new instance
*/
class MultiLinkSelection{
static create(typeCheckedProperties? : Partial<MultiLinkSelection>, uncheckedProperties?: Object): MultiLinkSelection;







}

/**
* A dialog that allows the user to set up complex sorting arrangements by defining a group of
*  SortSpecifiers.
*  
*  Each SortSpecifier applies to a single property and direction - so, for instance, in 
*  a grid with two columns, year and monthNumber, you could sort first 
*  by year in descending order and then by monthNumber in ascending 
*  order. This would producing a grid sorted by year from largest (most 
*  recent) to smallest (least recent) and, within each year, by monthNumber from smallest 
*  (January) to largest (December).
*  
*  See MultiSortDialog.askForSort, DataBoundComponent.askForSort
*/
interface MultiSortDialog extends Window {
/**
* Automatically generated MultiSortPanel displayed within this
*  component.
* @type {MultiSortPanel}
* @default null
*/
multiSortPanel?: MultiSortPanel;

/**
* The title-text to appear in this Dialog's Header-bar.
* @type {string}
* @default "Sort"
*/
title?: string;

/**
* The title-text to appear on the addLevelButton. 
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiSortPanel contained in this dialog. You only need to
*  consider the properties on the MultiSortPanel for i18n.
* @type {string}
* @default "Add Level"
*/
addLevelButtonTitle?: string;

/**
* The title-text to appear on the deleteLevelButton
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiSortPanel contained in this dialog. You only need to
*  consider the properties on the MultiSortPanel for i18n.
* @type {string}
* @default "Delete Level"
*/
deleteLevelButtonTitle?: string;

/**
* The title-text to appear on the copyLevelButton
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiSortPanel contained in this dialog. You only need to
*  consider the properties on the MultiSortPanel for i18n.
* @type {string}
* @default "Copy Level"
*/
copyLevelButtonTitle?: string;

/**
* The hover-prompt for the Level Up button.
* @type {string}
* @default "Move Level Up"
*/
levelUpButtonTitle?: string;

/**
* The hover-prompt for the Level Down button.
* @type {string}
* @default "Move Level Down"
*/
levelDownButtonTitle?: string;

/**
* This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed.
*  
*  Default value returns 
*  
*  Columns may only be used once: [some field's title] is used multiple times
*  
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiSortPanel contained in this dialog. You only need to
*  consider the properties on the MultiSortPanel for i18n.
* @type {HTMLString}
* @default "Columns may only be used once: '${title}' is used multiple times."
*/
invalidListPrompt?: HTMLString;

/**
* The title-text to appear in the header of the "property" field.
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiSortPanel contained in this dialog. You only need to
*  consider the properties on the MultiSortPanel for i18n.
* @type {string}
* @default "Column"
*/
propertyFieldTitle?: string;

/**
* The title-text to appear in the header of the "direction" field.
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiSortPanel contained in this dialog. You only need to
*  consider the properties on the MultiSortPanel for i18n.
* @type {string}
* @default "Order"
*/
directionFieldTitle?: string;

/**
* The title-text to appear in the "direction" field's SelectItem for an "ascending" sort
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiSortPanel contained in this dialog. You only need to
*  consider the properties on the MultiSortPanel for i18n.
* @type {string}
* @default "Ascending"
*/
ascendingTitle?: string;

/**
* The title-text to appear in the "direction" field's SelectItem for a "descending" sort
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiSortPanel contained in this dialog. You only need to
*  consider the properties on the MultiSortPanel for i18n.
* @type {string}
* @default "Descending"
*/
descendingTitle?: string;

/**
* The title-text to appear in the first column for the first sort-level.
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiSortPanel contained in this dialog. You only need to
*  consider the properties on the MultiSortPanel for i18n.
* @type {string}
* @default "Sort by"
*/
firstSortLevelTitle?: string;

/**
* The title-text to appear in the first column for all sort-levels other than the first.
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiSortPanel contained in this dialog. You only need to
*  consider the properties on the MultiSortPanel for i18n.
* @type {string}
* @default "Then by"
*/
otherSortLevelTitle?: string;

/**
* The initial sort configuration to show in the 
*  MultiSortPanel.optionsGrid.
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiSortPanel contained in this dialog.
* @type {Array<Partial<SortSpecifier>>}
* @default null
*/
initialSort?: Array<Partial<SortSpecifier>>;

/**
* The maximum number of levels of sorting that can be applied. Since each sort-property or
*  field-name can be used only once in a given multi-sort operation, if no maxLevels value
*  or a value larger than the total number of available properties is specified, it will 
*  default to the total number of available properties.
*  
*  Note, this is a passthrough property which, when set, is passed through to the 
*  MultiSortPanel contained in this dialog.
* @type {number}
* @default null
*/
maxLevels?: number;

/**
* The title-text to appear on the applyButton
* @type {string}
* @default "Apply"
*/
applyButtonTitle?: string;

/**
* The title-text to appear on the cancelButton
* @type {string}
* @default "Cancel"
*/
cancelButtonTitle?: string;

/**
* Automatically generated IButton providing the mechanism for accepting
*  the current sort configuration. Fires the passed callback with a single parameter,
*  sortLevels, representing the current sort configuration as an array of 
*  SortSpecifiers.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortDialog.applyButtonProperties and 
*  multiSortDialog.applyButtonDefaults.
* @type {IButton}
* @default null
*/
applyButton?: IButton;

/**
* Automatically generated IButton providing the mechanism for closing this
*  Dialog without accepting the current sort configuration. The passed callback is fired 
*  with a single null parameter, indicating that the operation was cancelled.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortDialog.cancelButtonProperties and 
*  multiSortDialog.cancelButtonDefaults.
* @type {IButton}
* @default null
*/
cancelButton?: IButton;

/**
* Automatically generated IButton providing a mechanism for adding new levels
*  to the sort configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortPanel.addLevelButtonProperties and 
*  multiSortPanel.addLevelButtonDefaults.
* @type {IButton}
* @default null
*/
addLevelButton?: IButton;

/**
* Automatically generated IButton providing a mechanism for deleting levels
*  from the sort configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortPanel.deleteLevelButtonProperties and 
*  multiSortPanel.deleteLevelButtonDefaults.
* @type {IButton}
* @default null
*/
deleteLevelButton?: IButton;

/**
* Automatically generated IButton providing a mechanism for duplicating levels
*  in the sort configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortPanel.copyLevelButtonProperties and 
*  multiSortPanel.copyLevelButtonDefaults.
* @type {IButton}
* @default null
*/
copyLevelButton?: IButton;

/**
* Automatically generated ImgButton providing a mechanism for moving existing
*  sort-levels up in the sort configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortPanel.levelUpButtonProperties and 
*  multiSortPanel.levelUpButtonDefaults.
* @type {ImgButton}
* @default null
*/
levelUpButton?: ImgButton;

/**
* Automatically generated ImgButton providing a mechanism for moving existing
*  sort-levels down in the sort configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortPanel.levelDownButtonProperties and 
*  multiSortPanel.levelDownButtonDefaults.
* @type {ImgButton}
* @default null
*/
levelDownButton?: ImgButton;

/**
* The list of fields which the user can choose to sort by.
* @type {Array<Partial<DataSourceField>>}
* @default null
*/
fields?: Array<Partial<DataSourceField>>;

/**
* Automatically generated ListGrid allowing the user to configure a set of 
*  SortSpecifiers.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortPanel.optionsGridProperties and 
*  multiSortPanel.optionsGridDefaults.
* @type {ListGrid}
* @default null
*/
optionsGrid?: ListGrid;



/**
*  Return the number of levels of sorting that have been configured.
* 
* 
*/
getNumLevels(): number;

/**
*  Return a SortSpecifier object for the requested levelNum.
* 
* 
* @param {number} The index of the level to return a SortSpecifier for
*/
getSortLevel(levelNum:number): SortSpecifier;

/**
*  Returns all configured sorting levels, as an array of SortSpecifiers.
* 
* 
*/
getSort(): Array<Partial<SortSpecifier>>;

/**
*  Validate that no two SortSpecifiers sort on the same 
*  SortSpecifier.property.
* 
* 
*/
validate(): boolean;



}
/**
* Creates a new MultiSortDialog
* 
* @param typeCheckedProperties {Partial<MultiSortDialog>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MultiSortDialog} The new instance
*/
class MultiSortDialog{
static create(typeCheckedProperties? : Partial<MultiSortDialog>, uncheckedProperties?: Object): MultiSortDialog;




/**
*  Launches a MultiSortDialog and obtains a sort-definition from the user.
* 
* 
* @param {DataBoundComponent | DataSource | Array<Partial<DataSourceField>>} A source for Fields which the user can choose to sort by
* @param {Array<Partial<SortSpecifier>>} The initial sort definition.
* @param {Callback} Called when the user defines and accepts one or more SortSpecifiers. Single parameter sortLevels is an Array of  SortSpecifier or null if the user cancelled the dialog.
* @param {Partial<MultiSortDialog>=} Configuration to apply to the generated dialog
*/
static askForSort(fieldSource:DataBoundComponent | DataSource | Array<Partial<DataSourceField>>, initialSort:Array<Partial<SortSpecifier>>, callback:Callback, properties?:Partial<MultiSortDialog>): void;




}

/**
* A widget that allows the user to set up complex sorting arrangements by defining a group of
*  SortSpecifiers.
*  
*  Each SortSpecifier applies to a single property and direction - so, for instance, in 
*  a grid with two columns, year and monthNumber, you could sort first 
*  by year in descending order and then by monthNumber in ascending 
*  order. This would producing a grid sorted by year from largest (most 
*  recent) to smallest (least recent) and, within each year, by monthNumber from smallest 
*  (January) to largest (December).
*/
interface MultiSortPanel extends Layout {
/**
* The list of fields which the user can choose to sort by.
* @type {Array<Partial<DataSourceField>>}
* @default null
*/
fields?: Array<Partial<DataSourceField>>;

/**
* The title-text to appear on the addLevelButton
* @type {string}
* @default "Add Level"
*/
addLevelButtonTitle?: string;

/**
* The title-text to appear on the deleteLevelButton
* @type {string}
* @default "Delete Level"
*/
deleteLevelButtonTitle?: string;

/**
* The title-text to appear on the copyLevelButton
* @type {string}
* @default "Copy Level"
*/
copyLevelButtonTitle?: string;

/**
* This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed.
*  
*  Default value returns 
*  
*  Columns may only be used once: [some field's title] is used multiple times
*  
* @type {HTMLString}
* @default "Columns may only be used once: '${title}' is used multiple times."
*/
invalidListPrompt?: HTMLString;

/**
* The title-text to appear in the header of the "property" field.
* @type {string}
* @default "Column"
*/
propertyFieldTitle?: string;

/**
* The title-text to appear in the header of the "direction" field.
* @type {string}
* @default "Order"
*/
directionFieldTitle?: string;

/**
* The title-text to appear in the "direction" field's SelectItem for an "ascending" sort
* @type {string}
* @default "Ascending"
*/
ascendingTitle?: string;

/**
* The title-text to appear in the "direction" field's SelectItem for a "descending" sort
* @type {string}
* @default "Descending"
*/
descendingTitle?: string;

/**
* The title-text to appear in the first column for the first sort-level.
* @type {string}
* @default "Sort by"
*/
firstSortLevelTitle?: string;

/**
* The title-text to appear in the first column for all sort-levels other than the first.
* @type {string}
* @default "Then by"
*/
otherSortLevelTitle?: string;

/**
* Automatically generated IButton providing a mechanism for adding new levels
*  to the sort configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortPanel.addLevelButtonProperties and 
*  multiSortPanel.addLevelButtonDefaults.
* @type {IButton}
* @default null
*/
addLevelButton?: IButton;

/**
* Automatically generated IButton providing a mechanism for deleting levels
*  from the sort configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortPanel.deleteLevelButtonProperties and 
*  multiSortPanel.deleteLevelButtonDefaults.
* @type {IButton}
* @default null
*/
deleteLevelButton?: IButton;

/**
* Automatically generated IButton providing a mechanism for duplicating levels
*  in the sort configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortPanel.copyLevelButtonProperties and 
*  multiSortPanel.copyLevelButtonDefaults.
* @type {IButton}
* @default null
*/
copyLevelButton?: IButton;

/**
* The hover-prompt for the Level Up button.
* @type {string}
* @default "Move Level Up"
*/
levelUpButtonTitle?: string;

/**
* Automatically generated ImgButton providing a mechanism for moving existing
*  sort-levels up in the sort configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortPanel.levelUpButtonProperties and 
*  multiSortPanel.levelUpButtonDefaults.
* @type {ImgButton}
* @default null
*/
levelUpButton?: ImgButton;

/**
* The hover-prompt for the Level Down button.
* @type {string}
* @default "Move Level Down"
*/
levelDownButtonTitle?: string;

/**
* Automatically generated ImgButton providing a mechanism for moving existing
*  sort-levels down in the sort configuration.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortPanel.levelDownButtonProperties and 
*  multiSortPanel.levelDownButtonDefaults.
* @type {ImgButton}
* @default null
*/
levelDownButton?: ImgButton;

/**
* Automatically generated ListGrid allowing the user to configure a set of 
*  SortSpecifiers.
*  
*  This component is an AutoChild and as such may be customized via 
*  multiSortPanel.optionsGridProperties and 
*  multiSortPanel.optionsGridDefaults.
* @type {ListGrid}
* @default null
*/
optionsGrid?: ListGrid;

/**
* The initial sort configuration to show in the 
*  MultiSortPanel.optionsGrid.
* @type {Array<Partial<SortSpecifier>>}
* @default null
*/
initialSort?: Array<Partial<SortSpecifier>>;

/**
* The maximum number of levels of sorting that can be applied. Since each sort-property or
*  field-name can be used only once in a given multi-sort operation, if no maxLevels value
*  or a value larger than the total number of available properties is specified, it will 
*  default to the total number of available properties.
* @type {number}
* @default null
*/
maxLevels?: number;



/**
*  Return the number of levels of sorting that have been configured.
* 
* 
*/
getNumLevels(): number;

/**
*  Return a SortSpecifier object for the requested levelNum.
* 
* 
* @param {number} The index of the level to return a SortSpecifier for
*/
getSortLevel(levelNum:number): SortSpecifier;

/**
*  Returns all configured sorting levels, as an array of SortSpecifiers.
* 
* 
*/
getSort(): Array<Partial<SortSpecifier>>;

/**
*  Validate that no two SortSpecifiers sort on the same 
*  SortSpecifier.property.
* 
* 
*/
validate(): boolean;

/**
*  Fired whenever the sort configuration changes. The single parameter is an array of
*  SortSpecifiers that represent the list of sort-levels as they appear after 
*  whatever change has occurred.
* 
* 
* @param {Array<Partial<SortSpecifier>>} The current sort configuration, after any changes
*/
sortChanged(sortLevels:Array<Partial<SortSpecifier>>): void;



}
/**
* Creates a new MultiSortPanel
* 
* @param typeCheckedProperties {Partial<MultiSortPanel>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MultiSortPanel} The new instance
*/
class MultiSortPanel{
static create(typeCheckedProperties? : Partial<MultiSortPanel>, uncheckedProperties?: Object): MultiSortPanel;







}

/**
* Provides tracking of other SmartClient browser windows opened by the original window,
*  as RemoteWindow.
*  
*  Includes APIs for:
*  
* - Registering event listeners for events on other windows
*  
* - Opening a new window and finding a window by name
*  
* - Moving, activating, or deactiving a window by name
*  
* - Sharing DataSources and their caches between SmartClient windows
*  
* - Sharing messaging channels between SmartClient windows
*  
*  Within the OpenFin environment,
*  the underlying implementation is actually via the OpenFin class.
*  
*  Reloading of child windows is in general supported (but see
*  MultiWindow.autoCopyDataSources), while reloading the
*  MultiWindow.isMainWindow currently is not.
*  
*  Note: this is currently an experimental feature and not supported except by special
*  arrangement.
*/
interface MultiWindow extends Class {




}
/**
* Creates a new MultiWindow
* 
* @param typeCheckedProperties {Partial<MultiWindow>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {MultiWindow} The new instance
*/
class MultiWindow{
static create(typeCheckedProperties? : Partial<MultiWindow>, uncheckedProperties?: Object): MultiWindow;
/**
* Should DataSource from other OpenFin windows with SmartClient loaded
*  be copied by reference into this window? Such DataSources will be copied:
*  
* - when a page in another window is loaded (potentially several DataSources at once)
*  
* - at the moment DataSources are created in a page loaded in another window (just that
*  DataSource)
*  
*  This property will default to true if OpenFin is present; otherwise, false.
*  
*  Note that reloading a page that created DataSources copied by reference into other
*  windows (via this property) is not supported.
* @type {boolean}
* @default "varies"
*/
static autoCopyDataSources?: boolean;

/**
* Should this window share messaging channels with other
*  windows?
*  
*  This property will default to true if OpenFin is present; otherwise, false.
* @type {boolean}
* @default "varies"
*/
static shareMessageChannels?: boolean;





/**
*  Returns the RemoteWindow instance for the current window (where the method was
*  called).
* 
*/
static getLocalWindow(): RemoteWindow;

/**
*  Returns the parent RemoteWindow instance that opened this window.
* 
*/
static getParentWindow(): RemoteWindow;

/**
*  Returns whether this RemoteWindow wraps the main application window.
* 
*/
static isMainWindow(): boolean;

/**
*  Returns a list of RemoteWindow for the other currently known windows (excluding
*  the MultiWindow.getLocalWindow). This would typically only be used to
*  initialize logic dependent on other windows. You'd want to observe
*  MultiWindow.otherWindowsChanged to ensure you keep things updated as windows are opened or closed.
* 
*/
static getOtherWindows(): Array<Partial<RemoteWindow>>;

/**
*  Registers a window event listener to be called whenever the event type occurs for any
*  window in the application.
* 
* @param {MultiWindowEvent} event type to register
* @param {MultiWindowEventCallback} function to be called when event fires
*/
static setEvent(eventType:MultiWindowEvent, listener:(remoteWindow:RemoteWindow, eventType:MultiWindowEvent, event:object) => void): number;

/**
*  Unregisters a previously registered window event listener. The event type and ID
*  returned by the original MultiWindow.setEvent call should be passed.
* 
* @param {MultiWindowEvent} event type to register
* @param {number=} ID of the event to clear. If not specified, all                   events of eventType will be cleared.
*/
static clearEvent(eventType:MultiWindowEvent, ID?:number): void;

/**
*  Notification fired when the set of other RemoteWindow changes or
*  requires re-synchronization due a call to Class.create,
*  RemoteWindow.close, or a page reload in a different RemoteWindow.
*  
*  This method has no default implementation.
* 
* @param {Array<Partial<RemoteWindow>>} current set of other RemoteWindows.
*/
static otherWindowsChanged(otherWindows:Array<Partial<RemoteWindow>>): void;

/**
*  Returns the DataContext provided by the MultiWindow.open call that opened this
*  window, or a newly created (on demand) DataContext if this is the main
*  application window, or no DataContext was provided.
* 
*/
static getDataContext(): DataContext;

/**
*  Opens a new window with the specified URL, name, and DataContext.
*  
*  Note that if the provided window name already exists, that window will just be
*  MultiWindow.activate, and though the callback will be run, the supplied url,
*  dataContext, windowSettings, and classSettings will be ignored.
*  
* 
* 
* @param {URL} url to open in the window or null to reuse the current url
* @param {string} unique window name to open as a new window
* @param {DataContext=} dataContext to apply to window
* @param {RemoteWindowBooleanCallback=} callback run after RemoteWindow                    created or activated (parameter true if created)
* @param {BrowserWindowSettings=} settings applied to child browser window
* @param {MultiWindowSettings=} settings for child MultiWindow class
*/
static open(url:URL, name:string, dataContext?:DataContext, callback?:(remoteWindow:RemoteWindow, result:boolean) => void, windowSettings?:BrowserWindowSettings, classSettings?:MultiWindowSettings): void;

/**
*  Returns window with the specified name, if it exists in the application. Note that,
*  without OpenFin, only windows created with MultiWindow.open (and the base window) can
*  be found with this method.
* 
* @param {string} unique window name
*/
static find(name:string): RemoteWindow;

/**
*  Closes the existing window with the specified name,
* 
* @param {string} unique window name
* @param {boolean=} whether to force it closed
* @param {RemoteWindowCallback=} callback run after RemoteWindow closed
*/
static close(name:string, force?:boolean, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Moves the existing window with the specified name,
* 
* @param {string} unique window name
* @param {number} desired x-offset of left edge
* @param {number} desired y-offset of top edge
* @param {RemoteWindowCallback=} callback run after RemoteWindow moved
*/
static move(name:string, x:number, y:number, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Resizes e existing window with the specified name,
* 
* @param {string} unique window name
* @param {number} desired new width
* @param {number} desired new height
* @param {RemoteWindowCallback=} callback run after RemoteWindow moved
*/
static resize(name:string, width:number, height:number, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Activates (focuses) the existing window with the specified name,
* 
* @param {string} unique window name
* @param {RemoteWindowCallback=} callback run after RemoteWindow                     activated
*/
static activate(name:string, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Deactivates (blurs) the existing window with the specified name,
* 
* @param {string} unique window name
* @param {RemoteWindowCallback=} callback run after RemoteWindow                     deactivated
*/
static deactivate(name:string, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Maximizes the existing window with the specified name,
* 
* @param {string} unique window name
* @param {RemoteWindowCallback=} callback run after RemoteWindow                        maximized
*/
static maximize(name:string, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Minimizes the existing window with the specified name,
* 
* @param {string} unique window name
* @param {RemoteWindowCallback=} callback run after RemoteWindow                        minimized
*/
static minimize(name:string, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Restores the existing window with the specified name,
* 
* @param {string} unique window name
* @param {RemoteWindowCallback=} callback run after RemoteWindow                        restored
*/
static restore(name:string, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Hides the existing window with the specified name,
* 
* @param {string} unique window name
* @param {RemoteWindowCallback=} callback run after RemoteWindow                        hidden
*/
static hide(name:string, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Shows the existing window with the specified name,
* 
* @param {string} unique window name
* @param {RemoteWindowCallback=} callback run after RemoteWindow                        shown
*/
static show(name:string, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Brings the existing window with the specified name to the top of the window stack.
* 
* @param {string} unique window name
* @param {RemoteWindowCallback=} callback run after RemoteWindow                        is brought to the front
*/
static bringToFront(name:string, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Checks whether a window with the specified name is showing. Callback returns null if
*  the window cannot be found; otherwise returns true or false according as it's showing.
* 
* @param {string} unique window name
* @param {RemoteWindowBooleanCallback} callback to receive output
*/
static isShowing(name:string, callback:(remoteWindow:RemoteWindow, result:boolean) => void): void;




}

/**
* A checkbox for manipulating 2-valued fields based on the native checkbox element.
*/
interface NativeCheckboxItem extends FormItem {
/**
* Base CSS class applied to this item's title text (rendered next to the checkbox element).
* @type {FormItemBaseStyle}
* @default "labelAnchor"
*/
textBoxStyle?: FormItemBaseStyle;

/**
* Should we show the label text next to the checkbox item.
* @type {boolean}
* @default true
*/
showLabel?: boolean;





}
/**
* Creates a new NativeCheckboxItem
* 
* @param typeCheckedProperties {Partial<NativeCheckboxItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {NativeCheckboxItem} The new instance
*/
class NativeCheckboxItem{
static create(typeCheckedProperties? : Partial<NativeCheckboxItem>, uncheckedProperties?: Object): NativeCheckboxItem;







}

/**
* The NativeScrollbar widget will render in the browser as a native 
*  scrollbar, and has APIs allowing it to be applied to scroll content any another widget
*  on the page. Essentially this behaves similarly to the Scrollbar class but will
*  be rendered as a native browser scrollbar rather than using media, thus providing the
*  advantages of an independant scrollbar (support for rendering the scrollbar separate from the
*  content it effects, support for "virtual scrolling" mechanisms where content size is unknown
*  at initial render, etc), with a native look and feel and without requiring the loading of
*  additional media on the page.
*  
*  To enable this for a component simply set Canvas.showCustomScrollbars to true and
*  set Canvas.scrollbarConstructor to "NativeScrollbar"
*/
interface NativeScrollbar extends Canvas {




}
/**
* Creates a new NativeScrollbar
* 
* @param typeCheckedProperties {Partial<NativeScrollbar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {NativeScrollbar} The new instance
*/
class NativeScrollbar{
static create(typeCheckedProperties? : Partial<NativeScrollbar>, uncheckedProperties?: Object): NativeScrollbar;







}

/**
* Provides a list or tree of NavItem, each of which specifies a
*  component to be displayed in a mutually exclusive fashion in the NavPanel.navDeck.
*  
*  A NavPanel can either have a flat list of NavItems or a hierarchy via
*  NavItem.items - use NavPanel.isTree to explicitly control this.
*  
*  Because NavPanel extends SplitPane, it automatically shifts between side-by-side vs
*  single panel display on handset-sized devices. Specifically, the NavPanel.navGrid is
*  set as the SplitPane.navigationPane and the NavPanel.navDeck is set as the
*  SplitPane.detailPane.
*  
*  Note that NavPanel is a fairly simple component to replicate by composing other
*  SmartClient widgets. If you need a component that looks roughly like a
*  NavPanel but will require lots of visual and behavioral customization, consider
*  using the underlying components directly instead of deeply customizing the
*  NavPanel class. A NavPanel is essentially just a TreeGrid
*  and Deck in a SplitPane, with a ListGrid.recordClick
*  handler to call Deck.setCurrentPane with a component ID stored as an attribute of
*  each Record.
*/
interface NavPanel {
/**
* Whether the NavItems form a Tree or are just a flat list. If
*  isTree is false, TreeGrid.showOpener will be set false on the
*  NavPanel.navGrid so that space isn't wasted.
*  
*  The setting for isTree is defaulted immediately before initial draw, based
*  on whether any NavItem has a list of subitems specified via NavItem.items.
*  If no NavItems are provided before draw, isTree defaults to
*  true. Auto-detection is never attempted again even if all
*  NavItems are replaced.
*  
*  Set isTree explicitly if auto-detection doesn't yield the correct result
*  for your application.
* @type {boolean}
* @default null
*/
isTree?: boolean;

/**
* The TreeGrid used to display NavItems.
* @type {TreeGrid}
* @default null
*/
navGrid?: TreeGrid;

/**
* The layout serving as the SplitPane.navigationPane of this panel.
*  By default it contains only the NavPanel.navGrid, but other members can be added before or
*  after, respectively, via NavPanel.navBeforeMembers and NavPanel.navAfterMembers.
* @type {VLayout}
* @default null
*/
navLayout?: VLayout;

/**
* Members to add before the NavPanel.navGrid in the NavPanel.navLayout when the panel is first
*  drawn.
* @type {Array<Partial<Canvas>>}
* @default null
*/
navBeforeMembers?: Array<Partial<Canvas>>;

/**
* Members to add after the NavPanel.navGrid in the NavPanel.navLayout when the panel is first
*  drawn.
* @type {Array<Partial<Canvas>>}
* @default null
*/
navAfterMembers?: Array<Partial<Canvas>>;

/**
* Pass through to set the ListGrid.baseStyle on the NavPanel.navGrid
*  autoChild.
* @type {CSSStyleName}
* @default null
*/
navItemBaseStyle?: CSSStyleName;

/**
* The Deck area where components specified via NavItem.pane are displayed.
* @type {Deck}
* @default null
*/
navDeck?: Deck;

/**
* CSS style for the NavPanel.navDeck.
* @type {CSSStyleName}
* @default null
*/
deckStyle?: CSSStyleName;

/**
* CSS style used when NavItem.isHeader is set on an item.
*  May be overridden for a specific header item by NavItem.customStyle.
* @type {CSSStyleName}
* @default "navItemHeader"
*/
headerStyle?: CSSStyleName;

/**
* Top-level navigation items to display. You can optionally specify a tree of items using
*  NavItem.items.
*  
*  A separator between navigation items can be created by setting NavItem.isSeparator,
*  and a header can be created via NavItem.isHeader.
*  
*  Each non-separator and non-header NavItem specifies a component to be displayed
*  in the NavPanel.navDeck via NavItem.pane.
*  
*  NavItems can also be individually styled via ListGridRecord._baseStyle
*  or NavItem.customStyle.
* @type {Array<Partial<NavItem>>}
* @default null
*/
navItems?: Array<Partial<NavItem>>;

/**
* The current NavItem whose NavItem.pane is showing in the
*  NavPanel.navDeck. This must be an item of this NavPanel if
*  set.
* @type {NavItem}
* @default null
*/
currentItem?: NavItem;

/**
* The ID of the current NavItem whose NavItem.pane is showing in the
*  NavPanel.navDeck. The NavItem must be an item of this
*  NavPanel if set.
*  
*  The ID of a NavItem is the item's NavItem.id if set; otherwise, it
*  is the ID of the item's NavItem.pane, though currentItemId may be
*  initialized to either identifier.
* @type {Identifier}
* @default null
*/
currentItemId?: Identifier;

/**
* Select the first NavItem on initialization if neither NavPanel.currentItemId nor
*  NavPanel.currentItem are provided.
* @type {boolean}
* @default false
*/
defaultToFirstItem?: boolean;



/**
*  Setter for NavPanel.currentItem. Note that NavPanel.currentItemId is also
*  updated by this setter.
* 
* @param {NavItem=} the new currentItem. May be null to hide the current item. If newCurrentItem is a separator or header item, then setCurrentItem() has no effect.
*/
setCurrentItem(newCurrentItem?:NavItem): void;

/**
*  Setter for NavPanel.currentItemId. Note that NavPanel.currentItem is also
*  updated by this setter and this.currentItemId may be normalized to a different
*  identifier.
* 
* @param {Identifier=} the ID of the new current item, which may be either the item's NavItem.id or the ID of the item's NavItem.pane. May be null or an empty string to hide the current item. If the item with ID newCurrentItemId is a separator or header item, then setCurrentItemId() has no effect.
*/
setCurrentItemId(newCurrentItemId?:Identifier): void;



}
/**
* Creates a new NavPanel
* 
* @param typeCheckedProperties {Partial<NavPanel>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {NavPanel} The new instance
*/
class NavPanel{
static create(typeCheckedProperties? : Partial<NavPanel>, uncheckedProperties?: Object): NavPanel;







}

/**
* Causes the list pane component to load data and update its title based on the current
*  selection in the source pane. Also shows the pane if it's not already visible.
*/
interface NavigateSplitPaneTask extends ComponentTask {
/**
* Pane target to be navigated from the current pane.
* @type {CurrentPane}
* @default null
*/
targetPane?: CurrentPane;

/**
* Title to show instead of the automatically chosen one.
* @type {string}
* @default null
*/
title?: string;

/**
* Set to false to not show a component's parents.
* @type {boolean}
* @default null
*/
showRecursively?: boolean;





}
/**
* Creates a new NavigateSplitPaneTask
* 
* @param typeCheckedProperties {Partial<NavigateSplitPaneTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {NavigateSplitPaneTask} The new instance
*/
class NavigateSplitPaneTask{
static create(typeCheckedProperties? : Partial<NavigateSplitPaneTask>, uncheckedProperties?: Object): NavigateSplitPaneTask;







}

/**
* Navigation control implemented as a horizontal layout showing back and forward controls 
*  and a title.
*/
interface NavigationBar extends HLayout {
/**
* Whether to animate a change of the view state via NavigationBar.setViewState.
*  
*  Enabling animation of state changes does have a performance impact because more components
*  need to be created by the NavigationBar to implement the animated transitions.
*  It is therefore recommended to leave animateStateChanges at its default value
*  of false unless NavigationBar.setViewState might be called on this
*  NavigationBar instance and animation is desired.
*  
*  Note also that when animation is enabled, certain AutoChild defaults and properties may
*  be used to create other AutoChildren that are internal to the animation implementation. This
*  generally does not cause an issue unless certain non-UI event handlers are added to the
*  defaults and/or properties (e.g. Canvas.visibilityChanged, Canvas.resized).
*  For those types of handlers, a check should be added to make sure that the handler is
*  running for the expected component.
* @type {boolean}
* @default false
*/
animateStateChanges?: boolean;

/**
* Button.title for the NavigationBar.leftButton.
* @type {HTMLString}
* @default null
*/
leftButtonTitle?: HTMLString;

/**
* Short title to display for the left button title if there is not enough room to show the title
*  for the navigation bar. Setting to null or an empty string ("") will avoid a shortened
*  title ever being used. See NavigationBar.title for a full description.
* @type {HTMLString}
* @default "Back"
*/
shortLeftButtonTitle?: HTMLString;

/**
* If set, the left button title will never be omitted in an attempt to fit the full title.
*  See the documentation of NavigationBar.title for details.
* @type {boolean}
* @default false
*/
alwaysShowLeftButtonTitle?: boolean;

/**
* Button.icon for the NavigationBar.leftButton.
* @type {SCImgURL}
* @default "[SKIN]back_arrow.png"
*/
leftButtonIcon?: SCImgURL;

/**
* If set to false, then the NavigationBar.leftButton is not shown.
* @type {boolean}
* @default null
*/
showLeftButton?: boolean;

/**
* The button displayed to the left of the title in this NavigationBar. By default this
*  will be a NavigationButton with NavigationButton.direction set
*  to
*  "back".
*  
*  
*  The following autoChildUsage apply:
*  
*  
* - NavigationBar.leftButtonTitle for Button.title
*  
* - NavigationBar.leftButtonIcon for Button.icon
*  
* @type {NavigationButton}
* @default null
*/
leftButton?: NavigationButton;

/**
* The title to display in the center of this navigation bar.
*  
*  If there is not enough room for the title with the current titles of the
*  NavigationBar.leftButton and NavigationBar.rightButton buttons,
*  space will be used as follows:
*  
*  
* - if the title can be fully visible without clipping if it is placed slightly off-center, it
*  will be placed off-center, up to a maximum of NavigationBar.maxCenterOffset pixels
*  
* - if that's not enough space, if a NavigationBar.shortLeftButtonTitle
*  is provided, it will be used in lieu of the normal left button title
*  
* - if that's still not enough space, the title of the left button will be hidden, leaving
*  only the icon. This will be skipped if either NavigationBar.alwaysShowLeftButtonTitle
*  has been set or the button has no icon, which would leave the space blank.
*  
* - if that's still not enough space, the title text will be clipped, showing an ellipsis
*  (where browser support allows this)
*  
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* Maximum amount in pixels that the title will be placed off center in an effort to avoid
*  clipping it - see NavigationBar.title.
* @type {Integer}
* @default "40"
*/
maxCenterOffset?: Integer;

/**
* The AutoChild label used to display the NavigationBar.title in this
*  NavigationBar.
* @type {Label}
* @default null
*/
titleLabel?: Label;

/**
* Button.title for the NavigationBar.rightButton.
* @type {HTMLString}
* @default "&amp;nbsp;"
*/
rightButtonTitle?: HTMLString;

/**
* Button.icon for the NavigationBar.rightButton.
* @type {SCImgURL}
* @default null
*/
rightButtonIcon?: SCImgURL;

/**
* If set to false, then the NavigationBar.rightButton is not shown.
* @type {boolean}
* @default null
*/
showRightButton?: boolean;

/**
* The button displayed to the right of the title in this NavigationBar. By default this
*  will be a NavigationButton with NavigationButton.direction set
*  to "forward".
*  
*  The following autoChildUsage apply:
*  
*  
* - NavigationBar.rightButtonTitle for Button.title
*  
* - NavigationBar.rightButtonIcon for Button.icon
*  
* @type {NavigationButton}
* @default null
*/
rightButton?: NavigationButton;

/**
* Controls to show in the navigation bar. The auto children names
*  "leftButton", "titleLabel", "rightButton" may be used to show the standard
*  navigation bar controls, as well as any Canvases (which will be embedded directly
*  in the navigation bar).
* @type {Array<Partial<string>> | Array<Partial<Canvas>>}
* @default null
*/
controls?: Array<Partial<string>> | Array<Partial<Canvas>>;

/**
* AutoChild of type MiniNavControl. Not shown by default (see
*  NavigationBar.showMiniNavControl). Also, if a
*  NavigationBar.customNavControl is provided, then the
*  customNavControl is used instead of an automatically created
*  miniNavControl.
* @type {MiniNavControl}
* @default null
*/
miniNavControl?: MiniNavControl;

/**
* If set to false, then the NavigationBar.miniNavControl
*  is not shown.
* @type {boolean}
* @default false
*/
showMiniNavControl?: boolean;

/**
* Placement of MiniNavControl, if present:
*  
*   
* - "right" alignment places the miniNav on the far right
*   
* - "center" alignment places the miniNav in the center, or to the right of the title
*     if the title is present
*   
* - "left" alignment will place the miniNav on the left, or to the right of the
*     NavigationBar.leftButton if its present.
*  
* @type {Alignment}
* @default "right"
*/
miniNavAlign?: Alignment;

/**
* An arbitrary component that will be placed where the NavigationBar.miniNavControl
*  AutoChild would normally be placed (see NavigationBar.miniNavAlign).
* @type {Canvas}
* @default null
*/
customNavControl?: Canvas;



/**
*  Updates the NavigationBar.title for this NavigationBar.
* 
* @param {HTMLString} new title HTML.
*/
setTitle(newTitle:HTMLString): void;

/**
*  Setter for NavigationBar.leftButtonTitle.
* 
* @param {HTMLString} new title HTML for the left button.
*/
setLeftButtonTitle(newTitle:HTMLString): void;

/**
*  Setter for NavigationBar.shortLeftButtonTitle.
* 
* 
* @param {HTMLString} new short title HTML.
*/
setShortLeftButtonTitle(newShortLeftButtonTitle:HTMLString): void;

/**
*  Setter for NavigationBar.alwaysShowLeftButtonTitle.
* 
* 
* @param {boolean} new value for alwaysShowLeftButtonTitle.
*/
setAlwaysShowLeftButtonTitle(newAlwaysShowLeftButtonTitle:boolean): void;

/**
*  Setter for NavigationBar.leftButtonIcon.
* 
* @param {SCImgURL} new icon for left button.
*/
setLeftButtonIcon(newIcon:SCImgURL): void;

/**
*  Show or hide the NavigationBar.leftButton. The leftButton
*  must be a NavigationBar.controls of this NavigationBar or else it will
*  still be hidden.
* 
* @param {boolean} if false, then the leftButton will be hidden. If unset or true then the leftButton will be shown as long as it is a member of the controls array.
*/
setShowLeftButton(show:boolean): void;

/**
*  Setter for NavigationBar.rightButtonTitle.
* 
* @param {HTMLString} new title HTML for the right button.
*/
setRightButtonTitle(newTitle:HTMLString): void;

/**
*  Setter for NavigationBar.rightButtonIcon.
* 
* @param {SCImgURL} new icon for the right button.
*/
setRightButtonIcon(newIcon:SCImgURL): void;

/**
*  Show or hide the NavigationBar.rightButton. The rightButton
*  must be a NavigationBar.controls of this NavigationBar or else it will
*  still be hidden.
* 
* @param {boolean} if false, then the rightButton will be hidden. If unset or true then the rightButton will be shown as long as it is a member of the controls array.
*/
setShowRightButton(show:boolean): void;

/**
*  Setter to update the NavigationBar.customNavControl at runtime.
* 
* @param {Array<Partial<string>> | Array<Partial<Canvas>>} */
setCustomNavControl(controls:Array<Partial<string>> | Array<Partial<Canvas>>): void;

/**
*  Sets multiple state attributes of this NavigationBar at once. If this
*  NavigationBar was created with NavigationBar.animateStateChanges
*  set to true, then the change-over to the new state attributes will be
*  animated if the direction is either
*  "forward"
*  
*  or
*  "back".
*  
* 
* @param {NavigationBarViewState} the new view state.
* @param {NavigationDirection=} an optional direction for animation. If not specified or set to "none"  then the state change will not be animated. The direction should be "forward"  for operations that reveal new content and "back"  for operations that reveal previously-displayed content.
*/
setViewState(viewState:NavigationBarViewState, direction?:NavigationDirection): void;

/**
*  Notification method fired when the user clicks the NavigationBar.leftButton or NavigationBar.rightButton
* 
* @param {NavigationDirection} direction in which the user is attempting to  navigate
*/
navigationClick(direction:NavigationDirection): void;

/**
*  Notification method fired when the up button on the NavigationBar.miniNavControl
*  is clicked.
* 
* 
*/
upClick(): void;

/**
*  Notification method fired when the down button on the NavigationBar.miniNavControl
*  is clicked.
* 
* 
*/
downClick(): void;



}
/**
* Creates a new NavigationBar
* 
* @param typeCheckedProperties {Partial<NavigationBar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {NavigationBar} The new instance
*/
class NavigationBar{
static create(typeCheckedProperties? : Partial<NavigationBar>, uncheckedProperties?: Object): NavigationBar;







}

/**
* Specially styled Button subclass used by the NavigationBar class.
*/
interface NavigationButton extends Button {
/**
* Default baseStyle for navigation buttons. Note that the special NavigationButton.backBaseStyle and
*  NavigationButton.forwardBaseStyle are applied if NavigationButton.direction is set.
* @type {CSSStyleName}
* @default "navButton"
*/
baseStyle?: CSSStyleName;

/**
* Base style for navigation buttons where Direction is set to "back"
* @type {CSSStyleName}
* @default "navBackButton"
*/
backBaseStyle?: CSSStyleName;

/**
* Base style for navigation buttons where Direction is set to "forward"
* @type {CSSStyleName}
* @default "navForwardButton"
*/
forwardBaseStyle?: CSSStyleName;

/**
* Navigation direction for this button. If set to "forward" or
*  "back" the special NavigationButton.forwardBaseStyle or NavigationButton.backBaseStyle
*  will be applied.
* @type {NavigationDirection}
* @default "none"
*/
direction?: NavigationDirection;





}
/**
* Creates a new NavigationButton
* 
* @param typeCheckedProperties {Partial<NavigationButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {NavigationButton} The new instance
*/
class NavigationButton{
static create(typeCheckedProperties? : Partial<NavigationButton>, uncheckedProperties?: Object): NavigationButton;







}

/**
* Notify provides a means to display on-screen messages that are automatically dismissed after
*  a configurable amount of time, as an alternative to isc.confirm
*  dialogs that can lower end user productivity. Messages may be shown at a particular
*  location, specified either with viewport-relative coordinates, or as an edge or center
*  location relative to the viewport or a canvas. Messages can be configured to appear and
*  disappear instantly, by sliding into (or out of) view, or by fading in (or out).
*  
*  One or more NotifyAction can be provided when Notify.addMessage is called to display a message. They will be rendered as links on which to
*  click to execute the action.
*  
*  The behavior and appearance of messages are configured per NotifyType, which is simply
*  a string that identifies that message group, similar to Class.logWarn.
*  By calling Notify.configureMessages with the
*  NotifyType, it can be assigned a NotifySettings configuration to control
*  message animation, placement, and the the Label used to render each message, allowing
*  styling and autofit behavior to be configured.
*  
*  Messages of the same NotifyType may be stacked to provide a visible
*  history, with a configurable stacking direction and maximum stacking depth. Details on how
*  to configure messages are provided in the documentation for NotifySettings.
*  
*  Messages for different NotifyTypes are stacked separately and animated by
*  independent Framework pipelines. It's up to you to configure the placement of supported
*  NotifyTypes in your app so that they don't overlap, as the Framework doesn't
*  manage it. For example, separate NotifyTypes could be assigned separate screen
*  edges, or assigned a different NotifySettings.positionCanvas.
*  
*  To dismiss a message manually before its scheduled duration expires, you may call
*  Notify.dismissMessage with a NotifyType (to dismiss
*  all such messages) or an ID previously returned by Notify.addMessage
*  (to dismiss that single message).
*  
*  Warnings and Errors
*  
*  Each notification may be assigned a NotifySettings.messagePriority in
*  the settings passed to Notify.addMessage. By default, all
*  NotifyTypes are configured to have priority Notify.MESSAGE, except for
*  "error" and "warn" NotifyTypes, which are configured with priority
*  Notify.ERROR and Notify.WARN, respectively.
*  
*  The NotifySettings.messagePriority determines the default styling of a
*  message, and which message to remove if a new message is sent while the message stack is
*  already at its limit. We recommended applying a NotifySettings.messagePriority as the best approach for showing pre-styled warnings and errors, since that
*  allows you to interleave them with ordinary messages in a single NotifyType.
*  
*  Alternatively, you can display pre-styled warnings and errors by calling
*  Notify.addMessage with the separate NotifyTypes
*  "warning" and "error", respectively, but then you must take care to
*  Notify.configureMessages used to a separate screen
*  location to avoid one rendering on top of the other.
*  
*  Viewport Considerations
*  
*  Messages are edge or corner-aligned based on the Page.getScrollWidth
*  and Page.getScrollHeight of the current page rather than screen, so
*  you may need to scroll to see the targeted corner or edge. Note that widgets placed
*  offscreen below or to the right of a page may cause the browser to report a larger viewport,
*  and prevent messages from being visible, even if no scrollbars are present. If you need to
*  stage widgets offscreen for measurement or other reasons, place them above or to the left.
*  
*  Modal Windows and Click Masks
*  
*  Messages are always shown above all other widgets, including Window.isModal and Canvas.showClickMask. This is because it's expected
*  that messages are "out of band" and logically indepedent of the widget hierarchy being shown.
*  We apply this layering policy even for windows and widgets created by NotifyActions.
*  If there may a scenario where a message can block a window created by an action, set
*  NotifySettings.canDismiss to true so that an unobstructed view of the underlying
*  widgets can be restored.
*  
*  In the linked sample, note how we take care to reuse the existing modal window, if any, if
*  the "Launch..." link is clicked, so that repeated clicks never stack windows over each other.
*/
interface Notify extends Class {




}
/**
* Creates a new Notify
* 
* @param typeCheckedProperties {Partial<Notify>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Notify} The new instance
*/
class Notify{
static create(typeCheckedProperties? : Partial<Notify>, uncheckedProperties?: Object): Notify;
/**
* Highest priority. Default priority of NotifyType: "error".
* @type {MessagePriority}
* @default "1"
*/
static ERROR?: MessagePriority;

/**
* Second-highest priority. Default priority of NotifyType: "warn".
* @type {MessagePriority}
* @default "2"
*/
static WARN?: MessagePriority;

/**
* Third-highest priority. Default priority for all NotifyTypes other than "error"
*  and "warn".
* @type {MessagePriority}
* @default "3"
*/
static MESSAGE?: MessagePriority;





/**
*  Displays a new message, subject to the Notify.configureMessages
*  for the passed notifyType, overridden by any passed settings.
*  Returns an opaque MessageID that can be passed to Notify.dismissMessage
*  to clear it.
*  
*  Note that an empty string may be passed for contents if actions
*  have been provided, so you may have the message consist only of your specified actions.
*  
*  Most users should do all configuration up front via a call to Notify.configureMessages.
*  The settings argument in this method is provided to allow adjustment of
*  properties that affect only one message, such as NotifySettings.autoFitWidth, NotifySettings.styleName, or 
*  NotifySettings.labelProperties. Making changes to 
*  MultiMessageMode-related properties via this argument isn't supported,
*  unless specifically documented on the property.
*  
* 
* @param {HTMLString} message to be displayed
* @param {Array<Partial<NotifyAction>>=} actions (if any) for this message
* @param {NotifyType=} category of message; default "message"
* @param {NotifySettings=} display and behavior settings for                        this message that override the                        Notify.configureMessages                        settings for the notifyType
*/
static addMessage(contents:HTMLString, actions?:Array<Partial<NotifyAction>>, notifyType?:NotifyType, settings?:NotifySettings): MessageID;

/**
*  Dismisses one or more messages currently being shown, subject to the existing settings
*  for their NotifyType. You may either pass the opaque message identifier returned
*  from the call to Notify.addMessage to dismiss a single message, or a
*  NotifyType to dismiss all such messages.
* 
* @param {MessageID | NotifyType} message identifier or category to dismiss
*/
static dismissMessage(messageID:MessageID | NotifyType): void;

/**
*  Can the message corresponding to the messageID be dismissed? Returns false
*  if the message is no longer being shown. The messageID must have been
*  previously returned by Notify.addMessage.
* 
* @param {MessageID} message identifier to dismiss
*/
static canDismissMessage(messageID:MessageID): boolean;

/**
*  Sets the default NotifySettings for the specified NotifyType. This
*  may be overridden by passing settings to Notify.addMessage when the message
*  is shown, but changing MultiMessageMode-related properties via
*  Notify.addMessage isn't supported,
*  
*  By default, the NotifyTypes "message", "warn", and "error" are predefined, each
*  with their own NotifySettings with different NotifySettings.styleNames. When configuring a new (non-predefined) NotifyType with this method, any
*  NotifySettings left unset will default to those for NotifyType "message".
*  
* 
* @param {NotifyType} category of message; null defaults to "message"
* @param {NotifySettings} settings to store for the notifyType
*/
static configureMessages(notifyType:NotifyType, settings:NotifySettings): void;

/**
*  Changes the default settings that are applied when you create a new NotifyType
*  with Notify.configureMessages. If you want to change the defaults for the built-in
*  NotifyTypes "message", "warn", and "error", with this method, it must be called before
*  the first call to Notify.configureMessages or Notify.addMessage. Once a NotifyType
*  has been created, you must use Notify.configureMessages to change its settings.
*  
*  Note that for defaults that depend on priority (and thus differ between the built-in
*  NotifyTypes), this method only sets the defaults for the "message" NotifyType.
*  
* 
* @param {NotifySettings} changes to NotifyType defaults
*/
static configureDefaultSettings(settings:NotifySettings): void;

/**
*  Updates the contents of the message from what was passed originally to
*  Notify.addMessage, while preserving any existing NotifyAction.
*  
*  The purpose of this method is to support messages that contain timer countdowns or other
*  data that perhaps need refreshing during display. If you find yourself replacing the
*  entire content with something new, you should probably just add it as a new message.
*  
*  Note that this method has minimal animation support. The change in message content and
*  corresponding resizing are instant, but the repositioning of the message or stack (if
*  stacked) to keep your requested NotifySettings.position is controlled
*  by NotifySettings.repositionMethod, allowing slide animation. However, that
*  setting is ignored and the repositioning is instant if you've chosen
*  NotifySettings.positionCanvas to a border or corner along
*  the NotifySettings.position viewport edge, or if an animation is
*  already in progress, in which case the instant repositioning will happen after it
*  completes.
* 
* @param {MessageID} message identifier from Notify.addMessage
* @param {HTMLString} updated message
*/
static setMessageContents(messageID:MessageID, contents:HTMLString): void;

/**
*  Updates the actions of the message from those, if any, passed originally to
*  Notify.addMessage, while preserving any existing Notify.addMessage.
*  
*  See Notify.setMessageContents for further guidance and animation details.
* 
* @param {MessageID} message identifier from Notify.addMessage
* @param {Array<Partial<NotifyAction>>} updated actions for this message
*/
static setMessageActions(messageID:MessageID, actions:Array<Partial<NotifyAction>>): void;

/**
* @param {MessageID} message identifier to check
*/
static messageHasActions(messageID:MessageID): boolean;




}

/**
* Static singleton class containing APIs for interacting with Numbers.
*/
interface NumberUtil extends Class {




}
/**
* Creates a new NumberUtil
* 
* @param typeCheckedProperties {Partial<NumberUtil>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {NumberUtil} The new instance
*/
class NumberUtil{
static create(typeCheckedProperties? : Partial<NumberUtil>, uncheckedProperties?: Object): NumberUtil;
/**
* The decimal symbol to use when formatting numbers
*  
* @type {string}
* @default "."
*/
static decimalSymbol?: string;

/**
* The grouping symbol, or thousands separator, to use when formatting numbers
*  
* @type {string}
* @default ","
*/
static groupingSymbol?: string;

/**
* The negative symbol to use when formatting numbers
*  
* @type {string}
* @default "-"
*/
static negativeSymbol?: string;

/**
* The currency symbol to use when formatting numbers
*  
* @type {string}
* @default "$"
*/
static currencySymbol?: string;

/**
* If true, show any currency symbol after the number when formatting numbers. If false, show
*  any currency symbol before the number
* @type {boolean}
* @default false
*/
static currencySymbolLast?: boolean;

/**
* The format to use when formatting negative numbers. Supported values are: 1 = before, 
*  2 = after, 3 = beforeSpace, 4 = afterSpace, 5 = parens
*  
* @type {number}
* @default "1"
*/
static negativeFormat?: number;

/**
* The grouping-format for numbers
*  
* @type {number}
* @default "1"
*/
static groupingFormat?: number;





/**
*  Return the passed number as a string padded out to digits length.
* 
* 
* @param {number} Number object to stringify
* @param {number=} Number of digits to pad to. (Default is 2)
*/
static stringify(number:number, digits?:number): string;

/**
*  Returns true if the number parameter falls between the 'first' and 'second' parameters.
* 
* 
* @param {number} Number object to be evaluated
* @param {number=} Number at the lower boundary
* @param {number=} Number at the upper boundary
* @param {number=} Whether or not the numbers at either end of the boundary should be included in the comparison
*/
static isBetween(number:number, first?:number, second?:number, inclusive?:number): boolean;

/**
*  Returns a clamped number between a min and max.
*  
*  
*  
*  var clamped = isc.NumberUtil.clamp(10, 0, 5); // Returns 5 because 10 is greater than 5
*  var clamped = isc.NumberUtil.clamp(-3, 0, 5); // Returns 0 because -3 is less than 0
*  var clamped = isc.NumberUtil.clamp(4, 0, 5); // Returns 4 because 4 is between 0 and 5
*  
*  
*  
* 
* @param {number} the number to clamp
* @param {number} the number to return if the number is lower than min
* @param {number} the number to return if the number is higher than max
*/
static clamp(number:number, min:number, max:number): number;

/**
*  Return the passed number as a currency-formatted string, or an empty string if not passed a 
*  number.
* 
* 
* @param {number} the number to convert
* @param {string=} Currency symbol, default taken from the locale and can be                 set to an empty string. If not passed and missing from the                 locale, defaults to "$".
* @param {string=} Decimal separator symbol, default taken from the locale. If                 if not passed and missing from the locale, defaults to                 ".".
* @param {boolean=} Should decimal portion be padded out to two digits? True                by default.
* @param {boolean=} Should the currency symbol be shown at the end of the                    string? If unspecified, and not defined in the locale,                    it will prefix the number.
*/
static toCurrencyString(number:number, currencyChar?:string, decimalChar?:string, padDecimal?:boolean, currencyCharLast?:boolean): string;

/**
*  Format the passed number for readability, with:
*  
*    
* - separators between three-digit groups
*    
* - optional fixed decimal precision (so decimal points align on right-aligned numbers)
*    
* - localized decimal, grouping, and negative symbols
*  
*  NumberUtil.decimalSymbol, 
*  NumberUtil.groupingSymbol, and 
*  NumberUtil.negativeSymbol will normally come from
*  SmartClient locale settings (which may come from either client OS or application locale
*  settings), but they are also supported as arguments for mixed-format applications
*  (eg normalize all currency to NumberUtil.toUSCurrencyString, but use the 
*  current locale format for other numbers).
* 
* 
* @param {number} the number object to convert
* @param {number=} decimal-precision for the formatted value
* @param {string=} the symbol that appears before the decimal part of the number
* @param {string=} the symbol shown between groups of 3 non-decimal digits
* @param {string=} the symbol that indicate a negative number
*/
static toLocalizedString(number:number, decimalPrecision?:number, decimalSymbol?:string, groupingSymbol?:string, negativeSymbol?:string): string;

/**
*  Format the passed number as a US string. Returns empty string if not passed a number.
* 
* 
* @param {number} the number object to format
* @param {number=} */
static toUSString(number:number, decimalPrecision?:number): string;

/**
*  Format the passed number as a US Dollar currency string. Returns empty string if not passed 
*  a number.
* 
* 
* @param {number} the number object to format
* @param {number=} */
static toUSCurrencyString(number:number, decimalPrecision?:number): string;

/**
*  Parse string that contains integer number. This method correctly handles locale based
*  separators and currency symbol.
* 
* 
* @param {string} the string to parse
*/
static parseInt(string:string): number;

/**
*  Parse string that contains float number. This method correctly handles locale based
*  separators, decimal points and currency symbol.
* 
* 
* @param {string} the string to parse
*/
static parseFloat(string:string): Float;

/**
* 
*  If given a numeric string (that is, a non-empty string which converts to a
*  number), will return the equivalent integer. Otherwise, returns the
*  parameter unchanged. Useful for dealing with values that can be numbers or
*  strings, but which you want to coerce to a numeric type if possible.
* 
* 
* @param {any} the string or number to parse
*/
static parseIfNumeric(numberOrString:any): any;

/**
*  Return the parameter number formatted according to the parameter FormatString.
*  This method is used to implement the DataSourceField.format functionality, but it can
*  also be used to format arbitrary numbers programmatically.
* 
* @param {number} The number to format
* @param {FormatString} The format to apply
*/
static format(number:number, format:FormatString): string;




}

/**
* The Offline class provides features for storing and retrieving values in persistent browser
*  storage. Because this storage is both persistent (ie, it remains available after the 
*  browser is closed and re-opened) and local to the browser, it allows applications to operate
*  when the client is not connected to a network, albeit with some obvious limitations.
*  
*  As well as providing straightforward APIs for storing, retrieving and removing values, 
*  Offline support is integrated into the normal request/response cycle. You can configure 
*  the framework so that server responses are automatically cached in Offline storage. Then,
*  at some future point when the client is offline, responses for known requests are returned 
*  from the Offline cache.
*  
*  SmartClient makes use of various underlying storage techniques, depending on what the 
*  browser supports, but the API to a SmartClient program is the same regardless of the 
*  underlying storage - simple key/value pairs. Currently, we provide Offline support for all 
*  HTML5-compliant browsers, plus earlier versions of Internet Explorer (6 and 7). The amount 
*  of storage available is dictated by the browser, and varies from approximately 500KB to 
*  approximately 5MB.
*/
interface Offline extends Class {




}
/**
* Creates a new Offline
* 
* @param typeCheckedProperties {Partial<Offline>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Offline} The new instance
*/
class Offline{
static create(typeCheckedProperties? : Partial<Offline>, uncheckedProperties?: Object): Offline;




/**
*  Returns true if the current browser session is offline (ie, not connected to a network).
*  If an online/offline state has been set explicitly (see Offline.goOffline and 
*  Offline.goOnline), the explicitly-set state will be returned. Otherwise, the 
*  offline state as reported by the browser will be returned. See 
*  Offline.useNativeOfflineDetection for important notes on 
*  browser detection of offline state.
* 
* 
*/
static isOffline(): boolean;

/**
*  Explicitly sets this session into offline mode. This setting will override whatever 
*  state the browser reports. This allows users to manually set an application into 
*  offline or online state.
*  
* 
*/
static goOffline(): void;

/**
*  Explicitly sets this session into online mode. This setting will override whatever 
*  state the browser reports. This allows users to manually set an application into 
*  offline or online state.
*  
* 
*/
static goOnline(): void;

/**
*  Tells the Offline system to query the browser for the current online/offline state.
*  Calling this method switches off the explicit offline mode setting switched on by 
*  calling Offline.goOnline or Offline.goOffline.
*  
*  It is important to note that browsers vary quite considerably in their ability to 
*  detect that they are offline. Many older browsers simply can't do it; HTML5 browsers
*  expose the navigator.onLine property, but each browser's implementation 
*  is different. Some browsers have a manual "Work Offline" mode which allows the user 
*  to make the decision, and SmartClient provides an equivalent mechanism with the 
*  goOffline and goOnline methods. Generally speaking, these 
*  methods are more reliable than allowing the browser to decide whether your application
*  is offline.
*  
* 
*/
static useNativeOfflineDetection(): void;

/**
*  Stores the passed-in value in browser-local storage, mapped to the passed-in key. If
*  there is no room left to store the value, we discard values from the offline store, 
*  oldest first, until there is room to store the value. If you don't want this behavior,
*  explicitly pass false in the recycleEntries parameter.
*  
*  Note that limitations in the underlying storage engines mean that only primitive 
*  values - Strings, numbers and booleans - can be stored. If you wish to store an Array
*  or Object, you will have to serialize it to JSON or XML first, and then eval it 
*  after retrieval to turn it back into an object. This is important because,
*  upon retrieval, all the values will be of String type. Additionally, if you want to 
*  preserve Date values, we recommend using JSONEncoder.
*  
*  Note: This method throws an exception if it could not store the value (either 
*  because storage is full and recycleEntries was false, or because the value to store is
*  simply too large)
*  
* 
* @param {string} The key to use when storing the value
* @param {any} The value to store
* @param {boolean=} If false, suppresses the default behavior of repeatedly                  discarding the oldest entry if there is insufficient                  space to store the value
*/
static put(key:string, value:any, recycleEntries?:boolean): void;

/**
*  Returns the value mapped to the passed-in key from browser-local storage, or null if 
*  no such key exists.
*  
* 
* @param {string} The key to retrieve a value for
*/
static get(key:string): any;

/**
*  Removes the key/value pair mapped by the passed-in key from browser-local storage
*  
* 
* @param {string} The key to remove
*/
static remove(key:string): void;




}

/**
* Contains OpenFin-specific code to
*  implement MultiWindow for OpenFin via special calls to the OpenFin Application API.
*  
*  Note: this is currently an experimental feature and not supported except by special
*  arrangement
*/
interface OpenFin extends Class {




}
/**
* Creates a new OpenFin
* 
* @param typeCheckedProperties {Partial<OpenFin>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {OpenFin} The new instance
*/
class OpenFin{
static create(typeCheckedProperties? : Partial<OpenFin>, uncheckedProperties?: Object): OpenFin;







}

/**
* Provides a SmartClient wrapper around an
*  OpenFin window.
*  
*  Note: this is currently an experimental feature and not supported except by special
*  arrangement
*/
interface OpenFinWindow extends RemoteWindow {




}
/**
* Creates a new OpenFinWindow
* 
* @param typeCheckedProperties {Partial<OpenFinWindow>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {OpenFinWindow} The new instance
*/
class OpenFinWindow{
static create(typeCheckedProperties? : Partial<OpenFinWindow>, uncheckedProperties?: Object): OpenFinWindow;







}

/**
* An operationBinding tells a DataSource how to execute one of the basic DS operations: fetch,
*  add, update, remove. See DataSource.operationBindings.
*/
interface OperationBinding extends Class {
/**
* For DataSources of DataSource.serverType "sql" only, indicates whether 
*  we should automatically add a sort field for ResultSet.fetchMode. 
*  Only applies to OperationBinding.operationType. If left 
*  unset, this property defaults first to the 
*  DataSource.forceSort, and then to one of the global 
*  values described in the DataSource.defaultSortField 
*  documentation.
*  
*  Note, the ability to set this property per-operation is only provided to allow for
*  complete configurability in unusual cases. See the defaultSortField docs
*  for details of why use of this property should be considered a red flag.
* @type {string}
* @default null
*/
forceSort?: string;

/**
* If set to "false", transformation of values for
*  DataSourceField.multiple fields, normally controlled by
*  DataSourceField.multipleStorage, is instead disabled for this OperationBinding.
* @type {boolean}
* @default null
*/
transformMultipleFields?: boolean;

/**
* Setting skipAudit to true indicates that DataSource.audit 
*  must be skipped for this operationBinding.
*  
*  Note, that this setting can be overridden by server-side API DSRequest.setSkipAudit().
* @type {boolean}
* @default null
*/
skipAudit?: boolean;

/**
* A per-operationBinding setting for beanClassName, otherwise also settable at the top-level
*  DataSource configuration.
* @type {string}
* @default null
*/
beanClassName?: string;

/**
* Which operationType this operationBinding is for. This property is only settable on an
*  operationBinding, not a DataSource as a whole.
* @type {DSOperationType}
* @default null
*/
operationType?: DSOperationType;

/**
* Optional operationId if this DataSource supports two or more variants of one of the basic
*  DataSource operations, for instance, a "fetch" that uses full text search and a "fetch" that
*  accepts per-field search criteria. See DSRequest.operationId for usage.
* @type {string}
* @default null
*/
operationId?: string;

/**
* An optional description of the operationBinding's behavior. Not automatically exposed on
*  any component, but useful for developer documentation, and as such is included on any
*  openapiSupport generated by the framework. Markdown is
*  a commonly used syntax, but you may also embed HTML content in a CDATA tag.
* @type {string}
* @default null
*/
description?: string;

/**
* A comma-separated list of field names that must be present in criteria / advancedCriteria provided by the caller.
*  Failure to provide any one of these will yield a RPCResponse.STATUS_CRITERIA_REQUIRED_ERROR from the server.
* @type {string}
* @default null
*/
requiredCriterion?: string;

/**
* The name of the method to invoke on the ServerObject for this operationBinding.
*  
*  NOTE: If you have a DataSource.serverObject and
*  wish to override this operation so that it simply calls a different method on the same 
*  server object, it is sufficient to specify just this property on the operationBinding:
*  there is no need to redefine the serverObject at the operationBinding level.
* @type {string}
* @default null
*/
serverMethod?: string;

/**
* You can explicitly declare the arguments to be passed to
*  OperationBinding.serverMethod using this attribute. This isn't required - in the
*  absence of methodArguments, the DMI implementation will still automatically
*  pass a stock set of arguments to your method (see the overview in ServerObject), but
*  specifying arguments gives you the ability to call pre-existing methods without adding
*  SmartClient-specific code.
*  
*  The format for specifying methodArguments is as a comma separated list of VTL
*  (Velocity Template Language) expressions. See the
*  VTL Reference
*  and Velocity User Guide 
*  for an overview of how to use VTL.
*  
*  The Velocity context is pre-populated with the following variables - you can pass these
*  verbatim as arguments, or call methods on these objects and pass the resulting values:
*  
*  
* - dsRequest: instance of the current DSRequest
*  
* - request: the current HttpServletRequest
*  
* - response: the current HttpServletResponse
*  
* - rpcManager: the instance of RPCManager for this request
*  
* - dataSource: a DataSource instance for this request
*  
*  So, for example, if you had a method signature like the following:
*  
*  public DSResponse fetch(SupplyItem criteria, long startRow, long endRow)
*  
*  You can invoke it by specifying methodArguments as follows:
*  
*  methodArguments="$dsRequest.criteria, $dsRequest.startRow, $dsRequest.endRow"
*  
*  Without methodArguments, there would be no way for you to specify
*  startRow/endRow as arguments. You could, of course, simply declare the method
*  to take a DSRequest object and call getStartRow()/getEndRow() in
*  the body of the method.
* @type {string}
* @default null
*/
methodArguments?: string;

/**
* Optional ServerObject declaration that specifies the ServerObject configuration for this
*  operationBinding. In the absence of a serverObject specification here, the one specified on
*  the DataSource itself is used by default, if present (DataSource.serverObject). If
*  neither is present, then Direct Method Invocation will not be enabled for this operationBinding.
*  
*  Note that if a dataSource configuration has both a OperationBinding.script block and
*  a specified serverObject for some operation, the
*  script block will be executed, and the serverObject ignored.
* @type {ServerObject}
* @default null
*/
serverObject?: ServerObject;

/**
* Scriptlet to be executed prior to the DataSource operation which is configured by this
*  operationBinding. This setting overrides any 
*  DataSource.script
*  for this operation.
*  
*  Scriptlets are used similarly to DMIs configured via OperationBinding.serverObject -
*  they can add business logic by modifying the DSRequest before it's executed, modifying the
*  default DSResponse, or taking other, unrelated actions. 
*  
*  Scriptlets are used similarly to DMIs configured via DataSource.serverObject or
*  OperationBinding.serverObject -
*  they can add business logic by modifying the DSRequest before it's executed, modifying the
*  default DSResponse, or taking other, unrelated actions. 
*  
*  For example:
*  
*   &lt;operationBindings&gt;
*     &lt;operationBinding operationType="add"&gt;
*       &lt;script language="groovy"&gt;
*        ... Groovy code ...
*       &lt;/script&gt;
*     &lt;/operationBinding&gt;
*   &lt;/operationBindings&gt;
*  
*  
*  Scriptlets can be written in any language supported by the "JSR 223" standard, including Java
*  itself. See the dmiOverview for rules on how to return data,
*  add additional imports, and other settings.
*  
*  The following variables are available for DMI scriptlets:
*  
*  
* - dataSource: the current DataSource
*  
* - dsRequest: the current DSRequest
*  
* - criteria: shortcut to DSRequest.getCriteria() (a Map)
*  
* - values: shortcut to DSRequest.getValues() (a Map)
*  
* - oldValues: shortcut to DSRequest.getOldValues() (a Map)
*  
* - sqlConnection: SQLDataSource only: the current SQLConnection object. If
*  using DataSource.autoJoinTransactions are enabled, this
*  SQLConnection is in the context of the current transaction.
*  
* - beanFactory: the spring BeanFactory (when applicable)
*  
* - cdiBeanManager: the CDI BeanManager (when applicable)
*  
*  
*  Scriptlets also have access to a set of contextual variables related to the Servlets
*  API, as follows:
*  
*  
* - servletRequest: the current ServletRequest
*  
* - session: the current HttpSession
*  
* - rpcManager: the current RPCManager
*  
* - servletResponse: the current ServletResponse (advanced use only)
*  
* - servletContext: the current ServletContext(advanced use only)
*  
*  As with DMI in general, be aware that if you write scriptlets that depend upon these variables,
*  you preclude your DataSource from being used in the widest possible variety of
*  circumstances. For example, adding a scriptlet that relies on the HttpSession
*  prevents your DataSource from being used in a command-line process.
*  
*  Note that if a dataSource configuration has both a &lt;script&gt; block and
*  a specified OperationBinding.serverObject for some operation, the
*  script block will be executed, and the serverObject ignored.
* @type {string}
* @default null
*/
script?: string;

/**
* Applicable to server-side REST DataSources only
*  
*  A scriptlet to be executed on the server after data has been fetched from the REST service
*  by this operation, but before it is processed through 
*  templating. See 
*  DataSource.transformRawResponseScript for further details.
*  
*  Note, unlike many OperationBinding-level properties, a transformRawResponseScript
*  at the OperationBinding level does not hide a transformRawResponseScript defined 
*  at the DataSource level. Instead, if you define transformRawResponseScript against
*  both the DataSource and the OperationBinding, both are run - first the 
*  DataSource-level script, then the OperationBinding-level one.
* @type {string}
* @default null
*/
transformRawResponseScript?: string;

/**
* Only applicable to "sql" dataSources. This property should be set to true if
*  your operation runs Data Definition Language (DDL) statements rather than 
*  Data Manipulation Language (DML) statements. DDL statements are concerned with 
*  creating, altering and dropping database objects, rather than fetching and updating data.
*  Some examples of DDL statements: 
*   CREATE VIEW myView ...
*   ALTER TABLE myTable DROP CONSTRAINT PK_myTable
*   DROP DATABASE xyz
*   TRUNCATE TABLE myTable
*  
*  When isDDL is true, we switch off certain JDBC flags to inhibit the return of
*  generated key information. This generated key information is very important in supporting
*  SmartClient's automatic cache-synchronization features, but it is never needed for DDL 
*  statements. In fact, the concept of generated keys makes no sense in the context of DDL, 
*  and asking for this information anyway causes broken behavior in some cases, with some 
*  databases (PostgreSQL and Microsoft SQL Server are the two products known to exhibit problems
*  at the time of writing, but there may be others)
*  
*  The default value of null is equivalent to false, and means that
*  this is an ordinary DML (data fetch or update) operation.
* @type {boolean}
* @default null
*/
isDDL?: boolean;

/**
* Name of the web service operation that will be invoked in order to execute this DataSource
*  operation.
*  
*  Valid only for a DataSource that has a WebService (DataSource.serviceNamespace is
*  set). Otherwise, use OperationBinding.dataURL.
*  
*  Setting wsOperation means that DSRequest.data will be serialized as the
*  request message for the specified web service operation, with namespacing and soap encoding
*  handled automatically. See DataSource.transformRequest for how to customize what
*  data is sent to the server.
* @type {string}
* @default null
*/
wsOperation?: string;

/**
* URL to contact to fulfill DSRequests for this operationBinding.
*  
*  dataURL is typically set as DataSource.dataURL rather than on each individual
*  operationBinding.
*  
*  dataURL can be omitted for a DataSource using a Web Service
*  (DataSource.serviceNamespace is set).
* @type {URL}
* @default null
*/
dataURL?: URL;

/**
* Controls the format in which inputs are sent to the dataURL.
*  
*  When a DataSource operation such as fetchData() is invoked on this DataSource or a component
*  bound to this DataSource, the data passed to the operation, if any, will be sent to the
*  dataURL. The dataProtocol property controls the format in
*  which the data is sent: SOAP message, HTTP GET or POST of parameters, etc.
*  
*  The dataProtocol property need not be set for a DataSource with a WebService
*  ( DataSource.serviceNamespace is set), in this case, SOAP messaging is used by
*  default.
*  
*  Developers may completely bypass the SmartClient comm system by setting dataProtocol to
*  "clientCustom". In this case SmartClient will not attempt to send any data
*  to the server after calling DataSource.transformRequest. Instead the developer is
*  expected to implement transformRequest() such that it performs the necessary
*  data action outside of SmartClient, and then calls DataSource.processResponse, 
*  passing in the DSRequest.requestId and an appropriate set of DSResponse properties to
*  indicate the result of the action.
*  
*  NOTE: when OperationBinding.dataFormat is "iscServer", dataProtocol is not consulted.
*  Instead, SmartClient uses a proprietary wire format to communicate with the SmartClient
*  server, and the server-side DSRequest and DSResponse objects should be used to access
*  request data and form responses.
* @type {DSProtocol}
* @default "getParams"
*/
dataProtocol?: DSProtocol;

/**
* Format for response data for this operation.
*  
*  Typically set once for the DataSource as a whole via DataSource.dataFormat.
* @type {DSDataFormat}
* @default "iscServer"
*/
dataFormat?: DSDataFormat;

/**
* Transport to use for this operation. Defaults to DataSource.dataTransport, which in
*  turn defaults to RPCManager.defaultTransport. This would typically only be set to
*  enable "scriptInclude" transport for contacting DataSource.dataFormat web
*  services hosted on servers other than the origin server.
*  
*  When using the "scriptInclude" transport, be sure to set DataSource.callbackParam or
*  OperationBinding.callbackParam to match the name of the query parameter name expected
*  by your JSON service provider.
* @type {RPCTransport}
* @default "xmlHttpRequest"
*/
dataTransport?: RPCTransport;

/**
* Whether to use the RPCManager.sendProxied servlet to send requests
*  described by this operationBinding. If unset, automatically detects whether using the
*  HttpProxy is necessary based on the same-origin policy.
*  
*  Valid only with OperationBinding.dataProtocol settings other than ISCServer.
* @type {boolean}
* @default null
*/
useHttpProxy?: boolean;

/**
* Applies only to dataFormat: "json". Specifies the name of the query parameter that
*  tells your JSON service what function to call as part of the response for this operation.
*  
*  Typically set once for the DataSource as a whole via DataSource.callbackParam.
* @type {string}
* @default "callback"
*/
callbackParam?: string;

/**
* Additional properties to pass through to the DSRequest created for this operation.
*  Note that these will be cumulative with and will override on a per-property basis any
*  properties set via DataSource.requestProperties.
*  
*  These properties are applied before DataSource.transformRequest is called.
* @type {Partial<DSRequest>}
* @default null
*/
requestProperties?: Partial<DSRequest>;

/**
* Configures DataSource.preventHTTPCaching on a per-operationType basis.
* @type {boolean}
* @default null
*/
preventHTTPCaching?: boolean;

/**
* HTTP parameters that should be submitted with every DSRequest.
*  
*  Useful for authenticated services that require a sessionId with every request.
*  
*  Can be set for all operations of a given DataSource as DataSource.defaultParams.
* @type {object}
* @default null
*/
defaultParams?: object;

/**
* Setting useFlatFields on an operationBinding is equivalent to setting
*  DSRequest.useFlatFields on all DataSource requests with the same OperationBinding.operationType
*  as this operationBinding.
*  
*  Typical usage is to combine operationBinding.useFlatFields with 
*  DataBoundComponent.useFlatFields, with the SearchForm
*  bound to the WebService.getInputDS of the web service operation set
*  as OperationBinding.wsOperation. This allows gratuitous nesting to be consistently
*  bypassed in both the user presentation and in the actual XML messaging.
*  
*  Note that useFlatFields is not generally recommended for use with input messages
*  where multiple simple type fields exist with the same name, however if used in this way,
*  the first field to use a given name wins. "first" means the first field encountered in a
*  depth first search. "wins" means only the first field will be available in data binding,
*  and only the first field will be populated in the generated XML message.
* @type {boolean}
* @default false
*/
useFlatFields?: boolean;

/**
* A mapping from field names to SummaryFunction to be applied
*  to each field.
*  
*  Valid only for an operation of type "fetch". See the 
*  serverSummaries for examples of usage.
* @type {object}
* @default null
*/
summaryFunctions?: object;

/**
* List of fields to group by when using serverSummaries.
*  
*  Valid only for an operation of type "fetch". See the 
*  serverSummaries for details and examples of usage.
* @type {Array<Partial<string>>}
* @default null
*/
groupBy?: Array<Partial<string>>;

/**
* For an XML or JSON DataSource, XPath expression used to retrieve the objects that will
*  become DataSource records.
*  
*  For example, an "ItemSearch" web service might return a "Results" structure containing
*  metadata along with the set of Items that one might want to display in a grid. An XPath
*  expression like "/Results/Items" could be used to retrieve just the Items, which would then
*  become DataSource records.
*  
*  For a JSON web service, the recordXPath is applied to the returned JSON data
*  via XMLTools.selectObjects. Only limited XPath syntax is allowed; see 
*  XMLTools.selectObjects for details.
*  
*  For processing XML results, see OperationBinding.xmlNamespaces for information
*  on the namespaces that are available in this XPath expression. If you are contacting a WSDL
*  web service, note that OperationBinding.recordName is an alternative way to specify
*  which records should be selected by their tagName or type, and this is usually simpler.
*  
*  To learn about XPath, try the following search:
*  http://www.google.com/search?q=xpath+tutorial
* @type {XPathExpression}
* @default null
*/
recordXPath?: XPathExpression;

/**
* For an XML DataSource, tagName of the elements to be used as records.
*  
*  This is a simple alternative to OperationBinding.recordXPath when the elements to be used as
*  records all share a tagName.
*  
*  When a DataSource has a WebService, recordName can also be set to the name
*  of any complexType declared within the WebService's WSDL file.
* @type {string}
* @default null
*/
recordName?: string;

/**
* For a DataSource contacting a DataSource.serviceNamespace, setting
*  this flag means the DataSource doesn't actually attempt to contact the server but generates
*  a sample response instead, based on the XML Schema of the response message embedded in the
*  WSDL.
*  
*  The spoofed response will include all complexType elements and will fill in appropriate
*  values by type for all simpleType elements, although the spoofed data will not conform to
*  all xs:restriction declarations (eg xs:pattern).
*  
*  Note that if your WSDL does not fully describe the response format (some WSDL services just
*  have a placeholder &lt;xs:any&gt; element), SmartClient can only produce a partial
*  response. To use a hand-generated sample response, just save an XML file to disk and use
*  the OperationBinding.dataURL setting to point to it.
* @type {boolean}
* @default false
*/
spoofResponses?: boolean;

/**
* Optional object declaring namespace prefixes for use in OperationBinding.recordXPath and
*  DataSourceField.valueXPath XPath expressions.
*  
*  xmlNamespaces should be specified as a mapping from namespace prefix to
*  namespace URI, for example:
*  
*   xmlNamespaces : {
*     az : "http://webservices.amazon.com/AWSECommerceService/2005-03-23"
*   }
*  
*  By default, all namespaces declared on the document element (outermost element of the
*  response) are made available with the prefix used in the document itself.
*  
*  Then, for non-WSDL-described XML results, if there is a default namespace on the document
*  element, it is made available with the special prefix "default".
*  
*  For results of WSDL-described operations, the prefix "service" means the service namespace,
*  that is, the "targetNamespace" on the &lt;definitions&gt; element from the WSDL file. The prefix
*  "schema" means the namespace of the outermost element in the output message for the current
*  operation. "default" will be the schema namespace if there is one, otherwise the service
*  namespace.
*  
*  For basic information on XML Namespaces and their use in XPath, try the following search:
*  http://www.google.com/search?q=XPath+xml+namespaces
* @type {object}
* @default null
*/
xmlNamespaces?: object;

/**
* Optional schema describing how to extract DataSource records from the XML elements selected.
*  
*  Once a set of XML elements have been selected via recordXPath or
*  recordName, those elements are normally transformed to JavaScript objects
*  using the fields of the DataSource that owns the operationBinding. A
*  responseDataSchema can be specified instead if the XML differs in some way between
*  different DataSource operations, such that different values for
*  DataSourceField.valueXPath may be necessary to extract the same
*  DataSource record from slightly different XML structures.
* @type {DataSource}
* @default null
*/
responseDataSchema?: DataSource;

/**
* When set, causes the results of the DataSource Operation to be exported to a file, whose 
*  name and format are indicated by OperationBinding.exportFilename and 
*  OperationBinding.exportAs respectively. When no exportFilename is provided, the 
*  default is Results and the default value of exportAs is csv. Once the Operation 
*  completes, DSRequest.exportDisplay specifies whether the 
*  exported data will be downloaded to the file-system or displayed in a new window.  The 
*  default value of exportDisplay is "download" which displays the Save As dialog. See 
*  ExportDisplay for more information.
*  
*  The export field-list can also be configured, see DSRequest.exportFields.
*  
*  You can also configure the style of line-breaks to use when generating the output. See
*  LineBreakStyle for more information.
*  
*  As well as setting this and other properties on the OperationBinding, Exports can be 
*  initiated in two other ways. You can set properties on the dsRequest by passing 
*  requestProperties into DataSource.exportData. Note that this method does
*  not support exporting to JSON format (see 
*  this post for more detail).
*  Additionally, custom server code may set export-related properties on the 
*  DSResponse.
*  
*  Format Examples
*  
*  XML format
*  
*    &lt;List&gt;
*      &lt;Object&gt;
*        &lt;id&gt;10101&lt;/id&gt;
*        &lt;displayName&gt;Record 10101&lt;/displayName&gt;
*      &lt;/Object&gt;
*   &lt;/List&gt;
*  
*  JSON Format
*  
*    [
*      { id: 10101, displayName: "Record 10101" }
*    ]
*  
*  CSV Format
*  
*    id,displayName
*    10101,"Record 10101"
*  
* @type {boolean}
* @default false
*/
exportResults?: boolean;

/**
* The format in which the data should be exported. Default is "csv".
*  See ExportFormat for more information.
* @type {ExportFormat}
* @default "csv"
*/
exportAs?: ExportFormat;

/**
* The name of the file to save the exported data into.
* @type {string}
* @default null
*/
exportFilename?: string;

/**
* The style of line-breaks to use in the exported output. See LineBreakStyle for
*  more information.
* @type {string}
* @default null
*/
lineBreakStyle?: string;

/**
* The list of field-names to export. If provided, the field-list in the exported output is 
*  limited and sorted as per the list.
*  
*  If exportFields is not provided, the exported output includes all visible fields 
*  from the DataSource (field.hidden=false), sorted in the order they're defined.
* @type {Array<any>}
* @default null
*/
exportFields?: Array<any>;

/**
* If true, causes requests against this operation to automatically start or join a transaction.
*  if false, causes requests against this operation to be committed individually. If null, 
*  falls back to DataSource.autoJoinTransactions.
*  
*  See DataSource.autoJoinTransactions for further details of SmartClient's automatic
*  transaction control.
* @type {boolean}
* @default null
*/
autoJoinTransactions?: boolean;

/**
* Sets or clears the useSpringTransaction flag for this specific operation.
*  
*  See DataSource.useSpringTransaction for details of the Spring transaction integration
*  feature
* @type {boolean}
* @default null
*/
useSpringTransaction?: boolean;

/**
* This setting makes difference if DataSource.useAnsiJoins are in use.
*  If true, causes ansi joins to be included in DefaultQueryClause,
*  otherwise ansi joins will be put into separate DefaultQueryClause.
*  
*  If omitted, system-wide sql.includeAnsiJoinsInTableClause setting from
*  server.properties will be used. If it is missing as well, false
*  is the default.
* @type {boolean}
* @default null
*/
includeAnsiJoinsInTableClause?: boolean;

/**
* This feature is available with Power or better licenses only. See 
*  smartclient.com/product for details.
*  
*  For a dataSource of DataSource.serverType "sql", this 
*  property can be specified on an operationBinding to provide the server with a bespoke
*  SELECT clause to use when constructing the SQL query to perform this operation. The
*  property should be a comma-separated list of column names and/or expressions, and you 
*  can refer to any scalar function supported by the underlying database. The server will 
*  insert the text of this property immediately after the "SELECT" token.
*  
*  Note that if you also specify a OperationBinding.groupClause, you can use aggregate 
*  functions such as SUM and COUNT in the selectClause.
*  
*  This property is only applicable to operationBindings of 
*  OperationBinding.operationType "fetch".
*  
*  See the documentation for OperationBinding.customSQL for usage examples
* @type {VelocityExpression}
* @default null
*/
selectClause?: VelocityExpression;

/**
* This feature is available with Power or better licenses only. See 
*  smartclient.com/product for details.
*  
*  For a dataSource of DataSource.serverType "sql", this 
*  property can be specified on an operationBinding to provide the server with a bespoke
*  table clause to use when constructing the SQL query to perform this operation. The
*  property should be a comma-separated list of tables and views, and you can use any
*  special language constructs supported by the underlying database. The server will 
*  insert the text of this property immediately after the "FROM" token.
*  
*  See the documentation for OperationBinding.customSQL for usage examples
* @type {VelocityExpression}
* @default null
*/
tableClause?: VelocityExpression;

/**
* This feature is available with Power or better licenses only. See
*  smartclient.com/product for details.
*  
*  For a dataSource of DataSource.serverType "sql", this
*  property can be specified on an operationBinding to provide the server with a bespoke
*  ANSI-style joins clause to use when constructing the SQL query to perform this operation.
*  The property should be a set of joins implemented with JOIN directives (as opposed to
*  additional join expressions in the where clause), joining related tables to the
*  main table or view defined in OperationBinding.tableClause.
*  The server will insert the text of this property immediately after the
*  OperationBinding.tableClause.
*  
*  See the documentation for OperationBinding.customSQL for usage examples
* @type {VelocityExpression}
* @default null
*/
ansiJoinClause?: VelocityExpression;

/**
* This feature is available with Power or better licenses only. See 
*  smartclient.com/product for details.
*  
*  For a dataSource of DataSource.serverType "sql", this 
*  property can be specified on an operationBinding to provide the server with a bespoke
*  WHERE clause to use when constructing the SQL query to perform this operation. The
*  property should be a valid expression in the syntax of the underlying database. The 
*  server will insert the text of this property immediately after the "WHERE" token.
*  
*  You may find the SmartClient-provided $criteria variable of particular use with
*  this property.
*  
*  See the documentation for OperationBinding.customSQL for usage examples
* @type {VelocityExpression}
* @default null
*/
whereClause?: VelocityExpression;

/**
* This feature is available with Power or better licenses only. See 
*  smartclient.com/product for details.
*  
*  For a dataSource of DataSource.serverType "sql", this 
*  property can be specified on an operationBinding to provide the server with a bespoke
*  GROUP BY clause to use when constructing the SQL query to perform this operation. The
*  property should be a comma-separated list of column names and/or expressions, forming
*  a valid GROUP BY clause in the syntax of the underlying database. The server will 
*  insert the text of this property immediately after the "GROUP BY" token.
*  
*  Note that specifying this property enables you to use aggregate functions (such as 
*  COUNT and SUM) in your OperationBinding.selectClause. Also note that care is required 
*  when using groupClause to ensure that the selectClause contains the fields you are grouping
*  by. Failure to do this correctly will result in a runtime SQL error.
*  
*  This property is only applicable to operationBindings of 
*  OperationBinding.operationType "fetch".
*  
*  See the documentation for OperationBinding.customSQL for usage examples
* @type {VelocityExpression}
* @default null
*/
groupClause?: VelocityExpression;

/**
* This feature is available with Power or better licenses only. See
*  smartclient.com/product for details.
*  
*  For a dataSource of DataSource.serverType "sql", this
*  property can be specified on an operationBinding to provide the server with a bespoke
*  HAVING clause to use when constructing the SQL query to perform this operation. The
*  property should be a valid expression in the syntax of the underlying database. The
*  server will insert the text of this property immediately after the outer "HAVING" token:
*  
*   select &lt;selectClause&gt;
*    from ...
*   where ...
*  group by &lt;groupClause&gt;
*   having &lt;groupWhereClause&gt;
*  
*  or after outer "WHERE" token if OperationBinding.useHavingClause was set to false:
*  
*  select *
*  from (select &lt;selectClause&gt; from ... where ... group by &lt;groupClause&gt;) work
*  where &lt;groupWhereClause&gt;
*  
*  Note that care is required when using groupWhereClause to ensure that the selectClause contains
*  all the the fields you are filtering by. Failure to do this correctly will result in a runtime SQL error.
*  
*  This property is only applicable to operationBindings of OperationBinding.operationType
*  "fetch".
*  
*  It fully supports OperationBinding.customSQL feature, for example:
*  
*  &lt;groupWhereClause&gt;$defaultGroupWhereClause AND field_name = value &lt;/groupWhereClause&gt;
*  
*  See the documentation for OperationBinding.customSQL for more details and
*  serverSummaries for how criteria is applied to aggregated fields. Also, see "Filtered summaries",
*  "Aggregation & Custom SQL" and "Custom Aggregation Query"
*  Server Summaries samples
*  in Smartclient Showcase.
*  
*  "Standalone" groupWhereClause usage
*  
*  Normally groupWhereClause is used in the context of the Server Summaries feature, but it is possible to use it
*  in non aggregated fetches as well. For example:
*  
*  &lt;DataSource ID="order" serverType="sql" tableName="order"&gt;
*    &lt;fields&gt;
*      &lt;field name="orderID" primaryKey="true" ... /&gt;
*      ...
*    &lt;/fields&gt;
*    &lt;operationBinding operationType="fetch"&gt;
*      &lt;tableClause&gt;$defaultTableClause JOIN orderItem oi on order.orderID = oi.orderID&lt;/tableClause&gt;
*      &lt;groupClause&gt;$defaultSelectClause&lt;/groupClause&gt;
*      &lt;groupWhereClause&gt;count(oi.orderID) &gt; 1 AND sum(oi.unitPrice * oi.quantity) &gt; 100&lt;/groupWhereClause&gt;
*    &lt;operationBinding&gt;
*  &lt;/DataSource>
*  
*  There's no builtin aggregation used in operationBinding above, but the regular "fetch" is limited by
*  requiring every order to have multiple order items with amount greater than 100.
* @type {VelocityExpression}
* @default null
*/
groupWhereClause?: VelocityExpression;

/**
* This feature is available with Power or better licenses only. See 
*  smartclient.com/product for details.
*  
*  For a dataSource of DataSource.serverType "sql", this 
*  property can be specified on an operationBinding to provide the server with a bespoke
*  ORDER BY clause to use when constructing the SQL query to perform this operation. The
*  property should be a comma-separated list of column names and/or expressions, forming
*  a valid ORDER BY clause in the syntax of the underlying database. The server will 
*  insert the text of this property immediately after the "ORDER BY" token.
*  
*  This property is only applicable to operationBindings of 
*  OperationBinding.operationType "fetch".
*  
*  See the documentation for OperationBinding.customSQL for usage examples
* @type {VelocityExpression}
* @default null
*/
orderClause?: VelocityExpression;

/**
* This feature is available with Power or better licenses only. See 
*  smartclient.com/product for details.
*  
*  For a dataSource of DataSource.serverType "sql", this 
*  property can be specified on an operationBinding to provide the server with a bespoke
*  set of values to add or update, for use when constructing the SQL query to perform this 
*  operation. The property should be one of the following, depending on the 
*  OperationBinding.operationType:
*  
*  For "add" operations, the syntax that would be valid for an INSERT INTO query: a 
*  comma-separated list of column names enclosed in parentheses, followed by a comma-separated
*  list of new values, enclosed in parentheses and preceded by the token "VALUES". For example:
*  
*  &lt;valuesClause&gt;(name, age) VALUES("Jane Doe", 48)&lt;/valuesClause&gt;
*  
*  For "update" operations, the syntax that would be valid for an UPDATE query: a 
*  comma-separated list of expressions equating a column name to its new value. For example:
*  
*  &lt;valuesClause&gt;name="Jane Doe", age=48&lt;/valuesClause&gt;
*  
*  You may find the SmartClient-provided $values variable of particular use with
*  this property.
*  
*  See the documentation for OperationBinding.customSQL for usage examples
* @type {VelocityExpression}
* @default null
*/
valuesClause?: VelocityExpression;

/**
* If set to "true" all criteria are applied before aggregation and and the
*  OperationBinding.groupWhereClause is not generated.
*  
*  This property is only applicable if you are using the SQL DataSource, the other built-in types
*  (Hibernate and JPA/JPA2) always perform filtering before summarization.
*  
*  NOTE this property exists principally for backcompat, as the results of apply criteria before
*  aggregation are usually senseless, and in this mode client-side filtering will not match server-side
*  filtering (since client-side filtering is always post-aggregation), so the ability to filter on those
*  fields would need to be disabled in the UI.
*  
*  Still, if you have this flag was set to "true" and the "avg" function is being applied to a "price" field,
*  criteria like "price &lt; 5" will eliminate records where price is less than 5 before the average
*  price is calculated. Note that this means that client-side filtering may not work as expected with
*  summarized results: client-side filter criteria are necessarily applied after summary functions
*  have been applied, so may not match the server's behavior. You can set ResultSet.useClientFiltering
*  to disable client-side filtering on a grid via ListGrid.dataProperties. Or individual fields can be marked
*  ListGridField.canFilter.
*  
*  Note that using customQuerying is an ultimate approach to make all kinds of modifications to the way
*  SQL query is generated and with new $sql.partialWhere(fieldNames), $sql.whereWithout(fieldNames),
*  $sql.partialHaving(fieldNames) and $sql.havingWithout(fieldNames) velocitySupport
*  APIs you have complete control over the automatically generated criteria.
* @type {boolean}
* @default null
*/
applyCriteriaBeforeAggregation?: boolean;

/**
* This setting switches between "having" clause and sub-select approaches when generating aggregated SQL queries.
*  It also affects how aggregated fields are referred in:
*  
*   generated OperationBinding.groupWhereClause
*   expressions generated by SQLDataSource.getPartialHaving(...) and
*  SQLDataSource.getHavingWithout(...) server-side API
*   expressions generated by $sql.partialHaving(...) and $sql.havingWithout(...)
*  velocitySupport variables used in customQuerying
*  
*  useHavingClause=true (default)
*  SQL query is generated using traditional "having" clause approach:
*  
*   select &lt;selectClause&gt;
*    from ...
*   where ...
*  group by &lt;groupClause&gt;
*   having &lt;groupWhereClause&gt;
*  
*  Aggregated fields are referred by SQL expressions:
*  
*   select customer, sum(quantity * price) as amount
*    from ...
*   where ...
*  group by customer
*   having sum(quantity * price) &gt; 10
*  
*  useHavingClause=false
*  SQL query is generated using sub-select approach when main query becomes sub-select and then it is
*  filtered in outer "where" clause:
*  
*  select *
*  from (select &lt;selectClause&gt; from ... where ... group by &lt;groupClause&gt;) work
*  where &lt;groupWhereClause&gt;
*  
*  Aggregated fields are referred by aliases they are selected by:
*  
*   select *
*    from (select customer, sum(quantity * price) as amount
*        from ...
*       where ...
*      group by customer) work
*   where amount &gt; 10
*  
*  If operationBinding.useHavingClause is omitted this may be controlled by
*  sql.MyDatabase.useHavingClause setting in sqlSettings on a per-database basis.
* @type {boolean}
* @default null
*/
useHavingClause?: boolean;

/**
* This feature is available with Power or better licenses only. See 
*  smartclient.com/product for details.
*  
*  For a dataSource of DataSource.serverType "sql", "hibernate" or "jpa", this 
*  property can be specified on an operationBinding to indicate that the server should run 
*  user-specified SQL, rather than the query it would normally generate to satisfy a 
*  dataSource operation. This property allows you to provide a fully-customized query.
*  
*  Hibernate and JPA dataSources also support customizing the query via
*  OperationBinding.customHQL or OperationBinding.customJQL respectively.
*  
*  For SQL dataSources, as an alternative to customSQL you can provide 
*  custom "pieces" to the query generator, via properties such OperationBinding.whereClause
*  and OperationBinding.valuesClause.
*  This feature is not available for Hibernate or JPA DataSources.
*  See the customQuerying for more details.
*  
*  For a dataSource of type "sql", the SmartClient server generates a number of useful
*  DefaultQueryClause, 
*  and makes them available to your custom SQL code via the Velocity templating 
*  language (note that this is not available for "hibernate" dataSources).
*  
*  We also make the template variables $criteria and $values available, to give
*  you direct access to the supplied criteria, and to the new field values for update and add
*  operations. These variables are available to both "sql" and "hibernate" dataSources.
*  
*  Note that you should use this feature with care. In particular, writing customSQL code
*  that makes use of a particular database engine's features or syntax will make your 
*  application less portable. 
*  
*  See customQuerying for an overview of writing custom queries and clauses.
*  
*  Examples
*  An example using the SmartClient-supplied query pieces. This custom query will give 
*  exactly the same result as the SmartClient-generated query:
*  
*  
*  &lt;operationBinding operationId="customFetch" operationType="fetch"&gt;
*  &nbsp;&nbsp;&lt;customSQL&gt;
*  &nbsp;&nbsp;&nbsp;&nbsp;SELECT $defaultSelectClause FROM $defaultTableClause
*  WHERE $defaultWhereClause ORDER BY $defaultOrderClause
*  &nbsp;&nbsp;&lt;/customSQL&gt;
*  &lt;/operationBinding&gt;
*  
*  
*  An example using the SmartClient-supplied $criteria template variable:
*  
*  
*  &lt;operationBinding operationId="customFetch" operationType="fetch"&gt;
*  &nbsp;&nbsp;&lt;customSQL&gt;
*  &nbsp;&nbsp;&nbsp;&nbsp;SELECT foo, bar, baz FROM MyTable WHERE bar &gt; $criteria.someValue
*  &nbsp;&nbsp;&lt;/customSQL&gt;
*  &lt;/operationBinding&gt;
*  
*  
*  An update example:
*  
*  
*  &lt;operationBinding operationId="myUpdateOp" operationType="update"&gt;
*  &nbsp;&nbsp;&lt;customSQL&gt;
*  &nbsp;&nbsp;&nbsp;&nbsp;UPDATE $defaultTableClause SET $defaultValuesClause WHERE bar 
*  &lt;= $criteria.someValue
*  &nbsp;&nbsp;&lt;/customSQL&gt;
*  &lt;/operationBinding&gt;
*  
* @type {VelocityExpression}
* @default null
*/
customSQL?: VelocityExpression;

/**
* This feature is available with Power or better licenses only. See 
*  smartclient.com/product for details.
*  
*  For a dataSource of DataSource.serverType "hibernate", this 
*  property can be specified on an operationBinding to indicate that the server should run 
*  user-specified HQL (Hibernate Query Language), rather than the Hibernate criteria query or 
*  saveOrUpdate call it would normally generate to satisfy a dataSource operation
*  via Hibernate. 
*  
*  Note that inserting new records via HQL is often impractical, due to intentional restrictions
*  in the language (it is only possible to perform an insert expressed in terms of a SELECT; the
*  "VALUES" construct commonly used when inserting new rows singly is not supported). If you are 
*  intending to use customHQL, we recommend that you avoid doing so for OperationBindings 
*  with OperationBinding.operationType "add", unless you have a special 
*  requirement such as a bulk insert; if you need custom queries to perform inserts 
*  on "hibernate" dataSources, we recommend you use OperationBinding.customSQL,
*  which is valid for "hibernate" DataSources as well as "sql" dataSources.
*  
*  For other operations on "hibernate" dataSources, however, HQL has the advantage of 
*  being more portable across different database engines than is plain SQL.
*  
*  Note that using customHQL affects paging implementation. If you use it, full data set is fetched 
*  from Hibernate and records that aren't in the requested range are dropped at the server side.
* @type {string}
* @default null
*/
customHQL?: string;

/**
* This feature is available with Power or better licenses only. See
*  smartclient.com/product for details.
*  
*  For a dataSource of DataSource.serverType "jpa", this
*  property can be specified on an operationBinding to indicate that the server should run
*  user-specified JQL (Java Persistence Query Language).
*  
*  Note that inserting new records via JQL is often impractical, due to intentional restrictions
*  in the language (it is only possible to perform an insert expressed in terms of a SELECT; the
*  "VALUES" construct commonly used when inserting new rows singly is not supported). If you are
*  intending to use customJQL, we recommend that you avoid doing so for OperationBindings
*  with OperationBinding.operationType "add", unless you have a special
*  requirement such as a bulk insert; if you need custom queries to perform inserts
*  on "jpa" dataSources, we recommend you use OperationBinding.customSQL,
*  which is valid for "jpa" DataSources as well as "sql" dataSources.
*  
*  For other operations on "jpa" dataSources, however, JQL has the advantage of
*  being more portable across different database engines than is plain SQL.
*  
*  Note that using customJQL affects paging implementation. If you use it, full data set is fetched
*  from JPA and records that aren't in the requested range are dropped at the server side.
* @type {string}
* @default null
*/
customJQL?: string;

/**
* This feature is available with Power or better licenses only. See
*  smartclient.com/product for details.
*  
*  For a dataSource of DataSource.serverType "jpa" or "hibernate", this
*  property can be specified on an operationBinding to indicate that the server should execute
*  a named query which has already been defined on an entity.
*  
*   &#64;Entity
*   &#64;Table (name="Countries")
*   &#64;NamedQuery(name = "Country.withPopulationLessThan", query = "SELECT country FROM Country country WHERE country.population &lt; :population")
*   public class Country { ... }
*  
* 
*  
*   &lt;operationBindings&gt;
*     &lt;operationBinding operationType="custom" operationId="withPopulationLessThan" namedQuery="Country.withPopulationLessThan"/&gt;
*   &lt;/operationBindings&gt;
*  
*  Substitution values can be used in order to build more dynamic named queries. When calling
*  DataSource.performCustomOperation the values are passed in using the data argument.
*  
*  Note that value substitution for named queries is slightly different to other custom queries.
*  Because of the way the persistence API works the JQL query written in the &#64;NamedQuery annotation
*  can only contain basic parameter names such as "population". Therefore the value substitution
*  becomes a simple name based mapping.
*  
*  Examples
*  Using Simple Criteria
*  
*  An example using a simple criteria for the above defined Country entity. In this case the named
*  query parameter ":population" will be swapped out for the value of the criteria objects "population"
*  field.
*  
*   var criteria = {
*     population: 596000
*   };
* 
*   countryDataSource.performCustomOperation("withPopulationLessThan", criteria);
*  
*  
*  Using Advanced Criteria
*  
*  If an advanced criteria is detected, access to all "fieldName" variables and their values will
*  be provided but still using simple name based mapping. In the below case only the deep-first
*  occurrence of the "population" fieldName will available. The operator is effectively ignored.
*  
*   var criteria = {
*     _constructor: "AdvancedCriteria",
*     operator:"or",
*     criteria:[
*       {
*         fieldName:"population",
*         operator:"lessThan",
*         value: 12000
*       },
*       {
*         fieldName:"name",
*         operator:"equals",
*         value: "Sweden"
*       },
*       {
*         _constructor: "AdvancedCriteria",
*         operator:"and",
*         criteria:[
*           {
*             fieldName:"population",
*             operator:"lessThan",
*             value: 0
*           }
*         ]
*       }
*     ]
*   };
* 
*   countryDataSource.performCustomOperation("withPopulationLessThan", criteria);
*  
*  
*  Note
*  Using namedQuery affects paging implementation. If you use it, full data set is fetched
*  from JPA and records that aren't in the requested range are dropped at the server side.
* @type {string}
* @default null
*/
namedQuery?: string;

/**
* A SQLDataSource will normally issue two queries for a "fetch" operation when paging is
*  enabled: one to determine the total rows available (the "row count query"), and one to fetch
*  the specific range of rows requested.
*  
*  Setting skipRowCount="true" will avoid the "row count query", but as a consequence
*  DSResponse.totalRows will be set to match the requested DSRequest.endRow since
*  the totalRows is unknown. You can avoid this by using a 
*  OperationBinding.sqlPaging of "jdbcScroll" or "dropAtServer", but be 
*  aware that these paging strategies can introduce significant delays when used with 
*  potentially large datasets (in fact, "dropAtServer" is almost guaranteed to do so if used
*  with datasets of more than 1000 or so rows)
*  
*  As an alternative, consider enabling 
*  DataSource.progressiveLoading, which avoids doing a query for row
*  counts, but will still allow the user to load more results using the scrollbar if viewing
*  results in a ListGrid.
* @type {boolean}
* @default null
*/
skipRowCount?: boolean;

/**
* Indicates that the listed fields should be included in the default
*  OperationBinding.selectClause and OperationBinding.whereClause generated for this 
*  operationBinding, even if they are marked DataSourceField.customSQL="true".
*  
*  If you need to apply different sets of overrides for the selectClause and the
*  whereClause, use OperationBinding.customValueFields and/or 
*  OperationBinding.customCriteriaFields instead. If you specify both
*  customFields and customCriteriaFields or 
*  customValueFields, the more specific variant wins. If you specify both 
*  customFields and OperationBinding.excludeCriteriaFields, 
*  customFields wins (this is another use case when you may wish to use 
*  customValueFields instead)
*  
*  You can specify this property as a comma-separated list (eg, "foo, bar, baz") or by just
*  repeating the &lt;customFields&gt; tag multiple times with one field each.
*  
*  This property is only applicable to DataSources of DataSource.serverType.
* @type {string | Array<any>}
* @default null
*/
customFields?: string | Array<any>;

/**
* Indicates that the listed fields should be included in the default
*  OperationBinding.selectClause and OperationBinding.valuesClause generated for this
*  operationBinding, even if they are marked DataSourceField.customSQL="true".
*  
*  You can specify this property as a comma-separated list (eg, "foo, bar, baz") or by just
*  repeating the &lt;customValueFields&gt; tag multiple times with one field each.
*  
*  This property is only applicable to DataSources of DataSource.serverType.
* @type {string | Array<any>}
* @default null
*/
customValueFields?: string | Array<any>;

/**
* Indicates that the listed fields should be included in the default
*  OperationBinding.whereClause generated for this operationBinding, even if they are marked 
*  DataSourceField.customSQL.
*  
*  You can specify this property as a comma-separated list (eg, "foo, bar, baz") or by just
*  repeating the &lt;customCriteriaFields&gt; tag multiple times with one field each.
*  
*  This property is only applicable to DataSources of DataSource.serverType.
* @type {string | Array<any>}
* @default null
*/
customCriteriaFields?: string | Array<any>;

/**
* Indicates that the listed fields should be excluded from the default
*  OperationBinding.whereClause generated for this operationBinding. 
*  
*  This enables you to use these fields in a customQuerying while
*  still allowing the $defaultWhereClause to be generated for all other fields. For example,
*  you might take a particular field and apply it in the WHERE clause of a subquery.
*  
*  You can specify this property as a comma-separated list (eg, "foo, bar, baz") or by just
*  repeating the &lt;customCriteriaFields&gt; tag multiple times with one field each.
*  Note that if a field is included in both excludeCriteriaFields and
*  OperationBinding.customCriteriaFields, customCriteriaFields
*  wins.
*  
*  This property is only applicable to DataSources of DataSource.serverType.
* @type {string | Array<any>}
* @default null
*/
excludeCriteriaFields?: string | Array<any>;

/**
* For an operationBinding of OperationBinding.operationType "fetch" which specifies no
*  OperationBinding.operationId, this property determines whether the operationBinding 
*  should be used for cache synchronization purposes (ie, to retrieve the record most recently
*  added or changed). This property has no effect on an operationBinding that specifies an 
*  operationId - see OperationBinding.cacheSyncOperation.
*  
*  In order to work correctly with SmartClient's cache synchronization system, an 
*  operationBinding marked useForCacheSync should have the following properties:
*  
*  
* - Able to complete its retrieval using no context other than the values of the primary key
*  fields declared in the dataSource (these will be provided in the $criteria object passed to 
*  the operation)
*  
* - Returns the entire record, including any values that may require joins to other tables
*  or other complexities
*  
*  This property is only applicable to DataSources of type "sql".
* @type {boolean}
* @default true
*/
useForCacheSync?: boolean;

/**
* For an operationBinding of OperationBinding.operationType "add" or "update", this 
*  property is the OperationBinding.operationId of a "fetch" operationBinding to use 
*  for cache synchronization purposes (ie, to fetch the row most recently added or changed).
*  This property, along with OperationBinding.useForCacheSync and 
*  OperationBinding.canSyncCache is provided so that you can use custom database operations
*  without sacrificing the benefits of SmartClient's automatic cache synchronization.
*  
*  This property is only applicable to DataSources of type "sql".
* @type {string}
* @default null
*/
cacheSyncOperation?: string;

/**
* For an operation of type "add" or "update", a SQLDataSource will normally obtain
*  data to return to the client by performing the "cacheSyncOperation": a SELECT statement that
*  retrieves the modified record by primary key. This accommodates sequence columns, columns
*  with default values, database triggers and other database features that may modify data
*  after insertion or update.
*  
*  Certain major SQL customizations can prevent the SQLDataSource from authoritatively
*  determining the primary key used in the SQL statement, such that re-selecting the saved
*  record may fail. By default, when canSyncCache has not been explicitly
*  set, in the following cases it is assumed that the normal cacheSyncOperation cannot be used:
*  
*  
* - &lt;customSQL&gt; has been used to define an entirely custom query
*  
* - a custom &lt;whereClause&gt; has been defined for an "update" or "remove" operation
*  
* - a custom &lt;valuesClause&gt; has been defined for an "add" operation
*  
*  If any of these cases apply or if canSyncCache has been set false, the server
*  will skip the cacheSyncOperation and return a DSResponse where
*  DSResponse.invalidateCache has been set to true to notify client-side components that
*  they may need to refresh their entire cache.
*  
*  Alternatively, if the default re-selection behavior will not work but a customized SQL
*  query would work, you can define that SQL operation as another operationBinding and use
*  OperationBinding.cacheSyncOperation to declare that it should be used. Setting
*  cacheSyncOperation implicitly sets canCacheSync to true.
*  
*  This property is only applicable to DataSources of type "sql".
* @type {boolean}
* @default null
*/
canSyncCache?: boolean;

/**
* For an operation of type "add" or "update", a SQLDataSource will normally obtain data to
*  return to the client by performing the OperationBinding.cacheSyncOperation. In certain edge cases
*  this is either not possible or causes problems (such as record locks); in this case, you 
*  mark the operation with OperationBinding.canSyncCache: false to inhibit this automatic behavior. 
*  However, this gives you an operation that does not properly cooperate with the client in 
*  keeping client-side caches fresh, which in turn leads to an application that must perform
*  more server turnarounds and explicit database fetches. 
*  
*  It is possible that the data needed for cache synchronization could be obtained by 
*  application code running in a DMI or serverScript, 
*  even if the normal automatic cache synchronization will not work. However, such application
*  code is almost certainly going to need to know the primary key(s) of any newly added 
*  record(s).
*  
*  When makeKeysAvailable is true, SmartClient Server will go through the process
*  of obtaining generated keys in accordance with the SequenceMode, even if 
*  canSyncCache is false (note, "in accordance with the sequenceMode"
*  implies that we do not attempt to get keys if the sequenceMode is "none").
*  The keys used in the operation (both generated keys, if any, and any keys provided in the 
*  operation's criteria or values) will be stored on the DSResponse and your 
*  server-side DMI or script will have access to them via the server API 
*  dsResponse.getKeys(). Please see the server-side documentation for that 
*  method for further details.
*  
*  Note, if you are using SequenceMode "jdbcDriver", the keys are available to SmartClient
*  Server at no cost, so makeKeysAvailable is always true. If you are using
*  sequenceMode "native", a separate native query must be run to obtain the keys. 
*  The overhead of this native query is likely to be insignificant, but because it is an extra
*  step that you may not want or need, makeKeysAvailable defaults to false in
*  this case. You can override this by adding setting 
*  sql.always.makeKeysAvailable:true to your server.properties file. 
*  
*  This property is only applicable to DataSources of type "sql".
* @type {boolean}
* @default "See below"
*/
makeKeysAvailable?: boolean;

/**
* For dataSources of DataSource.serverType "sql" and "hibernate" only, this 
*  property determines whether "custom" operations have their custom SQL or HQL sent to the 
*  underlying database via a JDBC executeQuery() or a JDBC 
*  executeUpdate(). The default value of null means the same as "query", so you
*  only need to use this property when your custom SQL or HQL updates data.
* @type {SQLType}
* @default null
*/
sqlType?: SQLType;

/**
* Ordinarily, "update" and "remove" operations are only allowed for DataSources
*  that have a DataSourceField.primaryKey, and all primary key values
*  are present in the request. This is because an update of a DataSource with no primary key, 
*  or an update request that has missing primary key values, cannot be guaranteed to affect 
*  only one record.
*  
*  Setting this property on an operationBinding circumvents this restriction for that operation
*  only.
*  
*  Warning: Be aware that this is a potentially dangerous setting and should be used
*  with care. With this flag set, you have no guarantee that an update will not change or 
*  remove every row in a table.
*  
*  Note, in the case of doing an update or delete operation with a primary key 
*  and additional criteria, allowMultiUpdate must be set or additional criteria 
*  will be dropped and just the primary key fields will be used in criteria.
*  
*  Also, running allowMultiUpdate operations directly from the client is not 
*  straightforward because it requires the ability to specify criteria and values separately
*  in the request, which is not currently supported. This can be worked around in various
*  ways, but really allowMultiUpdate is primarily intended for server-side 
*  operations. Therefore, the recommended pattern is to use a 
*  DataSource.performCustomOperation from the client to invoke a DMI on
*  the server which performs the multi-update operation via a second, server-side DSRequest. 
*  
*  In any case, it's normally a good idea to set OperationBinding.requiredCriterion 
*  on the multi-update operation to ensure that the alternative criteria is present as expected.
* @type {boolean}
* @default null
*/
allowMultiUpdate?: boolean;

/**
* Ordinarily, "update" and "remove" operations are only allowed if all primary key values
*  are present in the request. This is because an update request that has missing primary 
*  key values cannot be guaranteed to affect only one record.
*  
*  Setting this property on an operationBinding circumvents this restriction for that operation
*  only. Note, this property differs from OperationBinding.allowMultiUpdate in its 
*  intent: allowMultiUpdate tells the framework that this operation deliberately 
*  affects multiple records; providesMissingKeys tells the framework that this 
*  operation will only affect one record, and will ensure this by providing values for missing
*  keys during its operation. Unlike allowMultiUpdate, setting this flag does not
*  cause component caches to be ListGrid.invalidateCache
*  
*  Providing values for missing keys can be done in various ways:
*  
* - Operations that specify &lt;OperationBinding.customSQL&gt; or 
*    &lt;OperationBinding.whereClause&gt; can provide missing
*    key values from session storage or elsewhere in the provided record
*  
* - Operations that specify &lt;serverScript&gt; can provide
*    arbitrary code to manipulate the record in whatever way they like before executing the 
*    underlying built-in functionality
*  
* - Operations can specify &lt;OperationBinding.criteria&gt; 
*    to provide missing keys
*  
* - A request can contain DSRequest.fieldValueExpressions, 
*    which can be used to provide values for missing keys
*  
*  Note, you can also use a regular dmiOverview to arbitrarily manipulate the
*  record sent from the client, including providing values for any missing keys. If you do 
*  this, you do not need to specify providesMissingKeys because the request is 
*  not validated for the presence of key values until after the DMI has run.
*  
*  Warning: Be aware that this is a potentially dangerous setting and should be used
*  with care. With this flag set, the framework cannot guarantee that an update will not 
*  change or remove every row in a table: it becomes your code's responsibility to ensure 
*  that all PK values are provided to the operation by the time it actually needs them.
* @type {boolean}
* @default null
*/
providesMissingKeys?: boolean;

/**
* Specifies, for this specific operationBinding, whether to qualify column names with table
*  names in any SQL we generate. Overrides the DataSource.qualifyColumnNames property.
*  Only applicable to dataSources of DataSource.serverType "sql".
* @type {boolean}
* @default true
*/
qualifyColumnNames?: boolean;

/**
* If set, every invocation of this operationBinding will invalidate the local cache, forcing
*  a server visit to refresh the data.
* @type {boolean}
* @default null
*/
invalidateCache?: boolean;

/**
* Specifies, for this operationBinding only, the list of field names that should be returned
*  to the client. Typically this will be a subset of the DataSource.fields, but note 
*  that this is not a requirement; outputs can include fields that are not defined
*  in the DataSource's field list. In this case, the server will return extra fields even if 
*  DataSource.dropExtraFields is true.
*  
*  You specify this property as a string containing a comma-separated list of field names 
*  (eg, "foo, bar, baz")
* @type {string}
* @default null
*/
outputs?: string;

/**
* The paging strategy to use for this specific OperationBinding. If this property is not
*  set, we fall back to the DataSource.sqlPaging value, and the defaults described in
*  the documentation for that property.
* @type {SQLPagingStrategy}
* @default null
*/
sqlPaging?: SQLPagingStrategy;

/**
* If explicitly set true or false, forces the use of a "hint" in the SQL we generate for paged 
*  queries on or off as appropriate. If not set, defaults to the 
*  DataSource.sqlUsePagingHint value. Note this property is only applicable to 
*  DataSource.serverType DataSources, only when a 
*  DataSource.sqlPaging of "sqlLimit" is in force, and it only has an 
*  effect for those specific database products where we employ a native hint in the generated 
*  SQL in an attempt to improve performance.
* @type {boolean}
* @default null
*/
sqlUsePagingHint?: boolean;

/**
* Whether to use the subselect technique (see DataSource.useSubselectForRowCount for
*  details) to derive a rowcount query for this operation. If this property is not
*  set, we fall back to the useSubselectForRowCount setting on the DataSource, 
*  and the defaults described in the documentation for that property.
* @type {boolean}
* @default null
*/
useSubselectForRowCount?: boolean;

/**
* Sets DataSource.progressiveLoading for this particular 
*  operation, overriding the DataSource-level setting. Note that this setting applies only
*  to fetch operations - it has no effect if specified on any other kind of operation.
* @type {boolean}
* @default null
*/
progressiveLoading?: boolean;

/**
* This property indicates whether this operation supports AdvancedCriteria. This setting 
*  overrides DataSource.allowAdvancedCriteria for this operation only. See
*  DataSource.supportsAdvancedCriteria for further information.
*  
*  NOTE: If you specify this property in a DataSource descriptor 
*  (.ds.xml file), it is enforced on the server. This means that if you run 
*  a request containing AdvancedCriteria against an OperationBinding that advertises itself
*  as allowAdvancedCriteria:false, it will be rejected.
* @type {boolean}
* @default null
*/
allowAdvancedCriteria?: boolean;

/**
* Elements of this feature are only available with Power or better licenses. See 
*  smartclient.com/product for details.
*  
*  A list of DSRequestModifiers that will be used to modify the criteria of each 
*  DSRequest that uses this operationBinding. Note that the criteria elements are 
*  applied to DSRequest criteria as follows:
*  
*  
* - Simple criteria: The field and value are just applied as an extra key/value 
*  pair in the criteria map, as long as the DSRequestModifier.operator 
*  attribute is left unset, or is set to "equals". For any other setting of 
*  operator, the criteria is first converted to the equivalent AdvancedCriteria 
*  and then processed as described below
*  
* - AdvancedCriteria: If the topmost operator is "and", we add the new criterion 
*  as an additional criterion directly in the existing list. Otherwise, we create a new 
*  top-level AdvancedCriteria with an operator of "and". This is then set to have two 
*  elements in its criteria: the previous top-level criteria and the new criterion.
*  
*  The effect of this is to apply any criteria specifed here as additional constraints on top
*  of what the user has specified, and of course, the user is unable to affect this. Thus, 
*  this is a suitable and convenient place to enforce rules such as "Users can only ever see
*  their own records". 
*  Below is an example of the XML as it should be defined in your ds.xml, datasource 
*  definitions. 
*  &lt;operationBindings&gt;
*   &lt;operationBinding operationType="fetch" operationId="..."&gt;
*    &lt;criteria fieldName="USER_ROLE" value="ADMIN" operator="equals" /&gt;
*   &lt;/operationBinding&gt;
*  &lt;/operationBindings&gt;
*  
* @type {Array<Partial<DSRequestModifier>>}
* @default null
*/
criteria?: Array<Partial<DSRequestModifier>>;

/**
* Elements of this feature are only available with Power or better licenses. See 
*  smartclient.com/product for details.
*  
*  A list of DSRequestModifiers that will be used to modify the values object of each 
*  DSRequest that uses this operationBinding. See this example:
*  ${isc.DocUtils.linkForExampleId('queuedAdd')}. 
*  Below example of the xml as it should be defined in ds.xml: 
*  &lt;operationBinding operationType="add"&gt;
*   &lt;values fieldName="orderID" value="$responseData.last('queuedAdd_order','add').orderID" /&gt; 
*  &lt;/operationBinding&gt;
* @type {Array<Partial<DSRequestModifier>>}
* @default null
*/
values?: Array<Partial<DSRequestModifier>>;

/**
* Whether a user must be authenticated in order to access this operation. For details of 
*  what is meant by "authenticated", see DataSource.requiresAuthentication.
*  
*  To protect access to an entire operationType (eg, all "fetch" operations), declare an
*  operationBinding with requiresAuthentication="true", OperationBinding.operationType set
*  to the operationType to be protected, but no OperationBinding.operationId. This will then
*  prevent access to the "fetch" operationType unless another OperationBinding
*  declares requiresAuthentication="false" with a specific
*  OperationBinding.operationId.
* @type {boolean}
* @default null
*/
requiresAuthentication?: boolean;

/**
* Comma-separated list of user roles that are allowed to invoke the operation described by
*  this operationBinding. If the current user has any of the roles listed, they can
*  invoke the operation. Also note that authentication.superuserRole can be 
*  specified in the server_properties file. If set this denotes a "super user" 
*  role - any user with that role will have access to all operations, regardless of the
*  "requiresRole" settings for the operation.
*  
*  Whether the current user has a given role is determined by calling the standard Java
*  servlets method httpServletRequest.isUserInRole(), hence works with both simple
*  J2EE security (realms and form-based authentication) and JAAS (Java Authentication &
*  Authorization Service).
*  
*  If you wish to use a role-based security scheme that does not make use of the servlet API's 
*  standards, SmartClient Server also implements the setAuthenticated and
*  setUserRoles methods on RPCManager. 
*  You can use this API to tell SmartClient that all the requests in the queue currently 
*  being processed are associated with a user who has the roles you supply; in this case,
*  SmartClient will not attempt to resolve the user's roles via 
*  httpServletRequest.isUserInRole(). When taking this approach, the 
*  rpcManager.setUserRoles() method should be called on the server for 
*  each transaction received from the client. We recommend doing this by overriding the 
*  special IDACall servlet and checking server side state to determine the current user's 
*  roles, calling the API, and then calling handleDSRequest() or
*  handleRPCRequest() directly to handle the request(s) passed in.
*  Here's an example of this approach which assumes the current user's roles has been
*  set directly on the HttpSession object as a comma-separated-string attribute "roles":
*  
*  public class SecureIDACall extends IDACall {
*  
*    public void processRequest(HttpServletRequest request,
*        HttpServletResponse response)
*     throws ServletException, IOException
*    {
*      HttpSession session = request.getSession();
*      Object roles = session == null ? null : session.getAttribute("roles");
*  
*      if (roles != null) {
*        try {
*          RequestContext context = RequestContext.instance(this, request, response);  
*          RPCManager rpc = new RPCManager(request, response);
*          rpc.setAuthenticated(true);
*          rpc.setUserRoles((String) roles); 
*          
*          // call processRPCTransaction() to iterate through all RPCRequests and
*          // DSRequests and execute them
*          processRPCTransaction(rpc, context);
*  
*        } catch (Throwable e) {
*          handleError(response, e);
*        }
*      } else {
*        super.processRequest(request, response);
*      } 
*    }
*  }
*  
*  
*  If there is an operationBinding declared for a given operationType which does not have an
*  OperationBinding.operationId, that is, it is the default operationBinding for the type, then any other
*  operationBinding of the same type is assumed to have the same setting for
*  requiresRole as the default operationBinding for the operationType. For
*  example, given these declarations:
*  
*    &lt;operationBinding operationType="fetch" requiresRole="manager"&gt;
*       ... settings ...
*    &lt;/operationBinding&gt;
*    &lt;operationBinding operationType="fetch" operationId="fetchWithExtraFields"&gt;
*       ... settings ...
*    &lt;/operationBinding&gt;
*  
*  The second operationBinding requires the "manager" role even though there is no explicit
*  requiresRole declaration. To prevent the "manager" role being required by the
*  second operationBinding, add requireRole="".
*  
*  Note that if DataSource.requiresRole is set, all operations on the DataSource require
*  the roles set for the DataSource as a whole, even if they declare individual
*  requiresRole attributes.
*  
*  This property is valid only for a server-side DataSource when using the SmartClient Server.
*  
*  Special rules for cache sync
*  
*  After successfull "add" or "update" operation cache sync request is performed, which is using 
*  "fetch" operation of the same datasource. It may happen that user is allowed to add records, but 
*  is not allowed to fetch them, for example:
*  
*    &lt;operationBinding operationType="fetch" requiresRole="admin"&gt;
*       ... settings ...
*    &lt;/operationBinding&gt;
*    &lt;operationBinding operationType="add"&gt;
*       ... settings ...
*    &lt;/operationBinding&gt;
*  
*  User without "admin" role will be able to successfully add record, but the cache sync operation 
*  will fail due to security violation. In this case the record will be saved to database, but the 
*  added record will not be fetched from database, instead just DSRequest.oldValues 
*  overlaid with submitted values will be returned. So, any changes made to the new record during 
*  request execution, including generated values for primary key fields of "sequence" type, will not 
*  be returned to the client.
*  
*  However, if "add" or "update" operation explicitly declares OperationBinding.cacheSyncOperation,
*  cache sync request will be executed even if the user does not meet the security checks for the 
*  operationBinding. Note that field-level security still will be respected and disallowed fields will 
*  be excluded from returned data.
* @type {string}
* @default null
*/
requiresRole?: string;

/**
* Indicates that the specified VelocityExpression must be true for a user to access
*  this operationBinding.
*  
*  As with OperationBinding.requiresRole, if there an operationBinding that is the
*  default operationBinding for the operationType, its requires expression is
*  assumed to apply to all other operationBindings of the same type unless they explicitly set
*  requires=""
*  
*  DataSource.requires, if specified, applies before
*  operationBinding.requires is evaluated. In this case, both requires
*  expressions must be true for the request to be accepted.
* @type {VelocityExpression}
* @default null
*/
requires?: VelocityExpression;

/**
* Indicates that field-level declarative security rules are waived for rows that were 
*  created by the current user, as described in the discussion of 
*  DataSource.creatorOverrides. This setting overrides 
*  dataSource.creatorOverrides, for this operation only.
* @type {boolean}
* @default null
*/
creatorOverrides?: boolean;

/**
* Overrides the same setting at the DataSource.ownerIdField level, for 
*  this operation only. See the dataSource-level documentation for details.
* @type {string}
* @default null
*/
ownerIdField?: string;

/**
* Value to use for the OperationBinding.ownerIdField if no one 
*  has authenticated.
* 
*  Overrides the same setting at the DataSource.guestUserId level.
* @type {string}
* @default null
*/
guestUserId?: string;

/**
* Definition of an email message that will be sent as an after-effect of selecting or updating
*  data.
*  
*  Note that if a fixed number of different messages need to be sent, multiple 
*  &lt;mail&gt; tags may be specified. For example, one mail could be sent to an 
*  admin address, and a different message to every member of a user group.
* @type {Mail}
* @default null
*/
mail?: Mail;





}
/**
* Creates a new OperationBinding
* 
* @param typeCheckedProperties {Partial<OperationBinding>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {OperationBinding} The new instance
*/
class OperationBinding{
static create(typeCheckedProperties? : Partial<OperationBinding>, uncheckedProperties?: Object): OperationBinding;







}

/**
* The Operators class provides a collection of class-level properties that can be used to
*  modify the descriptions associated with the logical Operators used in 
*  AdvancedCriteria. This is primarily intended for internationalization.
*  
*  To add or override an operator's description, use
*  Operators.Class.addClassProperties. For example:
*  
*  Operators.addClassProperties({lessOrEqualTitle: "Less than or equal to"});
*/
interface Operators extends Class {




}
/**
* Creates a new Operators
* 
* @param typeCheckedProperties {Partial<Operators>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Operators} The new instance
*/
class Operators{
static create(typeCheckedProperties? : Partial<Operators>, uncheckedProperties?: Object): Operators;
/**
* Title for the "equals" operator
* @type {string}
* @default "equals (match case)"
*/
static equalsTitle?: string;

/**
* Title for the "iEquals" operator
* @type {string}
* @default "equals"
*/
static iEqualsTitle?: string;

/**
* Title for the "notEqual" operator
* @type {string}
* @default "not equal (match case)"
*/
static notEqualTitle?: string;

/**
* Title for the "iNotEqual" operator
* @type {string}
* @default "not equal"
*/
static iNotEqualTitle?: string;

/**
* Title for the "greaterThan" operator
* @type {string}
* @default "greater than"
*/
static greaterThanTitle?: string;

/**
* Title for the "lessThan" operator
* @type {string}
* @default "less than"
*/
static lessThanTitle?: string;

/**
* Title for the "greaterOrEqual" operator
* @type {string}
* @default "greater than or equal to"
*/
static greaterOrEqualTitle?: string;

/**
* Title for the "lessOrEqual" operator
* @type {string}
* @default "less than or equal to"
*/
static lessOrEqualTitle?: string;

/**
* Title for the "between" operator
* @type {string}
* @default "between (match case)"
*/
static betweenTitle?: string;

/**
* Title for the "iBetween" operator
* @type {string}
* @default "between"
*/
static iBetweenTitle?: string;

/**
* Title for the "betweenInclusive" operator
* @type {string}
* @default "between (inclusive, match case)"
*/
static betweenInclusiveTitle?: string;

/**
* Title for the "iBetweenInclusive" operator
* @type {string}
* @default "between (inclusive)"
*/
static iBetweenInclusiveTitle?: string;

/**
* Title for the "startsWith" operator
* @type {string}
* @default "starts with (match case)"
*/
static startsWithTitle?: string;

/**
* Title for the "iStartsWith" operator
* @type {string}
* @default "starts with"
*/
static iStartsWithTitle?: string;

/**
* Title for the "endsWith" operator
* @type {string}
* @default "ends with (match case)"
*/
static endsWithTitle?: string;

/**
* Title for the "iEndsWith" operator
* @type {string}
* @default "ends with"
*/
static iEndsWithTitle?: string;

/**
* Title for the "contains" operator
* @type {string}
* @default "contains (match case)"
*/
static containsTitle?: string;

/**
* Title for the "iContains" operator
* @type {string}
* @default "contains"
*/
static iContainsTitle?: string;

/**
* Title for the "notContains" operator
* @type {string}
* @default "does not contain (match case)"
*/
static notContainsTitle?: string;

/**
* Title for the "iNotContains" operator
* @type {string}
* @default "does not contain"
*/
static iNotContainsTitle?: string;

/**
* Title for the "notStartsWith" operator
* @type {string}
* @default "does not start with (match case)"
*/
static notStartsWithTitle?: string;

/**
* Title for the "iNotStartsWith" operator
* @type {string}
* @default "does not start with"
*/
static iNotStartsWithTitle?: string;

/**
* Title for the "notEndsWith" operator
* @type {string}
* @default "does not end with (match case)"
*/
static notEndsWithTitle?: string;

/**
* Title for the "iNotEndsWith" operator
* @type {string}
* @default "does not end with"
*/
static iNotEndsWithTitle?: string;

/**
* Title for the "regexp" operator
* @type {string}
* @default "matches expression (match case)"
*/
static regexpTitle?: string;

/**
* Title for the "iregexp" operator
* @type {string}
* @default "matches expression"
*/
static iregexpTitle?: string;

/**
* Title for the "matchesPattern" operator
* @type {string}
* @default "matches pattern (match case)"
*/
static matchesPatternTitle?: string;

/**
* Title for the "iMatchesPattern" operator
* @type {string}
* @default "matches pattern"
*/
static iMatchesPatternTitle?: string;

/**
* Title for the "containsPattern" operator
* @type {string}
* @default "contains pattern (match case)"
*/
static containsPatternTitle?: string;

/**
* Title for the "iContainsPattern" operator
* @type {string}
* @default "contains pattern"
*/
static iContainsPatternTitle?: string;

/**
* Title for the "equalsField" operator
* @type {string}
* @default "matches other field (match case)"
*/
static equalsFieldTitle?: string;

/**
* Title for the "iEqualsField" operator
* @type {string}
* @default "matches other field"
*/
static iEqualsFieldTitle?: string;

/**
* Title for the "notEqualField" operator
* @type {string}
* @default "differs from field (match case)"
*/
static notEqualFieldTitle?: string;

/**
* Title for the "iNotEqualField" operator
* @type {string}
* @default "differs from field"
*/
static iNotEqualFieldTitle?: string;

/**
* Title for the "greaterThanField" operator
* @type {string}
* @default "greater than field"
*/
static greaterThanFieldTitle?: string;

/**
* Title for the "lessThanField" operator
* @type {string}
* @default "less than field"
*/
static lessThanFieldTitle?: string;

/**
* Title for the "greaterOrEqualField" operator
* @type {string}
* @default "greater than or equal to field"
*/
static greaterOrEqualFieldTitle?: string;

/**
* Title for the "lessOrEqualField" operator
* @type {string}
* @default "less than or equal to field"
*/
static lessOrEqualFieldTitle?: string;

/**
* Title for the "containsField" operator
* @type {string}
* @default "contains another field value (match case)"
*/
static containsFieldTitle?: string;

/**
* Title for the "iContainsField" operator
* @type {string}
* @default "contains another field value"
*/
static iContainsFieldTitle?: string;

/**
* Title for the "startsWithField" operator
* @type {string}
* @default "starts with another field value (match case)"
*/
static startsWithFieldTitle?: string;

/**
* Title for the "iStartsWithField" operator
* @type {string}
* @default "starts with another field value"
*/
static iStartsWithFieldTitle?: string;

/**
* Title for the "endsWithField" operator
* @type {string}
* @default "ends with another field value (match case)"
*/
static endsWithFieldTitle?: string;

/**
* Title for the "iEndsWithField" operator
* @type {string}
* @default "ends with another field value"
*/
static iEndsWithFieldTitle?: string;

/**
* Title for the "notContainsField" operator
* @type {string}
* @default "does not contain another field value (match case)"
*/
static notContainsFieldTitle?: string;

/**
* Title for the "iNotContainsField" operator
* @type {string}
* @default "does not contain another field value"
*/
static iNotContainsFieldTitle?: string;

/**
* Title for the "notStartsWithField" operator
* @type {string}
* @default "does not start with another field value (match case)"
*/
static notStartsWithFieldTitle?: string;

/**
* Title for the "iNotStartsWithField" operator
* @type {string}
* @default "does not start with another field value"
*/
static iNotStartsWithFieldTitle?: string;

/**
* Title for the "notEndsWithField" operator
* @type {string}
* @default "does not end with another field value (match case)"
*/
static notEndsWithFieldTitle?: string;

/**
* Title for the "iNotEndsWithField" operator
* @type {string}
* @default "does not end with another field value"
*/
static iNotEndsWithFieldTitle?: string;

/**
* Title for the "startsWithPattern" operator
* @type {string}
* @default "starts with pattern (match case)"
*/
static startsWithPatternTitle?: string;

/**
* Title for the "iStartsWithPattern" operator
* @type {string}
* @default "starts with pattern"
*/
static iStartsWithPatternTitle?: string;

/**
* Title for the "endsWithPattern" operator
* @type {string}
* @default "ends with pattern (match case)"
*/
static endsWithPatternTitle?: string;

/**
* Title for the "iEndsWithPattern" operator
* @type {string}
* @default "ends with pattern"
*/
static iEndsWithPatternTitle?: string;

/**
* Title for the "and" operator
* @type {string}
* @default "Match All"
*/
static andTitle?: string;

/**
* Title for the "not" operator
* @type {string}
* @default "Match None"
*/
static notTitle?: string;

/**
* Title for the "or" operator
* @type {string}
* @default "Match Any"
*/
static orTitle?: string;

/**
* Title for the "inSet" operator
* @type {string}
* @default "is one of"
*/
static inSetTitle?: string;

/**
* Title for the "notInSet" operator
* @type {string}
* @default "is not one of"
*/
static notInSetTitle?: string;

/**
* Title for the "isBlank" operator
* @type {string}
* @default "is blank"
*/
static isBlankTitle?: string;

/**
* Title for the "notBlank" operator
* @type {string}
* @default "is not blank"
*/
static notBlankTitle?: string;

/**
* Title for the "isNull" operator
* @type {string}
* @default "is null"
*/
static isNullTitle?: string;

/**
* Title for the "notNull" operator
* @type {string}
* @default "is not null"
*/
static notNullTitle?: string;

/**
* The default usage description for operators with 
*  OperatorValueType "valueSet". A usage hint can also be set 
*  directly on a Operator.usageHint.
* @type {string}
* @default "Multiple values must be separated by the valueSeparator, '|'."
*/
static valueSetUsageHint?: string;

/**
* The default usage description for operators with 
*  OperatorValueType "valueRange". A usage hint can also be set 
*  directly on a Operator.usageHint.
* @type {string}
* @default "Range-operators require distinct start and end values, separated by the range (between) symbol, '...'."
*/
static valueRangeUsageHint?: string;

/**
* The default usage description for operators with 
*  OperatorValueType "fieldType". A usage hint can also be set 
*  directly on a Operator.usageHint.
* @type {string}
* @default null
*/
static fieldTypeUsageHint?: string;

/**
* The default usage description for operators with 
*  OperatorValueType "fieldName". A usage hint can also be set 
*  directly on a Operator.usageHint.
* @type {string}
* @default null
*/
static fieldNameUsageHint?: string;








}

/**
* Provides information about the page you're loaded in. We define "page" here to be roughly
*  equivalent to the browser window or frame the libraries have been loaded in.
*/
interface Page extends Class {




}
/**
* Creates a new Page
* 
* @param typeCheckedProperties {Partial<Page>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Page} The new instance
*/
class Page{
static create(typeCheckedProperties? : Partial<Page>, uncheckedProperties?: Object): Page;
/**
* On a call to Page.checkBrowserAndRedirect(), if no explicit URL
*    is passed in, and the browser is not supported by ISC, redirect to this URL.
* @type {URL}
* @default "[SKIN]/unsupported_browser.html"
*/
static defaultUnsupportedBrowserURL?: URL;

/**
* If a URL provided to various Page APIs begins with one of these Strings, it is treated
*  as an absolute URL.
*  
*  The default of protocols is:
*  
*    ["http://","https://","file://","mailto:", "app-resource:", "data:"]
*  
*  .. and can be replaced via Class.addClassProperties or
*  via setting the global variable isc_protocolURLs before SmartClient loads.
* @type {Array<Partial<string>>}
* @default "[...]"
*/
static protocolURLs?: Array<Partial<string>>;

/**
* Action to take when Page.checkBrowserAndRedirect is called in a browser for which
*  support is not guaranteed. Valid settings are:
*  
*  
* - "continue" Load the page without notifying the user of potential issues
*  
* - "confirm" Notify the user via a standard confirm dialog that their browser is
*  not supported. Provide options to continue anyway, or redirect to another page.
*  
* - "redirect" Automatically redirect to the another URL
*  
* @deprecated * As discussed in browserSupport, developers are recommended to consider which browsers they wish to support within their specific application rather than relying on framework
*  supported browser detection.

* @type {string}
* @default "continue"
*/
static unsupportedBrowserAction?: string;

/**
* A declared value of the enum type 
*  FireStyle.
* @type {Constant}
* @default "once"
*/
static FIRE_ONCE?: Constant;

/**
* By default most modern browsers will navigate back one page when the user hits the
*  backspace key.
*  
*  Setting this attribute to true will suppress this native behavior. Alternatively, 
*  developers can explicitly return false from the keyPress event
*  (either via event handlers applied to specific widgets, or via Page.registerKey
*  for example) to suppress the native navigation behavior.
* @type {boolean}
* @default true
*/
static suppressBackspaceNavigation?: boolean;





/**
*  Has the page finished loading?
* 
* 
*/
static isLoaded(): boolean;

/**
*  Set the title of the page, which is typically shown as part of the browser window title
* 
*/
static setTitle(): void;

/**
*  Returns the base URL of the application, which is the page URL minus the last non-directory 
*  path component. For example, if the page is loaded from
*  http://foo.com/bar/zoo.jsp, appDir will be http://foo.com/bar/.
*  
*  If other page-wide URLs such as Page.setIsomorphicDir are specified as
*  relative paths, they are considered relative to this URL.
* 
* 
*/
static getAppDir(): string;

/**
*  Specify the directory for app-specific images.
*  
*  This becomes the default location where any SmartClient component will load images from
*  unless the special "[SKIN]" prefix is used to indicate that an image is part of a skin.
*  
*  Default is "[APP]images/"
* 
* 
* @param {string=} New imgDir URL.
*/
static setAppImgDir(URL?:string): void;

/**
*  Return the directory for app-specific images.
* 
* 
*/
static getAppImgDir(): string;

/**
*  Specify the directory for miscellaneous app-specific files other than images, such as
*  HTMLFlow.contentsURL, ViewLoader, 
*  XML or JSON flat data files, videos, etc.
*  
*  This URL also becomes available via the prefix "[APPFILES]" for RPCRequest.actionURL.
*  
*  Defaults to the value of Page.getAppDir, that is, the current directory.
* 
* 
* @param {string=} New app files URL.
*/
static setAppFilesDir(URL?:string): void;

/**
*  Returns the directory for application-specific files (other than images).
* 
* 
* @param {string=} New app files URL.
*/
static getAppFilesDir(URL?:string): void;

/**
*  Specify the root directory for Isomorphic-supplied files - the directory containing 
*  the modules/ and system/ subdirectories shipped as part of
*  the SmartClient package.
*  
*  Note that this property is commonly specified directly in the bootstrap HTML file
*  by setting window.isomorphicDir before loading the SmartClient library files.
* 
* 
* @param {string=} New IsomorphicDir URL.
*/
static setIsomorphicDir(URL?:string): void;

/**
*  Return the root directory for Isomorphic-specific files.
* 
* 
*/
static getIsomorphicDir(): string;

/**
*  Specify the root directory for Isomorphic-supplied tools. Typicall tools/ under webRoot.
*  
*  Note that this property is commonly specified directly in the bootstrap HTML file
*  by setting window.isomorphicToolsDir before loading the SmartClient library
*  files. If unset, it defaults to $isomorphicDir/../tools/
* 
* 
* @param {string=} New IsomorphicToolsDir URL.
*/
static setIsomorphicToolsDir(URL?:string): void;

/**
*  Return the root directory for Isomorphic-supplied tools dir.
* 
* 
*/
static getIsomorphicToolsDir(): string;

/**
*  Return the images directory used by Isomorphic-supplied tools.
* 
* 
*/
static getToolsImgDir(): string;

/**
*  Specify the URL for media that's part of the skin
* 
* 
* @param {string=} New skinDir URL
*/
static setSkinDir(URL?:string): void;

/**
*  Return the directory for media that's part of the skin
* 
* 
*/
static getSkinDir(): string;

/**
*  Return the directory for a skin image.
* 
* 
* @param {URL=} Partial URL (relative to Page._skinDir) where the image lives.     If not supplied, will use "images/"
*/
static getSkinImgDir(imgDir?:URL): string;

/**
*  Specify the dir for Showcase-sample images which differ by skin.
* 
* 
* @param {string=} New URL for Showcase-sample images in the current skin
*/
static setSampleImgDir(URL?:string): void;

/**
*  Return the dir for Showcase-sample images which differ by skin.
* 
* 
*/
static getSampleImgDir(): string;

/**
*  Specify the dir for skin-specific sample-thumbnails in the product Showcase.
* 
* 
* @param {string=} New URL for Showcase sample thumbnails
*/
static setSampleThumbnailDir(URL?:string): void;

/**
*  Return the dir for skin-specific sample-thumbnails in the product Showcase.
* 
* 
* @param {string=} New URL for Showcase sample-thumbnails
*/
static getSampleThumbnailDir(URL?:string): string;

/**
*  Specify the dir for skin-specific sample-photos in the product Showcase.
* 
* 
* @param {string=} New URL for Showcase sample-photos
*/
static setSamplePhotoDir(URL?:string): void;

/**
*  Return the dir for skin-specific sample-photos in the product Showcase.
* 
* 
* @param {string=} New URL for Showcase sample-photos
*/
static getSamplePhotoDir(URL?:string): string;

/**
*  Return the full URL for app-specific or skin image.
*  
*  To use a skin image, start the URL with "[SKIN]". Any other relative URL is assumed
*  relative to the Page.getAppImgDir.
* 
* 
* @param {SCImgURL} Local file name for the image.
* @param {string=} User-specified image directory,     local to
*/
static getImgURL(src:SCImgURL, imgDir?:string): string;

/**
*  Return a full URL for a relative path that uses a special prefix such as "[APPFILES]" or
*  "[SKIN]".
*  
*  For images, use Page.getImgURL instead.
* 
* 
* @param {string} Local file name for the image.
*/
static getURL(fileName:string): string;

/**
* 
*  Return whether the page text direction is right to left. If you set "DIR=RTL" in the BODY tag of
*  the page, then this method will return true. If you set "DIR=LTR" then this method will return
*  false.
* 
* 
*/
static isRTL(): boolean;

/**
*  Load a styleSheet for this application. 
* 
*  The styleSheetURL parameter can use any special directories, eg:
*    &nbsp;&nbsp;Page.loadStylesheet("[SKIN]/skin_styles.css")
*    or
*    &nbsp;&nbsp;Page.loadStylesheet("[APP]/app_styles.css").
*    
*  If you don't specify a special directory, the app directory
*  	will be assumed.
*  
*  Note:  If the document's ONLOAD handler has already fired, this
*   will have no effect.
* 
* 
* @param {URL} URL to the stylesheet.
*/
static loadStyleSheet(styleSheetURL:URL): void;

/**
*  Resize the outer portion of the window to a specific width and height.
* 
* @param {number} new width for the window
* @param {number} new height for the window
*/
static resizeTo(width:number, height:number): void;

/**
*  Move the window to a specified top and left in screen coordinates.
* 
* 
* @param {number} new left coordinate for window
* @param {number} new top coordinate for window
*/
static moveTo(left:number, top:number): void;

/**
*  Scroll the window to a specified top and left coordinate.
* 
* 
* @param {number} new left coordinate for window
* @param {number} new top coordinate for window
*/
static scrollTo(left:number, top:number): void;

/**
*  Get the width of the visible portion of the window, not including browser chrome or the
*  scrollbar area.
*  
*  See also Page.getOrientation.
* 
* @param {object=} the window object
*/
static getWidth(wd?:object): number;

/**
*  Get the height of the visible portion of the window, not including browser chrome or the
*  scrollbar area.
*  
*  See also Page.getOrientation.
* 
* @param {object=} the window object
*/
static getHeight(wd?:object): number;

/**
*  Is the current page wider than it is tall ("landscape" orientation) or the reverse 
*  ("portrait" orientation). Note that the PageEvent
*  will be fired whenever the page orientation changes.
*  
*  This method is typically useful for apps developed for display on mobile devices,
*  though it will also return a valid value when running against a desktop browser.
*  See also mobileDevelopment on building applications
*  for mobile devices
* 
*/
static getOrientation(): PageOrientation;

/**
*  This method only applies to browsers that support the special viewport meta tag, such as
*  Mobile Safari running on the iPhone.
*  
*  This method will dynamically change the viewport configuration, allowing you to set an
*  initial size or scale level and enable / disable user-scaling. Typically this method will
*  be called with a value for scale, width or height rather than passing in values for all
*  three properties.
*  
*  See Apple's Safari Web Content Guide on configuring the viewport for more information:
*  https://developer.apple.com/library/safari/documentation/AppleApplications/Reference/SafariWebContent/UsingtheViewport/UsingtheViewport.html
*  
*  Note: Modifying the width/height or initial scale of the viewport has two user-visible
*  effects:
*  
*  
* - HTML elements may reflow to fit the specified size (or the implied size calculated
*    from the specified scale level and the native device size).
*  
* - If the user has not scaled the application explicitly, and no other scaling or sizing
*    attributes were specified via a viewport meta tag for this page, the application will
*    zoom to specified scale level (or the scale level required to fit the specified viewport
*    size to the device's screen).
*  
* 
* @param {Float=} Desired scale level where 1 indicates no scaling (each css pixel  will be displayed using 1px on the physical device). Pass in null to avoid setting  this property.
* @param {Integer=} Desired viewport width in pixels. This indicates how many pixels  should fit within the device screen. Pass in null to avoid setting this property.
* @param {Integer=} Desired viewport height in pixels. This indicates how many pixels  should fit within the device screen. Pass in null to avoid setting this property.
* @param {boolean=} Should the user be able to scale the application (using  pinch gestures, double tapping, rotating the device, etc.)? Pass in null to avoid setting  this property.
*/
static updateViewport(scale?:Float, width?:Integer, height?:Integer, scalable?:boolean): void;

/**
*  Returns the current ratio of the resolution in physical pixels to the resolution in CSS
*  pixels in the browser, known as the
*  device pixel ratio.
*  The ratio reflects the combination of OS-level zoom, browser-level zoom, and device/OS
*  defaults.
*  
*  When this value isn't 1, some browser bugs manifest, so it can be useful to check it as an
*  indicator that certain issues may be present and, in turn, whether workarounds fom them 
*  must run. However, note that on Safari it's not useful as the value is identically 1.
*  
*  A specific application of this function is to check whether any zoom is present after a
*  resize event so that the user can be informed that a page reload might be required for proper
*  rendering.To do this, first store its value after the page is loaded, then in a
*  resize handler check for the ratio transitioning away from 1:
* isc.Page.setEvent("load", function () {
*   isc.Page._ratio = isc.Page.getDevicePixelRatio();
* });
* isc.Page.setEvent("resize", function () {
*   var ratio = isc.Page.getDevicePixelRatio();
*   if (ratio != 1 && isc.Page._ratio == 1) {
*     isc.notify("Browser content is now zoomed. " + 
*       "A page reload may be needed for proper display.");
*   }
*   isc.Page._ratio = ratio;
* });
* 
*/
static getDevicePixelRatio(): void;

/**
*  Get the width of the window contents as they have been drawn. If the page scrolls, this may
*  be larger than the Page.getWidth.
* 
* 
*/
static getScrollWidth(): number;

/**
*  Get the height of the window contents as they have been drawn. If the page scrolls, this may
*  be larger than the Page.getHeight.
* 
* 
*/
static getScrollHeight(): number;

/**
*  Get the amount that the browser window has been scrolled horizontally.
* 
*/
static getScrollLeft(): number;

/**
*  Get the amount that the browser window has been scrolled vertically.
* 
*/
static getScrollTop(): number;

/**
*  Get the width of the user's screen, in pixels.
* 
*/
static getScreenWidth(): number;

/**
*  Get the height of the user's screen, in pixels.
* 
*/
static getScreenHeight(): number;

/**
*  Go back in the browser's history.
* 
*  If the history is empty and the window.opener is set, we assume we're a child window and just
*  close the window.
* 
* 
*/
static goBack(): void;

/**
*  Method to set up a static Class.observe on some target object. 
*  This allows developers to perform some action every time a particular method is invoked 
*  on a target object.
*  
*  This method returns a unique observation identifier string. To cancel the observation, 
*  pass this identifier to Page.ignore.
*  
*  If multiple observations are set up for the same target object and method, the 
*  notification actions will be fired in the order in which they were registered.
*  
*  This method is available as isc.Page.observe() or just isc.observe()
*  
*  Note [potential memory leak]: If the target object is a simple JavaScript object
*  (not an instance of a SmartClient class), developers should always call 
*  Page.ignore to stop observing the object if an observation is no longer necessary.
*  
*  This ensures that if the object is subsequently allowed to go out of scope by application 
*  code, the Page level observation system will not retain a reference to it (so the browser
*  can reclaim the allocated memory).
*  While cleaning up observations that are no longer required is always good practice, this
*  memory leak concern is not an issue if the target object is an instance of a SmartClient
*  class. In that case the observation is automatically released when the 
*  target is Class.destroy.
* 
* 
* @param {object} Object to observe. This may be any JavaScript object with the specified  target method, including native arrays, and instances of SmartClient classes such as   Canvas.
* @param {string} Name of the method to observe. Every time this method is invoked  on the target object the specified action will fire   (after the default implementation completes).
* @param {Function | string} Action to take when the observed method is invoked.  If action is a string to execute, certain keywords are available for context:    
- observed is the target object being observed (on which the method was invoked).  
- returnVal is the return value from the observed method (if there is one)  
- For functions defined with explicit parameters, these will also be available as keywords within    the action string    If action is a function, the arguments for the original method will also be   passed to this action function as arguments. If developers need to access the target   object being observed from the action function they may use native javascript techniques  such as   javascript closure  to do so. The return value from the observed method is not available to the action function.
*/
static observe(object:object, methodName:string, action:Function | string): string;

/**
*  Clear an observation set up by Page.observe.
*  
*  This method is available as isc.Page.ignore() or just isc.ignore()
* 
* 
* @param {string} ID returned from Page.observe call we want to clear
*/
static ignore(observerID:string): void;

/**
*  Wait for a method to fire on an object.
*  
*  waitFor is similar Class.observe, but fires once only.
* 
* 
* @param {object} any SmartClient object, eg, a ListGrid
* @param {string} name of a method on that object
* @param {Function} Callback to fire when the observed method completes
* @param {number=} Optional timeout period (in milliseconds). If you want a timeout,              you must also provide a timeoutCallback
* @param {Function=} Callback to fire if the timeout period elapses before the                   observed method completes
*/
static waitFor(object:object, methodName:string, callback:Function, timeout?:number, timeoutCallback?:Function): boolean;

/**
*  Wait for methods to fire on multiple objects.
*  
*  waitForMultiple is similar to Page.waitFor, except that it does not fire
*  its callback until all of the provided methods have fired.
* 
* 
* @param {Array<any>} an array of objects, each of which consists of two properties:            "object": any SmartClient object, eg a ListGrid             "method": name of a method on that object
* @param {Function} Callback to fire when all observed methods have fired
* @param {number=} Optional timeout period (in milliseconds). If you want a timeout,              you must also provide a timeoutCallback
* @param {Function=} Callback to fire if the timeout period elapses before all                   observed methods have fired
*/
static waitForMultiple(objects:Array<any>, callback:Function, timeout?:number, timeoutCallback?:Function): boolean;

/**
*  Check whether the browser is supported by the Isomorphic SmartClient system. Behavior depends
*  upon the value of Page.unsupportedBrowserAction:
*  
*  
* - "continue" Load the page without notifying the user of potential issues
*  
* - "confirm" Notify the user via a standard confirm dialog that their browser is
*  not supported. Provide options to continue anyway, or redirect to another page. Text of the
*  confirm dialog is retrieved by calling Page.getUnsupportedBrowserPromptString.
*  
* - "redirect" Automatically redirect to the another URL
*  
*  If redirecting to another page is necessary, and no explicit URL is provided we will use
*  Page.defaultUnsupportedBrowserURL.
*  
*  This method is commonly called as part of the skinning logic after page
*  load.
* 
* 
* @param {string=} URL of redirect page. May include Isomorphic special directories   such as [SKIN].
* @deprecated * As discussed in browserSupport, developers are recommended to consider which browsers they wish to support within their specific application rather than relying on framework
*  supported browser detection.

*/
static checkBrowserAndRedirect(URL?:string): void;

/**
*  Returns the text for the prompt shown to user from Page.checkBrowserAndRedirect
*  if they are accessing this page in an unsupported
*  browser and Page.unsupportedBrowserAction is set to "confirm". May be
*  overridden to return a different message.
* 
*/
static getUnsupportedBrowserPromptString(): string;

/**
*  Setting this to true will cause Page.loadStyleSheet to append
*  an "isc_version" parameter to the end of the url when loading a stylesheet.
* 
* 
* @param {boolean} pass in true to turn on automatic adding of version                       parameter to css urls.
*/
static setAddVersionToSkinCSS(addVersionToSkinCss:boolean): void;

/**
*  Returns true if add version to skin CSS is currently turned on.
* 
* 
*/
static isAddVersionToSkinCSS(): boolean;

/**
*  Register to be called whenever a given type of event occurs, at the page level.
*  
*  This includes events that also occur on widgets (like "click") and events that only occur at
*  the page level ("resize" and "load").
*  
*  For events that also occur on widgets, page level event registrations will fire BEFORE the
*  event handlers on Canvases.  If your action returns false, this will prevent
*  the event from getting to the intended Canvas.
*  
*  Capturing events on widgets can be done by setting one of the event methods available on Canvas
*  (and hence available to all widget classes). See widgetEvents.
* 
* 
* @param {PageEvent} event type to register for ("mouseDown", "load", etc)
* @param {string} string to be eval'd when event fires   (function) 	function to be executed when event fires            (object)      an object to call on which a method named "page" +                      eventType will be called
* @param {FireStyle=} Flag to set automatic removal of the event after      it fires one or more times
* @param {string=} optional - if an object was passed in as the second                     parameter, this is a name of a method to call on that                      object.
*/
static setEvent(eventType:PageEvent, action:string, fireStyle?:FireStyle, functionName?:string): number;

/**
* 	Clear event(s) under the given eventType.
* 	To clear all events, omit the ID parameter. To clear a specific event,
* 	pass the ID that was returned by Page.setEvent().
* 
* @param {PageEvent} event type to clear
* @param {number=} ID of the event to clear.     	If not specified, all events in eventType will be cleared.
*/
static clearEvent(eventType:PageEvent, ID?:number): void;

/**
*  Fire some action when the Page receives a keyPress event from a certain key.
*  Note that if a widget has keyboard focus, this action will fire only after any widget-level
*  keyPress handlers have fired and bubbled the event up to the top of their ancestor chain.
*  Multiple actions can be registered to fire on a single keyPress using this method, and can
*  be associated with different target objects (which will then be available as
*  a parameter when the action is fired).
*  
*  
*  This differs from calling Page.setEvent with the "keyPress"
*  events registered via setEvent() will fire before widget level handlers 
*  respond to the event, and will fire for every keyPress event, not just those
*  triggered by some specific key or key-combination.
*  
*  
*  
* 
* @param {KeyIdentifier} key name or identifier object.
* @param {string} Action to fire when key is pressed.       This can be a string of script to evaluate or a javascript function.        This action will be passed 2 parameters: The name of the key pressed will be        available as the first parameter or key keyword. The target        passed into this method will be available as the second parameter or        target keyword.
* @param {any=} If specified this object will be made available to the                 action fired as a parameter.
*/
static registerKey(key:KeyIdentifier, action:string, target?:any): void;

/**
*  Clears an action registered to fire on a specific a keyPress event via the Page.registerKey
*  method. 
* 
* @param {KeyName} Name of key to clear registry entries for.
* @param {object=} target specified when the action was registered for the key.
*/
static unregisterKey(actionID:KeyName, target?:object): void;




}

/**
* FormItem for password fields, where text input by the user should not be shown in readable text.
*/
interface PasswordItem extends TextItem {




}
/**
* Creates a new PasswordItem
* 
* @param typeCheckedProperties {Partial<PasswordItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {PasswordItem} The new instance
*/
class PasswordItem{
static create(typeCheckedProperties? : Partial<PasswordItem>, uncheckedProperties?: Object): PasswordItem;







}

/**
* ListGrid subclass used, by default, by FormItems which implement
*  PickList to display a PickList.dataSetType of selectable 
*  options.
*  
*  Can be subclassed, customized and assigned to FormItems 
*  via the ComboBoxItem.pickListConstructor attribute.
*/
interface PickListMenu extends ListGrid {
/**
* The string to display in the body of a listGrid with an empty data array, if
*  showEmptyMessage is true.
* @type {string}
* @default "No Items To Show"
*/
emptyMessage?: string;

/**
* For databound ListGrids, this attribute can be used to customize the ResultSet
*  object created for this grid when data is fetched
* @type {ResultSet}
* @default null
*/
dataProperties?: ResultSet;

/**
* Default CSS class for the ListGrid as a whole.
* @type {CSSStyleName}
* @default "pickListMenu"
*/
styleName?: CSSStyleName;

/**
* CSS style used for the body of this grid. If applying a background-color to the body
*  via a CSS style applied using this property, be sure to set
*  ListGrid.bodyBackgroundColor to null.
* @type {CSSStyleName}
* @default "pickListMenuBody"
*/
bodyStyleName?: CSSStyleName;

/**
* Option to show filter editor is disabled for pickListMenus by default
* @type {boolean}
* @default false
*/
canShowFilterEditor?: boolean;

/**
* Option to save searches is disabled for PickListMenus
* @type {boolean}
* @default false
*/
canSaveSearches?: boolean;

/**
* If ListGrid.baseStyle is unset, base style will be derived from
*  ListGrid.normalBaseStyle if this grid has fixed row heights and
*  the specified ListGrid.cellHeight matches this value. Otherwise
*  ListGrid.tallBaseStyle will be used.
* @type {number}
* @default "16"
*/
normalCellHeight?: number;





}
/**
* Creates a new PickListMenu
* 
* @param typeCheckedProperties {Partial<PickListMenu>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {PickListMenu} The new instance
*/
class PickListMenu{
static create(typeCheckedProperties? : Partial<PickListMenu>, uncheckedProperties?: Object): PickListMenu;







}

/**
* FormItem that allows picking a value from a hierarchical data model.
*/
interface PickTreeItem extends CanvasItem {
/**
* Should this item's value be saved in the form's values and hence returned from
*  DynamicForm.getValues?
*  
*  shouldSaveValue:false is used to mark formItems which do not correspond to
*  the underlying data model and should not save a value into the form's
*  DynamicForm.values. Example includes visual separators, password re-type fields,
*  or checkboxes used to show/hide other form items.
*  
*  A shouldSaveValue:false item should be given a value either via
*  FormItem.defaultValue or by calling
*  DynamicForm.setValue or 
*  FormItem.setValue. Providing a value via
*  DynamicForm.values or DynamicForm.setValues 
*  will automatically switch the item to shouldSaveValue:true.
*  
*  Note that 
*  
* - if an item is shouldSaveValue true, but has no name, a warning is logged, and 
*    shouldSaveValue will be set to false.
*  
* @type {boolean}
* @default true
*/
shouldSaveValue?: boolean;

/**
* The visible button created by a PickTreeItem is an AutoChild of type
*  TreeMenuButton by default.
* @type {Canvas}
* @default null
*/
button?: Canvas;

/**
* When FormItem.showPending is true, the Button.baseStyle
*  of the PickTreeItem.button when in the "Pending" visual state.
*  
*  If unset, then the baseStyle of the button is not changed.
* @type {CSSStyleName}
* @default "buttonPending"
*/
pendingButtonStyle?: CSSStyleName;

/**
* If specified, the tree of possible options will be derived from the dataSource as a
*  ResultTree, rather than using this.valueTree. Options can be loaded on demand or up
*  front according tp PickTreeItem.loadDataOnDemand.
* @type {DataSource | string}
* @default null
*/
dataSource?: DataSource | string;

/**
* For a PickTreeItem that uses a DataSource, these properties will be passed to
*  the automatically-created ResultTree. This can be used for various customizations such as
*  modifying the automatically-chosen Tree.parentIdField.
* @type {Partial<Tree>}
* @default null
*/
dataProperties?: Partial<Tree>;

/**
* A Tree of options from which the user can select.
* @type {Tree}
* @default null
*/
valueTree?: Tree;

/**
* If this is a databound item, should the load our set of possible options be loaded
*  on demand (as submenus are displayed), or upfront?
* @type {boolean}
* @default null
*/
loadDataOnDemand?: boolean;

/**
* Specifies an alternative field from which display values should be retrieved for this
*  item.
*  
*  If this item is not databound (PickTreeItem.dataSource is unset), this is
*  implemented by picking up the value of the specified field from the
*  PickTreeItem.valueTree.
*  
*  Otherwise this item will attempt to map its underlying value to a display value
*  by retrieving a record from the PickTreeItem.dataSource where the 
*  PickTreeItem.valueField matches this item's value, and displaying the 
*  displayField value from that record.
* @type {string}
* @default null
*/
displayField?: string;

/**
* Which field in the tree-data should be returned as this item's value?
*  If unspecified, the path will be used
* @type {string}
* @default null
*/
valueField?: string;

/**
* This message will be displayed as a single, disabled option in any empty menu/submenu
*  created from this item's data tree.
* @type {HTMLString}
* @default "No items to display"
*/
emptyMenuMessage?: HTMLString;

/**
* Text to display when this form item has a null or undefined value.
*  
*  If the formItem has a databound pickList, and its FormItem.displayField or
*  FormItem.valueField (if the former isn't set) has an undefined emptyCellValue
*  field, that field will automatically be set using the emptyDisplayValue property.
*  
*  If the emptyDisplayValue is null (the default) then this item will use the standard title
*  of the tree menu button that is shown when no values are selected.
* @type {HTMLString}
* @default null
*/
emptyDisplayValue?: HTMLString;

/**
* If FormItem.canEdit is set to false, how should this item
*  be displayed to the user?
*  
*  For PickTreeItems, this setting affects only the item's title - the button itself will 
*  always appear disabled when canEdit is false, since buttons don't provide 
*  readOnly or static appearances.
* @type {ReadOnlyDisplayAppearance}
* @default null
*/
readOnlyDisplay?: ReadOnlyDisplayAppearance;

/**
* If true, clicking or pressing Enter on a menu item that has a submenu will
*  select that item (with standard behavior of hiding the menus, calling click
*  handlers, etc) instead of showing the submenu.
* @type {boolean}
* @default null
*/
canSelectParentItems?: boolean;

/**
* If set, this FormItem will map stored values to display values as though a
*  ValueMap were specified, by fetching records from the 
*  specified optionDataSource and extracting the
*  FormItem.valueField and 
*  FormItem.displayField in loaded records, to derive one
*  valueMap entry per record loaded from the optionDataSource.
*  
*  With the default setting of FormItem.fetchMissingValues, fetches will be initiated against
*  the optionDataSource any time the FormItem has a non-null value and no corresponding
*  display value is available. This includes when the form is first initialized, as well
*  as any subsequent calls to FormItem.setValue, such as may happen when
*  DynamicForm.editRecord is called. Retrieved values are automatically cached by
*  the FormItem.
*  
*  Note that if a normal, static FormItem.valueMap is also specified for
*  the field (either directly in the form item or as part of the field definition in the
*  dataSource), it will be preferred to the data derived from the optionDataSource for
*  whatever mappings are present.
*  
*  In a databound form, if FormItem.displayField is specified for a FormItem and 
*  optionDataSource is unset, optionDataSource will default to
*  the form's current DataSource
* @type {DataSource}
* @default null
*/
optionDataSource?: DataSource;



/**
*  Setter to change the PickTreeItem.valueTree at runtime
* 
* @param {Tree} new value tree for the item
*/
setValueTree(valueTree:Tree): void;

/**
*  Notification method called when FormItem.showPending is enabled and
*  this PickTreeItem should either clear or show its pending visual state.
*  
*  The default behavior is that the FormItem.titleStyle and
*  FormItem.cellStyle are updated to include/exclude the "Pending" suffix.
*  In addition, when displayed in the pending state and a
*  PickTreeItem.pendingButtonStyle is set, then the PickTreeItem.button's
*  Button.baseStyle is set to pendingButtonStyle.
*  Returning false will cancel this default behavior.
* 
* @param {DynamicForm} the managing DynamicForm instance.
* @param {FormItem} the form item itself (also available as "this").
* @param {boolean} true if the item should show its pending visual state; false otherwise.
* @param {any} the current form item value.
* @param {any} the value that would be restored by a call to DynamicForm.resetValues.
*/
pendingStatusChanged(form:DynamicForm, item:FormItem, pendingStatus:boolean, newValue:any, value:any): boolean;

/**
*  Only applies to databound items (see PickTreeItem.optionDataSource).
*  Performs a fetch type operation on this item's DataSource to retrieve/refresh the tree
*  of data displayed as rows in this items menu.
* 
*/
fetchData(): void;

/**
*  Setter for PickTreeItem.emptyDisplayValue.
* 
* @param {HTMLString} */
setEmptyDisplayValue(emptyDisplayValue:HTMLString): void;



}
/**
* Creates a new PickTreeItem
* 
* @param typeCheckedProperties {Partial<PickTreeItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {PickTreeItem} The new instance
*/
class PickTreeItem{
static create(typeCheckedProperties? : Partial<PickTreeItem>, uncheckedProperties?: Object): PickTreeItem;







}

/**
* TreeGrid subclass used, by default, by FormItems which implement
*  PickList to display a PickList.dataSetType of 
*  selectable options.
*  
*  Can be subclassed, customized and assigned to FormItems 
*  via the ComboBoxItem.pickTreeConstructor attribute.
*/
interface PickTreeMenu extends TreeGrid {
/**
* Option to show filter editor is disabled for pickTreeMenus by default
* @type {boolean}
* @default false
*/
canShowFilterEditor?: boolean;

/**
* Option to save searches is disabled for PickTreeMenus
* @type {boolean}
* @default false
*/
canSaveSearches?: boolean;





}
/**
* Creates a new PickTreeMenu
* 
* @param typeCheckedProperties {Partial<PickTreeMenu>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {PickTreeMenu} The new instance
*/
class PickTreeMenu{
static create(typeCheckedProperties? : Partial<PickTreeMenu>, uncheckedProperties?: Object): PickTreeMenu;







}

/**
* A Placeholder is used in place of component while devTools when an
*  instance of the actual component cannot be created or when required properties are not yet
*  provided that allow correct creation. See the Designing Components for Editing &amp;
*  Using Placeholder section of reify for details.
*  
*  Placeholder is a special class in that the properties that are directly assigned to the
*  instance are not used by the placeholder. These are assumed to apply to the actual component
*  for which the placeholder is replacing. Properties that control the placeholder itself are
*  in Placeholder.placeholderDefaults.
*  
*  On creation a Placeholder will automatically replace itself with the target component
*  (PlaceholderDefaults.placeholderFor) when created from RPCManager.createScreen
*  unless the project was loaded with LoadProjectSettings.allowPlaceholders is
*  true.
*  
*  Note that Placeholder has a lowercase H because it is a single word in English.
*/
interface Placeholder extends Label {
/**
* Defaults applied to the placeholder instance when not replaced by the target component.
* @type {PlaceholderDefaults}
* @default null
*/
placeholderDefaults?: PlaceholderDefaults;





}
/**
* Creates a new Placeholder
* 
* @param typeCheckedProperties {Partial<Placeholder>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Placeholder} The new instance
*/
class Placeholder{
static create(typeCheckedProperties? : Partial<Placeholder>, uncheckedProperties?: Object): Placeholder;







}

/**
* A PortalLayout is a special subclass of Layout designed to display Portlet windows.
*  A PortalLayout displays Portlets in columns and supports drag-drop interaction for moving 
*  Portlets around within the PortalLayout. Portlets may be drag-reordered within columns, dragged
*  into other columns, or dragged next to other Portlets to sit next to them horizontally
*  within a column.
*/
interface PortalLayout extends Layout {
/**
* Controls how the PortalLayout reacts when column widths or Portlet widths
*  overflow the width of the PortalLayout. By default, the PortalLayout scrolls
*  when necessary. You can also use overflow: visible or overflow: hidden, with the 
*  usual results -- see PortalLayout.canResizePortlets for a further explanation of column widths.
*  
*  Note that overflowing height is also affected by PortalLayout.columnOverflow.
*  By default, each column will scroll individually -- you can change
*  columnOverflow to "auto" to scroll the whole PortalLayout instead.
* @type {Overflow}
* @default "auto"
*/
overflow?: Overflow;

/**
* Controls the Canvas.overflow setting for each column. If set to "auto" (the
*  default) then each column will scroll individually (if its Portlet overflow
*  the column height). You can also use "hidden" to clip overflowing heights, or "visible" to
*  show the overflow. The effect of "visible" will depend on the setting for PortalLayout.overflow
*  -- by default, the PortalLayout as a whole will scroll when necessary.
* @type {Overflow}
* @default "auto"
*/
columnOverflow?: Overflow;

/**
* Controls whether the PortalLayout will stretch column widths, if needed to accommodate the
*  width of Portlet. If set, columns will overflow their widths in order to accommodate 
*  the widths of their Portlets.
*  
*  With the default setting of Overflow: auto, the PortalLayout as a whole will scroll 
*  horizontally if needed. Depending on the setting of PortalLayout.canShrinkColumnWidths, 
*  other columns may shrink to avoid overflow on the PortalLayout as a whole.
*  
*  If canStretchColumnWidths is turned off, then individual rows will scroll
*  horizontally in order to accommodate Portlets that are wider than their column width allows.
* @type {boolean}
* @default true
*/
canStretchColumnWidths?: boolean;

/**
* Controls whether the PortalLayout will shrink column widths to avoid overflowing the PortalLayout 
*  horizontally. If the PortalLayout would otherwise overflow its width, it will check each column
*  to see whether it is wider than necessary to accommodate its Portlet. If so,
*  the column may shrink to avoid having to scroll the PortalLayout.
* @type {boolean}
* @default true
*/
canShrinkColumnWidths?: boolean;

/**
* When PortalLayout.canStretchColumnWidths, should we stretch all column widths proportionally,
*  or just stretch the columns that need extra width?
*  
*  Note that this implies turning off PortalLayout.canShrinkColumnWidths.
* @type {boolean}
* @default false
*/
stretchColumnWidthsProportionally?: boolean;

/**
* Controls whether the last column will be stretched to fill the PortalLayout's width,
*  or left at its specified width.
* @type {boolean}
* @default true
*/
preventUnderflow?: boolean;

/**
* Controls whether the last Portlet in a column will be stretched to fill the column's height,
*  or left at its specified height.
* @type {boolean}
* @default true
*/
preventColumnUnderflow?: boolean;

/**
* Controls whether the last Portlet in a row will be stretched to fill the row's width,
*  or left at its specified width.
* @type {boolean}
* @default true
*/
preventRowUnderflow?: boolean;

/**
* The space between portal columns.
*  
*  To set spacing between portlets on a row in the same column, see
*  PortalLayout.portletHSpacing.
* @type {Integer}
* @default "0"
*/
columnSpacing?: Integer;

/**
* The horizontal space between portlets placed into the same row.
*  
*  To set the spacing between portal columns, use PortalLayout.columnSpacing.
* @type {Integer}
* @default "3"
*/
portletHSpacing?: Integer;

/**
* The vertical space between portal rows.
* @type {Integer}
* @default "3"
*/
portletVSpacing?: Integer;

/**
* A convenience attribute which you can use to populate a PortalLayout with Portlet
*  on initialization. After initialization, use PortalLayout.addPortlet or drag-and-drop to add
*  Portlets, and PortalLayout.getPortlets or PortalLayout.getPortletArray
*  to get Portlets.
*  
*  To create one column, you can provide an array of Portlets.
*  
*  To create multiple columns, provide an array of arrays (where the first level represents columns,
*  and the second represents Portlets).
*  
*  To put multiple portlets in the same row, provide a third level to the array.
*  
*  Note that PortalLayout.numColumns is ignored if you provide the portlets attribute, since
*  the array will indicate how many columns to create. You can provide an empty second-level
*  array to create a blank column, if needed.
* @type {Array<Partial<Portlet>>}
* @default null
*/
portlets?: Array<Partial<Portlet>>;

/**
* Initial number of columns to show in this PortalLayout. Note that after initialization
*  columns should be added / removed via PortalLayout.addColumn and PortalLayout.removeColumn.
*  numColumns is ignored if you initialize the PortalLayout.portlets attribute, since the portlets
*  attribute will imply how many columns to create.
* @type {Integer}
* @default "2"
*/
numColumns?: Integer;

/**
* Should a menu be shown within each column with options to add / remove columns?
* @type {boolean}
* @default true
*/
showColumnMenus?: boolean;

/**
* Border to show around columns in this PortalLayout
* @type {string}
* @default "1px solid gray"
*/
columnBorder?: string;

/**
* Are columns in this PortalLayout drag-resizeable?
*  
*  Note that the displayed width of a column will automatically shrink and stretch
*  to accommodate the width of Portlet -- see PortalLayout.canStretchColumnWidths
*  and PortalLayout.canShrinkColumnWidths for an explanation. 
*  This setting affects the intrinsic width of a column --
*  that is, the width it will try to return to when not necessary to stretch or shrink
*  to accommodate Portlet widths.
* @type {boolean}
* @default true
*/
canResizeColumns?: boolean;

/**
* Should vertical drag-resize of portlets within columns be allowed?
* @deprecated * Use PortalLayout.canResizePortlets instead.

* @type {boolean}
* @default false
*/
canResizeRows?: boolean;

/**
* Should the height and width of Portlet be drag-resizable?
*  
*  Note that changing the height of a Portlet will change the height of all
*  the Portlets in the same row to match.
*  
*  If the height of Portlets causes a column to overflow, that column will scroll vertically
*  (independent of other columns), depending on the PortalLayout.columnOverflow setting.
*  
*  Changing the width of a Portlet will potentially cause columns to stretch
*  and shrink their displayed widths in order to accommodate the Portlets,
*  depending on the value of PortalLayout.canStretchColumnWidths and 
*  PortalLayout.canShrinkColumnWidths.
*  
*  However, the instrinsic width of the columns will remain the same,
*  so that the columns will resume their former widths when no longer necessary 
*  to stretch or shrink to accommodate the widths of Portlets. 
*  To allow drag-resizing of the intrinsic width of columns, see PortalLayout.canResizeColumns.
*  
*  The net effect is that (by default) PortalLayouts behave like layouts when Portlet sizes do
*  not cause overflow, but behave more like stacks when overflow occurs.
* @type {boolean}
* @default false
*/
canResizePortlets?: boolean;

/**
* The Canvas.dropTypes to be applied when dropping Portlet
*  on this PortalLayout, or when dropping other components to be auto-wrapped in
*  a Portlet. If you set this, then you will need to set an equivalent Canvas.dragType on
*  anything to be dragged into this PortalLayout (including Portlet).
* 
*  As a convenience, Portlet.dragType defaults to "Portlet". Thus, if you want 
*  to allow Portlet to be dropped on this PortalLayout, but disable 
*  auto-wrapping of other components, you can set portletDropTypes to 
*  ["Portlet"].
* 
*  If you want to allow some Portlet to be dropped on this PortalLayout but 
*  not others, then set a custom Portlet.dragType for the Portlet, and 
*  set portletDropTypes to match.
* 
*  If you want to have different dropTypes for PortalLayout.row and 
*  PortalLayout.rowLayout, you can specify dropType on the PortalLayout.row 
*  or PortalLayout.rowLayout autochildren instead.
* 
*  For more control over what can be dropped, you can also implement 
*  PortalLayout.willAcceptPortletDrop.
* @type {Array<Partial<string>>}
* @default null
*/
portletDropTypes?: Array<Partial<string>>;

/**
* dropTypes is set to ["PortalColumn"]
*  in order to allow the dragging of columns within the PortalLayout. 
*  To control dropTypes when Portlet or other components are dragged
*  into the PortalLayout, use PortalLayout.portletDropTypes instead.
* @type {Array<Partial<string>>}
* @default "["PortalColumn"]"
*/
dropTypes?: Array<Partial<string>>;

/**
* Automatically generated horizontal Layout used to create rows of Portlet via
*  Class.createAutoChild. Since this is an AutoChild, you can use 
*  rowDefaults and rowProperties to customize the rows.
*  
*  Rows are created inside PortalLayout.rowLayout, which in turn are inside PortalLayout.column.
* @type {Layout}
* @default null
*/
row?: Layout;

/**
* Automatically generated vertical Layout used to create columns of Portlet via
*  Class.createAutoChild. Since this is an AutoChild, you can use 
*  rowLayoutDefaults and rowLayoutProperties to customize the layout used to contain the rows.
*  
*  The rowLayout is the actual container for PortalLayout.row of Portlet. See PortalLayout.column for
*  the column as a whole, which may include a menu as well (depending on PortalLayout.showColumnMenus).
*  If you want to style the columns as a whole,
*  use columnDefaults or columnProperties, but if you want to style the layout that actually contains the
*  rows, use rowLayoutDefaults or rowLayoutProperties.
* @type {Layout}
* @default null
*/
rowLayout?: Layout;

/**
* Automatically generated vertical Layout used to create columns of Portlet via
*  Class.createAutoChild. Since this is an AutoChild, you can use 
*  columnDefaults and columnProperties to customize the columns.
*  
*  The column includes a menu, if PortalLayout.showColumnMenus is true, and a PortalLayout.rowLayout which
*  actually contains the PortalLayout.row. Therefore, if you want to style the columns as a whole,
*  use columnDefaults or columnProperties, but if you want to style the layout that contains the
*  rows, use rowLayoutDefaults or rowLayoutProperties.
* @type {Layout}
* @default null
*/
column?: Layout;



/**
*  Sets PortalLayout.columnOverflow and updates existing columns to reflect the new setting.
* 
* @param {Overflow} Overflow setting for columns
*/
setColumnOverflow(overflow:Overflow): void;

/**
*  Sets PortalLayout.canStretchColumnWidths and reflows to reflect the new setting.
* 
* @param {boolean} Whether columns can stretch to accommodate Portlet widths.
*/
setCanStretchColumnWidths(canStretch:boolean): void;

/**
*  Sets PortalLayout.canShrinkColumnWidths and reflows to reflect the new setting.
* 
* @param {boolean} Whether columns can shrink to avoid overflowing the PortalLayout's width.
*/
setCanShrinkColumnWidths(canShrink:boolean): void;

/**
*  Sets PortalLayout.stretchColumnWidthsProportionally and reflows to reflect the new setting.
* 
* @param {boolean} Whether to stretch column widths proportionally
*/
setStretchColumnWidthsProportionally(stretchProportionally:boolean): void;

/**
*  Sets PortalLayout.preventUnderflow and reflows the layout to implement it.
* 
* @param {boolean} Whether to stretch the last column to fill the PortalLayout's width.
*/
setPreventUnderflow(preventUnderflow:boolean): void;

/**
*  Sets PortalLayout.preventColumnUnderflow and reflows the layout to implement it.
* 
* @param {boolean} Whether to stretch the last Portlet in a column to fill the column's height.
*/
setColumnPreventUnderflow(preventColumnUnderflow:boolean): void;

/**
*  Sets PortalLayout.preventRowUnderflow and reflows the layout to implement it.
* 
* @param {boolean} Whether to stretch the last Portlet in a row to to fill the row's width.
*/
setPreventRowUnderflow(preventRowUnderflow:boolean): void;

/**
*  Sets PortalLayout.columnSpacing and reflows the layout to implement it.
* 
* @param {Integer} The amount of space to apply between columns
*/
setColumnSpacing(spacing:Integer): void;

/**
*  Sets PortalLayout.portletHSpacing and reflows the layout to implement it.
* 
* @param {Integer} The amount of space to apply between portlets in a row
*/
setPortletHSpacing(spacing:Integer): void;

/**
*  Sets PortalLayout.portletVSpacing and reflows the layout to implement it.
* 
* @param {Integer} The amount of space to apply between rows
*/
setPortletVSpacing(spacing:Integer): void;

/**
*  Returns the current number of columns displayed in this PortalLayout.
* 
*/
getNumColumns(): Integer;

/**
*  Sets PortalLayout.showColumnMenus and updates existing columns to reflect the new setting.
* 
* @param {boolean} Whether to show column menus
*/
setShowColumnMenus(showMenus:boolean): void;

/**
*  Sets the columnBorder for to the specified value and updates any drawn columns to reflect
*  this.
* 
* @param {string} New border to show around columns
*/
setColumnBorder(columnBorder:string): void;

/**
*  Set whether columns in this portalLayout are drag-resizable, and update any
*  drawn columns to reflect this.
* 
* @param {boolean} Whether columns are drag-resizable
*/
setCanResizeColumns(canResize:boolean): void;

/**
*  Set whether vertical drag-resize of portlets within columns is allowed, and
*  update any drawn columns to reflect this.
* 
* @param {boolean} Whether drag-resize of portlets within columns is allowed
* @deprecated * Use PortalLayout.setCanResizePortlets instead.

*/
setCanResizeRows(canResize:boolean): void;

/**
*  Set whether the height and width of Portlet should be drag-resizable, and
*  update any drawn Portlets to reflect this.
* 
* @param {boolean} Whether drag-resizing the height and width of portlets is allowed
*/
setCanResizePortlets(canResize:boolean): void;

/**
*  Sets the PortalLayout.portletDropTypes to be applied when dropping Portlet
*  on this PortalLayout, or when dropping other components to be auto-wrapped in
*  a Portlet.
* 
* 
* @param {Array<Partial<string>>} dropTypes to apply when dropping Portlet
*/
setPortletDropTypes(portletDropTypes:Array<Partial<string>>): void;

/**
*  Returns true if the dragged Portlet, or other component, can be dropped onto
*  this PortalLayout (other components would be auto-wrapped in a Portlet).
* 
*  The default implementation acts like Canvas.willAcceptDrop, except applying
*  PortalLayout.portletDropTypes rather than PortalLayout.dropTypes.
*  You can subclass to apply other (or additional) criteria
* 
* 
* @param {Canvas} The Portlet, or other component, being dragged
* @param {Integer} indicates which column the portlet would be dropped on.
* @param {Integer} indicates the row number being dropped on.
* @param {Integer=} Drop position within an existing row. If the dropPosition  is null, then that means that a new row will be created.
*/
willAcceptPortletDrop(dragTarget:Canvas, colNum:Integer, rowNum:Integer, dropPosition?:Integer): boolean;

/**
*  This method is called when the user drops components into the rows or columns of this
*  PortalLayout.
*  
*  Overriding this method allows you to modify drop behaviour when creating or reordering
*  portlets via drag & drop. You can return the dragTarget for the standard behavior, 
*  or null to cancel the drop.
*  
*  Otherwise, return the component you want to be dropped (as for Layout.getDropComponent).
*  You will generally want to return a Portlet or subclass. However, you can return
*  any Canvas, and it will automatically be wrapped in a Portlet if necessary.
* 
* @param {Canvas} drag target
* @param {Integer} indicates which column the portlet is being dropped on.
* @param {Integer} indicates the row number being dropped on.
* @param {Integer=} Drop position within an existing row. If the dropPosition  is null, then that means that a new row will be created.
*/
getDropPortlet(dragTarget:Canvas, colNum:Integer, rowNum:Integer, dropPosition?:Integer): Canvas;

/**
*  Adds a new portal column to this layout at the specified position
* 
* @param {Integer} target position for the new column
*/
addColumn(index:Integer): void;

/**
*  Removes the specified column from this layout.
*  All portlets displayed within this column will be destroyed when the column is removed.
* 
* @param {Integer} column number to remove
*/
removeColumn(index:Integer): void;

/**
*  Returns a flat array of all the Portlet in this PortalLayout.
* 
*/
getPortlets(): Array<Partial<Portlet>>;

/**
*  Returns a multi-level array of the Portlet in this PortalLayout,
*  where the first level corresponds to columns, the second to rows, and the third
*  to Portlets within rows.
* 
*/
getPortletArray(): Array<Partial<any>>;

/**
*  Gets the position of the Portlet within this PortalLayout. Returns null
*  if the Portlet is not in this PortalLayout.
* 
* @param {Portlet} the Portlet for which to get the position
*/
getPortalPosition(portlet:Portlet): PortalPosition;

/**
*  Adds a Portlet instance to this portalLayout in the specified position.
*  PortalLayouts use columns to manage the positions of their portlets.
*  Each column is a vertical stack containing a number of rows. By default 
*  a portlet within a column will take up the entire width of the column (so there is
*  one portlet per row within the column), but developers may also place
*  more than one portlet side-by-side on a row within a column - see the
*  positionInExistingRow parameter.
*  
* 
* @param {Portlet} Portlet to add to this layout.
* @param {Integer=} Column in which the Portlet should be added. If unspecified, portlet will be added to the first column. If specified, but the  specified column does not exist, a column is automatically added at the specified  colNum index.
* @param {Integer=} Row-position within the specified column for this portlet. If unspecified defaults to zero - the portlet will be added to the top of the column.  By default a new row will be added to the column for the portlet. Use the  positionInExistingRow parameter to add the portlet to an existing  row.
* @param {Integer=} Position within an existing row in the column. If this parameter is passed, this portlet will be added to the  existing row at rowWithinCol, at the specified position.  This allows developers to place multiple portlets side by side on a row within  the column.  If omitted a new row will be created in the column for the portlet.
*/
addPortlet(portlet:Portlet, colNum?:Integer, rowWithinCol?:Integer, positionInExistingRow?:Integer): void;

/**
*  Sets the width of a column in the PortalLayout.
*  
*  Note that this sets the intrinsic width of the column. Columns may also
*  automatically stretch and shrink to accommodate the width of
*  Portlet.
* 
* @param {Integer} Which column's width to set.
* @param {number | string} How wide to make the column
*/
setColumnWidth(colNumber:Integer, width:number | string): void;

/**
*  Gets the width of a column in the PortalLayout.
* 
* @param {Integer} Which column's width to get
*/
getColumnWidth(colNumber:Integer): Integer;

/**
*  Removes a Portlet which is currently rendered in this PortalLayout.
*  Portlet will not be destroyed by default - if this is desired the calling code should
*  do this explicitly.
* 
* @param {Portlet} portlet to remove
*/
removePortlet(portlet:Portlet): void;

/**
*  Method called when a Portlet in this PortalLayout is about to be
*  maximized. Note that this method is only called when the user explicitly
*  clicks on the portlet's Window.showMaximizeButton --
*  it is not called when programmatically maximizing a portlet via
*  Window.maximize.
*  
*  Return false to cancel the action.
*  
* 
* @param {Portlet} the Portlet which will be maximized
*/
willMaximizePortlet(portlet:Portlet): boolean;

/**
*  Notification method called after a portlet has been maximized (whether by
*  user action or programmatically).
* 
* 
* @param {Portlet} the Portlet which was maximized
*/
portletMaximized(portlet:Portlet): void;

/**
*  Method called when a Portlet in this PortalLayout is about to be
*  minimized. Note that this method is only called when the user explicitly
*  clicks on the portlet's Window.showMinimizeButton --
*  it is not called when programmatically minimizing a portlet via
*  Window.minimize.
*  
*  Return false to cancel the action.
*  
* 
* @param {Portlet} the Portlet which will be minimized
*/
willMinimizePortlet(portlet:Portlet): boolean;

/**
*  Notification method called after a portlet has been minimized (whether by
*  user action or programmatically).
* 
* 
* @param {Portlet} the Portlet which was minimized
*/
portletMinimized(portlet:Portlet): void;

/**
*  Method called when a Portlet in this PortalLayout is about to be
*  restored to its normal place (after having been
*  PortalLayout.portletMaximized. Note that this method is only
*  called when the user explicitly clicks on the portlet's
*  Window.restoreButton -- it is not called when
*  programmatically restoring a portlet via Window.restore.
*  
*  Return false to cancel the action.
*  
* 
* @param {Portlet} the Portlet which will be restored
*/
willRestorePortlet(portlet:Portlet): boolean;

/**
*  Notification method called after a portlet has been restored to its normal place
*  (after having been maximized). The method is called whether the restore is
*  via user action or done programmatically.
* 
* 
* @param {Portlet} the Portlet which was restored
*/
portletRestored(portlet:Portlet): void;

/**
*  Method called when a Portlet in this PortalLayout is about to be closed.
*  This method is called before Portlet.showCloseConfirmationMessage is applied.
*  Note that this method is called only when the user explicitly closes a Portlet.
*  It is not called when programmatically removing a Portlet via PortalLayout.removePortlet.
*  
*  Return false to cancel the action.
*  
* 
* @param {Portlet} the Portlet which will be closed
*/
willClosePortlet(portlet:Portlet): boolean;

/**
*  Fires at initialization if the PortalLayout has any initial
*  Portlet, and then fires whenever portlets are added,
*  removed or reordered.
* 
* 
*/
portletsChanged(): void;

/**
*  Fires when Portlet or columns in this PortalLayout are resized.
*  Note that this fires on a short delay -- otherwise, it would fire multiple times
*  for each change, since most portlet size changes will affect multiple portlets.
*  Does not fire when a portlet is PortalLayout.portletMaximized or
*  PortalLayout.portletRestored.
* 
*/
portletsResized(): void;



}
/**
* Creates a new PortalLayout
* 
* @param typeCheckedProperties {Partial<PortalLayout>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {PortalLayout} The new instance
*/
class PortalLayout{
static create(typeCheckedProperties? : Partial<PortalLayout>, uncheckedProperties?: Object): PortalLayout;







}

/**
* Custom subclass of Window configured to be embedded within a PortalLayout.
*/
interface Portlet extends Window {
/**
* By default, PortalLayout.portletDropTypes is set so that any component can be
*  dragged into a PortalLayout. If the component is not a Portlet,
*  it will be automatically be wrapped in a newly created Portlet.
* 
*  If you prefer to only allow real Portlet to be dragged into a 
*  PortalLayout, then you can set PortalLayout.portletDropTypes to 
*  ["Portlet"], since Portlet.dragType defaults to 
*  "Portlet".
* 
*  if you want to allow some Portlet to be dropped on a PortalLayout 
*  but not others, then you can specify a custom dragType for Portlet, 
*  and then set PortalLayout.portletDropTypes as appropriate.
* @type {string}
* @default "Portlet"
*/
dragType?: string;

/**
* Portlets have canDrop set to true to enable drag/drop reposition within the portalLayout
* @type {boolean}
* @default true
*/
canDrop?: boolean;

/**
* Specifies a minimum height for the Portlet. The height of rows within a PortalLayout
*  will be adjusted to take into account the minHeight of all the Portlets in that row.
* @type {number}
* @default "60"
*/
minHeight?: number;

/**
* Specifies a minimum width for the Portlet.
* @type {number}
* @default "70"
*/
minWidth?: number;

/**
* If you initialize the height of a Portlet, then that height will be used as the 
*  Portlet's Portlet.rowHeight (if no rowHeight is set).
*  
*  After initialization, the PortalLayout manages the height of Portlets. If you
*  want to change the height, use Portlet.setRowHeight.
* @type {number | string}
* @default null
*/
height?: number | string;

/**
* If you set the rowHeight of a Portlet before adding it to a PortalLayout, then
*  the height will be used when creating the new row. If adding the Portlet
*  to an existing row (or dragging it there), the Portlet's rowHeight will be used if 
*  the row's height has not already been specified. However, if you 
*  set the rowHeight of a Portlet after adding it to the PortalLayout, then the height
*  of the entire row will always be adjusted to match.
*  
*  You can also just specify Canvas.height when initializing a Portlet, and it
*  will be applied to the rowHeight when added to a PortalLayout. However, changing the Portlet's
*  height after initialization will not affect the row.
*  
*  Note that getting the rowHeight will indicate the rowHeight specified for this Portlet,
*  not the actual height of the row it is in.
* @type {number | string}
* @default null
*/
rowHeight?: number | string;

/**
* If true, Portlet.closeConfirmationMessage will be displayed before portlets are closed
* @type {boolean}
* @default true
*/
showCloseConfirmationMessage?: boolean;

/**
* Confirmation message to show the user when closing portlets if
*  Portlet.showCloseConfirmationMessage is true.
* @type {string}
* @default "Close portlet?"
*/
closeConfirmationMessage?: string;

/**
* If specified, this properties block will be passed to isc.confirm
*  
*  as the properties parameter when the Portlet.closeConfirmationMessage is shown, 
*  allowing developers to customize the appear of the confirmation
*  dialog (modifying its title, etc).
* @type {Partial<Dialog>}
* @default null
*/
closeConfirmationDialogProperties?: Partial<Dialog>;

/**
* Whether to call Canvas.destroy when closing the Portlet.
* @type {boolean}
* @default false
*/
destroyOnClose?: boolean;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "PortletEditProxy"
*/
editProxyConstructor?: SCClassName;



/**
* 
*  The height of a Portlet is managed by the PortalLayout. If you want to change
*  the Portlet's height, use Portlet.setRowHeight instead.
* 
* 
* @param {number} new height
*/
setHeight(height:number): void;

/**
*  Sets the height of the Portlet's row (and, thus, indirectly sets the Portlet's own height).
*  Use this instead of using Portlet.setHeight directly.
* 
* @param {number} new height
*/
setRowHeight(height:number): void;

/**
*  Gets the PortalLayout which encloses this Portlet (or null, if none).
* 
*/
getPortalLayout(): PortalLayout;

/**
*  Gets the position of the Portlet within its PortalLayout. Returns null
*  if the Portlet is not in a PortalLayout.
* 
*/
getPortalPosition(): PortalPosition;

/**
*  Handles a click on the close button of this portlet. The default implementation
*  calls Portlet.close.
*  
*  Override this method if you want
*  other actions to be taken.
*  
*  Custom implementations may call close() to trigger the default behavior.
* 
*/
closeClick(): boolean;

/**
*  close() method overridden to show 
*  Portlet.closeConfirmationMessage to the user before
*  removing the portlet from the PortalLayout via PortalLayout.removePortlet
* 
*/
close(): void;



}
/**
* Creates a new Portlet
* 
* @param typeCheckedProperties {Partial<Portlet>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Portlet} The new instance
*/
class Portlet{
static create(typeCheckedProperties? : Partial<Portlet>, uncheckedProperties?: Object): Portlet;







}

/**
* A FormItem for use with the FilterBuilder, allows the user to pick from a set of 
*  pre-configured search criteria such as specific ranges in numeric or date data, and provide 
*  user friendly titles for such criteria, such as "within the next two weeks" or 
*  "High (0.75 - 0.99)".
*/
interface PresetCriteriaItem extends SelectItem {
/**
* This attribute is not applicable to the PresetCriteriaItem. See 
*  PresetCriteriaItem.options instead.
* @type {object}
* @default null
*/
valueMap?: object;

/**
* An object whose properties are user-visible titles for the preset ranges, and whose 
*  values are Criteria / AdvancedCriteria objects representing the criteria to be used if 
*  the user selects that choice.
* @type {object}
* @default null
*/
options?: object;

/**
* If set, an additional option will be shown with the title PresetCriteriaItem.customOptionTitle, 
*  which will cause PresetCriteriaItem.getCustomCriteria to be called.
* @type {boolean}
* @default false
*/
showCustomOption?: boolean;

/**
* The title to show for the PresetCriteriaItem.showCustomOption.
* @type {string}
* @default "Custom..."
*/
customOptionTitle?: string;

/**
* Should this item's value be saved in the form's values and hence returned from
*  DynamicForm.getValues?
*  
*  shouldSaveValue:false is used to mark formItems which do not correspond to
*  the underlying data model and should not save a value into the form's
*  DynamicForm.values. Example includes visual separators, password re-type fields,
*  or checkboxes used to show/hide other form items.
*  
*  A shouldSaveValue:false item should be given a value either via
*  FormItem.defaultValue or by calling
*  DynamicForm.setValue or 
*  FormItem.setValue. Providing a value via
*  DynamicForm.values or DynamicForm.setValues 
*  will automatically switch the item to shouldSaveValue:true.
*  
*  Note that 
*  
* - if an item is shouldSaveValue true, but has no name, a warning is logged, and 
*    shouldSaveValue will be set to false.
*  
* @type {boolean}
* @default true
*/
shouldSaveValue?: boolean;



/**
*  This method is called when PresetCriteriaItem.showCustomOption is true and the user selects the 
*  custom option. Implement this method by allowing the user to enter custom criteria, for
*  example, by opening a modal dialog. Once the user has input custom criteria, fire the
*  callback method with the resulting criteria.
*  
* 
* @param {Callback} callback to fire when custom criteria has been gathered.       Expects parameters "criteria,title". The "title" will be displayed as the        currently selected value when custom criteria have been chosen.
*/
getCustomCriteria(callback:Callback): void;

/**
*  Get the criterion based on the value selected by the user.
* 
* 
*/
getCriterion(): Criterion | AdvancedCriteria;



}
/**
* Creates a new PresetCriteriaItem
* 
* @param typeCheckedProperties {Partial<PresetCriteriaItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {PresetCriteriaItem} The new instance
*/
class PresetCriteriaItem{
static create(typeCheckedProperties? : Partial<PresetCriteriaItem>, uncheckedProperties?: Object): PresetCriteriaItem;







}

/**
* Allows the user to pick from pre-set date ranges or choose a custom date range via a
*  DateRangeDialog.
*  
*  To use this item in the ListGrid.showFilterEditor or 
*  FilterBuilder, create a trivial ClassFactory.defineClass which 
*  defines PresetCriteriaItem.options, then set
*  ListGridField.filterEditorType to use this class with the FilterEditor, or define a
*  custom operator and set Operator.editorType to use it with the FilterBuilder.
*  
*  See the ${isc.DocUtils.linkForExampleId('dateRangeFilterPresets', 'Date Range (Presets)')} example for sample code.
*/
interface PresetDateRangeItem extends PresetCriteriaItem {


/**
*  Get the criterion based on the value selected by the user.
* 
* 
*/
getCriterion(): Criterion | AdvancedCriteria;



}
/**
* Creates a new PresetDateRangeItem
* 
* @param typeCheckedProperties {Partial<PresetDateRangeItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {PresetDateRangeItem} The new instance
*/
class PresetDateRangeItem{
static create(typeCheckedProperties? : Partial<PresetDateRangeItem>, uncheckedProperties?: Object): PresetDateRangeItem;







}

/**
* PrintCanvas is a subclass of canvas which renders printable content HTML and 
*  provides APIs for printing this content as a separate document.
*/
interface PrintCanvas extends Canvas {
/**
* Location of the special printFrame html file provided as part of the SmartClient libraries.
*  This file must be present at the specified location for the printCanvas printing APIs.
* @type {string}
* @default "[HELPERS]printFrame.html"
*/
printFrameURL?: string;

/**
* Setting this property will cause the specified stylesheet to be loaded in this print
*  canvas's frame.
*  The stylesheet should be specified as a URL to load.
* @type {string}
* @default null
*/
externalStylesheet?: string;



/**
*  Update the HTML content displayed in this print canvas. If the printCanvas is not yet
*  drawn the HTML will be displayed when the canvas is drawn.
*  
*  Note that if the printCanvas is Canvas.redraw, or 
*  Canvas.clear and then Canvas.draw again, the HTML will
*  be redisplayed inside the print canvas, and the specified callback will be fired again.
* 
* 
* @param {string} HTML to show in this print canvas
* @param {PrintCanvasCallback} callback function to fire when the HTML is displayed. The callback will be passed a pointer to this print canvas as the first parameter with the  name printPreview. If this canvas is not drawn when this method is called,  the callback will not be fired until the canvas is drawn and the HTML rendered out into  the page.
*/
setHTML(HTML:string, callback:(printPreview:PrintCanvas) => void): void;

/**
*  Show the native print dialog and allow the user to print the current HTML for
*  this printCanvas. Note that the PrintCanvas must be drawn to be printed.
* 
*/
print(): void;



}
/**
* Creates a new PrintCanvas
* 
* @param typeCheckedProperties {Partial<PrintCanvas>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {PrintCanvas} The new instance
*/
class PrintCanvas{
static create(typeCheckedProperties? : Partial<PrintCanvas>, uncheckedProperties?: Object): PrintCanvas;







}

/**
* Print canvas by showing print preview.
*/
interface PrintCanvasTask extends ComponentTask {
/**
* PrintProperties object for customizing the print HTML output.
* @type {PrintProperties}
* @default null
*/
printProperties?: PrintProperties;

/**
* Properties to apply to the generated print window.
* @type {PrintWindow}
* @default null
*/
printWindowProperties?: PrintWindow;





}
/**
* Creates a new PrintCanvasTask
* 
* @param typeCheckedProperties {Partial<PrintCanvasTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {PrintCanvasTask} The new instance
*/
class PrintCanvasTask{
static create(typeCheckedProperties? : Partial<PrintCanvasTask>, uncheckedProperties?: Object): PrintCanvasTask;







}

/**
* Subclass of Window used for displaying a printable view. Includes a "Print" button
*  header control to trigger printing of content.
*/
interface PrintWindow extends Window {
/**
* Title for the print window
* @type {string}
* @default "Print Preview"
*/
title?: string;

/**
* Title for the print button
* @type {string}
* @default "Print"
*/
printButtonTitle?: string;

/**
* Setting this property will cause the specified stylesheet to be loaded in this
*  window's printable HTML frame.
*  
*  The stylesheet should be specified as a URL to load.
* @type {string}
* @default null
*/
externalStylesheet?: string;



/**
*  Setter for title for the print button
* 
* @param {string} new title for the print button
*/
setPrintButtonTitle(printButtonTitle:string): void;



}
/**
* Creates a new PrintWindow
* 
* @param typeCheckedProperties {Partial<PrintWindow>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {PrintWindow} The new instance
*/
class PrintWindow{
static create(typeCheckedProperties? : Partial<PrintWindow>, uncheckedProperties?: Object): PrintWindow;







}

/**
* A instance of Process represents a stateful process executing a series of Tasks, 
*  which may be:
*  
*  
* - user interactions
*  
* - calls to DataSources (hence: any database or web service)
*  
* - arbitrary code
*  
* - other Processes
*  
*  A Process is stateful in the sense that it maintains Process.state
*  across the different tasks that are executed. This allows you to maintain context as you
*  walk a user through a multi-step business process in your application, which may involve
*  multiple operations on multiple entities. Each Task that executes can use the Process state
*  as inputs, and can output a result which is stored in the Process state - see
*  taskIO.
*  
*  A Process can have multiple branches, choosing the next Task to execute based on
*  Criteria - see DecisionTask and MultiDecisionTask.
*  
*  Because a Process may return to a previous Task in various situations, the data model of a
*  Process is strictly speaking a graph (a set of nodes connected by arbitary
*  interlinks). However, most processes have sequences of several tasks in a row, and the
*  definition format allows these to be represented as simple Arrays called "sequences",
*  specified via Process.sequences. This reduces the need to manually specify IDs and
*  interlinks for Tasks that simply proceed to the next task in a sequence.
*  
*  Processes follow all the standard rules for encoding as componentXML, however,
*  note that the &lt;Process&gt; tag allows any kind of ProcessElement (tasks, decisions
*  and sequences) to appear as a direct subelement of the &lt;Process&gt; tag without the need
*  for an intervening &lt;elements&gt; or &lt;sequences&gt; tag. The example below
*  demonstrates this shorthand format.
*  
*  &lt;Process ID="processId"&gt;
*    &lt;ServiceTask ID="serviceTaskId" nextElement="sequenceId" ..&gt;
*      &lt;inputFieldList&gt;
*        &lt;value&gt;order.countryName&lt;/value&gt;
*      &lt;/inputFieldList&gt;
*      &lt;outputFieldList&gt;
*        &lt;value&gt;order.countryName&lt;/value&gt;
*        &lt;value&gt;order.continent&lt;/value&gt;
*      &lt;outputFieldList&gt;
*    &lt;/ServiceTask&gt;
*    &lt;sequence ID="sequenceId" &gt;
*      &lt;StateTask ../&gt;
*      &lt;StateTask ../&gt;
*      &lt;StateTask ../&gt;
*      &lt;StateTask nextElement="userTaskId" ../&gt;
*    &lt;/sequence&gt;
*    &lt;UserTask ID="userTaskId" ../&gt;
*    ...
*  &lt;/Process&gt;
*  
*  NOTE: you must load the standard DataBinding module before you can use Process.
*/
interface Process extends Class {
/**
* Sequences of ProcessElements. By defining a sequences of elements you can make the
*  ProcessElement.nextElement implicit.
*  
*  You do not have to explicitly create a ProcessSequence,
*  you can instead use the shorthand:
*  
*  isc.Process.create({
*    startElement:"firstSequence", 
*    sequences: [
*      { ID:"something", elements: [ ... ] },
*      { ID:"somethingElse", elements: [ ... ] },
*      ...
*    ]
*    ...
*  });
*  
*  .. this is equivalent to ..
*  
*  isc.Process.create({
*    startElement:"firstSequence", 
*    sequences: [
*      isc.ProcessSequence.create({ 
*        ID:"something", 
*        elements: [ ... ] 
*      }),
*      isc.ProcessSequence.create({ 
*        ID:"somethingElement", 
*        elements: [ ... ] 
*      }),
*      ...              
*    ]
*    ...
*  });
*  
*  
*  
* @type {Array<Partial<ProcessSequence>>}
* @default null
*/
sequences?: Array<Partial<ProcessSequence>>;

/**
* Elements involved in this Process. You can also group elements into Process.sequences
*  to reduce the need to explicitly define IDs for elements and interlink them.
* @type {Array<Partial<ProcessElement>>}
* @default null
*/
elements?: Array<Partial<ProcessElement>>;

/**
* The ID of either a Process.sequences or an Process.elements which should
*  be the starting point of the process. If not specified, the first sequence is chosen,
*  or if there are no sequences, the first element.
*  - log a warning and do nothing if there are neither sequences or elements
*  
*  - an example of how a Process would be defined
*  
*  isc.Process.create({
*    startElement:"firstSequence", 
*    sequences: [
*      { 
*       id:"firstSequence",
*       elements : [
*         isc.ServiceTask.create({ .. }),
*         isc.MultiDecisionTask.create({ .. })
*       ]
*      },
*      {
*       id:"errorFlow",
*       elements : [ ... ]
*       
*      }
*    ],
*    elements: [
*     // standalone process elements not part of sequences
*     isc.ServiceTask.create({ .. })
*    ],
*    state : {
*      someField:"someValue"
*    }
*  })
*  
*  
* @type {string}
* @default null
*/
startElement?: string;

/**
* If wizard is set then current workflow will be handled as wizard. Every userTask will
*  hide associated form after user finished step.
* @type {boolean}
* @default false
*/
wizard?: boolean;

/**
* Enable mock mode on the workflow? By default, this setting does nothing but is available
*  for individual tasks to trigger special action. For example, a task that would normally
*  fail outside of its target environment can take an alternative action during testing.
*  
*  mockMode can also be enabled or disabled for an individual task with
*  ProcessElement.mockMode.
* @type {boolean}
* @default null
*/
mockMode?: boolean;

/**
* Identifier of canvas where UI elements created by using
*  UserTask.inlineView property should be added
*  using addMember.
* @type {GlobalId}
* @default null
*/
containerId?: GlobalId;

/**
* Current state of a process. As with Records in general, any field of a Record may
*  contain a nested Record or Array of Records, so the process state is essentially a
*  hierarchical data structure.
*  
*  Transient state
*  In addition to the explicit process state there is a "transient state." The transient
*  state represents the complete output of each of the last tasks of each type within the current
*  process execution. This allows easy reference to the previous task output with
*  taskInputExpression.
* @type {Record}
* @default null
*/
state?: Record;

/**
* Canvas.ID of the component that manages "rule context" for which
*  this process participates. The rule context can be used in
*  taskInputExpression.
* @type {string}
* @default null
*/
ruleScope?: string;

/**
* Cause the process to automatically call Process.start as soon as it is created.
* @type {boolean}
* @default false
*/
autoStart?: boolean;

/**
* Context object to be passed to Process.traceElement during process
*  execution.
* @type {object}
* @default null
*/
traceContext?: object;



/**
*  Retrieve a ProcessElement by it's ID
* 
* @param {string} id of the process element
*/
getElement(ID:string): ProcessElement;

/**
*  Set process state for current process
* 
* @param {Record} the new process state
*/
setState(state:Record): void;

/**
*  Starts this task by executing the Process.startElement. Also used by asynchronous
*  tasks to restart the workflow.
* 
*/
start(): void;

/**
*  Reset process to it's initial state, so process can be started again.
* 
* @param {Record=} new state of the process
*/
reset(state?:Record): void;

/**
*  Sets the task ID of the next task to execute after the current task finishes. If the
*  task is not found or null is passed as the nextElement, the current
*  process will be terminated instead.
* 
* 
* @param {string=} ID of the next task execute or null to terminate process
*/
setNextElement(nextElement?:string): void;

/**
*  Sets a Process.state variable for later reference with
*  Process.getStateVariable or more commonly with a taskInputExpression
*  property.
* 
* 
* @param {string} path to variable in process state to set.                  segments are separated by a decimal point (.)
* @param {any} the value to save
*/
setStateVariable(stateVariablePath:string, value:any): void;

/**
*  Returns a variable value from the Process.state. Values can
*  be written into process state by Process.setStateVariable, setting
*  ProcessElement.bindOutput, or various task output settings (See
*  taskIO.)
* 
* 
* @param {string} path to variable in process state to set.                  segments are separated by a decimal point (.)
*/
getStateVariable(stateVariablePath:string): any;

/**
*  Sets the task output of task in the State so it
*  can be used by later tasks with Process.getLastTaskOutput or more commonly with a
*  taskInputExpression property.
*  
*  If the task sets bindOutput the output value is also written into that
*  Process.state variable.
* 
* 
* @param {ProcessElement} the workflow task setting the output (i.e. this)
* @param {any} the output value for task
*/
setTaskOutput(task:ProcessElement, value:any): void;

/**
*  Takes the Process.getLastTaskOutput and sets it as the
*  Process.setTaskOutput for the task.
*  
*  This method is not just a shortcut to set output of a pass-thru task but it
*  also records the correct schema of the passed-thru output so it can be
*  quickly looked up.
*  
* 
* @param {ProcessElement} the workflow task setting the output (i.e. this)
*/
passThruTaskOutput(task:ProcessElement): void;

/**
*  Returns the task output of the last task executed. More commonly a
*  taskInputExpression property is used (see
*  ProcessElement.getDynamicValue).
* 
* 
* @param {string=} the optional task type to lookup in last task output
*/
getLastTaskOutput(taskType?:string): any;

/**
*  StringMethod called when a process completes, meaning the process executes a 
*  ProcessElement with no next element.
* 
* @param {Record} the final process state
*/
finished(state:Record): void;

/**
*  StringMethod called during process execution before each task
*  element is processed.
* 
* @param {Task} the Task being executed
* @param {object} the Process.traceContext, if set
*/
traceElement(element:Task, context:object): void;



}
/**
* Creates a new Process
* 
* @param typeCheckedProperties {Partial<Process>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Process} The new instance
*/
class Process{
static create(typeCheckedProperties? : Partial<Process>, uncheckedProperties?: Object): Process;
/**
* Value of failureElement in various ProcessElement to
*  indicate a placeholder is being used. Also applies to DecisionBranch.targetTask
*  and MultiDecisionTask.defaultElement.
* @type {string}
* @default "[placeholder]"
*/
static decisionPlaceholderSelection?: string;





/**
*  Loads an XML process definition stored in XML from the server.
*  
*  This method requires server-side support included in SmartClient Pro Edition or better.
*  If you are using SmartClient LGPL, Processes must be defined programmatically 
*  in JavaScript.
*  
*  
*  Process files are stored as .proc.xml files in componentXML
*  format, in the directory indicated by the project.processes setting in
*  server_properties
*  (webroot/processes by default). To load a process
*  saved in a file processId.proc.xml, pass just processId to this method.
*  
*  
* 
* @param {Identifier | Array<Partial<Identifier>>} process ID or IDs to load
* @param {ProcessCallback} called when the process is loaded with argument              "process", the first process. Other processes can be looked               up via Process.getProcess.
*/
static loadProcess(processId:Identifier | Array<Partial<Identifier>>, callback:(process:Process) => void): void;

/**
*  Get a Process instance by it's ID. See Process.loadProcess.
* 
* @param {Identifier} process ID to retrieve
*/
static getProcess(processId:Identifier): Process;




}

/**
* A ProcessElement is an abstract superclass for elements involved in a Process, such
*  as a Task or DecisionTask.
*/
interface ProcessElement extends Class {
/**
* Optional ID for this process element, allowing it to be referred to from 
*  MultiDecisionTask, or as the Process.startElement.
*  See ProcessSequence and Process to understand when this is required
*  or can be omitted.
*  
*  Unlike Canvas.ID a processElement's is a not a globally unique
*  variable, it need only by unique within it's process.
*  
*  When assigned an ID, a processElement can be retrieve via
*  Process.getElement.
* @type {string}
* @default null
*/
ID?: string;

/**
* Next Process.sequences or Process.elements to execute
*  after this one completes. 
*  
*  nextElement does not need to be specified on most elements if you use
*  Process.sequences.
*  
*  Note that if there is both a sequence and a normal element
*  with the same name in the current Process, the sequence will
*  be used.
* @type {string}
* @default null
*/
nextElement?: string;

/**
* Optional description for this specific instance of process element.
* @type {string}
* @default null
*/
description?: string;

/**
* Optional short, descriptive title for this process element. Used by an editor as a title
*  for process elements of this type.
* @type {string}
* @default null
*/
typeTitle?: string;

/**
* Optional description of the general nature of the kinds of tasks this this process
*  element performs. Not to be confused with ProcessElement.description which describes
*  what the specific instance of the process element has been configured to do.
*  
*  For example, the classDescription for a task to disable a field might be
*  "disables a field" whereas the description for a concrete instance might
*  be "disables the 'shipTo' field in the 'ordering' form".
*  
*  Used by editor to display additional details along with ProcessElement.typeTitle.
* @type {string}
* @default null
*/
classDescription?: string;

/**
* Enable mock mode on the task? If Process.mockMode is enabled, setting this
*  property to false disables mockMode on this task only. Otherwise,
*  mock mode can be enabled on this task by setting it to true.
*  
*  Note that it is up to each task determine what effect mock mode has.
* @type {boolean}
* @default null
*/
mockMode?: boolean;

/**
* When set, the output of the task will be automatically bound to the specified
*  value in the Process.state.
*  
*  See taskInputExpression for details on the
*  transient state outputs.
* @type {string}
* @default null
*/
bindOutput?: string;

/**
* Does this processElement pass through output from the last executed task
*  (i.e. transient state)?
*  
*  See taskInputExpression for details on the
*  transient state outputs.
*  
*  Note that this property does not affect the task at all but is an indicator
*  to the user and to the workflow editor of the behavior of the task as coded
*  (See Process.passThruTaskOutput).
* @type {boolean}
* @default true
*/
passThruOutput?: boolean;

/**
* Does this processElement support being called multiple times for multiple records
*  in the Process.setTaskOutput?
*  
*  By default a processElement is ProcessElement.executeElement exactly once, however,
*  for a task that can process records from the last task output it can be useful
*  to handle each incoming record individually. Setting this property indicates to the
*  Process that if the last task output is an array, it should
*  be executed once per value in the array. Normal processing of
*  taskInputExpression or use of
*  Process.getLastTaskOutput will have exactly one record except uses of the
*  output for criteria values where the full output is used at once.
*  
*  Processing of the task can determine that multiple incoming records should not
*  result in multiple calls and set ProcessElement.forceSingle. For example, a task that
*  uses last task output for a criteria or for values should set
*  forceSingle=true when a criteria is used because multiple calls do not
*  make sense.
* @type {boolean}
* @default null
*/
supportsMultipleInputRecords?: boolean;

/**
* Should ProcessElement.supportsMultipleInputRecords be suppressed
*  for this task instance? This property can be set at any time is checked before
*  executing the task and after each execution during processing of multiple last task
*  output records.
*  
*  Note that since this property applies to an instance of a task that could be used
*  multiple times in a process (by branching) care should be taken to restore the
*  property value after execution completes. See ProcessElement.completeElement or
*  ProcessElement.reset.
* @type {boolean}
* @default null
*/
forceSingle?: boolean;

/**
* Editor type used to edit instances of this type of process element.
* @type {string}
* @default null
*/
editorType?: string;



/**
*  Returns the workflow task editor type to be used edit instances of this type of
*  process element. The default implementation returns this.editorType but
*  a custom override could determine an editor type based on the property values.
* 
* 
*/
getEditorType(): void;

/**
*  Method called by Process to have the processElement perform its work. There is
*  no default implementation by ProcessElement, however, all of the system-provided
*  subclasses do implement this method. An implementation or override of this method is
*  one possible customization point. Some classes like ScriptTask perform other
*  means to add customization. For ScriptTask, custom code is expected to handle the
*  ScriptTask.execute method instead.
*  
*  Any implementation of this method must return true if all the work this
*  element needed to perform was completed. Return false if additional work
*  is being performed asynchronously and the process should be paused until element
*  restarts it. Once asynchronous work is complete the task must call
*  Process.start to restart the workflow with the next task.
* 
* 
* @param {Process} the process that is handling the workflow
*/
executeElement(process:Process): boolean;

/**
*  Returns a text description of the element derived from
*  the configuration.
*  
*  If no override is provided by the concrete ProcessElement
*  implementation the ProcessElement.description is
*  returned.
* 
* 
*/
getElementDescription(): string;

/**
*  Does the object have fields that reference the last task output (i.e. $last)?
* 
* 
* @param {object} object to be checked
* @param {Process} the enclosing process
*/
objectReferencesLastTaskOutput(object:object, process:Process): boolean;

/**
*  Updates references to a global ID within the properties of this process element
*  (i.e. rename). This method is not called as part of workflow execution but is used
*  by Reify to keep workflow event handlers in sync with ID changes within the
*  screen.
*  
*  Each processElement or Task that has properties that save global IDs (like a
*  component ID or criteria referencing Canvas.ruleScope) must 
*  be able to update its references on demand by overriding this method or defer to its
*  superclass.
*  
*  There are a number of helper methods to make this easier listed below.
* 
* 
* @param {Identifier} the ID being renamed
* @param {Identifier} the new ID to be assigned
*/
updateGlobalIDReferences(oldId:Identifier, newId:Identifier): boolean;

/**
*  Updates a taskInputExpression property value containing ruleScope references.
*  
*  This method is a helper to implement task-specific ProcessElement.updateGlobalIDReferences.
* 
* 
* @param {string} the property name to be updated in this task
* @param {Identifier} the ID being renamed
* @param {Identifier} the new ID to be assigned
*/
updateGlobalIDInValueProperty(propertyName:string, oldId:Identifier, newId:Identifier): boolean;

/**
*  Updates a set of taskInputExpression values containing ruleScope references.
*  
*  This method is a helper to implement task-specific ProcessElement.updateGlobalIDReferences.
* 
* 
* @param {object} the object to be updated
* @param {Identifier} the ID being renamed
* @param {Identifier} the new ID to be assigned
*/
updateGlobalIDInValues(values:object, oldId:Identifier, newId:Identifier): boolean;

/**
*  Updates AdvancedCriteria Criterion taskInputExpression values
*  containing ruleScope references.
*  
*  This method is a helper to implement task-specific ProcessElement.updateGlobalIDReferences.
* 
* 
* @param {AdvancedCriteria} the criteria to be updated in place
* @param {Identifier} the ID being renamed
* @param {Identifier} the new ID to be assigned
*/
updateGlobalIDInCriteria(criteria:AdvancedCriteria, oldId:Identifier, newId:Identifier): boolean;

/**
*  Update references to a binding $last within properties of this processElement.
*  This method is not called as part of workflow execution but is used
*  by the class:WorkflowEditor to adjust last task references as new tasks are inserted.
*  
*  Each processElement or Task that has properties supporting
*  taskInputExpression using the $last syntax must 
*  be able to update its references on demand by overriding this method or defer to its
*  superclass.
*  
*  There are a number of helper methods to make this easier listed below.
* 
* 
* @param {Identifier} the ID being renamed
* @param {Identifier} the new ID to be assigned
*/
updateLastElementBindingReferences(oldId:Identifier, newId:Identifier): boolean;

/**
*  Updates a taskInputExpression property value containing $last references.
*  Any implicit reference to the last task is updated to reference a last task of a
*  specified taskType.
*  
*  For example, a value of "$last.sequenceNo" would be replaced with
*  "$last[fetch].sequenceNo" if the taskType is "fetch". Existing "$last[...]" references
*  are left as-is.
*  
*  This method is a helper to implement task-specific ProcessElement.updateLastElementBindingReferences.
* 
* 
* @param {string} the property name to be updated in this task
* @param {string} the taskType to be used in new reference
*/
updateLastElementInValueProperty(propertyName:string, taskType:string): boolean;

/**
*  Updates a set of taskInputExpression values containing $last references.
*  Any implicit reference to the last task is updated to reference a last task of a
*  specified taskType.
*  
*  For example, a value of "$last.sequenceNo" would be replaced with
*  "$last[fetch].sequenceNo" if the taskType is "fetch". Existing "$last[...]" references
*  are left as-is.
*  
*  This method is a helper to implement task-specific ProcessElement.updateLastElementBindingReferences.
* 
* 
* @param {object} the object to be updated
* @param {string} the taskType to be used in new reference
*/
updateLastElementInValues(values:object, taskType:string): boolean;

/**
*  Updates AdvancedCriteria Criterion taskInputExpression values
*  containing $last references. Any implicit reference to the last task is updated to
*  reference a last task of a specified taskType.
*  
*  For example, a value of "$last.sequenceNo" would be replaced with
*  "$last[fetch].sequenceNo" if the taskType is "fetch". Existing "$last[...]" references
*  are left as-is.
*  
*  This method is a helper to implement task-specific ProcessElement.updateLastElementBindingReferences.
* 
* 
* @param {AdvancedCriteria} the criteria to be updated in place
* @param {string} the taskType to be used in new reference
*/
updateLastElementInCriteria(criteria:AdvancedCriteria, taskType:string): boolean;

/**
*  Resolves a dynamic value as taskInputExpression or
*  returns the value as-is.
* 
* 
* @param {string} the value to resolve
* @param {Process} the current process
*/
getDynamicValue(value:string, process:Process): string;

/**
*  Resolves a UserSummary value against the current
*  Canvas.ruleScope.
* 
* 
* @param {UserSummary} the UserSummary value to resolve
* @param {Process} the current process
*/
getTextFormulaValue(textFormula:UserSummary, process:Process): string;

/**
*  StringMethod called when a processElement completes. Typically used to clear transient
*  state applied to the task while running like ProcessElement.forceSingle so it
*  can be executed again later. See also ProcessElement.reset.
* 
* @param {Process} the containing process
*/
completeElement(process:Process): void;

/**
*  StringMethod called during Process.reset giving the task a chance to reset
*  any internal state so it can be executed later. See also ProcessElement.completeElement.
* 
*/
reset(): void;



}
/**
* Creates a new ProcessElement
* 
* @param typeCheckedProperties {Partial<ProcessElement>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ProcessElement} The new instance
*/
class ProcessElement{
static create(typeCheckedProperties? : Partial<ProcessElement>, uncheckedProperties?: Object): ProcessElement;







}

/**
* An Array of ProcessElements involved in a Process. A 
*  ProcessSequence is used to reduce the number of explicit
*  ProcessElement.IDs that need to be assigned, by creating an implicit next element -
*  the next in the sequence.
*  
*  A sequence cannot be executed outside of a Process and has no state.
*/
interface ProcessSequence extends ProcessElement {
/**
* The ProcessElements in this sequence.
* @type {Array<Partial<ProcessElement>>}
* @default null
*/
elements?: Array<Partial<ProcessElement>>;





}
/**
* Creates a new ProcessSequence
* 
* @param typeCheckedProperties {Partial<ProcessSequence>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ProcessSequence} The new instance
*/
class ProcessSequence{
static create(typeCheckedProperties? : Partial<ProcessSequence>, uncheckedProperties?: Object): ProcessSequence;







}

/**
* The Progressbar widget class extends the StretchImg class to implement image-based progress
*  bars (graphical bars whose lengths represent percentages, typically of task completion).
*/
interface Progressbar {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "ProgressbarProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Number from 0 to 100, inclusive, for the percentage to be displayed graphically in
*  this progressbar.
* @type {number}
* @default "0"
*/
percentDone?: number;

/**
* Length of the progressbar in pixels. This is effectively height for a vertical
*  progressbar, or width for a horizontal progressbar.
*  
*  This property must be set instead of setting width or height.
* @type {number | string}
* @default "100"
*/
length?: number | string;

/**
* Thickness of the progressbar in pixels. This is effectively width for a vertical
*  progressbar, or height for a horizontal progressbar.
*  
*  This property must be set instead of setting width or height.
* @type {number}
* @default "20"
*/
breadth?: number;

/**
* Indicates whether this is a vertical or horizontal progressbar.
* @type {boolean}
* @default false
*/
vertical?: boolean;

/**
* The base file name for the progressbar image.
* @type {SCImgURL}
* @default "[SKIN]progressbar.gif"
*/
src?: SCImgURL;

/**
* When set to true, styles the Progressbar via the Progressbar.baseStyle and 
*  Progressbar.progressStyle CSS styles.
* @type {boolean}
* @default false
*/
useCssStyles?: boolean;

/**
* StatefulCanvas.baseStyle for this Progressbar. Only used when 
*  Progressbar.useCssStyles is true.
* @type {CSSStyleName}
* @default "progressbar"
*/
baseStyle?: CSSStyleName;

/**
* StatefulCanvas.baseStyle used to style the percentage-done portion of this
*  Progressbar. Only used when Progressbar.useCssStyles is true.
* @type {CSSStyleName}
* @default "progressbarProgress"
*/
progressStyle?: CSSStyleName;



/**
*  Retrieves dynamically calculated default Canvas.ariaState 
*  properties for this canvas. These will be combined with explicitly specified aria
*  state as described in Canvas.getAriaState.
*  
*  Overridden by Progressbar to pick up aria-valuenow.
*  
* 
*/
getAriaStateDefaults(): object;

/**
*  Sets percentDone to newPercent.
* 
* 
* @param {number} percent to show as done (0-100)
*/
setPercentDone(newPercent:number): void;

/**
*  This method is called when the percentDone value changes. Observe
*  this method to be notified upon
*  a change to the percentDone value.
* 
* 
*/
percentChanged(): void;

/**
*  Returns the current width of a horizontal progressbar, or height of a vertical progressbar.
* 
* 
*/
getLength(): number;

/**
*  Returns the current height of a horizontal progressbar, or width of a vertical progressbar.
* 
* 
*/
getBreadth(): number;

/**
*  Sets the length of the progressbar to newLength. This is the width of a horizontal progressbar,
*  or the height of a vertical progressbar.
* 
* 
* @param {number} the new length of the progressbar
*/
setLength(newLength:number): void;

/**
*  Sets the breadth of the progressbar to newLength. This is the height of a horizontal progressbar,
*  or the width of a vertical progressbar.
* 
* 
* @param {number} the new breadth of the progressbar
*/
setBreadth(newBreadth:number): void;



}
/**
* Creates a new Progressbar
* 
* @param typeCheckedProperties {Partial<Progressbar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Progressbar} The new instance
*/
class Progressbar{
static create(typeCheckedProperties? : Partial<Progressbar>, uncheckedProperties?: Object): Progressbar;







}

/**
* EditProxy that handles Progressbar objects when editMode is enabled.
*/
interface ProgressbarEditProxy extends StatefulCanvasEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's percentDone.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's percentDone.
* 
* 
* @param {string} the new component percentDone
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new ProgressbarEditProxy
* 
* @param typeCheckedProperties {Partial<ProgressbarEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ProgressbarEditProxy} The new instance
*/
class ProgressbarEditProxy{
static create(typeCheckedProperties? : Partial<ProgressbarEditProxy>, uncheckedProperties?: Object): ProgressbarEditProxy;







}

/**
* Represents a Reify project loaded from the server via RPCManager.loadProject. A
*  project contains cached screens and DataSource that can be used to create
*  actual screens by calling Project.createScreen or Project.createStartScreen.
*/
interface Project extends Class {


/**
*  Creates a screen from screen definitions cached in the project.
* 
* @param {string} name of screen to create
* @param {CreateScreenSettings | Array<Partial<string>>=} widgets to allow to take their                           global IDs, or a widget remap config
*/
createScreen(screenName:string, settings?:CreateScreenSettings | Array<Partial<string>>): Canvas;

/**
*  Creates screen from first definition cached in the project.
* 
* @param {CreateScreenSettings | Array<Partial<string>>=} widgets to allow to take their                           global IDs, or a widget remap config
*/
createStartScreen(settings?:CreateScreenSettings | Array<Partial<string>>): Canvas;

/**
*  Return the names of the screens cached in this project.
* 
*/
getScreenNames(): Array<Partial<string>>;

/**
*  Return the names of the DataSource cached in this project.
* 
*/
getDataSourceNames(): Array<Partial<string>>;

/**
*  Returns an instance of the requested DataSource by creating it from the project's
*  cache. If the ID is not globally bound, the framework will globally bind the instance
*  before returning it.
*  
*  Note that when a screen cached in the project is created, all project DataSources in the
*  screen will be automatically instantiated from the project cache, so this method need
*  not be called before creating a screen just to ensure its DataSources are available.
* 
* @param {Identifier} ID of the DataSource to create
*/
getDataSource(ID:Identifier): DataSource;

/**
*  Releases cached screens and DataSource associated with this project,
*  and unregisters it so Project.get no longer can find it by name. After
*  destroying a project, it is an error to call any instance method on it.
* 
*/
destroy(): void;



}
/**
* Creates a new Project
* 
* @param typeCheckedProperties {Partial<Project>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Project} The new instance
*/
class Project{
static create(typeCheckedProperties? : Partial<Project>, uncheckedProperties?: Object): Project;




/**
*  Returns a cached project given its name.
* 
* @param {string} name of project to retrieve
*/
static get(projectName:string): Project;




}

/**
* Editor with a minimalist appearance, tuned for editing large numbers of properties in a
*  constrained space.
*/
interface PropertySheet extends DynamicForm {




}
/**
* Creates a new PropertySheet
* 
* @param typeCheckedProperties {Partial<PropertySheet>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {PropertySheet} The new instance
*/
class PropertySheet{
static create(typeCheckedProperties? : Partial<PropertySheet>, uncheckedProperties?: Object): PropertySheet;







}

/**
* A simple user interface for managing scheduled jobs via quartzAdapters adapters, as seen in the Scheduler tab of the adminConsole. Exercise caution when exposing any such feature to end users.
*/
interface QuartzManager extends Class {




}
/**
* Creates a new QuartzManager
* 
* @param typeCheckedProperties {Partial<QuartzManager>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {QuartzManager} The new instance
*/
class QuartzManager{
static create(typeCheckedProperties? : Partial<QuartzManager>, uncheckedProperties?: Object): QuartzManager;







}

/**
* RPCManager is a static singleton class that manages transparent client/server RPC (remote
*  procedure call). This class provides a generic, low-level client/server communication
*  integration point.
*  
*  SmartClient's powerful databinding subsystem (see DataSource, 
*  DataBoundComponent) automatically make use of this class to issue
*  RPCs as necessary, based on the 
*  dataSourceOperations. To integrate DataBoundComponents
*  with your server, clientServerIntegration.
*  
*  For arbitrary client/server interactions outside of the DataSource subsystem, the
*  SmartClient server also provides the dmiOverview feature.
*  
*  The RPCManager class can also be used directly to send data to a URL of your
*  choosing and optionally be called back with server-returned data when the server replies.
*  
*  The SmartClient iscServer has APIs for processing RPC requests 
*  providing features such as automatic Java &lt;--&gt; JavaScript object translation 
*  and handling of queued requests.
*  The servletDetails makes use of these features to handle standard
*  DataSource requests and DMI calls. Developers can also override the
*  actionURL of specific requests and use these APIs directly in a 
*  JSP, Servlet or Filter.
*  
*  Note: the client-side RPCManager class can also be used without the SmartClient server.
*  For an overview of client/server interactions without the SmartClient server, see
*  nonJavaBackend.
*  
*  Simple arbitrary Remote Procedure Call example (client code):
*  
*  
*  
*  var data = { here: "is some data", to: ["send to the server"]};
*  isc.RPCManager.sendRequest({ data: data, callback: "myCallback(data)", actionURL: "/rpcHandler.jsp"});
*  function myCallback(data) { alert("response from the server: " + data); }
*  
*  
*  
*  
*  Simple arbitrary Remote Procedure Call example (server code: /rpcHandler.jsp):
*  
*  RPCManager rpc = new RPCManager(request, response, out);
*  Object data = rpc.getData();
*  System.out.println("client sent: " + data.toString());
*  rpc.send("here's a response");
*  
*  
*  Queuing
*  
*  Because of browser limitations on the total number of simultaneous HTTP connections to a given
*  server, batching multiple RPC requests into a single HTTP request is highly advisable whenever
*  possible. The RPCManager provides a queuing mechanism that allows this.
*  
*  Queuing example (client code):
*  
*  
*  var wasQueuing = isc.RPCManager.startQueue();
*  isc.RPCManager.send("a string of data", "myCallback(data)", {actionURL: "/rpcHandler.jsp"});
*  isc.RPCManager.sendRequest({ data: ["some", "more data", 2], callback: "myCallback(data)", actionURL: "/rpcHandler.jsp"});
*  isc.RPCManager.sendRequest({ data: "different callback", callback: "myCallback2(data)", actionURL: "/rpcHandler.jsp"});
*  if (!wasQueuing) isc.RPCManager.sendQueue();
*  
*  function myCallback(data) { alert("response from the server: " + data); }
*  function myCallback2(data) { alert("response from the server (other callback): " + data); }
*  
*  
*  
*  
*  Queuing example (server code: /rpcHandler.jsp):
*  
*  RPCManager rpc = new RPCManager(request, response, out);
* 
*  for(Iterator i = rpc.getRequests().iterator(); i.hasNext();) {
*    RPCRequest rpcRequest = (RPCRequest)i.next();
*    Object data = rpcRequest.getData();
*    System.out.println("client sent:" + data.toString());
* 
*    //send back the data sent to us by the client
*    rpc.send(rpcRequest, new RPCResponse(data));
*  }
*  
*  
*  Error Handling
*  Please see this errorHandling on error handling.
*  
*/
interface RPCManager extends Class {




}
/**
* Creates a new RPCManager
* 
* @param typeCheckedProperties {Partial<RPCManager>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RPCManager} The new instance
*/
class RPCManager{
static create(typeCheckedProperties? : Partial<RPCManager>, uncheckedProperties?: Object): RPCManager;
/**
* In milliseconds, how long the RPCManager waits for an RPC request to complete before
*  returning an error. 
*  
*  Default of 240000 milliseconds is four minutes. If set to zero, the RPCManager will not
*  enforce a timeout, however, see RPCRequest.timeout for a discussion of default
*  timeouts that are built into browsers.
* @type {Integer}
* @default "240000"
*/
static defaultTimeout?: Integer;

/**
* If showPrompt is enabled for a given transaction, this is the defaultPrompt to be shown
*  to the user in a modal dialog while the transaction occurs.
*  May be overridden at the request level via RPCRequest.prompt.
*  More targetted default prompts are also supported for certain code-paths. See the following 
*  set of properties for details:
*  
* - RPCManager.removeDataPrompt
*  
* - RPCManager.saveDataPrompt
*  
* - RPCManager.fetchDataPrompt
*  
* @type {HTMLString}
* @default "Contacting Server..."
*/
static defaultPrompt?: HTMLString;

/**
* Default message displayed to user when an operation fails to return from the server within
*  the timeout period specified by RPCManager.defaultTimeout.
* @type {HTMLString}
* @default "Operation timed out"
*/
static timeoutErrorMessage?: HTMLString;

/**
* Default prompt displayed to user while an operation is running to remove data from
*  the server.
*  Displayed as a result of the ListGrid.removeSelectedData code path.
* @type {HTMLString}
* @default "Deleting Record(s)..."
*/
static removeDataPrompt?: HTMLString;

/**
* Default prompt displayed to the user while an operation is running to save data to
*  the server.
*  Displayed as a result of the DynamicForm.saveData code path.
* @type {HTMLString}
* @default "Saving form..."
*/
static saveDataPrompt?: HTMLString;

/**
* Default prompt displayed to the user while a server validation is pending.
* @type {HTMLString}
* @default "Validating..."
*/
static validateDataPrompt?: HTMLString;

/**
* Controls the default prompt style. Overrideable by RPCRequest.promptStyle.
* @type {PromptStyle}
* @default "dialog"
*/
static promptStyle?: PromptStyle;

/**
* If true, an image is shown to the right of the cursor when RPCRequest.promptStyle is
*  set to "cursor", otherwise the cursor itself is modified via css to the value of
*  RPCRequest.promptCursor. 
*  
*  This value can be overridden on a per-request basis via RPCRequest.useCursorTracker.
* @type {boolean}
* @default false
*/
static useCursorTracker?: boolean;

/**
* Controls the default cursor shown when RPCManager.promptStyle is set to
*  "cursor". Overrideable by RPCRequest.promptCursor.
* @type {string}
* @default "progress"
*/
static promptCursor?: string;

/**
* Default prompt displayed to the user while an operation is running to fetch data from
*  the server.
*  Displayed as a result of ListGrid.filterData, ListGrid.fetchData and
*  ListGrid.clearCriteria code paths.
* @type {string}
* @default "Finding Records that match your criteria..."
*/
static fetchDataPrompt?: string;

/**
* If set to true, the RPCManager will block the UI with a modal dialog containing
*  the text from RPCManager.defaultPrompt (or the per-RPCRequest override) until the RPC to the
*  server completes.
*  
*  If set to false, the RPC happens transparently, allowing the user to continue
*  interacting with the UI.
*  
*  DataSource requests, which are a particular type of RPCRequest, are controlled by the
*  more-specific DataSource-level setting DataSource.showPrompt.
* @type {boolean}
* @default false
*/
static showPrompt?: boolean;

/**
* If the request is configured to block user interactivity (RPCRequest.showPrompt), 
*  this property controls the delay in milliseconds before a visual indication is shown to the
*  user that interactivity is blocked.
*  
*  Studies have shown that users will perceive a short operation as occurring faster if they
*  are not shown a wait cursor, throbber or other busy indicator, but that a busy indicator
*  must appear after a briefy delay or the user will perceive the system as broken or
*  hung.
*  
*  Note that, regardless of this setting, interactivity is immediately blocked if showPrompt
*  is true, since the purpose of blocking is to prevent duplicate requests or prevent
*  interacting with components while they are in transition. This setting controls only how
*  fast a visual indication of blocking is shown.
* @type {number}
* @default "300"
*/
static promptDelay?: number;

/**
* Specifies the default URL for RPCRequests and DSRequests that do not specify a
*  URL.
*  
*  URLs can be set on a per-request basis via RPCRequest.actionURL, or on a
*  per-DataSource or per-operationType basis via DataSource.dataURL and
*  OperationBinding.dataURL respectively. However, note that in order to be able to
*  make use of RPCManager.startQueue, you should have all data loading and
*  saving requests go to a single URL unless you are forced to use distinct URLs by legacy
*  services.
*  
*  The primary use case for setting the default actionURL is to add a CSRF / XSRF 
*  (Cross-site Request Forgery) 
*  token. Assuming you are using a single URL for all data requests as covered above,
*  adding a CSRF token to the default actionURL as a simple HTTP parameter
*  will cause the CSRF token to be included in all RPCRequests and DSRequests from all
*  DataSources without further effort.
*  
*  If the actionURL is changed while transactions are suspended, any suspended
*  transactions whose actionURL was defaulted to this property (e.g. because
*  RPCRequest.actionURL wasn't set) will be updated to have the new
*  actionURL.
* @type {URL}
* @default "[ISOMORPHIC]/IDACall"
*/
static actionURL?: URL;

/**
* Whether errors during download should be reported inside the document, rather than
*  through the RPCResponse.status. Establishes a default for
*  RPCRequest.reportDownloadErrorsAsDocuments.
* @type {boolean}
* @default false
*/
static reportDownloadErrorsAsDocuments?: boolean;

/**
* The screenLoaderURL specifies the URL where ScreenLoaderServlet is installed.
* @type {URL}
* @default "[ISOMORPHIC]/screenLoader"
*/
static screenLoaderURL?: URL;

/**
* The projectLoaderURL specifies the URL where ProjectLoaderServlet is installed.
* @type {URL}
* @default "[ISOMORPHIC]/projectLoader"
*/
static projectLoaderURL?: URL;

/**
* Passing this special value to RPCManager.loadScreen indicates that
*  all global names should be preserved when evaluating loaded screen.
* @type {string}
* @default "-ALL_GLOBALS"
*/
static ALL_GLOBALS?: string;

/**
* Selects the default http transport for all RPC requests. If set to true, RPCManager
*  will use XMLHttp for requests to the server. If set to false, it will use hidden
*  frames. Overrideable on a per-request basis via RPCRequest.useXmlHttpRequest.
*  
*  Note that if the end user disables ActiveX controls in Internet Explorer, the XMLHttpRequest
*  object will not be available and SmartClient will automatically fall back on frames
*  communication.
* @deprecated * As of SmartClient 5.5, use RPCManager.defaultTransport. If you specify a value for this property, it will take precedence over
*  RPCManager.defaultTransport for requests that do not specify a
*  RPCRequest.transport or RPCRequest.useXmlHttpRequest.

* @type {boolean}
* @default "true if XMLHttpRequest is supported, false otherwise"
*/
static useXmlHttpRequest?: boolean;

/**
* Selects the transport use for RPC requests by default. You can override this setting on
*  a per-request basis by setting RPCRequest.transport.
* @type {RPCTransport}
* @default "xmlHttpRequest"
*/
static defaultTransport?: RPCTransport;

/**
* Whether the servletDetails should be used in order to get
*  around the "same origin policy" that prevents web pages from contacting other sites.
*  
*  Default behavior is to use the HttpProxyServlet whenever a URL appears to be pointing to
*  another site. Set RPCRequest.useHttpProxy false to have a particular request
*  avoid using the HttpProxyServlet even when it appears to be necessary, or set
*  RPCManager.useHttpProxy to false to avoid ever attempting to use the
*  HttpProxyServlet.
* @type {boolean}
* @default true
*/
static useHttpProxy?: boolean;

/**
* The URL to use for proxied requests. This is a global system-wide setting.
* @type {string}
* @default "[ISOMORPHIC]/HttpProxy"
*/
static httpProxyURL?: string;

/**
* Specifies URL where credentials should be submitted to attempt relogin when session
*  timeout is encountered during a background RPC. See relogin
* @type {string}
* @default "http://buildtest2:20011/isomorphic/login/loginSuccessMarker.html"
*/
static credentialsURL?: string;

/**
* By default SmartClient will show a warning message on attempted requests to another domain as
*  this is usually not supported at the browser level by default due to 
*  security considerations.
*  
*  Some browsers now do support cross domain requests through the use of Http Access Control headers
*  (See the W3C Cross-Origin Resource Sharing recommendation).
*  If your application intends to rely on this behavior to perform cross-domain requests, 
*  you can set allowCrossDomainCalls to true to disable the standard SmartClient 
*  warning when such calls occur.
*  
*  Note also that this is typically not an issue if you are using the SmartClient server 
*  (part of Pro, Power and Enterprise editions of SmartClient), as this includes the 
*  RPCManager.sendProxied.
* @type {boolean}
* @default false
*/
static allowCrossDomainCalls?: boolean;

/**
* String sequence which marks the response as a one which contains login status information.
*  
*  The default loginStatusCodeMarker is the following string:
*  "&lt;SCRIPT&gt;//'\"]]&gt;&gt;isc_"
* @type {string}
* @default ""
*/
static loginStatusCodeMarker?: string;

/**
* Marker the system will look for in XHTP responses in order to detect when login is required.
*  The default loginRequiredMarker is the following string:
*  "&lt;SCRIPT&gt;//'\"]]&gt;&gt;isc_loginRequired"
*  
*  As described in relogin, if this snippet is encountered in the response
*  to a SmartClient RPC request (with standard transport "xmlHttpRequest"), 
*  the RPCManager will suspend the current transaction and fire the RPCManager.loginRequired 
*  notification.
*  
*  The default loginRequired marker should generally not be customized. It is
*  designed to be safe to insert into any HTML page or other server response without
*  affecting display or functionality, for example, within an HTML comment. You should
*  *only* customize the loginRequiredMarker if you have absolutely no ability
*  to change the response that the server will send when login is required.
*  
*  If you do customize the loginRequiredMarker, then the loginRequiredMarker,
*  RPCManager.loginSuccessMarker and RPCManager.maxLoginAttemptsExceededMarker should all start with the 
*  RPCManager.loginStatusCodeMarker. If they do not, there will be a small impact on 
*  performance as every response must be separately scanned for each marker, instead of just scanning
*  once for the RPCManager.loginStatusCodeMarker.
*  
*  In addition, the RPCManager.loginStatusCodeMarker should ideally contain text that could not possibly
*  validly appear as a data value in a normal response, since if that were possible, end
*  users could enter the loginRequiredMarker as a data value and cause SmartClient to
*  falsely detect session timeout when handling an ordinary data response. This is why the
*  default marker has characters that make it impossible for it to be validly interpreted
*  as a JavaScript String, XML document or HTML content - there is no way that an end user
*  could enter this as a data value in an application and have it appear verbatim in a
*  server response.
* @type {string}
* @default ""
*/
static loginRequiredMarker?: string;

/**
* Marker the system will look for in order to detect when login was successful.
*  
*  The default loginSuccessMarker is the following string:
*  "&lt;SCRIPT&gt;//'\"]]&gt;&gt;isc_loginSuccess"
* @type {string}
* @default ""
*/
static loginSuccessMarker?: string;

/**
* Marker the system will look for in order to detect when the number of maximum logins was exceeded.
*  
*  The default maxLoginAttemptsExceededMarker is the following string:
*  "&lt;SCRIPT&gt;//'\"]]&gt;&gt;isc_maxLoginAttemptsExceeded"
* @type {string}
* @default ""
*/
static maxLoginAttemptsExceededMarker?: string;

/**
* In Internet Explorer 9, when a string of 
*  JavaScript is evaluated via the native eval() function, objects created 
*  within that evaluation are not released from browser memory until the page is 
*  reloaded.
*  
*  SmartClient uses the eval() function to evaluate JSON formatted
*  responses to RPCRequests by default, making long running applications potentially
*  susceptible to memory leaks over time.
*  
*  Note that this does not apply to DataSources which use
*  DataSource.useStrictJSON formatted responses as the framework avoids
*  calling eval() altogether and makes use of the native browser 
*  JSON.parse() method which does not have this issue. By default
*  we also use strict json formatted responses for all
*  DataSource.dataFormat dataSources in IE9, so these
*  leaks are mainly a concern only for dataSources with DataSource.dataFormat
*  set to "json".
*  
*  Setting this property to false enables a workaround suggested on the
*  Microsoft Knowledge Base to 
*  avoid such memory leaks by evaluating script in a hidden iframe and periodically
*  refresh that frame. However developers should be aware of the following
*  limitation with this setting: 
*  attempting to access certain object types including
*  Date or function objects generated from such an 
*  evaluation can subsequently lead to a JavaScript error with the message
*  "Can't execute code from a freed script".
*  
*  This workaround therefore may not be suitable for all transactions or dataSources
*  within a given application.
*  
*  This property may also be specified for specific RPCRequest.allowIE9Leak.
*  
*  This issue is discussed further in the online 
*  SmartClient FAQ.
* @type {boolean}
* @default true
*/
static allowIE9Leak?: boolean;

/**
* Message that will be reported in the relogin window if relogin fails due to communication
*  problems.
* @type {HTMLString}
* @default "The server is unavailable right now, please try again in a while"
*/
static reloginCommFailureMessage?: HTMLString;

/**
* When true, any problems found by either
*  LoadProjectSettings.verifyComponents or
*  LoadProjectSettings.verifyDataSources are shown together in a
*  dialog presented to the end user. Provides default values for
*  LoadProjectSettings.verifyAsError, LoadScreenSettings.verifyAsError,
*  and CreateScreenSettings.verifyAsError.
* @type {boolean}
* @default null
*/
static verifyAsError?: boolean;





/**
*  Setter for RPCManager.actionURL.
* 
* @param {URL} new actionURL
*/
static setActionURL(actionURL:URL): void;

/**
* 
*  This method is called by the RPCManager every time it sends a queue of requests to the
*  server (note that if you are not using queuing, the system simply sends queues
*  containing just one request, so this API is valid regardless).
*  There is no default implementation of this method; it is simply an override point. 
*  It is intended to be used by user code that needs to be notified when SmartClient sends 
*  requests to the server. Note that the list of RPCRequests passed to this
*  method is strictly read-only.
* 
* 
* @param {Array<Partial<RPCRequest>>} The queue of RPCRequests that was sent
*/
static queueSent(requests:Array<Partial<RPCRequest>>): void;

/**
* 
*  Returns true if the XMLHttpRequest object is available, false otherwise. See
*  platformDependencies for more information on when XMLHttpRequest parser may
*  not available and what features are
*  impacted as a result.
* 
* 
*/
static xmlHttpRequestAvailable(): boolean;

/**
* 
*  This method is a convenience wrapper on RPCManager.sendRequest() - it calls
*  through to sendRequest().
* 
* 
* @param {any} data to be passed to the server
* @param {RPCCallback=} method to call on RPC completion
* @param {object=} object literal containing any additional properties                    you want to set - these will be applied to the                     RPCRequest object that will be auto-created for you.
*/
static send(data:any, callback?:(response:RPCResponse, rawData:any, request:RPCRequest) => void, requestParams?:object): void;

/**
*  Send an HTTP request to a remote host, potentially through the HttpProxy servlet installed
*  on the SmartClient Server.
*  
*  This API allows contacting services which are hosted on servers other than the origin server
*  if the HttpProxy servlet is enabled on the SmartClient Server.
*  
*  The HttpProxy will be used if the RPCRequest.actionURL starts with "http" and uses a
*  hostname other than "localhost" or window.location.hostname, or if the port
*  number differs, or if request.useHttpProxy is explicitly set. Otherwise the
*  request goes to the origin server (the server that returned the current page).
*  
*  The RPCRequest properties that will be respected when relaying requests via the
*  HttpProxy are: 
*  RPCRequest.actionURL, RPCRequest.httpMethod, 
*  RPCRequest.params, RPCRequest.contentType, 
*  RPCRequest.httpHeaders, and RPCRequest.data. In this case
*  "data", if set, will be used as the request body for an HTTP POST.
*  
*  Higher-level APIs like DataSource or WebService call through this API, and so
*  automatically use the HttpProxy if DataSource.dataURL or
*  WebService.setLocation is set to a foreign server.
*  
*  This API is only suitable for direct use when loading unstructured data that will not be
*  shown in a DataBoundComponent. For a WSDL-described web service, use
*  XMLTools.loadWSDL instead. For other web services, use a DataSource with
*  DataSource.dataURL, and use DataSource.transformRequest and
*  DataSource.transformResponse as necessary to form requests for the service and
*  transform responses for display.
* 
* 
* @param {Partial<RPCRequest>} rpcRequest to be routed through the HttpProxy
*/
static sendProxied(request:Partial<RPCRequest>): void;

/**
* 
*  Send the passed RPCRequest to the server. If queuing is in effect, this queues
*  the request instead.
* 
* 
* @param {Partial<RPCRequest>} RPCRequest to send to the server
*/
static sendRequest(rpcRequest:Partial<RPCRequest>): void;

/**
*  Start queuing DSRequest and RPCRequest system-wide,
*  for later sending when RPCManager.sendQueue() is called, at which time all queued
*  requests will be sent as a single, combined HTTP request.
*  
*  Combining requests via queuing:
*  
*  
* - allows the server to implement transactional saving when multiple records are
*    affected by actions in the UI
*  
* - can reduce overhead by combining related requests, avoiding the use of multiple
*    network connections, redundant authentication checks, and other redundant resource
*    allocations that would otherwise happen if requests were processed separately
*  
* - can simplify application logic that otherwise has to deal with multiple outstanding
*    server requests that might complete in any order
*  
*  
*  Queuing is used automatically by many, many framework features, including
*  multi-row grid editing (ListGrid.autoSaveEdits), 
*  ${isc.DocUtils.linkForExampleId('databoundDragCopy', 'multi-row drag &amp; drop')}, 
*  ResultTree.fetchMode, 
*  ValidatorType,
*  ${isc.DocUtils.linkForExampleId('queuedAdd', 'Master-Detail saves')}, 
*  CubeGrid, and many others. 
*  
*  Queuing also has subtler architectural benefits in terms of building reusable services -
*  see the QuickStart Guide sections on Queuing for details.
*  
*  For all the reasons given above, it's extremely important to use DataSources that can
*  support queuing. Queuing is automatically supported when using server-based DataSources
*  with the SmartClient Server Framework, and is supported by RestDataSource. 
*  
*  Order of Execution
*  
*  When the SmartClient Server framework receives a queued request, it will process all
*  requests, in order, in a single thread, before any response is sent to the client. All
*  client-side actions related to queued requests, such as
*  ListGrid.fetchData on completion, likewise happen in queue
*  order, after all server-side processing has taken place.
*  
*  Therefore when using queuing you can use the callback argument of RPCManager.sendQueue to
*  detect that all operations have completed, which is much simpler than the logic needed
*  to track multiple asynchronous operations and wait for all to complete.
*  
*  Nested Queuing
*  
*  In some cases you may wish to combine requests being sent by application logic with
*  queued requests automatically sent by components. For example, you may want to call 
*  ListGrid.saveAllEdits but also add an additional request to the same queue.
*  
*  To do this, just call startQueue() before saveAllEdits() (or
*  whatever other API would also normally perform a queued request), then call
*  sendQueue(). Framework features that use queuing will automatically notice
*  that you have already started a queue, and will not automatically call
*  sendQueue() in this case. You can implement the same behavior in your own
*  reusable components by checking the return value of startQueue(), which
*  tells you whether queuing is already active.
*  
*  Requests that can't be queued
*  
*  When using queuing, all requests in a given queue must go to the same
*  RPCRequest.actionURL and use the same transport (XMLHttp or frames). If a
*  request specifies a different actionURL or transport than that of the requests currently
*  on the queue, it will be sent to the server separately, ahead of the queue, and a
*  warning will be logged to the Developer Console.
*  
*  Due to browser security restrictions, at most one request with a upload can be sent in a queue. If you attempt to add another, the existing queue
*  will be sent immediately, logging a warning, and queueing restarted for the new request.
*  
*  Note that whenever requests are not sent in a single queue, order is not guaranteed, and
*  the callback provided to RPCManager.sendQueue at the end of your transaction may fire
*  before requests not sent in the final queue have completed.
*  
*  Implementing your own Queuing
*  
*  If you are in the rare situation that:
*  
*  
* - you can't use the SmartClient Server framework
*  
* - the server you are integrating with some pre-existing support for combining
*    operations in a flexible way, similar to queuing
*  
* - you are totally unable to implement the RestDataSource protocol for this server,
*    even through approaches such as adding it as an additional service while leaving
*    the original services unchanged, or going through an intermediate server
*  
*  .. then you can implement a crude version of the built-in queuing feature by using
*  DataSource.dataProtocol to avoid HTTP requests being
*  immediately sent when a DataSource executes. In outline:
*  
*  
* - create an API similar to startQueue() for managing a global setting
*    reflecting whether your special queuing system is active. Your DataSources should
*    check for this global setting in DataSource.transformRequest, and, if
*    queuing is active, store the request you received in
*    DataSource.transformRequest in memory, for example in an Array
*  
* - implement your own equivalent of RPCManager.sendQueue() which sends an
*    HTTP request representing your combined requests, then once you receive your
*    combined response, call DataSource.processResponse for each request.
*  
*  Note that attempting to integrate with RPCManager's queuing system doesn't
*  really make sense - RPCManager won't be aware of your separate, special
*  queue of requests, so will reject calls to sendQueue() since RPCManager's
*  queue is empty. Similarly, enabling queuing on RPCManager may cause
*  inadvertent queuing of unrelated requests you did not intend to queue. Maintaining your
*  own separate notion of whether queuing is active is simpler and less error prone.
* 
* 
* @param {boolean=} whether queuing should be enabled, default true. Passing false           will disable queuing but not send the queue yet, so that any            queued requests will be sent along with the next            send()/sendRequest()
*/
static startQueue(shouldQueue?:boolean): boolean;

/**
*  Returns true if there is a current transaction (queue of requests)
*  
*  This method will return false if no requests are currently queued, even if
*  RPCManager.startQueue has been called.
*  
* 
*/
static hasCurrentTransactionQueued(): boolean;

/**
*  Synonym of RPCManager.getQueueTransactionId.
*  
* 
*/
static getCurrentTransactionId(): Integer;

/**
*  Returns the id of the current transaction (a queue of requests).
*  
*  This method will return null if no requests are currently queued, even if
*  RPCManager.startQueue has been called.
*  
* 
*/
static getQueueTransactionId(): Integer;

/**
*  Cancel a queue of requests (also called a transaction).
*  
*  If a transactionId is passed, that transaction will be cancelled, otherwise, the current 
*  (not yet sent) transaction is cancelled. You can retrieve the id of the current 
*  transaction, if there is one, by calling 
*  RPCManager.getQueueTransactionId before the
*  transaction has been sent.
*  
*  Note that cancelQueue() calls RPCManager.clearTransaction
*  and attempts to abort the request. However, note also that whilst cancelling a 
*  transaction that has already been sent will not necessarily stop the HTTP request that 
*  has been issued - this is only possible on some browsers and with some transports - it 
*  will reliably cause SmartClient to ignore any response returned by the server and not 
*  fire any callbacks that have been passed in.
*  
*  Also, take into account that this method removes all queued requests from the current queue,
*  but queueing is still active, so if you also want to disable queuing you should call
*  RPCManager.startQueue.
* 
* 
* @param {Integer=} transactionId of the queue.
*/
static cancelQueue(transactionNum?:Integer): void;

/**
*  Erase all client-side record of a transaction, such that any response from the server
*  will be ignored.
*  
*  A transaction means a batch of one or more RPCRequests that have already been sent to
*  the server via RPCManager.sendQueue.
*  
*  You can retrieve the id of the current transaction, if there is one, by 
*  RPCManager.getQueueTransactionId before the 
*  transaction is sent.
*  
* 
* @param {Integer} id of the transaction to be cleared
*/
static clearTransaction(transactionNum:Integer): void;

/**
*  Suspends the current transaction, such that all processing of the transaction is halted,
*  any remaining RPCRequest.callback in the transaction won't fire, and
*  the transaction can never RPCRequest.timeout.
*  
*  suspendTransaction() is typically used to handle total failures for an
*  entire transaction, such as HTTP status 500, or session timeout resulting in
*  RPCManager.loginRequired being called. In both cases the intent
*  is to put the transaction on hold so that a transient problem can be resolved, and then
*  the transaction can be re-sent successfully. By using suspendTransaction(), components
*  that submitted requests never realize there was a transient failure, and so error
*  handling logic does not have to be implemented in every component.
*  
*  Generally you can only validly suspend a transaction from either
*  RPCManager.loginRequired or RPCManager.handleError, and in the case of
*  handleError(), only when the first response in the transaction has an error.
*  Suspending and re-sending a partially processed transaction means that some responses
*  will be processed twice, with undefined results for requests issued automatically by UI
*  components.
*  
*  A suspended transaction must ultimately be either cleared via
*  RPCManager.clearTransaction or re-sent via RPCManager.resendTransaction or memory will be
*  leaked.
* 
* 
* @param {any | GlobalId=} transaction to delay. Defaults to the           current transaction if there is one
*/
static suspendTransaction(transaction?:any | GlobalId): void;

/**
*  Resend a suspended transaction to the server. See RPCManager.suspendTransaction for
*  context. 
*  
*  Note that the transaction must have been previously suspended, and in particular
*  suspended validly according to the rules described in the docs for
*  RPCManager.suspendTransaction, or undefined results will occur.
*  
*  You can resend all suspended transactions by calling
*  RPCManager.resendTransaction with no arguments.
* 
* 
* @param {Integer=} id of the transaction to be re-sent, or null to resend all               suspended transactions
*/
static resendTransaction(transactionNum?:Integer): void;

/**
* 
*  Send all currently queued requests to the server. You need only call this method if you are
*  using queuing otherwise your requests are synchronously submitted to the server.
*  
*  This method will do nothing and the callback will not be called if no requests have actually 
*  been queued. You can detect whether the queue is empty by calling 
*  RPCManager.getQueueTransactionId.
*  
*  NOTE: if you aren't the caller who first enables queuing (startQueue() returns
*  true), you should in general avoid calling sendQueue(), because whoever was
*  first to enable queuing may have more requests to add to the same queue.
*  
*  See RPCManager.startQueue for more information about queuing.
* 
* 
* @param {RPCQueueCallback=} Callback to fire when the queued operations complete. Callback will be fired with 1 parameter: responses an array of DSResponse or  RPCResponse objects that were part of the transaction fired by this method.
*/
static sendQueue(callback?:(response:Array<Partial<RPCResponse>>) => void): void;

/**
*  Remove an operationReplyCallback registered via classMethod:RPCManager.addOperationReplyCallback
* 
* @param {Integer} Identifier for the registered callback returned from classMethod:RPCManager.addOperationReplyCallback
*/
static removeOperationReplyCallback(id:Integer): void;

/**
*  handleError() will be called if RPCResponse.status is negative and
*  RPCRequest.willHandleError was not set. It is called for both DSResponses
*  and RPCResponses that have a non-success status. You can check whether the
*  response is a DSResponse by checking response.isDSResponse.
*  
*  By default handleError() always logs a warning. In addition, if
*  RPCResponse.data was set to a String, a warning dialog will be
*  shown to the user with response.data as the message, which allows the server to send
*  user error messages back without writing custom client-side error handling.
*  
*  To do custom error handling that is specific to a particular component or type of
*  request, set RPCRequest.willHandleError and deal with errors in the
*  rpcRequest.callback. To change the default system-wide error handling, 
*  override this
*  method. Note that since handleError() is a class method, to override it
*  you will call Class.addClassProperties rather than
*  addProperties(), like so:
*  
*    isc.RPCManager.addClassProperties({
*      handleError : function (response, request) { .. custom handling .. }
*    })
*  
*  
*  To invoke the default error handling in your new handler, you can simply call the method
*  RPCManager.runDefaultErrorHandling.
*  
*  If you're using the xmlHttpRequest RPCRequest.transport, you can access the 
*  HTTP status code
*  of the response (eg 404 Not Found or 500 Server Error) as RPCResponse.httpResponseCode.
*  
*  For very advanced usage, the response.xmlHttpRequest contains the native XMLHttpRequest
*  object used to make the request.  Accessing this object is
*  subject to possible cross-platform bugs and inconsistencies, and Isomorphic recommends
*  that you wrap any access to the XMLHttpRequest object in a try/catch block because some
*  browsers may throw exceptions when certain attributes of this object are accessed. For
*  example, if you try to access XMLHttpRequest.status (for the HTTP status code) when the
*  network cable is unpluged in Windows, you'll get an Exception in Firefox.
*  
*  See the errorHandling for additional guidance.
* 
* 
* @param {any} the RPCResponse or DSResponse object returned from the server
* @param {any} the RPCRequest or DSRequest that was sent to the server
*/
static handleError(response:any, request:any): void;

/**
*  Runs the default error handling normally performed by RPCManager.handleError. May be
*  called from a custom handler to achieve the default behavior if one has been installed.
* 
* 
* @param {DSResponse} response the response
* @param {DSRequest} request the request
*/
static runDefaultErrorHandling(response:DSResponse, request:DSRequest): void;

/**
*  handleTransportError() handles server error responses for submitted
*  transactions. When the server responds to a submitted transaction with an HTTP error
*  code this method will be called before any individual response callbacks are fired,
*  regardless of whether RPCRequest.willHandleError was specified on the submitted
*  request[s].
*  
*  This provides the developer with an opportunity to handle a server error by (for example)
*  suspending and resubmitting the transaction before any other handling occurs.
*  
*  The default implementation takes no action - by default transport errors 
*  are handled via RPCManager.handleError, or by the standard request callback
*  methods, depending on request.willHandleError. 
*  To perform custom handing for transport errors this classMethod may be
*  overridden as follows
*  
*    isc.RPCManager.addClassProperties({
*      handleTransportError : function (transactionNum, status, httpResponseCode, httpResponseText) 
*      {
*         .. custom handling .. 
*      }
*    })
*  
*  
*  Return an explicit false from this method to cancel default error handling,
*  so that RPCManager.handleError is not called for any DSResponse in this transaction.
*  
*  
*  Note: This method only applies to operations submitted via
*  RPCTransport - it is not possible to provide similar error handling
*  for other transports.
*  
*  See the errorHandling for additional guidance.
* 
* 
* @param {Integer} The submitted client-server transaction number
* @param {Integer} The RPCResponse status code
* @param {Integer} The HTTP Response code reported by the server
* @param {string} The raw HTTP Response text
*/
static handleTransportError(transactionNum:Integer, status:Integer, httpResponseCode:Integer, httpResponseText:string): boolean;

/**
*  Called when a session timeout is encountered while trying to do a background RPC. See
*  relogin.
*  
*  The transaction with the passed transactionId is suspended, and should either
*  be RPCManager.clearTransaction or RPCManager.resendTransaction
*  after the user has been re-authenticated. 
*  
*  The rpcRequest parameter can be used to determine whether the suspended
*  transaction can simply be dropped (eg, it's periodic polling request).
*  
*  The rpcResponse parameter has rpcResponse.data set to the raw text of the
*  response that triggered loginRequired(). Some very advanced relogin strategies
*  may need to inspect the raw response to get information needed for re-authentication.
* 
* 
* @param {Integer} id of the transaction
* @param {RPCRequest} first RPCRequest of the transaction
* @param {RPCResponse} RPCResponse containing the session timeout response that                 caused loginRequired() to be invoked
*/
static loginRequired(transactionNum:Integer, rpcRequest:RPCRequest, rpcResponse:RPCResponse): void;

/**
*  Exports the printable representation of a set of widgets as a .pdf that is then
*  downloaded to the user (triggering the "Save As.." dialog).
*  
*  As with other exports, the resulting file can be 
*  DSRequest.exportToFilesystem instead of, or in
*  addition to being downloaded to the user. See server-side docs for
*  com.isomorphic.contentexport.PdfExport for more details on server-side processing and
*  code samples for redirecting PDF output to a file or in-memory buffer, as well as
*  instructions for adding additional stylesheets.
*  
*  You can either pass any Canvas to exportContent, or you can
*  pass HTML that you have retrieved by calling Canvas.getPrintHTML. When 
*  calling getPrintHTML() to retrieve HTML for use with
*  exportContent(), you must pass the PrintProperties.printForExport 
*  or DrawPane and FacetChart instances will not export properly.
*  
*  You can use a custom skin when exporting your HTML content. To use a custom skin,
*  add a line to server_properties:
*  
*   skin.{skinName}.location: custom/skin
*  
*  Where {skinName} is the name of your custom skin, and the value is the path to your 
*  skin resources from the application webroot.
*  
*  Also, you can set the filename of the export via DSRequest.exportFilename. For example:
*  
*  
*  var settings = {
*    exportFilename: "export" // without .pdf
*    // ... other DSRequest properties
*  };
*  
*  
*  
*  
*  In case you do not provide a filename for your export, the system will assign the default filename "export".
*  
*  Requires the SmartClient server framework, but does not require use of server-based
*  databinding - no .ds.xml files need to exist.
*  
*  You can also inject a small amount of CSS from the browser via
*  DSRequest.exportCSS - this is intended primarily for switching the page size on
*  the fly, for exceptionally wide or tall exports.
* 
* 
* @param {Canvas | Array<Partial<Canvas>> | HTMLString} Canvas or canvas list that has exportable widgets,          or an HTML fragment derived from Canvas.getPrintHTML
* @param {Partial<DSRequest>=} Request properties for the export to pdf object
*/
static exportContent(canvas:Canvas | Array<Partial<Canvas>> | HTMLString, requestProperties?:Partial<DSRequest>): void;

/**
*  Converts an DrawPane.getSvgString to one of several possible image formats,
*  and can either initiate a download or return the base64-encoded image data.
*  
*  Control the image format via DSRequest.exportImageFormat.
*  
*  Default is to download the image (triggering the browser's save dialog).
*  DSRequest.exportFilename can be used to control the default filename provided in the
*  save dialog.
*  
*  To instead return the data as a normal DSResponse, set DSRequest.exportDisplay to
*  "return". In this case the data is always base64 encoded.
*  
*  Requires the SmartClient server framework, with the same set of
*  javaModuleDependencies as are required for PDF export of charts in
*  legacy IE. 
*  
*  See also DrawPane.getSvgString and DrawPane.getDataURL. 
* 
* 
* @param {string} XML string containing SVG data
* @param {Partial<DSRequest>=} request properties controlling options for export
* @param {ExportImageCallback=} optional callback when using                     exportDisplay:"return". Does not                     fire for other exportDisplay modes
*/
static exportImage(svgString:string, requestProperties?:Partial<DSRequest>, callback?:(imageData:string) => void): void;

/**
*  Returns the data that should be sent to the RPCManager.actionURL.
*  
*  In a manner analogous to DataSource.transformRequest, this method allows you to
*  transform an RPCRequest, such as by adding RPCRequest.httpHeaders, to ensure proper handling on the server.
*  
* 
* 
* @param {RPCRequest} the RPCRequest being processed
*/
static transformRequest(rpcRequest:RPCRequest): any;

/**
*  Loads a screen saved in componentXML format, using the
*  servletDetails.
*  
*  The ScreenLoaderServlet will look for a file named screenName.ui.xml in 
*  the directory given by the "project.ui" setting, which defaults
*  webroot/shared/ui and can be configured in 
*  server_properties.
*  
*  The screen provided by the callback will be the outermost component if your
*  loaded screen consists of a hierarchy of widgets all contained under one parent (which
*  is true of any screens created in Reify). 
*  
*  If you have multiple widget hierarchies in your screen, the screen returned
*  will be the last top-level component created.
*  
*  By default, components in the loaded screens that have Canvas.ID will not
*  actually be allowed to take those global IDs - instead, only widgets that have one of the
*  global IDs passed as the globals parameter will actually receive their global
*  IDs. To override this behavior, pass the special value RPCManager.ALL_GLOBALS
*  for the globals parameter.
*  
*  When globals are being suppressed, the screen available in the callback
*  will provide access to widgets that did not receive their global IDs via
*  Canvas.getByLocalId, and the suppressedGlobals available in the
*  callback will be a mapping from suppressed global ID to the widget or other component
*  that would have used that global ID if globals were not suppressed. In addition, any
*  other Canvas loaded with the screen also provides access to any suppressed
*  globals from the screen via getByLocalId().
*  
*  To load multiple screens at once, use RPCManager.cacheScreens and RPCManager.createScreen
*  instead.
*  
*  Components in the screen will default to having Canvas.autoDraw set to false.
*  This may be overridden by setting the RPCRequest.suppressAutoDraw attribute
*  explicitly to false on the request properties object.
*  
*  You can optionally provide a locale name to use when resolving any i18n tags in the 
*  screen's component XML. If you do not supply this, the locale will be derived from 
*  the servlet API, and so will generally be a locale appropriate to the client's operating
*  system settings. Only provide a locale manually if you have a special requirement that 
*  requires the user's operating system locale to be overridden in your application. If 
*  you provide a locale name, it should be of the form "xx" or "xx_YY", where "xx" is a 
*  valid language code and "YY" is a valid country code. For example, "fr" or "en_GB".
*  
*  This API assumes the ScreenLoaderServlet is installed at the default location - to use a
*  different location, use the requestProperties parameter to specify a different
*  URL via RPCRequest.actionURL. The requestProperties parameter can also
*  be used to pass additional params to a custom ScreenLoaderServlet - see the "Dynamic
*  Component XML" section of the componentXML.
* 
* 
* @param {string} name of the screen to load
* @param {LoadScreenCallback} callback for notification of screen being loaded
* @param {Array<Partial<string>> | LoadScreenSettings=} widgets to allow to take their                             global IDs, or load settings
* @param {string=} The name of a locale to use for resolving i18n tags in the     component XML of the screen
* @param {Partial<RPCRequest>=} optional properties for the request
* @param {boolean=} If true, server logic does not crash out if it cannot     load a DataSource specified in the screen definition. Instead, a stub DataSource is     returned, which consists of nothing except the ID and an unableToLoad     flag, which client-side code can use to determine that the DataSource could not be     loaded on the server. Optional, defaults to false (ie, a missing DataSource     causes a crash by default)
*/
static loadScreen(screenName:string, callback:(screen?:Canvas, rpcResponse?:RPCResponse, suppressedGlobals?:Map<any>) => void, settings?:Array<Partial<string>> | LoadScreenSettings, locale?:string, requestProperties?:Partial<RPCRequest>, missingDSIsNotFatal?:boolean): void;

/**
*  Loads projects using the servletDetails, reachable at 
*  RPCManager.projectLoaderURL, and fires the given callback after the project has
*  been cached. When a project is loaded, all of its DataSources and screens (except where
*  explicitly overridden by settings) are also cached in the project.
*  
*  Loading of a project merely caches DataSources and screens. No screens are created
*  automatically. To create a screen, you must call Project.createScreen or
*  Project.createStartScreen, which will automatically create any DataSource
*  needed by the screen that's not already globally bound. You can also manually
*  instantiate and globally bind (if not already bound) a screen DataSource without
*  creating the screen, by calling Project.getDataSource.
*  
*  Note that any screen cached in a loaded project will also be bound in the global cache
*  if no screen of that name is present there. You can create a screen from the global
*  cache using RPCManager.createScreen.
*  
*  When projects are cached, they're made available via Project.get. To remove a
*  cached project, and release storage for its cached screens and DataSources, simply call
*  Project.destroy.
* 
* 
* @param {string} Comma-separated string containing the names of                      project(s) to load.
* @param {LoadProjectCallback} Callback for notification of completion of                      project(s) loaded and screens cached.
* @param {LoadProjectSettings} Settings applicable to the loadProject                      operation.
*/
static loadProject(projectNames:string, callback:(project:Project, projects:Array<Partial<Project>>, rpcResponse:RPCResponse) => void, settings:LoadProjectSettings): void;

/**
*  Convenience method that returns the error status for a failed call to
*  RPCManager.loadProject. Applies the following rules:
*  
* - reports "badCredentials" if the RPCResponse.status is
*  STATUS_LOGIN_REQUIRED or STATUS_LOGIN_INCORRECT
*  
* - reports "timeout" if the RPCResponse.status is
*  STATUS_SERVER_TIMEOUT
*  
* - reports "noResponseFromURL" if the RPCResponse.httpResponseCode is 0
*  
* - reports "badReifyServerURL" if the RPCResponse.httpResponseCode is 404 or
*  another code representing a transport error, such as 408 or 503
*  
* - reports "projectNotFound" if the RPCResponse.httpResponseCode is 500 and the
*  RPCResponse.httpResponseText contains the string
*  "Unable to load any of the projects"
*  
* - otherwise reports "servletError"
* 
* 
* @param {RPCResponse} server response
*/
static getLoadProjectErrorStatus(rpcResponse:RPCResponse): string;

/**
*  Convenience method that returns a message describing the error for a failed call to
*  RPCManager.loadProject. Calls RPCManager.getLoadProjectErrorStatus.
* 
* 
* @param {RPCResponse} server response
*/
static getLoadProjectErrorMessage(rpcResponse:RPCResponse): string;

/**
*  Creates a screen previously cached by a call to RPCManager.cacheScreens.
*  
*  As with RPCManager.loadScreen, the default behavior is to prevent any global widget IDs
*  from being established, the returned Canvas will be the outermost component of the
*  screen, and that Canvas will provide access to other widgets in the screen via
*  Canvas.getByLocalId
*  
*  Alternatively, as with RPCManager.loadScreen, a list of IDs that should be allowed to
*  become globals can be passed, allowing those widgets to be retrieved via a call to
*  Canvas.getById after the screen has been created.
*  
*  If you do not pass globals and avoid depending on global IDs within the
*  screen definition itself (for example, by embedding JavaScript event handlers in the
*  screen definition that use global IDs), you can create the same screen multiple times.
*  
*  Creating a screen may or may not cause it to draw, depending on current global autoDraw
*  setting (isc.setAutoDraw) and any autoDraw settings
*  in the screen itself.
*  
*  Instead of globals, you may instead pass a CreateScreenSettings to change what classes are used to construct widgets, or
*  subsitute existing widgets for those to be constructed, by widget ID.
* 
* 
* @param {string} name of the screen to create
* @param {CreateScreenSettings | Array<Partial<string>>=} widgets to allow to take their                           global IDs, or a widget remap config
*/
static createScreen(screenName:string, settings?:CreateScreenSettings | Array<Partial<string>>): Canvas;

/**
*  Loads the definitions of a set of screens saved in componentXML
*  format, using the servletDetails.
*  
*  Unlike RPCManager.loadScreen, cacheScreens() does not cause any UI components to be
*  created or drawn, it just loads the definitions of the screens. This allows a subsequent,
*  synchronous call to RPCManager.createScreen to create the actual screen, rather than
*  contacting the ScreenLoader servlet and showing a loading message.
*  
*  See RPCManager.loadScreen for the meaning of the locale parameter.
*  
*  Calling cacheScreens twice with the same screenName will re-load the definition of
*  that screen from the server such that subsequent calls to createScreen() will use
*  the new definition.
*  
* 
* @param {Array<Partial<string>>} name of the screens to cache
* @param {Function} callback for notification of screens being successfully cached
* @param {string=} The name of a locale to use for resolving i18n tags in the             component XML of the screen
* @param {Partial<RPCRequest>=} optional properties for the request
*/
static cacheScreens(screenName:Array<Partial<string>>, callback:Function, locale?:string, requestProperties?:Partial<RPCRequest>): void;

/**
* 
*  Returns true if a screen with the given name has already been cached by a call to 
*  RPCManager.cacheScreens or similar (e.g. RPCManager.loadProject), false otherwise.
*  
* 
* @param {string} name of the screen
*/
static isScreenCached(screenName:string): void;

/**
*  Returns whether there are any pending RPC requests.
* 
*/
static requestsArePending(): boolean;




}

/**
* Encapsulates an RPC response from the server. Instances of this class are automatically created
*  and optionally passed to you in the callback you specify as part of your RPCRequest.
*/
interface RPCResponse extends Class {
/**
* The data sent by the server.
*  
*  When communicating with the SmartClient server, rpcResponse.data is the data passed to the
*  server-side method RPCResponse.setData() by your Java code.
*  This data is translated into JavaScript
*  objects by the rules described under RPCRequest.data.
*  
*  
*  When not communicating with the SmartClient server rpcResponse.data contains the
*  raw HTTP response body. See RPCRequest.useSimpleHttp,
*  RPCRequest.serverOutputAsString, RPCRequest.evalResult for details.
* @type {string | object}
* @default false
*/
data?: string | object;

/**
* Status code for this response. Status codes less than zero are considered errors by the
*  RPCManager, those greater than or equal to zero are considered successes. Please see the
*  error handling section the RPCManager for more information on
*  what the RPCManager does with the status code and how you can override this behavior.
*  
*  When using the SmartClient server you can set the rpcResponse.status by calling the
*  server-side method RPCResponse.setStatus(). 
*  
*  When not using the SmartClient server, the RPCManager makes no assumptions about the
*  structure of the response, so the status code just reflects the
*  RPCResponse.httpResponseCode: status will be 
*  RPCResponse.STATUS_TRANSPORT_ERROR if an HTTP-level error
*  occurred such as "500 server error". If you have a status code you need to transmit you can
*  simply embed it in the response (as part of RPCResponse.data) and interpret it from
*  the callback.
*  
*  With or without the SmartClient server, the relogin status codes (such as 
*  RPCResponse.STATUS_LOGIN_REQUIRED) are triggered whenever special markers, such as the
*  loginRequiredMarker, appear in the body of the response. See the relogin for details.
* @type {Integer}
* @default "0"
*/
status?: Integer;

/**
* This attribute (available when using the the xmlHttpRequest transport) contains
*  the HTTP response code sent by the server.
*  
*  Note that this is different from RPCResponse.status - that attribute is used to
*  indicate a status code for the RPC itself whereas httpResponseCode is the raw HTTP response
*  code for the HTTP request that contained the RPCRequest.
*  
*  This feature relies on the XMLHttpRequest object which can be disabled by end-users in some
*  supported browsers. See platformDependencies for more information.
*  
*  If you're using this attribute, you'll typically want to avoid the default error
*  handling response of RPCManager. To do so, set
*  RPCRequest.willHandleError to true.
* @type {Integer}
* @default null
*/
httpResponseCode?: Integer;

/**
* HTTP headers returned by the server as a map from header name to header value.
*  
*  Headers are available only when the default RPCTransport "xmlHttpRequest" is in use,
*  and browsers may limit access to headers for cross-domain requests or in other
*  security-sensitive scenarios.
* @type {object}
* @default null
*/
httpHeaders?: object;

/**
* The actual text of the HTTP response. Only available when the default 
*  RPCTransport "xmlHttpRequest" transport is in use,
* @type {string}
* @default null
*/
httpResponseText?: string;

/**
* The RPCRequest.clientContext object as set on the RPCRequest.
* @type {object}
* @default null
*/
clientContext?: object;

/**
* ID of the transaction sent to the server via RPCManager.sendQueue containing the
*  RPCRequest associated with this response.
* @type {Integer}
* @default null
*/
transactionNum?: Integer;





}
/**
* Creates a new RPCResponse
* 
* @param typeCheckedProperties {Partial<RPCResponse>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RPCResponse} The new instance
*/
class RPCResponse{
static create(typeCheckedProperties? : Partial<RPCResponse>, uncheckedProperties?: Object): RPCResponse;
/**
* Indicates successful completion of the request. This is the default status and is
*  automatically used by the RPCResponse on the server unless you override it with
*  setStatus().
*  
*  See the error handling section in RPCManager
*  for more information.
* @type {Integer}
* @default "0"
*/
static STATUS_SUCCESS?: Integer;

/**
* Indicates that the browser is currently offline, and that we do not hold a cached 
*  response for the request.
* @type {Integer}
* @default "1"
*/
static STATUS_OFFLINE?: Integer;

/**
* Indicates a generic failure on the server. 
*  See the error handling section in RPCManager
*  for more information.
* @type {Integer}
* @default "-1"
*/
static STATUS_FAILURE?: Integer;

/**
* Indicates a declarativeSecurity failure on the server.
*  See the error handling section in RPCManager
*  for more information.
* @type {Integer}
* @default "-3"
*/
static STATUS_AUTHORIZATION_FAILURE?: Integer;

/**
* Indicates a validation failure on the server.
*  See the error handling section in RPCManager
*  for more information.
* @type {Integer}
* @default "-4"
*/
static STATUS_VALIDATION_ERROR?: Integer;

/**
* Indicates that the RPC has been intercepted by an authenticator that requires the user
*  to log in.
* @type {Integer}
* @default "-5"
*/
static STATUS_LOGIN_INCORRECT?: Integer;

/**
* Indicates that too many authentication attempts have been made and the server refuses to
*  accept any more login attempts.
* @type {Integer}
* @default "-6"
*/
static STATUS_MAX_LOGIN_ATTEMPTS_EXCEEDED?: Integer;

/**
* Indicates that a login is required before this RPCRequest can proceed.
*  
*  Applications do not directly set this status code, instead, to trigger the relogin flow,
*  return the loginRequiredMarker in the response sent by your server when login is
*  required. See the relogin for details.
* @type {Integer}
* @default "-7"
*/
static STATUS_LOGIN_REQUIRED?: Integer;

/**
* Indicates that the login succeeded.
* @type {Integer}
* @default "-8"
*/
static STATUS_LOGIN_SUCCESS?: Integer;

/**
* Indicates that the client attempted an update or remove operation without providing 
*  primary key field(s)
* @type {Integer}
* @default "-9"
*/
static STATUS_UPDATE_WITHOUT_PK_ERROR?: Integer;

/**
* Indicates that the request was either never attempted or was rolled back, because 
*  automatic or user transactions are in force and another request in the same transaction
*  failed. Note that the request(s) that actually failed will have a code specific to the
*  failure; it is only the requests that would otherwise have succeeded that are marked 
*  with this failure code.
* @type {Integer}
* @default "-10"
*/
static STATUS_TRANSACTION_FAILED?: Integer;

/**
* Indicates that an uploaded file's size exceeded the maximum file size allowed.
* @type {Integer}
* @default "-11"
*/
static STATUS_MAX_FILE_SIZE_EXCEEDED?: Integer;

/**
* Indicates that the total size of the data sent to the server was more than the server is
*  configured to allow. Most servers limit the post size to prevent out of memory style
*  attack vectors that push a bunch of data at the server. Apache Tomcat, for example,
*  is pre-configured to limit post size to 2mb, where the default max post-size in 
*  Jetty, used by default in Eclipse, is just 200k.
*  
*  On internal networks, these limits can typically be safely raised or removed. With
*  Tomcat, for example, you can remove the post limit by specifying the following attribute
*  on the &lt;Connector&gt; element in conf/server.xml:
*  
*  maxPostSize="-1"
*  
*  
*  In Jetty, you can update or create war/WEB-INF/jetty-web.xml, adding a section like this
*  
*  &lt;Configure class="org.eclipse.jetty.webapp.WebAppContext"/&gt;
*    &lt;Set name="maxFormContentSize"&gt;2000000&lt;/Set&gt;
*  &lt;/Configure&gt;
*  
*  NOTE: this status code is used whenever the server framework receives a request
*  where the POST data has been removed, however, there are other possible causes,
*  including:
*  
*  
* - security software installed on the server or network that erroneously detects some
*    kind of exploit attempt, if its behavior is to just strip the POST data but allow
*    the rest of the request through (SiteMinder is one product known to do this)
*  
* - incorrectly written filter servlets that drop POST'd data
*  
* @type {Integer}
* @default "-12"
*/
static STATUS_MAX_POST_SIZE_EXCEEDED?: Integer;

/**
* Indicates that an operation binding configured to require OperationBinding.requiredCriterion has received none.
* @type {Integer}
* @default "-13"
*/
static STATUS_REQUIRED_CRITERIA_MISSING?: Integer;

/**
* @deprecated * as of Smartclient 12.1, use RPCResponse.STATUS_REQUIRED_CRITERIA_MISSING instead

* @type {Integer}
* @default "-13"
*/
static STATUS_CRITERIA_REQUIRED_ERROR?: Integer;

/**
* Indicates that an empty file was uploaded for a required 'binary' field.
* @type {Integer}
* @default "-15"
*/
static STATUS_FILE_REQUIRED_ERROR?: Integer;

/**
* Indicates that a response with invalid format has been received from server.
*  If the datasource is using "iscServer" dataFormat, this means that the response is
*  not recognized as a valid ISC frame.
*  
*  One possible cause for this error can be the reception of a RestDataSource JSON
*  response that lacks a valid RestDataSource.jsonPrefix and/or RestDataSource.jsonSuffix
*  
*  If it is using "xml" or "json" dataFormat, the response could not be parsed as XML or JSON.
* @type {Integer}
* @default "-16"
*/
static INVALID_RESPONSE_FORMAT?: Integer;

/**
* This response code is usable only with the XMLHttpRequest transport and indicates that
*  the server returned an HTTP response code outside the range 200-299 (all of these statuses
*  indicate success, but ordinarily only 200 is used). To get the actual
*  response code, you can query rpcResponse.httpResponseCode in your callback.
*  
*  Note that currently this error code will never occur for the hiddenFrame
*  transport - instead, use RPCResponse.STATUS_SERVER_TIMEOUT to detect
*  hiddenFrame transport errors.
* @type {Integer}
* @default "-90"
*/
static STATUS_TRANSPORT_ERROR?: Integer;

/**
* This response code only occurs when using the HTTP proxy. It is issued by the proxy 
*  servlet when the target host is unknown (ie, cannot be resolved through DNS). This
*  response probably indicates that you are attempting to contact a nonexistent server 
*  (though it might mean that you have DNS problems).
* @type {Integer}
* @default "-91"
*/
static STATUS_UNKNOWN_HOST_ERROR?: Integer;

/**
* This response code only occurs when using the HTTP proxy. It is issued by the proxy
*  servlet when the attempt to contact the target server results in a Java SocketException.
*  This response probably indicates that the target server is currently down.
* @type {Integer}
* @default "-92"
*/
static STATUS_CONNECTION_RESET_ERROR?: Integer;

/**
* Indicates a request timed out with no server response.
*  
*  This is a client-only error code - never sent by the server (since it's the server
*  that times out).
*  
*  NOTE that if using hiddenFrame as the transport (not the default), a
*  malformed response such as a "500 Server Error" or 404 errors will be reported as a
*  timeout.
* @type {Integer}
* @default "-100"
*/
static STATUS_SERVER_TIMEOUT?: Integer;








}

/**
* FormItem that shows a set of mutually exclusive options as a group of radio buttons.
*/
interface RadioGroupItem extends Omit<FormItem, 'valueHoverHTML'>  {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "RadioGroupItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* True == display options vertically, false == display in a single row
* @type {boolean}
* @default true
*/
vertical?: boolean;

/**
* If RadioGroupItem.vertical is false, and this item has a specified width, should
*  options be spread out evenly to fill the specified width?
* @type {boolean}
* @default false
*/
fillHorizontalSpace?: boolean;

/**
* Base CSS class applied to the text for items within this radio group.
* @type {FormItemBaseStyle}
* @default "labelAnchor"
*/
textBoxStyle?: FormItemBaseStyle;

/**
* Should the text for items within this radio group wrap?
* @type {boolean}
* @default null
*/
wrap?: boolean;

/**
* When set to false, replaces each native radio element in the group with a 
*  CheckboxItem which can be customized via 
*  RadioGroupItem.checkboxItemProperties.
* @type {boolean}
* @default true
*/
useNativeRadioItems?: boolean;

/**
* When set to true, allows a checked RadioItem to be unchecked when clicked, 
*  clearing the value from the RadioGroupItem. 
*  
*  Note that this is not a default behavior of native radios and could impact compliance 
*  with accessibility standards depending on the application.
* @type {boolean}
* @default false
*/
allowEmptyValue?: boolean;

/**
* Properties to apply to the customized CheckboxItem used for radioGroupItems 
*  when RadioGroupItem.useNativeRadioItems is false.
* @type {Partial<CheckboxItem>}
* @default null
*/
checkboxItemProperties?: Partial<CheckboxItem>;

/**
* Map of properties to apply to generated items within this RadioGroup. This allows you to 
*  customize the generated radio items for this item.
* @type {Partial<RadioItem>}
* @default null
*/
itemProperties?: Partial<RadioItem>;

/**
* This property allows you to specify an initial set of disabled options within
*  this radioGroup. Once the RadioGroupItem has been created, RadioGroupItem.setValueDisabled
*  should be used to enable and disable options.
* @type {Array<Partial<string>>}
* @default null
*/
disabledValues?: Array<Partial<string>>;



/**
*  Setter for RadioGroupItem.textBoxStyle.
* 
* @param {FormItemBaseStyle} new textBoxStyle.
*/
setTextBoxStyle(newTextBoxStyle:FormItemBaseStyle): void;

/**
*  Notification method called when FormItem.showPending is enabled and
*  this radio group should either clear or show its pending visual state.
*  
*  The default behavior is that the FormItem.titleStyle and
*  FormItem.cellStyle are updated to include/exclude the "Pending" suffix.
*  In addition, the label for the newly-selected option will have a different color.
*  Returning false will cancel this default behavior.
* 
* @param {DynamicForm} the managing DynamicForm instance.
* @param {FormItem} the form item itself (also available as "this").
* @param {boolean} true if the item should show its pending visual state; false otherwise.
* @param {any} the current form item value.
* @param {any} the value that would be restored by a call to DynamicForm.resetValues.
*/
pendingStatusChanged(form:DynamicForm, item:FormItem, pendingStatus:boolean, newValue:any, value:any): boolean;

/**
*  If defined, this method should return the HTML to display in a hover canvas when the 
*  user holds the mouse-pointer over one of the radio-items in this RadioGroupItem. Return 
*  null to suppress the hover canvas altogether.
* 
* 
* @param {any} The sub-value (radio-item) to get the hoverHTML for
* @param {RadioGroupItem} Pointer to this item
* @param {DynamicForm} This item's form
*/
valueHoverHTML(value:any, item:RadioGroupItem, form:DynamicForm): HTMLString;

/**
*  Disable or Enable a specific option within this radioGroup
* 
* @param {any} value of option to disable
* @param {boolean} true to disable the option, false to enable it
*/
setValueDisabled(value:any, disabled:boolean): void;



}
/**
* Creates a new RadioGroupItem
* 
* @param typeCheckedProperties {Partial<RadioGroupItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RadioGroupItem} The new instance
*/
class RadioGroupItem{
static create(typeCheckedProperties? : Partial<RadioGroupItem>, uncheckedProperties?: Object): RadioGroupItem;







}

/**
* EditProxy that handles RadioGroupItem when editMode is enabled.
*/
interface RadioGroupItemEditProxy extends SelectItemEditProxy {




}
/**
* Creates a new RadioGroupItemEditProxy
* 
* @param typeCheckedProperties {Partial<RadioGroupItemEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RadioGroupItemEditProxy} The new instance
*/
class RadioGroupItemEditProxy{
static create(typeCheckedProperties? : Partial<RadioGroupItemEditProxy>, uncheckedProperties?: Object): RadioGroupItemEditProxy;







}

/**
* Form item representing a member of a radio group, subclassed from NativeCheckboxItem.
*  RadioItems items are created and managed automatically by RadioGroupItem instances
*  and should not be instantiated directly.
*/
interface RadioItem extends NativeCheckboxItem {




}
/**
* Creates a new RadioItem
* 
* @param typeCheckedProperties {Partial<RadioItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RadioItem} The new instance
*/
class RadioItem{
static create(typeCheckedProperties? : Partial<RadioItem>, uncheckedProperties?: Object): RadioItem;







}

/**
* A "double slider" allowing the user to select a range via two draggable thumbs.
*/
interface RangeSlider extends Canvas {
/**
* Thumb for the start of the range.
* @type {Snapbar}
* @default null
*/
startThumb?: Snapbar;

/**
* Thumb for the end of the range
* @type {Snapbar}
* @default null
*/
endThumb?: Snapbar;

/**
* Optional track of the RangeSlider. Set showTrack false to avoid showing
*  a track so the RangeSlider can be superimposed over something else.
* @type {Canvas}
* @default null
*/
track?: Canvas;

/**
* Optional Scrollbar shown as a second way of adjusting the range.
* @type {Scrollbar}
* @default null
*/
scrollbar?: Scrollbar;

/**
* Whether the rangeSlider should be vertical or horizontal. Default is horizontal.
* @type {boolean}
* @default false
*/
vertical?: boolean;

/**
* Set the minimum value (left/top of slider).
* @type {Float}
* @default "0"
*/
minValue?: Float;

/**
* Set the maximum value (right/bottom of slider).
* @type {Float}
* @default "0"
*/
maxValue?: Float;

/**
* The beginning of the selected range.
* @type {Float}
* @default "0"
*/
startValue?: Float;

/**
* The end of the selected range.
* @type {Float}
* @default "0"
*/
endValue?: Float;

/**
* Base style name for CSS styles applied to the background of the rangeSlider. The following
*  suffixes are applied for different areas of the slider:
*  
*  
* - "Start": area of the slider before the startThumb
*  
* - "Selected": area of the slider between the thumbs (the selected range)
*  
* - "End": area of the slider after the endThumb
*  
*  .. and the following suffixes are applied in addition according to the slider's dynamic state:
*  
*  
* - "Over": if the mouse is over the segment
*  
* - "Down": if the mouse is down on the segment
*  
*  For example, if the mouse is down in the area before the start thumb, that area will have the
*  CSS style "rangeSliderStartDown".
* @type {CSSStyleName}
* @default "rangeSlider"
*/
baseStyle?: CSSStyleName;



/**
*  Notification fired when the selected range is changed by the end user.
* 
* 
* @param {Float} new start value
* @param {Float} new end value
* @param {boolean} whether the user is still in the middle of a drag, so that expensive operations can be avoided if needed
*/
changed(startValue:Float, endValue:Float, isDragging:boolean): void;



}
/**
* Creates a new RangeSlider
* 
* @param typeCheckedProperties {Partial<RangeSlider>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RangeSlider} The new instance
*/
class RangeSlider{
static create(typeCheckedProperties? : Partial<RangeSlider>, uncheckedProperties?: Object): RangeSlider;







}

/**
* Component for editing a single record.
*  RecordEditors are implemented as a subclass of ListGrid, showing no header and a single 
*  row always drawn in the editable state, allowing the user to modify the values at any time.
*  The RecordEditor.actionButton is automatically shown as a way for the user to act 
*  upon the edited values.
*  
*  The RecordEditor class exists as a helper class for ListGrids, used to provide
*  an interface for editing criteria when ListGrid.showFilterEditor 
*  is set to true.
*/
interface RecordEditor extends ListGrid {
/**
* GridRenderer.baseStyle for this listGrid.
*  If this property is unset, base style may be derived from ListGrid.normalBaseStyle
*  or ListGrid.tallBaseStyle as described in
*  ListGrid.getBaseStyle.
*  
*  See cellStyleSuffixes for details on how stateful suffixes are combined
*  with the base style to generate stateful cell styles.
* @type {CSSStyleName}
* @default "recordEditorCell"
*/
baseStyle?: CSSStyleName;

/**
* If showing any record summary fields (IE: fields of ListGridFieldType),
*  this attribute specifies a custom base style to apply to cells in the summary field
* @type {CSSStyleName}
* @default "recordEditorCell"
*/
recordSummaryBaseStyle?: CSSStyleName;

/**
* CanSaveSearches is explicitly disabled for RecordEditors by default.
* @type {boolean}
* @default false
*/
canSaveSearches?: boolean;

/**
* Where do 'skin' images (those provided with the class) live?
* @type {SCImgURL}
* @default "images/RecordEditor/"
*/
skinImgDir?: SCImgURL;

/**
* Button.icon to show on the RecordEditor.actionButton if this 
*  component is being used as a ListGrid.filterEditor.
*  
*  Note that this SCImgURL will be resolved using the RecordEditor.skinImgDir defined
*  for the RecordEditor.
* @type {SCImgURL}
* @default "[SKIN]filter.png"
*/
filterImg?: SCImgURL;

/**
* Automatically created Button auto-child shown at the edge of the recordEditor.
*  For a recordEditor acting as a ListGrid.filterEditor, this button will
*  show the RecordEditor.filterImg as an Button.icon by default.
*  
*  Clicking this button will call RecordEditor.performAction on the editor.
*  
*  May be customized using the standard AutoChild pattern, by overriding
*  RecordEditor.actionButtonProperties.
* @type {Button}
* @default null
*/
actionButton?: Button;

/**
* Properties to apply to the automatically generated RecordEditor.actionButton.
*  
*  Note that for a recordEditor being used as a ListGrid.filterEditor, the
*  ListGrid.filterButtonProperties can be used to specify actionButton properties
*  directly at the grid level.
* @type {Partial<Button>}
* @default null
*/
actionButtonProperties?: Partial<Button>;

/**
* Button.baseStyle for the RecordEditor.actionButton
* @type {CSSStyleName}
* @default "normal"
*/
actionButtonStyle?: CSSStyleName;

/**
* Should non-editable fields within this record-editor always display the 
*  ListGrid.emptyCellValue for null values rather than 
*  running any specified ListGridField.formatCellValue?
*  
*  This setting allows developers to easily account for the fact that a custom 
*  formatter for empty cells in a listGrid is often not appropriate to display
*  in the filter editor for a canFilter:false field with no specified
*  criterion.
* @type {boolean}
* @default true
*/
suppressNullValueFormat?: boolean;



/**
*  Fired when the user clicks the RecordEditor.actionButton for this RecordEditor.
*  May also be triggered from other user interaction with edit values (for
*  example filter-editor change - see ListGrid.filterOnKeypress).
*  
*  This is the method which initiates a filter in a listGrid
*  ListGrid.showFilterEditor. Note that for custom
*  filtering behavior, developers can use the ListGrid.filterEditorSubmit
*  notification method rather than overriding this method directly.
* 
* 
*/
performAction(): void;



}
/**
* Creates a new RecordEditor
* 
* @param typeCheckedProperties {Partial<RecordEditor>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RecordEditor} The new instance
*/
class RecordEditor{
static create(typeCheckedProperties? : Partial<RecordEditor>, uncheckedProperties?: Object): RecordEditor;







}

/**
* An application available within reifyOnSite that allows developers
*  to create and manage SmartClient screens and datasources. Only internal framework
*  code can create an instance of the Reify tool - do not try it directly in your
*  applications. If you want to create visual tools similar to Reify, see
*  devTools.
*  
*  Note that in the SmartClient SDK, this class present only to provide
*  reifyForDevelopers utility class method APIs, and is not an instantiable
*  widget. For example, you can call Reify.getMockDS to export a DataSource as
*  XML-formatted values and metadata for importing into Reify to create a
*  MockDataSource.
*/
interface Reify extends VLayout {




}
/**
* Creates a new Reify
* 
* @param typeCheckedProperties {Partial<Reify>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Reify} The new instance
*/
class Reify{
static create(typeCheckedProperties? : Partial<Reify>, uncheckedProperties?: Object): Reify;
/**
* URL of Reify server to use when calling Reify.loadProject. This URL is assumed
*  to only specify the server root, so Reify.projectLoaderPath will be appended to
*  it when sending the actual request. Can be overridden by
*  LoadProjectSettings.serverURL.
*  
*  Note that, unless this URL is on your local VPN, it is recommended to use
*  https to protect your login credentials.
* @type {URL}
* @default "https://reify.com"
*/
static serverURL?: URL;

/**
* Sets the path to reach the servletDetails relative to
*  the base Reify.serverURL. Can be overridden by
*  LoadProjectSettings.projectLoaderPath.
* @type {string}
* @default "isomorphic/projectLoader"
*/
static projectLoaderPath?: string;

/**
* Account name to use for authenticating with the Reify server when calling
*  Reify.loadProject. If proper credentials are not provided the project will not
*  be loaded. Can be overridden by LoadProjectSettings.userName.
*  
*  Note that you can set your email address into this property instead of your user name,
*  and the server should still be able to authenticate you for project loading.
* @type {string}
* @default null
*/
static userName?: string;

/**
* Account password to use for authenticating with the Reify server when calling
*  Reify.loadProject. If proper credentials are not provided the project will not
*  be loaded. Can be overridden by LoadProjectSettings.userName.
* @type {string}
* @default null
*/
static password?: string;

/**
* Controls whether LoadProjectSettings.verifyDataSources
*  is enabled by default for all RPCManager.loadProject operations.
* @type {boolean}
* @default null
*/
static verifyDataSources?: boolean;





/**
*  Exports or serializes the specified DataSource using the provided
*  settings.
*  
*  The "reifyCSV" MockDSExportSettings.format generates comma-separated
*  values to paste into the DataSource creation wizard in reifyForDevelopers. The use case for the other two formats is, if you have a SmartClient
*  application, and you plan to load MockDataSource to enable
*  people to add screens to your application using Reify, you may want to test your
*  application with the MockDataSources to ensure they have the right data to allow your
*  application to function (for example, that records in one MockDataSource that are
*  related to another MockDataSource match up). Similarly, you may want to test any custom
*  classes that you upload to Reify in a standalone file using MockDataSource.
*  
*  You can customize the settings, such as
*  MockDSExportSettings.numRows (or
*  MockDSExportSettings.numLevels for tree-DataSources) to keep the data
*  volume returned by the export low. When related DataSources are present, all related
*  records will be included in the export, even if numRows is exceeded. If
*  this is too much data, MockDSExportSettings.criteria can be used to
*  further restrict exported records. Note that settings supports an array of
*  MockDSExportSettings.requestProperties, so that you can provide
*  unique configuration for each DataSource being exported, rather than only global
*  configuration.
*  
*  Unless you need programmatic or expert control over the settings, you will likely find
*  it easier to use the "Reify Export" button in the dataSourcesTab.
*  as when using that route, useful global and per-DataSources settings can be configured
*  in an intuitively-arranged popup dialog.
* 
* 
* @param {Array<Partial<string>> | string} DataSource.IDs of the desired                      DataSources
* @param {MockDSExportCallback} called with the complete export or serialization
* @param {MockDSExportSettings} controls format and what records and metadata to                    include
*/
static getMockDS(dsNames:Array<Partial<string>> | string, callback:(allDSData:string, perDSData:Array<Partial<string>>) => void, settings:MockDSExportSettings): void;

/**
*  Shows the result of running Reify.getMockDS in a ModalWindow so
*  it can be copied and pasted as needed into reifyForDevelopers or
*  elsewhere.
*  
*  Note that the callback is fired when the window is closed, not when it's populated.
* 
* 
* @param {Array<Partial<string>> | string} DataSource.IDs of the desired                      DataSources
* @param {MockDSExportCallback} called with the complete export or serialization
* @param {MockDSExportSettings} controls format and what records and metadata to                    include
*/
static showMockDS(dsNames:Array<Partial<string>> | string, callback:(allDSData:string, perDSData:Array<Partial<string>>) => void, settings:MockDSExportSettings): void;

/**
*  Setter for Reify.serverURL.
* 
* @param {URL} */
static setServerURL(serverURL:URL): void;

/**
*  Setter for Reify.projectLoaderPath.
* 
* @param {string} */
static setProjectLoaderPath(path:string): void;

/**
*  Setter for Reify.userName.
* 
* @param {string} */
static setUserName(userName:string): void;

/**
*  Setter for Reify.password.
* 
* @param {string} */
static setPassword(password:string): void;

/**
*  Loads projects from the Reify server specified by Reify.serverURL (or
*  LoadProjectSettings.serverURL) using the 
*  servletDetails, reachable at the relative path
*  Reify.projectLoaderPath (or LoadProjectSettings.projectLoaderPath) underneath
*  the server URL, and fires the given callback after the project has been cached. When a
*  project is loaded, all of its DataSources and screens (except where explicitly
*  overridden by settings) are also cached in the project.
*  
*  See RPCManager.loadProject for further details.
* 
* 
* @param {string} Comma-separated string containing the names of                      project(s) to load.
* @param {LoadProjectCallback} Callback for notification of completion of                      project(s) loaded and screens cached.
* @param {LoadProjectSettings} Settings applicable to the loadProject                      operation.
*/
static loadProject(projectNames:string, callback:(project:Project, projects:Array<Partial<Project>>, rpcResponse:RPCResponse) => void, settings:LoadProjectSettings): void;




}

/**
* A FormItem for entering a date relative to today or relative to some other date, or a 
*  specific absolute date. Typically used for filtering data by date.
*  
*  The RelativeDateItem consists of a ComboBoxItem where the user may directly choose 
*  one of several RelativeDateItem.presetOptions, choose to enter a 
*  RelativeDateItem.quantityField and TimeUnit 
*  (eg "4 months ago" or "3 years from now") or directly type in 
*  an absolute date value (7/18/2009).
*  
*  This item can work with logical dates or datetimes, depending on the specified 
*  DataSourceField.type. For detailed information on working with dates, 
*  times and datetimes, see the 
*  dateFormatAndStorage.
*/
interface RelativeDateItem extends Omit<CanvasItem, 'valueField' | 'formatEditorValue'>  {
/**
* Does this item's relative date value refer to the start or end of the chosen date? Useful
*  when using this item to generate filter criteria, such as the from or to value for an
*  inclusive range.
*  
*  If unset "start" is assumed.
* @type {RelativeDateRangePosition}
* @default null
*/
rangePosition?: RelativeDateRangePosition;

/**
* List of time units that will be offered for relative dates.
*  
*  Each available time unit option will cause two options to appear in the 
*  RelativeDateItem.valueField. For example, if "day" is an available TimeUnit
*  option, there will be RelativeDateItem.daysAgoTitle and 
*  RelativeDateItem.daysFromNowTitle.
* @type {Array<Partial<TimeUnit>>}
* @default "["day", "week", "month"]"
*/
timeUnitOptions?: Array<Partial<TimeUnit>>;

/**
* Should we show time-unit options in the past? If set to false, for each 
*  RelativeDateItem.timeUnitOptions we will show only future options [for example
*  "N weeks from now"].
*  
*  Note: this does not change the RelativeDateItem.presetOptions, which show up in 
*  addition to the time-unit options ("N days from now", etc). The default
*  preset options include both past and future presets so developers may wish to modify the
*  presets to ensure only past options are available.
* @type {boolean}
* @default true
*/
showPastOptions?: boolean;

/**
* Should we show time-unit options in the future? If set to false, for each 
*  RelativeDateItem.timeUnitOptions we will show only past options [for example
*  "N weeks ago"].
*  
*  Note: this does not change the RelativeDateItem.presetOptions, which show up in 
*  addition to the time-unit options ("N days from now", etc). The default
*  preset options include both past and future presets so developers may wish to modify the
*  presets to ensure only future options are available.
* @type {boolean}
* @default true
*/
showFutureOptions?: boolean;

/**
* When set to false, only relative dates can be entered - in this mode, the 
*  RelativeDateItem.showChooserIcon is hidden and the RelativeDateItem.valueField
*  is switched from a ComboBoxItem, which allows text-entry, to a 
*  SelectItem which does not.
* @type {boolean}
* @default true
*/
allowAbsoluteDates?: boolean;

/**
* A map from a granularity of time specified by a user to the granularity of time used for 
*  rounding.
*  
*  A relative date such as "n days from now" is normally shifted to the end of the day when 
*  used as a range endpoint, and the beginning of the day when used as the beginning of a range.
*  (The rounding direction on some items can be specified via 
*  RelativeDateItem.rangePosition).
*  This causes the intuitive behavior that "from yesterday to today" is from the beginning of
*  yesterday to the end of today, and that "from today until 5 days from now" includes the 
*  entirety of Friday if today is Monday.
*  
*  This same rule can be applied to any time granularity, such that "from now until 
*  20 minutes from now" is up to 5:32 if it is now 5:11:34, and 
*  "from now until 2 months from now" means end of June if it is mid-April.
*  
*  User intuitions about where this rounding is expected for any given time period tend to
*  vary based on what kind of event is being discussed and subtle phrasing differences 
*  (consider "up to one year from now", "until next year", "within the next couple of years"). 
*  The defaults behaviors are:
*  
*  
* - for days, weeks and months round to day end/beginning
*  
* - for hours, round to minute end/beginning
*  
* - for minutes and seconds, round to second end/beginning
*  
*  To customize this rounding behavior, this attribute may be set to a simple javascript object
*  mapping each timeUnit to the granularity for that timeUnit.
*  For example the following config code would produce an item where the user could select
*  only day or week values, and the selected value would be rounded to the beginning of the day
*  if a day was selected, or the beginning of the week if a week was selected:
*  
*  {
*    name:"fromDate", type:"RelativeDateItem",
*    timeUnitOptions:["day", "week"],
*    rangePosition:"start",
*    rangeRoundingGranularity:{
*      "day":"day",
*      "week":"week"
*    }
*  }
*  
* @type {object}
* @default "{...}"
*/
rangeRoundingGranularity?: object;

/**
* The title to show for historical periods when the TimeUnit is "millisecond".
* @type {string}
* @default "N milliseconds ago"
*/
millisecondsAgoTitle?: string;

/**
* The title to show for historical periods when the TimeUnit is "second".
* @type {string}
* @default "N seconds ago"
*/
secondsAgoTitle?: string;

/**
* The title to show for historical periods when the TimeUnit is "minute".
* @type {string}
* @default "N minutes ago"
*/
minutesAgoTitle?: string;

/**
* The title to show for historical periods when the TimeUnit is "hour".
* @type {string}
* @default "N hours ago"
*/
hoursAgoTitle?: string;

/**
* The title to show for historical periods when the TimeUnit is "day".
* @type {string}
* @default "N days ago"
*/
daysAgoTitle?: string;

/**
* The title to show for historical periods when the TimeUnit is "week".
* @type {string}
* @default "N weeks ago"
*/
weeksAgoTitle?: string;

/**
* The title to show for historical periods when the TimeUnit is "month".
* @type {string}
* @default "N months ago"
*/
monthsAgoTitle?: string;

/**
* The title to show for historical periods when the TimeUnit is "quarter".
* @type {string}
* @default "N quarters ago"
*/
quartersAgoTitle?: string;

/**
* The title to show for historical periods when the TimeUnit is "year".
* @type {string}
* @default "N years ago"
*/
yearsAgoTitle?: string;

/**
* The title to show for future periods when the TimeUnit is "millisecond".
* @type {string}
* @default "N milliseconds from now"
*/
millisecondsFromNowTitle?: string;

/**
* The title to show for future periods when the TimeUnit is "second".
* @type {string}
* @default "N seconds from now"
*/
secondsFromNowTitle?: string;

/**
* The title to show for future periods when the TimeUnit is "minute".
* @type {string}
* @default "N minutes from now"
*/
minutesFromNowTitle?: string;

/**
* The title to show for future periods when the TimeUnit is "hour".
* @type {string}
* @default "N hours from now"
*/
hoursFromNowTitle?: string;

/**
* The title to show for future periods when the TimeUnit is "day".
* @type {string}
* @default "N days from now"
*/
daysFromNowTitle?: string;

/**
* The title to show for future periods when the TimeUnit is "week".
* @type {string}
* @default "N weeks from now"
*/
weeksFromNowTitle?: string;

/**
* The title to show for future periods when the TimeUnit is "month".
* @type {string}
* @default "N months from now"
*/
monthsFromNowTitle?: string;

/**
* The title to show for future periods when the TimeUnit is "quarter".
* @type {string}
* @default "N quarters from now"
*/
quartersFromNowTitle?: string;

/**
* The title to show for future periods when the TimeUnit is "year".
* @type {string}
* @default "N years from now"
*/
yearsFromNowTitle?: string;

/**
* Default value to show. Can be a concrete Date, a RelativeDateString that matches 
*  one of the RelativeDateItem.presetOptions, or one of the available 
*  RelativeDateItem.timeUnitOptions. If setting a TimeUnit,
*  use RelativeDateItem.defaultQuantity to establish a default 
*  value for the RelativeDateItem.quantityField.
* @type {Date | RelativeDateString | TimeUnit}
* @default "$today"
*/
defaultValue?: Date | RelativeDateString | TimeUnit;

/**
* What operator to use when RelativeDateItem.getCriterion
*  
*  is called.
* @type {OperatorId}
* @default "greaterThan"
*/
operator?: OperatorId;

/**
* Preset relative dates, such as "today" or "tomorrow", that the user can choose directly
*  from the RelativeDateItem.valueField.
*  
*  Format is an Object mapping user-visible titles to RelativeDateShortcut or
*  RelativeDateStrings. The default value (expressed in JSON) is:
*  
*  {
*    "$today" : "Today",
*    "$yesterday" : "Yesterday",
*    "$tomorrow" : "Tomorrow",
*    "$weekAgo" : "Current day of last week",
*    "$weekFromNow" : "Current day of next week",
*    "$monthAgo" : "Current day of last month",
*    "$monthFromNow" : "Current day of next month"
*  }
*  
*  In addition to these presets, options are shown for each of the 
*  TimeUnit.
* @type {object}
* @default "see below"
*/
presetOptions?: object;

/**
* The FormItem.width for the RelativeDateItem.valueField
*  in this item. Defaults to the current default value for the width attribute on the 
*  DateTimeItem class - this is assumed to be just wide enough to show a full 
*  datetime string, in the current global datetime format.
*  
*  Setting the width globally on the DateTimeItem class results
*  in all text-based datetime entry fields assuming the same default width - this caters for
*  custom date-time formatters that need differing amounts of space.
* @type {Integer | string}
* @default null
*/
valueFieldWidth?: Integer | string;

/**
* ComboBoxItem field where a user may choose among 
*  RelativeDateItem.presetOptions, 
*  TimeUnit plus RelativeDateItem.quantityField, or 
*  direct entry of a date as text.
* @type {ComboBoxItem}
* @default null
*/
valueField?: ComboBoxItem;

/**
* Default quantity to show in the RelativeDateItem.quantityField.
* @type {Integer}
* @default "1"
*/
defaultQuantity?: Integer;

/**
* Minimum value to allow in the RelativeDateItem.quantityField.
* @type {Integer}
* @default "0"
*/
minQuantity?: Integer;

/**
* Maximum value to allow in the RelativeDateItem.quantityField. Increasing this value may result in
*  date miscalculations for very large numbers, due to Javascript Date limitations.
* @type {Integer}
* @default "999999"
*/
maxQuantity?: Integer;

/**
* Field allowing user to pick units of time, eg, number of days.
* @type {SpinnerItem}
* @default null
*/
quantityField?: SpinnerItem;

/**
* Should we show the icon that displays a date-chooser?
* @type {boolean}
* @default true
*/
showChooserIcon?: boolean;

/**
* Icon that launches a DateChooser for choosing an absolute date.
* @type {FormItemIcon}
* @default null
*/
pickerIcon?: FormItemIcon;

/**
* Prompt to show when the user hovers the mouse over the picker icon for this 
*  RelativeDateItem. May be overridden for localization of your application.
* @type {HTMLString}
* @default "Show Date Chooser"
*/
pickerIconPrompt?: HTMLString;

/**
* SmartClient class for the DateChooser autoChild displayed to allow the user
*  to directly select dates.
* @type {string}
* @default "DateChooser"
*/
pickerConstructor?: string;

/**
* Base date for calculating the relative date entered by the user.
*  
*  The default is to use the current date.
* @type {Date}
* @default null
*/
baseDate?: Date;

/**
* Should the Calculated-Date be displayed to the right of the RelativeDateItem.pickerIcon.
* @type {boolean}
* @default true
*/
showCalculatedDateField?: boolean;

/**
* Field that shows the current calculated date by adding the user-entered relative date to
*  the RelativeDateItem.baseDate.
* @type {BlurbItem}
* @default null
*/
calculatedDateField?: BlurbItem;

/**
* Format for direct user input of date values.
*  
*  If unset, the input format will be determined based on the specified
*  RelativeDateItem.displayFormat if possible, otherwise picked up from the Date class (see
*  DateUtil.setInputFormat).
*  
*  
*  Note: if entirely custom date formatting/parsing logic is required for this item, 
*  this attribute may be set to a function which takes a single parameter (the formatted
*  date string) and returns a JavaScript date object.
*  
* @type {DateInputFormat}
* @default null
*/
inputFormat?: DateInputFormat;

/**
* Format for displaying dates in the RelativeDateItem.valueField and RelativeDateItem.calculatedDateField. 
*  Defaults to the system-wide default established by 
*  DateUtil.setShortDisplayFormat, or if this item has its type specified as
*  datetime, DateUtil.setShortDatetimeDisplayFormat.
* @deprecated * in favor of RelativeDateItem.dateFormatter

* @type {DateDisplayFormat}
* @default null
*/
displayFormat?: DateDisplayFormat;

/**
* Format for displaying dates in the RelativeDateItem.valueField and RelativeDateItem.calculatedDateField.
*  If unset a default DateDisplayFormat will be picked up from DynamicForm.dateFormatter
*  (or DynamicForm.datetimeFormatter for datetime fields} or otherwise from 
*  the system-wide default established by DateUtil.setShortDisplayFormat, or
*  if this item has its type specified as datetime, 
*  DateUtil.setShortDatetimeDisplayFormat.
*  
*  
*  Note: if entirely custom date formatting/parsing logic is required for this item, this
*  attribute may be set to a custom formatting function. In this case the function will be
*  applied to the Date being formatted (for example this.getFullYear() would
*  give you back the full year), and should return the formatted date string.
*  
* @type {DateDisplayFormat}
* @default null
*/
dateFormatter?: DateDisplayFormat;

/**
* Limits the range of the popup DateChooser. The default of null
*  causes the chooser's Year-picker to offer a range that starts 10 years before
*  the current selection.
* @type {Date}
* @default null
*/
startDate?: Date;

/**
* Limits the range of the popup DateChooser. The default of null
*  causes the chooser's Year-picker to offer a range that ends 10 years after the 
*  current selection.
* @type {Date}
* @default null
*/
endDate?: Date;

/**
* Only used if we're showing the date in a text field. When parsing a date, if the year
*  is specified with 1 or 2 digits and is less than the centuryThreshold, then the year will
*  be assumed to be 20xx; otherwise it will be interpreted according to default browser
*  behavior, which will consider it to be 19xx.
*  
*  By default, the centuryThreshold is calculated as the current year + 25.
*  
*  If you need to allow 1 and 2 digit years, set this attribute to 
*  null to have the control retain your year-value as entered.
* @type {number}
* @default "25"
*/
centuryThreshold?: number;

/**
* Should this item's value be saved in the form's values and hence returned from
*  DynamicForm.getValues?
*  
*  shouldSaveValue:false is used to mark formItems which do not correspond to
*  the underlying data model and should not save a value into the form's
*  DynamicForm.values. Example includes visual separators, password re-type fields,
*  or checkboxes used to show/hide other form items.
*  
*  A shouldSaveValue:false item should be given a value either via
*  FormItem.defaultValue or by calling
*  DynamicForm.setValue or 
*  FormItem.setValue. Providing a value via
*  DynamicForm.values or DynamicForm.setValues 
*  will automatically switch the item to shouldSaveValue:true.
*  
*  Note that 
*  
* - if an item is shouldSaveValue true, but has no name, a warning is logged, and 
*    shouldSaveValue will be set to false.
*  
* @type {boolean}
* @default true
*/
shouldSaveValue?: boolean;

/**
* When set to true (the default), use a single shared date-picker across all widgets that
*  use one. When false, create a new picker using the autoChild system. See 
*  DateItem.pickerDefaults and 
*  DateItem.pickerProperties for details on setting up an unshared
*  picker.
* @type {boolean}
* @default true
*/
useSharedPicker?: boolean;

/**
* When set to true, show a button that allows the calendar to be navigated by fiscal year.
* @type {boolean}
* @default false
*/
showChooserFiscalYearPicker?: boolean;

/**
* When set to true, show a button that allows the calendar to be navigated by week or
*  fiscal week, depending on the value of RelativeDateItem.showChooserFiscalYearPicker.
* @type {boolean}
* @default false
*/
showChooserWeekPicker?: boolean;

/**
* If this item is editing a field of type "datetime", should the
*  DateChooser display the DateChooser.showTimeItem, 
*  allowing the user to select a time?
*  
*  One case where developers will wish to suppress this time-field from being displayed
*  is if a custom RelativeDateItem.dateFormatter has been specified which does not display the
*  time portion of the selected date. In this case any value selected from the 
*  DateChooser's time field will be discarded when the picker is dismissed, making
*  it a confusing UI for the end user.
*  
*  Has no effect if the field type is "date" - in this case the
*  picker will never show the time field.
* @type {boolean}
* @default true
*/
showPickerTimeItem?: boolean;

/**
* A set of properties to apply to the TimeItem displayed in the picker when
*  RelativeDateItem.showPickerTimeItem is true.
*  
*  Has no effect for fields of type "date".
* @type {Partial<TimeItem>}
* @default null
*/
pickerTimeItemProperties?: Partial<TimeItem>;

/**
* When showing the DateChooser, should the 
*  DateChooser.showTimeItem be set to use 24-hour time?
*  Has no effect for fields of type "date" rather than 
*  "datetime", or if RelativeDateItem.showPickerTimeItem is false.
*  
*  Default is true.
* @type {boolean}
* @default true
*/
use24HourTime?: boolean;



/**
*  RelativeDateItems do not make use of the standard FormItem.formatEditorValue and
*  FormItem.parseEditorValue methods. Developers can customize the display values
*  for these items in the following ways:
*  
* - The RelativeDateItem.presetOptions map allows standard preset RelativeDateString 
*    and RelativeDateShortcut values to be mapped to custom display values
*  
* - The text displayed for each of the RelativeDateItem.timeUnitOptions (e.g:"N days ago") may be
*    customized via the per-time unit title attributes (RelativeDateItem.daysFromNowTitle, 
*    RelativeDateItem.daysAgoTitle, etc)
*  
* - The RelativeDateItem.dateFormatter and RelativeDateItem.inputFormat may be used modify how date values
*    are displayed (both in the text entry box and in the
*   RelativeDateItem.showCalculatedDateField
*  
* 
*/
formatEditorValue(): void;

/**
*  RelativeDateItems do not make use of the standard FormItem.formatEditorValue and
*  FormItem.parseEditorValue methods. Developers can customize the display values
*  for these items in the following ways:
*  
* - The RelativeDateItem.presetOptions map allows standard preset RelativeDateString 
*    and RelativeDateShortcut values to be mapped to custom display values
*  
* - The text displayed for each of the RelativeDateItem.timeUnitOptions (e.g:"N days ago") may be
*    customized via the per-time unit title attributes (RelativeDateItem.daysFromNowTitle, 
*    RelativeDateItem.daysAgoTitle, etc)
*  
* - The RelativeDateItem.dateFormatter and RelativeDateItem.inputFormat may be used modify how date values
*    are displayed (both in the text entry box and in the
*   RelativeDateItem.showCalculatedDateField
*  
* 
*/
parseEditorValue(): void;

/**
*  Returns the current RelativeDate object for this item. Only applies if the user entered
*  a relative date value (such as "Today") - if an absolute date was entered, this method
*  returns null.
*  Relative date objects have the following format:
*  
*    { _constructor: "RelativeDate", value: "$today" }
*  
* 
* 
*/
getRelativeDate(): object;

/**
*  Get the criterion based on the values the user has entered.
* 
* @param {boolean=} whether to use an absolute date in the Criterion produced.                By default a RelativeDate will be used if the user                entered a relative date value
*/
getCriterion(absolute?:boolean): Criterion;

/**
*  Returns the FiscalCalendar object that will be used by this item's DateChooser.
* 
* 
*/
getFiscalCalendar(): FiscalCalendar;

/**
*  Sets the FiscalCalendar object that will be used by this item's DateChooser. If 
*  unset, the DateUtil.getFiscalCalendar is used.
* 
* 
* @param {FiscalCalendar=} the fiscal calendar for this chooser, if set, or the global      one otherwise
*/
setFiscalCalendar(fiscalCalendar?:FiscalCalendar): void;

/**
*  If DateItem.useTextField is true this method returns a
*  standard DateInputFormat, determining how values entered
*  by the user are to be converted to Javascript Date objects.
*  
*  If an explicit DateItem.inputFormat has been specified it will be returned,
*  otherwise, if a custom DateItem.dateFormatter or FormItem.format are
*  specified, the input format will be automatically derived from that property.
*  
*  Otherwise, the global DateUtil.setInputFormat is used.
*  
*  Note that the inputFormat will ignore any separator characters and padding of values.
*  However if necessary entirely custom date formatting and parsing may be achieved via the
*  DateItem.formatEditorValue and
*  DateItem.parseEditorValue methods.
*  
*  
* 
*/
getInputFormat(): DateInputFormat;

/**
*  Returns the raw text value typed into this items value text field
* 
*/
getEnteredValue(): void;



}
/**
* Creates a new RelativeDateItem
* 
* @param typeCheckedProperties {Partial<RelativeDateItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RelativeDateItem} The new instance
*/
class RelativeDateItem{
static create(typeCheckedProperties? : Partial<RelativeDateItem>, uncheckedProperties?: Object): RelativeDateItem;




/**
*  Converts a RelativeDate, RelativeDateShortcut,
*  
*  or RelativeDateString to a concrete Date.
* 
* @param {RelativeDate} the relative date to convert
* @param {Date=} base value for conversion. Defaults to today
*/
static getAbsoluteDate(relativeDate:RelativeDate, baseDate?:Date): Date;




}

/**
* Provides APIs that manipulate a SmartClient browser window.
*  Within the OpenFin environment,
*  the underlying implementation is actually via the OpenFinWindow class.
*  
*  Note: this is currently an experimental feature and not supported except by special
*  arrangement
*/
interface RemoteWindow extends Class {


/**
*  Returns the name of this RemoteWindow.
* 
*/
getName(): string;

/**
*  Returns the parent RemoteWindow instance that opened this RemoteWindow.
* 
*/
getParent(): RemoteWindow;

/**
*  Returns the browser window object associated with this RemoteWindow.
* 
*/
getWindow(): object;

/**
*  Returns the container window, if present, wrapping the browser window for this
*  RemoteWindow. If OpenFin is present, this will return the associated OpenFin
*  Window.
* 
*/
getContainerWindow(): object;

/**
*  Closes this RemoteWindow.
* 
* @param {boolean=} whether to force it closed
* @param {RemoteWindowCallback=} callback run after it's closed
*/
close(force?:boolean, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Moves this RemoteWindow.
* 
* @param {number} desired x-offset of left edge
* @param {number} desired y-offset of top edge
* @param {RemoteWindowCallback=} callback run after it's moved
*/
move(x:number, y:number, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Resizes this RemoteWindow.
* 
* @param {number} desired new width
* @param {number} desired new height
* @param {RemoteWindowCallback=} callback run after it's resized
*/
resize(width:number, height:number, callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Focuses (activates) this RemoteWindow.
* 
* @param {RemoteWindowCallback=} callback run after it's activated
*/
focus(callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Focuses (activates) this RemoteWindow.
* 
* @param {RemoteWindowCallback=} callback run after it's activated
*/
activate(callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Blurs (deactivates) this RemoteWindow.
* 
* @param {RemoteWindowCallback=} callback run after it's deactivated
*/
blur(callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Blurs (deactivates) this RemoteWindow.
* 
* @param {RemoteWindowCallback=} callback run after it's deactivated
*/
deactivate(callback?:(remoteWindow:RemoteWindow) => void): void;

/**
*  Maximizes this RemoteWindow.
* 
* @param {RemoteWindowCallback=} callback run after maximization
* @param {RemoteWindowErrorCallback=} callback run if maximizing fails
*/
maximize(callback?:(remoteWindow:RemoteWindow) => void, errorCallback?:(remoteWindow:RemoteWindow, errorMessage:string) => void): void;

/**
*  Minimizes this RemoteWindow.
* 
* @param {RemoteWindowCallback=} callback run after minimization
* @param {RemoteWindowErrorCallback=} callback run if minimizing fails
*/
minimize(callback?:(remoteWindow:RemoteWindow) => void, errorCallback?:(remoteWindow:RemoteWindow, errorMessage:string) => void): void;

/**
*  Restores this RemoteWindow from being maximized or minimized.
* 
* @param {RemoteWindowCallback=} callback run after restoration
* @param {RemoteWindowErrorCallback=} callback run if restoring fails
*/
restore(callback?:(remoteWindow:RemoteWindow) => void, errorCallback?:(remoteWindow:RemoteWindow, errorMessage:string) => void): void;

/**
*  Hides this RemoteWindow.
* 
* @param {RemoteWindowCallback=} callback run after hiding the window
* @param {RemoteWindowErrorCallback=} callback run if hiding fails
*/
hide(callback?:(remoteWindow:RemoteWindow) => void, errorCallback?:(remoteWindow:RemoteWindow, errorMessage:string) => void): void;

/**
*  Shows this RemoteWindow.
* 
* @param {RemoteWindowCallback=} callback run after showing the window
* @param {RemoteWindowErrorCallback=} callback run if showing fails
*/
show(callback?:(remoteWindow:RemoteWindow) => void, errorCallback?:(remoteWindow:RemoteWindow, errorMessage:string) => void): void;

/**
*  Brings this RemoteWindow to the front in window stacking order.
* 
* @param {RemoteWindowCallback=} callback run after raising window
* @param {RemoteWindowErrorCallback=} callback run if raising fails
*/
bringToFront(callback?:(remoteWindow:RemoteWindow) => void, errorCallback?:(remoteWindow:RemoteWindow, errorMessage:string) => void): void;

/**
*  Checks whether this RemoteWindow is showing.
* 
* @param {RemoteWindowBooleanCallback} callback to receive output
*/
isShowing(callback:(remoteWindow:RemoteWindow, result:boolean) => void): void;

/**
*  Checks whether this RemoteWindow is showing.
* 
* @param {RemoteWindowMapCallback} callback to receive output
*/
getInfo(callback:(remoteWindow:RemoteWindow, result:Map<any>) => void): void;



}
/**
* Creates a new RemoteWindow
* 
* @param typeCheckedProperties {Partial<RemoteWindow>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RemoteWindow} The new instance
*/
class RemoteWindow{
static create(typeCheckedProperties? : Partial<RemoteWindow>, uncheckedProperties?: Object): RemoteWindow;







}

/**
* Button that resets the form to default values, by calling
*  DynamicForm.resetValues()
*  
*  If you define a click handler on this item, you can return false to cancel the reset.
*/
interface ResetItem extends ButtonItem {




}
/**
* Creates a new ResetItem
* 
* @param typeCheckedProperties {Partial<ResetItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ResetItem} The new instance
*/
class ResetItem{
static create(typeCheckedProperties? : Partial<ResetItem>, uncheckedProperties?: Object): ResetItem;







}

/**
* Show user password reset dialog by opening the Authentication.resetPasswordURL
*  in another tab or window.
*/
interface ResetPasswordTask extends ProcessElement {




}
/**
* Creates a new ResetPasswordTask
* 
* @param typeCheckedProperties {Partial<ResetPasswordTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ResetPasswordTask} The new instance
*/
class ResetPasswordTask{
static create(typeCheckedProperties? : Partial<ResetPasswordTask>, uncheckedProperties?: Object): ResetPasswordTask;







}

/**
* The RestDataSource implements the 4 core DataSource operations using a simple protocol of
*  XML or JSON requests and responses sent over HTTP, which can be easily fulfilled by any HTTP
*  server technology.
*  
*  RestDataSource is named for the
*  REST (REpresentational State
*  Transfer) pattern, which in brief says that simple messages passed over HTTP is a sufficient
*  protocol for many web applications, without the need for further protocols such as WSDL or
*  SOAP.
*  
*  A RestDataSource is used just like a normal DataSource. RestDataSources are pre-configured,
*  using the general-purpose databinding facilities of DataSources, to expect a particular
*  format for responses and to send requests in a specific format.  These request and
*  response formats represent Isomorphic's recommended best practices for binding SmartClient
*  to backends which do not already support a similar, pre-existing request and response
*  format and where the SmartClient Java Server cannot be used. 
*  
*  If you have a pre-existing REST or WSDL service which is difficult to change, consider
*  adapting SmartClient to the existing service instead, by starting with a normal
*  DataSource and using the 
*  clientDataIntegration facilities to create a
*  mapping between SmartClient's DSRequest and DSResponse objects and the message
*  formats of your existing services. NOTE: do not begin this process by
*  creating or subclassing RestDataSource; for a pre-existing service which is unrelated
*  to the protocol documented for RestDataSource, start by configuring or subclassing
*  DataSource instead.
*  
*  RestDataSource is typically used with PHP, Ruby, Python, Perl or custom server technologies,
*  and represents an alternative to installing the SmartClient Server in a Java technology
*  stack, or using wsdlBinding with .NET or other WSDL-capable
*  technologies. Note that SmartClient Server also provides built-in support for the REST
*  protocol via its RESTHandler servlet; this is primarily to allow non-SmartClient clients
*  to make use of DataSource operations. If you particularly wished to do so, you could use
*  RestDataSource to make a SmartClient app talk to the SmartClient Server using REST rather 
*  than the proprietary wire format normally used when communicating with SmartClient Server
*  (this is how we are able to write automated tests for the RESTHandler servlet). However,
*  doing this provides no benefit, imposes a number of inconveniences, and makes a handful 
*  of server-based features less useful 
*  (DataSourceField.viewRequiresAuthentication, for 
*  example), so we strongly recommend that you do not do this; it is only mentioned
*  here for completeness while we are discussing REST.
*  
*  The request and response formats used by the RestDataSource allow for many of the available
*  features of SmartClient's databinding system to be used, including data paging, searching &
*  sorting, DSRequest.oldValues, 
*  ResultSet, relogin and 
*  RPCManager.startQueue. However, advanced
*  features such as upload and 
*  ListGrid.exportData aren't available with RestDataSource and need to be 
*  re-implemented as needed. Most, though not all, iscServer
*  are still available when using RestDataSource, as long as you are also using the RESTHandler 
*  servlet that is part of SmartClient Server. However, as noted above, this approach is not 
*  recommended; if you are using Isomorphic technology both client- and server-side, it makes
*  more sense to use the proprietary wire format.
*  
*  RestDataSource and binary data
*  
*  Binary data in a response provided to a RestDataSource must be delivered as valid XML or
*  JSON Strings. Once delivered to the browser as Strings, there is no way to trigger the
*  browser's "Save As" dialog to download the data, and in most cases no way to trigger other
*  helper applications that might be launched to handle binary data (such as Excel or a PDF
*  viewer). Hence for binary it usually makes sense to make a direct request via
*  RPCManager.sendRequest() with downloadResult:true, separate from RestDataSource.
*  
*  If you are using the SmartClient Server included in Pro, Power end Enterprise to handle your
*  REST requests server-side, there is transparent support for conversion between Java 
*  InputStreams representing binary data, and Strings containing that binary 
*  data encoded using the Base64 algorithm.
*  Thus, on the server, the binary data is in its raw binary form, with transparent conversion
*  to or from Base64 for messages to or from the REST client.
*  
*  Examples
*  
*  XML formatted responses:
*  
*  RestDataSource expects a response like the following in response to a "fetch" request:
*  
*  &lt;response&gt;
*   &lt;status&gt;0&lt;/status&gt;
*   &lt;startRow&gt;0&lt;/startRow&gt;
*   &lt;endRow&gt;76&lt;/endRow&gt;
*   &lt;totalRows&gt;546&lt;/totalRows&gt;
*   &lt;data&gt;
*    &lt;record&gt;
*      &lt;field1&gt;value&lt;/field1&gt;
*      &lt;field2&gt;value&lt;/field2&gt;
*    &lt;/record&gt;
*    &lt;record&gt;
*      &lt;field1&gt;value&lt;/field1&gt;
*      &lt;field2&gt;value&lt;/field2&gt;
*    &lt;/record&gt;
*    ... 76 total records ... 
*   &lt;/data&gt;
*  &lt;/response&gt;
*  
*  The &lt;status&gt; element indicates whether the fetch operation was successful 
*  (see statusCodes).
*  
*  The &lt;data&gt; element contains a list of record nodes, each of which represents a record
*  returned by the server. The optional &lt;startRow&gt;, &lt;endRow&gt; and &lt;totalRows&gt;
*  elements are needed only if data paging is in use, and populate the
*  DSResponse.startRow, DSResponse.endRow and
*  DSResponse.totalRows properties of the DSResponse.
*  
*  Note: for a more compact format, simple field values may be specified on record 
*  nodes directly as attributes - in this case a record element might be structured like this:
*  
*    &lt;record field1="value" field2="value" /&gt;
*  
*  
*  Note that a RestDataSource will bypass browser caching of all responses by default. See
*  DataSource.preventHTTPCaching.
*  
*  Successful "add" or "update" request responses are similar in format - in this case the data
*  element would be expected to contain a single record object containing the details of the
*  record, as saved on the server.
*  
*  The response from a "remove" operation would again include status and data elements, but in
*  this case, only the primary key field value(s) of the removed record would be expected to be 
*  present under the data element.
*  
*  If a validation failure occurred on the server, the response would
*  have status set to RPCResponse.STATUS_VALIDATION_ERROR [-4],
*  and any validation errors could be included as per-field sub-elements of an "errors"
*  element. For a validation error, the response is not expected to contain any
*  &lt;data&gt; element. 
*  
*  A response showing a validation error might look like this:
*  
*  &lt;response&gt;
*   &lt;status&gt;-4&lt;/status&gt;
*   &lt;errors&gt;
*    &lt;field1&gt;
*      &lt;errorMessage&gt;A validation error occurred for this field&lt;/errorMessage&gt;
*    &lt;/field1&gt;
*   &lt;/errors&gt;
*  &lt;/response&gt;
*  
*  
*  An unrecoverable error, such as an unexpected server failure, can be flagged by setting
*  &lt;status&gt; to -1 and setting &lt;data&gt; to an error message. In this case the
*  &lt;errors&gt; element is not used (it's specific to validation errors). An unrecoverable
*  error causes all response processing to be skipped and RPCManager.handleError to be
*  invoked, which by default will show the provided error message as an alert using
*  isc.warn.
*  
*  JSON formatted responses:
*  
*  JSON format responses are expected to contain the same data / meta-data as XMLresponses,
*  encapsulated in a simple object with a "response" attribute.
*  The response to a "fetch" request would therefore have this format:
*  
*  {
*   "response": {
*     "status": 0,
*     "startRow": 0,
*     "endRow": 76,
*     "totalRows": 546,
*     "data": [
*       {"field1": "value", "field2": "value"},
*       {"field1": "value", "field2": "value"},
*       ... 76 total records ...
*     ]
*   }
*  }
*  
*  The structure successful for "add", "update" and "remove" responses would be similar, though
*  the data array would be expected to contain only a single object, representing the values as
*  saved. This allows the server to return values such as an auto-generated sequence
*  primaryKey, a last modified timestamp, or similar server-generated field values.
*  
*  For a remove, only the value for the primaryKey field[s] would be required.
*  
*  For a validation error, the status attribute would be set to 
*  RPCResponse.STATUS_VALIDATION_ERROR [-4], and errors would
*  be specified in the errors attribute of the response. For example:
*  
*  {  "response":
*    {  "status": -4,
*      "errors":
*        {  "field1": {"errorMessage": "A validation error on field1"},
*          "field2": {"errorMessage": "A validation error on field2"}
*        }
*    }
*  }
*  
*  An array of errors may also be returned for a single field, like this:
*  
*  {  "response":
*    {  "status": -4,
*      "errors":
*        {  "field1": [
*            {"errorMessage": "First error on field1"},
*            {"errorMessage": "Second error on field1"}
*          ]
*        }
*    }
*  }
*  
*  
*  As with the XML format above, an unrecoverable error is indicated by setting the
*  status attribute to -1 and the data property to the error message.
*  
*  Responses with related updates
*  
*  Related updates is a way to communicate additional changes that occur as a consequence of the
*  current DSResponse succeeding, such as changes to other records in the same DataSource or to
*  records from unrelated DataSources. Related updates can be attached to main response via
*  DSResponse.addRelatedUpdate(dsResponse) server-side API, see its docs for more details.
*  RestDataSource supports this on the client, DataSource.updateCaches will be called for all
*  related updates found in response. Here's schematic example of how they look like:
*  
*  &lt;response&gt;
*    ... normal response ...
*    &lt;relatedUpdates&gt;
*      &lt;response&gt;
*        ... normal response ...
*      &lt;/response&gt;
*      &lt;response&gt;
*        ... normal response ...
*      &lt;/response&gt;
*    &lt;/relatedUpdates&gt;
*  &lt;/response&gt;
*  
*  same in JSON format
*  
*  {
*   ... normal response ...,
*   relatedUpdates: [
*    {
*     ... normal response ...
*    },
*    {
*     ... normal response ...
*    }
*   ]
*  }
*  
*  
*  Server inbound data formats
*  
*  The format of data sent to the server is determined by the OperationBinding.dataProtocol
*  specified for the operation. Request data is sent as parameters if the format is 
*  specified as "getParams" or "postParams".
*  
*  In this case, the parameters sent to the server will consist of the DSRequest's data, and any
*  parameters explicitly specified on the DSRequest object (as RPCRequest.params. 
*  If RestDataSource.sendMetaData is true, the DSRequest meta 
*  data properties will also be present as parameters, prefixed with 
*  RestDataSource.metaDataPrefix.
*  
*  Example URL constructed with the metaDataPrefix set to "_" (the default):
*  
*  
*  &nbsp;&nbsp;&nbsp;[dataURL]?field1=value1&_operationType=fetch&_startRow=0&_endRow=50&_sortBy=-field2&_dataSource=dsName
*  
*  
*  In this case the server would be able to separate the request's data from the meta data 
*  via the "_" prefix.
*  
*  If data is sent to the server via the "postMessage" dataProtocol, the data will
*  be serialized as an XML or JSON message according to the dataFormat setting.
*  Both XML and JSON messages will contain request metadata such as startRow and endRow, and
*  will appear exactly as though the subset of the DSRequest that is meaningful to the
*  server had been passed to DataSource.xmlSerialize or JSON.encode
*  respectively.
*  
*  An example of an XML message might look like this:
*  
*   &lt;request&gt;
*     &lt;data&gt;
*       &lt;countryCode&gt;US&lt;/countryCode&gt;
*       &lt;countryName&gt;Edited Value&lt;/countryName&gt;
*       &lt;capital&gt;Edited Value&lt;/capital&gt;
*       &lt;continent&gt;Edited Value&lt;/continent&gt;
*     &lt;/data&gt;
*     &lt;dataSource&gt;countryDS&lt;/dataSource&gt;
*     &lt;operationType&gt;update&lt;/operationType&gt;
*   &lt/request&gt;
*  
*  An example of an XML message for a fetch operation passing simple criteria:
*  
*   &lt;request&gt;
*     &lt;data&gt;
*       &lt;continent&gt;North America&lt;/continent&gt;
*     &lt;/data&gt;
*     &lt;dataSource&gt;countryDS&lt;/dataSource&gt;
*     &lt;operationType&gt;fetch&lt;/operationType&gt;
*     &lt;startRow&gt;0&lt;/startRow&gt;
*     &lt;endRow&gt;75&lt;/endRow&gt;
*     &lt;componentId&gt;worldGrid&lt;/componentId&gt;
*     &lt;textMatchStyle&gt;exact&lt;/textMatchStyle&gt;
*   &lt/request&gt;
*  
*  And an example of an XML message for a fetch operation passing AdvancedCriteria:
*  
*   &lt;request&gt;
*     &lt;data&gt;
*       &lt;_constructor&gt;AdvancedCriteria&lt;/_constructor&gt;
*       &lt;operator&gt;or&lt;/operator&gt;
*       &lt;criteria&gt;
*         &lt;criterion&gt;
*           &lt;fieldName&gt;continent&lt;/fieldName&gt;
*           &lt;operator&gt;equals&lt;/operator&gt;
*           &lt;value&gt;North America&lt;/value&gt;
*         &lt;/criterion&gt;
*         &lt;criterion&gt;
*           &lt;operator&gt;and&lt;/operator&gt;
*           &lt;criteria&gt;
*             &lt;criterion&gt;
*               &lt;fieldName&gt;continent&lt;/fieldName&gt;
*               &lt;operator&gt;equals&lt;/operator&gt;
*               &lt;value&gt;Europe&lt;/value&gt;
*             &lt;/criterion&gt;
*             &lt;criterion&gt;
*               &lt;fieldName&gt;population&lt;/fieldName&gt;
*               &lt;operator&gt;greaterThan&lt;/operator&gt;
*               &lt;value&gt;50000000&lt;/value&gt;
*             &lt;/criterion&gt;
*           &lt;/criteria&gt;
*         &lt;/criterion&gt;
*       &lt;/criteria&gt;
*     &lt;/data&gt;
*     &lt;dataSource&gt;countryDS&lt;/dataSource&gt;
*     &lt;operationType&gt;fetch&lt;/operationType&gt;
*     &lt;startRow&gt;0&lt;/startRow&gt;
*     &lt;endRow&gt;75&lt;/endRow&gt;
*     &lt;componentId&gt;worldGrid&lt;/componentId&gt;
*   &lt/request&gt;
*  
*  An example of an XML message for a fetch operation when using serverSummaries:
*  
*   &lt;request&gt;
*     &lt;data&gt;&lt;/data&gt;
*     &lt;dataSource&gt;countryDS&lt;/dataSource&gt;
*     &lt;operationType&gt;fetch&lt;/operationType&gt;
*     &lt;summaryFunctions&gt;
*       &lt;pk&gt;count&lt;/pk&gt;
*     &lt;/summaryFunctions&gt;
*     &lt;groupBy&gt;member_g8&lt;/groupBy&gt;
*   &lt/request&gt;
*  
*  JSON messages are just the plain JSON form of the structures shown in the above XML
*  examples. The advanced criteria XML example above but in JSON form:
*  
*  {
*    data: {
*      _constructor: "AdvancedCriteria",
*      operator: "or",
*      criteria: [
*        {
*          fieldName: "continent",
*          operator: "equals",
*          value: "North America
*        },
*        {
*          operator: "and", criteria: [
*            {
*              fieldName: "continent",
*              operator: "equals",
*              value: "Europe"
*            },
*            {
*              fieldName: "population",
*              operator: "greaterThan",
*              value: 50000000
*            }
*          ]
*        }
*      ]
*    }
*    dataSource: "countryDS",
*    operationType: "fetch",
*    startRow: 0,
*    endRow: 75,
*    componentId: "worldGrid"
*  }
*  
*  The RestDataSource.operationBindings for a RestDataSource
*  specify dataProtocol as "getParams" for the fetch operation, and "postParams" for update,
*  add and remove operations. Note that most webservers impose a limit on the maximum size 
*  of GET requests (specifically, on the size of the request URL + HTTP headers). Using
*  dataProtocol:"getParams" for "fetch" operations that involve complex AdvancedCriteria
*  will result in a JSON serialization of the AdvancedCriteria in the request URL, and when
*  combined with large cookies this can easily overflow the default limits on certain
*  webservers (see
*  http://stackoverflow.com/questions/686217/maximum-on-http-header-values).
*  For this reason, we recommend that you use the "postMessage" protocol whenever you are
*  intending to use AdvancedCriteria with RestDataSource.
* 
*  
*  Date, time and datetime values
*  
*  Date, time and datetime values must be communicated using XML Schema format, as in the 
*  following examples:
*  
*  &nbsp;&nbsp;&lt;dateField&gt;2007-04-22&lt;/dateField&gt;
*  &nbsp;&nbsp;&lt;timeField&gt;11:07:13&lt;/timeField&gt;
*  &nbsp;&nbsp;&lt;dateTimeField&gt;2007-04-22T11:07:13&lt;/dateTimeField&gt;
*  &nbsp;&nbsp;&lt;dateTimeField&gt;2007-04-22T11:07:13.582&lt;/dateTimeField&gt;
*  
*  
*  And the equivalent in JSON:
*  
*  &nbsp;&nbsp;dateField: "2007-04-22"
*  &nbsp;&nbsp;timeField: "11:07:13"
*  &nbsp;&nbsp;dateTimeField: "2007-04-22T11:07:13"
*  &nbsp;&nbsp;dateTimeField: "2007-04-22T11:07:13.582"
*  
*  
*  Both RestDataSource on the client-side and the RESTHandler servlet on the server side 
*  automatically handle encoding and decoding temporal values using these formats. Both also
*  handle datetime formats including or excluding milliseconds automatically. When encoding,
*  both honor the DataSource.trimMilliseconds setting on the DataSource, falling back
*  to the server.properties setting rest.trimMilliseconds; when
*  decoding, both detect whether or not to try to parse milliseconds based on the string they 
*  receive.
*  
*  Fields of type "date" and "time" are considered to hold logical date and time values, as 
*  discussed in the dateFormatAndStorage, and are 
*  not affected by timezones. Fields of type "datetime" will be converted to UTC on the 
*  client side by RestDataSource, and will be sent back down to the client as UTC by the 
*  server-side RESTHandler. We recommend that your own REST client and/or server code do the
*  same thing (ie, transmit all datetime values in both directions as UTC). Note that the 
*  examples given above give no timezone information, and will be treated by the SmartClient
*  Server as UTC values. If you wish to work with datetime values in a particular timezone,
*  use a format like this:
*  
*  &nbsp;&nbsp;&lt;dateField&gt;2007-04-22T11:07:13-0800&lt;/dateField&gt;
*  &nbsp;&nbsp;&lt;dateField&gt;2012-11-19T22:12:04+0100&lt;/dateField&gt;
*  
*  
*  And the equivalent in JSON:
*  
*  &nbsp;&nbsp;dateTimeField: "2007-04-22T11:07:13-0800"
*  &nbsp;&nbsp;dateTimeField: "2012-11-19T22:12:04+0100"
*  
*  
*  NOTE: Although we refer above to XML Schema format, the format used for specifying
*  timezone offset is slightly different from XML Schema - as shown in the above examples, you
*  specify "+HHMM" or "-HHMM", as opposed to the XML Schema format which requires a ":" character
*  between the hours and minutes. The reason for this difference is simply that the Java 
*  SimpleDateFormat class imposes it.
*  
*  RestDataSource queuing support
*  
*  RestDataSource supports RPCManager.startQueue of DSRequests. This allows 
*  you to send multiple requests to the server in a single HTTP turnaround, thus minimizing 
*  network traffic and allowing the server to treat multiple requests as a single transaction,
*  if the server is able to do so (in Power Edition and above, the SmartClient Server
*  transparently supports grouping multiple REST requests in a queue into a single database
*  transaction when using one of the built-in DataSource types). Note that you can disable 
*  queuing support with the RestDataSource.disableQueuing flag.
*  
*  If you want to use queuing with RestDataSource, you must use the "postMessage" dataProtocol
*  with either XML or JSON dataFormat. Message format is similar to the non-queued examples 
*  shown earlier: it is simply extended to cope with the idea of multiple DSRequests 
*  encapsulated in the message.
*  
*  An example of the XML message sent from RestDataSource to the server for two update requests
*  combined into a queue, using XML dataFormat:
*  
*  &lt;transaction&gt;
*    &lt;operations&gt;
*      &lt;request&gt;
*        &lt;data&gt;
*          &lt;pk&gt;1&lt;/pk&gt;
*          &lt;countryName&gt;Edited Value&lt;/countryName&gt;
*          &lt;capital&gt;Edited Value&lt;/capital&gt;
*          &lt;continent&gt;Edited Value&lt;/continent&gt;
*        &lt;/data&gt;
*        &lt;dataSource&gt;countryDS&lt;/dataSource&gt;
*        &lt;operationType&gt;update&lt;/operationType&gt;
*      &lt/request&gt;
*      &lt;request&gt;
*        &lt;data&gt;
*          &lt;pk&gt;2&lt;/pk&gt;
*          &lt;capital&gt;Edited Value&lt;/capital&gt;
*          &lt;population&gt;123456&lt;/population&gt;
*        &lt;/data&gt;
*        &lt;dataSource&gt;countryDS&lt;/dataSource&gt;
*        &lt;operationType&gt;update&lt;/operationType&gt;
*      &lt/request&gt;
*    &lt;/operations&gt;
*  &lt;transaction&gt;
*  
*  And the same message in JSON format:
*  
*  { 
*    transaction: { 
*      operations: [{
*        dataSource:"countryDS", 
*        operationType:"update", 
*        data: {
*          pk: 1,
*          countryName: "Edited Value",
*          capital: "Edited Value",
*          continent: "Edited Value"
*        }
*      }, {
*        dataSource:"countryDS", 
*        operationType:"update", 
*        data: {
*          pk: 2,
*          capital: "Edited Value",
*          population: 123456
*        }
*      }]
*    }
*  }
*  
*  RestDataSource expects the response to a queue of requests to be a queue of responses in 
*  the same order as the original requests. Again, the message format is very similar to the 
*  unqueued REST format, it just has an outer container construct. Note also that the 
*  individual DSResponses in a queued response have an extra property, 
*  DSResponse.queueStatus. This allows each individual
*  response to determine whether the queue as a whole succeeded. For example, if the first
*  update succeeded but the second failed validation, the first response would have a
*  status of 0, but a queueStatus of -1, while the second response
*  would have both properties set to -1.
*  
*  The update queue example given above would expect a response like this (in XML):
*  
*  &lt;responses&gt;
*    &lt;response&gt;
*      &lt;status&gt;0&lt;/status&gt;
*      &lt;queueStatus&gt;0&lt;/queueStatus&gt;
*      &lt;data&gt;
*        &lt;record&gt;
*          &lt;countryName&gt;Edited Value&lt;/countryName&gt;
*          &lt;gdp&gt;1700.0&lt;/gdp&gt;
*          &lt;continent&gt;Edited Value&lt;/continent&gt;
*          &lt;capital&gt;Edited Value&lt;/capital&gt;
*          &lt;pk&gt;1&lt;/pk&gt;
*        &lt;/record&gt;
*      &lt;/data&gt;
*    &lt;/response&gt;
*    &lt;response&gt;
*      &lt;status&gt;0&lt;/status&gt;
*      &lt;queueStatus&gt;0&lt;/queueStatus&gt;
*      &lt;data&gt;
*        &lt;record&gt;
*          &lt;countryName&gt;United States&lt;/countryName&gt;
*          &lt;gdp&gt;7247700.0&lt;/gdp&gt;
*          &lt;continent&gt;North America&lt;/continent&gt;
*          &lt;independence&gt;1776-07-04&lt;/independence&gt;
*          &lt;capital&gt;Washington DC&lt;/capital&gt;
*          &lt;pk&gt;2&lt;/pk&gt;
*          &lt;population&gt;123456&lt;/population&gt;
*        &lt;/record&gt;
*      &lt;/data&gt;
*    &lt;/response&gt;
*  &lt;/responses&gt;
*  
*  And in JSON:
*  
*  [
*  {
*    "response": {
*      "queueStatus": 0,
*      "status": 0, 
*      "data": [{
*        "countryName": "Edited Value",
*        "gdp": 1700.0,
*        "continent": "Edited Value",
*        "capital": "Edited Value",
*        "pk": 1
*      }]
*    }
*  },
*  {
*    "response": {
*      "queueStatus": 0,
*      "status": 0,
*      "data": [{
*        "countryName": "United States",
*        "gdp": 7247700.0,
*        "continent": "North America",
*        "independence": "1776-07-04",
*        "capital": "Washington DC",
*        "pk": 2,
*        "population": 123456
*      }]
*    }
*  }
*  ]
*  
*  Hierarchical (Tree) data:
*  
*  To create a hierarchical DataSource, in the DataSource's fields array, a field 
*  must be specified as the parent id field - the field which will contain a pointer to the
*  id of each node's parent. 
*  This can be achieved by setting the DataSourceField.foreignKey and the 
*  DataSourceField.rootValue attributes on the field definition. For example:
*  
*  RestDataSource.create({
*   ID:"supplyItem",
*   fields : [
*     {name:"itemId", type:"sequence", primaryKey:true},
*     {name:"parentId", type:"integer", foreignKey:"supplyItem.itemId", rootValue:0},
*     ...
*   ]
*  });
*  
*  Tree Data is then treated on the server as a flat list of records linked by parent id.
*  
*  Tree data is typically displayed using a dataBound TreeGrid component. TreeGrids
*  automatically create a ResultTree data object, which requests data directly
*  from the DataSource. ResultTrees load data on demand, only requesting currently visible 
*  (open) nodes from the server. This is handled by including a specified value for the parent 
*  id field in the request criteria.
*  To implement a standard load-on-demand tree RestDataSource back end, you should therefore 
*  simply return the set of nodes that match the criteria passed in. 
*  For example, if your DataSource was defined as the "supplyItem" code snippet above, 
*  a fetch request for all children of a node with itemId set to 12 
*  would have "parentId" set to 12 in the request criteria.
*  A valid response would then contain all the records that matched this criteria. For example:
*  
*  &lt;response&gt;
*   &lt;status&gt;0&lt;/status&gt;
*   &lt;data&gt;
*    &lt;record&gt;
*      &lt;itemId&gt;15&lt;/itemId&gt;
*      &lt;parentId&gt;12&lt;/parentId&gt;
*    &lt;/record&gt;
*    &lt;record&gt;
*      &lt;itemId&gt;16&lt;/itemId&gt;
*      &lt;parentId&gt;12&lt;/parentId&gt;
*    &lt;/record&gt;
*   &lt;/data&gt;
*  &lt;/response&gt;
*  
*  The structure of responses for Add, Update and Delete type requests will be the 
*  same regardless of whether the data is hierarchical. However you should be aware that 
*  the underlying data storage may need to be managed slightly differently in some cases.
*  
*  Specifically, Add and Update operations may change the structure of the tree by returning a 
*  new parent id field value for the modified node. Depending on how your data is stored you 
*  may need to include special back-end logic to handle this.
*  
*  Also, if a user deletes a folder within a databound tree, any children of that folder will 
*  also be dropped from the tree, and can be removed from the back-end data storage.
*  
*  Note: For a general overview of binding components to Tree structured data, see 
*  treeDataBinding.
*/
interface RestDataSource extends DataSource {
/**
* Rather than setting DataSource.dataProtocol, to control the format in which 
*  inputs are sent to the dataURL, you must specify a replacement OperationBinding 
*  and specify OperationBinding.dataProtocol on that operationBinding.
*  
*  This is because RestDataSource specifies default
*  operationBindings for all operationTypes - see
*  RestDataSource.operationBindings.
* @type {DSProtocol}
* @default null
*/
dataProtocol?: DSProtocol;

/**
* Expected format for server responses. RestDataSources handle "json" and
*  "xml" format responses by default. See class overview documentation for 
*  examples of responses in each format.
* @type {DSDataFormat}
* @default "xml"
*/
dataFormat?: DSDataFormat;

/**
* recordXPath mapping to the data node of XML returned by the server.
*  Applies if this.dataFormat is set to "xml".
*  The default value will pick up data from a response structured as follows:
*  
*  &lt;response&gt;
*   &lt;status&gt;0&lt;/status&gt;
*   &lt;data&gt;
*    &lt;record&gt;
*      &lt;field1&gt;value&lt;/field1&gt;
*      &lt;field2&gt;value&lt;/field2&gt;
*    &lt;/record&gt;
*    &lt;record&gt;
*      &lt;field1&gt;value&lt;/field1&gt;
*      &lt;field2&gt;value&lt;/field2&gt;
*    &lt;/record&gt;
*   &lt;/data&gt;
*  &lt;/response&gt;
*  
* @type {string}
* @default "/response/data/*"
*/
xmlRecordXPath?: string;

/**
* When RestDataSource.dataFormat is "xml", xmlNamespaces configures the set of
*  namespace prefixes that are added to the document element of the XML message sent to the
*  server. Format is the same as DataSource.xmlNamespaces.
*  
*  By default, the "xsi" prefix is bound to "http://www.w3.org/2001/XMLSchema-instance" in
*  order to allow explicit null values in Records to be sent for
*  DataSourceField.nillable. Set to null to avoid any
*  prefixes being added.
* @type {object}
* @default "See below"
*/
xmlNamespaces?: object;

/**
* recordXPath mapping to the data node of json returned by the server.
*  Applies if this.dataFormat is set to "json"
*  The default value will pick up data from a response structured as follows:
*  
*  {response:
*  {status:0,
*   data:[
*    {field1:"value", field2:"value"},
*    {field1:"value", field2:"value"}
*   ]
*  }
*  
* @type {string}
* @default "/response/data"
*/
jsonRecordXPath?: string;

/**
* For RestDataSources, by default, either the RestDataSource.xmlRecordXPath or 
*  RestDataSource.jsonRecordXPath is used based on the RestDataSource.dataFormat
*  setting.
*  
*  Note that you can also apply record xpath binding via
*  OperationBinding.recordXPath.
* @type {string}
* @default null
*/
recordXPath?: string;

/**
* When using dataFormat:"json" and dataProtocol:"postMessage" should we use the
*  JSONEncoder.prettyPrint feature to enable indented, highly readable JSON messages.
*  
*  True by default because the bandwidth involved is generally negligible and the benefits for
*  troubleshooting are key.
* @type {boolean}
* @default true
*/
prettyPrintJSON?: boolean;

/**
* If set, disables RPCManager.startQueue for this RestDataSource.
* @type {boolean}
* @default false
*/
disableQueuing?: boolean;

/**
* Allows you to specify an arbitrary prefix string to apply to all json format responses 
*  sent from the server to this application. The client will expect to find this prefix 
*  on any JSON response received for this DataSource, and will strip it off before evaluating
*  the response text.
*  
*  The default prefix is "&lt;SCRIPT&gt;//'\"]]&gt;&gt;isc_JSONResponseStart&gt;&gt;".
*  
*  The inclusion of such a prefix ensures your code is not directly executable outside of 
*  your application, as a preventative measure against 
*  javascript hijacking.
*  
*  You can switch off JSON wrapping altogether by setting both this and RestDataSource.jsonSuffix
*  to empty strings. 
*  
*  If you are using SmartClient Server's RESTHandler servlet, see the server-side Javadocs
*  for details of how to change the way JSON wrapping works on the server side.
* @type {string}
* @default "See below"
*/
jsonPrefix?: string;

/**
* Allows you to specify an arbitrary suffix string to apply to all json format responses 
*  sent from the server to this application. The client will expect to find this suffix 
*  on any JSON response received for this DataSource, and will strip it off before evaluating
*  the response text.
*  
*  The default suffix is "//isc_JSONResponseEnd".
* @type {string}
* @default "See below"
*/
jsonSuffix?: string;

/**
* RestDataSource OperationBindings set to specify default dataProtocol per operationType.
*  Default databindings are:
*  
*   operationBindings : [
*    {operationType:"fetch", dataProtocol:"getParams"},
*    {operationType:"add", dataProtocol:"postParams"},
*    {operationType:"remove", dataProtocol:"postParams"},
*    {operationType:"update", dataProtocol:"postParams"} 
*   ],
*  
*  If you are integrating with a RestDataSource server that requires the more
*  obscure RPCRequest.httpMethods of "PUT", "DELETE" or "HEAD", you can specify these
*  httpMethod settings via OperationBinding.requestProperties. dataProtocol settings
*  that mention "GET" or "POST" are compatible with these additional HTTP methods as well.
*  Typical DataSource.operationBindings for a REST server that uses
*  "PUT" and "DELETE" are as follows:
*  
*   operationBindings:[
*    {operationType:"fetch", dataProtocol:"getParams"},
*    {operationType:"add", dataProtocol:"postParams"},
*    {operationType:"remove", dataProtocol:"getParams", requestProperties:{httpMethod:"DELETE"}},
*    {operationType:"update", dataProtocol:"postParams", requestProperties:{httpMethod:"PUT"}}
*   ],
*  
*  
*  Note that dataProtocol:"postMessage" is always used when
*  RPCManager.startQueue is used to send multiple DSRequests to the server
*  as a single HttpRequest. See RestDataSource docs, "queuing support". We also 
*  recommend that you use the "postMessage" protocol whenever you are intending to use 
*  AdvancedCriteria with RestDataSource - this is discussed in the section "Server inbound
*  data format" in the RestDataSource.
*  
* @type {Array<Partial<OperationBinding>>}
* @default "[...]"
*/
operationBindings?: Array<Partial<OperationBinding>>;

/**
* Default URL to contact to fulfill all DSRequests. 
*  RestDataSources also allow per-operationType dataURLs to be set via
*  
*  
* - RestDataSource.fetchDataURL
*  
* - RestDataSource.addDataURL 
*  
* - RestDataSource.updateDataURL
*  
* - RestDataSource.removeDataURL
*  
*  NOTE:: when using RPCManager.startQueue with RestDataSource, an
*  HTTP request containing mixed DSRequest.operationType (such as a
*  mixture of "add", "update" and "remove" operations resulting from
*  ListGrid.autoSaveEdits) can only go to one URL, so you
*  should not set distinct URLs for each operationType; doing so will break
*  queuing of mixed operationTypes: multiple requests will be sent to distinct URLs, and a
*  warning logged.
* @type {string}
* @default null
*/
dataURL?: string;

/**
* Custom DataSource.dataURL for DSRequest with
*  DSRequest.operationType "fetch".
*  
*  Use RestDataSource.dataURL to configure a single URL for all requests, which is
*  required to support RPCManager.startQueue.
* @type {string}
* @default null
*/
fetchDataURL?: string;

/**
* Custom DataSource.dataURL for DSRequest with
*  DSRequest.operationType "update".
*  
*  See RestDataSource.dataURL to configure a single URL for all requests, which is
*  required to support RPCManager.startQueue.
* @type {string}
* @default null
*/
updateDataURL?: string;

/**
* Custom DataSource.dataURL for DSRequest with
*  DSRequest.operationType "add".
*  
*  See RestDataSource.dataURL to configure a single URL for all requests, which is
*  required to support RPCManager.startQueue.
* @type {string}
* @default null
*/
addDataURL?: string;

/**
* Custom DataSource.dataURL for DSRequest with
*  DSRequest.operationType "remove".
*  
*  See RestDataSource.dataURL to configure a single URL for all requests, which is
*  required to support RPCManager.startQueue.
* @type {string}
* @default null
*/
removeDataURL?: string;

/**
* Should operation meta data be included when assembling parameters to send 
*  to the server? If true, meta data parameters will be prefixed with the 
*  RestDataSource.metaDataPrefix.
*  Applies to operations where OperationBinding.dataProtocol is set to 
*  "getParams" or "postParams" only.
*  
*  See RestDataSource.transformRequest to review the list of default metadata fields. In case
*  you need to add additional parameters to the dsRequest object, you can do it via
*  RPCRequest.params.
* @type {boolean}
* @default true
*/
sendMetaData?: boolean;

/**
* If RestDataSource.sendMetaData is true, this attribute is used to specify
*  the prefix to apply to 'meta data' properties when assembling parameters to send to the 
*  server. Applies to operations where OperationBinding.dataProtocol is set to 
*  "getParams" or "postParams" only.
* @type {string}
* @default "_"
*/
metaDataPrefix?: string;



/**
*  RestDataSource overrides transformRequest and handles serializing the request in the
*  appropriate format (determined by the specified
*  OperationBinding.dataProtocol), including the submitted
*  DSRequest.data as well as the meta data parameters, which may include - 
*  DSRequest.dataSource,
*  DSRequest.operationType, DSRequest.operationId;
*  DSRequest.startRow and DSRequest.endRow (for fetches);
*  DSRequest.sortBy and DSRequest.textMatchStyle 
*  (for fetches);
*  DSRequest.oldValues (for update and remove operations);
*  and possibly DSRequest.componentId.
*  
*  If you override this method in order to add additional data to the DSRequest, you must
*  call Class.Super or you will remove the functionality provided by
*  RestDataSource. For example:
*  
*   transformRequest : function (dsRequest) {
*     // modify dsRequest.data here, for example, add fixed criteria
*     dsRequest.data.userId = myApplication.getCurrentUserId();
*  
*     return this.Super("transformRequest", arguments);
*   }
*  
*  
*  See RestDataSource for a description of the 
*  standard formatting applied to requests.
* 
* 
*/
transformRequest(): void;

/**
*  RestDataSource implements transformResponse in order to extract data and meta-data
*  properties from server responses, as described in the 
*  RestDataSource.
*  
*  You can override transformResponse() in order to further modify the
*  response, but if you do so, call Class.Super as shown below or you will
*  wipe out the built-in response processing behavior of RestDataSource.
*  
*  transformResponse : function (dsResponse, dsRequest, data) {    
*    var dsResponse = this.Super("transformResponse", arguments);
*    // ... do something to dsResponse ...
*    return dsResponse;
*  }
*  
* 
* @param {DSResponse} default DSResponse derived from the response data
* @param {DSRequest} DSRequest object that initiated this request
* @param {XMLDocument | JSON} XML document or JSON objects returned by the web                  service
*/
transformResponse(dsResponse:DSResponse, dsRequest:DSRequest, data:XMLDocument | JSON): DSResponse;



}
/**
* Creates a new RestDataSource
* 
* @param typeCheckedProperties {Partial<RestDataSource>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RestDataSource} The new instance
*/
class RestDataSource{
static create(typeCheckedProperties? : Partial<RestDataSource>, uncheckedProperties?: Object): RestDataSource;







}

/**
* ResultSets are an implementation of the List interface that automatically fetches 
*  DataSource records when items are requested from the List. ResultSets provide robust,
*  customizable, high-performance cache management for ListGrids and other built-in SmartClient
*  components, and can be used as cache managers by custom components.
*  
*  ResultSets manage data paging, that is, loading records in batches as the user navigates
*  the data set. A ResultSet will switch to using client-side sorting and filtering when
*  possible to improve responsiveness and reduce server load. ResultSets also participate in
*  automatic cache synchronization, observing operations on DataSources and automatically
*  updating their caches.
*  
*  Creation
*  
*  A ResultSet can be passed to any component that expects a List, and the List APIs can be
*  called directly on the ResultSet as long as the caller is able to deal with asynchronous
*  loading; see ResultSet.getRange.
*  
*  Generally ResultSets do not need to be created directly, but are created by DataBound
*  components as an automatic consequence of calling 
*  dataBoundComponentMethods. 
*  For example, the ListGrid.fetchData causes ListGrid.data to become an
*  automatically created ResultSet object. Automatically created ResultSets
*  can be customized via properties on ListGrids such as ListGrid.dataPageSize and
*  ListGrid.dataProperties. All ResultSets for a given DataSource may also be 
*  customized via setting DataSource.resultSetClass to the name of a ResultSet 
*  isc.defineClass in which 
*  Class.addProperties.
*  
*  A ResultSet defaults to using data paging, setting DSRequest.startRow and
*  DSRequest.endRow in issued dsRequests. Server code may always return more rows than
*  the ResultSet requests and the ResultSet will correctly integrate those rows based on
*  DSResponse.startRow/DSResponse.endRow. 
*  Hence the server can always avoid paging mode by simply returning all matching rows.
*  
*  A ResultSet can be created directly with just the ID of a DataSource:
*  
*  
*    isc.ResultSet.create({
*      dataSource : "dataSourceID"
*    })
*  
*  
*  
*  
*  Directly created ResultSets are typically used by custom components, or as a means of
*  managing datasets that will be used by several components.
*  
*  When created directly rather than via a dataBoundComponent, a newly created ResultSet will
*  not issue it's first "fetch" DSRequest until data is accessed (for example, via
*  ResultSet.get). 
*  
*  Paging and total dataset length
*  
*  When using data paging, the server communicates the total number of records that match the
*  current search criteria by setting DSResponse.totalRows. The ResultSet will then
*  return this number from ResultSet.getLength, and ListGrids and other
*  components will show a scrollbar that allows the user to jump to the end of the dataset
*  directly.
*  
*  However, the ResultSet does not require that the server calculate the true length of the
*  dataset, which can be costly for an extremely large, searchable dataset. Instead, the
*  server may advertise a totalRows value that is one page larger
*  than the last row loaded. This results in a UI sometimes called "progressive loading",
*  where the user may load more rows by scrolling past the end of the currently loaded rows,
*  but is not allowed to skip to the end of the dataset.
*  
*  The SmartClient server offers built in support for progressive loading at the 
*  DataSource.progressiveLoading,
*  OperationBinding.progressiveLoading and
*  DSRequest.progressiveLoading level for SQL-backed dataSources.
*  Setting ResultSet.progressiveLoading or DataBoundComponent.progressiveLoading to
*  true will also enable this feature where available.
*  
*  Where available, progressive loading will also be enabled automatically
*  for very large data sets if a
*  DataSource.progressiveLoadingThreshold is specified.
*  
*  Note that the SmartClient server is not a requirement for progressive loading -
*  any DataSource implementation can enable progressive loading by simply populating 
*  the DSResponse.totalRows property with an appropriate value. 
*  We recommend the DSResponse.progressiveLoading attribute be set to true
*  as well - this allows client side logic to treat the reported totalRows 
*  value specially if necessary.
*  
*  No client-side settings are required to enable this mode - it is entirely server-driven.
*  However, it is usually coupled with ListGrid.canSort, since
*  server-side sorting would also force the server to traverse the entire dataset.
*  
*  Client-side Sorting and Filtering
*  
*  If a ResultSet obtains a full cache for the current set of filter criteria, it will 
*  automatically switch to client-side sorting, and will also use client-side filtering 
*  if the filter criteria are later changed but appear to be more restrictive than the
*  criteria in use when the ResultSet obtained a full cache.
*  
*  The ResultSet.useClientSorting and 
*  ResultSet.useClientFiltering flags can be used to disable
*  client-side sorting and filtering respectively if these behaviors don't match server-based
*  sorting and filtering. However, because client-side sorting and filtering radically improve
*  responsiveness and reduce server load, it is better to customize the ResultSet so that it
*  can match server-side sorting and filtering behaviors.
*  
*  Sorting behavior is primarily customized via the "sort normalizer" passed to
*  ResultSet.sortByProperty, either via direct calls on a standalone ResultSet, or via
*  ListGridField.sortNormalizer for a ListGrid-managed ResultSet.
*  
*  By default, client-side filtering interprets the Criteria passed to
*  ResultSet.setCriteria as a set of field values that records must match
*  (similarly to the built-in SQL/Hibernate connectors built into the SmartClient Server).
*  Custom client-side filtering logic can be implemented by overriding
*  ResultSet.applyFilter. Overriding
*  ResultSet.compareCriteria allows you to control when the ResultSet
*  uses client-side vs server-side filtering, and the ResultSet has two default 
*  ResultSet.criteriaPolicy built-in.
*  
*  Modifying ResultSets
*  
*  Records cannot be directly added or removed from a ResultSet via List
*  APIs such as List.removeAt, unless it always filters locally, since
*  this would break the consistency of server and client row numbering needed for data paging,
*  and also create some issues with automatic cache synchronization. Set 
*  ResultSet.modifiable to enable the List modification APIs on a
*  ResultSet.fetchMode:"local" ResultSet. Note that the special 
*  FilteredList class sets this property to allow developers to modify its data.
*  
*  Use DataSource.addData/DataSource.removeData to add/remove
*  rows from the DataSource, and the ResultSet will reflect the changes automatically.
*  Alternatively, the DataSource.updateCaches method may be called to only update
*  local caches of the DataSource in question, without generating any server traffic.
*  
*  To create a locally modifiable cache of Records from a DataSource, you
*  can use DataSource.fetchData to retrieve a List of Records which can
*  be modified directly, or you can create a client-only DataSource from
*  the retrieved data to share a modifiable cache between several
*  DataBoundComponents.
*  
*  Updates and Automatic Cache Synchronization
*  
*  Once a ResultSet has retrieved data or has been initialized with data, the ResultSet will observe any
*  successful "update", "add" or "remove" dsRequests against their DataSource, regardless of the
*  component that initiated them. A ResultSet with a full cache for the current filter criteria will
*  integrate updates into the cache automatically.
*  
*  Updated rows that no longer match the current filter criteria will be removed
*  automatically. To prevent this, you can set ResultSet.neverDropUpdatedRows.
*  Added rows will similarly be added to the cache only if they match current filter criteria.
*  
*  Note that the client-side filtering described above is also used to determine whether 
*  updated or added rows should be in the cache. If any aspect of automated cache update is
*  ever incorrect, ResultSet.dropCacheOnUpdate can be set for the
*  ResultSet or DSResponse.invalidateCache can be set for an individual dsResponse.
*  
*  If automatic cache synchronization isn't working, troubleshoot the problem using the steps
*  suggested in the FAQ.
*  
*  Regarding OperationBinding.operationId and how they affect caching,
*  take into account that cache sync is based on the fetch used - any add or update operation
*  uses a fetch to retrieve updated data, and the operationId of that fetch can be set via
*  OperationBinding.cacheSyncOperation.
*  If the operationId of the cache is different from the operationId of the cache update data,
*  it won't be used to update the cache, since the fields included and other aspects of the
*  data are allowed to be different across different operationIds. This allows to maintain
*  distinct caches on a per component basis, so when two components are using separate
*  operationIds they are assumed to have distinct caches, because updates performed with
*  one operationId will not affect the cache obtained via another operationId.
*  Also, take into account that operationId must be unique per DataSource, across all
*  operationTypes for that DataSource.
* 
*  
*  Data Paging with partial cache
*  
*  When in paging mode with a partial cache, a ResultSet relies on server side sorting, setting 
*  DSRequest.sortBy to the current sort field and direction. In order for the cache to 
*  remain coherent, row numbering must continue to agree between server and client as new
*  fetches are issued, otherwise, duplicate rows or missing rows may occur. 
*  
*  If concurrent modifications by other users are allowed, generally the server should set
*  DSResponse.invalidateCache to clear the cache when concurrent modification is
*  detected.
*  
*  In paging mode with a partial cache, any successful "update" or "add" operation may cause
*  client and server row numbering to become out of sync. This happens because the update
*  may affect the sort order, and client and server cannot be guaranteed to match for sets of
*  records that have equivalent values for the sort field.
*  
*  For this reason, after an "add" or "update" operation with a partial cache, the ResultSet
*  will automatically mark cache for invalidation the next time a fetch operation is performed.
*  Alternatively, if ResultSet.updatePartialCache is set to false, the ResultSet will
*  simply invalidate cache immediately in this circumstance.
*/
interface ResultSet extends Class, Omit<List, 'duplicate'>  {
/**
* Mode of fetching records from the server. If unset, will default to "local"
*  if ResultSet.allRows is specified, otherwise "paged".
* @type {FetchMode}
* @default null
*/
fetchMode?: FetchMode;

/**
* When true, allows the ResultSet to be modified by list APIs List.addAt,
*  List.set, and List.removeAt. Only applies to 
*  ResultSet.fetchMode:"local" ResultSets, since in all other cases, such
*  modifications would break the consistency of server and client row numbering needed for
*  data paging, and also create some issues with automatic cache synchronization. See the
*  "Modifying ResultSets" subtopic in the ResultSet for the
*  alternative approach of updating the DataSource.
*  
*  One known case where modification can be useful is when an array has been passed to
*  ListGrid.setData for a ListGrid with ListGrid.filterLocalData:true. If
*  the data is filtered using the ListGrid.showFilterEditor, then a new
*  local ResultSet will be created as ListGrid.data to reflect the filtering.
* @type {boolean}
* @default false
*/
modifiable?: boolean;

/**
* Initial set of data for the ResultSet.
*  
*  This data will be treated exactly as though it were the data returned from the
*  ResultSet's first server fetch.
*  
*  By default, initialData will be considered a complete response
*  (all rows that match the Criteria which the ResultSet was initialized with).
*  
*  Set ResultSet.initialLength to treat initialData as a partial response,
*  equivalent to receiving a DSResponse with startRow:0,
*  endRow:initialData.length and totalRows:initialLength.
*  Normal data paging will then occur if data is requested for row indices not filled via 
*  initialData.
*  
*  initialData may be provided as a "sparse" array, that is, slots may be left
*  null indicating rows that have not been loaded. In this way you can create a ResultSet
*  that is missing rows at the beginning of the dataset, but has loaded rows toward the end,
*  so that you can create a component that is scrolled to a particular position of a dataset
*  without loading rows at the beginning.
*  
*  To keep the logic simple and support partial initialData, the data is
*  assumed to be already sorted and filtered according to the ResultSet.sortSpecifiers and
*  ResultSet.criteria supplied to the ResultSet, since otherwise, for partial
*  initialData, sorting or filtering would immediately cause the data to be
*  discarded.
*  
*  If initialData is complete and needs to be sorted or filtered, then don't
*  pass the ResultSet.sortSpecifiers or ResultSet.criteria, respectively, when creating
*  the ResultSet. Instead, call ResultSet.setCriteria or ResultSet.setSort, respectively, on
*  the instance afterwards.
* @type {Array<Partial<Record>>}
* @default null
*/
initialData?: Array<Partial<Record>>;

/**
* Initial value of the data set length.
*  
*  To create a ResultSet with it's cache partly filled, see ResultSet.initialData.
* @type {Integer}
* @default null
*/
initialLength?: Integer;

/**
* Initial sort specifiers for a ResultSet. Use ResultSet.setSort and 
*  ResultSet.getSort to sort the data after initialization rather than
*  attempting to read or modify this property directly.
*  
*  Note: if ResultSet.initialData was specified, the data is assumed to already
*  be sorted to match this sort configuration.
* @type {Array<Partial<SortSpecifier>>}
* @default null
*/
sortSpecifiers?: Array<Partial<SortSpecifier>>;

/**
* If the complete set of records for a resultSet is available when the resultSet is created,
*  it can be made available to the resultSet via this property at initialization time.
*  This data will then be considered cached meaning sorting and filtering can occur on
*  the client (no need for server fetch).
*  
*  This cached data can be dropped via a call to ResultSet.invalidateCache.
*  
*  See also ResultSet.initialData and ResultSet.initialLength as an alternative approach for
*  initializing a ResultSet with a partial cache, such that data paging will occur as
*  uncached rows are requested.
*  
*  Note that developers wishing to synchronously access a filtered set of client side data
*  may wish to consider creating a FilteredList.
* @type {Array<Partial<Record>>}
* @default null
*/
allRows?: Array<Partial<Record>>;

/**
* How many rows to retrieve at once.
*  
*  Applicable only with fetchMode: "paged". When a paged ResultSet is asked
*  for rows that have not yet been loaded, it will fetch adjacent rows that are likely to
*  be required soon, in batches of this size.
* @type {Integer}
* @default "75"
*/
resultSize?: Integer;

/**
* Delay in milliseconds before fetching rows.
*  
*  When a get() or getRange() call asked for rows that haven't been loaded, the
*  ResultSet will wait before actually triggering the request. If, during the delay, more
*  get() or getRange() calls are made for missing rows, the final fetch to the server will
*  reflect the most recently requested rows.
*  
*  The intent of this delay is to avoid triggering many unnecessary fetches during
*  drag-scrolling and similar user interactions.
* @type {Integer}
* @default "0"
*/
fetchDelay?: Integer;

/**
* What DataSource is this resultSet associated with?
* @type {DataSource}
* @default null
*/
dataSource?: DataSource;

/**
* The DSRequest.operationId this ResultSet should use when performing
*  fetch operations.
*  
*  Note: if this property is not explicitly set and, for ResultSets automatically
*  created by a component, DataBoundComponent.fetchOperation is also unset, a
*  placeholder value of &lt;dataSourceId&gt;_&lt;operationType&gt; may be reported
*  (e.g. "supplyItem_fetch").
* @type {string}
* @default null
*/
fetchOperation?: string;

/**
* Allows to set a DSRequest properties to this ResulSet.
* @type {Partial<DSRequest>}
* @default null
*/
requestProperties?: Partial<DSRequest>;

/**
* Criteria that are never shown to or edited by the user and are cumulative with any 
*  criteria provided via DataBoundComponent.initialCriteria, ResultSet.setCriteria
*  etc.
* @type {Criteria}
* @default null
*/
implicitCriteria?: Criteria;

/**
* Filter criteria used whenever records are retrieved.
*  
*  Use ResultSet.setCriteria to change the criteria after initialization.
* @type {Criteria}
* @default null
*/
criteria?: Criteria;

/**
* Decides under what conditions the cache should be dropped when the Criteria
*  changes.
* @type {CriteriaPolicy}
* @default null
*/
criteriaPolicy?: CriteriaPolicy;

/**
* Whether to sort data locally when all records matching the current criteria have been
*  cached.
*  
*  This may need to be disabled if client-side sort order differs from server-side sort
*  order in a way that affects functionality or is surprising.
* @type {boolean}
* @default true
*/
useClientSorting?: boolean;

/**
* Whether to filter data locally when we have a complete cache of all DataSource records 
*  for the current criteria, and the user further restricts the criteria (see 
*  DataSource.compareCriteria).
*  
*  This may need to be disabled if client-side filtering differs from server-side filtering
*  in a way that affects functionality or is surprising.
*  
*  Note that you can also prevent client-side filtering for certain fields, by setting them
*  to ${isc.DocUtils.linkForRef('attr:DataSourceField.filterOn','filterOn: \'serverOnly\'')}.
*  
*  This setting is distinct from fetchMode:"local", which explicitly loads all
*  available DataSource records up front and always performs all filtering on the client.
*  
*  See ResultSet.applyFilter for default filtering behavior.
*  
*  NOTE: even with useClientFiltering false, client-side filtering will be used
*  during cache sync to determine if an updated or added row matches the current criteria.
*  To avoid relying on client-side filtering in this case, either:
*  - avoid returning update data when the updated row doesn't match the current filter
*  - set dropCacheOnUpdate
* @type {boolean}
* @default true
*/
useClientFiltering?: boolean;

/**
* When a successful Add, Update or Remove type operation fires on this ResultSet's 
*  dataSource, if DSResponse.data is unset, should we integrate the submitted
*  data values (from the request) into our data-set? This attribute will be passed to
*  DataSource.getUpdatedData as the useDataFromRequest parameter.
* @type {boolean}
* @default true
*/
updateCacheFromRequest?: boolean;

/**
* Whether to discard all cached rows when a modification operation (add, update, remove)
*  occurs on the ResultSet's DataSource.
*  
*  A ResultSet that has a complete cache for the current filter criteria can potentially
*  incorporate a newly created or updated row based on the data that the server returns
*  when a modification operation completes. However this is not always possible for
*  ResultSets that show some types of joins, or when the server cannot easily return update
*  data. In this case set dropCacheOnUpdate to cause the cache to be
*  discarded when an update occurs.
*  
*  dropCacheOnUpdate can be set either directly on a ResultSet, or on a
*  DataSource in order to affect all ResultSets on that DataSource.
* @type {boolean}
* @default false
*/
dropCacheOnUpdate?: boolean;

/**
* By default when the data of this ResultSet's dataSource is modified, the ResultSet will
*  be updated to display these changes.
*  Set this flag to true to disable this behavior.
* @type {boolean}
* @default false
*/
disableCacheSync?: boolean;

/**
* Sets DataSource.progressiveLoading for this ResultSet.
*  Any DSRequests issued by this ResultSet will copy this setting onto the request,
*  overriding the OperationBinding- and DataSource-level settings.
*  
*  This setting is applied automatically by DataBoundComponents that have their 
*  own explicit setting for DataBoundComponent.progressiveLoading
* @type {boolean}
* @default null
*/
progressiveLoading?: boolean;

/**
* If ResultSet.progressiveLoading is not explicitly set, but the resultset
*  recieves a response from the server where DSResponse.progressiveLoading is 
*  set to true, should subsequent requests for other rows in the same data set explicitly
*  request progressiveLoading via DSRequest.progressiveLoading, as long as
*  the criteria are unchanged and the cache is not explicitly invalidated?
*  
*  This property is useful for the case where the server side 
*  DataSource.progressiveLoadingThreshold enabled progressive loading after
*  the row-count query determined that the requested data set was very large.
*  By explicitly DSRequest.progressiveLoading
*  for subsequent fetches the server is able to avoid an unnecessary and potentially
*  expensive row-count query while returning other rows from the same data set.
* @type {boolean}
* @default true
*/
rememberDynamicProgressiveLoading?: boolean;

/**
* By default when a row is returned by the server, the current ResultSet.setCriteria are applied to it, and it may disappear from the cache.
*  
*  Set this flag to true to disable this behavior.
* @type {boolean}
* @default false
*/
neverDropUpdatedRows?: boolean;

/**
* If set to true, updated and added rows will be integrated into the client-side cache
*  even if paging is enabled and cache is partial. If updatePartialCache is
*  false, the cache will be invalidated and new data fetched.
*  
*  If updatePartialCache is enabled and an "add" or "update" operation succeeds with a partial
*  cache:
*  
*  
* - updated rows will remain in their current position. No attempt will be made to sort
*  them into a new position even if the sort field was updated.
*  
* - newly added rows will be added at either the end (first preference) or beginning of
*  the dataset if that part of the dataset is cached and was most recently requested.
*  If not, the new row is added at the end of the most recently requested contiguously
*  cached range.
*  
*  The cache will then be dropped the next time rows are fetched, to prevent problems with
*  inconsistent row numbering between the server and client, which could otherwise lead to
*  duplicate rows or rows being skipped entirely.
* @type {boolean}
* @default true
*/
updatePartialCache?: boolean;

/**
* If true, records requested only for visible area.
* @type {boolean}
* @default false
*/
alwaysRequestVisibleRows?: boolean;

/**
* To avoid needless work, the ResultSet by default doesn't refilter the data when methods such
*  as ListGrid.fetchData or ListGrid.filterData are called with unchanged
*  criteria. However, this property can be set true for backward compatibility to force
*  refiltering if we're ResultSet.filterLocalData and the criteria haven't
*  changed. but are narrower than the criteria used to fetch the current cache.
*  
*  Going forward, we may deprecate this property, so you should move to approach that
*  doesn't require such notification in the case of unchanged criteria.
* @type {boolean}
* @default null
*/
reapplyUnchangedLocalFilter?: boolean;



/**
*  Do we have a complete client-side cache of records for the current filter criteria?
*  
*  Returns false if this is a paged data set, and the entire set of records that
*  match the current criteria has not been retrieved from the server. In other words, a
*  return value of false means that this ResultSet has a partial cache.
* 
* 
*/
allMatchingRowsCached(): boolean;

/**
*  Do we have a complete client-side cache of all records for this DataSource?
*  
*  Becomes true only when the ResultSet obtains a complete cache after a fetch with empty
*  criteria.
* 
* 
*/
allRowsCached(): boolean;

/**
*  Get a map of the form { item[idField] -&gt; item[displayField] }, for all 
*  items in the list. If more than one item has the same idProperty, 
*  the value for the later item in the list will clobber the value for the earlier item.
*  
*  If this method is called when the ResultSet.allMatchingRowsCached, it
*  will trigger fetches, and will return a valueMap reflecting only the currently loaded rows.
* 
* 
* @param {string} Property to use as ID (data value) in the valueMap
* @param {string} Property to use as a display value in the valueMap
*/
getValueMap(idField:string, displayField:string): object;

/**
*  Return the total number of records that match the current filter criteria.
*  
*  This length can only be known, even approximately, when the first results are retrieved from
*  the server. Before then, the ResultSet returns a large length in order to encourage viewers
*  to ask for rows. ResultSet.lengthIsKnown can be called to
*  determine whether an actual length is known.
*  
*  Note that if DataSource.progressiveLoading is active, the length
*  advertised by the server may not be an accurate total row count for the data set.
*  ResultSet.lengthIsProgressive
*  can be called to determine whether this is the case.
* 
* 
*/
getLength(): number;

/**
*  Return the position in the list of the first instance of the specified object.
*  
*  If pos is specified, starts looking after that position.
*  
*  Returns -1 if not found.
*  
*  NOTE: ResultSet.indexOf() only inspects the current cache of records, so it is only
*  appropriate for temporary presentation purposes. For example, it would not be appropriate
*  to hold onto a record and attempt to use indexOf() to determine if it had been deleted.
* 
* 
* @param {any} object to look for
* @param {number=} earliest index to consider
* @param {number=} last index to consider
*/
indexOf(obj:any, pos?:number, endPos?:number): number;

/**
*  Returns the record at the specified position.
*  
*  All List access methods of the ResultSet have the semantics described in getRange().
* 
* @param {number} position of the element to get
*/
get(pos:number): object;

/**
*  Return the items between position start and end, non-inclusive at the end, possibly 
*  containing markers for records that haven't loaded yet.
*  
*  Calling getRange for records that have not yet loaded will trigger an asynchronous fetch. The
*  returned data will contain the ResultSet.getLoadingMarker as a placeholder
*  for records being fetched. If any rows needed to be fetched, dataArrived() will
*  fire when they arrive.
* 
* 
* @param {number} start position
* @param {number} end position
*/
getRange(start:number, end:number): Array<any>;

/**
*  Returns all rows that match the current criteria.
*  
*  This method will not trigger a fetch to load more records. getAllVisibleRows()
*  will return null if ResultSet.lengthIsKnown is false.
*  
*  Records are returned in a new List but the Records within it are the same
*  instances that the ResultSet is holding onto. Hence it's safe to add or remove records from
*  the List without affecting the ResultSet but modifying the Records themselves is a direct
*  modification of the client-side cache.
* 
*/
getAllVisibleRows(): Array<Partial<Record>>;

/**
*  Determine whether the ResultSet is showing a filtered, proper subset of the cached rows.
*  This happens if ResultSet.useClientFiltering is enabled. Rows may have been
*  loaded from the server when a more restrictive criteria is applied such that filtering could
*  be performed on the client side.
*  
*  This method returns false if data is not loaded yet.
* 
*/
usingFilteredData(): boolean;

/**
*  Returns a list of all rows that have been cached. This is potentially a superset of all rows
*  that are available via ResultSet.getAllVisibleRows if the ResultSet is using client-side
*  filtering to display a subset of loaded rows (see the ResultSet).
*  
*  If ResultSet.usingFilteredData returns false, this is the same list as would be returned by
*  ResultSet.getAllVisibleRows.
*  
*  This method will not trigger a fetch to load more records. getAllCachedRows() will return
*  null if ResultSet.lengthIsKnown is false.
*  
*  Records are returned in a new List but the Records within it are the same
*  instances that the ResultSet is holding onto. Hence it's safe to add or remove records from
*  the List without affecting the ResultSet but modifying the Records themselves is a direct
*  modification of the client-side cache.
* 
*/
getAllCachedRows(): Array<Partial<Record>>;

/**
*  Returns a clone of this ResultSet sharing the same DataSource, current criteria, sort
*  direction and cache state.
*  
*  Internal caches are copied such that methods such as ResultSet.lengthIsKnown,
*  ResultSet.allMatchingRowsCached and ResultSet.allRowsCached return identical results for the
*  new ResultSet.
*  
*  Subsequent changes to criteria or sort direction on the original ResultSet with not affect
*  the duplicate, and vice versa. Outstanding fetches on the original ResultSet will likewise
*  have no effect on the duplicate when they complete. However, actual Record instances are
*  shared between the original and copied ResultSet, so direct modification of Records will
*  affect both ResultSets.
*  
*  The new ResultSet will listen for DataSource changes and automatically update caches just
*  as normal ResultSets do.
*  
*  NOTE: if you are looking for a simple list of Records rather than a new, fully
*  functional ResultSet, consider ResultSet.getAllCachedRows or ResultSet.getAllVisibleRows. If
*  you are looking to create a filterable, sortable subset of the current ResultSet, consider
*  creating a new ResultSet and passing the results of getAllCachedRows or
*  getAllVisibleRows as the initial value of ResultSet.allRows.
* 
*/
duplicate(): ResultSet;

/**
*  Whether the ResultSet knows the length of its data set.
*  
*  When the ResultSet fetches data from the DataSource in response to ResultSet.getRange or
*  similar methods, DSResponse.totalRows from the fetch
*  lets the ResultSet know the full dataset size.
*  
*  Prior to the completion of the first fetch, (or after ResultSet.invalidateCache)
*  the ResultSet will not know how many records 
*  are available. At this time lengthIsKnown() will return false, and
*  a call to ResultSet.getLength will return an arbitrary, 
*  large value.
*  
*  Note: If DataSource.progressiveLoading is active the
*  reported DSResponse.totalRows value may not accurately reflect the true dataset 
*  size on the server. In this case lengthIsKnown()
*  returns true, but the ResultSet.getLength of the ResultSet may
*  change as additional rows are retrieved from the server. The
*  ResultSet.lengthIsProgressive method will indicate when the resultSet is in this
*  state.
* 
* 
*/
lengthIsKnown(): boolean;

/**
*  Does the length of this ResultSet as returned by ResultSet.getLength 
*  reflect the DSResponse.totalRows reported by a DataSource with
*  progressiveLoading enabled?
*  
*  If true, this row count may not be an accurate reflection of the true size
*  of the data set.
*  
*  This method relies on the DSResponse.progressiveLoading attribute having
*  been set accurately by the server. Note that if the user has scrolled to the
*  end of a progressively loaded data set, or ResultSet.setFullLength
*  has been explicitly called, this method will no longer return true.
* 
* 
*/
lengthIsProgressive(): boolean;

/**
*  Whether the given row has been loaded.
*  
*  Unlike get(), will not trigger a server fetch. 
* 
* 
* @param {number} row to check
*/
rowIsLoaded(rowNum:number): boolean;

/**
*  Whether the given range of rows has been loaded.
* 
*  Unlike getRange(), will not trigger a server fetch. 
* 
* 
* @param {number} start position, inclusive
* @param {number} end position, exclusive
*/
rangeIsLoaded(startRow:number, endRow:number): boolean;

/**
*  Returns the index of the first and last cached row around a given row, or null if the row
*  itself is not cached. The last cached row value is inclusive.
* 
* 
* @param {number} row to check
*/
getCachedRange(rowNum:number): Array<Partial<Integer>>;

/**
*  Returns a copy of all ResultSet.criteria and 
*  implicit criteria currently applied to 
*  this ResultSet. 
* 
*/
getCombinedCriteria(): Criteria | AdvancedCriteria;

/**
*  Attempt to find the record in the resultSet that has a primary key value that matches the 
*  passed in parameter value. Only the locally cached data will be searched. 
*  Checks only loaded rows and will not trigger a fetch. Returns null if there is no match, 
*  data is not loaded, or there is no ResultSet.dataSource.
*  
*  Note, if you pass a simple value to this method, it will be matched against the first 
*  primaryKey field. For DataSources with a composite primary key (multiple primaryKey fields),
*  pass
*  a criteria object containing just your primaryKeys, like this: 
*  { firstPkField: "value", secondPkField: 25 }.
*  
* 
* 
* @param {object} primary key value to search for
*/
findByKey(keyValue:object): Record;

/**
*  Set the filter criteria to use when fetching rows.
*  
*  Depending on the result of ResultSet.compareCriteria and settings for
*  ResultSet.useClientFiltering / FetchMode, setting criteria may cause a trip to the
*  server to get a new set of rows, or may simply cause already-fetched rows to be re-filtered
*  according to the new Criteria. In either case, the dataset length available from
*  ResultSet.getLength may change and rows will appear at different indices.
*  
*  The filter criteria can be changed while server fetches for data matching the old criteria
*  are still outstanding. If this is the case, the ResultSet will make sure that any records received
*  matching the old criteria are not added to the cache for the new criteria. Any callbacks
*  for responses to the outstanding requests are fired as normal, and the responses'
*  DSResponse.totalRows counts are kept (as they are still potentially meaningful
*  to components using the ResultSet), but the response data is cleared so that it won't be used
*  inadvertently as data matching the new criteria.
*  
*  Note: for simple Criteria, any field values in the criteria explicitly specified as null
*  will be passed to the server. By default the server then returns only records whose value
*  is null for that field. This differs from certain higher level methods such as 
*  ListGrid.fetchData which prune null criteria fields before performing a fetch
*  operation.
* 
* 
* @param {Criteria} the filter criteria
*/
setCriteria(newCriteria:Criteria): boolean;

/**
*  Update the ResultSet.allRows client-side cache of data at runtime.
*  
*  This method is useful for cases where a full, unfiltered set of data is present
*  on the client and developers wish to filter that data-set locally without going through
*  a fetch operation against a DataSource.
*  
*  Developers using this pattern may also wish to consider the FilteredList class.
*  
* 
* @param {Array<Partial<Record>>} New set of unfiltered cache data
*/
setAllRows(allRows:Array<Partial<Record>>): void;

/**
*  Get the current criteria for this ResultSet.
* 
*/
getCriteria(): Criteria;

/**
*  Default behavior is to call DataSource.compareCriteria to determine whether new
*  criteria is guaranteed more restrictive, equivalent to the old criteria, or not guaranteed
*  more restrictive, returning 1, 0 or -1 respectively. See
*  DataSource.compareCriteria for a full explanation of the default behavior.
*  
*  Override this method or DataSource.compareCriteria to implement your own client-side
*  filtering behavior.
* 
* 
* @param {Criteria} new filter criteria
* @param {Criteria} old filter criteria
* @param {Partial<DSRequest>=} dataSource request properties
* @param {string=} overrides CriteriaPolicy
*/
compareCriteria(newCriteria:Criteria, oldCriteria:Criteria, requestProperties?:Partial<DSRequest>, policy?:string): number;

/**
*  Will changing the criteria for this resultSet require fetching new data from the server, 
*  or can the new criteria be satisfied from data already cached on the client?
*  Second textMatchStyle parameter determines whether a change of text-match style
*  will require a server fetch - for example if filter is being changed between
*  an exact match (from e.g: ListGrid.fetchData) and a substring match 
*  (from e.g: ListGrid.filterData).
*  This method can be used to determine whether ListGrid.fetchData or 
*  ListGrid.filterData would cause a server side fetch when passed a certain set of 
*  criteria.
*  
*  Note that to predict correctly the decision that will be made by filter/fetch, you'll need to
*  pass the same TextMatchStyle that will be used by the future filter/fetch. Fetching
*  manually (e.g. ListGrid.fetchData) will by default use "exact" while filtering
*  (e.g. ListGrid.filterData) will by default use "substring". If the component
*  is configured for autofetch (i.e. ListGrid.autoFetchData: true), that will
*  use ListGrid.autoFetchTextMatchStyle, which defaults to "substring". If nothing/null
*  is passed for the style, this method assumes you want the style from the last filter/fetch.
*  
*  To determine what TextMatchStyle is being used, check the RPC Tab of the
*  debugging and check the relevant DSRequest. 
* 
* 
* @param {Criteria} new criteria to test.
* @param {TextMatchStyle=} New text match style. If not passed, assumes    textMatchStyle will not be modified.
*/
willFetchData(newCriteria:Criteria, textMatchStyle?:TextMatchStyle): boolean;

/**
*  Sort this ResultSet by a property of each record.
*  
*  Sorting is performed on the client for a ResultSet that has a full cache for the current
*  filter criteria. Otherwise, sorting is performed by the server, and changing the sort order
*  will invalidate the cache.
*  
*  NOTE: normalizers are not supported by ResultSets in "paged" mode, although valueMaps
*  in the DataSource are respected by the SQLDataSource.
* 
* 
* @param {string} name of the property to sort by
* @param {boolean} true == sort ascending, false == sort descending
* @param {Function | ValueMap=} May be specified as a function, with signature        normalize(item, propertyName, context), where item is        a pointer to the item in the array, propertyName is the        property by which the array is being sorted, and context is the        arbitrary context passed into this method. Normalizer function should return        the value normalized for sorting.        May also be specified as a ValueMap which maps property values to sortable values.
* @param {any=} Callers may pass an arbitrary context into the sort method, which             will then be made available to the normalizer function
*/
sortByProperty(property:string, up:boolean, normalizer?:Function | ValueMap, context?:any): List;

/**
*  Clear any ResultSet.setSort applied to this ResultSet, while
*  maintaining the current order of records. This feature is not supported for all
*  lists. This method returns true if supported.
*  
*  If a paged resultSet has a partial cache, the order
*  of records in the local data must always match the order of records as 
*  provided by the DataSource - otherwise the wrong set of records will be 
*  returned as new pages of data are fetched 
*  (in response to a user scrolling a ListGrid, for example).
*  
*  It's therefore not possible to unsort a paged resultSet with a partial cache and
*  maintain the current order of any loaded records. If this method is called on 
*  a resultSet in this state, it will always no-op and return false.
*  
*  If you need to force a partially loaded ResultSet to discard it's current sort,
*  and explicit call to ResultSet.setSort may be used. This will drop the
*  current sort specifiers and ResultSet.invalidateCache.
* 
* 
*/
unsort(): boolean;

/**
*  Return the current sort-specification for this ResultSet as an Array of SortSpecifiers.
*  
* 
*/
getSort(): Array<Partial<SortSpecifier>>;

/**
*  Sort this ResultSet by the passed list of SortSpecifiers. 
*  
*  If the ResultSet is already sorted and this method is called with an identical list of
*  specifiers, this method will no-op. To cause data to be resorted with the same set of
*  specifiers, use ResultSet.resort.
*  
*  To clear an existing sort, pass in explicit null or empty array.
* 
* 
*/
setSort(): void;

/**
*  Forcibly resort this ResultSet by the current list of SortSpecifiers.
* 
*/
resort(): void;

/**
*  Derive the current filtered set of data from the cache of all matching rows.
*  
*  This method is automatically called by ResultSet.setCriteria when criteria have actually
*  changed, as well as in various other situations. You could only need to call this method
*  directly if:
*  
*  
* - you know that client-side filtering is enabled (ResultSet.useClientFiltering:true) and
*  active ResultSet.allMatchingRowsCached.
*  
* - you have directly, programmatically modified data within the ResultSet such that it no
*  longer matches the filter criteria
*  
* - you want your modified records to disappear from the list of visible records (that is,
*  those accessible via ResultSet.get)
*  
*  
* 
*/
filterLocalData(): void;

/**
*  The ResultSet will call applyFilter() when it needs to determine whether rows match the
*  current filter criteria.
*  
*  Default behavior is to call DataSource.applyFilter to determine which rows match
*  that provided criteria.
*  
*  Override this method or DataSource.applyFilter to implement your own client-side
*  filtering behavior.
* 
* 
* @param {Array<any>} the list of rows
* @param {Criteria} the filter criteria
* @param {Partial<DSRequest>=} dataSource request properties
*/
applyFilter(data:Array<any>, criteria:Criteria, requestProperties?:Partial<DSRequest>): Array<any>;

/**
*  Set the total number of rows available from the server that match the current filter 
*  criteria for this ResultSet.
*  
*  This is an advanced feature. One use case for this method would be
*  for a ResultSet populated with progressiveLoading -
*  if application code determines an accurate row count for the current filter criteria
*  it may be applied directly to the ResultSet via this method.
* 
* 
* @param {number} total rows available from the server
*/
setFullLength(length:number): void;

/**
*  Manually invalidate this ResultSet's cache.
*  
*  Generally a ResultSet will observe and incorporate updates to the DataSource that provides its
*  records, but when this is not possible, invalidateCache() allows manual cache
*  invalidation.
*  
*  invalidateCache() fires dataChanged(), which may cause components 
*  using this ResultSet to request new data for display, triggering server fetches.
* 
*/
invalidateCache(): void;

/**
*  transformData() provides an opportunity to modify data that has been
*  returned from the server, before it has been integrated into the client-side cache.
*  
*  If data is not immediately suited for client-side use when it is returned from the
*  ultimate data store, this method allows it to be transformed on the client so that such
*  transform operations do not impact server scalability.
*  
*  It is legal for transformData() to modify not only the records, but also
*  their number (by modifying startRow and endRow on the DSResponse object).
*  
*  See also DataSource.transformResponse for an alternative entry point which
*  applies to all DSResponses for a DataSource.
* 
* 
* @param {any} data returned from the server
* @param {DSResponse} the DSResponse object returned by the               server
*/
transformData(newData:any, dsResponse:DSResponse): Array<Partial<object>>;

/**
*  Notification fired when data has arrived from the server and has been successfully
*  integrated into the cache.
*  
*  When dataArrived() fires, an immediate call to getRange() with
*  the startRow and endRow passed as arguments will return a List
*  with no ResultSet.getLoadingMarker.
*  
*  Note that dataArrived() won't fire in the case of the owning component
*  filtering with unchanged criteria (for example using ListGrid.fetchData or
*  ListGrid.filterData). To support backward compatibility, the property
*  ResultSet.reapplyUnchangedLocalFilter can be set to force dataArrived() to
*  be called if the ResultSet is ResultSet.filterLocalData and the criteria
*  haven't changed but are narrower than the criteria used to fetch the current cache.
* 
* 
* @param {Integer} starting index of rows that have just loaded
* @param {Integer} ending index of rows that have just loaded, non-inclusive
*/
dataArrived(startRow:Integer, endRow:Integer): void;

/**
*  Like List.findAll. Checks only loaded rows and will not trigger a fetch.
* 
* @param {string | object | AdvancedCriteria} property to match, or if an Object is passed, set of                    properties and values to match
* @param {any=} value to compare against (if propertyName is a string)
*/
findAll(propertyName:string | object | AdvancedCriteria, value?:any): Array<any>;

/**
*  Like List.find. Checks only loaded rows and will not trigger a fetch.
* 
* @param {string | object | AdvancedCriteria} property to match, or if an Object is passed, set of                    properties and values to match
* @param {any=} value to compare against (if propertyName is a string)
*/
find(propertyName:string | object | AdvancedCriteria, value?:any): object;

/**
*  Like List.findIndex. Checks only loaded rows and will not trigger a fetch.
* 
* @param {string | object | AdvancedCriteria} property to match, or if an Object is passed, set of                    properties and values to match
* @param {any=} value to compare against (if propertyName is a string)
*/
findIndex(propertyName:string | object | AdvancedCriteria, value?:any): Integer;

/**
*  Like List.findNextIndex. Checks only loaded rows and will not trigger a fetch.
* 
* @param {Integer} first index to consider.
* @param {string | Function | object | AdvancedCriteria} property to match; or, if a function is passed, the predicate function to call; or, if an object is passed, set of properties and values to match.
* @param {any=} value to compare against (if propertyName is a string) or the value of this when the predicate function is invoked (if propertyName is a function)
* @param {Integer=} last index to consider (inclusive).
*/
findNextIndex(startIndex:Integer, propertyName:string | Function | object | AdvancedCriteria, value?:any, endIndex?:Integer): Integer;

/**
*  Like List.getProperty. Checks only loaded rows and will not trigger a fetch.
* 
* @param {string} name of the property to look for
*/
getProperty(property:string): Array<any>;



}
/**
* Creates a new ResultSet
* 
* @param typeCheckedProperties {Partial<ResultSet>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ResultSet} The new instance
*/
class ResultSet{
static create(typeCheckedProperties? : Partial<ResultSet>, uncheckedProperties?: Object): ResultSet;




/**
*  Return the singleton marker object that is used as a placeholder for records that are being
*  loaded from the server.
* 
*/
static getLoadingMarker(): string;




}

/**
* ResultTrees are an implementation of the Tree API, used to handle hierarchical
*  data, whose nodes are DataSource records which are retrieved from a server.
*  
*  Modifying ResultTrees
*  
*  ResultTree nodes cannot be directly added or removed from a
*  ResultTree.fetchMode ResultTree via Tree APIs such as
*  Tree.add or Tree.remove, since such trees are considered to be read-only
*  by virtue of containing ResultSets, which are read-only data structures. Even in
*  other FetchModes, calling such APIs will only update the local cache of the
*  ResultTree, rather than triggering any server traffict to update the DataSource.
*  
*  Use DataSource.addData/DataSource.removeData to add/remove
*  rows from the DataSource, and the ResultTree will reflect the changes
*  automatically. Alternatively, the DataSource.updateCaches method may be called to
*  only update local caches of the DataSource in question, without generating any server
*  traffic.
*  
*  To create a locally modifiable cache of records from a DataSource, you can use
*  DataSource.fetchData to retrieve a List of records which can be modified directly,
*  or you can create a client-only DataSource from the retrieved data to share a
*  modifiable cache between several DataBoundComponents.
*/
interface ResultTree extends Tree {
/**
* Selects the model used to construct the tree representation. See TreeModelType for
*  the available options and their implications.
*  
*  If the "parent" modelType is used, you can provide the initial parent-linked data set to the
*  tree via the Tree.data attribute. If the "children" modelType is used, you can
*  provide the initial tree structure to the Tree via the Tree.root attribute.
* @type {TreeModelType}
* @default "parent"
*/
modelType?: TreeModelType;

/**
* Optional initial data for the tree. If the ResultTree.fetchMode is
*  "basic" or "local" then the format of this data is exactly
*  the same Tree.parentIdField-linked list of tree nodes as
*  documented on Tree.data (when the modelType is set to
*  "parent"). If the fetchMode is "paged" then the
*  format is extended to allow the ResultTree.childCountProperty
*  to be set on folder nodes.
*  
*  Providing an initial set of nodes in this way does not affect the behavior of the
*  ResultTree in its loading of unloaded folders. An equivalent result is achieved if the
*  first fetch from the server returns this same data.
*  
*  If fetchMode is "paged" then you may make folder-by-folder
*  choices as to whether to use paging for the childen of each folder. If you would like
*  to use paging in a folder then you may include a partial list of that folder's children
*  with the data, provided that you set the childCountProperty to the total
*  number of children. Otherwise you will need to include either all children of the
*  folder or none of the children. Open folders without any children provided will cause
*  immediate, new fetches for the children, as usual.
*  
*  Because the initial data is treated exactly as though it were returned from the tree's
*  first server fetch, the order of the initial data must match the initial sort order of
*  the TreeGrid displaying the data or, if no such sort is specified, the native storage
*  order on the server. For example, consider initial data containing n
*  records having the parentId "X", meaning they are all in
*  the same folder. These n records are the records at indices
*  0 through (n - 1) that are stored on the server under the
*  parent node. If the childCountProperty set on the parent node indicates
*  that there are m > n total rows under the parent node then the records at
*  indices n to (m - 1) will be fetched from the server as the user
*  scrolls the additional rows into view.
* @type {Array<Partial<TreeNode>>}
* @default null
*/
data?: Array<Partial<TreeNode>>;

/**
* What DataSource is this resultTree associated with?
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* Does this resultTree load data incrementally as folders within the tree are opened, or
*  is it all loaded in a single request? Must be true if ResultTree.fetchMode is
*  "paged".
* @type {boolean}
* @default true
*/
loadDataOnDemand?: boolean;

/**
* Controls what happens to the ResultTree.getOpenState - the set of 
*  nodes opened or closed by the end user after tree data is loaded - when an entirely
*  new tree of nodes is loaded from the server, as a consequence of calling 
*  ResultTree.invalidateCache or of changing criteria such that the current cache of
*  nodes is dropped.
* @type {PreserveOpenState}
* @default "whenUnique"
*/
autoPreserveOpenState?: PreserveOpenState;

/**
* Mode of fetching records from server.
*  
*  fetchMode:"local" implies that local filtering will always be performed. See
*  ResultTree.keepParentsOnFilter for additional filtering details.
*  
*  fetchMode:"basic" or "paged" implies that if search criteria change, the entire
*  tree will be discarded and re-fetched from the server. When retrieving the replacement
*  tree data, the default behavior will be to preserve the ResultTree.getOpenState
*  for any nodes that the server returns which were previously opened by the user. Note
*  that this implies that if ResultTree.loadDataOnDemand is enabled and the server returns only
*  root-level nodes, open state will be preserved only for root-level nodes, and children
*  of open root-level nodes will be immediately fetched from the server if
*  they are not included in the server's initial response.
*  
*  fetchMode:"paged" enables paging for nodes that have very large numbers of children.
*  Whenever the children of a folder are loaded, the resultTree will set
*  DSRequest.startRow and DSRequest.endRow when requesting children
*  from the DataSource, and will manage loading of further children on demand, similar to
*  how a ResultSet manages paging for lists. For a deeper discussion see the
*  Paging large sets of children section of the treeDataBinding
*  overview.
* @type {FetchMode}
* @default "basic"
*/
fetchMode?: FetchMode;

/**
* How many tree nodes to retrieve at once from each large set of children in the tree.
*  
*  Applicable only with fetchMode: "paged". When a paged ResultTree is asked
*  for rows that have not yet been loaded, it will fetch adjacent rows that are likely to
*  be required soon, in batches of this size.
* @type {Integer}
* @default "75"
*/
resultSize?: Integer;

/**
* When using FetchMode and providing multiple levels of the tree in
*  one DSResponse, childCountProperty must be set for any folders that include
*  only a partial list of children.
*  For a deeper discussion see the Paging large sets of children section of the
*  treeDataBinding overview.
* @type {string}
* @default "childCount"
*/
childCountProperty?: string;

/**
* When using FetchMode and providing multiple levels of the tree
*  in one DSResponse, canReturnOpenSubfoldersProperty may be set on any
*  folder to indicate whether child folders might be returned by the server already open.
*  If the property is set to false on a folder then subfolders of that folder are never
*  allowed to be returned already open. This enables the paging mechanism to be more
*  efficient in the amount of data that it requests from the server.
*  
*  For example, setting the canReturnOpenSubfoldersProperty value to
*  false on a node is appropriate if the server-side code determines that the
*  the node's children consist of entirely leaf nodes.
* @type {string}
* @default "canReturnOpenSubfolders"
*/
canReturnOpenSubfoldersProperty?: string;

/**
* When using FetchMode and providing multiple levels of the tree
*  in one DSResponse, this property specifies the default value assumed for the
*  ResultTree.canReturnOpenSubfoldersProperty when no value for that property is provided for
*  a node.
* @type {boolean}
* @default false
*/
canReturnOpenFolders?: boolean;

/**
* Sets DataSource.progressiveLoading for this
*  ResultTree. The ResultTree will copy this setting onto the DSRequests that it
*  issues, overriding the OperationBinding- and DataSource-level settings, in cases where
*  the use of progressive loading does not affect the correctness of the tree's paging
*  algorithm.
*  
*  This setting is applied automatically by DataBoundComponents that have their own
*  explicit setting for DataBoundComponent.progressiveLoading.
*  
*  Note: This property only has an effect for FetchMode
*  ResultTrees.
* @type {boolean}
* @default null
*/
progressiveLoading?: boolean;

/**
* Controls whether nodes are assumed to be folders or leaves by default.
*  
*  Nodes that have children or have the Tree.isFolderProperty set
*  to true will always be considered folders. Other nodes will be considered folders or
*  leaves by default according to this setting.
*  
*  If defaultIsFolder is unset, the ResultTree will automatically set it to
*  match the value of ResultTree.loadDataOnDemand. This means that, when using
*  folder-by-folder load on demand (loadDataOnDemand:true), by default a newly
*  loaded node will be considered to be a folder that has not loaded its children yet. 
*  
*  When not using folder-by-folder load on demand, by default a newly loaded node is
*  considered a leaf. If you set defaultIsFolder:true explicitly, by default
*  a newly loaded node is considered to be a folder with no children.
*  
*  See Tree.isFolder for details on how to explicitly mark nodes as folders or leaves.
* @type {boolean}
* @default null
*/
defaultIsFolder?: boolean;

/**
* This attribute may be used to specify a root value for the parentIdField of this resultTree.
*  This overrides the default DataSourceField.rootValue for this tree, allowing
*  a component to navigate a tree starting at a specific node.
*  
*  May be overridden via TreeGrid.treeRootValue for ResultTrees generated by a TreeGrid
*  component.
* @type {any}
* @default null
*/
rootNode?: any;

/**
* Which nodes should be opened automatically - applied whenever 
*  Tree.setRoot is called, including during initialization and as part of 
*  a re-fetch caused, for example, by Tree.duplicate or 
*  ResultTree.invalidateCache.
*  
*  Options are:
*  
*  
* - "none" - no nodes are opened automatically
*  
* - "root" - opens the ResultTree.rootNode - in databound 
*        trees, this node is always hidden
*  
* - "all" - when ResultTree.loadDataOnDemand, opens the
*        ResultTree.rootNode and all of it's direct
*        descendants - otherwise, opens all loaded nodes 
*  
* @type {string}
* @default "none"
*/
autoOpen?: string;

/**
* When data is loaded from the server, should nodes with an explicit value for
*  the Tree.parentIdField which doesn't map to a valid parent node be dropped?
*  If set to false, for TreeGrid.loadDataOnDemand:false trees, parentless nodes will be
*  added as children of the root node - for TreeGrid.loadDataOnDemand:true, they will be
*  added as children of the folder currently requesting children.
*  
*  This effectively allows nodes to be loaded into the current (or root) folder without
*  needing an explicit Tree.parentIdField that matches the folder's
*  ID or rootValue for the resultTree.
*  
*  Note: For loadDataOnDemand:false trees, if this property is unset at init time,
*  it will default to true if an explicit ResultTree.rootNode has been
*  specified. This ensures that if the data tree retrieved from the server includes ancestors
*  of the desired root-node we don't display them. Otherwise this property always defaults to
*  false.
* @type {boolean}
* @default null
*/
discardParentlessNodes?: boolean;

/**
* This attribute governs how to handle cache-synch when a new node is added to this dataSource 
*  with no explicit parentId.
*  
*  If set to true, when a new node is added to this dataSource via
*  DataSource.addData, with no explicit parentId, the node will be added as a 
*  child of the root node of this result tree. Otherwise it will be ignored.
*  
*  Similar logic applies to DataSource.updateData - if this property is
*  true and the parentId of an updated node is cleared, it will be moved to become a child of
*  root, otherwise it will be dropped from the tree.
* @type {boolean}
* @default false
*/
defaultNewNodesToRoot?: boolean;

/**
* When a successful Add, Update or Remove type operation fires on this ResultTree's 
*  dataSource, if DSResponse.data is unset, should we integrate the submitted
*  data values (from the request) into our data-set?
* @type {boolean}
* @default true
*/
updateCacheFromRequest?: boolean;

/**
* Criteria that are never shown to or edited by the user and are cumulative with any 
*  criteria provided via DataBoundComponent.initialCriteria, ResultTree.setCriteria
*  etc.
* @type {Criteria}
* @default null
*/
implicitCriteria?: Criteria;

/**
* By default when the data of this ResultTree's dataSource is modified, the ResultTree will
*  be updated to display these changes.
*  Set this flag to true to disable this behavior.
* @type {boolean}
* @default false
*/
disableCacheSync?: boolean;

/**
* This property allows you to specify the dataSource to be used for fetching link information
*  in a databound multilink tree. A multilink tree is one where the same node is 
*  allowed to appear in multiple places in the tree, and it is achieved by providing the node 
*  data and the link data separately. Nodes are provided via the normal 
*  ResultTree.dataSource; linkDataSource is only used for 
*  fetching and updating link information. 
*  
*  The linkDataSource is an ordinary DataSource that you implement
*  just like any other. However, for correct operation as a linkDataSource, it 
*  must have the following:
*  
* - A DataSourceField.primaryKey. Like any dataSource, a 
*  linkDataSource is not fully functional without a primaryKey field
*  
* - A field named the same as the Tree.parentIdField
*  
* - A field named the same as the Tree.idField
*  
* - Optionally, a field named the same as the Tree.linkPositionField
*  
* - Fields for other values you may wish to store with the link, if any
*  
*  Providing node data and link data
*  Consider a structure for the components of a bicycle, greatly simplified:
*      Frame
*     /  \
*    Wheel  Wheel
*   / \   / \
*  Hub Tire Hub Tire
*  
*  Here, the two wheels are the same assembly, so really it should look like this:
*     Frame
*     | |        
*     Wheel
*     /  \   
*    Hub Tire 
*  
*  Normal SmartClient trees cannot model this arrangement accurately because this is not 
*  really a tree, it is a graph; trees do not contain multiple paths to a given node. The 
*  only way to handle this arrangement of nodes in a formal tree would be to make two copies
*  of the "Wheel" node, at which point they are no longer the same thing. Either way, in a 
*  TreeGrid, we would have to visualise it like this: 
*   Frame
*    Wheel
*      Hub
*      Tire
*    Wheel
*      Hub
*      Tire
*  
*  But if we use copies so that the the two wheels are no longer the same thing, changing 
*  one of them will no longer change the other, which is a fundamental problem because in 
*  this scenario, the two wheels really are the same thing. Now, changing the name of the 
*  "Hub" in one "Wheel" would not change it in the other; adding a "Spokes" node to the 
*  second item would not also add it to the first. Drag-reordering child nodes in one 
*  "Wheel" would not re-order them in the other. All of these things are incorrect, because
*  the two wheels are the same thing.
*  
*  Multilink trees provide a way to handle this arrangement without physical copying of the
*  duplicate nodes, preserving the sameness of them and thus fixing all the problems
*  described above.
*  
*  The node data for the above tree, simplified, would be a flat list something like this:
*   [
*    { id: 1, description: "Frame" },
*    { id: 2, description: "Wheel" },
*    { id: 3, description: "Hub" },
*    { id: 4, description: "Tire" }
*   ]
*  
*  The link data would look like this:
*   [
*    { linkId: 1, parentId: 1, id: 2, position: 1 },
*    { linkId: 2, parentId: 2, id: 3, position: 1 },
*    { linkId: 3, parentId: 2, id: 4, position: 2 },
*    { linkId: 4, parentId: 1, id: 2, position: 2 }
*   ]
*  
*  Or, if you were using ResultTree.linkDataFetchMode "single", you would combine 
*  the node and link data into a single dataset like this:
*   [
*    { id: 1, position: 0, description: "Frame" },
*    { parentId: 1, id: 2, position: 1, description: "Wheel", linkId: 1 },
*    { parentId: 2, id: 3, position: 1, description: "Hub", linkId: 2 },
*    { parentId: 2, id: 4, position: 2, description: "Tire", linkId: 3 },
*    { parentId: 1, id: 2, position: 2, description: "Wheel", linkId: 4 }
*   ]
* 
*  
*  NOTE: It is also possible to create an unbound multilink tree - see Tree.linkData.
* @type {DataSource | GlobalId}
* @default null
*/
linkDataSource?: DataSource | GlobalId;

/**
* The fetch mode for this tree's link data; ignored if this is not a 
*  Tree.isMultiLinkTree
* @type {LinkDataFetchMode}
* @default "separate"
*/
linkDataFetchMode?: LinkDataFetchMode;

/**
* For Tree.isMultiLinkTrees only, should we send up the 
*  Tree.parentIdField in fetch criteria if the criteria value is null? If 
*  false, we remove the parentId from the criteria when 
*  ResultTree.linkDataSource, if the criteria value is 
*  null (as it will be by default when fetching the direct child nodes of the tree's root).
*  
*  Ignored for non-multiLink trees.
* @type {boolean}
* @default true
*/
sendNullParentInLinkDataCriteria?: boolean;

/**
* The DSRequest.operationId this ResultTree should use 
*  when performing fetch operations on its ResultTree.linkDataSource. Has no effect
*  if this is not a Tree.isMultiLinkTree
*  
*  Note, this value can be overridden by DSRequest.linkDataFetchOperation when 
*  calling fetchData() on the component (e.g. TreeGrid.fetchData) directly from application code.
* @type {string}
* @default null
*/
linkDataFetchOperation?: string;

/**
* The DSRequest.operationId this ResultTree should use 
*  when performing add operations on its ResultTree.linkDataSource. Has no effect
*  if this is not a Tree.isMultiLinkTree.
*  
*  Note, this property wll be used by internal update operations when you drag-move or 
*  drag-reparent nodes in a multi-link tree. Do not use it when adding records from 
*  application code by directly calling addData() on the 
*  ResultTree.linkDataSource; instead just use the regular 
*  operationId property in your add request. Also note, because this 
*  property is intended to allow your code to influence the operationId used by internal 
*  methods, and those methods never directly update link data (moved and re-parented links 
*  are always removed and then re-added), there is no corresponding 
*  linkDataUpdateOperation property.
* @type {string}
* @default null
*/
linkDataAddOperation?: string;

/**
* The DSRequest.operationId this ResultTree should use 
*  when performing remove operations on its ResultTree.linkDataSource. Has no effect
*  if this is not a Tree.isMultiLinkTree.
*  
*  See ResultTree.linkDataAddOperation for more information on how this property is 
*  intended to be used.
* @type {string}
* @default null
*/
linkDataRemoveOperation?: string;

/**
* For Tree.isMultiLinkTree, indicates that we should automatically
*  update the Tree.linkPositionField values of old and new sibling records 
*  after a drag reparent or reposition-within-parent operation. For example, say you have a 
*  tree like this (where the number in parentheses indicates the node's 
*  Tree.linkPositionField value):
*    A
*     - B (1)
*     - C (2)
*     - D (3)
*    E
*     - F (1)
*     - G (2)
*  and you drag node C out and drop it between nodes F and G. This drag operation will spawn
*  two update operations to the server: a "remove" to delete node C from parent A, and an "add"
*  to re-add it under parent E. With autoUpdateSiblingNodesOnDrag in force, we 
*  also automatically issue two "update" operations to the server - one to change the position 
*  on node D to 2, and another to change the position on node G to 3. The end result of this
*  is that node position values are kept correct.
*  
*  Please note the following:
*  
* - As noted above, these automatic updates are persistent - we send a queue of actual 
*  update requests to the server. This is convenient, but it may not be terribly efficient, 
*  particularly if you have just dropped a node at the head of a list of several hundred 
*  siblings. This is why we do not default this setting to true
*  
* - The automatic updates work by applying an integer delta value to the existing position
*  value. So in the above example, we would compute a delta of negative 1 for node D and 
*  positive 1 for node G. The upshot of this is that autoUpdateSiblingNodesOnDrag
*  only works well if your position values are consecutive integers
* @type {boolean}
* @default "(see below)"
*/
autoUpdateSiblingNodesOnDrag?: boolean;

/**
* If ResultTree.autoUpdateSiblingNodesOnDrag is in force, this is the value we will 
*  use to auto-update the position of a node when we cannot derive that value from the 
*  existing value of a neighbor. This happens when a node is dropped into the very first 
*  position below a parent (including the special case of the parent being previously empty)
* @type {Integer}
* @default "1"
*/
firstPositionValue?: Integer;

/**
* If set, tree-based filtering is performed such that parent nodes are kept as long as
*  they have children that match the filter criteria, even if the parents themselves do not
*  match the filter criteria. If not set, filtering will exclude parent nodes not matching
*  the criteria and all nodes below them in the tree.
*  
*  ResultTrees will default to FetchMode whenever 
*  keepParentsOnFilter is true, unless fetchMode was explicitly set to
*  "paged" (see below). This allows the filtering logic to
*  fetch a complete tree of nodes from the DataSource (or if 
*  ResultTree.loadDataOnDemand:true, a complete set of nodes under a given parent) and then
*  filter the resulting data locally on the client. 
*  
*  This means that the server does not need to implement special tree filtering logic
*  to support looking up nodes that match the specified criteria as well as ancestor nodes
*  that may not.
*  
*  If some criteria must be sent to the server in order to produce a valid tree of
*  data, but keepParentsInFilter is also required, the 
*  ResultTree.serverFilterFields attribute may be used to specify a list
*  of field names that will be sent to the server whenever they are present in the 
*  criteria. Note that for the subset of criteria applied to these fields,
*  keepParentsInFilter behavior will not occur without custom 
*  logic in the DataSource fetch operation.
*  
*  If FetchMode is explicitly set to "paged", it is not possible 
*  to implement keepParentsOnFilter by local filtering. Support for
*  keepParentsOnFilter for a paged ResultTree therefore also requires
*  custom logic in the DataSource fetch operation. To support this a developer
*  must ensure that their fetch operation returns the appropriate set of nodes -
*  all nodes that match the specified criteria plus their ancestor nodes even
*  if they do not match the specified criteria.
* @type {boolean}
* @default null
*/
keepParentsOnFilter?: boolean;

/**
* For FetchMode ResultTrees, this property lists field names 
*  that will be sent to the server if they are present in the criteria.
*  
*  This property may be used to ensure a dataSource receives the necessary criteria
*  to populate a ResultTree's data, and also support ResultTree.keepParentsOnFilter. 
*  
*  Note that for some AdvancedCriteria it will not be possible to extract the
*  subcriteria that apply to certain fields. See DataSource.splitCriteria
*  for details on how serverFilterFields-applicable subcriteria are extracted from
*  the specified criteria for the tree.
* @type {Array<Partial<string>>}
* @default null
*/
serverFilterFields?: Array<Partial<string>>;

/**
* Whether or not we should skip promotion of a simple criteria to an AdvancedCriteria
*  when sending the DSRequest to load the children of a node in a ResultTree.loadDataOnDemand
*  or ResultTree.fetchMode ResultTree. If the
*  DSRequest.textMatchStyle is not "exact", we normally convert the simple criteria to an
*  AdvancedCriteria for correctness in matching the node name, but setting this property
*  to true will allow that to be skipped for backcompat with older releases.
* @type {boolean}
* @default false
*/
useSimpleCriteriaLOD?: boolean;

/**
* The filter criteria to use when fetching rows. For usage see
*  ResultTree.setCriteria.
* @type {Criteria}
* @default null
*/
criteria?: Criteria;



/**
*  Returns a copy of all ResultTree.criteria and 
*  implicit criteria currently applied to 
*  this ResultTree. 
* 
*/
getCombinedCriteria(): Criteria | AdvancedCriteria;

/**
*  Load the children of a given node.
*  
*  For a databound tree this will trigger a fetch against the Tree's DataSource.
* 
* 
* 
* @param {TreeNode} node in question
* @param {DSCallback=} Optional callback (stringMethod) to fire when loading            completes. Has a single param node - the node whose            children have been loaded, and is fired in the scope of the Tree.
*/
loadChildren(node:TreeNode, callback?:DSCallback): void;

/**
*  Unload the children of a folder, returning the folder to the "unloaded" state.
* 
* 
* @param {TreeNode} folder in question
* @deprecated * It's recommended that you instead use Tree.reloadChildren to reload the children of a folder, or Tree.removeChildren if you need to clear the cached children
*  of a folder to add specific local data.

*/
unloadChildren(node:TreeNode): void;

/**
*  Replaces the existing children of a parent node. This leaves the node in the loaded state
*  (unless a partially loaded set of children is specified using the optional
*  totalChildren argument).
*  
*  The supplied array of children may be null or empty to indicate there are none, but
*  if present must be in the standard format as would be sent from the server, as described
*  by treeDataBinding.
*  
*  In particular, note that for a ResultTree.fetchMode ResultTree,
*  each child node::
*  
* - can have nested children spcified under the Tree.childrenProperty (but not via
*  TreeNode.id/TreeNode.parentId linking)
*  
* - cannot be open unless it includes either a complete set of children, or partial set of
*  children and a childCount
* 
* 
* @param {TreeNode} parent of children
* @param {Array<Partial<TreeNode>>} children to be set
* @param {Integer=} number of total children (if not all have been                     provided as newChildren); only allowed if paging
*/
setChildren(parent:TreeNode, newChildren:Array<Partial<TreeNode>>, totalChildren?:Integer): void;

/**
*  Manually invalidate this ResultTree's cache.
*  
*  Generally a ResultTree will observe
*  and incorporate updates to the DataSource that provides its
*  records, but when this is not possible, invalidateCache() allows manual cache
*  invalidation.
*  
*  Components bound to this ResultTree will typically re-request the currently visible portion
*  of the dataset, causing the ResultTree to re-fetch data from the server.
* 
*/
invalidateCache(): void;

/**
*  Get the item in the openList at a particular position.
* 
* @param {number} position of the node to get
*/
get(pos:number): TreeNode;

/**
*  	Get a range of items from the open list
* 
* @param {number} start position
* @param {number} end position (NOT inclusive)
*/
getRange(start:number, end:number): Array<Partial<TreeNode>>;

/**
*  Set the filter criteria to use when fetching rows.
*  
*  Depending on the result of ResultTree.compareCriteria and setting for
*  ResultTree.fetchMode, setting criteria may cause a trip to the server to get a new
*  set of nodes, or may simply cause already-fetched nodes to be re-filtered according to the
*  new criteria.
*  
*  For a basic overview on when server fetches are generally performed, see
*  ResultTree.fetchMode.
*  However, this is not the final determination of when server fetches occur. Criteria can
*  be split into local criteria and server criteria by specifying ResultTree.serverFilterFields.
*  Thus, even when using fetchMode:"local" a new server fetch will occur if the server
*  criteria changes. For details on how the criteria is split, see
*  DataSource.splitCriteria.
*  
*  Note: if criteria is being split to retrieve server criteria portion and the criteria
*  is an AdvancedCriteria, the criteria must consist of a single "and" operator
*  and one or more simple criteria below it. No other logical operators may be used. In
*  other words, the AdvancedCriteria provided must be exactly representable by a
*  simple criteria.
* 
* 
* @param {Criteria} the filter criteria
*/
setCriteria(newCriteria:Criteria): void;

/**
*  The ResultTree will call applyFilter() when it needs to locally filter the tree using the
*  current filter criteria.
*  
*  Default behavior is to call Tree.getFilteredTree to obtain a new, filtered tree.
*  
*  Override this method or Tree.getFilteredTree to implement your own client-side
*  filtering behavior. Note that the original tree should not be affected.
* 
* 
* @param {Tree} the source tree to be filtered
* @param {Criteria} the filter criteria
* @param {TreeFilterMode} mode to use for filtering
* @param {DataSource} dataSource for filtering if the Tree does not                    already have one
* @param {DSRequest=} Request properties block. This allows developers to specify properties that would impact the filter such as DSRequest.textMatchStyle
*/
applyFilter(tree:Tree, criteria:Criteria, filterMode:TreeFilterMode, dataSource:DataSource, requestProperties?:DSRequest): Tree;

/**
*  Default behavior is to call DataSource.compareCriteria to determine whether new
*  criteria is equivalent to the old criteria (returns 0) or not.
*  
*  See DataSource.compareCriteria for a full explanation of the default behavior.
*  The CriteriaPolicy used is "dropOnChange".
*  
*  Override this method or DataSource.compareCriteria to implement your own client-side
*  filtering behavior.
* 
* 
* @param {Criteria} new filter criteria
* @param {Criteria} old filter criteria
* @param {Partial<DSRequest>=} dataSource request properties
*/
compareCriteria(newCriteria:Criteria, oldCriteria:Criteria, requestProperties?:Partial<DSRequest>): number;

/**
*  Will changing the criteria for this resultTree require fetching new data from the server
*  or can the new criteria be satisfied from data already cached on the client?
*  
*  This method can be used to determine whether TreeGrid.fetchData or 
*  TreeGrid.filterData will cause a server side fetch when passed a certain set of 
*  criteria.
* 
* 
* @param {Criteria} new criteria to test.
*/
willFetchData(newCriteria:Criteria): boolean;

/**
*  Returns a snapshot of the current open state of this tree's data as
*  a TreeGridOpenState object.
*  
*  This object can be passed to ResultTree.setOpenState or
*  TreeGrid.setOpenState to open the same set of folders
*  within the tree's data (assuming the nodes are still present in the data).
* 
*/
getOpenState(): TreeGridOpenState;

/**
*  Reset the set of open folders within this tree's data to match the 
*  TreeGridOpenState object passed in.
*  
*  Used to restore previous state retrieved from the tree by a call to 
*  ResultTree.getOpenState.
* 
* 
* @param {TreeGridOpenState} Object describing the desired set of open folders.
*/
setOpenState(openState:TreeGridOpenState): void;

/**
*  This callback fires whenever the resultTree receives new nodes from the server, after
*  the new nodes have been integrated into the tree.
*  
* 
* @param {TreeNode} The parentNode for which children were just loaded
*/
dataArrived(parentNode:TreeNode): void;



}
/**
* Creates a new ResultTree
* 
* @param typeCheckedProperties {Partial<ResultTree>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ResultTree} The new instance
*/
class ResultTree{
static create(typeCheckedProperties? : Partial<ResultTree>, uncheckedProperties?: Object): ResultTree;







}

/**
* A ToolStrip class for showing 
*  RibbonGroup of RibbonButtons.
*/
interface RibbonBar extends ToolStrip {
/**
* If set, this attribute affects whether RibbonGroup
*  in this RibbonBar show their header control. You can override this at the 
*  RibbonGroup.setShowTitle level.
* @type {boolean}
* @default true
*/
showGroupTitle?: boolean;

/**
* If set, this attribute affects the alignment of the titles in 
*  RibbonGroup in this RibbonBar. You can 
*  override this at the RibbonGroup.titleAlign level.
* @type {Alignment}
* @default "center"
*/
groupTitleAlign?: Alignment;

/**
* If set, this attribute affects the orientation of the titles in 
*  RibbonGroup in this RibbonBar. You can 
*  override this at the RibbonGroup.titleOrientation level.
* @type {VerticalAlignment}
* @default "top"
*/
groupTitleOrientation?: VerticalAlignment;



/**
*  Add a new group to this RibbonBar. You can either create your group externally and pass 
*  it in, or you can pass a properties block from which to automatically construct it.
* 
* 
* @param {RibbonGroup} the new group to add to this ribbon
* @param {Integer=} the index at which to insert the new group
*/
addGroup(group:RibbonGroup, position?:Integer): void;



}
/**
* Creates a new RibbonBar
* 
* @param typeCheckedProperties {Partial<RibbonBar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RibbonBar} The new instance
*/
class RibbonBar{
static create(typeCheckedProperties? : Partial<RibbonBar>, uncheckedProperties?: Object): RibbonBar;







}

/**
* A Button subclass that displays an RibbonButton.icon, 
*  RibbonButton.showButtonTitle and optional RibbonButton.menuIconSrc 
*  and is capable of RibbonButton.vertical orientation.
*/
interface RibbonButton extends Button {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "RibbonButtonEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* The orientation of this RibbonButton. The default value, "vertical", renders 
*  RibbonButton.icon, RibbonButton.showButtonTitle and potentially 
*  RibbonButton.menuIconSrc, from top to bottom: "horizontal" does the same 
*  from top to bottom.
* @deprecated * in favor of RibbonButton.vertical which, if set, takes precendence over this setting

* @type {string}
* @default "vertical"
*/
orientation?: string;

/**
* Whether this button renders vertically. Renders the 
*  RibbonButton.icon, RibbonButton.showButtonTitle and potentially 
*  RibbonButton.menuIconSrc from top to bottom, when true, and from left to right
*  when false.
* @type {boolean}
* @default false
*/
vertical?: boolean;

/**
* When used in a RibbonBar, the number of rows this button should occupy in a
*  single RibbonGroup.columnLayout.
* @type {number}
* @default "1"
*/
rowSpan?: number;

/**
* Default stateful CSS class for this button. When RibbonButton.iconStyle or 
*  RibbonButton.menuIconStyle are unset, they will default to the value of this 
*  attribute, suffixed with H/VIcon or H/VMenuIcon respectively,
*  depending on the value of RibbonButton.vertical.
* @type {CSSStyleName}
* @default "ribbonButton"
*/
baseStyle?: CSSStyleName;

/**
* Default CSS class for this button's RibbonButton.icon. If unset, defaults to 
*  RibbonButton.baseStyle suffixed with VIcon or HIcon
*  depending on the value of RibbonButton.vertical.
* @type {CSSStyleName}
* @default null
*/
iconStyle?: CSSStyleName;

/**
* Default CSS class to apply to the element showing this button's 
*  RibbonButton.menuIconSrc. If unset, defaults to 
*  RibbonButton.baseStyle suffixed with VMenuIcon or HMenuIcon
*  depending on the value of RibbonButton.vertical.
* @type {CSSStyleName}
* @default null
*/
menuIconStyle?: CSSStyleName;

/**
* Whether to show the RibbonButton.menuIconSrc which fires the RibbonButton.menuIconClick 
*  notification method when clicked.
* @type {boolean}
* @default null
*/
showMenuIcon?: boolean;

/**
* Base URL for an Image that shows a Menu when clicked. See also 
*  RibbonButton.showMenuIconDisabled and RibbonButton.showMenuIconOver.
* @type {SCImgURL}
* @default "[SKINIMG]/Menu/submenu_down.png"
*/
menuIconSrc?: SCImgURL;

/**
* The width of the icon for this button.
* @type {number}
* @default "14"
*/
menuIconWidth?: number;

/**
* The height of the icon for this button.
* @type {number}
* @default "13"
*/
menuIconHeight?: number;

/**
* This attribute is not supported in this subclass. However, RTL mode is still supported.
* @type {string}
* @default null
*/
iconOrientation?: string;

/**
* This attribute is not supported in this subclass. However, RTL mode is still supported.
* @type {string}
* @default null
*/
iconAlign?: string;

/**
* Horizontal alignment of this button's content. If unset, 
*  RibbonButton.vertical are center-aligned and horizontal
*  buttons left-aligned by default.
* @type {Alignment}
* @default null
*/
align?: Alignment;

/**
* Vertical alignment of this button's content. If unset, 
*  RibbonButton.vertical are top-aligned and horizontal
*  buttons center-aligned by default.
* @type {VerticalAlignment}
* @default null
*/
valign?: VerticalAlignment;

/**
* showTitle is not applicable to this class - use RibbonButton.showButtonTitle
*  instead.
* @type {boolean}
* @default null
*/
showTitle?: boolean;

/**
* Whether to show the title-text for this RibbonButton. If set to false, title-text is omitted
*  altogether and just the icon is displayed.
* @type {boolean}
* @default true
*/
showButtonTitle?: boolean;

/**
* Whether to show an Icon in this RibbonButton. Set to false to render a text-only button.
* @type {boolean}
* @default null
*/
showIcon?: boolean;

/**
* Icon to show to the left of or above the title, according to the button's 
*  RibbonButton.vertical.
*  
*  When specifying vertical = true, this icon will be stretched to 
*  the RibbonButton.largeIconSize unless a RibbonButton.largeIcon is specified.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* The size of the normal icon for this button.
* @type {number}
* @default "16"
*/
iconSize?: number;

/**
* Icon to show above the title when Orientation is "vertical".
*  
*  If a largeIcon is not specified, the RibbonButton.icon will be stretched to 
*  the RibbonButton.largeIconSize.
* @type {SCImgURL}
* @default null
*/
largeIcon?: SCImgURL;

/**
* The size of the large icon for this button. If RibbonButton.largeIcon is not specified, the
*  RibbonButton.icon will be stretched to this size.
* @type {number}
* @default "32"
*/
largeIconSize?: number;

/**
* If set to true, shows this button's Menu when a user clicks anywhere 
*  in the button, rather than specifically on the RibbonButton.menuIconSrc.
*  
*  Note that this property has a different meaning than StatefulCanvas.showMenuOnClick in the ancestor class StatefulCanvas.
* @type {boolean}
* @default false
*/
showMenuOnClick?: boolean;

/**
* Whether to show an Over version of the RibbonButton.menuIconSrc.
* @type {boolean}
* @default true
*/
showMenuIconOver?: boolean;

/**
* Whether to show a Down version of the RibbonButton.menuIconSrc.
* @type {boolean}
* @default false
*/
showMenuIconDown?: boolean;

/**
* Whether to show a Disabled version of the RibbonButton.menuIconSrc.
* @type {boolean}
* @default true
*/
showMenuIconDisabled?: boolean;

/**
* The menu to show when the RibbonButton.menuIconSrc is clicked.
*  
*  For a menu button with no menu (menu: null) the up/down arrow image can
*  be suppressed by setting
*  RibbonButton.showMenuIcon: false.
* @type {Menu}
* @default null
*/
menu?: Menu;

/**
* Allows you to specify an animation effect to apply to the menu when it is being shown.
*  Valid options are "none" (no animation), "fade", "slide" and "wipe".
*  If unspecified falls through to menu.showAnimationEffect
* @type {string}
* @default null
*/
menuAnimationEffect?: string;

/**
* The horizontal alignment of this button's menu, in relation to the button. When unset,
*  default behavior is to align the right edges of button and menu if the page is in RTL 
*  mode, and the left edges otherwise.
* @type {Alignment}
* @default null
*/
menuAlign?: Alignment;

/**
* The menu drops down below the menu button.
*  Set to false if the menu should appear above the menu button.
* @type {boolean}
* @default true
*/
showMenuBelow?: boolean;



/**
*  Sets a new Icon for this button after initialization.
* 
* @param {SCImgURL} URL of new icon
*/
setIcon(icon:SCImgURL): void;

/**
*  Returns the URL for the current icon.
* 
*/
getIcon(): SCImgURL;

/**
*  Sets a new Large-Icon for vertical buttons after initialization - synonymous with 
*  RibbonButton.setIcon for normal horizontal buttons.
* 
*/
setLargeIcon(): void;

/**
*  Notification method fired when a user clicks on the menuIcon on this RibbonButton. 
*  Return false to suppress the standard click handling code.
*  
* 
* 
*/
menuIconClick(): boolean;

/**
*  Notification method fired when a user clicks on the RibbonButton.icon in this 
*  RibbonButton. 
*  Return false to suppress the standard click handling code.
*  
* 
* 
*/
iconClick(): boolean;

/**
*  Notification method fired when a user clicks anywhere on this button. If the click occurred
*  directly on the Button.icon or the RibbonButton.menuIconSrc, 
*  the related notifications RibbonButton.iconClick and 
*  RibbonButton.menuIconClick are fired first and must return false to 
*  prevent this notification from firing.
*  
*  If a Menu is installed then, by default, it is only displayed when a 
*  user clicks on the RibbonButton.menuIconSrc. This can be altered via 
*  RibbonButton.showMenuOnClick.
* 
* 
*/
click(): boolean;

/**
*  The menu to show when the RibbonButton.menuIconSrc is clicked.
*  
*  For a menu button with no menu (menu: null) the up/down arrow image can
*  be suppressed by setting
*  RibbonButton.showMenuIcon: false. Note that 
*  showMenuIcon is updated automatically by calls to 
*  RibbonButton.setMenu.
* 
* @param {Menu} a menu to assign to this button
*/
setMenu(menu:Menu): void;

/**
*  Shows this button's RibbonButton.menu. Called automatically when a user clicks the 
*  RibbonButton.menuIconSrc.
* 
*/
showMenu(): boolean;



}
/**
* Creates a new RibbonButton
* 
* @param typeCheckedProperties {Partial<RibbonButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RibbonButton} The new instance
*/
class RibbonButton{
static create(typeCheckedProperties? : Partial<RibbonButton>, uncheckedProperties?: Object): RibbonButton;







}

/**
* EditProxy that handles RibbonButton objects when editMode is enabled.
*/
interface RibbonButtonEditProxy extends StatefulCanvasEditProxy {




}
/**
* Creates a new RibbonButtonEditProxy
* 
* @param typeCheckedProperties {Partial<RibbonButtonEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RibbonButtonEditProxy} The new instance
*/
class RibbonButtonEditProxy{
static create(typeCheckedProperties? : Partial<RibbonButtonEditProxy>, uncheckedProperties?: Object): RibbonButtonEditProxy;







}

/**
* A widget that groups RibbonButtons for use in 
*  RibbonBar.
*/
interface RibbonGroup extends VLayout {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "RibbonGroupEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Widget class for controls RibbonGroup.createControl by this 
*  RibbonGroup. Since RibbonGroup.newControlConstructor are created via the 
*  autoChild system, they can be further customized via the newControlProperties property.
* @type {Class}
* @default "RibbonButton"
*/
newControlConstructor?: Class;

/**
* Properties used by RibbonGroup.createControl when creating new
*  controls.
* @type {RibbonButton}
* @default null
*/
newControlDefaults?: RibbonButton;

/**
* CSS class applied to this RibbonGroup.
* @type {CSSStyleName}
* @default "ribbonGroup"
*/
styleName?: CSSStyleName;

/**
* The array of controls to show in this group.
* @type {Array<Partial<Canvas>>}
* @default null
*/
controls?: Array<Partial<Canvas>>;

/**
* HLayout autoChild that houses the RibbonGroup.label 
*  in which the RibbonGroup.title is displayed.
*  
*  This can be customized via the standard AutoChild pattern.
* @type {HLayout}
* @default null
*/
labelLayout?: HLayout;

/**
* SmartClient class for the RibbonGroup.label AutoChild.
* @type {string}
* @default "Label"
*/
labelConstructor?: string;

/**
* AutoChild Label used to display the 
*  RibbonGroup.title for this group.
*  
*  Can be customized via the standard AutoChild pattern, and various 
*  convenience APIs exist for configuring it after initial draw: see 
*  RibbonGroup.setShowTitle, 
*  RibbonGroup.setTitle, 
*  RibbonGroup.setTitleAlign,
*  RibbonGroup.setTitleHeight,
*  RibbonGroup.setTitleOrientation and
*  RibbonGroup.setTitleStyle.
* @type {Label}
* @default null
*/
label?: Label;

/**
* Controls the horizontal alignment of the group's 
*  RibbonGroup.title, within its 
*  RibbonGroup.label. Setting this
*  attribute overrides the default specified by 
*  RibbonBar.groupTitleAlign on the containing 
*  RibbonBar.
* @type {Alignment}
* @default "center"
*/
titleAlign?: Alignment;

/**
* CSS class applied to the RibbonGroup.label in this group.
* @type {CSSStyleName}
* @default "ribbonGroupTitle"
*/
titleStyle?: CSSStyleName;

/**
* By default, RibbonGroups are assigned a minimum width that allows the 
*  entire title to be visible. To prevent this behavior and have group-titles cut off 
*  when they're wider than the buttons they contain, set this attribute to false
* @type {boolean}
* @default true
*/
autoSizeToTitle?: boolean;

/**
* Controls the RibbonGroup.titleOrientation of 
*  this group's RibbonGroup.label. Setting this
*  attribute overrides the default specified by 
*  RibbonBar.groupTitleOrientation on the containing 
*  RibbonBar.
* @type {VerticalAlignment}
* @default "top"
*/
titleOrientation?: VerticalAlignment;

/**
* AutoChild properties for fine customization of the 
*  RibbonGroup.label.
* @deprecated * set these properties directly via the RibbonGroup.label

* @type {Label}
* @default null
*/
titleProperties?: Label;

/**
* Controls the height of the RibbonGroup.label in this group.
* @type {Integer}
* @default "18"
*/
titleHeight?: Integer;

/**
* HLayout autoChild that manages multiple RibbonGroup.columnLayout 
*  containing controls.
* @type {HLayout}
* @default null
*/
body?: HLayout;

/**
* SmartClient class for the body.
* @type {string}
* @default "HLayout"
*/
bodyConstructor?: string;

/**
* AutoChild VLayouts created automatically by groups. Each manages a single column of
*  child controls in the group. Child controls that support rowSpan may 
*  specify it in order to occupy more than one row in a single column. See 
*  RibbonGroup.numRows for related information.
* @type {VLayout}
* @default null
*/
columnLayout?: VLayout;

/**
* The number of rows of controls to display in each column. Each control will take one
*  row in a RibbonGroup.columnLayout by default, but those that 
*  support the feature may specify rowSpan to override that.
*  
*  Note that settings like this, which affect the group's layout, are not applied directly
*  if changed at runtime - a call to RibbonGroup.reflowControls 
*  will force the group to reflow.
* @type {number}
* @default "1"
*/
numRows?: number;

/**
* The height of rows in each column.
* @type {number}
* @default "26"
*/
rowHeight?: number;

/**
* The title text to display in this group's 
*  RibbonGroup.label.
* @type {string}
* @default null
*/
title?: string;



/**
*  Creates a new control and adds it to this RibbonGroup. The control is created using the 
*  autoChild system, according to the specified
*  RibbonGroup.newControlConstructor and the passed properties are
*  applied to it.
* 
* 
* @param {Partial<Canvas>} properties to apply to the new control
* @param {Integer=} the index at which to insert the new control
*/
createControl(properties:Partial<Canvas>, position?:Integer): void;

/**
*  Sets the RibbonGroup.title to display in this group's
*  RibbonGroup.label after initial draw.
*  
* 
* @param {string} The new title for this group
*/
setTitle(title:string): void;

/**
*  This method forcibly shows or hides this group's 
*  RibbonGroup.label after initial draw.
* 
* @param {boolean} should the title be shown or hidden?
*/
setShowTitle(showTitle:boolean): void;

/**
*  This method forcibly sets the horizontal alignment of the 
*  RibbonGroup.title, within the 
*  RibbonGroup.label, after initial draw.
* 
* @param {Alignment} the new alignment for the text, left or right
*/
setTitleAlign(align:Alignment): void;

/**
*  This method forcibly sets the RibbonGroup.titleStyle 
*  for this group's RibbonGroup.label after initial draw.
*  
* 
* @param {CSSStyleName} the CSS class to apply to the                 RibbonGroup.label.
*/
setTitleStyle(styleName:CSSStyleName): void;

/**
*  This method forcibly sets the 
*  RibbonGroup.titleOrientation of this group's 
*  RibbonGroup.label after initial draw.
* 
* @param {VerticalAlignment} the new orientation for the title, either bottom or top
*/
setTitleOrientation(orientation:VerticalAlignment): void;

/**
*  This method forcibly sets the height of this group's 
*  RibbonGroup.label after initial draw.
*  
* 
* @param {Integer} the new height for the RibbonGroup.label
*/
setTitleHeight(titleHeight:Integer): void;

/**
*  Return the RibbonGroup.columnLayout that contains the passed 
*  control.
*  
* 
* @param {Canvas} the control to find in this group
*/
getControlColumn(control:Canvas): Layout;

/**
*  Clears the array of controls and then adds the passed array to this group, 
*  creating new RibbonGroup.columnLayout as necessary, according to each
*  control's rowSpan attribute and the group's 
*  RibbonGroup.numRows attribute.
*  
* 
* @param {Array<Partial<Canvas>>} an array of widgets to add to this group
*/
setControls(controls:Array<Partial<Canvas>>): void;

/**
*  Forces this group to reflow following changes to attributes that affect layout, like 
*  RibbonGroup.numRows.
*  
* 
*/
reflowControls(): void;

/**
*  Adds an array of controls to this group, creating new 
*  RibbonGroup.columnLayout as necessary, according to each control's 
*  rowSpan value and the group's 
*  RibbonGroup.numRows value.
*  
* 
* @param {Array<Partial<Canvas>>} an array of widgets to add to this group
*/
addControls(controls:Array<Partial<Canvas>>): void;

/**
*  Adds a control to this RibbonGroup, creating a new 
*  RibbonGroup.columnLayout as necessary, according to the control's 
*  rowSpan value and the group's 
*  RibbonGroup.numRows value.
*  
* 
* @param {Canvas} a widget to add to this group
* @param {Integer=} optional insertion index for this control
*/
addControl(control:Canvas, index?:Integer): void;

/**
*  Removes a control from this RibbonGroup, destroying an existing 
*  RibbonGroup.columnLayout if this is the last widget in that column.
*  
* 
* @param {Canvas} a widget to remove from this group
*/
removeControl(control:Canvas): void;



}
/**
* Creates a new RibbonGroup
* 
* @param typeCheckedProperties {Partial<RibbonGroup>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RibbonGroup} The new instance
*/
class RibbonGroup{
static create(typeCheckedProperties? : Partial<RibbonGroup>, uncheckedProperties?: Object): RibbonGroup;







}

/**
* EditProxy that handles RibbonGroup objects when editMode is enabled.
*/
interface RibbonGroupEditProxy extends Class {




}
/**
* Creates a new RibbonGroupEditProxy
* 
* @param typeCheckedProperties {Partial<RibbonGroupEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RibbonGroupEditProxy} The new instance
*/
class RibbonGroupEditProxy{
static create(typeCheckedProperties? : Partial<RibbonGroupEditProxy>, uncheckedProperties?: Object): RibbonGroupEditProxy;







}

/**
* A simple subclass of RibbonButton that shows a menuIcon by default and implements 
*  showMenu().
*  
*  This class has RibbonButton.showMenuIcon set to true by 
*  default, and has a RibbonButton.menuIconClick handler which will show the specified 
*  RibbonButton.menu via a call to RibbonButton.showMenu.
*  This menuIconClick handler cancels default click behavior, so, if a user clicks the menu 
*  icon, any specified Canvas.click for the button as a whole will not fire.
*/
interface RibbonMenuButton extends RibbonButton {
/**
* Whether to show the RibbonButton.menuIconSrc which fires the 
*  RibbonButton.menuIconClick notification method when clicked.
* @type {boolean}
* @default true
*/
showMenuIcon?: boolean;





}
/**
* Creates a new RibbonMenuButton
* 
* @param typeCheckedProperties {Partial<RibbonMenuButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RibbonMenuButton} The new instance
*/
class RibbonMenuButton{
static create(typeCheckedProperties? : Partial<RibbonMenuButton>, uncheckedProperties?: Object): RibbonMenuButton;







}

/**
* Canvas to be used for Rich Text Editing
*/
interface RichTextCanvas extends Canvas {
/**
* If the user presses the "Tab" key, should focus be taken from this editor?
*  If set to false a "Tab" keypress will cause a Tab character
*  to be inserted into the text, and focus will be left in the edit area.
* @type {boolean}
* @default true
*/
moveFocusOnTab?: boolean;

/**
* The CSS class applied to this widget as a whole. The editable HTML within this widget
*  will appear with this styling applied to it, but it should be noted that this css class
*  is not part of that HTML, so applying the generated content to other components will not
*  pick up styling attributes defined in this class automatically.
* @type {CSSStyleName}
* @default "normal"
*/
styleName?: CSSStyleName;

/**
* Should this editor use a separate IFRAME with special cross-browser support for editing 
*  HTML content? In SmartClient versions 13.0 and later, this feature is switched off on
*  account of better modern browser support for contentEditable containers.
* @type {boolean}
* @default false
*/
useDesignMode?: boolean;

/**
* When true, applies style attributes in markup instead of presentation elements.
* @type {boolean}
* @default null
*/
styleWithCSS?: boolean;



/**
*  Increases the indent for the currently selected paragraph. Within a list, increases the
*  list level.
* 
*/
indentSelection(): void;

/**
*  Decreases the indent for the currently selected paragraph. Within a list, decreases the
*  list level or breaks out of the list.
* 
*/
outdentSelection(): void;

/**
*  Converts the selection to an ordered list. If the selection is within an unordered list,
*  the unordered list is converted to an ordered list.
* 
*/
convertSelectionToOrderedList(): void;

/**
*  Converts the selection to an unordered list. If the selection is within an ordered list,
*  the ordered list is converted to an unordered list.
* 
*/
convertSelectionToUnorderedList(): void;

/**
*  Applies the given ListProperties to the currently selected HTML list, if any.
*  If there is no list selected, then this method does nothing.
* 
* @param {ListProperties} the list configuration to apply
*/
applyListProperties(listProperties:ListProperties): void;



}
/**
* Creates a new RichTextCanvas
* 
* @param typeCheckedProperties {Partial<RichTextCanvas>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RichTextCanvas} The new instance
*/
class RichTextCanvas{
static create(typeCheckedProperties? : Partial<RichTextCanvas>, uncheckedProperties?: Object): RichTextCanvas;







}

/**
* RichTextEditing component. Provides a rich-text editing area along with UI for executing
*  rich-text commands on selected content.
*  
*  The HTML generated from this component may vary by browser, and, as with any HTML 
*  value created on the client, we recommend values be sanitized on the server before 
*  storing and displaying to other users.
*/
interface RichTextEditor extends VLayout {
/**
* The edit canvas created automatically for this RichTextEditor.
* @type {Canvas}
* @default null
*/
editArea?: Canvas;

/**
* Background color for the RichTextEditor.editArea.
* @type {string}
* @default "white"
*/
editAreaBackgroundColor?: string;

/**
* Should this editor use a separate IFRAME with special cross-browser support for editing 
*  HTML content? In SmartClient versions 13.0 and later, this feature is switched off on
*  account of better modern browser support for contentEditable containers.
* @type {boolean}
* @default false
*/
useDesignMode?: boolean;

/**
* When true, applies style attributes in markup instead of presentation elements.
* @type {boolean}
* @default null
*/
styleWithCSS?: boolean;

/**
* Initial value for the edit area.  Use getValue() and 
*  setValue() to update at runtime.
* @type {string}
* @default ""
*/
value?: string;

/**
* Layout used to contain all of the RichTextEditor.toolbar AutoChildren 
*  that contain the RichTextEditor.controlGroups.
* @type {Layout}
* @default null
*/
toolArea?: Layout;

/**
* Layout used to contain each of the RichTextEditor.controlGroups.
* @type {Layout}
* @default null
*/
toolbar?: Layout;

/**
* The background color for the toolbar.
* @type {string}
* @default "#CCCCCC"
*/
toolbarBackgroundColor?: string;

/**
* An array of control groups specifying which groups of controls should be included in the
*  editor tool area. The values of this array may be the name of a control group such as
*  one of the StandardControlGroups, a Canvas, or the special string "break"
*  which causes the subsequent control groups to continue onto a new line.
*  
*  
*  For each control group name, this[controlGroupName] should be defined as an array of 
*  ControlNames or Canvas instances. This allows the controls of a control
*  group to be customized.
*  
* @type {Array<any>}
* @default "["fontControls", "formatControls", "styleControls", "colorControls"]"
*/
controlGroups?: Array<any>;

/**
* Default text styling control group. Consists of an array of ControlNames
*  and/or Canvas instances. To display this group of controls for some RichTextEditor,
*  include "styleControls" in the RichTextEditor.controlGroups array.
* @type {Array<Partial<ControlName>>}
* @default "["boldSelection", "italicSelection", "underlineSelection"]"
*/
styleControls?: Array<Partial<ControlName>>;

/**
* The AutoChild SelectItem used for choosing the font to apply
*  to the current selection.
* @type {SelectItem}
* @default null
*/
fontSelectorItem?: SelectItem;

/**
* The AutoChild SelectItem used for choosing the font-size to 
*  apply to the current selection.
* @type {SelectItem}
* @default null
*/
fontSizeSelectorItem?: SelectItem;

/**
* The prompt for the built-in RichTextEditor.fontSelectorItem.
* @type {string}
* @default "Set Font..."
*/
fontSelectorPrompt?: string;

/**
* The prompt for the built-in RichTextEditor.fontSizeSelectorItem.
* @type {string}
* @default "Set Font Size..."
*/
fontSizeSelectorPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Make selection bold"
*/
boldSelectionPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Make selection italic"
*/
italicSelectionPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Make selection underlined"
*/
underlineSelectionPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Strike through selection"
*/
strikethroughSelectionPrompt?: string;

/**
* Default font control group. Consists of an array of ControlNames and/or
*  Canvas instances. To display this group of controls for some RichTextEditor,
*  include "fontControls" in the RichTextEditor.controlGroups array.
* @type {Array<Partial<ControlName>>}
* @default "["fontSelector", "fontSizeSelector"]"
*/
fontControls?: Array<Partial<ControlName>>;

/**
* ValueMap of css fontName properties to font name titles to display in the font selector 
*  if "fontSelector" is included in RichTextEditor.controlGroups 
*  for this editor.
*  Default value for this attribute:
*  {
*  &nbsp;&nbsp;"arial,helvetica,sans-serif": "Arial",
*  &nbsp;&nbsp;'courier new,courier,monospace': "Courier New",
*  &nbsp;&nbsp;'georgia,times new roman,times,serif': "Georgia",
*  &nbsp;&nbsp;'tahoma,arial,helvetica,sans-serif': "Tahoma",
*  &nbsp;&nbsp;'times new roman,times,serif': "Times New Roman",
*  &nbsp;&nbsp;'verdana,arial,helvetica,sans-serif': "Verdana",
*  &nbsp;&nbsp;"impact": "Impact"}
* @type {object}
* @default "{}"
*/
fontNames?: object;

/**
* ValueMap of css font size property values to font size titles to display in the font size
*  selector if "fontSizeSelector" is included in 
*  RichTextEditor.controlGroups.
*  Default value for this attribute:
*  {
*  &nbsp;&nbsp;"1": "1 (8 pt)",
*  &nbsp;&nbsp;"2": "2 (10 pt)",
*  &nbsp;&nbsp;"3": "3 (12 pt)",
*  &nbsp;&nbsp;"4": "4 (14 pt)",
*  &nbsp;&nbsp;"5": "5 (18 pt)",
*  &nbsp;&nbsp;"6": "6 (24 pt)",
*  &nbsp;&nbsp;"7": "7 (36 pt)"}
* @type {object}
* @default "{}"
*/
fontSizes?: object;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Copy Selection"
*/
copySelectionPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Cut Selection"
*/
cutSelectionPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Paste"
*/
pasteSelectionPrompt?: string;

/**
* Default text formatting control group. Consists of an array of ControlNames
*  and/or Canvas instances. To display this group of controls for some RichTextEditor,
*  include "formatControls" in the RichTextEditor.controlGroups array.
* @type {Array<Partial<ControlName>>}
* @default "["alignLeft", "alignRight", "alignCenter", "justify"]"
*/
formatControls?: Array<Partial<ControlName>>;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Left align selection"
*/
alignLeftPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Center selection"
*/
alignCenterPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Right align selection"
*/
alignRightPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Full justify selection"
*/
justifyPrompt?: string;

/**
* Control group for modifying text color / background color. 
*  Consists of an array of ControlNames and/or Canvas instances.
*  To display this group of controls for some RichTextEditor,
*  include "formatControls" in the RichTextEditor.controlGroups array.
* @type {Array<Partial<ControlName>>}
* @default "["color", "backgroundColor"]"
*/
colorControls?: Array<Partial<ControlName>>;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Set selection text color"
*/
colorPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Set selection background color"
*/
backgroundColorPrompt?: string;

/**
* Default HTML list control group. Consists of an array of ControlNames and/or
*  Canvas instances. To display this group of controls for some RichTextEditor,
*  include "bulletControls" in the RichTextEditor.controlGroups array.
* @type {Array<Partial<ControlName>>}
* @default "["indent", "outdent", "orderedList", "unorderedList", "listProperties"]"
*/
bulletControls?: Array<Partial<ControlName>>;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Increase Indent"
*/
indentPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Decrease Indent"
*/
outdentPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Convert to a numbered list"
*/
orderedListPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Convert to a bullet list"
*/
unorderedListPrompt?: string;

/**
* The prompt for the built-in ControlName control.
* @type {string}
* @default "Configure the list"
*/
listPropertiesPrompt?: string;

/**
* If the user presses the "Tab" key, should focus be taken from this editor?
*  If set to false a "Tab" keypress will cause a Tab character
*  to be inserted into the text, and focus will be left in the edit area.
* @type {boolean}
* @default true
*/
moveFocusOnTab?: boolean;

/**
* Dialog shown when the ControlName is pressed.
*  Provides options for the user to control formatting of lists.
* @type {ListPropertiesDialog}
* @default null
*/
listPropertiesDialog?: ListPropertiesDialog;

/**
* The warning message displayed in a dialog when a user tries to configure a list without
*  first putting the cursor in an appropriate place.
* @type {string}
* @default "Place the cursor within a list to configure it"
*/
listPropertiesWarningText?: string;



/**
*  Display a warning if Rich Text Editing is not fully supported in this browser.
*  Default behavior logs a warning to the developer console - Override this if a user-visible
*  warning is required 
* 
*/
doWarn(): void;

/**
*  Does this browser support the full RichTextEditor feature set.
*  Returns false for browsers in which some features are not natively supported
*  (Safari before version 3.1 and Opera before version 9.50).
* 
*/
richEditorSupported(): boolean;

/**
*  Setter for RichTextEditor.moveFocusOnTab.
* 
* @param {boolean} new value for moveFocusOnTab
*/
setMoveFocusOnTab(moveFocusOnTab:boolean): void;

/**
*  Retrieves the current value of the edit area.
* 
*/
getValue(): void;

/**
*  Updates the current value of the edit area.
* 
*/
setValue(): void;



}
/**
* Creates a new RichTextEditor
* 
* @param typeCheckedProperties {Partial<RichTextEditor>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RichTextEditor} The new instance
*/
class RichTextEditor{
static create(typeCheckedProperties? : Partial<RichTextEditor>, uncheckedProperties?: Object): RichTextEditor;







}

/**
* FormItem for rich text (HTML) editing. Makes use of a RichTextEditor as the 
*  editing interface.
*/
interface RichTextItem extends CanvasItem {
/**
* If the user presses the "Tab" key, should focus be taken from this editor?
*  If set to false a "Tab" keypress will cause a Tab character
*  to be inserted into the text, and focus will be left in the edit area.
* @type {boolean}
* @default true
*/
moveFocusOnTab?: boolean;

/**
* Should this item's value be saved in the form's values and hence returned from
*  DynamicForm.getValues?
*  
*  shouldSaveValue:false is used to mark formItems which do not correspond to
*  the underlying data model and should not save a value into the form's
*  DynamicForm.values. Example includes visual separators, password re-type fields,
*  or checkboxes used to show/hide other form items.
*  
*  A shouldSaveValue:false item should be given a value either via
*  FormItem.defaultValue or by calling
*  DynamicForm.setValue or 
*  FormItem.setValue. Providing a value via
*  DynamicForm.values or DynamicForm.setValues 
*  will automatically switch the item to shouldSaveValue:true.
*  
*  Note that 
*  
* - if an item is shouldSaveValue true, but has no name, a warning is logged, and 
*    shouldSaveValue will be set to false.
*  
* @type {boolean}
* @default true
*/
shouldSaveValue?: boolean;

/**
* Don't show the title for rich text items by default
* @type {boolean}
* @default false
*/
showTitle?: boolean;

/**
* By default RichTextItems take up an entire row
* @type {boolean}
* @default true
*/
startRow?: boolean;

/**
* By default RichTextItems take up an entire row
* @type {boolean}
* @default true
*/
endRow?: boolean;

/**
* By default RichTextItems take up an entire row
* @type {Integer | string}
* @default "*"
*/
colSpan?: Integer | string;

/**
* RichTextEditor.controlGroups to display for this editor.
*  Each controlGroup should be a property set either on this item or on the RichTextEditor
*  prototype and should be set to an array of ControlNames.
* @type {Array<Partial<string>>}
* @default null
*/
controlGroups?: Array<Partial<string>>;



/**
*  Setter for RichTextItem.moveFocusOnTab.
* 
* @param {boolean} new value for moveFocusOnTab
*/
setMoveFocusOnTab(moveFocusOnTab:boolean): void;



}
/**
* Creates a new RichTextItem
* 
* @param typeCheckedProperties {Partial<RichTextItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RichTextItem} The new instance
*/
class RichTextItem{
static create(typeCheckedProperties? : Partial<RichTextItem>, uncheckedProperties?: Object): RichTextItem;







}

/**
* Form item that renders as a blank row in the form layout.
*  Set RowSpacerItem.startRow to false to create a rowSpacer that simply
*  takes up every remaining column in the current row rather than starting a new row.
*/
interface RowSpacerItem extends SpacerItem {
/**
* we never show a separate title cell for separators
* @type {boolean}
* @default false
*/
showTitle?: boolean;

/**
* by default, separators span all remaining columns
* @type {Integer | string}
* @default "*"
*/
colSpan?: Integer | string;

/**
* these items are in a row by themselves by default
* @type {boolean}
* @default true
*/
startRow?: boolean;

/**
* these items are in a row by themselves by default
* @type {boolean}
* @default true
*/
endRow?: boolean;





}
/**
* Creates a new RowSpacerItem
* 
* @param typeCheckedProperties {Partial<RowSpacerItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {RowSpacerItem} The new instance
*/
class RowSpacerItem{
static create(typeCheckedProperties? : Partial<RowSpacerItem>, uncheckedProperties?: Object): RowSpacerItem;







}

/**
* ISC Abstraction for SVG controls
*/
interface SVG extends BrowserPlugin {
/**
* This attribute specifies the page the user should go to to get the plugin required to view
*  this SVG.
*  
*  The default pluginsPage is: "http://www.adobe.com/svg/viewer/install/"
*  
* @type {URL}
* @default "see below"
*/
pluginsPage?: URL;

/**
* Location from which to load the SVG.
*  
*  Note: if you do not specify a src value, ISC will load the special svg 'svgCanvas.svg' from
*  the helpers directory. This SVG is simply an empty root element - essentially a blank
*  canvas. You can use this feature to write components that programmatically manipulate the
*  SVG DOM without needing to ship placeholder SVG files.
*  
* @type {URL}
* @default "[HELPERS]svgCanvas.svg"
*/
src?: URL;





}
/**
* Creates a new SVG
* 
* @param typeCheckedProperties {Partial<SVG>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SVG} The new instance
*/
class SVG{
static create(typeCheckedProperties? : Partial<SVG>, uncheckedProperties?: Object): SVG;







}

/**
* User Interface component allowing a user to add a new saved search or edit an existing search. Automatically used by
*  SavedSearchItem and ListGrid.canSaveSearches; cannot be used directly and is documented only
*  for skinning and internationalization purposes.
*/
interface SavedSearchEditor extends VLayout {
/**
* In "normal" the search editor shows a FilterBuilder for editing criteria. In "grid" mode there is
*  only a field for naming (or renaming) a search, since the grid has built-in interfaces for editing
*  criteria.
* @type {SearchEditorMode}
* @default "normal"
*/
mode?: SearchEditorMode;

/**
* Whether the editor is in admin mode, in which case the SavedSearchEditor.sharedSearchCheckbox appears to allow
*  admins to define shared searches, and the SavedSearchEditor.defaultSearchCheckbox appears to enable an admin to
*  designate a search as the (shared) default.
*  
*  If SavedSearches.adminRole is defined and Authentication.hasRole indicates the current
*  user has the adminRole, isAdmin is true by default, otherwise false.
* @type {boolean}
* @default null
*/
isAdmin?: boolean;

/**
* Checkbox shown when SavedSearchEditor.isAdmin is true, allowing an admin to designate a search as an admin search
*  that all users will see. Titled with SavedSearchEditor.sharedSearchTitle.
* @type {CheckboxItem}
* @default null
*/
sharedSearchCheckbox?: CheckboxItem;

/**
* Title for the SavedSearchEditor.sharedSearchCheckbox.
* @type {string}
* @default "Shared view (all users will see it)"
*/
sharedSearchTitle?: string;

/**
* Checkbox shown when SavedSearchEditor.isAdmin is true and the current search is marked as a shared
*  search (via the SavedSearchEditor.sharedSearchCheckbox), allowing an admin to designate a search 
*  as the default search for users that have not chosen some other search as their default.
*  
*  If checked, the SavedSearches.setDefaultAdminSearchOperation will be invoked 
*  after the search being edited has been saved. This will typically be invoked as a
*  RPCManager.startQueue in the same transaction that adds or
*  updates the search being edited.
*  
*  See the Saving default searches section of the SavedSearches
*  for more information on marking searches as a default.
*  
*  Only ever appears if there is a SavedSearches.adminDefaultField in the SavedSearches.defaultDataSource.
*  
*  Titled with SavedSearchEditor.defaultSearchTitle.
* @type {CheckboxItem}
* @default null
*/
defaultSearchCheckbox?: CheckboxItem;

/**
* Title for the SavedSearchEditor.sharedSearchCheckbox.
* @type {string}
* @default "Mark as default"
*/
defaultSearchTitle?: string;

/**
* Label at the top of the interface, used to show instruction text, which is either SavedSearchEditor.addSearchText or
*  SavedSearchEditor.editSearchText depending on whether the user is editing a pre-existing search.
*  
*  For SavedSearchEditor.mode, when adding a new search, the instructions text is SavedSearchEditor.gridAddSearchText instead.
* @type {Label}
* @default null
*/
promptLabel?: Label;

/**
* @type {string}
* @default "Enter a name for your Saved View, then enter criteria below."
*/
addSearchText?: string;

/**
* @type {string}
* @default "Enter a name for your Saved View. The current configuration and filter criteria of the grid will be saved."
*/
gridAddSearchText?: string;

/**
* @type {string}
* @default "Enter a new name for your Saved View, and edit criteria below."
*/
editSearchText?: string;

/**
* TextItem where the user enters the name for the saved search.
* @type {TextItem}
* @default null
*/
searchNameItem?: TextItem;

/**
* FilterBuilder used to enter criteria.
*  
*  By default, FilterBuilder.topOperatorAppearance is set to "radio" (considered the simpler mode)
*  and FilterBuilder.showModeSwitcher is set true.
*  
*  If existing criteria cannot be displayed in simple ("radio") mode, advanced mode is automatically chosen.
* @type {FilterBuilder}
* @default null
*/
filterBuilder?: FilterBuilder;

/**
* Button used to save changes.
* @type {IButton}
* @default null
*/
saveButton?: IButton;

/**
* Confirmation message to show when a user attempts to save a view with a searchName,
*  componentID, and (if present) screenID / projectID which collides with another existing 
*  shared saved view.
*  
*  This is the message that will be shown in a confirmation dialog if, for example, a user
*  attempts to save a new view with the same name as an existing shared saved view for the same
*  grid.
* @type {string}
* @default "There is already a shared Saved View with this name - Overwrite?"
*/
overwriteSharedSearchConfirmationMessage?: string;

/**
* Confirmation message to show when a user attempts to save a view with a searchName,
*  componentID, and (if present) screenID / projectID which collides with another existing 
*  saved view.
*  
*  This is the message that will be shown in a confirmation dialog if, for example, a user
*  attempts to save a new view with the same name as an existing saved view for the same
*  grid.
* @type {string}
* @default "You already have a Saved View with this name - Overwrite?"
*/
overwriteSearchConfirmationMessage?: string;

/**
* Notification message to show to the user in a warn dialog when an attempted save fails
*  due to server validation error(s).
*  
*  Validation errors will be appended to this string in the format
*  fieldName: errorMessage.
* @type {string}
* @default "A validation failure occurred:&lt;br&gt;"
*/
validationFailedMessage?: string;

/**
* Button used to cancel changes.
* @type {IButton}
* @default null
*/
cancelButton?: IButton;





}
/**
* Creates a new SavedSearchEditor
* 
* @param typeCheckedProperties {Partial<SavedSearchEditor>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SavedSearchEditor} The new instance
*/
class SavedSearchEditor{
static create(typeCheckedProperties? : Partial<SavedSearchEditor>, uncheckedProperties?: Object): SavedSearchEditor;







}

/**
* Simple DynamicForm wrapper for a SavedSearchItem.
*  
*  This provides a standalone UI for creating, editing and applying saved searches for
*  a SavedSearchItem.targetComponent using the SavedSearches system.
*  
*  Note that a SavedSearchItem may also be used directly by embedding it into any DynamicForm.
*/
interface SavedSearchForm extends DynamicForm {
/**
* SavedSearchForms show a single column by default
* @type {number}
* @default "1"
*/
numCols?: number;

/**
* SavedSearchForms are sized to fit their single item by default
* @type {number}
* @default "1"
*/
height?: number;

/**
* SavedSearchForms are sized to fit their single item by default
* @type {Overflow}
* @default "visible"
*/
overflow?: Overflow;

/**
* Automatically generated SavedSearchItem for this form.
*  
*  This may be customized using the autoChildUsage.
* @type {SavedSearchItem}
* @default null
*/
savedSearchItem?: SavedSearchItem;

/**
* If set this property will override SavedSearchItem.canAddSearch on the 
*  automatically generated SavedSearchItem.
* @type {boolean}
* @default null
*/
canAddSearch?: boolean;

/**
* If set this property will override SavedSearchItem.addSearchValue on the 
*  automatically generated SavedSearchItem.
* @type {string}
* @default null
*/
addSearchValue?: string;

/**
* If set this property will override SavedSearchItem.addSearchIcon on the 
*  automatically generated SavedSearchItem.
* @type {SCImgURL}
* @default null
*/
addSearchIcon?: SCImgURL;

/**
* If set this property will override SavedSearchItem.targetEditsCriteria on the 
*  automatically generated SavedSearchItem.
* @type {boolean}
* @default null
*/
targetEditsCriteria?: boolean;

/**
* If set this property will override SavedSearchItem.canModifyProperty on the 
*  automatically generated SavedSearchItem.
* @type {boolean}
* @default null
*/
canModifyProperty?: boolean;

/**
* If set this property will override SavedSearchItem.editSearchHoverText on the 
*  automatically generated SavedSearchItem.
* @type {string}
* @default null
*/
editSearchHoverText?: string;

/**
* If set this property will override SavedSearchItem.removeSearchHoverText on the 
*  automatically generated SavedSearchItem.
* @type {string}
* @default null
*/
removeSearchHoverText?: string;

/**
* If set this property will override SavedSearchItem.copySearchHoverText on the 
*  automatically generated SavedSearchItem.
* @type {string}
* @default null
*/
copySearchHoverText?: string;

/**
* If set this property will override SavedSearchItem.markAsDefaultHoverText on the 
*  automatically generated SavedSearchItem.
* @type {string}
* @default null
*/
markAsDefaultHoverText?: string;

/**
* If set this property will override SavedSearchItem.canEditSearch on the 
*  automatically generated SavedSearchItem.
* @type {boolean}
* @default null
*/
canEditSearch?: boolean;

/**
* If set this property will override SavedSearchItem.canRemoveSearch on the 
*  automatically generated SavedSearchItem.
* @type {boolean}
* @default null
*/
canRemoveSearch?: boolean;

/**
* If set this property will override SavedSearchItem.canCopySearch on the 
*  automatically generated SavedSearchItem.
* @type {string}
* @default null
*/
canCopySearch?: string;

/**
* If set this property will override SavedSearchItem.targetComponent on the 
*  automatically generated SavedSearchItem.
* @type {Canvas}
* @default null
*/
targetComponent?: Canvas;

/**
* If set this property will override SavedSearchItem.targetDataSource on the 
*  automatically generated SavedSearchItem.
* @type {DataSource}
* @default null
*/
targetDataSource?: DataSource;

/**
* If set this property will override SavedSearchItem.confirmRemoval on the 
*  automatically generated SavedSearchItem.
* @type {boolean}
* @default null
*/
confirmRemoval?: boolean;

/**
* If set this property will override SavedSearchItem.confirmRemovalMessage on the 
*  automatically generated SavedSearchItem.
* @type {HTMLString}
* @default null
*/
confirmRemovalMessage?: HTMLString;

/**
* If set this property will override SavedSearchItem.adminRole on the 
*  automatically generated SavedSearchItem.
* @type {string}
* @default null
*/
adminRole?: string;

/**
* If set this property will override SavedSearchItem.adminRole on the 
*  automatically generated SavedSearchItem.
* @type {Record}
* @default null
*/
newRecordValues?: Record;

/**
* If set this property will override SavedSearchItem.saveLastSearch on the 
*  automatically generated SavedSearchItem.
* @type {boolean}
* @default null
*/
saveLastSearch?: boolean;

/**
* If set this property will override SavedSearchItem.saveDefaultSearch on the 
*  automatically generated SavedSearchItem.
* @type {boolean}
* @default null
*/
saveDefaultSearch?: boolean;

/**
* If set this property will override SavedSearchItem.storedState on the 
*  automatically generated SavedSearchItem.
* @type {SavedSearchStoredState}
* @default null
*/
storedState?: SavedSearchStoredState;

/**
* If set this property will override SavedSearchItem.offlineStorageKey on the 
*  automatically generated SavedSearchItem.
* @type {string}
* @default null
*/
offlineStorageKey?: string;

/**
* If set this property will override SavedSearchItem.title on the 
*  automatically generated SavedSearchItem.
* @type {string}
* @default null
*/
title?: string;

/**
* If set this property will override SavedSearchItem.hint on the 
*  automatically generated SavedSearchItem.
* @type {string}
* @default null
*/
hint?: string;



/**
*  Changes the SavedSearchForm.targetComponent to the passed in newTargetComponent.
* 
* @param {DataBoundComponent} the newTargetComponent
*/
setTargetComponent(newTargetComponent:DataBoundComponent): void;

/**
*  Event fired whenever a user changes the currently selected saved search, modifies a saved
*  search or adds a new saved search.
*  
*  If a SavedSearchForm.targetComponent has been specified, searchChanged automatically
*  applies the new search to the targetComponent unless the event is cancelled
*  by returning false.
* 
* 
* @param {Criteria} new criteria
* @param {string} search data
*/
searchChanged(newCriteria:Criteria, searchData:string): boolean;



}
/**
* Creates a new SavedSearchForm
* 
* @param typeCheckedProperties {Partial<SavedSearchForm>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SavedSearchForm} The new instance
*/
class SavedSearchForm{
static create(typeCheckedProperties? : Partial<SavedSearchForm>, uncheckedProperties?: Object): SavedSearchForm;







}

/**
* (SSI for short) Provides a UI for creating, editing and applying saved searches for a SavedSearchItem.targetComponent
*  using the SavedSearches system.
*  
*  Normally, a SavedSearchItem is just provided a SavedSearchItem.targetComponent, and all other
*  configuration comes from the central SavedSearches class by default. The
*  targetComponent must be a DataBoundComponent with a
*  DataBoundComponent.dataSource.
*  
*  Searches are applied to the target by calling method:DataBoundComponent.fetchData, or, for ListGrids,
*  by calling ListGrid.setViewState. If SavedSearchItem.saveLastSearch is set, the name of
*  the last search is automatically stored in browser localStorage, and will be applied to the
*  targetComponent as soon as saved searches are loaded.
*  
*  By default, SavedSearchItem acquires the
*  SavedSearches.defaultDataSource and uses it as
*  SelectItem.optionDataSource. The displayed value is the user's name for the search (from
*  SavedSearches.searchNameField) followed by a user-readable summary of the stored
*  search, derived from DataSource.getAdvancedCriteriaDescription, with a hover to show long values
*  that may be clipped.
*  
*  If SavedSearchItem.canAddSearch, the SavedSearchItem either shows a
*  FormItemIcon (SavedSearchItem.addSearchIcon) or a pickList entry for adding searches
*  (SavedSearchItem.addSearchValue). Either interface opens an EditSearchWindow.
*  
*  The PickList is automatically configured to show the search name plus the search description (via
*  SavedSearchItem.pickListFields), plus additional columns for icons for 
*  SavedSearchItem.editSearchField,
*  SavedSearchItem.removeSearchField, SavedSearchItem.copySearchField,
*  and SavedSearchItem.markAsDefaultField.
*  
*  Admin-configured searches are displayed after user-created searches, after a
*  SavedSearchItem.adminSeparatorRecord.
*  
*  SavedSearchItem.searchChanged fires when the user selects a new saved search, saves
*  changes to an existing saved search, or saves a new search. Note that SelectItem.valueField is set to
*  SavedSearches.componentIdField and SelectItem.displayField to SavedSearches.searchNameField.
*  
*  Saving new searches also causes SavedSearchItem.targetDataSource to be required. You
*  can set SavedSearchItem.newRecordValues to a set of fixed values that should be saved
*  whenever the user saves a new search; this can be used to save searches related to the current
*  user's userId, for example.
*  
*  The special interface that allows an admin to save shared searches appears if the user has the
*  SavedSearchItem.adminRole as determined by Authentication.hasRole.
*  
*  Saving full "viewState" for grids
*  
*  If the SavedSearchItem.targetComponent is a ListGrid or TreeGrid, 
*  the default behavior is to store the
*  complete ListGrid.viewState rather that just search criteria. If you want to store just criteria,
*  set SavedSearchItem.storedState to "criteria".
*  
*  Note: this feature requires
*  SmartClient Pro or better.
*/
interface SavedSearchItem extends SelectItem {
/**
* Properties for the separator record between locally saved and admin searches.
* @type {ListGridRecord}
* @default "{isSeparator:true}"
*/
adminSeparatorRecord?: ListGridRecord;

/**
* ListGridField shown in the pickList to allow users to remove existing searches. The field is type "icon"
*  and displays the skin's standard "remove" icon.
*  
*  An optional confirmation dialog is shown if SavedSearchItem.confirmRemoval is set.
*  
*  By default, if a record is an admin search (because it has no value for SavedSearches.userIdField or because
*  SavedSearches.adminField is true on the record), only an admin will be able to delete it.
*  
*  Alternatively, if you are not using the SavedSearchItem's admin features, specific records
*  can be marked as unable to be removed via SavedSearchItem.canModifyProperty.
* @type {ListGridField}
* @default null
*/
removeSearchField?: ListGridField;

/**
* Hover text that appeares over the +{removeSearchField}
* @type {string}
* @default "Remove view"
*/
removeSearchHoverText?: string;

/**
* ListGridField shown in the pickList to allow users to copy existing searches. The field is type "icon"
*  and displays the skin's standard "copy" icon.
* @type {ListGridField}
* @default null
*/
copySearchField?: ListGridField;

/**
* Hover text that appeares over the +{copySearchField}
* @type {string}
* @default "Copy view"
*/
copySearchHoverText?: string;

/**
* ListGridField shown in the pickList to allow users to designate which field is the default search. The
*  field is type "icon" and displays the skin's standard "checkbox" media.
* @type {ListGridField}
* @default null
*/
markAsDefaultField?: ListGridField;

/**
* Hover text that appeares over the +{markAsDefaultField}
* @type {string}
* @default "Set as default view"
*/
markAsDefaultHoverText?: string;

/**
* ListGridField shown in the pickList to allow admin users to designate which field is the admin-default search. 
*  The field is type "icon" and displays the skin's standard "checkbox" media.
* @type {ListGridField}
* @default null
*/
markAsAdminDefaultField?: ListGridField;

/**
* Hover text that appeares over the +{markAsAdminDefaultField}
* @type {string}
* @default "Set as default view for all users&lt;br&gt;&lt;br&gt;&lt;i&gt;[May be overridden by users' own preferences.]&lt;/i&gt;"
*/
markAsAdminDefaultHoverText?: string;

/**
* HTML string to append to the search title in the search name field if this is the 
*  default search.
* @type {HTMLString}
* @default " &lt;i&gt;[default]&lt;/i&gt;"
*/
defaultSearchNameSuffix?: HTMLString;

/**
* Whether a confirmation message should be shown when a user removes a saved search. The
*  message shown is the SavedSearchItem.confirmRemovalMessage.
* @type {boolean}
* @default true
*/
confirmRemoval?: boolean;

/**
* Message shown when SavedSearchItem.confirmRemoval is enabled and user attempts to
*  remove a saved search. The variable "${title}" is available providing the display name of the
*  saved search.
* @type {HTMLString}
* @default "Remove saved view '${title}'?"
*/
confirmRemovalMessage?: HTMLString;

/**
* Modal pop-up window shown when the user adds or edits a search, instance of
*  of EditSearchWindow.
* @type {EditSearchWindow}
* @default null
*/
editSearchWindow?: EditSearchWindow;

/**
* Whether the SavedSearchItem.targetComponent has built-in criteria editing or does not. True by default if the
*  target is a ListGrid or TreeGrid (but not CubeGrid).
*  
*  When the target has built-in criteria editing, the SavedSearchItem does not try to provide a
*  FilterBuilder-based criteria editing interface, so the SavedSearchEditor is simplified to
*  just allow naming of searches.
* @type {boolean}
* @default null
*/
targetEditsCriteria?: boolean;

/**
* This flag controls whether adding new searches is allowed.
* @type {boolean}
* @default false
*/
canAddSearch?: boolean;

/**
* Setting this property moves the SavedSearchItem.canAddSearch functionality from an icon next to the form item (SavedSearchItem.addSearchIcon) to the dropdown.
*  When set, the SavedSearchItem will look for this value in SelectItem.specialValues and use it as the trigger action for SavedSearchItem.canAddSearch.
* @type {string}
* @default null
*/
addSearchValue?: string;

/**
* Icon to be used to show the EditSearchWindow.
* @type {SCImgURL}
* @default "[SKINIMG]actions/add.png"
*/
addSearchIcon?: SCImgURL;

/**
* Optional name of a boolean field in the records returned by the optionDataSource,
*  where setting the field to false means the Record cannot be edited or removed by
*  the current user.
* @type {string}
* @default null
*/
canModifyProperty?: string;

/**
* The SavedSearchItem pickListFields are automatically generated and contain fields for
*  the search name plus the search description, plus additional columns for icons for editing, 
*  removal, copying existing searches, and choosing a default search.
* @type {Array<Partial<ListGridField>>}
* @default "[...]"
*/
pickListFields?: Array<Partial<ListGridField>>;

/**
* ListGridField shown in the pickList to allow users to edit existing searches. The field is type "icon"
*  and displays the skin's standard "edit" icon.
*  
*  Does not appear if the SavedSearchItem.targetEditsCriteria, since the simplest way of editing a
*  search is just to select it, use the grid's built-in criteria editing to change it, and save as new.
*  
*  Specific records can be marked as unable to be edited via SavedSearchItem.canModifyProperty.
* @type {ListGridField}
* @default null
*/
editSearchField?: ListGridField;

/**
* Hover text that appeares over the +{editSearchField}
* @type {string}
* @default "Save View Configuration"
*/
editSearchHoverText?: string;

/**
* Whether existing searches can be edited.
*  
*  If no explicit value is set, it will be defaulted to false if the
*  SavedSearchItem.targetEditsCriteria. Searches can be edited by simply selecting
*  them, using the grid's standard UI to edit the search, and then saving that as the original
*  search.
* @type {boolean}
* @default null
*/
canEditSearch?: boolean;

/**
* Whether existing searches can be removed.
* @type {boolean}
* @default true
*/
canRemoveSearch?: boolean;

/**
* Whether existing searches can be copied.
*  
*  If no explicit value is set, it will be defaulted to false if the
*  SavedSearchItem.targetEditsCriteria. Searches can be copied by simply selecting
*  them, using the grid's standard UI to edit the search, and then saving that as a new search.
* @type {boolean}
* @default null
*/
canCopySearch?: boolean;

/**
* Component that saved searches should apply to. When set, whenever SavedSearchItem.searchChanged fires,
*  the search is automatically applied to the targetComponent unless the
*  searchChanged event is cancelled.
*  
*  To avoid leaking local storage, saving searches will not be allowed for the target component
*  unless it specifies DataBoundComponent.savedSearchId, or an explicit
*  Canvas.getLocalId is present.
* @type {DataBoundComponent}
* @default null
*/
targetComponent?: DataBoundComponent;

/**
* DataSource that the saved searches are performed against.
*  
*  Normally auto-derived from SavedSearchItem.targetComponent, but can be specified
*  manually if no targetComponent is provided. In this case, SavedSearchItem.searchChanged would be
*  implement to apply criteria in a custom way.
* @type {DataSource}
* @default null
*/
targetDataSource?: DataSource;

/**
* Role to check for (via Authentication.hasRole to determine whether admin interfaces are shown.
*  If not explicitly set, at initialization time this will be defaulted to SavedSearches.adminRole.
* @type {string}
* @default null
*/
adminRole?: string;

/**
* Additional, fixed Record values that should be used every time a new search is saved. 
*  
*  This can be used to create user-specific saved searches by adding the userId as part of the
*  saved Record's value.
*  
*  Since this property is settable on the fly, you could also add an external interface that would
*  allow a user to save system-wide searches that are not associated with their userId. For
*  example, system-wide searches might have userId set to blank or to a special marker value, and
*  the FormItem.optionCriteria could be set so that the SavedSearchItem
*  shows system-wide as well as user-specific saved searches.
* @type {Record}
* @default null
*/
newRecordValues?: Record;

/**
* If set, the name of the last search is automatically stored in browser localStorage, 
*  and will be applied to the targetComponent as soon as saved searches are loaded.
* @type {boolean}
* @default false
*/
saveLastSearch?: boolean;

/**
* Optional explicit identifier for saved searches. See SavedSearchItem.saveDefaultSearch for details.
* @type {string}
* @default null
*/
savedSearchId?: string;

/**
* Works identically to ListGrid.saveDefaultSearch. The default is stored in browser
*  localStorage using the DataBoundComponent.savedSearchId of the SavedSearchItem.targetComponent,
*  or the AutoTest.getMinimalLocator if no savedSearchId was specified.
*  If no targetComponent is specified, the savedSearchId or minimal locator of the SavedSearchItem itself will be used.
*  
*  Note that if the targetComponent is ListGrid.autoFetchData, and saveDefaultSearch is true, the SavedSearchItem
*  automatically registers with the target component to prevent an automatic fetch with default criteria,
*  and then, after looking up the default search, will perform either the default search or perform a
*  standard autoFetch if no default search is found.
* @type {boolean}
* @default true
*/
saveDefaultSearch?: boolean;

/**
* Set to "criteria" if you want only criteria to be stored for ListGrids and TreeGrids instead of the full viewState of the component.
* @type {SavedSearchStoredState}
* @default null
*/
storedState?: SavedSearchStoredState;

/**
* Optional override of SavedSearches.defaultDataSource for this SavedSearchItem.
* @type {string}
* @default null
*/
savedSearchDS?: string;

/**
* Optional key used for local storage of saved searches by this component.
*  
*  If unset, the default SavedSearches.offlineStorageKey will be used for local storage instead.
*  
*  Has no effect if SavedSearchItem.savedSearchDS is set.
* @type {string}
* @default null
*/
offlineStorageKey?: string;

/**
* Title of this FormItem. Mote that the title is hidden by default for SavedSearchItem.
* @type {string}
* @default "Views"
*/
title?: string;

/**
* Text shown inside the field that serves as an indicator of what this field is for.
* @type {string}
* @default "Saved views..."
*/
hint?: string;



/**
*  Changes the SavedSearchItem.targetComponent to the passed in newTargetComponent.
* 
* @param {DataBoundComponent} the newTargetComponent
*/
setTargetComponent(newTargetComponent:DataBoundComponent): void;

/**
*  Event fired whenever a user changes the currently selected saved search, modifies a saved
*  search or adds a new saved search.
*  
*  If a SavedSearchItem.targetComponent has been specified, searchChanged automatically
*  applies the new search to the targetComponent unless the event is cancelled
*  by returning false.
* 
* 
* @param {Criteria} new criteria
* @param {Record} savedSearch record
*/
searchChanged(newCriteria:Criteria, searchData:Record): boolean;



}
/**
* Creates a new SavedSearchItem
* 
* @param typeCheckedProperties {Partial<SavedSearchItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SavedSearchItem} The new instance
*/
class SavedSearchItem{
static create(typeCheckedProperties? : Partial<SavedSearchItem>, uncheckedProperties?: Object): SavedSearchItem;







}

/**
* Data components such as ListGrid or TileGrid may allow users to set up their own 
*  search criteria within an application, either via an external SearchForm, or through built 
*  in UI such as the ListGrid.filterEditor.
*  
*  The "Saved Search" subsystem allows users to save out their criteria (and in some cases
*  user-configured view state) so when the application is reloaded they can 
*  easily restore a search or view from a previous session.
*  
*  Note: The SavedSearches feature requires
*  SmartClient Pro or better.
*  
*  User interface components for storing and retreiving saved searches are available via
*  the built in ListGrid features ListGrid.canSaveSearches and
*  ListGrid.savedSearchStoredState, as well as the explicit
*  the SavedSearchForm and SavedSearchItem classes.
*  
*  The SavedSearches class
*  SavedSearches is a "singleton" class that provides
*  central handling of storing and loading saved search data.
*  
*  You acquire the SavedSearches singleton via SavedSearches.get and you
*  can configure defaults via Class.addProperties or Class.changeDefaults.
*  
*  Saved searches are stored JSONEncoder in DataSource Records.
*  By default saved searches are stored in HTML5 browser localStorage via automatically
*  generated custom DataSources. In this mode the searches are only retained for a specific user 
*  on a particular machine, and searches cannot be shared with other users - but this approach
*  is sufficient for many applications, works out of the box, does not require any storage and
*  has no impact on scalability.
*  
*  For more capable saved search storage and retrieval, an explicit saved search dataSource
*  backed by permanent storage such as an SQL database table may be specified.
*  Typically this is done by setting up the SavedSearches.defaultDataSource.
*  This will store searches for all users and for every component and the same dataSource
*  may even be used for multiple applications.
*  For finer grained control, individual saved search dataSources may also be specified 
*  ListGrid.savedSearchDS.
*  
*  See SavedSearches.getSavedSearchDataSource for how to retrieve the dataSource for
*  a component.
*  
*  A SavedSearch dataSource has the following fields, some of which are optional, all of
*  which can be renamed as needed. (Click the links to see the purpose of each field):
*  
*  
* - SavedSearches.primaryKeyField (primary key field, typically of type "sequence")
*  
* - SavedSearches.dataField
*  
* - SavedSearches.searchNameField
*  
* - SavedSearches.componentIdField
*  
* - SavedSearches.projectIdField
*  
* - SavedSearches.screenIdField
*  
* - SavedSearches.applicationIdField
*  
* - SavedSearches.userIdField
*  
* - SavedSearches.adminField See "Admin-Configured Shared Searches" below.
*  
* - SavedSearches.defaultField, See "Default Searches" below.
*  
* - SavedSearches.adminDefaultField, See "Default Searches" below.
*  
*  
*  In your SDK, look for sc_SavedSearches.ds.xml for a sample SQL-based implementation of
*  saved-search (entirely declarative). Note that DataSource.cacheAllData is set to true - this
*  causes all searches applicable to a given user to be loaded in advance, the first time any component
*  requests saved searches. For most applications, this is the right approach, and is much better than
*  performing server requests each time a new component is shown that might have saved searches.
*  
*  Note that the SavedSearches system can be used to store any kind of component setting; in particular,
*  ListGrid.canSaveSearches used SavedSearches to store complete
*  ListGrid.getViewState, which includes field order & visibility, sorting and group
*  state in addition to search criteria.
*  
*  Admin-Configured Shared Searches
*  
*  Shared searches (also referred to as "Admin Searches") are special
*  searches that appear for all users, as pre-configured default saved searches.
*  While any user can create and edit their own personal saved searches, admin searches 
*  can only be created or edited by users with the SavedSearches.adminRole (see below).
*  
*  Admin searches are not available when no dataSource was configured to store the savedSearch data.
*  In this case a user's searches are stored to their machine rather than in a central database, 
*  and the concept of sharing admin-created searches does not apply.
*  
*  Admin searches are identified by either 
*  having the SavedSearches.userIdField null, or by the separate boolean 
*  SavedSearches.adminField set to true.
*  
*  UI components such as SavedSearchItem will offer a UI for adding, 
*  editing and removing admin searches if the user has the SavedSearches.adminRole,
*  as determined by Authentication.hasRole.
*  
*  In addition to this, for security reasons
*  the server should also enforce role-based authentication for creating and editing admin searches.
*  Note: the sample sc_SavedSearches.ds.xml dataSource has no "admin" field to
*  identify shared or "admin" searches. Instead records in this dataSource with 
*  "userId" set to null are considered to be shared searches.
*  The ownerIdField, ownerIdNullAccess and
*  ownerIdNullRole attributes enforce the appropriate restrictions for editing
*  admin searches -- these settings allow all users to view records with userId:null
*  but only users with the "admin" role may create or edit them.
*  
*  Default Searches
*  
*  Default searches are searches which will be automatically applied to a component on draw.
*  The SavedSearch subsystem supports two notions of "default" searches. A user may select
*  their personal default search, in which case the next time they load the page this search will be 
*  applied to the component automatically. Or an administrator may mark a shared admin search 
*  as the default for all users. In this case anyone who loads the application will see this search applied.
*  
*  If a user has a stored personal default search preference, this always takes precedence over any
*  admin-default search.
*  
*  Like any admin-searches, admin-default searches only apply when a dataSource was configured to
*  store the savedSearch data.
*  
*  Saving default searches
*  If SavedSearches.saveDefaultSearchToServer is false (the default), the user-default search 
*  preference will be stored in browser localStorage.
*  This is done because the user may choose an Admin Search as
*  their default search, and since Admin Search records are shared between all users, we can't mark the
*  Admin Search record in the database as the default for a specific user (at least not without more complex storage).
*  The drawback is that a user switching browsers or device will not have their default search preserved.
*  
*  If you want to persist default searches chosen by the user to permanent storage on the server, you can 
*  set SavedSearches.saveDefaultSearchToServer to true. In this case the SavedSearch dataSource
*  must include a boolean SavedSearches.defaultField. Typically this will need to be populated via custom server logic
*  when fetching records to indicate that the record in question is the chosen default for the current user.
*  When saveDefaultSearchToServer is true, the client will issue a custom update operation 
*  when the user wants to modify their default search - see SavedSearches.setDefaultUserSearchOperation.
*  
*  Admin default searches are stored on the server regardless of whether SavedSearches.saveDefaultSearchToServer is true.
*  The dataSource should include a boolean SavedSearches.adminDefaultField to indicate that an admin search record is
*  the shared-default search.
*  If an admin user updates a record to be the new shared default search, the client will issue a different
*  custom update operation - see SavedSearches.setDefaultAdminSearchOperation.
*  Note that we'd recommend marking this operation as requiresRole="admin" on the server, as
*  in the sample sc_SharedSearches.ds.xml dataSource configuration.
* 
*  Identifying components associated with Saved Searches
*  
*  Stored SavedSearch records will have the componentId field set to the identifier 
*  returned by SavedSearches.getSavedSearchId for a component.
*  By default this method returns a AutoTest.getMinimalLocator. This means
*  that if you assign a unique ID to a component that saves searches, and don't change that ID, stored 
*  saved searches will always be associated with the component.
*  
*  If you don't have a unique ID on the component, the minimal locator is not guaranteed to be consistent
*  across changes to your page layout. While Minimal Locators have a good chance of being unaffected
*  even if you refactor your UI, but there are cases where they may change and previously
*  stored searches would no longer be associated with the component.
*  See the AutoTest.getMinimalLocator docs for details.
*  
*  Alternatively you can bypass minimal locator system altogether and define an explicit
*  DataBoundComponent.savedSearchId. This allows you to define a stable identifier for storing saved 
*  searches without setting a component Canvas.ID.
*  
*  Note that a component savedSearch ID string is not guaranteed to be immutable over a widget's lifespan.
*  Autotest minimal locators can change in various cases such as changing page structure or binding the
*  component to a new DataSource.
*  Additionally developers may explicitly change the DataBoundComponent.savedSearchId at runtime.
*  
*  In either case, if the saved search ID changes, the component will be associated with a new set of 
*  saved searches.
*  There are cases where this is desirable. For example if a compnent is bound to one dataSource, and
*  then gets bound to a new dataSource where previously stored searches would no longer be applicable,
*  developers may explicitly change the savedSearchId for the component.
*  
*  Security Concerns
*  
*  If you provide your own SavedSearches.defaultDataSource, you should
*  enforce the following restrictions on the server:
*  
*  
* - Limiting fetch operations such that the logged-in user can only see their own searches (searches
*    where the current userId matches the SavedSearches.userIdField), and 
*    shared or "admin" searches (searches where the SavedSearches.adminField
*    is true, or if there is no admin field, searches where the userIdField value
*    is null).
*  
* - Ensuring that logged in users without the SavedSearches.adminRole can only 
*    add, edit and remove records where the SavedSearches.userIdField matches
*    their userId.
*  
* - For logged in users with the SavedSearches.adminRole, also allowing users to
*    add, update and remove admin searches. What this means in concrete terms depends on whether
*    the dataSource includes an SavedSearches.adminField.
*    If an adminField exists, admin users are exempt from userId restrictions when editing or
*    removing records where this field value is set to true. When adding a new record it is still
*    recommended the admin user's own userId should be stored in the userIdField.
*    If no adminField exists, admin searches are identified by having the userIdField
*    set to null. In this case admin users should be able to add, remove and edit 
*    records where the userIdField matches their own userId, or is null.
*  
*  
*  searchName field escaping: by default, any character is allowed in the
*  searchName field, and searchName are DataSourceField.escapeHTML
*  when displayed by built-in UI components. If you provide your own UI for saved searches, you should
*  escape the searchName field before displaying. Otherwise, users could render your
*  application partly non-functional by using special characters or inline script, or a malicious admin
*  could inject code into other user's browsers. If you prefer to just limit the searchName
*  field to non-special characters, you can just add a DataSourceField.validators 
*  to do this, and the
*  built-in UIs for saved search will handle the validation error as expected (block the save or edit and
*  tell the user what's wrong).
*  
*  If you fail to implement all of the above, then it will be possible for users to save searches for other
*  users or as admin searches, and those saved searches could have malicious code that is then injected into
*  other user's browsers.
*  
*  As noted above - the SavedSearches feature requires
*  SmartClient Pro or better.
*/
interface SavedSearches extends Class {
/**
* Should a user's default search preferences be stored to the server?
*  
*  Default user searches are normally saved in browser localStorage - see the "Default Searches"
*  section of the SavedSearches doc and also the docs for ListGrid.saveDefaultSearch.
*  
*  If you want to store the default search on the server instead, you can add an
*  OperationBinding of type "update" called SavedSearches.setDefaultUserSearchOperation to your
*  SavedSearches.defaultDataSource. This will be called with the PK value of the record to be made
*  the default search, plus a boolean true value for the SavedSearches.defaultField. The
*  expectation is that the search identified by the PK is marked as default and any other searches
*  previously marked as default are unmarked.
*  
*  Since Admin Search records are shared among all users, the expectation is that the underlying storage
*  actually stores default markers separately from admin searches, and the SavedSearches.defaultDataSource is
*  acting as a dsFacade.
*  
*  Updating a user's default search in the dataSource data should not modify any admin-default search.
* @type {boolean}
* @default false
*/
saveDefaultSearchToServer?: boolean;

/**
* Default DataSource used for persistence of saved searches. This may be overridden for individual components via
*  ListGrid.savedSearchDS.
*  
*  If no default dataSource is explicitly provided, the system will automatically generated a custom DataSource
*  for each component that stores saved searches in HTML5 localStorage. See SavedSearches.getLocalDataSource.
*  
*  This means the searches are only retained for that specific user on that
*  particular machine, and searches cannot be shared with other users. However, this approach does mean
*  that you don't have to provide storage for saved searches and saving searches has no impact on
*  scalability.
* @type {DataSource | string}
* @default null
*/
defaultDataSource?: DataSource | string;

/**
* If no explicit SavedSearches.defaultDataSource was provided, a custom HTML5 local storage
*  backed dataSource will be automatically created per component to store saved 
*  searches (see SavedSearches.getLocalDataSource).
*  
*  This property denotes a base storage key for these dataSources.
*  The SavedSearches.getSavedSearchId
*  for the component will be appended to this value to create the key to use when storing
*  serialized searches in local storage.
* @type {string}
* @default "isc_savedSearch_"
*/
offlineStorageKey?: string;

/**
* Prefix applied to a specified DataBoundComponent.savedSearchId by
*  SavedSearches.getSavedSearchId. This is required to handle the case where a component's
*  savedSearchId matches the ID of some other component.
* @type {string}
* @default "SSID:"
*/
savedSearchIDPrefix?: string;

/**
* Type: "string". Name dataSource field used for storing the saved search itself with at least 8k of storage (it's possible for searches to get yet larger, but rare).
* @type {string}
* @default "data"
*/
dataField?: string;

/**
* Type: "string". Name dataSource field used as the primary key.
*  
*  This is expected to be populated automatically when new search records are added to the
*  data set, so will typically be of FieldType.
* @type {string}
* @default "pk"
*/
primaryKeyField?: string;

/**
* Type: "string". Name dataSource field used for storing names of saved searches.
* @type {string}
* @default "searchName"
*/
searchNameField?: string;

/**
* Type: "string". Stores a unique ID for the component the saved search is associated with. 
*  This does not have to be the Canvas.ID and is usually a
*  AutoTestLocator
* @type {string}
* @default "componentId"
*/
componentIdField?: string;

/**
* Type: "string". Required because component IDs are not unique if components are loaded as 
*  RPCManager.loadScreen, especially reifyForDevelopers.
* @type {string}
* @default "projectId"
*/
projectIdField?: string;

/**
* Type: "string". Required because component IDs are not unique if components are loaded as 
*  RPCManager.loadScreen, especially reifyForDevelopers.
* @type {string}
* @default "screenId"
*/
screenIdField?: string;

/**
* Type: "string" (optional). This field
*  exists to allow a single DataSource to be used across multiple applications without colliding on
*  componentId. Set SavedSearches.applicationId to cause all search lookups to use
*  applicationId as criteria, and all newly saved searches to store that applicationId
* @type {string}
* @default "applicationId"
*/
applicationIdField?: string;

/**
* Type: "string" (optional). This stores the
*  userId of the user saving the search, populated from
*  Authentication.getCurrentUserId. This field is only optional in
*  the sense that you could instead build a DataSource that returns user-specific saved
*  searches through some other mechanism (for example, by inserting a userId value and forcing
*  userId criteria on the server side).
* @type {string}
* @default "userId"
*/
userIdField?: string;

/**
* (optional), type "boolean". Designates this search as an admin search, visible to all users
* @type {string}
* @default "admin"
*/
adminField?: string;

/**
* optional, type "boolean". Designates this search
*  as the default search for this user.
*  
*  This field is only required if SavedSearches.saveDefaultSearchToServer is true.
*  See the "Default Searches" section of the 
*  SavedSearches for more information.
* @type {string}
* @default "isDefault"
*/
defaultField?: string;

/**
* Type "boolean". Designates an admin search as the shared default search for all users.
*  
*  The user's chosen default search is normally stored separately; the adminDefaultField
*  exists as a way to mark a search as the default for a user that hasn't chosen one yet.
*  
*  See the "Default Searches" section of the 
*  SavedSearches for more information.
* @type {string}
* @default "isSharedDefault"
*/
adminDefaultField?: string;

/**
* The applicationId that will be saved to SavedSearches.applicationIdField
*  to disambiguate from other applications that use the same dataSource.
*  
*  If no applicationId was specified, SavedSearches.getApplicationId will return the current 
*  window.location.pathname by
*  default. This behavior can be turned off by setting
*  SavedSearches.allowNullApplicationId.
*  
*  The applicationId allows the same dataSource to be used to store savedSearches
*  for different applications. It also ensures that if SavedSearches.defaultDataSource 
*  was specified, and searches are being stored to SavedSearches.getLocalDataSource,
*  saved searches will be associated with the current application even if another application
*  running under the same domain/port has a component with the same SavedSearches.getSavedSearchId.
* @type {string}
* @default null
*/
applicationId?: string;

/**
* If SavedSearches.applicationId is not explicitly specified, it will be defaulted to
*  the current 
*  window.location.pathname.
*  
*  Set this flag to true to disable this behavior and allow SavedSearches to be stored with no
*  explicit applicationId.
* @type {boolean}
* @default false
*/
allowNullApplicationId?: boolean;

/**
* The name of the adminRole (used via +link{Authentication.hasRole()) to check to see if the current user has admin privileges.
* @type {string}
* @default "admin"
*/
adminRole?: string;

/**
* OperationBinding.operationId for the custom update operation to invoke when updating
*  the default user search if SavedSearches.saveDefaultSearchToServer is true.
*  
*  The update data for this operation will be the saved search record to mark as the user default
*  with the SavedSearches.defaultField set to indicate whether the saved search is being set or cleared.
* @type {string}
* @default "setDefaultUserSearch"
*/
setDefaultUserSearchOperation?: string;

/**
* OperationBinding.operationId for the custom update operation to invoke when updating
*  the default admin search.
*  
*  The update data for this operation will be the saved search record to mark as the shared default
*  with the SavedSearches.adminDefaultField set to indicate whether the saved search is being set or cleared.
*  
*  When marking a search as the new shared default, the dataSource implementation for this custom 
*  update operation must clear the adminDefaultField value on any previous shared default search.
*  The server should also use the dsResponse.relatedUpdates feature to notify the client 
*  that the previous default was unset.
* @type {string}
* @default "setDefaultAdminSearch"
*/
setDefaultAdminSearchOperation?: string;



/**
*  Retrieves the DataSource used for persistence of saved searches. 
*  
*  If a SavedSearches.defaultDataSource was explicitly specified, it will be returned, otherwise
*  this method returns null.
*  
*  To retrieve the dataSource used to store savedSearches for a specific component see 
*  SavedSearches.getSavedSearchDataSource
* 
* 
*/
getDefaultDataSource(): DataSource;

/**
*  Retrieves the DataSource used for persistence of saved searches for some component.
*  
*  If ListGrid.savedSearchDS is specified this will be returned.
*  Otherwise if an explicit SavedSearches.defaultDataSource was specified, it will be used.
*  
*  If no explicit dataSource was provided either at the component level or as a default for all 
*  SavedSearches, this method will return the result of SavedSearches.getLocalDataSource.
* 
* 
* @param {DataBoundComponent} component to retrieve the SavedSearch dataSource for
*/
getSavedSearchDataSource(component:DataBoundComponent): DataSource;

/**
*  Returns the identifier used to uniquely identify saved searches for some component.
*  
*  This is the value that will be set on the SavedSearches.componentIdField when storing
*  saved searches to a DataSource.
*  If there is no explicly specified dataSource for storing saved searches, it will
*  also be used to configure the offline storage key for data stored by the
*  automatically generated SavedSearches.getLocalDataSource 
*  for the component.
*  
*  The returned value is calculated as follows:
*  
* - If DataBoundComponent.savedSearchId is set, this method will return
*      the savedSearchIdPrefix + component.savedSearchId.
*    
* - Otherwise a AutoTest.getMinimalLocator will be returned.
*      Note that for components with an explicitly specified Canvas.ID, this
*      will just be the widget ID.
* 
*  Note: The returned ID string is not guaranteed to be immutable over a widget's lifespan.
*  
*  If no explicit component savedSearchId was set, the system will rely on the 
*  AutoTest minimal locator which can change in various cases
*  (a component may be reparented, or bound to a new DataSource, etc).
*  Since this property is used to identify the savedSearch records associated with this component,
*  when it changes a different set of saved searches will be available to the user.
*  
*  If you want to control this explicitly, you can do so by setting an explicit
*  DataBoundComponent.savedSearchId. Changing
*  the savedSearchId at runtime also allows you to deliberately
*  associate a component with a new set of SavedSearches depending on application state.
*  One common case where this might be desirable is if a component is bound to a dataSource
*  and you want to bind to a new DataSource where the savedSearch criteria would not be applicable.
* 
* 
* @param {Canvas} component to retrieve the identifier for
*/
getSavedSearchId(component:Canvas): string;

/**
*  This method returns an automatically generated custom DataSource to store saved searches for
*  a component in HTML5 localStorage. Note that this dataSource will only be used to store
*  savedSearches if there is no SavedSearches.defaultDataSource specified,
*  and if ListGrid.savedSearchDS is not set for the component.
*  
*  The offline storage key for the data will be generated by combining the SavedSearches.offlineStorageKey
*  with the component identifier retrieved from SavedSearches.getSavedSearchId.
*  
*  Note: The dataSources returned by this method will suppress logging their requests to the
*  devConsoleRPCTab by default. This is done so developers
*  can more easily see dataSource requests and responses that were explicitly initiated by application
*  code, to simplify debugging. However, if a developer has explicitly created a SavedSearch dataSource
*  either as the SavedSearches.defaultDataSource or at the 
*  ListGrid.savedSearchDS, requests to access and update SavedSearch data 
*  will be logged as with any other dataSource.
* 
* 
* @param {Canvas | string} Component to retrieve the dataSource for, or  saved search component identifier as returned by SavedSearches.getSavedSearchId
*/
getLocalDataSource(componentId:Canvas | string): DataSource;

/**
*  Retrieves the value to saves as the link{SavedSearches.applicationIdField,"applicationIdField"} value
*  for saved searches within this application.
*  
*  Returns SavedSearches.applicationId if specified, otherwise the current 
*  window.location.pathname
*  will be returned by default.
*  Set SavedSearches.allowNullApplicationId to suppress this behavior of defaulting to the location.pathname.
* 
* 
*/
getApplicationId(): string;

/**
*  Update the user's default search.
*  
*  This will invoke the SavedSearches.setDefaultUserSearchOperation if SavedSearches.saveDefaultSearchToServer is true,
*  otherwise it will persist the default into offline storage
* 
* 
* @param {DataBoundComponent} component being updated
* @param {boolean} whether the default is being set to true or false
* @param {Record} record containing details of the search to be updated
* @param {Callback} callback to invoke when the search has been updated. Takes no arguments.
*/
setDefaultUserSearch(component:DataBoundComponent, isDefault:boolean, searchRecord:Record, callback:Callback): void;

/**
*  Update the admin default search for some component
*  
*  This will invoke the SavedSearches.setDefaultAdminSearchOperation to update the admin search
*  record on the server.
* 
* 
* @param {DataBoundComponent} component being updated
* @param {boolean} whether the default is being set to true or false
* @param {Record} record containing details of the search to be updated
* @param {Callback} callback to invoke when the search has been updated. Takes no arguments.
*/
setDefaultAdminSearch(component:DataBoundComponent, isDefault:boolean, searchRecord:Record, callback:Callback): void;



}
/**
* Creates a new SavedSearches
* 
* @param typeCheckedProperties {Partial<SavedSearches>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SavedSearches} The new instance
*/
class SavedSearches{
static create(typeCheckedProperties? : Partial<SavedSearches>, uncheckedProperties?: Object): SavedSearches;




/**
*  Singleton accessor method.
* 
*/
static get(): SavedSearches;




}

/**
* A set of schema derived from the &lt;xsd:schema&gt; element in a WSDL or XML schema file
*  loaded by XMLTools.loadWSDL or XMLTools.loadXMLSchema.
*/
interface SchemaSet extends Class {
/**
* Namespace of this SchemaSet, derived from the targetNamespace
*  attribute of the &lt;schema&gt; element.
* @type {URI}
* @default null
*/
schemaNamespace?: URI;



/**
*  Get the schema definition of any complexType or element of complexType defined within
*  the &lt;schema&gt; element this SchemaSet represents.
* 
* 
* @param {string} name of the schema to retrieve
* @param {string=} optional type of schema to return, either "element" for               xs:element definitions only or "type" for xs:complexType                definitions. If unspecified, either will be returned,                with types preferred if names collide
*/
getSchema(schemaName:string, schemaType?:string): DataSource;



}
/**
* Creates a new SchemaSet
* 
* @param typeCheckedProperties {Partial<SchemaSet>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SchemaSet} The new instance
*/
class SchemaSet{
static create(typeCheckedProperties? : Partial<SchemaSet>, uncheckedProperties?: Object): SchemaSet;




/**
*  Retrieve a SchemaSet object by it's schemaNamespace.
* 
* 
* @param {string} uri from the "targetNamespace" attribute of the &lt;xsd:schema&gt; element from the XML Schema or WSDL file this SchemaSet was derived from.
*/
static get(schemaNamespace:string): SchemaSet;




}

/**
* The ScreenLoader component can be used to load
*  componentXML into a running application.
*  
*  A ScreenLoader is a VLayout, and can be provided anywhere a Canvas can be used: as a Tab
*  pane, and Layout member, etc. When a ScreenLoader draws, it shows a
*  ScreenLoader.loadingMessage if
*  ScreenLoader.showLoadingMessage, performs an RPC to the
*  RPCManager.screenLoaderURL to load the screen, if necessary, and
*  finally embeds the ScreenLoader.screenName within the layout.
*  
*  The last top-level UI component (Canvas subclass) defined by the screen is set as the
*  layout contents.
*  Top-level in this case means that the UI component is not contained in another UI component
*  as a member or child.
*  
*  The ScreenLoader relies on the XMLHttpRequest object which can be disabled by end-users in some
*  supported browsers. See platformDependencies for more information.
*/
interface ScreenLoader extends VLayout {
/**
* Name of screen to be loaded.
* @type {string}
* @default null
*/
screenName?: string;

/**
* A DataContextBinding to be applied to the loaded screen via
*  Canvas.setDataContext.
* @type {DataContextBinding}
* @default null
*/
dataContextBinding?: DataContextBinding;

/**
* Specifies the URL where ScreenLoaderServlet is installed. If not set,
*  RPCManager.screenLoaderURL is used.
* @type {URL}
* @default null
*/
screenLoaderURL?: URL;

/**
* Should a ScreenLoader.loadingMessage be displayed while screen is loading?
* @type {boolean}
* @default false
*/
showLoadingMessage?: boolean;

/**
* Message to show while the screen is loading if ScreenLoader.showLoadingMessage.
*  Use "&#36;{loadingImage}" to include Canvas.loadingImageSrc.
* @type {HTMLString}
* @default "Loading Screen...&amp;nbsp;${loadingImage}"
*/
loadingMessage?: HTMLString;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "ScreenLoaderEditProxy"
*/
editProxyConstructor?: SCClassName;



/**
*  Set the dataContextBinding property.
* 
* @param {DataContext} the new DataContext binding
*/
setDataContextBinding(binding:DataContext): void;

/**
*  Notification method fired when DataContext is bound on the embedded screen.
*  This can occur on the initial draw or by an explicit call to Canvas.setDataContext
*  either via this ScreenLoader or from other components including the screen itself.
*  
* 
*/
dataContextChanged(): void;



}
/**
* Creates a new ScreenLoader
* 
* @param typeCheckedProperties {Partial<ScreenLoader>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ScreenLoader} The new instance
*/
class ScreenLoader{
static create(typeCheckedProperties? : Partial<ScreenLoader>, uncheckedProperties?: Object): ScreenLoader;







}

/**
* EditProxy that handles ScreenLoader objects when editMode is enabled.
*/
interface ScreenLoaderEditProxy extends CanvasEditProxy {




}
/**
* Creates a new ScreenLoaderEditProxy
* 
* @param typeCheckedProperties {Partial<ScreenLoaderEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ScreenLoaderEditProxy} The new instance
*/
class ScreenLoaderEditProxy{
static create(typeCheckedProperties? : Partial<ScreenLoaderEditProxy>, uncheckedProperties?: Object): ScreenLoaderEditProxy;







}

/**
* Task that executes arbitrary code, either synchronous or asynchronous. Override the
*  ScriptTask.execute method to provide custom logic.
*/
interface ScriptTask extends Task {
/**
* Does this processElement pass through output from the last executed task
*  (i.e. transient state)?
*  
*  See taskInputExpression for details on the
*  transient state outputs.
*  
*  Note that this property does not affect the task at all but is an indicator
*  to the user and to the workflow editor of the behavior of the task as coded
*  (See Process.passThruTaskOutput).
* @type {boolean}
* @default false
*/
passThruOutput?: boolean;

/**
* Whether the script task is asynchronous. A synchronous task is expected to return data
*  directly from execute() and is considered complete once the execute() method exits.
*  
*  An asnychronous task is expected to start processing in execute(), and will not be
*  considered complete until either ScriptTask.setOutputData or ScriptTask.setOutputRecord is
*  called.
* @type {boolean}
* @default false
*/
isAsync?: boolean;



/**
*  Get the process executing this task instance.
* 
*/
getProcess(): Process;

/**
*  Get the inputs to this task as specified by Task.inputField.
*  
*  For a task with a Task.inputFieldList, use ScriptTask.getInputRecord
*  to get access to other inputs.
* 
*/
getInputData(): any;

/**
*  Set the task output as specified by Task.outputField.
*  
*  NOTE: for an ScriptTask.isAsync, calling
*  setOutputData() indicates the task is complete. For a task with
*  Task.outputFieldList, call ScriptTask.setOutputRecord instead.
* 
* @param {any} task output
*/
setOutputData(taskOutput:any): void;

/**
*  Get all inputs to the task as specified by the 
*  Task.inputFieldList, as a Record.
* 
*/
getInputRecord(): Record;

/**
*  Set all outputs of the task as specified by the
*  Task.outputFieldList, by providing a Record.
* 
* @param {Record} output record
*/
setOutputRecord(outputRecord:Record): void;

/**
*  Execute the task. 
* 
* @param {any} the task input
* @param {Record} the task input record if an inputFieldList was specified. See taskIO
*/
execute(input:any, inputRecord:Record): any;



}
/**
* Creates a new ScriptTask
* 
* @param typeCheckedProperties {Partial<ScriptTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ScriptTask} The new instance
*/
class ScriptTask{
static create(typeCheckedProperties? : Partial<ScriptTask>, uncheckedProperties?: Object): ScriptTask;







}

/**
* Class used for the draggable "thumb" of a scrollbar. Do not use directly; this class is
*  documented only for skinning purposes.
*/
interface ScrollThumb extends StretchImg {




}
/**
* Creates a new ScrollThumb
* 
* @param typeCheckedProperties {Partial<ScrollThumb>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ScrollThumb} The new instance
*/
class ScrollThumb{
static create(typeCheckedProperties? : Partial<ScrollThumb>, uncheckedProperties?: Object): ScrollThumb;







}

/**
* The Scrollbar widget implements cross-platform, image-based scrollbars that control the
*  scrolling of content in other widgets. Scrollbar widgets are created and displayed
*  automatically for widgets that require them, based on settings for Canvas.overflow.
*  
*  The scrollbar's appearance is based on a StretchImg for the "track", which consists
*  of two fixed size buttons and a stretchable center segment, and the ScrollThumb,
*  the draggable portion of the scrollbar, also a StretchImg, with an optional
*  StretchImg.showGrip.
*/
interface Scrollbar extends StretchImg {
/**
* The widget whose contents should be scrolled by this scrollbar. The scrollbar thumb
*  is sized according to the amount of visible vs. scrollable content in this widget.
* @type {Canvas}
* @default null
*/
scrollTarget?: Canvas;

/**
* If true, displays a corner piece at the bottom end of a vertical scrollbar, or the
*  right end of a horizontal scrollbar. This is typically set only when both horizontal
*  and vertical scrollbars are displayed and about the same corner.
* @type {boolean}
* @default false
*/
showCorner?: boolean;

/**
* The size of the square buttons (arrows) at the ends of this scrollbar. This overrides 
*  Canvas.scrollbarSize to set the width of a vertical scrollbar or the height of a
*  horizontal scrollbar. If not set it will default to Canvas.scrollbarSize.
* @type {number}
* @default null
*/
btnSize?: number;

/**
* If true, this scrollbar will automatically enable when the scrollTarget is
*  scrollable (i.e., when the contents of the scrollTarget exceed its clip size in the
*  direction relevant to this scrollbar), and automatically disable when the
*  scrollTarget is not scrollable. Set this property to false for full manual control
*  over a scrollbar's enabled state.
* @type {boolean}
* @default true
*/
autoEnable?: boolean;

/**
* If true, the thumb's appearance changes when it's clicked on.
* @type {boolean}
* @default false
*/
allowThumbDownState?: boolean;

/**
* If true, the thumb's appearance changes when the user rolls over it.
* @type {boolean}
* @default false
*/
allowThumbOverState?: boolean;

/**
* If true, the scrollbar uses a 5-segment rather than 3-segment image representation,
*  where the 3 interior image segments have the same state (Down, Over, etc), independent
*  of the two outermost image segments. 
*  
*  This allows certain advanced skinning designs where the track-as-such (space in which
*  the thumb may be dragged) has curved endcaps, and is also visually stateful (that is,
*  changes when the mouse goes down, without affecting the appearance of the outermost
*  segments).
* @type {boolean}
* @default false
*/
showTrackEnds?: boolean;

/**
* Should the track buttons that allow page scrolling be shown?
*  
* @type {boolean}
* @default true
*/
showTrackButtons?: boolean;

/**
* The minimum pixel size of the draggable thumb regardless of how large the scrolling
*  region becomes.
* @type {number}
* @default "12"
*/
thumbMinSize?: number;

/**
* The minimum pixel width of the track end segments (if enabled with showTrackEnds).
* @type {number}
* @default "12"
*/
trackEndWidth?: number;

/**
* The minimum pixel height of the track end segments (if enabled with showTrackEnds).
* @type {number}
* @default "12"
*/
trackEndHeight?: number;

/**
* Number of pixels the thumb is allowed to overlap the buttons at each end of the track.
*  Default prevents doubling of 1px borders. Set higher to allow media that shows curved
*  joins between the track button and ScrollThumb.
* @type {number}
* @default "1"
*/
thumbOverlap?: number;

/**
* Number of pixels the thumb is allowed to overlap the buttons at the start of the track.
*  Default prevents doubling of 1px borders. Set higher to allow media that shows curved
*  joins between the track button and ScrollThumb.
* @type {number}
* @default null
*/
startThumbOverlap?: number;

/**
* Number of pixels the thumb is allowed to overlap the buttons at the end of the track.
*  Default prevents doubling of 1px borders. Set higher to allow media that shows curved
*  joins between the track button and ScrollThumb.
* @type {number}
* @default null
*/
endThumbOverlap?: number;

/**
* Inset of the thumb relative to the track. An inset of N pixels means the thumb is 2N
*  pixels smaller in breadth than the track.
* @type {number}
* @default null
*/
thumbInset?: number;

/**
* Where are the skin images for the Scrollbar. This is local to the Page.getSkinDir.
* @type {SCImgURL}
* @default "images/Scrollbar/"
*/
skinImgDir?: SCImgURL;

/**
* URL for the corner image, a singular image that appears in the corner when both h and v
*  scrollbars are showing.
* @type {SCImgURL}
* @default "[SKIN]corner.gif"
*/
cornerSrc?: SCImgURL;

/**
* Allows the size of the corner segment to be set independently of the Scrollbar.btnSize.
* @type {Integer}
* @default null
*/
cornerSize?: Integer;

/**
* Base URL for the images used for the horizontal scrollbar track and end buttons. 
*  
*  See StretchImg.items for a general explanation of how this base URL is
*  transformed into various pieces and states.
*  
*  For a normal 3-segment track, the suffixes "_start", "_track" and "_end" are added to
*  this URL. The "start" and "end" images should appear to be buttons (the user can click
*  on these segments to scroll slowly). The "track" segment provides a background for the
*  space in which the thumb can be dragged, and can also be clicked on to scroll quickly.
*  
*  For a 5-segment track (Scrollbar.showTrackEnds:true), the suffixes are "_start", "_track_start",
*  "_track", "_track_end" and "_end".
* @type {SCImgURL}
* @default "[SKIN]hscroll.gif"
*/
hSrc?: SCImgURL;

/**
* Base URL for the images used for the vertical scrollbar track and end buttons. See
*  Scrollbar.hSrc for usage.
* @type {SCImgURL}
* @default "[SKIN]vscroll.gif"
*/
vSrc?: SCImgURL;

/**
* The StretchItem for the start of a scrollbar (the "scroll up" or "scroll left" button
*  image). The default is:
*  { name:"start", width:"btnSize", height:"btnSize" }
*  
* @type {StretchItem}
* @default "see below"
*/
startImg?: StretchItem;

/**
* The StretchItem for the start of a scrollbar track. The default is:
*  { name:"track_start", width:"trackStartSize", height:"trackStartSize" }
*  
* @type {StretchItem}
* @default "see below"
*/
trackStartImg?: StretchItem;

/**
* The StretchItem for the middle part of a scrollbar track, which usually takes up the majority
*  of the width or height of the scrollbar. The default is:
*  { name:"track", width:"*", height:"*" }
*  
* @type {StretchItem}
* @default "see below"
*/
trackImg?: StretchItem;

/**
* The StretchItem for the end of a scrollbar track. The default is:
*  { name:"track_end", width:"trackEndSize", height:"trackEndSize" }
*  
* @type {StretchItem}
* @default "see below"
*/
trackEndImg?: StretchItem;

/**
* The StretchItem for the end of a scrollbar (the "scroll down" or "scroll right" button
*  image). The default is:
*  { name:"end", width:"btnSize", height:"btnSize" }
*  
* @type {StretchItem}
* @default "see below"
*/
endImg?: StretchItem;

/**
* The StretchItem for the corner between vertical and horizontal scrollbars. The width
*  and height are determined automatically, so StretchItem.width and StretchItem.height
*  set on the cornerImg StretchItem are ignored. The default is:
*  { name:"corner" }
*  
* @type {StretchItem}
* @default "see below"
*/
cornerImg?: StretchItem;



/**
*      Sets or clears the scrollbar's scrollTarget. If no argument is provided, then the
*      scrollTarget will be set to the scrollbar itself.
* 
* 
* @param {Canvas=} target canvas to be scrolled
*/
setScrollTarget(newTarget?:Canvas): void;



}
/**
* Creates a new Scrollbar
* 
* @param typeCheckedProperties {Partial<Scrollbar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Scrollbar} The new instance
*/
class Scrollbar{
static create(typeCheckedProperties? : Partial<Scrollbar>, uncheckedProperties?: Object): Scrollbar;







}

/**
* A SearchForm is a DynamicForm specialized for a user to enter search criteria.
*  
*  All DynamicForm properties and methods work on SearchForm. SearchForm extends and
*  specializes DynamicForm for searching; for example, SearchForm sets
*  hiliteRequiredFields false by default because fields are typically not 
*  required in a search.
*  
*  A SearchForm may be bound directly to a ListGrid by applying it to 
*  ListGrid.searchForm. Doing this ensures that when 
*  SearchForm.search is invoked (from a SubmitItem 
*  or SearchForm.searchOnEnter), the grid will be filtered by the values 
*  from the search form. 
*  
*  Alternatively, developers may implement their own SearchForm.search.
*/
interface SearchForm {
/**
* This property is overridden in SearchForm to allow editing of dataSource fields marked as
*  canFilter:true by default.
* @type {string}
* @default "canFilter"
*/
canEditFieldAttribute?: string;

/**
* For editable fields with a specified FormItem.displayField and 
*  FormItem.optionDataSource, if the user selects a new value (typically from
*  PickList based item such as a SelectItem), should the selected displayValue be updated
*  on the record being edited in addition to the value for the actual item.
*  Note that this only applies for fields using 
*  FormItem.useLocalDisplayFieldValue.
*  
*  Overriden to be false for searchForms. It is typically not necessary
*  to have the display value as well as the data value be included in generated criteria
*  when a user selects a new value from a field with a specified
*  FormItem.displayField.
*  
*  See DynamicForm.storeDisplayValues for more information on this property.
* @type {boolean}
* @default false
*/
storeDisplayValues?: boolean;

/**
* Causes the SearchForm.search event to be triggered when the user presses the
*  Enter key in any field of this form.
*  
*  This is the same as the DynamicForm.saveOnEnter property of DynamicForm - setting 
*  either property to true will cause the SearchForm.search event to fire on Enter keypress.
* @type {boolean}
* @default false
*/
searchOnEnter?: boolean;

/**
* When creating a SelectItem within a Search Form for editing criteria for a field with a ValueMap,
*  should the SelectItem default to SelectItem.multiple?
*  
*  Note that for ListGrids showing a ListGrid.showFilterEditor, this property
*  will be derived from ListGrid.useMultiSelectForFilterValueMaps
* @type {boolean}
* @default true
*/
useMultiSelectForValueMaps?: boolean;

/**
* Delay in milliseconds between user changing the criteria in the form and the
*  SearchForm.criteriaChanged notification method being fired. Set to zero to
*  respond to criteria changes synchronously after DynamicForm.itemChanged.
* @type {Integer}
* @default "200"
*/
criteriaChangedDelay?: Integer;

/**
* If this attribute is true any DataSourceField.canFilter fields
*  specified on the dataSource will not be shown unless explicitly included in this component's
*  DataBoundComponent.fields
* @type {boolean}
* @default true
*/
showFilterFieldsOnly?: boolean;



/**
*  Notification event fired indicating that a user is attempting to perform a search.
*  This is fired when a SearchForm is submitted either from
*  a click on a SubmitItem in the form, or from an Enter keypress if SearchForm.searchOnEnter
*  or DynamicForm.saveOnEnter is true.
*  
* 
* @param {Criteria} the search criteria from the form
* @param {SearchForm} the form being submitted
*/
search(criteria:Criteria, form:SearchForm): void;

/**
*  Notification method fired when the criteria are modified in this SearchForm.
*  As the user edits values, this method will be fired after a
*  SearchForm.criteriaChangedDelay. 
* 
* 
* @param {Criteria} Current criteria of the form    (matches DynamicForm.getValuesAsCriteria)
* @param {SearchForm} the SearchForm being edited
*/
criteriaChanged(criteria:Criteria, form:SearchForm): void;



}
/**
* Creates a new SearchForm
* 
* @param typeCheckedProperties {Partial<SearchForm>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SearchForm} The new instance
*/
class SearchForm{
static create(typeCheckedProperties? : Partial<SearchForm>, uncheckedProperties?: Object): SearchForm;







}

/**
* Simple SectionHeader class based on a Label with an icon, skinnable via CSS.
*/
interface SectionHeader extends Label {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "SectionStackSectionEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Title to show for the section
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* If the title for this section header is too large for the available space, should the title be
*  clipped?
*  
*  This feature is supported only in browsers that support the CSS UI text-overflow
*  property (IE6+, Firefox 7+, Safari, Chrome, Opera 9+).
* @type {boolean}
* @default true
*/
clipTitle?: boolean;

/**
* If true and the title is clipped, then a hover containing the full title of this section header
*  is enabled.
* @type {boolean}
* @default true
*/
showClippedTitleOnHover?: boolean;

/**
* Custom controls to be shown on top of this section header.
*  
*  These controls are shown in the SectionHeader.controlsLayout.
*  
*  Note that this is an init-time property. If you need to dynamically change what 
*  controls are displayed to the user, we would recommend embedding the
*  controls in a Layout or similar container. 
*  This will allow you to show/hide or add/remove members at runtime
*  by manipulating the existing control(s) set up at init time.
* @type {Array<Partial<Canvas>>}
* @default null
*/
controls?: Array<Partial<Canvas>>;

/**
* A Layout containing specified SectionHeader.controls if any. Sets
*  Layout.membersMargin:5, Layout.defaultLayoutAlign:"center", and
*  RTL-sensitive Layout.align (right by default).
* @type {Layout}
* @default null
*/
controlsLayout?: Layout;

/**
* Base filename of the icon that represents open and closed states. The default settings
*  also change the icon for disabled sections, so a total of four images are required
*  (opened, closed, Disabled_opened, Disabled_closed).
*  
*  Not shown if SectionStackSection.canCollapse is false.
* @type {SCImgURL}
* @default "[SKIN]SectionHeader/opener.gif"
*/
icon?: SCImgURL;

/**
* CSS class for the section header.
* @type {CSSStyleName}
* @default "sectionHeader"
*/
baseStyle?: CSSStyleName;

/**
* By default doubleClicks are disabled for SectionHeaders. All mouse click
*  events will be handled as single clicks. Set this property to false
*  to enable standard double-click handling.
* @type {boolean}
* @default true
*/
noDoubleClicks?: boolean;



/**
*  For a SectionHeader embedded in a SectionStack, this method will return 
*  a pointer to the SectionStack in which this section header is
*  embedded.
* 
*/
getSectionStack(): SectionStack;

/**
*  Is the title of this section header clipped by SectionHeader.controls
*  or the edge of the header?
* 
*/
titleClipped(): boolean;

/**
*  Returns the HTML that is displayed by the default SectionHeader.titleHover
*  handler. Return null or an empty string to cancel the hover.
*  
* 
* @param {HTMLString} the HTML that would have been displayed by default
*/
titleHoverHTML(defaultHTML:HTMLString): HTMLString;

/**
*  Optional stringMethod to fire when the user hovers over this section header and the title is
*  clipped. If SectionHeader.showClippedTitleOnHover is true, the default behavior is to
*  show a hover canvas containing the HTML returned by SectionHeader.titleHoverHTML.
*  Return false to suppress this default behavior.
* 
*/
titleHover(): boolean;



}
/**
* Creates a new SectionHeader
* 
* @param typeCheckedProperties {Partial<SectionHeader>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SectionHeader} The new instance
*/
class SectionHeader{
static create(typeCheckedProperties? : Partial<SectionHeader>, uncheckedProperties?: Object): SectionHeader;







}

/**
* Header item for a collapsible section in a DynamicForm. Each SectionItem
*  is associated with a number of other items in the form, which will be shown or
*  hidden as a group when the section is expanded or collapsed. Clicking on a
*  SectionItem will expand or collapse the section.
*  
*  To make a form where only one section is expanded at a time, set
*  DynamicForm.sectionVisibilityMode to "mutex".
*/
interface SectionItem extends CanvasItem {
/**
* Section items show their value as title text for the section.
*  Therefore the simplest way to specify this text on the form item directly is via the
*  defaultValue attribute.
* @type {string}
* @default "Section Header"
*/
defaultValue?: string;

/**
* IDs of the items that should be considered a member of this section.
* @type {Array<Partial<GlobalId>>}
* @default null
*/
itemIds?: Array<Partial<GlobalId>>;

/**
* Whether this form section should initially be visible.
* @deprecated * As of SmartClient version 5.5, use SectionItem.sectionExpanded       instead.

* @type {boolean}
* @default true
*/
sectionVisible?: boolean;

/**
* Whether this form section should be initially collapsed. Can be set programmatically
*  via SectionItem.expandSection and SectionItem.collapseSection.
* @type {boolean}
* @default true
*/
sectionExpanded?: boolean;

/**
* Name of the Canvas subclass to use as a header that labels the section and allows
*  showing and hiding. The default class be skinned, or trivial subclasses created to
*  allow different appearances for SectionItems in different forms.
*  Very advanced developers can use the following information to create custom header
*  classes.
* @type {string}
* @default "SectionHeader"
*/
sectionHeaderClass?: string;

/**
* Whether this section header can be collapsed. If set false, suppresses open/close state
*  icon
* @type {boolean}
* @default true
*/
canCollapse?: boolean;

/**
* If true, the header for this Section will be included in the page's tab
*  order for accessibility.
*  May also be set at the DynamicForm level via 
*  DynamicForm.canTabToSectionHeaders.
*  
*  See accessibility.
* @type {boolean}
* @default null
*/
canTabToHeader?: boolean;



/**
*  Returns a boolean indicating whether this SectionItem is expanded.
* 
* 
*/
isExpanded(): boolean;

/**
*  Expands a section, showing all the items contained within the section.
* 
*/
expandSection(): void;

/**
*  Collapse a sectionItem, and hide all the items within the section (not including
*  the header).
* 
*/
collapseSection(): void;



}
/**
* Creates a new SectionItem
* 
* @param typeCheckedProperties {Partial<SectionItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SectionItem} The new instance
*/
class SectionItem{
static create(typeCheckedProperties? : Partial<SectionItem>, uncheckedProperties?: Object): SectionItem;







}

/**
* A container that manages a list of sections of widgets, each with a header. Sometimes
*  called an "Accordion".
*  
*  SectionStack can be configured so that only one section is visible at a time (similar to MS
*  Outlook's left-hand Nav), or to allow multiple sections to be visible and share the
*  available space. For further details, see SectionStack.visibilityMode.
*  
*  To ensure all sections are accessible, you may need to set
*  SectionStack.overflow to enable scrolling.
*/
interface SectionStack extends VLayout {
/**
* When AutoTest.getElement is used to parse locator strings generated by
*  AutoTest.getLocator, how should sections within this stack be
*  identified? By default if a section has a specified
*  SectionStackSection.name this will always be used. For
*  sections with no name, the following options are available:
*  
*  
* - "title" use the title as an identifier
*  
* - "index" use the index of the section in the sections array as an identifier
*  
*  
*  If unset, and the section has no specified name, default behavior is to
*  identify by title (if available), otherwise by index.
* @type {LocatorStrategy}
* @default null
*/
locateSectionsBy?: LocatorStrategy;

/**
* LocatorTypeStrategy to use when finding Sections within this section Stack.
* @type {LocatorTypeStrategy}
* @default null
*/
locateSectionsType?: LocatorTypeStrategy;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "SectionStackEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Normal Overflow settings can be used on layouts, for example, an
*  overflow:auto Layout will scroll if sections are resized to exceed the specified size,
*  whereas an overflow:visible Layout will grow to accommodate the resized sections.
* @type {Overflow}
* @default "hidden"
*/
overflow?: Overflow;

/**
* Default CSS style for the SectionStack as a whole.
* @type {CSSStyleName}
* @default "sectionStack"
*/
styleName?: CSSStyleName;

/**
* Widget to use for section headers. 
*  
*  Must be a subclass of either ImgSectionHeader or SectionHeader. The
*  default class used depends on the skin; SectionHeader is the simpler and
*  lighter-weight class and uses CSS styling rather than image-based styling, and is
*  recommended for most use cases.
*  
*  
*  
*  Very advanced developers can use the following information to create custom header
*  classes.
*  
*  The SectionStack will instantiate this class, giving the following properties on init:
*  
*  
* - layout: the SectionStack
*  
* - expanded: true or false
*  
* - hidden: true or false
*  
* - title: section title
*  
*  Whenever the section is hidden or shown, sectionHeader.setExpanded(true|false) will be
*  called if implemented.
*  
*  
*  If you override event handlers on your custom SectionHeader or radically change it's
*  structure such that the default event handling no longer works, you can call 
*  SectionStack.sectionHeaderClick to replicate the built-in expand/collapse
*  handling for clicking a section header.
* @type {string}
* @default "SectionHeader"
*/
sectionHeaderClass?: string;

/**
* Height of headers for sections.
* @type {number}
* @default "20"
*/
headerHeight?: number;

/**
* List of sections of components managed by this SectionStack.
* @type {Array<Partial<Partial<SectionStackSection>>>}
* @default null
*/
sections?: Array<Partial<Partial<SectionStackSection>>>;

/**
* Whether sections can be drag resized by the user dragging the section header.
*  
*  Note that, with canResizeSections:true, not all sections can be resized:
*  sections that contain only Button.autoFit components or that are
*  marked with SectionStackSection.resizeable will not be
*  resizeable.
* @type {boolean}
* @default true
*/
canResizeSections?: boolean;

/**
* SectionStacks provide the same default implementation of drag and drop interactions as
*  Layout.canDropComponents, except that members are added as items into
*  the section over which they're dropped.
*  
*  If you want to completely suppress the builtin drag and drop logic, but still receive drag
*  and drop events for your own custom implementation, set Canvas.canAcceptDrop to
*  true and canDropComponents to false on your
*  SectionStack.
* @type {boolean}
* @default true
*/
canDropComponents?: boolean;

/**
* Whether sections can be drag reordered by the user dragging the section header.
*  
*  Note that, with canReorderSections:true, sections with 
*  SectionStackSection.canReorder will not be
*  able to be drag-reordered (though their index may still be changed by dropping other
*  sections above or below them in the section stack).
* @type {boolean}
* @default false
*/
canReorderSections?: boolean;

/**
* If true, the headers for the sections (if shown) will be included in the page's tab
*  order for accessibility. 
*  May be overridden at the Section level via SectionStackSection.canTabToHeader
*  
*  If unset, section headers will be focusable if isc.setScreenReaderMode
*  
*  has been called.
*  See accessibility.
* @type {boolean}
* @default null
*/
canTabToHeaders?: boolean;

/**
* If an expanded or shown section expands past the current viewport and this property is
*  true, then the viewport will auto-scroll to fit as much of the section content into the
*  viewport without scrolling the top of the section out of the viewport.
* @type {boolean}
* @default true
*/
scrollSectionIntoView?: boolean;

/**
* Should any specified SectionStackSection.ID be applied to the generated SectionHeader widget
*  for the section as a widget ID? If set to false, SectionStackSection.ID will behave as a synonym for
*  SectionStackSection.name.
* @type {boolean}
* @default false
*/
useGlobalSectionIDs?: boolean;

/**
* If true, sections are animated during expand/collapse and addition/removal of
*  SectionItems is likewise animated.
* @type {boolean}
* @default null
*/
animateSections?: boolean;

/**
* Whether multiple sections can be expanded.
* @type {VisibilityMode}
* @default "mutex"
*/
visibilityMode?: VisibilityMode;

/**
* In SectionStack.visibilityMode
*  "mutex",
*  whether to allow the last remaining expanded section to be collapsed. If false, collapsing the
*  last open section will open the next one (wrapping around at the end).
* @type {boolean}
* @default true
*/
canCollapseAll?: boolean;

/**
* Size, in pixels, of indentation of all member items. Items will be offset
*  and reduced in width by this amount. Overridden by
*  SectionStack.itemStartIndent or SectionStack.itemEndIndent.
*  Setting itemIndent is equivalent to setting itemStartIndent to the same amount
*  and itemEndIndent to 0.
* @type {number}
* @default "0"
*/
itemIndent?: number;

/**
* Size, in pixels, of indentation of all member items relative to the start of
*  the alignment axis. For instance, for left-aligned members, 
*  itemStartIndent specifies indentation for every item from the left side of the
*  section stack. Overrides SectionStack.itemIndent.
* @type {number}
* @default undefined
*/
itemStartIndent?: number;

/**
* Size, in pixels, of indentation of all member items relative to the end of
*  the alignment axis. For instance, for left-aligned members, 
*  itemStartIndent specifies indentation for every item from the right side of the
*  section stack.
* @type {number}
* @default undefined
*/
itemEndIndent?: number;

/**
* Whether to show the Expand/Collapse controls in the headers of sections. If false, hides
*  the expand/collapse controls and, instead, treats a click anywhere on the header as if 
*  it were a click on the expand control.
* @type {boolean}
* @default true
*/
showExpandControls?: boolean;

/**
* Should sections be closeable if SectionStackSection.canClose is not
*  explicitly specified?
*  
*  Closeable sections show a SectionStack.closeSectionButton 
*  which will invoke SectionStack.closeSection when clicked.
* @type {boolean}
* @default false
*/
canCloseSections?: boolean;

/**
* Default icon src for the SectionStack.closeSectionButton for
*  SectionStackSection.canClose sections.
*  
*  May be overridden by SectionStackSection.closeIcon.
* @type {SCImgURL}
* @default "[SKIN]/actions/close.png"
*/
closeSectionIcon?: SCImgURL;

/**
* Pixel width/height for the SectionStack.closeSectionIcon.
* @type {number}
* @default "16"
*/
closeSectionIconSize?: number;

/**
* Automatically generated close icon button to show for 
*  SectionStackSection.canClose sections.
*  This component will be automatically added to the SectionStackSection.controls
*  for SectionStackSection.canClose sections.
*  
*  Icon source is derived from SectionStack.closeSectionIcon or
*  SectionStackSection.closeIcon and related properties.
* @type {ImgButton}
* @default null
*/
closeSectionButton?: ImgButton;

/**
* Constructor class for SectionStack.closeSectionButton autochildren.
* @type {string}
* @default "ImgButton"
*/
closeSectionButtonConstructor?: string;

/**
* Default properties for the SectionStack.closeSectionButton.
*  
*  The default configuration includes a click handler to invoke SectionStack.closeSection
* @type {Partial<ImgButton>}
* @default "{...}"
*/
closeSectionButtonDefaults?: Partial<ImgButton>;



/**
*  setter for SectionStack.animateSections
* 
* @param {boolean} Should expand/collapse of section be animated?
*/
setAnimateSections(shouldAnimate:boolean): void;

/**
*  Setter for SectionStack.visibilityMode.
* 
* @param {VisibilityMode} new visibilityMode setting. If this is "mutex" then all but the first expanded section is collapsed.
*/
setVisibilityMode(newVisibilityMode:VisibilityMode): void;

/**
*  Add a canvas as an item to a section.
* 
* @param {string | number} ID or index of the section to add item to
* @param {Canvas} Item to insert into the section
* @param {number} Index into section to insert item
*/
addItem(section:string | number, item:Canvas, index:number): void;

/**
*  Remove an item from a section.
* 
* @param {string | number} ID or index of the section to remove item from
* @param {Canvas} Item to remove
*/
removeItem(section:string | number, item:Canvas): void;

/**
*  Sets a new list of canvii as items into the specified section by removing the existing
*  items, then adding the new ones. Initial items for a section should be specified using
*  the property SectionStackSection.items.
* 
* @param {string | number} ID or index of the section to set items on
* @param {Array<Partial<Canvas>>} new items to add
*/
setItems(section:string | number, items:Array<Partial<Canvas>>): void;

/**
*  Set arbitrary properties for a particular section in this SectionStack. Properties will
*  be applied to the sectionHeader for the section.
*  
*  Note that where APIs exist to explicitly manipulate section properties these should be
*  used in preference to this method. For example, to add or remove items in a section use
*  SectionStack.addItem or SectionStack.removeItem. To change the title of
*  a section, use SectionStack.setSectionTitle.
*  
*  Also note that to modify properties of items within a section, call
*  the appropriate setter methods directly on the item you want to modify.
* 
* 
* @param {string | Integer} ID or index of the section to modify
* @param {Partial<SectionStackSection>} properties to apply to the section.
*/
setSectionProperties(section:string | Integer, properties:Partial<SectionStackSection>): void;

/**
*  Close a section. This method is invoked from
*  SectionStack.closeSectionButton
*  and will SectionStack.removeSection the section by default.
* 
* 
* @param {SectionStackSection} section to close
*/
closeSection(section:SectionStackSection): void;

/**
* 
*  Add a section to the SectionStack.
*  
*  You may want to apply Canvas.overflow: "auto" to your stack if users can
*  add an unlimited/ number of sections, so that they're all accessible.
* 
* 
* @param {Partial<SectionStackSection> | Array<Partial<Partial<SectionStackSection>>>} Initialization block         for the section or a list of initialization blocks to add.
* @param {number=} index for the new section(s) (if not specified, the section           will be added at the end of the SectionStack).
*/
addSection(sections:Partial<SectionStackSection> | Array<Partial<Partial<SectionStackSection>>>, position?:number): void;

/**
* 
*  Remove a section or set of sections from the SectionStack. The removed sections' header
*  and controls (if any) are automatically destroyed. A section's
*  SectionStackSection.items will also be destroyed if
*  SectionStackSection.destroyOnRemove is set on the section.
* 
* 
* @param {Integer | string | Array<Partial<Integer>> | Array<Partial<string>>} Section(s) to remove.         For this parameter, you can pass the position of the section in the          SectionStack, the name of the section, or a List of          section names or indices.
*/
removeSection(sections:Integer | string | Array<Partial<Integer>> | Array<Partial<string>>): void;

/**
* 
*  Returns a list of all SectionStackSection.name in the order in which 
*  they appear in the SectionStack.
* 
* 
*/
getSectionNames(): List;

/**
* 
*  Returns a list of all SectionStackSection.name in the order in which 
*  they appear in the SectionStack.
* 
* 
* @deprecated * in favor of SectionStack.getSectionNames.

*/
getSections(): List;

/**
* 
*  Reorder the sections by shifting the specified section to a new position
* 
* 
* @param {Integer | string} Section to move. You can pass the position            of the section in the SectionStack or the name of the section.
* @param {number} new position index for the section.
* @deprecated * As of SmartClient version 5.5, use SectionStack.moveSection.

*/
reorderSection(section:Integer | string, position:number): void;

/**
* 
*  Moves the specified section(s) to a new position in the SectionStack order. If you pass
*  in multiple sections, then each section will be moved to newPosition in the
*  order specified by the sections argument.
* 
* 
* @param {Integer | string | Array<Partial<Integer>> | Array<Partial<string>>} Section(s) to move.         For this parameter, you can pass the position of the section in the          SectionStack, the name of the section, or a List of section names/positions.
* @param {Integer} new position index for the section(s).
*/
moveSection(sections:Integer | string | Array<Partial<Integer>> | Array<Partial<string>>, position:Integer): void;

/**
*  
*  Shows a section or sections. This includes the section header and its items. If the
*  section is collapsed, only the header is shown. If the section is expanded, the section
*  header and all items are shown.
* 
* 
* @param {Integer | string | Array<Partial<Integer>> | Array<Partial<string>>} Section(s) to show. For this parameter, you can pass the position            of the section in the SectionStack, the name of the section, or a            List of section names / positions.
* @param {ShowSectionCallback=} callback to fire when the sections have been shown.
*/
showSection(sections:Integer | string | Array<Partial<Integer>> | Array<Partial<string>>, callback?:() => void): void;

/**
*  
*  Expands a section or sections. This action shows all the items assigned to the section.
*  If the section is currently hidden, it is shown first and then expanded. Calling this
*  method is equivalent to the user clicking on the SectionHeader of a collapsed section.
*  This method is called when the user clicks on SectionHeaders
*  to expand / collapse sections and so may be overridden to act as a notification method
*  for the user expanding or collapsing sections.
* 
* 
* @param {Integer | string | Array<Partial<Integer>> | Array<Partial<string>>} Section(s) to expand. For this parameter, you can pass the position            of the section in the SectionStack, the name of the section, or a            List of section names/positions.
* @param {ExpandSectionCallback=} callback to fire when the section has been expanded.
*/
expandSection(sections:Integer | string | Array<Partial<Integer>> | Array<Partial<string>>, callback?:() => void): void;

/**
*  
*  Search for a section that contains passed item.
* 
* 
* @param {Canvas} item to show
*/
sectionForItem(item:Canvas): SectionStackSection;

/**
*  Reveals the child Canvas passed in by expanding the section containing that child if it
*  is currently collapsed. If no section in this sectionStack contains the passed-in Canvas, 
*  this method has no effect
* 
* 
* @param {GlobalId | Canvas} the child Canvas to reveal, or its global ID
*/
revealChild(child:GlobalId | Canvas): void;

/**
*  
*  Hides a section or sections. This includes the section header and its items. The space
*  vacated by this action is reassigned to the nearest visible section item above this
*  section. If there are no visible section items above this section, the space is
*  reassigned to the nearest visible section item below this section.
* 
* 
* @param {Integer | string | Array<Partial<Integer>> | Array<Partial<string>>} Section(s) to hide. For this parameter, you can pass the position            of the section in the SectionStack, the name of the section, or a            List of section names / positions.
* @param {HideSectionCallback=} to fire when the section has been hidden
*/
hideSection(sections:Integer | string | Array<Partial<Integer>> | Array<Partial<string>>, callback?:() => void): void;

/**
*  
*  Collapse a section or sections. This action hides all the items assigned to the
*  section. Calling this method is equivalent to the user clicking on the SectionHeader of
*  an expanded section.
*  This method is called when the user clicks on SectionHeaders
*  to expand / collapse sections and so may be overridden to act as a notification method
*  for the user expanding or collapsing sections.
* 
* 
* @param {Integer | string | Array<Partial<Integer>> | Array<Partial<string>>} Section(s) to collapse. For this parameter, you can pass the position            of the section in the SectionStack, the name of the section, or a            List of section positions / names
* @param {CollapseSectionCallback=} callback to fire when the section has been collapsed
*/
collapseSection(sections:Integer | string | Array<Partial<Integer>> | Array<Partial<string>>, callback?:() => void): void;

/**
* 
*  Returns true if the specified section is visible, false if it is not. A section is
*  visible if it shows a header and the header is visible or if it has items and the first
*  item is visible.
* 
* 
* @param {Integer | string} Section for which you want to obtain visibility information.            For this parameter, you can pass the position of the section in the            SectionStack, or the name of the section.
*/
sectionIsVisible(section:Integer | string): boolean;

/**
* 
*  Returns the list of currently visible sections. The list items are section names.
* 
* 
*/
getVisibleSections(): List;

/**
* 
*  Returns true if the specified section is expanded, false if it is collapsed.
* 
* 
* @param {Integer | string} Section for which you want to obtain information.            For this parameter, you can pass the position of the section in the            SectionStack, or the name of the section.
*/
sectionIsExpanded(section:Integer | string): boolean;

/**
* 
*  Returns the list of currently expanded sections. The list items are section IDs.
* 
* 
*/
getExpandedSections(): List;

/**
*  Changes the title of a SectionHeader.
* 
* 
* @param {string | number} ID or index of the section whose title you want to change
* @param {string} new title for the SectionHeader
*/
setSectionTitle(section:string | number, newTitle:string): void;

/**
*  Return the SectionHeader for a section.
*  
*  This will be an instance of the SectionStack.sectionHeaderClass. Since different
*  SectionStacks may use different header classes, be careful about what APIs you rely on
*  for the section header unless you have explicitly set the
*  sectionHeaderClass. In particular, use APIs such as
*  SectionStack.setSectionTitle to manipulate headers indirectly wherever possible, as high
*  performance SectionStacks designed for very large numbers of sections may cache and
*  re-use headers or use other strategies that would make it invalid to store a pointer to
*  a section header, assume the header is a layout member, etc.
* 
* 
* @param {string | number} ID or index of the section for which you want the header
*/
getSectionHeader(section:string | number): SectionHeader;

/**
* 
*  Returns the position of the specified section in the SectionStack. The numbering is
*  zero-based. 
* 
* 
* @param {string} name of a section for which you want to obtain the position.
*/
getSectionNumber(sectionName:string): number;

/**
*  Method intended to be called by the sectionHeader when it is clicked on.
* 
* 
* @param {Canvas} the sectionHeader clicked on
*/
sectionHeaderClick(sectionHeader:Canvas): void;



}
/**
* Creates a new SectionStack
* 
* @param typeCheckedProperties {Partial<SectionStack>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SectionStack} The new instance
*/
class SectionStack{
static create(typeCheckedProperties? : Partial<SectionStack>, uncheckedProperties?: Object): SectionStack;







}

/**
* EditProxy that handles SectionStack objects when editMode is enabled.
*/
interface SectionStackEditProxy extends LayoutEditProxy {




}
/**
* Creates a new SectionStackEditProxy
* 
* @param typeCheckedProperties {Partial<SectionStackEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SectionStackEditProxy} The new instance
*/
class SectionStackEditProxy{
static create(typeCheckedProperties? : Partial<SectionStackEditProxy>, uncheckedProperties?: Object): SectionStackEditProxy;







}

/**
* EditProxy that handles SectionStackSection objects when editMode is enabled.
*/
interface SectionStackSectionEditProxy extends LabelEditProxy {




}
/**
* Creates a new SectionStackSectionEditProxy
* 
* @param typeCheckedProperties {Partial<SectionStackSectionEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SectionStackSectionEditProxy} The new instance
*/
class SectionStackSectionEditProxy{
static create(typeCheckedProperties? : Partial<SectionStackSectionEditProxy>, uncheckedProperties?: Object): SectionStackSectionEditProxy;







}

/**
* FormItem that allows picking between several mutually exclusive options via a select list.
*  
*  Options may be derived from a dataSource or a valueMap.
*  
*  Note that to select the first option as a default value for the item,
*  SelectItem.defaultToFirstOption may be set.
*/
interface SelectItem extends FormItem, PickList {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "SelectItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Height of the FormItem. Can be either a number indicating a fixed height in pixels, a
*  percentage indicating a percentage of the overall form's height, or "*" indicating take
*  whatever remaining space is available. See the formLayout overview for details.
*  
*  
*  For form items having a SelectItem.showPickerIcon (e.g. SelectItem,
*  ComboBoxItem) and SpinnerItems, if there is no explicit 
*  FormItem.pickerIconHeight, the pickerIcon will be sized to match the available
*  space based on the specified item height.
*  Note that if spriting is being used, and the image to be displayed in these icons
*  is specified 
*  using css properties such as background-image, background-size,
*  changing this value may result in an unexpected appearance as the image will not
*  scale.
*  Scaleable spriting can be achieved using the SCSpriteConfig format.
*  See the section on spriting in the skinning for 
*  further information.
*  Alternatively, the SelectItem.pickerIconStyle could be changed to a
*  custom CSS style name, and in the case of SpinnerItems, 
*  the FormItemIcon.baseStyle and
*  FormItemIcon.src of the SpinnerItem.increaseIcon
*  and SpinnerItem.decreaseIcon AutoChildren could be customized.
*  
*  Note that when FormItem is rendered as read-only with readOnlyDisplay as "static"
*  the property FormItem.staticHeight is used instead.
* @type {number}
* @default "19"
*/
height?: number;

/**
* If this item's value is set (via SelectItem.setValue or similar) to a value which
*  is not present in the ValueMap, should the value be rejected.
*  
*  If set to false the setValue() call will have no effect if the value
*  is not a valid option.
*  
*  If set to true the item's value will be updated to the new value, and
*  the value will be added to the set of options displayed in the pick-list until the
*  next call to SelectItem.setValueMap or SelectItem.setValue.
*  
*  Exception: If the value is set to null but there is no null entry in the
*  valueMap for this item, setting addUnknownValues to true will not cause
*  a null option to show up at the top of the select item pickList. Whether an empty
*  option is shown in the pickList is governed by SelectItem.allowEmptyValue
*  instead.
*  
*  Note that this property has no effect if the selectItem has a specified
*  SelectItem.optionDataSource. If SelectItem.setValue is called on a databound SelectItem
*  and the new value does not match any loaded options, the value will be accepted,
*  but not added to the options displayed in the pickList. Also note that if a
*  SelectItem.displayField exists, a fetch will be performed in an attempt to retrieve
*  a valid display value, as described under FormItem.fetchMissingValues. 
*  If specified the FormItem.loadingDisplayValue will be displayed while the 
*  fetch is in progress, and then the real value (mapped to a display field value if
*  a matching record was found) will be displayed when the fetch completes.
* @type {boolean}
* @default true
*/
addUnknownValues?: boolean;

/**
* Static default value for this SelectItem. To default to the first option use
*  SelectItem.defaultToFirstOption instead.
* @type {any}
* @default null
*/
defaultValue?: any;

/**
* Select the first option as the default value for this SelectItem. 
*  
*  If options are derived from a dataSource, the first value returned by the server will be
*  used, otherwise the first value in the valueMap. Note that setting this property to
*  true will trigger a fetch at soon as the form is created, because the form will try to
*  establish a default value at that time.
*  
*  If enabled, this setting overrides SelectItem.defaultValue and
*  SelectItem.defaultDynamicValue.
* @type {boolean}
* @default false
*/
defaultToFirstOption?: boolean;

/**
* If true, multiple values may be selected.
*  
*  The SelectItem will either render as a drop-down allowing multiple selections, or a
*  multi-row list of options similar to a small headerless ListGrid, based on the
*  MultipleAppearance setting.
*  
*  The logical value of the formItem, as retrieved by FormItem.getValue
*  and set via FormItem.setValue, is an Array of Strings reflecting the
*  selected values.
*  
*  When this value is true, we disable doubleClick events by default, instead issuing two
*  single clicks by forcibly setting Canvas.noDoubleClicks.
*  If you need to work with doubleClick events, you can disable this default behavior by
*  explicitly setting formItem.pickListProperties.noDoubleClicks: false.
*  
*  Note: multiple:true SelectItems with multipleAppearance:"grid" do not
*  currently support optionDataSource binding. You can get around this by calling
*  DataSource.fetchData directly and calling
*  List.getValueMap to obtain a valueMap.
*  
*  If the multiple attribute is not explicitly specified, it will default
*  to false, unless thie item has a specified 
*  FormItem.valueMap and is part of a SearchForm 
*  with SearchForm.useMultiSelectForValueMaps set to true.
* @type {boolean}
* @default null
*/
multiple?: boolean;

/**
* How should items with SelectItem.multiple set to 'true' be displayed?
* @type {MultipleAppearance}
* @default "picklist"
*/
multipleAppearance?: MultipleAppearance;

/**
* For SelectItem.optionDataSource items, this property will be passed
*  to the generated ResultSet data object for the pickList as ResultSet.useClientFiltering.
*  Setting to false will disable filtering on the client and ensure criteria are
*  always passed to the DataSource directly.
* @type {boolean}
* @default null
*/
useClientFiltering?: boolean;

/**
* Should we show a special 'picker' FormItemIcon for this form item? Picker
*  icons are customizable via FormItem.pickerIconProperties. By default
*  they will be rendered inside the form item's FormItem.controlStyle
*  area. By default clicking the pickerIcon will call FormItem.showPicker.
* @type {boolean}
* @default true
*/
showPickerIcon?: boolean;

/**
* Text to display when this form item has a null or undefined value.
*  
*  If the formItem has a databound pickList, and its FormItem.displayField or
*  FormItem.valueField (if the former isn't set) has an undefined
*  ListGridField.emptyCellValue setting, that field's
*  emptyCellValue will automatically be set to the emptyDisplayValue.
* @type {HTMLString}
* @default "&amp;nbsp;"
*/
emptyDisplayValue?: HTMLString;

/**
* Base CSS class name for a form item's text box element.
*  
*  See formItemStyling for an overview of formItem styling, and 
*  the CompoundFormItem_skinning discussion for special
*  skinning considerations.
*  
*  If the textBoxStyle is changed at runtime, FormItem.updateState
*  must be called to update the visual state of this item.
* @type {FormItemBaseStyle}
* @default ""selectItemText", [IRA]"
*/
textBoxStyle?: FormItemBaseStyle;

/**
* Base CSS class name for a form item's "control box". This is an HTML element
*  which contains the text box and picker icon for the item.
*  
*  See FormItem.alwaysShowControlBox for details on when the control box
*  is written out.
*  
*  See formItemStyling for an overview of formItem styling, and 
*  the CompoundFormItem_skinning discussion for special
*  skinning considerations.
* @type {FormItemBaseStyle}
* @default ""selectItemControl", [IRA]"
*/
controlStyle?: FormItemBaseStyle;

/**
* Base CSS class name for a form item's picker icon cell. If unset, inherits from
*  this item's SelectItem.controlStyle.
* @type {FormItemBaseStyle}
* @default "selectItemPickerIcon"
*/
pickerIconStyle?: FormItemBaseStyle;

/**
* When this item receives focus, should it be re-styled to indicate it has focus?
*  
*  See formItemStyling for more details on formItem styling.
* @type {boolean}
* @default "true, [IRWA]"
*/
showFocused?: boolean;

/**
* If SelectItem.showPickerIcon is true for this item, this property governs the
*  size of the picker icon. If unset, the picker icon will be sized as a square to fit in the
*  available height for the icon.
*  
*  Note that if spriting is being used, and the image to be displayed is specified 
*  using css properties such as background-image, background-size,
*  changing this value may result in an unexpected appearance as the image will not
*  scale.
*  Scaleable spriting can be achieved using the SCSpriteConfig format.
*  See the section on spriting in the skinning for 
*  further information.
* @type {Integer}
* @default null
*/
pickerIconWidth?: Integer;

/**
* If SelectItem.showPickerIcon is true for this item, this property governs the
*  size of the picker icon. If unset, the picker icon will be sized as a square to fit in the
*  available height for the icon.
*  
*  Note that if spriting is being used, and the image to be displayed is specified 
*  using css properties such as background-image, background-size,
*  changing this value may result in an unexpected appearance as the image will not
*  scale.
*  Scaleable spriting can be achieved using the SCSpriteConfig format.
*  See the section on spriting in the skinning for 
*  further information.
* @type {Integer}
* @default null
*/
pickerIconHeight?: Integer;

/**
* If SelectItem.showPickerIcon is true for this item, this property governs the
*  FormItemIcon.src of the picker icon image to be displayed.
*  
*  When skinning is enabled, this property will not 
*  be used to locate an image, instead, the image is drawn via CSS based on the 
*  SelectItem.pickerIconStyle property.
* @type {SCImgURL}
* @default "[SKIN]/DynamicForm/SelectItem_PickButton_icon.gif"
*/
pickerIconSrc?: SCImgURL;

/**
* Should this SelectItem show a hilite when it receives keyboard focus?
* @deprecated * As of SmartClient version 5.5, use SelectItem.showFocused instead.

* @type {boolean}
* @default true
*/
hiliteOnFocus?: boolean;

/**
* Background color to apply to the select item's selected value when the SelectItem 
*  receives focus, if hiliteOnFocus is true.
* @deprecated * As of SmartClient version 5.5, if SelectItem.showFocused is true, styling will be updated for this form item on focus. The hiliting effect can therefore
*  be achieved via SelectItem.textBoxStyle and SelectItem.controlStyle 
*  instead.

* @type {string}
* @default "#316AC5"
*/
hiliteColor?: string;

/**
* Text color to apply to the select item's selected value when the SelectItem 
*  receives focus, if hiliteOnFocus is true.
* @deprecated * As of SmartClient version 5.5, if SelectItem.showFocused is true, styling will be updated for this form item on focus. The hiliting effect can therefore
*  be achieved via SelectItem.textBoxStyle and SelectItem.controlStyle 
*  instead.

* @type {string}
* @default "white"
*/
hiliteTextColor?: string;

/**
* How large should the pick button be rendered?
* @deprecated * As of SmartClient version 5.5, pickButtonWidth has been deprecated in  favor of SelectItem.pickerIconWidth.

* @type {number}
* @default null
*/
pickButtonWidth?: number;

/**
* How large should the pick button be rendered?
* @deprecated * As of SmartClient version 5.5, pickButtonHeight has been deprecated in  favor of SelectItem.pickerIconHeight.

* @type {number}
* @default null
*/
pickButtonHeight?: number;

/**
* Source for image to show for the pick button
* @deprecated * As of SmartClient version 5.5, pickButtonSrc has been deprecated in  favor of SelectItem.pickerIconSrc.

* @type {string}
* @default null
*/
pickButtonSrc?: string;

/**
* When the user rolls over this item, should it be re-styled to indicate it has focus?
*  
*  By default this property is true for SelectItems, and SelectItem.updateTextBoxOnOver and
*  SelectItem.updateControlOnOver are set to false. This means the picker icon will show
*  over styling when the user rolls over the control table.
*  These defaults may be overridden by different SmartClient skins.
*  
*  See formItemStyling for more details on formItem styling.
* @type {boolean}
* @default true
*/
showOver?: boolean;

/**
* If FormItem.showOver is true, setting this property to false will explicitly
*  disable showing the "Over" state for the TextBox element of this item.
* @type {boolean}
* @default false
*/
updateTextBoxOnOver?: boolean;

/**
* If FormItem.showOver is true, setting this property to false will explicitly
*  disable showing the "Over" state for the control table element of
*  this item (if present).
* @type {boolean}
* @default false
*/
updateControlOnOver?: boolean;

/**
* By default SelectItems do not allow users to select the text of the
*  selected value.
* @type {boolean}
* @default false
*/
canSelectText?: boolean;

/**
* If set to true, always show an empty option in this item's pickList, allowing the user
*  to clear the value (even if there is no empty entry in the valueMap for the item).
*  
*  The empty value will be displayed with the
*  FormItem.emptyDisplayValue.
*  
*  With a SelectItem.optionDataSource, enabling
*  allowEmptyValue disables data paging by default - all data matching the
*  PickList.pickListCriteria will be requested. However, enabling
*  SelectItem.separateSpecialValues allows data paging to be used if required.
*  
*  See also SelectItem.specialValues as a way of providing several different special values in
*  addition to an empty value, such as "Invalid". Note that setting
*  specialValues disables the use of allowEmptyValue - see
*  details of how to have an empty value while using specialValues in 
*  in SelectItem.specialValues.
* @type {boolean}
* @default false
*/
allowEmptyValue?: boolean;

/**
* A set of "special" values such as "All", "None" or "Invalid" that do not appear in the normal
*  ValueMap or in the data returned by the SelectItem.optionDataSource.
*  
*  Like other uses of ValueMap, either a list of values or a mapping from stored to
*  display value can be provided.
*  
*  These values can either be shown at the top of the list of values (in the order specified), or
*  can be shown in a separate, non-scrolling region - the setting
*  SelectItem.separateSpecialValues controls this. Note that data 
*  paging can only be used if separateSpecialValues is enabled.
*  
*  If specialValues are configured, SelectItem.allowEmptyValue is ignored - an empty
*  value, if desired, must be included in the specialValues. To provide a
*  specialValue which clears the value of the field, use the special constant
*  PickList.emptyStoredValue.
*  
*  specialValues can also be used to take a value that does appear in the
*  normal data and redundantly display it at the top of the list to make it more accessible. Note
*  that in this case it is expected that the special value appears both at the top of the
*  list and in it's normal position in the list, so this works best with
*  SelectItem.separateSpecialValues mode enabled.
*  
*  Also, if an SelectItem.optionDataSource is used, SelectItem.specialValues that appear in the normal
*  dataset will be updated by automatic cacheSync (if 
*  the SelectItem.displayField is updated). However when using a distinct SelectItem.valueField and
*  SelectItem.displayField, you are required to provide SelectItem.specialValues as a map (there is no
*  support for FormItem.fetchMissingValues automatically fetching appropriate display values).
*  
*  Note that specialValues are not supported in conjunction with MultiComboBoxItem. Whereas with
*  SelectItem.multiple, specialValues will never be normal values 
*  that may be selected. So, specialValues should have options such as "Select All", "Select None" and others.
* @type {ValueMap}
* @default null
*/
specialValues?: ValueMap;

/**
* If true, SelectItem.specialValues such as the empty value will be shown in a
*  separate non-scrolling area, in the SelectItem.separateValuesList. Aside from making these values
*  more easily accessible, showing them in a separate list allows data paging to be used, which is
*  disabled if the separateValues are shown in the normal drop-down list along with other values.
* @type {boolean}
* @default null
*/
separateSpecialValues?: boolean;

/**
* AutoChild used to show SelectItem.specialValues.
* @type {ListGrid}
* @default null
*/
separateValuesList?: ListGrid;

/**
* If this select item retrieves its options from a dataSource, should options
*  be fetched from the server when the item is first drawn, or should this fetch be
*  delayed until the user opens the pickList.
*  
*  The default is true in order to allow the user to select a value via keyboard input
*  while keyboard focus is on the SelectItem but the pickList has not actually been shown.
* @type {boolean}
* @default true
*/
autoFetchData?: boolean;

/**
* If showing a hint for this form item, should it be shown within the field?
*  CSS style for the hint is SelectItem.textBoxStyle with the suffix
*  "Hint" appended to it.
* @type {boolean}
* @default null
*/
showHintInField?: boolean;

/**
* Indicates whether or not this SelectItem will load its list of options 
*  DataSource.progressiveLoading. This property is copied onto the
*  underlying PickList.
* @type {boolean}
* @default null
*/
progressiveLoading?: boolean;

/**
* Select items will submit their containing form on enter keypress 
*  if DynamicForm.saveOnEnter is true. Setting this property to
*  false will disable this behavior.
*  
*  Note that if the drop down list of options (pickList) is visible an
*  Enter keypress is used to select a value from the available set of
*  options and will not automatically cause form submission.
* @type {boolean}
* @default true
*/
saveOnEnter?: boolean;

/**
* Causes the PickList to open when the spacebar is pressed, default false. 
*  
*  For native OS widgets, space opens the PickList on Macs, but not on Windows. Consider
*  using this setting if your users are almost entirely Mac users, or enabling it only for
*  users running MacOS. 
*  
*  However, before using this setting, consider that it means that Spacebar will not be able
*  to be used for another purpose when focus is in a SelectItem.
* @type {boolean}
* @default false
*/
openOnSpace?: boolean;

/**
* Causes the PickList to open when the down arrow is pressed, default false. 
*  
*  For native OS widgets, the down arrow changes the value of a select on Windows, but 
*  opens the select on Macs. This setting is not recommended unless you are certain that 
*  all users of your applications will expect the Mac convention.
* @type {boolean}
* @default false
*/
openOnDownArrow?: boolean;

/**
* When this item is showing a PickList.dataSetType, this is 
*  the SelectItem.valueField of the record to use as the 
*  Tree.rootValue node.
* @type {string | number}
* @default null
*/
rootNodeId?: string | number;

/**
* When this item is showing a PickList.dataSetType, which 
*  nodes should be opened automatically. Options are:
*  
*  
* - "none" - no nodes are opened automatically
*  
* - "root" - opens the PickList.rootNodeId - in databound 
*        tree-pickers, this node is always hidden
*  
* - "all" - when ResultTree.loadDataOnDemand, opens the
*        PickList.rootNodeId and all of it's direct
*        descendants - otherwise, opens all loaded nodes 
*  
* @type {string}
* @default "none"
*/
autoOpenTree?: string;

/**
* When this item is showing a PickList.dataSetType, the
*  default behavior is for folder open-state to be toggled by double-clicking. Set this 
*  attribute to true to toggle folders on a single-click instead.
*  
*  Note: when set to true, users can only choose leaf-nodes, since clicking folders would
*  simply toggle them.
* @type {boolean}
* @default false
*/
singleClickFolderToggle?: boolean;

/**
* Determines the behavior of the click-mask thrown up when this pickList is visible.
*  
*  The default value, "hard", matches the familiar behavior of combos and selects on 
*  Windows, Mac and other platforms - mouse-events such as rollovers are blocked and, when 
*  a click is received, the picker is hidden and the event is cancelled.
*  
*  When clickMaskMode is "soft", mouse-events continue to fire, meaning that 
*  rollover styles, for example, continue to be updated. When a click is received in this
*  mode, the picker is hidden and the click event is allowed to proceed to its target - 
*  this means that clicking an item with an open picker will re-open the picker.
* @type {ClickMaskMode}
* @default "hard"
*/
clickMaskMode?: ClickMaskMode;

/**
* Controls where the PickList is placed. 
*  Can be specified as a PanelPlacement
*  or a specific widget that should be filled (by specifying an actual Canvas or
*  Canvas.ID).
*  
*  Default behavior is to "fillPanel" if Browser.isHandset or
*  Browser.isTablet, to better accommodate the smaller screen real estate and 
*  less precise
*  pointing ability on such devices.
*  
*  When filling the whole screen, part of the screen or a specific panel, the expanded
*  interface is created as a FormItem.picker, and 
*  incorporates a SelectItem.pickerNavigationBar and 
*  SelectItem.pickerExitButton that hides the expanded interface.
* @type {PanelPlacement | Canvas | string}
* @default null
*/
pickListPlacement?: PanelPlacement | Canvas | string;

/**
* For PickList items with PickListItemIconPlacement set such that the pickList does
*  not render near-origin, should specified FormItem.icons be rendered inline within
*  the formItem itself, or within the ComboBoxItem.pickerNavigationBar.
*  
*  May be overridden at the icon level via FormItemIcon.iconPlacement.
*  
*  For mobile browsing with limited available screen space, icons rendered in
*  the navigation bar may be easier for the user to interact with.
* @type {PickListItemIconPlacement}
* @default "both"
*/
iconPlacement?: PickListItemIconPlacement;

/**
* NavigationButton to dismiss the picker interface, created when 
*  SelectItem.pickListPlacement indicates that the search interface takes over the
*  entire panel or screen.
*  
*  The following autoChildUsage apply:
*  
*  
* - SelectItem.pickerExitButtonTitle for Button.title
*  
* @type {NavigationButton}
* @default null
*/
pickerExitButton?: NavigationButton;

/**
* The title for the SelectItem.pickerExitButton.
* @type {HTMLString}
* @default "Done"
*/
pickerExitButtonTitle?: HTMLString;

/**
* NavigationButton to clear the picker value, created when SelectItem.pickListPlacement
*  indicates that the search interface takes over the entire panel or screen.
*  
*  This button will only be shown if SelectItem.allowEmptyValue is true.
*  
*  The following autoChildUsage apply:
*  
*  
* - SelectItem.pickerClearButtonTitle for
*  Button.title
*  
* @type {NavigationButton}
* @default null
*/
pickerClearButton?: NavigationButton;

/**
* The title for the SelectItem.pickerClearButton.
* @type {HTMLString}
* @default "Clear"
*/
pickerClearButtonTitle?: HTMLString;

/**
* Empty message to display in the selectItem if PickList.hideEmptyPickList
*  is false.
* @type {string}
* @default "No items to show"
*/
emptyPickListMessage?: string;

/**
* NavigationBar created when SelectItem.pickListPlacement indicates that the search
*  interface takes over the entire panel or screen.
* @type {NavigationBar}
* @default null
*/
pickerNavigationBar?: NavigationBar;

/**
* By default, if multiple keys are pressed in quick succession, a SelectItem will buffer
*  them together and use the resulting multi-char string when searching. Set this 
*  attribute to false to force the item to match only one character at a time.
* @type {boolean}
* @default true
*/
allowMultiCharSearch?: boolean;

/**
* By default HTML values in a selectItem will be interpreted by the browser.
*  Setting this flag to true will causes HTML characters to be escaped, meaning the
*  raw value of the field (for example "&lt;b&gt;AAA&lt;/b&gt;") is displayed
*  to the user rather than the interpreted HTML (for example "AAA")
* @type {boolean}
* @default false
*/
escapeHTML?: boolean;

/**
* The Class to use when creating a picker of PickList.dataSetType for 
*  a FormItem. Must be a subclass of the builtin default, 
*  PickListMenu.
* @type {SCClassName}
* @default "PickListMenu"
*/
pickListConstructor?: SCClassName;

/**
* The Class to use when creating a picker of PickList.dataSetType for 
*  a FormItem. Must be a subclass of the builtin default, 
*  PickTreeMenu.
* @type {SCClassName}
* @default "PickTreeMenu"
*/
pickTreeConstructor?: SCClassName;

/**
* Whether to show the picker as a flat list, or a collapsible tree.
*  
*  The default value, "list", will use an instance of the
*  PickList.pickListConstructor as the picker - "tree" will 
*  show an instance of PickList.pickTreeConstructor.
* @type {string}
* @default "list"
*/
dataSetType?: string;

/**
* If set, this FormItem will derive data to show in the PickList by fetching records from
*  the specified optionDataSource. The fetched data will be used as a
*  FormItem.valueMap by extracting the
*  FormItem.valueField and FormItem.displayField in
*  the loaded records, to derive one valueMap entry per record loaded from the
*  optionDataSource. Multiple fields from the fetched data may be shown in the pickList by
*  setting SelectItem.pickListFields.
*  
*  The data will be retrieved via a "fetch" operation on the DataSource, passing the 
*  PickList.pickListCriteria (if set) as criteria, and passing
*  SelectItem.optionFilterContext (if set) as DSRequest Properties.
*  
*  The fetch will be triggered when the pickList is first shown, or, you can set
*  SelectItem.autoFetchData to fetch when the FormItem is
*  first drawn.
*  
*  The pickList will not re-fetch its options each time it is opened, unless
*  the pickList criteria have changed. Developers who want to explicitly force a 
*  new fetch can achieve this by
*  overriding PickList.getPickListFilterCriteria
*  
*  to dynamically generate criteria that change whenever appropriate.
*  For example, to force a new fetch every time the pickList is shown,
*  the generated criteria could include a subcriterion for some arbitrary field
*  with operator notEqual and value set to a something
*  that will change each time the method runs, such as a date-stamp.
*  
*  Note that providing an initial value when
*  FormItem.fetchMissingValues is enabled, or enabling
*  SelectItem.defaultToFirstOption, can also cause a fetch to
*  be initiated immediately upon form creation. You can also call PickList.fetchData
*  at any time to manually trigger a fetch.
*  
*  Data paging is automatically enabled if the optionDataSource supports it. As the
*  pickList is scrolled by the user, requests for additional data will be automatically
*  issued.
*  
*  For a pickList attached to a ComboBoxItem, new fetches are 
*  issued as the user types, with criteria set as described under 
*  ComboBoxItem.getPickListFilterCriteria.
*  If your dataSource is not capable of filtering results by search criteria (eg, the
*  dataSource is backed by an XML flat file), you can set SelectItem.filterLocally to have the
*  entire dataset loaded up front and filtering performed in the browser. This disables
*  data paging.
*  
*  Note that if a normal, static FormItem.valueMap is also specified
*  for the field (either directly in the form item or as part of the field definition in
*  the dataSource), it will be preferred to the data derived from the optionDataSource for
*  whatever mappings are present.
* @type {DataSource | string}
* @default null
*/
optionDataSource?: DataSource | string;

/**
* When applying filter criteria to pickList data, what type of matching to use.
*  
*  For a databound pickList (SelectItem.optionDataSource set), textMatchStyle is
*  sent to the server as DSRequest.textMatchStyle.
*  
*  For a non-databound pickList, textMatchStyle is applied by 
*  PickList.filterClientPickListData.
* @type {TextMatchStyle}
* @default "startsWith"
*/
textMatchStyle?: TextMatchStyle;

/**
* This property allows the developer to specify which field[s] will be displayed in the 
*  drop down list of options.
*  
*  Only applies to databound pickLists (see PickList.optionDataSource, or pickLists
*  with custom data set up via the advanced PickList.getClientPickListData method.
*  
*  If this property is unset, we display the PickList.displayField, if specified, 
*  otherwise the PickList.valueField.
*  
*  If there are multiple fields, column headers will be shown for each field, the
*  height of which can be customized via the PickList.pickListHeaderHeight attribute.
*  
*  Each field to display should be specified as a ListGridField object. Note that
*  unlike in ListGrid, dataSource fields marked as 
*  DataSourceField.hidden will be hidden by default in pickLists. To
*  override this behavior, ensure that you specify an explicit value for 
*  ListGridField.showIf.
* @type {Array<Partial<ListGridField>>}
* @default null
*/
pickListFields?: Array<Partial<ListGridField>>;

/**
* For databound pickLists (see PickList.optionDataSource), by default SmartClient
*  will cache and re-use datasets shown by pickLists in an LRU (least recently used) caching
*  pattern.
*  
*  Setting this flag to false avoids this caching for situations where it is too aggressive.
* @type {boolean}
* @default true
*/
cachePickListResults?: boolean;

/**
* If this item has a specified optionDataSource and this property is
*  true, the list of fields used by this pickList will be passed to
*  the datasource as DSRequest.outputs. If the datasource supports this feature
*  the returned fields will be limited to this list. A custom datasource will need to
*  add code to implement field limiting.
*  
*  This list of used fields consists of the values of FormItem.valueField,
*  FormItem.displayField and SelectItem.pickListFields.
*  
*  NOTE: When enabled, FormItem.getSelectedRecord will only include the
*  fetched fields.
* @type {boolean}
* @default null
*/
fetchDisplayedFieldsOnly?: boolean;

/**
* If this item has a specified optionDataSource, and this property is not
*  null, this will be passed to the datasource as DSRequest properties when
*  performing the filter operation on the dataSource to obtain the set of options for the
*  list. This provides, among other capabilities, a way to trigger the server to return
*  summary records.
* @type {Partial<DSRequest>}
* @default null
*/
optionFilterContext?: Partial<DSRequest>;

/**
* If this item has a specified optionDataSource, this attribute may be set
*  to specify an explicit DSRequest.operationId when performing a fetch against the
*  option dataSource to pick up display value mapping.
* @type {string}
* @default null
*/
optionOperationId?: string;

/**
* If set, this item will display a value from another field to the user instead of
*  showing the underlying data value for the FormItem.name.
*  
*  This property is used in two ways:
*  
*  The item will display the displayField value from the 
*  DynamicForm.getValues if 
*  FormItem.useLocalDisplayFieldValue is true, (or if unset and the conditions
*  outlined in the documentation for that property are met).
*  
*  If this field has an FormItem.optionDataSource, this property is used by
*  default to identify which value to use as a display value in records from this
*  related dataSource. In this usage the specified displayField must be 
*  explicitly defined in the optionDataSource to be used - see 
*  SelectItem.getDisplayFieldName for more on this behavior.
*  If not using FormItem.useLocalDisplayFieldValue, the display
*  value for this item will be derived by performing a fetch against the 
*  FormItem.getOptionDataSource 
*  to find a record where the FormItem.getValueFieldName matches 
*  this item's value, and use the displayField value from that record.
*  In addition to this, PickList-based form items that provide a list of possible
*  options such as the SelectItem or ComboBoxItem will show the 
*  displayField values to the user by default, allowing them to choose
*  a new data value (see FormItem.valueField) from a list of user-friendly
*  display values.
*  
*  This essentially allows the specified optionDataSource to be used as
*  a server based valueMap.
*  
*  If FormItem.useLocalDisplayFieldValue
*  are being used and FormItem.storeDisplayValues is true, selecting a new value
*  will update both the value for this field and the associated display-field value
*  on the record being edited.
*  
*  Note: Developers may specify the FormItem.foreignDisplayField property 
*  in addition to displayField. This is useful for cases where the
*  display field name in the local dataSource differs from the display field name in
*  the optionDataSource. See the documentation for
*  DataSourceField.foreignDisplayField for more on this.
*  If a foreignDisplayField is specified, as with just displayField, if 
*  FormItem.useLocalDisplayFieldValue
*  are being used and FormItem.storeDisplayValues is true, when the user 
*  chooses a value the associated display-field value 
*  on the record being edited will be updated. In this case it would be set to the 
*  foreignDisplayField value from the related record. This means foreignDisplayField 
*  is always expected to be set to the equivalent field in the related dataSources.
*  Developers looking to display some other arbitrary field(s) from the
*  related dataSource during editing should consider using custom
*  PickList.pickListFields instead of setting a foreignDisplayField. 
*  
*  Note that if optionDataSource is set and no valid display field is
*  specified,
*  FormItem.getDisplayFieldName will return the dataSource title 
*  field by default.
*  
*  If a displayField is specified for a freeform text based item (such as a 
*  ComboBoxItem), any user-entered value will be treated as a display value.
*  In this scenario, items will derive the data value for the item from the
*  first record where the displayField value matches the user-entered value.
*  To avoid ambiguity, developers may wish to avoid this usage if display values
*  are not unique.
* @type {string}
* @default null
*/
displayField?: string;

/**
* If this form item maps data values to display values by retrieving the 
*  FormItem.displayField values from an 
*  FormItem.optionDataSource, this property 
*  denotes the the field to use as the underlying data value in records from the 
*  optionDataSource.
*  If not explicitly supplied, the valueField name will be derived as
*  described in FormItem.getValueFieldName.
* @type {string}
* @default null
*/
valueField?: string;

/**
* If filterLocally is set for this item, and this item is showing options 
*  from a dataSource, fetch the entire set of options from the server, and use these values
*  to map the item value to the appropriate display value. Also use "local"
*  type filtering on drop down list of options.
*  
*  This means data will only be fetched once from the server, and then filtered on the
*  client.
*  
*  Note - when this property is set to false, filtering will still be 
*  performed on the client if a complete set of data for some criteria has been cached
*  by a fetch, and a subsequent fetch has more restrictive criteria. To explicitly
*  disable client-side filtering set the SelectItem.useClientFiltering property to false.
* @type {boolean}
* @default false
*/
filterLocally?: boolean;

/**
* If this item has a databound pickList (for example PickList.optionDataSource is
*  set), this property can be used to provide static filter criteria when retrieving the data
*  for the pickList.
* @type {Criteria}
* @default null
*/
pickListCriteria?: Criteria;

/**
* If this item is part of a databound form, and has a specified valueMap,
*  by default we show the valueMap options in the pickList for the item.
*  Setting this property to true will ensure that the options displayed in our pickList
*  are derived from the form's dataSource.
* @type {boolean}
* @default null
*/
showOptionsFromDataSource?: boolean;

/**
* If specified this properties block will be applied to the PickListMenu
*  created for this FormItem.
*  
*  Note: Not every ListGrid property is supported when assigned to a pickList.
*  Where there is a dedicated API on the form item (such as 
*  PickList.pickListCellHeight), we recommend that be used in 
*  favor of setting the equivalent property on the pickListProperties block.
*  
*  PickLists and ListGrid.showFilterEditor:
*  ComboBoxItem do not support setting showFilterEditor 
*  to true on pickListProperties. This combination of settings leads to an ambiguous user
*  experience as the two sets of filter-criteria (those from the text-box and those from the 
*  pickList filter editor) interact with each other.
*  Setting ListGrid.showFilterEditor in 
*  SelectItem.pickListProperties is a valid way to create a filterable pickList, on 
*  a SelectItem, but this setting is not supported on a SelectItem with 
*  SelectItem.multiple set to true - this combination of settings can cause a 
*  selected value to be filtered out of view at which point further selection changes will 
*  discard that value.
*  In general we recommend the ComboBoxItem class (with ComboBoxItem.addUnknownValues 
*  set as appropriate) as a better interface for filtering pickList data.
* @type {Partial<ListGrid>}
* @default null
*/
pickListProperties?: Partial<ListGrid>;

/**
* ListGrid-based AutoChild created by the system to display a list of pickable options for this item. 
*  
*  The pickList is automatically generated and displayed by the system when necessary. It may be customized
*  via properties such as SelectItem.pickListConstructor, SelectItem.pickTreeConstructor, SelectItem.pickListProperties 
*  and more. See the PickList for more information.
*  
*  Accessing the generated pickList at runtime is an advanced usage. In most cases developers
*  should not modify this generated component directly but should instead use attributes
*  on the formItem to configure it.
* @type {ListGrid}
* @default null
*/
pickList?: ListGrid;

/**
* Specifies one or more fields by which this item should be initially sorted. It can be 
*  a ListGridField.name, or an array of field names - but note that, if
*  multiple fields are supplied, then each will be sorted in the same 
*  ListGrid.sortDirection.
*  
*  For full sorting control, set PickList.initialSort to a list of
*  custom SortSpecifier.
*  
*  This attribute can also be set to the index of a field in the fields array, but note 
*  that it will be converted to a string (field name) after initialization.
* @type {string | Array<Partial<string>> | Integer}
* @default null
*/
sortField?: string | Array<Partial<string>> | Integer;

/**
* An array of SortSpecifier objects used to set up the initial sort configuration 
*  for this pickList. If specified, this will be used instead of any 
*  PickList.sortField specified.
* @type {Array<Partial<SortSpecifier>>}
* @default null
*/
initialSort?: Array<Partial<SortSpecifier>>;



/**
*  Set the valueMap for this item.
* 
* @param {Array<any> | object} new valueMap
*/
setValueMap(valueMap:Array<any> | object): void;

/**
*  Expression evaluated to determine the SelectItem.defaultValue when no value is
*  provided for this item. To default to the first option use
*  SelectItem.defaultToFirstOption instead.
* 
* @param {FormItem} the form item itself (also available as "this")
* @param {DynamicForm} the managing DynamicForm instance
* @param {object} the current set of values for the form as a whole
*/
defaultDynamicValue(item:FormItem, form:DynamicForm, values:object): any;

/**
*  Notification method called when FormItem.showPending is enabled and
*  this SelectItem should either clear or show its pending visual state.
*  
*  The default behavior is that the FormItem.titleStyle and
*  FormItem.cellStyle are updated to include/exclude the "Pending" suffix.
*  In addition, a SelectItem.multiple SelectItem when displayed in
*  the pending state will apply FormItem.editPendingCSSText to any new value in the
*  text box and also append "Pending" to the cells' ListGrid.baseStyle for cells
*  in the pickList menu corresponding to new values. Returning false will
*  cancel this default behavior.
* 
* @param {DynamicForm} the managing DynamicForm instance.
* @param {FormItem} the form item itself (also available as "this").
* @param {boolean} true if the item should show its pending visual state; false otherwise.
* @param {any} the current form item value.
* @param {any} the value that would be restored by a call to DynamicForm.resetValues.
*/
pendingStatusChanged(form:DynamicForm, item:FormItem, pendingStatus:boolean, newValue:any, value:any): boolean;

/**
*  Method to show a picker for this item. By default this method is called if the user
*  clicks on a SelectItem.showPickerIcon. May also be called programmatically.
*  
*  Overridden from the default FormItem.showPicker implementation to show
*  the PickList
* 
* 
*/
showPicker(): void;

/**
*  Set the value of the form item to the value passed in
*  
*  NOTE: for valueMap'd items, newValue should be data value not displayed value
* 
* @param {any} value to set the element to
*/
setValue(newValue:any): void;

/**
*  For a SelectItem with an SelectItem.optionDataSource and allowing multiple selection
*  (SelectItem.multiple), returns the list of currently selected records, or
*  null if none are selected.
*  
* 
*/
getSelectedRecords(): Array<Partial<ListGridRecord>>;

/**
*  Only applies to databound items (see PickList.optionDataSource).
*  Performs a fetch type operation on this item's DataSource to retrieve the set of valid
*  options for the item, based on the current PickList.pickListCriteria.
* 
* @param {DSCallback=} Callback to fire when the fetch completes. Callback will        fire with 4 parameters:  
- item a pointer to the form item  
- dsResponse the DSResponse returned by the server  
- data the raw data returned by the server  
- dsRequest the DSRequest sent to the server  
* @param {Partial<DSRequest>=} properties to apply to the       dsRequest for this fetch.
*/
fetchData(callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Returns the displayField for this item.
*  
*  Behavior varies based on the configuration of this item, as follows:
*  
* - If this item has an SelectItem.optionDataSource and an explicit 
*  FormItem.foreignDisplayField is specified, this will be returned.
*  
* - Otherwise if an explicit SelectItem.displayField is specified it will be returned 
*  by default. If the displayField was specified on the underlying 
*  dataSource field, and no matching field is present in the SelectItem.optionDataSource for
*  the item, we avoid returning the specified displayField value and instead return the
*  title field of the option DataSource. We do this to
*  avoid confusion for the case where the displayField is intended as a display-field
*  value for showing another field value within the same record in the underlying
*  dataSource only.
*  
* - If no explicit foreignDisplay or displayField 
*  specification was found, and the FormItem.valueField for this item is hidden in the 
*  FormItem.optionDataSource, this method will return the title field for 
*  the optionDataSource.
* 
* 
*/
getDisplayFieldName(): FieldName;

/**
*  Getter method to retrieve the FormItem.valueField for this item. For
*  items with a specified FormItem.optionDataSource, this determines which
*  field in that dataSource corresponds to the value for this item.
*  
*  If unset, if a DataSourceField.foreignKey exists
*  between this field and the optionDataSource, this will be used,
*  otherwise default behavior will return the FormItem.name of this field.
* 
* 
*/
getValueFieldName(): string;

/**
*  If this item is showing a dataBound pickList, this notification method will be fired 
*  when new data arrives from the server.
* 
* @param {Integer} index of first row returned by the server
* @param {Integer} index of last row returned by the server
* @param {ResultSet} pointer to this pickList's data
*/
dataArrived(startRow:Integer, endRow:Integer, data:ResultSet): void;

/**
*  Get the record returned from the SelectItem.optionDataSource when FormItem.fetchMissingValues
*  is true, and the missing value is fetched.
*  
*  FormItem.fetchMissingValues kicks off the fetch when the form item is initialized
*  with a non null value or when setValue() is called on the item. Note that this method
*  will return null before the fetch completes, or if no record is found in the
*  optionDataSource matching the underlying value.
* 
*/
getSelectedRecord(): ListGridRecord;



}
/**
* Creates a new SelectItem
* 
* @param typeCheckedProperties {Partial<SelectItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SelectItem} The new instance
*/
class SelectItem{
static create(typeCheckedProperties? : Partial<SelectItem>, uncheckedProperties?: Object): SelectItem;







}

/**
* EditProxy that handles SelectItem and ComboBoxItem
*  when editMode is enabled.
*/
interface SelectItemEditProxy extends FormItemEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's valueMap one-per-line as specified in
*  FormItemEditProxy.valueMapDisplaySeparatorChar. Current value(s)
*  is indicated with FormItemEditProxy.valueMapSelectedChar.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's valueMap and current value.
* 
* 
* @param {string} the new component valueMap
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new SelectItemEditProxy
* 
* @param typeCheckedProperties {Partial<SelectItemEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SelectItemEditProxy} The new instance
*/
class SelectItemEditProxy{
static create(typeCheckedProperties? : Partial<SelectItemEditProxy>, uncheckedProperties?: Object): SelectItemEditProxy;







}

/**
* FormItem that shows a list of options, plus an "Other..." option that allows them to enter
*  another value.
*  
*  Note - SelectOtherItem does not support using an SelectItem.optionDataSource,
*  instead, use a normal SelectItem and use the SelectItem.specialValues 
*  to implement a way to add new DataSource records to the SelectItem.optionDataSource.
*  This creates a UI more appropriate to SelectItem.optionDataSource: 
*  the SelectOtherItem.otherValue option or options 
*  can be placed at the top of the list, so that scrolling to the bottom of a long list is not required.
*  In addition, the SelectItem.specialValues system allows you 
*  to open a custom form or other UI for adding new DataSource records, rather than just 
*  the simple single-value input dialog of SelectOtherItem.
*/
interface SelectOtherItem extends Class {
/**
* Title for the separator between normal items and the Other... item in the drop
*  down list. Selecting this item will not change the FormItem's value.
* @type {string}
* @default "--------------------"
*/
separatorTitle?: string;

/**
* Value for the separator item between normal items and the Other... value. 
*  If necessary the value may be changed to ensure it doesn't collide with any data values in
*  this item's FormItem.valueMap.
* @type {string}
* @default "----"
*/
separatorValue?: string;

/**
* Title for the Other... item. When this item is selected, the user will be 
*  shown a prompt allowing them to enter a new value for the item.
* @type {string}
* @default "Other..."
*/
otherTitle?: string;

/**
* Data value for the Other... item. If necessary this value may be changed to
*  ensure it doesn't collide with any data values in this item's
*  FormItem.valueMap.
* @type {string}
* @default "***other***"
*/
otherValue?: string;

/**
* Title to show in prompt for "other" value.
*  Note this is a dynamic string. JavaScript content is supported within &#36;{...}
*  tags, with local variables for item (a pointer to this item) and 
*  value a pointer to the currently selected item value.
* @type {HTMLString}
* @default "Other value for &lt;br&gt;${item.getTitle()}?"
*/
selectOtherPrompt?: HTMLString;

/**
* Width for the "other value" prompt dialog.
* @type {Integer}
* @default "250"
*/
dialogWidth?: Integer;





}
/**
* Creates a new SelectOtherItem
* 
* @param typeCheckedProperties {Partial<SelectOtherItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SelectOtherItem} The new instance
*/
class SelectOtherItem{
static create(typeCheckedProperties? : Partial<SelectOtherItem>, uncheckedProperties?: Object): SelectOtherItem;







}

/**
* Maintains a 'selected' subset of a List or Array of objects, such as records in a record
*  set, or widgets in a selectable header.
*  
*  Includes methods for selecting objects and checking which objects are selected, and also for
*  selecting objects as a result of mouse events, including drag selection support.
*  The selection object is used automatically to handle selection APIs on ListGrid
*  and TreeGrid instances.
*  
*  Note that selection and deselection are skipped for objects that aren't enabled, or that are
*  marked as non-selectable. For a ListGrid, the relevant properties are 
*  ListGrid.recordEnabledProperty and ListGrid.recordCanSelectProperty. The 
*  recommended approach to affect disabled objects via the Selection APIs is to temporarily
*  enable them beforehand.
*/
interface Selection extends Class {
/**
* Message to display to the user in a warn dialog if Selection.selectRange is
*  called for a selection on a ResultSet, where the range of records to be selected has not been
*  loaded.
* @type {string}
* @default "Some records in the range you selected are not loaded.  Scroll through the entire range before selecting it."
*/
selectionRangeNotLoadedMessage?: string;



/**
*  Return true if a particular item is selected
* 
* 
* @param {object} object to check
*/
isSelected(item:object): boolean;

/**
*  When using tree-oriented selection modes like TreeGrid.cascadeSelection, returns true
*  if the record is considered partially selected because only some of it's children are
*  selected.
* 
* 
* @param {object} object to check
*/
isPartiallySelected(item:object): boolean;

/**
*  Whether at least one item is selected
* 
*/
anySelected(): boolean;

/**
*  Returns the number of selected records.
* 
* 
*/
getLength(): Integer;

/**
* 	Whether multiple items are selected
* 
*/
multipleSelected(): boolean;

/**
*  Return an ordered array of all of the selected items
* 
* @param {boolean=} When true, partially selected records will not be returned.                  Otherwise, all fully and partially selected records are                  returned.
*/
getSelection(excludePartialSelections?:boolean): Array<any>;

/**
*  	Return the first item in the list that is selected.
* 
*  	Note that this should only be used if you know that one only one item
*   may be selected, or you really don't care about items after the first one.
* 
*  	To get all selected objects, use Selection.getSelection
* 
*/
getSelectedRecord(): object;

/**
*  Select or deselect a particular item.
*  All other selection routines go through this one, so by observing this routine you can
*  monitor all selection changes.
* 
* @param {object} object to select
* @param {boolean} turn selection on or off
*/
setSelected(item:object, newState:boolean): boolean;

/**
*  	Select a particular item
* 
* @param {object} object to select
*/
select(item:object): boolean;

/**
*  	Deselect a particular item
* 
* @param {object} object to select
*/
deselect(item:object): boolean;

/**
*  Select a single item and deselect everything else
* 
* @param {object} object to select
*/
selectSingle(item:object): boolean;

/**
*  Select an array of items (subset of the entire list)
* 
* @param {Array<Partial<object>>} array of objects to select
*/
selectList(list:Array<Partial<object>>): boolean;

/**
*  	Deselect an array of items (subset of the entire list)
* 
* @param {Array<Partial<object>>} array of objects to select
*/
deselectList(list:Array<Partial<object>>): boolean;

/**
*  Select all records of the list.
* 
* @param {boolean=} If this selection's data object is a tree,  if true is passed for this parameter,  only visible nodes will be selected. Nodes embedded in a closed parent folder  (and thus hidden from the user) will not be selected.
*/
selectAll(visibleNodesOnly?:boolean): boolean;

/**
*  	Deselect ALL records of the list
* 
*/
deselectAll(): boolean;

/**
*  Select a particular item by its position in the list
* 
* 
* @param {number} index of the item to be selected
*/
selectItem(position:number): boolean;

/**
*  Deselect a particular item by its position in the list
* 
* 
* @param {number} index of the item to be selected
*/
deselectItem(position:number): boolean;

/**
*  	Select range of records from start to end, non-inclusive.
* 
* @param {number} start index to select
* @param {number} end index (non-inclusive)
* @param {boolean=} optional new selection state to set. True means                   selected, false means unselected. Defaults to true.
*/
selectRange(start:number, end:number, newState?:boolean): boolean;

/**
*  	Deselect range of records from start to end, non-inclusive
* 
* 
* @param {number} start index to select
* @param {number} end index (non-inclusive)
*/
deselectRange(start:number, end:number): boolean;



}
/**
* Creates a new Selection
* 
* @param typeCheckedProperties {Partial<Selection>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Selection} The new instance
*/
class Selection{
static create(typeCheckedProperties? : Partial<Selection>, uncheckedProperties?: Object): Selection;
/**
* A declared value of the enum type 
*  SelectionStyle.
* @type {Constant}
* @default "none"
*/
static NONE?: Constant;

/**
* A declared value of the enum type 
*  SelectionStyle.
* @type {Constant}
* @default "single"
*/
static SINGLE?: Constant;

/**
* A declared value of the enum type 
*  SelectionStyle.
* @type {Constant}
* @default "multiple"
*/
static MULTIPLE?: Constant;

/**
* A declared value of the enum type 
*  SelectionStyle.
* @type {Constant}
* @default "simple"
*/
static SIMPLE?: Constant;








}

/**
* This static singleton class implements a component that can be used to highlight any other
*  Canvas or FormItem by drawing a line around it and optional label. The selection outline 
*  moves, resizes and hides with the target object, and does not occlude any part of it. The
*  outline is not a peer, child or member of the target object; the target object does not 
*  know about the outline.
*  
*  NOTE: This class is for internal use only by EditContext.
*/
interface SelectionOutline extends Class {




}
/**
* Creates a new SelectionOutline
* 
* @param typeCheckedProperties {Partial<SelectionOutline>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SelectionOutline} The new instance
*/
class SelectionOutline{
static create(typeCheckedProperties? : Partial<SelectionOutline>, uncheckedProperties?: Object): SelectionOutline;
/**
* Set the CSS border of this component as a CSS string including border-width,
*  border-style, and/or color (eg "1px dashed blue").
*  
*  This property applies the same border to all four sides of this component.
* @type {string}
* @default "1px dashed #44ff44"
*/
static border?: string;

/**
* The background color for the selection label. It corresponds to the CSS background-color
*  attribute. You can set this property to an RGB value (e.g. #22AAFF) or a named color
*  (e.g. red) from a list of browser supported color names.
* @type {string}
* @default "#44ff44"
*/
static labelBackgroundColor?: string;





/**
*  Set the CSS border of this component as a CSS string including border-width,
*  border-style, and/or color (eg "1px dashed blue").
*  
*  This property applies the same border to all four sides of this component.
* 
* 
* @param {string} ["1px dashed #44ff44"] css border string
*/
static setBorder(border:string): void;

/**
*  Returns the CSS border of this component.
* 
* 
*/
static getBorder(): void;

/**
*  Set the background color for the selection label. It corresponds to the CSS background-color
*  attribute. You can set this property to an RGB value (e.g. #22AAFF) or a named color
*  (e.g. red) from a list of browser supported color names.
* 
* 
* @param {string} ["#44ff44"] css color
*/
static setLabelBackgroundColor(color:string): void;

/**
*  Returns the background color for the selection label.
* 
* 
*/
static getLabelBackgroundColor(): void;




}

/**
* A simple subclass of Menu created by TreeMenuButton.
*  Shows the selected node's path in a custom style.
*  
*  Important Note: this class is not directly usable except for skinning and for
*  subclassing when setting TreeMenuButton.treeMenuConstructor on a TreeMenuButton.
*/
interface SelectionTreeMenu extends Menu {




}
/**
* Creates a new SelectionTreeMenu
* 
* @param typeCheckedProperties {Partial<SelectionTreeMenu>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SelectionTreeMenu} The new instance
*/
class SelectionTreeMenu{
static create(typeCheckedProperties? : Partial<SelectionTreeMenu>, uncheckedProperties?: Object): SelectionTreeMenu;







}

/**
* Sends an email using the group:reifyMessaging isc_sendEmail DataSource.
*  Refer to Mail to know how to set up access to an SMTP server.
*  
*  There is a matching Reify Workflow Editor task editor, class:SendEmailTaskEditor, that
*  can be enabled by attr:Reify.enableSendEmailTaskEditor.
*  
*  If Process.mockMode is enabled, instead of sending an email a
*  SendEmailTask.mockModeNotifyMessage is shown instead.
*/
interface SendEmailTask extends ProcessElement {
/**
* Email recipient address(es) separated by commas.
* @type {string}
* @default null
*/
to?: string;

/**
* Email subject. To assign a dynamic subject see SendEmailTask.subjectFormula.
* @type {string}
* @default null
*/
subject?: string;

/**
* Formula to be used to calculate the subject contents. Use SendEmailTask.subject property
*  to assign a static subject instead.
*  
*  Available fields for use in the formula are the current Canvas.ruleScope.
* @type {UserSummary}
* @default null
*/
subjectFormula?: UserSummary;

/**
* Message to be sent to recipients.
* @type {string}
* @default null
*/
message?: string;

/**
* Message displayed by Notify in lieu of sending an actual email when the workflow
*  is in Process.mockMode. 
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  The following dynamic variables are available:
*  
*  
* - to
*  
* - subject
*  
* - message
*  
* @type {string}
* @default "Workflow Mock Mode - in live mode, would send Email&lt;p&gt;To: ${to}&lt;br&gt;Subject: ${subject}&lt;br&gt;Message: ${message}"
*/
mockModeNotifyMessage?: string;

/**
* ID of the next sequence or element to proceed to if a failure condition arises
*  from operation.
* @type {string}
* @default null
*/
failureElement?: string;

/**
* Additional properties to set on the DSRequest that will be issued
*  to perform send.
*  
*  Note that operationId will always be set email and
*  willHandleError will always be set true.
* @type {DSRequest}
* @default null
*/
requestProperties?: DSRequest;





}
/**
* Creates a new SendEmailTask
* 
* @param typeCheckedProperties {Partial<SendEmailTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SendEmailTask} The new instance
*/
class SendEmailTask{
static create(typeCheckedProperties? : Partial<SendEmailTask>, uncheckedProperties?: Object): SendEmailTask;







}

/**
* Sends an SMS message using the group:reifyMessaging isc_sendSMS DataSource.
*  
*  There is a matching Reify Workflow Editor task editor, class:SendSMSTaskEditor, that
*  can be enabled by attr:Reify.enableSendSMSTaskEditor.
*  
*  If Process.mockMode is enabled, instead of sending an SMS message a
*  SendSMSTask.mockModeNotifyMessage is shown instead.
*/
interface SendSMSTask extends ProcessElement {
/**
* SMS recipient phone number(s) separated by commas.
* @type {string}
* @default null
*/
to?: string;

/**
* Message to be sent to recipients.
* @type {string}
* @default null
*/
message?: string;

/**
* Message displayed by Notify in lieu of sending an actual email when the workflow
*  is in Process.mockMode. 
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will 
*  be evaluated as JS code when the message is displayed.
*  
*  The following dynamic variables are available:
*  
*  
* - to
*  
* - message
*  
* @type {string}
* @default "Workflow Mock Mode - in live mode, would send Email&lt;p&gt;To: ${to}&lt;br&gt;Subject: ${subject}&lt;br&gt;Message: ${message}"
*/
mockModeNotifyMessage?: string;

/**
* ID of the next sequence or element to proceed to if a failure condition arises
*  from operation.
* @type {string}
* @default null
*/
failureElement?: string;

/**
* Additional properties to set on the DSRequest that will be issued
*  to perform send.
*  
*  Note that operationId will always be set sms and
*  willHandleError will always be set true.
* @type {DSRequest}
* @default null
*/
requestProperties?: DSRequest;





}
/**
* Creates a new SendSMSTask
* 
* @param typeCheckedProperties {Partial<SendSMSTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SendSMSTask} The new instance
*/
class SendSMSTask{
static create(typeCheckedProperties? : Partial<SendSMSTask>, uncheckedProperties?: Object): SendSMSTask;







}

/**
* Sends any currently queued DataSource operations, as a single transactional request to the server.
*/
interface SendTransactionTask extends ProcessElement {




}
/**
* Creates a new SendTransactionTask
* 
* @param typeCheckedProperties {Partial<SendTransactionTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SendTransactionTask} The new instance
*/
class SendTransactionTask{
static create(typeCheckedProperties? : Partial<SendTransactionTask>, uncheckedProperties?: Object): SendTransactionTask;







}

/**
* A ServiceTask is an element of a Process which calls a DataSource operation, 
*  optionally using part of the Process.state as inputs or storing outputs
*  in the process state. A special "export" ServiceTask.operationType is
*  supported to perform a server export based on criteria.
*  
*  By default a ServiceTask takes the data indicated by Task.inputField and/or
*  Task.inputFieldList as detailed in taskIO and uses the
*  inputRecord as DSRequest.data. This means the input data becomes
*  Criteria for "fetch" and "export" operations, new record values for an "add" operation, etc. 
*  For simplicity, if no inputFieldList is provided and inputField
*  specifies an object, inputData is used as dsRequest.data.
*  
*  Alternatively, you can set ServiceTask.criteria for a "fetch" and "export" operations, or
*  ServiceTask.values for other operationTypes. In both cases, you have the ability to
*  use simple expressions like $input.fieldName to take portions of the input data and
*  use it as part of the criteria or values.
*  
*  OutputData and outputFieldList work as filters. You should determine which properties should
*  be fetched into the process state. If you want to load all data without defining every
*  property manually you can pass a name started with '$' and fetched record or records will be 
*  placed as a record or an array of records by the name without this specific symbol.
*  
*  For example if you specify 'id' and 'name' in outputFieldList, only these properties will be
*  fetched in the process state. If you pass '$orderHeader' in outputField a whole record will be 
*  stored in process state under the 'orderHeader' key.
*/
interface ServiceTask extends Task {
/**
* DataSource ID or DataSource instance to be used.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* Type of operation to invoke. A special "export" operation type is
*  supported to perform a server export based on criteria.
* @type {DSOperationType}
* @default "fetch"
*/
operationType?: DSOperationType;

/**
* The OperationBinding.operationId to invoke.
* @type {string}
* @default null
*/
operationId?: string;

/**
* Criteria (including AdvancedCriteria) to use for "fetch" and "export" operations.
*  
*  Data values in this criteria prefixed with "$" will be treated as dynamic expressions
*  which can access the inputs to this task as $input - see
*  taskInputExpression. Specifically, this means that for simple criteria,
*  any property value that is a String and is prefixed with "$" will be assumed to be an
*  expression, and for AdvancedCriteria, the same treatment will be applied to
*  Criterion.value.
*  
*  If any data value should not be treated as dynamic (for example, a "$" should be taken
*  as literal), you can place it in ServiceTask.fixedCriteria instead.
*  
*  Ignored for any operationType other than "fetch" and "export". Update or delete operations should
*  place the primary key to update in ServiceTask.values.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Process.ruleScope.
* @type {Criteria}
* @default null
*/
criteria?: Criteria;

/**
* Values to be submitted for "update", "add" and "remove" operations.
*  
*  Similar to Criteria, data values prefixed with "$" will be treated as a
*  taskInputExpression. Use ServiceTask.fixedValues for any values that start with
*  "$" but should be treated as a literal.
* @type {Record}
* @default null
*/
values?: Record;

/**
* Criteria to be submitted as part of the DSRequest, regardless of inputs to the task.
*  Will be combined with the data from the Task.inputField or with
*  ServiceTask.criteria if specified, via DataSource.combineCriteria.
* @type {Criteria}
* @default null
*/
fixedCriteria?: Criteria;

/**
* Values to be submitted as part of the DSRequest, regardless of inputs to the task. Will 
*  be combined with the data from the Task.inputField or with
*  ServiceTask.values if specified, via simple copying of fields, with
*  fixedValues overwriting values provided by the inputField, but
*  explicitly specified ServiceTask.values overriding fixedValues.
* @type {Record}
* @default null
*/
fixedValues?: Record;

/**
* The format in which the data should be exported. See ExportFormat for more 
*  information.
* @type {ExportFormat}
* @default "csv"
*/
exportFormat?: ExportFormat;

/**
* Field in the Process.state where this task writes outputs. See
*  taskIO.
*  
*  See ServiceTask.outputFieldList for a shorthand method to save the full operation response
*  data.
* @type {string}
* @default null
*/
outputField?: string;

/**
* List of multiple fields in the Process.state where this task will
*  write outputs. See taskIO. 
*  
*  If ServiceTask.outputField is also specified, it will be implicitly added to the
*  outputFieldList if it is not already present.
*  
*  In addition to pulling individual fields from the task operation result and placing
*  them into the process state the full response data can also be written into the
*  process state without specifying individual fields. Prefix a destination
*  field path with a "$" (ex. $orderHeader) causes the entire dsResponse.data
*  to be saved.
* @type {Array<Partial<string>>}
* @default null
*/
outputFieldList?: Array<Partial<string>>;

/**
* ID of the next sequence or element to proceed to if a failure condition arises
*  from DataSource operation.
* @type {string}
* @default null
*/
failureElement?: string;

/**
* Does this processElement pass through output from the last executed task
*  (i.e. transient state)?
*  
*  See taskInputExpression for details on the
*  transient state outputs.
*  
*  Note that this property does not affect the task at all but is an indicator
*  to the user and to the workflow editor of the behavior of the task as coded
*  (See Process.passThruTaskOutput).
* @type {boolean}
* @default false
*/
passThruOutput?: boolean;





}
/**
* Creates a new ServiceTask
* 
* @param typeCheckedProperties {Partial<ServiceTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ServiceTask} The new instance
*/
class ServiceTask{
static create(typeCheckedProperties? : Partial<ServiceTask>, uncheckedProperties?: Object): ServiceTask;







}

/**
* Gets the value properties from a component and makes them available within the workflow
*  as the taskInputExpression.
*  
*  For a canvas the ComponentTask.componentId specifies everything
*  necessary to identify the target.
*  For a form control more information is needed. The
*  ComponentTask.componentId identifies the container
*  (i.e. DynamicForm) and the individual field is specified as
*  SetPropertiesTask.targetFieldName.
*  
*  SetPropertiesTask is an advanced and rarely used task.
*  If you need a component property to be dynamic, you can configure that property as a
*  Class.dynamicProperties without the need for a
*  Workflow. Dynamic Properties created this way automatically
*  update as your users make changes.
*/
interface SetPropertiesTask extends ComponentTask {
/**
* Properties and associated values to be set on ComponentTask.componentId.
* @type {Array<Partial<PropertyValue>>}
* @default null
*/
properties?: Array<Partial<PropertyValue>>;

/**
* If ComponentTask.componentId targets a DynamicForm, this property
*  specifies the name of the target field.
* @type {FieldName}
* @default null
*/
targetFieldName?: FieldName;





}
/**
* Creates a new SetPropertiesTask
* 
* @param typeCheckedProperties {Partial<SetPropertiesTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SetPropertiesTask} The new instance
*/
class SetPropertiesTask{
static create(typeCheckedProperties? : Partial<SetPropertiesTask>, uncheckedProperties?: Object): SetPropertiesTask;







}

/**
* Sets an embedded screen's Canvas.dataContext with the
*  SetScreenDataTask.dataContextBinding evaluated
*  in the scope of this task.
*/
interface SetScreenDataTask extends ComponentTask {
/**
* A DataContextBinding to be applied to the screen via
*  Canvas.setDataContext.
* @type {DataContextBinding}
* @default null
*/
dataContextBinding?: DataContextBinding;





}
/**
* Creates a new SetScreenDataTask
* 
* @param typeCheckedProperties {Partial<SetScreenDataTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SetScreenDataTask} The new instance
*/
class SetScreenDataTask{
static create(typeCheckedProperties? : Partial<SetScreenDataTask>, uncheckedProperties?: Object): SetScreenDataTask;







}

/**
* Sets the title of a tab, section, window, label, button or form control.
*  
*  For a label, button or window the ComponentTask.componentId
*  specifies everything necessary to identify the target.
*  
*  For a tab, section or form control more information is needed. The
*  ComponentTask.componentId identifies the container
*  (i.e. TabSet, SectionStack or DynamicForm) and the individual component is specified as:
*  
*  
* - Tab - SetTitleTask.targetTabName references the Tab.name.
*  
* - Section - SetTitleTask.targetSectionName for 
*         SectionStackSection.name or SetTitleTask.targetSectionTitle 
*         for SectionStackSection.title.
*  
* - FormItem - SetTitleTask.targetFieldName for FormItem.name.
*  
*/
interface SetTitleTask extends ComponentTask {
/**
* Title to assign to component. To assign a dynamic value see SetTitleTask.textFormula.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* Formula to be used to calculate the component title. Use SetTitleTask.title property
*  to assign a static title instead.
*  
*  Available fields for use in the formula are the current Canvas.ruleScope.
* @type {UserSummary}
* @default null
*/
textFormula?: UserSummary;

/**
* If ComponentTask.componentId targets a TabSet, this property
*  specifies the name or ID of the target tab to assign new title.
* @type {TabName | GlobalId}
* @default null
*/
targetTabName?: TabName | GlobalId;

/**
* If ComponentTask.componentId targets a DynamicForm, this property
*  specifies the name of the target field to assign new title.
* @type {FieldName}
* @default null
*/
targetFieldName?: FieldName;

/**
* If ComponentTask.componentId targets a SectionStack, this property
*  specifies the name of the target section. Alternately, the target section can be
*  specified by using SetTitleTask.targetSectionTitle.
* @type {string}
* @default null
*/
targetSectionName?: string;

/**
* If ComponentTask.componentId targets a SectionStack, this property
*  specifies the title of the target section. Alternately, the target section can be
*  specified by using SetTitleTask.targetSectionName.
* @type {string}
* @default null
*/
targetSectionTitle?: string;





}
/**
* Creates a new SetTitleTask
* 
* @param typeCheckedProperties {Partial<SetTitleTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SetTitleTask} The new instance
*/
class SetTitleTask{
static create(typeCheckedProperties? : Partial<SetTitleTask>, uncheckedProperties?: Object): SetTitleTask;







}

/**
* Show a currently hidden component.
*/
interface ShowComponentTask extends ComponentTask {




}
/**
* Creates a new ShowComponentTask
* 
* @param typeCheckedProperties {Partial<ShowComponentTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ShowComponentTask} The new instance
*/
class ShowComponentTask{
static create(typeCheckedProperties? : Partial<ShowComponentTask>, uncheckedProperties?: Object): ShowComponentTask;







}

/**
* Show or hide a component. When showing, reveals any hidden parents as well.
*/
interface ShowHideTask extends ComponentTask {
/**
* If ComponentTask.componentId targets a TabSet, this property
*  specifies the name or ID of the target tab to assign new title.
* @type {TabName | GlobalId}
* @default null
*/
targetTabName?: TabName | GlobalId;

/**
* If ComponentTask.componentId targets a DynamicForm, this property
*  specifies the name of the target field to assign new title.
* @type {FieldName}
* @default null
*/
targetFieldName?: FieldName;

/**
* If ComponentTask.componentId targets a SectionStack, this property
*  specifies the name of the target section. Alternately, the target section can be
*  specified by using ShowHideTask.targetSectionTitle.
* @type {string}
* @default null
*/
targetSectionName?: string;

/**
* If ComponentTask.componentId targets a SectionStack, this property
*  specifies the title of the target section. Alternately, the target section can be
*  specified by using ShowHideTask.targetSectionName.
* @type {string}
* @default null
*/
targetSectionTitle?: string;

/**
* Should the target form item be hidden?
* @type {boolean}
* @default null
*/
hide?: boolean;

/**
* Set to false to not show a component's parents.
* @type {boolean}
* @default null
*/
showRecursively?: boolean;

/**
* Set to false to prevent scrolling the component into view when showing.
* @type {boolean}
* @default null
*/
scrollIntoView?: boolean;

/**
* Should focus be moved to target component when showing?
* @type {boolean}
* @default null
*/
moveFocusToTarget?: boolean;





}
/**
* Creates a new ShowHideTask
* 
* @param typeCheckedProperties {Partial<ShowHideTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ShowHideTask} The new instance
*/
class ShowHideTask{
static create(typeCheckedProperties? : Partial<ShowHideTask>, uncheckedProperties?: Object): ShowHideTask;







}

/**
* Show an informational message and wait for the user to acknowledge.
*/
interface ShowMessageTask extends ProcessElement {
/**
* Message type.
* @type {ShowMessageType}
* @default "normal"
*/
type?: ShowMessageType;

/**
* Message to display. To display a dynamic message see ShowMessageTask.textFormula.
* @type {string}
* @default null
*/
message?: string;

/**
* Formula to be used to calculate the message contents. Use ShowMessageTask.message property
*  to assign a static message instead.
*  
*  Available fields for use in the formula are the current Canvas.ruleScope.
* @type {UserSummary}
* @default null
*/
textFormula?: UserSummary;





}
/**
* Creates a new ShowMessageTask
* 
* @param typeCheckedProperties {Partial<ShowMessageTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ShowMessageTask} The new instance
*/
class ShowMessageTask{
static create(typeCheckedProperties? : Partial<ShowMessageTask>, uncheckedProperties?: Object): ShowMessageTask;







}

/**
* Show a component next to some other component.
*/
interface ShowNextToComponentTask extends ComponentTask {
/**
* The other component where this component will show.
* @type {GlobalId}
* @default null
*/
nextToComponentId?: GlobalId;

/**
* Which side of the other canvas should we show? Options are
*  "top", "bottom", "left", "right". (Defaults to "right")
* @type {string}
* @default null
*/
side?: string;

/**
* Can this component can be positioned on top of the other
*  component if there isn't room to show next to it?
* @type {boolean}
* @default null
*/
canOcclude?: boolean;

/**
* Set to false to not use animation to show component.
* @type {boolean}
* @default null
*/
skipAnimation?: boolean;





}
/**
* Creates a new ShowNextToComponentTask
* 
* @param typeCheckedProperties {Partial<ShowNextToComponentTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ShowNextToComponentTask} The new instance
*/
class ShowNextToComponentTask{
static create(typeCheckedProperties? : Partial<ShowNextToComponentTask>, uncheckedProperties?: Object): ShowNextToComponentTask;







}

/**
* Show a message which fades out automatically using
*  Notify.
*/
interface ShowNotificationTask extends ProcessElement {
/**
* Auto-dismiss message after a short duration.
* @type {boolean}
* @default true
*/
autoDismiss?: boolean;

/**
* Message to display. To display a dynamic message see ShowNotificationTask.textFormula.
* @type {string}
* @default null
*/
message?: string;

/**
* Formula to be used to calculate the message contents. Use ShowNotificationTask.message property
*  to assign a static message instead.
*  
*  Available fields for use in the formula are the current Canvas.ruleScope.
* @type {UserSummary}
* @default null
*/
textFormula?: UserSummary;

/**
* Where to show the message, specified as an edge ("T", "B", "R", "L") similar to
*  Canvas.snapTo, or "C" for center. The message will be shown at the center of the
*  edge specified (or the very center for "C").
* @type {string}
* @default "T"
*/
position?: string;

/**
* NotifyType for message.
* @type {NotifyType}
* @default "message"
*/
notifyType?: NotifyType;





}
/**
* Creates a new ShowNotificationTask
* 
* @param typeCheckedProperties {Partial<ShowNotificationTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ShowNotificationTask} The new instance
*/
class ShowNotificationTask{
static create(typeCheckedProperties? : Partial<ShowNotificationTask>, uncheckedProperties?: Object): ShowNotificationTask;







}

/**
* Simple subclass of Button used for tabs in a TabSet if TabSet.useSimpleTabs
*  is true. See also TabSet.simpleTabButtonConstructor.
*/
interface SimpleTabButton extends Button {




}
/**
* Creates a new SimpleTabButton
* 
* @param typeCheckedProperties {Partial<SimpleTabButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SimpleTabButton} The new instance
*/
class SimpleTabButton{
static create(typeCheckedProperties? : Partial<SimpleTabButton>, uncheckedProperties?: Object): SimpleTabButton;







}

/**
* Default class used by a TileGrid to render each tile. See TileGrid.tile.
*  
*  SimpleTiles should not be created directly, instead, use a TileGrid and provide data and
*  SimpleTile instances are created for you.
*/
interface SimpleTile extends StatefulCanvas {
/**
* CSS style for the tile as a whole. As with StatefulCanvas.baseStyle, suffixes
*  are appended to this style to represent various states ("Over", "Selected", etc).
* @type {CSSStyleName}
* @default "simpleTile"
*/
baseStyle?: CSSStyleName;

/**
* The TileGrid that created this SimpleTile. This property will be null
*  if the tile was created by a user-provided TileGrid.createTile method.
* @deprecated * As of SmartClient 10.0, use SimpleTile.tileGrid.

* @type {TileGrid}
* @default null
*/
creator?: TileGrid;

/**
* The TileGrid that created this SimpleTile.
* @type {TileGrid}
* @default null
*/
tileGrid?: TileGrid;



/**
*  The default implementation will call TileGrid.getTileHTML.
* 
*/
getInnerHTML(): HTMLString;

/**
*  Return the record that this tile should render.
*  
*  NOTE: a TileGrid that is doing data paging may reuse tiles with different records, so a
*  subclass of SimpleTile should not cache the record returned by getRecord().
*  
* 
*/
getRecord(): TileRecord;



}
/**
* Creates a new SimpleTile
* 
* @param typeCheckedProperties {Partial<SimpleTile>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SimpleTile} The new instance
*/
class SimpleTile{
static create(typeCheckedProperties? : Partial<SimpleTile>, uncheckedProperties?: Object): SimpleTile;







}

/**
* An atomic type such as a string or number, that is generally stored, displayed and
*  manipulated as a single value.
*  
*  SimpleTypes can be created at any time, and subsequently referred to as a 
*  DataSourceField.type in DataSource and
*  DataBoundComponent. This allows you to define
*  SimpleType.validators, SimpleType.normalDisplayFormatter
*  and SimpleType.editorType behaviors for a type to be reused across all
*  DataBoundComponent.
*  
*  The SimpleType class also allows data to be stored in some opaque format but treated as
*  simple atomic values as far as SmartClient components are concerned by implementing
*  SimpleType.getAtomicValue and SimpleType.updateAtomicValue methods.
*  For example, if some record has a field value set to a javascript object with the
*  following properties:
*  
*  { stringValue:"A String", length: 9 }
*  
*  this value could be treated as a simple string by defining a SimpleType with 
*  SimpleType.inheritsFrom set to "text" and a custom 
*  getAtomicValue() method that simply extracted the "stringValue"
*  attribute from the data object. DataBoundComponents would then display
*  the string value, and use it for sorting and other standard databinding features.
*  
*  Note that the term "simpleType" is used in the same sense as in
*  XML Schema, and
*  XMLTools.loadXMLSchema will create new SimpleType definitions.
*  
*  When using the SmartClient Server, SimpleTypes can be defined server-side, and should
*  be defined server-side if validators are going to be declared so that the server will
*  enforce validation. To define server-side SimpleTypes using Component XML you should create
*  file {typeName}.type.xml in the following format:
*  
*   &lt;SimpleType name="{typeName}" inheritsFrom="{otherSimpleType}" 
*          editorType="{FormItemClassName}"&gt;
*    &lt;validators&gt;
*     &lt;!-- validator definition just like DataSourceField --&gt;
*    &lt;/validators&gt;
*   &lt;/SimpleType&gt;
*  
*  .. and place this file alongside your DataSource files (.ds.xml) files - in any of folders
*  listed in project.datasources property in server_properties.
*  
*  SimpleTypes can be loaded via DataSourceLoader or loadDSTag and
*  should be loaded before the definitions of any DataSources that use them (so
*  generally put all SimpleType definitions first).
*  
*  Define validators in the server-side type definition, for example:
*  
*   &lt;SimpleType name="countryCodeType" inheritsFrom="text"&gt;
*    &lt;validators&gt;
*     &lt;validator type="lengthRange" min="2" max="2"
*      errorMessage="Length of country code should be equal to 2." /&gt;
*     &lt;validator type="regexp" expression="[A-Z][A-Z]"
*      errorMessage="CountryCode should have only uppercase letters." /&gt;
*    &lt;/validators&gt;
*   &lt;/SimpleType&gt;
*  
*  
*  For client-side formatters, add these to the type definition after loading it from the
*  server, for example:
*  
*   
*    isc.SimpleType.getType("independenceDateType").addProperties({
*      normalDisplayFormatter : function (value) {
*        if (value == null) return "";
*        return "&lt;i&gt;" + (value.getYear() + 1900) + "&lt;/i&gt;";
*      }
*    });
*   
*  
*  
*  Note that formatters must be added to the SimpleType definition before any
*  DataBoundComponent binds to a DataSource that uses the SimpleType.
*  
*  An example is ${isc.DocUtils.linkForExampleId('formsCustomSimpleType', 'here')}.
*  
*/
interface SimpleType extends Class {
/**
* Name of the type, used to refer to the type from DataSourceField.type.
* @type {Identifier}
* @default null
*/
name?: Identifier;

/**
* Name of another SimpleType from which this type should inherit.
*  
*  Validators, if any, will be combined. All other SimpleType properties default to the
*  inherited type's value.
* @type {Identifier}
* @default null
*/
inheritsFrom?: Identifier;

/**
* Validators to apply to value of this type.
* @type {Array<Partial<Validator>>}
* @default null
*/
validators?: Array<Partial<Validator>>;

/**
* List of legal values for this type, like DataSourceField.valueMap.
* @type {ValueMap}
* @default null
*/
valueMap?: ValueMap;

/**
* Default value for DataSourceField.canEdit for fields of this type.
*  
*  This impacts client-side behavior only and is a way to simply disallow 
*  editing of this field type by default within FormItem.canEdit.
*  
*  This property is set to false for the "sequence" SimpleType by default.
* @type {boolean}
* @default null
*/
canEdit?: boolean;

/**
* Default value for DataSourceField.canFilter for fields of this type.
*  
*  This impacts client-side behavior only and may be used to explicitly enable
*  editing in filter interfaces, even if SimpleType.canEdit.
*  
*  This property is set to true for the "sequence" SimpleType by default.
* @type {boolean}
* @default null
*/
canFilter?: boolean;

/**
* Classname of the FormItem that should be the default for editing values of this type (eg
*  "SelectItem").
*  
*  You can create a simple custom FormItem by adding default FormItem.icons that
*  launch custom value picking dialogs (an example is in the QuickStart
*  Guide, Chapter 9, Extending SmartClient). By setting simpleType.editorType
*  to the name of your custom FormItem, forms will automatically use the custom FormItem,
*  as will grids performing ListGrid.canEdit.
* @type {FormItemClassName}
* @default null
*/
editorType?: FormItemClassName;

/**
* Default FormItem.readOnlyDisplay for fields of this type.
*  
*  For more sophisticated management of read-only behavior, see SimpleType.readOnlyEditorType.
* @type {ReadOnlyDisplayAppearance}
* @default null
*/
readOnlyDisplay?: ReadOnlyDisplayAppearance;

/**
* Classname of the FormItem that should be used to display values of this type when a field
*  is marked as DataSourceField.canEdit and the field is displayed
*  in an editor type component like a DynamicForm.
*  
*  May be overridden by DataSourceField.readOnlyEditorType.
* @type {FormItemClassName}
* @default null
*/
readOnlyEditorType?: FormItemClassName;

/**
* Classname of the FormItem that should be used to edit values of this type if it appears
*  in a filter row.
*  
*  May be overridden by DataSourceField.filterEditorType.
* @type {FormItemClassName}
* @default null
*/
filterEditorType?: FormItemClassName;

/**
* These are properties that are essentially copied onto any DataSourceField where the
*  property is applied. The supported properties are only client-side properties.
* @type {Partial<DataSourceField>}
* @default null
*/
fieldProperties?: Partial<DataSourceField>;

/**
* FormatString for numeric or date formatting. See DataSourceField.format.
* @type {FormatString}
* @default null
*/
format?: FormatString;

/**
* FormatString used during exports for numeric or date formatting. See
*  DataSourceField.exportFormat.
* @type {FormatString}
* @default null
*/
exportFormat?: FormatString;

/**
* Set of OperatorId valid for this SimpleType. 
*  
*  If not specified, the SimpleType.inheritsFrom type's operators will be used, finally
*  defaulting to the default operators for the basic types (eg, integer).
* @type {Array<Partial<OperatorId>>}
* @default null
*/
validOperators?: Array<Partial<OperatorId>>;

/**
* The default OperatorId for this data-type.
* @type {OperatorId}
* @default null
*/
defaultOperator?: OperatorId;

/**
* A set of key-value pairs that represent the names and titles of the grouping modes 
*  available to values of this type, for use in components that support grouping.
*  
*  Some types provide a set of builtin groupingModes, as covered 
*  builtinGroupingModes.
*  
*  Use SimpleType.getGroupValue and SimpleType.getGroupTitle to implement
*  custom grouping logic for each of the grouping modes you provide.
* @type {ValueMap}
* @default null
*/
groupingModes?: ValueMap;

/**
* In components that support grouping, the default mode from the available 
*  SimpleType.groupingModes to use when grouping values of this type.
* @type {string}
* @default null
*/
defaultGroupingMode?: string;



/**
*  Optional method to extract an atomic value (such as a string or number)
*  from some arbitrary live data value. If defined, this method will be called
*  for every field value of the specified type in order to convert from the
*  raw data value to an atomic type to be used for standard DataBinding features
*  such as sorting and editing.
*  
*  The "reason" parameter is passed by the framework to indicate why it is asking for the 
*  atomic value. Your method can use this value to affect the atomic value that is 
*  returned - for example, if the reason is "sort" you could return the atomic value 
*  converted to upper-case, to impose case-insensitive sorting. Reason strings used 
*  by the framework are:
*  
* - "edit" Retrieving the edit value of the field in a DynamicForm or 
*         ListGrid
*  
* - "format" Retrieving the value to format it for display
*  
* - "mask" Retrieving the value to present it for masked input
*  
* - "filter" Retrieving the value for use in a filter comparison
*  
* - "sort" Retrieving the value for use in a sort comparison
*  
* - "group" Retrieving the value for use in a group comparison
*  
* - "formula" Retrieving the value for use in a formula calculation
*  
* - "vm_getValue" Retrieving the value from ValuesManager.getValue
*  
* - "validate" Retrieving the value for validation, or setting the value if validation
*           caused it to change
*  
* - "compare" Retrieving the "old" or "new" value from ListGrid.cellHasChanges
*  
* - "getRawValue" Retrieving the raw value of a ListGrid cell
*  
* - "criteria" Setting the value from DynamicForm.setValuesAsCriteria
*  
* - "updateValue" Setting the value from internal methods of DynamicForm 
*            or ValuesManager 
*  
* - "setRawValue" Setting the raw value of a ListGrid cell
*  
* - "saveLocally" Setting the value from ListGrid.saveLocally
*  
* 
* 
* @param {any} Raw data value to convert. Typically this would be a field  value for some record.
* @param {string} The reason your getAtomicValue() method is being  called
*/
getAtomicValue(value:any, reason:string): any;

/**
*  Optional method to update a live data value with an edited atomic value
*  (such as a string or number). If defined this method will be called
*  when the user edits data in a field of this type, allowing the developer
*  to convert from the atomic type to a raw data value for storage.
*  
*  Note that if the user is editing a field which did not previously have a value, the
*  'currentValue' will be null. This method should handle this (creating a new data value).
* 
* 
* @param {any} New atomic value. This should match the underlying atomic type specified by the SimpleType.inheritsFrom attribute.
* @param {any} Existing data value to be updated.
* @param {string} The reason your updateAtomicValue() method is being  called. See SimpleType.getAtomicValue for the reason strings used by the framework
*/
updateAtomicValue(atomicValue:any, currentValue:any, reason:string): any;

/**
*  Optional method to allow you to write a custom comparator for this SimpleType. If 
*  implemented, this method will be used by the framework when it needs to compare two
*  values of a field for equality - for example, when considering if an edited field 
*  value has changed. If you do not implement this method, values will be compared using
*  standard techniques, so you should only provide an implementation if you have some 
*  unusual requirement.
*  
*  Implementations of this method should return the following:
*  
* - 0 if the two values are equal
*  
* - -1 if the first value is greater than the second
*  
* - 1 if the second value is greater than the first
*  
* 
* 
* @param {any} First value for comparison
* @param {any} Second value for comparison
* @param {DataSourceField | ListGridField | DetailViewerField | FormItem} Field definition from a dataSource or dataBoundComponent.
*/
compareValues(value1:any, value2:any, field:DataSourceField | ListGridField | DetailViewerField | FormItem): Integer;

/**
*  Formatter for values of this type when compact display is required, for example, in a
*  ListGrid or TreeGrid.
*  
*  When this formatter is called, the SimpleType object is available as "this". 
*  
*  A formatter can make itself configurable on a per-component or per-field basis by
*  checking properties on the component or field. For example, a formatter for account IDs
*  may want to omit a prefix in views where it is redundant, and could check a flag
*  listGridField.omitAccountIdPrefix for this purpose.
* 
* 
* @param {any} value to be formatted
* @param {string=} field descriptor from the component calling the formatter, if           applicable. Depending on the calling component, this could be a            ListGridField, TreeGridField, etc
* @param {DataBoundComponent=} component calling this formatter, if applicable
* @param {object=} Full record, if applicable
*/
shortDisplayFormatter(value:any, field?:string, component?:DataBoundComponent, record?:object): void;

/**
*  Normal formatter for values of this type used in a StaticTextItem or
*  DetailViewer.
*  
*  When this formatter is called, the SimpleType object is available as "this". 
*  
*  A formatter can make itself configurable on a per-component or per-field basis by
*  checking properties on the component or field. For example, a formatter for account IDs
*  may want to omit a prefix in views where it is redundant, and could check a flag
*  detailViewer.omitAccountIdPrefix for this purpose.
* 
* 
* @param {any} value to be formatted
* @param {string=} field descriptor from the component calling the formatter, if           applicable. Depending on the calling component, this could be a            FormItem, DetailViewerField, etc
* @param {DataBoundComponent=} component calling this formatter, if applicable
* @param {object=} Full record, if applicable
*/
normalDisplayFormatter(value:any, field?:string, component?:DataBoundComponent, record?:object): void;

/**
*  Formatter for values of this type when displayed in a freeform text editor, such as
*  a TextItem.
*  
*  See also SimpleType.parseInput for parsing an edited text value back to
*  a data value.
* 
* @param {any} value to be formatted
* @param {FormItem=} Editor for this field
* @param {DynamicForm=} DynamicForm containing this editor
* @param {Record=} Current edit values for this record, as displayed in   the edit component.
*/
editFormatter(value:any, field?:FormItem, form?:DynamicForm, record?:Record): string;

/**
*  Parser to convert some user-edited value to an underlying data value of this type.
*  This parser is called when storing out values edited in a freeform editor such as
*  a TextItem. Typically this will convert from the format produced by 
*  SimpleType.editFormatter back to a data value.
* 
* 
* @param {string} edited value provided by the user
* @param {FormItem=} Editor for this field
* @param {DynamicForm=} DynamicForm containing this editor
* @param {Record=} Current edit values for this record, as displayed in   the edit component.
*/
parseInput(value:string, field?:FormItem, form?:DynamicForm, record?:Record): any;

/**
*  Returns the set of SimpleType.groupingModes available for values 
*  of this type in components that support grouping.
* 
*/
getGroupingModes(): ValueMap;

/**
*  Returns a group value appropriate for the passed record, field and value, in the passed 
*  component.
*  
* 
* @param {any} the record value to return a group value for
* @param {Record} the record containing the passed value
* @param {object} the field relating to the value to be processed
* @param {string} the name of the field relating to the value to be processed
* @param {Canvas} the component, usually a ListGrid, containing the              passed record
*/
getGroupValue(value:any, record:Record, field:object, fieldName:string, component:Canvas): any;

/**
*  Returns a string value appropriate for the title of the group containing the passed
*  value.
*  
* 
* @param {any} the record value to return a group title for
* @param {Record} the record containing the passed group value
* @param {object} the field relating to the value to be processed
* @param {string} the name of the field relating to the value to be processed
* @param {Canvas} the component, usually a ListGrid, containing the              passed record
*/
getGroupTitle(value:any, record:Record, field:object, fieldName:string, component:Canvas): string;



}
/**
* Creates a new SimpleType
* 
* @param typeCheckedProperties {Partial<SimpleType>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SimpleType} The new instance
*/
class SimpleType{
static create(typeCheckedProperties? : Partial<SimpleType>, uncheckedProperties?: Object): SimpleType;




/**
*  Retrieve a simpleType definition by type name
* 
* @param {string} the name of the simpleType to return
*/
static getType(typeName:string): SimpleType;

/**
*  Registers a new SummaryFunction by name. After calling this method,
*  developers may specify the name passed in as a standard summaryFunction
*  (for example in ListGridField.summaryFunction).
* 
* @param {string} name for the newly registered summaryFunction
* @param {Function} New summary function. This function should take 2 parameters  
- records: an array of records for which a summary must be generated  
- field: a field definition  
- summaryConfig: summary configuration (see SummaryConfiguration)  and return a summary value for the field across the records.
*/
static registerSummaryFunction(functionName:string, method:Function): void;

/**
*  Set up a default summary function for some field type.
*  
*  Note that the following default summary functions are set up when SmartClient initializes:
*  - "integer" defaults to "sum"
*  - "float" defaults to "sum".
* 
* 
* @param {string} type name
* @param {SummaryFunction} summary function to set as the default for  this data type.
*/
static setDefaultSummaryFunction(typeName:string, summaryFunction:SummaryFunction): void;

/**
*  Retrieves the default summary function for some field type.
* 
* @param {string} type name
*/
static getDefaultSummaryFunction(typeName:string): SummaryFunction;

/**
*  Applies a SummaryFunction to an array of records
* 
* @param {Array<Partial<object>>} set of records to retrieve a summary value for
* @param {DataSourceField} field for which we're picking up a summary value
* @param {SummaryFunction} SummaryFunction to apply to the records in order to retrieve the summary value. May be specified as an explicit function  or string of script to execute, or a SummaryFunction identifier
* @param {SummaryConfiguration} config that affects summary calculation
*/
static applySummaryFunction(records:Array<Partial<object>>, field:DataSourceField, summaryFunction:SummaryFunction, summaryConfig:SummaryConfiguration): any;




}

/**
* The Slider class implements a GUI slider widget allowing the user to select a numeric 
*  value from within a range by dragging a visual indicator up and down a track.
* 	
*  The slider will generate events as the user interacts with it and changes its value.
*  If slider.sliderTarget is specified, moving the slider thumb generates a custom
* 	event named 'sliderMove', sent to the sliderTarget.
*  If a sliderMove handler stringMethod is defined on the target, it will be 
*  fired when the slider is moved. The second parameter (available via the variable name
*  eventInfo if the handler is a string) is a pointer back to the slider.
*  
*  The slider will also fire a valueChanged() method whenever its value is 
*  changed. This can be observed or overridden on the Slider instance to perform some action.
*/
interface Slider extends Canvas {
/**
* Optional display title for the slider.
* @type {string}
* @default "Set Value"
*/
title?: string;

/**
* Used to set slider height if vertical, slider width if horizontal.
*  Applied to the slider track, not necessarily the entire widget.
*  Overridden by an explicit width/height specification for the widget.
* @type {Integer}
* @default "200"
*/
length?: Integer;

/**
* Indicates whether this is a vertical or horizontal slider.
* @type {boolean}
* @default true
*/
vertical?: boolean;

/**
* The dimension of the thumb perpendicular to the slider track.
* @type {Integer}
* @default "23"
*/
thumbThickWidth?: Integer;

/**
* The dimension of the thumb parallel to the slider track.
* @type {Integer}
* @default "17"
*/
thumbThinWidth?: Integer;

/**
* The thickness of the track. This is the width, for a Slider.vertical 
*  slider, or the height, for a horizontal slider.
* @type {Integer}
* @default "7"
*/
trackWidth?: Integer;

/**
* Optional CSS style for the thumb for a Slider.vertical
*  slider.
*  
*  Will have the suffix "down" added when the mouse is down on the thumb, and "Disabled"
*  added when the slider is disabled.
* @type {CSSStyleName}
* @default null
*/
hThumbStyle?: CSSStyleName;

/**
* Optional CSS style for the thumb for a Slider.vertical 
*  slider. See Slider.hThumbStyle for state suffixes.
* @type {CSSStyleName}
* @default null
*/
vThumbStyle?: CSSStyleName;

/**
* Optional CSS style for the track for a Slider.vertical 
*  slider.
*  
*  Will have the suffix "Disabled" added when the slider is disabled.
* @type {CSSStyleName}
* @default null
*/
hTrackStyle?: CSSStyleName;

/**
* Optional CSS style for the track for a Slider.vertical 
*  slider.
*  
*  Will have the suffix "Disabled" added when the slider is disabled.
* @type {CSSStyleName}
* @default null
*/
vTrackStyle?: CSSStyleName;

/**
* The base filename for the slider thumb images.
*  The filenames for the thumb icons are assembled from this base filename and the state of the
*  thumb, as follows:
*  Assume the thumbSrc is set to {baseName}.{extension}
*  The full set of images to be displayed is:
*  For horizontal sliders:
*  
*  
* - h{baseName}.{extension}: default enabled appearance.
*  
* - h{baseName}_down.{extension}: appearance when the slider is enabled and the
*    thumb is clicked.
*  
* - h{baseName}_Disabled.{extension}: appearance when the slider is disabled.
*  
*  For vertical sliders:
*  
*  
* - v{baseName}.{extension}: default enabled appearance.
*  
* - v{baseName}_down.{extension}: appearance when the slider is enabled and the
*    thumb is clicked.
*  
* - v{baseName}_Disabled.{extension}: appearance when the slider is disabled.
*  
* @type {string}
* @default "thumb.gif"
*/
thumbSrc?: string;

/**
* The base filename for the slider track images.
*  The filenames for the track icons are assembled from this base filename and the state of the
*  slider, as follows:
*  Assume the trackSrc is set to {baseName}.{extension}
*  The full set of images to be displayed is:
*  For horizontal sliders:
*  
*  
* - h{baseName}_start.{extension}: start (left edge) of the track for a slider
*    that is enabled.
*  
* - h{baseName}_stretch.{extension}: the track for an enabled slider; this may
*    be centered, tiled, or stretched.
*  
* - h{baseName}_end.{extension}: end (right edge) of the track for a slider
*    that is enabled.
*  
* - h{baseName}_Disabled_start.{extension}: start (left edge) of the track for a slider
*    that is disabled.
*  
* - h{baseName}_Disabled_stretch.{extension}: the track for a disabled slider; this
*    may be centered, tiled, or stretched.
*  
* - h{baseName}_Disabled_end.{extension}: end (right edge) of the track for a slider
*    that is disabled.
*  
*  For vertical sliders:
*  
*  
* - v{baseName}_start.{extension}: start (bottom edge) of the track for a slider
*    that is enabled.
*  
* - v{baseName}_stretch.{extension}: the track for an enabled slider; this may
*    be centered, tiled, or stretched.
*  
* - v{baseName}_end.{extension}: end (top edge) of the track for a slider
*    that is enabled.
*  
* - v{baseName}_Disabled_start.{extension}: start (bottom edge) of the track for a slider
*    that is disabled.
*  
* - v{baseName}_Disabled_stretch.{extension}: the track for a disabled slider; this
*    may be centered, tiled, or stretched.
*  
* - v{baseName}_end.{extension}: end (top edge) of the track for a slider
*    that is disabled.
*  
* @type {string}
* @default "track.gif"
*/
trackSrc?: string;

/**
* The height of Slider.vertical slider start and end images, or width of 
*  horizontal slider start and end images.
* @type {Integer}
* @default "6"
*/
trackCapSize?: Integer;

/**
* The imageType setting for the slider track.
* @type {ImageStyle}
* @default "stretch"
*/
trackImageType?: ImageStyle;

/**
* Indicates whether the slider's Slider.title should be displayed. The 
*  default position for the title-label is to the left of a horizontal slider, or above a 
*  Slider.vertical slider.
* @type {boolean}
* @default true
*/
showTitle?: boolean;

/**
* Indicates whether labels for the Slider.rangeLabel of the 
*  slider should be displayed. The default positions for these labels are below the 
*  start/end of a horizontal slider, or to the right of the start/end of a 
*  Slider.vertical slider.
* @type {boolean}
* @default true
*/
showRange?: boolean;

/**
* Indicates whether a Slider.valueLabel for the value of the slider should 
*  be displayed. The default position for this label is to the right of a 
*  Slider.vertical slider, 
*  or below a horizontal slider.
* @type {boolean}
* @default true
*/
showValue?: boolean;

/**
* The width of the labels used to display the minimum, maximum and current values of the
*  slider.
* @type {Integer}
* @default "50"
*/
labelWidth?: Integer;

/**
* The height of the labels used to display the 
*  Slider.rangeLabel and Slider.valueLabel values 
*  of the slider.
* @type {Integer}
* @default "20"
*/
labelHeight?: Integer;

/**
* The space around the labels used to display the 
*  Slider.rangeLabel and Slider.valueLabel values 
*  of the slider.
* @type {Integer}
* @default "5"
*/
labelSpacing?: Integer;

/**
* The space around the labels used to display the 
*  Slider.rangeLabel and Slider.valueLabel values 
*  of the slider, when Slider.vertical is true. If unset, defaults to 
*  Slider.labelSpacing.
* @type {Integer}
* @default null
*/
vLabelSpacing?: Integer;

/**
* The space around the labels used to display the 
*  Slider.rangeLabel and Slider.valueLabel values 
*  of the slider, when Slider.vertical is false. If unset, defaults to 
*  Slider.labelSpacing.
* @type {Integer}
* @default null
*/
hLabelSpacing?: Integer;

/**
* The space between the Slider.showTitle and the track.
* @type {Integer}
* @default "5"
*/
titleSpacing?: Integer;

/**
* CSS style for the Slider.title, when 
*  Slider.showTitle is true.
* @type {CSSStyleName}
* @default "sliderTitle"
*/
titleStyle?: CSSStyleName;

/**
* CSS style for the Slider.rangeLabel range-labels, when 
*  Slider.showRange is true.
* @type {CSSStyleName}
* @default "sliderRange"
*/
rangeStyle?: CSSStyleName;

/**
* CSS style for the floating Slider.valueLabel, visible when
*  Slider.showValue is true.
* @type {CSSStyleName}
* @default "sliderValue"
*/
valueStyle?: CSSStyleName;

/**
* Optional CSS style for the floating Slider.valueLabel, visible when
*  Slider.showValue is true and Slider.vertical is true.
* @type {CSSStyleName}
* @default null
*/
vValueStyle?: CSSStyleName;

/**
* Optional CSS style for the floating Slider.valueLabel, visible when
*  Slider.showValue is true and Slider.vertical is false.
* @type {CSSStyleName}
* @default null
*/
hValueStyle?: CSSStyleName;

/**
* CSS style for the text in the floating Slider.valueLabel, 
*  visible when Slider.showValue is true.
* @type {CSSStyleName}
* @default "sliderValueText"
*/
valueTextStyle?: CSSStyleName;

/**
* The slider value. This value should lie between the minValue and maxValue and increases as
*  the thumb is moved up (for a vertical slider) or right (for a horizontal slider) unless
*  flipValues is set to true.
* @type {Float}
* @default "1"
*/
value?: Float;

/**
* The minimum slider value. The slider value is equal to minValue when the thumb is at the
*  bottom or left of the slider (unless flipValues is true, in which case the minimum value
*  is at the top/right of the slider)
* @type {Float}
* @default "1"
*/
minValue?: Float;

/**
* The text displayed in the label for the minimum value of the slider. If left as null, then
*  slider.minValue will be displayed.
* @type {string}
* @default null
*/
minValueLabel?: string;

/**
* The maximum slider value. The slider value is equal to maxValue when the thumb is at the
*  top or right of the slider (unless flipValues is true, in which case the maximum value
*  is at the bottom/left of the slider)
* @type {Float}
* @default "100"
*/
maxValue?: Float;

/**
* The text displayed in the label for the maximum value of the slider. If left as null, then
*  slider.maxValue will be displayed.
* @type {string}
* @default null
*/
maxValueLabel?: string;

/**
* The number of discrete values represented by slider. If specified, the range of valid
*  values (between minValue and maxValue) will be divided into
*  this many steps. As the thumb is moved along the track it will only select these values
*  and appear to jump between the steps.
* @type {Integer}
* @default null
*/
numValues?: Integer;

/**
* Specifies whether the slider value should be rounded to the nearest integer. If set to
*  false, values will be rounded to a fixed number of decimal places controlled by
*  Slider.roundPrecision.
* @type {boolean}
* @default true
*/
roundValues?: boolean;

/**
* If Slider.roundValues is false, the slider value will be rounded to this number of
*  decimal places. If set to null the value will not be rounded
* @type {Integer}
* @default "1"
*/
roundPrecision?: Integer;

/**
* Specifies whether the value range of the slider should be flipped so that values increase as
*  the thumb is moved down (for a Slider.vertical slider) or to the left 
*  (for a horizontal slider).
* @type {boolean}
* @default false
*/
flipValues?: boolean;

/**
* The target widget for the sliderMove event generated when the slider thumb 
*  is moved.
* @type {Canvas}
* @default null
*/
sliderTarget?: Canvas;

/**
* Indicates whether keyboard manipulation of the slider is allowed.
* @type {boolean}
* @default true
*/
canFocus?: boolean;

/**
* The percentage of the total slider that constitutes one discrete step. The slider will move
*  one step when the appropriate arrow key is pressed.
* @type {Float}
* @default "5"
*/
stepPercent?: Float;

/**
* Should the thumb be animated to its new position when the value is changed programmatically,
*  or by clicking in the slider track.
* @type {boolean}
* @default true
*/
animateThumb?: boolean;

/**
* Duration of thumb animation, in milliseconds.
* @type {Integer}
* @default "250"
*/
animateThumbTime?: Integer;

/**
* If thumb animation is enabled, should the thumb be animated to its initial value?
* @type {boolean}
* @default false
*/
animateThumbInit?: boolean;

/**
* Used to create both of the min and max range-labels, via the AutoChild pattern, hence
*  rangeLabelConstructor, rangeLabelDefaults and
*  rangeLabelProperties are valid.
* @type {Label}
* @default null
*/
rangeLabel?: Label;

/**
* FormatString for numeric formatting of the value and range labels.
* @type {FormatString}
* @default null
*/
valueFormat?: FormatString;

/**
* FormatString for numeric formatting of the range labels. If unset, defaults to 
*  Slider.valueFormat
* @type {FormatString}
* @default null
*/
rangeFormat?: FormatString;

/**
* AutoChild displaying the current value as a floating label when 
*  Slider.showValue is true.
* @type {Label}
* @default null
*/
valueLabel?: Label;

/**
* Whether to show the Slider.activeTrack, which highlights the 'active' 
*  portion of a slider, from its minimum to its current Slider.value.
* @type {boolean}
* @default false
*/
showActiveTrack?: boolean;

/**
* CSS style used to highlight the Slider.showActiveTrack part of the
*  slider track.
*  
*  Will have the suffix "Disabled" added when the slider is disabled.
* @type {CSSStyleName}
* @default "sliderTrackActive"
*/
activeTrackStyle?: CSSStyleName;

/**
* A styled canvas used to highlight the Slider.showActiveTrack part of the
*  slider track.
* @type {StatefulCanvas}
* @default null
*/
activeTrack?: StatefulCanvas;



/**
*  Sets the slider value to newValue and moves the slider thumb to the appropriate position for this
*  value. Sends the 'sliderMove' event to the sliderTarget.
* 
* 
* @param {Float} the new value 
* @param {boolean} do not animate the slider thumb to the new value
*/
setValue(newValue:Float, noAnimation:boolean): void;

/**
*  Returns the current slider value.
* 
* 
*/
getValue(): Float;

/**
*  This method is called when the slider value changes. This occurs when the Slider.setValue
*  method is called, or when the slider is moved. Observe this method to be notified when
*  the slider value
*  changes.
* 
* 
* @param {Double} the new value.
*/
valueChanged(value:Double): void;

/**
*  Call this method in your Slider.valueChanged handler to determine whether the
*  value change is due to an ongoing drag interaction (true) or due to a thumb-release,
*  mouse click, keypress, or programmatic event (false). You may choose to execute temporary or
*  partial updates while the slider thumb is dragged, and final updates or persistence of the value
*  in response to the other events.
* 
* 
*/
valueIsChanging(): boolean;

/**
*  Sets the Slider.minValue of the slider 
* 
* 
* @param {Float} the new minimum value 
*/
setMinValue(newValue:Float): void;

/**
*  Sets the Slider.maxValue of the slider 
* 
* 
* @param {Float} the new maximum value 
*/
setMaxValue(newValue:Float): void;

/**
*  Sets the Slider.numValues for the slider 
* 
* 
* @param {Integer} the new number of values 
*/
setNumValues(newNumValues:Integer): void;

/**
*  Sets the Slider.title of the slider
* 
* 
* @param {string} new title for the slider
*/
setTitle(newTitle:string): void;

/**
*  Sets the Slider.vertical property of the slider
* 
* 
* @param {boolean} is the slider vertical
*/
setVertical(isVertical:boolean): void;

/**
*  Sets the Slider.thumbThickWidth property of the slider
* 
* 
* @param {Integer} new thumbThickWidth
*/
setThumbThickWidth(newWidth:Integer): void;

/**
*  Sets the Slider.thumbThinWidth property of the slider
* 
* 
* @param {Integer} new thumbThinWidth
*/
setThumbThinWidth(newWidth:Integer): void;

/**
*  Sets the Slider.trackWidth property of the slider
* 
* 
* @param {Integer} new trackWidth
*/
setTrackWidth(newWidth:Integer): void;

/**
*  Sets the Slider.thumbSrc property of the slider
* 
* 
* @param {string} new thumbSrc
*/
setThumbSrc(newSrc:string): void;

/**
*  Sets the Slider.trackSrc property of the slider
* 
* 
* @param {string} new trackSrc
*/
setTrackSrc(newSrc:string): void;

/**
*  Sets the Slider.trackCapSize property of the slider
* 
* 
* @param {Integer} new trackCapSize
*/
setTrackCapSize(newSize:Integer): void;

/**
*  Sets the Slider.trackImageType property of the slider
* 
* 
* @param {ImageStyle} new trackImageType
*/
setTrackImageType(newType:ImageStyle): void;

/**
*  Sets the Slider.showTitle property of the slider
* 
* 
* @param {boolean} show the slider title?
*/
setShowTitle(showTitle:boolean): void;

/**
*  Sets the Slider.showRange property of the slider
* 
* 
* @param {boolean} show the slider range?
*/
setShowRange(showRange:boolean): void;

/**
*  Sets the Slider.showValue property of the slider
* 
* 
* @param {boolean} show the slider value?
*/
setShowValue(showValue:boolean): void;

/**
*  Sets the Slider.labelWidth property of the slider
* 
* 
* @param {Integer} new label width
*/
setLabelWidth(labelWidth:Integer): void;

/**
*  Sets the Slider.labelHeight property of the slider
* 
* 
* @param {Integer} new label height
*/
setLabelHeight(newHeight:Integer): void;

/**
*  Sets the Slider.labelSpacing property of the slider
* 
* 
* @param {Integer} new label spacing
*/
setLabelSpacing(labelWidth:Integer): void;

/**
*  Sets the Slider.maxValueLabel property of the slider
* 
* 
* @param {string} new label text
*/
setMaxValueLabel(labelText:string): void;

/**
*  Sets the Slider.roundValues property of the slider
* 
* 
* @param {boolean} round slider values?
*/
setRoundValues(roundValues:boolean): void;

/**
*  Sets the Slider.roundPrecision property of the slider
* 
* 
* @param {Integer} new round precision
*/
setRoundPrecision(roundPrecision:Integer): void;

/**
*  Sets the Slider.flipValues property of the slider
* 
* 
* @param {boolean} flip slider values?
*/
setFlipValues(flipValues:boolean): void;

/**
*  Sets the Slider.stepPercent property of the slider
* 
* 
* @param {Float} new slider step percent
*/
setStepPercent(stepPercent:Float): void;



}
/**
* Creates a new Slider
* 
* @param typeCheckedProperties {Partial<Slider>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Slider} The new instance
*/
class Slider{
static create(typeCheckedProperties? : Partial<Slider>, uncheckedProperties?: Object): Slider;







}

/**
* FormItem that uses a Slider component to present an interface for picking
*  from either a continuous range or a range with a small number of discrete values.
*/
interface SliderItem extends CanvasItem {
/**
* Default width of this item.
* @type {number}
* @default "150"
*/
width?: number;

/**
* Indicates whether this is a vertical or horizontal slider.
* @type {boolean}
* @default false
*/
vertical?: boolean;

/**
* The minimum slider value. The slider value is equal to minValue when the thumb is at the
*  bottom or left of the slider (unless flipValues is true, in which case the minimum value
*  is at the top/right of the slider)
* @type {Float}
* @default "1"
*/
minValue?: Float;

/**
* The maximum slider value. The slider value is equal to maxValue when the thumb is at the
*  top or right of the slider (unless flipValues is true, in which case the maximum value
*  is at the bottom/left of the slider)
* @type {Float}
* @default "100"
*/
maxValue?: Float;

/**
* The number of discrete values represented by slider. If specified, the range of valid
*  values (between minValue and maxValue) will be divided into
*  this many steps. As the thumb is moved along the track it will only select these values
*  and appear to jump between the steps.
* @type {Integer}
* @default null
*/
numValues?: Integer;

/**
* Specifies whether the slider value should be rounded to the nearest integer. If set to
*  false, values will be rounded to a fixed number of decimal places controlled by
*  SliderItem.roundPrecision.
* @type {boolean}
* @default true
*/
roundValues?: boolean;

/**
* If Slider.roundValues is false, the slider value will be rounded to this number of
*  decimal places. If set to null the value will not be rounded
* @type {number}
* @default "1"
*/
roundPrecision?: number;

/**
* Default value for this sliderItems is 1.
* @type {Float}
* @default "1"
*/
defaultValue?: Float;

/**
* This item is an autoChild generated Canvas displayed by
*  the SliderItem and is an instance of Slider by default. It is customizable 
*  via the standard AutoChild pattern, by customizing SliderItem.sliderProperties
*  and SliderItem.sliderConstructor.
* @type {Canvas}
* @default null
*/
slider?: Canvas;

/**
* Properties to add to the automatically created Slider used by this
*  FormItem. See the Slider class for reference.
* @type {Partial<Slider>}
* @default null
*/
sliderProperties?: Partial<Slider>;

/**
* Should this item's value be saved in the form's values and hence returned from
*  DynamicForm.getValues?
*  
*  shouldSaveValue:false is used to mark formItems which do not correspond to
*  the underlying data model and should not save a value into the form's
*  DynamicForm.values. Example includes visual separators, password re-type fields,
*  or checkboxes used to show/hide other form items.
*  
*  A shouldSaveValue:false item should be given a value either via
*  FormItem.defaultValue or by calling
*  DynamicForm.setValue or 
*  FormItem.setValue. Providing a value via
*  DynamicForm.values or DynamicForm.setValues 
*  will automatically switch the item to shouldSaveValue:true.
*  
*  Note that 
*  
* - if an item is shouldSaveValue true, but has no name, a warning is logged, and 
*    shouldSaveValue will be set to false.
*  
* @type {boolean}
* @default true
*/
shouldSaveValue?: boolean;

/**
* Constructor class for this item's Slider.
* @type {Class}
* @default "Slider"
*/
sliderConstructor?: Class;

/**
* Should this sliderItem update its value and fire change handlers while the user is
*  actively dragging the slider.
*  Setting this attribute value to false will suppress any change notifications
*  from the user dragging the slider thumb until the user releases the mouse at the final
*  position.
*  This can be useful to avoid repeatedly firing expensive operations such as server fetches
*  while the user drags through a range of values.
* @type {boolean}
* @default true
*/
changeOnDrag?: boolean;



/**
*  Notification method called when FormItem.showPending is enabled and
*  this SliderItem should either clear or show its pending visual state.
*  
*  The default behavior is that the FormItem.titleStyle and
*  FormItem.cellStyle are updated to include/exclude the "Pending" suffix.
*  In addition, when displayed in the pending state the value label changes color.
*  Returning false will cancel this default behavior.
* 
* @param {DynamicForm} the managing DynamicForm instance.
* @param {FormItem} the form item itself (also available as "this").
* @param {boolean} true if the item should show its pending visual state; false otherwise.
* @param {any} the current form item value.
* @param {any} the value that would be restored by a call to DynamicForm.resetValues.
*/
pendingStatusChanged(form:DynamicForm, item:FormItem, pendingStatus:boolean, newValue:any, value:any): boolean;

/**
*  Sets the Slider.minValue of the slider 
* 
* 
* @param {Float} the new minimum value 
*/
setMinValue(newValue:Float): void;

/**
*  Sets the Slider.maxValue of the slider 
* 
* 
* @param {Float} the new maximum value 
*/
setMaxValue(newValue:Float): void;

/**
*  Sets the Slider.numValues for the slider 
* 
* 
* @param {Integer} the new number of values 
*/
setNumValues(newNumValues:Integer): void;



}
/**
* Creates a new SliderItem
* 
* @param typeCheckedProperties {Partial<SliderItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SliderItem} The new instance
*/
class SliderItem{
static create(typeCheckedProperties? : Partial<SliderItem>, uncheckedProperties?: Object): SliderItem;







}

/**
* Subclass of the Splitbar class that uses the grip functionality
*  to show a stateful open / close indicator.
*/
interface Snapbar extends Splitbar {
/**
* Snapbars show rollover styling.
* @type {boolean}
* @default true
*/
showRollOver?: boolean;

/**
* Snapbars show mouse-down styling.
* @type {boolean}
* @default true
*/
showDown?: boolean;

/**
* Should we show a "grip" image floating above the center of this widget?
* @type {boolean}
* @default true
*/
showGrip?: boolean;

/**
* If StretchImg.showGrip is true, this property determines whether to show the
*  'Down' state on the grip image when the user mousedown's on this widget. 
*  Has no effect if StatefulCanvas.showDown is false.
* @type {boolean}
* @default true
*/
showDownGrip?: boolean;

/**
* If StretchImg.showGrip is true, this property determines whether to show the
*  'Over' state on the grip image when the user rolls over on this widget. 
*  Has no effect if StatefulCanvas.showRollOver is false.
* @type {boolean}
* @default true
*/
showRollOverGrip?: boolean;

/**
* If Splitbar.showGrip is true, this property determines whether the grip image
*  displayed should show the "Closed" state when the Splitbar.target 
*  is hidden. Note that if Splitbar.invertClosedGripIfTargetAfter is true, we
*  may show the "closed" state when the target is visible, rather than when it is hidden.
* @type {boolean}
* @default true
*/
showClosedGrip?: boolean;

/**
* Overridden from Splitbar.gripImgSuffix to simplify providing custom grip media
*  for this widget.
* @type {string}
* @default "snap"
*/
gripImgSuffix?: string;





}
/**
* Creates a new Snapbar
* 
* @param typeCheckedProperties {Partial<Snapbar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Snapbar} The new instance
*/
class Snapbar{
static create(typeCheckedProperties? : Partial<Snapbar>, uncheckedProperties?: Object): Snapbar;







}

/**
* SmartClient class for loading and playing audio files using the HTML5 &lt;AUDIO&gt; 
*  element.
*/
interface Sound extends BaseWidget {
/**
* URL of the audio file to be played by this sound instance. If multiple file URLs are
*  supplied, the browser will make use of the first file type for which it has support.
* @type {string | Array<Partial<string>>}
* @default null
*/
src?: string | Array<Partial<string>>;

/**
* Should the specified Sound.src be loaded automatically.
*  
*  If set to false developers may load the audio explicitly via 
*  Sound.load
* @type {boolean}
* @default false
*/
autoLoad?: boolean;

/**
* Should the specified Sound.src be played automatically?
*  
*  If set to false developers may play the audio explicitly via 
*  Sound.play.
* @type {boolean}
* @default false
*/
autoPlay?: boolean;



/**
*  Update the Sound.src of this sound instance at runtime. Note that 
*  Sound.autoLoad and Sound.autoPlay govern whether this media will
*  be loaded or played immediately when the src value is changed.
* 
* @param {string | Array<Partial<string>>} URL of new audio file to be played by this sound instance.
*/
setSrc(src:string | Array<Partial<string>>): void;

/**
*  This method will cause the Sound.src to be loaded
* 
* @param {CanPlayCallback=} notification to fire when the file is ready to play
*/
load(canPlayCallback?:() => void): void;

/**
*  Play the audio file.
*  If necessary the file will be loaded first.
* 
* @param {PlaybackCompleteCallback=} notification fired when playback completes
*/
play(playbackCompleteCallback?:() => void): void;

/**
*  Pause playback of the audio file.
* 
*/
pause(): void;

/**
*  If playback is currently paused, reset the playback position to the start of the audio
*  file so a call to Sound.play will play from the start, rather than resuming
*  playback from the current position.
* 
*/
reset(): void;

/**
*  Notification method fired repeatedly to indicate a change in currentTime
*  value while an audio file is playing.
* 
* @param {Float} Current playback position in seconds.
*/
timeChanged(currentTime:Float): void;

/**
*  Retrieves the duration of the current audio file in seconds.
* 
*/
getDuration(): Float;

/**
*  Retrieves the current playback time of a playing or paused audio file in seconds.
* 
*/
getCurrentTime(): Float;

/**
*  Move playback to a particular time in a loaded audio file.
* 
* @param {Float} time to move to. This method will have no effect if the file has not been loaded or no Sound.src element is defined.
*/
setCurrentTime(time:Float): void;



}
/**
* Creates a new Sound
* 
* @param typeCheckedProperties {Partial<Sound>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Sound} The new instance
*/
class Sound{
static create(typeCheckedProperties? : Partial<Sound>, uncheckedProperties?: Object): Sound;




/**
*  Returns true for browsers which natively support HTML5 Audio, used by the
*  Sound class
* 
*/
static isSupported(): boolean;

/**
*  Convenience method to load and play a specified audio file.
*  
*  For more explicit control over loading and playback of audio files, developers may
*  create an instance of Sound and call methods directly on that object.
* 
* @param {string} URL of the audio clip to play
* @param {PlaybackCompleteCallback=} callback to execute when the clip playback completes
*/
static play(src:string, playbackCompleteCallback?:() => void): void;




}

/**
* A SpacerItem takes up a single cell in the FormLayout, of arbitrary size.
*/
interface SpacerItem extends FormItem {
/**
* we never show a separate title cell for spacers
* @type {boolean}
* @default false
*/
showTitle?: boolean;

/**
* default width for the spacer
* @type {number}
* @default "20"
*/
width?: number;

/**
* default height for the spacer
* @type {number}
* @default "20"
*/
height?: number;





}
/**
* Creates a new SpacerItem
* 
* @param typeCheckedProperties {Partial<SpacerItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SpacerItem} The new instance
*/
class SpacerItem{
static create(typeCheckedProperties? : Partial<SpacerItem>, uncheckedProperties?: Object): SpacerItem;







}

/**
* Item for picking a number. Includes arrow buttons to increase / decrease the value
*/
interface SpinnerItem extends TextItem {
/**
* Not applicable to a SpinnerItem.
* @type {string}
* @default null
*/
mask?: string;

/**
* Not applicable to a SpinnerItem.
* @type {boolean}
* @default null
*/
maskSaveLiterals?: boolean;

/**
* Not applicable to a SpinnerItem.
* @type {string}
* @default " "
*/
maskPadChar?: string;

/**
* Not applicable to a SpinnerItem.
* @type {string}
* @default "_"
*/
maskPromptChar?: string;

/**
* Not applicable to a SpinnerItem.
* @type {boolean}
* @default null
*/
maskOverwriteMode?: boolean;

/**
* In SpinnerItem.writeStackedIcons, the base CSS class name for the
*  SpinnerItem's text box element.
*  
*  NOTE: See the CompoundFormItem_skinning discussion for special
*  skinning considerations.
* @type {FormItemBaseStyle}
* @default "textItem"
*/
unstackedTextBoxStyle?: FormItemBaseStyle;

/**
* In SpinnerItem.writeStackedIcons, the base CSS class name for the
*  SpinnerItem's text box element when printed. If unset, then SpinnerItem.unstackedTextBoxStyle
*  is used.
* @type {FormItemBaseStyle}
* @default null
*/
unstackedPrintTextBoxStyle?: FormItemBaseStyle;

/**
* @type {FormItemBaseStyle}
* @default null
*/
unstackedReadOnlyTextBoxStyle?: FormItemBaseStyle;

/**
* How much should the value be incremented / decremented when the user hits the icons
*  to increase / decrease the value?
*  
*  
*  When overriding SpinnerItem.getNextValue and/or SpinnerItem.getPreviousValue,
*  the sign of the step value determines whether these methods
*  
*  are expected to induce monotonically increasing or decreasing functions.
* @type {Double}
* @default "1"
*/
step?: Double;

/**
* Maximum valid value for this item. If this and SpinnerItem.min are both
*  null or unspecified, then
*  
*  SpinnerItem.getNextValue and SpinnerItem.getPreviousValue
*  are used to increase or decrease the value and these methods
*  
*  are also used to determine the maximum value.
* @type {Double}
* @default null
*/
max?: Double;

/**
* Minimum valid value for this item. If this and SpinnerItem.max are both
*  null or unspecified, then
*  
*  SpinnerItem.getNextValue and SpinnerItem.getPreviousValue
*  are used to increase or decrease the value and these methods
*  
*  are also used to determine the minimum value.
* @type {Double}
* @default null
*/
min?: Double;

/**
* In SpinnerItem.writeStackedIcons, the icon to increase the spinner's
*  value (an up arrow by default). This icon is generated automatically using
*  the AutoChild pattern. For skinning purposes, increaseIconDefaults
*  may be modified using Class.changeDefaults. 
*  
*  If sizes for the increase and decrease icons are not explicitly specified in their
*  autoChild configuration, they will be derived from the specified 
*  SpinnerItem.stackedIconsHeight and SpinnerItem.stackedIconsWidth 
*  properties.
*  
*  See the
*  skinning for details on how to provide a sprited
*  image for these icons.
* @type {FormItemIcon}
* @default null
*/
increaseIcon?: FormItemIcon;

/**
* FormItemIcon properties applied to the SpinnerItem.increaseIcon AutoChild of this
*  SpinnerItem.
* @type {Partial<FormItemIcon>}
* @default null
*/
increaseIconProperties?: Partial<FormItemIcon>;

/**
* In SpinnerItem.writeStackedIcons, the icon to decrease the spinner's
*  value (a down arrow by default). This icon is generated automatically using
*  the AutoChild pattern. For skinning purposes, decreaseIconDefaults
*  may be modified using Class.changeDefaults. 
*  
*  If sizes for the increase and decrease icons are not explicitly specified in their
*  autoChild configuration, they will be derived from the specified 
*  SpinnerItem.stackedIconsHeight and SpinnerItem.stackedIconsWidth 
*  properties.
*  
*  See the
*  skinning for details on how to provide a sprited
*  image for these icons.
* @type {FormItemIcon}
* @default null
*/
decreaseIcon?: FormItemIcon;

/**
* FormItemIcon properties applied to the SpinnerItem.decreaseIcon AutoChild of this
*  SpinnerItem.
* @type {Partial<FormItemIcon>}
* @default null
*/
decreaseIconProperties?: Partial<FormItemIcon>;

/**
* When set to true, the increase and decrease icons are stacked on top of
*  each other, also called stacked mode. When false, the increase and decrease
*  icons are placed on the same line as the SpinnerItem's text box, also called
*  unstacked mode. When null, a default setting depending on Browser.isTouch
*  is used (for touch browsers, the default is false/unstacked mode).
*  
*  In stacked mode, SpinnerItem.increaseIcon and SpinnerItem.decreaseIcon
*  control the appearance of the stacked icons.
*  
*  In unstacked mode, SpinnerItem.unstackedIncreaseIcon and SpinnerItem.unstackedDecreaseIcon
*  control the appearance of the unstacked icons.
* @type {boolean}
* @default null
*/
writeStackedIcons?: boolean;

/**
* In SpinnerItem.writeStackedIcons this property can be 
*  used to specify the width of both the increase and decrease icon. 
*  If a width property is explicitly
*  set for the icon via SpinnerItem.increaseIconProperties,
*  SpinnerItem.decreaseIconProperties, or related Defaults property
*  blocks, that will take precedence over any specified stackedIconsWidth.
*  
*  See also SpinnerItem.stackedIconsHeight.
* @type {Integer}
* @default "16"
*/
stackedIconsWidth?: Integer;

/**
* In SpinnerItem.writeStackedIcons this property can be
*  used to specify the height of both the increase and decrease icon. Since the
*  icons are stacked vertically, each icon will be sized to half this specified value.
*  If a height property is explicitly
*  set for the icon via SpinnerItem.increaseIconProperties,
*  SpinnerItem.decreaseIconProperties, or related Defaults property
*  blocks, that will take precedence over any specified stackedIconsHeight.
*  
*  See also SpinnerItem.stackedIconsWidth.
* @type {Integer}
* @default "18"
*/
stackedIconsHeight?: Integer;

/**
* In SpinnerItem.writeStackedIcons, the icon to increase the
*  SpinnerItem's value.
*  
*  By default, "[SKIN]/DynamicForm/Spinner_increase_icon.png" is stretched to
*  an 18x18 icon.
*  
*  When skinning is enabled, this property will not 
*  be used to locate an image, instead, the image is drawn via CSS based on the 
*  FormItemIcon.baseStyle property.
* @type {FormItemIcon}
* @default null
*/
unstackedIncreaseIcon?: FormItemIcon;

/**
* In SpinnerItem.writeStackedIcons, the icon to decrease the
*  SpinnerItem's value.
*  
*  By default, "[SKIN]/DynamicForm/Spinner_decrease_icon.png" is stretched to
*  an 18x18 icon.
*  
*  When skinning is enabled, this property will not 
*  be used to locate an image, instead, the image is drawn via CSS based on the 
*  FormItemIcon.baseStyle property.
* @type {FormItemIcon}
* @default null
*/
unstackedDecreaseIcon?: FormItemIcon;



/**
*  When SpinnerItem.min and SpinnerItem.max are both null or unspecified,
*  this method is called to get the next higher value from the currentValue. The default
*  implementation returns (currentValue&nbsp;+&nbsp;step).
*  
*  To indicate that the given currentValue is the maximum value, return currentValue again.
*  
*  Implementations should expect to be passed any value for currentValue. Also, if
*  SpinnerItem.step is non-negative, getNextValue() must induce a
*  monotonically increasing (non-decreasing) function;
*  otherwise, getNextValue() must induce a monotonically decreasing function.
* 
* @param {number} the current value of this SpinnerItem
* @param {number} a suggested step value based on SpinnerItem.step and how long the user has been continuously increasing the value.
*/
getNextValue(currentValue:number, step:number): number;

/**
*  When SpinnerItem.min and SpinnerItem.max are both null or unspecified,
*  this method is called to get the previous lower value from the currentValue. The default
*  implementation returns (currentValue&nbsp;+&nbsp;step) because the step parameter
*  is based on the opposite of SpinnerItem.step.
*  
*  To indicate that the given currentValue is the minimum value, return currentValue again.
*  
*  Implementations should expect to be passed any value for currentValue. Also, if
*  SpinnerItem.step is non-negative, getPreviousValue() must induce a
*  monotonically decreasing (non-increasing) function;
*  otherwise, getPreviousValue() must induce a monotonically increasing function.
* 
* @param {number} the current value of this SpinnerItem
* @param {number} a suggested step value based on the opposite of SpinnerItem.step and how long the user has been continuously decreasing the value.
*/
getPreviousValue(currentValue:number, step:number): number;



}
/**
* Creates a new SpinnerItem
* 
* @param typeCheckedProperties {Partial<SpinnerItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SpinnerItem} The new instance
*/
class SpinnerItem{
static create(typeCheckedProperties? : Partial<SpinnerItem>, uncheckedProperties?: Object): SpinnerItem;







}

/**
* A device- and orientation-sensitive layout that implements the common pattern of rendering
*  two panes side-by-side on desktop machines and on tablets in landscape orientation,
*  while switching to showing a single pane for handset-sized devices or tablets in portrait
*  orientation (this type of behavior is sometimes called "responsive design").
*  
*  A SplitPane can manage either two or three panes &mdash; a
*  SplitPane.navigationPane and the
*  SplitPane.detailPane are required, and a
*  SplitPane.listPane can also be provided which appears in the same place as
*  the navigation pane, with built-in navigation between the panes based on
*  NavigationBar. An example of 3-pane usage would be an email application:
*  
*  
* - navigationPane: TreeGrid of folders
*  
* - listPane: ListGrid showing messages in a folder
*  
* - detailPane: message detail view (perhaps a DetailViewer over an
*    HTMLFlow or similar arrangement)
*  
*  
*  The placement of the panes is by default sensitive to whether the device is detected as a
*  handset (phone), tablet or desktop device (see DeviceMode) and to the current
*  PageOrientation. You can also configure a SplitPane with a fixed
*  SplitPane.pageOrientation or SplitPane.deviceMode.
*  
*  Beyond providing the panes listed above, typical usage is simply to call
*  SplitPane.showListPane and SplitPane.showDetailPane when the
*  SplitPane should navigate to a new pane. For example, in an email application,
*  clicking on a folder in the navigationPane should cause the
*  listPane to show messages from the folder, then
*  showListPane("folder name") would be called to show the
*  listPane and give it a new title reflecting the name of the folder.
*  
*  Similarly, clicking on a message in the listPane should show the message
*  details in the detailPane and call
*  showDetailPane("message title") to reveal the detailPane
*  and give it an appropriate title.
*  
*  Auto-Navigation
*  
*  By default, SplitPane will analyze the controls placed in each pane and the DataSources they 
*  are bound to, and automatically navigate between panes. 
*  
*  For example, in a two-pane SplitPane with a ListGrid in the navigationPane and a DynamicForm 
*  in the detailPane, both with the same DataSource, when a record is selected in the grid, 
*  DynamicForm.editRecord will be called to populate the form, and the detailPane will be shown.
*  
*  In a 3-pane SplitPane with a TreeGrid and ListGrid in the navigationPane and listPane respectively, 
*  if there is a 1-to-Many relation from the TreeGrid's DataSource to the ListGrid's DataSource, 
*  ListGrid.fetchRelatedData will be used to load related records when tree nodes are clicked, 
*  and the listPane will be shown.
*  
*  For a full description of auto-navigation, see SplitPane.autoNavigate. Just set autoNavigate 
*  to false if you don't want these behaviors.
*  
*  Automatic control placement
*  
*  SplitPane.detailToolButtons allows you to define a set of controls that are specially
*  placed based on the deviceMode and pageOrientation. See
*  SplitPane.detailToolButtons for details.
*  
*  NavigationBar and ToolStrips
*  
*  By default, bars are created as follows:
*  
*  
* - in deviceMode:"tablet" and deviceMode "handset", the
*    SplitPane.navigationBar is always created.
*  
* - in deviceMode:"desktop", the navigationBar is created by
*    default only if the SplitPane.navigationTitle is specified and non-empty or
*    SplitPane.showRightButton and/or SplitPane.showLeftButton is true,
*    or SplitPane.showNavigationBar is true.
*  
* - in deviceMode:"desktop" and deviceMode "tablet", the
*    SplitPane.detailToolStrip is shown above the detailPane.
*  
* - in deviceMode:"handset", the SplitPane.detailToolStrip is created
*    only if detailToolButtons are specified, and is placed
*    underneath the detailPane.
*  
* - SplitPane.listToolStrip - separate bar for the listPane, only present
*    for deviceMode:"desktop" when a listPane is provided.
*  
*  All of these bars are autoChildUsage and hence completely
*  optional. You can omit them entirely, or, if you want navigation bars or tool strips but
*  want to customize them more than the AutoChild system allows, you can prevent the built-in
*  bars from being created and place your own NavigationBars either inside your
*  navigation, list or detail panes, or outside the SplitPane as a whole.
*  This allows you to completely customize your navigation but still use SplitPane
*  to handle device- and orientation-aware layout. See SplitPane.showNavigationBar,
*  SplitPane.showListToolStrip, and SplitPane.showDetailToolStrip.
*  
*  Note that in addition to the SplitPane.navigationBar, the other automatically
*  created bars are also instances of NavigationBar despite the "toolStrip" naming convention.
*  These controls will not generally contain navigation elements; the NavigationBar
*  class is used for consistent styling, since the navigationBar appears adjacent
*  to the toolstrips in some modes and orientations, so they should have the same height and
*  styling.
*/
interface SplitPane extends Layout {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "SplitPaneEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Note: This is a Layout property which is inapplicable on this class.
* @type {boolean}
* @default null
*/
vertical?: boolean;

/**
* Note: This is a Layout property which is inapplicable on this class. A SplitPane
*  always works from left to right.
* @type {boolean}
* @default null
*/
reverseOrder?: boolean;

/**
* Should default history-tracking support be enabled? If true, then a history
*  management scheme utilizing History.addHistoryEntry and History.registerCallback
*  is enabled. The history callback is registered as an additive callback, allowing other
*  history callbacks including the primary callback to be registered.
*  
*  The default history management scheme is as follows:
*  
*  
* - History entries are only added after Page.isLoaded and when the
*  SplitPane is drawn.
*  
* - A history entry is added for a pane that is hidden by SplitPane.showNavigationPane,
*  SplitPane.showListPane, or SplitPane.showDetailPane for the current SplitPane.deviceMode
*  and SplitPane.pageOrientation settings.
*  
*  Example 1: When deviceMode is "desktop", all 3 panes are shown simultaneously,
*  so no history entries are added.
*  
*  Example 2: When deviceMode is "handset", calling SplitPane.showDetailPane
*  will hide the current pane (the SplitPane.listPane if present, otherwise the SplitPane.navigationPane).
*  A history entry is added for the pane that was hidden
*  
*  
*  The default history management scheme can be supplemented with application-specific
*  history management. For example, when deviceMode is "tablet", the SplitPane.detailPane
*  is always visible, but changes to the content of the detailPane are transparent
*  to the SplitPane. The application can add history entries of its own when
*  the user causes new information to be displayed in the detailPane.
* @type {boolean}
* @default false
*/
addHistoryEntries?: boolean;

/**
* UI layout mode used for this SplitPane.
*  
*  A SplitPane can be configured with up to 3 panes: the SplitPane.navigationPane,
*  SplitPane.listPane and SplitPane.detailPane. Both DeviceMode and PageOrientation
*  influence the placement of these panes as follows:
*  
*  
* - "handset" deviceMode: only a single pane is shown at a time. Not
*    orientation sensitive
*  
* - "tablet" deviceMode with pageOrientation:"landscape": the
*    detailPane is shown side by side with either the
*    navigationPane or listPane
*  
* - "tablet" deviceMode with pageOrientation:"portrait": the
*    detailPane is shown only. End user navigation that would show the
*    listPane or navigationPane shows those panes on top of the
*    detailPane (temporarily covering part of its content)
*  
* - "desktop" deviceMode: all 3 panes are shown simultaneously. Not
*    orientation sensitive
*  
*  The listPane is optional; if not present, wherever the listPane
*  is mentioned above, the navigationPane is shown instead.
* @type {DeviceMode}
* @default null
*/
deviceMode?: DeviceMode;

/**
* Current PageOrientation. The default behavior of the SplitPane is to
*  register for orientation change notifications from the device (see
*  Page.getOrientation) and automatically change orientation based on the
*  SplitPane.deviceMode.
*  
*  You can instead set a specific value for pageOrientation if you only want to
*  use a specific layout, and not respond to orientation information from the device.
* @type {PageOrientation}
* @default null
*/
pageOrientation?: PageOrientation;

/**
* Whether or not to call SplitPane.navigationClick, if present, after navigation has already
*  occurred. This may be set to provide backcompat with legacy code, as by default the
*  Framework will call SplitPane.navigationClick before navigation to allow cancelation.
*  
*  Note that if this property is set, SplitPane.navigationClick cannot be canceled.
* @type {boolean}
* @default false
*/
notifyAfterNavigationClick?: boolean;

/**
* Sets a size for the navigation pane.
*  
*  This size is active only on platforms where multiple panes are showing at once; if a
*  single pane is showing, navigationPaneWidth is ignored.
*  
*  Note that setting a navigationPaneWidth which creates more size in one of
*  the panes may backfire on mobile, where all panes end up having the same width (the
*  device width). If you make one pane larger to accommodate more controls or content, make
*  sure you use techniques such as showing fewer columns on mobile, or using adaptive
*  components such as AdaptiveMenu.
*  
*  If you simply want side-by-side display with arbitrary proportions, and don't care about
*  tablet and mobile adaptation, use HLayout instead.
* @type {number | string}
* @default "320"
*/
navigationPaneWidth?: number | string;

/**
* If enabled, the SplitPane will add resize bars between the
*  SplitPane.navigationPane and SplitPane.detailPane
*  when these panes are shown side-by-side, and between the SplitPane.listPane
*  and SplitPane.detailPane in SplitPane.deviceMode.
* @type {boolean}
* @default true
*/
showResizeBars?: boolean;

/**
* A NavigationBar instance managed by this SplitPane that is
*  placed above the SplitPane.navigationPane.
*  
*  The following autoChildUsage apply:
*  
*  
* - SplitPane.animateNavigationBarStateChanges
*    for NavigationBar.animateStateChanges
*  
* - SplitPane.showRightButton for NavigationBar.showRightButton
*  
*  
*  Note that in SplitPane.deviceMode
*  "desktop"
*  
*  with SplitPane.showNavigationBar unset,
*  the navigationBar is automatically hidden when it would be empty
*  (SplitPane.navigationTitle is an empty string and
*  showRightButton and showLeftButton are both false).
*  The navigationBar will be shown if the navigationTitle
*  SplitPane.setNavigationTitle to a non-empty string, or
*  showRightButton or showLeftButton is set to true.
* @type {NavigationBar}
* @default null
*/
navigationBar?: NavigationBar;

/**
* If set to false, the SplitPane.navigationBar
*  will not be shown. If set to true, the navigationBar will
*  always be shown, even when the SplitPane.deviceMode is
*  "desktop"
*  
*  If this property is unset, the SplitPane.navigationBar will be shown if at least one
*  of the following conditions holds:
*  
* - the SplitPane.deviceMode is not
*  "desktop"
*  
*  
* - the SplitPane.navigationTitle is specified and non-empty
*  
* - SplitPane.showRightButton and/or SplitPane.showLeftButton is
*  true,
* @type {boolean}
* @default null
*/
showNavigationBar?: boolean;

/**
* Whether to animate state changes of the SplitPane.navigationBar.
*  Enabled by default except when the browser is known to have poor animation
*  performance.
* @type {boolean}
* @default true
*/
animateNavigationBarStateChanges?: boolean;

/**
* A NavigationButton shown to the left of the 
*  SplitPane.navigationTitle 
*  in the SplitPane.navigationBar. 
*  
*  In SplitPane.deviceMode other than "desktop", this button is 
*  automatically created and allows transitioning back to the 
*  SplitPane.navigationPane (in tablet and handset modes) or the 
*  SplitPane.listPane (in handset mode). In these 
*  SplitPane.deviceMode, setting 
*  SplitPane.showLeftButton to true shows the 
*  SplitPane.leftButton in addition to the 
*  automatically-created back button.
*  
*  When SplitPane.deviceMode is "desktop", this button is never shown.
*  See SplitPane.showLeftButton for more information.
*  
*  This button's Button.title is determined automatically by the 
*  SplitPane. See SplitPane.listTitle and 
*  SplitPane.detailTitle.
* @type {NavigationButton}
* @default null
*/
backButton?: NavigationButton;

/**
* An additional NavigationButton which may be shown to the left of the 
*  SplitPane.navigationTitle in the
*  SplitPane.navigationBar.
*  
*  Important note: by default, this button has no 
*  NavigationButton.direction and does not fire the 
*  SplitPane.navigationClick notification. You can provide a 
*  direction and apply a click handler via the autoChild system.
* @type {NavigationButton}
* @default null
*/
leftButton?: NavigationButton;

/**
* The most recently shown pane. In handset DeviceMode, the
*  currentPane is the only pane that is actually visible to the user. In other
*  modes more than one pane can be simultaneously visible, so the currentPane is
*  the most recent pane that was brought into view via a call to SplitPane.setCurrentPane or
*  SplitPane.showNavigationPane.
*  
*  The default value of currentPane is "navigation".
* @type {CurrentPane}
* @default "navigation"
*/
currentPane?: CurrentPane;

/**
* The title for the SplitPane.navigationPane, displayed in the
*  SplitPane.navigationBar and also used for the title of a back
*  button in some configurations.
* @type {HTMLString}
* @default null
*/
navigationTitle?: HTMLString;

/**
* An arbitrary widget that is visible in all configurations when the
*  SplitPane.currentPane is
*  "navigation"
*  
*  (it may also be visible when the currentPane is
*  "list"
*  
*  or
*  "detail").
*  
*  
*  The navigationPane is typically used for navigation, to initialize the
*  content of the SplitPane.listPane (when using a listPane)
*  or the SplitPane.detailPane. For example, in an email application
*  the navigationPane pane widget could be a TreeGrid of the inboxes
*  and folders.
* @type {Canvas}
* @default null
*/
navigationPane?: Canvas;

/**
* The title for the SplitPane.listPane.
* @type {HTMLString}
* @default null
*/
listTitle?: HTMLString;

/**
* An optional list pane displayed in the left-hand of the panes or in a side panel
*  according to the pane layout.
* @type {Canvas}
* @default null
*/
listPane?: Canvas;

/**
* Bar displayed above the SplitPane.listPane, if a listPane is present,
*  only for SplitPane.deviceMode "desktop".
* @type {NavigationBar}
* @default null
*/
listToolStrip?: NavigationBar;

/**
* If set to false, the SplitPane.listToolStrip
*  will not be shown. Otherwise, the SplitPane.listToolStrip will be shown if
*  the SplitPane.deviceMode is
*  "desktop"
*  and
*  a SplitPane.listPane is provided.
* @type {boolean}
* @default null
*/
showListToolStrip?: boolean;

/**
* The title for the SplitPane.detailPane.
* @type {HTMLString}
* @default null
*/
detailTitle?: HTMLString;

/**
* The right-hand of the two panes managed by this widget, used for viewing details.
* @type {Canvas}
* @default null
*/
detailPane?: Canvas;

/**
* Toolstrip servicing the SplitPane.detailPane.
*  
*  In SplitPane.deviceMode
*  "desktop"
*  
*  and deviceMode
*  "tablet",
*  
*  the detailToolStrip is shown above the detailPane.
*  In SplitPane.deviceMode
*  "handset",
*  
*  the detailToolStrip is created only if
*  SplitPane.detailToolButtons are specified, and is placed
*  underneath the detailPane.
* @type {NavigationBar}
* @default null
*/
detailToolStrip?: NavigationBar;

/**
* If set to false, the SplitPane.detailToolStrip
*  will not be shown. Otherwise, the SplitPane.detailToolStrip will be shown if either
*  the SplitPane.deviceMode is not
*  "handset"
*  or
*  SplitPane.detailToolButtons are specified.
* @type {boolean}
* @default null
*/
showDetailToolStrip?: boolean;

/**
* detailToolButtons allows you to specify a set of controls that are specially
*  placed based on the SplitPane.deviceMode and SplitPane.pageOrientation.
*  This is generally useful for a compact strip of ImgButton controls, approximately
*  5 of which will fit comfortably using typically-sized icons and in the most space-constricted
*  modes.
*  
*  These controls are placed as follows:
*  
*  
* - in deviceMode:"desktop" and deviceMode "tablet" with
*    pageOrientation "landscape", detailToolButtons appear in the
*    SplitPane.detailToolStrip shown above of the detailPane.
*  
* - in deviceMode:"handset", detailToolButtons appear in a
*     SplitPane.detailToolStrip underneath the detailPane. This toolstrip
*    is only created in "handset" mode if detailToolButtons are provided.
*  
* - in deviceMode:"tablet" and pageOrientation:"portrait",
*    detailToolButtons appear in splitPane.navigationBar.
*  
* @type {Array<Partial<Canvas>>}
* @default null
*/
detailToolButtons?: Array<Partial<Canvas>>;

/**
* If true, a MiniNavControl will be shown:
*  
*  
* - In the SplitPane.navigationBar when the device mode is
*    "handset"
*    
*    and the SplitPane.currentPane is
*    "detail".
*    
*  
* - In the SplitPane.detailToolStrip when the device mode is
*    "tablet"
*    
*    and the SplitPane.pageOrientation is
*    "portrait".
*    
*  
*  
* @type {boolean}
* @default false
*/
showMiniNav?: boolean;

/**
* Should the SplitPane.leftButton be shown in the 
*  SplitPane.navigationBar?
*  
*  When set to true, the SplitPane.leftButton is displayed to the left of the 
*  SplitPane.navigationTitle, and to the right of the SplitPane.backButton,
*  when SplitPane.deviceMode is not "desktop".
*  
* @type {boolean}
* @default false
*/
showLeftButton?: boolean;

/**
* Should the NavigationBar.rightButton be shown in the
*  SplitPane.navigationBar?
* @type {boolean}
* @default false
*/
showRightButton?: boolean;

/**
* Navigation control that appears only when the navigation pane is not showing (showing detail
*  pane on handset, or portrait mode on tablet).
*  
*  See also SplitPane.showMiniNav for a way to enable a built-in control.
* @type {Canvas}
* @default null
*/
detailNavigationControl?: Canvas;

/**
* Default value chosen for SplitPane.setListTitle when SplitPane.navigateListPane is called.
*  
*  Available variables are:
*  
*  
* - "titleField" - the value of the DataSource.titleField in the selected record from
*  the SplitPane.navigationPane
*  
* - "index" - position of the selected record
*  
* - "totalRows" - total number of rows in the component where the record is selected
*  
* - "record" - the entire selected Record
*  
* @type {HTMLString}
* @default "${titleField}"
*/
listPaneTitleTemplate?: HTMLString;

/**
* Default value chosen for SplitPane.setDetailTitle when SplitPane.navigateDetailPane is called.
*  
*  Available variables are the same as for SplitPane.listPaneTitleTemplate.
* @type {HTMLString}
* @default "${titleField}"
*/
detailPaneTitleTemplate?: HTMLString;

/**
* If set, the SplitPane will automatically monitor selection changes in the
*  SplitPane.navigationPane and SplitPane.listPane, and call SplitPane.navigateListPane or
*  SplitPane.navigateDetailPane when selections are changed.
*  
*  If a pane is not a DataBoundComponent, but contains a component (selected via a
*  breadth-first search), then that inner component will be monitored for selection changes
*  instead. In either case, autoNavigate does nothing unless the monitored
*  component has a valid DataSource and there is a DataSource relationship declared
*  between panes. Note that for Layouts, the Layout.members will be
*  searched when looking for a component rather than the Canvas.children.
*  
*  The selection of the pane or pane inner component for monitoring is done only when the
*  SplitPane is created, and when a new SplitPane.navigationPane or
*  SplitPane.listPane is assigned, except when the SplitPane is in
*  Canvas.setEditMode (e.g. when using reify). where the
*  component redetection logic gets run every time a pane's widget hierarchy changes.
* @type {boolean}
* @default true
*/
autoNavigate?: boolean;



/**
*  Setter for SplitPane.showNavigationBar.
*  
*  Note: If the property is set false after the
*  SplitPane.navigationBar autochild has already been created, it will be hidden but not
*  destroyed.
* 
* @param {boolean} new value
*/
setShowNavigationBar(showNavigationBar:boolean): void;

/**
*  Setter for SplitPane.showListToolStrip.
*  Note: If the property is set false after the
*  SplitPane.detailToolStrip autochild has already been created, it will be hidden but
*  not destroyed.
* 
* @param {boolean} new value
*/
setShowListToolStrip(showListToolStrip:boolean): void;

/**
*  Setter for SplitPane.showDetailToolStrip.
*  Note: If the property is set false after the
*  SplitPane.detailToolStrip autochild has already been created, it will be hidden but
*  not destroyed.
* 
* @param {boolean} new value
*/
setShowDetailToolStrip(showDetailToolStrip:boolean): void;

/**
*  Setter for SplitPane.addHistoryEntries.
* 
* @param {boolean} the new setting.
*/
setAddHistoryEntries(addHistoryEntries:boolean): void;

/**
*  Reveals the pane indicated by the newPane parameter.
*  
*  This has different effects based on the DeviceMode and PageOrientation. For
*  example, in "handset" mode, the new pane will be the only one showing. In other modes such
*  as "desktop", this method may do nothing, but should still be called in order to ensure
*  correct behavior with other DeviceMode settings.
* 
* 
* @param {CurrentPane} new pane to show.
*/
setCurrentPane(newPane:CurrentPane): void;

/**
*  Updates the SplitPane.detailToolButtons at runtime.
* 
* 
* @param {Array<Partial<Canvas>>} new controls for the toolstrip.
*/
setDetailToolButtons(buttons:Array<Partial<Canvas>>): void;

/**
*  Explicitly sets the page orientation to a fixed value instead of being responsive to device
*  orientation changes. Pass null to return to responding automatically to device
*  orientation.
*  
*  See PageOrientation for details of how page orientation affects layout.
* 
* 
* @param {PageOrientation} new orientation to use.
*/
setPageOrientation(newOrientation:PageOrientation): void;

/**
*  Show or hide the SplitPane.leftButton in the navigation bar.
* 
* @param {boolean} if true, the leftButton will be shown, otherwise hidden.
*/
setShowLeftButton(show:boolean): void;

/**
*  Setter for the NavigationBar.leftButtonTitle of the
*  SplitPane.navigationBar.
* 
* 
* @param {HTMLString} new title for the left button.
*/
setLeftButtonTitle(newTitle:HTMLString): void;

/**
*  Show or hide the NavigationBar.rightButton of the
*  SplitPane.navigationBar.
* 
* 
* @param {boolean} if true, the button will be shown, otherwise hidden.
*/
setShowRightButton(visible:boolean): void;

/**
*  Setter for the NavigationBar.rightButtonTitle of the
*  SplitPane.navigationBar.
* 
* 
* @param {HTMLString} new title for the right button.
*/
setRightButtonTitle(newTitle:HTMLString): void;

/**
*  Update the SplitPane.navigationPane at runtime.
* 
* @param {Canvas} new navigation pane for this widget.
*/
setNavigationPane(pane:Canvas): void;

/**
*  Sets the title for the SplitPane.navigationPane.
* 
* @param {HTMLString} new title for the navigation pane.
*/
setNavigationTitle(title:HTMLString): void;

/**
*  Causes a transition to the SplitPane.navigationPane.
* 
* @param {NavigationDirection=} when SplitPane.animateNavigationBarStateChanges is true, this is the direction passed to NavigationBar.setViewState.
*/
showNavigationPane(direction?:NavigationDirection): void;

/**
*  Sets a new SplitPane.listPane at runtime.
* 
* @param {Canvas} new list pane for this widget.
*/
setListPane(pane:Canvas): void;

/**
*  Causes a transition to the SplitPane.listPane, optionally updating the
*  SplitPane.setListTitle.
*  
*  If, based on the SplitPane.deviceMode and SplitPane.pageOrientation,
*  this causes the SplitPane.navigationPane to be hidden, the
*  SplitPane.backButton will be updated with the current title of the
*  navigationPane, or the backButtonTitle passed to this method.
*  When SplitPane.addHistoryEntries is enabled and backButtonTitle is passed,
*  then backButtonTitle will be used for the back button title if the user goes
*  back to the listPane.
* 
* 
* @param {HTMLString=} optional new list title.
* @param {HTMLString=} optional new title for the SplitPane.backButton.
* @param {NavigationDirection=} when SplitPane.animateNavigationBarStateChanges is true, this is the direction passed to NavigationBar.setViewState.
*/
showListPane(listPaneTitle?:HTMLString, backButtonTitle?:HTMLString, direction?:NavigationDirection): void;

/**
*  Sets the title for the SplitPane.listPane.
* 
* @param {HTMLString} new title for the list pane.
*/
setListTitle(title:HTMLString): void;

/**
*  Sets a new SplitPane.detailPane at runtime.
* 
* @param {Canvas} new detail pane for this widget.
*/
setDetailPane(pane:Canvas): void;

/**
*  Causes a transition to the SplitPane.detailPane, optionally updating
*  the SplitPane.setDetailTitle.
*  
*  If, based on the SplitPane.deviceMode and SplitPane.pageOrientation,
*  this causes the SplitPane.navigationPane or SplitPane.listPane
*  to be hidden, the SplitPane.backButton will be updated
*  with the current title of the navigationPane or listPane, or the
*  backButtonTitle passed to this method. When SplitPane.addHistoryEntries
*  is enabled and backButtonTitle is passed, then backButtonTitle
*  will be used for the back button title if the user goes back to the detailPane.
* 
* 
* @param {HTMLString=} optional new SplitPane.detailTitle.
* @param {HTMLString=} optional new title for the SplitPane.backButton.
* @param {NavigationDirection=} when SplitPane.animateNavigationBarStateChanges is true, this is the direction passed to NavigationBar.setViewState.
*/
showDetailPane(detailPaneTitle?:HTMLString, backButtonTitle?:HTMLString, direction?:NavigationDirection): void;

/**
*  Sets the title for the SplitPane.detailPane.
* 
* @param {HTMLString} new title for the detail pane.
*/
setDetailTitle(title:HTMLString): void;

/**
*  Navigation control that appears only when the navigation pane is not showing (showing detail
*  pane on handset, or portrait mode on tablet).
* 
* @param {Canvas} */
setDetailNavigationControl(control:Canvas): void;

/**
*  Sets a new SplitPane.listPaneTitleTemplate at runtime.
*  
*  By calling this method it is assumed you want the list pane title to change to the new
*  template.
* 
* 
* @param {HTMLString} new template, can use HTML to be styled.
*/
setListPaneTitleTemplate(template:HTMLString): void;

/**
*  Sets a new SplitPane.detailPaneTitleTemplate at runtime.
*  
*  By calling this method it is assumed you want the detail pane title to change to the new
*  template.
* 
* 
* @param {HTMLString} new template, can use HTML to be styled.
*/
setDetailPaneTitleTemplate(template:HTMLString): void;

/**
*  Causes the target pane component to load data and update its title based on the current
*  selection in the source pane. Also shows the target pane if it's not already visible.
*  
*  For the target pane to load data, both the source pane and target pane must be
*  DataBoundComponents or contain a component as a descendant widget, and have a
*  DataSource, and either:
*  
*  
* - The two DataSources must have a Many-To-One relationship declared via
*  DataSourceField.foreignKey, so that ListGrid.fetchRelatedData can be
*  used on the target pane. A common example of this would be navigation from a source
*  pane that's a TreeGrid to a ListGrid target.
*  
* - The two DataSources must be the same, so that the record selected in the source pane
*  can be displayed in the target pane via simply calling DetailViewer.setData. This would apply, for example, if the source pane is a ListGrid and
*  the target is a DynamicForm, so that DynamicForm.editRecord
*  gets called, or if the target is a DetailViewer.
*  
*  For purposes of this check, if the pane is not itself a component, we will use the first
*  component we can find in a breath-first search of the hierarchy underneath it. Note that
*  one or more records must be selected in the source component for related data to be
*  loaded (which should be automatically true for SplitPane.autoNavigate).
*  
*  Even if we can't load related data into the target pane by the above rules, we'll still
*  show the target pane if it's not already visible, except during SplitPane.autoNavigate.
*  
*  The default target is
*  "list"
*  
*  if the SplitPane.listPane is present,
*  otherwise
*  "detail".
*  
*  
*  The title applied to the target pane is based on SplitPane.listPaneTitleTemplate if the target
*  pane is the listPane, otherwise SplitPane.detailPaneTitleTemplate.
*  
*  The source pane usually does not need to be specified: if the
*  target pane is the detailPane, the default source pane
*  is the listPane if present, otherwise the SplitPane.navigationPane. If the
*  target pane is the listPane, the source pane is always
*  the navigationPane.
* 
* 
* @param {CurrentPane=} pane that should navigate
* @param {HTMLString=} optional title to use for target pane. If not specified, the title is based on SplitPane.listPaneTitleTemplate if the target pane is the listPane, otherwise SplitPane.detailPaneTitleTemplate.
* @param {CurrentPane=} source pane used for selection
*/
navigatePane(target?:CurrentPane, title?:HTMLString, source?:CurrentPane): void;

/**
*  Calls SplitPane.navigatePane with the SplitPane.listPane as the target pane.
* 
* @param {HTMLString=} optional title to use instead of the automatically chosen one
*/
navigateListPane(title?:HTMLString): void;

/**
*  Calls SplitPane.navigatePane with the SplitPane.detailPane as the target pane.
* 
* @param {HTMLString=} optional title to use instead of the automatically chosen one
*/
navigateDetailPane(title?:HTMLString): void;

/**
*  Notification method fired when the user clicks the default back / forward buttons
*  on the navigation bar for this SplitPane.
*  
*  Note that the return value will be ignored and cancelation won't be possible if
*  SplitPane.notifyAfterNavigationClick has been set true, since that forces this method to run
*  after we've already navigated to the new pane.
* 
* @param {NavigationDirection} direction in which the user is attempting to navigate.
*/
navigationClick(direction:NavigationDirection): boolean;

/**
*  Notification fired when the SplitPane.currentPane changes, either due to end-user
*  action or due to a programmatic call to SplitPane.setCurrentPane
*  or other APIs that can change the pane.
*  
*  Note that depending on the DeviceMode, this event may not signal that any pane has
*  actually been shown or hidden, since in some modes multiple panes are shown simultaneously.
*  
*  Never fires while the SplitPane is not drawn.
* 
* 
* @param {CurrentPane} new SplitPane.currentPane value.
* @param {CurrentPane} old SplitPane.currentPane value.
* @param {NavigationMethod} reason for pane change
*/
paneChanged(newPane:CurrentPane, oldPane:CurrentPane, navigationMethod:NavigationMethod): void;

/**
*  Notification method fired when the SplitPane.showMiniNav and the
*  up button on the SplitPane.navigationBar's MiniNavControl is
*  clicked.
* 
* 
*/
upClick(): void;

/**
*  Notification method fired when the SplitPane.showMiniNav and the
*  down button on the SplitPane.navigationBar's MiniNavControl is
*  clicked.
* 
* 
*/
downClick(): void;



}
/**
* Creates a new SplitPane
* 
* @param typeCheckedProperties {Partial<SplitPane>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SplitPane} The new instance
*/
class SplitPane{
static create(typeCheckedProperties? : Partial<SplitPane>, uncheckedProperties?: Object): SplitPane;







}

/**
* EditProxy that handles SplitPane objects when editMode is enabled.
*/
interface SplitPaneEditProxy extends LayoutEditProxy {




}
/**
* Creates a new SplitPaneEditProxy
* 
* @param typeCheckedProperties {Partial<SplitPaneEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SplitPaneEditProxy} The new instance
*/
class SplitPaneEditProxy{
static create(typeCheckedProperties? : Partial<SplitPaneEditProxy>, uncheckedProperties?: Object): SplitPaneEditProxy;







}

/**
* Resize bar for use in Layout.resizeBarClass, based on the
*  StretchImg class. As with the ImgSplitbar class, 
*  widgets of this class can be displayed as a resize-bar for widgets 
*  in Layouts where showResizeBar is set to true. Provides a different appearance from
*  the ImgSplitbar class.
*  
*  To specify the resizeBar class for some layout, use the Layout.resizeBarClass
*  property.
*  
*  On mobile devices, you may find that you need to increase the breadth of the bar to make
*  interacting with it easier (e.g. dragging or tapping). For Layout resize bars,
*  this can be done by setting Layout.resizeBarSize.
*/
interface Splitbar extends StretchImg {
/**
* When a Splitbar is created by a layout, the target property
*  of the Splitbar will be a pointer to the member for which it is acting as a resizeBar.
*  The Splitbar will be positioned next to its target, and will resize it on drag completion.
*  
*  See Layout.resizeBarClass, Canvas.showResizeBar and 
*  Canvas.resizeBarTarget for details on configuring the resize bars shown in Layouts.
* @type {Canvas}
* @default null
*/
target?: Canvas;

/**
* Is this split bar vertically orientated?
*  When a Splitbar is created by a layout to be the resizeBar for some
*  member of the layout, the vertical property will be set to true
*  if the layout is horizontal, meaning this resizeBar will be taller than it is wide, 
*  and will allow horizontal resizing of the member.
* @type {boolean}
* @default null
*/
vertical?: boolean;

/**
* The base URL for the image. 
*  
*  The State for the component will be combined with this URL using the
*  same approach as described in Img.src.
*  Then the image segment StretchItem.name as specified by each StretchItem
*  is added.
*  
*  For example, for a stretchImg in "Over" state with a src of "button.png"
*  and a segment name of "stretch", the resulting URL would be "button_Over_stretch.png".
* @type {SCImgURL}
* @default null
*/
src?: SCImgURL;

/**
* Base URL for the image if StretchImg.vertical is false and 
*  StretchImg.src is unset.
* @type {SCImgURL}
* @default null
*/
hSrc?: SCImgURL;

/**
* Base URL for the image if StretchImg.vertical is true and 
*  StretchImg.src is unset.
* @type {SCImgURL}
* @default null
*/
vSrc?: SCImgURL;

/**
* If the default items are used, capSize is the size in pixels of the first and last
*      images in this stretchImg.
* @type {Integer}
* @default null
*/
capSize?: Integer;

/**
* Grip length in pixels (the long icon axis, perpendicular to the Layout direction).
*  
*  If unset, grip will assume the natural length of image.
* @type {Integer}
* @default null
*/
gripLength?: Integer;

/**
* Grip breadth in pixels (the short icon axis, parallel to the Layout direction).
*  
*  If unset, grip will assume the natural breadth of image.
* @type {Integer}
* @default null
*/
gripBreadth?: Integer;

/**
* Default directory for skin images (those defined by the class), relative to the
*  Page-wide Page.getSkinDir.
* @type {SCImgURL}
* @default null
*/
skinImgDir?: SCImgURL;

/**
* Should we show a "grip" image floating above the center of this widget?
* @type {boolean}
* @default null
*/
showGrip?: boolean;

/**
* Suffix used the 'grip' image if StretchImg.showGrip is true.
* @type {string}
* @default "grip"
*/
gripImgSuffix?: string;

/**
* If StretchImg.showGrip is true, this property determines whether to show the
*  'Down' state on the grip image when the user mousedown's on this widget. 
*  Has no effect if StatefulCanvas.showDown is false.
* @type {boolean}
* @default null
*/
showDownGrip?: boolean;

/**
* If StretchImg.showGrip is true, this property determines whether to show the
*  'Over' state on the grip image when the user rolls over on this widget. 
*  Has no effect if StatefulCanvas.showRollOver is false.
* @type {boolean}
* @default null
*/
showRollOverGrip?: boolean;

/**
* If Splitbar.showGrip is true, this property determines whether the grip image
*  displayed should show the "Closed" state when the Splitbar.target 
*  is hidden. Note that if Splitbar.invertClosedGripIfTargetAfter is true, we
*  may show the "closed" state when the target is visible, rather than when it is hidden.
* @type {boolean}
* @default null
*/
showClosedGrip?: boolean;

/**
* Is the Splitbar.target being shown before or after the bar? This property is
*  automatically populated for splitbars created by a layout.
* @type {boolean}
* @default null
*/
targetAfter?: boolean;

/**
* If Splitbar.showClosedGrip is true, and Splitbar.targetAfter is true
*  should we show the "closed" state for the grip when the target is visible (rather than
*  when it is hidden).
*  
*  This property is useful for the case where the grip media is a simple directional arrow.
*  The same image can be used for expanded state on one side of the bar or collapsed
*  state on the other.
* @type {boolean}
* @default true
*/
invertClosedGripIfTargetAfter?: boolean;

/**
* canDrag set to true to allow dragging of the split bar. Dragging the
*  Splitbar will resize it's Splitbar.target
* @type {boolean}
* @default true
*/
canDrag?: boolean;

/**
* If this property is true, a click on the Splitbar will collapse its 
*  Splitbar.target, hiding it and shifting the Splitbar and other members
*  of the layout across to fill the newly available space. If the target is already hidden
*  a click will expand it again (showing it at its normal size).
*  
*  Note that on touch devices, to enable collapsing/uncollapsing the target
*  in response to a tap, Splitbar.canCollapseOnTap must be set to
*  true.
* @type {boolean}
* @default true
*/
canCollapse?: boolean;

/**
* If Splitbar.canCollapse is true, should a tap result in
*  collapsing/uncollapsing the Splitbar.target?
* @type {boolean}
* @default true
*/
canCollapseOnTap?: boolean;

/**
* Splitbars' cursors are set at init time based on whether they are to be used for vertical or
*  horizontal resize. To customize the cursor for this class, modify 
*  Splitbar.vResizeCursor or Splitbar.hResizeCursor rather than this property.
* @type {Cursor}
* @default "hand"
*/
cursor?: Cursor;

/**
* Cursor to display if this Splitbar is to be used for vertical resize of widgets.
* @type {Cursor}
* @default "row-resize"
*/
vResizeCursor?: Cursor;

/**
* Cursor to display if this Splitbar is to be used for horizontal resize of widgets.
* @type {Cursor}
* @default "col-resize"
*/
hResizeCursor?: Cursor;





}
/**
* Creates a new Splitbar
* 
* @param typeCheckedProperties {Partial<Splitbar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Splitbar} The new instance
*/
class Splitbar{
static create(typeCheckedProperties? : Partial<Splitbar>, uncheckedProperties?: Object): Splitbar;







}

/**
* Task that executes another Process inside the current one. A process cannot be
*  embedded within another process as a normal task element. Instead, a StartProcessTask
*  is used to provide the input state, execute the inner process, then write the output
*  back into the calling process state.
*/
interface StartProcessTask extends ScriptTask {
/**
* The Process to be run by this task. Input state is created from
*  Task.inputFieldList and container process state is updated from
*  the inner process state using Task.outputFieldList.
* @type {Process}
* @default null
*/
process?: Process;

/**
* Not applicable to StartProcessTask.
* @type {boolean}
* @default true
*/
isAsync?: boolean;



/**
*  Not applicable to StartProcessTask.
* 
*/
execute(): void;



}
/**
* Creates a new StartProcessTask
* 
* @param typeCheckedProperties {Partial<StartProcessTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {StartProcessTask} The new instance
*/
class StartProcessTask{
static create(typeCheckedProperties? : Partial<StartProcessTask>, uncheckedProperties?: Object): StartProcessTask;







}

/**
* Starts queuing all DataSource operations so they can be sent to the server all together
*  as a transaction.
*/
interface StartTransactionTask extends ProcessElement {




}
/**
* Creates a new StartTransactionTask
* 
* @param typeCheckedProperties {Partial<StartTransactionTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {StartTransactionTask} The new instance
*/
class StartTransactionTask{
static create(typeCheckedProperties? : Partial<StartTransactionTask>, uncheckedProperties?: Object): StartTransactionTask;







}

/**
* StateTask can either copy fields of Process.state to other fields, or apply hardcoded
*  values to Process.state via StateTask.value.
*  
*  Some examples:
*  
*  
* - inputField: "a", outputField: "b" - copies "a" to "b"
*  
* - inputField: "a", outputField: "b", type: "integer" - copies "a" to "b" converting "a" to an integer
*  
* - inputFieldList: ["a","b"], outputField: ["c","d"] - copies "a" and "b" to "c" and "d" respectively.
*  
*/
interface StateTask extends Task {
/**
* If a stateTask does not declare Task.inputField, it must declare a value
*  which should be assigned to the output field.
*  
*  See StateTask.type for how the value is interpreted.
* @type {any}
* @default null
*/
value?: any;

/**
* Type of the value for stateTask.outputField.
*  
*  This can be used in conjunction with StateTask.value to declare the type of the
*  value, or can be used to convert the type of the Task.inputField to
*  the declared type.
*  
*  If no type is declared, the value from an inputField is unchanged or provided via a call
*  to setValue() is unchanged.
*  
*  A value specified for stateTask.value via an attribute in
*  componentXML (see Process.loadProcess) is treated as a boolean if
*  it is the exact string "true" or "false", treated as a "decimal" or "integer" if it
*  parsable as a valid number, otherwise treated as a String. If these heuristics don't
*  work in your case, just declare the type explicitly via stateTask.type.
*  
*  A value of "record" type or "array" type can be declared in Component XML using the same
*  formats allowed for DataSourceField.valueMap. Each array value or
*  record attribute value undergoes the same heuristics as for StateTask.value
*  declared as an attribute.
*  
*  StateTask.type is invalid to use with multiple outputFields.
* @type {ProcessValueType}
* @default null
*/
type?: ProcessValueType;

/**
* ID of the next sequence or element to proceed to if a failure condition arises, such as
*  the output data not being convertible to the target StateTask.type.
* @type {string}
* @default null
*/
failureElement?: string;

/**
* Not applicable to a StateTask.
* @type {string}
* @default null
*/
outputExpression?: string;

/**
* Does this processElement pass through output from the last executed task
*  (i.e. transient state)?
*  
*  See taskInputExpression for details on the
*  transient state outputs.
*  
*  Note that this property does not affect the task at all but is an indicator
*  to the user and to the workflow editor of the behavior of the task as coded
*  (See Process.passThruTaskOutput).
* @type {boolean}
* @default false
*/
passThruOutput?: boolean;





}
/**
* Creates a new StateTask
* 
* @param typeCheckedProperties {Partial<StateTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {StateTask} The new instance
*/
class StateTask{
static create(typeCheckedProperties? : Partial<StateTask>, uncheckedProperties?: Object): StateTask;







}

/**
* A component that has a set of possible states, and which presents itself differently according to
*  which state it is in. An example is a button, which can be "up", "down", "over" or "disabled".
*/
interface StatefulCanvas extends Canvas {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "StatefulCanvasEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* The title HTML to display in this button.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* Should horizontal alignment-related attributes StatefulCanvas.align and StatefulCanvas.iconOrientation
*  be mirrored in RTL mode? This is the default behavior unless ignoreRTL is set to true.
* @type {boolean}
* @default false
*/
ignoreRTL?: boolean;

/**
* Whether this widget needs to redraw to reflect state change
* @type {boolean}
* @default false
*/
redrawOnStateChange?: boolean;

/**
* Whether this component is selected. For some components, selection affects appearance.
* @type {boolean}
* @default false
*/
selected?: boolean;

/**
* Current "state" of this widget. The state setting is automatically updated as the 
*  user interacts with the component (see StatefulCanvas.showRollOver, 
*  StatefulCanvas.showDown, StatefulCanvas.showDisabled).
*  
*  StatefulCanvases will have a different appearance based
*  on their current state. 
*  By default this is handled by changing the css className applied to
*  the StatefulCanvas - see StatefulCanvas.baseStyle and 
*  StatefulCanvas.getStateSuffix for a description of how this is done.
*  
*  For Img or StretchImg based subclasses of StatefulCanvas, the 
*  appearance may also be updated by changing the src of the rendered image. See
*  Img.src and StretchImgButton.src for a description of how the URL 
*  is modified to reflect the state of the widget in this case.
* @type {State}
* @default ""
*/
state?: State;

/**
* Should we visibly change state when the mouse goes over this object?
* @type {boolean}
* @default false
*/
showRollOver?: boolean;

/**
* Should we visibly change state when the canvas receives focus? Note that by default the
*  over state is used to indicate focus.
* @deprecated * as of SmartClient version 6.1 in favor of StatefulCanvas.showFocused

* @type {boolean}
* @default false
*/
showFocus?: boolean;

/**
* Should we visibly change state when the canvas receives focus? If
*  StatefulCanvas.showFocusedAsOver is true, then "over"
*  will be used to indicate focus. Otherwise a separate "focused" state
*  will be used.
* @type {boolean}
* @default false
*/
showFocused?: boolean;

/**
* If StatefulCanvas.showFocused is true for this widget, should the
*  "over" state be used to indicate the widget as focused. If set to false,
*  a separate "focused" state will be used.
* @type {boolean}
* @default true
*/
showFocusedAsOver?: boolean;

/**
* Should we visibly change state when the mouse goes down in this object?
* @type {boolean}
* @default false
*/
showDown?: boolean;

/**
* Should we visibly change state when disabled?
* @type {boolean}
* @default true
*/
showDisabled?: boolean;

/**
* Behavior on state changes -- BUTTON, RADIO or CHECKBOX
* @type {SelectionType}
* @default "button"
*/
actionType?: SelectionType;

/**
* String identifier for this canvas's mutually exclusive selection group.
* @type {string}
* @default null
*/
radioGroup?: string;

/**
* StatefulCanvases are styled by combining StatefulCanvas.baseStyle with State to build
*  a composite css style name. In most cases, statefulCanvas.styleName
*  will have no effect on statefulCanvas styling and should not be used.
*  
*  If the baseStyle is not explicitly specified for a class, the 
*  styleName will be used as a default baseStyle. Other than that, this
*  attribute will be ignored.
* @type {CSSStyleName}
* @default "normal"
*/
styleName?: CSSStyleName;

/**
* Base CSS style className applied to the component.
*  
*  Note that if specified, this property takes precedence over any specified
*  StatefulCanvas.styleName. If unset, the styleName will be used as a 
*  default baseStyle value.
*  
*  As the component changes StatefulCanvas.state and/or is selected, 
*  suffixes will be added to the base style. In some cases more than one suffix will 
*  be appended to reflect a combined state ("Selected" + "Disabled", for example).
*  
*  See StatefulCanvas.getStateSuffix for a description of the default set
*  of suffixes which may be applied to the baseStyle
*  
*  Rotated Titles
*  
*  The Framework doesn't have built-in support for rotating button titles in a fashion
*  similar to FacetChart.rotateLabels. However, you can manually configure
*  a button to render with a rotated title by applying custom CSS via this property.
*  
*  For example, given a button with a height of 120 and a width of 48, if you
*  copied the existing buttonXXX style declarations from skin_styles.css as new,
*  rotatedTitleButtonXXX declarations, and then added the lines:
*  
*    -ms-transform:   translate(-38px,0px) rotate(270deg);
*    -webkit-transform: translate(-38px,0px) rotate(270deg);
*    transform:     translate(-38px,0px) rotate(270deg);
*    overflow: hidden;
*    text-overflow: ellipsis;
*    width:116px;
*  in the declaration section beginning:
*  
*  .rotatedTitleButton,
*  .rotatedTitleButtonSelected,
*  .rotatedTitleButtonSelectedOver,
*  .rotatedTitleButtonSelectedDown,
*  .rotatedTitleButtonSelectedDisabled,
*  .rotatedTitleButtonOver,
*  .rotatedTitleButtonDown,
*  .rotatedTitleButtonDisabled {
*  then applying that style to the button with Canvas.overflow: "clip_h"
*  would yield a vertically-rendered title with overflow via ellipsis as expected, and also
*  wrap with Button.wrap.
*  
*  Note that:
*  
* - The explicit width applied via CSS is needed because rotated
*  elements don't inherit dimensions in their new orientation from the DOM - 
*  the transform/rotation occurs independently of layout.
*  
* - The translation transform required along the x-axis is roughly
*  (width - height) / 2, but may need slight offsetting for optimal centering.
*  
* - We've explicitly avoided describing an approach based on CSS "writing-mode", since
*  support is incomplete and bugs are present in popular browsers such as Firefox and
*  Safari that would prevent it from being used without Framework assistance.
*  
*  Note on css-margins: Developers should be aware that the css "margin" property is unreliable for
*  certain subclasses of StatefulCanvas, including Button. Developers may use 
*  the explicit Canvas.margin property to specify button margins, or for a 
*  button within a layout, consider the layout properties Layout.layoutMargin,
*  Layout.membersMargin
* @type {CSSStyleName}
* @default null
*/
baseStyle?: CSSStyleName;

/**
* Horizontal alignment of this component's title.
* @type {Alignment}
* @default "center"
*/
align?: Alignment;

/**
* Vertical alignment of this component's title.
* @type {VerticalAlignment}
* @default "center"
*/
valign?: VerticalAlignment;

/**
* If true, ignore the specified size of this widget and always size just large
*  enough to accommodate the title. If setWidth() is explicitly called on an
*  autoFit:true button, autoFit will be reset to false.
*  
*  Note that for StretchImgButton instances, autoFit will occur horizontally only, as 
*  unpredictable vertical sizing is likely to distort the media. If you do want vertical 
*  auto-fit, this can be achieved by simply setting a small height, and having 
*  overflow:"visible"
* @type {boolean}
* @default null
*/
autoFit?: boolean;

/**
* Size for this component's horizontal dimension. See Canvas.width for more
*  details.
*  
*  Note that if StatefulCanvas.autoFit is set, this property will be ignored so that the widget
*  is always sized just large enough to accommodate the title.
* @type {number | string}
* @default null
*/
width?: number | string;

/**
* Size for this component's vertical dimension. See Canvas.height for more
*  details.
*  
*  Note that if StatefulCanvas.autoFit is set on non-StretchImgButton instances, this
*  property will be ignored so that the widget is always sized just large enough to
*  accommodate the title.
* @type {number | string}
* @default null
*/
height?: number | string;

/**
* Optional icon to be shown with the button title text. 
*  
*  Specify as the partial URL to an image, relative to the imgDir of this component.
*  A sprited image can be specified using the SCSpriteConfig format.
*  
*  Note that the string "blank" is a valid setting for this attribute and will always 
*  result in the system blank image, with no state suffixes applied. Typically, this 
*  might be used when an iconStyle is also specified and the iconStyle renders the icon via 
*  a stateful background-image or other CSS approach.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* Size in pixels of the icon image.
*  
*  The StatefulCanvas.iconWidth and StatefulCanvas.iconHeight
*  properties can be used to configure width and height separately.
*  
*  Note: When configuring the properties of a StatefulCanvas (or derivative)
*  AutoChild, it is best to set the iconWidth and iconHeight
*  to the same value rather than setting an iconSize. This is because certain
*  skins or customizations thereto might set the iconWidth and iconHeight,
*  making the customization of the AutoChild's iconSize ineffective.
* @type {Integer}
* @default "16"
*/
iconSize?: Integer;

/**
* Width in pixels of the icon image.
*  
*  If unset, defaults to StatefulCanvas.iconSize.
* @type {Integer}
* @default null
*/
iconWidth?: Integer;

/**
* Height in pixels of the icon image.
*  
*  If unset, defaults to StatefulCanvas.iconSize.
* @type {Integer}
* @default null
*/
iconHeight?: Integer;

/**
* Base CSS style applied to the icon image. If set, as the StatefulCanvas changes
*  StatefulCanvas.state and/or is StatefulCanvas.selected,
*  suffixes will be appended to iconStyle to form the className set on the
*  image element.
*  
*  The following table lists out the standard set of suffixes which may be appended:
*  
*  CSS Class AppliedDescription
*  iconStyleDefault CSS style
*  iconStyle+Selected
*    Applied when StatefulCanvas.selected and StatefulCanvas.showSelectedIcon
*    are true.
*  iconStyle+Focused
*    Applied when the component has keyboard focus, if
*    StatefulCanvas.showFocusedIcon is true, and
*    StatefulCanvas.showFocusedAsOver is not true.
*  iconStyle+Over
*    Applied when StatefulCanvas.showRollOverIcon is set to true and either
*    the user rolls over the component or StatefulCanvas.showFocusedAsOver is true
*    and the component has keyboard focus.
*  iconStyle+Down
*    Applied when the user presses the mouse button on the component if
*      StatefulCanvas.showDownIcon is set to true
*  iconStyle+Disabled
*    Applied when the component is Canvas.disabled
*     if StatefulCanvas.showDisabledIcon is true.
*  Combined styles
*  iconStyle+SelectedFocused
*    Combined Selected and focused styling
*  iconStyle+SelectedOver
*    Combined Selected and rollOver styling
*  iconStyle+FocusedOver
*    Combined Focused and rollOver styling
*  iconStyle+SelectedFocusedOver
*    Combined Selected, Focused and rollOver styling
*  iconStyle+SelectedDown
*    Combined Selected and mouse-down styling
*  iconStyle+FocusedDown
*    Combined Focused and mouse-down styling
*  iconStyle+SelectedFocusedDown
*    Combined Selected, Focused and mouse-down styling
*  iconStyle+SelectedDisabled
*    Combined Selected and Disabled styling
*  
*  
*  In addition, if StatefulCanvas.showRTLIcon is true, then in RTL mode, a final
*  "RTL" suffix will be appended.
* @type {CSSStyleName}
* @default null
*/
iconStyle?: CSSStyleName;

/**
* If this button is showing an icon should it appear to the left or right of the title?
*  valid options are "left" and "right".
* @type {string}
* @default "left"
*/
iconOrientation?: string;

/**
* Pixels between icon and title text.
* @type {Integer}
* @default "6"
*/
iconSpacing?: Integer;

/**
* If using an icon for this button, whether to switch the icon image if the button becomes
*  disabled.
* @type {boolean}
* @default true
*/
showDisabledIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image on mouse rollover.
* @type {boolean}
* @default false
*/
showRollOverIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the mouse goes
*  down on the button.
* @type {boolean}
* @default false
*/
showDownIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the button
*  becomes selected.
* @type {boolean}
* @default false
*/
showSelectedIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the button
*  receives focus.
*  
*  If StatefulCanvas.showFocusedAsOver is true, the "Over" icon will be
*  displayed when the canvas has focus, otherwise a separate "Focused" icon
*  will be displayed
* @type {boolean}
* @default false
*/
showFocusedIcon?: boolean;

/**
* Is Page.isRTL media available for the icon? If true, then in RTL mode, the
*  image's src will have "_rtl" inserted immediately before the file extension. For example,
*  if StatefulCanvas.icon is "myIcon.png" and showRTLIcon is true, then in RTL mode, the image's
*  src will be set to "myIcon_rtl.png".
* @type {boolean}
* @default false
*/
showRTLIcon?: boolean;

/**
* When this property is set to true, this widget will create and show the
*  StatefulCanvas.overCanvas on user rollover.
* @type {boolean}
* @default false
*/
showOverCanvas?: boolean;

/**
* Auto generated child widget to be shown when the user rolls over this canvas if 
*  StatefulCanvas.showOverCanvas is true. See documentation for AutoChild
*  for information on how to customize this canvas.
* @type {Canvas}
* @default null
*/
overCanvas?: Canvas;

/**
* Constructor class name for this widget's StatefulCanvas.overCanvas
* @type {string}
* @default "Canvas"
*/
overCanvasConstructor?: string;

/**
* Default properties for this widgets StatefulCanvas.overCanvas. To modify
*  these defaults, use Class.changeDefaults
* @type {Canvas}
* @default "{ ... }"
*/
overCanvasDefaults?: Canvas;

/**
* If specified this property returns the StatefulCanvas.getAriaLabel 
*  attribute to write out in isc.setScreenReaderMode.
*  
*  If unset, aria-label will default to Canvas.prompt if specified,
*  otherwise StatefulCanvas.title.
*  
* @type {string}
* @default null
*/
ariaLabel?: string;

/**
* If true, this widget will fire Canvas.showContextMenu to
*  show the Canvas.contextMenu if one is defined, rather than
*  Canvas.click, when the left mouse is clicked.
*  
*  Note that this property has a different interpretation in IconButton as
*  RibbonButton.showMenuOnClick.
* @type {boolean}
* @default null
*/
showMenuOnClick?: boolean;



/**
*  Retrieves dynamically calculated default Canvas.ariaState 
*  properties for this canvas. These will be combined with explicitly specified aria
*  state as described in Canvas.getAriaState.
*  
*  Overridden by StatefulCanvas to pick up StatefulCanvas.getAriaLabel.
*  
* 
*/
getAriaStateDefaults(): object;

/**
*  Method to return the aria-label for this component 
*  (see StatefulCanvas.getAriaStateDefaults).
*  
*  Returns StatefulCanvas.ariaLabel if specified, otherwise Canvas.prompt, otherwise StatefulCanvas.title
* 
* 
*/
getAriaLabel(): string;

/**
*  Setter for StatefulCanvas.ignoreRTL.
* 
* @param {boolean} new value for ignoreRTL.
*/
setIgnoreRTL(ignoreRTL:boolean): void;

/**
*  Sets the base CSS style. As the component changes state and/or is selected, suffixes will be
*  added to the base style.
* 
* @param {CSSStyleName} new base style
*/
setBaseStyle(style:CSSStyleName): void;

/**
*  Sets the StatefulCanvas.state of this object, changing its appearance.
*  Note: newState cannot be
*  "Disabled"
*  
*  if StatefulCanvas.showDisabled is false.
* 
* 
* @param {State} the new state.
*/
setState(newState:State): void;

/**
*  Return the state of this StatefulCanvas
* 
*/
getState(): State;

/**
*  Set this object to be selected or deselected.
* 
* @param {boolean} new boolean value of whether or not the object is                     selected.
*/
setSelected(newIsSelected:boolean): void;

/**
*  Select this object.
* 
*/
select(): void;

/**
*  Deselect this object.
* 
*/
deselect(): void;

/**
*  Find out if this object is selected
* 
*/
isSelected(): boolean;

/**
*  Return the 'actionType' for this canvas (radio / checkbox / button)
* 
*/
getActionType(): void;

/**
*  Update the 'actionType' for this canvas (radio / checkbox / button)
*  If the canvas is currently selected, and the passed in actionType is 'button'
*  this method will deselect the canvas.
* 
*/
setActionType(): void;

/**
*  Add this widget to the specified mutually exclusive selection group with the ID
*  passed in.
*  Selecting this widget will then deselect any other StatefulCanvases with the same
*  radioGroup ID.
*  StatefulCanvases can belong to only one radioGroup, so this method will remove from 
*  any other radiogroup of which this button is already a member.
* 
* @param {string} - ID of the radiogroup to which this widget should be added
*/
addToRadioGroup(groupID:string): void;

/**
*  Remove this widget from the specified mutually exclusive selection group with the ID
*  passed in.
*  No-op's if this widget is not a member of the groupID passed in.
*  If no groupID is passed in, defaults to removing from whatever radioGroup this widget
*  is a member of.
* 
* @param {string=} - optional radio group ID (to ensure the widget is removed                    from the appropriate group.
*/
removeFromRadioGroup(groupID?:string): void;

/**
*  Enable or disable this object
* 
* @param {boolean} true if this widget is to be disabled
*/
setDisabled(disabled:boolean): void;

/**
*  Returns the suffix that will be appended to the StatefulCanvas.baseStyle 
*  as the component changes StatefulCanvas.state and/or is selected / focused.
*  
*  Note that suffixes will only be included if the relevant show[StateName]
*  attributes (EG StatefulCanvas.showRollOver, StatefulCanvas.showFocused, etc) are set to true.
*  
*  The following table lists out the standard set of suffixes which may be applied 
*  to the base style:
*  
*  CSS Class AppliedDescription
*  baseStyleDefault css style
*  baseStyle+Selected
*    Applied when StatefulCanvas.selected is set to true.
*  baseStyle+Focused
*    Applied when the component has keyboard focus, if 
*    StatefulCanvas.showFocused is true, and 
*    StatefulCanvas.showFocusedAsOver is not true.
*  baseStyle+Over
*    Applied when StatefulCanvas.showRollOver is set to true and either the user
*    rolls over the component or StatefulCanvas.showFocusedAsOver is true and the
*    component has keyboard focus.
*  baseStyle+Down
*    Applied when the user presses the mouse button on the component if
*      StatefulCanvas.showDown is set to true
*  baseStyle+Disabled
*    Applied when the component is Canvas.disabled
*     if StatefulCanvas.showDisabled is true.
*  Combined styles
*  baseStyle+SelectedFocused
*    Combined Selected and focused styling
*  baseStyle+SelectedOver
*    Combined Selected and rollOver styling
*  baseStyle+FocusedOver
*    Combined Focused and rollOver styling
*  baseStyle+SelectedFocusedOver
*    Combined Selected, Focused and rollOver styling
*  baseStyle+SelectedDown
*    Combined Selected and mouse-down styling
*  baseStyle+FocusedDown
*    Combined Focused and mouse-down styling
*  baseStyle+SelectedFocusedDown
*    Combined Selected, Focused and mouse-down styling
*  baseStyle+SelectedDisabled
*    Combined Selected and Disabled styling
*  
* 
* 
*/
getStateSuffix(): string;

/**
*  Changes the orientation of the icon relative to the text of the button.
* 
* 
* @param {string} The new orientation of the icon relative to the text of the button.
*/
setIconOrientation(orientation:string): void;

/**
*  Setter method for the StatefulCanvas.autoFit property. Pass in true or false to turn
*  autoFit on or off. When autoFit is set to false, canvas will be resized to
*  it's previously specified size.
* 
* @param {boolean} New autoFit setting.
*/
setAutoFit(autoFit:boolean): void;

/**
*  Return the title - HTML drawn inside the component.
*  
*  Default is to simply return this.title.
* 
*/
getTitle(): HTMLString;

/**
*  Setter for the StatefulCanvas.title.
* 
* @param {HTMLString} the new title HTML.
*/
setTitle(newTitle:HTMLString): void;

/**
*  Change the icon being shown next to the title text.
* 
* @param {SCImgURL} URL of new icon
*/
setIcon(icon:SCImgURL): void;

/**
*  Setter for StatefulCanvas.iconStyle.
* 
* @param {CSSStyleName} the new iconStyle (may be null to remove the className on the image).
*/
setIconStyle(iconStyle:CSSStyleName): void;



}
/**
* Creates a new StatefulCanvas
* 
* @param typeCheckedProperties {Partial<StatefulCanvas>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {StatefulCanvas} The new instance
*/
class StatefulCanvas{
static create(typeCheckedProperties? : Partial<StatefulCanvas>, uncheckedProperties?: Object): StatefulCanvas;
/**
* A declared value of the enum type 
*  State.
* @type {Constant}
* @default ""
*/
static STATE_UP?: Constant;

/**
* A declared value of the enum type 
*  State.
* @type {Constant}
* @default "Down"
*/
static STATE_DOWN?: Constant;

/**
* A declared value of the enum type 
*  State.
* @type {Constant}
* @default "Over"
*/
static STATE_OVER?: Constant;

/**
* A declared value of the enum type 
*  State.
* @type {Constant}
* @default "Disabled"
*/
static STATE_DISABLED?: Constant;

/**
* A declared value of the enum type 
*  SelectionType.
* @type {Constant}
* @default "button"
*/
static BUTTON?: Constant;

/**
* A declared value of the enum type 
*  SelectionType.
* @type {Constant}
* @default "checkbox"
*/
static CHECKBOX?: Constant;

/**
* A declared value of the enum type 
*  SelectionType.
* @type {Constant}
* @default "radio"
*/
static RADIO?: Constant;

/**
* A declared value of the enum type 
*  Selected.
* @type {Constant}
* @default "Focused"
*/
static FOCUSED?: Constant;

/**
* A declared value of the enum type 
*  Selected.
* @type {Constant}
* @default "Selected"
*/
static SELECTED?: Constant;

/**
* A declared value of the enum type 
*  Selected.
* @type {Constant}
* @default ""
*/
static UNSELECTED?: Constant;








}

/**
* EditProxy that handles StatefulCanvas objects when editMode is enabled.
*/
interface StatefulCanvasEditProxy extends CanvasEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's title.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's title.
* 
* 
* @param {string} the new component title
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new StatefulCanvasEditProxy
* 
* @param typeCheckedProperties {Partial<StatefulCanvasEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {StatefulCanvasEditProxy} The new instance
*/
class StatefulCanvasEditProxy{
static create(typeCheckedProperties? : Partial<StatefulCanvasEditProxy>, uncheckedProperties?: Object): StatefulCanvasEditProxy;







}

/**
* A FormItem that displays an uneditable value.
*/
interface StaticTextItem extends FormItem {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "TextItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* If FormItem.height is specified, should it be applied to the
*  item's text box element?
*  
*  Overridden to be false for StaticTextItems by default.
*  
*  See FormItem.shouldApplyHeightToTextBox for more information.
* @type {boolean}
* @default false
*/
applyHeightToTextBox?: boolean;

/**
* If the FormItem.textAlign is unset, should the FormItem.align
*  setting, if set, be used for this StaticTextItem's textAlign?
* @type {boolean}
* @default true
*/
applyAlignToText?: boolean;

/**
* If true, item contents can wrap. If false, all the contents should appear on a single line.
* @type {boolean}
* @default true
*/
wrap?: boolean;

/**
* If true, text that exceeds the specified size of the form item will be clipped.
*  Note that for horizontal clipping to occur, StaticTextItem.wrap should be
*  set to false - otherwise the text will typically wrap at the specified width.
*  For vertical clipping to occur, StaticTextItem.applyHeightToTextBox should be
*  explicitly set to true as the Text Box element is responsible for
*  clipping the content.
* @type {boolean}
* @default false
*/
clipValue?: boolean;

/**
* Base CSS class for this item
* @type {FormItemBaseStyle}
* @default "staticTextItem"
*/
textBoxStyle?: FormItemBaseStyle;

/**
* Should the user be able to select the text in this item?
* @type {boolean}
* @default true
*/
canSelectText?: boolean;

/**
* By default HTML values in a staticTextItem will be interpreted by the browser.
*  Setting this flag to true will causes HTML characters to be escaped, meaning the
*  raw value of the field (for example "&lt;b&gt;AAA&lt;/b&gt;") is displayed
*  to the user rather than the interpreted HTML (for example "AAA")
* @deprecated * in favor of StaticTextItem.escapeHTML

* @type {boolean}
* @default null
*/
outputAsHTML?: boolean;

/**
* Display format to use for date type values within this formItem.
*  
*  Note that Fields of type "date", "datetime" or "time" will
*  be edited using a DateItem or TimeItem by default, but 
*  this can be overridden - for canEdit:false fields, a
*  StaticTextItem is used by default, and the developer can always specify 
*  a custom FormItem.editorType as well as FormItem.type.
*  
*  The FormItem.timeFormatter may also be used to format underlying Date values as
*  times (ommitting the date part entirely). If both dateFormatter and
*  timeFormatter are specified on an item, for
*  fields specified as FormItem.type the
*  timeFormatter will be used, otherwise the dateFormatter
*  
*  If item.dateFormatter and item.timeFormatter is unspecified,
*  date display format may be defined at the component level via
*  DynamicForm.dateFormatter, or for fields of type "datetime"
*  DynamicForm.datetimeFormatter. Otherwise for fields of type "date",
*  default is to use the system-wide default short date format, configured via
*  DateUtil.setShortDisplayFormat. For fields of type "datetime" or for Date values
*  in fields whose type does not inherit from the logical "date" type, default is to use
*  the system-wide normal date format configured via DateUtil.setNormalDisplayFormat 
*  (using "toNormalDate()" on logical "date" type fields is not desirable as this
*  would display the time component of the date object to the user).
*  Specify any valid DateDisplayFormat to 
*  change the format used by this item.
* @type {DateDisplayFormat}
* @default null
*/
dateFormatter?: DateDisplayFormat;

/**
* By default HTML values in a staticTextItem will be interpreted by the browser.
*  Setting this flag to true causes HTML characters to be escaped, meaning the
*  raw value of the field (for example "&lt;b&gt;AAA&lt;/b&gt;") is displayed
*  to the user rather than the interpreted HTML (for example "AAA")
* @type {boolean}
* @default false
*/
escapeHTML?: boolean;





}
/**
* Creates a new StaticTextItem
* 
* @param typeCheckedProperties {Partial<StaticTextItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {StaticTextItem} The new instance
*/
class StaticTextItem{
static create(typeCheckedProperties? : Partial<StaticTextItem>, uncheckedProperties?: Object): StaticTextItem;







}

/**
* The StretchImg widget class implements a widget type that displays a list of multiple images
*  that make up a single image.
*/
interface StretchImg extends StatefulCanvas {
/**
* Indicates whether the list of images is drawn vertically from top to bottom (true),
*  or horizontally from left to right (false).
* @type {boolean}
* @default true
*/
vertical?: boolean;

/**
* If the default items are used, capSize is the size in pixels of the first and last
*      images in this stretchImg.
* @type {number}
* @default "2"
*/
capSize?: number;

/**
* The base URL for the image. 
*  
*  The State for the component will be combined with this URL using the
*  same approach as described in Img.src.
*  Then the image segment StretchItem.name as specified by each StretchItem
*  is added.
*  
*  For example, for a stretchImg in "Over" state with a src of "button.png"
*  and a segment name of "stretch", the resulting URL would be "button_Over_stretch.png".
* @type {SCImgURL}
* @default null
*/
src?: SCImgURL;

/**
* Base URL for the image if StretchImg.vertical is false and 
*  StretchImg.src is unset.
* @type {SCImgURL}
* @default null
*/
hSrc?: SCImgURL;

/**
* Base URL for the image if StretchImg.vertical is true and 
*  StretchImg.src is unset.
* @type {SCImgURL}
* @default null
*/
vSrc?: SCImgURL;

/**
* Indicates whether the image should be tiled/cropped, stretched, or centered when the
*      size of this widget does not match the size of the image. See ImageStyle for
*      details.
* @type {ImageStyle}
* @default "stretch"
*/
imageType?: ImageStyle;

/**
* The list of images to display as an array of objects specifying the image names and
*  sizes.
*  
*  The StretchItem.name is appended as a suffix to the StretchImg.src URL in order
*  to fetch separate media files for each image. Alternatively a StretchItem may specify
*  its own StretchItem.src.
*  
*  The StretchItem.height and StretchItem.width can be set to a number,
*  "*" (remaining space, divided amongst all images that specify "*") or to the name of a
*  property on this StretchImg component, such as "capSize" for the StretchImg.capSize.
*  
*  Height or width is only used for the axis along which images are stacked. For example, if
*  StretchImg.vertical is true, images stack vertically and heights are used to size images on
*  the vertical axis, but all images will have width matching the overall component size.
*  
*  For example, the default setting for items, which is used to produce
*  stretchable buttons and headers with fixed-size endcaps, is as follows:
*  
*   items:[
*     {height:"capSize", name:"start", width:"capSize"},
*     {height:"*", name:"stretch", width:"*"},
*     {height:"capSize", name:"end", width:"capSize"}
*   ]
*  
*  Note that by default horizontal StretchImg instances will always render their items
*  in left-to-right order, even if the page is localized for right-to-left display
*  (see Page.isRTL). This default behavior may be overridden by setting the
*  StretchImg.ignoreRTL flag to false.
* @type {Array<Partial<StretchItem>>}
* @default "see below"
*/
items?: Array<Partial<StretchItem>>;

/**
* Should the StretchImg.items for this StretchImg display left-to-right even
*  if this page is displaying Page.isRTL?
*  
*  Only has an effect if this StretchImg is horizontal (StretchImg.vertical
*  is set to false).
*  
*  Having this property set to true is usually desirable for the common pattern of media
*  consisting of fixed size "end caps" and a stretchable center, because it allows the same
*  media to be used for LTR and RTL pages.
*  
*  If set to false, items will be displayed in RTL order for RTL pages.
* @type {boolean}
* @default true
*/
ignoreRTL?: boolean;

/**
* Should we show a "grip" image floating above the center of this widget?
* @type {boolean}
* @default null
*/
showGrip?: boolean;

/**
* Suffix used the 'grip' image if StretchImg.showGrip is true.
* @type {string}
* @default "grip"
*/
gripImgSuffix?: string;

/**
* If StretchImg.showGrip is true, this property determines whether to show the
*  'Down' state on the grip image when the user mousedown's on this widget. 
*  Has no effect if StatefulCanvas.showDown is false.
* @type {boolean}
* @default null
*/
showDownGrip?: boolean;

/**
* If StretchImg.showGrip is true, this property determines whether to show the
*  'Over' state on the grip image when the user rolls over on this widget. 
*  Has no effect if StatefulCanvas.showRollOver is false.
* @type {boolean}
* @default null
*/
showRollOverGrip?: boolean;

/**
* Determines whether any specified StatefulCanvas.getTitle will be 
*  displayed for this component.
*  Applies to Image-based components only, where the title will be rendered out in a label
*  floating over the component
* @type {boolean}
* @default false
*/
showTitle?: boolean;

/**
* If specified this css class will be applied to the individual item images within this StretchImg.
*  May be overridden by specifying item-specific base styles to each object in the
*  StretchImg.items. This base style will have standard stateful suffixes
*  appended to indicate the state of this component (as described in 
*  StatefulCanvas.baseStyle).
* @type {CSSStyleName}
* @default null
*/
itemBaseStyle?: CSSStyleName;



/**
*  Set the specified image's state to newState and update the displayed image given by
*  whichPart, or set the state for all images to newState and update the displayed images
*  if whichPart is not provided.
* 
* @param {string} name for the new state ("off", "down", etc)
* @param {string=} name of the piece to set ("start", "stretch" or "end")     	if not specified, sets them all
*/
setState(newState:string, whichPart?:string): void;

/**
*  Changes the base StretchImg.src for this stretchImg, redrawing if necessary.
* 
* 
* @param {SCImgURL} new URL for the image
*/
setSrc(src:SCImgURL): void;

/**
*  Setter for StretchImg.items.
* 
* @param {Array<Partial<StretchItem>>} the new array of items.
*/
setItems(items:Array<Partial<StretchItem>>): void;

/**
*  Setter for StretchImg.ignoreRTL.
* 
* @param {boolean} new value for ignoreRTL.
*/
setIgnoreRTL(ignoreRTL:boolean): void;



}
/**
* Creates a new StretchImg
* 
* @param typeCheckedProperties {Partial<StretchImg>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {StretchImg} The new instance
*/
class StretchImg{
static create(typeCheckedProperties? : Partial<StretchImg>, uncheckedProperties?: Object): StretchImg;







}

/**
* A StretchImg that behaves like a button, going through up/down/over state transitions in response
*  to user events. Supports an optional title, and will auto-size to accommodate the title text if
*  overflow is set to "visible".
*  
*  Examples of use include fancy buttons, poplist headers, and tabs.
*/
interface StretchImgButton extends StretchImg {
/**
* The title HTML to display in this button.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* If true and the title is clipped, then a hover containing the full title of this button
*  is enabled.
* @type {boolean}
* @default false
*/
showClippedTitleOnHover?: boolean;

/**
* Should the title for this button wrap? If unset, default behavior is to allow wrapping
*  if this.vertical is true, otherwise disallow wrapping
* @type {boolean}
* @default null
*/
wrap?: boolean;

/**
* Optional icon to be shown with the button title text. 
*  
*  Specify as the partial URL to an image, relative to the imgDir of this component.
*  A sprited image can be specified using the SCSpriteConfig format.
*  
*  Note that the string "blank" is a valid setting for this attribute and will always 
*  result in the system blank image, with no state suffixes applied. Typically, this 
*  might be used when an iconStyle is also specified and the iconStyle renders the icon via 
*  a stateful background-image or other CSS approach.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* Size in pixels of the icon image.
*  
*  The StatefulCanvas.iconWidth and StatefulCanvas.iconHeight
*  properties can be used to configure width and height separately.
*  
*  Note: When configuring the properties of a StatefulCanvas (or derivative)
*  AutoChild, it is best to set the iconWidth and iconHeight
*  to the same value rather than setting an iconSize. This is because certain
*  skins or customizations thereto might set the iconWidth and iconHeight,
*  making the customization of the AutoChild's iconSize ineffective.
* @type {Integer}
* @default "16"
*/
iconSize?: Integer;

/**
* Width in pixels of the icon image.
*  
*  If unset, defaults to StatefulCanvas.iconSize.
* @type {Integer}
* @default null
*/
iconWidth?: Integer;

/**
* Height in pixels of the icon image.
*  
*  If unset, defaults to StatefulCanvas.iconSize.
* @type {Integer}
* @default null
*/
iconHeight?: Integer;

/**
* If this button is showing an icon should it appear to the left or right of the title?
*  valid options are "left" and "right".
* @type {string}
* @default "left"
*/
iconOrientation?: string;

/**
* If this button is showing an icon should it be right or left aligned?
* @type {string}
* @default null
*/
iconAlign?: string;

/**
* Pixels between icon and title text.
* @type {Integer}
* @default "6"
*/
iconSpacing?: Integer;

/**
* If using an icon for this button, whether to switch the icon image if the button becomes
*  disabled.
* @type {boolean}
* @default true
*/
showDisabledIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image on mouse rollover.
* @type {boolean}
* @default false
*/
showRollOverIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the button
*  receives focus.
*  
*  If StatefulCanvas.showFocusedAsOver is true, the "Over" icon will be
*  displayed when the canvas has focus, otherwise a separate "Focused" icon
*  will be displayed
* @type {boolean}
* @default false
*/
showFocusedIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the mouse goes
*  down on the button.
* @type {boolean}
* @default false
*/
showDownIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the button
*  becomes selected.
* @type {boolean}
* @default false
*/
showSelectedIcon?: boolean;

/**
* If true, ignore the specified size of this widget and always size just large
*  enough to accommodate the title. If setWidth() is explicitly called on an
*  autoFit:true button, autoFit will be reset to false.
*  
*  Note that for StretchImgButton instances, autoFit will occur horizontally only, as 
*  unpredictable vertical sizing is likely to distort the media. If you do want vertical 
*  auto-fit, this can be achieved by simply setting a small height, and having 
*  overflow:"visible"
* @type {boolean}
* @default null
*/
autoFit?: boolean;

/**
* Size for this component's horizontal dimension. See Canvas.width for more
*  details.
*  
*  Note that if StatefulCanvas.autoFit is set, this property will be ignored so that the widget
*  is always sized just large enough to accommodate the title.
* @type {number | string}
* @default null
*/
width?: number | string;

/**
* Size for this component's vertical dimension. See Canvas.height for more
*  details.
*  
*  Note that if StatefulCanvas.autoFit is set on non-StretchImgButton instances, this
*  property will be ignored so that the widget is always sized just large enough to
*  accommodate the title.
* @type {number | string}
* @default null
*/
height?: number | string;

/**
* Base CSS style className applied to the component.
*  
*  Note that if specified, this property takes precedence over any specified
*  StatefulCanvas.styleName. If unset, the styleName will be used as a 
*  default baseStyle value.
*  
*  As the component changes StatefulCanvas.state and/or is selected, 
*  suffixes will be added to the base style. In some cases more than one suffix will 
*  be appended to reflect a combined state ("Selected" + "Disabled", for example).
*  
*  See StatefulCanvas.getStateSuffix for a description of the default set
*  of suffixes which may be applied to the baseStyle
*  
*  Rotated Titles
*  
*  The Framework doesn't have built-in support for rotating button titles in a fashion
*  similar to FacetChart.rotateLabels. However, you can manually configure
*  a button to render with a rotated title by applying custom CSS via this property.
*  
*  For example, given a button with a height of 120 and a width of 48, if you
*  copied the existing buttonXXX style declarations from skin_styles.css as new,
*  rotatedTitleButtonXXX declarations, and then added the lines:
*  
*    -ms-transform:   translate(-38px,0px) rotate(270deg);
*    -webkit-transform: translate(-38px,0px) rotate(270deg);
*    transform:     translate(-38px,0px) rotate(270deg);
*    overflow: hidden;
*    text-overflow: ellipsis;
*    width:116px;
*  in the declaration section beginning:
*  
*  .rotatedTitleButton,
*  .rotatedTitleButtonSelected,
*  .rotatedTitleButtonSelectedOver,
*  .rotatedTitleButtonSelectedDown,
*  .rotatedTitleButtonSelectedDisabled,
*  .rotatedTitleButtonOver,
*  .rotatedTitleButtonDown,
*  .rotatedTitleButtonDisabled {
*  then applying that style to the button with Canvas.overflow: "clip_h"
*  would yield a vertically-rendered title with overflow via ellipsis as expected, and also
*  wrap with Button.wrap.
*  
*  Note that:
*  
* - The explicit width applied via CSS is needed because rotated
*  elements don't inherit dimensions in their new orientation from the DOM - 
*  the transform/rotation occurs independently of layout.
*  
* - The translation transform required along the x-axis is roughly
*  (width - height) / 2, but may need slight offsetting for optimal centering.
*  
* - We've explicitly avoided describing an approach based on CSS "writing-mode", since
*  support is incomplete and bugs are present in popular browsers such as Firefox and
*  Safari that would prevent it from being used without Framework assistance.
*  
*  Note on css-margins: Developers should be aware that the css "margin" property is unreliable for
*  certain subclasses of StatefulCanvas, including Button. Developers may use 
*  the explicit Canvas.margin property to specify button margins, or for a 
*  button within a layout, consider the layout properties Layout.layoutMargin,
*  Layout.membersMargin
* @type {CSSStyleName}
* @default "stretchImgButton"
*/
baseStyle?: CSSStyleName;

/**
* CSS style applied to the title text only. Defaults to StretchImgButton.baseStyle when unset.
*  
*  With a separate titleStyle and StretchImgButton.baseStyle set, you can provide a
*  backgroundColor via baseStyle that will allow translucent .png media to be
*  "tinted" by the underlying background color, so that a single set of media can provide
*  range of color options. In this usage, the titleStyle should generally not
*  specify a background color as this would block out the media that appears behind the
*  title.
* @type {CSSStyleName}
* @default null
*/
titleStyle?: CSSStyleName;

/**
* Specifies a skinImgDir to apply to the label containing the title of this 
*  StretchImgButton. May be null in which case this.skinImgDir
*  will be applied to the label as well.
*  
*  Note that icons displayed in the title may make use of the skin img dir set here
* @type {URL}
* @default null
*/
labelSkinImgDir?: URL;

/**
* Whether this component is selected. For some components, selection affects appearance.
* @type {boolean}
* @default false
*/
selected?: boolean;

/**
* String identifier for this canvas's mutually exclusive selection group.
* @type {string}
* @default null
*/
radioGroup?: string;

/**
* Behavior on state changes -- BUTTON, RADIO or CHECKBOX
* @type {SelectionType}
* @default "button"
*/
actionType?: SelectionType;

/**
* Current "state" of this widget. The state setting is automatically updated as the 
*  user interacts with the component (see StatefulCanvas.showRollOver, 
*  StatefulCanvas.showDown, StatefulCanvas.showDisabled).
*  
*  StatefulCanvases will have a different appearance based
*  on their current state. 
*  By default this is handled by changing the css className applied to
*  the StatefulCanvas - see StatefulCanvas.baseStyle and 
*  StatefulCanvas.getStateSuffix for a description of how this is done.
*  
*  For Img or StretchImg based subclasses of StatefulCanvas, the 
*  appearance may also be updated by changing the src of the rendered image. See
*  Img.src and StretchImgButton.src for a description of how the URL 
*  is modified to reflect the state of the widget in this case.
* @type {State}
* @default ""
*/
state?: State;

/**
* Should we visibly change state when disabled?
* @type {boolean}
* @default true
*/
showDisabled?: boolean;

/**
* Should we visibly change state when the mouse goes down in this object?
* @type {boolean}
* @default false
*/
showDown?: boolean;

/**
* Should we visibly change state when the canvas receives focus? Note that by default the
*  over state is used to indicate focus.
* @deprecated * as of SmartClient version 6.1 in favor of StatefulCanvas.showFocused

* @type {boolean}
* @default false
*/
showFocus?: boolean;

/**
* Should we visibly change state when the canvas receives focus? If
*  StatefulCanvas.showFocusedAsOver is true, then "over"
*  will be used to indicate focus. Otherwise a separate "focused" state
*  will be used.
* @type {boolean}
* @default false
*/
showFocused?: boolean;

/**
* Should we visibly change state when the mouse goes over this object?
* @type {boolean}
* @default false
*/
showRollOver?: boolean;

/**
* Horizontal alignment of this component's title.
* @type {Alignment}
* @default "center"
*/
align?: Alignment;

/**
* Vertical alignment of this component's title.
* @type {VerticalAlignment}
* @default "center"
*/
valign?: VerticalAlignment;

/**
* Determines whether any specified StatefulCanvas.getTitle will be 
*  displayed for this component.
*  Applies to Image-based components only, where the title will be rendered out in a label
*  floating over the component
* @type {boolean}
* @default true
*/
showTitle?: boolean;

/**
* The padding for a StretchImgButton's label is determined as follows.
*  
*  If labelHPad is set it will specify the horizontal padding applied to the
*  label. Similarly if labelVPad is set it will specify the vertical padding
*  for the label, regardless of the button's StretchImgButton.vertical setting.
*  
*  Otherwise labelLengthPad can be set to specify the label padding along the
*  length axis (ie: horizontal padding if StretchImgButton.vertical is false,
*  otherwise vertical padding), and 
*  labelBreadthPad can be set to specify the label padding along the other axis.
*  
*  Otherwise the padding on the length axis will match the StretchImgButton.capSize and
*  will be set to zero on the breadth axis.
*  
*  So by default the label will be sized to match the center image of the StretchImgButton, but
*  these settings allow the label to partially or wholly overlap the caps.
* @type {number}
* @default null
*/
labelHPad?: number;

/**
* The padding for a StretchImgButton's label is determined as follows.
*  
*  If labelHPad is set it will specify the horizontal padding applied to the
*  label. Similarly if labelVPad is set it will specify the vertical padding
*  for the label, regardless of the button's StretchImgButton.vertical setting.
*  
*  Otherwise labelLengthPad can be set to specify the label padding along the
*  length axis (ie: horizontal padding if StretchImgButton.vertical is false,
*  otherwise vertical padding), and 
*  labelBreadthPad can be set to specify the label padding along the other axis.
*  
*  Otherwise the padding on the length axis will match the StretchImgButton.capSize and
*  will be set to zero on the breadth axis.
*  
*  So by default the label will be sized to match the center image of the StretchImgButton, but
*  these settings allow the label to partially or wholly overlap the caps.
* @type {number}
* @default null
*/
labelVPad?: number;

/**
* The padding for a StretchImgButton's label is determined as follows.
*  
*  If labelHPad is set it will specify the horizontal padding applied to the
*  label. Similarly if labelVPad is set it will specify the vertical padding
*  for the label, regardless of the button's StretchImgButton.vertical setting.
*  
*  Otherwise labelLengthPad can be set to specify the label padding along the
*  length axis (ie: horizontal padding if StretchImgButton.vertical is false,
*  otherwise vertical padding), and 
*  labelBreadthPad can be set to specify the label padding along the other axis.
*  
*  Otherwise the padding on the length axis will match the StretchImgButton.capSize and
*  will be set to zero on the breadth axis.
*  
*  So by default the label will be sized to match the center image of the StretchImgButton, but
*  these settings allow the label to partially or wholly overlap the caps.
* @type {number}
* @default null
*/
labelLengthPad?: number;

/**
* The padding for a StretchImgButton's label is determined as follows.
*  
*  If labelHPad is set it will specify the horizontal padding applied to the
*  label. Similarly if labelVPad is set it will specify the vertical padding
*  for the label, regardless of the button's StretchImgButton.vertical setting.
*  
*  Otherwise labelLengthPad can be set to specify the label padding along the
*  length axis (ie: horizontal padding if StretchImgButton.vertical is false,
*  otherwise vertical padding), and 
*  labelBreadthPad can be set to specify the label padding along the other axis.
*  
*  Otherwise the padding on the length axis will match the StretchImgButton.capSize and
*  will be set to zero on the breadth axis.
*  
*  So by default the label will be sized to match the center image of the StretchImgButton, but
*  these settings allow the label to partially or wholly overlap the caps.
* @type {number}
* @default null
*/
labelBreadthPad?: number;

/**
* If set to true, if the StatefulCanvas.title of this button contains the
*  specified Canvas.accessKey, when the title is displayed to the user
*  it will be modified to include HTML to underline the accessKey.
*  Note that this property may cause titles that include HTML (rather than simple strings)
*  to be inappropriately modified, so should be disabled if your title string includes
*  HTML characters.
* @type {boolean}
* @default true
*/
hiliteAccessKey?: boolean;

/**
* Base URL for the image. By default, StretchImgButtons consist of three image parts: A
*  start image (displayed at the top or left), a scalable central image and an end image
*  displayed at the bottom or right.
*  
*  The images displayed in the stretchImgButton are derived from this property in the 
*  following way:
*  
*  
*  
* - When the button is in its standard state the suffixes "_start", "_end" and 
*    "_stretch" are applied to the src (before the file extension), so by default 
*    the images displayed will be "button_start.gif" (sized to be 
*    this.capSize by the specified width of the stretchImgButton), 
*    "button_stretch.gif" (stretched to the necessary width) and "button_end.gif" 
*    (sized the same as the start image).
*  
* - As the button's state changes, the images will have suffixes appended before
*    the "_start" / "_end" / "_stretch" to represent the button state.
*    See Img.src for an overview of how states are combined into a compound
*    URL.
*  
*  For example the center piece of a selected stretchImgButton with the mouse hovering
*  over it might have the URL: "button_Selected_Down_stretch.gif".
*  
*  Media should be present for each possible state of the _start, _end and _stretch images.
* @type {SCImgURL}
* @default "button.gif"
*/
src?: SCImgURL;

/**
* Default is a horizontal button. Vertical StretchImgButtons are allowed, but title text,
*  if any, will not be automatically rotated.
* @type {boolean}
* @default false
*/
vertical?: boolean;

/**
* How big are the end pieces by default
* @type {number}
* @default "12"
*/
capSize?: number;



/**
*  Return the title - HTML drawn inside the component.
*  
*  Default is to simply return this.title.
* 
*/
getTitle(): HTMLString;

/**
*  Setter for the StatefulCanvas.title.
* 
* @param {HTMLString} the new title HTML.
*/
setTitle(newTitle:HTMLString): void;

/**
*  Changes the orientation of the icon relative to the text of the button.
* 
* 
* @param {string} The new orientation of the icon relative to the text of the button.
*/
setIconOrientation(orientation:string): void;

/**
*  Change the icon being shown next to the title text.
* 
* @param {SCImgURL} URL of new icon
*/
setIcon(icon:SCImgURL): void;

/**
*  Setter method for the StatefulCanvas.autoFit property. Pass in true or false to turn
*  autoFit on or off. When autoFit is set to false, canvas will be resized to
*  it's previously specified size.
* 
* @param {boolean} New autoFit setting.
*/
setAutoFit(autoFit:boolean): void;

/**
*  Sets the base CSS style. As the component changes state and/or is selected, suffixes will be
*  added to the base style.
* 
* @param {CSSStyleName} new base style
*/
setBaseStyle(style:CSSStyleName): void;

/**
*  Sets the StretchImgButton.titleStyle, which is applied to the title text.
* 
* @param {CSSStyleName} new title style
*/
setTitleStyle(style:CSSStyleName): void;

/**
*  setter for StretchImgButton.labelSkinImgDir.
* 
* @param {URL} new skin img dir to apply to the label holding title text for  this widget.
*/
setLabelSkinImgDir(URL:URL): void;

/**
*  Select this object.
* 
*/
select(): void;

/**
*  Select this object.
* 
*/
deselect(): void;

/**
*  Find out if this object is selected
* 
*/
isSelected(): boolean;

/**
*  Select this object.
* 
*/
setSelected(): void;

/**
*  Add this widget to the specified mutually exclusive selection group with the ID
*  passed in.
*  Selecting this widget will then deselect any other StatefulCanvases with the same
*  radioGroup ID.
*  StatefulCanvases can belong to only one radioGroup, so this method will remove from 
*  any other radiogroup of which this button is already a member.
* 
* @param {string} - ID of the radiogroup to which this widget should be added
*/
addToRadioGroup(groupID:string): void;

/**
*  Remove this widget from the specified mutually exclusive selection group with the ID
*  passed in.
*  No-op's if this widget is not a member of the groupID passed in.
*  If no groupID is passed in, defaults to removing from whatever radioGroup this widget
*  is a member of.
* 
* @param {string=} - optional radio group ID (to ensure the widget is removed                    from the appropriate group.
*/
removeFromRadioGroup(groupID?:string): void;

/**
*  Update the 'actionType' for this canvas (radio / checkbox / button)
*  If the canvas is currently selected, and the passed in actionType is 'button'
*  this method will deselect the canvas.
* 
*/
setActionType(): void;

/**
*  Return the 'actionType' for this canvas (radio / checkbox / button)
* 
*/
getActionType(): void;

/**
*  Sets the StatefulCanvas.state of this object, changing its appearance.
*  Note: newState cannot be
*  "Disabled"
*  
*  if StatefulCanvas.showDisabled is false.
* 
* 
* @param {State} the new state.
*/
setState(newState:State): void;

/**
*  Enable or disable this object
* 
* @param {boolean} true if this widget is to be disabled
*/
setDisabled(disabled:boolean): void;

/**
*  Return the state of this StatefulCanvas
* 
*/
getState(): State;

/**
*  This property contains the default 'action' for the Button to fire when activated.
* 
*/
action(): void;

/**
*  Is the title of this button clipped?
* 
*/
titleClipped(): boolean;

/**
*  Returns the HTML that is displayed by the default StretchImgButton.titleHover
*  handler. Return null or an empty string to cancel the hover.
*  
* 
* @param {HTMLString} the HTML that would have been displayed by default
*/
titleHoverHTML(defaultHTML:HTMLString): HTMLString;

/**
*  If this button is showing an StretchImgButton.icon, a separate click
*  handler for the icon may be defined as this.iconClick.
*  Returning false will suppress the standard button click handling code.
* 
*/
iconClick(): boolean;

/**
*  Optional stringMethod to fire when the user hovers over this button and the title is
*  clipped. If StretchImgButton.showClippedTitleOnHover is true, the default behavior is to
*  show a hover canvas containing the HTML returned by StretchImgButton.titleHoverHTML.
*  Return false to suppress this default behavior.
* 
*/
titleHover(): boolean;



}
/**
* Creates a new StretchImgButton
* 
* @param typeCheckedProperties {Partial<StretchImgButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {StretchImgButton} The new instance
*/
class StretchImgButton{
static create(typeCheckedProperties? : Partial<StretchImgButton>, uncheckedProperties?: Object): StretchImgButton;







}

/**
* Button that saves the data in the form, by calling DynamicForm.submit when clicked.
*  DynamicForm.submit for details on how to control what happens when a form is
*  submitted.
*/
interface SubmitItem extends ButtonItem {
/**
* SubmitItems show a title of "Submit" by default. May be overridden.
* @type {string}
* @default "Submit"
*/
title?: string;





}
/**
* Creates a new SubmitItem
* 
* @param typeCheckedProperties {Partial<SubmitItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SubmitItem} The new instance
*/
class SubmitItem{
static create(typeCheckedProperties? : Partial<SubmitItem>, uncheckedProperties?: Object): SubmitItem;







}

/**
* Shows an interface allowing a user to create or edit fields by typing simple
*  format-strings into a text field. The format-strings can include the values of other fields
*  and additional text as required.
*  
*  Available values for the format-string are determined by the DataSource fields, and are given
*  simple single-letter aliases (such as "A", "B", ...) similar to column names in Excel.
*  The set of available values is shown in the FormulaBuilder.fieldKey as a simple
*  mapping between the DataSourceField.title and it's short name.
*  
*  To include a field in the format-string, prefix it with a hash sign (#).
*/
interface SummaryBuilder extends FormulaBuilder {
/**
* Indicates whether to use "summary" or some other keyword in various captions and text
* @type {string}
* @default "Summary"
*/
builderTypeText?: string;

/**
* DataSource providing the available fields for the formulaBuilder.
*  
*  By default the formulaBuilder will include only fields of numeric type or derived
*  from a numeric type. Set FormulaBuilder.fields to override this.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* DataSource providing the available fields for the SummaryBuilder.
*  
*  By default the SummaryBuilder will include all fields. Set SummaryBuilder.fields to 
*  override this.
* @type {Array<Partial<string>>}
* @default null
*/
fields?: Array<Partial<string>>;

/**
* Are we editing an existing field?
* @type {boolean}
* @default false
*/
editMode?: boolean;

/**
* The Field object representing the field being created or edited.
* @type {string}
* @default null
*/
field?: string;

/**
* TextItem that users type into when entering a formula.
* @type {TextItem}
* @default null
*/
formulaField?: TextItem;

/**
* TextItem that allows users to set the title for this field.
* @type {TextItem}
* @default null
*/
titleField?: TextItem;

/**
* Whether to show the help icon that appears after the SummaryBuilder.formulaField.
* @type {boolean}
* @default true
*/
showHelpIcon?: boolean;

/**
* Icon that appears after the SummaryBuilder.formulaField, showing help on hover.
* @type {FormItemIcon}
* @default null
*/
helpIcon?: FormItemIcon;

/**
* Whether to show a checkbox offering the user the ability to automatically
*  hide any fields involved in the formula.
* @type {boolean}
* @default true
*/
showAutoHideCheckBox?: boolean;

/**
* CheckBox that, when selected, hides columns in the component that are used in this formula.
* @type {TextItem}
* @default null
*/
autoHideCheckBox?: TextItem;

/**
* Record to use when showing sample output for the format string.
*  
*  If not specified, the selected record in the component that launched the SummaryBuilder will
*  be used, or if there's no selection, the first visible row, or with no component, a dummy
*  data row derived automatically from the provided DataSource.
* @type {Record}
* @default null
*/
testRecord?: Record;

/**
* Text label for the checkbox that allows the user to automatically hide the
*  fields used in the summary format.
* @type {string}
* @default "Auto hide fields used in summary"
*/
autoHideCheckBoxLabel?: string;

/**
* Text that appears in the hover from the SummaryBuilder.helpIcon, as a pre-amble to the list of
*  available format-tokens.
* @type {string}
* @default "Building Summary Columns"
*/
helpTextIntro?: string;



/**
*  Call to set the format-string in this SummaryBuilder.
*  
*  Note that calling setSummary() will update the UI, generate the summary's function and 
*  test it automatically.
* 
* 
* @param {string} The new format-string for the summary
*/
setSummary(newValue:string): void;

/**
*  Call to retrieve the text the SummaryBuilder would show by default for help, or override to
*  provide alternate help text.
* 
* 
*/
getHelpText(): string;

/**
*  Override to execute a callback function when the Format is Cancelled or Saved.
*  
* 
*/
fireOnClose(): void;

/**
*  Call to finish working, test the formula and call 
*  FormulaBuilder.fireOnClose. Called automatically
*  when the Save button is clicked.
* 
* 
*/
save(): void;



}
/**
* Creates a new SummaryBuilder
* 
* @param typeCheckedProperties {Partial<SummaryBuilder>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SummaryBuilder} The new instance
*/
class SummaryBuilder{
static create(typeCheckedProperties? : Partial<SummaryBuilder>, uncheckedProperties?: Object): SummaryBuilder;







}

/**
* Abstract base class for regular expression-based source code colorizer.
*  
*  An instance of this class is never instantiated. Instead, use one of the source-specific
*  subclasses: XMLSyntaxHiliter or JSSyntaxHiliter. 
*  
*  NOTE: This class exists only for use with the SmartClient Feature Explorer and 
*  SmartGWT Showcases and cannot be used in any other environment.
*/
interface SyntaxHiliter extends Class {


/**
*  Highlights the passed in source by applying span style elements to matched tokens
*  and returns it as a string.
* 
* 
* @param {string} the source to be colorized
*/
hilite(source:string): string;



}
/**
* Creates a new SyntaxHiliter
* 
* @param typeCheckedProperties {Partial<SyntaxHiliter>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {SyntaxHiliter} The new instance
*/
class SyntaxHiliter{
static create(typeCheckedProperties? : Partial<SyntaxHiliter>, uncheckedProperties?: Object): SyntaxHiliter;







}

/**
* Shows a set of Tabs. TabBars are automatically created by TabSets and shouldn't be used
*  directly. The TabBar is documented for skinning purposes.
*/
interface TabBar extends Toolbar {
/**
* Tab for this TabBar.
* @type {Array<Partial<Partial<Tab>>>}
* @default null
*/
tabs?: Array<Partial<Partial<Tab>>>;

/**
* Breadth of the tabBar (including baseline breadth)
* @type {number}
* @default "21"
*/
breadth?: number;

/**
* SmartClient component used for the tabs of the tabBar. 
*  Must be Button or Button subclass.
* @type {Class}
* @default "ImgTab"
*/
buttonConstructor?: Class;

/**
* An array of shortcut keyboard commands which will close the currently selected tab, if
*  the currently selected tab is closeable. Either this TabBar or the currently
*  selected tab must have keyboard focus.
*  
*  By default, this is an array of two KeyIdentifiers: Alt+Delete,
*  which is the keyboard command recommended by
*  WAI-ARIA Authoring Practices,
*  and Ctrl+W.
*  Notes:
*  
*  
* - On Mac, the Alt+Delete keyboard command is accomplished via
*  Fn-Option-Delete.
*  
* - Alt+Delete is a
*  JAWS Keystroke
*  to "Say Active Cursor". If using JAWS, pressing Alt+Shift+Delete will close
*  the tab.
*  
* - In Chrome, Firefox, and Internet Explorer on Windows, Ctrl+W will also
*  close the browser tab/window if focus is not within a TabBar.
*  If Ctrl+W will be used frequently by the application's users, it may be useful to
*  
*  Page.registerKey to cancel it by default:
*  isc.Page.registerKey({ctrlKey: true, keyName: "W"}, "return false");
*  
*  
*  
* @type {Array<Partial<KeyIdentifier>>}
* @default "see below"
*/
closeTabKeys?: Array<Partial<KeyIdentifier>>;

/**
* Should tabs exceeding TabBar.moreTabCount be shown on a "more" tab?
*  
*  This setting is used to emulate an iPhone-style tab bar "more" button.
* @type {boolean}
* @default null
*/
showMoreTab?: boolean;

/**
* This property defines the number tab buttons that should be shown before
*  automatically adding a "more" button to handle the remaining tabs. This
*  property is only used when TabBar.showMoreTab is enabled.
* @type {Integer}
* @default "5"
*/
moreTabCount?: Integer;

/**
* Tab to show as the "more" tab when TabBar.showMoreTab is enabled and the number
*  of tabs to show exceeds TabBar.moreTabCount.
* @type {Tab}
* @default null
*/
moreTab?: Tab;

/**
* Thickness of the baseLine, in pixels. This should be set to match the media specified
*  by TabBar.baseLineSrc. The baseLineThickness also determines the degree of overlap with
*  the TabSet's paneContainer when using decorative edges - see TabSet.paneContainer
*  for details.
* @type {number}
* @default "1"
*/
baseLineThickness?: number;

/**
* Sets StretchImg.src for the baseLine StretchImg.
* @type {SCImgURL}
* @default "[SKIN]baseline.gif"
*/
baseLineSrc?: SCImgURL;

/**
* Set StretchImg.capSize for the baseLine stretchImg.
* @type {number}
* @default "2"
*/
baseLineCapSize?: number;

/**
* Default size (length) in pixels for tabs within this tabBar
* @type {number}
* @default "80"
*/
defaultTabSize?: number;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "TabBarEditProxy"
*/
editProxyConstructor?: SCClassName;



/**
*  Select a tab
* 
* @param {number} index of tab to select
*/
selectTab(tabNum:number): void;



}
/**
* Creates a new TabBar
* 
* @param typeCheckedProperties {Partial<TabBar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TabBar} The new instance
*/
class TabBar{
static create(typeCheckedProperties? : Partial<TabBar>, uncheckedProperties?: Object): TabBar;







}

/**
* Singleton class with static APIs for managing automatically assigned tab order for
*  SmartClient components and other focusable elements.
*  
*  The TabIndexManager separates the logic required to maintain a sensible tab-order for
*  a page's components from the logic to handle allocation of actual tab index values.
*  It is common to have non-focusable components with an implied position in the page's
*  tab order - for example Layouts containing focusable buttons, or DynamicForms containing
*  focusable items, and this class handles maintaining relative tab order within such
*  groups, and supplying explicit TabIndex values for the items which actually need them.
*  
*  Entries are registered with the TabIndexManager via the TabIndexManager.addTarget 
*  API. A 
*  numeric tab index for each entry will be lazily generated when requested via 
*  TabIndexManager.getTabIndex. The class provides APIs to modify the position of entries 
*  in the tab tree. When a target is registered, a couple of custom callback functions
*  can be provided. The first is a notification method for the tab index being updated
*  (due to, for example, a parent being repositioned and all its children having new
*  tab indices assigned), and can be used to take an appropriate action such as updating
*  the tab index of an element in the DOM. The second callback will be fired when a
*  call to the special TabIndexManager.focusInTarget or 
*  TabIndexManager.shiftFocus API requests focus be passed to an entry. This
*  allows a developer to take an appropriate action (such as programmatically focussing
*  in some DOM element).
*  
*  For standard SmartClient components (focusable Canvas 
*  and FormItem), developers will typically use APIs directly on the widget
*  to customize tab sequence behavior rather than interacting with the TabIndexManager
*  class. See the tabOrderOverview topic for more 
*  information on tab order management for components in SmartClient.
*  Developers wishing to embed focusable components into a page which are not
*  SmartClient components (native HTML elements and third party widgets), may use
*  TabIndexManager APIs to do so. This process is described in 
*  customTabElements.
*/
interface TabIndexManager extends Class {




}
/**
* Creates a new TabIndexManager
* 
* @param typeCheckedProperties {Partial<TabIndexManager>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TabIndexManager} The new instance
*/
class TabIndexManager{
static create(typeCheckedProperties? : Partial<TabIndexManager>, uncheckedProperties?: Object): TabIndexManager;




/**
*  Register a target to have its tab order position managed by the TabIndexManager.
* 
* @param {string} Unique ID to associate with a tab position. For a Canvas this  would typically be the Canvas.ID but any unique string is valid.
* @param {boolean} Is this target directly focusable? Governs whether an   explicit tabIndex will be created for this target. This parameter should be   passed as false for targets which do not require an explicit tabIndex   as they are not focusable, or not explicit tab-stops for the user tabbing through the   page. They will still have an implicit tab order position which   governs where descendants appear, and would be used to generate a tabIndex if   canFocus is subsequently updated via TabIndexManager.setCanFocus.
* @param {string=} For cases where the tab position should be treated part of a   group to be moved together, the ID of the parent target containing all members of this   group. An example of this would be a Layout managing the tab order of all its members.   If present, the passed parentID must already be being managed by this TabIndexManager.   May be updated for registered targets via TabIndexManager.moveTarget.
* @param {Integer=} Position in the tab-order within the specified parent [or  within top level widgets]. Omitting this parameter will add the target to the end of   the specified parent's tab group.   May be updated for registered targets via TabIndexManager.moveTarget.
* @param {TabIndexUpdatedCallback=} This notification method will   be fired when the tabIndex is actually updated, typically due to the target, or some   parent of it being re-positioned in the managed Tab order. In some cases tab indices   may also be updated to make space for unrelated entries being added to the   TabIndexManager. This notification is typically used to update the appropriate element   in the DOM to reflect a new tab index.
* @param {ShiftFocusCallback=} This notification method will be fired  when the special TabIndexManager.shiftFocus method is called to   programmatically move focus through the registered targets (simulating the user tabbing   through elements in the tab index chain). The implementation should attempt to update   the UI state by focusing in the appropriate UI for this target -- typically this means   putting browser focus into a DOM element, and return true to indicate success.   Returning false indicates the element is currently not focusable (disabled, masked, etc),   and cause the TabIndexManager to call the shiftFocusCallback on the next registered   entry (skipping over this entry).   If this method was not supplied, calls to TabIndexManager.shiftFocus will simply skip   this entry.
*/
static addTarget(ID:string, canFocus:boolean, parentID?:string, position?:Integer, tabIndexUpdatedCallback?:(ID:string) => void, shiftFocusCallback?:(ID:string) => void): void;

/**
*  Has the specified target been added to this TabIndexManager via
*  TabIndexManager.addTarget?
* 
* @param {string} Unique ID to test for.
*/
static hasTarget(ID:string): boolean;

/**
*  Move a target to the newly specified parent / position. This method may change the
*  calculated tab index for this entry, or other canFocus:true entries which already
*  have a calculated tabIndex. The registered tabIndexUpdated notification method will
*  for for any entry where this occurs.
* 
* 
* @param {string} ID of the target to move
* @param {string=} ID of the new parent (if null, will move to the top level)
* @param {Integer=} Position within the specified parent. If null will be the  last entry.
*/
static moveTarget(ID:string, parentID?:string, position?:Integer): void;

/**
*  Move a list of targets to the newly specified parent / position. This method may change the
*  calculated tab index for these entries, or other canFocus:true entries which already
*  have a calculated tabIndex. The registered tabIndexUpdated notification method will
*  for for any entry where this occurs.
* 
* 
* @param {Array<Partial<string>>} IDs of the targets to move
* @param {string=} ID of the new parent (if null, will move to the top level)
* @param {Integer=} Position within the specified parent. If null will be added at the end
*/
static moveTargets(IDs:Array<Partial<string>>, parentID?:string, position?:Integer): void;

/**
*  Temporarily suppress firing any tabIndexChanged callback passed into 
*  TabIndexManager.addTarget for the 
*  specified targets should their tab index change.
*  
*  This is useful for cases where a developer is managing a list of items and wants to
*  avoid any potential for multiple notifications until the entire list has been managed
* 
* @param {Array<Partial<string>>} targets for which callbacks should be suppressed
*/
static suppressCallbacks(targets:Array<Partial<string>>): void;

/**
*  Resume firing any callbacks suppressed by TabIndexManager.suppressCallbacks
* 
* @param {Array<Partial<string>>} targets for which callbacks should be resumed
*/
static resumeCallbacks(targets:Array<Partial<string>>): void;

/**
*  Removes a target from this TabIndexManager.
*  Any children of this target will also be removed - developers wishing to preserve
*  children should first call TabIndexManager.moveTarget to move the children out of this parent
* 
* @param {string} target to remove
*/
static removeTarget(ID:string): void;

/**
*  Modifies whether or not some specified target should be treated as focusable and
*  provide a meaningful TabIndex on a call to TabIndexManager.getTabIndex.
* 
* @param {string} target ID
* @param {boolean} new value for canFocus
*/
static setCanFocus(ID:string, canFocus:boolean): void;

/**
*  Returns a tabIndex number for some target ID registered via TabIndexManager.addTarget.
*  Generated tab indices are guaranteed to be in order.
*  
*  As targets are added to, or moved within the TabIndexManager, their tab index may become invalid.
*  The tabIndexUpdated notification will be fired when this occurs, giving developers
*  a way to pick up the new tab index, and assign it to the appropriate DOM element if appropriate.
* 
* @param {string} ID of the target for which you want to get a numeric tabIndex.
*/
static getTabIndex(ID:string): Integer;

/**
*  Request the TabIndexManager shift focus to a registered focus target.
*  
*  This method does not directly change the focus within the DOM - instead it invokes the 
*  shiftFocusCallback registered for the specified target if it is marked as
*  canFocus:true.
*  
*  Returns false if the target had no no shiftFocusCallback, 
*  the shiftFocusCallback returned false, or if the target is marked
*  as not canFocus:true
* 
* @param {string} target to shift focus to
*/
static focusInTarget(ID:string): boolean;

/**
*  Method to shift focus to the next registered focusable target.
*  
*  This method does not directly change the focus within the DOM - instead it finds the 
*  next target marked as canFocus:true, and invokes the 
*  shiftFocusCallback registered for that target.
*  This callback is expected to take the appropriate action (typically shifting native
*  focus to an element in the DOM), and return true (or return false, if the target
*  could not receieve focus for some reason, in which case we'll find the next
*  canFocus:true target and repeat the action there.
*  
*  Targets with no shiftFocusCallback will be skipped entirely in this process.
*   
* 
* @param {string} current focus target. If null, focus will be applied to the first  focusable target (or the last if the forward parameter is false).
* @param {boolean} should focus move forward to the next focusable target, or  backward to the previous focusable target.
*/
static shiftFocus(ID:string, forward:boolean): boolean;

/**
*  Method to shift focus to the next registered focusable target within some group.
*  This method will move focus forward or backward, considering only the specified
*  target and any targets within its group (registered as children of the target via
*  TabIndexManager.addTarget or TabIndexManager.moveTarget).
*  
*  The second parameter can be passed to specify an explicit starting position to
*  shift focus from. If this is not present, this method will attempt to focus into the
*  group target itself if moving forward (or its last child, if moving backward) and
*  failing that, shift focus from there.
*  
*  This method does not directly change the focus within the DOM - instead it finds the 
*  next target marked as canFocus:true, and invokes the 
*  shiftFocusCallback registered for that target.
*  This callback is expected to take the appropriate action (typically shifting native
*  focus to an element in the DOM), and return true (or return false, if the target
*  could not receieve focus for some reason, in which case we'll find the next
*  canFocus:true target and repeat the action there.
*  
*  Targets with no shiftFocusCallback will be skipped entirely in this process.
*  
*  A return value of false indicates that this method was unable to shift focus to a new 
*  target.
*   
* 
* @param {string} ID of registered target. Focus will be shifted within  this target and its descendants only.
* @param {string} Optional ID of current focus target within the group  focus will be shifted in the specified direction from this node.
* @param {boolean} should focus move forward to the next focusable target, or  backward to the previous focusable target.
*/
static shiftFocusWithinGroup(targetGroup:string, currentTarget:string, forward:boolean): boolean;

/**
*  Method to shift focus to the next registered focusable target beyond some registered
*  target and any targets registered as children within its group via 
*  TabIndexManager.addTarget or TabIndexManager.moveTarget.
*  
*  This method does not directly change the focus within the DOM - instead it finds the 
*  next target marked as canFocus:true, and invokes the 
*  shiftFocusCallback registered for that target.
*  This callback is expected to take the appropriate action (typically shifting native
*  focus to an element in the DOM), and return true (or return false, if the target
*  could not receieve focus for some reason, in which case we'll find the next
*  canFocus:true target and repeat the action there.
*  
*  Targets with no shiftFocusCallback will be skipped entirely in this process.
*  
*  A return value of false indicates that this method was unable to shift focus to a new 
*  target.
*   
* 
* @param {string} ID of registered target. Focus will be shifted to the  next registered focusable element, skipping this group and its descendants.
* @param {boolean} should focus move forward to the next focusable target, or  backward to the previous focusable target.
*/
static shiftFocusAfterGroup(targetGroup:string, forward:boolean): boolean;

/**
*  Should focus navigation be achieved by explicitly calling the TabIndexManager
*  TabIndexManager.shiftFocus method for the specified node?
*  
*  Developers integrating custom focusable element's into a SmartClient based application 
*  can use this method to ensure the elements in question interact correctly with 
*  Canvas.showClickMask and ListGrid.canEdit.
*  See the tabOrderOverview topic for more information on
*  integrating custom focusable UI into a SmartClient application.
*  
*  This method will return true if the TabIndexManager.setAlwaysUseExplicitFocusNavigation
*  has been set to true (typically because a Canvas.showClickMask is
*  showing), or if the entry or some ancestor has been marked as 
*  TabIndexManager.setUseExplicitFocusNavigation.
*  Note that this is the case for entries registered under a canvas
*  with Canvas.alwaysManageFocusNavigation set to true.
* 
* 
* @param {string} TabIndexManager registered target ID
*/
static useExplicitFocusNavigation(ID:string): boolean;

/**
*  Should TabIndexManager.useExplicitFocusNavigation to always return true?
* 
* @param {boolean} whether we should always use explicit focus navigation
*/
static setAlwaysUseExplicitFocusNavigation(newValue:boolean): void;

/**
*  Mark the specified node (and its descendents) as using explicit focus navigation
*  rather than relying on native browser Tab event handling behavior. See
*  TabIndexManager.useExplicitFocusNavigation for more information.
* 
* @param {string} registered TabIndexManager target
* @param {boolean} should explicit focus navigation be used for the specified  target and its descendents
*/
static setUseExplicitFocusNavigation(ID:string, newValue:boolean): void;

/**
*  Show the current hierarchy of targets passed to TabIndexManager.addTarget together with
*  current canFocus state and tabIndex (if assigned). Results are output to the developer console.
* 
*/
static showAllocatedTabChain(): void;

/**
*  Get a report of the current hierarchy of targets passed to TabIndexManager.addTarget
*  together with current canFocus state and tabIndex (if assigned).
* 
*/
static getAllocatedTabChain(): string;




}

/**
* The TabSet class allows components on several panes to share the same space. The tabs at 
*  the top can be selected by the user to show each pane. 
*  
*  Tabs are configured via the tabs property, each of which has a
*  pane property which will be displayed in the main pane when that tab is
*  selected.
*/
interface TabSet extends Canvas {
/**
* An array of Tab objects, specifying the title and pane contents of each tab in the TabSet.
*  When developing in JavaScript, tabs are specified as an array of object literals, 
*  not instances - see Tab. 
*  
*  
*  After providing Tab instances to setTabs(), the TabSet creates actual UI 
*  widgets to serve as interactive tabs. Any further modifications to tabs should be performed 
*  via TabSet APIs such as TabSet.setTabTitle, TabSet.setTabIcon and 
*  TabSet.setTabPane.
*  
*  You can add and remove tabs after creating the TabSet by calling TabSet.addTab and 
*  TabSet.removeTab
* @type {Array<Partial<Tab>>}
* @default null
*/
tabs?: Array<Partial<Tab>>;

/**
* TabBar for this TabSet, an instance of TabBar.
* @type {TabBar}
* @default null
*/
tabBar?: TabBar;

/**
* Properties to apply to all Tabs created by this TabSet.
* @type {Partial<Tab>}
* @default null
*/
tabProperties?: Partial<Tab>;

/**
* If set, is passed as "width" to all tabs when TabSet.tabBarPosition is set to 
*  "top" or "bottom".
*  
*  If unset, width will be picked up from
*  the Tab constructor class defaults. Tabs expand to fit their content, so 
*  this width acts as a minimum.
*  Setting width:1 will result in tabs that are
*  only as wide as their titles. May be customized by individual
*  skinning.
* @type {number}
* @default null
*/
defaultTabWidth?: number;

/**
* If set, is passed as "height" to all tabs when TabSet.tabBarPosition is set to 
*  "left" or "right". 
*  
*  If unset, height will be picked up from
*  the Tab constructor class defaults. Note that tabs expand to fit their content so
*  this height acts as a minimum. May be customized by individual
*  skinning.
* @type {number}
* @default null
*/
defaultTabHeight?: number;

/**
* Should we use simple button based tabs styled with CSS rather than
*  image based ImgTab tabs?
*  
*  
*  If set to true the TabSet.simpleTabButtonConstructor will be used and tabs will
*  by styled according to TabSet.simpleTabBaseStyle.
*  
*  
* @type {boolean}
* @default false
*/
useSimpleTabs?: boolean;

/**
* If TabSet.useSimpleTabs is true, simpleTabBaseStyle will be the base style
*  used to determine the css style to apply to the tabs.
*  This property will be suffixed with the side on which the tab-bar will appear, followed
*  by with the tab's state (selected, over, etc), resolving to a className like 
*  "tabButtonTopOver".
* @type {CSSStyleName}
* @default "tabButton"
*/
simpleTabBaseStyle?: CSSStyleName;

/**
* If TabSet.useSimpleTabs is true, simpleTabIconOnlyBaseStyle will be the
*  base style used to determine the css style to apply to the tabs if
*  Tab.canAdaptWidth is set and the title is not being shown.
*  
*  This property will be suffixed with the side on which the tab-bar will appear, followed
*  by with the tab's state (selected, over, etc), resolving to a className like 
*  "iconOnlyTabButtonTopOver".
*  
*  Note that this property is only defined for certain skins, where it's needed. If not
*  defined, TabSet.simpleTabBaseStyle will serve as base style whether or not the title is
*  hidden.
* @type {CSSStyleName}
* @default "varies"
*/
simpleTabIconOnlyBaseStyle?: CSSStyleName;

/**
* Which side of the TabSet the TabBar should appear on.
* @type {Side}
* @default "top"
*/
tabBarPosition?: Side;

/**
* Alignment of the tabBar.
*  
*  If the TabSet.tabBarPosition is "top" or "bottom", then 
*  this attribute may be set to "left", "right" or "center". The default is "left", or
*  "right" in Page.isRTL.
*  
*  If the TabSet.tabBarPosition is "left" or "right", then this
*  attribute may be set to "top", "bottom" or "center". The default is "top".
* @type {Side | Alignment}
* @default "see below"
*/
tabBarAlign?: Side | Alignment;

/**
* Thickness of tabBar, applies to either orientation (specifies height for horizontal,
*  width for vertical orientation). Note that overriding this value for TabSets that are
*  skinned with images generally means providing new media for the borders.
* @type {number}
* @default "21"
*/
tabBarThickness?: number;

/**
* Specifies the index of the initially selected tab.
* @type {Tab | Integer}
* @default "0"
*/
selectedTab?: Tab | Integer;

/**
* Should tabs in this tabSet show an icon allowing the user to dismiss the tab by
*  clicking on it directly. May be overridden for individual tabs by setting 
*  Tab.canClose.
*  
*  The URL for this icon's image will be derived from TabSet.closeTabIcon by 
*  default, but may be overridden by explicitly specifying Tab.closeIcon.
*  
*  Note: Currently, tabs can only show a single icon, so a closable tab will show
*  the close icon only even if Tab.icon is set. To work around this, add the icon
*  as an HTML &lt;img&gt; tag to the Tab.title property, for example:
*  
*  
*   title : "&lt;span&gt;" + isc.Canvas.imgHTML("path/to/icon.png") + " Tab Title&lt;/span&gt;"
*  
*  
*  
* @type {boolean}
* @default null
*/
canCloseTabs?: boolean;

/**
* Default src for the close icon for tabs to display if TabSet.canCloseTabs is true.
* @type {SCImgURL}
* @default "[SKIN]/TabSet/close.png"
*/
closeTabIcon?: SCImgURL;

/**
* Size in pixels of the icon for closing tabs, displayed when TabSet.canCloseTabs is true.
* @type {Integer}
* @default "16"
*/
closeTabIconSize?: Integer;

/**
* When isc.setScreenReaderMode is enabled and a tab is
*  TabSet.canCloseTabs, the ariaCloseableSuffix is a string
*  that is appended to the label of closeable tabs. This suffix is hidden from sighted
*  users, but is announced by screen readers to indicate that the tab may be closed.
*  
*  Set to null to disable appending this suffix.
* @type {string}
* @default ", closeable"
*/
ariaCloseableSuffix?: string;

/**
* If true, tabs can be reordered by dragging on them.
*  
*  To disallow drag-reorder of a specific tab, see Tab.canReorder.
* @type {boolean}
* @default null
*/
canReorderTabs?: boolean;

/**
* Should tabs exceeding TabSet.moreTabCount be shown on a "more" tab?
*  
*  This setting is used to emulate an iPhone-style tab bar "more" button.
* @type {boolean}
* @default null
*/
showMoreTab?: boolean;

/**
* This property defines the number tab buttons that should be shown before
*  automatically adding a "more" button to handle the remaining tabs. This
*  property is only used when TabSet.showMoreTab is enabled.
* @type {number}
* @default "5"
*/
moreTabCount?: number;

/**
* Title for the "More" tab.
* @type {string}
* @default "More"
*/
moreTabTitle?: string;

/**
* If TabSet.showMoreTab is enabled this property determines the image to display on
*  the "More" tab button.
* @type {SCImgURL}
* @default "[SKINIMG]/iOS/more.png"
*/
moreTabImage?: SCImgURL;

/**
* Tab to be shown when TabSet.showMoreTab is enabled
*  more than TabSet.moreTabCount tabs are provided.
* @type {Tab}
* @default null
*/
moreTab?: Tab;

/**
* Properties to apply to the "more" tab created by this TabSet.
* @type {Partial<Tab>}
* @default null
*/
moreTabProperties?: Partial<Tab>;

/**
* Pane contents for the "more" tab based on a VLayout. Typically contains
*  a NavigationBar and TableView.
* @type {VLayout}
* @default null
*/
moreTabPane?: VLayout;

/**
* Properties to apply to the "more" tab's pane created by this TabSet.
* @type {Partial<Canvas>}
* @default null
*/
moreTabPaneProperties?: Partial<Canvas>;

/**
* Default properties for the "more" tab's pane.
*  
*  Currently constructs a VLayout with a NavigationBar and TableView.
* @type {Partial<Canvas>}
* @default null
*/
moreTabPaneDefaults?: Partial<Canvas>;

/**
* Navigation bar shown in the TabSet.moreTabPane;
* @type {NavigationBar}
* @default null
*/
moreTabPaneNavBar?: NavigationBar;

/**
* TableView used to show links to other tabs in the TabSet.moreTabPane;
* @type {TableView}
* @default null
*/
moreTabPaneTable?: TableView;

/**
* This property determines what controls should show up after the tabBar for this TabSet.
*  Standard controls can be included using the strings "tabScroller" and 
*  "tabPicker". These correspond to the TabSet.scroller and TabSet.tabPicker
*  AutoChildren, respectively. The "tabScroller" standard control shows two
*  buttons for scrolling through the tabs in order and the "tabPicker" standard
*  control allows tabs to be picked directly from a menu. The standard controls show up only if
*  TabSet.showTabScroller or TabSet.showTabPicker is true and there is not
*  enough space available to show all of the tabs in the tabBar.
*  
*  ${isc.DocUtils.linkForExampleId('layout_tabs_custom_controls', 'This sample')} illustrates the usage of this property
*  
*  Additional controls can be included by adding any widget to this array. Controls will
*  show up in the order in which they are specified. For example, the following code would
*  add a button in the tabBar area, while preserving the normal behavior of the tabScroller
*  and tabPicker:
*  
*  
*  isc.TabSet.create({
*    width:300,
*    tabs : [
*      { title: "Tab one" }
*    ],
*    tabBarControls : [
*      isc.ImgButton.create({
*        src:"[SKINIMG]/actions/add.png",
*        width:16, height:16,
*        layoutAlign:"center"
*      }),
*      "tabScroller", "tabPicker"
*    ]
*  });
*  
*  
*  
*  You can also refer to the default tabPicker/tabScroll controls
*  from Component XML:
*  
*  &lt;TabSet width="300"&gt;
*    &lt;tabBarControls&gt;
*      &lt;Button title="Custom Button"/&gt;
*      &lt;value xsi:type="string"&gt;tabPicker&lt;/value&gt;
*      &lt;value xsi:type="string"&gt;tabScroller&lt;/value&gt;
*    &lt;/tabBarControls&gt;
*    &lt;tabs&gt;
*      &lt;tab title="Foo"/&gt;
*      &lt;tab title="Bar"/&gt;
*    &lt;/tabs&gt;
*  &lt;/TabSet&gt;
*  
*  
*  When Browser.isTouch is true and native touch scrolling is supported,
*  then by default, only the "tabPicker" is shown. The "tabScroller"
*  control is omitted by default on touch devices because the tabs in the tab bar are native
*  touch-scrollable, so the "tabScroller" control is unnecessary. To override
*  the omission of the "tabScroller", simply add
*  "tabScroller"
*  
*  to the tabBarControls array.
*  
*  Note: Due to tabs supporting Tab.canAdaptWidth and other
*  complexities of TabSet widget layout, Canvas.width controls
*  (including LayoutSpacer) aren't supported in tabBarControls.
*  However, if you take into account the width of your tabs and whether the TabSet.tabPicker and TabSet.scroller are present, you can add a fixed-width spacer to achieve the
*  desired appearance, as long as the set of tabs and TabSet width are static.
* @type {Array<any>}
* @default ""tabScroller"|"tabPicker"|Canvas"
*/
tabBarControls?: Array<any>;

/**
* If there is not enough space to display all the tab-buttons in this tabSet, should 
*  scroll buttons be displayed to allow access to tabs that are clipped? If unset, 
*  defaults to false for Browser.isHandset and true otherwise.
* @type {boolean}
* @default null
*/
showTabScroller?: boolean;

/**
* If there is not enough space to display all the tab-buttons in this tabSet, should
*  a drop-down "picker" be displayed to allow selection of tabs that are clipped?
* @type {boolean}
* @default true
*/
showTabPicker?: boolean;

/**
* AutoChild of type Layout that holds the TabSet.tabBarControls as well as
*  the built-in controls such as the TabSet.showTabPicker.
* @type {Layout}
* @default null
*/
tabBarControlLayout?: Layout;

/**
* If TabSet.showTabScroller is true, should tabs be scrolled into view via an 
*  animation when the user interacts with the scroller buttons?
* @type {boolean}
* @default true
*/
animateTabScrolling?: boolean;

/**
* A component containing back and forward buttons for scrolling through all of the tabs
*  of the TabSet. The scroller is created automatically when needed and when "tabScroller"
*  is specified in the TabSet.tabBarControls.
*  
*  By default, the scroller constructor is StretchImgButton. Note that the scroller
*  StretchImg.items are determined automatically, so any items set in
*  scrollerProperties will be ignored.
* @type {StretchImgButton}
* @default null
*/
scroller?: StretchImgButton;

/**
* If TabSet.showTabScroller is true, this property governs the size of scroller
*  buttons. Applied as the width of buttons if the tabBar is horizontal, or the height
*  if tabBar is vertical. Note that the other dimension is determined by 
*  TabSet.tabBarThickness
* @type {number}
* @default "16"
*/
scrollerButtonSize?: number;

/**
* A button control that allows tabs to be picked directly from a popup menu. The tabPicker
*  is created automatically when needed and when "tabPicker" is specified in
*  the TabSet.tabBarControls.
* @type {ImgButton}
* @default null
*/
tabPicker?: ImgButton;

/**
* If TabSet.showTabPicker is true and Browser.isTouch
*  is false, this property governs the size of the tab picker button. This value
*  is applied as the width of the tab picker button if the TabSet.tabBar is
*  horizontal, or the height if the tabBar is vertical. Note that the other
*  dimension is determined by TabSet.tabBarThickness.
*  
*  On touch browsers (where Browser.isTouch is true),
*  TabSet.touchPickerButtonSize is used instead.
* @type {Integer}
* @default "16"
*/
pickerButtonSize?: Integer;

/**
* The size of the tab picker button when Browser.isTouch is true.
* @type {Integer}
* @default "16"
*/
touchPickerButtonSize?: Integer;

/**
* Default directory for skin images (those defined by the class), relative to the
*  Page-wide Page.getSkinDir.
* @type {SCImgURL}
* @default "images/TabSet/"
*/
skinImgDir?: SCImgURL;

/**
* If this TabSet is showing TabSet.showTabScroller, this property 
*  determines whether the TabSet.scrollerHSrc and TabSet.scrollerVSrc media
*  will be used for vertical and horizontal tab-bar scroller buttons, or whether separate
*  media should be used for each possible TabSet.tabBarPosition based
*  on the TabSet.scrollerSrc property for this tabSet.
* @type {boolean}
* @default true
*/
symmetricScroller?: boolean;

/**
* If this TabSet is showing TabSet.showTabScroller, and 
*  TabSet.symmetricScroller is false, this property governs the base
*  URL for the tab bar back and forward scroller button images.
*  
*  Note that if TabSet.symmetricScroller is true, 
*  TabSet.scrollerHSrc and TabSet.scrollerVSrc will be used instead.
*  
*  To get the path to the image to display, this base URL will be modified as follows:
*  
*  
* - If appropriate a state suffix of "Down" or "Disabled" will be
*    appended.
*  
* - The TabSet.tabBarPosition for this tabSet will be appended.
*  
* - A suffix of "forward" or "back" will be appended for the
*    forward or backward scrolling button.
*  
*  For example - if the scrollerSrc is set to "[SKIN]scroll.gif", the image
*  displayed for the back-scroller button on a tabSet with tabBarPosition set to
*  "top" and symmetricScroller set to false would be one of 
*  "[SKIN]scroll_top_back.gif", "[SKIN]scroll_Down_top_back.gif",
*  and "[SKIN]scroll_Disabled_top_back.gif".
*  
*  Note that for best results the media should be sized to match the scroller button sizes, 
*  determined by TabSet.tabBarThickness and TabSet.scrollerButtonSize.
* @type {SCImgURL}
* @default "[SKIN]/scroll.gif"
*/
scrollerSrc?: SCImgURL;

/**
* If this TabSet is showing TabSet.showTabScroller, and 
*  TabSet.symmetricScroller is true, this property governs the base
*  URL for the tab bar back and forward scroller button images for horizontal tab bars [IE for
*  tab sets with TabSet.tabBarPosition set to "top" or "bottom"].
*  
*  Note that if TabSet.symmetricScroller is false, 
*  TabSet.scrollerSrc will be used instead.
*  
*  To get the path to the image to display, this base URL will be modified as follows:
*  
*  
* - If appropriate a state suffix of "Down" or "Disabled" will be
*    appended.
*  
* - A suffix of "forward" or "back" will be appended for the
*    forward or backward scrolling button.
*  
*  For example - if the scrollerHSrc is set to "[SKIN]hscroll.gif", the image
*  displayed for the back-scroller button on a tabSet with tabBarPosition set to
*  "top" and symmetricScroller set to true would be one of 
*  "[SKIN]hscroll_back.gif", "[SKIN]hscroll_Down_back.gif",
*  and "[SKIN]hscroll_Disabled_back.gif".
*  
*  Note that for best results the media should be sized to match the scroller button sizes, 
*  determined by TabSet.tabBarThickness and TabSet.scrollerButtonSize.
* @type {SCImgURL}
* @default "[SKIN]hscroll.gif"
*/
scrollerHSrc?: SCImgURL;

/**
* If this TabSet is showing TabSet.showTabScroller, and 
*  TabSet.symmetricScroller is true, this property governs the base
*  URL for the tab bar back and forward scroller button images for vertical tab bars [IE for
*  tab sets with TabSet.tabBarPosition set to "left" or "right"].
*  
*  Note that if TabSet.symmetricScroller is false, 
*  TabSet.scrollerSrc will be used instead.
*  
*  To get the path to the image to display, this base URL will be modified as follows:
*  
*  
* - If appropriate a state suffix of "Down" or "Disabled" will be
*    appended.
*  
* - A suffix of "forward" or "back" will be appended for the
*    forward or backward scrolling button.
*  
*  For example - if the scrollerVSrc is set to "[SKIN]vscroll.gif", the image
*  displayed for the back-scroller button on a tabSet with tabBarPosition set to
*  "left" and symmetricScroller set to true would be one of 
*  "[SKIN]vscroll_back.gif", "[SKIN]vscroll_Down_back.gif",
*  and "[SKIN]vscroll_Disabled_back.gif".
*  
*  Note that for best results the media should be sized to match the scroller button sizes, 
*  determined by TabSet.tabBarThickness and TabSet.scrollerButtonSize.
* @type {SCImgURL}
* @default "[SKIN]vscroll.gif"
*/
scrollerVSrc?: SCImgURL;

/**
* If this TabSet is showing a TabSet.showTabPicker, this
*  property determines whether the TabSet.pickerButtonHSrc and
*  TabSet.pickerButtonVSrc media will be used for vertical and horizontal tab-bar
*  picker buttons, or whether separate media should be used for each possible 
*  TabSet.tabBarPosition based on the TabSet.pickerButtonSrc
*  property for this tabSet.
* @type {boolean}
* @default true
*/
symmetricPickerButton?: boolean;

/**
* If TabSet.showTabPicker is true, this property governs the base URL for the picker
*  button image, when TabSet.symmetricPickerButton is set to false
*  
*  Note that if symmetricPickerButton is true, the TabSet.pickerButtonHSrc 
*  and TabSet.pickerButtonVSrc properties will be used instead.
*  
*  To get the path to the image to display, this base URL will be modified as follows:
*  
*  
* - If appropriate a state suffix of "Down" or "Disabled" will be
*    appended.
*  
* - The TabSet.tabBarPosition for this tabSet will be appended.
*  
* @type {SCImgURL}
* @default "[SKIN]/picker.gif"
*/
pickerButtonSrc?: SCImgURL;

/**
* If TabSet.showTabPicker is true, and TabSet.symmetricPickerButton is 
*  set to true, this property governs the base URL for the picker
*  button image, when displayed in a horizontal tab-bar [IE TabSet.tabBarPosition is
*  set to "top" or "bottom"].
*  
*  Note that if symmetricPickerButton is false, the TabSet.pickerButtonSrc
*  property will be used instead.
*  
*  This base URL will have a suffix of "Down" appended when the user holds the
*  mouse down over the button, and "Disabled" if the tabset as a whole is 
*  disabled.
* @type {SCImgURL}
* @default "[SKIN]hpicker.gif"
*/
pickerButtonHSrc?: SCImgURL;

/**
* If TabSet.showTabPicker is true, and TabSet.symmetricPickerButton is 
*  set to true, this property governs the base URL for the picker
*  button image, when displayed in a verricaL tab-bar [IE TabSet.tabBarPosition is
*  set to "LEFT" or "right"].
*  
*  Note that if symmetricPickerButton is false, the TabSet.pickerButtonSrc
*  property will be used instead.
*  
*  This base URL will have a suffix of "Down" appended when the user holds the
*  mouse down over the button, and "Disabled" if the tabset as a whole is 
*  disabled.
* @type {SCImgURL}
* @default "[SKIN]vpicker.gif"
*/
pickerButtonVSrc?: SCImgURL;

/**
* Container where the component specified by Tab.pane is shown.
*  
*  Note: paneContainer and showEdges:true for rounded tabsets: you can enable decorative
*  image-based edges on the paneContainer by setting Canvas.showEdges
*  via paneContainerDefaults (to skin all tabsets) or paneContainerProperties (to use
*  edges on one instance). In this structure, the baseLine should use media
*  that matches the appearance of the decorative edges and fully overlaps the edge of the
*  paneContainer that it is adjacent to. In the most typical appearance (symmetric edges
*  on all 4 sides), both TabBar.baseLineCapSize and TabBar.baseLineThickness
*  match the Canvas.edgeSize set on the paneContainer. See the
*  load_skin.js file for the "SmartClient" skin for an example of setting all relevant
*  properties.
*  
*  To disable edges for a particular TabSet, which you may want to do for a TabSet that
*  is already within a clearly defined container, configure the paneContainer to show only
*  it's top edge:
*  
*    paneContainerProperties : { customEdges:["T"] },
*  
*  To completely flatten even the top edge of the TabSet:
*  
*    paneContainerProperties : { customEdges:["T"] },
*    tabBarProperties :{ baseLineCapSize:0 },
*  
*  This "flattens" the baseLine so that only the center image is used.
* @type {VLayout}
* @default null
*/
paneContainer?: VLayout;

/**
* CSS style used for the paneContainer.
* @type {CSSStyleName}
* @default null
*/
paneContainerClassName?: CSSStyleName;

/**
* Specifies the overflow of the pane container (the component that holds the pane contents
*  for all tabs). By default this is set to "auto", meaning the pane container will
*  automatically introduce scrolling when the pane contents exceed the TabSet's specified
*  size.
*  
*  For other values and their meaning, see Overflow
* @type {Overflow}
* @default "auto"
*/
paneContainerOverflow?: Overflow;

/**
* If this tabSet will TabSet.showPaneContainerEdges for the paneContainer,
*  this property determines whether the same edge media will be used regardless of the tab
*  bar position, or whether different media should be used (necessary if the edge appearance is
*  not symmetrical on all sides).
*  
*  If this property is set to false the paneContainer edge image URLs will be prefixed with
*  the tabBarPosition of the tabSet - for example "[SKIN]edge_top_T.gif" rather
*  than just "[SKIN]edge_T.gif".
*  
*  When symmetricEdges is false, custom edge sizes for the pane container may be
*  specified via TabSet.topEdgeSizes et al, and custom edge offsets via 
*  TabSet.topEdgeOffsets et al.
* @type {boolean}
* @default true
*/
symmetricEdges?: boolean;

/**
* If this tabSet will TabSet.showPaneContainerEdges for the paneContainer,
*  and TabSet.symmetricEdges is set to false, the leftEdgeSizes, 
*  rightEdgeSizes, topEdgeSizes and bottomEdgeSizes 
*  properties allow the sizes of edges for the paneContainer to be customized depending on
*  the TabSet.tabBarPosition.
*  
*  The attribute should be specified an EdgeSizes, specifying the
*  desired edge sizes where for the appropriate TabSet.tabBarPosition.
* @type {EdgeSizes}
* @default null
*/
leftEdgeSizes?: EdgeSizes;

/**
* If this tabSet will TabSet.showPaneContainerEdges for the paneContainer,
*  and TabSet.symmetricEdges is set to false, the leftEdgeSizes, 
*  rightEdgeSizes, topEdgeSizes and bottomEdgeSizes 
*  properties allow the sizes of edges for the paneContainer to be customized depending on
*  the TabSet.tabBarPosition.
*  
*  The attribute should be specified an EdgeSizes, specifying the
*  desired edge sizes where for the appropriate TabSet.tabBarPosition.
* @type {EdgeSizes}
* @default null
*/
topEdgeSizes?: EdgeSizes;

/**
* If this tabSet will TabSet.showPaneContainerEdges for the paneContainer,
*  and TabSet.symmetricEdges is set to false, the leftEdgeSizes, 
*  rightEdgeSizes, topEdgeSizes and bottomEdgeSizes 
*  properties allow the sizes of edges for the paneContainer to be customized depending on
*  the TabSet.tabBarPosition.
*  
*  The attribute should be specified an EdgeSizes, specifying the
*  desired edge sizes where for the appropriate TabSet.tabBarPosition.
* @type {EdgeSizes}
* @default null
*/
bottomEdgeSizes?: EdgeSizes;

/**
* If this tabSet will TabSet.showPaneContainerEdges for the paneContainer,
*  and TabSet.symmetricEdges is set to false, the leftEdgeSizes, 
*  rightEdgeSizes, topEdgeSizes and bottomEdgeSizes 
*  properties allow the sizes of edges for the paneContainer to be customized depending on
*  the TabSet.tabBarPosition.
*  
*  The attribute should be specified an EdgeSizes, specifying the
*  desired edge sizes where for the appropriate TabSet.tabBarPosition.
* @type {EdgeSizes}
* @default null
*/
rightEdgeSizes?: EdgeSizes;

/**
* If this tabSet will TabSet.showPaneContainerEdges for the paneContainer,
*  and TabSet.symmetricEdges is set to false, the leftEdgeOffsets, 
*  rightEdgeOffsets, topEdgeOffsets and bottomEdgeOffsets 
*  properties allow the offsets of edges for the paneContainer to be customized depending on
*  the TabSet.tabBarPosition.
*  
*  The attribute should be specified an EdgeSizes, specifying the
*  desired edge offsets where for the appropriate TabSet.tabBarPosition.
* @type {EdgeSizes}
* @default null
*/
leftEdgeOffsets?: EdgeSizes;

/**
* If this tabSet will TabSet.showPaneContainerEdges for the paneContainer,
*  and TabSet.symmetricEdges is set to false, the leftEdgeOffsets, 
*  rightEdgeOffsets, topEdgeOffsets and bottomEdgeOffsets 
*  properties allow the offsets of edges for the paneContainer to be customized depending on
*  the TabSet.tabBarPosition.
*  
*  The attribute should be specified an EdgeSizes, specifying the
*  desired edge offsets where for the appropriate TabSet.tabBarPosition.
* @type {EdgeSizes}
* @default null
*/
rightEdgeOffsets?: EdgeSizes;

/**
* If this tabSet will TabSet.showPaneContainerEdges for the paneContainer,
*  and TabSet.symmetricEdges is set to false, the leftEdgeOffsets, 
*  rightEdgeOffsets, topEdgeOffsets and bottomEdgeOffsets 
*  properties allow the offsets of edges for the paneContainer to be customized depending on
*  the TabSet.tabBarPosition.
*  
*  The attribute should be specified an EdgeSizes, specifying the
*  desired edge offsets where for the appropriate TabSet.tabBarPosition.
* @type {EdgeSizes}
* @default null
*/
topEdgeOffsets?: EdgeSizes;

/**
* If this tabSet will TabSet.showPaneContainerEdges for the paneContainer,
*  and TabSet.symmetricEdges is set to false, the leftEdgeOffsets, 
*  rightEdgeOffsets, topEdgeOffsets and bottomEdgeOffsets 
*  properties allow the offsets of edges for the paneContainer to be customized depending on
*  the TabSet.tabBarPosition.
*  
*  The attribute should be specified an EdgeSizes, specifying the
*  desired edge offsets where for the appropriate TabSet.tabBarPosition.
* @type {EdgeSizes}
* @default null
*/
bottomEdgeOffsets?: EdgeSizes;

/**
* Should the paneContainer for this tabset show Canvas.showEdges.
* @type {boolean}
* @default null
*/
showPaneContainerEdges?: boolean;

/**
* Space to leave around the panes in our paneContainer
*  
*  Note that this property may be specified on a per-tab basis via Tab.paneMargin.
* @type {Integer}
* @default "0"
*/
paneMargin?: Integer;

/**
* If true, users can edit the titles of tabs in this TabSet when the 
*  TabSet.titleEditEvent fires. You can override this behavior per tab 
*  with the Tab.canEditTitle property.
*  
*  Note that this TabSet's TabSet.titleEditEvent must be set to a
*  supported TabTitleEditEvent in order for users to be able to edit the titles of
*  tabs.
* @type {boolean}
* @default false
*/
canEditTabTitles?: boolean;

/**
* The event that triggers title editing on this TabSet.
* @type {TabTitleEditEvent}
* @default null
*/
titleEditEvent?: TabTitleEditEvent;

/**
* TextItem we use to edit tab titles in this TabSet. You can override this property 
*  using the normal AutoChild facilities.
* @type {TextItem}
* @default null
*/
titleEditor?: TextItem;

/**
* Properties for the auto-generated TabSet.titleEditor. This is the text item
*  we use to edit tab titles in this tabSet.
* @type {Partial<TextItem>}
* @default null
*/
titleEditorProperties?: Partial<TextItem>;

/**
* If set, offsets the tab title editor further in from the left-hand edge of the tab, by
*  the number of pixels set in this property. Note that the editor is always offset to
*  avoid overlapping the endcaps of the tab; this property is applied on top of that 
*  default offset.
* @type {Integer}
* @default null
*/
titleEditorLeftOffset?: Integer;

/**
* If set, offsets the tab title editor further in from the right-hand edge of the tab, by
*  the number of pixels set in this property. Note that the editor is always offset to
*  avoid overlapping the endcaps of the tab; this property is applied on top of that 
*  default offset.
* @type {Integer}
* @default null
*/
titleEditorRightOffset?: Integer;

/**
* If set, offsets the tab title editor further down from the top edge of the tab, by the
*  number of pixels set in this property. You can use this property, together with the 
*  left and right offset properties, to fine tune positioning of the editor within or 
*  around the tab button.
*  Note: The height of the editor is an attribute of the editor itself, and can be
*  set by specifying a "height" property in TabSet.titleEditor.
* @type {Integer}
* @default null
*/
titleEditorTopOffset?: Integer;

/**
* Setting this to true turns on a different appearance for tabs, similar to iOS tabs from 
*  the "Music" app, where the tab.icon is enlarged and shown as a black and white mask. 
*  This mode does not support a clickable icon - clicking the enlarged icon just switches 
*  tabs.
*  
*  This attribute only has an effect for tabs that are not Tab.canClose,
*  and only for Mobile WebKit.
* @type {boolean}
* @default false
*/
useIOSTabs?: boolean;

/**
* Causes the TabSet.addTabButton to appear after the TabSet.tabs and before the
*  TabSet.tabBarControls. 
*  
*  There is no default behavior for what happens when the addTabButton is
*  clicked. Add a handler for the TabSet.addTabClicked event to implement a behavior.
* @type {boolean}
* @default null
*/
canAddTabs?: boolean;

/**
* Appears when TabSet.canAddTabs is enabled.
* @type {ImgButton}
* @default null
*/
addTabButton?: ImgButton;

/**
* Icon for the TabSet.addTabButton.
* @type {SCImgURL}
* @default "[SKIN]actions/add.png"
*/
addTabButtonIcon?: SCImgURL;

/**
* Should the tabBar be displayed or not
*  If shrinkElementOnHide is true, the paneContainer will expand over the space
*  occupied by TabBar
* @type {boolean}
* @default true
*/
showTabBar?: boolean;

/**
* Tab button constructor if TabSet.useSimpleTabs is true.
* @type {Class}
* @default "SimpleTabButton"
*/
simpleTabButtonConstructor?: Class;

/**
* This attribute allows developers to specify custom properties for this tabset's
*  TabSet.tabBar
* @type {Partial<TabBar>}
* @default null
*/
tabBarProperties?: Partial<TabBar>;

/**
* If the paneContainer for this tab set is showing Canvas.showEdges, setting this
*  attribute to true will set the paneContainer to show
*  Canvas.customEdges for the three sides opposing the tabBarPosition.
* @type {boolean}
* @default false
*/
showPartialEdges?: boolean;

/**
* Whether Canvas.destroy should be called on Tab.pane when it a tab is
*  removed via TabSet.removeTab. 
*  
*  With the default setting of null panes will be automatically destroyed.
*  An application might set this to false in order to re-use panes in different tabs or in
*  different parts of the application.
* @type {boolean}
* @default null
*/
destroyPanes?: boolean;

/**
* When AutoTest.getElement is used to parse locator strings generated by
*  AutoTest.getLocator, how should tabs within this tabset be identified?
*  If the locator has a specified Tab.ID or Tab.name, no fallback
*  approach will be used as those attributes (with Tab.ID having priority) are
*  each alone considered to definitively locate it.
*  
*  Otherwise, the following options are available:
*  
*  
* - "title" use the title as an identifier
*  
* - "index" use the index of the tab in the tabset as an identifier
*  
*  If unset, and the locator has no specified ID or name, default behavior is to
*  identify by title (if available), otherwise by index.
* @type {string}
* @default null
*/
locateTabsBy?: string;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "TabSetEditProxy"
*/
editProxyConstructor?: SCClassName;



/**
*  Update TabSet.paneContainerOverflow after creation.
* 
* 
* @param {Overflow} new overflow setting
*/
setPaneContainerOverflow(newOverflow:Overflow): void;

/**
*  If the paneContainer for this tab set is showing Canvas.showEdges, this 
*  method can be used to specify (dynamically) which Canvas.customEdges to
*  show. Called when the pane creator is created.
*  
*  Default implementation will return null unless TabSet.showPartialEdges
*  is true, in which case it will return the three edges opposite the
*  TabSet.tabBarPosition.
* 
*/
getPaneContainerEdges(): Array<any>;

/**
*  Changes the title of a tab
* 
* @param {Tab | number | GlobalId | TabName} * @param {HTMLString} new title
*/
setTabTitle(tab:Tab | number | GlobalId | TabName, title:HTMLString): void;

/**
*  Changes the title of the picker menu item of a tab
* 
* @param {Tab | number | GlobalId | TabName} * @param {HTMLString} new title
*/
setTabPickerTitle(tab:Tab | number | GlobalId | TabName, pickerTitle:HTMLString): void;

/**
*  Changes the icon for a tab
* 
* @param {Tab | number | GlobalId | TabName} tab to update
* @param {SCImgURL} new icon
*/
setTabIcon(tab:Tab | number | GlobalId | TabName, icon:SCImgURL): void;

/**
*  If the specified tab is disabled, enable it now.
* 
* @param {Tab | number | GlobalId | TabName} */
enableTab(tab:Tab | number | GlobalId | TabName): void;

/**
*  If the specified tab is enabled, disable it now.
* 
* @param {Tab | number | GlobalId | TabName} */
disableTab(tab:Tab | number | GlobalId | TabName): void;

/**
*  Apply properties to an existing tab in a tabSet.
* 
* @param {Tab | number | GlobalId | TabName} Identifier for the tab to be modified
* @param {object} Javascript object containing the set of properties to be applied to the tab.
*/
setTabProperties(tab:Tab | number | GlobalId | TabName, properties:object): void;

/**
*  Add a tab
* 
* @param {Tab} new tab
* @param {number=} position where tab should be added
*/
addTab(tab:Tab, position?:number): void;

/**
*  Add one or more tabs
* 
* @param {Tab | Array<Partial<Tab>>} new tab or tabs
* @param {number} position where tab should be added (or array of positions)
*/
addTabs(tabs:Tab | Array<Partial<Tab>>, position:number): void;

/**
*  Is the tab Tab.hidden?
*  
* 
* @param {number | string | Tab} Tab to test
*/
tabIsVisible(tab:number | string | Tab): boolean;

/**
*  Hide a tab in this tabset at runtime. If the tab is selected, it will be deselected
*  and the tab button will be hidden from the user.
*  
*  Note that this does not remove a tab from the tabset entirely (see TabSet.removeTab)
*  The tab will no longer be visible to the user or selectable by the user, but the configuration
*  will still existing in the TabSet.tabs for this tabSet. Developers should
*  particularly be aware of this when calling methods that refer to tabs by index - the
*  index includes both hidden and visible tabs in the tabset.
*  
*  Tabs may be marked as hidden at init-time via Tab.hidden.
*  
*  To test whether a tab is currently visible, use TabSet.tabIsVisible
* 
* 
* @param {number | string | Tab} Tab to hide
*/
hideTab(tab:number | string | Tab): void;

/**
*  Show a Tab.hidden at runtime.
*  
*  To test whether a tab is currently visible, use TabSet.tabIsVisible
* 
* 
* @param {number | string | Tab} Tab to hide
*/
showTab(tab:number | string | Tab): void;

/**
*  Apply a new Tab.pane to an existing tab in this tabSet
* 
* @param {number | string | Tab} Tab to update (may be referenced by ID or index)
* @param {Canvas} new Pane for the tab
*/
setTabPane(tab:number | string | Tab, pane:Canvas): void;

/**
*  Remove a tab.
*  
*  The pane associated with the removed tab is automatically destroyed when you
*  call this method. To avoid this, call TabSet.updateTab with null as the new
*  pane immediately before removing the tab, or set TabSet.destroyPanes to false.
*  
* 
* @param {Tab | GlobalId | TabName | number | Array<Partial<Tab>>} list of tabs, tabIDs,                                    or tab numbers
*/
removeTab(tabs:Tab | GlobalId | TabName | number | Array<Partial<Tab>>): void;

/**
*  Remove one or more tabs. The pane(s) associated with the removed tab(s) is automatically
*  destroyed when you call this method.
* 
* 
* @param {Tab | GlobalId | TabName | number} list of tabs, tabIDs, tab names, or                             tab numbers
*/
removeTabs(tabs:Tab | GlobalId | TabName | number): void;

/**
*  Removes the last tab in the TabSet, excluding the TabSet.moreTab if present.
* 
*/
removeLastTab(): void;

/**
*  Move a tab to another location in the tabset.
* 
* @param {Tab | GlobalId | TabName | number} tab to move
* @param {number=} the index to move the tab to - defaults to the end of the                 tabset if not passed
*/
reorderTab(tab:Tab | GlobalId | TabName | number, moveToPosition?:number): void;

/**
*  Sets the given tab's Tab.canClose property to the boolean parameter canClose.
*  If canClose is null, this will have the effect of causing the tab to fall back on TabSet.canCloseTabs.
* 
* @param {Tab | GlobalId | TabName | number} tab to change
* @param {boolean} new value for the tab's canClose property, or null to clear it
*/
setCanCloseTab(tab:Tab | GlobalId | TabName | number, canClose:boolean): void;

/**
*  Changes this TabSet's TabSet.canCloseTabs property.
* 
* @param {boolean} the new value for canCloseTabs.
*/
setCanCloseTabs(canCloseTabs:boolean): void;

/**
*  When TabSet.canCloseTabs is set, method fired when the user clicks the "close" icon for a
*  tab.
*  
*  Default implementation will remove the tab from the tabSet via TabSet.removeTab.
* 
* 
* @param {Tab} tab to close
*/
closeClick(tab:Tab): void;

/**
*  Method fired when the user clicks the icon for a tab, as specified via Tab.icon.
*  
*  Default behavior will fire icon.click() if specified, with two parameters
*  tab (a pointer to the tab object and tabSet a pointer to the tabSet
*  instance.
* 
* @param {Tab} with click handler being fired
*/
tabIconClick(tab:Tab): void;

/**
*  Get the tab Object originally passed to TabSet.tabs, by index, name or ID.
*  If passed a tab Object, just returns it.
* 
* 
* @param {Integer | GlobalId | TabName | Tab} */
getTabObject(tab:Integer | GlobalId | TabName | Tab): Tab;

/**
*  Get the live Canvas representing a tab by index, ID, reference, or name. 
*  If passed a tab Canvas, just returns it.
*  
*  Note that live Tab instances are not available until Canvas.draw.
*  
*  The returned Tab is considered an internal component of the TabSet. In order to maximize
*  forward compatibility, manipulate tabs through APIs such as a TabSet.setTabTitle instead.
*  Also note that a super-lightweight TabSet implementation may not use a separate Canvas per
*  Tab, and code that accesses and manipulates Tabs as Canvases won't be compatible with that
*  implementation.
* 
* 
* @param {Integer | GlobalId | TabName | Canvas} identifier for the tab or tab button
*/
getTab(tab:Integer | GlobalId | TabName | Canvas): StatefulCanvas;

/**
*  Returns the pane for a given tab.
* 
* 
* @param {object | number | GlobalId | TabName | Tab} */
getTabPane(tab:object | number | GlobalId | TabName | Tab): Canvas;

/**
*  Get the index of a tab, from the tab, tab ID or tab name. If passed a number, just returns it.
*  Note that the index returned will include Tab.hidden as well as visible
*  tabs.
* 
* 
* @param {number | GlobalId | TabName | Tab} */
getTabNumber(tab:number | GlobalId | TabName | Tab): number;

/**
*  Set the pane for a tab.
*  
*  Pass in the index of a tab (or a tab object), and a new pane.
*  
*  NOTE: the old pane for the tab is not destroy()d
*  
* 
* @param {number | GlobalId | TabName | Tab} tab to update
* @param {Canvas | GlobalId} new pane for the tab
*/
updateTab(tab:number | GlobalId | TabName | Tab, pane:Canvas | GlobalId): void;

/**
*  Reveals the child Canvas passed in by selecting the tab containing that child if it is not
*  already selected. If no tab in this TabSet contains the passed-in Canvas, this method has 
*  no effect
* 
* 
* @param {GlobalId | Canvas} the child Canvas to reveal, or its global ID
*/
revealChild(child:GlobalId | Canvas): void;

/**
*  If there is not enough space to display all the tabs in this tabSet, this method will 
*  scroll the next tab (that first tab that is clipped at the end of the tab-bar) into view.
* 
*/
scrollForward(): void;

/**
*  If there is not enough space to display all the tabs in this tabSet, this method will 
*  scroll the previous tab (that first tab that is clipped at the beginning of the tab-bar) 
*  into view.
* 
*/
scrollBack(): void;

/**
*  Returns the currently selected tab object. This is the object literal used to configure the
*  tab, rather than the tab button widget.
* 
*/
getSelectedTab(): Tab;

/**
*  Returns the index of the currently selected tab object. 
* 
*/
getSelectedTabNumber(): number;

/**
* 	Select a tab. Note that this method will have no effect if the tab is Tab.hidden.
* 
* 
* @param {number | GlobalId | TabName | Tab} tab to select
*/
selectTab(tab:number | GlobalId | TabName | Tab): void;

/**
* Search for a tab that contains a pane.
* 
* @param {Canvas} pane to show
*/
tabForPane(pane:Canvas): Tab;

/**
*  Places an editor in the title of the parameter tab and allows the user to edit the title.
*  Note that this programmatic method will always allow editing of the specified tab's
*  title, regardless of the settings of TabSet.canEditTabTitles or Tab.canEditTitle.
* 
* @param {Tab | string | Integer} The tab whose title should be edited (may be  specified by ID or index)
*/
editTabTitle(tab:Tab | string | Integer): void;

/**
*  If the user is currently editing a tab title (see TabSet.canEditTabTitles), dismiss
*  the editor and discard the edit value entered by the user.
* 
*/
cancelTabTitleEditing(): void;

/**
*  If the user is currently editing a tab title (see TabSet.canEditTabTitles), save
*  the edited tab title and hide the editor.
* 
*/
saveTabTitle(): void;

/**
*  Event that fires when the TabSet.addTabButton is clicked.
*  No default behavior.
* 
* 
*/
addTabClicked(): void;

/**
*  Notification fired when a tab is selected. Note that this will only fire if 
*  this tabSet is drawn. If a tab is selected before Canvas.draw 
*  is called, the tabSelected() notification
*  
*  will fire on draw().
* 
* @param {Integer} the index of the newly selected tab
* @param {Canvas} the newly selected tab's pane if set
* @param {GlobalId} the ID of the newly selected tab
* @param {Tab} the tab object (not tab button instance)
* @param {TabName} the name of the newly selected tab
*/
tabSelected(tabNum:Integer, tabPane:Canvas, ID:GlobalId, tab:Tab, name:TabName): void;

/**
*  Optional handler to fire when a tab is deselected.
*  Returning false will cancel the new selection, leaving tab ID selected. As with
*  TabSet.tabSelected this method
*  only fires when the tabset is drawn.
* 
* 
* @param {Integer} the index of the deselected tab
* @param {Canvas} the deselected tab's pane if set
* @param {GlobalId} the ID of the deselected tab
* @param {Tab} the deselected tab object (not tab button instance)
* @param {Tab} the tab object being selected
* @param {TabName} the name of the deselected tab
*/
tabDeselected(tabNum:Integer, tabPane:Canvas, ID:GlobalId, tab:Tab, newTab:Tab, name:TabName): boolean;

/**
*  This notification method fired when the user changes the title of a tab in this TabSet.
*  This can happen either through user interaction with the UI if 
*  TabSet.canEditTabTitles is set, or programmatically if application 
*  code calls TabSet.editTabTitle.
*  Return false from this method to cancel the change.
* 
* @param {string} the new title
* @param {string} the old title
* @param {Tab} the tab whose title has changed
*/
titleChanged(newTitle:string, oldTitle:string, tab:Tab): boolean;

/**
*  Notification fired when the user right-clicks on a tab.
*  Event may be cancelled by returning false
* 
* @param {TabSet} This tabset
* @param {Tab} the tab object that recieved the context click event
*/
showTabContextMenu(tabSet:TabSet, tab:Tab): boolean;

/**
*  Notification method executed when one or more tabs in the TabSet are reordered.
* 
* @param {StatefulCanvas} the live Canvas representing the tab that was moved
* @param {Integer} the new index of the tab in the tabSet
*/
tabsReordered(tabCanvas:StatefulCanvas, tabIndex:Integer): void;



}
/**
* Creates a new TabSet
* 
* @param typeCheckedProperties {Partial<TabSet>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TabSet} The new instance
*/
class TabSet{
static create(typeCheckedProperties? : Partial<TabSet>, uncheckedProperties?: Object): TabSet;







}

/**
* EditProxy that handles TabSet objects when editMode is enabled.
*/
interface TabSetEditProxy extends CanvasEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns a comma-separated list of tab titles. A " [x]" suffix is added
*  for any tab with canClose enabled.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Takes a comma-separated list of tab titles. Add " [x]" to a title
*  to enable canClose for the tab.
* 
* 
* @param {string} the new component state
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new TabSetEditProxy
* 
* @param typeCheckedProperties {Partial<TabSetEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TabSetEditProxy} The new instance
*/
class TabSetEditProxy{
static create(typeCheckedProperties? : Partial<TabSetEditProxy>, uncheckedProperties?: Object): TabSetEditProxy;







}

/**
* Shows a listing of records with one or more fields from each record, with
*  built-in support for navigation and editing of lists of records.
*  
*  The TableView provides built-in controls such as TableView.showNavigation and
*  shows fields from the provided records in one of several built-in RecordLayouts.
*  
*  NOTE: This widget is intended primarily for creating handset/phone-sized interfaces
*  and does not have an appearance in any skin other than Mobile.
*/
interface TableView extends ListGrid {
/**
* Option to show filter editor is disabled for TableView
* @type {boolean}
* @default false
*/
canShowFilterEditor?: boolean;

/**
* Option to save searches is disabled for TableView
* @type {boolean}
* @default false
*/
canSaveSearches?: boolean;

/**
* This property allows the developer to specify the icon displayed next to a record.
*  Set record[tableView.iconField] to the URL of the desired icon to display.
*  Only applies if TableView.showIconField is true.
* @type {string}
* @default "icon"
*/
iconField?: string;

/**
* Should an icon field be shown for each record? A column in the table is set
*  aside for an icon as specified on each record in the TableView.iconField.
* @type {boolean}
* @default true
*/
showIconField?: boolean;

/**
* Field to display for an individual record as the main title.
* @type {string}
* @default "title"
*/
titleField?: string;

/**
* Field to display as part of individual record in "summary" RecordLayouts.
* @type {string}
* @default "info"
*/
infoField?: string;

/**
* Field to display as part of individual record in "summary" RecordLayouts.
* @type {string}
* @default "data"
*/
dataField?: string;

/**
* Field to display as part of individual record in all RecordLayouts 
*  except "titleOnly".
* @type {string}
* @default "description"
*/
descriptionField?: string;

/**
* Boolean property on each record that controls whether navigation controls are shown for
*  that record. If property is not defined on the record a navigation icon is shown
*  if TableView.showNavigation is true.
* @type {string}
* @default "_navigate"
*/
recordNavigationProperty?: string;

/**
* The display mode of the table.
* @type {TableMode}
* @default "plain"
*/
tableMode?: TableMode;

/**
* Sets the arrangement of data fields from the record.
*  
*  Note that controls supported by the TableView itself, such as navigation icons, are
*  implicitly added to the data fields described in the RecordLayout. If an
*  TableView.iconField has been configured, it too is an implicitly shown field, to the left
*  of the area controlled by RecordLayout.
* @type {RecordLayout}
* @default "titleOnly"
*/
recordLayout?: RecordLayout;

/**
* The navigation icon shown next to records when
*  TableView.showNavigation is true and NavigationMode is set to
*  "navIconOny".
* @type {SCImgURL}
* @default "[SKINIMG]/iOS/listArrow_button.png"
*/
navIcon?: SCImgURL;

/**
* The navigation icon shown next to records when TableView.showNavigation
*  is true and NavigationMode is set to "wholeRecord".
* @type {SCImgURL}
* @default "[SKINIMG]/iOS/listArrow.png"
*/
wholeRecordNavIcon?: SCImgURL;

/**
* Whether to show navigation controls by default on all records. Can also be configured
*  per-record with TableView.recordNavigationProperty.
* @type {boolean}
* @default null
*/
showNavigation?: boolean;

/**
* Set navigation mode for this TableView.
* @type {NavigationMode}
* @default "wholeRecord"
*/
navigationMode?: NavigationMode;

/**
* Default style for title.
* @type {CSSStyleName}
* @default "recordTitle"
*/
recordTitleStyle?: CSSStyleName;

/**
* Default style for description.
* @type {CSSStyleName}
* @default "recordDescription"
*/
recordDescriptionStyle?: CSSStyleName;

/**
* Default style for data field.
* @type {CSSStyleName}
* @default "recordData"
*/
recordDataStyle?: CSSStyleName;

/**
* Default style for info field.
* @type {CSSStyleName}
* @default "recordInfo"
*/
recordInfoStyle?: CSSStyleName;



/**
*  Executed when the user clicks on a record, or on the navigate icon for a
*  record depending on NavigationMode.
* 
* 
* @param {ListGridRecord} record clicked
*/
recordNavigationClick(record:ListGridRecord): void;

/**
*  Executed when the user clicks on the image displayed in a record if
*  TableView.iconField has been specified.
* 
* 
* @param {ListGridRecord} record clicked
*/
imageClick(record:ListGridRecord): void;

/**
*  Formatter to apply to record display.
* 
* 
* @param {ListGridRecord} record to format
*/
formatRecord(record:ListGridRecord): HTMLString;



}
/**
* Creates a new TableView
* 
* @param typeCheckedProperties {Partial<TableView>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TableView} The new instance
*/
class TableView{
static create(typeCheckedProperties? : Partial<TableView>, uncheckedProperties?: Object): TableView;
/**
* A declared value of the enum type 
*  TableMode.
* @type {Constant}
* @default "plain"
*/
static PLAIN?: Constant;

/**
* A declared value of the enum type 
*  TableMode.
* @type {Constant}
* @default "grouped"
*/
static GROUPED?: Constant;

/**
* A declared value of the enum type 
*  RecordLayout.
* @type {Constant}
* @default "titleOnly"
*/
static TITLE_ONLY?: Constant;

/**
* A declared value of the enum type 
*  RecordLayout.
* @type {Constant}
* @default "titleAndDescription"
*/
static TITLE_DESCRIPTION?: Constant;

/**
* A declared value of the enum type 
*  RecordLayout.
* @type {Constant}
* @default "summaryInfo"
*/
static SUMMARY_INFO?: Constant;

/**
* A declared value of the enum type 
*  RecordLayout.
* @type {Constant}
* @default "summaryData"
*/
static SUMMARY_DATA?: Constant;

/**
* A declared value of the enum type 
*  RecordLayout.
* @type {Constant}
* @default "summaryFull"
*/
static SUMMARY_FULL?: Constant;

/**
* A declared value of the enum type 
*  NavigationMode.
* @type {Constant}
* @default "wholeRecord"
*/
static WHOLE_RECORD?: Constant;

/**
* A declared value of the enum type 
*  NavigationMode.
* @type {Constant}
* @default "navIconOnly"
*/
static NAVICON_ONLY?: Constant;








}

/**
* A Task is an abstract superclass for Process and for all Task types that can be
*  involved in a Process, such as a ServiceTask.
*/
interface Task extends ProcessElement {
/**
* Field in the Process.state which is provided as input data to this
*  task.
*  See taskIO.
* @type {string}
* @default null
*/
inputField?: string;

/**
* List of multiple fields from the Process.state which are provided
*  as input data to this task. See taskIO.
*  
*  If Task.inputField is also specified, it will be implicitly added to the
*  inputFieldList if it is not already present.
* @type {Array<Partial<string>>}
* @default null
*/
inputFieldList?: Array<Partial<string>>;

/**
* Field in the Process.state where this task writes outputs. See
*  taskIO.
* @type {string}
* @default null
*/
outputField?: string;

/**
* List of multiple fields in the Process.state where this task will
*  write outputs. See taskIO. 
*  
*  If Task.outputField is also specified, it will be implicitly added to the
*  outputFieldList if it is not already present.
* @type {Array<Partial<string>>}
* @default null
*/
outputFieldList?: Array<Partial<string>>;

/**
* Special expression to write task output directly into a DataBoundComponent. See
*  taskIO.
* @type {string}
* @default null
*/
outputExpression?: string;





}
/**
* Creates a new Task
* 
* @param typeCheckedProperties {Partial<Task>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Task} The new instance
*/
class Task{
static create(typeCheckedProperties? : Partial<Task>, uncheckedProperties?: Object): Task;







}

/**
* Class for editable multi-line text areas (uses HTML &lt;TEXTAREA&gt; object)
*/
interface TextAreaItem extends FormItem {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "TextAreaItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Text wrapping style.
* @type {TEXTAREA_WRAP}
* @default "SOFT"
*/
wrap?: TEXTAREA_WRAP;

/**
* default width of this item
* @type {number}
* @default "150"
*/
width?: number;

/**
* Default height of this item
*  
*  Note that when item is rendered as read-only with readOnlyDisplay as "static"
*  the property FormItem.staticHeight is used instead.
* @type {number}
* @default "100"
*/
height?: number;

/**
* TextAreaItem has an explicit pickerIconHeight as we don't want the picker icon, if shown,
*  to size itself to match the height of the item.
* @type {Integer}
* @default "20"
*/
pickerIconHeight?: Integer;

/**
* Height of the FormItem when canEdit is false and
*  readOnlyDisplay is "static". The normal TextAreaItem.height is used
*  if this property is not set.
* @type {Integer}
* @default "1"
*/
staticHeight?: Integer;

/**
* Base CSS class to apply to this item's input element.
*  NOTE: See the CompoundFormItem_skinning discussion for special skinning considerations.
* @type {FormItemBaseStyle}
* @default "textItem"
*/
textBoxStyle?: FormItemBaseStyle;

/**
* @type {boolean}
* @default null
*/
browserAutoCapitalize?: boolean;

/**
* In Mobile Safari, should automatic correction be offered for text in the item's text box?
*  If null, then Mobile Safari determines automatically whether to enable
*  autocorrect.
*  
*  When enabled, Mobile Safari displays "autocorrect bubbles" to suggest automatic corrections:
*  
* @type {boolean}
* @default null
*/
browserAutoCorrect?: boolean;

/**
* If set, maximum number of characters for this field. If TextAreaItem.enforceLength is
*  set to true, user input will be limited to this value, and values exceeding this
*  length passed to FormItem.setValue will be trimmed. Otherwise values exceeding the
*  specified length will raise an error on validation.
*  
*  See also DataSourceField.length.
* @type {number}
* @default null
*/
length?: number;

/**
* If a TextAreaItem.length is specified for this item, should user input be limited
*  to the specified length? If set to true, user input and values passed to 
*  FormItem.setValue will be trimmed to the specified length. Otherwise values
*  exceeding the specified length will raise an error on validation.
*  
*  Note that having this value set to true limits user interactivity in some ways.
*  For example users would be unable to paste a longer string into the field for
*  editing without seeing it be truncated. Given how text areas are typically
*  used to edit longer values than non-wrapping TextItems, this value is
*  false by default for textAreaItems.
* @type {boolean}
* @default false
*/
enforceLength?: boolean;

/**
* How should an empty string entered by the user be stored?
*  This value is typically set to null or "".
*  
*  Note that a call to FormItem.setValue or FormItem.setValue
*  automatically updates this property to ensure that "empty" values are stored in a 
*  consistent format.
* @type {any}
* @default null
*/
emptyStringValue?: any;

/**
* Align icons with the top edge of text area icons by default.
* @type {VerticalAlignment}
* @default "top"
*/
iconVAlign?: VerticalAlignment;

/**
* Allows the DynamicForm.selectOnFocus behavior to be configured on a
*  per-FormItem basis. Normally all items in a form default to the value of
*  DynamicForm.selectOnFocus.
* @type {boolean}
* @default null
*/
selectOnFocus?: boolean;

/**
* Allows the DynamicForm.selectOnClick behavior to be configured on a
*  per-FormItem basis. Normally all items in a form default to the value of
*  DynamicForm.selectOnClick.
* @type {boolean}
* @default null
*/
selectOnClick?: boolean;

/**
* Should this form item fire its FormItem.change handler (and store its
*  value in the form) on every keypress? Set to false to suppress the 'change'
*  handler firing (and the value stored) on every keypress.
*  
*  Note: If false, the value returned by FormItem.getValue
*  will not reflect the value displayed in the form item element as long as focus is in
*  the form item element.
* @type {boolean}
* @default true
*/
changeOnKeypress?: boolean;

/**
* Does the current formItem support native cut and paste events?
*  
*  This attribute only applies to freeform text entry fields such as TextItem and
*  TextAreaItem, and only if TextAreaItem.changeOnKeypress is true. 
*  If true, developers can detect the user editing the value 
*  via cut or paste interactions (triggered from keyboard shortcuts or the native 
*  browser menu options) using the FormItem.isCutEvent and
*  FormItem.isPasteEvent methods. This allows custom cut/paste handling
*  to be added to the various change notification flow methods including 
*  FormItem.change, and
*  FormItem.transformInput.
* @type {boolean}
* @default true
*/
supportsCutPasteEvents?: boolean;

/**
* If FormItem.showHint, should the hint be shown
*  within the field?
*  
*  Unless the HTML5 placeholder attribute is used to display the hint
*  (see TextAreaItem.usePlaceholderForHint),
*  the value of the &lt;input&gt; element for this item will be set to the hint
*  whenever this item is not focused. Also, when displaying the hint, the CSS style of the
*  data element will be set to the TextItem.textBoxStyle with the suffix
*  "Hint" appended to it; or, if the item is disabled, the suffix "DisabledHint" will be used.
*  In Page.isRTL when FormItem.showRTL is true,
*  an additional "RTL" suffix will be appended; i.e. the CSS style of the data element when
*  the hint is displayed will be the textBoxStyle plus "HintRTL" or 
*  "DisabledHintRTL".
*  
*  To change this attribute after being drawn, it is necessary to call FormItem.redraw
*  or redraw the form.
*  
*  Styling the in-field hint
*  The in-field hint can be styled with CSS for the textBoxStyle + "Hint" /
*  "HintRTL" / "DisabledHint" / "DisabledHintRTL" styles. For example, if this item's
*  textBoxStyle is set to "mySpecialItem", then changing the hint color to
*  blue can be accomplished with the following CSS:
*  .mySpecialItemHint,
* .mySpecialItemHintRTL,
* .mySpecialItemDisabledHint,
* .mySpecialItemDisabledHintRTL {
*   color: blue;
* }
*  
*  In DynamicForm.linearMode, this property will be defaulted true if left unset.
* @type {boolean}
* @default null
*/
showHintInField?: boolean;

/**
* If TextAreaItem.showHintInField and if supported by the browser, should the HTML5
*  placeholder attribute
*  be used to display the hint within the field? If set to false, then use of
*  the placeholder attribute is disabled and an alternative technique to display
*  the hint in-field is used instead.
*  
*  The HTML5 placeholder attribute is supported in the following major browsers:
*  
*  
* - Chrome 4+
*  
* - Firefox 4+
*  
* - Internet Explorer 10+
*  
* - Safari 5+
*  
* - Opera 11.50+
*  
* - Android 2.1+ WebView (used by the stock Browser app and when
*    phonegapIntegration)
*  
* - Mobile Safari for iOS 3.2+
*  
*  
*  In browsers other than the above, in-field hints are implemented via a different technique.
*  
*  Note that placeholder behavior is known to differ in Internet Explorer and certain old
*  versions of the above browsers due to a recent change in the HTML5 specification regarding
*  the placeholder attribute. Under the old rules, the placeholder is cleared
*  when the element is focused. In the latest HTML5 spec as published by WHATWG, the placeholder
*  is still displayed when the element is focused as long as the value is an empty string.
*  
*  Styling the placeholder
*  While there isn't a standard way to style the placeholder text, Chrome, Firefox,
*  Internet Explorer, and Safari provide vendor-prefixed pseudo-classes and/or pseudo-elements
*  that can be used in CSS selectors:
*  
*  
*   Browser
*   Pseudo-class or pseudo-element
*  
*  
*   Chrome, Safari
*   ::-webkit-input-placeholder
*  
*  
*   Firefox 4 - 18
*   :-moz-placeholder
*  
*  
*   Firefox 19+
*   ::-moz-placeholder
*  
*  
*   Internet Explorer
*   :-ms-input-placeholder
*  
*  
*  
*  Note that unlike other browsers, Firefox 19+ applies opacity:0.4 to the placeholder text.
*  See Bug 556145 - Placeholder text default style should use opacity instead of GrayText
*  
*  Because browsers are required to ignore the entire rule if a selector is invalid,
*  separate rules are needed for each browser. For example:
*  ::-webkit-input-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
* :-moz-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
* ::-moz-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
* :-ms-input-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
*  
*  If using Sass, it may be useful to utilize Sass'
*  parent selector feature.
*  For example:
*  .myCustomItem,
* .myCustomItemRTL,
* .myCustomItemDisabled,
* .myCustomItemDisabledRTL,
* .myCustomItemError,
* .myCustomItemErrorRTL,
* .myCustomItemFocused,
* .myCustomItemFocusedRTL,
* .myCustomItemHint,
* .myCustomItemHintRTL,
* .myCustomItemDisabledHint,
* .myCustomItemDisabledHintRTL {
*   // ...
* 
*   &amp;::-webkit-input-placeholder {
*     color: blue;
*     opacity: 1;
*   }
*   &amp;:-moz-placeholder {
*     color: blue;
*     opacity: 1;
*   }
*   &amp;::-moz-placeholder {
*     color: blue;
*     opacity: 1;
*   }
*   &amp;:-ms-input-placeholder {
*     color: blue;
*     opacity: 1;
*   }
* &#125;
*  
*  If using Compass, the
*  input-placeholder mixin
*  that was added in version 1.0 can further simplify the code to style the placeholder text
*  For example:
*  .myCustomItem,
* .myCustomItemRTL,
* .myCustomItemDisabled,
* .myCustomItemDisabledRTL,
* .myCustomItemError,
* .myCustomItemErrorRTL,
* .myCustomItemFocused,
* .myCustomItemFocusedRTL,
* .myCustomItemHint,
* .myCustomItemHintRTL,
* .myCustomItemDisabledHint,
* .myCustomItemDisabledHintRTL {
*   // ...
* 
*   &#64;include input-placeholder {
*     color: blue;
*     opacity: 1;
*   }
* &#125;
*  Accessibility concerns
*  The HTML5 specification notes that a placeholder should not be used as a replacement
*  for a title. The placeholder is intended to be a short hint that assists the user
*  who is entering a value into the empty field. The placeholder can be mistaken by some
*  users for a pre-filled value, particularly in Internet Explorer because the same color
*  is used, and the placeholder text color may provide insufficient contrast, particularly
*  in Firefox 19+ because of the default 0.4 opacity. Furthermore, not having a title reduces
*  the hit area available for setting focus on the item.
* @type {boolean}
* @default true
*/
usePlaceholderForHint?: boolean;

/**
* When generating a print-view of the component containing this TextArea, should
*  the form item expand to accommodate its value? If set to false the text box not expand
*  to fit its content in the print view, instead showing exactly as it does in the
*  live form, possibly with scrollbars.
* @type {boolean}
* @default true
*/
printFullText?: boolean;

/**
* By default HTML characters will be escaped when FormItem.canEdit is false and 
*  FormItem.readOnlyDisplay is "static", so that the raw value of the field (for
*  example "&lt;b&gt;AAA&lt;/b&gt;") is displayed to the user rather than
*  the interpreted HTML (for example "AAA"). Setting
*  escapeHTML false will instead force HTML values in a textAreaItem to be
*  interpreted by the browser in that situation.
* @type {boolean}
* @default true
*/
escapeHTML?: boolean;

/**
* When set to false, prevents this item's input element from being written into the DOM.
*  If there are FormItem.valueIcons or a 
*  FormItem.showPickerIcon, these are displayed as normal, and the item
*  will auto-sizing to that content if its FormItem.width is set to null.
* @type {boolean}
* @default true
*/
showInputElement?: boolean;

/**
* Minimum valid height for this TextAreaItem in px. If the specified TextAreaItem.height
*  is less than this value, the text area will still render at this height.
* @type {Integer}
* @default "16"
*/
minHeight?: Integer;

/**
* With formatOnBlur enabled, this textAreaItem will format its value
*  according to the rules described in FormItem.mapValueToDisplay as long as the 
*  item does not have focus. Once the user puts focus into the item
*  the formatter will be removed. This provides a simple way for developers to
*  show a nicely formatted display value in a freeform text field, without the need
*  for an explicit FormItem.formatEditorValue 
*  and FormItem.parseEditorValue pair.
* @type {boolean}
* @default false
*/
formatOnBlur?: boolean;



/**
*  Returns the raw text value that currently appears in the text field, which can differ from 
*  FormItem.getValue in various cases - for example:
*  
*  
* - for items that constrain the value range, such as a DateItem with
*  DateItem.enforceDate:true, or a ComboBoxItem with
*  ComboBoxItem.addUnknownValues:false
*  
* - for items with a defined valueMap or edit value formatter and parser functions
*  which converts display value to data value
*  
* - while the item has focus if TextItem.changeOnKeypress is false
*  
* 
*/
getEnteredValue(): string;

/**
*  For text-based items, this method returns the indices of the start/end of the current
*  selection if the item currently has the focus. In browsers other than Internet Explorer 6-9,
*  if this item does not have focus, then this method returns the indices of the start/end
*  of the selection the last time that this item had focus. In IE 6-9, returns null if the
*  item does not have focus.
*  
*  In all browsers, clicking anywhere outside of the item causes the item to lose focus;
*  hence, in IE 6-9, this method will not work in other components' event handlers for
*  certain events. For example, within the Canvas.click handler of a button,
*  this item will have already lost focus, so in IE 6-9, this method will return null
*  if called within the button's click() handler. One cross-browser solution to this issue
*  is to save the selection range for later in a Canvas.mouseDown or
*  Canvas.mouseOver handler.
*  
*  Notes:
*  
*   
* - In browsers other than IE 6-9, calling FormItem.setValue
*  or otherwise changing the TextAreaItem.getEnteredValue invalidates the past
*  selection range.
*   
* - The returned indices are indices within the entered value rather than the item's
*  value as returned by FormItem.getValue.
*  The distinction is particularly important for TextAreaItems because browsers
*  normalize the line endings in the &lt;textarea&gt; element's value.
*  Internet Explorer 6, 7, and 8 convert line endings to "\r\n" while other browsers
*  convert line endings to "\n"
*  as specified by the HTML5 standard.
*  
* 
*/
getSelectionRange(): Array<Partial<Integer>>;

/**
*  Puts focus into this form item and selects characters between the given indices.
*  Only applies to drawn text based items.
* 
* @param {Integer} selection starting character index
* @param {Integer} end of selection character index
*/
setSelectionRange(start:Integer, end:Integer): void;

/**
*  Put focus in this item and select the entire value.
*  Only applies to text based items
* 
*/
selectValue(): void;

/**
*  If this item currently has focus, clear the current selection. leaving focus in the item.
*  Has no effect if the item is undrawn or unfocused.
*  Only applies to text-based items.
* 
* @param {boolean=} By default the text insertion cursor will be moved to the end of the  current value - pass in this parameter to move to the start instead
*/
deselectValue(start?:boolean): void;

/**
*  Notification fired in response to a clipboard "paste" event on freeform text
*  items, giving developers an opportunity to reformat the pasted text. The 
*  pastedValue argument contains the text pasted from the clipboard.
*  This method should return the text value to actually insert into the input element.
* 
* 
* @param {FormItem} Item into which the user pasted a value
* @param {DynamicForm} Pointer to the item's form
* @param {string} Pasted text value
*/
transformPastedValue(item:FormItem, form:DynamicForm, pastedValue:string): string;



}
/**
* Creates a new TextAreaItem
* 
* @param typeCheckedProperties {Partial<TextAreaItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TextAreaItem} The new instance
*/
class TextAreaItem{
static create(typeCheckedProperties? : Partial<TextAreaItem>, uncheckedProperties?: Object): TextAreaItem;
/**
* A declared value of the enum type 
*  TEXTAREA_WRAP.
* @type {Constant}
* @default "OFF"
*/
static OFF?: Constant;

/**
* A declared value of the enum type 
*  TEXTAREA_WRAP.
* @type {Constant}
* @default "SOFT"
*/
static SOFT?: Constant;

/**
* A declared value of the enum type 
*  TEXTAREA_WRAP.
* @type {Constant}
* @default "HARD"
*/
static HARD?: Constant;








}

/**
* EditProxy that handles TextAreaItem when editMode is enabled.
*/
interface TextAreaItemEditProxy extends TextItemEditProxy {




}
/**
* Creates a new TextAreaItemEditProxy
* 
* @param typeCheckedProperties {Partial<TextAreaItemEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TextAreaItemEditProxy} The new instance
*/
class TextAreaItemEditProxy{
static create(typeCheckedProperties? : Partial<TextAreaItemEditProxy>, uncheckedProperties?: Object): TextAreaItemEditProxy;







}

/**
* Settings for use with DataSource.recordsAsText.
*/
interface TextExportSettings extends TextSettings {
/**
* Separator between Records. Default is a newline character ("\n").
* @type {string}
* @default "\n"
*/
lineSeparator?: string;

/**
* Whether to surround each value with quotes ("").
* @type {boolean}
* @default true
*/
quoteValues?: boolean;

/**
* Text to export for a field with a null value. If this property is null, then
*  null fields will be assumed to have the default value for their field type.
* @type {string}
* @default ""
*/
nullValueText?: string;

/**
* Whether to convert each field's value to the corresponding display value
*  for export. Default of false will directly export the field's value.
* @type {boolean}
* @default false
*/
useDisplayValue?: boolean;

/**
* If set, all text fields will use the indicated ForceTextApproach unless they have
*  a specific setting for DataSourceField.exportForceText.
* @type {ForceTextApproach}
* @default null
*/
forceText?: ForceTextApproach;

/**
* Format to use when outputting date values. Default is to use the format expected by
*  Microsoft Excel (eg 1-2-2011), which Excel will turn into a real date value (see
*  excelPasting). The current month-day-year order as set by
*  DateUtil.setInputFormat will be used.
* @type {DateDisplayFormat}
* @default null
*/
dateFormat?: DateDisplayFormat;

/**
* Format to use when outputting datetime values. Default is to combine the configured date
*  and time formats with a space (" ").
* @type {DateDisplayFormat}
* @default null
*/
dateTimeFormat?: DateDisplayFormat;

/**
* Format to use when outputting time values. Default is 24 hour time.
* @type {TimeDisplayFormat}
* @default null
*/
timeFormat?: TimeDisplayFormat;





}
/**
* Creates a new TextExportSettings
* 
* @param typeCheckedProperties {Partial<TextExportSettings>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TextExportSettings} The new instance
*/
class TextExportSettings{
static create(typeCheckedProperties? : Partial<TextExportSettings>, uncheckedProperties?: Object): TextExportSettings;







}

/**
* Settings for use with DataSource.recordsFromText.
*/
interface TextImportSettings extends TextSettings {
/**
* If set to true, the data is assumed to have a header line that lists titles for each field,
*  which should be parsed. 
*  
*  recordsFromText will then try to find a same-named
*  DataSourceField by checking parsed titles against both DataSourceField.title and
*  DataSourceField.name (titles first), doing a case-insensitive comparison with any
*  leading or trailing whitespace removed from the title. If no field matches, data will
*  appear in the returned Records under the exact title parsed from the header line.
*  
*  If this approach will not find appropriate DataSourceFields, parse the header line before
*  calling recordsFromText(), and provide the list of field names to use when
*  parsing data as TextSettings.fieldList.
* @type {boolean}
* @default false
*/
hasHeaderLine?: boolean;





}
/**
* Creates a new TextImportSettings
* 
* @param typeCheckedProperties {Partial<TextImportSettings>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TextImportSettings} The new instance
*/
class TextImportSettings{
static create(typeCheckedProperties? : Partial<TextImportSettings>, uncheckedProperties?: Object): TextImportSettings;







}

/**
* FormItem for managing a text field.
*/
interface TextItem extends FormItem {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "TextItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Default width for fields.
* @type {number}
* @default "150"
*/
width?: number;

/**
* Default height for text items.
* @type {number}
* @default "19"
*/
height?: number;

/**
* Base CSS class name for this item's input element.
*  NOTE: See the CompoundFormItem_skinning discussion for special skinning considerations. 
*  
*  For a rounded text item, you can set textBoxStyle to "roundedTextItem".
*  This style exists only in Enterprise, EnterpriseBlue and Graphite skins. There is
*  no corresponding rounded style for SelectItem or ComboBoxItem as this creates an
*  awkward seam with the pop-up list (and a rounded pop-up list wouldn't help: data could
*  not be flush to corners). For these reasons we recommend rounded inputs only in
*  limited cases like single standalone fields.
* @type {FormItemBaseStyle}
* @default "textItem"
*/
textBoxStyle?: FormItemBaseStyle;

/**
* If set, the maximum number of characters for this field. If 
*  TextItem.enforceLength is set to true, user input will be limited 
*  to this value, and values exceeding this length passed to 
*  FormItem.setValue will be trimmed. Otherwise values exceeding the 
*  specified length will raise an error on validation.
*  
*  If the item has a numeric type, like IntegerItem or 
*  FloatItem, length is applied to the raw number value, after any
*  specified FormItem.decimalPrecision and 
*  FormItem.decimalPad but before any formatters - this means the string
*  measured includes sign and decimal placeholder, and padded decimal places as required,
*  but not thousands separators or any custom formatting.
*  
*  See also DataSourceField.length.
* @type {number}
* @default null
*/
length?: number;

/**
* If a TextItem.length is specified for this item, should user input be limited
*  to the specified length? If set to true, user input and values passed to 
*  FormItem.setValue will be trimmed to the specified length. Otherwise values
*  exceeding the specified length will raise an error on validation.
*  
*  Note that having this value set to true limits user interactivity in some ways.
*  For example users would be unable to paste a longer string into the field for
*  editing without seeing it be truncated.
* @type {boolean}
* @default true
*/
enforceLength?: boolean;

/**
* How should an empty string entered by the user be stored?
*  This value is typically set to null or "".
*  
*  Note that a call to FormItem.setValue or FormItem.setValue
*  automatically updates this property to ensure that "empty" values are stored in a 
*  consistent format.
* @type {any}
* @default null
*/
emptyStringValue?: any;

/**
* @type {boolean}
* @default null
*/
browserAutoCapitalize?: boolean;

/**
* In Mobile Safari, should automatic correction be offered for text in the item's text box?
*  If null, then Mobile Safari determines automatically whether to enable
*  autocorrect.
*  
*  When enabled, Mobile Safari displays "autocorrect bubbles" to suggest automatic corrections:
*  
* @type {boolean}
* @default null
*/
browserAutoCorrect?: boolean;

/**
* This property corresponds to the HTML5 "inputType" attribute applied to the &lt;input&gt;
*  element for this TextItem.
*  
*  The only currently supported use of this attribute is hinting to touch-enabled mobile
*  devices that a particular keyboard layout should be used. Even here, be careful; to
*  take a random example, using type "number" on Android up to at least 3.2 leads to a
*  keyboard with no "-" key, so negative numbers cannot be entered.
*  
*  Valid values:
*  
*   
*   
*    "text"
*    Normal text keyboard
*   
*   
*    "digits"
*    Makes the text field more suitable for entering a string of digits 0 - 9. On iOS,
*      this causes the virtual keyboard to show a numeric keypad with only "0", "1",
*      "2", ..., "9", and delete keys.
*   
*   
*    "email"
*    Makes the text field more suitable for entering an e-mail address. On iOS, this
*      causes the virtual keyboard to show special "@" and "." keys on the alphabetic
*      keys screen.
*   
*   
*    "tel"
*    Makes the text field more suitable for entering a telephone number. On iOS, this
*      causes the virtual keyboard to show a numeric keypad with a "+*#" key for
*      displaying punctuation keys.
*   
*   
*    "number"
*    Makes the text field more suitable for entering a floating-point value. On iOS,
*      this causes the virtual keyboard to start on the number and punctuation keys screen.
*      
*      NOTE: This is not an appropriate text input type for credit card numbers,
*      postal codes, ISBNs, and other formats that are not strictly parsable as floating-point
*      numbers. This is because the browser is required to perform floating-point value
*      sanitization to ensure that the value is a valid floating-point number.
*   
*   
*    "url"
*    Makes the text field more suitable for entering a URL. On iOS, this causes the
*      virtual keyboard to show a special ".com" key.
*   
*   
*    Any&nbsp;vendor-specific value
*    If a browser supports another input type.
*   
*   
*  
* @type {string}
* @default null
*/
browserInputType?: string;

/**
* Allows the DynamicForm.selectOnFocus behavior to be configured on a
*  per-FormItem basis. Normally all items in a form default to the value of
*  DynamicForm.selectOnFocus.
* @type {boolean}
* @default null
*/
selectOnFocus?: boolean;

/**
* Allows the DynamicForm.selectOnClick behavior to be configured on a
*  per-FormItem basis. Normally all items in a form default to the value of
*  DynamicForm.selectOnClick.
* @type {boolean}
* @default null
*/
selectOnClick?: boolean;

/**
* Should this form item fire its FormItem.change handler (and store its
*  value in the form) on every keypress? Set to false to suppress the 'change'
*  handler firing (and the value stored) on every keypress.
*  
*  Note: If false, the value returned by FormItem.getValue
*  will not reflect the value displayed in the form item element as long as focus is in
*  the form item element.
* @type {boolean}
* @default true
*/
changeOnKeypress?: boolean;

/**
* Does the current formItem support native cut and paste events?
*  
*  This attribute only applies to freeform text entry fields such as TextItem and
*  TextAreaItem, and only if TextItem.changeOnKeypress is true. 
*  If true, developers can detect the user editing the value 
*  via cut or paste interactions (triggered from keyboard shortcuts or the native 
*  browser menu options) using the FormItem.isCutEvent and
*  FormItem.isPasteEvent methods. This allows custom cut/paste handling
*  to be added to the various change notification flow methods including 
*  FormItem.change, and
*  FormItem.transformInput.
* @type {boolean}
* @default true
*/
supportsCutPasteEvents?: boolean;

/**
* If this form item has a specified FormItem.optionDataSource, should the
*  item ever perform a fetch against this dataSource to retrieve the related record.
*  
*  Note that for editable textItems, behavior differs slightly than for other
*  item types as we will not issue fetches unless FormItem.alwaysFetchMissingValues has
*  been set to true. 
*  See TextItem.shouldFetchMissingValue for more details.
* @type {boolean}
* @default true
*/
fetchMissingValues?: boolean;

/**
* If FormItem.showHint, should the hint be shown
*  within the field?
*  
*  Unless the HTML5 placeholder attribute is used to display the hint
*  (see TextItem.usePlaceholderForHint),
*  the value of the &lt;input&gt; element for this item will be set to the hint
*  whenever this item is not focused. Also, when displaying the hint, the CSS style of the
*  data element will be set to the TextItem.textBoxStyle with the suffix
*  "Hint" appended to it; or, if the item is disabled, the suffix "DisabledHint" will be used.
*  In Page.isRTL when FormItem.showRTL is true,
*  an additional "RTL" suffix will be appended; i.e. the CSS style of the data element when
*  the hint is displayed will be the textBoxStyle plus "HintRTL" or 
*  "DisabledHintRTL".
*  
*  To change this attribute after being drawn, it is necessary to call FormItem.redraw
*  or redraw the form.
*  
*  Styling the in-field hint
*  The in-field hint can be styled with CSS for the textBoxStyle + "Hint" /
*  "HintRTL" / "DisabledHint" / "DisabledHintRTL" styles. For example, if this item's
*  textBoxStyle is set to "mySpecialItem", then changing the hint color to
*  blue can be accomplished with the following CSS:
*  .mySpecialItemHint,
* .mySpecialItemHintRTL,
* .mySpecialItemDisabledHint,
* .mySpecialItemDisabledHintRTL {
*   color: blue;
* }
*  
*  In DynamicForm.linearMode, this property will be defaulted true if left unset.
* @type {boolean}
* @default null
*/
showHintInField?: boolean;

/**
* If TextItem.showHintInField and if supported by the browser, should the HTML5
*  placeholder attribute
*  be used to display the hint within the field? If set to false, then use of
*  the placeholder attribute is disabled and an alternative technique to display
*  the hint in-field is used instead.
*  
*  The HTML5 placeholder attribute is supported in the following major browsers:
*  
*  
* - Chrome 4+
*  
* - Firefox 4+
*  
* - Internet Explorer 10+
*  
* - Safari 5+
*  
* - Opera 11.50+
*  
* - Android 2.1+ WebView (used by the stock Browser app and when
*    phonegapIntegration)
*  
* - Mobile Safari for iOS 3.2+
*  
*  
*  In browsers other than the above, in-field hints are implemented via a different technique.
*  
*  Note that placeholder behavior is known to differ in Internet Explorer and certain old
*  versions of the above browsers due to a recent change in the HTML5 specification regarding
*  the placeholder attribute. Under the old rules, the placeholder is cleared
*  when the element is focused. In the latest HTML5 spec as published by WHATWG, the placeholder
*  is still displayed when the element is focused as long as the value is an empty string.
*  
*  Styling the placeholder
*  While there isn't a standard way to style the placeholder text, Chrome, Firefox,
*  Internet Explorer, and Safari provide vendor-prefixed pseudo-classes and/or pseudo-elements
*  that can be used in CSS selectors:
*  
*  
*   Browser
*   Pseudo-class or pseudo-element
*  
*  
*   Chrome, Safari
*   ::-webkit-input-placeholder
*  
*  
*   Firefox 4 - 18
*   :-moz-placeholder
*  
*  
*   Firefox 19+
*   ::-moz-placeholder
*  
*  
*   Internet Explorer
*   :-ms-input-placeholder
*  
*  
*  
*  Note that unlike other browsers, Firefox 19+ applies opacity:0.4 to the placeholder text.
*  See Bug 556145 - Placeholder text default style should use opacity instead of GrayText
*  
*  Because browsers are required to ignore the entire rule if a selector is invalid,
*  separate rules are needed for each browser. For example:
*  ::-webkit-input-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
* :-moz-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
* ::-moz-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
* :-ms-input-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
*  
*  If using Sass, it may be useful to utilize Sass'
*  parent selector feature.
*  For example:
*  .myCustomItem,
* .myCustomItemRTL,
* .myCustomItemDisabled,
* .myCustomItemDisabledRTL,
* .myCustomItemError,
* .myCustomItemErrorRTL,
* .myCustomItemFocused,
* .myCustomItemFocusedRTL,
* .myCustomItemHint,
* .myCustomItemHintRTL,
* .myCustomItemDisabledHint,
* .myCustomItemDisabledHintRTL {
*   // ...
* 
*   &amp;::-webkit-input-placeholder {
*     color: blue;
*     opacity: 1;
*   }
*   &amp;:-moz-placeholder {
*     color: blue;
*     opacity: 1;
*   }
*   &amp;::-moz-placeholder {
*     color: blue;
*     opacity: 1;
*   }
*   &amp;:-ms-input-placeholder {
*     color: blue;
*     opacity: 1;
*   }
* &#125;
*  
*  If using Compass, the
*  input-placeholder mixin
*  that was added in version 1.0 can further simplify the code to style the placeholder text
*  For example:
*  .myCustomItem,
* .myCustomItemRTL,
* .myCustomItemDisabled,
* .myCustomItemDisabledRTL,
* .myCustomItemError,
* .myCustomItemErrorRTL,
* .myCustomItemFocused,
* .myCustomItemFocusedRTL,
* .myCustomItemHint,
* .myCustomItemHintRTL,
* .myCustomItemDisabledHint,
* .myCustomItemDisabledHintRTL {
*   // ...
* 
*   &#64;include input-placeholder {
*     color: blue;
*     opacity: 1;
*   }
* &#125;
*  Accessibility concerns
*  The HTML5 specification notes that a placeholder should not be used as a replacement
*  for a title. The placeholder is intended to be a short hint that assists the user
*  who is entering a value into the empty field. The placeholder can be mistaken by some
*  users for a pre-filled value, particularly in Internet Explorer because the same color
*  is used, and the placeholder text color may provide insufficient contrast, particularly
*  in Firefox 19+ because of the default 0.4 opacity. Furthermore, not having a title reduces
*  the hit area available for setting focus on the item.
* @type {boolean}
* @default true
*/
usePlaceholderForHint?: boolean;

/**
* When generating a print-view of the component containing this TextItem, should
*  the form item expand to accommodate its value? If set to false the text box will not expand
*  to fit its content in the print view, instead showing exactly as it does in the
*  live form.
* @type {boolean}
* @default false
*/
printFullText?: boolean;

/**
* Text items will submit their containing form on enter keypress 
*  if DynamicForm.saveOnEnter is true. Setting this property to
*  false will disable this behavior.
* @type {boolean}
* @default true
*/
saveOnEnter?: boolean;

/**
* By default HTML characters will be escaped when FormItem.canEdit is false and 
*  FormItem.readOnlyDisplay is "static", so that the raw value of the field (for
*  example "&lt;b&gt;AAA&lt;/b&gt;") is displayed to the user rather than
*  the interpreted HTML (for example "AAA"). Setting
*  escapeHTML false will instead force HTML values in a textItem to be
*  interpreted by the browser in that situation.
* @type {boolean}
* @default true
*/
escapeHTML?: boolean;

/**
* When set to false, prevents this item's input element from being written into the DOM.
*  If there are FormItem.valueIcons or a 
*  FormItem.showPickerIcon, these are displayed as normal, and the item
*  will auto-sizing to that content if its FormItem.width is set to null.
* @type {boolean}
* @default true
*/
showInputElement?: boolean;

/**
* This attribute currently only has an effect in Internet Explorer.
*  That browser will dynamically add a native "clear" icon to
*  &lt;input type="text" &gt; elements when the user enters a value.
*  Setting suppressBrowserClearIcon to true
*  will write out HTML to suppress this icon. This can be particularly useful for
*  items which define their own clear icon as in 
*  ${isc.DocUtils.linkForExampleId('inlineFormIcons', 'this sample')}.
*  
*  If this property is not set at the item level,
*  DynamicForm.suppressBrowserClearIcons will be used instead.
*  
*  Note that as an alternative to using this feature, the icon may also be suppressed 
*  (or have other styling applied to it) directly via CSS, using the 
*  ::-ms-clear css pseudo-element (proprietary Internet Explorer feature).
*  
*  Implementation note: This feature makes use of the automatically generated
*  TextItem.suppressClearIconClassName css class.
* @type {boolean}
* @default null
*/
suppressBrowserClearIcon?: boolean;

/**
* Should entered characters be converted to upper or lowercase?
*  Also applies to values applied with FormItem.setValue.
*  
*  Note: character casing cannot be used at the same time as a TextItem.mask.
* @type {CharacterCasing}
* @default "default"
*/
characterCasing?: CharacterCasing;

/**
* Sets a keypress filter regular expression to limit valid characters
*  that can be entered by the user. If defined, keys that match the
*  regular expression are allowed; all others are suppressed. The
*  filter is applied after character casing, if defined.
*  
*  Note: keypress filtering cannot be used at the same time as a TextItem.mask.
* @type {string}
* @default null
*/
keyPressFilter?: string;

/**
* With formatOnBlur enabled, this textItem will format its value
*  according to the rules described in FormItem.mapValueToDisplay as long as the 
*  item does not have focus. Once the user puts focus into the item
*  the formatter will be removed. This provides a simple way for developers to
*  show a nicely formatted display value in a freeform text field, without the need
*  for an explicit FormItem.formatEditorValue 
*  and FormItem.parseEditorValue pair.
* @type {boolean}
* @default false
*/
formatOnBlur?: boolean;

/**
* Should FormItem.formatEditorValue re-run whenever this item recieves or loses focus?
*  Setting this property allows developers to conditionally format the display value based on
*  item.hasFocus, typically to display a longer, more informative string while the item does
*  not have focus, and simplifying it down to an easier-to-edit string when the user puts
*  focus into the item.
* @type {boolean}
* @default false
*/
formatOnFocusChange?: boolean;

/**
* Input mask used to restrict and format text within this item.
*  
*  Overview of available mask characters
*  
*  
*  CharacterDescription
*  0Digit (0 through 9) or plus [+] or minus [-] signs
*  9Digit or space
*  #Digit
*  LLetter (A through Z)
*  ?Letter (A through Z) or space
*  ALetter or digit
*  aLetter or digit
*  CAny character or space
*  &nbsp;
*  &lt;Causes all characters that follow to be converted to lowercase
*  &gt;Causes all characters that follow to be converted to uppercase
*  &nbsp;
*  [ ... ]Square brakets denote the start and end of a custom 
*    regular expression character set or range.
*  
*  
*  The mask can also contain literals - arbitrary non editable characters 
*  to be displayed as part of the formatted text. Any character not matching one
*  of the above mask characters will be considered a literal. To use one of the
*  mask characters as a literal, it must be escaped with a pair of backslashes (\\). By 
*  default literals are formatting characters only and will not be saved as part of the 
*  item's value. This behavior is controlled via TextItem.maskSaveLiterals.
*  
*  When a TextItem with a mask has focus, the formatted mask string will be displayed, 
*  with the TextItem.maskPromptChar displayed as a placeholder for characters that have not yet
*  been entered.
*  As the user types in the field, input will be restricted to the appropriate character class
*  for each character, with uppercase/lowercase conversion occurring automatically. When focus
*  is moved away from the field, the displayed value will be formatted to include any literals
*  in the appropriate places.
*  
*  Sample masks:
*  
*  
* - Phone number: (###) ###-####
*  
* - Social Security number: ###-##-####
*  
* - First name: &gt;?&lt;??????????
*  
* - Date: ##/##/####
*  
* - State: &gt;LL
*  
*  
*  Custom mask characters can be defined by standard regular expression character set
*  or range. For example, a hexadecimal color code mask could be:
*  
*  
* - Color: \\#&gt;[0-9A-F][0-9A-F][0-9A-F][0-9A-F][0-9A-F][0-9A-F]
*  
*  
*  Note: input mask cannot be used at the same time as a TextItem.keyPressFilter.
*  Also note that this property is not supported for
*  ComboBoxItem or SpinnerItem, or for items with TextItem.browserInputType
*  set to "digits" or "number".
* @type {string}
* @default null
*/
mask?: string;

/**
* During entry into a TextItem.mask, should keystrokes overwrite current position value?
*  By default new keystrokes are inserted into the field.
* @type {boolean}
* @default null
*/
maskOverwriteMode?: boolean;

/**
* Should entered TextItem.mask value be saved with embedded literals?
* @type {boolean}
* @default null
*/
maskSaveLiterals?: boolean;

/**
* Character that is used to fill required empty TextItem.mask positions
*  to display text while control has no focus.
* @type {string}
* @default " "
*/
maskPadChar?: string;

/**
* Character that is used to fill required empty TextItem.mask positions
*  to display text while control has focus.
* @type {string}
* @default "_"
*/
maskPromptChar?: string;



/**
* 	Returns the hint text for this item. Default implementation returns FormItem.hint, or
*  null if there is no hint to show.
* 
* 
*/
getHint(): HTMLString;

/**
*  For text-based items, this method returns the indices of the start/end of the current
*  selection if the item currently has the focus. In browsers other than Internet Explorer 6-9,
*  if this item does not have focus, then this method returns the indices of the start/end
*  of the selection the last time that this item had focus. In IE 6-9, returns null if the
*  item does not have focus.
*  
*  In all browsers, clicking anywhere outside of the item causes the item to lose focus;
*  hence, in IE 6-9, this method will not work in other components' event handlers for
*  certain events. For example, within the Canvas.click handler of a button,
*  this item will have already lost focus, so in IE 6-9, this method will return null
*  if called within the button's click() handler. One cross-browser solution to this issue
*  is to save the selection range for later in a Canvas.mouseDown or
*  Canvas.mouseOver handler.
*  
*  Notes:
*  
*   
* - In browsers other than IE 6-9, calling FormItem.setValue
*  or otherwise changing the TextItem.getEnteredValue invalidates the past
*  selection range.
*   
* - The returned indices are indices within the entered value rather than the item's
*  value as returned by FormItem.getValue.
*  The distinction is particularly important for TextAreaItems because browsers
*  normalize the line endings in the &lt;textarea&gt; element's value.
*  Internet Explorer 6, 7, and 8 convert line endings to "\r\n" while other browsers
*  convert line endings to "\n"
*  as specified by the HTML5 standard.
*  
* 
*/
getSelectionRange(): Array<Partial<Integer>>;

/**
*  Puts focus into this form item and selects characters between the given indices.
*  Only applies to drawn text based items.
* 
* @param {Integer} selection starting character index
* @param {Integer} end of selection character index
*/
setSelectionRange(start:Integer, end:Integer): void;

/**
*  Put focus in this item and select the entire value.
*  Only applies to text based items
* 
*/
selectValue(): void;

/**
*  If this item currently has focus, clear the current selection. leaving focus in the item.
*  Has no effect if the item is undrawn or unfocused.
*  Only applies to text-based items.
* 
* @param {boolean=} By default the text insertion cursor will be moved to the end of the  current value - pass in this parameter to move to the start instead
*/
deselectValue(start?:boolean): void;

/**
*  If this field has a specified FormItem.optionDataSource, should we perform a fetch against
*  that dataSource to find the record that matches this field's value?
*  
*  For textItems this method will return false if the item is
*  FormItem.canEdit unless FormItem.alwaysFetchMissingValues is true, even
*  if there is a specified FormItem.displayField. 
*  We do this as, for a freeform text-entry field with a specified displayField, the
*  correct behavior when the user enters an unrecognized value is somewhat ambiguous.
*  The user could have entered a complete display-field value, in which case it
*  might be appropriate to issue a fetch against the display-field of the optionDataSource, 
*  and set the underlying item value.
*  If a match was not found though, we necessarily treat the entered value as the new "dataValue"
*  for the field. Should we then issue a second fetch against the optionDataSource comparing
*  the user-entered value with the value-field of the dataSource?
*  
*  There are still cases where it could make sense to issue the fetch against the dataSource,
*  and developers who want this behavior can set FormItem.alwaysFetchMissingValues to true.
*  
*  See FormItem.shouldFetchMissingValue for how this method behaves for other
*  item types.
* 
* 
* @param {any} The new data value of the item.
*/
shouldFetchMissingValue(newValue:any): boolean;

/**
*  Setter for the TextItem.suppressBrowserClearIcon
* 
* @param {boolean} new value for suppressBrowserClearIcon
*/
setSuppressBrowserClearIcon(newValue:boolean): void;

/**
*  Returns the raw text value that currently appears in the text field, which can differ from 
*  FormItem.getValue in various cases - for example:
*  
*  
* - for items that constrain the value range, such as a DateItem with
*  DateItem.enforceDate:true, or a ComboBoxItem with
*  ComboBoxItem.addUnknownValues:false
*  
* - for items with a defined valueMap or edit value formatter and parser functions
*  which converts display value to data value
*  
* - while the item has focus if TextItem.changeOnKeypress is false
*  
* 
*/
getEnteredValue(): string;

/**
*  Set the TextItem.keyPressFilter for this item
* 
* @param {string} new keyPress filter for the item
*/
setKeyPressFilter(filter:string): void;

/**
*  Notification fired in response to a clipboard "paste" event on freeform text
*  items, giving developers an opportunity to reformat the pasted text. The 
*  pastedValue argument contains the text pasted from the clipboard.
*  This method should return the text value to actually insert into the input element.
* 
* 
* @param {FormItem} Item into which the user pasted a value
* @param {DynamicForm} Pointer to the item's form
* @param {string} Pasted text value
*/
transformPastedValue(item:FormItem, form:DynamicForm, pastedValue:string): string;

/**
*  Set the TextItem.mask for this item. Pass null to clear an existing mask.
*  
*  Note that the current value of the field is cleared when changing the mask.
* 
* @param {string} mask to apply to text item
*/
setMask(mask:string): void;

/**
*  Notification method called when FormItem.showPending is enabled and
*  this text item should either clear or show its pending visual state.
*  
*  The default behavior is that the FormItem.titleStyle,
*  FormItem.cellStyle, and TextItem.textBoxStyle are
*  updated to include/exclude the "Pending" suffix. Returning false will cancel
*  this default behavior.
* 
* @param {DynamicForm} the managing DynamicForm instance.
* @param {FormItem} the form item itself (also available as "this").
* @param {boolean} true if the item should show its pending visual state; false otherwise.
* @param {any} the current form item value.
* @param {any} the value that would be restored by a call to DynamicForm.resetValues.
*/
pendingStatusChanged(form:DynamicForm, item:FormItem, pendingStatus:boolean, newValue:any, value:any): boolean;



}
/**
* Creates a new TextItem
* 
* @param typeCheckedProperties {Partial<TextItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TextItem} The new instance
*/
class TextItem{
static create(typeCheckedProperties? : Partial<TextItem>, uncheckedProperties?: Object): TextItem;
/**
* A declared value of the enum type 
*  CharacterCasing.
* @type {Constant}
* @default "default"
*/
static DEFAULT?: Constant;

/**
* A declared value of the enum type 
*  CharacterCasing.
* @type {Constant}
* @default "upper"
*/
static UPPER?: Constant;

/**
* A declared value of the enum type 
*  CharacterCasing.
* @type {Constant}
* @default "lower"
*/
static LOWER?: Constant;

/**
* Reserved css className applied to TextItem &lt;input &gt; elements where 
*  TextItem.suppressBrowserClearIcon is true.
*  
*  Note that this is an advanced property and in most cases developers do not need to
*  be aware of it. The style definition for this style-name will be 
*  generated automatically with appropriate css content to suppress the browser
*  clear icon. It is documented only so that developers are aware that this css className
*  (and can choose a different name in the extremely rare case where this name has
*  meaning within their application code).
* @type {string}
* @default "isc_suppressClearIcon"
*/
static suppressClearIconClassName?: string;








}

/**
* EditProxy that handles TextItem
*  and StaticTextItem when editMode is enabled.
*/
interface TextItemEditProxy extends FormItemEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's defaultValue.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's defaultValue.
* 
* 
* @param {string} the new component defaultValue
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new TextItemEditProxy
* 
* @param typeCheckedProperties {Partial<TextItemEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TextItemEditProxy} The new instance
*/
class TextItemEditProxy{
static create(typeCheckedProperties? : Partial<TextItemEditProxy>, uncheckedProperties?: Object): TextItemEditProxy;







}

/**
* Common base class of TextImportSettings.
*/
interface TextSettings extends Class {
/**
* For export, a set of fields to export. Default is to export all DataSource fields. 
*  
*  Fields may be specified that are not in the DataSource but for which data values are present
*  in the provided Records. In this case the field is assumed to be of type "text".
*                 
*  For import, names of DataSource fields to use to parse values, in order.
*  
*  If fieldList is unset, DataSource fields are used, in order.
*  
*  If more values exist in a given Record than the listed fields or than all DataSource fields,
*  remaining values are ignored.
* @type {Array<Partial<string>>}
* @default null
*/
fieldList?: Array<Partial<string>>;

/**
* Separator between field values. Default is a comma character, producing CSV 
*  (comma-separated values) format.
* @type {string}
* @default ","
*/
fieldSeparator?: string;

/**
* Separator between Records. For import, default of null means that either the Unix/Mac
*  format of just a newline ("\n") or the typical DOS/Windows format of a carriage return
*  and newline ("\r\n") will be accepted. For export, overridden in TextExportSettings.
* @type {string}
* @default null
*/
lineSeparator?: string;

/**
* EscapingMode expected for escaping special characters embedded in text values.
* @type {EscapingMode}
* @default "backslash"
*/
escapingMode?: EscapingMode;





}
/**
* Creates a new TextSettings
* 
* @param typeCheckedProperties {Partial<TextSettings>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TextSettings} The new instance
*/
class TextSettings{
static create(typeCheckedProperties? : Partial<TextSettings>, uncheckedProperties?: Object): TextSettings;







}

/**
* A TileGrid is a DataBoundComponent that displays a list of objects as a set
*  of "tiles", where each tile represents one object, and the tiles are laid out in a grid with
*  multiple tiles per row. Each tile displays one or more properties of the object it
*  represents.
*/
interface TileGrid extends TileLayout, Omit<DataBoundComponent, 'fields'>  {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "GridEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Array of field definitions to control the default rendering of tiles.
*  
*  If not specified, if the DataSource has an DataSource.iconField, only the
*  iconField and DataSource.titleField will be shown.
*  Otherwise, all non-DataSourceField.hidden
*  non-DataSourceField.detail fields will be shown, similar to the default set of
*  fields shown by a ListGrid.
*  
*  Only applicable if using the default SimpleTile class for tiles. 
*  (See TileGrid.tile for more information).
*  
*  For SimpleTiles, it is possible to use DetailViewerField.getCellStyle and 
*  StatefulCanvas.getStateSuffix to make a single field statefully styled:
*  
*  
*  isc.TileGrid.create({
*    fields:[
*      {name:'animalName',
*       getCellStyle : function (value, field, record, viewer) {
*         if (value == "Tiger") return "tigerStyle" + viewer.currentTile.getStateSuffix();
*         else return viewer.tileGrid.tileValueStyle + viewer.currentTile.getStateSuffix();
*       }
*      }
*    ]
*  });
*  
*  
*  
* @type {Array<Partial<DetailViewerField>>}
* @default null
*/
fields?: Array<Partial<DetailViewerField>>;

/**
* List of tiles that may be used by the TileGrid to show its current data set. Note that the
*  SmartClient framework manages this array for optimal performance, and not all tiles in the
*  array are necessarily visible or assigned a record. This is true regardless of whether
*  TileGrid.recycleTiles is set or not.
*  
*  The number of records in the TileGrid's current TileGrid.data
*  may be determined by calling List.getLength on it.
* @type {Array<Partial<Canvas>>}
* @default null
*/
tiles?: Array<Partial<Canvas>>;

/**
* By default, TileGrids will not show fields marked DataSourceField.detail
*  in the DataSource. See also TileGrid.fields.
* @type {boolean}
* @default false
*/
showDetailFields?: boolean;

/**
* When using the default SimpleTile, CSS style for each value shown within a tile.
* @type {CSSStyleName}
* @default "tileValue"
*/
tileValueStyle?: CSSStyleName;

/**
* Should tile values change state when the mouse goes over them?
* @type {boolean}
* @default false
*/
valuesShowRollOver?: boolean;

/**
* Should tile values change state when they are selected?
* @type {boolean}
* @default true
*/
valuesShowSelected?: boolean;

/**
* Should tile values change state when the mouse goes down on them?
* @type {boolean}
* @default false
*/
valuesShowDown?: boolean;

/**
* Horizontal alignment for tile values: "left", "right" or "center".
* @type {string}
* @default "center"
*/
tileValueAlign?: string;

/**
* Whether values should be allowed to wrap by default, or should be shown on one line
*  regardless of length.
* @type {boolean}
* @default false
*/
wrapValues?: boolean;

/**
* A List of TileRecord objects, specifying the data to be used to create the
*  tiles. 
*  
*  This property will typically not be explicitly specified for databound TileGrids, where
*  the data is returned from the server via databound component methods such as
*  TileGrid.fetchData. In this case the data objects will be set to a 
*  ResultSet rather than a simple array.
* @type {Array<Partial<Record>> | Array<Partial<TileRecord>> | any}
* @default null
*/
data?: Array<Partial<Record>> | Array<Partial<TileRecord>> | any;

/**
* How many tiles should be present in a line when printing?
* @type {number}
* @default null
*/
printTilesPerLine?: number;

/**
* The DataSource that this component should bind to for default fields and for performing
*  DSRequest.
*  
*  Can be specified as either a DataSource instance or the String ID of a DataSource.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* A TileGrid is a DataBoundComponent.dataArity:multiple component.
* @type {string}
* @default "multiple"
*/
dataArity?: string;

/**
* If true, when this component is first drawn, automatically call this.fetchData().
*  Criteria for this fetch may be picked up from TileGrid.initialCriteria, and textMatchStyle may
*  be specified via ListGrid.autoFetchTextMatchStyle.
*  
*  NOTE: if autoFetchData is set, calling
*  ListGrid.fetchData before draw will cause two requests to be issued, one from the manual
*  call to fetchData() and one from the autoFetchData setting. The second request will use
*  only TileGrid.initialCriteria and not any other criteria or settings from the first request.
*  Generally, turn off autoFetchData if you are going to manually call ListGrid.fetchData at any time.
* 
*  Note: If you are using saved searches - either via SavedSearchItem or ListGrid.saveDefaultSearch, autoFetchData
*  will be automatically suspended and replaced with the saved criteria/view state, if applicable.
* @type {boolean}
* @default false
*/
autoFetchData?: boolean;

/**
* If TileGrid.autoFetchData is true, this attribute allows the developer to
*  specify a textMatchStyle for the initial ListGrid.fetchData call.
* @type {TextMatchStyle}
* @default "substring"
*/
autoFetchTextMatchStyle?: TextMatchStyle;

/**
* Criteria to be used when TileGrid.autoFetchData is set.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
initialCriteria?: Criteria;

/**
* Criteria that are never shown to or edited by the user and are cumulative with any criteria
*  provided via DataBoundComponent.initialCriteria and related methods.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
implicitCriteria?: Criteria;

/**
* How to fetch and manage records retrieve from the server. See FetchMode.
*  
*  This setting only applies to the ResultSet automatically created by calling
*  ListGrid.fetchData. If a pre-existing ResultSet is passed to setData() instead, it's
*  existing setting for ResultSet.fetchMode applies.
* @type {FetchMode}
* @default "paged"
*/
dataFetchMode?: FetchMode;

/**
* Defines a tileGrid's clickable-selection behavior.
* @type {SelectionStyle}
* @default "multiple"
*/
selectionType?: SelectionStyle;

/**
* A TileGrid automatically creates one tile per record in the dataset, via the
*  AutoChild pattern.
*  
*  By default, the SimpleTile class will be used. This class automatically invokes
*  TileGrid.getTileHTML on the tileGrid to generate its content. The standard TileGrid.getTileHTML
*  method uses a TileGrid.detailViewer to render html for the tile's record, based on the 
*  provided TileGrid.fields (or on the default set of fields).
*  
*  
*  To create a completely different appearance, override TileGrid.tileConstructor with the name
*  of the custom SmartClient class to use for each tile. For example, subclass
*  SimpleTile and override Canvas.getInnerHTML, returning custom
*  HTML for each tile.
*  
*    isc.defineClass("MyCustomTile", "SimpleTile").addProperties({
*     getInnerHTML : function () {
*       return this.Super("getInnerHTML", arguments) +
*         this.getRecord().width + " x " + this.getRecord().height;
*     }
*    });
* 
*    isc.TileGrid.create({
*     tileConstructor:"MyCustomTile"
*    });
*  
*  
*  Note that you can also override tile behaviors on a per-record basis, via
*  TileRecord.tileConstructor and TileRecord.tileProperties.
*  
*  
* @type {SimpleTile}
* @default null
*/
tile?: SimpleTile;

/**
* Common properties to use when creating every tile.
* @type {Partial<Canvas>}
* @default null
*/
tileProperties?: Partial<Canvas>;

/**
* Class-name of a SmartClient component to use for each tile rendered by this TileGrid. Tiles
*  are created by the AutoChild pattern; see TileGrid.tile.
*  
*  Any subclass of Canvas is allowed, but typically any custom class will derive from
*  SimpleTile.
*  
*  When using a custom component for tileConstructor, DataBoundComponents that display multiple
*  Records (ListGrid, DetailViewer) will have data provided via ListGrid.setData, and
*  components that display a single Record (DynamicForm) will have
*  DynamicForm.setValues called on them.
*  
*  If the component is not a recognized DataBoundComponent subclass, the Record can be
*  accessed via 
*  this.record.
*  
*  
*  If you implement particularly simple or particularly complex tile interfaces, you may wish
*  to adjust the property TileGrid.drawAllMaxTiles.
* @type {SCClassName}
* @default "SimpleTile"
*/
tileConstructor?: SCClassName;

/**
* Automatically genereated DetailViewer instance used to render the content shown in Tiles by default.
*  
*  This detailViewer is never actually drawn or displayed to the user - it is simply used to generate
*  the contents of SimpleTiles as described in TileGrid.getTileHTML.
* @type {DetailViewer}
* @default null
*/
detailViewer?: DetailViewer;

/**
* Properties for the DetailViewer that is automatically created to render the contents of tiles
*  by default.
* @type {Partial<DetailViewer>}
* @default null
*/
detailViewerProperties?: Partial<DetailViewer>;

/**
* This property determines whether tiles that are no longer visible (due to scrolling) are
*  recycled, allowing a large number of records to be displayed using a (potentially) much
*  smaller set of tiles.
*  
*  Recycling tiles may significantly reduce the number of live tile widgets needed to support
*  a particular TileGrid, but may also result in extra work when the TileGrid is scrolled, as
*  a scroll that brings off-screen tiles into view will require recycling tiles that have
*  left the view, even if the new tiles have been visited before (in previous scrolling).
*  
*  Recycling will occur when TileGrid.getTile is called, unless the supplied record
*  (or record specified by index) is currently bound to an existing tile. Even if recycling
*  is not enabled, the record associated with a given tile may change if the TileGrid data
*  changes.
*  
*  For more control over the tile creation and recycling process, 
*  see TileGrid.createTile and TileGrid.updateTile.
* @type {boolean}
* @default true
*/
recycleTiles?: boolean;

/**
* Whether tiles are created and drawn for all records, or only for those currently visible.
*  
*  This setting is incompatible with TileGrid.dataFetchMode: "paged" as it requires all
*  records matching the criteria to be fetched from the server at once.
* @type {boolean}
* @default false
*/
showAllRecords?: boolean;

/**
* The string to display in the body of a tileGrid while data is being loaded.
*  Use "&#36;{loadingImage}" to include Canvas.loadingImageSrc.
* @type {HTMLString}
* @default "${loadingImage}&amp;nbsp;Loading data..."
*/
loadingDataMessage?: HTMLString;

/**
* The CSS style name applied to the loadingDataMessage string if displayed.
* @type {CSSStyleName}
* @default "loadingDataMessage"
*/
loadingDataMessageStyle?: CSSStyleName;

/**
* If drawing all tiles would cause no more than drawAllMaxTiles tiles to be
*  rendered, the full dataset will instead be drawn even if TileGrid.showAllRecords
*  is false and incremental rendering would have otherwise been used.
*  
*  The drawAllMaxTiles setting prevents incremental rendering from being used in
*  situations where it's really unnecessary, such as a 25 record dataset which happens to be
*  in a grid with a viewport showing only 15 or so tiles. Incremental rendering causes a brief
*  "flash" during scrolling as the visible portion of the dataset is redrawn, and a better 
*  scrolling experience can be obtained in this situation by drawing the entire dataset up
*  front, which in this example would have negligible effect on initial draw time.
*  
*  drawAllMaxTiles:0 disables this features. You may want to disable this feature
*  if performance is an issue and:
*  
*  
* - you very frequently redraw a grid
*  
* - you do a lot of computation when rendering each tile
*  
* - you are showing many grids on one screen and the user won't scroll most of them
*  
* @type {Integer}
* @default "25"
*/
drawAllMaxTiles?: Integer;

/**
* If set, when the dataset changes due to filtering, sorting or other actions, any tiles that
*  were showing before and after the change will animate from their old positions to their new
*  positions.
* @type {boolean}
* @default true
*/
animateTileChange?: boolean;

/**
* Style for the overall TileGrid component.
* @type {CSSStyleName}
* @default "tileGrid"
*/
styleName?: CSSStyleName;

/**
* Screen to create (via RPCManager.createScreen) for the tile
*  in lieu of calling TileGrid.getTile.
*  
*  If this grid has a DataBoundComponent.dataSource, the created screen is
*  provided with a Canvas.dataContext that includes the record being expanded.
*  Be sure the tile screen meets these Canvas.autoPopulateData
*  to utilize the dataContext.
* @type {string}
* @default null
*/
tileScreen?: string;

/**
* If true, when an update operation occurs on a selected tile's record in a
*  TileGrid.dataSource tileGrid, ensure the updated tile is
*  re-selected when the operation completes.
*  The TileGrid.reselectOnUpdateNotifications attributes governs whether
*  DataBoundComponent.selectionUpdated and
*  ListGrid.selectionChanged will fire when this occurs.
* @type {boolean}
* @default true
*/
reselectOnUpdate?: boolean;

/**
* if TileGrid.reselectOnUpdate is true, this property governs what selection changed
*  notifications should be triggered when a selected tile's record is edited then automatically
*  reselected when the edited data is merged into the data set.
* @type {SelectionNotificationType}
* @default "selectionChanged"
*/
reselectOnUpdateNotifications?: SelectionNotificationType;

/**
* Visual appearance to show when the tile is being dragged.
* @type {DragAppearance}
* @default "tracker"
*/
tileDragAppearance?: DragAppearance;

/**
* Indicates whether tiles can be reordered by dragging within this TileGrid.
*  
*  NOTE: If canReorderTiles is initially enabled or might be
*  TileGrid.setCanReorderTiles after the grid is created,
*  it may be desirable to disable Canvas.useTouchScrolling
*  so that touch-dragging a tile starts a reorder operation rather than a scroll. If
*  Canvas.disableTouchScrollingForDrag is set to true, then touch
*  scrolling will be disabled automatically. However, for accessibility
*  reasons, it is recommended to leave touch scrolling enabled and provide an alternative
*  set of controls that can be used to perform drag-reordering of tiles.
* @type {boolean}
* @default false
*/
canReorderTiles?: boolean;

/**
* Indicates whether tiles can be dragged from this TileGrid and dropped elsewhere.
*  
*  NOTE: If canDragTilesOut is initially enabled or might be
*  TileGrid.setCanDragTilesOut after the grid is created,
*  it may be desirable to disable Canvas.useTouchScrolling
*  so that touch-dragging a tile starts a drag operation rather than a scroll. If
*  Canvas.disableTouchScrollingForDrag is set to true, then touch
*  scrolling will be disabled automatically. However, for accessibility
*  reasons, it is recommended to leave touch scrolling enabled and provide an alternative
*  set of controls that can be used to perform drag and drop of tiles out of the grid.
* @type {boolean}
* @default false
*/
canDragTilesOut?: boolean;

/**
* Indicates whether records can be dropped into this TileGrid.
* @type {boolean}
* @default false
*/
canAcceptDroppedRecords?: boolean;

/**
* CSS Style to apply to the drag tracker when dragging occurs on this component.
* @type {CSSStyleName}
* @default "gridDragTracker"
*/
dragTrackerStyle?: CSSStyleName;

/**
* Indicates what to do with data dragged into another DataBoundComponent. See
*  DragDataAction type for details.
* @type {DragDataAction}
* @default "move"
*/
dragDataAction?: DragDataAction;

/**
* If you have a databound tileGrid and you scroll out of the currently loaded dataset, by
*  default you will see blank tiles until the server returns the data for those rows. The
*  loadingMessage attribute allows you to specify arbitrary html that will be shown in each
*  such "blank" tile while the data for that tile is loading.
*  (e.g. "&lt;DIV ALIGN='CENTER'&gt;LOADING&lt;/DIV&gt;")
* @type {HTMLString}
* @default null
*/
loadingMessage?: HTMLString;

/**
* Indicates whether the text of the emptyMessage property should be displayed if no data is
*  available.
* @type {boolean}
* @default true
*/
showEmptyMessage?: boolean;

/**
* The string to display in the body of a tileGrid with an empty data array, if
*  TileGrid.showEmptyMessage is true.
* @type {HTMLString}
* @default "No tiles to show."
*/
emptyMessage?: HTMLString;

/**
* The CSS style name applied to the TileGrid.emptyMessage if displayed.
* @type {CSSStyleName}
* @default "emptyMessage"
*/
emptyMessageStyle?: CSSStyleName;



/**
*  Sends the current filter criteria and sort direction to the server, then exports data in the
*  requested DSRequest.exportAs.
*  
*  A variety of DSRequest settings, such as 
*  DSRequest.exportAs and DSRequest.exportFilename, affect the 
*  exporting process: see DSRequest.exportResults for further detail.
*  
*  Note that data exported via this method skips client-side fields defined only in
*  the component, excludes any client-side formatting and
*  relies on both the SmartClient server and server-side DataSources. To export client-data 
*  including client-only fields and with client-side formatting applied, 
*  see ListGrid.exportClientData, which still requires the
*  SmartClient server but does not rely on server-side DataSource definitions (.ds.xml files).
*  
*  For more information on exporting data, see DataSource.exportData.
* 
* 
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
* @param {DSCallback=} callback to invoke on completion. Note that this parameter                 only applies where DSRequest.exportToClient is                  explicitly set to false, because file downloads do not                  provide ordinary SmartClient callbacks
*/
exportData(requestProperties?:DSRequest, callback?:DSCallback): void;

/**
*  Retrieves data from the DataSource that matches the specified criteria.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
*  
*  When fetchData() is first called, if data has not already been provided via
*  ListGrid.setData, this method will create a ResultSet, which will be
*  configured based on component settings such as DataBoundComponent.fetchOperation
*  and DataBoundComponent.dataPageSize, as well as the general purpose
*  ListGrid.dataProperties. The created ResultSet will automatically send a DSRequest
*  to retrieve data from ListGrid.dataSource, and from then on will 
*  automatically manage paging through large datasets, as well as performing filtering and
*  sorting operations inside the browser when possible - see the ResultSet docs for
*  details.
*  
*  NOTE: do not use both DataBoundComponent.autoFetchData and a
*  call to fetchData() - this may result in two DSRequests to fetch data. Use
*  either DataBoundComponent.autoFetchData and Criteria or a manual call to fetchData()
*  passing criteria.
*  
*  Whether a ResultSet was automatically created or provided via ListGrid.setData, subsequent
*  calls to fetchData() will simply call ResultSet.setCriteria.
*  
*  Changes to criteria may or may not result in a DSRequest to the server due to
*  ResultSet.useClientFiltering. You can call
*  DataBoundComponent.willFetchData to determine if new criteria will result in a
*  server fetch.
*  
*  If you need to force data to be re-fetched, you can call
*  ListGrid.invalidateCache and new data will automatically be fetched
*  from the server using the current criteria and sort direction. NOTE: when using
*  invalidateCache() there is no need to also call fetchData()
*  and in fact this could produce unexpected results.
*  
*  This method takes an optional callback parameter (set to a DSCallback) to fire when
*  the fetch completes. Note that this callback will not fire if no server fetch is performed.
*  In this case the data is updated synchronously, so as soon as this method completes you
*  can interact with the new data. If necessary, you can use
*  DataBoundComponent.willFetchData to determine whether or not a server
*  fetch will occur when fetchData() is called with new criteria.
*  
*  In addition to the callback parameter for this method, developers can use 
*  ListGrid.dataArrived to be notified every time data is loaded.
*  
*  By default, this method assumes a TextMatchStyle of "exact"; that can be overridden
*  by supplying a different value in the requestProperties parameter. 
*  See DataBoundComponent.willFetchData;
*  
*  Changing the request properties
*  
*  Changes to TextMatchStyle made via requestProperties will be honored in
*  combination with the fetch criteria, possibly invalidating cache and triggering a server
*  request if needed, as documented for DataBoundComponent.willFetchData.
*  In contrast, changes to
*  DSRequest.operationId in the request properties will cause
*  the ResultSet or ResultTree to be rebuilt, always refetching from the server.
*  However, changes to other request properties after the initial fetch won't be detected, and
*  no fetch will get triggered based on that new request context.
*  
*  To pick up such changes, we recommend that you call TileGrid.setData (passing an
*  empty array to ensure the data model is cleared), and then call this method to fetch again.
*  If you try to do it by calling ListGrid.invalidateCache, you may see duplicate fetches if you
*  haven't already updated the data context by calling this method with the new request
*  properties, and fail to do so before the component is Canvas.redraw.
* 
* 
* @param {Criteria=} Search criteria. If a DynamicForm is passed                     in as this argument instead of a raw criteria                      object, will be derived by calling                      DynamicForm.getValuesAsCriteria
* @param {DSCallback=} callback to invoke when a fetch is complete. Fires                     only if server contact was required
* @param {DSRequest=} additional properties to set on the DSRequest                     that will be issued
*/
fetchData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Retrieves data that matches the provided criteria and displays the matching data in this
*  component.
*  
*  This method behaves exactly like ListGrid.fetchData except that
*  DSRequest.textMatchStyle is automatically set to "substring" so that String-valued
*  fields are matched by case-insensitive substring comparison.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use 
*  them, see the gridFiltering.
* 
* @param {Criteria=} Search criteria.            If a DynamicForm is passed in as this argument            instead of a raw criteria object, will be derived by calling            DynamicForm.getValuesAsCriteria
* @param {DSCallback=} callback to invoke when a fetch is complete. Fires                     only if server contact was required; see                      ListGrid.fetchData for details
* @param {DSRequest=} for databound components only - optional              additional properties to set on the DSRequest that will be issued
*/
filterData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Invalidate the current data cache for this databound component via a call to
*  the dataset's invalidateCache() method, for example,
*  ResultSet.invalidateCache.
*  
*  NOTE: there is no need to call invalidateCache() when a save operation
*  is performed on a DataSource. Automatic cache synchronization features will automatically
*  update caches - see ResultSet for details. If automatic cache synchronization isn't
*  working, troubleshoot the problem using the steps suggested 
*  in the FAQ rather
*  than just calling invalidateCache(). Calling invalidateCache() unnecessarily
*  causes extra server load and added code complexity.
*  
*  Calling invalidateCache() will automatically cause a new fetch to 
*  be performed with the current set of criteria if data had been previously fetched and the
*  component is currently drawn with data visible - there is no need to manually call
*  fetchData() after invalidateCache() and this could result in duplicate fetches. 
*  
*  While data is being re-loaded after a call to invalidateCache(), the widget is
*  in a state similar to initial data load - it doesn't know the total length of the dataset
*  and any APIs that act on records or row indices will necessarily fail and should not be
*  called. To detect that the widget is in this state, call ResultSet.lengthIsKnown.
*  
*  invalidateCache() only has an effect if this components dataset is a data
*  manager class that manages a cache (eg ResultSet or ResultTree). If data was provided as a
*  simple Array or List, invalidateCache() does nothing.
*  
* 
*/
invalidateCache(): void;

/**
*  Remove the currently selected records from this component.
*  If this is a databound grid, the records will be removed directly from the DataSource.
*  The grid will automatically be updated if the record deletion succeeds.
*  
*  If no records are selected, no action is taken and neither callback will be called.
*  
*  For a grid with no DataSource or where saveLocally is true, the data removal
*  is performed on the client and both callbacks will fire with no arguments.
* 
* 
* @param {DSCallback=} callback to fire when each record has been removed
* @param {DSRequest=} additional properties to set on the DSRequest                     that will be issued
* @param {RPCQueueCallback=} callback to fire after all selected data has                      been removed
*/
removeSelectedData(callback?:DSCallback, requestProperties?:DSRequest, queueCallback?:(response:Array<Partial<RPCResponse>>) => void): void;

/**
* 
*  Displays the currently selected record(s) of the selectionComponent widget (typically a
*  listGrid) in this component.
*  
*  For a DynamicForm the first record of the selection is shown after the form is placed
*  into DynamicForm.canEdit.
*  A subsequent call to DynamicForm.editRecord or similar will return the
*  form to editability.
*  
*  Note that since field-level canEdit:true settings override the form-level
*  canEdit setting the automatic change to read-only may not change every field.
* 
* 
* @param {ListGrid | TileGrid | GlobalId} the ListGrid or TileGrid or ID of a ListGrid/TileGrid whose currently   selected record(s) is/are to be viewed
*/
viewSelectedData(selectionComponent:ListGrid | TileGrid | GlobalId): void;

/**
*  Loads all records that match this grid's current filter-criteria, optionally firing a 
*  callback when the data arrives.
*  
*  If the length of the data is ResultSet.lengthIsKnown, or is greater than 
*  the passed maxRecords, this call returns false. No fetch 
*  is issued and the callback, if passed, is not fired.
*  
*  If all data is ResultSet.allMatchingRowsCached, no fetch is issued 
*  and this call returns true. The callback, if passed, will be fired, but its 
*  parameters will be null, since there was no fetch to provide the values from.
*  
*  In all other cases, this call returns true and a fetch is issued for all necessary records. 
*  When the data arrives, the callback is fired.
* 
* 
* @param {Integer=} optional maximum record count - if passed, no fetch takes place                if maxRecords is below the known length of the data
* @param {DSCallback=} callback to fire if a fetch is issued - if all data was                 already loaded, the callback is fired with no parameters
*/
loadAllRecords(maxRecords?:Integer, callback?:DSCallback): boolean;

/**
*  Returns all selected records, as an Array.
* 
* 
* @param {boolean=} When true, partially selected records will not be returned.                  Otherwise, both fully and partially selected records are                  returned.
*/
getSelection(excludePartialSelections?:boolean): Array<Partial<ListGridRecord>>;

/**
*  Returns the first selected record in this component.
*  
*  NOTE: If a record is returned, it should be treated as read-only and not
*  modified.
* 
*/
getSelectedRecord(): TileRecord;

/**
* 
*  Select/deselect a Record passed in explicitly, or by index.
* 
* 
* @param {Record | number} record (or row number) to select
* @param {boolean=} new selection state (if null, defaults to true)
*/
selectRecord(record:Record | number, newState?:boolean): void;

/**
* 
*  Deselect a Record passed in explicitly, or by index.
*  
*  Synonym for selectRecord(record, false)
* 
* 
* @param {Record | number} record (or row number) to deselect
*/
deselectRecord(record:Record | number): void;

/**
* 
*  Select/deselect a list of Records passed in explicitly, or by index.
*  
*  Note that developers may wish to use TileGrid.selectRange to select a single
*  contiguous range.
* 
* 
* @param {Array<Partial<Record>> | number} records (or row numbers) to select
* @param {boolean=} new selection state (if null, defaults to true)
*/
selectRecords(records:Array<Partial<Record>> | number, newState?:boolean): void;

/**
*  Select a contiguous range of records by index
* 
* @param {Integer} start of selection range
* @param {Integer} end of selection range (non-inclusive)
* @param {boolean=} new selection state (if null, defaults to true)
*/
selectRange(startRow:Integer, endRow:Integer, newState?:boolean): void;

/**
* 
*  Deselect a list of Records passed in explicitly, or by index.
*  
*  Synonym for selectRecords(records, false)
*  
*  Note that developers may wish to use TileGrid.deselectRange to select a single
*  contiguous range.
* 
* 
* @param {Array<Partial<Record>> | number} records (or row numbers) to deselect
*/
deselectRecords(records:Array<Partial<Record>> | number): void;

/**
*  Deselect a contiguous range of records by index.
*  
*  This is a synonym for selectRange(startRow, endRow, false);
* 
* @param {Integer} start of selection range
* @param {Integer} end of selection range (non-inclusive)
*/
deselectRange(startRow:Integer, endRow:Integer): void;

/**
*  Select all records
* 
* 
*/
selectAllRecords(): void;

/**
* 
*  Deselect all records
* 
* 
*/
deselectAllRecords(): void;

/**
*  Whether at least one item is selected
* 
*/
anySelected(): boolean;

/**
*  Only supported on ListGrid for now.
* 
* @param {Array<Partial<Hilite>>} Array of hilite objects
*/
setHilites(hilites:Array<Partial<Hilite>>): void;

/**
*  If defined, this method will be called when a new tile is required. Note that
*  this method is in complete control of how the tile is constructed, so that
*  properties such as TileGrid.tileProperties and others needed by TileGrid
*  will be applied only after this method returns.
* 
* 
* @param {Record} record that will be associated with new tile
* @param {Integer} index of the record in the tileGrid
*/
createTile(record:Record, tileIndex:Integer): Canvas;

/**
*  If both this method and TileGrid.createTile are defined and
*  TileGrid.recycleTiles is true, this method will be called when the
*  framework needs to recycle a tile to be used with a new record. This
*  notification provides an opportunity to update any widget properties that
*  depend on the specifics of the record.
* 
* 
* @param {Record} record that will be associated with the recycled tile
* @param {Integer} index of the record in the tileGrid
* @param {Canvas} the tile to be recycled
*/
updateTile(record:Record, tileIndex:Integer, reclaimedTile:Canvas): void;

/**
*  Given a tile within this this tile-grid, this method will return the associated record.
* 
* 
* @param {Canvas} Tile you want to get the record for
*/
getTileRecord(tile:Canvas): TileRecord;

/**
*  Provides a new data set to the TileGrid after the grid has been created or drawn.
*  The TileGrid will redraw to show the new data automatically.
* 
* @param {Array<Partial<Record>> | Array<Partial<TileRecord>> | ResultSet} data to show in the list
*/
setData(newData:Array<Partial<Record>> | Array<Partial<TileRecord>> | ResultSet): void;

/**
*  Generate the tile HTML for a record in the data set.
*  
*  This method is invoked automatically by each tile created as an instance of 
*  the SimpleTile class to get its content. If 
*  TileGrid.tileConstructor has been modified this 
*  method therefore may not be called. (See also TileGrid.tile).
*  
*  The default implementation of this method uses an automatically generated 
*  DetailViewer to generate HTML for the specified
*  tileRecord based on the specified 
*  TileGrid.fields for this tileGrid.
*  
* 
* @param {TileRecord} the tile for which HTML should be retrieved
*/
getTileHTML(tileRecord:TileRecord): HTMLString;

/**
*  Returns the tile for the passed record or record index.
*  
*  Note that this method may be overridden but developers should be aware that this
*  method may be called repeatedly for the same record each time the TileGrid refreshes
*  that row. If you override this API, you will need to cache and re-use the same
*  tile objects per record. Typically this would be achieved by storing a pool of Tile
*  objects that are re-used if a Record with the same primaryKey is passed to getTile().
*  
*  When calling this method directly, if TileGrid.showAllRecords is false, this may 
*  return null for records that are not currently visible.
* 
* 
* @param {TileRecord | Integer} record or index of record in this.data
*/
getTile(tile:TileRecord | Integer): Canvas;

/**
*  Returns the index of the specified tile.
* 
* @param {Canvas} Tile you want to get the index for
*/
getTileIndex(tile:Canvas): Integer;

/**
*  Returns the tile currently under the mouse.
* 
*/
getCurrentTile(): SimpleTile;

/**
*  Executed when the tileGrid receives a 'click' event on a
*  tile. The default implementation does nothing -- override to perform some action
*  when any record is clicked.
*  A record event handler can be specified either as
*  a function to execute, or as a string of script to evaluate. If the handler is defined
*  as a string of script, all the parameters below will be available as variables for use
*  in the script.
*  If you want to cancel the click based on the parameters, return false. Otherwise, return 
*  true so that the click event be registered with the tile.
* 
* 
* @param {TileGrid} the TileGrid itself
* @param {Canvas} the tile that was clicked on
* @param {TileRecord} the record that was clicked on
*/
recordClick(viewer:TileGrid, tile:Canvas, record:TileRecord): void;

/**
*  Executed when the tileGrid receives a 'doubleclick' event on a
*  tile. The default implementation does nothing -- override to perform some action
*  when any record is doubleclicked.
*  A record event handler can be specified either as
*  a function to execute, or as a string of script to evaluate. If the handler is defined
*  as a string of script, all the parameters below will be available as variables for use
*  in the script.
*  If you want to cancel the doubleclick based on the parameters, return false. Otherwise, return 
*  true so that the doubleclick event be registered with the tile.
* 
* 
* @param {TileGrid} the TileGrid itself
* @param {Canvas} the tile that was doubleclicked on
* @param {TileRecord} the record that was doubleclicked on
*/
recordDoubleClick(viewer:TileGrid, tile:Canvas, record:TileRecord): void;

/**
*  Executed when the tileGrid receives a context-click (right mouse button) event on a
*  tile. The default implementation does nothing -- override to perform some action
*  when any record is right-clicked.
*  Return false to cancel the native behavior (suppressing
*  the browser context menu).
*  
*  
*  A record event handler can be specified either as
*  a function to execute, or as a string of script to evaluate. If the handler is defined
*  as a string of script, all the parameters below will be available as variables for use
*  in the script.
*  If you want to cancel the click based on the parameters, return false. Otherwise, return 
*  true so that the click event be registered with the tile.
* 
* 
* @param {TileGrid} the TileGrid itself
* @param {Canvas} the tile that was clicked on
* @param {TileRecord} the record that was clicked on
*/
recordContextClick(viewer:TileGrid, tile:Canvas, record:TileRecord): boolean;

/**
*  Called when selection changes within this tileGrid. Note this method fires for
*  each record for which selection is modified - so when a user clicks inside a tileGrid this
*  method will typically fire twice (once for the old record being deselected, and once for
*  the new record being selected).
* 
* 
* @param {Record} record for which selection changed
* @param {boolean} New selection state (true for selected, false for unselected)
*/
selectionChanged(record:Record, state:boolean): void;

/**
*  This is not allowed for tileGrid. Instead, use TileGrid.addData.
* 
* 
*/
addTile(): void;

/**
*  This is not allowed for tileGrid. Instead, use TileGrid.removeData.
* 
* 
* @param {Canvas | Integer | GlobalId} index or String ID of the tile
*/
removeTile(tileID:Canvas | Integer | GlobalId): boolean;

/**
*  Perform a DataSource "add" operation to add new records to this component's DataSource.
* 
* 
* @param {Record} new record
* @param {DSCallback=} method to call on operation completion
* @param {Partial<DSRequest>=} additional properties to set on the DSRequest                     that will be issued
*/
addData(newRecord:Record, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Perform a DataSource "remove" operation to remove records from this component's
*  DataSource.
* 
* 
* @param {Record} primary key values of record to delete,                      (or complete record)
* @param {DSCallback=} method to call on operation completion
* @param {Partial<DSRequest>=} additional properties to set on the DSRequest                     that will be issued
*/
removeData(data:Record, callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  Setter for TileGrid.canReorderTiles.
* 
*/
setCanReorderTiles(): void;

/**
*  Setter for TileGrid.canDragTilesOut.
* 
*/
setCanDragTilesOut(): void;

/**
*  Setter for TileGrid.canAcceptDroppedRecords.
* 
*/
setCanAcceptDroppedRecords(): void;

/**
*  Sets the custom tracker HTML to display next to the mouse when the user initiates a drag
*  operation on this component. Default implementation will examine ListGrid.dragTrackerMode
*  and set the custom drag tracker to display the appropriate HTML based on the selected record.
*  
*  To display custom drag tracker HTML, this method may be overridden - call 
*  EventHandler.setDragTracker to actually update the drag tracker HTML.
* 
*/
setDragTracker(): boolean;

/**
*  Method to return the fieldName which represents the "title" for records in this
*  Component.
*  If this.titleField is explicitly specified it will always be used.
*  Otherwise, default implementation will check DataSource.titleField for databound
*  compounds.
*  For non databound components returns the first defined field name of "title",
*  "name", or "id" where the field is visible. If we don't find any
*  field-names that match these titles, the first field in the component will be used instead.
* 
*/
getTitleField(): string;

/**
*  Return "title" HTML to display as a drag tracker when the user drags some record.
*  Default implementation will display the cell value for the title field (see 
*  ListGrid.getTitleField) for the record(s) being dragged (including any
*  icons / custom formatting / styling, etc).
*  
*  Note: Only called if ListGrid.dragTrackerMode is set to "title".
* 
* @param {ListGridRecord} First selected record being dragged
* @param {number} row index of first record being dragged
*/
getDragTrackerTitle(record:ListGridRecord, rowNum:number): string;

/**
*  Returns the record index of the tile that would currently be dropped on by the drag in
*  process. Returns one beyond the last valid index to indicate a drop after all records.
*  Except for that special case, a non-null index returned by this method may be passed to
*  TileGrid.getTile to get the corresponding visible tile.
* 
* 
*/
getDropIndex(): Integer;

/**
* 
*  During a drag-and-drop interaction, this method is called to transfer a set of records that
*  were dropped onto some other component. This method is called after the set of records has
*  been copied to the other component. Whether or not this component's data is modified is 
*  determined by the value of DataBoundComponent.dragDataAction.
*  
*  With a dragDataAction of "move", a databound component will issue "remove"
*  dsRequests against its DataSource to actually remove the data, via
*  DataSource.removeData.
* 
* 
*/
transferDragData(): Array<any>;

/**
* 
*  During a drag-and-drop interaction, this method returns the set of records being dragged out
*  of the component. In the default implementation, this is the list of currently selected
*  records.
*  
*  This method is consulted by ListGrid.willAcceptDrop.
*  
*  NOTE: If this component is a Tree.isMultiLinkTree TreeGrid, 
*  this method returns a list of NodeLocators rather than a list of records. Each 
*  nodeLocator contains a pointer to the associated record in its node
*  property.
* 
* 
* @param {DataBoundComponent} source component from which the records will be transferred
*/
getDragData(source:DataBoundComponent): Array<Partial<Record>>;

/**
*  Simulates a drag / drop type transfer of the selected records in some other component to this
*  component, without requiring any user interaction. This method acts on the dropped records 
*  exactly as if they had been dropped in an actual drag / drop interaction, including any 
*  special databound behavior invoked by calling
*  DataBoundComponent.getDropValues for each dropped record.
*  
*  To transfer all data in, for example, a ListGrid, call 
*  ListGrid.selectAllRecords first.
*  
*  Note that drag/drop type transfers of records between components are asynchronous operations:
*  SmartClient may need to perform server turnarounds to establish whether dropped records 
*  already exist in the target component. Therefore, it is possible to issue a call to 
*  transferSelectedData() and/or the ListGrid.drop method of a databound 
*  component whilst a transfer is still active. When this happens, SmartClient adds the 
*  second and subsequent transfer requests to a queue and runs them one after the other. If 
*  you want to be notified when a transfer process has actually completed, either provide a 
*  callback to this method or implement DataBoundComponent.dropComplete.
*  
*  See the dragging documentation for an overview of list grid drag/drop data
*  transfer.
*  
* 
* @param {DataBoundComponent} source component from which the records will be transferred
* @param {Integer=} target index (drop position) of the rows within this grid.
* @param {Callback=} optional callback to be fired when the transfer process has            completed. The callback will be passed a single parameter "records",            the list of records actually transferred to this component.
*/
transferSelectedData(source:DataBoundComponent, index?:Integer, callback?:Callback): void;

/**
*  Sets some presentation properties (visibility, width, userFormula and userSummary) of the 
*  grid fields based on the ListGridFieldState object passed in.
*  Used to restore previous state retrieved from the grid by a call to TileGrid.getFieldState.
*  
*  The optional isSparse parameter may be passed to indicate
*  whether the fieldState object is "sparse" - whether it includes explicit
*  state information for hidden fields. In this case any fields defined on the component
*  not explicitly included in the fieldState object will be hidden.
*  If isSparse is not explicitly passed as a parameter, 
*  sparseness will be assumed if DataBoundComponent.sparseFieldState
*  is true.
* 
* 
* @param {ListGridFieldState} state to apply to the grid's fields.
* @param {boolean=} If true, the fieldState passed in is assumed to be "sparse".  Any fields defined on this component without explicit field state values will be  hidden.
*/
setFieldState(fieldState:ListGridFieldState, isSparse?:boolean): void;

/**
*  Returns a snapshot of the current presentation of this grid's fields as 
*  a ListGridFieldState object.
*  
*  This object can be passed to TileGrid.setFieldState to reset this grid's fields to
*  the current state.
*  
*  Note that the information stored includes the current width and visibility of each of this 
*  grid's fields.
*  
*  The optional sparse parameter governs whether the returned field state
*  should omit state information for hidden fields. If this parameter is not passed
*  explicitly, field state will be sparse if DataBoundComponent.sparseFieldState
*  is true.
*  When applying sparse field state to a component via TileGrid.setFieldState, any 
*  explicitly defined fields on the component that were not captured in the stored
*  state object will be hidden.
* 
* 
* @param {boolean=} If true, field state will be ommitted for hidden fields.
*/
getFieldState(sparse?:boolean): ListGridFieldState;

/**
*  Notification method fired when new data arrives from the server to be displayed in this
*  tileGrid, (for example in response to the user scrolling a new set of tiles into view).
*  Only applies to databound tileGrid where the TileGrid.data attribute is a
*  ResultSet.
*  This method is fired directly in
*  response to ResultSet.dataArrived firing on the data object.
* 
* @param {Integer} starting index of the newly loaded set of records
* @param {Integer} ending index of the newly loaded set of records (non inclusive).
*/
dataArrived(startRecord:Integer, endRecord:Integer): void;



}
/**
* Creates a new TileGrid
* 
* @param typeCheckedProperties {Partial<TileGrid>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TileGrid} The new instance
*/
class TileGrid{
static create(typeCheckedProperties? : Partial<TileGrid>, uncheckedProperties?: Object): TileGrid;







}

/**
* EditProxy that handles TileGrid components when editMode is enabled.
*/
interface TileGridEditProxy extends LayoutEditProxy {




}
/**
* Creates a new TileGridEditProxy
* 
* @param typeCheckedProperties {Partial<TileGridEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TileGridEditProxy} The new instance
*/
class TileGridEditProxy{
static create(typeCheckedProperties? : Partial<TileGridEditProxy>, uncheckedProperties?: Object): TileGridEditProxy;







}

/**
* Lays out a series of components, called "tiles", in a grid with multiple tiles per row.
*/
interface TileLayout extends Canvas {
/**
* List of tiles to lay out.
* @type {Array<Partial<Canvas>>}
* @default null
*/
tiles?: Array<Partial<Canvas>>;

/**
* Policy for laying out tiles. See TileLayoutPolicy for options.
* @type {TileLayoutPolicy}
* @default "fit"
*/
layoutPolicy?: TileLayoutPolicy;

/**
* Size of each tile in pixels. Depending on the TileLayoutPolicy, tileSize
*  may be taken as a maximum, minimum or exact size of tiles, or may be irrelevant.
*  
*  Width and height may be separately set via TileLayout.tileHeight and TileLayout.tileWidth.
* @type {Integer}
* @default "100"
*/
tileSize?: Integer;

/**
* Width of each tile in pixels. See TileLayout.tileSize.
*  If TileLayoutPolicy is "fit", TileLayout.expandMargins is false, TileLayout.tilesPerLine is set,
*  Orientation is "horizontal",
*  and tileWidth is not set, tileWidth will be computed automatically based on TileLayout.tilesPerLine.
* @type {Integer}
* @default null
*/
tileWidth?: Integer;

/**
* Height of each tile in pixels. See TileLayout.tileSize.
*  If TileLayoutPolicy is "fit", TileLayout.expandMargins is false, TileLayout.tilesPerLine is set, 
*  Orientation is "vertical",
*  and tileHeight is not set, tileHeight will be computed automatically based on TileLayout.tilesPerLine.
* @type {Integer}
* @default null
*/
tileHeight?: Integer;

/**
* Margin in between tiles. Can be set on a per-axis basis with TileLayout.tileHMargin and
*  TileLayout.tileVMargin.
* @type {Integer}
* @default "10"
*/
tileMargin?: Integer;

/**
* Horizontal margin in between tiles. See TileLayout.tileMargin.
* @type {Integer}
* @default null
*/
tileHMargin?: Integer;

/**
* Vertical margin in between tiles. See TileLayout.tileMargin.
* @type {Integer}
* @default null
*/
tileVMargin?: Integer;

/**
* A margin left around the outside of all tiles.
* @type {Integer}
* @default "5"
*/
layoutMargin?: Integer;

/**
* If this widget has padding specified (as Canvas.padding or in the
*  CSS style applied to this layout), should it show up as space outside the members,
*  similar to layoutMargin?
*  
*  If this setting is false, padding will not affect member positioning (as CSS padding
*  normally does not affect absolutely positioned children). Leaving this setting true
*  allows a designer to more effectively control layout purely from CSS.
*  
*  Note that Layout.layoutMargin if specified, takes precedence over this value.
* @type {boolean}
* @default true
*/
paddingAsLayoutMargin?: boolean;

/**
* If set, tiles animate to their new positions when a tile is added, removed, or reordered via
*  drag and drop.
* @type {boolean}
* @default true
*/
animateTileChange?: boolean;

/**
* Direction of tiling. See also TileLayoutPolicy.
* @type {Orientation}
* @default "horizontal"
*/
orientation?: Orientation;

/**
* Number of tiles to show in each line. Auto-derived from TileLayout.tileSize for some layout
*  modes. See TileLayoutPolicy.
*  This can also affect TileLayout.tileWidth or TileLayout.tileHeight. See those properties for details.
* @type {Integer}
* @default null
*/
tilesPerLine?: Integer;

/**
* Normal Overflow settings can be used on TileLayouts, for example, an
*  overflow:auto TileLayout will scroll if members exceed its specified size, whereas an
*  overflow:visible TileLayout will grow to accommodate members.
* @type {Overflow}
* @default "auto"
*/
overflow?: Overflow;

/**
* With LayoutPolicy:"fit", should margins be expanded so that tiles fill the
*  available space in the TileLayout on the breadth axis?
*  This can also affect TileLayout.tileWidth or TileLayout.tileHeight. See those properties for details.
* @type {boolean}
* @default true
*/
expandMargins?: boolean;

/**
* When LayoutPolicy is "flow", should we automatically start a new line when there's
*  not enough room to fit the next tile on the same line?
*  
*  If set to false, a new line will only be started if a tile specifies
*  Canvas.startLine or Canvas.endLine.
* @type {boolean}
* @default true
*/
autoWrapLines?: boolean;

/**
* Canvas used to display a drop indicator when a another canvas is dragged over this widget.
* @type {Canvas}
* @default null
*/
dragLine?: Canvas;

/**
* The CSS class applied to the TileLayout.dragLine autochild.
* @type {CSSStyleName}
* @default "layoutDropLine"
*/
dragLineStyle?: CSSStyleName;

/**
* Thickness of the TileLayout.dragLine autochild.
* @type {Integer}
* @default "2"
*/
dragLineThickness?: Integer;

/**
* Set false to prevent the TileLayout.dragLine autochild from showing during dragging.
* @type {boolean}
* @default null
*/
showDragLine?: boolean;

/**
* Indicates what to do with data dragged into another DataBoundComponent. See
*  DragDataAction type for details.
* @type {DragDataAction}
* @default "move"
*/
dragDataAction?: DragDataAction;



/**
*  Retrieve a tile by index. 
*  
*  The TileLayout consistently uses this method to access tiles, in order to allow subclasses
*  to create tiles on demand.
* 
* 
* @param {Integer} index of the tile
*/
getTile(index:Integer): Canvas;

/**
*  Add a tile to the layout, dynamically.
* 
* 
* @param {Canvas} new tile to add
* @param {Integer=} position where the tile should be added. Defaults to adding the tile at the end.
*/
addTile(tile:Canvas, index?:Integer): void;

/**
*  Remove a tile from the layout.
* 
* 
* @param {Canvas | Integer | GlobalId} index or String ID of the tile
*/
removeTile(tileID:Canvas | Integer | GlobalId): boolean;

/**
*  Transforms the input tile Canvas.setRect to an absolute page rect that
*  you can apply to your own drop indicator canvas. The supplied rect is automatically
*  clipped along the direction perpendicular to the layout's TileLayout.orientation if
*  it extends beyond the visible edges of the layout, just like the TileLayout.dragLine autochild.
*  
*  Note: Only code your own drop indicator if you can't get what you need by customizing
*  and TileLayout.dragLineStyle the built-in TileLayout.dragLine autochild!
*  
*  To build your own:
*  
* - Create a separate indicator Canvas, positioned off screen with the appropriate
*  color, opacity, and Canvas.styleName that you want. Set the indicator's
*  Canvas.dropTarget to be the layout so the indicator is ignored as a target itself.
*  
* - Override Canvas.dropMove to call TileLayout.getDropIndex, retrieve the tile, get
*  the ${isc.DocUtils.linkForRef('method:Canvas.setRect','tile\'s relative rect')}, and modify the rect as you need to size your
*  indicator properly.
*  
* - You'll have to manually handle the case of the drop index pointing beyond the last
*  record, perhaps by grabbing the last tile rect, but narrowing it to the opposite side.
*  
* - Pass the modified rect to this method to clip and transform it to an absolute rect,
*  and then Canvas.setRect into your drop indicator and Canvas.show it.
*  
* - You will need to Canvas.hide the drop indicator in Canvas.dropOut.
*  
*  
*  Sample code that can be added to the SmartClient Showcase "Basic Tiling" sample:
*  
*   showDragLine: false,
*   canReorderTiles: true,
* 
*   showDropIndicator : function (left, top, width, height) {
*     if (!this.dropIndicator) {
*       this.dropIndicator = isc.Canvas.create({
*         top: -1000, opacity: 40, 
*         autoDraw: true, dropTarget: this,
*         backgroundColor:"yellow"
*       });
*     }
*     var rect = this.transformTileRect(left, top, width, height);
*     this.dropIndicator.setRect(rect);
*     this.dropIndicator.show();
*   },
* 
*   dropMove: function() {
*     this.Super("dropMove",arguments);
* 
*     var index = this.getDropIndex(),
*       length = this.data.getLength();
* 
*     // you can drop after last tile (special case)
*     var after = index >= length;
*     if (after) index = length - 1;
* 
*     // transform tile rect to indicator rect
*     var tile = this.getTile(index);
*     if (tile) {
*       var left = tile.getLeft(), width = tile.getVisibleWidth(),
*         top = tile.getTop(), height = tile.getVisibleHeight()
*       ;
*       // to show drop after last tile, shift indicator over
*       if (after) left += width - 20;
* 
*       // now show the indicator
*       this.showDropIndicator(left, top, 20, height);
*     }
*   },
* 
*   dropOut : function() {
*     this.Super("dropOut", arguments);
* 
*     if (this.dropIndicator) {
*       this.dropIndicator.hide();
*     }
*   },
* 
*   destroy : function () {
*     this.Super("destroy", arguments);
* 
*     if (this.dropIndicator) {
*       this.dropIndicator.destroy();
*     }
*   },
*  
* 
* @param {number | Array<Partial<number>>} new left coordinate,                        Array of coordinates in parameter order,                         If an Array is passed, the remaining                         parameters are ignored.
* @param {number=} new top coordinate
* @param {number=} new width
* @param {number=} new height
*/
transformTileRect(left:number | Array<Partial<number>>, top?:number, width?:number, height?:number): Array<Partial<number>>;

/**
*  Returns the tile index of the tile that would currently be dropped on by the drag in process.
*  Returns one beyond the last valid index to indicate a drop after all tiles. Except for that
*  special case, a non-null index returned by this method may be passed to TileLayout.getTile to
*  get the corresponding visible tile.
* 
* 
*/
getDropIndex(): Integer;

/**
* 
*  During a drag-and-drop interaction, this method is called to transfer a set of records that
*  were dropped onto some other component. This method is called after the set of records has
*  been copied to the other component. Whether or not this component's data is modified is 
*  determined by the value of DataBoundComponent.dragDataAction.
*  
*  With a dragDataAction of "move", a databound component will issue "remove"
*  dsRequests against its DataSource to actually remove the data, via
*  DataSource.removeData.
* 
* 
*/
transferDragData(): Array<any>;

/**
* 
*  During a drag-and-drop interaction, this method returns the set of records being dragged out
*  of the component. In the default implementation, this is the list of currently selected
*  records.
*  
*  This method is consulted by ListGrid.willAcceptDrop.
*  
*  NOTE: If this component is a Tree.isMultiLinkTree TreeGrid, 
*  this method returns a list of NodeLocators rather than a list of records. Each 
*  nodeLocator contains a pointer to the associated record in its node
*  property.
* 
* 
* @param {DataBoundComponent} source component from which the records will be transferred
*/
getDragData(source:DataBoundComponent): Array<Partial<Record>>;

/**
*  Sets the height and width of tiles.
* 
* @param {Integer} size
*/
setTileSize(size:Integer): void;

/**
*  Sets the width of tiles.
* 
* @param {Integer} width
*/
setTileWidth(width:Integer): void;

/**
*  Sets the height of tiles.
* 
* @param {Integer} height
*/
setTileHeight(height:Integer): void;

/**
*  Sets the number of tiles per line.
* 
* @param {Integer} New TileLayout.tilesPerLine value
*/
setTilesPerLine(tilesPerLine:Integer): void;

/**
*  Sets the vertical and horizontal margin of tiles.
* 
* @param {Integer} margin
*/
setTileMargin(margin:Integer): void;

/**
*  Sets the horizontal margin of tiles.
* 
* @param {Integer} margin
*/
setTileHMargin(margin:Integer): void;

/**
*  Sets the vertical margin of tiles.
* 
* @param {Integer} margin
*/
setTileVMargin(margin:Integer): void;



}
/**
* Creates a new TileLayout
* 
* @param typeCheckedProperties {Partial<TileLayout>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TileLayout} The new instance
*/
class TileLayout{
static create(typeCheckedProperties? : Partial<TileLayout>, uncheckedProperties?: Object): TileLayout;







}

/**
* A TileGrid that implements the Palette behavior, so it can be used as the source for 
*  drag and drop instantiation of components when combined with an EditContext as 
*  the drop target.
*  
*  Each TileGrid.tile can be a PaletteNode.
*/
interface TilePalette extends TileGrid, Palette {




}
/**
* Creates a new TilePalette
* 
* @param typeCheckedProperties {Partial<TilePalette>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TilePalette} The new instance
*/
class TilePalette{
static create(typeCheckedProperties? : Partial<TilePalette>, uncheckedProperties?: Object): TilePalette;







}

/**
* Helper methods and system-wide defaults for dealing with time values and time display formats.
*  
*  This class includes utility methods for the creation and display of logical time values, as well
*  as modifying the default display timezone for datetime type values. See
*  dateFormatAndStorage for more information on working with dates, times and datetimes
*  in SmartClient.
*/
interface Time extends Class {




}
/**
* Creates a new Time
* 
* @param typeCheckedProperties {Partial<Time>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Time} The new instance
*/
class Time{
static create(typeCheckedProperties? : Partial<Time>, uncheckedProperties?: Object): Time;
/**
* Hour offset from UTC to use when formatting FieldType type fields for 
*  display to the user.
*  
*  Has no effect on fields specified as logical date (field.type = "date";) and
*  logical time (field.type = "time") fields.
* @deprecated * As of 7.0 this attribute has been deprecated in favor of Time.setDefaultDisplayTimezone

* @type {number}
* @default null
*/
static UTCHoursOffset?: number;

/**
* Standard formatter to be used when converting a date to a time-string via Time.toTime
* @type {TimeDisplayFormat | Function}
* @default "toTime"
*/
static displayFormat?: TimeDisplayFormat | Function;

/**
* Standard formatter to be used when converting a date to a time-string via Time.toShortTime
* @type {TimeDisplayFormat | Function}
* @default "toShortTime"
*/
static shortDisplayFormat?: TimeDisplayFormat | Function;

/**
* String appended to times to indicate am (when not using 24 hour format).
* @type {string}
* @default " am"
*/
static AMIndicator?: string;

/**
* String appended to times to indicate am (when not using 24 hour format).
* @type {string}
* @default " pm"
*/
static PMIndicator?: string;

/**
* Determines whether datetime formatters should consider the effect of Daylight Saving
*  Time when computing offsets from UTC. By default, this flag is set during framework
*  initialization if SmartClient detects that it is running in a locale that is observing 
*  DST this year. If you do not want DST adjustments to be applied, set this flag to 
*  false.
*  Note that setting this flag to true will have no effect unless you are in a locale 
*  that is observing Daylight Saving Time for the date in question; this is because
*  we rely on the browser for offset information, and browsers are only capable of 
*  returning local date and time information for the computer's current locale.
*  
*  This setting will not have any impact on the display of fields specified as type "time" or
*  "date" (logical dates and logical times) - only on datetime type values. See
*  dateFormatAndStorage for information on working with dates, times and datetimes
*  in SmartClient.
* @type {boolean}
* @default "true (see description)"
*/
static adjustForDST?: boolean;

/**
* Indicate whether calls to DateUtil.format with no formatter show times in
*  24-hour format. This includes UI elements, such as TimeItem and
*  Calendar, when no formatters are provided.
* @type {boolean}
* @default true
*/
static use24HourTime?: boolean;

/**
* The separator character to include between hours, minutes and seconds when parsing and
*  formatting Time strings.
* @type {string}
* @default ":"
*/
static defaultTimeSeparator?: string;

/**
* The separator character to include before the milliseconds element when parsing and
*  formatting Time strings.
* @type {string}
* @default "."
*/
static defaultMillisecondSeparator?: string;





/**
*  Sets the offset from UTC to use when formatting values of type FieldType 
*  with standard display formatters.
*  
*  This property affects how dates are displayed and also the
*  assumed timezone for user-input. For a concrete example - assume this method has been called 
*  and passed a value of "+01:00", and an application has a DateTimeItem visible in
*  a DynamicForm. If the value of this field is set to the current date, with UTC time set to
*  "10:00", the time portion of the value displayed in the form item will be "11:00".
*  Similarly if a user modifies the time value in the text box to be "16:00", a call to 
*  FormItem.getValue for the item will return a date object with UTC time set to 15:00.
*  
*  Interaction with daylight savings time: The specified "defaultDisplayTimezone" should
*  reflect the correct UTC offset for the current date, for which it will always be exactly respected;
*  adjustment will only be made for dates that fall outside the current daylight savings time mode.
*  
*  In other words if DST is currently not in effect (IE: the current date is a Winter date),
*  any other dates where DST is not in effect will be formatted to exactly respect the specified
*  defaultDisplayTimezone (so for defaultDisplayTimezone of "+01:00", the display
*  string will be 1 hour ahead of the UTC time on the date in question), and any
*  dates where DST is in effect would be further adjusted to account for DST
*  (so the display string would be 2 hours ahead for dates that fall in the Summer).
*  Alternatively if DST currently is in effect (EG: Current date is a Summer date)
*  the situation is reversed. Any date value for which DST should be applied
*  will be be formatted for display with an offset of 1 hour from UTC - and any date value
*  for which DST should not be applied would be formatted with an offset of 0 hours from UTC.
*  
*  Note that the Time.adjustForDST property may be set to false to
*  disable this logic - in this case the time portion of dates will always be offset from
*  UTC by exactly the specified defaultDisplayOffset, regardless of whether they fall in the
*  range where Daylight Savings Time would usually be applied or not.
*  
*  Note that if a custom timezone is specified, it will not effect native javascript 
*  date formatting functions such as toLocaleString().
*  See dateFormatAndStorage for more on how SmartClient handles date and time
*  formatting and storage.
*  
*  If this method is never called, the default display timezone for times and datetimes will
*  be derived from the native browser local timezone.
*  
*  Note that the displayTimezone effects datetime fields only and has no effect on fields
*  specified as logical date (field.type = "date";) or
*  logical time (field.type = "time").
* 
* 
* @param {string} offset from UTC. This should be a string in the format  +/-HH:MM for example "-08:00"
*/
static setDefaultDisplayTimezone(offset:string): void;

/**
*  Returns the default display timezone set up by Time.setDefaultDisplayTimezone.
*  If no explicit timezone has been set this will return the browser locale timezone offset.
* 
*/
static getDefaultDisplayTimezone(): string;

/**
*  Given a date object, return the time associated with the date as a formatted string.
*  If no formatter is passed, use the standard formatter set up via 
*  Time.setNormalDisplayFormat.
*  
* 
* @param {Date} Date to convert to a time string.
* @param {TimeDisplayFormat | FormatString | Function=} Optional custom formatter to use. Will accept a function (which will be passed a pointer to the  Date to format), a format string, or a string designating a standard formatter
* @param {boolean=} Is the date passed in a representation of a logical time value such as a value from a "time" type field on a dataSource or a datetime value?  For datetime values the formatted string will respect any custom  Time.setDefaultDisplayTimezone. If not explicitly specified, the date passed in will be assumed to be a datetime unless it was created explicitly as a time via Time.createLogicalTime or similar APIs.
*/
static toTime(date:Date, formatter?:TimeDisplayFormat | FormatString | Function, logicalTime?:boolean): void;

/**
*  Given a date object, return the time associated with the date as a short string.
*  If no formatter is passed, use the standard formatter set up via Time.setShortDisplayFormat
* 
* @param {Date} Date to convert to a time string.
* @param {TimeDisplayFormat | FormatString | Function=} Optional custom formatter to use. Will accept a function (which will be passed a pointer to the  Date to format), a format string, or a string designating a standard formatter
* @param {boolean=} Is the date passed in a representation of a logical time value such as a value from a "time" type field on a dataSource or a datetime value?  For datetime values the formatted string will respect any custom  Time.setDefaultDisplayTimezone. If not explicitly specified, the date passed in will be assumed to be a datetime unless it was created explicitly as a time via Time.createLogicalTime or similar APIs.
*/
static toShortTime(date:Date, formatter?:TimeDisplayFormat | FormatString | Function, logicalTime?:boolean): void;

/**
*  Converts a time-string such as 1:00pm to a new Date object 
*  representing a logical time value (rather than a specific datetime
*  value), typically for display in a DataSourceField.type. 
*  Accepts most formats of time string. The generated
*  Date value will have year, month and date set to the epoch date (Jan 1 1970), and time 
*  elements set to the supplied hour, minute and second (in browser native local time).
*  
*  See dateFormatAndStorage for more information on date, time and datetime 
*  values in SmartClient.
*  
*  It may be a common hack for a server framework (e.g. aspx) where there is no "time of
*  day" type - only a full DateTime, or a TimeSpan - to return a string in
*  ISO 8601 duration format for
*  field values with type Time in the DataSource definition. We recognize and parse that
*  format here, but if you have the ability to change the type, the best approach would
*  probably be to switch the type to one fully supported server-side, such as DateTime.
* 
* 
* @param {string} time string to convert to a date
* @param {boolean} If this method is passed a timeString in an unrecognized format, return null rather than a date object with time set to 00:00:00
*/
static parseInput(timeString:string, validTime:boolean): void;

/**
*  Creates a date object with the time set to the hours, minutes and seconds passed in.
*  Unless the UTCTime parameter is passed in, parameters are assumed
*  to specify the time in native local display time.
* 
* @param {number=} Hours for the date (defaults to zero)
* @param {number=} Minutes for the date (defaults to zero)
* @param {number=} Seconds for the date (defaults to zero)
* @param {number=} Milliseconds for the date (defaults to zero)
* @param {boolean=} If true, treat the time passed in as UTC time rather than local time
* @deprecated * use Time.createLogicalTime instead.

*/
static createDate(hours?:number, minutes?:number, seconds?:number, milliseconds?:number, UTCTime?:boolean): void;

/**
*  Create a new Date object to represent a logical time value (rather than a specific datetime
*  value), typically for display in a DataSourceField.type. The generated
*  Date value will have year, month and date set to the epoch date (Jan 1 1970), and time 
*  elements set to the supplied hour, minute and second (in browser native local time).
*  
*  For simplicity, you can pass a Date-instance in the first parameter to create a logical
*  time from that Date instance. Passing null is the same as passing 
*  new Date(). When the first parameter is a Date instance, the 
*  minutes and seconds parameters may still be passed to 
*  fine-tune the result - for example, createLogicalTime(null, 0, 0) will 
*  return a Date instance with time-values representing the start of the current hour.
*  
*  See dateFormatAndStorage for more information on date, time and datetime 
*  values in SmartClient.
* 
* 
* @param {Integer | Date} integer hour (0-23) or a Date instance - if passed a Date     instance, its time-elements are mapped to hour, minute and second parameters.       Passing null is the same as passing new Date()
* @param {Integer} minute (0-59) - defaults to zero or, if the hour     parameter is a Date instance, the minutes from that Date instance
* @param {Integer} second (0-59) - defaults to zero or, if the hour     parameter is a Date instance, the seconds from that Date
*/
static createLogicalTime(hour:Integer | Date, minute:Integer, second:Integer): Date;

/**
*  Sets the default format for strings returned by Time.toShortTime.
* 
* @param {TimeDisplayFormat | FormatString | Function} Optional custom formatter to use. Will accept a function (which will be passed a pointer to the  Date to format), a format string, or a string designating a standard formatter
*/
static setShortDisplayFormat(formatter:TimeDisplayFormat | FormatString | Function): void;

/**
*  Sets the default format for strings returned by Time.toTime.
* 
* @param {TimeDisplayFormat | FormatString | Function} Optional custom formatter to use. Will accept a function (which will be passed a pointer to the  Date to format), a format string, or a string designating a standard formatter
*/
static setNormalDisplayFormat(formatter:TimeDisplayFormat | FormatString | Function): void;

/**
*  Compares the times of 2 dates, or strings. If a string is passed as one of the 
*  parameters it should be in a format that converts to a valid time such as "1:30pm", 
*  "13:30", or "1:30:45pm"
* 
* @param {Date | string} First time to compare
* @param {Date | string} Second time to compare
*/
static compareTimes(time1:Date | string, time2:Date | string): boolean;




}

/**
* A FormItem for editing DateUtil.createLogicalTime values, which
*  are Date instances where only the time-portion is relevant.
*  
*  The item renders with one of two appearances, depending on the value of 
*  TimeItem.useTextField - when set to true, the default appearance, times are edited 
*  directly as text-values. In this mode, values are formatted according to 
*  TimeItem.timeFormatter, with defaults coming from TimeItem.timeFormatter24Hour 
*  and TimeItem.timeFormatter12Hour, depending on the value of 
*  TimeItem.use24HourTime.
*  See also Time.setNormalDisplayFormat for system-wide settings.
*  
*  TimeItem automatically accepts both 12 and 24 hour time as well as partial times and a
*  variety of possible time value separators. Examples:
*  
*  11:34:45 AM	=> 11:34:45
*  1:3:5 AM	=> 01:30:50
*  1:3p => 13:30:00
*  11 34 am	=> 11:34:00
*  11-34 => 11:34:00
*  113445 => 11:34:45
*  13445 => 01:34:45
*  1134 => 11:34:00
*  134 	=> 01:34:00
*  
*  
*  When useTextField is set to false, the item provides separate pickers for 
*  TimeItem.hourItem, TimeItem.minuteItem and 
*  TimeItem.secondItem values. By default, the pickers edit times in 
*  TimeItem.use24HourTime, meaning the hourItem shows 
*  values from 0-23. When TimeItem.use24HourTime is set to false, the 
*  hourItem is limited to a range of 1-12, and the 
*  TimeItem.ampmItem is displayed. Note that 
*  FormItem.getValue always returns a Date instance that represents a
*  DateUtil.createLogicalTime in 24-hour format.
*  
*  Values entered by the user are stored as JavaScript Date objects in local time. 
*  The day, month and year values of this Date object are not relevant and should 
*  be ignored.
*  
*  By default, when used in a SearchForm or as a field in a ListGrid's 
*  ListGrid.showFilterEditor, TimeItems will automatically generate 
*  AdvancedCriteria - for example, entering "11:00" into the item will generate a 
*  OperatorId Criterion that selects all times between 
*  11:00:00 and 11:59:59. If the form is databound and the DataSource is marked as being
*  DataSource.allowAdvancedCriteria:false, the criteria generated
*  will be simple, checking for data with logical time values equal to the displayed value.
*  
*  To edit DateUtil.createLogicalDate, see DateItem,
*  and to edit DateUtil.createDatetime, see DateTimeItem.
*  For RelativeDateString, see RelativeDateItem.
*  
*  For detailed information on working with dates, times and datetimes, see the 
*  dateFormatAndStorage.
*/
interface TimeItem extends FormItem {
/**
* Text field to hold the entire time in "type in" format, if 
*  TimeItem.useTextField is true.
* @type {TextItem}
* @default null
*/
textField?: TextItem;

/**
* Custom properties to apply to the TimeItem.textField generated for 
*  this timeItem when TimeItem.useTextField is true.
* @type {Partial<TextItem>}
* @default null
*/
textFieldProperties?: Partial<TextItem>;

/**
* If TimeItem.useTextField is true and browserInputType is set to
*  "time", then a native HTML5 time input
*  is used in place of a text input.
*  
*  The use of a native HTML5 time input causes certain features to be disabled. Input masks
*  and a custom TimeItem.timeFormatter are not supported.
*  TimeItem.showHintInField are currently supported, but future browser
*  changes might force this support to be removed. Therefore, it is safest to not
*  use in-field hints (set showHintInField to false) in conjunction with a native HTML5 time
*  input.
*  
*  NOTE: This feature requires specific CSS changes. Currently these changes have
*  been made to the Enterprise, EnterpriseBlue, and Graphite skins only.
* @type {string}
* @default null
*/
browserInputType?: string;

/**
* If this item is showing a FormItem.hint, should the hint text be allowed to
*  wrap? Setting this property to false will render the hint on a single line
*  without wrapping, expanding the width required to render the item if necessary.
*  
*  If unset this property will be picked up from the DynamicForm.wrapHintText
*  setting.
*  
*  This setting does not apply to hints that are TextItem.showHintInField.
* @type {boolean}
* @default false
*/
wrapHintText?: boolean;

/**
* If TimeItem.useTextField is true and a FormItem.hint is
*  set, should the hint be shown within the field?
*  
*  Note that when using a native HTML5 time input (see TimeItem.browserInputType),
*  in-field hints are currently supported, but future browser changes might not allow
*  in-field hints to be supported. Therefore, it is safest to not use in-field
*  hints in conjunction with a native HTML5 time input.
*  
*  To change this attribute after being drawn, it is necessary to call FormItem.redraw
*  or redraw the form.
* @type {boolean}
* @default null
*/
showHintInField?: boolean;

/**
* If TimeItem.showHintInField and if supported by the browser, should the HTML5
*  placeholder attribute
*  be used to display the hint within the field? If set to false, then use of
*  the placeholder attribute is disabled and an alternative technique to display
*  the hint in-field is used instead.
*  
*  The HTML5 placeholder attribute is supported in the following major browsers:
*  
*  
* - Chrome 4+
*  
* - Firefox 4+
*  
* - Internet Explorer 10+
*  
* - Safari 5+
*  
* - Opera 11.50+
*  
* - Android 2.1+ WebView (used by the stock Browser app and when
*    phonegapIntegration)
*  
* - Mobile Safari for iOS 3.2+
*  
*  
*  In browsers other than the above, in-field hints are implemented via a different technique.
*  
*  Note that placeholder behavior is known to differ in Internet Explorer and certain old
*  versions of the above browsers due to a recent change in the HTML5 specification regarding
*  the placeholder attribute. Under the old rules, the placeholder is cleared
*  when the element is focused. In the latest HTML5 spec as published by WHATWG, the placeholder
*  is still displayed when the element is focused as long as the value is an empty string.
*  
*  Styling the placeholder
*  While there isn't a standard way to style the placeholder text, Chrome, Firefox,
*  Internet Explorer, and Safari provide vendor-prefixed pseudo-classes and/or pseudo-elements
*  that can be used in CSS selectors:
*  
*  
*   Browser
*   Pseudo-class or pseudo-element
*  
*  
*   Chrome, Safari
*   ::-webkit-input-placeholder
*  
*  
*   Firefox 4 - 18
*   :-moz-placeholder
*  
*  
*   Firefox 19+
*   ::-moz-placeholder
*  
*  
*   Internet Explorer
*   :-ms-input-placeholder
*  
*  
*  
*  Note that unlike other browsers, Firefox 19+ applies opacity:0.4 to the placeholder text.
*  See Bug 556145 - Placeholder text default style should use opacity instead of GrayText
*  
*  Because browsers are required to ignore the entire rule if a selector is invalid,
*  separate rules are needed for each browser. For example:
*  ::-webkit-input-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
* :-moz-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
* ::-moz-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
* :-ms-input-placeholder {
*   color: blue;
*   opacity: 1;
* &#125;
*  
*  If using Sass, it may be useful to utilize Sass'
*  parent selector feature.
*  For example:
*  .myCustomItem,
* .myCustomItemRTL,
* .myCustomItemDisabled,
* .myCustomItemDisabledRTL,
* .myCustomItemError,
* .myCustomItemErrorRTL,
* .myCustomItemFocused,
* .myCustomItemFocusedRTL,
* .myCustomItemHint,
* .myCustomItemHintRTL,
* .myCustomItemDisabledHint,
* .myCustomItemDisabledHintRTL {
*   // ...
* 
*   &amp;::-webkit-input-placeholder {
*     color: blue;
*     opacity: 1;
*   }
*   &amp;:-moz-placeholder {
*     color: blue;
*     opacity: 1;
*   }
*   &amp;::-moz-placeholder {
*     color: blue;
*     opacity: 1;
*   }
*   &amp;:-ms-input-placeholder {
*     color: blue;
*     opacity: 1;
*   }
* &#125;
*  
*  If using Compass, the
*  input-placeholder mixin
*  that was added in version 1.0 can further simplify the code to style the placeholder text
*  For example:
*  .myCustomItem,
* .myCustomItemRTL,
* .myCustomItemDisabled,
* .myCustomItemDisabledRTL,
* .myCustomItemError,
* .myCustomItemErrorRTL,
* .myCustomItemFocused,
* .myCustomItemFocusedRTL,
* .myCustomItemHint,
* .myCustomItemHintRTL,
* .myCustomItemDisabledHint,
* .myCustomItemDisabledHintRTL {
*   // ...
* 
*   &#64;include input-placeholder {
*     color: blue;
*     opacity: 1;
*   }
* &#125;
*  Accessibility concerns
*  The HTML5 specification notes that a placeholder should not be used as a replacement
*  for a title. The placeholder is intended to be a short hint that assists the user
*  who is entering a value into the empty field. The placeholder can be mistaken by some
*  users for a pre-filled value, particularly in Internet Explorer because the same color
*  is used, and the placeholder text color may provide insufficient contrast, particularly
*  in Firefox 19+ because of the default 0.4 opacity. Furthermore, not having a title reduces
*  the hit area available for setting focus on the item.
* @type {boolean}
* @default true
*/
usePlaceholderForHint?: boolean;

/**
* When TimeItem.useTextField is false, whether titles should be shown
*  for for child-items in this TimeItem. By default, showItemTitles is true,
*  and titles are displayed FormItem.titleOrientation.
* @type {boolean}
* @default true
*/
showItemTitles?: boolean;

/**
* When TimeItem.useTextField is false, the default orientation of 
*  titles for child-items, such as the TimeItem.hourItem, 
*  TimeItem.minuteItem and TimeItem.secondItem pickers.
*  TitleOrientation lists valid options.
*  
*  Note that titles on the left or right take up a cell in tabular
*  formLayout, but titles on top do not.
* @type {TitleOrientation}
* @default "top"
*/
itemTitleOrientation?: TitleOrientation;

/**
* When TimeItem.useTextField is false, the default title-alignment of 
*  child-items such as the TimeItem.hourItem, 
*  TimeItem.minuteItem and TimeItem.secondItem pickers,
*  within their cells.
* @type {Alignment}
* @default "center"
*/
itemTitleAlign?: Alignment;

/**
* Controls whether to display the TimeItem.hourItem when TimeItem.useTextField is false.
* @type {boolean}
* @default true
*/
showHourItem?: boolean;

/**
* Select item to hold the hours portion of the time or 
*  TimeItem.getDuration when TimeItem.useTextField 
*  is false.
* @type {SelectItem}
* @default null
*/
hourItem?: SelectItem;

/**
* Custom properties to apply to this timeItem's generated 
*  TimeItem.hourItem.
* @type {Partial<SelectItem>}
* @default null
*/
hourItemProperties?: Partial<SelectItem>;

/**
* Title to show for the TimeItem.hourItem.
* @type {HTMLString}
* @default "Hour"
*/
hourItemTitle?: HTMLString;

/**
* The hover prompt to show for the TimeItem.hourItem.
* @type {HTMLString}
* @default "Choose hours"
*/
hourItemPrompt?: HTMLString;

/**
* Controls whether to display the TimeItem.minuteItem when TimeItem.useTextField is false.
* @type {boolean}
* @default true
*/
showMinuteItem?: boolean;

/**
* Select item to hold the minutes portion of the time or 
*  TimeItem.getDuration when TimeItem.useTextField 
*  is false.
* @type {SelectItem}
* @default null
*/
minuteItem?: SelectItem;

/**
* Custom properties to apply to this timeItem's generated 
*  TimeItem.minuteItem.
* @type {Partial<SelectItem>}
* @default null
*/
minuteItemProperties?: Partial<SelectItem>;

/**
* Title to show for the TimeItem.minuteItem.
* @type {HTMLString}
* @default "Min"
*/
minuteItemTitle?: HTMLString;

/**
* The hover prompt to show for the TimeItem.minuteItem.
* @type {HTMLString}
* @default "Choose minutes"
*/
minuteItemPrompt?: HTMLString;

/**
* Controls whether to display the TimeItem.secondItem when TimeItem.useTextField is false.
* @type {boolean}
* @default true
*/
showSecondItem?: boolean;

/**
* Select item to hold the seconds portion of the time or 
*  TimeItem.getDuration when TimeItem.useTextField 
*  is false.
* @type {SelectItem}
* @default null
*/
secondItem?: SelectItem;

/**
* Custom properties to apply to this timeItem's generated 
*  TimeItem.secondItem.
* @type {Partial<SelectItem>}
* @default null
*/
secondItemProperties?: Partial<SelectItem>;

/**
* Title to show for the TimeItem.secondItem.
* @type {HTMLString}
* @default "Sec"
*/
secondItemTitle?: HTMLString;

/**
* The hover prompt to show for the TimeItem.secondItem.
* @type {HTMLString}
* @default "Choose seconds"
*/
secondItemPrompt?: HTMLString;

/**
* Controls whether to display the TimeItem.millisecondItem when TimeItem.useTextField is false.
* @type {boolean}
* @default false
*/
showMillisecondItem?: boolean;

/**
* Select item to hold the milliseconds portion of the time or 
*  TimeItem.getDuration when TimeItem.useTextField 
*  is false.
* @type {SelectItem}
* @default null
*/
millisecondItem?: SelectItem;

/**
* Custom properties to apply to this timeItem's generated 
*  TimeItem.millisecondItem.
* @type {Partial<SelectItem>}
* @default null
*/
millisecondItemProperties?: Partial<SelectItem>;

/**
* Title to show for the TimeItem.millisecondItem.
* @type {HTMLString}
* @default "Ms"
*/
millisecondItemTitle?: HTMLString;

/**
* The hover prompt to show for the TimeItem.millisecondItem.
* @type {HTMLString}
* @default "Choose milliseconds"
*/
millisecondItemPrompt?: HTMLString;

/**
* Select item to hold the AM/PM value for the timeItem when 
*  TimeItem.useTextField is false.
* @type {SelectItem}
* @default null
*/
ampmItem?: SelectItem;

/**
* Custom properties to apply to this timeItem's generated 
*  TimeItem.ampmItem.
* @type {Partial<SelectItem>}
* @default null
*/
ampmItemProperties?: Partial<SelectItem>;

/**
* Title to show for the TimeItem.ampmItem.
* @type {HTMLString}
* @default "AM/PM"
*/
ampmItemTitle?: HTMLString;

/**
* What format should this item's time string be presented in?
*  
*  This attribute does not have an effect if a native HTML5 time input is being used.
*  See TimeItem.browserInputType.
* @deprecated * in favor of TimeItem.timeFormatter

* @type {TimeDisplayFormat}
* @default null
*/
displayFormat?: TimeDisplayFormat;

/**
* If TimeItem.useTextField is true, what format should this item's time string be 
*  presented in?
*  
*  If unset, the default formatter will be TimeItem.timeFormatter24Hour or
*  TimeItem.timeFormatter12Hour depending on the value of TimeItem.use24HourTime.
*  If the property cannot be derived in this way (none of these properties are set),
*  we'll check DynamicForm.timeFormatter, or finally back off to the
*  standard system-wide Time.displayFormat will be applied.
*  
*  This attribute does not have an effect if a native HTML5 time input is being used.
*  See TimeItem.browserInputType.
* @type {TimeDisplayFormat}
* @default null
*/
timeFormatter?: TimeDisplayFormat;

/**
* If TimeItem.useTextField is true, and TimeItem.use24HourTime is true,
*  what format should this item's time string be presented in?
*  
*  May be overridden via an explicitly specified TimeItem.timeFormatter.
*  
*  This attribute does not have an effect if a native HTML5 time input is being used.
*  See TimeItem.browserInputType.
* @type {TimeDisplayFormat}
* @default "toShort24HourTime"
*/
timeFormatter24Hour?: TimeDisplayFormat;

/**
* If TimeItem.useTextField is true, and TimeItem.use24HourTime is false,
*  what format should this item's time string be presented in?
*  
*  May be overridden via an explicitly specified TimeItem.timeFormatter.
*  
*  This attribute does not have an effect if a native HTML5 time input is being used.
*  See TimeItem.browserInputType.
* @type {TimeDisplayFormat}
* @default "toShortTime"
*/
timeFormatter12Hour?: TimeDisplayFormat;

/**
* If true, a data entry mask will be enabled in the field based on the
*  TimeItem.timeFormatter.
*  
*  Note that if a non-padded TimeItem.timeFormatter is specified, it
*  will be changed to the corresponding padded version (ex. "toShort24HourTime"
*  will be changed to "toShortPadded24HourTime").
*  
*  This attribute does not have an effect if a native HTML5 time input is being used.
*  See TimeItem.browserInputType.
* @type {boolean}
* @default null
*/
useMask?: boolean;

/**
* Should we show the time in a text field, or as a number of SelectItems?
* @type {boolean}
* @default true
*/
useTextField?: boolean;

/**
* If TimeItem.useTextField is true, this property governs the alignment
*  of text within the text field. Defaults to "left" by default or
*  "right" if the page is in Page.isRTL.
*  
*  This attribute does not have an effect if a native HTML5 time input is being used.
*  See TimeItem.browserInputType.
* @type {Alignment}
* @default "varies"
*/
textAlign?: Alignment;

/**
* Validation error message to display if the user enters an invalid time string.
* @type {HTMLString}
* @default "Invalid time"
*/
invalidTimeStringMessage?: HTMLString;

/**
* Whether to enforce 24-hour time in the UI. If unset, assumes to the 
*  Time.use24HourTime.
* @type {boolean}
* @default null
*/
use24HourTime?: boolean;

/**
* Base CSS class for this item's text box.
*  If specified this style will be applied to the TimeItem.textField if 
*  TimeItem.useTextField is set to true.
* @type {CSSStyleName}
* @default null
*/
textBoxStyle?: CSSStyleName;

/**
* An array of values to make available in the TimeItem.hourItem when
*  TimeItem.useTextField is false.
*  Used for specifying a limited set of valid Hour values, or when using the 
*  TimeItem to record duration, rather than time per-se.
*  See TimeItem.hourMinValue, 
*  TimeItem.hourMaxValue and 
*  TimeItem.hourIncrement for another method of controlling the 
*  content in the hour picker.
* @type {Array<Partial<number>>}
* @default null
*/
hourValues?: Array<Partial<number>>;

/**
* When TimeItem.useTextField is false and 
*  TimeItem.hourValues is unset, this attribute specifies the minimum
*  value present in the hour picker.
*  Used for specifying a limited set of valid Hour values, or when using the 
*  TimeItem to record duration, rather than time per-se. The default is zero in all cases.
*  See also TimeItem.hourMaxValue and 
*  TimeItem.hourIncrement.
* @type {number}
* @default null
*/
hourMinValue?: number;

/**
* When TimeItem.useTextField is false and 
*  TimeItem.hourValues is unset, this attribute specifies the maximum
*  value present in the hour picker.
*  Used for specifying a limited set of valid Hour values, or when using the 
*  TimeItem to record duration, rather than time per-se. The default is 11 or 23, 
*  according to the value of TimeItem.use24HourTime and
*  TimeItem.timeFormatter.
*  See also TimeItem.hourMinValue and 
*  TimeItem.hourIncrement.
* @type {number}
* @default null
*/
hourMaxValue?: number;

/**
* When TimeItem.useTextField is false and 
*  TimeItem.hourValues is unset, this attribute specifies the increment
*  to use when generating entries for the hour picker. For example, if this attribute is
*  set to 5, the hour picker will contain only every fifth value between the 
*  TimeItem.hourMinValue and 
*  TimeItem.hourMaxValue.
* @type {number}
* @default null
*/
hourIncrement?: number;

/**
* An array of values to make available in the TimeItem.minuteItem when
*  TimeItem.useTextField is false.
*  Used for specifying a limited set of valid Minute values, or when using the 
*  TimeItem to record duration, rather than time per-se.
*  See TimeItem.minuteMinValue, 
*  TimeItem.minuteMaxValue and 
*  TimeItem.minuteIncrement for another method of controlling the 
*  content in the minute picker.
* @type {Array<Partial<number>>}
* @default null
*/
minuteValues?: Array<Partial<number>>;

/**
* When TimeItem.useTextField is false and 
*  TimeItem.minuteValues is unset, this attribute specifies the minimum
*  value present in the minute picker.
*  Used for specifying a limited set of valid Minute values, or when using the 
*  TimeItem to record duration, rather than time per-se. The default is zero in all cases.
*  See also TimeItem.minuteMaxValue and 
*  TimeItem.minuteIncrement.
* @type {number}
* @default null
*/
minuteMinValue?: number;

/**
* When TimeItem.useTextField is false and 
*  TimeItem.minuteValues is unset, this attribute specifies the maximum
*  value present in the minute picker.
*  Used for specifying a limited set of valid Minute values, or when using the 
*  TimeItem to record duration, rather than time per-se. The default 59.
*  See also TimeItem.minuteMinValue and 
*  TimeItem.minuteIncrement.
* @type {number}
* @default null
*/
minuteMaxValue?: number;

/**
* When TimeItem.useTextField is false and 
*  TimeItem.minuteValues is unset, this attribute specifies the increment
*  to use when generating entries for the minute picker. For example, if this attribute is
*  set to 5, the minute picker will contain only every fifth value between the 
*  TimeItem.minuteMinValue and 
*  TimeItem.minuteMaxValue.
* @type {number}
* @default null
*/
minuteIncrement?: number;

/**
* An array of values to make available in the TimeItem.secondItem when
*  TimeItem.useTextField is false.
*  Used for specifying a limited set of valid Second values, or when using the 
*  TimeItem to record duration, rather than time per-se.
*  See TimeItem.secondMinValue, 
*  TimeItem.secondMaxValue and 
*  TimeItem.secondIncrement for another method of controlling the 
*  content in the second picker.
* @type {Array<Partial<number>>}
* @default null
*/
secondValues?: Array<Partial<number>>;

/**
* When TimeItem.useTextField is false and 
*  TimeItem.secondValues is unset, this attribute specifies the minimum
*  value present in the second picker.
*  Used for specifying a limited set of valid Second values, or when using the 
*  TimeItem to record duration, rather than time per-se. The default is zero in all cases.
*  See also TimeItem.secondMaxValue and 
*  TimeItem.secondIncrement.
* @type {number}
* @default null
*/
secondMinValue?: number;

/**
* When TimeItem.useTextField is false and 
*  TimeItem.secondValues is unset, this attribute specifies the maximum
*  value present in the second picker.
*  Used for specifying a limited set of valid Second values, or when using the 
*  TimeItem to record duration, rather than time per-se. The default is 59.
*  See also TimeItem.secondMinValue and 
*  TimeItem.secondIncrement.
* @type {number}
* @default null
*/
secondMaxValue?: number;

/**
* When TimeItem.useTextField is false and 
*  TimeItem.secondValues is unset, this attribute specifies the increment
*  to use when generating entries for the second picker. For example, if this attribute is
*  set to 5, the second picker will contain only every fifth value between the 
*  TimeItem.secondMinValue and 
*  TimeItem.secondMaxValue.
* @type {number}
* @default null
*/
secondIncrement?: number;

/**
* An array of values to make available in the 
*  TimeItem.millisecondItem when
*  TimeItem.useTextField is false.
*  Used for specifying a limited set of valid Millisecond values, or when using the 
*  TimeItem to record duration, rather than time per-se.
*  See TimeItem.millisecondMinValue, 
*  TimeItem.millisecondMaxValue and 
*  TimeItem.millisecondIncrement for another method of 
*  controlling the content in the millisecond picker.
* @type {Array<Partial<number>>}
* @default null
*/
millisecondValues?: Array<Partial<number>>;

/**
* When TimeItem.useTextField is false and 
*  TimeItem.millisecondValues is unset, this attribute specifies 
*  the minimum value present in the millisecond picker.
*  Used for specifying a limited set of valid Millisecond values, or when using the 
*  TimeItem to record duration, rather than time per-se. The default is zero in all cases.
*  See also TimeItem.millisecondMaxValue and 
*  TimeItem.millisecondIncrement.
* @type {number}
* @default null
*/
millisecondMinValue?: number;

/**
* When TimeItem.useTextField is false and 
*  TimeItem.millisecondValues is unset, this attribute specifies 
*  the maximum value present in the millisecond picker.
*  Used for specifying a limited set of valid Millisecond values, or when using the 
*  TimeItem to record duration, rather than time per-se. The default is 999.
*  See also TimeItem.millisecondMinValue and 
*  TimeItem.millisecondIncrement.
* @type {number}
* @default null
*/
millisecondMaxValue?: number;

/**
* When TimeItem.useTextField is false and 
*  TimeItem.millisecondValues is unset, this attribute specifies 
*  the increment to use when generating entries for the millisecond picker. For example, 
*  if this attribute is set to 5, the millisecond picker will contain only every fifth 
*  value between the TimeItem.millisecondMinValue and 
*  TimeItem.millisecondMaxValue.
* @type {number}
* @default null
*/
millisecondIncrement?: number;



/**
*  Sets the array of valid TimeItem.hourValues to use when
*  TimeItem.useTextField is false.
*  Used for limiting available valid Hour values, or when using the TimeItem to record
*  duration, rather than time per-se.
*  See TimeItem.hourMinValue, 
*  TimeItem.hourMaxValue and 
*  TimeItem.hourIncrement for another method of controlling the 
*  content in the hour picker.
* 
* @param {Array<Partial<number>>} array of available Hour values
*/
setHourValues(values:Array<Partial<number>>): void;

/**
*  Returns an array of the current valid hour values, whether set directly as 
*  TimeItem.hourValues or generated according to 
*  TimeItem.hourMinValue, 
*  TimeItem.hourMaxValue and 
*  TimeItem.hourIncrement.
* 
*/
getHourValues(): Array<Partial<number>>;

/**
*  Sets the array of valid TimeItem.minuteValues to use when
*  TimeItem.useTextField is false.
*  Used for limiting available valid Minute values, or when using the TimeItem to record
*  duration, rather than time per-se.
*  See TimeItem.minuteMinValue, 
*  TimeItem.minuteMaxValue and 
*  TimeItem.minuteIncrement for another method of controlling the 
*  content in the minute picker.
* 
* @param {Array<Partial<number>>} array of available Minute values
*/
setMinuteValues(values:Array<Partial<number>>): void;

/**
*  Returns an array of the current valid minute values, whether set directly as 
*  TimeItem.minuteValues or generated according to 
*  TimeItem.minuteMinValue, 
*  TimeItem.minuteMaxValue and 
*  TimeItem.minuteIncrement.
* 
*/
getMinuteValues(): Array<Partial<number>>;

/**
*  Sets the array of valid TimeItem.secondValues to use when
*  TimeItem.useTextField is false.
*  Used for limiting available valid Second values, or when using the TimeItem to record
*  duration, rather than time per-se.
*  See TimeItem.secondMinValue, 
*  TimeItem.secondMaxValue and 
*  TimeItem.secondIncrement for another method of controlling the 
*  content in the second picker.
* 
* @param {Array<Partial<number>>} array of available Second values
*/
setSecondValues(values:Array<Partial<number>>): void;

/**
*  Returns an array of the current valid second values, whether set directly as 
*  TimeItem.secondValues or generated according to 
*  TimeItem.secondMinValue, 
*  TimeItem.secondMaxValue and 
*  TimeItem.secondIncrement.
* 
*/
getSecondValues(): Array<Partial<number>>;

/**
*  Sets the array of valid TimeItem.millisecondValues to use when
*  TimeItem.useTextField is false.
*  Used for limiting available valid Millisecond values, or when using the TimeItem to record
*  duration, rather than time per-se.
*  See TimeItem.millisecondMinValue, 
*  TimeItem.millisecondMaxValue and 
*  TimeItem.millisecondIncrement for another method of controlling the 
*  content in the millisecond picker.
* 
* @param {Array<Partial<number>>} array of available Millisecond values
*/
setMillisecondValues(values:Array<Partial<number>>): void;

/**
*  Returns an array of the current valid millisecond values, whether set directly as 
*  TimeItem.millisecondValues or generated according to 
*  TimeItem.millisecondMinValue, 
*  TimeItem.millisecondMaxValue and 
*  TimeItem.millisecondIncrement.
* 
*/
getMillisecondValues(): Array<Partial<number>>;

/**
*  Set the hour value of this TimeItem.
*  If the item value has not been initialized with TimeItem.setValue, 
*  the minute will be established to current minute.
*  
*  You can use TimeItem.setValue to set both hours and minutes at the same time.
*  
*    
* 
* @param {number} new hours value for this TimeItem.
*/
setHours(hours:number): void;

/**
*  Set the minute value of this TimeItem.
*  If the item value has not been initialized with TimeItem.setValue, 
*  the hour will be established to current hour.
*  
*  You can use TimeItem.setValue to set both hours and minutes at the same time.
*  
*    
* 
* @param {number} new minutes value for this TimeItem.
*/
setMinutes(minutes:number): void;

/**
*  Set the seconds value of this TimeItem.
* 
* @param {number} new seconds value for this TimeItem.
*/
setSeconds(seconds:number): void;

/**
*  Set the milliseconds value of this TimeItem.
* 
* @param {number} new milliseconds value for this TimeItem.
*/
setMilliseconds(milliseconds:number): void;

/**
*  When TimeItem.useTextField is set to false, this method 
*  returns the value of the time expressed as a duration in the TimeUnit
*  provided. If no timeUnit is passed, the default is the smallest unit for which a 
*  picker is visible.
* 
* @param {TimeUnit=} the unit of the return value
*/
getDuration(timeUnit?:TimeUnit): Integer;

/**
*  Notification method called when FormItem.showPending is enabled and
*  this time item should either clear or show its pending visual state.
*  
*  The default behavior is that the FormItem.titleStyle and
*  FormItem.cellStyle are updated to include/exclude the "Pending" suffix.
*  In addition, when displayed in the pending state:
*  
*  
* - If TimeItem.useTextField is true, then the "Pending"
*    suffix will be appended to the FormItem.textBoxStyle applied to the
*    TimeItem.textField; otherwise
*  
* - (useTextField is false) the color of the
*    TimeItem.hourItem, TimeItem.minuteItem,
*    TimeItem.secondItem, TimeItem.millisecondItem,
*    and/or TimeItem.ampmItem will change when the hour, minute, second,
*    millisecond, or whether the time is AM or PM is different, respectively.
*  
*  Returning false will cancel this default behavior.
* 
* @param {DynamicForm} the managing DynamicForm instance.
* @param {FormItem} the form item itself (also available as "this").
* @param {boolean} true if the item should show its pending visual state; false otherwise.
* @param {any} the current form item value.
* @param {any} the value that would be restored by a call to DynamicForm.resetValues.
*/
pendingStatusChanged(form:DynamicForm, item:FormItem, pendingStatus:boolean, newValue:any, value:any): boolean;

/**
*  Set the value of the form item to the value passed in
*  
*  NOTE: for valueMap'd items, newValue should be data value not displayed value
* 
* @param {any} value to set the element to
*/
setValue(newValue:any): void;

/**
*  Returns the raw text value typed into this items text field if TimeItem.useTextField 
*  is true (otherwise returns the result of this.getValue()).
* 
*/
getEnteredValue(): string;

/**
*  If TimeItem.useTextField is true, falls through to standard
*  TextItem.setSelectionRange implementation on this items freeform text entry field.
*  Otherwise has no effect.
* 
* @param {Integer} character index for start of new selection
* @param {Integer} character index for end of new selection
*/
setSelectionRange(start:Integer, end:Integer): void;

/**
*  If TimeItem.useTextField is true, falls through to standard
*  TextItem.getSelectionRange implementation on this items freeform text entry field.
*  Otherwise has no effect.
* 
*/
getSelectionRange(): Array<any>;

/**
*  If TimeItem.useTextField is true, falls through to standard
*  TextItem.selectValue implementation on this items freeform text entry field.
*  Otherwise has no effect.
* 
*/
selectValue(): void;

/**
*  If TimeItem.useTextField is true, falls through to standard
*  TextItem.deselectValue implementation on this items freeform text entry field.
*  Otherwise has no effect.
* 
* @param {boolean=} If this parameter is passed, new cursor insertion position will be  moved to the start, rather than the end of this item's value.
*/
deselectValue(start?:boolean): void;



}
/**
* Creates a new TimeItem
* 
* @param typeCheckedProperties {Partial<TimeItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TimeItem} The new instance
*/
class TimeItem{
static create(typeCheckedProperties? : Partial<TimeItem>, uncheckedProperties?: Object): TimeItem;







}

/**
* Timeline is a trivial subclass of Calendar that configures the Calendar with settings
*  typical for a standalone timeline view: hides the Calendar.dayView, 
*  Calendar.weekView and Calendar.monthView tabs and the 
*  Calendar.controlsBar by default.
*  
*  Note that the loadingOptionalModules must be loaded to make 
*  use of the Timeline class.
*/
interface Timeline extends Calendar {




}
/**
* Creates a new Timeline
* 
* @param typeCheckedProperties {Partial<Timeline>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Timeline} The new instance
*/
class Timeline{
static create(typeCheckedProperties? : Partial<Timeline>, uncheckedProperties?: Object): Timeline;







}

/**
* The Timer class provides a predictable cross-browser system for creating
*  timed events.
*/
interface Timer extends Class {




}
/**
* Creates a new Timer
* 
* @param typeCheckedProperties {Partial<Timer>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Timer} The new instance
*/
class Timer{
static create(typeCheckedProperties? : Partial<Timer>, uncheckedProperties?: Object): Timer;




/**
*   
*  Execute an action in a given amount of time. This method wraps the native setTimeout() method,
*  correcting for browser-specific memory leaks.
* 
* 
* @param {string | Function} Function to be called when delay has elapsed.      Can also be a string representation of an expression.     Passing a string is preferred.
* @param {number} Time until action is executed (in milliseconds). If not specified, the            default is 100 milliseconds.
*/
static setTimeout(action:string | Function, delay:number): any;

/**
* 
*  Cancels the processing of a timerEvent if it has not already fired.
* 
* 
* @param {object} timerEvent object previously returned from Timer.setTimeout()
*/
static clear(timerEvent:object): void;




}

/**
* Base class for creating toolstrips like those found in browsers and office applications: a
*  mixed set of controls including ImgButton, 
*  Button.radioGroup, MenuButton,
*  ComboBoxItem, LayoutSpacer, Label and 
*  SelectItem. 
*  
*  All of the above components are placed in the ToolStrip.members to form
*  a ToolStrip. Note that the FormItem mentioned above (ComboBox and
*  drop-down selects) need to be placed within a DynamicForm as usual.
*  
*  
*  The following strings can be used to add special behaviors:
*  
*  
* - the String "separator" will cause a separator to be created (instance of 
*  ToolStrip.separatorClass)
*  
* - the String "resizer" will cause a resizer to be created (instance of
*  ToolStrip.resizeBarClass). This is equivalent to setting
*  Canvas.showResizeBar on the preceding member.
*  
* - the String "starSpacer" will cause a spacer to be created (instance of
*  LayoutSpacer).
*  
*  
*  
*/
interface ToolStrip extends Layout {
/**
* Array of components that will be contained within this Toolstrip, like
*  Layout.members. Built-in special behaviors can be indicated as
*  describe ToolStrip.
* @type {Array<Partial<Canvas>>}
* @default null
*/
members?: Array<Partial<Canvas>>;

/**
* ToolStrips set a default Canvas.height to avoid being stretched by
*  containing layouts.
* @type {number}
* @default "20"
*/
height?: number;

/**
* CSS class applied to this toolstrip.
*  
*  Note that if ToolStrip.vertical is true for this toolStrip, 
*  ToolStrip.verticalStyleName will be used instead of this value if it is non-null.
* @type {CSSStyleName}
* @default "toolStrip"
*/
styleName?: CSSStyleName;

/**
* Default stylename to use if ToolStrip.vertical is true.
*  If unset, the standard ToolStrip.styleName will be used for both vertical and horizontal
*  toolstrips.
*  
*  Note that this property only applies to the widget at init time. To modify the 
*  styleName after this widget has been initialized, you should
*  simply call Canvas.setStyleName rather than updating this 
*  property.
* @type {CSSStyleName}
* @default null
*/
verticalStyleName?: CSSStyleName;

/**
* Indicates whether the components are drawn horizontally from left to right (false), or
*  vertically from top to bottom (true).
* @type {boolean}
* @default false
*/
vertical?: boolean;

/**
* Customized resizeBar with typical appearance for a ToolStrip.
* @type {string}
* @default "ToolStripResizer"
*/
resizeBarClass?: string;

/**
* Thickness of the resizeBars in pixels.
* @type {Integer}
* @default "14"
*/
resizeBarSize?: Integer;

/**
* Class to create when the string "separator" appears in ToolStrip.members.
* @type {string}
* @default "ToolStripSeparator"
*/
separatorClass?: string;

/**
* Separator thickness in pixels
* @type {Integer}
* @default "8"
*/
separatorSize?: Integer;

/**
* DynamicForm instance created by ToolStrip.addFormItem to contain form items for
*  display in this toolStrip. Each time addFormItem() is run, a new formWrapper
*  autoChild will be created, picking up properties according to the standard
*  AutoChild pattern.
* @type {DynamicForm}
* @default null
*/
formWrapper?: DynamicForm;

/**
* SmartClient class for generated ToolStrip.formWrapper components.
* @type {string}
* @default "DynamicForm"
*/
formWrapperConstructor?: string;

/**
* Default properties to apply to ToolStrip.formWrapper components.
*  
*  The default configuration will has the following settings:
*  
*  
* - DynamicForm.numCols set to 2
*  
* - Canvas.overflow set to "visible"
*  
* - Canvas.width and Canvas.height set to 1
*  
*  To customize the wrapper form for an individual item, use the formProperties argument
*  of ToolStrip.addFormItem.
* @type {Partial<DynamicForm>}
* @default "..."
*/
formWrapperDefaults?: Partial<DynamicForm>;

/**
* Properties to apply to ToolStrip.formWrapper components.
* @type {Partial<DynamicForm>}
* @default null
*/
formWrapperProperties?: Partial<DynamicForm>;



/**
*  Add a form item to this toolStrip. This method will create a DynamicForm autoChild with the
*  item passed in as a single item, based on the 
*  ToolStrip.formWrapper, and add it to the toolstrip
*  as a member.
*  
*  Returns a pointer to the generated formWrapper component.
* 
* @param {Partial<FormItem>} properties for the form item to add to this toolStrip.
* @param {Partial<DynamicForm>=} properties to apply to the generated formWrapper component. If passed, specified properties will be overlaid onto the  properties derived from ToolStrip.formWrapperDefaults and  ToolStrip.formWrapperProperties.
* @param {Integer=} desired position for the form item in the tools
*/
addFormItem(formItem:Partial<FormItem>, formProperties?:Partial<DynamicForm>, position?:Integer): DynamicForm;



}
/**
* Creates a new ToolStrip
* 
* @param typeCheckedProperties {Partial<ToolStrip>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ToolStrip} The new instance
*/
class ToolStrip{
static create(typeCheckedProperties? : Partial<ToolStrip>, uncheckedProperties?: Object): ToolStrip;







}

/**
* Simple subclass of Button with appearance appropriate for a ToolStrip button.
*  Can be used to create an icon-only button, and icon with text, or a text only button by setting the 
*  icon and title attributes as required.
*/
interface ToolStripButton extends Button {




}
/**
* Creates a new ToolStripButton
* 
* @param typeCheckedProperties {Partial<ToolStripButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ToolStripButton} The new instance
*/
class ToolStripButton{
static create(typeCheckedProperties? : Partial<ToolStripButton>, uncheckedProperties?: Object): ToolStripButton;







}

/**
* A simple subclass of RibbonGroup, which groups other controls for use in 
*  RibbonBar.
* @deprecated * this is an old synonym for RibbonGroup, scheduled for removal in the next release

*/
interface ToolStripGroup extends RibbonGroup {




}
/**
* Creates a new ToolStripGroup
* 
* @param typeCheckedProperties {Partial<ToolStripGroup>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ToolStripGroup} The new instance
*/
class ToolStripGroup{
static create(typeCheckedProperties? : Partial<ToolStripGroup>, uncheckedProperties?: Object): ToolStripGroup;







}

/**
* Simple subclass of MenuButton with appearance appropriate for a ToolStrip menu button.
*  Can be used to create an icon-only menu button, and icon with text, or a text only button by setting the 
*  icon and title attibutes as required.
*/
interface ToolStripMenuButton extends MenuButton {




}
/**
* Creates a new ToolStripMenuButton
* 
* @param typeCheckedProperties {Partial<ToolStripMenuButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ToolStripMenuButton} The new instance
*/
class ToolStripMenuButton{
static create(typeCheckedProperties? : Partial<ToolStripMenuButton>, uncheckedProperties?: Object): ToolStripMenuButton;







}

/**
* Simple subclass of ImgSplitbar with appearance appropriate for a ToolStrip resizer.
*/
interface ToolStripResizer extends ImgSplitbar {
/**
* Path to resizer image.
* @type {SCImgURL}
* @default "images/ToolStrip/"
*/
skinImgDir?: SCImgURL;

/**
* Image for resizer
* @type {SCImgURL}
* @default "[SKIN]resizer.png"
*/
vSrc?: SCImgURL;

/**
* Image for horizontal resizer for a vertical Toolstrip
* @type {SCImgURL}
* @default "[SKIN]hresizer.png"
*/
hSrc?: SCImgURL;





}
/**
* Creates a new ToolStripResizer
* 
* @param typeCheckedProperties {Partial<ToolStripResizer>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ToolStripResizer} The new instance
*/
class ToolStripResizer{
static create(typeCheckedProperties? : Partial<ToolStripResizer>, uncheckedProperties?: Object): ToolStripResizer;







}

/**
* Simple subclass of Img with appearance appropriate for a ToolStrip separator
*/
interface ToolStripSeparator extends Img {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "ToolStripSeparatorEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Path to separator image.
* @type {SCImgURL}
* @default "images/ToolStrip/"
*/
skinImgDir?: SCImgURL;

/**
* Image for vertically oriented separator (for horizontal toolstrips).
* @type {SCImgURL}
* @default "[SKIN]separator.png"
*/
vSrc?: SCImgURL;

/**
* Image for horizontally oriented separator (for vertical toolstrips).
* @type {SCImgURL}
* @default "[SKIN]hseparator.png"
*/
hSrc?: SCImgURL;





}
/**
* Creates a new ToolStripSeparator
* 
* @param typeCheckedProperties {Partial<ToolStripSeparator>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ToolStripSeparator} The new instance
*/
class ToolStripSeparator{
static create(typeCheckedProperties? : Partial<ToolStripSeparator>, uncheckedProperties?: Object): ToolStripSeparator;







}

/**
* EditProxy that handles ToolStripSeparator objects when editMode is enabled.
*/
interface ToolStripSeparatorEditProxy extends Class {




}
/**
* Creates a new ToolStripSeparatorEditProxy
* 
* @param typeCheckedProperties {Partial<ToolStripSeparatorEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ToolStripSeparatorEditProxy} The new instance
*/
class ToolStripSeparatorEditProxy{
static create(typeCheckedProperties? : Partial<ToolStripSeparatorEditProxy>, uncheckedProperties?: Object): ToolStripSeparatorEditProxy;







}

/**
* Simple subclass of LayoutSpacer with appearance appropriate for a ToolStrip spacer
*/
interface ToolStripSpacer extends LayoutSpacer {
/**
* Size of spacer. If not specified, spacer fills remaining space.
* @type {number}
* @default null
*/
space?: number;





}
/**
* Creates a new ToolStripSpacer
* 
* @param typeCheckedProperties {Partial<ToolStripSpacer>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ToolStripSpacer} The new instance
*/
class ToolStripSpacer{
static create(typeCheckedProperties? : Partial<ToolStripSpacer>, uncheckedProperties?: Object): ToolStripSpacer;







}

/**
* A Toolbar creates a vertical or horizontal strip of similar components (typically Buttons)
*  and provides managed resizing and reordering behavior over those components.
*  
*  If you are creating a bar with a mixture of different elements (eg some MenuButtons, some
*  Labels, some Buttons, some custom components), you want to use a ToolStrip. A
*  Toolbar is better suited for managing a set of highly similar, interchangeable components,
*  such as ListGrid headers.
*/
interface Toolbar extends Layout {
/**
* An array of button object initializers. See the Button Widget Class for standard
*  button properties. The following additional properties can also be specified for
*  button sizing and positioning on the toolbar itself:
*  
* - width--Specifies the width of this button as an absolute number of pixels, a
*  named property of the toolbar that specifies an absolute number of pixels, a
*  percentage of the remaining space (e.g. '60%'), or "*" (default) to allocate an
*  equal portion of the remaining space.
*  
* - height--Specifies the height of this button.
*  
* - extraSpace--Specifies an optional amount of extra space, in pixels, to separate
*  this button from the next button in the toolbar.
* @type {Array<Partial<Partial<Button>>>}
* @default null
*/
buttons?: Array<Partial<Partial<Button>>>;

/**
* Indicates whether the buttons are drawn horizontally from left to right (false), or
*  vertically from top to bottom (true).
* @type {boolean}
* @default false
*/
vertical?: boolean;

/**
* Default constructor for toolbar items.
* @type {Class}
* @default "Button"
*/
buttonConstructor?: Class;

/**
* If true, items can be reordered by dragging on them.
* @type {boolean}
* @default false
*/
canReorderItems?: boolean;

/**
* If true, items (buttons) can be resized by dragging on them.
* @type {boolean}
* @default false
*/
canResizeItems?: boolean;

/**
* If true, items (buttons) can be dropped into this toolbar, and the toolbar will
*  show a drop line at the drop location. Override drop() to decide what happens when the
*  item is dropped.
* @type {boolean}
* @default false
*/
canAcceptDrop?: boolean;

/**
* Settings to apply to all buttons of a toolbar. Properties that can be applied to
*  button objects can be applied to all buttons of a toolbar by specifying them in
*  buttonDefaults using the following syntax:
*  buttonDefaults:{property1:value1, property2:value2, ...}
*  See the Button Widget Class for standard button properties.
* @type {object}
* @default "varies"
*/
buttonDefaults?: object;

/**
* If set to true, causes child buttons to be created during initialization, instead of waiting until
*  draw().
*  
*  This property principally exists for backwards compatibility; the default behavior of waiting 
*  until draw makes certain pre-draw operations more efficient (such as adding, removing or 
*  reordering buttons). However, if you have code that assumes Buttons are created early
*  and crashes if they are not, createButtonsOnInit will allow that code to
*  continue working, with a minor performance penalty.
* @type {boolean}
* @default null
*/
createButtonsOnInit?: boolean;



/**
*  Apply a new set of buttons to render in this toolbar as Toolbar.buttons.
*  
* 
* @param {Array<Partial<Partial<Button>>>=} properties to create each button from
*/
setButtons(newButtons?:Array<Partial<Partial<Button>>>): void;

/**
*  Add a list of buttons to the toolbar
* 
* @param {Array<Partial<object>>=} list of button object initializers.
* @param {number=} position to add the new buttons at
*/
addButtons(buttons?:Array<Partial<object>>, position?:number): void;

/**
* 	Remove a list of buttons from the toolbar
* 
* 
* @param {Array<any>=} Array of buttons to remove. Buttons may be specified as pointers to the button instances contained in this toolbar, or numbers indicating the index of the buttons in this.buttons.
*/
removeButtons(buttons?:Array<any>): void;

/**
*      Retrieves a button widget instance (within this toolbar) from the name / ID / index / 
*      descriptor object for the button (as with the getButtonNumber() method)
*      This provides a way to access a toolbar button's properties and methods directly.
* 
* @param {number | string | object} identifier for the button to retrieve
*/
getButton(index:number | string | object): Button;

/**
*  	get the index of a button in the buttons array
*      The button can be specified as - 
*      
*      
* - an index within this.buttons (just returned)
*      
* - the ID property of a button
*      
* - a pointer to the button descriptor object in this.buttons
*      
* - the actual button widget in this.members
*      
*  	returns -1 if not found
* 
* 
* @param {number | string | any | any} */
getButtonNumber(button:number | string | any | any): number;

/**
*  Setter for updating Toolbar.canResizeItems at runtime.
* 
* @param {boolean} New value for this.canResizeItems
*/
setCanResizeItems(canResizeItems:boolean): void;

/**
*  Given an identifier for a button, select it.
*  The button identifier can be a number (index), string (id), or object (widget or init block),
*  as with the getButtonNumber() method.
*  
* 
* @param {number | string | object | Canvas} Button / Button identifier
*/
selectButton(buttonID:number | string | object | Canvas): void;

/**
* 	Deselects the specified button from the toolbar, where buttonID is the index of
*  the button's object initializer. The button will be redrawn if necessary.
*  The button identifier can be a number (index), string (id), or object (widget or init block),
*  as with the getButtonNumber() method.
* 
* @param {number | string | object | Canvas} Button / Button identifier
*/
deselectButton(buttonID:number | string | object | Canvas): void;

/**
*  Called when one of the buttons receives a click event.
* 
* @param {StatefulCanvas} button in question
* @param {number} number of the button in question
*/
itemClick(item:StatefulCanvas, itemNum:number): void;

/**
* 	Called when one of the buttons receives a double-click event
* 
* @param {Button} pointer to the button in question
* @param {number} number of the button in question
*/
itemDoubleClick(item:Button, itemNum:number): void;

/**
*  Observable, overrideable method - called
*  when one of the Toolbar buttons is drag resized.
* 
* 
* @param {number} the index of the item that was resized
* @param {number} the new size of the item
*/
itemDragResized(itemNum:number, newSize:number): void;



}
/**
* Creates a new Toolbar
* 
* @param typeCheckedProperties {Partial<Toolbar>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Toolbar} The new instance
*/
class Toolbar{
static create(typeCheckedProperties? : Partial<Toolbar>, uncheckedProperties?: Object): Toolbar;







}

/**
* Set of horizontally arranged buttons.
*/
interface ToolbarItem extends CanvasItem {
/**
* List of buttons for the toolbar.
*  Each button should be specified as a simple JS object with properties to apply to the
*  button to be displayed. Note that any click stringMethod applied to the
*  button will be passed 2 parameters: form and item.
* @type {Array<Partial<StatefulCanvas>>}
* @default null
*/
buttons?: Array<Partial<StatefulCanvas>>;

/**
* Don't show a title for toolbars
* @type {boolean}
* @default false
*/
showTitle?: boolean;

/**
* Space between the buttons of this toolbar. Configures the Layout.membersMargin 
*  property on the created ToolbarItem.canvas.
* @type {number}
* @default "4"
*/
buttonSpace?: number;

/**
* these items are in a row by themselves by default
* @type {boolean}
* @default true
*/
startRow?: boolean;

/**
* these items are in a row by themselves by default
* @type {boolean}
* @default true
*/
endRow?: boolean;

/**
* these items span all columns
* @type {Integer | string}
* @default "*"
*/
colSpan?: Integer | string;

/**
* This item is an autoChild generated Canvas displayed by
*  the ToolbarItem and is an instance of Toolbar by default, 
*  customizable via the ToolbarItem.canvasConstructor attribute.
* @type {Canvas}
* @default null
*/
canvas?: Canvas;

/**
* Constructor class for this toolbarItem's ToolbarItem.canvas.
* @type {string}
* @default "Toolbar"
*/
canvasConstructor?: string;

/**
* Should the toolbar stack its buttons vertically or horizontally?
* @type {boolean}
* @default false
*/
vertical?: boolean;

/**
* Default properties for this toolbar's buttons.
* @type {object}
* @default null
*/
buttonProperties?: object;

/**
* Default Button.autoFit for buttons - true by default. Note that 
*  autoFit:true buttons will fit to their title regardless of specified width.
* @type {boolean}
* @default true
*/
buttonAutoFit?: boolean;

/**
* If specified this baseStyle will be applied to the buttons in this toolbar.
* @type {CSSStyleName}
* @default null
*/
buttonBaseStyle?: CSSStyleName;

/**
* If set to true, causes the toolbar created by this item to create its child buttons 
*  during initialization, instead of waiting until draw().
*  
*  See the corresponding Toolbar.createButtonsOnInit for more
*  information.
* @type {boolean}
* @default null
*/
createButtonsOnInit?: boolean;





}
/**
* Creates a new ToolbarItem
* 
* @param typeCheckedProperties {Partial<ToolbarItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ToolbarItem} The new instance
*/
class ToolbarItem{
static create(typeCheckedProperties? : Partial<ToolbarItem>, uncheckedProperties?: Object): ToolbarItem;







}

/**
* The Tour class allows you to build tours and tutorials for end users, highlighting specific 
*  components on the page and prompting for specific user-interactions.
*  
*  Tours are a specific type of Process and 
*  consist of a series of TourStep. Typically each step refers to an element 
*  on the page, which is either being highlighted for the user, or, for a tutorial, is an element 
*  that the user is being prompted to interact with (see TourStep.clickTarget, for example).
*  
*  Most steps have a TourWindow, which shows explanatory or instructional text. For tutorial steps 
*  that request a user action, the target can be highlighted and a large arrow is shown pointing at 
*  the target.
*  
*  The placement of the TourWindow and arrow is automatic, and intelligently takes into account the 
*  type of interaction requested. For example, if a tutorial requires a user to change a value in a 
*  SelectItem, the TourWindow and arrow will avoid appearing underneath the select, where they could 
*  be occluded by the drop-down picklist. Similarly, for a drag and drop interaction, the TourWindow 
*  avoids overlapping both the drag source and the drop area.
*  
*  Because of all this automatic behavior, a tutorial step typically consists of just:
*  
*  1. explanatory text (tourStep.title and tourStep.description)
*  2. the target
*  3. the expected interaction (eg click the target, or type something in, or drag something to 
*  another widget)
*  4. validation of input (what was typed or chosen), if applicable
*  
*  Developers may use AutoTest.getLocator to specify specific components 
*  or elements as action targets. Using locators means that tours and tutorials will typically still 
*  work even if your application changes (for example, new controls are added). Note that we recommend 
*  making use of the AutoTest.installLocatorShortcut to rapidly generate 
*  locators.
*  
*  A tour is typically written as a .proc.xml document stored in the folder configured 
*  via the project.processes setting in server_properties
*  (webroot/processes by default). They may be loaded via the Tour.loadTour method.
*  
*  The Tour feature is available with Power or better licenses only See 
*  smartclient.com/product for details.
*  It requires the standard DataBinding and loadingOptionalModules Tour module.
*/
interface Tour extends Process {
/**
* The ProcessElements representing steps in this tour. These steps are
*  typically TourSteps.
* @type {Array<Partial<ProcessElement>>}
* @default null
*/
steps?: Array<Partial<ProcessElement>>;

/**
* The ProcessElements representing steps that should be performed if
*  the user cancels the tour. These steps are typically TourSteps.
*  
*  Alternately, or in addition to these cancelation steps, the Process.finished
*  method will be called once there are no more steps to process in the tour.
* @type {Array<Partial<ProcessElement>>}
* @default null
*/
cancelSteps?: Array<Partial<ProcessElement>>;

/**
* Should a progress bar be shown in each step to indicate progress through the tour?
* @type {boolean}
* @default null
*/
showProgress?: boolean;

/**
* Show progress percent next to progress bar when Tour.showProgress is enabled?
* @type {boolean}
* @default null
*/
showProgressPercent?: boolean;

/**
* Should tour be auto-reset once it finishes in any manner so it can restarted?
* @type {boolean}
* @default true
*/
autoReset?: boolean;

/**
* The tour mode allows step configuration to be simplified by using the context to
*  apply appropriate defaults.
* @type {TourMode}
* @default "tour"
*/
mode?: TourMode;

/**
* Default value for TourStep.allowDropOnDescendants within this tour.
* @type {boolean}
* @default false
*/
allowDropOnDescendants?: boolean;

/**
* Default value for TourStep.actionButtonTitle for steps other than the first
*  and last ones.
*  
*  Default titles for the first and last steps are configured by
*  Tour.firstStepActionButtonTitle and
*  Tour.lastStepActionButtonTitle respectively.
* @type {string}
* @default "Next"
*/
stepActionButtonTitle?: string;

/**
* Default value of TourStep.actionButtonTitle for the first step in the tour.
*  
*  Default titles for the middle and last steps are configured by
*  Tour.stepActionButtonTitle and
*  Tour.lastStepActionButtonTitle respectively.
* @type {string}
* @default "Let's go!"
*/
firstStepActionButtonTitle?: string;

/**
* Default value of TourStep.actionButtonTitle for the last step in the tour.
*  
*  Default titles for the first and middle steps are configured by
*  Tour.firstStepActionButtonTitle and
*  Tour.stepActionButtonTitle respectively.
* @type {string}
* @default "Done"
*/
lastStepActionButtonTitle?: string;

/**
* Default value for TourStep.cancelButtonTitle for all steps where the
*  cancel button is shown.
*  
*  Default title for the first step is configured by
*  Tour.firstStepCancelButtonTitle.
* @type {string}
* @default "Skip this tour"
*/
stepCancelButtonTitle?: string;

/**
* Default value of TourStep.cancelButtonTitle for the first step in the tour.
*  
*  Default title for the middle steps is configured by
*  Tour.stepCancelButtonTitle. No cancel button is shown
*  for the last step.
* @type {string}
* @default "Skip this tour"
*/
firstStepCancelButtonTitle?: string;

/**
* Should TourStep.inputValidationNotifyMessage be shown
*  as detailed in TourInputValidationMode for the entire tour?
*  Set to false to suppress reporting messages to the user.
*  Tour.notifyValidationMessage will not be called.
*  
*  This setting can be overridden for individual TourStep.
* @type {boolean}
* @default null
*/
showInputValidationMessage?: boolean;

/**
* Set the CSS border of the outline of non-drop targets as a CSS string including border-width,
*  border-style, and/or color (eg "1px dashed blue"). See Tour.dropOutlineBorder for
*  properties applied to drop targets.
*  
*  This property applies the same border to all four sides of the outlined component(s).
* @type {string}
* @default "1px solid red"
*/
outlineBorder?: string;

/**
* Set the CSS border of the outline for drop targets as a CSS string including border-width,
*  border-style, and/or color (eg "1px dashed blue"). See Tour.outlineBorder for
*  properties applied to non-drop targets.
*  
*  This property applies the same border to all four sides of the outlined component(s).
* @type {string}
* @default "1px solid #44ff44"
*/
dropOutlineBorder?: string;

/**
* Defaults for each TourStep.targetView autoChild in the tour.
*  These defaults are applied before TourStep.windowDefaults if any.
* @type {Partial<Canvas>}
* @default "{...}"
*/
windowDefaults?: Partial<Canvas>;

/**
* Category of message to use in +{notifyValidationMessage,notifyValidationMessage} by default.
*  
*  If this category has not been configured in Notify it will be automatically
*  configured using +{notifyMessageSettings,notifyMessageSettings}.
* @type {NotifyType}
* @default "tourExpectedValue"
*/
notifyType?: NotifyType;

/**
* Defaults settings for NotifyType message displayed by
*  Tour.notifyValidationMessage.
* @type {NotifySettings}
* @default null
*/
notifyMessageSettings?: NotifySettings;



/**
*  Notify user of expected value entry validation message. Default implementation uses the 
*  Notify to show the message in it's default position. See
*  NotifyType for notification configuration details.
*  
*  Override this method to display message by a different means.
* 
* 
* @param {string} message provided by TourStep.inputValidationNotifyMessage
*/
notifyValidationMessage(message:string): void;



}
/**
* Creates a new Tour
* 
* @param typeCheckedProperties {Partial<Tour>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Tour} The new instance
*/
class Tour{
static create(typeCheckedProperties? : Partial<Tour>, uncheckedProperties?: Object): Tour;




/**
*  Loads an XML tour definition stored in XML from the server.
*  
*  Tour files are stored as .proc.xml files in componentXML
*  format, in the directory indicated by the project.processes setting in
*  server_properties
*  (webroot/processes by default). To load a tour
*  saved in a file tourId.proc.xml, pass just tourId to this method.
*  
* 
* @param {Identifier | Array<Partial<Identifier>>} tour ID or IDs to load
* @param {ProcessCallback} called when the tour is loaded with argument              "process", the first tour. Other tours can be looked               up via Tour.getTour.
*/
static loadTour(tourId:Identifier | Array<Partial<Identifier>>, callback:(process:Process) => void): void;

/**
*  Get a Tour instance by it's ID.
* 
* @param {Identifier} tour ID to retrieve
*/
static getTour(tourId:Identifier): Tour;




}

/**
* Chooses one or another next process element based on confirmation of a message shown to user.
*  
*  If the user clicks OK, the TourConfirmStep.nextElement is
*  chosen, otherwise the choice is TourConfirmStep.failureElement.
*/
interface TourConfirmStep extends UserTask {
/**
* Title for the Window.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* Text to show in body of window.
* @type {HTMLString}
* @default null
*/
instructions?: HTMLString;

/**
* Should the TourConfirmStep.targetView TourWindow.showCancelButton be shown
*  for this step?
*  
*  If no value is provided it will be defaulted to true.
* @type {boolean}
* @default null
*/
showCancelButton?: boolean;

/**
* Applied directly to TourConfirmStep.targetView.
* @type {HTMLString}
* @default ""Cancel"
*/
cancelButtonTitle?: HTMLString;

/**
* Should the TourConfirmStep.targetView TourWindow.showActionButton be shown
*  for this step?
*  
*  If no value is provided it will be defaulted to true.
* @type {boolean}
* @default null
*/
showActionButton?: boolean;

/**
* Applied directly to TourConfirmStep.targetView.
* @type {HTMLString}
* @default "OK"
*/
actionButtonTitle?: HTMLString;

/**
* Automatically generated view (typically a window) to show for tour step.
*  
*  The following autoChildUsage apply:
*  
*  
* - TourConfirmStep.showCancelButton for the TourWindow.showCancelButton
*  
* - TourConfirmStep.cancelButtonTitle for the TourWindow.cancelButtonTitle
*  
* - TourConfirmStep.actionButtonTitle for the TourWindow.actionButtonTitle
*  
* @type {Canvas}
* @default null
*/
targetView?: Canvas;

/**
* The name of the widget class (as a string) to use for the target view.
* @type {string}
* @default "TourWindow"
*/
targetViewConstructor?: string;

/**
* Defaults for the TourConfirmStep.targetView autoChild.
* @type {Partial<Canvas>}
* @default "{...}"
*/
targetViewDefaults?: Partial<Canvas>;

/**
* Next ProcessElement to execute if the user clicks the
*  actionButton.
*  
*  nextElement does not need to be specified if this gateway is part of a
*  Process.sequences and has a next element in the sequence. This is normal
*  for a tour.
* @type {string}
* @default null
*/
nextElement?: string;

/**
* ID of the next element to process if the user clicks the cancelButton.
*  If not set, the process will exit.
* @type {string}
* @default null
*/
failureElement?: string;





}
/**
* Creates a new TourConfirmStep
* 
* @param typeCheckedProperties {Partial<TourConfirmStep>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TourConfirmStep} The new instance
*/
class TourConfirmStep{
static create(typeCheckedProperties? : Partial<TourConfirmStep>, uncheckedProperties?: Object): TourConfirmStep;







}

/**
* User task configuration for a step within a Tour. As each step is executed, it will show 
*  a TourWindow to the user (the automatically generated TourStep.targetView).
*  
*  At a minimum tourSteps will typically have a TourStep.title and/or 
*  ProcessElement.description. Without a TourStep.target the
*  window will be shown in the center of the screen.
*  
*  To relate the step to a component provide a TourStep.target value. An
*  TourStep.actionType value determines what action, if any, is expected
*  with the target. Some actionTypes like "none" and "any"
*  just highlight the target for review and the user must click the next button to complete
*  the step. Others like "click" or "change" expect the user to interact with the target
*  component in a certain way. Many of these will advance to the next step when the
*  required action is completed. See TourActionType for 
*  details.
*  
*  In most cases the actionType can be left unset because a default is applied
*  based on the other step properties and the Tour.mode. However, actions like
*  doubleClick, rightClick and mouseOver must be provided.
*  
*  To explicitly suppress automatic completion of a step on user interaction, devlopers may
*  set TourStep.autoComplete to false.
*/
interface TourStep extends UserTask {
/**
* Title for the Window.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* Text to show in body of window.
* @type {HTMLString}
* @default null
*/
instructions?: HTMLString;

/**
* Applied directly to TourStep.targetView Canvas.pointerSettings.
* @type {string}
* @default null
*/
pointerSnapTo?: string;

/**
* Should the TourStep.targetView TourWindow.showCancelButton be shown
*  for this step?
*  
*  If no value is provided it will be defaulted to true unless this step is
*  the last step in the tour.
* @type {boolean}
* @default null
*/
showCancelButton?: boolean;

/**
* Applied directly to TourStep.targetView. Defaults from Tour.stepCancelButtonTitle.
* @type {HTMLString}
* @default null
*/
cancelButtonTitle?: HTMLString;

/**
* Should the TourStep.targetView TourWindow.showActionButton be shown
*  for this step?
*  
*  If no value is provided it will be defaulted to true when
*  AutoComplete is not false and any of the following
*  are true:
*  
*   
* - TourStep.actionType is "click"
*   
* - actionType is "mouseDown"
*   
* - actionType is "doubleClick"
*   
* - actionType is "rightClick"
*   
* - actionType is "menuItemOpen"
*   
* - actionType is "menuItemSelect"
*   
* - actionType is "change" and an TourStep.expectedValue is provided
*   
* - a TourStep.dropTarget is provided
*  
*  
*  If no value is provided and autoComplete is false but auto completion
*  would normally occur (i.e. actionType "click", "change" with expectedValue, etc.)
*  then the action button is shown but is disabled until the expected action is completed
*  at which time the button is enabled so the user can continue.
* @type {boolean}
* @default null
*/
showActionButton?: boolean;

/**
* Applied directly to TourStep.targetView. Defaults from Tour.stepActionButtonTitle.
* @type {HTMLString}
* @default null
*/
actionButtonTitle?: HTMLString;

/**
* Automatically generated view (typically a TourWindow) to show for tour step.
*  
*  The following autoChildUsage apply:
*  
*  
* - TourStep.pointerSnapTo for the Canvas.pointerSettings
*  
* - TourStep.showCancelButton for the TourWindow.showCancelButton
*  
* - TourStep.cancelButtonTitle for the TourWindow.cancelButtonTitle
*  
* - TourStep.actionButtonTitle for the TourWindow.actionButtonTitle
*  
* @type {TourWindow}
* @default null
*/
targetView?: TourWindow;

/**
* The name of the widget class (as a string) to use for the target view.
* @type {string}
* @default "TourWindow"
*/
targetViewConstructor?: string;

/**
* Defaults for the TourStep.targetView autoChild.
* @type {Partial<Canvas>}
* @default "{...}"
*/
targetViewDefaults?: Partial<Canvas>;

/**
* Defaults for the TourStep.targetView autoChild. These defaults are applied
*  after Tour.windowDefaults if any.
* @type {Partial<Canvas>}
* @default "{...}"
*/
windowDefaults?: Partial<Canvas>;

/**
* Component that is the target of this step. The interaction for this step is indicated
*  in TourStep.actionType. For actions that need multiple targets, this target
*  is the primary one.
*  
*  Target may be specified as a GlobalId for a Component or FormItem, or as a
*  AutoTestLocator for an element.
* @type {string}
* @default null
*/
target?: string;

/**
* Indicates the type of action that is expected with the TourStep.target.
*  
*  If not specified a default type is determined by the Tour.mode setting, the
*  target and other tourStep properties.
*  
*  Details:
*  
*  when tourMode is "tour"
*    
*     
* - then actionType = "none"
*    
*  
*  when tourMode is "tutorial"
*    
*     
* - and expectedValue or afterInputTarget are set
*      
*       
* - then actionType = "change"
*      
*     
*     
* - and dropTarget is set
*      
*       
* - then actionType = "drag"
*      
*     
*     
* - otherwise
*      
*       
* - actionType = "click"
*      
*     
*    
* @type {TourActionType}
* @default null
*/
actionType?: TourActionType;

/**
* Target component on which a TourStep.target must be dropped to complete step.
*  If TourStep.actionType is not specified and this property is provided,
*  actionType defaults to "drag".
*  
*  Target may be specified as a GlobalId for a Component or FormItem, or as a
*  AutoTestLocator for an element.
* @type {string}
* @default null
*/
dropTarget?: string;

/**
* For TourStep.actionType tourSteps with a TourStep.dropTarget,
*  should we allow the drop to complete and the tour to proceed if the user drops on a
*  descendant of the drop target.
*  
*  For example - if a user has been prompted to drag a component into a Layout, but 
*  the layout already contains a nested child layout, should the user be able to 
*  drop into that child layout and continue the tour?
*  
*  If unset this will be derived from Tour.allowDropOnDescendants
* @type {boolean}
* @default null
*/
allowDropOnDescendants?: boolean;

/**
* Target component which must clicked to process TourStep.expectedValue entered into
*  TourStep.target with TourStep.actionType of "change".
*  
*  This is commonly used for a dialog where the must select a value and then click
*  a submit button.
*  
*  Target may be specified as a GlobalId for a Component or FormItem, or as a
*  AutoTestLocator for an element.
* @type {string}
* @default null
*/
afterInputTarget?: string;

/**
* Should the target(s) be outlined? Defaults to true in
*  Tour.mode:"tutorial".
* @type {boolean}
* @default null
*/
showOutline?: boolean;

/**
* Should a Canvas.showPointer be shown pointing to the 
*  target? Defaults to true in Tour.mode:"tour".
* @type {boolean}
* @default null
*/
showPointer?: boolean;

/**
* For a step with an TourStep.actionType a large arrow shows to indicate the
*  target or direction of required action. Setting showArrow to
*  false suppresses the display of the arrow. Note that the tour window
*  is shown as if the arrow were there.
*  
*  TourStep.showPointer takes precedence over showArrow:true.
* @type {boolean}
* @default null
*/
showArrow?: boolean;

/**
* The expected value for TourStep.actionType:"change". The value must be
*  matched exactly ignoring case for string values. A string value can also be matched
*  case-sensitive by setting TourStep.expectedValueCaseSensitive
*  to true.
*  
*  A regular expression can be used to match the entered value by providing the
*  expected value as a string with leading and trailing slashes (ex. "/get.&#42;/").
* @type {string | boolean | number | Date}
* @default null
*/
expectedValue?: string | boolean | number | Date;

/**
* Should the TourStep.expectedValue be matched as case-sensitive?
* @type {boolean}
* @default null
*/
expectedValueCaseSensitive?: boolean;

/**
* How should the TourStep.expectedValue be validated and notified to the user?
*  
*  Defaults to "onExit" if TourStep.afterInputTarget is specified. Otherwise the default
*  is "notify".
* @type {TourInputValidationMode}
* @default null
*/
inputValidation?: TourInputValidationMode;

/**
* The time between keypresses that is used during TourStep.inputValidation before showing
*  the expected text notification. Does not apply to inputValidation="onExit".
* @type {Integer}
* @default "500"
*/
inputValidationNotifyDelay?: Integer;

/**
* Should TourStep.inputValidationNotifyMessage be shown
*  as detailed in TourInputValidationMode? Defaults from
*  Tour.showInputValidationMessage if not set.
*  
*  If set to false reporting messages to the user are suppressed and
*  therefore Tour.notifyValidationMessage will not be called.
* @type {boolean}
* @default null
*/
showInputValidationMessage?: boolean;

/**
* Message to be displayed by Tour.notifyValidationMessage.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables
*  and will be evaluated as JS code when the message is displayed.
*  
*  Only two dynamic variables, value and expectedValue, are available and represent
*  the currently entered and expected values.
* @type {string}
* @default "Please enter '${expectedValue}'"
*/
inputValidationNotifyMessage?: string;

/**
* When set on a step with an TourStep.actionType of "change",
*  the entered or selected value will be automatically bound to the specified value
*  in the Process.state.
* @type {string}
* @default null
*/
bindEnteredText?: string;

/**
* If this step has a target that prompts a user for a specific action, should 
*  the step be automatically completed and have the tour move forward to the 
*  next step when the user takes this action?
*  
*  Has no effect except on TourActionType that can
*  auto-complete.
*  
*  If autoComplete is false the action button is shown but is disabled
*  until the expected action is completed at which time the button is enabled so the
*  user can continue.
* @type {boolean}
* @default true
*/
autoComplete?: boolean;





}
/**
* Creates a new TourStep
* 
* @param typeCheckedProperties {Partial<TourStep>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TourStep} The new instance
*/
class TourStep{
static create(typeCheckedProperties? : Partial<TourStep>, uncheckedProperties?: Object): TourStep;







}

/**
* A specific purpose Window class for showing individual steps in a tour.
*/
interface TourWindow extends Window {
/**
* Title for this Window, shown if Window.showTitle is true.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* Text to show in body of window.
* @type {HTMLString}
* @default null
*/
contents?: HTMLString;

/**
* Additional content components to show below TourWindow.contents or TourWindow.title if there
*  are no other content.
* @type {Array<Partial<Canvas>>}
* @default null
*/
contentComponents?: Array<Partial<Canvas>>;

/**
* If true, show a cancel button below the contents which calls
*  TourWindow.cancelClick when clicked.
* @type {boolean}
* @default null
*/
showCancelButton?: boolean;

/**
* Title for the TourWindow.cancelButton, shown if TourWindow.showCancelButton is true.
* @type {HTMLString}
* @default "Skip this tour"
*/
cancelButtonTitle?: HTMLString;

/**
* If true, show an action button below the contents which calls 
*  TourWindow.actionClick when clicked.
* @type {boolean}
* @default true
*/
showActionButton?: boolean;

/**
* Title for the TourWindow.actionButton, shown if TourWindow.showActionButton is true.
* @type {HTMLString}
* @default "Next"
*/
actionButtonTitle?: HTMLString;

/**
* URL to open in another tab or window when the the TourWindow.actionButton is clicked.
*  The normal TourWindow.actionClick event is still called.
* @type {URL}
* @default null
*/
actionButtonURL?: URL;

/**
* Button show below contents that will call TourWindow.cancelClick when clicked.
* @type {IButton}
* @default null
*/
cancelButton?: IButton;

/**
* Button show below contents that will call TourWindow.actionClick when clicked.
* @type {IButton}
* @default null
*/
actionButton?: IButton;

/**
* Should a progress bar be shown to indicate progress through the tour?
* @type {boolean}
* @default null
*/
showProgress?: boolean;

/**
* Show progress percent next to progress bar when TourWindow.showProgress is enabled?
*  The percent can also be shown over the progress bar by setting
*  TourWindow.showProgressPercentInline.
* @type {boolean}
* @default null
*/
showProgressPercent?: boolean;

/**
* Show progress percent over the progress bar instead of next to it
*  when TourWindow.showProgress is enabled?
* @type {boolean}
* @default null
*/
showProgressPercentInline?: boolean;



/**
*  Handles a click on the cancel button of this window. The default implementation
*  calls Window.close and returns false to prevent bubbling of the click event.
*  
*  Override this method if you want
*  other actions to be taken.
*  
*  Custom implementations may call close() to trigger the default behavior.
* 
*/
cancelClick(): boolean;

/**
*  Handles a click on the action button of this window. No default implementation.
*  
*  Implement this method to act on this event.
*  
* 
*/
actionClick(): boolean;



}
/**
* Creates a new TourWindow
* 
* @param typeCheckedProperties {Partial<TourWindow>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TourWindow} The new instance
*/
class TourWindow{
static create(typeCheckedProperties? : Partial<TourWindow>, uncheckedProperties?: Object): TourWindow;







}

/**
* A Tree is a data model representing a set of objects linked into a hierarchy.
*  
*  A Tree has no visual presentation, it is displayed by a TreeGrid or ColumnTree
*  when supplied as TreeGrid.data or ColumnTree.data.
*  
*  A Tree can be constructed out of a List of objects interlinked by IDs or via explicitly
*  specified Arrays of child objects. See Tree.modelType for an explanation of how
*  to pass data to a Tree.
*  
*  Typical usage is to call TreeGrid.fetchData to cause automatic creation of a 
*  ResultTree, which is a type of Tree that automatically handles loading data on 
*  demand. For information on DataBinding Trees, see treeDataBinding.
*/
interface Tree extends Class, Omit<List, 'duplicate' | 'add' | 'addList' | 'removeList'>  {
/**
* Selects the model used to construct the tree representation. See TreeModelType for
*  the available options and their implications.
*  
*  If the "parent" modelType is used, you can provide the initial parent-linked data set to the
*  tree via the Tree.data attribute. If the "children" modelType is used, you can
*  provide the initial tree structure to the Tree via the Tree.root attribute.
* @type {TreeModelType}
* @default "children"
*/
modelType?: TreeModelType;

/**
* Name of property that defines whether a node is a folder. By default this is set to
*  TreeNode.isFolder.
* @type {string}
* @default "isFolder"
*/
isFolderProperty?: string;

/**
* Controls whether nodes are assumed to be folders or leaves by default.
*  
*  Nodes that have children or have the Tree.isFolderProperty set to true will be considered
*  folders by default. Other nodes will be considered folders or leaves by default according
*  to this setting.
*  
*  See also ResultTree.defaultIsFolder for more details on how
*  defaultIsFolder interacts with 
*  TreeGrid.loadDataOnDemand.
* @type {boolean}
* @default null
*/
defaultIsFolder?: boolean;

/**
* If new nodes are added to a tree with modelType:"parent" which have the same
*  Tree.idField as existing nodes, the existing nodes are removed when
*  the new nodes are added.
*  
*  If reportCollisions is true, the Tree will log a warning in the developer console about this.
*  
*  Note that if an id collision occurs between a new node and its ancestor, the ancestor will be
*  removed and the new node will not be added to the tree.
* @type {boolean}
* @default true
*/
reportCollisions?: boolean;

/**
* Specifies the delimiter between node names. The pathDelim is used to construct a unique
*  path to each node. A path can be obtained for any node by calling
*  Tree.getPath and can be used to find any node in the tree by calling
*  Tree.find. Note that you can also hand-construct a path - in other words
*  you are not required to call Tree.getPath in order to later use
*  Tree.find to retrieve it.
*  
*  The pathDelim can be any character or sequence of characters, but must be a unique string
*  with respect to the text that can appear in the Tree.nameProperty that's used
*  for naming the nodes. So for example, if you have the following tree:
*  
*  one
*   two
*    three/four
*  
*  Then you will be unable to find the three/four node using
*  Tree.find if your tree is using the default pathDelim of /.
*  In such a case, you can use a different pathDelim for the tree. For example if you used |
*  for the path delim, then you can find the three/four node in the tree above by
*  calling tree.find("one|two|three/four").
*  
*  The pathDelim is used only by Tree.getPath and Tree.find and
*  does not affect any aspect of the tree structure or other forms of tree navigation (such as
*  via Tree.getChildren).
* @type {string}
* @default "/"
*/
pathDelim?: string;

/**
* Name of the property on a TreeNode that holds a name for the node that is unique
*  among its immediate siblings, thus allowing a unique path to be used to identify the node,
*  similar to a file system. Default value is "name". See TreeNode.name for usage.
* @type {string}
* @default "name"
*/
nameProperty?: string;

/**
* Name of the property on a TreeNode that holds the title of the node as it should be
*  shown to the user. Default value is "title". See TreeNode.title for usage.
* @type {string}
* @default "title"
*/
titleProperty?: string;

/**
* Name of the property on a TreeNode that holds an id for the node which is unique
*  across the entire Tree. Required for all nodes for trees with modelType "parent".
*  Default value is "id". See TreeNode.id for usage.
* @type {string}
* @default "id"
*/
idField?: string;

/**
* For trees with modelType "parent", this property specifies the name of the property
*  that contains the unique parent ID of a node. Default value is "parentId". See
*  TreeNode.parentId for usage.
* @type {string}
* @default "parentId"
*/
parentIdField?: string;

/**
* For trees with the modelType "children", this property specifies the name of the property
*  that contains the list of children for a node.
* @type {string}
* @default "children"
*/
childrenProperty?: string;

/**
* The property consulted by the default implementation of Tree.isOpen to determine if the
*  node is open or not. By default, this property is auto-generated for you, but you can set
*  it to a custom value if you want to declaratively specify this state, but be careful - if
*  you display this Tree in multiple TreeGrids at the same time, the open state will not be
*  tracked independently - see sharingNodes for more info on this.
*  
*  For Tree.isMultiLinkTrees, we do not track open state on the nodes 
*  themselves, because this would mean that multiple instances of a node in the tree would open
*  and close in lockstep. Instead, open state is tracked in an internal index structure, and
*  the openProperty is not used at all.
* @type {string}
* @default null
*/
openProperty?: string;

/**
* Optional initial data for the tree. How this data is interpreted depends on this tree's
*  Tree.modelType.
*  
*  If modelType is "parent", the list that you provide will be passed 
*  to Tree.linkNodes, integrating the nodes into the tree.
*  
*  In this case the root node may be supplied explicitly via Tree.root, or auto generated,
*  picking up its id via Tree.rootValue. Any nodes in the data with no
*  explicitly specified TreeNode.parentId will be added as children to this root element.
*  
*  To create this tree:
*  
*  foo
*   bar
*  zoo
*  
*  with modelType:"parent", you can do this:
*  
*  Tree.create({
*   data: [
*    {name: "foo", id: "foo"},
*    {name: "bar", id: "bar", parentId: "foo"},
*    {name: "zoo", id: "zoo"}
*  });
*  
*  Or this (explicitly specified root):
*  
*  Tree.create({
*   root: {id: "root"},
*   data: [
*    {name: "foo", id: "foo", parentId: "root"},
*    {name: "bar", id: "bar", parentId: "foo"},
*    {name: "zoo", id: "zoo", parentId: "root"}
*  });
*  
*  Or this (explicitly specified rootValue):
*  
*  Tree.create({
*   rootValue: "root",
*   data: [
*    {name: "foo", id: "foo", parentId: "root"},
*    {name: "bar", id: "bar", parentId: "foo"},
*    {name: "zoo", id: "zoo", parentId: "root"}
*  });
*  
*  Specifying the root node explicitly allows you to give it a name, changing the way path
*  derivation works (see Tree.root for more on naming the root node).
*  
*  For modelType:"children" trees, the data passed in will be assumed to be an 
*  array of children of the tree's root node.
* @type {Array<Partial<TreeNode>>}
* @default null
*/
data?: Array<Partial<TreeNode>>;

/**
* If you are using the "parent" modelType and did not specify a root node via Tree.root
*  with an id (Tree.idField), then you can provide the root node's id via this property.
*  
*  This setting is invalid if TreeGrid.keepParentsOnFilter is set and 
*  TreeGrid.dataFetchMode is set to anything other than
*  "paged" - a root-value cannot be 
*  used in this case because there is no efficient way to load a subtree to include all
*  parents. If fetchMode is specifically set to "paged", the rootValue is passed
*  as criteria and the DSRequest.keepParentsOnFilter flag is set on 
*  the request so the server knows to use special filtering rules, but these rules are not 
*  built-in - the developer is responsible for this logic.
*  
*  See Tree.data for an example.
* @type {string | number}
* @default null
*/
rootValue?: string | number;

/**
* If you're using the "parent" modelType, you can provide the root node configuration via this
*  property. If you don't provide it, one will be auto-created for you with an empty name.
*  Read on for a description of what omitting the name property on the root node means for path
*  derivation.
*  
*  If you're using the "children" modelType, you can provide the initial tree data via this
*  property. So, for example, to construct the following tree:
*  
*  foo
*   bar
*  zoo
*  
*  You would initialize the tree as follows: 
*  
*  
*  Tree.create({
*    root: { name:"root", children: [
*      { name:"foo", children: [
*        { name: "bar" }
*      ]},
*      { name: "zoo" }
*    ]}
*  });
*  
*  Note that if you provide a name property for the root node, then the path to
*  any node underneath it will start with that name. So in the example above, the path to the
*  bar node would be root/foo/bar (assuming you're using the default
*  Tree.pathDelim. If you omit the name attribute on the root node, then its name
*  is automatically set to the Tree.pathDelim value. So in the example above, if
*  you omitted name:"root", then the path to the bar node would be
*  /foo/bar.
*  
*  
*  
*  Note: if you initialize a Tree with no root value, a root node will be
*  auto-created for you. You can then call Tree.add to construct the tree.
* @type {TreeNode}
* @default null
*/
root?: TreeNode;

/**
* If this tree has Tree.modelType, should nodes in the data array for the
*  tree be dropped if they have an explicitly specified value for the Tree.parentIdField
*  which doesn't match any other nodes in the tree. If set to false these nodes will be added as
*  children of the root node.
* @type {boolean}
* @default false
*/
discardParentlessNodes?: boolean;

/**
* Controls whether the implicit root node is returned as part of the visible tree,
*  specifically, whether it is returned in Tree.getOpenList, which is the API view
*  components typically use to get the list of visible nodes.
*  
*  Default is to have the root node be implicit and not included in the open list, which means
*  that the visible tree begins with the children of root. This allows multiple nodes to
*  appear at the top level of the tree.
*  
*  You can set showRoot:true to show the single, logical root node as the only
*  top-level node. This property is only meaningful for Trees where you supplied a value for
*  Tree.root, otherwise, you will see an automatically generated root node that is
*  meaningless to the user.
* @type {boolean}
* @default false
*/
showRoot?: boolean;

/**
* If true, the root node is automatically opened when the tree is created or
*  Tree.setRoot is called.
* @type {boolean}
* @default true
*/
autoOpenRoot?: boolean;

/**
* Should folders be sorted separately from leaves or should nodes be ordered according to
*  their sort field value regardless of whether the node is a leaf or folder?
* @type {boolean}
* @default false
*/
separateFolders?: boolean;

/**
* If Tree.separateFolders is true, should folders be displayed above or below leaves?
*  When set to true folders will appear above leaves when the
*  sortDirection applied to the tree is SortDirection
* @type {boolean}
* @default true
*/
sortFoldersBeforeLeaves?: boolean;

/**
* Title assigned to nodes without a Tree.titleProperty value or a
*  Tree.nameProperty value.
* @type {string}
* @default "Untitled"
*/
defaultNodeTitle?: string;

/**
* If true, indicates this is a "multiLink" tree - ie, one that can contain the same node in more
*  than one place. Note, multiLink trees must use the "parent" 
*  Tree.modelType
*  
*  See Tree.linkData and ResultTree.linkDataSource for further details of 
*  multiLink trees.
* @type {boolean}
* @default null
*/
multiLinkTree?: boolean;

/**
* For a Tree.multiLinkTree, this property specifies the parent-child 
*  relationships between the nodes. The nodes themselves are provided in Tree.data.
*  Note that multi-link trees must specify a Tree.modelType of "parent".
*  
*  For a regular, non-multiLink tree, the linkData property is ignored.
*  
*  Minimally, the link data should include a node id, parent id and optionally the position of 
*  the child within that parent 
*  To describe this multi-link tree:
*   foo
*    bar
*     baz
*    zoo
*     bar
*      baz
*  
*  you would provide node information in the data property like this:
*   [
*    {id: "foo"},
*    {id: "bar"},
*    {id: "baz"},
*    {id: "zoo"}
*   ]
*  
*  and link information in linkData like this:
*   [
*    {id: "bar", parentId: "foo"},
*    {id: "baz", parentId: "bar"},
*    {id: "zoo", parentId: "foo"},
*    {id: "bar", parentId: "zoo"}
*   ]
*  
*  For information on databinding multi-link trees, and further discussion on multi-link trees
*  generally, see ResultTree.linkDataSource
* @type {Array<Partial<Record>>}
* @default null
*/
linkData?: Array<Partial<Record>>;

/**
* For a Tree.isMultiLinkTree, indicates whether client-side filtering
*  is allowed on the fields of the ResultTree.linkDataSource. When this 
*  property is true, filtering operations involving link fields work as expected (ie, as if those 
*  fields were present on the main ResultTree.dataSource); when this value is
*  not true, criterions involving link fields are simply ignored.
*  
*  Note, setting this property true causes filtering operations to perform an additional record
*  duplication per node in the dataset to be filtered. This adds some overhead, so you should
*  consider likely data volumes before enabling it (though in fact, client-side filtering of 
*  trees is relatively expensive anyway, so acceptable use cases probably already involve quite
*  low data volumes)
*  
*  This property has no effect for regular, non-multiLink trees.
* @type {boolean}
* @default null
*/
allowFilterOnLinkFields?: boolean;

/**
* The name of the "position" field in this Tree.linkData's link data.
*  Ignored if this tree is not a multi-link tree. Note,
*  these values are only used to order the nodes within a parent - so a node with link position
*  -123 will appear before a node with link position 87, but that is the only significance of 
*  the link position values
* @type {string}
* @default "position"
*/
linkPositionField?: string;

/**
* Specifies what DataSource this tree is associated with.
*  
*  A DataSource is required when filtering a tree, even if it isn't a ResultTree,
*  though it may be passed to Tree.getFilteredTree rather than set on the tree itself. If
*  a DataSource is specified it will also affect sorting, where relevant, such as if the
*  tree is set as TreeGrid.data.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;



/**
*  Create a copy of tree. If includeData is true, the tree nodes are copied.
*  Otherwise, just the tree settings and an empty root node are in the new tree.
* 
* 
* @param {boolean=} Should tree nodes be copied?
* @param {boolean=} Should tree node loadState be retained?
*/
duplicate(includeData?:boolean, includeLoadState?:boolean): Tree;

/**
* 
*  Return true if the passed node is the root node.
* 
* 
* @param {TreeNode} node to test
*/
isRoot(node:TreeNode): boolean;

/**
*  Adds an array of tree nodes into a Tree of Tree.modelType "parent".  
*  
*  The provided TreeNodes must contain, at a minimum, a field containing a unique ID for the
*  node (specified by Tree.idField) and a field containing the ID of the node's 
*  parent node (specified by Tree.parentIdField).
*  
*  This method handles receiving a mixture of leaf nodes and parent nodes, even out of order and
*  with any tree depth.
*  
*  Nodes may be passed with the Tree.childrenProperty already populated with an Array of
*  children that should also be added to the Tree, and this is automatically handled.
* 
* 
* @param {Array<Partial<TreeNode>>} list of nodes to link into the tree.
*/
linkNodes(nodes:Array<Partial<TreeNode>>): void;

/**
*  Returns true if this is a multi-link tree - ie, one that can contain the same node in
*  more than one place. Otherwise, returns false.
*  The default implementation simply returns the value of the 
*  multiLinkTree flag
*  
*  See Tree.linkData and ResultTree.linkDataSource for further details of 
*  multiLink trees.
* 
* 
*/
isMultiLinkTree(): void;

/**
*  For a Tree.isMultiLinkTree, this method returns the
*  NodeLocator associated with the particular occurence of the node at the
*  specified index within the current Tree.getOpenList of nodes in the tree. 
*  Not applicable to non-multilink trees (always returns null)
* 
* 
* @param {Integer} position of a node occurence within the open list of the tree
*/
getNodeLocator(recordIndex:Integer): NodeLocator;

/**
*  This method returns the path to the node at the specified index within the current open
*  list of nodes in this tree. Note that for a node that appears in more than one place in a
*  Tree.isMultiLinkTree, the returned path will be the visible
*  path to the node in the specified index.
*  
*  See Tree.getPath for more information on paths for TreeNodes.
* 
* 
* @param {Integer} position of a node within the open list of the tree
*/
getPathForOpenListIndex(recordIndex:Integer): string;

/**
*  Returns a NodeLocator object suitable for passing to methods, such as
*  Tree.getParent, which require a NodeLocator when the tree is 
*  Tree.isMultiLinkTree. Note, NodeLocators are specific to
*  multiLink trees; they are never required for regular trees.
* 
* 
* @param {TreeNode} the child node
* @param {TreeNode} the parent node
* @param {Integer} the child node's position within the parent
* @param {string} the full path to the child node
* @param {Integer=} the index of the node occurence in the tree's current openList.                This is the same as the record index of the node in an                 associated TreeGrid
*/
createNodeLocator(node:TreeNode, parent:TreeNode, position:Integer, path:string, openListIndex?:Integer): void;

/**
* 
*  Returns the root node of the tree.
* 
* 
*/
getRoot(): TreeNode;

/**
* 
*  Set the root node of the tree. Called automatically on this Tree during initialization
*  and on the Tree returned from a call to Tree.duplicate. 
* 
* 
* @param {TreeNode} new root node
* @param {boolean} set to true to automatically open the new root node.
*/
setRoot(newRoot:TreeNode, autoOpen:boolean): void;

/**
* 
*  Get the 'name' of a node. This is node[Tree.nameProperty]. If that value has not
*  been set on the node, the node's 'ID' value will be tried (this is 
*  node[Tree.idField]). If that value is not present on the node, a unique value 
*  (within this parent) will be auto-generated and returned.
* 	
* 
* @param {TreeNode | NodeLocator} node in question, or a suitable NodeLocator
*/
getName(node:TreeNode | NodeLocator): string;

/**
* 
*  Return the title of a node -- the name as it should be presented to the user. This method
*  works as follows:
*  
*  
* - If a Tree.titleProperty is set on the node, the value of that property is
*  returned.
*  
* - Otherwise, if the Tree.nameProperty is set on the node, that value is
*  returned, minus any trailing Tree.pathDelim.
*  
* - Finally, if none of the above yielded a title, the value of
*  Tree.defaultNodeTitle is returned.
*  
*  You can override this method to return the title of your choice for a given node.
*  
*  To override the title for an auto-constructed tree (for example, in a databound TreeGrid),
*  override TreeGrid.getNodeTitle instead.
* 
* 
* @param {TreeNode} node for which the title is being requested
*/
getTitle(node:TreeNode): string;

/**
* 
*  Returns the path of a node - a path has the following format:
*  ([name][pathDelim]?)*
*  
*  For example, in this tree:
*  
*  root
*   foo
*    bar
*  
*  Assuming that Tree.pathDelim is the default /, the bar
*  node would have the path root/foo/bar and the path for the foo
*  node would be root/foo.
*  
*  Once you have a path to a node, you can call find(path) to retrieve a reference to the node
*  later.
*  
*  Note: Nodes in Tree.isMultiLinkTree do not have a single path, 
*  because a given node can occur in multiple places in the tree. Therefore, if you pass a
*  TreeNode instance to this method, it returns the path to one occurence of the 
*  node; which particular occurence it chooses is not predictable, and there may be other paths
*  to other occurences of the same node in the tree. The only way to obtain an unambiguous 
*  path for a particular occurence of a node is to call Tree.getPathForOpenListIndex, 
*  passing in the position of the node occurence in the tree's openList (which will be the same 
*  as the record number of the node's visual occurence in a TreeGrid); if
*  the node occurence is not yet in the tree's openList - either because its parent has not yet
*  been opened, or because the tree is in the process of being built - the tree is not able to 
*  provide a path to the node occurence. In this case, you would have to obtain the path 
*  in application code, by reference to the original Tree.data and Tree.linkData 
* 
* 
* @param {TreeNode} node in question
*/
getPath(node:TreeNode): string;

/**
* 
*  Given a node, return the path to its parent. This works just like
*  Tree.getPath except the node itself is not reported as part of the path.
* 
* 
* @param {TreeNode} node in question
*/
getParentPath(node:TreeNode): string;

/**
* 
*  Returns the parent of this node. For Tree.isMultiLinkTree, you must
*  pass in a NodeLocator rather than a node, otherwise it will just return
*  the first parent, which is unlikely to be useful unless you know that this node only has 
*  one parent, or you just want to know whether the node has at least one parent. See also
*  Tree.getMultiLinkParents.
* 
* 
* @param {TreeNode | string | Integer | NodeLocator} the node in question, or its ID,                                or a NodeLocator object
*/
getParent(node:TreeNode | string | Integer | NodeLocator): TreeNode;

/**
* 
*  Returns true if "parent" is the parent of "node". This is straightforward and definitive 
*  for ordinary trees, because nodes can only have one parent. In
*  Tree.isMultiLinkTree, however, nodes can have multiple parents, so
*  this method returning true only means that "parent" is a parent of "node" - there may
*  or may not be others.
* 
* 
* @param {TreeNode} the node in question
* @param {TreeNode} the node to query to see if is a parent of the other node
*/
isParent(node:TreeNode, parent:TreeNode): boolean;

/**
* 
*  For Tree.isMultiLinkTree, returns the array of this node's direct
*  parents (the actual node objects, not the IDs). For non-multiLink trees, returns an array 
*  containing the single parent of this node. See also Tree.getParentsAndPositions.
* 
* 
* @param {TreeNode} node in question
*/
getMultiLinkParents(node:TreeNode): Array<Partial<TreeNode>>;

/**
* 
*  Given a node, return an array of the node's ancestors, with the immediate parent first, then
*  the grandparent, and so on. The node itself is not included in the result. For example, 
*  for the following tree:
*  
*  root
*   foo
*    bar
*  
*  Calling tree.getParents(bar) would return: [foo, root]. Note that
*  the returned array will contain references to the nodes, not the names.
*  
*  Note, for reasons of backwards compatibility, if you pass this method a TreeNode
*  instance on a Tree.isMultiLinkTree, it will return an array 
*  representing one path through the node's ancestors, which is unlikely to be useful. To get 
*  the ancestor chain of a specific node occurence, you must pass a NodeLocator
*  that specifies the full ID-based path to that occurence. If what you actually want is a 
*  list of the node's direct parents, see Tree.getMultiLinkParents.
* 
* 
* 
* @param {TreeNode | NodeLocator} node in question, or a NodeLocator
*/
getParents(node:TreeNode | NodeLocator): Array<any>;

/**
* 
*  For Tree.isMultiLinkTree, returns the array of this node's direct
*  parents and the node's position within each parent. Each entry is a record like this:
*  
*  [
*    {parent: [reference-to-parent-node], position: [this-node's-position-within-the-parent]},
*    {parent: [reference-to-parent-node], position: [this-node's-position-within-the-parent]}
*  ]
*  
*  For non-multiLink trees, returns null (calling this method makes no sense for non-multiLink
*  trees).
* 
* 
* @param {TreeNode} node in question
*/
getParentsAndPositions(node:TreeNode): Array<Partial<Record>>;

/**
* 
*  Return the number of levels deep this node is in the tree. For example, for this tree:
*  
*  root
*   foo
*    bar
*  
*  Calling tree.getLevel(bar) will return 2. 
*  
*  Note Tree.showRoot defaults to false so that multiple nodes can be shown at top level. In
*  this case, the top-level nodes still have root as a parent, so have level 1, even though
*  they have no visible parents.
*  
*  For Tree.isMultiLinkTree, passing a TreeNode to this 
*  method will return the level of one of that node's occurences; it is not predictable which
*  occurence will be used. For multi-link trees, therefore, you should pass a 
*  NodeLocator with a path that unambiguously identifies the node occurence you 
*  are interested in
* 
* 
* @param {TreeNode | NodeLocator} node in question, or a suitable                        NodeLocator
*/
getLevel(node:TreeNode | NodeLocator): number;

/**
* 
*  Determines whether a particular node is a folder. The logic works as follows:
*  
*  
* - If the TreeNode has a value for the Tree.isFolderProperty
*  (TreeNode.isFolder by default) that value is returned.
*  
* - Next, the existence of the Tree.childrenProperty (by default
*  TreeNode.children) is checked on the TreeNode. If the node has the children
*  property defined (regardless of whether it actually has any children), then isFolder()
*  returns true for that node.
*  
*  
*  
*  You can override this method to provide your own interpretation of what constitutes a folder.
*  However, you cannot change the return value for a node after the associated folder is loaded.
*  
* 
* 
* @param {TreeNode} node in question
*/
isFolder(node:TreeNode): boolean;

/**
* 
*  Returns true if the passed in node is a leaf.
* 
* 
* @param {TreeNode} node in question
*/
isLeaf(node:TreeNode): boolean;

/**
* 
*  Find the node with the specified ID. Specifically, it returns the node whose idField
*  matches the id passed to this method. If the tree is using the "parent" modelType, this
*  lookup will be constant-time. For all other modelTypes, the tree will be searched
*  recursively.
* 
* 
* @param {string} ID of the node to return.
*/
findById(id:string): object;

/**
* 
*  Find nodes within this tree using a string path or by attribute value(s). 
*  
*  This method can be called with 1 or 2 arguments. 
* 
*  If a single String argument is supplied, the value of the argument is treated as the path to the node. If a 
*  single argument of type Object is provided, it is treated as a set of field name/value 
*  pairs to search for (see List.find).
*  
*  If 2 arguments are supplied, this method will treat the first argument as a fieldName, and
*  return the first node encountered where node[fieldName] matches the second 
*  argument. So for example, given this tree:
*  
*  foo
*   zoo
*    bar
*   moo
*    bar
*  
*  Assuming your Tree.pathDelim is the default / and foo
*  is the name of the root node, then
*  tree.find("foo/moo/bar") would return the bar node under the
*  moo node.
*  
*  
*  tree.find("name", "bar") would return the first bar node because
*  it is the first one in the list whose name (default value of
*  Tree.nameProperty) property matches the value
*  bar. The two argument usage is generally more interesting when your tree nodes
*  have some custom unique property that you wish to search on. For example if your tree nodes
*  had a unique field called "UID", their serialized form would look something like this:
*  
*  { name: "foo", children: [...], UID:"someUniqueId"}
*  
*  You could then call tree.find("UID", "someUniqueId") to find that node. Note
*  that the value doesn't have to be a string - it can be any valid value, but since this
*  data generally comes from the server, the typical types are string, number, and boolean.
*  Also note that a find() on the Tree.idField will be constant time, and that find() will 
*  not work on the idField if idField is set to a property that is not unique or not present 
*  on all nodes in the Tree.
*  
*  The usage where you pass a single object is interesting when your tree nodes have a number
*  of custom properties that you want to search for in combination. Say your tree nodes had 
*  properties for "color" and "shape"; tree.find({color: "green", shape: "circle"})
*  would return the first node in the tree where both properties matched.
*  
*  When searching by path, trailing path delimiters are ignored. So for example
*  tree.find("foo/zoo/bar") is equivalent to
*  tree.find("foo/zoo/bar/")
* 
* 
* @param {string} Either the path to the node to be found, or the name of                   a field which should match the value passed as a second                    parameter
* @param {any=} If specified, this is the desired value for the                  appropriate field
*/
find(fieldNameOrPath:string, value?:any): object;

/**
*  Like List.findIndex, but operates only on the list of currently opened nodes. To search all loaded nodes
*  open or closed, use Tree.findNodeIndex.
* 
* 
* @param {string | object | AdvancedCriteria} property to match, or if an Object is passed, set of                    properties and values to match
* @param {any=} value to compare against (if propertyName is a string)
*/
findIndex(propertyName:string | object | AdvancedCriteria, value?:any): Integer;

/**
*  Like Tree.findIndex, but searches all tree nodes regardless of their open/closed state.
* 
* 
* @param {string | object | AdvancedCriteria} property to match, or if an Object is passed, set of                    properties and values to match
* @param {any=} value to compare against (if propertyName is a string)
*/
findNodeIndex(propertyName:string | object | AdvancedCriteria, value?:any): Integer;

/**
*  Like Tree.findIndex, but inspects a range from startIndex to endIndex. Note
*  that as in Tree.findIndex, only open nodes are included. To include both open and closed nodes, use
*  Tree.findNextNodeIndex.
*  
*  
*  For convenience, findNextIndex() may also be called with a function (called the predicate
*  function) for the propertyName parameter. In this usage pattern, the predicate
*  function is invoked for each value of the list until the predicate returns a true value.
*  The predicate function is passed three parameters: the current value, the current index, and
*  the list. The value of this when the predicate function is called is the
*  value parameter. For example:
*  var currentUserRecord = recordList.findNextIndex(0, function (record, i, recordList) {
*   if (record.username == currentUsername && !record.accountDisabled) {
*     return true;
*   }
* });
*  
* 
* 
* @param {Integer} first index to consider.
* @param {string | Function | object | AdvancedCriteria} property to match; or, if a function is passed, the predicate function to call; or, if an object is passed, set of properties and values to match.
* @param {any=} value to compare against (if propertyName is a string) or the value of this when the predicate function is invoked (if propertyName is a function)
* @param {Integer=} last index to consider (inclusive).
*/
findNextIndex(startIndex:Integer, propertyName:string | Function | object | AdvancedCriteria, value?:any, endIndex?:Integer): Integer;

/**
*  Like Tree.findNextIndex, but includes both open and closed nodes.
* 
* @param {Integer} first index to consider.
* @param {string | Function | object | AdvancedCriteria} property to match; or, if a function is passed, the predicate function to call; or, if an object is passed, set of properties and values to match.
* @param {any=} value to compare against (if propertyName is a string) or the value of this when the predicate function is invoked (if propertyName is a function)
* @param {Integer=} last index to consider (inclusive).
*/
findNextNodeIndex(startIndex:Integer, propertyName:string | Function | object | AdvancedCriteria, value?:any, endIndex?:Integer): Integer;

/**
* 
*  Returns all children of a node. If the node is a leaf, this method returns null.
*  
*  For databound trees the return value could be a ResultSet rather than a simple
*  array - so it's important to access the return value using the List 
*  interface instead of as a native Javascript Array.
*  
*  The case that a ResultSet may be returned can only happen if the tree is a
*  ResultTree and the ResultTree.fetchMode is set to "paged".
*  
*  
* 
* 
* @param {TreeNode} The node whose children you want to fetch.
*/
getChildren(node:TreeNode): Array<Partial<TreeNode>>;

/**
*  Returns a ResultSet that provides access to any partially-loaded children of a node. If the
*  node is a leaf, this method returns null.
* 
* @param {TreeNode} The node whose children you want to fetch.
*/
getChildrenResultSet(node:TreeNode): ResultSet;

/**
* 
*  Returns all the first-level folders of a node.
*  
*  For load on demand trees (those that only have a partial representation client-side), this
*  method will return only nodes that have already been loaded from the server.
* 
* 
* @param {TreeNode} node in question
*/
getFolders(node:TreeNode): List;

/**
* 
*  Return all the first-level leaves of a node.
*  
*  For load on demand trees (those that only have a partial representation client-side), this
*  method will return only nodes that have already been loaded from the server.
* 
* 
* @param {TreeNode} node in question
*/
getLeaves(node:TreeNode): List;

/**
* 
*  Returns true if this node has any children.
* 
* 
* @param {TreeNode} node in question
*/
hasChildren(node:TreeNode): boolean;

/**
* 
*  Return true if this this node has any children that are folders.
* 
* 
* @param {TreeNode} node in question
*/
hasFolders(node:TreeNode): boolean;

/**
* 
*  Return whether this node has any children that are leaves.
* 
* 
* @param {TreeNode} node in question
*/
hasLeaves(node:TreeNode): boolean;

/**
*  	Is one node a descendant of the other?
* 
* 
* @param {TreeNode} child node
* @param {TreeNode} parent node
*/
isDescendantOf(child:TreeNode, parent:TreeNode): boolean;

/**
* 
*  Returns the list of all descendants of a node. Note: this method can be very slow,
*  especially on large trees because it assembles a list of all descendants recursively.
*  Generally, Tree.find in combination with Tree.getChildren will
*  be much faster.
*  
*  For load on demand trees (those that only have a partial representation client-side), this
*  method will return only nodes that have already been loaded from the server.
* 
* 
* @param {TreeNode=} node in question (the root node is assumed if none is specified)
*/
getDescendants(node?:TreeNode): List;

/**
* 
*  Returns a list of link{type:NodeLocator)s identifying all descendants of a node (identified
*  by the parameter NodeLocator). This method
*  is the equivalent of Tree.getDescendants, but for 
*  Tree.isMultiLinkTree. The list of descendant nodes returned from
*  both methods is identical - a node's descendants are the same regardless of where or how 
*  many times that node appears in the tree - but the NodeLocators returned by this
*  method provide additional context that allows you to determine particular occurences of
*  descendant nodes. This is necessary for some use cases - for example, when trying to 
*  determine if a particular node occurence is open, or selected.
* 
* 
* @param {TreeNode=} node in question (the root node is assumed if none is specified)
*/
getDescendantNodeLocators(node?:TreeNode): List;

/**
* 
*  Returns the list of all descendants of a node that are folders. This works just like
*  Tree.getDescendants, except leaf nodes are not part of the returned list.
*  Like Tree.getDescendants, this method can be very slow for large trees.
*  Generally, Tree.find in combination with Tree.getFolders 
*  will be much faster.
*  
*  For load on demand trees (those that only have a partial representation client-side), this
*  method will return only nodes that have already been loaded from the server.
* 
* 
* @param {TreeNode=} node in question (the root node is assumed if none is specified)
*/
getDescendantFolders(node?:TreeNode): List;

/**
* 
*  Returns the list of all descendants of a node that are leaves. This works just like
*  Tree.getDescendants, except folders are not part of the returned list.
*  Folders are still recursed into, just not returned. Like Tree.getDescendants,
*  this method can be very slow for large trees. Generally, Tree.find in
*  combination with Tree.getLeaves will be much faster.
*  
*  For load on demand trees (those that only have a partial representation client-side), this
*  method will return only nodes that have already been loaded from the server.
* 
* 
* @param {TreeNode=} node in question (the root node is assumed if none specified)
*/
getDescendantLeaves(node?:TreeNode): List;

/**
* 
*  Called when the structure of this tree is changed in any way. Intended to be observed.
*  
*  Note that on a big change (many items being added or deleted) this may be called multiple times
* 
* 
*/
dataChanged(): void;

/**
* 
*  For Tree.isMultiLinkTrees only, called when links are added to or removed
*  form the tree. Intended to be observed.
*  
*  Note that on a big change (many items being added or deleted) this may be called multiple times
* 
* 
*/
linkDataChanged(): void;

/**
* 
*  Add a single node under the specified parent. See ResultTree
*  when working with a ResultTree for limitations.
* 
* 
* @param {TreeNode} node to add
* @param {string | TreeNode} Parent of the node being added. You can pass                     in either the TreeNode itself, or a path to                     the node (as a String), in which case a                     Tree.find is performed to find                     the node.
* @param {number=} Position of the new node in the children list. If not                 specified, the node will be added at the end of the list.
* @param {Record=} Optional record containing attributes associated with the                 link between the nodes, rather than either of the nodes                  themselves. Only applicable to                  Tree.multiLinkTree
*/
add(node:TreeNode, parent:string | TreeNode, position?:number, linkRecord?:Record): TreeNode;

/**
* 
*  Add a list of nodes to some parent. See ResultTree
*  when working with a ResultTree for limitations.
* 
* 
* @param {Array<Partial<TreeNode>>} The list of nodes to add
* @param {string | TreeNode} Parent of the nodes being added. You can pass                     in either the TreeNode itself, or a path to                     the node (as a String), in which case a                     Tree.find is performed to find                     the node.
* @param {number=} Position of the new nodes in the children list. If not                 specified, the nodes will be added at the end of the list.
*/
addList(nodeList:Array<Partial<TreeNode>>, parent:string | TreeNode, position?:number): List;

/**
* 
*  Moves the specified node to a new parent.
* 
* 
* @param {TreeNode} node to move
* @param {TreeNode} new parent to move the node to
* @param {Integer=} Position of the new node in the children list. If not                 specified, the node will be added at the end of the list.
*/
move(node:TreeNode, newParent:TreeNode, position?:Integer): void;

/**
* 
*  Removes a node, along with all its children. See ResultTree
*  when working with a ResultTree for limitations. Note, if this is a 
*  Tree.isMultiLinkTree, you must pass in a NodeLocator rather than
*  a node or id.
* 
* 
* @param {TreeNode | string | Integer | NodeLocator} node to remove, or the node's ID,                               or a NodeLocator
*/
remove(node:TreeNode | string | Integer | NodeLocator): boolean;

/**
* 
*  Remove a list of nodes (not necessarily from the same parent), and all children of those
*  nodes. See ResultTree when working with a
*  ResultTree for limitations.
* 
* 
* @param {Array<Partial<TreeNode>>} list of nodes to remove
*/
removeList(nodeList:Array<Partial<TreeNode>>): boolean;

/**
*  What is the loadState of a given folder?
* 
* 
* @param {TreeNode} folder in question
*/
getLoadState(node:TreeNode): LoadState;

/**
*  For a databound tree, has this folder either already loaded its children or is it in the
*  process of loading them.
* 
* 
* @param {TreeNode} folder in question
*/
isLoaded(node:TreeNode): boolean;

/**
*  For a databound tree, do the children of this folder form a ResultSet with a full cache.
*  
*  Note that this method only applies to ResultTree.fetchMode "paged".
* 
* @param {TreeNode} folder in question
*/
allChildrenLoaded(node:TreeNode): boolean;

/**
*  Load the children of a given node.
*  
*  For a databound tree this will trigger a fetch against the Tree's DataSource.
* 
* 
* 
* @param {TreeNode} node in question
* @param {DSCallback=} Optional callback (stringMethod) to fire when loading            completes. Has a single param node - the node whose            children have been loaded, and is fired in the scope of the Tree.
*/
loadChildren(node:TreeNode, callback?:DSCallback): void;

/**
*  Unload the children of a folder, returning the folder to the "unloaded" state.
* 
* 
* @param {TreeNode} folder in question
* @deprecated * It's recommended that you instead use Tree.reloadChildren to reload the children of a folder, or Tree.removeChildren if you need to clear the cached children
*  of a folder to add specific local data.

*/
unloadChildren(node:TreeNode): void;

/**
*  Reload the children of a folder.
* 
* 
* @param {TreeNode} node in question
*/
reloadChildren(node:TreeNode): void;

/**
*  Removes all children of the node and sets it to a loaded state. For non-ResultTrees,
*  or non-ResultTree.fetchMode ResultTrees, Tree.add or
*  Tree.addList can then be used to provide new children. For
*  ResultTree.fetchMode ResultTrees, DataSource.updateCaches
*  must be used to insert nodes into the cache as local data, since such
*  ResultTrees are considered read-only, and Tree.add and Tree.addList are
*  not perrmitted.
* 
* 
* @param {TreeNode} folder in question
*/
removeChildren(node:TreeNode): void;

/**
*  Replaces the existing children of a parent node, leaving the node in the loaded state.
*  Only a flat list of children nodes is supported, as in Tree.addList.
*  
* 
* @param {TreeNode} parent of children
* @param {Array<Partial<TreeNode>>} children to be set
*/
setChildren(parent:TreeNode, newChildren:Array<Partial<TreeNode>>): void;

/**
* 
*  Whether a particular node is open or closed (works for leaves and folders). Note, for
*  Tree.isMultiLinkTrees, passing a NodeLocator is the only
*  unambiguous way to specify the node.
* 
* 
* @param {TreeNode | string | Integer | NodeLocator} the node in question, or the                                the node's ID, or a NodeLocator                                object
*/
isOpen(node:TreeNode | string | Integer | NodeLocator): boolean;

/**
* 
*  Open a particular node. Note, for Tree.isMultiLinkTrees, passing a 
*  NodeLocator is the only unambiguous way to specify the node.
* 
* 
* @param {TreeNode | string | Integer | NodeLocator} the node to open, or its                                 ID, or a NodeLocator object
* @param {Callback=} Optional callback (stringMethod) to fire when loading            completes. Has a single param node - the node whose            children have been loaded, and is fired in the scope of the Tree.
*/
openFolder(node:TreeNode | string | Integer | NodeLocator, callback?:Callback): void;

/**
* 
*  Open a set of folders, specified by path or as pointers to nodes.
* 
* 
* @param {Array<Partial<TreeNode>>} List of nodes or node paths or NodeLocators
*/
openFolders(nodeList:Array<Partial<TreeNode>>): void;

/**
* 
*  Closes a folder. Note, for Tree.isMultiLinkTrees, passing a 
*  NodeLocator is the only unambiguous way to specify the node.
* 
* 
* @param {TreeNode | string | Integer | NodeLocator} the node to open, or its                                 ID, or a NodeLocator object
*/
closeFolder(node:TreeNode | string | Integer | NodeLocator): void;

/**
* 
*  Close a set of folders, specified by path or as pointers to nodes.
* 
* 
* @param {Array<Partial<TreeNode>>} List of nodes or node paths or NodeLocators
*/
closeFolders(nodeList:Array<Partial<TreeNode>>): void;

/**
* 
*  Open all nodes under a particular node.
* 
* 
* @param {TreeNode | string | Integer | NodeLocator=} node from which to open folders,                                or the node's ID, or a                                NodeLocator object (if not                                specified, the root node is used)
*/
openAll(node?:TreeNode | string | Integer | NodeLocator): void;

/**
*  Close all nodes under a particular node
* 
* 
* @param {TreeNode | NodeLocator=} node from which to close folders (if not specified,                      the root node is used). If this is a                       Tree.isMultiLinkTree, you must                       provide a NodeLocator for any node other                      than the root node
*/
closeAll(node?:TreeNode | NodeLocator): void;

/**
*  Setter for Tree.showRoot.
* 
* @param {boolean} new showRoot value
*/
setShowRoot(showRoot:boolean): void;

/**
*  Setter for Tree.separateFolders.
* 
* @param {boolean} new separateFolders value
*/
setSeparateFolders(separateFolders:boolean): void;

/**
*  Setter for Tree.sortFoldersBeforeLeaves.
* 
* @param {boolean} new sortFoldersBeforeLeaves value
*/
setSortFoldersBeforeLeaves(sortFoldersBeforeLeaves:boolean): void;

/**
*  Return a flattened list of all nodes that are visible under some parent based on whether the node
*  itself or any folders underneath it are open. Returned list will include the passed node.
*  
*  If the passed in node is a leaf, this method returns null.
* 
* 
* @param {TreeNode | string | Integer | NodeLocator} the node in question, or the                                the node's ID, or a NodeLocator                                object. Defaults to the root node
*/
getOpenList(node:TreeNode | string | Integer | NodeLocator): Array<Partial<TreeNode>>;

/**
*  Get all the nodes that exist in the tree under a particular node, as a flat list, in
*  depth-first traversal order.
* 
* 
* @param {TreeNode=} optional node to start from. Default is root.
*/
getAllNodes(node?:TreeNode): Array<Partial<TreeNode>>;

/**
* 
*  Returns the number of items in the current open list.
* 
* 
*/
getLength(): number;

/**
*  Return the position in the list of the first instance of the specified object.
*  
*  If pos is specified, starts looking after that position.
*  
*  Returns -1 if not found.
* 
* 
* @param {any} object to look for
* @param {number=} earliest index to consider
* @param {number=} last index to consider
*/
indexOf(obj:any, pos?:number, endPos?:number): number;

/**
*  Return the position in the list of the last instance of the specified object.
*  
*  If pos is specified, starts looking before that position.
*  
*  Returns -1 if not found.
* 
* 
* @param {any} object to look for
* @param {number=} last index to consider
* @param {number=} earliest index to consider
*/
lastIndexOf(obj:any, pos?:number, endPos?:number): number;

/**
*  Filters this tree by the provided criteria, returning a new Tree containing just the nodes
*  that match the criteria.
*  
*  If filterMode is "keepParents", parents are retained if
*  any of their children match the criteria even if those parents do not match the criteria.
*  
*  Note that the DataSource argument is required if one is not
*  DataSource on the tree.
*  
*  If you want a TreeGrid with local tree data that supports filtering, please consider
*  using a TreeGrid bound to a DataSource.clientOnly
*  rather than writing your own code to filter the tree's data with this method.
*  
* 
* @param {Criteria | AdvancedCriteria} criteria to filter by
* @param {TreeFilterMode=} mode to use for filtering, defaults to "strict"
* @param {DataSource=} dataSource to use for filtering, if this Tree does not                 already have one
* @param {DSRequest=} Request properties block. This allows developers to   specify properties that would impact the filter such as DSRequest.textMatchStyle
*/
getFilteredTree(criteria:Criteria | AdvancedCriteria, filterMode?:TreeFilterMode, dataSource?:DataSource, requestProperties?:DSRequest): Tree;



}
/**
* Creates a new Tree
* 
* @param typeCheckedProperties {Partial<Tree>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Tree} The new instance
*/
class Tree{
static create(typeCheckedProperties? : Partial<Tree>, uncheckedProperties?: Object): Tree;
/**
* A declared value of the enum type 
*  LoadState.
* @type {Constant}
* @default "unloaded"
*/
static UNLOADED?: Constant;

/**
* A declared value of the enum type 
*  LoadState.
* @type {Constant}
* @default "loading"
*/
static LOADING?: Constant;

/**
* A declared value of the enum type 
*  LoadState.
* @type {Constant}
* @default "foldersLoaded"
*/
static FOLDERS_LOADED?: Constant;

/**
* A declared value of the enum type 
*  LoadState.
* @type {Constant}
* @default "loaded"
*/
static LOADED?: Constant;

/**
* A declared value of the enum type 
*  LoadState.
* @type {Constant}
* @default "loadedPartialChildren"
*/
static LOADED_PARTIAL_CHILDREN?: Constant;

/**
* Name of property that identifies a NodeLocator object for any 
*  Tree.isMultiLinkTrees.
* @type {string}
* @default "isNodeLocator"
*/
static isNodeLocatorProperty?: string;





/**
*  heuristically find a property that appears to contain child objects.
*  Searches through a node and find a property that is either Array or Object valued.
* 
* 
* @param {TreeNode} the node to check
* @param {ChildrenPropertyMode} determines how to chose the property that appears to contain child objects
*/
static findChildrenProperty(node:TreeNode, mode:ChildrenPropertyMode): string;

/**
*  given a hierarchy of objects with children under mixed names, heuristically discover the
*  property that holds children and copy it to a single, uniform childrenProperty. Label each
*  discovered child with a configurable "typeProperty" set to the value of the property
*  that held the children.
* 
* 
* @param {Array<Partial<TreeNode>>} list of nodes to link into the tree.
* @param {DiscoverTreeSettings} configures how the tree will be explored
* @param {string} */
static discoverTree(nodes:Array<Partial<TreeNode>>, settings:DiscoverTreeSettings, parentChildrenField:string): void;




}

/**
* The SmartClient system supports hierarchical data (also referred to as tree data
*  due to its "branching" organization) with:
*  
*   
* - the Tree class, which manipulates hierarchical data sets
*   
* - the TreeGrid widget class, which extends the ListGrid class to visually
*     present tree data in an expandable/collapsible format.
*  
*  For information on DataBinding Trees, see treeDataBinding.
*  
*  A TreeGrid works just like a ListGrid, except one column (specified by
*  TreeGridField.treeField) shows a hierarchical Tree. A TreeGrid is not limited
*  to displaying just the Tree column - you can define additional columns (via
*  TreeGrid.fields) which will render just like the columns of a ListGrid, and
*  support all of the functionality of ListGrid columns, such as
*  ListGridField.formatCellValue.
*  
*  Except where explicitly overridden, ListGrid methods, callbacks, and properties
*  apply to TreeGrids as well. The ListGrid defines some methods as taking/returning
*  ListGridField and ListGridRecord. When using those methods in a TreeGrid,
*  those types will be TreeGridField and TreeNode, respectively.
*/
interface TreeGrid extends Omit<ListGrid, 'getData' | 'transferSelectedData' | 'data' | 'dataProperties' | 'setData' | 'recordDoubleClick' | 'dataArrived'>  {
/**
* The DataSource that this component should bind to for default fields and for performing
*  DSRequest.
*  
*  Can be specified as either a DataSource instance or the String ID of a DataSource.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* A Tree object containing of nested TreeNodes to 
*  display as rows in this TreeGrid. 
*  The data property will typically not be explicitly specified for 
*  databound TreeGrids, where the data is returned from the server via databound component
*  methods such as fetchData()
* @type {Tree}
* @default null
*/
data?: Tree;

/**
* You can specify the initial set of data for a databound TreeGrid using this property.
*  The value of this attribute should be a list of parentId-linked
*  TreeNodes in a format equivalent to that documented on Tree.data or, for
*  TreeGrids with TreeGrid.dataFetchMode set to
*  FetchMode, on ResultTree.data.
*  
*  If you create a standalone Tree or ResultTree as the
*  TreeGrid's TreeGrid.data then you may equivalently specify this initial set
*  of tree nodes in that tree's Tree.data property.
* @type {Array<Partial<TreeNode>>}
* @default null
*/
initialData?: Array<Partial<TreeNode>>;

/**
* For databound treeGrid instances, should the entire tree of data be loaded on initial 
*  fetch, or should folders load their children as they are opened.
*  
*  If unset, calling TreeGrid.fetchData will default it to true, otherwise, if a ResultTree
*  is passed to TreeGrid.setData, the ResultTree.loadDataOnDemand setting is
*  respected. Must be enabled on the underlying ResultTree when using
*  TreeGrid.dataFetchMode: "paged".
*  
*  Note that when using loadDataOnDemand, every node returned by the server is
*  assumed be a folder which may load further children. See
*  ResultTree.defaultIsFolder for how to control this behavior.
* @type {boolean}
* @default null
*/
loadDataOnDemand?: boolean;

/**
* If set, tree-based filtering is performed such that parent nodes are kept as long as
*  they have children that match the filter criteria, even if the parents themselves do not
*  match the filter criteria. If not set, filtering will exclude parent nodes not matching
*  the criteria and all nodes below them in the tree.
*  
*  ResultTrees will default to FetchMode whenever 
*  keepParentsOnFilter is true, unless fetchMode was explicitly set to
*  "paged" (see below). This allows the filtering logic to
*  fetch a complete tree of nodes from the DataSource (or if 
*  TreeGrid.loadDataOnDemand:true, a complete set of nodes under a given parent) and then
*  filter the resulting data locally on the client. 
*  
*  This means that the server does not need to implement special tree filtering logic
*  to support looking up nodes that match the specified criteria as well as ancestor nodes
*  that may not.
*  
*  If some criteria must be sent to the server in order to produce a valid tree of
*  data, but keepParentsInFilter is also required, the 
*  ResultTree.serverFilterFields attribute may be used to specify a list
*  of field names that will be sent to the server whenever they are present in the 
*  criteria. Note that for the subset of criteria applied to these fields,
*  keepParentsInFilter behavior will not occur without custom 
*  logic in the DataSource fetch operation.
*  
*  If FetchMode is explicitly set to "paged", it is not possible 
*  to implement keepParentsOnFilter by local filtering. Support for
*  keepParentsOnFilter for a paged ResultTree therefore also requires
*  custom logic in the DataSource fetch operation. To support this a developer
*  must ensure that their fetch operation returns the appropriate set of nodes -
*  all nodes that match the specified criteria plus their ancestor nodes even
*  if they do not match the specified criteria.
* @type {boolean}
* @default null
*/
keepParentsOnFilter?: boolean;

/**
* Mode of fetching records from server.
*  
*  fetchMode:"local" implies that local filtering will always be performed. See
*  TreeGrid.keepParentsOnFilter for additional filtering details.
*  
*  fetchMode:"basic" or "paged" implies that if search criteria change, the entire
*  tree will be discarded and re-fetched from the server. When retrieving the replacement
*  tree data, the default behavior will be to preserve the TreeGrid.getOpenState
*  for any nodes that the server returns which were previously opened by the user. Note
*  that this implies that if TreeGrid.loadDataOnDemand is enabled and the server returns only
*  root-level nodes, open state will be preserved only for root-level nodes, and children
*  of open root-level nodes will be immediately fetched from the server if
*  they are not included in the server's initial response.
*  
*  fetchMode:"paged" enables paging for nodes that have very large numbers of children.
*  Whenever the children of a folder are loaded, the resultTree will set
*  DSRequest.startRow and DSRequest.endRow when requesting children
*  from the DataSource, and will manage loading of further children on demand, similar to
*  how a ResultSet manages paging for lists. For a deeper discussion see the
*  Paging large sets of children section of the treeDataBinding
*  overview.
* @type {FetchMode}
* @default "basic"
*/
dataFetchMode?: FetchMode;

/**
* For FetchMode ResultTrees, this property lists field names 
*  that will be sent to the server if they are present in the criteria.
*  
*  This property may be used to ensure a dataSource receives the necessary criteria
*  to populate a ResultTree's data, and also support TreeGrid.keepParentsOnFilter. 
*  
*  Note that for some AdvancedCriteria it will not be possible to extract the
*  subcriteria that apply to certain fields. See DataSource.splitCriteria
*  for details on how serverFilterFields-applicable subcriteria are extracted from
*  the specified criteria for the tree.
* @type {Array<Partial<string>>}
* @default null
*/
serverFilterFields?: Array<Partial<string>>;

/**
* A TreeGrid is a DataBoundComponent.dataArity:multiple component.
* @type {string}
* @default "multiple"
*/
dataArity?: string;

/**
* With TreeGrid.loadDataOnDemand:true, TreeGrids fetch data by selecting the child nodes of
*  each parent, which should be exact match, so we default to
*  autoFetchTextMatchStyle:"exact" when autoFetchData is true.
*  
*  See ListGrid.autoFetchTextMatchStyle for details.
* @type {TextMatchStyle}
* @default "exact"
*/
autoFetchTextMatchStyle?: TextMatchStyle;

/**
* Should children be selected when parent is selected? And should parent be
*  selected when any child is selected?
* @type {boolean}
* @default false
*/
cascadeSelection?: boolean;

/**
* Should partially selected parents be shown with special icon?
* @type {boolean}
* @default false
*/
showPartialSelection?: boolean;

/**
* If specified, the selection object for this list will use this property to mark records
*  as selected. In other words, if this attribute were set to "isSelected"
*  any records in the listGrid data where "isSelected" is true
*  will show up as selected in the grid. Similarly if records are selected within the grid
*  after the grid has been created, this property will be set to true on the selected
*  records.
* @type {string}
* @default null
*/
selectionProperty?: string;

/**
* This property is not supported on TreeGrid, and only applies to the ListGrid
*  superclass.
* @type {boolean}
* @default null
*/
canSelectAll?: boolean;

/**
* For databound trees, use this attribute to supply a DataSourceField.rootValue for this
*  component's generated data object.
*  
*  This property allows you to have a particular component navigate a tree starting from any
*  given node as the root.
*  
*  This setting is invalid if TreeGrid.keepParentsOnFilter is set and 
*  TreeGrid.dataFetchMode is set to anything other than
*  "paged" - a root-value cannot be 
*  used in this case because there is no efficient way to load a subtree to include all
*  parents. If fetchMode is specifically set to "paged", the 
*  DSRequest.keepParentsOnFilter flag is set on the request so 
*  the server knows to use special filtering rules, but these rules are not built-in - the 
*  developer is responsible for this logic.
* @type {any}
* @default null
*/
treeRootValue?: any;

/**
* An array of field objects, specifying the order, layout, dynamic calculation, and
*  sorting behavior of each field in the treeGrid object. In TreeGrids, the fields
*  array specifies columns. Each field in the fields array is TreeGridField object.
*  
*  If TreeGrid.dataSource is also set, this value acts as a set of overrides as
*  explained in DataBoundComponent.fields.
* @type {Array<Partial<TreeGridField>>}
* @default null
*/
fields?: Array<Partial<TreeGridField>>;

/**
* Visible title for the tree column (field).
* @type {string}
* @default "Name"
*/
treeFieldTitle?: string;

/**
* If this grid was passed an explicit set of fields, but no field was specified as the
*  "tree-field" (showing indentations for tree hierarchy
*  and tree icons), should we assign one of the other fields to be the tree-field?
*  
*  When true, if we're showing a field for the Tree.titleProperty of the tree, 
*  this will be displayed as a Tree Field by default. If not, the first entry in the 
*  specified fields array will be used.
*  
*  This may be set to false to display a tree or partial tree as a flattened list within
*  a TreeGrid.
* @type {boolean}
* @default true
*/
autoAssignTreeField?: boolean;

/**
* Specifies whether the root node should be displayed in the treeGrid.
*  
*  This property is only available for "children" modelType trees, hence is not allowed for
*  trees that load data from the server dynamically via TreeGrid.fetchData. 
*  
*  To get the equivalent of a visible "root" node in a tree that loads data dynamically,
*  add a singular, top-level parent to the data. However, note that this top-level parent
*  will technically be the only child of root, and the implicit root object will be
*  returned by Tree.getRoot.
* @type {boolean}
* @default false
*/
showRoot?: boolean;

/**
* If specified, this attribute will override Tree.separateFolders on the
*  data for this treeGrid.
*  
*  Specifies whether folders and leaves should be segregated in the treeGrid display.
*  Use Tree.sortFoldersBeforeLeaves to customize whether folders appear before 
*  or after their sibling leaves.
*  
*  If unset, at the treeGrid level, the property can be set directly on
*  TreeGrid.data or for dataBound TreeGrids on the
*  TreeGrid.dataProperties.
* @type {boolean}
* @default null
*/
separateFolders?: boolean;

/**
* For a TreeGrid that uses a DataSource, these properties will be passed to
*  the automatically-created ResultTree. This can be used for various customizations such as
*  modifying the automatically-chosen Tree.parentIdField.
* @type {Tree}
* @default null
*/
dataProperties?: Tree;

/**
* If specified, this attribute will override Tree.sortFoldersBeforeLeaves on
*  the data for this treeGrid.
*  
*  Specifies whether when Tree.separateFolders is true, folders should be displayed
*  before or after their sibling leaves in a sorted tree. If set to true, with
*  sortDirection set to Array.ASCENDING, folders are displayed before their sibling leaves
*  and with sort direction set to Array.DESCENDING they are displayed after. To invert
*  this behavior, set this property to false.
* @type {boolean}
* @default null
*/
sortFoldersBeforeLeaves?: boolean;

/**
* Specifies the type of nodes displayed in the treeGrid.
* @type {DisplayNodeType}
* @default null
*/
displayNodeType?: DisplayNodeType;

/**
* For dataBound treeGrids this specifies the ResultTree.autoPreserveOpenState,
*  governing whether the open state of the tree should be preserved when new data
*  arrives due to cache invalidation.
* @type {PreserveOpenState}
* @default null
*/
autoPreserveOpenState?: PreserveOpenState;

/**
* Indicates whether records can be dragged from this listGrid and dropped elsewhere.
*  
*  NOTE: If canDragRecordsOut is initially enabled or might be
*  dynamically enabled after the grid is created, it may be desirable to disable
*  Canvas.useTouchScrolling so that touch-dragging a record starts
*  a drag operation rather than a scroll, but see the discussion of 
*  ListGrid.showDragHandles. If 
*  Canvas.disableTouchScrollingForDrag is set to true, then touch
*  scrolling will be disabled automatically. However, for
*  accessibility reasons, it is recommended to leave touch
*  scrolling enabled and provide an alternative set of controls that can be used to perform
*  drag and drop of records out of the grid.
* @type {boolean}
* @default false
*/
canDragRecordsOut?: boolean;

/**
* Indicates whether records can be dropped into this listGrid.
* @type {boolean}
* @default false
*/
canAcceptDroppedRecords?: boolean;

/**
* Indicates whether records can be reordered by dragging within this ListGrid.
*  
*  NOTE: If canReorderRecords is initially enabled or might be
*  ListGrid.setCanReorderRecords after the grid is created,
*  it may be desirable to disable Canvas.useTouchScrolling
*  so that touch-dragging a record starts a reorder operation rather than a scroll,
*  but see the discussion of ListGrid.showDragHandles. If 
*  Canvas.disableTouchScrollingForDrag is set to true, then touch
*  scrolling will be disabled automatically. However, for accessibility
*  reasons, it is recommended to leave touch scrolling enabled and provide an alternative
*  set of controls that can be used to perform drag-reordering of records.
* @type {boolean}
* @default false
*/
canReorderRecords?: boolean;

/**
* Whether drops are allowed on leaf nodes.
*  
*  Dropping is ordinarily not allowed on leaf nodes unless TreeGrid.canReorderRecords is
*  set. 
*  
*  The default action for a drop on a leaf node is to place the node in that leaf's parent
*  folder. This can be customized by overriding TreeGrid.folderDrop.
*  
*  Note that enabling canDropOnLeaves is usually only appropriate where you
*  intend to add a custom TreeGrid.folderDrop implementation that does not add a
*  child node under the leaf. If you want to add a child nodes to a leaf, instead of
*  enabling canDropOnLeaves, use empty folders instead - see Tree.isFolder for how
*  to control whether a node is considered a folder.
* @type {boolean}
* @default false
*/
canDropOnLeaves?: boolean;

/**
* If set this property allows the user to reparent nodes by dragging them from their
*  current folder to a new folder.
*  Backcompat: For backwards compatibility with versions prior to SmartClient 1.5,
*  if this property is unset, but this.canAcceptDroppedRecords is true, we
*  allow nodes to be dragged to different folders.
* @type {boolean}
* @default null
*/
canReparentNodes?: boolean;

/**
* Specifies what to do with data dragged from this TreeGrid (into another component, or
*  another node in this TreeGrid. The default action is to move the data. A setting of
*  "none" is not recommended for trees because Trees maintain the node open state on the nodes
*  themselves, and hence having multiple Tree objects share a reference to a node can have
*  unintended consequences (such as opening a folder in one tree also triggering an open in
*  another tree that shares the same node).
*  
*  See TreeGrid.folderDrop for a full explanation of default behaviors on drop, and how to
*  customize them.
* @type {DragDataAction}
* @default "move"
*/
dragDataAction?: DragDataAction;

/**
* Message displayed when user attempts to drag a node into a parent that already contains
*  a child of the same name/ID.
* @type {string}
* @default "This item already contains a child item with that name."
*/
parentAlreadyContainsChildMessage?: string;

/**
* Message displayed when user attempts to drop a dragged node onto itself.
* @type {string}
* @default "You can't drag an item into itself."
*/
cantDragIntoSelfMessage?: string;

/**
* Message displayed when user attempts to drop a node into a child of itself.
* @type {string}
* @default "You can't drag an item into one of it's children."
*/
cantDragIntoChildMessage?: string;

/**
* For Tree.isMultiLinkTree only, the message displayed when the 
*  user attempts to drag two or more occurrences of the same node into a parent.
* @type {string}
* @default "You can't drag two occurrences of the same node into a parent."
*/
cantDragMultipleNodeOccurencesMessage?: string;

/**
* The amount of indentation (in pixels) to add to a node's icon/title for each level
*  down in this tree's hierarchy.
*  
*  This value is ignored when TreeGrid.showConnectors is
*  true because fixed-size images are used to render the connectors.
* @type {number}
* @default "20"
*/
indentSize?: number;

/**
* The amount of gap (in pixels) between the extraIcon (see TreeGrid.getExtraIcon)
*  or checkbox icon and the TreeGrid.nodeIcon/
*  TreeGrid.folderIcon or node text.
* @type {Integer}
* @default "2"
*/
extraIconGap?: Integer;

/**
* The standard size (same height and width, in pixels) of node icons in this
*      treeGrid.
* @type {number}
* @default "16"
*/
iconSize?: number;

/**
* Default width and height in pixels of the opener icons, that is, the icons which show
*  the open or closed state of the node, typically a [+] or [-] symbol, if not overridden
*  by TreeGrid.openerIconWidth/TreeGrid.openerIconHeight.
*  
*  If TreeGrid.showConnectors is true, the opener icon includes the connector line, and
*  defaults to ListGrid.cellHeight.
*  
*  Otherwise, openerIconSize defaults to TreeGrid.iconSize.
* @type {number}
* @default null
*/
openerIconSize?: number;

/**
* Width in pixels of the opener icons, that is, the icons which show the open or closed
*  state of the node, typically a [+] or [-] symbol.
*  
*  If not specified, TreeGrid.openerIconSize is used instead.
* @type {number}
* @default null
*/
openerIconWidth?: number;

/**
* Height in pixels of the opener icons, that is, the icons which show the open or closed
*  state of the node, typically a [+] or [-] symbol.
*  
*  If not specified, TreeGrid.openerIconSize is used instead.
* @type {number}
* @default null
*/
openerIconHeight?: number;

/**
* If set, when a folder is loading its children from the server (Tree.getLoadState
*  returns "loading"), it uses a distinct icon image given by TreeGrid.loadingIcon. This is
*  typically used to show a small animating "spinner" icon to let the user know data is being
*  fetched.
* @type {boolean}
* @default true
*/
showLoadingIcons?: boolean;

/**
* If TreeGrid.showLoadingIcons is set, this icon will be used when the folder is 
*  Tree.getLoadState.
* @type {SCImgURL}
* @default "[SKIN]folder_loading.gif"
*/
loadingIcon?: SCImgURL;

/**
* The URL of the base icon for all folder nodes in this treeGrid. Note that this URL will
*  have TreeGrid.openIconSuffix, TreeGrid.closedIconSuffix or 
*  TreeGrid.dropIconSuffix appended to indicate state changes if appropriate - 
*  see documentation on TreeGrid.showOpenIcons, TreeGrid.showSelectedIcons
*  and TreeGrid.showDropIcons.
*  
*  See TreeGrid.showNodeIcons and TreeGrid.showFolderIcons for details on suppressing 
*  display of icons
* @type {SCImgURL}
* @default "[SKIN]folder.gif"
*/
folderIcon?: SCImgURL;

/**
* Default padding to show between the folder or leaf node icon and cell value in the tree cell.
*  
*  May be overridden for TreeGrid.folderIcon via TreeGrid.folderIconPadding.
*  May also be overridden for individual nodes by setting the TreeGrid.iconPaddingProperty 
*  value on individual nodes
* @type {Integer}
* @default "2"
*/
iconPadding?: Integer;

/**
* Default padding to show between the TreeGrid.showOpenIcons and the 
*  extra or folder icon in the tree cell.
* @type {Integer}
* @default null
*/
openIconPadding?: Integer;

/**
* Default padding to show between folder icon and cell value in the tree cell.
*  This property is only consulted for folder nodes. If unset, TreeGrid.iconPadding will be
*  applied to both folder and leaf nodes.
*  
*  To set the icon padding for individual nodes, use TreeGrid.iconPaddingProperty
* @type {Integer}
* @default null
*/
folderIconPadding?: Integer;

/**
* This property allows the developer to specify custom 
*  TreeGrid.iconPadding for specific nodes
* @type {string}
* @default "iconPadding"
*/
iconPaddingProperty?: string;

/**
* If TreeGrid.showDropIcons is true, this suffix will be appended to the
*  TreeGrid.folderIcon when the user drop-hovers over some folder.
* @type {string}
* @default "drop"
*/
dropIconSuffix?: string;

/**
* If TreeGrid.showOpenIcons is true, this suffix will be appended to the
*  TreeGrid.folderIcon for open folders in this grid.
* @type {string}
* @default "open"
*/
openIconSuffix?: string;

/**
* This suffix will be appended to the TreeGrid.folderIcon for closed folders.
*  If TreeGrid.showOpenIcons is set to false this suffix will also be
*  appended to open folders' icons.
* @type {string}
* @default "closed"
*/
closedIconSuffix?: string;

/**
* If TreeGrid.showSelectedIcons is true, this suffix will be appended to the
*  TreeGrid.folderIcon for selected nodes in this grid.
* @type {string}
* @default "selected"
*/
selectedIconSuffix?: string;

/**
* The filename of the default icon for all leaf nodes in this grid. To specify a 
*  custom image for an individual node, set the TreeGrid.customIconProperty directly on
*  the node.
*  
*  See TreeGrid.showNodeIcons and TreeGrid.showFolderIcons for details on suppressing 
*  display of icons
* @type {SCImgURL}
* @default "[SKIN]file.gif"
*/
nodeIcon?: SCImgURL;

/**
* If true, show a different icon for open folders than closed folders.
*  This is achieved by appending the TreeGrid.openIconSuffix onto the 
*  TreeGrid.folderIcon URL [for example "[SKIN]/folder.gif" might be 
*  replaced by "[SKIN]/folder_open.gif".
*  Note If this property is set to false the same icon is shown for
*  open folders as for closed folders, unless a custom folder icon was specified. This will be
*  determined by TreeGrid.folderIcon plus the TreeGrid.closedIconSuffix.
* @type {boolean}
* @default true
*/
showOpenIcons?: boolean;

/**
* If true, show a different icon for selected nodes than unselected nodes.
*  This is achieved by appending the TreeGrid.selectedIconSuffix onto the 
*  TreeGrid.folderIcon URL or TreeGrid.nodeIcon for selected records.
*  
*  If appropriate, this suffix will be combined with the 
*  TreeGrid.openIconSuffix or TreeGrid.closedIconSuffix (see 
*  TreeGrid.showOpenIcons. So a treeGrid with its folderIcon
*  property set to "[SKIN]/folder.gif", with both
*  showSelectedIcons and showOpenIcons set to true would show
*  an icon with the URL "[SKIN]/folder_open_selected.gif" for a
*  folder that was both selected and opened.
* @type {boolean}
* @default false
*/
showSelectedIcons?: boolean;

/**
* If true, when the user drags a droppable target over a folder in this TreeGrid, show 
*  a different folder icon.
*  This is achieved by appending the TreeGrid.dropIconSuffix onto the
*  TreeGrid.folderIcon URL (for example "[SKIN]/folder.gif" may be
*  replaced by "[SKIN]/folder_drop.gif").
* @type {boolean}
* @default true
*/
showDropIcons?: boolean;

/**
* This property allows the developer to rename the 
*  TreeNode.icon property.
* @type {string}
* @default "icon"
*/
customIconProperty?: string;

/**
* This property allows the developer to rename the 
*  TreeNode.showOpenIcon property.
* @type {string}
* @default "showOpenIcon"
*/
customIconOpenProperty?: string;

/**
* This property allows the developer to rename the 
*  TreeNode.showDropIcon property.
* @type {string}
* @default "showDropIcon"
*/
customIconDropProperty?: string;

/**
* This property allows the developer to rename the 
*  TreeNode.showSelectedIcon property.
* @type {string}
* @default "showSelectedIcon"
*/
customIconSelectedProperty?: string;

/**
* Should folder nodes showing custom icons (set via the TreeGrid.customIconProperty),
*  show open state images when the folder is opened.
*  If true, the TreeGrid.openIconSuffix will be appended to the image URL
*  (so "customFolder.gif" might be replaced with 
*  "customFolder_open.gif").
*  Note that the TreeGrid.closedIconSuffix is never appended to custom folder icons.
*  Can be overridden at the node level via the default property TreeNode.showOpenIcon
*  and that property can be renamed via TreeGrid.customIconOpenProperty.
* @type {boolean}
* @default false
*/
showCustomIconOpen?: boolean;

/**
* Should folder nodes showing custom icons (set via the TreeGrid.customIconProperty,
*  default TreeNode.icon),
*  show drop state images when the user is drop-hovering over the folder.
*  If true, the TreeGrid.dropIconSuffix will be appended to the image URL
*  (so "customFolder.gif" might be replaced with 
*  "customFolder_drop.gif").
*  Can be overridden at the node level via the default property TreeNode.showDropIcon
*  and that property can be renamed via TreeGrid.customIconDropProperty.
* @type {boolean}
* @default false
*/
showCustomIconDrop?: boolean;

/**
* Should folder nodes showing custom icons (set via the TreeGrid.customIconProperty),
*  show selected state images when the folder is selected, 
*  if TreeGrid.showSelectedIcons is true?
*  
*  If true, the TreeGrid.selectedIconSuffix will be appended to the image URL
*  (so "customFolder.gif" might be replaced with 
*  "customFolder_selected.gif").
*  Can be overridden at the node level via the default property TreeNode.showSelectedIcon
*  and that property can be renamed via TreeGrid.customIconSelectedProperty.
* @type {boolean}
* @default false
*/
showCustomIconSelected?: boolean;

/**
* Should tree nodes show a disabled checkbox 
*  ListGrid.selectionAppearance:"checkbox" 
*  is set on the treegrid, and a node can't be selected? 
*  
*  If set to false the treeGrid will use 
*  TreeGrid.leaveSelectionCheckboxGap to determine whether to leave
*  a blank space where the checkbox would normally appear.
* @type {boolean}
* @default false
*/
showDisabledSelectionCheckbox?: boolean;

/**
* If ListGrid.selectionAppearance:"checkbox" 
*  is set on the treegrid, and a node can't be selected, should a gap be left where
*  the checkbox icon would normally appear, in order to make the node's icon and title
*  line up with the content for other nodes in the same parent?
*  
*  Has no effect if TreeGrid.showDisabledSelectionCheckbox is true
* @type {boolean}
* @default true
*/
leaveSelectionCheckboxGap?: boolean;

/**
* The filename of the icon displayed use as the default drag tracker when for multiple
*  files and/or folders are being dragged.
* @type {SCImgURL}
* @default "[SKIN]folder_file.gif"
*/
manyItemsImage?: SCImgURL;

/**
* Should this treeGrid show connector lines illustrating the tree's hierarchy?
*  
*  For the set of images used to show connectors, see TreeGrid.connectorImage.
*  
*  Note: in order for connector images to be perfectly connected, all styles for
*  cells must have no top or bottom border or padding. If you see small gaps in connector
*  lines, check your CSS files. See the example below for an example of correct
*  configuration, including example CSS.
* @type {boolean}
* @default false
*/
showConnectors?: boolean;

/**
* If TreeGrid.showConnectors is true, this property determines whether we should show
*  vertical continuation lines for each level of indenting within the tree. Setting to
*  false will show only the hierarchy lines for the most indented path ("sparse"
*  connectors).
* @type {boolean}
* @default true
*/
showFullConnectors?: boolean;

/**
* Should the opener icon be displayed next to folder nodes? This is an icon
*  which visually indicates whether the folder is opened or closed (typically via
*  a [+] or [-] image, or a turn-down arrow) and may be clicked to expand or collapse
*  the folder.
*  
*  For folders with no children, this icon is not shown unless 
*  TreeGrid.alwaysShowOpener is true. Note that for trees which
*  TreeGrid.loadDataOnDemand, we may not know if a folder
*  has any descendants if it has never been opened. As such we will show the
*  opener icon next to the folder. Once the user opens the icon and a fetch occurs,
*  if the folder is empty, and TreeGrid.alwaysShowOpener is false, the opener icon
*  will be hidden.
*  
*  For more information on load on demand trees, and how we determine whether
*  a node is a a folder or a leaf, please refer to the treeDataBinding
*  documentation.
*  
*  The opener icon URL is derived from the specified
*  TreeGrid.openerImage or TreeGrid.connectorImage depending on
*  TreeGrid.showConnectors. If TreeGrid.showSelectedOpener is specified
*  a separate opener icon will be displayed for selected nodes.
* @type {boolean}
* @default true
*/
showOpener?: boolean;

/**
* If TreeGrid.showOpener is true, should we display the opener icon
*  for folders even if they have no children?
*  
*  Note that for trees which
*  TreeGrid.loadDataOnDemand, we may not know if a folder
*  has any descendants if it has never been opened. As such we will show the
*  opener icon next to the folder. Once the user opens the icon and a fetch occurs,
*  if the folder is empty, and this property is false, the opener icon
*  will be hidden.
*  
*  For more information on load on demand trees, and how we determine whether
*  a node is a a folder or a leaf, please refer to the treeDataBinding
*  documentation.
* @type {boolean}
* @default false
*/
alwaysShowOpener?: boolean;

/**
* If TreeGrid.showOpener is true, should a different opener icon be displayed
*  for selected nodes? This provides a way for developers to show a "selected"
*  version of the opener icon set which looks optimal with the 
*  cellStyleSuffixes applied the selected record.
*  
*  The selected icon URL is created by appending the suffix 
*  "_selected" to the TreeGrid.openerImage or
*  TreeGrid.connectorImage.
* @type {boolean}
* @default false
*/
showSelectedOpener?: boolean;

/**
* The base filename of the opener icon for the folder node when 'showConnectors' is false
*  for this TreeGrid.
*  The opener icon is displayed beside the folder icon in the Tree column for folder nodes.
*  Clicking on this icon will toggle the open state of the folder.
*  The filenames for these icons are assembled from this base filename and the state of the
*  node, as follows:
*  If the openerImage is set to {baseName}.{extension}, 
*  {baseName}_opened.{extension} will be displayed next to opened folders, and
*  {baseName}_closed.{extension} will be displayed next to closed folders, or
*  if this page is in RTL mode, {baseName}_opened_rtl.{extension} and
*  {baseName}_closed_rtl.{extension} will be used.
*  
*  If TreeGrid.showSelectedOpener is true the URL for selected nodes will append
*  the string "_selected" to the image URLs described above.
*  So for an openerImage set to {baseName}.{extension}, the URLs
*  for selected records would be
*  {baseName}_opened_selected.{extension}, 
*  {baseName}_closed_selected.{extension}, etc.
* @type {SCImgURL}
* @default "[SKIN]opener.gif"
*/
openerImage?: SCImgURL;

/**
* The base filename for connector icons shown when TreeGrid.showConnectors is true.
*  Connector icons are rendered into the title field of each row and show the dotted
*  hierarchy lines between siblings of the same parent node. For each node, a connector icon
*  may be shown:
*  
* - As an opener icon for folder nodes, next to the folder icon
*  
* - In place of an opener icon for leaf nodes, next to the leaf icon
*  
* - As a standalone vertical continuation line in the indent to the left of the node, to show
*    a connection between some ancestor node's siblings (only relevant if
*    TreeGrid.showFullConnectors is true).
*  
*  Note that TreeGrid.showFullConnectors governs whether connector lines will be
*  displayed for all indent levels, or just for the innermost level of the tree.
*  
*  The filenames for these icons are assembled from this base filename and the state of the
*  node. Assuming the connectorImage is set to {baseName}.{extension}, the
*  full set of images to be displayed will be:
*  
*  {baseName}_ancestor[_rtl].{extension} if TreeGrid.showFullConnectors
*  is true, this is the URL for the vertical continuation image to be displayed at the
*  appropriate indent levels for ancestor nodes with subsequent children.
*  
*  For nodes with no children:
*  
*  
* - {baseName}_single[_rtl].{extension}: Shown when there is no connector line
*  attached to the parent or previous sibling, and no connector line to the next sibling. For
*  TreeGrid.showFullConnectors trees, there will always be a
*  connector leading to the parent or previous sibling if its present in the tree so this
*  icon can only be displayed for the first row.
*  
* - {baseName}_start[_rtl].{extension}: Shown when the there is no connector
*  line attached to the parent or previous sibling, but there is a connector to the next
*  sibling. As with _single this will only ever be used for the first row if
*  TreeGrid.showFullConnectors is true
*  
* - {baseName}_end[_rtl].{extension}: Shown if we are not showing a connector 
*  line attached to the next sibling of this node (but are showing a connection to the previous
*  sibling or parent).
*  
* - {baseName}_middle[_rtl].{extension}: Shown where the we have a connector
*  line leading to both the previous sibling (or parent) and the next sibling.
*  
*  For folders with children. Note that if TreeGrid.showFullConnectors is false, open
*  folders will never show a connector to subsequent siblings:
*  
*  
* - {baseName}_opened_single[_rtl].{extension} opened folder node with 
*  children when no connector line is shown attaching to either the folder's previous sibling
*  or parent, or to any subsequent siblings.
*  
* - {baseName}_opened_start[_rtl].{extension}: opened folder with children
*  when the there is no connector line attached to the parent or previous sibling, but there 
*  is a connector to the next sibling.
*  
* - {baseName}_opened_end[_rtl].{extension}: opened folder with children 
*  if we are not showing a connector line attached to the next sibling of this node (but are
*  showing a connection to the previous sibling or parent).
*  
* - {baseName}_opened_middle[_rtl].{extension}: opened folder with children 
*  where the we have a connector line leading to both the previous sibling (or parent) and the
*  next sibling.
*  
*  
*  
* - {baseName}_closed_single[_rtl].{extension} closed folder node with 
*  children when no connector line is shown attaching to either the folder's previous sibling
*  or parent, or to any subsequent siblings.
*  
* - {baseName}_closed_start[_rtl].{extension}: closed folder with children
*  when the there is no connector line attached to the parent or previous sibling, but there 
*  is a connector to the next sibling.
*  
* - {baseName}_closed_end[_rtl].{extension}: closed folder with children 
*  if we are not showing a connector line attached to the next sibling of this node (but are
*  showing a connection to the previous sibling or parent).
*  
* - {baseName}_closed_middle[_rtl].{extension}: closed folder with children 
*  where the we have a connector line leading to both the previous sibling (or parent) and the
*  next sibling.
*  
*  (Note '[_rtl]' means that "_rtl" will be attached if isRTL() is true for this widget).
*  
*  If TreeGrid.showSelectedOpener is true the URL for selected nodes will append
*  the string "_selected" to the image URLs described above.
*  So for a connectorImage set to {baseName}.{extension}, the URLs
*  for selected records would be
*  {baseName}_ancestor[_rtl]_selected.{extension}, 
*  {baseName}_single[_rtl]_selected.{extension}, etc.
* @type {SCImgURL}
* @default "[SKIN]connector.gif"
*/
connectorImage?: SCImgURL;

/**
* For TreeGrids with loadDataOnDemand: true, a message to show the user if an attempt is 
*  made to open a folder, and thus load that node's children, while we are offline and 
*  there is no offline cache of that data. The message will be presented to the user in 
*  in a pop-up dialog box.
* @type {string}
* @default "This data not available while offline"
*/
offlineNodeMessage?: string;

/**
* For record components placed "within" the TreeGridField.treeField
*  column, should the component be indented to the position where a title would normally
*  show?
*  
*  For more general placement of embedded components, see
*  ListGrid.addEmbeddedComponent.
* @type {boolean}
* @default true
*/
indentRecordComponents?: boolean;

/**
* If no fields are specified, create a single field with 
*  TreeGridField.treeField set to true to show the tree.
*  
*  This automatically generated field will display values derived by calling
*  TreeGrid.getNodeTitle, and have the column title set to the specified
*  TreeGrid.treeFieldTitle.
*  
*  Has no effect if fields are explicitly specified.
*  
*  This is a convenience setting to allow a TreeGrid to be created without specifying a
*  field list. If fields are specified, refer to the documentation on property
*  TreeGrid.autoAssignTreeField for a way to automatically have one of the fields be
*  use as the tree field if no fields have TreeGridField.treeField set.
*  
*  For databound treeGrids, if there is no explicit fields array specified, developers
*  who wish to pick up all fields from the DataSource definition rather than displaying
*  this single automatically generated tree field may 
*  either set this property to false, or set TreeGrid.useAllDataSourceFields
*  to true.
* @type {boolean}
* @default true
*/
createDefaultTreeField?: boolean;

/**
* If true, the set of fields given by the "default binding" (see 
*  DataBoundComponent.fields) is used, with any fields specified in
*  component.fields acting as overrides that can suppress or modify the
*  display of individual fields, without having to list the entire set of fields that
*  should be shown.
*  
*  If component.fields contains fields that are not found in the DataSource,
*  they will be shown after the most recently referred to DataSource field. If the new
*  fields appear first, they will be shown first.
*  
*  ${isc.DocUtils.linkForExampleId('validationFieldBinding', 'This example')} shows a mixture of component
*  fields and DataSource fields, and how they interact for validation.
*  
*  This setting may be cleared if a FieldPicker is used to edit the component's field
*  order.
* @type {boolean}
* @default null
*/
useAllDataSourceFields?: boolean;

/**
* Which nodes should be opened automatically. This applies directly to 
*  ResultTree.autoOpen.
* @type {string}
* @default null
*/
autoOpenTree?: string;

/**
* Not applicable to TreeGrids, as the TreeGrid.data already represents a tree.
* @type {string | Array<Partial<string>>}
* @default null
*/
groupByField?: string | Array<Partial<string>>;

/**
* If true, when folders are opened / closed children will be animated into view.
*  
*  Folder animations are automatically disabled if ListGrid.autoFitData is set to "vertical" or "both", or 
*  if ListGrid.showRecordComponents are used.
* @type {boolean}
* @default true
*/
animateFolders?: boolean;

/**
* If TreeGrid.animateFolders is true for this grid, this number can be set to designate
*  the maximum number of rows to animate at a time when opening / closing a folder.
* @type {Integer}
* @default null
*/
animateFolderMaxRows?: Integer;

/**
* When animating folder opening / closing, if TreeGrid.animateFolderSpeed is not
*  set, this property designates the duration of the animation in ms.
* @type {number}
* @default "100"
*/
animateFolderTime?: number;

/**
* When animating folder opening / closing, this property designates the speed of the
*  animation in pixels shown (or hidden) per second. Takes precedence over the 
*  TreeGrid.animateFolderTime property, which allows the developer to specify a
*  duration for the animation rather than a speed.
* @type {number}
* @default "3000"
*/
animateFolderSpeed?: number;

/**
* When animating folder opening / closing, this property can be set to apply an
*  animated acceleration effect. This allows the animation speed to be "weighted", for
*  example expanding or collapsing at a faster rate toward the beginning of the animation
*  than at the end.
* @type {AnimationAcceleration}
* @default null
*/
animateFolderEffect?: AnimationAcceleration;

/**
* Should the current TreeGrid.getOpenState of the tree be included 
*  along with other details when saving this grid's TreeGrid.getViewState?
* @type {boolean}
* @default true
*/
saveOpenStateInViewState?: boolean;

/**
* If TreeGrid.showDropEndSpace is set to true, this property governs how large the space under 
*  the last node during drop should be. If unset, the spacer will be sized to be
*  half the specified ListGrid.cellHeight for the grid.
* @type {Integer}
* @default null
*/
dropEndSpace?: Integer;

/**
* When the user drags over the treeGrid body, should the grid show some space under
*  the last node in the grid allowing the user to drop after the last node? The
*  height of this space can be customized via TreeGrid.dropEndSpace
*  
*  See also ListGrid.canDropInEmptyArea
*  and TreeGrid.canDropSiblingAfterLastNode
* @type {boolean}
* @default true
*/
showDropEndSpace?: boolean;

/**
* If TreeGrid.canAcceptDroppedRecords is true for this treeGrid, this property governs
*  whether the user can drop between, or over records within the grid.
*  This controls what RecordDropPosition is passed to the TreeGrid.recordDrop
*  event handler.
* @type {RecordDropAppearance}
* @default "both"
*/
recordDropAppearance?: RecordDropAppearance;

/**
* Should nodes in this TreeGrid show folder / leaf node icons by default?
*  
*  May be overridden for folder nodes via TreeGrid.showFolderIcons
*  
*  See TreeGrid.getIcon for more details on treeGrid icons
* @type {boolean}
* @default true
*/
showNodeIcons?: boolean;

/**
* Should folder nodes in this TreeGrid show icons by default?
*  
*  If unset, folder node icons will be shown if TreeGrid.showNodeIcons is true
*  
*  See TreeGrid.getIcon for more details on treeGrid icons
* @type {boolean}
* @default null
*/
showFolderIcons?: boolean;

/**
* When performing a drag and drop to add or move data within the tree, should users be
*  able to make the dropped node a sibling of the last node in the tree by dropping just below
*  it?
*  
*  When set to true, if a user performs a drop action in the space immediately below the
*  last node, (less than half the grid's specified cellHeight away), the dropped data 
*  will be added to the parent of that last node, making them siblings. If the parent
*  TreeNode.canAcceptDrop, the dropped data will be added to
*  the first ancestor that will accept a drop.
*  
*  If the user performs the drop lower down in the empty area below the last row, of if
*  this property is set to false, the dropped data will be added as a 
*  last child to the root node instead.
* @type {boolean}
* @default true
*/
canDropSiblingAfterLastNode?: boolean;



/**
*  Uses a "fetch" operation on the current DataSource to retrieve data
*  that matches the provided criteria, and displays the matching data in this component as a
*  tree.
*  
*  This method will create a ResultTree to manage tree data, which will
*  subsequently be available as treeGrid.data. DataSource records
*  returned by the "fetch" operation are linked into a tree structure according to
*  DataSourceField.primaryKey and
*  DataSourceField.foreignKey declarations on DataSource fields. See the
*  treeDataBinding topic for complete details.
*  
*  By default, the created ResultTree will use folder-by-folder load on demand, asking the
*  server for the children of each folder as the user opens it.
*  
*  The ResultTree created by fetchData() can be customized by setting
*  ListGrid.dataProperties to an Object containing properties and methods to apply to
*  the created ResultTree. For example, the property that determines whether a node is a
*  folder (Tree.isFolderProperty) can be customized, or
*  level-by-level loading can be disabled via
*  ResultTree.loadDataOnDemand.
*  
*  If TreeGrid.loadDataOnDemand is true, this grid will issue fetch requests each time the
*  user opens a folder to load its child data.
*  The criteria on this fetch request will consist of the appropriate value for the
*  foreignKey field, combined with the criteria passed to fetchData()
*  when the data was first loaded.
*  This allows you to retrieve multiple different tree structures from the same DataSource.
*  However note that the server is expected
*  to always respond with an intact tree - returned nodes which do not have parents are dropped
*  from the dataset and not displayed.
*  
*  The callback passed to fetchData will fire once, the first time that data is
*  loaded from the server. If using folder-by-folder load on demand, use the
*  ResultTree.dataArrived notification to be notified each time new nodes are loaded.
*  
*  Note that when calling 'fetchData()', changes to criteria may or may not result in a
*  DSRequest to the server due to client-side filtering (see ResultTree.fetchMode. 
*  You can call willFetchData(criteria) to determine if new criteria will result in a 
*  server fetch.
*  
*  If you need to force data to be re-fetched, you can call invalidateCache() and 
*  new data will automatically be fetched from the server using the current criteria 
*  and sort direction.
*  When using invalidateCache() there is no need to also call fetchData() and 
*  in fact this could produce unexpected results.
*  
* 
* 
* @param {Criteria=} Search criteria. If a DynamicForm is passed                     in as this argument instead of a raw criteria                      object, will be derived by calling                      DynamicForm.getValuesAsCriteria
* @param {DSCallback=} callback to invoke when a fetch is complete. Fires                     only if server contact was required
* @param {DSRequest=} additional properties to set on the DSRequest                     that will be issued
*/
fetchData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Retrieves data that matches the provided criteria and displays the matching data in this
*  component.
*  
*  This method behaves exactly like TreeGrid.fetchData except that
*  DSRequest.textMatchStyle is automatically set to "substring" so that String-valued
*  fields are matched by case-insensitive substring comparison.
* 
* 
* @param {Criteria=} Search criteria.            If a DynamicForm is passed in as this argument            instead of a raw criteria object, will be derived by calling            DynamicForm.getValuesAsCriteria
* @param {DSCallback=} callback to invoke when a fetch is complete. Fires                     only if server contact was required; see                      ListGrid.fetchData for details
* @param {DSRequest=} for databound components only - optional              additional properties to set on the DSRequest that will be issued
*/
filterData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Exports this component's data with client-side formatters applied, so is suitable for direct
*  display to users. See ListGrid.exportClientData for details of the general 
*  requirements and restrictions when exporting client data.
*  
*  The following notes apply when exporting client data from TreeGrids:
*  
* - Export only works correctly if you specify TreeGrid.fields; if you allow it to
*    generate a TreeGrid.createDefaultTreeField, nothing will be exported
*  
* - Only visible nodes are exported; if you close a node, its children are not exported 
*    even if they are loaded and known to the client
*  
* - Tree nodes are exported as a flat list, in the same order they are displayed in the 
*    TreeGrid
*  
*  
*  If your TreeGrid has custom formatters, formatted values will be exported by default, with
*  HTML normalized to text where possible. Since some levels of HTML normalizing aren't 
*  possible, this may result in missing or incorrect export values. In this case, you have 
*  two possible approaches:
*  
* - Set ListGridField.exportRawValues on the field. This will export
*    the raw underlying value of the field; your formatter will not be called
*  
* - Have your formatter call TreeGrid.isExportingClientData
*    and perform whatever alternative formatting you require if that method returns true
*  
* 
* @param {Partial<DSRequest>=} Request properties for the export. Note that specifying DSRequest.exportData on the request properties  allows the developer to pass in an explicit data set to export.
* @param {RPCCallback=} Optional callback. If you specify DSRequest.exportToClient: false in the request  properties, this callback will fire after export completes. Otherwise the callback will  fire right before the download request is made to the server.
*/
exportClientData(requestProperties?:Partial<DSRequest>, callback?:(response:RPCResponse, rawData:any, request:RPCRequest) => void): void;

/**
*  Returns true if this component is currently 
*  TreeGrid.exportClientData. This method can be called from
*  custom cell formatters if you need to return a different formatted value for an export
*  than for a live TreeGrid
* 
*/
isExportingClientData(): boolean;

/**
*  Opens the folder specified by node if it's closed, and closes it if it's open.
*  TreeGrid will redraw if there's a change in the folder's open/closed state.
* 
* 
* @param {TreeNode | string | Integer | NodeLocator} the node in question, or the                                the node's ID, or a NodeLocator                                object
*/
toggleFolder(node:TreeNode | string | Integer | NodeLocator): void;

/**
*  Opens a folder.
*  
*  Executed when a folder node receives a 'doubleClick' event.
*  If you override this method, the single parameter passed will be
*  a reference to the relevant folder node in the tree's data.
*  
*  See the ListGrid Widget Class for inherited recordClick and recordDoubleClick events.
* 
* 
* @param {TreeNode} node to open
* @param {string=} optional parameter containing the full path to the node.                   This is essential context for a                    Tree.multiLinkTree, but is not                    required in ordinary trees
*/
openFolder(node:TreeNode, path?:string): void;

/**
*  Closes a folder.
* 
* 
* @param {TreeNode} node to close
*/
closeFolder(node:TreeNode): void;

/**
*  Not applicable to TreeGrids, as the TreeGrid.data already represents a tree.
* 
*/
groupBy(): void;

/**
*  Should this folder be animated when opened / closed? Default implementation will
*  return true if TreeGrid.animateFolders is true, the folder being actioned
*  has children and the child-count is less than the result of
*  TreeGrid.getAnimateFolderMaxRows.
* 
* @param {TreeNode} folder being opened or closed.
*/
shouldAnimateFolder(folder:TreeNode): boolean;

/**
*  If TreeGrid.animateFolders is true for this treeGrid, this method returns the 
*  the maximum number of rows to animate at a time when opening / closing a folder.
*  This method will return TreeGrid.animateFolderMaxRows if set. Otherwise
*  the value will be calculated as 3x the number of rows required to fill a viewport,
*  capped at a maximum value of 75.
* 
*/
getAnimateFolderMaxRows(): Integer;

/**
*  Returns a snapshot of the current open state of this grid's data as
*  a TreeGridOpenState object.
*  This object can be passed to TreeGrid.setOpenState to open the same set of folders
*  within the treeGrid's data (assuming the nodes are still present in the data).
* 
*/
getOpenState(): TreeGridOpenState;

/**
*  Reset this set of open folders within this grid's data to match the 
*  TreeGridOpenState object passed in.
*  Used to restore previous state retrieved from the grid by a call to 
*  TreeGrid.getOpenState.
* 
* 
* @param {TreeGridOpenState} Object describing the desired set of open folders.
*/
setOpenState(openState:TreeGridOpenState): void;

/**
*  Returns a snapshot of the current selection within this treeGrid as 
*  a ListGridSelectedState object.
*  This object can be passed to TreeGrid.setSelectedPaths to reset this grid's selection
*  the current state (assuming the same data is present in the grid).
* 
*/
getSelectedPaths(): ListGridSelectedState;

/**
*  Reset this grid's selection to match the ListGridSelectedState object passed in.
*  Used to restore previous state retrieved from the grid by a call to 
*  TreeGrid.getSelectedPaths.
* 
* 
* @param {ListGridSelectedState} Object describing the desired selection state of                       the grid
*/
setSelectedPaths(selectedPaths:ListGridSelectedState): void;

/**
*  Overridden to return a TreeGridViewState object for the grid including
*  open state if TreeGrid.saveOpenStateInViewState is true.
* 
*/
getViewState(): TreeGridViewState;

/**
*  Overridden to take a TreeGridViewState object.
* 
* 
* @param {TreeGridViewState} Object describing the desired view state for the grid
*/
setViewState(viewState:TreeGridViewState): void;

/**
*  Set the Tree object this TreeGrid will view and manipulate.
* 
* 
* @param {Tree} Tree to show
*/
setData(newData:Tree): void;

/**
*  This method is not currently supported for this grid-type. See 
*  ListGrid.loadAllRecords for more information.
* 
* 
* @param {Integer=} optional maximum record count - if passed, no fetch takes place                if maxRecords is below the known length of the data
* @param {DSCallback=} callback to fire if a fetch is issued - if all data was                 already loaded, the callback is fired with no parameters
*/
loadAllRecords(maxRecords?:Integer, callback?:DSCallback): boolean;

/**
*  Overridden to disallow editing of the TreeNode.name field of this grid's data
*  tree. Also disallows editing of the auto-generated tree field, which displays the result
*  of Tree.getTitle on the node.
* 
*/
canEditCell(): boolean;

/**
*  This inherited ListGrid.startEditingNew is not supported by the TreeGrid
*  since adding a new tree node arbitrarily at the end of the tree is usually not useful.
*  Instead, to add a new tree node and begin editing it, use either of these two strategies:
*  
*  
* - add a new node to the client-side Tree model via Tree.add, then use
*  ListGrid.startEditing to begin editing this node. Note that if using a DataSource, when the
*  node is saved, an "update" operation will be used since adding a node directly to the
*  client-side ResultTree effectively means a new node has been added server side.
*  
* - use DataSource.addData to immediately save a new node. Automatic cache sync
*  by the ResultTree will cause the node to be integrated into the tree. When the
*  callback to addData() fires, locate the new node by matching primary key and call
*  ListGrid.startEditing to begin editing it.
*  
* 
* 
* @param {object=} Optional initial set of properties for the new record
* @param {boolean=} Whether to suppress the default behavior of moving focus                  to the newly shown editor.
*/
startEditingNew(newValues?:object, suppressFocus?:boolean): void;

/**
* 
*  This override to ListGrid.rowClick. This implementation calls through to the
*  TreeGrid.nodeClick, TreeGrid.folderClick, TreeGrid.leafClick methods, as
*  appropriate unless the click was on the expand/collapse control of a folder - in which case
*  those callbacks are not fired.
*  
*  Do not override this method unless you need a rowClick callback that fires even when the
*  user clicks on the expand/collapse control. If you do override this method, be sure to call
*  return this.Super("rowClick", arguments); at the end of your override to
*  preserver other handler that are called from the superclass (for example,
*  ListGrid.recordClick.
*  
* 
* 
* @param {TreeNode} record that was clicked on
* @param {number} index of the row where the click occurred
* @param {number} index of the col where the click occurred
*/
rowClick(record:TreeNode, recordNum:number, fieldNum:number): boolean;

/**
* 
*  Handle a doubleClick on a tree node - override of ListGrid stringMethod of same name. If
*  the node is a folder, this implementation calls TreeGrid.toggleFolder on it. If
*  the node is a leaf, calls TreeGrid.openLeaf on it.
* 
* 
* @param {TreeGrid} the treeGrid that contains doubleclick event
* @param {TreeNode} the record that was double-clicked
* @param {number} number of the record clicked on in the current set of                 displayed records (starts with 0)
* @param {TreeGridField} the field that was clicked on (field definition)
* @param {number} number of the field clicked on in the treeGrid.fields                 array
* @param {object} value of the cell (after valueMap, etc. applied)
* @param {object} raw value of the cell (before valueMap, etc applied)
*/
recordDoubleClick(viewer:TreeGrid, record:TreeNode, recordNum:number, field:TreeGridField, fieldNum:number, value:object, rawValue:object): boolean;

/**
*  Executed when a leaf node receives a 'doubleClick' event. This handler must be
*  specified as a function, whose single parameter is a reference to the relevant leaf node in
*  the tree's data.
*  See the ListGrid Widget Class for inherited recordClick and recordDoubleClick events.
* 
* 
* @param {TreeNode} node to open
*/
openLeaf(node:TreeNode): void;

/**
* 
*  During a drag-and-drop interaction, this method is called to transfer a set of records that
*  were dropped onto some other component. This method is called after the set of records has
*  been copied to the other component. Whether or not this component's data is modified is 
*  determined by the value of DataBoundComponent.dragDataAction.
*  
*  With a dragDataAction of "move", a databound component will issue "remove"
*  dsRequests against its DataSource to actually remove the data, via
*  DataSource.removeData.
* 
* 
*/
transferDragData(): Array<any>;

/**
*  Return an icon to display as a drag tracker when the user drags some node(s).
*  Default implementation:
*  If multiple nodes are selected and TreeGrid.manyItemsImage is defined, this
*  image will be returned.
*  Otherwise returns the result of TreeGrid.getIcon for the first node being 
*  dragged.
*  
*  Note: Only called if ListGrid.dragTrackerMode is set to "icon". 
* 
* @param {Array<Partial<ListGridRecord>>} Records being dragged
*/
getDragTrackerIcon(records:Array<Partial<ListGridRecord>>): string;

/**
* 
*  NOTE: Applicable only to Tree.multiLinkTree; if called on a 
*  regular TreeGrid, returns an empty array.
*  
*  During a drag-and-drop interaction, this method returns the set of node occurrences being 
*  dragged out of the component, wrapped inside NodeLocators. In the default 
*  implementation, this is the list of currently selected node occurrences
* 
* 
* @param {TreeGrid} source grid from which the records will be transferred
*/
getDraggedNodeLocators(source:TreeGrid): Array<Partial<NodeLocator>>;

/**
*  
*  Should this treeGrid process a specific drop event?
*  Note: See treeGridDrop for an overview of TreeGrid drag and drop behavior. 
*  
*  This method overrides ListGrid.willAcceptDrop and works as follows:
*  
*  First, ListGrid.willAcceptDrop (the superclass definition) is consulted. If it
*  returns false, then this method returns false immediately.
*  This handles the following cases:
*  - reordering of records withing this TreeGrid when ListGrid.canReorderRecords is true
*  - accepting dropped records from another dragTarget when ListGrid.canAcceptDroppedRecords 
*   is true and the dragTarget gives us a valid set of records to drop into place.
*  - disallowing drop over disabled nodes, or nodes with canAcceptDrop:false
*  
*  This method will also return false if the drop occurred over a leaf node whos immediate 
*  parent has canAcceptDrop set to false
*  If TreeGrid.canReparentNodes is true, and the user is dragging a node from one folder
*  to another, this method will return true to allow the change of parent folder.
*  
*  Otherwise this method returns true.
* 
* 
*/
willAcceptDrop(): boolean;

/**
*  Returns the row number of the most recent mouse event.
* 
* @param {Integer=} optional y-coordinate to obtain row number, in lieu of the y            coordinate of the last mouse event
*/
getEventRow(y?:Integer): Integer;

/**
*  Simulates a drag / drop type transfer of the selected records in some other grid to this
*  treeGrid, without requiring any user interaction.
*  See the dragging documentation for an overview of grid drag/drop data
*  transfer.
* 
* @param {ListGrid} source grid from which the records will be transferred
* @param {TreeNode=} parent node into which records should be dropped - if null  records will be transferred as children of the root node.
* @param {Integer=} target index (drop position) within the parent folder
* @param {Callback=} optional callback to be fired when the transfer process has            completed. The callback will be passed a single parameter "records",            the list of nodes actually transferred to this component (it is called             "records" because this logic is shared with ListGrid).
*/
transferSelectedData(sourceGrid:ListGrid, folder?:TreeNode, index?:Integer, callback?:Callback): void;

/**
*  Return the horizontal alignment for cell contents. Default implementation will always
*  left-align the special TreeGridField.treeField [or right-align if the page is in
*  RTL mode] - otherwise will return ListGridField.cellAlign if specified, otherwise
*  ListGridField.align.
* 
* 
* 
* @param {ListGridRecord} this cell's record
* @param {number} row number for the cell
* @param {number} column number of the cell
*/
getCellAlign(record:ListGridRecord, rowNum:number, colNum:number): Alignment;

/**
* 
*  Returns the title to show for a node in the tree column. If the field specifies the
*  name attribute, then the current node[field.name] is returned.
*  Otherwise, the result of calling Tree.getTitle on the node is called.
*  
*  You can override this method to return a custom title for node titles in the tree column.
*  
*  Note: if a default tree field is generated for you by TreeGrid.createDefaultTreeField
*  being true, and you've overridden this method, it will be called with recordNum: -1
*  during sorting of the tree field, if the tree is ResultTree.fetchMode.
* 
* 
* @param {TreeNode} The node for which the title is being requested.
* @param {number} The index of the node.
* @param {string} The field for which the title is being requested.
*/
getNodeTitle(node:TreeNode, recordNum:number, field:string): HTMLString;

/**
*  Returns true if the last event occurred over the indented area or over the
*  open / close icon of a folder node in this TreeGrid. Returns false if the event
*  did not occur over a folder node.
* 
* 
*/
isOverOpenArea(): boolean;

/**
*  Returns true if the last event occurred over TreeGrid.getExtraIcon
*  for the current node.
*  
*  Returns false if the event did not occur over an extraIcon, or if no extraIcon is
*  showing for the node in question.
* 
* 
*/
isOverExtraIcon(): boolean;

/**
*  Get the appropriate open/close opener icon for a node. Returns null if TreeGrid.showOpener is
*  set to false.
* 
* 
* @param {TreeNode} tree node in question
*/
getOpenIcon(node:TreeNode): URL;

/**
*  Set the icon for a particular treenode to a specified URL
* 
* 
* @param {TreeNode} tree node
* @param {SCImgURL} path to the resource
*/
setNodeIcon(node:TreeNode, icon:SCImgURL): void;

/**
* 
*  This method is called when a folder is opened either via the user manipulating the
*  expand/collapse control in the UI or via TreeGrid.openFolder. You can return
*  false to cancel the open.
* 
* 
* @param {TreeNode} the folder (record) that is being opened
* @param {string=} optional parameter containing the full path to the node.             This is essential context for a              Tree.multiLinkTree, but is not              required in ordinary trees
*/
folderOpened(node:TreeNode, path?:string): boolean;

/**
* 
*  This method is called when a folder is closed either via the user manipulating the
*  expand/collapse control in the UI or via TreeGrid.closeFolder. You can return
*  false to cancel the close.
* 
* 
* @param {TreeNode} the folder (record) that is being closed
*/
folderClosed(node:TreeNode): boolean;

/**
* 
*  This method is called when a folder record is clicked on.
* 
* 
* @param {TreeGrid} The TreeGrid on which folderClick() occurred.
* @param {TreeNode} The folder (record) that was clicked
* @param {number} Index of the row where the click occurred.
*/
folderClick(viewer:TreeGrid, folder:TreeNode, recordNum:number): void;

/**
* 
*  This method is called when a leaf record is clicked on.
* 
* 
* @param {TreeGrid} The TreeGrid on which leafClick() occurred.
* @param {TreeNode} The leaf (record) that was clicked
* @param {number} Index of the row where the click occurred.
*/
leafClick(viewer:TreeGrid, leaf:TreeNode, recordNum:number): void;

/**
* 
*  This method is called when a leaf or folder record is clicked on. Note that if you set
*  up a callback for nodeClick() and e.g. TreeGrid.leafClick, then
*  both will fire (in that order) if a leaf is clicked on.
* 
* 
* @param {TreeGrid} The TreeGrid on which leafClick() occurred.
* @param {TreeNode} The node (record) that was clicked
* @param {number} Index of the row where the click occurred.
*/
nodeClick(viewer:TreeGrid, node:TreeNode, recordNum:number): void;

/**
* 
*  This method is called when a context click occurs on a folder record.
* 
* 
* @param {TreeGrid} The TreeGrid on which the contextclick occurred.
* @param {TreeNode} The folder (record) on which the contextclick occurred.
* @param {number} Index of the row where the contextclick occurred.
*/
folderContextClick(viewer:TreeGrid, folder:TreeNode, recordNum:number): boolean;

/**
* 
*  This method is called when a context click occurs on a leaf record.
* 
* 
* @param {TreeGrid} The TreeGrid on which the contextclick occurred.
* @param {TreeNode} The leaf (record) on which the contextclick occurred.
* @param {number} Index of the row where the contextclick occurred.
*/
leafContextClick(viewer:TreeGrid, leaf:TreeNode, recordNum:number): boolean;

/**
* 
*  This method is called when a context click occurs on a leaf or folder record. Note that
*  if you set up a callback for nodeContextClick() and
*  e.g. TreeGrid.leafContextClick, then both will fire (in that order) if a leaf
*  is contextclicked - unless nodeContextClick() returns false, in which case
*  no further contextClick callbacks will be called.
* 
* 
* @param {TreeGrid} The TreeGrid on which the contextclick occurred.
* @param {TreeNode} The node (record) on which the contextclick occurred.
* @param {number} Index of the row where the contextclick occurred.
*/
nodeContextClick(viewer:TreeGrid, node:TreeNode, recordNum:number): boolean;

/**
*  Notification method fired whenever this TreeGrid receives new data nodes from the 
*  dataSource. Only applies to databound TreeGrids where TreeGrid.data is a 
*  ResultTree - either explicitly created and applied via TreeGrid.setData or
*  automatically generated via a TreeGrid.fetchData call.
*  
*  Note that dataArrived(), unlike TreeGrid.dataChanged, only fires in limited
*  circumstances - when data for a ResultTree arrives from the server due to a fetch
*  or cache invalidation, or as a result of filtering. If you want to catch all data
*  changes, you should instead react to TreeGrid.dataChanged.
* 
* 
* @param {TreeNode} The parentNode for which children were just loaded
*/
dataArrived(parentNode:TreeNode): void;

/**
*  Notification method fired when the TreeGrid's data changes, for any reason.
*  If overridden (rather than Class.observe), you must Class.Super to ensure proper Framework behavior.
*  
*  Examples of why data changed might be:
*  
* - a call to ListGrid.addData, ListGrid.updateData, or ListGrid.removeData
*  
* - DataSource updates from the server for ResultTree data
*  (triggered by record editing, etc.)
*  
* - fetches arriving back from the server for ResultTree data
*  
* - programmatic changes to Tree data if made through APIs such as
*  Tree.add, Tree.remove, etc.
*  
* - cache invalidation
*  
* - filtering
*  
*  Calling TreeGrid.setData doesn't call this notification directly, but it may
*  fire if one of the above listed events is triggered (e.g. a server fetch for 
*  ResultTree data).
*  
*  Note that the operationType parameter is optional and will be passed and
*  contain the operation (e.g. "update") if this notification was triggered by a fetch,
*  an ListGrid.addData, ListGrid.updateData, or ListGrid.removeData, or a DataSource
*  update for ResultTree data (the first three reasons listed above) but otherwise
*  will be undefined.
* 
* 
* @param {string=} optionally passed operation causing the change
*/
dataChanged(operationType?:string): void;

/**
*  Get an additional icon to show between the open icon and folder/node icon for a particular 
*  node.
*  
*  NOTE: If ListGrid.selectionAppearance is "checkbox", this method will
*  NOT be called. Extra icons cannot be shown for that appearance.
* 
* 
* @param {TreeNode} tree node in question
*/
getExtraIcon(node:TreeNode): URL;

/**
*  Get the appropriate icon for a node.
*  
*  By default icons are derived from TreeGrid.folderIcon and TreeGrid.nodeIcon.
*  Custom icons for individual nodes can be overridden by setting the TreeGrid.customIconProperty
*  on a node.
*  
*  To suppress icons altogether, set TreeGrid.showNodeIcons, or, 
*  for folder icons only TreeGrid.showFolderIcons.
*  You can also provide an override of this method that returns null to suppress icons for
*  specific nodes.
*  
*  Note that the full icon URL will be derived by applying Canvas.getImgURL to the
*  value returned from this method.
* 
* 
* @param {TreeNode} tree node in question
* @param {Integer=} the row number of the node in the TreeGrid. This additional               context is required for Tree.multiLinkTree               because the same node can appear in multiple places
*/
getIcon(node:TreeNode, rowNum?:Integer): URL;

/**
*  The superclass event ListGrid.recordDrop does not fire on a TreeGrid, use
*  TreeGrid.folderDrop instead.
* 
* 
*/
recordDrop(): void;

/**
* 
*  Process a drop of one or more nodes on a TreeGrid folder.
*  Note: See treeGridDrop for an overview of TreeGrid drag and drop behavior. 
*  
*  
*  This method can be overridden to provide custom drop behaviors and is a more appropriate
*  override point than the lower level Canvas.drop handler.
*  
*  
*  
*  The default behavior is to simply delegate to the TreeGrid.transferNodes method; thus, the 
*  correct way to perform a programmatic folder drop, with all the built-in behaviors described
*  below, is to call transferNodes()
*  
*  If this is a self-drop, nodes are simply reordered. An "update" operation will
*  be submitted to update the Tree.parentIdField field of the moved node(s). 
*  
*  For a drop from another widget, TreeGrid.transferDragData is called which,
*  depending on the TreeGrid.dragDataAction specified on the source
*  widget, may either remove the source nodes from the original list (dragDataAction:"move")
*  or just provide a copy to this tree (dragDataAction:"copy").
*  
*  In either case the new row(s) appear in the folder at the index
*  specified by the arguments of the same name.
*  
*  If this grid is databound, the new nodes will be added to the dataset by calling
*  DataSource.addData. Further, if the new nodes were dragged from another
*  databound component, and DataBoundComponent.addDropValues
*  is true, DataBoundComponent.getDropValues will be called for every item
*  being dropped.
*  
*  As a special case, if the sourceWidget is also databound and a
*  DataSourceField.foreignKey relationship is declared from the
*  sourceWidget's DataSource to this TreeGrid's DataSource, the interaction will
*  be treated as a "drag recategorization" use case such as files being placed in folders,
*  employees being assigned to teams, etc. "update" DSRequests will be submitted that
*  change the foreignKey field in the dropped records to point to the tree folder that was the
*  target of the drop. In this case no change will be made to the Tree data as such, only to
*  the dropped records. 
*  
*  For multi-record drops, Queuing is automatically used to combine all DSRequests into a
*  single HTTP Request (see QuickStart Guide, Server Framework chapter). This allows the
*  server to persist all changes caused by the drop in a single transaction (and this is
*  automatically done when using the built-in server DataSources with Power Edition and
*  above).
*  
*  If these default persistence behaviors are undesirable, 
*  return false to cancel them
*  , then implement your own behavior, 
*  typically by using grid.updateData() or addData() to add new records.
*  NOTE: the records you receive in this event are the actual Records from the source
*  component. Use DataSource.copyRecords to create a copy before modifying the records
*  or using them with updateData() or addData().
* 
* 
* @param {Array<Partial<TreeNode>>} List of nodes being dropped
* @param {TreeNode} The folder being dropped on
* @param {Integer} Within the folder being dropped on, the index at which the drop is          occurring. Only passed if TreeGrid.canReorderRecords is true.
* @param {Canvas} The component that is the source of the nodes (where the nodes               were dragged from)
*/
folderDrop(nodes:Array<Partial<TreeNode>>, folder:TreeNode, index:Integer, sourceWidget:Canvas): void;

/**
* 
*  Transfer a list of TreeNodes within this treeGrid or from from some other component 
*  (does not have to be a databound component) into this component. This method is only 
*  applicable to list-type components, such as ListGrid, 
*  TreeGrid or TileGrid. Please see the paragraph below for
*  special rules concerning Tree.isMultiLinkTree.
*  
*  This method implements the automatic drag-copy and drag-move behavior and calling it is
*  equivalent to completing a drag and drop of the nodes (the default 
*  TreeGrid.folderDrop implementation simply calls transferNodes())
*  
*  Note that this method is asynchronous - it may need to perform server turnarounds to prevent
*  duplicates in the target component's data. If you wish to be notified when the transfer
*  process has completed, you can either pass the optional callback to this method or implement
*  the DataBoundComponent.dropComplete method on this component.
*  
*  For a TreeGrid, see also TreeGrid.transferSelectedData.
*  
*  Multi-link trees
*  If both the target treeGrid and the sourceWidget for this transfer are 
*  multi-link treeGrids, the nodes parameter must be an array of 
*  NodeLocators rather than TreeNodes. Likewise, if the target (this) component
*  is a multi-link treeGrid, the folder parameter must be a NodeLocator.
*  
*  You can obtain a NodeLocator for a visible node occurence in a multi-link TreeGrid by 
*  calling its data model's Tree.getNodeLocator method.
* 
* 
* @param {Array<Partial<TreeNode>> | Array<Partial<NodeLocator>>} Nodes to transfer to this component
* @param {TreeNode} The target folder (eg, of a drop interaction), for context
* @param {Integer} Insert point within the target folder data for the transferred nodes
* @param {Canvas} The databound or non-databound component from which the nodes               are to be transferred.
* @param {Callback=} optional callback to be fired when the transfer process has            completed. The callback will be passed a single parameter "records",            the list of nodes actually transferred to this component (it is called            "records" because this is logic shared with ListGrid)
*/
transferNodes(nodes:Array<Partial<TreeNode>> | Array<Partial<NodeLocator>>, folder:TreeNode, index:Integer, sourceWidget:Canvas, callback?:Callback): void;

/**
*  When the user is dragging a droppable element over this grid, this method returns the folder
*  which would contain the item if dropped. This is the current drop node if the user is hovering
*  over a folder, or the node's parent if the user is hovering over a leaf.
* 
*/
getDropFolder(): TreeNode | NodeLocator;



}
/**
* Creates a new TreeGrid
* 
* @param typeCheckedProperties {Partial<TreeGrid>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TreeGrid} The new instance
*/
class TreeGrid{
static create(typeCheckedProperties? : Partial<TreeGrid>, uncheckedProperties?: Object): TreeGrid;







}

/**
* Button used to display a hierarchical Menu group for representing / selecting tree data.
*  
*  Important Note: this class should not be used directly - it is exposed purely for
*  i18nMessages
*/
interface TreeMenuButton extends MenuButton {
/**
* Title for this button. If not specified, the selected value from the tree will
*  be displayed instead.
* @type {string}
* @default null
*/
title?: string;

/**
* If TreeMenuButton.title is null, this value will be displayed as a title 
*  when the user has not selected any value from the hierachichal menu.
* @type {HTMLString}
* @default "Choose a value"
*/
unselectedTitle?: HTMLString;

/**
* If this button's menu (or any of its submenus) are empty, this property can be used
*  to specify the message to display (as a disabled item) in the empty menu.
* @type {HTMLString}
* @default null
*/
emptyMenuMessage?: HTMLString;

/**
* If TreeMenuButton.title is null, when the user selects an item, should we 
*  show the full path to the item, or just the item's title as the button's title?
* @type {boolean}
* @default false
*/
showPath?: boolean;

/**
* If TreeMenuButton.showPath is true, this property specifies what will 
*  appear between the folders in the selected value's path.
* @type {HTMLString}
* @default "&amp;nbsp;&amp;gt;&amp;nbsp;"
*/
pathSeparatorString?: HTMLString;

/**
* Base style to apply to the selected path within the menu. (The "over" version of this
*  style should also be defined in the stylesheet applied to this widget).
* @type {CSSStyleName}
* @default "treeMenuSelected"
*/
selectedBaseStyle?: CSSStyleName;

/**
* For a TreeMenuButton that uses a DataSource, these properties will be passed to
*  the automatically-created ResultTree. This can be used for various customizations such as
*  modifying the automatically-chosen Tree.parentIdField.
* @type {Tree}
* @default null
*/
dataProperties?: Tree;

/**
* Widget class for the menu created by this button. The default is 
*  SelectionTreeMenu.
* @type {SCClassName}
* @default "SelectionTreeMenu"
*/
treeMenuConstructor?: SCClassName;

/**
* AutoChild menu displayed when the button is clicked.
* @type {Menu}
* @default null
*/
treeMenu?: Menu;





}
/**
* Creates a new TreeMenuButton
* 
* @param typeCheckedProperties {Partial<TreeMenuButton>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TreeMenuButton} The new instance
*/
class TreeMenuButton{
static create(typeCheckedProperties? : Partial<TreeMenuButton>, uncheckedProperties?: Object): TreeMenuButton;







}

/**
* A TreeGrid that implements the Palette behavior, so it can be used as the source for 
*  drag and drop instantiation of components when combined with an EditContext as 
*  the drop target.
*  
*  Each TreeNode within TreeGrid.data can be a PaletteNode.
*/
interface TreePalette extends TreeGrid, Palette {
/**
* Defaults to apply to all components originating from this palette.
* @type {object}
* @default null
*/
componentDefaults?: object;

/**
* Option to show filter editor is disabled for treePalettes
* @type {boolean}
* @default false
*/
canShowFilterEditor?: boolean;

/**
* Option to save searches is disabled for treePalettes
* @type {boolean}
* @default false
*/
canSaveSearches?: boolean;





}
/**
* Creates a new TreePalette
* 
* @param typeCheckedProperties {Partial<TreePalette>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TreePalette} The new instance
*/
class TreePalette{
static create(typeCheckedProperties? : Partial<TreePalette>, uncheckedProperties?: Object): TreePalette;







}

/**
* This class is a synonym for SplitPane that can be used to make intent clearer.
*  It is used by some development tools for that purpose.
*/
interface TriplePane extends SplitPane {




}
/**
* Creates a new TriplePane
* 
* @param typeCheckedProperties {Partial<TriplePane>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {TriplePane} The new instance
*/
class TriplePane{
static create(typeCheckedProperties? : Partial<TriplePane>, uncheckedProperties?: Object): TriplePane;







}

/**
* The Tutorial class is a specialized version of the Tour class with Tour.mode
*  set to "tutorial".
*/
interface Tutorial extends Tour {
/**
* The tour mode allows step configuration to be simplified by using the context to
*  apply appropriate defaults.
* @type {TourMode}
* @default "tutorial"
*/
mode?: TourMode;





}
/**
* Creates a new Tutorial
* 
* @param typeCheckedProperties {Partial<Tutorial>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Tutorial} The new instance
*/
class Tutorial{
static create(typeCheckedProperties? : Partial<Tutorial>, uncheckedProperties?: Object): Tutorial;







}

/**
* FormItem that creates an HTML &lt;input type="file"&gt; control, with an interface that
*  allows a user to pick a file from his machine to upload to the server.
*  
*  NOTE: use FileItem, not UploadItem, if you are using the SmartClient
*  Server framework. FileItem is much easier to use and addresses all the limitations of
*  UploadItem discussed below. See the upload overview for
*  details.
*  
*  If a form containing an UploadItem is Canvas.redraw (which may
*  happen if other form items are shown or hidden, the form is
*  Canvas.redrawOnResize, or this or other items show validation errors) then 
*  the value in the upload item is lost (because an HTML upload field may not be created with a
*  value). For this reason, if you are building a form that combines an UploadItem with other
*  FormItems that could trigger redraw()s, recommended practice is to place each UploadItem in
*  a distinct DynamicForm instance and create the visual appearance of a single logical form 
*  via combining the DynamicForms in a Layout. For the same reason, do not apply 
*  Validator to UploadItems: if such a validator fails, it will 
*  cause the form to be redrawn and the UploadItem's value to be lost.
*  
*  NOTE: Browser-specific behaviors: 
*  
*  
* - while getDisplayValue() can be used to retrieve the filesystem path of the uploaded file on some
*  browsers, different browsers will return either just the file name without path or the full path. It
*  is plausible that some browsers may switch behavior in the future to not supply this value at all. Do
*  not rely on this value.
*  
* - the appearance of the UploadItem is not consistent across browsers and we do not
*  recommend trying to make it consistent or trying to apply styling to the upload control at all. It is
*  a potential security problem if an end user is unable to reliably recognize the upload control, hence,
*  all browsers limit what styling can be applied. Various hacks exists to get further control of
*  styling, but it is likely these hacks will be broken by browser upgrades in the future.
*  
*/
interface UploadItem extends TextItem {
/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "FileItemEditProxy"
*/
editProxyConstructor?: SCClassName;

/**
* Base CSS class name for this UploadItem's native file input element.
*  
*  Note that the customization via CSS of a native file input element allowable by the
*  browser varies widely; in some browsers on certain platforms, it may be possible to
*  customize certain CSS properties, but not in others; or, it may be that the CSS property
*  (e.g. border) is applied differently in some browsers.
*  
*  If the textBoxStyle is changed at runtime, FormItem.updateState must be called
*  to update the visual state. However, calling updateState() will clear any file selected
*  by the user to be uploaded.
* @type {FormItemBaseStyle}
* @default "uploadItem"
*/
textBoxStyle?: FormItemBaseStyle;

/**
* When true, allow the file-selection dialog shelled by the browser to select multiple 
*  files.
*  
*  Support is not full-cycle at the server - that is, there are server APIs for retrieving
*  each file that was uploaded, but no built-in support for storing multiple files against
*  a single DataSource field. However, you can write custom server DMI code to do
*  something with the files - for instance, you could create multiple new DataSource 
*  records for each file via a server DMI like this below:
* 
*  
*   String fileNameStr = (String)dsRequest.getValues().get("image_filename").toString();
* 
*   String[] fileNames = fileNameStr.split(", ");
*   List files = dsRequest.getUploadedFiles();
* 
*   for (int i = 0; i 
* @type {boolean}
* @default true
*/
multiple?: boolean;

/**
* A comma-separated list of valid MIME types, used as a filter for the file picker window.
*  
*  Note that this property makes use of the HTML accept attribute, and so
*  relies on the browser to perform the desired filtering. For further study, see:
*  
* - HTML &lt;input&gt; accept Attribute
*  
* - The Input (Form Input) element
*  
* - File input 'accept' attribute - is it useful?
*  
* @type {string}
* @default null
*/
accept?: string;

/**
* This attribute enables camera capture functionality for mobile devices, accepting the 
*  following values:
*  
*  
* - Set it to "user" to capture using the front-facing camera.
*  
* - Set it to "environment" to capture using the rear-facing camera.
*  
*  
*  Please note that in the latest versions of Android and iOS, utilizing this attribute 
*  will consistently load the rear camera. This behavior is due to the direct camera software's 
*  ability to switch between the two cameras seamlessly.
*  
*  When working with the capture functionality of iPhones and Android devices, it's important 
*  to consider the supported DataSourceField.mimeTypes for audio, video, and image files that 
*  can be used with the fileItem.accept attribute. Here's a list of commonly supported mime types 
*  for capturing on these devices:
*  
*  Supported Image Mime Types:
*  
*  
* - image/jpeg: JPEG image format (.jpg, .jpeg)
*  
* - image/png: Portable Network Graphics format (.png)
*  
*  Supported Audio Mime Types:
*  
*  
* - audio/3gpp: 3GPP format, commonly used for audio capture.
*  
* - audio/mp4: MP4 format, widely supported for audio capture.
*  
*  Supported Video Mime Types:
*  
*  
* - video/3gpp: 3GPP format, commonly used for video capture.
*  
* - video/mp4: MP4 format, widely supported for video capture.
*  
*  Behavior Based on 'accept' Attribute
*  
*  The behavior of using the capture attribute depends on the value used in the accept 
*  attribute. For example:
*  
*  
* - accept="image/*" will load the camera ready to take pictures.
*  
* - accept="audio/*" will load the default audio recorder, not the camera.
*  
* - accept="video/*" will load the camera in video mode, ready to capture videos.
*  
*  Please note that the SmartClient framework does not control the native device behavior 
*  beyond these attribute settings.
*  
*  Lastly, keep in mind that these settings have no effect on desktop browsers; they apply 
*  exclusively to mobile devices.
*  
*  This information is "circa 2023" and may not apply to all devices.
*  
* @type {string}
* @default null
*/
capture?: string;

/**
* Width for this uploadItem. Note that SmartClient will not apply this size to the
*  native HTML &lt;input ...&gt; element written out by this formItem as this leads to
*  inconsistent appearance across different browsers. The specified width
*  acts as a minimum cell width for the item.
* @type {number}
* @default "150"
*/
width?: number;

/**
* Height for this uploadItem. Note that SmartClient will not apply this size to the
*  native HTML &lt;input ...&gt; element written out by this formItem as this leads to
*  inconsistent appearance across different browsers. The specified height
*  acts as a minimum cell width for the item.
* @type {number}
* @default "19"
*/
height?: number;



/**
*  Attempting to set the value for an upload form item is disallowed for security reasons.
*  Therefore this method will just log a warning, and not modify the value of the item.
* 
*/
setValue(): void;

/**
*  This method is not supported by UploadItem.
* 
*/
getSelectionRange(): Array<Partial<Integer>>;

/**
*  This method is not supported by UploadItem.
* 
*/
setSelectionRange(): void;

/**
*  This method is not supported by UploadItem.
* 
*/
selectValue(): void;

/**
*  This method is not supported by UploadItem.
* 
* @param {boolean=} By default the text insertion cursor will be moved to the end of the  current value - pass in this parameter to move to the start instead
*/
deselectValue(start?:boolean): void;



}
/**
* Creates a new UploadItem
* 
* @param typeCheckedProperties {Partial<UploadItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {UploadItem} The new instance
*/
class UploadItem{
static create(typeCheckedProperties? : Partial<UploadItem>, uncheckedProperties?: Object): UploadItem;







}

/**
* Chooses one or another next process element based on confirmation of a message shown to user.
*  
*  If the user clicks OK, the DecisionTask.nextElement is
*  chosen, otherwise the choice is DecisionTask.failureElement.
* @deprecated * In favor of UserConfirmationTask as of SmartClient release 13.0

*/
interface UserConfirmationGateway extends ProcessElement {




}
/**
* Creates a new UserConfirmationGateway
* 
* @param typeCheckedProperties {Partial<UserConfirmationGateway>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {UserConfirmationGateway} The new instance
*/
class UserConfirmationGateway{
static create(typeCheckedProperties? : Partial<UserConfirmationGateway>, uncheckedProperties?: Object): UserConfirmationGateway;







}

/**
* Chooses one or another next process element based on confirmation of a message shown to user.
*  
*  If the user clicks OK, the UserConfirmationTask.nextElement is
*  chosen, otherwise the choice is UserConfirmationTask.failureElement.
*/
interface UserConfirmationTask extends ProcessElement {
/**
* Message to display to the user for confirmation. To display a dynamic message see
*  UserConfirmationTask.textFormula.
* @type {string}
* @default null
*/
message?: string;

/**
* Formula to be used to calculate the message contents. Use UserConfirmationTask.message property
*  to assign a static message instead.
*  
*  Available fields for use in the formula are the current Canvas.ruleScope.
* @type {UserSummary}
* @default null
*/
textFormula?: UserSummary;

/**
* Next Process.sequences or Process.elements to execute
*  if the criteria match the process state. 
*  
*  nextElement does not need to be specified if this gateway is part of a
*  Process.sequences and has a next element in the sequence.
*  
*  Note that if there is both a sequence and a normal element
*  with the same name in the current Process, the sequence will
*  be used.
* @type {string}
* @default null
*/
nextElement?: string;

/**
* ID of the next sequence or element to proceed to if the criteria do not match.
* @type {string}
* @default null
*/
failureElement?: string;





}
/**
* Creates a new UserConfirmationTask
* 
* @param typeCheckedProperties {Partial<UserConfirmationTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {UserConfirmationTask} The new instance
*/
class UserConfirmationTask{
static create(typeCheckedProperties? : Partial<UserConfirmationTask>, uncheckedProperties?: Object): UserConfirmationTask;







}

/**
* A task that involves showing a user interface to the end user allowing the user to view and
*  input data and press a button (or do some other UI gesture) to complete the task.
*  
*  A UserTask takes the following steps:
*  
*  
* - Optionally show() or otherwise make visible the UserTask.targetView
*    or UserTask.inlineView
*  
* - Provide values to either a DynamicForm designated as the UserTask.targetForm or to
*    a ValuesManager designated as the UserTask.targetVM, via ValuesManager.setValues
*  
* - Waits for notification of completion or cancellation. The UserTask is notified of
*    completion if a SubmitItem is pressed in either the targetForm or
*    any form that is a member of the targetVM. Likewise a CancelItem
*    triggers cancellation. Direct calls to DynamicForm.cancelEditing or
*    DynamicForm.completeEditing achieve the same result.
*  
* - if cancellation occurs, the process continues to the UserTask.cancelElement
*    if specified. Otherwise the workflow is immediately finished.
*  
* - if completion occurs, values are retrieved from the form or valuesManager and applied
*    to the process state based on Task.outputField,
*    Task.outputFieldList or Task.inputField, in that order.
*  
*/
interface UserTask extends Task {
/**
* Widget that should be shown to allow user input. If this widget is a DynamicForm,
*  it will also be automatically used as the UserTask.targetForm unless either
*  targetForm or UserTask.targetVM is set.
*  
*  UserTask will automatically handle various scenarios of the
*  targetView not currently visible or draw()n, according to the following
*  rules:
*  
*  
* - if the view itself is marked hidden, it will be show()n
*  
* - if the view is inside a hidden parent, the parent will be show()n
*  
* - if the view is the Tab.pane of a tab in a TabSet, the tab will be selected
*  
* - if the view is listed in SectionStackSection.items for a which is either
*    collapsed or hidden section, the section will be shown and expanded
*  
* - if the view is listed in Window.items for a Window, the Window will be shown
*  
* - if any of these conditions apply to any parent of the targetView, the rules will be
*    applied to that parent as well. For example, the targetView is in a collapsed section
*    inside a tab which is not selected, the section will be expanded and the tab
*    selected
*  
* @type {Canvas | string}
* @default null
*/
targetView?: Canvas | string;

/**
* An inline definition of the form. Can be used in place of UserTask.targetView to encode form
*  directly in process xml.
* @type {Canvas}
* @default null
*/
inlineView?: Canvas;

/**
* DynamicForm that should be populated with data and that should provide the data for the task
*  outputs. If UserTask.targetView is a DynamicForm and would also be the targetForm, the
*  targetForm attribute can be left unset.
*  
*  Use UserTask.targetVM to use a ValuesManager instead.
* @type {DynamicForm | string}
* @default null
*/
targetForm?: DynamicForm | string;

/**
* Optional ValuesManager which will receive task inputs and provide task outputs.
*  
*  Use UserTask.targetForm instead if you want to use a DynamicForm.
* @type {ValuesManager | string}
* @default null
*/
targetVM?: ValuesManager | string;

/**
* If saveToServer is set then the associated form (UserTask.targetForm) will perform the normal 
*  DynamicForm.submit actions when submitted (typically from a SubmitItem).
*  By default the form submit action is bypassed.
* @type {boolean}
* @default false
*/
saveToServer?: boolean;

/**
* If wizard is set then associated form will be hidden after user goes to next or prev
*  step of current workflow.
* @type {boolean}
* @default false
*/
wizard?: boolean;

/**
* Does this processElement pass through output from the last executed task
*  (i.e. transient state)?
*  
*  See taskInputExpression for details on the
*  transient state outputs.
*  
*  Note that this property does not affect the task at all but is an indicator
*  to the user and to the workflow editor of the behavior of the task as coded
*  (See Process.passThruTaskOutput).
* @type {boolean}
* @default false
*/
passThruOutput?: boolean;

/**
* Next element to proceed to if the task is cancelled because the UserTask.targetForm or
*  UserTask.targetVM had cancelEditing() called on it.
*  
*  if no value is provided the workflow immediately completes.
* @type {string}
* @default null
*/
cancelElement?: string;

/**
* Previous workflow Process.sequences or Process.elements
*  that is helpful for wizards. This element will be executed if UserTask.goToPrevious
*  method of userTask will be invoked. You can get userTask for attached form by using 
*  DynamicForm.userTask property.
* @type {string}
* @default null
*/
previousElement?: string;



/**
*  Set UserTask.previousElement as next element of workflow. This method could be used to 
*  create wizard-like UI behavior. 
* 
*/
goToPrevious(): void;

/**
*  Revert any changes made in a form and finish this userTask execution. 
*  UserTask.cancelElement will be proceed as the next element of current process. 
* 
*/
cancelEditing(): void;

/**
*  Finish editing and store edited values in Process.state. 
* 
*/
completeEditing(): void;



}
/**
* Creates a new UserTask
* 
* @param typeCheckedProperties {Partial<UserTask>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {UserTask} The new instance
*/
class UserTask{
static create(typeCheckedProperties? : Partial<UserTask>, uncheckedProperties?: Object): UserTask;







}

/**
* A subclass of Layout that applies a sizing policy along the vertical axis, interpreting
*  percent and "*" sizes as proportions of the height of the layout. VLayouts will set any
*  members that do not have explicit widths to match the layout.
*/
interface VLayout extends Layout {




}
/**
* Creates a new VLayout
* 
* @param typeCheckedProperties {Partial<VLayout>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {VLayout} The new instance
*/
class VLayout{
static create(typeCheckedProperties? : Partial<VLayout>, uncheckedProperties?: Object): VLayout;







}

/**
* A subclass of Layout that simply stacks members on the vertical axis without trying to
*  manage their height. On the horizontal axis, any members that do not have explicit widths
*  will be sized to match the width of the stack.
*/
interface VStack extends Layout {




}
/**
* Creates a new VStack
* 
* @param typeCheckedProperties {Partial<VStack>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {VStack} The new instance
*/
class VStack{
static create(typeCheckedProperties? : Partial<VStack>, uncheckedProperties?: Object): VStack;







}

/**
* A validator describes a check that should be performed on a value the user is trying to
*  save.
*  
*  Validators are specified for DataSource fields via the DataSourceField.validators
*  property and validatorExecution.
*  Validators that need not be run on the server can also be specified for a
*  specific FormItem or ListGridField.
*  
*  SmartClient supports a powerful library of ValidatorType which
*  have identical behavior on both the client and the server. 
*  
*  Beyond this, custom validators can be defined on the client and custom validation logic
*  added on the server. Note that the regexp and mask validator
*  types are very flexible and can be used to perform virtually any kind of formatting check
*  that doesn't involve some large external dataset.
*  
*  Custom validators can be reused on the client by adding them to the global validator list,
*  via the Validator.addValidator method.
*/
interface Validator extends Class {
/**
* Type of the validator.
*  
*  This can be one of the built-in ValidatorType, the string "custom" to define
*  a custom validator, or the string "serverCustom" to define a server-only custom validator.
* @type {ValidatorType | string}
* @default null
*/
type?: ValidatorType | string;

/**
* Used to create a conditional validator based on AdvancedCriteria.
*  The criteria defines when the validator applies. The form current values or ListGrid record
*  is used as reference for the criteria. If the criteria match, then the validator will be
*  processed. Otherwise the validator is skipped and assumed valid.
*  
*  To use an applyWhen criteria the form or grid must use a DataSource.
*  
*  NOTE: applyWhen is not supported for "binary" fields.
*  
*  Server and client use
*  Conditional validators are enforced both on the server and on the client-side
*  when defined on a DataSource field as shown in the examples below. Note the
*  applyWhen element is treated as a Criterion.
*  
*  &lt;!-- Normal format --&gt;
*  &lt;field name="age" type="integer"&gt;
*   &lt;validators&gt;
*    &lt;validator type="integerRange" min="0" max="100"&gt;
*     &lt;applyWhen operator="or"&gt;
*      &lt;criteria&gt;
*       &lt;criterion fieldName="restrictAge" operator="equals" value="true"/&gt;
*       &lt;criterion fieldName="gender" operator="equals" value="female"/&gt;
*      &lt;/criteria&gt; 
*     &lt;/applyWhen&gt;
*    &lt;/validator&gt;
*   &lt;/validators&gt;
*  &lt;/field&gt;
* 
*  &lt;!-- Conditional requirement --&gt;
*  &lt;field name="reason" type="text"&gt;
*   &lt;validators&gt;
*    &lt;validator type="required"&gt;
*     &lt;applyWhen fieldName="willAttend" operator="equals" value="false"/&gt;
*    &lt;/validator&gt;
*   &lt;/validators&gt;
*  &lt;/field&gt;
*  
* 
*  The last example above shows an alternate to the requiredIf validator using a
*  xmlCriteriaShorthand which is only available for client-side
*  use. On the client the reason field will change appearance to match other
*  required or non-required fields when willAttend changes. Note that using
*  applyWhen for a validator of type required as in the example may
*  result in validation request being set to the server where a fetch is made against the
*  DataSource. For more details, see the discussion at the end of the
*  DataSourceField.required help topic.
*  
*  Component XML and client-only use 
*  Conditional validators can also be applied to componentXML
*  similarly to provide client-only validations or read-only state management. A common
*  use case is conditionally displaying or enabling fields. Use the readOnly
*  validator with an applyWhen value to control the read-only appearance of a
*  field. The example below shows a field which is hidden when willAttend=true.
*  
*  &lt;!-- field definition within a Component XML DynamicForm --&gt;
*  &lt;field name="reason" type="text"&gt;
*   &lt;validators&gt;
*    &lt;validator type="readOnly" fieldAppearance="hidden"&gt;
*     &lt;applyWhen fieldName="willAttend" operator="equals" value="true"/&gt;
*    &lt;/validator&gt;
*   &lt;/validators&gt;
*  &lt;/field&gt;
*  
*  
*  Conditional validators can be applied to DynamicForm or ListGrid fields in 
*  JavaScript code as well.
* @type {AdvancedCriteria}
* @default null
*/
applyWhen?: AdvancedCriteria;

/**
* User-defined list of fields on which this validator depends. Primarily used for validators
*  of type "custom" but can also be used to supplement Validator.applyWhen criteria.
*  
*  Note that for validators run on the server, fields required due to
*  dependentFields but not present in the DSRequest.data of an update
*  because they haven't changed will be filled in from the server DataSource.
* @type {Array<Partial<string>>}
* @default null
*/
dependentFields?: Array<Partial<string>>;

/**
* For a validator that is not a built-in ValidatorType, a Callback, 
*  function, or JavaScript expression to evaluate to see if this validator passes or fails.
*  
*  Because the validator declaration itself is passed as a parameter to
*  condition(), you can effectively parameterize the validator. For example, to
*  create a validator that checks that the value is after a certain date: 
*    { type:"custom", afterDate:new Date(), 
*     condition:"value.getTime() > validator.afterDate.getTime()" }
*  
*  Reusable validators, like the above, can be registered as a standard validatorType by
*  calling Validator.addValidator.
*  
*  Note that, if a field is declared with a builtin FieldType, the value passed in
*  will already have been converted to the specified type, if possible.
*  
*  For the required parameters, see the documentation for
*  Callbacks.ValidatorConditionCallback.
* @type {StringMethod | ValidatorConditionCallback}
* @default null
*/
condition?: StringMethod | ValidatorConditionCallback;

/**
* For validators of type "serverCustom" only: a scriptlet in any supported JSR223 scripting
*  language which is run in order to see if validation passes. For example:
*  
*  
*    &lt;validator type="serverCustom"&gt;
*      &lt;serverCondition language="groovy"&gt;&lt;![CDATA[
*        value &lt; dataSources.StockItem.fetchById(record.itemId).quantity
*      ]]&gt;&lt;/serverCondition&gt;
*    &lt;/validator&gt;
*  
*  The scriptlet should return a boolean true or false value - failing to return a value will
*  be considered a false result (validator failed). If your expression is syntactically
*  invalid, an exception is thrown and the error message is displayed in the client.
*  
*  See serverScript for general information on Server Scripting and JSR223, and
*  velocitySupport for general information on Velocity support, and also see below
*  for special rules for Velocity.
*  
*  Available variables
*  The following variables are available in a serverCondition:
*  
*  
* - dataSource - The current DataSource
*  
* - record - Other submitted values part of the same update
*  
* - value - The value of the field
*  
* - validator - The config of this validator, including all attributes declared on
*             the &lt;validator&gt; tag, presented as a Map
*  
* - field - The field (as a DSField object)
*  
*  Note that "record" will contain only other values submitted at the same time, not the
*  complete DataSource record. For most types of cross-field validation, you should fetch the
*  current saved record using the server-side API DataSource.fetchById(). For example, in
*  Velocity:
*  
*    $dataSource.fetchById($record.primaryKeyField).otherFieldName
*  
*  Note that, while a DSRequest provides dsRequest.oldValues, these values cannot be relied
*  upon for a security check since they could be faked.
*  
*  Server-side custom validators also have access to the standard set of context variables that
*  come from the Servlet API. However, be aware that if you write conditions that depend upon 
*  these variables, you preclude your Validator from being used in the widest possible variety 
*  of circumstances; for example, in a command-line process. Rather, it will be tied to 
*  operating in the context of, say, an HttpSession. 
*  
*  Given the above caveat, the following context variables are also available:
*  
*  
* - servletRequest - The associated HttpServletRequest
*  
* - session - The associated HttpSession
*  
* - httpParameters - This variable gives you access to the parameters Map
*      of the associated HttpServletRequest; it is an alternate form of 
*      $servletRequest.getParameter
*  
* - requestAttributes - This variable gives you access to the attributes Map 
*      of the associated HttpServletRequest; it is an alternate form of 
*      $servletRequest.getAttribute
*  
* - sessionAttributes - This variable gives you access to the attributes Map 
*      of the associated HttpSession; it is an alternate form of 
*      $session.getAttribute
*  
*  
*  Special considerations for Velocity
*  
*  To return a true or false value in Velocity, you script can either be just an expression
*  that returns a boolean value, or the result of evaluating the Velocity template can result
*  in output of "true" or "false". All of the following are valid forms:
*  
*  &nbsp;&nbsp;$value &lt; 100
*  &nbsp;&nbsp;$util.contains($value, "some string")
*  &nbsp;&nbsp;$record.someField(assuming that "someField" contains a boolean value)
*  &nbsp;&nbsp;$value &gt; $record.otherField
*  
*  For additional troubleshooting information when Velocity expressions aren't working as
*  expected, set the log category org.apache.Velocity to DEBUG in log4j.isc.config.xml.
*  
*  Because it's tricky to call arbitrary Java methods in Velocity, the following special
*  objects are passed to Velocity for convenience:
*  
*  
* - dataSources - The list of all DataSources, accessible by name (so, for example, 
*    $dataSources.supplyItem refers to the supplyItem DataSource
*    object).
*  
* - util - A com.isomorphic.util.DataTools object, giving you access to
*         all of that class's useful helper functions
*  
* @type {string}
* @default null
*/
serverCondition?: string;

/**
* For validators of type "serverCustom" only, a ServerObject declaration that allows
*  the SmartClient Server to find a Java class via a variety of possible approaches, and call a
*  method on it to perform validation.
*  
*  The target object must implement a method whose first 4 arguments are:
*  
*   Object value, Validator validator, String fieldName, Map record
*  
*  (com.isomorphic.datasource.Validator is a subclass of Map that 
*  represents a validator's configuration, and also provides APIs for implementing templated
*  error messages).
*  You provide the name of the method to call by specifying 
*  ServerObject.methodName
*  as part of the serverObject declaration. If you do not specify a methodName, SmartClient 
*  expects to find a compliant method called "condition".
*  
*  Additional arguments may be declared and are automatically supplied based on the declared
*  argument type, via dmiOverview. Available objects include:
*  
*  
* - DataSource - the DataSource where this validator is declared, an instance of
*              com.isomorphic.datasource.DataSource or a subclass
*  
* - HttpServletRequest - from standard Java servlets API
*  
* - HttpServletResponse - from standard Java servlets API
*  
* - ServletContext - from standard Java servlets API
*  
* - HttpSession - from standard Java servlets API
*  
* - RequestContext - an instance of com.isomorphic.servlet.RequestContext
*  
* - RPCManager - the RPCManager associated with the transaction this validation is 
*              part of; an instance of com.isomorphic.rpc.RPCManager
*  
* - DSRequest - the DSRequest this validation is part of; an instance of com.isomorphic.datasource.DSRequest
*  
* - DSField - the datasource field which value is validated; an instance of com.isomorphic.datasource.DSField
*  
* - ValidationContext - the context where value is validated; an instance of com.isomorphic.datasource.ValidationContext
*  
*  Note that any servlet-related objects will not be available if your validator is run outside
*  of the scope of an HTTP servlet request, such as a command-line process.
*  
*  Note that "record" will contain only other values submitted at the same time, not the
*  complete DataSource record. For most types of cross-field validation, you should fetch the
*  current saved record. For example:
*  
*    final Map existingRecord = dataSource.fetchById(record);
*  
* @type {ServerObject}
* @default null
*/
serverObject?: ServerObject;

/**
* To transform the incoming value that is validated into a different value or format set this
*  property from Validator.condition to the desired value.
* @type {object}
* @default null
*/
resultingValue?: object;

/**
* Text to display if the value does not pass this validation check.
*  
*  If unspecified, default error messages exist for all built-in validators, and a generic
*  message will be used for a custom validator that is not passed.
*  
*  Server-side this string evaluates in a Velocity context where the variables $value and
*  $fieldName are available and refer to the supplied value and the field name, respectively.
*  Note that if the validator is intended to run both on the client and server, you shouldn't
*  use these velocity vars as they will not be expanded on the client and the user may then see
*  raw uninterpolated strings.
* @type {string}
* @default null
*/
errorMessage?: string;

/**
* Normally, all validators defined for a field will be run 
*  validatorExecution 
*  even if one of the validators has already failed. 
*  However, if stopIfFalse is set, validation will not proceed
*  beyond this validator if the check fails.
*  
*  This is useful to prevent expensive validators from being run unnecessarily, or to allow
*  custom validators that don't need to be robust about handling every conceivable type of
*  value.
* @type {boolean}
* @default false
*/
stopIfFalse?: boolean;

/**
* Indicates that if this validator is not passed, the user should not be allowed to exit
*  the field - focus will be forced back into the field until the error is corrected.
*  
*  This property defaults to FormItem.stopOnError if unset.
*  
*  Enabling this property also implies FormItem.validateOnExit is automatically
*  enabled. If this is a server-based validator, setting this property also implies that
*  FormItem.synchronousValidation is forced on.
* @type {boolean}
* @default null
*/
stopOnError?: boolean;

/**
* Indicates this validator runs on the client only.
*  
*  Normally, if the server is trying to run validators and finds a validator that it can't
*  execute, for safety reasons validation is considered to have failed. Use this flag to
*  explicitly mark a validator that only needs to run on the client.
* @type {boolean}
* @default false
*/
clientOnly?: boolean;

/**
* If true, validator will be validated when each item's "change" handler is fired
*  as well as when the entire form is submitted or validated. If false, this validator
*  will not fire on the item's "change" handler.
*  
*  Note that this property can also be set at the form/grid or field level;
*  If true at any level and not explicitly false on the validator, the validator will be
*  fired on change - displaying errors and rejecting the change on validation failure.
* @type {boolean}
* @default null
*/
validateOnChange?: boolean;

/**
* Applies only to the "isUnique" and "hasRelatedRecord" validators and controls
*  whether the search for existing records is case sensitive or not.
* @type {boolean}
* @default false
*/
caseSensitive?: boolean;

/**
* Applies only to the "isUnique" validator; allows you to name a specific 
*  DataSource.operationBindings for the server-side uniqueness check.
*  Ignored on the client.
* @type {string}
* @default null
*/
operationId?: string;

/**
* Indicates the minimum value for range-based validators. By default, range-validators are 
*  inclusive, meaning that values that equal the min value will pass validation. 
*  To make a validator-range exclusive, and have equal values fail validation, set 
*  Validator.exclusive to true and provide a 
*  Validator.errorMessage.
* @type {any}
* @default null
*/
min?: any;

/**
* Indicates the maximum value for range-based validators. By default, range-validators are 
*  inclusive, meaning that values that equal the max value will pass validation. 
*  To make a validator-range exclusive, and have equal values fail validation, set 
*  Validator.exclusive to true and provide a 
*  Validator.errorMessage.
* @type {any}
* @default null
*/
max?: any;

/**
* When set to true, values that equal the specified Validator.min or 
*  Validator.max values will fail validation. When setting this property, consider
*  also adding a Validator.errorMessage.
* @type {boolean}
* @default null
*/
exclusive?: boolean;

/**
* Indicates this validator runs on the server only.
* @type {boolean}
* @default null
*/
serverOnly?: boolean;





}
/**
* Creates a new Validator
* 
* @param typeCheckedProperties {Partial<Validator>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Validator} The new instance
*/
class Validator{
static create(typeCheckedProperties? : Partial<Validator>, uncheckedProperties?: Object): Validator;
/**
* Default error message to display when standard isBoolean type validator
*  returns false.
* @type {string}
* @default "Must be a true/false value."
*/
static notABoolean?: string;

/**
* Default error message to display when standard isString type validator
*  returns false.
* @type {string}
* @default "Must be a string."
*/
static notAString?: string;

/**
* Default error message to display when standard isInteger type validator
*  returns false.
* @type {string}
* @default "Must be a whole number."
*/
static notAnInteger?: string;

/**
* Default error message to display when standard isFloat type validator
*  returns false.
* @type {string}
* @default "Must be a valid decimal."
*/
static notADecimal?: string;

/**
* Default error message to display when standard isDate type validator
*  returns false.
* @type {string}
* @default "Must be a date."
*/
static notADate?: string;

/**
* Default error message to display when standard integerRange type validator
*  returns false because the value passed in is greater than the specified maximum.
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with max and min available as
*  variables mapped to validator.max and validator.min.
* @type {string}
* @default "Must be no more than ${max}"
*/
static mustBeLessThan?: string;

/**
* Default error message to display when standard integerRange type validator
*  returns false because the value passed in is less than the specified minimum.
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with max and min available as
*  variables mapped to validator.max and validator.min.
* @type {string}
* @default "Must be at least ${min}"
*/
static mustBeGreaterThan?: string;

/**
* Default error message to display when standard dateRange type validator
*  returns false because the value passed in is greater than the specified maximum date.
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with max and min available as
*  variables mapped to validator.max and validator.min.
* @type {string}
* @default "Must be ${min.toShortDate()} or later"
*/
static mustBeLaterThan?: string;

/**
* Default error message to display when standard timeRange type validator
*  returns false because the time-portion of the date-value passed in is less than the 
*  specified minimum time.
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with max and min available as
*  variables mapped to validator.max and validator.min.
* @type {string}
* @default "Must be ${isc.Time.toShortTime(min)} or later"
*/
static mustBeLaterThanTime?: string;

/**
* Default error message to display when standard dateRange type validator
*  returns false because the value passed in is less than the specified maximum date.
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with max and min available as
*  variables mapped to validator.max and validator.min.
* @type {string}
* @default "Must be ${max.toShortDate()} or earlier"
*/
static mustBeEarlierThan?: string;

/**
* Default error message to display when standard timeRange type validator
*  returns false because the time-portion of the date-value passed in is greater than the 
*  specified minimum time.
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with max and min available as
*  variables mapped to validator.max and validator.min.
* @type {string}
* @default "Must be ${isc.Time.toShortTime(max)} or earlier"
*/
static mustBeEarlierThanTime?: string;

/**
* Default error message to display when standard lengthRange type validator
*  returns false because the value passed in has more than validator.max characters.
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with max and min available as
*  variables mapped to validator.max and validator.min.
* @type {string}
* @default "Must be no more than ${max} characters"
*/
static mustBeShorterThan?: string;

/**
* Default error message to display when standard lengthRange type validator
*  returns false because the value passed in has fewer than validator.min characters.
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with max and min available as
*  variables mapped to validator.max and validator.min.
* @type {string}
* @default "Must be at least ${min} characters"
*/
static mustBeLongerThan?: string;

/**
* Default error message to display when standard lengthRange type validator
*  has validator.max and validator.min set to the same value,
*  and returns false because the value passed is not the same length as these limits.
*  This is a dynamic string - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with max and min available as
*  variables mapped to validator.max and validator.min.
* @type {string}
* @default "Must be exactly ${max} characters"
*/
static mustBeExactLength?: string;

/**
* Default error message to display when validation fails for a field marked as required
*  or with a standard required type validator.
*  The message is also displayed for a field with a standard requiredIf type
*  validator whose condition evaluates to true, because the field has no value.
* @type {string}
* @default "Field is required"
*/
static requiredField?: string;

/**
* Default error message to display when standard isOneOf type validator
*  is not passed.
* @type {string}
* @default "Not a valid option"
*/
static notOneOf?: string;

/**
* Default error message to display when the standard maxFileSize type validator
*  returns false.
* @type {HTMLString}
* @default "Size of '${uploadedFileName}' (${isc.NumberUtil.toMiBString(uploadedFileSize, maxFileSize)} MiB) exceeds maximum allowed file size of ${isc.NumberUtil.toMiBString(maxFileSize, uploadedFileSize)} MiB."
*/
static maxFileSizeExceeded?: HTMLString;

/**
* A declared value of the enum type 
*  FieldAppearance.
* @type {Constant}
* @default "readOnly"
*/
static READONLY?: Constant;

/**
* A declared value of the enum type 
*  FieldAppearance.
* @type {Constant}
* @default "hidden"
*/
static HIDDEN?: Constant;

/**
* A declared value of the enum type 
*  FieldAppearance.
* @type {Constant}
* @default "disabled"
*/
static DISABLED?: Constant;





/**
*  Add a new validator type that can be specified as Validator.type anywhere
*  validators are declared, such as DataSourceField.validators or
*  FormItem.validators.
*  
*  The condition argument should be a method of the same signature as
*  Validator.condition.
*  
*  This method is essentially a shortcut for building a ValidatorDefinition object
*  and passing that to Validator.addValidatorDefinition
* 
* 
* @param {string} type name for the new validator
* @param {ValidatorConditionCallback} callback, function or expression to                  evaluate to determine whether validation was successful
*/
static addValidator(type:string, condition:(item:DataSourceField | ListGridField | FormItem, validator:Validator, value:any, record:object, additionalContext:object) => void): void;

/**
*  Add several new validator types at once, as though Validator.addValidator were called
*  several times.
*  
* 
* @param {object} Set of validators to add. This parameter should   be a JavaScript object where the property names are validator type names, and the    property values are condition functions or expressions, for example:    &nbsp;&nbsp;&nbsp;{type1:condition1, type2:condition2}.
*/
static addValidators(newValidators:object): void;

/**
*  Add a new validator type that can be specified as Validator.type anywhere
*  validators are declared, such as DataSourceField.validators or
*  FormItem.validators.
* 
* 
* @param {string} type name for the new validator
* @param {ValidatorDefinition} the validator definition
*/
static addValidatorDefinition(type:string, definition:ValidatorDefinition): void;

/**
*  Add several new validator types at once, as though Validator.addValidatorDefinition
*  were called several times.
*  
* 
* @param {object} Set of validators to add. This parameter should   be a JavaScript object where the property names are validator type names, and the    property values are ValidatorDefinitions.
*/
static addValidatorDefinitions(newDefinitions:object): void;




}

/**
* The ValuesManager manages data from multiple member forms.
*  
*  If a single logical form needs to be separated into multiple DynamicForm instances for
*  Layout purposes (for example, spanning one logical form across multiple Tabs), a
*  ValuesManager can be used to make the forms act as one logical form, supporting all
*  value-related APIs otherwise called on DynamicForm directly.
*  
*  A ValuesManager has no visual representation - it is strictly a logical entity, and the
*  member forms provide the user interface. You can initialize a ValuesManager with a set of
*  member forms (by setting ValuesManager.members at init) or add and remove member
*  forms dynamically.
*  
*  Calling ValuesManager.setValues on a ValuesManager will automatically route new
*  field values to whichever member form is showing an editor for that field. Likewise,
*  calling ValuesManager.validate will validate all member forms, and
*  ValuesManager.saveData will initiate a save operation which aggregates values from
*  all member forms.
*  
*  Like a DynamicForm, a ValuesManager can be databound by setting
*  ValuesManager.dataSource. In this case all member forms must also be bound to the
*  same DataSource.
*  
*  In general, when working with a ValuesManager and its member forms, call APIs on the
*  ValuesManager whenever you are dealing with values that span multiple forms, and only call
*  APIs on member forms that are specific to that form or its fields.
*  
*  Note that, just as a DynamicForm can track values that are not shown in any FormItem, a
*  ValuesManager may track values for which there is no FormItem in any member form. However,
*  when using a ValuesManager these extra values are only allowed on the ValuesManager itself.
*  Member forms will not track values for which they do not have FormItems.
*/
interface ValuesManager extends Class {
/**
* Default DSOperationType to be performed when DynamicForm.saveData is called.
*  This property is automatically set on a call to DynamicForm.editRecord or
*  DynamicForm.editNewRecord, or may be set directly via 
*  DynamicForm.setSaveOperationType.
*  
*  If saveOperationType is unset, the form will heuristically determine
*  whether an "add" or "update" operation is intended based on whether the primaryKey field
*  is present and editable.
* @type {DSOperationType}
* @default null
*/
saveOperationType?: DSOperationType;

/**
* When calling ValuesManager.saveData on a form or valuesManager, by default if the server
*  returns an error code, any callback passed into saveData() will not be fired.
*  If the error code returned by the server indicates a validation error, it will be
*  displayed to the user by updating the form items to show the error messages, and firing
*  any specified hiddenValidationErrors handler, otherwise the standard RPCManager
*  error handling logic would be invoked.
*  
*  Developers who want to handle errors themselves can override this default by specifying
*  RPCRequest.willHandleError on the DSRequest. In this case the callback passed in 
*  will be fired even if the server returns an error status code.
*  
*  If suppressValidationErrorCallback is set to true, if a save attempt returns
*  a validation error code, the user-specified callback will not be fired
*  even if willHandleError:true was specified on the dsRequest 
*  - though for other error codes, the callback would be fired if willHandleError is 
*  specified on the request.
*  Note that this is the historical behavior for
*  SmartClient builds 8.0 and earlier
*  
* @type {boolean}
* @default false
*/
suppressValidationErrorCallback?: boolean;

/**
* Specifies a dataSource for this valuesManager. This dataSource will then be used for
*  validation and client-server flow methods. Can be specified as a dataSource object or
*  an identifier for the dataSource.
*  Note that member forms should have the same dataSource applied to them to allow their
*  items to inherit properties from the DataSource fields.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* DSRequest.operationId to use when performing add operations.
* @type {string}
* @default null
*/
addOperation?: string;

/**
* DSRequest.operationId to use when performing update operations.
* @type {string}
* @default null
*/
updateOperation?: string;

/**
* DSRequest.operationId to use when performing remove operations.
* @type {string}
* @default null
*/
removeOperation?: string;

/**
* DSRequest.operationId to use when performing fetch operations.
* @type {string}
* @default null
*/
fetchOperation?: string;

/**
* The set of member components for this valuesManager. These can be specified at init time
*  via the members property, or updated at runtime via addMember()
*  and removeMember().
*  Note: Alternatively a DataBoundComponent can be initialized as a member of a valuesManager
*  by setting the valuesManager property of the component to a pre-defined 
*  valuesManager instance, or by calling setValuesManager on the component.
* @type {Array<Partial<DynamicForm>>}
* @default null
*/
members?: Array<Partial<DynamicForm>>;

/**
* If set to true, client-side validators will not run on the form when validate() is
*  called. Server-side validators (if any) will still run on attempted save.
* @type {boolean}
* @default null
*/
disableValidation?: boolean;

/**
* If explicitly set to false, prevents the ValuesManager from automatically propagating
*  data value changes to its members. You can manually synchronize member data values 
*  at any time with a call to ValuesManager.synchronizeMembers.
* @type {boolean}
* @default null
*/
autoSynchronize?: boolean;

/**
* Before we start editing the values of this ValuesManager in one or more DataBoundComponents, 
*  should we perform a deep clone of the underlying values. See 
*  DataSource.deepCloneOnEdit for details of what this means.
*  
*  If this value is not explicitly set, it defaults to the value of DataSource.deepCloneOnEdit.
*  This value can be overridden per-field with DataSourceField.deepCloneOnEdit.
*  
*  Like the other deepCloneOnEdit settings, this flag only has an effect if you are 
*  editing a values object that contains nested objects or arrays, using 
*  Canvas.dataPaths.
* @type {boolean}
* @default null
*/
deepCloneOnEdit?: boolean;

/**
* When editing values in this ValuesManager in one or more DataBoundComponents, should we
*  perform a deep clone of values that are not associated with a field (ie, attributes on
*  the record that do not map to a component field either directly by name, or by
*  FormItem.dataPath. If this value is not explicitly set, it defaults to the value of 
*  DataSource.deepCloneNonFieldValuesOnEdit if there is a dataSource, or to the value 
*  of the static DataSource.deepCloneNonFieldValuesOnEdit if there is no
*  dataSource.
*  
*  Like the other deepCloneOnEdit settings, this flag only has an effect if you are 
*  editing a values object that contains nested objects or arrays.
* @type {boolean}
* @default null
*/
deepCloneNonFieldValuesOnEdit?: boolean;

/**
* What operator should be used to combine sub-criteria from member forms when
*  ValuesManager.getValuesAsCriteria is called?
* @type {OperatorId}
* @default "and"
*/
operator?: OperatorId;



/**
* 
*  Prepare to edit a new record by clearing the current set of values (or replacing them 
*  with initialValues if specified).
*  
*  This method will also call DynamicForm.setSaveOperationType to ensure
*  subsequent calls to saveData() will use an add rather than
*  an update operation.
* 
* 
* @param {Record | object=} initial set of values for the editor as a map of field names to their corresponding   values
*/
editNewRecord(initialValues?:Record | object): void;

/**
* 
*  Edit an existing record. Updates this editors values to match the values of the record 
*  passed in, via ValuesManager.setValues.
*  
*  This method will also call DynamicForm.setSaveOperationType to ensure 
*  subsequent calls to saveData() will use an update rather than
*  an add operation.
* 
* 
* @param {Record} the record to be edited as a map of field names to their corresponding values
*/
editRecord(record:Record): void;

/**
*  Setter for the default DSOperationType when DynamicForm.saveData is called.
*  Note that this property can also be set by calling DynamicForm.editRecord or 
*  DynamicForm.editNewRecord
* 
* 
* @param {DSOperationType} Operation type to use as a default. Valid values are "add" or "update".
*/
setSaveOperationType(operationType:DSOperationType): void;

/**
*  Returns true if ValuesManager.saveOperationType is currently "add". See
*  ValuesManager.saveOperationType.
* 
* 
*/
isNewRecord(): boolean;

/**
*  Returns the DSOperationType to be performed when DynamicForm.saveData
*  or ValuesManager.saveData is called.
*  Valid options are "add" or "update".
*  
*  If a DSRequest configuration object is passed in containing an explicit operationType
*  this will be returned. Otherwise DynamicForm.saveOperationType 
*  will be returned if set. Note that
*  saveOperationType is automatically set via calls to data binding methods such as
*  DynamicForm.editNewRecord, or it may be 
*  DynamicForm.setSaveOperationType.
*  
*  If no explicit saveOperationType is present, the system will use the following 
*  heuristic to determine the save operationType: 
*  
*  
* - If the form has no value for the DataSource.getPrimaryKeyField
*    this method will return "add". The assumption is that this is a new record, and the
*    field will be populated when the record is created, 
*    (as with a "sequence" type field).
*  
* - If, ${isc.DocUtils.linkForRef('method:DynamicForm.setValues','when the form\'s values were populated')}, 
*    the form had value for the DataSource.getPrimaryKeyField
*    but it has subsequently be changed, this method will return "add". In this case
*    the value has been changed, either by the user or programmatically so a different
*    (new) record is assumed. This is determined by looking at the
*    DynamicForm.getOldValues for the form.
*  
* - If the DataSource.getPrimaryKeyField is editable and
*    a value is now present for the primary key field, but was not present in the
*    DynamicForm.getOldValues for the form, this method will return
*    "add". In this case either no initial values were provided, or a 'sparse' 
*    set of values for a new record (with no primary key) were provided to the form
*    and the user has subsequently explicitly entered a new primaryKey field value.
*  
* - Otherwise this method will return "update". Either the primaryKey field is non
*    editable, or the user has not changed it from its initial value.
*  
* 
* 
* @param {Partial<DSRequest>=} Optional DSRequest config block for the save operation
*/
getSaveOperationType(requestProperties?:Partial<DSRequest>): DSOperationType;

/**
* 
*  Edit the record selected in the specified selection component (typically a
*  ListGrid).
*  
*  Updates the values of this editor to match the selected record's values.
*  
*  If this form has a dataSource, then saving via ValuesManager.saveData will use the 
*  "update" operation type.
* 
* 
* @param {ListGrid | TileGrid | GlobalId} the ListGrid or TileGrid or ID of a ListGrid/TileGrid whose currently   record(s) is/are to be edited
*/
editSelectedData(selectionComponent:ListGrid | TileGrid | GlobalId): void;

/**
* 
*  Validate and then save the form's current values to the DataSource this form is
*  bound to.
*  
*  If client-side validators are defined, they are executed first, and if any errors are
*  found the save is aborted and the form will show the errors.
*  
*  If client-side validation passes, a DSRequest will be sent, exactly as though
*  DataSource.addData or DataSource.updateData had been called with 
*  ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','the form\'s values')} as data. The
*  DSRequest.operationType will be either "update" or "add", depending on the
*  current DynamicForm.saveOperationType.
*  
*  On either a client-side or server-side validation failure, validation errors will be
*  displayed in the form. Visible items within a DynamicForm will be redrawn to display
*  errors. Validation failure occurring on hidden items, or DataSource fields with no 
*  associated form items may be handled via DynamicForm.handleHiddenValidationErrors
*  or ValuesManager.handleHiddenValidationErrors.
*  
*  In the case of a validation error, the callback will not be called by default
*  since the form has already handled the failed save by displaying the validation errors
*  to the user. If you need to do something additional in this case, you can set
*  RPCRequest.willHandleError via the requestProperties parameter to
*  force your callback to be invoked. However, first consider:
*  
*  
* - if you are trying to customize display of validation errors, there are several
*  DynamicForm.showErrorIcons and DynamicForm.showErrors may be a better
*  place to put customizations.
*  
* - for unrecoverable general errors (eg server is down),
*  RPCManager.handleError in invoked, so consider placing
*  customizations there unless an unrecoverable error should be handled specially by this
*  specific form.
*  
*  Note: If a form is to be cleared after saving data, we recommend clearing the
*  form from the callback rather than calling saveData() and then synchronously clearing
*  the form. This gives users a chance to view and respond to any validation errors
*  returned by the server. It is also required to ensure forms containing an
*  FileItem, have a chance to upload the file to the server.
* 
* 
* @param {DSCallback=} callback to invoke on completion
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
*/
saveData(callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  submit() is automatically called when a SubmitItem in a member form
*  is clicked, or if DynamicForm.saveOnEnter is set for some member form,
*  when the
*  "Enter" key is pressed in a text input. Submit can also be manually called.
*  
*  If ValuesManager.submitValues exists, it
*  will be called, and no further action will be taken.
*  
*  Otherwise, ValuesManager.saveData will be called to
*  handle saving via SmartClient databinding. 
*  
*  The parameters to submit() apply only if submit() will be
*  calling ValuesManager.saveData. If you override submit(), you can safely
*  ignore the parameters as SmartClient framework code does not pass them.
* 
* 
* @param {DSCallback=} callback to invoke on completion.
* @param {DSRequest=} additional properties to set on the DSRequest                     that will be issued
*/
submit(callback?:DSCallback, requestProperties?:DSRequest): void;

/**
* 
*  This method exists for clean integration with existing server frameworks that have a
*  'cancel' feature which typically clears session state associated with the form. When
*  this method is called, an RPC is sent to the server. You must pass the appropriate
*  params property in the request properties as required by your server framework.
*  
*  For example:
*  
*  var requestProperties = {
*    params: {CANCEL: "cancel"}
*  };
*  
* 
*  Note that no other form data is sent. By default the current top-level page is replaced
*  with the reply. If you wish to ignore the server reply instead, include the following
*  request properties:
*  
*  var requestProperties = { 
*    // other props ...
*    ignoreTimeout: true,
*    target: null
*  };
*  
* 
* @param {DSRequest} additional properties to set on the RPCRequest                    that will be issued
* @deprecated * As of Smartclient version 13.0, due to the removal of Apache Struts.

*/
cancel(requestProperties:DSRequest): void;

/**
*  Retrieve data that matches the provided criteria, and edit the first record returned.
*  Differs from DynamicForm.fetchData in that a case insensitive substring match
*  will be performed against the criteria to retrieve the data.
*  
* 
* @param {Criteria=} search criteria
* @param {DSCallback=} callback to invoke on completion
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
*/
filterData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Retrieve data that matches the provided criteria, and edit the first record returned
*  
* 
* @param {Criteria=} search criteria
* @param {DSCallback=} callback to invoke on completion
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
*/
fetchData(criteria?:Criteria, callback?:DSCallback, requestProperties?:DSRequest): void;

/**
*  Update all of this ValuesManager's members to reflect the current values held by the
*  ValuesManager. It is not normally necessary to manually synchronize members, but you
*  will need to do so if you switch off ValuesManager.autoSynchronize.
* 
*/
synchronizeMembers(): void;

/**
*  Update just those of this ValuesManager's members that have the parameter 
*  Canvas.dataPath, to reflect the current values held by the
*  ValuesManager. Note, it is not normally necessary to manually synchronize members
* 
* @param {string} dataPath to synchronize
*/
synchronizeMembersOnDataPath(dataPath:string): void;

/**
*  Update the parameter ValuesManager member to reflect the current values held by the
*  ValuesManager. Note, it is not normally necessary to manually synchronize members
* 
* @param {Canvas} Member component to synchronize
*/
synchronizeMember(member:Canvas): void;

/**
*  Fires after a member component's values have been synchronized from the ValuesManager's
*  values, upon completion of the ValuesManager.synchronizeMember call. 
*  No default implementation.
* 
* @param {Canvas} Member component that has just completed synchronization
*/
memberSynchronized(member:Canvas): void;

/**
*  Specifies a dataSource for this valuesManager. This dataSource will then be used for
*  validation and client-server flow methods.
* 
* @param {DataSource | GlobalId} Datasource object or identifier to bind to.
*/
setDataSource(dataSource:DataSource | GlobalId): void;

/**
*  Returns the dataSource for this valuesManager. Will return null if this is not a 
*  data-bound valuesManager instance.
* 
*/
getDataSource(): DataSource;

/**
*  Retrieve a FormItem from this ValuesManager.
*  
*  Takes a field FormItem.name or DataPath, and searches through the
*  members of this valuesManager for an editor for that field. If found the appropriate
*  formItem will be returned. If the "retrieveAll" parameter is true, this method will return 
*  all FormItems that are bound to the supplied name or dataPath (a dataPath can be bound
*  to more than one FormItem, as long as those FormItems are on different forms); if 
*  "retrieveAll" is false or unset, and there is more than one binding for the dataPath, 
*  this method just returns the first one it finds.
*  Note that if a dataPath is passed in, it should be the full data path for the item, 
*  including any canvas level Canvas.dataPath specified on the member 
*  form containing this form item.
*  Note: Unlike the DynamicForm class, this method will not return an 
*  item by index
* 
* @param {FieldName | DataPath} item fieldName or dataPath identifier
* @param {boolean=} If true, return the list of all FormItems that                   are bound to this name or dataPath on a member                    form of this ValuesManager
*/
getItem(itemID:FieldName | DataPath, retrieveAll?:boolean): FormItem;

/**
*  Retrieves an array of pointers to all the members for this valuesManager.
* 
*/
getMembers(): Array<Partial<DynamicForm>>;

/**
*  Returns a pointer to a specific member.
* 
* @param {string} ID of the member component to retrieve
*/
getMember(ID:string): Canvas;

/**
*  Given a fieldName or dataPath, this method will find the member responsible for
*  interacting with that field's value.
*  If no form is found, returns null.
* 
* @param {string} fieldName or dataPath to check
*/
getMemberForField(fieldName:string): Canvas;

/**
*  Validate the current set of values for this values manager against validators defined
*  in the member forms. For databound valuesManagers, also perform validation against any
*  validators defined on datasource fields.
*  
*  Note that if validation errors occur for a value that is not shown in any member forms,
*  those errors will cause a warning and ValuesManager.handleHiddenValidationErrors will be
*  called. This can occur if:
*  - A datasource field has no corresponding item in any member form
*  - The item in question is hidden
*  - The member form containing the item is hidden.
*  
*  If this form has any fields which require server-side validation (see 
*  Validator.serverCondition) this will also be initialized. Such validation will
*  occur asynchronously. Developers can use ValuesManager.isPendingAsyncValidation
*  and ValuesManager.handleAsyncValidationReply to detect and respond to
*  asynchronous validation.
*  
*  Note that for silent validation, ValuesManager.valuesAreValid (client-side) and 
*  ValuesManager.checkForValidationErrors (client and server-side) can be used instead.
* 
* 
*/
validate(): boolean;

/**
*  Method to determine whether the current set of values for this values manager would pass
*  validation by the validators defined in the member forms. This method operates
*  client-side, without contacting the server, running validators on the forms' values and
*  returning a value indicating whether validation was successful.
*  
*  Note that, like ValuesManager.validate, this method will perform value validation even if:
*  
* - A datasource field has no corresponding item in any member form
*  
* - The item in question is hidden
*  
* - The member form containing the item is hidden
*  
*  Unlike ValuesManager.validate this method will not store the errors on the forms
*  or display them to the user.
* 
* @param {boolean=} If unset, this method returns a simple boolean value indicating success or failure of validation. If this parameter is passed, this method will return an object mapping each field name to the errors(s) encountered on validation failure, or null if validation was successful.
*/
valuesAreValid(returnErrors?:boolean): boolean | Map<any>;

/**
*  Performs silent validation of the value manager values, like ValuesManager.valuesAreValid. In
*  contrast to ValuesManager.valuesAreValid, this method allows checking for server-side errors, and
*  finding out what the errors are. 
*  
*  The callback must be passed unless server-side validation is being skipped, and If passed,
*  it always fires, errors or not, firing synchronously if server validation is skipped.
* 
* 
* @param {ValidationStatusCallback} callback to invoke after validation is complete
* @param {boolean=} whether to skip doing server-side validation
*/
checkForValidationErrors(callback:(errorMap:Map<any>) => void, skipServerValidation?:boolean): Map<any>;

/**
*  Call ValuesManager.validate to check for validation errors. If no errors are found,
*  return the current values for this valuesManager, otherwise return null.
* 
*/
getValidatedValues(): object;

/**
*  Sets validation errors for this valuesManager to the specified set of errors.
*  Errors should be of the format:
*  {field1:errors, field2:errors}
*  where each errors object is either a single error message string or an
*  array of error messages.
*  If showErrors is passed in, error messages will be displayed in the 
*  appropriate member form items. For fields with no visible form item, 
*  ValuesManager.handleHiddenValidationErrors will be fired instead.
*  Note that if showErrors is false, errors may be shown at any time via
*  a call to ValuesManager.showErrors.
* 
* 
* @param {object} list of errors as an object with the field names as keys
* @param {boolean} If true display errors now.
*/
setErrors(errors:object, showErrors:boolean): void;

/**
*  Adds validation errors to the existing set of errors for the field in question.
*  Errors passed in should be a string (for a single error message) or an array of strings.
*  Pass in the showErrors parameter to immediately display the errors to the user by 
*  redrawing the appropriate member form item (or if no visible item is found for the field
*  firing ValuesManager.handleHiddenValidationErrors.
* 
* @param {string} name of field to apply errors to
* @param {string | Array<Partial<string>>} error messages for the field
* @param {boolean} should the error(s) be immediately displayed to the user?
*/
addFieldErrors(fieldName:string, errors:string | Array<Partial<string>>, showErrors:boolean): void;

/**
*  Sets validation errors for some field in the valuesManager.
*  Errors passed in should be a string (for a single error message) or an array of strings.
*  Pass in the showErrors parameter to immediately display the errors to the user by 
*  redrawing the appropriate member form item (or if no visible item is found for the field
*  firing ValuesManager.handleHiddenValidationErrors.
* 
* @param {string} name of field to apply errors to
* @param {string | Array<Partial<string>>} error messages for the field
* @param {boolean} should the error(s) be immediately displayed to the user?
*/
setFieldErrors(fieldName:string, errors:string | Array<Partial<string>>, showErrors:boolean): void;

/**
*  	Clears all errors from member forms.
* 
* @param {boolean} If true, clear any visible error messages.
*/
clearErrors(showErrors:boolean): void;

/**
*  Clear all validation errors associated with some field in this form
* 
* @param {string} field for which errors should be cleared
* @param {boolean} if true, and the field is present in one of our member forms,            redraw it to clear any currently visible validation errors
*/
clearFieldErrors(fieldName:string, show:boolean): void;

/**
*  Returns the set of errors for this valuesManager.
*  Errors will be returned as an object of the format 
*  {field1:errors, field2:errors}
*  Where each errors object is either a single error message or an array of error message
*  strings.
* 
*/
getErrors(): object;

/**
*  Returns any validation errors for some field in this valuesManager.
*  Errors will be returned as either a string (a single error message), or an array 
*  of strings. If no errors are present, will return null.
* 
* @param {string} fieldName to check for errors
*/
getFieldErrors(fieldName:string): string | Array<Partial<string>>;

/**
*  Are there any errors associated with any fields in this valuesManager?
* 
*/
hasErrors(): boolean;

/**
*  Are there any errors associated with a field in this valuesManager?
* 
* @param {string} field to check for errors
*/
hasFieldErrors(fieldName:string): boolean;

/**
*  Method to explicitly show the latest set of validation errors present on this 
*  ValuesManager.
*  Will redraw all member forms to display (or clear) currently visible errors, and
*  fire ValuesManager.handleHiddenValidationErrors to allow custom handling of
*  hidden errors.
* 
*/
showErrors(): void;

/**
*  Method to explicitly show the latest set of validation errors present on some field 
*  within this ValuesManager.
*  If the field in question is present as a visible item in a member form, the form item
*  will be redrawn to display the error message(s).
*  Otherwise ValuesManager.handleHiddenValidationErrors will be fired to allow 
*  custom handling of hidden errors.
* 
*/
showFieldErrors(): void;

/**
*  Notification fired when an asynchronous validation completes.
* 
* @param {boolean} true if validation succeeded, false if it failed
* @param {object} Map of errors by fieldName. Will be null if validation succeeded.
*/
handleAsyncValidationReply(success:boolean, errors:object): void;

/**
*  Is this ValuesManager waiting for an asynchronous validation to complete?
*  This method will return true after ValuesManager.validate is called on a
*  component with server-side validators for some field(s), until the server responds.
*  
*  Note that the notification method ValuesManager.handleAsyncValidationReply will
*  be fired when validation completes.
* 
*/
isPendingAsyncValidation(): boolean;

/**
*  Returns the current set of values for the values manager instance. This includes the
*  values from any form managed by this manager, as well as any values explicitly applied
*  via ValuesManager.setValues.
*  
*  Note that modifying the returned object is not a supported way of adding or modifying
*  values. Instead use ValuesManager.setValue or ValuesManager.setValues.
* 
*/
getValues(): object;

/**
*  Replaces the current values of the ValuesManager and all member components with the 
*  values passed in.
*  
*  Values should be provided as an Object containing the new values as properties, where
*  each propertyName is the name of a Items in one of the member forms,
*  and each property value is the value to apply to that form item via
*  FormItem.setValue.
*  
*  Values with no corresponding form item may also be passed, will be tracked by the
*  valuesManager and returned by subsequent calls to ValuesManager.getValues.
*  
*  Any FormItem for which a value is not provided will revert to its
*  FormItem.defaultValue. To cause all FormItems to revert to default
*  values, pass null.
*  
*  This method also calls ValuesManager.rememberValues so that a subsequent later call to
*  ValuesManager.resetValues will revert to the passed values.
*  
* 
* @param {object} new set of values for this values manager.
*/
setValues(values:object): void;

/**
*  Clear out all the values managed by this values manager.
* 
*/
clearValues(): void;

/**
*  Returns the subset of this valuesManager's values associated with some member form.
*  
* 
* @param {string} ID of the member form for which we want to retrieve the values.
*/
getMemberValues(ID:string): object;

/**
*  Set the values for some member form.
* 
* @param {string} ID of the member form to update
* @param {object} new values for the form
*/
setMemberValues(ID:string, values:object): void;

/**
*  Retrieves the combined DynamicForm.getValuesAsCriteria 
*  for all member forms.
*  
*  As with the DynamicForm getValuesAsCriteria, this method may return
*  AdvancedCriteria or simple Criteria depending on whether
*  the advanced parameter was passed, whether the ValuesManager.operator
*  is set to "or" rather than "and", and whether any member
*  forms return AdvancedCriteria.
*  
*  Note that developers can also use DataSource.combineCriteria to combine
*  sub-criteria from various sources, including member forms of a ValuesManager, into
*  a combined criteria object.
* 
* 
* @param {boolean} if true, return an AdvancedCriteria object even if the  form item values could be represented in a simple Criterion object.
* @param {TextMatchStyle=} This parameter may be passed to indicate whether  the criteria are to be applied to a substring match (filter) or exact match (fetch).  When advanced criteria are returned this parameter will cause the appropriate  operator to be generated for individual fields' criterion clauses.
*/
getValuesAsCriteria(advanced:boolean, textMatchStyle?:TextMatchStyle): Criteria | AdvancedCriteria;

/**
*  Return an AdvancedCriteria object based on the current set of values within memberForms.
*  
*  Similar to ValuesManager.getValuesAsCriteria, except the returned criteria object
*  is guaranteed to be an AdvancedCriteria object, even if none of the form's fields has a
*  specified FormItem.operator
* 
* 
* @param {TextMatchStyle=} If specified the text match style will be used to  generate the appropriate operator for per field criteria.
*/
getValuesAsAdvancedCriteria(textMatchStyle?:TextMatchStyle): AdvancedCriteria;

/**
*  	Make a snapshot of the current set of values, so we can reset to them later.
*  	Creates a new object, then adds all non-method properties of values
*  	to the new object. Use resetValues() to revert to these values.
*      Note that this method is automatically called when the values for this form are
*      set programmatically via a call to DynamicForm.setValues.
* 
* 
*/
rememberValues(): object;

/**
*  Returns the set of values last stored by DynamicForm.rememberValues.
*  Note that rememberValues() is called automatically by
*  DynamicForm.setValues, and on form initialization, so this typically contains
*  all values as they were before the user edited them.
* 
* 
*/
getOldValues(): object;

/**
*  Returns all values within this DynamicForm that have changed since 
*  DynamicForm.rememberValues last ran. Note that DynamicForm.rememberValues
*  runs on dynamicForm initialization, and with every call to DynamicForm.setValues
*  so this will typically contain all values the user has explicitly edited since then.
* 
*/
getChangedValues(): object;

/**
* 
*  Same as DynamicForm.reset.
* 
* 
*/
resetValues(): void;

/**
*  Compares the current set of values with the values stored by the call to the
*  DynamicForm.rememberValues method. rememberValues() runs when the
*  form is initialized and on every call to DynamicForm.setValues.
*  Returns true if the values have changed, and false otherwise.
* 
*/
valuesHaveChanged(): boolean;

/**
*  Returns the value for some field.
* 
* @param {string} Which value to be returned
* @param {Canvas=} Optional, the component for which we are trying to                 retrieve a value. This is used to identify which of                  the potential records to use when the ValuesManager                  is managing a complex structure involving nested Lists
*/
getValue(fieldName:string, component?:Canvas): any;

/**
*  Set the value for some field.
* 
* @param {string} Which field to set the value for
* @param {any} New value for the field.
*/
setValue(fieldName:string, newValue:any): void;

/**
*  Clear the value for some field.
* 
* @param {string} Which field to set the value for
*/
clearValue(fieldName:string): void;

/**
* 
*  Add a new member to this valuesManager. Any Canvas can be a member of a
*  valuesManager, even components like Layout or TabSet that do
*  not actually have any values to manage. When "valueless" components like these bind to
*  a ValuesManager, it is in order to provide their own child components with a shared
*  valuesManager so that complex data can be displayed and edited - see 
*  DataPath for more details.
*  
*  For components like DynamicForm and ListGrid, which do have 
*  a set of values to manage, the component's values will subsequently be available through
*  this valuesManager.
*  
*  Note on pre-existent values when the member component is a DynamicForm:
*  If the valuesManager has a value specified for some field, for which the member form has
*  an item, this value will be applied to the member form. This is true whether the item
*  has a value or not.
*  However if the member form has a value for some field, and the ValuesManager does not
*  have a specified value for the same field, we allow the valuesManager to pick up the 
*  value from the member form.
*  
*  Caution: If a DynamicForm without a DataSource is passed to this method,
*  DataBoundComponent.setDataSource will be called on that form, recreating the
*  items from copies of the item configuration stored at the time the form was created.
*  This means that any properties or handlers added to the items after form creation
*  will be lost. When in doubt, set the DataSource in the form as soon as possible.
* 
* 
* @param {DynamicForm | string} component (or ID of component) to add to                      this valuesManager as a member.
*/
addMember(member:DynamicForm | string): void;

/**
*  Add multiple new member forms to this valuesManager.
* 
* @param {Array<Partial<DynamicForm>>} array of forms to add to this valuesManager as members.
*/
addMembers(members:Array<Partial<DynamicForm>>): void;

/**
*  Remove a member form from this valuesManager, so its values are no longer managed
*  by this instance.
*  This does not clear the values associated with the form from the valuesManager - they
*  will still be available via valuesManager.getValues(), but will not be
*  updated as the form is manipulated.
* 
* @param {DynamicForm | string} form (or ID of form) to remove from this valuesManager
*/
removeMember(member:DynamicForm | string): void;

/**
*  Remove multiple member forms from this valuesManager.
* 
* @param {Array<Partial<DynamicForm>>} array of forms to remove
*/
removeMembers(members:Array<Partial<DynamicForm>>): void;

/**
*  Method to display validation error messages for a valuesManager when there is not
*  currently visible form item displaying the errors.
*  This will be called when validation fails for
*  - a field in a hidden or undrawn member form
*  - a hidden field in a visible member form
*  - for databound ValuesManagers, a datasource field with specified validators, but not
*   associated item in any member.
*  Implement this to provide custom validation error handling for these fields.
*  By default hidden validation errors will be logged as warnings in the developerConsole.
*  Return false from this method to suppress that behavior.
* 
* @param {object} The set of errors returned - this is an object of the form           &nbsp;&nbsp;{fieldName:errors}            Where the 'errors' object is either a single string or an array            of strings containing the error messages for the field.
*/
handleHiddenValidationErrors(errors:object): boolean;

/**
*  Optional stringMethods to fire when ValuesManager.submit is called
*  on this valuesManager (or any form included in this valuesManager).
*  
* 
* @param {object} the valuesManager values
* @param {ValuesManager} the valuesManager being submitted
*/
submitValues(values:object, valuesManager:ValuesManager): void;

/**
*  Handler fired whenever a change to a FormItem fires itemChanged() on one of the 
*  member forms. Fires after that event.
* 
* 
* @param {FormItem} the FormItem where the change event occurred
* @param {any} new value for the FormItem
*/
itemChanged(item:FormItem, newValue:any): void;



}
/**
* Creates a new ValuesManager
* 
* @param typeCheckedProperties {Partial<ValuesManager>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ValuesManager} The new instance
*/
class ValuesManager{
static create(typeCheckedProperties? : Partial<ValuesManager>, uncheckedProperties?: Object): ValuesManager;




/**
*  Retrieve a ValuesManager by it's global Canvas.ID.
* 
* @param {string} global ID of the ValuesManager
*/
static getById(ID:string): ValuesManager;




}

/**
* EditProxy that handles ValuesManager objects when editMode is enabled.
*/
interface ValuesManagerEditProxy extends EditProxy {




}
/**
* Creates a new ValuesManagerEditProxy
* 
* @param typeCheckedProperties {Partial<ValuesManagerEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ValuesManagerEditProxy} The new instance
*/
class ValuesManagerEditProxy{
static create(typeCheckedProperties? : Partial<ValuesManagerEditProxy>, uncheckedProperties?: Object): ValuesManagerEditProxy;







}

/**
* A simple subclass of FileItem for displaying the contents of "imageFile" fields in 
*  DynamicForms. 
*  
*  Displays one of two UIs, according to the value of 
*  FileItem.showFileInline. If showFileInline is false, this Item
*  displays the View and Download icons and the filename. Otherwise, it streams the image-file 
*  and displays it inline.
*/
interface ViewFileItem extends FileItem {




}
/**
* Creates a new ViewFileItem
* 
* @param typeCheckedProperties {Partial<ViewFileItem>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ViewFileItem} The new instance
*/
class ViewFileItem{
static create(typeCheckedProperties? : Partial<ViewFileItem>, uncheckedProperties?: Object): ViewFileItem;







}

/**
* The ViewLoader component can be used to load new SmartClient-based user interfaces into a
*  running application.
*  
*  NOTE: before using a ViewLoader, be sure that you have read about and understood the
*  smartArchitecture. The most responsive and
*  scalable application architecture preloads views rather than using ViewLoaders.
*  
*  A ViewLoader is a Canvas, and can be provided anywhere a Canvas can be provided: as a Tab
*  pane, and Layout member, etc. When a ViewLoader draws, it shows a
*  ViewLoader.loadingMessage, then
*  performs an RPC to the ViewLoader.viewURL to load components.
*  
*  The response from the viewURL should be SmartClient components defined in JavaScript, with no
*  surrounding &lt;SCRIPT&gt; tags or other HTML framing. The returned script can be
*  dynamically generated, for example, it may be the result of a JSP containing an XML view
*  description enclosed in xmlTag tags.
*  
*  In the returned script, the special variable "viewLoader" is available to refer to the
*  ViewLoader instance that is loading components. The intended usage is that the returned
*  script creates a view consisting of SmartClient components, then calls
*  viewLoader.setView(myView) to place the loaded view into the ViewLoader.
*  If the view does not call setView() explicitly, the viewLoader will find the last top-level
*  UI component (Canvas subclass) created by the view and set that as the current view.
*  Top-level in this case means that the UI component is not contained in another UI component
*  as a member or child.
*  
*  The ViewLoader relies on the XMLHttpRequest object which can be disabled by end-users in some
*  supported browsers. See platformDependencies for more information.
*/
interface ViewLoader extends Label {
/**
* Message to show while the view is loading.
*  Use "&#36;{loadingImage}" to include Canvas.loadingImageSrc.
* @type {HTMLString}
* @default "Loading View...&amp;nbsp;${loadingImage}"
*/
loadingMessage?: HTMLString;

/**
* URL to load components from.
* @type {URL}
* @default null
*/
viewURL?: URL;

/**
* Parameters to be sent to the viewURL when fetching the view.
* @type {object}
* @default null
*/
viewURLParams?: object;

/**
* RPCRequest properties to be sent with every RPCRequest issued by the ViewLoader. Very
*  advanced; could be used to, for example, set HTTP headers.
* @type {Partial<RPCRequest>}
* @default null
*/
viewRPCProperties?: Partial<RPCRequest>;

/**
* Selects the HTTP method that will be used when fetching content. Valid values are "POST"
*  and "GET".
* @type {string}
* @default "GET"
*/
httpMethod?: string;

/**
* By default a ViewLoader will explicitly prevent browser caching.
*  
*  Set to true to allow browser caching if the browser would normally do so, in other
*  words, if the HTTP headers returned with the response indicate that the response can be
*  cached.
* @type {boolean}
* @default false
*/
allowCaching?: boolean;



/**
*  Change the URL this component loads a view from. Triggers a fetch from the new URL.
*  
*  Can also be called with no arguments to reload the view from the existing
*  ViewLoader.viewURL.
* 
* 
* @param {URL=} URL to retrieve view from
* @param {object=} Parameters to send to the viewURL. Merged with              component.viewURLParams if both are set.
* @param {Partial<RPCRequest>=} Additional properties for the RPCRequest             sent by the ViewLoader. Very advanced; could be used to, for             example, set HTTP headers.
*/
setViewURL(url?:URL, params?:object, rpcProperties?:Partial<RPCRequest>): void;

/**
* 
*  This method is called when a transport error occurs. Typically, this is the result of the
*  server returning an HTTP error code such as 404 - document not found. You can inspect the
*  RPCResponse object for the reasons for the error and take appropriate action. Typical
*  properties to look at are rpcResponse.status, and rpcResponse.httpResponseCode.
*  
*  This method is called from the response processing pipeline. If you want to provide your
*  own HTML response as the result of the error, you can do
*  so by setting rpcResponse.data to your HTML string. Returning false from this method
*  suppresses any further response handling. The default implementation of this method causes
*  an error message to be logged to the Developer Console and sets the HTML to the error
*  string.
* 
* 
* @param {RPCRequest} The RPCRequest that was made to the server
* @param {RPCResponse} The RPCResponse that was received
*/
handleError(rpcRequest:RPCRequest, rpcResponse:RPCResponse): boolean;

/**
*  Retrieve the current view. May be null if the view has not yet been loaded, or has been
*  explicitly set to null.
* 
* 
*/
getView(): Canvas;

/**
*  StringMethod fired when the view has been loaded. Has no default implementation. 
*  May be observed or overridden
*  to fire custom logic when loading completes.
* 
* 
* @param {Canvas} the view that was loaded
*/
viewLoaded(view:Canvas): void;



}
/**
* Creates a new ViewLoader
* 
* @param typeCheckedProperties {Partial<ViewLoader>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ViewLoader} The new instance
*/
class ViewLoader{
static create(typeCheckedProperties? : Partial<ViewLoader>, uncheckedProperties?: Object): ViewLoader;







}

/**
* A WSDataSource is a DataSource that is preconfigured to contact the WSDL web service built
*  into the SDK (see isomorphic/system/schema/SmartClientOperations.wsdl). This WSDL service
*  can be easily implemented on Java and non-Java backends. 
*  
*  WSDataSource supports all 4 DataSource operations (fetch, add, update, remove) and can be
*  used with ListGrids, DynamicForms and other DataBoundComponents just like other
*  DataSources.
*  
*  Note that WSDataSource is specifically designed for use with SmartClientOperations.wsdl. If
*  you are trying to connect to a pre-existing WSDL service, start with just DataSource, 
*  not WSDataSource, and see the wsdlBinding chapter for an
*  overview.
*/
interface WSDataSource extends DataSource {




}
/**
* Creates a new WSDataSource
* 
* @param typeCheckedProperties {Partial<WSDataSource>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {WSDataSource} The new instance
*/
class WSDataSource{
static create(typeCheckedProperties? : Partial<WSDataSource>, uncheckedProperties?: Object): WSDataSource;







}

/**
* Class representing a WebService definition derived from a WSDL file.
*  
*  A Web Service object allows you to invoke operations (via
*  WebService.callOperation), inspect schema declared in the
*  WSDL file (WebService.getSchema), and perform simple read-only
*  databinding WebService.getFetchDS. 
*  
*  Once a WebService has been loaded, a DataSource can be declared with a
*  DataSource.serviceNamespace to connect it to the web service, allowing DataSource
*  data to be loaded and saved to the web service using
*  OperationBinding.
*/
interface WebService extends Class {
/**
* Namespace of this WebService, derived from the targetNamespace
*  attribute of the &lt;wsdl:definitions&gt; element.
* @type {URI}
* @default null
*/
serviceNamespace?: URI;

/**
* Namespaces definitions to add to the root element of outbound XML messages sent to a web
*  service, as a mapping from namespace prefix to namespace URI.
*  
*  The default value is:
*  
*   globalNamespaces : {
*    xsi: "http://www.w3.org/2001/XMLSchema-instance",
*    xsd: "http://www.w3.org/2001/XMLSchema"
*   },
*  
*  This default value allows the use of the xsi:type and xsi:nil attributes without further
*  declarations.
*  
*  Note that some web services will only accept specific revisions of the XML Schema URI.
*  If xsi-namespaced attributes seem to be ignored by an older webservice, try the URI
*  "http://www.w3.org/1999/XMLSchema-instance" instead.
* @type {object}
* @default "..."
*/
globalNamespaces?: object;



/**
*/
getOperationNames(): Array<any>;

/**
*  Get the schema definition of any complexType or element of complexType defined in any
*  &lt;schema&gt; blocks in the WSDL file this WebService represents.
*  
* 
* @param {string} name of type or element
* @param {string=} optional type of schema to return, either "element" for               xs:element definitions only or "type" for xs:complexType                definitions. If unspecified, either will be returned,                with types preferred if names collide
*/
getSchema(schemaName:string, schemaType?:string): DataSource;

/**
*  Invoke a web service operation.
*  
*  The data parameter will be serialized to XML to form the input message for
*  the operation, as described by DataSource.xmlSerialize. Namespacing,
*  element ordering, and SOAP encoding rules are automatically followed. If the web
*  service you are trying to contact requires a complicated nested structure, consider
*  using WSRequest.useFlatFields to simplify the required JavaScript input data.
*  
*  The resultType selects what part of the message should be decoded to
*  JavaScript and made available as the "data" variable in the callback. The
*  resultType parameter can be either:
*  
*  
* - an XPath. "data" will be always be an Array, containing the selected elements as
*  decoded by XMLTools.toJS. All properties will have String value.
*  
* - the name of an XML Schema type found somewhere in the response. You can use the
*  WSDL tab of the Developer Console to analyze the WSDL file for an appropriate type name.
*  "data" will be an Array, containing the decoded elements as decoded by
*  DataSource.recordsFromXML. In this case, since the XML Schema type of the
*  selected data is known, properties will have correct type (eg "date" fields will
*  have JavaScript Date objects)
*  
* - null. "data" will an Object representing the entire &lt;SOAP:Body&gt; as decoded
*  to JavaScript. As above, properties will have correct type.
*  
*  In the callback, you also receive the XML document returned by the web service as
*  "xmlDoc".
*  
*  NOTE: callOperation() is appropriate for simple operations that do not
*  involve DataBound Components, such as logging into a web service, or retrieving simple
*  String data. callOperation() can also be used to retrieve small, read-only
*  datasets such as the option list for a SelectItem, but only if the dataset is guaranteed
*  to remain small enough for paging to be unnecessary. For any larger datasets or
*  anything that will be edited, DataSource integration is more appropriate.
* 
* 
* @param {string} Name of the operation to invoke
* @param {object} data to serialize as XML to form the inbound message of                 the operation
* @param {string | string | string} Type, Element name, or XPath that                 should be selected from the result. For XPaths, see                  WSRequest.xmlNamespaces for available namespace                  prefixes and how to add more.
* @param {Callback} Callback to invoke on completion. Signature                 callback(data, xmlDoc, rpcResponse, wsRequest)
* @param {Partial<WSRequest>} Additional properties for the WSRequest, such                   as HTTPHeaders
*/
callOperation(operationName:string, data:object, resultType:string | string | string, callback:Callback, requestProperties:Partial<WSRequest>): void;

/**
*  Return the SOAP message that will be formed from this WSRequest.
*  
* 
* @param {Partial<WSRequest>} web service request object
*/
getSoapMessage(wsRequest:Partial<WSRequest>): string;

/**
*  Get a DataSource representing the input message to a web service operation.
*  
*  This DataSource is suitable for use as
*  DataBoundComponent.dataSource for a form that the user fills out
*  when providing inputs to call this web service operation.
* 
* 
* @param {string} name of the web service operation whose inputs the                returned DataSource will represent
*/
getInputDS(operationName:string): DataSource;

/**
*  Get the schema for each part of the SOAP header for the input message of a given
*  operation, as a mapping from part name to schema. For example, given WSDL like:
*  
*    &lt;soap:header part="SessionHeader" message="tns:HeaderMessage"/&gt;
*    &lt;soap:header part="CallOptions" message="tns:HeaderMessage/&gt;
*  
*  The following schema would be returned:
*  
*    { SessionHeader : sessionHeaderPartSchema,
*     CallOptions : callOptionsPartSchema }
*  
*  The schema are instances of DataSource that can be inspected to discover the
*  elements and types that are legal in that header part, and can construct a valid SOAP
*  header part if DataSource.xmlSerialize is invoked.
*  
* 
* @param {string} name of an operation from this web service
*/
getInputHeaderSchema(operationName:string): object;

/**
*  Get the schema for each part of the SOAP header for the output message of a given
*  operation, as a mapping from part name to schema. For example, given WSDL like:
*  
*    &lt;soap:header part="SessionHeader"/&gt;
*    &lt;soap:header part="CallOptions"/&gt;
*  
*  The following schema would be returned:
*  
*    { SessionHeader : sessionHeaderPartSchema,
*     CallOptions : callOptionsPartSchema }
*  
*  The schema are instances of DataSource that can be inspected to discover the
*  elements and types that are legal in that header part, and can construct a valid SOAP
*  header part if DataSource.xmlSerialize is invoked.
* 
* 
* @param {string} name of an operation from this web service
*/
getOutputHeaderSchema(operationName:string): object;

/**
*  Override this method to return data that should be serialized as SOAP headers for the
*  current operation, such as a sessionId.
*  
*  Format of the returned data is the same as that documented for
*  DSRequest.headerData.
*  
*  The object passed to this method will be a true DSRequest in the case of a DataSource
*  operation, or just an Object with a "data" property for web service operations
*  initiated by WebService.callOperation.
*  
*  If headerData is instead provided via either dsRequest.headerData or as
*  part of the requestProperties parameter to
*  WebService.callOperation, this method will never be called.
* 
* 
* @param {DSRequest} */
getHeaderData(dsRequest:DSRequest): object;

/**
*  Retrieve a DataSource that provides read-only access to records returned by a web
*  service operation.
*  
*  DataBoundComponent can be bound to the returned
*  DataSource, and the ListGrid.fetchData method can be invoked
*  to retrieve data from the web service.
*  
*  The returned DataSource is only capable of the "fetch" 
*  dataSourceOperations, not "update", "add" or
*  "remove". To create a DataSource capable of full read-write access, use
*  DataSource.operationBindings with the
*  OperationBinding.wsOperation property set to associate each
*  DataSource operation with a web service operation. 
* 
* 
* @param {string} name of the web service operation to invoke to fetch                records
* @param {string} tag or type name of the XML element to be returned as                DataSource records
* @param {Partial<OperationBinding>=} Optional additional properties for the operationType:"fetch"  OperationBinding which this method automatically creates. This can be used to set properties such as OperationBinding.useFlatFields or OperationBinding.recordXPath
*/
getFetchDS(operationName:string, resultType:string, operationBindingProperties?:Partial<OperationBinding>): void;

/**
*  Set location can be used when the actual URL where a service will be accessible isn't
*  known until runtime, or changes at runtime, hence can't be embedded in the service
*  definition.
*  
*  With an operation parameter, setLocation() can be used to set a distinct
*  URL for each web service operation. This is a development-time only feature that allows
*  XML flat files to be placed at various URLs on a server, to serve as spoofed responses
*  for each web service operation.
* 
* 
* @param {URL} URL where web service can be contacted
* @param {string=} optional operation name to set the location for, for               debugging only
*/
setLocation(location:URL, operation?:string): void;



}
/**
* Creates a new WebService
* 
* @param typeCheckedProperties {Partial<WebService>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {WebService} The new instance
*/
class WebService{
static create(typeCheckedProperties? : Partial<WebService>, uncheckedProperties?: Object): WebService;




/**
*  Retrieve a WebService object by the targetNamespace declared on the &lt;wsdl:definitions&gt;
*  element in the WSDL file from which the WebService was derived.
*  
*  If you have more than one &lt;wsdl:service&gt; in the same target namespace, use
*  WebService.getByName to disambiguate.
* 
* 
* @param {string} uri from the "targetNamespace" attribute of the &lt;wsdl:definitions&gt; element in the WSDL file
*/
static get(serviceNamespace:string): WebService;

/**
*  Retrieve a WebService object by the name attribute declared on the &lt;wsdl:service&gt; tag.
* 
* 
* @param {string} name attribute from the &lt;wsdl:service&gt; tag
* @param {string=} optional serviceNamespace if needed to disambiguate
*/
static getByName(serviceName:string, serviceNamespace?:string): WebService;




}

/**
* A general purpose Window class for implementing dialogs, portlets, alerts, prompts, wizards
*  and desktop-like windowing interfaces.
*  
*  Windows can contain arbitrary SmartClient components, configured via the Window.items
*  property. Windows may be Window.isModal or non-modal.
*  
*  Windows provide a series of highly configurable and skinnable AutoChild
*  including a header, various header controls, footer, and corner resizer.
*  
*  The more specialized Dialog subclass of Window has additional functionality targetted
*  at simple prompts and confirmations, such as buttons with default actions, and single-method
*  isc.warn for common application dialogs.
*/
interface Window extends Layout {
/**
* If true, this Window may be moved around by the user by dragging on the Window header. 
*  Note that if the header is not showing, the Window can't be drag-repositioned regardless
*  of this setting.
* @type {boolean}
* @default true
*/
canDragReposition?: boolean;

/**
* Constrains drag-resizing and drag-repositioning of this canvas to either the rect of its
*  parent (if set to true) or an arbitrary rect based on its parent (if set to a
*  [Left,Top,Width,Height] rect array). In the latter mode you may use negative offsets
*  for left/top and a width/height greater than the visible or scroll width of the parent
*  to allow positioning beyond the confines of the parent.
*  
*  If this canvas has no parent, constrains dragging to within the browser window.
*  
*  Affects target and outline dragAppearance, not tracker.
*  
*  Note: keepInParentRect affects only user drag interactions, not programmatic moves.
*  
*  Example use cases:
*  keepInParentRect: true - confine to parent
*  keepInParentRect: [0, 0, 500, 500] - confine to top left 500x500 region within parent
*  keepInParentRect: [0, 0, 10000, 10000] - in combination with
*  overflow: "auto", confine to parent, but allow moving off the right and bottom of the
*  parent to force scrolling (and hence enlarge the scrollWidth of the parent).
* @type {boolean | Array<Partial<Integer>>}
* @default true
*/
keepInParentRect?: boolean | Array<Partial<Integer>>;

/**
* Can the window be drag-resized? If true the window may be drag resized from its edges,
*  and if showing, via the resizer icon in the footer.
* @type {boolean}
* @default false
*/
canDragResize?: boolean;

/**
* When drag resizing is enabled via Window.canDragResize, restricts resizes to only certain
*  edges or corners.
*  
*  This property on Window overrides the default defined by Canvas.resizeFrom.
* @type {Array<Partial<EdgeName>>}
* @default "["R","B","BR"]"
*/
resizeFrom?: Array<Partial<EdgeName>>;

/**
* By default Windows show a Canvas.useBackMask in Internet Explorer
*  versions predating Internet Explorer 9. This is a workaround for a native browser
*  issue whereby certain DOM elements such as IFRAMEs (whether rendered
*  within SmartClient components via features such as HTMLFlow.contentsURL or 
*  explicitly written into the HTML of the page) will not be properly occluded
*  by DOM elements which overlap them but have a higher z-index.
*  
*  A side-effect of this is that the Canvas.opacity can not be modified
*  for the entire window. Developers may disable the backmask in order to support
*  opacity in IE versions less than 9 by setting this property to false, however you
*  should be aware that in doing this there is a potential for the "burn through"
*  problem described above.
* @type {boolean}
* @default "varies"
*/
useBackMask?: boolean;

/**
* Renders the widget to be partly transparent. A widget's opacity property may
*  be set to any number between 0 (transparent) to 100 (opaque).
*  Null means don't specify opacity directly, 100 is fully opaque.
*  Note that heavy use of opacity may have a performance impact on some older
*  browsers.
*  
*  In older versions of Internet Explorer (Pre IE9 / HTML5), opacity is achieved
*  through proprietary filters. If 
*  Canvas.neverUseFilters within this application
*  developers must set Canvas.useOpacityFilter to true for specific components
*  on which opacity support is required.
*  
*  Also note that opacity is incompatible 
*  with Canvas.useBackMask, and that this property is enabled
*  by default for Window instances.
* @type {Integer}
* @default null
*/
opacity?: Integer;

/**
* If true, when shown this Window will intercept and block events to all other existing
*  components on the page.
*  
*  Use Window.showModalMask to darken all other elements on the screen when a modal dialog
*  is showing.
*  
*  Chained modal windows - that is, modal windows that launch other modal windows - are
*  allowed. You can accomplish this by simply creating a second modal Window while a modal
*  Window is showing.
*  
*  Note only top-level Windows (Windows without parents) can be modal.
* @type {boolean}
* @default false
*/
isModal?: boolean;

/**
* A ScreenSpan instance used to darken the rest of a page when a modal window is
*  active. To use, set Window.showModalMask to true, add a CSS style 
*  "modalMask" to the active skin (generally with background-color set), 
*  and adjust Window.modalMaskOpacity.
* @type {Canvas}
* @default null
*/
modalMask?: Canvas;

/**
* If true, displays a translucent mask over the rest of the page when a modal window
*  is displayed.
* @type {boolean}
* @default null
*/
showModalMask?: boolean;

/**
* Controls the opacity of the modal mask displayed behind modal windows.
* @type {number}
* @default "50"
*/
modalMaskOpacity?: number;

/**
* Specifies the CSS style for the modal mask.
* @type {string}
* @default "modalMask"
*/
modalMaskStyle?: string;

/**
* If true, this Window widget will automatically be centered on the page when shown.
*  If false, it will show up in the last position it was placed (either programmatically,
*  or by user interaction).
*  
*  Note: If an auto-centering Window is either programmatically moved or dragged 
*  by an end user, auto-centering behavior is automatically turned off. To manually center 
*  a Window, you can use Window.centerInPage. Auto-centering will also be disabled if
*  you pass an explicit Canvas.left or Canvas.top value at Class.create.
* @type {boolean}
* @default false
*/
autoCenter?: boolean;

/**
* If true, a click outside the bounds of the Window will have the same effect as
*    pressing its cancel button.
*    Note: Applies only to modal windows.
* @type {boolean}
* @default false
*/
dismissOnOutsideClick?: boolean;

/**
* Should this window be dismissed (same effect as pressing the "Cancel" button) when the 
*  user presses the "Escape" key?
*  Windows with this setting will dismiss on Escape keypresses in any of the following
*  cases:
*  
*  
* - The window or one of its descendants has focus (and does not cancel the Escape 
*    keypress)
*  
* - The window is Window.isModal, and not itself masked. This ensures
*    that focus is not on some unrelated element on the page.
*  
*  
*  If unset default behavior depends on whether a close / cancel button is visible for
*  this item.
* @type {boolean}
* @default null
*/
dismissOnEscape?: boolean;

/**
* Body of the Window, where Items or Window.src
*  is shown.
*  
*  The following autoChildUsage apply:
*  
*  
* - Window.bodyStyle for the Canvas.styleName
*  
* - Window.printBodyStyle for the styleName to use when
*    printing
*  
* - Window.bodyColor / Window.hiliteBodyColor for
*    the Canvas.backgroundColor
*  
* @type {Canvas}
* @default null
*/
body?: Canvas;

/**
* If true, draw the body contents when this Window is drawn.
* @type {boolean}
* @default true
*/
showBody?: boolean;

/**
* Style of the Window body.
* @type {CSSStyleName}
* @default "windowBody"
*/
bodyStyle?: CSSStyleName;

/**
* Style for the Window body in printed output.
* @type {CSSStyleName}
* @default "printHeader"
*/
printBodyStyle?: CSSStyleName;

/**
* Color of the Window body. Overrides the background color specified in the style.
* @type {CSSColor}
* @default "#FFFFFF"
*/
bodyColor?: CSSColor;

/**
* Highlight color for the Window body (shown when the body is flashed).
* @type {CSSColor}
* @default "#EEEEEE"
*/
hiliteBodyColor?: CSSColor;

/**
* The contents of the Window body. Can be specified three different ways:
*    
* - an Array of Canvases that will become the children of the Window's body when it
*    is initialized; the canvases in this array should be created, but not drawn (autodraw:
*    false).
*    
* - a single canvas that will become a child of the Window body.
*    
* - a string that will be set as the body's contents.
* @type {Array<Partial<Canvas>> | Canvas | string}
* @default null
*/
items?: Array<Partial<Canvas>> | Canvas | string;

/**
* A URL to load as content for the Window's body. If specified, this
*  attribute will take precedence over the items attribute.
*  
*  Note that setting window.src is essentially a shortcut for setting Window.items
*  to a single HTMLflow with a specified HTMLFlow.contentsURL.
* @type {string}
* @default null
*/
src?: string;

/**
* If this window has Window.src specified, this property can be used to indicate
*  whether the source is a standalone HTML page or an HTML fragment.
*  
*  This is similar to the HTMLFlow.contentsType property - be sure to read the
*  HTMLFlow documentation to understand circumstances where contentsType:"page" is
*  unsafe and not recommended.
* @type {string}
* @default "page"
*/
contentsType?: string;

/**
* The name of the widget class (as a string) to use for the body. If unset the appropriate
*  constructor type will be determined as follows:
*  - if Window.items is defined as an array of widgets, and Window.contentLayout 
*   is not set to "none", bodyConstructor defaults to a VLayout
*  - if Window.src is set, bodyConstructor defaults to an HTMLFlow
*  - otherwise bodyConstructor will default to a simple Canvas
*  Note that if this property is overridden for some window, the specified constructor 
*  should be a subclass of one of these defaults to ensure the window renders out as 
*  expected.
* @type {string}
* @default null
*/
bodyConstructor?: string;

/**
* Default properties for the body of the Window
*  You can change the class-level bodyDefaults for all Windows by changing this item
*  or set instance.body to be another object of properties to override for your instance only
* @type {object}
* @default "..."
*/
bodyDefaults?: object;

/**
* The layout policy that should be used for widgets within the Window body.
*  
*  See ContentLayoutPolicy and Window.bodyConstructor for details.
* @type {ContentLayoutPolicy}
* @default "vertical"
*/
contentLayout?: ContentLayoutPolicy;

/**
* If true, the window is resized automatically to accommodate the contents
*  	of the body, if they would otherwise require scrolling.
* @type {boolean}
* @default false
*/
autoSize?: boolean;

/**
* Header for the Window, based on an HLayout. The header contains the title and some standard
*  controls for the window, which may be configured via Window.headerControls.
*  
*  The following autoChildUsage apply:
*  
*  
* - Window.headerStyle for Canvas.styleName
*  
* - Window.printHeaderStyle for the styleName to use
*    when printing.
*  
* @type {HLayout}
* @default null
*/
header?: HLayout;

/**
* If true, show a Window.header for this Window. 
*  
*  Note that in certain Smartclient skins Window.showHeaderBackground may be set to
*  false and the header's appearance implemented as part of the
*  window's Canvas.showEdges. In this case suppressing the header can be achieved
*  by overriding the edge media as well as setting this property to false. For example, to
*  create a headerless window with a similar appearance to a Menu in the
*  TreeFrog skin, the following attributes could be used:
*  
*    showHeader:false,
*    edgeImage:"[SKIN]/Menu/m.png",
*    edgeSize:10, edgeTop:17, edgeBottom:17,
*    edgeCenterBackgroundColor:"#F7F7F7"
*  
* @type {boolean}
* @default true
*/
showHeader?: boolean;

/**
* Img background component for the header, for gradient or image-based display
* @type {Img}
* @default null
*/
headerBackground?: Img;

/**
* Should the window header show a background image? Default value is true for all browsers
*  except for Internet Explorer.
*  If set to true the image source is derived from Window.headerSrc and 
*  Window.hiliteHeaderSrc, otherwise the background will be styled according to 
*  Window.headerStyle / Window.hiliteHeaderStyle.
* @type {boolean}
* @default "varies"
*/
showHeaderBackground?: boolean;

/**
* Style for the Window header.
* @type {CSSStyleName}
* @default "WindowHeader"
*/
headerStyle?: CSSStyleName;

/**
* CSS Style for header in printed output
* @type {CSSStyleName}
* @default "printHeader"
*/
printHeaderStyle?: CSSStyleName;

/**
* If Window.showHeaderBackground is true, this property provides
*  the URL of the background image for the header.
* @type {SCImgURL}
* @default ""[SKIN]Window/headerGradient.gif" | null"
*/
headerSrc?: SCImgURL;

/**
* Array of members to show in the Window header. 
*  
*  The default value of headerControls is an Array of Strings listing the
*  standard header controls in their default order:
*  
*   headerControls : ["headerIcon", "headerLabel", 
*            "minimizeButton", "maximizeButton", "closeButton"]
*  
*  You can override headerControls to change the order of standard controls in
*  the header. You can also omit standard controls this way, although it more efficient to
*  use the related "show" property if available (eg Window.showMinimizeButton). 
*  
*  By embedding a Canvas directly in this list you can add arbitrary additional controls to
*  the header, for example, an additional button (eg return to dock) or a DynamicForm with
*  various kinds of input controls. 
*  
*  Note that having added controls to headerControls, you can still call APIs directly on
*  those controls to change their appearance, and you can also show() and hide() them if
*  they should not be shown in some circumstances.
*  
*  Tip: custom controls need to set layoutAlign:"center" to appear vertically centered.
*  
*  Component XML:
*  
*  To define headerControls in Component XML a special set of components
*  are used as markers. The standard header controls can be explicitly specified as: 
*  
*  &lt;headerControls&gt;
*    &lt;WindowHeaderIcon/&gt;
*    &lt;WindowHeaderLabel/&gt;
*    &lt;WindowMinimizeButton/&gt;
*    &lt;WindowMaximizeButton/&gt;
*    &lt;WindowCloseButton/&gt;
*  &lt;/headerControls&gt;
*  
* @type {Array<Partial<string>>}
* @default "(see below)"
*/
headerControls?: Array<Partial<string>>;

/**
* Highlight style for the Window header. Displayed when a window 
*  is Window.flash
* @type {CSSStyleName}
* @default "WindowHeader"
*/
hiliteHeaderStyle?: CSSStyleName;

/**
* If Window.showHeaderBackground is true, this governs the URL of the image to 
*  use in the header's highlighted state when the window is Window.flash
* @type {SCImgURL}
* @default ""[SKIN]Window/headerGradient_hilite.gif" | null"
*/
hiliteHeaderSrc?: SCImgURL;

/**
* Label that shows Window title in header.
*  
*  The following autoChildUsage applies:
*  Window.title for Label.contents.
* @type {Label}
* @default null
*/
headerLabel?: Label;

/**
* Show a title (typically just text) on the header for this window.
* @type {boolean}
* @default true
*/
showTitle?: boolean;

/**
* Title for this Window, shown if Window.showTitle is true in the Window.header
*  (if drawn).
* @type {HTMLString}
* @default "Untitled Window"
*/
title?: HTMLString;

/**
* This is an object literal property block specifying various properties of the header
*  label that displays the Window.title. Overrideable defaults are as follows:
*  
*  
* - styleName- defaults to "windowHeaderText" and specifies the css style
*  that is used to render the Window.title text.
*  
*  You can override the the above properties by calling Class.changeDefaults.
* @type {object}
* @default "see below"
*/
headerLabelDefaults?: object;

/**
* Header icon shown at left end of header by default.
* @type {Img}
* @default null
*/
headerIcon?: Img;

/**
* If true, we show an icon on the left in the header.
* @type {boolean}
* @default true
*/
showHeaderIcon?: boolean;

/**
* This is an object literal property block specifying the various properties of the
*  headerIcon - the icon that appears at the top left of the window and is by default the
*  Isomorphic logo. Overrideable defaults are as follows:
*  
*  
* - width - default to 16 and specifies the width of the headerIcon.
*  
* - height - default to 14 and specifies the height of the headerIcon.
*  
* - src - defaults to "[SKIN]/Window/minimize.gif" and specifies the image
*  for the headerIcon.
*  
*  You can override the the above properties by calling Class.changeDefaults.
* @type {object}
* @default "..."
*/
headerIconDefaults?: object;

/**
* If true, the user can give the header buttons focus (see Window.minimizeButton, 
*  Window.maximizeButton, Window.restoreButton and Window.closeButton).
* @type {boolean}
* @default false
*/
canFocusInHeaderButtons?: boolean;

/**
* Button show in the header that will close this Window by calling Window.closeClick.
* @type {ImgButton}
* @default null
*/
closeButton?: ImgButton;

/**
* If true, show a close button in the header, which will dismiss this window by 
*  calling Window.closeClick.
* @type {boolean}
* @default true
*/
showCloseButton?: boolean;

/**
* ImgButton shown in the header that will minimize this Window by calling Window.minimize.
* @type {ImgButton}
* @default null
*/
minimizeButton?: ImgButton;

/**
* If true, show a minimize button in the header--clicking it minimizes the Window.
* @type {boolean}
* @default true
*/
showMinimizeButton?: boolean;

/**
* Is this window minimized. If true at init time, the window will be drawn minimized.
*  To set this property at runtime use Window.minimize or Window.restore.
* @type {boolean}
* @default false
*/
minimized?: boolean;

/**
* If Window.minimizeHeight is unset, by the window will shrink to the height of the
*  header when minimized.
*  
*  If there is no header, the defaultMinimizeHeight will be used instead.
* @type {number}
* @default "16"
*/
defaultMinimizeHeight?: number;

/**
* Height for the window when minimized.
*  If unset the window will shrink to the height of the header, if present, otherwise
*  Window.defaultMinimizeHeight
* @type {number}
* @default null
*/
minimizeHeight?: number;

/**
* Should this window minimize, maximize, and restore as an animation, or as a 
*  simple 1-step transition?
* @type {boolean}
* @default null
*/
animateMinimize?: boolean;

/**
* If this window is minimizeable, and animateMinimize is true, what should the duration of 
*  the minimize / maximize be (in ms)? If unset defaults to canvas.animationTime.
* @type {number}
* @default null
*/
minimizeTime?: number;

/**
* Default acceleration function for performing an animated minimize / maximize. If unset, 
*  this.animateAcceleration will be used by default instead
* @type {AnimationAcceleration}
* @default null
*/
minimizeAcceleration?: AnimationAcceleration;

/**
* ImgButton that restores the Window via Window.restore.
* @type {ImgButton}
* @default null
*/
restoreButton?: ImgButton;

/**
* Is this window maximized. If true at init time, the window will be drawn maximized.
*  To set this property at runtime use Window.maximize or Window.restore.
* @type {boolean}
* @default false
*/
maximized?: boolean;

/**
* Button that will make this Window fill the browser via Window.maximize.
* @type {ImgButton}
* @default null
*/
maximizeButton?: ImgButton;

/**
* If true, show a maximize button in the header - clicking it maximizes the Window
* @type {boolean}
* @default false
*/
showMaximizeButton?: boolean;

/**
* Optional footer for the window, providing space for controls such as the resizer and 
*  status bar.
* @type {HLayout}
* @default null
*/
footer?: HLayout;

/**
* If true, show a footer for this Window, including resizer, statusBar, etc.
*  This setting is commonly overridden for skinning purposes.
* @type {boolean}
* @default true
*/
showFooter?: boolean;

/**
* The height of the footer, in pixels.
* @type {number}
* @default "18"
*/
footerHeight?: number;

/**
* Array of members to show in the Window footer. 
*  
*  The default value of footerControls is an Array of Strings listing the
*  standard footer controls in their default order:
*  
*   footerControls : ["spacer", "resizer"]
*  
*  As with Window.headerControls, you can override footerControls
*  to change the order of standard controls in the footer. "spacer" is a special
*  value which will create a LayoutSpacer in the footer bar. "resizer"
*  will show the Window.resizer in the footer.
*  
*  By embedding a Canvas directly in this list you can add arbitrary additional controls to
*  the footer. 
*  
*  Note that the Window.statusBar is not part of the set of footer controls - it is a
*  separate canvas rendered behind all footer controls. If you include some custom status bar
*  directly in the footerControls you may want to set Window.showFooter to false.
*  
*  Tip: custom controls need to set layoutAlign:"center" to appear vertically centered.
*  
*  Component XML:
*  
*  To define footerControls in Component XML a special set of components
*  are used as markers. The standard footer controls can be explicitly specified as: 
*  
*  &lt;footerControls&gt;
*    &lt;WindowFooterSpacer/&gt;
*    &lt;WindowResizer/&gt;
*  &lt;/footerControls&gt;
*  
* @type {Array<Partial<string>>}
* @default "(see below)"
*/
footerControls?: Array<Partial<string>>;

/**
* Simple Canvas-based status bar, shown in the footer. Window.setStatus can be used to
*  show text here.
* @type {Canvas}
* @default null
*/
statusBar?: Canvas;

/**
* If true, show a statusBar for this Window, including resizer.
*  Note that the status bar will only be displayed if the footer
*  is showing for the window (Window.showFooter).
* @type {boolean}
* @default true
*/
showStatusBar?: boolean;

/**
* ImgButton-based resizer, shown in the footer.
* @type {ImgButton}
* @default null
*/
resizer?: ImgButton;

/**
* If true, show a button in the lower right corner that allows users to resize the Window.
*  Note that the resizer will only be displayed if the footer is showing for the window
*  (Window.showFooter) and Window.canDragResize is true.
* @type {boolean}
* @default true
*/
showResizer?: boolean;

/**
* Where should the window be placed on the screen? Valid settings include
*  "fillScreen", "fillPanel", "halfScreen"
*  and "none"
*  
*  If not explicitly specified, default is to use PanelPlacement "fillScreen" 
*  if Browser.isHandset, and "none" for non-handset devices.
*  
*  If window.placement is something other than "none",
*  sizing and positioning settings (either explicit left, top, width, height settings or
*  the Window.autoCenter and Window.autoSize features) will have no effect.
* @type {PanelPlacement}
* @default null
*/
placement?: PanelPlacement;

/**
* showEdges dynamically defaults to false when the Window.placement setting
*  indicates the Window will be filling a portion of the screen or a panel.
* @type {boolean}
* @default null
*/
showEdges?: boolean;

/**
* Whether to show a drop shadow for this Canvas. 
*  
*  Developers should be aware that the drop shadow
*  is drawn outside the specified width and height of the widget meaning a widget with shadows 
*  takes up a little more space than it otherwise would. A full screen canvas with showShadow set 
*  to true as this would be likely to cause browser scrollbars to appear - developers can handle
*  this by either setting this property to false on full-screen widgets, or by setting
*  overflow to "hidden" on the &lt;body&gt; element browser-level scrolling is never intended to occur.
*  
*  showShadow dynamically defaults to false when the Window.placement setting
*  indicates the Window will be filling a portion of the screen or a panel.
* @type {boolean}
* @default null
*/
showShadow?: boolean;

/**
* Alternative style for the window used whenever Window.placement settings indicate the
*  menu will be filling a portion of the screen or a panel. Generally this alternative
*  style should not have rounded or excessively large edges.
* @type {CSSStyleName}
* @default "windowBackgroundFill"
*/
fillSpaceStyleName?: CSSStyleName;

/**
* Should this window automatically be shown at the top of the page's z-order and
*  be brought to front via Canvas.bringToFront whenever the user clicks it?
*  
*  If Window.isModal is true for this window, this setting will have no effect - we
*  always bring the window to the front on initial display and on mouseDown.
*  By default we also do this for non-modal windows (which matches user expectation for
*  most standard interfaces - think of switching between OS-level application windows), but 
*  this may be disabled for cases where it is not appropriate by setting this attribute to
*  false
* @type {boolean}
* @default true
*/
bringToFrontOnMouseUp?: boolean;

/**
* Text to show in the status bar of the window (if one is visible)
* @type {string}
* @default null
*/
status?: string;

/**
* Default class used to construct the EditProxy for this component
*  when the component is Canvas.setEditMode.
* @type {SCClassName}
* @default "WindowEditProxy"
*/
editProxyConstructor?: SCClassName;



/**
*  Setter for the Window.autoCenter attribute.
* 
* @param {boolean} new value for autoCenter
*/
setAutoCenter(autoCenter:boolean): void;

/**
*  Setter for Window.headerStyle.
* 
* @param {CSSStyleName} new Canvas.styleName for the Window.header.
*/
setHeaderStyle(newHeaderStyle:CSSStyleName): void;

/**
*  Dynamically update Window.showCloseButton to show / hide the closeButton
* 
*/
setShowCloseButton(): void;

/**
*  Dynamically update Window.showMinimizeButton to show / hide the minimizeButton
* 
*/
setShowMinimizeButton(): void;

/**
*  Dynamically update Window.showMaximizeButton to show / hide the maximizeButton
* 
*/
setShowMaximizeButton(): void;

/**
*  Dynamically update Window.showHeaderIcon to show / hide the headerIcon
* 
*/
setShowHeaderIcon(): void;

/**
*  Sets the Window.title that appears in the window Window.header.
*  The header will be redrawn if necessary.
* 
* @param {HTMLString} new title.
*/
setTitle(newTitle:HTMLString): void;

/**
*  Updates whether the Window.title is shown in the Window.header.
*  No impact unless the header is being shown. The header will be redrawn if necessary.
* 
* @param {boolean} new showTitle
*/
setShowTitle(newShowTitle:boolean): void;

/**
*  	Sets the text in the status bar of the window, redrawing if necessary.
* 
* @param {string} new text for the status bar
*/
setStatus(statusString:string): void;

/**
*  Sets the URL of the contents to display in the body of the window, redrawing if
*  necessary.
* 
* @param {string} URL of new contents to be displayed in the window body
*/
setSrc(url:string): void;

/**
*  Setter for Window.bodyStyle.
* 
* @param {CSSStyleName} new Canvas.styleName for the Window.body.
*/
setBodyStyle(newBodyStyle:CSSStyleName): void;

/**
*  Setter for Window.bodyColor.
* 
* @param {CSSColor} new Canvas.backgroundColor for the Window.body.
*/
setBodyColor(newBodyColor:CSSColor): void;

/**
*  Setter for Window.hiliteBodyColor.
* 
* @param {CSSColor} new hiliteBodyColor.
*/
setHiliteBodyColor(newHiliteBodyColor:CSSColor): void;

/**
*  Adds a widget to the body area of the window.
* 
* @param {Canvas} the widget to be added
*/
addItem(item:Canvas): Array<any>;

/**
*  Removes a widget from the body area of the window. 
* 
* @param {Canvas} the widget to be removed
*/
removeItem(item:Canvas): Array<any>;

/**
*  	Adds an array of widgets to the window.
* 
* @param {Array<Partial<Canvas>>} an array of widgets to be added
*/
addItems(items:Array<Partial<Canvas>>): Array<any>;

/**
*  	Removes an array of widgets from the window.
* 
* @param {Array<Partial<Canvas>>} an array of widgets to be removed
*/
removeItems(items:Array<Partial<Canvas>>): Array<any>;

/**
*  Same as Layout.addMember. Note that in order to add items to Window.body,
*  you use Window.addItem rather than addMember. Adding a member to 
*  a Window adds the member as a sibling to the header, body and other built-in Window
*  subcomponents.
* 
* @param {Canvas} the canvas object to be added to the layout
* @param {Integer=} the position in the layout to place newMember (starts with 0);              if omitted, it will be added at the last position
*/
addMember(newMember:Canvas, position?:Integer): void;

/**
*  Same as Layout.addMembers. Note that in order to add items to Window.body,
*  you use Window.addItem rather than addMembers. Adding a member to 
*  a Window adds the member as a sibling to the header, body and other built-in Window
*  subcomponents.
* 
* @param {Array<Partial<Canvas>> | Canvas} array of canvases to be added or single Canvas
* @param {number=} position to add newMembers; if omitted newMembers will be added              at the last position
*/
addMembers(newMembers:Array<Partial<Canvas>> | Canvas, position?:number): void;

/**
*  Reveals the child Canvas passed in by showing it if it is currently hidden. Note, in the 
*  case of Window, "child Canvas" means widgets in the Window's "items" collection as well as 
*  real children (the children of a Window - ie, the elements of its "children" array - are 
*  its component parts like header and body)
* 
* 
* @param {GlobalId | Canvas} the child Canvas to reveal, or its global ID
*/
revealChild(child:GlobalId | Canvas): void;

/**
*  Setter for Window.autoSize
* 
* @param {boolean} true if the window should auto-size to its content
*/
setAutoSize(autoSize:boolean): void;

/**
*  Should this window be dismissed (same effect as pressing the "Cancel" button) when the 
*  user presses the "Escape" key?
*  Default behavior: if Window.dismissOnEscape is set, just return it. Otherwise return
*  true if this window is showing a "close" control in the header 
*  (see Window.headerControls).
* 
*/
shouldDismissOnEscape(): boolean;

/**
*  Centers the Window in the page. This is called automatically in window.show() if
*  Window.autoCenter is true.
*  Note - if the Window is a child of another widget, we center in the parent widget
*  rather than centering in the page.
* 
*/
centerInPage(): void;

/**
*      Makes the window header flash if it's visible; if there's no header, or the header
*      is hidden, makes the window body flash instead.
*      
*  	This method is executed when users click outside the bounds of a modal window
*  	so they'll notice that they have to do something with the window.
* 
*/
flash(): void;

/**
*  Minimize the window. Fired when the user clicks the minimize button if 
*  Window.showMinimizeButton is true.
*  Default implementation shrinks the window to just the height of the header bar, hiding the
*  body. If Window.animateMinimize is true, the resize will be animated.
*  A restore button will be displayed in place of the minimize button when the window is 
*  minimized.
* 
*/
minimize(): void;

/**
*  Restores the window to its specified height and width after a call to Window.minimize or 
*  Window.maximize. Called from a click on the restore button shown in place of the 
*  minimize or maximize button when the window is minimized or maximized.
*  Resizing will occur as an animation if Window.animateMinimize is true.
* 
*/
restore(): void;

/**
*  Maximize the window. Fired when the user clicks the maximize button if 
*  Window.showMaximizeButton is true.
*  Default implementation moves the window to 0, 0 and resizes the window to 
*  "100%" on both axes, so it will fill the browser window (or the parent
*  of the Window instance, if appropriate).
*  If Window.animateMinimize is true, the maximize will be animated.
*  A restore button will be displayed in place of the maximize button when the window is 
*  maximized.
*  
* 
*/
maximize(): void;

/**
*  Handles a click on the close button of this window. The default implementation
*  calls Window.close and returns false to prevent bubbling of the click event.
*  
*  Override this method if you want
*  other actions to be taken.
*  
*  Custom implementations may call close() to trigger the default behavior.
* 
*/
closeClick(): boolean;

/**
*  Close this window. 
*  This method is fired by the default Window.closeClick implementation.
*  Default implementation will hide the window.
* 
*/
close(): void;



}
/**
* Creates a new Window
* 
* @param typeCheckedProperties {Partial<Window>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {Window} The new instance
*/
class Window{
static create(typeCheckedProperties? : Partial<Window>, uncheckedProperties?: Object): Window;
/**
* A declared value of the enum type 
*  ContentLayoutPolicy.
* @type {Constant}
* @default "none"
*/
static NONE?: Constant;

/**
* A declared value of the enum type 
*  ContentLayoutPolicy.
* @type {Constant}
* @default "vertical"
*/
static VERTICAL?: Constant;

/**
* A declared value of the enum type 
*  ContentLayoutPolicy.
* @type {Constant}
* @default "horizontal"
*/
static HORIZONTAL?: Constant;








}

/**
* EditProxy that handles Window objects when editMode is enabled.
*/
interface WindowEditProxy extends LayoutEditProxy {


/**
*  Returns the text based on the current component state to be edited inline.
*  Called by the EditProxy.inlineEditForm to obtain the starting edit value.
*  
*  Returns the component's title.
* 
* 
*/
getInlineEditText(): void;

/**
*  Save the new value into the component's state. Called by the
*  EditProxy.inlineEditForm to commit the change.
*  
*  Updates the component's title.
* 
* 
* @param {string} the new component title
*/
setInlineEditText(newValue:string): void;



}
/**
* Creates a new WindowEditProxy
* 
* @param typeCheckedProperties {Partial<WindowEditProxy>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {WindowEditProxy} The new instance
*/
class WindowEditProxy{
static create(typeCheckedProperties? : Partial<WindowEditProxy>, uncheckedProperties?: Object): WindowEditProxy;







}

/**
* A DataSource preconfigured to use the RPCTransport transport
*  (sometimes called "JSONP") for cross-domain calls to JSON services.
*  
*  To use this DataSource, provide the URL of the service as DataSource.dataURL, and
*  provide DataSource.fields that describe the structure of the data you want to
*  extract from the service's response.
*  
*  DataSource.recordXPath and DataSourceField.valueXPath can be used to extract
*  data from the JSON structure returned by the service. See
*  clientDataIntegration for an overview of how to
*  control what parts of the JSON structure are included in the DSResponse object, and
*  hence provided to DataBoundComponents that are bound to this DataSource.
*  
*  This XJSONDataSource is really a subclass of DataSource with just a few property settings:
*  
*   dataFormat : "json",
*   dataTransport : "scriptInclude"
*   callbackParam : "callback"
*  
*/
interface XJSONDataSource extends DataSource {




}
/**
* Creates a new XJSONDataSource
* 
* @param typeCheckedProperties {Partial<XJSONDataSource>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {XJSONDataSource} The new instance
*/
class XJSONDataSource{
static create(typeCheckedProperties? : Partial<XJSONDataSource>, uncheckedProperties?: Object): XJSONDataSource;







}

/**
* Regular expression-based source code colorizer for XML source.
*  
*  NOTE: This class exists only for use with the SmartClient Feature Explorer and 
*  SmartGWT Showcases and cannot be used in any other environment.
*/
interface XMLSyntaxHiliter extends SyntaxHiliter {




}
/**
* Creates a new XMLSyntaxHiliter
* 
* @param typeCheckedProperties {Partial<XMLSyntaxHiliter>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {XMLSyntaxHiliter} The new instance
*/
class XMLSyntaxHiliter{
static create(typeCheckedProperties? : Partial<XMLSyntaxHiliter>, uncheckedProperties?: Object): XMLSyntaxHiliter;







}

/**
* Utility methods for dealing with XML elements, XML Schema, WSDL files, XSLT, and other
*  XML-related functionality.
*/
interface XMLTools extends Class {




}
/**
* Creates a new XMLTools
* 
* @param typeCheckedProperties {Partial<XMLTools>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {XMLTools} The new instance
*/
class XMLTools{
static create(typeCheckedProperties? : Partial<XMLTools>, uncheckedProperties?: Object): XMLTools;




/**
*  Load an XML document from the origin server or from a foreign server by relaying through the
*  origin server. An asynchronous callback provides both the XML document and raw text of the
*  response.
*  
*  Relaying through the origin server requires that the ISC HttpProxyServlet be installed and
*  accessible.
*  
* 
* @param {URL} URL to load the schema from
* @param {Callback} callback to fire when the XML is loaded. Signature is               callback(xmlDoc, xmlText)
* @param {RPCRequest=} additional properties to set on the RPCRequest                     that will be issued
*/
static loadXML(URL:URL, callback:Callback, requestProperties?:RPCRequest): void;

/**
*  Disables an Internet Explorer-specific work around for the MSXML bug that the 'xml' namespace
*  prefix cannot be explicitly declared.
*  
*  Though redundant,
*  the Namespaces in XML spec allows
*  XML documents to explicitly declare namespace prefix 'xml' bound to namespace name
*  http://www.w3.org/XML/1998/namespace; e.g.
*  xmlns:xml="http://www.w3.org/XML/1998/namespace"
*  MSXML does not allow the 'xml' namespace prefix to be declared, and will raise the XML
*  parse error: The namespace prefix is not allowed to start with the reserved string "xml".
*  Microsoft has disclosed this bug as a Normative Variation in MSXML:
*  http://msdn.microsoft.com/en-us/library/ff460535(v=vs.85).aspx.
*  A framework-level work around is used by default in XMLTools.parseXML where if the
*  string xmlns:xml="http://www.w3.org/XML/1998/namespace" or
*  xmlns:xml='http://www.w3.org/XML/1998/namespace' is found in the first 1000
*  characters of the xmlText parameter to parseXML(), then these two strings are
*  removed from xmlText wherever they appear. This work around may be disabled by
*  calling disableIEXMLHackaround() at any time before parseXML() is called.
* 
*/
static disableIEXMLHackaround(): void;

/**
*  Parse XML text into an XMLDocument. Parse errors, if any, are reported to the log.
*  
*  NOTE: Internet Explorer's XML parser implementation, MSXML, has a bug in its
*  handling of namespace name http://www.w3.org/XML/1998/namespace. Though redundant,
*  the Namespaces in XML spec allows
*  XML documents to explicitly declare namespace prefix 'xml' bound to namespace name
*  http://www.w3.org/XML/1998/namespace; e.g.
*  xmlns:xml="http://www.w3.org/XML/1998/namespace"
*  MSXML does not allow the 'xml' namespace prefix to be declared, and will raise the XML
*  parse error: The namespace prefix is not allowed to start with the reserved string "xml".
*  Microsoft has disclosed this bug as a Normative Variation in MSXML:
*  http://msdn.microsoft.com/en-us/library/ff460535(v=vs.85).aspx.
*  A framework-level work around is used by default where if the string
*  xmlns:xml="http://www.w3.org/XML/1998/namespace" or
*  xmlns:xml='http://www.w3.org/XML/1998/namespace' is found in the first 1000
*  characters of xmlText, then these two strings are removed from xmlText
*  wherever they appear. This work around may be disabled by calling XMLTools.disableIEXMLHackaround
*  at any time before parseXML() is called.
* 
* 
* @param {string} XML text to be parsed
*/
static parseXML(xmlText:string): XMLDocument;

/**
* 
*  Returns true if the current browser exposes an XML parser that can be used for SmartClient
*  XML operations like web service bindings and XML processing. See
*  platformDependencies for more information on when the XML parser may not
*  available and what features are impacted as a result.
* 
* 
*/
static nativeXMLAvailable(): boolean;

/**
*  Translates an XML fragment to JavaScript collections. This method works just like the
*  server-side method XML.toJS(Element, Writer):
*  
*  
* -  Elements become JavaScript Objects with each attribute becoming a property
*  
* -  Subelements with just text (no child elements or attributes) become properties
*  
* -  Subelements with child elements or attributes become sub objects
*  
*  For example, if you pass the following fragment to this method:
*  
*  &lt;foo&nbsp;bar="zoo"&gt;
*    &nbsp;&nbsp;&lt;x&gt;y&lt;/x&gt;
*  &lt;/foo&gt;
*  
*  You will get back the following JS structure: 
*  
*  { bar:"zoo", x:"y"}
*  
*  All atomic property values will be of String type. Use DataSource.recordsFromXML
*  to do schema-driven XML to JS transform, which can produce correctly typed values.
*  
* 
* @param {XMLElement | XMLDocument} The element to transform to JS
*/
static toJS(element:XMLElement | XMLDocument): object;

/**
*  Applies an XPath expression to JavaScript objects, returning matching objects.
*  
*  Both child and attribute names are interpreted as property names, and array access notation
*  can be used to select elements from Arrays. For example:
*    var results = {
*     searchResults:[
*       { title:"Page One", relevance:6.3 },
*       { title:"Page Two", relevance:5.2, 
*        summary: "Summary of Page One" }
*     ]
*    };
* 
*    // returns the "searchResults" two-item Array
*    isc.XMLTools.selectObjects(results, "/searchResults");
* 
*    // returns the first item under "searchResults", in an Array (NOTE: in XPath, Array
*    // index starts at 1, not 0)
*    isc.XMLTools.selectObjects(results, "/searchResults[1]");
* 
*    // returns ["Page One"]
*    isc.XMLTools.selectObjects(results, "/searchResults[1]/title");
* 
*    // also returns ["Page One"]
*    isc.XMLTools.selectObjects(results, "/searchResults[1]@title");
*  
*  A limited form of XPath "predicates", that is, expressions with brackets that filter
*  returned objects, is allowed. A predicate can be either:
*  
*  
* - a number only, eg [5], for Array access
*  
* - the XPath function call "last()", eg [last()], to retrieve the last item
*  
* - a property name (*without* any leading "@"), meaning that the property contains a value
*    that is considered "true" in JavaScript. For example: [summary]
*  
* - a property name, comparison operator, and either a number or String literal, for
*    example, [name = "bob"]. In this case the property can also be the XPath function
*    position(), for example, [position() > 5]
*  
*  Some examples of using simple predicates with the sample data above:
*  
*    // returns an Array with only the first result
*    isc.XMLTools.selectObjects(results, "/searchResults[relevance > 5.5]");
*  
*    // return an Array with only the second result, since the first has no summary
*    isc.XMLTools.selectObjects(results, "/searchResults[summary]");
*  
*  Details of the XPath -> Objects mapping:
*  
*  
* - JavaScript Object properties are considered element children, and text children do not
*    exist (in the XML model, text children exist *between* element children, but nothing
*    exists between JavaScript properties)
*  
* - The contents of Array-valued properties are considered immediate element children (this
*    is consistent with the predicate "[5]" acting like Array access)
*  
* - "*" in XML selects all element children, so "*" in Object XPath selects the values of
*    all properties, that is, isc.getValues, except
*    that Array-valued properties are "flattened" into the returned list.
*  
* 
* 
* @param {object} Object to select results from
* @param {string} XPath expression
*/
static selectObjects(object:object, xPath:string): Array<any>;

/**
*  Retrieve a set of nodes from an XML element or document based on an XPath expression.
*  
*  If the target document is namespaced, namespace prefixes declared in the document element of
*  the target document will be available, as well as the default namespace, if declared, under
*  the prefix "default".
*  
*  To declare your own namespace prefixes, provide a prefix to URI mapping as a simple JS
*  Object, for example:
*  
*   {
*    az : "http://webservices.amazon.com/AWSECommerceService/2005-03-23",
*    xsd : "http://www.w3.org/2001/XMLSchema"
*   }
*  
*  
*  NOTE: this API cannot be supported on the Safari web browser for versions earlier
*  than 3.0.3.
* 
* 
* @param {XMLElement | XMLDocument | string} Native XMLElement,document, or xml string                           to select from
* @param {string} XPath expression to use to select nodes
* @param {Map<URI>=} namespace mapping used by the expression
*/
static selectNodes(element:XMLElement | XMLDocument | string, expression:string, namespaces?:Map<URI>): Array<any>;

/**
*  Retrieve a string value from an XML element or document based on an XPath expression.
*  
*  If more than one node matches, only the first node's value will be returned.
*  
*  Namespacing works as described under XMLTools.selectNodes
*  
*  NOTE: this API cannot be supported on the Safari web browser for versions prior to
*  3.0.3.
* 
* 
* @param {XMLElement | XMLDocument | string} Native XMLElement,document, or xml string                           to select from
* @param {string} XPath expression to use to select nodes
* @param {Map<URI>=} namespace mapping used by the expression
*/
static selectString(element:XMLElement | XMLDocument | string, expression:string, namespaces?:Map<URI>): string;

/**
*  Retrieve a numeric value from an XML element or document based on an XPath expression.
*  
*  If more than one node matches, only the first node's value will be returned.
*  
*  Namespacing works as described under XMLTools.selectNodes
*  
*  NOTE: this API cannot be supported on the Safari web browser for versions prior to
*  3.0.3.
* 
* 
* @param {XMLElement | XMLDocument | string} Native XMLElement,document, or xml string                           to select from
* @param {string} XPath expression to use to select nodes
* @param {Map<URI>=} namespace mapping used by the expression
*/
static selectNumber(element:XMLElement | XMLDocument | string, expression:string, namespaces?:Map<URI>): number;

/**
*  Apply an XSLT Stylesheet to an XML Document.
*  
*  This method cannot currently be supported on the Safari web browser versions prior to
*  3.0.3.
* 
* 
* @param {XMLDocument} XML document to apply the transform to
* @param {XMLDocument} XSLT stylesheet to use for transform
*/
static transformNodes(inputDocument:XMLDocument, styleSheet:XMLDocument): string;

/**
*  Takes an XMLDocument and returns it as a String.
*  
*  This method is not supported on the Safari web browser versions prior to 3.0.3.
* 
* 
* @param {XMLDocument} XML document to apply the transform to
*/
static serializeToString(inputDocument:XMLDocument): string;

/**
*  Load an XML file containing XML schema definitions and create DataSource and SimpleType
*  objects to represent the schema. You can use to loaded schema to bind ISC components,
*  perform validation, create editing interfaces, and build other metadata-driven interfaces.
*  You can also use DataSource.inheritsFrom to overlay
*  presentation-specific data (such as user-visible titles) on top of XML Schema.
*  
*  In the loaded XML Schema, all &lt;xsd:complexType&gt; declarations become SmartClient
*  DataSources, and all &lt;xsd:simpleType&gt; definitions become SmartClient
*  SimpleType. 
*  
*  By default, named complexType definitions and named element definitions containing
*  complexTypes become global DataSources, that is, they can be fetched with
*  DataSource.getDataSource. Inline complexType definitions get
*  automatically generated names.
*  
*  Named simpleType declarations become global SimpleType, that is,
*  subsequently defined DataSources can use them for DataSourceField.type. XML
*  schema "restrictions" for simple types are automatically translated to
*  DataSourceField.valueMap or DataSourceField.validators as
*  appropriate.
*  
*  The created SchemaSet object is available in the callback as the single parameter
*  "schemaSet", or can retrieved via SchemaSet.get(schemaNamespace).
*  
*  NOTE: unless you are building an application that dynamically loads XML Schema 
*  without prior knowledge, instead of calling loadXMLSchema(), you should either:
*  
*  
* - use the loadXMLSchemaTag tag to eliminate the need for an asynchronous
*  download of an XML Schema file as part of application startup, OR
*  
* - use the "WSDL" tab in the Developer Console to obtain the XML Schema definition as a
*  JavaScript file that can be loaded via a normal HTML &lt;SCRIPT SRC=&gt; tag and/or
*  combined with other JavaScript files. 
*  
*  
*  NOTE: required fields: the XML Schema concept of "required" for an attribute or subelement, 
*  expressed via use="required" (for an attribute) or minOccurs > 0 (for a subelement), is
*  that the attribute or element must be present in the XML document but can have any
*  value, including being empty or null. The SmartClient notion of required means
*  non-null. You can express the SmartClient notion of required in XML Schema with the
*  combination of minOccurs>0 and a minLength or length "restriction", and SmartClient 
*  will recognize the field as SmartClient-required, with all of the behaviors that implies
*  (eg, specially styled form titles, automatic validation, etc).
* 
* 
* @param {URL} URL to load the schema from
* @param {Callback} signature is callback(schemaSet)
* @param {RPCRequest} additional properties to set on the RPCRequest                     that will be issued
* @param {boolean} if set, xsd:import statements will be processed                 automatically to load dependent XSD files where a                  "location" is specified. The callback will not fire until                  all dependencies have been loaded
*/
static loadXMLSchema(schemaURL:URL, callback:Callback, requestProperties:RPCRequest, autoLoadImports:boolean): void;

/**
*  Load a WSDL file and create an instance of WebService that allows invoking operations and
*  binding DataSources to web service operations.
*  
*  The created WebService object is available in the callback as the single parameter
*  "service", or can be retrieved via WebService.get(serviceNamespace).
*  
*  XML Schema present in the WSDL file will also be processed as described in
*  XMLTools.loadXMLSchema. However note that imported XML Schema
*  (&lt;xs:import&gt; tag) will not be automatically loaded and must be loaded manually using
*  XMLTools.loadXMLSchema before the loaded service will be usable. This is because the WSDL
*  spec allows but does not require a valid URL to be provided for loading imported XML Schema.
*  
*  NOTE: unless you are building an application that dynamically contacts WSDL web services
*  without prior knowledge, instead of calling loadWSDL(), you should either:
*  
*  
* - use the loadWSDLTag tag to eliminate the need for an asynchronous
*  download of a WSDL file as part of application startup, OR
*  
* - use the "WSDL" tab in the Developer Console to obtain the WebService definition as a
*  JavaScript file that can be loaded via a normal HTML &lt;SCRIPT SRC=&gt; tag and/or combined
*  with other JavaScript files.
*  
*  Platform notes:
*  
*  
* - loadWSDL() is not supported in Safari 2.0 (but is supported in Safari 3.0.3 and greater)
*  However, you can use either approach mentioned above (loadWSDLTag or JavaScript file) with 
*  Safari pre 3.0.3.
*  
* - if you are using a non-Java server, in order to obtain a JavaScript file representing a
*  web service, you must run the Developer Console in the Java-based SmartClient SDK
*  
* 
* 
* @param {URL} URL to load the WSDL file from
* @param {Callback} signature is callback(service)
* @param {RPCRequest} additional properties to set on the RPCRequest                     that will be issued
* @param {boolean} if set, xsd:import statements will be processed                 automatically to load dependent XSD files where a                  "location" is specified. The callback will not fire until                  all dependencies have been loaded
*/
static loadWSDL(wsdlURL:URL, callback:Callback, requestProperties:RPCRequest, autoLoadImports:boolean): void;




}

/**
* Chooses one or another next process element based on AdvancedCriteria applied to
*  Process.state.
*  
*  If the AdvancedCriteria evaluate to true, the DecisionTask.nextElement is
*  chosen, otherwise the DecisionTask.failureElement.
*  
*  Note that "XOR" in XORGateway means "exclusive or" - only one next element is
*  chosen.
* @deprecated * In favor of DecisionTask as of SmartClient release 13.0

*/
interface XORGateway extends DecisionTask {




}
/**
* Creates a new XORGateway
* 
* @param typeCheckedProperties {Partial<XORGateway>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {XORGateway} The new instance
*/
class XORGateway{
static create(typeCheckedProperties? : Partial<XORGateway>, uncheckedProperties?: Object): XORGateway;







}

/**
* A subclass of EventCanvas, used to render 
*  Calendar.zones in CalendarView.
*  
*  A ZoneCanvas is a semi-transparent canvas that highlights a portion of a 
*  calendar view, by rendering across all lanes and behind normal Calendar.data.
*  
*  By default, the canvas shows a bottom-aligned label containing the 
*  CalendarEvent.name.
*  Default styling is specified at the Calendar.zoneStyleName 
*  and can be overridden for CalendarEvent.styleName.
*/
interface ZoneCanvas extends EventCanvas {




}
/**
* Creates a new ZoneCanvas
* 
* @param typeCheckedProperties {Partial<ZoneCanvas>=} Object literal containing init properties that will be checked against the class field types.
* @param uncheckedProperties {Object} Object literal containing init properties that will not be checked against the class field types.
* @return {ZoneCanvas} The new instance
*/
class ZoneCanvas{
static create(typeCheckedProperties? : Partial<ZoneCanvas>, uncheckedProperties?: Object): ZoneCanvas;







}























































































































































































































































































































































































































































































































































































































































































































































































































































/**
* Generic Chart properties and interfaces to be mixed into concrete charting implementations.
*  
*  Components such as the ListGrid.chartData and
*  CubeGrid.makeChart expect this interface and can drive charting engines
*  that support it.
*  
*  Concrete Chart implementations may expose whatever properties they want for configuration,
*  however, to enable easy switching of charting engines (different engines may be used for
*  different end users based on that user's installed plugins), they should support
*  the properties of this interface to the maximum extent possible.
*  
*  This interface also provides core data model management (see
*  Chart.getValue) for charting engines.
*/
interface Chart {
/**
* Property in each record that holds a data value.
*  
*  Not used if there is an inline facet, see Chart.data.
* @type {string}
* @default "_value"
*/
valueProperty?: string;

/**
* Dataset for this chart.
*  
*  Two basic formats are supported:
*  
*  
* - "Standard model": data is an array of CellRecords where each record
*  contains one data value. Each record also contains a property named after each facetId
*  whose value is a facetValueId from that facet.
*  
*  For example, with a facet with id "regions" and facetValues "west", "north" and "east", and
*  with Chart.valueProperty with it's default value "_value", the data property
*  could be:
*  
*   isc.Chart.create({
*     facets:[{ id:"regions" }],
*     data : [
*      {regions:"west", _value:4},
*      {regions:"north", _value:2},
*      {regions:"east", _value:5}
*     ]
*   })
*  If there were a second facet with id "product" and facetValues "cars" and "trucks", a Chart
*  with a complete set of values would be:
*  
*   isc.Chart.create({
*     facets:[{ id:"regions" }, { id:"product" }],
*     data : [
*      {product:"cars", regions:"west", _value:4},
*      {product:"cars", regions:"north", _value:2},
*      {product:"cars", regions:"east", _value:5},
*      {product:"trucks", regions:"west", _value:1},
*      {product:"trucks", regions:"north", _value:9},
*      {product:"trucks", regions:"east", _value:3}
*     ]
*   })
*  This 2 facet (or "2 dimensional") dataset, if rendered as a bar chart, would use stacked or
*  clustered bars and a legend.
*  
* 
*  
* - "Inlined facet": data is a single CellRecord or Array of CellRecords where
*  each record contains multiple data values. In this case, one facet definition is considered
*  "inlined", meaning that the facetValueIds from this facet appear as properties in each
*  record, and each such property holds one data value. For example, a complete chart
*  definition whose dataset is equivalent to the previous example would be:
*  
*   isc.Chart.create({
*     facets: [{ 
*      inlinedValues:true,
*      values : [ { id:"west" }, { id:"north" }, { id : "east" } ]
*     }],
*     data : { west:4, north:5, east:2 }
*   })
*  Note that the property "inlinedValues" must be set on the facet definition, and the set of
*  facetValueIds must be specified outside of the data array (with the "standard model",
*  facetValueIds can be automatically derived from the data). 
*  
*  A two facet chart with an inlined facet can be defined as follows:
*  
*   isc.Chart.create({
*     facets: [
*      { 
*       inlinedValues:true,
*       values : [ { id:"west" }, { id:"north" }, { id : "east" } ]
*      }, 
*      { id:"product" }
*     ],
*     data : [
*       { product:"cars", west:4, north:5, east:2 },
*       { product:"trucks", west:1, north:9, east:3 }
*     ]
*   })
*  
*  Comparing between the formats, the "standard model" format treats all facets identically,
*  which can be of use when integrating with server technology that likewise treats all facets
*  identically. The "inlined facet" format is a more compact data representation and allows
*  easier conversion from data displayed in a ListGrid.
* @type {Array<Partial<CellRecord>> | CellRecord}
* @default null
*/
data?: Array<Partial<CellRecord>> | CellRecord;

/**
* An Array of facets, exactly analogous to CubeGrid.facets, except that:
*  
*  
* - the "inlinedValues" property can be set on a facet to change data representation as
*  described under Chart.data.
*  
* - for a non-inlined facet, Charts support auto-derivation of facetValues from the data.
*  
* @type {Array<Partial<Facet>> | Facet}
* @default null
*/
facets?: Array<Partial<Facet>> | Facet;

/**
* Whether to use stacking for charts where this makes sense (bar, column, line and radar
*  charts). If stacked is not set and two facets are supplied, clustering is assumed.
* @type {boolean}
* @default false
*/
stacked?: boolean;

/**
* Type of chart to draw, see ChartType for a list of known types.
* @type {ChartType}
* @default "Column"
*/
chartType?: ChartType;

/**
* A label for the data values, such as "Sales in Thousands", typically used as the label for
*  the value axis.
* @type {string}
* @default null
*/
valueTitle?: string;

/**
* Title for the chart as a whole.
* @type {string}
* @default null
*/
title?: string;

/**
* Subtitle for the chart as a whole, typically used to describe fixed facet values (such as
*  "for Q1, 1999").
*  
*  The subTitle should appear under the title in less emphasized text.
* @type {string}
* @default null
*/
subTitle?: string;

/**
* Whether to show chart in a 3D appearance, for charts that support this.
* @type {boolean}
* @default true
*/
threeD?: boolean;

/**
* Whether the chart should animate when shown (eg, bars or columns grow into place)
* @type {boolean}
* @default true
*/
shouldAnimateShow?: boolean;

/**
* Whether to show labels on each individual value (bar, column or line point)
* @type {boolean}
* @default false
*/
labelValues?: boolean;



/**
*  General facet and data model setup, including auto-derivation of facetValues from data if
*  necessary. Should be called by any concrete charting implementation before calling any
*  other Chart method.
* 
* 
*/
setupChart(): void;

/**
*  Get a facet definition by facetId.
* 
* @param {string} the id of the facet to retrieve
*/
getFacet(facetId:string): Facet;

/**
*  Get facet value definition by facetId and facetValueId.
* 
* @param {string} the id of the facet to retrieve
* @param {string} the id of the facet value to retrieve
*/
getFacetValue(facetId:string, facetValueId:string): FacetValue;

/**
*  Lookup a data value by the set of matching facetValues expressed as a
*  FacetValueMap. Automatically handles the Chart.data, if any.
*  
*  This method is designed to be called by a concrete Chart implementation.
*  
*  As a special case, if Chart.data is a single Object and the only facet is
*  Chart.data, any value can be used as the single facetId required. For example, a
*  legal FacetValueMap in this case would be {inlined:"west"}.
* 
* 
* @param {FacetValueMap} set of facetValues describing the data value to retrieve
*/
getValue(criteria:FacetValueMap): any;

/**
*  Change the dataset for this chart on the fly. May or may not be supported by concrete chart
*  implementations.
* 
* 
* @param {Array<Partial<CellRecord>> | CellRecord} new dataset
*/
setData(newData:Array<Partial<CellRecord>> | CellRecord): void;




}

/**
* A DataBoundComponent is a widget that can configure itself for viewing or editing objects which
*  share a certain schema by "binding" to the schema for that object (called a "DataSource").
*  
*  A schema (or DataSource) describes an object as consisting of a set of properties (or
*  "fields").
*  
*  DataBoundComponents have a DataBoundComponent.dataSource for
*  dealing with binding to DataSources, 
*  DataBoundComponent.fields the schema information
*  provided by a DataSource, and manipulating objects or sets of object from the DataSource.
*  
*  The following visual components currently support databinding:
*   DynamicForm
*   DetailViewer
*   ListGrid
*   TreeGrid
*   TileGrid
*   ColumnTree
*   CubeGrid
*  
*  The following non-visual components also support databinding:
*   ValuesManager
*   ResultSet
*   ResultTree
*  
*/
interface DataBoundComponent {
/**
* Before we start editing values in this DataBoundComponent, should we perform a deep clone 
*  of the underlying values. See DataSource.deepCloneOnEdit for details of what this means.
*  
*  If this value is not explicitly set, it defaults to the value of DataSource.deepCloneOnEdit.
*  This value can be overridden per-field with DataSourceField.deepCloneOnEdit.
*  
*  Like the other deepCloneOnEdit settings, this flag only has an effect if you are 
*  editing a values object that contains nested objects or arrays, using 
*  Canvas.dataPaths.
* @type {boolean}
* @default null
*/
deepCloneOnEdit?: boolean;

/**
* When editing values in this DataBoundComponent, should we perform a deep clone of values 
*  that are not associated with a field (ie, attributes on the record that do not map to a 
*  component field either directly by name, or by FormItem.dataPath. If this value is
*  not explicitly set, it defaults to the value of 
*  DataSource.deepCloneNonFieldValuesOnEdit if there is a dataSource, or to the value 
*  of the static DataSource.deepCloneNonFieldValuesOnEdit if there is no
*  dataSource.
*  
*  Like the other deepCloneOnEdit settings, this flag only has an effect if you are 
*  editing a values object that contains nested objects or arrays.
* @type {boolean}
* @default null
*/
deepCloneNonFieldValuesOnEdit?: boolean;

/**
* The DataSource that this component should bind to for default fields and for performing
*  DSRequest.
*  
*  Can be specified as either a DataSource instance or the String ID of a DataSource.
* @type {DataSource | GlobalId}
* @default null
*/
dataSource?: DataSource | GlobalId;

/**
* How to fetch and manage records retrieve from the server. See FetchMode.
*  
*  This setting only applies to the ResultSet automatically created by calling
*  ListGrid.fetchData. If a pre-existing ResultSet is passed to setData() instead, it's
*  existing setting for ResultSet.fetchMode applies.
* @type {FetchMode}
* @default "paged"
*/
dataFetchMode?: FetchMode;

/**
* When using DataBoundComponent.dataFetchMode, how many records to fetch at a time. If set to
*  a positive integer, dataPageSize will override the default 
*  ResultSet.resultSize for ResultSets automatically created when you call
*  ListGrid.fetchData (and similarly for the ResultTree.resultSize of
*  ResultTrees). Leaving dataPageSize at its default means to just use the default page
*  size of the data container.
*  
*  Note that regardless of the dataPageSize setting, a component will always fetch
*  all of data that it needs to draw. Settings such as
*  ListGrid.showAllRecords,
*  ListGrid.drawAllMaxCells and
*  ListGrid.drawAheadRatio can cause more rows than the configured
*  dataPageSize to be fetched.
* @type {Integer}
* @default null
*/
dataPageSize?: Integer;

/**
* Delay in milliseconds before fetching data.
*  
*  This setting only applies to the ResultSet automatically created by calling
*  ListGrid.fetchData. If a pre-existing ResultSet is passed to setData() instead, it's
*  existing setting for ResultSet.fetchDelay applies.
* @type {Integer}
* @default null
*/
dataFetchDelay?: Integer;

/**
* A DataBoundComponent manipulates records with one or more fields, and
*  component.fields tells the DataBoundComponent which fields to present, in what
*  order, and how to present each field.
*  
*  When both component.fields and 
*  DataBoundComponent.dataSource are set,
*  any fields in component.fields with the same name as a DataSource field
*  inherit properties of the DataSource field. This allows you to centralize data model
*  information in the DataSource, but customize presentation of DataSource fields on a
*  per-component basic. For example, in a ListGrid, a shorter title or format for a field
*  might be chosen to save space.
*  
*  By default, only fields specified on the component are shown, in the order specified on
*  the component. The DataBoundComponent.useAllDataSourceFields flag can be set to show all fields
*  from the DataSource, with component.fields acting as field-by-field
*  overrides and/or additional fields.
*  
*  If a DataBoundComponent is given a DataSource, but no component.fields, the
*  "default binding" is used: fields are shown in DataSource order, according
*  to the properties DataBoundComponent.showHiddenFields and 
*  DataBoundComponent.showDetailFields.
* @type {Array<Partial<DataSourceField>>}
* @default null
*/
fields?: Array<Partial<DataSourceField>>;

/**
* If true, the set of fields given by the "default binding" (see 
*  DataBoundComponent.fields) is used, with any fields specified in
*  component.fields acting as overrides that can suppress or modify the
*  display of individual fields, without having to list the entire set of fields that
*  should be shown.
*  
*  If component.fields contains fields that are not found in the DataSource,
*  they will be shown after the most recently referred to DataSource field. If the new
*  fields appear first, they will be shown first.
*  
*  ${isc.DocUtils.linkForExampleId('validationFieldBinding', 'This example')} shows a mixture of component
*  fields and DataSource fields, and how they interact for validation.
*  
*  This setting may be cleared if a FieldPicker is used to edit the component's field
*  order.
* @type {boolean}
* @default false
*/
useAllDataSourceFields?: boolean;

/**
* Whether to show fields marked hidden:true when a DataBoundComponent is given a
*  DataSource but no component.fields.
*  
*  The hidden property is used on DataSource fields to mark fields that are
*  never of meaning to an end user.
* @type {boolean}
* @default false
*/
showHiddenFields?: boolean;

/**
* Whether to show fields marked detail:true when a DataBoundComponent is 
*  given a DataSource but no component.fields.
*  
*  The detail property is used on DataSource fields to mark fields that 
*  shouldn't appear by default in a view that tries to show many records in a small space.
* @type {boolean}
* @default false
*/
showDetailFields?: boolean;

/**
* Whether to show fields of non-atomic types when a DataBoundComponent is given a
*  DataSource but no component.fields.
*  
*  If true, the component will show fields that declare a complex type, for example, a
*  field 'shippingAddress' that declares type 'Address', where 'Address' is the ID of a
*  DataSource that declares the fields of a shipping address (city, street name, etc).
*  
*  Such fields may need custom formatters or editors in order to create a usable interface,
*  for example, an Address field in a ListGrid might use a custom formatter to combine the
*  relevant fields of an address into one column, and might use a pop-up dialog for
*  editing.
* @type {boolean}
* @default true
*/
showComplexFields?: boolean;

/**
* DSRequest.operationId this component should use when performing fetch 
*  operations, and export operations where 
*  DataBoundComponent.exportOperation is not set
* @type {string}
* @default null
*/
fetchOperation?: string;

/**
* DSRequest.operationId this component should use when performing update operations.
* @type {string}
* @default null
*/
updateOperation?: string;

/**
* DSRequest.operationId this component should use when performing add operations.
* @type {string}
* @default null
*/
addOperation?: string;

/**
* DSRequest.operationId this component should use when performing remove operations.
* @type {string}
* @default null
*/
removeOperation?: string;

/**
* DSRequest.operationId this component should use when performing 
*  server-driven exports via the exportData() API. If this property is not set, exportData()
*  exports will use the DataBoundComponent.fetchOperation instead. If 
*  you want to switch off this automatic use of the fetchOperation for 
*  exportData() exports, specify exportOperation: ""
* @type {string}
* @default null
*/
exportOperation?: string;

/**
* The list of field-names to export. If provided, the field-list in the exported output is 
*  limited and sorted as per the list.
*  
*  If exportFields is not provided, the exported output includes all visible fields 
*  from this component, sorted as they appear.
* @type {Array<Partial<string>>}
* @default null
*/
exportFields?: Array<Partial<string>>;

/**
* Setting exportAll to true prevents the component from passing it's list of fields to the 
*  export call. The result is the export of all visible fields from DataSource.fields.
*  
*  If exportAll is false, an export operation will first consider 
*  DataBoundComponent.exportFields, if it's set, and fall back on all visible fields from
*  DataSource.fields otherwise.
* @type {boolean}
* @default false
*/
exportAll?: boolean;

/**
* The format in which the data should be exported if DataBoundComponent.exportData is called without specifying 
*  a format. Otherwise, the format passed to DataBoundComponent.exportData via the requestProperties parameter 
*  will be used.
*  
*  See ExportFormat for more information.
* @type {ExportFormat}
* @default null
*/
exportAs?: ExportFormat;

/**
* Specifies whether the exported data will be downloaded as an attachment or displayed in a 
*  new browser window. This option will be used if DataBoundComponent.exportData is called without specifying
*  it. Otherwise, the exportDisplay passed to DataBoundComponent.exportData via the requestProperties parameter 
*  will be used.
*  
*  See ExportDisplay for more information.
* @type {ExportDisplay}
* @default null
*/
exportDisplay?: ExportDisplay;

/**
* If Summary rows exist for this component, whether to include them when exporting client data.
* @type {boolean}
* @default true
*/
exportIncludeSummaries?: boolean;

/**
* Flag controlling when to recategorize records being dropped on a databound
*  component from another databound component.
* @type {RecategorizeMode}
* @default "checked"
*/
dragRecategorize?: RecategorizeMode;

/**
* If set, detect and prevent duplicate records from being transferred to this component, either via
*  drag and drop or via DataBoundComponent.transferSelectedData. When a duplicate transfer is detected,
*  a dialog will appear showing the DataBoundComponent.duplicateDragMessage.
*  
*  If the component either does not have a DataSource or has a DataSource with no
*  DataSourceField.primaryKey declared, duplicate checking is off by
*  default. If duplicate checking is enabled, it looks for an existing record in the dataset
*  that has all of the properties of the dragged record, and considers that a duplicate.
*  
*  For DragDataAction:"copy" where the target DataSource is related to the source
*  DataSource by foreignKey, a duplicate means that the target list, as filtered by the current
*  criteria, already has a record whose value for the foreignKey field matches the
*  primaryKey of the record being transferred.
*  
*  For example, consider dragging "employees" to "teams", where "teams" has a field
*  "teams.employeeId" which is a foreignKey pointing to "employees.id", and the target
*  grid has search criteria causing it to show all the members of one team. A duplicate -
*  adding an employee to the same team twice - is when the target grid's dataset contains an
*  record with "employeeId" matching the "id" field of the dropped employee.
* @type {boolean}
* @default null
*/
preventDuplicates?: boolean;

/**
* Message to show when a user attempts to transfer duplicate records into this component, and
*  DataBoundComponent.preventDuplicates is enabled.
*  
*  If set to null, duplicates will not be reported and the dragged duplicates will not be
*  saved.
* @type {string}
* @default "Duplicates not allowed"
*/
duplicateDragMessage?: string;

/**
* Indicates whether the text of the offlineMessage property should be displayed if no data is
*  available because we do not have a suitable offline cache
* @type {boolean}
* @default true
*/
showOfflineMessage?: boolean;

/**
* Message to display when this DataBoundComponent attempts to load data that is not available
*  because the browser is currently offline. Depending on the component, the message is either
*  displayed in the component's body, or in a pop-up warning dialog.
* @type {string}
* @default "This data not available while offline."
*/
offlineMessage?: string;

/**
* Message to display when this DataBoundComponent attempts to save data while the application
*  is offline.
* @type {string}
* @default "Data cannot be saved because you are not online"
*/
offlineSaveMessage?: string;

/**
* Indicates whether to add "drop values" to items dropped on this component, if both 
*      the source and target widgets are databound, either to the same DataSource or 
*      to different DataSources that are related via a foreign key. "Drop values" are 
*      properties of the dropped item that you wish to change (and persist) as a 
*      result of the item being dropped on this grid.
*      
*      If this value is true and this component is databound, DataBoundComponent.getDropValues will 
*      be called for every databound item dropped on this grid, and an update performed
*      on the item
* @type {boolean}
* @default true
*/
addDropValues?: boolean;

/**
* When an item is dropped on this component, and DataBoundComponent.addDropValues is true and both 
*      the source and target widgets are databound, either to the same DataSource or 
*      to different DataSources that are related via a foreign key, this object 
*      provides the "drop values" that SmartClient will apply to the dropped object 
*      before updating it.
*      
*      If this property is not defined, SmartClient defaults to returning the selection
*      criteria currently in place for this component. Thus, any databound items (for example, 
*      rows from other grids bound to the same DataSource) dropped on the grid will,
*      by default, be subjected to an update that makes them conform to the grid's 
*      current filter criteria.
* @type {object}
* @default null
*/
dropValues?: object;

/**
* Does this component represent singular or multiple "records" objects?
*  Options are "multiple", "single" or "either".
*  
*  This property is used within a ValuesManager and also controls how auto-population
*  works if the component is in a Canvas.dataContext and Canvas.autoPopulateData
*  is set.
* @type {string}
* @default "multiple"
*/
dataArity?: string;

/**
* Indicates whether or not this component will load its data 
*  DataSource.progressiveLoading.
* @type {boolean}
* @default null
*/
progressiveLoading?: boolean;

/**
* If this component is bound to a dataSource, this attribute may be specified to customize
*  what fields from the dataSource may be edited by default. For example the SearchForm
*  class has this attribute set to "canFilter" which allows search forms to edit
*  dataSource fields marked as canEdit:false (but not those marked as
*  canFilter:false).
*  
*  Note that if canEdit is explicitly specified on a field in 
*  the DataBoundComponent.fields array, that property will be respected in preference to 
*  the canEditAttribute value. (See FormItem.canEdit, ListGridField.canEdit).
*  Also note that individual dataBoundComponents may have additional logic around whether a field
*  can be edited - for example ListGrid.canEditCell may be overridden.
* @type {Identifier}
* @default "canEdit"
*/
canEditFieldAttribute?: Identifier;

/**
* Should DataBoundComponent.getFieldState and DataBoundComponent.setFieldState omit state information
*  for hidden fields by default?
* @type {boolean}
* @default false
*/
sparseFieldState?: boolean;

/**
* The useFlatFields flag causes all simple type fields anywhere in a nested
*  set of DataSources to be exposed as a flat list for form binding. 
*  
*  useFlatFields is typically used with imported metadata, such as 
*  XMLTools.loadXMLSchema from a 
*  XMLTools.loadWSDL, as a means of eliminating levels of XML
*  nesting that aren't meaningful in a user interface, without the cumbersome and fragile
*  process of mapping form fields to XML structures.
*  
*  For example, having called WebService.getInputDS to retrieve the input message
*  schema for a web service operation whose input message looks like this:
*  
*  &lt;FindServices&gt;
*    &lt;searchFor&gt;search text&lt;/searchFor&gt;
*    &lt;Options&gt;
*      &lt;caseSensitive&gt;false&lt;/caseSensitive&gt;
*    &lt;/Options&gt;
*    &lt;IncludeInSearch&gt;
*      &lt;serviceName&gt;true&lt;/serviceName&gt;
*      &lt;documentation&gt;true&lt;/documentation&gt;
*      &lt;keywords&gt;true&lt;/keywords&gt;
*    &lt;/IncludeInSearch&gt;
*  &lt;/FindServices&gt;
*  
*  Setting useFlatFields on a DynamicForm that is bound to this input
*  message schema would result in 5 FormItem reflecting the 5 simple type
*  fields in the message.
*  
*  For this form, the result of DynamicForm.getValues might look
*  like:
*  
*  {
*   searchFor: "search text",
*   caseSensitive: false,
*   serviceName: true,
*   documentation : true,
*   keywords : true
*  }
*  When contacting a WebService, these values can be automatically
*  mapped to the structure of the input message for a web service operation by setting
*  WSRequest.useFlatFields (for use with WebService.callOperation) or by setting
*  DSRequest.useFlatFields (for use with a DataSource that is
*  wsdlBinding via
*  OperationBinding.wsOperation). 
*  
*  Using these two facilities in conjunction (component.useFlatFields and
*  request.useFlatFields) allows gratuitous nesting to be consistently bypassed in both the user
*  presentation and when providing the data for XML messages.
*  
*  You can also set OperationBinding.useFlatFields to automatically enable 
*  "flattened" XML serialization (request.useFlatFields) for all DataSource requests of a
*  particular operationType.
*  
*  Note that useFlatFields is not generally recommended for use with structures
*  where multiple simple type fields exist with the same name, however if used with such a
*  structure, the first field to use a given name wins. "first" means the first field
*  encountered in a depth first search. "wins" means only the first field will be present as a
*  field when data binding.
* @type {boolean}
* @default null
*/
useFlatFields?: boolean;

/**
* If this attribute is set to false, any attributes in the component's values object that do
*  not map to a DataSourceField or FormItem will not 
*  be tracked when checking for changes. You should only set this flag to false if you know 
*  that your code does not store additional, non-field values in the component's data, or if 
*  you do store such values, but you don't care that they are not checked for changes. 
*  This flag is primarily provided to avoid performance issues in cases where developers are
*  storing large numbers of extra attributes in component data; generally speaking, you should
*  only consider setting it to false if you have a use case like this.
*  
*  Note, even with this flag set to false, these extra values will still be managed and stored 
*  by SmartClient; they just will not be checked when the component's values are inspected to 
*  see if they have changed. This may lead to methods like ListGrid.rowHasChanges
*  returning false when you are expecting it to return true. In this case, either switch this 
*  flag back to true (or just do not set it false), or provide a field definition for the 
*  affected attribute(s).
* @type {boolean}
* @default true
*/
canChangeNonFieldValues?: boolean;

/**
* Optional identifier for saved searches that should be applied to this component.
*  
*  By default SavedSearches are associated with a component via a 
*  AutoTest.getMinimalLocator. This property allows 
*  developers to override this behavior and explicitly associate a component with
*  a set of saved searches. This can provide a couple of benefits:
*  Firstly this ensures that saved searches will be unambiguously associated with
*  the particular component even if the page changes such that a stored minimal
*  locator would no longer applied to the component, without requiring 
*  an explicit Canvas.ID.
*  Secondly this allows the same set of saved searches to be applied to 
*  more than one component on a page. This may be valueable for cases where
*  the same information from the same dataSource is presented to users in multiple
*  places.
* @type {string}
* @default null
*/
savedSearchId?: string;

/**
* SavedSearches are associated with a component by an id string value calculated
*  as described in SavedSearches.getSavedSearchId.
*  
*  If an application changes the dataSource associated with a component at runtime 
*  via DataBoundComponent.setDataSource, if this id is not also modified, any previously created
*  savedSearches will still be associated with the component, but may not be applicable
*  to the new DataSource.
*  
*  SmartClient has logic to automatically detect this case and log a warning by default.
*  This property may be set to false to suppress this warning - useful for cases where
*  you want to use the same set of stored SavedSearches even after a component's dataSource
*  has been changed.
* @type {boolean}
* @default true
*/
logSavedSearchIdWarningOnDSChange?: boolean;

/**
* If true, when this component is first drawn, automatically call this.fetchData().
*  Criteria for this fetch may be picked up from DataBoundComponent.initialCriteria, and textMatchStyle may
*  be specified via ListGrid.autoFetchTextMatchStyle.
*  
*  NOTE: if autoFetchData is set, calling
*  ListGrid.fetchData before draw will cause two requests to be issued, one from the manual
*  call to fetchData() and one from the autoFetchData setting. The second request will use
*  only DataBoundComponent.initialCriteria and not any other criteria or settings from the first request.
*  Generally, turn off autoFetchData if you are going to manually call ListGrid.fetchData at any time.
* 
*  Note: If you are using saved searches - either via SavedSearchItem or ListGrid.saveDefaultSearch, autoFetchData
*  will be automatically suspended and replaced with the saved criteria/view state, if applicable.
* @type {boolean}
* @default false
*/
autoFetchData?: boolean;

/**
* Criteria to be used when DataBoundComponent.autoFetchData is set.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
initialCriteria?: Criteria;

/**
* Criteria that are never shown to or edited by the user and are cumulative with any criteria
*  provided via DataBoundComponent.initialCriteria and related methods.
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Canvas.ruleScope.
* @type {Criteria}
* @default null
*/
implicitCriteria?: Criteria;

/**
* Adds an item to the header context menu allowing users to launch a dialog to define
*  grid hilites using the HiliteEditor.
*  
*  User-added hilites can be persisted via DataBoundComponent.getHiliteState and 
*  DataBoundComponent.setHiliteState.
* @type {boolean}
* @default false
*/
canEditHilites?: boolean;

/**
* Hilites to be applied to the data for this component. See hiliting.
* @type {Array<Partial<Hilite>>}
* @default null
*/
hilites?: Array<Partial<Hilite>>;

/**
* Specifies a list of icons that can be used in DataBoundComponent.editHilites.
*  
*  hiliteIcons should be specified as an Array of SCImgURL.
*  When present, the hilite editing interface shown when DataBoundComponent.editHilites is called 
*  will offer the user a drop down for picking one of these icons when defining either a 
*  simple or advanced hilite rule.
*  
*  If the user picks an icon, the created hiliting rule will have Hilite.icon set to 
*  the chosen icon. DataBoundComponent.hiliteIconPosition controls where the icon will 
*  appear for that field -- the default is that it appears in front of the normal cell content.
*  This can also be overridden at the field level.
* @type {Array<Partial<string>>}
* @default "["[SKINIMG]/Dialog/notify.png", "[SKINIMG]/Dialog/warn.png", "[SKINIMG]/actions/approve.png"]"
*/
hiliteIcons?: Array<Partial<string>>;

/**
* When DataBoundComponent.hiliteIcons are present, where the hilite icon will be placed 
*  relative to the field value. See HiliteIconPosition. Can be overridden at the
*  field level.
* @type {HiliteIconPosition}
* @default "before"
*/
hiliteIconPosition?: HiliteIconPosition;

/**
* Default width and height of DataBoundComponent.hiliteIcons for this component.
*  Can be overridden at the component level via explicit 
*  DataBoundComponent.hiliteIconWidth and
*  DataBoundComponent.hiliteIconHeight, or at the field level via 
*  ListGridField.hiliteIconSize,
*  ListGridField.hiliteIconWidth and 
*  ListGridField.hiliteIconHeight
* @type {number}
* @default "12"
*/
hiliteIconSize?: number;

/**
* Width for hilite icons for this component.
*  Overrides DataBoundComponent.hiliteIconSize.
*  Can be overridden at the field level.
* @type {number}
* @default null
*/
hiliteIconWidth?: number;

/**
* Height for hilite icons for this listGrid.
*  Overrides DataBoundComponent.hiliteIconSize.
*  Can be overridden at the field level
* @type {number}
* @default null
*/
hiliteIconHeight?: number;

/**
* How much padding should there be on the left of DataBoundComponent.hiliteIcons by default?
*  Can be overridden at the field level
* @type {number}
* @default "2"
*/
hiliteIconLeftPadding?: number;

/**
* How much padding should there be on the right of DataBoundComponent.hiliteIcons by default?
*  Can be overridden at the field level
* @type {number}
* @default "2"
*/
hiliteIconRightPadding?: number;

/**
* Marker that can be set on a record to flag that record as hilited. Should be set to a value
*  that matches Hilite.id for a hilite defined on this component.
* @type {string}
* @default "_hilite"
*/
hiliteProperty?: string;

/**
* Initial hilite state for the grid. 
*  
*  ListGrid.viewState can be used to initialize all view properties of the grid.
*  When doing so, hiliteState is not needed because viewState
*  includes it as well. If both are provided, hiliteState has priority for
*  hilite state.
*  
*  
*  To retrieve current state call DataBoundComponent.getHiliteState.
*  
* @type {string}
* @default null
*/
hiliteState?: string;

/**
* The strategy to use when generating names for new fields in this component. The default 
*  strategy, "simple", combines the field-type with an index maintained by field-type and 
*  component instance. For example, "formulaField1".
* @type {FieldNamingStrategy}
* @default "simple"
*/
fieldNamingStrategy?: FieldNamingStrategy;

/**
* Text for a menu item allowing users to edit grid highlights.
* @type {string}
* @default "Edit Highlights..."
*/
editHilitesText?: string;

/**
* The title for the DataBoundComponent.editHilites dialog.
* @type {string}
* @default "Edit Highlights"
*/
editHilitesDialogTitle?: string;

/**
* The title for the DataBoundComponent.fieldEditorWindow used to edit calculated
*  fields.
*  
*  This is a dynamic string - text within &#36;{...} are dynamic variables and will
*  be evaluated as JS code whenever the message is displayed.
*  
*  Two dynamic variables are available - "builderType", either Formula or Summary, and 
*  "fieldTitle", which is the title of the calculated field being edited.
* @type {HTMLString}
* @default "${builderType} Editor [${fieldTitle}]"
*/
fieldEditorWindowTitle?: HTMLString;

/**
* The Window used to edit calculated fields for this component.
* @type {Window}
* @default null
*/
fieldEditorWindow?: Window;

/**
* This component's HiliteEditor instance used to allow the user to create, modify, or delete
*  hilites.
*  
*  The following autoChildUsage apply:
*  
*  
* - DataBoundComponent.hiliteIcons for HiliteEditor.hiliteIcons
*  
* @type {HiliteEditor}
* @default null
*/
hiliteEditor?: HiliteEditor;

/**
* Indicates what to do with data dragged into another DataBoundComponent. See
*  DragDataAction type for details.
* @type {DragDataAction}
* @default "move"
*/
dragDataAction?: DragDataAction;

/**
* CSS Style to apply to the drag tracker when dragging occurs on this component.
* @type {CSSStyleName}
* @default "gridDragTracker"
*/
dragTrackerStyle?: CSSStyleName;

/**
* If the result of a formula evaluation is invalid (specifically, if isNaN(result)==true),
*  badFormulaResultValue is displayed instead. The default value is ".".
* @type {string}
* @default "."
*/
badFormulaResultValue?: string;

/**
* If a summary format string contains an invalid field reference, replace the reference
*  with the missingSummaryFieldValue. The default value is "-".
* @type {string}
* @default "-"
*/
missingSummaryFieldValue?: string;

/**
* Adds an item to the header context menu allowing users to launch a dialog to define a new
*  field based on values present in other fields, using the FormulaBuilder.
*  
*  User-added formula fields can be persisted via ListGrid.getFieldState and 
*  ListGrid.setFieldState.
* @type {boolean}
* @default false
*/
canAddFormulaFields?: boolean;

/**
* Text for a menu item allowing users to add a formula field
* @type {string}
* @default "Add formula column..."
*/
addFormulaFieldText?: string;

/**
* Text for a menu item allowing users to edit a formula field
* @type {string}
* @default "Edit formula..."
*/
editFormulaFieldText?: string;

/**
* Text for a menu item allowing users to remove a formula field
* @type {string}
* @default "Remove formula"
*/
removeFormulaFieldText?: string;

/**
* Adds an item to the header context menu allowing users to launch a dialog to define a new
*  text field that can contain both user-defined text and the formatted values present in other 
*  fields, using the SummaryBuilder.
*  
*  User-added summary fields can be persisted via ListGrid.getFieldState and 
*  ListGrid.setFieldState.
* @type {boolean}
* @default false
*/
canAddSummaryFields?: boolean;

/**
* Text for a menu item allowing users to add a formula field
* @type {string}
* @default "Add summary column..."
*/
addSummaryFieldText?: string;

/**
* Text for a menu item allowing users to edit the formatter for a field
* @type {string}
* @default "Edit summary format..."
*/
editSummaryFieldText?: string;

/**
* Text for a menu item allowing users to remove a summary field
* @type {string}
* @default "Remove summary column..."
*/
removeSummaryFieldText?: string;

/**
* When assembling a value for a DataBoundComponent.canAddSummaryFields, if a referenced
*  field is hilited, should the hilite HTML be included in the summary field value?
*  
*  To control hilites showing in group summaries, see ListGrid.showHilitesInGroupSummary.
* @type {boolean}
* @default true
*/
includeHilitesInSummaryFields?: boolean;

/**
* Best field to use for a user-visible title for an individual record from this
*  component.
*  
*  This attribute has the same function as DataSource.iconField but can be
*  set for a component with no dataSource, or can be used to override the dataSource setting.
* @type {string}
* @default null
*/
titleField?: string;

/**
* Designates a field of FieldType:"image" as the field to use when rendering a
*  record as an image, for example, in a TileGrid. 
*  
*  This attribute has the same function as DataSource.iconField but can be
*  set for a component with no dataSource, or can be used to override the dataSource setting.
* @type {string}
* @default null
*/
iconField?: string;

/**
* Name of the field that has the second most pertinent piece of textual information in the
*  record, for use when a DataBoundComponent needs to show a short summary of a record.
*  
*  This attribute has the same function as DataSource.infoField but can be
*  set for a component with no dataSource, or can be used to override the dataSource setting.
* @type {string}
* @default null
*/
infoField?: string;

/**
* Name of the field that has the most pertinent numeric, date, or enum value, for use when a
*  DataBoundComponent needs to show a short summary of a record.
*  
*  This attribute has the same function as DataSource.dataField but can be
*  set for a component with no dataSource, or can be used to override the dataSource setting.
* @type {string}
* @default null
*/
dataField?: string;

/**
* Name of the field that has a long description of the record, or has the primary text data
*  value for a record that represents an email message, SMS, log or similar.
*  
*  This attribute has the same function as DataSource.descriptionField but can be
*  set for a component with no dataSource, or can be used to override the dataSource setting.
* @type {string}
* @default null
*/
descriptionField?: string;

/**
* The message to display to the user if an export of a DataBoundComponent's data is attempted
*  while the DataBoundComponent's data is null or an empty list.
* @type {string}
* @default "You are attempting to export an empty dataset"
*/
emptyExportMessage?: string;

/**
* Properties to apply to the MultiSortDialog which gets automatically generated
*  when DataBoundComponent.askForSort is called.
*  
*  See also ListGrid.showHeaderSpanTitlesInSortEditor and ListGrid.sortEditorSpanTitleSeparator
* @type {Partial<MultiSortDialog>}
* @default null
*/
multiSortDialogProperties?: Partial<MultiSortDialog>;

/**
* Class level defaults to apply to the MultiSortDialog which gets automatically
*  generated when DataBoundComponent.askForSort is called.
*  
*  See also ListGrid.showHeaderSpanTitlesInSortEditor and ListGrid.sortEditorSpanTitleSeparator
* @type {Partial<MultiSortDialog>}
* @default null
*/
multiSortDialogDefaults?: Partial<MultiSortDialog>;

/**
* For databound components that support editing, the error message for a failed validator
*  that does not specify its own errorMessage.
* @type {HTMLString}
* @default "Invalid value"
*/
unknownErrorMessage?: HTMLString;

/**
* A message to display to the user if server-side validation fails with an error but the
*  server did not provide an error message
* @type {string}
* @default "Error during validation; no error details were provided"
*/
noErrorDetailsMessage?: string;



/**
*  This method is invoked on the source component whenever a drag operation or 
*  DataBoundComponent.transferSelectedData completes. This method is called when the entire chain of 
*  operations - including, for databound components, server-side updates and subsequent 
*  integration of the changes into the client-side cache - has completed.
*  There is no default implementation of this method; you are intended to override it if you 
*  are interested in being notified when drag operations complete.
* 
* 
*/
dragComplete(): void;

/**
*  This method is invoked whenever a drop operation or DataBoundComponent.transferSelectedData 
*  targeting this component completes. A drop is considered to be complete when all the client-
*  side transfer operations have finished. This includes any server turnarounds SmartClient 
*  needs to make to check for duplicate records in the target component; it specifically does 
*  not include any add or update operations sent to the server for databound components. If 
*  you want to be notified when the entire drag operation - including server updates and cache
*  synchronization - has completed, override DataBoundComponent.dragComplete
*  on the source component.
*  There is no default implementation of this method; you are intended to override it if you 
*  are interested in being notified when drop operations complete.
* 
* 
* @param {Array<Partial<Record>>} The list of records actually transferred to          this component (note that this is not necessarily the same thing as the           list of records dragged out of the source component because it doesn't           include records that were excluded because of collisions with existing           records)
*/
dropComplete(transferredRecords:Array<Partial<Record>>): void;

/**
*  For a component with a specified DataSource, find the associated dataSource field object
*  from a specified DataPath.
* 
* @param {DataPath} dataPath for which the field definition should be returned.
*/
getDataPathField(dataPath:DataPath): DataSourceField;

/**
*  Can the field be edited? This base method always returns false, but it's
*  overridden by subclasses such as DynamicForm and ListGrid.
* 
* 
* @param {object | number | string} field object or identifier
*/
fieldIsEditable(field:object | number | string): boolean;

/**
*  Return a field by a field index or field name.
* 
* 
* @param {FieldName | number} field index or field.name
*/
getField(fieldID:FieldName | number): object;

/**
*  Find the index of a currently visible field.
* 
* 
* @param {FieldName | string} field name or field
*/
getFieldNum(fieldName:FieldName | string): Integer;

/**
*  Returns a snapshot of the current view state of this Component.
*  The default implementation at the DataBoundComponent level will track 
*  DataBoundComponent.getFieldState only. See documentation on specific component classes 
*  for whether additional configuration is included in the viewState.
*  
*  The viewState subsystem is typically used to store and reapply user preferences, for example
*  after fields are shown or hidden using a FieldPickerWindow.
*  The viewState returned by this method may be stored over page-reloads and then applied
*  to a component at runtime via DataBoundComponent.setViewState.
* 
*/
getViewState(): ViewState;

/**
*  Reset this component's view state to match the ViewState object passed in.
*  Used to restore previous state retrieved from the component by a call to
*  DataBoundComponent.getViewState.
* 
* 
* @param {ViewState} Object describing the desired view state for the component
*/
setViewState(viewState:ViewState): void;

/**
*  Should the title be included in field state for the field passed in.
*  Default implementation will return true if ListGrid.canEditTitles 
*  is true
*  
*  
*  This method may be overridden to customize this behavior. For example
*  the following implementation would include titles for fields where 
*  an explicitly component-level field-title was set that differed from 
*  the DataSourceField.title for the underlying dataSource field:
*  
*  
*  shouldIncludeTitleInFieldState : function (field, sparse) {
*    var dsField = this.getDataSource().getField(field.name);
*    if (dsField != null && dsField.title != field.title) return true;
*    return false;
*  }
*  
*    
*  
* 
* @param {ListGridField} field to provide field state for
* @param {boolean} true if DataBoundComponent.getFieldState was requested.
*/
shouldIncludeTitleInFieldState(field:ListGridField, sparse:boolean): boolean;

/**
*  Retrieve a snapshot of current field state for this component. This includes an object for
*  each field, indicating its name, visibility and order, allowing you to track user 
*  preferences if they have been modified, for example via a FieldPickerWindow.
*  
*  The optional sparse parameter governs whether the returned field state
*  should omit state information for hidden fields. If this parameter is not passed
*  explicitly, field state will be sparse if DataBoundComponent.sparseFieldState
*  is true.
*  When applying sparse field state to a component via DataBoundComponent.setFieldState, any 
*  explicitly defined fields on the component that were not captured in the stored
*  state object will be hidden.
*  
*  Note that this method may be overridden by specific component types to handle
*  more than just field visibility and order. For example ListGrids support
*  extensive customization of fields' appearance and behavior by users and 
*  consequently tracks many variables in the field state 
*  (see the ListGrid.getFieldState for specifics).
*  
*  Note that, as a shorthand, for fields that are visible but have no other state-variables, 
*  only the field-name is included in the field-state array - so this array can contain a mix 
*  of strings and objects which are interpreted automatically when the state is reapplied.
*  
* 
* @param {boolean=} If true, field state will be omitted for hidden fields.
* @param {FieldState} Current field state
*/
getFieldState(sparse?:boolean, return:FieldState): void;

/**
*  Apply a field state derived from DataBoundComponent.getFieldState to this component. This
*  will manage fields' visibility and order if they have been modified, for example
*  via a FieldPickerWindow.
*  
*  The optional isSparse parameter may be passed to indicate
*  whether the fieldState object is "sparse" - whether it includes explicit
*  state information for hidden fields. In this case any fields defined on the component
*  not explicitly included in the fieldState object will be hidden.
*  If isSparse is not explicitly passed as a parameter, 
*  sparseness will be assumed if DataBoundComponent.sparseFieldState
*  is true.
*  
*  Note that this method may be overridden by specific component types to handle
*  more than just field visibility and order. For example ListGrids support
*  extensive customization of fields' appearance and behavior by users and 
*  consequently tracks many variables in the field state 
*  (see the ListGrid.getFieldState for specifics).
*  
* 
* @param {FieldState} field state to apply
* @param {boolean=} If true, the fieldState passed in is assumed to be "sparse".  Any fields defined on this component without explicit field state values will be  hidden.
*/
setFieldState(fieldState:FieldState, isSparse?:boolean): void;

/**
*  This API is equivalent to List.find but searches for a matching record among already-loaded data only. 
*  Use ListGrid.fetchData to load data from the server.
* 
* @param {AdvancedCriteria} AdvancedCriteria to use with
*/
find(advancedCriteria:AdvancedCriteria): object;

/**
*  This API is equivalent to List.findAll but searches for a matching record among already-loaded data only. 
*  Use ListGrid.fetchData to load data from the server.
* 
* @param {AdvancedCriteria} AdvancedCriteria to use with
*/
findAll(advancedCriteria:AdvancedCriteria): Array<any>;

/**
*  This API is equivalent to List.findIndex but searches for a matching record among already-loaded data only. 
*  Use ListGrid.fetchData to load data from the server.
* 
* @param {AdvancedCriteria} AdvancedCriteria to use with
*/
findIndex(advancedCriteria:AdvancedCriteria): Integer;

/**
*  This API is equivalent to List.findNextIndex but searches for a matching record among already-loaded data only. 
*  Use ListGrid.fetchData to load data from the server.
* 
* @param {Integer} first index to consider
* @param {AdvancedCriteria} AdvancedCriteria to use with
* @param {Integer=} last index to consider
*/
findNextIndex(startIndex:Integer, advancedCriteria:AdvancedCriteria, endIndex?:Integer): Integer;

/**
*  Bind to a new DataSource.
*  
*  Like passing the "dataSource" property on creation, binding to a DataSource means that the
*  component will use the DataSource to provide default data for its fields.
*  
*  When binding to a new DataSource, if the component has any existing "fields" or has a dataset,
*  these will be discarded by default, since it is assumed the new DataSource may represent a
*  completely unrelated set of objects. If the old "fields" are still relevant, pass them to
*  setDataSource().
*  
* 
* @param {GlobalId | DataSource} DataSource to bind to
* @param {Array<Partial<DataSourceField>>=} optional array of fields to use
*/
setDataSource(dataSource:GlobalId | DataSource, fields?:Array<Partial<DataSourceField>>): void;

/**
*  Sends the current filter criteria and sort direction to the server, then exports data in the
*  requested DSRequest.exportAs.
*  
*  A variety of DSRequest settings, such as 
*  DSRequest.exportAs and DSRequest.exportFilename, affect the 
*  exporting process: see DSRequest.exportResults for further detail.
*  
*  Note that data exported via this method skips client-side fields defined only in
*  the component, excludes any client-side formatting and
*  relies on both the SmartClient server and server-side DataSources. To export client-data 
*  including client-only fields and with client-side formatting applied, 
*  see ListGrid.exportClientData, which still requires the
*  SmartClient server but does not rely on server-side DataSource definitions (.ds.xml files).
*  
*  For more information on exporting data, see DataSource.exportData.
* 
* 
* @param {DSRequest=} additional properties to set on the DSRequest                      that will be issued
* @param {DSCallback=} callback to invoke on completion. Note that this parameter                 only applies where DSRequest.exportToClient is                  explicitly set to false, because file downloads do not                  provide ordinary SmartClient callbacks
*/
exportData(requestProperties?:DSRequest, callback?:DSCallback): void;

/**
*  Returns an array of Alignment for this grid
* 
*/
getFieldAlignments(): Array<Partial<Alignment>>;

/**
*  Returns a copy of the DataBoundComponent.implicitCriteria currently applied to this
*  component.
* 
*/
getImplicitCriteria(): void;

/**
*  Setter for the DataBoundComponent.implicitCriteria attribute, which can be 
*  called directly at runtime to update the implicitCriteria on this component.
*  
*  If this component has fetched data before, a fetch will be issued if the new 
*  implicit-criteria is less restrictive when re-combined with any additional criteria applied 
*  to the component via other means.
*  
*  If data has never been fetched for this component, this method will not issue an initial 
*  fetch unless the initialFetch parameter is passed.
*  
* 
* @param {Criteria} non user-editable criteria to apply to this component
* @param {DSCallback=} callback to fire on successful fetch - will not fire if     willFetchData is false or if an initial fetch is required but     initialFetch was not passed
* @param {boolean=} flag to allow initial fetch if this grid has never     fetched data before - default behavior is not to perform an initial fetch
*/
setImplicitCriteria(criteria:Criteria, callback?:DSCallback, initialFetch?:boolean): boolean;

/**
*  Compares the specified criteria with the current criteria applied to this component's
*  data object and determines whether the new criteria could be satisfied from the currently
*  cached set of data, or if a new filter/fetch operation will be required.
*  
*  This is equivalent to calling this.data.willFetchData(...).
*  Always returns true if this component is not showing a set of data from the dataSource.
*  
*  Note that to predict correctly the decision that will be made by filter/fetch, you'll need to
*  pass the same TextMatchStyle that will be used by the future filter/fetch. Fetching
*  manually (e.g. ListGrid.fetchData) will by default use "exact" while filtering
*  (e.g. ListGrid.filterData) will by default use "substring". If the component
*  is configured for autofetch (i.e. ListGrid.autoFetchData: true), that will
*  use ListGrid.autoFetchTextMatchStyle, which defaults to "substring". If nothing/null
*  is passed for the style, this method assumes you want the style from the last filter/fetch.
*  
*  To determine what TextMatchStyle is being used, check the RPC Tab of the
*  debugging and check the relevant DSRequest. 
*  
* 
* @param {Criteria} new criteria to test.
* @param {TextMatchStyle=} New text match style. If not passed assumes    textMatchStyle will not be modified.
*/
willFetchData(newCriteria:Criteria, textMatchStyle?:TextMatchStyle): boolean;

/**
*  Attempt to find the record in the resultSet that has a primary key value that matches the 
*  passed in parameter value. Only the locally cached data will be searched. 
*  Checks only loaded rows and will not trigger a fetch. Returns null if there is no match, 
*  data is not loaded, or there is no ResultSet.dataSource.
*  
*  Note, if you pass a simple value to this method, it will be matched against the first 
*  primaryKey field. For DataSources with a composite primary key (multiple primaryKey fields),
*  pass
*  a criteria object containing just your primaryKeys, like this: 
*  { firstPkField: "value", secondPkField: 25 }.
*  
* 
* 
* @param {object} primary key value to search for
*/
findByKey(keyValue:object): Record;

/**
* 
*  Select/deselect a Record passed in explicitly, or by index.
* 
* 
* @param {Record | number} record (or row number) to select
* @param {boolean=} new selection state (if null, defaults to true)
*/
selectRecord(record:Record | number, newState?:boolean): void;

/**
*  Select a single Record passed in explicitly, or by index, and deselect everything else.
*  When programmatic selection of records is a requirement and 
*  ListGrid.selectionType is "single", use this method rather than 
*  DataBoundComponent.selectRecord to 
*  enforce mutually-exclusive record-selection.
* 
* 
* @param {Record | number} record (or row number) to select
*/
selectSingleRecord(record:Record | number): void;

/**
* 
*  Deselect a Record passed in explicitly, or by index.
*  
*  Synonym for selectRecord(record, false)
* 
* 
* @param {Record | number} record (or row number) to deselect
*/
deselectRecord(record:Record | number): void;

/**
* 
*  Select/deselect a list of Records passed in explicitly, or by index.
*  
*  Note that developers may wish to use DataBoundComponent.selectRange to select a single
*  contiguous range.
* 
* 
* @param {Array<Partial<Record>> | number} records (or row numbers) to select
* @param {boolean=} new selection state (if null, defaults to true)
*/
selectRecords(records:Array<Partial<Record>> | number, newState?:boolean): void;

/**
* 
*  Deselect a list of Records passed in explicitly, or by index.
*  
*  Synonym for selectRecords(records, false)
*  
*  Note that developers may wish to use DataBoundComponent.deselectRange to select a single
*  contiguous range.
* 
* 
* @param {Array<Partial<Record>> | number} records (or row numbers) to deselect
*/
deselectRecords(records:Array<Partial<Record>> | number): void;

/**
*  Select all records
* 
* 
*/
selectAllRecords(): void;

/**
* 
*  Deselect all records
* 
* 
*/
deselectAllRecords(): void;

/**
*  Select a contiguous range of records by index
* 
* @param {Integer} start of selection range
* @param {Integer} end of selection range (non-inclusive)
* @param {boolean=} new selection state (if null, defaults to true)
*/
selectRange(startRow:Integer, endRow:Integer, newState?:boolean): void;

/**
*  Deselect a contiguous range of records by index.
*  
*  This is a synonym for selectRange(startRow, endRow, false);
* 
* @param {Integer} start of selection range
* @param {Integer} end of selection range (non-inclusive)
*/
deselectRange(startRow:Integer, endRow:Integer): void;

/**
*  Whether at least one item is selected
* 
*/
anySelected(): boolean;

/**
*  Returns the number of selected records.
* 
* 
*/
getSelectionLength(): Integer;

/**
*  Return the set of hilite-objects currently applied to this DataBoundComponent. These
*  can be serialized for storage and then restored to a component later via 
*  DataBoundComponent.setHilites.
* 
* 
*/
getHilites(): Array<Partial<Hilite>>;

/**
*  Accepts an array of hilite objects and applies them to this DataBoundComponent. See also
*  DataBoundComponent.getHilites for a method of retrieving the hilite
*  array for storage, including hilites manually added by the user.
* 
* 
* @param {Array<Partial<Hilite>>} Array of hilite objects
*/
setHilites(hilites:Array<Partial<Hilite>>): void;

/**
*  For components that support hiliting, gets the current hilites encoded as a String, for saving.
* 
* 
*/
getHiliteState(): string;

/**
*  For components that support hiliting, sets the current hilites based on a hiliteState String previously returned from
*  DataBoundComponent.getHiliteState.
* 
* @param {string} hilites state encoded as a String
*/
setHiliteState(hiliteState:string): void;

/**
*  Enable / disable a DataBoundComponent.hilites
* 
* 
* @param {string} ID of hilite to enable
* @param {boolean=} new enabled state to apply - if null, defaults to true
*/
enableHilite(hiliteID:string, enable?:boolean): void;

/**
*  Disable a hilite
* 
* 
* @param {string} ID of hilite to disable
*/
disableHilite(hiliteID:string): void;

/**
*  Enable all hilites.
* 
* 
* @param {boolean=} new enabled state to apply - if null, defaults to true
*/
enableHiliting(enable?:boolean): void;

/**
*  Disable all hilites.
* 
* 
*/
disableHiliting(): void;

/**
*  Shows a HiliteEditor interface allowing end-users to edit
*  the data-hilites currently in use by this DataBoundComponent.
* 
* 
*/
editHilites(): void;

/**
* 
*  Transfer a list of Records from another component (does not have to be a databound
*  component) into this component. This method is only applicable to list-type components,
*  such as ListGrid, TreeGrid or TileGrid
*  
*  This method implements the automatic drag-copy and drag-move behaviors of components like
*  ListGrid, and calling it is equivalent to completing a drag and drop of the
*  dropRecords.
*  
*  Note that this method is asynchronous - it may need to perform server turnarounds to prevent
*  duplicates in the target component's data. If you wish to be notified when the transfer 
*  process has completed, you can either pass the optional callback to this method or implement
*  the DataBoundComponent.dropComplete method on this component.
*  
*  See also DataBoundComponent.transferSelectedData.
* 
* 
* @param {Array<Partial<Record>>} Records to transfer to this component
* @param {Record} The target record (eg, of a drop interaction), for context
* @param {Integer} Insert point in this component's data for the transferred records
* @param {Canvas} The databound or non-databound component from which the records              are to be transferred.
* @param {Callback=} optional callback to be fired when the transfer process has completed
*/
transferRecords(dropRecords:Array<Partial<Record>>, targetRecord:Record, index:Integer, sourceWidget:Canvas, callback?:Callback): void;

/**
*  Returns the "drop values" to apply to a record dropped on this component prior to update. Only
*  applicable to databound components - see DataBoundComponent.dropValues for more details. If multiple records 
*  are being dropped, this method is called for each of them in turn.
*  
*  The default implementation of this method returns the following:
*  
*  
* - Nothing, if DataBoundComponent.addDropValues is false
*  
* - dropValues, if that property is set. If the component's criteria object is applicable (as explained
*  in the next item), it is merged into dropValues, with properties in dropValues taking precedence.
*  
* - The component's criteria object, if the most recent textMatchStyle for the component was "exact" 
*    and it is simple criteria (ie, not an AdvancedCriteria object)
*  
* - Otherwise nothing
*  
*  
*  You can override this method if you need more complex setting of drop values than can be 
*  provided by simply supplying a dropValues object.
*  
* 
* @param {Record} record being dropped
* @param {DataSource} dataSource the record being dropped is bound to
* @param {Record} record being dropped on
* @param {Integer} index of record being dropped on
* @param {Canvas} widget where dragging began
*/
getDropValues(record:Record, sourceDS:DataSource, targetRecord:Record, index:Integer, sourceWidget:Canvas): object;

/**
* 
*  During a drag-and-drop interaction, this method is called to transfer a set of records that
*  were dropped onto some other component. This method is called after the set of records has
*  been copied to the other component. Whether or not this component's data is modified is 
*  determined by the value of DataBoundComponent.dragDataAction.
*  
*  With a dragDataAction of "move", a databound component will issue "remove"
*  dsRequests against its DataSource to actually remove the data, via
*  DataSource.removeData.
* 
* 
*/
transferDragData(): Array<any>;

/**
* 
*  During a drag-and-drop interaction, this method returns the set of records being dragged out
*  of the component. In the default implementation, this is the list of currently selected
*  records.
*  
*  This method is consulted by ListGrid.willAcceptDrop.
*  
*  NOTE: If this component is a Tree.isMultiLinkTree TreeGrid, 
*  this method returns a list of NodeLocators rather than a list of records. Each 
*  nodeLocator contains a pointer to the associated record in its node
*  property.
* 
* 
* @param {DataBoundComponent} source component from which the records will be transferred
*/
getDragData(source:DataBoundComponent): Array<Partial<Record>>;

/**
*  Simulates a drag / drop type transfer of the selected records in some other component to this
*  component, without requiring any user interaction. This method acts on the dropped records 
*  exactly as if they had been dropped in an actual drag / drop interaction, including any 
*  special databound behavior invoked by calling
*  DataBoundComponent.getDropValues for each dropped record.
*  
*  To transfer all data in, for example, a ListGrid, call 
*  ListGrid.selectAllRecords first.
*  
*  Note that drag/drop type transfers of records between components are asynchronous operations:
*  SmartClient may need to perform server turnarounds to establish whether dropped records 
*  already exist in the target component. Therefore, it is possible to issue a call to 
*  transferSelectedData() and/or the ListGrid.drop method of a databound 
*  component whilst a transfer is still active. When this happens, SmartClient adds the 
*  second and subsequent transfer requests to a queue and runs them one after the other. If 
*  you want to be notified when a transfer process has actually completed, either provide a 
*  callback to this method or implement DataBoundComponent.dropComplete.
*  
*  See the dragging documentation for an overview of list grid drag/drop data
*  transfer.
*  
* 
* @param {DataBoundComponent} source component from which the records will be transferred
* @param {Integer=} target index (drop position) of the rows within this grid.
* @param {Callback=} optional callback to be fired when the transfer process has            completed. The callback will be passed a single parameter "records",            the list of records actually transferred to this component.
*/
transferSelectedData(source:DataBoundComponent, index?:Integer, callback?:Callback): void;

/**
*  Sets the custom tracker HTML to display next to the mouse when the user initiates a drag
*  operation on this component. Default implementation will examine ListGrid.dragTrackerMode
*  and set the custom drag tracker to display the appropriate HTML based on the selected record.
*  
*  To display custom drag tracker HTML, this method may be overridden - call 
*  EventHandler.setDragTracker to actually update the drag tracker HTML.
* 
*/
setDragTracker(): boolean;

/**
*  Convenience method to display a FormulaBuilder to create a new Formula Field. This 
*  is equivalent to calling DataBoundComponent.editFormulaField with 
*  no parameter.
* 
* 
*/
addFormulaField(): void;

/**
*  Method to display a FormulaBuilder to edit a formula Field. If the function is called
*  without a parameter, a new field will be created when the formula is saved.
* 
* 
* @param {DBCField} Field to edit or null to add a new formula field
*/
editFormulaField(field:DBCField): void;

/**
*  Get the computed value of a DataBoundComponent.canAddFormulaFields.
* 
* @param {DBCField} field that has a formula
* @param {Record} record to use to compute formula value
*/
getFormulaFieldValue(field:DBCField, record:Record): Double | string;

/**
*  Convenience method to display a SummaryBuilder to create a new Summary Field. This 
*  is equivalent to calling DataBoundComponent.editSummaryField with 
*  no parameter.
* 
* 
*/
addSummaryField(): void;

/**
*  Method to display a SummaryBuilder to edit a Summary Field. If the function is called
*  without a parameter, a new field will be created when the summary is saved.
* 
* 
* @param {DBCField} Field to edit or null to add a new summary column
*/
editSummaryField(field:DBCField): void;

/**
*  Get the computed value of a DataBoundComponent.canAddSummaryFields.
* 
* @param {DBCField} field that has a summary format
* @param {Record} record to use to compute formula value
*/
getSummaryFieldValue(field:DBCField, record:Record): string;

/**
*  When assembling a value for a DataBoundComponent.canAddSummaryFields, if a referenced
*  field is hilited, should the hilite HTML be included in the summary field value?
*  
*  Example use case: Consider a grid containing a numeric field, and a summary field
*  which contains some string value, plus the contents of the numeric field.
*  If a hilite is defined for the grid which turns the numeric field text red when
*  the value is negative, this property will govern whether the number will also be
*  rendered in red within the summary field cells. Any other text in the summary field
*  cells would not be effected by this hilite.
*  
*  Default implementation returns DataBoundComponent.includeHilitesInSummaryFields.
*  
*  To control hilites showing in group summaries, see ListGrid.showHilitesInGroupSummary.
* 
* 
* @param {string} name of the summary field
* @param {string} name of the field referenced by this summary
*/
shouldIncludeHiliteInSummaryField(summaryFieldName:string, usedFieldName:string): boolean;

/**
*  Get the index of the provided record.
*  
*  Override in subclasses to provide more specific behavior, for instance, when data holds a
*  large number of records
* 
* 
* @param {Record} the record whose index is to be retrieved
*/
getRecordIndex(record:Record): number;

/**
*  Get the value of the titleField for the passed record
*  
*  Override in subclasses 
* 
* 
* @param {Record} the record whose index is to be retrieved
*/
getTitleFieldValue(record:Record): string;

/**
*  Method to return the fieldName which represents the "title" for records in this
*  Component.
*  If this.titleField is explicitly specified it will always be used.
*  Otherwise, default implementation will check DataSource.titleField for databound
*  compounds.
*  For non databound components returns the first defined field name of "title",
*  "name", or "id" where the field is visible. If we don't find any
*  field-names that match these titles, the first field in the component will be used instead.
* 
*/
getTitleField(): string;

/**
*  Return all CSS style declarations associated with the hilites of a record's field.
* 
* @param {Record} * @param {string} if set, returned CSS will be appended to this text
* @param {DBCField} field object identifying whose CSS is to be returned
*/
getRecordHiliteCSSText(record:Record, cssText:string, field:DBCField): string;

/**
*  Return the SortSpecifiers representing the current sort configuration of this
*  component.
* 
*/
getSort(): Array<Partial<SortSpecifier>>;

/**
*  Sort this component by a list of SortSpecifiers. If the component's data is not a 
*  ResultSet, only the first specifier is applied.
*  
* 
* @param {Array<Partial<SortSpecifier>>} List of SortSpecifier objects, one  per sort-field and direction
*/
setSort(sortSpecifiers:Array<Partial<SortSpecifier>>): void;

/**
*  Show a dialog to configure the sorting of multiple fields on this component. Calls through
*  to MultiSortDialog.askForSort, passing this component as the fieldSource and the
*  current DataBoundComponent.getSort if there is one.
*  
*  The generated multiSortDialog can be customized via DataBoundComponent.multiSortDialogDefaults, 
*  DataBoundComponent.multiSortDialogProperties.
* 
* 
*/
askForSort(): void;

/**
*  Returns true if the component's current data model is marked as offline. This does not 
*  necessarily mean that the component has no data; it may have data that was supplied from
*  the Offline.
* 
*/
isOffline(): boolean;

/**
*  Shows a FieldPicker interface allowing end-users to edit
*  the fields currently shown by this DataBoundComponent.
* 
* 
*/
editFields(): void;

/**
*  Notification method fired when a user-generated field is added to this component via
*  DataBoundComponent.editFormulaField or DataBoundComponent.editSummaryField.
*  
*  Returning false from this method will prevent the field being added at all. Note that
*  this also provides an opportunity to modify the generated field object - any changes
*  made to the field parameter will show up when the field is displayed in the ListGrid.
* 
* 
* @param {ListGridField} User generated summary or formula field
*/
userAddedField(field:ListGridField): boolean;

/**
*  Called when the selection changes. Note that this method fires exactly once for any given
*  change to the selection unlike the ListGrid.selectionChanged event.
*  
*  This event is fired once after selection/deselection has completed. The result is
*  one event per mouse-down event. For a drag selection there will be two events fired:
*  one when the first record is selected and one when the range is completed.
*  
*  This event is also fired when selection is updated by a direct call to one of the
*  DataBoundComponent select/deselect methods. Calls on the Selection
*  object do not trigger this event.
* 
* 
* @param {object} first selected record in the selection, if any, which may or may not be the first record in sort order if the DataBoundComponent is sorted. This parameter is typically used when only one record can be selected at a time.
* @param {Array<Partial<object>>} List of records that are now selected
*/
selectionUpdated(record:object, recordList:Array<Partial<object>>): void;




}

/**
* An interface for an ordered collection of items.
*  
*  This is the interface that is expected by list-oriented display components such as the
*  ListGrid. The JavaScript native Array object is retrofitted to support the List interface.
*  Also, a valid List can be created by mixing the List interface into any class that supports:
*  
*  
* - for read-only support: get(position), getLength()
*  
* - for modifiable support: set(position), addAt(object, position), removeAt(position)
*  
*  
*  NOTE: this interface is compatible with the java.util.List interface, except that:
*  
*  
* - for removal by index, removeAt() must be called instead of remove(). In Java, remove()
*  is an overloaded method that takes either an int or Object, whereas in JavaScript, a
*  Number is an Object.
*  
* - Iterators don't exist.
*  
*  Some methods from the Java List interface have been omitted from the documentation to avoid
*  redundancy.
*/
interface List {


/**
*  Return the item at a particular position
* 
* @param {number} position of the element to get
*/
get(pos:number): object;

/**
*  Return the number of items in this list
* 
* 
*/
getLength(): number;

/**
*  Return whether or not this array is empty
* 
* 
*/
isEmpty(): boolean;

/**
*  Return the first item in this list
* 
* 
*/
first(): any;

/**
*  Return the last item in this list
* 
* 
*/
last(): any;

/**
*  Return the position in the list of the first instance of the specified object.
*  
*  If pos is specified, starts looking after that position.
*  
*  Returns -1 if not found.
* 
* 
* @param {any} object to look for
* @param {number=} earliest index to consider
* @param {number=} last index to consider
*/
indexOf(obj:any, pos?:number, endPos?:number): number;

/**
*  Return the position in the list of the last instance of the specified object.
*  
*  If pos is specified, starts looking before that position.
*  
*  Returns -1 if not found.
* 
* 
* @param {any} object to look for
* @param {number=} last index to consider
* @param {number=} earliest index to consider
*/
lastIndexOf(obj:any, pos?:number, endPos?:number): number;

/**
*  Find the index of the first Object where property == value in the object.
*  
*  Pass an Object instead to match multiple properties.
*  
*  Note: for string values, matches are case sensitive.
* 
* 
* @param {string | object | AdvancedCriteria} property to match, or if an Object is passed, set of                    properties and values to match
* @param {any=} value to compare against (if propertyName is a string)
*/
findIndex(propertyName:string | object | AdvancedCriteria, value?:any): Integer;

/**
*  Like List.findIndex, but inspects a range from startIndex to endIndex.
*  
*  
*  For convenience, findNextIndex() may also be called with a function (called the predicate
*  function) for the propertyName parameter. In this usage pattern, the predicate
*  function is invoked for each value of the list until the predicate returns a true value.
*  The predicate function is passed three parameters: the current value, the current index, and
*  the list. The value of this when the predicate function is called is the
*  value parameter. For example:
*  var currentUserRecord = recordList.findNextIndex(0, function (record, i, recordList) {
*   if (record.username == currentUsername && !record.accountDisabled) {
*     return true;
*   }
* });
*  
* 
* 
* @param {Integer} first index to consider.
* @param {string | Function | object | AdvancedCriteria} property to match; or, if a function is passed, the predicate function to call; or, if an object is passed, set of properties and values to match.
* @param {any=} value to compare against (if propertyName is a string) or the value of this when the predicate function is invoked (if propertyName is a function)
* @param {Integer=} last index to consider (inclusive).
*/
findNextIndex(startIndex:Integer, propertyName:string | Function | object | AdvancedCriteria, value?:any, endIndex?:Integer): Integer;

/**
*  Like List.findIndex, but returns the object itself instead of its index.
* 
* 
* @param {string | object | AdvancedCriteria} property to match, or if an Object is passed, set of                    properties and values to match
* @param {any=} value to compare against (if propertyName is a string)
*/
find(propertyName:string | object | AdvancedCriteria, value?:any): object;

/**
*  Find all objects where property == value in the object.
*  
*  Pass an Object as the propertyName argument to match multiple properties.
* 
* 
* @param {string | object | AdvancedCriteria} property to match, or if an Object is passed, set of                    properties and values to match
* @param {any=} value to compare against (if propertyName is a string)
*/
findAll(propertyName:string | object | AdvancedCriteria, value?:any): Array<any>;

/**
*  Return if this list contains the specified object.
*  
*  If pos is specified, starts looking after that position.
* 
* 
* @param {any} item to look for
* @param {number=} optional position in the list to look after
*/
contains(obj:any, pos?:number): boolean;

/**
*  Return whether this list contains all the item in the specified list.
* 
* 
* @param {List} items to look for
*/
containsAll(list:List): boolean;

/**
*  Return the list of items that are in both this list and the passed-in list(s).
* 
* 
* @param {any} lists to intersect with
*/
intersect(lists:any): List;

/**
*  Return whether this list is equal to another list.
*  
*  Two lists are equal only if they have the same length and all contained items are in the same
*  order and are also equal.
* 
* 
* @param {List} list to check for equality
*/
equals(list:List): boolean;

/**
*  Return the items at a list of specified positions.
* 
* 
* @param {Array<Partial<number>>} array of positions
*/
getItems(itemList:Array<Partial<number>>): Array<any>;

/**
*  Return the items between position start and end, non-inclusive at the end.
* 
* 
* @param {number} start position
* @param {number} end position
*/
getRange(start:number, end:number): Array<any>;

/**
*  Return an Array that is a shallow copy of the list, that is, containing the same items.
* 
* 
*/
duplicate(): Array<any>;

/**
*  Change the array element at a particular position.
*  
*  set() can be used to expand the length of the list.
* 
* 
* @param {number} position in the list to change
* @param {object} new value for that position
*/
set(pos:number, obj:object): object;

/**
*  Add a single item to this array at a specific position in the list, sliding other items over
*  to fit.
* 
* 
* @param {object} object to add
* @param {number} position in the list to add at
*/
addAt(obj:object, pos:number): object;

/**
*  Remove the item at the specified position, rearranging all subsequent items to fill the gap
* 
* 
* @param {number} position to remove
*/
removeAt(pos:number): any;

/**
*  Add an object to this list, at the end
* 
* 
* @param {any} object to add
*/
add(object:any): any;

/**
*  Add a list of items to this array.
*  
*  Note: you can specify that a subset range be added by passing start and end indices
* 
* 
* @param {Array<any>} list of items to add
* @param {number=} optional start index in list
* @param {number=} optional end index in list (non-inclusive)
*/
addList(list:Array<any>, listStartRow?:number, listEndRow?:number): List;

/**
*  Set the length of this list.
*  
*  If the length of the list is shortened, any elements past the new length of the list are removed.
*  If the length is increased, all positions past the old length have the value
*  undefined.
* 
* 
* @param {number} new length
*/
setLength(length:number): void;

/**
*  Add list of items list to this array at item pos. All items after array[pos] will slide down to
*  fit new items.
* 
* 
* @param {Array<any>} new array of items
* @param {number} position in this list to put the new items
*/
addListAt(list:Array<any>, pos:number): Array<any>;

/**
*  Remove first instance of the passed object from this array, sliding other items around to
*  fill gaps.
* 
* 
* @param {any} item to remove
*/
remove(obj:any): boolean;

/**
*  Remove all instances of objects in the specified list from this list, sliding the remaining
*  objects around to fill gaps.
* 
* 
* @param {Array<any>} list of items to remove
*/
removeList(list:Array<any>): List;

/**
*  Sorts the elements of the List in place.
*  
*  The optional comparator function should take two parameters "a" and "b" which are the two list
*  items to compare, and should return:
*  
*  
* - a value less than zero, if "a" is less than "b" such that "a" should appear earlier in the
*    list
*  
* - zero, if "a" and "b" are equal
*  
* - a value greater than zero, if "a" is greater than "b" such that "b" should appear earlier in
*    the list
*  
*  
* 
* @param {Function=} comparator function to use
*/
sort(comparator?:Function): List;

/**
*  Return a new Array where the value of item i is the value of "property" of item i in this
*  array. If an item doesn't have that property or is null, return item will be null.
* 
* 
* @param {string} name of the property to look for
*/
getProperty(property:string): Array<any>;

/**
*  Sort a list of objects by a given property of each item.
*  
*  The optional normalizer, if passed as a function, is called for each item in the List, and
*  should return whatever value should be used for sorting, which does not have to agree with
*  the property value. By passing a normalizer function you can achieve any kind of sorting
*  you'd like, including sorting by multiple properties.
*  
*  NOTE: string sort is case INsensitive by default
* 
* 
* @param {string} name of the property to sort by
* @param {boolean} true == sort ascending, false == sort descending
* @param {Function | ValueMap=} May be specified as a function, with signature        normalize(item, propertyName, context), where item is        a pointer to the item in the array, propertyName is the        property by which the array is being sorted, and context is the        arbitrary context passed into this method. Normalizer function should return        the value normalized for sorting.        May also be specified as a ValueMap which maps property values to sortable values.
* @param {any=} Callers may pass an arbitrary context into the sort method, which             will then be made available to the normalizer function
*/
sortByProperty(property:string, up:boolean, normalizer?:Function | ValueMap, context?:any): List;

/**
*  Get a map of the form { item[idField] -&gt; item[displayField] }, for all 
*  items in the list. Note that if more than one item has the same value for the
*  idField, 
*  the value for the later item in the list will clobber the value for the earlier item.
* 
* 
* @param {string} Property to use as ID (data value) in the valueMap
* @param {string} Property to use a display value in the valueMap
*/
getValueMap(idField:string, displayField:string): object;

/**
*  Method called when this array changes in some way. Observe the method to react to changes in
*  this list.
*  
*  Note: dataChanged() will only fire when items are added, removed or rearranged. If a list
*  contains objects, dataChanged() will not fire if changes are made to objects within the list
*  without changing their position within the list. If an observer of dataChanged() needs to react
*  to such a change, you can manually fire dataChanged() by simply calling it.
*  
*  Note: may be called multiple times as the result of a multi-item add or remove, etc.
* 
* 
*/
dataChanged(): void;




}

/**
* An interface that provides the ability to create components from a PaletteNode.
*/
interface Palette {
/**
* Default EditContext that this palette should use. Palettes generally create components via
*  drag and drop, but may also support creation via double-click or other UI idioms when a
*  defaultEditContext is set.
* @type {EditContext | EditTree | EditPane}
* @default null
*/
defaultEditContext?: EditContext | EditTree | EditPane;

/**
* Whether created components should have their "ID" or "name" property automatically set
*  to a unique value based on the component's type, eg, "ListGrid0".
* @type {boolean}
* @default true
*/
generateNames?: boolean;



/**
*  Sets the default EditContext that this palette should use. Palettes generally create components via
*  drag and drop, but may also support creation via double-click or other UI idioms when a
*  defaultEditContext is set.
* 
* @param {EditContext | EditTree | EditPane} the default EditContext used by this Palette
*/
setDefaultEditContext(defaultEditContext:EditContext | EditTree | EditPane): void;

/**
*  Given a PaletteNode, make an EditNode from it by creating a 
*  EditNode.liveObject from the PaletteNode.defaults
*  and copying presentation properties (eg PaletteNode.title
*  to the editNode.
*  
*  If editNodeProperties is specified as an object on
*  on the paletteNode, each property in this object will also be copied across to
*  the editNode.
* 
* 
* @param {PaletteNode} paletteNode to create from
*/
makeEditNode(paletteNode:PaletteNode): EditNode;




}

/**
* Interface to show a drop-down list of pickable options. Used by the SelectItem and
*  ComboBoxItem classes. Depending on the value of PickList.dataSetType, the 
*  generated drop down list of options must be an instance of PickListMenu or
*  PickTreeMenu, or a subclass thereof.
*/
interface PickList {
/**
* Determines the behavior of the click-mask thrown up when this pickList is visible.
*  
*  The default value, "hard", matches the familiar behavior of combos and selects on 
*  Windows, Mac and other platforms - mouse-events such as rollovers are blocked and, when 
*  a click is received, the picker is hidden and the event is cancelled.
*  
*  When clickMaskMode is "soft", mouse-events continue to fire, meaning that 
*  rollover styles, for example, continue to be updated. When a click is received in this
*  mode, the picker is hidden and the click event is allowed to proceed to its target - 
*  this means that clicking an item with an open picker will re-open the picker.
* @type {ClickMaskMode}
* @default "hard"
*/
clickMaskMode?: ClickMaskMode;

/**
* Maximum height to show the pick list before it starts to scroll.
*  Note that by default the pickList will be sized to the height required by its content
*  so it will be taller when more rows are available as selectable options
* @type {number}
* @default "300"
*/
pickListHeight?: number;

/**
* Height for an empty pick list (showing the empty message), if the pick list has no
*  records and PickList.hideEmptyPickList is false.
* @type {number}
* @default "100"
*/
emptyPickListHeight?: number;

/**
* Message to display in the pickList if there's no data and 
*  PickList.hideEmptyPickList is false.
* @type {string}
* @default "No items to show"
*/
emptyPickListMessage?: string;

/**
* If this pickList contains no options, should it be hidden?
*  If unset, default behavior is to allow the empty pickList to show if it is databound.
* @type {boolean}
* @default null
*/
hideEmptyPickList?: boolean;

/**
* Default width to show the pickList.
*  If not specified, the width of this form item's element will be used instead.
*  
*  Note that this is a minimum value - by default if the values displayed in this pickList
*  are wider than the specified width the list will expand to accommodate them.
* @type {number}
* @default null
*/
pickListWidth?: number;

/**
* Maximum width for this item's pickList.
*  By default if the values displayed in this pickList are wider than the specified
*  PickList.pickListWidth the pickList will render wide enough to accommodate them.
*  This property allows the developer to limit how wide the pickList will render.
* @type {number}
* @default "400"
*/
pickListMaxWidth?: number;

/**
* Base Style for pickList cells. See cellStyleSuffixes for details on how
*  stateful suffixes are combined with the pickListBaseStyle to generate stateful cell styles.
*  
*  Note: if PickList.pickListTallBaseStyle is specified, this property will be used as the
*  ListGrid.normalBaseStyle and that property will be applied
*  to cells that do not match the default cell height, or if ListGrid.fastCellUpdates
*  is true for the pickList in Internet Explorer.
* @type {CSSStyleName}
* @default "pickListCell"
*/
pickListBaseStyle?: CSSStyleName;

/**
* Optional ListGrid.tallBaseStyle for pickList cells. If unset,
*  PickList.pickListBaseStyle will be applied to all cells.
* @type {CSSStyleName}
* @default null
*/
pickListTallBaseStyle?: CSSStyleName;

/**
* Default value for ListGrid.applyRowNumberStyle for this item's generated
*  pickList.
* @type {boolean}
* @default false
*/
pickListApplyRowNumberStyle?: boolean;

/**
* If true, when the pickList is shown, it will be shown via an animated reveal effect
* @type {boolean}
* @default false
*/
animatePickList?: boolean;

/**
* If PickList.animatePickList is true, this attribute specifies the millisecond 
*  duration of the animation effect.
* @type {number}
* @default "200"
*/
pickListAnimationTime?: number;

/**
* For a ComboBox or other pickList that accepts user-entered criteria, how many
*  milliseconds to wait after the last user keystroke before fetching data from the server.
*  The default setting will initiate a fetch if the user stops typing or pauses briefly.
* @type {number}
* @default "200"
*/
fetchDelay?: number;

/**
* Cell Height for this item's pickList.
* @type {number}
* @default "16"
*/
pickListCellHeight?: number;

/**
* If specified this properties block will be applied to the PickListMenu
*  created for this FormItem.
*  
*  Note: Not every ListGrid property is supported when assigned to a pickList.
*  Where there is a dedicated API on the form item (such as 
*  PickList.pickListCellHeight), we recommend that be used in 
*  favor of setting the equivalent property on the pickListProperties block.
*  
*  PickLists and ListGrid.showFilterEditor:
*  ComboBoxItem do not support setting showFilterEditor 
*  to true on pickListProperties. This combination of settings leads to an ambiguous user
*  experience as the two sets of filter-criteria (those from the text-box and those from the 
*  pickList filter editor) interact with each other.
*  Setting ListGrid.showFilterEditor in 
*  SelectItem.pickListProperties is a valid way to create a filterable pickList, on 
*  a SelectItem, but this setting is not supported on a SelectItem with 
*  SelectItem.multiple set to true - this combination of settings can cause a 
*  selected value to be filtered out of view at which point further selection changes will 
*  discard that value.
*  In general we recommend the ComboBoxItem class (with ComboBoxItem.addUnknownValues 
*  set as appropriate) as a better interface for filtering pickList data.
* @type {Partial<ListGrid>}
* @default null
*/
pickListProperties?: Partial<ListGrid>;

/**
* ListGrid-based AutoChild created by the system to display a list of pickable options for this item. 
*  
*  The pickList is automatically generated and displayed by the system when necessary. It may be customized
*  via properties such as PickList.pickListConstructor, PickList.pickTreeConstructor, PickList.pickListProperties 
*  and more. See the PickList for more information.
*  
*  Accessing the generated pickList at runtime is an advanced usage. In most cases developers
*  should not modify this generated component directly but should instead use attributes
*  on the formItem to configure it.
* @type {ListGrid}
* @default null
*/
pickList?: ListGrid;

/**
* If this pick list is showing multiple fields, this property determines the height of
*  the column headers for those fields. Set to zero to suppress the headers entirely.
* @type {number}
* @default "22"
*/
pickListHeaderHeight?: number;

/**
* When this item is showing a PickList.dataSetType, the
*  default behavior is for folder open-state to be toggled by double-clicking. Set this 
*  attribute to true to toggle folders on a single-click instead.
*  
*  Note: when set to true, users can only choose leaf-nodes, since clicking folders would
*  simply toggle them.
* @type {boolean}
* @default false
*/
singleClickFolderToggle?: boolean;

/**
* When this item is showing a PickList.dataSetType, this is 
*  the SelectItem.valueField of the record to use as the 
*  Tree.rootValue node.
* @type {string | number}
* @default null
*/
rootNodeId?: string | number;

/**
* When this item is showing a PickList.dataSetType, which 
*  nodes should be opened automatically. Options are:
*  
*  
* - "none" - no nodes are opened automatically
*  
* - "root" - opens the PickList.rootNodeId - in databound 
*        tree-pickers, this node is always hidden
*  
* - "all" - when ResultTree.loadDataOnDemand, opens the
*        PickList.rootNodeId and all of it's direct
*        descendants - otherwise, opens all loaded nodes 
*  
* @type {string}
* @default "none"
*/
autoOpenTree?: string;

/**
* If this form item maps data values to display values by retrieving the 
*  FormItem.displayField values from an 
*  FormItem.optionDataSource, this property 
*  denotes the the field to use as the underlying data value in records from the 
*  optionDataSource.
*  If not explicitly supplied, the valueField name will be derived as
*  described in FormItem.getValueFieldName.
* @type {string}
* @default null
*/
valueField?: string;

/**
* If set, this item will display a value from another field to the user instead of
*  showing the underlying data value for the FormItem.name.
*  
*  This property is used in two ways:
*  
*  The item will display the displayField value from the 
*  DynamicForm.getValues if 
*  FormItem.useLocalDisplayFieldValue is true, (or if unset and the conditions
*  outlined in the documentation for that property are met).
*  
*  If this field has an FormItem.optionDataSource, this property is used by
*  default to identify which value to use as a display value in records from this
*  related dataSource. In this usage the specified displayField must be 
*  explicitly defined in the optionDataSource to be used - see 
*  PickList.getDisplayFieldName for more on this behavior.
*  If not using PickList.useLocalDisplayFieldValue, the display
*  value for this item will be derived by performing a fetch against the 
*  FormItem.getOptionDataSource 
*  to find a record where the FormItem.getValueFieldName matches 
*  this item's value, and use the displayField value from that record.
*  In addition to this, PickList-based form items that provide a list of possible
*  options such as the SelectItem or ComboBoxItem will show the 
*  displayField values to the user by default, allowing them to choose
*  a new data value (see FormItem.valueField) from a list of user-friendly
*  display values.
*  
*  This essentially allows the specified optionDataSource to be used as
*  a server based valueMap.
*  
*  If FormItem.useLocalDisplayFieldValue
*  are being used and FormItem.storeDisplayValues is true, selecting a new value
*  will update both the value for this field and the associated display-field value
*  on the record being edited.
*  
*  Note: Developers may specify the FormItem.foreignDisplayField property 
*  in addition to displayField. This is useful for cases where the
*  display field name in the local dataSource differs from the display field name in
*  the optionDataSource. See the documentation for
*  DataSourceField.foreignDisplayField for more on this.
*  If a foreignDisplayField is specified, as with just displayField, if 
*  FormItem.useLocalDisplayFieldValue
*  are being used and FormItem.storeDisplayValues is true, when the user 
*  chooses a value the associated display-field value 
*  on the record being edited will be updated. In this case it would be set to the 
*  foreignDisplayField value from the related record. This means foreignDisplayField 
*  is always expected to be set to the equivalent field in the related dataSources.
*  Developers looking to display some other arbitrary field(s) from the
*  related dataSource during editing should consider using custom
*  PickList.pickListFields instead of setting a foreignDisplayField. 
*  
*  Note that if optionDataSource is set and no valid display field is
*  specified,
*  FormItem.getDisplayFieldName will return the dataSource title 
*  field by default.
*  
*  If a displayField is specified for a freeform text based item (such as a 
*  ComboBoxItem), any user-entered value will be treated as a display value.
*  In this scenario, items will derive the data value for the item from the
*  first record where the displayField value matches the user-entered value.
*  To avoid ambiguity, developers may wish to avoid this usage if display values
*  are not unique.
* @type {string}
* @default null
*/
displayField?: string;

/**
* If FormItem.displayField is specified for a field, should the
*  display value for the field be picked up from the 
*  DynamicForm.getValues?
*  
*  This behavior is typically valuable for dataBound components where the
*  displayField is specified at the DataSourceField level. See
*  DataSourceField.displayField for more on this.
*  
*  Note that for DataSources backed by the
*  serverDataIntegration, fields with a specified
*  DataSourceField.foreignKey and DataSourceField.displayField will
*  automatically have this property set to true if not explicitly set to false
*  in the dataSource configuration.
*  
*  Otherwise, if not explicitly set, local display value will be used unless:
*  
*  
* - This item has an explicitly specified optionDataSource, rather than
*    deriving its optionDataSource from a specified 
*    DataSourceField.foreignKey specification
*  
* - The FormItem.name differs from the 
*    FormItem.getValueFieldName for the item
*  
* @type {boolean}
* @default null
*/
useLocalDisplayFieldValue?: boolean;

/**
* Whether to show the picker as a flat list, or a collapsible tree.
*  
*  The default value, "list", will use an instance of the
*  PickList.pickListConstructor as the picker - "tree" will 
*  show an instance of PickList.pickTreeConstructor.
* @type {string}
* @default "list"
*/
dataSetType?: string;

/**
* The Class to use when creating a picker of PickList.dataSetType for 
*  a FormItem. Must be a subclass of the builtin default, 
*  PickListMenu.
* @type {SCClassName}
* @default "PickListMenu"
*/
pickListConstructor?: SCClassName;

/**
* The Class to use when creating a picker of PickList.dataSetType for 
*  a FormItem. Must be a subclass of the builtin default, 
*  PickTreeMenu.
* @type {SCClassName}
* @default "PickTreeMenu"
*/
pickTreeConstructor?: SCClassName;

/**
* This property allows the developer to specify which field[s] will be displayed in the 
*  drop down list of options.
*  
*  Only applies to databound pickLists (see PickList.optionDataSource, or pickLists
*  with custom data set up via the advanced PickList.getClientPickListData method.
*  
*  If this property is unset, we display the PickList.displayField, if specified, 
*  otherwise the PickList.valueField.
*  
*  If there are multiple fields, column headers will be shown for each field, the
*  height of which can be customized via the PickList.pickListHeaderHeight attribute.
*  
*  Each field to display should be specified as a ListGridField object. Note that
*  unlike in ListGrid, dataSource fields marked as 
*  DataSourceField.hidden will be hidden by default in pickLists. To
*  override this behavior, ensure that you specify an explicit value for 
*  ListGridField.showIf.
* @type {Array<Partial<ListGridField>>}
* @default null
*/
pickListFields?: Array<Partial<ListGridField>>;

/**
* For Databound formItems, this property determines which column 
*  FormItem.valueIcons should show up in for this formItem's pickList.
*  If unset, valueIcons show up in the PickList.displayField column if specified, 
*  otherwise the PickList.valueField column.
*  In most cases only the displayField or valueField will be visible.
*  This property is typically only required if custom PickList.pickListFields 
*  have been specified for this item.
* @type {string}
* @default null
*/
valueIconField?: string;

/**
* If this item has a databound pickList (for example PickList.optionDataSource is
*  set), this property can be used to provide static filter criteria when retrieving the data
*  for the pickList.
* @type {Criteria}
* @default null
*/
pickListCriteria?: Criteria;

/**
* If set, this FormItem will derive data to show in the PickList by fetching records from
*  the specified optionDataSource. The fetched data will be used as a
*  FormItem.valueMap by extracting the
*  FormItem.valueField and FormItem.displayField in
*  the loaded records, to derive one valueMap entry per record loaded from the
*  optionDataSource. Multiple fields from the fetched data may be shown in the pickList by
*  setting PickList.pickListFields.
*  
*  The data will be retrieved via a "fetch" operation on the DataSource, passing the 
*  PickList.pickListCriteria (if set) as criteria, and passing
*  PickList.optionFilterContext (if set) as DSRequest Properties.
*  
*  The fetch will be triggered when the pickList is first shown, or, you can set
*  SelectItem.autoFetchData to fetch when the FormItem is
*  first drawn.
*  
*  The pickList will not re-fetch its options each time it is opened, unless
*  the pickList criteria have changed. Developers who want to explicitly force a 
*  new fetch can achieve this by
*  overriding PickList.getPickListFilterCriteria
*  
*  to dynamically generate criteria that change whenever appropriate.
*  For example, to force a new fetch every time the pickList is shown,
*  the generated criteria could include a subcriterion for some arbitrary field
*  with operator notEqual and value set to a something
*  that will change each time the method runs, such as a date-stamp.
*  
*  Note that providing an initial value when
*  FormItem.fetchMissingValues is enabled, or enabling
*  SelectItem.defaultToFirstOption, can also cause a fetch to
*  be initiated immediately upon form creation. You can also call PickList.fetchData
*  at any time to manually trigger a fetch.
*  
*  Data paging is automatically enabled if the optionDataSource supports it. As the
*  pickList is scrolled by the user, requests for additional data will be automatically
*  issued.
*  
*  For a pickList attached to a ComboBoxItem, new fetches are 
*  issued as the user types, with criteria set as described under 
*  ComboBoxItem.getPickListFilterCriteria.
*  If your dataSource is not capable of filtering results by search criteria (eg, the
*  dataSource is backed by an XML flat file), you can set PickList.filterLocally to have the
*  entire dataset loaded up front and filtering performed in the browser. This disables
*  data paging.
*  
*  Note that if a normal, static FormItem.valueMap is also specified
*  for the field (either directly in the form item or as part of the field definition in
*  the dataSource), it will be preferred to the data derived from the optionDataSource for
*  whatever mappings are present.
* @type {DataSource | string}
* @default null
*/
optionDataSource?: DataSource | string;

/**
* If this item is part of a databound form, and has a specified valueMap,
*  by default we show the valueMap options in the pickList for the item.
*  Setting this property to true will ensure that the options displayed in our pickList
*  are derived from the form's dataSource.
* @type {boolean}
* @default null
*/
showOptionsFromDataSource?: boolean;

/**
* If this item has a specified optionDataSource and this property is
*  true, the list of fields used by this pickList will be passed to
*  the datasource as DSRequest.outputs. If the datasource supports this feature
*  the returned fields will be limited to this list. A custom datasource will need to
*  add code to implement field limiting.
*  
*  This list of used fields consists of the values of FormItem.valueField,
*  FormItem.displayField and PickList.pickListFields.
*  
*  NOTE: When enabled, FormItem.getSelectedRecord will only include the
*  fetched fields.
* @type {boolean}
* @default null
*/
fetchDisplayedFieldsOnly?: boolean;

/**
* If this item has a specified optionDataSource, and this property is not
*  null, this will be passed to the datasource as DSRequest properties when
*  performing the filter operation on the dataSource to obtain the set of options for the
*  list. This provides, among other capabilities, a way to trigger the server to return
*  summary records.
* @type {Partial<DSRequest>}
* @default null
*/
optionFilterContext?: Partial<DSRequest>;

/**
* If filterLocally is set for this item, and this item is showing options 
*  from a dataSource, fetch the entire set of options from the server, and use these values
*  to map the item value to the appropriate display value. Also use "local"
*  type filtering on drop down list of options.
*  
*  This means data will only be fetched once from the server, and then filtered on the
*  client.
*  
*  Note - when this property is set to false, filtering will still be 
*  performed on the client if a complete set of data for some criteria has been cached
*  by a fetch, and a subsequent fetch has more restrictive criteria. To explicitly
*  disable client-side filtering set the PickList.useClientFiltering property to false.
* @type {boolean}
* @default false
*/
filterLocally?: boolean;

/**
* Specifies one or more fields by which this item should be initially sorted. It can be 
*  a ListGridField.name, or an array of field names - but note that, if
*  multiple fields are supplied, then each will be sorted in the same 
*  ListGrid.sortDirection.
*  
*  For full sorting control, set PickList.initialSort to a list of
*  custom SortSpecifier.
*  
*  This attribute can also be set to the index of a field in the fields array, but note 
*  that it will be converted to a string (field name) after initialization.
* @type {string | Array<Partial<string>> | Integer}
* @default null
*/
sortField?: string | Array<Partial<string>> | Integer;

/**
* An array of SortSpecifier objects used to set up the initial sort configuration 
*  for this pickList. If specified, this will be used instead of any 
*  PickList.sortField specified.
* @type {Array<Partial<SortSpecifier>>}
* @default null
*/
initialSort?: Array<Partial<SortSpecifier>>;

/**
* For PickList.optionDataSource items, this property will be passed
*  to the generated ResultSet data object for the pickList as ResultSet.useClientFiltering.
*  Setting to false will disable filtering on the client and ensure criteria are
*  always passed to the DataSource directly.
* @type {boolean}
* @default null
*/
useClientFiltering?: boolean;

/**
* For databound pickLists (see PickList.optionDataSource), by default SmartClient
*  will cache and re-use datasets shown by pickLists in an LRU (least recently used) caching
*  pattern.
*  
*  Setting this flag to false avoids this caching for situations where it is too aggressive.
* @type {boolean}
* @default true
*/
cachePickListResults?: boolean;

/**
* For PickList items with PickListItemIconPlacement set such that the pickList does
*  not render near-origin, should specified FormItem.icons be rendered inline within
*  the formItem itself, or within the ComboBoxItem.pickerNavigationBar.
*  
*  May be overridden at the icon level via FormItemIcon.iconPlacement.
*  
*  For mobile browsing with limited available screen space, icons rendered in
*  the navigation bar may be easier for the user to interact with.
* @type {PickListItemIconPlacement}
* @default "both"
*/
iconPlacement?: PickListItemIconPlacement;

/**
* For databound pickList-based formItems, if there are both explicitly specified 
*  FormItem.optionCriteria and PickList.pickListCriteria, these will
*  be combined when determining the final criteria for retrieving the set of data to
*  display in the drop down pickList.
*  
*  If the optionCriteria are specified as simple crition objects
*  (rather than AdvancedCriteria) this property determines the textMatchStyle to
*  apply to the optionCriteria - whether to use FormItem.optionTextMatchStyle 
*  or consult PickList.textMatchStyle.
*  
*  Wherever possible, this property should be set to true, to ensure the
*  optionCriteria are interpreted the same way when 
*  PickList.getPickListFilterCriteria as 
*  FormItem.optionDataSource.
*  
*  However if the specified textMatchStyles do not match this
*  will cause AdvancedCriteria to be generated by PickList.getPickListFilterCriteria
*  even if both FormItem.optionCriteria and 
*  PickList.pickListCriteria are specified as 
*  simple criteria. Since not all DataSources support AdvancedCriteria
*  developers may choose to set this value to false and avoid them being created.
* @type {boolean}
* @default true
*/
useOptionTextMatchStyleInPickList?: boolean;

/**
* When applying filter criteria to pickList data, what type of matching to use.
*  
*  For a databound pickList (PickList.optionDataSource set), textMatchStyle is
*  sent to the server as DSRequest.textMatchStyle.
*  
*  For a non-databound pickList, textMatchStyle is applied by 
*  PickList.filterClientPickListData.
* @type {TextMatchStyle}
* @default "startsWith"
*/
textMatchStyle?: TextMatchStyle;

/**
* If true, even non-matching options will be shown, with configurable 
*  PickList.separatorRows in between. Not valid for
*  PickList.optionDataSource.
* @type {boolean}
* @default null
*/
showAllOptions?: boolean;

/**
* Array of records to show between matching and non-matching rows in the PickList.
*  
*  Not valid for PickList.optionDataSource.
* @type {Array<Partial<ListGridRecord>>}
* @default "[{isSeparator:true}]"
*/
separatorRows?: Array<Partial<ListGridRecord>>;

/**
* A set of "special" values such as "All", "None" or "Invalid" that do not appear in the normal
*  ValueMap or in the data returned by the PickList.optionDataSource.
*  
*  Like other uses of ValueMap, either a list of values or a mapping from stored to
*  display value can be provided.
*  
*  These values can either be shown at the top of the list of values (in the order specified), or
*  can be shown in a separate, non-scrolling region - the setting
*  SelectItem.separateSpecialValues controls this. Note that data 
*  paging can only be used if separateSpecialValues is enabled.
*  
*  If specialValues are configured, SelectItem.allowEmptyValue is ignored - an empty
*  value, if desired, must be included in the specialValues. To provide a
*  specialValue which clears the value of the field, use the special constant
*  PickList.emptyStoredValue.
*  
*  specialValues can also be used to take a value that does appear in the
*  normal data and redundantly display it at the top of the list to make it more accessible. Note
*  that in this case it is expected that the special value appears both at the top of the
*  list and in it's normal position in the list, so this works best with
*  SelectItem.separateSpecialValues mode enabled.
*  
*  Also, if an PickList.optionDataSource is used, PickList.specialValues that appear in the normal
*  dataset will be updated by automatic cacheSync (if 
*  the PickList.displayField is updated). However when using a distinct PickList.valueField and
*  PickList.displayField, you are required to provide PickList.specialValues as a map (there is no
*  support for FormItem.fetchMissingValues automatically fetching appropriate display values).
*  
*  Note that specialValues are not supported in conjunction with MultiComboBoxItem. Whereas with
*  SelectItem.multiple, specialValues will never be normal values 
*  that may be selected. So, specialValues should have options such as "Select All", "Select None" and others.
* @type {ValueMap}
* @default null
*/
specialValues?: ValueMap;



/**
*  Only applies to databound items (see PickList.optionDataSource).
*  Performs a fetch type operation on this item's DataSource to retrieve the set of valid
*  options for the item, based on the current PickList.pickListCriteria.
* 
* @param {DSCallback=} Callback to fire when the fetch completes. Callback will        fire with 4 parameters:  
- item a pointer to the form item  
- dsResponse the DSResponse returned by the server  
- data the raw data returned by the server  
- dsRequest the DSRequest sent to the server  
* @param {Partial<DSRequest>=} properties to apply to the       dsRequest for this fetch.
*/
fetchData(callback?:DSCallback, requestProperties?:Partial<DSRequest>): void;

/**
*  PickLists can derive their data directly from a valueMap, or retrieve data from a 
*  dataSource to display as options.
*  
*  This method will return the dataSource used to populate the pickList, or null if 
*  none is specified (implies this list will derive its data from the valueMap for the item).
*  Default implementation will return PickList.optionDataSource if specified,
*  otherwise if this is a field with a specified foreignKey in a databound
*  form, returns the dataSource for the foreignKey.
*  Otherwise picks up this.form.dataSource if set.
* 
* 
*/
getOptionDataSource(): DataSource;

/**
*  stringMethods to return a set of filter criteria to be applied to
*  the data displayed in the pickList when it is shown.
*  
*  If this is a databound item the criteria will be passed as criteria to
*  DataSource.fetchData. Otherwise an equivalent client-side filter will be
*  performed on the data returned by PickList.getClientPickListData.
*  
*  By default combines FormItem.optionCriteria with
*  PickList.pickListCriteria if specified, otherwise an empty 
*  set of criteria so all records will be displayed.
*  
*  Note that if no FormItem.optionCriteria are 
*  present and PickList.pickListCriteria were
*  specified as a simple criteria object rather than an AdvancedCriteria, the
*  generated fetch operation to populate the pickList will contain these simple criteria
*  (and PickList.textMatchStyle will determine how these
*  criteria are interpreted).
*  However if FormItem.optionCriteria are specified as a 
*  simple criteria object, an AdvancedCriteria object may be generated for the
*  fetch even if pickListCriteria are not present, or also defined in simple criteria
*  object format. See PickList.useOptionTextMatchStyleInPickList for details.
* 
* 
*/
getPickListFilterCriteria(): Criteria;

/**
*  Getter method to retrieve the FormItem.valueField for this item. For
*  items with a specified FormItem.optionDataSource, this determines which
*  field in that dataSource corresponds to the value for this item.
*  
*  If unset, if a DataSourceField.foreignKey exists
*  between this field and the optionDataSource, this will be used,
*  otherwise default behavior will return the FormItem.name of this field.
* 
* 
*/
getValueFieldName(): string;

/**
*  Returns the displayField for this item.
*  
*  Behavior varies based on the configuration of this item, as follows:
*  
* - If this item has an PickList.optionDataSource and an explicit 
*  FormItem.foreignDisplayField is specified, this will be returned.
*  
* - Otherwise if an explicit PickList.displayField is specified it will be returned 
*  by default. If the displayField was specified on the underlying 
*  dataSource field, and no matching field is present in the PickList.optionDataSource for
*  the item, we avoid returning the specified displayField value and instead return the
*  title field of the option DataSource. We do this to
*  avoid confusion for the case where the displayField is intended as a display-field
*  value for showing another field value within the same record in the underlying
*  dataSource only.
*  
* - If no explicit foreignDisplay or displayField 
*  specification was found, and the FormItem.valueField for this item is hidden in the 
*  FormItem.optionDataSource, this method will return the title field for 
*  the optionDataSource.
* 
* 
*/
getDisplayFieldName(): FieldName;

/**
*  Returns the set of data to be displayed in this item's PickList.
*  
*  This method will be called for non-databound form items implementing the PickList
*  interface. The default implementation will derive data from the item's valueMap - 
*  can be overridden to allow a custom set of options to be displayed.
*  
*  Note that for PickLists that filter data based on user input
*  (ComboBoxItem), this method should return the data before
*  filtering. To customize the data returned after filtering, override
*  PickList.filterClientPickListData instead.
*  
*  As an example, for a formItem with PickList.valueField set to "valueFieldName", the
*  default implementation would take a valueMap like the following:
*  
*    valueMap: { value1: "display 1", value2: "display 2" }
*  
*  .. and returning the following set of records: 
*  
*    [
*      { valueFieldName : "value1" },
*      { valueFieldName : "value2" }
*    ]
*  
*  Due to the valueMap, these records will appear as a two row pickList displayed as
*  "display 1" and "display 2".
* 
* 
*/
getClientPickListData(): Array<Partial<ListGridRecord>>;

/**
*  If this item is showing a dataBound pickList, this notification method will be fired 
*  when new data arrives from the server.
* 
* @param {Integer} index of first row returned by the server
* @param {Integer} index of last row returned by the server
* @param {ResultSet} pointer to this pickList's data
*/
dataArrived(startRow:Integer, endRow:Integer, data:ResultSet): void;

/**
*  Returns the data to display in the pick list.
*  
*  The default implementation applies the criteria returned by 
*  PickList.getPickListFilterCriteria to the data returned by
*  PickList.getClientPickListData. A record passes the filter if it has a
*  matching value for all fields in the criteria object. Matching is performed according
*  to TextMatchStyle.
*  
*  If PickList.showAllOptions is set, all values are shown, with matching values
*  shown below a PickList.separatorRows.
* 
* 
*/
filterClientPickListData(): Array<Partial<ListGridRecord>>;




}

/**
* AdvancedCriteria is a format for representing search criteria which may include
*  operators on field values such as "less than", or may include sub-clauses such as several 
*  criteria applied to fields joined by an "OR" operator.
*  
*  SmartClient DataSources can use AdvancedCriteria to search a list of Records, and
*  the SmartClient Java Server can translate AdvancedCriteria to either SQL or Hibernate
*  queries (Note: The server-side AdvancedCriteria handling feature is only available 
*  with the Power and Enterprise Editions of SmartClient; the Pro Edition is 
*  limited to ordinary criteria handling on the server side).
*  If the entire dataset is cached locally, SmartClient can perform AdvancedCriteria filtering 
*  on the client, avoiding a server call.
*  
*  
*  An AdvancedCriteria is an ordinary JavaScript object which can be created directly
*  with JavaScript literal notation. For example:
*  
*  var advancedCriteria = {
*     _constructor:"AdvancedCriteria",
*     operator:"and",
*     criteria:[
*       // this is a Criterion
*       { fieldName:"salary", operator:"lessThan", value:80000 },
*       { operator:"or", criteria:[
*          { fieldName:"title", operator:"iContains", value:"Manager" },
*          { fieldName:"reports", operator:"notNull" }
*        ] 
*       },
*       { fieldName:"startDate", operator:"greaterThan", value:new Date(1388552400000) }
*     ]
*   }
*  
*  This makes AdvancedCriteria very easy to store and retrieve as JSON strings, using
*  JSON.encode.
*  
*  
*  
*  AdvancedCriteria can also be specified in componentXML:
*  
*  &lt;AdvancedCriteria operator="and" _constructor="AdvancedCriteria"&gt;
*    &lt;criteria&gt;
*      &lt;Criterion fieldName="salary" operator="lessThan"&gt;
*        &lt;value xsi:type="xsd:float"&gt;80000&lt;/value&gt;
*      &lt;/Criterion&gt;
*      &lt;Criterion operator="or"&gt;
*        &lt;criteria&gt;
*          &lt;Criterion fieldName="title" operator="iContains"&gt;
*            &lt;value xsi:type="xsd:text"&gt;Manager&lt;/value&gt;
*          &lt;/Criterion&gt;
*          &lt;Criterion fieldName="reports" operator="notNull"/&gt;
*        &lt;/criteria&gt;
*      &lt;/Criterion&gt;
*      &lt;Criterion fieldName="startDate" operator="greaterThan"&gt;
*        &lt;value xsi:type="xsd:datetime"&gt;2014-01-01T05:00:00.000&lt;/value&gt;
*      &lt;/Criterion&gt;
*    &lt;/criteria&gt;
*  &lt;/AdvancedCriteria&gt;
*  
*  An AdvancedCriteria is in effect a Criterion that has been marked with
*  _constructor:"AdvancedCriteria" to mark it as complete criteria.
*  
*  In addition to directly creating an AdvancedCriteria object as described above, the
*  DataSource.convertCriteria and DataSource.combineCriteria methods
*  may be used to create and modify criteria based on simple fieldName / value mappings.
*  
*  xmlCriteriaShorthand when defining AdvancedCriteria.
*  
*  When passed to the SmartClient Server, a server-side AdvancedCriteria instance (in the
*  package com.isomorphic.criteria) can be retrieved from a DSRequest via
*  com.isomorphic.datasource.DSRequest.getAdvancedCriteria(). These same AdvancedCriteria
*  objects can be directly created server side, and applied to a DSRequest via
*  setAdvancedCriteria().
*  
*  RestDataSource, the recommended way of integration with servers that are not running
*  the SmartClient Server Framework, defines a standard XML and JSON serialization of
*  AdvancedCriteria. Date, DateTime and Time values use the same XML Schema
*  representation used for other XML serialization like RestDataSource. Further details can
*  be found at dateFormatAndStorage.
*  
*  It's a best practice for XML representation to have &lt;value&gt; as a subelement
*  with xsi:type. Although most systems will auto-convert criteria explicitly
*  setting type leaves the least room for error or ambiguity.
*  
*  For other servers, you can translate AdvancedCriteria into whatever format is
*  expected by the server, typically by implementing DataSource.transformRequest.
*  
*  
*  See criteriaEditing for information about
*  editing AdvancedCriteria in a DynamicForm.
*  
*  When using the SmartClient Server, AdvancedCriteria created on the client and stored 
*  as JSON can be used directly by server code (without involvement of the browser and client-side system).
*  Use the server-side API AdvancedCriteria.decodeClientCriteria() to obtain an AdvancedCriteria that can 
*  then be used with a server-created DSRequest object. Note that the client must be serialized by the 
*  JSONEncoder class, using JSONEncoder.dateFormat "logicalDateConstructor".
*/
interface AdvancedCriteria extends Criterion {
/**
* When set to true, causes filtering using this criteria object to follow SQL99 behavior for 
*  dealing with NULL values. See DataSource.strictSQLFiltering for
*  more detail.
* @type {boolean}
* @default null
*/
strictSQLFiltering?: boolean;





}


/**
* Configuration object for effect to apply during an animated show or hide.
*/
interface AnimateShowEffect {
/**
* Effect to apply
* @type {AnimateShowEffectId}
* @default null
*/
effect?: AnimateShowEffectId;

/**
* For show animations of type "wipe" and
*   "slide" this attribute specifies where the wipe / slide should originate.
*   Valid values are "T" (vertical animation from the top down, the 
*   default behavior), and "L" (horizontal animation from the left side).
* @type {string}
* @default null
*/
startFrom?: string;

/**
* For hide animations of type "wipe and 
*   "slide" this attribute specifies where the wipe / slide should finish.
*   Valid options are "T" (vertical animation upwards to the top of the canvas,
*   the default behavior) and "L" (horizontal animation to the left of the
*   canvas).
* @type {string}
* @default null
*/
endAt?: string;

/**
* Use AnimateShowEffect.endAt instead.
* @deprecated * This property was misnamed.

* @type {string}
* @default null
*/
endsAt?: string;





}






/**
* Allows specifying additional browser window settings when calling the underlying
*  JavaScript or OpenFin call to create the child window in MultiWindow.open.
*/
interface BrowserWindowSettings {
/**
* Initial width to assign to a new child window created by MultiWindow.open.
* @type {Integer}
* @default null
*/
defaultWidth?: Integer;

/**
* Initial height to assign to a new child window created by MultiWindow.open.
* @type {Integer}
* @default null
*/
defaultHeight?: Integer;

/**
* Should the window be opened in a tab or a new browser window in fallback mode without
*  OpenFin? Unless set explicitly false, fallback mode will open tabs for the new window.
* @type {boolean}
* @default null
*/
tab?: boolean;

/**
* Should a new window be activated when opened to ensure it's on top?
* @type {boolean}
* @default true
*/
activateOnOpen?: boolean;

/**
* Persistent setting to position the window at the top of the window stack.
* @type {boolean}
* @default false
*/
alwaysOnTop?: boolean;





}


/**
* A type of Record which represents an event to occur at a specific time, displayed
*  within the calendar.
*/
interface CalendarEvent {
/**
* Date object which represents the start date of a CalendarEvent.
*  The name of this field within the CalendarEvent can be changed via
*  Calendar.startDateField
* @type {Date}
* @default null
*/
startDate?: Date;

/**
* Date object which represents the end date of a CalendarEvent
*  The name of this field within the CalendarEvent can be changed via
*  Calendar.endDateField
* @type {Date}
* @default null
*/
endDate?: Date;

/**
* The duration of this event. May be specified instead of an 
*  CalendarEvent.endDate and implies that this is a "Period" type event. If
*  set to zero, implies an "Instant" type event - an event with a start date but no length.
* @type {Integer}
* @default null
*/
duration?: Integer;

/**
* When a CalendarEvent.duration is set for this event, this is the unit of
*  that duration. The default is minutes.
* @type {TimeUnit}
* @default "minute"
*/
durationUnit?: TimeUnit;

/**
* String which represents the name of a CalendarEvent
*  The name of this field within the CalendarEvent can be changed via
*  Calendar.nameField
* @type {string}
* @default null
*/
name?: string;

/**
* String which represents the description of a CalendarEvent
*  The name of this field within the CalendarEvent can be changed via
*  Calendar.descriptionField
* @type {string}
* @default null
*/
description?: string;

/**
* Optional boolean value controlling the editability of this particular calendarEvent.
*  The name of this field within the CalendarEvent can be changed via
*  Calendar.canEditField.
* @type {boolean}
* @default null
*/
canEdit?: boolean;

/**
* Optional boolean value controlling whether this event can be dragged with the mouse.
*  The name of this field within the CalendarEvent can be changed via
*  Calendar.canDragEventField. Only has an effect when 
*  Calendar.canEditEvents is enabled.
*  
*  You can separately disallow drag-resize via CalendarEvent.canResize.
* @type {boolean}
* @default null
*/
canDrag?: boolean;

/**
* Optional boolean value controlling whether this event can be drag-resized with the mouse.
*  The name of this field within the CalendarEvent can be changed via
*  Calendar.canResizeEventField.
*  
*  Only has an effect if Calendar.canEditEvents and 
*  Calendar.canDragEvents are also enabled.
* @type {boolean}
* @default null
*/
canResize?: boolean;

/**
* Boolean indicating whether this event can be moved between lanes. Can also be set at the
*  Calendar.canEditLane.
*  
*  The name of this field within the CalendarEvent can be changed via 
*  Calendar.canEditLaneField.
* @type {boolean}
* @default null
*/
canEditLane?: boolean;

/**
* Boolean indicating whether this event can be moved between lanes. Can also be set at the
*  Calendar.canEditSublane.
*  
*  The name of this field within the CalendarEvent can be changed via 
*  Calendar.canEditSublaneField.
* @type {boolean}
* @default null
*/
canEditSublane?: boolean;

/**
* An optional background color for the body portion of EventCanvas
*  representing this event in the various CalendarView.
*  
*  Note that the recommended approach for styling events is to set a
*  CalendarEvent.styleName, which allows more complete customization
*  of both header and body portions.
* @type {string}
* @default null
*/
backgroundColor?: string;

/**
* An optional text color for the body portion of EventCanvas
*  representing this event in the various CalendarView.
*  
*  Note that the recommended approach for styling events is to set a
*  CalendarEvent.styleName, which allows more complete customization
*  of both header and body portions.
* @type {string}
* @default null
*/
textColor?: string;

/**
* An optional border color for the body portion of EventCanvas
*  representing this event in the various CalendarView.
*  
*  Note that the recommended approach for styling events is to set a
*  CalendarEvent.styleName, which allows more complete customization
*  of both header and body portions.
* @type {string}
* @default null
*/
borderColor?: string;

/**
* An optional background color for the header portion of EventCanvas
*  representing this event in the various CalendarView.
*  
*  Note that the recommended approach for styling events is to set a
*  CalendarEvent.styleName, which allows more complete customization
*  of both header and body portions.
* @type {string}
* @default null
*/
headerBackgroundColor?: string;

/**
* An optional text color for the header portion of EventCanvas
*  representing this event in the various CalendarView.
*  
*  Note that the recommended approach for styling events is to set a
*  CalendarEvent.styleName, which allows more complete customization
*  of both header and body portions.
* @type {string}
* @default null
*/
headerTextColor?: string;

/**
* An optional border color for the header portion of EventCanvas
*  representing this event in the various CalendarView.
*  
*  Note that the recommended approach for styling events is to set a
*  CalendarEvent.styleName, which allows more complete customization
*  of both header and body portions.
* @type {string}
* @default null
*/
headerBorderColor?: string;

/**
* CSS style series to use for the draggable event window that represents this event. If
*  specified, overrides Calendar.eventWindowStyle for this specific event.
*  
*  The name of this field within the CalendarEvent can be changed via
*  Calendar.eventWindowStyleField
* @deprecated * in favor of CalendarEvent.styleName

* @type {CSSStyleName}
* @default null
*/
eventWindowStyle?: CSSStyleName;

/**
* CSS style series to use for Calendar.eventCanvas that 
*  represent this event in the various CalendarView. The basic 
*  series should include three classes - the base style and others suffixed "Header" and "Body".
*  
*  If not specified on the event, the style can be specified on the 
*  Calendar.eventStyleName, the CalendarView.eventStyleName or
*  individually on each Lane.eventStyleName or Lane.sublanes.
*  
*  The name of this field within the CalendarEvent can be changed via
*  Calendar.eventStyleNameField
* @type {CSSStyleName}
* @default null
*/
styleName?: CSSStyleName;

/**
* When in Timeline mode, or when Calendar.showDayLanes is true, a string that 
*  represents the name of the Calendar.lanes this CalendarEvent should 
*  sit in. The name of this field within the CalendarEvent can be changed via
*  Calendar.laneNameField.
* @type {string}
* @default null
*/
lane?: string;

/**
* When in Timeline mode, or when Calendar.showDayLanes is true, a string that 
*  represents the name of the Lane.sublanes this CalendarEvent should 
*  sit in. The name of this field within the CalendarEvent can be changed via
*  Calendar.sublaneNameField.
* @type {string}
* @default null
*/
sublane?: string;





}


/**
* A CellRecord represents the data for one cell of the body area.
*  
*  Each CellRecord should be an object that minimally has a property named after each
*  visible facetId with the value being a facetValueId from that facet, and also a value
*  for CubeGrid.valueProperty.
*  
*  Cell records can contain any other properties desired, such as cell ids, or values for
*  facets not initially shown.
*/
interface CellRecord {
/**
* Default property name denoting whether this record is enabled. Property name may be
*  modified for some grid via ListGrid.recordEnabledProperty.
* @type {boolean}
* @default null
*/
enabled?: boolean;





}


/**
* Sets up a real inheritance structure for Javascript objects.
* 	We separate out class objects from prototypes, so each gets its own inheritance chain. 
* 	This allows us to set up superclass calls, maintain class vs. instance variables and more!
* 
* 	The ClassFactory is a singleton object that holds the miscellaneous pieces of our inheritance
* 	mechanism.
* 
* 	Your main interaction with the ClassFactory is to create new classes:
*  ClassFactory.defineClass("MyClass", "mySuperClass");
*/
interface ClassFactory {




}


/**
* An object containing properties that is used in Gradient types
*/
interface ColorStop {
/**
* The relative offset for the color.
* @type {Float}
* @default "0.0"
*/
offset?: Float;

/**
* 0 is transparent, 1 is fully opaque
* @type {Float}
* @default "1.0"
*/
opacity?: Float;

/**
* eg #ff0000 or red or rgb(255,0,0)
* @type {CSSColor}
* @default null
*/
color?: CSSColor;





}


/**
* Controls what class and instance substitutions, if any, are applied during a call to
*  RPCManager.createScreen. Classes and instances can be mapped (constructed as)
*  other classes, and existing widget instances can be returned for new ones.
*/
interface CreateScreenSettings {
/**
* Maps class names of widgets in the screen to new class names, so that if a widget would
*  normally be constructed as an instance of a class, and that class is in the map, it's
*  instead constructed as an instance of the new class.
* @type {Map<any>}
* @default null
*/
classSubstitutions?: Map<any>;

/**
* Defines the map of new widget Class.getID to existing class instances, or
*  to new instances that will be of a different class. A substituted class instance is
*  returned immediately from Class.create without modification. Otherwise, the
*  constructor for the new instance is run, but targeting the substituted class rather than
*  the original.
*  
*  Note that where we return an existing instance, not even its Canvas.ID will be
*  changed. An alternative is programmtic replacement using Layout.replaceMember.
* @type {Map<any>}
* @default null
*/
componentSubstitutions?: Map<any>;

/**
* Should Placeholder be rendered as placeholders?
*  If property is not set actual components are created instead of the Placeholders.
* @type {boolean}
* @default null
*/
allowPlaceholders?: boolean;

/**
* DataContext that will be provided to the top-level component as
*  Canvas.dataContext in the screen.
*  
*  To understand how dataContext is used to automatically populate 
*  DataBoundComponent, see Canvas.autoPopulateData.
* @type {DataContext}
* @default null
*/
dataContext?: DataContext;

/**
* Should DataSources referenced by the screen clobber existing, globally-bound DataSources
*  on the client when the screen is created? The default of false means that any
*  DataSources defined in the screen will be discarded if they collide with existing,
*  globally-bound DataSources.
*  
*  Note that here we consider a DataSource to be "globally bound" if it can be retrieved by
*  ID using the method DataSource.get, regardless of whether it's actually bound
*  to the browser window object.
* @type {boolean}
* @default false
*/
clobberDataSources?: boolean;

/**
* If true, prevents any screen from being drawn when it's created, even if
*  there's an explicit Canvas.autoDraw:true setting on it.
* @type {boolean}
* @default false
*/
suppressAutoDraw?: boolean;

/**
* Simplifies integrating a screen with an existing JavaScript app. Does the following:
*  
*  
* - Sets CreateScreenSettings.suppressAutoDraw: true
*  
* - Sets Canvas.htmlElement of the screen's top widget to the DOM element
*  
* - Sets the screen's top widget to have Canvas.position: "relative"
*  
* - Draws the screen by calling Canvas.draw on the top widget
*  If you need to apply additional, custom configuration, for example setting
*  Canvas.htmlPosition, then instead of using this property, you can call
*  RPCManager.createScreen with CreateScreenSettings.suppressAutoDraw: true, and then
*  manually configure and draw the screen:
*  
*    var screen = isc.RPCManager.createScreen("My Screen", {suppressAutoDraw: true});
*    screen.setHtmlElement(element);
*    screen.setHtmlPosition("replace");
*    screen.position = "relative";
*    screen.draw();
*  
* @type {DOMElement | string}
* @default null
*/
htmlElement?: DOMElement | string;

/**
* Enable RPCManager.verifyAsError behavior only for
*  requests using these settings.
* @type {boolean}
* @default null
*/
verifyAsError?: boolean;

/**
* Enables verification that the created screen contains a component having a
*  localId equal to the given key, and that it is an instance (or subclass)
*  if the key's value. Example:
*  
*   {'customerListGrid: 'ListGrid'}
*  
*  You may verify presence and type of Tabs, SectionStackSections, and FormItems by
*  providing their names following the parent component's id in dot-separated notation.
*  Example:
*  
*   {
*    'mainTabSet.customersTab': 'ImgTab',
*    'mainSectionStack.customersSection': 'SectionHeader',
*    'customerDetailsForm.customerNameItem': 'TextItem'
*   }
*  
*  Findings are always reported to the console, and may also be presented to the user
*  with a warning dialog by setting CreateScreenSettings.verifyAsError or
*  RPCManager.verifyAsError.
* @type {any}
* @default null
*/
verifyComponents?: any;





}


/**
* An object representing a criterion to apply to a record. 
*  
*  A criterion is part of the definition of an AdvancedCriteria object, which is used to
*  filter records according to search criteria. 
*  
*  A criterion consists of an Criterion.operator and typically a
*  DataSourceField.name from a
*  Record and a Criterion.value to compare to. However some operators
*  either don't require a value (eg, isNull) or act on other criteria rather than directly on a
*  Record's fields (eg, the "and" and "or" logical operators).
*  
*  A shortcut form is also allowed where only fieldName and value
*  values are provided. In this case the operator is assumed to be "equals".
*/
interface Criterion {
/**
* Operator this criterion applies.
* @type {OperatorId}
* @default null
*/
operator?: OperatorId;

/**
* Name of the field in each Record that this criterion applies to. Not applicable for a
*  criterion with Criterion.criteria. Can be specified as a dataPath to 
*  allow matching nested objects. Use '/' as delimiters for dataPath. See 
*  DataPath for more information.
* @type {string}
* @default null
*/
fieldName?: string;

/**
* Value to be used in the application of this criterion.
*  
*  Value may be required or not required, or may be an Array, according to the
*  OperatorValueType of the operator.
* @type {any}
* @default null
*/
value?: any;

/**
* Wherever dynamicCriteria are supported, valuePath can be
*  specified as a path in the current Canvas.ruleScope as an alternative to setting a fixed
*  Criterion.value.
*  
*  Note: valuePath vs setting a path for Criterion.fieldName:
*  
*  
* - use a path for criterion.fieldName when criteria will be matched against a
*    nested data structure.
*  
* - use criterion.valuePath when the values used in filtering should be
*    dynamically derived based on the Canvas.ruleScope. This does not imply that the
*    criteria will be matched against a nested structure.
*  
* @type {string}
* @default null
*/
valuePath?: string;

/**
* For a criterion with an operator that acts on other criteria (eg "and", "or"), a list of
*  sub-criteria that are grouped together by the operator.
* @type {Array<Partial<Criterion>>}
* @default null
*/
criteria?: Array<Partial<Criterion>>;

/**
* Start value of a criterion with an operator of type "valueRange".
* @type {any}
* @default null
*/
start?: any;

/**
* End value of a criterion with an operator of type "valueRange".
* @type {any}
* @default null
*/
end?: any;





}


/**
* Represents a field in a DataBoundComponent.
*/
interface DBCField {




}


/**
* Represents a native DOM element used by the browser.
*  
*  For example, you can retrieve the DOM element representing the "Set Data" button from
*  "Empty Grid" Sample
*  in the SmartClient Feature Explorer using AutoTest.getElement:
*  
*  var buttonElement = isc.AutoTest.getElement("scLocator=//testRoot[]/child[Class=IButton||index=1||length=3||classIndex=0||classLength=2||roleIndex=0||roleLength=2||title=Set%20Data||scRole=button]/");
*  This is a more robust way to obtain a DOM element than retreiving it by ID:
*  
*  var buttonElement = document.getElementById("isc_FZ")
*/
interface DOMElement {




}


/**
* Settings to control optional DataSource.load features.
*/
interface DSLoadSettings {
/**
* Forcibly reload a dataSource if it's already loaded.
* @type {boolean}
* @default null
*/
forceReload?: boolean;

/**
* Load parent DataSources
* @type {boolean}
* @default null
*/
loadParents?: boolean;

/**
* Place loaded DataSource into DataSource.mockMode
* @type {boolean}
* @default null
*/
mockMode?: boolean;





}


/**
* Request sent to the server to initiate a 
*  dataSourceOperations. All properties which are legal on
*  RPCRequest are legal, in addition to the properties listed here.
*/
interface DSRequest extends RPCRequest {
/**
* Should the HTTP response to this request be formatted using the
*  strict JSON subset of the javascript language?
*  If set to true, responses returned by the server should match the format described
*  here.
*  
*  Only applies to requests sent a server with
*  DataSource.dataFormat set to "json" or "iscServer".
* @type {boolean}
* @default null
*/
useStrictJSON?: boolean;

/**
* Whether formatting settings should be applied to data being exported. Default behavior and
*  the effect of setting of exportRawValues is described in the
*  exportFormatting.
* @type {boolean}
* @default null
*/
exportRawValues?: boolean;

/**
* Determines the PropertyIdentifier to be used in the exported data. This
*  essentially means, should we export internal field names like "countryCode" or
*  "EMPLOYEE_NO", or localized descriptive field titles like "code du pays" or
*  "Employee Number". This setting has a lot in common with DSRequest.exportRawValues;
*  both are largely dependent on whether the exported data is intended for direct consumption by
*  an end user (in which case it is appropriate to export formatted values and localized field
*  titles), or for interface to some downstream computer system (in which case you probably
*  want raw, unformatted values and internal field names).
*  
*  If this property is not set, the following defaults apply:
*  
* - If the export format is a native spreadsheet format (XLS or OOXML), localized field 
*  titles are used
*  
* - If the export format is CSV, XML or JSON and this is a client-driven export (ie it was
*  initiated by a call to ListGrid.exportClientData), 
*  localized field titles are used
*  
* - If the export format is CSV, XML or JSON and this is not a client-driven export,
*  internal field names are used 
*  
* @type {PropertyIdentifier}
* @default null
*/
exportPropertyIdentifier?: PropertyIdentifier;

/**
* DataSource this DSRequest will act on.
*  
*  This property is generally automatically populated, for example when calling
*  DataSource.fetchData the dataSource property is set to the target
*  DataSource.
* @type {string}
* @default null
*/
dataSource?: string;

/**
* Type of operation being performed: "fetch", "add", "remove", "update" or "custom".
*  
*  This property is generally automatically populated, for example when calling
*  fetchData() on a DataSource or DataBound component the operationType is
*  automatically set to "fetch". Note that "custom" operations are never generated
*  automatically, they are always fired by your code.
* @type {DSOperationType}
* @default null
*/
operationType?: DSOperationType;

/**
* Data, such as search criteria or an updated record, to be acted upon. Contents differ by
*  operationType, see dataSourceOperations for
*  details.
*  
*  This field is generally filled in by passing the "data" argument to methods such as
*  ListGrid.fetchData.
* @type {object | Array<Partial<object>>}
* @default null
*/
data?: object | Array<Partial<object>>;

/**
* A callback method that will be called with an instance of DSResponse, as sent by the server.
*  Queuing does not affect callbacks in any way - your specified callback will be invoked for 
*  each DSRequest that contained a callback regardless of whether the request was sent as part
*  of a queue or not.
*  
*  Note that if the request encounters an error (such as 500 server error), by default the
*  callback will not be fired. Instead, RPCManager.handleError is called to
*  invoke the default system-wide error handling. Set RPCRequest.willHandleError:true to have your
*  callback invoked regardless of whether there are errors; however, make sure your callback
*  properly handles malformed responses when RPCResponse.status is non-zero. See the
*  errorHandling below for more details.
* @type {DSCallback}
* @default null
*/
callback?: DSCallback;

/**
* Starting row of requested results, used only with fetch operations. If unset, 0 is assumed.
*  
*  Note that startRow and endRow are zero-based, inclusive at the beginning and exclusive at
*  the end (like substring), so startRow: 0, endRow: 1 is a request for the first record.
* @type {number}
* @default null
*/
startRow?: number;

/**
* End row of requested results, used only with fetch operations.
*  
*  Note that startRow and endRow are zero-based, inclusive at the beginning and exclusive at
*  the end (like substring), so startRow: 0, endRow: 1 is a request for the first record.
* @type {number}
* @default null
*/
endRow?: number;

/**
* Field name to sortBy, prefixed with optional "-" indicating descending sort. For example, to
*  sort by the field "userName" in ascending order, set sortBy to just
*  "userName". For descending sort on "userName", set sortBy to "-userName". 
*  
*  To sort by multiple fields, an array of field names is also supported. For example, to sort
*  by the field "department" in ascending order, followed by the field "userName" in descending
*  order, set sortBy to: 
*  
*  [ "department", "-userName" ]
*  
*  Additionally, this property supports an array of SortSpecifier objects. Setting 
*  sortBy to the following SortSpecifier array results in the same multi-level
*  sort mentioned above:
*  
*  
*  [
*  &nbsp;&nbsp;{ property: "department", direction: "ascending" },
*  &nbsp;&nbsp;{ property: "userName", direction: "descending" }
*  ]
*  
* @type {FieldName | Array<Partial<FieldName>> | Array<Partial<SortSpecifier>>}
* @default null
*/
sortBy?: FieldName | Array<Partial<FieldName>> | Array<Partial<SortSpecifier>>;

/**
* A mapping from field names to SummaryFunction to be applied
*  to each field.
*  
*  Valid only for an operation of type "fetch". See the 
*  serverSummaries for examples of usage.
* @type {Map<SummaryFunction>}
* @default null
*/
summaryFunctions?: Map<SummaryFunction>;

/**
* When using RPCManager.exportContent to produce a .pdf from a SmartClient UI, this property allows 
*  dynamic CSS to be passed to the server. Since the exportContent() system already provides 
*  a way to specify a custom skin or additional stylesheet for export, exportCSS should only be
*  used for small bits of CSS that are necessarily dynamic.
*  
*  For example, when printing a very wide page, such as a grid with many columns or a very wide chart,
*  you could send the string "@page {size: A4 landscape; }" as exportCSS to cause the
*  generated PDF to use landscape mode, so that all content fits without clipping.
* @type {string}
* @default null
*/
exportCSS?: string;

/**
* List of fields to group by when using serverSummaries.
*  
*  Valid only for an operation of type "fetch". See the 
*  serverSummaries for details and examples of usage.
* @type {Array<Partial<string>>}
* @default null
*/
groupBy?: Array<Partial<string>>;

/**
* The list of fields to return in the response, specified as a comma-separated string (eg,
*  "foo, bar, baz"). You can use this property to indicate to the server that 
*  you are only interested in a subset of the fields that would normally be returned.
*  
*  Note that you cannot use this property to request a superset of the fields that
*  would normally be returned, because that would be a security hole. It is possible to 
*  configure individual OperationBindings to return extra fields, but this must
*  be done in the server's DataSource descriptor; it cannot be altered on the fly
*  from the client side.
* @type {string}
* @default null
*/
outputs?: string;

/**
* For fetch, add or update operation, an optional comma separated list of fields to fetch from another,
*  related DataSource.
*  
*  Fields should be specified in the format 
*  "localFieldName!relatedDataSourceID.relatedDataSourceFieldName".
*  where relatedDataSourceID is the ID of the related dataSource, and
*  relatedDataSourceFieldName is the field for which you want to
*  fetch related values. The returned field values will be stored on 
*  the data returned to the client under the specified localFieldName.
*  Note that this will be applied in addition to any specified DSRequest.outputs.
*  
*  Note that as with DataSourceField.includeFrom, the related dataSource must be
*  linked to the primary datasource via a foreignKey relationship.
*  
*  Note additionalOutputs sent in request from the browser can be completely disabled in 
*  server_properties by setting
*  datasource.allowClientAdditionalOutputs:
*  
*    datasource.allowClientAdditionalOutputs: false
*  
*  In this case DSRequest.additionalOutputs sent from the browser will be cleared before 
*  executing request. Note that programatically configured additionalOutputs are always allowed, but 
*  you can't modify them from within a DMI method, so the only way to execute a request 
*  with additionalOutputs that differ from what was sent by the client is to create a new DSRequest
* @type {string}
* @default null
*/
additionalOutputs?: string;

/**
* Specifies should related updates have to be generated.
*  If not set (or set to null) then related updates will be generated only for
*  "add" and "update" operations. This property has to be explicitly set to true
*  to generate related updates for "remove" operation.
*  This functionality loads related objects from database thus affecting operation performance.
*  For "add" and "update" operations related objects are loaded anyway and performance impact is minimal.
*  Simple "remove" operation does not need to load related objects. Depending on database structure
*  performance impact can be significant if this property is set to true.
*  
*  Note this feature works only with Hibernate/JPA data sources, see 
*  jpaHibernateRelations for instructions how to 
*  set up relations. Table below uses "country -&gt; cities" sample data model.
*  
*  
*   
*    Relation and Operation type
*    Loading complete related objects
*    Loading related IDs
*   
*   
*    Many-to-one (cities -&gt; country): ADD/UPDATE
*    If operation affected country, for example new city added with existing 
*  countryId, then relatedUpdate is generated. Otherwise if city is added or updated without 
*  countryId set, relatedUpdate is not generated.
*  Note that if provided countryId does not exist, it is created.
*    Same as with complete related objects, except if provided countryId does not exist, 
*  then it is not created, but reset to NULL.
*   
*   
*    Many-to-one (cities -&gt; country): REMOVE
*    Removes record, depending on setting generates or not relatedUpdate for parent record.
*  For example if city record is removed and countryId is sent to the server in remove request, then 
*  country record will be generated in relatedUpdates.
*   
*   
*    One-to-many (country -&gt; cities): ADD/UPDATE
*    If add or update operation provides value sets for cities as well as for country, then 
*  cities are created/updated if necessary and relatedUpdates are generated.
*  Note that all fields in cities value sets can be sent to server.
*    Same as with complete related objects, except you can only sent primary key values for cities.
*   
*    One-to-many (country -&gt; cities): REMOVE
*    Removes country, depending on setting returns or not relatedUpdates for the cities of removed country, 
*  which can be either REMOVE operations of all cities if cascade enabled, or UPDATE operations setting countryId=null to 
*  all cities if cascade is disabled
*   
*  
*  
*  Note that Many-to-Many works the same way as One-to-Many.
* @type {boolean}
* @default null
*/
generateRelatedUpdates?: boolean;

/**
* For "fetch" operations, how search criteria should be interpreted for text fields: one of
*  "exact" for exact match, "exactCase" for case-sensitive exact match, "startsWith" for 
*  matching at the beginning only, or "substring" for substring match. All 
*  textMatchStyle settings except "exactCase" are case-insensitive; use
*  AdvancedCriteria for greater control over matching.
*  
*  This property defaults to the value of DataSource.defaultTextMatchStyle if it is not
*  explicitly provided on the DSRequest. Note, however, that DSRequests issued 
*  by ListGrids and other DataBoundComponent will generally have a 
*  setting for textMatchStyle on the component itself 
*  (see ListGrid.autoFetchTextMatchStyle, for example).
*  
*  This setting is respected by the built-in server-side connectors for SQL, JPA and Hibernate.
*  A custom server-side DataSource implementation should generally respect this flag as well,
*  or server-side filtering will not match client-side filtering, which will require
*  ResultSet.useClientFiltering, a huge performance
*  loss.
* @type {TextMatchStyle}
* @default null
*/
textMatchStyle?: TextMatchStyle;

/**
* For an update or remove operation, the original values from the
*  record that is being updated or removed. oldValues is automatically added to
*  DSRequests submitted by DataBound Components. Available on the server via
*  DSRequest.getOldValues().
*  
*  The server can compare the oldValues to the most recent stored values in order
*  to detect that the user was looking at stale values when the user submitted changes (NOTE:
*  this means of detecting concurrent edit is sometimes called "optimistic concurrency" or
*  "long transactions"). 
*  
*  In applications where a policy of "last update wins" is not appropriate when updating
*  certain fields, special UI can be shown for this case. For example, on detecting concurrent
*  edit, the server may send back a special dsResponse.status code that the client
*  application detects, offering the user a choice of proceeding with the operation,
*  discarding edits, or reconciling new and old values in a special interface.
*  
*  See the concurrentEdits overview for more on handling concurrent edits in
*  SmartClient DataSources.
* @type {Record}
* @default null
*/
oldValues?: Record;

/**
* For requests submitted by a DataBoundComponent, the Canvas.ID of the
*  submitting component.
*  
*  This ID will be present for operations including automatic saves by a ListGrid 
*  editing, or calls to
*  DynamicForm.saveData. It will not be present for a direct
*  call to a DataSource method such as DataSource.fetchData.
*  
*  Note this is the component's String ID - you can retrieve the component itself
*  via Canvas.getById.
*  
*  This property should be used for debugging purposes only - do not use it to trigger
*  differences in server-side behavior, instead, use DSRequest.operationId because only
*  operationId is considered when assessing
*  dsRequestEquivalence.
* @type {string}
* @default null
*/
componentId?: string;

/**
* When a DataBoundComponent sends a DSRequest, the
*  dsRequest.operationId will be automatically picked up from the
*  fetchOperation, addOperation, etc properties of the
*  DataBoundComponent. 
*  
*  The operationId serves as an identifier that you can use to create
*  variations on the 4 basic DataSource operations that are used by different components in
*  different parts of your application. For example, you may be using a standard
*  fetch operation in one part of your application, however on another screen you
*  want to perform a fetch operation on the same DataSource but interpret search
*  criteria differently (eg full text search). 
*  
*  If you declare more than one OperationBinding for the same
*  OperationBinding.operationType, you can specify an operationId
*  OperationBinding.operationId which will cause that
*  operationBinding to be used for dsRequests containing a matching operationId.
*  This allows all the possible settings of an operationBinding, including
*  OperationBinding.wsOperation or dmiOverview settings, to be switched on a
*  per-component or per-request basis. 
*  
*  For example, by setting the fetchOperation on a particular ListGrid, you could
*  cause it to invoke a different server method via DMI, different
*  OperationBinding.dataURL or different 
*  OperationBinding.wsOperation.
*  
*  The operationId can also be directly received by the server in order to affect
*  behavior. When using the SmartClient Server, operationId can be accessed via
*  dsRequest.getOperationId(). The RestDataSource will also send the
*  operationId to the server as part of the
*  RestDataSource.metaDataPrefix. 
*  
*  Note that if you DataSource.fetchData a DataSource operation, you can
*  also specify operationId via the requestProperties parameter.
*  
*  Note that the operationId has special significance in terms of whether two
*  DSRequests are considered equivalent for caching and synchronization purposes - see
*  dsRequestEquivalence.
* @type {string}
* @default null
*/
operationId?: string;

/**
* For a databound Tree.isMultiLinkTree, this is the 
*  operationId to use for the separate fetch on the ResultTree.linkDataSource
*  that will be generated if LinkDataFetchMode is "separate". This property 
*  overrides the ResultTree.linkDataFetchOperation property on 
*  ResultTree, for this fetch only.
*  
*  Ignored if this DSRequest is not a fetch against a multi-link tree.
* @type {string}
* @default null
*/
linkDataFetchOperation?: string;

/**
* DataSource.dataProtocol for this particular request.
*  
*  Note: Typically developers should use DataSource.operationBindings
*  to specify an explicit data protocol for a request.
*  
*  One exception: advanced developers may wish to have a custom 
*  DataSource.transformRequest with entirely client-side
*  handling for some requests. This may be achieved by setting the request's
*  dataProtocol to DSProtocol within transformRequest,
*  and also triggering application code which will fire DataSource.processResponse
*  when complete.
*  
*  The DataSource.getDataProtocol method may be used to determine what data protocol
*  will be used to handle a specific request based on this property (if set), otherwise the
*  settings at the OperationBinding.dataProtocol or 
*  DataSource.dataProtocol levels.
* @type {DSProtocol}
* @default null
*/
dataProtocol?: DSProtocol;

/**
* Automatically generated unique ID for this request. This ID will be required by developers 
*  making use of the DSProtocol.
* @type {string}
* @default "varies"
*/
requestId?: string;

/**
* An object to be held onto for the duration of the DSRequest turnaround to track
*  application-specific context.
*  
*  When a DataSource request completes, the clientContext is available in the
*  DSCallback as dsResponse.clientContext. The
*  clientContext is never sent to the server.
*  
*  The clientContext is useful for holding onto state that will be used when the
*  DSCallback fires, such as the name of a component that will receive the
*  returned data.
* @type {object}
* @default "{}"
*/
clientContext?: object;

/**
* For advanced use in integrating trees that 
*  TreeGrid.loadDataOnDemand with web services,
*  parentNode is automatically set in "fetch" DSRequests issued by a
*  databound TreeGrid that is loading children for that parentNode.
*  
*  This is sometimes needed if a web service requires that additional properties beyond the
*  ID of the parentNode must be passed in order to accomplished level-by-level loading.
*  A custom implementation of DataSource.transformRequest can access
*  dsRequest.parentNode and add any such properties to DSRequest.data.
*  
*  parentNode will also be automatically set by a TreeGrid performing databound
*  reparenting of nodes, as implemented by TreeGrid.folderDrop.
*  
*  This property can only be read. There is no meaning to setting this property
*  yourself.
* @type {TreeNode}
* @default null
*/
parentNode?: TreeNode;

/**
* For advanced use in integrating trees that 
*  ResultTree.loadDataOnDemand with web services,
*  the ResultTree that issued this "fetch" DSRequest is automatically made available as the
*  resultTree property.
*  
*  This property can only be read. There is no meaning to setting this property
*  yourself.
* @type {ResultTree}
* @default null
*/
resultTree?: ResultTree;

/**
* This property is for advanced use in integrating trees that 
*  TreeGrid.loadDataOnDemand using data paging.
*  When this flag is set, a server fetch operation is expected to return all of the tree nodes
*  that either match the provided criteria or have one or more children that match the
*  criteria.
*  
*  A ResultTree with ResultTree.fetchMode and with
*  ResultTree.keepParentsOnFilter enabled will automatically set
*  this property to true on all DSRequests that it sends to the server.
*  
*  Currently, no built-in server-side connectors (SQL, JPA, Hibernate) implement support for
*  the keepParentsOnFilter flag.
* @type {boolean}
* @default null
*/
keepParentsOnFilter?: boolean;

/**
* For advanced use in integrating dataset paging with web services,
*  the ResultSet that issued this "fetch" DSRequest is automatically made available as the
*  resultSet property.
*  
*  This property can only be read. There is no meaning to setting this property
*  yourself.
* @type {ResultSet}
* @default null
*/
resultSet?: ResultSet;

/**
* This is a per-request flag for explicitly controlling whether the cache is used (bypassing
*  it when not wanted, or using it when settings would indicate otherwise). See 
*  DataSource.cacheAllData, DataSource.cacheAllOperationId and
*  DataSource.cacheAcrossOperationIds for caching management for all requests of a 
*  dataSource.
* @type {boolean}
* @default null
*/
shouldUseCache?: boolean;

/**
* When useFlatFields is set for a request to be sent to a WSDL web service, when
*  creating the input XML message to send to the web service, properties in
*  DSRequest.data will be used as the values for XML elements of the same name, at
*  any level of nesting.
*  
*  useFlatFields allows you to ignore gratuitous XML message structure, such as
*  extra levels of nested elements, and provides some insulation against changes in the
*  required structure of the input message.
*  
*  For example, given this input message:
*  
*  &lt;FindServices&gt;
*    &lt;searchFor&gt;search text&lt;/searchFor&gt;
*    &lt;Options&gt;
*      &lt;caseSensitive&gt;false&lt;/caseSensitive&gt;
*    &lt;/Options&gt;
*    &lt;IncludeInSearch&gt;
*      &lt;serviceName&gt;true&lt;/serviceName&gt;
*      &lt;documentation&gt;true&lt;/documentation&gt;
*      &lt;keywords&gt;true&lt;/keywords&gt;
*    &lt;/IncludeInSearch&gt;
*  &lt;/FindServices&gt;
*  
*  If useFlatFields were not set, in order to fill out this message
*  correctly, request.data would need to be:
*  {
*   searchFor: "search text",
*   Options : {
*     caseSensitive: false,
*   },
*   IncludeInSearch : {
*     serviceName: true,
*     documentation : true,
*     keywords : true
*   }
*  }
*  However if useFlatFields were set, request.data could be just:
*  {
*   searchFor: "search text",
*   caseSensitive: false,
*   serviceName: true,
*   documentation : true,
*   keywords : true
*  }
*  useFlatFields is often set when the input data comes from a DynamicForm
*  to avoid the cumbersome and fragile process of mapping input fields to an XML structure.
*  
*  OperationBinding.useFlatFields can also be set to cause all dsRequests of a
*  particular type to useFlatFields automatically.
*  
*  For DataBoundComponent, 
*  DataBoundComponent.useFlatFields can be set use "flattened"
*  binding to fields of a WSDL message or XML Schema.
*  
*  Note that useFlatFields is not generally recommended for use with XML input
*  messages where multiple simple type fields exist with the same name, however if used in this
*  way, the first field to use a given name wins. "first" means the first field encountered in a
*  depth first search. "wins" means only the first field will be populated in the generated
*  XML message.
* @type {boolean}
* @default null
*/
useFlatFields?: boolean;

/**
* Cause the DSRequest.useFlatFields XML serialization behavior to be used
*  for all soap headers in the request. See also DSRequest.headerData.
* @type {boolean}
* @default null
*/
useFlatHeaderFields?: boolean;

/**
* For DataSources using SOAP messaging with a WSDL web service, data to be serialized to form
*  SOAP headers, as a map from the header part name to the data. See
*  WSRequest.headerData for more information.
*  
*  SOAP headers typically contain request metadata such as a session id for authentication, and
*  so dsRequest.headerData is typically populated by
*  DataSource.transformRequest, or, for data that applies to every request sent to the
*  server, by WebService.getHeaderData.
*  
*  Note that this only applies to SOAP headers. General HTTP headers for requests may be
*  modified using RPCRequest.httpHeaders.
* @type {object}
* @default null
*/
headerData?: object;

/**
* When set, causes the results of the DSRequest to be exported to a file, whose name 
*  and format are indicated by DSRequest.exportFilename and 
*  DSRequest.exportAs respectively. When no exportFilename is provided, the default is
*  Results.csv and the default value of exportAs is csv. 
*  
*  The export field-list can also be configured, see DSRequest.exportFields. Formats 
*  for exported date and numeric are controlled by several settings - see 
*  exportFormatting for an overview.
*  
*  Once the operation completes, DSRequest.exportDisplay specifies whether the exported
*  data should be downloaded to the file-system or displayed in a new window. The default value
*  of exportDisplay is "download" which displays the Save As dialog. See ExportDisplay 
*  for more information.
*  
*  You can configure the style of LineBreakStyle to use when 
*  generating the output, the DSRequest.exportDelimiter to use when exporting
*  to CSV and the DSRequest.exportTitleSeparatorChar to use in 
*  field-titles when exporting to XML.
*  
*  Additionally, you can output arbitrary text before and after the exported data by setting 
*  DSRequest.exportHeader and DSRequest.exportFooter.
*  
*  Note that for security reasons, an export initiated using dsRequest properties does not
*  provide support for JSON format (see 
*  this post for more detail). 
*  However, you can use operationBinding.exportAs:"json" in a server-side .ds.xml file to force 
*  JSON export to be allowed.
*  
*  As well as setting dsRequest.exportResults and related properties, exports can be initiated
*  in two other ways, via OperationBindings and via custom server code which sets 
*  export-related properties on the DSResponse. Both of those methods support exporting
*  to JSON format.
*  
*  Format Examples
*  XML format
*  
*    &lt;List&gt;
*      &lt;Object&gt;
*        &lt;id&gt;10101&lt;/id&gt;
*        &lt;displayName&gt;Record 10101&lt;/displayName&gt;
*      &lt;/Object&gt;
*   &lt;/List&gt;
*  
*  JSON Format
*  
*    [
*      { id: 10101, displayName: "Record 10101" }
*    ]
*  
*  CSV Format
*  
*    id,displayName
*    10101,"Record 10101"
*  
* @type {boolean}
* @default false
*/
exportResults?: boolean;

/**
* The format in which the data should be exported. Note that 'JSON' is not allowed as a
*  client-side option. See ExportFormat for more information.
* @type {ExportFormat}
* @default "csv"
*/
exportAs?: ExportFormat;

/**
* The name of the file to save the exported data into. If 
*  DSRequest.exportToFilesystem is set, this is the name of the 
*  file the server creates on its filesystem. If DSRequest.exportToClient
*  is set, this is the filename that will appear to the browser.
*  
*  If the exportFilename that you specify does not include an extension, one will be added to
*  it based on the ExportFormat specified by DSRequest.exportAs. Filename is
*  forced to have the correct extension to work around bugs in IE, but if you don't want the
*  filename to be manipulated, use "custom" ExportFormat, see example.
* @type {string}
* @default null
*/
exportFilename?: string;

/**
* The image format in which the SVG graphic should be exported.
* @type {ExportImageFormat}
* @default "png"
*/
exportImageFormat?: ExportImageFormat;

/**
* If exporting in ExportImageFormat, the output JPEG quality level.
*  This is a number from 0 to 1, with 1 representing the best quality and 0 representing the
*  least quality but smallest file size.
* @type {Float}
* @default null
*/
exportImageQuality?: Float;

/**
* If DSRequest.exportToFilesystem is set, optionally specifies a 
*  path to use when saving the file. This path is relative to the default export path, which
*  is set using the server_properties setting export.location; this
*  is the project webRoot by default. For example, with the default setting of 
*  export.location, an exportPath of "shared/ds" and 
*  an DSRequest.exportFilename of "exportedData.csv",
*  SmartClient Server would export to file $webRoot/shared/ds/exportedData.csv.
*  
*  If you do not specify this property, SmartClient Server will export to the file indicated 
*  by exportFilename directly in the default export location.
*  
*  This property is only applicable when DSRequest.exportToFilesystem
*  is set.
* @type {string}
* @default null
*/
exportPath?: string;

/**
* If set, SmartClient Server will export data to a file on the server filesystem. The
*  file we export to is determined by the DSRequest.exportFilename and 
*  DSRequest.exportPath. Note that filesystem exports are disabled by 
*  default, for security reasons. To enable them, set export.allow.filesystem to
*  true in your server.properties file. If you enable filesystem exports, you 
*  should also consider setting a default export path, as described in the 
*  DSRequest.exportPath documentation.
*  
*  Note that it is perfectly valid to specify both this property and 
*  DSRequest.exportToClient; in this case the data is both exported to a
*  file on the server filesystem and downloaded to the client. If you specify 
*  neither property, the export no-ops.
*  
*  It is possible to redirect the filesystem export to make use of an OutputStream
*  you provide. You use this when you want to make some use of the export document other than 
*  writing it to a disk file - for example, attaching it to an email or writing it to a 
*  database table. See the server-side Javadocs for DSRequest.setExportTo().
* @type {boolean}
* @default false
*/
exportToFilesystem?: boolean;

/**
* If set to true (the default), SmartClient Server will export data back to the client,
*  either as a file download or as content in a new browser window, depending on the setting of
*  DSRequest.exportDisplay.
*  
*  Setting this property to false disables this. This may be useful when developers 
*  wish to DSRequest.exportToFilesystem,
*  but do not need to display it in the browser in response to the export request.
*  
*  Note that it is perfectly valid to have both this property and 
*  DSRequest.exportToFilesystem set to true; in this case the data is both 
*  exported to a file on the server filesystem, and downloaded to the client. If you specify 
*  neither property, the export no-ops.
* @type {boolean}
* @default true
*/
exportToClient?: boolean;

/**
* The character with which to replace spaces in field-titles when exporting to XML. If not
*  specified in the request, the server uses "".
* @type {string}
* @default null
*/
exportTitleSeparatorChar?: string;

/**
* Specifies whether the exported data will be downloaded as an attachment or displayed in a 
*  new browser window. See ExportDisplay for more information.
* @type {ExportDisplay}
* @default "download"
*/
exportDisplay?: ExportDisplay;

/**
* The style of line-breaks to use in the exported output. See LineBreakStyle for
*  more information.
* @type {string}
* @default null
*/
lineBreakStyle?: string;

/**
* The character to use as a field-separator in CSV exports. The default delimiter is comma.
* @type {string}
* @default ","
*/
exportDelimiter?: string;

/**
* When you erxport a ListGrid that has ListGrid.headerSpans,
*  should headerSpans also be exported. See DSRequest.exportSpanTitleSeparator for details of
*  of what it means to export headerSpans to different export targets.
*    
*  Note that DSRequest.exportPropertyIdentifier controls whether field names or titles
*  are appended to the headerSpan titles (and used for fields without headerSpans).
* @type {boolean}
* @default true
*/
exportShowHeaderSpanTitles?: boolean;

/**
* When you export a ListGrid that has ListGrid.headerSpans
*  defined and DSRequest.exportShowHeaderSpanTitles is true, the behavior is dependent
*  on the export type. Direct exports to Excel formats (both XLS and OOXML) place the 
*  headerSpans in merged cells in the spreadsheet, giving the same visual effect as the 
*  original ListGrid. This is not possible with exports to CSV format; instead, we alter the 
*  exported headers so that they contain the titles of the ancestor headerSpan(s).
*  
*  For example, if you had a field titled "Population" inside a headerSpan titled "National",
*  nested inside another headerSpan titled "Demographics", that would result in the exported
*  field being titled "Demographics - National - Population".
*  
*  The exportSpanTitleSeparator property allows you to override the separator 
*  string used when constructing these amalgamated headers.
* @type {string}
* @default " - "
*/
exportSpanTitleSeparator?: string;

/**
* The list of field names to export. If provided, the field list in the exported output is 
*  limited and sorted as per the list.
*  
*  If exportFields is not provided: 
*  
* - If we are exporting via DSRequest.exportData, the field list in the exported
*  output is every non-hidden field defined in the DataSource, in DataSource definition order
*  
* - If we are exporting via ListGrid.exportClientData and we are not 
*  exporting to OOXML, or we are exporting to OOXML but we are not 
*  DSRequest.exportStreaming, the field list in the exported output is based on the 
*  client data sent up, taking every row into account (so if there is a value for field "foo"
*  only in row 57, we will output a column "foo", the cells of which are empty except for 
*  row 57)
*  
* - If we are exporting via ListGrid.exportClientData and we are 
*  exporting to OOXML and streaming is in force (the default for OOXML), the field list in 
*  the exported output is based on the client data sent up, taking just the first row into 
*  account (so if there is a value for field "foo" only in row 57, we will not output a column
*  "foo" at all) 
*  
* @type {Array<Partial<string>>}
* @default null
*/
exportFields?: Array<Partial<string>>;

/**
* Optional text to appear at the beginning of the file.
* @type {string}
* @default null
*/
exportHeader?: string;

/**
* This property allows omitting column names from CSV and Excel exports (no effect on JSON or XML exports).
* @type {boolean}
* @default false
*/
exportHeaderless?: boolean;

/**
* Optional text to appear at the end of the file.
* @type {string}
* @default null
*/
exportFooter?: string;

/**
* Only applies to request properties passed to ListGrid.exportClientData.
*  If specified this property contains an arbitrary set of data to be exported.
* @type {Array<Partial<Record>>}
* @default null
*/
exportData?: Array<Partial<Record>>;

/**
* This flag has a different meaning depending on whether you are doing a client-driven or
*  server-driven export.
*  
*  For ListGrid.exportClientData calls (client-driven), ordinarily
*  any fields that have a ListGridField.displayField defined
*  have the value of that displayField exported, rather than the underlying value in the 
*  ListGridField.valueField. If you set the exportValueFields
*  property, we export both the underlying value and the displayField value.
*  
*  Again for exportClientData() calls, any fields that have a
*  ListGridField.valueMap defined ordinarily have the mapped value of the 
*  field exported, rather than the underlying data value. If you set the 
*  exportValueFields property, we instead export the underlying data value. 
*  Note, there is only one field in this scenario, not a valueField and a 
*  separate displayField, so we export either the underlying data value or
*  the mapped value, not both as in the displayField/valueField case 
*  described above.
*  
*  For DataBoundComponent.exportData calls (server-driven), we ordinarily
*  export the underlying data value of all fields. However, if you set the 
*  exportValueFields property explicitly to false, any fields that
*  have a DataSource-defined DataSourceField.valueMap will have the mapped
*  value exported instead. This is similar to the client-side treatment of valueMaps, except
*  that the defaults are reversed.
*  
*  For exportData() calls, if we encounter a field that has an in-record 
*  DataSourceField.displayField declared in the DataSource, by 
*  default we export both the underlying value and the display value, so you end up with two
*  columns in the exported data for that field. You can influence this by setting 
*  exportValueFields explicitly: if set true we export only the 
*  value field, and if set false we export only the display field. Note, the 
*  reason for the similar but not identical behavior of this flag between exportData()
*  and exportClientData() is backwards compatibility.
* @type {boolean}
* @default null
*/
exportValueFields?: boolean;

/**
* When exporting via ListGrid.exportClientData to an XLS or
*  OOXML spreadsheet, forces dates to export as a string rather than a true date
*  value.
*  
*  If a date value is provided to a spreadsheet as a string, Excel or other spreadsheet
*  applications may not recognize them as being date values that are valid for use in
*  date-specific functions in formulas, filters, etc.
*  
*  For this reason, the default behavior of exportClientData is to provide date
*  values to the spreadsheet as true date values. If
*  FormatString are provided via properties like
*  DataSourceField.format these will be translated to Excel /
*  OpenOffice format strings and used when generating spreadsheets. Other formatting logic,
*  such as ListGridField.formatCellValue, will not be used since they
*  cannot be automatically translated to an Excel format string. If no translatable format
*  string is available, date values will be provided to the spreadsheet with no formatter and
*  the spreadsheet program's default formatting for date values will be used.
*  
*  If exportDatesAsFormattedString is set to true, date fields will appear as
*  strings that exactly match the formatting shown in the DataBoundComponent.
*  As noted above, this means the spreadsheet program will not recognize the value as
*  a date.
* @type {boolean}
* @default null
*/
exportDatesAsFormattedString?: boolean;

/**
* When exporting via ListGrid.exportClientData to an XLS or
*  OOXML spreadsheet, forces numbers to export as a string rather than a true
*  numerical value.
*  
*  If a number is provided to a spreadsheet as a string, Excel or other spreadsheet
*  applications may not recognize them as being numbers that are valid for use in
*  numerical formulas, filters, etc.
*  
*  For this reason, the default behavior of exportClientData is to provide
*  numerical values to the spreadsheet as true numbers. If
*  FormatString are provided via properties like
*  DataSourceField.format these will be translated to Excel /
*  OpenOffice format strings and used when generating spreadsheets. Other formatting logic,
*  such as ListGridField.formatCellValue, will not be used since they
*  cannot be automatically translated to an Excel format string. If no translatable format
*  string is available, numbers will be provided to the spreadsheet with no formatter and
*  the spreadsheet program's default formatting for numerical values will be used.
*  
*  If exportNumbersAsFormattedString is set to true, numbers will appear as
*  strings that exactly match the formatting shown in the DataBoundComponent.
*  As noted above, this means the spreadsheet program will not recognize the value as
*  a number.
* @type {boolean}
* @default null
*/
exportNumbersAsFormattedString?: boolean;

/**
* For server-side export with ExportFormat "xls" or "ooxml" only, timezone to use
*  when saving values from FieldType "datetime" to the spreadsheet.
*  
*  This setting exists because MS Excel&trade; has no concept of providing a true datetime value
*  that is timezone-independent and will display in the local timezone where the Excel program is
*  launched. This setting sets the timezone of the Excel workbook, so that it will display dates
*  in the same timezone regardless of the local timezone where the Excel program is launched.
*  Alternative approach is to set
*  DSRequest.exportDatesAsFormattedString
*  telling datetime values must be provided as a rendered string, which implies
*  rendering in a particular timezone when the spreadsheet is generated.
*  
*  exportTZ can either be specified as a timezone offset in the same format
*  expected by Time.setDefaultDisplayTimezone (for example, "+01:00" for one hour after
*  GMT) or as the special constants "client" (meaning the current client display timezone) or
*  "server" (meaning the timezone of the server).
*  
*  Default if unspecified is "server".
*  
*  This setting does not affect fields of type "date" or "time", which are timezone-independent
*  values. See dateFormatAndStorage for more information on how SmartClient
*  handles date, time and datetime values.
*  
*  All non-spreadsheet export formats always use UTC. This setting also does not affect
*  client-driven exports (DataSource.exportClientData), which always use client-side
*  time.
* @type {string}
* @default null
*/
exportTZ?: string;

/**
* When exporting to OOXML format (this is the standard file format used by Excel 2007 and 
*  later), we default to using streaming mode, for memory efficiency. You can override this 
*  for individual exports by setting this flag false. You may wish to do this if you need to
*  grab the spreadsheet object in a DMI and do something with it. The underlying object in 
*  use - POI's SXSSFWorkbook - is intended for write only and cannot usefully be
*  read.
*  
*  You can switch off Excel streaming altogether by setting "excel.useStreaming" false in 
*  server.properties.
*  
*  Note, OOXML is the only native Excel format that supports streaming: when exporting to
*  the older XLS format, we build the spreadsheet in its entirety in server-side memory 
*  before writing it to disk or returning it to the client. This is unlikely to change:
*  streaming the XLS format is impractical bcause it is a self-referential binary format, and
*  in any case the problem of huge exports overflowing JVM memory is less likely to arise with 
*  XLS, because it is innately limited to 65535 rows.
* @type {boolean}
* @default true
*/
exportStreaming?: boolean;

/**
* Mode of validation for entered data.
* @type {ValidationMode}
* @default "full"
*/
validationMode?: ValidationMode;

/**
* Indicates that a validation request is being made for a record that will ultimately be 
*  saved with an "add" request, as opposed to an "update" request. This context is necessary
*  for some validators because the nature of the validation depends on whether we are adding
*  or updating a record. The system sets this flag when processing interim validations, such
*  as those fired when DynamicForm.validateOnChange is in force.
* @type {boolean}
* @default null
*/
pendingAdd?: boolean;

/**
* If true, results will be streamed on the server, rather than all records being read into 
*  server memory at once; this approach is appropriate for retrieving or exporting large
*  datasets without swamping the server. 
*  
*  Although this property can be set without any particular concerns (small datasets can be 
*  streamed just as readily as large ones), bear in mind that although streaming enables the
*  processing of very large datasets, processing and downloading very large datasets in a 
*  normal client/server flow will very rarely give an acceptable user experience. Streaming
*  is of more practical use in a batch setting - for example, a disconnected 
*  DSRequest.exportToFilesystem.
*  
*  Note that streaming requires specific server support; of SmartClient's built-in DataSource
*  types, only SQLDataSource is able to stream results. This property is ignored
*  by other DataSource types. If you wish to implement the necessary server-side behavior to 
*  support streaming with a custom DataSource, see the the server-side Javadocs for 
*  DSResponse.hasNextRecord() and DSResponse.nextRecordAsObject().
*  
*  See also the server-side documentation for DSResponse, 
*  SQLDataSource and StreamingResponseIterator.
*  
*  Note, that streaming results does not support fields with SummaryFunction 
*  on non-Oracle databases. Such fields will be skipped.
* @type {boolean}
* @default false
*/
streamResults?: boolean;

/**
* Sets DataSource.progressiveLoading for this particular
*  request, overriding the OperationBinding- and DataSource-level settings. This setting
*  overrides the DataSource.progressiveLoadingThreshold
*  setting as well, meaning that if DSRequest.progressiveLoading is explicitly
*  set to false SmartClient won't automatically switch to loading data progressively
*  even if DataSource.progressiveLoadingThreshold is exceeded.
*  
*  Note that this setting applies only to fetch requests - it has no effect if specified
*  on any other kind of request.
* @type {boolean}
* @default null
*/
progressiveLoading?: boolean;

/**
* A set of key:value pairs, mapping field names to expressions that will be evaluated 
*  server-side to derive a value for that field. This property allows for client-driven 
*  transactionChaining, with some restrictions for security
*  reasons:
*  
* - Normal OperationBinding.values for a 
*  field take precedence over this property, so server-defined rules cannot be overridden 
*  from the client
*  
* - Arbitrary Velocity expressions are not allowed in DSRequests sent from the client 
*  (fieldValueExpressions is also a valid property on a server-side DSRequest,
*  and normal Velocity expressions are allowed in that case - see the server-side
*  Javadoc for DSRequest.setFieldValueExpressions()). For client-originated 
*  requests, only the following bindings are allowed - see the 
*  velocitySupport for details of what these values mean:
*   
* - $currentDate
*   
* - $currentDateUTC
*   
* - $transactionDate
*   
* - $transactionDateUTC
*   
* - $userId
*   
* - $masterId - see DSRequestModifier.value for details
*   
* - References to specific fields in prior responses, via $responseData.first and 
*     $responseData.last, with or without parameters. For example, 
*     $responseData.first("myDataSource", "fetch")[0].myField. See the
*     velocitySupport for details of $responseData
*   
* - References to certain metadata properties of prior responses, via $responses.first 
*     and $responses.last, with or without parameters. For example, 
*     $responses.last("myDataSource", "fetch").totalRows. Note that the only 
*     properties allowed in a client-driven fieldValueExpression are:
*     "startRow", "endRow", "totalRows" and "status"; this restriction does not apply to 
*     server-driven fieldValueExpressions. See the Velocity overview for 
*     details of $responses
*  
*  
* - Normal DataSourceField.editRequiresRole apply: if a
*  field is not valid for writing, its fieldValueExpression will be ignored.
*  Again, this only applies to client-originated requests.
*  
*  Note, it is possible to globally disable fieldValueExpression in 
*  client-originated requests by setting a flag in your server.properties file:
*   dataSource.allowClientFieldValueExpressions: false
* @type {object}
* @default null
*/
fieldValueExpressions?: object;





}


/**
* An object that modifies a dsRequest based on several available 
*  values, including preceding responses from the same queue. 
*  
*  Some elements of this feature are only available with Power or better licenses. See
*  smartclient.com/product for details.
*  
*  You provide a list of DSRequestModifiers as the 
*  OperationBinding.criteria or OperationBinding.values
*  of an OperationBinding. The ability to modify a request based on responses to 
*  earlier requests in the same queue is called transactionChaining.
*  
*  A DSRequestModifier consists of a DSRequestModifier.fieldName, usually a
*  DSRequestModifier.value and possibly an DSRequestModifier.operator
*  and DSRequestModifier.start and/or DSRequestModifier.end values 
*  (applicable to advanced criteria only). The value, start and end settings can be static, or - 
*  with Power or better licenses - they can be expressions in the Velocity template language, 
*  which will be resolved at runtime, immediately before the DSRequest is executed.
*  
*  In addition to the standard velocitySupport, special Velocity 
*  variables are available for the value field when used in a &lt;values&gt; declaration - 
*  see DSRequestModifier.value for details.
*  
*  See below some examples of OperationBinding.criteria declarations:
*  
*  &lt;operationBindings&gt;
*    &lt;operationBinding operationType="fetch" operationId="..."&gt;
*      &lt;criteria fieldName="lifeSpan" value="10"/&gt;
*      &lt;criteria fieldName="scientificName" value="Gazella thomsoni"/&gt;
*    &lt;/operationBinding&gt;
*    
*    &lt;operationBinding operationType="fetch" operationId="..."&gt;
*      &lt;criteria fieldName="lifeSpan" operator="greaterThan" value="10" /&gt;
*    &lt;/operationBinding&gt;
*  
*    &lt;operationBinding operationType="fetch" operationId="..."&gt;
*      &lt;criteria _constructor="AdvancedCriteria" operator="or"&gt;
*        &lt;criteria&gt;
*          &lt;Criterion fieldName="lifeSpan" operator="greaterThan" value="10" /&gt;
*          &lt;Criterion fieldName="scientificName" operator="contains" value="Octopus" /&gt;
*        &lt;/criteria&gt;
*      &lt;/criteria&gt;
*    &lt;/operationBinding&gt;
*  &lt;/operationBindings&gt;
*  
*  
*  And an example of OperationBinding.values declarations:
*  
*  &lt;operationBinding operationType="add"&gt;
*    &lt;values fieldName="orderID" value="$responseData.last('queuedAdd_order','add').orderID" /&gt;
*  &lt;/operationBinding&gt;
*  
*  
*  DSRequestModifiers are intended partly as a security mechanism, so like declarativeSecurity 
*  features, they apply to requests from browsers, but by default do not apply to requests initiated by server code. 
*  To have DSRequestModifiers apply to a server-initiated DSRequest, use either setRPCManager() or setClientRequest().
*  
*  See the Master/Detail Add Example 
*  to see this in action.
*/
interface DSRequestModifier {
/**
* The name of the field to add or replace on the DSRequest - whether this appears in the DSRequest's
*  values or criteria depends on whether this is part of a OperationBinding.values or
*  OperationBinding.criteria attribute.
* @type {FieldName}
* @default null
*/
fieldName?: FieldName;

/**
* The name of the operator to apply when constructing criteria. This property only applies 
*  to criteria; it is ignored if specified as part of a OperationBinding.values 
*  attribute.
* @type {string}
* @default null
*/
operator?: string;

/**
* The value to assign to the field named by DSRequestModifier.fieldName. This
*  value can be static, and for Pro licenses that is the only option. With Power and better
*  licenses, this value can be an expression in the Velocity template language. In this latter
*  case, all the standard velocitySupport provided by
*  SmartClient Server are available to you.
*  
*  Note, dsRequestModifier values are evaluated during DSRequest 
*  setup, before the request's execute() method is called. This means that 
*  variables added to the Velocity context by calling addToTemplateContext()
*  from a dmiOverview method or 
*  DataSource.serverConstructor will not be available.
*  In this case, you can either
*  
* - Apply the variable values directly to the DSRequest's criteria and values
*  from your Java code. See the server-side Javadoc for DSRequest
*  
* - Add your template variables to the DSRequest's template context before 
*  dsRequestModifier evaluation takes place, in a custom override of 
*  serverDataIntegration
*  
*  masterId and responseData
*  There is also one additional Velocity context variable available in this specific case: 
*  $masterId. If there is a DataSourceField.foreignKey from the 
*  DataSource for the current operation to another DataSource for which an add or update 
*  operation has taken place earlier in the queue, this is the value of the target field of 
*  the foreign key, taken from the response data of that earlier operation (the most recent 
*  one, if there are several). This is useful because it will typically yield the (possibly 
*  just generated) primary key of the "master" record.
*  
*  Consider a queued batch of "add" operations for an order header and its details. The 
*  detail additions need to know the unique primary key that was assigned to the order, but 
*  this will typically be generated at the time of inserting the order row into the database,
*  so it is not known up-front. However, this value will be in the response to the DSRequest
*  that added the order header, so it is accessible via $responseData; if there is a 
*  declared foreign key relationship from the detail DataSource to the header DataSource, the 
*  header's unique key value will also be accesible as $masterId. See this example: 
*  ${isc.DocUtils.linkForExampleId('queuedAdd')}.
*  
*  $responseData - which is an instance of 
*  com.isomorphic.velocity.ResponseDataHandler - exposes various overloads of
*  first() and last() APIs that can be called to obtain the actual 
*  record data of prior responses. These methods return an instance of 
*  com.isomorphic.velocity.ResponseDataWrapper, which allows convenient handling
*  of response data whether it is a single record or a list. Response data can be treated as
*  a single record even if it is a List, so you can access the response data directly, with no
*  need for an index; when you do this, and the data is actually a List or array, you get the 
*  first record. If the response data is a list or array, you can also access individual 
*  records in that list using Velocity index notation, and you can use the special value 
*  "last" to access the last element of a List or array.
*  
*  Examples of the Velocity syntax needed:
*  
*  $responseData.first.myField is the myField property of the first response in
*  the queue. Note, this works whether that response returned a single record or a list. If it
*  returned a list, this Velocity expression gets the first record in the list. This is a
*  particularly useful shorthand for 'add' and 'update' operations, where the response data 
*  is typically a List containing a single record
*  
*  $responseData.first('order').myField is the myField property of the first response to an 
*  operation (any operation) on the "order" DataSource
*  
*  $responseData.first('order', 'add').myField is the myField property of the first 
*  response to an "add" operation on the "order" DataSource
*  
*  $responseData.first('order', 'fetch').last.myField
*  is the myField property of the last record in the response data of the first fetch in the 
*  queue (fetch operations always return a List of records)
*  
*  $responseData.first('order', 'fetch')[0].myField is the myField property of a
*  specific record (in this case, the first) in the response data of the first response in the 
*  queue. Note that this is shown for completeness only: there is no need to use index 
*  notation to explicitly request the first record, unless you are iterating over the entire 
*  list or have some other out-of-the-ordinary use case. The first record is assumed if you 
*  omit the index notation, so this example is equivalent to the simpler: 
*  $responseData.first('order', 'fetch')[0].myField
*  
*  All of these syntactic variations are also available on the $responseData.last object - 
*  "last" here meaning the most recent response matching the DataSource and operation type (if 
*  applicable). Note, "last" potentially has three different meanings, depending on context:
*  If your DataSource contains a field that is actually called "last", the following expression 
*  would be the correct way to obtain the value of the field called "last", on the last record
*  of the last (most recent) response:
*  $responseData.last.last.last
*  
*  Please see the server-side Javadoc for the 
*  com.isomorphic.velocity.ResponseDataHandler class.
* @type {VelocityExpression}
* @default null
*/
value?: VelocityExpression;

/**
* The value to use for the start of a range. This property only applies to criteria, and it 
*  only applies to operators of type "rangeCheck" - for example, the "between" operator.
*  It is ignored if specified as part of a OperationBinding.values attribute, or for 
*  an inapplicable operator type.
*  
*  The same rules apply to this attribute as apply to DSRequestModifier.value, so
*  you can use Velocity expressions if you have a Power or better license.
* @type {VelocityExpression}
* @default null
*/
start?: VelocityExpression;

/**
* The value to use for the end of a range. This property only applies to criteria, and it 
*  only applies to operators of type "rangeCheck" - for example, the "between" operator.
*  It is ignored if specified as part of a OperationBinding.values attribute, or for 
*  an inapplicable operator type.
*  
*  The same rules apply to this attribute as apply to DSRequestModifier.value, so
*  you can use Velocity expressions if you have a Power or better license.
* @type {VelocityExpression}
* @default null
*/
end?: VelocityExpression;





}


/**
* A mapping from DataSource IDs to specific Record.
*  
*  To understand how dataContext is used to automatically populate
*  DataBoundComponent, see Canvas.autoPopulateData.
*  
*  
*  For example, in JavaScript:
*  
*   {
*    "Customer": { customerNumber: "15", name: "Trish Joiner" },
*    "Employee": { employeeID: "4231", name: "Fred Smith" }
*   }
*  
*  
*  
*/
interface DataContext {




}


/**
* Identical to a DataContext but in addition to fixed values,
*  Canvas.getRuleContext values can be specified by prefixing the
*  ruleContext path with $ruleScope. as shown below:
*  
*  
*  For example, in JavaScript:
*  
*   {
*    "Customer": { customerNumber: "$ruleScope.customerGrid.values.customerNumber" }
*   }
*  
*  
*  
*  
*  When used within a Workflow SetScreenDataTask or AddScreenTask, any
*  applicable taskInputExpression can be used as a value.
*  
*  To use a literal value that starts with one of the expressions described above, prefix
*  the leading dollar sign ($) with a backslash (\) (ex. "\$ruleScope.goes.here") to prevent
*  the value from being resolved as an expression.
*/
interface DataContextBinding {




}


/**
* Metadata about a DataSourceField, including its type and validators.
*/
interface DataSourceField {
/**
* See DataSource.tableCode.
* @type {string}
* @default null
*/
fkTableCode?: string;

/**
* See DataSource.tableCode.
* @type {string}
* @default null
*/
fkColumnCode?: string;

/**
* See DataSource.tableCode.
* @type {string}
* @default null
*/
columnCode?: string;

/**
* Name for this field. 
*  
*  Must be unique across all fields within the DataSource as well as a valid JavaScript
*  identifier - see FieldName for details and how to check for validity.
*  
*  The field name is also the property in each DataSource record which holds the value for this
*  field.
* @type {FieldName}
* @default null
*/
name?: FieldName;

/**
* An optional description of the DataSourceField's meaning. Not automatically exposed on any
*  component, but useful for developer documentation, and as such is included on any
*  openapiSupport generated by the framework. Markdown is
*  a commonly used syntax, but you may also embed HTML content in a CDATA tag.
* @type {string}
* @default null
*/
description?: string;

/**
* Type of this field. Required for all DataSource fields.
*  
*  Field type may imply automatic validators (for example, an integer field cannot accept the
*  value "foo"). Field type also affects the default behaviors of DataBound components, for
*  example, if a field is declared as type "date", components that edit that field will
*  automatically choose a date-editing interface with pop-up date picker.
* @type {FieldType}
* @default null
*/
type?: FieldType;

/**
* For a binaryFields, sets a fixed mime type for all files stored
*  to this field. Should be set to a standard mime type string, for example, "text/plain" for
*  a .txt file.
*  
*  This is useful if a binary field only stored one type of file and you chose not to store 
*  filenames, since the extension of the file name is normally used to determine the mime type.
* @type {string}
* @default null
*/
mimeType?: string;

/**
* Indicates this field must be non-null in order for a record to pass validation. Or, in the
*  case of a "binary" field, a non-empty file must be uploaded. Setting this property has the
*  same effect as giving the field a ValidatorType. 
*  
*  Note that required should not be set for a server-generated field, such as a
*  sequence, or validation will fail on the client.
*  Conditionally required fields
*  Adding an Validator.applyWhen condition to a required 
*  validator introduces subtle complexities to the process of validating an existing record. 
*  The client is not guaranteed to know the the full and complete state of the record it is 
*  editing because it is common for a DynamicForm to be editing a subset of
*  fields. When a field is unconditionally required, things are simple: if the 
*  DynamicForm has a FormItem for that field, then the required validation
*  passes if that FormItem has a value, and fails if it does not. If the form has no FormItem
*  for the field, it can assume that the field has a value because otherwise it would have 
*  failed validation when we attempted to add it (when we are adding a record, we of course do 
*  know the full and complete state of the record - it is whatever we are about to add).
*  
*  When a field is conditionally required, the client can no longer assume that all 
*  required fields will have a value. It may be the case that the condition of requirement 
*  was not met when the record was added, but now it is. For example, consider these field
*  definitions:
*   &lt;field name="yearsAtCurrentAddress" type="integer" /&gt;
*   &lt;field name="previousAddress" type="text" &gt;
*    &lt;validator type="required" errorMessage="Previous address is required if you have been at your current address less than three years"&gt;
*     &lt;applyWhen operator="and"&gt;
*      &lt;criteria&gt;
*       &lt;criterion fieldName="yearsAtCurrentAddress" operator="lessThan" value="3" /&gt;
*      &lt;/criteria&gt;
*     &lt;/applyWhen&gt;
*    &lt;/validator&gt;
*   &lt;/field&gt;
*  
*  Imagine a record for this DataSource is added where the user has entered a value of "3" for 
*  "yearsAtCurrentAddress", and no previous address. Later, the value of that field is 
*  changed to "2". If this is done using a form that is also showing the "previousAddress" 
*  field, we will know that "previousAddress" has not been provided, so we can fail the 
*  validation and the user will get a helpful error message explaining what the problem is.
*  
*  However, if the form does not also show the "previousAddress" field, we may choose to use an
*  OperationBinding that uses OperationBinding.outputs to trim the
*  record down to just the fields the form does contain, in the interests of avoiding 
*  information leakage. Or perhaps that value is automatically culled from the record before
*  the client sees it by the application of a DataSourceField.viewRequires.
*  Whatever the reason, if the client does not have the complete record, it is not possible 
*  for the client to sensibly apply this validation. And because the client has no way of 
*  knowing if a value is missing because it is genuinely null, or because it has been trimmed
*  away by the server, we must treat any null value with suspicion (unless it has a matching 
*  FormItem - the presence of the FormItem means that the user can edit the value, so it 
*  would make no sense to pair it with a trimmed record that excludes that field value).
*  
*  When this happens, we mark the validation as having passed on the client, but in need of 
*  running on the server. The server validation makes use of the "storedRecord" facility 
*  (look for the description of $storedRecord in the 
*  velocitySupport) to overlay the changed record on
*  top of the existing record as it currently exists in the database. This gives the validator
*  the complete record including both changed and unchanged values, so it is able to carry 
*  out the required check in a meaningful way. However, you should be aware that the 
*  combination of conditional "required" validators and DynamicForms that edit partial 
*  records can result in a validation that cannot run on the client and must do both a server 
*  roundtrip and a database fetch.
* @type {boolean}
* @default null
*/
required?: boolean;

/**
* The required message when a field that has been marked as
*  DataSourceField.required is not filled in by the user.
*  
*  Note that this setting wins over DataSource.requiredMessage if both are set.
* @type {string}
* @default null
*/
requiredMessage?: string;

/**
* Indicates that values for this field are automatically generated by the database or ORM 
*  provider. Setting this flag makes the field behave somewhat like a 
*  DataSourceField.sequenceName, in that server-side logic does not expect client code
*  to provide values for the field on "add" operations. However, it is not entirely the same
*  as a sequence:
*  
* - Sequences must be integers, whereas autoGenerated fields can be of any type
*  
* - SmartClient Server's DataSource implementations are capable of discovering the value 
*    that was generated by the database for sequence fields, which may not be possible 
*    with an autoGenerated field. See the docs for 
*    DataSourceField.customInsertExpression for a discussion
*    of this
*  
*  Unrelated to the autoGenerated flag, you have a general ability for field 
*  values to be generated by application code (as opposed to being generated by the database
*  or ORM provider). For example, you can use dmiOverview, 
*  serverScript, or customQuerying. (Note, 
*  if you want to have application code generate values for primaryKey fields, you may need to
*  use an OperationBinding that specifies 
*  OperationBinding.providesMissingKeys).
* @type {boolean}
* @default null
*/
autoGenerated?: boolean;

/**
* Controls whether an explicit null-valued Record attribute for this field
*  should result in xsi:nil being used to transmit the value when serializing
*  to XML, like so:
*  
*  &lt;book&gt;
*    &lt;title&gt;Beowulf&lt;/title&gt;
*    &lt;author xsi:nil="true"/&gt;
*  &lt;/book&gt;
*  
*  If nillable is not set, no XML element will be generated for
*  the explicit null value.
*  
*  A DataSourceField that specifies a foreignKey is automatically nillable
*  unless this property is explicitly set to false.
* @type {boolean}
* @default null
*/
nillable?: boolean;

/**
* A ValueMap is a set of legal values for a field.
*  
*  The valueMap can be specified as either an Array of legal values, or as an
*  Object where each property maps a stored value to a user-displayable value.
*  
*  To enforce that a field should be constrained to only the values in the valueMap, either
*  declare DataSourceField.type as "enum", or use a
*  ValidatorType of "isOneOf" with explicitly listed values. Otherwise,
*  although a normal SelectItem control will only allow values from the valueMap to be
*  entered, other controls such as a ComboBoxItem will allow other values 
*  to be entered.
*  
*  In XML, a valueMap that specifies only a list of legal values is specified as follows:
*  
*   &lt;valueMap&gt;
*   &lt;value&gt;Pens &amp; Pencils&lt;/value&gt;
*   &lt;value&gt;Stationery&lt;/value&gt;
*   &lt;value&gt;Computer Products&lt;/value&gt;
*   &lt;value&gt;Furniture&lt;/value&gt;
*   &lt;value&gt;Misc&lt;/value&gt;
*   &lt;/valueMap&gt;
*  
*  A ValueMap that specifies stored values mapped to user-visible values is specified as
*  follows:
*  
*   &lt;valueMap&gt;
*   &lt;value ID="1"&gt;Pens &amp; Pencils&lt;/value&gt;
*   &lt;value ID="2"&gt;Stationery&lt;/value&gt;
*   &lt;value ID="3"&gt;Computer Products&lt;/value&gt;
*   &lt;value ID="4"&gt;Furniture&lt;/value&gt;
*   &lt;value ID="5"&gt;Misc&lt;/value&gt;
*   &lt;/valueMap&gt;
*  
* @type {ValueMap}
* @default null
*/
valueMap?: ValueMap;

/**
* Fully qualified Java className of an Enum that should be used to derive this field's
*  ValueMap.
*  
*  This has the same behavior of auto-deriving a valueMap from a Java Enum as
*  DataSource.autoDeriveSchema except it applies only to one field.
*  
*  If you enable autoDeriveSchema there is no need to set
*  valueMapEnum for Enum fields unless you want to actually override the
*  behavior to use a different Enum for the field (eg, a superclass Enum that is more
*  restrictive).
* @type {string}
* @default null
*/
valueMapEnum?: string;

/**
* Validators to be applied to this field.
*  
*  Validators are applied whenever there is an attempt to save changes to a field.
*  
*  For the available set of built-in validators, and how to define a custom validator, see the
*  Validator class.
* @type {Array<Partial<Validator>>}
* @default null
*/
validators?: Array<Partial<Validator>>;

/**
* Maximum number of characters allowed. Applicable only to fields of text type.
*  For fields of this type a ValidatorType will be automatically
*  generated on both the client and server side to enforce this maximum length
*  (unless such a validator is explicitly present for the field already).
*  
*  The TextItem.enforceLength attribute can also 
*  explicitly limit user input for freeform text items editing fields with an explicit
*  length specified.
*  
*  NOTE: For DataSources of type "sql", this property has a bearing on the type of 
*  column we use when the underlying table is created by a DataSource 
*  sqlDataSource in the adminConsole. Below 
*  a certain length (which is database-specific, see below), we use standard VARCHAR
*  columns; above that length, we use an alternate strategy (again, database-specific). For 
*  these long fields, we sometimes also generate different SQL for "update" and "add" 
*  operations, using JDBC "?" replacement parameters rather than embedding values directly in 
*  the generated SQL; whether or not this is done depends entirely on what the underlying 
*  database product and/or JDBC driver will allow.
*  
*  Table of field length limits for supported databases:
*  
*  Database product
*    VARCHAR limit *
*    Type used above limit
*  HSQLDBNone-
*  IBM DB24000CLOB
*  Firebird32767BLOB with subtype 1
*  Informix255 / 32739LVARCHAR / TEXT ** 
*  Microsoft SQL Server 8000TEXT
*  MySQL 255 / 65535 / 16M TEXT / MEDIUMTEXT / LONGTEXT *** 
*  MariaDB 255 / 65535 / 16M TEXT / MEDIUMTEXT / LONGTEXT *** 
*  Oracle4000CLOB
*  PostgreSQL4000TEXT
*  
*  * The "VARCHAR limit" referred to here is a limit used by the SmartClient Server; it
*  is not necessarily imposed by the database. For example, DB2's VARCHAR limit is not 4000
*  characters; it actually varies from about 4K to about 32K, depending on how the server has 
*  been configured.
*  ** Informix has a limit of just 255 characters for VARCHAR, but has a native LVARCHAR
*  type which supports nearly 32K characters without needing to fall back on long datatypes. 
*  Therefore, with that one product, we have two thresholds for a change in storage type.
*  *** MySQL has a limit of 255 characters for VARCHAR, 65,535 characters for TEXT and 
*  16,777,215 for MEDIUMTEXT; therefore, with that one product, we have three thresholds for a 
*  change in storage type.
* @type {number}
* @default null
*/
length?: number;

/**
* XPath expression used to retrieve the field's value.
*  
*  This XPath expression will be evaluated in the scope of the record objects selected by 
*  the DataSource.recordXPath. For XML data 
*  (DataSource.dataFormat) this means a call to
*  XMLTools.selectString passing the selected XML element. For JSON data
*  (DataSource.dataFormat), this means a call to 
*  XMLTools.selectObjects passing the selected JSON object.
*  
*  In the absence of a valueXPath, for JSON data the value for the field will
*  be the value of the same-named property in the record object selected by
*  DataSource.recordXPath. 
*  
*  For XML data, the value will be the attribute or subelement named after the field name.
*  For example, for a field "author" on a record element &lt;book&gt;, the following structures
*  require no valueXPath:
*  
*   &lt;book author="Mark Jones"/&gt;
* 
*   &lt;book&gt;
*     &lt;author&gt;Mark Jones&lt;/author&gt;
*   &lt;/book&gt;
*  
*  
*  If valueXPath is not required for your field because of the default handling
*  described above, don't specify it, as it's slightly slower.
*  
*  To learn about XPath, try the following search:
*  http://www.google.com/search?q=xpath+tutorial
*  
*  Using valueXPath with the SmartClient server
*  
*  If you're using the SmartClient server to return data via the DSResponse object (or
*  indirectly doing so using DataSource DMI), the valueXPath you specify on the DataSource
*  fields will be applied to the data you return via the 
*  JXPath library.
*  See also the server side Java APIs DataSource.setProperties() and 
*  DSResponse.setData().
*  
*  If you are returning Java Beans as your DSResponse data, normally each dataSource field
*  receives the value of the same-named Java Bean property, that is, a field "zipCode" is
*  populated by looking for "getZipCode()" on the objects passed as DSResponse data. You can
*  use valueXPath to retrieve properties from subobjects, so long as a chain of
*  getter methods exists that corresponds to the valueXPath. For example, a valueXPath of
*  "address/zipCode" expects to call "getAddress()" on the bean(s) passed to
*  DSResponse.setData(), followed by "getZipCode()" on whatever object "getAddress()" returns.
*  
*  When you are saving data, the inbound DSRequest values, available as a Java Map, will use 
*  just dataSource field names as Map keys, not the valueXPath used to derive them. However,
*  to achieve bidirectional valueXPath binding, you can use the server-side method
*  dataSource.setProperties() to use the valueXPath when setting properties on your server
*  object model. When applied as a setter, an XPath like "address/zipCode" attempts
*  "getAddress()" followed by "setZipCode()" on the returned object. JXPath also has some
*  ability to auto-create intervening objects if they are missing, such as auto-creating an
*  "address" subobject when applying "address/zipCode" as a valueXPath.
*  
*  See the
*  JXPath library documentation for
*  complete details, including other types of server object models supported, such as
*  server-side XML.
* @type {XPathExpression}
* @default null
*/
valueXPath?: XPathExpression;

/**
* Alternative XPath expression used to set the field's value.
*  
*  If is not set, then DataSourceField.valueXPath is
*  used, see its description for details.
*  
*  See also SmartClient server java APIs DataSource.setProperties() and
*  DSResponse.setData().
* @type {XPathExpression}
* @default null
*/
valueWriteXPath?: XPathExpression;

/**
* A Callback, function, or JavaScript expression used to retrieve the field's value from
*  the XML element or JSON record returned from a web service.
*  
*  This is an advanced attribute for use when a DataSourceField.valueXPath
*  setting is insufficient to derive a field's value, yet an implementation of
*  DataSource.transformResponse is overkill.
*  
*  For the required parameters, see the documentation for
*  Callbacks.GetFieldValueCallback.
* @type {GetFieldValueCallback}
* @default null
*/
getFieldValue?: GetFieldValueCallback;

/**
* Indicates that getting DataSourceField.valueXPath for this field should
*  not perform any validation at all and will return null for non existing XPaths.
*  Otherwise warning message will be logged for non-existing XPath or with null objects
*  in the middle of XPath.
*  
*  NOTE: this applies to server-side processing of valueXPath only.
* @deprecated * No longer needs to be set since the framework now defaults to suppressing errors for null values in the middle of Xpath, and only invalid XPath will cause warning be logged.

* @type {boolean}
* @default null
*/
lenientXPath?: boolean;

/**
* Default user-visible title for this field.
*  
*  This will be picked up by DataBound components and other views over this DataSource.
*  
*  Note this property frequently does not need to be set since DataSource.autoDeriveTitles (on by
*  default) usually picks an appropriate user-visible title if you have a reasonable naming convention
*  for your field names.
*  
*  Note that if this field is being displayed in a ListGrid bound to this dataSource,
*  the ListGridField.headerTitle attribute may be used to specify a
*  different string for display in the listGrid column header.
* @type {string}
* @default null
*/
title?: string;

/**
* When data values are displayed in DataBound components, by default strings will be interpreted
*  as HTML by the browser in most cases.
*  
*  If set, this property will be picked up by components bound to this dataSource, notifying them
*  that any HTML characters should be escaped when displaying values for this field.
* @type {boolean}
* @default null
*/
escapeHTML?: boolean;

/**
* If false, this property indicates that this field is considered "server only". This means:
*  
*  
* - Components cannot bind to the field; even if you explicitly add a field with the same 
*    name to your DataBoundComponent, it will be dropped
*  
* - If you are using SmartClient Server, the client will never be sent a value for the 
*    field
*  
* - If you are using SmartClient Server, then similar to DataSourceField.canEdit, no updates
*    to the field are allowed from the client. If you explicitly add a value for the field 
*    to, eg, a record you are passing to DataSource.updateData, the server will strip
*    the value out of the record before processing the update request.
*  
*  canView:false is not the same thing as DataSourceField.hidden. Use 
*  canView:false when you want to prevent the client from ever seeing a field's 
*  definition or values; use hidden:true if it is fine from a security perspective
*  that a field's definition and values are sent to the browser, but the field should not by
*  default appear in user interface elements (but could do in some cases, like a special screen
*  for advanced users or administrators, for example).
*  
*  Note that this property must be set explicitly to false to have an effect; a null or 
*  undefined setting is treated the same as true.
*  
*  This property is used to implement field-level view security: failing a 
*  DataSourceField.viewRequiresAuthentication, 
*  DataSourceField.viewRequiresRole or DataSourceField.viewRequires test is 
*  equivalent to setting canView:false on the field (and, indeed, from the 
*  client's perspective, the field has had canView:false set).
* @type {boolean}
* @default null
*/
canView?: boolean;

/**
* Controls whether, by default, DataBoundComponents consider this field editable. 
*  Set to false to draw this field read-only. 
*  
*  For a field that should never be changed from the UI, consider using DataSourceField.canSave instead,
*  which will both mark a field as non-editable and reject any attempt to programmatically
*  change the value on the server (when using the SmartClient Server).
*  
*  This attribute may not effect all dataBoundComponents - the 
*  DataBoundComponent.canEditFieldAttribute may be set at the component level to look
*  for a different attribute on the dataSourceField, and components allow developers to explicitly
*  override this default (see ListGridField.canEdit. FormItem.canEdit for example).
* @type {boolean}
* @default null
*/
canEdit?: boolean;

/**
* Whether values in this field can be updated and saved to the dataSource.
*  
*  If set to false, this field will default to being non-editable in standard editing
*  components (DynamicForm, editable ListGrid), but will be editable when displayed
*  for filtering purposes only (in a SearchForm or ListGrid.showFilterEditor. If DataSourceField.canEdit is explicitly specified it will take 
*  precedence over this client-side behavior, but the server will still enforce the no-save
*  policy (described below).
*  
*  NOTE: If you are using SmartClient Server and have specified canSave: false 
*  for a field in a DataSource definition (.ds.xml file), this is enforced on 
*  the server. This means that we will strip out any attempt to set the value of such a 
*  field before trying to process any update or add request, similar to what happens when 
*  a DataSourceField.editRequiresAuthentication fails.
* @type {boolean}
* @default null
*/
canSave?: boolean;

/**
* Should the user be able to filter data by this field?
*  Affects whether this field will show up in dataBoundComponents with UI for filtering data.
*  
*  Note that setting canFilter:false only affects UI and is not a security
*  feature. Enforcing that filtering cannot be performed server side does not meaningfully
*  increase security, since as long as a field can be viewed by an end user, they can
*  effectively search the field themselves even if the UI doesn't offer a means to do so.
*  If a field should be unable to be viewed entirely by some users, use 
*  DataSourceField.viewRequiresRole and related settings.
*  
*  Rather than a security setting, canFilter:false is intended for situations
*  where it would be redundant or nonsensical to be able to search on a field, or where
*  searching isn't implemented for that field.
* @type {boolean}
* @default null
*/
canFilter?: boolean;

/**
* Sets the default FormItem to be used whenever this field is edited (whether in a grid, form,
*  or other component).
*  
*  If unset, a FormItem will be automatically chosen based on the type of the field, by the
*  rules explained FormItemType.
* @type {FormItemClassName}
* @default null
*/
editorType?: FormItemClassName;

/**
* Sets the default FormItem to be used if this field is marked as 
*  DataSourceField.canEdit and displayed in an editor component such
*  as a DynamicForm.
*  
*  This property may also be specified at the type level by specifying 
*  SimpleType.readOnlyEditorType.
* @type {FormItemClassName}
* @default null
*/
readOnlyEditorType?: FormItemClassName;

/**
* Sets the default FormItem to be used for this field if it appears in a filter row,
*  and DataSourceField.canFilter is not false.
*  
*  Note: If this is not specified, the edit-formItem type may be derived from the
*  DataSourceField.editorType property, or from the field's DataSourceField.type.
* @type {FormItemClassName}
* @default null
*/
filterEditorType?: FormItemClassName;

/**
* The default date formatter to use for displaying this field. Only applicable to fields of 
*  type "date" and "datetime". Note that this property is honored when exporting directly to 
*  Excel spreadsheets (ie, when using XLS or XLSX/OOXML form, not CSV); "date" and
*  "datetime" fields with this property set will deliver real dates and formatting information
*  to Excel, rather than formatted strings or unformatted dates.
* @deprecated * Use DataSourceField.dateFormatter or DataSourceField.timeFormatter  instead.

* @type {DateDisplayFormat}
* @default null
*/
displayFormat?: DateDisplayFormat;

/**
* Preferred display format to use for date type values within this field.
*  If this property is set on a field displayed in a databound component such as a
*  DynamicForm or ListGrid it will be respected (See FormItem.dateFormatter and
*  ListGridField.dateFormatter).
*  
*  Note that this property is also honored when exporting directly to 
*  Excel spreadsheets (ie, when using XLS or XLSX/OOXML form, not CSV); "date" and
*  "datetime" fields with this property set will deliver real dates and formatting information
*  to Excel, rather than formatted strings or unformatted dates.
* @type {DateDisplayFormat}
* @default null
*/
dateFormatter?: DateDisplayFormat;

/**
* Preferred time-format to apply to date type values within this field. If this property is
*  specified on a field displayed within a dataBound component such as a ListGrid or
*  DynamicForm, any dates displayed in this field will be formatted as times using the
*  appropriate format.
*  
*  This is most commonly only applied to fields specified as type "time" though
*  if no explicit FormItem.dateFormatter is specified it will be respected for other 
*  fields as well.
*  
*  See ListGridField.timeFormatter and FormItem.timeFormatter for more information.
* @type {TimeDisplayFormat}
* @default null
*/
timeFormatter?: TimeDisplayFormat;

/**
* Applies only to fields of type "float" and affects how many significant digits are shown.
*  
*  For example, with decimalPrecision 3, if the field value is 343.672677, 343.673 is shown.
*  
*  If the value is 125.2, 125.2 is shown - decimalPrecision will not cause extra zeros to be
*  added. Use DataSourceField.decimalPad for this.
*  
*  A number is always shown with its original precision when edited.
* @type {number}
* @default null
*/
decimalPrecision?: number;

/**
* Applies only to fields of type "float" and enforces a minimum number of digits shown after
*  the decimal point.
*  
*  For example, a field value of 343.1, 343.104 and 343.09872677 would all be shown as 343.10
*  if decimalPad is 2.
*  
*  The original unpadded value is always shown when the value is edited.
* @type {number}
* @default null
*/
decimalPad?: number;

/**
* Applies only to fields of type "float" or "integer" and affects how many significant
*  digits are shown.
*  
*  For example, a field value of 343.1, 343.104 and 343.09872677 would all be shown as 343.10
*  if precision is 5.
*  
*  The original value is always shown when the value is edited.
* @type {number}
* @default null
*/
precision?: number;

/**
* Whether this field should be hidden from users by default within a DataBound component.
*  This is generally used for internal IDs and other fields not meaningful to users.
*  
*  See DataSourceField.detail for fields that should be hidden in a summary view such as
*  a ListGrid, but still available to the user.
*  
*  NOTE: This property is not a security setting - data for hidden fields is 
*  still delivered to the client, it just isn't shown to the user. If you wish to make sure 
*  that only appropriate data reaches the client, use OperationBinding.outputs,
*  DataSourceField.canView:false on the field, or a field-level declarative security setting like 
*  DataSourceField.viewRequiresRole.
* @type {boolean}
* @default false
*/
hidden?: boolean;

/**
* Whether this field should be completely excluded from this dataSource, as if it had never been
*  defined.
*  
*  If set to true, the field will be entirely omitted when serving a DataSource derived 
*  from a server-side definition (typically a .ds.xml file) to the client.
* @type {boolean}
* @default false
*/
ignore?: boolean;

/**
* Whether this field should be considered a "detail" field by a DataBoundComponent.
*  
*  Detail fields won't be shown by default in a DataBoundComponent where 
*  DataBoundComponent.showDetailFields is false. This allows for some DataBound
*  components, like a ListGrid, to show a summary view of records which displays only 
*  the most commonly viewed fields by default, while other DataBoundComponents, like a
*  DetailViewer, show all fields by default.
*  
*  In addition, the FormItem.showIf property is supported in multiple
*  components for conditional visibility - see for example ListGridField.showIf and
*  FormItem.showIf).
* @type {boolean}
* @default false
*/
detail?: boolean;

/**
* If true, then this field is excluded from the bound component's view state. In addition,
*  the field will not be selected as the default title field
*  by DataBoundComponent.getTitleField
*  if DataBoundComponent.titleField is not provided.
* @type {boolean}
* @default null
*/
excludeFromState?: boolean;

/**
* Text to be used for display by client-side components when this field has a null or 
*  undefined value. This value will be overridden by a component's emptyCellValue, if set.
* @type {HTMLString}
* @default null
*/
emptyDisplayValue?: HTMLString;

/**
* Indicates either that this field holds a value unique across all records in this
*  DataSource, or that it is one of a number of fields marked as primary keys, and the
*  combination of the values held in all of those fields is unique across all records in the
*  DataSource. Note that the latter usage - so-called "composite" or "multipart" keys - is 
*  intended for support of legacy databases only: if you are able to choose an approach, 
*  Isomorphic recommends the use of one primaryKey field per DataSource, and 
*  ideally this field should be of DataSourceField.type "sequence". If you have
*  control of the underlying tables, there is nothing to stop you from adding a field like 
*  this (a so-called "synthetic" or "surrogate" key), even for tables that already have a set
*  of columns that could combine to make a composite key (a so-called "natural" key). Also, 
*  it is neither necessary nor correct to use a composite primaryKey because you want to 
*  enforce uniqueness across that combination of fields. You can achieve that by declaring
*  a unique constraint in the table schema, or use an ValidatorType 
*  validator with validator.criteriaFields, or both; there is no need to use a 
*  composite key to enforce uniqueness
*  
*  Note that composite primaryKeys are not supported in tree-structured datasets 
*  (Tree and ResultTree) or components (TreeGrid, 
*  ColumnTree). Tree-structured data requires that nodes have a unique 
*  Tree.idField, with the parent/child relationship expressed through the 
*  Tree.parentIdField. This implies that binding a Tree to a DataSource
*  requires that the DataSource have a singular primaryKey, and that the primaryKey field is 
*  also the idField, as described in the treeDataBinding
*  
*  A DataSource that can only perform the "fetch" operation does not require a
*  primaryKey unless it contains binaryFields. If a DataSource 
*  allows modification of DataSource records through add, update and remove DataSource 
*  operations, or it contains one or more binary fields, one or more fields must be marked as 
*  the primary key.
*  
*  SmartClient requires a primary key value to uniquely identify records when communicating
*  updates or deletions to the server. There is no requirement that the primaryKey field be
*  mapped to an actual "primary key" in your object model, web service, or database (though 
*  this is the most obvious and natural thing to do, of course). The only requirement is 
*  that the combined values of the primaryKey fields be unique for a given browser instance
*  for the lifetime of the page. 
*  
*  If using SmartClient's sqlDataSource and generating SQL tables using
*  the dbConfigTool, the table column generated from a
*  primaryKey field will have a unique constraint applied in the database table and, if the field
*  is of type "sequence", the database column will also be created as an "identity column" in those
*  databases that implement sequence-type handling with identity columns.
* @type {boolean}
* @default false
*/
primaryKey?: boolean;

/**
* Declares that this field holds values that can be matched to values from another DataSource
*  field, to create a relationship between records from different DataSources or even records
*  within the same DataSource.
*  
*  The format of foreignKey is
*  dataSourceId.fieldName.
*  
*  For a foreignKey within the same dataSource, you can omit the dataSourceId
*  and just specify fieldName. For example, to create a tree relationship
*  within a DataSource:
*  
*  
*   isc.DataSource.create({
*    ID:"supplyItem",
*    fields : [
*     {name:"itemId", type:"sequence", primaryKey:true},
*     {name:"parentId", type:"integer", foreignKey:"itemId"},
*     ...
*    ]
*   });
*  
*  
*  
*  
*  foreignKey declarations also allow other automatic behaviors by
*  DataBoundComponent, such as ListGrid.fetchRelatedData.
*  
*  For SQLDataSources foreign keys can be automatically discovered from SQL tables if
*  DataSource.autoDeriveSchema is set.
* @type {string}
* @default false
*/
foreignKey?: string;

/**
* If true, this property indicates that this field will hold an explicit array of child nodes 
*  for the current node.
*  This has the same effect as specifying DataSource.childrenField to this field's 
*  name.
* @type {boolean}
* @default false
*/
childrenProperty?: boolean;

/**
* For a field that is a foreignKey establishing a tree relationship, what value indicates a
*  root-level node. Defaults to null.
*  
*  Note that the rootValue may be overridden on a specific ResultTree instance by setting
*  ResultTree.rootNode, or if the ResultTree is auto-generated by a TreeGrid,
*  by setting TreeGrid.treeRootValue. This allows a component to navigate a subtree of
*  the hierarchical data from this dataSource starting at a particular node.
* @type {any}
* @default null
*/
rootValue?: any;

/**
* For a DataSource with DataSource.serverType with a field of type
*  "sequence", the name of the SQL sequence that should be used when inserting new records into
*  this table.
*  
*  Note that this is never required for SQL tables that are generated from SmartClient
*  DataSources (a default sequence name of tableName + "_" + columnName is chosen, but see the
*  notes below regarding this), and is never required for databases where inserting null into a 
*  sequence column is sufficient (MySQL, SQL Server, DB2 and others). 
*  
*  You would only need to set sequenceName if you are integrating with a pre-existing table
*  stored in a database where the sequence must be named for insertion to work (Oracle,
*  Postgres, Firebird) OR you are trying to use the same sequence across multiple DataSources.
*  
*  Note: If you specify the sql.{database type}.sequence.name.prefix and/or 
*  sql.{database type}.sequence.name.suffix properties in your 
*  server_properties file,the generated sequence name will include the prefix and/or
*  suffix. For example, with a prefix of "order_system_" and a suffix of "_seq", the sequence
*  generated for column "orderNumber" on table "orders" would be:
*    order_system_orders_orderNumber_seq
*  
* @type {string}
* @default null
*/
sequenceName?: string;

/**
* For a field of FieldType "sequence" on a DataSource with 
*  DataSource.serverType, this flag indicates that the field is 
*  implicitly bound to a sequence. This setting means that SmartClient does not expect to 
*  find a value for the field in "add" operations, even if it is marked as a 
*  DataSourceField.primaryKey, as such fields usually are. It also means that SmartClient
*  does not attempt to retrieve the field value from an actual database sequence, instead
*  relying on the JDBC driver to return the generated value (see the note on
*  sequenceMode below).
*  
*  Implicitly bound columns are a syntactic convenience provided by some databases to simulate 
*  the "auto-increment" or "identity" columns available natively in other products, without 
*  the underlying sequence having to be explicitly referenced in SQL queries. Currently, these
*  database products support this idea:
*  
* - PostgreSQL has had a "serial" column type for a long time - 
*  
*  http://www.postgresql.org/docs/9.3/static/datatype-numeric.html#DATATYPE-SERIAL
*  
* - DB2 has long supported a "GENERATED AS IDENTITY" notation for numeric fields. This may
*  or may not be implemented with a sequence - the documentation does not specify - but we 
*  support it via the implicitSequence mechanism because it is so similar to the implicit
*  sequence approach in Oracle.  
*  
*  http://publib.boulder.ibm.com/infocenter/dzichelp/v2r2/index.jsp?topic=%2Fcom.ibm.db2z10.doc.apsg%2Fsrc%2Ftpc%2Fdb2z_identitycols.htm
*  
* - Oracle introduced a "GENERATED AS IDENTITY" notation for numeric fields in version 12c - 
*  
*  http://www.oracle-base.com/articles/12c/identity-columns-in-oracle-12cr1.php
*  
*  If you have columns like these in your database, declare them as type "sequence" and mark 
*  them with the implicitSequence flag.
*  
*  NOTE: If you use this property, you should also set the 
*  DataSource.sequenceMode to "jdbcDriver". This is necessary
*  because SmartClient cannot directly query the implicit sequence that is being used behind the 
*  scenes, so we must rely on the JDBC driver to do that.
*  
*  If you use DataSource.autoDeriveSchema to automatically derive a 
*  dataSource from table metadata, SmartClient attempts to identify these special Oracle and
*  Postgres columns by heuristic examination of the metadata. When it identifies such a 
*  column, it marks the corresponding dataSourceField implicitSequence: true, 
*  and changes the sequenceMode on the DataSource to "jdbcDriver". If your table contains
*  one of these columns and SmartClient does not automatically identify it, bear in mind that
*  you can always set this flag manually, even if you are using autoDeriveSchema.
*  
*  This setting has no effect for non-SQL dataSources, or for databases other than those 
*  mentioned above.
* @type {boolean}
* @default null
*/
implicitSequence?: boolean;

/**
* Indicates that this field should be fetched from another, related DataSource.
*  
*  The incluedFrom attribute should be of the form
*  "dataSourceId.fieldName", for example:
*  
*   &lt;field includeFrom="supplyItem.itemName"/&gt;
*  
*  
*  A DataSourceField.foreignKey declaration must exist between the two DataSources, establishing either
*  a 1-to-1 relationship or a many-to-1 relationship from this DataSource to the related
*  DataSource. The inclusion can be indirect (traverse multiple DataSources) so long as there
*  is a chain of foreignKey declarations from the target DataSource to the
*  DataSource where the includeFrom field is declared. You may use dot-notation
*  to provide an explicit path between DataSources, or provide the name of only the last
*  DataSource in the chain to have the complete path calculated for you at runtime.
*  i.e., either of the following are acceptable forms, where foreign keys
*  are defined to link records in the current DataSource to Employee records and in turn to
*  Office records:
*  
*   &lt;field includeFrom="Employee.Office.territory"/&gt;
*   &lt;!-- OR --&gt;
*   &lt;field includeFrom="Office.territory"/&gt;
*  
*  Note that when using the shorthand form, there is potential ambiguity: there could be
*  multiple ways in which two DataSources are related via different intervening DataSources,
*  so the auto-discovered relation may be different depending on which other DataSources are
*  loaded in the page. For this reason, explicitly spelling out the inclusion path is
*  preferred.
*  
*  Nested inclusions, where an
*  included field is itself an included field, are also supported - for details on this and
*  other complex scenarios see DataSourceField.includeVia docs.
*  
*  In all cases, DataSourceField.name will default to the name of the included field,
*  or you can specify a different name.
*  
*  If both DataSources are SQLDataSources, HibernateDataSources or JPADataSources (with
*  Hibernate as the provider) the related data will be retrieved via a SQL join and criteria
*  and sort directions applied to the field work normally (they become part of the generated
*  SQL query).
*  
*  Note that includeFrom is also supported between two clientOnly or MockDataSources, but not
*  for any other combination (for example, a RestDataSource cannot use includeFrom with a
*  clientOnly DataSource). Here, the related data (including any values derived via
*  DataSourceField.includeSummaryFunction) will be retrieved from cacheData after the
*  primary (fetch, add, or update) operation has returned its response.
*  
*  Otherwise, the related data will be retrieved via performing a DSRequest against
*  the related DataSource once the data from the primary DataSource has been retrieved. In
*  this case, criteria or sorting directions applied to the included field are only allowed if
*  data paging is not in use (for example ListGrid.dataFetchMode:"basic"); otherwise,
*  criteria and sort direction are ignored for the included field and a warning is logged on
*  the server.
*  
*  Editing included fields
*  
*  An included field is DataSourceField.canEdit by default. Note that 
*  included fields are not updatable, even if you set canEdit:true; the server will simply drop
*  values for included fields if client code sends them. 
*  
*  When thinking about editing an included field value, typically what is really intended is to
*  edit the value of the foreignKey field. For example, take the scenario of a
*  system that tracks accounts and the employees assigned to manage them. Given a DataSource
*  "account" related one-to-one with DataSource "employee" by a "managerId" foreignKey field,
*  we might declare an includeFrom so that the name of the account manager can
*  be shown with each "account" record. 
*  
*  Editing the manager's name while viewing the account would be intended to pick a new account
*  manager, and not to change the legal name of the employee who happens to be the
*  current account manager. 
*  
*  To correctly set up this scenario, declare an includeFrom field that is hidden,
*  but is used as the DataSourceField.displayField for the foreign key
*  field:
*  
*  &lt;field name="managerId" foreignKey="employee.id" displayField="managerName" /&gt; 
*  &lt;field name="managerName" includeFrom="employee.name" hidden="true"/&gt; 
*  
*  Now:
*  
*  
* - the "managerId" foreignKey field is shown in grids and forms, but takes its displayed
*  value from the hidden includeFrom field. Note that when the
*  foreignKey and displayField are specified, the 
*  framework automatically defaults DataSourceField.useLocalDisplayFieldValue to 
*  true to ensure the displayed value is picked up from the record being edited.
*  
* - the automatically chosen editor will be a SelectItem with
*  SelectItem.optionDataSource set to "employees": it will allow
*  picking a different "employee" record from the "employee" DataSource.
*  
* - saving will save the ID of a new "employee" record to the "managerId" foreign key
*  field, as intended
*  
* 
*  You can alternatively set editorType="ComboBoxItem" on the
*  "managerId" field to allow typeahead search of the "employee" DataSource.
*  Note that the DataSourceField.foreignDisplayField attribute allows developers to have a different
*  fieldName be used locally as a displayField from the field name for the display field
*  in the foreign dataSource.
*  
*  Including fields that use summary functions
*  
*  The DataSourceField.includeSummaryFunction feature is used
*  for including from a related DataSource where there are multiple related records. It applies
*  a SummaryFunction to the related records aggregating them into single value.
*  It is regularly used on directly included fields, but it supports indirect inclusions as well,
*  when entire includeFrom+includeSummaryFunction setup is included from
*  another DataSource. See DataSourceField.includeSummaryFunction docs for more details.
* @type {string}
* @default null
*/
includeFrom?: string;

/**
* For a field that uses DataSourceField.includeFrom, specifies which
*  DataSourceField.foreignKey field should be used to find records in the
*  related DataSource.
*  
*  includeVia only needs to be set when you have more than one
*  foreignKey to the same related DataSource. If you have multiple
*  foreignKeys to multiple different DataSources, there is no need to set
*  includeVia.
*  
*  For example, perhaps you have a DataSource "moneyTransfer" where each record represents a
*  money transfer, where the source and payment currencies are different, and the list of
*  currencies is stored in a related DataSource "currency". Each "moneyTransfer" record is
*  linked to 2 "currency" records, through two different foreignKey fields,
*  "sourceCurrencyId" and "paymentCurrencyId".
*  
*  The following declarations would be required to use includeFrom to get a
*  include the field "currencySymbol" from each of the two related "currency" records.
*  
*   &lt;field name="sourceCurrencyId" foreignKey="currency.id"/&gt;
*   &lt;field name="paymentCurrencyId" foreignKey="currency.id"/&gt;
*   &lt;field name="sourceCurrencySymbol" includeFrom="currency.currencySymbol" includeVia="sourceCurrencyId"/&gt;
*   &lt;field name="paymentCurrencySymbol" includeFrom="currency.currencySymbol" includeVia="paymentCurrencyId"/&gt;
*  
*  
*  SQL Templating and includeVia
*  
*  The includeVia feature uses SQL table aliases in the generated SQL when generating multiple SQL joins
*  to the same SQL table. When using customQuerying, it's sometimes necessary to know 
*  the names of the aliases in the generated SQL. The table alias used can be configured via
*  setting DataSourceField.relatedTableAlias on the foreignKey field, for
*  example, using the declarations below, aliases "source" and "payment" would be used for the
*  two "currency" tables.
*  
*   &lt;field name="sourceCurrencyId" foreignKey="currency.id" relatedTableAlias="source"/&gt;
*   &lt;field name="paymentCurrencyId" foreignKey="currency.id" relatedTableAlias="payment"/&gt;
*   &lt;field name="sourceCurrencySymbol" includeFrom="currency.currencySymbol" includeVia="sourceCurrencyId"/&gt;
*   &lt;field name="paymentCurrencySymbol" includeFrom="currency.currencySymbol" includeVia="paymentCurrencyId"/&gt;
*  
*  
*  Multiple indirection and relatedTableAlias
*  
*  General rule is that if relatedTableAlias is present it is used as alias or its segment, otherwise
*  foreignKey field name linked by includeVia is used instead. See the "Automatically
*  generated table aliases" section of the customQuerying for the complete set of
*  general rules how aliases are generated. Also, see some samples below.
*  
*  Sometimes you may have two includeFrom fields that include a field which is itself included from 
*  another DataSource, for example:
*  
*   &lt;field name="sourceCurId" nativeName="sourceCurrencyId" foreignKey="currency.id" relatedTableAlias="source"/&gt;
*   &lt;field name="sourceCurrencySymbol" includeFrom="currency.currencySymbol" includeVia="sourceCurId"/&gt;
*   &lt;field name="sourceCurrencyGroup" includeFrom="currency.groupName" includeVia="sourceCurId"/&gt;
*   &lt;field name="paymentCurId" nativeName="paymentCurrencyId" foreignKey="currency.id" relatedTableAlias="payment"/&gt;
*   &lt;field name="paymentCurrencySymbol" includeFrom="currency.currencySymbol" includeVia="paymentCurId"/&gt;
*   &lt;field name="paymentCurrencyGroup" includeFrom="currency.groupName" includeVia="paymentCurId"/&gt;
*  
*  .. where the "currency" DataSource used above is related to the "currencyGroup" DataSource via fields:
*  
*   &lt;field name="groupId" type="integer" foreignKey="currencyGroup.id" relatedTableAlias="group"/&gt;
*   &lt;field name="groupName" type="text" includeFrom="currencyGroup.groupName"/&gt;
*  
*  Fields "sourceCurrencyGroup" and "paymentCurrencyGroup" include a field that is itself an included field.
*  In this case "currencyGroup" table will be referenced two times, and its relatedTableAlias defined in 
*  "currency" DataSource will be prefixed with the referenced by includeVia field's relatedTableAlias
*  value to make aliases unique in generated SQL: "source_group" and "payment_group".
*  
*  The same aliases would be used if "sourceCurrencyGroup" and "paymentCurrencyGroup" fields
*  were to include "currencyGroup.groupName" indirectly:
*  
*   &lt;field name="sourceCurrencyGroup" includeFrom="currency.currencyGroup.groupName" includeVia="sourceCurId"/&gt;
*   &lt;field name="paymentCurrencyGroup" includeFrom="currency.currencyGroup.groupName" includeVia="paymentCurId"/&gt;
*  
*  
*  This works the same for more complex relationships. If we add a "moneyTransferDetail"
*  DataSource to the sample above which has multiple references to "moneyTransfer" 
*  DataSource and would include fields from "currency" and "currencyGroup" DataSources:
*  
*   &lt;field name="mtId" nativeName="moneyTransferId" type="integer" foreignKey="moneyTransfer.id" relatedTableAlias="main" /&gt;
*   &lt;field name="mainTransferName" includeFrom="moneyTransfer.name" includeVia="mtId" /&gt;
*   &lt;field name="mainSourceSymbol" includeFrom="moneyTransfer.sourceCurrencySymbol" includeVia="mtId" /&gt;
*   &lt;field name="mainSourceGroup" includeFrom="moneyTransfer.sourceCurrencyGroup" includeVia="mtId" /&gt;
*   &lt;field name="mainPaymentSymbol" includeFrom="moneyTransfer.paymentCurrencySymbol" includeVia="mtId" /&gt;
*   &lt;field name="mainPaymentGroup" includeFrom="moneyTransfer.paymentCurrencyGroup" includeVia="mtId" /&gt;
*   &lt;field name="mtPrevId" nativeName="moneyTransferPreviousId" type="integer" foreignKey="moneyTransfer.id" relatedTableAlias="prev" /&gt;
*   &lt;field name="previousTransferName" includeFrom="moneyTransfer.name" includeVia="mtPrevId" /&gt;
*   &lt;field name="previousSourceSymbol" includeFrom="moneyTransfer.sourceCurrencySymbol" includeVia="mtPrevId" /&gt;
*   &lt;field name="previousSourceGroup" includeFrom="moneyTransfer.sourceCurrencyGroup" includeVia="mtPrevId" /&gt;
*   &lt;field name="previousPaymentSymbol" includeFrom="moneyTransfer.paymentCurrencySymbol" includeVia="mtPrevId" /&gt;
*   &lt;field name="previousPaymentGroup" includeFrom="moneyTransfer.paymentCurrencyGroup" includeVia="mtPrevId" /&gt;
*  
*  In this scenario the "currencyGroup" table will be joined 4 times - for all main/prev transfer
*  and payment/source currency combinations. So, aliases will be prefixed with both
*  intermediate relatedTableAlias values: "main_source_group",
*  "main_payment_group", "prev_source_group", "prev_payment_group".
*  
*  It is also allowed to specify a series of FK fields in includeVia, for example
*  "moneyTransferDetail" could declare:
*  
*   &lt;field name="mainSourceCurrencyGroup" includeFrom="moneyTransfer.currency.currencyGroup.groupName" includeVia="mtId.sourceCurId"/&gt;
*   &lt;field name="mainPaymentCurrencyGroup" includeFrom="moneyTransfer.currency.currencyGroup.groupName" includeVia="mtId.paymentCurId"/&gt;
*  
*  In this case the prefix used for table aliases will be the same, cause relations referenced in includeVia
*  are the same as in previous example: "main_source_group" and "main_payment_group".
*  
*  Note that if DataSourceField.relatedTableAlias are completely missing then, according to general rule,
*  foreignKey field names will be used in aliases: "mtId_sourceCurId_groupId" and "mtId_paymentCurId_groupId".
* @type {string}
* @default null
*/
includeVia?: string;

/**
* For a sqlDataSource field that specifies a
*  DataSourceField.foreignKey, this property defines the table alias name to
*  use in generated SQL.
*  
*  Aliasing is necessary when the same table appears more than once in a query. This can 
*  happen when using DataSourceField.includeVia. It can also happen when a
*  DataSourceField.foreignKey definition references the same dataSource
*  that the field is defined in; this happens with hierarchical structures, for example where
*  every Employee reports to another Employee, a so-called "self join", and it
*  always requires relatedTableAlias to be specified; failure to do so
*  will result in invalid SQL.
*  
*  In case of indirect relationship, when more than single join is needed to join the target table, and
*  DataSourceField.includeVia is missing, generated alias is a concatenation of
*  relatedTableAlias and FK field names starting with the first relatedTableAlias 
*  met in chain of relations leading to the target table.
*  
*  See the "Automatically generated table aliases" section of the customQuerying
*  for more details.
* @type {string}
* @default null
*/
relatedTableAlias?: string;

/**
* In addition to regular DataSourceField.foreignKey this property allows
*  defining multiple foreign keys if the field is related to more than one DataSource.
*  
*  The format of the otherFKs is coma separated foreign key sets, for example:
*  "&lt;firstDS&gt;.&lt;fieldName&gt;,&lt;secondDS&gt;.&lt;fieldName&gt;".
*  Note that this property works only in addition to the regular FK, i.e. if
*  DataSourceField.foreignKey is not set, then foreign keys defined in otherFKs
*  will be completely ignored. If otherFKs is specified, it can include the value of the
*  foreignKey attribute or it can omit it, listing only additional foreignKeys. There
*  is no impact on functionality from doing it either way.
*  
*  For example, perhaps you have a moneyTransfer DataSource where each record represents
*  a money transfer, which has receipt and invoice information stored in
*  receiptDS and receiverDS DataSources. Each money transfer is linked to a
*  receipt and an invoice records through the same transferDS.id field via the foreign keys
*  defined in foreignKey and otherFKs properties:
*  
*  &lt;DataSource ID="moneyTransfer" ... &gt;
*    &lt;field name="id" primaryKey="true"
*      foreignKey="receiptDS.id"
*      otherFKs="invoiceDS.id"/&gt;
*    &lt;field name="receiptInfo" includeFrom="receiptDS.info"/&gt;
*    &lt;field name="invoiceInfo" includeFrom="invoiceDS.info"/&gt;
*  &lt;/DataSource&gt;
*  
*  Here, we are including from two different related DataSources using the same
*  foreignKey field.
*  
*  Note that otherFKs feature is meant to be used in more complex scenarios (see below), so if the
*  example above is all that's needed, then there's no need to use otherFKs feature. The result
*  would be the same as if foreign keys would be declared at the "other side", i.e. in receiptDS
*  and invoiceDS DataSources via regular DataSourceField.foreignKey property.
*  
*  If we extend the sample above with a parentId joining the moneyTransfer DataSource
*  (so called "self-join" relation) and include from receiptDS and invoiceDS for
*  the "parent" money transfer as well, then otherFKs feature must be used. The following
*  declarations would be required to get receipt and invoice information for both the original money transfer
*  and its "parent":
*  
*  &lt;field name="id" primaryKey="true" foreignKey="receiptDS.id" otherFKs="invoiceDS.id"/&gt;
*  &lt;field name="receiptInfo" includeFrom="receiptDS.info"/&gt;
*  &lt;field name="invoiceInfo" includeFrom="invoiceDS.info"/&gt;
*  &lt;field name="parentId" foreignKey="moneyTransfer.id" relatedTableAlias="parent"/&gt;
*  &lt;field name="parentReceiptInfo" includeFrom="moneyTransfer.receiptDS.info"/&gt;
*  &lt;field name="parentInvoiceInfo" includeFrom="moneyTransfer.invoiceDS.info"/&gt;
*  
*  
*  SQL Templating and otherFKs
*  
*  Similar to the includeVia, otherFKs feature uses SQL table aliases in the
*  generated SQL when generating multiple SQL joins to the same SQL table. When using
*  SQL Templating, it's sometimes necessary to know the names of the aliases in the generated
*  SQL. See the "Automatically generated table aliases" section of the customQuerying
*  for the complete set of general rules how aliases are generated.
*  
*  The otherFKs feature uses aliases constructed from two parts:
*  - the DataSourceField.name, which otherFKs property is defined on;
*  - the related DataSource.ID or the DataSource.relatedTableAlias if set.
*  So, in the extended sample above the aliases for the "parent" money transfer related tables would be
*  "parent_id" (regular FK) and "parent_id_invoiceDS" (other FK).
*  
*  See below the same sample extended with the detailDS DataSource storing multiple records per money
*  transfer, joined to moneyTransfer DataSource on a one-to-many basis, and include receipt and
*  invoice information for money transfer and its "parent". Also, we'll add relatedTableAliases
*  to the invoice DataSource, see all related declarations below:
*  
*  &lt;DataSource ID="detailDS" ... &gt;
*    &lt;field name="id" type="integer" primaryKey="true" &gt;
*    &lt;field name="name"/&gt;
*    &lt;field name="mtId" foreignKey="moneyTransfer.id" relatedTableAlias="main"/&gt;
*    &lt;field name="mainReceipt" includeFrom="moneyTransfer.receiptInfo" includeVia="mtId"/&gt;
*    &lt;field name="mainInvoice" includeFrom="moneyTransfer.invoiceInfo" includeVia="mtId"/&gt;
*    &lt;field name="mainParentReceipt" includeFrom="moneyTransfer.parentReceiptInfo" includeVia="mtId"/&gt;
*    &lt;field name="mainParentInvoice" includeFrom="moneyTransfer.parentInvoiceInfo" includeVia="mtId"/&gt;
*  &lt;/DataSource&gt;
* 
*  &lt;DataSource ID="receiptDS" ... &gt;
*    &lt;fields&gt;
*      &lt;field name="id" primaryKey="true"/&gt;
*      &lt;field name="info"/&gt;
*    &lt;/fields&gt;
*  &lt;/DataSource&gt;
* 
*  &lt;DataSource ID="invoiceDS" relatedTableAlias="invoice" ... &gt;
*    &lt;fields&gt;
*      &lt;field name="id" primaryKey="true"/&gt;
*      &lt;field name="info"/&gt;
*    &lt;/fields&gt;
*  &lt;/DataSource&gt;
*  
*  Generated aliases for the receipt/invoice tables will be main_id, main_id_invoice
*  for the money transfer and main_parent_id, main_parent_id_invoice for the "parent"
*  money transfer.
*  
*  otherFKs + includeVia
*  
*  The otherFKs and the DataSourceField.includeVia features may be used together in even
*  more complex scenarios. In the example below we extend detailDS DataSource with a link to
*  the "previous" money transfer and include all of its receipt/invoice information:
*  
*  &lt;field name="mtPrevId" foreignKey="moneyTransfer.id" relatedTableAlias="prev"/&gt;
*  &lt;field name="prevReceipt" includeFrom="moneyTransfer.receiptInfo" includeVia="mtPrevId"/&gt;
*  &lt;field name="prevInvoice" includeFrom="moneyTransfer.invoiceInfo" includeVia="mtPrevId"/&gt;
*  &lt;field name="prevParentReceipt" includeFrom="moneyTransfer.parentReceiptInfo" includeVia="mtPrevId"/&gt;
*  &lt;field name="prevParentInvoice" includeFrom="moneyTransfer.parentInvoiceInfo" includeVia="mtPrevId"/&gt;
*  
*  Additionally generated aliases will be prev_id, prev_id_invoice
*  for the money transfer and prev_parent_id, prev_parent_id_invoice
*  for the "parent" money transfer.
*  
*  The inclusions may as well be indirect like shown below and they would produce same chains of relations,
*  so the generated SQL and table aliases will be the same as above.
*  
*  &lt;field name="mtPrevId" foreignKey="moneyTransfer.id" relatedTableAlias="prev"/&gt;
*  &lt;field name="prevReceipt" includeFrom="moneyTransfer.receiptDS.info" includeVia="mtPrevId"/&gt;
*  &lt;field name="prevInvoice" includeFrom="moneyTransfer.invoiceDS.info" includeVia="mtPrevId"/&gt;
*  &lt;field name="prevParentReceipt" includeFrom="moneyTransfer.moneyTransfer.receiptDS.info" includeVia="mtPrevId"/&gt;
*  &lt;field name="prevParentInvoice" includeFrom="moneyTransfer.moneyTransfer.invoiceDS.info" includeVia="mtPrevId"/&gt;
*  
* @type {string}
* @default null
*/
otherFKs?: string;

/**
* When records from this dataSource are displayed in a dataBoundComponent such as a
*  ListGrid, the displayField attribute may be used to cause some 
*  field to display a value from another field in the record.
*  
*  This is typically used for editable DataSourceField.foreignKey fields. 
*  In this scenario, a dataSource
*  field has a foreignKey field which stores an ID value used to identify records in another,
*  related dataSource. Rather than display this ID to users, developers may wish to display
*  another, user-friendly field from the related record. This is easy to achieve by 
*  having a second field on the dataSource which will be populated with the "display value"
*  from this related dataSource, and using dataSourceField.displayField 
*  to show this value. The DataSourceField.includeFrom feature handles populating
*  this field automatically for dataSources backed by the
*  serverDataIntegration. See the "Editing included fields"
*  section of the DataSourceField.includeFrom documentation
*  for more on editing included foreignKey fields.
*  
*  Editable dataSourceFields with a specified displayField and 
*  foreignKey will typically be edited using a SelectItem 
*  or ComboBoxItem. In this case, in addition to identifying the field to use as a
*  static display value within the record being edited, displayField
*  will also identify which field on the related dataSource
*  to use as a display field when showing a set of options to the user. This behavior may
*  be modified in a couple of ways:
*  
*  
* - The DataSourceField.foreignDisplayField attribute may be used to handle the
*    case where the name of the field used as a displayField within the dataSource is
*    different from the name of the included/equivalent field in the related dataSource.
*  
* - The DataSourceField.useLocalDisplayFieldValue attribute may be explicitly
*    set to false to avoid picking up a display value from the local record altogether.
*    Instead the displayField will be used only to derive the display value from
*    a related record from the optionDataSource
*  
*  For more on how FormItems use the displayField property, see FormItem.displayField.
* @type {string}
* @default null
*/
displayField?: string;

/**
* The FormItem.useLocalDisplayFieldValue attribute may be specified
*  within a dataSource configuration.
*  
*  This property governs whether, when displaying a record in an editor component,
*  the displayField value for this field should be picked up directly from
*  the record value (as opposed to being retrieved via an explicit fetch operation against
*  the FormItem.optionDataSource). See FormItem.useLocalDisplayFieldValue
*  for further details.
*  
*  If not explicitly set, dataSources backed by the 
*  serverDataIntegration will set this property to true
*  automatically for fields where the specified DataSourceField.displayField values
*  are retrieved from another dataSource using the DataSourceField.includeFrom 
*  feature.
* @type {boolean}
* @default null
*/
useLocalDisplayFieldValue?: boolean;

/**
* Name of another field in a separate dataSource that should be used as the display
*  value for this field in the case where a foreignKey relationship
*  exists. Derived automatically when using the displayField / includeFrom pattern as
*  illustrated in the following example.
*  
*  For a general overview on picking up display values from a separate field,
*  see DataSourceField.displayField property.
*  
*  The foreignDisplayField property is useful for editable fields, where the 
*  name of the display field within the record being edited differs from the display field 
*  for related records in the option dataSource.
*  For example consider a "countryDS" dataSource with the following fields:
*  
*  &lt;field name="id"   type="sequence"  hidden="true"   primaryKey="true" /&gt;
*  &lt;field name="name"  type="text"    title="Country"  required="true" /&gt;
*  
*  ...and a "city" dataSource which uses a foreignKey relationship identify associated
*  country records:
*  
*  &lt;field name="id"    type="sequence" hidden="true"  primaryKey="true" /&gt;
*  &lt;field name="name"   type="text"   title="City"  required="true" /&gt;
*  &lt;field name="countryId" type="integer" editorType="SelectItem" 
*        foreignKey="countryDS.id" 
*        displayField="countryName" foreignDisplayField="name" title="Country" /&gt;
*  &lt;field name="countryName" includeFrom="countryDS.name" hidden="true"  /&gt;
*  
*  A DynamicForm bound to this "city" dataSource would display a SelectItem editor by
*  default for the country field. The initial display value would be the local value from
*  the "countryName" field, populated from the related countryDS automatically via the
*  DataSourceField.includeFrom feature.
*  If the user showed the drop-down list of options for this field, the display values
*  within that list would be picked up from the "name" field values for the related
*  "countryDS" records. Again, note that the foreignDisplayField would have been derived in
*  this case, and need not be specified explicitly.
*  
*  Note that when specified, foreignDisplayField is always expected to be set to 
*  the related dataSource field containing equivalent values to the displayField
*  in the local dataSource. This is important as, when editing the field, foreignDisplayField values from 
*  the related dataSource will be displayed to the user, and when a value is selected
*  the local record's displayField value will be updated to match the selected
*  foreignDisplayField value from the related dataSource's record. This behavior
*  is documented under FormItem.displayField.
* @type {string}
* @default null
*/
foreignDisplayField?: string;

/**
* This property is only applicable to fields of SQL DataSources that also specify a 
*  DataSourceField.foreignKey property; it is ignored for all other fields. Indicates the 
*  type of join to make between the tables underlying this DataSource and the other DataSource
*  referred to in the foreignKey property, when resolving DataSourceField.includeFrom 
*  fields. The default value of null is the same as specifying "inner".
*  
*  Note, outer joins are allowed for all supported database products only if you are using 
*  DataSource.useAnsiJoins, which is not the case by default. If you
*  are using the older strategy of additional join expressions in the WHERE clause, outer 
*  joins are only supported for database products that provide a proprietary native syntax for 
*  expressing outer joins. Those products are:
*  
* - Oracle
*  
* - Versions of Microsoft SQL Server earlier than 2012, and running in compatibility mode 80
*  
* @type {JoinType}
* @default null
*/
joinType?: JoinType;

/**
* This property is only applicable to fields of SQL DataSources, it overrides default behavior
*  of OperatorId case insensitive operator on the server-side. Set to
*  true to force case insensitivity by changing case of both comparison expression
*  sides or false to rely on database LIKE operator directly.
*  See sqlSettings for more details including database specific defaults and other
*  configuration options.
* @type {boolean}
* @default null
*/
sqlForceInsensitive?: boolean;

/**
* If ListGrid.showGridSummary or ListGrid.showGroupSummary is true,
*  this attribute can be used to specify
*  an explicit SummaryFunction for calculating the summary value to
*  display.
*  
*  If an array of summaryFunctions is specified, they will be executed in turn and the
*  grid will show multiple summary rows at the grid or group level (or both)
*  containing the resulting values.
* @type {SummaryFunction}
* @default null
*/
summaryFunction?: SummaryFunction;

/**
* Title to show in a SummaryFunction for this field. If unspecified
*  title summaries will show the DataSourceField.title for the field.
* @type {string}
* @default null
*/
summaryValueTitle?: string;

/**
* Indicates that DataSource.xmlSerialize should serialize this value as an XML
*  attribute.
*  
*  Note this does not need to be declared in order for DataSource records to be derived from
*  XML data: a field will be populated with either an attribute or subelement with matching
*  name.
* @type {boolean}
* @default null
*/
xmlAttribute?: boolean;

/**
* For a field that is DataSourceField.multiple, controls the name of the XML tag used for each
*  subelement during DataSource.xmlSerialize.
*  
*  If unset, the default tag name is "value" for a field of simple type, and for a field of
*  DataSource type, is the tagName or ID of the DataSource (as though
*  xmlSerialize() were called on the child DataSource).
* @type {string}
* @default null
*/
childTagName?: string;

/**
* For use in componentSchema for fields that contain other components, this flag
*  suppresses auto-construction for subcomponents that appear under this field.
*  
*  For example, the VLayout schema sets this for its Layout.members
*  property, so that when a VLayout is constructed via XML as follows:
*  
*  &lt;VLayout&gt;
*    &lt;members&gt;
*      &lt;ListGrid ID="myGrid" .../&gt;
*      &lt;Toolstrip ID="myToolStrip" ... /&gt;
*    &lt;/members&gt;
*  &lt;/VLayout&gt;
*  
*  The ListGrid and ToolStrip do not construct themselves automatically. Instead, the VLayout
*  receives the properties of the ListGrid and ToolStrip as ordinary JavaScript Objects, with
*  the special property _constructor set to the name of the class that should be
*  constructed.
* @type {boolean}
* @default null
*/
propertiesOnly?: boolean;

/**
* For use in componentSchema, a field DataSource.inheritsFrom from
*  another schema can be redeclared with this property set in order to indicate that the
*  property should not be used.
*  
*  This is primarily used to influence reify. For simple type properties,
*  this avoids the property appearing in the Component Editor. 
*  
*  For fields that hold subcomponents, this prevents inappropriate drag and drop. For example,
*  a custom class called MyDialog may automatically create a series of children,
*  and not allow arbitrary other children to be added. In this case, the inherited property
*  Canvas.children should be marked inapplicable in order to prevent arbitrary
*  components being dropped onto a MyDialog instance.
* @type {boolean}
* @default null
*/
inapplicable?: boolean;

/**
* For use in componentSchema, indicates what group to place the property in when
*  editing in Reify.
* @type {string}
* @default null
*/
group?: string;

/**
* For use in componentSchema for fields that, when changed in
*  reify, should discard the current live component and recreate it
*  with the current configured properties.
*  
*  This property is typically set when a custom component is being used that doesn't have
*  a setter for the field.
* @type {boolean}
* @default null
*/
recreateOnChange?: boolean;

/**
* Indicates that this field should always be Array-valued. If the value derived from
*  DataSource.dataFormat is singular, it will be wrapped in an Array.
*  
*  JPA and Hibernate DataSources use multiple:true as part of the declaration of
*  One-To-Many and Many-to-Many relations - see jpaHibernateRelations for details.
*  
*  Criteria on multiple:true fields: client-side filtering
*  
*  For simple Criteria, the criteria value is compared to each field value in the
*  multiple:true field, according to the
*  DSRequest.textMatchStyle. If any field value matches the
* j filter value, the field is considered to match the criteria.
*  
*  For AdvancedCriteria, for normal OperatorId the field
*  value is considered as matching the Criterion if any of the field values
*  match the Criterion. Specifically, this is true of all operators that have an 
*  OperatorValueType of "fieldType" or "valueRange".
*  
*  For operators that compare against other fields in same record, such as "equalsField", 
*  if the other field is not multiple:true, matching works the same as for
*  normal operators, that is, as if criterion.value directly contained the value
*  rather than the name of another field.
*  
*  If the other field is also multiple:true, only "equalsField", "notEqualsField",
*  "iEqualsField" and "iNotEqualsField" are allowed (any other operator will
*  cause a warning and be ignored) and the set of values in the field must be identical (aside
*  from case, for operators prefixed with "i") and in identical order to match.
*  
*  For the inSet operator, the field matches if there is any intersection between
*  the field values and the array of values provided in criterion.value.
*  notInSet is the reverse.
*  
*  "Finally, for "isNull" and "notNull", an empty Array is considered non-null, however,
*  for "isBlank" and "notBlank", an empty Array is considered blank, since a control such
*  as a multiple:true SelectItem or MultiComboBoxItem would appear blank to the end user if
*  it had an empty Array as its value.
*  
*  Server-side Representation and Storage
*  
*  Values for multiple:true fields appear as Java Lists when received in server code such as a
*  DMI. The SmartClient Server supports simple storage of values that are multiple:true, controlled
*  via the DataSourceField.multipleStorage setting.
*  
*  For server-side behavior of JPA and Hibernate relation fields that are multiple:true, see
*  jpaHibernateRelations.
*  
*  For non-relation fields, the SmartClient Server supports simple storage of values that are
*  multiple:true, controlled via the DataSourceField.multipleStorage setting, with some limited support
*  for server-side filtering, as described in the DataSourceField.multipleStorage docs.
*  
*  For the built-in SQL, Hibernate and JPA connectors, if criteria are specified for a
*  multiple:true field where multipleStorage is null or "none", the SmartClient
*  server knows nothing about how the multiple values are stored, so as a fallback the criteria
*  will operate as though the field were a normal, non-multiple "text" field. This will
*  generally not match the client-side filtering behavior described above, so filtering
*  should either be performed entirely on the client (for example, via
*  ListGrid.dataFetchMode or entirely on the server (via
*  ResultSet.useClientFiltering:"false")
*  
*  The server-side filtering is done through a criteria transform which happens with
*  DataSource.transformMultipleFields.
*  
*  XML Serialization
*  
*  Specifically for XML serialization and deserialization, multiple:true behaves
*  similarly to the 
*  SOAP array idiom, that is,
*  there will be a "wrapper element" named after the field name, whose contents will be several
*  elements of the specified DataSourceField.type. 
*  
*  For example, Layout.members is declared with type:"Canvas",
*  multiple:true. The correct XML format is thus:
*  
*  &lt;VLayout&gt;
*    &lt;members&gt;
*      &lt;Canvas ID="myCanvas" ... /&gt;
*      &lt;ListGrid ID="myGrid" .../&gt;
*      &lt;Toolstrip ID="myToolStrip" ... /&gt;
*    &lt;/members&gt;
*  &lt;/VLayout&gt;
*  
*  
*  See DataSourceField.childTagName for customizing the tagName used for subelements.
* @type {boolean}
* @default null
*/
multiple?: boolean;

/**
* For fields that are DataSourceField.multiple, the separator used 
*  between values when they are displayed.
* @type {string}
* @default ", "
*/
multipleValueSeparator?: string;

/**
* How values for a DataSourceField.multiple field should be stored.
*  
*  On the server, field values for multiple:true fields are represented as Java
*  Lists in DSRequest and DSResponse objects, but when multipleStorage is used,
*  are ultimately stored as Strings. 
*  
*  This storage mechanism is "denormalized" in the sense of
*  database normalization
*  because the field contains a compound value rather than an atomic value. Specifically, this
*  makes the field harder to use with database features like SQL joins, since its value cannot
*  be directly compared to other, singular fields. For this reason, there's really only a
*  narrow set of cases where it makes sense to use this storage mechanism, consider it if:
*  
*  
* - the values don't appear in any other tables or columns elsewhere in your database (so
*    joins and other comparisons wouldn't be possible anyway)
*  
* - there is no authoritative list of all possible values, or even of popular values; it's
*    basically freeform, like arbitrary, uncurated "tags" applied to photos or links
*  
*  
*  A multiple:true field can be of any SimpleType, including "date",
*  "time", "datetime", "float", "int" etc. Type will be preserved across a store and reload
*  cycle, so for example a List of values for a field of type "int" will arrive at the server
*  in a DSRequest as a List of Integer, be stored as a String, but appear again as a List of
*  Integer inside a DSResponse. When storing "date", "time" or "datetime" values as Strings,
*  XML schema formats are used - the same that are used for transmission in XML or JSON (see
*  dateFormatAndStorage for details).
*  
*  The multipleStorage feature works by transforming from List to String and back
*  inside the server-side DataSource.execute() method. Specifically, inbound DSRequests are
*  transformed before methods that perform actual storage are called (executeUpdate() et al),
*  and DSResponses received from these methods have multiple:true field values
*  transformed back into Lists before DataSource.execute() returns the DSResponse.
*  
*  This transformation approach means that the built-in SQL, JPA and Hibernate connectors all
*  support multipleStorage, and multipleStorage is also automatically
*  available for any custom DataSource that is capable of storing String values, and implements
*  its storage methods via overrides of executeUpdate(), executeAdd() et al as described in the
*  writeCustomDataSource.
*  
*  Note that when using the built-in SQL, JPA and Hibernate connectors, the underlying SQL
*  column should be of 'text' type. The dbConfigTool will
*  automatically generate columns of the correct type for a SQLDataSource. For JPA or
*  Hibernate DataSources, just ensure the type of the Java bean property on your Java object is
*  String.
*  
*  The default multipleStorage setting of null means that the "simpleString"
*  MultipleFieldStorage approach will be used if:
*  
*  
* - the field is of a known SimpleType
*  
* - the field is not a DataSourceField.foreignKey
*  
*  Otherwise, MultipleFieldStorage "none" will be used.
*  
*  Criteria and AdvancedCriteria handling
*  
*  For the built-in SQL, JPA and Hibernate connectors, when
*  multipleStorage:"simpleString" or "json" is used, criteria are transformed to
*  replicate the client-side filtering behavior for multiple:true fields, where possible.
*  The following operators are supported with the same behavior as client-side filtering:
*  
*  
* - all String-oriented operators including patternOperators
*  
* - regexp / iRegexp (built-in SQL only, JPA and Hibernate do not support these)
*  
* - isBlank / notBlank
*  
* - isNull / notNull
*  
* - inSet / notInSet
*  
* - equalsField / notEqualsField / iEqualsField / iNotEqualsField
*  
*  The following operators, which are supported for client-side filtering of multiple:true
*  fields, are not supported for server filtering when using multipleStorage:
*  
*  
* - greaterThan/lessThan(OrEqual)
*  
* - "between" and all other operators with OperatorValueType of "valueRange" 
*  
* - regexp / iRegexp as noted above
*  
*  Note that for string-based filtering operators such as "equals", no characters which are
*  part of the DataSourceField.multipleStorageSeparator may be used in the filter string. If any
*  characters from the multipleStorageSeparator are present in the filter value,
*  it will always fail to match. For "json" mode, the multipleStorageSeparator is
*  effectively the String '","'.
* @type {MultipleFieldStorage}
* @default null
*/
multipleStorage?: MultipleFieldStorage;

/**
* For fields that are DataSourceField.multiple and use DataSourceField.multipleStorage, the
*  separator used in the "simpleString" MultipleFieldStorage mode.
*  
*  Default value of null means the DataSourceField.multipleValueSeparator is used instead.
* @type {string}
* @default null
*/
multipleStorageSeparator?: string;

/**
* Explicitly declares the Java class that should be used when data from the client is
*  validated by the SmartClient server and/or applied to Java Beans / POJOs via the server-side
*  API com.isomorphic.datasource.DataSource.setProperties(). This includes 
*  auto-populating POJO arguments of a dmiOverview method, or populating
*  JPA/Hibernate beans with data when using the built-in JPA and Hibernate DataSources.
*  
*  For DataSources that do not use Java Beans, javaClass can be used to force a
*  particular representation for validated DSRequest data (e.g. data passed to a DMI):
*  
*  
* - for fields declared to be of type "integer" or "float" valid settings include "BigInteger",
*  "Long", "Integer", "Short", "Byte", "AtomicInteger", "AtomicLong", "BigDecimal", "Double", "Float".
*  
* - for fields declared to be of type "date" valid settings include basic "java.util.Date",
*  "java.sql.Date" and "java.sql.Time" types as well as Joda-Time types "DateTime", "DateMidnight",
*  "LocalDateTime", "LocalDate", "LocalTime" and Java 8 Date/Time API types "LocalDate", "LocalDateTime",
*  "LocalTime".
*  
*  When populating Java Beans/ POJOs, javaClass does not normally have to
*  specified: SmartClient will use Java reflection to inspect the type of argument expected by
*  a setter method and will attempt conversion of inbound data to that type. As described in
*  the documentation for DataTools.setProperties(), this works for almost all
*  typical cases. However field.javaClass is useful for:
*  
*  
* - subobject of abstract or interface type: in this case Java Reflection is not sufficient
*  to discover the concrete type that should be instantiated, and javaClass should be
*  set instead.
*  
* - subobject of Collection or Map type, when Java generics are not used or the Collection
*  member type or Map value type is abstract. When Java generics are used (for example the
*  setter takes an argument is of type Collection&lt;SomePOJO&gt; or Map&lt;KeyType,SomePOJO&gt;,
*  SmartClient will automatically attempt to convert inbound data to the type of the members of
*  the Collection or values of the Map. Without generics, javaClass needs to be
*  specified. Note that javaClass will take precedence over generics if both 
*  are used. Also note that DataSourceField.javaCollectionClass can be specified if a
*  particular Collection or Map type is needed, and DataSourceField.javaKeyClass can be
*  specified for a field of type java.util.Map.
*  
*  NOTE: It is also possible to make SmartClient Server determine the javaClass to use 
*  dynamically at runtime. See the description of DataSource.getPropertyJavaClass() 
*  in the server documentation for details.
* @type {string}
* @default null
*/
javaClass?: string;

/**
* See DataSourceField.javaClass - when auto-populating of Java Beans / POJOs based on 
*  inbound DSRequest data, for a field of type Collection or Map, 
*  javaCollectionClass can 
*  be used to specify a particular concrete class to use. If not specified, and a concrete
*  Collection or Map class cannot be discovered using Java Reflection, the following concrete
*  classes are used:
*  
*  
* - java.util.ArrayList is used for fields of type List
*  
* - java.util.HashSet is used for fields of type Set
*  
* - java.util.LinkedList is used for fields of type Queue
*  
* - java.util.LinkedHashMap is used for fields of type Map
*  
* - java.util.ArrayList is used for fields that are otherwise of type Collection
*  
*  Note that this value is used even if the target Collection or Map is declared as a concrete
*  class. So, for example, if you set javaCollectionClass to 
*  java.util.LinkedList but your setter method accepts a 
*  java.util.ArrayList, you will get a ClassCastException.
* @type {string}
* @default null
*/
javaCollectionClass?: string;

/**
* See DataSourceField.javaClass - when auto-populating of Java Beans / POJOs based on 
*  inbound DSRequest data, for a field of Map type, javaKeyClass can be used to
*  specify a particular concrete class for the map keys. If not specified, and a concrete
*  type cannot be discovered using Java Reflection, java.lang.Object is used.
* 
*  Note that javaKeyClass take precedence over generics if both are used.
* @type {string}
* @default null
*/
javaKeyClass?: string;

/**
* Optional different field-title used for exports.
* @type {string}
* @default null
*/
exportTitle?: string;

/**
* Dictates whether the data in this field be exported. Explicitly setting 
*  canExport to false overrides the setting on any component-fields, such
*  as ListGridField.canExport.
* @type {boolean}
* @default null
*/
canExport?: boolean;

/**
* Used by the BatchUploader to map a field in an upload file to this 
*  dataSourceField. This is only necessary if the dataSourceField's name and title differ 
*  from the name of the field in the upload file (SmartClient will automatically map upload 
*  fields using the dataSourceField's title, if possible, if it does not get a direct match 
*  on field name).
* @type {string}
* @default null
*/
uploadFieldName?: string;

/**
* Used to look up key values for fields with DataSourceField.importStrategy
*  while importing data using BatchUploader or DataImport server-side API.
*  
*  Normally key values are looked up performing "fetch" operation on related data source, but
*  setting batchUploadOperationId allows to control which operation will be performed.
*  This can be useful to avoid unnecessary load when importing big data sets, if default "fetch"
*  operation is too heavy.
*  
*  By default the look up is not case sensitive but this can be controlled through the
*  DataSourceField.batchUploadCaseSensitive attribute.
* @type {string}
* @default null
*/
batchUploadOperationId?: string;

/**
* Applies to the "fetch" operation that is used to look up key values for fields with
*  DataSourceField.importStrategy while importing data using
*  BatchUploader or DataImport server-side API. Controls whether look up
*  for key values is case sensitive or not. Default value is "false".
* @type {boolean}
* @default false
*/
batchUploadCaseSensitive?: boolean;

/**
* For a field of type:"imageFile", indicates whether to stream the image and display it
*  inline or to display the View and Download icons.
* @type {boolean}
* @default null
*/
showFileInline?: boolean;

/**
* Width of the image-content of this field. If set as a string, represents the name of 
*  another field in the record that holds the imageWidth. Applicable only to fields of image 
*  type or fields that use a ViewFileItem as an editor.
* @type {number | string}
* @default null
*/
imageWidth?: number | string;

/**
* Height of the image-content of this field. If set as a string, represents the name of 
*  another field in the record that holds the imageHeight. Applicable only to fields of image 
*  type or fields that use a ViewFileItem as an editor.
* @type {number | string}
* @default null
*/
imageHeight?: number | string;

/**
* Width and height of the image-content of this field. If set as a string, represents the 
*  name of another field in the record that holds the imageSize. Applicable only to fields 
*  of image type or fields that use a ViewFileItem as an editor.
* @type {number | string}
* @default null
*/
imageSize?: number | string;

/**
* Whether a user must be authenticated in order to fetch this field. If this property is 
*  set and the user is not authenticated, SmartClient Server will not return values for this
*  field to the client.
*  
*  The way this behavior works is to remove the field from the expressed or implied list of
*  fields to return (note, this means the field is removed from the list of fields returned 
*  in data records; the actual field definition in the dataSource is still returned,
*  but in a stripped-down state, as described below). 
*  A list of fields can be expressed using OperationBinding.outputs
*  or DSRequest.outputs; in the absence of those, the entire set of fields defined in 
*  the DataSource is implied. You can override this behavior on a per-request basis by 
*  providing a DMI that just re-adds the names of the fields you want for that particular 
*  request to the "consolidated outputs". See the server-side Javadoc for 
*  DSRequest.getConsolidatedOutputs().
*  
*  In addition to this behavior of trimming off values in fetched records, a field that fails
*  a field-level security check (either this one or one of the two more specific checks, 
*  DataSourceField.viewRequiresRole and DataSourceField.viewRequires) will be 
*  altered in the version of the dataSource returned to the client by the 
*  loadDSTag or 
*  dataSourceDeclaration.
*  
*  
* - If the field is also not editable (see the discussion of this below), it will be marked 
*  DataSourceField.canView:false, and thus effectively be inert and unusable on the client
*  
* - If the field is editable, it will be marked DataSourceField.hidden. By default, this means
*  that DataBoundComponents will not show the field, though this can be overridden for 
*  individual components. If you do choose to unhide the field, bear in mind that it will 
*  never actually show any data, because the underlying security test has still failed
*  
*  By default, view constraints automatically imply edit constraints - almost all of the time,
*  if a user is not allowed to view a field, he should not be allowed to update it either. If
*  you specify a view constraint, the system behaves as if you had also specified the 
*  corresponding edit constraint - so viewRequiresAuthentication on a field 
*  implies DataSourceField.editRequiresAuthentication.
*  You override this behavior simply by specifying some kind of edit constraint for the field,
*  either a different one (eg, "users" can view a field, but only "managers" can update it) or 
*  a blank one (if you have some unusual requirement whereby users are not allowed to view a 
*  field, but they can update it).
* @type {boolean}
* @default null
*/
viewRequiresAuthentication?: boolean;

/**
* Whether a user must be authenticated in order to write to this field. If this property is 
*  set and the user is not authenticated, the SmartClient Server will not allow this field to
*  be updated or initialized.
*  
*  This property affects two things. Firstly, the server determines when the DataSource is 
*  first loaded if we have an authenticated user; if we don't, the field is marked 
*  canEdit: false. Secondly, when an insert or update request is received from 
*  the client, the server removes the field from the values clause before the
*  request is processed.
*  
*  Note that you can override this behavior. The canEdit setting can be overridden 
*  on the the client-side DataSource like any other client-side property. The value-removing 
*  behavior can be overridden on a per-request basis by providing a DMI that re-adds values for
*  the fields you want for that particular request to the values object (the values originally 
*  sent up by the client are available on the DSRequest). See the server-side Javadocs for 
*  DSRequest.getValues() and DSRequest.getClientSuppliedValues().
* @type {boolean}
* @default null
*/
editRequiresAuthentication?: boolean;

/**
* Whether a user must be authenticated in order to initialize to this field. If this property is 
*  set and the user is not authenticated, the SmartClient Server will not allow this field to
*  be initialized.
* @type {boolean}
* @default null
*/
initRequiresAuthentication?: boolean;

/**
* Whether a user must be authenticated in order to update to this field. If this property is 
*  set and the user is not authenticated, the SmartClient Server will not allow this field to
*  be updated.
* @type {boolean}
* @default null
*/
updateRequiresAuthentication?: boolean;

/**
* Comma-separated list of user roles that are allowed to fetch this field. If the current user
*  has any of the roles listed, values for the field will be fetched. Please see 
*  OperationBinding.requiresRole for further details of SmartClient's declarative 
*  role-based security. Please see 
*  DataSourceField.editRequiresAuthentication [or 
*  DataSourceField.viewRequiresAuthentication for viewRequiresRole]
*  for the UI consequences if the user does not have the required roles, as well as for information 
*  on customizing field-level security.
* @type {string}
* @default null
*/
viewRequiresRole?: string;

/**
* Comma-separated list of user roles that are allowed to write to this field. If the current
*  user does not have any of the roles listed, the system will not allow this field to be 
*  initialized or updated. Please see OperationBinding.requiresRole for further 
*  details of SmartClient's declarative role-based security. Please see 
*  DataSourceField.editRequiresAuthentication [or 
*  DataSourceField.viewRequiresAuthentication for viewRequiresRole]
*  for the UI consequences if the user does not have the required roles, as well as for information 
*  on customizing field-level security.
*  
*  NOTE: This property prevents both initialization and updates for a field. If you 
*  have a need to prevent just initialization or just updates, you can use
*  DataSourceField.initRequiresRole or DataSourceField.updateRequiresRole.
* @type {string}
* @default null
*/
editRequiresRole?: string;

/**
* Comma-separated list of user roles that are allowed to initialize this field. If the current
*  user does not have any of the roles listed, the system will not allow this field to be 
*  initialized.
*  
*  NOTE: This property only prevents initialization of a field; updates will still be 
*  allowed. You should only use this property if you have a special requirement; ordinarily,
*  use DataSourceField.editRequiresRole, which applies security for both types of write.
* @type {string}
* @default null
*/
initRequiresRole?: string;

/**
* Comma-separated list of user roles that are allowed to update this field. If the current
*  user does not have any of the roles listed, the system will not allow this field to be 
*  updated.
*  
*  NOTE: This property only prevents a field from being updated; initialization will 
*  still be allowed. You should only use this property if you have a special requirement; 
*  ordinarily, use DataSourceField.editRequiresRole, which applies security for both 
*  types of write.
* @type {string}
* @default null
*/
updateRequiresRole?: string;

/**
* Indicates that the specified VelocityExpression must evaluate to true if
*  values for the field are to be fetched. If the specified expression does not evaluate
*  to true, the field will be dropped as described for 
*  DataSourceField.viewRequiresAuthentication.
*  
*  In addition to the normal context variables available to Velocity expressions in SmartClient,
*  expressions you write for field-level requires clauses - viewRequires, 
*  DataSourceField.editRequires, DataSourceField.initRequires and 
*  DataSourceField.updateRequires - can reference two additional variables: 
*  $fieldName and $dsName. These are the names of the dataSource and 
*  field currently undergoing requires checks. They are helpful because they allow
*  you to write a generic checker function that can be used to handle requires
*  checks for multiple fields and dataSources.
* @type {VelocityExpression}
* @default null
*/
viewRequires?: VelocityExpression;

/**
* Indicates that the specified VelocityExpression must evaluate to true if
*  writes to this field are to be allowed. If the specified expression does not evaluate
*  to true, the field will be removed from the request as described for 
*  DataSourceField.editRequiresAuthentication.
*  
*  In addition to the normal context variables available to Velocity expressions in SmartClient,
*  expressions you write for field-level requires clauses - editRequires, 
*  DataSourceField.viewRequires, DataSourceField.initRequires and 
*  DataSourceField.updateRequires - can reference two additional variables: 
*  $fieldName and $dsName. These are the names of the dataSource and 
*  field currently undergoing requires checks. They are helpful because they allow
*  you to write a generic checker function that can be used to handle requires
*  checks for multiple fields and dataSources.
*  
*  NOTE: This property prevents both initialization and updates for a field. If you 
*  have a need to prevent just initialization or just updates, you can use
*  DataSourceField.initRequires or DataSourceField.updateRequires.
* @type {VelocityExpression}
* @default null
*/
editRequires?: VelocityExpression;

/**
* Indicates that the specified VelocityExpression must evaluate to true if
*  initializations of this field are to be allowed. If the specified expression does not 
*  evaluate to true, the field will be removed from the request as described for 
*  DataSourceField.editRequiresAuthentication.
*  
*  In addition to the normal context variables available to Velocity expressions in SmartClient,
*  expressions you write for field-level requires clauses - initRequires, 
*  DataSourceField.viewRequires, DataSourceField.editRequires and 
*  DataSourceField.updateRequires - can reference two additional variables: 
*  $fieldName and $dsName. These are the names of the dataSource and 
*  field currently undergoing requires checks. They are helpful because they allow
*  you to write a generic checker function that can be used to handle requires
*  checks for multiple fields and dataSources.
*  
*  NOTE: This property only prevents initialization of a field; updates will still be 
*  allowed. You should only use this property if you have a special requirement; ordinarily,
*  use DataSourceField.editRequires, which applies security for both types of write.
* @type {VelocityExpression}
* @default null
*/
initRequires?: VelocityExpression;

/**
* Indicates that the specified VelocityExpression must evaluate to true if
*  updates to this field are to be allowed. If the specified expression does not 
*  evaluate to true, the field will be removed from the request as described for 
*  DataSourceField.editRequiresAuthentication.
*  
*  In addition to the normal context variables available to Velocity expressions in SmartClient,
*  expressions you write for field-level requires clauses - updateRequires, 
*  DataSourceField.viewRequires, DataSourceField.editRequires and 
*  DataSourceField.initRequires - can reference two additional variables: 
*  $fieldName and $dsName. These are the names of the dataSource and 
*  field currently undergoing requires checks. They are helpful because they allow
*  you to write a generic checker function that can be used to handle requires
*  checks for multiple fields and dataSources.
*  
*  NOTE: This property only prevents update of a field; initialization will still be 
*  allowed. You should only use this property if you have a special requirement; ordinarily,
*  use DataSourceField.editRequires, which applies security for both types of write.
* @type {VelocityExpression}
* @default null
*/
updateRequires?: VelocityExpression;

/**
* Indicates that normal declarative security rules specified on this field are waived for 
*  rows that were created by the current user, as described in the discussion of 
*  DataSource.creatorOverrides. This setting overrides 
*  dataSource.creatorOverrides, for this field only.
*  
*  Note that field-level creatorOverrides is conceptually slightly different to
*  the setting at DataSource and OperationBinding levels. To give the example of a fetch 
*  operation, at the other two levels, it results in a limited set of rows being returned, 
*  rather than a security exception being thrown. At the field-level, it results in values 
*  for individual fields being present in those rows, rather than them being stripped out on
*  the server.
* @type {boolean}
* @default null
*/
creatorOverrides?: boolean;

/**
* Determines when the field value should be saved to the audit DataSource when
*  DataSource.audit is enabled. Setting the property to "change" will conserve
*  storage by only saving out the value when it's changed, and setting it to "never" will
*  exclude the field values from ever being saved during auditing.
*  Binary fields
*  Because binary data can be very large compared to other data types, and in many use cases 
*  does not typically change (for example, an application that stores scanned documents), 
*  binary fields are treated specially for auditing purposes:
*  
* - The default audit setting is "change"
*  
* - The implied _filename, _filesize and _date_created
*  metadata fields inherit their audit setting from the associated binary field,
*  if no explicit setting is provided for the metadata field. This includes inheriting the 
*  the implicit "change" default. See the binaryFields 
*  for more details about the binary metadata fields, including how to provide explicit 
*  properties for them
*  
* - By default, we determine if the binary content has changed by comparing the 
*  _filename and _filesize metadata values. If both those values are
*  unchanged, we assume the content is also unchanged. If you want to compare the actual 
*  binary content instead, set DataSource property 
*  DataSource.compareMetadataForAuditChangeStatus to false, but bear in mind this may
*  have performance implications, as described in the documentation for that property
* @type {FieldAuditMode}
* @default "always"
*/
audit?: FieldAuditMode;

/**
* For a DataSource with DataSource.serverType "sql", "hibernate", or "jpa",
*  indicates that this field should be omitted by default from all SQL operations,
*  and will only be used with customQuerying.
*  
*  Having marked a field as customSQL you can refer to it via
*  $criteria.fieldName or $values.fieldName in customized queries.
*  
*  The following are situations where you would not use customSQL:
*  
*  
* - simple joins where you want to enable users to see and search on a field from another
*  table; consider DataSourceField.includeFrom instead
*  
* - fields where you want to calculate or transform values in SQL on load or save, but
*  always perform the same calculation for each operationType; consider instead
*  DataSourceField.sqlStorageStrategy for some common cases, or 
*  DataSourceField.customSelectExpression, DataSourceField.customUpdateExpression and
*  DataSourceField.customInsertExpression for full customization
*  
* - a special fetch is needed where the field needs to be excluded from the
*  $defaultWhereClause so that it can be used in a custom &lt;whereClause&gt; - consider
*  OperationBinding.excludeCriteriaFields instead
*  
*  
*  Use customSQL in situations like:
*  
*  
* - there are multiple variations of the "fetch" operation with different
*  OperationBinding.operationId, and the field is only used in some of them;
*  in that case, consider using OperationBinding.customFields to selectively re-introduce
*  SQL generation for the field only in operations where it's used.
*  
* - the field represents hidden criteria on a field in another table where the field is
*  never shown to the user
*  
* - the field is a write-only value only saved in some operations
*  
* - more than one data access strategy is in use (eg direct SQL for fetch and bean-based
*  persistence accessed via DMI for saves) and certain fields are not available in SQL
*  
* @type {boolean}
* @default null
*/
customSQL?: boolean;

/**
* This property indicates that this field represents a custom expression that should be 
*  embedded in the generated SQL instead of a reference to this field. For example, if 
*  you have a field partialName where this value is set to 
*  SUBSTR(surname, 2), the generated SQL would look similar to this:
*  
*   SELECT ... SUBSTR(surname, 2) AS partialName ...
*  
*  Note that adding "AS partialName" as shown above is required - this is how the framework
*  identifies the expression as producing a value for this particular DataSourceField when
*  results are received from the database.
*  
*  Fields with customSelectExpression set can be used for sorting and filtering in
*  the normal way, but they are only applicable to update-type operations if you also provide a
*  corresponding DataSourceField.customUpdateExpression and/or 
*  DataSourceField.customInsertExpression. See the documentation 
*  for those methods for the rules of how they are applied.
*  
*  You can use Velocity expressions in your customSelectExpressions, and the 
*  velocitySupport are available. Note that the results
*  of Velocity evaluations are automatically quoted and escaped by default; this behavior can
*  be switched off - see DataSourceField.autoQuoteCustomExpressions, and 
*  particularly note the warnings about its use.
*  
*  Note that this property only applies to users of the SmartClient server using dataSources of
*  type "sql".
* @type {VelocityExpression}
* @default null
*/
customSelectExpression?: VelocityExpression;

/**
* This property specifies the value to use to update this column on "update" operations.
*  The value of this property will be passed through Velocity evaluation and then embedded
*  directly in the SQL generated to perform the update. It can be used in conjunction with
*  DataSourceField.customSelectExpression to provide bi-directional mapping 
*  between application data formats and persisted data formats. Or, it can be used 
*  unilaterally as a means of silently enforcing data rules - for example, ensuring that all 
*  values for a given field are in upper case.
*  
*  You can use Velocity expressions in your customUpdateExpressions, and the 
*  velocitySupport are available. Note that the results
*  of Velocity evaluations are automatically quoted and escaped by default; this behavior can
*  be switched off - see DataSourceField.autoQuoteCustomExpressions, and 
*  particularly note the warnings about its use.
*  
*  The value must be a string that will end up as a valid SQL snippet after Velocity
*  evaluation. The following examples are valid:
*  
*  
* - "SUBSTR(ANOTHER_COLUMN, 1, 10)"
*  
* - "SUBSTR($values.someTextField.toUpperCase(), 1, 10)"
*  
* - "SOME_NUMERIC_COLUMN * 1000"
*  
* - "$values.someTextField.toUpperCase()" (this is a technically valid but 
*  incorrect usage - see below)
*  
*  NOTE: although all these examples are valid, customUpdateExpression is
*  intended to be used when you have a need to customize the actual SQL. If you simply want
*  to use a server-derived value in the query (as is the case with the last of these 
*  examples), you should use transactionChaining, 
*  dmiOverview, or a serverScript instead. 
*  
*  In the examples above, the first is an actual SQL snippet and must not be enclosed
*  in quotes. The second shows an SQL snippet containing a literal - because the literal is 
*  the result of a Velocity evaluation, it will be automatically quoted so there is no need 
*  to do so manually. The third example yields a number, so it must not be quoted.
*  The last example is also the result of a Velocity evaluation and thus does not need to
*  be quoted. Note, however, that this example involves no SQL customization - this kind of 
*  usage should be avoided, as noted above.
*  
*  When an "update" operation runs, any fields that specify 
*  DataSourceField.customSelectExpression but do not specify 
*  customUpdateExpression will be ignored. When an "add" operation runs, this property acts 
*  as a default for any fields that do not specify a 
*  DataSourceField.customInsertExpression; similar to update, any fields that 
*  specify a customSelectExpression but do not specify either a customUpdateExpression or 
*  customInsertExpression, will be ignored when "add" operations run.
*  
*  Note that this property only applies to users of the SmartClient server using dataSources of
*  type "sql".
* @type {VelocityExpression}
* @default null
*/
customUpdateExpression?: VelocityExpression;

/**
* This property is similar to DataSourceField.customUpdateExpression; its value
*  is used during "add" operations. If you do not specify this property, but do specify a 
*  customUpdateExpression, the customUpdateExpression is used instead.
*  
*  Everything that applies to customUpdateExpression also applies to this property,
*  including the observation that fields which specify a 
*  DataSourceField.customSelectExpression but no corresponding 
*  customUpdateExpression or customInsertExpression, will be ignored
*  when adding new records.
*  
*  As with customUpdateExpression, this property is only intended to be used when
*  you have a need to customize the actual SQL. If you simply want to use a server-derived 
*  value in the query, you should use transactionChaining, 
*  dmiOverview, or a serverScript instead.
* 
*  customInsertExpression with primaryKey fields
* 
*  if you use customInsertExpression to derive a value for a 
*  DataSourceField.primaryKey field, the key value will not by default be 
*  available to the framework, so the built-in cache synchronization system will not work.
*  It may be possible to to work around this with the 
*  DataSourceField.autoGenerated flag, as described below, but otherwise 
*  you must also create a OperationBinding.cacheSyncOperation which 
*  can retrieve the record just added to the database. Please also see the documentation for
*  SequenceMode, which talks further about cacheSyncOperation.
*  
*  Typically, primaryKey fields that need to be auto-generated are implemented
*  using a sequence field, which at the database level translates to either an actual sequence,
*  or an auto-increment or "identity" column. JDBC drivers have special support for such
*  columns, so we are able to determine the generated value. This is important for primaryKey 
*  fields because, without it, we cannot re-fetch the record just inserted for cache 
*  synchronization purposes, as described above. Where a value is generated by something other
*  than a sequence field, this special ability is lost - or at least, it is no longer 
*  guaranteed to be present.
*  
*  There are use cases where this may seem to be a restriction. A common case would be where 
*  you wish to use a generated UUID as primaryKey value, rather than an ordinary ascending 
*  sequence. If you had intended to do this using the database itself - for example, 
*  specifying a customInsertExpression that calls something like MySQL's 
*  UUID() built-in function - this is potentially a problem. What we
*  would recommend in this case and anything similar, is to use the general ability for field 
*  values to be generated by application code, as described above. (Note, if you want to have
*  application code generate values for primaryKey fields, you may need to use an 
*  OperationBinding that specifies 
*  OperationBinding.providesMissingKeys). 
*  
*  That said, different database vendors implement their JDBC drivers in different ways, and 
*  with the SQLDataSource and certain database products it is possible to retrieve 
*  values that were generated at the database level by a customInsertExpression 
*  that expresses either a user-defined function call or an inline subselect, as long as the 
*  field is marked with the autoGenerated flag. Because this behavior
*  is potentially useful, we do not inhibit it; however, we also do not recommend that you make 
*  use of it, because the behavior is not portable across databases, and may also not be 
*  portable across JDBC driver versions (because the behavior is unspecified and undocumented,
*  and relies on implementation details that may change). Databases where this behavior has 
*  been observed to work include Oracle, Postgres and HSQLDB; databases where this behavior is 
*  known not to work include MySQL, SQL Server, DB2 and Firebird (the latter because its JDBC 
*  driver does not support the getGeneratedKeys() API). Note that in order to
*  retrieve values that were generated at database level SequenceMode needs to be set to
*  "jdbcDriver" via DataSource.sequenceMode or via "sql.MyDatabase.sequence.mode"
*  sqlSettings.
*  
*  Note that this property only applies to users of the SmartClient server using dataSources of
*  type "sql".
* @type {VelocityExpression}
* @default null
*/
customInsertExpression?: VelocityExpression;

/**
* This property indicates that this field should be represented by a custom expression
*  embedded in the WHERE clause of the generated SQL, instead of the generated expression
*  containing this field name that would ordinarily be used. You use this property when 
*  you have to accommodate some special way of using a field's value in criteria, other than by
*  directly comparing it to the criteria. For example, if you have a column that contains 
*  bit-packed information you will generally need to perform a bitwise AND to filter on that 
*  column, rather than an equality check. In this case, you would use a 
*  customCriteriaExpression something like this (in Oracle):
*  
*  &nbsp;&nbsp;&nbsp;BITAND(myField, $criteria.myField)+0 = $criteria.myField
*  
*  Or this (in MySQL or SQL Server):
*  
*  &nbsp;&nbsp;&nbsp;myField & $criteria.myField = $criteria.myField
*  
*  As this example shows, a customCriteriaExpression is expected to be a complete
*  logical expression that provides both sides of the comparison. After Velocity processing,
*  this example would result in SQL similar to this (for the MySQL case); the colored part 
*  comes entirely from the customCriteriaExpression:
*  
*  SELECT myField, myOtherField FROM myTable WHERE myField & 32 = 32
*  
*  customCriteriaExpression can also be used with AdvancedCriteria.
*  Note that the point mentioned above, about a customCriteriaExpression being a 
*  complete logical expression that provides both sides of the comparison still applies. This
*  means that when you use customCriteriaExpression in conjunction with 
*  AdvancedCriteria, you effectively switch off support for different 
*  OperatorIds to be used on that field, unless you use Velocity expressions 
*  containing the special $criteriaOperator variable (see below).
*  
*  When you use customCriteriaExpression with AdvancedCriteria, the 
*  way you access criteria values differs slightly. One option is to use the 
*  $advancedCriteria Velocity variable, as described in the "Using AdvancedCriteria"
*  section of the customQuerying. However, this 
*  variable only returns the value from the first Criterion that uses the field,
*  as found by depth-first search. If your AdvancedCriteria contains multiple 
*  references to this field, this will not be satisfactory. Therefore, specifically for 
*  customCriteriaExpression, we provide another Velocity variable, 
*  $criteriaValue. This variable means "the value on a Criterion referencing 
*  this field". An example may serve to clarify this:
*  
*  Assume you need to extend the above example to filter records where a certain bit is set 
*  OR a certain other bit is set. An AdvancedCriteria like this would be 
*  required (note that the choice of "equals" as the operator in these clauses is completely
*  arbitrary - as discussed above, the operator is ignored when we have a 
*  customCriteriaExpression, so it would make no difference to the end product 
*  if we had used different operators here):
*   { _constructor: "AdvancedCriteria", operator: "or", criteria: [
*     { fieldName: "myField", operator: "equals", value: 32 },
*     { fieldName: "myField", operator: "equals", value: 64 },
*    ]
*   }    
*  
*  If you used the customCriteriaExpression from above unchanged with this criteria,
*  the following SQL would be generated (in the MySQL case):
*  
*  SELECT myField, myOtherField FROM myTable WHERE myField & null = null OR myField & null = null
*  
*  This is because $criteria is a variable that only works correctly with simple
*  criteria. If you changed the $criteria references in the expression to 
*  $advancedCriteria, the generated SQL would be:
*  
*  SELECT myField, myOtherField FROM myTable WHERE myField & 32 = 32 OR myField & 32 = 32
*  
*  This is because $advancedCriteria always returns the first value it finds for
*  a particular field, as described above. However, if you change the expression so it reads:
*  
*  &nbsp;&nbsp;&nbsp;myField & $criteriaValue = $criteriaValue
*  
*  you will now get the correct SQL generated:
*  
*  SELECT myField, myOtherField FROM myTable WHERE myField & 32 = 32 OR myField & 64 = 64
*  
*  Similar to $criteriaValue, we provide a special variable, 
*  $criteriaOperator. This is intended for use in Velocity expressions, so that 
*  your customCriteriaExpression can process different operators in different ways.
*  How you choose to interpret an operator is entirely up to you; this variable is both 
*  advanced and rarely used. This example is contrived and does not show a usage that would be 
*  remotely likely in a real application, but it does show how to use the variable in a Velocity
*  expression:
*  
*  &nbsp;&nbsp;&nbsp;myField #if ($criteriaOperator == "greaterThan") & #else | #end $criteriaValue = $criteriaValue
*  
*  For simple criteria, note that $criteriaOperator will vary depending on field
*  type and the TextMatchStyle in force, as follows:
*  
* - Text fields with textMatchStyle "substring" - "iContains"
*  
* - Text fields with textMatchStyle "startsWith" - "iStartsWith"
*  
* - Text fields with textMatchStyle "exact" - "iEquals"
*  
* - All other cases (including text fields with textMatchStyle "exactCase") - "equals"
*  
*  You can use Velocity expressions in your customCriteriaExpressions, and the 
*  velocitySupport are available. Note that the results
*  of Velocity evaluations are automatically quoted and escaped by default; this behavior can
*  be switched off - see DataSourceField.autoQuoteCustomExpressions, and 
*  particularly note the warnings about its use.
*  
*  Note that this property only applies to users of the SmartClient server using dataSources of
*  type "sql".
* @type {VelocityExpression}
* @default null
*/
customCriteriaExpression?: VelocityExpression;

/**
* If explicitly set to false, switches off automatic quoting and escaping of values in
*  custom expressions that are derived from Velocity evaluations.
*  
*  By default, any part of a DataSourceField.customSelectExpression,
*  DataSourceField.customCriteriaExpression,
*  DataSourceField.customUpdateExpression or 
*  DataSourceField.customInsertExpression that comes ultimately from evaluating
*  a Velocity expression, will be automatically quoted and escaped according to the syntax 
*  of the underlying database. We do this because "raw" values are vulnerable to 
*  SQL injection attacks.
*  
*  However, it is possible that the exact SQL logic you desire for a custom expression cannot
*  be expressed without access to the raw, unquoted value. To work around this, you can either
*  use the special velocitySupport context variable, or you can set this
*  property to false to switch off auto-quoting completely for all custom expressions on this
*  field.
*  
*  Warning: Bear in mind that it is dangerous to use raw values. There are some
*  cases where using the raw value is necessary, but even so, all such cases are likely to be
*  vulnerable to injection attacks. Generally, the presence of $rawValue in a 
*  custom expression, or any fields with autoQuoteCustomExpressions: false 
*  specified, should be viewed as a red flag.
* @type {boolean}
* @default true
*/
autoQuoteCustomExpressions?: boolean;

/**
* Only applicable to "sql" dataSources. If set, this property indicates that the field is 
*  bound to the column named in the property. It is used to bind a DataSourceField to a 
*  database column with a different name.
*  
*  nativeName is useful for disambiguating in cases where you have customized 
*  SQL. For example, assume you are joining two tables - call them product and
*  supplier - that both contain a column called "name". Only one field in the 
*  DataSource can be called "name"; to disambiguate, we need to specify both 
*  DataSourceField.tableName and nativeName. A field declaration 
*  like this would be needed:
*    &lt;field name="supplierName" type="text" tableName="supplier" nativeName="name" />
*  
*  You may also wish to use this property to map legacy column names to more meaningful field
*  names in the DataSource. For example:
*    &lt;field name="productName" type="text" nativeName="PRDNM" />
*  
*  Note that it is not allowed to have multiple fields with the same nativeName 
*  bound to the same database table.
* @type {string}
* @default null
*/
nativeName?: string;

/**
* Only applicable to "sql" dataSources, setting tableName indicates that this
*  field will be coming from another table by way of a 
*  customQuerying.
*  
*  By default, the SQL generator will not include this field in generated SQL for "add"
*  or "update" operations. It will include the field in the SELECT clause and WHERE clause of
*  customized "fetch" operations, where it is expected that you will make the table available
*  via a custom &lt;tableClause&gt; as shown in ${isc.DocUtils.linkForExampleId('largeValueMapSQL', 'this sample')}.
*  
*  The table name will also be used to qualifying the column name unless
*  DataSource.qualifyColumnNames or OperationBinding.qualifyColumnNames has been
*  disabled.
*  
*  Note: This property is only required when including fields from tables other than the
*  default table specified by DataSource.tableName.
* @type {string}
* @default null
*/
tableName?: string;

/**
* When using DataSource.recordsAsText, determines what approach (if any) should be used
*  to force values to be intepreted as text instead of heuristically parsed as dates, times or
*  other structured types.
* @type {boolean}
* @default false
*/
exportForceText?: boolean;

/**
* Declares the FieldImportStrategy for this field. The default setting of null 
*  has the same meaning as "auto". Note, this attribute only has an effect if all of the 
*  following are true:
*  
* - The field declares a DataSourceField.foreignKey
*  
* - The field declares a DataSourceField.displayField
*  
* - The displayField so declared is DataSourceField.includeFrom the 
*  same DataSource that the foreignKey declaration refers to
*  
*  In any other circumstances, this attribute is ignored.
*  
*  importStrategy is applied by the server-side 
*  DataImport.importToRows() method, which is in turn used by
*  BatchUploader and several other DataImport APIs such as 
*  importToDataSource(), importDataSourceRecords() and 
*  importAndValidateDSRows(). See the server-side documentation for
*  details of these APIs. The actual data transformation is carried out by the 
*  server-side API DataSource.transformImportValue(); this method can 
*  be overridden to carry out any other transformation you may require.
* @type {FieldImportStrategy}
* @default null
*/
importStrategy?: FieldImportStrategy;

/**
* Customizable and i18n error message to use when "displayRequired"
*  DataSourceField.importStrategy fails to find corresponding key value.
*  Default message: Key mapping for display value is required, but was not found.
* @type {string}
* @default null
*/
importStrategyFailedErrorMessage?: string;

/**
* For certain field types, indicates the strategy to be used to store values to the underlying
*  SQL column.
*  
*  Fields of type "boolean"
*  The default strategy for boolean fields is to assume the underlying type of the field is 
*  text and store boolean true and false values as the character strings "true" and "false". 
*  The following additional strategies are available:
*  
*  
* - "number" or "integer": store true as 1, false as 0
*  
* - "singleChar10": store true as "1", false as "0"
*  
* - "singleCharYN": store true as "Y", false as "N"
*  
* - "singleCharTF": store true as "T", false as "F"
*  
* - "singleChar": store as a single character, using the values defined by 
*  DataSourceField.sqlTrueValue and 
*  DataSourceField.sqlFalseValue
*  
* - "string": store the entire values defined by 
*  DataSourceField.sqlTrueValue and 
*  DataSourceField.sqlFalseValue
*  
*  
*  Fields of type "date", "time", and "datetime"
*  The default strategy for these types is to use a native type that supports date and 
*  time values to the nearest second; the actual type used varies by database. The following
*  additional strategies are available:
*  
*  
* - "nativeDate": Use a native SQL DATE type
*  
* - "number" or "integer": Store the value as a plain numeric value
*  
* - "text": Store the value as a plain text value
*  
*  The "number" and "text" strategies for date/time/datetime fields are provided to allow transparent
*  two-way mapping of legacy date values; you would not ordinarily use them in the absence of a
*  legacy database. In both cases, the actual format of the number or text string is determined
*  by specifying a DataSourceField.sqlDateFormat on the field; if no 
*  format is specified, the framework defaults to "yyyyMMdd".
*  
*  
*  Fields of type "text"
*  NOTE: This section is only applicable for users of Microsoft SQL Server. The "ntext"
*  strategy is ignored for all other databases.
*  If you are integrating with Microsoft SQL Server, you may run into an issue with that database
*  and double-byte character sets. Although the documentation indicates that string values are 
*  sent to the database as Unicode by default, they may undergo conversion at the database end, 
*  depending on the collation settings of the database. 
*  This Microsoft support article describes
*  the situation, which is that Unicode is converted to the code page of the database before
*  being used, in order to "provide backward compatibility with existing applications".
*  
*  To work around this, we provide an sqlStorageStrategy of "ntext". Fields that
*  declare this strategy are inserted into queries using the "N syntax", like so:
*  &nbsp;&nbsp;&nbsp;UPDATE MyTable SET MyTextColumn = N'some text' WHERE MyPK = N'some key'
*  
*  This syntax overrides defaults and forces the database to treat strings as Unicode.
*  
*  Note that declaring a field to be of FieldType "ntext" is exactly 
*  equivalent to specifying type "text" and an sqlStorageStrategy of "ntext". 
*  If you declare a field of type "ntext" when the back-end database is something other than 
*  SQL Server, it will be treated as a normal field of type "text".
*  
*  This property only applies to users of the SmartClient server using dataSources of
*  type "sql".
* @type {string}
* @default null
*/
sqlStorageStrategy?: string;

/**
* For fields of type "date", "time", and "datetime" that specify an
*  DataSourceField.sqlStorageStrategy of "number" or "text", this
*  property allows you to specify the format SmartClient should use to map to/from the 
*  underlying value stored in the database. The string provided must be a valid 
*  
*  SimpleDateFormat pattern string.
*  
*  For fields of type "datetime" and "time" that specify an
*  DataSourceField.sqlStorageStrategy of "number", two special
*  sqlDateFormat values are supported:
*  
*   
* - "epoch" - The datetime or time value is stored as the number of full seconds that have
*     elapsed since January 1, 1970, 00:00:00 GMT.
*   
* - "epochms" - The datetime or time value is stored as the number of milliseconds that
*     have elapsed since January 1, 1970, 00:00:00 GMT. Millisecond precision is preserved
*     through database roundtrips with this setting
*  
*  
*  This property and the "number" and "text" strategies for date/datetime fields are provided
*  to allow transparent two-way mapping of legacy date values that are stored in numeric or
*  text columns respectively rather than a true "date" column. You would not ordinarily use them
*  in the absence of a legacy database.
*  
*  Note that this property only applies to users of SmartClient server and data sources of
*  type "sql".
* @type {string}
* @default null
*/
sqlDateFormat?: string;

/**
* For fields of type "datetime", indicates whether the framework should attempt to preserve
*  the millisecond part when persisting to native SQL datetime-type columns. Whether this 
*  is possible depends on the type and version of the underlying database product; some 
*  databases - for example SQL Server earlier than 2008 and MySQL versions earlier than 
*  5.6 - do not provide fractional seconds support. If you set this flag when your underlying 
*  database cannot support fractional seconds, it will be ignored.
*  
*  Other than the two cases just mentioned which are known to fail, storeMilliseconds 
*  has been tested and is officially supported with the following products and minimum 
*  versions:
*  
* - Oracle 10g
*  
* - Microsoft SQL Server 2008
*  
* - MySQL 5.6
*  
* - MariaDB 5.3
*  
* - PostgreSQL 9.1
*  
* - HSQLDB 2.2
*  
* - DB2 for Unix/Linux 9.7
*  
* - Firebird 11.2
*  
* - Informix IDS 11.5
*  
*  Note that you can use DataSourceField.sqlStorageStrategy to store a datetime 
*  value with sub-second precision in any database - see below.
*  
*  When the system generates SQL to insert or update datetime values including a millisecond
*  element, it does so using a database-specific format string specified in property 
*  defaultDateTimeFormatWithMilliseconds in your server.properties 
*  file. These format strings should conform to the rules of the
*  
*  Java SimpleDateFormat class. Suitable format strings are provided for all default 
*  databases out of the box; if you need to override one or provide a new one, you do so as
*  shown in this example:
*   sql.MyNewDatabase.defaultDateTimeFormatWithMilliseconds: yy-MM-dd'T'hh.mm.ss.SSS
*  
*  Note that this property only controls the persistence of milliseconds into the database;
*  there is no built-in support for sub-second elements of datetimes in any SmartClient visual
*  components. See also the documentation for DataSource.trimMilliseconds for more 
*  details of how sub-second datetime elements are handled in the client-server flow.
*  
*  This property is only applicable to DataSources of DSServerType 
*  "sql" and fields of FieldType "datetime". It is ignored in all other 
*  cases. However, note that the built-in JPA and Hibernate dataSources will always persist
*  millisecond values automatically if the underlying Java type, database and column support
*  it.
*  
*  Finally, note that this property only has an effect on native datetime-type fields.
*  When you use DataSourceField.sqlStorageStrategy to store a temporal value in a 
*  text or number column, it is the DataSourceField.sqlDateFormat you use that dictates 
*  the precision. For example, a field with sqlStorageStrategy:"text" and 
*  sqlDateFormat:"yyyy-MM-dd hh:mm:ss.SSS" would store and retrieve with millisecond
*  precsion; the same field with sqlDateFormat:"yyyy-MM-dd hh:mm" would store
*  and retrieve with precision to the nearest minute only.
* @type {boolean}
* @default null
*/
storeMilliseconds?: boolean;

/**
* For fields of type "boolean" that specify a 
*  DataSourceField.sqlStorageStrategy of "singleChar", this is the
*  value to persist for true values. Note that the common use cases of storing booleans as 
*  T/F, Y/N and 1/0 are already catered for without the need for this property.
*  For fields of type "boolean" that specify a 
*  DataSourceField.sqlStorageStrategy of "string", this is the value
*  to persist for true values that require using the entire value.
* @type {string}
* @default null
*/
sqlTrueValue?: string;

/**
* For fields of type "boolean" that specify a 
*  DataSourceField.sqlStorageStrategy of "singleChar", this is the
*  value to persist for false values. Note that the common use cases of storing booleans as 
*  T/F, Y/N and 1/0 are already catered for without the need for this property.
*  For fields of type "boolean" that specify a 
*  DataSourceField.sqlStorageStrategy of "string", this is the value
*  to persist for false values that require using the entire value.
*  
*  You can specify NULL as a valid value for sqlFalseValue, using this XML notation:
*  
*  &lt;sqlFalseValue xsi:nil="true"&gt;&lt;/sqlFalseValue&gt;
*  
* @type {string}
* @default null
*/
sqlFalseValue?: string;

/**
* If DataSource.noNullUpdates is set, the value to use for this field when a null
*  value is assigned to it on an update operation. This property overrides the DataSource-level
*  properties DataSource.nullStringValue, 
*  DataSource.nullIntegerValue, 
*  DataSource.nullFloatValue and 
*  DataSource.nullDateValue, for this field only.
*  
*  NOTE: If you use this property, you are responsible for ensuring that the replacement value
*  is appropriate and valid.
* @type {any}
* @default null
*/
nullReplacementValue?: any;

/**
* Server-side setting that causes values for fields of type "integer" or "float" to be represented 
*  as Strings when delivered to a web browser, in order to avoid mangling values which cannot be
*  represented faithfully in JavaScript.
*  
*  JavaScript has a single "Number" type which internally stores numbers in a format equivalent to
*  Java's "Double" - double-precision floating point. This means it cannot represent the full
*  range of Java's Long type: Java Longs can represent integral values between -2^63 to 2^63-1,
*  whereas JavaScript Number can only represent exact integer values between -2^53 and 2^53.
*  Similarly, JavaScript's Number type cannot represent Java's unlimited-size BigInteger or
*  unlimited-precision BigDecimal objects at all.
*  
*  The stringInBrowser setting can be used to deliver numeric values as Strings to the
*  browser. This is intended to allow read-only display and successful round-tripping of the
*  numeric value, however, it will not cause number-oriented features such as SpinnerItem,
*  Slider, ListGrid.showGridSummary or range-checking
*  Validator or AdvancedCriteria to actually work.
*  
*  If stringInBrowser is not set, the default behavior is configured by the 
*  server.properties setting datasource.defaultStringInBrowser. If 
*  this flag is false, numeric values are delivered to the client as numbers, even where 
*  this will lead to a loss of precision. If the flag is true (which is the the default),
*  the behavior is to prevent range overflow for numeric values:
*  
*  
* - Java values of type Long, BigInteger and BigDecimal will be delivered as
*    String only if they exceed JavaScript's number range. 
*  
* - Client-side validation will allow inputs that are outside of JavaScript's normal
*    integer range, and such numbers will remain as Strings after validation, instead of being
*    converted to Numbers
*  
* - Values submitted to the server will be converted to BigInteger if they exceed the range
*    of Java's Long type. This happens during DSRequest validation, before any defined
*    dmiOverview are called.
*  
*  
*  This default set of behaviors is intended to automatically deal with cases like numeric
*  DataSourceField.primaryKey or DataSourceField.foreignKey values that are never used as numbers, but happen to use
*  the full Java Long range. To disable the above behaviors, set stringInBrowser="false".
*  
*  If stringInBrowser is explicitly set to true:
*  
*  
* - the value for the field will always be delivered to the browser as a String (no
*    attempt is made to detect the value as out of range / too high precision)
*  
* - client-side validation will do a check for valid format only, and always leave the value
*    as a String. Numeric range validators are ignored client-side (always pass), but are
*    still active server-side.
*  
* - AdvancedCriteria OperatorId that check if values are in a particular
*    range will always pass
*  
* - the field is treated as though DataSourceField.canFilter were set, to avoid showing
*    non-functional search interfaces to the user. Set canFilter="true" to avoid
*    this effect
*  
*  
*  This setting is recommended for presenting out-of-range issues or precision loss on fields that
*  represent actual quantities (as opposed to primaryKey or foreignKey fields, which really
*  represent IDs). To ensure unlimited-precision BigDecimal values are preserved, you should also
*  set DataSourceField.javaClass.
*  
*  Note that responses delivered as JSON or XML, such as responses from the
*  servletDetails, are not affected. This setting applies only
*  to responses delivered to a web browser.
*  
*  The entirety of stringInBrowser processing can be completely disabled by setting 
*  server.properties flag datasource.disableStringInBrowser to true.
*  This will cause all numeric values to be delivered as numbers without even attempting to 
*  detect if this will lead to a loss of precision. This setting overrides both the 
*  stringInBrowser field setting and the 
*  datasource.defaultStringInBrowser server.properties flag.
*  
*  stringInBrowser and client-side DataSources
*  
*  For DataSources that are not based on the SmartClient Server, the client-side behaviors described above (such as leaving
*  user input in string form if precision would be lost) are active by default.
*  
*  In addition, if DataSource.dataFormat is used, values that would lose precision remain as strings. For JSON,
*  if behavior similar to stringInBrowser is desired, your server response must send the values as JSON strings
*  rather than JSON numeric literals.
*  
*  You can use DataSource.defaultStringInBrowser to disable these behaviors. NOTE: don't use this setting if
*  you are using the SmartClient Server, use the server.properties approach described above instead.
* @type {boolean}
* @default null
*/
stringInBrowser?: boolean;

/**
* Causes values for this field to be sorted according to values for another field, for
*  both client- and server-side sorting.
*  
*  This can be used to establish a sort order for a field that is not the normal sorting
*  order indicated by the field value, typically by having the sortByField as
*  a DataSourceField.hidden. 
*  
*  If using SQLDataSource, consider using a DataSourceField.customSelectExpression as an efficient
*  way to populate the sortByField with the results of a SQL expression.
* @type {string}
* @default null
*/
sortByField?: string;

/**
* NOTE: Only applicable to DataSource.clientOnly DataSources and the
*  built-in sqlDataSource, jpaIntegration and 
*  hibernateIntegration DataSources available in Pro, Power and 
*  Enterprise versions of SmartClient.
*  
*  Use this flag to inhibit the normal use of TextMatchStyle for this field. A 
*  field with this flag set will always be tested for exact equality in generated queries, 
*  even for filter-style queries where normal behavior would be to use a substring match or 
*  similar.
*  
*  Whether or not the exact match is case-sensitive is determined by the DataSource's 
*  DataSource.ignoreTextMatchStyleCaseSensitive
*  setting.
* @type {boolean}
* @default null
*/
ignoreTextMatchStyle?: boolean;

/**
* When true, this field can only be used for sorting if the data is entirely client-side.
* @type {boolean}
* @default false
*/
canSortClientOnly?: boolean;

/**
* If set, causes the field to be securely hashed before saving on an "add" or "update" 
*  operation.
* @type {HashAlgorithm}
* @default null
*/
storeWithHash?: HashAlgorithm;

/**
* For fields of a type that are represented by binary data initially uploaded from a file
*  (currently "binary" and "imageFile", see FieldType), this sets the maximum 
*  file size allowed, in bytes.
* @type {Integer}
* @default null
*/
maxFileSize?: Integer;

/**
* When set for a field with binary data, binary data will be delivered as a Base64 encoded
*  string. 
*  
*  When encodeInResponse is not set normal responses will not contain values for
*  binary fields at all. Instead, UI components, such as TileGrid issue a second
*  binary fetch for binary data when appropriate, and APIs such as DataSource.downloadFile and
*  DataSource.viewFile can be used to programmatically trigger downloads.
*  
*  Only applicable to fields of a type that are represented by binary data (currently "binary"
*  and "imageFile", see FieldType).
* @type {boolean}
* @default null
*/
encodeInResponse?: boolean;

/**
* Format string to use when rendering the value in any DataBoundComponent or when
*  exporting via DataSource.exportData or ListGrid.exportData or 
*  ListGrid.exportClientData.
*  
*  Supported for fields of type "date", "time", "datetime", "int", "float" or any derived
*  SimpleType. 
*  
*  To configure a different format for export, use DataSourceField.exportFormat.
*  
*  This is a per-field setting; you can alternatively set a default format for all "date",
*  "time" or "datetime" fields via 
*  
*  DateUtil.setNormalDatetimeDisplayFormat and related methods on Date.
*  
*  
*  See also localizedNumberFormatting for built-in FieldType
*  that handle localized currency formatting.
*  
*  Also note, this property takes precedence over any specified 
*  DataSourceField.dateFormatter, but can be overridden on a per-component
*  basis by providing a formatter directly on the component, for example, via
*  ListGrid.formatCellValue or FormItem.formatValue.
* @type {FormatString}
* @default null
*/
format?: FormatString;

/**
* An optional FormatString for this field, for use when 
*  DataBoundComponent.exportData data to spreadsheet formats (XLS and 
*  OOXML/XLSX), XML, JSON or CSV.  You can use this property to override the normal
*  DataSourceField.format of this field, if any, specifically for exports,
*  for both server-based and client-driven exports.
*  
*  
*  Note, for server-driven exports you can specify default formats for date, time and 
*  datetime fields by specifying properties export.format.default.date, 
*  export.format.default.time and export.format.default.datetime
*  in your server.properties file. Similarly, default formats for float and
*  integer fields can be specified with export.format.default.float and 
*  export.format.default.integer, respectively. These formats will be used for
*  fields that do not have a "format" or "exportFormat" property specified in the 
*  .ds.xml file.
*  
*  Specifically when exporting to spreadsheet formats, the FormatString is
*  translated to the type of format string used by spreadsheet programs like Excel. A handful
*  of features are not present in Excel format strings, and some features behave slightly
*  differently. These differences are explained below.
*  
*  Excel cannot handle dates prior to January 1st 1900
*  This is a well-known limitation of Excel dates; it is not a formatting issue as such.
*  
*  Currency symbols become fixed to the current locale at export time
*  The placeholder currency symbol "&#x00A4" (\u00A4) in a numeric
*  format string is rendered as the 
*  
*  NumberUtil.currencySymbol.
*  
*  
*  When exporting, the format string provided to Excel contains the currency symbol for the
*  current locale of the SmartClient application, and not a placeholder currency symbol that
*  would make Excel pick up a currency symbol based on the operating system locale. We do this
*  to ensure that the spreadsheet reflects the application's localization, rather than
*  the localization of the current user's computer, because they may be different.
*  
*  Rounding differences
*  The approach to rounding a positive number to a set number of decimal places is fairly 
*  universally agreed upon for non-specialized requirements: add 0.5 (or 0.05, or 0.005, or 
*  whatever) to the number and then truncate. This leads to the well understood convention 
*  that numbers exactly halfway between two possible rounding outcomes, go to the larger one. 
*  So 7.5 becomes 8 and 7.15 becomes 7.2.
*  
*  However, there is no such universal agreement when it come to rounding negative numbers. 
*  Some take the view that you should round by taking the number to the larger absolute value,
*  so -7.15 becomes -7.2. Others feel that you should round by taking the number to the larger
*  value in the sense of it being "less negative", so -7.15 becomes -7.1.
*  
*  SmartClient formatting takes the first approach and rounds negative numbers away from zero.
*  We do this simply because that is what Java DecimalFormat does. Unfortunately, Excel does
*  the opposite. Therefore, you will see rounding differences on negative numbers on exact 
*  50% boundaries: SmartClient will format -7.15 as -7.2, while Excel will format the same 
*  value as -7.1.
*  
*  Different treatment of '#'
*  Both SmartClient and Excel use '#' to mean "digit, do not show zeroes". However, Excel
*  does not implement this strictly in the integer part when the number it is formatting has
*  a value of 0 in the integer part. So, with a format of "#.##", the value 0.25 is formatted 
*  as "0.25". SmartClient (like Java DecimalFormat) is strict: with that format, 0.25 is 
*  formatted as ".25"; if you want to match Excel's output, you must use the format "0.##".
*  
*  Miscellaneous edge cases
*  There is quite a lot of variation in behavior between Java DecimalFormat and Excel around 
*  the edges. For actual error cases - for example, a format string that just contains 
*  nonsense - it is normal and expected that the SmartClient behavior and the Excel behavior
*  do not match - this is just two systems producing different garbage out for the same 
*  garbage in, which is to be expected. For valid but weird usages - for example, a format 
*  with multiple percent signs - SmartClient's formatting is in line with what DecimalFormat 
*  does, unless DecimalFormat throws an Exception, in which case we just do the thing that 
*  seems most sensible, or the thing that was easiest to implement.
*  
*  Known differences in behavior in edge cases include:
*  
* - SmartClient ignores formatting characters in the number part of the format string, 
*  whereas Excel rejects the format (this behavior may vary with different releases of Excel
*  and supposedly compatible libraries: POI, for example, will accept such format strings). 
*  If you attempt to format 5.175 with the format string "#b0.#a#", SmartClient will return
*  "5.72", and Excel will reject the format
*  
* - SmartClient ignores quoted characters in the number part of the format string, 
*  whereas Excel preserves them. If you attempt to format 5.175 with the format string 
*  "#'b'0.#'a'#", SmartClient will return "5.72", and Excel will return "b5.7a2"
*  
* - If you specify the empty string as a format string, SmartClient returns the result of 
*  calling toString() on the number; Excel uses the built-in "General" format. These two 
*  approaches will generally give the same or very similar results, but that is really a 
*  coincidence: the correct way to obtain matching results in the browser and the spreadsheet
*  is to specify a valid format string
*  
* - If you specify a format string that contains no number part (ie, no '#' or '0' 
*  characters), SmartClient does what DecimalFormat does, which is to output the integer part
*  of the number alongside the fixed part of the format. Excel just outputs the fixed part.
*  So, if you attempt to format -5.3 with the format string "'Hello world'", SmartClient will
*  output "-Hello world5", whereas Excel will output just "Hello world"
*  
* - If you specify multiple percent signs in the format, SmartClient treats all but the 
*  first one as static text, so 0.5 formatted with "#%%%" is "50%%%" (ie, all the signs are 
*  preserved but there is only one multiplcation by 100). Excel multiplies for each percent
*  sign, so 0.5 formatted with "#%%%" is "500000%%%"
*  
*  
*  Date format functionality not supported by Excel
*  The following date/time/datetime functionality is not supported by Excel; if you use 
*  formatters that use any of this functionality, your values will be exported to Excel 
*  incorrectly formatted. In cases like this, we recommend that you use a separate 
*  exportFormat, with the intent of exporting your values in a way that is 
*  similar to your application formatting (which would be specified with the 
*  DataSourceField.format property), but within the confines of what Excel 
*  supports.
*  
*  
* - Fiscal year, week and day (LL, LLLL, C, CC, c, cc)
*  
* - Week in year (w, ww)
*  
* - Day in year (D, DD)
*  
* - Day number in week (u)
*  
* - Explicit control over whether to use 12- or 24-hour notation. In Excel, this is 
*  implied by the presence or absence of the AM/PM designator
*  
* - If the user's operating system locale is different to the locale in use in the 
*  SmartClient application, day and month names may be different in the spreadsheet
*  
*  Number format functionality not supported by Excel
*  The only SmartClient number-formatting functionality not supported for 
*  export to Excel is "multiply by 1000 and show as per mille".
*  
*  Limit on number of custom Excel formats
*  Excel limits the number of custom format strings in a single spreadsheet to somewhere
*  between 200 and 250, depending on your locale and language. Hitting this limit in an export
*  would require hundreds of field definitions, each defining unique
*  FormatStrings. If you do hit the limit, the only workaround is to use fewer
*  unique FormatStrings.
*  
*  Controlling number format
*  If we give Excel a formatted number like "500,000" it will not treat that value as a number,
*  so sums and other basic spreadsheet features won't work. So we use the heuristic that if your
*  formatted value parses as a number, you probably want it treated as a number in the
*  spreadsheet, so we give Excel the unformatted numeric value and tell Excel it's a numeric
*  data value.
*  
*  You might expect that we would give Excel both the formatted value and the numeric
*  value, but this is only possible by creating what's called a "custom format" for that cell,
*  which as the section above mentions, is limited in that only a few hundred can be created.
*  
*  With this Excel limitation in mind, it makes sense to just go with the default behavior. If
*  you decide otherwise, one option is to use 
*  DSRequest.exportNumbersAsFormattedString, but see the
*  docs for that property for the drawbacks of doing this.
* @type {FormatString}
* @default null
*/
exportFormat?: FormatString;

/**
* When DataSourceField.includeFrom is specified and multiple records
*  exist in the related DataSource per record in the including DataSource,
*  includeSummaryFunction indicates which SummaryFunction is used to
*  produce the field value.
*  
*  This feature is available with Power or better licenses only. 
*  See smartclient.com/product for details.
*  
*  For example, given a DataSource "order" and related DataSource "orderItem", the "itemCount"
*  field below will show the total number of items in each order.
*  
*  &lt;DataSource ID="order"&gt;
*    &lt;fields&gt;
*     &lt;field name="orderID" type="integer" primaryKey="true" foreignKey="orderItem.orderID" /&gt;
*     &lt;field name="customerName" type="text" /&gt;
*     &lt;field name="orderDate" type="date" /&gt;
*     &lt;field name="itemsCount" includeFrom="orderItem.quantity" includeSummaryFunction="sum" /&gt;
*    &lt;/fields&gt;
*  &lt;/DataSource&gt;
*  
*  This is analogous to the following SQL statement: 
*  
*  SELECT
*    order.orderID,
*    order.customerName,
*    order.orderDate,
*    (select sum(_itemsCount.quantity)
*      from orderItem _itemsCount
*      where _itemsCount.orderID = order.orderID) as itemsCount
*  FROM
*    order
*  
*  The includeSummaryFunction feature supports indirect inclusion as well, when the field using
*  it is included from the other DataSource via regular includeFrom. For example, given the
*  "order" DataSource above and related "orderItem" DataSource, the "orderItemsCount" field below will show the
*  total number of items in the order each order item record belongs to:
*  
*  &lt;DataSource ID="orderItem"&gt;
*    &lt;fields&gt;
*     &lt;field name="id" type="integer" primaryKey="true" /&gt;
*     &lt;field name="orderID" type="integer" foreignKey="order.orderID"/&gt;
*     &lt;field name="quantity" type="integer" /&gt;
*     &lt;field name="unitPrice" type="float" /&gt;
*     &lt;field name="orderItemsCount" includeFrom="order.itemsCount" /&gt;
*    &lt;/fields&gt;
*  &lt;/DataSource&gt;
*  
*  This is analogous to the following SQL statement:
*  
*  SELECT
*    orderItem.id,
*    orderItem.orderID,
*    orderItem.quantity,
*    orderItem.unitPrice,
*    (select sum(_orderItemsCount_orderID.quantity) AS itemsCount
*      from order _orderItemsCount
*          join orderItem _orderItemsCount_orderID
*            on _orderItemsCount.orderID = _orderItemsCount_orderID.orderID
*      where orderItem.orderID = _orderItemsCount.orderID) as orderItemsCount
*  FROM
*    orderItem
*  
*  
*  Note that tables in sub-select will always use DataSourceField.relatedTableAlias if set or automatically
*  generated aliases. Automatic aliases are generated according to the rule: first table in possible chain of
*  relations being the name of the field sub-select is getting value for (with underscore "_" in front) and the
*  rest aliases are built up using foreign key field names in the chained relations leading to the target table.
*  This allows to avoid any conflicts with the tables/aliases from the main query. Like in the second example
*  table "orderItem" is used in both main query without alias and sub-select under
*  _orderItemsCount_orderID alias.
*  
*  Some other common uses:
*  
*  
* - using "max" to show the most recent "order" for a "customer"
*  
* - using "avg" to show the average order size for a "customer"
*  
* - using "concat" to show the names of all "salesReps" involved in an "order" (note:
*    "concat" has limited support - see SummaryFunction).
*  
*  
*  NOTE: includeSummaryFunction and
*  serverSummaries cannot be used in the same DSRequest.
*  If both configurations are present, Server Summaries settings always take priority.
* @type {SummaryFunction}
* @default null
*/
includeSummaryFunction?: SummaryFunction;

/**
* Defines prefix before concatenated values if field is used with 
*  serverSummaries feature and the 
*  SummaryFunction is "concat".
* @type {string}
* @default null
*/
joinPrefix?: string;

/**
* Defines the delimiter between concatenated values if field is used with 
*  serverSummaries feature and the 
*  SummaryFunction is "concat". The default 
*  value is ", ".
* @type {string}
* @default null
*/
joinString?: string;

/**
* Defines suffix after concatenated values if field is used with 
*  serverSummaries feature and the 
*  SummaryFunction is "concat".
* @type {string}
* @default null
*/
joinSuffix?: string;

/**
* Affects whether client dsRequests can request summaries for this field.
*  
*  Default setting of null means that the DataSource-wide default setting
*  DataSource.allowClientRequestedSummaries is used for this field.
* @type {boolean}
* @default null
*/
allowClientRequestedSummaries?: boolean;

/**
* Before we start editing this field in a DataBoundComponent, should we perform a deep clone 
*  of the underlying field value. See DataSource.deepCloneOnEdit for details of what 
*  this means.
*  
*  If this value is not explicitly set, it defaults first to the value of 
*  DataBoundComponent.deepCloneOnEdit, then to the value of DataSource.deepCloneOnEdit.
*  
*  Like the other deepCloneOnEdit settings, this flag only has an effect if you are 
*  editing a values object that contains nested objects or arrays, using 
*  Canvas.dataPaths.
* @type {boolean}
* @default null
*/
deepCloneOnEdit?: boolean;

/**
* Set of search-operators valid for this field. 
*  
*  If not specified, all operators that are valid for the field type are allowed.
* @type {Array<Partial<OperatorId>>}
* @default null
*/
validOperators?: Array<Partial<OperatorId>>;

/**
* The default search-operator for this field.
*  
*  If not specified, falls back to the default specified for the field's
*  SimpleType.defaultOperator.
* @type {OperatorId}
* @default null
*/
defaultOperator?: OperatorId;

/**
* This property limits where criteria for this field can be applied. By default, criteria can 
*  be applied either at the server, or at the ResultSet.useClientFiltering where 
*  possible, to avoid server round-trips.
*  
*  When this property is set to serverOnly, any change to criteria for this field 
*  causes cache-invalidation and a trip to the server.
* @type {FieldFilterMode}
* @default null
*/
filterOn?: FieldFilterMode;





}


/**
* Extensions to the Date class, including added static methods on the Date object, and
*  additional instance methods available on all date instances.
*/
interface Date {


/**
*    Copy the value of this date into a new Date() object for independent manipulation
* 
*/
duplicate(): void;

/**
*  Return the abbreviated (up to 3 chars) day of week name for this date (Mon, Tue, etc).
*  To modify the value returned by this method, set DateUtil.shortDayNames
* 
* 
* @param {Integer} Number of characters to return (Defaults to 3, can't be                 longer than 3)
*/
getShortDayName(length:Integer): string;

/**
*  Return the full day of week name for this date (Monday, Tuesday, etc).
*  To modify the value returned by this method, set DateUtil.dayNames
* 
* 
*/
getDayName(): string;

/**
*  Return the abbreviated name of the month for this date (Jan, Feb, etc)
*  To modify the value returned by this method,
*  set DateUtil.shortMonthNames
*  .
* 
* @param {Integer} Number of characters to return (Defaults to 3, can't be longer than 3)
*/
getShortMonthName(length:Integer): string;

/**
*  Return the full name of the month for this date (January, February, etc)
*  To modify the value returned by this method,
*  set DateUtil.shortMonthNames
*  .
* 
*/
getMonthName(): string;

/**
*    Return a 2 digit year for this date.
* 
*/
getShortYear(): string;

/**
*  Returns an integer containing the week number.
* 
*/
getWeek(): Integer;

/**
*  Returns the FiscalYear object appropriate for the the current date, according to the
*  FiscalCalendar.
* 
*/
getFiscalYear(): FiscalYear;

/**
*  Returns the fiscal week number of the current date, according to the global
*  DateUtil.setFiscalCalendar.
* 
* @param {FiscalCalendar=} the object representing the starts of fiscal years
*/
getFiscalWeek(fiscalCalendar?:FiscalCalendar): Integer;

/**
*  	Return this date in the format (UTC timezone):
*   YYYYMMDDTHHMMSS[Z]
* 
*/
toDateStamp(): string;

/**
*  Returns the date as a formatted string using the format set up via the
*  setNormalDisplayFormat() method. Note that the default formatter for this
*  method is "toLocaleString".
* 
* @param {DateDisplayFormat} Optional Format for the date returned
*/
toNormalDate(format:DateDisplayFormat): string;

/**
*  Returns the datetime as a formatted string using the format set up via the
*  setNormalDatetimeDisplayFormat() method.
* 
* @param {DateDisplayFormat} Optional Format for the date returned
* @param {boolean=} If a custom timezone has been set via  Time.setDefaultDisplayTimezone(), by default date formatters will respect this timezone.  To suppress this behavior, this parameter should be set to false.
*/
toNormalDatetime(format:DateDisplayFormat, useCustomTimezone?:boolean): string;

/**
*  Returns the date as a formatted string using the format set up via the
*  setShortDisplayFormat() method.
* 
* @param {DateDisplayFormat} Optional Format for the date returned
* @param {boolean=} If a custom timezone has been set via  Time.setDefaultDisplayTimezone(), by default date formatters will respect this timezone.  to suppress this behavior, this parameter should be set to false.
*/
toShortDate(format:DateDisplayFormat, useCustomTimezone?:boolean): string;

/**
*  Returns the datetime as a formatted string using the format set up via the
*  setShortDatetimeDisplayFormat() method.
* 
* @param {DateDisplayFormat} Optional Format for the date returned
* @param {boolean=} If a custom timezone has been set via  Time.setDefaultDisplayTimezone(), by default date formatters will respect this timezone.  to suppress this behavior, this parameter should be set to false.
*/
toShortDateTime(format:DateDisplayFormat, useCustomTimezone?:boolean): string;

/**
*  Sets a new default separator that will be used when formatting dates. By default, this
*  is a forward slash character: "/"
* 
* @param {string} separator to use in dates
*/
setDefaultDateSeparator(separator:string): void;

/**
*  gets the default date separator string
* 
*/
getDefaultDateSeparator(): string;

/**
*  	Return this date in the format: MM/DD/YYYY
* 
*/
toUSShortDate(): string;

/**
*  Return this date in the format: MM/DD/YYYY HH:MM
* 
* 
*/
toUSShortDateTime(): string;

/**
*  	Return this date in the format: DD/MM/YYYY
* 
*/
toEuropeanShortDate(): string;

/**
*  Return this date in the format: DD/MM/YYYY HH:MM.
* 
*/
toEuropeanShortDateTime(): string;

/**
*  Return the date in this format: YYYY/MM/DD
* 
*/
toJapanShortDate(): string;

/**
*  	Return this date in the format: YYYY/MM/DD HH:MM:SS
* 
*/
toJapanShortDateTime(): string;

/**
*  Return this date in 'serialized' format YYYY-MM-DD HH:MM:SS
* 
*/
toSerializeableDate(): string;

/**
*  Set the formatter for this date object to the method name passed in. After this call
*  wherever appropriate SmartClient components will use this formatter function to return
*  the date as a string.
* 
* @param {string} name of a date formatter method on this Date
* @deprecated * As of SmartClient 5.5 use the static methods       DateUtil.setNormalDisplayFormat and
*        DateUtil.setShortDisplayFormat to set default formatters for all dates

*/
setFormatter(functionName:string): void;

/**
*  	Return this date in the format: MM/DD/YY HH:MM
* 
* @deprecated * As of SmartClient 5.5 use Date.toShortDate instead

*/
toPrettyString(): string;



}


/**
* A JavaScript object specifying a range of dates. Values are DateRange.start
*  and DateRange.end. If either value is omitted, the range is assumed to be
*  open-ended in that direction - so if dateRange.start is omitted, the range will include any
*  date earlier than the value specified in dateRange.end.
*/
interface DateRange {
/**
* The start of this DateRange.
* @type {RelativeDate | Date}
* @default null
*/
start?: RelativeDate | Date;

/**
* The end of this DateRange.
* @type {RelativeDate | Date}
* @default null
*/
end?: RelativeDate | Date;





}


/**
* Identifies a potential branch within a MultiDecisionTask. Each decision
*  has a criteria and a target ProcessElement ID.
*/
interface DecisionBranch {
/**
* Criteria identifying when the DecisionBranch.targetTask should be chosen within a
*  MultiDecisionTask.decisionList.
*  
*  Data values in this criteria prefixed with "$" will be treated as dynamic expressions
*  as detailed in taskInputExpression. Specifically, this means that for 
*  simple criteria, any property value that is a String and is prefixed with "$" will be
*  assumed to be an expression, and for AdvancedCriteria, the same treatment will be
*  applied to Criterion.value.
*  
*  Note that dynamic expressions starting with "$input" are not applicable
*  in this context but "$inputRecord" can be used for direct reference to Process.state. 
*  
*  This property supports dynamicCriteria - use Criterion.valuePath
*  to refer to values in the Process.ruleScope.
* @type {Criteria}
* @default null
*/
criteria?: Criteria;

/**
* ProcessElement.ID of element to be used as next element if
*  DecisionBranch.criteria matches.
* @type {string}
* @default null
*/
targetTask?: string;





}


/**
* An object literal with a particular set of properties used to configure the display of
*  and interaction with the rows of a DetailViewer.
*/
interface DetailViewerField extends DBCField {
/**
* Name property used to identify the field, and determines which attribute from
*  records will be displayed in this field.
*  
*  Must be unique within the DetailViewer as well as a valid JavaScript identifier - see
*  FieldName for details and how to check for validity.
*  
*  The attribute of the records to display in this field may also be set by
*  DetailViewerField.displayField.
* @type {FieldName}
* @default null
*/
name?: FieldName;

/**
* If displayField is defined for the field then the DetailViewer will
*  display the displayField attribute of records instead of the attribute
*  given by the DetailViewerField.name of the field.
* @type {string}
* @default null
*/
displayField?: string;

/**
* dataPath property allows this field to display detail from nested data structures
* @type {string}
* @default null
*/
dataPath?: string;

/**
* Display format to use for date type values within this field. 
*  
*  The DetailViewerField.timeFormatter may also be used to format underlying Date values as
*  times (omitting the date part entirely). If both dateFormatter and
*  timeFormatter are specified on a field, for
*  fields specified as DetailViewerField.type the
*  timeFormatter will be used, otherwise the dateFormatter
*  
*  If field.dateFormatter and field.timeFormatter is unspecified,
*  date display format may be defined at the component level via
*  DetailViewer.dateFormatter, or for fields of type "datetime"
*  DetailViewer.datetimeFormatter. Otherwise the
*  default is to use the system-wide default normal date format, configured via
*  DateUtil.setNormalDisplayFormat. Specify any valid 
*  DateDisplayFormat to change the format used by this item.
* @type {DateDisplayFormat}
* @default null
*/
dateFormatter?: DateDisplayFormat;

/**
* Time-format to apply to date type values within this field. If specified, any
*  dates displayed in this field will be formatted as times using the appropriate format.
*  This is most commonly only applied to fields specified as type "time" though
*  if no explicit DetailViewerField.dateFormatter is specified it will be respected for other 
*  fields as well.
*  
*  If unspecified, a timeFormatter may be defined 
*  DetailViewer.timeFormatter and will be respected by fields
*  of type "time".
* @type {TimeDisplayFormat}
* @default null
*/
timeFormatter?: TimeDisplayFormat;

/**
* Applies only to fields of type "float" and affects how many significant digits are shown.
*  
*  For example, with decimalPrecision 3, if the field value is 343.672677, 343.673 is shown.
*  
*  If the value is 125.2, 125.2 is shown - decimalPrecision will not cause extra zeros to be
*  added. Use DataSourceField.decimalPad for this.
*  
*  A number is always shown with its original precision when edited.
* @type {number}
* @default null
*/
decimalPrecision?: number;

/**
* Applies only to fields of type "float" and enforces a minimum number of digits shown after
*  the decimal point.
*  
*  For example, a field value of 343.1, 343.104 and 343.09872677 would all be shown as 343.10
*  if decimalPad is 2.
*  
*  The original unpadded value is always shown when the value is edited.
* @type {number}
* @default null
*/
decimalPad?: number;

/**
* Size of images shown for fieldTypes image in this field.
*  
*  If set to a String, assumed to be a property on each record that specifies the image
*  height. For example, if field.imageSize is "logoSize",
*  record.logoSize will control the size of the image.
* @type {Integer | string}
* @default null
*/
imageSize?: Integer | string;

/**
* Width of images shown for fieldTypes image in this field.
*  
*  If set to a String, assumed to be a property on each record that specifies the image
*  width. For example, if field.imageWidth is "logoWidth",
*  record.logoWidth will control the width of the image.
* @type {Integer | string}
* @default null
*/
imageWidth?: Integer | string;

/**
* Height of image shown for fieldTypes image in this field.
*  
*  If set to a String, assumed to be a property on each record that specifies the image
*  height. For example, if field.imageHeight is "logoHeight",
*  record.logoHeight will control the height of the image.
* @type {Integer | string}
* @default null
*/
imageHeight?: Integer | string;

/**
* If this field has type set to "image"
*  and the URL for the image displayed is not absolute, the path of the URL will be relative
*  to this string.
* @type {string}
* @default null
*/
imageURLPrefix?: string;

/**
* If this field has type set to "image", the value
*  of this property will be appended to the end of the URL for the icon displayed.
* @type {string}
* @default null
*/
imageURLSuffix?: string;

/**
* The HTML to display for values of this field if the field type is set to "link". 
*  
*  This property sets linkText that will be the same for all records. You can set linkText
*  on a per-record basis via DetailViewerRecord.linkText.
* @type {string}
* @default null
*/
linkText?: string;

/**
* Name of the property in a DetailViewerRecord that holds the HTML to display for values
*  of this field if the field type is set to "link".
* @type {string}
* @default null
*/
linkTextProperty?: string;

/**
* If this field has type set to "link", setting this property will apply a
*  standard prefix to the link URL when displaying values of this field.
* @type {string}
* @default null
*/
linkURLPrefix?: string;

/**
* If this field has type set to "link", setting this property will apply a
*  standard suffix to the link URL when displaying values of this field.
* @type {string}
* @default null
*/
linkURLSuffix?: string;

/**
* By default, clicking a link rendered by this item opens it in a new browser window. You 
*  can alter this behavior by setting this property. The value of this property will be 
*  passed as the value to the target attribute of the anchor tag used to render 
*  the link. target is applicable only if the field type is set to "link".
* @type {string}
* @default "_blank"
*/
target?: string;

/**
* FormatString for numeric or date formatting. See DataSourceField.format.
* @type {FormatString}
* @default null
*/
format?: FormatString;

/**
* FormatString used during exports for numeric or date formatting. See
*  DataSourceField.exportFormat.
* @type {FormatString}
* @default null
*/
exportFormat?: FormatString;

/**
* Dictates whether the data in this field should be exported raw by
*  DetailViewer.exportClientData. If set to true for a
*  field, the values in the field-formatters will not be executed for data in this field.
* @type {boolean}
* @default null
*/
exportRawValues?: boolean;

/**
* Determines whether this field can be hilited. Set to false to prevent this
*  field from appearing in HiliteEditor.
* @type {boolean}
* @default null
*/
canHilite?: boolean;

/**
* When DetailViewer.hiliteIcons are present, where the hilite icon will be placed 
*  relative to the field value. See HiliteIconPosition.
*  Overrides DetailViewer.hiliteIconPosition
* @type {HiliteIconPosition}
* @default null
*/
hiliteIconPosition?: HiliteIconPosition;

/**
* Default width and height of DetailViewer.hiliteIcons in this field.
*  Takes precedence over hiliteIconWidth, hiliteIconHeight and hiliteIconSize specified at
*  the component level.
*  Can be overridden via DetailViewerField.hiliteIconWidth and DetailViewerField.hiliteIconHeight
* @type {number}
* @default null
*/
hiliteIconSize?: number;

/**
* Width for hilite icons for this field.
*  Overrides DetailViewer.hiliteIconSize, DetailViewer.hiliteIconWidth, and
*  DetailViewerField.hiliteIconSize.
* @type {number}
* @default null
*/
hiliteIconWidth?: number;

/**
* Height for hilite icons for this field.
*  Overrides DetailViewer.hiliteIconSize, DetailViewer.hiliteIconHeight, and
*  DetailViewerField.hiliteIconSize.
* @type {number}
* @default null
*/
hiliteIconHeight?: number;

/**
* How much padding should there be on the left of DetailViewer.hiliteIcons 
*  for this field?
*  Overrides DetailViewer.hiliteIconLeftPadding
* @type {number}
* @default null
*/
hiliteIconLeftPadding?: number;

/**
* How much padding should there be on the right of DetailViewer.hiliteIcons 
*  for this field?
*  Overrides DetailViewer.hiliteIconRightPadding
* @type {number}
* @default null
*/
hiliteIconRightPadding?: number;

/**
* Specifies the type of this DetailViewerField. By default (value is null)
*  the field shows a field title on the left and the field value on the right. There are
*  four special values for this attribute:
*  
*  
* - "header" - If you specify type "header", the field spans both the field name and
*  field value columns and contains text defined in the DetailViewerField.value
*  attribute with the style specified by DetailViewer.headerStyle. You can use this
*  field type as a titled separator.
*  
* - "separator" - If you specify type "separator", the field spans both the field name
*  and the field value columns with no text, and is styled using the style specified via
*  DetailViewer.separatorStyle. The height of the separator field can be controlled
*  via DetailViewerField.height.
*  
* - "image" For viewing, a thumbnail image is rendered in the field. The URL of the
*  image is the value of the field, and should be absolute. The size of the image is
*  controlled by DetailViewerField.imageSize,
*  DetailViewerField.imageWidth, DetailViewerField.imageHeight
*  
* - "link" For viewing, a clickable html link (using an HTML anchor tag: &lt;A&gt;)
*  is rendered in the field. The target URL is the value of the field, which is also the
*  default display value. You can override the display value by setting
*  DetailViewerRecord.linkText or DetailViewerField.linkText.
*  
*  Clicking the link opens the URL in a new window by default. To change this behavior,
*  you can set field.target, which works identically to the "target"
*  attribute on an HTML anchor (&lt;A&gt;) tag. See DetailViewerField.target for
*  more information.
*  
* @type {string}
* @default null
*/
type?: string;

/**
* The title of the field as displayed on the left-hand side. If left unspecified, the
*  title of the field is derived by looking up the value of
*  DetailViewer.fieldIdProperty on this field. So, by default, the title of a field
*  is the value of its "name" property.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* A property list (or an expression that evaluates to a property list)
*  specifying a mapping of internal values to display values for the field (row).
* @type {object}
* @default null
*/
valueMap?: object;

/**
* By default HTML values in DetailViewer cells will be interpreted by the browser.
*  Setting this flag to true will causes HTML characters to be escaped, meaning the
*  raw value of the field (for example "&lt;b&gt;AAA&lt;/b&gt;") is displayed
*  to the user rather than the interpreted HTML (for example "AAA")
* @type {boolean}
* @default null
*/
escapeHTML?: boolean;

/**
* When a field specifies its DetailViewerField.type to be "header", the value of
*  this attribute specifies the header text.
* @type {HTMLString}
* @default "undefined"
*/
value?: HTMLString;

/**
* For DetailViewerField.type: "separator", this attribute specifies
*  the height of the separator.
* @type {number}
* @default null
*/
height?: number;

/**
* If specified, cells in this field will be rendered using this css className rather than
*  DetailViewer.cellStyle
* @type {CSSStyleName}
* @default null
*/
cellStyle?: CSSStyleName;

/**
* If specified, when generating print HTML for this detailViewer, 
*  cells in this field will be rendered using this css className rather than
*  DetailViewer.printCellStyle
* @type {CSSStyleName}
* @default null
*/
printCellStyle?: CSSStyleName;

/**
* For a field of type:"imageFile", indicates whether to stream the image and display it
*  inline or to display the View and Download icons.
* @type {boolean}
* @default null
*/
showFileInline?: boolean;

/**
* Dictates whether the data in this field be exported. Explicitly set this
*  to false to prevent exporting. Has no effect if the underlying 
*  DataSourceField.canExport is explicitly set to 
*  canExport: false.
* @type {boolean}
* @default null
*/
canExport?: boolean;

/**
* Indicates this field's values come from another, related DataSource. 
*  The individual field will inherit settings such as DetailViewerField.type
*  and DetailViewerField.title from the related DataSource just like
*  fields from the primary DataSource.
* @type {string}
* @default null
*/
includeFrom?: string;

/**
* The value to display for a cell whose value is null or the empty
*  string after applying formatCellValue and valueMap (if any).
*  
*  This is the field-specific attribute. You may also set the emptyCellValue at the viewer
*  level to define the emptyCellValue for all empty fields in the viewer.
* @type {HTMLString}
* @default null
*/
emptyCellValue?: HTMLString;



/**
*  Optional method to format the value to display for this field's cells. Takes precedence 
*  over DetailViewer.formatCellValue for cells in this field.
* 
* @param {string} the raw value of the cell
* @param {DetailViewerRecord} the record being displayed
* @param {DetailViewerField} the field being displayed
* @param {DetailViewer} the detailViewer containing this field
*/
formatCellValue(value:string, record:DetailViewerRecord, field:DetailViewerField, viewer:DetailViewer): void;

/**
* 
*  If specified on a field, this method is evaluated at draw time to determine whether or
*  not to show this particular field.
*  
*  This method can be specified either as a function or a string that will be
*  auto-converted to a function.
* 
* 
* @param {DetailViewer} The DetailViewer
* @param {Array<Partial<DetailViewerRecord>>} */
showIf(viewer:DetailViewer, valueList:Array<Partial<DetailViewerRecord>>): boolean;

/**
*  Optional method to return the CSS class for cells in this field. If specified, this method
*  will be called from DetailViewer.getCellStyle, and should return a css class name.
* 
* 
* @param {string} actual value of this cell
* @param {object} field object for this cell
* @param {object} record object for this cell
* @param {DetailViewer} the viewer instance to which this cell belongs
*/
getCellStyle(value:string, field:object, record:object, viewer:DetailViewer): CSSStyleName;



}


/**
* A DetailViewerRecord is an object literal with properties that define the values for the
*  various fields of a DetailViewer.
*  
*  For example a DetailViewer that defines the following fields:
*  
*  fields : [
*    {name: "field1"},
*    {name: "field2"}
*  ],
*  
*  Might have the following data:
*  
*  data : [
*    {field1: "foo", field2: "bar"},
*    {field1: "field1 value", field2: "field2 value"}
*  ]
*  
*  Each element in the data array above is an instance of DetailViewerRecord - notice that
*  these are specified simply as object literals with properties.
*/
interface DetailViewerRecord {
/**
* The HTML to display in this record for fields with type set to "link". This overrides
*  DetailViewerField.linkText.
* @type {string}
* @default null
*/
linkText?: string;





}


/**
* Defines a set of properties that specify how the tree will be explored by Tree.discoverTree
*/
interface DiscoverTreeSettings {
/**
* When heuristically finding a property that appears to contain child objects, the childrenMode
*  determines how to chose the property that appears to contain child objects
* @type {ChildrenPropertyMode}
* @default "ChildrenPropertyMode.ANY"
*/
childrenMode?: ChildrenPropertyMode;

/**
* Determines how to scan for the Tree.childrenProperty
* @type {ScanMode}
* @default "ScanMode.BRANCH"
*/
scanMode?: ScanMode;

/**
* What to do if there is more than one possible Tree.childrenProperty
*  when using scanMode "branch" or "level"
* @type {TieMode}
* @default "TieMode.NODE"
*/
tieMode?: TieMode;

/**
* What to rename the array of children once discovered.
*  If not set, it will default to the value of Tree.childrenProperty inside discoverTree()
* @type {string}
* @default "null (see below)"
*/
newChildrenProperty?: string;

/**
* Each discovered child is labeled with a configurable "typeProperty" set to the value
*  of the property that held the children
* @type {string}
* @default null
*/
typeProperty?: string;

/**
* For string leaf nodes (if allowed), the name of the property to store the string under
*  in the auto-created object
* @type {string}
* @default null
*/
nameProperty?: string;





}


/**
* Holds the information of a drawing command.
*/
interface DrawShapeCommand {
/**
* The command type.
* @type {DrawShapeCommandType}
* @default null
*/
type?: DrawShapeCommandType;

/**
* The command arguments. The number of arguments and their types depend on this command's DrawShapeCommand.type.
* @type {Array<any>}
* @default null
*/
args?: Array<any>;





}


/**
* Returns information about how a data value is rendered in a chart.
*/
interface DrawnValue {
/**
* X coordinate where the data value is rendered. In pie mode, returns the X coordinate of the
*  center of the pie where the data value appears.
* @type {Integer}
* @default null
*/
x?: Integer;

/**
* Y coordinate where the data value is rendered. In pie mode, returns the Y coordinate of the
*  center of the pie where the data value appears.
* @type {Integer}
* @default null
*/
y?: Integer;

/**
* For pie mode only, start angle of the segment for the data value.
* @type {Integer}
* @default null
*/
startAngle?: Integer;

/**
* For pie mode only, start angle of the segment for the data value.
* @type {Integer}
* @default null
*/
endAngle?: Integer;

/**
* For pie mode only, the radius of the segment for the data value.
* @type {Double}
* @default null
*/
radius?: Double;

/**
* For bar and column charts, thickness of the bar representing this data value.
* @type {Integer}
* @default null
*/
barThickness?: Integer;

/**
* Data value this drawnValue represents.
* @type {Float}
* @default null
*/
value?: Float;

/**
* FacetValues for the data value.
* @type {FacetValueMap}
* @default null
*/
facetValues?: FacetValueMap;

/**
* The data record of the data point from which this drawnValue was created.
*  
*  Note that a chart with an Facet.inlinedValues or a
*  FacetChart.extraAxisMetrics may define multiple data points in
*  the same record, each of which will correspond to a different drawnValue.
*  The way to uniquely identify the data value of this particular drawnValue
*  is to use the DrawnValue.facetValues.
* @type {CellRecord}
* @default null
*/
record?: CellRecord;





}


/**
* An object representing a component that is currently being edited within an
*  EditContext.
*  
*  An EditNode is essentially a copy of a PaletteNode, initially with the same properties
*  as the PaletteNode from which it was generated. However unlike a PaletteNode, an EditNode 
*  always has a EditNode.liveObject - the object created from the 
*  PaletteNode.defaults or other properties defined on a paletteNode.
*  
*  Like a Palette, an EditContext may use properties such as PaletteNode.icon or 
*  PaletteNode.title to display EditNodes.
*  
*  An EditContext generally offers some means of editing EditNodes and, as edits are made,
*  updates EditNode.defaults with the information required to re-create the component.
*/
interface EditNode extends PaletteNode {
/**
* Properties required to recreate the current EditNode.liveObject.
* @type {Properties}
* @default null
*/
defaults?: Properties;

/**
* Properties to be applied to the
*  EditNode.liveObject.Canvas.editProxy when created.
*  
*  Note that the editProxy is created the first time a component is placed into
*  editMode, so any editProxyProperties must be set before then.
* @type {Partial<EditProxy>}
* @default null
*/
editProxyProperties?: Partial<EditProxy>;

/**
* SCClassName of the EditNode.liveObject
*  , for example, "ListGrid".
* @type {SCClassName}
* @default null
*/
type?: SCClassName;

/**
* Live version of the object created from the EditNode.defaults. For example, 
*  if EditNode.type is "ListGrid", liveObject would be a ListGrid.
* @type {object}
* @default null
*/
liveObject?: object;

/**
* Shortcut property to be applied to the
*  EditNode.liveObject.Canvas.editProxy when created.
* @type {boolean}
* @default null
*/
useEditMask?: boolean;

/**
* See PaletteNode.canDuplicate.
* @type {boolean}
* @default null
*/
canDuplicate?: boolean;





}


/**
* Configuration object for AutoTest.waitForElement
*/
interface ElementWaitConfig extends SystemWaitConfig {
/**
* Timeout for AutoTest.waitForElement in ms. If this is exceeded the callback will be fired 
*  with element set to null, and the callback will be passed false as 
*  its "done" parameter.
*  
*  Note that AutoTest.getElement can return a valid element but this method can still 
*  timeout if the element is not clickable or the system is busy.
*  
*  If unset a default timeout of 30 seconds will be used.
* @type {number}
* @default null
*/
timeout?: number;

/**
* This attribute configures how waitForElement() determines 
*  that the application is read to retrieve the element.
*  See ElementWaitStyle for options.
*  
*  If not explicitly set, the AutoTest.defaultElementWaitStyle will be used.
* @type {ElementWaitStyle}
* @default null
*/
waitStyle?: ElementWaitStyle;





}


/**
* Facet value definition object made use of by the CubeGrid and
*  FacetChart classes (contained by facets).
*/
interface FacetValue {
/**
* id of this facetValue. Any string or number.
* @type {string | number}
* @default null
*/
id?: string | number;

/**
* User-visible title of this facetValue. Shown on the field header.
* @type {string}
* @default null
*/
title?: string;

/**
* For tree facets (Facet.isTree), id of this facetValue's
*  parent facetValue.
*  
*  Note: This property is specific to CubeGrid.
* @type {string | number}
* @default null
*/
parentId?: string | number;

/**
* Whether cells for this facetValue can be edited. Defaults to CubeGrid.canEdit.
*  
*  Note: This property is specific to CubeGrid.
* @type {boolean}
* @default "cubeGrid.canEdit"
*/
canEdit?: boolean;

/**
* For tree facets, initial collapse state for this node. Defaults to
*  Facet.collapsed.
*  
*  Note: This property is specific to CubeGrid.
* @type {boolean}
* @default "facet.collapsed"
*/
collapsed?: boolean;

/**
* Initial CubeGrid.canMinimizeFacets for this node. Defaults to
*  Facet.minimized.
*  
*  Note: This property is specific to CubeGrid.
* @type {boolean}
* @default "facet.minimized"
*/
minimized?: boolean;

/**
* For individual parent facetValues within a hierarchical facet, this flag controls
*  whether an expand/collapse control will be shown.
*  
*  Note: This property is specific to CubeGrid (see, for
*  example, CubeGrid.canCollapseFacets).
* @type {boolean}
* @default true
*/
canCollapse?: boolean;

/**
* If CubeGrid.canMinimizeFacets is enabled, whether this facet
*  value should show controls to minimize the next facet. Generally a tree facet should not
*  also allow minimizing the next facet the interaction of the two types of collapsing can
*  be confusing.
*  
*  Note: This property is specific to CubeGrid (see, for
*  example, CubeGrid.canMinimizeFacets).
* @type {boolean}
* @default true
*/
canMinimize?: boolean;

/**
* Used to determine which facetValue is to be shown when the facet is minimized.
*  
*  Note: This property is specific to CubeGrid. See
*  CubeGrid.canMinimizeFacets.
* @type {boolean}
* @default false
*/
isMinimizeValue?: boolean;

/**
* Width of the cube grid facetValue in pixels.
*  
*  Note: This property is specific to CubeGrid.
* @type {number}
* @default "facet.width"
*/
width?: number;

/**
* Default alignment for facet label title and cells for this facetValue.
*  Can be overridden by setting titleAlign or cellAlign on the facetValue.
*  
*  Note: This property is specific to CubeGrid (see, for
*  example, CubeGrid.facetValueAlign).
* @type {Alignment}
* @default null
*/
align?: Alignment;

/**
* Default alignment of cells (in the body) for this facetValue.
*  
*  Note: This property is specific to CubeGrid (see, for
*  example, CubeGrid.cellAlign).
* @type {Alignment}
* @default "facet.cellAlign"
*/
cellAlign?: Alignment;

/**
* CSS line style to apply as a border before this facetValue.
*  
*  Note: This property is specific to CubeGrid.
* @type {string}
* @default null
*/
borderBefore?: string;

/**
* CSS line style to apply as a border after this facetValue.
*  
*  Note: This property is specific to CubeGrid.
* @type {string}
* @default null
*/
borderAfter?: string;

/**
* Selection boundary determining what facets / facetValues can be selected together by
*  drag selection / shift+click selection.
*  
*  Note: This property is specific to CubeGrid.
* @type {SelectionBoundary}
* @default null
*/
selectionBoundary?: SelectionBoundary;

/**
* Hilite style to apply to the title for this facetValue.
*  
*  Note: This property is specific to CubeGrid. See
*  CubeGrid.hilites.
* @type {string}
* @default null
*/
titleHilite?: string;



/**
*  Callout to determine custom value to display for cells displayed for this facetValue.
*  
*  Can only be set on the CubeGrid.metricFacetId or, if no metric facet
*  is specified, on the innermost column facet.
*  
*  Note: This method is specific to CubeGrid.
* 
* 
* @param {CubeGrid} this facetValue's CubeGrid
* @param {object} cell record
* @param {number} row value for the cell
* @param {number} column value for the cell
*/
getCellValue(viewer:CubeGrid, record:object, rowNum:number, colNum:number): string;

/**
*  Formatter to apply to values displayed for cells under this facetValue.
*  
*  Can only be set on the CubeGrid.metricFacetId or, if no metric facet
*  is specified, on the innermost column facet.
*  
*  If a single, grid-wide formatting style is desired, implement
*  ListGrid.formatCellValue instead.
*  
*  Note: This method is specific to CubeGrid.
* 
* 
* @param {any} raw value for the cell being
* @param {CellRecord} record object for the cell.  Note: If this is a new cell that has not been saved, in an   editable grid, it has no associated record object. In this case the edit values will  be passed in as this parameter.
* @param {number} row number for the cell
* @param {number} column number for the cell.
*/
formatCellValue(value:any, record:CellRecord, rowNum:number, colNum:number): string;



}


/**
* 
*  A JavaScript Object where each property name is a facetId and each property value is a
*  facetValueId for that facet.
*  
*  
*  
*  The facetId &rarr; facetValueId mappings in a FacetValueMap describe a specific slice of the
*  dataset. If mappings are included for all facets, a FacetValueMap describes a unique
*  cell. If some facets are omitted, it describes a row, column, or set of rectangular
*  areas, or equivalently, a particular row or column header (if all facetIds in the map are
*  displayed on the same axis)
*  
*  FacetValueMaps are used in various contexts to describe headers, datasets to be loaded,
*  screen regions, etc.
*/
interface FacetValueMap {




}


/**
* This class enables background (deferred) loading and caching of JS, CSS and Image files. It is
*  designed to work standalone from the rest of the SmartClient framework to provide a lightweight
*  caching and loading mechanism for SmartClient modules as well as user-built application
*  modules/fragments.
*  
*  The most common usage scenarios are:
*  
*   
* - Caching JS, CSS, Image files in the browser in anticipation of a transition to a page
*   that requires these files. For example, a plain HTML (non-SmartClient) login page or
*   landing page can begin caching SmartClient in the background while allowing the user
*   to login, or giving the user something to read. Normally, loading SmartClient or other
*   large JavaScript files would block page loading and display. By loading SmartClient in
*   the background only after a simple HTML landing page has loaded, you can completely
*   eliminate perceived download time associated with loading a rich UI application, making a
*   much larger difference in user experience than any difference in framework/application size.
*   
* - Loading a multi-phase UI. In this scenario, an initial rendering of a page is done with
*   minimal data transfer to the browser. Then JS, CSS, and Image files are fetched in the
*   background to provide richer UI components. During this time the user can continue to normally
*   interact with the initial page. Once loading is complete, the UI is updated with richer
*   components.
*  
* 
*  The recommended usage pattern is to use the loadISC custom tag provided as part of
*  the SmartClient SDK. You can specify cacheOnly="true" to loadISC to cache the
*  SmartClient framework in the background or alternately defer="true" to load the
*  SmartClient framework and make it available in the current page. You can specify the
*  onload attribute of the tag to provide a JavaScript callback to your code that will
*  be called when the framework loading is complete.
*  
*  If you're not working in a JSP environment, you can use the
*  FileLoader.cache/FileLoader.load APIs to accomplish
*  the same effect as the loadISC JSP tag.
*  
*  Additional APIs are provided for performing dynamic caching and loading of other JS, CSS, and
*  Image files to improve the performance of your application. See below.
*  
*  You must set window.isomorphicDir before loading and using this module unless the
*  default of '../isomorphic/' is acceptable. E.g. if your html file is in your toplevel webroot
*  directory, then your HTML file should say (note the trailing slash):
*  
*  &lt;SCRIPT&gt;window.isomorphicDir='isomorphic/'&lt;/SCRIPT&gt;
*  &lt;SCRIPT SRC=isomorphic/system/modules/ISC_FileLoader.js&gt;&lt;/SCRIPT&gt;
*  
*  In addition, if you are using Smart GWT, you must set FileLoader.modulesDir to
*  "modules/", as follows:
*  
*   isc.FileLoader.modulesDir = "modules/";
*  
*  
*  This module is usable independent of the rest of SmartClient - you can use it on pages that
*  don't load any other modules. In practice, the general pattern is to use this module on
*  static HTML pages such as a login page to pre-cache SmartClient modules, application logic,
*  skin files, and css so that once the user logs in, there's no latency to load the rich UI.
*  
*  You can also load the FileLoader itself dynamically - see FileLoader.ensureLoaded
*  
*  Note: You can also reference this class via the alias isc.FL
*/
interface FileLoader {




}


/**
* A record which specifies files for use with fileSource.
*/
interface FileSpec {
/**
* The name of the file, without any extension to indicate FileSpec.fileType or
*  FileSpec.fileFormat.
* @type {string}
* @default null
*/
fileName?: string;

/**
* The type of the file, e.g. "ds" for datasource, or "proj" for project.
* @type {string}
* @default null
*/
fileType?: string;

/**
* The format of the file, e.g. "xml" or "js"
* @type {string}
* @default null
*/
fileFormat?: string;





}


/**
* An object representing the start date for fiscal years in the current locale.
*  
*  A fiscal year spans a configurable date range - it may not exactly
*  match a calendar year in length and it can start on any date within the calendar year
*  and potentially end in the next calendar year.
*  
*  Developers may specify explicit fiscal year start dates by adding FiscalYear
*  objects to the FiscalCalendar.fiscalYears.
*  If none are provided, or if there is no entry for the given year, one is
*  manufactured based on the default FiscalCalendar.defaultMonth
*  and FiscalCalendar.defaultDate.
*/
interface FiscalCalendar {
/**
* The default zero-based month-number to use for calculating fiscal dates when no
*  FiscalCalendar.fiscalYears are provided. This value together
*  with FiscalCalendar.defaultDate will be used as the start date for the
*  fiscal years where no explicitly specified fiscalYear configuration is present.
*  
*  See also FiscalCalendar.defaultYearMode.
* @type {Integer}
* @default null
*/
defaultMonth?: Integer;

/**
* The default one-based day-number in the FiscalCalendar.defaultMonth
*  to use for calculating fiscal dates when no FiscalCalendar.fiscalYears
*  are provided. This value together
*  with FiscalCalendar.defaultMonth will be used as the start date for the
*  fiscal years where no explicitly specified fiscalYear configuration is present.
*  
*  See also FiscalCalendar.defaultYearMode.
* @type {Integer}
* @default null
*/
defaultDate?: Integer;

/**
* This attribute controls how the displayed fiscalYear value should be calculated for
*  dates falling within a period not explicitly listed in the
*  FiscalCalendar.fiscalYears.
*  
*  The FiscalCalendar.defaultMonth and FiscalCalendar.defaultDate will be
*  used to calculate the start of the fiscal year period. The defaultYearMode
*  determines whether the reported fiscalYear for this period matches the year in which
*  the period starts or the year in which it ends (so whether a fiscal year spanning
*  dates within both 2020 and 2021 is reported as fiscalYear 2020 or 2021).
* @type {FiscalYearMode}
* @default "end"
*/
defaultYearMode?: FiscalYearMode;

/**
* An array of FiscalYear which each represent the start date of a
*  single fiscal year.
* @type {Array<Partial<FiscalYear>>}
* @default null
*/
fiscalYears?: Array<Partial<FiscalYear>>;





}


/**
* An object representing the start of a given Fiscal Year in the current locale.
*  
*  See FiscalCalendar for more information on how FiscalYears are set up and used.
*/
interface FiscalYear {
/**
* The actual fiscal year that this date relates to.
*  
*  A fiscal year ends when the next one begins. A fiscal year may span the boundary
*  between two calendar years in which case the FiscalYear.fiscalYear value may
*  not match the FiscalYear.year value.
*  
*  For example fiscalYear 2020 may start in July of 2019 and end in July of 2020. In this
*  case the fiscalYear would be set to 2020 and the
*  FiscalYear.year would be set to 2019
* @type {Integer}
* @default null
*/
fiscalYear?: Integer;

/**
* The 4-digit calendar year when this fiscal year starts.
* @type {Integer}
* @default null
*/
year?: Integer;

/**
* The zero-based month-number when this fiscal year starts.
* @type {Integer}
* @default null
*/
month?: Integer;

/**
* The one-based day-number in the FiscalYear.month when this fiscal
*  year starts.
* @type {Integer}
* @default null
*/
date?: Integer;





}


/**
* An object containing details for mouse events occurring over a FormItem.
*/
interface FormItemEventInfo {
/**
* Item over which the event occurred.
* @type {FormItem}
* @default null
*/
item?: FormItem;

/**
* True if the event occurred within the item's FormItem.textBoxStyle.
* @type {boolean}
* @default null
*/
overTextBox?: boolean;

/**
* True if the event occurred over the item's data or input element. Note that it can be bad 
*  practice to implement custom context menus when overElement is true, since this will
*  replace browser-default menus that users might expect.
* @type {boolean}
* @default null
*/
overElement?: boolean;

/**
* True if the event occurred over the main body of the item (for example the text-box), rather
*  than over the title or within the form item's cell in the DynamicForm but outside the
*  text box area.
* @type {boolean}
* @default null
*/
overItem?: boolean;

/**
* True if the event occurred over the item's title.
* @type {boolean}
* @default null
*/
overTitle?: boolean;

/**
* True if the event occurred over the form's 
*  DynamicForm.errorItemProperties.
* @type {boolean}
* @default null
*/
overInlineError?: boolean;

/**
* If this event occurred over a formItemIcon this attribute contains the 
*  FormItemIcon.name for the icon.
* @type {string}
* @default null
*/
icon?: string;





}


/**
* Form item icon descriptor objects used by Form Items to specify the appearance and
*  behavior of icons displayed after the item in the page flow.
*/
interface FormItemIcon {
/**
* Base CSS style. If set, as the component changes state and/or is focused, suffixes
*  will be added to the base style. Possible suffixes include "Over" if the user mouses
*  over the icon and FormItemIcon.showOver is true, "Disabled" if
*  the icon is disabled, and "Focused". In addition, if FormItemIcon.showRTL
*  is enabled, then an "RTL" suffix will be added.
* @type {CSSStyleName}
* @default null
*/
baseStyle?: CSSStyleName;

/**
* Identifier for this form item icon. This identifier (if set) should be unique
*  within this form item and may be used to get a pointer to the icon object
*  via FormItem.getIcon.
* @type {Identifier}
* @default null
*/
name?: Identifier;

/**
* When set, this icon is rendered inside the FormItem.textBoxStyle area
*  of the FormItem (where input occurs in a TextItem, TextAreaItem or
*  ComboBoxItem) as opposed to as a trailing icon.
*  
*  Use FormItemIcon.inlineIconAlign to control alignment of the
*  icon. Multiple icons can be inlined on both the left and right side of the
*  textBox area. FormItemIcon.hspace is honored for spacing
*  between multiple adjacent icons.
*  
*  Inline icons are not supported in Internet Explorer 6, or when the FormItem
*  is not a TextItem, TextAreaItem or ComboBoxItem.
*  When unsupported, the icon will fall back to non-inline mode.
*  
*  The FormItem.showPickerIcon, if any, cannot be inlined.
*  
*  As an alternative to displaying an image, an inline icon may display a string of
*  HTML instead. See FormItemIcon.text.
* @type {boolean}
* @default false
*/
inline?: boolean;

/**
* Horizontal alignment for icons marked FormItemIcon.inline.
*  
*  By default, the first icon that specifies inline is aligned left, and the second and all
*  subsequent icons to the right. "center" alignment is invalid and will be
*  ignored.
*  
*  In RTL mode, the alignment is automatically mirrored; inlineIconAlign:"left"
*  results in the icon being placed on the right and inlineIconAlign:"right"
*  results in the icon being placed on the left.
* @type {Alignment}
* @default null
*/
inlineIconAlign?: Alignment;

/**
* If set, this property determines this icon's image source.
*  If unset the form item's defaultIconSrc property will be used
*  instead.
*  As with defaultIconSrc this URL will be modified by adding
*  "_Over" or "_Disabled" if appropriate to show the icon's over or disabled state.
*  If FormItemIcon.showRTL is enabled, then "_rtl" will be added to the
*  source URL before the extension.
*  
*  The special value "blank" means that no image will be shown for this icon. This
*  is particularly useful together with FormItemIcon.baseStyle to implement
*  spriting of the different icon states.
*  
*  For an FormItemIcon.inline FormItemIcon,
*  FormItemIcon.text may be specified to show a string of HTML instead of
*  an image.
*  
*  skinning can be used for this image, by setting this property to
*  a SCSpriteConfig formatted string.
* @type {SCImgURL | SCStatefulImgConfig}
* @default null
*/
src?: SCImgURL | SCStatefulImgConfig;

/**
* As an alternative to displaying an image, an FormItemIcon.inline
*  FormItemIcon can display a string of HTML where the icon's image would
*  have appeared. This enables advanced customizations such as using text, icon font symbols,
*  Unicode dingbats and emoji, and/or SVG in place of an image.
*  
*  Setting an inline icon's text property will cause the HTML to be used instead of an
*  image, as long as the browser and form item support inline icons.
*  
*  This property only has an effect on inline icons. If the inline property is false,
*  or the browser or form item does not support inline icons, then the image specified
*  by FormItemIcon.src (or the form item's FormItem.defaultIconSrc)
*  will be used.
*  
*  Typically, the HTML is styled via FormItemIcon.baseStyle.
*  
*  Auto-sizing of the HTML is not supported; the HTML will be clipped to the icon's
*  FormItemIcon.width and FormItemIcon.height.
* @type {HTMLString}
* @default null
*/
text?: HTMLString;

/**
* Specifies the cursor image to display when the mouse pointer is
*  over this icon. It corresponds to the CSS cursor attribute. See Cursor type for
*  different cursors.
*  
*  See also FormItemIcon.disabledCursor.
* @type {Cursor}
* @default "pointer"
*/
cursor?: Cursor;

/**
* Specifies the cursor image to display when the mouse pointer is
*  over this icon if this icon is disabled. It corresponds to the CSS cursor
*  attribute. See Cursor type for different cursors.
* @type {Cursor}
* @default "default"
*/
disabledCursor?: Cursor;

/**
* Should this icon's image and/or FormItemIcon.baseStyle switch to the
*  appropriate "Over" value when the user rolls over or focuses on the icon?
*  
*  Note if FormItem.showOver is true and FormItemIcon.showOverWhen
*  is set to "textBox", this icon will show over state when the user rolls over the
*  text box (or control table, if visible) for the item. This is most commonly used
*  for FormItemIcon.inline icons.
* @type {boolean}
* @default null
*/
showOver?: boolean;

/**
* If FormItemIcon.showOver or FormItem.showOverIcons is true,
*  this property may be set to customize when the 'over' styling is applied to
*  the item. If unset, rollover styling will be applied when the user is over
*  the icon only.
* @type {IconOverTrigger}
* @default null
*/
showOverWhen?: IconOverTrigger;

/**
* Should this icon's image and/or FormItemIcon.baseStyle switch to the
*  appropriate "Focused" value when the user puts focus on the form item or icon?
* @type {boolean}
* @default null
*/
showFocused?: boolean;

/**
* Should this icon's FormItemIcon.src and/or FormItemIcon.baseStyle
*  switch to the appropriate RTL value when the FormItem is in RTL mode? If true, then
*  the image URL for all states will have "_rtl" added before the extension. Also, if
*  baseStyle is set, all style names will have an "RTL" suffix. This should only be
*  enabled if RTL media is available.
*  
*  For example, if an icon's src is "[SKINIMG]formItemIcons/myFormIcon.png" and the baseStyle
*  is "myFormIcon", then in the "Down" state, SmartClient will use "[SKINIMG]formItemIcons/myFormIcon_Down_rtl.png"
*  for the image source and "myFormIconDownRTL" for the style name.
* @type {boolean}
* @default null
*/
showRTL?: boolean;

/**
* If this icon will be updated to show focus (see FormItemIcon.showFocused, 
*  FormItem.showFocusedIcons), this property governs whether the focused state should
*  be shown when the item as a whole receives focus or just if the icon receives focus.
*  If this property is unset, default behavior is to show focused state when the item
*  receives focus.
* @type {boolean}
* @default null
*/
showFocusedWithItem?: boolean;

/**
* Set to false to suppress all focus features for this icon. Clicking the
*  icon will not apply focus to the icon or to the form item.
* @type {boolean}
* @default null
*/
canFocus?: boolean;

/**
* If icon.neverDisable is true, when this form item is disabled, the 
*  icon will remain enabled. 
*  Note that disabling the entire form will disable all items, together with their 
*  icons including those marked as neverDisable - this property only has an effect 
*  if the form is enabled and a specific item is disabled within it.
*  
*  If this property is true, the icons will also remain enabled if a form item 
*  is marked as FormItem.canEdit. For finer grained control over
*  whether icons are enabled for read-only icons see FormItem.disableIconsOnReadOnly
*  and FormItemIcon.disableOnReadOnly
* @type {boolean}
* @default null
*/
neverDisable?: boolean;

/**
* If FormItem.canEdit is set to false, should this icon be disabled.
*  If unset this is determined by FormItem.disableIconsOnReadOnly.
*  Note that if FormItemIcon.neverDisable is set to true, the icons will
*  be rendered enabled regardless of this setting and whether the item is editable.
* @type {boolean}
* @default null
*/
disableOnReadOnly?: boolean;

/**
* Whether this icon is disabled. Can be updated at runtime via the FormItem.setIconDisabled
*  method. Note that if the formItem containing this icon is disabled, the icon will
*  behave in a disabled manner regardless of the setting of the icon.disabled property.
* @type {boolean}
* @default null
*/
disabled?: boolean;

/**
* Criteria to be evaluated to determine whether this icon should appear enabled.
*  
*  Criteria are evaluated against the ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','form\'s current values')} as well as 
*  the current Canvas.ruleScope. Criteria are re-evaluated every time
*  form values or the rule context changes, whether by end user action or by programmatic calls.
*  
*  A basic criteria uses textMatchStyle:"exact". When specified in
*  componentXML this property allows
*  xmlCriteriaShorthand for defining criteria.
*  
*  Note: A FormItemIcon using enableWhen must have a FormItem.name defined on
*  its FormItem.
* @type {AdvancedCriteria}
* @default null
*/
enableWhen?: AdvancedCriteria;

/**
* TabIndex for this formItemIcon.
*  
*  Set to -1 to remove the icon from the tab order, but be cautious doing so: if the
*  icon triggers important application functionality that cannot otherwise be accessed
*  via the keyboard, it would be a violation of accessibility standard to remove the
*  icon from the tab order.
*  
*  Any usage other than setting to -1 is extremely advanced in the same way as using
*  FormItem.globalTabIndex.
* @type {Integer}
* @default null
*/
tabIndex?: Integer;

/**
* If set, this property determines the width of this icon in px.
*    If unset the form item's iconWidth property will be used instead.
* @type {number}
* @default null
*/
width?: number;

/**
* If set, this property determines the height of this icon in px.
*    If unset the form item's iconHeight property will be used instead.
* @type {number}
* @default null
*/
height?: number;

/**
* If set, this property will be displayed as a prompt (and tooltip text) for this form
*  item icon.
*  
*  If unset the form item's iconPrompt property will be used instead.
* @type {HTMLString}
* @default null
*/
prompt?: HTMLString;

/**
* Show this icon when its item gets focus, and hide it when it loses focus. If
*  non-null, overrides the default behavior specified by 
*  FormItem.showIconsOnFocus or FormItem.showPickerIconOnFocus, as
*  appropriate. This feature allows space to be saved in the form for items not being
*  interacted with, and helps draw attention to the item currently in focus.
* @type {boolean}
* @default null
*/
showOnFocus?: boolean;

/**
* If show-on-focus behavior is enabled for this icon via FormItemIcon.showOnFocus
*  or related properties at the item level, and this icon is marked as disabled,
*  should it be shown on focus? If unset, will be derived from the
*  FormItem.showDisabledIconsOnFocus or 
*  FormItem.showDisabledPickerIconOnFocus settings.
* @type {boolean}
* @default null
*/
showDisabledOnFocus?: boolean;

/**
* If set, this property determines the number of pixels space to be displayed on 
*  the left of this form item icon, or for FormItemIcon.inline icons
*  whose FormItemIcon.inlineIconAlign is
*  "left",
*  
*  on the right of this form item icon. Must be non-negative.
*  If unset, the form item's FormItem.iconHSpace will be used instead.
* @type {Integer}
* @default null
*/
hspace?: Integer;

/**
* Criteria to be evaluated to determine whether this icon should be visible.
*  
*  Criteria are evaluated against the ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','form\'s current values')} as well as 
*  the current Canvas.ruleScope. Criteria are re-evaluated every time
*  form values or the rule context changes, whether by end user action or by programmatic calls.
*  
*  A basic criteria uses textMatchStyle:"exact". When specified in
*  componentXML this property allows
*  xmlCriteriaShorthand for defining criteria.
*  
*  Note: A FormItemIcon using visibleWhen must have a FormItem.name defined on
*  its FormItem.
* @type {AdvancedCriteria}
* @default null
*/
visibleWhen?: AdvancedCriteria;

/**
* For PickList items with PickListItemIconPlacement set such that the pickList does
*  not render near-origin, should this icon be rendered inline within
*  the formItem itself, or within the ComboBoxItem.pickerNavigationBar.
*  
*  If not explicitly specified at the icon level, this will be picked up from
*  PickList.iconPlacement.
*  
*  For mobile browsing with limited available screen space, icons rendered in
*  the navigation bar may be easier for the user to interact with.
* @type {PickListItemIconPlacement}
* @default null
*/
iconPlacement?: PickListItemIconPlacement;



/**
*  Click handler for this icon.
*  
*  
*  Return false to cancel this event. 
*  
*  
*  If this event is not cancelled by the icon-level click handler, it may also
*  be handled at the FormItem level via FormItem.pickerIconClick [for the
*  picker icon only], and then FormItem.iconClick
* 
* 
* @param {DynamicForm} The Dynamic Form to which this icon's item belongs.
* @param {FormItem} The Form Item containing this icon
* @param {FormItemIcon} A pointer to the form item icon clicked
*/
click(form:DynamicForm, item:FormItem, icon:FormItemIcon): boolean;

/**
*    StringMethod action to fire when this icon has focus and receives a keypress
*    event.
*    If unset the form item's iconKeyPress method will be fired instead 
*    (if specified).
* 
* @param {KeyName} Name of the key pressed
* @param {string} character produced by the keypress
* @param {DynamicForm} The Dynamic Form to which this icon's item belongs.
* @param {FormItem} The Form Item containing this icon
* @param {FormItemIcon} A pointer to the form item icon
*/
keyPress(keyName:KeyName, character:string, form:DynamicForm, item:FormItem, icon:FormItemIcon): void;

/**
*  If specified, icon.showIf will be evaluated when the form item is
*  drawn or redrawn. Return true if the icon should be visible, or false if it
*  should be hidden. Note that if FormItem.showIcon or FormItem.hideIcon
*  is called, this method will be overridden.
* 
* @param {DynamicForm} the DynamicForm in which the icon is embedded
* @param {FormItem} the item to which this icon is attached.
*/
showIf(form:DynamicForm, item:FormItem): boolean;



}


/**
* Represents a sector on the gauge.
*/
interface GaugeSector {
/**
* @type {CSSColor}
* @default null
*/
fillColor?: CSSColor;

/**
* @type {Float}
* @default "0"
*/
startAngle?: Float;

/**
* @type {Float}
* @default "0"
*/
endAngle?: Float;

/**
* @type {Float}
* @default "0"
*/
value?: Float;





}


/**
* Defines a simple gradient vertical gradient between Gradient.startColor
*  Gradient.endColor, or using Gradient.colorStops. See
*  SimpleGradient, LinearGradient and RadialGradient for further
*  properties to define more advanced gradients.
*/
interface Gradient {
/**
* Identifier which can be used by one or more DrawItems when gradient is assigned
*  to DrawPane.gradients. The ID property is optional when gradient is assigned directly
*  to a DrawItem.
*  
*  The ID must be unique within DrawPane.gradients if defined.
* @type {Identifier}
* @default null
*/
id?: Identifier;

/**
* An array of color stops for this gradient.
* @type {Array<Partial<ColorStop>>}
* @default null
*/
colorStops?: Array<Partial<ColorStop>>;

/**
* A start color for the gradient. If both startColor and Gradient.endColor
*  are set then Gradient.colorStops is ignored.
* @type {CSSColor}
* @default null
*/
startColor?: CSSColor;

/**
* An end color for the gradient. If both Gradient.startColor and endColor
*  are set then Gradient.colorStops is ignored.
* @type {CSSColor}
* @default null
*/
endColor?: CSSColor;





}


/**
* An auto-generated subclass of TreeNode representing the group nodes
*  in a grouped ListGrid.
*/
interface GroupNode {
/**
* Array of ListGridRecord that belong to this group, or, for multi-field grouping, array
*  of groupNodes of subgroups under this groupNode.
* @type {Array<Partial<ListGridRecord>> | GroupNode}
* @default "see below"
*/
groupMembers?: Array<Partial<ListGridRecord>> | GroupNode;

/**
* The computed title for the group, which results from ListGridField.getGroupTitle
* @type {HTMLString}
* @default "see below"
*/
groupTitle?: HTMLString;

/**
* The value from which groups are computed for a field,
*  which results from ListGridField.getGroupValue
* @type {any}
* @default "see below"
*/
groupValue?: any;

/**
* Name of the field being grouped by this node.
* @type {FieldName}
* @default "see below"
*/
groupName?: FieldName;





}


/**
* A Javascript object defining the details of a single group operation. Used by the 
*  MultiGroupDialog to edit multi-level grouping scenarios for
*  use by components that support grouping.
*/
interface GroupSpecifier {
/**
* The property name, eg a ListGridField, to which this groupSpecifier applies.
* @type {string}
* @default null
*/
property?: string;

/**
* The grouping mode applied by this specifier. The list of available modes can come either 
*  from the associated ListGridField.groupingModes or from it's specified 
*  SimpleType.groupingModes.
* @type {string}
* @default null
*/
grouping?: string;

/**
* A normalizer function which this groupSpecifier will use to group.
* @type {Function}
* @default null
*/
normalizer?: Function;

/**
* A DataBoundComponent providing the context for the group-normalizer.
* @type {DataBoundComponent}
* @default null
*/
context?: DataBoundComponent;





}


/**
* Group Summary.
*/
interface GroupSummary {




}


/**
* Defines one level of headers shown above the event area in a Timeline.
*/
interface HeaderLevel {
/**
* Unit of time shown at this level of header.
* @type {TimeUnit}
* @default null
*/
unit?: TimeUnit;

/**
* If set, the width for each of the spans in this headerLevel. Note that this setting only 
*  has an effect on the innermost headerLevel.
* @type {Integer}
* @default null
*/
headerWidth?: Integer;

/**
* Optional sparse array of titles for the spans on this headerLevel. If a given span in this
*  headerLevel has a corresponding entry in this array, it will be used as the span's title.
*  
*  If not specified, default titles are generated (eg "Q1" for unit "quarter") and then passed
*  into the HeaderLevel.titleFormatter, if one has been installed,
*  for further customization.
* @type {Array<Partial<string>>}
* @default null
*/
titles?: Array<Partial<string>>;



/**
*  An optional function for providing formatted HTML for the title of a given span in this 
*  HeaderLevel. If unset, Timelines use the HeaderLevel.titles, if one is
*  set, or generate default titles based on the unit-type and date-range.
*  
*  Note that this method will not run for spans in this headerLevel that have a non-null entry 
*  in the HeaderLevel.titles array.
*  
* 
* @param {HeaderLevel} a reference to this headerLevel
* @param {Date} the start of the date-range covered by this span in this level
* @param {Date} the end of the date-range covered by this span in this level - may be            null
* @param {string} the default title as generated by the Timeline
* @param {Calendar} a reference to the Calendar or Timeline
*/
titleFormatter(headerLevel:HeaderLevel, startDate:Date, endDate:Date, defaultValue:string, viewer:Calendar): HTMLString;



}


/**
* A header span appears as a second level of headers in a ListGrid, spanning one or more
*  ListGrid columns and their associated headers.
*  
*  See ListGrid.headerSpans.
*  
*  In addition to the properties documented here, all other properties specified on the
*  headerSpan object will be passed to the Class.create method of the
*  ListGrid.headerSpanConstructor. This allows you to set properties such as
*  Button.baseStyle or StretchImgButton.src directly in a
*  headerSpan.
*/
interface HeaderSpan {
/**
* Name for this headerSpan, for use in APIs like ListGrid.setHeaderSpanTitle.
*  
*  Name is optional, but if specified, must be unique for this ListGrid (but not globally
*  unique) as well as a valid JavaScript identifier, as specified by ECMA-262 Section 7.6
*  (the String.isValidID function can be used to test whether a name is a valid JavaScript
*  identifier).
* @type {Identifier}
* @default null
*/
name?: Identifier;

/**
* List of fields that this header spans. Fields should be identified by their value for
*  ListGridField.name.
*  
*  Developers may define multiple levels of header-spans by specifying HeaderSpan.spans
*  however a span cannot be specified with both fields and spans.
* @type {Array<Partial<string>>}
* @default null
*/
fields?: Array<Partial<string>>;

/**
* This property allows developer to "nest" header spans, grouping fields together by
*  multiple layers of granularity.
*  
*  For example a group of fields could be nested within two layers of header spans as follows:
*  
*  
*  { title:"Europe", spans:[
*    {title:"France", fields:["Paris", "Lyon"]},
*    {title:"UK", fields:["London", "Glasgow"]},
*    {title:"Spain", fields:["Barcelona"]}
*  ]
*  }
*  
*  
*  
*  Note that a span definition can not include both spans
*  and HeaderSpan.fields.
* @type {Array<Partial<HeaderSpan>>}
* @default null
*/
spans?: Array<Partial<HeaderSpan>>;

/**
* A title for this headerSpan, to display in the headerSpan button for this headerSpan
*  and in other contexts such as the ListGrid.canPickFields.
* 
*  Note: if you want to use HTML tags to affect the display of the header, you should do so
*  via HeaderSpan.headerTitle instead so that other places where the title
*  appears in the UI are not affected. Refer to discussion at ListGridField.title.
* @type {string}
* @default null
*/
title?: string;

/**
* Optional title for the headerSpan button for this headerSpan. If specified this will be
*  displayed in the headerSpan button instead of HeaderSpan.title. Set to an empty
*  string to suppress the title in the header button entirely.
* @type {string}
* @default null
*/
headerTitle?: string;

/**
* Custom base style to apply to the header button created for this span instead
*  of ListGrid.headerBaseStyle.
*  
*  Note that depending on the header button constructor, you may have to specify
*  HeaderSpan.headerTitleStyle as well.
* @type {CSSStyleName}
* @default null
*/
headerBaseStyle?: CSSStyleName;

/**
* Custom titleStyle to apply to the header button created for this span instead of
*  ListGrid.headerTitleStyle.
*  
*  Note that this will typically only have an effect if
*  ListGrid.headerButtonConstructor is set to StretchImgButton or a subclass
*  thereof.
* @type {CSSStyleName}
* @default null
*/
headerTitleStyle?: CSSStyleName;

/**
* Height of this headerSpan. Defaults to ListGrid.headerSpanHeight.
* @type {Integer}
* @default null
*/
height?: Integer;

/**
* Vertical alignment of the title of this headerSpan.
*  
*  Defaults to listGrid.headerSpanVAlign if unset.
* @type {VerticalAlignment}
* @default null
*/
valign?: VerticalAlignment;

/**
* Horizontal alignment of the title of this headerSpan.
* @type {Alignment}
* @default "center"
*/
align?: Alignment;

/**
* Should the span title wrap if there is not enough space horizontally to accommodate it.
*  If unset, default behavior is derived from ListGrid.wrapHeaderSpanTitles. (This
*  is a soft-wrap - if set the title will wrap at word boundaries.)
* @type {boolean}
* @default null
*/
wrap?: boolean;





}


/**
* An object representing a user-created and user-modifiable hilite, which can be created and
*  edited with a HiliteEditor either directly or via the
*  ListGrid.canEditHilites behavior.
*  
*  See hiliting for an overview.
*/
interface Hilite {
/**
* Unique id for this hilite definition. 
*  
*  For hilites that include Hilite.criteria this is not required.
*  
*  If you are explicitly marking records for hiliting, set
*  DataBoundComponent.hiliteProperty on the record to this id.
* @type {string}
* @default null
*/
id?: string;

/**
* CSS text to be applied to cells where this hilite is applied, for example,
*  "background-color:#FF0000"
* @type {CSSText}
* @default null
*/
cssText?: CSSText;

/**
* Name of the field, or array of fieldNames, this hilite should be applied to. 
*  
*  If unset, hilite is applied to every field of the record.
* @type {FieldName | Array<Partial<FieldName>>}
* @default null
*/
fieldName?: FieldName | Array<Partial<FieldName>>;

/**
* Criteria defining what records this hilite should apply to.
* @type {Criteria | AdvancedCriteria}
* @default null
*/
criteria?: Criteria | AdvancedCriteria;

/**
* HTML to pre-pend to cell values where this hilite is applied.
* @type {HTMLString}
* @default null
*/
htmlBefore?: HTMLString;

/**
* HTML to append to the end of cell values where this hilite is applied.
* @type {HTMLString}
* @default null
*/
htmlAfter?: HTMLString;

/**
* Value to show in place of the actual value from the record, for a record that matches
*  this hilite.
*  
*  This can be used to take ranges of numeric values and simplify them to "Low", "Medium",
*  "High" or similar textual values, translate very small or very large values to "Outlier" or
*  "Negligible", and similar use cases.
* @deprecated * htmlValue is deprecated in favor of Hilite.replacementValue.  Note that unlike replacementValue, this property does not respect
*  Hilite.disabled, and will be applied even if disabled is set to
*  true

* @type {string}
* @default null
*/
htmlValue?: string;

/**
* Whether this hilite is currently disabled.
*  
*  Hilites can be programmatically enabled and disabled via DataBoundComponent.enableHilite.
* @type {boolean}
* @default false
*/
disabled?: boolean;

/**
* Can highlight be edited from header context menu? Setting attribute to
*  false prevents editing. A null or true
*  value allows editing.
* @type {boolean}
* @default null
*/
canEdit?: boolean;

/**
* User-visible title for this hilite. Used for interfaces such as menus that can enable or
*  disable hilites.
* @type {string}
* @default null
*/
title?: string;

/**
* When edited via a HiliteEditor, the value for the foreground color of this 
*  hilite. If this is omitted, it will be automatically derived from the textColor
*  attribute of Hilite.cssText. When a hilite is saved in a HiliteEditor, both 
*  attributes are set automatically.
* @type {string}
* @default null
*/
textColor?: string;

/**
* When edited via a HiliteEditor, the value for the background color of this 
*  hilite. If this is omitted, it will be automatically derived from the backgroundColor
*  attribute of Hilite.cssText. When a hilite is saved in a HiliteEditor, both 
*  attributes are set automatically.
* @type {string}
* @default null
*/
backgroundColor?: string;

/**
* URL of an icon to show when this hilite is applied to a cell. Position of the icon 
*  is controlled by DataBoundComponent.hiliteIconPosition or 
*  ListGridField.hiliteIconPosition.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* HTML which replaces the cell's textual value where this hilite is applied.
*  
*  Note that sorting, filtering, etc behavior will still operate on the underlying value.
*  For example, if there is a date field with the FilterEditor enabled, the default search
*  interface will still offer date-range based filtering even if hilites have caused values
*  to be displayed as text such as "current" or "past due".
* @type {HTMLString}
* @default null
*/
replacementValue?: HTMLString;





}


/**
* This class provides synthetic history support. Using this class, you can create history
*  entries at any point and be called back when the user next navigates to any of these history
*  entries via any of the browser mechanisms that enable navigation: back/forward buttons,
*  history dropdown and bookmarks.
*  
*  The history entries created using this mechanism work just like history entries created
*  natively by the browser, except you get a callback whenever a transition occurs. This
*  implementation correctly handles "deep" history - i.e. it correctly maintains forward and
*  back history when the user navigates forward or back away from the page that uses this
*  module.
*  
*  This module is usable independent of the rest of SmartClient - you can use it on pages that
*  don't load any other modules.
*  
*  Platform Notes:
*  In Safari (4.0 and above), this module has the limitation that the arbitrary data parameter
*  in addHistoryEntry() is not reliable.
*  Internet Explorer: If you set document.domain on the top-level page, the History
*  mechanism will behave sub-optimally in IE - three clicks one the forward/back buttons will
*  be required to transition to the next history entry.
*  
*  Usage overview
*  Synthetic history entries are added to the browser history via History.addHistoryEntry.
*  When this method is called, the page's URL will be modified and the native browser back button
*  will become active.
*  The History.registerCallback allows the developer to register a callback method to
*  fire when the user navigates to these generated history entries. This method will be fired
*  with an appropriate history ID when the user hits the back-button or explicitly navigates to
*  the URL generated for some synthetic history entry.
*/
interface History {




}


/**
* A set of properties that can be used to create an image.
*/
interface ImgProperties {
/**
* Specifies the URL of the image local to the skin or application directory.
* @type {URL}
* @default null
*/
src?: URL;

/**
* Specifies the width of the image.
* @type {number}
* @default null
*/
width?: number;

/**
* Specifies the height of the image.
* @type {number}
* @default null
*/
height?: number;

/**
* Specifies the name of the image. This is an identifier unique to the canvas, and subsequent
*  calls to Canvas.getImage and
*  Canvas.setImage
*  with this name will act on the image object created using this ImgProperties object.
* @type {string}
* @default null
*/
name?: string;

/**
* Specifies the additional attributes to write in the tag. Event-related attributes should be
*  added to ImgProperties.eventStuff instead to guarantee proper behavior when using SVG images.
* @type {string}
* @default null
*/
extraStuff?: string;

/**
* Specifies the additional event-related attributes to write in the tag.
* @type {string}
* @default null
*/
eventStuff?: string;

/**
* Specifies the image-specific image directory to override the default.
* @type {URL}
* @default null
*/
imgDir?: URL;





}


/**
* Identifier for a key pressed by the user, optionally specifying whether the Shift, Control,
*  and/or Alt keys should be held or not held when the key is pressed, used by various methods.
*/
interface KeyIdentifier {
/**
* Name of the key.
* @type {KeyName}
* @default null
*/
keyName?: KeyName;

/**
* If set, whether the Shift key must be held down.
* @type {boolean}
* @default null
*/
shiftKey?: boolean;

/**
* If set, whether the Control key must be held down.
* @type {boolean}
* @default null
*/
ctrlKey?: boolean;

/**
* If set, whether the Alt key must be held down.
* @type {boolean}
* @default null
*/
altKey?: boolean;





}


/**
* Lane shown in a Timeline view, or in a Calendar.dayView when 
*  Calendar.showDayLanes is true. Each lane is a row or column, 
*  respectively, that can contain a set of CalendarEvents. CalendarEvents are placed in
*  lanes by matching the Lane.name property to the value of the 
*  Calendar.laneNameField property on the CalendarEvent.
*  
*  Lanes are typically used to show tasks assigned to different people, broadcasts planned for
*  different channels, and similar displays.
*/
interface Lane {
/**
* To determine whether a CalendarEvent should be placed in this lane, the value of this 
*  attribute is compared with the Calendar.laneNameField property on the CalendarEvent.
* @type {string}
* @default null
*/
name?: string;

/**
* In Timelines, the height of this Lane's row. Has no effect when set on a Lane 
*  being displayed in a Calendar.dayView as a result of 
*  Calendar.showDayLanes being true.
*  
*  If set directly on a Lane.sublanes, overrides the default behavior of 
*  dividing the height equally among the lane's sublanes. Each sublane is still initially 
*  assigned an equal slice of the parent height, and the value for this sublane is 
*  then updated. So the overall height of the parent lane will change by the delta between the 
*  initial slice and the specified one.
* @type {number}
* @default null
*/
height?: number;

/**
* When set on a Lane being displayed in a Calendar.dayView as a result of 
*  Calendar.showDayLanes being set, dictates the width of the Lane's column. Has no 
*  effect in Timelines.
*  
*  If set directly on a Lane.sublanes, overrides the default behavior of 
*  dividing the width equally among the lane's sublanes. Each sublane is still initially 
*  assigned an equal slice of the original parent width, and the value for this sublane is then
*  updated. So the overall width of the parent lane will change by the delta between the 
*  initial slice and the specified one.
* @type {number}
* @default null
*/
width?: number;

/**
* Title to show for this lane. Has no effect if set directly on Lane.sublanes.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* Array of Lane objects that will share the available space in the parent Lane,
*  vertically in Calendar.timelineView and horizontally in 
*  Calendar.dayView.
*  
*  Only one level of sublanes is supported, so this attribute only has an effect on
*  Calendar.lanes.
*  
*  Note that this feature is mutually exclusive with the 
*  Calendar.eventAutoArrange of events that share time.
* @type {Array<Partial<Lane>>}
* @default null
*/
sublanes?: Array<Partial<Lane>>;

/**
* The base name for the CSS class applied to Calendar.eventCanvas when they're
*  rendered in this lane. See Calendar.eventStyleName.
*  
*  If set directly on a Lane.sublanes, overrides the corresponding value on
*  the parent Calendar.lanes. See 
*  Calendar.getEventCanvasStyle for more information.
* @type {CSSStyleName}
* @default null
*/
eventStyleName?: CSSStyleName;

/**
* The base style-name for normal cells in this Lane.
* @type {CSSStyleName}
* @default null
*/
styleName?: CSSStyleName;

/**
* The base style-name for Calendar.laneFields displayed in this Lane.
* @type {CSSStyleName}
* @default null
*/
fieldStyleName?: CSSStyleName;





}


/**
* Definition of a linear gradient between two points, (LinearGradient.x1, LinearGradient.y1)
*  and (LinearGradient.x2, LinearGradient.y2).
*/
interface LinearGradient extends Gradient {
/**
* X coordinate of the start point. This can be a number or a percentage of the width of the
*  bounding box of the DrawItem to which it is applied.
* @type {string}
* @default null
*/
x1?: string;

/**
* Y coordinate of the start point. This can be a number or a percentage of the height of the
*  bounding box of the DrawItem to which it is applied.
* @type {string}
* @default null
*/
y1?: string;

/**
* X coordinate of the end point. This can be a number or a percentage of the width of the
*  bounding box of the DrawItem to which it is applied.
* @type {string}
* @default null
*/
x2?: string;

/**
* Y coordinate of the end point. This can be a number or a percentage of the height of the
*  bounding box of the DrawItem to which it is applied.
* @type {string}
* @default null
*/
y2?: string;





}


/**
* An ordinary JavaScript object containing properties that configures the display of
*  and interaction with the columns of a ListGrid.
*/
interface ListGridField extends DBCField {
/**
* When exporting data to Excel/OpenOffice format using ListGrid.exportData or
*  ListGrid.exportClientData with ListGrid.exportFieldWidths set,
*  set this flag false to cause this field to "opt out" of width export. Fields that opt out
*  in this way have the corresponding spreadsheet column autosized (ie, made just wide enough
*  that content is not clipped).
*  
*  This setting has no effect if listGrid.exportFieldWidths is not set.
* @type {boolean}
* @default true
*/
exportFieldWidth?: boolean;

/**
* Canvas.ariaRole for the header button for this field.
* @type {string}
* @default null
*/
headerButtonAriaRole?: string;

/**
* Canvas.ariaState for the header button for this field.
* @type {object}
* @default null
*/
headerButtonAriaState?: object;

/**
* If this field can be grouped, this attribute represents the set of grouping styles that 
*  are available. For example, a "date" field might be able to be 
*  grouped by week or month, as well as by the date itself.
*  
*  If groupingModes are present and
*  ListGrid.canGroupBy, the menu for this field includes a
*  submenu of possible grouping modes generated from the groupingModes valueMap.
*  When the user selects a particular grouping mode,
*  ListGridField.groupingMode is set to the user's chosen mode,
*  and this choice can be detected via the field parameter to
*  ListGridField.getGroupValue in order to provide different modes of grouping.
*  
*  The user may also choose to group records without specifying a grouping mode, in this case,
*  the ListGridField.defaultGroupingMode is used.
*  
*  Note that getGroupValue, groupingModes et al can be specified on
*  SimpleType declarations. See this list of
*  builtinGroupingModes for more information.
* @type {ValueMap}
* @default null
*/
groupingModes?: ValueMap;

/**
* For a field that allows multiple ListGridField.groupingModes,
*  the current grouping mode.
*  
*  This property is set when a user chooses a particular grouping mode, and may be set on
*  ListGrid creation to affect the initial grouping.
* @type {Identifier}
* @default null
*/
groupingMode?: Identifier;

/**
* Default groupingMode used when the user does not specify a mode or grouping is triggered
*  programmatically and ListGridField.groupingMode is unset.
*  See ListGridField.groupingModes.
* @type {Identifier}
* @default null
*/
defaultGroupingMode?: Identifier;

/**
* For fields of type:"float" or derived from float, number of digits after the decimal point
*  to consider when grouping.
*  
*  For example, groupPrecision:2 indicates that 45.238 and 45.231 group together,
*  but 45.22 and 45.27 are separate.
*  
*  See also ListGridField.groupGranularity for grouping by broader
*  ranges.
* @type {Integer}
* @default null
*/
groupPrecision?: Integer;

/**
* Granularity of grouping for numeric fields.
*  
*  Groups will be formed based on ranges of values of size groupGranularity. For
*  example, if groupGranularity were 1000, groups would be 0-1000, 1000-2000, etc.
* @type {Integer}
* @default null
*/
groupGranularity?: Integer;

/**
* Determines whether this field can be hilited. Set to false to prevent this
*  field from appearing in HiliteEditor.
* @type {boolean}
* @default null
*/
canHilite?: boolean;

/**
* Determines whether hiliting for this field is shown in a group summary.
*  Set to false to prevent this field from showing hilite in a group summary.
*  
*  All hilites in group summary rows can be controlled with the
*  ListGrid.showHilitesInGroupSummary property.
* @type {boolean}
* @default null
*/
showHilitesInGroupSummary?: boolean;

/**
* Determines whether this field will be groupable in the header context menu.
* @type {boolean}
* @default true
*/
canGroupBy?: boolean;

/**
* When true, this field can only be used for sorting if the data is entirely client-side.
* @type {boolean}
* @default false
*/
canSortClientOnly?: boolean;

/**
* When set to false, this field will not show a context menu in its header.
* @type {boolean}
* @default true
*/
showDefaultContextMenu?: boolean;

/**
* Dictates whether the data in this field be exported. Explicitly set this
*  to false to prevent exporting. Has no effect if the underlying
*  DataSourceField.canExport is explicitly set to
*  canExport: false.
* @type {boolean}
* @default null
*/
canExport?: boolean;

/**
* Dictates whether the data in this field should be exported raw by
*  ListGrid.exportClientData. If set to true for a
*  field, the values in the field-formatters will not be executed for data in this field.
*  Decreases the time taken for large exports.
* @type {boolean}
* @default null
*/
exportRawValues?: boolean;

/**
* Dictates whether numeric values should be exported as raw numbers instead of
*  formatted values when using ListGrid.exportClientData.
*  
*  This property is only consulted if exportRawValues is not set to
*  true at the ListGrid.exportRawValues or
*  ListGridField.exportRawValues level. That property causes all values,
*  including numeric values, to be exported unformatted.
*  
*  This is useful for cases where an explicit ListGrid formatter function simply displays the number
*  as a formatted string for the user (for example "1,234"). Exporting that formatted
*  string rather than the underlying numeric value causes spreadsheet applications such as
*  Excel to lose some functionality.
*  
*  If this property is not explicitly set, numeric values will be exported as raw
*  numbers for DSRequest.exportAs only.
*  
*  This property overrides the setting at the ListGrid.exportRawNumbers level.
* @type {boolean}
* @default null
*/
exportRawNumbers?: boolean;

/**
* The value to display for a ListGridField when it appears in the ListGrid.summaryRow. The
*  default for normal fields is null and for special fields, like the ListGrid.checkboxField,
*  the default is to show a blank value (a non-breaking space).
* @type {HTMLString}
* @default null
*/
summaryValue?: HTMLString;

/**
* This property is a mapping from data values for this field to SCImgURL for
*  icons to display for those data values. 
*  
*  For example, given a field named "status" with possible values
*  "Normal", "Slow", "Offline", the follow definition would show various icons for that
*  field:
*  
*  
*  
*  fields : [
*    { name:"status",
*     valueIcons: {
*       Normal : "greenIcon.png",
*       Slow : "yellowIcon.png",
*       Offline : "redIcon.png"
*     }
*    },
*    ... other fields ...
*  ]
*  
*  
*  
*  
*  If a simple value-to-URL mapping is not enough, you can override ListGrid.getValueIcon
*  to customize the behavior. You can even specify an empty valueIcons map
*  and use ListGrid.getValueIcon to return arbitrary icons with no fixed mapping.
*  
*  valueIcons can either be displayed alongside the normal value or can
*  replace the normal field value so that only the icon is shown. See
*  ListGridField.showValueIconOnly. When placed alongside the value, use
*  ListGridField.valueIconOrientation to control left- vs right-side placement.
*  
*  If inline editing is enabled for this field, editors displayed for this field will also
*  show valueIcons. This may be overridden by explicitly setting
*  ListGridField.editorValueIcons.
*  
*  Note that the following attributes related to valueIcon styling will also be picked up
*  by the editor from the ListGridField object unless explicitly specified via the
*  equivalent editor_ attributes:
*  ListGridField.valueIconWidth
*  ListGridField.valueIconHeight
*  ListGridField.valueIconSize
*  ListGridField.valueIconLeftPadding
*  ListGridField.valueIconRightPadding
*  ListGridField.imageURLPrefix
*  ListGridField.imageURLSuffix
*  
*  If ListGridField.valueIconClick is defined for the field, a pointer
*  cursor will be shown when the user rolls over the valueIcon, and the valueIconClick
*  method will execute when the user clicks the icon.
* @type {Map<String>}
* @default null
*/
valueIcons?: Map<String>;

/**
* Default width and height of value icons in this field.
*  Takes precedence over valueIconWidth, valueIconHeight and valueIconSize specified at
*  the ListGrid level.
*  Can be overridden via ListGridField.valueIconWidth and {ListGridField.valueIconHeight}
* @type {number}
* @default null
*/
valueIconSize?: number;

/**
* Width for value icons for this listGrid field.
*  Overrides ListGrid.valueIconSize, ListGrid.valueIconWidth, and
*  ListGridField.valueIconSize.
* @type {number}
* @default null
*/
valueIconWidth?: number;

/**
* Height for value icons for this listGrid field.
*  Overrides ListGrid.valueIconSize, ListGrid.valueIconHeight, and
*  ListGridField.valueIconSize.
* @type {number}
* @default null
*/
valueIconHeight?: number;

/**
* How much padding should there be on the left of valueIcons for this field
*  Overrides ListGrid.valueIconLeftPadding
* @type {number}
* @default null
*/
valueIconLeftPadding?: number;

/**
* How much padding should there be on the right of valueIcons for this field
*  Overrides ListGrid.valueIconRightPadding
* @type {number}
* @default null
*/
valueIconRightPadding?: number;

/**
* When some cell in this field is being edited, setting this property will specify the
*  value icons to display in the cell's editor. If unset, the editor's valueIcons
*  will be determined in the same way as it would be for a static cell.
* @type {Map<String>}
* @default null
*/
editorValueIcons?: Map<String>;

/**
* When some cell in this field is being edited, setting this property will specify the
*  width for value icons in the cell's editor. If unset, the editor's valueIcon width and
*  height will be determined in the same way as it would be for a static cell.
* @type {number}
* @default null
*/
editorValueIconWidth?: number;

/**
* When some cell in this field is being edited, setting this property will specify the
*  height for value icons in the cell's editor. If unset, the editor's valueIcon width and
*  height will be determined in the same way as it would be for a static cell.
* @type {number}
* @default null
*/
editorValueIconHeight?: number;

/**
* If this field has a valueIcons property specified, setting this property causes
*  the valueIcon for each value to be displayed in the cell without also showing the
*  record's value for the field.
*  
*  If unset the default behavior is to show the icon only if an explicit valueMap is
*  specified as well in addition to a valueIcons map, otherwise show both the valueIcon and
*  value for the cell.
*  
*  Note that if this field is editable FormItem.showValueIconOnly will be passed
*  through to editors displayed in this field.
* @type {boolean}
* @default null
*/
showValueIconOnly?: boolean;

/**
* If this field has a valueIcons property specified, setting this property to true will
*  prevent the valueIcon being written out into this field's cells.
*  
*  Note this property may also be set to false to avoid showing the standard
*  ListGrid.booleanTrueImage and ListGrid.booleanFalseImage for fields of type
*  boolean.
* @type {boolean}
* @default null
*/
suppressValueIcon?: boolean;

/**
* If we're showing a valueIcon for this field should it appear to the left or the right
*  of the text? By default the icon will appear to the left of the textual value -
*  set this to "right" to show the icon on the right of the text.
*  Has no effect if ListGridField.showValueIconOnly is true
* @type {string}
* @default null
*/
valueIconOrientation?: string;

/**
* When ListGrid.hiliteIcons are present, where the hilite icon will be placed
*  relative to the field value. See HiliteIconPosition.
*  Overrides ListGrid.hiliteIconPosition.
* @type {HiliteIconPosition}
* @default null
*/
hiliteIconPosition?: HiliteIconPosition;

/**
* Default width and height of ListGrid.hiliteIcons in this field.
*  Takes precedence over hiliteIconWidth, hiliteIconHeight and hiliteIconSize specified at
*  the component level.
*  Can be overridden via ListGridField.hiliteIconWidth and
*  ListGridField.hiliteIconHeight
* @type {number}
* @default null
*/
hiliteIconSize?: number;

/**
* Width for hilite icons for this field.
*  Overrides ListGrid.hiliteIconSize, ListGrid.hiliteIconWidth, and
*  ListGridField.hiliteIconSize.
* @type {number}
* @default null
*/
hiliteIconWidth?: number;

/**
* Height for hilite icons for this field.
*  Overrides ListGrid.hiliteIconSize, ListGrid.hiliteIconHeight, and
*  ListGridField.hiliteIconSize.
* @type {number}
* @default null
*/
hiliteIconHeight?: number;

/**
* How much padding should there be on the left of DataBoundComponent.hiliteIcons
*  for this field?
*  Overrides ListGrid.hiliteIconLeftPadding
* @type {number}
* @default null
*/
hiliteIconLeftPadding?: number;

/**
* How much padding should there be on the right of DataBoundComponent.hiliteIcons
*  for this field?
*  Overrides ListGrid.hiliteIconRightPadding
* @type {number}
* @default null
*/
hiliteIconRightPadding?: number;

/**
* If this field has type [ListGridFieldType] set to "image"
*  and the URL for the image displayed is not absolute, the path of the URL will be relative
*  to this string
*  Alternatively, if this field displays any valueIcons, this prefix will be applied to
*  the beginning of any ListGridField.valueIcons when determining the
*  URL for the image.
* @type {string}
* @default null
*/
imageURLPrefix?: string;

/**
* If any cells in this field are showing a value icon (see: ListGridField.valueIcons)
*  or this is has ListGridFieldType set to "image", the value
*  of this property will be appended to the end of the URL for the icon displayed.
*  Typical usage might be to append a file type such as ".gif" to the
*  filename of the image.
*  For editable fields, this property will also be passed through to any editors as
*  FormItem.imageURLSuffix.
* @type {string}
* @default null
*/
imageURLSuffix?: string;

/**
* When some cell in this field is being edited, this property can be used to apply
*  an explicit FormItem.imageURLPrefix to the editor in question.
*  This can be used to modify the valueIcons within the editor.
*  If unset, but ListGridField.imageURLPrefix is specified, that will be used
*  instead.
* @type {string}
* @default null
*/
editorImageURLPrefix?: string;

/**
* When some cell in this field is being edited, this property can be used to apply
*  an explicit FormItem.imageURLSuffix to the editor in question.
*  This can be used to modify the valueIcons within the editor.
*  If unset, but ListGridField.imageURLPrefix is specified, that will be used
*  instead.
* @type {string}
* @default null
*/
editorImageURLSuffix?: string;

/**
* Size of images shown for fieldTypes image and imageFile in this field.
*  This setting overrides the global ListGrid default ListGrid.imageSize.
*  
*  If set to a String, assumed to be a property on each record that specifies the image
*  height. For example, if field.imageSize is "logoSize",
*  record.logoSize will control the size of the image.
* @type {number}
* @default "16"
*/
imageSize?: number;

/**
* Width of images shown for fieldTypes image and imageFile in this field.
*  
*  If set to a String, assumed to be a property on each record that specifies the image
*  width. For example, if field.imageWidth is "logoWidth",
*  record.logoWidth will control the width of the image.
* @type {number}
* @default "16"
*/
imageWidth?: number;

/**
* Height of image shown for fieldTypes image and imageFile in this field.
*  
*  If set to a String, assumed to be a property on each record that specifies the image
*  height. For example, if field.imageHeight is "logoHeight",
*  record.logoHeight will control the height of the image.
* @type {number}
* @default "16"
*/
imageHeight?: number;

/**
* ListGrids picks a renderer for the view and edit mode of a field based on this attribute.
*  See ListGridFieldType for a summary of how types are rendered.
* @type {ListGridFieldType}
* @default "text"
*/
type?: ListGridFieldType;

/**
* Name of this field. Must be unique within this ListGrid as well as a valid JavaScript
*  identifier - see FieldName for details and how to check for validity.
*  
*  The name of field is also the property in each record which holds the value for that
*  field.
*  
*  If a ListGrid.dataSource is specified and the DataSource has a field with the
*  same name, the ListGridField and DataSourceField are merged, with any properties on the
*  ListGridField overriding those on the DataSourceField.
* @type {FieldName}
* @default null
*/
name?: FieldName;

/**
* dataPath for this field. This property allows the grid to display details of nested data
*  structures in a flat list of columns.
* @type {string}
* @default null
*/
dataPath?: string;

/**
* A title for this field, to display in the header for the field and in other
*  contexts such as the ListGrid.canPickFields.
*  
*  Note: if you want to use HTML tags to affect the display of the header, you should do so
*  via ListGridField.headerTitle instead so that other places where the title
*  appears in the UI are not affected. For example, you might set headerTitle
*  to an empty string to suppress the header title on a narrow column, but you would retain
*  the normal title in the title property to avoid a blank menu item in the
*  field picker menu, DataBoundComponent.editHilites and other contexts.
*  
*  Alternately you can specify a ListGridField.getFieldTitle method on the field to return the
*  HTML for the field title.
* @type {string}
* @default null
*/
title?: string;

/**
* This property may be set to false to explicitly suppress display of
*  the field title in the column header button for the field.
* @type {boolean}
* @default null
*/
showTitle?: boolean;

/**
* Should the field title wrap if there is not enough space horizontally to accommodate it.
*  If unset, default behavior is derived from ListGrid.wrapHeaderTitles. (This is a
*  soft-wrap - if set the title will wrap at word boundaries.)
*  
*  Notes:
*  
* - If autofitting is active, ListGridField.width and ListGridField.minWidth can be set to control the
*  minimum field width - see the links for details.
*  
* - This feature is incompatible with ListGrid.clipHeaderTitles, and
*  clipHeaderTitles will be disabled for wrapping fields.
* @type {boolean}
* @default null
*/
wrap?: boolean;

/**
* Whether to rotate the field's title so it's rendered vertically from bottom to top.
*  If unset, default behavior is derived from ListGrid.rotateHeaderTitles.
* @type {boolean}
* @default null
*/
rotateTitle?: boolean;

/**
* Specifies vertical alignment in the column header for this field: "top", "center", or 
*  "bottom". If unset, default behavior is derived from ListGrid.headerTitleVAlign.
* @type {VerticalAlignment}
* @default null
*/
valign?: VerticalAlignment;

/**
* HTML to show in a hover over cells in this field. Useful for fixed hover text, such as 
*  a tooltip for an icon in a field of type "icon". To show a custom prompt per cell,
*  see ListGridField.hoverHTML.
* @type {HTMLString}
* @default null
*/
cellPrompt?: HTMLString;

/**
* This property may be set to customize the wrap attribute for the
*  canvas shown when the mouse hovers over cells in this field. Note that this causes a 
*  soft-wrap - if set, the hover text will wrap at word boundaries.
*  
*  If unset, default behavior is derived from ListGrid.headerHoverWrap.
* @type {boolean}
* @default null
*/
hoverWrap?: boolean;

/**
* Specifies the width of the canvas shown when the mouse hovers over cells in this field. 
*  
*  If unset, default behavior is derived from ListGrid.headerHoverWidth.
* @type {Integer}
* @default null
*/
hoverWidth?: Integer;

/**
* By default, clicking a link rendered by this item opens it in a new browser window. You
*  can alter this behavior by setting this property. The value of this property will be
*  passed as the value to the target attribute of the anchor tag used to render
*  the link.
*  
*  If you set listGridField.target to "javascript", the default behavior is to catch and
*  consume mouse-clicks that would result in the link being followed. Instead, the
*  ListGrid.cellClick event is fired for the containing cell.
* @type {string}
* @default "_blank"
*/
target?: string;

/**
* Marks field as initially hidden.
*  The user will still be able to show the field via a context menu. 
*  This may be suppressed by setting ListGridField.canHide to false, or by 
*  setting ListGrid.canPickFields to false to suppress the
*  field-picker entirely.
*  
*  To mark a field as completely hidden (not shown to a user at all, in any component), set
*  DataSourceField.hidden instead.
* @type {boolean}
* @default null
*/
hidden?: boolean;

/**
* Criteria to be evaluated to determine whether this field should be visible.
*  
*  This criteria is dynamic and will be renterpreted each time the rule context changes.
*  Note that calling ListGrid.showField or ListGrid.hideField explicitly
*  will cause any visibleWhen attribute to be dropped.
* @type {Criteria}
* @default null
*/
visibleWhen?: Criteria;

/**
* Criteria to be evaluated to determine whether this field should be enabled.
*  
*  This criteria is dynamic and will be renterpreted each time the rule context changes.
* @type {Criteria}
* @default null
*/
enableWhen?: Criteria;

/**
* Whether this field should be "frozen" for the purposes of horizontal scrolling. See
*  frozenFields.
* @type {boolean}
* @default null
*/
frozen?: boolean;

/**
* Whether this field should display freezing/unfreezing options in its header context menu.
*  See frozenFields.
* @type {boolean}
* @default null
*/
canFreeze?: boolean;

/**
* Whether this field should be automatically frozen when other fields are frozen. When
*  true, the field will be automatically frozen to the extreme of the grid. The
*  automatically generated ListGrid.checkboxField,
*  ListGrid.expansionField and
*  ListGrid.rowNumberField fields are examples of fields that specify
*  autoFreeze: true.
*  
*  You can control the position of this field in the array of frozen fields by providing a
*  ListGridField.getAutoFreezePosition implementation.
* @type {boolean}
* @default null
*/
autoFreeze?: boolean;

/**
* If set to false, this field will be omitted from the column picker that appears in the
*  header context menu when ListGrid.canPickFields is enabled. This means that the
*  end user will not be able to hide it if it's currently shown, or show it if it's
*  currently hidden.
*  
*  If this property is set to false, and the 
*  ListGrid.useAdvancedFieldPicker is shown, if the field
*  is ListGridField.hidden, the field will not show in the list of available fields. If the
*  field is visible, it will be displayed in the list of currently visible fields, but
*  the advanced field picker user interface will disallow hiding it.
* @type {boolean}
* @default null
*/
canHide?: boolean;

/**
* Whether this field can be dragResized using the mouse. If unset, the default behavior
*  is governed by ListGrid.canResizeFields.
* @type {boolean}
* @default null
*/
canDragResize?: boolean;

/**
* Whether this field can be reordered using the mouse. If unset, the default behavior is
*  governed by ListGrid.canReorderFields. Note that setting this property to
*  false will lock this field from being moved - that is, the user is
*  prevented from moving this field directly by dragging with the mouse, or by dropping
*  another field onto this field.
*  
*  Note that setting canReorder:false on a field in the middle of a grid is
*  mostly useless, since it's possible that such a "locked" field may still be reordered
*  automatically, as a result of the user dragging one unlocked field onto another unlocked
*  field.
*  
*  Fields in ListGrid.headerSpans are treated as if they have
*  canReorder:false to keep the fields in the span together, except that
*  unlocked fields not in a span may be drag-reordered across them, even if they're at the
*  header's extreme left or right. HeaderSpans themselves may not be drag-reordered.
* @type {boolean}
* @default null
*/
canReorder?: boolean;

/**
* If the user is navigating through the grid using the keyboard, record click or double click
*  events may be generated via keyboard interactions (see ListGrid.generateClickOnSpace,
*  ListGrid.generateClickOnEnter, ListGrid.generateDoubleClickOnSpace,
*  ListGrid.generateDoubleClickOnEnter and ListGrid.arrowKeyAction).
*  
*  These synthetic events have both a target row and column.
*  Setting this flag to true ensures that this field will never be considered the target for
*  a keyboard click event.
* @type {boolean}
* @default null
*/
ignoreKeyboardClicks?: boolean;

/**
* If true, then this field is excluded from the bound component's view state. In addition,
*  the field will not be selected as the default title field
*  by DataBoundComponent.getTitleField
*  if DataBoundComponent.titleField is not provided.
* @type {boolean}
* @default null
*/
excludeFromState?: boolean;

/**
* If ListGrid.showGridSummary is true, should this field show a summary value.
*  If unset, this field will show a summary value in the summary row if an
*  explicit ListGridField.summaryFunction is specified or if a
*  SimpleType.getDefaultSummaryFunction is defined
*  for the specified field type.
* @type {boolean}
* @default null
*/
showGridSummary?: boolean;

/**
* If ListGrid.showGroupSummary is true, should this field show a summary value
*  in a summary row when the grid is grouped?
*  If unset, this field will show a summary value in the summary row if an
*  explicit ListGridField.summaryFunction is specified or if a
*  SimpleType.getDefaultSummaryFunction is defined
*  for the specified field type.
* @type {boolean}
* @default null
*/
showGroupSummary?: boolean;

/**
* If ListGrid.showGridSummary or ListGrid.showGroupSummary is true,
*  this attribute can be used to specify
*  an explicit SummaryFunction for calculating the summary value to
*  display.
*  
*  If an array of summaryFunctions is specified, they will be executed in turn and the
*  grid will show multiple summary rows at the grid or group level (or both)
*  containing the resulting values.
* @type {SummaryFunction | Array<Partial<SummaryFunction>>}
* @default null
*/
summaryFunction?: SummaryFunction | Array<Partial<SummaryFunction>>;

/**
* If ListGrid.showGridSummary or ListGrid.showGroupSummary is true and the
*  ListGridField.summaryFunction is set to "title", this attribute may be
*  set to a string to display in the group and/or grid summary. If unspecified the
*  ListGridField.title will show up in the summary.
* @type {string}
* @default null
*/
summaryValueTitle?: string;

/**
* Optional stringMethod to format the summary value displayed
*  in the ListGrid.showGridSummary.
*  Takes a single parameter value and should return the formatted version
*  of that value. If specified this will be applied instead of any formatting logic applied
*  via ListGridField.formatCellValue, ListGrid.formatCellValue, etc.
*  
*  Note that for fields with a specified summary function of "count", if no custom formatting
*  is applied, we default to formatting the count value by appending
*  field.pluralTitle if defined, otherwise field.title to the
*  numeric count value returned by the standard count function. To change this behavior for
*  such fields, specify an explicit 'formatGridSummary' and/or 'formatGroupSummary' method
* @type {StringMethod}
* @default null
*/
formatGridSummary?: StringMethod;

/**
* Optional stringMethod to format the group level summary values for this field displayed via
*  ListGrid.showGroupSummary.
*  Takes a single parameter value and should return the formatted version
*  of that value. If specified this will be applied instead of any formatting logic applied
*  via ListGridField.formatCellValue, ListGrid.formatCellValue, etc.
*  
*  Note that for fields with a specified summary function of "count", if no custom formatting
*  is applied, we default to formatting the count value by appending
*  field.pluralTitle if defined, otherwise field.title to the
*  numeric count value returned by the standard count function. To change this behavior for
*  such fields, specify an explicit 'formatGridSummary' and/or 'formatGroupSummary' method
* @type {StringMethod}
* @default null
*/
formatGroupSummary?: StringMethod;

/**
* Only applies to fields of type "summary".
*  Fields of this type will display a calculated value based on the other field values
*  within the current record.
*  
*  This attribute specifies how the summary field value will be calculated. See
*  RecordSummaryFunction for valid options.
*  
*  A subset of the ListGrid's fields will be passed to the RecordSummaryFunction.
*  
*  The fields to be used for the summary calculation can be determined either by explicitly 
*  listing them (ListGridField.includeInRecordSummaryFields), or by setting 
*  ListGridField.includeInRecordSummary to true on fields that should be included and 
*  false on fields that should not. 
*  
*  By default, all visible fields of numeric type are used. Note that, in this mode, if the 
*  user is able to hide some fields, this could change the formula. If this isn't desirable, 
*  consider setting ListGridField.canHide to prevent fields from being hidden. 
*  Consider this even if fields to include in the summary are explicitly listed, as hidden 
*  fields involved in a visible calculation can be confusing for an end user.
*  
*  If ListGrid.showGridSummary or ListGrid.showGroupSummary is true, this
*  field's value in the summary row[s] will still be calculated by calling this method.
*  In this case, the record object passed in will contain summary values for each field.
*  If custom handling is required for this case, it may be detected by checking the
*  record object's ListGridRecord.isGroupSummary and ListGridRecord.isGridSummary
*  attributes.
* @type {RecordSummaryFunction}
* @default null
*/
recordSummaryFunction?: RecordSummaryFunction;

/**
* Only applies to fields of type "summary".
*  This attribute is set on a summary field, when calculating the summary value from
*  some record, the summary function will only be passed the fields before this summary field.
*  This may be useful for displaying running totals across a record.
*  
*  Note that this feature would typically be used with
*  ListGrid.canReorderFields
* @type {boolean}
* @default null
*/
partialSummary?: boolean;

/**
* If a listGrid is showing a field of type summary, should this field be passed to the
*  recordSummaryFunction when calculating the summary value to display.
*  If unset, fields are included if they are of type "integer" or "float" only (since most
*  summary functions perform numeric calculations). See also
*  ListGridField.includeInRecordSummaryFields.
* @type {boolean}
* @default null
*/
includeInRecordSummary?: boolean;

/**
* If this listGrid has any fields of type "summary" and
*  this field will be ListGridField.includeInRecordSummary in summary calculations
*  by default, this attribute provides an opportunity to explicitly specify which summary fields
*  the record should be displayed in.
*  
*  Specified as an array of fieldNames. If set, this field value will only be included for
*  record summary value calculations for summary fields who's name is included in this array.
* @type {Array<Partial<FieldName>>}
* @default null
*/
includeInRecordSummaryFields?: Array<Partial<FieldName>>;

/**
* If ListGridField.userFormula is set for this field, and this grid is showing
*  ListGrid.showGroupSummary or a
*  ListGrid.showGridSummary, this property determines what field value
*  should be present in those summary rows. Should the field apply the user-formula to the
*  calculated summary row, or should it apply a standard grid or group summary to the
*  user-formula values displayed in the grid?
*  
*  Default behavior may be specified at the grid level via ListGrid.applyFormulaAfterSummary
* @type {boolean}
* @default null
*/
applyAfterSummary?: boolean;

/**
* Optional icon to show next to the title for this field.
*  Should be set to a URL to an image. Relative paths will be evaluated starting at
*  the imgDir of this component. This URL is partial - it may be updated to indicate
*  the current disabled (etc) state of the field.
*  
*  If ListGridField.type is set to "icon", this icon will also be shown
*  in every cell of this field - see also ListGridField.cellIcon.
*  
*  To change this property after fields have been passed to ListGrid.setFields,
*  use ListGrid.setFieldIcon.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* If ListGridField.icon is specified, this property can be used to specify the
*  size of the icon to be displayed in the ListGrid header button.
*  (See StatefulCanvas.iconSize)
* @type {Integer}
* @default null
*/
iconSize?: Integer;

/**
* If ListGridField.icon is specified, this property can be used to specify the
*  width of the icon to be displayed in the ListGrid header button.
*  (See StatefulCanvas.iconWidth)
*  If this field is editable, and ListGridField.editorIconWidth is unset, this
*  property will be passed onto the editors for this field as FormItem.iconWidth,
*  which will effect the default size for ListGridField.icons displayed
*  in the editor.
* @type {Integer}
* @default null
*/
iconWidth?: Integer;

/**
* If ListGridField.icon is specified, this property can be used to specify the
*  height of the icon to be displayed in the ListGrid header button.
*  (See StatefulCanvas.iconHeight)
*  If this field is editable, and ListGridField.editorIconHeight is unset, this
*  property will be passed onto the editors for this field as FormItem.iconWidth,
*  which will effect the default size for ListGridField.icons displayed
*  in the editor.
* @type {Integer}
* @default null
*/
iconHeight?: Integer;

/**
* If this field is showing an icon, should it appear to the left or right of the title?
*  Valid options are "left" or "right"
* @type {string}
* @default "left"
*/
iconOrientation?: string;

/**
* Pixels between icon and title text.
* @type {Integer}
* @default "6"
*/
iconSpacing?: Integer;

/**
* If using an icon for this button, whether to switch the icon image if the button becomes
*  disabled.
* @type {boolean}
* @default true
*/
showDisabledIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image on mouse rollover.
* @type {boolean}
* @default false
*/
showRollOverIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the button
*  receives focus.
*  
*  If StatefulCanvas.showFocusedAsOver is true, the "Over" icon will be
*  displayed when the canvas has focus, otherwise a separate "Focused" icon
*  will be displayed
* @type {boolean}
* @default false
*/
showFocusedIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the mouse goes
*  down on the button.
* @type {boolean}
* @default false
*/
showDownIcon?: boolean;

/**
* If using an icon for this button, whether to switch the icon image when the button
*  becomes selected.
* @type {boolean}
* @default false
*/
showSelectedIcon?: boolean;

/**
* For a field of type:"icon" only, set the icon that appears in body cells. Unless
*  setting ListGridField.icon, setting field.cellIcon will not show an
*  icon in the header.
*  
*  To change this property after fields have been passed to ListGrid.setFields,
*  use ListGrid.setFieldCellIcon.
* @type {SCImgURL}
* @default null
*/
cellIcon?: SCImgURL;

/**
* Specifies the cursor to display when the mouse pointer is over an icon image in a cell
*  for either a field of type ListGridField.icon or a 
*  ListGridField.valueIcons.
*  
*  If not explicitly specified, see ListGrid.getIconCursor for how "icon"
*  fields determine icon image cursors, and ListGrid.getValueIconCursor for how
*  value icon image cursors are determined.
* @type {Cursor}
* @default null
*/
iconCursor?: Cursor;

/**
* For a field of type:"imageFile", indicates whether to stream the image and display it
*  inline or to display the View and Download icons.
* @type {boolean}
* @default null
*/
showFileInline?: boolean;

/**
* Should ellipses be displayed when this field's cell content is clipped? To set this 
*  property at the grid level, use ListGrid.showEllipsisWhenClipped
* @type {boolean}
* @default null
*/
showEllipsisWhenClipped?: boolean;

/**
* FormatString for numeric or date formatting. See DataSourceField.format.
* @type {FormatString}
* @default null
*/
format?: FormatString;

/**
* FormatString used during exports for numeric or date formatting. See
*  DataSourceField.exportFormat.
* @type {FormatString}
* @default null
*/
exportFormat?: FormatString;

/**
* If this field is editable, this property can be used to specify
*  FormItem.icons to be displayed in the editors displayed for this field
* @type {Array<Partial<Partial<FormItemIcon>>>}
* @default null
*/
icons?: Array<Partial<Partial<FormItemIcon>>>;

/**
* If this field is editable, this property will be passed to editors displayed for
*  cells within this field as FormItem.iconWidth.
*  If this property unset, the iconWidth property from the editor can be picked up from
*  ListGridField.iconWidth instead.
* @type {number}
* @default null
*/
editorIconWidth?: number;

/**
* If this field is editable, this property will be passed to editors displayed for
*  cells within this field as FormItem.iconHeight.
*  If this property unset, the iconHeight property from the editor can be picked up from
*  ListGridField.iconHeight instead.
* @type {number}
* @default null
*/
editorIconHeight?: number;

/**
* If this field is editable, this property will be passed to editors displayed for
*  cells within this field as FormItem.defaultIconSrc.
* @type {string}
* @default null
*/
defaultIconSrc?: string;

/**
* If this field is editable, this property will be passed to editors displayed for
*  cells within this field as FormItem.iconVAlign.
* @type {string}
* @default null
*/
iconVAlign?: string;

/**
* Optional long summary title for this field, provided in addition to
*  ListGridField.title. This gives the developer an option to use a very short,
*  or empty title for the ListGrid column (where space may be a factor), but have a longer
*  value available to be used elsewhere.
*  
*  By default this value will be used for the title of the context-menu item
*  for showing/hiding the listGrid field when the user right-clicks on the ListGrid header.
* @deprecated * Rather than customizing the summaryTitle developers should typically use the ListGridField.headerTitle attribute to show a different
*  title in the column header button than the title used elsewhere.

* @type {string}
* @default null
*/
summaryTitle?: string;

/**
* The width of this field, specified as either an absolute number of pixels,
*  a percentage of the remaining space like "25%", or "*" to split remaining space among
*  all fields which have "*". 
*  Caution: stretch sizes are currently ignored if the field is being autofitted
*  (see ListGrid.autoFitFieldWidths), unless ListGrid.showHeader is false.
*  
*  Note: if autofitting is active for a field, the width will default to the numerical
*  autofit width for that field (so it will not be stretched larger to fill available
*  space). Otherwise, if not autofitting, the width will default to "*" causing it to be
*  automatically stretched.
*  
*  The width may be defaulted to a numerical value based on DataSourceField.length
*  if no ListGridField.valueMap is set, subject to the initial values of
*  ListGridField.minWidth and ListGrid.minFieldWidth. If you'd rather have the field
*  stretched-sized to fit the available space, set its initial width to "*".
*  
*  See also ListGrid.minFieldWidth to ensure no field goes below a minimum size.
*  
*  Use ListGrid.resizeField to programmatically change field width after creation.
*  
*  Use ListGrid.getFieldWidth to access the rendered field width after
*  the ListGrid is drawn.
* @type {number | string}
* @default "*"
*/
width?: number | string;

/**
* When a field is subject to autofitting (see ListGrid.autoFitFieldWidths), sets the
*  minimum width of the field. The actual allowed minimum will be the maximum of:
*  
* - this property,
*  
* - ListGridField.width (if a number),
*  
* - the aufofit value determined by the widest value content in this field's column
*  
* - ListGrid.minFieldWidth
*  
* @type {number}
* @default null
*/
minWidth?: number;

/**
* When ListGrid.showHeader is false and a field is subject to autofitting (see
*  ListGrid.autoFitFieldWidths), sets the maximum width of the field. The actual
*  effective maximum will be the largest of this property, ListGridField.minWidth, and 
*  ListGrid.minFieldWidth. That is, ListGridField.minWidth and 
*  ListGrid.minFieldWidth dominate this property.
* @type {number}
* @default null
*/
maxWidth?: number;

/**
* Horizontal alignment for field's column header: "left", "right"
*  or "center". Applied to the column header title and cells by default. A separate
*  alignment for cells can be specified via ListGridField.cellAlign.
*  
*  If null, the default alignment depends on the field's declared ListGridField.type - generally
*  "left" except for numbers which are "right" - and if ListGridField.rotateTitle has been
*  specified, the default is always "center".
*  
*  Note that if this field is editable, the alignment of cells in the body will also be
*  reflected in any editors for the field.
* @type {Alignment}
* @default null
*/
align?: Alignment;

/**
* Custom base style to apply to this field's header button instead of
*  ListGrid.headerBaseStyle.
*  Note that depending on the header button constructor, you may have to override
*  ListGridField.headerTitleStyle as well.
* @type {CSSStyleName}
* @default null
*/
headerBaseStyle?: CSSStyleName;

/**
* Custom base style to apply to this field's header button instead of
*  ListGrid.spannedHeaderBaseStyle when the grid is showing header spans.
* @type {CSSStyleName}
* @default null
*/
spannedHeaderBaseStyle?: CSSStyleName;

/**
* Custom titleStyle to apply to this field's header button instead of
*  ListGrid.headerTitleStyle.
*  Note that this will typically only have an effect if
*  ListGrid.headerButtonConstructor is set to StretchImgButton or a subclass
*  thereof.
* @type {CSSStyleName}
* @default null
*/
headerTitleStyle?: CSSStyleName;

/**
* Optional title for the header button for this field. If specified this will be
*  displayed in the header button instead of ListGridField.title or
*  ListGridField.name. Set to an empty string to suppress the title in the
*  header button entirely.
* @type {string}
* @default null
*/
headerTitle?: string;

/**
* Horizontal alignment for cells in this field's column: "left", "right"
*  or "center".
*  If null, alignment is derived from ListGridField.align. If this field is editable,
*  the alignment of cells in the body will also be reflected in any editors for the field.
* @type {Alignment}
* @default null
*/
cellAlign?: Alignment;

/**
* Custom base style to apply to all cells in this field instead of ListGrid.baseStyle
*  To override the baseStyle at the row level, use
*  ListGrid.recordBaseStyleProperty
*  instead.
* @type {CSSStyleName}
* @default null
*/
baseStyle?: CSSStyleName;

/**
* Enables or disables sorting by this column. If false, interactive
*  sorting via header-clicks or menu-items will have no effect, but direct scripted calls
*  to ListGrid.sort or ListGrid.setSort will work as
*  expected.
* @type {boolean}
* @default true
*/
canSort?: boolean;

/**
* Specifies the default sorting direction for this column. If specified on the
*  ListGrid.sortField for the listGrid, sorting occurs
*  automatically, otherwise this will be the default direction when the user clicks the
*  field header, or calls ListGrid.sort without specifying an explicit sort
*  direction.
*  
*  Overrides ListGrid.sortDirection
* @type {SortDirection}
* @default null
*/
sortDirection?: SortDirection;

/**
* Should the cell content be natively selected (ready for copying to clip-board) 
*  on click?
*  
*  See ListGrid.selectCellTextOnClick for more information.
* @type {boolean}
* @default null
*/
selectCellTextOnClick?: boolean;

/**
* This property establishes default editability for the field. May be overridden by setting
*  the 'canEdit' property at the listGrid level. If not explicitly set and this grid is bound
*  to a dataSource, the ListGrid.canEditFieldAttribute may be used to set default
*  editability at the field level.
*  
*  An override of ListGrid.canEditCell can be used for more dynamic control over whether
*  fields can be edited.
* 
*  
* @type {boolean}
* @default null
*/
canEdit?: boolean;

/**
* If this field ListGridField.canEdit, this property can be used to
*  specify a default value for this field's editor when adding new rows to the grid.
* @type {any}
* @default null
*/
defaultValue?: any;

/**
* What to do when a user hits enter while editing this field?
*  Overrides the enterKeyEditAction as specified at the listGrid level while
*  focus is in this field.
* @type {EnterKeyEditAction}
* @default "done"
*/
enterKeyEditAction?: EnterKeyEditAction;

/**
* What to do when a user hits escape while editing this field?
*  Overrides the escapeKeyEditAction as specified at the listGrid level while
*  focus is in this field.
* @type {EscapeKeyEditAction}
* @default "cancel"
*/
escapeKeyEditAction?: EscapeKeyEditAction;

/**
* What to do when a user hits arrow key while editing this field?
*  See ListGrid.getArrowKeyEditAction.
* @type {ArrowKeyEditAction}
* @default null
*/
arrowKeyEditAction?: ArrowKeyEditAction;

/**
* Name of form item class to use for the form item created to edit this field.
*    (Only used if this field is editable).
*    Note: If this is not specified, the edit-form item type may be derived from the
*    editorType property, typically inherited from datasource fields, or
*    from the type of the field (showing the appropriate form item for
*    the data-type). See the editing overview for more on editing ListGrid
*    fields.
* @type {FormItemClassName}
* @default null
*/
editorType?: FormItemClassName;

/**
* Properties to apply the the form item created to edit this field. (Only used if
*  this field is editable).
*  
*  For example, if you have a field "shoeSize" with DataSourceField.editorType set
*  to "SpinnerItem" in order to use a SpinnerItem as your field editor, and you want to pass the
*  SpinnerItem.step property to the created SpinnerItem:
*  
*   fields : [
*     { name:"shoeSize", editorType:"SpinnerItem",
*      editorProperties : { step:0.5 } },
*     ... other fields ...
*   ]
*  
* @type {Partial<FormItem>}
* @default null
*/
editorProperties?: Partial<FormItem>;

/**
* In an editable ListGrid, initial value for this field when the user begins editing a new
*  record.
*  
*  initialValue applies only if a new record is created by end user action (such
*  as navigating past the end of the data when ListGrid.listEndEditAction is "next") or
*  by a call to ListGrid.startEditingNew that does not specify the
*  initialValues argument.
* @type {any}
* @default null
*/
initialValue?: any;

/**
* Array of Validator objects for this field. When the user edits cells in
*  this field, these validators will be applied to the edited value.
*  Note: for databound listGrids, this property may be specified on the
*  DataSourceField, enabling both client and server side validation.
* @type {Array<Partial<Validator>>}
* @default null
*/
validators?: Array<Partial<Validator>>;

/**
* If set to true, any ListGridField.validators for this field will be run whenever
*  the value of the field is changed.
*  
*  Analogous to the FormItem.validateOnChange property.
* @type {boolean}
* @default null
*/
validateOnChange?: boolean;

/**
* When the user edits cells in this field, is this value required to be non-empty
*  in order for validation to pass.
*  Note: for databound listGrids, this property may be specified on the
*  DataSourceField, enabling both client and server side validation.
* @type {boolean}
* @default null
*/
required?: boolean;

/**
* For fields of type "date", set this property to a valid
*  DateDisplayFormat to specify how the date should be formatted.
*  For fields of type "time", set this property to a valid
*  TimeDisplayFormat to specify how the time should be formatted.
*  If unset, display format may be set at the listGrid level via ListGrid.dateFormatter
*  or ListGrid.timeFormatter.
*  
*  If this field is editable the displayFormat will also be passed to the editor created
*  to edit this field. For dates you may also need to set ListGridField.inputFormat.
* @deprecated * Use ListGridField.dateFormatter and ListGridField.timeFormatter instead.

* @type {any}
* @default null
*/
displayFormat?: any;

/**
* Display format to use for date type values within this field.
*  
*  The ListGridField.timeFormatter may also be used to format underlying Date values as
*  times (omitting the date part entirely). If both dateFormatter and
*  timeFormatter are specified on a field, for
*  fields specified as ListGridField.type the
*  timeFormatter will be used, otherwise the dateFormatter
*  
*  If field.dateFormatter and field.timeFormatter is unspecified,
*  date display format may be defined at the component level via
*  ListGrid.dateFormatter, or for fields of type "datetime"
*  ListGrid.datetimeFormatter. Otherwise the
*  default is to use the system-wide default short date format, configured via
*  DateUtil.setShortDisplayFormat. Specify any valid DateDisplayFormat to
*  change the format used by this item.
*  
*  If this field is editable the dateFormatter will also be passed to the editor created
*  to edit this field. Note that you can also specify an explicit ListGridField.inputFormat
*  which will be passed through to the editor as well, though this is not typically required
*  as the input format should be automatically derived by the SmartClient system
*  for standard DateDisplayFormats.
* @type {DateDisplayFormat}
* @default null
*/
dateFormatter?: DateDisplayFormat;

/**
* Time-format to apply to date type values within this field. If specified, any
*  dates displayed in this field will be formatted as times using the appropriate format.
*  This is most commonly only applied to fields specified as type "time" though
*  if no explicit ListGridField.dateFormatter is specified it will be respected for other
*  fields as well.
*  
*  If unspecified, a timeFormatter may be defined
*  ListGrid.timeFormatter and will be respected by fields
*  of type "time".
*  
*  If this field is editable, the timeFormatter will also be passed to the editor created to
*  edit this field as FormItem.timeFormatter.
* @type {TimeDisplayFormat}
* @default null
*/
timeFormatter?: TimeDisplayFormat;

/**
* Applies only to fields of type "float" and affects how many significant digits are shown.
*  
*  For example, with decimalPrecision 3, if the field value is 343.672677, 343.673 is shown.
*  
*  If the value is 125.2, 125.2 is shown - decimalPrecision will not cause extra zeros to be
*  added. Use DataSourceField.decimalPad for this.
*  
*  A number is always shown with its original precision when edited.
* @type {number}
* @default null
*/
decimalPrecision?: number;

/**
* Applies only to fields of type "float" and enforces a minimum number of digits shown after
*  the decimal point.
*  
*  For example, a field value of 343.1, 343.104 and 343.09872677 would all be shown as 343.10
*  if decimalPad is 2.
*  
*  The original unpadded value is always shown when the value is edited.
* @type {number}
* @default null
*/
decimalPad?: number;

/**
* For fields of type "date" or "datetime", if this is an editable
*  listGrid, this property allows you to specify the DateItem.inputFormat
*  applied to the editor for this field.
* @type {DateInputFormat}
* @default null
*/
inputFormat?: DateInputFormat;

/**
* If set to true and ListGrid.canRemoveRecords is true, this field will be rendered
*  as the remove-field for this grid. In most common usage scenarios this field will essentially
*  be a placeholder indicating where the remove field should be rendered, meaning properties
*  other than isRemoveField, such as name or title, may
*  be left unset.
* @type {boolean}
* @default null
*/
isRemoveField?: boolean;

/**
* If showing a filter row for this ListGrid, should the filter criteria for this
*    field be editable
* @type {boolean}
* @default null
*/
canFilter?: boolean;

/**
* If this listGrid is showing a filter row, this property can be used to specify a
*  mapping of internal data to/from display values to be in the appropriate filter
*  row form item.
* @type {object}
* @default null
*/
filterEditorValueMap?: object;

/**
* If this ListGrid is showing a filter row, this property can be used to
*    specify the form item class to use for the filter form item associated with this
*    field
*    (Only used if this field is not canFilter:false).
*    Note: If this is not specified, the edit-form item type may be derived from the
*    'editorType' property, typically inherited from datasource fields, or from the
*    'type' of the field (showing the appropriate form item for the data-type).
* @type {FormItemClassName}
* @default null
*/
filterEditorType?: FormItemClassName;

/**
* If this ListGrid is showing a filter row, this property can be used to apply a default
*  value to show in the filter editor for this field.
* @type {any}
* @default null
*/
defaultFilterValue?: any;

/**
* If this ListGrid is showing a filter row
*  (ListGrid.showFilterEditor:true), this property
*  can be used to specify properties for the appropriate filter form item.
* @type {Partial<FormItem>}
* @default null
*/
filterEditorProperties?: Partial<FormItem>;

/**
* When ListGrid.allowFilterOperators is true, this read-only
*  property is set to the 
*  Operator currently assigned to this field. You can also
*  retrieve a field's current operator by calling ListGrid.getFieldSearchOperator.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use
*  them, see the gridFiltering.
*  
*  While this property can be affected by users, via the
*  ListGrid.filterUsingText submenu of the grid's 
*  ListGrid.showHeaderContextMenu, it can't be set directly in 
*  code.
*  
*  However, developers can call ListGrid.setFieldSearchOperator to modify a field's 
*  current operator, or ListGrid.clearFieldSearchOperator to reset a field to it's 
*  default operator.
*  
*  To specify a particular default operator for a field, see 
*  ListGridField.filterOperator.
* @type {OperatorId}
* @default null
*/
operator?: OperatorId;

/**
* If set this will override the 
*  ListGrid.filterOnKeypress
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use
*  them, see the gridFiltering.
* @type {boolean}
* @default null
*/
filterOnKeypress?: boolean;

/**
* When set to false, no hover is shown for the field editor in this field. Otherwise,
*  a hover shows the current field's criteria description along with the
*  ListGrid.filterWindowCriteria description if configured.
*  
*  All hovers for the filter editor can be disabled using
*  ListGrid.showFilterEditorHovers.
*  
*  The descriptive text for criteria is formatted by DataSource.getAdvancedCriteriaDescription.
* @type {boolean}
* @default null
*/
showFilterEditorHovers?: boolean;

/**
* Whether this field should be included in the printable representation of the grid.
* @type {boolean}
* @default null
*/
shouldPrint?: boolean;

/**
* Whether to allow browser autoComplete when editing this field.
*  
*  If unset, defaults to listGrid.autoComplete
* @type {AutoComplete}
* @default null
*/
autoComplete?: AutoComplete;

/**
* By default HTML values in ListGrid cells will be interpreted by the browser.
*  Setting this flag to true will causes HTML characters to be escaped, meaning the
*  raw value of the field (for example "&lt;b&gt;AAA&lt;/b&gt;") is displayed
*  to the user rather than the interpreted HTML (for example "AAA")
* @type {boolean}
* @default null
*/
escapeHTML?: boolean;

/**
* By default, when ListGridField.escapeHTML, we convert line breaks (\r\n, \r, and
*  \n) to HTML &lt;br&gt tags so that visible cell content respects the original break
*  characters. Set this property true to instead show the content as a single line (or
*  potentially wrapped to avoid clipping if ListGrid.wrapCells is true).
*  
*  If defaulted to null, behavior is determined by ListGrid.skipLineBreaks.
* @type {boolean}
* @default null
*/
skipLineBreaks?: boolean;

/**
* The HTML to display in cells of this field if the fieldType is set to link.
*  
*  This property sets linkText that will be the same for all records. You can set linkText
*  on a per-record basis via ListGridRecord.linkText.
* @type {string}
* @default null
*/
linkText?: string;

/**
* Name of the property in a ListGridRecord that holds the HTML to display in cells of this
*  field if the fieldType is set to "link".
* @type {string}
* @default null
*/
linkTextProperty?: string;

/**
* If this field has type [ListGridFieldType] set to "link",
*  setting this property will apply a standard prefix to the link URL for cells in this field.
* @type {string}
* @default null
*/
linkURLPrefix?: string;

/**
* If this field has type [ListGridFieldType] set to "link",
*  setting this property will apply a standard suffix to the link URL for cells in this field.
* @type {string}
* @default null
*/
linkURLSuffix?: string;

/**
* Array of legal values for this field, or an Object where each property maps a stored
*  value to a user-displayable value.
*  Note that if this field is editable (see ListGrid.canEdit,
*  ListGridField.canEdit), editors displayed for this field will pick up their
*  valueMap either from this value or from ListGridField.editorValueMap.
*  
*  See also DataSourceField.valueMap.
* @type {object | Array<Partial<string>>}
* @default null
*/
valueMap?: object | Array<Partial<string>>;

/**
* If ListGridField.valueMap is set, and the grid is ListGrid.setSort
*  by this field, should the data be sorted by the underlying data value or the
*  mapped display value. If unset, will sort by display value. Set to false
*  to sort by underlying data value. Note that this has no effect if
*  a ListGridField.sortNormalizer has been specified.
* @type {boolean}
* @default null
*/
sortByMappedValue?: boolean;

/**
* Indicates that this field should always be Array-valued. This property will be
*  passed through to the generated edit-item when editing the field - so if
*  ListGridField.valueMap is set, the default editor will be a SelectItem with
*  SelectItem.multiple set to true.
*  
*  Note that for databound grids it typically makes sense to set DataSourceField.multiple
*  rather than setting the property directly on the ListGridField object.
* @type {boolean}
* @default null
*/
multiple?: boolean;

/**
* A valueMap to use for editors shown for this field. By default if this is not
*  specified ListGridField.valueMap will be used instead.
*  
*  Dynamic valueMaps can be provided by implementing ListGrid.getEditorValueMap.
* @type {ValueMap}
* @default null
*/
editorValueMap?: ValueMap;

/**
* The value to display for a cell whose value is null or the empty string after
*  applying ListGridField.formatCellValue and valueMap (if any).
*  
*  This is the field-specific attribute. You may also set the emptyCellValue at the grid
*  level to define the emptyCellValue for all empty fields in the grid.
* @type {HTMLString}
* @default "&amp;nbsp;"
*/
emptyCellValue?: HTMLString;

/**
* If true, automatically fetches records and derives a valueMap from
*  ListGridField.optionDataSource.
*  
*  Same as ListGrid.autoFetchDisplayMap, but defined on a per-field basis.
* @type {boolean}
* @default null
*/
autoFetchDisplayMap?: boolean;

/**
* If a ListGridField.displayField is set, should this field show record values from
*  the displayField?
*  
*  If ListGridField.displayField is specified, and there is no separate
*  ListGridField.optionDataSource, by default we will show display-field values
*  from the same record. Setting this property to false would disable this behavior.
*  
*  Alternatively, if there is a ListGridField.optionDataSource (and
*  ListGridField.autoFetchDisplayMap is false), the displayField would be ignored
*  for the field and the underlying ListGridField.name would
*  be displayed to the user. This property may be set to true to override this behavior and
*  pick up values from the displayField for display in this field even when there
*  is an optionDataSource set.
*  
*  Note that this property has no effect on fields with an explicitly specified valueMap, or
*  with an optionDataSource where ListGridField.autoFetchDisplayMap is true.
* @type {boolean}
* @default null
*/
displayValueFromRecord?: boolean;

/**
* For fields with an ListGridField.optionDataSource, where
*  ListGridField.autoFetchDisplayMap is true, this property will govern
*  the textMatchStyle attribute of the DSRequest parameter passed to
*  DataSource.fetchData when retrieving the remote data set to be used as
*  a basis for this field's valueMap.
* @type {TextMatchStyle}
* @default null
*/
optionTextMatchStyle?: TextMatchStyle;

/**
* If this field has an optionDataSource specified and
*  ListGridField.autoFetchDisplayMap is set, this attribute
*  provides a way to customize the dataSource request issued to fetch the display map from
*  the option dataSource. This provides, among other capabilities, a way to trigger the
*  server to return summary records.
* @type {Partial<DSRequest>}
* @default null
*/
optionFilterContext?: Partial<DSRequest>;

/**
* If this field has an optionDataSource specified and
*  ListGridField.autoFetchDisplayMap is set, this attribute
*  provides a way to customize the DSRequest.operationId passed to
*  dataSource.fetchData() when retrieving the display map from the option
*  dataSource.
* @type {string}
* @default null
*/
optionOperationId?: string;

/**
* Derive a ValueMap by fetching records from another DataSource and extracting
*  the ListGridField.valueField and
*  ListGridField.displayField in the loaded records, to derive one
*  valueMap entry per record loaded from the optionDataSource.
*  
*  Unlike the similar use of PickList.optionDataSource for PickList
*  used during editing or filtering, listGridField.optionDataSource causes the
*  entire set of records from the optionDataSource to be fetched, without paging.
*  Hence listGridField.optionDataSource is appropriate only for smaller valueMaps, and in 
*  this situation, DataSource.cacheAllData may be a better choice, since it creates a
*  write-through cache usable across all components.
*  
*  For very large valueMap situations, such as an accountId field that should be displayed 
*  as an accountName where there are thousands of accounts, the correct approach is:
*  
*  
* - do not set listGridField.optionDataSource
*  
* - declare two fields in the DataSource, eg "accountId" and "accountName".
*  
* - Set the ListGridField.displayField attribute on the data field to the
*    name of the display field.
*  
* - When fetching records for display in a grid, have your server send back values for
*    both fields, but show only the data field ("accountId") in the grid.
*  
*  In this case the cells in the accountId field will show the record value from the
*  accountName field. This approach means the valueMap will never be loaded in its
*  entirety, instead, each loaded record contains the valueMapping for that one record, as
*  a pair of fields within the record.
*  
*  If you are using the SmartClient Server Framework with the SQL or JPA/Hibernate built-in
*  connectors, this entire approach can be achieved very easily using the
*  DataSourceField.includeFrom setting - see the
*  DataSourceField.includeFrom for details.
*  
*  Notes:
*  
*  
* - When using the above approach, it is key that the server return both
*  the underlying stored value and the display value, as suggested above.
*  This approach allows the PickList.optionDataSource property to be used to
*  provide paged valueMaps during inline editing and
*  ListGrid.showFilterEditor. This can be achieved by setting the
*  optionDataSource attribute on the form item used to edit the field
*  via ListGridField.editorProperties (for editing) or
*  ListGridField.filterEditorProperties (for
*  filtering), without specifying an optionDataSource at the listGridField level.
*  Alternatively developers can use ListGridField.autoFetchDisplayMap to
*  suppress the fetch against the optionDataSource at the listGrid level.
*  
* - Setting listGridField.optionDataSource to the same dataSource as the
*  listGrid is not the same as omitting the optionDataSource setting entirely. Unless
*  ListGridField.autoFetchDisplayMap has been set to explicitly disable fetching,
*  a fetch will be performed against the dataSource to build a valueMap which will be used
*  as the definitive mapping from data to display values, rather than picking up the display
*  values from the records themselves. This distinction is required to support cases where
*  the ListGridField.valueField points to a different field in the grid (useful for
*  hierarchical relationships, for example), or where ListGridField.optionCriteria or
*  ListGridField.optionOperationId are specified and return different data from
*  the records displayed within the grid.
*  
* - If a displayField is specified, with no associated optionDataSource, and the field is
*  editable, updating the edit value for the field may not automatically update the displayField
*  edit value, meaning the user may not realize the edit value has been modified.
*  If the new value came from the user editing the field, and the edit item has
*  a valueMap or optionDataSource specified, the display value is picked up automatically and
*  stored out on the displayField for the record. However if the value was set programmatically,
*  the developer should also set the edit value for the display field to ensure the displayed
*  value reflects the new edit value. Note that when this occurs, a warning will be logged
*  which can be disabled via ListGrid.warnOnUnmappedValueFieldChange.
*  
* - For very advanced usage a developer can use ListGridField.displayValueFromRecord
*  to explicitly tell the grid whether or not to display the display field value for the record
*  in this field when a displayField is specified. See documentation on that property for more
*  information
*  
* @type {string}
* @default null
*/
optionDataSource?: string;

/**
* Specifies the ListGridField.optionDataSource field used to retrieve the stored
*  values that are to be mapped to the display values (specified by
*  ListGridField.displayField). Note that if this field is editable this will also
*  be applied to this field's editors.
* @type {string}
* @default null
*/
valueField?: string;

/**
* Specifies the ListGridField.optionDataSource field used to retrieve the display
*  values that are to be mapped from the internal values specified by ListGridField.valueField.
*  
*  If no optionDataSource is defined for the field, the cell will display
*  the displayField value for the current record instead of the underlying value for
*  this field. This approach can be used for situations where field values need a stored
*  value to displayed value mapping, but the set of all possible values is too large to
*  load as a ValueMap - see ListGridField.optionDataSource for more details
*  on this approach. Note that if this field is editable this will also be applied to this
*  field's editors. ${isc.DocUtils.linkForExampleId('largeValueMapSQL', 'This sample')} illustrates this
*  approach achieved via a server-side SQL join.
*  
*  The display value for a record with a specified displayField can be
*  picked up via ListGrid.getDisplayValue.
* @type {string}
* @default null
*/
displayField?: string;

/**
* For a field with displayField configured, should client-side sorting
*  be performed on the display field value? Unless explicitly set to false
*  the display field value is used.
* @type {boolean}
* @default null
*/
sortByDisplayField?: boolean;

/**
* If ListGridField.optionDataSource is set for this ListGridField, criteria specified in this
*  attribute will be passed to the dataSource when performing the fetch operation to
*  determine data-value to display-value mappings
*  
*  See also ListGridField.optionTextMatchStyle.
* @type {Criteria}
* @default null
*/
optionCriteria?: Criteria;

/**
* Indicates this field's values should be fetched from another, related DataSource.
*  The individual field will inherit settings such as ListGridField.type
*  and ListGridField.title from the related DataSource just like
*  fields from the primary DataSource.
*  
*  When ListGrid.fetchData is called, the automatically created DSRequest will
*  specify DSRequest.additionalOutputs requesting the field, and any Criteria
*  generated by the component will likewise refer to the field from the related DataSource.
*  
*  It's an error to use this property if the ListGrid does not have a DataSource at all.
*  The related DataSource must be loaded or a warning will be logged and the field
*  definition ignored.
*  
*  This value is expected to be set to the following format
*  dataSourceID.fieldName where dataSourceID is the ID of the
*  related dataSource and fieldName is the name of the field from that dataSource
*  from which you wish to retrieve values. Note that if this property is set and
*  ListGridField.name is not explicitly specified, this field's name will
*  default to the fieldName value from this property.
*  
*  Note about automatic cache updates: "update" and "add" operations
*  submitted automatically by this ListGrid will include
*  dsRequest.additionalOutputs to ensure all data necessary
*  for cache updates is returned by the server.
*  
*  If your grid shows data that can be edited elsewhere in the UI (not by
*  inline editing), to avoid problems with ResultSet automatic
*  cache synchronization, you may need to switch from using
*  listGridField.includeFrom to
*  dataSourceField.includeFrom. This is because
*  server responses to "add" and "update" operations which are initiated
*  outside of this grid do not know about the
*  listGridField.includeFrom setting, and so will not
*  automatically return data for fields included in this way. Switching
*  to dataSourceField.includeFrom ensures the field is
*  always included in server responses, avoiding the issue.
* @type {string}
* @default null
*/
includeFrom?: string;

/**
* Should the user be allowed to perform one-time autofitting of this field via
*  a header context-menu option?
*  
*  When enabled, the default header context menu for this field will
*  include an item to auto-fit the field and users will be able to autofit the field
*  via the ListGrid.headerAutoFitEvent.
*  
*  If unset, these behaviors are enabled when ListGrid.canAutoFitFields is true.
*  
*  If this property is set to false, and ListGrid.canAutoFitFields is true,
*  this field will be ommitted from auto-fit when the user selects the header menu 
*  option to ListGrid.autoFitAllText.
*  
*  Note - this property governs user-initiated auto-fit only. It has no impact on
*  autoFit set up via ListGridField.autoFitWidth and ListGrid.autoFitFieldWidths.
*  
*  Note that if ListGrid.showRecordComponents, per-cell record
*  components are not taken into account when determining the size for column auto fit.
*  The default ListGrid.getDefaultFieldWidth implementation looks at cell content
*  only. We typically recommend that, for fields showing record-components, 
*  ListGridField.autoFitWidth and ListGrid.canAutoFitFields be disabled, or if 
*  the record components are of a predictable size, a ListGridField.defaultWidth
*  be specified.
*  This is particularly pertinent where ListGrid.recordComponentPosition
*  is set to "within", in which case cells' content is often empty or completely covered
*  by record-components.
* @type {boolean}
* @default null
*/
canAutoFitWidth?: boolean;

/**
* Should this listGrid field autofit its width to either titles or content?
*  
*  This overrides the ListGrid.autoFitFieldWidths attribute on a per-field basis.
*  
*  Note that if ListGrid.showRecordComponents, per-cell record
*  components are not taken into account when determining the size for column auto fit.
*  The default ListGrid.getDefaultFieldWidth implementation looks at cell content
*  only. We typically recommend that, for fields showing record-components, 
*  ListGridField.autoFitWidth and ListGridField.canAutoFitWidth be disabled, or if 
*  the record components are of a predictable size, a ListGridField.defaultWidth
*  be specified.
*  This is particularly pertinent where ListGrid.recordComponentPosition
*  is set to "within", in which case cells' content is often empty or completely covered
*  by record-components.
* @type {boolean}
* @default null
*/
autoFitWidth?: boolean;

/**
* Optional "default width" for this field. If set, this value will be returned by the
*  ListGrid.getDefaultFieldWidth method, and used as the autoFit size for
*  the field's content.
* @type {Integer}
* @default null
*/
defaultWidth?: Integer;

/**
* When a user requests column autofitting via the
*  ListGrid.getHeaderContextMenuItems or via a
*  ListGrid.headerAutoFitEvent, what autofit approach is used. If set, this
*  setting overrides the autoFitWidthApproach specified at the ListGrid level.
* @type {AutoFitWidthApproach}
* @default null
*/
autoFitWidthApproach?: AutoFitWidthApproach;

/**
* If set to true, custom HTML applied as part of hiliting will be applied after
*  ListGrid.formatCellValue for each cell in this column. If false, hilite
*  HTML will be applied before formatting.
*  
*  This attribute overrides ListGrid.hiliteHTMLAfterFormat as defined at the
*  component level.
* @type {boolean}
* @default null
*/
hiliteHTMLAfterFormat?: boolean;

/**
* Whether to show hovers for this field. The default hover will be the contents of the
*  cell the user is hovering over, and can be customized via
*  ListGridField.hoverHTML.
*  
*  ListGrid.showHover can be set to true to cause hovers to be shown for all fields
*  by default. In this case, field.showHover can be set to false to suppress
*  hovers for an individual field.
*  
*  All hovers can be disabled, regardless of other settings, by setting
*  ListGrid.canHover to false.
* @type {boolean}
* @default null
*/
showHover?: boolean;

/**
* When set to true and showHover is also true for the field, shows a widget hovering at 
*  the mouse point.
*  
*  A number of builtin modes are provided - see HoverMode.
*  
*  Also supported at the ListGrid.showHoverComponents.
* @type {boolean}
* @default null
*/
showHoverComponents?: boolean;

/**
* Causes a tooltip hover to appear on the header generated for this field (effectively
*  sets Canvas.prompt for the header).
* @type {HTMLString}
* @default null
*/
prompt?: HTMLString;

/**
* Per-field setting for ListGrid.allowFilterOperators. Can be used to enable
*  the filter operators UI for a particular field if the ListGrid-level setting is not
*  enabled, or to disable filter operators for a particular field if the ListGrid-level setting
*  is enabled.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use
*  them, see the gridFiltering.
* @type {boolean}
* @default null
*/
allowFilterOperators?: boolean;

/**
* Field-level setting for ListGrid.allowFilterExpressions:true - controls whether search 
*  expressions can be entered directly into the filter item for this field. You can also have
*  parsing of the expression remove the operator symbol and apply it to an 
*  ListGrid.allowFilterOperators in the editor.,
*  
*  This attribute can also be set at the ListGrid.allowFilterExpressions.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use
*  them, see the gridFiltering.
* @type {boolean}
* @default null
*/
allowFilterExpressions?: boolean;

/**
* Per-field setting for ListGrid.alwaysShowOperatorIcon. Can be used to force a particular
*  field to always show it's ListGrid.operatorIcon, even if it has no 
*  filter-value, or is using the default ListGridField.filterOperator.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use
*  them, see the gridFiltering.
* @type {boolean}
* @default null
*/
alwaysShowOperatorIcon?: boolean;

/**
* With the ListGrid.showFilterEditor showing, the default Operator 
*  to use when matching values for this field.
*  
*  For a discussion of the various filtering and criteria-management APIs and when to use
*  them, see the gridFiltering.
*  
*  Users can override this default operator, or revert to it, with the 
*  ListGrid.filterUsingText submenu, in the grid's 
*  ListGrid.showHeaderContextMenu; developers can use 
*  ListGrid.setFieldSearchOperator to modify the operator and 
*  ListGrid.clearFieldSearchOperator to reset to the default.
*  
*  Note that you can set all FilterEditor fields to default to either substring or exact
*  match via ListGrid.autoFetchTextMatchStyle, but if you 
*  want a mix of exact vs substring match on different fields, you need to use this 
*  property, and your ListGrid will produce AdvancedCriteria rather than the 
*  simpler Criteria format. This is automatically and transparently handled by the 
*  SmartClient Server's SQLDataSource and HibernateDataSource in Power Edition or above, 
*  but if you have your own filtering implementation, it will need to be able to handle 
*  AdvancedCriteria.
* @type {OperatorId}
* @default null
*/
filterOperator?: OperatorId;

/**
* Allows a boolean or ValueMap field to be edited without going into
*  edit mode. When this attribute is set, clicking on the field will change the value - for
*  boolean fields toggling between true and false, and for valueMapped
*  fields, advancing the value to the next option in the valueMap.
*  
*  To enable this feature, ListGrid.canEdit must be set to true.
*  For boolean type fields canToggle is true by default, meaning setting
*  canEdit to true implies the user can toggle the value via a single click
*  without going into edit mode. You can disable this by explicitly setting
*  canToggle to false for a boolean field.
*  Note that you can enable toggling only (without allowing the user to edit other fields)
*  by just setting ListGrid.editEvent.
*  
*  If ListGrid.editEvent is set to "click", when the user clicks on the field,
*  the value will be toggled, and inline editing will be triggered as usual.
*  Otherwise the toggled value will be saved immediately to the server, or if
*  ListGrid.autoSaveEdits has been set to false, will be stored as an edit value
*  for the record.
* @type {boolean}
* @default "varies"
*/
canToggle?: boolean;

/**
* If ListGrid.showHeaderMenuButton is true, when auto-fitting fields to
*  the title width via ListGrid.autoFitFieldWidths or ListGridField.autoFitWidth,
*  should the button be sized such that there is enough space for the header menu button to
*  show without covering the field title?
*  
*  May be explicitly specified at the ListGridField.leaveHeaderMenuButtonSpace
*  or at the ListGrid.leaveHeaderMenuButtonSpace. If not explicitly
*  specified space will be left for fields with
*  ListGridField.align set to "left" or "right", but not for
*  fields with align set to "center".
* @type {boolean}
* @default null
*/
leaveHeaderMenuButtonSpace?: boolean;

/**
* Set this property to true to suppress showing this field on mobile phones 
*  (handset-sized devices). To update this property at runtime use 
*  ListGrid.setHideOnPhone
*  
*  Note that if ListGridField.hidden is set, or an explicit ListGridField.showIf 
*  function exists that will take precedence over this setting. 
*  Similarly, an explicit call to ListGrid.showField or
*  ListGrid.hideField will clear this setting.
*  
*  See also the related property ListGridField.hideOnTablet.
* @type {boolean}
* @default null
*/
hideOnPhone?: boolean;

/**
* Set this property to true to suppress showing this field on tablets 
*  (tablet-sized devices). To update this property at runtime use 
*  ListGrid.setHideOnTablet
*  
*  Note that if ListGridField.hidden is set, or an explicit ListGridField.showIf 
*  function exists that will take precedence over this setting. 
*  Similarly, an explicit call to ListGrid.showField or
*  ListGrid.hideField will clear this setting.
*  
*  See also the related property ListGridField.hideOnPhone.
* @type {boolean}
* @default null
*/
hideOnTablet?: boolean;

/**
* Delay in ms for hovers shown for cells in this field. If unset, the grid's
*  hoverDelay will be used.
*  
*  This property also governes the hoverDelay for the header button. Developers wishing
*  to have a different delay for hovers on the header can use ListGridField.headerHoverDelay
*  in addition to this property.
* @type {Integer}
* @default null
*/
hoverDelay?: Integer;

/**
* Delay in ms for hovers shown over this field's header.
*  
*  If unset, any ListGridField.hoverDelay will be used for both header hovers
*  and hovers shown over cells in the grid's body.
* @type {Integer}
* @default null
*/
headerHoverDelay?: Integer;

/**
* When set to false, don't apply alternate-row styling to this field.
* @type {boolean}
* @default null
*/
showAlternateStyle?: boolean;

/**
* Formula definition for this field.
*  
*  Advanced applications that wish to save formulas separately from a grid's
*  ListGrid.getViewState can provide a UserFormula as part of the field
*  definition, and may subsequently access the formula is updated via the
*  ListGrid.formulaUpdated notification.
*  
*  Note that the current implementation of UserFormula simply executes UserFormula.text
*  as a JavaScript string after making special variables and methods available to the formula.
*  It is safe to allow users to define formulas for themselves (since an end user can always
*  execute whatever JavaScript they want via the browser's built-in developer tools), and is
*  safe to allow formulas to be shared between trusted users. However it would not be safe to
*  allow an untrusted user to create formulas that are shared to other users.
*  
*  Also, while the current implementation would allow creation of a formula that calls
*  JavaScript functions that are not part of the standard or custom
*  MathFunction, this should not be relied upon, as future versions of the
*  formula engine may prohibit such calls.
* @type {UserFormula}
* @default null
*/
userFormula?: UserFormula;

/**
* Can formula be edited from header context menu? Setting attribute to
*  false prevents editing. A null or true
*  value allows editing.
*  
*  Has no effect when ListGrid.canAddFormulaFields is false.
* @type {boolean}
* @default null
*/
canEditFormula?: boolean;

/**
* Shortcut to configure a FormItem.formula for the ListGridField.editorType used when this
*  field is ListGrid.canEdit.
* @type {UserFormula}
* @default null
*/
editorFormula?: UserFormula;

/**
* Shortcut to configure a FormItem.textFormula for the ListGridField.editorType used when this
*  field is ListGrid.canEdit.
* @type {UserSummary}
* @default null
*/
editorTextFormula?: UserSummary;

/**
* Summary definition for this field.
*  
*  Advanced applications that wish to save summaries separately from a grid's
*  ListGrid.getViewState can provide a UserSummary as part of the
*  field definition, and may subsequently access the summary is updated via the
*  ListGrid.summaryUpdated notification.
* @type {UserSummary}
* @default null
*/
userSummary?: UserSummary;

/**
* Can summary be edited from header context menu? Setting attribute to
*  false prevents editing. A null or true
*  value allows editing.
*  
*  Has no effect when ListGrid.canAddSummaryFields is false.
* @type {boolean}
* @default null
*/
canEditSummary?: boolean;



/**
*  Return the value which records should be grouped by.
*  
*  All records for which getGroupValue() returns the same value appear in the same
*  group. Default is the result of ListGrid.getCellValue.
*  
*  While any type of value may be returned, avoiding the use of string values may
*  result in improved performance. In this case, ListGridField.getGroupTitle
*  may be implemented to map a numeric group value into a legible string.
* 
* 
* @param {any} raw value for the cell, from the record for the row
* @param {ListGridRecord} Record object for the cell. Note: If this is a new row that has not been saved, in an  editable grid, it has no associated record object. In this case the edit values will  be passed in as this parameter (see ListGrid.getEditValues)
* @param {object} Field object for which to get group value
* @param {string} The name of the field
* @param {ListGrid} the ListGrid displaying the cell
*/
getGroupValue(value:any, record:ListGridRecord, field:object, fieldName:string, grid:ListGrid): any;

/**
*  Return the title that should be shown to the user for the group with the
*  groupValue passed as a parameter.
*  
*  Default title is the groupValue itself.
* 
* 
* @param {any} the value from the group is created, the result of ListGridField.getGroupValue
* @param {GroupNode} the node in the grid containing the group.
* @param {object} Field object for which to get group value
* @param {string} The name of the field
* @param {ListGrid} the ListGrid displaying the cell
*/
getGroupTitle(groupValue:any, groupNode:GroupNode, field:object, fieldName:string, grid:ListGrid): any;

/**
* 
*  Executed when the user clicks on a ListGridField.valueIcons within
*  this field. Return false to suppress default behavior of firing ListGridField.recordClick 
*  handlers, etc.
* 
* 
* @param {ListGrid} the listGrid that contains the click event
* @param {ListGridRecord} the record that was clicked on
* @param {number} number of the record clicked on in the current set of                 displayed records (starts with 0)
* @param {ListGridField} the field that was clicked on (field definition)
* @param {any} raw value of the cell (before valueMap, etc applied)
* @param {FormItem} If this cell is being ListGrid.canEdit,  this method will fire when the user clicks the valueIcon on the edit item for the  cell, passing in the editor item as the editor parameter. If the cell  is not being edited, this value will be null.
*/
valueIconClick(viewer:ListGrid, record:ListGridRecord, recordNum:number, field:ListGridField, rawValue:any, editor:FormItem): boolean;

/**
*  If your derivation of the field title is more complex than specifying a static string,
*  you can specify a getFieldTitle() method on your field to return the title string.
*  Otherwise you can use the ListGridField.title attribute on the field to specify the title.
*  
*  You can use ListGrid.setFieldProperties to dynamically
*  update the title.
* 
* 
* @param {ListGrid} pointer back to the ListGrid
* @param {number} index of this field in the grid's fields array.
*/
getFieldTitle(viewer:ListGrid, fieldNum:number): string;

/**
*  An optional stringMethods which if provided, is evaluated to
*  conditionally determine whether this field should be displayed.
*  Evaluated on initial draw, then reevaluated on explicit
*  calls to listGrid.refreshFields() or listGrid.setFields().
*  
*  Use ListGridField.hidden:true or showIf:"false"
*  to set a ListGrid field to initially hidden.
*  The user will still be able to show the field via a context menu. 
*  This may be suppressed by setting ListGridField.canHide to false, or by 
*  setting ListGrid.canPickFields to false to suppress the
*  field-picker entirely.
*  
*  Note that explicit calls to ListGrid.showField or hideField()
*  will wipe out the showIf expression, as will the end user showing and
*  hiding columns via the ListGrid.showHeaderContextMenu.
*  
*  Also note that fields marked as DataSourceField.detail will be hidden by
*  default even if ListGrid.showDetailFields is true. To show detail fields
*  inherited from a DataSource, include an explicit field definition for the field and
*  set this property to return true.
*  
*  Note that the ListGridField.visibleWhen attribute also exists as a criteria-based
*  way to dynamically determine field visibility.
* 
* 
* @param {ListGrid} A pointer to the listGrid containing the field
* @param {ListGridField} the ListGridField object
* @param {Integer} the index of the field
*/
showIf(list:ListGrid, field:ListGridField, fieldNum:Integer): boolean;

/**
*  When a field has ListGridField.autoFreeze set to true, developers can
*  implement this method to indicate where in the frozen-fields array this field should
*  appear.
*  
*  Some automatically generated fields, such as
*  ListGrid.rowNumberField,
*  ListGrid.expansionField and
*  ListGrid.checkboxField, provide default implementations of this
*  method.
* 
*/
getAutoFreezePosition(): number;

/**
*  If ListGrid.showGridSummary is true, and this method is specified it will be
*  called to generate the summary value to be displayed in the grid summary row. Note that
*  this is called instead of making use of the ListGridField.summaryFunction.
*  
*  As with ListGrid.getGridSummary this method may return an array of results -
*  in this case each result will show up in a separate row in the ListGrid.summaryRow
*  grid.
*  
*  If this grid is grouped, and ListGrid.showGroupSummary is true, this method
*  will be passed a third parameter - an array of group-level summaries.
* 
* @param {Array<Partial<ListGridRecord>>} records for which a summary is being generated
* @param {ListGridField} pointer to the field for which summary value is being generated
* @param {Array<Partial<object>>=} If this grid is grouped and ListGrid.showGridSummary is specified, this parameter contains an array of already-  calculated summary values for each group in the grid. Each element in this array will  be an object containing calculated summary values for each field in the grid, as well as  a specified groupValue and groupName, allowing the developer to determine which group this  summary value comes from
*/
getGridSummary(records:Array<Partial<ListGridRecord>>, field:ListGridField, groupSummaries?:Array<Partial<object>>): any;

/**
*  If ListGrid.showGroupSummary is true, and this method is specified it will be
*  called to generate the field summary value to be displayed for each group level summary row.
*  Note that this is called instead of making use of the ListGridField.summaryFunction.
*  
*  This method may return an array of results - in this case the group will show multiple summary
*  rows, with each entry in the array showing up in a different record.
* 
* 
* @param {Array<Partial<ListGridRecord>>} records for which a summary is being generated (so all records in the group).
* @param {ListGridField} pointer to the field for which summary value is being generated
* @param {object=} object with specified groupValue and groupName for this group
*/
getGroupSummary(records:Array<Partial<ListGridRecord>>, field:ListGridField, groupNode?:object): any;

/**
*  Only applies to ListGridFieldType fields. If specified, this
*  method will be called to generate the record summary value to be displayed for each row
*  in this field. When this method is called, current values for other 
*  ListGridFieldType fields have not yet been stored on the record, but
*  are accessible via ListGrid.getRecordSummary.
*  
*  Note that if specified, this is called instead of making use of the
*  ListGridField.recordSummaryFunction.
*  
*  If ListGrid.showGridSummary or ListGrid.showGroupSummary is true, this
*  field's value in the summary row[s] will still be calculated by calling this method.
*  In this case, the record object passed in will contain summary values for each field.
*  If custom handling is required for this case, it may be detected by checking the
*  record object's ListGridRecord.isGroupSummary and ListGridRecord.isGridSummary
*  attributes.
* 
* @param {ListGridRecord} record for which a summary is being generated
* @param {ListGridField} this field
* @param {ListGrid} the grid
*/
getRecordSummary(record:ListGridRecord, field:ListGridField, grid:ListGrid): any;

/**
*  Optional string method to return a long summary title for this field, if a dynamic
*  summary title is required for this field.
* 
* 
* @param {ListGrid} pointer back to the ListGrid
* @param {ListGridField} pointer to the field object
* @deprecated * Rather than customizing the summaryTitle developers should typically use the ListGridField.headerTitle attribute to show a different
*  title in the column header button than the title used elsewhere.

*/
getSummaryTitle(viewer:ListGrid, field:ListGridField): void;

/**
*  Optional function to return the value that should be used when sorting this field.
*  
*  Note that, if the dataset exceeds ListGrid.dataPageSize and hence paging is
*  introduced, the grid relies on the server to provide sorting, and the sortNormalizer
*  will no longer be called.
*  
*  For custom sort orders that can be executed both client and server, consider
*  DataSourceField.sortByField.
* 
* 
* @param {object} record to normalize
* @param {string} name of the field on which sorting occurred
* @param {ListGrid} A pointer back to the list grid displaying this field will  be available as the context argument. Note that you can also get a pointer  to the field definition object by calling context.getField(fieldName)
*/
sortNormalizer(recordObject:object, fieldName:string, context:ListGrid): any;

/**
*  FormItem.defaultDynamicValue to be applied to the cell editor for this field
*  when ListGridField.canEdit.
*  
*  Note that editors are only generated for fields that are actually written out, and
*  as such defaultDynamicValue will not apply to
*  ListGridField.hidden, or fields that are not drawn due to
*  ListGrid.showAllColumns.
*  
*  To apply arbitrary default values to the grid when editing new records, developers may
*  use the ListGrid.startEditingNew.
* 
* 
* @param {FormItem} The editor for the cell itself (also available as "this").               Note that in addition to the standard FormItem APIs available                on the editor, it also has:                - a pointer back to the containing listGrid                [item.grid]                - the colNum being edited [item.colNum]                - the rowNum being edited [item.rowNum]
* @param {DynamicForm} the managing DynamicForm instance
* @param {object} the current set of values for the form as a whole
*/
defaultDynamicValue(item:FormItem, form:DynamicForm, values:object): any;

/**
*  Callback fired when the user first starts editing a cell.
*  
*  This callback is typically used to establish dynamic default values via
*  ListGrid.setEditValue or ListGrid.setEditValues.
* 
* 
* @param {ListGridRecord} record for the cell being edited. Will be null                for a new, unsaved record.
* @param {any} value for the cell being edited
* @param {Integer} row number for the cell
* @param {Integer} column number of the cell
* @param {ListGrid} ListGrid to which this field belongs
*/
editorEnter(record:ListGridRecord, value:any, rowNum:Integer, colNum:Integer, grid:ListGrid): void;

/**
*  Callback fired when the user attempts to navigate away from the current edit cell,
*  or complete the current edit.
*  Return false from this method to cancel the default behavior (Saving / cancelling the
*  current edit / moving to the next edit cell)
* 
* 
* @param {EditCompletionEvent} What interaction triggered this                             edit cell exit
* @param {ListGridRecord} record for the cell being edited
* @param {any} new edit value for the cell being edited. Note that if the  user has not made any changes this will be undefined
* @param {Integer} row number for the cell
* @param {Integer} column number of the cell
* @param {ListGrid} ListGrid to which this field belongs
*/
editorExit(editCompletionEvent:EditCompletionEvent, record:ListGridRecord, newValue:any, rowNum:Integer, colNum:Integer, grid:ListGrid): boolean;

/**
*  Callback fired when field changes value as the result of a cell edit. Fired only on
*  successful save of edit, when the new value doesn't match the value before editing.
*  
*  Same signature as ListGrid.cellChanged, but defined on a per-field
*  basis.
* 
* 
* @param {ListGridRecord} record for the cell being changed
* @param {any} new value for the cell
* @param {any} old value for the cell
* @param {number} row number for the cell
* @param {number} column number of the cell
* @param {ListGrid} grid where cell was changed.
*/
cellChanged(record:ListGridRecord, newValue:any, oldValue:any, rowNum:number, colNum:number, grid:ListGrid): void;

/**
* 
*  Executed when this field is clicked on. Note that if ListGrid.recordClick is
*  also defined, it will be fired for fields that define a recordClick handler if the
*  field-level handler returns true. Return false to prevent the grid-level handler from firing.
* 
* 
* @param {ListGrid} the listGrid that contains the click event
* @param {ListGridRecord} the record that was clicked on
* @param {number} number of the record clicked on in the current set of                 displayed records (starts with 0)
* @param {ListGridField} the field that was clicked on (field definition)
* @param {number} number of the field clicked on in the listGrid.fields                 array
* @param {any} value of the cell (after valueMap, etc. applied)
* @param {any} raw value of the cell (before valueMap, etc applied)
* @param {ListGridRecord} the clicked record with any unsaved                  edit values overlaid (see listGrid.getEditedRecord()).
*/
recordClick(viewer:ListGrid, record:ListGridRecord, recordNum:number, field:ListGridField, fieldNum:number, value:any, rawValue:any, editedRecord:ListGridRecord): boolean;

/**
* 
*  Executed when this field is double-clicked. Note that if
*  ListGrid.recordDoubleClick is also defined, it will be fired for fields that define
*  a recordDoubleClick handler if the field-level handler returns true. Return false to prevent
*  the grid-level handler from firing.
* 
* 
* 
* @param {ListGrid} the listGrid that contains doubleclick event
* @param {ListGridRecord} the record that was double-clicked
* @param {number} number of the record clicked on in the current set of                 displayed records (starts with 0)
* @param {ListGridField} the field that was clicked on (field definition)
* @param {number} number of the field clicked on in the listGrid.fields                 array
* @param {object} value of the cell (after valueMap, etc. applied)
* @param {object} raw value of the cell (before valueMap, etc applied)
*/
recordDoubleClick(viewer:ListGrid, record:ListGridRecord, recordNum:number, field:ListGridField, fieldNum:number, value:object, rawValue:object): boolean;

/**
*  Return the HTML to display in cells of this field.
*  
*  Given the raw value for this field as taken from the record Formatter to apply to the
*  static values displayed in cells for this field.
*  
*  Example usage: formatting a currency value stored in cents (so "100" to "$1.00")
*  The value passed to this method is the raw value for the cell.
*  Takes precedence over formatCellValue defined at the grid level for cells
*  in this field.
*  
*  Note: this formatter will not be applied to the values displayed in cells being edited.
*  The ListGridField.formatEditorValue is provided for that purpose.
* 
* 
* @param {any} raw value for the cell, from the record for the row
* @param {ListGridRecord} Record object for the cell. Note: If this is a new row that has not been saved, in an  editable grid, it has no associated record object. In this case the edit values will  be passed in as this parameter (see ListGrid.getEditValues)
* @param {number} row number for the cell
* @param {number} column number for the cell.
* @param {ListGrid} the ListGrid displaying the cell
*/
formatCellValue(value:any, record:ListGridRecord, rowNum:number, colNum:number, grid:ListGrid): HTMLString;

/**
*  Field-level formatter for inactive content.
*  
*  If present, this method will be invoked instead of ListGridField.formatCellValue in cases 
*  where the grid is rendering non-interactive content outside. 
*  See ListGrid.formatInactiveCellValue for more details.
*  
* 
* @param {any} raw value for the cell, from the record for the row
* @param {ListGridRecord} Record object for the cell. Note: If this is a new row that has not been saved, in an  editable grid, it has no associated record object. In this case the edit values will  be passed in as this parameter (see ListGrid.getEditValues)
* @param {number} row number for the cell
* @param {number} column number for the cell.
* @param {ListGrid} the ListGrid displaying the cell
*/
formatInactiveCellValue(value:any, record:ListGridRecord, rowNum:number, colNum:number, grid:ListGrid): HTMLString;

/**
*  Return the value to display in cells of this field which are being edited.
*  
*  Example usage: converting a stored value in cents (100) to a dollar-and-cents
*  value in the editor (1.00)
*  
*  The value passed to this method is the raw value for the cell.
*  
*  formatEditorValue takes precedence over ListGrid.formatEditorValue
*  defined at the grid level for cells in this field.
*  
*  To convert the formatted value displayed within an editor back to a raw value, implement
*  ListGridField.parseEditorValue as well.
* 
* 
* @param {any} raw value for the cell being edited
* @param {ListGridRecord} Record object for the cell. Note: If this is a new row that has not been saved, in an  editable grid, it has no associated record object. In this case the edit values will  be passed in as this parameter.
* @param {number} row number for the cell
* @param {number} column number for the cell.
* @param {ListGrid} A pointer to the ListGrid displaying the cell
*/
formatEditorValue(value:any, record:ListGridRecord, rowNum:number, colNum:number, grid:ListGrid): any;

/**
*  Method used to convert the value displayed in an editor for some cell in this field into
*  a raw value for saving.
*  Takes precedence over parseEditorValue defined at the grid level.
* 
* 
* @param {any} value displayed in the editor for the cell
* @param {object} record object for the row being edited. May be null if this             is a new row being added to the end of the list.
* @param {number} row number for the cell
* @param {number} column number for the cell.
* @param {ListGrid} A pointer to the ListGrid displaying the cell
*/
parseEditorValue(value:any, record:object, rowNum:number, colNum:number, grid:ListGrid): any;

/**
*  If this field is editable, any FormItem.change handler specified
*  on the ListGridField will be passed onto the editors for this field.
*  
*  Note that if ListGridField.canToggle is true, the user may change the value of
*  a boolean field without going into edit mode by single clicking on the field. In this
*  case the ListGridField.change and ListGridField.changed handlers will
*  fire but the form and item parameters will be null.
* 
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the editor (form item) itself (also available as "this").               Note that in addition to the standard FormItem APIs available                on the editor, it also has:                - a pointer back to the containing listGrid                [item.grid]                - the colNum being edited [item.colNum]                - the rowNum being edited [item.rowNum]
* @param {any} The new value of the form item
* @param {any} The previous value of the form item
*/
change(form:DynamicForm, item:FormItem, value:any, oldValue:any): boolean;

/**
*  If this field is editable, any FormItem.changed handler specified
*  on the ListGridField will be passed onto the editors for this field.
*  Note that if ListGridField.canToggle is true, the user may change the value of
*  a boolean field without going into edit mode by single clicking on the field. In this
*  case the ListGridField.change and ListGridField.changed handlers will
*  fire but the form and item parameters will be null.
* 
* 
* @param {DynamicForm} the managing DynamicForm instance
* @param {FormItem} the editor (form item) itself (also available as "this").               Note that in addition to the standard FormItem APIs available                on the editor, it also has:                - a pointer back to the containing listGrid                [item.grid]                - the colNum being edited [item.colNum]                - the rowNum being edited [item.rowNum]
* @param {any} The current value (after the change).
*/
changed(form:DynamicForm, item:FormItem, value:any): void;

/**
*  StringMethod override point for returning HTML to be shown in hovers over cells in the
*  column described by this field.
*  
*  Called only when ListGrid.canHover and ListGrid.showHover
*  are both true.
*  
*  The value of "this" within the method will by the ListGridField.
*  
*  Note that the field described by the column might provide a custom
*  ListGridField.cellPrompt which won't be applied unless you return it 
*  from your implementation. 
* 
* 
* @param {ListGridRecord} record being hovered over
* @param {any} value of the cell being hovered over
* @param {number} row number where hover occurred
* @param {number} column number where hover occurred
* @param {ListGrid} ListGrid this field is a part of
*/
hoverHTML(record:ListGridRecord, value:any, rowNum:number, colNum:number, grid:ListGrid): HTMLString;

/**
*  Optional stringMethod to get a ListGridField.valueMap for a specific field.
*  If present this method will be called from ListGrid.getEditorValueMap and the resulting
*  valueMap will be used instead of any static specified valueMap for the field.
* 
* @param {object} Field values for record being edited. Note that this will include            the current edit values for fields that have not yet been saved.             May be null, if editing a new record.
* @param {ListGridField} pointer to the listGridField
* @param {ListGrid} pointer back to this ListGrid instance.
*/
getEditorValueMap(values:object, field:ListGridField, grid:ListGrid): ValueMap;



}


/**
* A ListGridRecord is a JavaScript Object whose properties contain values for each
*  ListGridField.
*  A ListGridRecord may have additional properties 
*  which affect the record's appearance or behavior, or which hold data for use by custom logic 
*  or other, related components.
*  
*  
*  For example a ListGrid that defines the following fields:
*  
*  fields : [
*    {name: "field1"},
*    {name: "field2"}
*  ],
*  
*  
*  
*  Might have the following data:
*  
*  
*  data : [
*    {field1: "foo", field2: "bar", customProperty:5},
*    {field1: "field1 value", field2: "field2 value", enabled:false}
*  ]
*  
*  
*  Each line of code in the data array above creates one JavaScript Object via
*  JavaScript ObjectLiteral notation. These JavaScript Objects are
*  used as ListGridRecords.
*  
*  
*  
*  Both records shown above have properties whose names match the name property of a
*  ListGridField, as well as additional properties. The second record will be disabled due to
*  enabled:false; the first record has a property "customProperty" which will
*  have no effect by default but which may be accessed by custom logic.
*  
*  
*  
*  After a ListGrid is created and has loaded data, records may be accessed via
*  
*  
*  ListGrid.data, for example, listGrid.getData().get(0) retrieves the first record.
*  ListGrid.data may be a ResultSet if the listGrid is bound to a DataSource.
*  
*  ListGridRecords are also passed to many events, such as
*  ListGrid.cellClick.
*  
*  A ListGridRecord is always
*  an ordinary JavaScript Object regardless of how the grid's
*  dataset is loaded (static data, java server, XML web service, etc), 
*  
*  and so supports the normal behaviors of JavaScript Objects, including accessing and 
*  assigning to properties via dot notation:
*  
*    var fieldValue = record.fieldName;
*    record.fieldName = newValue;
*  
*  
*  
*  
*  Note however that simply assigning a value to a record won't cause the display to be
*  automatically refreshed - ListGrid.refreshCell needs to be called. Also,
*  consider editing when directly modifying
*  ListGridRecords.
*  
*  See the attributes in the API tab for the full list of special properties on
*  ListGridRecords that will affect the grid's behavior.
*/
interface ListGridRecord extends Record {
/**
* Default property name denoting whether this record is enabled. Property name may be
*  modified for some grid via ListGrid.recordEnabledProperty.
* @type {boolean}
* @default null
*/
enabled?: boolean;

/**
* Default property name denoting whether this record can be expanded. Property name may be
*  modified for the grid via ListGrid.canExpandRecordProperty.
* @type {boolean}
* @default null
*/
canExpand?: boolean;

/**
* Default property name denoting whether this record can be removed. Property name may be
*  modified for the grid via ListGrid.recordCanRemoveProperty.
* @type {boolean}
* @default null
*/
_canRemove?: boolean;

/**
* Default property name denoting a separator row.
*  When set to true, defines a horizontal separator in the listGrid
*  object. Typically this is specified as the only property of a record object, since a
*  record with isSeparator:true will not display any values.
*  Note: this attribute name is governed by ListGrid.isSeparatorProperty.
* @type {boolean}
* @default null
*/
isSeparator?: boolean;

/**
* Name of a CSS style to use for all cells for this particular record.
*  
*  Note that using this property assigns a single, fixed style to the record, so rollover
*  and selection styling are disabled. To provide a series of stateful styles for a record
*  use ListGridRecord._baseStyle instead.
*  
*  See ListGrid.getCellStyle for an overview of various ways to customize styling,
*  both declarative and programmatic.
*  
*  If this property is changed after draw(), to refresh the grid call
*  ListGrid.refreshRow (or ListGrid.markForRedraw if several rows are
*  being refreshed).
*  
*  If your application's data uses the "customStyle" attribute for something else, the
*  property name can be changed via ListGrid.recordCustomStyleProperty.
* @type {CSSStyleName}
* @default null
*/
customStyle?: CSSStyleName;

/**
* Name of a CSS style to use as the ListGrid.baseStyle for all cells for this
*  particular record.
*  
*  The styleName specified with have suffixes appended to it as the record changes state
*  ("Over", "Selected" and so forth) as described by ListGrid.getCellStyle. For a
*  single, fixed style for a record, use ListGridRecord.customStyle instead.
*  
*  See ListGrid.getCellStyle for an overview of various ways to customize styling,
*  both declarative and programmatic.
*  
*  If this property is changed after draw(), to refresh the grid call
*  ListGrid.refreshRow (or ListGrid.markForRedraw if several rows are
*  being refreshed).
*  
*  If your application's data uses the "_baseStyle" attribute for something else, the
*  property name can be changed via ListGrid.recordBaseStyleProperty.
* @type {CSSStyleName}
* @default null
*/
_baseStyle?: CSSStyleName;

/**
* Default property name denoting the single value to display for all fields of this row.
*  If this property is set for some record, the record will be displayed as a single
*  cell spanning every column in the grid, with contents set to the value of this
*  property.
*  Note: this attribute name is governed by ListGrid.singleCellValueProperty.
* @type {HTMLString}
* @default null
*/
singleCellValue?: HTMLString;

/**
* When set to false, this record cannot be dragged. If canDrag is false for
*  any record in the current selection, none of the records will be draggable.
* @type {boolean}
* @default null
*/
canDrag?: boolean;

/**
* When set to false, other records cannot be dropped on (i.e., inserted
*  via drag and drop) immediately before this record.
* @type {boolean}
* @default null
*/
canAcceptDrop?: boolean;

/**
* The HTML to display in this row for fields with fieldType set to link. This overrides
*  ListGridField.linkText.
* @type {string}
* @default null
*/
linkText?: string;

/**
* Set to false to disable rollover for this individual record when ListGrid.showRollOver
*  is true.
*  
*  Note this property can be renamed to prevent collision with data members - see
*  ListGrid.recordShowRollOverProperty.
* @type {boolean}
* @default null
*/
showRollOver?: boolean;

/**
* A component that should be rendered on top of this record, similar to a
*  ListGrid.showRecordComponents but statically defined on the record.
*  
*  The embedded component will default to covering all fields of the record, but specific fields
*  can be specified via ListGridRecord.embeddedComponentFields.
*  
*  By default, the embeddedComponent will fill the entire vertical and horizontal space of the
*  record (or of the specified fields). ListGridRecord.embeddedComponentPosition can be set to control
*  exact sizing behavior.
*  
*  
*  When creating a component to use as an embedded component the component will most likely
*  end up drawing before the record it is due to be embedded within, therefore it is recommended
*  to set Canvas.autoDraw to false on the embedded component.
*  
*  
*  When a record with an embeddedComponent is eliminated from view by filtering or
*  because it is not currently rendered due to ListGrid.showAllRecords, the
*  ListGrid may Canvas.hide or Canvas.clear it.
*  
*  If the current dataset is completely replaced (by a call to ListGrid.setData or
*  ListGrid.setDataSource, for example), any embedded component is
*  Canvas.deparent (which implies being Canvas.clear).
*  
*  When a ListGrid is Canvas.destroy, it will destroy() all embedded components
*  regardless of whether they are currently visible. Use a call to ListGrid.setData immediately
*  before destroying the ListGrid to avoid this effect when unwanted.
*  
*  For more advanced control over the lifecycle of components displayed over records, including
*  deferred creation and pooling, use the ListGrid.showRecordComponents
*  subsystem.
* @type {Canvas}
* @default null
*/
embeddedComponent?: Canvas;

/**
* Sizing policy applied to the embedded component. Default behavior if unspecified is the same
*  as EmbeddedPosition "within" (fill space allocated to the record, including the ability
*  use percentage sizing and snapTo offset). Use "expand" to have the record expand to accommodate
*  the embedded components' specified sizes instead.
* @type {EmbeddedPosition}
* @default null
*/
embeddedComponentPosition?: EmbeddedPosition;

/**
* Fields where the ListGridRecord.embeddedComponent will be displayed, if specified.
*  
*  Regardless of the order of fields specified, the component will appear from whichever field is
*  earlier in the current visible order to whichever field is later, inclusive of the specified
*  fields.
*  
*  To have the component appear in just one field, either specify a single-element Array or
*  specific a two element Array with both fields the same.
*  
*  If either field is hidden or invalid (no such field), the component will occupy only a single
*  field. If both fields are hidden, the component will be hidden until one or more of the fields
*  are shown.
* @type {Array<Partial<string>>}
* @default null
*/
embeddedComponentFields?: Array<Partial<string>>;

/**
* Default property name denoting whether this record can be selected. Property name may be
*  modified for the grid via ListGrid.recordCanSelectProperty.
* @type {boolean}
* @default null
*/
canSelect?: boolean;

/**
* Default property name denoting whether this record can be edited. Property name may be
*  modified for the grid via ListGrid.recordEditProperty.
* @type {boolean}
* @default null
*/
_canEdit?: boolean;

/**
* Has no effect unless ListGrid.showBackgroundComponents is true.
*  
*  Canvas created and embedded in the body behind a given record.  When set, either as
*  a Canvas or Canvas Properties, will be constructed if necessary, combined with the
*  autoChild properties specified for ListGrid.backgroundComponent and displayed
*  behind this record in the page's z-order, meaning
*  it will only be visible if the cell styling is transparent.
* @type {Canvas}
* @default null
*/
backgroundComponent?: Canvas;

/**
* The default value of ListGrid.recordDetailDSProperty.
* @type {DataSource}
* @default null
*/
detailDS?: DataSource;

/**
* If specified as false this record should be ignored when calculating summary totals
*  to be shown in the ListGrid.showGridSummary for this grid.
*  
*  Note that includeInSummary is the default property name for this attribute,
*  but it may be modified via ListGrid.includeInSummaryProperty.
* @type {boolean}
* @default null
*/
includeInSummary?: boolean;

/**
* This attribute will automatically be set to true for the record representing the
*  grid-level summary row shown if ListGrid.showGridSummary is true.
*  
*  Note that isGridSummary is the default property name for this attribute but
*  it may be modified by setting ListGrid.gridSummaryRecordProperty
* @type {boolean}
* @default null
*/
isGridSummary?: boolean;

/**
* This attribute will automatically be set to true for records representing
*  group-level summary rows shown if ListGrid.showGroupSummary is true.
*  
*  Note that isGroupSummary is the default property name for this attribute but
*  it may be modified by setting ListGrid.groupSummaryRecordProperty
* @type {boolean}
* @default null
*/
isGroupSummary?: boolean;





}


/**
* Configuration of an HTML list in a RichTextEditor.
*/
interface ListProperties {
/**
* The style of list item marker. If "custom-image", the ListProperties.image
*  should be specified.
* @type {ListStyleType}
* @default null
*/
style?: ListStyleType;

/**
* When the list item marker ListProperties.style is "custom-image", the image
*  to use for the markers.
* @type {SCImgURL}
* @default "[SKIN]/RichTextEditor/bullet_blue.png"
*/
image?: SCImgURL;

/**
* For ordered lists, the number to start the first item with. Must be non-negative.
* @type {Integer}
* @default "1"
*/
startNumber?: Integer;





}


/**
* LoadProjectSettings is the bundle of settings that can be passed to loadProject() as
*  the "settings" argument, including optional http parameters for the request to 
*  servletDetails.
*  
*  There is no need to instantiate an LoadProjectSettings instance. Just pass a normal
*  JavaScript object with the desired properties.
*/
interface LoadProjectSettings {
/**
* The name of the screen within the project to return first when loading. 
*  A null value means to use the currentScreenName as specified in the project file.
*  
*  This setting only affects the first project specified in the projectNames
*  argument to RPCManager.loadProject.
* @type {string}
* @default null
*/
currentScreenName?: string;

/**
* A comma-separated string containing the names of screens within the project that should
*  be loaded. A null value causes all screens to be loaded.
*  
*  This setting only affects the first project specified in the projectNames
*  argument to RPCManager.loadProject.
* @type {string}
* @default null
*/
screenNames?: string;

/**
* Determines whether the LoadProjectSettings.currentScreenName screen is drawn
*  after all screens have been loaded. If not drawn, the screen will not be created.
*  
*  This setting only affects the first project specified in the projectNames
*  argument to RPCManager.loadProject.
* @type {boolean}
* @default false
*/
drawFirstScreen?: boolean;

/**
* The name of a locale to use for resolving i18n tags in the component XML of the screen.
*  The default value of null omits locale loading, which effectively means the framework
*  default "en" locale is used.
* @type {string}
* @default null
*/
locale?: string;

/**
* Use this attribute to specify a project owner.
*  Only applicable if project source supports owner identification.
* @type {string}
* @default null
*/
ownerId?: string;

/**
* Should Placeholder in loaded screens be rendered as placeholders?
*  If property is not set actual components are created instead of the Placeholders.
* @type {boolean}
* @default null
*/
allowPlaceholders?: boolean;

/**
* DataContext that will be provided to the top-level component as
*  Canvas.dataContext in each screen cached for the project.
*  
*  To understand how dataContext is used to automatically populate 
*  DataBoundComponent, see Canvas.autoPopulateData.
* @type {DataContext}
* @default null
*/
dataContext?: DataContext;

/**
* Whether to call the provided Callbacks.LoadProjectCallback even if an
*  error was encountered trying to load the project, so that you can run your own error
*  handling. In this case, the list of projects loaded by the callback will be reported as
*  empty.
*  
*  If true, the framework won't log any messages specifically reporting the failure to
*  load the requested projects, but depending on the situation, the browser itself may
*  report errors from the servlet request in the console.
* @type {boolean}
* @default false
*/
willHandleError?: boolean;

/**
* Allows customizing the projectLoader servlet request properties. Properties that
*  conflict with proper operation of the servlet will be overridden by
*  RPCManager.loadProject.
* @type {Partial<RPCRequest>}
* @default null
*/
requestProperties?: Partial<RPCRequest>;

/**
* Sets the timeout for the projectLoader servlet request. This is a convenience property
*  so that LoadProjectSettings.requestProperties need not be set. If unset, the timeout is determined
*  by RPCManager.defaultTimeout.
* @type {Integer}
* @default null
*/
timeout?: Integer;

/**
* Should DataSources referenced by the LoadProjectSettings.drawFirstScreen clobber
*  existing, globally-bound DataSources on the client if the screen is created? The
*  default of false means that any DataSources defined in the screen will be discarded if
*  they collide with existing, globally-bound DataSources.
*  
*  Here we consider a DataSource to be "globally bound" if it can be retrieved by
*  ID using the method DataSource.get, regardless of whether it's actually bound
*  to the browser window object.
*  
*  Note that this setting only has an impact if LoadProjectSettings.drawFirstScreen is true.
* @type {boolean}
* @default false
*/
clobberDataSources?: boolean;

/**
* DataSource IDs in the project to skip and not load when the project is loaded. It is
*  assumed that these IDs represent DataSources that are already globally-bound on the
*  client.
*  
*  The special value of "*" can be specified for this property to indicate that all
*  DataSources should be omitted.
*  
*  Note that, unless LoadProjectSettings.omitLoadedDataSources is false, all loaded DataSources will by
*  default be added to whatever value you provide (making that also the default for this
*  property).
* @type {Array<Partial<Identifier>>}
* @default "varies"
*/
omitDataSources?: Array<Partial<Identifier>>;

/**
* Whether to implicitly add all DataSources currently loaded on the client to
*  LoadProjectSettings.omitDataSources in RPCManager.loadProject. Setting this false would
*  only make sense in connection with setting LoadProjectSettings.clobberDataSources true, and would
*  create more potential work for the server since many more DataSources could be output.
*  
*  Note that here we consider the "loaded DataSources" to be those that are registered
*  with the DataSource module (i.e. available by ID via DataSource.get),
*  regardless of whether they're actually bound to the browser window object.
* @type {boolean}
* @default true
*/
omitLoadedDataSources?: boolean;

/**
* Enables DataSource verification, causing warnings
*  to be output about differences between DataSources loaded with a project vs those
*  that are already present in the page (if any are present). Findings are always reported
*  to the console, and may also be presented to the user with a warning dialog by setting
*  LoadProjectSettings.verifyAsError. For discussion of which issues will be
*  reported and how, see DataSource.verifyDataSourcePair.
*  
*  Setting this property will default LoadProjectSettings.omitLoadedDataSources to false, and will cause
*  the special LoadProjectSettings.omitDataSources value of "*" to be ignored (but not other ID values).
*  It's important to keep in mind when using this property that loading a project
*  with all of its DataSources can be very slow, if there is a large amount of test data,
*  for example.
*  
*  Note that this behavior can be enabled globally by setting
*  Reify.verifyDataSources to true. It often makes sense to set it there during
*  development, and turn off in production, since it slightly slows down DataSource
*  loading.
* @type {boolean}
* @default null
*/
verifyDataSources?: boolean;

/**
* Enable RPCManager.verifyAsError behavior only for
*  requests using these settings.
* @type {boolean}
* @default null
*/
verifyAsError?: boolean;

/**
* Enables verification that any screen LoadProjectSettings.drawFirstScreen
*  on load contains a component having a localId equal to the given key,
*  and that it is an instance (or subclass) of the key's value. Example:
*  
*   {'customerListGrid': 'ListGrid'}
*  
*  You may verify the presence of Tabs, SectionStackSections, and FormItems by
*  providing their names following the parent component's id in dot-separated notation.
*  Example:
*  
*   {
*    'mainTabSet.customersTab': 'ImgTab',
*    'mainSectionStack.customerStackSection': 'SectionStackSection',
*    'customerDetailsForm.customerNameItem': 'TextItem'
*   }
*  
*  Findings are always reported
*  to the console, and may also be presented to the user with a warning dialog by setting
*  LoadProjectSettings.verifyAsError or RPCManager.verifyAsError.
* @type {any}
* @default null
*/
verifyComponents?: any;

/**
* URL of Reify server to use when calling Reify.loadProject instead of
*  Reify.serverURL.
*  
*  Note that this setting only applies when using Reify.loadProject.
* @type {URL}
* @default "varies"
*/
serverURL?: URL;

/**
* Path relative to the LoadProjectSettings.serverURL, to target to use the project loader
*  servlet, instead of Reify.projectLoaderPath.
*  
*  Note that this setting only applies when using Reify.loadProject.
* @type {string}
* @default "varies"
*/
projectLoaderPath?: string;

/**
* Overrides Reify.userName setting the account name for Reify.loadProject.
*  
*  Note that this setting only applies when using Reify.loadProject.
* @type {string}
* @default "varies"
*/
userName?: string;

/**
* Overrides Reify.password setting the account password for
*  Reify.loadProject.
*  
*  Note that this setting only applies when using Reify.loadProject.
* @type {string}
* @default "varies"
*/
password?: string;





}


/**
* This is the bundle of settings that can be passed to RPCManager.loadScreen as
*  the "settings" argument. Some settings can also be passed as separate arguments; if
*  these are present both as separate arguments and in settings, loadScreen() will use the
*  value from the settings.
*  
*  There is no need to instantiate a LoadScreenSettings instance. Just pass a
*  normal JavaScript object with the desired properties.
*/
interface LoadScreenSettings {
/**
* The name of a locale to use for resolving i18n tags in the component XML of the screen.
* @type {string}
* @default null
*/
locale?: string;

/**
* Specifies optional RPCRequest properties for the request.
* @type {Partial<RPCRequest>}
* @default null
*/
requestProperties?: Partial<RPCRequest>;

/**
* If true, server logic does not crash out if it cannot load a DataSource specified in the
*  screen definition. Instead, a stub DataSource is returned, which consists of nothing
*  except the ID and an unableToLoad flag, which client-side code can use to
*  determine that the DataSource could not be loaded on the server. Optional, defaults to
*  false (ie, a missing DataSource causes a crash by default)
* @type {boolean}
* @default null
*/
missingDSIsNotFatal?: boolean;

/**
* Whether to call the provided Callbacks.LoadScreenCallback even if an
*  error was encountered trying to load the screen (or it was simply not found), so that
*  you can run your own error handling. In this case, the screen and "suppressed globals"
*  will be reported as null.
*  
*  Note that for backward compatibility, the default of null means that the callback will
*  still happen if the screen is not found, but not for a response with a server error.
*  To have all such cases handled automatically, set this property explicitly false.
* @type {boolean}
* @default null
*/
willHandleError?: boolean;

/**
* Should DataSources referenced by the screen clobber existing, globally-bound DataSources
*  on the client when the screen is loaded? The default of false means that any
*  DataSources defined in the screen will be discarded if they collide with existing,
*  globally-bound DataSources.
*  
*  Note that here we consider a DataSource to be "globally bound" if it can be retrieved by
*  ID using the method DataSource.get, regardless of whether it's actually bound
*  to the browser window object.
* @type {boolean}
* @default false
*/
clobberDataSources?: boolean;

/**
* DataSource IDs in the screen to skip and not load when the screen is loaded. It is
*  assumed that these IDs represent DataSources that are already globally-bound on the
*  client.
*  
*  The special value of "*" can be specified for this property to indicate that all
*  DataSources should be omitted.
*  
*  Note that, unless LoadScreenSettings.omitLoadedDataSources is false, all loaded DataSources will by
*  default be added to whatever value you provide (making that also the default for this
*  property).
* @type {Array<Partial<Identifier>>}
* @default "varies"
*/
omitDataSources?: Array<Partial<Identifier>>;

/**
* Whether to implicitly add all DataSources currently loaded on the client to
*  LoadScreenSettings.omitDataSources in RPCManager.loadScreen. Setting this false would only
*  make sense in connection with setting LoadScreenSettings.clobberDataSources true, and would create
*  more potential work for the server since many more DataSources could be output.
*  
*  Note that here we consider the "loaded DataSources" to be those that are registered
*  with the DataSource module (i.e. available by ID via DataSource.get),
*  regardless of whether they're actually bound to the browser window object.
* @type {boolean}
* @default true
*/
omitLoadedDataSources?: boolean;

/**
* If true, prevents any screen from being drawn when it's loaded, even if
*  there's an explicit Canvas.autoDraw:true setting on it.
* @type {boolean}
* @default true
*/
suppressAutoDraw?: boolean;

/**
* Caches screen much like RPCManager.cacheScreens, so
*  RPCManager.isScreenCached reports true.
* @type {boolean}
* @default false
*/
cacheScreen?: boolean;

/**
* Enables verification that the created screen contains a component having a
*  localId equal to the given key, and that it is an instance (or subclass)
*  if the key's value. Example:
*  
*   {customerListGrid: 'ListGrid'}
*  
*  You may verify the presence of Tabs, SectionStackSections, and FormItems by
*  providing their names following the parent component's id in dot-separated notation.
*  Example:
*  
*   {
*    'mainTabSet.customersTab': 'ImgTab',
*    'mainSectionStack.customerStackSection': 'SectionStackSection',
*    'customerDetailsForm.customerNameItem': 'TextItem'
*   }
*  
*  Findings are always reported to the console, and may also be presented to the user
*  with a warning dialog by setting LoadScreenSettings.verifyAsError or
*  RPCManager.verifyAsError.
* @type {any}
* @default null
*/
verifyComponents?: any;

/**
* Enable RPCManager.verifyAsError behavior only for
*  requests using these settings.
* @type {boolean}
* @default null
*/
verifyAsError?: boolean;





}


/**
* Set of properties to configure the loading indicator displayed by FileLoader.showLoadingIndicator
*/
interface LoadingIndicatorSettings {
/**
* Loading indiciator image to display.
* @type {SCImgURL}
* @default null
*/
image?: SCImgURL;

/**
* Width for the loading indiciator image.
* @type {Integer}
* @default null
*/
imageWidth?: Integer;

/**
* Height for the loading indiciator image.
* @type {Integer}
* @default null
*/
imageHeight?: Integer;

/**
* Optional HTML text message to display in the loading indicator.
* @type {any}
* @default null
*/
message?: any;

/**
* CSS class to apply to the loading indicator.
* @type {CSSStyleName}
* @default null
*/
style?: CSSStyleName;

/**
* CSS class to apply to the loading indicator message text.
* @type {CSSStyleName}
* @default null
*/
textStyle?: CSSStyleName;

/**
* CSS class to apply to the loading indicator.
* @type {CSSStyleName}
* @default null
*/
imageStyle?: CSSStyleName;

/**
* Target position / sizing for the loading indiciator. May be specified as a four element
*  array [left,top,width,height] or as a DOM element, in which case the
*  indicator will be sized and positioned to match the client bounding rect of the target element.
*  If unspecified, the loading indicator will be sized to fit its content and centered on the page.
* @type {Array<Partial<Integer>> | DOMElement}
* @default null
*/
target?: Array<Partial<Integer>> | DOMElement;

/**
* Target z-index to apply to the loading indiciator.
* @type {Integer}
* @default null
*/
zIndex?: Integer;





}


/**
* Object specifying an item in a Menu. Each MenuItem can have a
*  MenuItem.title, MenuItem.icon, MenuItem.keys, optional MenuItem.submenu and various other settings. Alternatively, a
*  MenuItem can contain an arbitrary widget via MenuItem.embeddedComponent.
*  
*  MenuItems are specified as plain Object, usually with
*  ObjectLiteral notation. For example:
*  
*  isc.Menu.create({
*    items : [
*      {title: "item1", click: "alert(1)"},
*      {title: "item2"}
*    ]
*  });
*  
*  Do not use isc.MenuItem.create() - this is invalid.
*  
*  
*  
*  Alternatively, Menus support binding to a Menu.dataSource.
*  
*  As another option, here's a sample of a Menu in componentXML:
*  
*  &lt;Menu&gt;
*   &lt;items&gt;
*     &lt;MenuItem title="item1" click="alert(1)"/&gt;
*     &lt;MenuItem title="item2"/&gt;
*   &lt;/items&gt;
*  &lt;/Menu&gt;
*  
*/
interface MenuItem extends ListGridRecord {
/**
* The text displayed for the menu item
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* A reference to another menu, to display as a submenu when the mouse cursor hovers over
*  this menu item.
* @type {Menu}
* @default null
*/
submenu?: Menu;

/**
* A MenuItem that has a submenu normally cannot be selected, instead clicking or hitting Enter
*  while keyboard focus is on the item shows the submenu. Setting canSelectParent:true allows
*  a menu item with a submenu to be selected directly.
* @type {boolean}
* @default null
*/
canSelectParent?: boolean;

/**
* When set to true, this menu item shows a horizontal separator instead of
*  the MenuItem.title text. Typically specified as the only property of a menu item,
*  since the separator will not respond to mouse events.
* @type {boolean}
* @default false
*/
isSeparator?: boolean;

/**
* Affects the visual style and interactivity of the menu item. If set to
*  false, the menu item will not respond to mouse rollovers or clicks.
*  
*  If you need to set this state dynamically, use MenuItem.enableIf instead.
* @type {boolean}
* @default true
*/
enabled?: boolean;

/**
* If true, this item displays a standard checkmark image to the left of its title. You
*  can set the checkmark image URL by setting Menu.checkmarkImage.
*  
*  If you need to set this state dynamically, use MenuItem.checkIf instead.
* @type {boolean}
* @default null
*/
checked?: boolean;

/**
* Default Menu.itemHiddenProperty for menu items.
*  If true, this item will be hidden wihin the menu by default.
*  
*  To update item visibility at runtime, call Menu.setItemHidden
* @type {boolean}
* @default null
*/
hidden?: boolean;

/**
* The filename for this item's custom icon when the item is disabled. If both this property
*  and MenuItem.checked are both specified, only the icon specified by this property
*  will be displayed. The path to the loaded skin directory and the skinImgDir are prepended
*  to this filename to form the full URL.
*  
*  If you need to set this state dynamically, use MenuItem.dynamicIcon instead.
* @type {string}
* @default null
*/
disabledIcon?: string;

/**
* The filename for this item's custom icon. If both this property and
*  MenuItem.checked are both specified, only the icon specified by this property will be
*  displayed. The path to the loaded skin directory and the skinImgDir are prepended to
*  this filename to form the full URL. If this item is disabled, and MenuItem.disabledIcon
*  is set, then that icon will be used instead.
*  
*  If you need to set this state dynamically, use MenuItem.dynamicIcon instead.
* @type {string}
* @default null
*/
icon?: string;

/**
* The width applied to this item's icon. The default of 16 can be changed
*  for all MenuItems by overriding Menu.iconWidth.
* @type {number}
* @default "16"
*/
iconWidth?: number;

/**
* The height applied to this item's icon. The default of 16 can be changed
*  for all MenuItems by overriding Menu.iconHeight.
* @type {number}
* @default "16"
*/
iconHeight?: number;

/**
* Shortcut key(s) to fire the menu item action. Each key can be defined as a KeyIdentifier.
*  To apply multiple shortcut keys to this item, set this property to an array of such key
*  identifiers.
* @type {KeyIdentifier | Array<Partial<KeyIdentifier>>}
* @default null
*/
keys?: KeyIdentifier | Array<Partial<KeyIdentifier>>;

/**
* A string to display in the shortcut-key column for this item. If not
*  specified, the first KeyName value in MenuItem.keys will be used by default.
* @type {string}
* @default "see below"
*/
keyTitle?: string;

/**
* If false, no submenus will be fetched for this MenuItem. This can be set globally via
*  Menu.fetchSubmenus.
* @type {boolean}
* @default true
*/
fetchSubmenus?: boolean;

/**
* Criteria to be evaluated to determine whether this MenuItem should be disabled. Re-evaluated
*  each time the menu is shown.
*  
*  A basic criteria uses textMatchStyle:"exact". When specified in
*  componentXML this property allows
*  xmlCriteriaShorthand for defining criteria.
* @type {AdvancedCriteria}
* @default null
*/
enableWhen?: AdvancedCriteria;

/**
* Criteria to be evaluated to determine whether this MenuItem should be visible. Re-evaluated
*  each time the menu is shown.
*  
*  A basic criteria uses textMatchStyle:"exact". When specified in
*  componentXML this property allows
*  xmlCriteriaShorthand for defining criteria.
* @type {AdvancedCriteria}
* @default null
*/
visibleWhen?: AdvancedCriteria;

/**
* Arbitrary UI component that should appear in this MenuItem. See
*  ListGridRecord.embeddedComponent for an overview and options for controlling placement.
*  
*  When embeddedComponent is used in a MenuItem certain default behaviors apply:
*  
*  
* - MenuItem.autoDismiss defaults to false and clicks on embeddedComponents are 
*    not bubbled to the menuItem - if an interaction with an embeddedComponent is 
*    expected to dismiss the menu, custom code should call menu.Canvas.hide or 
*    Menu.hideAllMenus as appropriate, before proceeding
*  
* - the default behavior for MenuItem.embeddedComponentPosition is "expand".
*  
* - the component is placed over the title and key fields by default
*  - use MenuItem.embeddedComponentFields to override
*  
* - rollOver styling is disabled by default (as though ListGridRecord.showRollOver were
*  set to false)
*  
* @type {Canvas}
* @default null
*/
embeddedComponent?: Canvas;

/**
* See ListGridRecord.embeddedComponentPosition, except that when used in a
*  menuItem, default behavior is EmbeddedPosition "expand".
* @type {EmbeddedPosition}
* @default null
*/
embeddedComponentPosition?: EmbeddedPosition;

/**
* See ListGridRecord.embeddedComponentFields. Default for a MenuItem is to cover the
*  title and key fields, leaving the icon and submenu fields visible.
* @type {Array<Partial<string>>}
* @default null
*/
embeddedComponentFields?: Array<Partial<string>>;

/**
* Whether a click on this specific menuItem automatically dismisses the menu. See
*  Menu.autoDismiss.
* @type {boolean}
* @default null
*/
autoDismiss?: boolean;

/**
* When used in an AdaptiveMenu, should this MenuItem show only it's
*  MenuItem.icon when displayed inline?
* @type {boolean}
* @default null
*/
showIconOnlyInline?: boolean;



/**
*  Contains the condition that will enable or disable the current menuItem. The handler must be specified
*  as a function or string of script. Return false to disable the menuItem or true to enable it
*  
*  If you don't need to set this state dynamically, use MenuItem.enabled instead.
*  
*   Alternatively, you can use Criteria to declare when a MenuItem is
*   enabled via MenuItem.enableWhen.
*  
*  May be defined as a stringMethods.
* 
* 
* @param {Canvas} Menu.target attribute for the top level menu.
* @param {Menu} Menu contains the reference to the menu that contains the current item
* @param {MenuItem} contains the reference to the current item
*/
enableIf(target:Canvas, menu:Menu, item:MenuItem): boolean;

/**
*  Contains the condition that will check or uncheck the current menuItem. The handler must be specified
*  as a function or string of script. Return false to uncheck the menuItem or true to check it
*  
*  If you don't need to set this state dynamically, use MenuItem.checked instead.
*  
*  May be defined as a stringMethods.
*  
* 
* @param {Canvas} Menu.target attribute for the top level menu.
* @param {Menu} Menu contains the reference to the menu that contains the current item
* @param {MenuItem} contains the reference to the current item
*/
checkIf(target:Canvas, menu:Menu, item:MenuItem): boolean;

/**
*  Contains the condition that will change the current items' title when met. The handler must be specified
*  as a function or string of script.
*  
*  If you don't need to set this state dynamically, use MenuItem.title instead.
*  
*  May be defined as a stringMethods.
*  
* 
* @param {Canvas} Menu.target attribute for the top level menu.
* @param {Menu} Menu contains the reference to the menu that contains the current item
* @param {MenuItem} contains the reference to the current item
*/
dynamicTitle(target:Canvas, menu:Menu, item:MenuItem): string;

/**
*  Contains the condition that will change the current items' icon when met. The handler must be specified
*  as a function or string of script.
*  
*  If you don't need to set this state dynamically, use MenuItem.icon instead.
*  
*  May be defined as a stringMethods.
*  
* 
* @param {Canvas} Menu.target attribute for the top level menu.
* @param {Menu} Menu contains the reference to the menu that contains the current item
* @param {MenuItem} contains the reference to the current item
*/
dynamicIcon(target:Canvas, menu:Menu, item:MenuItem): SCImgURL;

/**
*  Executed when this menu item is clicked by the user. The click handler must be specified
*  as a function or string of script. Return false to suppress the Menu.itemClick
*  handler if specified.
* 
* 
* @param {Canvas} for a menu shown as a context menu, the Canvas the menu was shown            on. Otherwise the Menu instance of which this             MenuItem is a member.
* @param {MenuItem} The MenuItem that was clicked on.
* @param {Menu} The Menu instance of which this MenuItem is a             member.
* @param {number=} Index of the column the user clicked. May be null if the             user activated the menu via a keyboard event.
*/
click(target:Canvas, item:MenuItem, menu:Menu, colNum?:number): void;

/**
*  Action to fire when this menu is activated.
* 
*/
action(): void;



}


/**
* Opaque message identifier for messages shown by the Notify class
*/
interface MessageID {




}


/**
* Defines a set of properties that are applicable to
*  FacetChart.extraAxisMetrics of a FacetChart.
*  
*  These properties behave identically to the same-named properties of FacetChart. Two
*  exceptions are the properties
*  MetricSettings.gradationLineProperties and
*  MetricSettings.gradationZeroLineProperties
*  because they are used to draw tick marks along an extra value axis instead of lines
*  through the chart rectangle. Also the default behavior of the
*  MetricSettings.dataColors property is different, in that the
*  default is to use unused colors from, and in the order of, the 
*  FacetChart.dataColors.
*  
*  The extra value axes that are not positioned directly adjacent to the
*  FacetChart.showChartRect of the FacetChart may additionally
*  have an axis line to connect the gradation tick marks. Whether to draw this
*  line is controlled by the MetricSettings.showAxisLine property,
*  and the properties of the line are specified using
*  MetricSettings.axisLineProperties. The default is to draw
*  the axis lines whenever the chart rect. is drawn and to draw the line using the same
*  properties as the lines of the chart rect. (for example, lineColor and lineWidth).
*  
*  The extra value axis charts may be 
*  MetricSettings.multiFacet charts. If the underlying
*  FacetChart.data is multi-facet then the extra value axes may use the facet
*  whose values are labelled on the data axis or the facet used to create the legend (the
*  first and second facets, respectively). Both types of charts vary the first facet to
*  get a data series to plot. Multi-facet charts also vary the second facet, but
*  single-facet charts keep the second facet fixed at a constant FacetValue.
*  
*  The exact facet value to use for the second facet is specified using the
*  MetricSettings.fixedFacetValue property.
*/
interface MetricSettings {
/**
* Whether this extra value axis plots values while varying the facet values of just the
*  first facet (single-facet) or both first and second facets (multi-facet).
* @type {boolean}
* @default true
*/
multiFacet?: boolean;

/**
* For a MetricSettings.multiFacet chart of an extra value axis, this property provides a constant facet value
*  for the second facet. By varying the value of the other facet in multi-facet data, the chart
*  obtains the series of values to plot. The default facet value is the first facet value of the second facet.
* @type {string | number}
* @default null
*/
fixedFacetValue?: string | number;

/**
* For MetricSettings.multiFacet charts embedded in a multi-facet main
*  chart, the legendLabel defines the text of the legend label for this chart.
*  The default text is the FacetValue.title of the metric facet value of this
*  value axis concatenated with the FacetValue.title of the
*  MetricSettings.fixedFacetValue in parentheses. Set the
*  legendLabel to provide custom text for the legend label.
* @type {string}
* @default null
*/
legendLabel?: string;

/**
* Whether to show the extra value axis.
* @type {boolean}
* @default null
*/
showAxis?: boolean;

/**
* When this property is set to the metric of another MetricSettings object, the extra
*  value axis and chart corresponding to these settings will use the same scale for the
*  gradations as the extra value axis and chart of the other MetricSettings object. The
*  value of matchGradations can only be one of the metrics of the metric
*  facet whose values will be displayed by the chart.
* @type {string}
* @default null
*/
matchGradations?: string;

/**
* Whether to show an axis line for this extra value axis if it is not placed
*  directly adjacent to the chart rect. The default setting is the value
*  of the FacetChart.showChartRect property of the FacetChart.
* @type {boolean}
* @default null
*/
showAxisLine?: boolean;

/**
* Properties for the axis line drawn if this extra value axis is not positioned
*  directly adjacent to the chart rect. The default is to match the
*  DrawItem.lineCap, DrawItem.lineColor,
*  DrawItem.lineOpacity, DrawItem.linePattern,
*  and DrawItem.lineWidth of the FacetChart's
*  FacetChart.chartRectProperties for each axis line.
* @type {Partial<DrawLine>}
* @default null
*/
axisLineProperties?: Partial<DrawLine>;

/**
* A label for the data values, such as "Sales in Thousands", typically used as the label for
*  the value axis.
* @type {string}
* @default null
*/
valueTitle?: string;

/**
* Whether to show the MetricSettings.valueTitle (or, in the case of
*  MetricSettings.proportional,
*  MetricSettings.proportionalAxisLabel) as a label on this extra value axis.
* @type {boolean}
* @default null
*/
showValueAxisLabel?: boolean;

/**
* Properties for labels of value axis.
* @type {Partial<DrawLabel>}
* @default null
*/
valueAxisLabelProperties?: Partial<DrawLabel>;

/**
* See ChartType for a list of known types - Column, Bar, Line, Pie, Doughnut,
*  Area, Radar, and Histogram charts are supported.
* @type {ChartType}
* @default null
*/
chartType?: ChartType;

/**
* If the ChartType is "Column" then the metric settings may include
*  a setting for FacetChart.stacked.
*  
* @type {boolean}
* @default null
*/
stacked?: boolean;

/**
* Whether shapes are filled, for example, whether a multi-series line chart appears as a
*  stack of filled regions as opposed to just multiple lines.
*  
*  If unset, fills will be automatically used when there are multiple facets and stacking
*  is active (so Line and Radar charts will show stacked regions). 
*  
*  You can explicitly set filled:false to create multi-facet Line or Radar charts where
*  translucent regions overlap, or filled:true to fill in a single-facet Line or Radar
*  chart.
* @type {boolean}
* @default null
*/
filled?: boolean;

/**
* For multi-facet charts, render data values as a proportion of the sum of all data
*  values that have the same label.
*  
*  Gradation labels will be switched to show percentage instead of absolute values.
*  
*  This setting is valid only for Column, Bar, Area and Radar chart types and only in
*  MetricSettings.stacked mode. Stacked columns will be as tall as the chart rect and
*  stacked bars will be as wide as the chart rect. Area and Radar charts will be
*  completely filled except for facet values where all values are 0.
* @type {boolean}
* @default null
*/
proportional?: boolean;

/**
* Default title for the value axis label when the chart is in
*  MetricSettings.proportional. This title will be used unless the
*  FacetChart.getLegendFacet defines a
*  Facet.proportionalTitle.
* @type {string}
* @default "Percent"
*/
proportionalAxisLabel?: string;

/**
* Same as FacetChart.minDataSpreadPercent but affects only one metric. Default of
*  null means that the chart-wide setting facetChart.minDataSpreadPercent
*  will be used.
* @type {Integer}
* @default null
*/
minDataSpreadPercent?: Integer;

/**
* Same as FacetChart.axisStartValue but affects only one metric.
* @type {Float}
* @default null
*/
axisStartValue?: Float;

/**
* Same as FacetChart.xAxisEndValue but affects only one metric.
* @type {Float}
* @default null
*/
xAxisEndValue?: Float;

/**
* Properties for gradation labels
* @type {Partial<DrawLabel>}
* @default null
*/
gradationLabelProperties?: Partial<DrawLabel>;

/**
* Properties for gradation lines
* @type {Partial<DrawLine>}
* @default null
*/
gradationLineProperties?: Partial<DrawLine>;

/**
* Properties for the gradation line drawn for zero (slightly thicker by default).
* @type {Partial<DrawLine>}
* @default null
*/
gradationZeroLineProperties?: Partial<DrawLine>;

/**
* For Line, Area, Radar, Scatter or Bubble charts, whether to show data points for each
*  individual data value.
*  
*  If shown, the MetricSettings.pointClick and MetricSettings.getPointHoverHTML APIs can be used to create
*  interactivity.
* @type {boolean}
* @default false
*/
showDataPoints?: boolean;

/**
* Size in pixels for data points drawn for line, area, radar and other chart types.
* @type {Integer}
* @default "6"
*/
dataPointSize?: Integer;

/**
* Common properties to apply for all data points (see MetricSettings.showDataPoints).
* @type {Partial<DrawItem>}
* @default null
*/
dataPointProperties?: Partial<DrawItem>;

/**
* Whether to use logarithmic scaling for values.
*  
*  Logarithmic scale charts show an equivalent percentage increase as equivalent distance
*  on the chart. That is, 10 and 100 are the same distance apart as 100 and 1000 (each
*  being a 10 times or 1000% increase).
* @type {boolean}
* @default false
*/
logScale?: boolean;

/**
* Whether to use classic logarithmic gradations, where each order of magnitude is shown as
*  a gradation as well as a few intervening lines.  Gradations also begin and end on an order
*  of magnitude. For example, 1,2,4,6,8,10,20,40,60,80,100.
*  
*  Default gradations can be overridden via MetricSettings.logBase and MetricSettings.logGradations.
* @type {boolean}
* @default false
*/
useLogGradations?: boolean;

/**
* When MetricSettings.useLogGradations, base value for logarithmic gradation lines. Gradation
*  lines will be shown at every power of this value plus intervening values specified by
*  MetricSettings.logGradations.
* @type {Integer}
* @default "10"
*/
logBase?: Integer;

/**
* When MetricSettings.useLogGradations is set, gradation lines to show in between powers,
*  expressed as a series of integer or float values between 1 and MetricSettings.logBase.
*  
*  Some common possibilities (for base 10):
*  
*   [ 1 ] // show only orders of magnitude (0.1, 1, 10, 100, etc)
*   [ 1, 5 ] // show only orders of magnitude plus halfway mark
*   [ 1, 2, 4, 8 ] // show powers of 2 between orders
*   [ 1, 2.5, 5, 7.5 ] // show quarters
*  
*  Or base 2:
*  
*   [ 1 ]
*   [ 1, 1.5 ]
*  
* @type {Array<Partial<Float>>}
* @default "[ 1,2,4,6,8 ]"
*/
logGradations?: Array<Partial<Float>>;

/**
* The FacetChart.decimalPrecision used to render the numeric labels of this metric axis.
* @type {Integer}
* @default "2"
*/
decimalPrecision?: Integer;

/**
* Whether to automatically show shadows for various charts.
* @type {boolean}
* @default true
*/
showShadows?: boolean;

/**
* Properties for shadows.
* @type {Partial<DrawOval>}
* @default null
*/
shadowProperties?: Partial<DrawOval>;

/**
* Properties for lines that show data (as opposed to gradations or borders around the data
*  area).
* @type {Partial<DrawLine>}
* @default null
*/
dataLineProperties?: Partial<DrawLine>;

/**
* Properties for lines that outline a data shape (in filled charts such as area or radar
*  charts).
* @type {Partial<DrawItem>}
* @default null
*/
dataOutlineProperties?: Partial<DrawItem>;

/**
* Properties for data shapes (filled areas in area or radar charts).
* @type {Partial<DrawPath>}
* @default null
*/
dataShapeProperties?: Partial<DrawPath>;

/**
* Properties for a "value line" - a line shows where a particular discrete value is
*  placed, eg, vertical lines connecting points of a line chart to the X axis, or radial
*  lines in a Radar chart.
* @type {Partial<DrawLine>}
* @default null
*/
valueLineProperties?: Partial<DrawLine>;

/**
* An array of colors to use for a series of visual elements representing data (eg columns,
*  bars, pie slices), any of which may be adjacent to any other.
*  
*  Colors must be in the format of a leading hash (#) plus 6 hexadecimal digits, for example, 
*  "#FFFFFF" is white, "#FF0000" is pure red.
* @type {Array<Partial<CSSColor>>}
* @default "see below"
*/
dataColors?: Array<Partial<CSSColor>>;

/**
* A dictionary of gradients to use for a series of visual elements representing data (eg columns,
*  bars, pie slices), any of which may be adjacent to any other.
* @type {any}
* @default "see below"
*/
dataGradients?: any;

/**
* Should data values be shown as text labels near the shape representing the value, for example,
*  above columns of a column chart, or adjacent to points in a line chart?
*  
*  If set to false, then data values will not be shown.
*  
*  If set to true, data values will be shown unless the data density is high enough that
*  labels will potentially overlap, in which case, data values will not be shown and
*  hovers will be shown instead, in the same way as MetricSettings.showValueOnHover shows hovers.
* @deprecated * in favor of FacetChart.showDataValuesMode, a compound setting that supports showing data-values in the chart and in hovers in various combinations. The
*  equivalent to showDataValues:true is ShowDataValuesMode.inChartOnly 
*  or ShowDataValuesMode.inChartOrHover if showValueOnHover was also
*  set to true.

* @type {boolean}
* @default null
*/
showDataValues?: boolean;

/**
* Shows the value of the nearest data value in a floating label whenever the mouse moves
*  within the main chart area. The visual element representing the data
*  value will also be emphasized by brightening or highlighting it (appearance differs by
*  chart type).
*  
*  Calculates nearest value based on FacetChart.getNearestDrawnValue.
*  
*  The data value will be formatted using FacetChart.formatDataValue.
*  The label's appearance is controlled by FacetChart.hoverLabelProperties.
* @deprecated * in favor of FacetChart.showDataValuesMode, a compound setting that supports showing data-values in the chart and in hovers in various combinations. The
*  equivalent to showValueOnHover:true is ShowDataValuesMode.inHoverOnly.

* @type {boolean}
* @default null
*/
showValueOnHover?: boolean;



/**
*  Returns the X coordinate where the passed data value either was or would be drawn.
*  For example, this would be the X coordinate where a bar would end in a bar
*  chart.
*  
*  This is only allowed to be called after FacetChart.chartDrawn fires.
*  
*  If the FacetChart.chartType is
*  "Bar", "Bubble", or "Scatter"
*  
*  then the value argument should be a number. For other rectangular charts,
*  this method expects a FacetValueMap that uniquely identifies the data cell whose
*  X-axis coordinate is to be retrieved.
*  
*  Note that when FacetChart.canZoom is enabled, this API is valid only for
*  data values between FacetChart.zoomStartValue and
*  FacetChart.zoomEndValue.
* 
* 
* @param {Float | FacetValueMap} the value to be drawn.
*/
getXCoord(value:Float | FacetValueMap): Float;

/**
*  Returns the Y coordinate where the passed data value either was or would be drawn.
*  For example, this would be the Y coordinate that a line would pass through on
*  a line chart, or the top of a column on a column chart.
*  
*  This is only allowed to be called after FacetChart.chartDrawn fires.
*  
*  If the FacetChart.chartType is
*  "Area", "Bubble", "Column", "Histogram", "Line", or "Scatter"
*  
*  then the value argument should be a number. For
*  "Bar"
*  charts this method expects a FacetValueMap that uniquely identifies the data cell
*  whose Y-axis coordinate is to be retrieved.
* 
* 
* @param {Float} the value to be drawn.
*/
getYCoord(value:Float): Float;

/**
*  Return an array of the gradation values used in the current chart. Pass these values
*  to MetricSettings.getXCoord / MetricSettings.getYCoord (depending on the orientation of the chart)
*  to discover the coordinates where gradations are drawn.
*  
*  This is only allowed to be called when FacetChart.chartDrawn fires.
* 
* 
*/
getGradations(): Array<Partial<Float>>;

/**
*  Method to change the current ChartType.
*  Will redraw the chart if drawn. 
*  Will use default settings for the new chart type for MetricSettings.stacked
*  and MetricSettings.filled if those values are null.
*  
*  Note that for FacetChart.extraAxisMetrics charts this method changes the
*  chartType for the main value axis only.
* 
* 
* @param {ChartType} new chart type
*/
setChartType(chartType:ChartType): void;

/**
*  Method to change MetricSettings.stacked. Use null to
*  apply a default value for the current ChartType.
* 
* 
* @param {boolean} new value
*/
setStacked(stacked:boolean): void;

/**
*  Method to change MetricSettings.filled.
*  Use null to apply a default value for the current ChartType.
* 
* 
* @param {boolean} new value
*/
setFilled(filled:boolean): void;

/**
*  Setter for MetricSettings.proportional.
* 
* @param {boolean} Whether the chart should now use proportional mode.
*/
setProportional(proportional:boolean): void;

/**
*  Setter for MetricSettings.dataColors.
* 
* @param {Array<Partial<CSSColor>>} New set of data colors
*/
setDataColors(dataColors:Array<Partial<CSSColor>>): void;

/**
*  Get a color from the MetricSettings.dataColors Array.
*  
*  
*  Override to provide a dynamic color generation scheme.
*  
*  In most cases, the method is passed the associated data Record, which can
*  be used to retrieve the value being processed.
*  
*  Must return a color in the format of a leading hash (#) plus 6 hexadecimal digits 
*  as specified for MetricSettings.dataColors.
*  
* 
* 
* @param {number} index of the legend facet value to be colored
* @param {string | number | Date} id of the legend facet value to be colored
* @param {string} purpose for the requested color - such as "legend", "line",             "area", "points", etc.
*/
getDataColor(index:number, facetValueId:string | number | Date, purpose:string): CSSColor;

/**
*  Specifies the color to use for data lines in the chart. No default implementation. If not
*  defined or null is returned, the Framework will default to value of MetricSettings.getDataColor.
*  
*  
*  Note that this method is simply an override point, since it has no default implementation -
*  must return a color in the format of of a leading hash (#) plus 6 hexadecimal digits as
*  specified for MetricSettings.dataColors.
*  
* 
* 
* @param {number} index of the legend facet value to be colored
* @param {string | number | Date} id of the legend facet value to be colored
* @param {string} purpose for the requested color - such as "legend", "line",             "area", "points", etc.
*/
getDataLineColor(index:number, facetValueId:string | number | Date, purpose:string): CSSColor;

/**
*  Specifies the width to use for data lines in the chart. No default implementation. If not
*  defined or null is returned, the line width will be set by the appropriate chart properties,
*  such as FacetChart.dataLineProperties, FacetChart.barProperties, or FacetChart.bubbleProperties.
*  
*  
*  Note that this method is simply an override point, since it has no default implementation.
*  
* 
* 
* @param {number} index of the legend facet value to target
* @param {string | number | Date} id of the legend facet value to target
* @param {string} purpose for the requested width - such as "legend", "line",             "area", "points", etc.
*/
getDataLineWidth(index:number, facetValueId:string | number | Date, purpose:string): Integer;

/**
*  Get a gradient from the MetricSettings.dataGradients Array.
*  
*  Override to provide a dynamic gradient generation scheme.
* 
* 
* @param {number} index of the legend facet value to be colored
* @param {string | number | Date} id of the legend facet value to be colored
* @param {string} purpose for the requested gradient - such as "legend", "line",             "area", "points", etc.
*/
getDataGradient(index:number, facetValueId:string | number | Date, purpose:string): string;

/**
*  When MetricSettings.showDataPoints is true, fires when a point is clicked on.
* 
* 
* @param {Float} the value at the point
* @param {Record} the record at the point
* @param {string} the ID of the metric at the point
*/
pointClick(value:Float, record:Record, metricId:string): void;

/**
*  When MetricSettings.showDataPoints is true and the mouse hovers over a point, this method is called
*  and may return HTML to show in a hover.
* 
* 
* @param {Float} the value at the point
* @param {Record} the record at the point
* @param {string} the ID of the metric at the point
*/
getPointHoverHTML(value:Float, record:Record, metricId:string): string;

/**
*  Return the text string to display in MetricSettings.gradationLabelProperties
*  given the raw value for the metric to show on the value axis. This formatter will only
*  be called if the axis has gradation labels, meaning labels drawn at regular intervals not
*  associated with any particular facet value.
*  
*  Note that the rendering of values in hovers or via MetricSettings.showDataValues is handled by
*  MetricSettings.formatDataValue.
* 
* @param {any} raw value of the metric
*/
formatAxisValue(value:any): string;

/**
*  Return the text string to display in labels 
*  FacetChart.showDataValuesMode given the raw value 
*  for the metric displayed on the value axis.
*  
*  This method may also be passed the FormattingContext for which the 
*  value is being formatted, the record associated with the value and the id of its 
*  FacetChart.facets.
*  
*  Note that the rendering of values for gradation labels is handled by
*  MetricSettings.formatAxisValue.
* 
* @param {any} raw value of the metric
*/
formatDataValue(value:any): string;



}


/**
* Settings used to control the export or serialization of a DataSource by
*  Reify.getMockDS.
*/
interface MockDSExportSettings {
/**
* Determines the format emitted by Reify.getMockDS.
* @type {MockDSExportFormat}
* @default "xmlMockDS"
*/
format?: MockDSExportFormat;

/**
* Should DataSourceField.foreignKey relationships be included in the
*  export or serialization of the DataSource?
* @type {boolean}
* @default true
*/
includeFKs?: boolean;

/**
* If including DataSourceField.foreignKey relationships, those
*  relationships to skip. This can be used to avoid dangling references to 
*  DataSource that are not being exported or serialized.
* @type {Array<Partial<string>>}
* @default null
*/
omitRelations?: Array<Partial<string>>;

/**
* Whether to include custom-defined SimpleType.
* @type {boolean}
* @default false
*/
includeCustomSimpleTypes?: boolean;

/**
* Should FieldType be included in the export or serialization of the
*  DataSource? They are excluded by default since the stored paths are unlikely to
*  be correct when placed in any other environment, such as Reify.
* @type {boolean}
* @default false
*/
includeImageFields?: boolean;

/**
* The Criteria used to fetch the records returned as part of the
*  export or serialization.
* @type {Criteria}
* @default null
*/
criteria?: Criteria;

/**
* The number of rows of data to include, if more meet the MockDSExportSettings.criteria.
* @type {Integer}
* @default "20"
*/
numRows?: Integer;

/**
* The number of levels of nodes to include, for DataSources that define a
*  treeDataBinding between fields by declaring a
*  DataSourceField.foreignKey on one field that refers to another from
*  that same DataSource.
* @type {Integer}
* @default "3"
*/
numLevels?: Integer;

/**
* For DataSources that define a treeDataBinding between
*  fields by declaring a DataSourceField.foreignKey on one field that
*  refers to another from that same DataSource, should MockDSExportSettings.criteria be applied only
*  to the root node? If false, the criteria will be applied to all nodes.
* @type {boolean}
* @default false
*/
rootCriteriaOnly?: boolean;

/**
* The properties that will be specified on the DSRequest when fetching records.
*  You can pass an array of different request properties matching the length of the
*  dsNames param of Reify.getMockDS or Reify.showMockDS if you want the
*  fetch for each DataSource made with different properties.
* @type {Array<Partial<Partial<DSRequest>>> | Partial<DSRequest>}
* @default null
*/
requestProperties?: Array<Partial<Partial<DSRequest>>> | Partial<DSRequest>;

/**
* Controls which Validator, if any, to include in the fields of the
*  exported MockDataSource. Since MockDataSources are client-only, server-only
*  validators are not exported. Auto-generated validators are also not exported, since
*  they will be recreated based on the type of the field during the import process.
* @type {MockDSExportValidatorMode}
* @default "standard"
*/
validatorMode?: MockDSExportValidatorMode;





}


/**
* Allows specifying settings to apply to the MultiWindow of a child window.
*/
interface MultiWindowSettings {
/**
* Should DataSource from other OpenFin windows with SmartClient loaded
*  be copied by reference into this window? Such DataSources will be copied:
*  
* - when a page in another window is loaded (potentially several DataSources at once)
*  
* - at the moment DataSources are created in a page loaded in another window (just that
*  DataSource)
*  
*  This property will default to true if OpenFin is present; otherwise, false.
*  
*  Note that reloading a page that created DataSources copied by reference into other
*  windows (via this property) is not supported.
* @type {boolean}
* @default "varies"
*/
autoCopyDataSources?: boolean;

/**
* Should this window share messaging channels with other
*  windows?
*  
*  This property will default to true if OpenFin is present; otherwise, false.
* @type {boolean}
* @default "varies"
*/
shareMessageChannels?: boolean;





}


/**
* Properties for a navigation item in a NavPanel.
*/
interface NavItem extends TreeNode {
/**
* An optional ID for this NavItem. If specified, this must be unique within
*  the NavPanel.
* @type {Identifier}
* @default null
*/
id?: Identifier;

/**
* Title to show for this NavItem.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* Icon to show for this NavItem. If not specified, the
*  ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon','navGrid\'s folderIcon')} is used.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* Optional subitems of this NavItem.
* @type {Array<Partial<NavItem>>}
* @default null
*/
items?: Array<Partial<NavItem>>;

/**
* If set, this NavItem will be styled like a header. In this case NavItem.pane
*  is ignored and nothing happens when the header is clicked. However, NavItem.items can
*  still be configured to place items hierarchically under the header.
* @type {boolean}
* @default null
*/
isHeader?: boolean;

/**
* CSS style name used for this NavItem. If set and this NavItem
*  is a NavItem.isHeader, this overrides the NavPanel's
*  NavPanel.headerStyle.
* @type {CSSStyleName}
* @default null
*/
customStyle?: CSSStyleName;

/**
* If set, this NavItem will be styled as a separator. A separator does not
*  have a NavItem.pane and nothing happens when the separator is clicked.
* @type {boolean}
* @default null
*/
isSeparator?: boolean;

/**
* Component to display in the NavPanel.navDeck when this NavItem is
*  selected.
*  
*  A component can be provided directly, or its ID can be provided.
* @type {Canvas | Identifier}
* @default null
*/
pane?: Canvas | Identifier;





}


/**
* Encapsulates state of a NavigationBar's view. A NavigationBarViewState
*  object is created to pass to NavigationBar.setViewState so that multiple properties
*  of the NavigationBar can be changed at once.
*/
interface NavigationBarViewState {
/**
* The new NavigationBar.showLeftButton setting. If unset, the
*  showLeftButton setting is not changed.
* @type {boolean}
* @default null
*/
showLeftButton?: boolean;

/**
* The new NavigationBar.leftButtonTitle setting. If unset, the
*  leftButtonTitle is not changed.
* @type {HTMLString}
* @default null
*/
leftButtonTitle?: HTMLString;

/**
* The new NavigationBar.shortLeftButtonTitle setting. If unset, the
*  shortLeftButtonTitle is not changed.
* @type {HTMLString}
* @default null
*/
shortLeftButtonTitle?: HTMLString;

/**
* The new NavigationBar.alwaysShowLeftButtonTitle setting. If unset, the
*  alwaysShowLeftButtonTitle setting is not changed.
* @type {boolean}
* @default null
*/
alwaysShowLeftButtonTitle?: boolean;

/**
* The new NavigationBar.title setting. If unset, the title is not changed.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* The new NavigationBar.controls setting. If unset, the controls array
*  is not changed.
* @type {Array<Partial<string>> | Array<Partial<Canvas>>}
* @default null
*/
controls?: Array<Partial<string>> | Array<Partial<Canvas>>;





}


/**
* An object containing sufficient context to unambiguously identify a single node in the tree.
*  For normal trees, the node itself - or its ID - is sufficient for this purpose, but for 
*  Tree.isMultiLinkTree, we also need to know the node's parent, and its
*  position within that parent. For cases where we need to propagate change back up the 
*  node's parent chain, in order to maintain a given parent node's openList, the node, parent 
*  and position are not enough context; for those cases, we need either the node's position in
*  the tree's openList, or a full path to the node. NodeLocator objects contain 
*  this extra context, and can be passed to APIs such as Tree.openFolder, which would
*  ordinarily accept a parameter of type TreeNode.
*/
interface NodeLocator {




}


/**
* Represents an action that's associated with a message. Similar to the object form of
*  Callback, except a title must also be specified, which is rendered as a clickable
*  link in the message (unless NotifyAction.wholeMessage is set).
*/
interface NotifyAction {
/**
* The title of the action to render into the message.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* Overrides NotifySettings.actionSeparator for this action.
* @type {HTMLString}
* @default null
*/
separator?: HTMLString;

/**
* The object that will be passed as this when the action is executed.
* @type {object}
* @default null
*/
target?: object;

/**
* The method to invoke on the NotifyAction.target when the action is executed.
* @type {string}
* @default null
*/
methodName?: string;

/**
* Allows a click anywhere on the notification to execute the action. If true, the action won't
*  be rendered as a link.
* @type {boolean}
* @default null
*/
wholeMessage?: boolean;





}


/**
* An object used to configure how messages shown by Notify.addMessage are drawn and
*  behave.
*/
interface NotifySettings {
/**
* Length of time a message is shown before being auto-dismissed, in milliseconds.
*  A value of 0 means that the message will not be dismissed automatically.
*  Messages can always be dismissed by calling Notify.dismissMessage or,
*  if NotifySettings.canDismiss is set, by performing a "close click".
* @type {Integer}
* @default "5000"
*/
duration?: Integer;

/**
* If true, pauses the auto-dismiss countdown timer when the mouse is over the
*  messasge.
* @type {boolean}
* @default false
*/
stayIfHovered?: boolean;

/**
* Where to show the message, as a viewport-relative x coordinate offset to the
*  left edge of the Label rendering the message. Properties NotifySettings.position
*  and NotifySettings.positionCanvas will only be used to place messages if coordinates
*  aren't provided.
* @type {Integer}
* @default null
*/
x?: Integer;

/**
* Where to show the message, as a viewport-relative y coordinate offset to the top
*  edge of the Label rendering the message.
* @type {Integer}
* @default null
*/
y?: Integer;

/**
* Where to show the message, specified as an edge ("T", "B", "R", "L"), a corner
*  ("TL", "TR", "BL", "BR), or "C" for center, similar to Canvas.snapTo.
*  If an edge is specified, the message will be shown at its center (or the very
*  center for "C"). Only used if NotifySettings.x haven't been provided.
*  
*  If a NotifySettings.positionCanvas has been specified, the position is
*  interpreted relative to it instead of the viewport, and this property defaults
*  to "C". Otherwise, if no positionCanvas is present, the default is
*  to use NotifySettings.slideInOrigin or NotifySettings.slideOutOrigin, or "L" if neither property
*  is defined.
*  
*  To place the message at an offset from the specified position, use NotifySettings.leftOffset
*  or NotifySettings.topOffset.
* @type {EdgeName}
* @default "varies"
*/
position?: EdgeName;

/**
* Canvas over which to position the message, available as an alternative means of
*  placement if viewport-relative NotifySettings.x aren't provided. Note that
*  the canvas is only used to compute where to the place message, and will not be
*  altered.
* @type {Canvas}
* @default null
*/
positionCanvas?: Canvas;

/**
* Specifies a left offset from the position specified by NotifySettings.position or
*  NotifySettings.positionCanvas where the message should be shown. Ignored if
*  NotifySettings.x are provided to position the message.
* @type {Integer}
* @default null
*/
leftOffset?: Integer;

/**
* Specifies a top offset from the position specified by NotifySettings.position or
*  NotifySettings.positionCanvas where the message should be shown. Ignored if
*  NotifySettings.y are provided to position the message.
* @type {Integer}
* @default null
*/
topOffset?: Integer;

/**
* Controls how messages appear at or reach their requested location. The default
*  of "slide" is recommended because the motion will draw the user's attention to
*  the notification.
* @type {NotifyTransition}
* @default "slide"
*/
appearMethod?: NotifyTransition;

/**
* Controls how messages disappear from or leave their requested location. The
*  default of "fade" is recommended because a slide animation would draw too much
*  attention to a notification that is no longer current, whereas a subtle fade
*  should draw a minimum of attention (less even than instantaneously
*  disappearing).
* @type {NotifyTransition}
* @default "fade"
*/
disappearMethod?: NotifyTransition;

/**
* Controls how the stack or message is repositioned, if required, after
*  Notify.setMessageContents has been called. Valid values are "slide" and
*  "instant".
* @type {NotifyTransition}
* @default "slide"
*/
repositionMethod?: NotifyTransition;

/**
* Displays an icon to allow a message to be dismissed through the UI. Messages
*  can always be dismissed programmatically by calling
*  Notify.dismissMessage.
* @type {boolean}
* @default false
*/
canDismiss?: boolean;

/**
* Determines where messages originate when they appear for NotifySettings.appearMethod:
*  "slide". Possible values are "L", "R", "T", and "B".
*  
*  If not specified, the edge nearest the message's requested coordinates or
*  position is used.
* @type {string}
* @default null
*/
slideInOrigin?: string;

/**
* Determines where messages go when they disappear for NotifySettings.disappearMethod:
*  "slide". Possible values are "L", "R", "T", and "B".  If not specified, the
*  edge nearest the message's requested coordinates or position is used.
* @type {string}
* @default null
*/
slideOutOrigin?: string;

/**
* Animation speed for NotifyTransition: "slide", in pixels/second.
* @type {Integer}
* @default "300"
*/
slideSpeed?: Integer;

/**
* Time over which the fade-in effect runs for NotifyTransition: "fade", in
*  milliseconds.
* @type {Integer}
* @default "500"
*/
fadeInDuration?: Integer;

/**
* Time over which the fade-out effect runs for NotifyTransition: "fade", in
*  milliseconds.
* @type {Integer}
* @default "500"
*/
fadeOutDuration?: Integer;

/**
* Determines what happens if a message appears while there's still another one of
*  the same NotifyType being shown. Such messages are either stacked or
*  replace one another,
* @type {MultiMessageMode}
* @default "stack"
*/
multiMessageMode?: MultiMessageMode;

/**
* Determines how messages are stacked if MultiMessageMode is "stack". For
*  example, "down" means that older messages move down when a new message of the
*  same NotifyType appears.
* @type {StackDirection}
* @default "down"
*/
stackDirection?: StackDirection;

/**
* Space between each message when MultiMessageMode is "stack".
* @type {Integer}
* @default "2"
*/
stackSpacing?: Integer;

/**
* Sets a limit on how many messages may be stacked if MultiMessageMode is
*  "stack". The oldest message of the affected NotifyType will be dismissed
*  to enforce this limit.
* @type {Integer}
* @default "3"
*/
maxStackSize?: Integer;

/**
* Controls how older messages' NotifySettings.duration countdowns are affected when a new
*  message of the same NotifyType appears. We either continue the
*  countdowns on the older messages as if they are unrelated, or we reset any
*  countdowns that are less than the new message's duration.
*  
*  Note that you can set this property in a call to Notify.addMessage even
*  though it has "stack" in its name, since it governs the logic run on behalf of
*  this message.
* @type {StackPersistence}
* @default "none"
*/
stackPersistence?: StackPersistence;

/**
* Specifies how to pick which message to dismiss when the NotifySettings.maxStackSize is
*  reached, and the lowest priority value (highest numerical 
*  NotifySettings.messagePriority) is shared by more than one
*  message. 
*  
*  We can simply dismiss the oldest message of that 
*  NotifySettings.messagePriority, or we can pick the message
*  with the least time left until it's auto-dismissed.
* @type {MaxStackDismissMode}
* @default "oldest"
*/
maxStackDismissMode?: MaxStackDismissMode;

/**
* HTML to be added before each action to separate it from the previous action.
*  For the first action, it will only be added if the message contents aren't
*  empty.
*  
*  You may override this on a per action basis using NotifyAction.separator.
*  
*  Besides the default, some other known useful values are "&amp;emsp;" and "&amp;nbsp;".
* @type {HTMLString}
* @default " "
*/
actionSeparator?: HTMLString;

/**
* Optional icon to be shown in the Label drawn for this message. Default is
*  
*  
* - "[SKIN]/Notify/error.png" for NotifyType: "error",
*  
* - "[SKIN]/Notify/warning.png" for NotifyType: "warn", and
*  
* - "[SKIN]/Notify/checkmark.png" for all other NotifyTypes.
*  
*  However, if you specify a NotifySettings.messagePriority, 
*  it will determine the default rather than the actual NotifyType, if
*  NotifySettings.applyPriorityToAppearance is true.
* @type {SCImgURL}
* @default "varies"
*/
messageIcon?: SCImgURL;

/**
* Width in pixels of the icon image.
* @type {Integer}
* @default "17"
*/
messageIconWidth?: Integer;

/**
* Height in pixels of the icon image.
* @type {Integer}
* @default "17"
*/
messageIconHeight?: Integer;

/**
* Pixels between icon and title text.
* @type {Integer}
* @default "20"
*/
messageIconSpacing?: Integer;

/**
* Optional specified padding to apply after the message content when showing a
*  NotifySettings.canDismiss so that the button doesn't occlude any content.
*  Only needed if the message NotifySettings.styleName doesn't already provide
*  enough padding.
* @type {Integer}
* @default null
*/
messageControlPadding?: Integer;

/**
* Sets the priority of the message. Priority is used to determine which message to
*  dismiss if NotifySettings.maxStackSize is hit. Lower numerical values have higher
*  priority.
*  The default is:
*  
*  
* - Notify.ERROR for NotifyType: "error",
*  
* - Notify.WARN for NotifyType: "warn", and
*  
* - Notify.MESSAGE for all other NotifyTypes
*  
*  Impact on Appearance
*  
*  If you specify messagePriority, and NotifySettings.applyPriorityToAppearance
*  is set, the properties:
*  
*  
* - NotifySettings.messageIcon,
*  
* - NotifySettings.styleName, and 
*  
* - NotifySettings.actionStyleName
*  
*  will be assigned, if not specified, to the default values from:
*  
* - NotifyType: "error" for priority Notify.ERROR,
*  
* - NotifyType: "warn" for priority Notify.WARN, or
*  
* - NotifyType: "message" for priorities at or below 
*              Notify.MESSAGE (greater or equal numerically)
*  
*  This allows you to automatically set "error" or "warn" styling, on a per-message
*  basis, for any non-"error" or "warn" NotifyType by simply supplying
*  a messagePriority for that message.
* @type {MessagePriority}
* @default "varies"
*/
messagePriority?: MessagePriority;

/**
* Whether to default properties affecting the message appearance to those of the
*  built-in NotifyType corresponding to the 
*  NotifySettings.messagePriority. Default is true except
*  for NotifyTypes "error" and "warn", which default to false.
* @type {boolean}
* @default "varies"
*/
applyPriorityToAppearance?: boolean;

/**
* If an icon is present, should it appear to the left or right of the title?
*  valid options are "left" and "right". If unset,
*  default is "left" unless Page.isRTL is active, in which case it's
*  "right".
*  
*  Note that the icon will automatically be given an alignment matching its
*  orientation, so "left" for messageIconOrientation "left", and vice
*  versa.
* @type {string}
* @default "varies"
*/
messageIconOrientation?: string;

/**
* The CSS class to apply to the Label drawn for this message. Default is:
*  
*  
* - "notifyError" for NotifyType: "error",
*  
* - "notifyWarn" for NotifyType: "warn", and
*  
* - "notifyMessage" for all other NotifyTypes.
*  
*  However, if you specify a NotifySettings.messagePriority,
*  it will determine the default rather than the actual NotifyType, if
*  NotifySettings.applyPriorityToAppearance is true.
*  
*  Note that if Page.isRTL is active, the default will be as above, but
*  with an "RTL" suffix added.
* @type {CSSStyleName}
* @default "varies"
*/
styleName?: CSSStyleName;

/**
* The CSS class to apply to action text in this message. default is:
*  
*  
* - "notifyErrorActionLink" for NotifyType: "error",
*  
* - "notifyWarnActionLink" for NotifyType: "warn", and
*  
* - "notifyMessageActionLink" for all other NotifyTypes.
*  
*  However, if you specify a NotifySettings.messagePriority,
*  it will determine the default rather than the actual NotifyType, if
*  NotifySettings.applyPriorityToAppearance is true.
* @type {CSSStyleName}
* @default "varies"
*/
actionStyleName?: CSSStyleName;

/**
* If true, the specified width of the Label drawn for this message will be
*  treated as a minimum width. If the message content string exceeds this, the
*  Label will expand to accommodate it up to NotifySettings.autoFitMaxWidth (without
*  the text wrapping).
*  
*  Using this setting differs from simply disabling wrapping via
*  Label.wrap as the content will wrap if the
*  NotifySettings.autoFitMaxWidth is exceeded.
* @type {boolean}
* @default null
*/
autoFitWidth?: boolean;

/**
* Maximum auto-fit width for a message if NotifySettings.autoFitWidth is enabled. May be
*  specified as a pixel value, or a percentage of page width.
* @type {Integer | string}
* @default "300"
*/
autoFitMaxWidth?: Integer | string;

/**
* Configures the properties, such as Label.autoFit, Label.align, and
*  Label.width. of the Label autochildren that will be used to draw messages,
*  where not already determined by message layout or other NotifySettings
*  properties such as NotifySettings.styleName.
*  
*  Not all label properties are guaranteed to work here, as the Notify system is
*  assumed to layout message content and manage positioning messages. In particular,
*  the following properties should be avoided:
* 
*  
*  Property NameIssueGuidance
*  
*  margin
*  Layout and positioning of the messages is handled by the Notify system.
*  Use NotifySettings.stackSpacing to configure the separation between messages, and
*  NotifySettings.leftOffset and NotifySettings.topOffset to fine-tine stack
*  NotifySettings.position.
*  
*  
*  
*  padding
*  Padding is set by notification CSS so that children are positioned corrected
*  relative to content.
*  You can apply your own NotifySettings.styleName to messages via CSS. Or you can
*  use HTML as the message contents to create whatever sort of interior layout you
*  like.
*  
*  
*  wrap
*  Autowrap behavior is managed by the Notify system.
*  To have NotifySettings.autoFitWidth content not wrap, set
*  NotifySettings.autoFitMaxWidth higher than your expected message widths. You can set it to
*  "100%" if needed to allow the message to expand across the entire page.
*  
*  
* @type {Partial<Label>}
* @default null
*/
labelProperties?: Partial<Label>;





}




/**
* Specification of an operator for use in filtering, for example "equals".
*  Use with DataSource.addSearchOperator to define custom filtering behaviors for
*  client-side filtering.
*/
interface Operator {
/**
* Unique id for an operator, which appears within AdvancedCriteria as the
*  Operator property.
*  
*  A list of built-in identifiers is OperatorId.
* @type {OperatorId}
* @default null
*/
ID?: OperatorId;

/**
* User-visible title for this operator, such as "doesn't contain".
*  
*  To simplify internationalization by separating titles from operator code, you can use
*  specify Operator.titleProperty instead of this property.
* @type {string}
* @default null
*/
title?: string;

/**
* Name of a property on the Operators class that provides the title for this operator.
* @type {Identifier}
* @default null
*/
titleProperty?: Identifier;

/**
* User-visible title for this operator when used with text-based fields - eg, 
*  "equals (match case)" rather than just "equals".
*  
*  To simplify internationalization by separating titles from operator code, you can use
*  specify Operator.textTitleProperty instead of this property.
* @type {string}
* @default null
*/
textTitle?: string;

/**
* Name of a property on the Operators class that provides the title for this operator
*  when used with text-based fields.
* @type {Identifier}
* @default null
*/
textTitleProperty?: Identifier;

/**
* List of types that this Operator is valid for.
*  
*  If omitted, the operator is assumed to be valid for all FieldTypes unless a list of 
*  FieldTypes is passed to DataSource.addSearchOperator.
* @type {Array<Partial<FieldType>>}
* @default null
*/
fieldTypes?: Array<Partial<FieldType>>;

/**
* Whether this operator needs to be executed on the server side.
*  
*  This implies that if a Criterion using this operator is either introduced into
*  AdvancedCriteria or is changed, the server will need to be contacted to
*  perform filtering.
* @type {boolean}
* @default false
*/
requiresServer?: boolean;

/**
* Whether this operator should be offered to users by default in interfaces such as the
*  FilterBuilder. 
*  
*  Setting hidden:true means the operator can be used in a programmatic search, for example,
*  by calling ResultSet.setCriteria, but does not appear in the UI.
* @type {boolean}
* @default false
*/
hidden?: boolean;

/**
* Indicates the kind of value expected in a Criterion that uses this operator.
*  OperatorValueType lists possibilities.
*  
*  The default of null is equivalent to "fieldType", indicating that
*  Criterion.value is expected to contain a value of the same type as the field
*  indicated by Criterion.fieldName.
* @type {OperatorValueType}
* @default null
*/
valueType?: OperatorValueType;

/**
* Usage hint text specific to this Operator and shown, for example, in the hover-text of 
*  ListGrid ListGrid.showFilterEditor fields.
*  
*  If unset, the default for all operators, this value is derived from an attribute on the 
*  Operators class, in the format [operator.valueType]UsageHint - for example, the 
*  Operators.betweenTitle operator uses Operators.valueRangeUsageHint.
* @type {string}
* @default null
*/
usageHint?: string;

/**
* For an operator with Operator.valueType:"custom", indicates what kind of FormItem to use to
*  provide a user interface for creating a valid Criterion. The default of
*  null means an ordinary TextItem is fine.
* @type {FormItemClassName}
* @default null
*/
editorType?: FormItemClassName;

/**
* The text use when using this operator as an 
*  FormItem.allowExpressions in a FormItem.
* @type {string}
* @default null
*/
symbol?: string;



/**
*  Method which actually evaluates whether a given record meets a Criterion.
*  
*  For operators that act on Criterion.criteria, call
*  DataSource.evaluateCriterion to evaluate sub-criteria.
*  
*  Because criteria are sometimes applied to user-entered data that has not been validated, a
*  robust condition() function should expect that data found in a Record
*  may be null, NaN, not the correct type (eg "NA" for a type:"date" field) or otherwise out of
*  the expected range.
*  
*  Note that an Operator has access both to the Criterion object, allowing operators
*  that act on more than one field or perform calculations, and access to the Operator
*  object itself, allowing a condition() function to be shared across a range of
*  related operators with different OperatorIds.
*  
* 
* @param {any} value from the field supplied as Criterion.fieldName, if  applicable
* @param {Record} record being evaluated
* @param {any} fieldName supplied as Criterion.fieldName, if applicable
* @param {Criterion} criterion definition
* @param {Operator} operator definition
*/
condition(value:any, record:Record, fieldName:any, criterion:Criterion, operator:Operator): boolean;

/**
*  Compare two criteria, both of which use this operator, and report whether the newCriteria is
*  definitely more restrictive than the previous criteria.
*  
*  This is used by the ResultSet to understand whether client-side filtering can
*  continue using cached data, or whether server-side filtering must be used instead.
*  
* 
* @param {Criterion} new criterion
* @param {Criterion} previous criterion
*/
compareCriteria(newCriterion:Criterion, oldCriterion:Criterion): number;

/**
*  In combination with Operator.editorType, this override point allows you to define a
*  client-side only Operator that simply provides a custom UI for creating a Criterion based on
*  one of the built-in operators.
*  
*  For example, the "between" operator allows AdvancedCriteria to be created that can select any
*  date range, however in a given application certain specific date ranges might be more
*  meaningful (eg "next week", "last quarter") and you might want to offer the user a picker for
*  those date ranges. You could create an operator "presetDateRange" with an editorType 
*  indicating a custom SelectItem that shows available ranges, and then implement 
*  operation.getCriterion() to take the value from this SelectItem and produce a Criterion 
*  selecting the chosen date range.
*  
*  Note that another approach, if it's not required that this custom interface appear in the
*  FilterBuilder, is just to have a separate DynamicForm for picking special date
*  ranges, and use DataSource.combineCriteria to merge the criteria with the
*  FilterBuilder's criteria, as in ${isc.DocUtils.linkForExampleId('dynamicReporting', 'this sample')}.
*  
*  If not implemented, returns the result of calling 
*  FormItem.getCriterion on the passed FormItem.
*  
* 
* @param {FieldName} * @param {FormItem} */
getCriterion(fieldName:FieldName, item:FormItem): Criterion;



}


/**
* An object representing a component which the user may create dynamically within an
*  application.
*  
*  A PaletteNode expresses visual properties for how the palette will display it (eg
*  PaletteNode.title, PaletteNode.icon) as well as instructions for
*  creating the component the paletteNode represents (PaletteNode.type,
*  PaletteNode.defaults).
*  
*  Various types of palettes (ListPalette, TreePalette, MenuPalette,
*  TilePalette) render a PaletteNode in different ways, and allow the user to trigger
*  creation in different ways (eg drag and drop, or just click). All share a common pattern
*  for how components are created from palettes.
*  
*  Note that in a TreePalette, a PaletteNode is essentially a TreeNode and can have
*  properties expected for a TreeNode (eg,
*  TreeGrid.customIconDropProperty). Likewise
*  a PaletteNode in a MenuPalette can have the properties of a MenuItem, such as
*  MenuItem.enableIf.
*/
interface PaletteNode {
/**
* Icon for this paletteNode.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* Textual title for this paletteNode.
* @type {string}
* @default null
*/
title?: string;

/**
* SCClassName this paletteNode creates, for example, "ListGrid".
* @type {SCClassName}
* @default null
*/
type?: SCClassName;

/**
* Prefix used to create unique component ID. If not specified, PaletteNode.type
*  is used.
* @deprecated * As of SmartClient version 12.1, deprecated in favor of PaletteNode.idName

* @type {string}
* @default null
*/
idPrefix?: string;

/**
* Name used to create unique component ID. If not specified, PaletteNode.type
*  is used.
*  
*  Note: idName must follow all rules for a Identifier.
* @type {string}
* @default null
*/
idName?: string;

/**
* Defaults for the component to be created from this palette. 
*  
*  For example, if PaletteNode.type is "ListGrid", properties valid to pass to
*  Class.create.
*  
*  Note that event handlers or method overrides cannot be configured as defaults,
*  as they cannot be serialized or restored. Instead, create a subclass that implements the
*  desired behaviors, and use that subclass as PaletteNode.type. 
*  
* @type {Properties}
* @default null
*/
defaults?: Properties;

/**
* Properties to be applied to the 
*  PaletteNode.liveObject.Canvas.editProxy when created.
* @type {Partial<EditProxy>}
* @default null
*/
editProxyProperties?: Partial<EditProxy>;

/**
* Properties to be applied to the EditNode when created.
* @type {Partial<EditNode>}
* @default null
*/
editNodeProperties?: Partial<EditNode>;

/**
* For a paletteNode which should be a "singleton", that is, always provides the exact same
*  object (==) rather than a dynamically created copy, provide the singleton object as
*  liveObject.
*  
*  Instead of dynamically creating an object from defaults, the liveObject will
*  simply be assigned to EditNode.liveObject for the created editNode.
* @type {object}
* @default null
*/
liveObject?: object;

/**
* If set to false, indicates that this node cannot be 
*  EditProxy.useCopyPasteShortcuts, including disallowing calls to
*  EditContext.makePaletteNode for EditNode created from this
*  PaletteNode.
* @type {boolean}
* @default null
*/
canDuplicate?: boolean;

/**
* If set to true, indicates instead of updating the changed property on the target
*  live component a new live component is created with the current configured properties.
*  
*  This property is typically set when a custom component is being used that doesn't support
*  setters for any or most of its properties and a change can be reflected by recreating the
*  component.
*  
*  Individual properties of the target component can be marked similarly on the component's
*  schema for more fine-grained control. See DataSourceField.recreateOnChange.
* @type {boolean}
* @default null
*/
recreateOnChange?: boolean;

/**
* Comma separated list of properties for this component that must be provided in
*  EditNode.defaults before the component will be created. Otherwise a
*  Placeholder will be used until the required properties are satisfied.
* @type {string}
* @default null
*/
requiredProperties?: string;

/**
* If set to true, indicates that a Placeholder should always be shown in place of
*  the actual component.
* @type {boolean}
* @default null
*/
alwaysUsePlaceholder?: boolean;

/**
* Image to display in lieu of the usual placeholder text.
* @type {SCImgURL}
* @default null
*/
placeholderImage?: SCImgURL;

/**
* Properties to be applied to the PaletteNode.liveObject when created as a
*  Placeholder.
* @type {Partial<Label>}
* @default null
*/
placeholderProperties?: Partial<Label>;





}


/**
* An object representing the configuration of the placeholder. When a placeholder is
*  created and not replaced by the target component, these are the properties that are
*  applied.
*/
interface PlaceholderDefaults {
/**
* Name of the class that the placeholder replaces.
* @type {string}
* @default null
*/
placeholderFor?: string;

/**
* List of required properties on PlaceholderDefaults.placeholderFor that are not yet provided.
* @type {string}
* @default null
*/
requiredProperties?: string;

/**
* Image to display in lieu of the usual placeholder text.
* @type {SCImgURL}
* @default null
*/
image?: SCImgURL;





}


/**
* X/Y position in pixels, specified as an Array with two members, for example: [30, 50]
*/
interface Point {
/**
* The x coordinate of this point.
* @type {Coordinate}
* @default "0"
*/
x?: Coordinate;

/**
* The y coordinate of this point.
* @type {Coordinate}
* @default "0"
*/
y?: Coordinate;





}


/**
* A set of properties that can be used to configure a Canvas.showPointer.
*/
interface PointerSettings {
/**
* Specifies the position of the pointer when Canvas.showPointer. Accepts
*  similar values as Canvas.snapTo. 
*  
*  Possible basic values: TL, TR, LT, LB, RT, RB, BL, BR, R, L, B, T where B=Bottom, T=Top,
*  L=Left, R=right. The first letter indicates the edge and the second optional letter
*  indicates the placement along the edge.
*  
*  In addition to the basic values above, two different basic values can be combined with
*  a separating slash (/) to position the pointer between the two locations giving more
*  precise control. For example, BL/L positions the pointer along the bottom
*  edge halfway between the bottom-left and the bottom-center.
*  
*  If Canvas.showPointer is enabled and a Canvas.pointerTarget is specified
*  but snapTo is null, a reasonable location is calculated
*  based on the location of the pointerTarget.
* @type {string}
* @default null
*/
snapTo?: string;

/**
* If PointerSettings.snapTo is defined, this property can be
*  used to specify an offset in px or percentage for the left coordinate of the pointer.
*  
*  For example if snapTo is specified as "L" and
*  snapOffsetLeft is set to 6, the pointer will be rendered 6px inside the left
*  edge of the window. Alternatively if snapTo was set
*  to "R", a snapOffsetLeft value of -6 would cause the
*  pointer to be rendered 6px inside the right edge of the window.
* @type {Integer}
* @default null
*/
snapOffsetLeft?: Integer;

/**
* If PointerSettings.snapTo is defined, this property can be
*  used to specify an offset in px or percentage for the top coordinate of the pointer.
*  
*  For example if snapTo is specified as "T" and
*  snapOffsetTop is set to 6, the pointer will be rendered 6px below the top
*  edge of the window. Alternatively if snapTo was set
*  to "B", a snapOffsetTop value of -6 would cause the
*  pointer to be rendered 6px inside the bottom edge of the window.
* @type {Integer}
* @default null
*/
snapOffsetTop?: Integer;

/**
* Specifies the amount of offset in pixels from the corner to position the pointer
*  when PointerSettings.snapTo is TL, TR, BL or BR. This is useful for a canvas with a
*  rounded corner.
* @type {Integer}
* @default null
*/
cornerOffset?: Integer;

/**
* Specifies the position of the pointer into the target when Canvas.showPointer.
*  Accepts the same values as PointerSettings.snapTo.
*  
*  If not specified the center location on the appropriate edge is chosen to match up
*  with the PointerSettings.snapTo value.
* @type {string}
* @default null
*/
targetSnapTo?: string;

/**
* If PointerSettings.targetSnapTo is defined, this property can be
*  used to specify an offset in px or percentage for the coordinate of the pointer
*  into the Canvas.pointerTarget.
*  
*  While PointerSettings.targetOffsetLeft and PointerSettings.targetOffsetTop can be used to fine tune
*  the pointer's target position the user must be aware of the orientation of the
*  pointer. Instead, this property will be applied to the correct offset so that the
*  pointer points inside (or outside if negative) the target.
*  
*  For example if targetSnapTo is specified as "L" and
*  targetOffsetInto is set to 6, the pointer will be rendered 6px inside
*  the left edge of the position target.
* @type {Integer}
* @default null
*/
targetOffsetInto?: Integer;

/**
* If PointerSettings.targetSnapTo is defined, this property can be
*  used to specify an offset in px or percentage for the left coordinate of the pointer.
*  
*  For example if targetSnapTo is specified as "L" and
*  targetOffsetLeft is set to 6, the pointer will be rendered 6px inside the left
*  edge of the pointer target canvas. Alternatively if targetSnapTo was set
*  to "R", a targetOffsetLeft value of -6 would cause the
*  pointer to be rendered 6px inside the right edge of the pointer target canvas.
*  
*  Note that PointerSettings.targetOffsetInto is likely more suitable for simple customizations.
* @type {Integer}
* @default null
*/
targetOffsetLeft?: Integer;

/**
* If PointerSettings.targetSnapTo is defined, this property can be
*  used to specify an offset in px or percentage for the top coordinate of the pointer.
*  
*  For example if targetSnapTo is specified as "T" and
*  targetOffsetTop is set to 6, the pointer will be rendered 6px below the top
*  edge of the pointer target canvas. Alternatively if targetSnapTo was set
*  to "B", a targetOffsetTop value of -6 would cause the
*  pointer to be rendered 6px inside the bottom edge of the pointer target canvas.
*  
*  Note that PointerSettings.targetOffsetInto is likely more suitable for simple customizations.
* @type {Integer}
* @default null
*/
targetOffsetTop?: Integer;

/**
* By default, the pointer will assume the same color as its parent canvas. Set 
*  this attribute to enforce a custom color for the pointer.
* @type {CSSColor}
* @default null
*/
color?: CSSColor;

/**
* By default, the pointer will show the same shadow as its parent canvas. Set 
*  showShadow to false to render the pointer without a shadow.
* @type {boolean}
* @default null
*/
showShadow?: boolean;





}


/**
* Represents the position of a Portlet within a PortalLayout, indicating the
*  column, row, and position within the row.
*/
interface PortalPosition {
/**
* The column number occupied by a Portlet within a PortalLayout.
* @type {Integer}
* @default "0"
*/
colNum?: Integer;

/**
* The row number occupied by a Portlet within a PortalLayout column.
* @type {Integer}
* @default "0"
*/
rowNum?: Integer;

/**
* The position occupied by a Portlet within a PortalLayout row
*  (generally 0, unless there is more than one Portlet in the row).
* @type {Integer}
* @default "0"
*/
position?: Integer;





}


/**
* Settings for generating printable HTML for components.
*/
interface PrintProperties {
/**
* An array of Strings indicating the classNames of controls that should be omitted from
*  printing. By default, omitControls includes all button-based controls, menus
*  and similar interactive controls that are typically useless in printed output.
*  
*  All subclasses of the specified classes are also omitted.
*  
*  See also PrintProperties.includeControls.
* @type {Array<Partial<string>>}
* @default null
*/
omitControls?: Array<Partial<string>>;

/**
* An array of Strings indicating the classNames of controls that should be specifically
*  included when printing, even if a superclass is listed in PrintProperties.omitControls.
* @type {Array<Partial<string>>}
* @default null
*/
includeControls?: Array<Partial<string>>;

/**
* If true, generates HTML for export.
*  
*  Some components, specifically DrawPane and FacetChart on IE8 and earlier, need to generate
*  different HTML for export versus in-browser print preview. When using RPCManager.exportContent
*  the printForExport property is set to true automatically. If not using RPCManager.exportContent(),
*  but the generated HTML will be sent for export, the PrintProperties passed to
*  Canvas.getPrintHTML must have printForExport:true.
* @type {boolean}
* @default null
*/
printForExport?: boolean;





}


/**
* Identifies a property name and value to be assigned to a component by the
*  SetPropertiesTask.
*/
interface PropertyValue {
/**
* Name of the property to be assigned the new value.
* @type {string}
* @default null
*/
name?: string;

/**
* Name of the property to be assigned the new value.
*  
*  A value prefixed with "$" will be treated as a taskInputExpression
*  excluding "$input" and "$inputRecord" references.
* @type {any}
* @default null
*/
value?: any;





}


/**
* An object returned from AutoTest.getLocatorWithIndicators that includes the
*  locator and properties that describe the quality of the locator.
*/
interface QualityIndicatedLocator {
/**
* The AutoTestLocator associated with some DOM element in a SmartClient
*  application page.
* @type {AutoTestLocator}
* @default null
*/
locator?: AutoTestLocator;

/**
* True if the returned QualityIndicatedLocator.locator includes
*  a reference using an auto-generated global ID.
* @type {boolean}
* @default null
*/
containsGlobalId?: boolean;

/**
* The ID of the component within the locator segments that has an auto-generated global
*  ID.
* @type {string}
* @default null
*/
globalID?: string;

/**
* True if the returned QualityIndicatedLocator.locator includes
*  references using index positions.
* @type {boolean}
* @default null
*/
containsIndices?: boolean;

/**
* The ID of the first parent within the locator segments that has a child referenced
*  by index. Note that a child component with an explicit
*  Canvas.locatorName will be excluded since the name is a reliable
*  means to locate the component.
* @type {string}
* @default null
*/
firstParentOfIndex?: string;





}


/**
* Encapsulates a client/server RPC request. You'll need to provide an instance of this class (or a
*  constructor for it) to the RPCManager.sendRequest method. If you use the 
*  RPCManager.send method, an instance of RPCRequest will be created for you.
*/
interface RPCRequest {
/**
* This attribute specifies the payload of the RPCRequest. 
*  
*  When using the iscServer,
*  any JavaScript simple type or arbitrarily nested set of Objects and Arrays can be sent
*  to server and automatically translated to Java Objects. 
*  
*  
*  
*  Here are the 
*  mapping of JavaScript types to their corresponding server object types:
* 
*  
*   JS Type   Java Type
*   Object: {}     Map
*   Array: []     List
*   String       String
*   Number       Long|Double
*   Boolean      Boolean
*   Date        java.util.Date
*   String       com.smartgwt.client.types.ValueEnum
* 
*  
*  
*  Note that the order of keys/values in the Maps created on the server is not guaranteed
*  because JavaScript Object literals do not guarantee order.
*  
*  When using JPA or Hibernate Java value used can be affected by the Java Bean declaration. 
*  See dsRequestBeanTypes for details.
*  
*  Server->client conversion follows this table as well, with some extras. See the toJS()
*  method on JSTranslater in the server documentation for a description of additional
*  behaviors.
*  
*  When not communicating with the SmartClient server, rpcRequest.data
*  becomes simple HTTP parameters or an HTTP request body - see RPCRequest.useSimpleHttp
*  for details.
* @type {string | Record | object}
* @default null
*/
data?: string | Record | object;

/**
* Overrides RPCManager.actionURL for this request only. If you're using queuing, note that queues
*  as per-URL - in other words all RPCRequests in a queue must go to a single URL. If you attempt
*  to send a request with an actionURL that is different from those already in the queue, it
*  will be sent to the server separately, ahead of the queue, and a warning will be logged to
*  the Developer Console.
* @type {URL}
* @default "[ISOMORPHIC]/IDACall"
*/
actionURL?: URL;

/**
* Whether errors during download should be reported inside the document, rather than through
*  the RPCResponse.status. If unset, this will be defaulted from
*  RPCManager.reportDownloadErrorsAsDocuments.
* @type {boolean}
* @default null
*/
reportDownloadErrorsAsDocuments?: boolean;

/**
* Indicates whether this request should use the HttpProxyServlet in order to enable contacting
*  hosts other than the origin server (available only in Pro Edition or better).
*  
*  When various UI components issues requests automatically, or when a call to
*  RPCManager.sendProxied is made, the HttpProxy will automatically be used for a URL
*  that starts with "http" and uses a hostname other than "localhost" or
*  window.location.hostname, or if the port number differs.
*  
*  rpcRequest.useHttpProxy should only be used to force requests to go through the
*  HttpProxy when the above rules don't work, or to avoid using the HttpProxy when contacting
*  hosts that allow cross-site calls via the
*  Http Access Control
*  standard.
*  
*  You can also set RPCManager.useHttpProxy:false to avoid ever using the
*  HttpProxyServlet.
* @type {boolean}
* @default null
*/
useHttpProxy?: boolean;

/**
* The proxy URL to use for this request (if RPCRequest.useHttpProxy is set for this
*  request). If unset, the value of RPCManager.httpProxyURL will be used instead.
* @type {string}
* @default null
*/
httpProxyURL?: string;

/**
* In browsers that support Cross-Origin Resource Sharing
*  and XMLHttpRequest 2,
*  and where the service at the RPCRequest.actionURL allows the origin to send credentials
*  (see Access-Control-Allow-Credentials),
*  should user credentials such as cookies,
*  HTTP authentication, and client-side SSL certificates be sent with the actual CORS request?
*  
*  This setting only applies when the request RPCRequest.transport is
*  "xmlHttpRequest".
*  
*  
*  Note that Internet Explorer 10 and 11 do not send cookies as part of user credentials:
*  IE10 doesn't support cookies on cross origin XMLHttpRequest withCredentials=true.
* @type {boolean}
* @default null
*/
withCredentials?: boolean;

/**
* Controls the prompt style for this request only. Defaults to
*  RPCManager.promptStyle.
* @type {PromptStyle}
* @default "cursor"
*/
promptStyle?: PromptStyle;

/**
* If true, an image is shown to the right of the cursor when RPCRequest.promptStyle is
*  set to "cursor", otherwise the cursor itself is modified via css to the value of
*  RPCRequest.promptCursor.
*  
*  If left unspecified, the default value is set by RPCManager.useCursorTracker.
* @type {boolean}
* @default false
*/
useCursorTracker?: boolean;

/**
* Controls the cursor shown when RPCManager.promptStyle is set to
*  "cursor" for this request only. Defaults to RPCManager.promptCursor.
* @type {string}
* @default "progress"
*/
promptCursor?: string;

/**
* Overrides RPCManager.defaultPrompt for this request only. If you're using queuing, note that the
*  prompt string from the first request in the queue is the one that is shown to the user.
* @type {HTMLString}
* @default "Contacting server..."
*/
prompt?: HTMLString;

/**
* Overrides RPCManager.showPrompt for this request only.
*  
*  If you're using queuing, note that if any of the requests in the queue specify
*  showPrompt:true, then a prompt will be shown for the entire queue with the prompt text of
*  the first request in the queue to specify a custom prompt if promptStyle is set to "dialog".
*  
*  If promptStyle is set to "cursor" for the request that specified showPrompt: true, then the
*  entire queue uses the "cursor" style for the prompt.
* @type {boolean}
* @default null
*/
showPrompt?: boolean;

/**
* Overrides RPCManager.promptDelay for this request only. Defaults to RPCManager.promptDelay.
*  
*  If you're using queuing, note that the promptDelay of the first request is used for the entire queue.
* @type {number}
* @default 300
*/
promptDelay?: number;

/**
* If you expect to receive a response to your RPC request, you can specify a callback that
*  will be called with an instance or RPCResponse class as sent by the server. Queuing does
*  not affect callbacks in any way - your specified callback will be invoked for each
*  RPCRequest that contained a callback regardless of whether the request was sent as part of a
*  queue or not.
*  
*  Note that if the request encounters an error (such as 500 server error), by default the
*  callback will not be fired, instead, RPCManager.handleError is called to
*  invoke the default system-wide error handling. Set RPCRequest.willHandleError:true to have your
*  callback invoked regardless of whether there are errors, however, make sure your callback
*  properly handles malformed responses when RPCResponse.status is non-zero. See the
*  errorHandling for more details.
*  
*  This callback will not be invoked for RPCRequest.downloadResult download requests.
* @type {RPCCallback}
* @default null
*/
callback?: RPCCallback;

/**
* An object to be held onto for the duration of the RPC turnaround to track
*  application-specific context.
*  
*  When an RPC turnaround completes, the clientContext is available in the
*  Callbacks.RPCCallback as rpcResponse.clientContext. The
*  clientContext is never sent to the server.
*  
*  The clientContext is useful for holding onto state that will be used when the
*  Callbacks.RPCCallback fires, such as the name of a component that will receive the
*  returned data.
* @type {object}
* @default null
*/
clientContext?: object;

/**
* With willHandleError:false, rpcResponses that indicate an error go through centralized
*  handling in the RPCManager and rpcRequest.callback is never invoked.
*  
*  Setting willHandleError:true means that your rpcRequest.callback will receive rpcResponses
*  that have an error status and must handle them.
*  
*  See also the error handling section in the RPCManager docs.
* @type {boolean}
* @default false
*/
willHandleError?: boolean;

/**
* Sets the timeout on this request. Default is to use RPCManager.defaultTimeout. 
*  
*  If you're using RPCManager.startQueue, note that the timeout setting derived
*  from the last request in the queue is used for the entire queue. If you want to override
*  the timeout for the queue, make sure to set your override at least on the last request in
*  the queue.
*  
*  For the "xmlHttpRequest" RPCRequest.transport, this timeout can only happen
*  if the server actually fails to respond within the specified number of milliseconds. For
*  the "hiddenFrame" transport, this timeout will occur for non-200 (HTTP_OK) responses.
*  
*  If timeout is set to zero, the RPCManager will not enforce a timeout for this
*  request. However, note that all browsers enforce their own timeouts on HTTP requests, and
*  may have different timeouts for different kinds of failures (no response at all from server,
*  hung response after receiving headers, hung response after receiving partial data, etc).
*  Also, intervening web proxies or firewalls may impose timeouts of their own.
*  
*  As a rough rule of thumb, if your server response will have a lengthy pause before data
*  begins to be sent, 1-2 minutes is the maximum allowable pause for a public site and still may
*  not work for a minority of users, but up to 4 minutes may be allowable in a controlled
*  environment (intranet or extranet with well-known user base).
*  
*  Above these limits, your code should return some kind of immediate response to the browser,
*  then kick off a server-side process to complete processing. The browser can then either
*  poll for completion, or use a server-push notification system such as SmartClient Real-Time
*  Messaging (see http://smartclient.com/product).
* @type {Integer}
* @default null
*/
timeout?: Integer;

/**
* Values to be sent as simple HTTP params, as a JavaScript Object where each property/value
*  pair will become an HTTP parameter name and value. These parameters are then accessible on
*  the server, for example, using servletRequest.getParameter(paramName) in Java Servlets. 
*  
*  Array-valued parameters will be submitted as multiple instances of the same parameter,
*  similar to an HTML form with a multi-select (?paramName=value1&amp;paramName=value2 ...),
*  accessible as getParameterValues(paramName) in Java Servlets. Any non-atomic type, such as
*  an Object, will be serialized to JSON by the
*  JSONEncoder. If this isn't desirable, serialize the data in advance so that the
*  value provided in rpcRequest.params is a String.
*  
*  Note that this API is primarily used in combination with RPCRequest.useSimpleHttp -
*  when contacting the SmartClient Server, use RPCRequest.data instead, which provides
*  full JavaScript &lt;-&gt; Java translation of arbitrary structures.
*  rpcRequest.params can also be used with the SmartClient Server, where it
*  provides an an opportunity to send additional data aside from the main
*  RPCRequest.data payload. This is useful for adding data to DataSource requests which
*  will be kept separate from the automatically sent DataSource data, or for making parts of
*  the request visible in the URL for HTTP-level logging or layer 4 switches.
*  
*  Note that in contrast to RPCRequest.data object, the data in
*  rpcRequest.params is not deserialized by the SmartClient server, and
*  all values arrive on the server as String type (like HTTP parameters always do).
*  
*  
*  The params value can also be specified as a componentID or component instance that provides
*  a method getValues() that returns an Object containing parameter names and values.
*  SmartClient components DynamicForm, ValuesManager are two such
*  classes. Lastly, you may specify the ID of a native form element (retrievable via
*  getElementById()) and the params will be populated from there. If there is an error
*  resolving your params directive, it will be logged to the Developer Console.
*  
*  
*  Note: The params are submitted once per http transaction. If you are using 
*  RPCManager.startQueue to bundle multiple RPCRequests or DSRequests
*  into a single HTTP turnaround, the params from the various RPCRequests will be merged,
*  with the later-queued transactions winning on parameter name collisions. A warning will be
*  logged in the Developer Console if multiple RPCRequests specified params.
* @type {object}
* @default null
*/
params?: object;

/**
* This works similarly to RPCRequest.serverOutputAsString except the resulting String
*  is automatically evaluated as JavaScript. The result of the evaluation is then passed to
*  any specified RPCRequest.callback as RPCResponse.data.
*  
*  This feature can be used to dynamically load new application modules into a running
*  application. An RPCRequest with evalResult enabled can be used to fetch a
*  static .js file or JavaScript dynamically generated by the server. The returned JavaScript
*  can contain anything that a JavaScript file loaded at init time can contain, including new
*  views and new SmartClient class definitions.
*  
*  Example usage with RPCManager.sendRequest:
*  
*  
*  isc.RPCManager.sendRequest({
*    evalResult:true,
*    actionURL:"js/loadLabel.js",
*    evalVars:{var1:"A Value"}
*  });
*  
*  
*  
*  This call would execute the code from loadLabel.js, and make the variable
*  var1 available to that code. Therefore if the .js file contained this code:
*  
*  
*  isc.Label.create({
*    contents:var1
*  })
*  
*  
*  
*  A label would be created with contents set to the value of var1 - the string
*  "A Value".
*  
*  
*  This feature relies on the XMLHttpRequest object which can be disabled by end-users in some
*  supported browsers. See platformDependencies for more information.
* @type {boolean}
* @default false
*/
evalResult?: boolean;

/**
* If you've set RPCRequest.evalResult : true, then the property values of this object
*  will be available in the evaluation scope of the result under the variable names
*  specified by the property names. 
*  
*  So e.g. if evalVars is: {foo: "bar"} then a reference to the
*  variable foo in the result will evaluate to "bar".
* @type {object}
* @default null
*/
evalVars?: object;

/**
* For use only with the RPCTransport transport, this attribute
*  specifies the name of the URL parameter which is used to specify the callback function that
*  the server is expected to call by writing out JavaScript code. The actual function to call
*  is automatically generated and differs for every request (to allow concurrency).
*  
*  For example, with callbackParam set to it's default value of "callback", the
*  server might be contacted with a URL like:
*  
*   loadData?callback=isc_scriptIncludeCallback_5
*  
*  .. then the server's response should look like:
*  
*   isc_scriptIncludeCallback_5({ .. data .. });
*  
*  The name "isc_scriptIncludeCallback_5" is automatically generated and will differ each time
*  the server is contacted.
*  
*  SmartClient makes of this server-provided callback mechanism, then calls
*  RPCRequest.callback normally.
*  
*  rpcRequest.callbackParam is ignored by all transport other than
*  scriptInclude.
* @type {string}
* @default "callback"
*/
callbackParam?: string;

/**
* If RPCRequest.evalResult is set, setting this property to true causes
*  Canvas.autoDraw to be set to false for the duration of the result evaluation -
*  which is generally what you want if you're returning new components from the server.
*  
*  This also effects components loaded via the RPCManager.loadScreen API.
* @type {boolean}
* @default true
*/
suppressAutoDraw?: boolean;

/**
* Setting this flag makes the body of the HTTP response available as a String in the
*  RPCRequest.callback as RPCResponse.data. This is typically only useful if you
*  are sending a request that will not be received by the SmartClient Java Server,
*  however in that case, set RPCRequest.useSimpleHttp:true instead, which implies
*  serverOutputAsString:true.
*  
*  serverOutputAsString:true allows you to, for example, load the contents of
*  static files off your webserver into a string for processing on the client with no server
*  support. The RPCRequest.actionURL must be in the same domain as the current page for
*  this to work.
*  
*  This feature relies on the XMLHttpRequest object which can be disabled by end-users in some
*  supported browsers. See platformDependencies for more information.
*  
*  Generally this API is used for either nonJavaBackend 
*  or for advanced usage such as content that requires processing before it can be used in
*  SmartClient components (such as client-side web scraping). Note that SmartClient provides
*  higher-level APIs for loading common types of data, see eg HTMLFlow for HTML content,
*  ViewLoader for loading SmartClient components, XMLTools.loadXML for loading
*  XML, RPCRequest.evalResult for loading JSON, and
*  DataSource for loading structured data in various formats.
* @type {boolean}
* @default false
*/
serverOutputAsString?: boolean;

/**
* Advanced flag to avoid a potential memory leak in Internet Explorer 9 for requests
*  with JSON formatted responses.
*  
*  This attribute may be set to false to explicitly enable the 
*  workaround described RPCManager.allowIE9Leak for this request, 
*  avoiding a potential memory leak in Internet Explorer 9.
*  
*  This workaround has a limitation in that if parsing the JSON response generates
*  certain object types including JavaScript Date or function
*  objects, attempts to interact with these objects can subsequently lead to a 
*  JavaScript error with the message "Can't execute code from a freed script".
*  
*  This workaround therefore may not be suitable for all transactions or dataSources
*  within a given application.
*  
*  This property may also be set globally within an application (via 
*  RPCManager.allowIE9Leak)_.
*  
*  Note: This memory leak and workaround is discussed further in the online 
*  SmartClient FAQ.
* @type {boolean}
* @default null
*/
allowIE9Leak?: boolean;

/**
* Selects the transport used for this RPCRequest. If unset, the value of
*  RPCManager.defaultTransport will be used.
*  
*  If you're using queueing, note that all requests in the queue must use the same transport.
*  If you attempt to send a request via a different transport than those that are currently on
*  the queue, it will be sent to the server separately, ahead of the queue, and a warning will
*  be logged to the Developer Console.
*  
*  If you specify an unknown transport, an error will be logged to the DeveloperConsole and
*  RPCManager.defaultTransport will be used instead.
*  
*  If you specify the xmlHttpRequest transport and it is not available, a warning will be
*  logged to the Developer Console and the RPCManager will attempt to use the
*  hiddenFrame transport instead for this request. Note that some features like
*  RPCRequest.serverOutputAsString require the xmlHttpRequest transport and will not
*  work if the xmlHttpRequest transport is unavailable (this can happen if the end user is
*  using Internet Explorer and has disabled ActiveX). You can check whether or not the
*  xmlHttpRequest transport is currently available by calling
*  RPCManager.xmlHttpRequestAvailable.
* @type {RPCTransport}
* @default "xmlHttpRequest"
*/
transport?: RPCTransport;

/**
* Selects the default http transport for this RPCRequest. If set to true, this request will use
*  XMLHttpRequest for the transport to the server. If set to false it will use a hidden frame. If
*  left unset, the transport mechanism is determined from the RPCManager default set in 
*  RPCManager.useXmlHttpRequest
*  
*  If you're using queueing, note that all requests in the queue must use the same transport.
*  If you attempt to send a request via a different transport than those that are currently on
*  the queue, it will be sent to the server separately, ahead of the queue, and a warning will
*  be logged to the Developer Console.
*  
*  If you specify true for this attribute and XMLHttp is not available, a warning
*  will be logged to the Developer Console and RPCManager will attempt to use the frames
*  transport for this request. Note that some features like
*  RPCRequest.serverOutputAsString require the XMLHttp transport and will not work if the
*  XMLHttp transport is unavailable (this can happen if the end user is using Internet Explorer
*  and has disabled ActiveX). You can query the availability of XMLHttp by calling
*  RPCManager.xmlHttpRequestAvailable
* @deprecated * As of SmartClient 5.5, use RPCRequest.transport. If you specify a value for this property, it will take precedence over RPCRequest.transport.

* @type {boolean}
* @default undefined
*/
useXmlHttpRequest?: boolean;

/**
* Selects the HTTP method that will be used for the request. Typical values are "POST" and
*  "GET".
*  
*  The more obscure "PUT", "DELETE" and "HEAD" methods are also valid, however, none of these
*  are supported by the Safari browser previous to version 3.0.
* @type {string}
* @default "POST"
*/
httpMethod?: string;

/**
* Valid with the xmlHttpRequest transport only and only when
*  RPCRequest.httpMethod is set to "POST".
* @type {string}
* @default "application/x-www-form-urlencoded"
*/
contentType?: string;

/**
* HTTP headers to send, as a Object mapping Header name -> Header value, eg
*  { "Content-Type" : "text/xml" }
*  
*  Valid with the xmlHttpRequest RPCRequest.transport only.
* @type {object}
* @default null
*/
httpHeaders?: object;

/**
* For use during relogin, this property marks this request an attempt to
*  login, therefore a response containing the loginRequiredMarker is a normal
*  condition and should result in the status code RPCResponse.STATUS_LOGIN_INCORRECT
*  rather than a call to RPCManager.loginRequired.
*  
*  It is not required to set containsCredentials, however, it does typically
*  simplify relogin logic by separating the handling of RPCs that are login attempts from RPCs
*  that are not.
* @type {boolean}
* @default false
*/
containsCredentials?: boolean;

/**
* When set to true, no reply is expected from the server. However, if a reply is received, it will
*  be processed.
* 
*  Note: setting this to true, forces RPCRequest.sendNoQueue to true for
*  this request.
* @type {boolean}
* @default false
*/
ignoreTimeout?: boolean;

/**
* When set to true, this request is sent to the server immediately, bypassing any current queue.
* @type {boolean}
* @default false
*/
sendNoQueue?: boolean;

/**
* When set to true, assume the request is not going to the SmartClient server, and hence send
*  a simple HTTP request. Values specified in RPCRequest.params are sent to to the
*  server as HTTP request parameters. If RPCRequest.httpMethod method is POST and
*  RPCRequest.data is supplied, it is assumed to be a string to post as the HTTP
*  requestBody.
*  
*  Setting this to true automatically defaults RPCRequest.serverOutputAsString to true
*  as well.
* @deprecated * As of SmartClient 5.6, use RPCRequest.useSimpleHttp instead.

* @type {boolean}
* @default false
*/
paramsOnly?: boolean;

/**
* When set to true, assume the request is not going to the SmartClient server, and hence send
*  a simple HTTP request that does not use SmartClient-specific request encoding.
*  
*  Values specified in RPCRequest.params are sent to to the server as HTTP request
*  parameters. If RPCRequest.httpMethod is "GET", parameters appear in the request URL, otherwise
*  if httpMethod is "POST", parameters are encoded in the request body (exactly like an HTML form
*  does). These parameters are then accessible via typical server-side APIs for retrieving
*  HTTP parameters, eg, servletRequest.getParameter(paramName) in Java Servlets. 
*  
*  Note that if RPCRequest.httpMethod method is POST and RPCRequest.data is supplied,
*  RPCRequest.data is assumed to be a string to post as the HTTP request body, and
*  RPCRequest.params are sent as URL parameters instead. This usage is for sending
*  custom request bodies such as the XML payloads used for SOAP. In this case,
*  RPCRequest.contentType is typically also set to indicate the content type of the
*  request body.
*  
*  Setting useSimpleHttp to true also automatically sets
*  RPCRequest.serverOutputAsString to true as well.
* @type {boolean}
* @default false
*/
useSimpleHttp?: boolean;

/**
* For xmlHttp transport + httpMethod: "GET" only, set to true to force a conditional
*  GET request even if the browser thinks it has a current cached response.
* @type {boolean}
* @default false
*/
bypassCache?: boolean;

/**
* If enabled, the server omits any key/value pairs in map that have null values from the
*  response. This can reduce the size of the response when many fields have null values.
*  
*  To enable this globally for all responses you can set RPCManager.omitNullMapValuesInResponse
*  in server_properties.
*  
*  Note that sqlDataSource don't add nulls to results for null
*  values so this flag does nothing in that case.
* @type {boolean}
* @default false
*/
omitNullMapValuesInResponse?: boolean;

/**
* If set true, tells the server to use strict JSON format when serializing the response data. 
*  If set false, tells the server to use a more permissive encoding that is still valid JS, but
*  is not technically valid JSON. The default value of null tells the server to use the 
*  default global setting (see below).
*  
*  To enable this globally for all responses you can set RPCManager.useStrictJSON
*  in server_properties. If the global flag is not set either 
*  way in server.properties, it defaults to false.
* @type {boolean}
* @default null
*/
useStrictJSON?: boolean;

/**
* If enabled, causes the RPCRequest to download the requested resource as a file, either 
*  showing the browser's Save dialog or displaying the file-content in 
*  RPCRequest.downloadToNewWindow.
*  
*  Download requests do not fire a callback.
*  By default the request will use RPCRequest.transport: "hiddenFrame".
*  Developers may override this by explicitly setting request.transport to "xmlHttpRequest".
*  This mode allows developers to send RPCRequest.httpHeaders to the server, but has some drawbacks:
*  
* - xmlHttpRequest download does not have a built-in progress bar to indicate download progress
*    
* - The browser must hold the entire XHR response in memory, whereas with normal download
*      data is streamed to disk or to another application
*    
* - This mode does not currently support RPCRequest.downloadToNewWindow
* @type {boolean}
* @default false
*/
downloadResult?: boolean;

/**
* When RPCRequest.downloadResult is true, setting this attribute to
*  true causes the content of the downloaded file to be displayed in a new browser window. 
*  
*  Note that this setting is currently incompatible with
*  RPCRequest.transport. See the 
*  RPCRequest.downloadResult documentation for more details on 
*  xmlHttpRequest downloads
* @type {boolean}
* @default false
*/
downloadToNewWindow?: boolean;

/**
* If enabled and request is applied to RPCManager.cacheScreens or
*  RPCManager.loadScreen indicates that
*  referenced DataSources should be loaded in mock mode.
* @type {boolean}
* @default null
*/
mockMode?: boolean;





}


/**
* Definition of a radial gradient.
*/
interface RadialGradient extends Gradient {
/**
* x coordinate of outer radial
* @type {string}
* @default null
*/
cx?: string;

/**
* y coordinate of outer radial
* @type {string}
* @default null
*/
cy?: string;

/**
* radius
* @type {string}
* @default null
*/
r?: string;

/**
* x coordinate of inner radial
* @type {string}
* @default null
*/
fx?: string;

/**
* y coordinate of inner radial
* @type {string}
* @default "0"
*/
fy?: string;





}


/**
* A Record is an ordinary JavaScript Object with properties that are treated as data to
*  be displayed and edited by a DataBoundComponent. 
*  
*  DataBoundComponents have a concept of DataBoundComponent.fields,
*  where values for each field are found under the same-named property in a Record.
*  
*  A Record is always an ordinary JavaScript Object regardless of how the record is 
*  loaded (static data, java server, XML web service, etc), and so supports the
*  normal behaviors of JavaScript Objects, including accessing and assigning to properties
*  via dot notation:
*  
*    var fieldValue = record.fieldName;
*    record.fieldName = newValue;
*  
*  
*  The concept of working with Records is common to all DataBoundComponents, although
*  individual DataBoundComponents may work with singular records (DynamicForm) or
*  may work with lists (ListGrid), trees (TreeGrid), or cubes
*  (CubeGrid) of records.
*  
*  Individual DataComponents may also look for special properties on Records which
*  control styling or behavior for those records, such as
*  ListGrid.recordEditProperty.
*/
interface Record {




}


/**
* A simple object representing the path for navigation from source to target DataSources
*  via foreign key declarations defined between each.
*/
interface RelationPath {
/**
* A slash-separated path consisting of each DataSource ID in the path between source and
*  target
* @type {string}
* @default null
*/
path?: string;

/**
* A number representing the distance between source and target DataSources. Equal to the
*  number of separators in the path.
* @type {Integer}
* @default null
*/
distance?: Integer;





}


/**
* An object representing a relative date, useful for representing date ranges etc in criteria.
*  RelativeDate objects may be created directly by SmartClient components such as the
*  RelativeDateItem.
*  
*  RelativeDate objects will have "_constructor" set to "RelativeDate"
*  and must have a specified RelativeDate.value. Any other attributes are optional.
*/
interface RelativeDate {
/**
* The value of this relative date, specified as a RelativeDateString 
*  or RelativeDateShortcut.
* @type {RelativeDateString | RelativeDateShortcut}
* @default null
*/
value?: RelativeDateString | RelativeDateShortcut;

/**
* If this relative date has its value specified as a RelativeDateShortcut which doesn't
*  specify an exact time-period boundary - for example "$yesterday", this attribute
*  may be set to specify whether the date should be interpreted as the start or end boundary of
*  the time period.
* @type {RelativeDateRangePosition}
* @default null
*/
rangePosition?: RelativeDateRangePosition;





}


/**
* A configuration object containing image URLs for a set of possible
*  images to display based on the StatefulCanvas.state of some components.
*  See the statefulImages for more information.
*  
*  Each attribute in this configuration object maps a state to a target URL.
*  Each URL may be specified in one of three ways
*  
* - a standard SCImgURL may be used to refer directly to an image file.
*    
* - the "#state:" prefix may be used to display media from another
*      specified state.
*    
* - the "#modifier:" prefix may be used to specify a modifier 
*      string to apply to the SCStatefulImgConfig._base.
*      The modifier will be applied to the base file name before the file type suffix.
*  
*  For example, consider a stateful image config with the following properties:
*  
*  {  _base:"button.png",
*    Over:"bright_button.png",
*    Focused:"#state:Over",
*    Selected:"#state:Over",
*    Disabled:"#modifier:_Disabled",
*    SelectedDisabled:"#state:Selected"
*  }
*  
*  In this case
*  
*  
* - the base image URL and the the "Over" state image URL would be determined using
*    the standard SCImgURL rules
*  
* - the "Focused" and "Selected" state images would re-use the "Over" state image
*    ("bright_button.png")
*  
* - the "Disabled" state image would be the base state image with a
*    "_Disabled" suffix applied to the file name
*    ("button_Disabled.png")
*  
* - the "SelectedDisabled" entry would be used for the combined 
*    "Selected" and "Disabled" states, and would
*    re-use the "Selected" state image (which in turn maps back to
*    the "Over" state, resolving to "bright_button.png")
*  
*  
*  The default set of standard states are explicitly documented, but this object format
*  is extensible.
*  A developer may specify additional attributes on a SCStatefulImgConfig beyond the
*  standard documented states and they may be picked up if a custom state is applied to 
*  a component (via a call to StatefulCanvas.setState, for example).
*  
*  In some cases, an icon may have only custom states - for example, a tree-folder icon 
*  is always either opened or closed. In these cases, a _base entry is only 
*  required if entries in the object use the #state or #modifier components. 
*  
*  Combined states and missing entries:
*  The Canvas.isFocused and StatefulCanvas.selected 
*  states may be applied to a component in combination with other states. For example an ImgButton
*  marked both Selected and Disabled will look for media to
*  represent this combined state. To provide such media in a SCStatefulImgConfig, 
*  use the combined state names (in this case SelectedDisabled).
*  If a component is both Selected and Focused,
*  three-part combined states are also possible (Selected + Focused + Over gives
*  SelectedFocusedOver for example).
*  
*  The SCStatefulImgConfig format may be sparse - developers may skip providing values for
*  certain states (or combined states) in the SCStatefulImgConfig object. 
*  In this case the system will back off to using one of the state image entries
*  that has been explicitly provided, according to the following rules:
*  
*  State(s)
*    Stateful image attributes to consider (in order of preference)
*  
*  Focused and Selected
*    If both focused and selected states are applied, the system will use the first
*      (populated) value from the following attribute list:
*      
* - "FocusedSelected"
*        
* - "Focused"
*        
* - "Selected"
*    
*  
*  Over or Down in combination with Focused 
*      / Selected 
*    System will check for a combined state attribute with the Focused / Selected state first.
*      For example for Focused + Selected + Over, consider the following attributes:
*      
* - "FocusedSelectedOver"
*        
* - "FocusedOver"
*        
* - "SelectedOver"
*      If no combined state entry is specified, back off to considering just the
*      Focused / Selected state:
*      
* - "FocusedSelected"
*        
* - "Focused"
*        
* - "Selected"
*      
*      If no focused / selected state entry is present in the config object, 
*      look for an entry for the unmodified state name
*      
* - "Over"
*    
*  
*  All other states, including Disabled (in combination with 
*      Focused / Selected) 
*    Check for a combined state attribute with the Focused / Selected state first.
*      For example for Focused + Selected + "CustomState", consider the following attributes:
*      
* - "FocusedSelectedCustomState"
*        
* - "FocusedCustomState"
*        
* - "SelectedCustomState"
*      If no combined state entry is specified, back off to considering just the
*      unmodified state name
*      
* - "CustomState"  
*      If there is no explicit entry for the state name, use the Focused / Selected
*      state without a state name:
*      
* - "FocusedSelected"
*        
* - "Focused"
*        
* - "Selected"
*      
*    
*  
*  
*  If no entry can be found for the specified state / combined states using the above 
*  approach, the "_base" attribute will be used.
*/
interface SCStatefulImgConfig {
/**
* The base filename for the image. This will be used if no state is applied to the
*  stateful component displaying this image, or if no explicit entry exists for
*  a state that is applied.
*  It will also be used as a base file name for entries specified using the
*  "#modifier:some_value" format.
*  
*  In some cases, an icon may have only custom states - for example, a tree-folder icon 
*  is always either opened or closed, so a _base entry is not 
*  required unless entries in the object use the #state or #modifier 
*  components - in this case, a warning will be logged if no _base is set. 
*  
*  See SCStatefulImgConfig for further information.
* @type {SCImgURL}
* @default null
*/
_base?: SCImgURL;

/**
* Image to display when the component is StatefulCanvas.selected.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
Selected?: string;

/**
* Image to display when the component is Canvas.isFocused.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
Focused?: string;

/**
* Image to display on StatefulCanvas.showRollOver.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
Over?: string;

/**
* Image to display on StatefulCanvas.showDown.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
Down?: string;

/**
* Image to display when the component is Canvas.disabled.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
Disabled?: string;

/**
* Image to display when the component is StatefulCanvas.selected on
*  StatefulCanvas.showRollOver.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
SelectedOver?: string;

/**
* Image to display when the component is StatefulCanvas.selected on
*  StatefulCanvas.showDown.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
SelectedDown?: string;

/**
* Image to display when the component is StatefulCanvas.selected and
*  Canvas.disabled.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
SelectedDisabled?: string;

/**
* Image to display when the component is Canvas.isFocused on
*  StatefulCanvas.showRollOver.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
FocusedOver?: string;

/**
* Image to display when the component is Canvas.isFocused on
*  StatefulCanvas.showDown.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
FocusedDown?: string;

/**
* Image to display when the component is StatefulCanvas.selected and
*  Canvas.isFocused.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
SelectedFocused?: string;

/**
* Image to display when the component is StatefulCanvas.selected and
*  Canvas.isFocused on StatefulCanvas.showRollOver.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
SelectedFocusedOver?: string;

/**
* Image to display when the component is StatefulCanvas.selected and
*  Canvas.isFocused on StatefulCanvas.showDown.
*  
*  May be specified as 
*  
* - A SCImgURL indicating the media to load
*    
* - A reference to another entry in this SCStatefulImgConfig via the format 
*      "#state:otherStateName"
*    
* - A modifier to apply to the SCStatefulImgConfig._base media via the
*      format "#modifier:modifierString"
*  
*  See SCStatefulImgConfig for further information.
* @type {string}
* @default null
*/
SelectedFocusedDown?: string;





}


/**
* Section descriptor used by a SectionStack to describe a section of items which are shown
*  or hidden together along with their associated header. 
*  
*  A section header (see SectionStack.sectionHeaderClass) is created from this
*  descriptor when the SectionStack is created. Any changes after creation must be made to
*  the section header: SectionStack.getSectionHeader.
*  
*  Additional SectionHeader properties set on the SectionStackSection not explicitly
*  documented, such as "iconAlign" or "prompt", are supported.
*/
interface SectionStackSection {
/**
* Identifier for the section. This can be used later in calls to SectionStack APIs such as
*  SectionStack.expandSection and SectionStack.collapseSection. Note that if no name
*  is specified for the section, one will be auto-generated when the section is created.
*  This property should be a string which may be used as a valid JavaScript identifier
*  (should start with a letter and not contain space or special characters such as "*").
* @type {string}
* @default null
*/
name?: string;

/**
* Optional ID for the section. If SectionStack.useGlobalSectionIDs is true, this property will
*  be applied to the generated SectionStackHeader widget as a standard widget ID, meaning
*  it should be unique within a page.
*  
*  Backcompat Note: Section stack sections may be uniquely identified within a stack
*  via the SectionStackSection.name attribute (introduced in Jan 2010). Prior to this,
*  the section ID attribute was used in this way (and would not be applied to the section header 
*  as a widget ID). For backwards compatibility this is still supported: If 
*  section.name is unspecified for a section but section.ID is set,
*  the ID will be used as a default name attribute for the section. For backwards compatibility
*  we also disable the standard behavior of having the section.ID being applied to the generated
*  section header (thereby avoiding the page-level uniqueness requirement) by defaulting 
*  SectionStack.useGlobalSectionIDs to false.
* @type {string}
* @default null
*/
ID?: string;

/**
* Custom controls to be shown on top of this section header.
*  
*  These controls are shown in the SectionHeader.controlsLayout.
*  
*  Note that this is an init-time property. If you need to dynamically change what 
*  controls are displayed to the user, we would recommend embedding the
*  controls in a Layout or similar container. 
*  This will allow you to show/hide or add/remove members at runtime
*  by manipulating the existing control(s) set up at init time.
*  
*  For SectionStackSection.canClose sections, a
*  SectionStack.closeSectionButton will be added to the section
*  controls automatically.
* @type {Array<Partial<Canvas>>}
* @default null
*/
controls?: Array<Partial<Canvas>>;

/**
* Title to show for the section
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* If the title for this section header is too large for the available space, should the title be
*  clipped?
*  
*  This feature is supported only in browsers that support the CSS UI text-overflow
*  property (IE6+, Firefox 7+, Safari, Chrome, Opera 9+).
* @type {boolean}
* @default true
*/
clipTitle?: boolean;

/**
* If true and the title is clipped, then a hover containing the full title of this section header
*  is enabled.
* @type {boolean}
* @default true
*/
showClippedTitleOnHover?: boolean;

/**
* List of Canvases that constitute the section. These Canvases will be shown and hidden
*  together.
*  
*  Along with live Canvases, you may also pass special string autochild shortcuts of the
*  form "autoChild:autoChildName" as discussed at the bottom of help topic
*  autoChildren.
* @type {Array<Partial<Canvas>> | Array<Partial<AutoChildShortcut>>}
* @default null
*/
items?: Array<Partial<Canvas>> | Array<Partial<AutoChildShortcut>>;

/**
* If true, a header will be shown for this section. If false, no header will be shown.
* @type {boolean}
* @default true
*/
showHeader?: boolean;

/**
* If true, the header for this Section will be included in the page's tab
*  order for accessibility. May also be set at the SectionStack level via 
*  SectionStack.canTabToHeaders.
*  
*  See accessibility.
* @type {boolean}
* @default null
*/
canTabToHeader?: boolean;

/**
* Base filename of the icon that represents open and closed states. The default settings
*  also change the icon for disabled sections, so a total of four images are required
*  (opened, closed, Disabled_opened, Disabled_closed).
*  
*  Not shown if SectionStackSection.canCollapse is false.
* @type {SCImgURL}
* @default "[SKIN]SectionHeader/opener.gif"
*/
icon?: SCImgURL;

/**
* If set to false, then the items in this section will not be resized by sectionHeader
*  repositioning. You may also set this flag directly on any of the items in any section to
*  cause that item to not be resizeable.
* @type {boolean}
* @default null
*/
resizeable?: boolean;

/**
* If set to false, then this sectionHeader will not be able to be dragged to perform a drag
*  reorder, if SectionStack.canReorderSections is true.
*  You can also disable dropping other sections before this one by setting 
*  Canvas.canDropBefore to false.
* @type {boolean}
* @default null
*/
canReorder?: boolean;

/**
* When explicitly set to false, disallows drop before this member in the Layout.
* @type {boolean}
* @default null
*/
canDropBefore?: boolean;

/**
* Sections default to the collapsed state unless SectionStackSection.showHeader is
*  set to false in which case they default to the expanded state. This
*  attribute allows you to explicitly control the expand/collapse state of the
*  section by overriding the above default behavior.
* @type {boolean}
* @default false
*/
expanded?: boolean;

/**
* Sections default to the visible state. This
*  attribute allows you to explicitly control the visible/hidden state of the
*  section by overriding the above default behavior.
* @type {boolean}
* @default false
*/
hidden?: boolean;

/**
* This attribute controls whether or not the expand/collapse UI control is shown on the
*  header of this section. Any section can still be expanded/collapsed programmatically,
*  regardless of this setting.
* @type {boolean}
* @default true
*/
canCollapse?: boolean;

/**
* Should the SectionStackSection.items be Canvas.destroy if this section is
*  SectionStack.removeSection? The section header itself and any controls
*  will always be destroyed.
* @type {boolean}
* @default false
*/
destroyOnRemove?: boolean;

/**
* Allows properties for the header (a SectionHeader or ImgSectionHeader subclass) to
*  be set on the section before it's added to the SectionStack.
* @type {Partial<SectionHeader> | Partial<ImgSectionHeader>}
* @default null
*/
headerProperties?: Partial<SectionHeader> | Partial<ImgSectionHeader>;

/**
* Is this section closeable?
*  
*  Closeable sections show a SectionStack.closeSectionButton which will invoke
*  SectionStack.closeSection when clicked.
*  
*  This property overrides the default SectionStack.canCloseSections setting.
* @type {boolean}
* @default null
*/
canClose?: boolean;

/**
* Icon src for the SectionStack.closeSectionButton if 
*  SectionStackSection.canClose is true.
*  
*  If specified this takes precedence over SectionStack.closeSectionIcon.
* @type {SCImgURL}
* @default null
*/
closeIcon?: SCImgURL;

/**
* Pixel width/height for this sections SectionStackSection.closeIcon.
*  If unset SectionStack.closeSectionIconSize will be used.
* @type {number}
* @default null
*/
closeIconSize?: number;





}


/**
* Flags for XML serialization
*/
interface SerializationContext {
/**
* Enables flat serialization mode, as described for DSRequest.useFlatFields.
* @type {boolean}
* @default null
*/
useFlatFields?: boolean;





}


/**
* Settings to control EditContext serialization.
*/
interface SerializationSettings {
/**
* When true specify DataSources in full rather than assuming they can be
*  downloaded from the server.
* @type {boolean}
* @default null
*/
serverless?: boolean;

/**
* Overrides the default indention setting during serialization. XML defaults
*  to indented and JSON defaults to non-indented.
* @type {boolean}
* @default null
*/
indent?: boolean;

/**
* Overrides the default component output setting during serialization. By default
*  Canvas and DrawItem components are serialized individually and referenced by their
*  containers.
* @type {boolean}
* @default true
*/
outputComponentsIndividually?: boolean;





}


/**
* The ServerObject tells the ISC server how to find or create a server-side object involved in
*  dmiOverview (Direct Method Invocation). 
*  
*  A ServerObject declaration appears in the XML definition of a DataSource (for
*  responding to DSRequests) or in an Application configuration file (.app.xml) for
*  responding to RPCRequests.
*  
*  NOTE: Please take note of the points made in 
*  serverDataSourceImplementation of caching and thread-safety
*  issues in server-side DataSources.
*/
interface ServerObject {
/**
* Specifies the mechanism for locating the class instance on which to invoke the method.
*  Valid values are as follows:
*  
*  
* - "spring": For use with the Spring framework.
*  ServerObject.bean contains the name of the bean to invoke. Which application context
*  is used can be configured via web.xml (see the example web.xml in the SDK). See also
*  serverInit for special concerns with framework initialization when using
*  Spring.
*  
* - "cdi": For use with 
*  CDI (Contexts and Dependency Injection).
*  Use ServerObject.bean to configure the name of the 
*  bean to invoke or, alternatively, ServerObject.className to configure its class name.
*  
* - "new": A new instance of the class specified by ServerObject.className will be
*  created and the DMI method will be invoked on that instance (unless the specified method is
*  static, in which case no instance is created, but the class specified by
*  ServerObject.className is still used).
*  
* - "factory": A custom factory provides the class instance on which the DMI method is to be
*  invoked. In this case, ServerObject.className specifies the className of the factory
*  that will provide the instance on which the DMI method is to be invoked. The class
*  specified by ServerObject.className must provide exactly one method named
*  create that must return the class instance on which you wish the DMI method to
*  be invoked. Like the DMI methods, the create method can request a standard set of
*  values as arguments. See dmiOverview for a list of available values.
*  
* - "attribute": The instance on which the DMI method is to be invoked is looked up in the
*  scope defined by ServerObject.attributeScope via the attribute name specified in
*  ServerObject.attributeName.
*  
* @type {"new" | "factory" | "attribute" | "spring" | "cdi"}
* @default "new"
*/
lookupStyle?: "new" | "factory" | "attribute" | "spring" | "cdi";

/**
* Specifies the fully-qualified class name that provides the server-side endpoint of the DMI
*  (ServerObject.lookupStyle:"new") or the class name of the factory that produces the
*  DMI instance (ServerObject.lookupStyle:"factory").
*  
*  This is one of the values that you need to pass to DMI.call to invoke the DMI from
*  the client.
*  
*  The value of this attribute is used for "new" and "factory"
*  values of ServerObject.lookupStyle.
*  
*  It is also used for "cdi" value of ServerObject.lookupStyle, to provide 
*  class name of the bean to ask CDI to create.
* @type {string}
* @default null
*/
className?: string;

/**
* Specifies the name of the method to call for operations using this ServerObject. This is 
*  a DataSource-level default; you can override it for individual operations either by 
*  specifying the OperationBinding.serverMethod attribute, or by declaring an 
*  operation-level serverObject that specifies a different methodName (if you specify both 
*  an operationBinding.serverMethod and an operation-level serverObject.methodName, the 
*  latter takes precedence)
* @type {string}
* @default null
*/
methodName?: string;

/**
* You can optionally specify an ID on the ServerObject config block - in which case you can
*  use that value as the "className" argument when calling DMI.call. This allows you to
*  hide the name of the server-side class used as the factory or implementer of the DMI from
*  the browser as a security precaution.
* @type {string}
* @default null
*/
ID?: string;

/**
* For use when ServerObject.lookupStyle is "spring" or "cdi", 
*  id (name) of the bean to ask Spring (CDI) to create.
* @type {string}
* @default null
*/
bean?: string;

/**
* Specifies the name of the attribute by which to look up the DMI instance. This attribute is
*  consulted only when the value of ServerObject.lookupStyle is "attribute".
* @type {string}
* @default null
*/
attributeName?: string;

/**
* Specifies the scope in which the DMI instance is to be looked up. Valid values are: 
*  "request", "session", and "application". If
*  attributeScope is left out of the ServerObject definition, then
*  all scopes are searched in the order in which they are listed above.
*  
*  This attribute is consulted only when the value of ServerObject.lookupStyle is
*  "attribute".
* @type {"request" | "session" | "application"}
* @default null
*/
attributeScope?: "request" | "session" | "application";

/**
* When the ServerObject appears in a .app.xml file (for RPC DMI), this property
*  specifies the list of methods on the ServerObject that are callable from the client. See
*  the builtin.app.xml file in the /shared/app directory of the SDK for an example of a
*  visibleMethods declaration block.
* @type {List}
* @default null
*/
visibleMethods?: List;

/**
* By default, for DMI DSResponses, DSResponse.data is filtered on the server to just the set
*  of fields defined on the DataSource. This behavior can be overridden in several ways - see
*  the overview in dmiOverview for details. The value of this attribute overrides
*  DataSource.dropExtraFields.
* @type {boolean}
* @default null
*/
dropExtraFields?: boolean;

/**
* If set, the SmartClient server will use JXPath to call your server code. The
*  JXPathContext (start point) will be the object arrived at by applying the 
*  ServerObject.lookupStyle and related ServerObject properties. The 
*  intention of this property is to allow easier access to your existing Java objects 
*  and reduce the need to write SmartClient-specific server code.
* @type {string}
* @default null
*/
targetXPath?: string;

/**
* For a ServerObject defined at the DataSource.serverObject, by 
*  default we only allow it to intercept standard CRUD operations (ie, ordinary fetches, adds,
*  updates and removes). To allow the ServerObject to intercept other types of operation - 
*  custom operations, validations, etc - set this property to false. Note that ServerObjects 
*  declared at the OperationBinding.serverObject always intercept
*  that operation, whatever its type, and this property has no effect.
*  
*  NOTE: If you are intercepting operations on the server because you wish to inspect
*  them before deciding whether to process them with bespoke code or allow them to proceed 
*  with normal processing, the way to invoke normal processing without causing any interference
*  is:
*   return dsRequest.execute();
*  
* @type {boolean}
* @default null
*/
crudOnly?: boolean;





}


/**
* A class used to define a shadow used in a Draw&lt;Shape&gt; Types.
*/
interface Shadow {
/**
* @type {CSSColor}
* @default "black"
*/
color?: CSSColor;

/**
* @type {Integer}
* @default "10"
*/
blur?: Integer;

/**
* @type {Point}
* @default "[0,0]"
*/
offset?: Point;





}


/**
* Definition of a simple linear gradient defined by 2 colors and a SimpleGradient.direction.
*/
interface SimpleGradient extends Gradient {
/**
* Angle of the direction vector in degrees. The default of 0.0 causes the gradient to sweep
*  from the start color on the left to the end color on the right. Positive direction angles
*  correspond to clockwise rotations of the default gradient.
* @type {Float}
* @default "0.0"
*/
direction?: Float;

/**
* The start color of the gradient.
* @type {CSSColor}
* @default null
*/
startColor?: CSSColor;

/**
* The end color of the gradient.
* @type {CSSColor}
* @default null
*/
endColor?: CSSColor;





}


/**
* A Javascript object defining the details of a single sort operation.
*  
*  You can convert between SortSpecifiers and the string format required by 
*  DSRequest.sortBy by calling DataSource.getSortBy and 
*  DataSource.getSortSpecifiers.
*/
interface SortSpecifier {
/**
* The property name, eg a ListGridField, to which this sortSpecifier applies.
* @type {string}
* @default null
*/
property?: string;

/**
* The direction in which this specifier should sort.
* @type {SortDirection}
* @default null
*/
direction?: SortDirection;

/**
* A normalizer function which this sortSpecifier will use to sort.
* @type {Function}
* @default null
*/
normalizer?: Function;

/**
* A DataBoundComponent providing the context for the sort-normalizer.
* @type {DataBoundComponent}
* @default null
*/
context?: DataBoundComponent;





}


/**
* An object representing one of the image segments displayed by a StretchImg. Each item of
*  a StretchImg's StretchImg.items array is a StretchItem.
*/
interface StretchItem {
/**
* The width of the image. This can either be a number (for the number of pixels wide), the string
*  "*" (remaining space, divided amongst all items that specify width:"*"), or the name of a property
*  on the StretchImg component, such as "capSize" for the StretchImg's StretchImg.capSize.
*  
*  NOTE: The width is only used if the StretchImg stacks its images horizontally
*  (StretchImg.vertical is false).
* @type {number | string}
* @default null
*/
width?: number | string;

/**
* The height of the image. This can either be a number (for the number of pixels tall), the string
*  "*" (remaining space, divided amongst all items that specify height:"*"), or the name of a property
*  on the StretchImg component, such as "capSize" for the StretchImg's StretchImg.capSize.
*  
*  NOTE: The height is only used if the StretchImg stacks its images vertically
*  (StretchImg.vertical is true).
* @type {number | string}
* @default null
*/
height?: number | string;

/**
* A string that is appended as a suffix to the StretchImg's StretchImg.src
*  URL in order to fetch the media file for this StretchItem, if a separate StretchItem.src is
*  not provided. Note that the special name "blank", possibly suffixed by one or more digits
*  which are used to differentiate blank items, means no image will be shown for this StretchItem.
*  
*  For example, for a StretchImg in "Over" state with a StretchImg.src of "button.png"
*  and a name of "stretch", the resulting URL would be "button_Over_stretch.png".
* @type {string}
* @default null
*/
name?: string;

/**
* The URL of the media file for this StretchItem.
* @type {SCImgURL}
* @default null
*/
src?: SCImgURL;





}




/**
* An ordinary JavaScript object with properties that define an editable group of 
*  StyleSetting for use in CSSEditors.
*/
interface StyleGroup {
/**
* Unique name that identifies this group.
* @type {string}
* @default null
*/
name?: string;

/**
* Title to display in the header for this group.
* @type {HTML}
* @default null
*/
title?: HTML;

/**
* Should this group's section be expanded?
* @type {boolean}
* @default null
*/
expanded?: boolean;

/**
* Can this group's section be collapsed?
* @type {boolean}
* @default null
*/
canCollapse?: boolean;

/**
* Does this group allow items to be edited asymmetrically? For example, should settings
*  like padding allow each side to be specified separately?
* @type {boolean}
* @default null
*/
allowAsymmetry?: boolean;

/**
* The array of StyleSetting to make available for editing in this 
*  group.
* @type {Array<Partial<StyleSetting>>}
* @default null
*/
settings?: Array<Partial<StyleSetting>>;





}


/**
* An ordinary JavaScript object with properties that configure a setting for use in a
*  CSSEditor.
*/
interface StyleSetting {
/**
* Unique name that identifies this setting.
* @type {string}
* @default null
*/
name?: string;

/**
* Title for this setting when applied to a FormItem.
* @type {string}
* @default null
*/
title?: string;

/**
* The CSS attribute name used when generating CSS from this item. Rarely used to override
*  the default behavior of using the StyleSetting.name.
* @type {string}
* @default null
*/
settingName?: string;

/**
* Name of the StyleGroup this setting will appear in for editing.
* @type {string}
* @default null
*/
groupName?: string;

/**
* Can this setting be edited asymmetrically? For example, a border-setting can be applied
*  to all sides at once, or asymmetrically to each side, as four separate border-settings.
* @type {boolean}
* @default null
*/
allowAsymmetry?: boolean;

/**
* If this setting allows StyleSetting.allowAsymmetry, should it
*  be edited in that mode by default?
* @type {boolean}
* @default null
*/
showAsymmetry?: boolean;

/**
* The type of editor to use for this setting.
* @type {FormItemType}
* @default null
*/
defaultEditorType?: FormItemType;

/**
* FormItem properties to apply to editors for this setting.
* @type {Partial<FormItem>}
* @default null
*/
editorProperties?: Partial<FormItem>;





}


/**
* Settings for use with SimpleType.applySummaryFunction.
*/
interface SummaryConfiguration {
/**
* The field value to treat as the bad result of a user formula or summary evaluation.
*  If a summary function actually uses the value (rather than say "count"), this usually
*  means that the value will simply be skipped rather than voiding evaluation of the
*  entire summary.
* @type {string}
* @default "."
*/
badFormulaResultValue?: string;

/**
* The field value to treat as an invalid value from a summary row (see 
*  ListGrid.showGridSummary or ListGrid.showGroupSummary) or as an invalid value
*  in a summary-type field (see ListGridFieldType).
*  If a summary function actually uses the value (rather than say "count"), this usually
*  means that the value will simply be skipped rather than voiding evaluation of the
*  entire summary.
* @type {string}
* @default "&amp;nbsp;"
*/
invalidSummaryValue?: string;





}


/**
* Configuration object for AutoTest.isSystemDone.
*  
*  Note that this format is extended by SystemWaitConfig and
*  ElementWaitConfig, used by
*  AutoTest.waitForSystemDone and method:AutoTest.waitForElementDone.
*  
*  Note also that method:waitForSystemDone may have different default configuration
*  from method:isSystemDone - the default behavior for each option is documented
*  on the appropriate config object type for the method.
*/
interface SystemDoneConfig {
/**
* Should the system wait for any outstanding redraws to complete before
*  method:isSystemDone returns true?
* @type {boolean}
* @default false
*/
includeRedraws?: boolean;

/**
* Should the system wait for any outstanding RPCManager to complete before
*  method:isSystemDone returns true?
*  
*  If not explicitly set, AutoTest.implicitNetworkWait will be consulted
* @type {boolean}
* @default null
*/
includeNetworkOperations?: boolean;

/**
* Should the system wait for any outstanding FileLoader requests?
*  to complete before method:isSystemDone returns true?
* @type {boolean}
* @default true
*/
includeFileLoader?: boolean;

/**
* Should the system wait for all outstanding registered Timer.setTimeout
*  to complete before method:isSystemDone returns true?
* @type {boolean}
* @default false
*/
includeTimers?: boolean;





}


/**
* Configuration object for AutoTest.waitForSystemDone
*/
interface SystemWaitConfig extends SystemDoneConfig {
/**
* Timeout for AutoTest.waitForElement in ms.
*  
*  If unset a default timeout of 30 seconds will be used.
* @type {number}
* @default null
*/
timeout?: number;

/**
* Should the system wait for any outstanding redraws to complete before
*  method:isSystemDone returns true?
* @type {boolean}
* @default true
*/
includeRedraws?: boolean;

/**
* Should the system wait for any outstanding RPCManager to complete before
*  method:isSystemDone returns true?
*  
*  If not explicitly set, AutoTest.implicitNetworkWait will be consulted
* @type {boolean}
* @default true
*/
includeNetworkOperations?: boolean;

/**
* Should the system wait for any outstanding FileLoader requests?
*  to complete before method:isSystemDone returns true?
* @type {boolean}
* @default true
*/
includeFileLoader?: boolean;

/**
* Should the system wait for all outstanding registered Timer.setTimeout
*  to complete before method:isSystemDone returns true?
* @type {boolean}
* @default true
*/
includeTimers?: boolean;





}


/**
* 
*  Tabs are specified as objects, not class instances. For example, when
*  developing in JavaScript, a typical initialization block for a TabSet would look like
*  this:
*  
*  TabSet.create({
*    tabs: [
*      {title: "tab1", pane: "pane1"},
*      {title: "tab2"}
*    ]
*  });
*  
*  And in XML:
*  
*  &lt;TabSet&gt;
*   &lt;tabs&gt;
*     &lt;Tab title="tab1" pane="pane1"/&gt;
*     &lt;Tab title="tab2"/&gt;
*   &lt;/tabs&gt;
*  &lt;/TabSet&gt;
*  
*  
*  
*/
interface Tab {
/**
* Specifies the title of the this tab. To change the title after the TabSet
*  has been created, call TabSet.setTabTitle.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* If specified, overrides the TabSet.canEditTabTitles setting, for this one tab
*  only.
*  
*  Note that the TabSet's TabSet.titleEditEvent must be set to a
*  supported TabTitleEditEvent in order for users to be able to edit this tab's
*  title.
*  
*  After the TabSet has been created, you can change a tab's canEditTtile
*  property by calling TabSet.setTabProperties.
* @type {boolean}
* @default null
*/
canEditTitle?: boolean;

/**
* Specifies the prompt to be displayed when the mouse hovers over the tab.
*  
*  After the TabSet has been created, you can change a tab's prompt property by
*  calling TabSet.setTabProperties.
* @type {HTMLString}
* @default null
*/
prompt?: HTMLString;

/**
* If TabSet.showTabPicker is true for this TabSet, if set this property will determine
*  the title of the picker menu item for this tab. If unset, Tab.title will be used
*  instead.
*  
*  After the TabSet has been created, you can change a tab's pickerTitle
*  property by calling TabSet.setTabProperties.
* @type {HTMLString}
* @default null
*/
pickerTitle?: HTMLString;

/**
* Specifies the pane associated with this tab. You have three options for the value of
*  the pane attribute:
*  
*  
* - ID - The global ID of an already created Canvas (or subclass).
*  
* - Canvas - A live instance of a Canvas (or subclass).
*  
* - AutoChildShortcut - String with format "autoChild:autoChildName"
*  
*  You can change the pane associated with a given tab after the TabSet has
*  been created by calling TabSet.updateTab.
* @type {Canvas | GlobalId | AutoChildShortcut}
* @default null
*/
pane?: Canvas | GlobalId | AutoChildShortcut;

/**
* Space to leave around the pane within this Tab.
*  If specified, this property takes precedence over TabSet.paneMargin
* @type {Integer}
* @default null
*/
paneMargin?: Integer;

/**
* Optional ID for the tab, which can later be used to reference the tab.
*  APIs requiring a reference to a tab will accept the tab's ID 
*  [including TabSet.selectTab, TabSet.updateTab, TabSet.removeTab].
*  The ID will also be passed to the TabSet.tabSelected and TabSet.tabDeselected
*  handler functions, if specified.
*  
*  Note that if you provide an ID, it must be globally unique. If you do not want a
*  globally unique identifier, set Tab.name instead.
* @type {GlobalId}
* @default null
*/
ID?: GlobalId;

/**
* Optional name for the tab, which can later be used to reference the tab.
*  APIs requiring a reference to a tab will accept the tab's name
*  [including TabSet.selectTab, TabSet.updateTab, TabSet.removeTab].
*  This name will also be passed to the TabSet.tabSelected and TabSet.tabDeselected
*  handler functions, if specified.
*  
*  This identifier is requred to be locally unique to the TabSet and cannot be used to get
*  a global reference to the Tab. If you want a global reference, set Tab.ID instead.
* @type {TabName}
* @default null
*/
name?: TabName;

/**
* You can specify an explicit width for the tab using this property. Note that tabs
*  automatically size to make room for the full title, but if you want to e.g. specify a
*  uniform width for all tabs in a TabSet, this property enables you to do so.
*  
*  After the TabSet has been created, you can change a tab's width
*  property by calling TabSet.setTabProperties.
* @type {number}
* @default "100"
*/
width?: number;

/**
* If enabled, the tab will collapse to show just its icon when showing the title would
*  cause overflow of a containing TabBar. While collapsed, the tab will show its
*  title on hover, unless an explicit hover has been specified such as by Tab.prompt.
* @type {boolean}
* @default false
*/
canAdaptWidth?: boolean;

/**
* If specified, this tab will initially be rendered in a disabled state. To
*  enable or disable tabs on the fly use the TabSet.enableTab, and
*  TabSet.disableTab.
*  methods.
* @type {boolean}
* @default null
*/
disabled?: boolean;

/**
* If specified this tab will be hidden by default. To show and hide tabs at runtime
*  use TabSet.showTab and TabSet.hideTab
* @type {boolean}
* @default null
*/
hidden?: boolean;

/**
* Criteria to be evaluated to determine whether this Tab should be visible.
*  
*  A basic criteria uses textMatchStyle:"exact". When specified in
*  componentXML this property allows
*  xmlCriteriaShorthand for defining criteria.
* @type {Criteria}
* @default null
*/
visibleWhen?: Criteria;

/**
* Criteria to be evaluated to determine whether this Tab should be enabled.
*  
*  A basic criteria uses textMatchStyle:"exact". When specified in
*  componentXML this property allows
*  xmlCriteriaShorthand for defining criteria.
* @type {Criteria}
* @default null
*/
enableWhen?: Criteria;

/**
* If specified, this tab will show an icon next to the tab title. 
*  
*  NOTE: if you enable TabSet.canCloseTabs,
*  tab.icon is used for the close icon. TabSet.canCloseTabs describes
*  a workaround to enable both a closeIcon and a second icon to be shown.
*  
*  Use TabSet.tabIconClick to add an event handler specifically for clicks on the icon.
*  
*  If a tab Tab.disabled, a different icon will be loaded by adding
*  a suffix to the image name (see Button.icon).
*  
*  You should specify a size for the icon via Tab.iconSize or Tab.iconWidth
*  and Tab.iconHeight. Without an explicitly specified size, tabs may be drawn
*  overlapping or with gaps the first time a page is loaded, because the icon is not cached
*  and therefore its size isn't known.
*  
*  After the TabSet has been created, you can change a tab's icon property by
*  calling TabSet.setTabIcon.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* If Tab.icon is specified, this property may be used to specify a size for the icon.
*  Per side sizing may be specified instead via Tab.iconWidth and Tab.iconHeight.
*  
*  After the TabSet has been created, you can change a tab's iconSize property
*  by calling TabSet.setTabProperties.
* @type {Integer}
* @default "16"
*/
iconSize?: Integer;

/**
* If Tab.icon is specified, this property may be used to specify a width for the
*  icon.
*  
*  After the TabSet has been created, you can change a tab's iconWidth property
*  by calling TabSet.setTabProperties.
* @type {Integer}
* @default null
*/
iconWidth?: Integer;

/**
* If Tab.icon is specified, this property may be used to specify a height for the
*  icon.
*  
*  After the TabSet has been created, you can change a tab's iconHeight
*  property by calling TabSet.setTabProperties.
* @type {Integer}
* @default null
*/
iconHeight?: Integer;

/**
* If TabSet.canReorderTabs is set to true, setting canReorder
*  explicitly to false for some tab will disallow drag-reordering of
*  this tab. Has no effect if canReorderTabs is not true at the tabSet level.
*  
*  Note that this setting also disallows a reorder of another tab into the slot before
*  or following this tab. This means for tabs located at the beginning or end of the 
*  tab-bar, users cannot changing the index of the tab by dropping another
*  before or after it. However if you have a canReorder:false
*  tab which is not at the beginning or end of the tab bar, users can
*  drag reorder other tabs around it which may ultimately change its position.
* @type {boolean}
* @default null
*/
canReorder?: boolean;

/**
* Determines whether this tab should show a close icon allowing the user to dismiss the tab
*  by clicking on the close icon directly. The URL for the close icon's image will be derived from 
*  TabSet.closeTabIcon by default, but may be overridden by explicitly specifying
*  Tab.closeIcon.
*  
*  If unset or null, this property is derived from TabSet.canCloseTabs.
*  
*  Note that setting canClose means that Tab.icon cannot be used,
*  because it's used for the Tab.closeIcon - see
*  TabSet.canCloseTabs for a workaround.
*  
*  After the TabSet has been created, you can change a tab's canClose property
*  by calling TabSet.setCanCloseTab.
* @type {boolean}
* @default null
*/
canClose?: boolean;

/**
* Custom src for the close icon for this tab to display if it is closeable.
*  See Tab.canClose and TabSet.canCloseTabs.
* @type {SCImgURL}
* @default null
*/
closeIcon?: SCImgURL;

/**
* Size of the Tab.closeIcon for this tab. If unspecified the icon will be sized
*  according to TabSet.closeTabIconSize
* @type {number}
* @default null
*/
closeIconSize?: number;



/**
*  Optional handler to fire when a tab is selected. As with TabSet.tabSelected this
*  method only fires when the tabset is drawn.
* 
* 
* @param {TabSet} the tabSet containing the tab.
* @param {Integer} the index of the newly selected tab
* @param {Canvas} the newly selected tab's pane if set
* @param {GlobalId} the ID of the newly selected tab
* @param {Tab} the tab object (not tab button instance)
* @param {TabName} the name of the newly selected tab
*/
tabSelected(tabSet:TabSet, tabNum:Integer, tabPane:Canvas, ID:GlobalId, tab:Tab, name:TabName): void;

/**
*  Optional handler to fire when a tab is deselected. Returning false
*  will cancel the new selection, leaving this tab selected.
*  As with TabSet.tabSelected this
*  method only fires when the tabset is drawn.
* 
* 
* @param {TabSet} the tabSet containing the tab.
* @param {Integer} the index of the deselected tab
* @param {Canvas} the deselected tab's pane if set
* @param {GlobalId} the ID of the deselected tab
* @param {Tab} the deselected tab object (not tab button instance)
* @param {Tab} the tab object being selected
* @param {TabName} the name of the deselected tab
*/
tabDeselected(tabSet:TabSet, tabNum:Integer, tabPane:Canvas, ID:GlobalId, tab:Tab, newTab:Tab, name:TabName): boolean;



}


/**
* Identifies a potential decision (branch) within a MultiDecisionTask. Each decision
*  has a criteria and a target ProcessElement ID.
* @deprecated * In favor of DecisionBranch as of SmartClient release 13.0

*/
interface TaskDecision extends DecisionBranch {




}


/**
* A TestFunctionResult is an ordinary JavaScript Object with properties that indicate the 
*  status of an attempt to generate and execute a function for FormulaBuilder and 
*  it's subclasses.
*  
*  Because TestFunctionResult is always an ordinary JavaScript Object, it supports the
*  normal behaviors of JavaScript Objects, including accessing and assigning to properties
*  via dot notation:
*  
*    var propValue = testFunctionResult.propName;
*    testFunctionResult.propName = newValue;
*  
*  
*/
interface TestFunctionResult {
/**
* Set to true if there is a syntax error in the formula or summary being checked.
*  
*  When set to true, TestFunctionResult.errorText contains the exception message.
* @type {boolean}
* @default false
*/
failedGeneration?: boolean;

/**
* Set to true if calling the formula or summary format resulted in a JavaScript Error.
*  This would generally indicate a reference to non-existent data values. See 
*  TestFunctionResult.failedGeneration for other types of failure.
*  
*  When set to true, TestFunctionResult.errorText contains the exception message.
* @type {boolean}
* @default false
*/
failedExecution?: boolean;

/**
* Set to true if the formula or summary definition passed in was empty.
* @type {boolean}
* @default false
*/
emptyTestValue?: boolean;

/**
* If the formula or summary format caused a JavaScript error, this contains the JavaScript error text.
* @type {string}
* @default null
*/
errorText?: string;

/**
* When a formula or summary format is valid, result contains the result returned by the
*  generated function when it was executed.
* @type {string}
* @default null
*/
result?: string;

/**
* Set to the record that was used when testing the generated function. This is the record
*  selected by FormulaBuilder.getTestRecord.
* @type {Record}
* @default null
*/
record?: Record;





}


/**
* A TileRecord is a JavaScript Object whose properties contain values for each
*  TileField. A TileRecord may have additional properties which affect the record's
*  appearance or behavior, or which hold data for use by custom logic or other, related
*  components.
*/
interface TileRecord {
/**
* By default, image-type fields shown in a Tile are auto-scaled to fit a tile's full width, 
*  and an equal share of tile-height not already required by other visible (non-image) fields. 
*  This attribute enforces a minimum image-height, below which a given image will not shrink
*  when being scaled.
* @type {Integer}
* @default "16"
*/
minimumImageFieldSize?: Integer;

/**
* SmartClient Class to use to construct the tile for this particular record.
* @type {string}
* @default null
*/
tileConstructor?: string;

/**
* Additional properties to be passed when creating a tile for this record.
* @type {Partial<Canvas>}
* @default null
*/
tileProperties?: Partial<Canvas>;





}


/**
* An object literal with a particular set of properties used to configure the display of
*  and interaction with the columns of a TreeGrid.
*  TreeGrid is a subclass of ListGrid and as a result, for all fields except
*  the field containing the Tree itself (specified by
*  TreeGridField.treeField, all properties settable on
*  ListGridField apply to TreeGridField as well.
*  
*  This class documents just those properties that are specific to TreeGridFields - see
*  ListGridField for the set of inherited properties.
*/
interface TreeGridField extends ListGridField {
/**
* The field containing treeField: true will display the Tree. If no
*  field specifies this property, if a field named after the Tree.titleProperty of
*  the Tree is present in TreeGrid.fields, that field will show the tree. Note that
*  when using a DataSource, you typically define the title field via
*  DataSource.titleField and the generated ResultTree automatically uses this
*  field.
* 
*  If none of the above rules apply, the first field in TreeGrid.fields is assigned to
*  display the Tree.
* @type {boolean}
* @default "see below"
*/
treeField?: boolean;

/**
* Dictates whether the data in this field be exported. Explicitly set this
*  to false to prevent exporting. Has no effect if the underlying 
*  DataSourceField.canExport is explicitly set to 
*  canExport: false.
* @type {boolean}
* @default null
*/
canExport?: boolean;





}


/**
* Every node in the tree is represented by a TreeNode object which is an object literal with a
*  set of properties that configure the node.
*  
*  When a Tree is supplied as TreeGrid.data to TreeGrid, you can also set
*  properties from ListGridRecord on the TreeNode (e.g. setting
*  ListGridRecord.enabled:false on the node).
*/
interface TreeNode {
/**
* Default property name denoting whether this record is enabled. Property name may be
*  modified for some grid via ListGrid.recordEnabledProperty.
* @type {boolean}
* @default null
*/
enabled?: boolean;

/**
* Governs whether this node can be dragged. Only has an effect if this node is displayed in
*  a TreeGrid where TreeGrid.canDragRecordsOut, TreeGrid.canReorderRecords
*  or TreeGrid.canReparentNodes is true.
* @type {boolean}
* @default null
*/
canDrag?: boolean;

/**
* Governs whether dragged data (typically other treeNodes) may be dropped over
*  this node. Only has an effect if this node is displayed in a TreeGrid where
*  TreeGrid.canAcceptDroppedRecords, TreeGrid.canReorderRecords or 
*  TreeGrid.canReparentNodes is true.
* @type {boolean}
* @default null
*/
canAcceptDrop?: boolean;

/**
* Set to true or a string that is not equal to (ignoring case)
*  "false" to explicitly mark this node as a folder. See Tree.isFolder for
*  a full description of how the Tree determines whether a node is a folder or not.
*  
*  Note: the name of this property can be changed by setting Tree.isFolderProperty.
* @type {boolean | string}
* @default null
*/
isFolder?: boolean | string;

/**
* Provides a name for the node that is unique among its immediate siblings, thus allowing a
*  unique path to be used to identify the node, similar to a file system. See
*  Tree.getPath.
*  
*  If the nameProperty is not set on a given node, the TreeNode.id will be used instead. If
*  this is also missing, Tree.getName and Tree.getPath will auto-generate a
*  unique name for you. Thus names are not required, but if the dataset you are using already
*  has usable names for each node, using them can make APIs such as Tree.find more
*  useful. Alternatively, if your dataset has unique ids consider providing those as
*  TreeNode.id.
*  
*  If a value provided for the nameProperty of a node (e.g. node.name) is not a
*  string, it will be converted to a string by the Tree via ""+value.
*  
*  This property is also used as the default title for the node (see Tree.getTitle)
*  if TreeNode.title is not specified.
*  
*  Note: the name of this property can be changed by setting Tree.nameProperty.
* @type {string}
* @default "null, but see below"
*/
name?: string;

/**
* The title of the node as it should appear next to the node icon in the Tree. If left
*  unset, the value of TreeNode.name is used by default. See the description in
*  Tree.getTitle for full details.
*  
*  Note: the name of this property can be changed by setting Tree.titleProperty.
* @type {HTMLString}
* @default null
*/
title?: HTMLString;

/**
* Specifies the unique ID of this node. 
*  
*  Required for trees with Tree.modelType "parent". With modelType:"parent", the unique
*  ID of a node, together with the unique ID of its parent (see TreeNode.parentId) is
*  used by Tree.linkNodes to link a list of nodes into a tree.
*  
*  Note: the name of this property can be changed by setting Tree.idField.
* @type {string | number}
* @default null
*/
id?: string | number;

/**
* For trees with modelType:"parent", this property specifies the unique ID of this node's 
*  parent node.
*  The unique ID of a node, together with the unique ID of its parent is used by
*  Tree.linkNodes to link a list of nodes into a tree.
*  
*  Note: the name of this property can be changed by setting Tree.parentIdField.
* @type {string | number}
* @default null
*/
parentId?: string | number;

/**
* For trees with the modelType "children", this property specifies the children of this
*  TreeNode.
*  
*  Note: the name of this property can be changed by setting Tree.childrenProperty
* @type {Array<Partial<TreeNode>>}
* @default null
*/
children?: Array<Partial<TreeNode>>;

/**
* This Property allows the developer to customize the icon displayed next to a node.
*  Set node.icon to the URL of the desired icon to display and
*  it will be shown instead of the standard TreeGrid.nodeIcon for this node.
*  Note that if TreeNode.showOpenIcon and/or TreeNode.showDropIcon 
*  is true for this node, customized icons for folder nodes will be appended with the 
*  TreeGrid.openIconSuffix or TreeGrid.dropIconSuffix suffixes on state change 
*  as with the standard TreeGrid.folderIcon for this treeGrid. Also note that for
*  custom folder icons, the TreeGrid.closedIconSuffix will never be appended.
*  You can change the name of this property by setting 
*  TreeGrid.customIconProperty.
* @type {SCImgURL}
* @default null
*/
icon?: SCImgURL;

/**
* For folder nodes showing custom icons (set via TreeNode.icon),
*  this property allows the developer to specify on a per-node basis whether an
*  open state icon should be displayed when the folder is open.
*  Set node.showOpenIcon to true to show the open state
*  icons, or false to suppress this.
*  If not specified, this behavior is determined by TreeGrid.showCustomIconOpen
*  for this node.
*  You can change the name of this property by setting 
*  TreeGrid.customIconOpenProperty.
* @type {boolean}
* @default false
*/
showOpenIcon?: boolean;

/**
* For folder nodes showing custom icons (set via TreeNode.icon),
*  this property allows the developer to specify on a per-node basis whether a
*  drop state icon should be displayed when the 
*  user drop-hovers over this folder.
*  Set node.showDropIcon to true to show the drop state
*  icon, or false to suppress this.
*  If not specified, this behavior is determined by TreeGrid.showCustomIconDrop
*  for this node.
*  You can change the name of this property by setting 
*  TreeGrid.customIconDropProperty.
* @type {boolean}
* @default false
*/
showDropIcon?: boolean;

/**
* For folder nodes showing custom icons (set via TreeNode.icon),
*  this property allows the developer to specify on a per-node basis whether a
*  selected state icon should be displayed when the folder is open.
*  Set node.showSelectedIcon to true to show the selected state
*  icons, or false to suppress this.
*  If not specified, this behavior is determined by TreeGrid.showCustomIconSelected
*  for this node.
*  You can change the name of this property by setting 
*  TreeGrid.customIconSelectedProperty.
* @type {boolean}
* @default false
*/
showSelectedIcon?: boolean;

/**
* Developers may customize the padding between the folder or leaf node icon and the
*  text content of the tree cell for individual nodes.
*  
*  You can change the name of this property by setting TreeGrid.iconPaddingProperty
* @type {Integer}
* @default null
*/
iconPadding?: Integer;





}


/**
* An object representing a user-created and user-modifiable formula, which can be created and
*  edited with a FormulaBuilder either directly or via the
*  ListGrid.canAddFormulaFields behavior.
*  
*  Note that the current implementation of UserFormula simply executes UserFormula.text
*  as a JavaScript string after making special variables and methods available to the formula.
*  It is safe to allow users to define formulas for themselves (since an end user can always
*  execute whatever JavaScript they want via the browser's built-in developer tools), and is
*  safe to allow formulas to be shared between trusted users. However it would not be safe to
*  allow an untrusted user to create formulas that are shared to other users.
*  
*  Also, while the current implementation would allow creation of a formula that calls
*  JavaScript functions that are not part of the standard or custom
*  MathFunction, this should not be relied upon, as future versions of the
*  formula engine may prohibit such calls.
*/
interface UserFormula {
/**
* Formula to be evaluated.
*  
*  There are two contexts where a UserFormula is used: ListGridField.userFormula and
*  FormItem.formula or ListGridField.editorFormula. For the grid field formula
*  all variables used by the formula must be single-letter capital characters (eg A). These
*  are derived from field values for the record in question - see UserFormula.formulaVars.
*  
*  In addition to these variables, the keyword record may be used to
*  refer directly to the record for which the formula is being displayed.
*  
*  In the second usage context variables are dot-separated (.) names representing the nested
*  hierarchy path to the desired value within the Canvas.ruleScope. No
*  mapping with UserFormula.formulaVars is needed.
*  
*  The formula text must be valid JavaScript code and may only call either the built-in
*  MathFunction or a MathFunction.registerFunction.
* @type {string}
* @default null
*/
text?: string;

/**
* Object mapping from variable names to fieldNames. All variable names must be single-letter
*  capital characters (eg A). For example, for a formula that should divide the field
*  "population" over the field "area", the formula might be "E/L" and formula vars would be:
*  
*  
*   {
*     E: "population",
*     L: "area"
*   }
*  
*  
*  
*  
*  When used in ListGridField.userFormula context, field names are evaluated against the
*  grid record.
*  
*  When used in FormItem.formula or ListGridField.editorFormula this property is
*  not used for formula mapping. Instead, field names are evaluated directly against the
*  current Canvas.ruleScope.
* @type {Map<any>}
* @default null
*/
formulaVars?: Map<any>;





}


/**
* An object representing a user-created and user-modifiable summary, which can be created and
*  edited with a SummaryBuilder either directly or via the
*  ListGrid.canAddSummaryFields behavior.
*/
interface UserSummary {
/**
* Summary to be evaluated.
*  
*  There are two contexts where a UserSummary is used: ListGridField.userSummary and
*  FormItem.textFormula or ListGridField.editorTextFormula. For the grid field
*  summary all variables used by the summary
*  must be all-capital letter names surrounded by braces and escaped with a # sign (eg #{A}).
*  These are derived from field values for the record in question - see UserSummary.summaryVars.
*  
*  In the second usage context variables are dot-separated (.) names representing the nested
*  hierarchy path to the desired value within the Canvas.ruleScope. No
*  mapping with UserSummary.summaryVars is needed.
* @type {string}
* @default null
*/
text?: string;

/**
* Map from the all-capital letter names used as summary variables in the UserSummary
*  String to the fieldNames these variables should represent in the context where the
*  summary is evaluated.
*  
*  When used in ListGridField.userSummary context, field names are evaluated against the
*  grid record.
*  
*  When used in FormItem.textFormula or ListGridField.editorTextFormula this property
*  is not used for summary mapping. Instead, field names are evaluated directly against the
*  current Canvas.ruleScope.
* @type {Map<any>}
* @default null
*/
summaryVars?: Map<any>;





}


/**
* Validator definition for a built-in Validator.type.
*/
interface ValidatorDefinition {
/**
* Type of the validator unique in ValidatorType.
* @type {string}
* @default null
*/
type?: string;

/**
* Optional name to be shown in tools that edit validators. If not specified,
*  the tools will derive the short name from the ValidatorDefinition.type by assuming it is
*  camelCaps similar to DataSource.getAutoTitle.
* @type {string}
* @default null
*/
shortName?: string;

/**
* Does this validator only run server-side?
* @type {boolean}
* @default false
*/
requiresServer?: boolean;

/**
* Default error message to be shown when validator fails validation. Can be overridden
*  for an individual validator by setting Validator.errorMessage.
* @type {string}
* @default null
*/
defaultErrorMessage?: string;

/**
* Callback, function or JavaScript expression invoked to perform the actual validation
*  of a value.
*  
*  Because the validator itself is passed as a parameter, you can effectively parameterize it.
*  For example, to create a validator that checks that the value is after a certain date:
*    { type:"custom", afterDate:new Date(), 
*     condition:"value.getTime() > validator.afterDate.getTime()" }
*  
*  Note that, if a field is declared with a builtin FieldType, the value passed in
*  will already have been converted to the specified type, if possible.
*  
*  For the required parameters, see the documentation for
*  Callbacks.ValidatorConditionCallback.
* @type {StringMethod | ValidatorConditionCallback}
* @default null
*/
condition?: StringMethod | ValidatorConditionCallback;

/**
* Callback, function, or JavaScript expression called after every validation (i.e. call
*  to ValidatorDefinition.condition) whether it passed or failed. This allows the
*  validator perform an operation on the field based on the validation outcome.
*  
*  An action() function is not needed to report an error message only.
*  
*  For the required parameters, see the documentation for
*  Callbacks.ValidatorActionCallback.
* @type {StringMethod | ValidatorActionCallback}
* @default null
*/
action?: StringMethod | ValidatorActionCallback;





}


/**
* A WSRequest (or "web service request") is an extended RPCRequest with additional properties
*  applicable to WSDL/SOAP web services.
*  
*  All properties which are legal on RPCRequest are legal on a WSRequest, in
*  addition to the properties listed here.
*/
interface WSRequest {
/**
* Name of the web service operation to invoke.
* @type {string}
* @default null
*/
wsOperation?: string;

/**
* Data to be serialized to XML to form the SOAP body.
* @type {any}
* @default null
*/
data?: any;

/**
* When useFlatFields is set for a request to be sent to a WSDL web service, when
*  creating the input XML message to send to the web service, properties in
*  WSRequest.data will be used as the values for XML elements of the same name, at
*  any level of nesting.
*  
*  useFlatFields allows you to ignore gratuitous XML message structure, such as
*  extra levels of nested elements, and provides some insulation against changes in the
*  required structure of the input message.
*  
*  For example, given this input message:
*  
*  &lt;FindServices&gt;
*    &lt;searchFor&gt;search text&lt;/searchFor&gt;
*    &lt;Options&gt;
*      &lt;caseSensitive&gt;false&lt;/caseSensitive&gt;
*    &lt;/Options&gt;
*    &lt;IncludeInSearch&gt;
*      &lt;serviceName&gt;true&lt;/serviceName&gt;
*      &lt;documentation&gt;true&lt;/documentation&gt;
*      &lt;keywords&gt;true&lt;/keywords&gt;
*    &lt;/IncludeInSearch&gt;
*  &lt;/FindServices&gt;
*  
*  If useFlatFields were not set, in order to fill out this message
*  correctly, request.data would need to be:
*  {
*   searchFor: "search text",
*   Options : {
*     caseSensitive: false,
*   },
*   IncludeInSearch : {
*     serviceName: true,
*     documentation : true,
*     keywords : true
*   }
*  }
*  However if useFlatFields were set, request.data could be just:
*  {
*   searchFor: "search text",
*   caseSensitive: false,
*   serviceName: true,
*   documentation : true,
*   keywords : true
*  }
*  useFlatFields is often set when the input data comes from a DynamicForm
*  to avoid the cumbersome and fragile process of mapping input fields to an XML structure.
*  
*  OperationBinding.useFlatFields can also be set to cause all dsRequests of a
*  particular type to useFlatFields automatically.
*  
*  For DataBoundComponent, 
*  DataBoundComponent.useFlatFields can be set use "flattened"
*  binding to fields of a WSDL message or XML Schema.
*  
*  Note that useFlatFields is not generally recommended for use with XML input
*  messages where multiple simple type fields exist with the same name, however if used in this
*  way, the first field to use a given name wins. "first" means the first field encountered in a
*  depth first search. "wins" means only the first field will be populated in the generated
*  XML message.
* @type {boolean}
* @default null
*/
useFlatFields?: boolean;

/**
* Optional object declaring namespace prefixes for use in evaluating the
*  resultType parameter of WebService.callOperation, if resultType is an
*  XPath.
*  
*  Format is identical to OperationBinding.xmlNamespaces, and default namespaces
*  bindings are also identical.
* @type {object}
* @default null
*/
xmlNamespaces?: object;

/**
* Valid only with WebService.callOperation. If set, do not transform XML results to
*  JavaScript. Instead just return the XML nodes selected by the passed XPath or recordName,
*  or all nodes within the SOAP body if no XPath was passed.
* @type {boolean}
* @default false
*/
xmlResult?: boolean;

/**
* Data to be serialized to form the SOAP headers, as a map from the header part name to the
*  data. For example, given WSDL like this:
*  
*    &lt;soap:header part="SessionHeader" message="tns:HeaderMessage"/&gt;
*    &lt;soap:header part="CallOptions" message="tns:HeaderMessage/&gt;
*  
*  headerData like this might be provided:
*  
*    dsRequest.headerData = 
*      { SessionHeader : data
*       CallOptions : data };
*  
*  The provided data will be serialized to XML by the
*  WebService.getInputHeaderSchema via
*  DataSource.xmlSerialize
* @type {any}
* @default null
*/
headerData?: any;





}


/**
* A library of functions for determining the types of other objects.
* 
*  The "isA" methods for the basic JavaScript types are much faster and more consistent across
*  platforms than JavaScript's "typeof" operator.
* 
*  An isA method is automatically created for every ISC Class and Interface definition, for
*  example, isA.Canvas().
*/
interface isA {




}




/**
*/
type Alignment = 
/**
* Center within container.
*/
"center" |

/**
* Stick to left side of container.
*/
"left" |

/**
* Stick to right side of container.
*/
"right";


/**
* String specifying effect to apply during an animated show or hide.
*/
type AnimateShowEffectId = 
/**
* content slides into or out of view as the widget grows or shrinks
*/
"slide" |

/**
* content is revealed or wiped as the widget grows or shrinks
*/
"wipe" |

/**
* widget's opacity smoothly fades into or out of view
*/
"fade" |

/**
* widget moves into position from offscreen
*/
"fly";


/**
* Acceleration effect for animations. Can either be a ratio function or a string.
*  Ratio functions take a value between 0 and 1 which represents how much of the 
*  animation's duration has elapsed, and return another value between 0 and 1 indicating
*  how close the animation is to completion. For a completely linear animation, the 
*  function would return the value it was passed. This allows you to bias animations to
*  [for example] speed up toward the end of the animation.
*  The following strings are also supported for common ratio bias effects:
*/
type AnimationAcceleration = 
/**
* - animation will speed up as time elapses
*/
"smoothStart" |

/**
* - animation will slow down as time elapses
*/
"smoothEnd" |

/**
* - animation will speed up in the middle
*/
"smoothStartEnd" |

/**
* - no bias
*/
"none";


/**
* During a Canvas.animateResize. when should the layout of the
*  children be updated?
*/
type AnimationLayoutMode = 
/**
* - for every change to the target's width or height
*/
"always" |

/**
* - for every size change which leaves the target overflowed
*/
"overflow" |

/**
* - only layout the children at the end of the animation
*/
"atEnd";


/**
* What to do if the user hits Up or Down arrow key while editing a cell.
*/
type ArrowKeyEditAction = 
/**
* The grid will take no special action when the user presses up or down  arrow keys within an editor
*/
"none" |

/**
* The grid will intercept up and down arrow keypresses and navigate  to the next or previous edit row by generating an appropriate EditCompletionEvent
*/
"editNext";


/**
* Supported styles for arrowheads.
*/
type ArrowStyle = 
/**
* Filled triangle.
*/
"block" |

/**
* Two lines are drawn on each side of the path.
*/
"open";


/**
* An autoChild is an automatically generated subcomponent that a component creates to
*  handle part of its presentation or functionality. An example is the Window component and
*  its subcomponent the "header".
*  
*  See autoChildUsage for more information.
*/
type AutoChild = object;


/**
* A string with the format "autoChild:autoChildName" passed as a Tab.pane or
*  SectionStackSection.items instead of an actual Canvas or ID.
*/
type AutoChildShortcut = string;


/**
* AutoComplete behavior for FormItem.
*/
type AutoComplete = 
/**
* Disable browser autoComplete. Note that some browsers will disregard  this setting and still perform native autoComplete for certain items - typically
*   only for log in / password forms. See the discussion FormItem.autoComplete.
*/
"none" |

/**
* Allow native browser autoComplete.
*/
"native";


/**
* Event on a listGrid header to trigger auto-fit of the listgrid field.
*/
type AutoFitEvent = 
/**
* React to a double click on the listGrid header.
*/
"doubleClick" |

/**
* React to a click on the listGrid header.
*/
"click" |

/**
* No event will trigger auto-fit.
*/
"none";


/**
* How should fields of ListGridFieldType be sized by default?
*/
type AutoFitIconFieldType = 
/**
* Apply no special sizing to icon fields - treat them like any other  field in the grid
*/
"none" |

/**
* size the field to accommodate the width of the icon
*/
"iconWidth" |

/**
* size the field to accommodate the title (or the width of the icon if  it exceeds the width of the title.
*/
"title";


/**
* How should field width be determined when ListGridField.autoFitWidth is true?
*/
type AutoFitWidthApproach = 
/**
* Size field to fit to the data value(s) contained in the field.
*/
"value" |

/**
* Size field to fit the field title
*/
"title" |

/**
* Size field to fit either the field title or the data values in the field (whichever requires more space).
*/
"both";


/**
* What should drive the automatic expansion of the chart?
*/
type AutoScrollDataApproach = 
/**
* Expand the chart to make room for data label facet value. Unused in Bar-type charts
*/
"labels" |

/**
* Expand the chart to accommodate FacetChart.barMargin,          FacetChart.minBarThickness, and FacetChart.getMinClusterSize.
*/
"clusters" |

/**
* Expand the chart to make room for both labels and clusters (whichever requires        more space).
*/
"both";


/**
* Selection model for CubeGrid indicating which cells in the body should
*  be selected when row or column headers are selected.
*/
type AutoSelectionModel = 
/**
* Rows and Columns will be selected on header selection
*/
"both" |

/**
* Rows will be selected on row-header selection
*/
"rows" |

/**
* Columns will be selected on column-header selection
*/
"cols" |

/**
* Selecting row or column headers will not select cells in the body.
*/
"none";


/**
* An autoTestLocator is an xpath-like string used by the AutoTest subsystem to robustly 
*  identify DOM elements within a SmartClient application.
*  
*  Typically AutoTestLocators will not be hand-written - they can be retrieved by a
*  call to AutoTest.getLocator, or by clicking on the page after invoking the
*  AutoTest.installLocatorShortcut method.
*/
type AutoTestLocator = string;


/**
* Possible values to change the behavior of how data will fill the ListGrid.
*/
type Autofit = 
/**
* expand vertically to accommodate records.
*/
"vertical" |

/**
* expand horizontally to accommodate fields.
*/
"horizontal" |

/**
* expand horizontally and vertically to accommodate content.
*/
"both";


/**
* An axis or "side" of a table.
*/
type Axis = 
/**
* Row axis
*/
"row" |

/**
* Column axis
*/
"column";


/**
* Possible values for Canvas.backgroundRepeat.
*/
type BackgroundRepeat = 
/**
* Tile the background image horizontally and vertically.
*/
"repeat" |

/**
* Don't tile the background image at all.
*/
"no-repeat" |

/**
* Repeat the background image horizontally but not vertically.
*/
"repeat-x" |

/**
* Repeat the background image vertically but not horizontally.
*/
"repeat-y";


/**
* CSS color specification applied to a specific HTML element on this page.
*  
*  This is a string matching the syntax as specified in CSS1, and can be
*  formatted in one of the following ways:
*  
*  
* - A keyword color, &ldquo;white&rdquo;
*  
* - Six-digit hex notation, &ldquo;#ffffff&rdquo;
*  
* - Three-digit hex notation, &ldquo;#fff&rdquo;
*  
* - 8-bit decimal notation, &ldquo;rgb(255, 255, 255)&rdquo;
*  
* - Percentage notation, &ldquo;rgb(100%, 100%, 100%)&rdquo;
*  
*  
*  Note that when working with FacetCharts, it's required that colors be specified
*  using the six-digit hex format listed above, rather than any of the others, since the
*  Framework needs to perform math on the subfields. Affected properties include
*  FacetChart.dataColors, and affected methods include 
*  overrides of FacetChart.getDataColor and FacetChart.getDataLineColor.
*  
*/
type CSSColor = string;


/**
* CSS class name to apply to some HTML element on this page. This is a string that should
*  match the css class defined for the page in an external stylesheet or in inline
*  html &lt;STYLE&gt; tags.
*  
*  As a general rule, wherever it is possible to provide a CSS styleName (such as
*  Canvas.styleName or Button.baseStyle, your CSS style can specify border,
*  margins, padding, and any CSS attributes controlling background or text styling. You
*  should not specify any CSS properties related to positioning, clipping, sizing or
*  visibility (such as "overflow", "position", "display", "visibility" and "float") - use
*  SmartClient APIs for this kind of control.
*  
*  Because text wrapping cannot be consistently controlled cross-browser from CSS alone,
*  you should use SmartClient properties such as Button.wrap instead of the
*  corresponding CSS properties, when provided.
*  
*  Content contained within SmartClient components can use arbitrary CSS, with the
*  caveat that the content should be tested on all supported browsers, just as content
*  outside of SmartClient must be.
*  
*  Special note on CSS margins: wherever possible, use CSS padding and border in
*  lieu of CSS margins, or non-CSS approaches such as Layout.layoutMargin,
*  Canvas.snapTo, or absolute positioning (including specifying percentage left/top
*  coordinates). We recommend this because CSS specifies a very complicated and widely
*  criticized "margin-collapse" behavior which has surprising effects when margins exist on
*  both parents and children. Compounding the problem, margins are implemented very
*  differently on different browsers, especially when it comes to HTML margins.
*  
*  Note about CSS "box models"
*  
*  The CSS "box model" defines whether the size applied to a DOM element includes padding,
*  borders or margins, or whether such settings effectively increase the size of the
*  component beyond the size specified in CSS.
*  
*  In SmartClient, the size configured for a component includes border, padding and
*  margins if specified (in CSS terminology, the box model is "margin-box"). This allows
*  CSS borders, margins and padding to be treated as purely visual properties with no
*  effect on sizing or layout.
*/
type CSSStyleName = string;


/**
* A String of CSS that can be added directly to a style attribute.
*/
type CSSText = string;


/**
* Indicates how a given Calendar Save was initiated - values are dialog, editor
*  and mouse.
*/
type CalendarSaveScenario = 
/**
* the user saved via the eventDialog
*/
"dialog" |

/**
* the user saved via the eventEditor
*/
"editor" |

/**
* the user moved or resized an EventCanvas with the mouse
*/
"mouse";


/**
* A Callback is an arbitrary action to be fired - usually passed into a 
*  method to be fired asynchronously as a notificaction of some event.
*  The callback can be defined in the following formats:
*  
* - a function
*  
* - A string containing an expression to evaluate
*  
* - An object with the following properties:
*    - target: fire in the scope of this target - when the action fires,
*     the target will be available as this.
*    - methodName: if specified we'll check for a method on the target object with this 
*     name.
*  
*  Callbacks are fired via the Class.fireCallback method, which allows
*  named parameters to be passed into the callback at runtime. If the Callback was specified
*  as a string of script, these parameters are available as local variables at eval time.
*  For specific SmartClient methods that make use of Callback objects, see
*  local documentation for information on parameters and scope.
*/
type Callback = object;


/**
*/
type CharacterCasing = 
/**
* No character translation
*/
"default" |

/**
* Map characters to uppercase
*/
"upper" |

/**
* Map characters to lowercase
*/
"lower";


/**
* Known chart types. These are visual representations of data, not separate data models,
*  although some chart types are only capable of showing a single facet of data.
*  
*  Concrete charting implementations may use any value for chartType but should
*  support the provided chartType values for charts that correspond to the visual
*  presentation described below, to enable easy switching between charting engines.
*/
type ChartType = 
/**
* Values represented by area, with stacked values representing multiple          facet values. This is equivalent to ChartType "Line" with stacking enabled.
*/
"Area" |

/**
* Values represented by vertical columns. Typically supports stacking to         represent two facets. May support simultaneous stacking and clustering for
*          3 facets.
*/
"Column" |

/**
* Values represented by horizontal bars. Typically supports stacking to         represent two facets. May support simultaneous stacking and clustering for
*          3 facets.
*/
"Bar" |

/**
* Values represented by a lines between data points, or stacked areas for         multiple facets.
*/
"Line" |

/**
* Values represented on a circular background by a line around the center, or         stacked areas for multiple facets
*/
"Radar" |

/**
* Circular chart with wedges representing values. Multiple or stacked pies         for multiple facets.
*/
"Pie" |

/**
* Like a pie chart with a central hole. Multiple or stacked doughnuts for         multiple facets.
*/
"Doughnut" |

/**
* A chart with two continuous numeric axes and up to one discrete facet.
*/
"Scatter" |

/**
* A chart with two continuous numeric axes and up to one discrete facet         that also displays values from a third continuous, numeric domain
*          represented as the sizes of the data point shapes.
*/
"Bubble" |

/**
* Like a column chart, except instead of showing stacked or clustered values         for each position on the horizontal axis, the data values are used together
*          with the associated FacetChart.endValueMetric values to show a series
*          of line segments.
*/
"Histogram";


/**
* when heuristically finding a property that appears to contain child objects,
*  the ChildrenPropertyMode determines how to chose the property appears to contain child objects
*/
type ChildrenPropertyMode = 
/**
* assume the first object or array value we find is the children property
*/
"any" |

/**
* assume the first array we find is the children property, no matter the contents
*/
"array" |

/**
* assume the first object or array of objects we find is the children property     (don't allow arrays that don't have objects)
*/
"object" |

/**
* accept only an array of objects as the children property
*/
"objectArray";


/**
* Passed as a parameter to Canvas.showClickMask to determine the masks behavior
*  when clicked.
*/
type ClickMaskMode = 
/**
* When the mask receives a click, it will fire its click action,          and cancel the event, leaving the clickMask up.
*/
"hard" |

/**
* When the mask receives a click, it will fire its click action,          then dismiss the clickMask and allow the event to proceed to its target.
*/
"soft";


/**
*/
type ColorPickerMode = 
/**
* Display a palette of 40 basic colors from which to pick.
*/
"simple" |

/**
* In addition to the 40 basic colors, the user can specify a color from anywhere in the spectrum, with an optional alpha component.
*/
"complex";


/**
* Supported styles of connector orientations.
*/
type ConnectorOrientation = 
/**
* Tail segments are always horizontal; best for left-to-right connectors
*/
"horizontal" |

/**
* Tail segments are always vertical; best for top-to-bottom connectors
*/
"vertical" |

/**
* Tail segments flip orientation according to longer axis of bounding box: if the        bounding box is wider than it is tall, center segment is vertical
*/
"auto";


/**
* Supported styles of connector styles.
*/
type ConnectorStyle = 
/**
* Center segment is drawn diagonally between tail connector segments
*/
"diagonal" |

/**
* Center segment is drawn perpendicular to tail connector segments
*/
"rightAngle";


/**
* A string used to define the value of a class attribute when it's referenced as one of the
*  potential values of an enum type. For example, one documented value of Overflow is
*  Canvas.HIDDEN, which is itself a class attribute with type Constant and
*  the value "hidden".
*/
type Constant = string;


/**
* Policy controlling how the window will manage content within its body.
*/
type ContentLayoutPolicy = 
/**
* Window does not try to size members at all on either axis. Window body defaults to
*  a Canvas if not autosizing. Otherwise a Layout is used with policies on both axes set
*  to LayoutPolicy "none".
*/
"none" |

/**
* Window body defaults to VLayout behavior. (Body is actually just a Layout with 
*  Layout.vertical: true.)
*/
"vertical" |

/**
* Window body defaults to HLayout behavior. (Body is actually just a Layout with 
*  Layout.vertical: false.)
*/
"horizontal";


/**
* What type of content is found at the HTMLFlow's HTMLFlow.contentsURL?
*/
type ContentsType = 
/**
* the contentsURL is assumed to be a standalone HTML page, and is loaded in an IFRAME.
*/
"page" |

/**
* the default setting - indicates that HTML loaded from the contentsURL is assumed to be an HTML fragment rather than a complete page.
*/
"fragment";


/**
* Names for the standard controls built into the RichTextEditor. You can use these
*  ControlNames in APIs like RichTextEditor.styleControls to control
*  the order in which controls appear, to omit default controls or to show controls that
*  are not shown by default.
*  
*  Every ControlName is also the name of an AutoChild, so all the
*  built-in controls can be skinned or otherwise customized via the
*  autoChildUsage. 
*/
type ControlName = 
/**
* A button to make the current selection bold.
*/
"boldSelection" |

/**
* A button to make the current selection italic.
*/
"italicSelection" |

/**
* A button to make the current selection underlined.
*/
"underlineSelection" |

/**
* A select item allowing the user to change the font of the current text selection.
*/
"fontSelector" |

/**
* A select item allowing the user to change the font size of the current text selection.
*/
"fontSizeSelector" |

/**
* A button to left-align the selected text.
*/
"alignLeft" |

/**
* A button to right-align the selected text.
*/
"alignRight" |

/**
* A button to center the selected text.
*/
"alignCenter" |

/**
* A button to justify the selected line of text.
*/
"justify" |

/**
* A color-picker allowing the user to set the text color.
*/
"color" |

/**
* A color picker allowing the user to set the text background color.
*/
"backgroundColor" |

/**
* Within text, indents the paragraph. Within a list, increases the list level.
*/
"indent" |

/**
* Within text, outdents the paragraph. Within a list, decreases the list level.
*/
"outdent" |

/**
* Turns the current selection into an ordered list (HTML &lt;ol&gt;) or converts an unordered list to an ordered list.
*/
"orderedList" |

/**
* Turns the current selection into an unordered list (HTML &lt;ul&gt;) or converts an ordered list to an unordered list.
*/
"unorderedList" |

/**
* Shows the RichTextEditor.listPropertiesDialog to allow configuring the options of the currently selected HTML list.
*/
"listProperties";


/**
* A number representing a horizontal or vertical offset from the origin of a
*  DrawPane in the DrawPane. DrawRect.left is
*  an example of a DrawItem attribute that's a Coordinate.
* 
*  A Coordinate is not limited to integers except for
*  DrawingType "vml".
*/
type Coordinate = Double;


/**
* Criteria for selecting only a matching set of records from a DataSource. Criteria can
*  be applied on the client and server. Unless configured otherwise, criteria will generally
*  be applied client-side by ResultSets via ResultSet.applyFilter. 
*  
*  Client- and server-side systems built into SmartClient understand two criteria formats by
*  default: simple key-value pairs (Criteria) or the AdvancedCriteria format. 
*  
*  
*  Simple key-value Criteria are represented via a JavaScript Object where each property
*  specifies the name and required value for a field. Multiple legal values for a field can be
*  provided as an Array. For example:
*  
*  var criteria = {
*   field1 : "value1",
*   field2 : ["value2", "value3"]
*  }
*  
*  Would select all records where field1 has value "value1" and where field2 has either
*  "value2" or "value3".
*  
*  
*  Use DataSource.combineCriteria to combine two Criteria objects (including Criteria and
*  AdvancedCriteria) or DataSource.convertCriteria to convert simple Criteria to the
*  AdvancedCriteria format.
*  
*  
*  When writing custom client and server-side filtering logic, criteria must be a JavaScript
*  Object but the properties of that Object can contain whatever data you want. 
*  
*  When sent to the SmartClient server, the Java representation of the criteria is described
*  RPCRequest.data. When sent to other servers, the
*  DSProtocol affects the format.
*/
type Criteria = object;


/**
* The logical operator to use when combining criteria objects with the 
*  DataSource.combineCriteria method.
*/
type CriteriaCombineOperator = 
"and" |

"or";


/**
*/
type CriteriaPolicy = 
/**
* Cache is dropped whenever criteria changes.
*/
"dropOnChange" |

/**
* Cache is retained as long as the only changes to criteria               make the criteria more restrictive as determined by
*                ResultSet.compareCriteria.
*/
"dropOnShortening";


/**
* Possible values for the current pane showing in a SplitPane. See
*  SplitPane.currentPane for details.
*/
type CurrentPane = 
/**
* SplitPane.navigationPane is the most recently shown
*/
"navigation" |

/**
* SplitPane.listPane is the most recently shown
*/
"list" |

/**
* SplitPane.detailPane is the most recently shown
*/
"detail";


/**
* You can use whatever cursors are valid for your deployment platforms, but keep in mind that
*  visual representation may vary by browser and OS. See the
*  MDN cursor page
*  for a live demonstration.
*/
type Cursor = 
/**
* Use the default arrow cursor for this browser/OS.
*/
"default" |

/**
* Use the default cursor for this element type in this                browser/OS
*/
"auto" |

/**
* Use the wait cursor.
*/
"wait" |

/**
* Use the hand cursor.
*/
"hand" |

/**
* Use the "move" (crosshairs) cursor.
*/
"move" |

/**
* Use the 'help' cursor.
*/
"help" |

/**
* Use the 'text' (i-beam) cursor.
*/
"text" |

/**
* Use the normal hand pointer that appears when you hover                over a link
*/
"pointer" |

"arrow" |

"all-scroll" |

/**
* Use the 'crosshair' ( + ) cursor.
*/
"crosshair" |

/**
* Use the column resize cursor (horizontal double-tipped arrow)
*/
"col-resize" |

/**
* Use the row resize cursor (vertical double-tipped arrow)
*/
"row-resize" |

/**
* Use the "east resize" cursor.
*/
"e-resize" |

/**
* Use the "west resize" cursor.
*/
"w-resize" |

/**
* Use the "north resize" cursor.
*/
"n-resize" |

/**
* Use the "south resize" cursor.
*/
"s-resize" |

/**
* Use the "south-east resize" cursor.
*/
"se-resize" |

/**
* Use the "north-east resize" cursor.
*/
"ne-resize" |

/**
* Use the "north-west resize" cursor.
*/
"nw-resize" |

/**
* Use the "south-west resize" cursor.
*/
"sw-resize" |

/**
* Use the "not-allowed" cursor.
*/
"not-allowed";


/**
* A Callback to evaluate when a DataSource request completes.
*  
*  The parameters available in the DSCallback expression are:
*  
*  
* - dsResponse: a DSResponse instance with metadata about the returned data
*  
* - data: data returned to satisfy the DataSource request. See the
*    dataSourceOperations topic for expected results
*    for each type of DataSource operation
*  
* - dsRequest: the DSRequest that was sent. You can use
*    DSRequest.clientContext to track state during the server turnaround.
*  
*  For example, if you had a DynamicForm with ID "myForm" and you wanted to retrieve a 
*  record from a DataSource "myUsers", where each record has a "userId" field:
*  
*    myUsers.fetchData({ userId : "fred" }, "myForm.setValues(data)");
*  
*  or
*  
*    myUsers.fetchData({ userId : "fred" }, function (dsResponse, data, dsRequest) {
*                        myForm.setValues(data);
*                       });
*  
*  
*  Note that if the request encounters a low-level error (such as 500 server error), by
*  default the callback will not be fired, instead, DataSource.handleError
*  is called to invoke the default system-wide error handling. Set
*  RPCRequest.willHandleError:true to have your callback invoked
*  regardless of whether there are errors, however, make sure your callback properly
*  handles malformed responses when DSResponse.status is non-zero.
*/
type DSCallback = object;


/**
* Indicates the format to be used for HTTP requests and responses when
*  fulfilling DSRequests (eg, when DataSource.fetchData is called).
*  
*  Note that RPCManager.startQueue is only available for
*  "iscServer" requests.
*/
type DSDataFormat = 
/**
* Make an HTTP request in a format recognized by the ISC server and expect ISC server
*  response format. serverDataIntegration can then be
*  used to connect to any kind of data store or web service.
*/
"iscServer" |

/**
* Expect XML responses. Request format depends on the setting for
*  OperationBinding.dataProtocol. This is the correct setting when
*  consuming RSS feeds, XML web services (whether SOAP, REST, XML-RPC or custom format),
*  and XML flat files directly from the browser.
*  
*  Values for "date", "time" or "datetime" fields in responses should be specified in the
*  applicable XML Schema
*  date format.
*  If no timezone is explicitly specified, dates / datetimes received by the client are 
*  assumed to be GMT. Note that "date" type fields represent logical dates and may omit
*  time information entirely, and "time" type fields may omit date information.
*  See dateFormatAndStorage for more information
*  on how date values are serialized in requests sent to the server.
*  
*  
*  A DSResponse will be derived from the returned XML via the process described under
*  DataSource.transformResponse.
*/
"xml" |

/**
* Expect response in JSON (Java Script Object Notation)
*  format, ready to be eval()'d. Response should either be a naked object literal:
*  
*    { "property":"value1", "property2" : "value2", ... }
*  
*  or a string that evals to return a valid response object:
*  
*    var response = { "property":"value1", "property2" : "value2", ... }
*    response;
*  
*  
*  A DSResponse will be derived from the returned JSON via the process described under
*  DataSource.transformResponse.
*  
*  As with "xml" responses, values for "date" or "datetime" fields should be
*  specified as a String in 
*  XML Schema date format
*  and may include a timezone. In the absence of a timezone they will be assumed to be GMT.
*  
*  Request format depends on the setting for OperationBinding.dataProtocol. 
*  See also XJSONDataSource.
*/
"json" |

/**
* SmartClient will not attempt to parse the response, instead,
*  DataSource.transformResponse must be implemented.
*  transformResponse will receive the "data" parameter as a String, and must
*  parse this String into an Array of Objects, which should be set as
*  DSResponse.data. Request format depends on the setting for
*  OperationBinding.dataProtocol.
*  
*  Note that, unlike either the "json" or "xml" settings of dataFormat, you
*  are responsible for ensuring that parsed values are the correct type, for example, using
*  the JavaScript built-ins parseInt and parseFloat on integer
*  and decimal values respectively, and using new Date() to construct valid
*  Dates.
*/
"custom";


/**
* For DataSources of type "sql" and "hibernate", specifies the kind of inheritance to use when a dataSource
*  specifies DataSource.inheritsFrom.
*/
type DSInheritanceMode = 
/**
* Inherit fields by copying them onto the inheriting DataSource's underlying table. When we 
*  import a DataSource with this inheritanceMode, we create actual columns for inherited fields
*  on the table we create. With this inheritanceMode, inherited fields are updatable.
*/
"full" |

/**
* Do not physically inherit fields onto the inheriting DataSource's SQL table. Columns will
*  not be created for inherited fields on import, and all generated SQL operations will exclude
*  inherited fields. However, those fields are still part of the DataSource's schema so you
*  can, for example, write customQuerying that returns values for 
*  the inherited fields, and they will be delivered to the client.
*/
"none";


/**
* One of the four basic operations that can be performed on DataSource data: "fetch",
*  "add", "update", "remove". Elsewhere called CRUD operations, where CRUD stands for
*  "create", "retrieve", "update", "delete", which correspond to "add", "fetch", "update" and
*  "remove" in SmartClient terminology. See dataSourceOperations for a full
*  description.
*  
*  There are also additional, non-CRUD operations explained below.
*/
type DSOperationType = 
/**
* Fetch one or more records that match a set of search criteria.
*/
"fetch" |

/**
* Store new records
*/
"add" |

/**
* Update an existing record
*/
"update" |

/**
* Remove (delete) an existing record
*/
"remove" |

/**
* perform some arbitrary custom logic that is not a CRUD operation.            Format of the inputs and outputs is unconstrained, and the operation
*             will be ignored for cache sync purposes by ResultSets. See 
*             DataSource.performCustomOperation.
*/
"custom" |

/**
* Run server-side validation for "add" or "update" without actually            adding or updating anything. See DataSource.validateData.
*/
"validate" |

/**
* Retrieve a file stored in a binary field in a DataSource record, and            allow the browser to choose whether to view it directly or prompt the
*             user to save. See binaryFields.
*/
"viewFile" |

/**
* Like "viewFile", but the HTTP header Content-Disposition is used to            suggest that the browser show a save dialog. See binaryFields.
*/
"downloadFile" |

/**
* Takes a List of Maps and stores the data in Admin Console XML test            data format
*/
"storeTestData" |

/**
* Upload formatted client data and export it to Excel, XML and other            formats. Used automatically by 
*             DataSource.exportClientData 
*             and cannot be used directly. Usable only with the SmartClient server
*             framework.
*/
"clientExport" |

/**
* Use the DataSource as a fileSource.            Used automatically by DataSource.getFile, and
*             would not normally be used directly. Usable only with
*             the SmartClient server framework.
*/
"getFile" |

/**
* Use the DataSource as a fileSource.            Used automatically by DataSource.hasFile, and
*             would not normally be used directly. Usable only with
*             the SmartClient server framework.
*/
"hasFile" |

/**
* Use the DataSource as a fileSource.            Used automatically by DataSource.listFiles, and
*             would not normally be used directly. Usable only with
*             the SmartClient server framework.
*/
"listFiles" |

/**
* Use the DataSource as a fileSource.            Used automatically by DataSource.removeFile, and
*             would not normally be used directly. Usable only with
*             the SmartClient server framework.
*/
"removeFile" |

/**
* Use the DataSource as a fileSource.            Used automatically by DataSource.saveFile, and
*             would not normally be used directly. Usable only with
*             the SmartClient server framework.
*/
"saveFile" |

/**
* Use the DataSource as a fileSource.            Used automatically by DataSource.renameFile, and
*             would not normally be used directly. Usable only with
*             the SmartClient server framework.
*/
"renameFile" |

/**
* Use the DataSource as a fileSource.            Used automatically by DataSource.getFileVersion, and
*             would not normally be used directly. Usable only with
*             the SmartClient server framework.
*/
"getFileVersion" |

/**
* Use the DataSource as a fileSource.            Used automatically by DataSource.hasFileVersion, and
*             would not normally be used directly. Usable only with
*             the SmartClient server framework.
*/
"hasFileVersion" |

/**
* Use the DataSource as a fileSource.            Used automatically by DataSource.listFileVersions, and
*             would not normally be used directly. Usable only with
*             the SmartClient server framework.
*/
"listFileVersions" |

/**
* Use the DataSource as a fileSource.            Used automatically by DataSource.removeFileVersion, and
*             would not normally be used directly. Usable only with
*             the SmartClient server framework.
*/
"removeFileVersion";


/**
* OperationBinding.dataProtocol affects how the data in the DSRequest (DSRequest.data)
*  is sent to the DataSource.dataURL. Listed below are the valid values for
*  OperationBinding.dataProtocol and their behavior.
*  
*  Note that, when using the SmartClient server, data is automatically translated from
*  JavaScript to Java according to the rules described RPCRequest.data;
*  dataProtocol does not apply and is ignored.
*  
*  If you are integrating with a RestDataSource server that requires the more
*  obscure RPCRequest.httpMethods of "PUT", "DELETE" or "HEAD", you can specify these
*  httpMethod settings via OperationBinding.requestProperties. dataProtocol settings
*  that mention "GET" or "POST" are compatible with these additional HTTP methods as well.
*  Typical DataSource.operationBindings for a REST server that uses
*  "PUT" and "DELETE" are as follows:
*  
*   operationBindings:[
*     {operationType:"fetch", dataProtocol:"getParams"},
*     {operationType:"add", dataProtocol:"postParams"},
*     {operationType:"remove", dataProtocol:"getParams", requestProperties:{httpMethod:"DELETE"}},
*     {operationType:"update", dataProtocol:"postParams", requestProperties:{httpMethod:"PUT"}}
*   ],
*  
*/
type DSProtocol = 
/**
* Data is added to the dataURL, with each property in the data becoming an HTTP parameter,
*   eg http://service.com/search?keyword=foo
*/
"getParams" |

/**
* Data is POST'd to the dataURL, with each property becoming an HTTP parameter, 
*   exactly as an HTML form would submit them if it had one input field per property in the
*   data.
*/
"postParams" |

/**
* Data is serialized as a JSON string and POST'd to the dataURL.
*/
"postJSON" |

/**
* Data is serialized as XML via DataSource.xmlSerialize and POST'd as the
*   HTTP request body with contentType "text/xml".
*/
"postXML" |

/**
* Data is serialized as XML via DataSource.xmlSerialize, wrapped in a SOAP
*   envelope, and POST'd as the HTTP request body with contentType "text/xml". Generally
*   only used in connection with a wsdlBinding.
*/
"soap" |

/**
* dsRequest.data is assumed to be a String set up by DataSource.transformRequest
*   and is POST'd as the HTTP request body.
*/
"postMessage" |

/**
* This setting entirely bypasses the SmartClient comm system. Instead of the DataSource sending
*   an HTTP request to a URL, the developer is expected to implement
*   DataSource.transformRequest to perform their own custom logic, and then call
*   DataSource.processResponse to handle the results of this action.
*   This dataProtocol setting can be used to implement access to in-browser
*   resources such as HTML5 "localStorage", native APIs available to applications
*   mobileDevelopment, or to implement the 
*   dsFacade.
*/
"clientCustom";


/**
* Indicates what SmartClient Server will do with a DataSource request if you call
*  dsRequest.execute() in server code.
*  
*  If you use a Java-based persistence layer not provided by SmartClient, such as
*  EJB or your own custom object model, you don't need to set
*  dataSource.serverType and should follow the
*  serverDataIntegration.
*/
type DSServerType = 
/**
* Use SmartClient's sqlDataSource to talk directly
*  to relational databases.
*/
"sql" |

/**
* Use Hibernate, either using a real mapped bean or by automatically generating a 
*  Hibernate configuration based on a SmartClient DataSource file 
*  (dataSourceID.ds.xml). See hibernateIntegration for details.
*/
"hibernate" |

/**
* Use SmartClient's built-in JPA 2.0 connector.
*/
"jpa" |

/**
* Use SmartClient's built-in JPA 1.0 connector.
*/
"jpa1" |

/**
* Requests will be delivered to the server and you are expected to write Java code to
*  create a valid response. Throws an error if the server side method dsRequest.execute()
*  is called. This is appropriate if you intend an entirely custom implementation, and you
*  want an error thrown if there is an attempt to call an operation you have not
*  implemented.
*/
"generic" |

/**
* Requests will be delivered to the server and processed as 
*  fileSource, using directories or other DataSources
*  which you configure via DataSource.projectFileKey or DataSource.projectFileLocations
*/
"projectFile";


/**
* Algorithms for drawing lines between points.
*/
type DataLineType = 
/**
* simple straight lines between points that are adjacent in the dataset
*/
"straight" |

/**
* smooth spline curve between points that are adjacent in the dataset
*/
"smooth";


/**
* String specifying a nested data field structure.
*  
*  NOTE: the dataPath feature is intended to help certain legacy architectures, 
*  such as systems that work in terms of exchanging large messages with several different 
*  entity types in one message, and are incapable of providing separate access to each 
*  entity type. Don't use dataPath if this is not your situation.
*  
*  
*  If you are not forced by legacy issues to use hierarchical data structures, 
*  we recommend:
*  
*  
* - If a subobject exists just to bundle several related fields together, and is 
*    not a separate entity (cannot be separately created or separately referenced by 
*    other objects), "flatten" the fields using features such as 
*    DataSourceField.valueXPath.
*  
* - If a subobject is actually a separate entity, make a DataSource for it and use
*    operations on that DataSource to load and modify it.
*    
*    For example, when representing a sales order with a "deliveryAddress" (consisting of
*    multiple fields), you'd typically want an "orders" dataSource to define the fields
*    for the order as a whole, and an "address" dataSource to define the structure of the
*    deliveryAddress object.
*    
*    It may seem like a good idea to work with a single hierarchical order object, where
*    the "deliveryAddress" attribute is set to a sub-object that matches the structure
*    defined in the "address" dataSource. DataPaths could be used to extract individual
*    address fields for editing in a form alongside other fields from the order, and 
*    edits would be saved via a simple "add" or "update" operation, passing in the 
*    modified nested data object.
*    
*    In fact, this has a number of disadvantages. Since there is no call to the
*    "update" operation on the "address" subobject, the address will be modified without
*    the normal features of a dataSource update. You can't specify 
*    DataSource.requiresAuthentication,
*    dmiOverview logic, DSRequestModifier, 
*    and no logging or DataSource.audit will run.
*    The same "bypassing" problem occurs, in perhaps worse form, if a subobject does not yet 
*    exist and the framework creates it automatically, skipping a DataSource "add" 
*    operation that may have established defaults, not been allowed for the user, etc.
*    
*    Loading and saving nested data objects as a single hierarchical block also offers
*    no advantages in terms of performance or simplicity.
*    The RPCManager.startQueue system makes it extremely easy to load and save multiple types of 
*    entities in a single HTTP request, and takes far less code to implement properly as 
*    multiple DataSource operations, with equivalent or better performance.
*    
*    Loading arrays of related objects (such as all LineItems in an Order) as a 
*    hierarchical object has the further drawback that paging cannot be used for the 
*    list of related objects, and all such objects will not participate in 
*    ResultSet.
*  
*  
*  In short, we do not recommend structuring your data as a hierarchy of nested data objects
*  and using dataPath to navigate these structures unless you are forced to by a legacy system 
*  that doesn't allow separate operations on each entity type.
*  
*  How to use dataPaths
*  
*  Each dataPath string is a slash-delimited set of field identifiers, for example
*  "id1/id2/id3". DataPaths may be applied directly to a
*  Canvas.dataPath, and/or to a databound component field specification.
*  A datapath denotes a path to a nested field value in a hierarchical structure, giving
*  developers the opportunity to easily view or edit nested data structures.
*  Specifically:
*  
* - if the component is viewing or editing a record, the value for fields 
*      will be derived from a nested structure of records
*    
* - if the component is bound to a dataSource, field attributes may be picked up by
*      following the dataPath to a field definition on another dataSource
*  Examples:
*  If a dynamicForm is defined with the following fields:
*  
*   [
*    { name:"name" },
*    { name:"street", dataPath:"address/street" }
*   ]
*  
*  If the "name" field is set to "Joe Smith" and the "street" field
*  is set to "1221 High Street", when the values for this form are retrieved via a
*  getValues() call they will return an object in the following format:
*  
*   {name:"Joe Smith", address:{street:"1221 High Street"}}
*  
*  
*  For databound components, dataPath also provides a way to pick up field attributes from nested
*  dataSources. Given the following dataSource definitions:
*  
*  
*  isc.DataSource.create({
*    ID:"contacts",
*    fields:[
*      {name:"name"},
*      {name:"email"},
*      {name:"organization"},
*      {name:"phone"},
*      {name:"address", type:"Address"}
*    ]
*  });
*  
*  isc.DataSource.create({
*    ID:"Address",
*    fields:[
*      {name:"street"},
*      {name:"city"},
*      {name:"state"},
*      {name:"zip"}
*    ]
*  });
*  
*  
*  
*  and a databound component bound to the 'contacts' dataSource, specifying a field with a dataPath
*  of "address/street" would ensure the field attributes were derived from the 
*  "street" field of the 'Address' dataSource.
*  
*  dataPaths are also cumulative. In other words if a component has a specified dataPath, 
*  the dataPath of any fields it contains will be appended to that component level path when
*  accessing data. For example the following form:
*  
*  
*  isc.DynamicForm.create({
*    dataPath:"contact",
*    fields:[
*      {dataPath:"address/email"}
*    ]
*  });
*  
*  
*  
*  Might be used to edit a data structure similar to this:
*  {contact:{name:'Ed Jones', address:{state:"CA", email:"ed@ed.jones.com"}}}
*  Nested canvases can also have dataPaths specified, which will similarly be combined. See
*  the Canvas.dataPath attribute for more information and examples of this.
*/
type DataPath = string;


/**
* The data URL MIME type to use when DrawPane.getDataURL is called to convert the
*  drawing to a data URL.
*/
type DataURLFormat = 
/**
* Any MIME type supported by the browser is acceptable. Note: The exact MIME type used may depend on the browser, and may change from version to version of SmartClient.
*/
"any" |

/**
* Generate an image/png data URL.
*/
"png";


/**
* Valid display formats for dates. These strings are the names of formatters which can be
*  passed to DateUtil.setNormalDisplayFormat() or 
*  DateUtil.setShortDisplayFormat() and will be subsequently used as default long
*  or short formatters for date objects by SmartClient components.
*  Default set of valid display formats is as follows:
*/
type DateDisplayFormat = 
/**
* Default native browser 'toString()' implementation. May vary by browser.
*  Example: Fri Nov 04 2005 11:03:00 GMT-0800 (Pacific Standard Time)
*/
"toString" |

/**
* Default native browser 'toLocaleString()' implementation. May vary by browser.
*  Example: Friday, November 04, 2005 11:03:00 AM
*/
"toLocaleString" |

/**
* Short date in format MM/DD/YYYY. Example: 11/4/2005
*/
"toUSShortDate" |

/**
* Short date with time in format MM/DD/YYYY HH:MM Example: 11/4/2005 11:03
*/
"toUSShortDatetime" |

/**
* Short date in format DD/MM/YYYY. Example: 4/11/2005
*/
"toEuropeanShortDate" |

/**
* Short date with time in format DD/MM/YYYY HH:MM Example: 4/11/2005 11:03
*/
"toEuropeanShortDatetime" |

/**
* Short date in format YYYY/MM/DD. Example: 2005/11/4
*/
"toJapanShortDate" |

/**
* Short date with time in format YYYY/MM/DD HH:MM Example: 2005/11/4 11:03
*/
"toJapanShortDatetime" |

/**
* Date in the format YYYY-MM-DD HH:MM:SS Example: 2005-11-04 11:09:15
*/
"toSerializeableDate" |

/**
* Date in the format &lt;YYYYMMDD&gt;T&lt;HHMMSS&gt;Z Example: 20051104T111001Z
*  
*  
*  Note: In addition to these standard formats, custom formatting can be set by passing
*  a function directly to DateUtil.setNormalDisplayFormat et al. This
*  function will then be executed whenever the appropriate formatter method is called [eg
*  Date.toNormalDate], in the scope of the date instance in question.
*  
*  Custom formatting can also be applied by passing a FormatString instead of a 
*  DateDisplayFormat string to DateUtil.setNormalDisplayFormat et al. See
*  the FormatString docs for details.
*/
"toDateStamp";


/**
* The type of date/time editing style to use when editing an event.
*/
type DateEditingStyle = 
/**
* allows editing of the logical start and end dates of the event
*/
"date" |

/**
* allows editing of both date and time
*/
"datetime" |

/**
* allows editing of the time portion of the event only
*/
"time";


/**
* The direction in which an item should lay out it's fields.
*/
type DateFieldLayout = 
/**
* fields are placed horizontally
*/
"horizontal" |

/**
* fields are placed vertically
*/
"vertical";


/**
* 3 character string containing the "M", "D" and "Y"
*  characters to indicate the format of strings being parsed into Date instances via
*  DateUtil.parseInput().
*  
*  As an example - an input format of "MDY" would parse "01/02/1999" to Jan 2nd 1999
*  
*  
*  Note: In addition to these standard formats, a custom date string parser function may be
*  passed directly to DateUtil.setInputFormat or passed into 
*  DateUtil.parseInput as the inputFormat parameter.
*  
*/
type DateInputFormat = string;


/**
* Order of pickers and which pickers are present when using a DateItem with
*  DateItem.useTextField false.
*/
type DateItemSelectorFormat = 
/**
* Output fields in day, month, year order.
*/
"DMY" |

/**
* Output fields in month, day, year order.
*/
"MDY" |

/**
* Output fields in year, month, day order.
*/
"YMD" |

/**
* Output only day, month fields.
*/
"DM" |

/**
* Output only month, day fields.
*/
"MD" |

/**
* Output only year, month fields.
*/
"YM" |

/**
* Output only month, year fields.
*/
"MY";


/**
* The Velocity variable names of the "pieces" of SQL that SmartClient generates to form a
*  complete fetch or update query. You can use these variables in you own custom queries 
*  and query clause overrides to build on the SmartClient functionality. See 
*  customQuerying for a full discussion.
*/
type DefaultQueryClause = 
/**
* The column names to select, for a fetch operation only
*/
"$defaultSelectClause" |

/**
* The table name(s) to select from or update
*/
"$defaultTableClause" |

/**
* The DataSource.useAnsiJoins to join                tables to select from, if enabled.
*/
"$defaultAnsiJoinClause" |

/**
* The "where" condition, which will be derived from supplied                 criteria or a primary key value, depending on the type of 
*                 operation
*/
"$defaultWhereClause" |

/**
* For a fetch operation when using the                serverSummaries feature,
*                 "group by" part of aggregated query
*/
"$defaultGroupClause" |

/**
* For a fetch operation when using the                serverSummaries feature,
*                 "having" part of aggregated query (or outer "where" part if
*                 sub-select approach is used, see
*                 OperationBinding.useHavingClause for more details)
*/
"$defaultGroupWhereClause" |

/**
* The column names to update and the update values, for an                update or add operation
*/
"$defaultValuesClause" |

/**
* The column names to sort by, for a fetch operation only
*/
"$defaultOrderClause";


/**
* Some interfaces, for example the FieldPicker and 
*  HiliteEditor widgets, can use data from an associated 
*  DataBoundComponent to express live sample values at runtime.
*/
type DefaultSampleRecord = 
/**
* Uses the first record in the DataBoundComponent as sample data
*/
"first" |

/**
* Uses the first open leaf-node in the DataBoundComponent as sample data
*/
"firstOpenLeaf";


/**
* An object containing the stored grouping information for a detailViewer.
*  Note that this object is not intended to be interrogated directly, but may be stored 
*  (for example) as a blob on the server for state persistence across sessions.
*/
type DetailViewerViewState = string;


/**
* Possible layout modes for UI components that are sensitive to the device type being used
*  (a.k.a. "responsive design"). See for example SplitPane.deviceMode.
*/
type DeviceMode = 
/**
* mode intended for handset-size devices (phones). Generally only one UI         panel will be shown at a time.
*/
"handset" |

/**
* mode intended for tablet-size devices. Generally, up to two panels are         shown side by side in "landscape" PageOrientation, and only one
*          panel is shown in "portrait" orientation.
*/
"tablet" |

/**
* mode intended for desktop browsers. Three or more panels may be shown         simultaneously.
*/
"desktop";


/**
* Default buttons that you can use in your Dialogs.
*  
*  
*  
*  Refer to these buttons via the syntax isc.Dialog.OK when passing them into
*  Dialog.buttons or into the properties argument of helper
*  methods such as isc.say.
*  
*  
*  All buttons added via setButtons will fire the
*  Dialog.buttonClick (whether they are built-in or custom
*  buttons). Built-in buttons automatically close a Dialog, with the exception of the
*  "Apply" button.
*/
type DialogButtons = 
/**
* Dismisses dialog by calling Dialog.okClick.       Title derived from Dialog.OK_BUTTON_TITLE.
*/
"OK" |

/**
* Does not dismiss dialog.      Calls Dialog.applyClick
*        Title derived from Dialog.APPLY_BUTTON_TITLE.
*/
"APPLY" |

/**
* Dismisses dialog by calling Dialog.yesClick.       Title derived from Dialog.YES_BUTTON_TITLE.
*/
"YES" |

/**
* Dismisses dialog by calling Dialog.noClick.       Title derived from Dialog.NO_BUTTON_TITLE.
*/
"NO" |

/**
* Dismisses dialog by calling Dialog.cancelClick.         Title derived from Dialog.CANCEL_BUTTON_TITLE.
*/
"CANCEL" |

/**
* Dismisses dialog by calling Dialog.doneClick.          Title derived from Dialog.DONE_BUTTON_TITLE.
*/
"DONE";


/**
*/
type Direction = 
/**
* above
*/
"up" |

/**
* below
*/
"down" |

/**
* to the left of
*/
"left" |

/**
* to the right of
*/
"right";


/**
* Flag passed to functions as displayNodeType, telling the function whether it should work on
*  folders, leaves or both at once.
*/
type DisplayNodeType = 
/**
* operate on both folders and leaves
*/
"null/unset" |

/**
* operate on folders only, ignoring leaves
*/
"folders" |

/**
* operate on leaves only, ignoring folders
*/
"leaves";


/**
* A number representing the width, height, or radius of a DrawItem in the
*  DrawPane. DrawRect.width is an example of a DrawItem attribute
*  that's a Distance.
* 
*  A Distance is not limited to integers except for
*  DrawingType "vml".
*/
type Distance = Double;


/**
* A decimal (or "floating point") number, for example, 5.5. Null is allowed.
*/
type Double = number;


/**
* Different types of effects for showing dragging behavior.
*/
type DragAppearance = 
/**
* No default drag appearance is indicated. Your custom dragging routines should
*    implement some behavior that indicates that the user is in a dragging situation,
*    and where the mouse is.
*/
"none" |

/**
* A "drag tracker" object is automatically shown and moved around with the
*    mouse. This is generally set to an icon that represents what is being dragged. The
*    default tracker is a 10 pixel black square, but you can customize this icon. This
*    dragAppearance is not recommended for use with drag resizing or drag moving.
*/
"tracker" |

/**
* The target object is actually moved, resized, etc. in real time. This is
*    recommended for drag repositioning, but not for drag resizing of complex objects.
*/
"target" |

/**
* An outline the size of the target object is moved, resized, etc. with the
*    mouse. This is recommended for drag resizing, especially for objects that take a
*    significant amount of time to draw.
*/
"outline";


/**
* What do we do with data that's been dropped into another list?
*/
type DragDataAction = 
/**
* Don't do anything, resulting in the same data being in both lists.
*/
"none" |

/**
* Copy the data leaving the original in our list.
*/
"copy" |

/**
* Remove the data from this list so it can be moved into the other list.
*/
"move";


/**
* Different styles of determining intersection: with mouse or entire rect of target
*/
type DragIntersectStyle = 
/**
* Look for drop targets that are under the current mouse cursor position.
*/
"mouse" |

/**
* Look for drop targets by intersection of the entire rect of the drag        target with the droppable target.
*/
"rect";


/**
* What kind of mask to use for masking dragged element.
*/
type DragMaskType = 
/**
* creates an element with ordinary HTML content that will block events
*/
"div" |

/**
* creates an iframe with empty content
*/
"iframe" |

/**
* hides the contents of this widget temporarily
*/
"hide" |

/**
* no mask
*/
"none";


/**
* When records are being dragged from within a ListGrid, what sort of drag-tracker
*  should be displayed?
*/
type DragTrackerMode = 
/**
* Don't display a drag tracker at all
*/
"none" |

/**
* Display an icon to represent the record(s) being dragged. Icon src is       derived from ListGrid.getDragTrackerIcon
*/
"icon" |

/**
* Display a title for the record being dragged. Title derived from       ListGrid.getDragTrackerTitle
*/
"title" |

/**
* Display the entire record being dragged
*/
"record";


/**
*/
type DrawPosition = 
/**
* insert before the target element
*/
"beforeBegin" |

/**
* insert as the target element's first child
*/
"afterBegin" |

/**
* insert as the target element's last child
*/
"beforeEnd" |

/**
* insert after the target element
*/
"afterEnd" |

/**
* replace the target element
*/
"replace";


/**
*/
type DrawShapeCommandType = 
/**
* Draws a straight line from the current point to the last "moveto" point. There are no arguments.
*/
"close" |

/**
* Start a new sub-path at a given (x,y) coordinate. The args array for this command type is a two-element array of the X and Y coordinates.
*/
"moveto" |

/**
* Draw a line from the current point to the given (x,y) coordinate which becomes the new current point. Multiple (x,y) coordinates may be specified to draw a path, in which
*  case the last point becomes the new current point. The args array for this command type is
*  an array of one or more Points (two-element arrays of the X and Y coordinates).
*/
"lineto" |

/**
* Draw a segment of a specified circle. A straight line (the "initial line segment") is drawn from the current point to the start of the circular arc. The args array
*  for this command type contains 4 values:
*  
*  
* - The center (cx,cy) Point (two-element array) of the circle.
*  
* - radius
*  
* - startAngle - Start angle in degrees
*  
* - endAngle - End angle in degrees
*  
*  Note that the ${isc.DocUtils.linkForExampleId('circletoCommand', '"circleto" Command example')} can be very helpful
*  when learning how to write "circleto" commands.
*/
"circleto";


/**
* The type of drawing back-end used by a DrawPane to draw its DrawPane.drawItems
*  on screen.
* 
*  SVG and bitmap tradeoffs
*  
*  
* - SVG is best supported in IE9+ and Opera 12.16 and earlier, but may also be used in Chrome,
*  Opera 15+, Firefox 4+, and Safari.
*  
* - SVG tends to be higher quality, especially when it comes to text and drawing on high-DPI
*  displays and mobile devices.
*  
* - DrawItem.zIndex, DrawItem.bringToFront, and DrawItem.sendToBack
*  are not supported on iOS devices in bitmap drawing mode due to platform limitations. These
*  APIs are supported on iOS in SVG drawing mode.
*  
* - Incremental updates to just a few elements of a drawing are faster in SVG, whereas
*  whole drawing refreshes are faster in bitmap mode.
*  
* - Browsers have limits on the maximum dimensions or area of the &lt;canvas&gt; element
*  used in bitmap drawing. The limits on a &lt;canvas&gt; imposed by the browser translate to
*  the same limits on the dimensions or area of a DrawPane using bitmap drawing.
*  Internet Explorer, for example, limits the width and height of a &lt;canvas&gt; to 8192 pixels:
*  http://msdn.microsoft.com/en-us/library/ie/ff975062(v=vs.85).aspx; therefore,
*  in IE using bitmap drawing, a DrawPane can be at most 8192&times;8192 in size.
*  To make larger drawings, you can either switch to SVG drawing, use multiple DrawPanes,
*  or enable DrawPane.canDragScroll.
*  
* - In bitmap drawing, each pixel uses around 4 to 8 bytes of memory. Large bitmap drawings
*  can therefore use a lot of memory. A 4000&times;2000 bitmap drawing will use around 31 to 61
*  Megabytes of memory.
*  Note: To minimize memory use when using bitmap drawing for a large drawing, it may be useful
*  to employ slicing if
*  possible. For example, if the drawing is mostly a solid color except for content located at
*  a few small-area places on screen, a Canvas can be created with the solid background
*  color and one DrawPane can be created for each content area. Each DrawPane
*  is Canvas.addChild to the Canvas. The Canvas.left
*  and Canvas.top properties of the DrawPanes are used to absolutely-position
*  them within the Canvas parent.
*  
*/
type DrawingType = 
/**
* Use Scalable Vector Graphics (SVG). SVG is a W3C standard supported by IE9+, Chrome, Firefox 4+, Safari, and Opera.
*/
"svg" |

/**
* Use an HTML5 &lt;canvas&gt; element. "bitmap" drawing is supported by IE9+, Chrome, Firefox, Safari, and Opera.
*/
"bitmap" |

/**
* Use Vector Markup Language (VML). VML is a deprecated vector graphics technology supported only by Internet Explorer 6 through 9. In IE 6, 7, and 8, "vml" drawing is the only
*  supported drawing back-end.
*/
"vml";


/**
* An edge or corner of a rectange, or it's center. Used in APIs such as
*  Canvas.resizeFrom and Canvas.getEventEdge.
*/
type EdgeName = 
/**
* top edge
*/
"T" |

/**
* bottom edge
*/
"B" |

/**
* left edge
*/
"L" |

/**
* right edge
*/
"R" |

/**
* top-left corner
*/
"TL" |

/**
* top-right corner
*/
"TR" |

/**
* bottom-left corner
*/
"BL" |

/**
* bottom-right corner
*/
"BR" |

/**
* center
*/
"C";


/**
* Object used to specify custom edge sizes or offsets.
*  Specified as an object where defaultSize will map to the default edge size or 
*  offset for the canvas (Canvas.edgeSize, or Canvas.edgeOffset and
*  top, left, right and
*  bottom will map to the
*  EdgedCanvas.edgeTop/EdgedCanvas.edgeOffsetTop, 
*  EdgedCanvas.edgeLeft/EdgedCanvas.edgeOffsetLeft,
*  EdgedCanvas.edgeRight/EdgedCanvas.edgeOffsetRight,
*  and EdgedCanvas.edgeBottom/EdgedCanvas.edgeOffsetBottom
*  attributes on the paneContainer respectively. Note that not all these properties have to be
*  set - if unset standard edge sizing rules will apply.
*/
type EdgeSizes = object;


/**
* What event / user interaction type caused cell editing to complete.
*/
type EditCompletionEvent = 
/**
* User clicked outside editor during edit.
*/
"click_outside" |

/**
* User started editing another row by clicking on it
*/
"click" |

/**
* User started editing another row by double                clicking
*/
"doubleClick" |

/**
* Enter pressed.
*/
"enter" |

/**
* User pressed Escape.
*/
"escape" |

/**
* Up arrow key pressed.
*/
"arrow_up" |

/**
* down arrow key.
*/
"arrow_down" |

/**
* User pressed Tab.
*/
"tab" |

/**
* User pressed Shift+Tab.
*/
"shift_tab" |

/**
* Edit moved to a different field (same row)
*/
"field_change" |

/**
* Edit completed via explicit function call
*/
"programmatic";


/**
* How should AutoTest.waitForElement determine
*  that the application is ready to retrieve the element?
*  
*  Note: In most cases "system" is the appropriate setting. This will wait for standard
*  SmartClient-initiated asychronous actions to complete, including timer-instantiated
*  actions such as Canvas.markForRedraw or actions 
*  configured through the Timer, and oustanding 
*  RPCManager. If the locator cannot be resolved to a clickable
*  element after system quiescence, this usually implies that it will not ever resolve,
*  and there's no need to wait for the timeout to complete and slow down your test suite.
*  
*  The only time you'd want to use "element"
*  would be if your application relies on some asynchronous event that isn't tracked by
*  the SmartClient system. Examples might include waiting for asynchronous actions instantiated
*  by a third-party tool on the page, or waiting for a server notification to come in through
*  the RealtimeMessaging system.
*/
type ElementWaitStyle = 
/**
* use AutoTest.waitForSystemDone to wait     for actions to complete,
*      then return the result of AutoTest.getElement, even if
*      this method doesn't resolve to an element.
*/
"system" |

/**
* wait until AutoTest.getElement resolves     to an element, and the element is AutoTest.isElementClickable.
*/
"element";


/**
* How a component should be embedded within its record or cell
*/
type EmbeddedPosition = 
/**
* component should be placed underneath normal record or cell content,    expanding the records. Expanding records can result in variable height rows,
*     in which case ListGrid.virtualScrolling should be
*     enabled.
*/
"expand" |

/**
* component should be placed within the normal area of the record or cell.    Percentage sizes will be treated as percentages of the record and
*     Canvas.snapTo positioning settings are also allowed and refer to the
*     rectangle of the record or cell. Note that for components embedded within cells,
*     cell align and vAlign will be used if snapTo is unset (so top / left alignment
*     of cell content will map to snapTo of "TL", etc).
*/
"within";


/**
* Form encoding types - these translate to Form ENCTYPE parameters.
*/
type Encoding = 
/**
* normal form encoding ("application/x-www-form-urlencoded")
*/
"normal" |

/**
* form encoding for forms with INPUT file elements, that                  is, forms that upload files ("multipart/form-data")
*/
"multipart";


/**
* What to do when a user hits enter while editing a cell
*/
type EnterKeyEditAction = 
/**
* end editing (will save edit values if ListGrid.autoSaveEdits is true).
*/
"done" |

/**
* edit the next editable cell in the record
*/
"nextCell" |

/**
* edit the same field in the next editable record
*/
"nextRow" |

/**
* edit the first editable cell in next editable record
*/
"nextRowStart";


/**
* Determines how Java enums are translated to and from Javascript by the SmartClient server.
*/
type EnumTranslateStrategy = 
/**
* Translates to/from a String matching the constant name. This is the default if not set.
*/
"name" |

/**
* Translates to/from a String matching the enum.toString().
*/
"string" |

/**
* Translates to/from an integer matching the ordinal number of the constant within 
*   the enumeration
*/
"ordinal" |

/**
* Translates to/from a Javascript object containing one property for each property defined 
*   within the enum. The constant itself and the ordinal number are included in the JS object.
*   By default they are called "_constant" and "_ordinal", but this can be overridden with 
*   the DataSource.enumOrdinalProperty and DataSource.enumConstantProperty
*   properties
*/
"bean";


/**
* What to do if the user hits escape while editing a cell.
*/
type EscapeKeyEditAction = 
/**
* cancels the current edit and discards edit values
*/
"cancel" |

/**
* end editing (will save edit values if ListGrid.autoSaveEdits is true).
*/
"done" |

/**
* exit the editor (edit values will be left intact but not saved).
*/
"exit" |

/**
* do nothing special when the Escape key is pressed (ie, just ignore it)
*/
"ignore";


/**
* Mode for escaping text values when using DataSource.recordsAsText or
*  DataSource.recordsFromText.
*/
type EscapingMode = 
/**
* Literal double quotes in data values are doubled (""), as expected by Microsoft         Excel when pasting text values
*/
"double" |

/**
* double quotes in data values have a blackslash (\) prepended, similar to          String escaping in JavaScript and Java
*/
"backslash";


/**
* The method of pooling to employ for ListGrid.canExpandRecords.
*  
*/
type ExpansionComponentPoolingMode = 
/**
* auto-created, built-in components are destroyed when record are    ListGrid.collapseRecord.
*/
"destroy" |

/**
* all expansion components are deparented from the grid when a record is    ListGrid.collapseRecord but are not destroyed. It is the responsibility
*     of the developer to handle component destruction
*/
"none";


/**
* When ListGrid.canExpandRecords is true, ExpansionMode
*  dictates the type of UI to be displayed in the expanded portion of the row.
*  
*  There are a number of builtin ExpansionModes and you can override
*  ListGrid.getExpansionComponent to create your own
*  expansion behaviors.
*/
type ExpansionMode = 
/**
* Show a single field's value in an HTMLFlow. Field   to use is ListGrid.detailField.
*/
"detailField" |

/**
* Show a DetailViewer displaying those fields from the   record which are not already displayed in the grid.
*/
"details" |

/**
* Show a separate ListGrid containing related-records.   See ListGridRecord.detailDS and ListGrid.recordDetailDSProperty for
*    more information.
*/
"related" |

/**
* Show a DynamicForm to edit those fields from the   record which are not already present in the grid. If the record is collapsed with
*    unsaved changes and ListGrid.expansionEditorShowSaveDialog is not set, Edits
*    will be saved automatically, or stored as editing if
*    ListGrid.autoSaveEdits is false. Otherwise, a confirmation dialog is
*    displayed. Can optionally show a
*    ListGrid.showExpansionEditorSaveButton and
*    ListGrid.expansionEditorCollapseOnSave when save is pressed.
*    If a record fails validation on save and the field in question is not visible in
*    the grid, the record is automatically expanded and validated to show the errors.
*/
"editor" |

/**
* Show a DetailViewer displaying those fields   from the record not already displayed in the grid, together with a separate
*    ListGrid containing related-records.
*/
"detailRelated";


/**
* Method to use for displaying the exported data.
*/
type ExportDisplay = 
/**
* Show the Save As dialog and download the file
*/
"download" |

/**
* Show the data in a new browser window
*/
"window" |

/**
* Return the data for further programmatic processing in the browser
*/
"return";


/**
* One of the supported formats for data-export. If you are doing a 
*  ListGrid.exportClientData to one of the native spreadsheet
*  formats (xls or ooxml), we also export Hilite coloring. So, if
*  Hilites are causing a particular cell to be rendered as green text on a blue background, 
*  the corresponding cell in the exported spreadsheet document will also be colored that way.
*/
type ExportFormat = 
/**
* Export data as XML records
*/
"xml" |

/**
* Export data as JSON objects. Not allowed as a client-side option.
*/
"json" |

/**
* Export data in comma-separated format
*/
"csv" |

/**
* Export data in native Microsoft Excel 97 format
*/
"xls" |

/**
* Export data in native Microsoft Excel 2007 format (also called XLSX)
*/
"ooxml" |

/**
* Custom server-side logic will do the export
*/
"custom";


/**
* One of the supported formats for image export.
*/
type ExportImageFormat = 
/**
* Export as PNG
*/
"png" |

/**
* Export as JPEG
*/
"jpeg";


/**
* Specifies the direction of facet indenting for a Facet.isTree facet.
*/
type FacetIndentDirection = 
/**
* means the top of the hierarchy renders highest (column                facet) or leftmost (row facet). For example, the text for
*                 the top of a column facet hierarchy will be flush to the
*                 top of the header with the hierarchy expanded.
*/
"descending" |

/**
* means that the bottom of the hierarchy renders highest                (column facet) or leftmost (row facet).
*/
"ascending";


/**
* Mode of fetching records from the server.
*  
*  Generally, "paged" mode should be used unless the maximum number of records is
*  guaranteed to be small.
*/
type FetchMode = 
/**
* All records that match the current filter are fetched. Sorting is        performed on the client.
*/
"basic" |

/**
* Only requested ranges of records are fetched, with predictive fetch        ahead. Sorting is performed on the server.
*/
"paged" |

/**
* All records available from the DataSource are fetched. Filtering by        search criteria and sorting are both performed on the client.
*/
"local";


/**
*/
type FieldAppearance = 
/**
* Show in read-only appearance
*/
"readOnly" |

/**
* Hide field
*/
"hidden" |

/**
* Disable field
*/
"disabled";


/**
*/
type FieldAuditMode = 
/**
* the field value will be saved for all audited adds and updates, regardless of         whether it's changed or present in the update
*/
"always" |

/**
* the field value will be saved for all audited adds, and for updates in which         the value changed
*/
"change" |

/**
* the field value will never be saved for any DataSource audit operation
*/
"never";


/**
* Indicates where a given field can be legally filtered. By default, any field where
*  DataSourceField.canFilter isn't false allows filtering either at the 
*  server or at the ResultSet.useClientFiltering, according to related settings.
*  
*  This is the recommended behavior, but it can be useful or necessary to limit filtering 
*  in some cases.
*/
type FieldFilterMode = 
/**
* - filter on the client where possible and contact the server when necessary
*/
"both" |

/**
* - any change to criteria for a field causes cache invalidation and a  trip to the server. This mode is quite useful for cases where client filtering can't 
*  replicate server filtering, such as:
*  
*   
* - search engines that consider something a match based on word roots (like treating 
*    "sunken" as a match for "sink")
*   
* - server-side formatting, where values are delivered as HTML or even as images, 
*    where the value seen in the browser no longer matches the search text
*   
* - searching large files where the files won't be delivered to the client
*  
*  By using this mode only for fields that are special, you can preserve the performance 
*  benefits of client-side filtering on other fields.
*/
"serverOnly";


/**
* Options for how values in the import dataset (for example, the CSV file) are transformed
*  during import if the field is a DataSourceField.foreignKey mapped to a 
*  DataSourceField.displayField
*/
type FieldImportStrategy = 
/**
* The import process expects values in the import dataset to be the real     underlying key values, and performs no transformation
*/
"key" |

/**
* The import process expects values in the import dataset to be display     values, and it will transform them to the corresponding underlying keys if found
*/
"display" |

/**
* Same as "display", but results in    DataSourceField.importStrategyFailedErrorMessage if
*     corresponding underlying keys were not found
*/
"displayRequired" |

/**
* The import process will attempt to discover the best setting to use, based    on the values in the first record, and use that setting for every remaining record
*     in the import dataset
*/
"auto";


/**
* Name for a field.
*  
*  Must be unique across all fields within its container as well as a valid JavaScript
*  identifier, as specified by ECMA-262 Section 7.6.
*  
*  NOTE: The String.isValidID
*  
*  function can be used to test whether a name is a valid JavaScript identifier.
*/
type FieldName = string;


/**
* The strategy to use when generating field names - for example, for new formula or summary 
*  fields created using the built-in editors.
*/
type FieldNamingStrategy = 
/**
* generate names in the format fieldTypeX, where field type might be     "formulaField" and X is an index specific to the field-type and component instance
*/
"simple" |

/**
* generates a UUID for all generated field names
*/
"uuid";


/**
* An object containing the stored presentation information for the fields of some dataBoundComponent.
*  Information contained in a FieldState object includes the
*  visibility and order of the component's fields.
*  Note that this object is a JavaScript string, and may be stored (for example) as a blob
*  on the server for state persistence across sessions.
*/
type FieldState = string;


/**
* The types listed below are built-in types that DataBoundComponent understand and treat specially (using type-specific form controls, validators,
*  formatters, sorting logic, etc). 
*  
*  You can declare custom types via SimpleType, with settings that
*  will influence DataBound components. You can also create your own subclasses of databound
*  components to add further custom, reusable behaviors based on field.type.
*  
*  field.type can also be the ID of another DataSource, which allows you to
*  model nested structures such as XML documents (in fact, XMLTools.loadXMLSchema
*  models XML schema in this way). Nested DataSource declarations affect how XML and JSON data
*  is deserialized into JavaScript objects in the 
*  clientDataIntegration pipeline, so that you can load
*  complex XML documents and have them deserialized into a correctly typed nested data
*  structure.
*  
*  Note: to declare related but separate objects, as in an "Account" object that can be
*  related to both a "Contact" object and "Order" objects, use
*  DataSourceField.foreignKey, not a nested structure declaration.
*/
type FieldType = 
/**
* Generic text, e.g. "John Doe". This is the default field type.          Use field.length to set length.
*/
"text" |

/**
* A boolean value, e.g. true
*/
"boolean" |

/**
* A whole number, e.g. 123
*/
"integer" |

/**
* A floating point (decimal) number, e.g. 1.23
*/
"float" |

/**
* A logical date, with no time value (such as a holiday or birthday).          Represented on the client as a JavaScript Date object where
*           time values are ignored. See dateFormatAndStorage for more
*           information on date display and serialization formats.
*/
"date" |

/**
* A time of day, with no date. Represented on the client as a JavaScript          Date object in UTC/GMT by default (see also
*           dateFormatAndStorage and the Time class).
*/
"time" |

/**
* A date and time, accurate to the DataSourceField.storeMilliseconds. Represented on the client as a JavaScript Date
*           object. See also dateFormatAndStorage.
*/
"datetime" |

/**
* A text value constrained to a set of legal values specified by the          field's DataSourceField.valueMap, as though a
*           ValidatorType of "isOneOf" had been declared.
*/
"enum" |

/**
* An enum whose values are numeric.
*/
"intEnum" |

/**
* If you are using the SmartClient SQL datasource connector, a          sequence is a unique, increasing whole number, incremented
*           whenever a new record is added. Otherwise, sequence behaves
*           identically to integer. This type is typically used with
*           field.primaryKey to auto-generate unique primary keys.
*           See also DataSourceField.sequenceName and 
*           DataSource.sequenceMode
*/
"sequence" |

/**
* A string representing a well-formed URL. Some components          will render this as an HTML link (using an anchor tag for example).
*/
"link" |

/**
* A string representing a well-formed URL that points to an          image. Some components will render an IMG tag with the value of this field
*           as the 'src' attribute to render the image.
*/
"image" |

/**
* Arbitrary binary data. When this field type is present, three additional fields are automatically generated. They are: &lt;fieldName&gt;_filename, &lt;fieldName&gt;_filesize, and
*  &lt;fieldName&gt;_date_created where &lt;fieldName&gt; is the value of the name
*  attribute of this field. These fields are marked as
*  DataSourceField.hidden:true to suppress their rendering by default. You
*  can show one or more of these fields by specifying the field with a hidden:false
*  override in the fields array of the databound component.
*  Stream / view file support for custom DataSources: a custom DataSource or DMI 
*  must implement the "viewFile" and "downloadFile" operationTypes and return a single Record
*  with a byte[] as the field value for the binary field. For more detail see
*  the overview of binaryFields.
*/
"binary" |

/**
* Binary data comprising an image. Causes ViewFileItem to be used          when the field is displayed in a form, allowing the image to optionally
*           be displayed inline.
*/
"imageFile" |

/**
* Fields of this type can contain any data value and have no default           formatting or validation behavior. This is useful as the 
*           SimpleType.inheritsFrom for SimpleTypes
*           where you do not want any of the standard validation or formatting logic
*           to be inherited from the standard built-in types.
*/
"any" |

/**
* Synonymous with "any".
*/
"custom" |

/**
* Fields of this type are automatically populated by the SmartClient Server           with the current authenticated userId as part of "add" and "update" 
*           operations. By default, fields of this type are hidden and not editable;
*           the server ignores any value that the client sends in a field of this type.
*           Note that the "authenticated user" can be set explicitly on the server-side 
*           RPCManager using the setUserId() method, or it can come from
*           the servlet API if you are using its built-in authentication scheme. See
*           the server-side Javadocs for RPCManager.
*/
"modifier" |

/**
* Fields of this type are automatically populated by the SmartClient          Server with the current date and time as part of "add" and "update" 
*           operations. By default, fields of this type are hidden and not editable;
*           the server ignores any value that the client sends in a field of this type.
*/
"modifierTimestamp" |

/**
* Fields of this type are automatically populated by the SmartClient Server           with the current authenticated userId as part of "add" operations.
*           By default, fields of this type are hidden and not editable;
*           the server ignores any value that the client sends in a field of this type.
*           The notes about type "modifier" also apply to fields of this type.
*/
"creator" |

/**
* Fields of this type are automatically populated by the SmartClient          Server with the current date and time as part of an "add" operation (when
*           the record is first created). By default, fields of this type are hidden
*           and not editable; the server ignores any value that the client sends in a
*           field of this type.
*/
"creatorTimestamp" |

/**
* Same as "text", but causes PasswordItem to be used by default for          editing (hides typed-in value).
*/
"password" |

/**
* A special field type specifically for use with Unicode data in conjunction          with the Microsoft SQL Server database. Field type "ntext" implies the 
*           use of DataSourceField.sqlStorageStrategy "ntext";
*           other than that, this type is identical to "text"
*/
"ntext" |

/**
* An integer number with locale-based formatting, e.g. 12,345,678.          See localizedNumberFormatting
*           for more info.
*/
"localeInt" |

/**
* A float number with locale-based formatting, e.g. 12,345.67.          See localizedNumberFormatting
*           for more info.
*/
"localeFloat" |

/**
* A float number with locale-based formatting and using currency          symbol, e.g. $12,345.67.
*           See localizedNumberFormatting
*           for more info.
*/
"localeCurrency" |

/**
* A telephone number. Uses FormItem.browserInputType "tel" to          hint to the device to restrict input. On most mobile devices that have
*           software keyboards, this cause a specialized keyboard to appear which
*           only allows entry of normal phone numbers. When displayed read-only,
*           a "phoneNumber" renders as an HTML link with the "tel:" URL scheme,
*           which will invoke the native phone dialing interface on most mobile
*           devices. In addition, the CSS style "sc_phoneNumber" is applied.
*           
*           By default, "phoneNumber" fields do not include validators, however the
*           following validator definition would limit to digits, dashes and the
*           "+" character:
*           xml:
*           
*           &lt;validator type="regexp" expression="^(\(?\+?[0-9]*\)?)?[0-9_\- \(\)]*$"
*             errorMessage="Phone number should be in the correct format e.g. +#(###)###-##-##" /&gt;
*           
*             
*             or directly in JavaScript:
*             
*  
*  {type:"regexp", expression:"^(\\(?\\+?[0-9]*\\)?)?[0-9_\\- \\(\\)]*$", 
*    errorMessage:"Phone number should be in the correct format e.g. +#(###)###-##-##"}
*  
*           
*           
*           and adding "#" and "*" to the regular expressions above would allow for
*           users to enter special keys sometimes used for extension numbers or
*           pauses
*/
"phoneNumber";


/**
* Flags to set automatic removal of events from the page event registry.
*/
type FireStyle = 
/**
* Call the registered handler any time the event occurs
*/
null |

/**
* Call the registered handler the first time the event                occurs, then unregister the handler as though
*                 Page.clearEvent had been called
*/
"once";


/**
* Strategies for calculating the FiscalYear within a FiscalCalendar from the
*  specified FiscalCalendar.defaultDate and FiscalCalendar.defaultMonth
*  If the specified fiscal year date starts in one calendar year and ends in the next.
*/
type FiscalYearMode = 
/**
* The fiscalYear value for the date range will match the calendar year in which the period ends. For example if the defaultDate and defaultMonth were set
*  to represent April 1st, the fiscal year starting on April 1st 2020 would end on
*  April 1st 2021. Setting the fiscalYearMode to end would mean the
*  fiscalYear value for this block would be 2021.
*/
"end" |

/**
* The fiscalYear value for the date range will match the calendar year in which the period starts. For example if the defaultDate and defaultMonth were set
*  to represent April 1st, the fiscal year starting on April 1st 2020 would end on
*  April 1st 2021. Setting the fiscalYearMode to start would mean the
*  fiscalYear value for this block would be 2020.
*/
"start";


/**
* A decimal (or "floating point") number, for example, 5.5. Null is allowed.
*/
type Float = number;


/**
* Approach to force a text value to be interpreted as text rather than parsed as a date, time
*  or other structured types, as can happen with Microsoft Excel. For background information,
*  see excelPasting.
*/
type ForceTextApproach = 
/**
* a leading space character is added
*/
"leadingSpace" |

/**
* text value is turned into a trivial Excel formula (eg "car" becomes ="car"). In Excel, this renders just the value "car" but editing the cell reveals the formula.
*/
"formula";


/**
* This string is the base CSS class name applied to a FormItem (or some part of a form item). 
*  See the formItemStyling for more information about
*  styling formItems.
*  
*  The specified style name will be modified as the 'state' of the form item changes. 
*  Developers should provide appropriately named CSS classes for each stateful style
*  described below:
*  
* - If FormItem.showPending is true, when the current value differs from the
*    value that would be restored by a call to DynamicForm.resetValues, this style
*    will have the suffix "Pending" appended to it.
*  
* - If FormItem.showFocused is true, when the form item receives focus, this
*    style will have the suffix "Focused" appended to it.
*  
* - If FormItem.showOver is true, roll-over will be indicated by appending the
*    suffix "Over" appended to the style name. This applies to the
*    FormItem.textBoxStyle and
*    FormItem.controlStyle only.
*  
* - If FormItem.showErrorStyle is true, if the form item has errors, this
*    style will have the suffix "Error" appended to it.
*  
* - If FormItem.showDisabled is true, when the form item is disabled, this
*    style will have the suffix "Disabled" appended to it.
*  
* - Finally, if FormItem.showRTL is true, when the form item is in RTL mode, this
*    style will have the suffix "RTL" appended to it.
*  So for example if the cellStyle for some form item is set to "formCell" and showFocused
*  is true, when the form item receives focus, the form item's cell will have the "formCellFocused"
*  style applied to it.
*/
type FormItemBaseStyle = string;


/**
* Name of a SmartClient Class that subclasses FormItem. Some values with this type:
*  
* - TextItem
*  
* - SliderItem, 
*  
* - CanvasItem
*  
*/
type FormItemClassName = string;


/**
* HTML elements that make up a complete FormItem (note, not all FormItems use all of 
*  these elements)
*  
*/
type FormItemElementType = 
/**
* The form item as a whole, including the text element, any icons,           and any hint text for the item. This is the cell containing the 
*            form item
*/
"cell" |

/**
* The "control" cell containing the text box and picker
*/
"control" |

/**
* The cell containing the item's picker icon, if it has one
*/
"pickerIcon" |

/**
* The item's native text box, if it has one
*/
"textBox" |

/**
* The cell containing the title
*/
"title";


/**
* DynamicForms automatically choose the FormItem type for a field based on the
*  type property of the field. The table below describes the default FormItem
*  chosen for various values of the type property.
*  
*  You can also set FormItem.editorType to the classname of a
*  FormItem to override this default mapping. You can alternatively override
*  DynamicForm.getEditorType to create a form with different rules for which
*  FormItems are chosen.
*  
*/
type FormItemType = 
/**
* Rendered as a TextItem, unless the length of the field (as specified by DataSourceField.length attribute) is larger than the value
*  specified by DynamicForm.longTextEditorThreshold, a
*  TextAreaItem is shown.
*/
"text" |

/**
* Rendered as a CheckboxItem
*/
"boolean" |

/**
* Rendered as an IntegerItem, a trivial subclass of           TextItem, by default. 
*           Consider setting editorType:SpinnerItem.
*/
"integer" |

/**
* Rendered as a FloatItem, a trivial subclass of           TextItem, by default.
*           Consider setting editorType:SpinnerItem.
*/
"float" |

/**
* Rendered as a DateItem
*/
"date" |

/**
* Rendered as a TimeItem
*/
"time" |

/**
* Rendered as a DateTimeItem
*/
"datetime" |

/**
* Rendered as a SelectItem. Also true for any field that          specifies a FormItem.valueMap. 
*           Consider setting editorType:ComboBoxItem.
*/
"enum" |

/**
* Same as text
*/
"sequence" |

/**
* If DataSourceField.canEdit:false is set on the field,          the value is rendered as a LinkItem. Otherwise the field 
*           is rendered as a TextItem.
*/
"link" |

/**
* If the field is editable, rendered as a TextItem to edit the           URL or partial URL
*           If FormItem.canEdit, and 
*           DynamicForm.readOnlyDisplay is "static", an
*           image will be rendered out, deriving the URL from the field value
*           combined with FormItem.imageURLPrefix and 
*           FormItem.imageURLSuffix if present. This behavior may be
*           suppressed via DynamicForm.showImageAsURL, in which case
*           the value (URL or partial URL) will be rendered out as static text.
*/
"image" |

/**
* Rendered as a FileItem, or a ViewFileItem if not editable
*/
"imageFile" |

/**
* Rendered as a FileItem, or a ViewFileItem if not editable
*/
"binary";


/**
* Form METHOD parameters - how the form fields are submitted to the server
*/
type FormMethod = 
/**
* GET request -- URL encoding (~4K max)
*/
"GET" |

/**
* POST request -- separate field encoding (no max)
*/
"POST";


/**
* A String to be used as a format specifier for a date, datetime, time or numeric field, via
*  the DataSourceField.format and DataSourceField.exportFormat 
*  properties. 
*  
*  For fields with a numeric type, the format string is similar to java.text.NumberFormat (see
*  DecimalFormat JavaDoc),
*  and for date, time, and datetime types, the format string is similar to
*  java.text.SimpleDateFormat (see 
*  SimpleDateFormat JavaDoc).
*  
*  Note that all the client-side formatting described in this section is is done by the 
*  NumberUtil.format and 
*  DateUtil.format
*  
*  methods. These are static utility methods that your own code can call if you need to 
*  programmatically format a date or number using a FormatString
*  
*  There are 3 possible contexts where a FormatString may be interpreted, and each
*  has slightly different limitations:
* 
*  in-browser rendering &amp; client-driven exports
*  Almost complete support for Java's SimpleDateFormat/DecimalFormat, as described below, with
*  some small extensions for formatting with awareness of a FiscalCalendar.
*  
*  This category includes cases where code running in the browser does the rendering and
*  the rendered result is passed to the server, such as 
*  ListGrid.exportClientData and
*  RPCManager.exportContent.
* 
*  Excel export 
*  Almost the same as in-browser rendering, with minor limitations due to missing features in
*  Excel. Exact differences are described under DataSourceField.exportFormat.
* 
*  non-Excel server export
*  For example, CSV, XML or JSON ExportFormat provided via
*  DataSource.exportData. Full support for SimpleDateFormat/DecimalFormat
*  as provided by whichever Java version you have installed on the server. However note that
*  depending on the context of the export, the default behavior may be to ignore format
*  strings, since formatting intended for end users wouldn't be desirable if data exchange is
*  the goal. See the exportFormatting for details.
* 
*  
*  Date Format
*  
*  Format token
*    Description
*    Sample value
*  yy   Year as a two-digit number              "99"  or "07"
*  yyyy  Year as a four-digit number              "1999" or "2007"
*  YY   Week year as a two-digit number (week year is the year associated with the entire week deemed to contain a given date, and it may differ from calendar year. For example, the week year of December 31 2012 is 2013)"99"  or "07"
*  YYYY  Week year as a four-digit number           "1999" or "2007"
*  LL   Fiscal year as a two-digit number (FiscalCalendar)"99"  or "07"
*  LLLL  Fiscal year as a four-digit number          "1999" or "2007"
*  M   Month in year                     "1" to "12"
*  MM   Month in year with leading zero if required      "01" to "12"
*  MMM  Short month name (DateUtil.shortMonthNames)    "Jan" to "Dec"
*  MMMM  Full month name (DateUtil.monthNames)       "January" to "December"
*  w   Week in year                     "1" to "52"
*  ww   Week in year with leading zero if required      "01" to "52"
*  C   Week in fiscal year (FiscalCalendar)      "7" or "29"
*  CC   Week in fiscal year with leading zero if required   "07" or "29"
*  d   Day of the month                   "1" to "31"
*  dd   Day of the month with leading zero if required    "01" to "31"
*  ddd  Short day name (DateUtil.shortDayNames)    "Mon" to "Sun"
*  dddd  Full day name. (DateUtil.dayNames)       "Monday" to "Sunday"
*  E   Short day name ("EE" and "EEE" are equivalent; all are exactly the same as "ddd" - "E" is supported purely to conform with SimpleDateFormat)"Mon" to "Sun"
*  EEEE  Full day name (exactly the same as "dddd")      "Monday" to "Sunday"
*  D   Day in year                      "1" to "366"
*  DD   Day in year with leading zero if required       "01" to "366"
*  c   Day in fiscal year (FiscalCalendar)      "5" or "204"
*  cc   Day in fiscal year with leading zero if required   "05" or "204"
*  u   Day number in week (1 is Monday)           "1" to "7"
*  H   Hour in day, 0-23 (24-hour clock)           "0" to "23"
*  HH   Hour in day with leading zero if required (24-hour)  "00" to "23"
*  h   Hour in day, 1-12 (12-hour clock)           "1" to "12"
*  hh   Hour in day with leading zero if required (12-hour)  "01" to "12"
*  m   Minute in hour                    "0" to "59"
*  mm   Minute in hour with leading zero if required     "00" to "59"
*  s   Second in minute                   "0" to "59"
*  ss   Second in minute with leading zero if required    "00" to "59"
*  S   Millisecond in second                 "0" to "999"
*  SSS  Millisecond in second with leading zero(s) if required"000" to "999"
*  a   The AM/PM designator (Time.AMIndicator)    " am" or " pm"
*  
*  
*  Note that all text that doesn't represent tokens specified above will be passed through to
*  the output, but such unmapped character sequences are reserved for future use. To
*  future-proof your code, you may single quote "'" any text to escape it to ensure
*  no formatting is applied, guaranting that it's passed through unaltered. Thus, a format of
*  "h'h'" might end up as "5h". To create a single quote itself, use
*  two in a row - for example "h o''clock".
*  
*  Examples - various formatted versions of the datetime "2006-08-03 11:26:18"
*  
*  "M/d/yy"8/3/06
*  "MMMM yyyy"August 2006
*  "HH:mm"11:26
*  "d MMM yyyy, H:ma"3 Aug 2006, 11:26 am
*  "dd/MM/yyyy"03/08/2006
*  "CC/LLLL"53/2006 (assuming the fiscal year ends in the first week of August)
*  
*  
*  SimpleDateFormat specifiers that we do not support:
*  
*  
* - Era designator, BC/AD (G)
*  
* - Day of week in month (F)
*  
* - Hour in day, 24-hour, with 1-based instead of normal 0-based numbering (hours are 1-24) (k)
*  
* - Hour in day, 12-hour, with 0-based instead of normal 1-based numbering (hours are 0-11) (K)
*  
* - Timezone (z, Z, X)
*  
*  &nbsp;
*  Number Format
*  
*  Format char
*    Description
*  0     Digit, zero is displayed
*  #     Digit, zero is not displayed
*  -     Minus sign
*  .     Decimal separator
*  %     Multiply by 100 and show as percentage
*  &#x2030; (\u2030) Multiply by 1000 and show as per mille. See below.
*  ,     Indicates digit grouping should be used - eg 1,000,000. See below.
*  ;     Separates positive and negative subpatterns. See below.
*  &#x00a4; (\u00A4) As a prefix or suffix, indicates the local currency symbol should be used. Note that you must use special notation to include this character in an XML file (such as a .ds.xml file). See below.
*  '     Used to quote special characters in a prefix or suffix, for example, "'#'#" formats 123 to "#123". To create a single quote itself, use two in a row: "# o''clock".
*  
*  
*  All other characters in the format string are taken literally and output unchanged.
*  
*  Examples
*  
*  Format string
*    Zero value
*    Positive value: 12345.678
*    Negative value: -2345.123
*  "0.00"0.0012345.68-2345.12
*  ",##0.00"0.0012,345.68-2,345.12
*  "0.###"012345.678-2345.123
*  "&#x00a4;,0.00"$0.00$12,345.68-$2345.12
*  "0.0#%"0.0%1234567.8%-234512.3%
*  "0.0#&#x2030;"0.0&#x2030;12345678.0&#x2030;-2345123.0&#x2030;
*  "0.0#'%'"0.0%12345.68%-2345.12%
*  "0.00;(0.00)"0.0%12345.68(2345.12)
*  
*  Note, the above examples include cases where there are multiple '#' characters in the integer
*  part of the number format (ie, to the left of the decimal separator, or the entire format 
*  if there is no separator). We support this pattern simply because DecimalFormat
*  does: the extra '#' characters are not significant. In other words, the format "##0.00" 
*  produces exactly the same formatting as "0.00", and "##,###,###.##" produces exactly the 
*  same formatting as ",#.##". However, multiple '0' characters in the integer part of the 
*  format are significant, as are both '#' and '0' characters in the decimal part 
*  of the format (ie, to the right of any decimal separator).
*  
*  The ";" character marks the boundary between two subpatterns - the first to be used for 
*  formatting positive numbers (and 0), the second for negative numbers. Specifying a separate
*  pattern for negative numbers is optional: if no negative subpattern is specified, negative
*  numbers are formatted like positive numbers, but with a leading "-" sign.
*  
*  The "&#x00a4;" symbol (\u00A4) is documented in the Java DecimalFormat class as a placeholder for the currency
*  symbol appropriate to the current locale. For client-driven exports, we replace it with
*  
*  the NumberUtil.currencySymbol.
*  Likewise, we use NumberUtil.decimalSymbol and
*  NumberUtil.groupingSymbol to localize the formatting of numbers.
*  
*  
*  Note that "\u00A4" is the correct way to specify this character in Javascript code. If you
*  need to specify the character in an XML file - the common requirement is in a .ds.xml
*  DataSource descriptor file - you must use the code "&amp;#x00A4;" instead.
*  
*  The "&permil;" per mille symbol is specified as "\u2030" in Javascript code; in XML or HTML 
*  you can use either the equivalent notation "&amp;#x2030;" or the special HTML code 
*  "&amp;permil;".
* 
*  
*  DecimalFormat features that we do not support:
*  
*  
* - Scientific notation
*  
* - Doubled currency symbol means "use international currency symbol"
*  
* - We do not support arbitrary digit grouping, by providing patterns of the '#' and ','
*  characters, like DecimalFormat does. Grouping in SmartClient FormatStrings
*  is enabled with a single "," character somewhere before or within the number-formatting
*  part of the string - extra "," characters within the number-formatting part of the string 
*  are tolerated, but they have no effect. Grouping in SmartClient always causes digits to 
*  be gathered in groups of three
*  
*/
type FormatString = string;


/**
* The context for which a data-value is being formatted.
*/
type FormattingContext = 
/**
* for static display in the chart body
*/
"static" |

/**
* for transient display in a hover
*/
"hover";


/**
* An Identifier that's unique in the global scope. For example, the 
*  Canvas.ID of a Canvas is a GlobalId.
*/
type GlobalId = Identifier;


/**
* Possible values for the state of ListGrid groups when groupBy is called
*/
type GroupStartOpen = 
/**
* open all groups
*/
"all" |

/**
* open the first group
*/
"first" |

/**
* start with all groups closed
*/
"none";


/**
* Type of change to the ListGrid.groupTree.
*/
type GroupTreeChangeType = 
/**
* ListGrid.groupBy responsible (grouping fields changed)
*/
"groupBy" |

/**
* ListGrid.regroup responsible (grid already grouped)
*/
"regroup" |

/**
* incremental regroup of a single record
*/
"incremental";


/**
* A synonym for HTMLString.
* @deprecated * in favor of HTMLString.

*/
type HTML = string;


/**
* A String of HTML, such as "text".
*  
*  In many contexts, such as Button.title and ListGrid.formatCellValue, an
*  HTML String can be specified, allowing you to use normal HTML tags and CSS to do
*  formatting or styling.
*  
*  However, bear in mind that if you attempt any kind of layout or advanced styling in such
*  an HTML string, different browsers may render the HTML differently - use SmartClient
*  Layout and Canvas.styleName features wherever possible to
*  avoid this. See also CSSStyleName.
*/
type HTMLString = string;


/**
* Algorithms to use when hashing field values (see DataSourceField.storeWithHash)
*/
type HashAlgorithm = 
/**
* Message Digest algorithm 5
*/
"MD5" |

/**
* Secure Hashing Algorithm (SHA-1)
*/
"SHA" |

/**
* bcrypt is a password hashing function designed by Niels Provos and David          Mazieres, based on the Blowfish cipher
*/
"bcrypt";


/**
* Where a DataBoundComponent.hiliteIcons will be placed relative to 
*  normal cell content.
*/
type HiliteIconPosition = 
/**
* icon will be placed before the normal cell contents
*/
"before" |

/**
* icon will be placed after the normal cell contents
*/
"after" |

/**
* icon will be shown instead of the normal cell contents
*/
"replace";


/**
* Hoop selection modes.
*/
type HoopSelectionStyle = 
/**
* Components completely enclosed by the hoop are selected
*/
"encloses" |

/**
* Components enclosed or intersected by the hoop are selected
*/
"intersects";


/**
* When ListGrid.canHover and
*  ListGrid.showHoverComponents are both true, HoverMode
*  dictates the type of UI to be displayed when a user hovers over a row or cell.
*  
*  There are a number of builtin HoverModes and you can override
*  ListGrid.getCellHoverComponent to create your own
*  hover behaviors.
*/
type HoverMode = 
/**
* Show a single field's value in an HTMLFlow. Field   to use is ListGrid.detailField.
*/
"detailField" |

/**
* Show a DetailViewer displaying those fields from the   record which are not already displayed in the grid.
*/
"details" |

/**
* Show a separate ListGrid containing related-records.   See ListGridRecord.detailDS and ListGrid.recordDetailDSProperty for
*    more information.
*/
"related" |

/**
* Show a DetailViewer displaying those fields   from the record not already displayed in the grid, together with a separate
*    ListGrid containing related-records.
*/
"detailRelated";


/**
* Customizes how users can interact with hovers.
*/
type HoverPersistMode = 
/**
* Hover is dismissed as soon as the mouse moves off the target. This default        type of hover cannot offer an interaction because the hover vanishes 
*         on mouseOut from the target
*/
"none" |

/**
* the user has a brief window to move the cursor over the hover to           prevent it from being dismissed, until mouseOut from the hover, 
*            allowing interactive hovers. Note, if you want your hover to disappear 
*            as soon as something is clicked (like a link, for example), call 
*            Hover.hide
*/
"underMouse" |

/**
* like "underMouse", but if the user clicks in the hover, it is pinned until          the user clicks outside of the hover
*/
"clickPin" |

/**
* like "clickPin", but the hover is pinned as soon as the user places the          cursor over the hover, and needs an outside-click to dismiss
*/
"autoPin";


/**
* Property to govern when the 'over' styling is applied to
*  a formItemIcon.
*/
type IconOverTrigger = 
/**
* Show rollover styling and media when the user is over the icon only
*/
"icon" |

/**
* Show rollover styling and media when the user is over the icon or over the textBox (or control-table, if present) for this icon. Only has
*  an effect when FormItem.showOver is true.
*/
"textBox" |

/**
* Show rollover styling and media when the user is over any part of the FormItem, including the entire cell within a DynamicForm table containing the item.
*/
"item";


/**
* A string which is a valid 
*  JavaScript identifier, as specified by ECMA-262 Section 7.6.
*  
*  NOTE: The String.isValidID
*  
*  function can be used to test whether a name is a valid JavaScript identifier.
*/
type Identifier = string;


/**
*/
type ImageStyle = 
/**
* Center (and don't stretch at all) the image if smaller than its enclosing frame.CENTER:"center",
*/
"center" |

/**
* Tile (repeat) the image if smaller than its enclosing frame.
*/
"tile" |

/**
* Stretch the image to the size of its enclosing frame.
*/
"stretch" |

/**
* Allow the image to have natural size
*/
"normal";


/**
*/
type ImportFormat = 
/**
* XML format: same as that expected by the adminConsole for         DataSource testData
*/
"xml" |

/**
* JSON format: a JSON Array of JSON Objects
*/
"json" |

/**
* Comma-separated values, or in general delimiter-separated values based         on a provided delimiter.
*/
"csv" |

/**
* Auto-detect format
*/
"auto";


/**
* Event that will trigger inline editing. See EditProxy.inlineEditEvent.
*/
type InlineEditEvent = 
/**
* A single mouse click triggers inline editing
*/
"click" |

/**
* A double click triggers inline editing
*/
"doubleClick" |

/**
* No mouse event will trigger inline editing, but it can still              be triggered by a call to EditProxy.startInlineEditing.
*/
"none" |

/**
* A double click triggers inline editing. In addition, if              the widget is selected, starting to type triggers inline editing.
*/
"dblOrKeypress";


/**
* A whole number, for example, 5. Decimal numbers, for example 5.5, are not allowed. Null is
*  allowed.
*/
type Integer = number;


/**
* What the JSONEncoder should do when it encounters a circular reference in an object
*  structure.
*/
type JSONCircularReferenceMode = 
/**
* circular references in Arrays will be represented as a null entry, and objects        will have a property with a null value
*/
"omit" |

/**
* leave a string marker, the JSONEncoder.circularReferenceMarker,         wherever a circular reference is found
*/
"marker" |

/**
* leave a string marker followed by the path to the first occurrence of        the circular reference from the top of the object tree that was serialized.
*         This potentially allows the original object graph to be reconstructed.
*/
"path";


/**
* Format for encoding dates in JSON. Note you can override JSONEncoder.encodeDate
*  for a custom format.
*/
type JSONDateFormat = 
/**
* dates are is encoded as a String in XML Schema date format in UTC,
*     for example, "2005-08-02" for logical date fields or "2005-08-01T21:35:48.350"
*     for datetime fields. See dateFormatAndStorage for more information.
*/
"xmlSchema" |

/**
* dates are encoded as raw JavaScript code for creating a Date object,    that is:
*     
*     new Date(1238792738633)
*     
*     This is not strictly valid JSON, but if eval()d, will result in an identical date object,
*     regardless of timezone. However, it does not preserve the distinction between
*     logical dates vs full datetime values - use "logicalDateConstructor" mode for that.
*/
"dateConstructor" |

/**
* serializes Date instances in a way that preserves the    distinction between logical dates, logical times, and full datetime values, as
*     explained dateFormatAndStorage. Like 'dateConstructor' mode, this
*     does not produce strictly valid JSON, and instead embeds JavaScript calls. 
*     
*     In addition, unlike 'dateConstructor' mode, using eval() to reconstruct the original
*     JavaScript objects will only work in the presence of SmartClient, and not just in a
*     generic JavaScript interpreter.
*/
"logicalDateConstructor";


/**
* Controls the output of the JSONEncoder when instances of SmartClient classes (eg a ListGrid)
*  are included in the data to be serialized.
*/
type JSONInstanceSerializationMode = 
/**
* instances will be shown as a specially formatted JSON listing the most        relevant properties of the instance. Result is not expected to
*         decode()/eval() successfully if instances are included.
*/
"long" |

/**
* instances will be shown in a shorter format via a call to         isc.echoLeaf.
*         Result is not expected to decode()/eval() successfully if instances are
*         included.
*/
"short" |

/**
* no output will be shown for instances (as though they were not present in the        data). Result should decode()/eval() successfully (depending on other
*         settings)
*/
"skip";


/**
* The type of join to make between two SQL tables when resolving 
*  DataSourceField.includeFrom fields.
*/
type JoinType = 
/**
* A regular inner join, whereby rows are only included in the resultset          where the join can be satisified, so a missing row in the table being
*           joined to results in the entire row being omitted.
*/
"inner" |

/**
* An outer join. All outer joins generated by SmartClient's SQL subsystem          are left outer joins, meaning that every row in the join-from (or "left")
*           table that matches the criteria is included, and missing rows in the 
*           join-to (or "right") table cause columns to be set to null.
*/
"outer";


/**
* Strings to identify the various keys on the keyboard.
*  
*  
* -  For alpha keys, the single (uppercase) character value is used, such as "Q"
*  
* -  For Numeric keys, the number is used as in a single character string, like "1"
*  
* -  Function keys are identified as "f1" - "f12"
*  
* -  Non alpha-numeric character keys (such as the key for "[" / "{") are identified by
*     their unmodified character value (the value obtained by hitting the key without 
*     holding shift down), by default - exceptions are listed below.
*  
* -  Additional key names:
*    - Space  
*    - Tab  
*    - Enter
*    - Escape
*    - Backspace
*    - Insert
*    - Delete
*    - Arrow_Up
*    - Arrow_Down
*    - Arrow_Left
*    - Arrow_Right      
*    - Home
*    - End
*    - Page_Up  
*    - Page_Down  
*    - Shift
*    - Ctrl
*    - Alt
*  
*  [Note: Some keys may not be available for capture on every platform]
*/
type KeyName = string;


/**
* Entries for the DrawItem.knobs array. Each specified knobType will enable some UI 
*  allowing the user to manipulate the DrawItem directly.
*  
*  NOTE: Not all knob types are supported by each DrawItem type. Refer to the DrawItem
*  type's DrawItem.knobs attribute documentation for a list of the supported knob types.
*/
type KnobType = 
/**
* Display up to 8 control knobs at the corners specified by DrawItem.resizeKnobPoints,
*  allowing the user to drag-resize the item.
*  See also DrawItem.cornerResizeKnob and DrawItem.sideResizeKnob.
*/
"resize" |

/**
* Display a control knob for moving the item around. See also DrawItem.moveKnobPoint
*  and DrawItem.moveKnobOffset
*/
"move" |

/**
* Control knob to manipulate DrawLine.startPoint.
*/
"startPoint" |

/**
* Control knob to manipulate DrawLine.endPoint.
*/
"endPoint" |

/**
* Display a draggable control knob along with a DrawLine indicating the angle between controlPoint1
*  and the startPoint. Dragging the knob will adjust controlPoint1.
*/
"controlPoint1" |

/**
* Display a draggable control knob along with a DrawLine indicating the angle between controlPoint2
*  and the endPoint. Dragging the knob will adjust controlPoint2.
*/
"controlPoint2" |

/**
* Display a rotation knob above the top resize knob, allowing the user to rotate the item.
*  See also DrawItem.rotateKnob.
*/
"rotate";


/**
*/
type LabelAlignment = 
/**
* Justify label's left edge against its left coordinate              (right in RTL configurations)
*/
"start" |

/**
* Justify label's right edge against its left coordinate              (left in RTL configurations)
*/
"end" |

/**
* Center label about its left coordinate
*/
"center" |

/**
* Justify label's left edge against its left coordinate
*/
"left" |

/**
* Justify label's right edge against its left coordinate
*/
"right";


/**
* Strategy to apply when there is too little room for labels to be shown for all data points with
*  comfortable padding (FacetChart.minLabelGap).
*/
type LabelCollapseMode = 
/**
* Show all labels regardless, even though they will overlap
*/
"none" |

/**
* Show significant time values such as the first day of the month or week. Data values in Records must be true Date objects, not Strings.
*/
"time" |

/**
* Pick round numbers in the range and show labels for just those numbers. Best for continuous datasets that are not time-based
*/
"numeric" |

/**
* Pick periodic values from the dataset and show labels for those. Best when the there are no particular points that would clearly be the best to label
*/
"sample";


/**
* Strategy for determining whether and when to rotate certain labels.
*/
type LabelRotationMode = 
/**
* do not rotate labels
*/
"never" |

/**
* rotate labels if needed in order to make them legible and non-overlapping
*/
"auto" |

/**
* always rotate labels
*/
"always";


/**
* Policy controlling how the Layout will manage member sizes on this axis.
*  
*  Note that, by default, Layouts do not automatically expand the size of all members
*  to match a member that overflows the layout on the breadth axis. This means that a
*  DynamicForm or other component that can't shrink beyond a minimum width will 
*  "stick out" of the Layout, wider than any other member and wider than automatically
*  generated components like resizeBars or sectionHeaders (in a SectionStack).
*  
*  This is by design: matching the size of overflowing members would cause expensive redraws
*  of all members in the Layout, and with two or more members potentially overflowing, could
*  turn minor browser size reporting bugs or minor glitches in custom components into
*  infinite resizing loops.
*  
*  If you run into this situation, you can either:
*  
* - set the overflowing member to Canvas.overflow: "auto", so that it
*  scrolls if it needs more space
*  
* - set the Layout as a whole to Canvas.overflow:"auto", so that the
*  whole Layout scrolls when the member overflows
*  
* - define a Canvas.resized handler to manually update the breadth
*  of the layout
*  
* - set Layout.minBreadthMember to ensure that the available breadth used to
*  expand all (other) members is artificially increased to match the current breadth of the
*  minBreadthMember member; the layout will still be overflowed in this case
*  and the reported size from Canvas.getWidth or Canvas.getHeight won't
*  change, but all members should fill the visible width or height along the breadth axis
*  
*  For the last approach, given the VLayout myLayout and a member 
*  myWideMember, then we could define the following Canvas.resized handler on myLayout:
*  
*  
*  resized : function () {
*    var memberWidth = myWideMember.getVisibleWidth();
*    this.setWidth(Math.max(this.getWidth(), memberWidth + offset));
*  }
*  where offset reflects the difference in width (due to margins, padding,
*  etc.) between the layout and its widest member. In most cases, a fixed offset can
*  be used, but it can also be computed via the calculation:
*  
*  
*    myLayout.getWidth() - myLayout.getViewportWidth()
*  
*  in an override of Canvas.draw
*  for myLayout. (That calculation is not always valid inside the 
*  Canvas.resized handler itself.)
*  
*  Note: the HLayout case is similar- just substitute height where width appears above.
*  
*  See also Layout.overflow.
*/
type LayoutPolicy = 
/**
* Layout does not try to size members on the axis at all, merely stacking them (length
*  axis) and leaving them at default breadth.
*/
"none" |

/**
* Layout sizes members so that they fill the specified size of the layout. The rules
*  are:
*  
*  
* - Any component given an initial pixel size, programmatically resized to a specific
*  pixel size, or drag resized by user action is left at that exact size
*  
* - Any component that Button.autoFit is given exactly the space it
*  needs, never forced to take up more.
*  
* - All other components split the remaining space equally, or according to their
*  relative percentages.
*  
* - Any component that declares a Canvas.minWidth or Canvas.minHeight
*  will never be sized smaller than that size
*  
* - Any component that declares a Canvas.maxWidth or Canvas.maxHeight
*  will never be sized larger than that size
*  
*  In addition, components may declare that they have 
*  Canvas.canAdaptWidth, and may coordinate with the Layout to render
*  at different sizes according to the amount of available space.
*/
"fill";


/**
* Policy for whether resize bars are shown on members by default.
*/
type LayoutResizeBarPolicy = 
/**
* resize bars are only shown on members marked         Canvas.showResizeBar
*/
"marked" |

/**
* resize bars are shown on all resizable members that are not explicitly marked       showResizeBar:false, except the last member. Appropriate for a
*        LayoutPolicy of "fill" (VLayout, HLayout) since the overall space will
*        always be filled.
*/
"middle" |

/**
* resize bars are shown on all resizable members that are not explicitly marked       showResizeBar:false, including the last member. Can be appropriate for a
*        LayoutPolicy of "none" (VStack, HStack) since the overall size of the
*        layout is dictated by it's member's sizes.
*/
"all" |

/**
* resize bars are not shown even if members are marked with         Canvas.showResizeBar
*/
"none";


/**
* Supported positioning of the chart Legend.
*/
type LegendAlign = 
/**
* Align to the left of the legend section
*/
"left" |

/**
* Align centrally in the legend section
*/
"center" |

/**
* Align to the right of the legend section
*/
"right";


/**
* The style of line-breaks to use when exporting data
*/
type LineBreakStyle = 
/**
* Use the default line-break style of the server OS
*/
"default" |

/**
* Use UNIX-style line-breaks (LF only)
*/
"unix" |

/**
* Use MAC-style line-breaks (CR only)
*/
"mac" |

/**
* Use DOS-style line-breaks (both CR & LF)
*/
"dos";


/**
* Supported styles of drawing the endpoints of a line
*/
type LineCap = 
/**
* Semicircular rounding
*/
"round" |

/**
* Squared-off endpoint
*/
"square" |

/**
* Square endpoint, stops exactly at the line's end coordinates instead          of extending 1/2 lineWidth further as "round" and "square" do
*/
"butt";


/**
* Supported styles of drawing lines.
*/
type LinePattern = 
/**
* Solid line
*/
"solid" |

/**
* Dotted line
*/
"dot" |

/**
* Dashed line
*/
"dash" |

/**
* Dotted line, with more tightly spaced dots
*/
"shortdot" |

/**
* Dashed line, with shorter, more tightly spaced dashes
*/
"shortdash" |

/**
* Dashed line, with longer, more widely spaced dashes
*/
"longdash";


/**
*/
type LinkDataFetchMode = 
/**
* In this mode, link data is fetched from the ResultTree.linkDataSource and nodes are separately fetched from the 
*  ResultTree.dataSource. The two fetches are sent together in a 
*  RPCManager.startQueue, with the link data fetch first and the separate 
*  node fetch second. This makes it possible for your server-side code to use the results
*  of the link data fetch to constrain the node fetch (ie, only fetch node information 
*  for nodes that appear in a link)
*/
"separate" |

/**
* In this mode, nodes and link data are fetched together from the main ResultTree.dataSource, and any duplicated node IDs are handled by creating multiple
*  links to a single node. In this mode, the ResultTree.linkDataSource is only used
*  for update operations.
*  
*  Note that the end result of a "single" fetch is exactly the same as fetching link data 
*  and nodes separately using "separate" mode; "separate" mode is also conceptually clearer
*  since it emphasises the fact that nodes and link data are separate things. We provide
*  "single" mode because, in some cases, it may be more efficient to fetch the two types of
*  data together in a single database fetch, using DataSourceField.includeFrom or 
*  some other kind of join technique on the server.
*/
"single";


/**
* Standard component-type displayed within a ListGrid, as contained by ListGrid.gridComponents.
*/
type ListGridComponent = 
/**
* The standard filter-component displayed when ListGrid.showFilterEditor  is true
*/
"filterEditor" |

/**
* The header-component displayed when ListGrid.showHeader is true.
*/
"header" |

/**
* The body component for the grid.
*/
"body" |

/**
* The summary-row component displayed when ListGrid.showGridSummary is  true.
*/
"summaryRow";


/**
* Event that will trigger inline editing.
*/
type ListGridEditEvent = 
/**
* A single mouse click triggers inline editing
*/
"click" |

/**
* A double click triggers inline editing
*/
"doubleClick" |

/**
* No mouse event will trigger editing. Editing must be           programmatically started via ListGrid.startEditing
*            (perhaps from an external button) or may be triggered by
*            keyboard navigation if ListGrid.editOnFocus is set.
*/
"none";


/**
* An object containing the stored presentation information for the fields of a listGrid.
*  Information contained in a ListGridFieldState object includes the
*  visibility and widths of the listGrid's fields.
*  Note that this object is a JavaScript string, and may be stored (for example) as a blob
*  on the server for state persistence across sessions.
*/
type ListGridFieldState = string;


/**
* ListGrids format data for viewing and editing based on the type attribute of the
*  field. This table describes how the ListGrid deals with the various built-in types.
*/
type ListGridFieldType = 
/**
* Simple text rendering for view. For editing a text entry field is shown. If the length of the field (as specified by the DataSourceField.length
*  attribute) is larger than the value specified by ListGrid.longTextEditorThreshold, a
*  text input icon is shown that, when clicked on (or field is focused in) opens a larger
*  editor that expands outside the boundaries of the cell (textarea by default, but
*  overrideable via ListGrid.longTextEditorType).
*/
"text" |

/**
* For viewing and editing a checkbox is shown with a check mark for the true value and no check mark for the false value. This behavior
*  may be suppressed by setting ListGridField.suppressValueIcon for the field. See
*  ListGrid.booleanTrueImage for customization.
*/
"boolean" |

/**
* A whole number, e.g. 123. Consider setting ListGridField.editorType to use a SpinnerItem.
*/
"integer" |

/**
* A floating point (decimal) number, e.g. 1.23. Consider setting ListGridField.editorType to use a SpinnerItem.
*/
"float" |

/**
* Field value should be a Date instance representing a logical date, with no time of day information. See dateFormatAndStorage for
*  details of the logical date type and how it is represented and manipulated.
*  
*  Dates will be formatted using ListGridField.dateFormatter
*  if specified, otherwise
*  ListGrid.dateFormatter.
*  If both these attributes are unset, dates are formatted
*  using the standard DateUtil.setShortDisplayFormat for dates.
*  
*  For editing, by default a DateItem is used with DateItem.useTextField set
*  to true, providing textual date entry plus a pop-up date picker. The
*  DateItem.dateFormatter and DateItem.inputFormat
*  for the editor will be picked up from the ListGridField, if specified.
*/
"date" |

/**
* Field value should be a Date instance representing a logical time, meaning time value that does not have a specific day and also has no timezone. See
*  dateFormatAndStorage for details of the logical time type and how it is
*  represented and manipulated.
*  
*  Times will be formatted using ListGridField.timeFormatter
*  if specified,
*  otherwise ListGrid.timeFormatter.
*  
*  If both these attributes are unset, times are formatted using the standard
*  Time.shortDisplayFormat for times.
*  
*  For editing, by default a TimeItem is used. The
*  TimeItem.timeFormatter for the editor will be picked up from
*  the ListGridField, if specified.
*/
"time" |

/**
* Field value should be a Date instance representing a specific date and time value. See dateFormatAndStorage for details of the
*  datetime type and how it is represented and manipulated.
*  
*  Dates will be formatted using ListGridField.dateFormatter
*  if specified, otherwise
*  ListGrid.datetimeFormatter.
*  If both these attributes are unset, dates are formatted
*  using the standard DateUtil.setShortDatetimeDisplayFormat for
*  datetime values.
*  
*  For editing, by default a DateTimeItem is used, providing textual date entry plus
*  a pop-up date picker. The DateItem.dateFormatter and
*  DateItem.inputFormat for the editor will be picked up from the
*  ListGridField, if specified.
*/
"datetime" |

/**
* Same as text
*/
"sequence" |

/**
* Renders a clickable html link (using an HTML anchor tag: &lt;A&gt;). The target URL is the value of the field, which is also the default display value. You
*  can override the display value by setting ListGridRecord.linkText or
*  ListGridField.linkText.
*  
*  Clicking the link opens the URL in a new window by default. To change this behavior,
*  you can set field.target, which works identically to the "target"
*  attribute on an HTML anchor (&lt;A&gt;) tag. See ListGridField.target for more
*  information.
*  
*  In inline edit mode, this type works like a text field.
*  
*  To create a link not covered by this feature, consider using
*  ListGridField.formatCellValue along with Canvas.linkHTML, or simply
*  ListGrid.getCellStyle to look like a link, and providing
*  interactivity via ListGridField.recordClick.
*/
"link" |

/**
* Renders a different image in each row based on the value of the field. If this URL is not absolute, it is assumed to be relative to
*  ListGridField.imageURLPrefix if specified. The size of the image is controlled by
*  ListGridField.imageSize, ListGridField.imageWidth,
*  ListGridField.imageHeight (and by the similarly-named global default
*  attributes on the ListGrid itself).
*  
*  You can also specify the following attributes on the field: activeAreaHTML, and
*  extraStuff - these are passed to Canvas.imgHTML to generate the
*  final URL.
*  
*  Note if you want to display icons in addition to the normal cell value, you
*  can use ListGridField.valueIcons instead.
*/
"image" |

/**
* Shows ListGridField.icon in every cell, and also in the header. Useful for a field that is used as a button, for example, launches a detail
*  window or removes a row. Implement a ListGridField.recordClick
*  to define a behavior for the button.
*  
*  NOTE: for a field that shows different icons depending on the field value, see
*  ListGridField.valueIcons.
*  
*  type:"icon" also defaults to a small field width, accommodating just the icon
*  with padding, and to a blank header title, so that the header shows the icon only.
*  
*  ListGridField.iconWidth and related properties configure
*  the size of the icon both in the header and in body cells.
*  
*  If you want the icon to appear only in body cells and not in the header, set
*  ListGridField.cellIcon instead, leaving field.icon null.
*/
"icon" |

/**
* For viewing, the grid renders a 'view' icon (looking glass) followed by a 'download' icon and then the name of the file is displayed in text. If the user clicks the
*  'view' icon, a new browser window is opened and the file is streamed to that browser
*  instance, using DataSource.viewFile. For images and other file types with
*  known handlers, the content is typically displayed inline - otherwise the browser will
*  ask the user how to handle the content. If the download icon is clicked,
*  DataSource.downloadFile is used to cause the browser to show a "save" dialog.
*  There is no inline editing mode for this field type.
*/
"binary" |

/**
* Same as binary
*/
"imageFile" |

/**
* Show a calculated summary based on other field values within the current record. See ListGridField.recordSummaryFunction for more information
*/
"summary" |

/**
* Fields of this type can contain any data value and have no default formatting or validation behavior. This is useful as the 
*  SimpleType.inheritsFrom for SimpleTypes
*  where you do not want any of the standard validation or formatting logic
*  to be inherited from the standard built-in types.
*/
"any" |

/**
* An integer number with locale-based formatting, e.g. 12,345,678. See localizedNumberFormatting
*  for more info.
*/
"localeInt" |

/**
* A float number with locale-based formatting, e.g. 12,345.67. See localizedNumberFormatting
*  for more info.
*/
"localeFloat" |

/**
* A float number with locale-based formatting and using currency symbol, e.g. $12,345.67.
*  See localizedNumberFormatting
*  for more info.
*/
"localeCurrency" |

/**
* A telephone number. Uses FormItem.browserInputType "tel" to hint to the device to restrict input. On most mobile devices that have
*  software keyboards, this cause a specialized keyboard to appear which
*  only allows entry of normal phone numbers. When displayed read-only,
*  a "phoneNumber" renders as an HTML link with the "tel:" URL scheme,
*  which will invoke the native phone dialing interface on most mobile
*  devices. In addition, the CSS style "sc_phoneNumber" is applied.
*  
*  By default, "phoneNumber" fields do not include validators, however the
*  following validator definition would limit to digits, dashes and the
*  "+" character:
*  xml:
*  
*    &lt;validator type="regexp" expression="^(\(?\+?[0-9]*\)?)?[0-9_\- \(\)]*$"
*      errorMessage="Phone number should be in the correct format e.g. +#(###)###-##-##" /&gt;
*    
*  
*  or directly in JavaScript:
*  
*  
*  {type:"regexp", expression:"^(\\(?\\+?[0-9]*\\)?)?[0-9_\\- \\(\\)]*$", 
*    errorMessage:"Phone number should be in the correct format e.g. +#(###)###-##-##"}
*  
*  
*  
*  and adding "#" and "*" to the regular expressions above would allow for
*  users to enter special keys sometimes used for extension numbers or
*  pauses
*/
"phoneNumber";


/**
* An object containing the stored grouping information for a listGrid.
*  Note that this object is not intended to be interrogated directly, but may be stored
*  (for example) as a blob on the server for state persistence across sessions.
*/
type ListGridGroupState = string;


/**
* An object containing the stored selection information for a listGrid.
*  Note that this object is not intended to be interrogated directly, but may be stored
*  (for example) as a blob on the server for state persistence across sessions.
*/
type ListGridSelectedState = string;


/**
* An object containing the stored sort information for a listGrid.
*  Note that this object is not intended to be interrogated directly, but may be stored
*  (for example) as a blob on the server for state persistence across sessions.
*/
type ListGridSortState = string;


/**
* An object containing the stored criteria as applied by the user. Does not include criteria
*  applied in other ways.
*/
type ListGridUserCriteriaState = string;


/**
* An object containing the "view state" information for a listGrid.
*  
*  This object contains state information reflecting the following states in the grid:
*  
*  
* - ListGridFieldState
*  
* - ListGridSortState
*  
* - ListGridSelectedState
*  
* - ListGridGroupState
*  
* - ListGridUserCriteriaState
*  
* - hilite state
*  
* - filterEditor visibility state
*  
*  Note that this object is a JavaScript string, and may be stored (for example) as a blob
*  on the server for state persistence across sessions.
*/
type ListGridViewState = string;


/**
*/
type ListGridViewStatePart = 
/**
* All parts of the view state
*/
"all" |

/**
* Group state
*/
"group" |

/**
* Field state
*/
"field" |

/**
* Selected state
*/
"selected" |

/**
* Sort state
*/
"sort" |

/**
* Hilite state
*/
"hilite" |

/**
* Criteria state
*/
"userCriteria";


/**
* The style of list item marker for a list.
*/
type ListStyleType = 
/**
* A filled, black dot (&bull;)
*/
"disc" |

/**
* An unfilled circle (&#9702;)
*/
"circle" |

/**
* A filled, black square (&#9632;)
*/
"square" |

/**
* Numbers (1., 2., 3., etc.)
*/
"decimal" |

/**
* Uppercase Roman numerals (I., II., III., IV., etc.)
*/
"upper-roman" |

/**
* Lowercase Roman numerals (i., ii., iii., iv., etc.)
*/
"lower-roman" |

/**
* Uppercase letters (A., B., C., etc.)
*/
"upper-alpha" |

/**
* Lowercase letters (a., b., c., etc.)
*/
"lower-alpha" |

/**
* An image used in place of a marker.
*/
"custom-image";


/**
*/
type LoadProjectErrorStatus = 
/**
* server has not responded within configured request timeout
*/
"timeout" |

/**
* can't reach server at Reify.serverURL
*/
"noResponseFromURL" |

/**
* no project loader at Reify.serverURL
*/
"badReifyServerURL" |

/**
* invalid Reify.userName or Reify.password
*/
"badCredentials" |

/**
* none of the requested projects were found
*/
"projectNotFound" |

"servletError";


/**
* Trees that dynamically load nodes keep track of whether each node has loaded its children.
*/
type LoadState = 
/**
* children have not been loaded and are not loading
*/
"unloaded" |

/**
* currently in the process of loading
*/
"loading" |

/**
* folders only are already loaded
*/
"foldersLoaded" |

/**
* already fully loaded
*/
"loaded" |

/**
* children form a ResultSet having only a partial      cache (applies only to the "paged"
*       ResultTree.fetchMode)
*/
"loadedPartialChildren";


/**
* The AutoTest subsystem relies on generating and parsing identifier strings to identify
*  components on the page. A very common pattern is identifying a specific component
*  within a list of possible candidates. There are many many cases where this pattern
*  is used, for example - members in a layout, tabs in a tabset, sections in a section stack.
*  
*  In order to make these identifiers as robust as possible across minor
*  changes to an application, (such as skin changes, minor layout changes, etc) the
*  system will store multiple pieces of information about a component when generating
*  an identification string to retrieve it from a list of candidates.
*  The system has a default strategy for choosing the order in which to look at these
*  pieces of information but in some cases this can be overridden by setting
*  a LocatorStrategy.
*  
*  By default we use the following strategies in order to identify a component from a list of
*  candidates:
*  
* - name: Does not apply in all cases but in cases where a specified
*  name attribute has meaning we will use it - for example for
*  SectionStackSection.name or Img.name.
*  
*  
* - title: If a title is specified for the component this may be used
*   as a legitimate identifier if it is unique within the component - for example
*   differently titled tabs within a tabset.
*  
* - index: Locating by index is typically less robust than by name or
*   title as it is likely to be affected by layout changes on the page.
*  
*  If an explicit strategy is specified, that will be used to locate the component if
*  possible. If no matching component is found using that strategy, we will continue to
*  try the remaining strategies in order as described above. In other words setting
*  a locatorStrategy to "title" will skip attempting to find a component by name, and
*  instead attempt to find by title - or failing that by index.
*  
*  In cases where the name is considered definitive, such as for Tab.name or
*  FormItem.name, no fallback check will occur if a name is provided in the
*  locator but doesn't match a live object - the locator will fail to match anything.
*  Furthermore, in the case of Tab, FormItem, or collections
*  other than Canvas.children, if a title is
*  present in the locator and you haven't specified specified "index" as the strategy,
*  there may be no fallback check using the index if the locator title fails to match.
*  
*  To avoid the Framework trying to match by name or title where they are assumed definitive
*  and we skip fallback to the remaining locator attributes, you'll need to remove the name
*  or title from the locator in question (or set the locatorStrategy to "index" in the case
*  of the title).
*  
*  Note that we also support matching by type (see LocatorTypeStrategy).
*  Matching by type is used if we were unable to match by name or title or to disambiguate
*  between multiple components with a matching title.
*/
type LocatorStrategy = 
/**
* Match by name if possible.
*/
"name" |

/**
* Match by title if possible.
*/
"title" |

/**
* Match by index
*/
"index";


/**
* When attempting to identify a component from within a list of possible candidates
*  as described LocatorStrategy, if we are unable to find a unique match
*  by name or title, we will use the recorded "type" of the component to verify
*  an apparent match.
*  
*  By default we check the following properties in order:
*  
* - Does the Class match?
*    
* - If this is not a Class.isFrameworkClass, does the
*      core framework superclass match?
*    
* - Does the role match?
*  
*  In some cases an explicit locatorTypeStrategy can be specified to modify this
*  behavior. As with LocatorStrategy, if we are unable to match using the
*  specified type strategy we continue to test against the remaining strategies in order - 
*  so if a type strategy of "scClass" was specified but we were unable to find a match
*  with the appropriate core superclass, we will attempt to match by role.
*  Possible values are:
*/
type LocatorTypeStrategy = 
/**
* Match by class if possible
*/
"Class" |

/**
* Ignore specific class and match by the SmartClient framework superclass.
*/
"scClass" |

/**
* Ignore class altogether and attempt to match by role
*/
"role" |

/**
* Don't attempt to compare type in any way
*/
"none";


/**
* Priority levels for log messages
*/
type LogPriority = 
/**
* unrecoverable error
*/
1 |

/**
* error, may be recoverable
*/
2 |

/**
* apparent problem, misused API, partial result
*/
3 |

/**
* significant events in normal operation
*/
4 |

/**
* diagnostics for developers
*/
5;


/**
* Operators that can evaluate a set of criteria and produce a combined result.
*/
type LogicalOperator = 
/**
* true if all criteria are true
*/
"and" |

/**
* true if any criteria are true
*/
"or" |

/**
* true if all criteria are false
*/
"not";


/**
* How should a canvas match its target Canvas.htmlElement size?
*/
type MatchElementSizeMode = 
/**
* canvas will be sized to match the element
*/
"matchElement" |

/**
* htmlElement should be sized to match the specified size of the canvas
*/
"sizeElement" |

/**
* do not attempt to match the canvas / element size
*/
"none";


/**
*/
type MaxStackDismissMode = 
/**
* dismiss the oldest message
*/
"oldest" |

/**
* dismiss the message with least time left
*/
"countdown";


/**
* Simple string identifiers for standard menu fields.
*/
type MenuFieldID = 
/**
* Displays the icon field for the menu. This field contains the items         specified icon (if there is one), or if the item is checked, the
*          checkmark icon for the item.
*/
"icon" |

/**
* Displays the item's title
*/
"title" |

/**
* Displays the key field for the menu. This field contains the name or        title of any shortcut keys for this menu item.
*/
"key" |

/**
* Field to display the submenu image for items that have a submenu.
*/
"subMenu";


/**
* A positive integer representing the priority of a message. Lower numerical values have
*  higher priority.
*/
type MessagePriority = Integer;


/**
*/
type MinimalScrollbarContrastSuffix = 
/**
* suffix to apply to MinimalScrollbar.trackBaseStyle and  MinimalScrollbar.thumbBaseStyle to get a light appearance 
*   (appropriate for contrasting against a dark background)
*/
"Light" |

/**
* suffix to apply to MinimalScrollbar.trackBaseStyle and  MinimalScrollbar.thumbBaseStyle to get a dark
*   appearance (appropriate for contrasting against a light background)
*/
"Dark";


/**
*/
type MockDSExportFormat = 
/**
* export as Reify-specific CSV
*/
"reifyCSV" |

/**
* serialize as XML
*/
"xmlMockDS" |

/**
* serialize as JavaScript
*/
"jsMockDS";


/**
*/
type MockDSExportValidatorMode = 
/**
* export all explicit, client-side validators
*/
"all" |

/**
* export all explicit, client-side standard (non-custom) validators
*/
"standard" |

/**
* export no validators
*/
"none";


/**
* Specifies the format of the mock data.
*/
type MockDataFormat = 
/**
* Mock data in "balsamiq" format
*/
"mock" |

/**
* Mock data in CSV format
*/
"csv" |

/**
* Mock data in XML format
*/
"xml" |

/**
* Mock data in JSON format
*/
"json";


/**
* Whether the mock data is for a flat grid-like dataset or for a tree. If "grid" is
*  specified, text shortcuts that would cause a hierarchy to be created (such as starting a
*  line with "[+]") will not have special meaning and be considered to be just a normal
*  data value.
*/
type MockDataType = 
/**
* Mock data for a ListGrid
*/
"grid" |

/**
* Mock data for a TreeGrid
*/
"tree";


/**
* Specifies the starting point of a move knob with respect to its draw item. The move knob is
*  positioned relative to the move knob point at the DrawItem.moveKnobOffset.
*/
type MoveKnobPoint = 
/**
* Top Left corner
*/
"TL" |

/**
* Top Right corner
*/
"TR" |

/**
* Bottom Left corner
*/
"BL" |

/**
* Bottom Right corner
*/
"BR";


/**
*/
type MultiAutoChild = object;


/**
* Specifies the layout of the combo box and buttons in a MultiComboBoxItem.
*/
type MultiComboBoxLayoutStyle = 
/**
* use a FlowLayout, showing values first, then the text entry area
*/
"flow" |

/**
* use a FlowLayout, with the text entry first and values shown afterwards
*/
"flowReverse" |

/**
* Use a horizontal layout with the combo box on the right
*/
"horizontal" |

/**
* Use a horizontal layout with the combo box on the left
*/
"horizontalReverse" |

/**
* Use a vertical layout
*/
"vertical" |

/**
* Use a vertical layout with the combo box at the bottom
*/
"verticalReverse";


/**
*/
type MultiMessageMode = 
/**
* messages of the same NotifyType are arranged in a stack
*/
"stack" |

/**
* messages of the same NotifyType replace each other
*/
"replace";


/**
* Controls when primary keys are required for "update" and "remove" server operations, when allowMultiUpdate 
*  has not been explicitly configured on either the OperationBinding.allowMultiUpdate or via the server-side API DSRequest.setAllowMultiUpdate().
*/
type MultiUpdatePolicy = 
/**
* having a PK is never required, even for requests from a browser. Note: dangerous setting 
*  that allows end users to wipe out entire tables
*/
"never" |

/**
* having a PK is required for requests that come from the client or are specifically marked 
*  via dsRequest.setClientRequest(true)
*/
"clientRequest" |

/**
* having a PK is required for any request that is associated with an RPCManager, which 
*  includes clientRequests and server-created DSRequests where an RPCManager was explicitly provided
*/
"rpcManager" |

/**
* having a PK is always required no matter what
*/
"always";


/**
* Various events triggered by any window in the current application.
*  
*  Aside from the standard event types listed, other types may be supported depending upon
*  whether OpenFin is in use:
*  
* - With OpenFin, application event types
*  are allowed.
*  
* - Without OpenFin, HTML Dom Events
*  supported by window.addEventListener()
*  are allowed.
*/
type MultiWindowEvent = 
/**
* fires when a window finishes loading
*/
"load" |

/**
* fires when a window is about to be reloaded or closed
*/
"unload" |

/**
* fires when a window is moved
*/
"move" |

/**
* fires when a window is resized
*/
"resize" |

/**
* fires when a window is closed
*/
"close" |

/**
* fires when a window gets focus
*/
"focus" |

/**
* fires when a window loses focus
*/
"blur" |

/**
* (synonym for "focus")
*/
"activate" |

/**
* (synonym for "blur")
*/
"deactivate";


/**
* Appearance for a SelectItem that allows multiple selection
*/
type MultipleAppearance = 
/**
* a drop-down picklist that allows multiple choices by       clicking on a checkbox next to each item
*/
"picklist" |

/**
* a grid that displays all items in-place. Multiple selection is        accomplished by ctrl-click or shift-click.
*/
"grid";


/**
* Options for how values are stored for a field that is
*  ListGridField.multiple. See DataSourceField.multipleStorage.
*/
type MultipleFieldStorage = 
/**
* values are saved as a simple delimeter-separated string. Delimeter    can be configured via DataSourceField.multipleStorageSeparator. An empty
*     array is stored as "", and null as the database null value.
*/
"simpleString" |

/**
* values are serialized to JSON. Empty array as a distinct value     from null (it appears as the text "[]").
*/
"json" |

/**
* no transformation is applied to values; server-side field value remains a Java    List when passed to the execute(Fetch|Add|Update|Remove) method of the server-side
*     DataSource class
*/
"none";


/**
* Navigation direction.
*/
type NavigationDirection = 
/**
* Back
*/
"back" |

/**
* Forward
*/
"forward" |

/**
* none
*/
"none";


/**
* Method of, or reason for, navigation between panes.
*/
type NavigationMethod = 
/**
* a "back" NavigationButton has been clicked
*/
"backClick" |

/**
* a side panel NavigationButton has been clicked
*/
"sideClick" |

/**
* application code called            SplitPane.setCurrentPane,
*             SplitPane.showNavigationPane,
*             SplitPane.showListPane,
*             SplitPane.showDetailPane, etc.
*/
"programmatic" |

/**
* application code directly called            SplitPane.navigatePane
*/
"navigatePane" |

/**
* a seletion change automatically called              SplitPane.navigatePane
*/
"selectionChanged" |

/**
* browser navigation triggered SplitPane navigation
*/
"historyCallback";


/**
* Controls the navigation mode of records.
*/
type NavigationMode = 
/**
* Clicking anywhere on the record navigates
*/
"wholeRecord" |

/**
* Only clicking directly on the navigation icon                  triggers navigation
*/
"navIconOnly";


/**
*/
type NotifyTransition = 
/**
* message slide-animates onto or off of the screen
*/
"slide" |

/**
* message appears or disappears via a fade effect
*/
"fade" |

/**
* message instantly appears or disappears
*/
"instant";


/**
* An identifier passed to Notify APIs to group related messages together so that they
*  all use the same behavior and display settings.
*/
type NotifyType = Identifier;


/**
* The possible access types for records with a null DataSource.ownerIdField
*  (only applicable if ownerIdField is specified)
*/
type NullAccessType = 
/**
* The default value, means that users have no access to records with a null
*  ownerIdField. In this case, users can only see their own records (ie, 
*  those where the ownerIdField matches the currently authenticaed user's id)
*/
"none" |

/**
* Users are allowed read-only access to records with a null
*  ownerIdField. In this case, users can see records with a null owner as 
*  well as their own records.
*/
"view" |

/**
* Users are allowed read, update and delete access to records with a null
*  ownerIdField. In this case, users can see and fully manage records with 
*  a null owner, as well as their own records.
*/
"edit";




/**
* An "Object literal" is JavaScript shorthand for defining a JavaScript Object with a set
*  of properties. For example, code like this:
*  
*   var request = {
*     actionURL : "/foo.do",
*     showPrompt:false
*   };
*  .. is equivalent to ..
*  
*   var request = new Object();
*   request.actionURL = "/foo.do";
*   request.showPrompt = false;
*  In situations where a set of Properties may be passed to a
*  method, the Object literal notation is much more compact. For example:
*  
*   isc.RPCManager.sendRequest({
*     actionURL : "/foo.do",
*     showPrompt:false
*   });
*  NOTE: if you have a 'trailing comma' in an object literal, like so:
*  
*   var request = {
*     actionURL : "/foo.do",
*     showPrompt:false, // TRAILING COMMA
*   };
*  This is considered a syntax error by Internet Explorer, but not by Firefox. This is by
*  far the #1 cause of Internet Explorer-specific errors that do not occur in other
*  browsers. Pay special attention to this error, and, if you can, install the
*  JSSyntaxScannerFilter into your development environment (as described in the
*  iscInstall).
*/
type ObjectLiteral = object;


/**
* An operator is used as part of a Criterion when specifying AdvancedCriteria.
*  
*  This list of operators indicates the set of operators built into SmartClient DataSources,
*  which can be used for both client and server-side filtering. Some operators offer 
*  case-insensitive versions, prefixed with a lower-case i, such as 
*  iContains. Note that such operators are intended for text-based searches and 
*  are not available to numeric or date fields (integer/float/date/datetime and derivatives), 
*  where there is no use for case.
*  
*  You can extend the list of operators with DataSource.addSearchOperator.
*/
type OperatorId = 
/**
* exactly equal to
*/
"equals" |

/**
* not equal to
*/
"notEqual" |

/**
* exactly equal to, if case is disregarded
*/
"iEquals" |

/**
* not equal to, if case is disregarded
*/
"iNotEqual" |

/**
* Greater than
*/
"greaterThan" |

/**
* Less than. Note that null is treated     as equivalent to an arbitrarily small value, so null field values will 
*     always be returned by lessThan / lessOrEqual 
*     filter operations by default.
*/
"lessThan" |

/**
* Greater than or equal to
*/
"greaterOrEqual" |

/**
* Less than or equal to
*/
"lessOrEqual" |

/**
* Contains as sub-string (match case)
*/
"contains" |

/**
* Starts with (match case)
*/
"startsWith" |

/**
* Ends with (match case)
*/
"endsWith" |

/**
* Contains as sub-string (case insensitive)
*/
"iContains" |

/**
* Starts with (case insensitive)
*/
"iStartsWith" |

/**
* Ends with (case insensitive)
*/
"iEndsWith" |

/**
* Does not contain as sub-string (match case)
*/
"notContains" |

/**
* Does not start with (match case)
*/
"notStartsWith" |

/**
* Does not end with (match case)
*/
"notEndsWith" |

/**
* Does not contain as sub-string (case insensitive)
*/
"iNotContains" |

/**
* Does not start with (case insensitive)
*/
"iNotStartsWith" |

/**
* Does not end with (case insensitive)
*/
"iNotEndsWith" |

/**
* shortcut for "greaterThan" + "and" + "lessThan" (case insensitive)
*/
"iBetween" |

/**
* shortcut for "greaterOrEqual" + "and" + "lessOrEqual" (case insensitive)
*/
"iBetweenInclusive" |

/**
* Basic GLOB matching using wildcards    (see DataSource.translatePatternOperators for more information on available patterns)
*/
"matchesPattern" |

/**
* Basic GLOB matching using wildcards (case insensitive)    (see DataSource.translatePatternOperators for more information on available patterns)
*/
"iMatchesPattern" |

/**
* GLOB matching using wildcards. Value is considered to meet the    criterion if it contains the pattern. See DataSource.translatePatternOperators
*     for more information on available patterns)
*/
"containsPattern" |

/**
* GLOB mathcing using wildcards. Value is considered to meet the    criterion if it starts with the pattern.See DataSource.translatePatternOperators
*     for more information on available patterns)
*/
"startsWithPattern" |

/**
* GLOB mathcing using wildcards. Value is considered to meet the    criterion if it starts with the pattern.See DataSource.translatePatternOperators
*     for more information on available patterns)
*/
"endsWithPattern" |

/**
* GLOB matching using wildcards. Value is considered to meet the    criterion if it contains the pattern. Matching is case insensitive. See
*     DataSource.translatePatternOperators for more information on available patterns)
*/
"iContainsPattern" |

/**
* GLOB matching using wildcards. Value is considered to meet the    criterion if it starts with the pattern. Matching is case insensitive.See 
*     DataSource.translatePatternOperators for more information on available patterns)
*/
"iStartsWithPattern" |

/**
* GLOB matching using wildcards.Value is considered to meet the    criterion if it ends with the pattern. Matching is case insensitive. See 
*     DataSource.translatePatternOperators for more information on available patterns)
*/
"iEndsWithPattern" |

/**
* Regular expression match
*/
"regexp" |

/**
* Regular expression match (case insensitive)
*/
"iregexp" |

/**
* value is either null or the empty string. For numeric fields it behaves as isNull
*/
"isBlank" |

/**
* value is neither null nor the empty string ("")
*/
"notBlank" |

/**
* value is null
*/
"isNull" |

/**
* value is non-null. Note empty string ("") is non-null
*/
"notNull" |

/**
* value is in a set of values. Specify criterion.value as an Array
*/
"inSet" |

/**
* value is not in a set of values. Specify criterion.value as an Array
*/
"notInSet" |

/**
* matches another field (match case, specify fieldName as criterion.value)
*/
"equalsField" |

/**
* does not match another field (match case, specify fieldName as criterion.value)
*/
"notEqualField" |

/**
* matches another field (case insensitive, specify fieldName as criterion.value)
*/
"iEqualsField" |

/**
* does not match another field (case insensitive, specify fieldName as criterion.value)
*/
"iNotEqualField" |

/**
* Greater than another field (specify fieldName as criterion.value)
*/
"greaterThanField" |

/**
* Less than another field (specify fieldName as criterion.value)
*/
"lessThanField" |

/**
* Greater than or equal to another field    (specify fieldName as criterion.value)
*/
"greaterOrEqualField" |

/**
* Less than or equal to another field    (specify fieldName as criterion.value)
*/
"lessOrEqualField" |

/**
* Contains as sub-string (match case) another field value    (specify fieldName as criterion.value)
*/
"containsField" |

/**
* Starts with (match case) another field value    (specify fieldName as criterion.value)
*/
"startsWithField" |

/**
* Ends with (match case) another field value    (specify fieldName as criterion.value)
*/
"endsWithField" |

/**
* Contains as sub-string (case insensitive) another field value    (specify fieldName as criterion.value)
*/
"iContainsField" |

/**
* Starts with (case insensitive) another field value    (specify fieldName as criterion.value)
*/
"iStartsWithField" |

/**
* Ends with (case insensitive) another field value    (specify fieldName as criterion.value)
*/
"iEndsWithField" |

/**
* Does not contain as sub-string (match case) another field value    (specify fieldName as criterion.value)
*/
"notContainsField" |

/**
* Does not start with (match case) another field value    (specify fieldName as criterion.value)
*/
"notStartsWithField" |

/**
* Does not end with (match case) another field value    (specify fieldName as criterion.value)
*/
"notEndsWithField" |

/**
* Does not contain as sub-string (case insensitive) another field value    (specify fieldName as criterion.value)
*/
"iNotContainsField" |

/**
* Does not start with (case insensitive) another field value    (specify fieldName as criterion.value)
*/
"iNotStartsWithField" |

/**
* Does not end with (case insensitive) another field value    (specify fieldName as criterion.value)
*/
"iNotEndsWithField" |

/**
* all subcriteria (criterion.criteria) are true
*/
"and" |

/**
* all subcriteria (criterion.criteria) are false
*/
"not" |

/**
* at least one subcriteria (criterion.criteria) is true
*/
"or" |

/**
* shortcut for "greaterThan" + "lessThan" + "and". Specify criterion.start     and criterion.end
*/
"between" |

/**
* shortcut for "greaterOrEqual" + "lessOrEqual" + "and". Specify     criterion.start and criterion.end
*/
"betweenInclusive";


/**
* Indicates the kind of value expected in a Criterion that uses this operator.
*/
type OperatorValueType = 
/**
* Criterion.value should contain a value of the same type as the          field.
*/
"fieldType" |

/**
* Criterion.value should be the name of another field in the record
*/
"fieldName" |

/**
* no criterion.value or other setting required (used for operators like          isBlank or isNull).
*/
"none" |

/**
* Criterion.criteria should be an Array of criteria (used for logical          operators like "and").
*/
"criteria" |

/**
* Criterion.start and Criterion.end should contain start and           end values, both of the same type as the field.
*/
"valueRange" |

/**
* Criterion.value should contain an Array of valid field values.
*/
"valueSet" |

/**
* Criterion.value should contain a value which is not necessarily the         same type as the field (used for regexp and similar operators). 
*          Operator.editorType can be specified as a FormItem to use to
*          enter a value for the criterion.
*/
"custom";


/**
*/
type Orientation = 
/**
* members laid out vertically
*/
"vertical" |

/**
* members laid out horizontally
*/
"horizontal";


/**
*/
type Overflow = 
/**
* Content that extends beyond the widget's width or height is               displayed.
*                Note: To have the content be sized only by the drawn size of
*                the content set the overflow to be Canvas.VISIBLE and specify
*                a small size, allowing the size to expand to the size required
*                by the content.
*                Leaving the width / height for the widget undefined will use the
*                default value of 100, and setting the size to zero may cause the
*                widget not to draw.
*/
"visible" |

/**
* Content that extends beyond the widget's width or height is               clipped (hidden).
*/
"hidden" |

/**
* Horizontal and/or vertical scrollbars are displayed only if               necessary. Content that extends beyond the remaining visible
*                area is clipped.
*/
"auto" |

/**
* Horizontal and vertical scrollbars are always drawn inside the               widget. Content that extends beyond the remaining visible area
*                is clipped, and can be accessed via scrolling.
*/
"scroll" |

/**
* Clip horizontally but extend the canvas's clip region               vertically if necessary. Note: only supported for
*                specific widget subclasses.
*/
"clip-h" |

/**
* Clip vertically but extend the canvas's clip region               horizontally if necessary. Note: only supported
*                for specific widget subclasses.
*/
"clip-v";


/**
* Events registerable via Page.setEvent
*/
type PageEvent = 
/**
* Fires repeatedly (every 10 ms by default) when the system is idle (i.e.,
*     not busy running other scripts) after the page is loaded.
*/
"idle" |

/**
* Fires when the page has finished loading. It corresponds to the
*     browser 'load' event normally handled by window.onload.
*/
"load" |

/**
* Fires when the page is exited or unloaded. It corresponds to the
*     browser 'unload' event normally handled by window.onunload.
*/
"unload" |

/**
* Fires when the browser window is resized by the user. It corresponds
*     to the browser 'resize' event normally handled by window.onresize.
*/
"resize" |

/**
* Fires when the left mouse button is pressed on the Page.
*/
"mouseDown" |

/**
* Fires when the right mouse button is pressed on the Page.
*/
"rightMouseDown" |

/**
* Fires when the mouse moves on the Page.
*/
"mouseMove" |

/**
* Fires when the left mouse button released on the Page.
*/
"mouseUp" |

/**
* Fires when the user clicks the mouse on the Page.
*/
"click" |

/**
* Fires when the uesr double-clicks on the Page.
*/
"doubleClick" |

/**
* Fires when the right mouse button is clicked on the page. If your event handler
*     for this event returns false, the native browser context menu will be suppressed.
*     Note: On the Macintosh platform, Command+Click may be used instead
*     of right-button click to trigger a context menu event.
*     On the Opera browser, Ctrl+Shift+Click should be used instead of
*     right-button click.
*/
"showContextMenu" |

/**
* Fires when a user presses a key on the keyboard.
*/
"keyPress" |

/**
* Fires when the Page.getOrientation changes due    to browser-window resize or rotation of a mobile device.
*/
"orientationChange" |

/**
* Fires when the FontLoader completes loading custom fonts.
*/
"fontsLoaded" |

/**
* Fires after a timeout if the FontLoader fails to load all custom
*     fonts.
* 
*  see classMethod:Page.setEvent()
*  see classMethod:Page.clearEvent()
*/
"fontLoadingFailed";


/**
* Is this page being viewed in landscape or portrait orientation? Typically used with
*  mobile devices.
*/
type PageOrientation = 
/**
* Landscape orientation: page is wider than it is tall.
*/
"landscape" |

/**
* Portrait orientation: page is taller than it is wide.
*/
"portrait";


/**
* Possible placements for pop-up choosers, menus, dialogs or other temporary UIs,
*  which may need to expand to take up additional room for smaller screens.
*/
type PanelPlacement = 
/**
* classic placement for menus, pop-up lists and pickers in desktop           interfaces: near the control that was clicked (a search field,
*            MenuButton, etc). Note: this setting does not apply when there is
*            no originating control (such as a dialog that appears due to session
*            timeout), in which case centering will generally be used
*/
"nearOrigin" |

/**
* fill the nearest containing panel managed by a device-aware layout such as          SplitPane, which will generally be equivalent to "fillScreen" for a
*           Browser.isHandset. Note: this setting does not
*           apply if there is no clear container for the component originating the UI,
*           in which case, "fillScreen" will generally be used.
*/
"fillPanel" |

/**
* fill the entire screen
*/
"fillScreen" |

/**
* fill the bottom half of the screen. This is the default behavior on iOS6/7           for plain HTML &lt;select&gt;, but note that native apps rarely use
*            this interface for picking from lists and it is not generally recommended.
*/
"halfScreen" |

/**
* this setting disables all panelPlacement sizing and positioning logic.           Explicitly specified size and positioning will be used.
*/
"none";


/**
* Action to take if a user attempts to save the dataset produced by a BatchUploader
*  whilst it still contains errors.
*/
type PartialCommitOption = 
/**
* Silently allow the partial commit to proceed (note that this will result         in the user losing those records that contain errors)
*/
"allow" |

/**
* Pop up a message window showing the          BatchUploader.partialCommitError and prevent the partial commit
*/
"prevent" |

/**
* Pop up a confirmation window with the         BatchUploader.partialCommitPrompt and allow the user to choose
*          whether or not to proceed
*/
"prompt" |

/**
* Commit any records that are error-free and remove them from the         grid. If any records had errors, leave them in the grid and leave the
*          grid visible. If no records had errors, run normal cleanup as we would 
*          for "allow". This option allows the user to fix errors iteratively,
*          rather than having to fix everything upfront before committing
*/
"retain";


/**
* Determines sizing model when sizing / positioning a canvas relative to its
*  Canvas.percentBox.
*/
type PercentBoxModel = 
/**
* use coordinates relative to the Canvas.getVisibleHeight and width of the other canvas
*/
"visible" |

/**
* use coordinates relative to the Canvas.getViewportHeight and width of the other canvas
*/
"viewport" |

/**
* use coordinates relative to the  Canvas.getHeight and width of the other canvas. For 
*  Canvas.overflow canvases this may be smaller than drawn size.
*/
"specified" |

/**
* use coordinates relative to the  Canvas.getInnerHeight and width of the other canvas
*/
"inner";


/**
* For PickList items with PickListItemIconPlacement set such that the pickList does
*  not render near-origin, possible location for rendering formItemIcons.
*/
type PickListItemIconPlacement = 
/**
* icon will be displayed in the ComboBoxItem.pickerNavigationBar only (and not rendered
*  inline within the formItem itself)
*/
"pickerNavigationBar" |

/**
* icon will be displayed inline within the form item itself (and not within the
*  ComboBoxItem.pickerNavigationBar
*/
"formItem" |

/**
* icon will be displayed both inline (within the form item itself) and within the
*  ComboBoxItem.pickerNavigationBar
*/
"both";


/**
* Standard pickers
*/
type PickerIconName = 
/**
* Picker icon to clear a field value.
*/
"clear" |

/**
* Picker icon to start a search.
*/
"search" |

/**
* Picker icon to refresh a value.
*/
"refresh" |

/**
* Picker icon for date value.
*/
"date" |

/**
* Picker icon for a general combobox.
*/
"comboBox";


/**
* Supported data point shapes for FacetChart.pointShapes are:
*/
type PointShape = 
"Oval" |

"Square" |

"Diamond" |

"Triangle";


/**
*/
type Positioning = 
/**
* The canvas is absolutely positioned with respect to its                 parent.
*/
"absolute" |

/**
* The canvas is relatively positioned according to the                 document flow.
*/
"relative";


/**
* A positive whole number or 0, for example, 5. Negative values are not allowed. Null is
*  allowed.
*/
type PositiveInteger = Integer;


/**
*/
type PreserveOpenState = 
/**
* Never try to automatically preserve the openState. Nodes will be initially open 
*  or closed based solely on the Tree.openProperty optionally set by the server.
*/
"never" |

/**
* If either the Tree.idField or Tree.nameProperty has been set on 
*  the Tree, (so that nodes have either unique ids or unique paths), 
*  preserve openState by respecting the Tree.openProperty set by the server, 
*  then applying the openState.
*/
"whenUnique" |

/**
* Like "whenUnique" but automatically preserves openState even if nodes cannot be
*  uniquely identified. This means that nodes at the same tree positions 
*  (eg 3rd child of 5th node under root) will be placed in the same openState, 
*  regardless of whether that node has anything to do with the node that previously 
*  was at that tree position.
*/
"always";


/**
*/
type ProcessValueType = 
/**
* values that are not already strings are converted via toString()
*/
"string" |

/**
* the strings "true" and "false" become boolean true and false. All other         Strings non-empty String values are true, all numbers are true except 0, and
*          all other non-null values are true
*/
"boolean" |

/**
* values are converted via toString() and parsing as a decimal number.         Invalid values trigger a transition to the StateTask.failureElement
*/
"decimal" |

/**
* values are converted via toString(), parsing as a number, and rounding to         nearest integer. Invalid values trigger a transition to the
*          StateTask.failureElement
*/
"integer" |

/**
* any input which is not already a Record or Map becomes null
*/
"record" |

/**
* generic array type - will convert value to an array of the same type as the        existing value
*/
"array";


/**
*/
type PromptStyle = 
/**
* Displays a centered modal prompt with text specified by         RPCRequest.prompt
*/
"dialog" |

/**
* Changes the current cursor to the style specified by         RPCRequest.promptCursor
*/
"cursor";


/**
* When the type for a parameter mentions "properties" as in "ListGrid Properties" or
*  "RPCRequest Properties", it means that the expected value is a JavaScript Object
*  containing any set of properties generally legal when creating an object of that type.
*  
*  For example, the first parameter of RPCManager.sendRequest is of type
*  "RPCRequest Properties". This means it should be called like:
*  
*   isc.RPCManager.sendRequest({
*     actionURL : "/foo.do",
*     showPrompt:false
*   });
*  RPCRequest.actionURL and RPCRequest.showPrompt are 
*  properties of RPCRequest. 
*  
*  Note that the notation shown above is an example of a
*  ObjectLiteral.
*/
type Properties = object;


/**
* A means of identifying the properties in an exported dataset - either the property name
*  or its title.
*/
type PropertyIdentifier = 
/**
* Identify properties by internal name
*/
"name" |

/**
* Identify properties by localized descriptive title
*/
"title";


/**
*/
type ProportionalResizeMode = 
/**
* proportional resizing is never used
*/
"none" |

/**
* proportional resizing is always used
*/
"always" |

/**
* proportional resize is off by default, but holding down one of the Canvas.proportionalResizeModifiers turns proportional resize on for a given
*  resize interaction
*/
"modifier" |

/**
* proportional resize is on by default, but holding down one of the Canvas.proportionalResizeModifiers turns proportional resize off for a given
*  resize interaction
*/
"modifierOff";


/**
* SmartClient supports multiple RPC transports for maximum compatibility and feature richness.
*  All of transports use HTTP as the underlying protocol, but use different mechanisms for
*  sending the HTTP request and processing the response. The transport is typically
*  auto-selected for by based on the feature being used and the current browser settings. For
*  advanced use cases, RPCRequest.transport and RPCManager.defaultTransport are
*  exposed as override points.
*  
*/
type RPCTransport = 
/**
* Uses the XMLHttpRequest object to make the request to the server. Note that in some browsers with certain configurations, this transport may not be
*  available. See platformDependencies for more information. This transport is
*  not useful with file uploads. Cannot be used to target cross-domain URLs directly.
*/
"xmlHttpRequest" |

/**
* Write a SCRIPT tag into the DOM with a SRC attribute that targets an arbitrary URL. This transport is the only one that allows direct cross-domain URL
*  access. 
*  
*  For RPCRequest.callback to work, the server being contacted must support the ability
*  to generate JavaScript code in the response that will call a JavaScript function generated
*  by SmartClient. SmartClient passes the name of the function to call via a URL parameter,
*  which can be controlled with RPCRequest.callbackParam. This callback mechanism is
*  sometimes called the "JSONP" (JSON with Padding) approach.
*/
"scriptInclude" |

/**
* Available with SmartClient Server only. An HTML form is dynamically assembled that targets a hidden IFRAME. This mechanism is supported on all
*  browsers and cannot be disabled by end users. 
*  
*  If using the SmartClient Server and using 
*  serverDataIntegration, the "hiddenFrame" transport
*  is automatically used for all RPCManager and DataSource requests if the "xmlHttpRequest"
*  transport is not available.
*  
*  Cannot be used to target cross-domain URLs directly.
*/
"hiddenFrame";


/**
* Dictates the appearance of form items when FormItem.canEdit is set to
*  false.
*/
type ReadOnlyDisplayAppearance = 
/**
* Item value should appear within the form as a static block of text, similar to the default appearance of a StaticTextItem. This appearance may be
*  modified via FormItem.readOnlyTextBoxStyle and FormItem.clipStaticValue.
*/
"static" |

/**
* Item should appear unchanged, but user interaction to edit the item will be disallowed. Note that some interactions will be allowed, such as selecting text
*  within a read-only TextItem for copy and paste. Exact implementation may vary by
*  form item type.
*/
"readOnly" |

/**
* Item will appear disabled.
*/
"disabled";


/**
* Type for controlling when a "recategorize" is applied to records being dropped
*  on a databound component from another databound component.
*/
type RecategorizeMode = 
/**
* recategorize is always applied
*/
"always" |

/**
* recategorize if normal checks pass
*/
"checked" |

/**
* never recategorize
*/
"never";


/**
* The method of component-pooling to employ for ListGrid.showRecordComponents.
*  
*/
type RecordComponentPoolingMode = 
/**
* components are destroyed when the record is not being rendered. Best    for large datasets where embedded components differ greatly per record.
*/
"viewport" |

/**
* components are Canvas.clear when not in the viewport, but    stay with a record until the record is dropped from cache. Best for guaranteed
*     small datasets.
*/
"data" |

/**
* components are pooled and will be passed to    ListGrid.updateRecordComponent with the
*     recordChanged parameter set to true. Best for large datasets where
*     embedded components are uniform across different records and can be efficiently
*     reconfigured to work with a new record
*/
"recycle";


/**
* Controls how ListGrid record drop events report their
*  ListGrid.getRecordDropPosition, and where the drop indicator will be displayed
*  if appropriate.
*/
type RecordDropAppearance = 
/**
* When the user drops onto a record, dropPosition will always be "over"
*/
"over" |

/**
* When the user drops onto a record, dropPosition will be either  "before" or "after" depending on whether the mouse was over the top or bottom of
*   the target record
*/
"between" |

/**
* When the user drops onto a record, if the drop occurs centered over the  record, the dropPosition will be reported as "over", otherwise it will be
*   "before" or "after" depending on whether the mouse was over the top or bottom of the
*   target record.
*/
"both" |

/**
* No dropPosition will be reported
*/
"body";


/**
* Position of a ListGrid.recordDrop operation with respect to the target record.
*/
type RecordDropPosition = 
/**
* User dropped directly onto the record
*/
"over" |

/**
* User dropped before the record
*/
"before" |

/**
* User dropped after the record
*/
"after" |

/**
* Drop position is not over a record
*/
"none";


/**
* Controls the style of TableView record display
*/
type RecordLayout = 
/**
* Show TableView.titleField only
*/
"titleOnly" |

/**
* Show TableView.titleField and                   TableView.descriptionField
*                    fields only
*/
"titleAndDescription" |

/**
* Show TableView.titleField,                    TableView.descriptionField and
*                    TableView.infoField fields only
*/
"summaryInfo" |

/**
* Show TableView.titleField,                    TableView.descriptionField and
*                    TableView.dataField fields only
*/
"summaryData" |

/**
* Show TableView.titleField,                    TableView.descriptionField,
*                    TableView.infoField and
*                    TableView.dataField fields similar to
*                    the iPhoneOS Mail application
*/
"summaryFull";


/**
* Function to produce a summary value based on field values within a record.
*  Example usage is record-level summaries in 
*  ListGridFieldType.
*  
*  SummaryFunctions may be specified in one of 2 ways:
*  
* - as an explicit function or executable
*  stringMethods. Takes 3 parameters:
*  record the record for which the value is being calculated
*  fields Array of listGridFields from which summaries should be calculated
*  summaryField pointer to the summary type field on which the summary function
*  is being run.
*  
* - As a registered RecordSummaryFunction identifier
*  Note that DataSource.registerRecordSummaryFunction may be used to expand the
*  set of registered RecordSummaryFunctions.
*/
type RecordSummaryFunction = 
/**
* Iterates through each field, picking up the numeric field values from the record and summing them.
*/
"sum" |

/**
* Iterates through each field, picking up the numeric field values from the record and calculating the mean value.
*/
"avg" |

/**
* Iterates through each field, picking up the numeric field values from the record and calculating the maximum value.
*/
"max" |

/**
* Iterates through each field, picking up the numeric field values from the record and calculating the minimum value.
*/
"min" |

/**
* Iterates through each field, picking up the numeric field values from the record and multiplying them together.
*/
"multiplier";


/**
* Supported regression algorithms for fitting the data points of a scatter plot.
*/
type RegressionLineType = 
/**
* linear regression
*/
"line" |

/**
* polynomial regression
*/
"polynomial";


/**
* When relative dates are specified in a date range, typically in a RelativeDateItem or
*  DateRangeItem, in order to make the range inclusive or exclusive, it is useful to be able 
*  to specify whether we're referring to the start or end of the date in question.
*/
type RelativeDateRangePosition = 
/**
* Indicates this relative date should be treated as the start of the specified  logical date.
*/
"start" |

/**
* Indicates this relative date should be treated as the end of the specified logical  date.
*/
"end";


/**
* A RelativeDateShortcut is a special string that represents a shortcut to a date phrase that can 
*  be automatically mapped to a RelativeDateString for use in widgets that 
*  leverage relative-dates, such as the RelativeDateItem.
*  
*  Note that some shortcuts indicate a time period but do not directly indicate whether the value
*  refers to the start or end of the time period in question. This ambiguity
*  can be resolved by specifying an explicit RelativeDateRangePosition when calling APIs that 
*  convert from RelativeDates to absolute date values. This is the case for $today, 
*  $tomorrow, $yesterday, $weekAgo, $weekFromNow, $monthAgo
*  and $monthFromNow. If a range position is not explicitly passed, these will all default
*  to the start of the day in question.
*  
*  Builtin options include
*  
*  
* - $now - this moment 
*  
* - $today - the current day. By default this resolves to the start of the current day though
*   an explicit RelativeDateRangePosition may be used to specify the end of the current day.
*  
* - $startOfToday - the start of today
*  
* - $endOfToday - the end of today (one millisecond before the $startOfTomorrow) 
*  
* - $yesterday - the previous day.
*  
* - $startOfYesterday - the start of yesterday
*  
* - $endOfYesterday - the end of yesterday (one millisecond before the $startOfToday) 
*  
* - $tomorrow - the following day
*  
* - $startOfTomorrow - the start of tomorrow 
*  
* - $endOfTomorrow - the end of tomorrow 
*  
* - $weekAgo - the current day of the previous week 
*  
* - $weekFromNow - the current day of the next week 
*  
* - $startOfWeek - the start of the current week 
*  
* - $endOfWeek - the end of the current week 
*  
* - $monthAgo - the current day of the previous month 
*  
* - $monthFromNow - the current day of the following month 
*  
* - $startOfMonth - the start of the current month 
*  
* - $endOfMonth - the end of the current month 
*  
* - $startOfYear - the start of the current year 
*  
* - $endOfYear - the end of the current year 
*  
*  
*  
*/
type RelativeDateShortcut = string;


/**
* A string of known format used to specify a datetime offset. For example, a 
*  RelativeDateString that represents "one year from today" is written as "+1y".
*  
*  RelativeDateStrings are comprised of the following parts:
*  
*  
* - direction: the direction in which the quantity applies - one of + or - 
*  
* - quantity: the number of units of time to apply - a number 
*  
* - timeUnit: an abbreviated timeUnit to use - one of ms/MS (millisecond), s/S (second), 
*    mn/MN (minute), h/H (hour), d/D (day), w/W (week), m/M (month), q/Q (quarter, 3-months), 
*    y/Y (year), dc/DC (decade) or c/C (century). 
*    The timeUnit is case sensitive. A lowercase timeUnit implies an exact offset, so +1d
*    refers to the current date / time increased by exactly 24 hours. If the timeUnit is 
*    uppercase, it refers to the start or end boundary of the period of time in question, so
*    +1D would refer to the end of the day (23:39:59:999) tomorrow, and
*    -1D would refer to the start of the day (00:00:00:000) yesterday.
*  
* - [qualifier]: an optional timeUnit encapsulated in square-brackets and used to offset 
*    the calculation - eg. if +1d is "plus one day", +1d[W] is "plus one day from the 
*    end of the current week". You may also specify another complete RelativeDateString as the
*    [qualifier], which offers more control - eg, +1d[+1W] indicates "plus one day from 
*    the end of NEXT week".
*  
*  
*  This format is very flexible. Here are a few example relative date strings:
*  +0D: End of today. There are often multiple ways to represent the same time
*  using this system - for example this could also be written as -1ms[+1D]
*  -0D: Beginning of today.
*  +1W: End of next week.
*  +1w[-0W]: Beginning of next week.
*  +1w[-0D]: Beginning of the current day of next week.
*/
type RelativeDateString = string;


/**
* Controls where a drag-item should be dropped in relation to the target row
*/
type ReorderPosition = 
/**
* Drop the drag-item before the target-row
*/
"before" |

/**
* Drop the drag-item after the target-row
*/
"after" |

/**
* Drop the drag-item over (onto) the target-row
*/
"over";


/**
*/
type ResizeDirection = 
/**
* resize bar targets the canvas before it in the layout
*/
"before" |

/**
* resize bar targets the canvas after it in the layout
*/
"after";


/**
* Specifies the position of a resize knob with respect to its draw item.
*/
type ResizeKnobPoint = 
/**
* Top Left corner
*/
"TL" |

/**
* Top Right corner
*/
"TR" |

/**
* Bottom Left corner
*/
"BL" |

/**
* Bottom Right corner
*/
"BR" |

/**
* Centered on the top edge
*/
"T" |

/**
* Centered on the bottom edge
*/
"B" |

/**
* Centered on the left edge
*/
"R" |

/**
* Centered on thie right edge
*/
"L";


/**
* While editing a ListGrid, what cell should we edit when the user attempts to navigate
*  into a cell past the end of an editable row, via a Tab keypress, or a programmatic
*  saveAndEditNextCell() call?
*/
type RowEndEditAction = 
/**
* navigate to the first editable cell in the same record
*/
"same" |

/**
* navigate to the first editable cell in the next record
*/
"next" |

/**
* complete the edit.
*/
"done" |

/**
* Leave focus in the cell being edited (take no action)
*/
"stop" |

/**
* take no action
*/
"none";


/**
* When ListGrid.allowRowSpanning is enabled, certain cells may span multiple
*  rows. In this case, the cell displays the value from the record in the first row.
*  If the grid is ListGrid.canEdit (and the
*  ListGridField.canEdit), these settings allow the user
*  to specify what happens to the data when the user edits this cell.
*  
*  Note that in this scenario, a user may begin an edit on the row-spanning cell directly
*  (via double-click for example), or on a cell in another column in any of the rows
*  spanned by the cell. The appropriate behavior with respect to user-experience and how
*  the data is manipulated will depend on the application in question. Developers may
*  of course entirely disable editing for the field via ListGridField.canEdit or
*  ListGrid.canEditCell.
*  
*  See also: ListGrid.useRowSpanStyling
*/
type RowSpanEditMode = 
/**
* This setting assumes that only the field-value for the first record spanned by this cell is significant. In this case the editor will only show for this
*  cell if the user is editing the first spanned record. If the user initialized the edit
*  on another spanned row, the editor will not show for this field.
*/
"first" |

/**
* This setting assumes that each row's values are logically separate, so if a cell spans multiple rows, and a user initializes an edit on some cell in the
*  second spanned row, the spanning cell will show an editor containing the value for
*  the second spanned row. This may differ from the value displayed when not
*  in edit mode (which is derived from the first spanned row by default). This setting may
*  be useful for developers who which to implement their own logic on how to handle
*  spanning cell display values and/or edit values (for example by using custom
*  ListGridField.formatCellValue and applying custom logic to handle
*  editing on ListGridField.editorEnter and ListGridField.editorExit).
*/
"each";


/**
* Behavior of selection when row spanning is active. See ListGrid.useRowSpanStyling.
*/
type RowSpanSelectionMode = 
/**
* when a cell is clicked on, select any cells in subsequent columns which         are at least partially spanned by the clicked cell
*/
"forward" |

/**
* when a cell is clicked on, selects any cells in any other columns which         are at least partially spanned by the clicked cell
*/
"both" |

/**
* behaves like "forward", except as though the cell in the first column          was clicked instead. If the largest row spans are in the first
*           column and all cells in subsequent columns do not extend out of the
*           first cell's span, this creates a row-like selection model where the
*           span of the left-most cell defines the "row" of cells being selected.
*/
"outerSpan";


/**
* Name of a SmartClient Class, that is, a Class that has been created via
*  isc.defineClass, including Classes built into SmartClient, such as "ListGrid".
*  
*  
*/
type SCClassName = string;


/**
* Properties that refer to images by URL, such as Img.src and Button.icon, are
*  specially interpreted in SmartClient to allow for simpler and more uniform image URLs,
*  and to allow applications to be restructured more easily.
*  
*  the application image directory
*  
*  When specifying URLs to image files via SmartClient component properties such as
*  StretchImg.src, any relative path is assumed to be relative to the "application image
*  directory" (appImgDir). The application image directory can be set via
*  Page.setAppImgDir, and defaults to "images/", representing the typical practice of
*  placing images in a subdirectory relative to the URL at which the application is accessed.
*  
*  For applications that may be launched from multiple URLs, the appImgDir can be
*  set to the correct relative path to the image directory by calling
*  Page.setAppImgDir before any SmartClient components are created. This enables
*  applications or components of an application to be launched from multiple locations, or to
*  be relocated, without changing any image URLs supplied to SmartClient components.
*  
*  the "[SKIN]" URL prefix
*  
*  The special prefix "[SKIN]" can be used to refer to images within the skin folder
*  whenever image URLs are supplied to SmartClient components.
*  
*  The value of "[SKIN]" is the combination of:
*  
*  
* - the "skin directory", established in load_skin.js via Page.setSkinDir,
*  plus..
*  
* - the setting for Canvas.skinImgDir on the component where you set an
*  image URL property
*  
*  skinImgDir defaults to "images/", so creating an Img component with
*  Img.src set to "[SKIN]myButton/button.gif" will expand to Page.getSkinDir() +
*  "/images/myButton/button.gif".
*  
*  Some components that use a large number of images use skinImgDir to group them
*  together and make it possible to relocate all the media for the component with a single setting.
*  For example, the TreeGrid class sets skinImgDir to "images/TreeGrid/".
*  This allows TreeGrid.folderIcon to be set to just "[SKIN]folder.gif" but refer to
*  Page.getSkinDir() + "/images/TreeGrid/folder.gif".
*  
*  A custom subclass of TreeGrid can set skinImgDir to a different path, such as
*  "/images/MyTreeGrid", to source all media from a different location.
*  
*  TIPS:
*  
*  
* - subcomponents may not share the parent component's setting for skinImgDir. For
*  example, the Window.minimizeButton has the default setting for "skinImgDir"
*  ("images/"), so the Img.src property used with this component is set to
*  "[SKIN]/Window/minimize.png" (in the "SmartClient" sample skin).
*  
* - for a particular image, the skinImgDir setting on the component may not be
*  convenient. The prefix "[SKINIMG]" can be used to refer to Page.getSkinDir() +
*  "/images" regardless of the setting for skinImgDir
*  
*  Stateful image URLs
*  
*  Many image URLs in SmartClient are "stateful", meaning that the actual URL used to fetch an
*  image will vary according to the component's state (eg, "Disabled"), generally, by adding a
*  suffix to the image URL. See the skinning for more
*  information on statefulness and the Img.src documentation for information on how
*  stateful image URLs are formed.
*  
*  SVG Images
*  
*  If the URL represents an SVG image, you may specify tag as a query param to
*  control whether it's rendered in an object or image tag, provided
*  Canvas.useImageForSVG isn't set. If that query param is present and has any value
*  other than object, then the SVG image will be rendered in an image tag.
*  Otherwise, it will be rendered in an object tag. For example, an SCImgURL of
*  "circle.svg?tag=image" will render in an image tag.
*/
type SCImgURL = URL;


/**
* Image sprites are a 
*  technique to minimize server http requests when displaying media to a user. Given a series of small
*  images (icons, say), instead of having a separate image file be loaded for each icon, 
*  the server can provide a single large composite image to the client, and the client can
*  use CSS to display sections of this larger image to the user.
*  See also the "spriting" discussion in the skinning 
*  documentation.
*  
*  The SmartClient framework supports the SCSpriteConfig format for sprited images.
*  spriteConfigs have the following format:
*  "sprite:&lt;image URL&gt;;offset:&lt;Left&gt;,&lt;Top&gt;;size:&lt;Width&gt;,&lt;Height&gt;;cssClass:&lt;className&gt;"
*  
*  This format allows developers to specify the image source and the native size and
*  offset of the sprite within the image.
*  
*  The media to load will be retrieved from the specified image URL. Standard 
*  SCImgURL directory prefixes such as "[SKIN]" can be included in this URL.
*  An explicit URL is not required - developers may instead specify a css class which can include
*  an explicit background-image attribute.
*  
*  The offset: property specifies the left and top coordinate to apply to the 
*  composite image element such that the desired sprite is visible. The size: 
*  property indicates the size of the sprite in pixels.
*  For example, a sprite configuration of 
*  "sprite:composite.png;offset:0,-20;size:20,20;" would load the image file
*  "composite.png", and display a 20 pixel square from it. The origin of the larger image would
*  be offset vertically by -20px, so the sprite actually visible to the user would have its
*  y-origin at 20px within the larger image.
*  As with the image URL, explicit size and offset are not required - a developer may
*  use css properties (width, height and background-offset)
*  from the specified class to specify the specific sprite to display.
*  
*  The cssClass: denotes the css class to apply to the rendered element displaying
*  the sprite. This is optional - a sprite can be specified with an image URL and explicit sizing and offset
*  coordinates, in which case no css class is actually required.
*  (Of course for a valid sprite, it is expected that the image URL and size are specified
*  either explicitly in the string, or within the css class definition. If the offset is
*  omitted, it will be assumed to be zero on both axes).
*  
*  Sprites will scale automatically. If a spriteConfig is used to provide an image for an icon
*  and some other property is used to configure the drawn size of the icon in question
*  (Button.iconSize,for example), the sprite image will be scaled to render at the
*  appropriate size.
*  
*  Sprited image configuration and "stateful" images
*  Many image URLs in SmartClient are "stateful", meaning that variants of the image should be 
*  displayed to the user depending on the current state of the widget. 
*  (For example a custom button icon may be Button.showRollOverIcon,
*  or a selection checkbox icon in a treeGrid 
*  TreeGrid.showDisabledSelectionCheckbox for unselectable nodes).
*  
*  There are a couple of approaches to display stateful versions of sprited images.
*  
*  Wherever SCStatefulImgConfig objects are supported, a developer may include 
*  spriteConfig strings as entries for specific states.
*  For example, the following SCStatefulImgConfig definition would display different sprites from
*  "compositeImg.png" for base and "Over" states:
*  &nbsp;&nbsp;{ _base:"sprite:compositeImg.png;offset:-100,-100;size:20,20",
*       &nbsp;&nbsp;&nbsp;&nbsp;Over:"sprite:compositeImg.png;offset:-100,-120;size:20,20" }
*  
*  Alternatively, if a property that behaves as the base URL for a stateful image 
*  (such as Button.icon) is set to a sprite configuration string, 
*  the framework will use the state name as a suffix to apply to the
*  source URL or the css class specified in the sprite.
*  As with standard statefulImages, if a URL was specified
*  for the sprite, the stateful suffix will be applied with a preceding "_" character.
*  So if the state "Over" was applied to a sprite configuration of 
*  "sprite:compositeImg.png;offset:100,100;size:20,20", the generated HTML
*  would attempt to load an image from the URL "compositeImg_Over.png" (and
*  display a sprite from that image with the specified size/offset).
*  If a class name was specified in the sprite, the stateful suffix would be appended with
*  no leading underscore, similar to how
*  StatefulCanvas.getStateSuffix are applied to the 
*  baseStyle of a StatefulCanvas.
*  For example a sprite config of "sprite:cssClass:buttonIcon" would display 
*  styling from "buttonIconOver" when the "Over" state was applied.
*/
type SCSpriteConfig = SCImgURL;


/**
* The technique SmartClient Server's SQL DataSource should use to select a "page" of data 
*  from a table.
*/
type SQLPagingStrategy = 
/**
* Specify the paging directly in the SQL query we generate. The way this is done varies 
*  considerably from database to database: with some it is a straightforward built-in 
*  facility while others require arcane tricks or simply don't support the idea. This is 
*  the most efficient method, where available. Note that this strategy is not supported 
*  for operations that make use of a OperationBinding.customSQL clause, 
*  because it depends upon being able to determine the size of the whole dataset without
*  actually retrieving the whole dataset. Ordinary operations do this by means of an 
*  automatically-generated "row count query", but we cannot generate such a query for a 
*  customSQL operation.
*/
"sqlLimit" |

/**
* Implement the paging behavior by use of the absolute() method of the
*  JDBC ResultSet.
*/
"jdbcScroll" |

/**
* Implement the paging behavior by fetching the entire resultset from the database and
*  dropping any unnecessary rows on the server before returning the data to the client.
*  This approach is extremely inefficient, but also extremely straightforward; it is 
*  intended as a fallback option, for situations where the more sophisticated approaches
*  cause problems (a JDBC driver that throws vague exceptions when absolute()
*  is called, for example)
*/
"dropAtServer" |

/**
* No paging behavior: we always return the entire resultset to the client.
*/
"none";


/**
* The types of custom query that can be handled by SmartClient's built-in "sql" and 
*  "hibernate" DataSources. Note, only applies to OperationBinding.operationType
*  "custom".
*/
type SQLType = 
/**
* The custom SQL or HQL is read-only
*/
"query" |

/**
* The custom SQL or HQL updates data
*/
"update";


/**
* How should a window be closed?
*/
type SavedSearchStoredState = 
/**
* only criteria will be stored, even for components that are capable of storing additional viewState.
*/
"criteria";


/**
* When discovering a tree, the scanMode determines how to scan for the childrenProperty
*  "node": take each node individually
*  "branch": scan direct siblings as a group, looking for best fit
*  "level": scan entire tree levels as a group, looking for best fit
*/
type ScanMode = 
/**
* take each node individually
*/
"node" |

/**
* scan direct siblings as a group, looking for best fit
*/
"branch" |

/**
* scan entire tree levels as a group, looking for best fit
*/
"level";


/**
* Affects the appearance and behavior of the builtin SavedSearchEditor.
*/
type SearchEditorMode = 
/**
* the editor shows a FilterBuilder for editing criteria
*/
"normal" |

/**
* the editor shows only a field for naming (or renaming) a search, since the        grid has built-in interfaces for editing criteria
*/
"grid";


/**
* Controls whether and when individual items are selected when clicking on a form in editMode.
*/
type SelectItemsMode = 
/**
* select an individual item if the item itself it clicked on, but not its title cell
*/
"item" |

/**
* select an individual item if either the item or its title cell is clicked on. NOTE: this mode is not the default because it can be make it difficult to select the form as a whole
*/
"itemOrTitle" |

/**
* never allow selection of an individual item
*/
"never";


/**
*/
type Selected = 
/**
* StatefulCanvas should show                   focused state
*/
"Focused" |

/**
* StatefulCanvas is selected
*/
"Selected" |

/**
* StatefulCanvas is not selected
*/
"";


/**
* Appearance when a component is in Canvas.setEditMode and is
*  selected. 
*  
*  Modes such as "tintMask" or "outlineMask" create an 
*  EditProxy.editMask that is layered over the selected component, and 
*  blocks all normal interaction with the component, so that behaviors like
*  EditProxy.supportsInlineEdit can completely take the place of the component's
*  normal interactivity.
*  
*  "outlineEdges" mode allows normal interaction with the component, which allows the end
*  user to do things like ListGrid.canFreezeFields, which
*  the GridEditProxy can implement as a 
*  ${isc.DocUtils.linkForRef('attr:GridEditProxy.saveFieldFrozenState','persistent change to grid\'s configuration')}.
*/
type SelectedAppearance = 
/**
* editMask on top of the component is updated with EditProxy.selectedTintColor            and EditProxy.selectedTintOpacity
*/
"tintMask" |

/**
* editMask on top of the component is updated with EditProxy.selectedBorder
*/
"outlineMask" |

/**
* MultiAutoChild is created on top of the component. This constructs a border around            the component using 4 separate outlineEdge components so that interactivity is not blocked.
*/
"outlineEdges" |

/**
* no change in appearance. Override EditProxy.showSelectedAppearance to create a custom appearance.
*/
"none";


/**
* How data selection should be presented to the user.
*/
type SelectionAppearance = 
/**
* selected rows should be shown with different appearance - see          ListGrid.getCellStyle and optionally
*           ListGrid.selectionCanvas.
*/
"rowStyle" |

/**
* an extra, non-data column should be automatically added to the ListGrid,          showing checkboxes that can be toggled to select rows.
*           See ListGrid.getCheckboxField.
*/
"checkbox";


/**
* Boundary type for limiting where contiguous selection (via shift+click or drag selection)
*  can be applied across Facet.selectionBoundary or 
*  FacetValue.selectionBoundary.
*/
type SelectionBoundary = 
/**
* selection boundary applies to the bottom / right of the cells
*/
"after" |

/**
* selection boundary applies to the top / left of the cells
*/
"before" |

/**
* selection boundary applies to both edges.
*/
"both";


/**
* Enum to indicate selection change notification types. Used by
*  ListGrid.reselectOnUpdateNotifications and
*  TileGrid.reselectOnUpdateNotifications.
*/
type SelectionNotificationType = 
/**
* No selection change notification should fire
*/
"none" |

/**
* ListGrid.selectionChanged should fire but ListGrid.selectionUpdated should not fire.
*/
"selectionChanged" |

/**
* ListGrid.selectionChanged and ListGrid.selectionUpdated should both fire.
*/
"selectionUpdated";


/**
* Different styles of selection that a list, etc. might support
*/
type SelectionStyle = 
/**
* don't select at all
*/
"none" |

/**
* select only one item at a time
*/
"single" |

/**
* select one or more items
*/
"multiple" |

/**
* select one or more items as a toggle      so you don't need to hold down control keys to select
*                  more than one object
*/
"simple";


/**
* Controls how an object changes state when clicked
*/
type SelectionType = 
/**
* object moves to "down" state temporarily (normal button)
*/
"button" |

/**
* object remains in "down" state until clicked again (checkbox)
*/
"checkbox" |

/**
* object moves to "down" state, causing another object to go up (radio)
*/
"radio";


/**
*/
type SendMethod = 
/**
* GET method (regular URL encoding)
*/
"GET" |

/**
* POST method (form field encoding)
*/
"POST";


/**
* The possible types of sequence handling SmartClient Server can apply. This refers to the
*  technique used to obtain the primary keys of the most recent insert, which the product 
*  uses to enable automatic cache synchronization (updating client-side components bound to a 
*  dataSource to reflect updates to that dataSource). Only applicable to
*  DataSourceField of FieldType "sequence".
*  
*/
type SequenceMode = 
/**
* Use the JDBC 3.0 API "getGeneratedKeys()" to get the most recent           sequence value. Obviously, this is only an option for JDBC 3.0+ drivers
*/
"jdbcDriver" |

/**
* Use a database-specific native technique to obtain the most recent           sequence value. The actual technique used varies widely depending on
*            the vagaries of the underlying database (and sometimes the vagaries of
*            particular releases of a database product)
*/
"native" |

/**
* No automatic attempt is made to retrieve the most recent sequence value.           You are expected to handle this by providing a 
*            OperationBinding.cacheSyncOperation that is 
*            able to return the entire row without needing generated PK values for
*            context. For example, a query that uses MAX(pk) would be
*            capable of this. To give a more complex example, say you have a 
*            sequence value that is retrieved from a legacy system: you could store 
*            that sequence value in the HTTP session and then have your custom 
*            cacheSyncOperation reference that session attribute in 
*            its WHERE clause. Also note that cacheSyncOperations, 
*            like any other OperationBinding, 
*            can be OperationBinding.serverObject or any 
*            OperationBinding.script -
*            you do not have to use SQL
*/
"none";


/**
* Strategy for determining whether and when to show 
*  FacetChart.formatDataValue for data-values. There are 
*  two basic mechanisms:
*  
*  
* - inChart - data-values are displayed in the chart-body close to their data-points - 
*         not all FacetChart.chartType support showing 
*         data-values in the chart-body, and none will show them all if data-density
*         is such that the labels would overlap or extend beyond their available area
*  
*  
* - inHover - as the mouse moves over the chart body, the data-value for the
*    FacetChart.getNearestDrawnValue is displayed in a 
*    FacetChart.hoverLabelProperties floating near the shape it represents.
*    When showing values in hovers, the visual element representing the data value is also 
*    emphasized by brightening or highlighting it (appearance differs by chart type).
*  
*  Support for showing data-values differs by FacetChart.chartType; for
*  example, FacetChart.stacked cannot show data-values in the chart
*  body; Column charts can, and they can also 
*  FacetChart.rotateDataValues to fit if necessary; pie charts, can 
*  show just the data-values that fit inside their segments in the chart-body; all types can 
*  show data-values in hovers.
*  
*/
type ShowDataValuesMode = 
/**
* never show data-values at all
*/
"never" |

/**
* show data-values in the chart, when the chartType supports it and when         they fit without overlapping (including after 
*         FacetChart.rotateDataValues), and in hovers otherwise
*/
"auto" |

/**
* show data-values in the chart body, but only if they all fit - no hovers
*/
"inChartOnly" |

/**
* show data-values in the chart body and, if any don't fit, switch to hovers
*/
"inChartOrHover" |

/**
* show data-values that fit in the chart body and also switch on hovers
*/
"inChartAndHover" |

/**
* always show data-values in hovers
*/
"inHoverOnly";


/**
* Type of message to display in ShowMessageTask. Controls the display of the icon.
*/
type ShowMessageType = 
/**
* Normal message
*/
"normal" |

/**
* Warning message
*/
"warning" |

/**
* Error message
*/
"error";


/**
* Side of a component.
*/
type Side = 
/**
* Left side
*/
"left" |

/**
* Right side
*/
"right" |

/**
* Top side
*/
"top" |

/**
* Bottom side
*/
"bottom";


/**
*/
type SkipJSONValidation = 
/**
* Skip validation for all fields. In this mode, valueXPaths and           getFieldValue implementations are skipped.
*/
"full" |

/**
* Skip validation for fields with boolean, numeric, or string-drived           types. Note that you must deliver all 
*            DataSourceField.multiple:true values as arrays.
*/
"partial" |

/**
* Always do validation.
*/
"none";


/**
*/
type SnapGridStyle = 
/**
* show array of crosses to indicate snap points
*/
"crosses" |

/**
* show grid of lines to indicate snap points
*/
"lines";


/**
* Do we display an arrow for the sorted field ?
*/
type SortArrow = 
/**
* Don't show a sort arrow at all.
*/
"none" |

/**
* Display sort arrow in the upper-right corner (above the scrollbar) only.
*/
"corner" |

/**
* Display sort arrow above each field header only.
*/
"field" |

/**
* Display sort arrow above each field header AND in corner above scrollbar.BOTH:"both", // NOTE: Canvas establishes this constant
*/
"both";


/**
*/
type SortDirection = 
/**
* Sort in ascending order (eg: A-Z, larger items later in the list)
*/
"ascending" |

/**
* Sort in descending order (eg: Z-A, larger items earlier in the list)
*/
"descending";


/**
*/
type StackDirection = 
/**
* older messages move up
*/
"up" |

/**
* older messages move down
*/
"down" |

/**
* older messages move right
*/
"right" |

/**
* older messages move left
*/
"left";


/**
*/
type StackPersistence = 
/**
* older messages disappear as if unrelated
*/
"none" |

/**
* older messages have their duration timers reset so they stick         around as long as the new message if they've got less time left 
*          than that message
*/
"reset";


/**
*/
type StandardControlGroup = 
/**
* RichTextEditor.fontControls
*/
"fontControls" |

/**
* RichTextEditor.formatControls
*/
"formatControls" |

/**
* RichTextEditor.styleControls
*/
"styleControls" |

/**
* RichTextEditor.colorControls
*/
"colorControls" |

/**
* RichTextEditor.bulletControls
*/
"bulletControls";


/**
* Constants for the standard states for a StatefulCanvas.
*/
type State = 
/**
* state when mouse is not acting on this StatefulCanvas
*/
"" |

/**
* state when mouse is down
*/
"Down" |

/**
* state when mouse is over
*/
"Over" |

/**
* disabled
*/
"Disabled";


/**
* A String containing a valid JavaScript expression that's automatically converted to a
*  function with a return value matching the value of the last statement.
*  
*  See stringMethods for an example.
*/
type StringMethod = string;


/**
* This is used for client-side or server-side summaries
*  
* - Client-side: Function to produce a summary value based on an array of records and a field definition. 
*  An example usage is the ListGrid.showGridSummary, where
*  a row is shown at the bottom of the listGrid containing summary information about each
*  column.
*  
* - Server-side: Function used for getting summarized field value using 
*  serverSummaries or when 
*  DataSourceField.includeFrom
*  
*  For the client-side SummaryFunctions may be specified in one of 2 ways:
*  
* - as an explicit function or executable
*  stringMethods, which will be passed records (an array of records)
*  and field (the field definition for which the summary is required).
*  
* - as a standard SummaryFunction identifier
*  For valid ways to configure SummaryFunctions to use server-side feature see the
*  serverSummaries, including how to implement custom
*  summary functions, not just builtin ones listed here (look for "Custom Aggregation" section).
*/
type SummaryFunction = 
/**
* Client: iterates through the set of records, picking up and summing all numeric values for the specified field. Returns null to indicate invalid summary value if
*  any non numeric field values are encountered.
*  Server: acts exactly like SQL SUM function.
*/
"sum" |

/**
* Client: iterates through the set of records, picking up all numeric values for the specified field and determining the mean value. Returns null to indicate invalid
*  summary value if any non numeric field values are encountered.
*  Server: acts exactly like SQL AVG function.
*/
"avg" |

/**
* Client: iterates through the set of records, picking up all values for the specified field and finding the maximum value. Handles numeric fields and
*  date/time/datetime type fields only. Returns null to indicate invalid
*  summary value if any non numeric/date field values are encountered.
*  Server: acts exactly like SQL MAX function.
*/
"max" |

/**
* Client: iterates through the set of records, picking up all values for the specified field and finding the minimum value. Handles numeric fields and
*  date/time/datetime type fields only. Returns null to indicate invalid summary value if
*  any non numeric field values are encountered.
*  Server: acts exactly like SQL MIN function.
*/
"min" |

/**
* Client: iterates through the set of records, picking up all numeric values for the specified field and multiplying them together.
*  Returns null to indicate invalid summary value if
*  any non numeric field values are encountered.
*  Server: not supported.
*/
"multiplier" |

/**
* Client: returns a numeric count of the total number of records passed in. Server: acts exactly like SQL COUNT function.
*/
"count" |

/**
* Client: returns field.summaryValueTitle if specified, otherwise field.title
*  Server: not supported.
*/
"title" |

/**
* Client: Currently the same as the min function. Server: implemented as SQL MIN function.
*/
"first" |

/**
* Client: iterates through the set of records, producing a string with each value concatenated to the end.
*  Server: implemented as SQL CONCAT function. Supported only by SQLDataSource. Note that it 
*  is natively supported only by Oracle DB driver, other drivers perform additional query to fetch 
*  values for concatenation. See also DataSourceField.joinPrefix, 
*  DataSourceField.joinString and DataSourceField.joinSuffix 
*  related datasource field attributes.
*/
"concat";


/**
*/
type TEXTAREA_WRAP = 
/**
* don't allow wrapping at all
*/
"OFF" |

/**
* when the entered text reaches the edge of the text area,                 wrap visibly but don't include line breaks in the textarea
*                  value
*/
"SOFT" |

/**
* when the entered text reaches the edge of the text area,                  insert a line break
*/
"HARD";


/**
* An Identifier that must be locally unique within the containing TabSet.
*/
type TabName = Identifier;


/**
* An event that triggers title editing in a TabSet.
*/
type TabTitleEditEvent = 
/**
* Start editing when the user single-clicks a tab title
*/
"click" |

/**
* Start editing when the user double-clicks a tab title
*/
"doubleClick";


/**
* Controls the display mode of TableView record display
*/
type TableMode = 
/**
* The default mode which displays a list of rows
*/
"plain" |

/**
* Grouped table is a set of rows embedded in a rounded                rectangle
*/
"grouped";


/**
* For "fetch" operations, how simple search criteria should be interpreted for text fields 
*  (and to some extent numeric fields, see the notes below). For greater flexibility, use 
*  AdvancedCriteria.
*/
type TextMatchStyle = 
/**
* case-insensitive exact match ("foo" matches "foo" and "FoO", but not "FooBar")
*/
"exact" |

/**
* case-sensitive exact match ("foo" matches only "foo")
*/
"exactCase" |

/**
* case-insenstive substring match ("foo" matches "foobar" and "BarFoo")
*/
"substring" |

/**
* case-insensitive prefix match ("foo" matches "FooBar" but not "BarFoo") 
*  textMatchStyle behavior is consistent across the client and all three of the built-in
*  server DataSource implementations - SQL, Hibernate and JPA - to the maximum extent that is
*  practical. Known inconsistencies, some of which can be tweaked with configuration settings,
*  are described below
*  
* - textMatchStyle can be disabled for any individual field with the 
*  DataSourceField.ignoreTextMatchStyle declaration. This behavior is consistent
*  across client and server implementations, with the proviso that case-sensitive behavior 
*  is inherently inconsistent in the server-side implementations (see below for a more
*  detailed discussion of this)
*  
* - textMatchStyle is forced to "exactCase" for DataSourceField.primaryKey 
*  fields on update and remove operations, unless
*  OperationBinding.allowMultiUpdate is in force. This means that a normal update
*  or remove operation - which is constrained by a unique primaryKey value or combination, to 
*  affect only a single data row - will use an equality match on all primaryKey fields. This
*  is desirable because it makes index usage more likely. Note that this means the key values
*  "foo" and "FOO" are distinct, which is a change from previous behavior. For most databases,
*  this change will have no effect because most databases do not by default allow primaryKey 
*  values that only differ in case anyway - if a table has a row with a PK of "foo" and you 
*  try to insert a row with a PK of "FOO", most databases will fail with a constraint violation.
*  However, if you are using a database hat allows key values that differ only in case, and you 
*  need to support this for some reason, you can switch the framework back to the previous 
*  behavior by setting property allow.case.sensitive.pks to true in
*  your server.properties file. Note, we do not recommend this; in fact, we do 
*  not recommend the use of text primaryKey fields at all. Where you have control ,we would 
*  always recommend the use of automatically incrementing numeric sequence keys.
*  
* - textMatchStyle does not apply to boolean or temporal (date, datetime, time)
*  DataSourceField.type. Such fields are always checked for equality. If 
*  you have a need for more complex matching, such as date-range checking, use AdvancedCriteria
*  
* - Critiera values for temporal fields must be real dates (Javascript Date
*  objects on the client, instances of java.util.Date on the server). Attempting
*  to match against a string or any other type of value will result in an "always fails" 
*  clause being generated. This behavior is how the client code has always worked, but is a 
*  change for the server-side DataSources, in the interests of consistency. The previous
*  behavior was:
*   
* - SQLDataSource used to attempt to match the date column to the criteria value provided;
*   if that value was an appropriately-formatted string, that attempt might have 
*   worked, but it would be database-specific. If you have an application that depends on 
*   this old behavior, you can switch it back on by adding the following to your
*   server.properties file: 
*   sql.temporal.fields.require.date.criteria: false
*   
* - HibernateDataSource and JPADataSource used to skip such clauses altogether, which 
*   results in the opposite behavior (there is an implied "always succeeds" clause, because 
*   the clause is not present in the query at all). If you have an application that depends on 
*   this old behavior, you can switch it back on by adding the following to your
*   server.properties file: 
*   jpa.temporal.fields.require.date.criteria: false or 
*   hibernate.temporal.fields.require.date.criteria: false, as appropriate
*  
*  
* - textMatchStyle does not apply to fields used in the joins generated to satisfy 
*  DataSourceField.includeFrom declarations or entity relations in
*  Hibernate or JPA. Again, such comparisons are always for equality
*  
* - textMatchStyle is applied to numeric fields only if the supplied criteria value
*  is a string. So for a numeric field "myNumber", we will match 10, 100, 110 and 1000 if a 
*  criteria value of "10" (the quoted string) is supplied, but we will only match 10 if a 
*  criteria value of 10 (the actual number) is supplied
*   
* - This is a change in behavior for the server-side DataSource implementations, in the 
*   interests of consistency with the client behavior. Previously, all three of the built-in
*   DataSource implementations applied textMatchStyle to numeric fields regardless of the 
*   criteria value's type. If you have an application that depends on this old behavior, 
*   you can switch it back on by adding the following to your server.properties
*   file: sql.substring.numeric.criteria: true or 
*   hibernate.substring.numeric.criteria: true or 
*   jpa.substring.numeric.criteria: true, as appropriate
*  
*  
* - "exactCase" matching cannot be made consistent across all database providers because 
*  each database has its own idea of whether case-sensitive matching should be the default, 
*  and how to switch it on and off (if that is even possible)
*   
* - Microsoft SQL Server must have case-sensitive matching enabled by setting a 
*   case-sensitive collation when the database is created (there are more fine-grained 
*   options, but we do not recommend them because of the scope for further inconsistency, 
*   where "exactCase" works correctly on table A but not on table B). See 
*   MSDN for details
*   
* - HSQLDB has case sensitivity set with the SET IGNORECASE directive -
*   see here
*   
* - MySQL is case-insensitive, but since it provides an easy way to do per-query,
*   case-sensitive matching in SQL, we work around it for that database, but only in
*   SQL DataSources
*  
*  
* - If you are using Hibernate or JPA, whether "exactCase" works as expected depends 
*  entirely on the underlying database. Unlike the SQL DataSource, where we have fine-grained
*  control of the SQL that the database executes, we are constrained by what the ORM will 
*  allow. This prevents us from compensating for database quirks, like we do with MySQL's
*  case-insensitivity in the SQL DataSource
*  
*/
"startsWith";


/**
* what to do if there is more than one possible childrenProperty when using
*  scanMode "branch" or "level"
*  "node": continue, but pick childrenProperty on a per-node basis (will detect
*        mixed) 
*  "highest": continue, picking the childrenProperty that occurred most as the single
*       choice
*  "stop": if there's a tie, stop at this level (assume no further children)
*/
type TieMode = 
/**
* continue, but pick childrenProperty on a per-node basis (will detect mixed)
*/
"node" |

/**
* continue, picking the childrenProperty that occurred most as the single choice
*/
"highest" |

/**
* if there's a tie, stop at this level (assume no further children)
*/
"stop";


/**
* Policy for laying out tiles. 
*  
*  Because a TileLayout can be either horizontally or vertically oriented, the general term
*  "line" is used to mean either a row or column of tiles. 
*  
*  NOTE: for typical form layouts (a variety of input fields and buttons in a
*  tabular layout with col-spanning and row-spanning support), use a DynamicForm and
*  see the formLayout topic.
*/
type TileLayoutPolicy = 
/**
* Each line has the same number of tiles, based on TileLayout.tilesPerLine if set, otherwise,
*  based on fitting as many tiles per line as possible consistent with
*  TileLayout.tileSize and TileLayout.tileMargin. 
*  
*  Tiles that do not specify a size will be sized to fill available space.
*/
"fit" |

/**
* Tiles are laid out with varying numbers of tiles per line according to each tile's size,
*  the TileLayout.tileMargin, and the available space.
*  
*  Tiles are never resized by the TileLayout and TileLayout.tileSize is ignored.
*/
"flow";


/**
* String designating a standard time format for displaying the times associated with 
*  dates strings.
*/
type TimeDisplayFormat = 
/**
* String will display with seconds and am/pm indicator:[H]H:MM:SS am|pm. 
*  Example: 3:25:15 pm
*/
"toTime" |

/**
* String will display with seconds in 24 hour time: [H]H:MM:SS. 
*  Example: 15:25:15
*/
"to24HourTime" |

/**
* String will display with seconds, with a 2 digit hour and am/pm indicator: 
*  HH:MM:SS am|pm 
*  Example: 03:25:15 pm
*/
"toPaddedTime" |

/**
* String will display with seconds, with a 2 digit hour in 24 hour format: 
*  HH:MM:SS 
*  Examples: 15:25:15, 03:16:45
*/
"toPadded24HourTime" |

/**
* String will have no seconds and be in 12 hour format:[H]H:MM am|pm
*  Example: 3:25 pm
*/
"toShortTime" |

/**
* String will have no seconds and be in 24 hour format: [H]H:MM
*  Example:15:25
*/
"toShort24HourTime" |

/**
* String will have no seconds and will display a 2 digit hour, in 12 hour clock format:
*  HH:MM am|pm
*  Example: 03:25 pm
*/
"toShortPaddedTime" |

/**
* String will have no seconds and will display with a 2 digit hour in 24 hour clock format:
*  HH:MM
*  Examples: 15:25, 03:16
*/
"toShortPadded24HourTime";


/**
* An enum of time-units available for use with the RelativeDateItem,
*  TimeItem and Calendar widgets.
*/
type TimeUnit = 
/**
* a millisecond time-unit
*/
"millisecond" |

/**
* a second time-unit
*/
"second" |

/**
* a minute time-unit
*/
"minute" |

/**
* an hour time-unit
*/
"hour" |

/**
* a day time-unit
*/
"day" |

/**
* a week time-unit
*/
"week" |

/**
* a month time-unit
*/
"month" |

/**
* a quarter (3 month) time-unit
*/
"quarter" |

/**
* a year time-unit
*/
"year";


/**
* Supported positioning of the chart Title.
*/
type TitleAlign = 
/**
* Align to the left of the chart title header
*/
"left" |

/**
* Align centrally in the chart title header
*/
"center" |

/**
* Align to the right of the chart title header
*/
"right";


/**
* Strategy for determining how to place maximum-sized labels for DrawItem.titleAutoFit.
*/
type TitleAutoFitRotationMode = 
/**
* do not rotate
*/
"never" |

/**
* rotate only if doing so would allow label to be larger
*/
"auto" |

/**
* always rotate
*/
"always";


/**
* Orientation of titles relative to the FormItem being labeled. Can be set a the
*  DynamicForm level as a default, or on individual items.
*/
type TitleOrientation = 
"left" |

"top" |

"right";


/**
* The different ways in which the DrawItem.titleLabel of a DrawItem
*  can be rotated with the item.
*  
*  NOTE: The effect of the
*  "withItemAlwaysUp" and "withLineAlwaysUp"
*  
*  settings is not affected by the global rotation, if any (see DrawPane.rotation).
*/
type TitleRotationMode = 
/**
* the titleLabel is never rotated with the item.
*/
"neverRotate" |

/**
* the titleLabel is rotated exactly to match the item's rotation (see DrawItem.rotation).
*/
"withItem" |

/**
* the titleLabel is rotated exactly to match the item's rotation, except that at certain rotations, the titleLabel is flipped
*  by 180&deg; so that the title text is never upside down.
*/
"withItemAlwaysUp" |

/**
* (applies only to DrawLine and DrawLinePath) the titleLabel is rotated to match the line or center segment.
*  
*  If used on a DrawItem that is not a DrawLine or DrawLinePath,
*  then the effect is the same as
*  "withItem".
*  
*/
"withLine" |

/**
* (applies only to DrawLine and DrawLinePath) the titleLabel is rotated to match the line or center segment, except that
*  at certain rotations, the titleLabel is flipped by 180&deg; so that the
*  title text is never upside down.
*  
*  If used on a DrawItem that is not a DrawLine or DrawLinePath,
*  then the effect is the same as
*  "withItemAlwaysUp".
*  
*/
"withLineAlwaysUp";


/**
* Interface to use for showing and editing the FilterBuilder.topOperator 
*  of a FilterBuilder.
*/
type TopOperatorAppearance = 
/**
* radio buttons appear at the top of the form
*/
"radio" |

/**
* a SelectItem appears with a "bracket" spanning all top-level clauses, exactly the same appearance used for showing
*  FilterBuilder.showSubClauseButton, if enabled.
*/
"bracket" |

/**
* each line in the FilterBuilder is a top-level item, with a SelectItem shown on the left that allows the user to choose between the main operator in force (either "and"
*  or "or", depending on the setting of topOperator) and "and not".
*/
"inline" |

/**
* no interface is shown. The top-level operator is expected to be shown to the user outside the FilterBuilder, and, if editable, FilterBuilder.setTopOperator
*  should be called to update it
*/
"none";


/**
*/
type TourActionType = 
/**
* The TourStep.target must be clicked to complete the step
*/
"click" |

/**
* The target will complete the step on mouseDown
*/
"mouseDown" |

/**
* The target must be double-clicked to complete the step
*/
"doubleClick" |

/**
* The target must be right-clicked to complete the step
*/
"rightClick" |

/**
* The target must be moused-over to complete the step
*/
"mouseOver" |

/**
* The target must be dragged to complete step.           If TourStep.dropTarget is also specified, the
*            target component must be dropped onto the dropTarget.
*/
"drag" |

/**
* The target FormItem must change to complete the step.           Specify TourStep.expectedValue
*            if a specific value must be entered to continue. Depending on the
*            FormItem type this may occur by the user typing or by selecting
*            a value. See TourStep.inputValidation for how validation
*            is performed. Note: for composite formItems such as 
*            DateItem or RadioGroupItem, you may specify either the
*            item as a whole or a sub item as the target. Either way the
*            tour will look for the expected value on the composite (parent) item.
*/
"change" |

/**
* The target locator should resolve to an item within a Menu that has           a sub-menu. The submenu must be opened (by rolling over, or clicking the
*            target) to complete the step.
*/
"menuItemOver" |

/**
* The target locator should resolve to an item within a Menu.            The item must be selected (clicked) to complete the step.
*/
"menuItemSelect" |

/**
* No interaction is allowed with the target and the user must click            the next button to complete the step
*/
"none" |

/**
* Interaction is allowed with the target but the user must click           the next button to complete the step
*/
"any";


/**
* Policy for how TourStep.inputValidation is performed for 
*  TourStep.actionType:"change". When users enter incorrect values,
*  they are informed via the Notify system, and the specific message
*  displayed is controlled by
*  TourStep.inputValidationNotifyMessage.
*  
*  The notification can be disabled for the step by setting
*  TourStep.showInputValidationMessage to false.
*/
type TourInputValidationMode = 
/**
* When there is a brief pause in typing, notify the user that they         have typed something wrong. The pause delay is controlled by
*          TourStep.inputValidationNotifyDelay.
*/
"notify" |

/**
* Prevent the user from typing any characters that don't match the         TourStep.expectedValue, and tell them immediately
*          when they have typed something wrong.
*/
"strict" |

/**
* text entry is only validated on field exit or click on         TourStep.afterInputTarget.
*/
"onExit";


/**
*/
type TourMode = 
/**
* For a tourStep if you only specify instructions and target, the          pointer appearance is used and points at the target - no interaction
*           is expected. To make the step require an interaction, set actionType.
*/
"tour" |

/**
* For a tourStep if you only specify instructions and target, the          actionType is defaulted and the big red arrow appearance is used. 
*           Set actionType:"none" or "any" if your step has a target but
*           doesn't require an interaction.
*/
"tutorial";


/**
* Mode for applying criteria to a tree.
*/
type TreeFilterMode = 
/**
* only nodes that actually match criteria are shown. If a parent does not         match the criteria, it will not be shown, even if it has children that do
*          match the criteria
*/
"strict" |

/**
* parent nodes are kept if they have children which match the criteria,           or, in a tree with
*            ResultTree.loadDataOnDemand, if they have
*            not loaded children yet.
*/
"keepParents";


/**
* An object containing the open state for a treeGrid.
*  Note that this object is not intended to be interrogated directly, but may be stored 
*  (for example) as a blob on the server for state persistence across sessions.
*/
type TreeGridOpenState = string;


/**
* An object containing the "view state" information for a treeGrid. In addition to the 
*  state data contained by a ListGridViewState object, this will also contain the 
*  current open state of the treeGrid in question.
*  Note that this object is not intended to be interrogated directly, but may be stored 
*  (for example) as a blob on the server for view state persistence across sessions.
*/
type TreeGridViewState = string;


/**
*/
type TreeModelType = 
/**
* In this model, each node has an ID unique across the whole tree and a parent ID that points to its parent. The name of the unique ID property can be specified
*  via Tree.idField and the name of the parent ID property can be specified via
*  Tree.parentIdField. The initial set of nodes can be passed in as a list to
*  Tree.data and also added as a list later via Tree.linkNodes.
*  Whether or not a given node is a folder is determined by the value of the property specified
*  by Tree.isFolderProperty.
*  
*  The "parent" modelType is best for integrating with relational storage (because nodes can
*  map easily to rows in a table) and collections of Beans and is the model used for DataBound
*  trees.
*/
"parent" |

/**
* In this model, nodes specify their children as a list of nodes. The property that holds the children nodes is determined by Tree.childrenProperty.
*  Nodes are not required to have an ID that is unique across the whole tree (in fact, no ID is
*  required at all). Node names (specified by the Tree.nameProperty, unique within
*  their siblings, are optional but not required. Whether or not a given node is a folder is
*  determined by the presence of the children list (Tree.childrenProperty).
*/
"children";


/**
* A Uniform Resource Identifier string, as defined by 
*  https://tools.ietf.org/html/rfc3986.
*/
type URI = string;


/**
* A Uniform Resource Locator string, as defined by 
*  https://www.w3.org/Addressing/URL/url-spec.html. For example,
*  "https://www.smartclient.com/product/" is a valid URL.
*/
type URL = URI;


/**
* Mode of validation used for running validation on partially entered data.
*/
type ValidationMode = 
/**
* Assume provided data is complete and run all validation rules
*/
"full" |

/**
* Validate only the data values that are present ignoring the         absence of required fields, validators that refer to fields
*          that are not present, and any conditional validators dependent
*          on fields that are not present.
*/
"partial";


/**
* Used to name a validator or reference a standard, built-in Validator - see list below.
*  
*  To make use of a standard validator type for a field in a DataSource, or 
*  DynamicForm instance, specify the validators property to an array 
*  containing a validator definition where the type property is set to 
*  the appropriate type. 
*  
*  A custom error message can be specified for any validator type by setting the
*  errorMessage property on the validator definition object, and some
*  validator types make use of additional properties on the validator definition 
*  object such as max or min.
*  
*  For example, to use the integerRange validator type:
*  &nbsp;&nbsp;field:{
*  &nbsp;&nbsp;&nbsp;&nbsp;validators:[
*  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;{type:"integerRange", min:1, max:100}
*  &nbsp;&nbsp;&nbsp;&nbsp;]
*  &nbsp;&nbsp;}
*  
*  
*  Custom validators can be reused on the client by adding them to the global validator list,
*  via the Validator.addValidatorDefinition method.
*/
type ValidatorType = 
/**
* Validation will fail if this field is non-empty and has a non-boolean value.
*/
"isBoolean" |

/**
* Validation will fail if the value is not a string value.
*/
"isString" |

/**
* Tests whether the value for this field is a whole number. If 
*  validator.convertToInteger is true, float values will be converted 
*  into integers and validation will succeed.
*/
"isInteger" |

/**
* Tests whether the value for this field is a valid floating point number.
*/
"isFloat" |

/**
* Tests whether the value for this field is a valid expression or function; if it is
*  valid, creates a stringMethods object with the value
*  and set the resultingValue to the StringMethod.
*/
"isFunction" |

/**
* RequiredIf type validators should be specified with an expression
*  property set to a stringMethods, which takes four
*  parameters:
*  
* - item - the DynamicForm item on which the error occurred (may be null)
*  
* - validator - a pointer to the validator object
*  
* - value - the value of the field in question
*  
* - record - the "record" object - the set of values being edited by the widget
*  
*  When validation is performed, the expression will be evaluated (or executed). If it
*  returns true, the field will be treated as a required field, so validation
*  will fail if the field has no value, or, in the case of a FileItem or UploadItem
*  and if client-side validation is supported by the browser, if no file is selected for upload
*  or the selected file is empty.
*  To allow server-side enforcement, a required validator can be used instead.
*  With the exception of "binary" fields, conditional criteria can be specified with the
*  Validator.applyWhen property.
*  See ${isc.DocUtils.linkForExampleId('conditionallyRequired')}.
*  
*  
*  NOTE: A requiredIf validator cannot be used to guarantee that a non-empty
*  file is uploaded. The user's browser might not support client-side file validation.
*  Using a requiredIf validator on a "binary" field may be appropriate in scenarios where the
*  application does not technically require a non-empty file to be uploaded by the user.
*  For example, in a bug tracking application, a file upload may be required if the
*  "Have a test case?" checkbox is checked, but the value of the "Have a test case?" checkbox is
*  not actually saved by the application; instead, whether the user is providing a test case is
*  inferred by whether a non-empty test case file was uploaded.
*/
"requiredIf" |

/**
* Tests whether the value for this field matches the value of some other field.
*  The field to compare against is specified via the otherField property
*  on the validator object (should be set to a field name).
*  See ${isc.DocUtils.linkForExampleId('matchValue')}.
*/
"matchesField" |

/**
* Tests whether the value for this field matches some value specified via
*  value.
*/
"equals" |

/**
* Tests whether the value for this field does not match some value specified via
*  value.
*/
"notEqual" |

/**
* Tests whether the value for this field matches any value from an arbitrary
*  list of acceptable values. The set of acceptable values is specified via
*  the list property on the validator, which should be set to an array of
*  values. If validator.list is not supplied, the valueMap for the field will be used.
*  If there is no valueMap, not providing validator.list is an error.
*/
"isOneOf" |

/**
* Tests whether the value for this field matches any value from an arbitrary
*  list of acceptable values. The set of acceptable values is specified via
*  the list property on the validator, which should be set to an array of
*  values. If validator.list is not supplied, the valueMap for the field will be used.
*  If there is no valueMap, not providing validator.list is an error.
*/
"inSet" |

/**
* Tests whether the value for this field does not match any value from an arbitrary
*  list of unacceptable values. The set of unacceptable values is specified via
*  the list property on the validator, which should be set to an array of
*  values. Not providing validator.list is an error.
*/
"notInSet" |

/**
* Tests whether the value for this field is a whole number within the range 
*  specified. The max and min properties on the validator
*  are used to determine the acceptable range, inclusive. To specify the range as
*  exclusive of the min/mix values, set exclusive to true.
*  See ${isc.DocUtils.linkForExampleId('validationBuiltins')}.
*/
"integerRange" |

/**
* This validator type applies to string values only. If the value is a string value
*  validation will fail if the string's length falls outside the range specified by 
*  validator.max and validator.min.
*  
*  Note that non-string values will always pass validation by this validator type.
*  
*  Note that the errorMessage for this validator will be evaluated as
*  a dynamicString - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with max and min available as
*  variables mapped to validator.max and validator.min.
*/
"lengthRange" |

/**
* Determine whether a string value contains some substring specified via 
*  validator.substring.
*/
"contains" |

/**
* Determine whether a string value does not contain some substring specified via 
*  validator.substring.
*/
"doesntContain" |

/**
* Determine whether a string value contains some substring multiple times.
*  The substring to check for is specified via validator.substring.
*  The validator.operator property allows you to specify how to test
*  the number of substring occurrences. Valid values for this property are
*  ==, !=, &lt;, &lt;=,
*  &gt;, &gt;=.
*  
*  The number of matches to check for is specified via validator.count.
*/
"substringCount" |

/**
* regexp type validators will determine whether the value specified 
*  matches a given regular expression. The expression should be specified on the
*  validator object as the expression property.
*  See ${isc.DocUtils.linkForExampleId('formsRegularExpression')}.
*/
"regexp" |

/**
* Validate against a regular expression mask, specified as validator.mask.
*  If validation is successful a transformation can also be specified via the
*  validator.transformTo property. This should be set to a string in the
*  standard format for string replacement via the native JavaScript replace()
*  method.
*  See ${isc.DocUtils.linkForExampleId('formsValueTransform')}.
*/
"mask" |

/**
* Tests whether the value for a date field is within the range specified.
*  Range is inclusive, and is specified via validator.min and
*  validator.max, which should be specified in
*  XML Schema
*  date format or as a live JavaScript Date object (for client-only validators only).
*  To specify the range as exclusive of the min/mix values, set exclusive
*  to true.
*  
*  Note that the errorMessage for this validator will be evaluated as
*  a dynamicString - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with max and min available as
*  variables mapped to validator.max and validator.min.
*/
"dateRange" |

/**
* Validate a field as a valid floating point value within a value range.
*  Range is specified via validator.min and validator.max.
*  Also checks precision, specified as number of decimal places in 
*  validator.precision. If validator.roundToPrecision is set 
*  a value that doesn't match the specified number of decimal places will be rounded
*  to the nearest value that does.    
*  
*  For backwards compatibility only. Use "floatRange" and/or "floatPrecision" instead.
*/
"floatLimit" |

/**
* Tests whether the value for this field is a floating point number within the range 
*  specified. The max and min properties on the validator
*  are used to determine the acceptable range, inclusive. To specify the range as
*  exclusive of the min/mix values, set exclusive to true.
*  
*  Note that the errorMessage for this validator will be evaluated as
*  a dynamicString - text within &#36;{...} will be evaluated as JS code
*  when the message is displayed, with max and min available as
*  variables mapped to validator.max and validator.min.
*/
"floatRange" |

/**
* Tests whether the value for this field is a floating point number with the 
*  appropriate number of decimal places - specified in validator.precision
*  If the value is of higher precision and validator.roundToPrecision 
*  is specified, the value will be rounded to the specified number of decimal places
*  and validation will pass, otherwise validation will fail.
*/
"floatPrecision" |

/**
* A non-empty value is required for this field to pass validation.
*  In the case of a "binary" field, a non-empty file must be uploaded.
*/
"required" |

/**
* Disallows changes to this field, based on the Validator.applyWhen condition. 
*  
*  In addition to server-side enforcement, this validator will automatically cause editors 
*  for the field to become read-only, by rendering in a disabled appearance, readOnly 
*  appearance, or even by becoming hidden.
*  
*  The appearance of the read-only field can be controlled by validator.fieldAppearance. 
*  See FieldAppearance for choices. If fieldAppearance is not 
*  specified, the default is "readOnly".
*/
"readOnly" |

/**
* Change the visibility of this field.
*/
"visibility" |

/**
* Change the enable/disable state of this field.
*/
"enable" |

/**
* Returns true if the value for this field is unique. The uniqueness check is performed across
*  the whole DataSource unless you specify property validator.criteriaFields as a 
*  comma-separated string of field names; in that case, the uniqueness check is done in the 
*  context of those extra criteria, allowing you to check, for example, whether an employee 
*  number is unique for the department and location found on the record being validated.
*  By default the uniqueness check is not case sensitive but this can be controlled through
*  the Validator.caseSensitive attribute. You can specify the 
*  DataSource.operationBindings to use for the uniqueness check with the 
*  Validator.operationId attribute.
*  
*  Validators of this type have ValidatorDefinition.requiresServer 
*  set to true and do not run on the client, unless all of the following are true:
*  
*   
* - The validation is run in the context of a DataBoundComponent or ValuesManager
*     bound to some DataSource.
*   
* - The DataSource is either clientOnly:true or cacheAllData: true and all data is loaded
*   
* - The item is made available to the validator. Note that the item is not be available
*     during a save performed without a form (eg programmatic save), or if the field is
*     not available in the form.
*  
*  
*  Note when isUnique validator is executed as part of validation process during update 
*  operation, it will perform uniqueness check only for single row updates. If update targets 
*  OperationBinding.allowMultiUpdate, then isUnique validator will 
*  be skipped. If uniqueness check is needed when updating multiple records, consider using 
*  dmiOverview approach to add this check manually.
*  See ${isc.DocUtils.linkForExampleId('uniqueCheckValidation')}.
*/
"isUnique" |

/**
* Returns true if the record implied by a relation exists. The relation can be 
*  derived automatically from the DataSourceField.foreignKey attribute of 
*  the field being validated, or you can specify it manually via 
*  validator.relatedDataSource and validator.relatedField.
*  
*  You can specify at DataSource level that this validator should be automatically 
*  applied to all fields that specify a DataSourceField.foreignKey -
*  see DataSource.validateRelatedRecords.
*  
*  By default the uniqueness check is not case sensitive but this can be controlled through
*  the Validator.caseSensitive attribute.
*  
*  Validators of this type have ValidatorDefinition.requiresServer 
*  set to true and do not run on the client.
*  
*  Note that this validation is generally unnecessary for data coming from a UI. The 
*  typical UI uses a SelectItem or ComboBoxItem with an 
*  FormItem.optionDataSource for user entry, such that the user 
*  can't accidentally enter a related record if that doesn't exist, and a typical SQL 
*  schema will include constraints that prevent a bad insert if the user attempts to 
*  circumvent the UI. The primary purpose of declaring this validation explicitly is 
*  to provide clear, friendly error messages for use cases such as BatchUploader, 
*  where values aren't individually chosen by the user. See also the example
*  ${isc.DocUtils.linkForExampleId('hasRelatedValidation', 'Related Records')}.
*/
"hasRelatedRecord" |

/**
* This validator type is not for direct usage, instead DataSourceField.maxFileSize can 
*  be set and maxFileSize validator will be added automatically. Use 
*  DataSource.maxFileSizeExceededMessage to customize validation error message.
*  
*  In supported browsers (Internet Explorer 10+, Chrome, Firefox, Safari 6+, Opera 11.1+), returns
*  true if the file(s) selected by the user are not larger than the field's
*  DataSourceField.maxFileSize. If not supported by the browser, the validator will always
*  return true.
*  
*  Note that server-side enforcement of the maxFileSize is always required because
*  the user's browser might not support client-side file size checks. Also, any client-side
*  check can be bypassed by a malicious user.
*/
"maxFileSize" |

/**
* Custom client-side validator. 
*  Validator.condition will be called to verify data.
*  
*/
"custom" |

/**
* Custom server-side validator that either evaluates the Velocity expression provided in 
*  Validator.serverCondition (see ${isc.DocUtils.linkForExampleId('velocityValidation')})
*  or makes DMI call to Validator.serverObject to evaluate condition
*  (see ${isc.DocUtils.linkForExampleId('dmiValidation')}).
*  
*  Validators of this type have ValidatorDefinition.requiresServer 
*  set to true and do not run on the client.
*/
"serverCustom";


/**
* Enum used within the FilterBuilder class to indicate the role of a particular 
*  value-field form item within a filter clause.
*/
type ValueItemType = 
/**
* This is the single form item that will populate the generated  Criterion.value for this clause. This applies for operators with
*  Operator.valueType of "fieldType" or "custom".
*/
"value" |

/**
* This is the single form item that will populate the generated  Criterion.value for Operator.valueType of "fieldName".
*/
"name" |

/**
* Indicates this item will generate the lower-bound value (or "start") when generating criteria with Operator.valueType "valueRange".
*/
"start" |

/**
* Indicates this item will generate the higher-bound value (or "end") when generating criteria with Operator.valueType "valueRange".
*/
"end";


/**
* A ValueMap defines the set of legal values for a field, and optionally allows you to provide
*  a mapping from stored values to values as seen by the end user.
*  
*  A valueMap can be specified as either an Array of legal values, or as an Object
*  where each property maps a stored value to a user-displayable value. See
*  DataSourceField.valueMap for how to express a ValueMap in
*  componentXML.
*  
*  A ValueMap can be entirely static or entirely dynamic, with many options in between. For
*  example, a ValueMap may be:
*  
*  
* - statically defined in a JavaScript or XML file. Such a valueMap changes only when
*  application code is upgraded.
*  
* - generated dynamically by server code when the application first loads, for example,
*  by generating JavaScript or XML dynamically in a .jsp or .asp file. Such a valueMap may be
*  different for each session and for each user.
*  
* - loaded on demand from a DataSource, via the
*  PickList.optionDataSource property, or via a call to
*  DataSource.fetchData where a valueMap is derived dynamically from the returned data
*  (see DataSource.fetchData for an example). Such a valueMap may be updated at any
*  time, for example, every time the user changes a related field while editing data.
*  
*  See also the smartArchitecture to understand
*  the best architecture from a performance and caching perspective.
*/
type ValueMap = object;


/**
* An expression in the 
*  Velocity Template Language (VTL). For more information on SmartClient's Velocity support, 
*  see velocitySupport.
*  
*  Note that a VelocityExpression must often evaluate to a particular type of value 
*  to be useful. For example, DataSource.requires must evaluate to true or false 
*  (Boolean objects or strings containing those two words), and Mail.messageData must
*  evaluate to a Java Map object, or a Java List containing only 
*  Maps.
*/
type VelocityExpression = string;


/**
*/
type VerticalAlignment = 
/**
* At the top of the container
*/
"top" |

/**
* Center within container.
*/
"center" |

/**
* At the bottom of the container
*/
"bottom";


/**
* The names of the Calendar views.
*/
type ViewName = 
/**
* day view
*/
"day" |

/**
* week view
*/
"week" |

/**
* month view
*/
"month" |

/**
* timeline view
*/
"timeline";


/**
* An object containing the "view state" information for a databound component.
*  
*  Note that this object is a JavaScript string, and may be stored (for example) as a blob
*  on the server for state persistence across sessions.
*/
type ViewState = string;


/**
*/
type Visibility = 
/**
* The widget visibility will match that of its parent                 (usually visible).
*/
"inherit" |

/**
* The widget will always be visible whether its parent is                 or not.
*/
"visible" |

/**
* The widget will always be hidden even when its parent is                 visible.
*/
"hidden";


/**
* Settings for whether multiple sections can be in the expanded state simultaneously.
*/
type VisibilityMode = 
/**
* Only one section can be expanded at a time.
*/
"mutex" |

/**
* Multiple sections can be expanded at the same time, and will share space.
*/
"multiple";


/**
* XMLDocument is the "parsed" or object form of XML, which allows XML to be navigated as
*  a tree of nodes with attributes, namespaces and other metadata, as opposed to being
*  manipulated as just a String.
*  
*  XMLDocument is a native object supplied directly by the browser. The SmartClient-supported
*  interfaces for this object are methods that take an XMLDocument as an argument (such as
*  XMLTools.selectNodes). If you want to retrieve XML data and display it in a
*  SmartClient component, read about clientDataIntegration. To
*  extract data as JavaScript Objects from XML, see XMLTools.toJS. Direct
*  manipulation of XMLDocument is subject to cross-browser inconsistencies, bugs, memory leaks
*  and performance issues.
*/
type XMLDocument = object;


/**
* An XMLElement represents one complete XML tag, including any subelements contained between
*  the start and end tags.
*  
*  XMLElement is a native object supplied directly by the browser. The SmartClient-supported
*  interfaces for this object include methods that take an XMLElement as an argument (such as
*  XMLTools.selectNodes). If you want to retrieve XML data and display it in a
*  SmartClient component, read about clientDataIntegration. To
*  extract data as JavaScript Objects from XML, see XMLTools.toJS. Direct
*  manipulation of XMLElements objects is subject to cross-browser inconsistencies, bugs,
*  memory leaks and performance issues.
*/
type XMLElement = object;


/**
* A standard XPath expression as a string. To learn about XPath, try the following search:
*  http://www.google.com/search?q=xpath+tutorial
*/
type XPathExpression = string;


/**
*/
type ZoomStartPosition = 
/**
* start at the beginning of the range
*/
"start" |

/**
* start at the end of the range
*/
"end";




/**
* A decimal (or "floating point") number, for example, 5.5. May not be null.
*/
type double = Double;


/**
* A decimal (or "floating point") number, for example, 5.5. May not be null.
*/
type float = Float;


/**
* A whole number, for example, 5. Decimal numbers, for example 5.5, are not allowed. May not
*  be null.
*/
type int = Integer;


}
